require "utils"
local jwt = require "shared.jwt"
local md5 = require "md5"
local http = require "socket.http"
local ltn12 = require("ltn12")

module(..., package.seeall)

function getVccConfiguration(keyspace, vcc)
	local column = keyspace.GetColumn(keyspace, vcc, "VCCs", "ApiConfiguration")
	return json.decode(column.value)
end

function makeToken(keyspace, vcc, userId)
	userId = userId or "__migration"
	local configuration = getVccConfiguration(keyspace, vcc)
	local data = {
		userName = userId,
		virtualContactCenter = vcc,
		userType = "full",
		api = true
	}
	local token = jwt.encode(data, configuration.key, configuration.algorithm)
	return token .. "@" .. vcc
end

function call(token, domain, port, method, path, body, headers)
	body = body or ""
	headers = headers or {}
	headers["Content-Length"] = tostring(#body)
	headers["authorization"] = token
	local source = ltn12.source.string(body)
	local response_body = {}
	local ok, response_code, response_headers, response_status_line = http.request({
		host = domain,
		port = port,
		method = method,
		headers = headers,
		url = "/inconcert/api/" .. path,
		source = source,
		sink = ltn12.sink.table(response_body)
	})
	response_body = table.concat(response_body)
	if not ok and type(response_code) ~= "number" then
		response_code = -1
	end
	return response_code, response_body, response_headers
end
