local ir = require "lua_iragentlink" 
local apiModel = require "api.model"
local models = apiModel:GetModels()
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local repositoryController = require("apps.repository")
local allegro = require "apps.allegro.proxy"
local allegro_repository = require "apps.allegro.controllers.repository"
local ltn12 = require "ltn12"
local m_reportAPI = require "api.controllers.reports"

require "LuaOrm.Date"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local function StartRepository ()
	repositoryController.ManualStart(inconcert.environment.api.cassandra.ip, inconcert.environment.api.cassandra.port)
end

local function GetFacebookAttachments (message)
	if not message then
		return
	end
	if message.attachment then
		local attachment = utils.JSON_Decode(message.attachment)

		if utils.IsArray(attachment) then
			return attachment
		end
		if utils.IsArray(attachment.data) then
			return attachment.data
		elseif type(attachment.data) == "string" then
			local data = utils.JSON_Decode(attachment.data)
			return data
		else
			return {attachment}
		end

	elseif message.attachments then
		local attachments = utils.JSON_Decode(message.attachments)

		if utils.IsArray(attachments) then
			return attachments
		end
		if utils.IsArray(attachments.data) then
			return attachments.data
		elseif type(attachments.data) == "string" then
			local data = utils.JSON_Decode(attachments.data)
			return data
		else
			return {attachments.attachment}
		end
	end
end

local listMediaByType = {
	MAIL = function(interaction)
		local mediaList = {}
		local mediaById = {}
		for _, event in pairs(interaction.events) do
			for _, attachment in pairs(event.attachments or {}) do
				if not mediaById[attachment.id] then
					local media = {
						creator = event.creator,
						agent = event.agent,
						source = "repository",
						id = attachment.id,
						name = attachment.name,
						format = utils.GetFileExtension(attachment.name),
						section = -1,
						part = -1
					}
					table.insert(mediaList, media)
					mediaById[attachment.id] = true
				end
			end
		end
		return mediaList
	end,
	FACEBOOK = function(interaction)
		local mediaList = {}

		local FillRecursiveAttachmentsMediaList		
		FillRecursiveAttachmentsMediaList = function(attachment, event)
			if not attachment then
				return
			end
			local media = nil
			if attachment.image_data and attachment.image_data.url then
				media = {
					type = "image",
					url = attachment.image_data.url
				}
			elseif attachment.video_data and attachment.video_data.url then
				media = {
					type = "video",
					url = attachment.video_data.url
				}
			elseif attachment.file_url then
				media = {
					type = "file",
					url = attachment.file_url
				}
			elseif attachment.type then
				if attachment.type == "photo" then
					media = {
						type = "image",
						url = attachment.media and attachment.media.image and 
							attachment.media.image.src or ""
					}
				elseif attachment.type == "sticker" then
					media = {
						type = "sticker",
						url = attachment.media and attachment.media.image and 
							attachment.media.image.src
					}
				elseif attachment.type == "image" then
					media = {
						type = "image",
						url = attachment.payload.url
					}
				elseif attachment.type == "file" then
					media = {
						type = "file",
						url = attachment.payload.url
					}
				elseif attachment.type == "album" then
					local subattachments = (attachment.subattachments or {}).data or {}
					for _, subattachament in ipairs(subattachments) do
						FillRecursiveAttachmentsMediaList(subattachament, event)
					end
				else
					media = {
						url = attachment.payload and attachment.payload.url or attachment.url
					}
				end
			end
			if media and media.url then
				media.source = "url"
				media.creator = event.creator
				media.agent = event.agent
				table.insert(mediaList, media)
			end
		end

		for _, event in pairs(interaction.events) do
			local attachments = GetFacebookAttachments(event.message)
			if attachments then
				for _, attachment in pairs(attachments) do
					FillRecursiveAttachmentsMediaList(attachment, event)
				end
			elseif event.message.shares and event.message.shares.data then
				for _, sharedData in pairs(event.message.shares.data) do
					if sharedData.link then
						local media = {
							source = "url",
							type = "sticker",
							url = sharedData.link,
							creator = event.creator,
							agent = event.agent
						}
						table.insert(mediaList, media)
					end
				end
			else
				local msgtype = event.message.type or "text"
				local media = nil
				if msgtype == "photo" then
					media = {
						type = "image",
						url = event.message.picture or event.message.link
					}
				elseif msgtype == "video" then
					media = {
						type = "video",
						url = event.message.source or event.message.link
					}
				else
					media = {
						url = event.message.source or event.message.link
					}
				end
				if media and media.url then
					media.source = "url"
					media.creator = event.creator
					media.agent = event.agent
					table.insert(mediaList, media)
				end
			end
		end
		return mediaList
	end,
	INSTAGRAM = function(interaction)
		local mediaList = {}
		local function processMessage(event, message)
			local msgtype = message.media_type or "TEXT"
			if msgtype == "IMAGE" then
				table.insert(mediaList, {
					creator = event.creator,
					agent = event.agent,
					type = "image",
					source = "url",
					url = message.media_url
				})
			elseif msgtype == "VIDEO" then
				table.insert(mediaList, {
					creator = event.creator,
					agent = event.agent,
					type = "video",
					source = "url",
					url = message.media_url
				})
			elseif msgtype == "CAROUSEL_ALBUM" then
				for _, child in pairs(message.children or {}) do
					processMessage(event, child)
				end
			end
		end
		
		for _, event in pairs(interaction.events) do
			processMessage(event, event.message)
		end
		return mediaList
	end,
	YOUTUBE = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			local msgtype = event.message and event.message.msgtype or "text"
			if msgtype == "attachment" then
				local media = {
					creator = event.creator,
					agent = event.agent,
					type = event.message.fileType
				}
				if event.message.attachmentType == "repository" then
					media.source = "repository"
					media.id = event.message.fileId
					media.name = event.message.fileName
					media.format = event.message.extension
					media.section = -1
					media.part = -1
				elseif event.message.attachmentType == "url" then
					media.source = "url"
					media.url = event.message.url
				end
				if event.message.previewFileId then
					media.previewSource = "repository"
					media.previewId = previewFileId
					media.previewSection = -1
					media.previewPart = -1
				elseif event.message.previewUrl then
					media.previewSource = "url"
					media.previewUrl = event.message.previewUrl
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	TWITTER = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			for _, attachment in pairs(event.entities and event.entities.media or {}) do
				local media = {
					creator = event.creator,
					agent = event.agent
				}
				--type
				if media.type == "photo" then
					media.type = "image"
				elseif media.type == "video" then
					media.type = "video"
				else
					media.type = "file"
				end
				--source
				if attachment.repositoryFileId then
					media.source = "repository"
					media.id = attachment.repositoryFileId
					media.section = -1
					media.part = -1
				else
					media.source = "url"
					media.url = attachment.media_url or attachment.media_url_https
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	WHATSAPP = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			local msgtype = event.message and event.message.msgtype or "text"
			if msgtype == "attachment" then
				local media = {
					creator = event.creator,
					agent = event.agent,
					type = event.message.fileType
				}
				if event.message.attachmentType == "repository" then
					media.source = "repository"
					media.id = event.message.fileId
					media.name = event.message.fileName
					media.format = event.message.extension
					media.section = -1
					media.part = -1
				elseif event.message.attachmentType == "url" then
					media.source = "url"
					media.url = event.message.url
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	APPSTORE = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			local msgtype = event.message and event.message.msgtype or "text"
			if msgtype == "attachment" then
				local media = {
					creator = event.creator,
					agent = event.agent,
					type = event.message.fileType
				}
				if event.message.attachmentType == "repository" then
					media.source = "repository"
					media.id = event.message.fileId
					media.name = event.message.fileName
					media.format = event.message.extension
					media.section = -1
					media.part = -1
				elseif event.message.attachmentType == "url" then
					media.source = "url"
					media.url = event.message.url
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	GOOGLEPLAY = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			local msgtype = event.message and event.message.msgtype or "text"
			if msgtype == "attachment" then
				local media = {
					creator = event.creator,
					agent = event.agent,
					type = event.message.fileType
				}
				if event.message.attachmentType == "repository" then
					media.source = "repository"
					media.id = event.message.fileId
					media.name = event.message.fileName
					media.format = event.message.extension
					media.section = -1
					media.part = -1
				elseif event.message.attachmentType == "url" then
					media.source = "url"
					media.url = event.message.url
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	RCS = function(interaction)
		local mediaList = {}
		for _, event in pairs(interaction.events) do
			local msgtype = event.message and event.message.msgtype or "text"
			if msgtype == "attachment" then
				local media = {
					creator = event.creator,
					agent = event.agent,
					type = event.message.fileType
				}
				if event.message.attachmentType == "repository" then
					media.source = "repository"
					media.id = event.message.fileId
					media.name = event.message.fileName
					media.format = event.message.extension
					media.section = -1
					media.part = -1
				elseif event.message.attachmentType == "url" then
					media.source = "url"
					media.url = event.message.url
				end
				table.insert(mediaList, media)
			end
		end
		return mediaList
	end,
	CHAT = function(interaction)
		local mediaList = {}
		if interaction.interactionData and interaction.interactionData.repositoryAttachments then
			for id, metadata in pairs(interaction.interactionData.repositoryAttachments) do
				local media = {
					creator = metadata.creator,
					agent = metadata.agent,
					type = metadata.type,
					format = metadata.format,
					name = metadata.name,
					timestamp = metadata.timestamp,
					source = "repository",
					id = id,
					section = -1,
					part = -1
				}
				table.insert(mediaList, media)
			end
			table.sort(mediaList, function(a, b)
				return (a.timestamp or 0) < (b.timestamp or 0)
			end)
			for _, media in pairs(mediaList) do
				media.timestamp = nil
			end
		end
		return mediaList
	end,
	CALL = function(interaction)
		local records, err

		if interaction.interactionData and interaction.interactionData.isAllegro
		then
			records, err = allegro.supervisor.GetInteractionRecords(interaction.id)
		elseif interaction.interactionData.records then
			records = { status = true, records = interaction.interactionData.records }
		end

		if err then
			LogError("[API.Interactions.MediaTypes] Error retrieving records: %s", err)
			return nil , err
		elseif not records then
			LogWarning("[API.Interactions.MediaTypes] Not records found for interaction: %s", interaction.id)
			return {}, nil
		end

		local mediaList = {}
		for _, record in pairs(records.records) do
			local media = {
				creator = "agent",
				agent = utils.AppendVcc(record.AgentId, inconcert.authUser.virtualContactCenter),
				source = "repository",
				format = record.Format,
				id = interaction.id,
				section = record.Section or record.Part,
				part = record.Part
			}
			table.insert(mediaList, media)
		end
		return mediaList
	end,
}

local function __listMedia(interaction)
	if not listMediaByType[interaction.type] then
		return {}
	end
	return listMediaByType[interaction.type](interaction)
end

local function __getMediaDefinition(interaction, id, section, part)
	local mediaList = __listMedia(interaction)
	if mediaList then
		for _, media in pairs(mediaList) do
			if media.source == "repository" and media.id == id and media.section == section and media.part == part then
				return media
			end
		end
	end	
	return nil
end

local function __getMediaFromAllegro(interaction, media)
	local vcc = inconcert.authUser.virtualContactCenter
	local success, request, errorCode, headers = allegro_repository.Retrieve(vcc, media.id, media.section, media.part, true)
	if not success then
		return errorCode, api_utils.makeResponseStr(false, "error getting media")
	end
	return true, coroutine.wrap(function()
		local mySink = function(chunk, err)
			if chunk then
				local ok = coroutine.yield(chunk)
				if not ok then
					return nil
				end
			end
			return 1
		end
		local receive = request.receivebody
		rawset(request, "receivebody", receive)
		request:receivebody(headers, mySink, ltn12.pump.step)
		request:close()
	end)
end

local function Get (web, interactionId)
	if utils.alltrim(interactionId) == "" then
		-- ERROR
		return api_utils.replyNotFound(web, "interaction id is required")
	end
	local interaction = models.Interactions.Get(interactionId)
	if not interaction then
		-- ERROR
		return api_utils.replyNotFound(web, "interaction not found")
	end
	-- OK
	return api_utils.replyOk(web, { interaction = interaction })
end

local function getReportDataByType(type)
	local data = {
		CALL       = {dimensions = {"Campaign", "StartDate", "Initiative", "IsWebCall", "LastAttentionLevel", "CallDnis", "ContactAddress",
						"ContactName", "FirstAgent", "LastAgent","WasSentToAgentSearch","IsTaken","IsAbandoned","IsCanceled",
						"SLPositive","IsShort","IsLong","IsGhost","IsOutOfScheduler","Ender","StartAttention","EndDate",
						"DurationTime","WaitingTime","IVRTime","ACDTime","RingingTime","AnswerTime","AttentionTime", "TransferAgentBots",
						"TransferAgentBotTime","WrapupTime","PreviewTime","DispositionCode","DispositionTreePath","DispositionIsGoal",
						"IsAppointment","HasAppointment","WasTransferredFromCampaign","OriginalCampaign","IsTransferred","TransferSuccessful",
						"TransferDestinationType","TransferDestination","TransferTime","TransferExternalCalls","TransferExternalCallsTime","TransferExternalTime",
						"RequeuedTime","HasConference","ConferenceSuccessful","ConferenceDestinationType","ConferenceDestination","ConferenceTime","Holds","HoldTime",
						"Ticket","VCC","ID","AudioRecords","AudioDuration"},
						filterName = "DetailMandatoryTypeOptionalCampaignVCC"
					},
		VOICEMAIL  = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","ContactAddress",
						"ContactName","FirstTakeAgent","LastTakeAgent","IsTaken","StartAttention","EndDate",
						"DurationTime","WaitingTime","AttentionTime","WrapupTime","DispositionCode","DispositionTreePath",
						"DispositionIsGoal","HasAppointment","IsTransferred","TransferSuccessful","TransferDestinationType",
						"TransferDestination","Ticket","VCC", "ID"},
						filterName = "DetailMandatoryTypeOptionalCampaignVCC"
					},
		CHAT       = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","ContactName","FirstTakeAgent","LastTakeAgent","IsIVR","PureIVR",
						"IsTaken","IsFinished","IsAbandoned","SLPositive","HasAnswerFailures","IsGhost","IsOutOfScheduler","Ender",
						"StartAttention","EndDate","DurationTime","IVRTime","WaitingTime","ACDTime","WaitForAnswerTime",
						"AttentionTime","WrapupTime","DesertionTime","DispositionCode","DispositionTreePath","DispositionIsGoal",
						"IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination","ChatMessages","ChatClientMessages","ChatAgentMessages",
						"ChatBotMessages","HasLogin","HasPreSurvey","HasPostSurvey","HasCCI","Ticket","VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignVCC",
					},
		FACEBOOK   = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","SubType","FirstAgent",
						"ContactName","IsIVR","PureIVR","HasResponse","FirstResponseDate","TotalMessages",
						"TotalClientMessages","TotalAgentMessages","TotalBotMessages","StartAttention","EndDate","DurationTime",
						"IVRTime","WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode","DispositionTreePath",
						"DispositionIsGoal","ContactAddress","Page","PostId","PostMessage","InitialMessageId","TextOfInitialMessage",
						"IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination","HasCCI","Ticket",
						"VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		INSTAGRAM  = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","SubType","FirstAgent","ContactName","IsIVR","PureIVR","HasResponse",
						"FirstResponseDate","TotalMessages","TotalClientMessages", "TotalAgentMessages","TotalBotMessages","StartAttention","EndDate","DurationTime","IVRTime",
						"WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode","DispositionTreePath","DispositionIsGoal",
						"ContactAddress","IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination",
						"HasCCI","Ticket","VCC","ID","AccountID","Page", "PostId", "PostMessage","InitialMessageId","TextOfInitialMessage"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		YOUTUBE    = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","SubType","FirstAgent","ContactName","HasResponse",
						"FirstResponseDate","TotalMessages","TotalMessages","TotalClientMessages", "TotalAgentMessages", "StartAttention",
						"EndDate","DurationTime","WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode","DispositionTreePath",
						"DispositionIsGoal","ContactAddress","PostId", "PostMessage", "TextOfInitialMessage", "IsTransferred","TransferSuccessful",
						"TransferDestinationType","TransferDestination","HasCCI","Ticket","VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		MAIL       = { dimensions = {"Campaign", "StartDate", "Account", "LastAttentionLevel", "Initiative",
		               "ParentInteractionType", "FirstAgent", "ContactName", "IsIVR", "PureIVR",
					   "MailHasResponse", "MailResponseDate", "MailTotalMessages", "MailAgentResponseCount",
					   "MailBotResponseCount", "StartAttention", "EndDate", "DurationTime", "IVRTime", "WaitingTime",
					   "ACDTime", "WaitForAnswerTime", "AttentionTime", "DispositionCode", "DispositionTreePath", "DispositionIsGoal",
					   "MailFrom", "MailTo", "MailCc", "MailSubject", "MailAttachCount", "WasSPAM", "MailBatchId", "MailBatchMessageSubject",
					   "IsTransferred", "TransferSuccessful", "TransferDestinationType", "TransferDestination", "HasCCI", "Ticket", "FolderFrom",
					   "FolderTo", "VCC", "ID", "AccountID"},
					   filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		SMS        = {dimensions = {"Campaign","StartDate","Account","LastAttentionLevel","Initiative","FirstAgent","ContactName","IsIVR","PureIVR","HasResponse",
						"FirstResponseDate","SmsTotalMessages", "SmsTotalClientMessages", "SmsTotalAgentMessages","StartAttention",
						"EndDate","DurationTime","IVRTime","WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode",
						"DispositionTreePath","DispositionIsGoal","ContactAddress","SmsPostId","SmsPostMessage","SmsInitialMessageId","SmsTextOfInitialMessage",
						"IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination","HasCCI","Ticket","VCC","ID","AccountID", "SmsTotalBotMessages"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		WHATSAPP   = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","Initiative","FirstAgent","LastAgent","ContactName",
						"IsIVR", "PureIVR", "HasResponse","FirstResponseDate","WhatsappTotalMessages", "WhatsappTotalClientMessages",
						"WhatsappTotalAgentMessages","WhatsappTotalBotMessages","WhatsappTotalHsmMessages","StartAttention","EndDate",
						"DurationTime","IVRTime","WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode",
						"DispositionTreePath", "DispositionIsGoal","ContactAddress","WhatsappPostId","WhatsappPostMessage",
						"WhatsappInitialMessageId","WhatsappTypeOfInitialMessage","WhatsappTextOfInitialMessage","IsTransferred",
						"TransferSuccessful","TransferDestinationType","TransferDestination","HasCCI","Ticket","VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		TWITTER    = {dimensions = {
						"Campaign","StartDate","Account","LastAttentionLevel","SubType","FirstAgent","ContactName","IsIVR","PureIVR","HasResponse",
						"FirstResponseDate","TotalMessages","TotalAgentTweets","TotalBotTweets","StartAttention","EndDate","DurationTime","IVRTime",
						"WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode","DispositionTreePath","DispositionIsGoal",
						"ContactAddress","RepliedMessage","QuotedMessage","TextOfInitialTweet","InitialTweetDeleted","InitialTweetRetweets",
						"RetweetedTweets","RetweetedAgentTweets","IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination",
						"HasCCI","Ticket","VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignInitiativeVCC"
					},
		WEBCONTACT = {dimensions = {
						"Campaign","StartDate","Account", "LastAttentionLevel","FirstAgent","Finished","ContactName",
						"HasResponse","FirstResponseDate","FirstResponseChannelType","FirstResponseAddress","TotalTypeOfChannelUsed",
						"TotalResponses","TotalAgentResponses","StartAttention","EndDate","DurationTime",
						"WaitingTime","ACDTime","WaitForAnswerTime","AttentionTime","DispositionCode","DispositionTreePath",
						"DispositionIsGoal","IsTransferred","TransferSuccessful","TransferDestinationType","TransferDestination","HasCCI",
						"Ticket","VCC","ID","AccountID"},
						filterName = "DetailMandatoryTypeOptionalCampaignVCC"
					}
	}
	return data[type]
end

local function GetCompositeInteraction (web, id)
	LogInfo("[API.interactions:GetCompositeInteraction] Interaction ID %s",id)

	if not id or id == "" then
		return api_utils.replyNotFound(web, "interaction not found")
	end

	local key = inconcert.authUser.virtualContactCenter .. "@Interaction:" .. id
	local details = utils.GetAllColumns {
			keyspace = "SummarizerData",
			columnFamily = "DetailsData",
			key = key,
			columns = {"Type"}
		} or {}

	LogInfo("[API.interactions:GetCompositeInteraction] details  %s", utils.JSON_Encode(details))	
	if not details or not details.Type then
		return api_utils.replyNotFound(web, "interaction not found")
	end

	local dataReport = getReportDataByType(details.Type)
	local queryParams = {
		interactionId = {
			vcc = inconcert.authUser.virtualContactCenter,
			id = id
		},
		projectFields = dataReport.dimensions,
		queryFields = {
			Type = details.Type,
			Campaign = ""
		  },
		filterName = dataReport.filterName,
		resultSize = 1,
		composite = true
	}

	local success, content = m_reportAPI.ApiExecuteComposite({type = "composite"}, queryParams)
	return api_utils.replyOk(web, {content = content})
end


local function GetAgent (web, proxyType, interactionId)
	--local interaction = models.Interactions.Get(interactionId)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:GetAgentInteraction({
		proxyType = proxyType,
		interactionID = interactionId
	})
	local interaction = result.interaction
	if interaction then
		return api_utils.replyOk(web, { interaction = interaction })
	else
		return api_utils.replyNotFound(web, "interaction not found")
	end
end

local function ListMedia (web, interactionId)
	local interaction = models.Interactions.Get(interactionId)
	if not interaction then
		return api_utils.replyNotFound(web, "interaction not found")
	end
	local media, err = __listMedia(interaction)
	if media then
		return api_utils.replyOk(web, { media = media })
	else
		return api_utils.replyServerError(web, err or "error getting media list")
	end
end

local function GetMedia (wsapi_env)
	--LogDebug("[API.interactinos:GetMedia] ----> ENTRA")
	StartRepository()
	return repositoryController.retrieveFile(wsapi_env,
		function(request)
			--LogDebug("[API.interactinos:GetMedia]    1----> repositoryController.retrieveFile (callback) ")
			local status, reason = api_utils.validateManualRequest("API:interactions:GetMedia",
				request.env.HTTP_AUTHORIZATION, request.env.HTTP_X_CLIENT_IP, "read", request.params, {
					{	id = "interaction",	required = true,	type = "string" },
					{	id = "id",			required = true,	type = "string" },
					{	id = "section",		required = false,	type = "number", default = -1 },
					{	id = "part",		required = false,	type = "number", default = -1 },
			})
			--LogDebug("[API.interactinos:GetMedia]   2 ----> repositoryController.retrieveFile (callback) status: %s", status)
			if status ~= 200 then
				return status, api_utils.makeResponseStr(false, reason)
			end
			local interaction = models.Interactions.Get(request.params.interaction)
			--LogDebug("[API.interactinos:GetMedia]   3 ----> repositoryController.retrieveFile (callback) request.params.interaction: %s, interaction: %s", request.params.interaction, interaction)
			if not interaction then
				return 404, api_utils.makeResponseStr(false, "interaction not found")
			end
			local realSection = request.params.section
			local realPart = request.params.part
			if not interaction.interactionData.isAllegro then
				realSection = realPart
			end
			--LogDebug("[API.interactinos:GetMedia]   4 ----> repositoryController.retrieveFile (callback) realSection: %s, realPart: %s", realSection, realPart)
			local media = __getMediaDefinition(interaction, request.params.id, realSection, realPart)
			--LogDebug("[API.interactinos:GetMedia]   5 ----> repositoryController.retrieveFile (callback) media: %s", media)
			if not media then
				return 404, api_utils.makeResponseStr(false, "media not found in interaction")
			end
			if utils.IsAllegroChannel(interaction.type) then
				return __getMediaFromAllegro(interaction, media)
			end
			request.params.section = realSection
			--LogDebug("[API.interactinos:GetMedia]   6 ----> repositoryController.retrieveFile (callback) request.params.section: %s", request.params.section)
			return 200
		end
	)
end

--[[
	ConversationsDetail

		Función que ejecuta la App "Reports" para obtener la información necesaria.
		Si fuese necesario los datos resultantes se pueden procesar.
]]--
local function ConversationsDetail (web, request)
	local encodedParams = web.input["post_data"]
	LogInfo("[API.interactions:ConversationsDetail] Invoked API Conversations Detail ........ post_data: '%s'.", encodedParams)
	local data = utils.JSON_Decode(encodedParams)
	if data.queryFields and request.apiDefinition.config.type then
		data.queryFields.Type = request.apiDefinition.config.type:lower()
	end
	local reportId = request.apiDefinition.config.reportId
	local success, content = m_reportAPI.API_Execute(reportId, data)
	if not success and content == "other_node" then
		return api_utils.replyOtherNode(web, "cache not found in this node")
	end
	-- TODO: aca llegan los datos retornados desde la app "reports" rocka "reporter". Si fuese necesario se puede procesar la información:
	--	realizar algo contra ES, REDIS, Cassandra, etc.
	return api_utils.replyOk(web, {content = content})
end

local function SplitString(inputstr, sep)
	if sep == nil then
			sep = "%s"
	end
	local t={} ; i=1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
			t[i] = str
			i = i + 1
	end
	return t
end

local function LoadInteractionsData (web)
	local encodedParams = web.input["post_data"]
	--LogDebug("[API.interactions:LoadInteractionsData] --> Posted Data: '%s'.", encodedParams)
	local vcc = inconcert.authUser.virtualContactCenter
	local data = utils.JSON_Decode(encodedParams)
	data.apiNotValidateUserCampaigns = true
	data.mustValidateSearchPermission = false
	local query = {
		condition = "AND",
		rules = {}
	}
	local directRules = {
		attention_level = true,
		disposition_code = true,
		contact_name = true,
		type = true,
		campaign = true,
		agent = true,
		businessdata = true,
		account = true,
		ticket = true,
		contact = true,
		contact_address = true,
		state = true,
		initiative = true,
		disposition_tree_code = true
	}

	local mapBusinessDataOperator = {
		["="] = "equal",
		["<>"] = "not_equal",
		["contains"] = "contains",
		["<"] = "less",
		[">"] = "greater",
		["<="] = "les_or_equal",
		[">="] = "greater_or_equal",
		["between"] = "between"
	}

	--Se lleva los agents y campaigns a minusculas
	local convertTableStringToLower = function (tableStr)
		if type(tableStr) == "table" then
			local tableResult = {}
			for _, value in ipairs(tableStr) do
				table.insert(tableResult, value:lower())
			end
			return tableResult
		else
			return tableStr:lower()
		end
	end

	-- Se valida que el formato de fecha sea correcto.
	local validateDate = function (strDate)
		local year, month, day, hour, minute, second  = string.match(strDate, "^(%d+)-(%d+)-(%d+) (%d+):(%d+):(%d+)$");
		return year and month and day and hour and minute and second and true or false
	end

	local updateRule = function(ruleName, operator, value, value_to) 
		for _, rule in pairs(query.rules) do
			if rule.id == ruleName then
				rule.operator = operator
				rule.value = rule.value or value
				rule.value_to = rule.value_to or value_to
				return true
			end
		end
		return false
	end

	local existsFilterByState = false

	for filterFieldName, filterFieldValue in pairs(data.query) do
		--New format
		if filterFieldName == "rules" then
			for _, rule in pairs(filterFieldValue) do
				if rule.id == "agent" or rule.id == "campaign" then
					rule.value = convertTableStringToLower(rule.value)
				elseif rule.id == "type" then
					rule.value = rule.value:upper()
				elseif rule.id == "state" then
					existsFilterByState = true
				end
			end
			query[filterFieldName] = filterFieldValue
		else
			--Old format means operator "equal", except for content (contains) and ranges
			local rule = {
				operator = "equal"
			}
			if directRules[filterFieldName] then
				rule["id"] = filterFieldName
				if filterFieldName == "agent" or rule.id == "campaign" then
					rule["value"] = convertTableStringToLower(filterFieldValue)
				elseif filterFieldName == "type" then
					rule["value"] = filterFieldValue:upper()
				elseif filterFieldName == "businessdata" then
					for _, businessDataField in pairs(filterFieldValue) do
						businessDataField.operator = mapBusinessDataOperator[businessDataField.operator] or businessDataField.operator
					end
					rule["value"] = filterFieldValue
				else
					rule["value"] = filterFieldValue
				end
				if filterFieldName == "state" then
					existsFilterByState = true
				end
			elseif filterFieldName == "text" then
				rule["id"] = "content"
				rule["value"] = filterFieldValue
				rule["operator"] = "contains"
			elseif filterFieldName == "date_from" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("start_date", "between", filterFieldValue) then
					rule["id"] = "start_date"
					rule["value"] = filterFieldValue
					rule["operator"] = "greater_or_equal"
				end
			elseif filterFieldName == "date_to" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("start_date", "between", nil, filterFieldValue) then
					rule["id"] = "start_date"
					rule["value_to"] = filterFieldValue
					rule["operator"] = "less_or_equal"
				end
			elseif filterFieldName == "min_duration" then
				if not updateRule("attention_time", "between", filterFieldValue) then
					rule["id"] = "attention_time"
					rule["value"] = filterFieldValue
					rule["operator"] = "greater_or_equal"
				end
			elseif filterFieldName == "max_duration" then
				if not updateRule("attention_time", "between", nil, filterFieldValue) then
					rule["id"] = "attention_time"
					rule["value_to"] = filterFieldValue
					rule["operator"] = "less_or_equal"
				end
			elseif filterFieldName == "date_last_event_from" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("last_event_timestamp", "between", filterFieldValue) then
					rule["id"] = "last_event_timestamp"
					rule["value"] = filterFieldValue
					rule["operator"] = "greater_or_equal"
				end
			elseif filterFieldName == "date_last_event_to" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("last_event_timestamp", "between", nil, filterFieldValue) then
					rule["id"] = "last_event_timestamp"
					rule["value_to"] = filterFieldValue
					rule["operator"] = "less_or_equal"
				end
			elseif filterFieldName == "date_state_from" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("state_timestamp", "between", filterFieldValue) then
					rule["id"] = "state_timestamp"
					rule["value"] = filterFieldValue
					rule["operator"] = "greater_or_equal"
				end
			elseif filterFieldName == "date_state_to" then
				if not validateDate(filterFieldValue) then
					web.status = 500
					return api_utils.replyError(web, nil, {result = "ERROR_DATETIME_FORMAT", reason = filterFieldName})
				end
				if not updateRule("state_timestamp", "between", nil, filterFieldValue) then
					rule["id"] = "state_timestamp"
					rule["value_to"] = filterFieldValue
					rule["operator"] = "less_or_equal"
				end
			end

			if rule.id then
				table.insert(query.rules, rule)
			end
		end
	end
		
	if not existsFilterByState then
		local allStatesRule = {"ABANDONED", "FINISHED", "QUEUED", "LOCKED", "TAKEN", "TRANSFER QUEUED", "WAITING", "WRAPUP", "CANCELED"}
		table.insert(query.rules, {id="state", operator="in", value = allStatesRule})
	end

	query.isFullDate = true

	data.query = query

	-- punto en el tiempo (PIT)
	--[[
		1- Guardar Pit en el last (last Row + Pit + time de creado) Ej: 1|28982309nxaknska9u323902830jdasnk|2022-01-01 00:00:00
	]]--

	local pit
	local pitStatus
	local dateTimePit
	local last = data.last
	local splitLast = SplitString(last, "|")

	LogInfo("[API.interactions:LoadInteractionsData] splitLast '%s'", utils.JSON_Encode(splitLast))

	local elasticSearchConnectionConfig = api_utils.GetApiConnectionConfig("elasticsearch")
	local agentInteractionsController, agentInteractionsModel = api_utils.GetAgentController({controllerName = "interactions", modelName = "Interactions"})

	local elasticSearchInstance = utils.GetElasticSearchInstance()

	--Valido que el from no sea mayor al MAX_ALLOWED_ROWS_IN_ES_FROM_API
	local maxAllowedRowsInES = elasticSearchInstance:GetMaxAllowedRowsInES(true)
	local requestFrom
	if #splitLast > 1 then
		requestFrom = tonumber(splitLast[1])
	else
		requestFrom = tonumber(data.last)
	end
	if requestFrom and requestFrom > maxAllowedRowsInES then
		LogWarning("[API.interactions:LoadInteractionsData] Exceed max allowed rows in Elastic Search. requestFrom: %s - maxAllowedRowsInES: %s",requestFrom, maxAllowedRowsInES)
		web.status = 500
		local result = {
			result = "REQUEST_FAILED",
			reason = "Exceed max allowed rows in Elastic Search."
		}
		return api_utils.replyError(web, nil, result)
	end
	local utcDateNow = utils.ConvertDateToUTCDate(LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S"), nil, nil, true)
	local interval = #splitLast > 1 and math.floor( math.abs( utcDateNow:time() - utils.ConvertDateToUTCDate(splitLast[3], nil, nil, true):time()))

	if #splitLast > 1 then
		pit = splitLast[2]
		last = tonumber(splitLast[1])
		--dateTimePit = splitLast[3]
		if interval > 180 then
			LogWarning("[API.interactions:LoadInteractionsData] PIT '%s' has expired", pit)
			web.status = 500
			local result = {
				result = "REQUEST_FAILED",
				reason = "PIT has expired"
			}
			return api_utils.replyError(web, nil, result)
		end
		dateTimePit = utcDateNow:toString("%Y-%m-%d %H:%M:%S")
	else
		-- Comprobar si hay que generar pit
		local querySuccess, query_ES, error_message, withPit = agentInteractionsController.GetESqueryForSearchInteractionsHelper(data)
		if not querySuccess then
			web.status = error_message and error_message.status or query_ES
			local result = {
				result = error_message and error_message.result or "ERROR",
				reason = error_message and error_message.reason and error_message.reason or ""
			}
			return api_utils.replyError(web, nil, result)
		end	
		-- Para poder hacer el count de lo que nos va a devolver ES, eliminamos atributos de la query que no son compatibles con el count
		query_ES.from = nil
		query_ES.size = nil
		query_ES.sort = nil
		-- Solicitamos el count de la query a ES
		--LogDebug("[API.interactions:LoadInteractionsData] 1------> voy por utils.GetElasticSearchInstance():CountElasticSearch")
		local countSuccess, resultCount = utils.GetElasticSearchInstance():CountElasticSearch(query_ES, vcc, elasticSearchConnectionConfig)
		if not countSuccess then
			web.status = 500
			local result = {
				result = "REQUEST_FAILED",
				reason = "Cant count search results"
			}
			return api_utils.replyError(web, nil, result)
		end
		local countRows = utils.JSON_Decode(resultCount)
		if countRows.count and countRows.count > data.rows then	
			-- Nos va a devolver mas rows que el tamanio de pagina indicado, por lo que generamos el PIT
			pitStatus, pit = utils.GetElasticSearchInstance():PitElasticSearch(vcc, elasticSearchConnectionConfig)
			if not pitStatus then
				web.status = 500
				return api_utils.replyError(web, nil, pit)
			end
			dateTimePit = utcDateNow:toString("%Y-%m-%d %H:%M:%S")
		end
	end

	data.last = tonumber(last)
	LogInfo("[API.interactions:LoadInteractionsData] data.last '%s' pit '%s'",data.last, pit)
	data.pit =  pit and {
		id = pit,
		keep_alive = "180s"
	}
	data.fromAPI = true --This is to extgend search limit just for API requests
	local user_attention_levels = nil
	local excludeSrtipAttributes = nil
	local returnOnlyElastcSearchInfo = true -- Flag para que la informacion de las interacciones que se retornarnar se obtengan desde ES y no vaya ni a Cassandra ni a Redis.
	local status, content = agentInteractionsController.SearchInteractionsHelper(data, user_attention_levels, elasticSearchConnectionConfig, excludeSrtipAttributes, returnOnlyElastcSearchInfo)
	if  status ~= 200 then
		web.status = content and content.status or status
		local result = {
			result = content and content.result or "ERROR",
			reason = content and content.reason and content.reason or ""
		}
		return api_utils.replyError(web, nil, result)
	end

	if content.interactions and #content.interactions > 0 then
		local interactions = {}
		for _, interaction in ipairs(content.interactions) do
			--LogDebug("[API.interactions:LoadInteractionsData] ---> interaction '%s'", json.encode(interaction))
			local interactionData = {
				id                     = interaction.id,
				state                  = interaction.state,
				start_timestamp        = interaction.start_timestamp,
				state_timestamp        = interaction.state_timestamp,
				last_event_timestamp   = interaction.last_event_timestamp,
				type                   = interaction.type,
				total_records_time_sec = interaction.total_records_time_sec or 0,
				total_characters       = interaction.total_characters or 0,
				campaign               = interaction.campaign,
				disposition_code       = interaction.disposition_code,
				agent                  = interaction.agent
			}
			-- agrego a la lista de interacciones
			interactions[interaction.id] = interactionData
		end
		content.interactions = interactions
		content.last = content.last.. (pit and ("|"..pit .."|"..dateTimePit) or "")
	end

	return api_utils.replyOk(web, {content = content})
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en el módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get       = Get
_M.GetAgent  = GetAgent
_M.ListMedia = ListMedia
_M.GetMedia  = GetMedia
_M.LoadInteractionsData = LoadInteractionsData
_M.GetCompositeInteraction = GetCompositeInteraction
-- ARCHIVE & LIVE DETAIL: ejecuta contra la app reports
_M.ConversationsDetail = ConversationsDetail
-------------------------------------------------------------------------

function MapUris (m, authFilter, prefix)
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "interactions"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn) --, Log)
	--
	m_htmlRender = htmlRender
end
