local md5 = require "md5"
local lua_globaltimer = require "lua_globaltimer"
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
local Fullcontact = require "fullcontact"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local appName = "outbound_engine_contact"

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

local function Get (web, contactId)
	local request = {
		contactId = contactId,
		vcc = inconcert.authUser.virtualContactCenter,
	}
	local result = m_outboundEngineClient.GetClient():GetContact(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Get] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function List (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():GetContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:List] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Search (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SearchContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Search] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function GetByName (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():GetContactByName(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:GetByName] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function GetByAddress (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():GetContactByAddress(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:GetByAddress] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Summary (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SummaryContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Summary] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Store (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():StoreContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Store] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function SetAddresses (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SetContactAddresses(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:SetAddresses] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function SetValues (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SetContactValues(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:SetValues] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function SetSkills (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SetContactSkills(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:SetSkills] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function RemoveAddress (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():RemoveContactAddress(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:RemoveAddress] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function EnableAddress (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():EnableContactAddress(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:EnableAddress] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function DisableAddress (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():DisableContactAddress(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:DisableAddress] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Reschedule (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	request.cause = "API"
	local result = m_outboundEngineClient.GetClient():RescheduleContact(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Reschedule] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function GetResults (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():GetContactResults(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:GetResults] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function ValidateAddress (web, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local success, result
	ExecuteFullcontactMethod(function()
		success, result = Fullcontact.Dialer:ValidatePhone(
			request.number, request.country, request.area, vcc, true)
	end)
    if not success then
		LogError("[API:%s:ValidateAddress] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get                       = Get
_M.List                      = List
_M.Search                    = Search
_M.GetByName                 = GetByName
_M.GetByAddress              = GetByAddress
_M.Summary                   = Summary
_M.Store                     = Store
_M.SetAddresses              = SetAddresses
_M.SetValues	             = SetValues
_M.SetSkills	             = SetSkills
_M.RemoveAddress             = RemoveAddress
_M.EnableAddress             = EnableAddress
_M.DisableAddress            = DisableAddress
_M.Reschedule                = Reschedule
_M.GetResults                = GetResults
_M.ValidateAddress           = ValidateAddress
-------------------------------------------------------------------------

function MapUris (m, authFilter, prefix)
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "outbound_engine_contact"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
