local oo = require "loop.base"
local date = require "LuaOrm.Date"
require "LuaOrm.Mapper"
local lua_globaltimer = require "lua_globaltimer"
local ir = require "lua_iragentlink"
local apiModel = require "api.model"
local models = apiModel:GetModels()
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local repositoryController = require("apps.repository")

module(..., package.seeall, inconcert.orbit.application.Decorate)

local function StartRepository()
	repositoryController.ManualStart(inconcert.environment.api.cassandra.ip, 
		inconcert.environment.api.cassandra.port)
end

local function Get (wsapi_env)
	LogDebug("[API.repository:Get] ----> ENTRA")
	StartRepository()
	return repositoryController.retrieveFile(wsapi_env,
		function(request)
			--LogDebug("[API.repository:Get]    1----> repositoryController.retrieveFile (callback)")
			local status, reason = api_utils.validateManualRequest("API:repository:Get",
				request.env.HTTP_AUTHORIZATION, request.env.HTTP_X_CLIENT_IP, "read", request.params, {
					{	id = "id",			required = true,	type = "string" },
					{	id = "section",		required = false,	type = "number", default = -1 },
					{	id = "part",		required = false,	type = "number", default = -1 },
			})
			--LogDebug("[API.repository:Get]   2 ----> repositoryController.retrieveFile (callback) status: %s", status)
			if status ~= 200 then
				return status, api_utils.makeResponseStr(false, reason)
			end
			return 200
		end
	)
end

local function Exists (web)
	StartRepository()
	local status, request = api_utils.validateGet(web, "API:repository:Exists", {
		{	id = "id",			required = true,	type = "string" },
		{	id = "section",		required = false,	type = "number", default = -1 },
		{	id = "part",		required = false,	type = "number", default = -1 },
	})
	if not status then
		return request
	end
	local exists, metadata = repositoryController.existsFile(request.id, request.section, request.part)
	return api_utils.replyOk(web, { exists = exists and true or false, metadata = metadata or nil })
end

local function Add (wsapi_env)
	LogDebug("[API.repository:Add] ----> ENTRA")
	StartRepository()
	return repositoryController.storeFile(wsapi_env, "application/json",
		function(request)
			LogDebug("[API.repository:Add]    ----> repositoryController.storeFile (callback)")
			local status, reason = api_utils.validateManualRequest("API:repository:Add", 
				request.env.HTTP_AUTHORIZATION, request.env.HTTP_X_CLIENT_IP, "write", request.params, {
					{	id = "id",			required = true,	type = "string" },
					{	id = "format",		required = true,	type = "string" },
					{	id = "contentType",	required = true,	type = "string" },
					{	id = "fileName",	required = true,	type = "string" },
					{	id = "section",		required = false,	type = "number", default = -1 },
					{	id = "path",		required = false,	type = "string" },
					{	id = "relation",	required = false,	type = "string" },
			})
			if status ~= 200 then
				return status, api_utils.makeResponseStr(false, reason)
			end
			return 200
		end,
		function(status, reason)
			return api_utils.makeResponseStr(status, reason)
		end
	)
end

local function Delete (web, request)
	StartRepository()
	local status = repositoryController.deleteFile(request.id, request.section)
	if not status then
		return api_utils.replyServerError(web, "error deleting file")
	end
	return api_utils.replyOk(web)
end

local function BatchDelete (web, request)
	StartRepository()
	local result = repositoryController.deleteMultipleFiles(request.files)
	return api_utils.replyOk(web, result)
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Exists      = Exists
_M.Delete      = Delete
_M.BatchDelete = BatchDelete
_M.Get         = Get
_M.Add         = Add
-------------------------------------------------------------------------

function MapUris( m, authFilter, prefix )
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	--
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "repository"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
