local ir = require "lua_iragentlink"
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
require "uuid"
local repositoryController = require("apps.repository")
local campaignsModel = require "api.models.campaigns"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local function Start (web, request)
	LogInfo("[API.whatsapp:Start] Starting")
	--send to ir
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignAtVCC = utils.AppendVcc(request.campaign, vcc)

	--Check if account is a whatsapp account
	local account = campaignsModel.GetCampaignAccount(campaignAtVCC, request.account, nil, nil)	
	if account.proxyType ~= "WHATSAPP" then
		if web.request and web.request.attachments and #web.request.attachments > 0 then
			repositoryController.deleteMultipleFiles(web.request.attachments)
		end		
		reason = "Account '" .. request.account_name .. "' is not a WHATSAPP account"
		LogError("[API:whatsapp:Start] %s", reason)
		return api_utils.replyError(web, 400, reason)
	end
	
	--Check text or at least one attachment is specified
	if request.text == nil then
		if #request.attachments > 0 then
			request.text = request.attachments[1].name
		else
			if web.request and web.request.attachments and #web.request.attachments > 0 then
				repositoryController.deleteMultipleFiles(web.request.attachments)
			end
			reason = "text or attachment is required"
			LogError("[API:whatsapp:Start] %s", reason)
			return api_utils.replyError(web, 400, reason)
		end
	end
	
	local timestamp = os.time()
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:NewWhatsappFromAPI({
		interactionID	= request.interactionId,
		account			= request.account,
		campaignAtVCC	= campaignAtVCC,
		name            = request.name,
		address			= request.address,
		text			= request.text,
		attachments		= request.attachments,
		timestamp = timestamp
	})
	local status, reason = api_utils.getHttpStatus(result)
	if status ~= 200 then
		if web.request and web.request.attachments and #web.request.attachments > 0 then
			repositoryController.deleteMultipleFiles(web.request.attachments)
		end		
		LogError("[API:whatsapp:Start] %s - %s", json.encode(request), reason)
		return api_utils.replyError(web, status, reason)
	end
	LogInfo("[API:whatsapp:Start] %s - started with interactionId %s", json.encode(request), request.interactionId)
	return api_utils.replyOk(web, { interactionId = interactionId })
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Start = Start
-------------------------------------------------------------------------

function MapUris (m, authFilter, prefix)
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	--local replyGetFn = Log .. f .. r
	local apiType = "whatsapp"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, nil, function(web, success)
		LogInfo("[API.whatsapp:MapUris] callback ...")
		if success then
			return
		end
		-- error
		if web.request and web.request.attachments and #web.request.attachments > 0 then
			repositoryController.deleteMultipleFiles(web.request.attachments)
		end
	end)
	--
	m_htmlRender = htmlRender
end
