local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "batch_management"
local baseUri = string.format("/%s/", entityType)

local AVAILABLE_CHANNELS = {
	WHATSAPP = true, 
	SMS = true, 
	RCS = true, 
	MAIL = true
}

local AVAILABLE_VIEWS = {
	ALL = true,
	ACTIVE = true, 
	SCHEDULED = true, 
	ACTIVE_SCHEDULED = true, 
	ARCHIVE = true
}

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {}
}

------------------------------------------------
--[[
	GetBatch batch_management NNNNNNNNNNNNN
		url: /batch_management/get_batch/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "7be36f2e-e605-4214-a7f6-6aec9ad41b79",
	name = "GetBatch",
	description = "Get batch management",
	url = baseUri .. "get_batch/",
	method = "POST",
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = true, type = "string"},
		{id = "account", required = true, account = {name = "account_name", campaign = "campaign"}},
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end , type = "string"
		},
		{id = "batchId", required = true, type = "string"}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetBatch",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------


------------------------------------------------
--[[
	Search batch_management NNNNNNNNNNNNN
		url: /batch_management/search/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "c941a813-aa04-431a-b850-e44e5f75a4e9",
	name = "SearchBatchManagements",
	description = "Search batch managements",
	url = baseUri .. "search/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "view", required = false,	type = "string", domain = AVAILABLE_VIEWS, default = "ALL" },
		{id = "query", required = false,	type = "table", default = {}, children = {
				{id = "type",     required = false, type = "string", domain = AVAILABLE_CHANNELS },
				{id = "campaign", required = false, type = "string"	},
				{id = "account",  required = false, account = { name = "account_name", campaign = "campaign" } },
				{id = "page",     required = false, type = "string" },
				{id = "content",  required = false, type = "string" },
			}
		},
		{id = "from", required = false,	type = "number", default = 0	},
		{id = "rows", required = false,	type = "number", default = 20	},
		{id = "sort", required = false,	type = "string", default = "publish_timestamp" },
		{id = "desc", required = false,	type = "boolean", default = false }
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "Search",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------


------------------------------------------------
--[[
	GetBatchAddress batch_management NNNNNNNNNNNNN
		url: /batch_management/get_batch_address/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "adf2116c-12b5-4812-836a-795a14d2aa83",
	name = "GetBatchAddress",
	description = "Get batch address",
	url = baseUri .. "get_batch_address/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = true, type = "string"},
		{id = "account",  required = true, account = {name = "account_name", campaign = "campaign"}},
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end , type = "string"
		},
		{id = "batchId", required = true, type = "string"},
		{id = "address", required = true, type = "string"}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetBatchAddress",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	GetBatchAddresses batch_management NNNNNNNNNNNNN
		url: /batch_management/get_batch_addresses/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "7b6dfdde-f279-4064-86af-cce3037aca3c",
	name = "GetBatchAddresses",
	description = "Get batch addresses",
	url = baseUri .. "get_batch_addresses/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = true, type = "string"},
		{id = "account", required = true, account = {name = "account_name", campaign = "campaign"}},
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end , type = "string"
		},
		{id = "batchId",   required = true,	 type = "string"},
		{id = "addresses", required = false, type = "table", array = true}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetBatchAddresses",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	SearchBatchAddresses batch_management NNNNNNNNNNNNN
		url: /batch_management/search_batch_addresses/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "004d6d1e-b047-41dc-b21c-ef82e0d36871",
	name = "SearchBatchAddresses",
	description = "Search Batch Addresses",
	url = baseUri .. "search_batch_addresses/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = true, type = "string"	},
		{id = "account", required = true, account = {name = "account_name", campaign = "campaign"}},
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end ,
			type = "string"
		},
		{id = "batchId", required = true, type = "string"},
		{id = "addresses", required = false, type = "table", array = true},
		{id = "query", required = false, type = "table", default = {}, children = {
			{id = "status", required = false, type = "string"},
			{id = "from_status_timestamp", required = false, type = "number"},
			{id = "to_status_timestamp", required = false, type = "number"},
		}},
		{id = "limit",      required = false, type = "number", default = 1000},
		{id = "numberPage",	required = false, type = "number"},
		{id = "sort",       required = false, type = "string", default = "address"},
		{id = "desc",       required = false, type = "boolean", default = false}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "SearchBatchAddresses",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	CreateBatch batch_management NNNNNNNNNNNNN
		url: /batch_management/create_batch/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "c9162e9f-5212-40a4-96ca-4b1bc678a284",
	name = "CreateBatch",
	description = "Create batch",
	url = baseUri .. "create_batch/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "type",		required = true,	type = "string", domain = AVAILABLE_CHANNELS },
		{id = "campaign",	required = true,	type = "string"	},
		{id = "account",		required = true,	account = { name = "account_name", campaign = "campaign" } },
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end , type = "string"
		},
		{id = "batchId",		required = true,	type = "string"	},
		{id = "message",		required = false,	type = "table"	},
		{id = "addresses",	required = false,	type = "table", array = true, children = {
				{id = "address",		required = true,	type = "string" },
				{id = "name",		required = false,	type = "string" },
				{id = "data",		required = false,	type = "table" },
				{id = "agentData",	required = false,	type = "table" },
				{id = "message",		required = false,	type = "table" },
			}
		},
		{id = "reportingGroup", required = false, type = "string" },
		{id = "async",		required = false,	type = "boolean", default = false },
		{id = "timeout",		required = false,	type = "number", default = 3600 },
		{id = "pollingSeconds", required = false, type = "number", default = 10 },
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "CreateBatch",
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	AddAddressToBatch batch_management NNNNNNNNNNNNN
		url: /batch_management/add_address_to_batch/ 
		method: POST
]]--
local definition =	{
	entityType = entityType,
	id = "983b9367-5f3b-4106-a583-2149047982fa",
	name = "AddAddressToBatch",
	description = "Add address to batch",
	url = baseUri .. "add_address_to_batch/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "type",		required = true,	type = "string", domain = AVAILABLE_CHANNELS },
		{id = "campaign",	required = true,	type = "string"	},
		{id = "account",		required = true,	account = { name = "account_name", campaign = "campaign" } },
		{id = "page",
			required = function(request)
				return request.type == "FACEBOOK"
			end , type = "string"
		},
		{id = "batchId",		required = true,	type = "string"	},
		{id = "address",		required = true,	type = "string"	},
		{id = "name",		required = false,	type = "string"	},
		{id = "message",		required = false,	type = "table"	},
		{id = "data",		required = false,	type = "table"	},
		{id = "agentData",	required = false,	type = "table"	},
		{id = "reportingGroup", required = false, type = "string" },
		{id = "async",		required = false,	type = "boolean", default = false },
		{id = "timeout",		required = false,	type = "number", default = 30 },
		{id = "pollingSeconds", required = false, type = "number", default = 3 }
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "AddAddressToBatch",
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "medium_light", -- low, medium_light, medium, high
		--filters    = api_utils.GetApiURLFilters,
		--paged      = api_utils.GetApiURLPaged,
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end