local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "campaigns"
local baseUri = string.format("/%s/", entityType)

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {}
}

------------------------------------------------
--[[
	Get campaign NNNNNNNNNNNNN
		url: /campaigns/get/ 
		method: GET
]]--
local definition =	{
	entityType = entityType,
	id = "ba74934e-bae4-4920-95a5-e092b8c94bf4",
	name = "GetCampaign",
	description = "Get campaign information",
	url = baseUri .. "get/(.+)",
	method = "GET",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {},
	response = {
		campaign = {
			dispositionCodes = {},
			Description = "Campaña de Mail 1",
			VCC = "vcc1",
			LastUpdate = "2019-11-18 11:41:29.000",
			CannedMessages = {},
			permissions = {},
			scheduler = {},
			defaultAgentCriteria = "CIRCULAR",
			outboundPrefix = "7*",
			campaignId = "campmail1",
			ValidationTimeZone = "UYT",
			Id = "campmail1@vcc1",
			Rules = {},
			AttentionLevels = {},
			InteractionNotifications = {},
			updated_at = "2019-11-18 11:41:29.000",
			AlwaysValid = "1",
			Name = "CampMail1",
			Thresholds = {},
			accounts = {},
			Integration = {},
			Labels = {}
		},
		status = true
	},
	responseErrors = {},
	config = {
		urlLinkedFunction = "Get",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	GetDispositions NNNNNNNNNNNNN
		url: /campaigns/get_dispositions/ 
		method: GET
]]--
local definition =	{
	entityType = entityType,
	id = "a532b864-c8a1-4946-a593-6ff828fe2c1e",
	name = "GetCampaignDispositions",
	description = "Get campaign dispositions codes",
	url = baseUri .. "get_dispositions/(.+)",
	method = "GET",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetDispositions",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	GetAccountTemplateMessages NNNNNNNNNNNNN
		url: /campaigns/get_account_template_messages/ 
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "0ebc3c5a-b3e1-4e2d-8b69-5c20231fa070",
	name = "GetAccountTemplateMessages",
	description = "Get account template messages",
	url = baseUri .. "get_account_template_messages/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = true, type = "string"},
		{id = "account",  required = true, account = { name = "account_name", campaign = "campaign" }}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetAccountTemplateMessages",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	GetConfiguredChannels NNNNNNNNNNNNN
		url: /campaigns/get_configured_channels/ 
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "f4c1d4ab-b8b6-40cd-9ee5-d3f0648f2223",
	name = "GetCampaignChannels",
	description = "Get campaign configured channels",
	url = baseUri .. "get_configured_channels/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaign", required = false, type = "string"},
		{id = "enabled",  required = false, type = "boolean", default = false}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetConfiguredChannels",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	GetCampaignMembers NNNNNNNNNNNNN
		url: /campaigns/get_campaign_members/ 
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "624333fc-4513-496a-86e6-e6409a073cc0",
	name = "GetCampaignMembers",
	description = "Get campaign members",
	url = baseUri .. "get_campaign_members/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "query", required = false, type = "table"}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "GetCampaignMembers",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	Search NNNNNNNNNNNNN
		url: /campaigns/search/ 
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "f4c1d4ab-b8b6-40cd-9ee5-d3f0648f2224",
	name = "SearchCampaigns",
	description = "Search campaigns",
	url = baseUri .. "search/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "query",   required = false, type = "table"},
		{id = "columns", required = false, type = "table",
			default = {
				"Id",
				"Name",
				"Description",
				"Thresholds",
				"Accounts",
				"Scheduler"
			}
		},
		-- {id = "limit",      required = false,  type = "number"}, --"rowsPerPage"
		-- {id = "numberPage",	required = false, type = "number"}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "Search",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	SaveIntegrationCCI NNNNNNNNNNNNN
		url: /campaigns/save_integration_cci/ 
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "f4c1d4ab-b8b6-40cd-9ee5-d3f0648f2225",
	name = "SaveCampaignCCI",
	description = "Update CCI in Campaign",
	url = baseUri .. "save_integration_cci/",
	method = "POST",
		header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "campaignId",	 required = true, type = "string"},
		{id = "integration", required = true, type = "table" }
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "SaveIntegrationCCI",
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end