local api_utils = require "api.utils"

module((...), package.seeall)

local entityType = "interactions"
local baseUri = string.format("/%s/", entityType)
local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {
		--[[
			Get Interaction
				url: /interactions/get/interactionid
				method: GET
		]]--
		{
			entityType = entityType,
			id = "5a376978-6763-49c1-bf41-013a2e23051f",
			name = "GetConversation",
			description = "Get Conversation",
			url = baseUri .. "get/(.*)",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "Get"
			}
		},
		--[[
			Get Composite Interaction
				url: /interactions/composite/get/id
				method: GET
		]]--
		{
			entityType = entityType,
			id = "GetCompositeInteraction",
			name = "GetCompositeInteraction",
			description = "Get Composite Interaction",
			url = baseUri .. "composite/get/(.*)",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "GetCompositeInteraction"
			}
		},

		--[[
			Get Agent Interaction
				url: /interactions/get_agent/proxyType/interactionId
				method: GET
		]]--
		{
			entityType = entityType,
			id = "b4a9b0f1-e846-472e-841b-1034b1407e2c",
			name = "GetAgentConversation",
			description = "Get agent of conversation",
			url = baseUri .. "get_agent/(.*)/(.*)",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "GetAgent"
			}
		},

		--[[
			Get Interaction Media List
				url: /interactions/media/list/interactionId
				method: GET
		]]--
		{
			entityType = entityType,
			id = "abb711da-04fc-4802-8904-739df18cd383",
			name = "GetConversationsMediaList",
			description = "Get Conversations Media List",
			url = baseUri .. "media/list/(.*)",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				filters    = api_utils.GetApiURLFilters,
				paged      = api_utils.GetApiURLPaged,
				urlLinkedFunction = "ListMedia"
			}
		},

		--[[
			Get Interaction Media
				url: /interactions/media/get/            asdfasd asdf asdfasdfadsf
				method: GET
				wsapi: true
		]]--
		{
			entityType = entityType,
			id = "1e1e1d02-adcd-4bc0-9638-780db31ff234",
			name = "GetConversationMedia",
			description = "Get Conversation Media",
			url = baseUri .. "media/get/(.*)",
			method = "GET",
			wsapi = true,
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				filters    = api_utils.GetApiURLFilters,
				paged      = api_utils.GetApiURLPaged,
				urlLinkedFunction = "GetMedia"
			}
		},
		{
			entityType = entityType,
			id = "LoadInteractionsData",
			name = "LoadInteractionsData",
			description = "Load interactions data",
			url = baseUri .. "load/",
			method = "POST",
				header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{
					id = "query", required = true,  type = "table", children = {}
				},
				{id = "rows", required = true, type = "rowsPerPage"}
			},
			response = {
			},
			responseErrors = {
				{
					httpCode = 429,
					code     = 3,
					status   = false,
					errType  = "Rate limits error in URL '/interactions/load'. Has exceeded the maximum limit supported in url. Supports 3 requests in 10 seconds",
					reason   = "user request has reached its rate limit"
				},
				{
					httpCode = 400,
					code     = 5,
					status   = false,
					errType  = "has exceeded lines allowed at query",
					reason   = "only 50 rows allowed per page"
				}
			},
			config = {
				urlLinkedFunction = "LoadInteractionsData",
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium", -- low, medium_light, medium, high
				filters    = api_utils.GetApiURLFilters,
				paged      = api_utils.GetApiURLPaged,
			}
		}
	}
}

---------------------------


-----------------
--[[
	Conversations Live Detail
		funcion genercia que agrega la definicion del API para los diferentes tipos de conversaciones, para las conversaciones de detalle de tipo live (active runtime)

		Pueden venir valores especificos para el canal como puede ser la "url", la función a ser ejecutada "ExcuteFnName", etc.
		Si estos valores específicos no llegan, se asignan los valores por defecto al template.
			Ej:
				Si en el parametro "def" no llega el atributo "ExcuteFnName", se ejecutará la función por defecto "ConversationsDetail" que se encuentra en el controller "interactions"
]]--
local function MakeLiveDef (def)
	assert(def.type)

	local id = def.id or ("%sConversationsLiveDetail"):format(def.type)
	local name = def.name or id
	local description = def.description or ("Detail Live Conversations of %s"):format(def.type:upper())
	local url = baseUri
	if def.url then
		url = url .. def.url
	else
		-- Ej: "facebook/live/"
		url = url .. ("%s/live/"):format(def.type:lower())
	end
	local reportId = def.reportId
	local urlLinkedFunction = def.ExcuteFnName or "ConversationsDetail"
	local permission = def.permission or m_apiDefaultConfig.permissions
	local rateLimits = def.rateLimits or api_utils.GetApiURLRateLimit
	local paged = def.paged or api_utils.GetApiURLPaged
	local filters = def.filters and def.filters or api_utils.GetApiURLFilters
	local resourceType = "high"

	local templateDef = {
		entityType = entityType,
		id = id,
		name = name,
		description = description,
		url = url,
		method = "POST",
			header = {
			Authorization = "....",
			["Content-Type"] = "application/json"
		},
		body = {
			{
				id = "queryFields", required = true, type = "table",
				children = {
					{id = "startDate", required = true,	type = "rangeDate", to   = "endDate"  },
					{id = "endDate"  , required = true, type = "rangeDate", from = "startDate"}
				}
			},
			{id = "rowsPerPage", required = true, type = "rowsPerPage"},
			{id = "lastRowOfPage" , required = false     }
		},
		response = {
			{
				content = {
					result = {
						rows ={
							{
								id = "7596940111CB361F5F9867A5D38A2FE1"
							},
							{
								id = "62d9a21b52824392aa34f34df5036fe5"
							},
							{
								id = "8DCAC0B3FB7AE47DE38D82C4C7CA2B7C"
							}
						}
					},
					last_handle = "lastRowHandleId"
				},
				status = true
			}
		},
		responseErrors = {
			{
				httpCode = 429,
				code     = 3,
				status   = false,
				errType  = "Rate limits error in URL '/interactions/mail/archive/by_campaign/'. Has exceeded the maximum limit supported in url. Supports 3 requests in 10 seconds",
				reason   = "user request has reached its rate limit"
			},
			{

				httpCode = 400,
				code     = 4,
				status   = false,
				errType  = "date parameters out of range",
				reason   = "date parameters out of range. from: startDate, to: endDate. Queries with a maximum range of 360 days are allowed."
			},
			{
				httpCode = 400,
				code     = 5,
				status   = false,
				errType  = "has exceeded lines allowed at query",
				reason   = "only 50 rows allowed per page"
			}
		},
		config = {
			reportId          = reportId,  -- Ej: "ReportDetailLiveFacebookByCampaign",
			type              = def.type,
			urlLinkedFunction = urlLinkedFunction, -- Ej: se ejecutará la función "ConversationsLiveDetail" que se encuentra en el controller "interactions"
			permission        = permission.read,
			rateLimits        = rateLimits,
			resourceType      = resourceType,
			filters           = filters,
			paged             = paged
		}
	}

	return templateDef
end

--[[
	Tabla con la definición del API para conversaciones de detalle de tipo live.
	Las mismas se utilizarán en la función "MakeLiveDef" para crear la definición del API.
]]--
local conversationsLiveDetailDefinitions = {
	{
		type     = "facebook", -- se utilizará como parte de la definicón de la URL
		reportId = "ReportDetailLiveFacebookByCampaign" -- reporte a ser ejecutado
	},
	{
		type     = "mail",
		reportId = "ReportDetailLiveMailsByCampaign",
		---- configuraciones especificas (no necesarias)
		-- filters = {
		-- 	rangeDate = {
		-- 		minutes = 518400 -- 365 dias
		-- 	}
		-- },
		-- rateLimits = {
		-- 	timeLimitSec = 10,
		-- 	maxRequests = 1
		-- },
		-- paged = {
		-- 	rowsPerPage = 1
		-- }
	},
	{
		type     = "twitter",
		reportId = "ReportDetailLiveTwitterByCampaign"
	},
	{
		type     = "call",
		reportId = "ReportDetailLiveCallsByCampaign"
	},
	{
		type     = "chat",
		reportId = "ReportDetailLiveChatsByCampaign"
	},
	{
		type     = "instagram",
		reportId = "ReportDetailLiveInstagramByCampaign"
	},
	{
		type     = "whatsapp",
		reportId = "ReportDetailLiveWhatsappByCampaign"
	},
	{
		type     = "webcontact",
		reportId = "ReportDetailLiveWebcontactByCampaign"
	},
	{
		type     = "voicemail",
		reportId = "ReportDetailLiveVoicemailsByCampaign"
	},
}

for _, def in ipairs(conversationsLiveDetailDefinitions) do
	local itemDef = MakeLiveDef(def)
	table.insert(m_apiDefinition.items, itemDef)
end
-- ---------------------------------------------

-----------------
--[[
	Conversations Live Detail
		funcion genercia que agrega la definicion del API para los diferentes tipos de conversaciones, para las conversaciones de detalle de tipo live (active runtime)

		Pueden venir valores especificos para el canal como puede ser la "url", la función a ser ejecutada "ExcuteFnName", etc.
		Si estos valores específicos no llegan, se asignan los valores por defecto al template.
			Ej:
				Si en el parametro "def" no llega el atributo "ExcuteFnName", se ejecutará la función por defecto "ConversationsLiveDetail" que se encuentra en el controller "interactions"
]]--
local function MakeArchiveDef (def)
	assert(def.type)

	local id = def.id or ("%sConversationsArchiveDetail"):format(def.type)
	local name = def.name or id
	local description = def.description or ("Detail Archive Conversations of %s"):format(def.type:upper())
	local url = baseUri
	if def.url then
		url = url .. def.url
	else
		-- Ej: "facebook/archive/"
		url = url .. ("%s/archive/"):format(def.type:lower())
	end
	local reportId = def.reportId
	local urlLinkedFunction = def.ExcuteFnName or "ConversationsDetail"
	local permission = def.permission or m_apiDefaultConfig.permissions
	local rateLimits = def.rateLimits or api_utils.GetApiURLRateLimit
	local paged = def.paged or api_utils.GetApiURLPaged
	local filters = def.filters and def.filters or api_utils.GetApiURLFilters
	local resourceType = "high"

	local templateDef = {
		entityType = entityType,
		id = id,
		name = name,
		description = description,
		url = url,
		method = "POST",
			header = {
			Authorization = "....",
			["Content-Type"] = "application/json"
		},
		body = {
			{
				id = "queryFields", required = true   ,  type = "table", children = {
					{id = "startDate", required = true,	 type = "rangeDate", to   = "endDate"  },
					{id = "endDate"  , required = true,  type = "rangeDate", from = "startDate"},
					{id = "CCI"      , required = false, type = "table",
						children = {
							{name = "FieldId_1", value = "Value1", operator = "=" },
							{name = "FieldId_2", value = "Value2", operator = "<>"},
							{name = "FieldId_3", value = 1, value_to = 10, operator = "between"}
						}
					}
				}
			},
			{id = "rowsPerPage", required = true, type = "rowsPerPage"},
			{id = "lastRowOfPage" , required = false     }
		},
		response = {
			{
				content = {
					result = {
						rows ={
							{
								id = "7596940111CB361F5F9867A5D38A2FE1"
							},
							{
								id = "62d9a21b52824392aa34f34df5036fe5"
							},
							{
								id = "8DCAC0B3FB7AE47DE38D82C4C7CA2B7C"
							}
						}
					},
					last_handle = "lastRowHandleId"
				},
				status = true
			}
		},
		responseErrors = {
			{
				httpCode = 429,
				code     = 3,
				status   = false,
				errType  = "Rate limits error in URL '/interactions/mail/archive/by_campaign/'. Has exceeded the maximum limit supported in url. Supports 3 requests in 10 seconds",
				reason   = "user request has reached its rate limit"
			},
			{

				httpCode = 400,
				code     = 4,
				status   = false,
				errType  = "date parameters out of range",
				reason   = "date parameters out of range. from: startDate, to: endDate. Queries with a maximum range of 360 days are allowed."
			},
			{
				httpCode = 400,
				code     = 5,
				status   = false,
				errType  = "has exceeded lines allowed at query",
				reason   = "only 50 rows allowed per page"
			}
		},
		config = {
			reportId = reportId,  -- Ej: "ReportDetailMailsByCampaign",
			resourceType = resourceType,
			type     = def.type,
			urlLinkedFunction = urlLinkedFunction, -- Ej: se ejecutará la función "ConversationsDetail" que se encuentra en el controller "interactions"
			permission = permission.read,
			rateLimits = rateLimits,
			filters    = filters,
			paged      = paged
		}
	}

	return templateDef
end

--[[
	Tabla con la definición del API para conversaciones de detalle de tipo live.
	Las mismas se utilizarán en la función "MakeArchiveDef" para crear la definición del API.
]]--
local conversationsArchiveDetailDefinitions = {
	-- facebook
	{
		type     = "facebook",
		url      = "facebook/archive/by_campaign/",
		reportId = "ReportDetailFacebookByCampaign"
	},
	{
		type     = "facebook",
		url      = "facebook/archive/by_agent/",
		reportId = "ReportDetailFacebookByAgent"
	},
	-- mail
	{
		type     = "mail",
		url      = "mail/archive/by_campaign/",
		reportId = "ReportDetailMailsByCampaign"
	},
	{
		type     = "mail",
		url      = "mail/archive/by_agent/",
		reportId = "ReportDetailMailsByAgent"
	},
	-- twitter
	{
		type     = "twitter",
		url      = "twitter/archive/by_campaign/",
		reportId = "ReportDetailTwitterByCampaign"
	},
	{
		type     = "twitter",
		url      = "twitter/archive/by_agent/",
		reportId = "ReportDetailTwitterByAgent"
	},
	-- call
	{
		type     = "call",
		url      = "call/archive/by_campaign/",
		reportId = "ReportDetailInOutCallsByCampaignI6"
	},
	{
		type     = "call",
		url      = "call/archive/by_agent/",
		reportId = "ReportDetailInOutCallsByAgentI6"
	},
	-- Chat
	{
		type     = "chat",
		url      = "chat/archive/by_campaign/",
		reportId = "ReportDetailInboundChatsByCampaign"
	},
	{
		type     = "chat",
		url      = "chat/archive/by_agent/",
		reportId = "ReportDetailInboundChatsByAgent"
	},
	-- Whatsapp
	{
		type     = "whatsapp",
		url      = "whatsapp/archive/by_campaign/",
		reportId = "ReportDetailWhatsappByCampaign"
	},
	{
		type     = "whatsapp",
		url      = "whatsapp/archive/by_agent/",
		reportId = "ReportDetailWhatsappByAgent"
	},
	-- Instagram
	{
		type     = "instagram",
		url      = "instagram/archive/by_campaign/",
		reportId = "ReportDetailInstagramByCampaign"
	},
	{
		type     = "instagram",
		url      = "instagram/archive/by_agent/",
		reportId = "ReportDetailInstagramByAgent"
	},
	-- Webcontacts
	{
		type     = "webcontact",
		url      = "webcontact/archive/by_campaign/",
		reportId = "ReportDetailWebcontactsByCampaign"
	},
	{
		type     = "webcontact",
		url      = "webcontact/archive/by_agent/",
		reportId = "ReportDetailWebcontactsByAgent"
	},
	-- Voicemails
	{
		type     = "voicemail",
		url      = "voicemail/archive/by_campaign/",
		reportId = "ReportDetailVoicemailsByCampaign"
	},
	{
		type     = "voicemail",
		url      = "voicemail/archive/by_agent/",
		reportId = "ReportDetailVoicemailsByAgent"
	}
}

for _, def in ipairs(conversationsArchiveDetailDefinitions) do
	local itemDef = MakeArchiveDef(def)
	table.insert(m_apiDefinition.items, itemDef)
end

-----------------------------------------------------------------------------
function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end