require "uuid"
local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "mail"
local baseUri = string.format("/%s/", entityType)

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {
		--[[
			Start 
				url: /mail/start/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "b39c497c-f346-4878-b0e7-40c59497b710",
			name = "StartMailConversation",
			description = "Start Mail Conversation",
			url = baseUri .. "start/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{	id = "campaign",					required = true,	type = "string"	},
				{	id = "account",						required = true,	account = { name = "account_name", campaign = "campaign" } },
				{	id = "interactionId", 				required = false,	type = "string", default = uuid.new() },
						
				{	id = "head",						required = true,	type = "table", children = {
						{	id = "fromEMail",			required = true,	type = "string" },
						{	id = "fromName",			required = true,	type = "string" },
						{	id = "cc",					required = false,	type = "table", array = true },
						{	id = "subject",				required = true,	type = "string" },
					}		
				},		
				{	id = "body",						required = false,	type = "string" },
				{	id = "attachments",					required = false,	type = "table", array = true, default = {}, children = {
						{	id = "name",				required = true,	type = "string" },
						{	id = "type",				required = true,	type = "string" },
						{	id = "id",					required = true,	type = "string" },
						{	id = "content-disposition",	required = true,	type = "string" },
						{	id = "section",	            required = true,	type = "number" },
					}
				}
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Start"
			}
		}
	}
}

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end