local api_utils = require "api.utils"
local oe_utils = require "api.outbound_engine_utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "outbound_engine_batch"
local baseUri = "/outbound_engine/batches/"

local AVAILABLE_CHANNELS = {
	CALL = true, 
}

local AVAILABLE_ENGINE_TYPES = {
	PROGRESSIVE = true,
	[""] = true,
}

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {

		--[[
			Get Action Status
				url: /outbound_engine/batches/action_status
				method: GET
		]]--
		{
			entityType = entityType,
			id = "fc06c65d-c96b-4b25-ab35-f45aa67d641f",
			name = "GetActionStatus",
			description = "Get Batch Action Status",
			url = baseUri .. "action_status/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "actionId", 		required = true,	type = "string" },
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "GetActionStatus"
			}
		},

		--[[
			Create Batch
				url: /outbound_engine/batches/create/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "661e5f66-6ba9-4a6f-b87f-2f826cceb42d",
			name = "CreateBatch",
			description = "Create a batch",
			url = baseUri .. "create/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "configuration", 	required = true,	type = "table", children = {
						{id = "name",     		required = true, type = "string" },
						{id = "startDate",  	required = false, type = "string" },
						{id = "endDate",  		required = false, type = "string" },
						{id = "batchPriority",  required = false, type = "number" },
						{id = "priority",  		required = false, type = "number" },
						{id = "percentage",  	required = false, type = "number" },
						{id = "scheduleDate",  	required = false, type = "string" },
						{id = "agent",  		required = false, type = "string" },
						{id = "engineType",  	required = false, type = "string" },
						{id = "channel",  		required = false, type = "string" },
						{id = "kind",  			required = false, type = "string" },
					}
				},
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "high", -- low, medium_light, medium, high
				urlLinkedFunction = "Create"
			}
		},

		--[[
			Update Batch
				url: /outbound_engine/batches/update/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "a7be145e-3e51-476e-a128-12297d3e62dc",
			name = "UpdateBatch",
			description = "Update a batch",
			url = baseUri .. "update/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "batchId", 		required = true,	type = "string" },
				{id = "configuration", 	required = true,	type = "table", children = {
						{id = "startDate",  	required = false, type = "string" },
						{id = "endDate",  		required = false, type = "string" },
						{id = "batchPriority",  required = false, type = "number" },
						{id = "basePriority",  	required = false, type = "number" },
						{id = "percentage",  	required = false, type = "number" },
					}
				},
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "high", -- low, medium_light, medium, high
				urlLinkedFunction = "Update"
			}
		},

		--[[
			Destroy Batch
				url: /outbound_engine/batches/destroy/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "114b317a-2bba-4e42-a69a-923fa6a5f17e",
			name = "DestroyBatch",
			description = "Destroy a batch",
			url = baseUri .. "destroy/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "batchId", 		required = true,	type = "string" },
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "high", -- low, medium_light, medium, high
				urlLinkedFunction = "Destroy"
			}
		},

		--[[
			Add Contacts to Batch
				url: /outbound_engine/batches/add_contacts/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "07d1d356-98f9-43e0-bd4d-b50812747978",
			name = "AddContactsToBatch",
			description = "Add contacts to a batch",
			url = baseUri .. "add_contacts/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "batchId", 		required = true,	type = "string" },
				{id = "configuration", 	required = true,	type = "table", children = {
						{id = "scheduleDate",  	required = false, type = "string" },
						{id = "agent",  		required = false, type = "string" },
						{id = "engineType",  	required = false, type = "string", domain = AVAILABLE_ENGINE_TYPES },
						{id = "channel",  		required = false, type = "string", domain = AVAILABLE_CHANNELS },
						{id = "kind",  			required = false, type = "string" },
						{id = "address",  		required = false, type = "table", children = oe_utils.GetAddressFieldsDefinition() },
						{id = "contactData", 	required = false, type = "table", children = {
								{id = "Name",     			required = false, type = "string" },
								{id = "Campaign",     		required = false, type = "string" },
								{id = "Category",     		required = false, type = "string" },
								{id = "AccountOfficer",     required = false, type = "string" },
								{id = "ExternalId",     	required = false, type = "string" },
								{id = "ImportId",     		required = false, type = "string" },
								{id = "AccountGroup",     	required = false, type = "string" },
								{id = "IsClient",     		required = false, type = "boolean" },
								{id = "Addresses",     		required = false, type = "table", array = true,
									children = oe_utils.GetAddressFieldsDefinition() },
								{id = "ManualScheduleDate", required = false, type = "string" },
								{id = "Agent",     			required = false, type = "string" },
								{id = "Skills", 			required = false, type = "table", array = true, children = {
										{id = "SkillId",     	required = true, type = "string" },
										{id = "Mode",  			required = false, type = "string", default = "SORT",
											domain = { SORT = true, REQUIRED = true, PREFERRED = true } },
										{id = "Value",  	required = true, type = "number" },
									}
								},
								{id = "NameValuesSimple", 	required = false, type = "table", array = true, children = {
										{id = "Name",     		required = true, type = "string" },
										{id = "Value",  		required = true, type = "string" },
									}
								},
								{id = "NameValuesSearchDate", 	required = false, type = "table", array = true, children = {
										{id = "Name",     		required = true, type = "string" },
										{id = "Value",  		required = true, type = "string" },
									}
								},
								{id = "NameValuesSearchNumeric", required = false, type = "table", array = true, children = {
										{id = "Name",     		required = true, type = "string" },
										{id = "Value",  		required = true, type = "number" },
									}
								},
								{id = "NameValuesSearchText", 	required = false, type = "table", array = true, children = {
										{id = "Name",     		required = true, type = "string" },
										{id = "Value",  		required = true, type = "string" },
									}
								},
							}
						},
					}
				},
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "AddContacts"
			}
		},

		--[[
			Pause Contacts
				url: /outbound_engine/batches/pause/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "aef571a7-d763-4522-9347-0015626c9aaa",
			name = "PauseContacts",
			description = "Pause contacts",
			url = baseUri .. "pause/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Pause"
			}
		},

		--[[
			Resume Contacts
				url: /outbound_engine/batches/resume/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "7f95dd6f-4819-4464-8f3e-db860ec3ad47",
			name = "ResumeContacts",
			description = "Resume contacts",
			url = baseUri .. "resume/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Resume"
			}
		},

		--[[
			Cancel Contacts
				url: /outbound_engine/batches/cancel/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "5d3aaa7e-adad-43f7-be07-a1775cf8cf21",
			name = "CancelContacts",
			description = "Cancel contacts",
			url = baseUri .. "cancel/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Cancel"
			}
		},

		--[[
			Finish Contacts
				url: /outbound_engine/batches/finish/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "66fd7e17-cfdf-4409-b7a9-ca7e68474e33",
			name = "FinishContacts",
			description = "Finish contacts",
			url = baseUri .. "finish/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Finish"
			}
		},

		--[[
			Remove Contacts
				url: /outbound_engine/batches/remove/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "66fd7e17-cfdf-4409-b7a9-ca7e68474e33",
			name = "RemoveContacts",
			description = "Remove contacts",
			url = baseUri .. "remove/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Remove"
			}
		},

		--[[
			Recycle Contacts
				url: /outbound_engine/batches/recycle/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "ce3bf40a-71c3-4605-8cb7-604645769d0a",
			name = "RecycleContacts",
			description = "Recycle contacts",
			url = baseUri .. "recycle/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "configuration", 	required = true,	type = "table", children = {
						{id = "restartRetries",  	required = false, type = "boolean" },
						{id = "scheduleDate",  		required = false, type = "string" },
						{id = "agent",  			required = false, type = "string" },
					}
				},
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Recycle"
			}
		},

		--[[
			Schedule Contacts
				url: /outbound_engine/batches/schedule/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "49ccf69f-7ed5-43a4-81e8-57b488abb1b9",
			name = "ScheduleContacts",
			description = "Schedule contacts",
			url = baseUri .. "schedule/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "configuration", 	required = true,	type = "table", children = {
						{id = "restartRetries",  	required = false, type = "boolean" },
						{id = "scheduleDate",  	required = false, type = "string" },
						{id = "agent",  		required = false, type = "string" },
						{id = "channel",  		required = false, type = "string" },
						{id = "kind",  			required = false, type = "string" },
						{id = "address",  		required = false, type = "table", children = oe_utils.GetAddressFieldsDefinition() },
					}
				},
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "Schedule"
			}
		},

		--[[
			Change Priority Contacts
				url: /outbound_engine/batches/change_priority/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "f855bc5b-9e1e-4a69-a745-eda36f949b94",
			name = "ChangePriorityContacts",
			description = "Change Priority contacts",
			url = baseUri .. "change_priority/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "priority",  		required = true, 	type = "number" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "ChangePriority"
			}
		},

		--[[
			Set Engine Type Contacts
				url: /outbound_engine/batches/set_engine_type/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "69af719d-35ef-4f41-b137-3630bf381c11",
			name = "SetEngineTypeContacts",
			description = "Set Engine Type contacts",
			url = baseUri .. "set_engine_type/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "engineType", 	optional = true,	type = "string", domain = AVAILABLE_ENGINE_TYPES },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "SetEngineType"
			}
		},

		--[[
			Set Agent Contacts
				url: /outbound_engine/batches/set_agent/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "d6f1b48b-4514-4c26-bf09-ddd88111951d",
			name = "SetAgentContacts",
			description = "Set Agent contacts",
			url = baseUri .. "set_agent/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "processId", 		required = false,	type = "string" },
				{id = "campaignId", 	required = false,	type = "string" },
				{id = "agent", 			required = true,	type = "string" },
				oe_utils.GetScopeDefinition(),
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				urlLinkedFunction = "SetAgent"
			}
		},
	}
}

-- ---------------------------------------------

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end