local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "users"
local baseUri = string.format("/%s/", entityType)

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {
		--[[
			Search Users
				url: /users/search/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "d12a4d5d-a550-467f-8b64-5d1459b2aaa9",
			name = "SearhUsers",
			description = "Search Users Information",
			url = baseUri .. "search/",
			method = "POST",
			header = {

			},
			body = {
				{id = "query",      required = false, type = "table"},
				{id = "columns",    required = false, type = "table", default = {
						"Id", 
						"Name",
						"FirstSurname",
						"UserGroup",
						"Status",
						"StatusReason",
						"StatusTimestamp",
						--Solicitados por WFM
						"TimeZoneId",
						"skills",
						"Mail",
						"LanguageId",
						"Campaigns"
					}
				},
				{id = "limit",      required = false, type = "number"},
				{id = "numberPage", required = false, type = "number"},
						--	{id = "TENES_QUE_FALLAR", required = true, type = "number"},
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				filters    = api_utils.GetApiURLFilters,
				paged      = api_utils.GetApiURLPaged,
				urlLinkedFunction = "Search"
			}
		},

		--[[
			Get User
				url: /users/get/userid@vcc
				method: GET
		]]--
		{
			entityType = entityType,
			id = "1e8a9e51-b464-42cb-b13e-1e0f59ca73ae",
			name = "GetUser",
			description = "Get User Information",
			url = baseUri .. "get/(.+)",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "Get",
				-- Ejemplo de validacion de permisos (override de VCCs) solo override a nivel VCC !!
				occPermissions = {
					appId    = "administration",
					actionId = "viewusers"
				}
			}
		},
		--[[
			Get Users Export Info
				url: /users/get_export_info
				method: GET
		]]--
		{
			entityType = entityType,
			id = "1e8a9e51-b464-42cb-b13e-1e0f59ca73oi",
			name = "GetUser",
			description = "Get Users Export Information",
			url = baseUri .. "get_export_info/",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "GetUsersExportInfo"
			}
		},

		--[[
			Get Available telephony peers
				url: /users/get_available_telephony_peers/
				method: GET
		]]--
		{
			entityType = entityType,
			id = "805e51ac-4337-420a-98d7-bba6af6b794d",
			name = "GeAvailablePeers",
			description = "Get available telephony peers",
			url = baseUri .. "get_available_telephony_peers/",
			method = "GET",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.read,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				filters    = api_utils.GetApiURLFilters,
				paged      = api_utils.GetApiURLPaged,
				urlLinkedFunction = "GetAvailableTelephonyPeers"
			}
		},


		--[[
			Save User
				url: /users/save/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "b86a1bc5-209f-4ba9-ba8c-2125f1057549",
			name = "SaverUser",
			description = "Create or update User",
			url = baseUri .. "save/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{	id = "Id",				required = true,	type = "string" },
				{	id = "Name",			required = true,	type = "string" },
				{	id = "FirstSurname",	required = true,	type = "string" },
				{	id = "LastName",		required = false,	type = "string" },
				{	id = "LastSurname",		required = false,	type = "string" },
				{	id = "Mail",			required = false,	type = "string" },
				{	id = "TimeZoneId",		required = true,	type = "string" },
				{	id = "LanguageId",		required = false,	type = "string", default = "en" },
				{	id = "countryCode",		required = true,	type = "string" },
				{	id = "areaCode",		required = false,	type = "string" },
				{	id = "UserGroup",		required = true,	type = "string" },
				{	id = "WorkgroupId",		required = false,	type = "string" },
				{	id = "DefaultCampaign",	required = false,	type = "string" },
				{	id = "initialState",	required = false,	type = "string" },
				{	id = "IsSystemUser",	required = false,	type = "string", domain = { ["0"] = true, ["1"] = true }, default = "0" },
				{	id = "CompanyInformation",	required = false,	type = "table", children = {
						{	id = "Location",		required = false,	type = "string" },
						{	id = "Position",		required = false,	type = "string" },
						{	id = "Organization",	required = false,	type = "string" },
					} 
				},
				{	id = "CustomInformation",	required = false,	type = "table", children = {
						{	id = "custom_agent_info_1",		required = false,	type = "string" },
						{	id = "custom_agent_info_2",		required = false,	type = "string" },
						{	id = "custom_agent_info_3",		required = false,	type = "string" },
						{	id = "custom_agent_info_4",		required = false,	type = "string" },
						{	id = "custom_agent_info_5",		required = false,	type = "string" }
					} 
				},
				{	id = "Password",	type = "string",	required = 
					function(data)
						return data.isNew
					end, 
				},
				{	id = "skills",		required = false,	type = "table", array = true, children = {
						{	id = "skill",					required = true,	type = "string" },
						{	id = "weight",					required = true,	type = "numeric" },
					} 
				},
				{	id = "groups",		required = false,	type = "table", array = true, children = {
						{	id = "id",						required = true,	type = "string" },
						{	id = "role",					required = false,	type = "string" },
						{	id = "invite",					required = false,	type = "boolean" },
					} 
				},
				{	id = "custom_permissions",	required = false,	type = "table", array = true, children = {
						{	id = "has_override",	required = true,	type = "string", domain = { ["0"] = true, ["1"] = true }, default = "0" },
						{	id = "apps",					required = false,	type = "table" },
						{	id = "actions",					required = false,	type = "table" },
					} 
				},
				{	id = "AllegroIntegration",	required = false,	type = "table", children = {
						{	id = "peer",	required = false,	type = "table", children = {
								{	id = "service",		required = true,	type = "string" },
								{	id = "peer",		required = true,	type = "string" },
								{	id = "technology",	required = true,	type = "string" },
							} 
						}
					} 
				}
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "Save"
			}
		},

		--[[
			Activate User
				url: /users/activate/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "8fc651b3-88d1-4de5-8c08-e45ac80a73a4",
			name = "ActivateUser",
			description = "Activate User",
			url = baseUri .. "activate/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "memberId",	required = true,	type = "string" },
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "Activate"
			}
		},

		--[[
			Deactivate User
				url: /users/deactivate/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "4c674e21-563a-460e-948f-c78cc47412c7",
			name = "DeactivateUser",
			description = "Deactivate User",
			url = baseUri .. "deactivate/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{id = "memberId",	required = true,	type = "string" },
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "low", -- low, medium_light, medium, high
				urlLinkedFunction = "Deactivate"
			}
		}
	}
}

-----------------
--[[
	Users States
		funcion genercia que agrega la definicion del API para los diferentes tipos de conversaciones, para las conversaciones de detalle de tipo live (active runtime)

		Pueden venir valores especificos para el canal como puede ser la "url", la función a ser ejecutada "ExcuteFnName", etc.
		Si estos valores específicos no llegan, se asignan los valores por defecto al template.
			Ej:
				Si en el parametro "def" no llega el atributo "ExcuteFnName", se ejecutará la función por defecto "ExecuteReport" que se encuentra en el controller "users"
]]--
local function MakeReportsDefinitions (def)
	assert(def.type)

	local id = def.id or ("%susersReportsDefinitionsDetail"):format(def.type)
	local name = def.name or id
	local description = def.description or ("Detail Archive Users States of %s"):format(def.type:upper())
	local url = baseUri
	if def.url then
		url = url .. def.url
	else
		-- Ej: "facebook/archive/"
		url = url .. ("%s/archive/"):format(def.type:lower())
	end
	local reportId = def.reportId
	local urlLinkedFunction = def.ExcuteFnName or "ExecuteReport"
	local permission = def.permission or m_apiDefaultConfig.permissions
	local rateLimits = def.rateLimits or api_utils.GetApiURLRateLimit
	local response = def.response or {}
	local responseErrors = def.responseErrors or {}
	local paged = def.paged or api_utils.GetApiURLPaged
	local filters = def.filters and def.filters or api_utils.GetApiURLFilters
	local body = def.body
	local resourceType = "high"

	local templateDef = {
		entityType = entityType,
		id = id,
		name = name,
		description = description,
		url = url,
		method = "POST",
			header = {
			Authorization = "....",
			["Content-Type"] = "application/json"
		},
		body = body or {
			{
				id = "queryFields", required = true, type = "table",
				children = {
					{id = "startDate", required = true,	type = "rangeDate", to   = "endDate"  },
					{id = "endDate"  , required = true, type = "rangeDate", from = "startDate"}
				}
			},
			{id = "rowsPerPage", required = true, type = "rowsPerPage"},
			{id = "lastRowOfPage" , required = false     }
		},
		response = response,
		responseErrors = responseErrors,
		config = {
			reportId            = reportId,  -- Ej: "ReportDetailAgentStatesByCampaign",
			resourceType        = resourceType,
			type                = def.type,
			urlLinkedFunction = urlLinkedFunction, -- Ej: se ejecutará la función "ConversationsDetail" que se encuentra en el controller "interactions"
			permission          = permission.read,
			rateLimits          = rateLimits,
			filters             = filters,
			paged               = paged
		}
	}

	return templateDef
end

--[[
	Tabla con la definición del API para estados de usuarios de detalle de tipo.
	Las mismas se utilizarán en la función "MakeReportsDefinitions" para crear la definición del API.
]]--
local usersReportsDefinitions = {
	-- realtime
	{
		type        = "realtime",
		url         = "realtime/logged_in/",
		name        = "Agents Logged In",
		description = "Detail Real Time of Agents Logged In.",
		reportId = "ReportRealTimeAgentLoggedIn",
		body        = {}, -- tiene que ir vacio para que no tome el por defecto.
		response    = {
			{
				status = true,
				content = {
					result = {
						rows = {
							{
								LastStatusChangeTime = "2022-10-07 16:07:32",
								Campaign = "campfacebook",
								VCC = "vcc1",
								LastLoginTime = "2022-10-07 16:07:32",
								Agent = "agente01",
								SessionId = "3224ac5853444d07b3cb27b4bd7b3c5e",
								StateRaw = "BREAK"
							},
							{}
						}
					}
				}
			}
		}
	},
	-- states
	{
		type     = "states",
		url      = "states/by_campaign/",
		reportId = "ReportDetailAgentStatesByCampaign"
	},
	{
		type     = "states",
		url      = "states/by_agent/",
		reportId = "ReportDetailAgentStatesByAgent",
		body     = {
			{
				id = "queryFields", required = true, type = "table",
				children = {
					{id = "startDate", required = true,	type = "rangeDate", to   = "endDate"  },
					{id = "endDate"  , required = true, type = "rangeDate", from = "startDate"},
					{id = "Agent"    , required = true, type = "string"                       }
				}
			},
			{id = "rowsPerPage", required = true, type = "rowsPerPage"},
			{id = "lastRowOfPage" , required = false     }
		}
	}
}

for _, def in ipairs(usersReportsDefinitions) do
	local itemDef = MakeReportsDefinitions(def)
	table.insert(m_apiDefinition.items, itemDef)
end
-- ---------------------------------------------

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end