require "uuid"
local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "webcontact"
local baseUri = string.format("/%s/", entityType)

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {
		--[[
			Start 
				url: /webcontact/start/
				method: POST
		]]--
		{
			entityType = entityType,
			id = "6a5d2158-2a55-400e-9160-db822e78d622",
			name = "StartWebContactConversation",
			description = "Start WebContact Conversation",
			url = baseUri .. "start/",
			method = "POST",
			header = {
				Authorization = "....",
				["Content-Type"] = "application/json"
			},
			body = {
				{	id = "campaign",	required = true,	type = "string"	},
				{	id = "account",		required = true,	account = { name = "account_name", campaign = "campaign" } },
				{	id = "interactionId", required = false,	type = "string", default = uuid.new() },
				{	id = "contact",		required = false,	type = "table", default = {}, children = {
						{	id = "externalId",	required = false,	type = "string" },
						{	id = "name",		required = false,	type = "string" },
						{	id = "addresses",	required = false,	type = "table", array = true, children = {
								{	id = "type",		required = true,	type = "string", 
									domain = { MAIL = true, CELLULAR = true, HOME = true, OFFICE = true, TWITTER = true } },
								{	id = "address",		required = true,	type = "string" },
							}
						},
					}
				},
				{	id = "reportingGroup", required = false, type = "string", default = "" },
				{	id = "reportingData",	required = false,	type = "table", array = true, default = {}, children = {
						{	id = "name",		required = true,	type = "string" },
						{	id = "value",		required = true,	type = "string" },
					}
				},
				{	id = "agentData",	required = false,	type = "table", array = true, default = {}, children = {
						{	id = "name",		required = true,	type = "string" },
						{	id = "value",		required = true,	type = "string" },
						{	id = "displayName",	required = false,	type = "string" },
						{	id = "hidden",		required = false,	type = "boolean" },
					}
				}
			},
			response = {},
			responseErrors = {},
			config = {
				permission = m_apiDefaultConfig.permissions.write,
				rateLimits = api_utils.GetApiURLRateLimit,
				resourceType = "medium_light", -- low, medium_light, medium, high
				--filters    = api_utils.GetApiURLFilters,
				--paged      = api_utils.GetApiURLPaged,
				urlLinkedFunction = "Start"
			}
		}
	}
}

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end