module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "ApiUsers"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetApiUsers (vcc, userId, keyspace)
	local users = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		key = vcc,
		columns = userId and {userId} or {}
	}
	if userId then
		return users and users[userId] or {}
	else
		return users or {}
	end
end

function SaveApiUsers (vcc, users, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		rows = {
			[vcc] = users
		}
	}
	return true
end

function RemoveApiUsers (vcc, users, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		rows = {
			[vcc] = users
		}
	}
	return true
end

local function GetApiSystemConfig ()
	local systemModel = require "api.models.system"
	local API_SDK_SystemConfig = systemModel.Get("API_SDK")
	return API_SDK_SystemConfig
end

-------
local m_apiConfiguration = {}
m_apiConfiguration.cachedConfig = false
m_apiConfiguration.cachedExpiredTimeSec = nil -- segundos
m_apiConfiguration.cachedTime = nil

m_apiConfiguration.GetApiConfiguration = function(self, vcc, keyspace)
	vcc = vcc:lower()
	if not self.cachedExpiredTimeSec then
		local config = GetApiSystemConfig()
		self.cachedExpiredTimeSec = config.Cache.vccApiConfiguration.expiredTimeSec
		--LogDebug("[API-api_users.GetApiConfiguration] 1 -----> self.cachedExpiredTimeSec: %s", self.cachedExpiredTimeSec)
	end
	if self.cachedConfig and self.cachedConfig[vcc] then
		local nowTime = os.time()
		local elapsedTimeSec = nowTime - self.cachedTime
		if elapsedTimeSec <= self.cachedExpiredTimeSec then
			--LogDebug("[API-api_users.GetApiConfiguration] 1.1 -----> Loading VCC '%s' API Configuration from CACHE. Expired in %s minutes.", vcc, (self.cachedExpiredTimeSec - elapsedTimeSec) / 60)
			return self.cachedConfig[vcc]
		else
			--LogDebug("[API-api_users.GetApiConfiguration] 1.2 ===> EXPIRED cache of VCC '%s'.", vcc)
		end
	end

	--LogDebug("[API-api_users.GetApiConfiguration] 2 -----> Loading VCC '%s' API Configuration from DATABASE. Time: %s, expired in %s minutes.", vcc, self.cachedTime, self.cachedExpiredTimeSec / 60)

	self.cachedTime = os.time()
	self.cachedConfig = self.cachedConfig or {}
	self.cachedConfig[vcc] = {}

	local closeKeyspace = false
	if not keyspace then
		local cassandra = m_datasource and m_datasource.cassandra or utils.GetDatasource()
		keyspace = cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local res, column = pcall( keyspace.GetColumn, keyspace, vcc, "VCCs", "ApiConfiguration" )

	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		self.cachedConfig[vcc] = column and column.value and json.decode(column.value) or nil
		return self.cachedConfig[vcc]
	end
end

function GetApiConfiguration (vcc, keyspace)
	return m_apiConfiguration:GetApiConfiguration(vcc, keyspace)
end
