module(..., package.seeall)

local m_apiUtils = require "api.utils"
local m_datasource

function GetName()
	return "VCCs"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function SanitizeVccInfo (vccInfo)
	assert(type(vccInfo) == "table")
	--LogDebug("[API.VCCS.SanitizeVccInfo] 1- BEFORE santized vccInfo: '%s'.", json.encode(vccInfo))
	vccInfo.Accounts = nil
	vccInfo.AppsIntegrations = nil
	vccInfo.Security = nil
	if not vccInfo.ApiConfiguration then
		return vccInfo
	end
	-- sanitize de Api configuaration
	if vccInfo.ApiConfiguration.urlsConfig and vccInfo.ApiConfiguration.urlsConfig.resources then
		local sanitizedApiConfiguration = {
			urlsConfig = {
				resources = {}
			}
		}
		for resourceType in pairs(vccInfo.ApiConfiguration.urlsConfig.resources) do
			local resourceInfo = {resourceType = resourceType}
			local config = {
				rateLimits = {},
				paged      = {}
			}
			-- rate limit
			local vccRateLimit, defaultRateLimits = m_apiUtils.GetApiURLRateLimit(resourceInfo, vccInfo.Id)
			for name, value in pairs(defaultRateLimits) do
				config.rateLimits[name] = vccRateLimit[name] or value
			end
			-- paged
			local vccPaged, defaultPaged = m_apiUtils.GetApiURLPaged(resourceInfo, vccInfo.Id)
			config.paged = vccPaged or defaultPaged
			-- santized resource
			sanitizedApiConfiguration.urlsConfig.resources[resourceType] = config
		end

		vccInfo.ApiConfiguration = sanitizedApiConfiguration
	end

	--LogDebug("[API.VCCS.SanitizeVccInfo] 2- AFTER santized vccInfo: '%s'.", json.encode(vccInfo))
	return vccInfo
end

function Get (vcc, keyspace, mustSantizeInfo)
	local vccInfo = m_apiUtils.GetAdmModel("VCCs").Get(vcc, keyspace)
	if not mustSantizeInfo then
		return vccInfo
	end
	return SanitizeVccInfo(vccInfo)
end

function LoadAllVCCs (keyspace)
	return m_apiUtils.GetAdmModel("VCCs").LoadAllVCCs(keyspace)
end

function LoadVccsWithEnabledApi (keyspace)
	local vccsWithEnabledApi = {}
	local vccs = LoadAllVCCs(keyspace)
	for _, vcc in pairs(vccs or {}) do
		if vcc.ApiConfiguration then
			if vcc.ApiConfiguration.Enabled then
				vcc.ApiConfiguration.vccId = vcc.Id
				table.insert(vccsWithEnabledApi, vcc.ApiConfiguration)
			else
				LogDebug("[API.VCCS.LoadVccsWithEnabledApi] vcc: '%s' has disabled API SDK.", vcc.Id)
			end
		else
			LogDebug("[API.VCCS.LoadVccsWithEnabledApi] vcc: '%s' without API SDK configuration.", vcc.Id)
		end
	end
	LogDebug("[API.VCCS.LoadVccsWithEnabledApi] ---> vccsWithEnabledApi: '%s'.", json.encode(vccsWithEnabledApi))
	return vccsWithEnabledApi
end
