local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local md5 = require "md5"

m_htmlRender = nil
m_appLocation = nil

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model:BuildModels()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

--[[
	Funcion que es invocada de forma externa desde otra App "provisioning.vccs"
]]--
function UpdateUsers (apiUsersList)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.API_USERS:UpdateUsers] user %s, vcc %s", userId, vcc)
	--LogDebug("[ADMINISTRATION.API_USERS:UpdateUsers] --> apiUsersList: '%s'", json.encode(apiUsersList or {}))

	--Get all Users from model
	local keyspace  = utils.GetDatasource():CassandraConnection("Agent")
	local apiUsers  = {}
	local idMapping = {}
	local userApiVCC

	for apiUserId, apiUser in pairs(apiUsersList or {}) do
		if not userApiVCC then
			userApiVCC = apiUser.Vcc or vcc
		end
		local existingApiUser = GetModel("ApiUsers").GetApiUsers(userApiVCC, apiUserId, keyspace)
		local exists = next(existingApiUser)

		if type(apiUser.Type) ~= "string" then
			keyspace:Close()
			LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Api user %s doesnt have a Type", apiUserId)
			return false, {result = "MUST_HAVE_A_TYPE", reason = "User must have a type"}
		end

		if type(apiUser.ExpirationTime) ~= "number" then
			keyspace:Close()
			LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Api user %s doesnt have a ExpirationTime", apiUserId)
			return false, {result = "MUST_HAVE_A_EXPIRATION_TIME", reason = "User must have a expiration time"}
		end

		--If this is a new entity, try to find an unused Id for it
		if apiUser.isNew then
			if exists then
				keyspace:Close()
				LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Api user %s already exists", apiUserId)
				return false, {result = "ID_ALREADY_EXISTS", reason = "Already exists another user with same id"}
			end
			if type(apiUser.Password) ~= "string" then
				keyspace:Close()
				LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Api user %s doesnt have a Password", apiUserId)
				return false, {result = "MUST_HAVE_A_PASSWORD", reason = "User must have a password"}
			end

			idMapping[apiUserId] = apiUserId
			apiUser.Password     = md5.sumhexa(apiUser.Password)
			apiUser.isNew        = nil
		else
			if not exists then
				keyspace:Close()
				LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Api user %s does not exist", apiUserId)
				return false, {result = "ID_DOES_NOT_EXIST", reason = "Id does not exist"}
			end
			if type(apiUser.Password) == "string" then
				apiUser.Password = md5.sumhexa(apiUser.Password)
			else
				apiUser.Password = existingApiUser.password
			end
		end

		apiUsers[apiUserId] = apiUser
	end

	local success, err = GetModel("ApiUsers").SaveApiUsers(userApiVCC, apiUsers, keyspace)
	keyspace:Close()

	if success then
		-- OK
		return true, {result = "OK", idMapping = idMapping}
	end

	-- Error
	LogError("[ADMINISTRATION.API_USERS:UpdateUsers] Saving api users error: %s.", json.encode(err))
	return false, err
end

local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form api_users for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_api_user_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML api_users does not exist: %s", html_file)
	end
	return content
end

--Returns all api_users in the VCC
local function Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.API_USERS:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
	--Get all api_users from model
	local rows = GetModel("ApiUsers").GetApiUsers(vcc)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[ADMINISTRATION.API_USERS:SEARCH] 2 - Filtering rows")

	for userId, user in pairs(rows) do
		if tostring(user.IsSystem) ~= "1" then
			user.Id = userId
			if utils.MatchFilter(user, postData) then
				totalAdded = totalAdded + 1
				user.Password = nil
				table.insert(filteredRows, user)
			end
		end
	end

	LogInfo("[ADMINISTRATION.API_USERS:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = 0
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.API_USERS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

local function Get (web, apiUserId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Retrieving api userId %s, user %s, vcc %s", apiUserId, userId, vcc)

	local apiUsers = GetModel("ApiUsers").GetApiUsers(vcc, apiUserId)
	LogInfo("Api userId %s: %s", apiUserId, json.encode(apiUsers))

	if next(apiUsers) then
		apiUsers.Password = nil
		return apiUsers
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

--[[
Remove apiUsers from a vcc
]]
local function RemoveApiUsers (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.RemoveApiUsers] user %s, vcc %s, users: '%s'", userId, vcc, web.input["post_data"])

	local apiUsers = web.request
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	local success, err = GetModel("ApiUsers").RemoveApiUsers(vcc, apiUsers, keyspace)
	if not success then
		keyspace:Close()
		web.status = 400
		LogError("[ADMINISTRATION.RemoveApiUsers] Removing users error: %s.", json.encode(err))
		return err
	end

	keyspace:Close()
	return {result = "OK"}
end

local function GetKey (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Retrieving api users key, vcc %s", vcc)

	local configuration = m_models.ApiUsers.GetApiConfiguration(vcc)
	LogInfo("Api configuration: %s", json.encode(configuration))

	if configuration and configuration.key then
		return { key = configuration.key }
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

local function SetKey (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Setting api users key, vcc %s", vcc)

	local key = web.request.key
	if not key or key == "" then
		web.status = 400
		return {result = "KEY_NOT_FOUND"}
	end

	local configuration = m_models.ApiUsers.GetApiConfiguration(vcc)
	if not configuration then
		web.status = 404
		return {result = "NOT_FOUND"}
	end
	configuration.key = key
	m_models.ApiUsers.SetApiConfiguration(vcc, configuration)
	LogInfo("Api configuration: %s", json.encode(configuration))
	return { result = "OK" }
end

local function GetIpFilters (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Retrieving api users ip filters, vcc %s", vcc)

	local configuration = m_models.ApiUsers.GetApiConfiguration(vcc)
	LogInfo("Api configuration: %s", json.encode(configuration))

	if configuration then
		return { ip_filters = configuration.ip_filters or {} }
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

local function SetIpFilters (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Setting api users ip filters, vcc %s", vcc)

	local ip_filters = web.request.ip_filters
	if not ip_filters then
		web.status = 400
		return {result = "FILTERS_NOT_FOUND"}
	end

	local configuration = m_models.ApiUsers.GetApiConfiguration(vcc)
	if not configuration then
		web.status = 404
		return {result = "NOT_FOUND"}
	end
	configuration.ip_filters = ip_filters
	m_models.ApiUsers.SetApiConfiguration(vcc, configuration)
	LogInfo("Api configuration: %s", json.encode(configuration))
	return { result = "OK" }
end

--[[
	Add/Modifies apiUsers in a vcc
]]
local function SaveApiUsers (web)
	local postData = web.request
	local success, result = UpdateUsers(postData)
	if success then
		-- ok
		web.status = 200
		return result
	end
	-- error
	local err = result
	web.status = 400
	return err
end

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_api_users", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_api_users", SaveApiUsers), prefix.."/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_api_users", RemoveApiUsers), prefix.."/delete/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "api_users_key", GetKey), prefix.."/get_key/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "api_users_key", SetKey), prefix.."/set_key/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "api_ip_filters", GetIpFilters), prefix.."/get_ip_filters/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "api_ip_filters", SetIpFilters), prefix.."/set_ip_filters/")
	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
