local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local modelPermissions = require "shared.permissions.models.permissions"
--local agentservice = require "apps.agent.agentservice"
local agentModule = require "apps.agent.agent"
local admGroups = require "apps.administration.controllers.groups"
local agentNotifications = require("apps.agent.agentNotifications")

local m_moduleLocation = select(2, ...)
m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_apps_launcher", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_apps_launcher", SaveAppLauncher), prefix.."/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_apps_launcher", RemoveAppLauncher), prefix.."/delete/")

	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render (web)
	_M.response.DontCache(web)

	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:Render] Render form AppsLauncher for user %s, group %s", userId, inconcert.authUser.userGroup)

	local htmlFile = m_appLocation .. "/pages/frm_apps_launcher_content.html"
	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[ADMINISTRATION.APPS_LAUNCHER:Render] HTML AppsLauncher does not exist: %s", htmlFile)
	end
	return content
end

--Returns all APPS_LAUNCHER in the VCC
function Search (web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
	--Get all apps_launcher from model
	local rows = modelPermissions.GetApplicationActions("apps_launcher")

	--LogDebug("[ADMINISTRATION.APPS_LAUNCHER:SEARCH] rows: '%s'", json.encode(rows))

	local totalUnfilteredRows = utils.GetRowCount(rows)
	local totalAdded = 0
	local filteredRows = {}

	--include only APPS_LAUNCHER that belong to logged user vcc
	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:SEARCH] 2 - Filtering rows")

	for skillId, apps_launcher in pairs(rows) do
		skillId = utils.GetId(skillId)
		apps_launcher.Id = skillId
		apps_launcher.Name = apps_launcher.Name or skillId
		if utils.MatchFilter(apps_launcher, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, apps_launcher)
		end
	end

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:SEARCH] 4 - (END) Search rows. retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

function Get (web, appLauncherId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:Get] Retrieving app launcher %s, user %s, vcc %s", appLauncherId, userId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local appId = "apps_launcher"
	local rows = modelPermissions.GetApplicationActions(appId, keyspace)
	local appLauncher = rows and rows[appLauncherId] or {}

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:Get] App Launcher '%s': '%s'", appLauncherId, json.encode(appLauncher))

	if next(appLauncher) then
		appLauncher.Name = appLauncher.Name or appLauncherId
		local roles = modelPermissions.GetRolesHaveTheActionAssigned(appId, appLauncherId, vcc, keyspace, "TRUE")
		for _, role in ipairs(roles or {}) do
			role.RoleId = utils.GetId(role.RoleId)
			role.Permissions = nil -- no es necesario retornar al cliente todos los permisos del rol.
		end
		appLauncher.RolesPermissions = roles
		keyspace:Close()
		return appLauncher
	end

	keyspace:Close()
	LogWarning("[ADMINISTRATION.APPS_LAUNCHER:Get] App Launcher '%s' does not exist.", appLauncherId)
	web.status = 404
	return {result = "NOT_FOUND"}
end

function SaveAppLauncher (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER.SaveAppLauncher] User %s Saving App Launcher: '%s'.", userId, web.input["post_data"])

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local _, postData = next(web.request)
	--LogDebug("------> postData: '%s'", json.encode(postData))

	local appId = "apps_launcher"
	local actionId = postData.Id:lower()

	if postData.isNew then
		local actions = modelPermissions.GetApplicationActions(appId, keyspace)
		if actions then
			if actions[actionId] then
				LogError("[ADMINISTRATION.APPS_LAUNCHER:SaveAppLauncher] Unable to add Action: '%s' to App: '%s'. This action already exist.", actionId, appId)
				web.status = 400
				return {result = "ACTION_ALREADY_EXIST", reason = "Error saving application actions. Action already exist.", actionId = actionId, appId = appId}
			end
		end
	end

	local actions = {}
	table.insert(actions, {
		Id          = actionId,
		Description = postData.Description,
		Data        = {
			webdesigner = {
				projectId = postData.ApplicationId,
				windowMode = postData.windowMode
			}
		}
	})
	local clonedActions = utils.DeepCopy(actions)
	-- 1 - Almaceno las custom actions (permisos) para la app
	local isOk = modelPermissions.SaveVccApplicationActions(vcc,
		{
			ApplicationId = appId,
			Actions       = actions
		},
		keyspace
	)

	if not isOk then
		LogError("[ADMINISTRATION.APPS_LAUNCHER:SaveAppLauncher] Unable to add Action: '%s' to App: '%s'. Server error.", actionId, appId)
		web.status = 400
		return {result = "ERROR_SAVING_ACTION", reason = "Error saving application actions"}
	end

	--[[---------------
		Actualizo los permisos del Rol
	]]--
	local function UpdateAppRolePermissions (currentVcc, appId, vccRoleId, role, rolePermissions, isEnabled)
		if not role then
			return false
		end
		--LogDebug("  --> currentVcc: %s, vccRoleId: '%s', IsDeployer '%s', IsFullSystemAccess '%s', rolePermissions: '%s'", currentVcc, vccRoleId, role.IsDeployer, role.IsFullSystemAccess, json.encode(rolePermissions))
		if tostring(role.IsDeployer) == "1" then
			return false
		end
		if not rolePermissions then
			return false
		end
		role.Permissions = nil -- esto no se debe de almacenar en la CF Group
		local roleToSave = {
			[role.Id] = role
		}
		for _, action in ipairs(clonedActions) do
			action.Enabled = isEnabled or false
		end
		modelPermissions.SaveGroup(currentVcc, roleToSave, keyspace)
		modelPermissions.SaveGroupPermissions(vccRoleId, rolePermissions, keyspace)
		modelPermissions.UpdateAppRolePermissions(appId, vccRoleId, clonedActions, keyspace, rolePermissions, currentVcc)
	end
	---------------------
	-- 2 - Ahora tengo que asignar permisos a estas acciones.
	local cleanUpCache = true
	local systemRoles = modelPermissions.GetGroups("system", keyspace, cleanUpCache)
	--LogDebug("0.1 - systemRoles: '%s'", json.encode(systemRoles or {}))
	local vccRoles = modelPermissions.GetGroups(vcc, keyspace, cleanUpCache)
	--LogDebug("0.2 - vccRoles: '%s'", json.encode(vccRoles or {}))
	-- aca mantenemos los roles que hemos procesado y validado, ya que en si en el POST llegan roles repetidos, la info queda incorrecta.
	local passedRoles = {}
	-- los roles posteados
	local postedRolesPermissions = postData.RolesPermissions or {}
	-- Agrego el rol del sistema "full-system-access"
	table.insert(postedRolesPermissions, {RoleId = "full-system-access", Enabled = true})
	-- comienzo a iterar sobre los roles e ir agregando los permisos a los mismos.
	for _, roleInfo in ipairs(postedRolesPermissions) do
		local vccRoleId = utils.AppendVcc(roleInfo.RoleId, vcc)
		if not passedRoles[vccRoleId:lower()] then
			passedRoles[vccRoleId:lower()] = true
			local role = modelPermissions.GetGroup(vccRoleId, keyspace, vccRoles)
			--LogDebug("1 - vccRoleId: '%s', role: '%s'", vccRoleId, role)
			local rolePermissions
			if not role then
				local systemRoleId = utils.AppendVcc(roleInfo.RoleId, "system")
				role = modelPermissions.GetGroup(systemRoleId, keyspace, systemRoles)
				--LogDebug(" 2 - systemRoleId: '%s', role: '%s'", systemRoleId, role)
				rolePermissions = modelPermissions.GetGroupPermissions(systemRoleId, keyspace)
			else
				--(" 3 - vccRoleId: '%s', role: '%s'", vccRoleId, role)
				rolePermissions = modelPermissions.GetGroupPermissions(vccRoleId, keyspace)
			end
			-- actualizo los permisos
			UpdateAppRolePermissions(vcc, appId, vccRoleId, role, rolePermissions, roleInfo.Enabled)
		end
	end

	-- notifico al agente
	local applicationsActions = agentModule.GetTranslatedApplicationsActions(keyspace, vcc)
	agentNotifications.NotifyAgents({
		eventType = "administrator_applications_actions_change",
		agentsLevel = "VCC_USERS",
		pollerMessage = applicationsActions
	}, keyspace)

	keyspace:Close()
	local result = {result = "OK", idMapping = {}}
	return result
end

--[[
]]
function RemoveAppLauncher (web)
	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER.RemoveAppLauncher] -> user '%s', vcc '%s', Removing APPS_LAUNCHER: '%s'.", inconcert.authUser.userName, vcc, web.input["post_data"])

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local _, appLauncherId = next(web.request)

	--LogDebug("appLauncherId: '%s'", appLauncherId)

	local actions = {}
	table.insert(actions, {Id = appLauncherId, Enabled = "REMOVE"})
	local clonedActions = utils.DeepCopy(actions)
	-- 1 - Elimino las custom actions (permisos) de la app
	local appId = "apps_launcher"
	local isOk = modelPermissions.DeleteVccApplicationActions(vcc,
		{
			ApplicationId = appId,
			Actions       = actions
		},
		keyspace
	)

	if not isOk then
		web.status = 400
		return {result = "ERROR_DELETING_ACTION", reason = "Error deleting application actions"}
	end

	--[[
	]]--
	local function RemovePermissions (currentVcc)
		local roles = modelPermissions.GetRolesHaveTheActionAssigned(appId, appLauncherId, currentVcc, keyspace, "TRUE_AND_FALSE")
		LogInfo("[ADMINISTRATION.APPS_LAUNCHER.RemoveAppLauncher] -> Roles Have The '%s' Action Assigned: '%s'. VCC: '%s'.", appLauncherId, json.encode(roles or {}), currentVcc)
		for _, role in ipairs(roles or {}) do
			local fullRoleId = role.RoleId
			--LogDebug("  1--- - app: '%s' fullRoleId: '%s', perm: '%s'.", appId, fullRoleId, json.encode(role.Permissions[appId]))
			if role.Permissions and role.Permissions[appId] and type(role.Permissions[appId][appLauncherId]) == "boolean" then
				--LogDebug("SI 11111")
				modelPermissions.UpdateAppRolePermissions(appId, fullRoleId, actions, keyspace, role.Permissions, currentVcc)
			--else
				--LogDebug("NO 2222")
			end
		end
	end
	--[[
		Remuevo del los roles del VCC.
		Remuevo de los roles del Sistema (especificamente para el full-system-access)
	]] 
	RemovePermissions(vcc)
	-- -- Remuevo de los roles del Sistema (especificamente para el full-system-access)
	-- RemovePermissions("system")

	-- notifico al agente
	local applicationsActions = agentModule.GetTranslatedApplicationsActions(keyspace, vcc)
	agentNotifications.NotifyAgents({
		eventType = "administrator_applications_actions_change",
		agentsLevel = "VCC_USERS",
		pollerMessage = applicationsActions
	}, keyspace)

	keyspace:Close()
	local result = {result = "OK", idMapping = {}}
	return result
end

function InUseLauncherApp (tweetyId, keyspace)
	assert(tweetyId)
	assert(keyspace)

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.APPS_LAUNCHER:InUseLauncherApp] Validating tweety '%s' project in use on Launcher App. User '%s', group '%s'", tweetyId, userId, vcc)

	local response = {InUse = false, tweetyId = tweetyId, appsLauncherInUse = {}}

	local appId = "apps_launcher"
	local appLauncherActions = modelPermissions.GetApplicationActions(appId, keyspace) or {}

	if next(appLauncherActions) then
		for actionId, action in pairs(appLauncherActions) do
			if action.Data and action.Data.webdesigner and (action.Data.webdesigner.projectId or ""):lower() == tweetyId:lower() then
				table.insert(response.appsLauncherInUse, actionId)
			end
		end
	end

	response.InUse = #response.appsLauncherInUse > 0
	if response.InUse then
		LogWarning("[ADMINISTRATION.APPS_LAUNCHER:InUseLauncherApp] Tweety project '%s' in use on '%d' Apps Launcher.\n%s", tweetyId, #response.appsLauncherInUse, json.encode(response.appsLauncherInUse))
	else
		LogInfo("[ADMINISTRATION.APPS_LAUNCHER:InUseLauncherApp] Tweety project '%s' is not in use on Launcher Apps.", tweetyId)
	end

	return response
end
