local utils = require "shared.apps.utils"
local audits = require "audits"
local cjson = require "cjson"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil
m_appLocation = nil

local function ExecuteSearch (postData)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)
	local startCurrentProcessTime = os.time()

	LogInfo("[ADMINISTRATION.AUDIT:ExecuteSearch] 1 - Search AUDITs MainProcess for user %s. Posted data: '%s'", fullUserId, json.encode(postData))

	local postedFilters = postData.query and postData.query.OR
	if postedFilters and type(postedFilters) ~= "table" then
		LogError("Error - Invalid request Load Audit Data (Filter Type).")
		web.status = 400
		return false, { result = "INVALID_REQUEST", reason = "Missing field filters is not a table."}
	end

	local filters = {}
	--LogDebug("----> postedFilters %s", postedFilters)
	-- me quedo solo con el primer valor del array
	for _, filter in ipairs(postedFilters) do
		-- LogDebug("%s", _)
		-- LogDebug("--- %s", json.encode(filter))
		local name, value = next(filter)
		filters[name] = filters[name] or value
	end

	local now = LuaOrm.Date()
	local function MakeFromDate (increase)
		local from = now:add(unpack(increase))
		local stringFrom = from:toString("%Y-%m-%d")
		return stringFrom
	end
	local stringNowDate = now:toString("%Y-%m-%d")
	local period = (filters.period or ""):upper()
	if period == "NOW" then
		filters.initialDate = stringNowDate
		filters.endDate = stringNowDate
	elseif period == "YESTERDAY" then
		filters.initialDate = MakeFromDate({0, 0, -1})
		filters.endDate = filters.initialDate
	elseif period == "LAST WEEK" then
		filters.initialDate = MakeFromDate({0, 0, -7})
		filters.endDate = stringNowDate
	elseif period == "MONTH" then
		filters.initialDate =  MakeFromDate({0, -1, 0})
		filters.endDate = stringNowDate
	else
		if not filters.initialDate or filters.initialDate == "" then
			filters.initialDate = stringNowDate
		end
		if not filters.endDate or filters.endDate == "" then
			filters.endDate = stringNowDate
		end
		if filters.operation and filters.operation == "" then
			filters.operation = nil
		end
	end
	filters.period = nil

	--LogDebug("[ADMINISTRATION.AUDIT:ExecuteSearch] 2 - filters: '%s'", json.encode(filters))

	if not filters or not filters.initialDate or not filters.endDate then
		LogError("Error - Invalid request Load Audit Data (Missing field).")
		return false, { result = "INVALID_REQUEST", reason = "Missing field filters"}
	end

	-- filters.initialDate = "2022-10-24"
	-- filters.endDate = "2022-10-24"

	LogInfo("[ADMINISTRATION-AUDIT:ExecuteSearch]---> Load audit data by filters, user '%s', vcc '%s' -> '%s'", fullUserId, vcc, json.encode(filters))

	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	filters.vcc = vcc
	filters.connection = ksHistorical

	if postData.numberPage and postData.limit then
		filters.paging = {
			numberPage = postData.numberPage,
			rowsPerPage = postData.limit
		}
	end
	filters.timeZoneInfo = inconcert.authUser.timezone
	local startSubProcessTime = os.time()
	local success, auditData, totalRecords = audits.Filter(filters)
	LogInfo("[ADMINISTRATION-AUDIT:ExecuteSearch]    END Subprocess totalRecords: %s, (rock -> audits.Filter(..)) ELAPSED Time '%s' seconds.", totalRecords, utils.GetElapsedTimeSec(startSubProcessTime))

	ksHistorical:Close()

	if not success then
		local err = auditData
		return false, err
	end 
	--LogDebug("[ADMINISTRATION-AUDIT:ExecuteSearch] 1---> auditData -> '%s'", json.encode(auditData))

	local records = auditData
	LogInfo("[ADMINISTRATION-AUDIT:ExecuteSearch] End Search MainProcess audit ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
	return true, records, totalRecords
end

local function ExportToCsv (wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)
	_T = m_htmlRender.extension.makeTextTranslator()
	local request = wsapi.request.new(wsapi_env)
	local query = request.params["query"]

	LogInfo("[ADMINISTRATION.AUDIT:ExportToCsv] 1 - Exporting AUDITs for user %s. Posted data: '%s'", fullUserId, query)
	local startSubProcessTime = os.time()
	local success, records = ExecuteSearch({
		query = json.decode(query)
	})
	LogInfo("[ADMINISTRATION.AUDIT:ExportToCsv] 2 - Exporting AUDITs. END Executed Search ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

	if not success then
		local err = records
		wsapi_env.status = 400
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write("ERROR_NOT_FOUND")
		response.headers["Response"] = json.encode(err)
		response.status = 400
		return response:finish()
	end

	--LogDebug("[ADMINISTRATION.AUDIT:ExportToCsv] 2 - Exporting AUDITs for user %s. records: '%s'", fullUserId, json.encode(records))
	LogInfo("[ADMINISTRATION.AUDIT:ExportToCsv] 3 - Exporting AUDITs for user %s. records: '%s'", fullUserId, #records)

	local auditInfoTranslate = {}
	if not utils.TableIsEmpty(records) then
		startSubProcessTime = os.time()
		for _, value in ipairs(records) do
			local auditInfo = {
				[_T("Date")] = value.displayTime,
				[_T("User")] = value.user,
				[_T("Entity")] = value.entity,
				[_T("Key")] = value.keyField,
				[_T("Sub Entity")] = value.subEntity or "",
				[_T("Operation_without_accent")] = _T(value.operation),
				[_T("Previous_Version_without_accent")] = value.beforeFields and string.gsub(json.encode(value.beforeFields), '"', '""' ) or "",
				[_T("Current_Version_without_accent")] = value.currentFields and string.gsub(json.encode(value.currentFields), '"', '""' ) or ""
			}
			table.insert(auditInfoTranslate, auditInfo)
		end
		--LogDebug("[ADMINISTRATION.AUDIT:ExportToCsv] 4 - Exporting AUDITs. END Prepared translated table to convert to CSV. ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	else
		wsapi_env.status = 400
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write("ERROR_NOT_FOUND")
		response.headers["Response"] = "ERROR_NOT_FOUND"
		response.status = 400
		return response:finish()
	end

	local orderHeaders = {
		_T("Date"),
		_T("User"),
		_T("Entity"),
		_T("Key"),
		_T("Sub Entity"),
		_T("Operation_without_accent"),
		_T("Previous_Version_without_accent"),
		_T("Current_Version_without_accent")
	}

	startSubProcessTime = os.time()
	local csv = utils.Table2Csv(auditInfoTranslate, nil, nil, "|", orderHeaders)
	if not csv then
		wsapi_env.status = 400
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write("ERROR_CSV_NOT_FOUND")
		response.headers["Response"] = "ERROR_CSV_NOT_FOUND"
		response.status = 400
		return response:finish()
	end

	LogInfo("[ADMINISTRATION.AUDIT:ExportToCsv] 5 - Exporting AUDITs. END Created. ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	csv = string.gsub('"'..csv..'"', "|", '","')
	local LN = "\n"
	csv = string.gsub(csv, LN, '"'..LN..'"')
	local updateDate = utils.ConvertUTCDateToDate( LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S"), nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil ) 
	local suggestedFileName = string.format("%s(%s).CSV", "audits_export_info_".. vcc, updateDate)
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = #csv
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/"

	LogInfo("[ADMINISTRATION.AUDIT:ExportToCsv] Exporting audits. CSV File Name '%s'.", suggestedFileName)

	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		--LogDebug("[ADMINISTRATION.AUDIT:ExportToCsv] 6.%d - Exporting AUDITs. coroutine.wrap.", coroutineIter)
		coroutine.yield(csv)
	end)
end

-- load audit data by filters
local function Filter (web)
	local startCurrentProcessTime = os.time()
	LogInfo("[ADMINISTRATION-AUDIT:Filter] + START Filter audit url: /../filter/ ...")
	local postData = web.request
	local success, records, totalRecords = ExecuteSearch(postData)
	if not success then
		local err = records
		web.status = 400
		return err
	end
	--local startSubProcessTime = os.time()
	--LogDebug("[ADMINISTRATION-AUDIT:Filter]      START Subprocess (utils.MakeSearchResponse).")
	postData.limit = nil
	postData.numberPage = nil
	local response = {
		rows = records,
		last = "",
		total_rows = totalRecords,
		total_unfiltered_rows = totalRecords
	}
	local resultData = utils.MakeSearchResponse(response, postData, {
		noOrder = true
	})
	--LogDebug("[ADMINISTRATION-AUDIT:Filter]      END Subprocess (utils.MakeSearchResponse) ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	LogInfo("[ADMINISTRATION-AUDIT:Filter] ---> END Search audit url: /../filter/ ... . ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
	--LogDebug("[ADMINISTRATION-AUDIT:Filter] ---> resultData '%s'", json.encode(resultData))
	return resultData
end

local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("[AUDIT:Render] Render form audit for user %s, group %s", userId, inconcert.authUser.userGroup)
	local html_file = m_appLocation .. "/pages/frm_audit_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[AUDIT:Render] HTML Audit does not exist: %s", html_file)
	end
	return content
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	-- frm audit render
	module:dispatch_get (Render, prefix .. "/frm/render/")
	-- audit
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. Filter, prefix.."/filter/" )
	--export
	module:dispatch_wsapi(ExportToCsv, prefix .. "/export/(.*)")
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
