local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local notificationsEngine = require("apps.agent.notifications")
local admCampaignController = require("apps.administration.controllers.campaigns")
local agentCampaignModel = require "apps.agent.models.campaigns"
local audits = require "audits"
local m_moduleLocation = select(2, ...)
m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	-- frm capability render
	module:dispatch_get ( Render, prefix .. "/frm/render/")

	-- capability
	module:dispatch_get ( utils.JsonReply          .. LogRequest .. GetAttentionCapability        , prefix .. "/get/" )
	module:dispatch_get ( utils.JsonReply          .. LogRequest .. GetCampaignAttentionCapability, prefix .. "/get_campaign/(.+)" )
	module:dispatch_post( utils.JsonValidatorReply .. LogRequest .. permissions:CheckPermission("administration", "save_attention_capability", SaveAttentionCapability ), prefix.."/save/" )
	module:dispatch_post( utils.JsonValidatorReply .. LogRequest .. permissions:CheckPermission("administration", "save_attention_capability", SaveCampaignAttentionCapability ), prefix.."/save_campaign/" )

	-- telephony
	module:dispatch_get ( utils.JsonReply          .. LogRequest .. LoadPackages         , prefix .. "/load_packages/([^.]*)"           )
	module:dispatch_get ( utils.JsonReply          .. LogRequest .. LoadAccountDids    	 , prefix .. "/load_account_dids/([^.]*)"           	)

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function make_capability_id( channel_id, full_camp_id, full_user_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local campaign_without_vcc = utils.GetId( full_camp_id )

	-- 1 - concatena unicamente el channel + vcc (Ej: FACEBOOK@inconcert)
	local id = channel_id .. "@" .. vcc
	-- 2 - al id le concatena la campana, quedando channel + vcc + campaign (Ej: FACEBOOK@inconcert@campaign1)
	id = id .. "@" .. campaign_without_vcc
	if full_user_id then
	-- 3 - si tenemos el id del usuario, al id le concatena la user id, quedando channel + vcc + campaign + user (Ej: FACEBOOK@inconcert@campaign1@agent1)
		local user_without_vcc = utils.GetId( full_user_id )
		id = id .. "@" .. user_without_vcc
	end
	return id
end

-- el channel de CALL no es configurable, siempre tiene el valor en 1, que es el valor por defecto de una instalacion de cero.
function get_availables_channels (ksAgent)
	local channels = {
		CALL       = { is_configurable = false },
		VOICEMAIL  = { is_configurable = false },
		CHAT       = { is_configurable = true  },
		FACEBOOK   = { is_configurable = true  },
		INSTAGRAM  = { is_configurable = true  },
		YOUTUBE    = { is_configurable = true  },
		APPSTORE   = { is_configurable = true  },
		GOOGLEPLAY = { is_configurable = true  },
		RCS	       = { is_configurable = true  },
		IM         = { is_configurable = true  },
		MAIL       = { is_configurable = true  },
		SMS        = { is_configurable = true  },
		WHATSAPP   = { is_configurable = true  },
		TWITTER    = { is_configurable = true  },
		WEBCONTACT = { is_configurable = true  }
	}
	local vcc = inconcert.authUser.virtualContactCenter
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, ksAgent)
	if not hasAllegroIntegration then
		channels.VOICEMAIL.is_configurable = true
	end
	return channels
end

function GetInteractionTypes( keyspace )
	return m_models.Channels.GetInteractionTypes( keyspace )
end

function Render(web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:Render] Render form Channels for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_channels_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[CHANNELS:Render] HTML Channels does not exist: %s", html_file)
	end
	return content
end

function GetAttentionCapability(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo("[CHANNELS:GetAttentionCapability] Loading VCC attention capability (interaction types), user '%s', vcc '%s'", userId, vcc)

	local interactionTypes = GetInteractionTypes( keyspace )
	if utils.TableIsEmpty(interactionTypes) then
		web.status = 404
		return { result = "NOT_FOUND" }
	end
	local values = {}
	local channels = get_availables_channels()
	for channel in pairs( channels ) do
		local value = interactionTypes[ channel .. "@" .. vcc ] or interactionTypes[ channel .. "@system" ]
		values[ channel ] = value
	end

	LogInfo("[CHANNELS:GetAttentionCapability] Retrieving VCC attention capability (Interaction Types) %s", json.encode( values ))

	return values
end

function GetCapabilityByCampaignAndMember( full_camp_id, full_user_id, all_interaction_types, keyspace )
	--LogDebug("[CHANNELS:GetCapabilityByCampaignAndMember] --> Loading (interaction types) FROM %s.", all_interaction_types and "CACHE" or "DATABASE")
	all_interaction_types = all_interaction_types or GetInteractionTypes( keyspace ) or {}
	local channels = get_availables_channels()
	local capabilities
	for channel_id in pairs( channels ) do
		local id = make_capability_id( channel_id, full_camp_id, full_user_id )
		local value = all_interaction_types[ id ]
		if not utils.TableIsEmpty(value) then
			capabilities = capabilities or {}
			capabilities[ channel_id ] = value
		end
	end
	if capabilities then
		LogInfo("[CHANNELS:GetCapabilityByCampaignAndMember] Loading Member attention capability (interaction types), Campaign : '%s', Member: '%s'.", full_camp_id, full_user_id)
	end
	return capabilities, all_interaction_types
end

function GetCampaignAttentionCapability( web, full_camp_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo("[CHANNELS:GetCampaignAttentionCapability] Retrieving '%s' Campaign attention capability (interaction types), user '%s', vcc '%s'", full_camp_id, userId, vcc)

	local interaction_types = GetInteractionTypes()
	if utils.TableIsEmpty(interaction_types) then
		web.status = 404
		return { result = "NOT_FOUND" }
	end

	local campaign_capabilities = GetCapabilityByCampaignAndMember( full_camp_id, nil, interaction_types ) or {}

	--LogDebug("[CHANNELS:GetCampaignAttentionCapability] Retrieving Campaign attention capability (Interaction Types) '%s'", json.encode( campaign_capabilities or "" ))

	return campaign_capabilities
end

-- guarda los attention capability a nivel VCC
function SaveAttentionCapability (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:SaveAttentionCapability] Saving VCC Attention Capability (Interaction Types), user '%s', vcc '%s'-> '%s'", userId, vcc, web.input["post_data"])

	local posted_data = web.request or {}
	local postedChannels = posted_data.channels or {}

	if type(postedChannels) ~= "table" then
		LogError("Error - Invalid request Saving VCC Attention Capability (Interaction Types).")
		web.status = 400
		return { result = "INVALID_REQUEST", reason = "Missing field channels is not a table."}
	end

	local channels = get_availables_channels()
	for channel, values in pairs( postedChannels ) do
		if not channels[ channel ] then
			local err = LogError("Error - Invalid request Saving VCC Attention Capability (Interaction Types). %s channel not supported", channel)
			web.status = 400
			return { result = "INVALID_REQUEST", reason = err }
		end
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	for channel, values in pairs( postedChannels ) do
		if channels[ channel ].is_configurable then
			local channelWithVCC = utils.AppendVcc( channel, vcc )
			-- guarda los attention capability a nivel VCC
			m_models.Channels.SaveInteractionTypes( channelWithVCC, values, nil, nil, keyspace )
		end
	end
	keyspace:Close()
	LogInfo("[CHANNELS:SaveAttentionCapability] Saved VCC Attention Capability (Interaction Types), user '%s', vcc '%s'", userId, vcc)

	--notifications
	notificationsEngine.SendNotification("VccCapabilitiesUpdated", 0, { capabilities = channels })

	return { result = "OK" }
end

-- guarda los attention capability a nivel Campaign & Member
function UpdateCapabilityCampaignChannelAndMember( full_camp_id, full_user_id, new_channels_values, current_interaction_types, keyspace )

	local vcc = inconcert.authUser.virtualContactCenter
	if full_camp_id then
		full_camp_id = utils.AppendVcc( utils.GetId( full_camp_id ), vcc )
	end
	if full_user_id then
		full_user_id = utils.AppendVcc( utils.GetId( full_user_id ), vcc )
	end

	-- obtengo los current interation types para poder realizar el merge, segun lo que tenemos en la BD y lo que ha llega posteado
	current_interaction_types = current_interaction_types or GetInteractionTypes()

	local hasChanges = false
	local channels = get_availables_channels()
	-- primero remuevo los channels a los cuales ya no se le realiza override
	for channel_id, channel in pairs( channels ) do
		if channel.is_configurable and ( not new_channels_values or not new_channels_values[ channel_id ] ) then
			-- remuevo
			local id = make_capability_id( channel_id, full_camp_id, full_user_id )
			local current_values = current_interaction_types[id]
			if not utils.TableIsEmpty(current_values) then
				m_models.Channels.RemoveInteractionTypes( id, full_camp_id, full_user_id, nil, keyspace )
				hasChanges = true
			end
		end
	end

	if new_channels_values then
		-- aca reailzo el merge
		local function merge( values, current_values )
			local new_values = current_values
			for name, value in pairs(values) do
				new_values[ name ] = value
			end
			return new_values
		end

		-- recorro la lista de channels que han llegado posteado, realizo el merge y mando guardar!!
		for channel_id, values in pairs( new_channels_values ) do
			if channels[ channel_id ].is_configurable then
				local id = make_capability_id( channel_id, full_camp_id, full_user_id )
				local new_values
				-- Siempre realizo el merge contra lo que dice system. Esto porque los valores que no son de attentionCapability no varian y system siempre existe.
				local id_channel_system = channel_id .. "@system"
				local current_values = current_interaction_types[ id_channel_system ]
				if not utils.TableIsEmpty(current_values) then
					new_values = merge( values, current_values )
				end
				new_values = new_values or values

				m_models.Channels.SaveInteractionTypes( id, new_values, full_camp_id, full_user_id, keyspace )
				hasChanges = true
			end
		end
	end

	if hasChanges then
		--notifications
		if full_user_id then
			notificationsEngine.SendNotification("CampaignUserCapabilitiesUpdated", 0, { campaignId = utils.GetId(full_camp_id), userId = utils.GetId(full_user_id), capabilities = new_channels_values or {} })
		else
			notificationsEngine.SendNotification("CampaignCapabilitiesUpdated", 0, { campaignId = utils.GetId(full_camp_id), capabilities = new_channels_values or {} })
		end
	end

	return current_interaction_types
end

-- Cuando la campana es removida tiene que limpiar los overrides de la campana/channels y campaign/members/channels
function OnRemovedCampaign( full_camp_id, members_of_campaign, keyspace )

	local keys_to_remove = {}

	-- recorro la lista de capability x campana, y luego los agrego en la lista de keys a eliminar
	local interaction_types_by_campaign = m_models.Channels.GetInteractionTypesByCampaign( full_camp_id, keyspace ) or {}
	for capability_id in pairs( interaction_types_by_campaign ) do
		table.insert( keys_to_remove, { capability_id = capability_id } )
	end

	-- recorro la lista members de la campana y obtengo los capability de este y los filtro x la campana, y luego los agrego en la lista de keys a eliminar
	for user_id in pairs( members_of_campaign or {} ) do
		local interaction_types_by_user = m_models.Channels.GetInteractionTypesByUser( user_id, keyspace )
		for capability_id, info in pairs( interaction_types_by_campaign ) do
			if info.campaign == full_camp_id then
				table.insert( keys_to_remove, { capability_id = capability_id, user_id = user_id } )
			end
		end
	end

	-- mando a remover los overrides de la campana y de los members
	for _, info in ipairs( keys_to_remove ) do
		m_models.Channels.RemoveInteractionTypes( info.capability_id, full_camp_id, info.user_id, nil, keyspace )
	end
end

-- Cuando la cuenta es removida y era la ultima que existia para este channel en la campana, tiene que limpiar los overrides de la campana/channels
function OnRemovedAccount( full_camp_id, account_channel, keyspace )

	account_channel = (account_channel or ""):upper()
	local can_delete_capability = true
	local accounts = m_models.Campaigns.GetCampaignAccounts( full_camp_id, keyspace )

	-- aca valida no exista ninguna account para este channel en la campana
	for _, account in pairs( accounts ) do
		if account.proxyType and account.proxyType:upper() == account_channel then
			can_delete_capability = false
			break
		end
	end

	if not can_delete_capability then
		return
	end

	local keys_to_remove = {}
	-- recorro la lista de capability x campana, busco el channel especifico, y luego los agrego en la lista de keys a eliminar
	local interaction_types_by_campaign = m_models.Channels.GetInteractionTypesByCampaign( full_camp_id, keyspace ) or {}
	for capability_id in pairs( interaction_types_by_campaign ) do

		local capability_channel, capability_user
		local position = 1
		local str = capability_id

		-- en este loop obtengo el CHANNEL que se encuentra en al posicion 1 del string, y el user id que se encuentra en la posicion 4 del string.
		-- Ej: TWITTER@inconcert@camp5@user186
		while str and position <=3 do
			local value, more = string.match(str, "([^@]+)@(.+)")
			str = more
			if position == 1 then
				capability_channel = value
			elseif position == 3 and more then
				capability_user = utils.AppendVcc( more, inconcert.authUser.virtualContactCenter )
			end
			position = position + 1
		end

		capability_channel = (capability_channel or ""):upper()
		if capability_channel == account_channel then
		-- aca inserto en la lista de capabilities a eliminar.
			table.insert( keys_to_remove, { capability_id = capability_id, user_id = capability_user } )
		end
	end

	-- mando a remover los overrides de la campana/channel
	for _, info in ipairs( keys_to_remove ) do
		m_models.Channels.RemoveInteractionTypes( info.capability_id, full_camp_id, info.user_id, nil, keyspace )
	end
end

function SaveCampaignAttentionCapability( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local posted_data = web.request or {}
	local campaign_id = posted_data.campaign_id

	LogInfo( "Saving Campaign Attention Capability (Interaction Types), user '%s', vcc '%s' campaign: '%s' -> '%s'", userId, vcc, campaign_id, web.input["post_data"] )

	local new_channels_values = posted_data.channels or {}

	if type(new_channels_values) ~= "table" then
		LogError("Error - Invalid request Saving Campaign Attention Capability (Interaction Types).")
		web.status = 400
		return { result = "INVALID_REQUEST", reason = "Missing field channels is not a table."}
	end

	local channels = get_availables_channels()
	for channel, values in pairs( new_channels_values ) do
		if not channels[ channel ] then
			local err = LogError("Error - Invalid request Saving Campaign Attention Capability (Interaction Types). '%s' channel not supported", channel)
			web.status = 400
			return { result = "INVALID_REQUEST", reason = err }
		end
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local oldAuditCapabilities = admCampaignController.GetAuditCampaign( utils.AppendVcc(utils.GetId(campaign_id), vcc), keyspace, "Channels")
	UpdateCapabilityCampaignChannelAndMember( campaign_id, nil, new_channels_values, nil, keyspace )
	local updatedAuditCapabilities = admCampaignController.GetAuditCampaign( utils.AppendVcc(utils.GetId(campaign_id), vcc), keyspace, "Channels")

	if oldAuditCapabilities or updatedAuditCapabilities then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaign_id),
			oldFields = oldAuditCapabilities,
			newFields =  updatedAuditCapabilities,
			subEntity = "Channels",
			connection = ksHistorical
		})
		ksHistorical:Close()
	else
		LogWarning("[ADMINISTRATION.Channels] Audit record not created")
	end

	keyspace:Close()
	LogInfo( "Saved Campaign Attention Capability (Interaction Types), user '%s', vcc '%s', campaign '%s'", userId, vcc, campaign_id )

	return { result = "OK" }
end

function LoadPackages (web, interactionType)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("[CHANNELS:LoadPackages] Retrieving packages %s on vcc %s for interaction type %s", name, vcc, interactionType )
	local packages = m_models.Packages.GetPackages(vcc, interactionType)
	LogInfo("[CHANNELS:LoadPackages] Packages for interaction type '%s': %s", interactionType, json.encode(packages))

	return packages
end

function LoadAccountDids (web, interactionType)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("[CHANNELS:LoadAccountDids] Retrieving dids %s on vcc %s for interaction type %s", name, vcc, interactionType )
	local packages = m_models.Packages.GetPackages(vcc, interactionType)
	local dids = agentCampaignModel.GetVccDids(vcc, packages)
	LogInfo("[CHANNELS:LoadAccountDids] dids for interaction type '%s': %s", interactionType, json.encode(dids))

	return dids
end
