local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"

m_htmlRender = nil
m_appLocation = nil

--[[

]]--
local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.PHONE_VALIDATOR:Render] Render form Phone Validator " ..
			"with sections for user %s, group %s",
			userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_phone_validator_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[ADMINISTRATION.PHONE_VALIDATOR:Render] HTML PHONE_VALIDATOR does not exist: %s", html_file)
	end
	return content
end

--[[

]]--
local function Get (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo("[CHANNELS:Get] Loading phone validator schemas, user '%s', vcc '%s'", userId, vcc)
	local schemas = m_models.PhoneValidator.GetMergedSchemas(vcc)
	return schemas
end

--[[

]]--
function SaveArea (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:SaveArea] country %s area, user '%s', vcc '%s'-> '%s'",
			countryId, userId, vcc, web.input["post_data"])
	m_models.PhoneValidator.SaveVccArea(vcc, countryId, web.request)

	return { result = "OK" }
end

--[[

]]--
function DeleteArea (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:DeleteArea] country %s area, user '%s', vcc '%s'-> '%s'",
			countryId, userId, vcc, web.input["post_data"])
	m_models.PhoneValidator.DeleteVccArea(vcc, countryId, web.request)

	return { result = "OK" }
end

--[[

]]--
function DeleteAreas (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:DeleteAreas] country %s areas, user '%s', vcc '%s'",
			countryId, userId, vcc)
	m_models.PhoneValidator.DeleteVccAreas(vcc, countryId)

	return { result = "OK" }
end

--[[

]]--
function SaveRule (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:SaveRule] country %s area, user '%s', vcc '%s'-> '%s'",
			countryId, userId, vcc, web.input["post_data"])
	m_models.PhoneValidator.SaveVccRule(vcc, countryId, web.request)

	return { result = "OK" }
end

--[[

]]--
function DeleteRule (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:DeleteRule] country %s area, user '%s', vcc '%s'-> '%s'",
			countryId, userId, vcc, web.input["post_data"])
	m_models.PhoneValidator.DeleteVccRule(vcc, countryId, web.request)

	return { result = "OK" }
end

--[[

]]--
function DeleteGlobalRules (web, countryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[CHANNELS:DeleteGlobalRules] , user '%s', vcc '%s'",
			countryId, userId, vcc)
	m_models.PhoneValidator.RemoveGlobalDialing(vcc, countryId)

	return { result = "OK" }
end

--[[
****************************************************************
	MAPEO DE URLS
****************************************************************
]]--

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local permissionView  = "view_phone_validator"
	local permissionSave  = "save_phone_validator"
	local permissionErase = "delete_phone_validator"


	local function HasPermission (fn, actionId, responseFn)
		actionId = actionId or permissionSave
		return LogRequest .. permissions:CheckPermission("administration", actionId, fn)
	end

	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")
	-- peticiones
	module:dispatch_get ( utils.JsonReply          .. LogRequest ..
		permissions:CheckPermission("administration", permissionView, Get  ) , prefix .. "/get/"  )
	module:dispatch_post( utils.JsonValidatorReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionSave, SaveArea ), prefix .. "/save_area/([^/]*)" )
	module:dispatch_delete( utils.JsonValidatorReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionErase, DeleteArea ), prefix .. "/delete_area/([^/]*)" )
	module:dispatch_delete( utils.JsonReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionErase, DeleteAreas ), prefix .. "/delete_areas/([^/]*)" )
	module:dispatch_post( utils.JsonValidatorReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionSave, SaveRule ), prefix .. "/save_rule/([^/]*)" )
	module:dispatch_delete( utils.JsonValidatorReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionErase, DeleteRule ), prefix .. "/delete_rule/([^/]*)" )
	module:dispatch_delete( utils.JsonReply .. LogRequest ..
		permissions:CheckPermission("administration", permissionErase, DeleteGlobalRules ), prefix .. "/delete_global_rules/([^/]*)" )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
