local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroRoutingGroups = require "apps.allegro.controllers.administration_routing_groups"

m_htmlRender = nil
m_appLocation = nil

local function __Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Render] Render form RoutingGroups for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_routing_group_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[ADMINISTRATION_ROUTING_GROUPS:__Render] HTML RoutingGroups does not exist: %s", html_file)
	end

	return content
end

--Returns all Routing Groups in the VCC
local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Search] Loading Routing Groups in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroRoutingGroups.LoadPaginatedRoutingGroups(encodedInfo)

	if success and data then
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[ADMINISTRATION_ROUTING_GROUPS:__Search] Error loading Routing Groups. Err: %s", json.encode(err))

	return err
end

--[[

Return

]]--
local function __Get (web, routingGroupId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Get] Retrieving Routing Group %s, user %s, vcc %s", routingGroupId, userId, vcc)

	local success, routingGroup = allegroRoutingGroups.GetRoutingGroup(routingGroupId)
	if success and routingGroup then
		return json.encode(routingGroup)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __Save (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Save] Saving Routing Groups in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroRoutingGroups.SaveRoutingGroup(encodedInfo)

	if success then
		LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Save] Success Saved Routing Groups %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[ADMINISTRATION_ROUTING_GROUPS:__Save] Error Saving Routing Groups %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __Delete (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Delete] Deleting Routing Groups in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroRoutingGroups.DeleteRoutingGroup(encodedInfo)

	if success then
		LogInfo("[ADMINISTRATION_ROUTING_GROUPS:__Delete] Success Deleted Routing Groups %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[ADMINISTRATION_ROUTING_GROUPS:__Delete] Error deleting Routing Groups %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

function MapUris (module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..__Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewroutinggroups", __Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "saveroutinggroups", __Save), prefix.."/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deleteroutinggroups", __Delete), prefix.."/delete/")

	module:dispatch_get(__Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
