local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local m_moduleLocation = select(2, ...)
local agentUserModel = require "apps.agent.models.users"
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")

m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest.. Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewuserstates", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "saveuserstate", SaveStates), prefix.."/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deleteuserstate", RemoveStates), prefix.."/delete/")
	module:dispatch_get(utils.JsonReply..LogRequest.. RemoveValidation, prefix.."/delete_validation/(.+)")

	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render(web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form States for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/" .. ( utils.HasAllegroIntegration(vcc) and "frm_state_allegro_content.html" or "frm_state_content.html" )

	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML States does not exist: %s", html_file)
	end
	return content
end

function Get(web, stateId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Retrieving stateId %s, user %s, vcc %s", stateId, userId, vcc)

	local idAtVcc = utils.AppendVcc(stateId, vcc)
	local states = m_models.States.GetStates(idAtVcc)
	LogInfo("State %s: %s", stateId, json.encode(states))
	local state = states and states[idAtVcc]

	if state then
		state.Id = stateId
		state.Name = state.Name or stateId
		return states[idAtVcc]
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

function GetAllStates()
	local states = m_models.States and m_models.States.GetStates() or {}
	LogInfo("GetAllStates %s", json.encode(states))
	return states
end

--Returns all states editables in the VCC
function Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.STATES:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request

	--Get all states from model who belongs to the vcc
	local rows = utils.FilterEntities(m_models.States.GetStates(), {vcc = vcc})
	local totalUnfilteredRows = utils.GetRowCount(rows)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[ADMINISTRATION.STATES:SEARCH] 2 - Filtering rows")
	--LogDebug("[ADMINISTRATION.STATES:SEARCH] ===> 2.1 - rows: %s", json.encode(rows or {}))

	for stateId, state in pairs(rows) do
		stateId = utils.GetId(stateId)
		state.Id = stateId
		state.Name = state.Name or stateId
		if utils.MatchFilter(state, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, state)
		end
	end

	LogInfo("[ADMINISTRATION.STATES:SEARCH] 3 - Sorting rows %d", #filteredRows)
	--LogDebug("[ADMINISTRATION.STATES:SEARCH] ===> 3.1 - filteredRows: %s", json.encode(filteredRows or {}))

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.STATES:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

--[[
	Add/Modifies states in a vcc
	posted data must have the form:	
{
	stateId1 = {
		Description = "state description 1",
		isBreak = "1", //must be string "1" or "0"
	},
	stateId2 = {
		Description = "state description 2",
		isBreak = "0", //must be string "1" or "0"
	},
	...
}
]]
function SaveStates(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.SaveStates] user %s, vcc %s", userId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	--Get all states from model who belongs to the vcc
	local existingStates = utils.FilterEntities(m_models.States.GetStates( nil, keyspace ), {vcc = vcc})
	--Insert Id in each state
	utils.Map(existingStates, function(id, state) state.Id = utils.GetId(id) end)

	local hasAllegro = utils.HasAllegroIntegration(vcc, keyspace)
	local idMapping = {}
	local pollerStates = {}
	local states = {}
	local maxlengthId = 12 -- solo para los VCCs con integración
	local maxlengthDesc = 20
	local newStates = {}

	for stateId, state in pairs(web.request) do
		if type(state.Description) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.SaveStates] State %s doesnt have a description", stateId)
			return {result = "MUST_HAVE_A_DESCRIPTION", reason = "State must have a description"}
		end
		if #state.Description > maxlengthDesc then
			state.Description = string.sub(state.Description, 1, maxlengthDesc)
		end
		if type(state.Name) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.SaveStates] State %s doesnt have a Name", stateId)
			return {result = "MUST_HAVE_A_NAME", reason = "State must have a name"}
		end

		if hasAllegro then
			if #stateId > maxlengthId then
				keyspace:Close()
				web.status = 400
				local reason = string.format("Overflow in  the '%s' State  Id length. Supported Max length is %d.", stateId, maxlengthId)
				LogError("[ADMINISTRATION.SaveStates] %s", reason)
				return {result = "ALLEGRO_INTEGRATION_OVERFLOW_NAME", reason = reason}
			end
			local timeoutLimit = tonumber(state.Timeout)
			if timeoutLimit < 0 or timeoutLimit > 1400 then
				keyspace:Close()
				web.status = 400
				local reason = LogError("[ADMINISTRATION.SaveStates] '%s' state Time Limit out of range '%s'. Please enter values between 1 to 1400 minutes.", stateId, timeoutLimit)
				return {result = "ALLEGRO_INTEGRATION_ERROR_STATE_LIMIT_OUT_OF_RANGE", reason = reason}
			end
		end

		local action
		--If this is a new entity, try to find an unused Id for it
		if state.isNew then
			action = "NEW"
			local n = 0
			repeat
				n = n + 1
				local repeated = utils.FilterEntities(existingStates, function(id) return utils.GetId(id) == utils.Id_n(stateId, n) end)
			until not next(repeated)
			idMapping[stateId] = utils.Id_n(stateId, n)
			stateId = idMapping[stateId]
			newStates[stateId] = true
			state.isNew = nil
		else
			action = "UPDATE"
			--modification of an existing entity, must ensure that the entity already exists
			local exists = false
			utils.FilterEntities(existingStates, function(id) exists = exists or utils.GetId(id) == stateId end)
			if not exists then
				keyspace:Close()
				web.status = 400
				LogError("[ADMINISTRATION.SaveStates] State %s does not exist", stateId)
				return {result = "ID_DOES_NOT_EXIST", reason = "Id does not exist"}
			end
		end

		--Check repeated states (those with same Id or Name)
		local repeated = utils.FilterEntities(existingStates, {Name = state.Name})
		for _, state in pairs(repeated) do
			if state.Id ~= stateId then
				keyspace:Close()
				web.status = 400
				LogError("[ADMINISTRATION.SaveStates] State %s has repeated Name or Id with %s", stateId,  state.Id)
				return {result = "MUST_HAVE_UNIQUE_NAME", reason = "State must have an unique name"}
			end
		end

		--default values
		state.Id = stateId
		state.isBreak =  "1"
		state.overrideable = "1"
		state.isInitial = "0"
		state.isFinal = "0"
		state.UserState = "1"
		state.vcc = vcc

		local stateIdWithVCC = utils.AppendVcc(stateId, vcc)
		existingStates[stateIdWithVCC] = state
		states[stateIdWithVCC] = state

		pollerStates[stateIdWithVCC] = {
			action = action,
			state  = state
		}
	end

	local success, err = m_models.States.SaveStates(states, keyspace)
	if not success then
		web.status = 400
		LogError("[ADMINISTRATION.SaveStates] Saving states error: %s.", json.encode(err))
		keyspace:Close()
		return err
	end

	--Notify logged users that States has changed
	agentNotifications.NotifyAgents({
		eventType = "administrator_state_change",
		agentsLevel = "VCC_USERS",
		pollerMessage = pollerStates
	}, keyspace)
	
	keyspace:Close()

	--notifications
	for _, state in pairs(states) do
		notificationsEngine.SendNotification(newStates[state.Id] and "StateCreated" or "StateUpdated", 0, 
			{ state = state })
	end

	return {result = "OK", idMapping = idMapping}
end

--[[
Remove states from a vcc
postedData must be of the form: ["stateId1", "stateId2", ...]
]]
function RemoveStates(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("[ADMINISTRATION:STATES] RemoveStates, user %s, vcc %s", userId, vcc)
	--LogDebug("[ADMINISTRATION:STATES] States %s", web.input["post_data"])

	local pollerStates = {}
	local toRemove = {}
	for i, stateId in ipairs(web.request) do
		local stateIdWithVCC = utils.AppendVcc(stateId, vcc)
		toRemove[i] = stateIdWithVCC
		pollerStates[stateIdWithVCC] = {
			action = "DELETE"
		}
	end
	--LogDebug("[ADMINISTRATION:STATES] toRemove %s", json.encode(toRemove))

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local success, response = m_models.States.RemoveStates(toRemove, keyspace)

	if not success then
		keyspace:Close()
		web.status = 400
		return response
	end

	-- Notify logged users that States has changed
	agentNotifications.NotifyAgents({
		eventType = "administrator_state_change",
		agentsLevel = "VCC_USERS",
		pollerMessage = pollerStates
	}, keyspace)
	keyspace:Close()

	--notifications
	for _, stateId in pairs(web.request) do
		notificationsEngine.SendNotification("StateDeleted", 0, { stateId = stateId })
	end

	return {result = "OK"}
end

function RemoveValidation(web, stateId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Validate removing custom state %s. User %s, vcc %s", stateId, userId, vcc)

	local idAtVcc = utils.AppendVcc(stateId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local states = m_models.States.GetStates(idAtVcc, keyspace)
	LogInfo("State %s: %s", stateId, json.encode(states))
	local state = states and states[idAtVcc]

	if state then
		local result = {result = "OK"}
		local users = agentUserModel.FindUsersByState(idAtVcc, keyspace)
		keyspace:Close()
		LogInfo("State: %s, have users: %s", idAtVcc, json.encode(users))
		if users and next(users) ~= nil then
			result.warning = {
				info = "STATE_BEING_USED_ON_INITIAL_STATE",
				reason = "The state '" .. idAtVcc .. "' to delete is currently in use on initial state."
			}
		end
		return result
	else
		keyspace:Close()
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end
