local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", function()
	return require ("ssl.https")
end)
local ltn12 = require "ltn12"

local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"

m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..List, prefix.."/list/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_ttsasr_engine", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_ttsasr_engine", SaveTtsAsrEngine), prefix.."/save/")

	module:dispatch_get(Render, prefix .. "/frm/render/")
	
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render(web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form ttsasr_engines for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_ttsasr_engines_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML ttsasr_engines does not exist: %s", html_file)
	end
	return content
end

--Returns all ttsasr_engines in the VCC
function Search(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.TTSASR_ENGINES:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
	--Get all ttsasr_engines from model
	local rows = m_models.TtsAsrEngines.GetTtsAsrEngines(vcc)

	local response = {
		rows = rows,
		last = "",
		total_rows = utils.GetRowCount( rows )
	}
	postData.__order_attr_name = "name"
	local resultData = utils.MakeSearchResponse(response, postData)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.TTSASR_ENGINES:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, elapsedTime)

	return resultData
end

--Returns all ttsasr_engines in the VCC
function List(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	--Get all ttsasr_engines from model
	local rows = m_models.TtsAsrEngines.GetTtsAsrEngines(vcc)
	
	return { ttsasr_engines = rows }
end

function Get(web, Id)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Retrieving TTS/ASR Id %s, user %s, vcc %s", Id, userId, vcc)
	local result = nil
	
	local rows = m_models.TtsAsrEngines.GetTtsAsrEngines(vcc)
	if rows then
		for _, engine in pairs(rows) do
			local composedId = engine.engineId .. "_" .. engine.profileId
			if composedId == Id then
				result = engine
			end
		end
		if result then
			return result
		else
			LogError("TTS/ASR Id %s not found", Id)
			web.status = 404
			return {result = "NOT_FOUND"}
		end
	else
		LogError("TTS/ASR Id %s not found", Id)
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

--[[
	Add/Modifies appNotifications in a vcc
]]
function SaveTtsAsrEngine(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.SaveTtsAsrEngine], user %s, vcc %s", userId, vcc)
	local postData = web.request

	--Get all apps from model
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local engineToUpdate = {}
	for _, engine in pairs(postData) do
		engineToUpdate = engine
	end
	
	--Clean isNew field
	engineToUpdate.isNew = nil
	
	local currentEngines = m_models.TtsAsrEngines.GetTtsAsrEngines(vcc, keyspace)
	for _, engine in pairs(currentEngines) do
		if engine.engineId == engineToUpdate.engineId and engine.profileId == engineToUpdate.profileId then
			currentEngines[_] = engineToUpdate
		end
	end

	local result
	local success, err = m_models.TtsAsrEngines.SaveTtsAsrEngine(vcc, currentEngines, keyspace)
	if success then
		result = {result = "OK"}
	else
		web.status = 400
		LogError("[ADMINISTRATION.SaveTtsAsrEngine] Saving TTS/ASR error: %s.", json.encode(err))
		result = err
	end

	keyspace:Close()
	return result
end
