local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local modelRestrictions = require "apps.agent.models.config_restrictions"
local m_validations = require("apps.administration.controllers.validations.workgroup").GetDefinitions()
m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewworkgroup", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "saveworkgroup",
			utils.WebJsonValidatorData(
				m_validations.basic,
				SaveWorkgroups
			)
		),
		prefix.."/save/"
	)
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deleteworkgroup", RemoveWorkgroups), prefix.."/delete/")

	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render(web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form Workgroups for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_workgroup_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML Workgroups does not exist: %s", html_file)
	end
	return content
end

--Returns all workgroups in the VCC
function Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.WORKGROUPS:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
	--Get all workgroups from model
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local rows = m_models.Workgroups.GetWorkgroups(vcc, nil, keyspace)
	local totalUnfilteredRows = 0
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[ADMINISTRATION.WORKGROUPS:SEARCH] 2 - Filtering rows")

	local tableWorkGroups = {}
	for workgroupId, workgroup in pairs(rows) do
		workgroupId = utils.GetId(workgroupId)
		totalUnfilteredRows = totalUnfilteredRows + 1
		workgroup.Id = workgroupId
		workgroup.Name = workgroup.Name or workgroupId
		if utils.MatchFilter(workgroup, postData) then
			totalAdded = totalAdded + 1
			tableWorkGroups[workgroupId] = true
			table.insert(filteredRows, workgroup)
		end
	end

	LogInfo("[ADMINISTRATION.WORKGROUPS:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}

	local hashColumns = utils.ArrayToHash(postData.columns or {}) or {}
	local supervisorsWorkGroups = {}
	local vccUsers
	if hashColumns.Supervisors then
		--[[ Si se quieren los Supervisores, se obtienen todos los users
		{
			campaign1: {wkg1: true, wkg2: true, wkg3:true},
			campaign2: {wkg4: true},
			..
		}]]--
		vccUsers = m_models.Users.GetVCCUsers(vcc, keyspace) or {}
		for userId, userInfo in pairs(vccUsers) do
			for campaignId, campaignWorkgroups in pairs(userInfo.SupervisorWorkgroups or {}) do -- campaigns
				for workgroup in pairs(campaignWorkgroups) do
					if tableWorkGroups[workgroup] then
						supervisorsWorkGroups[workgroup] = supervisorsWorkGroups[workgroup] or {}
						local userSupWkg = supervisorsWorkGroups[workgroup][userId] or {}

						userSupWkg.Id = userId
						userSupWkg.Name = userInfo.Name or userId
						userSupWkg.WorkgroupCampaigns = userSupWkg.WorkgroupCampaigns or {}
						userSupWkg.WorkgroupCampaigns[campaignId] = true

						supervisorsWorkGroups[workgroup][userId] = userSupWkg
					end
				end
			end
		end
		--LogInfo("[DMINISTRATION.WORKGROUPS:SEARCH] - supervisorsWorkGroups %s", json.encode(supervisorsWorkGroups))
	end
	keyspace:Close()

	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end,
		TransformRowFn = function (row)
			--LogInfo("[DMINISTRATION.WORKGROUPS:SEARCH] 3.1 - TransformRowFn %s", json.encode(row))
			row.Agents = row.Agents or utils.EMPTY_OBJECT

			if vccUsers then
				for userId in pairs(row.Agents) do
					local userInfo = vccUsers[userId] or {}
					row.Agents[userId] = {
						Id = userId,
						Name = userInfo.Name or userId,
						FirstSurname = userInfo.FirstSurname or userId
					}
				end
			end

			row.Supervisors = supervisorsWorkGroups[row.Id] or utils.EMPTY_OBJECT
			if vccUsers then
				for userId in pairs(row.Supervisors) do
					local userInfo = vccUsers[userId] or {}
					row.Supervisors[userId] = {
						Id = userId,
						Name = userInfo.Name or userId,
						FirstSurname = userInfo.FirstSurname or userId
					}
				end
			end
			return row
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)
	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.WORKGROUPS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

function Get(web, workgroupId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Retrieving workgroupId %s, user %s, vcc %s", workgroupId, userId, vcc)

	local workgroups = m_models.Workgroups.GetWorkgroups(vcc, workgroupId)
	LogInfo("Workgroup %s: %s", workgroupId, json.encode(workgroups))

	if next(workgroups) then
		workgroups.Name = workgroups.Name or workgroupId
		return workgroups
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

--[[
	Add/Modifies workgroups in a vcc
	posted data must have the form:	
{
	workgroupId1 = {
		Description = "workgroup description 1",
		Agents = "['niko', 'albert', 'tito']",
	},
	workgroupId2 = {
		Description = "workgroup description 2",
		Agents = "[]",, 
	},
	...
}

]]
function SaveWorkgroups(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.SaveWorkgroups], user %s, vcc %s", userId, vcc)

	local postData = web.request
	--Get all Workgroup from model
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local existingWorkgroups = m_models.Workgroups.GetWorkgroups(vcc, nil, keyspace)
	local idMapping = {}

	local workgroups = {}
	local cont_add_workgroups = 0
	for workgroupId, workgroup in pairs(postData) do
		if type(workgroup.Description) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("Workgroup %s doesnt have a description", workgroupId)
			return {result = "MUST_HAVE_A_DESCRIPTION", reason = "Workgroup must have a description"}
		end
		if type(workgroup.Name) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("Workgroup %s doesnt have a Name", workgroupId)
			return {result = "MUST_HAVE_A_NAME", reason = "Workgroup must have a name"}
		end

		--If this is a new entity, try to find an unused Id for it
		if workgroup.isNew then
			local n = 0
			repeat
				n = n + 1
				local repeated = utils.FilterEntities(existingWorkgroups, function(id) return id == utils.Id_n(workgroupId, n) end)
			until not next(repeated)
			idMapping[workgroupId] = utils.Id_n(workgroupId, n)
			workgroupId = idMapping[workgroupId]
			workgroup.isNew = nil
			cont_add_workgroups = cont_add_workgroups + 1
		else
			--modification of an existing entity, must ensure that the entity already exists
			local exists = false
			utils.FilterEntities(existingWorkgroups, function(id) exists = exists or id == workgroupId end)
			if not exists then
				keyspace:Close()
				web.status = 400
				LogError("Workgroup %s does not exist", workgroupId)
				return {result = "ID_DOES_NOT_EXIST", reason = "Id does not exist"}
			end
		end

		--Check repeated workgroups Names
		local repeated = utils.FilterEntities(existingWorkgroups, {Name = workgroup.Name})
		for id in pairs(repeated) do
			if id ~= workgroupId then
				keyspace:Close()
				web.status = 400
				LogError("Workgroup %s has repeated Name or Id with %s", workgroupId, id)
				return {result = "MUST_HAVE_UNIQUE_NAME", reason = "Workgroup must have an unique name"}
			end
		end

		--allright
		workgroup.Agents = workgroup.Agents or {}

		--put in the memory existing workgroups
		existingWorkgroups[workgroupId] = workgroup

		workgroups[workgroupId] = workgroup
	end

	local result

	if cont_add_workgroups > 0 then
		--Valido que la cantidad a agregar no supere el limite permitido
		local success, error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxWorkgroups", keyspace, cont_add_workgroups)
		if not success then
			web.status = 400
			keyspace:Close()
			return { result = error or "ERROR_WORKGROUPS_EXCEEDS_LIMIT_ALLOWED" }
		end
	end
		
	local success, err = m_models.Workgroups.SaveWorkgroups(vcc, workgroups, keyspace)
	if success then
		if cont_add_workgroups > 0 then
			-- Actualizo los totales en redis de los workgroups
			modelRestrictions.UpdateRedisConfigTotals(vcc, {
				MaxWorkgroups = cont_add_workgroups
			}, keyspace)
		end
		result = {result = "OK", idMapping = idMapping}
	else
		web.status = 400
		LogError("[ADMINISTRATION.SaveWorkgroups] Saving workgroups error: %s.", json.encode(err))
		result = err
	end

	keyspace:Close()
	return result
end

function RemoveWorkgroups (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.RemoveWorkgroups], user %s, vcc %s, workgroups: '%s'", userId, vcc, web.input["post_data"])

	local workgroups = web.request
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local has_allegro = utils.HasAllegroIntegration( vcc, keyspace )

	-- obtengo los agentes que pertenecen a los workgroups, para luego ser actualizada su referencia (*)
	local usersWithWorkgroups = {}
	local cont_delete_workgroups = 0
	local cont_delete_workgroups_users = 0
	for _, workgroupId in ipairs(workgroups) do
		cont_delete_workgroups = cont_delete_workgroups + 1
		local workgroup = m_models.Workgroups.GetWorkgroups(vcc, workgroupId, keyspace)
		for agentId in pairs(workgroup.Agents) do
			cont_delete_workgroups_users = cont_delete_workgroups_users + 1
			if has_allegro then
			-- si tiene configuracion con allegro, no se permite eliminiar los wkg si se encuentran asociados a algun agente.
				keyspace:Close()
				web.status = 400
				local err = { result = "WORKGROUP_BEING_USED_ON_USERS", user_id = agentId }
				LogError("[ADMINISTRATION.RemoveWorkgroups] Removing workgroups error: %s.", json.encode(err))
				return err
			end
			usersWithWorkgroups[agentId] = workgroupId
		end
	end

	--LogDebug("[ADMINISTRATION.RemoveWorkgroups] --> usersWithWorkgroups: '%s'", userId, vcc, json.encode(usersWithWorkgroups))

	local success, err = m_models.Workgroups.RemoveWorkgroups( vcc, workgroups, keyspace )
	if not success then
		keyspace:Close()
		web.status = 400
		LogError("[ADMINISTRATION.RemoveWorkgroups] Removing workgroups error: %s.", json.encode(err))
		return err
	end

	-- Actualizo los totales en redis de workgroups
	modelRestrictions.UpdateRedisConfigTotals(vcc, {
		MaxWorkgroups = -cont_delete_workgroups,
		MaxWorkgroupsUsers = -cont_delete_workgroups_users
	}, keyspace)

	-- (*) Remuevo de los agentes la columna correspondiente al workgroup asignado.
	for agentId, workgroupId in pairs(usersWithWorkgroups) do
		LogInfo("[ADMINISTRATION.RemoveWorkgroups] Removing workgroup %s for agent %s", workgroupId, agentId)
		utils.RemoveColumns {
			keyspace = keyspace,
			columnFamily = "Users",
			rows = {
				[utils.AppendVcc(agentId, vcc)] = {"WorkgroupId"}
			}
		}
	end

	keyspace:Close()
	return {result = "OK"}
end
