module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "ApiUsers"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

function GetApiUsers(vcc, userId, keyspace)
	local users = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		key = vcc,
		columns = userId and {userId} or {}
	}
	if userId then
		return users and users[userId] or {}
	else
		return users or {}
	end
end

function SaveApiUsers(vcc, users, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		rows = {
			[vcc] = users
		}
	}
	return true
end

function RemoveApiUsers(vcc, users, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ApiUsers",
		superColumns = true,
		rows = {
			[vcc] = users
		}
	}
	return true
end

function GetApiConfiguration(vcc, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local res, column = pcall( keyspace.GetColumn, keyspace, vcc, "VCCs", "ApiConfiguration" )
	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		return column and column.value and json.decode(column.value) or nil
	end
end

function SetApiConfiguration(vcc, configuration, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	keyspace:InsertColumn(vcc, "VCCs", "ApiConfiguration", json.encode(configuration))
	
	if closeKeyspace then
		keyspace:Close()
	end
end
