module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "AppsNotifications"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

local function AddSystemAppNotificationData(appId, appData, integrations)
	appData.System = true
	local url = integrations and integrations[appId] and integrations[appId].Enabled == "1" and 
		integrations[appId].Notifications and integrations[appId].Notifications.Url or ""
	if url ~= "" then
		appData.Url = url
		return true
	else
		appData.Url = nil
		return false
	end
end

function GetAppsNotifications(vcc, appId, keyspace, integrations)
	local apps = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AppsNotifications",
		superColumns = true,
		key = vcc,
		columns = appId and {appId} or {}
	}
	apps = apps or {}
	if appId then
		local appData = apps[appId] or nil
		if not appData and vcc ~= "system" then
			appData = GetAppsNotifications("system", appId, keyspace, integrations)
		end
		if appData and vcc == "system" then
			if not AddSystemAppNotificationData(appId, appData, integrations) then
				return nil
			end
		end
		return appData
	else
		if vcc == "system" then
			local systemApps = apps
			apps = {}
			for id, app in pairs(systemApps) do
				if AddSystemAppNotificationData(id, app, integrations) then
					apps[id] = app
				end
			end
		else
			local systemApps = GetAppsNotifications ("system", appId, keyspace, integrations)
			for id, app in pairs(systemApps) do
				apps[id] = app
			end
		end
		return apps
	end
end

function SaveAppsNotifications(vcc, apps, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "AppsNotifications",
		superColumns = true,
		rows = {
			[vcc] = apps
		}
	}
	return true
end

function RemoveAppsNotifications(vcc, apps, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AppsNotifications",
		superColumns = true,
		rows = {
			[vcc] = apps
		}
	}
	return true
end
