local Repository = require "icc_repository"
module(..., package.seeall)
require "LuaOrm.Date"
local utils = require "shared.apps.utils"
local agent_model = require("apps.agent.model"):GetModels()
local SeparateVCC = require "apps.agent.helpers".SeparateVCC
local allegro_campaigns = require "apps.allegro.controllers.campaigns"
local allegro_repository = require "apps.allegro.controllers.repository"
local m_datasource

-- TODO: Esto esta mal, no puedo acceder a esto que es del agent
local agentservice = require "apps.agent.agentservice"

function GetName ()
	return "Campaigns"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function CampaignExists ( campaign_at_vcc, keyspace )
	if not campaign_at_vcc then
		return
	end
	local vcc = inconcert.authUser.virtualContactCenter
	local campaign, vccAtCampaign = utils.SeparateVCC(campaign_at_vcc)
	-- Si viene sin campaign uso la del usuario logueado
	if not vccAtCampaign then
		campaign_at_vcc = utils.AppendVcc(utils.GetId(campaign_at_vcc), vcc)
	end
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaign_at_vcc,
		column = "Id"
	})
end

function GetSpecificColumn ( campaign_at_vcc, specific_column, keyspace )
	if not specific_column then
		return
	end
	local vcc = inconcert.authUser.virtualContactCenter
	campaign_at_vcc = utils.AppendVcc(utils.GetId(campaign_at_vcc), vcc)
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaign_at_vcc,
		column = specific_column
	})
end

function GetVCCCampaigns (vcc, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCCampaigns",
		key = vcc
	} or {}
end

function GetAnotherUserAssignedCampaign (vcc, userId, keyspace, defaultCampaign)
	local assignedCampaigns  = GetUserCampaigns (vcc, userId, keyspace)
	local notAllowedCampaign = utils.AppendVcc("internal_calls", vcc)

	if assignedCampaigns then
		for campaign in pairs(assignedCampaigns) do
			if campaign ~= notAllowedCampaign and campaign ~= defaultCampaign then
				return campaign
			end
		end
	end

	return nil
end

function GetCampaign (vcc, Id, keyspace, columns)
	return utils.GetAllColumns {
		keyspace		= keyspace or "Agent",
		columnFamily	= "Campaigns",
		key				= utils.AppendVcc(Id, vcc),
		columns			= columns
	} or {}
end

function SaveCampaign (vcc, campaign, keyspace, allegroInfo)
	local campaignIdWithVcc = campaign.Id
	if allegroInfo then
		local success, err = UpdateAllegroCampaignIn55(campaignIdWithVcc, keyspace, allegroInfo)
		if not success then
			-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
			--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
			--  la actualizacion en i6.
			if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
				return false, { result = err.type, err = (err.err or err.sub_type) }
			end
		end
	end
	UpdateCampaign(campaignIdWithVcc, campaign, keyspace)
	return true
end

local function IncludeAllegroOtherInfo (allegro_info, allegroOtherInfo)
	if not allegro_info then
		return
	end
	allegro_info.otherInfo = allegroOtherInfo
end

function RemoveIntegration (campaignIdWithVcc, columns, keyspace, allegroOtherInfo)
	local allegroInfo = { action_id = "REMOVE_INTEGRATION" }
	IncludeAllegroOtherInfo(allegroInfo, allegroOtherInfo)

	local success, err = UpdateAllegroCampaignIn55(campaignIdWithVcc, keyspace, allegroInfo)
	if not success then
		-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
		--  la actualizacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			return false, { result = err.type, err = (err.err or err.sub_type) }
		end
	end
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		rows = {
			[campaignIdWithVcc] = columns
		}
	}
	return true
end

function RemoveInteractionNotifications (campaignIdWithVcc, columns, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		rows = {
			[campaignIdWithVcc] = columns
		}
	}
	return true
end

function GetAllVCCsAccounts ( keyspace )
	local params = {
		keyspace = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		startKey = "",
		endKey = ""
	}
	return utils.GetAllColumns( params ) or {}
end

function GetCampaignAccounts (campaign_id_with_vcc, keyspace, columns)
	assert(campaign_id_with_vcc, "campaign_id_with_vcc id is required")

	local simpleCampaignId, vccCampaign = utils.SeparateVCC(campaign_id_with_vcc)
	if not vccCampaign then
		local vcc = inconcert.authUser.virtualContactCenter
		campaign_id_with_vcc = utils.AppendVcc(campaign_id_with_vcc, vcc)
	end

	local params = {
		keyspace = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		key = campaign_id_with_vcc,
		columns = columns
	}

	return utils.GetAllColumns( params ) or {}
end

function GetCampaignAccount (campaignIdWithVcc, accountId, keyspace, columns)
	assert(campaignIdWithVcc, "campaignIdWithVcc id is required")
	assert(accountId, "campaignIdWithVcc id is required")

	local vcc = inconcert.authUser.virtualContactCenter
	campaignIdWithVcc = utils.AppendVcc(utils.GetId(campaignIdWithVcc), vcc)

	local accounts = GetCampaignAccounts(campaignIdWithVcc, keyspace, columns)
	return accounts[accountId]
end

function GetVccAccounts (vcc, keyspace)

	assert(vcc, "vcc is required")
	vcc = vcc:lower()
	local all_accounts = GetAllVCCsAccounts( keyspace )
	local vcc_accounts = {}

	for campaign_id, campaign_accounts in pairs( all_accounts ) do
		local _, campaign_vcc = utils.SeparateVCC(campaign_id)
		if campaign_vcc and (campaign_vcc:lower() == vcc) then
			vcc_accounts[campaign_id] = campaign_accounts
		end
	end

	return vcc_accounts
end

function SaveAccount (campaignWithVCC, account, is_new_account, keyspace)
	return agent_model.Campaigns.SaveCampaignAccount( campaignWithVCC, account, is_new_account, keyspace)
end

function UpdateAccountStateInRedis (activeKey, stateAccount, accountId, campaign_id)

	if stateAccount == "1" then
		m_datasource.redis:hset(activeKey, accountId, campaign_id)
	elseif stateAccount == "0" then
		m_datasource.redis:hdel(activeKey, accountId)
	end

end

function GetUserCampaigns (vcc, userId, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaigns",
		key = utils.AppendVcc(userId, vcc)
	} or {}
end

function GetCampaignMembers (vcc, campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignMembers",
		key = utils.AppendVcc(campaign_id, vcc)
	} or {}
end

function DeleteCampaignMembers (vcc, campaign_id, keyspace)
	LogInfo("[administration.campaigns.DeleteCampaignMembers] Removing Members of '%s' campaign.", campaign_id)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignMembers",
		rows = {utils.AppendVcc(campaign_id, vcc)}
	}
end

-- canned messages
function SaveCannedMessages (campaign_id, messages, keyspace)
	return utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignCannedMessages",
		rows = {
			[campaign_id] = messages
		}
	}
end

function DeleteCannedMessages (campaign_id, columns, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignCannedMessages",
		rows = {
			[campaign_id] = columns
		}
	}
end

function GetCannedMessages (campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignCannedMessages",
		key = campaign_id
	} or {}
end

-- labels
function GetLabels (campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignLabels",
		key = campaign_id,
		superColumns = true
	} or {}
end

function SaveLabels (campaign_id, labels, keyspace)
	return utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignLabels",
		superColumns = true,
		rows = {
			[campaign_id] = labels
		}
	}
end

function DeleteLabels (campaign_id, labels, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignLabels",
		superColumns = true,
		rows = {
			[campaign_id] = labels
		}
	}
end

-- rules
function GetRules (campaign_id, keyspace)
	local rules = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignRules",
		key = campaign_id,
		superColumns = true
	} or {}
	for rule_id, rule in pairs(rules) do
		local rule_error = m_datasource.redis:get("RuleError:" .. campaign_id .. ":" .. rule_id)
		if rule_error and rule_error ~= "" then
			rule.error = json.decode(rule_error)
		end
	end
	return rules
end

function SaveRules (campaign_id, rules, keyspace)
	for rule_id, rule in pairs(rules) do
		if rule.error then
			rule.error = nil
		else
			m_datasource.redis:del("RuleError:" .. campaign_id .. ":" .. rule_id)
		end
	end

	return utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignRules",
		superColumns = true,
		rows = {
			[campaign_id] = rules
		}
	}
end

function DeleteRules (campaign_id, rules_to_delete, keyspace)

	if not rules_to_delete then
		-- Si no se ha especifiado rules a eliminar (rules_to_delete), debo eliminar todos los rules de la campana.
		local campaign_rules = GetRules(campaign_id, keyspace)
		rules_to_delete = {}
		for rule_id in pairs(campaign_rules) do
			table.insert(rules_to_delete, rule_id)
		end
	end

	for _, rule_id in pairs(rules_to_delete) do
		LogInfo("[DeleteRules] Deleting rule '%s' of campaign: '%s'.", rule_id, campaign_id)
		m_datasource.redis:del("RuleError:" .. campaign_id .. ":" .. rule_id)
	end

	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignRules",
		superColumns = true,
		rows = {
			[campaign_id] = rules_to_delete
		}
	}
end

function GetAttentionLevels (campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAttentionLevels",
		key = campaign_id
	} or {}
end

function GetAttentionLevelsByUser (user_id, keyspace)
	local user_id_without_vcc = utils.GetId( user_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local full_user_id = utils.AppendVcc(user_id_without_vcc, vcc )

	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AttentionLevelsByUser",
		key = full_user_id
	} or {}
end

function IsUserInAttentionLevels (user_id, keyspace)
	local user_id_without_vcc = utils.GetId( user_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local full_user_id = utils.AppendVcc(user_id_without_vcc, vcc )
	local als_by_user = GetAttentionLevelsByUser( full_user_id, keyspace )
	local has_als = not utils.TableIsEmpty(als_by_user)
	return has_als
end

function CountWaitingConversationsForAttentionLevel (fullCampaignId, accountId, attentionLevelId)
	local key = string.format("InboundWaitingInteractionsByCampaignAccountAttentionLevel:%s:%s:%s", fullCampaignId, accountId, attentionLevelId)
	local waitingConversations = m_datasource.redis:zrange(key, 0, -1) or {}
	--LogDebug("[CAMPAIGNS:CountWaitingConversationsForAttentionLevel] key: '%s', number waiting conversations: '%s'", key, #waitingConversations)
	return #waitingConversations
end

local function SaveAttentionLevelsUpdateAgentService ( campaignWithVCC, new_attention_levels, old_attention_levels )

	new_attention_levels = new_attention_levels or {}
	old_attention_levels = old_attention_levels or {}

	LogInfo("+Save Campaign Attention Levels, Update AgentService -> campaign:'%s', Attention Levels: '%s'.",campaignWithVCC, json.encode(new_attention_levels) )

	local _, camp_vcc = SeparateVCC(campaignWithVCC)

	-- Map the data to the format expected by the agentservice, which is:
	-- { { id = "attention_level_id", members = { "agent_id", ... , "agent_id" } }, removed_members = { "agent_id", ... , "agent_id" } }, })
	local levels = {}

	for new_al_id, attn_level in pairs(new_attention_levels) do

		local old_al = old_attention_levels[new_al_id]
		local level = { id = attn_level.id, members = {}, removed_members = {} }

		-- (1) loop que recorre los usuarios del nuevo AL
		for new_al, user in pairs(attn_level.users) do
			table.insert( level.members, utils.AppendVcc(user.id, camp_vcc) )
			if old_al then
				-- se elimina, porque en el otro loop (el 2), se van a recorrer la lista members que ya no pertenezcan al AL
				-- para luego agegarlo a la lista de removidos.
				old_al.users[ user.id ] = nil
			end
		end

		if old_al then
		-- (2) loop que recorre los usuarios del viejo AL
			for _, user in pairs(old_al.users) do
				table.insert( level.removed_members, utils.AppendVcc(user.id, camp_vcc) )
			end
		end

		table.insert(levels, level)
	end

	agentservice:GetAgentClient():UpdateAttentionLevelsMembers(campaignWithVCC, levels)

	-- Store a way to lookup which levels a user belongs to.
	for _, level in ipairs(levels) do
		for _, member_id in ipairs(level.members) do
			m_datasource.redis:sadd("AttentionLevels::ByUser:"..member_id, level.id)
		end
		for _, member_id in ipairs(level.removed_members) do
			m_datasource.redis:srem("AttentionLevels::ByUser:"..member_id, level.id)
		end
	end

	-- Hint the agent poller that it needs to refresh attention levels
	m_datasource.redis:publish("AgentPoller::api", json.encode({ command = "AttentionLevelsModified" }))
end

---
-- to_delete is a table in the form: { "attention_level_id" = { "account_id", "account_id", ... "account_id" } }
--
local function DeleteAttentionLevelsUpdateAgentService (campaign_id, accounts_by_al_to_delete, members_by_al_to_delete)
	assert(type(accounts_by_al_to_delete) == "table")
	assert(type(members_by_al_to_delete) == "table")

	LogInfo("-Delete Campaign Attention Levels, Update AgentService -> campaign:'%s', Accounts By Attention Levels: '%s', Members By Attention Levels: '%s'.", campaign_id, json.encode(accounts_by_al_to_delete), json.encode(members_by_al_to_delete))

	agentservice:GetAgentClient():DeleteAttentionLevels(campaign_id, accounts_by_al_to_delete)

	-- Store a way to lookup which levels a user belongs to.
	for _, level in ipairs(members_by_al_to_delete) do
		for _, member_id in ipairs(level.removed_members) do
			m_datasource.redis:srem("AttentionLevels::ByUser:"..member_id, level.id)
		end
	end

	-- Hint the agent poller that it needs to refresh attention levels
	m_datasource.redis:publish("AgentPoller::api", json.encode({ command = "AttentionLevelsModified" }))
end

--
-- new_users = { user1={ al1="campaign1@inconcert", al2="campaign2@inconcert"}, user2={ al1="campaign1@inconcert"} ... }
--
local function AddAttentionLevelsByUser( new_users, vcc, keyspace )

	vcc = vcc or inconcert.authUser.virtualContactCenter

	for new_user_id, als in pairs(new_users or {}) do
		local full_user_id = utils.AppendVcc(new_user_id, vcc)
		local rows = {}
		rows[full_user_id] = als
		utils.SaveRows {
			keyspace = keyspace or "Agent",
			columnFamily = "AttentionLevelsByUser",
			rows = rows
		}
	end
end

--
-- removed_users = { user1={ 1="al1", 2="al2", 3="al3" .. }, user2={ 1="al1", 2="al2" ..} ... }
--
local function RemoveAttentionLevelsByUser( removed_users, vcc, keyspace )

	vcc = vcc or inconcert.authUser.virtualContactCenter

	for removed_user_id, als in pairs(removed_users or {}) do
		local full_user_id = utils.AppendVcc(removed_user_id, vcc)
		local rows = {}
		rows[full_user_id] = als
		utils.RemoveColumns {
			keyspace = keyspace or "Agent",
			columnFamily = "AttentionLevelsByUser",
			rows = rows
		}
	end
end

-- Actualiza la lista Attention levels por usuarios
local function UpdateAttentionLevelsByUser (campaignWithVCC, new_attention_levels, old_attention_levels, keyspace)

	new_attention_levels = new_attention_levels or {}
	old_attention_levels = old_attention_levels or {}

	local difference = { new_users = {}, removed_users = {} }

	for new_al_id, new_attn_level in pairs(new_attention_levels) do

		local old_al = old_attention_levels[new_al_id]

		for new_al, user in pairs(new_attn_level.users) do
			if not old_al or not old_al.users[ user.id ] then
				difference.new_users[ user.id ] = difference.new_users[ user.id ] or {}
				difference.new_users[ user.id ][ new_al_id ] = campaignWithVCC
			end
		end
	end

	for old_al_id, old_attn_level in pairs(old_attention_levels) do

		local new_al = new_attention_levels[old_al_id]
		if new_al then
			for old_al, user in pairs(old_attn_level.users) do
				if not new_al.users[ user.id ] then
					difference.removed_users[ user.id ] = difference.removed_users[ user.id ] or {}
					table.insert(difference.removed_users[ user.id ], old_al_id)
				end
			end
		end
	end

	--LogDebug("[UpdateAttentionLevelsByUser] difference: %s", json.encode(difference))

	local _, camp_vcc = utils.SeparateVCC(campaignWithVCC)
	-- agrego los nuevos usuarios asignados al attention level
	AddAttentionLevelsByUser(difference.new_users, camp_vcc, keyspace)
	-- remuevo los usuarios que ya no pertenecen al attention level
	RemoveAttentionLevelsByUser(difference.removed_users, camp_vcc, keyspace)
end

--[[
	
]]--
function UpdateCampaign (campaignWithVCC, campaignInfo, ksAgent)
	assert(campaignWithVCC)
	assert(campaignInfo)
	utils.SaveRows {
		keyspace = ksAgent or "Agent",
		columnFamily = "Campaigns",
		rows = {
			[campaignWithVCC] = campaignInfo
		}
	}
end

-- Save Attention levels
function SaveAttentionLevels (campaignWithVCC, new_attention_levels, keyspace)

	local old_attention_levels = GetAttentionLevels(campaignWithVCC, keyspace)

	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAttentionLevels",
		rows = {
			[campaignWithVCC] = new_attention_levels
		}
	}
	UpdateAttentionLevelsByUser( campaignWithVCC, new_attention_levels, old_attention_levels, keyspace )
	SaveAttentionLevelsUpdateAgentService( campaignWithVCC, new_attention_levels, old_attention_levels )
end

function DeleteAttentionLevels (campaignWithVCC, att_levels_to_remove, keyspace)

-- (1) Obtengo los ALs de la campana, antes de ser eliminados
--
	local current_attention_levels = GetAttentionLevels( campaignWithVCC, keyspace )

-- (2) Elimino el attention level de la CF "CampaignAttentionLevels"
--
	local att_levels = {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAttentionLevels"
	}

	if att_levels_to_remove then
		-- Remueve una columna especifica ( o sea un attention level de la campana )
		att_levels.rows = {
			[campaignWithVCC] = att_levels_to_remove
		}
	else
		-- Remueve todos los attention levels de la campana
		--   (esto por lo gral es utilizado cuando se elimina una campana por completo )
		att_levels.rows = {campaignWithVCC}
	end

	utils.RemoveColumns(att_levels) -- remuevo los ALs

	local _, camp_vcc = utils.SeparateVCC(campaignWithVCC)

	local members_by_al_to_delete = {}
	-- funcion que agrega los members que pertenecen a los ALs que se estan eliminando.
	local function AddMembersToDelete( al )
		if not al then return end
		local al_to_delete = { id = al.id, removed_members = {} }
		for _, user in pairs(al.users) do
			table.insert( al_to_delete.removed_members, utils.AppendVcc(user.id, camp_vcc) )
		end
		table.insert( members_by_al_to_delete, al_to_delete )
	end

	local accounts_by_al_to_delete = {}

	if not att_levels_to_remove then
	-- (3.1) Si esta eliminando la campana por completo, tengo que agregar todos los members de todos los ALs de la campana
		for _, al in pairs(current_attention_levels) do
			AddMembersToDelete(al)
		end

	else
	-- (3.2) Si esta eliminando algunos ALs especificos, tengo que agregar todos los members de estos ALs especificos
		for _, att_level_id in pairs(att_levels_to_remove) do
			local al = current_attention_levels[att_level_id]
			AddMembersToDelete(al)
		end

	-- (3.3) Actualizo los attention levels asignados en las cuentas.
	--   Para esto tengo que eliminar el/los attention level/s previamente obtenido/s en (1), del JSON de attention levels de las cuentas de la campana --> "CampaignAccounts->AttentionLevels"
	--

		-- build an inverted index of attention levels to be removed
		local inverted = {}
		for _, v in ipairs(att_levels_to_remove) do inverted[v] = true end

		local accounts = GetCampaignAccounts( campaignWithVCC, keyspace )

		for _, account in pairs( accounts ) do
			local account_attention_levels = account.AttentionLevels or {}
			if type(account_attention_levels) == "string" then
				account_attention_levels = json.decode(account.AttentionLevels)
			end

			-- build a table where the key is an attenion level and the value is an array of accounts
			for _, att_level_id in pairs(account_attention_levels) do
				if inverted[att_level_id] then
					accounts_by_al_to_delete[att_level_id] = accounts_by_al_to_delete[att_level_id] or {}
					table.insert(accounts_by_al_to_delete[att_level_id], account.id)
				end
			end

			for _, att_level_id in ipairs( att_levels_to_remove ) do
				account_attention_levels[att_level_id] = nil
			end

			account.AttentionLevels = account_attention_levels
			local success, err = SaveAccount(campaignWithVCC, account, false, keyspace) -- false = update
		end
	end

	-- (4) remuevo los usuarios que ya no pertenecen al attention level CF "AttentionLevelsByUser"
	local als_by_users = {}
	for _, level in ipairs(members_by_al_to_delete) do
		for _, member_id in ipairs(level.removed_members) do
			als_by_users[ member_id ] = als_by_users[ member_id ] or {}
			table.insert(als_by_users[ member_id ], level.id)
		end
	end

	RemoveAttentionLevelsByUser(als_by_users, camp_vcc, keyspace)

	-- (5) invoca funcion que actualiza la info en redis. Esta info es utilizada por el agentservice.
	DeleteAttentionLevelsUpdateAgentService(campaignWithVCC, accounts_by_al_to_delete, members_by_al_to_delete)
end

---

function MakeAccountMessageId ( campaign_id, accountId ) return ( accountId or "" ).. "_" .. ( campaign_id or "" ) end

local function DeleteRepositoryFiles (files)
	files = files or {}
	LogInfo("[CAMPAIGNS:DeleteRepositoryFiles] Removing Account Messages Repository Files: '%s'.", json.encode(files))
	for _, file in ipairs(files) do
		Repository.erase_file(file.id, file.section)
	end
end

function GetAccountMessages ( campaign_id, accountId, keyspace )
	local key = MakeAccountMessageId( campaign_id, accountId )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAccountMessages",
		key = key
	} or {}
end

function GetCampaignAccountsMessages (campaignId, campaignAccounts, keyspace)
	local accountsMessages = {}
	for _, account in pairs(campaignAccounts or {}) do
		accountsMessages[account.id] = GetAccountMessages(campaignId, account.id, keyspace)
	end
	return accountsMessages
end


local function CleanUpMailAccountMessagesRepoFiles (campaignId, accountId, newMessages, oldMessages, keyspace)
	local forceDeleteOldMessages = false
	if not newMessages then
		forceDeleteOldMessages = true
	end

	LogInfo("CleanUpMailAccountMessagesRepoFiles 1 campaignId: '%s', account: '%s' forceDeleteOldMessages: '%s'", campaignId, accountId, forceDeleteOldMessages)

	local filesToDelete
	local function IncludeFiles (files)
		filesToDelete = filesToDelete or {}
		for _, file in ipairs(files or {}) do
			table.insert(filesToDelete, {id = file.id, section = "-1"})
		end
	end

	oldMessages = oldMessages or GetAccountMessages(campaignId, accountId, keyspace) or {}

	for oldMessageId, oldMessage in pairs(oldMessages) do

		local oldAttachments = utils.JSON_Decode(oldMessage.attachments)

		if forceDeleteOldMessages then
			IncludeFiles(oldAttachments)

		elseif newMessages and newMessages[oldMessageId] then

			if newMessages[oldMessageId].attachments then
				local newAttachments = utils.JSON_Decode(newMessages[oldMessageId].attachments)

				for _, oldAttachment in ipairs(oldAttachments or {}) do
					local canDelete = true

					for _, newAttachment in ipairs(newAttachments or {}) do
						if oldAttachment.id == newAttachment.id then
							canDelete = false
							break
						end
					end
					if canDelete then
						local oldAttachmentsToDelete = {}
						table.insert(oldAttachmentsToDelete, oldAttachment)
						IncludeFiles(oldAttachmentsToDelete)
					end
				end
			else
				IncludeFiles(oldAttachments)
			end
		else
			IncludeFiles(oldAttachments)
		end
	end

	DeleteRepositoryFiles(filesToDelete)
end

function CleanUpMailCampaignAccountsMessagesRepoFiles (campaignId, accountsMessages, keyspace)
	LogInfo("[administration.campaigns.CleanUpMailCampaignAllAccountsMessagesRepoFiles] Removing Repository Images files in Campaign Messages '%s'.", campaignId)

	for accountId, messages in pairs(accountsMessages or {}) do
		CleanUpMailAccountMessagesRepoFiles (campaignId, accountId, nil, messages, keyspace)
	end
end

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model:BuildModels()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function SaveAccountMessages (campaign_id, account_id, newMessages, keyspace, allegroOtherInfo, vccId)
	local is_allegro_campaign = utils.IsAllegroCampaign( campaign_id, keyspace, vccId)
	local vcc = vccId or inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.CAMPAIGNS:SaveAccountMessages] - Saving newMessages in campaign: '%s', account: '%s', is_allegro_campaign: '%s'.", campaign_id, account_id, is_allegro_campaign )
	if is_allegro_campaign then
		local success, err = allegro_repository.SaveMMResources(newMessages, campaign_id, account_id, vcc)

		if success then
			local allegro_info = { action_id = "SAVE_ACCOUNT_MESSAGES", AccountMessages = newMessages }
			IncludeAllegroOtherInfo(allegro_info, allegroOtherInfo)

			local success, err = UpdateAllegroCampaignIn55( campaign_id, keyspace, allegro_info )
			if not success then
				return success, err
			end
		else
			err = err or {}
			if err.type == "ALLEGRO_INTEGRATION_ERROR" then
				LogError("[ADMINISTRATION.CAMPAIGNS:SaveAccountMessages] - Allegro integration error. Updating mm resources '%s'. Response: '%s'.", campaign_id, json.encode(err))
			else
				LogInfo("[ADMINISTRATION.CAMPAIGNS:SaveAccountMessages] - Allegro integration. Updating mm resources '%s'. Response: '%s'.", campaign_id, json.encode(err))
			end
			return false, err
		end
	end

	local hasRemoveRepositoryFiles = false
	RemoveAccountMessages(campaign_id, account_id, hasRemoveRepositoryFiles, newMessages, keyspace, vcc)

	local key = MakeAccountMessageId( campaign_id, account_id )
	local success = utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAccountMessages",
		rows = {
			[key] = newMessages
		}
	}

	local info = {
		vcc = vcc,
		campaignId = campaign_id,
		accountId = account_id,
		messagesAccountId = key,
		messages = newMessages
	}

	GetModel("SharedFiles").SaveSharedFilesAccountMessagesRelations(info, keyspace)

	return success
end

local function RemoveAccountMessagesRepositoryFiles ( campaign_id, account_id, keyspace )
	local is_allegro_campaign = utils.IsAllegroCampaign( campaign_id, keyspace )
	if not is_allegro_campaign then
		return false
	end

	local account_messages = GetAccountMessages( campaign_id, account_id, keyspace ) or {}

	local files_to_delete
	for message_id, message in pairs( account_messages ) do
		if ( message.type or "" ):lower() == "file" and message.fileInformation then
			files_to_delete = files_to_delete or {}
			table.insert( files_to_delete, { id = message.fileInformation.id, section = message.fileInformation.section } )
		end
	end

	DeleteRepositoryFiles(files_to_delete)
	return true
end

--[[
Opcionales:
	newMessages, keyspace
]]--
function RemoveAccountMessages (campaign_id, account_id, hasRemoveRepositoryFiles, newMessages, keyspace, vccId)
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.CAMPAIGNS.RemoveAccountMessages] Removing Account Messages in Campaign %s. account: %s", campaign_id, account_id)

	-- Tengo que depurar las imagenes de mail que ya no se encuentran asociados a la cuenta.
	CleanUpMailAccountMessagesRepoFiles(campaign_id, account_id, newMessages, nil, keyspace)

	if hasRemoveRepositoryFiles then
		RemoveAccountMessagesRepositoryFiles( campaign_id, account_id, keyspace )
	end

	local key = MakeAccountMessageId( campaign_id, account_id )
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAccountMessages",
		rows = { key }
	}

	local info = {
		vcc = vcc,
		campaignId = campaign_id,
		accountId = account_id,
		messagesAccountId = key
	}

	GetModel("SharedFiles").SaveSharedFilesAccountMessagesRelations(info, keyspace)
end

function UserIsMember (campaign_id, userId, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local res, column = pcall(keyspace.GetColumn, keyspace, campaign_id, "CampaignMembers", userId)
	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		return column and column.value
	end
end

function RemoveAccountsSkills ( campaign_id, campaign_accounts, keyspace )
	LogInfo("[administration.campaigns.RemoveAccountsSkills] Removing skills accounts in '%s' campaign. campaign_accounts: %s", campaign_id, json.encode(campaign_accounts))

	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	for _, account in pairs(campaign_accounts or {}) do
		agent_model.Campaigns.RemoveAccountsSkills(keyspace, campaign_id, account)
	end

	if closeKeyspace then
		keyspace:Close()
	end
end

function RemoveAccountsSignature ( campaign_id, campaign_accounts, keyspace )
	LogInfo("[administration.campaigns.RemoveAccountsSignature] Removing signatures accounts in '%s' campaign. campaign_accounts: %s", campaign_id, json.encode(campaign_accounts))

	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	for _, account in pairs(campaign_accounts or {}) do
		agent_model.Campaigns.RemoveAccountsSignature(keyspace, campaign_id, account)
	end

	if closeKeyspace then
		keyspace:Close()
	end
end

function RemoveAccountsFlow ( campaign_id, campaign_accounts, keyspace )
	LogInfo("[administration.campaigns.RemoveAccountsFlow] Removing flows accounts in '%s' campaign. campaign_accounts: %s", campaign_id, json.encode(campaign_accounts))

	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	for _, account in pairs(campaign_accounts or {}) do
		agent_model.Campaigns.RemoveAccountsFlow(keyspace, campaign_id, account)
	end

	if closeKeyspace then
		keyspace:Close()
	end
end

function RemoveAccountsChannelTasks ( campaign_id, campaign_accounts, keyspace, removeData )
	LogInfo("[administration.campaigns.RemoveAccountsChannelTasks] Doing channel tasks to remove accounts in '%s' campaign. campaign_accounts: %s", campaign_id, json.encode(campaign_accounts))

	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	for _, account in pairs(campaign_accounts or {}) do
		agent_model.Campaigns.RemoveAccountChannelTasks(keyspace, campaign_id, account, removeData)
	end

	if closeKeyspace then
		keyspace:Close()
	end
end

function RemoveCalli6AccountPackagesRelations (campaignIdWithVCC, campaignAccounts, ksAgent, ksProv)
	LogInfo("[administration.campaigns.RemoveCalli6AccountPackagesRelations] Removing Packages relations in Call campaign %s", campaignIdWithVCC)
	for _, account in pairs(campaignAccounts or {}) do
		-- actualizo la relación de la campaña de Call de i6 (sin allegro), con los paquetes (vccpackages)
		agent_model.Campaigns.RemoveCalli6AccountPackagesRelations(campaignIdWithVCC, account.id, nil, ksAgent, ksProv)
	end
end

function RemoveBlacklistsAccounts (campaignIdWithVCC, campaignAccounts, keyspace)
	LogInfo("[administration.campaigns.RemoveBlacklistsAccounts] Removing Blacklists relations in campaign %s", campaignIdWithVCC)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end
	for _, account in pairs(campaignAccounts or {}) do
		agent_model.Campaigns.RemoveBlacklistsAccount(keyspace, campaignIdWithVCC, account.id, account)
	end
	if closeKeyspace then
		keyspace:Close()
	end
end

--[[
	-- Guarda en Allegro la campana
]]--
function UpdateAllegroCampaignIn55 ( campaignWithVCC, keyspace, allegro_info )
	local vcc = inconcert.authUser.virtualContactCenter
	local info = utils.DeepCopy(allegro_info)

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateAllegroCampaignIn55] - Allegro integration. Updating campaign '%s'. Vcc '%s' has not configured integration with allegro.", campaignWithVCC, vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_campaigns.Update(campaignWithVCC, vcc, info, keyspace)
	if not success then
		if err.type == "ALLEGRO_INTEGRATION_ERROR" then
			LogError("[ADMINISTRATION.CAMPAIGNS:UpdateAllegroCampaignIn55] - Allegro integration error. Updating campaign '%s'. Response: '%s'.", campaignWithVCC, json.encode(err))
		else
			LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateAllegroCampaignIn55] - Allegro integration. Updating campaign '%s'. Response: '%s'.", campaignWithVCC, json.encode(err))
		end
		return false, err
	end

	return true
end

function SaveScheduler (campaignWithVCC, scheduler, keyspace, allegroOtherInfo)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	if not CampaignExists(campaignWithVCC, keyspace) then
		if closeKeyspace then
			keyspace:Close()
		end
		return false
	end

	if scheduler then
		local allegro_info = { action_id = "SAVE_SCHEDULE", scheduler = scheduler }
		IncludeAllegroOtherInfo(allegro_info, allegroOtherInfo)

		local success, err = UpdateAllegroCampaignIn55( campaignWithVCC, keyspace, allegro_info )
		if not success then
			-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
			--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
			--  la actualizacion en i6.
			if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
				if closeKeyspace then
					keyspace:Close()
				end
				return false, { result = err.type, err = (err.err or err.sub_type) }
			end
		end

		local days = {}
		for _, schedule_day in ipairs(scheduler.scheduleDays or {}) do
			local sched_key = string.format( "%s_%s_%s", schedule_day.dayOfWeek, schedule_day.startTime, schedule_day.endTime)
			days[sched_key] = ""
		end

		utils.SaveRows {
			keyspace     = keyspace or "Agent",
			columnFamily = "CampaignSchedulers",
			removeBefore = true,
			rows = {
				[campaignWithVCC] = days
			}
		}

		-- Aqui almacenamos el Special Days ----------
		local scheduleSpecialDays = scheduler.scheduleSpecialDays or {}
		local scheduleSpecialDayAllYearss = scheduler.scheduleSpecialDayAllYearss or {}

		local specialDays = {
			scheduleSpecialDays = json.encode(scheduleSpecialDays),
			scheduleSpecialDayAllYearss = json.encode(scheduleSpecialDayAllYearss)
		}

		utils.SaveRows {
			keyspace     = keyspace or "Agent",
			columnFamily = "CampaignSchedulersSpecialDays",
			removeBefore = true,
			rows = {
				[campaignWithVCC] = specialDays
			}
		}
	end

	if closeKeyspace then
		keyspace:Close()
	end

	return true
end

--[[
En esta funcion se remueven los Special Days y no el Scheduler en si, ya que el a esta CF la de los schedulers, se da de baja en el AS.
]]--
function RemoveSchedulerSpecialDays (campaignWithVCC, keyspace)
	LogInfo("[administration.campaigns.RemoveSchedulerSpecialDays] Removing Campaign '%s' Schedulers Special Days.", campaignWithVCC)

	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignSchedulersSpecialDays",
		rows = {campaignWithVCC}
	}

	if closeKeyspace then
		keyspace:Close()
	end

	return true
end

function SaveCampaignDisposition (campaignWithVCC, disposition_codes, keyspace, allegroOtherInfo)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local allegro_info = { action_id = "SAVE_DISPOSITIONS", dispositions = disposition_codes }
	IncludeAllegroOtherInfo(allegro_info, allegroOtherInfo)

	local success, err = UpdateAllegroCampaignIn55( campaignWithVCC, keyspace, allegro_info )
	if not success then
		if not success then
			-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
			--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
			--  la actualizacion en i6.
			if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
				return false, { result = err.type, err = (err.err or err.sub_type) }
			end
		end
	end

	UpdateCampaign(campaignWithVCC, {
		dispositionCodes = disposition_codes
	}, keyspace)

	if closeKeyspace then
		keyspace:Close()
	end

	return true
end

local function UpdateWhatsappAuthorizationTimestamp(accountId)
	m_datasource.redis:set("WhatsappAuthenticationTimestamp:" .. accountId, LuaOrm.Date():time())
end

function StartWhatsappAuthorization(accountId)
	m_datasource.redis:del("WhatsappQRCode:" .. accountId)
	UpdateWhatsappAuthorizationTimestamp(accountId)
end

function StopWhatsappAuthorization(accountId)
	m_datasource.redis:del("WhatsappAuthenticationTimestamp:" .. accountId)
end

function SetWhatsappAuthorizationCode(accountId, code)
	m_datasource.redis:set("WhatsappAccountValidationCode:" .. accountId, code)
end

function GetWhatsappAuthorizationStatus(accountId)
	UpdateWhatsappAuthorizationTimestamp(accountId)
	local status_s = m_datasource.redis:get("WhatsappAuthenticationStatus:" .. accountId)
	if not status_s or status_s == "" then
		return { status = "NOT_FOUND" }
	end
	local status = json.decode(status_s)
	if status.status == "LOGIN" then
		status.qr = m_datasource.redis:get("WhatsappQRCode:" .. accountId)
	end
	return status
end

function GetOEConfig (campaignIdAtVCC, keyspace)
	local vcc = inconcert.authUser.virtualContactCenter
	campaignIdAtVCC = utils.AppendVcc(utils.GetId(campaignIdAtVCC), vcc)
	local encodedOEConfig = GetSpecificColumn(campaignIdAtVCC, "OEConfig", keyspace)
	--LogDebug("---> campaignIdAtVCC %s, encodedOEConfig: %s", campaignIdAtVCC, encodedOEConfig)
	if not encodedOEConfig then
		return false
	end
	local OEConfig = utils.JSON_Decode(encodedOEConfig)
	if utils.TableIsEmpty(OEConfig) then
		return false
	end
	return true, OEConfig
end

local function DecodeAccountInformation ( account )
	if not account then
		return
	end
	if type(account.ContactManagement) == "string" then
		account.ContactManagement = json.decode(account.ContactManagement)
	end
	if type(account.FlowInformation) == "string" then
		account.FlowInformation = json.decode(account.FlowInformation)
	end
	if type(account.TicketEmitter) == "string" then
		local t = json.decode(account.TicketEmitter)
		if is_empty(t) then
			account.TicketEmitter = nil
		else
			account.TicketEmitter = t
		end
	end
	if type(account.Skills) == "string" then
		account.Skills = json.decode(account.Skills)
	end
	if type(account.Survey) == "string" then
		local t = json.decode(account.Survey)
		if is_empty(t) then
			account.Survey = nil
		else
			account.Survey = t
		end
	end
	if type(account.AttentionLevels) == "string" then
		account.AttentionLevels = json.decode(account.AttentionLevels)
	end
	if type(account.QnAMaker) == "string" then
		account.QnAMaker = json.decode(account.QnAMaker)
	end
	if type(account.HSMConfig) == "string" then
		account.HSMConfig = json.decode(account.HSMConfig)
	end
	if type(account.incomingConfig) == "string" then
		account.incomingConfig = json.decode(account.incomingConfig)
	end
	if type(account.outgoingConfig) == "string" then
		account.outgoingConfig = json.decode(account.outgoingConfig)
	end
	if type(account.packages) == "string" then
		account.packages = json.decode(account.packages)
	end
	if type(account.GUI) == "string" then
		account.GUI = json.decode(account.GUI)
	end
	return account
end

function FindAccountById (accountId, keyspace)
	assert(accountId, "account is required")
	local close_keyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		close_keyspace = true
	end
	local accountColumns = keyspace:GetSliceRange(accountId, "AccountsById", "",
		{
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = true,
				count = 2000
			}
		}
	)
	if close_keyspace then
		keyspace:Close()
		keyspace = nil
	end
	local account, campaign
	for i, column in ipairs(accountColumns) do
		campaign = column.name
		account = json.decode(column.value)
		account = DecodeAccountInformation(account)
		break
	end
	return account, campaign
end

function GetChannelCampaignThresholds (campaignIdWithVCC, channel, keyspace)
	assert(type(channel) == "string")
	assert(type(campaignIdWithVCC) == "string")
	local thresholdKey = string.format("%s:%s", channel, campaignIdWithVCC)
	local result = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignThresholds",
		key = thresholdKey
	} or {}
	--LogDebug("[CAMPAIGNS:GetChannelCampaignThresholds] Loading campign thresholds. thresholdKey: '%s'. result: %s", thresholdKey, json.encode(result))
	return result
end
