module(..., package.seeall)
local utils = require "shared.apps.utils"
local Fullcontact = require "fullcontact"
local m_datasource

local function ExecuteFullcontactMethod (callback)
	--LogDebug("[ADMINISTRATION.CONTACTS.ExecuteFullcontactMethod] --------> Executing fullcontact method.")
	local administrationApp = require "apps.administration"
	administrationApp.InitializeFullcontact(callback)
end

function GetName()
	return "Contacts"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

--------------------------
-- ADDRESSES BOOK
--------------------------

local m_addressesBook = {}
function GetAddressesBookModel ()
	return m_addressesBook
end

function m_addressesBook.Search (params)
	assert(type(params) == "table")
	params.withContactsInfo = false

	local success, addressesBook
	ExecuteFullcontactMethod(function()
		success, addressesBook = Fullcontact.AddressesBook.List(params)
	end)
	if not success then
		local err = addressesBook
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] Error Searching Addresses Book. Err: %s", json.encode(err or {}))
		return false, err
	end

	-- ok
	return true, addressesBook
end

function m_addressesBook.Get (addressIdWithVcc)
	assert(type(addressIdWithVcc) == "string")

	local success, addressesBook
	ExecuteFullcontactMethod(function()
		success, addressesBook = Fullcontact.AddressesBook.Get(addressIdWithVcc)
	end)
	if not success then
		local err = addressesBook
		LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Get] Retrieving Addresses Book '%s'. Info: %s", addressIdWithVcc, json.encode(err or {}))
		return false, err
	end

	-- ok
	return true, addressesBook
end

function m_addressesBook.GetAddressByRawInfo (addressInfo)
	assert(type(addressInfo) == "table")

	local vcc = inconcert.authUser.virtualContactCenter
	local success, addressId, err
	ExecuteFullcontactMethod(function()
		success, addressId, err = Fullcontact.BuildAddressId(addressInfo.Type, addressInfo, vcc)
	end)
	if not success then
		LogWarning("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:GetAddressByRawAddress] Unable to build address '%s', error '%s', errType: '%s'.", json.encode(addressInfo), json.encode(err), err.errType)
		return false, err
	end
	local addressIdWithVcc = (addressId .. "@" .. vcc):upper()

	local success, addressesBook = m_addressesBook.Get(addressIdWithVcc)
	if not success then
		local err = addressesBook
		LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:GetAddressByRawAddress] Retrieving Addresses Book By Raw Info '%s'. Err: %s", addressIdWithVcc, json.encode(err or {}))
		return false, err, addressIdWithVcc
	end

	-- ok
	return true, addressesBook, addressIdWithVcc
end

function m_addressesBook.Save (addressInfo)
	assert(type(addressInfo) == "table")

	addressInfo.isNew = nil
	addressInfo.AddressId = nil
	addressInfo.VCC = inconcert.authUser.virtualContactCenter

	local success, response, err
	ExecuteFullcontactMethod(function()
		success, response, err = Fullcontact.AddressesBook.Save(addressInfo)
	end)
	if not success then
		local errType = response
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] Error Saving Addresses Book '%s', errType: %s, err: %s", json.encode(addressInfo), errType, json.encode(err or {}))
		return false, errType, err
	end

	-- ok
	return true
end

function m_addressesBook.Remove (addressId)
	assert(type(addressId) == "string")

	local success, err
	ExecuteFullcontactMethod(function()
		success, err = Fullcontact.AddressesBook.Remove(addressId)
	end)
	if not success then
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Remove] Error Removing Addresses Book '%s'. Err: %s", addressId, json.encode(err or {}))
		return false, err
	end

	-- ok
	return true
end


