module(..., package.seeall)
local utils = require "shared.apps.utils"
local agent_model = require("apps.agent.model"):GetModels()
local m_datasource

function GetName ()
	return "Signatures"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetSignatures (vcc, signatureId, keyspace)
	local signatures = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Signatures",
		superColumns = true,
		key = vcc,
		columns = signatureId and {signatureId} or {}
	}
	if signatureId then
		return signatures and signatures[signatureId] or {}
	else
		return signatures or {}
	end
end

function SaveSignatures (vcc, signatures, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "Signatures",
		superColumns = true,
		rows = {
			[vcc] = signatures
		}
	}
end

function RemoveSignatures (vcc, signatures_to_remove, keyspace)

	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Signatures",
		superColumns = true,
		rows = {
			[vcc] = signatures_to_remove
		}
	}

	-- 1- Elimino los signatures utilizados por las cuentas.
	--
	-- Recorro la lista de signatures a remover, luego por cada uno de ellos recorro la lista de accounts que tienen el signature y mando a actualizar
	--  la cuenta con los signatures que le han quedado asociados.
	--
	for _, signature_id_to_remove in ipairs( signatures_to_remove ) do
		local signature_id_without_vcc = utils.GetId(signature_id_to_remove)
		-- obtengo la lista de ids cuentas por signature (los signatures a remover)
		local accounts_signature = GetAccountsBySignature( vcc, signature_id_to_remove, keyspace ) or {}
		-- ahora recorro esta lista de ids de cuentas.
		for _, account_signature in pairs( accounts_signature ) do
			-- obtengo la campana y recorro sus cuentas buscando en el array de cuentas los signatures a eliminar.
			local camp = agent_model.Campaigns.Get(account_signature.campaign_id)

			local accounts = agent_model.Campaigns.GetCampaignAccounts(account_signature.campaign_id, keyspace)
			for _, account in ipairs( accounts or {} ) do
				if account_signature.account_id == account.id then
					if account.Signature == signature_id_without_vcc then
						-- aca remuevo del array el signature que ya no pertenece y mando a guardar
						account.Signature = ""
						agent_model.Campaigns.SaveAccount(
							keyspace,
							account_signature.campaign_id,
							account_signature.account_id,
							account,
							false -- false == actualizando la account
						)
						break
					end
				end
			end
		end
	end
end

function GetAccountsBySignature (vcc, signature_id, keyspace)
	signature_id = utils.AppendVcc( signature_id, vcc )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AccountsBySignature",
		key = signature_id
	}
end
