module(..., package.seeall)

local agentUserModel = require "apps.agent.models.users"
local utils = require "shared.apps.utils"
local allegro_states = require "apps.allegro.controllers.states"

local m_datasource

function GetName()
	return "States"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

function GetStates(stateId, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentStates",
		startKey = stateId or "",
		endKey = stateId or "",
	}
end

--[[
	-- Guarda en Allegro los states
]]--
local function UpdateAllegroStatesIn55( states, keyspace )

	local vcc = inconcert.authUser.virtualContactCenter

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION:UpdateAllegroStatesIn55] - Allegro integration. Updating states '%s'. Vcc '%s' has not configured integration with allegro.", json.encode(states), vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_states.Save( states, vcc )
	if not success then
		LogError("[ADMINISTRATION:UpdateAllegroStatesIn55] - Allegro integration error. Updating states '%s'. Error: '%s'.", json.encode(states), json.encode(err))
		return false, { type = "ALLEGRO_INTEGRATION_ERROR", err = err }
	end

	return true
end

function SaveStates(states, keyspace)
	-- Guarda en Allegro los states
	-- Debo mandar a actualizar aca, en este lugar, antes de guardar algo en cassandra, asi me evito el Rollback y toda esas cosas!!!
	--
	local success, err = UpdateAllegroStatesIn55( states, keyspace )
	if not success then
		-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
		--  la actualizacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			if close_ks then keyspace:Close() end
			err.result = err.type
			return false, err
		end
	end

	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentStates",
		rows = states
	}

	return true
end

--[[
	-- Delete en Allegro los states
]]--
local function DeleteAllegroStatesIn55( states, keyspace )
	local vcc = inconcert.authUser.virtualContactCenter

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration. Deleting states '%s'. Vcc '%s' has not configured integration with allegro.", json.encode(states), vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_states.Delete( states, vcc )
	if not success then
		if err.type == "ALLEGRO_INTEGRATION_ERROR" then
			LogError("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration error. Deleting states '%s'. Response: '%s'.", json.encode(states), json.encode(err))
		else
			LogInfo("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration. Deleting states '%s'. Response: '%s'.", json.encode(states), json.encode(err))
		end
		return false, err
	end

	return true
end

function RemoveStates (states, keyspace)
	local success, err = DeleteAllegroStatesIn55(states, keyspace)
	if not success then
		-- Solo tiene que retornar sin eliminar el state en i6, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar la eliminacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			keyspace:Close()
			return false, {result = err.type, err = (err.err or err.sub_type)}
		end
	end
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentStates",
		rows = states
	}
	agentUserModel.UpdateUserStates(states, keyspace)
	return true
end
