/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if (!this.inConcert.Administration) {
	this.inConcert.Administration = {};
}

(function () {

	"use strict";

	const format = inConcert.Utils.format;

	var mark_chk = function( ctrl, checked ) { $( ctrl ).prop( "checked", checked ); };
	var is_enabled_chk = function( option ) { return !option || option === "1"; };

	var get_chk_bool_to_string  = function( chk_id ) {
		var $chk = $( chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

	function MailAccountHnd( params )
	{
		this.owner = params.owner;
		this.parent = params.parent;
		this.config_tabs = params.config_tabs || "all"; // default incoming and outgoing
		this.current_account = null;
		this.m_validate_name_before_incomming_section = typeof(params.validate_name_before_incomming_section) !== "boolean" ? true: params.validate_name_before_incomming_section;

		this.prefix_id             = params.prefix_id || "adm_form_campaign_accounts_mail_"  ;

		// common controls
		this.account_type_row      = "#" + this.prefix_id + "account_type_row"               ;
		this.account_type          = "#" + this.prefix_id + "account_type"                   ;
		this.account_name          = "#" + this.prefix_id + "account"                        ;
		this.display_name          = "#" + this.prefix_id + "display_name"                   ;
		this.outgoing_name         = "#" + this.prefix_id + "outgoing_name"                  ;
		this.info_automatic_update = "#" + this.prefix_id + "info_automatic_update"          ;

		// tabs and panels
		this.tabs_panels           = "#" + this.prefix_id + "tabs_panels"                    ;
		this.basic_tab             = "#" + this.prefix_id + "panel_tab_basic"                ;
		this.inc_out_tab           = "#" + this.prefix_id + "inc_out_tabs"                   ;

		// incoming controls
		this.user_incoming         = "#" + this.prefix_id + "user_incoming"                  ;
		this.pwd_incoming          = "#" + this.prefix_id + "pwd_incoming"                   ;
		this.protocol_lbl          = "#" + this.prefix_id + "protocol_lbl_incoming"          ;
		this.protocol_pop3         = "#" + this.prefix_id + "protocol_pop3_incoming"         ;
		this.protocol_imap         = "#" + this.prefix_id + "protocol_imap_incoming"         ;
		this.protocol_office365_api_graph_incoming = "#" + this.prefix_id + "protocol_office365_api_graph_incoming"         ;
		this.url_incoming          = "#" + this.prefix_id + "url_incoming"                   ;
		this.port_incoming         = "#" + this.prefix_id + "port_incoming"                  ;
		this.req_auth_incoming     = "#" + this.prefix_id + "req_auth_incoming"              ;
		this.req_encryp_incoming   = "#" + this.prefix_id + "req_encryp_incoming"            ;
		this.use_ssl_incoming      = "#" + this.prefix_id + "use_ssl_incoming"               ;
		this.use_thread            = "#" + this.prefix_id + "use_thread"                     ;
		this.spam_filter           = "#" + this.prefix_id + "spam_filter"                    ;

		// outgoing controls
		this.user_outgoing         = "#" + this.prefix_id + "user_outgoing"                  ;
		this.pwd_outgoing          = "#" + this.prefix_id + "pwd_outgoing"                   ;
		this.url_outgoing          = "#" + this.prefix_id + "url_outgoing"                   ;
		this.port_outgoing         = "#" + this.prefix_id + "port_outgoing"                  ;
		this.req_auth_outgoing     = "#" + this.prefix_id + "req_auth_outgoing"              ;
		this.req_encryp_outgoing   = "#" + this.prefix_id + "req_encryp_outgoing"            ;
		this.use_ssl_outgoing      = "#" + this.prefix_id + "use_ssl_outgoing"               ;
		this.auto_add_out_contact  = "#" + this.prefix_id + "auto_add_outgoing_contact"      ;
		this.protocol_smtp_outgoing = "#" + this.prefix_id + "protocol_smtp_outgoing"         ;
		this.protocol_office365_api_graph_outgoing = "#" + this.prefix_id + "protocol_office365_api_graph_outgoing"         ;

		this.tagline_new           = "<%=_T('New e-mail account.')%>";
		this.tagline_edit          = "<%=_T('Edit e-mail account {0}.')%>";
		this.tagline_readonly      = "<%=_T('E-mail account {0}.')%>";
	}

	if ( typeof inConcert.Administration.MailAccountHnd !== "function" ) {
		inConcert.Administration.MailAccountHnd = MailAccountHnd;
	}

	MailAccountHnd.prototype.initialize = function()
	{
		// cargo los templates de accounts
		this.fill_accounts_template();
		// dejo siempre seleccionada la primer solapa (incoming)
		this.select_tab_panel( "incoming" );
		// cuando ingreso reseteo el flag de que la pwd ha sido modificada. Tanto incoming como outgoing
		$( this.pwd_incoming ).attr( "pwd_changed", "0" );
		$( this.pwd_outgoing ).attr( "pwd_changed", "0" );
		// cuando ingreso reseteo el flag que indica que el user ha cambiado de forma automatica en la solapa outgoing
		$( this.user_outgoing ).attr( "manual_changed", "0" );
		$( this.pwd_outgoing ).attr( "manual_changed", "0" );
	};

	MailAccountHnd.prototype.set_account = function( account )
	{
		this.current_account = account || null;
		this.m_mailUid       = null;
		this.m_mailUPN       = null; // Used for Office 365 as outgoing account.
		this.m_mailOauth     = null;
    };

	MailAccountHnd.prototype.select_tab_panel = function( tab_id, on_selected )
	{
		on_selected = on_selected || $.noop;
		tab_id = ( tab_id || "" ).toLowerCase();
		var config_tabs = this.config_tabs.toLowerCase();
		if ( config_tabs !== "all" && config_tabs !== tab_id ) {
			tab_id = this.config_tabs;
		}

		$( this.inc_out_tab ).find( "li[tab_type]" ).removeClass( "tabsMenuItemSelect" );
		$( this.tabs_panels ).find( "div[panel]" ).hide();

		$( this.inc_out_tab ).find( "li[tab_type='" + tab_id + "']" ).addClass( "tabsMenuItemSelect" );
		$( this.tabs_panels ).find( "div[panel='" + tab_id + "']" ).show();

		on_selected();
	};

	MailAccountHnd.prototype.get_template_accounts = function()
	{
		var self = this;

		var __get_port = function($port) {
			var port = $port.val().trim();
			if ( port.length === 0 ) {
				return 0;
			}
			return parseInt( port, 10 );
		};

		return {
			others : { 
				name: "<%=_T('Generic')%>",
				order: 1,
				suggested_values : {
					account_name                           : { value: ""                      ,  enabled: true					  } ,
					user_outgoing                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_outgoing                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_outgoing                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					port_outgoing                          : { value: ""                      ,  enabled: true ,  required: false } ,
					user_incoming                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_incoming                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_incoming                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					port_incoming                          : { value: ""                      ,  enabled: true ,  required: false } ,
					protocol                               : { value: "imap"                  ,  enabled: true ,  required: false } ,
					req_auth_incoming                      : { value: false                   ,  enabled: true ,  required: false } ,
					req_auth_outgoing                      : { value: false                   ,  enabled: true ,  required: false } ,
					req_encryp_incoming                    : { value: false                   ,  enabled: true ,  required: false } ,
					req_encryp_outgoing                    : { value: false                   ,  enabled: true ,  required: false } ,
					use_ssl_incoming                       : { value: false                   ,  enabled: true ,  required: false } ,
					use_ssl_outgoing                       : { value: false                   ,  enabled: true ,  required: false } ,
					use_thread                             : { value: true                    ,  enabled: true ,  required: false } , 
					use_smtp_outgoing                      : { value: false                   ,  enabled: false,  required: false } ,
					use_office365_api_graph_outgoing       : { value: false                   ,  enabled: false,  required: false } ,
				},
				get_port: __get_port,
				set_default_values : function( account )
				{
					if ( self.has_config_type( "incoming" ) ) {
					// En el caso de que se haya agregado una URL entrante de tipo gmail, se debe de forzar al atributo "type" al valor "GMAIL"
					//   para que el emailproxy no realice el chequeo cada 30 segundos, sino cada 10 minutos.
					//
						var url_incoming = $( self.url_incoming ).val().toLowerCase();
						var gmail_incomming_servers = [ "imap.gmail.com", "pop.gmail.com" ];
						var is_gmail = false;
						$.each( gmail_incomming_servers, function(_, incomming_srv )
						{
							var index = url_incoming.indexOf(incomming_srv);
							if ( index > -1 ) {
								is_gmail = true;
								return false; // break
							}
						});
						if ( is_gmail ) {
							// gmail default values
							account.type = "GMAIL";
						}
						else {
							// en el caso que no sea GMAIL deben dejar los valores segun el protocolo
							var protocol_pop3 = $( self.protocol_pop3 ).prop("checked");
							account.type = protocol_pop3 ? "POP3" : "IMAP";
						}
					}
				}
			},
			gmail : {
				name : "<%=_T('Gmail')%>" ,
				order: 2,
				suggested_values : {
					account_name                           : { value: ""                      ,  enabled: false					  } ,
					user_outgoing                          : { value: ""                      ,  enabled: false,  required: true  } ,
					pwd_outgoing                           : { value: ""                      ,  enabled: false,  required: false } ,
					url_outgoing                           : { value: "smtp.gmail.com"        ,  enabled: false,  required: false } ,
					port_outgoing                          : { value: ""                      ,  enabled: false,  required: false } ,
					user_incoming                          : { value: ""                      ,  enabled: false,  required: true  } ,
					pwd_incoming                           : { value: ""                      ,  enabled: false,  required: false } ,
					url_incoming                           : { value: "imap.gmail.com"        ,  enabled: false,  required: false } ,
					port_incoming                          : { value: ""                      ,  enabled: false,  required: false } ,
					protocol                               : { value: "imap"                  ,  enabled: true ,  required: false } ,
					req_auth_incoming                      : { value: true                    ,  enabled: false,  required: false } ,
					req_auth_outgoing                      : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_incoming                    : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_outgoing                    : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_incoming                       : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_outgoing                       : { value: true                    ,  enabled: false,  required: false } ,
					use_thread                             : { value: true                    ,  enabled: true ,  required: false } , 
					use_smtp_outgoing                      : { value: false                   ,  enabled: false,  required: false } ,
					use_office365_api_graph_outgoing       : { value: false                   ,  enabled: false,  required: false } ,
					on_changed_inc_protocol : function(protocol)
					{
						var $url_incoming =  $( self.url_incoming );
						var $port_incoming =  $( self.port_incoming );

						if (protocol === "POP3") {
							$url_incoming.val("pop.gmail.com");
							$port_incoming.val("995");
						}
						else {
							$url_incoming.val("imap.gmail.com");
							$port_incoming.val("");
						}
					}
				},
				use_protocol_oauth: true,
				get_port: __get_port,
				set_default_values : function( account ) {
					// default values
					account.type = "GMAIL";
				},
				load : function() { self.__loadMailData(); },
				validate : function() { return self.__validateMail(); },
				set_data : function(account) { self.__setMailData(account); }
			},
			outlook : {
				order: 2,
				name : "<%=_T('Outlook')%>",
				suggested_values : {
					account_name                           : { value: ""                      ,  enabled: true					  } ,
					user_outgoing                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_outgoing                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_outgoing                           : { value: "smtp-mail.outlook.com" ,  enabled: false,  required: true  } ,
					port_outgoing                          : { value: "587"                   ,  enabled: false,  required: true  } ,
					user_incoming                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_incoming                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_incoming                           : { value: "imap-mail.outlook.com" ,  enabled: false,  required: true  } ,
					port_incoming                          : { value: "993"                   ,  enabled: false,  required: true  } ,
					protocol                               : { value: "imap"                  ,  enabled: false,  required: false } ,
					req_auth_incoming                      : { value: true                    ,  enabled: false,  required: false } ,
					req_auth_outgoing                      : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_incoming                    : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_outgoing                    : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_incoming                       : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_outgoing                       : { value: true                    ,  enabled: false,  required: false } ,
					use_thread                             : { value: true                    ,  enabled: true ,  required: false } , 
					use_smtp_outgoing                      : { value: false                   ,  enabled: false,  required: false } ,
					use_office365_api_graph_outgoing       : { value: false                   ,  enabled: false,  required: false } ,
				},
				get_port: __get_port,
				set_default_values : function( account )
				{
					if ( self.has_config_type( "incoming" ) ) {
						var protocol_pop3  = $( self.protocol_pop3 ).prop("checked");
						var use_ssl_incoming = $( self.use_ssl_incoming ).prop("checked");
						var incoming_protocol;
						if ( use_ssl_incoming ) {
							incoming_protocol = protocol_pop3 ? "pop3s" : "imaps";
						}
						else {
							incoming_protocol = protocol_pop3 ? "pop3" : "imap";
						}
						account.incomingConfig.url = incoming_protocol + "://" + $( self.url_incoming ).val();
					}
					if ( self.has_config_type( "outgoing" ) ) {
						var outgoing_protocol = "smtp";
						account.outgoingConfig.url = outgoing_protocol + "://" + $( self.url_outgoing ).val();
					}
				}
			},
			yahoo : { 
				name: "<%=_T('Yahoo')%>",
				order: 2,
				suggested_values : {
					account_name                           : { value: ""                      ,  enabled: true					  } ,
					user_outgoing                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_outgoing                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_outgoing                           : { value: "smtp.mail.yahoo.com"   ,  enabled: false,  required: true  } ,
					user_incoming                          : { value: ""                      ,  enabled: true ,  required: true  } ,
					pwd_incoming                           : { value: ""                      ,  enabled: true ,  required: true  } ,
					url_incoming                           : { value: "pop.mail.yahoo.com"    ,  enabled: false,  required: true  } ,
					port_outgoing                          : { value: ""                      ,  enabled: false,  required: false } ,
					port_incoming                          : { value: ""                      ,  enabled: false,  required: false } ,
					protocol                               : { value: "pop3"                  ,  enabled: false,  required: false } ,
					req_auth_incoming                      : { value: true                    ,  enabled: false,  required: false } ,
					req_auth_outgoing                      : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_incoming                    : { value: true                    ,  enabled: false,  required: false } ,
					req_encryp_outgoing                    : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_incoming                       : { value: true                    ,  enabled: false,  required: false } ,
					use_ssl_outgoing                       : { value: true                    ,  enabled: false,  required: false } ,
					use_thread                             : { value: true                    ,  enabled: true ,  required: false } ,
					use_smtp_outgoing                      : { value: false                   ,  enabled: false,  required: false } ,
					use_office365_api_graph_outgoing       : { value: false                   ,  enabled: false,  required: false } ,
				},
				get_port: __get_port
			},
			office365 : { 
				name: "<%=_T('Office 365')%>",
				order: 2,
				suggested_values : {
					account_name                           : { value: ""                     ,  enabled: false					} ,
					user_outgoing                          : { value: ""                     ,  enabled: false, required: true  } ,
					pwd_outgoing                           : { value: ""                     ,  enabled: false, required: false } ,
					url_outgoing                           : { value: "smtp.office365.com"   ,  enabled: false, required: false } ,
					user_incoming                          : { value: ""                     ,  enabled: false, required: false } ,
					pwd_incoming                           : { value: ""                     ,  enabled: false, required: false } ,
					url_incoming                           : { value: "outlook.office365.com",  enabled: false, required: false } ,
					port_outgoing                          : { value: "587"                  ,  enabled: false, required: false } ,
					port_incoming                          : { value: "993"                  ,  enabled: false, required: false } ,
					protocol                               : { value: "imap"                 ,  enabled: true , required: false } ,
					req_auth_incoming                      : { value: true                   ,  enabled: false, required: false } ,
					req_auth_outgoing                      : { value: true                   ,  enabled: false, required: false } ,
					req_encryp_incoming                    : { value: true                   ,  enabled: false, required: false } ,
					req_encryp_outgoing                    : { value: true                   ,  enabled: false, required: false } ,
					use_ssl_incoming                       : { value: true                   ,  enabled: false, required: false } ,
					use_ssl_outgoing                       : { value: false                  ,  enabled: false, required: false } ,
					use_thread                             : { value: true                   ,  enabled: true , required: false } ,
					url_office365_api_graph                : { value: "graph.microsoft.com"  ,  enabled: false, required: false } ,
					use_smtp_outgoing                      : { value: true                   ,  enabled: true , required: false } ,
					use_office365_api_graph_outgoing       : { value: false                  ,  enabled: true , required: false } ,
					on_changed_inc_protocol : function(protocol)
					{
						var $port_incoming =  $( self.port_incoming );
						var $port_outgoing =  $( self.port_outgoing );
						var $url_incoming =  $( self.url_incoming );
						var $url_outgoing =  $( self.url_outgoing );
						var $use_ssl_outgoing =  $( self.use_ssl_outgoing );
						mark_chk( $use_ssl_outgoing, this.use_ssl_outgoing.value, this.use_ssl_outgoing.enabled);

						if (protocol === "POP3") {
							$port_incoming.val("995");
							$port_outgoing.val(this.port_outgoing.value);
							$url_incoming.val(this.url_incoming.value);
							$url_outgoing.val(this.url_outgoing.value);
							mark_chk( $use_ssl_outgoing, this.use_ssl_outgoing.value, this.use_ssl_outgoing.enabled);
						}
						else if (protocol === "IMAP"){
							$port_incoming.val("993");
							$port_outgoing.val(this.port_outgoing.value);
							$url_incoming.val(this.url_incoming.value);
							$url_outgoing.val(this.url_outgoing.value);
							mark_chk( $use_ssl_outgoing, this.use_ssl_outgoing.value, this.use_ssl_outgoing.enabled);
						}
						else if (protocol === "OFFICE365_API_GRAPH"){
							$port_incoming.val("");
							$port_outgoing.val("");
							$url_incoming.val(this.url_office365_api_graph.value);
							$url_outgoing.val(this.url_office365_api_graph.value);
							mark_chk( $use_ssl_outgoing, true, this.use_ssl_outgoing.enabled);
						}
					},
					on_changed_out_protocol : function(protocol)
					{
						var $port_outgoing =  $( self.port_outgoing );
						var $url_outgoing =  $( self.url_outgoing );
						var $use_ssl_outgoing =  $( self.use_ssl_outgoing );

						if (protocol === "OFFICE365_API_GRAPH"){
							$port_outgoing.val("");
							$url_outgoing.val(this.url_office365_api_graph.value);
							mark_chk( $use_ssl_outgoing, true, this.use_ssl_outgoing.enabled);
						}
						else {
							$port_outgoing.val("587");	//SMTP
							$url_outgoing.val(this.url_outgoing.value);
							mark_chk( $use_ssl_outgoing, this.use_ssl_outgoing.value, this.use_ssl_outgoing.enabled);
						}
					}
				},
				use_xoauth_two_line: true,
				use_protocol_oauth: true,
				get_port: __get_port,
                set_default_values : function( account ) {
					// default values
					account.type = "OFFICE365";
				},
				load : function() { self.__loadMailData(); },
				validate : function() { return self.__validateMail(); },
				set_data: function (account) { self.__setMailData(account); }
			}
		};
	};

	MailAccountHnd.prototype.fill_accounts_template = function()
	{
		var template_accounts = this.get_template_accounts();
		var is_blank_first_item = false;
		var attr_id = null;
		var attr_name = "name";
		var sorted = "order";
		var $combo = $( this.account_type );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, template_accounts, attr_id, attr_name, sorted );
	};

	MailAccountHnd.prototype.set_template_to_ctrls = function( account_type )
	{
		const self = this;

		const $basic_tab = $(this.basic_tab);
		$basic_tab
			.find("[row_toggle_type]")
			.hide()
			.filter(format("[row_toggle_type='{0}']", account_type.toUpperCase()))
				.show();

		const $tabs_panels = $(this.tabs_panels);
		$tabs_panels
			.find("[row_toggle_type]")
			.hide()
			.filter(format("[row_toggle_type='{0}']", account_type.toUpperCase()))
				.show();

		var account_name = "", user_outgoing = "", pwd_outgoing = "", url_outgoing = "", user_incoming = "", pwd_incoming = "", 
			url_incoming = "",  port_outgoing = "", port_incoming = "", protocol_pop3 = true,
			req_auth_incoming = false, req_auth_outgoing = false, req_encryp_incoming = false, 
			req_encryp_outgoing = false,use_ssl_incoming = false, use_ssl_outgoing = false, use_thread = false,
			use_smtp_outgoing = false, use_office365_api_graph_outgoing = false;

		var enabled_account_name = true, enabled_user_outgoing = true, enabled_pwd_outgoing = true, enabled_url_outgoing = true, 
			enabled_user_incoming = true, enabled_pwd_incoming = true, enabled_url_incoming = true, 
			enabled_port_outgoing = true, enabled_port_incoming = true, enabled_protocol_pop3 = true,
			enabled_req_auth_incoming = true, enabled_req_auth_outgoing = true, enabled_req_encryp_incoming = true, 
			enabled_req_encryp_outgoing = true, enabled_use_ssl_incoming = true, enabled_use_ssl_outgoing = true, 
			enabled_use_thread = true, enabled_use_smtp_outgoing = true, enabled_use_office365_api_graph_outgoing = true;

		var required_account_name = true, required_user_outgoing = true, required_pwd_outgoing = true, required_url_outgoing = true, 
			required_user_incoming = true, required_pwd_incoming = true, required_url_incoming = true, 
			required_port_outgoing = true, required_port_incoming = true, required_protocol_pop3 = false,
			required_req_auth_incoming = false, required_req_auth_outgoing = false, required_req_encryp_incoming = false, 
			required_req_encryp_outgoing = false, required_use_ssl_incoming = false, required_use_ssl_outgoing = false, 
			required_use_thread = false, required_use_smtp_outgoing = false, required_use_office365_api_graph_outgoing = false;

		let template_accounts = this.get_template_accounts();
		let template = template_accounts[ account_type ];

		if ( template && template.suggested_values ) {
		// si tiene valores sugeridos
			var sv = template.suggested_values;

			account_name                              = sv.account_name.value                        ;
			enabled_account_name                      = this.is_new() && sv.account_name.enabled     ;

			user_outgoing                             = sv.user_outgoing.value                       ;
			enabled_user_outgoing                     = sv.user_outgoing.enabled                     ;
			required_user_outgoing                    = sv.user_outgoing.required                    ;
																							         
			pwd_outgoing                              = sv.pwd_outgoing.value                        ;
			enabled_pwd_outgoing                      = sv.pwd_outgoing.enabled                      ;
			required_pwd_outgoing                     = sv.pwd_outgoing.required                     ;
																							         
			url_outgoing                              = sv.url_outgoing.value                        ;
			enabled_url_outgoing                      = sv.url_outgoing.enabled                      ;
			required_url_outgoing                     = sv.url_outgoing.required                     ;
																							         
			user_incoming                             = sv.user_incoming.value                       ;
			enabled_user_incoming                     = sv.user_incoming.enabled                     ;
			required_user_incoming                    = sv.user_incoming.required                    ;
																							         
			pwd_incoming                              = sv.pwd_incoming.value                        ;
			enabled_pwd_incoming                      = sv.pwd_incoming.enabled                      ;
			required_pwd_incoming                     = sv.pwd_incoming.required                     ;
																							         
			url_incoming                              = sv.url_incoming.value                        ;
			enabled_url_incoming                      = sv.url_incoming.enabled                      ;
			required_url_incoming                     = sv.url_incoming.required                     ;
																							         
			port_outgoing                             = sv.port_outgoing.value                       ;
			enabled_port_outgoing                     = sv.port_outgoing.enabled                     ;
			required_port_outgoing                    = sv.port_outgoing.required                    ;
																							         
			port_incoming                             = sv.port_incoming.value                       ;
			enabled_port_incoming                     = sv.port_incoming.enabled                     ;
			required_port_incoming                    = sv.port_incoming.required                    ;
																							         
			protocol_pop3                             = sv.protocol.value === "pop3"                 ;
			enabled_protocol_pop3                     = sv.protocol.enabled                          ;
			required_protocol_pop3                    = sv.protocol.required                         ;
																							         
			req_auth_incoming                         = sv.req_auth_incoming.value                   ;
			enabled_req_auth_incoming                 = sv.req_auth_incoming.enabled                 ;
			required_req_auth_incoming                = sv.req_auth_incoming.required                ;
																							         
			req_auth_outgoing                         = sv.req_auth_outgoing.value                   ;
			enabled_req_auth_outgoing                 = sv.req_auth_outgoing.enabled                 ;
			required_req_auth_outgoing                = sv.req_auth_outgoing.required                ;
																							         
			req_encryp_incoming                       = sv.req_encryp_incoming.value                 ;
			enabled_req_encryp_incoming               = sv.req_encryp_incoming.enabled               ;
			required_req_encryp_incoming              = sv.req_encryp_incoming.required              ;
																							         
			req_encryp_outgoing                       = sv.req_encryp_outgoing.value                 ;
			enabled_req_encryp_outgoing               = sv.req_encryp_outgoing.enabled               ;
			required_req_encryp_outgoing              = sv.req_encryp_outgoing.required              ;
																							         
			use_ssl_incoming                          = sv.use_ssl_incoming.value                    ;
			enabled_use_ssl_incoming                  = sv.use_ssl_incoming.enabled                  ;
			required_use_ssl_incoming                 = sv.use_ssl_incoming.required                 ;
																							         
			use_ssl_outgoing                          = sv.use_ssl_outgoing.value                    ;
			enabled_use_ssl_outgoing                  = sv.use_ssl_outgoing.enabled                  ;
			required_use_ssl_outgoing                 = sv.use_ssl_outgoing.required                 ;
																							         
			use_thread                                = sv.use_thread.value                          ;
			enabled_use_thread                        = sv.use_thread.enabled                        ;
			required_use_thread                       = sv.use_thread.required                       ;

			use_smtp_outgoing                         = sv.use_smtp_outgoing.value                   ;
			enabled_use_smtp_outgoing                 = sv.use_smtp_outgoing.enabled                 ;
			required_use_smtp_outgoing                = sv.use_smtp_outgoing.required                ;

			use_office365_api_graph_outgoing          = sv.use_office365_api_graph_outgoing.value    ;
			enabled_use_office365_api_graph_outgoing  = sv.use_office365_api_graph_outgoing.enabled  ;
			required_use_office365_api_graph_outgoing = sv.use_office365_api_graph_outgoing.required ;
		}

		if(template)
		{
			if (template.use_protocol_oauth) {
				$basic_tab
					.find("[row_toggle_type]")
					.filter("[row_toggle_type='OAUTH']")
					.show();
			}

		}

		var $account_name = $( this.account_name );
		$account_name.val( account_name );

		var $user_outgoing = $( this.user_outgoing );
		$user_outgoing.val( user_outgoing );

		var $pwd_outgoing = $( this.pwd_outgoing );
		$pwd_outgoing.val( pwd_outgoing );

		var $url_outgoing = $( this.url_outgoing );
		$url_outgoing.val( url_outgoing );

		var $user_incoming = $( this.user_incoming );
		$user_incoming.val( user_incoming );

		var $pwd_incoming = $( this.pwd_incoming );
		$pwd_incoming.val( pwd_incoming );

		var $url_incoming = $( this.url_incoming );
		$url_incoming.val( url_incoming );

		var $port_outgoing = $( this.port_outgoing );
		$port_outgoing.val( port_outgoing );

		var $port_incoming = $( this.port_incoming );
		$port_incoming.val( port_incoming );

		mark_chk( this.protocol_pop3                          , protocol_pop3                                 , enabled_protocol_pop3                             );
		mark_chk( this.protocol_imap                          , !protocol_pop3                                , enabled_protocol_pop3                             );
																																			                      
		mark_chk( this.req_auth_incoming                      , req_auth_incoming                             , enabled_req_auth_incoming                         );
		mark_chk( this.req_auth_outgoing                      , req_auth_outgoing                             , enabled_req_auth_outgoing                         );
																																			                      
		mark_chk( this.req_encryp_incoming                    , req_encryp_incoming                           , enabled_req_encryp_incoming                       );
		mark_chk( this.req_encryp_outgoing                    , req_encryp_outgoing                           , enabled_req_encryp_outgoing                       );
																																			                      
		mark_chk( this.use_ssl_incoming                       , use_ssl_incoming                              , enabled_use_ssl_incoming                          );
		mark_chk( this.use_ssl_outgoing                       , use_ssl_outgoing                              , enabled_use_ssl_outgoing                          );
																																			                      
		mark_chk( this.use_thread                             , use_thread                                    , enabled_use_thread                                );

		mark_chk( this.protocol_smtp_outgoing                 , use_smtp_outgoing                             , enabled_use_smtp_outgoing                         );
		mark_chk( this.protocol_office365_api_graph_outgoing  , use_office365_api_graph_outgoing              , enabled_use_office365_api_graph_outgoing          );

		if ( !this.parent.is_readonly() ) {
		// si en el this.parent no esta en modo readonly, activa / desactiva los ctrls segun corresponda

			// el label del protocol tengo que hacerlo diferente a los demas, ya que no es un label y un control, sino
			//  un label y 2 controles (los 2 options groups "pop3" y "imap")
			var $lbl_protocol = $( this.protocol_lbl );
			if ( enabled_protocol_pop3 ) {
				$lbl_protocol.removeClass("formLabel-ds readonly").prop("disabled", false);
			}
			else {
				$lbl_protocol.addClass("formLabel-ds readonly").prop("disabled", true);
			}
			// los demas controles
			var ro_ctrl = function( $ctrl, readonly, required ) {
				self.parent.frm.readonly_control( $ctrl, readonly, required );
			};

			ro_ctrl( $account_name                                         , !enabled_account_name                                                             , required_account_name                           );
																																															                     
			ro_ctrl( $user_outgoing                                        , !enabled_user_outgoing                                                            , required_user_outgoing                          );
			ro_ctrl( $user_incoming                                        , !enabled_user_incoming                                                            , required_user_incoming                          );
																																															                     
			ro_ctrl( $pwd_outgoing                                         , !enabled_pwd_outgoing                                                             , required_pwd_outgoing                           );
			ro_ctrl( $pwd_incoming                                         , !enabled_pwd_incoming                                                             , required_pwd_incoming                           );
																																															                     
			ro_ctrl( $url_outgoing                                         , !enabled_url_outgoing                                                             , required_url_outgoing                           );
			ro_ctrl( $url_incoming                                         , !enabled_url_incoming                                                             , required_url_incoming                           );
																																															                     
			ro_ctrl( $port_outgoing                                        , !enabled_port_outgoing                                                            , required_port_outgoing                          );
			ro_ctrl( $port_incoming                                        , !enabled_port_incoming                                                            , required_port_incoming                          );
																																															                     
			ro_ctrl( $( this.protocol_pop3  )                              , !enabled_protocol_pop3                                                            , required_protocol_pop3                          );
			ro_ctrl( $( this.protocol_imap  )                              , !enabled_protocol_pop3                                                            , required_protocol_pop3                          );
			ro_ctrl( $( this.protocol_office365_api_graph_incoming  )      , (!enabled_protocol_pop3 || account_type !== "office365")                          , required_protocol_pop3                          );
																																				               
			ro_ctrl( $( this.protocol_smtp_outgoing  )                     , (!enabled_use_smtp_outgoing || account_type !== "office365")                      , required_use_smtp_outgoing                      );
			ro_ctrl( $( this.protocol_office365_api_graph_outgoing  )      , (!enabled_use_office365_api_graph_outgoing || account_type !== "office365")       , required_use_office365_api_graph_outgoing       );

			ro_ctrl( $( this.req_auth_incoming )                           , !enabled_req_auth_incoming                                                        , required_req_auth_incoming                      );
			ro_ctrl( $( this.req_auth_outgoing )                           , !enabled_req_auth_outgoing                                                        , required_req_auth_outgoing                      );
																																															                     
			ro_ctrl( $( this.req_encryp_incoming )                         , !enabled_req_encryp_incoming                                                      , required_req_encryp_incoming                    );
			ro_ctrl( $( this.req_encryp_outgoing )                         , !enabled_req_encryp_outgoing                                                      , required_req_encryp_outgoing                    );
																																															                     
			ro_ctrl( $( this.use_ssl_incoming )                            , !enabled_use_ssl_incoming                                                         , required_use_ssl_incoming                       );
			ro_ctrl( $( this.use_ssl_outgoing )                            , !enabled_use_ssl_outgoing                                                         , required_use_ssl_outgoing                       );
																																															                     
			ro_ctrl( $( this.use_thread )                                  , !enabled_use_thread                                                               , required_use_thread                             );
		}

		if (template && template.load) {
			template.load();
		}
	};

	MailAccountHnd.prototype.readonly = function()
	{
		var self = this;

		$( this.account_type_row ).hide();
		$( this.info_automatic_update ).hide();

		var ro_ctrl = function( $ctrl, readonly, required ) {
			self.parent.frm.readonly_control( $ctrl, readonly, required );
		};

		ro_ctrl( $( this.user_incoming )          , true /* disabled */ );
		ro_ctrl( $( this.user_outgoing )          , true /* disabled */ );

		ro_ctrl( $( this.pwd_incoming )           , true /* disabled */ );
		ro_ctrl( $( this.pwd_outgoing )           , true /* disabled */ );

		ro_ctrl( $( this.display_name )           , true /* disabled */ );

		ro_ctrl( $( this.url_outgoing )           , true /* disabled */ );
		ro_ctrl( $( this.url_incoming )           , true /* disabled */ );

		ro_ctrl( $( this.port_outgoing )          , true /* disabled */ );
		ro_ctrl( $( this.port_incoming )          , true /* disabled */ );

		ro_ctrl( $( this.protocol_pop3 )          , true /* disabled */ );
		ro_ctrl( $( this.protocol_imap )          , true /* disabled */ );
		ro_ctrl( $( this.protocol_office365_api_graph_incoming )          , true /* disabled */ );

		ro_ctrl( $( this.protocol_smtp_outgoing )          , true /* disabled */ );
		ro_ctrl( $( this.protocol_office365_api_graph_outgoing )          , true /* disabled */ );

		ro_ctrl( $( this.req_auth_incoming )      , true /* disabled */ );
		ro_ctrl( $( this.req_auth_outgoing )      , true /* disabled */ );

		ro_ctrl( $( this.req_encryp_incoming )    , true /* disabled */ );
		ro_ctrl( $( this.req_encryp_outgoing )    , true /* disabled */ );

		ro_ctrl( $( this.use_ssl_incoming )       , true /* disabled */ );
		ro_ctrl( $( this.use_ssl_outgoing )       , true /* disabled */ );

		ro_ctrl( $( this.spam_filter )            , true /* disabled */ );
		ro_ctrl( $( this.outgoing_name )          , true /* disabled */ );
	};

	MailAccountHnd.prototype.is_new = function ()
	{
		return this.current_account === null;
	};

	MailAccountHnd.prototype.get_account_name = function()
	{
		if ( !this.current_account ) {
			return "";
		}
		return this.current_account.name;
	};

	MailAccountHnd.prototype.edit = function()
	{
		var acc = this.current_account;

		var set_server = function( ctrl, url )
		{
			if ( typeof url !== "string" ) {
				return;
			}
			var match = url.match( new RegExp( "[^:]+://(.*)" ) );
			if ( match ) {
				$( ctrl ).val( match[1] );
			}
		};

		var set_port = function( ctrl_port, value ) {
			if ( !value || parseInt(value, 10) === 0 ) {
				value = "";
			}
			$(ctrl_port).val( value );
		};

		$( this.pwd_incoming ).off( "blur", "focus" ); // se le quita el evento cuando se edita
		$( this.pwd_outgoing ).off( "blur", "focus" ); // se le quita el evento cuando se edita

		var incomingConfig = acc.incomingConfig;
		var outgoingConfig = acc.outgoingConfig;

 // TODO: ATENCION ELLIO !!!!! ELIMINAME CUANDO ESTE TERMINADO EL ISSUE #4287  --------------------
this.advanced = incomingConfig.advanced;
//----------------------------------------------------------------

		var protocol_pop3;
		var protocol_imap = false;
		var protocol_office365_api_graph_incoming = false;
		var protocol_office365_api_graph_outgoing = false;
		// si tiene el atributo interno "__protocol" debe validar contra este, sino contra "type" que puede llegar a tener el valor "GMAIL"
		if ( acc.__protocol ) {
			protocol_pop3 = acc.__protocol === "POP3";
			protocol_imap = acc.__protocol === "IMAP";
			protocol_office365_api_graph_incoming = acc.__protocol === "OFFICE365_API_GRAPH";
			protocol_office365_api_graph_outgoing = acc.__protocol === "OFFICE365_API_GRAPH";
		}
		else {
			protocol_pop3 = acc.type === "POP3";
			protocol_office365_api_graph_outgoing = acc.__protocol === "OFFICE365_API_GRAPH";
		}

		// basic
		$( this.account_type ).val( acc.accountType );
		this.set_template_to_ctrls( acc.accountType ); // primero setea los valores por defecto, por si en el server no se ha guardado la info. Caso gmail que no guarda todos los atributos.

		var $account_name = $( this.account_name );
		$account_name.val( this.get_account_name() );
		this.parent.frm.readonly_control( $account_name, true /* disabled */ );

		$( this.user_incoming ).val( incomingConfig.user );
		$( this.user_outgoing ).val( outgoingConfig.user );

		$( this.pwd_incoming ).val( incomingConfig.password );
		$( this.pwd_outgoing ).val( outgoingConfig.password );

		$( this.display_name ).val( acc.defaultName );

		set_server( this.url_incoming , incomingConfig.url ); // server entrante
		set_server( this.url_outgoing , outgoingConfig.url ); // server saliente

		set_port( this.port_incoming, incomingConfig.port );
		set_port( this.port_outgoing, outgoingConfig.port );

		mark_chk( this.protocol_pop3  , protocol_pop3                                                  );
		mark_chk( this.protocol_imap  , protocol_imap                                                  );
		mark_chk( this.protocol_office365_api_graph_incoming  , protocol_office365_api_graph_incoming                    );
		
		mark_chk( this.protocol_smtp_outgoing  , !protocol_office365_api_graph_outgoing                    );
		mark_chk( this.protocol_office365_api_graph_outgoing  , protocol_office365_api_graph_outgoing                    );

		mark_chk( this.req_auth_incoming       , ( is_enabled_chk( incomingConfig.authentication   ) ) );
		mark_chk( this.req_auth_outgoing       , ( is_enabled_chk( outgoingConfig.authentication   ) ) );

		mark_chk( this.req_encryp_incoming     , ( is_enabled_chk( incomingConfig.encryption       ) ) );
		mark_chk( this.req_encryp_outgoing     , ( is_enabled_chk( outgoingConfig.encryption       ) ) );

		mark_chk( this.use_ssl_incoming        , ( is_enabled_chk( incomingConfig.useSSL           ) ) );
		mark_chk( this.use_ssl_outgoing        , ( is_enabled_chk( outgoingConfig.useSSL           ) ) );

		mark_chk( this.use_thread              , ( is_enabled_chk( acc.groupInThreads              ) ) );
		mark_chk( this.spam_filter             , ( is_enabled_chk( acc.enableSpamFilter            ) ) );

		$( this.outgoing_name ).val( acc.outgoingBCC || "" );

		return acc;
	};

	MailAccountHnd.prototype.empty_controls = function()
	{
 // TODO: ATENCION ELLIO !!!!! ELIMINAME CUANDO ESTE TERMINADO EL ISSUE #4287  --------------------
this.advanced = undefined;
//----------------------------------------------------------------
		var default_account_type = "others";
		$( this.account_type ).val( default_account_type ); // el por default
		this.set_template_to_ctrls( default_account_type );

		// basic
		var $account_name = $( this.account_name );
		$account_name.val( "" );
		this.parent.frm.readonly_control( $account_name, false /* enabled */, true /* required */ );

		$( this.user_incoming ).val("");
		$( this.user_outgoing ).val("");

		$( this.pwd_incoming ).val("");
		$( this.pwd_outgoing ).val("");

		$( this.display_name ).val("");

		$( this.url_outgoing ).val("");
		$( this.url_incoming ).val("");

		$( this.port_outgoing ).val("");
		$( this.port_incoming ).val("");

		$( this.outgoing_name ).val("");

		$(this.account_name).focus();
		
	};

	MailAccountHnd.prototype.set_events = function()
	{
		var self = this;

		$( this.inc_out_tab ).find( "[tab_type]" )
			.off( "click" )
			.on({
				click: function() {
					var selected_tab = $( this ).attr( "tab_type" );
					self.select_tab_panel( selected_tab );
				}
			});

		if ( this.parent.is_readonly() ) {
			return;
		}

		var $pwd_incoming = $( this.pwd_incoming );
		var $pwd_outgoing = $( this.pwd_outgoing );

		if ( this.is_new() ) {
			// cambia de tipos de accounts templates
			$( this.account_type )
				.off( "change" )
				.on({
					change: function() {
						var account_type = $( this ).val();
						self.set_template_to_ctrls(account_type);
						let template = self.get_template_accounts()[account_type];
						if (template.use_protocol_oauth) {
							$("#" + self.prefix_id + "oauth_client_id").val("");
							$("#" + self.prefix_id + "oauth_client_secret").val("");
						}
					}
				});

			// cambio de user en la solapa incoming, se actualiza el mismo campo en la solapa outgoing
			var $user_outgoing = $( this.user_outgoing );
			$( this.user_incoming )
				.off( "keyup" )
				.on({
					keyup: function() {
						var manual_changed = $user_outgoing.attr( "manual_changed" );
						if ( manual_changed === "1" ) {
							return;
						}
						$user_outgoing.val( $( this ).val() );
					}
				});
			$user_outgoing
				.off( "keyup" )
				.on({
					keyup: function() {
						$( this ).attr( "manual_changed", "1" );
					}
				});

			// cambio de pwd en la solapa incoming, se actualiza el mismo campo en la solapa outgoing
			$pwd_incoming
				.off( "keyup" )
				.on({
					keyup: function() {
						var manual_changed = $pwd_outgoing.attr( "manual_changed" );
						if ( manual_changed === "1" ) {
							return;
						}
						$pwd_outgoing.val( $( this ).val() );
					}
				});
			$pwd_outgoing
				.off( "keyup" )
				.on({
					keyup: function() {
						$( this ).attr( "manual_changed", "1" );
					}
				});
		}
		// los eventos 'blur' y 'focus' se les quita el evento si se esta editando.

		var pwd_events_fn = {
			blur: function() {
				var $this = $( this );
				var pwd_changed = $this.attr( "pwd_changed" );
				if ( pwd_changed !== "1" ) {
					$this.val( "xxxxxxxx" );
				}
			},
			focus: function() {
				var $this = $( this );
				var pwd_changed = $this.attr( "pwd_changed" );
				if ( pwd_changed !== "1" ) {
					$this.val( "" );
				}
			},
			keyup: function( event ) {
				var filtered_control_keys = inConcert.Utils.getControlCharacters();
				filtered_control_keys[ 17 ] = false; //CTRL is allowed for pasting
				if ( filtered_control_keys[ event.keyCode ] ) {
					return;
				}
				$( this ).attr( "pwd_changed", "1" );
			}
		};

		$pwd_incoming
			.off( "blur", "focus", "keyup" )
			.on( pwd_events_fn );

		$pwd_outgoing
			.off( "blur", "focus", "keyup" )
			.on( pwd_events_fn );

		$(this.protocol_pop3)
			.off("click")
			.on("click", function() {
				self.on_changed_inc_protocol($(this).val());
			});

		$(this.protocol_imap)
			.off("click")
			.on("click", function() {
				self.on_changed_inc_protocol($(this).val());
			});

		$(this.protocol_office365_api_graph_incoming)
			.off("click")
			.on("click", function() {
				self.on_changed_inc_protocol($(this).val());
			});

		$(this.protocol_smtp_outgoing)
			.off("click")
			.on("click", function() {
				self.on_changed_out_protocol($(this).val());
			});

		$(this.protocol_office365_api_graph_outgoing)
			.off("click")
			.on("click", function() {
				self.on_changed_out_protocol($(this).val());
			});
		
	};

	MailAccountHnd.prototype.on_changed_inc_protocol = function(protocol)
	{
		var account_type = $( this.account_type ).val();
		var template_accounts = this.get_template_accounts();
		var template = template_accounts[ account_type ];

		if ( !template || !template.suggested_values ) {
			return;
		}

		var sv = template.suggested_values;
		if (!sv || !sv.on_changed_inc_protocol) {
			return;
		}
		sv.on_changed_inc_protocol(protocol);
	};

	MailAccountHnd.prototype.on_changed_out_protocol = function(protocol)
	{
		var account_type = $( this.account_type ).val();
		var template_accounts = this.get_template_accounts();
		var template = template_accounts[ account_type ];

		if ( !template || !template.suggested_values ) {
			return;
		}

		var sv = template.suggested_values;
		if (!sv || !sv.on_changed_out_protocol) {
			return;
		}
		sv.on_changed_out_protocol(protocol);
	};

	MailAccountHnd.prototype.on_show = function()
	{
		// hide del template de tipos accounts y del info si estoy editando
		var $account_type_row = $( this.account_type_row );
		var $info_automatic_update = $( this.info_automatic_update );

		if ( this.is_new() ) {
			$account_type_row.show();
			$info_automatic_update.show();
			this.empty_controls();
		}
		else {
			$account_type_row.hide();
			$info_automatic_update.hide();
		}

		$( this.account_name ).focus();
	};

	MailAccountHnd.prototype.has_config_type = function( config_type )
	{
		return this.config_tabs === "all" || this.config_tabs === config_type;
	};

	MailAccountHnd.prototype.validate_field = function( $ctrl, validation_type, message, options, callback_err )
	{
		return this.parent.frm.validation.validate_field( $ctrl, validation_type, message, options, callback_err );
	};

	MailAccountHnd.prototype.hide_marked_errors = function ()
	{
		$(".formFieldError").hide();
	};

	MailAccountHnd.prototype.validate = function( owner_last_validation_fn, middle_last_validation_fn )
	{
		var self = this;

		var account_info = {
			success: false,
			account: null,
			is_new:  this.is_new(),
			error_message: null
		};

		var empty_success_fn = function() {return {success: true};};
		owner_last_validation_fn  = owner_last_validation_fn  || empty_success_fn;
		middle_last_validation_fn = middle_last_validation_fn || empty_success_fn;

		this.parent.frm.hide_messages(); // oculto todos los errores de los campos

		var error_in_basic_tab = function() {
			if ( self.owner.tab ) {
				self.owner.tabs.switch_tab( "basic" );
			}
			return account_info;
		};
		
	// 1 - valido el template
		var account_type = $( this.account_type ).val();
		var template = this.get_template_accounts()[ account_type ];
		if (template.validate && !template.validate()) {
			return error_in_basic_tab();
		}

	// 2 - valido la generic section ----------
	//
		// mail account - validation
		var $account_name = $( this.account_name );
		$account_name.val( $account_name.val().trim() );
		if ( !this.validate_field( $account_name, "required", "<%=_T('E-mail account is required.')%>" ) ) {
			return error_in_basic_tab();
		}
		if ( !this.validate_field( $account_name, "email", "<%=_T('E-mail account must be a valid e-mail address.')%>" ) ) {
			return error_in_basic_tab();
		}

	// 2.1 - Middle validaciones particulares sobre quien me ha instanciado. (distribution, priority, etc)
	//
		var owner_middle_result = middle_last_validation_fn( account_info, error_in_basic_tab );
		if ( !owner_middle_result.success ) {
			return error_in_basic_tab( owner_middle_result.error_message, owner_middle_result.$ctrl );
		}

	// 2.2 - continuo con las genericas - valido la generic section ----------
	//
		// display name - validation
		var $display_name = $( this.display_name );
		var validate_display_name = function () {
			return self.validate_field($display_name, "required", "<%=_T('Display name is required.')%>" );
		};

		// (*) Realizo la validacion del name
		if (this.m_validate_name_before_incomming_section) {
			if (!validate_display_name()) {
				return error_in_basic_tab();
			}
		}

		// outgoing bcc name - validation
		var $outgoing_name = $( this.outgoing_name );
		if ( $outgoing_name.length > 0 && $outgoing_name.val().trim().length > 0 ) {
			var mail_options = {
				msg_errors: {
					invalid_mail     : "<%=_T('Outgoing BCC must be a valid email address: {0}.')%>",
					invalid_mail_list: "<%=_T('Outgoing BCC must have a valid list of e-mails addresses. Must use the separator: ;. Example: sample@sample.com; ....')%>"
				}
			};
			if (!this.validate_field($outgoing_name, "email_list", undefined, mail_options)) {
				return error_in_basic_tab();
			}
		}
	// 3 - valido la incoming section ----------
	//
		var callback;
		var $user_incoming;
		var $pwd_incoming;
		var $url_incoming;
		var $port_incoming;
		if ( this.has_config_type( "incoming" ) ) {
			// user incoming - validation
			callback = function() { self.select_tab_panel( "incoming" ); };
			$user_incoming = $( this.user_incoming );
			if ( !this.validate_field( $user_incoming, "required", "<%=_T('Incoming user is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}
			if ( !this.validate_field( $user_incoming, "alpha", 
					"<%=_T('Only letters numbers - and . are allowed in the incoming user name.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// incoming pwd - validation
			$pwd_incoming = $( this.pwd_incoming );
			if ( !this.validate_field( $pwd_incoming, "required", "<%=_T('Incoming password is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// url incoming srv - validation
			$url_incoming = $( this.url_incoming );
			$url_incoming.val( $url_incoming.val().trim() );
			if ( !this.validate_field( $url_incoming, "required", "<%=_T('Incoming server is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}
			if ( !this.validate_field( $url_incoming, "domain", "<%=_T('Incoming server must be a valid domain name.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// port incoming - validation
			$port_incoming = $( this.port_incoming );
			if ( $port_incoming.val().trim().length > 0 ) {
				if ( !this.validate_field( $port_incoming, "digits", "<%=_T('Incoming port is not a number.')%>", undefined, callback ) ) {
					return error_in_basic_tab();
				}
			}
		}

	// 4 - valido la outgoing section ----------
	//
		var $user_outgoing;
		var $pwd_outgoing;
		var $url_outgoing;
		var $port_outgoing;
		if ( this.has_config_type( "outgoing" ) ) {
			callback = function() { self.select_tab_panel( "outgoing" ); };
			// user outgoing - validation
			$user_outgoing = $( this.user_outgoing );
			if ( !this.validate_field( $user_outgoing, "required", "<%=_T('Outgoing user is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}
			if ( !this.validate_field( $user_outgoing, "alpha",
					"<%=_T('Only letters numbers - and . are allowed in the outgoing user name.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// outgoing pwd - validation
			$pwd_outgoing = $( this.pwd_outgoing );
			if ( !this.validate_field( $pwd_outgoing, "required", "<%=_T('Outgoing password is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// url outgoing smtp - validation
			$url_outgoing = $( this.url_outgoing );
			$url_outgoing.val( $url_outgoing.val().trim() );
			if ( !this.validate_field( $url_outgoing, "required", "<%=_T('Outgoing server is required.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}
			if ( !this.validate_field( $url_outgoing, "domain", "<%=_T('Outgoing server must be a valid domain name.')%>", undefined, callback ) ) {
				return error_in_basic_tab();
			}

			// port outgoing - validation
			$port_outgoing = $( this.port_outgoing );
			if (  $port_outgoing.val().trim().length > 0 ) {
				if ( !this.validate_field( $port_outgoing, "digits", "<%=_T('Outgoing port is not a number.')%>", undefined, callback ) ) {
					return error_in_basic_tab();
				}
			}
		}

	// 4.1 - Si realizo la validacion del name si es que no la realizo antes (*).
		if (!this.m_validate_name_before_incomming_section) {
			if (!validate_display_name()) {
				return error_in_basic_tab();
			}
		}

	// 5 - Ultimas validaciones particulares sobre quien me ha instanciado. (survey, contact mng, etc)
		var owner_last_result = owner_last_validation_fn( account_info, error_in_basic_tab );
		if ( !owner_last_result.success ) {
			return error_in_basic_tab( owner_last_result.error_message, owner_last_result.$ctrl );
		}

	//------------------------------------------------------
	// paso la validacion correctamente, ahora a guardar!!!

		var protocol_pop3 = $( this.protocol_pop3 ).prop("checked");
		var protocol_imap = $( this.protocol_imap ).prop("checked");
		var protocol_office365_api_graph_incoming = $( this.protocol_office365_api_graph_incoming ).prop("checked");
		var protocol_office365_api_graph_outgoing = $( this.protocol_office365_api_graph_outgoing ).prop("checked");
		var protocol_office365_api_graph = self.config_tabs === "all" ? protocol_office365_api_graph_incoming : protocol_office365_api_graph_outgoing;
		var type = protocol_pop3 ? "POP3" : (protocol_imap ? "IMAP" : (protocol_office365_api_graph ? "OFFICE365_API_GRAPH" : ""));

		var get_port = template.get_port ||
			function( $port ) {
				var port = $port.val().trim();
				if ( port.length === 0 ) {
					return undefined;
				}
				return parseInt( port, 10 );
			};

		var account_name = $account_name.val();
		account_info.success = true;
		account_info.account = {
			name                   : account_name,
			uid                    : account_name,
			accountType            : account_type,
			proxyType              : "MAIL",
			defaultName            : $display_name.val(),
			type                   : type, // (POP3, IMAP, GMAIL) account.type es el atributo que puede cambiar a "GMAIL" en los default de GMAIL
			__protocol             : type, // (POP3, IMAP       ) account.__protocol es el atributo interno con valor posta del protocolo utilizado
			enableSpamFilter       : get_chk_bool_to_string( this.spam_filter ),
			groupInThreads         : get_chk_bool_to_string( this.use_thread ),
			incomingConfig : {} ,
			outgoingConfig : {},
			ActionOnAcdWaitTimeout : owner_last_result.ActionOnAcdWaitTimeout,
			InactivityTimeout : owner_last_result.InactivityTimeout
		};

		if ( $outgoing_name.length > 0 ) {
			account_info.account.outgoingBCC = $outgoing_name.val();
		}

		if ( this.has_config_type( "incoming" ) ) {
			var incoming_protocol;
			if ( $( this.use_ssl_incoming ).prop("checked") ) {
				incoming_protocol = protocol_pop3 ? "pop3s" : (protocol_imap ? "imaps" : (protocol_office365_api_graph_incoming ? "https" : "")) ;
			}
			else {
				incoming_protocol = protocol_pop3 ? "pop3" : (protocol_imap ? "imap" : (protocol_office365_api_graph_incoming ? "http" : ""));
			}
			account_info.account.incomingConfig = {
				user             : $user_incoming.val(),
				password         : $pwd_incoming.val(),
				url              : incoming_protocol + "://" + $url_incoming.val(),
				port             : get_port( $port_incoming ),
				authentication   : get_chk_bool_to_string( this.req_auth_incoming ),
				encryption       : get_chk_bool_to_string( this.req_encryp_incoming ),
				useSSL           : get_chk_bool_to_string( this.use_ssl_incoming )
			};
		}

		if ( this.has_config_type( "outgoing" ) ) {
			var outgoing_protocol = $( this.use_ssl_outgoing ).prop("checked") ? (protocol_office365_api_graph ? "https" : "smtps") : (protocol_office365_api_graph ? "http" : "smtp");

			account_info.account.outgoingConfig = {
				user             : $user_outgoing.val(),
				password         : $pwd_outgoing.val(),
				url              : outgoing_protocol + "://" + $url_outgoing.val(),
				port             : get_port( $port_outgoing ),
				authentication   : get_chk_bool_to_string( this.req_auth_outgoing ),
				encryption       : get_chk_bool_to_string( this.req_encryp_outgoing ),
				useSSL           : get_chk_bool_to_string( this.use_ssl_outgoing )
			};
		}
		
		if (template.set_data) {
			template.set_data(account_info.account);
		}

		if (account_info.account.oauth && template.use_xoauth_two_line) {
			account_info.account.oauth.xoauth_two_line = account_info.account.__protocol === "POP3" ? "1" : "";
		}

 // TODO: ATENCION ELLIO !!!!! ELIMINAME CUANDO ESTE TERMINADO EL ISSUE #4287  --------------------
if ( this.advanced ) {
	account_info.account.incomingConfig.advanced = this.advanced;
}
//----------------------------------------------------------------

		// normalizes the id to the form MAIL_someone@example.com
		account_info.account.id = inConcert.Utils.normalize_account_id( account_info.account, account_info.account.name );

		// the account address is taken from the 'name' field (someone@example.com)
		account_info.account.address = account_info.account.name;

		if ( template && template.set_default_values ) {
			template.set_default_values( account_info.account );
		}

		return account_info;
	};
	
	/* Mail OAUTH */
	
	MailAccountHnd.prototype.__loadMailData = function()
	{
		const self = this;
		this.oauth_client_id = "#"+ this.prefix_id + "oauth_client_id";
		this.oauth_client_secret = "#"+ this.prefix_id + "oauth_client_secret";
		this.oauth_authorize_id = "#"+ this.prefix_id + "oauth_account_auth";
		this.oauth_authorize_img_id = "#"+ this.prefix_id + "oauth_account_auth_img";
		this.oauth_authorize_container_id = "#"+ this.prefix_id + "oauth_account_auth_container";
		//readonly
		if (this.parent.is_readonly() ) {
			$(this.oauth_authorize_id).off("click");
			$(this.oauth_authorize_container_id).hide();
			return;
		}

		//current account oauth
		if (!this.m_mailOauth && !this.is_new() && (this.current_account.type === "GMAIL" || this.current_account.type === "OFFICE365")) {
			this.m_mailOauth = this.current_account.oauth;
			this.m_mailUid = this.current_account.uid;
		}

		//fields
		if (this.m_mailUPN) {
			$(this.user_outgoing).val(this.m_mailUPN);
		}
		else{
			$(this.user_outgoing).val(this.m_mailUid || "");
		}

		$(this.account_name + ", " + this.user_incoming).val(this.m_mailUid || "");
		$(this.oauth_client_id).val(this.m_mailOauth && this.m_mailOauth.clientId || "");
		$(this.oauth_client_secret).val(this.m_mailOauth && this.m_mailOauth.clientSecret || "");
		//ui
		$(this.oauth_authorize_id).val(!this.m_mailOauth? "<%=_T('Authorize Application')%>" : 
			"<%=_T('Re-Authorize Application')%>");
		$(this.oauth_authorize_img_id)
			.toggleClass("formImageTrue", !!this.m_mailOauth)
			.toggleClass("formImageFalse", !this.m_mailOauth);
		//event
		$(this.oauth_authorize_id)
			.off("click")
			.on("click", function() {
				self.__authorizeMail();
			});
	};
	
	MailAccountHnd.prototype.__authorizeMail = function()
	{
		const self = this;
		if ( !this.parent.has_permission_to_save() ) {
			this.parent.frm.show_status_message("<%=_T('You do not have permissions for save account.')%>", "Error", 5000 );
			return;
		}
		this.hide_marked_errors();
		let $clientId = $(this.oauth_client_id);
		let $clientSecret = $(this.oauth_client_secret);

		let validate_oauth_credentials = $(this.account_type).val() === "office365";
		if (!validate_oauth_credentials && !this.validate_field($clientId, "required", "<%=_T('Client id is required.')%>")) {
			return;
		}
		if (!validate_oauth_credentials && !this.validate_field($clientSecret, "required", "<%=_T('Client secret is required.')%>")) {
			return;
		}
		
		let postback_data = JSON.stringify({
			origin: window.location.origin,
			host : window.location.host
		});
		let parameters = '?&post_data=' + encodeURIComponent(postback_data) +
			"&client_id=" + encodeURIComponent($clientId.val()) +
			"&client_secret=" + encodeURIComponent($clientSecret.val());
			
		
		let href = inConcert.app.getBaseDomain() + "/inconcert/apps/oauth/"+ $(this.account_type).val() +"/authorize_user/" + parameters;
		let window_options = "height=600,width=800,status=yes,toolbar=yes,menubar=no,location=yes";
		let authorization_window = window.open(href, "_blank", window_options);
		
		if (window.MailAccountHnd_message_listener) {
			window.removeEventListener("message", window.MailAccountHnd_message_listener);
			window.MailAccountHnd_message_listener = undefined;
		}
		
		let close_popup_window = function( can_add ) {
			let timeout_ms = can_add ? 1500 : undefined;
			window.setTimeout( function() {
				authorization_window.close();
				authorization_window = undefined;
			}, timeout_ms );
		};

		let listener = function(ev) {
			if (ev.origin.search( window.location.host ) > 0 && ev.data && (ev.data.authorization === "GMAIL" || ev.data.authorization === "OFFICE365")) {
				self.__onMailAuthorized(ev.data, close_popup_window);
				window.removeEventListener("message", window.MailAccountHnd_message_listener);
				window.MailAccountHnd_message_listener = undefined;
			}
		};

		window.MailAccountHnd_message_listener = listener;
		window.addEventListener("message", window.MailAccountHnd_message_listener, false);
	};

	MailAccountHnd.prototype.__onMailAuthorized = function(data, callback = $.noop) 
	{
		if (this.current_account && this.current_account.uid !== data.uid) {
			this.parent.frm.show_status_message(format("<%=_T('Only can be reauthorized using same {0} account.')%>", this.current_account.type === "GMAIL" ? "google" : $( this.account_type ).val() === "office365" ? "microsoft": "" ), "Error", 5000 );
			return callback(false);
		}

		this.m_mailUPN         = (data.authorization === "OFFICE365") ? data.name : null;
		this.m_mailUid         = data.uid;
		this.m_mailOauth       = {
									clientId     : data.client_id,
									clientSecret : data.client_secret,
									refreshToken : data.refresh_token,
									oauthTenant  : data.tenant
								 };

		this.__loadMailData();
		callback(true);
	};

	MailAccountHnd.prototype.__validateMail = function()
	{
		if (!this.m_mailOauth || this.__authorizationRequired()) {
			this.parent.frm.show_status_message(format("<%=_T('Must authorize account in {0}.')%>", $( this.account_type ).val() === "gmail" ? "google" : $( this.account_type ).val() === "office365" ? "microsoft": ""), "Error", 5000 );
			return false;
		}
		return true;
	};

	MailAccountHnd.prototype.__setMailData = function(account)
	{
		account.oauth = this.m_mailOauth;
	};

	MailAccountHnd.prototype.__authorizationRequired = function()
	{
		var protocol_pop3 = $( this.protocol_pop3 ).prop("checked");
		var protocol_imap = $( this.protocol_imap ).prop("checked");
		var protocol_office365_api_graph_incoming = $( this.protocol_office365_api_graph_incoming ).prop("checked");
		var protocol_office365_api_graph_outgoing = $( this.protocol_office365_api_graph_outgoing ).prop("checked");
		var protocol_office365_api_graph = this.config_tabs === "all" ? protocol_office365_api_graph_incoming : protocol_office365_api_graph_outgoing;
		var type = protocol_pop3 ? "POP3" : (protocol_imap ? "IMAP" : (protocol_office365_api_graph ? "OFFICE365_API_GRAPH" : ""));
		
		return this.current_account && this.current_account.__protocol !== type && this.m_mailOauth === this.current_account.oauth;
	};
}());
