/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	let format = inConcert.Utils.format;
	
	let available_events = [
		{
			group : "agent",
			name : "<%=_T('Select Agent Notifications to be fired')%>",
			description : "<%=_T('Mark agent notifications to be fired.')%>",
			campaignScope : true,
			events : [
				{
					id : "LoggedIn",
					name : "<%=_T('Logged In')%>",
					description : "<%=_T('When an user logged in.')%>"
				},
				{
					id : "LoggedOut",
					name : "<%=_T('Logged Out')%>",
					description : "<%=_T('When an user logged out.')%>"
				},
				{
					id : "ChangedState",
					name : "<%=_T('Changes State')%>",
					description : "<%=_T('When an user changes his state.')%>"
				},
				{
					id : "AssignedToCampaign",
					name : "<%=_T('Assigned to Campaign')%>",
					description : "<%=_T('When an user is assigned to a campaign.')%>"
				},
				{
					id : "RemovedFromCampaign",
					name : "<%=_T('Removed from Campaign')%>",
					description : "<%=_T('When an user is removed from a campaign.')%>"
				}
			]
		},
		{
			group : "interaction",
			name : "<%=_T('Select Interactions Notifications to be fired')%>",
			description : "<%=_T('Mark interaction notifications to be fired.')%>",
			campaignScope : true,
			events : [
				{
					id : "NewIncoming",
					name : "<%=_T('New incoming')%>",
					description : "<%=_T('When a new incoming interaction is created.')%>"
				},
				{
					id : "NewOutgoing",
					name : "<%=_T('New outgoing')%>",
					description : "<%=_T('When a new outgoing interaction is created.')%>"
				},
				{
					id : "NewIncomingMessage",
					name : "<%=_T('New Incoming Message')%>",
					description : "<%=_T('When a new incoming message is received.')%>"
				},
				{
					id : "NewOutgoingMessage",
					name : "<%=_T('New Outgoing Message')%>",
					description : "<%=_T('When a new outgoing message is sent.')%>"
				},
				{
					id : "FirstAgentMessage",
					name : "<%=_T('First Agent Message')%>",
					description : "<%=_T('When first agent message is sent.')%>"
				},
				{
					id : "Taken",
					name : "<%=_T('Interaction Taken')%>",
					description : "<%=_T('When an interaction is taken.')%>"
				},
				{
					id : "Queued",
					name : "<%=_T('Interaction Queued')%>",
					description : "<%=_T('When an interaction is put on the queue.')%>"
				},
				{
					id : "Transferred",
					name : "<%=_T('Interaction Transferred')%>",
					description : "<%=_T('When an interaction is transferred.')%>"
				},
				{
					id : "Removed",
					name : "<%=_T('Interaction Removed')%>",
					description : "<%=_T('When an interaction is removed to an agent. Example: exceeds answer time.')%>"
				},
				{
					id : "Locked",
					name : "<%=_T('Interaction Locked')%>",
					description : "<%=_T('When an interaction is assigned to an agent.')%>"
				},
				{
					id : "Wrapup",
					name : "<%=_T('Change State to Wrapup')%>",
					description : "<%=_T('When an interaction enters in wrapup.')%>"
				},
				{
					id : "Closed",
					name : "<%=_T('Change State to Closed')%>",
					description : "<%=_T('When an interaction is closed.')%>"
				},
				{
					id : "Canceled",
					name : "<%=_T('Change State to Canceled')%>",
					description : "<%=_T('When an interaction is canceled. Example: out of schedule.')%>"
				},
				{
					id : "Reopen",
					name : "<%=_T('Change State to Reopen')%>",
					description : "<%=_T('When an interaction is reopen.')%>"
				},
				{
					id : "Waiting",
					name : "<%=_T('Change State to Waiting')%>",
					description : "<%=_T('When an interaction enters in waiting.')%>"
				}
			]
		},
		{
			group : "outgoingbatches",
			name : "<%=_T('Select Outgoing Batches to be fired')%>",
			description : "<%=_T('Mark outgoing batches notifications to be fired.')%>",
			campaignScope : true,
			events : [
				{
					id : "NewBatch",
					name : "<%=_T('New batch')%>",
					description : "<%=_T('When a new batch is created.')%>"
				},
				{
					id : "NewBatchMessagesStatus",
					name : "<%=_T('New messages status')%>",
					description : "<%=_T('When batch messages status are updated.')%>"
				}
			]
		},
		{
			group : "configuration",
			name : "<%=_T('Select Configuration Notifications to be fired')%>",
			description : "<%=_T('Mark configuration notifications to be fired.')%>",
			campaignScope : false,
			events : [
				{
					id : "AdminUsers",
					name : "<%=_T('Users')%>",
					description : "<%=_T('When an user is created, updated or disabled.')%>",
					events : [
						"UserCreated",
						"UserUpdated",
						"UserDisabled",
						"UserEnabled"
					]
				},
				{
					id : "AdminCampaigns",
					name : "<%=_T('Campaigns')%>",
					description : "<%=_T('When a campaign is created, updated or deleted.')%>",
					events : [
						"CampaignCreated",
						"CampaignUpdated",
						"CampaignDeleted",
						"CampaignSchedulerUpdated",
						"CampaignIntegrationUpdated",
						"CampaignDispositionsUpdated"
					]
				},
				{
					id : "AdminAccounts",
					name : "<%=_T('Accounts')%>",
					description : "<%=_T('When an account is created, updated or deleted.')%>",
					events : [
						"AccountCreated",
						"AccountUpdated",
						"AccountEnabled",
						"AccountDisabled",
						"AccountDeleted"
					]
				},
				{
					id : "AdminAttentionLevels",
					name : "<%=_T('Att Levels')%>",
					description : "<%=_T('When an attention level is created, updated or deleted.')%>",
					events : [
						"AttLevelCreated",
						"AttLevelUpdated",
						"AttLevelDeleted"
					]
				},
				{
					id : "AdminCapabilities",
					name : "<%=_T('Capabilities')%>",
					description : "<%=_T('When attention capabilities are modified (at global, campaign or campaign/user level).')%>",
					events : [
						"VccCapabilitiesUpdated",
						"CampaignCapabilitiesUpdated",
						"CampaignUserCapabilitiesUpdated"
					]
				},
				{
					id : "AdminRoles",
					name : "<%=_T('Roles and Permissions')%>",
					description : "<%=_T('When roles and permissions are modified (at global, campaign, user or campaign/user level).')%>",
					events : [
						"RoleCreated",
						"RoleUpdated",
						"RoleDeleted",
						"SystemRoleOverrided",
						"SystemRoleRestored",
						"CampaignRolesUpdated"
					]
				},
				{
					id : "AdminStates",
					name : "<%=_T('States')%>",
					description : "<%=_T('When a state is created, updated or deleted.')%>",
					events : [
						"StateCreated",
						"StateUpdated",
						"StateDeleted"
					]
				},
				{
					id : "AdminSkills",
					name : "<%=_T('Skills')%>",
					description : "<%=_T('When a skill is created, updated or deleted.')%>",
					events : [
						"SkillCreated",
						"SkillUpdated",
						"SkillDeleted"
					]
				}
			]
		}
	];
	
	if (!inConcert.available_apps_notifications) {
		inConcert.available_apps_notifications = available_events;
		inConcert.available_apps_notifications_d = available_events.reduce(function(events_d, eventsGroup) {
			return eventsGroup.events.reduce(function(events_d, event) {
				event.group = eventsGroup;
				events_d[event.id] = event;
				return events_d;
			}, events_d);
		}, {});
	}

	//---------------------------------------------------------------------
	// form api users class
	function adm_form_apps_notifications(params)
	{
		/* jshint validthis: true*/
		const success = adm_form_apps_notifications.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Link app to notify')%>";
		this.update_title = "<%=_T('Update App {0} Notifications')%>";
		this.view_title = "<%=_T('View App {0} Notifications')%>";
		this.tagline = "<%=_T('Tagline form App Notifications!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/apps_notifications/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/apps_notifications/save/";
		this.get_uri        = "/inconcert/apps/administration/apps_notifications/get/";

		return true;
	}

	inConcert.Utils.inherit(adm_form_apps_notifications, inConcert.conf_form);

	if (typeof inConcert.adm_form_apps_notifications !== "function") {
		inConcert.adm_form_apps_notifications = adm_form_apps_notifications;
	}
	
	adm_form_apps_notifications.prototype.__loadEvent = function(event) {
		let description = event.description + (event.events?
			("  <%=_T('Events')%><%=_T(':')%> " + inConcert.Utils.arrayToText(event.events)) :
			("  <%=_T('Event')%><%=_T(':')%> " + event.id)
		);
		return $(format("".concat(
			"<div class='col-3'>",
				"<div class='form-group mb-0'>",
					"<div class='form-check'>",
						"<label for='adm_form_apps_notifications_{0}' class='form-check-label form-check-label-title'>",
							"<input notification_type id='adm_form_apps_notifications_{0}' class='form-check-input' type='checkbox' />",
							"{1} <i class='fa fa-question-circle' title='{2}'></i>",
						"</label>",
					"</div>",
				"</div>",
			"</div>"
		), event.id, event.name, description));
	};
	
	adm_form_apps_notifications.prototype.__loadEventsGroup = function(eventsGroup) {
		let self = this;
		let $eventGroup = $(format("".concat(
			"<div class='appsNotificationsCategory card' id='adm_form_apps_notifications_{0}_events'>",
				"<div class='card-header'>", 
					"<h5>{1}</h5>", 
				"</div>", 
				"<div class='card-block'>",
					"<div class='row adm_form_apps_notifications_checks'>", 
					"</div>",
				"</div>",
			"</div>",
		), eventsGroup.group, eventsGroup.name, eventsGroup.description));
		$eventGroup.find(".adm_form_apps_notifications_checks").append(eventsGroup.events.reduce(function($events, event) {
			$events.push(self.__loadEvent(event));
			return $events;
		}, []));
		return $eventGroup;
	};

	adm_form_apps_notifications.prototype.load_data = function(callback_complete_requests)
	{
		let self = this;
		$(".appsNotificationsCategory").remove();
		$("#conf_form_data_container").append(available_events.reduce(function($content, eventsGroup) {
			$content.push(self.__loadEventsGroup(eventsGroup));
			return $content;
		}, []));

		adm_form_apps_notifications.uber.load_data.call(this, callback_complete_requests);
	};

	adm_form_apps_notifications.prototype.get = function()
	{
		let app = {
			Id: $("#adm_form_apps_notifications_id").val(),
			Description: $("#adm_form_apps_notifications_description").val(),
			Url: $("#adm_form_apps_notifications_url").val(),
			AllCampaigns: $("#adm_form_apps_notifications_all_campaigns").is(":checked")? "1" : "0",
			Events: {}
		};
		$(".appsNotificationsCategory")
			.find("[notification_type]")
			.each(function() {
				if ($(this).prop("checked")) {
					app.Events[this.id.replace("adm_form_apps_notifications_", "")] = true;
				} 

			});
		if (!this.is_new) {
			let action = this.dashboard.get_current_action_obj();
			let id = action.view.row_id;
			app[id] = this.$current_row.attr("row_id");
		}
		return app;
	};

	adm_form_apps_notifications.prototype.set = function(success, app)
	{
		if (!success) {
			return;
		}
		$("#adm_form_apps_notifications_id").val(app.Id);
		$("#adm_form_apps_notifications_description").val(app.Description);
		$("#adm_form_apps_notifications_url").val(app.Url);
		$("#adm_form_apps_notifications_all_campaigns").prop("checked", app.AllCampaigns === "1");
		$.each(app.Events || {}, function(event) {
			$("#adm_form_apps_notifications_" + event).prop("checked", true);
		});
	};
	
	adm_form_apps_notifications.prototype.can_save_entity = function(app) 
	{
		return !app.System;
	};

	adm_form_apps_notifications.prototype.empty_controls = function()
	{
		$("#adm_form_apps_notifications_id").val("").focus();
		$("#adm_form_apps_notifications_description").val("");
		$("#adm_form_apps_notifications_url").val("");
		$("#adm_form_apps_notifications_all_campaigns").prop("checked", false);
		$(".appsNotificationsCategory")
			.find("[notification_type]")
			.prop("checked", false);
	};

	adm_form_apps_notifications.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;
		let message = null;

		if ( typeof err === "object" ) {
			let error_type = err.result.toUpperCase();
			if (error_type === "ID_DOES_NOT_EXIST" || error_type === "ID_ALREADY_EXISTS") {
				$("#adm_form_apps_notifications_id").focus();
				if (error_type === "ID_ALREADY_EXISTS") {
					message = "<%=_T('Already exists another app with same id')%>";
				}
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				$("#adm_form_apps_notifications_description").focus();
			}
			else if (error_type === "MUST_HAVE_A_URL") {
				$("#adm_form_apps_notifications_url").focus();
			}
			if (message) {
				is_custom_message = true;
				err = message;
			}
		}

		adm_form_apps_notifications.uber.err_save_messages.call( this, err, is_custom_message ); //invokes the base class method
	};

	adm_form_apps_notifications.prototype.get_validation_def = function()
	{
		let definitions = [
			{
				control_id: "adm_form_apps_notifications_id",
				messages: {
					required: "<%=_T('Please enter the app id.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the app id.')%>"
				}
			},
			{
				control_id: "adm_form_apps_notifications_description",
				messages: {
					required: "<%=_T('Please enter the app description.')%>"
				}
			},
			{
				control_id: "adm_form_apps_notifications_url",
				messages: {
					required: "<%=_T('Please enter the app notifications url.')%>",
					url: "<%=_T('Url must be valid.')%>"
				}
			}
		];
		return definitions;
	};

	adm_form_apps_notifications.prototype.validate = function( save_callback, validation_def )
	{
		save_callback = save_callback || $.noop;
		
		if (!$("[notification_type]:checked").length) {
			this.validation.show_validation_error(null, 
				"<%=_T('Must select almost one event.')%>");
			return save_callback(false);
		}
		
		if ($("#adm_form_apps_notifications_all_campaigns").is(":checked") && !available_events.find(function(eventGroup) {
			return eventGroup.campaignScope && 
				$(format("#adm_form_apps_notifications_{0}_events", eventGroup.group))
					.find("[notification_type]:checked").length? true : false;
		})) {
			this.validation.show_validation_error(null, 
				"<%=_T('Must select almost one agent or interaction event if apply to all campaigns.')%>");
			return save_callback(false);
		}

		adm_form_apps_notifications.uber.validate.call(this, save_callback, validation_def);
	};

	adm_form_apps_notifications.prototype.ready = function()
	{
		adm_form_apps_notifications.uber.ready.call( this );  //invokes the base class method

		var $id = $("#adm_form_apps_notifications_id");
		if (!this.is_new) {
			this.readonly_control($id, true);
		}
		$id.focus();
	};
	
}());
