/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if (!this.inConcert.Administration) {
	this.inConcert.Administration = {};
}

if (!this.inConcert.Administration.Campaigns) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form campaigns class - accounts section
	function adm_form_campaigns_section_accounts( params )
	{
		/* jshint validthis: true*/
		const self = this;
		var success = adm_form_campaigns_section_accounts.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.view_action_id       = "view_campaigns_accounts";      // view action id de permisos
		this.save_action_id       = "save_campaign_accounts";       // save action id de permisos
		this.erase_action_id      = "delete_campaign_accounts";     // delete action id de permisos
		this.deactivate_action_id = "deactivate_campaign_accounts"; // deactivate action id de permisos
		this.current_panel        = null;

		var is_enable_menu_option = function( int_type, callback ) {
			self.is_enable_menu_option( int_type, callback );
		};

		this.has_allegro_integration = inConcert.app.has_allegro_integration();
		// instancia de la clase account dependiendo de si tiene o no integracion con allegro
		var call_account_instance = this.has_allegro_integration ?
			inConcert.Administration.Campaigns.CallAllegroAccountPanel :
			inConcert.Administration.Campaigns.CallAccountPanel;

		this.accounts_panels = {
			"CALL": {
				instance           : call_account_instance,
				frm                : null                                           ,
				name               : "<%=_T('Call')%>"                                     ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("CALL");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "CALL", callback ); } ,
				class_name         : "accountTypeCall"
			} ,
			"WEBCONTACT": {
				instance           : inConcert.Administration.Campaigns.ContactFormAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('Contact Form')%>"                             ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("WEBCONTACT");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "WEBCONTACT", callback ); } ,
				class_name         : "accountTypeContactForm"
			} ,
			"CHAT": {
				instance           : inConcert.Administration.Campaigns.WebChatAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('Web Chat')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("CHAT");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "CHAT", callback ); } ,
				class_name         : "accountTypeWebchat"
			} ,
			"FACEBOOK":{
				instance           : inConcert.Administration.Campaigns.FacebookAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Facebook')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("FACEBOOK");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "FACEBOOK", callback ); } ,
				class_name         : "accountTypeFacebook"
			} ,
			"INSTAGRAM":{
				instance           : inConcert.Administration.Campaigns.InstagramAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Instagram')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("INSTAGRAM");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "INSTAGRAM", callback ); } ,
				class_name         : "accountTypeInstagram"
			} ,
			"YOUTUBE":{
				instance           : inConcert.Administration.Campaigns.YoutubeAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Youtube')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("YOUTUBE");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "YOUTUBE", callback ); } ,
				class_name         : "accountTypeYoutube"
			} ,
			"IM": {
				instance           : inConcert.Administration.Campaigns.IMAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('Chat')%>"                                     ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("IM");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "IM", callback ); } ,
				class_name         : "accountTypeIM"
			} ,
			"MAIL": {
				instance           : inConcert.Administration.Campaigns.MailAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('E-mail')%>"                                     ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("MAIL");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "MAIL", callback ); } ,
				class_name         : "accountTypeMail"
			} ,
			"SMS": {
				instance           : inConcert.Administration.Campaigns.SMSAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('SMS')%>"                                      ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("SMS");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "SMS", callback ); } ,
				class_name         : "accountTypeSMS"
			} ,
			"RCS": {
				instance           : inConcert.Administration.Campaigns.RcsAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('RCS')%>"                                      ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("RCS");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "RCS", callback ); } ,
				class_name         : "accountTypeRCS"
			} ,
			"WHATSAPP": {
				instance           : inConcert.Administration.Campaigns.WHATSAPPAccountPanel ,
				frm                : null                                           ,
				name               : "<%=_T('Whatsapp')%>"                                      ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("WHATSAPP");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "WHATSAPP", callback ); } ,
				class_name         : "accountTypeWHATSAPP"
			} ,
			"TWITTER": {
				instance           : inConcert.Administration.Campaigns.TwitterAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Twitter')%>"                                  ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("TWITTER");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "TWITTER", callback ); } ,
				class_name         : "accountTypeTwitter"
			} ,
			"APPSTORE":{
				instance           : inConcert.Administration.Campaigns.AppstoreAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Appstore')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("APPSTORE");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "APPSTORE", callback ); } ,
				class_name         : "accountTypeAppstore"
			} ,
			"GOOGLEPLAY":{
				instance           : inConcert.Administration.Campaigns.GoogleplayAccountPanel,
				frm                : null                                           ,
				name               : "<%=_T('Googleplay')%>"                                 ,
				is_visible_in_menu : function() { return inConcert.app.hasLicensesForChannel("GOOGLEPLAY");  } ,
				is_enabled_in_menu : function( callback ) { is_enable_menu_option( "GOOGLEPLAY", callback ); } ,
				class_name         : "accountTypeGoogleplay"
			} 
		};

		$.each( this.accounts_panels, function( _, account ) {
			account.frm = new account.instance( {section: self} );
		});
		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_accounts, inConcert.conf_form_section );

	if (typeof inConcert.adm_form_campaigns_section_accounts !== "function") {
		inConcert.adm_form_campaigns_section_accounts = adm_form_campaigns_section_accounts;
	}

	adm_form_campaigns_section_accounts.prototype.dispose = function()
	{
		this.messages_cleanup( this.current_panel, true );
		if ( this.accounts_popup_instance ) {
			if ( this.accounts_popup_instance.isVisible() ) {
				this.accounts_popup_instance.hide();
			}
			this.accounts_popup_instance = null;
		}
		this.current_panel = null;
	};

	adm_form_campaigns_section_accounts.prototype.is_enable_menu_option = function( int_type, callback )
	{
		var account_panel = this.accounts_panels[ int_type ];
		if ( !account_panel ) {
			callback( true );
		}
		else {
			account_panel.frm.is_enable_menu_option( callback );
		}
	};

	adm_form_campaigns_section_accounts.prototype.get_account_button_info = function(account_enabled)
	{
		var account_button_info = {};
		// var html_img = "<i class='buttons-icons-off'></i>";
		if (account_enabled === "1" || account_enabled) {
			account_button_info.html = "<i class='buttons-icons-deactivate'></i><label><%=_T('Deactivate')%>";
			account_button_info.id = "deactivate";
			account_button_info.title = "<%=_T('Deactivate')%>";
		}
		else {
			account_button_info.html = "<i class='buttons-icons-activate'></i><label><%=_T('Activate')%>";
			account_button_info.id = "activate";
			account_button_info.title = "<%=_T('Activate')%>";
		}
		return account_button_info;
	};

	adm_form_campaigns_section_accounts.prototype.update_account_in_grid = function( account )
	{
		if (typeof account !== "object") {
			return;
		}
		// obtengo el html row
		var $account_row = $("#adm_form_campaign_accounts_list").find("li[account_id='" + account.id + "']");
		// actualizo la data de la account en la row
		$account_row.data("account", account);
		// actualizo la descripcion de la cuenta
		var account_description = this.frm.getAccountDescription( { account: account, message_type: "WITH_SPECIFIC_PROXY" } );
		$account_row.find("label[account_desc]").text( account_description );
	};

	adm_form_campaigns_section_accounts.prototype.show_accounts_list = function()
	{
		var campaign = this.frm.campaign;
		var accounts = campaign.accounts;
		var has_account = $.isArray(accounts) && accounts.length > 0;
		var availables_channels = this.count_availables_channels();
		var $message = $("#adm_form_campaign_accounts_list_not_results_found");
		var $account_list = $("#adm_form_campaign_accounts_list");

		if ( availables_channels.count > 0 && has_account ) {
			$account_list.show();
			$message.text( "" ).hide();
			return;
		}

		$account_list.hide();
		$message.text( "<%=_T('No accounts found.')%>" ).show();
	};

	adm_form_campaigns_section_accounts.prototype.get_account_status = function( account )
	{
		if ( typeof account !== "object" ) {
			return;
		}

		var account_enabled = account.state === "1";
		var in_conflict = false;

		if ( account_enabled && ( !account.status || account.status.status.toUpperCase() !== "OK" ) ) {
			in_conflict = true;
		}

		var get_account_class = function()
		{
			// disabled class
			var disabled_class = !account_enabled ? " accountListItemDisabled" : "";
			// conflict class
			var conflict_class = in_conflict ? " accountListItemStatusConflict" : "";

			return disabled_class + conflict_class;
		};

		var get_conflict = function()
		{
			var response = {
				current_status: "OK",
				html: "<div conflict_message='true' current_status='OK' class='TU_CLASE_SIN_CONFLICTO'></div>"
			};

			if ( !account_enabled ) {
			// desactivada y sin conflicto
				return response;
			}

			var html_conflict = "<div conflict_message='true' current_status='{0}' class='accountListErrorMessage'><i class='far fa-times-circle'></i>{1}</div>";
			if ( !account.status ) {
				response.current_status = "GENERIC_ERROR";
				// mensaje generico
				response.html = format(
					html_conflict,
					response.current_status, // current status
					"<%=_T('Proxys not checking this account yet, please wait a moment, its coming up!')%>" // mensaje
				);
			}
			else {
				if ( account.status.status.toUpperCase() !== "OK" ) {
					response.current_status = account.status.status;
					// mensaje especifico
					response.html = format(
						html_conflict,
						response.current_status,    // current status
						inConcert.app.getAccountStatusLabel(account.proxyType, account.status.status)   //mensaje
					);
				}
			}

			return response;
		};

		var conflict = get_conflict();
		return {
			account_class  : get_account_class(),
			html_conflict  : conflict.html,
			current_status : conflict.current_status
		};
	};

	adm_form_campaigns_section_accounts.prototype.can_add_account_to_grid = function( account )
	{
		if ( typeof account !== "object" ) {
			return false;
		}
		var $account_row = $("#adm_form_campaign_accounts_list").find("li[account_id='" + account.id + "']");
		if ( $account_row && $account_row.length > 0 ) {
			// ya se ha agregado la row de la account
			return false;
		}
		return inConcert.app.hasLicensesForChannel( account.proxyType );
	};

	// valida si la campana es de telefonia / allegro
	adm_form_campaigns_section_accounts.prototype.__is_allegro_campaign = function() { return this.frm.campaign && this.frm.campaign.isAllegro(); };

	adm_form_campaigns_section_accounts.prototype.has_permission_to_deactivate = function()
	{
		if ( this.__is_allegro_campaign() ) {
			return false;
		}
		var campaign_id;
		var entity = this.frm.get_entity_instance();
		if ( entity !== null && entity !== undefined ) {
			campaign_id = entity.Id;
		}
		var app_id = this.frm.dashboard.current_app;
		return this.frm.dashboard.permissions.to_action( app_id, this.deactivate_action_id, undefined /* category type */, campaign_id );
	};

	adm_form_campaigns_section_accounts.prototype.has_permission_to_delete = function()
	{
		if ( this.__is_allegro_campaign() ) {
			this.has_perm_to_erase = false;
			return this.has_perm_to_erase;
		}
		this.has_perm_to_erase = adm_form_campaigns_section_accounts.uber.has_permission_to_delete.call( this );
		return this.has_perm_to_erase;
	};
	
	adm_form_campaigns_section_accounts.prototype._add_whatsapp_extra_action_buttons = function( account, buttons )
	{
		var self = this;
		if (self.frm.campaign.hasAccountWhatsappGroups(account)) {
			//groups
			var $button = $("".concat(
				"<button class='buttonColorGray gridActionButton whatsappGroupsButton' type='button' action_type='whatsapp_groups' enabled_required  title='"+ "<%=_T('Groups')%>" + "'>", 
					"<i class='buttons-icons-group'></i>",
					"<%=_T('Groups')%>",
				"</button>"));
			$button.click(function() {
				var manager = new inConcert.WhatsappGroupsManager(self.frm.campaign, account);
				manager.Show();
			});
			buttons.first.push($button);
		}
	};
	
	adm_form_campaigns_section_accounts.prototype._get_extra_action_buttons = function( account )
	{
		var buttons = {
			first : [],
			last : []
		};
		switch(account.proxyType) {
			case "WHATSAPP":
				this._add_whatsapp_extra_action_buttons(account, buttons);
		}
		return buttons;
	};

	adm_form_campaigns_section_accounts.prototype.add_account_to_grid = function( account )
	{
		var self = this;
		if ( !this.can_add_account_to_grid(account )) {
			return;
		}
		
		var account_enabled = account.state === "1";

		var get_account_class = function() {
			var proxy_type = account.proxyType.toUpperCase();
			//gmail is treated as a special case, since it's a MAIL proxy but has its own class differentiator
			var type = (proxy_type === "MAIL" && account.type === "GMAIL") ? "GMAIL" : proxy_type;
			return "accountListItemType" + type;
		};

		var html_buttons = "";

		// deactivate or activate buttons
		if ( this.has_permission_to_deactivate() ) {
			var account_button_info = this.get_account_button_info( account_enabled );
			html_buttons+= "<button class='buttonColorGray gridActionButton deactivate' type='button' action_type='" + account_button_info.id + "' title='" + account_button_info.title + "'>" + account_button_info.html + "</button>" + "<span class='action-buttons-separator'></span>";
		}
		
		// save or view buttons
		if ( this.has_permission_to_save() ) {
			html_buttons+= "<button class='buttonColorGray gridActionButton edit' type='button' action_type='edit' title='"+ "<%=_T('Edit')%>" + "'><i class='buttons-icons-edit'></i>" + "<%=_T('Edit')%>" + "</button>";
		}
		else {
			html_buttons+= "<button class='buttonColorGray gridActionButton view' type='button' action_type='edit' title='"+ "<%=_T('View')%>" + "'><i class='buttons-icons-zoomin'></i>" + "<%=_T('View')%>" + "</button>";
		}

		// delete button
		if ( this.has_permission_to_delete() ) {
			html_buttons+= "<button class='buttonColorGray gridActionButton delete' type='button' action_type='delete' title='"+ "<%=_T('Delete')%>" + "'><i class='buttons-icons-delete'></i>" + "<%=_T('Delete')%>" + "</button>";
		}

		var $account_row = $(
			"<li class='viewListItem accountListWaiting " + get_account_class() + "' account_id='" + account.id + "'>" +

				"<i class='accountListItemType' title='" + "<%=_T('Loading status')%>" + "'></i>" +
				"<label account_desc='true' class='viewListItemText account'>" + this.frm.getAccountDescription( { account: account, message_type: "WITH_SPECIFIC_PROXY" } ) + "</label>" +
				html_buttons +

			"</li>");
	
		var extraButtons = self._get_extra_action_buttons(account);
		$account_row.find(".gridActionButton:first").before(extraButtons.last);
		$account_row.find(".gridActionButton:last").after(extraButtons.first);
	
		$account_row.dblclick( function(){
			self.show_account_panel( account.proxyType, $(this).data("account") );
		});

		$account_row.data("account", account);

		var $last_action_button = $account_row.find(".gridActionButton:last");
		if ( $last_action_button.length ) {
			$last_action_button.addClass( "last" );
		}
		
		$account_row.find("[enabled_required]").prop("disabled", !account_enabled);

		$("#adm_form_campaign_accounts_list").append( $account_row );
	};

	adm_form_campaigns_section_accounts.prototype.is_account_type = function( account, proxy_type )
	{
		if ( typeof account !== "object" || typeof account.proxyType !== "string" ) {
			return false;
		}
		return account.proxyType.toUpperCase() === proxy_type;
	};

	adm_form_campaigns_section_accounts.prototype.fill_grids = function()
	{
		var self = this;
		var $account_list = $("#adm_form_campaign_accounts_list");
		// list header
		var $header =  $(
			"<li header='true' class='viewListTitle'>" +
				"<span class='firstItemIcon'></span>" +
				"<label class='viewListTitleText'>" + "<%=_T('Account')%>" + "</label>" +
				"<label class='gridAction3ButtonCol'></label>" +
			"</li>");
		$account_list.html("").append($header);

		var accounts = this.frm.campaign.accounts;
		$.each(accounts || {}, function( _, account) {
			self.add_account_to_grid(account);
		});
		this.show_accounts_list();
	};

	adm_form_campaigns_section_accounts.prototype.get_accounts = function()
	{
		var campaign = this.frm.campaign;
		return campaign.accounts;
	};

	adm_form_campaigns_section_accounts.prototype.show_account_panel = function( account_type, account )
	{
		var self = this;

		if ( !this.has_permission_to_section() ) {
			this.frm.dashboard.hide_waiting();
			var account_name = this.frm.getAccountDescription( { account: account, message_type: "WITH_PROXY" } );
			var message = format( "<%=_T('Unable to view {0} account: You dont have permissions for this action.')%>", account_name );
			this.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		this.hide_account_panel();

		this.frm.dashboard.show_waiting("<%=_T('Please wait, loading account information.')%>");

		var account_panel = this.accounts_panels[ account_type ];
		if ( !account_panel ) {
			throw "Panel account type '" + account_type + "' does not implemented";
		}
		var show = function()
		{
			self.frm.dashboard.$get_dom_element_by_id("form_custom_buttons").hide();
			self.destroy_header_custom_buttons();
			self.frm.hide_messages();
			self.current_panel = account_type;
			self.set_edit_mode(true);
			account_panel.frm.show( account );
		};
		var load = function() {
			account_panel.frm.load( function() {
				show();
			});
		};

		account_panel.frm.set_account( account );
		if ( this.accounts_popup_instance ) {
			this.accounts_popup_instance.hide( load );
		}
		else {
			load();
		}
	};

	adm_form_campaigns_section_accounts.prototype.hide_account_panel = function( is_cancel )
	{
		this.frm.hide_messages();
		this.frm.dashboard.$get_dom_element_by_id("form_data_container div[form_section='accounts'] div[account_panel]").hide(); // todos los panels de edicion (lo de todas las accounts)
		var current_panel = this.current_panel;
		if (!current_panel) {
			return;
		}
		if (!this.is_readonly()) {
			if ( is_cancel ) {
				this.messages_cleanup( current_panel, is_cancel );
			}
			this.frm.dashboard.$get_dom_element_by_id("form_custom_buttons").show();
		}
		var panel = this.accounts_panels[ current_panel ];
		panel.frm.on_hide();
		this.current_panel = null;
		$("#adm_form_campaign_accounts_edit_buttons_panel").hide();
		$("#adm_form_campaign_accounts_list_panel").show();
		this.on_show(); // on show la lista de accounts
		this.set_edit_mode(false);
	};

	adm_form_campaigns_section_accounts.prototype.realtime_update_accounts_status = function( /* data */ )
	{
		// aqui el parametro "data" no se utiliza, ya que la info de la campana se obtiene actualizada desde la clase application.js
		//	queda solo por compatibilidad del metodo

		var self = this;

		// TODO: Aca tengo que cambiar esto, solo debo de tomar la campana desde la "get_entity_instance" y eliminar "inConcert.app.getCampaign"
		//	ya que esta ultima solo nos sirve para las campanas a las cuales pertenecemos. Parecido al problema del issue Ref #4775
		//  Aca de alguna forma tengo que actualizar realtime la instancia que estoy editando, sino no funciona correctamente.
		var current_campaign = this.frm.get_entity_instance();
		if ( !current_campaign ) {
			return;
		}
		var campaign = inConcert.app.getCampaign( current_campaign.Id );
		if ( !campaign ) {
			return;
		}
		//---------------------------

		$.each( campaign.accounts || {}, function( _, account )
		{
			var account_id = account.id;
			var $account_row = $("#adm_form_campaign_accounts_list > li[account_id='" + account_id + "'] ");
			if ( $account_row.length === 0 ) {
				return true; // continue
			}

			var account_class_status = "";
			var html_account_conflict = "";
			var new_status = "";
			var account_status = self.get_account_status( account );

			if ( account_status ) {
				account_class_status  = account_status.account_class;
				html_account_conflict = account_status.html_conflict;
				new_status            = account_status.current_status;
			}

			var $conflict = $account_row.find( "[conflict_message='true']" );
			var previous_status = $conflict.attr( "current_status" );
			if ( new_status === previous_status ) {
				// si el estado de la cuenta no ha cambiado, no actualizo el DOM
				return true; // continue
			}
			// actualizo el DOM
			$account_row
				.removeClass( "accountListWaiting accountListItemDisabled accountListItemStatusConflict" )
				.addClass( account_class_status );
			$conflict.remove();
			$account_row.append( html_account_conflict );
			$account_row.find(".accountListItemType").attr("title", "");
		});
	};

	adm_form_campaigns_section_accounts.prototype.realtime_update = function( type, data )
	{
		if ( type === "update_accounts" ) {
			this.realtime_update_accounts_status( data );
		}
	};

	adm_form_campaigns_section_accounts.prototype.save_channel_thresholds = function( callback )
	{
		callback = callback || $.noop;

		var self = this;

		this.frm.dashboard.show_waiting("<%=_T('Please wait, updating threshold configuration.')%>");

		var panel = this.accounts_panels[ this.current_panel ];
		var th_instance = panel.frm.get_thresholds_instance();
		if ( !th_instance ) {
			callback( true );
			return;
		}
		if ( !panel.frm.can_configure_thresholds() ) {
			callback( true );
			return;
		}
		var channel_thresholds = {};
		var data = th_instance.get();
		channel_thresholds[ th_instance.type ] = data.thresholds;

		this.frm.campaign.saveThresholds({
			thresholds : channel_thresholds   ,
			callback : function(success/*, response_error*/)
			{
				if ( !success ) {
					var message = "<%=_T('Failed to save account threshold configuration...')%>";
					var message_type = "Error";
					var timeout_ms = 5000;
					self.frm.show_status_message(message, message_type, timeout_ms);
				}
				callback(success);
			},
			account_is_created : true // optional en el api: pero aca es requerido ya que se invoca de una lugar externo
		});
	};

	// clean up de los files upload
	adm_form_campaigns_section_accounts.prototype.messages_cleanup = function(current_panel, is_cancel)
	{
		if (!current_panel) {
			return;
		}
		var panel = this.accounts_panels[current_panel];
		var messages_instance = panel.frm.get_messages_instance();
		if (!messages_instance) {
			return;
		}

		messages_instance.messages_cleanup(is_cancel);
		if (is_cancel) {
			messages_instance.on_cleanup();
		}
	};

	adm_form_campaigns_section_accounts.prototype.save_messages = function( account_id, callback )
	{
		callback = callback || $.noop;

		var self = this;

		this.frm.dashboard.show_waiting("<%=_T('Please wait, updating account messages configuration.')%>");

		var panel = this.accounts_panels[ this.current_panel ];
		var messages_instance = panel.frm.get_messages_instance();
		if ( !messages_instance ) {
			callback(true);
			return;
		}

		var current_panel = this.current_panel;
		var messages = messages_instance.validate();

		this.frm.campaign.saveAccountMessages(
			account_id,
			messages,
			function( error )
			{
				var message, message_type, timeout_ms;
				if ( !error ) {
					var is_cancel = false;
					self.messages_cleanup( current_panel, is_cancel ); // clean up de los files upload
					callback(true);
					return;
				}

				callback(false);

				if (typeof error === "object") {
					error = error.result;
				}
				switch (error)
				{
					case "INVALID_REQUEST":
						message = "<%=_T('Invalid request, saving campaign account messages.')%>";
						break;
					case "UNABLE_TO_SAVE":
						message = "<%=_T('Unable to save campaign account messages, please try again in a few minutes....')%>";
						break;
					default:
						message = "<%=_T('Failed to save campaign account messages, please try again in a few minutes...')%>";
				}
				message_type = "Error";
				timeout_ms = 5000;

				self.frm.show_status_message(message, message_type, timeout_ms);
			});
	};

	adm_form_campaigns_section_accounts.prototype.save_account = function()
	{
		const self = this;

		if (!this.current_panel || !this.accounts_panels.hasOwnProperty(this.current_panel)) {
			return;
		}

		this.frm.hide_messages();
		let panel = this.accounts_panels[ this.current_panel ];

		this.frm.dashboard.show_waiting("<%=_T('Please wait.')%>");

		panel.frm.before_saving(function(must_save)
		{
			if (!must_save) {
				self.frm.dashboard.hide_waiting();
				return;
			}

			let account_info = panel.frm.validate();
			if (!account_info.success) {
				self.frm.dashboard.hide_waiting();
				return;
			}

			let campaign = self.frm.campaign;
			let account = account_info.account;
			let waiting_message, method, callback;

			// #18912 To check if is initial config and must apply to all accounts in the channel.
			let applyAccountGlobalActions = false;
			if (account.ActionOnAcdWaitTimeout || account.InactivityTimeout)
			{
				if (account.ActionOnAcdWaitTimeout) {
					if (account.ActionOnAcdWaitTimeout.initCfg !== undefined) {
						if (account.ActionOnAcdWaitTimeout.initCfg === true) {
							applyAccountGlobalActions = true;
						}
						delete account.ActionOnAcdWaitTimeout.initCfg;
					}
				}

				if (account.InactivityTimeout) {
					if (account.InactivityTimeout.initCfg !== undefined) {
						if (account.InactivityTimeout.initCfg === true) {
							applyAccountGlobalActions = true;
						}
						delete account.InactivityTimeout.initCfg;
					}
				}
			}

			if ( typeof panel.frm.save_account === "function" ) {
				callback = $.noop;
				panel.frm.save_account( account, callback );
				self.frm.dashboard.hide_waiting();
				return;
			}

			let account_type = "account";
			let proxy = ( account.proxyType || "" ).toUpperCase();

			const show_error = function( error_message, error )
			{
				error_message = error_message || "<%=_T('Error saving account, please try again later...')%>";
				const timeout_ms = false; //infinite
				self.frm.show_status_message( error_message, "Error", timeout_ms );

				if ( error.mail ) {
					panel.frm.select_tab_panel( error["typeof"], function() {
						if ( error["typeof"] === "OUTGOING" ) {
							$("#adm_form_campaign_accounts_mail_user_outgoing").focus();
						}
						else {
							$("#adm_form_campaign_accounts_mail_user_incoming").focus();
						}
					});
				}
			};

			const get_proxy_name = function( )
			{
				const name = self.frm.proxies_desc[ proxy ];
				if ( !name ) {
					throw "adm_form_campaigns_section_accounts.save_account - unrecognized proxy '" + proxy + "'";
				}
				return name;
			};

			let check_update_channels_accounts = function()
			{
				$.each( campaign.accounts, function( _, nextAccount)
				{
					if (applyAccountGlobalActions === false || nextAccount.proxyType !== proxy || account.id === nextAccount.id) {
						return true;
					}

					nextAccount.ActionOnAcdWaitTimeout = account.ActionOnAcdWaitTimeout;
					nextAccount.InactivityTimeout      = account.InactivityTimeout;

					campaign.updateAccount( nextAccount, undefined );
				});
			};

			if (account_info.is_new) {
				waiting_message = format("<%=_T('Please wait until {0} is added.')%>", account_type);
				method = "addAccount";
				account.state = "1";
				callback = function(success, new_account)
				{
					if (success) {
					// Los request relacionados ("save_channel_thresholds" y "save_messages") deben ejecutarse en orden para que todo funcione correctamente.
						panel.frm.on_saved(account);
						//panel.frm.on_saved(account, null /*old account*/, function() {
						// guarda los datos de la instancia de thresholds
							self.save_channel_thresholds(function( /*success */ ) {
							// guarda los messages
								self.save_messages( account.id, function( /*success */ ) {
									self.add_account_to_grid(new_account);
									window.setTimeout(function() {
										panel.frm.show_message_after_save(new_account,
											function() {
												self.hide_account_panel();
												var account_description = self.frm.getAccountDescription( { account: account, message_type: "WITH_PROXY" } );
												var message_success = format("<%=_T('{0} {1} was successfully added.')%>", account_description, account_type);
												self.frm.show_status_message(message_success, "Info");
												// lo agrego en un timeout para que se realice despues de que termina el callback del request "camaign.addAccount(..)"
												// y que haya ejecutado la funcion "campaign.setAccount(..)"
												window.setTimeout(function() {
													self.show_accounts_list();
												}, 1000);
											}
										);
									}, 500);
								});
							});
						//});
						check_update_channels_accounts();
						return;
					}

					let error = new_account;
					let campaign_err = error.campaign || "";
					let fb_page_name_err = error.page_name || "";
					let mail_err = error.mail || "";
					let user_error = error.user || "";
					let mail_user_url_err = user_error;
					let call_package_id = error.packageId || "";
					let call_package_camp_id = error.campaignId || "";
					let related_flow_camp_id = error.campaignId || "";
					let related_flow_acc_name = error.accountName || "";

					if ( ( error.url || "" ).length > 0 ) {
						mail_user_url_err = mail_user_url_err + " - " + error.url;
					}

					// Error actualizando
					let default_message = format("<%=_T('Error adding {0}, please try again later...')%>", account_type);
					let messages_error = {
					// GENERIC errors
						FORBIDDEN                                                         : "<%=_T('You dont have permissions for add account.')%>",
						ALREADY_EXISTS_IN_ANOTHER_CAMPAIGN                                : format("<%=_T('Unable to add {0}. Already exists in campaign {1}.')%>", account_type, campaign_err),
						ALREADY_EXISTS_IN_ANOTHER_VCC                                     : format("<%=_T('Unable to add {0}. Already exists in another company. For inquiries contact our technical service.')%>", account_type),
						ALREADY_EXISTS_IN_CURRENT_CAMPAIGN                                : format("<%=_T('Unable to add {0}. Already exists in current campaign.')%>", account_type),
						ACCOUNT_ALREADY_EXISTS_IN_CURRENT_VCC                             : format("<%=_T('Unable to add {0}. Account already exists in current VCC.')%>", account_type),
						CAMPAIGN_DOES_NOT_EXIST                                           : format("<%=_T('Unable to add {0}. Campaign does not exist.')%>", account_type),

						FAILED_SAVING_ON_DATABASE                                         : default_message,
						INVALID_REQUEST                                                   : default_message,
						UNABLE_TO_SAVE_ACCOUNT                                            : default_message,
						NOT_LICENSES_IN_VCC                                               : format("<%=_T('Unable to add account. Account Type {0} is not licensed.')%>", get_proxy_name() ),

					// CALL i6 errors
						CALL_I6_PACKAGE_IS_REQUIRED                                       : "<%=_T('Unable to add phone account. Packages are required.')%>",
						CALL_I6_PACKAGE_DOES_NOT_EXIST                                    : format("<%=_T('Unable to add phone account. Package {0} does not exist.')%>", call_package_id),
						CALL_I6_INBOUND_PACKAGES_CAN_ONLY_BE_IN_ONE_CAMPAIGN              : format("<%=_T('Unable to add phone account. Inbound package {0} can only be in one campaign. Found in the {1} campaign.')%>", call_package_id, call_package_camp_id),

					// FACEBOOK errors
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN          : format( "<%=_T('Unable to add account. Facebook page {0} in use in current campaign.')%>", fb_page_name_err ),
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_CAMPAIGN                          : format( "<%=_T('Unable to add account. Facebook page {0} in use in campaign {1}.')%>", fb_page_name_err, campaign_err ),
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_VCC                               : format( "<%=_T('Unable to add account. Facebook page {0} in use in another company. For inquiries contact our technical service.')%>", fb_page_name_err ),

					// FLOW errors
						FLOW_INBOUND_CONFLICT_ACCOUNT_RELATION                            : format("<%=_T('Unable to add account. The inbound flow in your activities references account {0} from campaign {1}.')%>", related_flow_acc_name, related_flow_camp_id),
						FLOW_OUT_OF_HOURS_CONFLICT_ACCOUNT_RELATION                       : format("<%=_T('Unable to add account. The out of hours flow in your activities references account {0} from campaign {1}.')%>", related_flow_acc_name, related_flow_camp_id),

					// INSTAGRAM errors
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN         : format( "<%=_T('Unable to add account. Instagram user {0} in use in current campaign.')%>", user_error ),
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_CAMPAIGN                         : format( "<%=_T('Unable to add account. Instagram user {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_VCC                              : format( "<%=_T('Unable to add account. Instagram user {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),

					// MAIL errors
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN     : format( "<%=_T('Unable to add account. Incoming user/server {0} in use in account {1} of the current campaign.')%>", mail_user_url_err, mail_err ),
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_CAMPAIGN                     : format( "<%=_T('Unable to add account. Incoming user/server {0} in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err ),
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_VCC                          : format( "<%=_T('Unable to add account. Incoming user/server {0} in use in another company. For inquiries contact our technical service.')%>", mail_user_url_err ),

						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN     : format( "<%=_T('Unable to add account. Outgoing user/server {0} in use in account {1} of the current campaign.')%>", mail_user_url_err, mail_err ),
						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_CAMPAIGN                     : format( "<%=_T('Unable to add account. Outgoing user/server {0}in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err ),
						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_VCC                          : format( "<%=_T('Unable to add account. Outgoing user/server {0} in use in another company. For inquiries contact our technical service.')%>", mail_user_url_err ),
						MAIL_OUTGOING_USER_IN_USE_VCCS_ACCOUNTS                           : format( "<%=_T('Unable to add account. Outgoing user/server {0} in use in outgoing account {1} of vcc.')%>", mail_user_url_err, mail_err ),

					// whatsapp errors
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN : format( "<%=_T('Unable to add account. Wavy user {0} in use in current campaign.')%>", user_error ),
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_CAMPAIGN                 : format( "<%=_T('Unable to add account. Wavy user {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_VCC                      : format( "<%=_T('Unable to add account. Wavy user {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN         : format( "<%=_T('Unable to add account. Whatsapp phone {0} in use in current campaign.')%>", user_error ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_CAMPAIGN                         : format( "<%=_T('Unable to add account. Whatsapp phone {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_VCC                              : format( "<%=_T('Unable to add account. Whatsapp phone {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),
						WHATSAPP_ACTIVATION_CODE_IS_REQUIRED                              : format( "<%=_T('Activation Code is required.')%>", user_error ),
						WHATSAPP_ACTIVATION_CODE_NOT_LOCKED                               : format( "<%=_T('Activation Code is not loked by account.')%>", user_error ),
						WHATSAPP_ACTIVATION_CODE_INVALID                                  : format( "<%=_T('Activation Code is invalid.')%>", user_error ),

					// YOTUBE errors
						YOUTUBE_UID_CHANGED                                               : "<%=_T('Unable to add account. Youtube channel can not be changed.')%>",
					
					// OTHERS errors - helpers.validations
						ERROR_FIELDS_VALIDATION                                           : error.reason
					};

					const err_code = ( error.result || "" ).toUpperCase();
					show_error( messages_error[ err_code ], error );
				};
			}
			else {
				let old_account = campaign.getAccount( account.id );
				waiting_message = format("<%=_T('Please wait until {0} is updated.')%>", account_type);
				method = "updateAccount";
				callback = function( success, error )
				{
					if (success) {
					// Los request relacionados ("save_channel_thresholds" y "save_messages") deben ejecutarse en orden para que todo funcione correctamente.
						// guarda los datos de la instancia de thresholds
						panel.frm.on_saved(account, old_account, function() {
							self.save_channel_thresholds(function( /*success */ ) {
								// guarda los messages
								self.save_messages( account.id, function( /*success */ ) {
									// agrega en la grilla
									self.update_account_in_grid(account);
									// realiza actualizaciones posteriores a guardar la cuenta
									self.frm.hide_messages();
									panel.frm.show_message_after_save(account,
										function() {
											self.hide_account_panel();
											var account_description = self.frm.getAccountDescription( { account: account, message_type: "WITH_PROXY" } );
											var message_success = format("<%=_T('{0} {1} was successfully updated.')%>", account_description, account_type);
											self.frm.show_status_message(message_success, "Info");
										}
									);
								});
							});
						});
						check_update_channels_accounts();
						return;
					}

					let campaign_err = error.campaign || "";
					let fb_page_name_err = error.page_name || "";
					let mail_err = error.mail || "";
					let user_error = error.user || "";
					let mail_user_url_err = user_error;
					let call_package_id = error.packageId || "";
					let call_package_camp_id = error.campaignId || "";
					let related_flow_camp_id = error.campaignId || "";
					let related_flow_acc_name = error.accountName || "";

					if ( ( error.url || "" ).length > 0 ) {
						mail_user_url_err = mail_user_url_err + " - " + error.url;
					}

					let attention_level_name = "";
					if (error.att_level_id) {
						let campaign = self.frm.campaign;
						let attention_level = campaign.getAttentionLevel(error.att_level_id);
						if (attention_level) {
							attention_level_name = attention_level.name;
						}
					}

					// Error actualizando
					let default_message = format("<%=_T('Error updating {0}, please try again later...')%>", account_type);
					let messages_error = {
					// GENERIC errors
						FORBIDDEN                 : "<%=_T('You dont have permissions for update account.')%>",
						CAMPAIGN_DOES_NOT_EXIST   : format("<%=_T('Unable to update {0}. Campaign does not exist.')%>", account_type),
						INVALID_REQUEST           : default_message,
						FAILED_SAVING_ON_DATABASE : default_message,
						UNABLE_TO_SAVE_ACCOUNT    : default_message,
						NOT_LICENSES_IN_VCC       : format("<%=_T('Unable to update account. Account Type {0} is not licensed.')%>", get_proxy_name() ),

					// ATTENTION LEVELS errors
						ATTENTION_LEVEL_WITH_INTERACTIONS_IN_WAITING: function() {
							var msg;
							if (error.number_waiting_conversations === 1) {
								msg = "<%=_T('Unable to remove {0} attention level of account. Attention level in account has {1} conversation waiting.')%>";
							}
							else {
								msg = "<%=_T('Unable to remove {0} attention level of account. Attention level in account has {1} conversations waiting.')%>";
							}
							return format(msg, attention_level_name, error.number_waiting_conversations);
						},
					// SOCIAL NETWORKS
						HAS_ACTIVE_POSTS:    "<%=_T('A removed page has active social network posts. Close them before delete it.')%>",

					// CALL i6 errors
						CALL_I6_PACKAGE_IS_REQUIRED                                       : "<%=_T('Unable to update phone account. Packages are required.')%>",
						CALL_I6_PACKAGE_DOES_NOT_EXIST                                    : format("<%=_T('Unable to update phone account. Package {0} does not exist.')%>", call_package_id),
						CALL_I6_INBOUND_PACKAGES_CAN_ONLY_BE_IN_ONE_CAMPAIGN              : format("<%=_T('Unable to update phone account. Inbound package {0} can only be in one campaign. Found in the {1} campaign.')%>", call_package_id, call_package_camp_id),
						CALL_I6_NOT_ALL_OUTGOING_PACKETS_CAN_BE_REMOVED_THE_ACCOUNT_IS_CONFIGURED_IN_AN_OUTGOING_PROCESS:
							format("<%=_T('It is not possible drop all outbound packages. Account is configured in ({0}) outbound process, and requires at least one package.')%>", error.process_id),

					// FACEBOOK errors
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN          : format( "<%=_T('Unable to update account. Facebook page {0} in use in current campaign.')%>", fb_page_name_err ),
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_CAMPAIGN                          : format( "<%=_T('Unable to update account. Facebook page {0} in use in campaign {1}.')%>", fb_page_name_err, campaign_err ),
						FACEBOOK_PAGE_IN_USE_IN_ANOTHER_VCC                               : format( "<%=_T('Unable to update account. Facebook page {0} in use in another company. For inquiries contact our technical service.')%>", fb_page_name_err ),

					// FLOW errors
						FLOW_INBOUND_CONFLICT_ACCOUNT_RELATION                            : format("<%=_T('Unable to update account. The inbound flow in your activities references account {0} from campaign {1}.')%>", related_flow_acc_name, related_flow_camp_id),
						FLOW_OUT_OF_HOURS_CONFLICT_ACCOUNT_RELATION                       : format("<%=_T('Unable to update account. The out of hours flow in your activities references account {0} from campaign {1}.')%>", related_flow_acc_name, related_flow_camp_id),

					// INSTAGRAM errors
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN         : format( "<%=_T('Unable to update account. Instagram user {0} in use in current campaign.')%>", user_error ),
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_CAMPAIGN                         : format( "<%=_T('Unable to update account. Instagram user {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						INSTAGRAM_USER_IN_USE_IN_ANOTHER_VCC                              : format( "<%=_T('Unable to update account. Instagram user {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),

					// MAIL errors
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN     : format( "<%=_T('Unable to update account. Incoming user/server {0} in use in account {1} of the current campaign.')%>", mail_user_url_err, mail_err ),
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_CAMPAIGN                     : format( "<%=_T('Unable to update account. Incoming user/server {0} in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err ),
						MAIL_INCOMING_USER_IN_USE_IN_ANOTHER_VCC                          : format( "<%=_T('Unable to update account. Incoming user/server {0} in use in another company. For inquiries contact our technical service.')%>", mail_user_url_err ),

						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN     : format( "<%=_T('Unable to update account. Outgoing user/server {0} in use in account {1} of the current campaign.')%>", mail_user_url_err, mail_err ),
						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_CAMPAIGN                     : format( "<%=_T('Unable to update account. Outgoing user/server {0}in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err ),
						MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_VCC                          : format( "<%=_T('Unable to update account. Outgoing user/server {0} in use in another company. For inquiries contact our technical service.')%>", mail_user_url_err ),
						MAIL_OUTGOING_USER_IN_USE_VCCS_ACCOUNTS                           : format( "<%=_T('Unable to update account. Outgoing user/server {0} in use in outgoing account {1} of vcc.')%>", mail_user_url_err, mail_err ),

					// WHATSAPP errors
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN : format( "<%=_T('Unable to update account. Wavy user {0} in use in current campaign.')%>", user_error ),
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_CAMPAIGN                 : format( "<%=_T('Unable to update account. Wavy user {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						WHATSAPP_WAVY_USERNAME_IN_USE_IN_ANOTHER_VCC                      : format( "<%=_T('Unable to update account. Wavy user {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN         : format( "<%=_T('Unable to update account. Whatsapp phone {0} in use in current campaign.')%>", user_error ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_CAMPAIGN                         : format( "<%=_T('Unable to update account. Whatsapp phone {0} in use in campaign {1}.')%>", user_error, campaign_err ),
						WHATSAPP_PHONE_IN_USE_IN_ANOTHER_VCC                              : format( "<%=_T('Unable to update account. Whatsapp phone {0} in use in another company. For inquiries contact our technical service.')%>", user_error ),

						// youtube errors
						YOUTUBE_UID_CHANGED                                               : "<%=_T('Unable to update account. Youtube channel can not be changed.')%>"
					};

					let err_code = ( error.result || "" ).toUpperCase();
					let msg_err = messages_error[err_code];
					if (typeof(msg_err) === "function") {
						msg_err = msg_err();
					}
					show_error(msg_err, error);
				};
			}

			self.frm.dashboard.show_waiting(waiting_message);
			// pequeno timeout para dar feedback (tiempo al waiting)
			window.setTimeout(function() {
				// addAccount o UpdateAccount
				campaign[method](account, callback);
			}, 500);
		});

	};

	adm_form_campaigns_section_accounts.prototype.erase_account = function(account)
	{
		var self = this;

		var account_description = this.frm.getAccountDescription( { account: account, message_type: "WITH_SPECIFIC_PROXY" } );
		var proxy_desc = this.frm.proxies_desc[ account.proxyType.toUpperCase() ];
		var confirmation_message, response_success_message, response_error_message, waiting_message;

		confirmation_message     = format( "<%=_T('Do you want to delete {0} account {1} ?')%>"                   , proxy_desc , account_description );
		response_success_message = format( "<%=_T('{0} {1} account was successfully deleted.')%>"                 , proxy_desc , account_description );
		response_error_message   = format( "<%=_T('Error removing {0} {1} account, please try again later...')%>" , proxy_desc , account_description );
		waiting_message          = format( "<%=_T('Please wait until {0} account {1} is deleted.')%>"             , proxy_desc , account_description );

		var campaign = this.frm.campaign;
		var account_id = account.id;

		var callback_delete = function( success, error )
		{
			var message, message_type, timeout_ms;
			if (success) {
				var $row = $("#adm_form_campaign_accounts_list > li[account_id='" + account_id + "'] ");
				$row.fadeOut("slow", function() {
					$row.remove();
					self.show_accounts_list();
				});
				message = response_success_message;
				message_type = "Info";
				var attention_level_section = self.frm.get_frm_section( "attention_level" );
				if (attention_level_section) {
					attention_level_section.on_account_delete(account_id);
				}
			}
			else {
				let display_account_id = account_id || "";
				let messages_error = {
					FORBIDDEN:  "<%=_T('You dont have permissions for remove account.')%>",
					UNABLE_TO_DELETE_ALLEGRO_ACCOUNT : format("<%=_T('Unable to delete {0} call allegro account.')%>", display_account_id),
					HAS_ACTIVE_POSTS : "<%=_T('Account has active social network posts. Close them before delete it.')%>",
					HAS_ACTIVE_BATCHES : "<%=_T('Account has active outgoing batches. Close them before delete it.')%>",
					"CAMPAIGN IS USED IN RULES OF OTHER CAMPAIGNS": "<%=_T('It is in use in the Rules of other Campaigns')%>",
					"CAMPAIGN IS USED IN ACD TIMEOUT OF OTHER CAMPAIGNS": "<%=_T('It is in use in the ACD timeout of other Campaigns')%>",
					"CAMPAIGN IS USED IN ACD TIMEOUT AND RULES OF OTHER CAMPAIGNS": "<%=_T('It is in use in the ACD timeout AND Rules of other Campaigns')%>",
					"ACCOUNT HAS AN OUTBOUND PROCESS CONFIGURED THAT IS RUNNING": format("<%=_T('Unable to delete {0} account. The Account has an Outbound Process configured that is Running. Process: {1}.')%>", display_account_id, error.processId),
					"ACCOUNT HAS AN OUTBOUND PROCESS CONFIGURED": format("<%=_T('Unable to delete {0} account. The Account has an Outbound Process configured. Process: {1}.')%>", display_account_id, error.processId)
				};

				if (error.errors) {
					let popup_message = messages_error[(error.errType || "").toUpperCase()] || "<%=_T('Is in use')%>";
					let title, detail, msg_type;
					title = format("<%=_T('Unable to delete {0} account')%>", account_description);
					const action_view = self.frm.dashboard.get_current_action_obj();
					detail = action_view.make_delete_warning_detail(error.errors);
					msg_type = "Error";
					self.frm.dashboard.hide_waiting();
					self.frm.show_popup_message({
						messages: {
							title  : title,
							message: popup_message + "<%=_T(':')%>",
							detail : detail,
							type   : msg_type
						}
					});
					return;
				}
				let err_code = (error.result || "").toUpperCase();
				message = messages_error[err_code] || response_error_message;
				message_type = "Error";
				timeout_ms = 5000;
			}
			self.frm.show_status_message( message, message_type, timeout_ms );
		};

		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			confirmation_message,
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					return;
				}
				campaign.beforeDeleteAccount(account_id, account.proxyType, function(confirmation, deleteData) {
					if (confirmation) {
						self.frm.dashboard.show_waiting( waiting_message );
						campaign.deleteAccount(account_id, account.proxyType, deleteData, callback_delete);
					}
				});
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_campaigns_section_accounts.prototype.change_account_status = function(account, new_status)
	{
		var self = this;

		var account_id = account.id;
		let display_account_id = account_id || "";
		var account_description = this.frm.getAccountDescription( { account: account, message_type: "WITH_PROXY" } );

		var state, old_state, confirmation_message, waiting_message, success_message, error_message;
		if (new_status === "activate") {
			state = "1";
			old_state = "0";
			confirmation_message = format("<%=_T('Do you want activate account {0} ?')%>", account_description);
			waiting_message = format("<%=_T('Please wait until account {0} is activated.')%>", account_description);
			success_message = format("<%=_T('{0} account was successfully activated.')%>", account_description);
			error_message = format("<%=_T('Error activating {0} account, please try again later...')%>", account_description);
		}
		else {
			state = "0";
			old_state = "1";
			confirmation_message = format("<%=_T('Do you want deactivate account {0} ?')%>", account_description);
			waiting_message = format("<%=_T('Please wait until account {0} is deactivated.')%>", account_description);
			success_message = format("<%=_T('{0} account was successfully deactivated.')%>", account_description);
			error_message = format("<%=_T('Error deactivating {0} account, please try again later...')%>", account_description);
		}

		var account_enabled = state === "1";

		var callback_change_status = function( success, error )
		{
			var message, message_type, timeout_ms;
			if (success) {
				var $row = $("#adm_form_campaign_accounts_list > li[account_id='" + account_id + "'] ");
				if (account_enabled) {
					$row.removeClass("accountListItemDisabled accountListItemStatusConflict");
					$row.find(".formListAccountListMessage").hide();
				}
				else {
					$row.addClass("accountListItemDisabled");
				}
				var $button = $row.find("button[action_type='" + new_status + "']");
				var account_button_info = self.get_account_button_info(account_enabled);
				$button.attr("action_type", account_button_info.id);
				$button.attr("title", account_button_info.title);
				$button.html(account_button_info.html);
				$row.find("[enabled_required]").prop("disabled", !account_enabled);

				message = success_message;
				message_type = "Info";
			}
			else {
				let messages_error = {
					FORBIDDEN                                                   : format( "<%=_T('You dont have permissions for {0} account.')%>", new_status || "" ),
					UNABLE_TO_CHANGE_STATE_ALLEGRO_ACCOUNT                      : format( "<%=_T('Unable to change status for {0} call allegro account.')%>", display_account_id),
					"ACCOUNT HAS AN OUTBOUND PROCESS CONFIGURED THAT IS RUNNING": format("<%=_T('Unable to change status in {0} account. The Account has an Outbound Process configured that is Running. Process: {1}.')%>", display_account_id, error.processId),
					"ACCOUNT HAS AN OUTBOUND PROCESS CONFIGURED"                : format("<%=_T('Unable to change status in {0} account. The Account has an Outbound Process configured. Process: {1}.')%>", display_account_id, error.processId),

				// INSTAGRAM errors
					"INSTAGRAM_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN" : format( "<%=_T('Unable to change status in {0} account. Instagram page {1} is in use in account {2} of the current campaign.')%>", account.name, error.page_name, error.account_name),
					"INSTAGRAM_USER_IN_USE_IN_ANOTHER_CAMPAIGN"                 : format( "<%=_T('Unable to change status in {0} account. Instagram page {1} is in use in account {2} of the campaign {3}.')%>", account.name, error.page_name, error.account_name, error.campaign_name),
					"INSTAGRAM_USER_IN_USE_IN_ANOTHER_VCC"                      : format( "<%=_T('Unable to change status in {0} account. Instagram page {1} is in use in another company.')%>", account.name, error.page_name ),
				// WHATSAPP errors
					"WHATSAPP_PHONE_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN" : format( "<%=_T('Unable to change status in {0} account. Whatsapp phone {1} is in use in account {2} of the current campaign.')%>", account.name, error.phone_number, error.account_name),
					"WHATSAPP_PHONE_IN_USE_IN_ANOTHER_CAMPAIGN"                 : format( "<%=_T('Unable to change status in {0} account. Whatsapp phone {1} is in use in account {2} of the campaign {3}.')%>", account.name, error.phone_number, error.account_name, error.campaign_name),
					"WHATSAPP_PHONE_IN_USE_IN_ANOTHER_VCC"                      : format( "<%=_T('Unable to change status in {0} account. Whatsapp phone {1} is in use in another company.')%>", account.name, error.phone_number )
				};
				var err_code = ( error.result || "" ).toUpperCase();
				message = messages_error[ err_code ] || error_message;
				message_type = "Error";
				timeout_ms = 5000;
			}

			self.frm.show_status_message(message, message_type, timeout_ms);
		};

		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			confirmation_message,
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					return;
				}
				self.frm.dashboard.show_waiting(waiting_message);
				self.frm.campaign.changeAccountState(account, account_enabled, callback_change_status);
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_campaigns_section_accounts.prototype.build_header_custom_buttons = function()
	{
		var html_buttons = "<button id='adm_form_campaign_accounts_new' class='pageButtonGeneric buttonColorPrimary pageButtonNew pageButtonDropdown' ><%=_T('Add Account')%><i class='far fa-angle-down mr-0'></i></button>";

		if ( this.__is_allegro_campaign() || this.count_availables_channels() === 0 ) {
		// si no tiene channels disponibles, no crea el boton de new account
			html_buttons = "";
			$("#adm_form_campaign_accounts_cancel")
				.attr("value", "<%=_T('Close')%>");
		}

		adm_form_campaigns_section_accounts.uber.build_header_custom_buttons.call(this, html_buttons);

		if ( this.current_panel !== null ) {
			this.frm.dashboard.$get_dom_element_by_id("form_custom_buttons").hide();
		}
	};

	adm_form_campaigns_section_accounts.prototype.count_availables_channels = function()
	{
		var availables = { count: 0, channels: {} };

		$.each( this.accounts_panels, function( proxy, account_info )
		{
			availables.channels[ proxy ] = account_info.is_visible_in_menu();
			if ( availables.channels[ proxy ] ) {
				availables.count++;
			}
		});

		return availables;
	};

	adm_form_campaigns_section_accounts.prototype.before_jump_to_other_menunav = function(){
		this.hide_account_panel( true /* cancel */ );
	};

	adm_form_campaigns_section_accounts.prototype.set_events = function()
	{
		var self = this;

		$("#adm_form_campaign_accounts_list_cancel")
			.off("click")
			.on("click", function() {
				self.frm.close();
			});

		// cierra la seccion del frm de webchat y vuelve a la lista
		$("#adm_form_campaign_accounts_cancel")
			.off("click")
			.on("click", function() {
				self.hide_account_panel( true /* cancel */ );
			});

		var event_selector;

		// new account event (abre el frm para agregar una nueva account)
		event_selector = "div[account_type]";
		this.frm.dashboard.$get_dom_element_by_id("form_menu")
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				if ( !self.has_permission_to_save() ) {
					var message = "<%=_T('You dont have permissions for add account.')%>";
					self.frm.show_status_message( message, "Error", 5000 );
					return;
				}
				var account_type = $(this).attr("account_type");
				self.show_account_panel( account_type );
			});

		// new account event (abre el frm para agregar una nueva account)
		event_selector = "#adm_form_campaign_accounts_new";
		var event_selector_new = "#adm_form_campaign_accounts_new";
		var $header_buttons = this.frm.dashboard.$get_dom_element_by_id( "form_custom_buttons" ); //conf_form_custom_buttons
		$header_buttons
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				if ( !self.has_permission_to_save() ) {
					window.setTimeout( function() {
						const message = "<%=_T('You dont have permissions for add account.')%>";
						self.frm.show_status_message( message, "Error", 5000 );
					}, 200 );
					return;
				}
				var availables_channels = self.count_availables_channels();

				if ( availables_channels.count === 0 ) {
				// si no tiene channels disponibles me voy
					window.setTimeout( function() {
						const message = "<%=_T('You dont have available channels.')%>";
						self.frm.show_status_message( message, "Error", 5000 );
					}, 200 );
					return;
				}

				if ( availables_channels.count === 1 ) {
				// si solo existe un tipo de cuenta, no muestra el menu
					var account_type;
					// me quedo con el primer elemento del array
					$.each( availables_channels.channels, function( type, enabled ) {
						if ( enabled ) {
							account_type = type;
							return false; // break
						}
					});
					if ( !account_type ) {
						return;
					}
					self.show_account_panel( account_type );
				}
				else {
				// si existe mas de un tipo de cuenta para configurar, muestra menu
					var popup = self.accounts_popup_instance;
					if ( !popup ) {
						$(event_selector_new).addClass('show');
						return;
					}
					if ( popup.isVisible() ) {
						popup.hide();
						$(event_selector_new).removeClass('show');
					}
					else {
						var count_panels = Object.keys( self.accounts_panels || {} ).length;
						$.each( self.accounts_panels, function( proxy, account_info )
						{
							account_info.is_enabled_in_menu(
								function( response ) 
								{
									var selector = "div[account_type='" + proxy + "']";
									count_panels--;
									var $option = self.frm.dashboard.$get_dom_element_by_id("form_menu " + selector );
									var classes_names_ds = "accountTypeItem-ds disabledInfo";

									var $menu = self.frm.dashboard.$get_dom_element_by_id("form_menu");
									if ( response.enabled ) {
										$option
											.removeClass( classes_names_ds )
											.prop( "disabled", false );
										$menu.inconcert_tooltip({ detach: true, bind_selector: selector });
									}
									else {
										var tooltip = response.message || "<%=_T('Unable to add new account.')%>";
										$option
											.addClass( classes_names_ds )
											.prop( "disabled", true );
										$menu.inconcert_tooltip({ message: tooltip, bind_selector: selector });
									}

									if ( count_panels === 0 ) {
										popup.show();									
										$(event_selector_new).addClass('show');
									}
								});
						});
					}
				}
			});

		// guarda la account (crea un nueva o actualiza)
		$("#adm_form_campaign_accounts_save")
			.off("click")
			.on("click", function() {
				if ( !self.has_permission_to_save() ) {
					var message = "<%=_T('You dont have permissions for save account.')%>";
					self.frm.show_status_message( message, "Error", 5000 );
					return;
				}
				self.save_account();
			});


		var grid_actions = function()
		{
			var $this = $(this);
			var action_type = $this.attr("action_type");
			var account = $this.parent().data("account");
			var account_type = account.proxyType.toUpperCase();

			switch( action_type ) {
				case "edit":
					self.show_account_panel( account_type, account );
					return;

				case "delete":
					self.erase_account( account );
					return;

				case "activate":
					self.change_account_status( account, action_type );
					return;

				case "deactivate":
					self.change_account_status( account, action_type );
					return;
			}
		};

		event_selector = "li button[action_type]";
		$( "#adm_form_campaign_accounts_list" )
			.off( "click", event_selector               )
			.on(  "click", event_selector, grid_actions );
	};

	adm_form_campaigns_section_accounts.prototype.initialize = function()
	{
		this.frm.load_im_protocols();
		var visible_in_menu_count = 0;
		var $menu = this.frm.dashboard.$get_dom_element_by_id("form_menu");
		$menu.html("");

		var sorted_accounts_panels = inConcert.Utils.hash_to_sorted_array( this.accounts_panels, function(panel) { 
			return inConcert.app.channelSortNumber(panel.Id);
		});
		$.each(sorted_accounts_panels,
			function( _, account )
			{
				if ( account.is_visible_in_menu() ) {
					var account_type = account.Id;
					$menu.append(
						"<div class='accountTypeItem " + account.class_name +  "'" +
							" account_type='" +  account_type + "'>" + account.name + "</div>");
					visible_in_menu_count++;
				}
			});

		var prefix_id = this.frm.dashboard.prefix_id;
		this.accounts_popup_instance = null;
		if ( visible_in_menu_count > 1 ) {
			this.accounts_popup_instance = new inConcert.PopupWindow(
			{
				ownerId   : prefix_id + "form_menu", // container id
				eventName : prefix_id + "form_menu_accounts", // evento click personalizado
				callbacks : {
					onClose: function() {
						this.hide();
					}
				}
			});
		}
	};

	adm_form_campaigns_section_accounts.prototype.set = function(campaign)
	{
		adm_form_campaigns_section_accounts.uber.set.call(this, [campaign]);
		this.initialize();
		this.fill_grids();
	};

	adm_form_campaigns_section_accounts.prototype.editable = function()
	{
		adm_form_campaigns_section_accounts.uber.editable.call( this );

		$("#adm_form_campaign_accounts_save")
			.prop("disabled", false)
			.show();

		$("#adm_form_campaign_accounts_cancel")
			.attr("value", "<%=_T('Cancel')%>");
	};

	adm_form_campaigns_section_accounts.prototype.readonly = function()
	{
		adm_form_campaigns_section_accounts.uber.readonly.call( this );

		$("#adm_form_campaign_accounts_new")
			.prop("disabled", true)
			.hide();

		$("#adm_form_campaign_accounts_save")
			.prop("disabled", true)
			.hide();

		$("#adm_form_campaign_accounts_cancel")
			.attr("value", "<%=_T('Close')%>");
	};

	adm_form_campaigns_section_accounts.prototype.on_show = function()
	{
		adm_form_campaigns_section_accounts.uber.on_show.call( this, {show: false} );
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_campaigns_section_accounts.prototype.on_hide = function()
	{
		adm_form_campaigns_section_accounts.uber.on_hide.call( this );
		if ( this.accounts_popup_instance && this.accounts_popup_instance.isVisible() ) {
			this.accounts_popup_instance.hide();
		}
	};

	adm_form_campaigns_section_accounts.prototype.set_tweety_contact_form_projects = function( projects )
	{
		var contact_form_panel = this.accounts_panels.WEBCONTACT;
		if ( contact_form_panel && contact_form_panel.frm ) {
			contact_form_panel.frm.set_tweety_contact_form_projects( projects );
		}
	};

	adm_form_campaigns_section_accounts.prototype.set_tweety_webchat_projects = function( projects )
	{
		var chat_panel = this.accounts_panels.CHAT;
		if ( chat_panel && chat_panel.frm ) {
			chat_panel.frm.set_tweety_webchat_projects( projects );
		}
	};

	adm_form_campaigns_section_accounts.prototype.set_tweety_webcall_projects = function( projects )
	{
		var call_panel = this.accounts_panels.CALL;
		if ( call_panel && call_panel.frm && call_panel.frm.set_tweety_webcall_projects ) {
			call_panel.frm.set_tweety_webcall_projects( projects );
		}
	};

	adm_form_campaigns_section_accounts.prototype.attach_load_request = function( request )
	{
		if ( !$.isArray( request ) ) {
			return;
		}
		var self = this;

		// font family
		request.push(
			{
				request_function: function( success_fn )
				{
					var fonts = inConcert.app.loadFontsList();
					return success_fn( true, fonts);
				},
				success: function( list ) {
					$.each( self.accounts_panels, function( _, account ) {
						if ( account.frm ) {
							account.frm.fill_font_family( list );
						}
					});
				}
			}
		);

		// font size
		request.push(
			{
				request_function: function( success_fn )
				{
					var fontsizes = [];

					var push = function( font_size ) {
						fontsizes.push({ id: font_size.toString(), name: font_size + "px" });
					};

					var i;
					for ( i=8 ; i <= 11; ++i  ) { push(i); }
					for ( i=12; i <= 28; i+=2 ) { push(i); }
					push(36);
					push(48);
					push(72);

					return success_fn( true, fontsizes);
				},
				success: function( list ) {
					$.each( self.accounts_panels, function( _, account ) {
						if ( account.frm ) {
							account.frm.fill_font_size( list );
						}
					});
				}
			}
		);

		// mail signatures
		request.push(
			{
				request_function: function( success_fn ) {
					return inConcert.app.loadMailSignatures( success_fn, true /* from cache */ );
				},
				success: function( list ) {
					$.each( self.accounts_panels, function( _, account ) {
						if ( account.frm ) {
							account.frm.fill_signatures( list );
						}
					});
				}
			}
		);

		// spell languages
		request.push(
			{
				request_function: function( success_fn ) {
					return inConcert.app.loadLanguages( success_fn );
				},
				success: function( list ) {
					$.each( self.accounts_panels, function( _, account ) {
						if ( account.frm ) {
							account.frm.fill_languages( list );
						}
					});
				}
			}
		);

		// distribution algorithms
		request.push(
			{
				request_function: function( success_fn ) {
					return inConcert.app.loadDistributionAlgorithms( success_fn );
				},
				success: function( list ) {
					$.each( self.accounts_panels, function( proxy_type, account ) {
						if ( account.frm ) {
							account.frm.fill_distribution( inConcert.Utils.sortArray(list[proxy_type], "description", "asc") );
						}
					});
				}
			}
		);

		// tweety web chat projects
		request.push({
			request_function: function( success, response ) {
				var tweety = inConcert.app.getTweetyAPI();
				return tweety.load_webchat_projects(success, response);
			},
			success: function( projects ) {
				self.set_tweety_webchat_projects( projects );
			},
			error: function() {
				var message = "<%=_T('Error retrieving inconcert web chat applications.')%>";
				var timeout_ms = 5000;
				self.frm.show_status_message(message, "Error", timeout_ms);
			}
		});

		// tweety web chat projects
		request.push({
			request_function: function( success, response ) {
				var tweety = inConcert.app.getTweetyAPI();
				return tweety.load_webcall_projects(success, response);
			},
			success: function( projects ) {
				self.set_tweety_webcall_projects( projects );
			},
			error: function() {
				var message = "<%=_T('Error retrieving inconcert web call applications.')%>";
				var timeout_ms = 5000;
				self.frm.show_status_message(message, "Error", timeout_ms);
			}
		});

		// tweety contact form projects
		request.push({
			request_function: function( success, response ) {
				var tweety = inConcert.app.getTweetyAPI();
				return tweety.load_contactform_projects(success, response);
			},
			success: function( projects ) {
				self.set_tweety_contact_form_projects( projects );
			},
			error: function() {
				var message = "<%=_T('Error retrieving inconcert contact form applications.')%>";
				var timeout_ms = 5000;
				self.frm.show_status_message(message, "Error", timeout_ms);
			}
		});
	};

}());
