/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	//---------------------------------------------------------------------
	// form campaigns class - basic section
	function adm_form_campaigns_section_basic( params )
	{
		const success = adm_form_campaigns_section_basic.uber.constructor.call( this, params );
		if (!success) {
			return;
		}
		this.view_action_id = "view_campaigns_basic";  // view action id de permisos
		this.save_action_id  = { create: "create_campaign", update: "update_campaign_basic" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos
		this.cbo_timezone_id = "#adm_form_campaign_timezone";
		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_basic, inConcert.conf_form_section );

	if ( typeof inConcert.adm_form_campaigns_section_basic !== "function" ) {
		inConcert.adm_form_campaigns_section_basic = adm_form_campaigns_section_basic;
	}

	adm_form_campaigns_section_basic.prototype.get = function(campaign = {})
	{
		if (this.frm.is_new) {
			campaign.Id   = $("#adm_form_campaign_name").val();
			campaign.Name = $("#adm_form_campaign_name").val();
		}
		campaign.Description        = $( "#adm_form_campaign_desc").val();
		campaign.ValidationTimeZone = $( this.cbo_timezone_id ).val();
		return campaign;
	};

	adm_form_campaigns_section_basic.prototype.set = function(campaign)
	{
		$( "#adm_form_campaign_name" ).val( campaign.Name );
		$( "#adm_form_campaign_desc" ).focus().val( campaign.Description );
		$( this.cbo_timezone_id ).val( campaign.ValidationTimeZone );
	};

	adm_form_campaigns_section_basic.prototype.get_validation_def = function(definitions = [])
	{
		definitions.push(
			{
				control_id: "adm_form_campaign_name",
				messages: {
					required: "<%=_T('Please enter the campaign Name.')%>",
					id_without_dots:    "<%=_T('Only letters, numbers and dashes are allowed in the campaign Name.')%>"
				}
			},
			{
				control_id: "adm_form_campaign_desc",
				messages: {
					required: "<%=_T('Please enter the campaign description.')%>"
				}
			},
			{
				control_id: "adm_form_campaign_timezone",
				messages: {
					required: "<%=_T('Please select the timezone.')%>"
				}
			}
		);
	};

	adm_form_campaigns_section_basic.prototype.attach_load_request = function( request )
	{
		if (!$.isArray(request)) {
			return;
		}
		const self = this;
		request.push(
			//timezones
			{
				request_function: function(success_fn) {
					return inConcert.app.loadTimezones(success_fn);
				},
				success: function(response) {
					const $combo_timezone = $(self.cbo_timezone_id);
					self.frm.fill_timezones(response, $combo_timezone);
				}
			}
		);
	};

	adm_form_campaigns_section_basic.prototype.editable = function()
	{
		adm_form_campaigns_section_basic.uber.editable.call( this );
		this.frm.editable_buttons();
	};

	adm_form_campaigns_section_basic.prototype.readonly = function()
	{
		adm_form_campaigns_section_basic.uber.readonly.call( this );
		this.frm.readonly_buttons();
	};

	adm_form_campaigns_section_basic.prototype.readonly_campaign_name = function()
	{
		var $name = $("#adm_form_campaign_name");
		var $desc = $("#adm_form_campaign_desc");
		var $timezone = $("#adm_form_campaign_timezone");
		if ( this.frm.is_new ) {
			this.frm.readonly_control( $name, false /* disabled */ , true /* is_required */ );
			$name.focus();
		}
		else {
			this.frm.readonly_control( $name, true /* disabled */ );
			if(this.frm && this.frm.campaign && this.frm.campaign.Id === inConcert.app.get_internal_campaign_id()) {
				this.frm.readonly_control( $desc, true /* disabled */ );
				$timezone.focus();
			}
			else {
				$desc.focus();
			}
		}
	};

	adm_form_campaigns_section_basic.prototype.ready = function()
	{
		this.readonly_campaign_name();
	};

	adm_form_campaigns_section_basic.prototype.on_show = function()
	{
		adm_form_campaigns_section_basic.uber.on_show.call(this, {show: false});
		const is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	adm_form_campaigns_section_basic.prototype.can_jump_to_other_menunav = function()
	{
		let resp = true;
		let curr_campaign = this.get();
		curr_campaign.Name =  $("#adm_form_campaign_name").val();
		const old_campaign = this.frm && this.frm.campaign && {
				Name : this.frm.campaign.Name,
				Description : this.frm.campaign.Description,
				ValidationTimeZone : this.frm.campaign.ValidationTimeZone,
		};
			
		if (!curr_campaign || !old_campaign || inConcert.Utils.diffObjects(curr_campaign, old_campaign)) {
			resp = false;
		}
		return resp;
	};
	
}());
