/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	// TO-DO: Remove strict mode warnings
	//"use strict";

	const format = inConcert.Utils.format;
	const INCOMING = "incoming";
	const OUTGOING = "outgoing";

	var __get_chk_value = function(ctrl)
	{
		var $chk = $( "#" + ctrl.chk || "" );
		if ( !$chk.prop( "checked" ) ) {
			return -1;
		}
		var $txt = $( "#" + ctrl.txt || "" );
		var value = parseInt( $txt.val(), 10 );
		var unity = (ctrl.unity || "SECONDS").toUpperCase();
		if (unity === "SECONDS") {
			return value;
		}
		else if (unity === "MINUTES") {
			value = parseInt(value * 60, 10); // convierte de minutos a segundos
			return value;
		}
		else if (unity === "HOURS") {
			value = parseInt(value * 60 * 60); // convierte de horas a segundos
			return value;
		}
		throw("__get_chk_value - Unsupported unity: " + unity);
	};

	var __set_chk_value = function(ctrl, value )
	{
		var checked;
		if ( value === undefined ) {
			checked = false;
		}
		else {
			value = parseInt( value, 10 );
			if (value < 0) {
				checked = false;
			}
			else {
				checked = true;
				var unity = (ctrl.unity || "SECONDS").toUpperCase();
				if (unity === "MINUTES") {
					value = parseInt(value / 60, 10); // convierte de segundos a minutos
				}
				else if (unity === "HOURS") {
					value = parseInt(value / 60 / 60, 10); // convierte de segundos a horas
				}
				else if (unity !== "SECONDS") {
					throw("__set_chk_value - Unsupported unity: " + unity);
				}
				var $txt = $( "#" + ctrl.txt || "" );
				$txt.val( value );
			}
		}
		var $chk = $( "#" + ctrl.chk || "" );
		$chk.prop( "checked", checked ).trigger( "change" );
	};

	var __validate_chk_txt_value = function($chk, $txt, data)
	{
		let response = {
			success: true,
			$ctrl  : $txt
		};

		let is_disabled = $chk.is(":disabled");
		if (is_disabled) {
			return response;
		}

		let is_selected = $chk.prop("checked");
		if ( !data.required && !is_selected) {
			return response;
		}

		var validate_def = data.validate;
		var messages = validate_def.messages;
		var value = $txt.val();

		if ( value.trim().length === 0 ) {
			response.success = false;
			response.error_message = messages.EMPTY;
			return response;
		}
		var is_digits = inConcert.Utils.validate( value, "digits" );
		if ( !is_digits.success ) {
			response.success = false;
			response.error_message = messages.NUMERIC;
			return response;
		}
		// JSLINT / JSHINT: fallan pero necesito comparar por si viene un numero dentro de un string.
		/* jshint -W116 */
		if ( Math.floor( value ) != value ) {
			response.success = false;
			response.error_message = messages.INTEGER;
			return response;
		}
		/* jshint +W116 */

		var range = validate_def.range;
		value = parseInt( value, 10 );
		if ( $.isArray( range ) ) {
			var range_minor = range[ 0 ];
			var range_major = range[ 1 ];

			if ( value < range_minor || value > range_major ) {
				response.success  = false;
				response.error_message = format( messages.RANGE, range_minor, range_major );
				return response;
			}
		}
		return response;
	};

	var __disable_chk_txt_values = function( self, $chk, $value, $cmb2, $cmb3, $cmb4, default_value, default_value_cmb2 )
	{
		var disabled, is_required;
		var checked = $chk.prop( "checked" );
		var previous_value;

		if ( checked ) {
			disabled = false;
			is_required = true;
			previous_value = $chk.attr( "previous_value" );
			if ( previous_value !== undefined ) {
				$value.val( previous_value );
			}
			else {
				if ( default_value !== undefined && $value.val().trim().length === 0 ) {
					$value.val( default_value );
				}
			}
			if ( default_value_cmb2 !== undefined ) {
				$cmb2.val( default_value_cmb2 );
			}
		}
		else {
			disabled = true;
			is_required = false;
			previous_value = $value.val();
			if ( previous_value.trim().length > 0 ) {
				$chk.attr( "previous_value" , previous_value);
			}
			$value.val("");
		}
		self.parent.frm.disable_control( $value, disabled, is_required );
		self.parent.frm.disable_control( $cmb2, disabled, is_required );
		self.parent.frm.disable_control( $cmb3, disabled || $cmb2.val() !== "TransferToCampaign", $cmb2.val() === "TransferToCampaign" );
		self.parent.frm.disable_control( $cmb4, disabled || $cmb2.val() !== "TransferToCampaign" , $cmb2.val() === "TransferToCampaign" );

		$value.focus();
	};

	var __get_chk_txt_markup = function(config)
	{
		var label_chk         = config.label_chk || "";
		var label_txt         = config.label_txt || "";
		var label_info        = config.label_info;
		var id_chk            = config.chk       || "";
		var id_txt            = config.txt       || "";
		var id_txt_validation = id_txt + "_validation";
		var required          = config.required || false;
		var hr_top_visible    = config.hr_top_visible || false;

		return "".concat( 
			"<hr class='", (hr_top_visible ? "hr-col-width" : "d-none"),"' />",
			// "<div class='row'>",
				"<div class='col-3'>",
					"<div class='form-group mb-2'>",
						"<div class='form-check mt-2'>",
							"<label for='", id_chk, "' class='form-check-label form-check-label-title'>",
								"<input id='", id_chk, "' class='form-check-input' type='checkbox'  ", (required ? "hidden" : ""), " />",
								label_chk,
							"</label>",
						"</div>",
					"</div>",
					"<div class='form-group'>",
						"<label for='", id_txt, "'>", label_txt, "<%=_T(':')%>", "</label>",
						(label_info ? ("<span class='fa fa-question-circle' title='" + label_info + "'></span>") : ""),
						"<input id='", id_txt, "' class='form-control' type='text' ", (required ? "" : "disabled='true'"), " />",
						"<i id='", id_txt_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
			// "</div>"
		);
	};

	var __get_chk_txt_cmb_markup = function(config)
	{
		var label_chk                   = config.label_chk  || "";
		var label_txt                   = config.label_txt  || "";
		var id_chk                      = config.chk        || "";
		var label_cmb2                  = config.label_cmb2 || "";
		var id_cmb2                     = config.cmb2       || "";
		var label_cmb3                  = config.label_cmb3 || "";
		var id_cmb3                     = config.cmb3       || "";
		var label_cmb4                  = config.label_cmb4 || "";
		var id_cmb4                     = config.cmb4       || "";
		var id_txt                      = config.txt        || "";
		var id_txt_validation           = id_txt + "_validation";
		var required                    = config.required   || false;
		var id_cmb2_validation          = id_cmb2 + "_validation";
		var id_cmb3_validation          = id_cmb3 + "_validation";
		var id_cmb4_validation          = id_cmb4 + "_validation";
		var id_cmb2_disclaimer          = id_cmb2 + "_disclaimer";

		return "".concat( 
			"<hr class='hr-col-width' />",
			"<div class='col-12'>",
			"<div class='row mt-2'>",
				"<div class='col-6'>",
					"<div class='form-group'>",
						"<div class='form-check mt-0'>",
							"<label for='", id_chk, "' class='form-check-label form-check-label-title'>",
								"<input id='", id_chk, "' class='form-check-input' type='checkbox' ", (required ? "hidden" : ""), " />",
								label_chk,
							"</label>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='row'>",
				"<div class='col-6'>",
					"<div class='form-group'>",
						"<label for='", id_txt, "' >", label_txt, "<%=_T(':')%>", "</label>",
						"<input id='", id_txt, "' class='form-control' type='text' ", (required ? "" : "disabled='true'"), " />",
						"<i id='", id_txt_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
				"<div class='col-6'>",
					"<div class='form-group'>",
						"<label for='", id_cmb2, "' >", label_cmb2, "<%=_T(':')%>", "</label>",
						"<select id='", id_cmb2, "' class='form-control'></select>",
						"<i id='", id_cmb2_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
						"<div id='", id_cmb2_disclaimer, "' class='formDataDisclaimer explanation' style='display:none;'><%=_T([[This value will overwrite the accounts configuration option 'Voicemail On Timeout'.]])%></div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='row' style='display:", config.cmb3 && config.cmb4 ? "" : "none","'>",
				"<div class='col-6'>",
					"<div class='form-group'>",
						"<label for='", id_cmb3, "' >", label_cmb3, "<%=_T(':')%>", "</label>",
						"<select id='", id_cmb3, "' class='form-control'></select>",
						"<i id='", id_cmb3_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
				
				"<div class='col-6'>",
				"<div class='form-group'>",
						"<label for='", id_cmb4, "' >", label_cmb4, "<%=_T(':')%>", "</label>",
						"<select id='", id_cmb4, "' class='form-control'></select>",
						"<i id='", id_cmb4_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
			"</div></div>"
		);
	};

	var __get_txt_markup = function(config)
	{
		var label_chk         = config.label_chk || "" ;
		var label_txt         = config.label_txt || "" ;
		var id_chk            = config.chk       || "" ;
		var id_txt            = config.txt       || "" ;
		var id_txt_validation = id_txt + "_validation" ;
		var hr_visible	=  config.hr_visible || false;

		// Cuando el atributo siempre es requerido, el chk no se debe de visualizar, pero sigue estando para que el resto del codigo funcione con la misma logica.
		return "".concat( 
			// "<div class='row'>",
				"<div class='col-3' hidden>",
					"<div class='form-group'>",
						"<div class='form-check form-check-right-input'>",
							"<label for='", id_chk, "' class='form-check-label form-check-label-title'>",
								"<input id='", id_chk, "' class='form-check-input' type='checkbox' hidden />",
								label_chk,
							"</label>",
						"</div>",
					"</div>",
				"</div>",

				"<div class='col-6'>",
					"<div class='form-group'>",
						"<label for='", id_txt, "' class='required'>", label_txt, "<%=_T(':')%>", "</label>",
						"<input id='", id_txt, "' class='form-control' type='text' disabled='true' />",
						"<i id='", id_txt_validation, "' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
			// "</div>",
			"<hr class='", (hr_visible ? "hr-col-width" : "d-none"),"' />"
		);
	};

	var __get_actions_on_acd_wait_timeout = function(config)
	{
		var actions = [];
		if(config && config.proxy && config.proxy !== "CALL" && config.proxy !== "CHAT") {
			actions.push({ 
				id: "Enqueue", 
				name: "<%=_T('Enqueue')%>",
			});
		}
		actions.push({ 
			id: "Finalize", 
			name: "<%=_T('Finalize Conversation')%>",
		});
		actions.push({ 
			id: "TransferToCampaign", 
			name: "<%=_T('Transfer to campaign')%>",
		});
		return actions;
	};

	var __get_actions_on_inactivity_time = function(config)
	{
		
		const proxy = (config && config.proxy) || "";

		const actions_by_channel = {
			MAIL: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "TransferToCampaign", name: "<%=_T('Transfer to campaign')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			FACEBOOK: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			INSTAGRAM: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			WHATSAPP: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "TransferToCampaign", name: "<%=_T('Transfer to campaign')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			RCS: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "TransferToCampaign", name: "<%=_T('Transfer to campaign')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			WEBCONTACT: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			YOUTUBE: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			APPSTORE: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			GOOGLEPLAY: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			TWITTER: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
			SMS: [
				{ id: "Enqueue", name: "<%=_T('Enqueue')%>"},
				{ id: "Finalize", name: "<%=_T('Finalize Conversation')%>"},
				{ id: "TransferToCampaign", name: "<%=_T('Transfer to campaign')%>"},
				{ id: "Waiting", name: "<%=_T('Reassign')%>"}
			],
		};
		let actions = actions_by_channel[proxy] || []; 
		return actions;
	};

	var __get_campaigns_to_transfer = function(fromCampaignId, proxyType)
	{
		var campaignsToTransfer = [];
		var allCampaigns = inConcert.app.getCampaigns();
		$.each(allCampaigns, function(_, campaign) {
			if(fromCampaignId !== campaign.Id) {
				$.each(campaign.accounts, function(_, account) {
					if (account.proxyType === proxyType) {
						campaignsToTransfer.push(campaign);
						return false;
					}
				});
			}
		});
		return campaignsToTransfer;
	};

	var __get_accounts_to_transfer = function(toCampaignId, proxyType)
	{
		var accountsToTransfer = [];
		var allCampaigns = inConcert.app.getCampaigns();
		$.each(allCampaigns, function(_, campaign) {
			if(campaign.Id === toCampaignId) {
				$.each(campaign.accounts, function(_, account) {
					if (account.proxyType === proxyType) {
						accountsToTransfer.push(account);
					}
				});
			}
		});
		return accountsToTransfer;
	};

	/*
		Clase BASE 'account_config'
	*/
	function account_config( params )
	{
		this.parent = params.parent;
		this.capability_handle = null;
		if ( !params.only_config_thresholds && params.capability ) {
			this.capability_handle = new inConcert.adm_capability_handle_override({
				override_type : "CAMPAIGN",
				container_id  : params.capability.container_id,
				prefix_id     : params.capability.prefix_id,
				parent_frm    : this.parent,
				parent_type   : "FORM-SECTION",
				proxies       : params.capability.proxies
			});
		}
		this.create_markup();
		this.set_events();
		this.load_data();
		this.th_internal_tabs = null;
	}

	account_config.prototype.edit_from_channel_section = function() { this.editable(); };

	account_config.prototype.editable = function( internal_tab, account_id )
	{
		var frm = this.parent.frm;

		// capability
		if ( this.capability_handle ) {
			this.capability_handle.editable();
		}

		// thresholds
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data )
			{
				if (internal_tab === undefined || internal_tab === data.tab ) {
					if ( data.ctrl_type === "chk" ) {
						var $chk = $( "#" + data.chk || "" );
						var $txt = $( "#" + data.txt || "" );
						let threshold_value = $txt.val();

						var checked;
						if ( data.required ) {
							checked = true;
							$chk.prop( "checked", true );
						}
						else {
							checked = $chk.prop( "checked" );
						}

						let disabled_txt = !checked;
						if (account_id && threshold_value) {
							frm.disable_control( $chk, true );
							frm.disable_control( $txt, true, true, true, true );
						}
						else {

							frm.disable_control( $chk, false );
							frm.disable_control( $txt, disabled_txt );
						}
						
						if(checked) {
							if (data.cmb2) {
								const $cmb2 = $( "#" + data.cmb2 || "" );
								const $cmb3 = $( "#" + data.cmb3 || "" );
								const $cmb4 = $( "#" + data.cmb4 || "" );
								frm.disable_control( $cmb2, disabled_txt, undefined /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
								if(!disabled_txt) {
									let cmb2Val = $cmb2.val();
									if(cmb2Val !== "TransferToCampaign") {
										frm.disable_control( $cmb3, true, true /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
										frm.disable_control( $cmb4, true, true /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
									}
									else {
										frm.disable_control( $cmb3, disabled_txt, undefined /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
										frm.disable_control( $cmb4, disabled_txt, undefined /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
									}
								}
								else {
									frm.disable_control( $cmb3, disabled_txt, undefined /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
									frm.disable_control( $cmb4, disabled_txt, undefined /*is_required*/, false /*is_disable_lbl*/, true /*keep_value*/);
								}
							}
						}
						else {
							if (data.cmb2) {
								const $cmb2 = $( "#" + data.cmb2 || "" );
								const $cmb3 = $( "#" + data.cmb3 || "" );
								const $cmb4 = $( "#" + data.cmb4 || "" );
								frm.disable_control( $cmb2, true);
								frm.disable_control( $cmb3, true);
								frm.disable_control( $cmb4, true);
							}
						}
					}
					else if ( data.ctrl_type === "cbo" ) {
						var $cbo = $( "#" + data.cbo || "" );
						frm.disable_control( $cbo, false );
					}
				}
			});
	};

	account_config.prototype.readonly = function( internal_tab )
	{
		var frm = this.parent.frm;

		// capability
		if ( this.capability_handle ) {
			this.capability_handle.readonly();
		}

		// thresholds
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data )
			{
				if (internal_tab === undefined || internal_tab === data.tab ) {
					if ( data.ctrl_type === "chk" ) {
						var $chk = $( "#" + data.chk || "" );
						var $txt = $( "#" + data.txt || "" );
						frm.readonly_control( $chk, true /*readonly*/ );
						frm.readonly_control( $txt, true /*readonly*/ );
						if (data.cmb2) {
							var $cmb2 = $( "#" + data.cmb2 || "" );
							var $cmb3 = $( "#" + data.cmb3 || "" );
							var $cmb4 = $( "#" + data.cmb4 || "" );
							frm.readonly_control( $cmb2, true /*readonly*/ );
							frm.readonly_control( $cmb3, true /*readonly*/ );
							frm.readonly_control( $cmb4, true /*readonly*/ );
						}
					}
					else if ( data.ctrl_type === "cbo" ) {
						var $cbo = $( "#" + data.cbo || "" );
						frm.readonly_control( $cbo, true /*readonly*/ );
					}
				}
			});
	};

	account_config.prototype.disable = function(internal_tab)
	{
		// capability
		if ( this.capability_handle ) {
			this.capability_handle.readonly();
		}
		// thresholds
		const has_allegro_integration = inConcert.app.has_allegro_integration();
		if (has_allegro_integration && this.type === "CALL") {
			//las cuentas de telefonia de Allegro se manejan diferentes internamemnte para saber si hay que validarlas
			const frm = this.parent.frm;
			$.each( this.tab_ctrls.thresholds || {},
				function( _, data )
				{
					if (internal_tab === undefined || internal_tab === data.tab) {
						if ( data.ctrl_type === "chk" ) {
							const $chk = $( "#" + data.chk || "" );
							const $txt = $( "#" + data.txt || "" );
							frm.disable_control( $chk, true /*readonly*/ );
							if (!data.required) {
								frm.disable_control( $txt, true /*readonly*/ );
							}
							if (data.cmb2) {
								const $cmb2 = $( "#" + data.cmb2 || "" );
								const $cmb3 = $( "#" + data.cmb3 || "" );
								const $cmb4 = $( "#" + data.cmb4 || "" );
								frm.disable_control( $cmb2, true /*readonly*/ );
								frm.disable_control( $cmb3, true /*readonly*/ );
								frm.disable_control( $cmb4, true /*readonly*/ );
							}
						}
						else if ( data.ctrl_type === "cbo" ) {
							const $cbo = $( "#" + data.cbo || "" );
							frm.disable_control( $cbo, true /*readonly*/ );
						}
					}
				}
			);
		}
	};

	account_config.prototype.create_markup = function()
	{
		let self = this;
		var $tab_section = $( "#" + this.section_id );
		if ( $tab_section.length === 0 ) {
			throw( "Error - account_config.prototype.create_markup: tab_section thresholds container '" + this.section_id || "" + "' does not exist");
		}

		// thresholds configuration
		var $containers = {};
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data ) {
				var $th_markup = $(data.get_markup( data ));
				var container_id = data.tab || "true"; // el true es un valor cuando no se encuentran TABs
				if ( !$containers[container_id] ) {
					$containers[container_id] = $tab_section.find( "[thresholds_container='" + container_id + "']" );
					$containers[container_id].html("");
				}
				$containers[container_id].append($th_markup);
				if(data.cmb2 && typeof data.get_actions  === "function") {
					var $combo = $containers[container_id].find("#" + data.cmb2 );
					var actions = data.get_actions(data);
					self.parent.frm.fill_combo( $combo, false /*is_blank_first_item*/, actions, "id" /*attr_id*/, "name" /*attr_name*/, false /*sorted*/ );
				}
			});

		// capability configuration
		if ( this.capability_handle ) {
			this.capability_handle.initialize();
		}
	};

	account_config.prototype.__can_get_value = function( /*ctrl*/ ) { return true; };

	account_config.prototype.get = function()
	{
		var self = this;
		var data = {  capability: {}, thresholds: {} };

		// obtengo los capability
		if ( this.capability_handle ) {
			data.capability = this.capability_handle.get();
		}

		// obtengo los thresholds
		$.each( this.tab_ctrls.thresholds || {},
			function( _, ctrl ) {
				if (self.__can_get_value(ctrl)) {
					if ( ctrl.ctrl_type === "chk" ) {
						data.thresholds[ ctrl.field ] = __get_chk_value(ctrl);
					}
					else if ( ctrl.ctrl_type === "cbo" ) {
						var $cbo = $( "#" + ctrl.cbo || "" );
						data.thresholds[ ctrl.field ] = parseInt( $cbo.val(), 10 );
					}
				}
			}
		);

		return data;
	};

	account_config.prototype.set_current_config = function(account_id)
	{
		let self = this;
		var campaign = this.parent.frm.get_entity_instance();
		if ( !campaign ) {
			return;
		}
		var proxy_type = this.type;

		// thresholds set data
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data ) {
				var threshold_value = campaign.getThreshold( proxy_type, data.field );
				if ( data.ctrl_type === "chk" ) {
					let isACDWaitThreshold = data.field === "ACDWaitThreshold" || data.field === "ACDWaitThreshold_INBOUND";
					let isInactivityTime = data.field === "InactivityTimeout";
					__set_chk_value( data, threshold_value );
					if ((isInactivityTime || isACDWaitThreshold) && data.cmb2) {
						//Recorremos la lista de accounts del tipo que corresponde al canal que estamos evaluando
						$.each( campaign.accounts, function( _, account) {
							if (account.proxyType === proxy_type && (account_id === undefined || account_id === account.id))
							{
								//Setear el combo data.cmb2 con el valor que tenga en ActionOnAcdWaitTimeout ese account
								var $cmb2 = $( "#" + data.cmb2 || "" );
								var $cmb3 = $( "#" + data.cmb3 || "" );
								var $cmb4 = $( "#" + data.cmb4 || "" );
								var $cmb2_disclaimer = $( "#" + data.cmb2 + "_disclaimer" || "" );
								
								if (isACDWaitThreshold) {
									$cmb2.val(account.ActionOnAcdWaitTimeout ? account.ActionOnAcdWaitTimeout.action : "");
									if (account.ActionOnAcdWaitTimeout && account.ActionOnAcdWaitTimeout.action === "TransferToCampaign") {
										self.parent.frm.disable_control($cmb3, false, true);
										self.parent.frm.disable_control($cmb4, false, true);
										let campaignsToTransfer = __get_campaigns_to_transfer(self.parent.frm.campaign.Id, self.type);
										self.parent.frm.fill_combo($cmb3, false /*is_blank_first_item*/, campaignsToTransfer, "Id" /*attr_id*/, "Name" /*attr_name*/, true /*sorted*/);
										$cmb3.val(account.ActionOnAcdWaitTimeout.campaign);
										if (account.ActionOnAcdWaitTimeout.campaign) {
											let accountsToTransfer = __get_accounts_to_transfer(account.ActionOnAcdWaitTimeout.campaign, self.type);
											self.parent.frm.fill_combo($cmb4, false /*is_blank_first_item*/, accountsToTransfer, "id" /*attr_id*/, "name" /*attr_name*/, true /*sorted*/);
											$cmb4.val(account.ActionOnAcdWaitTimeout.account);
										}
										if (data.proxy === "CALL") {
											$cmb2_disclaimer.show();
										}
									}
									else {
										self.parent.frm.disable_control($cmb3, true);
										self.parent.frm.disable_control($cmb4, true);
										if (data.proxy === "CALL") {
											$cmb2_disclaimer.hide();
										}
									}
								}
								else if (isInactivityTime) {
									const InactivityTimeAction = (account.InactivityTimeout && account.InactivityTimeout.action) || "";
									$cmb2.val(InactivityTimeAction);
									if (InactivityTimeAction === "TransferToCampaign") {
										self.parent.frm.disable_control($cmb3, false, true);
										self.parent.frm.disable_control($cmb4, false, true);
										let campaignsToTransfer = __get_campaigns_to_transfer(self.parent.frm.campaign.Id, self.type);
										self.parent.frm.fill_combo($cmb3, false /*is_blank_first_item*/, campaignsToTransfer, "Id" /*attr_id*/, "Name" /*attr_name*/, true /*sorted*/);
										let timeout_campaign = account.InactivityTimeout.campaign;
										let timeout_account = account.InactivityTimeout.account;
										$cmb3.val(timeout_campaign);
										if (timeout_campaign) {
											let accountsToTransfer = __get_accounts_to_transfer(timeout_campaign, self.type);
											self.parent.frm.fill_combo($cmb4, false /*is_blank_first_item*/, accountsToTransfer, "id" /*attr_id*/, "name" /*attr_name*/, true /*sorted*/);
											$cmb4.val(timeout_account);
										}
									}
									else {
										self.parent.frm.disable_control($cmb3, true);
										self.parent.frm.disable_control($cmb4, true);
									}
								}
								
							}
						});
					}
				}
				else if ( data.ctrl_type === "cbo" ) {
					var $cbo = $( "#" + data.cbo || "" );
					$cbo.val( threshold_value || "");
				}
			});

		// capability set data
		if ( this.capability_handle ) {
			this.capability_handle.load_from_campaign();
		}
	};

	account_config.prototype.enable = function()
	{
		$( "#adm_form_campaign_channels" ).find( "." + this.handler_id ).removeClass( "formTabsMenuItem-ds" ).show();
	};

	account_config.prototype.is_enabled = function()
	{
		return !$( "#adm_form_campaign_channels" ).find( "." + this.handler_id ).hasClass( "formTabsMenuItem-ds" );
	};

	account_config.prototype.set_events = function()
	{
		var self = this;

		// thresholds events
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data ) {
				if ( data.ctrl_type === "chk" && !data.required ) {
					var $chk = $( "#" + data.chk || "" );
					var $txt = $( "#" + data.txt || "" );
					var $cmb2 = $( "#" + data.cmb2 || "" );
					var $cmb3 = $( "#" + data.cmb3 || "" );
					var $cmb4 = $( "#" + data.cmb4 || "" );
					var $cmb2_disclaimer = $( "#" + data.cmb2 + "_disclaimer" || "" );
					$chk
						.off("change")
						.on("change", function () {
							__disable_chk_txt_values( self, $(this), $txt, $cmb2, $cmb3, $cmb4, "", "Finalize" );
							if(self.tab_ctrls && self.tab_ctrls.thresholds && self.tab_ctrls.thresholds.answer && self.tab_ctrls.thresholds.answer.chk === $(this)[0].id && self.after_answer_time_chk_change && typeof self.after_answer_time_chk_change === "function") {
								self.after_answer_time_chk_change($(this));
							}
						});
					$cmb2
						.off("change")
						.on("change", function () {
							if ($(this).val() === "TransferToCampaign") {
								self.parent.frm.disable_control( $cmb3, false, true );
								self.parent.frm.disable_control( $cmb4, false, true );
								let campaignsToTransfer = __get_campaigns_to_transfer(self.parent.frm.campaign.Id, self.type);
								self.parent.frm.fill_combo( $cmb3, false /*is_blank_first_item*/, campaignsToTransfer, "Id" /*attr_id*/, "Name" /*attr_name*/, true /*sorted*/ );
								if($cmb3.val()) {
									let accountsToTransfer = __get_accounts_to_transfer($cmb3.val(), self.type);
									self.parent.frm.fill_combo( $cmb4, false /*is_blank_first_item*/, accountsToTransfer, "id" /*attr_id*/, "name" /*attr_name*/, true /*sorted*/ );
								}
								if (data.proxy === "CALL") {
									$cmb2_disclaimer.show();
									$("#adm_form_campaign_accounts_call_voicemail_timeout_incoming").prop("disabled", true);
								}
							}
							else {
								self.parent.frm.disable_control( $cmb3, true );
								self.parent.frm.disable_control( $cmb4, true );
								if(data.proxy === "CALL") {
									$cmb2_disclaimer.hide();
									$("#adm_form_campaign_accounts_call_voicemail_timeout_incoming").prop("disabled", false);
								}
							}
						});
					$cmb3
						.off("change")
						.on("change", function () {
							let accountsToTransfer = __get_accounts_to_transfer($cmb3.val(), self.type);
							self.parent.frm.fill_combo( $cmb4, false /*is_blank_first_item*/, accountsToTransfer, "id" /*attr_id*/, "name" /*attr_name*/, true /*sorted*/ );
						});
				}
			});
	};

	account_config.prototype.can_validate_item = function(/*data, account_config*/)
	{
		return true;
	};

	account_config.prototype.validate = function(account_config = {})
	{
		const self = this;
		let result = { success: false };

		if ( this.capability_handle && !this.capability_handle.validate() ) {
			result.builtin_msg_err = true;
			return result;
		}

		const __mark_error = function( owner_id, tab_container ) {
			if (tab_container && self.th_internal_tabs) {
				self.th_internal_tabs.switch_tab( tab_container );
			}
			const $mark = $( "#" + owner_id + "_validation" );
			if ( $mark.length > 0 ) {
				$mark.show();
			}
		};

		//Guardamos el estado de la opcion ActionOnAcdWaitTimeout
		let ActionOnAcdWaitTimeout = {};
		let InactivityTimeout = {};
		$.each( this.tab_ctrls.thresholds || {},
			function( _, data )
			{
				if (!self.can_validate_item(data, account_config)) {
					result = {success: true};
					return true; // continue
				}
				if ( data.ctrl_type === "chk" ) {
					const $chk = $( "#" + data.chk || "" );
					const $txt = $( "#" + data.txt || "" );
					const isACDWaitThreshold = data.field === "ACDWaitThreshold" || data.field === "ACDWaitThreshold_INBOUND";
					const isInactivityTime = data.field === "InactivityTimeout";
					if ( (isACDWaitThreshold || isInactivityTime) && $chk.prop("checked") && data.cmb2) {
						const $cmb2 = $( "#" + data.cmb2 || "" );
						const $cmb3 = $( "#" + data.cmb3 || "" );
						const $cmb4 = $( "#" + data.cmb4 || "" );
						const action = $cmb2.val() || undefined;
						const campaign = $cmb3.val() || undefined;
						const account = $cmb4.val() || undefined;
						
						if (isACDWaitThreshold) {
							ActionOnAcdWaitTimeout.action   = action;
							ActionOnAcdWaitTimeout.campaign = campaign;
							ActionOnAcdWaitTimeout.account  = account;
							ActionOnAcdWaitTimeout.initCfg  = (!$chk.is(":disabled") && $chk.prop("checked")) ? true : false; // #18912 To check if is initial config and must apply to all accounts in the channel.
						}
						else{
							InactivityTimeout.action   = action;
							InactivityTimeout.campaign = campaign;
							InactivityTimeout.account  = account;
							InactivityTimeout.initCfg  = (!$chk.is(":disabled") && $chk.prop("checked")) ? true : false;      // #18912 To check if is initial config and must apply to all accounts in the channel.
						}
						
						if(!action || action === "") {
							result.success = false;
							result.$ctrl = $cmb2;
							result.error_message = data.validate.messages.ACTION;
							__mark_error( data.cmb2, data.tab );
							return false; // break
						}
						if(action === "TransferToCampaign" && (!campaign|| campaign === "")) {
							result.success = false;
							result.$ctrl = $cmb3;
							result.error_message = data.validate.messages.CAMPAIGN;
							__mark_error( data.cmb3, data.tab );
							return false; // break
						}
						if(action === "TransferToCampaign" && (!account || account === "")) {
							result.success = false;
							result.$ctrl = $cmb4;
							result.error_message = data.validate.messages.ACCOUNT;
							__mark_error( data.cmb4, data.tab );
							return false; // break
						}
					}

					result = __validate_chk_txt_value( $chk, $txt, data );
					if ( !result.success ) {
						__mark_error( data.txt, data.tab );
						return false; // break
					}
				}
				else if ( data.ctrl_type === "cbo" ) {
					const $cbo = $( "#" + data.cbo || "" );
					if ( $cbo.length > 0 && !$cbo.is(":disabled") ) {
						const value = $cbo.val();
						if ( value.trim().length === 0 ) {
							result.success = false;
							result.$ctrl = $cbo;
							result.error_message = data.validate.messages.EMPTY;
							__mark_error( data.cbo, data.tab );
							return false; // break
						}
					}
				}
			});

		//Devolvemos el ActionOnAcdWaitTimeout en el result
		result.ActionOnAcdWaitTimeout = ActionOnAcdWaitTimeout;
		result.InactivityTimeout = InactivityTimeout;
		return result;
	};

	account_config.prototype.select = function()
	{
		this.parent.hide_channels_sections();

		$( "#adm_form_campaign_channels" ).find( "." + this.handler_id )
			.addClass( "tabsMenuItemSelect" );
			// .removeClass( "formTabsMenuItem-ds" )
			// .prop("disabled", false);

		$( "#" + this.section_id ).show();
	};

	account_config.prototype.load_data = $.noop;

	account_config.prototype.enable_internal_tab = function( tab_id )
	{
		if (!this.th_internal_tabs) {
			return;
		}
		this.th_internal_tabs.enable_tab(tab_id);
	};

	account_config.prototype.disable_internal_tab = function( tab_id )
	{
		if (!this.th_internal_tabs) {
			return;
		}
		this.th_internal_tabs.disable_tab(tab_id);
	};

	account_config.prototype.show_internal_tab = function( tab_id )
	{
		if (!this.th_internal_tabs) {
			return;
		}
		this.enable_internal_tab(tab_id);
		this.th_internal_tabs.show_tab_with_content(tab_id);
	};

	account_config.prototype.hide_internal_tab = function( tab_id )
	{
		if (!this.th_internal_tabs) {
			return;
		}
		this.disable_internal_tab(tab_id);
		this.th_internal_tabs.hide_tab_with_content(tab_id);
	};

	account_config.prototype.switch_internal_tab = function(tab_id, callback = $.noop)
	{
		if (!this.th_internal_tabs) {
			return;
		}
		this.th_internal_tabs.switch_tab(tab_id, callback);
	};

/*
  Clase - 'Call_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Call_config( params )
	{
		this.type       = "CALL";
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_call";
		let pf          = this.prefix_id;
		this.section_id = pf + "_panel";
		this.handler_id = "tabsMenuItemPhone";
		const callbacks = params.callbacks || {};
		const all_tabs_hidden = params.all_tabs_hidden || false;

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_call_capability_config",
			prefix_id    : "adm_form_campaign_channels_call",
			proxies      : { VOICEMAIL : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
			// entrante ---------------------------------
			//
				service_level_inbound: {
					tab        : INCOMING,
					ctrl_type  : "chk",
					chk        : pf + "_service_level_time_inbound",
					txt        : pf + "_service_level_time_inbound_time",
					label_chk  : "<%=_T('Service Level')%>",
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel_INBOUND",
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call service level time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer service level time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric service level time.')%>",
							RANGE   : "<%=_T('Please enter incoming call service level time between {0} and {1}.')%>"
						}
					},
					hr_visible	: true,
					get_markup : __get_txt_markup
				},
				ghost_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_ghost_inbound",
					txt       : pf + "_ghost_time_inbound",
					label_chk : "<%=_T('Ghost Threshold')%>",
					label_txt : "<%=_T('Ghost Threshold')%> (<%=_T('seconds')%>)",
					field     : "Ghost_INBOUND",
					//required  : true,
					validate  : {
						range    : [ 1, 5 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call ghost time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer ghost time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric ghost time.')%>",
							RANGE   : "<%=_T('Please enter incoming call ghost time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				inactivity_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk    : pf + "_inactivity_inbound",
					txt    : pf + "_inactivity_time_inbound",
					label_chk  : "<%=_T('Inactivity Threshold')%>",
					label_txt  : "<%=_T('Inactivity Threshold')%> (<%=_T('seconds')%>)",
					field  : "InactivityTimeout_INBOUND",
					validate  : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call inactivity time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer inactivity time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric inactivity time.')%>",
							RANGE   : "<%=_T('Please enter incoming call inactivity time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				long_interaction_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_long_interaction_inbound",
					txt       : pf + "_long_interaction_time_inbound",
					label_chk : "<%=_T('Long Conversation')%>",
					label_txt : "<%=_T('Long conversation')%> (<%=_T('seconds')%>)",
					field     : "LongInteraction_INBOUND",
					//required  : true,
					validate  : {
						range    : [ 30, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call long conversation time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer long conversation time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric long conversation time.')%>",
							RANGE   : "<%=_T('Please enter incoming call long conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				short_interaction_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_short_interaction_inbound",
					txt       : pf + "_short_interaction_time_inbound",
					label_chk : "<%=_T('Short Conversation')%>",
					label_txt : "<%=_T('Short conversation')%> (<%=_T('seconds')%>)",
					field     : "ShortInteraction_INBOUND",
					//required  : true,
					validate  : {
						range    : [ 6, 20 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call short conversation time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer short conversation time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric short conversation time.')%>",
							RANGE   : "<%=_T('Please enter incoming call short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				acd_inbound: {
					proxy: "CALL",
					tab       : INCOMING,
					ctrl_type : "chk",
					chk    : pf + "_acd_wait_inbound",
					txt    : pf + "_acd_wait_time_inbound",
					label_chk  : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt  : "<%=_T('ACD wait Threshold')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field  : "ACDWaitThreshold_INBOUND",
					validate  : {
						range    : [ 10, 10800 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call wait message time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter incoming call wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				acd_first_message_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk    : pf + "_acd_first_message_inbound",
					txt    : pf + "_acd_first_message_time_inbound",
					label_chk  : "<%=_T('Wait First Message')%>",
					label_txt  : "<%=_T('Wait First Message')%> (<%=_T('seconds')%>)",
					field  : "ACDFirstMessageThreshold_INBOUND",
					validate  : {
						range    : [ 0, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call first message time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer first message time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric first message time.')%>",
							RANGE   : "<%=_T('Please enter incoming call first message time between {0} and {1}.')%>",
						}
					},
					hr_top_visible	: true,
					get_markup : __get_chk_txt_markup
				},
				acd_repeated_message_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk    : pf + "_acd_repeated_message_inbound",
					txt    : pf + "_acd_repeated_message_time_inbound",
					label_chk  : "<%=_T('Wait Repeated Msg')%>",
					label_txt  : "<%=_T('Wait repeated message')%> (<%=_T('seconds')%>)",
					field  : "ACDRepeatedMessageThreshold_INBOUND",
					validate  : {
						range    : [ 10, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call repeated message time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer repeated message time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric repeated message time.')%>",
							RANGE   : "<%=_T('Please enter incoming call repeated message time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
			// saliente ---------------------------------
			//
				service_level_outbound: {
					tab        : OUTGOING,
					ctrl_type  : "chk",
					chk        : pf + "_service_level_time_outbound",
					txt        : pf + "_service_level_time_outbound_time",
					label_chk  : "<%=_T('Service Level')%>",
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel_OUTBOUND",
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call service level time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer service level time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric service level time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call service level time between {0} and {1}.')%>"
						}
					},
					hr_visible	: true,
					get_markup : __get_txt_markup
				},
				average_duration_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_average_duration_outbound",
					txt       : pf + "_average_duration_time_outbound",
					label_chk : "<%=_T('Average Duration')%>",
					label_txt : "<%=_T('Average duration')%> (<%=_T('seconds')%>)",
					field     : "AverageCallDuration_OUTBOUND",
					//required  : true,
					validate  : {
						range    : [ 20, 900 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call average duration time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer average duration time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric average duration time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call average duration time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				long_interaction_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_long_interaction_outbound",
					txt       : pf + "_long_interaction_time_outbound",
					label_chk : "<%=_T('Long Conversation')%>",
					label_txt : "<%=_T('Long conversation')%> (<%=_T('seconds')%>)",
					field     : "LongInteraction_OUTBOUND",
					//required  : true,
					validate  : {
						range    : [ 30, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call long conversation time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer long conversation time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric long conversation time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call long conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				short_interaction_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_short_interaction_outbound",
					txt       : pf + "_short_interaction_time_outbound",
					label_chk : "<%=_T('Short Conversation')%>",
					label_txt : "<%=_T('Short conversation')%> (<%=_T('seconds')%>)",
					field     : "ShortInteraction_OUTBOUND",
					//required  : true,
					validate  : {
						range : [ 6, 20 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call short conversation time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer short conversation time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric short conversation time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				}
			}
		};

		Call_config.uber.constructor.call( this, params );

		this.th_internal_tabs = new inConcert.conf_form_tab({
			parent      : this            ,
			frm         : this.parent.frm ,
			container_id: pf + "_tabs"    ,
			all_tabs_hidden: all_tabs_hidden,
			callbacks: callbacks,
			tabs_list   : {
				incoming: {
					id               : "incoming",
					name             : "<%=_T('Incoming')%>",
					tab_container_id : pf + "_tab_incoming",
					selected         : true,
					initial_state    : "disabled"
				},
				outgoing : {
					id               : "outgoing",
					name             : "<%=_T('Outgoing')%>",
					tab_container_id : pf + "_tab_outgoing",
					initial_state    : "disabled"
				}
			}
		});

		this.th_internal_tabs.show_tabs_container();
	}

	inConcert.Utils.inherit( Call_config, account_config );

	Call_config.prototype.can_validate_item = function(data, account_config)
	{
		if (data.tab === INCOMING && (account_config.incoming_config && !account_config.incoming_config.enabled)) {
			return false;
		}
		if (data.tab === OUTGOING && (account_config.outgoing_config && !account_config.outgoing_config.enabled)) {
			return false;
		}
		return true;
	};

	Call_config.prototype.edit_from_channel_section = function() 
	{
		const self = this;

		var ds_all_tabs = function() {
			self.th_internal_tabs.enable_disable_all_tabs( false /*enabled*/ );
		};

		let campaign = this.parent.frm.campaign;
		if (!campaign) {
			ds_all_tabs();
			return;
		}
		let accounts = campaign.accounts;
		if (!$.isArray(accounts) ) {
			ds_all_tabs();
			return;
		}
		let call_accounts = [];
		for (let i = 0; i < accounts.length; i++) {
			let current_account = accounts[i];
			if (current_account.proxyType === "CALL") {
				call_accounts.push(current_account);
			}
		}
		if (call_accounts.length === 0) {
			ds_all_tabs();
			return;
		}

		let is_enabled_incoming_config = false;
		let is_enabled_outgoing_config = false;

		for (let i = 0; i < call_accounts.length; i++) {
			let call_account = call_accounts[i];
			if (call_account.incomingConfig) {
				is_enabled_incoming_config = call_account.incomingConfig.enabled;
			}
			if (call_account.outgoingConfig) {
				is_enabled_outgoing_config = call_account.outgoingConfig.enabled;
			}
			if (is_enabled_outgoing_config && is_enabled_outgoing_config) {
				break;
			}
		}

		if (!is_enabled_incoming_config && !is_enabled_outgoing_config) {
			ds_all_tabs();
			return;
		}
		let is_selected_tab = false;
		if (is_enabled_incoming_config) {
			this.enable_internal_tab( INCOMING );
			this.editable( INCOMING );
			this.switch_internal_tab(INCOMING);
			is_selected_tab = true;
		}
		else {
			this.disable_internal_tab( INCOMING );
			this.disable( INCOMING );
		}

		if (is_enabled_outgoing_config) {
			this.enable_internal_tab( OUTGOING );
			this.editable( OUTGOING );
			if (!is_selected_tab) {
				this.switch_internal_tab(OUTGOING);
			}
		}
		else {
			this.disable_internal_tab( OUTGOING );
			this.disable( OUTGOING );
		}
		this.th_internal_tabs.select_first_visible_tab();
	};

	Call_config.prototype.__can_get_value = function(ctrl)
	{
		var tab_id = ctrl.tab;
		var is_enabled_tab = this.th_internal_tabs.is_enabled_tab(tab_id);
		return is_enabled_tab;
	};

/*
  Clase - 'Call_allegro_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Call_allegro_config( params )
	{
		this.type       = "CALL";
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_call_allegro";
		var pf          = this.prefix_id;
		this.section_id = pf + "_panel";
		this.handler_id = "tabsMenuItemPhone";
		var callbacks = params.callbacks || {};
		var all_tabs_hidden = params.all_tabs_hidden || false;

		this.tab_ctrls = {
			// thresholds config (entrante y saliente)
			//
			thresholds: {
			// entrante ---------------------------------
			//
				// AlgorithmTimeLimit en 5.5
				acd_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_acd_wait_inbound",
					txt       : pf + "_acd_wait_time_inbound",
					label_chk : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt : "<%=_T('ACD wait Threshold')%> (<%=_T('seconds')%>)",
					field     : "AlgorithmTimeLimit_INBOUND",
					required  : true,
					validate  : {
						range    : [ 6, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call wait message time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter incoming call wait message time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// AverageCallDuration en 5.5
				avergage_duration_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_average_duration_inbound",
					txt       : pf + "_average_duration_time_inbound",
					label_chk : "<%=_T('Average Duration')%>",
					label_txt : "<%=_T('Average duration')%> (<%=_T('seconds')%>)",
					field     : "AverageCallDuration_INBOUND",
					required  : true,
					validate  : {
						range    : [ 20, 900 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call average duration time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer average duration time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric average duration time.')%>",
							RANGE   : "<%=_T('Please enter incoming call average duration time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// GhostCallThreshold en 5.5
				ghost_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_ghost_inbound",
					txt       : pf + "_ghost_time_inbound",
					label_chk : "<%=_T('Ghost Threshold')%>",
					label_txt : "<%=_T('Ghost Threshold')%> (<%=_T('seconds')%>)",
					field     : "Ghost_INBOUND",
					required  : true,
					validate  : {
						range    : [ 1, 5 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call ghost time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer ghost time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric ghost time.')%>",
							RANGE   : "<%=_T('Please enter incoming call ghost time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// LongCallThreshold en 5.5
				long_interaction_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_long_interaction_inbound",
					txt       : pf + "_long_interaction_time_inbound",
					label_chk : "<%=_T('Long Conversation')%>",
					label_txt : "<%=_T('Long conversation')%> (<%=_T('seconds')%>)",
					field     : "LongInteraction_INBOUND",
					required  : true,
					validate  : {
						range    : [ 30, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call long conversation time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer long conversation time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric long conversation time.')%>",
							RANGE   : "<%=_T('Please enter incoming call long conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// ShortCallThreshold en 5.5
				short_interaction_inbound: {
					tab       : INCOMING,
					ctrl_type : "chk",
					chk       : pf + "_short_interaction_inbound",
					txt       : pf + "_short_interaction_time_inbound",
					label_chk : "<%=_T('Short Conversation')%>",
					label_txt : "<%=_T('Short conversation')%> (<%=_T('seconds')%>)",
					field     : "ShortInteraction_INBOUND",
					required  : true,
					validate  : {
						range    : [ 6, 20 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call short conversation time.')%>",
							INTEGER : "<%=_T('Please enter incoming call integer short conversation time.')%>",
							NUMERIC : "<%=_T('Please enter incoming call numeric short conversation time.')%>",
							RANGE   : "<%=_T('Please enter incoming call short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// ServiceLevelThreshold en 5.5
				service_level_inbound: {
					tab        : INCOMING,
					ctrl_type  : "chk"                                                                      ,
					chk        : pf + "_service_level_time_inbound"                                                           ,
					txt        : pf + "_service_level_time_inbound_time"                                                      ,
					label_chk  : "<%=_T('Service Level')%>"                                                   ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel_INBOUND"                                                         ,
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter incoming call service level time.')%>"                            ,
							INTEGER : "<%=_T('Please enter incoming call integer service level time.')%>"                    ,
							NUMERIC : "<%=_T('Please enter incoming call numeric service level time.')%>"                    ,
							RANGE   : "<%=_T('Please enter incoming call service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},

			// saliente -----------------------------
			//
				// AverageCallDuration en 5.5
				average_duration_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_average_duration_outbound",
					txt       : pf + "_average_duration_time_outbound",
					label_chk : "<%=_T('Average Duration')%>",
					label_txt : "<%=_T('Average duration')%> (<%=_T('seconds')%>)",
					field     : "AverageCallDuration_OUTBOUND",
					required  : true,
					validate  : {
						range    : [ 20, 900 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call average duration time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer average duration time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric average duration time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call average duration time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// LongCallThreshold en 5.5
				long_interaction_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_long_interaction_outbound",
					txt       : pf + "_long_interaction_time_outbound",
					label_chk : "<%=_T('Long Conversation')%>",
					label_txt : "<%=_T('Long conversation')%> (<%=_T('seconds')%>)",
					field     : "LongInteraction_OUTBOUND",
					required  : true,
					validate  : {
						range    : [ 30, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call long conversation time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer long conversation time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric long conversation time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call long conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// ShortCallThreshold en 5.5
				short_interaction_outbound: {
					tab       : OUTGOING,
					ctrl_type : "chk",
					chk       : pf + "_short_interaction_outbound",
					txt       : pf + "_short_interaction_time_outbound",
					label_chk : "<%=_T('Short Conversation')%>",
					label_txt : "<%=_T('Short Conversation')%> (<%=_T('seconds')%>)",
					field     : "ShortInteraction_OUTBOUND",
					required  : true,
					validate  : {
						range : [ 6, 20 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call short conversation time.')%>",
							INTEGER : "<%=_T('Please enter outgoing call integer short conversation time.')%>",
							NUMERIC : "<%=_T('Please enter outgoing call numeric short conversation time.')%>",
							RANGE   : "<%=_T('Please enter outgoing call short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				// ServiceLevelThreshold en 5.5
				service_level_outbound: {
					tab        : OUTGOING,
					ctrl_type  : "chk"                                                                      ,
					chk        : pf + "_service_level_time_outbound"                                                           ,
					txt        : pf + "_service_level_time_outbound_time"                                                      ,
					label_chk  : "<%=_T('Service Level')%>"                                                   ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel_OUTBOUND"                                                         ,
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter outgoing call service level time.')%>"                            ,
							INTEGER : "<%=_T('Please enter outgoing call integer service level time.')%>"                    ,
							NUMERIC : "<%=_T('Please enter outgoing call numeric service level time.')%>"                    ,
							RANGE   : "<%=_T('Please enter outgoing call service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}
			}
		};

		Call_allegro_config.uber.constructor.call( this, params );

		this.th_internal_tabs = new inConcert.conf_form_tab({
			parent      : this            ,
			frm         : this.parent.frm ,
			container_id: pf + "_tabs"    ,
			all_tabs_hidden: all_tabs_hidden,
			callbacks: callbacks,
			tabs_list   : {
				incoming: {
					id               : "incoming",
					name             : "<%=_T('Incoming')%>",
					tab_container_id : pf + "_tab_incoming",
					selected         : true,
					initial_state    : "disabled"
				},
				outgoing : {
					id               : "outgoing",
					name             : "<%=_T('Outgoing')%>",
					tab_container_id : pf + "_tab_outgoing",
					initial_state    : "disabled"
				}
			}
		});

		this.th_internal_tabs.show_tabs_container();
	}

	inConcert.Utils.inherit( Call_allegro_config, account_config );

	Call_allegro_config.prototype.edit_from_channel_section = function() 
	{
		var self = this;

		var ds_all_tabs = function() {
			self.th_internal_tabs.enable_disable_all_tabs( false /*enabled*/ );
		};

		var campaign = this.parent.frm.campaign;
		if (!campaign) {
			ds_all_tabs();
			return;
		}
		var accounts = campaign.accounts;
		if (!$.isArray(accounts) || accounts.length > 1 ) {
			ds_all_tabs();
			return;
		}
		var call_account = accounts[0];
		if (!call_account) {
			ds_all_tabs();
			return;
		}
		if (call_account.proxyType !== "CALL") {
			ds_all_tabs();
			return;
		}
		if (!call_account.incomingConfig || !call_account.outgoingConfig) {
			ds_all_tabs();
			return;
		}

		if ( call_account.incomingConfig.enabled ) {
			this.enable_internal_tab( INCOMING );
			this.editable( INCOMING );
		}
		else {
			this.disable_internal_tab( INCOMING );
			this.disable( INCOMING );
		}

		if ( call_account.outgoingConfig.enabled ) {
			this.enable_internal_tab( OUTGOING );
			this.editable( OUTGOING );
		}
		else {
			this.disable_internal_tab( OUTGOING );
			this.disable( OUTGOING );
		}
		this.th_internal_tabs.select_first_visible_tab();
	};

	Call_allegro_config.prototype.__can_get_value = function(ctrl)
	{
		var tab_id = ctrl.tab;
		var is_enabled_tab = this.th_internal_tabs.is_enabled_tab(tab_id);
		return is_enabled_tab;
	};

/*
  Clase - 'Facebook_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Facebook_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_facebook";
		this.type       = "FACEBOOK";
		this.handler_id = "tabsMenuItemFacebook";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_facebook_capability_config",
			prefix_id    : "adm_form_campaign_channels_facebook",
			proxies      : { FACEBOOK : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                  ,
					chk        : this.prefix_id + "_service_level"      ,
					txt        : this.prefix_id + "_service_level_time" ,
					label_chk  : "<%=_T('Service Level')%>"             ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                         ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Facebook service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Facebook integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Facebook numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Facebook service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "FACEBOOK",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Facebook wait message time.')%>",
							INTEGER : "<%=_T('Please enter Facebook integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Facebook numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Facebook wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "FACEBOOK",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "FACEBOOK"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "FACEBOOK"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "FACEBOOK"),
							RANGE   : "<%=_T('Please enter FACEBOOK inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on inactivity time.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on inactivity time.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Facebook_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Facebook_config, account_config );

/*
  Clase - 'Instagram_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Instagram_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_instagram";
		this.type       = "INSTAGRAM";
		this.handler_id = "tabsMenuItemInstagram";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_instagram_capability_config",
			prefix_id    : "adm_form_campaign_channels_instagram",
			proxies      : { INSTAGRAM : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)" ,
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Instagram service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Instagram integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Instagram numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Instagram service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "INSTAGRAM",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Instagram wait message time.')%>",
							INTEGER : "<%=_T('Please enter Instagram integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Instagram numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Instagram wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "INSTAGRAM",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "INSTAGRAM"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "INSTAGRAM"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "INSTAGRAM"),
							RANGE   : "<%=_T('Please enter INSTAGRAM inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Instagram_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Instagram_config, account_config );

/*
  Clase - 'Youtube_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Youtube_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_youtube";
		this.type       = "YOUTUBE";
		this.handler_id = "tabsMenuItemYoutube";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_youtube_capability_config",
			prefix_id    : "adm_form_campaign_channels_youtube",
			proxies      : { YOUTUBE : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Youtube service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Youtube integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Youtube numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Youtube service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "YOUTUBE",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Youtube wait message time.')%>",
							INTEGER : "<%=_T('Please enter Youtube integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Youtube numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Youtube wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "YOUTUBE",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "YOUTUBE"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "YOUTUBE"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "YOUTUBE"),
							RANGE   : "<%=_T('Please enter YOUTUBE inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Youtube_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Youtube_config, account_config );

/*
  Clase - 'Appstore_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Appstore_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_appstore";
		this.type       = "APPSTORE";
		this.handler_id = "tabsMenuItemAppstore";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_appstore_capability_config",
			prefix_id    : "adm_form_campaign_channels_appstore",
			proxies      : { APPSTORE : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)" ,
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Appstore service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Appstore integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Appstore numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Appstore service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "APPSTORE",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Appstore wait message time.')%>",
							INTEGER : "<%=_T('Please enter Appstore integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Appstore numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Appstore wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "APPSTORE",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "APPSTORE"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "APPSTORE"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "APPSTORE"),
							RANGE   : "<%=_T('Please enter APPSTORE inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Appstore_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Appstore_config, account_config );

/*
  Clase - 'Googleplay_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Googleplay_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_googleplay";
		this.type       = "GOOGLEPLAY";
		this.handler_id = "tabsMenuItemGoogleplay";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_googleplay_capability_config",
			prefix_id    : "adm_form_campaign_channels_googleplay",
			proxies      : { GOOGLEPLAY : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Googleplay service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Googleplay integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Googleplay numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Googleplay service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				acd: {
					proxy: "GOOGLEPLAY",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Googleplay wait message time.')%>",
							INTEGER : "<%=_T('Please enter Googleplay integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Googleplay numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Googleplay wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "GOOGLEPLAY",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "GOOGLEPLAY"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "GOOGLEPLAY"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "GOOGLEPLAY"),
							RANGE   : "<%=_T('Please enter GOOGLEPLAY inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Googleplay_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Googleplay_config, account_config );

/*
  Clase - 'Rcs_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Rcs_config( params )
	{
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_rcs";
		this.type       = "RCS";
		this.handler_id = "tabsMenuItemRcs";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_rcs_capability_config",
			prefix_id    : "adm_form_campaign_channels_rcs",
			proxies      : { RCS : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Rcs service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Rcs integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Rcs numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Rcs service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "RCS",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Rcs wait message time.')%>",
							INTEGER : "<%=_T('Please enter Rcs integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Rcs numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Rcs wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "RCS",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					cmb3                        : this.prefix_id + "_transferToCampaignOnInactivityTime",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnInactivityTime",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "RCS"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "RCS"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "RCS"),
							RANGE   : "<%=_T('Please enter RCS inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on inactivity time.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on inactivity time.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Rcs_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Rcs_config, account_config );

/*
  Clase - 'IM_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function IM_config( params )
	{
		this.type       = "IM";
		this.handler_id = "tabsMenuItemIM";
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_im";
		var pf          = this.prefix_id;
		this.section_id = pf + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_im_capability_config",
			prefix_id    : "adm_form_campaign_channels_im",
			proxies      : { IM : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				answer: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_answer"                                                                   ,
					txt    : pf + "_answer_time"                                                              ,
					label_chk  : "<%=_T('Answer')%>"                                                                 ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "AnswerTimeout"                                                                  ,
					validate  : {
						range    : [ 1, 50 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM answer time.')%>"                                ,
							INTEGER : "<%=_T('Please enter IM integer answer time.')%>"                        ,
							NUMERIC : "<%=_T('Please enter IM numeric answer time.')%>"                        ,
							RANGE   : "<%=_T('Please enter IM answer time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				ghost: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_ghost"                                                                    ,
					txt    : pf + "_ghost_time"                                                               ,
					label_chk  : "<%=_T('Ghost Threshold')%>"                                                        ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "Ghost"                                                                          ,
					validate  : {
						range    : [ 1, 120 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM ghost time.')%>"                                 ,
							INTEGER : "<%=_T('Please enter IM integer ghost time.')%>"                         ,
							NUMERIC : "<%=_T('Please enter IM numeric ghost time.')%>"                         ,
							RANGE   : "<%=_T('Please enter IM ghost time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				inactivity: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_inactivity"                                                               ,
					txt    : pf + "_inactivity_time"                                                          ,
					label_chk  : "<%=_T('Inactivity Threshold')%>"                                                   ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "InactivityTimeout"                                                              ,
					validate  : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM inactivity time.')%>"                            ,
							INTEGER : "<%=_T('Please enter IM integer inactivity time.')%>"                    ,
							NUMERIC : "<%=_T('Please enter IM numeric inactivity time.')%>"                    ,
							RANGE   : "<%=_T('Please enter IM inactivity time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				short_interaction: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_short_interaction"                                                        ,
					txt    : pf + "_short_interaction_time"                                                   ,
					label_chk  : "<%=_T('Short Conversation')%>"                                                      ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ShortInteraction"                                                               ,
					validate  : {
						range    : [ 1, 50 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM short conversation time.')%>"                     ,
							INTEGER : "<%=_T('Please enter IM integer short conversation time.')%>"             ,
							NUMERIC : "<%=_T('Please enter IM numeric short conversation time.')%>"             ,
							RANGE   : "<%=_T('Please enter IM short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				acd: {
					proxy: "IM",
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_wait"                                                                 ,
					txt    : pf + "_acd_wait_time"                                                            ,
					label_chk  : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt  : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					field  : "ACDWaitThreshold"                                                               ,
					validate  : {
						range    : [ 10, 10800 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM wait message time.')%>"                          ,
							INTEGER : "<%=_T('Please enter IM integer wait message time.')%>"                  ,
							NUMERIC : "<%=_T('Please enter IM numeric wait message time.')%>"                  ,
							RANGE   : "<%=_T('Please enter IM wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				acd_first_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_first_message"                                                        ,
					txt    : pf + "_acd_first_message_time"                                                   ,
					label_chk  : "<%=_T('Wait First Message')%>"                                                     ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ACDFirstMessageThreshold"                                                       ,
					validate  : {
						range    : [ 0, 30 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM first message time.')%>"                         ,
							INTEGER : "<%=_T('Please enter IM integer first message time.')%>"                 ,
							NUMERIC : "<%=_T('Please enter IM numeric first message time.')%>"                 ,
							RANGE   : "<%=_T('Please enter IM first message time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				acd_repeated_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_repeated_message"                                                     ,
					txt    : pf + "_acd_repeated_message_time"                                                ,
					label_chk  : "<%=_T('Wait Repeated Msg')%>"                                                      ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ACDRepeatedMessageThreshold"                                                    ,
					validate  : {
						range    : [ 10, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM repeated message time.')%>"                      ,
							INTEGER : "<%=_T('Please enter IM integer repeated message time.')%>"              ,
							NUMERIC : "<%=_T('Please enter IM numeric repeated message time.')%>"              ,
							RANGE   : "<%=_T('Please enter IM repeated message time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				automatic_wrapup: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_automatic_wrapup"                                                         ,
					txt    : pf + "_automatic_wrapup_time"                                                    ,
					label_chk  : "<%=_T('Wrapup')%>"                                                                 ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "AutoWrapupTimeout"                                                              ,
					validate  : {
						range    : [ 1, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM wrapup time.')%>"                                ,
							INTEGER : "<%=_T('Please enter IM integer wrapup time.')%>"                        ,
							NUMERIC : "<%=_T('Please enter IM numeric wrapup time.')%>"                        ,
							RANGE   : "<%=_T('Please enter IM wrapup time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				service_level: {
					ctrl_type  : "chk"                     ,
					chk        : pf + "_service_level"     ,
					txt        : pf + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>",
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel"            ,
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 5, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter IM service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter IM integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter IM numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter IM service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}
			}
		};

		IM_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( IM_config, account_config );

/*
  Clase - 'Mail_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Mail_config( params )
	{
		this.type       = "MAIL";
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_mail";
		var pf          = this.prefix_id;
		this.section_id = pf + "_panel";
		this.handler_id = "tabsMenuItemMail";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_mail_capability_config",
			prefix_id    : "adm_form_campaign_channels_mail",
			proxies      : { MAIL : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                      ,
					chk        : pf + "_service_level"      ,
					txt        : pf + "_service_level_time" ,
					label_chk  : "<%=_T('Service Level')%>" ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"             ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter e-mail service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter e-mail integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter e-mail numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter e-mail service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				acd: {
					proxy: "MAIL",
					ctrl_type                   : "chk",
					chk                         : pf + "_acd_wait",
					txt                         : pf + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : pf + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter e-mail wait message time.')%>",
							INTEGER : "<%=_T('Please enter e-mail integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter e-mail numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter e-mail wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "MAIL",
					ctrl_type                   : "chk",
					chk                         : pf + "_inactivity_time",
					txt                         : pf + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : pf + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					cmb3                        : this.prefix_id + "_transferToCampaignOnInactivityTime",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnInactivityTime",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "EMAIL"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "EMAIL"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "EMAIL"),
							RANGE   : "<%=_T('Please enter EMAIL inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on inactivity time.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on inactivity time.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Mail_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Mail_config, account_config );

/*
  Clase - 'SMS_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function SMS_config( params )
	{
		this.prefix_id = params.prefix_id || "adm_form_campaign_channels_sms";
		this.type       = "SMS";
		this.handler_id = "tabsMenuItemSMS";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_sms_capability_config",
			prefix_id    : "adm_form_campaign_channels_sms",
			proxies      : { SMS : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter SMS service level time.')%>"                   ,
							INTEGER : "<%=_T('Please enter SMS integer service level time.')%>"           ,
							NUMERIC : "<%=_T('Please enter SMS numeric service level time.')%>"           ,
							RANGE   : "<%=_T('Please enter SMS service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "SMS",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter SMS wait message time.')%>",
							INTEGER : "<%=_T('Please enter SMS integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter SMS numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter SMS wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "SMS",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					cmb3                        : this.prefix_id + "_transferToCampaignOnInactivityTime",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnInactivityTime",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "SMS"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "SMS"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "SMS"),
							RANGE   : "<%=_T('Please enter SMS inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on inactivity time.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on inactivity time.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		SMS_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( SMS_config, account_config );

/*
  Clase - 'WHATSAPP_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function WHATSAPP_config( params )
	{
		this.prefix_id = params.prefix_id || "adm_form_campaign_channels_whatsapp";
		this.type       = "WHATSAPP";
		this.handler_id = "tabsMenuItemWHATSAPP";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_whatsapp_capability_config",
			prefix_id    : "adm_form_campaign_channels_whatsapp",
			proxies      : { WHATSAPP : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter WHATSAPP service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter WHATSAPP integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter WHATSAPP numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter WHATSAPP service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "WHATSAPP",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter WHATSAPP wait message time.')%>",
							INTEGER : "<%=_T('Please enter WHATSAPP integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter WHATSAPP numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter WHATSAPP wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				acd_first_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : this.prefix_id + "_acd_first_message"                                                        ,
					txt    : this.prefix_id + "_acd_first_message_time"                                                   ,
					label_chk  : "<%=_T('Wait First Message')%>"                                                     ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ACDFirstMessageThreshold"                                                       ,
					validate  : {
						range    : [ 0, 60 ],
						messages : {
							EMPTY   : "<%=_T('Please enter whatsapp first message time.')%>"                           ,
							INTEGER : "<%=_T('Please enter whatsapp integer first message time.')%>"                   ,
							NUMERIC : "<%=_T('Please enter whatsapp numeric first message time.')%>"                   ,
							RANGE   : "<%=_T('Please enter whatsapp first message time between {0} and {1}.')%>"  
						}
					},
					hr_top_visible: true,
					get_markup : __get_chk_txt_markup
				},
				acd_repeated_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : this.prefix_id + "_acd_repeated_message"                                                     ,
					txt    : this.prefix_id + "_acd_repeated_message_time"                                                ,
					label_chk  : "<%=_T('Wait Repeated Msg')%>"                                                      ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					label_info  : "<%=_T('The message will repeat at intervals of the time in seconds entered, continuously, until a total of one hour has passed.')%>",
					field  : "ACDRepeatedMessageThreshold"                                                    ,
					validate  : {
						range    : [ 300, 600 ],
						messages : {
							EMPTY   : "<%=_T('Please enter whatsapp repeated message time.')%>"                      ,
							INTEGER : "<%=_T('Please enter whatsapp integer repeated message time.')%>"                ,
							NUMERIC : "<%=_T('Please enter whatsapp numeric repeated message time.')%>"                ,
							RANGE   : "<%=_T('Please enter whatsapp repeated message time between {0} and {1}.')%>"  
						}
					},
					get_markup : __get_chk_txt_markup
				},
				inactivity_time: {
					proxy: "WHATSAPP",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					cmb3                        : this.prefix_id + "_transferToCampaignOnInactivityTime",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnInactivityTime",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "WHATSAPP"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "WHATSAPP"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "WHATSAPP"),
							RANGE   : "<%=_T('Please enter WHATSAPP inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on inactivity time.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on inactivity time.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		WHATSAPP_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( WHATSAPP_config, account_config );

/*
  Clase - 'Twitter_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Twitter_config( params )
	{
		this.prefix_id = params.prefix_id || "adm_form_campaign_channels_twitter";
		this.type       = "TWITTER";
		this.handler_id = "tabsMenuItemTwitter";
		this.section_id = this.prefix_id + "_panel";

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_twitter_capability_config",
			prefix_id    : "adm_form_campaign_channels_twitter",
			proxies      : { TWITTER : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                 ,
					chk        : this.prefix_id + "_service_level"     ,
					txt        : this.prefix_id + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>"            ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel"                        ,
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Twitter service level time.')%>"                    ,
							INTEGER : "<%=_T('Please enter Twitter integer service level time.')%>"            ,
							NUMERIC : "<%=_T('Please enter Twitter numeric service level time.')%>"            ,
							RANGE   : "<%=_T('Please enter Twitter service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				},
				acd: {
					proxy: "TWITTER",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_acd_wait",
					txt                         : this.prefix_id + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter Twitter wait message time.')%>",
							INTEGER : "<%=_T('Please enter Twitter integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter Twitter numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter Twitter wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "TWITTER",
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "TWITTER"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "TWITTER"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "TWITTER"),
							RANGE   : "<%=_T('Please enter TWITTER inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Twitter_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Twitter_config, account_config );

/*
  Clase - 'Web_chat_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Web_chat_config( params )
	{
		this.type       = "CHAT";
		this.prefix_id  = params.prefix_id || "adm_form_campaign_channels_webchat";
		var pf          = this.prefix_id;
		this.section_id = pf + "_panel";
		this.handler_id = "tabsMenuItemWebchat";
		this.after_answer_time_chk_change = params.after_answer_time_chk_change;

		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_webchat_capability_config",
			prefix_id    : "adm_form_campaign_channels_webchat",
			proxies      : { CHAT : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk"                                                                      ,
					chk        : pf + "_service_level"                                                           ,
					txt        : pf + "_service_level_time"                                                      ,
					label_chk  : "<%=_T('Service Level')%>"                                                   ,
					label_txt  : "<%=_T('Service level time')%> (<%=_T('seconds')%>)",
					field      : "ServiceLevel"                                                         ,
					required   : true,
					unity      : "SECONDS",
					validate   : {
						range    : [ 5, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat service level time.')%>"                            ,
							INTEGER : "<%=_T('Please enter web chat integer service level time.')%>"                    ,
							NUMERIC : "<%=_T('Please enter web chat numeric service level time.')%>"                    ,
							RANGE   : "<%=_T('Please enter web chat service level time between {0} and {1}.')%>"
						}
					},
					hr_visible	: true,
					get_markup : __get_txt_markup
				},
				answer: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_answer"                                                                   ,
					txt    : pf + "_answer_time"                                                              ,
					label_chk  : "<%=_T('Answer')%>"                                                                 ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "AnswerTimeout"                                                                  ,
					validate  : {
						range    : [ 1, 50 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat answer time.')%>"                                ,
							INTEGER : "<%=_T('Please enter web chat integer answer time.')%>"                        ,
							NUMERIC : "<%=_T('Please enter web chat numeric answer time.')%>"                        ,
							RANGE   : "<%=_T('Please enter web chat answer time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				ghost: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_ghost"                                                                    ,
					txt    : pf + "_ghost_time"                                                               ,
					label_chk  : "<%=_T('Ghost Threshold')%>"                                                        ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "Ghost"                                                                          ,
					validate  : {
						range    : [ 1, 120 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat ghost time.')%>"                                 ,
							INTEGER : "<%=_T('Please enter web chat integer ghost time.')%>"                         ,
							NUMERIC : "<%=_T('Please enter web chat numeric ghost time.')%>"                         ,
							RANGE   : "<%=_T('Please enter web chat ghost time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				inactivity: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_inactivity"                                                               ,
					txt    : pf + "_inactivity_time"                                                          ,
					label_chk  : "<%=_T('Inactivity Threshold')%>"                                                   ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "InactivityTimeout"                                                              ,
					validate  : {
						range    : [ 1, 1200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat inactivity time.')%>"                            ,
							INTEGER : "<%=_T('Please enter web chat integer inactivity time.')%>"                    ,
							NUMERIC : "<%=_T('Please enter web chat numeric inactivity time.')%>"                    ,
							RANGE   : "<%=_T('Please enter web chat inactivity time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				short_interaction: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_short_interaction"                                                        ,
					txt    : pf + "_short_interaction_time"                                                   ,
					label_chk  : "<%=_T('Short Conversation')%>"                                                      ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ShortInteraction"                                                               ,
					validate  : {
						range    : [ 1, 50 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat short conversation time.')%>"                     ,
							INTEGER : "<%=_T('Please enter web chat integer short conversation time.')%>"             ,
							NUMERIC : "<%=_T('Please enter web chat numeric short conversation time.')%>"             ,
							RANGE   : "<%=_T('Please enter web chat short conversation time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				acd: {
					proxy: "CHAT",
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_wait"                                                                 ,
					txt    : pf + "_acd_wait_time"                                                            ,
					label_chk  : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt  : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : this.prefix_id + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field  : "ACDWaitThreshold"                                                               ,
					validate  : {
						range    : [ 10, 10800 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat wait message time.')%>"                          ,
							INTEGER : "<%=_T('Please enter web chat integer wait message time.')%>"                  ,
							NUMERIC : "<%=_T('Please enter web chat numeric wait message time.')%>"                  ,
							RANGE   : "<%=_T('Please enter web chat wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				acd_first_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_first_message"                                                        ,
					txt    : pf + "_acd_first_message_time"                                                   ,
					label_chk  : "<%=_T('Wait First Message')%>"                                                     ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ACDFirstMessageThreshold"                                                       ,
					validate  : {
						range    : [ 0, 30 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat first message time.')%>"                         ,
							INTEGER : "<%=_T('Please enter web chat integer first message time.')%>"                 ,
							NUMERIC : "<%=_T('Please enter web chat numeric first message time.')%>"                 ,
							RANGE   : "<%=_T('Please enter web chat first message time between {0} and {1}.')%>"
						}
					},
					hr_top_visible: true,
					get_markup : __get_chk_txt_markup
				},
				acd_repeated_message: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_acd_repeated_message"                                                     ,
					txt    : pf + "_acd_repeated_message_time"                                                ,
					label_chk  : "<%=_T('Wait Repeated Msg')%>"                                                      ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "ACDRepeatedMessageThreshold"                                                    ,
					validate  : {
						range    : [ 10, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat repeated message time.')%>"                      ,
							INTEGER : "<%=_T('Please enter web chat integer repeated message time.')%>"              ,
							NUMERIC : "<%=_T('Please enter web chat numeric repeated message time.')%>"              ,
							RANGE   : "<%=_T('Please enter web chat repeated message time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				},
				automatic_wrapup: {
					ctrl_type : "chk"                                                                         ,
					chk    : pf + "_automatic_wrapup"                                                         ,
					txt    : pf + "_automatic_wrapup_time"                                                    ,
					label_chk  : "<%=_T('Wrapup')%>"                                                                 ,
					label_txt  : "<%=_T('Time')%> (<%=_T('seconds')%>)"                                                             ,
					field  : "AutoWrapupTimeout"                                                              ,
					validate  : {
						range    : [ 1, 180 ],
						messages : {
							EMPTY   : "<%=_T('Please enter web chat wrapup time.')%>"                                ,
							INTEGER : "<%=_T('Please enter web chat integer wrapup time.')%>"                        ,
							NUMERIC : "<%=_T('Please enter web chat numeric wrapup time.')%>"                        ,
							RANGE   : "<%=_T('Please enter web chat wrapup time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_chk_txt_markup
				}
			}
		};

		Web_chat_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Web_chat_config, account_config );

/*
  Clase - 'Contact_form_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
*/
	function Contact_form_config( params )
	{
		this.type       = "WEBCONTACT";
		this.prefix_id = params.prefix_id || "adm_form_campaign_channels_contactform";
		var pf         = this.prefix_id;
		this.section_id = pf + "_panel";
		this.handler_id = "tabsMenuItemWebcontact";
		// attention capability configuration
		params.capability = {
			container_id : "adm_form_campaign_channels_contactform_capability_config",
			prefix_id    : "adm_form_campaign_channels_contactform",
			proxies      : { WEBCONTACT : true }
		};

		this.tab_ctrls = {
			// capability config ( solo como referencia )
			//  
			// thresholds config
			thresholds: {
				service_level: {
					ctrl_type  : "chk",
					chk        : pf + "_service_level",
					txt        : pf + "_service_level_time",
					label_chk  : "<%=_T('Service Level')%>",
					label_txt  : "<%=_T('Service level time')%> (<%=_T('minutes')%>)",
					field      : "ServiceLevel",
					required   : true,
					unity      : "MINUTES",
					validate   : {
						range    : [ 1, 10080 ],
						messages : {
							EMPTY   : "<%=_T('Please enter contact form service level time.')%>",
							INTEGER : "<%=_T('Please enter contact form integer service level time.')%>",
							NUMERIC : "<%=_T('Please enter contact form numeric service level time.')%>",
							RANGE   : "<%=_T('Please enter contact form service level time between {0} and {1}.')%>"
						}
					},
					get_markup : __get_txt_markup
				}, 
				acd: {
					proxy: "WEBCONTACT",
					ctrl_type                   : "chk",
					chk                         : pf + "_acd_wait",
					txt                         : pf + "_acd_wait_time",
					label_chk                   : "<%=_T('Enable ACD Wait Threshold')%>",
					label_txt                   : "<%=_T('ACD time')%> (<%=_T('seconds')%>)",
					cmb2                        : pf + "_actionOnAcdWaitTimeout",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions 				: __get_actions_on_acd_wait_timeout,
					cmb3                        : this.prefix_id + "_transferToCampaignOnAcdWaitTimeout",
					label_cmb3                  : "<%=_T('Campaign')%>",
					cmb4                        : this.prefix_id + "_transferToAccountOnAcdWaitTimeout",
					label_cmb4                  : "<%=_T('Account')%>",
					field                       : "ACDWaitThreshold",
					validate                    : {
						range    : [ 10, 259200 ],
						messages : {
							EMPTY   : "<%=_T('Please enter contact form wait message time.')%>",
							INTEGER : "<%=_T('Please enter contact form integer wait message time.')%>",
							NUMERIC : "<%=_T('Please enter contact form numeric wait message time.')%>",
							RANGE   : "<%=_T('Please enter contact form wait message time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on timeout.')%>",
							CAMPAIGN: "<%=_T('Please select campaign to transfer on timeout.')%>",
							ACCOUNT : "<%=_T('Please select account to transfer on timeout.')%>"
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				},
				inactivity_time: {
					proxy: "WEBCONTACT",  
					ctrl_type                   : "chk",
					chk                         : this.prefix_id + "_inactivity_time",
					txt                         : this.prefix_id + "_inactivity_time_txt",
					label_chk                   : "<%=_T('Inactivity Threshold')%>",
					label_txt                   : "<%=_T('Time')%> (<%=_T('minutes')%>)",
					cmb2                        : this.prefix_id + "_actionOnInactivityTime",
					label_cmb2                  : "<%=_T('Action on timeout')%>",
					get_actions                 : __get_actions_on_inactivity_time,
					field                       : "InactivityTimeout",
					unity                       : "MINUTES",
					validate                    : {
						range    : [ 5, 86400 ], /* 5min - 15 dias. */
						messages : {
							EMPTY   : format("<%=_T('Please enter {0} inactivity time.')%>", "WEBCONTACT"),
							INTEGER : format("<%=_T('Please enter {0} integer inactivity time.')%>", "WEBCONTACT"),
							NUMERIC : format("<%=_T('Please enter {0} numeric inactivity time.')%>", "WEBCONTACT"),
							RANGE   : "<%=_T('Please enter WEBCONTACT inactivity time between {0} and {1}.')%>",
							ACTION  : "<%=_T('Please select action on inactivity time.')%>",
						}
					},
					get_markup : __get_chk_txt_cmb_markup
				}
			}
		};

		Contact_form_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( Contact_form_config, account_config );

	//---------------------------------------------------------------------
	// form campaigns class - messages section
	function adm_form_campaigns_section_channels( params )
	{
		var success = adm_form_campaigns_section_channels.uber.constructor.call( this, params );
		if (!success) {
			return;
		}

		this.view_action_id = "view_campaigns_channels";  // view action id de permisos
		this.save_action_id = "save_campaign_channels"; // action id de permisos
		this.proxy_list = [ "MAIL", "CHAT", "IM", "FACEBOOK", "INSTAGRAM", "YOUTUBE", "APPSTORE", "GOOGLEPLAY", "RCS", "TWITTER", "SMS", "WHATSAPP", "CALL", "WEBCONTACT" ];
		this.channels_sections = {};

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_channels, inConcert.conf_form_section );

	adm_form_campaigns_section_channels.prototype.hide_channels_sections = function( all_disabled )
	{
		//disable all the section
		var $account_items = $( "#adm_form_campaign_channels_panel_list li" );
		$account_items.removeClass( "tabsMenuItemSelect" );
		if (all_disabled) {
			$account_items.hide();
		}
		//hide all the sections
		$( "#adm_form_campaign_channels" ).find( ".formTabsPanel" ).hide();
	};

	adm_form_campaigns_section_channels.prototype.get_account_section = function( proxy )
	{
		if (proxy === "GMAIL") {
			proxy = "MAIL";
		}
		return this.channels_sections[proxy];
	};

	adm_form_campaigns_section_channels.prototype.build_account_section = function( proxy, params )
	{
		var config_section;
		if ( proxy === "GMAIL" ) {
			proxy = "MAIL";
		}
		if ( this.channels_sections[ proxy ] ) {
			return this.channels_sections[ proxy ];
		}
		params = params || { parent: this };
		switch ( proxy )
		{
			case "CALL":
				if ( inConcert.app.has_allegro_integration() ) {
					config_section = new Call_allegro_config( params );
				}
				else {
					config_section = new Call_config( params );
				}
				break;

			case "CHAT":
				config_section = new Web_chat_config( params );
				break;

			case "IM":
				config_section = new IM_config( params );
				break;

			case "FACEBOOK":
				config_section = new Facebook_config( params );
				break;

			case "INSTAGRAM":
				config_section = new Instagram_config( params );
				break;

			case "YOUTUBE":
				config_section = new Youtube_config( params );
				break;

			case "APPSTORE":
				config_section = new Appstore_config( params );
				break;

			case "GOOGLEPLAY":
				config_section = new Googleplay_config( params );
				break;

			case "RCS":
				config_section = new Rcs_config( params );
				break;

			case "MAIL": // MAIL o GMAIL
				config_section = new Mail_config( params );
				break;

			case "SMS":
				config_section = new SMS_config( params );
				break;

			case "WHATSAPP":
				config_section = new WHATSAPP_config( params );
				break;

			case "TWITTER":
				config_section = new Twitter_config( params );
				break;

			case "WEBCONTACT":
				config_section = new Contact_form_config( params );
				break;

			default:
				//throw("Unknown configuration type " + proxy);
				return;
		}

		this.channels_sections[ proxy ] = config_section;
		return config_section;
	};

	adm_form_campaigns_section_channels.prototype.load_advanced_config = function()
	{
		$.each( this.channels_sections || {},
			function( _, config_section ) {
				config_section.set_current_config();
			});
	};

	adm_form_campaigns_section_channels.prototype.enabled_channels_sections = function()
	{
		var self = this;
		var campaign = this.frm.campaign;
		var accounts = campaign.accounts;
		var enabled_proxies = {};
		this.hide_channels_sections( true /* disable all */ );
		var $save_button = $( "#adm_form_campaign_channels_save" );
		$save_button.prop( "disabled", true );

		if (!$.isArray(accounts) || accounts.length === 0) {
			return;
		}

		$save_button.prop( "disabled", false );
		//first enable the configurations given by each campaign account types
		$.each( accounts, function( _, account )
		{
			if ( !inConcert.app.hasLicensesForChannel( account.proxyType ) ) {
				return true; // continue
			}
			var config_section = self.get_account_section( account.proxyType );
			if ( config_section ) {
				config_section.enable();
				enabled_proxies[ account.proxyType ] = true;
			}
		});

		//finally, select the first enabled configuration in descending order(according to the html markup)
		$.each(self.proxy_list || {}, function( _, proxy )
		{
			if ( !enabled_proxies[ proxy ] ) {
				return true; // continue
			}
			var config_section = self.get_account_section( proxy );
			if ( config_section ) {
				config_section.select();
				return false; //break
			}
		});

		var enabled_proxies_keys = Object.keys( enabled_proxies );
		if ( enabled_proxies_keys.length === 0 ) {
		// si no existen tabls enableds oculto el boton "save"
			this.hide_save_action();
		}

		this.load_advanced_config();
	};

	adm_form_campaigns_section_channels.prototype.show_save_action = function()
	{
		$( "#adm_form_campaign_channels_save" )
			.prop( "disabled", false )
			.show();
		$( "#adm_form_campaign_channels_cancel" ).attr( "value", "<%=_T('Cancel')%>" );
	};

	adm_form_campaigns_section_channels.prototype.hide_save_action = function()
	{
		$( "#adm_form_campaign_channels_save" )
			.prop( "disabled", true )
			.hide();
		$( "#adm_form_campaign_channels_cancel" ).attr( "value", "<%=_T('Close')%>" );
	};

	adm_form_campaigns_section_channels.prototype.build_channels_sections = function()
	{
		var self = this;
		$.each( this.proxy_list, function( _, proxy ) {
			self.build_account_section( proxy );
		});
	};

	adm_form_campaigns_section_channels.prototype.save_advanced_config = function()
	{
		var self = this;

		this.frm.hide_messages();

		var validation = { success : false };
		var capability = { channels : {} };
		var thresholds = {};
		$("#adm_form_campaign_channels .formFieldValidation").hide();

		// aca realiza la valdacion, control por control, segun la seccion ( validacion de capability y thresholds )
		//
		$.each( this.channels_sections, function( _, config_section )
		{
			if ( !config_section.is_enabled() ) {
				return true; // continue
			}
			validation = config_section.validate();
			if ( validation.success ) {
				var data = config_section.get();
				if ( data.capability && data.capability.channels ) {
					// aca por cada valor retornado de capability, genero la data x channel a enviar a guardar al server.
					for ( var channel in data.capability.channels )
					{
						if ( data.capability.channels.hasOwnProperty( channel ) ) {
							var channel_data = data.capability.channels[ channel ];
							if ( typeof channel_data.attentionCapability === "number" ) {
								capability.channels[ channel ] = channel_data;
							}
						}
					}
				}
				// aca estan los thresholds
				thresholds[ config_section.type ] = data.thresholds;
				
				//Si tenemos algo en validation[ActionOnAcdWaitTimeout|InactivityTime], debemos guardar el dato en la configuracion de la cuenta
				let hasActionOnAcdWaitTimeout = validation.ActionOnAcdWaitTimeout !== undefined;
				let hasInactivityTime = validation.InactivityTimeout !== undefined;
				if(hasActionOnAcdWaitTimeout || hasInactivityTime)
				{
					var campaign = self.frm.campaign;
					//Recorremos la lista de accounts del tipo que corresponde al canal que estamos evaluando
					$.each( self.frm.campaign.accounts, function( _, account)
					{
						if (account.proxyType !== config_section.type) {
							return true;
						}
						let can_update_account = false;
						//Si no ha cambiado el ActionOnAcdWaitTimeout, nos evitamos este request
						if(hasActionOnAcdWaitTimeout && (account.ActionOnAcdWaitTimeout.action !== validation.ActionOnAcdWaitTimeout.action || 
							account.ActionOnAcdWaitTimeout.campaign !== validation.ActionOnAcdWaitTimeout.campaign || 
							account.ActionOnAcdWaitTimeout.account !== validation.ActionOnAcdWaitTimeout.account))
						{
							account.ActionOnAcdWaitTimeout = validation.ActionOnAcdWaitTimeout;
							can_update_account = true;
							//TODO: Para proxy CALL, si la accion es Transfer hay que desmarcar la opcion de voicemail on timeout
							if(account.proxyType === "CALL" && account.ActionOnAcdWaitTimeout.action === "TransferToCampaign" && account.incomingConfig && account.incomingConfig.ContactCenter && account.incomingConfig.ContactCenter.VoicemailOnTimeout) {
								account.incomingConfig.ContactCenter.VoicemailOnTimeout = false;
							}
						}

						//Si no ha cambiado el InactivityTimeout, nos evitamos este request
						if(hasInactivityTime && (!account.InactivityTimeout || account.InactivityTimeout.action !== validation.InactivityTimeout.action || 
							account.InactivityTimeout.campaign !== validation.InactivityTimeout.campaign || 
							account.InactivityTimeout.account !== validation.InactivityTimeout.account))
						{
							account.InactivityTimeout = validation.InactivityTimeout;
							can_update_account = true;
						}

						if(can_update_account){
							campaign.updateAccount( account, undefined );
						}
					});
				}
			}
			else {
				config_section.select();
				if ( !validation.builtin_msg_err ) {
					var message = validation.error_message || "<%=_T('Error validating campaign advanced config, please verify the marked fields')%>";
					var $ctrl = validation.$ctrl;
					if ( $ctrl ) {
						$ctrl.focus();
					}
					self.frm.show_status_message(
						message,
						"Error" /* message type */,
						false /* timeout infinite */
					);
				}
				return false; // break
			}
		});

		if ( !validation.success ) {
			return;
		}

		// aca guarda la configuracion de los thresholds
		var save_thresholds = function( save_callback )
		{
			self.frm.campaign.saveThresholds(
			{
				thresholds : thresholds,
				callback   : function( success, response_error )
				{
					if ( success ) {
						save_callback( true );
					}
					else {
						var message;
						var error = response_error.error;
						if ( typeof error === "object" ) {
							error = ( error.result || "").toUpperCase();
						}
						switch ( error )
						{
							case "INVALID_REQUEST":
								message = "<%=_T('Invalid request, saving campaign advanced configuration.')%>";
								break;

							case "UNABLE_TO_SAVE":
								message = "<%=_T('Unable to save campaign advanced configuration, please try again in a few minutes....')%>";
								break;

							case "FORBIDDEN":
								message = "<%=_T('You dont have permissions for save campaign advanced configuration.')%>";
								break;

							default:
								message = "<%=_T('Failed to save campaign advanced configuration, please try again in a few minutes...')%>";
						}
						save_callback( false, message );
					}
				}
			});
		};

		// aca guarda la configuracion de los capability
		var save_capability = function( save_callback )
		{
			self.frm.campaign.save_attention_capability({
				capability: capability,
				callback  : function( success, err )
				{
					if ( success ) {
						save_callback( true );
					}
					else {
						var message;
						var error = err.error;
						if ( typeof error === "object" ) {
							error = ( error.result || "").toUpperCase();
						}
						switch ( error )
						{
							case "INVALID_REQUEST":
								message = "<%=_T('Invalid request, saving campaign advanced configuration ( Capability ).')%>";
								break;
						}
						save_callback( false, message );
					}
				}
			});
		};

		var start_save_advanced_config = function()
		{
			// aca existe la configuracion guadar, un array de funciones, si en algun momento se necesitan guardar mas configuraciones
			// deberia ir aca
			var config_to_save = [
				save_capability,
				save_thresholds
			];
			// la cantidad de pendientes
			var count_pending_config = config_to_save.length;

			// recorro el array u e invoco a la funcion correspondiente para que guarde.
			$.each( config_to_save, function(_, fn ) {
				fn( function( success, err_msg ) 
					{
						if ( success ) {
							count_pending_config--;
							if ( count_pending_config === 0 ) {
								self.frm.show_status_message( "<%=_T('Campaign advanced configuration have been saved successfully.')%>", "Info" );
							}
						}
						else {
							self.frm.show_status_message( err_msg, "Error" );
						}
					}
				);
			});
		};

		// aca comienza a guardar las diferentes configuraciones ( capability, thresholds )
		this.frm.dashboard.show_waiting( "<%=_T('Please wait until campaign advanced config are saved.')%>" );
		start_save_advanced_config();
	};

	adm_form_campaigns_section_channels.prototype.set_events = function()
	{
		var self = this;

		$( "#adm_form_campaign_channels_save" )
			.off( "click" )
			.on( "click", function() {
				self.save_advanced_config();
			});

		$( "#adm_form_campaign_channels_cancel" )
			.off( "click" )
			.on( "click", function() {
				self.frm.close();
			});
		var event_selector = ".formTabsMenuItem";
		$( "#adm_form_campaign_channels" )
			.off( "click", event_selector)
			.on( "click", event_selector, function()
			{
				var $this = $(this);
				if ($this.hasClass("tabsMenuItemSelect")) {
					return;
				}
				$.each( self.channels_sections,
					function( proxy ) {
						var config_section = self.get_account_section( proxy );
						if ( config_section && $this.hasClass( config_section.handler_id ) ) {
							config_section.select();
						}
					});
			});
	};

	adm_form_campaigns_section_channels.prototype.initialize = function()
	{
		this.channels_sections = {};
		this.build_channels_sections();
		this.load_advanced_config();
	};

	adm_form_campaigns_section_channels.prototype.set = function( _ )
	{
		this.initialize();
	};

	adm_form_campaigns_section_channels.prototype.editable = function()
	{
		// La siguiente linea de codigo la dejo comentada, solo como referencia, ya que especificamente en esta seccion no se utiliza ( de momento )
		//   las propias acciones de editable la realiza la funcion de la propia clase.
		//
		// adm_form_campaigns_section_channels.uber.editable.call( this );

		this.show_save_action();

		$.each( this.channels_sections, function ( _, config_section ) {
			config_section.edit_from_channel_section();
		});
	};

	adm_form_campaigns_section_channels.prototype.readonly = function()
	{
		// La siguiente linea de codigo la dejo comentada, solo como referencia, ya que especificamente en esta seccion no se utiliza ( de momento )
		//   las propias acciones de readonly la realiza la funcion de la propia clase.
		//
		// adm_form_campaigns_section_channels.uber.readonly.call( this );

		this.hide_save_action();

		$.each( this.channels_sections, function ( _, config_section ) {
			config_section.readonly();
		});
	};

	adm_form_campaigns_section_channels.prototype.disable = function()
	{
		// La siguiente linea de codigo la dejo comentada, solo como referencia, ya que especificamente en esta seccion no se utiliza ( de momento )
		//   las propias acciones de disable la realiza la funcion de la propia clase.
		//
		// adm_form_campaigns_section_channels.uber.readonly.call( this );

		this.hide_save_action();

		$.each( this.channels_sections, function ( _, config_section ) {
			config_section.disable();
		});
	};

	adm_form_campaigns_section_channels.prototype.on_show = function()
	{
		adm_form_campaigns_section_channels.uber.on_show.call( this, { show: true } );
		this.frm.dashboard.$get_dom_element_by_id( "form_buttons_actions" ).hide();
		this.enabled_channels_sections();

		var is_readonly = this.is_readonly();
		if ( is_readonly ) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	//---------------------------------------------------------------------
	// form campaigns class - thresholds external config
	function adm_thresholds_external_config()
	{
		this.channels_sections = {};
		return true;
	}

	adm_thresholds_external_config.prototype.build_account_section = function( proxy, params )
	{
		var config_section;
		if ( proxy === "GMAIL" ) {
			proxy = "MAIL";
		}

		params.only_config_thresholds = true;

		switch ( proxy )
		{
			case "CALL":
				if ( inConcert.app.has_allegro_integration() ) {
					config_section = new Call_allegro_config( params );
				}
				else {
					config_section = new Call_config( params );
				}
				break;

			case "CHAT":
				config_section = new Web_chat_config( params );
				break;

			case "IM":
				config_section = new IM_config( params );
				break;

			case "FACEBOOK":
				config_section = new Facebook_config( params );
				break;

			case "INSTAGRAM":
				config_section = new Instagram_config( params );
				break;

			case "YOUTUBE":
				config_section = new Youtube_config( params );
				break;

			case "APPSTORE":
				config_section = new Appstore_config( params );
				break;

			case "GOOGLEPLAY":
				config_section = new Googleplay_config( params );
				break;

			case "RCS":
				config_section = new Rcs_config( params );
				break;

			case "MAIL": // MAIL o GMAIL
				config_section = new Mail_config( params );
				break;

			case "SMS":
				config_section = new SMS_config( params );
				break;

			case "WHATSAPP":
				config_section = new WHATSAPP_config( params );
				break;

			case "TWITTER":
				config_section = new Twitter_config( params );
				break;

			case "WEBCONTACT":
				config_section = new Contact_form_config( params );
				break;

			default:
				//throw("Unknown configuration type " + proxy);
				return;
		}

		this.channels_sections[ proxy ] = config_section;
		return config_section;
	};

	adm_thresholds_external_config.prototype.initialize = function( proxy, params )
	{
		var config_section = this.build_account_section( proxy, params );
		if ( config_section ) {
			config_section.set_current_config(params.account_id);
		}
		return config_section;
	};

	if ( typeof inConcert.adm_form_campaigns_section_channels !== "function" ) {
		inConcert.adm_thresholds_external_config = adm_thresholds_external_config;
		inConcert.adm_form_campaigns_section_channels = adm_form_campaigns_section_channels;
	}
}());
