/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, quotmark:false, -W014, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	var format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form campaigns class - basic section
	function adm_form_campaigns_section_groups( params )
	{
		var success = adm_form_campaigns_section_groups.uber.constructor.call( this, params );
		if (!success) {
			return;
		}

		this.view_action_id = "view_campaigns_roles_permissions"; // view action id de permisos
		this.save_action_id = "save_campaign_roles_permissions";  // save action id de permisos
		this.container_id   = "#adm_form_campaign_groups_permission_list_2014";

		this.perm_gui_override = new inConcert.perm_gui_override( {
			container_id: this.container_id,
			override_type: "campaign",
			cleanup_external_container_id: "adm_form_campaign_groups_cancel"
		});
		this.editing_row_id    = null;

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_groups, inConcert.conf_form_section );

	if ( typeof inConcert.adm_form_campaigns_section_groups !== "function" ) {
		inConcert.adm_form_campaigns_section_groups = adm_form_campaigns_section_groups;
	}

	adm_form_campaigns_section_groups.prototype.get_permissions_with_override = function()
	{
		var override_campaign_groups_permissions = {};
		var domain = inConcert.app.getDomain();
		var $rows = $("#adm_form_campaign_groups_roles_list li[row_type='role']");

		$.each( $rows, function( _, row )
		{
			var info = $( row ).data( "info" );
			var override = info.override_permissions;
			if ( override && ( override.apps || override.actions ) ) {
				// creo el role para realizar override
				var role_id = info.role_id + "@" + ( info.is_system ? "system" : domain );
				override_campaign_groups_permissions[ role_id ] = {};

				if ( override.apps ) {
				// override de las apps
					override_campaign_groups_permissions[ role_id ].apps = override.apps;
				}
				if ( override.actions ) {
				// override de las actions
					override_campaign_groups_permissions[ role_id ].actions = override.actions;
				}
			}
		});

		return override_campaign_groups_permissions;
	};

	adm_form_campaigns_section_groups.prototype.update_row = function()
	{
		if ( this.editing_row_id === null ) {
			return;
		}
		var $row = $( "#adm_form_campaign_groups_roles_list" ).find( "li[role_id='" + this.editing_row_id + "']" );
		if ( $row.length === 0 ) {
			return;
		}
		var info = $row.data( "info" );
		var previous_info = inConcert.Utils.cloneObject( info );
		$row.data( "previous_override_permissions", previous_info );
		var new_overrides = this.perm_gui_override.get_overrides();

		// actualizo la clase (css) que indica si se ha realizado override sobre el grupo en la lista
		if ( $row.hasClass( "override" ) ) {
			$row.removeClass( "override" );
		}
		if ( new_overrides.has_override === "1" ) {
			new_overrides = {
				apps    : new_overrides.apps,    // override de apps
				actions : new_overrides.actions  // override de actions
			};
			$row.addClass( "override" );
		}
		else {
			new_overrides = null;
		}

		// actualizo la info de la data a guardar
		info.override_permissions = new_overrides;

		$row.data( "info", info );
	};

	adm_form_campaigns_section_groups.prototype.rollback_update_row = function()
	{
		if ( this.editing_row_id === null ) {
			return;
		}
		var $row = $( "#adm_form_campaign_groups_roles_list" ).find( "li[role_id='" + this.editing_row_id + "']" );
		if ( $row.length === 0 ) {
			return;
		}
		var previous_info = $row.data( "previous_override_permissions" );
		var info = inConcert.Utils.cloneObject( previous_info );
		$row.data( "info", info );
		$row.data( "previous_override_permissions", {} );
		$row.removeClass( "override" );
		// actualizo la clase (css) que indica si se ha realizado override sobre el member en la lista
		if ( previous_info.override_permissions ) {
			$row.addClass( "override" );
		}
	};

	adm_form_campaigns_section_groups.prototype.validate_groups_permissions = function()
	{
		// valida que si se marco alguna aplicacion, que al menos se marque una accion de esta.
		var result = this.perm_gui_override.validate( "mark-at-least-a-action-of-the-app" );
		return result;
	};

	adm_form_campaigns_section_groups.prototype.save_override_groups_permissions = function()
	{
		var self = this;

		var response = this.validate_groups_permissions();
		if ( !response.success ) {
			this.frm.show_status_message( response.message, "Error" /* message type */, 5000 /* timeout ms */ );
			return;
		}

		this.frm.dashboard.show_waiting("<%=_T('Please wait until roles permissions are saved.')%>");

		this.update_row();
		var permissions = this.get_permissions_with_override();

		this.frm.campaign.saveRolePermissions( permissions, function( error )
		{
			var message, message_type, timeout_ms;
			if ( !error ) {
				message = "<%=_T('Permissions have been saved successfully.')%>";
				message_type = "Info";
				timeout_ms = null; // default time
				// aca lo que hago actualizar la seccion de members, en la lista de members ( adm_form_campaigns_section_members )
				//  el override de grupo que se ha guardado, para cada member pertenciente a este role.
				var members_override_section = self.frm.get_section_def( "members" );
				if ( members_override_section ) {
					var $row = $( "#adm_form_campaign_groups_roles_list li[row_type='role'][role_id='" + self.editing_row_id + "']" );
					var info = $row.data( "info" );
					var full_role_id = self.editing_row_id + "@" + ( info.is_system ? "system" : inConcert.app.getDomain() );
					var override_role;
					if ( permissions[ full_role_id ] ) {
						override_role = permissions[ full_role_id ];
					}
					members_override_section.frm.update_members_permissions_by_role( self.editing_row_id, override_role );
				}
			}
			else {
				var raw_error = error;
				if (typeof error === "object") {
					error = ( error.result || "").toUpperCase();
				}
				timeout_ms = 10000;
				switch (error)
				{
					case "INVALID_REQUEST":
						message = "<%=_T('Invalid request.')%>";
						break;

					case "UNABLE_TO_SAVE":
						message = "<%=_T('Unable to save campaign permissions, please try again in a few minutes....')%>";
						break;

					case "FORBIDDEN":
						message = "<%=_T('You dont have permissions for save permissions.')%>";
						self.rollback_update_row();
						break;

					case "USER_HAS_INTERACTIONS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP":
						message = format(
							"<%=_T('{0} {1} has assigned conversations. Unable to modify role permission, role not have permission to interaction app.')%>",
							raw_error.count_users,
							raw_error.count_users === 1 ? "<%=_T('user')%>" : "<%=_T('users')%>"
						);
						self.rollback_update_row();
						break;

					case "USER_IN_ATTENTION_LEVELS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP":
						message = "<%=_T('Attention levels conflict. Unable to modify role permission, role not have permission to interaction app.')%>";
						self.rollback_update_row();
						break;

					default:
						message = "<%=_T('Failed to save campaign permissions, please try again in a few minutes...')%>";
				}
				message_type = "Error";
			}
			self.hide_role_permissions();
			self.frm.show_status_message( message, message_type, timeout_ms );
		});
	};

	adm_form_campaigns_section_groups.prototype.show_role_permissions = function( $row )
	{
		var self = this;
		if ( !$row || $row.length === 0 ) {
			return;
		}

		var info = $row.data("info");
		this.editing_row_id = $row.attr( "role_id" );

		if ( !this.has_permission_to_section() ) {
			var message = format( "<%=_T('Unable to view {0} permissions: You dont have permissions for this action.')%>", info.role_name );
			this.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		if ( this.has_permission_to_save() ) {
			let title = format(format("<%=_T('Override Permissions: {0}')%>", "<span>\"{0}\"</span>"), info.role_name);
			$( "#adm_form_campaign_groups_permission_title" ).html( title );
			this.editable();
		}
		else {
			let title = format(format("<%=_T('View Permissions: {0}')%>", "<span>\"{0}\"</span>"), info.role_name);
			$( "#adm_form_campaign_groups_permission_title" ).html( title );
			this.readonly();
		}

		this.frm.change_title_section(this.section_id, info.role_name);

		this.perm_gui_override.mark_actions(
			info.group_permissions,
			function() {
				if ( info.override_permissions ) {
					//marco/desmarco las diferencias (los permisos customs)
					self.perm_gui_override.mark_difference_actions( info.override_permissions );
				}
				$("#adm_form_campaign_groups_roles_list_panel").hide();
				$("#adm_form_campaign_groups_roles_edit_panel").show();
				// google analytics
				self.AnalyticsSend();
			}
		);
	};

	adm_form_campaigns_section_groups.prototype.hide_role_permissions = function(canSendAnalytics = true)
	{
		this.editing_row_id = null;
		$("#adm_form_campaign_groups_roles_list_panel").show();
		$("#adm_form_campaign_groups_roles_edit_panel").hide();
		this.set_edit_mode(false);
		// google analytics
		if (canSendAnalytics)
			this.AnalyticsSend();
	};

	adm_form_campaigns_section_groups.prototype.before_jump_to_other_menunav = function(){
		this.hide_role_permissions(false /*canSendAnalytics*/);
	};

	adm_form_campaigns_section_groups.prototype.set_events = function()
	{
		var self = this;

		$("#adm_form_campaign_groups_cancel")
			.off("click")
			.on("click", function() {
				self.hide_role_permissions();
			});

		// seteo los eventos del container de acciones
		this.perm_gui_override.set_events();

		var grid_actions = function()
		{
			var $this = $( this );
			var action_type = $this.attr("action_type");
			var $row = $this.parent();

			if ( action_type === "edit" ) {
				self.show_role_permissions( $row );
			}
			else {
				throw "Unrecognized permission button action '" + action_type + "' ";
			}
		};

		var event_selector = "li button[action_type]";
		$( "#adm_form_campaign_groups_roles_list" )
			.off( "click", event_selector               )
			.on(  "click", event_selector, grid_actions );

		$("#adm_form_campaign_groups_save")
			.off("click")
			.on("click", function() {
				self.save_override_groups_permissions();
			});
	};

	adm_form_campaigns_section_groups.prototype.attach_load_request = function( request )
	{
		if ( !$.isArray( request ) ) {
			return;
		}
		var self = this;
		request.push(
			// acciones
			{
				// esto tiene que estar en una funcion dentro de otra, porque sino
				//	dentro de "load_apps_actions, no accede al this que corresponde.
				request_function: function( success ) {
					//realiza un return del resultado del air.invoke.
					//	el mismo es utilizado por si se desea cancelar el request.
					return inConcert.permissions.load_apps_actions( success );
				},
				success: function(application_actions) {
					self.perm_gui_override.build_apps_permissions( application_actions );
				}
			}
		);
	};

	adm_form_campaigns_section_groups.prototype.fill_grid = function()
	{
		var self = this;
		var sorted_roles = this.frm.get_sorted_roles();
		var $roles_list = $("#adm_form_campaign_groups_roles_list");

		// list header
		var $header =  $(
			"<li header='true' class='viewListTitle'>" +
				"<label class='viewListTitleText'>" + "<%=_T('Permission')%>" + "</label>" +
				"<label class='gridAction1ButtonCol'></label>" +
			"</li>");

		$roles_list.html("").append( $header );

		if (!$.isArray( sorted_roles ) || sorted_roles.length === 0) {
			return;
		}

		var is_readonly = this.is_readonly();
		var added = 0;

		$.each( sorted_roles, function( _, role )
		{
			var is_system_role = role.IsSystemGroup === "1";
			// si es un role del sistema y el VCC tiene integracion con Allegro, el mismo no queda disponible para realizar override.
			if (is_system_role && self.isAllegro()) {
				return true; // continue
			}

			var html_buttons = "";
			if ( is_readonly ) {
				html_buttons = "<button class='buttonColorGray gridActionButton viewPermissions last' type='button' action_type='edit' title='"+ "<%=_T('View')%>" + "'><i class='far fa-eye'></i>" + "<%=_T('View')%>" + "</button>";
			}
			else {
				html_buttons = "<button class='buttonColorGray gridActionButton editPermissions last' type='button' action_type='edit' title='"+ "<%=_T('Override')%>" + "'><i class=' buttons-icons-edit '></i>" + "<%=_T('Override')%>" + "</button>";
			}
			var name = role.Name || role.Id;

			var override_role_campaign = self.frm.campaign.getOverrideSpecificRolePermissions( role );
			var override_class = override_role_campaign ? " override" : "";
			var $role_row = $(
				"<li class='viewListItem" + override_class + "' role_id='" + role.Id + "' row_type='role' >" +
					"<label account_desc='true' class='viewListItemText'>" + name + "</label>" +
					html_buttons +
				"</li>");

			$role_row.data( 
				"info",
				{
					role_id              : role.Id               ,
					role_name            : name                  ,
					is_system            : is_system_role        ,
					group_permissions    : role.grouppermissions ,
					override_permissions : override_role_campaign
				}
			);

			$roles_list.append( $role_row );
			added++;
		});

		var $not_results_found = $("#adm_form_campaign_groups_roles_list_not_results_found");

		if (added > 0) {
			$roles_list.show();
			$not_results_found.text("").hide();
			return;
		}
		$roles_list.hide();
		$not_results_found.text("<%=_T('No roles found.')%>").show();
	};

	adm_form_campaigns_section_groups.prototype.initialize = function()
	{
		this.perm_gui_override.initialize();
		this.fill_grid();
	};

	adm_form_campaigns_section_groups.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_campaigns_section_groups.prototype.on_show = function()
	{
		adm_form_campaigns_section_groups.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id( "form_buttons_actions" ).hide();
	};

	adm_form_campaigns_section_groups.prototype.editable = function()
	{
		// La siguiente linea de codigo la dejo comentada, solo como referencia, ya que especificamente en esta seccion no se utiliza ( de momento )
		//   las propias acciones de readonly la realiza la funcion de propia clase.
		//
		// adm_form_campaigns_section_groups.uber.editable.call( this );

		this.perm_gui_override.editable();
		$( "#adm_form_campaign_groups_save" ).show();
		$( "#adm_form_campaign_groups_cancel" ).html( "<%=_T('Cancel')%>" );

		this.set_edit_mode(true);
	};

	adm_form_campaigns_section_groups.prototype.readonly = function()
	{
		// La siguiente linea de codigo la dejo comentada, solo como referencia, ya que especificamente en esta seccion no se utiliza ( de momento )
		//   las propias acciones de readonly la realiza la funcion de propia clase.
		//
		// adm_form_campaigns_section_groups.uber.readonly.call( this );

		this.perm_gui_override.readonly();
		$( "#adm_form_campaign_groups_save" ).hide();
		$( "#adm_form_campaign_groups_cancel" ).html( "<%=_T('Close')%>" );
	};

	adm_form_campaigns_section_groups.prototype.AnalyticsSend = function(info = {})
	{
		// basic info
		const analyticsInfo = {};
		// el path viene se envia en un array
		if (this.editing_row_id) {
			analyticsInfo.source_path = [
				{name: "role_id"  , value: `${this.editing_row_id}/override`} // "/main/administration/campaigns/edit/camp1@vcc1/groups/agent/override"
			].concat(info.source_path);
		}
		// invoco a la clase base de secciones
		adm_form_campaigns_section_groups.uber.AnalyticsSend.call(this, analyticsInfo);
	};
}());
