/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:false, -W014, esversion: 11 */
/* global inConcert
*/

/* --------------------------------------------------------------------------------------------------------------------------------------

	Descripcion de clases de la integracion CCI
	Tanto con o sin integracion con Allegro.
	Tambien estan las clases de cada tipo de integracion CCI.
	Es decir, si es web (externo) o si es contra el webdesigner.

	A continuacion agrego un diagrama de la herencia de las mismas y donde se utilizan.

	La clase de las secciones de la campana "inConcert.adm_form_campaigns" (adm_form_campaigns.js) es quien instancia las clases
		"adm_form_campaigns_section_cci" o "adm_form_campaigns_section_cci_allegro" dependiendo si la campana tiene integracion con Allegro.

		-(A)-----------------------------------------------------------------
		inConcert.conf_form_section
			|
			|_adm_form_campaigns_section_cci_base
					|
					|_adm_form_campaigns_section_cci
					|
					|_adm_form_campaigns_section_cci_allegro
					
		------------------------------------------------------------------

	Estas son las clases para cada tipo de configuracion CCI, es decir, si es web o si es de webdesigner.

	A continuacion existe el diagrama de herencia y en que clase de (A) se instancia instancia.

	Campana SIN integracion con Allegro (adm_form_campaigns_section_cci) instancia las clases:
		"cci_unselected_panel"
		"cci_web_tweety_panel_i6"
		"cci_web_external_panel"

	Campana CON integracion con Allegro (adm_form_campaigns_section_cci_allegro) instancia las clases:
		"cci_unselected_panel"
		"cci_web_tweety_panel_i6"
		"cci_web_tweety_panel_allegro"
		"cci_web_external_panel_allegro"

		-(B)-----------------------------------------------------------------
		cci_panel
			|
			|_ cci_unselected_panel
			|
			|_ cci_base_web_panel
					|
					|_cci_web_tweety_panel
					|		|
					|		|_cci_web_tweety_panel_i6
					|		|
					|		|_cci_web_tweety_panel_allegro
					|
					|_cci_web_external_panel
							|
							|_cci_web_external_panel_allegro

		------------------------------------------------------------------

		-(C)-----------------------------------------------------------------
		cci_web_parameters
	
			Esta clase es la encargada de realizar el ABM de parametros.
			A la misma la instancia la clase base "cci_base_web_panel",
			es decir todas sus clases hijas la utilizan (Ej: cci_web_tweety_panel->cci_web_tweety_panel_i6) etc
		------------------------------------------------------------------

 --------------------------------------------------------------------------------------------------------------------------------------*/


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//"use strict";

	const FORMAT = inConcert.Utils.format;
	const sanitize_id = function(id) { return id.replace("#", ""); };

	//---------------------------------------------------------------------
	// class base panel
	function cci_panel(params)
	{
		this.section = params.section;
		this.data = params.data;
		this.additional_prefix_id = params.additional_prefix_id;
		this.initialize(params);
	}

	cci_panel.prototype.get = function() {
		return {
			type: this.get_panel_id()
		};
	};

	cci_panel.prototype.set        = $.noop;
	cci_panel.prototype.load_data  = $.noop;
	cci_panel.prototype.editable   = $.noop;
	cci_panel.prototype.readonly   = $.noop;
	cci_panel.prototype.set_events = $.noop;
	cci_panel.prototype.dispose    = $.noop;

	cci_panel.prototype.get_panel_name = function() { return this.panel_name;              };
	cci_panel.prototype.get_panel_id   = function() { return this.panel_id;                };
	cci_panel.prototype.hide           = function() { $(this.container_id).hide();         };
	cci_panel.prototype.show           = function() { $(this.container_id).show();         };
	cci_panel.prototype.initialize     = function(params) {
		this.set_initial_data(params);
		this.set_events();
		this.load_data();
	};

	cci_panel.prototype.validate = function()
	{
		this.section.frm.hide_messages();
		return { success: true, error_message: "" };
	};

	cci_panel.prototype.validate_field = function($ctrl, validation_type, message)
	{
		return this.section.frm.validate_field($ctrl, validation_type, message);
	};

	cci_panel.prototype.show_validation_error = function($ctrl, message)
	{
		return this.section.frm.show_validation_error($ctrl, message);
	};

	cci_panel.prototype.set_initial_data = function(params)
	{
		this.prefix_id = params.prefix_id;
		let full_prefix = this.get_full_prefix();
		this.container_id = full_prefix + params.suffix_id;
	};

	cci_panel.prototype.get_full_prefix = function(params)
	{
		let self = params || this;
		if (!self.additional_prefix_id) {
			return self.prefix_id;
		}
		return self.prefix_id + self.additional_prefix_id;
	};

	//---------------------------------------------------------------------
	// class unselected panel
	function cci_unselected_panel(params)
	{
		this.panel_id = "UNSELECTED";
		this.panel_name = "Unselected panel";
		cci_unselected_panel.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_unselected_panel, cci_panel);

	cci_unselected_panel.prototype.get = function() { return null; /* empty */ };

	cci_unselected_panel.prototype.validate = function()
	{
		var response = cci_unselected_panel.uber.validate.call(this);
		if (!$(this.prefix_id + "_enabled_integration").prop("checked")) {
			return response;
		}
		var $cbo_cci_types = $(this.prefix_id + "_type");
		var message_error = "<%=_T('Please select integration type.')%>";
		if (!this.validate_field($cbo_cci_types, "required", message_error)) {
			response.success = false;
			return response;
		}
		return response;
	};

	//---------------------------------------------------------------------
	// class base web panel
	function cci_base_web_panel(params)
	{
		this.prefix_id = params.prefix_id;
		this.has_override = params.has_override;

		const full_prefix = this.get_full_prefix(params);
		this.app_name = full_prefix + "_app_name";
		if (this.has_override) {
			this.override = full_prefix + "_override_finish_action";
		}

		this.parameters = new cci_web_parameters({
			panel                        : this,
			popup_id                     : params.popup_id,
			full_prefix                  : full_prefix,
			system_parameters_names      : params.system_parameters_names || {}, // excluded or reserved word,
			system_interaction_parameters: this.get_system_interaction_parameters(),
			context_interaction_types	 : this.get_context_interaction_types()
		});

		cci_base_web_panel.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_base_web_panel, cci_panel);

	/*
		Esta lista debe coincidir con los parametros reemplazables en:
		En el archivo:  inconcert\agent\inconcert\apps\agent\scripts\integration.js
		Funcion: function getParameterValue
	*/
	cci_base_web_panel.prototype.get_system_interaction_parameters = function()
	{
		return {
			"{ADDRESS}"        : "<%=_T('Address')%>",
			"{CALLERID}"       : "<%=_T('Caller Id')%>",
			"{CONTACT_COMPANY}": "<%=_T('Contact Company')%>",
			"{CONTACT_ID}"     : "<%=_T('Contact Id')%>",
			"{CONTACT_NAME}"   : "<%=_T('Contact Name')%>",
			"{FIRST_MESSAGE}"  : "<%=_T('First Message')%>",
			"{FROM}"           : "<%=_T('From')%>",
			"{INTERACTIONID}"  : "<%=_T('Conversation Id')%>",
			"{SUBJECT}"        : "<%=_T('Subject')%>",
			"{TIMESTAMP}"      : "<%=_T('Time Stamp')%>",
			"{TO}"             : "<%=_T('To')%>",
			"{USER}"           : "<%=_T('User')%>",
			"{USER_AUTH_TOKEN}": "<%=_T('User Token')%>",
			"{USER_NAME}"      : "<%=_T('User Name')%>",
			"{USER_STATUS}"    : "<%=_T('User Status')%>",
			"{VIRTUALCC}"      : "<%=_T('Vcc Id')%>"
		};
	};

	cci_base_web_panel.prototype.get_context_interaction_types = function()
	{
		return {
			"params"           : "<%=_T('Webchat Params')%>",
			"pre_survey"       : "<%=_T('Webchat Pre Survey')%>",
			"login"            : "<%=_T('Webchat Login')%>",
			"contactform"      : "<%=_T('Contact Form')%>",
			"api"              : "<%=_T('Contact Form API')%>",
			"dsbatch_data"     : "<%=_T('Outgoing Batch Data')%>",
			"process_flow_data": "<%=_T('Process Flow')%>"
		};
	};

	cci_base_web_panel.prototype.load_data = function()
	{
		this.parameters.load_data();
	};

	cci_base_web_panel.prototype.set_events = function()
	{
		this.parameters.set_events();
	};

	cci_base_web_panel.prototype.editable = function()
	{
		// enable actions popup parameters (new, delete, edit)
		this.parameters.disable_actions( false );
	};

	cci_base_web_panel.prototype.readonly = function()
	{
		// disable actions popup parameters (new, delete, edit)
		this.parameters.disable_actions( true );
	};

	cci_base_web_panel.prototype.dispose = function()
	{
		cci_base_web_panel.uber.dispose.call(this);
		if (this.parameters) {
			this.parameters.dispose();
			this.parameters = null;
		}
	};

	cci_base_web_panel.prototype.get = function()
	{
		var integration = cci_base_web_panel.uber.get.call(this);
		integration.application = $(this.app_name).val();
		integration.popup = {
			parameters: this.parameters.get()
		};
		integration.override_actions = this.get_override_actions();
		return integration;
	};

	cci_base_web_panel.prototype.set = function(integration)
	{
		if (typeof integration !== "object") {
			return;
		}
		$(this.app_name).val(integration.application);
		this.set_override_actions(integration.override_actions);
		this.parameters.set(integration.popup.parameters);

		return integration;
	};

	cci_base_web_panel.prototype.show = function()
	{
		cci_base_web_panel.uber.show.call(this);

		var $app_name = $(this.app_name);
		$app_name.focus();
	};

	cci_base_web_panel.prototype.get_override_actions = function()
	{
		if (!this.has_override) {
			return undefined;
		}
		// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
		var $override_finish = $(this.override);
		if ($override_finish.prop("checked")) {
			return { finish: { show: "popup"} };
		}
		return undefined;
	};

	cci_base_web_panel.prototype.set_override_actions = function(actions)
	{
		if (!this.has_override) {
			return;
		}
		// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
		var override = false;
		var finish_action = "finish";
		if (typeof actions === "object" &&
			typeof actions[finish_action] === "object" &&
			actions[finish_action].show === "popup") {
				override = true;
			}
		var $override_finish = $(this.override);
		$override_finish.prop("checked", override);
	};

	cci_base_web_panel.prototype.validate = function()
	{
		var response = cci_base_web_panel.uber.validate.call(this);

		var $app_name = $(this.app_name);
		var message_error = "<%=_T('Please enter the application name.')%>";
		if (!this.validate_field($app_name, "required", message_error)) {
			response.success = false;
			return response;
		}
		message_error = "<%=_T('Only letters, numbers, dashes and dots are allowed in the application name.')%>";
		if (!this.validate_field($app_name, "html_element_id", message_error)) {
			response.success = false;
			return response;
		}
		return response;
	};

	cci_base_web_panel.prototype.__get_simple_prefix_id = function(prefix_id)
	{
		var pf = prefix_id || this.prefix_id || "";
		return sanitize_id(pf);
	};

	//---------------------------------------------------------------------
	// class web tweety panel
	function cci_web_tweety_panel(params)
	{
		params.suffix_id = "_panel";
		let full_prefix = this.get_full_prefix(params);
		var simple_prefix_id           = this.__get_simple_prefix_id(full_prefix);
		params.popup_id                = simple_prefix_id + "_popup_parameters"; // no id de jquery
		params.system_parameters_names = { 
			interaction_id			: true,
			interactionid			: true,
			agentid					: true,
			virtualcc				: true
		}; // excluded or reserved words

		cci_web_tweety_panel.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_web_tweety_panel, cci_base_web_panel);

	cci_web_tweety_panel.prototype.load_data = function()
	{
		cci_web_tweety_panel.uber.load_data.call(this);
		this.__fill_combo_projects();
	};

	cci_web_tweety_panel.prototype.get_projects = function()
	{
		return this.data.projects || {};
	};

	cci_web_tweety_panel.prototype.__fill_combo_projects = function()
	{
		var $combo = $(this.app_name);
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "name";
		var sorted = true;
		var list = this.get_projects();

		this.section.frm.fill_combo($combo, is_blank_first_item, list, attr_id, attr_name, sorted);
	};

	cci_web_tweety_panel.prototype.get = function()
	{
		var integration = cci_web_tweety_panel.uber.get.call(this);
		integration.popup.method = "GET";
		var app_name = $(this.app_name).val();
		integration.popup.url = inConcert.app.getTweetyDesignerUrl() + "/" + app_name;
		return integration;
	};

	cci_web_tweety_panel.prototype.set_events = function() { cci_web_tweety_panel.uber.set_events.call(this); };


	//---------------------------------------------------------------------
	// class web tweety panel i6
	function cci_web_tweety_panel_i6(params)
	{
		this.panel_id = "CTIPOPUP";
		this.panel_name = "<%=_T('inConcert Omnichannel Web Application')%>";
		cci_web_tweety_panel_i6.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_web_tweety_panel_i6, cci_web_tweety_panel);

	//---------------------------------------------------------------------
	// class web tweety panel allegro
	function cci_web_tweety_panel_allegro(params)
	{
		this.panel_id = "CTIALLEGROPOPUP";
		this.panel_name = "<%=_T('inConcert Allegro Web Application')%>";
		cci_web_tweety_panel_allegro.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_web_tweety_panel_allegro, cci_web_tweety_panel);

	/*
		En esta funcion de load_data de la clase de Allegro, tengo que realizar el request correspondiente para cargar el cbo con la lista de proyectos.
		Esto lo tengo que realizar asi, y no lo puedo hacer en la funcion "attach_load_request", que implementan cada una de las secciones,
		porque si llega a fallar el request que obtiene la lista de proyectos de tweety de 5.5, el formulario no carga correctamente.
		Arreglar esto, demandaria revisar todo lo que hoy existe, con el mecanismo de adjuntar request de cada seccion.
		Entonces para poder realizarlo mas rapido (Total va a funcionar igual), lo realizo aqui al request (una unica vez).
		Despues tengo que leer el resto de los datos locales "self.load_integration();"
	*/
	cci_web_tweety_panel_allegro.prototype.load_data = function()
	{
		cci_web_tweety_panel_allegro.uber.load_data.call(this);

		var self = this;

		var allegro_api = inConcert.app.getAllegroAPI();
		allegro_api.loadCCIProjects(function(success, projects)
		{
			if (!success) {
				var message = "<%=_T('Error retrieving inconcert allegro web applications.')%>";
				var timeout_ms = false; //infinite
				self.section.frm.show_status_message(message, "Error", timeout_ms);
				return;
			}
			self.section.set_tweety_allegro_projects( projects );
			self.__fill_combo_projects();
			self.section.load_integration();
		});
	};

	//---------------------------------------------------------------------
	// class web external panel
	function cci_web_external_panel(params)
	{
		this.panel_id = "CTIPOPUPWEB";
		this.panel_name = "<%=_T('Web Application')%>";
		params.suffix_id = "_panel";

		this.cbo_method = params.prefix_id + "_web_external_method";
		this.uri = params.prefix_id + "_web_external_uri";

		var simple_prefix_id = this.__get_simple_prefix_id(params.prefix_id);
		params.popup_id = simple_prefix_id + "_web_external_popup_parameters"; // no id de jquery
		params.system_parameters_names = {}; // excluded or reserved word

		cci_web_external_panel.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_web_external_panel, cci_base_web_panel);

	cci_web_external_panel.prototype.load_data = function()
	{
		cci_web_external_panel.uber.load_data.call(this);

		var $combo = $(this.cbo_method);
		var is_blank_first_item = true;
		var attr_id = null;
		var attr_name = null;
		var sorted = true;
		var list = {
			GET:  "<%=_T('Get')%>",
			POST: "<%=_T('Post')%>"
		};

		this.section.frm.fill_combo($combo, is_blank_first_item, list, attr_id, attr_name, sorted);
	};

	cci_web_external_panel.prototype.get = function()
	{
		var integration = cci_web_external_panel.uber.get.call(this);
		integration.popup.url = $(this.uri).val();
		integration.popup.method = $(this.cbo_method).val();
		return integration;
	};

	cci_web_external_panel.prototype.set = function(integration)
	{
		cci_web_external_panel.uber.set.call(this, integration);
		if (typeof integration !== "object" || typeof integration.popup !== "object") {
			return;
		}
		$(this.uri).val(integration.popup.url);
		$(this.cbo_method).val(integration.popup.method);
		return integration;
	};

	cci_web_external_panel.prototype.validate_uri = function(response)
	{
		var $uri = $(this.uri);
		var message_error = "<%=_T('Invalid URI.')%>";
		if (!this.validate_field($uri, "url", message_error)) {
			response.success = false;
			return response;
		}
		return response;
	};

	cci_web_external_panel.prototype.validate = function()
	{
		var response = cci_web_external_panel.uber.validate.call(this);
		if (!response.success) {
			return response;
		}

		var $uri = $(this.uri);
		var message_error = "<%=_T('Please enter the URI.')%>";
		if (!this.validate_field($uri, "required", message_error)) {
			response.success = false;
			return response;
		}
		response = this.validate_uri(response);
		if (!response.success) {
			return response;
		}

		var $method = $(this.cbo_method);
		message_error = "<%=_T('Please select the method type.')%>";
		if (!this.validate_field($method, "required", message_error)) {
			response.success = false;
			return response;
		}
		if ($method.val() === "POST") {
			var total_parameters = this.parameters.count();
			if (total_parameters === 0) {
				var $parameters = $(this.parameters.param_new);
				message_error = "<%=_T('When using POST, you must specify at least one parameter.')%>";
				this.show_validation_error($parameters, message_error);
				response.success = false;
				return response;
			}
		}
		return response;
	};

	//---------------------------------------------------------------------
	// class allegro web external panel
	function cci_web_external_panel_allegro(params)
	{
		cci_web_external_panel_allegro.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(cci_web_external_panel_allegro, cci_web_external_panel);

	cci_web_external_panel_allegro.prototype.validate_uri = function(response)
	{
		var $uri = $(this.uri);
		var uri = $uri.val();
		var is_uri = inConcert.Utils.validate(uri, "url");
		if (is_uri.success) {
			return response;
		}
		var is_variable = inConcert.Utils.validate(uri, "variable");
		if (is_variable.success) {
			return response;
		}
		// error no es una URI ni una variable valida.
		var message_error = "<%=_T('Please enter a valid URI! Or a valid variable (Letters, numbers, underscores).')%>";
		this.show_validation_error($uri, message_error);
		response.success = false;
		return response;
	};

	cci_web_external_panel_allegro.prototype.get = function()
	{
		var integration = cci_web_external_panel_allegro.uber.get.call(this);
		integration.popup.url_is_variable = inConcert.Utils.validate(integration.popup.url, "variable").success;
		return integration;
	};

	//---------------------------------------------------------------------
	// class parameters (web)
	function cci_web_parameters(params)
	{
		var self = this;
		this.panel = params.panel;
		this.prefix_id = params.full_prefix;

		this.param_section         = this.prefix_id + "_parameters_section"                     ;
		this.param_new             = this.prefix_id + "_parameter_new"                          ;
		this.param_popup           = this.prefix_id + "_popup_parameters"                       ;
		this.param_popup_title     = this.prefix_id + "_popup_title"                            ;
		this.param_create          = this.prefix_id + "_param_create"                           ;
		this.param_create_continue = this.prefix_id + "_param_create_and_continue"              ;
		this.param_cancel          = this.prefix_id + "_param_cancel"                           ;
		this.param_message         = this.prefix_id + "_popup_message"                          ;
		this.param_list            = this.prefix_id + "_parameters"                             ;
		this.param_name            = this.prefix_id + "_popup_name"                             ;
		this.param_custom_value    = this.prefix_id + "_popup_custom_value"                     ;
		this.param_context_type    = this.prefix_id + "_popup_context_type"                     ;
		this.param_context_value   = this.prefix_id + "_popup_context_value"                    ;
		this.param_system_value    = this.prefix_id + "_popup_system_value"                     ;
		this.param_options_id      = this.prefix_id + "_popup_options_value"                    ;
		this.param_options_value   = this.prefix_id + "_popup_options_value input[type='radio']";
		this.param_option_custom   = this.prefix_id + "_popup_opt_custom"                       ;
		this.param_option_context  = this.prefix_id + "_popup_opt_context"                      ;
		this.param_option_system   = this.prefix_id + "_popup_opt_system"                       ;

		this.system_parameters_names = params.system_parameters_names || {}; // excluded or reserved word
		this.system_interaction_parameters = params.system_interaction_parameters;
		this.context_interaction_types = params.context_interaction_types;

		this.popup_instance = new inConcert.PopupWindow(
		{
			ownerId: params.popup_id, // container id
			eventName: params.popup_id, // evento click personalizado
			callbacks: {
				onClose: function(){
					self.hide_popup();
				}
			}
		});

		this.render();
	}

	cci_web_parameters.prototype.__get_system_params = function() { return this.system_interaction_parameters; };

	cci_web_parameters.prototype.__get_context_types = function() { return this.context_interaction_types; };

	cci_web_parameters.prototype.render = function()
	{
		let html_values = "";

		if ( this.__get_system_params() ) {
			const customTooltip = "<%=_T('Fixed value')%>";
			const contextTooltip = "<%=_T('Context attribute')%>";
			const systemTooltip = "<%=_T('System attribute')%>";
			html_values = "".concat(
				// "<div class='row'>",
					"<div class='col-6'>",
						"<div class='form-group' id='", sanitize_id(this.param_options_id), "'>",
							"<div class='form-radiobutton'>",
								"<label for='", sanitize_id(this.param_option_custom), "' title='", customTooltip, "' class='form-radiobutton-label'>",
									"<input id='", sanitize_id(this.param_option_custom), "' type='radio' name='custom_system_value' value='custom' class='form-radiobutton-input' title='", customTooltip, "' />",
									"<span>", "<%=_T('Fixed Parameter')%>", "</span>",
								"</label>",
							"</div>",
							"<div class='form-radiobutton'>",
								"<label for='", sanitize_id(this.param_option_context), "' title='", contextTooltip, "' class='form-radiobutton-label'>",
									"<input id='", sanitize_id(this.param_option_context), "' type='radio' name='custom_system_value' value='context' class='form-radiobutton-input' title='", contextTooltip, "' />",
									"<span>", "<%=_T('Contextualized Parameter')%>", "</span>",
								"</label>",
							"</div>",
							"<div class='form-radiobutton'>",
								"<label for='", sanitize_id(this.param_option_system), "' title='", systemTooltip, "' class='form-radiobutton-label'>",
									"<input id='", sanitize_id(this.param_option_system), "' type='radio' name='custom_system_value' value='system' class='form-radiobutton-input' title='", systemTooltip, "' />",
									"<span>", "<%=_T('Use System Value')%>", "</span>",
								"</label>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",
				"<hr class='mt-0' />",
				"<div class='row'>",
					"<div class='col-6' id='", sanitize_id(this.param_context_type), "_container'>",
						"<div class='form-group'>",
							"<label class='required' id='", sanitize_id(this.param_context_type), "_label' style='display:none;'>", "<%=_T('Type')%>", "<%=_T(':')%>", "</label>",
							"<select id='", sanitize_id(this.param_context_type), "' class='form-control' style='display:none;'></select>",
						"</div>",
					"</div>",
					"<div class='col-6'>",
						"<div class='form-group'>",
							"<label class='required'>" + "<%=_T('Value')%>" + "<%=_T(':')%>" + "</label>",
							"<input id='", sanitize_id(this.param_custom_value), "' class='form-control' type='text' style='display:none;' maxlength='100' />",
							"<i id='", sanitize_id(this.param_custom_value), "_validation' class='formFieldValidation formFieldError' hidden ></i>",
							"<input id='", sanitize_id(this.param_context_value), "' class='form-control' type='text' style='display:none;' maxlength='100' style='display:none;' />",
							"<i id='", sanitize_id(this.param_context_value), "_validation' class='formFieldValidation formFieldError' hidden ></i>",
							"<select id='", sanitize_id(this.param_system_value), "' class='form-control' style='display:none;'></select>",
							"<i id='", sanitize_id(this.param_system_value), "_validation' class='formFieldValidation formFieldError' hidden ></i>",
						"</div>",
					"</div>",
				"</div>"
			);
		}
		else {
			html_values = "".concat(
				"<hr class='mt-0' />",
				"<div class='row'>",
					"<div class='col-6'>",
						"<div class='form-group'>",
							"<label class='required'>" + "<%=_T('Value')%>" + "<%=_T(':')%>" + "</label>",
							"<input id='", sanitize_id(this.param_custom_value), "' class='form-control' type='text' maxlength='100' placeholder='", "<%=_T('Enter value')%>", "...' />",
							"<i id='", sanitize_id(this.param_custom_value), "_validation' class='formFieldValidation formFieldError' hidden ></i>",
						"</div>",
					"</div>",
				"</div>",
			);
		}
		const sanitized_id_param_new = sanitize_id(this.param_new);
		let html_parameters = "".concat(
			"<div class='row'>",
				"<div class='col-12'>",
					"<div class='card-header d-flex flex-row justify-content-between'>",
						"<h5><%=_T('Parameters')%></h5>",
						"<button id='", sanitized_id_param_new, "' class='pageButtonGeneric buttonColorWhite pageButtonNew'> <i class='far fa-plus'> </i> <%=_T('Add Parameter')%> </button>",
					"</div>",
				"</div>",
			"</div>",
			"<div id='", sanitize_id(this.param_popup), "' style='display:none;'>",
				"<div class='row'>",
					"<div class='col-6'>",
						"<div class='form-group'>",
							"<label for='", sanitize_id(this.param_name), "' class='required'>", "<%=_T('Name')%>", "<%=_T(':')%>", "</label>",
							"<input id='", sanitize_id(this.param_name), "' class='form-control' type='text' maxlength='50' placeholder='", "<%=_T('Enter name')%>", "...' />",
							"<i id='", sanitize_id(this.param_name), "_validation' class='formFieldValidation formFieldError' hidden ></i>",
						"</div>",
					"</div>",
				// "</div>",
				html_values,
				// buttons
				"<div class='row custom-form-buttons-container'>",
					"<div class='col-12'>",
						"<button id='", sanitize_id(this.param_create), "' class='pageButtonGeneric buttonColorWhite mr-2'><%=_T('Confirm')%></button>",
						"<button id='", sanitize_id(this.param_create_continue), "' class='pageButtonGeneric buttonColorWhite mr-3'><%=_T('Confirm & continue')%></button>",
						"<button id='", sanitize_id(this.param_cancel), "' class='pageButtonGeneric buttonColorWhite' ><%=_T('Cancel')%></button>",
						"<div id='", sanitize_id(this.param_message), "' class='genericFormMessagesContainer messageError' hidden>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='row'>",
					"<div class='col-12'>",
						"<ul id='", sanitize_id(this.param_list), "' class='formList formListAsGrid mt-0'>",
							"<li class='formListTitle'>",
								"<label head='true' class='formListTitleText nameColumn'>", "<%=_T('Name')%>", "</label>",
								"<label head='true' class='formListTitleText valueColumn'>", "<%=_T('Value')%>", "</label>",
								"<label class='gridAction2ButtonCol'></label>",
							"</li>",
						"</ul>",
				"</div>",
			"</div>"
			);
		$(this.param_section).html(html_parameters);
	};

	cci_web_parameters.prototype.load_data = function()
	{
		var system_params = this.__get_system_params();
		var context_types = this.__get_context_types();
		if (!system_params) {
			return;
		}
		this.panel.section.frm.fill_combo($(this.param_system_value), true, system_params, null, null, true);
		this.panel.section.frm.fill_combo($(this.param_context_type), false, context_types, null, null, false);
	};

	cci_web_parameters.prototype.dispose = function()
	{
		if (this.popup_instance) {
			if ( this.popup_instance.isVisible() ) {
				this.popup_instance.hide();
			}
			this.popup_instance = null;
		}
	};

	cci_web_parameters.prototype.count = function()
	{
		// retorna la cantidad de parametros
		var count = 0;
		$(this.param_list).find("li[param_data]").each(function() { count++; });
		return count;
	};

	cci_web_parameters.prototype.get = function()
	{
		var parameters = {};
		// carga la lista de parameters
		$(this.param_list).find("li[param_data]").each(function(_, param_data)
		{
			var $param_data = $(param_data);
			var name = $param_data.find("label[param_name]").text();
			var value = $param_data.find("label[param_value]").attr("value_id");
			parameters[name] = value;
		});
		return parameters;
	};

	cci_web_parameters.prototype.set = function(parameters)
	{
		var self = this;
		$(this.param_list).find("li[param_data]").remove();
		if (!parameters) {
			return;
		}
		$.each(parameters, function(name, value)
		{
			self.add_row(name, value);
		});
	};

	cci_web_parameters.prototype.show_popup = function( is_new )
	{
		this.hide_errors();

		if ( !this.panel.section.has_permission_to_section() || !this.panel.section.has_permission_to_save() ) {
			var message;
			if ( is_new ) {
				message = "<%=_T('Unable to add cci parameter: You dont have permissions for this action.')%>";
			}
			else {
				message = "<%=_T('Unable to edit cci parameter: You dont have permissions for this action.')%>";
			}
			this.panel.section.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		this.popup_instance.show();
		// disable actions popup parameters (new, delete, edit)
		this.disable_actions(true);
		var type;
		if (!this.$row_editing) {
		// si presione el boton para crear uno nuevo
			$(this.param_popup_title).text("<%=_T('New Parameter')%>");
			$(this.param_create).val("<%=_T('Create')%>");
			$(this.param_create_continue).show();
			$(this.param_name).focus().val("");
			$(this.param_system_value).val("");
			$(this.param_context_type).val("");
			$(this.param_context_value).val("");
			$(this.param_custom_value).val("");
			$(this.param_option_custom).prop("checked", true);
			type = "custom";
		}
		else {
		// si presione el boton para editar
			$(this.param_popup_title).text("<%=_T('Update Parameter')%>");
			$(this.param_create).val("<%=_T('Update')%>");
			$(this.param_create_continue).hide();

			var name = this.$row_editing.find("label[param_name='true']").text();
			$(this.param_name).focus().val(name);

			var value = this.$row_editing.find("label[param_value='true']").attr("value_id");
			if (this.is_system_value(value)) {
				value = value.toUpperCase();
				$(this.param_system_value).val(value);
				$(this.param_custom_value).val("");
				$(this.param_context_type).val("");
				$(this.param_context_value).val("");
				$(this.param_option_system).prop("checked", true);
				type = "system";
			}
			else if (this.is_context_value(value)) {
				value = this.get_context_value(value);
				$(this.param_system_value).val("");
				$(this.param_custom_value).val("");
				$(this.param_context_type).val(value.type);
				$(this.param_context_value).val(value.value);
				$(this.param_option_context).prop("checked", true);
				type = "context";
			}
			else {
				$(this.param_system_value).val("");
				$(this.param_context_type).val("");
				$(this.param_context_value).val("");
				$(this.param_custom_value).val(value);
				$(this.param_option_custom).prop("checked", true);
				type = "custom";
			}
		}
		this.switch_popup_value(type);
	};

	cci_web_parameters.prototype.hide_popup = function()
	{
		if (this.popup_instance) {
			this.popup_instance.hide();
		}
		// enable actions popup parameters (new, delete, edit)
		this.disable_actions(false);
	};

	cci_web_parameters.prototype.switch_popup_value = function(value_type)
	{
		if (value_type === "system") {
			$(this.param_custom_value).hide();
			$(this.param_context_type).hide();
			$(this.param_context_type + "_label").hide();
			$(this.param_context_type + "_container").hide();
			$(this.param_context_value).hide();
			$(this.param_system_value).show();
		}
		else if (value_type === "context") {
			$(this.param_system_value).hide();
			$(this.param_custom_value).hide();
			$(this.param_context_type).show();
			$(this.param_context_type + "_label").show();
			$(this.param_context_type + "_container").show();
			$(this.param_context_value).show();
		}
		else {
			$(this.param_system_value).hide();
			$(this.param_context_type).hide();
			$(this.param_context_type + "_label").hide();
			$(this.param_context_type + "_container").hide();
			$(this.param_context_value).hide();
			$(this.param_custom_value).show();
		}
	};

	cci_web_parameters.prototype.is_system_value = function(value)
	{
		if (typeof value !== "string") {
			return false;
		}
		var system_params = this.__get_system_params();
		if (system_params && system_params.hasOwnProperty(value.toUpperCase())) {
			return true;
		}
		return false;
	};

	cci_web_parameters.prototype.is_context_value = function(value)
	{
		if (typeof value !== "string") {
			return false;
		}
		let context_types = this.__get_context_types();
		return value.indexOf(".") > 0 && Object.keys(context_types || {}).find(function(contextType) {
			return value.indexOf(contextType + ".") === 0;
		}) || false;
	};
	
	cci_web_parameters.prototype.get_context_value = function(value) {
		let value_a = value.split(".");
		return {
			type : value_a[0],
			value : value_a[1]
		};
	};

	cci_web_parameters.prototype.is_system_name = function(name)
	{
		if (typeof name !== "string") {
			return false;
		}
		name = name.trim().toLowerCase();
		if (this.system_parameters_names.hasOwnProperty(name)) {
			return true;
		}
		return false;
	};

	cci_web_parameters.prototype.can_add_row = function(name)
	{
		name = (name || "").trim().toLowerCase();
		if (this.system_parameters_names.hasOwnProperty(name)) {
			return false;
		}
		return true;
	};

	cci_web_parameters.prototype.add_row = function(name, value, apply_effect)
	{
		if (typeof name !== "string" || typeof value !== "string") {
			return;
		}
		if (!this.can_add_row(name)) {
			return;
		}

		var self = this;
		var unique_id = name.trim().toLowerCase();
		var $make_new_row = function()
		{
			value = value || "";
			var description = value;
			if (self.is_system_value(value)) {
				value = value.toUpperCase();
				var system_params = self.__get_system_params();
				description = system_params[value];
			}
			else if (self.is_context_value(value)) {
				let contextValue = self.get_context_value(value);
				let contextTypes = self.__get_context_types();
				description = contextValue.value + " (" + (contextTypes[contextValue.type] || contextValue.type) + ")";
			}
			var display = "";
			if (apply_effect) {
				display = "style='display:none;' ";
			}
			return $(
				"<li param_data='true' class='formListItem'" + display + ">" +
					"<label param_name='true' class='formItemText nameColumn' unique_id='" + unique_id + "'>" + name  + "</label>" +
					"<label param_value='true' class='formItemText valueColumn' value_id='" + value + "'>" + description + "</label>" +
					"<button action_edit='true' class='buttonColorGray gridActionButton edit' title='"+ "<%=_T('Edit')%>" + "'><i class='far fa-pencil'></i>" + "<%=_T('Edit')%>" + "</button>"+
					"<button action_del='true' class='buttonColorGray gridActionButton last' title='"+ "<%=_T('Delete')%>" + "'><i class='far fa-trash-alt'></i>" + "<%=_T('Delete')%>" + "</button>"+
				"</li>");
		};

		var added = false;
		var $list = $(this.param_list);
		var $new_row;
		$list.find("li[param_data]").each(function(_, row)
		{
			var $row = $(row);
			var current_unique_id = $row.find("label[param_name]").attr("unique_id");

			if (unique_id < current_unique_id) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	cci_web_parameters.prototype.hide_errors = function()
	{
		$(this.param_message).hide();
		$(this.param_section).find(".formFieldError").hide();
	};

	cci_web_parameters.prototype.validate = function(name, value, is_system_value, is_context_value)
	{
		var self = this;

		var show_error = function(message, $control)
		{
			$(self.param_message).text(message).show();
			if ($control) {
				var validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};
		var message;
		var $name = $(this.param_name);
		if (!this.is_valid_data(name, "required")) {
			message = "<%=_T('Please enter the parameter name.')%>";
			show_error(message, $name);
			return false;
		}

		if (this.is_system_name(name)) {
			message = FORMAT("<%=_T('Parameter name {0} is reserved.')%>", name);
			show_error(message, $name);
			return false;
		}
		if (!this.is_valid_data(name, "alpha")) {
			message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the parameter name.')%>";
			show_error(message, $name);
			return false;
		}
		//valido que no sean solo numeros
		if (this.is_valid_data(name, "number")) {
			message = "<%=_T('The name of the parameter could not be a number.')%>";
			show_error(message, $name);
			return false;
		}

		if (is_system_value) {
		// valido el value de tipo system. solo si es requerido ya que es un cbo
			var $system_value = $(this.param_system_value);
			if (!this.is_valid_data(value, "required")) {
				message = "<%=_T('Please select the system parameter value.')%>";
				show_error(message, $system_value);
				return false;
			}
		}
		else if (is_context_value) {
		// valido el value de tipo context. si es requerido y el tipo de texto ingresado, ya que es un txt
			var $context_value = $(this.param_custom_value);
			if (!this.is_valid_data(value, "required")) {
				message = "<%=_T('Please enter the custom parameter value.')%>";
				show_error(message, $context_value);
				return false;
			}
			if (!this.is_valid_data(value, "alpha")) {
				message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the custom parameter value.')%>";
				show_error(message, $context_value);
				return false;
			}
		}
		else {
		// valido el value de tipo custom. si es requerido y el tipo de texto ingresado, ya que es un txt
			var $custom_value = $(this.param_custom_value);
			if (!this.is_valid_data(value, "required")) {
				message = "<%=_T('Please enter the custom parameter value.')%>";
				show_error(message, $custom_value);
				return false;
			}
			if (!this.is_valid_data(value.replace(".", ""), "alpha")) {
				message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the custom parameter value.')%>";
				show_error(message, $custom_value);
				return false;
			}
		}

		var unique_id = name.trim().toLowerCase();
		var already_exist = function()
		{
			var current_param = $(self.param_list).find("li[param_data] label[param_name='true'][unique_id='" + unique_id + "']");
			var len = current_param.length;
			if (len > 0) {
				var message = FORMAT("<%=_T('Already exist parameter name {0}.')%>", name);
				show_error(message, $name);
				return true;
			}
			return false;
		};

		if (!this.$row_editing) {
			// si es un parametro nuevo valida que el mismo ya no exista en la lista
			if (already_exist()) {
				return false;
			}
		}
		else {
			var actual_name = this.$row_editing.find("[param_name='true']").text();
			actual_name = actual_name.trim().toLowerCase();
			if (actual_name !== unique_id) {
			// si es name del parametro ha cambiado valido que el nuevo nombre ya no exista en la lista
				if (already_exist()) {
					return false;
				}
			}
		}

		// validation successufully
		return true;
	};

	cci_web_parameters.prototype.add_or_update = function( has_continue )
	{
		var self = this;
		this.hide_errors();

		if ( !this.panel.section.has_permission_to_section() || !this.panel.section.has_permission_to_save() ) {
			var message = "<%=_T('Unable to update cci parameter: You dont have permissions for this action.')%>";
			this.panel.section.frm.show_status_message( message, "Error", 5000 );
			this.hide_popup();
			return;
		}

		var name = $(this.param_name).val();
		var value;
		var is_system_value = $(this.param_option_system).prop("checked");
		var is_context_value = $(this.param_option_context).prop("checked");
		if (is_system_value) {
			value = $(this.param_system_value).val();
		}
		else if (is_context_value) {
			let contextType = $(this.param_context_type).val();
			let contextValue = $(this.param_context_value).val();
			if (contextType && contextValue) {
				value = contextType + "." + contextValue;
			}
		}
		else {
			value = $(this.param_custom_value).val();
		}

		// validate parameters
		var result = this.validate( name, value, is_system_value, is_context_value );
		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			var apply_effect = true;
			this.add_row(name, value, apply_effect);
		}
		else {
		// update actual parameter
			this.$row_editing.fadeOut("slow", function()
			{
				var $param_name = $(self.$row_editing.find("[param_name='true']"));
				var old_name = $param_name.attr("unique_id");

				if (old_name !== name) {
					// si el grupo o el name cambio, tengo que cambiarlo de posicion dentro de la table (alfabeticamente)
					self.$row_editing.remove();
					var apply_effect = true;
					self.add_row(name, value, apply_effect);
				}
				else {
					var description = value;
					is_system_value = self.is_system_value(value);
					if (is_system_value) {
						value = value.toUpperCase();
						var system_params = self.__get_system_params();
						description = system_params[value];
					}
					else if (is_context_value) {
						let contextValue = self.get_context_value(value);
						let contextTypes = self.__get_context_types();
						description = contextValue.value + " (" + (contextTypes[contextValue.type] || contextValue.type) + ")";
					}
					var $param_value = $(self.$row_editing.find("[param_value='true']"));
					// actualizo el name
					$param_name.attr("unique_id", name);
					$param_name.text(name);
					// actualizo el value
					$param_value.attr("value_id", value);
					$param_value.text(description);

					// aca realizo el fade in
					self.$row_editing.fadeIn("slow");
				}
			});
		}

		$(this.param_custom_value).val("");
		$(this.param_name).val("");
		if (!has_continue) {
			return;
		}
		$(this.param_name).focus();
	};

	cci_web_parameters.prototype.disable_actions = function( disable )
	{
		disable = disable === undefined ? true : disable;

		if (this.panel.section.is_readonly()) {
			disable = true;
		}

		$(this.param_new).prop("disabled", disable);
		var $param_data = $(this.param_list).find("li[param_data]");
		// boton edit de la lista de parametros
		$param_data.find("button[action_edit]").prop("disabled", disable);
		// boton delete de la lista de parametros
		$param_data.find("button[action_del]").prop("disabled", disable);
		this.panel.section.disable_actions(disable);
	};

	cci_web_parameters.prototype.is_valid_data = function(value, validation_type)
	{
		var response = this.panel.section.frm.validation.valid( value, validation_type );
		return response.success;
	};

	cci_web_parameters.prototype.set_events = function()
	{
		var self = this;
		if ( this.param_new === undefined ) {
			return;
		}

		// boton new que abre el popup para poder agregar un nuevo parametro
		$(this.param_new)
			.off("click")
			.on("click", function() {
				self.$row_editing = null;
				self.show_popup( true /*is_new*/ );
			});

		$(this.param_create)
			.off("click")
			.on("click", function() {
				var has_continue = false;
				self.add_or_update(has_continue);
			});
		$(this.param_create_continue)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.add_or_update(has_continue);
			});
		$(this.param_cancel)
			.off("click")
			.on("click", function() {
				self.hide_popup();
			});

		$(this.param_options_value)
			.off("change")
			.on("change", function() {
				var value_type = $(this).val();
				self.switch_popup_value(value_type);
			});

		// grilla botones edit / delete
		var $param_list = $(this.param_list);
		var event_selector = "li[param_data]";
		// doble click sobre la row edita la misma
		$param_list
			.off("dblclick", event_selector)
			.on("dblclick", event_selector, function(event) {
				event.stopPropagation();
				self.$row_editing = $(this);
				self.show_popup( false /*is_new*/ );
			});
		// boton edit de la lista de parametros
		event_selector = "button[action_edit]";
		$param_list
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.$row_editing = $(this).parent();
				self.show_popup( false /*is_new*/ );
			});

		// boton delete de la lista de parametros
		event_selector = "button[action_del]";
		$param_list
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $parameter = $(this).parent();
				var message = FORMAT(
					"<%=_T('Do you want to delete parameter ({0}: {1}) ?')%>",
					$parameter.find("[param_name='true']").text(),
					$parameter.find("[param_value='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					jQuery.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.panel.section.has_permission_to_section() || !self.panel.section.has_permission_to_save() ) {
							var message = "<%=_T('Unable to delete cci parameter: You dont have permissions for this action.')%>";
							self.panel.section.frm.show_status_message( message, "Error", 5000 );
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});
	};

	//---------------------------------------------------------------------
	// form campaigns class - CC integration section (clase base)
	function adm_form_campaigns_section_cci_base( params )
	{
		const success = adm_form_campaigns_section_cci_base.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.prefix_id = "#" + params.prefix_id;
		this.view_action_id = "view_campaigns_cc_integration"; // view action id de permisos
		this.save_action_id = "save_campaign_cc_integration"; // save action id de permisos
		this.current_panel_id = "";
		this.panels = {};
		this.init_supported_ext_integrations();

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_cci_base, inConcert.conf_form_section );

	adm_form_campaigns_section_cci_base.prototype.init_supported_ext_integrations = function()
	{
		const self = this;
		this.m_supported_ext_integrations = {
			"HELPDESK": {
				entity_id: "HELPDESK",
				get_integration: function() {
					return self.get_helpdesk_integration();
				},
				set_integration: function(integracion) {
					return self.set_helpdesk_integration(integracion);
				},
				is_available_in_vcc: function() {
					return self.is_available_helpdesk_integration();
				},
				is_available_in_campaign: function() {
					if (!self.frm.campaign) {
						return false;
					}
					return self.frm.campaign.hasIntegrationWithHelpDesk();
				}
//
//Se deja comentada la integracion de notificaciones relacionada con HELPDESK por si en algun momento se necesita activar!!!
//
/*
				,
				dependency: {
					is_available: function() {
						return self.has_permission_to_campaign_notifications();
					}
				}
*/
			},
			"MAS": {
				entity_id: "MAS",
				get_integration: function() {
					return self.get_mas_integration();
				},
				set_integration: function(integracion) {
					return self.set_mas_integration(integracion);
				},
				is_available_in_vcc: function() {
					return self.is_available_mas_integration();
				},
				is_available_in_campaign: function() {
					if (!self.frm.campaign) {
						return false;
					}
					return self.frm.campaign.hasIntegrationWithMarketingAndSales();
				},
				dependency: {
					is_available: function() {
						return self.has_permission_to_campaign_notifications();
					}
				}
			}
		};
		this.m_widgets = inConcert.app.get_vcc_all_widgets_integrations();
		this.m_widgets.forEach(widget => {
			self.m_supported_ext_integrations[widget.Id] = {
				is_dynamic: true,
				entity_id: widget.Id,
				name: FORMAT("<%=_T('{0} Widget')%>", widget.DisplayName),
				get_integration: function() {
					return self.get_widget_integration(widget.Id);
				},
				is_available_in_vcc: function() {
					return self.is_available_widget_integration(widget.Id);
				},
				is_available_in_campaign: function() {
					if (!self.frm.campaign) {
						return false;
					}
					return self.frm.campaign.hasIntegrationWithOccWidget(widget.Id);
				},
				dependency: {
					is_available: function() {
						return self.has_permission_to_campaign_notifications();
					}
				}
			};
		});
	};

	adm_form_campaigns_section_cci_base.prototype.make_widgets_markup = function() {
		const $template = $(this.prefix_id + "_template_widgets_section");
		const html_template = $template.html();
		if (!html_template) {
			return;
		}
		let html_widgets = "";
		for (let external_integration_id in this.m_supported_ext_integrations) {
			if (this.m_supported_ext_integrations.hasOwnProperty(external_integration_id)) {
				let external_integration = this.m_supported_ext_integrations[external_integration_id];
				if (external_integration.is_dynamic) {
					let html = FORMAT(html_template, external_integration.entity_id, external_integration.name);
					html_widgets = html_widgets + html;
				}
			}
		}
		if (html_widgets.length > 0) {
			const $widget_section = $(this.prefix_id + "_widgets_section");
			$widget_section.html(html_widgets).show();
		}
	};

	/*
		Funcion que valida si usuario tiene permisos de edicion sobre la seccion notificaciones de la campana.
	*/
	adm_form_campaigns_section_cci_base.prototype.has_permission_to_campaign_notifications = function() {
		const campaign = this.frm.get_entity_instance();
		if (!campaign) {
			return false;
		}
		const has_permision_to_camp_view_notifications = inConcert.permissions.to_action("administration" , "view_campaigns_interaction_notifications", undefined, campaign.Id);
		if (!has_permision_to_camp_view_notifications) {
			return false;
		}
		const has_permision_to_camp_save_notifications = inConcert.permissions.to_action("administration" , "save_campaign_interaction_notifications", undefined, campaign.Id);
		if (!has_permision_to_camp_save_notifications) {
			return false;
		}
		return true;
	};

	adm_form_campaigns_section_cci_base.prototype.is_available_helpdesk_integration = function()
	{
		return inConcert.app.has_helpdesk_integration();
	};

	adm_form_campaigns_section_cci_base.prototype.get_helpdesk_integration = function()
	{
		if (!this.is_available_helpdesk_integration()) {
			return;
		}
		const self = this;
		
		const get_override_actions = function () {
			// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
			var $override_finish = $(self.prefix_id + "_helpdesk_base_panel_override_finish_action");
			if ($override_finish.prop("checked")) {
				return { finish: { show: "popup" } };
			}
			return undefined;
		};

		let integration = {
			Enable: $(this.prefix_id + "_helpdesk_enabled_integration").prop("checked"),
			Config: {
			// Esta estructura es necesaria en el integration bridge.
			// Como el iframe de helpdesk la comunicacion se realiza por medio del bridge, tiene que respetar esta estructura.
			//
				"application": "HelpDesk Application",
				"type": "helpdesk_integration",
				"trigger": {
				  "embed": true
				},
				"popup": {
				  "parameters": {
					// "VIRTUALCC": "{VIRTUALCC}",
					// "INTERACTIONID": "{INTERACTIONID}",
					// "AGENTID": "{AGENTID}",
					// "interaction_id": "{INTERACTIONID}"
				  }
				}
			},
		};

		if (integration.Enable) {
			integration.override_actions = get_override_actions();
		}
		return integration;
	};

	adm_form_campaigns_section_cci_base.prototype.set_helpdesk_integration = function(integration)
	{
		if (typeof integration !== "object") {
			return;
		}
		const self = this;
		const set_override_actions = function (actions) {
			// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
			var override = false;
			var finish_action = "finish";
			if (typeof actions === "object" &&
				typeof actions[finish_action] === "object" &&
				actions[finish_action].show === "popup") {
				override = true;
			}
			var $override_finish = $(self.prefix_id + "_helpdesk_base_panel_override_finish_action");
			$override_finish.prop("checked", override);
			self.frm.disable_control($override_finish, self.is_readonly());
		};

		set_override_actions(integration.override_actions);
	};

	adm_form_campaigns_section_cci_base.prototype.is_available_mas_integration = function()
	{
		return inConcert.app.has_mas_integration();
	};

	adm_form_campaigns_section_cci_base.prototype.get_mas_integration = function()
	{
		if (!this.is_available_mas_integration()) {
			return;
		}
		const self = this;
		const get_override_actions = function () {
			// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
			var $override_finish = $(self.prefix_id + "_mas_base_panel_override_finish_action");
			if ($override_finish.prop("checked")) {
				return { finish: { show: "popup" } };
			}
			return undefined;
		};

		let integration = {
			Enable: $(this.prefix_id + "_mas_enabled_integration").prop("checked"),
			Config: {}
		};

		if (integration.Enable) {
			integration.override_actions = get_override_actions();
		}

		return integration;
	};

	adm_form_campaigns_section_cci_base.prototype.set_mas_integration = function(integration)
	{
		if (typeof integration !== "object") {
			return;
		}
		const self = this;
		const set_override_actions = function (actions) {
			// TODO: cuando se realice el override de las demas acciones, esto deberia cambiar. Por ahora unicamente existe override de la accion finish.
			var override = false;
			var finish_action = "finish";
			if (typeof actions === "object" &&
				typeof actions[finish_action] === "object" &&
				actions[finish_action].show === "popup") {
				override = true;
			}
			var $override_finish = $(self.prefix_id + "_mas_base_panel_override_finish_action");
			$override_finish.prop("checked", override);
			self.frm.disable_control($override_finish, self.is_readonly());
		};

		set_override_actions(integration.override_actions);
	};

	adm_form_campaigns_section_cci_base.prototype.is_available_widget_integration = function(widget_id)
	{
		return inConcert.app.has_widget_integration(widget_id);
	};

	adm_form_campaigns_section_cci_base.prototype.$get_widget_section = function(widget_id)
	{
		widget_id = widget_id.toLowerCase();
		return $(this.prefix_id + FORMAT("_{0}_section", widget_id));
	};

	adm_form_campaigns_section_cci_base.prototype.$get_widget_enabled = function(widget_id)
	{
		widget_id = widget_id.toLowerCase();
		return $(this.prefix_id + FORMAT("_{0}_enabled_integration", widget_id));
	};

	adm_form_campaigns_section_cci_base.prototype.get_widget_integration = function(widget_id)
	{
		if (!this.is_available_widget_integration(widget_id)) {
			return;
		}
		const $chk = this.$get_widget_enabled(widget_id);
		let widget_integration = {
			Enable  : $chk.prop("checked"),
			Config  : {},
			IsWidget: true
		};
		return widget_integration;
	};

	adm_form_campaigns_section_cci_base.prototype.is_enabled_external_integration_in_vcc = function(external_integration_type)
	{
		if (!this.m_supported_ext_integrations[external_integration_type]) {
			return false;
		}
		const is_configured_in_vcc = this.m_supported_ext_integrations[external_integration_type].is_available_in_vcc();
		if (!is_configured_in_vcc) {
			return false;
		}
		// Bueno si tiene integracion
		return true;
	};

	adm_form_campaigns_section_cci_base.prototype.is_enabled_external_integration_in_campaign = function(external_integration_type)
	{
		if (!this.is_enabled_external_integration_in_vcc(external_integration_type)) {
			return false;
		}
		const is_configured_in_campaign = this.m_supported_ext_integrations[external_integration_type].is_available_in_campaign();
		if (!is_configured_in_campaign) {
			return false;
		}
		// Bueno si tiene integracion
		return true;
	};

	/*
		Se invoca a la funcion de dependencia para saber si esta disponible para el usuario.
	*/
	adm_form_campaigns_section_cci_base.prototype.is_available_dependency = function(external_integration_type)
	{
		const is_available_dependency = this.m_supported_ext_integrations[external_integration_type].dependency.is_available();
		if (!is_available_dependency) {
			return false;
		}
		// esta disponible la seccion notifications
		return true;
	};

	adm_form_campaigns_section_cci_base.prototype.load_ext_integrations = function(integration)
	{
		integration = integration || {};

		for (let integration_type in this.m_supported_ext_integrations) {
			if (this.m_supported_ext_integrations[integration_type]) {
				let supported_ext_integration = this.m_supported_ext_integrations[integration_type];
				let ext_integration = supported_ext_integration.get_integration();
				if (ext_integration) {
					integration.othersIntegrations = integration.othersIntegrations || {};
					integration.othersIntegrations[supported_ext_integration.entity_id] = ext_integration;
				}
			}
		}

		return integration;
	};

	adm_form_campaigns_section_cci_base.prototype.current_panel = function()
	{
		var self = this;
		if (!this.current_panel_id) {
			throw("Current panel is undefined");
		}
		var current_panel;
		$.each(this.panels, function(_, panel) {
			if (panel.frm.get_panel_id() === self.current_panel_id ) {
				current_panel = panel;
				return false; // break
			}
		});
		return current_panel.frm;
	};

	adm_form_campaigns_section_cci_base.prototype.switch_integration = function( panel_id )
	{
		this.frm.hide_messages();
		this.current_panel_id = panel_id || "UNSELECTED";
		$.each(this.panels, function( _, panel)
		{
			var current_panel_id = panel.frm.get_panel_id();
			if ( current_panel_id !== panel_id) {
				panel.frm.hide();
			}
			else {
				panel.frm.show();
			}
		});
	};

	adm_form_campaigns_section_cci_base.prototype.save_cci = function()
	{
		const self = this;
		const current_panel = this.current_panel();
		const response = current_panel.validate();

		if (!response.success) {
			return false;
		}

		this.frm.dashboard.show_waiting("<%=_T('Please wait until integration are saved.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function() {
			let integration = current_panel.get();
			integration = self.load_ext_integrations(integration);

			self.frm.campaign.saveIntegrationData(integration, function(error) 
			{
				let message, message_type, timeout_ms;
				if (!error) {
					message      = "<%=_T('Integration have been saved successfully.')%>";
					message_type = "Info";
					timeout_ms   = 3000; // default time
				}
				else {
					if (typeof error === "object") {
						error = ( error.result || "").toUpperCase();
					}
					switch (error)
					{
						case "INVALID_REQUEST":
							message = "<%=_T('Invalid request.')%>";
							break;
						case "CAMPAIGN_DOES_NOT_EXIST":
							message = "<%=_T('Unable to save campaign integration. Campaign does not exist..')%>";
							break;
						case "CCI_CONFIGURATION_TYPE_NOT_RECOGNISED":
							message = "<%=_T('Unable to save campaign integration. Integration type not recognised..')%>";
							break;
						case "CCI_CONFIGURATION_BAD_INFORMATION":
							message = "<%=_T('Unable to save campaign integration. Integration information is invalid..')%>";
							break;
						case "FORBIDDEN":
							message = "<%=_T('You dont have permissions for save campaign integration.')%>";
							break;
						case "ALLEGRO_INTEGRATION_ERROR":
							message = "<%=_T('Unable to save campaign integration. Allegro integration error.')%>";
							break;
						default:
							message = "<%=_T('Failed to save integration, please try again in a few minutes...')%>";
					}
					message_type = "Error";
					timeout_ms =   false; // infinite
				}
				self.saving = false;
				self.frm.show_status_message(message, message_type, timeout_ms);
			});
		}, 150);
	};

	adm_form_campaigns_section_cci_base.prototype.set_events = function()
	{
		const self = this;

		const $section_container = this.$get_section_container();
		if (!$section_container) {
			return;
		}

		const $cci_basic_info = $section_container.find("[basic_info]");
		const $cbo_cci_types = $(this.prefix_id + "_type");
		const $cancel = $(this.prefix_id + "_cancel");

		$(this.prefix_id + "_save")
			.off("click")
			.on("click", function() {
				self.save_cci();
			});

		$cancel
			.off("click")
			.on("click", function() {
				self.frm.close();
			});

		$(this.prefix_id + "_enabled_integration")
			.off("change")
			.on("change", function() {
				let panel_name;
				let checked = $(this).prop("checked");
				if (checked) {
					var disabled = self.is_readonly();
					self.frm.disable_control($cbo_cci_types, disabled);
					panel_name = $cbo_cci_types.val();
					$cci_basic_info.show();
				}
				else {
					$cbo_cci_types.val("");
					self.frm.disable_control($cbo_cci_types, false);
					$cci_basic_info.hide();
				}
				self.switch_integration( panel_name );
			});

		$cbo_cci_types
			.off("change")
			.on("change", function() {
				var elapsed_time_ms = 5;
				window.setTimeout(function()
				{
					var panel_name = $cbo_cci_types.val();
					self.switch_integration( panel_name );
				}, elapsed_time_ms);
			});

	/* M&S Integration Events -------------------------------------------------------*/
		const $mas_enabled_integration = $(this.prefix_id + "_mas_enabled_integration");
		$mas_enabled_integration.off("change")
			.on("change", function() {
				const checked = $(this).prop("checked");
				if (checked) {
					// Obtengo la seccion "Notifications" de la configuracion de la campana.
					//  para que actualice el check correspondiente a notificaciones.
					const notifications_frm_section = self.frm.get_frm_section("interaction_notifications");
					if (notifications_frm_section) {
						notifications_frm_section.external_mark_app("MAS", checked);
					}
					$(self.prefix_id + "_mas_base_panel").show();
				}
				else{
					$(self.prefix_id + "_mas_base_panel").hide();
					$(self.prefix_id + "_mas_base_panel_override_finish_action").prop("checked", false);
				}
			});
	// ------------------------------------------------------------------------

	/* HELPDESK Integration Events -------------------------------------------------------*/

		// Este evento NO realiza nada de momento. Pero lo dejo bindeado por si en algun momento se necesita realizar algo.
		const $helpdesk_enabled_integration = $(this.prefix_id + "_helpdesk_enabled_integration");
		$helpdesk_enabled_integration.off("change")
			.on("change", function() {
				const checked = $(this).prop("checked");
				 if (checked) {
					/*
					Se deja comentada la integracion de notificaciones relacionada con HELPDESK por si en algun momento se necesita activar!!!
					Obtengo la seccion "Notifications" de la configuracion de la campana.
				 		para que actualice el check correspondiente a notificaciones.
					const notifications_frm_section = self.frm.get_frm_section("interaction_notifications");
					if (notifications_frm_section) {
						notifications_frm_section.external_mark_app("HELPDESK", checked);
					}
					*/
					$(self.prefix_id + "_helpdesk_base_panel").show();
				 }else{
					$(self.prefix_id + "_helpdesk_base_panel").hide();
					$(self.prefix_id + "_helpdesk_base_panel_override_finish_action").prop("checked", false);
				 }
			});
	// ------------------------------------------------------------------------

	/* WIDGETS Dynamic Integration Events -------------------------------------------------------*/
		for (let external_integration_id in this.m_supported_ext_integrations) {
			if (this.m_supported_ext_integrations.hasOwnProperty(external_integration_id)) {
				let external_integration = this.m_supported_ext_integrations[external_integration_id];
				if (external_integration.is_dynamic) {
					let widget_id = external_integration_id;
					const $chk = this.$get_widget_enabled(widget_id);
					$chk.off("change")
						.on("change", function() {
					//		const checked = $(this).prop("checked");
					//		if (checked) {
								// // Obtengo la seccion "Notifications" de la configuracion de la campana.
								// //  para que actualice el check correspondiente a notificaciones.
								// const notifications_frm_section = self.frm.get_frm_section("interaction_notifications");
								// if (notifications_frm_section) {
								// 	notifications_frm_section.external_mark_app("MAS", checked);
								// }
						//	}
						});
				}
			}
		}
	};

	adm_form_campaigns_section_cci_base.prototype.fill_combo_cci_types = function()
	{
		var list = [];
		$.each(this.panels, function(_, panel) {
			var id = panel.frm.get_panel_id();
			var description = panel.frm.get_panel_name();
			if (id === "UNSELECTED") {
				id = ""; description = "";
			}
			list.push({ id: id, description: description });
		});

		var $combo = $(this.prefix_id + "_type");
		var is_blank_first_item = false;
		var attr_id = "id";
		var attr_name = "description";
		var sorted = true;

		this.frm.fill_combo($combo, is_blank_first_item, list, attr_id, attr_name, sorted);
	};

	adm_form_campaigns_section_cci_base.prototype.set_tweety_i6_projects = function(projects) {
		this.panels.tweety_i6.data.projects = projects;
	};

	adm_form_campaigns_section_cci_base.prototype.set_tweety_allegro_projects = function(projects) {
		this.panels.tweety_allegro.data.projects = projects;
	};

	adm_form_campaigns_section_cci_base.prototype.attach_load_request = function(request)
	{
		var self = this;
		if (!$.isArray(request)) {
			return;
		}
		request.push({
			request_function: function(success, response) {
				var tweety = inConcert.app.getTweetyAPI();
				return tweety.load_cci_projects(success, response);
			},
			success: function( projects ) {
				self.set_tweety_i6_projects( projects );
			},
			error: function( /*err*/ ) {
				var message = "<%=_T('Error retrieving inconcert web applications.')%>";
				var timeout_ms = false; //infinite
				self.frm.show_status_message(message, "Error", timeout_ms);
			}
		});
	};

	adm_form_campaigns_section_cci_base.prototype.load_integration = function()
	{
		if (!this.frm.campaign) {
			return;
		}
		this.frm.dashboard.show_waiting("<%=_T('Loading campaign integration.')%>");
		var integration = this.frm.campaign.getIntegrationData();
		var panel_name = "UNSELECTED";
		var enabled_integration = false;
		if (integration && integration.type) {
			panel_name = integration.type;
			enabled_integration = true;
		}
		$(this.prefix_id + "_type").val(panel_name);
		$(this.prefix_id + "_enabled_integration").prop("checked", enabled_integration).trigger("change");
		var current_panel = this.current_panel();
		current_panel.set(integration);

		// HELPDESK - section -------------
		const $helpdesk_section = $(this.prefix_id + "_helpdesk_section");
		const $helpdesk_enabled_integration = $(this.prefix_id + "_helpdesk_enabled_integration");
		const is_enabled_helpdesk_integration_in_vcc = this.is_enabled_external_integration_in_vcc("HELPDESK");

		if (is_enabled_helpdesk_integration_in_vcc) {
			const is_enabled_helpdesk_integration_in_campaign = this.is_enabled_external_integration_in_campaign("HELPDESK");
			$helpdesk_enabled_integration.prop("checked", is_enabled_helpdesk_integration_in_campaign).trigger("change");
			$helpdesk_section.show();
		}
		else {
			$helpdesk_enabled_integration.prop("checked", false).trigger("change");
			$helpdesk_section.hide();
		}

		// -----------------------------

		// M&S - section -------------
		const $mas_section = $(this.prefix_id + "_mas_section");
		const $mas_enabled_integration = $(this.prefix_id + "_mas_enabled_integration");
		const is_enabled_mas_integration_in_vcc = this.is_enabled_external_integration_in_vcc("MAS");

		if (is_enabled_mas_integration_in_vcc) {
			const is_enabled_mas_integration_in_campaign = this.is_enabled_external_integration_in_campaign("MAS");
			$mas_enabled_integration.prop("checked", is_enabled_mas_integration_in_campaign).trigger("change");
			$mas_section.show();
		}
		else {
			$mas_enabled_integration.prop("checked", false).trigger("change");
			$mas_section.hide();
		}
		// -----------------------------

		// WIDGETS - section -------------
		for (let external_integration_id in this.m_supported_ext_integrations) {
			if (this.m_supported_ext_integrations.hasOwnProperty(external_integration_id)) {
				let external_integration = this.m_supported_ext_integrations[external_integration_id];
				if (external_integration.is_dynamic) {
					let widget_id = external_integration_id.toLowerCase();
					const $widget_section = this.$get_widget_section(widget_id);
					const $widget_enabled_integration = this.$get_widget_enabled(widget_id);
					const is_enabled_widget_integration_in_vcc = this.is_enabled_external_integration_in_vcc(widget_id);

					if (is_enabled_widget_integration_in_vcc) {
						const is_enabled_widget_integration_in_campaign = this.is_enabled_external_integration_in_campaign(widget_id);
						$widget_enabled_integration.prop("checked", is_enabled_widget_integration_in_campaign).trigger("change");
						$widget_section.show();
					}
					else {
						$widget_enabled_integration.prop("checked", false).trigger("change");
						$widget_section.hide();
					}
				}

				if (typeof external_integration.set_integration === "function" && integration && typeof integration.othersIntegrations === "object") {
					external_integration.set_integration(integration.othersIntegrations[external_integration.entity_id]);
				}
			}
		}
		//-----------------------------

		this.frm.dashboard.hide_waiting();
	};

	adm_form_campaigns_section_cci_base.prototype.build_panels = function()
	{
		const self = this;
		$.each(this.panels, function(_, panel) {
			panel.frm = new panel.instance({
				section: self,
				popup_id: panel.popup_id,
				prefix_id: self.prefix_id,
				has_override: panel.has_override,
				additional_prefix_id: panel.additional_prefix_id,
				data: panel.data
			});
			if (panel.selected) {
				self.current_panel_id = panel.frm.get_panel_id();
			}
		});
	};

	adm_form_campaigns_section_cci_base.prototype.destroy_panels = function()
	{
		$.each(this.panels || {}, function(_, panel) {
			if (panel.frm) {
				panel.frm.dispose();
				panel.frm = null;
			}
		});
	};

	adm_form_campaigns_section_cci_base.prototype.dispose = function()
	{
		adm_form_campaigns_section_cci_base.uber.dispose.call( this );
		this.destroy_panels();
	};

	adm_form_campaigns_section_cci_base.prototype.initialize = function()
	{
		this.build_panels();
		this.make_widgets_markup();
		this.fill_combo_cci_types();
		this.load_integration();
	};

	adm_form_campaigns_section_cci_base.prototype.set = function( /*data*/ ) {
		this.initialize();
	};

	adm_form_campaigns_section_cci_base.prototype.editable = function()
	{
		adm_form_campaigns_section_cci_base.uber.editable.call( this );

		$( this.prefix_id + "_save" ).show();
		$( this.prefix_id + "_cancel" ).attr("value", "<%=_T('Cancel')%>");

		$.each( this.panels, function( _, panel ) {
			panel.frm.editable();
		});

		// HELPDESK - section -------------
//
//Se deja comentada la integracion de notificaciones relacionada con HELPDESK por si en algun momento se necesita activar!!!
//
		// const is_enabled_helpdesk_integration_in_vcc = this.is_enabled_external_integration_in_vcc("HELPDESK");
		// if (is_enabled_helpdesk_integration_in_vcc) {
		// 	const $helpdesk_enabled_integration = $(this.prefix_id + "_helpdesk_enabled_integration");
		// 	// Valido si el usuario tiene permisos de edicion sobre la seccion "notificaciones" de la campana.
		// 	//  De esto depende si el checkbox queda readonly o no
		// 	const is_enabled_notifications_in_campaign = this.is_available_dependency("HELPDESK");
		// 	this.frm.readonly_control($helpdesk_enabled_integration, !is_enabled_notifications_in_campaign /* disabled */ );
		// }

		// M&S - section -------------
		const is_enabled_mas_integration_in_vcc = this.is_enabled_external_integration_in_vcc("MAS");
		if (is_enabled_mas_integration_in_vcc) {
			const $mas_enabled_integration = $(this.prefix_id + "_mas_enabled_integration");
			// Valido si el usuario tiene permisos de edicion sobre la seccion "notificaciones" de la campana.
			//  De esto depende si el checkbox queda readonly o no
			const is_enabled_notifications_in_campaign = this.is_available_dependency("MAS");
			this.frm.readonly_control($mas_enabled_integration, !is_enabled_notifications_in_campaign /* disabled */);
		}

		// WIDGETS - section -------------
		for (let external_integration_id in this.m_supported_ext_integrations) {
			if (this.m_supported_ext_integrations.hasOwnProperty(external_integration_id)) {
				let external_integration = this.m_supported_ext_integrations[external_integration_id];
				if (external_integration.is_dynamic) {
					let widget_id = external_integration_id.toLowerCase();
					const is_enabled_widget_integration_in_vcc = this.is_enabled_external_integration_in_vcc(widget_id);
					if (is_enabled_widget_integration_in_vcc) {
						const $widget_enabled_integration = this.$get_widget_enabled(widget_id);
						// Valido si el usuario tiene permisos de edicion sobre la seccion "notificaciones" de la campana.
						//  De esto depende si el checkbox queda readonly o no
						const is_enabled_notifications_in_campaign = this.is_available_dependency(widget_id);
						this.frm.readonly_control($widget_enabled_integration, !is_enabled_notifications_in_campaign /* disabled */);
					}
				}
			}
		}
		//-----------------------------
	};

	adm_form_campaigns_section_cci_base.prototype.readonly = function()
	{
		adm_form_campaigns_section_cci_base.uber.readonly.call( this );

		$( this.prefix_id + "_save" ).hide();
		$( this.prefix_id + "_cancel" ).attr("value", "<%=_T('Close')%>");

		$.each( this.panels, function( _, panel ) {
			panel.frm.readonly();
		});
	};

	adm_form_campaigns_section_cci_base.prototype.on_show = function()
	{
		adm_form_campaigns_section_cci_base.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		$(this.prefix_id + "_buttons").show();

		var is_readonly = this.is_readonly();
		if ( is_readonly ) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	//---------------------------------------------------------------------
	// form campaigns class - CC integration section SIN ALLEGRO
	//
	function adm_form_campaigns_section_cci( params )
	{
		params = params || {};
		params.prefix_id = "adm_form_campaign_cci";

		var success = adm_form_campaigns_section_cci.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.panels = {
			__unselected:
			{
				selected     : true,
				instance     : cci_unselected_panel,
				frm          : null,
				data         : {}
			},
			tweety_i6:
			{
				instance     : cci_web_tweety_panel_i6,
				additional_prefix_id: "_tweety_i6",
				has_override : true,
				frm          : null,
				data         : {}
			},
			external:
			{
				instance     : cci_web_external_panel,
				additional_prefix_id: "_web_external",
				has_override : true,
				frm          : null,
				data         : {}
			}
		};
		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_cci, adm_form_campaigns_section_cci_base );

	if ( typeof inConcert.adm_form_campaigns_section_cci !== "function" ) {
		inConcert.adm_form_campaigns_section_cci = adm_form_campaigns_section_cci;
	}

	adm_form_campaigns_section_cci.prototype.initialize = function()
	{
		adm_form_campaigns_section_cci.uber.initialize.call( this );
		this.load_integration();
	};

	//---------------------------------------------------------------------
	// form campaigns class - CC integration section CON ALLEGRO
	//
	function adm_form_campaigns_section_cci_allegro( params )
	{
		params = params || {};
		params.prefix_id = "adm_form_campaign_cci_allegro";

		var success = adm_form_campaigns_section_cci_allegro.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.panels = {
			__unselected:
			{
				selected     : true,
				instance     : cci_unselected_panel,
				frm          : null,
				data         : {}
			},
			tweety_i6:
			{
				instance     : cci_web_tweety_panel_i6,
				additional_prefix_id: "_tweety_i6",
				has_override : true,
				frm          : null,
				data         : {}
			},
			tweety_allegro:
			{
				instance     : cci_web_tweety_panel_allegro,
				additional_prefix_id: "_tweety_allegro",
				has_override : true,
				frm          : null,
				data         : {}
			},
			external:
			{
				instance     : cci_web_external_panel_allegro,
				additional_prefix_id: "_web_external",
				has_override : true,
				frm          : null,
				data         : {}
			}
		};
		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_cci_allegro, adm_form_campaigns_section_cci_base );

	if ( typeof inConcert.adm_form_campaigns_section_cci_allegro !== "function" ) {
		inConcert.adm_form_campaigns_section_cci_allegro = adm_form_campaigns_section_cci_allegro;
	}

	adm_form_campaigns_section_cci_allegro.prototype.initialize = function()
	{
		adm_form_campaigns_section_cci_allegro.uber.initialize.call( this );
		this.load_integration();
	};

	adm_form_campaigns_section_cci_allegro.prototype.on_show = function()
	{
		adm_form_campaigns_section_cci_allegro.uber.on_show.call(this, {show: false});
	};

}());
