/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 9 */

/* global inConcert
*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//"use strict";
	
	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form campaigns class
	//
	function adm_form_campaigns_section_interaction_notifications (params)
	{
		const success = adm_form_campaigns_section_interaction_notifications.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.prefix_id = "#adm_form_campaign_interaction_notifications";
		this.view_action_id = "view_campaigns_interaction_notifications"; // view action id de permisos
		this.save_action_id = "save_campaign_interaction_notifications"; // save action id de permisos
		return true;
	}

	inConcert.Utils.inherit(adm_form_campaigns_section_interaction_notifications, inConcert.conf_form_section);

	if (typeof inConcert.adm_form_campaigns_section_interaction_notifications !== "function") {
		inConcert.adm_form_campaigns_section_interaction_notifications = adm_form_campaigns_section_interaction_notifications;
	}

	adm_form_campaigns_section_interaction_notifications.prototype.set_events = function()
	{
		const self = this;

		$(this.prefix_id + "_save")
			.off("click")
			.on("click", function() {
				self.__save();
			});

		$(this.prefix_id + "_cancel")
			.off("click")
			.on("click", function() {
				self.frm.close();
			});
	};

	adm_form_campaigns_section_interaction_notifications.prototype.attach_load_request = $.noop;

	adm_form_campaigns_section_interaction_notifications.prototype.initialize = function()
	{
		this.__load();
	};

	adm_form_campaigns_section_interaction_notifications.prototype.set = function(/*data*/) {
		this.initialize();
	};

	adm_form_campaigns_section_interaction_notifications.prototype.editable = function()
	{
		adm_form_campaigns_section_interaction_notifications.uber.editable.call( this );
		$( this.prefix_id + "_save" ).show();
		$( this.prefix_id + "_cancel" ).attr("value", "<%=_T('Cancel')%>");
	};

	adm_form_campaigns_section_interaction_notifications.prototype.readonly = function()
	{
		adm_form_campaigns_section_interaction_notifications.uber.readonly.call( this );
		$( this.prefix_id + "_save" ).hide();
		$( this.prefix_id + "_cancel" ).attr("value", "<%=_T('Close')%>");
	};

	adm_form_campaigns_section_interaction_notifications.prototype.on_show = function()
	{
		adm_form_campaigns_section_interaction_notifications.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		$(this.prefix_id + "_buttons").show();

		const is_readonly = this.is_readonly();
		if ( is_readonly ) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__validate = function()
	{
		this.hide_messages();
		let response = {
			success: true
		};
		return response;
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__get = function()
	{
		let info = {
			Apps: {}
		};
		$("#adm_form_campaign_interaction_notifications_panel .card-block")
			.find("[notification_app]:enabled:checked")
			.each(function() {
				info.Apps[this.id.replace("adm_form_campaign_interaction_notifications_app_", "")] = true;
			});
		return info;
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__save = function()
	{
		var self = this;
		var response = this.__validate();
		if (!response.success) {
			return false;
		}
		this.frm.dashboard.show_waiting("<%=_T('Please wait until Notifications are saved.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function() {
			var info = self.__get();
			self.frm.campaign.saveInteractionNotificationsInfo(info, function(error) 
			{
				var message, message_type, timeout_ms;
				if (!error) {
					message      = "<%=_T('Notifications have been saved successfully.')%>";
					message_type = "Info";
					timeout_ms   = 3000; // default time
				}
				else {
					if (typeof error === "object") {
						error = ( error.result || "").toUpperCase();
					}
					switch (error)
					{
						case "INVALID_REQUEST":
							message = "<%=_T('Invalid request.')%>";
							break;
						case "CAMPAIGN_DOES_NOT_EXIST":
							message = "<%=_T('Unable to save campaign Notifications. Campaign does not exist.')%>";
							break;
						case "FORBIDDEN":
							message = "<%=_T('You dont have permissions for save campaign Notifications.')%>";
							break;
						case "NOT_SUPPORTED_INTERACTION_NOTIFICATIONS_IN_VCC":
							message = "<%=_T('Unable to save campaign Notifications. The VCC does not support Notifications.')%>";
							break;
						default:
							message = "<%=_T('Failed to save Notifications, please try again in a few minutes...')%>";
					}
					message_type = "Error";
					timeout_ms =   false; // infinite
				}
				self.saving = false;
				self.frm.show_status_message(message, message_type, timeout_ms);
			});
		}, 50);
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__load = function()
	{
		const self = this;
		self.__load_data(function() {
			let info = self.frm.campaign && self.frm.campaign.getInteractionNotificationsInfo() || {};
			$.each(info.Apps || {}, function(app) {
				$("#adm_form_campaign_interaction_notifications_app_" + app).prop("checked", true);
			});
		});
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__load_data = function(callback) {
		const self = this;
		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/apps_notifications/list/",
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(data) {
				self.__load_apps(data.apps || {});
				callback();
			},
			error: function(/*_, error*/) {
				self.frm.show_status_message("<%=_T('Error loading apps notifications')%>", "Error", 3000);
				callback();
			}
		});
	};

	adm_form_campaigns_section_interaction_notifications.prototype.__load_apps = function(apps) {
		let apps_a = inConcert.Utils.hash_to_sorted_array(apps).filter(function(app) {
			app.CampaignEvents = inConcert.Utils.boolMapToArray(app.Events).filter(function(event) {
				let eventData = inConcert.available_apps_notifications_d[event];
				return eventData && eventData.group.campaignScope || false;
			});
			return app.CampaignEvents.length > 0;
		});
		$("#adm_form_campaign_interaction_notifications_panel .card-block").children("[notification_app_container]").remove();
		$("#adm_form_campaign_interaction_notifications_panel .card-block").append(apps_a.reduce(function($apps, app)
		{
			let events_msg = app.CampaignEvents.reduce(function(text, event, index) {
						return text + (index > 0? ", " : "") + event;
					}, "<%=_T('Events')%><%=_T(':')%> ");

			let notification_app = app.Id;
			let $app = $(format(
				"".concat(
					"<div class='row check-with-bottom-separator' notification_app_container title='{4}'>" +
						"<div class='col-12'>" +
						 	"<div class='form-group'>" +
								"<div class='form-check form-switch'>" +
									"<label for='{0}' class='form-check-label form-check-label-title'>" +
										"<input id='{0}' notification_app='{5}' class='form-check-input' type='checkbox'>" +
										"<span></span>" +
										"{1} <small class='form-check-label-disclaimer explanation ml-1'>{2} (<%=_T('Url')%><%=_T(':')%> {3})</small>" +
									"</label>" +
								"</div>" +
							"</div>" +
						"</div>" +
					"</div>" 
					
				),
				"adm_form_campaign_interaction_notifications_app_" + app.Id,
				app.Id,
				app.Description,
				app.Url,
				events_msg,
				notification_app
			));
			if (app.AllCampaigns === "1") {
				$app.find(".form-group")
					.addClass("customEditable");
				$app.find("input")
					.prop("checked", true)
					.prop("disabled", true)
					.addClass("readonly");
				$app.find("label")
					.addClass("formLabel-ds readonly");
			}
			$apps.push($app);
			return $apps;
		}, []));
	};

	/*
		Esta funcion es invocada de una seccion de campana externa a la de notificaciones.
		Especificamente desde la seccion CCI integration, cuando activan el checkbox de "MAS" o "HELPDESK"
	*/
	adm_form_campaigns_section_interaction_notifications.prototype.external_mark_app = function(app_id, checked)
	{
		const supported_apps = {
		//
		//Se deja comentada la integracion de notificaciones relacionada con HELPDESK por si en algun momento se necesita activar!!!
			// "HELPDESK": {
			// 	"Id": "HelpDesk"
			// },
			"MAS": {
				"Id": "MarketingAndSales"
			}
		};

		const notification_app = supported_apps[app_id];
		if (!notification_app) {
			return;
		}

		const $notification_app = $("#adm_form_campaign_interaction_notifications_panel .card-block").find("[notification_app='" + notification_app.Id + "']");
		if ($notification_app.length > 0) {
			$notification_app.prop("checked", checked);
		}
	};

}());
