/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, quotmark:false, -W014, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var format = inConcert.Utils.format;

	var get_chk_bool_to_string  = function( chk_id ) {
		var $chk = $( chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

	/*---------------------------------------------------------------------
	CLASS LABELS FRM.
	---------------
		Esta clase es la encargada de realizar el Alta/Modificacion client/side desde el frm de labels.
		Aqui se encuentra el nombre, la descripcion.
	*/

	function labels_frm( params )
	{
		this.parent = params.parent;
	// dom ids
		// frm
		this.id_frm_container   = "adm_form_campaign_labels_edition";
		// inputs
		this.id_label_name      = "adm_form_campaign_labels_edition_name";
		this.id_label_is_system = "adm_form_campaign_labels_edition_is_system";
		// buttons actions
		this.id_frm_actions     = "adm_form_campaign_labels_edition_actions";
		this.id_create          = "adm_form_campaign_labels_edition_create";
		this.id_cancel          = "adm_form_campaign_labels_edition_close";
		this.id_color           = "adm_form_campaign_labels_edition_color";
		this.initialize();
	}

	labels_frm.prototype.initialize = function() { this.set_events();
	};

	labels_frm.prototype.__initial_color_picker_selector = function()
	{
		const self = this;
		$("#" + this.id_color).colorPick({
			'initialColor': this.selectColor,
			'paletteLabel' : "<%=_T('Color')%>:",
			'allowRecent': false,
			'allowCustomColor': false,
			'palette': ["#E6F0F5", "#E6FAF0", "#F5E6F5", "#FFF5EB", "#EBF5FA", "#FFF0F0", "#FFE6F5", "#F5F5FF", "#FFF5FA", "#EBF0F0"],
			'onColorSelected': function() {
				this.element.css({'backgroundColor': this.color, 'color': this.color});
				self.selectColor = this.color;
			}
		});
	};

	labels_frm.prototype.get = function()
	{
		var id, name;
		name = $("#" + this.id_label_name).val();
		if ( this.parent.$row_editing ) {
			id = this.parent.$row_editing.attr("label_id");
		}
		else {
			id = name;
		}
		id = id.toUpperCase();
		return {
			id     : id,
			name   : name,
			color  : this.selectColor,
			system : get_chk_bool_to_string( "#" + this.id_label_is_system )
		};
	};

	labels_frm.prototype.set = function( label )
	{
		label = label || {};
		$("#" + this.id_label_name).val( label.name || "" );
		$("#" + this.id_label_is_system).prop( "checked", label.system === "1" );
		this.selectColor = label.color || "#FFF5EB";
		this.__initial_color_picker_selector();
	};

	labels_frm.prototype.show = function( current_label, callback )
	{
		callback = callback || $.noop;
		this.set( current_label );
		var $create = $("#" + this.id_create);

		if ( !current_label ) {
		// si presione el boton para crear uno nuevo
			$create.val("<%=_T('Create')%>");
		}
		else {
		// si presione el boton para editar
			$create.val("<%=_T('Update')%>");
		}
		$("#" + this.id_frm_container).show();
		$("#" + this.id_frm_actions).show();

		callback();
		this.on_show();
	};

	labels_frm.prototype.on_show = function()
	{
		this.parent.frm.add_css_class("pageFormForVisibleButtons");
		var $name = $("#" + this.id_label_name);
		if ( this.parent.$row_editing ) {
			this.parent.readonly_control( $name, true, false );
			//TODO: dar el foco al resto de los controles, para el siguiente target version
		}
		else {
			this.parent.readonly_control( $name, false, true );
			$name.focus();
		}
	};

	labels_frm.prototype.hide = function()
	{
		this.parent.hide_label_frm_edition();
	};

	labels_frm.prototype.on_hide = function()
	{
		$("#" + this.id_frm_container).hide();
		$("#" + this.id_frm_actions).hide();
		this.parent.frm.remove_css_class("pageFormForVisibleButtons");
	};

	labels_frm.prototype.validate = function()
	{
		var self = this;

		this.parent.frm.hide_messages();

		var show_error = function(message_error, $control)
		{
			self.parent.frm.show_status_message( message_error, "Error", false );
			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			if ($control) {
				$control.focus();
			}
		};

		var $name = $("#" + this.id_label_name );
		var label_info = this.get();
		var new_name = label_info.name.trim().toUpperCase();

		var validation_result = inConcert.Utils.validate( new_name, "required" );
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			show_error("<%=_T('Name is required.')%>", $name);
			return false;
		}

		validation_result = inConcert.Utils.validate( new_name, "id" );
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			show_error("<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>", $name);
			return false;
		}

		validation_result = inConcert.Utils.validate(new_name, "html_malicious_code");
		if ( !validation_result.success ) {
			// valida codigo malicioso
			show_error("<%=_T('Possible malicious code in the field value.')%>", $name);
			return false;
		}

		// valida que el label no se una palabra reservada.
		var is_reserved_word = { "PENDING": true, "RESULT": true }.hasOwnProperty(new_name);
		if (is_reserved_word) {
			show_error("<%=_T('This label is a reserved word.')%>", $name);
			return false;
		}

		var is_new = this.parent.$row_editing ? false : true;
		var $rows = this.parent.$get_rows();
		var already = function()
		{
			var edit_label_id;
			if ( !is_new ) {
				edit_label_id = self.parent.$row_editing.attr("label_id");
			}
			var already_exist = false;
			$rows.each(function( _, row )
			{
				var $row = $(row);
				if ( edit_label_id !== undefined ) {
				// si estoy editando, no valido sobre el mismo row.
					var row_label_id = $row.attr("label_id");
					if (row_label_id === edit_label_id) {
						return true; // continue
					}
				}
				var current_name = $row.find( self.parent.selector_row_name ).text();
				already_exist = current_name.trim().toUpperCase() === new_name;
				if (already_exist) {
					show_error("<%=_T('Name already exists.')%>", $name);
					return false; // break
				}
			});
			return already_exist;
		};

		// valida que si estoy agregando un nuevo label, el mismo ya no exista.
		if (already()) {
			return false;
		}

		$("#" + this.id_frm_container).find(".formFieldValidation").hide();
		$name.val("");

		// validation successfully
		return label_info;
	};

	labels_frm.prototype.set_events = function()
	{
		var self = this;

		$("#" + this.id_create)
			.off("click")
			.on("click", function() {
				self.parent.add_or_update_label();
			});
		$("#" + this.id_cancel)
			.off("click")
			.on("click", function() {
				self.hide();
				self.parent.hide_messages();
			});
		
	};
	//
	//-----------------------------------------


	//---------------------------------------------------------------------
	// 
	/*---------------------------------------------------------------------
	CLASS LABELS SECTION 
	------------------------
		Esta clase hereda de la clase "conf_form_section" utilizado en las diferentes secciones de la campana.

		Es la encargada de mostrar la lista de labels, y de realizar el ABM de estos.
	*/
	function adm_form_campaigns_section_labels(params)
	{
		var success = adm_form_campaigns_section_labels.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		// permisos
		this.view_action_id           = "view_campaigns_labels";   // view action id de permisos
		this.save_action_id           = "save_campaign_labels";   // save action id de permisos
		this.erase_action_id          = "delete_campaign_labels"; // delete action id de permisos
		this.has_perm_to_save         = null;
		this.has_perm_to_erase        = null;

		// dom ids
		// 
		this.id_list_container        = "adm_form_campaign_labels_list_container";
		this.labels                   = "adm_form_campaign_labels_list";
		this.id_new_label             = "adm_form_campaign_labels_new";
		this.labels_not_result_found  = "adm_form_campaign_labels_not_results_found";

		// selectors - row
		this.selector_row_name        = "label[col_name]";
		this.selector_row_sys         = "label[col_sys]";
		this.selector_row_color         = "div[col_color]";

		this.$row_editing = null; // Aca se encuentra el row html de jquery que se esta editando
		this.all_labels = {}; // todos los labels de la campana (utilizado para la busqueda general)
		this.list_to_save = {}; // los added/updated labels que se agregan en el popup y se deben guardar al cerrar el mismo. TODO: Cambiar !! feo muy feo, pero de momento no encontre otra forma para hacerlo rapido y sin romper todo lo que esta armado.

		this.labels_frm = null; // instancia de la clase que maneja la actualizacion.

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_labels, inConcert.conf_form_section );

	if (typeof inConcert.adm_form_campaigns_section_labels !== "function") {
		inConcert.adm_form_campaigns_section_labels = adm_form_campaigns_section_labels;
	}

	adm_form_campaigns_section_labels.prototype.dispose = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
	};

	// Alta en el hash de todos los labels
	adm_form_campaigns_section_labels.prototype.add_label_to_full_list = function(id, label)
	{
		this.all_labels[id.toUpperCase()] = label;
	};

	// Actualiza en el hash de todos los labels
	adm_form_campaigns_section_labels.prototype.update_label_on_full_list = function(id, label)
	{
		if (!this.all_labels.hasOwnProperty(id.toUpperCase())) {
			return;
		}
		this.all_labels[id.toUpperCase()] = label;
	};

	// Baja en el hash todos los labels
	adm_form_campaigns_section_labels.prototype.erase_label_on_full_list = function(id)
	{
		if (!this.all_labels.hasOwnProperty(id.toUpperCase())) {
			return;
		}
		delete this.all_labels[id.toUpperCase()];
	};

	adm_form_campaigns_section_labels.prototype.get_label_from_full_list = function(id)
	{
		if (!id) {
			return {};
		}
		return this.all_labels[id.toUpperCase()];
	};

	// Alta en el hash label que se han modificado en el popup y se enviaran a guardar al server.
	adm_form_campaigns_section_labels.prototype.update_list_to_save = function( label, is_new)
	{
		this.list_to_save = this.list_to_save || {};
		this.list_to_save[label.id.toUpperCase()] = { is_new: is_new || false, label: label };
	};

	// Vacia el hash que mantiene la lista de labels a guardar en el server.
	adm_form_campaigns_section_labels.prototype.empty_list_to_save = function()
	{
		this.list_to_save = {};
	};

	// Valida si el hash de labels a guardar se encuentra vacio.
	adm_form_campaigns_section_labels.prototype.is_empty_list_to_save = function()
	{
		return $.isEmptyObject(this.list_to_save);
	};

	adm_form_campaigns_section_labels.prototype.$get_row = function(id)
	{
		if ( typeof id !== "string" ) {
			id = "";
		}
		return $("#" + this.labels).find("li[row_data][label_id='" + id + "']");
	};

	adm_form_campaigns_section_labels.prototype.$get_rows = function()
	{
		return $("#" + this.labels).find("li[row_data]");
	};

	// agrega un row ordenado alfabeticamente
	adm_form_campaigns_section_labels.prototype.add_sorted_row = function(new_name, $new_row)
	{
		var self = this;
		var $grid = $("#" + this.labels);
		var new_sorted_key = this.make_sorted_key(new_name);
		var added = false;
		$grid.find("li[row_data]").each(function( _, row )
		{
			var $row = $(row);
			var id = $row.attr("label_id");
			var label_data = self.get_label_from_full_list(id);
			var current_sorted_key = self.make_sorted_key(label_data.name);
			if (new_sorted_key < current_sorted_key) {
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$grid.append($new_row);
		}
	};

	adm_form_campaigns_section_labels.prototype.has_permission_to_delete = function()
	{
		if ( this.has_perm_to_erase !== null ) {
			return this.has_perm_to_erase;
		}
		this.has_perm_to_erase = adm_form_campaigns_section_labels.uber.has_permission_to_delete.call( this );
		return this.has_perm_to_erase;
	};

	adm_form_campaigns_section_labels.prototype.add_row = function(label, is_new)
	{
		var self = this;
		if (!$.isPlainObject(label)) {
			return;
		}
		var id = label.id;
		var name = label.name || id;
		const color = label.color || "#FFF5EB";
		var class_is_system = label.system === "1" ? "checkboxImageChecked" : "checkboxImageUnchecked";
		var $row = this.$get_row(id);

		var already_label = $row.length > 0;
		if ( already_label ) {
		// si el label a agregar ya existe en la lista de la GUI me voy
			return;
		}
		var get_actions = function()
		{
			var has_perm_to_save  = self.has_permission_to_save();
			var has_perm_to_erase = self.has_permission_to_delete();
			if ( !has_perm_to_save && ! has_perm_to_erase ) {
				return "";
			}
			var erase_html = "";
			if ( has_perm_to_erase ) { 
				erase_html = "<button action_del='true' class='buttonColorGray gridActionButton delete last' type='button' title='"+ "<%=_T('Delete')%>" + "'><i class='buttons-icons-delete'></i>" + "<%=_T('Delete')%>" + "</button>";
			}
			var save_html = "";
			if ( has_perm_to_save ) {
				save_html = "<button action_edit='true' class='buttonColorGray gridActionButton edit' type='button' title='"+ "<%=_T('Edit')%>" + "'><i class='buttons-icons-edit'></i>" + "<%=_T('Edit')%>" + "</button>";
			}
			return "".concat(save_html , erase_html);
		};
		var $row_html = $("".concat(
			"<li row_data='true' label_id='", id, "' class='viewListItem'>",
				"<label col_name='true' class='viewListItemText'>", name, "</label>",
				"<div class='viewListItemText checkboxImageCol'><div col_color='true' class='labelColorContainer' style='background-color:"+ color +";'></div></div>",
				"<label col_sys='true' class='viewListItemText checkboxImageCol'><span boolean='true' class='" + class_is_system + "' ></span></label>",
				get_actions(),
			"</li>")
		);

		if (!is_new) {
			$("#" + this.labels).append($row_html);
		}
		else {
			this.add_sorted_row(name, $row_html); // agregar ordenada
		}
		// add label in full hash
		this.add_label_to_full_list(id, label);
		// update message found
		this.show_or_hide_not_results_found();
	};

	adm_form_campaigns_section_labels.prototype.update_row = function( new_row_data )
	{
		if ( !$.isPlainObject( new_row_data ) ) {
			return;
		}
		var $current_row = this.$get_row( new_row_data.id );
		// name
		var $name = $( $current_row.find( this.selector_row_name ) );
		var old_name = $name.text().toLowerCase();
		var new_name = ( new_row_data.name || "" ).toLowerCase();

		// actualiza la lista completa de labels
		this.update_label_on_full_list( new_row_data.id, new_row_data );
		$name.text( new_row_data.name );
		// celda system
		var $system = $( $current_row.find( this.selector_row_sys ) ).find("span[boolean]");
		const $color = $( $current_row.find( this.selector_row_color ) );
		$color.css('background-color', new_row_data.color || "#FFF5EB");
		var class_is_system = new_row_data.system === "1" ? "checkboxImageChecked" : "checkboxImageUnchecked";
		$system.removeClass().addClass(class_is_system);

		if ( old_name !== new_name ) {
			// si el Nombre cambio, tengo que cambiarlo de posicion dentro de la lista (alfabeticamente)
			var $row = $current_row;
			this.$row_editing.remove();
			this.add_sorted_row( new_name, $row );
		}
		this.show_or_hide_not_results_found();
	};

	adm_form_campaigns_section_labels.prototype.make_sorted_key = function(name)
	{
		return ( name || "" ).toLowerCase();
	};

	adm_form_campaigns_section_labels.prototype.show_not_results_found = function( message )
	{
		$("#" + this.labels_not_result_found)
			.text( message || "<%=_T('No labels found.')%>" )
			.show();

		$("#" + this.labels).hide();
	};

	adm_form_campaigns_section_labels.prototype.hide_not_results_found = function()
	{
		$("#" + this.labels).show();
		$("#" + this.labels_not_result_found).hide();
	};

	adm_form_campaigns_section_labels.prototype.show_or_hide_not_results_found = function()
	{
		var count = Object.keys( this.all_labels || {} ).length;
		if ( count > 0 ) {
			this.hide_not_results_found();
		}
		else {
			this.show_not_results_found();
		}
	};

	adm_form_campaigns_section_labels.prototype.set_labels = function( labels )
	{
		var self = this;

		$("#" + this.labels).find("li[row_data]").remove();
		this.all_labels = {};

		if (!$.isPlainObject(labels)) {
			this.show_or_hide_not_results_found();
			return;
		}

		var sorted_labels = inConcert.Utils.hash_to_sorted_array(labels,
			function(label) {
				return self.make_sorted_key(label.name);
			}) || {};

		if ( sorted_labels.length > 0 ) {
			$.each(sorted_labels, function( _, item )
			{
				item.id = item.Id;
				delete item.Id; // la funcion hash to sorted array, mete el id, lo pelo porque aca no lo necesito.
				self.add_row(item);
			});
		}
		this.show_or_hide_not_results_found();
		this.frm.dashboard.hide_waiting();
	};

	adm_form_campaigns_section_labels.prototype.after_save_labels = function(success, message_error)
	{
		var self = this;
		var label_list = this.list_to_save;

		if (success) {
			if (this.$row_editing) {
			// update actual
				$.each(label_list, function( _, label_info )
				{
					self.update_row( label_info.label );
				});
			}
			else {
			// add all new
				var is_new = true;
				$.each(label_list, function( _, label_info )
				{
					if ( label_info.is_new ) {
						self.add_row( label_info.label, is_new );
					}
					else {
						self.update_row( label_info.label );
					}
				});
			}
			this.empty_list_to_save();
			this.$row_editing = null;
			var message = "<%=_T('Labels have been saved successfully.')%>";
			this.frm.show_status_message(
				message ,                /* mensaje          */
				"Info"  ,                /* tipo de mensaje  */
				3000                     /* timeout ms       */);
		}
		else {
			this.frm.show_status_message(
				message_error,           /* mensaje                                            */
				"Error",                 /* tipo de mensaje                                    */
				false,                   /* timeout ms (false = inifito)                       */
				false,                   /* scroll to top (false = no se mueve)                */
				{                        /* muestra la accion personalizada (en este caso retry) */
					message: "<%=_T('Retry')%>",
					callback: function(){
						self.save_labels();
					}
				}
			);
		}
	};

	adm_form_campaigns_section_labels.prototype.save_labels = function()
	{
		var self = this;
		this.hide_label_frm_edition();

		var label_list = this.list_to_save;
		if ( $.isEmptyObject( label_list ) ) {
			return;
		}
		this.saving = true;

		this.frm.dashboard.show_waiting("<%=_T('Please wait until labels are saved.')%>");
		// Lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//   parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function()
		{
			var labels = [];
			$.each(label_list, function(id, label_info)
			{
				var label = label_info.label;
				label.id = label.id || id;
				labels.push(label);
			});

			self.frm.campaign.saveLabels(labels, function(error)
			{
				var success = !error;
				var message_error;
				if ( !success ) {
					var err_type;
					if (typeof error === "object") {
						err_type = error.result;
					}
					else {
						err_type = error;
					}
					switch ((err_type || "").toUpperCase())
					{
						case "CAMPAIGN_DOES_NOT_EXIST":
							message_error = "<%=_T('Unable to save campaign labels, campaign does not exist...')%>";
							break;

						case "FORBIDDEN":
							message_error = "<%=_T('You dont have permissions for save campaign labels.')%>";
							break;

						case "INVALID_REQUEST":
							message_error = "<%=_T('Invalid request.')%>";
							break;

						case "RESERVED_WORD":
							message_error = format("<%=_T('Unable to save campaign labels, {0} label is a reserved word.')%>", error.data.label_id);
							break;

						case "OVERFLOW":
							message_error = format("<%=_T('Unable to save campaign labels, label has exceeded the size of {0} characters.')%>", error.data.maxLength);
							break;

						default:
							message_error = "<%=_T('Failed to save labels, please try again in a few minutes...')%>";
					}
				}
				self.after_save_labels( success, message_error );
				self.saving = false;
			});
		}, 50);
	};

	adm_form_campaigns_section_labels.prototype.remove_labels = function(labels, callback)
	{
		var self = this;
		callback = callback || $.noop;

		if ( !$.isArray( labels ) || labels.length === 0 ) {
			callback(false);
			return;
		}

		this.frm.dashboard.show_waiting("<%=_T('Please wait until labels are deleted.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function()
		{
			self.frm.campaign.deleteLabels(labels, function(error)
			{
				var success = !error;
				var message, type, timeout_ms;

				if ( success ) {
					message = "<%=_T('Labels have been deleted successfully.')%>";
					type = "Info";
					timeout_ms = 3000;
				}
				else {
					if (typeof error !== "object") {
						error = {result: error};
					}
					switch ((error.result || "").toUpperCase())
					{
						case "INVALID_REQUEST":
							message = "<%=_T('Invalid request.')%>";
							break;
						case "CAMPAIGN_DOES_NOT_EXIST":
							message = "<%=_T('Unable to delete labels. Campaign does not exist...')%>";
							break;
						case "FORBIDDEN":
							message = "<%=_T('You dont have permissions for delete labels.')%>";
							break;
						case "NO_CONFIGURED_LABELS":
							message = "<%=_T('Unable to delete labels. No configured labels in campaign.')%>";
							break;
						case "NO_LABELS_TO_DELETE":
							message = "<%=_T('Unable to delete labels. No exist assigned labels.')%>";
							break;
						default:
							message = "<%=_T('Failed to delete labels, please try again in a few minutes...')%>";
					}
					type = "Error";
					timeout_ms = false; // infinito
				}

				self.frm.show_status_message(
					message,        /* mensaje          */
					type,           /* tipo de mensaje  */
					timeout_ms      /* timeout ms       */
				);
				callback( success );
			});
		}, 50);
	};

	adm_form_campaigns_section_labels.prototype.build_header_custom_buttons = function()
	{
		var html_buttons =
			"<button id='" + this.id_new_label + "' class='pageButtonGeneric buttonColorPrimary pageButtonNew' ><i class='fal fa-plus'></i><%=_T('Create label')%></button>";

		adm_form_campaigns_section_labels.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	adm_form_campaigns_section_labels.prototype.add_or_update_label = function()
	{
		var label_info = this.labels_frm.validate();
		if ( !label_info ) {
			return;
		}
		var is_new = !this.$row_editing;
		this.update_list_to_save( label_info, is_new );
		this.save_labels();
	};

	adm_form_campaigns_section_labels.prototype.before_jump_to_menu = function(){
		if (this.labels_frm) {
			this.labels_frm.hide();
			this.hide_messages();
		}
	};

	adm_form_campaigns_section_labels.prototype.show_label_frm_edition = function(info = {})
	{
		var self = this;
		this.frm.hide_messages();

		var label_data;
		if ( this.$row_editing ) {
		// si presione el boton para editar
			var id = this.$row_editing.attr("label_id");
			label_data = this.get_label_from_full_list(id);
		}

		if ( !this.has_permission_to_section() || !this.has_permission_to_save() ) {
			var message;
			if ( !label_data ) {
				message = "<%=_T('Unable to view label: You dont have permissions for this action.')%>";
			}
			else {
				message = format( "<%=_T('Unable to view {0} label: You dont have permissions for this action.')%>", label_data.name );
			}
			this.$row_editing = null;
			this.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		this.empty_list_to_save();
		// show frm
		this.labels_frm.show( label_data, function() {
			$("#" + self.id_list_container).hide();
			$("#" + self.id_new_label).hide();
		});

		this.set_edit_mode(true);
		if (label_data) {
			this.frm.change_title_section(this.section_id, label_data?.name);
		}
		// google analytics
		this.AnalyticsSend(info);
	};

	adm_form_campaigns_section_labels.prototype.hide_label_frm_edition = function(canSendAnalytics = true)
	{
		this.labels_frm.on_hide();
		$("#" + this.id_list_container).show();
		$("#" + this.id_new_label).show();
		if ( this.is_empty_list_to_save() ) {
			this.$row_editing = null;
		}
		this.set_edit_mode(false);
		// google analytics
		if (canSendAnalytics)
			this.AnalyticsSend();
	};

	adm_form_campaigns_section_labels.prototype.set_events_frm_abm = function()
	{
		var self = this;
		// new message event
		var event_selector = "#" + this.id_new_label;
		var $header_buttons = this.frm.dashboard.$get_dom_element_by_id( "form_custom_buttons" ); //conf_form_custom_buttons
		$header_buttons
			.off("click", event_selector)
			.on("click", event_selector, function() {
				const analyticsInfo = {
					source_path: [
						{name: "label_action", value: "new"} // "/main/administration/campaigns/edit/camp1@vcc1/labels/new"
					]
				};
				self.show_label_frm_edition(analyticsInfo);
			});
	};

	adm_form_campaigns_section_labels.prototype.set_events = function()
	{
		var self = this;

		var $grid = $("#" + this.labels);
		var rows_event_selector = "li[row_data]";

		// boton delete de la lista de parametros
		var event_selector = rows_event_selector + " button[action_del]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.frm.hide_messages();
				self.empty_list_to_save();
				self.$row_editing = null;
				var $row = $(this).parent();
				var id = $row.attr("label_id");
				var label = self.get_label_from_full_list(id);
				var message = format(
					"<%=_T('Do you want to delete label {0} ?')%>",
					label.name);

				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() !== "yes") {
							return;
						}
						var labels_list = [];
						labels_list.push(label.id);
						// envia a remover al server
						self.remove_labels(
							labels_list,
							function(success) {
								if (!success) {
									return;
								}
								// elimino del hash de labels
								self.erase_label_on_full_list(label.id.toUpperCase());
								$row.fadeOut("slow", function() {
									$row.remove();
									self.show_or_hide_not_results_found();
								});
							});
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});

		/*
		*/
		const show_label_frm_edition = () => {
			const label_id = this.$row_editing.attr("label_id");
			const analyticsInfo = {
				source_path: [
					{name: "label_action", value: "edit"      }, // "/main/administration/campaigns/edit/camp1@vcc1/labels/edit"
					{name: "label_id"    , value: label_id    }  // "/main/administration/campaigns/edit/camp1@vcc1/labels/edit/label1"
				]
			};
			this.show_label_frm_edition(analyticsInfo);
		};

		// doble click sobre la row edita la misma
		$grid
			.off("dblclick", rows_event_selector)
			.on("dblclick", rows_event_selector, function(event) {
				event.stopPropagation();
				self.$row_editing = $(this);
				show_label_frm_edition();
			});

		// boton edit de la lista de parametros
		event_selector = rows_event_selector + " button[action_edit]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.$row_editing = $(this).parent();
				show_label_frm_edition();
			});

		// eventos del abm del frm
		this.set_events_frm_abm();
	};

	adm_form_campaigns_section_labels.prototype.load_campaign_labels = function()
	{
		var self = this;
		if ( this.frm.campaign.isNew === true ) {
			// si estoy creando una nueva campana, asigno los labels vacios
			this.set_labels();
			return;
		}
		// si estoy modificando una campana, voy a buscar la lista de labels al server
		this.frm.campaign.loadLabels(function(labels, error)
		{
			if (error) {
				var message_type = "Error";
				var timeout_ms = false; // infinite
				self.frm.show_status_message("<%=_T('Error retrieving campaign labels.')%>", message_type, timeout_ms);
				return;
			}
			self.set_labels(labels);
		});
	};

	adm_form_campaigns_section_labels.prototype.initialize = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
		this.labels_frm = new labels_frm( {
			parent: this
		});
		this.load_campaign_labels();
	};

	adm_form_campaigns_section_labels.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_campaigns_section_labels.prototype.on_show = function()
	{
		adm_form_campaigns_section_labels.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_campaigns_section_labels.prototype.on_hide = function()
	{
		adm_form_campaigns_section_labels.uber.on_hide.call( this );
		this.hide_label_frm_edition(false /*canSendAnalytics*/);
	};

	adm_form_campaigns_section_labels.prototype.editable = function()
	{
		adm_form_campaigns_section_labels.uber.editable.call( this );
	};

	adm_form_campaigns_section_labels.prototype.readonly = function() {
		adm_form_campaigns_section_labels.uber.readonly.call( this );
	};

}());
