/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
    strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W014, esversion: 11 */

/* global inConcert, inConcertLanguages, _ */

if (!this.inConcert) {
    this.inConcert = {};
}

(function () {

//  "use strict";

	const format = inConcert.Utils.format;
	const _T = inConcertLanguages.getLanguageLabel;

	const DISCLAIMERS = {
		type: {
			validation: "<%=_T('Rules to check number is valid. It will be splitted into country, area and number.')%>",
			schema: "<%=_T('Rules to determine how a phone number is composed based on country, area and number.')%>",
			timezone: "<%=_T('Rules to determine the timezone based on country, area and number.')%>",
		},
		schema_type: {
			Global: "<%=_T('International number format to be called from outside.')%>",
			SameCountry: "<%=_T('National number format to be called around the entire country.')%>",
			Local: "<%=_T('Local number format to be called from the same state/region.')%>",
		},
		validation_rule: {
			valid: "<%=_T('Splits the source number into country, area and number. Then validates it based on the length.')%>",
			invalid: "<%=_T('Matched number is automatically invalid.')%>",
			forward: "<%=_T('Jumps into another country/area rule.')%>",
		}
	};

    //---------------------------------------------------------------------
    // form phone_validator class
    function adm_form_phone_validator( params )
    {
        let success = adm_form_phone_validator.uber.constructor.call(this, params);
        if ( !success ) {
            return;
        }
        this.view_title     = "<%=_T('Phone Number Rules')%>";
        this.update_title   = "<%=_T('Phone Number Rules')%>";

        this.html_file_name = "/inconcert/apps/administration/phone_validator/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/phone_validator/save/";
        this.get_uri        = "/inconcert/apps/administration/phone_validator/get/";

		this.id_country_selector = "adm_form_ph_validator_country_selector";
		this.id_validate_button = "adm_form_phone_validator_country_validate_button";
		this.id_body_container = "adm_form_phone_validator_updating_container";

		this.id_global_type = "adm_form_phone_validator_global_type";
		this.id_global_schema_type = "adm_form_phone_validator_global_schema_type";
		this.id_global_actions = "adm_form_phone_validator_global_actions";
		this.id_global_add_rule_button = "adm_form_phone_validator_global_add_rule_button";
		this.id_global_restore_button = "adm_form_phone_validator_global_restore_button";
		this.id_global_table = "adm_form_phone_validator_global_table";

		this.id_area_search = "adm_form_phone_validator_area_search_input";
		this.id_area_type = "adm_form_phone_validator_area_type";
		this.id_area_schema_type = "adm_form_phone_validator_area_schema_type";
		this.id_area_actions = "adm_form_phone_validator_area_actions";
		this.id_area_add_area_button = "adm_form_phone_validator_area_add_area_button";
		this.id_area_restore_button = "adm_form_phone_validator_area_restore_button";
		this.id_area_list = "adm_form_phone_validator_areas_list";

		this.id_area_popup = "adm_form_phone_validator_area_popup";
		this.id_validation_popup = "adm_form_phone_validator_validation_popup";
		this.id_schema_popup = "adm_form_phone_validator_schema_popup";
		this.id_timezone_popup = "adm_form_phone_validator_timezone_popup";
		this.id_test_popup = "adm_form_phone_validator_test_popup";

		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : "adm_form_phone_validator_updating_tabs",
			must_intialize: false,
			tabs_list   : {
				edit: {
					id              : "global",
					name            : "<%=_T('Global')%>",
					tab_container_id: "adm_form_phone_validator_global_tab_container",
					selected        : true
				},
				publish: {
					id              : "area",
					name            : "<%=_T('Areas')%>",
					tab_container_id: "adm_form_phone_validator_area_tab_container"
				}
			}
		});
        return true;
    }

    inConcert.Utils.inherit( adm_form_phone_validator, inConcert.conf_form );

    if ( typeof inConcert.adm_form_phone_validator !== "function" ) {
        inConcert.adm_form_phone_validator = adm_form_phone_validator;
    }

    adm_form_phone_validator.prototype.initialize = function()
	{
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();   	  
	};

	adm_form_phone_validator.prototype.uri_get_entity = function()
	{
		return inConcert.app.getBaseDomain() + this.get_uri;
	};

	adm_form_phone_validator.prototype.build_header_custom_buttons = function()
	{
		const buttons = `
			<button
				id="${this.id_validate_button}"
				class="pageButtonGeneric buttonColorPrimary"
			>
				<%=_T('Validate Number')%>
			</button>
		`;
		adm_form_phone_validator.uber.build_header_custom_buttons.call(this, buttons);
	};

	adm_form_phone_validator.prototype.set_events = function()
	{
		adm_form_phone_validator.uber.set_events.call(this);

		$(`#${this.id_validate_button}`).off("click").on("click", () => this.show_test_rule_popup());

		this.set_global_events();
		this.set_area_events();
		this.fill_timezone_combo();
		[
			"#adm_form_phone_validator_test_popup_validation_table",
			"#adm_form_phone_validator_test_popup_schema_table",
			"#adm_form_phone_validator_test_popup_timezone_table",

		].forEach((id) => {
			const $container = $(id);
			const $header = $container.find("[accordion_header]");
			const $body = $container.find("[accordion_body]");
			$header.off("click").on("click", () => {
				$body.toggle();
				$container.toggleClass("active");
			});
		});
	};

	adm_form_phone_validator.prototype.set_global_events = function()
	{
		$(`#${this.id_global_add_rule_button}`).off("click").on("click", () => this.handle_add_global_rule());
		$(`#${this.id_global_restore_button}`).off("click").on("click", () => this.handle_global_restore());

		$(`#${this.id_global_type}`).off("change").on("change", () => this.handle_global_type_selected());
		$(`#${this.id_global_schema_type}`).off("change").on("change", () =>
			this.handle_global_schema_type_selected());
	};

	adm_form_phone_validator.prototype.set_area_events = function()
	{
		$(`#${this.id_area_add_area_button}`).off("click").on("click", () => this.handle_add_area());
		$(`#${this.id_area_restore_button}`).off("click").on("click", () => this.handle_restore_areas());

		$(`#${this.id_area_type}`).off("change").on("change", () => this.handle_area_type_selected());
		$(`#${this.id_area_schema_type}`).off("change").on("change", () =>
			this.handle_area_schema_type_selected());

		$(`#${this.id_area_search}`).off("keyup").on("keyup", _.debounce(
			() => this.handle_area_search(), 300));
	};

	adm_form_phone_validator.prototype.fill_timezone_combo = function()
	{
		const $timezone = $(`#${this.id_timezone_popup}_timezone`);
		const $areaTimezone = $(`#${this.id_area_popup}_timezone`);
		const timezones = Object.keys(inConcert.app.m_timezones).map((timezone) => ({
			id: timezone,
			label: `${timezone} - ${this.get_timezone_offset(timezone)}`
		}));
		this.fill_combo($timezone, false, timezones, "id", "label");
		this.fill_combo($areaTimezone, false, timezones, "id", "label");
	};

	adm_form_phone_validator.prototype.ready_for_save = function()
	{
		this.readonly();
	};

	adm_form_phone_validator.prototype.readonly_all_crtls = function()
	{
		
	};

    adm_form_phone_validator.prototype.set = function(success)
    {		
        if (!success) {
            return;
        }

		this.can_save = this.has_permission_to_save();
		this.toggle_edit();
		this.set_countries();
		this.set_types_disclaimers();
	};

	adm_form_phone_validator.prototype.set_disclaimer = function(options, id)
	{
		$(`#${id}_disclaimer`).attr("title", options[$(`#${id}`).val()] || "");
	};

	adm_form_phone_validator.prototype.set_types_disclaimers = function()
	{
		const set = (section) => {
			this.set_disclaimer(DISCLAIMERS.type, this[`id_${section}_type`]);
			this.set_disclaimer(DISCLAIMERS.schema_type, this[`id_${section}_schema_type`]);
		};
		set("global");
		set("area");
	};

	adm_form_phone_validator.prototype.toggle_edit = function()
	{
		$(`#${this.id_global_actions}`).toggle(this.can_save);
	};

	adm_form_phone_validator.prototype.country_description = function(countryId)
	{
		return countryId === "Global"
			? "<%=_T('Global')%>"
			: `${_T(this.entity[countryId].description)} - ${countryId}`;
	};

	adm_form_phone_validator.prototype.make_countries_list = function()
	{
		const countries = Object.keys(this.entity)
			.filter(countryId => countryId !== "Global")
			.map(countryId => ({
				value: countryId,
				label: this.country_description(countryId)
			}))
			.sort((country1, country2) => country1.label.localeCompare(country2.label));
		return [
			{ value: "Global", label: "<%=_T('Global')%>" },
			...countries
		];
	};

    adm_form_phone_validator.prototype.set_countries = function()
    {		
		const $countries = $(`#${this.id_country_selector}`);
		this.make_autocomplete(
			$countries,
			this.make_countries_list(),
			() => this.handle_country_selected(),
			"SELECT",
			null,
			true
		);
		this.selected_country_id = "Global";
	};

	adm_form_phone_validator.prototype.selected_country = function()
	{
		return this.entity[this.selected_country_id];
	};

    adm_form_phone_validator.prototype.handle_country_selected = function()
    {
		const $countries = $(`#${this.id_country_selector}`);
		const newCountryId = $countries.attr("current_value");
		if (newCountryId && this.entity[newCountryId])
			this.selected_country_id = newCountryId;
		if (!this.selected_country_id)
			return;

		$(`#${this.id_body_container}`).show();
		$(`#${this.id_area_search}`).val("");
		this.set_global();
		this.open_areas = new Set();
		this.set_areas();
	};

	adm_form_phone_validator.prototype.handle_global_type_selected = function()
	{
		$(`#${this.id_global_schema_type}_container`).toggle($(`#${this.id_global_type}`).val() === "schema");
		this.set_global();
		this.set_types_disclaimers();
	};

	adm_form_phone_validator.prototype.handle_global_schema_type_selected = function()
	{
		this.set_global();
		this.set_types_disclaimers();
	};

    adm_form_phone_validator.prototype.set_global = function()
    {
		const type = $(`#${this.id_global_type}`).val();
		const schemaType = $(`#${this.id_global_schema_type}`).val();
		const country = this.selected_country();
		this.populate_table({
			type,
			$table: $(`#${this.id_global_table}`),
			$headers: this[`make_${type}_table_headers`](),
			$body: this[`make_${type}_table_body`](country, schemaType)
		});

		$(`#${this.id_global_restore_button}_container`).parent().toggle(
			!!this.has_customizations(country)
		);
	};

	adm_form_phone_validator.prototype.has_customizations = function(data)
	{
		const checkPrefixes = (prefixes) =>
			Object.keys(prefixes || {}).some((prefix) => prefixes[prefix].status !== "original");

		return checkPrefixes(data.prefixes) ||
			checkPrefixes(data.timezones) ||
			(!!data.dialingSchemas &&
				Object.keys(data.dialingSchemas).some((schema) => checkPrefixes(data.dialingSchemas[schema])));
	};

	adm_form_phone_validator.prototype.handle_area_type_selected = function()
	{
		$(`#${this.id_area_schema_type}_container`).toggle($(`#${this.id_area_type}`).val() === "schema");
		this.reload_areas();
		this.set_types_disclaimers();
	};

	adm_form_phone_validator.prototype.handle_area_schema_type_selected = function()
	{
		this.reload_areas();
		this.set_types_disclaimers();
	};

	adm_form_phone_validator.prototype.handle_area_search = function()
	{
		this.set_areas();
	};

    adm_form_phone_validator.prototype.set_areas = function()
    {
		const searchFilter = ($(`#${this.id_area_search}`).val() || "").toLowerCase();
		const areas = this.selected_country().areas;
		const sortedAreas = Object.keys(areas)
			.sort()
			.map((areaCode) => ({ ...areas[areaCode], areaCode }));
		const $areas = sortedAreas
			.filter((area) => this.filter_area(area, searchFilter))
			.map((area) => this.make_area(area));
		
		$(`#${this.id_area_list}`).html("").append($areas);
		$(`#${this.id_area_list}_not_found`).toggle($areas.length === 0);
		$(`#${this.id_area_restore_button}_container`).parent().toggle(
			sortedAreas.some((area) => area.status !== "original")
		);
	};

	adm_form_phone_validator.prototype.reload_areas = function()
	{
		this.open_areas.forEach((areaCode) => this.reload_area(areaCode));
	};

	adm_form_phone_validator.prototype.reload_area = function(areaCode)
	{
		const area = { ...this.selected_country().areas[areaCode], areaCode };
		if (!area) {
			this.open_areas.delete(areaCode);
			return;
		}

		this.show_area(area, this.$area(area));
	};

	adm_form_phone_validator.prototype.filter_area = function({ areaCode, description }, searchFilter)
	{
		return areaCode.indexOf(searchFilter) >= 0 || description.toLowerCase().indexOf(searchFilter) >= 0;
	};

	adm_form_phone_validator.prototype.make_area = function(area)
	{
		const $area = $(`
			<div area_code="${area.areaCode || "empty"}" class="phoneValidatorAccordionElement">
				<div area_header class="phoneValidatorAccordionHeader">
					<div area_title class="phoneValidatorAccordionHeaderTitle">
						${area.areaCode || "<%=_T('Empty')%>"} - ${area.description}
					</div>
					<div area_buttons class="phoneValidatorAccordionHeaderButtons">
					</div>
				</div>
				<div area_table class="phoneValidatorTableContainer" style="display: none;">
					<table cellpadding="0" cellspacing="0" border="0" class="viewGrid">
						<thead class="viewGridHeader">
						</thead>
						<tbody class="viewGridBody">
						</tbody>
					</table>
					<div
						empty_table
						class="inlineMessagesContainer messageInfo m-2 w-auto"
						style="display:none"
					>
						<%=_T('No results found.')%>
					</div>
				</div>
			</div>
		`);
		$area.find("[area_header]").off("click").on("click", (e) => {
			if ($(e.target).is("[dropdown]"))
				return;

			this.handle_toggle_area(area);
		});
		if (this.open_areas.has(area.areaCode)) {
			this.show_area(area, $area);
		}
		return $area;
	};
	

	adm_form_phone_validator.prototype.$area = function(area)
	{
		return $(`#${this.id_area_list}`).find(`[area_code=${area.areaCode || "empty"}]`);
	};

	adm_form_phone_validator.prototype.handle_toggle_area = function(area)
	{
		const $area = this.$area(area);
		if (!this.open_areas.has(area.areaCode)) {
			this.show_area(area, $area);
		} else {
			this.hide_area(area, $area);
		}
	};

	adm_form_phone_validator.prototype.show_area = function(area, $area)
	{
		const type = $(`#${this.id_area_type}`).val();
		const schemaType = $(`#${this.id_area_schema_type}`).val();
		this.populate_table({
			type,
			$table: $area.find("[area_table]"),
			$headers: this[`make_${type}_table_headers`](),
			$body: this[`make_${type}_table_body`](area, schemaType)
		});
		$area.addClass("active");
		$area.find("[area_table]").show();
		$area.find("[area_buttons]")
			.html("")
			.append(this.make_area_actions(area));
		this.open_areas.add(area.areaCode);
	};

	adm_form_phone_validator.prototype.hide_area = function(area, $area)
	{
		$area.removeClass("active");
		$area.find("[area_table]").hide();
		$area.find("[thead], [tbody], [area_buttons]").html("");
		this.open_areas.delete(area.areaCode);
	};

	adm_form_phone_validator.prototype.make_area_actions = function(area)
	{
		if (!this.can_save) return null;

		const $actions = $(`
			<button
				area_add_rule
				class="pageButtonGeneric buttonColorWhite pageButtonNew"
				type="button"
			>
				<i class="far fa-plus"></i>
				<%=_T('Add Rule')%>
			</button>
			<div dropdown class="dropdown">
				<button
					class="buttonToggle buttonToggleOutlined wh-32 ml-1"
					type="button" id="first-dropdown"
					data-toggle="dropdown"
					aria-haspopup="true"
					aria-expanded="false"
					dropdown
				>
					<i dropdown class="far fa-ellipsis-h"></i>
				</button>
					<div
						class="dropdown-menu dropdown-menu-right"
						aria-labelledby="first-dropdown"
						id="adm_form_phone_validator_global_restore_button_container"
					>
						<a
							class="dropdown-item"
							edit_area
							onclick=""
						>
							<%=_T('Edit')%>
						</a>
						${area.status !== "original"
							? (`
							<a
								class="dropdown-item"
								delete_area
								onclick=""
							>
								${area.status === "added" ? "<%=_T('Delete')%>" : "<%=_T('Restore')%>"}
							</a>
							`) : ""}
					</div>
			</div>
		`);
		$actions.filter("[area_add_rule]").click((e) => {
			e.stopPropagation();
			this.handle_add_area_rule(area.areaCode);
		});
		$actions.find("[edit_area]").click((e) => {
			e.stopPropagation();
			this.handle_edit_area(area);
		});
		$actions.find("[delete_area]").click((e) => {
			e.stopPropagation();
			this.handle_delete_area(area);
		});
		return $actions;
	};

	adm_form_phone_validator.prototype.populate_table = function({ type, $table, $headers, $body })
	{
		$table.attr("class", `phoneValidatorTableContainer ${type}PhoneValidatorTableContainer`);
		$table.find("thead").html("").append($headers);
		$table.find("tbody").html("").append($body);
		$table.find("table").toggle($body.length > 0);
		$table.find("[empty_table]").toggle($body.length === 0);
	};

	adm_form_phone_validator.prototype.make_row_actions = function(prefix, type, areaCode)
	{
		if (!this.can_save) return null;

		const $actions = $(`
			<button
				edit_button
				class="buttonColorGray gridActionButton edit"
				title="<%=_T('Edit')%>"
			>
				<i class="buttons-icons-edit"></i>
			</button>
			${ prefix.status !== "original"
				? (`
					<button
						delete_button
						class="buttonColorGray gridActionButton delete"
						title="${prefix.status === "added" ? "<%=_T('Delete')%>" : "<%=_T('Restore')%>"}"
					>
						<i class="buttons-icons-delete"></i>
					</button>
				`) : ("")
			}
		`);		
		$actions.filter("[edit_button]").click(() => this[`handle_edit_${type}_rule`](prefix, areaCode));
		$actions.filter("[delete_button]").click(() => this.handle_delete_rule(type, prefix, areaCode));
		return $actions;
	};

	adm_form_phone_validator.prototype.make_prefixes = function(prefixes)
	{
		const numericPrefixes = Object.keys(prefixes).filter(prefix => !isNaN(prefix)).sort();
		const nonNumericPrefixes = Object.keys(prefixes).filter(prefix => isNaN(prefix)).sort();
		const sortedPrefixes = [...numericPrefixes, ...nonNumericPrefixes];
		return sortedPrefixes.map((prefix) => ({
			...prefixes[prefix],
			prefix,
		}));
	};

	adm_form_phone_validator.prototype.make_edited_cell = function({status})
	{
		if (status === "original") return "<td></td>";

		return `
			<td class='editedRuleColumn'>
				<i
					overrided_prefix
					class='fa fa-check-circle'
					title='${ status === "added" ?  "<%=_T('Added')%>" : "<%=_T('Overrided')%>"}'
				></i>
			</td>
		`;
	};

	adm_form_phone_validator.prototype.make_validation_table_headers = function()
	{
		return $(`
			<th>
				<%=_T('Prefix')%>
				<span
					class="far fa-question-circle"
					title="<%=_T('Numeric prefix (blank means any). The Closest prefix will be applied.')%>"
				></span>
			</th>
			<th><%=_T('Rule')%></th>
			<th class='editedRuleColumn'><%=_T('Edited')%></th>
			<th><%=_T('Comments')%></th>
			<th></th>
		`);
	};

	adm_form_phone_validator.prototype.make_validation_table_body = function(data)
	{
		const prefixes = this.make_prefixes(data.prefixes);
		return prefixes.map(prefix => this.make_validation_table_row(prefix, data.areaCode));
	};

	adm_form_phone_validator.prototype.make_validation_table_row = function(prefix, areaCode)
	{
		const $row = $(`
			<tr prefix='${prefix.prefix}'>
				<td>${prefix.prefix}</td>
				<td>
					${prefix.data}
					<span
						class="fa fa-question-circle"
						title="${this.make_validation_rule_disclaimer(prefix.data)}"
					></span>
				</td>
				${this.make_edited_cell(prefix)}
				<td>${prefix.description || ""}</td>
				<td action_cell></td>
			</tr>
		`);
		$row.find("[action_cell]").append(this.make_row_actions(prefix, "validation", areaCode));
		return $row;
	};

	adm_form_phone_validator.prototype.make_schema_table_headers = function()
	{
		return $(`
			<th>
				<%=_T('Prefix')%>
				<span
					class="far fa-question-circle"
					title="<%=_T('Numeric prefix (blank means any). The Closest prefix will be applied.')%>"
				></span>
			</th>
			<th><%=_T('Rule')%></th>
			<th><%=_T('Edited')%></th>
			<th><%=_T('Comments')%></th>
			<th></th>
		`);
	};

	adm_form_phone_validator.prototype.make_schema_table_body = function(data, schemaType)
	{
		const prefixes = this.make_prefixes(data.dialingSchemas[schemaType] || {});
		return prefixes.map(prefix => this.make_schema_table_row(prefix, data.areaCode));
	};

	adm_form_phone_validator.prototype.make_schema_table_row = function(prefix, areaCode)
	{
		const $row = $(`
			<tr prefix='${prefix.prefix || "empty"}'>
				<td>${prefix.prefix || "<%=_T('Any')%>"}</td>
				<td>${prefix.data}</td>
				${this.make_edited_cell(prefix)}
				<td>${prefix.description || ""}</td>
				<td action_cell></td>
			</tr>
		`);
		$row.find("[action_cell]").append(this.make_row_actions(prefix, "schema", areaCode));
		return $row;
	};

	adm_form_phone_validator.prototype.make_timezone_table_headers = function()
	{
		return $(`
			<th>
				<%=_T('Prefix')%>
				<span
					class="far fa-question-circle"
					title="<%=_T('Numeric prefix (blank means any). The Closest prefix will be applied.')%>"
				></span>
			</th>
			<th><%=_T('Offset')%></th>
			<th><%=_T('Code')%></th>
			<th><%=_T('Edited')%></th>
			<th><%=_T('Comments')%></th>
			<th></th>
		`);
	};

	adm_form_phone_validator.prototype.make_timezone_table_body = function(data)
	{
		const prefixes = this.make_prefixes(data.timezones);
		return prefixes.map(prefix => this.make_timezone_table_row(prefix, data.areaCode));
	};

	adm_form_phone_validator.prototype.get_timezone_offset = function(timezoneId)
	{
		const timezone = inConcert.app.m_timezones[timezoneId] || {};
		const offset = timezone.TimeZone || "0";
		const minutes = timezone.TimeZoneMinutes || "0";
		return `UTC${offset > 0 ? "+" : ""}${offset}${minutes !== "0" ? `:${minutes}` : ""}`;
	};

	adm_form_phone_validator.prototype.make_timezone_table_row = function(prefix, areaCode)
	{
		const $row = $(`
			<tr prefix='${prefix.prefix}'>
				<td>${prefix.prefix}</td>
				<td>${this.get_timezone_offset(prefix.data)}</td>
				<td>${prefix.data}</td>
				${this.make_edited_cell(prefix)}
				<td>${prefix.description || ""}</td>
				<td action_cell></td>
			</tr>
		`);
		$row.find("[action_cell]").append(this.make_row_actions(prefix, "timezone", areaCode));
		return $row;
	};

	adm_form_phone_validator.prototype.handle_add_area = function()
	{
		this.show_area_popup();
	};

	adm_form_phone_validator.prototype.handle_edit_area = function(area)
	{
		this.show_area_popup(area);
	};

	adm_form_phone_validator.prototype.hide_popup_errors = function($popup)
	{
		this.hide_messages(null, null, $popup.find("[popup_error]"));
	};

	adm_form_phone_validator.prototype.show_popup_error = function($popup, message, type = "Error")
	{
		this.show_status_message(message, type, undefined, undefined, undefined, $popup.find("[popup_error]"));
	};

	adm_form_phone_validator.prototype.popup_waiting = function($popup, waiting)
	{
		$popup.find("[popup_waiting]").toggle(waiting);
	};

	adm_form_phone_validator.prototype.popup_title = function($popup, title)
	{
		$popup.find("[popup_title]").text(title);
	};

	adm_form_phone_validator.prototype.popup_actions = function($popup, onSave)
	{
		$popup.find("[popup_close], [cancel_button]")
			.off("click")
			.on("click", () => this.popup_close($popup));
		$popup.find("[confirm_button]").off("click").on("click", onSave);
	};

	adm_form_phone_validator.prototype.popup_close = function($popup)
	{
		this.hide_popup_errors($popup);
		$popup.hide();
	};

	adm_form_phone_validator.prototype.popup_show = function($popup)
	{
		$popup.show();
	};

	adm_form_phone_validator.prototype.popup_validate = function($popup, $field, type, message, options)
	{
		return this.validate_field(
			$field,
			type,
			message,
			options,
			null,
			null,
			$popup.find("[popup_error]")
		);
	};

	adm_form_phone_validator.prototype.show_area_popup = function(area)
	{
		const $popup = $(`#${this.id_area_popup}`);
		const $country = $(`#${this.id_area_popup}_country`);
		const $code = $(`#${this.id_area_popup}_code`);
		const $name = $(`#${this.id_area_popup}_name`);
		const $timezone = $(`#${this.id_area_popup}_timezone`);
		const $timezoneContainer = $(`#${this.id_area_popup}_timezone_container`);
		const isNew = !area;
		area = area || {};

		$country.val(this.country_description(this.selected_country_id));
		this.readonly_control($code, !isNew);
		$code.val(area.areaCode || "");
		$name.val(area.description || "");
		$timezone.val("");
		$timezoneContainer.toggle(isNew);

		this.popup_title($popup, isNew ? "<%=_T('Add Area')%>" : "<%=_T('Edit Area')%>");
		this.popup_actions($popup, () => this.save_area_popup($popup, isNew));
		this.popup_show($popup);
	};

	adm_form_phone_validator.prototype.save_area_popup = async function($popup, isNew)
	{
		const $code = $(`#${this.id_area_popup}_code`);
		const $name = $(`#${this.id_area_popup}_name`);
		const $timezone = $(`#${this.id_area_popup}_timezone`);

		this.hide_popup_errors($popup);
		if ($code.val() && !this.popup_validate($popup, $code, "digits", "<%=_T('Area code is not a number.')%>"))
			return;
		if (!this.popup_validate($popup, $name, "required", "<%=_T('Area name is required.')%>"))
			return;
		if (isNew && !this.popup_validate($popup, $timezone, "required", "<%=_T('Timezone is required.')%>"))
			return;

		this.popup_waiting($popup, true);
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("saveArea", this.selected_country_id, {
				areaCode: $code.val() || "",
				description: $name.val(),
				timezone: isNew ? $timezone.val() : null
			});
			await this.refresh_areas();
			this.popup_close($popup);
		} catch(_e) {
			this.show_popup_error($popup, "<%=_T('Error saving area.')%>");
		}
		this.popup_waiting($popup, false);
	};

	adm_form_phone_validator.prototype.handle_delete_area = function(area)
	{
		const confirmationMessage = area.status === "added"
			? "<%=_T('Do you want to delete area {0}?')%>"
			: "<%=_T('Do you want to restore area {0}?')%>";
		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			format(confirmationMessage, area.areaCode),
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			(response) => {
				if (response.toLowerCase() !== "yes") return;
				
				this.delete_area(area);
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_phone_validator.prototype.delete_area = async function(area)
	{
		this.show_waiting("<%=_T('Please wait')%>" + "...");
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("deleteArea", this.selected_country_id, {
				areaCode: area.areaCode
			});
			await this.refresh_areas();
		} catch(_e) {
			this.show_status_message("<%=_T('Error deleting area.')%>", "Error");
		}
		this.hide_waiting();
	};

	adm_form_phone_validator.prototype.handle_restore_areas = function()
	{
		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			"<%=_T('Do you want to restore all areas?')%>",
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			(response) => {
				if (response.toLowerCase() !== "yes") return;
				
				this.delete_areas();
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_phone_validator.prototype.delete_areas = async function()
	{
		this.show_waiting("<%=_T('Please wait')%>" + "...");
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("deleteAreas", this.selected_country_id);
			await this.refresh_areas();
		} catch(_e) {
			this.show_status_message("<%=_T('Error deleting rule.')%>", "Error");
		}
		this.hide_waiting();
	};

	adm_form_phone_validator.prototype.handle_global_restore = function()
	{
		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			"<%=_T('Do you want to restore global customizations?')%>",
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			(response) => {
				if (response.toLowerCase() !== "yes") return;
				
				this.delete_global_rules();
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_phone_validator.prototype.delete_global_rules = async function()
	{
		this.show_waiting("<%=_T('Please wait')%>" + "...");
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("deleteGlobalRules", this.selected_country_id);
			await this.refresh_global();
		} catch(_e) {
			this.show_status_message("<%=_T('Error deleting area.')%>", "Error");
		}
		this.hide_waiting();
	};

	adm_form_phone_validator.prototype.handle_delete_rule = function(type, prefix, areaCode)
	{
		const confirmationMessage = prefix.status === "added"
			? "<%=_T('Do you want to delete rule {0}?')%>"
			: "<%=_T('Do you want to restore rule {0}?')%>";
		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			format(confirmationMessage, prefix.prefix),
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			(response) => {
				if (response.toLowerCase() !== "yes") return;
				
				this.delete_rule(type, prefix, areaCode);
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_form_phone_validator.prototype.delete_rule = async function(type, prefix, areaCode)
	{
		const schemaType =
			$(`#${areaCode !== undefined ? this.id_area_schema_type : this.id_global_schema_type}`).val();
		this.show_waiting("<%=_T('Please wait')%>" + "...");
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("deleteRule", this.selected_country_id, {
				type,
				areaCode,
				schemaType,
				prefix: prefix.prefix
			});
			await this.refresh_all();
		} catch(_e) {
			this.show_status_message("<%=_T('Error deleting area.')%>", "Error");
		}
		this.hide_waiting();
	};

	adm_form_phone_validator.prototype.refresh_data = async function()
	{
		const api = inConcert.app.getPhoneValidatorAPI();
		this.entity = await api.asyncCall("getSchema");
	};

	adm_form_phone_validator.prototype.refresh_areas = async function()
	{
		await this.refresh_data();
		this.set_areas();
	};

	adm_form_phone_validator.prototype.refresh_global = async function()
	{
		await this.refresh_data();
		this.set_global();
	};

	adm_form_phone_validator.prototype.refresh_all = async function()
	{
		await this.refresh_data();
		this.set_areas();
		this.set_global();
	};

	adm_form_phone_validator.prototype.handle_add_global_rule = function()
	{
		const type = $(`#${this.id_global_type}`).val();
		const schemaType = $(`#${this.id_global_schema_type}`).val();
		this[`show_${type}_rule_popup`](null, null, schemaType);
	};

	adm_form_phone_validator.prototype.handle_add_area_rule = function(areaCode)
	{
		const type = $(`#${this.id_area_type}`).val();
		const schemaType = $(`#${this.id_area_schema_type}`).val();
		this[`show_${type}_rule_popup`](null, areaCode, schemaType);
	};

	adm_form_phone_validator.prototype.handle_edit_validation_rule = function(prefix, areaCode)
	{
		this.show_validation_rule_popup(prefix, areaCode);
	};

	adm_form_phone_validator.prototype.handle_edit_schema_rule = function(prefix, areaCode)
	{
		const schemaType =
			$(`#${areaCode !== undefined ? this.id_area_schema_type : this.id_global_schema_type}`).val();
		this.show_schema_rule_popup(prefix, areaCode, schemaType);
	};

	adm_form_phone_validator.prototype.handle_edit_timezone_rule = function(prefix, areaCode)
	{
		this.show_timezone_rule_popup(prefix, areaCode);
	};

	adm_form_phone_validator.prototype.show_validation_rule_popup = function(prefix, areaCode)
	{
		const $popup = $(`#${this.id_validation_popup}`);
		const isNew = !prefix;
		prefix = prefix || {};

		this.initialize_validation_rule_popup(
			$popup, prefix.prefix, this.extract_validation_rule(prefix.data), prefix.description, isNew);
		this.popup_title($popup, isNew ? "<%=_T('New Validation Rule')%>" : "<%=_T('Edit Validation Rule')%>");
		this.popup_actions($popup, () => this.save_validation_popup($popup, areaCode));
		this.popup_show($popup);
	};

	adm_form_phone_validator.prototype.make_validation_rule_disclaimer = function(ruleStr)
	{
		const rule = this.extract_validation_rule(ruleStr);
		return this[`make_validation_rule_${rule.rule}_disclaimer`](rule);
	};

	adm_form_phone_validator.prototype.make_validation_rule_valid_disclaimer = function(rule)
	{
		const country = this.make_validation_rule_section_disclaimer(rule.country);
		const area = this.make_validation_rule_section_disclaimer(rule.area) || "<%=_T('Empty')%>";
		const number = this.make_validation_rule_section_disclaimer(rule.number);
		const length = rule.lengthFrom !== rule.lengthTo
			? format("<%=_T('fixed length {0}')%>", rule.lengthFrom)
			: format("<%=_T('length from {0} to {1}')%>", rule.lengthFrom, rule.lengthTo);

		return format(
			"<%=_T('Result number has country {0}, area {1} and number {2}. Will be validated based on a {3}.')%>",
			country, area, number, length);
	};

	adm_form_phone_validator.prototype.make_validation_rule_invalid_disclaimer = function()
	{
		return "<%=_T('Matched number is automatically invalid.')%>";
	};

	adm_form_phone_validator.prototype.make_validation_rule_forward_disclaimer = function(rule)
	{
		const country = this.make_validation_rule_section_disclaimer(rule.country);
		const area = this.make_validation_rule_section_disclaimer(rule.area) || "<%=_T('Global')%>";
		const number = this.make_validation_rule_section_disclaimer(rule.number);

		return format("<%=_T('Forward to country {0}, area {1}. Sent number is {2}.')%>",
			country, area, number);
	};

	adm_form_phone_validator.prototype.make_validation_rule_section_disclaimer = function(section)
	{
		if (section.mode === "fixed")
			return section.fixed;

		let text = format("<%=_T('part of the source number from {0} to {1}')%>",
			section.from, section.to || "<%=_T('end')%>");
		if (section.prefix)
			text += format(" <%=_T('with prefix {0}')%>", section.prefix);
		return text;
	};

	adm_form_phone_validator.prototype.extract_validation_rule = function(ruleStr)
	{
		if (!ruleStr) return { rule: "valid" };

		const ruleParts = ruleStr.split("|");
		return this[`extract_${ruleParts[0]}_validation_rule`](ruleParts);
	};

	adm_form_phone_validator.prototype.extract_rule_brackets = function(section)
	{
		const extractBegin = section.indexOf("{");
		if (extractBegin === -1)
			return {
				mode: "fixed",
				fixed: section
			};

		const prefix = section.substring(0, extractBegin);
		const sectionParts = section.substring(extractBegin + 1, section.length - 1).split(":");
		return {
			mode: "number",
			prefix: prefix,
			from: sectionParts[0],
			to: sectionParts[1],
		};
	};

	adm_form_phone_validator.prototype.extract_valid_validation_rule = function(ruleParts)
	{
		const numberPart = ruleParts[1].split("-");
		const lengthPart = ruleParts[2].split("-");

		return {
			rule: "valid",
			lengthFrom: lengthPart[0],
			lengthTo: lengthPart[1] || lengthPart[0],
			country: this.extract_rule_brackets(numberPart[0]),
			area: this.extract_rule_brackets(numberPart[1]),
			number: this.extract_rule_brackets(numberPart[2])
		};
	};

	adm_form_phone_validator.prototype.extract_invalid_validation_rule = function()
	{
		return { rule: "invalid" };
	};

	adm_form_phone_validator.prototype.extract_fw_validation_rule = function(ruleParts)
	{
		const countryAreaPart = ruleParts[1].split("-");
		const numberPart = ruleParts[2];

		return {
			rule: "forward",
			country: this.extract_rule_brackets(countryAreaPart[0]),
			area: this.extract_rule_brackets(countryAreaPart[1] || ""),
			number: this.extract_rule_brackets(numberPart)
		};
	};

	adm_form_phone_validator.prototype.make_preview_validation_rule = function()
	{
		const rule = $(`#${this.id_validation_popup}_rule`).val();
		return this[`preview_${rule}_validation_rule`]();
	};

	adm_form_phone_validator.prototype.preview_validation_rule = function()
	{
		const htmlPreview = ["|", ":", "-", "{", "}"].reduce(
			(code, sym) => code.replaceAll(sym, `<span class="previewSymbol">${sym}</span>`),
			this.make_preview_validation_rule()
		);
		$(`#${this.id_validation_popup}_preview`).html(htmlPreview);
	};

	adm_form_phone_validator.prototype.preview_rule_brackets = function(prefix, from, to)
	{
		return `${prefix}{${from}:${to}}`;
	};

	adm_form_phone_validator.prototype.preview_validation_rule_section = function(section)
	{
		const mode = $(`#${this.id_validation_popup}_${section}_mode`).val();
		if (mode === "fixed")
			return $(`#${this.id_validation_popup}_${section}_fixed`).val() || "";

		return this.preview_rule_brackets(
			$(`#${this.id_validation_popup}_${section}_prefix`).val() || "",
			$(`#${this.id_validation_popup}_${section}_from`).val() || "",
			$(`#${this.id_validation_popup}_${section}_to`).val() || ""
		);
	};

	adm_form_phone_validator.prototype.preview_valid_validation_rule = function()
	{
		const country = this.preview_validation_rule_section("country");
		const area = this.preview_validation_rule_section("area");
		const number = this.preview_validation_rule_section("number");
		
		const lengthFrom = $(`#${this.id_validation_popup}_length_from`).val() || "";
		const lengthTo = $(`#${this.id_validation_popup}_length_to`).val() || lengthFrom;
		const length = lengthTo !== lengthFrom ? `${lengthFrom}-${lengthTo}` : lengthFrom;

		return `valid|${country}-${area}-${number}|${length}`;
	};

	adm_form_phone_validator.prototype.preview_invalid_validation_rule = function()
	{
		return "invalid";
	};

	adm_form_phone_validator.prototype.preview_forward_validation_rule = function()
	{	
		const country = this.preview_validation_rule_section("country");
		const area = this.preview_validation_rule_section("area");
		const number = this.preview_validation_rule_section("number");

		return `fw|${country}${area ? `-${area}` : ""}|${number}`;
	};

	adm_form_phone_validator.prototype.initialize_validation_rule_popup = function($popup, prefix, data, description, isNew)
	{
		$(`#${this.id_validation_popup}_comments`).val(description || "");
		this.readonly_control($(`#${this.id_validation_popup}_prefix`), !isNew);
		$(`#${this.id_validation_popup}_prefix`).val(prefix || "");
		$(`#${this.id_validation_popup}_prefix`).attr(
			"placeholder", isNew ? "<%=_T('Enter prefix number')%>" : "");
		$(`#${this.id_validation_popup}_rule`)
			.val(data.rule)
			.off("change")
			.on("change", () => {
				this.handle_validation_rule_change($popup);
				this.preview_validation_rule();
			});

		this.handle_validation_rule_change($popup, data);
		this.preview_validation_rule();
	};

	adm_form_phone_validator.prototype.handle_validation_rule_change = function($popup, data = {})
	{
		const rule = $(`#${this.id_validation_popup}_rule`).val();
		$popup
			.find("[validation_rule]")
			.hide()
			.filter(`[validation_rule=${rule}]`)
			.show();

		this[`handle_validation_rule_${rule}_change`]($popup, data);
		this.set_disclaimer(DISCLAIMERS.validation_rule, `${this.id_validation_popup}_rule`);
	};

	adm_form_phone_validator.prototype.handle_validation_rule_valid_change = function($popup, data = {})
	{
		$(`#${this.id_validation_popup}_length_from`)
			.val(data.lengthFrom || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());

		$(`#${this.id_validation_popup}_length_to`)
			.val(data.lengthTo || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());

		this.handle_validation_rule_section_change($popup, "country", data);
		this.handle_validation_rule_section_change($popup, "area", data);
		this.handle_validation_rule_section_change($popup, "number", data);
	};

	adm_form_phone_validator.prototype.handle_validation_rule_invalid_change = function() {};

	adm_form_phone_validator.prototype.handle_validation_rule_forward_change = function($popup, data = {})
	{
		this.handle_validation_rule_section_change($popup, "country", data);
		this.handle_validation_rule_section_change($popup, "area", data);
		this.handle_validation_rule_section_change($popup, "number", data);
	};

	adm_form_phone_validator.prototype.handle_validation_rule_section_change = function($popup, section, data = {})
	{
		$popup.find(`[validation_rule=${section}]`).show();

		const sectionData = data[section] || {};
		$(`#${this.id_validation_popup}_${section}_mode`)
			.val(sectionData.mode || (section === "number" ? "number" : "fixed"))
			.off("change")
			.on("change", () => {
				this.handle_validation_rule_section_mode_change($popup, section);
				this.preview_validation_rule();
			});

		$(`#${this.id_validation_popup}_${section}_fixed`)
			.val(sectionData.fixed || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());

		$(`#${this.id_validation_popup}_${section}_prefix`)
			.val(sectionData.prefix || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());
		
		$(`#${this.id_validation_popup}_${section}_from`)
			.val(sectionData.from || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());
		
		$(`#${this.id_validation_popup}_${section}_to`)
			.val(sectionData.to || "")
			.off("change")
			.on("change", () => this.preview_validation_rule());

		this.handle_validation_rule_section_mode_change($popup, section);
	};

	adm_form_phone_validator.prototype.handle_validation_rule_section_mode_change = function($popup, section)
	{
		const mode = $(`#${this.id_validation_popup}_${section}_mode`).val();
		$popup
			.find(`[${section}_section_mode]`)
			.hide()
			.filter(`[${section}_section_mode=${mode}]`)
			.show();
	};

	adm_form_phone_validator.prototype.validate_validation_popup_section = function($popup, section)
	{
		if ($(`#${this.id_validation_popup}_${section}_mode`).val() === "fixed") {
			const $fixed = $(`#${this.id_validation_popup}_${section}_fixed`);
			return section === "area" || this.popup_validate(
				$popup, $fixed, "required", "<%=_T('Fixed value is required.')%>");
		}

		const $from = $(`#${this.id_validation_popup}_${section}_from`);
		const $to = $(`#${this.id_validation_popup}_${section}_from`);

		return this.popup_validate($popup, $from, "required", "<%=_T('From position is required.')%>") &&
			this.popup_validate($popup, $from, "digits", "<%=_T('From position is not a number.')%>") &&
			(!$to.val() ||
				this.popup_validate($popup, $to, "digits", "<%=_T('To position is not a number.')%>"));
	};

	adm_form_phone_validator.prototype.validate_validation_sections_popup = function($popup)
	{
		return ["country", "area", "number"]
			.every((section) => this.validate_validation_popup_section($popup, section));
	};

	adm_form_phone_validator.prototype.validate_validation_valid_popup_existing_country = function($popup)
	{
		if ($(`#${this.id_validation_popup}_country_mode`).val() !== "fixed")
			return true;

		const countryId = $(`#${this.id_validation_popup}_country_fixed`).val();
		if (!this.entity[countryId]) {
			this.show_popup_error($popup, format("<%=_T('Country {0} does not exist.')%>", countryId));
			return false;
		}

		return true;
	};

	adm_form_phone_validator.prototype.validate_validation_valid_popup_existing_area = function($popup)
	{
		if ($(`#${this.id_validation_popup}_country_mode`).val() !== "fixed" ||
			$(`#${this.id_validation_popup}_area_mode`).val() !== "fixed" )
			return true;

		const countryId = $(`#${this.id_validation_popup}_country_fixed`).val();
		const areaId = $(`#${this.id_validation_popup}_area_fixed`).val();
		if (areaId && !this.entity[countryId].areas[areaId]) {
			this.show_popup_error($popup,
				format("<%=_T('Country {0} area {1} does not exist. Please create it.')%>", countryId, areaId));
			return false;
		}

		return true;			
	};

	adm_form_phone_validator.prototype.validate_validation_valid_popup = function($popup)
	{
		const $from = $(`#${this.id_validation_popup}_length_from`);
		const $to = $(`#${this.id_validation_popup}_length_to`);
		return this.popup_validate($popup, $from, "required", "<%=_T('From length is required.')%>") &&
			this.popup_validate($popup, $from, "digits", "<%=_T('From length is not a number.')%>") &&
			(!$to.val() ||
				this.popup_validate($popup, $to, "digits", "<%=_T('To length is not a number.')%>"))
			&& this.validate_validation_sections_popup($popup)
			&& this.validate_validation_valid_popup_existing_country($popup)
			&& this.validate_validation_valid_popup_existing_area($popup);
	};

	adm_form_phone_validator.prototype.validate_validation_invalid_popup = function()
	{
		return true;
	};

	adm_form_phone_validator.prototype.validate_validation_forward_popup = function($popup)
	{
		return this.validate_validation_sections_popup($popup);
	};

	adm_form_phone_validator.prototype.validate_validation_popup = function($popup)
	{
		const rule = $(`#${this.id_validation_popup}_rule`).val();
		return this[`validate_validation_${rule}_popup`]($popup);
	};

	adm_form_phone_validator.prototype.save_validation_popup = async function($popup, areaCode)
	{
		this.hide_popup_errors($popup);
		if (!this.validate_validation_popup($popup))
			return;

		const rule = {
			areaCode,
			type: "validation",
			prefix: $(`#${this.id_validation_popup}_prefix`).val() || "",
			description: $(`#${this.id_validation_popup}_comments`).val() || "",
			data: this.make_preview_validation_rule(),
		};

		this.popup_waiting($popup, true);
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("saveRule", this.selected_country_id, rule);
			await this.refresh_all();
			this.popup_close($popup);
		} catch(_e) {
			this.show_popup_error($popup, "<%=_T('Error saving rule.')%>");
		}
		this.popup_waiting($popup, false);
	};

	adm_form_phone_validator.prototype.show_schema_rule_popup = function(prefix, areaCode, schemaType)
	{
		const $popup = $(`#${this.id_schema_popup}`);
		const isNew = !prefix;
		prefix = prefix || {};

		this.initialize_schema_rule_popup(
			prefix.prefix, this.extract_schema_rule(prefix.data), prefix.description, isNew);
		this.popup_title($popup, isNew ? "<%=_T('New Schema Rule')%>" : "<%=_T('Edit Schema Rule')%>");
		this.popup_actions($popup, () => this.save_schema_popup($popup, areaCode, schemaType));
		this.popup_show($popup);
	};

	adm_form_phone_validator.prototype.extract_schema_rule = function(ruleStr)
	{
		if (!ruleStr) return { number: { prefix: "" } };

		const parts = ruleStr.split(/\{|\}/);
		let lastPrefix = "";
		return parts.reduce((data, part, index) => {
			if (index % 2) {
				const partSections = part.split(":");
				data[partSections[0]] = { prefix: lastPrefix, from: partSections[1] };
			} else {
				lastPrefix = part;
			}
			return data;
		}, {});
	};

	adm_form_phone_validator.prototype.preview_schema_rule = function()
	{
		$(`#${this.id_schema_popup}_preview`).text(this.make_preview_schema_rule());
	};

	adm_form_phone_validator.prototype.make_preview_schema_rule = function()
	{
		return ["country", "area", "number"]
			.map((section) => this.make_preview_schema_rule_section(section))
			.join("");
	};

	adm_form_phone_validator.prototype.make_preview_schema_rule_section = function(section)
	{
		if (!$(`#${this.id_schema_popup}_${section}`).is(":checked"))
			return "";

		const prefix = $(`#${this.id_schema_popup}_${section}_prefix`).val() || "";
		const from = $(`#${this.id_schema_popup}_${section}_from`).val() || "";
		return `${prefix}{${section}${from ? `:${from}` : ""}}`;
	};

	adm_form_phone_validator.prototype.initialize_schema_rule_popup = function(prefix, data, description, isNew)
	{
		$(`#${this.id_schema_popup}_comments`).val(description || "");
		this.readonly_control($(`#${this.id_schema_popup}_prefix`), !isNew);
		$(`#${this.id_schema_popup}_prefix`).attr(
			"placeholder", isNew ? "<%=_T('Enter prefix number')%>" : "");
		$(`#${this.id_schema_popup}_prefix`).val(prefix || "");

		["country", "area", "number"]
			.forEach((section) => this.initialize_schema_rule_popup_section(section, data[section]));
		this.preview_schema_rule();
	};

	adm_form_phone_validator.prototype.initialize_schema_rule_popup_section = function(section, data)
	{
		$(`#${this.id_schema_popup}_${section}`)
			.prop("checked", !!data)
			.off("change")
			.on("change", () => {
				this.handle_schema_rule_popup_section_change(section);
				this.preview_schema_rule();
			});

		$(`#${this.id_schema_popup}_${section}_prefix`)
			.val(data?.prefix || "")
			.off("change")
			.on("change", () => this.preview_schema_rule());

		$(`#${this.id_schema_popup}_${section}_from`)
			.val(data?.from || "")
			.off("change")
			.on("change", () => this.preview_schema_rule());

		this.handle_schema_rule_popup_section_change(section);
	};

	adm_form_phone_validator.prototype.handle_schema_rule_popup_section_change = function(section)
	{
		const disabled = !$(`#${this.id_schema_popup}_${section}`).is(":checked");
		this.disable_control($(`#${this.id_schema_popup}_${section}_prefix`), disabled);
		this.disable_control($(`#${this.id_schema_popup}_${section}_from`), disabled);
	};

	adm_form_phone_validator.prototype.validate_schema_popup = function($popup)
	{
		return ["country", "area", "number"].every((section) => {
			if (!$(`#${this.id_schema_popup}_${section}`).is(":checked"))
				return true;
				
			const $prefix = $(`#${this.id_schema_popup}_${section}_prefix`);
			const $from = $(`#${this.id_schema_popup}_${section}_from`);
			return (
					!$prefix.val() ||
					this.popup_validate($popup, $prefix, "digits", "<%=_T('Prefix is not a number.')%>")
				) && (
					section !== "number" ||
					!$from.val() ||
					this.popup_validate($popup, $from, "digits", "<%=_T('From position is not a number.')%>")
				);
		});
	};

	adm_form_phone_validator.prototype.save_schema_popup = async function($popup, areaCode, schemaType)
	{
		this.hide_popup_errors($popup);
		if (!this.validate_schema_popup($popup))
			return;

		const rule = {
			areaCode,
			schemaType,
			type: "schema",
			prefix: $(`#${this.id_schema_popup}_prefix`).val() || "",
			description: $(`#${this.id_schema_popup}_comments`).val() || "",
			data: this.make_preview_schema_rule(),
		};

		this.popup_waiting($popup, true);
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("saveRule", this.selected_country_id, rule);
			await this.refresh_all();
			this.popup_close($popup);
		} catch(_e) {
			this.show_popup_error($popup, "<%=_T('Error saving rule.')%>");
		}
		this.popup_waiting($popup, false);
	};

	adm_form_phone_validator.prototype.show_timezone_rule_popup = function(prefix, areaCode)
	{
		const $popup = $(`#${this.id_timezone_popup}`);
		const isNew = !prefix;
		prefix = prefix || {};

		this.initialize_timezone_rule_popup(
			prefix.prefix, prefix.data, prefix.description, isNew);
		this.popup_title($popup, isNew ? "<%=_T('New Timezone Rule')%>" : "<%=_T('Edit Timezone Rule')%>");
		this.popup_actions($popup, () => this.save_timezone_popup($popup, areaCode));
		this.popup_show($popup);
	};

	adm_form_phone_validator.prototype.initialize_timezone_rule_popup = function(prefix, data, description, isNew)
	{
		$(`#${this.id_timezone_popup}_comments`).val(description || "");
		this.readonly_control($(`#${this.id_timezone_popup}_prefix`), !isNew);
		$(`#${this.id_timezone_popup}_prefix`).attr(
			"placeholder", isNew ? "<%=_T('Enter prefix number')%>" : "");
		$(`#${this.id_timezone_popup}_prefix`).val(prefix || "");
		$(`#${this.id_timezone_popup}_timezone`).val(data || "");
	};

	adm_form_phone_validator.prototype.validate_timezone_popup = function($popup)
	{
		const $timezone = $(`#${this.id_timezone_popup}_timezone`);
		return this.popup_validate($popup, $timezone, "required", "<%=_T('Timezone is required.')%>");
	};

	adm_form_phone_validator.prototype.save_timezone_popup = async function($popup, areaCode)
	{
		this.hide_popup_errors($popup);
		if (!this.validate_timezone_popup($popup))
			return;

		const rule = {
			areaCode,
			type: "timezone",
			prefix: $(`#${this.id_timezone_popup}_prefix`).val() || "",
			description: $(`#${this.id_timezone_popup}_comments`).val() || "",
			data: $(`#${this.id_timezone_popup}_timezone`).val(),
		};

		this.popup_waiting($popup, true);
		try {
			const api = inConcert.app.getPhoneValidatorAPI();
			await api.asyncCall("saveRule", this.selected_country_id, rule);
			await this.refresh_all();
			this.popup_close($popup);
		} catch(_e) {
			this.show_popup_error($popup, "<%=_T('Error saving rule.')%>");
		}
		this.popup_waiting($popup, false);
	};

	adm_form_phone_validator.prototype.show_test_rule_popup = function()
	{
		const $popup = $(`#${this.id_test_popup}`);
		this.initialize_test_rule_popup();
		this.popup_title($popup, "<%=_T('Validate Number')%>");
		this.popup_actions($popup, () => this.run_test_rule_popup($popup));
		this.popup_show($popup);
	};

	adm_form_phone_validator.prototype.initialize_test_rule_popup = function()
	{
		this.set_test_countries();
		this.clean_test_rule_popup_result();
	};

	adm_form_phone_validator.prototype.set_test_countries = function()
	{
		const $country = $(`#${this.id_test_popup}_country`);
		const countryId = $country.val() || this.selected_country_id || "Global";
		this.fill_combo($country, false, this.make_countries_list(), "value", "label");
		$country.val(countryId).off("change").on("change", () => this.handle_test_country_change());
		this.handle_test_country_change($(`#${this.id_test_popup}_area`).val());
	};

	adm_form_phone_validator.prototype.handle_test_country_change = function(areaId)
	{
		const $area = $(`#${this.id_test_popup}_area`);
		const areas = this.entity[$(`#${this.id_test_popup}_country`).val()]?.areas || {};
		const sortedAreas = Object.keys(areas)
			.sort()
			.map((areaCode) => ({
				value: areaCode,
				label: `${areaCode || "<%=_T('Empty')%>"} - ${areas[areaCode].description}`
			}));
		this.fill_combo($area, false, sortedAreas, "value", "label");
		$area.val(areaId || "");
	};

	adm_form_phone_validator.prototype.validate_test_popup = function($popup)
	{
		const $country = $(`#${this.id_test_popup}_country`);
		const $number = $(`#${this.id_test_popup}_number`);
		return this.popup_validate($popup, $country, "required", "<%=_T('Country Code is required.')%>") &&
			this.popup_validate($popup, $number, "required", "<%=_T('Phone Number is required.')%>");
	};

	adm_form_phone_validator.prototype.run_test_rule_popup = async function($popup)
	{
		this.hide_popup_errors($popup);
		this.clean_test_rule_popup_result();
		if (!this.validate_test_popup($popup))
			return;

		const test = {
			country: $(`#${this.id_test_popup}_country`).val(),
			area: $(`#${this.id_test_popup}_area`).val() || "",
			number: $(`#${this.id_test_popup}_number`).val(),
			withTrace: true
		};

		this.popup_waiting($popup, true);
		try {
			const api = inConcert.app.getOutboundEngineAPI();
			const result = await api.asyncCall("ValidateAddress", test);
			this.process_test_rule_popup_success($popup, result);
		} catch(error) {
			this.process_test_rule_popup_error($popup, error);
		}
		$(`#${this.id_test_popup}_result`).show();
		this.popup_waiting($popup, false);
	};

	adm_form_phone_validator.prototype.clean_test_rule_popup_result_table = function(section)
	{
		const $table = $(`#${this.id_test_popup}_${section}_table`);
		$table.find("tbody").html("");
		$table.find("table").hide();
		$table.find("[empty_table]").show();
	};

	adm_form_phone_validator.prototype.clean_test_rule_popup_result = function()
	{
		$(`#${this.id_test_popup}_result`).hide();
		this.clean_test_rule_popup_result_table("validation");
		this.clean_test_rule_popup_result_table("schema");
		this.clean_test_rule_popup_result_table("timezone");
	};

	adm_form_phone_validator.prototype.process_test_rule_popup_success = function($popup, result)
	{
		this.show_popup_error($popup, "<%=_T('Phone validated successfuly.')%>", "Ok");
		this.process_test_rule_validation(result.trace.validations, result);
		this.process_test_rule_schema(result.trace.schema, result);
		this.process_test_rule_timezone(result.trace.timezone, result);
	};

	adm_form_phone_validator.prototype.process_test_rule_popup_error_message = function(result)
	{
		switch(result?.result || "") {
		case "TIMEZONE UNSPECIFIED":
			return "<%=_T('No timezone rule found for this country/area.')%>";
		case "LOOP":
			return "<%=_T('There was a loop validating phone.')%>";
		case "GLOBAL_NUMBER_INCONSISTENT":
			return "<%=_T('Global number was validated as a diferent number.')%>";
		default:
			switch(result?.reason || "") {
			case "Normalization not found":
				return "<%=_T('No validation rule found for this country/area.')%>";
			case "Normalization is invalid":
				return "<%=_T('Phone marked invalid by rules.')%>";
			case "phone is invalid":
				return "<%=_T('Phone validation regex is invalid.')%>";
			case "phone parts not found":
				return "<%=_T('Phone does not match validation regex.')%>";
			case "phone length invalid":
				return "<%=_T('Phone length is invalid.')%>";
			case "dialing number not found":
				return "<%=_T('No dialing number rule found for this country/area.')%>";
			default:
				return "<%=_T('Phone is invalid.')%>";
			}
		}
	};

	adm_form_phone_validator.prototype.process_test_rule_popup_error = function($popup, result)
	{
		this.show_popup_error($popup, this.process_test_rule_popup_error_message(result));
		this.process_test_rule_validation(result?.trace?.validations || [], result);
	};

	adm_form_phone_validator.prototype.validation_retry_message = function(retry)
	{
		switch(retry) {
			case "area":
				return "<%=_T('Will retry without area.')%>";
			case "local":
				return "<%=_T('Will validate global number consistency.')%>";
			default:
				return "<%=_T('Will retry with global number.')%>";
		}
	};

	adm_form_phone_validator.prototype.process_test_rule_validation = function(validations, finalResult)
	{
		const makeResultRow = (result, sufix) => {
			let message = result.result === "OK"
				? "<%=_T('Phone validated successfuly.')%>"
				: this.process_test_rule_popup_error_message(result);
			if (sufix) message = message.slice(0, -1);
			return $(`
				<tr>
					<td colspan="5">${message}${sufix ? ` - ${sufix}` : ""}</td>
				</tr>
			`);
		};

		const $table = $(`#${this.id_test_popup}_validation_table`);
		$table.find("tbody").append(validations.map((validation) => {
			if (validation.retry)
				return makeResultRow(validation.result, this.validation_retry_message(validation.retry));

			return $(`
				<tr>
					<td>${validation.ref.split("-")[0]}</td>
					<td>${validation.ref.split("-")[1] || "<%=_T('Empty')%>"}</td>
					<td>${validation.number}</td>
					<td>${validation.prefix.replace(".", "") || "<%=_T('Any')%>"}</td>
					<td>${validation.rule}</td>
				</tr>
			`);
		}));
		$table.find("tbody").append(makeResultRow(finalResult));
		$table.find("table").show();
		$table.find("[empty_table]").hide();
	};

	adm_form_phone_validator.prototype.process_test_rule_schema = function(traceSchema, result)
	{
		const phoneTypeLabels = {
			Global: "<%=_T('Global')%>",
			SameCountry: "<%=_T('Same Country')%>",
			Local: "<%=_T('Local')%>"
		};

		const $table = $(`#${this.id_test_popup}_schema_table`);
		$table.find("tbody").append(Object.keys(traceSchema).sort().map((section) => $(`
			<tr>
				<td>${result.countryCode}</td>
				<td>${result.areaCode || "<%=_T('Empty')%>"}</td>
				<td>${phoneTypeLabels[section]}</td>
				<td>${traceSchema[section].prefix.replace(".", "") || "<%=_T('Any')%>"}</td>
				<td>${traceSchema[section].rule}</td>
				<td>${result[`dial${section}`]}</td>
			</tr>
		`)));
		$table.find("table").show();
		$table.find("[empty_table]").hide();
	};

	adm_form_phone_validator.prototype.process_test_rule_timezone = function(traceTimezone, result)
	{
		const $table = $(`#${this.id_test_popup}_timezone_table`);
		$table.find("tbody").append($(`
			<tr>
				<td>${result.countryCode}</td>
				<td>${result.areaCode || "<%=_T('Empty')%>"}</td>
				<td>${traceTimezone.replace(".", "") || "<%=_T('Any')%>"}</td>
				<td>${result.timezone.id}</td>
				<td>${this.get_timezone_offset(result.timezone.id)}</td>
			</tr>
		`));
		$table.find("table").show();
		$table.find("[empty_table]").hide();
	};

}());

