/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, ace */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form asterisk class
	function form_asterisk(params)
	{
		this.frm = params.parent;
		this.code_editor = null;
		this.is_new = false;
		this.current_dialing_rule = null;
		this.annotations = null;
		this.current_annotation = -1;
	}

	form_asterisk.prototype.__initialize = function()
	{
		this.hide_all_errors();
		this.__initialize_ctrls();
		this.__set_events();
	};

	form_asterisk.prototype.dispose = function()
	{
		this.is_new = false;
		this.code_editor = null;
		this.current_dialing_rule = null;
		this.annotations = null;
		this.current_annotation = -1;
	};

	form_asterisk.prototype.is_new_dialign_rule = function()
	{
		return this.is_new;
	};

	form_asterisk.prototype.__initialize_ctrls = function()
	{
		this.__make_code_editor();

		var dialing_rule = this.current_dialing_rule || {};
		var $id = $("#adm_form_routing_group_dialing_rules_asterisk_popup_id");
		var $desc = $("#adm_form_routing_group_dialing_rules_asterisk_popup_name");

		$id.val(dialing_rule.Id || "");
		$desc.val(dialing_rule.Description || "");
		this.code_editor.setValue(dialing_rule.Rules || "");

		if (dialing_rule.Rules) {
			this.code_editor.focus();
			this.code_editor.gotoLine(0);
		}

		var $title = $("#adm_form_routing_group_dialing_rules_asterisk_popup_title");
		var $create = $("#adm_form_routing_group_dialing_rules_asterisk_popup_create");
		var $previous = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_previous");
		var $next = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_next");

		var is_readonly_frm = this.frm.is_readonly();

		if (this.is_new_dialign_rule()) {
			$title.text("<%=_T('New Dialign Rule')%>");
			$create.val("<%=_T('Create')%>");
			this.frm.readonly_control($id, false, true);
			$id.focus();
		}
		else {
			if (is_readonly_frm) {
				$title.text("<%=_T('View Dialign Rule')%>");
				$create.hide();
				var $sintax = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_check_sintax");
				$sintax.hide();
				$previous.hide();
				$next.hide();
				var $help = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_help");
				$help.hide();
				this.code_editor.setReadOnly(true);
			}
			else {
				$title.text("<%=_T('Update Dialign Rule')%>");
				$create.val("<%=_T('Update')%>");
			}

			this.frm.readonly_control($id, true);
			$desc.focus();
		}

		// disabled previous / next errors buttons
		if (!is_readonly_frm) {
			$previous.prop("disabled", true);
			$next.prop("disabled", true);
		}
	};

	form_asterisk.prototype.__make_code_editor = function()
	{
		this.code_editor = null;
		const $code_block = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block");
		const code_block = $code_block[0];
		ace.require("ace/ext/language_tools");
		this.code_editor = ace.edit(code_block);
		this.code_editor.setTheme("ace/theme/clouds");
		this.code_editor.getSession().setMode("ace/mode/dialplan");
		this.code_editor.setOptions({
			enableBasicAutocompletion: true, // TODO: revisar \mode-dialplan_completions.js
			enableSnippets: true // TODO: expandir ..\snippets\dialplan.js
		});
	};

	form_asterisk.prototype.get_entity_instance = function()
	{
		return this.frm.get_entity_instance() || {};
	};

	form_asterisk.prototype.hide_all_errors = function()
	{
		var $msg_container = $("#adm_form_routing_group_dialing_rules_asterisk_popup_message");
		$msg_container.hide();
		this.frm.hide_messages();
	};

	form_asterisk.prototype.__show_status_message = function(msg_info)
	{
		if (!msg_info) {
			return;
		}
		var $msg_container = $("#adm_form_routing_group_dialing_rules_asterisk_popup_message");
		this.frm.show_status_message(
			msg_info.msg,
			msg_info.type || "Info",
			msg_info.timeout_ms || 5000,
			undefined, // scrolling
			undefined, // show custom button
			$msg_container
		);
	};

	form_asterisk.prototype.__show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		var $err_msg_container = $("#adm_form_routing_group_dialing_rules_asterisk_popup_message");
		this.frm.show_validation_error(err_info.$ctrl, err_info.message, 5000, undefined, undefined, $err_msg_container);
	};

	form_asterisk.prototype.__get_dialing_rule = function()
	{
		var routing_group = this.get_entity_instance();
		var routing_group_id = routing_group.Id || $("#adm_form_routing_group_name").val();
		return {
			GroupId: routing_group_id,
			Id: $("#adm_form_routing_group_dialing_rules_asterisk_popup_id").val(),
			Description: $("#adm_form_routing_group_dialing_rules_asterisk_popup_name").val(),
			Rules: this.code_editor.getValue()
		};
	};

	/*
		La asterisk solo manda códigos, por un tema de traducción de mensajes aca es donde se decodifica en algo para la UI...
	*/
	form_asterisk.prototype.__get_message_info = function(type)
	{
		switch(type) {
			// generados en este javascript
			case -100:
				return {desc: "<%=_T('Dialing to unassigned trunk on this routing group')%>", annotationType: "warning"};
			// generados en windows por iWH (outgoinggroup.lua)
			case -1:
				return {desc: "<%=_T('Dialing to an undefined trunk')%>", annotationType: "warning"};
			case 0:
				return {desc: "<%=_T('You must define some rules')%>", annotationType: "error"};
			// generados en PBX de lado linux (config.c)
			case 1:
				return {desc: "<%=_T('No closing ]')%>", annotationType: "error"};
			case 2:
				return {desc: "<%=_T('PBX out of memory')%>", annotationType: "error"};
			case 3:
				return {desc: "<%=_T('No closing )')%>", annotationType: "error"};
			case 4:
				return {desc: "<%=_T('Category addition requested, but this category does not exist')%>", annotationType: "error"};
			case 5:
				return {desc: "<%=_T('Inheritance requested, but this category does not exist')%>", annotationType: "error"};
			case 6:
				return {desc: "<%=_T('This directive needs an argument')%>", annotationType: "info"};
			case 7:
				return {desc: "<%=_T('Unknown directive')%>", annotationType: "warning"};
			case 8:
				return {desc: "<%=_T('No category context')%>", annotationType: "error"};
			case 9:
				return {desc: "<%=_T('No = (equal sign) defined')%>", annotationType: "warning"};
			case 10:
				return {desc: "<%=_T('Cannot perform #exec unless execincludes option is enabled in asterisk.conf (options section)!')%>", annotationType: "info"};
			case 11:
				return {desc: "<%=_T('Maximum nest limit reached')%>", annotationType: "error"};
			case 12:
				return {desc: "<%=_T('Unterminated comment detected')%>", annotationType: "error"};
			case 13:
				return {desc: "<%=_T('You are not allowed to define new contexts')%>", annotationType: "error"};
			case 14:
				return {desc: "<%=_T('Multi-line comment end-tag (--;) unmatched... you should add a start-tag (;--) or remove end tag')%>", annotationType: "error"};
			case 15:
				return {desc: "<%=_T('Label missing trailing )')%>", annotationType: "error"};
			case 16:
				return {desc: "<%=_T('Invalid priority/label')%>", annotationType: "warning"};
			case 17:
				return {desc: "<%=_T('No closing parenthesis found?')%>", annotationType: "error"};
			case 18:
				return {desc: "<%=_T('The use of _. for an extension is strongly discouraged and can have unexpected behavior.  Please use _X. instead')%>", annotationType: "warning"};
			case 19:
				return {desc: "<%=_T('Memory allocation failure. Contact to administrator')%>", annotationType: "error"};
		}
		return {desc: "<%=_T('Unknown issue')%>", annotationType: "error"};
	};

	form_asterisk.prototype.__show_sintax_annotations = function(parse_messages)
	{
		var self = this;

		this.annotations = [];

		$.each(parse_messages || [], function(_, message)
		{
			var info = self.__get_message_info(message.type);
			self.annotations.push({
				column: 0,
				row   : message.line - 1,
				type  : info.annotationType,
				text  : info.desc,
				raw   : info.desc
			});
		});

		this.annotations.sort(function(a, b)
		{
			if (a.row === b.row) {
				return 0;
			}
			else if (a.row < b.row) {
				return -1;
			}
			return 1;
		});

		this.code_editor.getSession().setAnnotations(this.annotations);
		this.__set_editor_after_annotations_load();
	};

	form_asterisk.prototype.__set_editor_after_annotations_load = function()
	{
		// Trato de dejarlo en la primera nueva...
		if (this.annotations[this.current_annotation] === undefined) {
			// Intento con la annotations anteriores que quizás no arreglo o podrian ser nuevas...
			while (--this.current_annotation >= 0 && this.annotations[this.current_annotation] === undefined){}
			// Una annotation tiene que haber, por sanidad igual controlo indice:
			if (this.current_annotation < 0) {
				this.current_annotation = 0;
			}
		}
		// boton siguiente
		var $next = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_next");
		if (this.annotations[this.current_annotation + 1] !== undefined) {
			$next.prop("disabled", false);
		}
		else {
			$next.prop("disabled", true);
		}
		// boton previo
		var $previous = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_previous");
		if (this.annotations[this.current_annotation - 1] !== undefined) {
			$previous.prop("disabled", false);
		}
		else {
			$previous.prop("disabled", true);
		}
		if (this.annotations[this.current_annotation] !== undefined) {
			this.code_editor.focus();
			this.code_editor.gotoLine(this.annotations[this.current_annotation].row + 1);
		}
	};

	form_asterisk.prototype.__goto_previous_error = function()
	{
		const $next = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_next");
		$next.prop("disabled", false);

		if (this.annotations[--this.current_annotation] !== undefined) {
			this.code_editor.focus();
			this.code_editor.gotoLine(this.annotations[this.current_annotation].row + 1);

			if (this.annotations[this.current_annotation - 1] === undefined) {
				const $previous = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_previous");
				$previous.prop("disabled", true);
			}
		}
		else {
			// Hubo un cambio en annotations...
			this.__set_editor_after_annotations_load();
		}
	};

	form_asterisk.prototype.__goto_next_error = function()
	{
		const $previous = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_previous");
		$previous.prop("disabled", false);

		if (this.annotations[++this.current_annotation] !== undefined) {
			this.code_editor.focus();
			this.code_editor.gotoLine(this.annotations[this.current_annotation].row + 1);

			if (this.annotations[this.current_annotation + 1] === undefined) {
				const $next = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_next");
				$next.prop("disabled", true);
			}
		}
		else {
			// Hubo un cambio en annotations...
			this.__set_editor_after_annotations_load();
		}
	};

	form_asterisk.prototype.show_waiting = function(message)
	{
		var $waiting_container = $("#adm_form_routing_group_dialing_rules_asterisk_popup_waiting");
		this.frm.show_waiting(message, $waiting_container);
	};

	/*
Sintax Sample:
exten=> _XX.,1,Dial(SIP/100,60,tT)
exten=> _XX.,n,Hangup
	*/
	form_asterisk.prototype.__check_sintax_code_block = function(callback)
	{
		var self = this;

		callback = callback || $.noop;

		this.hide_all_errors();
		this.show_waiting("<%=_T('Please wait validating Dialing Rule extensions sintax....')%>");

		var dialing_rule = this.__get_dialing_rule();

		var routing_group_id = dialing_rule.GroupId;
		var allegro_api = inConcert.app.getAllegroAPI();

		allegro_api.asteriskValidateDialingRule(dialing_rule, routing_group_id, function(success, response)
		{
			var validation_result = success ? response.result : response.err.result;
			// Valido aca los trunks del RG para no mandar todo al server...
			self.__validate_trunks_used(validation_result);

			var has_warnings = validation_result.ParseMessages.length > 0;
			self.__show_sintax_annotations(validation_result.ParseMessages);

			var msg;
			if (success) {
				if (has_warnings) {
					msg = "<%=_T('Sintax verification completed successfully (with warnings)')%>";
				}
				else {
					msg = "<%=_T('Sintax verification completed successfully')%>";
				}
				self.__show_status_message({
					msg: msg
				});
			}
			else {
				if (has_warnings) {
					msg = "<%=_T('Exists sintax error. Please review...')%>";
				}
				else {
					msg = "<%=_T('Server error, please contact system administrator')%>";
				}
				var $sintax = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_check_sintax");
				var err = {success: false, $ctrl: $sintax, item: null, message: msg};
				self.__show_ctrl_err(err);
			}
			self.frm.hide_waiting();
			callback(success, has_warnings);
		});
	};

	form_asterisk.prototype.__is_trunk_used_on_current_routing_group = function(trunkId, trunkType)
	{
		var routing_group = this.get_entity_instance();
		var trunks = routing_group.OutgoingRoutingGroupTrunks;
		if (!trunks) {
			return false;
		}
		for (var i = 0; i < trunks.length; i++) {
			var rgTrunk = trunks[i];
			if (rgTrunk.TrunkId === trunkId && rgTrunk.TrunkType === trunkType) {
				return true;
			}
		}
		return false;
	};

	form_asterisk.prototype.__validate_trunks_used = function(validator)
	{
		var dials = validator.Dial;
		if (dials) {
			for (var i = 0; i < dials.length; i++) {
				var dial = dials[i];
				var trunkType = dial.ip ? "IP" : "DIGITAL";
				if (!this.__is_trunk_used_on_current_routing_group(dial.trunkId, trunkType)) {
					validator.ParseMessages.push({type: -100, line: dial.line});
				}
			}
		}
	};

	form_asterisk.prototype.__validate_dialing_rule = function(callback)
	{
		callback = callback || $.noop;
		var result = {success: true};

		// Validacion Name/ID ----------------
		var $dialing_id = $("#adm_form_routing_group_dialing_rules_asterisk_popup_id");
		var dialing_id = $dialing_id.val();

		result = inConcert.Utils.validate(dialing_id, "required");
		if (!result.success) {
			result.message = "<%=_T('Dialign Rule Name is required.')%>";
			result.$ctrl = $dialing_id;
			callback(result);
			return;
		}

		result = inConcert.Utils.validate(dialing_id, "id");
		if (!result.success) {
			result.message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Dialign Rule Name.')%>";
			result.$ctrl = $dialing_id;
			callback(result);
			return;
		}

		if (this.is_new_dialign_rule()) {
			var dialing_rule = this.frm.find_dialing_rule(dialing_id);
			if (dialing_rule) {
				result.message = FORMAT("<%=_T('Dialign Rule {0} already exist.')%>", dialing_id);
				result.$ctrl = $dialing_id;
				result.success = false;
				callback(result);
				return;
			}
		}

		// Validacion Codigo  ----------------
		var code = this.code_editor.getValue();

		result = inConcert.Utils.validate(code, "required");
		if (!result.success) {
			result.message = "<%=_T('Rule Asterik Context is required.')%>";
			var $sintax = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_check_sintax");
			result.$ctrl = $sintax;
			callback(result);
			return;
		}

		this.__check_sintax_code_block(function(success, has_warnings)
		{
			if (!success) {
				result.success = false;
				result.not_show_err = true;
			}
			else {
				result.has_warnings = has_warnings;
			}
			callback(result);
		});
	};

	form_asterisk.prototype.__update_dialing_rule = function()
	{
		var self = this;

		var update = function()
		{
			var dialing_rule = self.__get_dialing_rule();
			self.frm.add_update_dialing_rule(dialing_rule, function() {
				self.hide();
			});
		};

		this.__validate_dialing_rule(function(validation_result)
		{
			if (!validation_result.success && !validation_result.not_show_err) {
				self.__show_ctrl_err(validation_result);
				return;
			}
			if (!validation_result.has_warnings) {
				update();
			}
			else {
				$.inconcertPopup.Show(
					"<%=_T('Confirmation')%>",
					"<%=_T('Do you want to continue despite the warnings?')%>",
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() === "yes") {
							update();
						}
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		});
	};

	form_asterisk.prototype.__set_events = function()
	{
		const self = this;

		const $create = $("#adm_form_routing_group_dialing_rules_asterisk_popup_create");
		$create
			.off("click")
			.on("click", function() {
				self.__update_dialing_rule();
			});

		const $close = $("#adm_form_routing_group_dialing_rules_asterisk_popup_close");
		$close
			.off("click")
			.on("click", function() {
				self.hide();
			});

		const $close_times = $("#adm_form_routing_group_dialing_rules_asterisk_popup_close_times");
		$close_times
			.off("click")
			.on("click", function() {
				self.hide();
			});

		const $code_block_check_sintax = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_check_sintax");
		$code_block_check_sintax
			.off("click")
			.on("click", function() {
				self.__check_sintax_code_block();
			});

		const $previous = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_previous");
		$previous
			.off("click")
			.on("click", function() {
				self.__goto_previous_error();
			});

		const $next = $("#adm_form_routing_group_dialing_rules_asterisk_popup_code_block_sintax_err_next");
		$next
			.off("click")
			.on("click", function() {
				self.__goto_next_error();
			});
	};

	form_asterisk.prototype.show = function()
	{
		this.__initialize();
		$("body").removeClass("popupDisplay").addClass("popupDisplay");
		$("#adm_form_routing_group_dialing_rules_asterisk_popup").show();
	};

	form_asterisk.prototype.hide = function()
	{
		$("#adm_form_routing_group_dialing_rules_asterisk_popup").hide();
		$("body").removeClass("popupDisplay");
		this.hide_all_errors();
		this.dispose();
	};

	form_asterisk.prototype.edit = function(current_dialing_rule)
	{
		this.is_new = current_dialing_rule === null;
		this.current_dialing_rule = current_dialing_rule;
		this.show();
	};

	//---------------------------------------------------------------------
	// form Routing Groups class
	function adm_form_routing_groups(params)
	{
		/* jshint validthis: true*/
		var success = adm_form_routing_groups.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Create Routing Group')%>";
		this.update_title = "<%=_T('Edit Routing Group {0}')%>";
		this.view_title = "<%=_T('View Routing Groups {0}')%>";
		this.tagline = "<%=_T('Tagline form Routing Groups!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/routing_groups/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/routing_groups/save/";
		this.get_uri        = "/inconcert/apps/administration/routing_groups/get/";

		this.form_paginator_availables_trunks = null;
		this.form_paginator_dialing_rules = null;

		this.availables_trunks = null;
		this.__new_grids();

		this.frm_asterisk = new form_asterisk({
			parent: this
		});

		return true;
	}

	inConcert.Utils.inherit(adm_form_routing_groups, inConcert.conf_form);

	if (typeof inConcert.adm_form_routing_groups !== "function") {
		inConcert.adm_form_routing_groups = adm_form_routing_groups;
	}

	adm_form_routing_groups.prototype.get = function()
	{
		let routing_group = {
			__IsNew                   : this.is_creating(),
			Id                        : $("#adm_form_routing_group_name").val(),
			Description               : $("#adm_form_routing_group_desc").val(),
			OutgoingRoutingGroupTrunks: [],
			OutgoingDialingRules      : [],
			DisplayCapacity           : $("#adm_form_routing_group_availables_trunks_capacity_assigned").val()
		};

		const trunks = this.form_paginator_availables_trunks.get_items();
		trunks.forEach(trunk => {
			trunk.CapacityAssigned = trunk.RoutingGroupCapacity;
			if (trunk.CapacityAssigned > 0) {
				trunk.DisplayCapacity = trunk.CapacityAssigned;
				routing_group.OutgoingRoutingGroupTrunks.push(trunk);
			}
		});

		const dialing_rules = this.form_paginator_dialing_rules.get_items();
		routing_group.OutgoingDialingRules = dialing_rules || [];

		return routing_group;
	};

	adm_form_routing_groups.prototype.set = function(success, routing_group)
	{
		if (!success) {
			return;
		}
		$("#adm_form_routing_group_name").val(routing_group.Id);
		$("#adm_form_routing_group_desc").val(routing_group.Description);

		this.form_paginator_availables_trunks.initialize();
		this.form_paginator_dialing_rules.initialize();
	};

	adm_form_routing_groups.prototype.empty_controls = function()
	{
		$("#adm_form_routing_group_name").val("").focus();
		$("#adm_form_routing_group_desc").val("");
	};

	adm_form_routing_groups.prototype.save_execute = function( after_save, entity, hash_id )
	{
		hash_id = null;
		adm_form_routing_groups.uber.save_execute.call( this, after_save, entity, hash_id ); //invokes the base class method
	};

	adm_form_routing_groups.prototype.err_save_messages = function(response_err)
	{
		var is_custom_message = false;

		if ( typeof response_err === "object" ) {
			response_err.result = response_err.result || response_err.type || "";
			if (response_err.err && response_err.err.result === "ALREADY_EXIST") {
				response_err = "<%=_T('Routing Group Name is already in use.')%>";
				$("#adm_form_routing_group_name").focus();
				is_custom_message = true;
			}
			else {
				if (response_err.result === "ALLEGRO_INTEGRATION_ERROR") {
					response_err = "<%=_T('Error updating information on Allegro integration...')%>";
					if (this.is_creating()) {
						$("#adm_form_routing_group_name").focus();
					}
					else {
						$("#adm_form_routing_group_desc").focus();
					}
					is_custom_message = true;
				}
			}
		}

		adm_form_routing_groups.uber.err_save_messages.call( this, response_err, is_custom_message ); //invokes the base class method
	};

	adm_form_routing_groups.prototype.get_validation_def = function()
	{
		var self = this;

		var allegro_api = inConcert.app.getAllegroAPI();
		var allegro_uri = allegro_api.getBaseURI();

		var definitions = [
			{
				control_id: "adm_form_routing_group_name",
				messages: {
					required: "<%=_T('Please enter the Routing Group Name.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the Routing Group Name.')%>"
				}
			},
			{
				control_id: "adm_form_routing_group_desc",
				messages: {
					required: "<%=_T('Please enter the Routing Group description.')%>"
				}
			},
			{
				control_id: "adm_form_routing_group_availables_trunks_list",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							var response = {
								success: true
							};
							var success = self.form_paginator_availables_trunks.start_validation().success;
							if (!success) {
								response.success = false;
								response.message = self.form_paginator_availables_trunks.get_validation_message_err("required");
							}
							return response;
						}
					}
				}
			},
			{
				control_id: "adm_form_routing_group_availables_trunks_capacity_assigned",
				messages: {
					range: {
						type   : "range",
						between: {from: 1, to: 10000000},
						must_validate_disabled: true,
						message: "<%=_T('It should include trunks with at least one capacity assigned. Allowed values {0} to {1}.')%>"
					},
					remote: {
						type: "remote",
						message: "<%=_T('You cannot assign less than currently assigned on Campaign.')%>",
						timeout_ms: allegro_api.getRequestTimeout(),
						uri: function(_)
						{
							var routing_group = self.get_entity_instance();
							var routing_group_id = routing_group ? routing_group.Id : $("#adm_form_routing_group_name").val();
							/*
							 * sample URI:
							 *   http://192.168.4.11:8082/inconcert/apps/allegro/routinggroup/check_max_assigned/ROUTING_GROUP_ID
							 */
							var uri = FORMAT("{0}/routinggroup/check_max_assigned/{1}", allegro_uri, routing_group_id);
							return uri;
						},
						success: function(response, callback)
						{
							var success = true;
							var result = response.response;
							var current_capacity_assigned = parseInt($("#adm_form_routing_group_availables_trunks_capacity_assigned").val(), 10);
							if (result.capAssigned > current_capacity_assigned) {
								success = false;
							}
							callback(success);
						}
					}
				}
			}
		];

		return definitions;
	};

	adm_form_routing_groups.prototype.save = function(callback)
	{
		adm_form_routing_groups.uber.save.call( this, callback ); //invokes the base class method
	};

	adm_form_routing_groups.prototype.ready = function()
	{
		adm_form_routing_groups.uber.ready.call( this ); //invokes the base class method

		var $name = $("#adm_form_routing_group_name");
		if (!this.is_new) {
			if ( inConcert.app.has_allegro_integration() ) {
				this.readonly_control($name, true);
				$("#adm_form_routing_group_desc").focus();
				return;
			}
		}
		$name.focus();
	};

	adm_form_routing_groups.prototype.ready_for_create = function()
	{
		this.form_paginator_availables_trunks.initialize();
		this.form_paginator_dialing_rules.initialize();
		adm_form_routing_groups.uber.ready_for_create.call( this ); //invokes the base class method
		this.form_paginator_availables_trunks.set_events();
		this.form_paginator_dialing_rules.set_events();
	};

	adm_form_routing_groups.prototype.ready_for_update = function()
	{
		this.form_paginator_availables_trunks.initialize();
		this.form_paginator_dialing_rules.initialize();
		adm_form_routing_groups.uber.ready_for_update.call( this ); //invokes the base class method
		this.form_paginator_availables_trunks.set_events();
		this.form_paginator_dialing_rules.set_events();
	};

	adm_form_routing_groups.prototype.readonly = function()
	{
		adm_form_routing_groups.uber.readonly.call(this); //invokes the base class method
		$("#adm_form_routing_group_dialing_rules_new").hide();
		this.form_paginator_availables_trunks.set_events();
		this.form_paginator_dialing_rules.set_events();
	};

	adm_form_routing_groups.prototype.load_data = function( callback_complete_requests, routing_group_id )
	{
		var self = this;

		var allegro_api = inConcert.app.getAllegroAPI();
		this.requests = [];
		this.requests.push({
			request_function: function(success) {
				return allegro_api.loadAvailablesTrunksFromRoutingGroup( routing_group_id, success );
			},
			success: function(trunks) {
				self.__set_availables_trunks(trunks);
			}
		});

		adm_form_routing_groups.uber.load_data.call( this, callback_complete_requests ); //invokes the base class method
	};

	adm_form_routing_groups.prototype.__set_availables_trunks = function(trunks)
	{
		this.availables_trunks = trunks;
	};

	adm_form_routing_groups.prototype.__new_grids = function()
	{
		var self = this;

		this.form_paginator_availables_trunks = new inConcert.conf_form_paginator({
			paginator_name: "ADMINISTRATION_AVAILABLES_TRUNKS",
			owner: this,
			frm: this,
			row_id: "TrunkId",
			display_name: "TrunkId",
			rows_per_page: 500,
			show_at_least: 499,
			sortable: true,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Availables Trunks ... ')%>",
			ids: {
				grid: "adm_form_routing_group_availables_trunks_list",
				paginator: "adm_form_routing_group_availables_trunks_paginator",
				not_results_found: "adm_form_routing_group_availables_trunks_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Availables Trunks found.')%>"
			},
			columns: [
				{id: "_number"             , name: "", type: "counter"   },
				{id: "TrunkId"             , name: "<%=_T('Name')%>"     },
				{id: "TrunkType"           , name: "<%=_T('Type')%>"     },
				{id: "CapacityAvailable"   , name: "<%=_T('Available')%>"},
				{id: "RoutingGroupCapacity", name: "<%=_T('Assigned')%>",
					type: "cell_input",
					get_value: function(item) {
						return self.get_grid_value("trunks", item, "RoutingGroupCapacity");
					}
				}
			],
			callbacks: {
				load_items: function(page, callback) {
					//console.log("3 - grid load_items -> form_paginator_destinations");
					self.__load_availables_trunks(page, callback);
				},
				on_loaded_items: function(callback) {
					self.__on_loaded_availables_trunks_grid(callback);
				},
				custom_validation_cell_value: function(info, callback) {
					self.__validate_trunks_cell_value(info, callback);
				}
			},
			validate: {
				required_msg: "<%=_T('It should include trunks with at least one capacity assigned.')%>",
				required: true
			}
		});

		this.form_paginator_dialing_rules = new inConcert.conf_form_paginator({
			paginator_name: "ADMINISTRATION_DIALING_RULES",
			owner: this,
			frm: this,
			row_id: "Id",
			display_name: "Id",
			rows_per_page: 500,
			show_at_least: 499,
			sortable: true,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Dialing Rules ...')%>",
			ids: {
				grid: "adm_form_routing_group_dialing_rules_list",
				paginator: "adm_form_routing_group_dialing_rules_paginator",
				not_results_found: "adm_form_routing_group_dialing_rules_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Dialing Rules found.')%>",
				can_remove : "<%=_T('Do you want to delete Dialing Rule: {0} ?')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "Id"         , name: "<%=_T('Name')%>"  },
				{id: "Description", name: "<%=_T('Type')%>"  },
				{id: "_action", name: "",
					system_action: { edit: true, erase: true }
				}
			],
			callbacks: {
				edit: function(dialing_rule) {
					self.frm_asterisk.edit(dialing_rule);
				},
				remove: function(row, callback) {
					self.remove(row, callback);
				},
				load_items: function(page, callback) {
					//console.log("3 - grid load_items -> form_paginator_destinations");
					self.__load_dialing_rules(page, callback);
				}
			}
		});
	};

	adm_form_routing_groups.prototype.get_grid_value = function(grid_id, item, field_id)
	{
		if (grid_id !== "trunks") {
			throw("adm_form_routing_groups.prototype.get_grid_value. Not implemented grid " + grid_id);
		}
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "RoutingGroupCapacity") {
			value.value = item.RoutingGroupCapacity || 0;
			value.search_value = value.value;
		}

		return value;
	};

	adm_form_routing_groups.prototype.__validate_trunks_cell_value = function(info, callback)
	{
		if (info.column_id !== "RoutingGroupCapacity") {
			callback(true);
			return;
		}

		var new_value = info.value;
		var old_value = info.old_value;

		if (new_value.indexOf("-") !== -1 || new_value.indexOf(".") !== -1) {
			callback(false, "rollback");
			return;
		}

		new_value = parseInt(new_value, 10);
		old_value = parseInt(old_value, 10);
		if (new_value === old_value) {
			callback(false, "rollback");
			return;
		}
		if (isNaN(new_value)) {
			callback(false, "rollback");
			return;
		}
		if (new_value < 0) {
			callback(false, "rollback");
			return;
		}

		var row_data = info.row_data;
		var available_capacity = parseInt(row_data.CapacityAvailable, 10);
		old_value = isNaN(old_value) ? 0 : old_value;

		var max_capacity = old_value + available_capacity;

		if (new_value > max_capacity) {
			var message = "<%=_T('You have exceeded maximum capacity available.')%>";
			this.show_status_message(message, "Error", 5000);
			callback(false, "rollback");
			return;
		}

		row_data.CapacityAvailable = available_capacity = max_capacity - new_value;
		row_data.RoutingGroupCapacity = new_value;
		this.form_paginator_availables_trunks.update_row(row_data);
		this.__update_total_capacity_assigned();

		callback(true);
	};

	adm_form_routing_groups.prototype.__load_availables_trunks = function(_, callback)
	{
		var availables_trunks = inConcert.Utils.cloneArray(this.availables_trunks || []);
		callback(true, availables_trunks);
	};

	adm_form_routing_groups.prototype.__on_loaded_availables_trunks_grid = function(callback)
	{
		callback = callback || $.noop;
		this.__update_total_capacity_assigned();
		callback();
	};

	adm_form_routing_groups.prototype.__update_total_capacity_assigned = function()
	{
		var total_capacity_assigned = 0;
		var trunks = this.form_paginator_availables_trunks.get_items();
		$.each(trunks, function(_, trunk) {
			total_capacity_assigned = total_capacity_assigned + (parseInt(trunk.RoutingGroupCapacity, 10) || 0);
		});
		$("#adm_form_routing_group_availables_trunks_capacity_assigned").val(total_capacity_assigned);
	};

	adm_form_routing_groups.prototype.__load_dialing_rules = function(_, callback)
	{
		var dialing_rules = [];
		if (!this.is_creating()) {
			var entity = this.get_entity_instance();
			if (entity !== null && entity !== undefined ) {
				dialing_rules = inConcert.Utils.cloneArray(entity.OutgoingDialingRules || []);
			}
		}
		callback(true, dialing_rules);
	};

	adm_form_routing_groups.prototype.set_events = function()
	{
		adm_form_routing_groups.uber.set_events.call(this); //invokes the base class method
		var self = this;

		$("#adm_form_routing_group_dialing_rules_new")
			.off("click")
			.on("click", function() {
				self.frm_asterisk.edit(null);
			});
	};

	adm_form_routing_groups.prototype.add_update_dialing_rule = function(dialing_rule, callback)
	{
		callback = callback || $.noop;
		this.form_paginator_dialing_rules.add_or_update_row(dialing_rule);
		var message;
		if (this.frm_asterisk.is_new_dialign_rule()) {
			message = "<%=_T('Added Dialing Rule.')%>";
		}
		else {
			message = "<%=_T('Updated Dialing Rule.')%>";
		}
		this.show_status_message(message, "Info", 5000);
		callback();
	};

	adm_form_routing_groups.prototype.find_dialing_rule = function(dialing_id)
	{
		var dialing_rule = this.form_paginator_dialing_rules.find_row_data(dialing_id);
		return dialing_rule;
	};

	adm_form_routing_groups.prototype.remove = function(dialing_rule, callback)
	{
		callback = callback || $.noop;
		const self = this;

		if (!dialing_rule) {
			throw("adm_form_routing_groups.prototype.remove. Undefined dialing_rule.");
		}

		var end = function(success) {
			let msg;
			if (success) {
				msg = FORMAT("<%=_T('Dialing Rule {0} was successfully deleted.')%>", dialing_rule.Id);
				self.show_status_message(msg, "Info", 5000);
				self.hide_waiting();
			}
			else {
				msg = FORMAT("<%=_T('Error deleting Dialing Rule {0}.')%>", dialing_rule.Id);
				self.show_status_message(msg, "Error", 5000);
				self.hide_waiting();
			}
			callback(success);
		};

		this.show_waiting(FORMAT("<%=_T('Deleting Dialing Rule {0}.')%>", dialing_rule.Id));

		const allegro_api = inConcert.app.getAllegroAPI();
		const routing_group = this.get_entity_instance();
		const routing_group_id = routing_group ? routing_group.Id : $("#adm_form_routing_group_name").val();

		allegro_api.asteriskDeleteDialingRule(routing_group_id, dialing_rule.Id,
			function(success) {
				end(success);
			}
		);
	};

}());
