/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";
	const format = inConcert.Utils.format;
	const base_length_default=10;

	/**
	Returns an object to be used in form validations.
	Makes sure the value of a field is an optional number.
	*/
	function make_optional_number_validation (control_id) {
		return {
			control_id: control_id,
			messages: {
				custom: {
					type: "custom",
					valid: function() {
						const response = { success: true };
						const number_width = $("#" + control_id).val();
						if( number_width.length === 0 ) {
							return response;
						}
						const is_digits = inConcert.Utils.validate( number_width, "digits" );
						if ( !is_digits.success ) {
							response.success = false;
							response.message = "<%=_T('Only numbers are allowed.')%>";
							return response;
						}
						if (number_width.length > base_length_default){
							response.success = false;
							response.message = format("<%=_T('The Base number cannot exceed {0} characters.')%>", base_length_default);
							return response;
						}
						// ok
						return response;
					}
				}
			}
		};
	}

	/**
	* Returns an object used in form validations, for prefixes or suffixes.
	*/
	function make_prefix_suffix_validation (control_id, fail_message) {
		return {
			control_id: control_id,
			optional: true,
			messages: {
				custom: {
					type: "custom",
					valid: function($field) {
						var value = $field.val();
						if(value.match(/^[\-a-zA-Z 0-9_.@#]+$/)) {
							return { success: true };
						}
						else {
							return { success: false, message: fail_message };
						}
					}
				}
			}
		};
	}

	function adm_form_tickets (params)
	{
		/* jshint validthis: true*/
		var success = adm_form_tickets.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.new_title = "<%=_T('Create ticket issuer')%>";
		this.update_title = "<%=_T('Edit Ticket Issuer {0}')%>";
		this.view_title = "<%=_T('View ticket {0}')%>";
		this.tagline = "<%=_T('tagline')%>";
		this.html_file_name = "/inconcert/apps/administration/pages/frm_ticket_content.<%=_L%>.html#grunt-cache-bust";
		this.save_uri       = {
			new   : "/inconcert/apps/administration/tickets/new",
			update: "/inconcert/apps/administration/tickets/update"
		};
		// Inject a method in the configuration (adm_config.js), so we don't polute the config with code
		params.action.view.uri.erase = this.view_uri_erase.bind(this);
	}

	inConcert.Utils.inherit(adm_form_tickets, inConcert.conf_form);

	/**
	Overrides action.view.uri.erase. We don't do batch deletes and we expect the id in the url
	*/
	adm_form_tickets.prototype.view_uri_erase = function(row_id, _, callback) {
		var url = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/tickets/delete/" + row_id;

		var request_handler = inConcert.invoke({
			url         : url,
			type        : "POST",
			dataType    : "json",
			success: function(response) {
				if(response.status) {
					return callback(response.status);
				}
				if(response.code === "E_EMITTER_IN_USE") {
					return callback(response.status, "E_EMITTER_IN_USE");
				}
				return callback(response.status); // generic error, improve
			},
			error: function(_, text_status) {
				var err;
				try {
					err = JSON.parse(text_status).result;
				}
				catch(e) {
					err = text_status;
				}
				callback(false, err);
			}
		});

		return request_handler;
	};

	/**
	Returns the data to be saved.
	*/
	adm_form_tickets.prototype.get = function()
	{
		var emitter = {
			description: $("#adm_form_ticket_desc").val(),
			number_type: $("#adm_form_ticket_number_type").val(),
			number_width: $("#adm_form_ticket_number_width").val(),
			base_number: $("#adm_form_ticket_base_number").val(),
			prefix: $("#adm_form_ticket_prefix").val(),
			suffix: $("#adm_form_ticket_suffix").val()
		};

		if(this.$current_row) {
			emitter.id = this.$current_row.attr("row_id");
		}

		// remove optional values
		if (emitter.prefix.length === 0) { delete emitter.prefix; }
		if (emitter.suffix.length === 0) { delete emitter.suffix; }

		// coerce optional values to numbers, or remove them
		if (emitter.number_width.length === 0) {
			delete emitter.number_width;
		}
		else {
			emitter.number_width = + emitter.number_width;
		}
		if (emitter.base_number.length === 0) {
			delete emitter.base_number;
		}
		else {
			emitter.base_number = + emitter.base_number;
		}

		return emitter;
	};

	/**
	Returns the uri used to retrieve an entity by id.
	*/
	adm_form_tickets.prototype.uri_get_entity = function()
	{
		if (this.is_new) {
			return "";
		}
		var emitter_id = this.$current_row.attr("row_id");
		var uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/tickets/get/" + emitter_id;
		return uri;
	};

	/**
	Called after the request to retrieve an entity (a ticket) is done.
	success is a boolean flag and ticket is an object with the ticket data
	*/
	adm_form_tickets.prototype.set = function (success, ticket)
	{
		if (!success) {
			return;
		}
		$("#adm_form_ticket_desc").val(ticket.description);
		$("#adm_form_ticket_number_type").val(ticket.number_type);
		if(ticket.number_width !== 0) {
			$("#adm_form_ticket_number_width").val(ticket.number_width);
		}
		$("#adm_form_ticket_base_number").val(ticket.base_number);
		$("#adm_form_ticket_prefix").val(ticket.prefix);
		$("#adm_form_ticket_suffix").val(ticket.suffix);

		this.__preview();
	};

	/**
	Returns the definition of the validations to be applied.
	*/
	adm_form_tickets.prototype.get_validation_def = function()
	{
		var self = this;
		
		return [
			{
				control_id: "adm_form_ticket_desc",
				messages: {
					required: "<%=_T('Please enter a description.')%>",
					alpha_no_initial_dots: "<%=_T('Only letters, numbers, dashes and dots are allowed in the description (you cannot start with multiple dots).')%>",
					remote: {
						uri: function(description) {
							var emitter_id = self.$current_row ? self.$current_row.attr("row_id") : "";
							return inConcert.app.getBaseDomain() + "/inconcert/apps/administration/tickets/is_available/" +
								encodeURIComponent(description) + "/" + encodeURIComponent(emitter_id);
						},
						type: "remote",
						message: "<%=_T('Emitter description is duplicated. Please enter a different one.')%>"
					}
				}
			},
			make_optional_number_validation("adm_form_ticket_base_number"),
			{
				control_id: "adm_form_ticket_number_type",
				messages: {
					custom: {
						type: "custom",
						valid: function() {
							var index = $("#adm_form_ticket_number_type").prop("selectedIndex");
							if(index === 0) {
								return {
									success: false,
									message: "<%=_T('Please select sequence type.')%>"
								};
							}
							return { success: true };
						}
					}
				}
			},
			//make_optional_number_validation("adm_form_ticket_number_width"),
			{
				control_id: "adm_form_ticket_number_width",
				optional: true,
				messages: {
					custom: {
						type: "custom",
						valid: function($field) {
							const value = $field.val();
							const is_digits = inConcert.Utils.validate( value, "digits" );
							if ( is_digits.success && (+ value <= 99) ) {
								return { success: true };
							}
							return { success: false, message: "<%=_T('Only a number from 0 to 99 is allowed.')%>" };
						}
					}
				}
			},
			make_prefix_suffix_validation("adm_form_ticket_prefix", "<%=_T('Only letters, numbers, dashes, dots and hashes are allowed in the prefix.')%>"),
			make_prefix_suffix_validation("adm_form_ticket_suffix", "<%=_T('Only letters, numbers, dashes, dots and hashes are allowed in the suffix.')%>")
		];
	};

	/*
	*/
	adm_form_tickets.prototype.save_execute = function(after_save, entity) {
		const self = this;

		after_save = "SAVE_AND_REFRESH_VIEW";
		const callback = function(success, response) {
			self.dashboard.hide_waiting();
			if (success) {
				if (response.status) {
					entity.id = response.emitter.id;
					self.on_saved(true, entity, after_save);
				}
				else {
					self.on_saved(false, entity, after_save, response.msg);
				}
				return;
			}
			// there was an error, but not from the API itself
			const err = response;
			self.on_saved(success, entity, after_save, err);
		};

		adm_form_tickets.uber.save_execute.call(
			this,
			undefined,
			entity,
			undefined,
			callback
		);
	};

	/**
	Gets a preview based on the currently entered data.
	*/
	adm_form_tickets.prototype.__preview = function ()
	{
		var self = this;
		if($("#adm_form_ticket_number_type").prop("selectedIndex") === 0) {
			// nothing to preview
			return;
		}

		const preview_data = self.get();
		const base_length = $("#adm_form_ticket_base_number").val().length;
		if (base_length > base_length_default) {
			// cannot preview
			$("#adm_form_ticket_preview").val(format("<%=_T('The Base number cannot exceed {0} characters.')%>", base_length_default));
			return;
		}

		if(preview_data.number_width > 99) {
			// cannot preview
			return;
		}

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/tickets/preview",
			type: "POST",
			data: JSON.stringify(preview_data),
			contentType: "application/json",
			success: function(preview/*, textStatus*/)
			{
				$("#adm_form_ticket_preview").val(preview.minimum);
				$("#adm_form_ticket_preview_max").val(preview.maximum);
			},
			error: function(/*xhr, textStatus, errorThrown*/)
			{
				// this is a preview, I don't care about errors
			}
		});
	};

	/**
	The form is ready to be shown. Fill combos, set focus, do whatever initialization is needed.
	*/
	adm_form_tickets.prototype.ready = function()
	{
		adm_form_tickets.uber.ready.call(this);

		var combo = $("#adm_form_ticket_number_type");
		combo.html("");

		var values = { "autoincrement": "<%=_T('Autoincrement')%>", "date": "<%=_T('Date with autoincrement')%>" };
		this.fill_combo(combo, true, values);

		$("#adm_form_ticket_desc").focus();

		var bound_preview = this.__preview.bind(this);
		this.dashboard.$get_dom_element_by_id( "form_data_container input"  ).keyup(bound_preview);
		this.dashboard.$get_dom_element_by_id( "form_data_container select" ).change(bound_preview);
	};

	/**
	Cleans up the form. Called, for instance, when clicking "Create & Continue" button.
	*/
	adm_form_tickets.prototype.empty_controls = function()
	{
		this.dashboard.$get_dom_element_by_id( "form_data_container input"  ).val("");
		this.dashboard.$get_dom_element_by_id( "form_data_container select" ).val("");
		$("#adm_form_ticket_desc").focus();
	};

	if (typeof inConcert.adm_form_tickets !== "function") {
		inConcert.adm_form_tickets = adm_form_tickets;
	}
}());
