/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	//---------------------------------------------------------------------
	// form users allegro integration class
	//
	function adm_form_users_allegro(params)
	{
		var success = adm_form_users_allegro.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
	}

	inConcert.Utils.inherit(adm_form_users_allegro, inConcert.adm_form_users);

	if (typeof inConcert.adm_form_users_allegro !== "function") {
		inConcert.adm_form_users_allegro = adm_form_users_allegro;
	}

	adm_form_users_allegro.prototype.get = function()
	{
		var user = adm_form_users_allegro.uber.get.call(this); //invokes the base class method
		user.AllegroIntegration = this.__get_allegro_integration();
		return user;
	};

	adm_form_users_allegro.prototype.set = function(success, user)
	{
		if (!success) {
			return;
		}
		adm_form_users_allegro.uber.set.call(this, success, user); //invokes the base class method
		this.__set_allegro_integration( user.AllegroIntegration );
	};

	// El initial state se debe asignar en undefined para allegro. A este estado se resuelve del lado del server y siempre es BREAK
	//
	adm_form_users_allegro.prototype.__get_initial_state = function() { return; };

	adm_form_users_allegro.prototype.__set_initial_state = function(/*initial_state*/) { $("#adm_form_user_initial_state").val(""); };

	adm_form_users_allegro.prototype.__get_allegro_integration = function()
	{
		var self = this;
		// retorna informacion del peer
		var get_peer = function()
		{
			var peer_info = {};
			var selected_peer_info = self.combo_get_selected_item_data( $("#adm_form_user_allegro_phone") );
			if (selected_peer_info) {
				peer_info.service    = selected_peer_info.service;    // "TELEPHONY_192_168_24_236";
				peer_info.technology = selected_peer_info.technology; // "ASTERISK,IAX2";
				peer_info.peer       = selected_peer_info.peer;       // "2";
			}
			return peer_info;
		};
		return {
			peer: get_peer()
		};
	};

	// TELEPHONY_192_168_24_236, 2, ASTERISK,IAX2
	adm_form_users_allegro.prototype.__make_peer_id = function(_, peer) { return peer.service + ", " + peer.peer + ", " + peer.technology; };

	// TELEPHONY_192_168_24_236, 2, ASTERISK,IAX2, AMI (OPTIONAL)
	adm_form_users_allegro.prototype.__make_peer_name = function(_, peer) {
		let peer_name = this.__make_peer_id(_, peer);
		if (peer.AMIMonitored) {
			peer_name = peer_name + ", AMI";
		}
		return peer_name;
	};

	adm_form_users_allegro.prototype.__set_allegro_integration = function(allegro_integration)
	{
		if (!allegro_integration) {
			return;
		}
		// informacion sobre el peer
		if (allegro_integration.peer) {
			var peer_id = this.__make_peer_id( undefined, allegro_integration.peer );
			$("#adm_form_user_allegro_phone").val( peer_id );
		}
	};

	adm_form_users_allegro.prototype.fill_peers = function(peers)
	{
		const self = this;
		const $phone = $("#adm_form_user_allegro_phone");
		this.fill_combo(
			$phone,
			true, /*blank first item*/
			peers,
			function(_, peer) {return self.__make_peer_id(_, peer);},   /*funcion attr id*/
			function(_, peer) {return self.__make_peer_name(_, peer);}, /*funcion attr name*/
		);
	};

	adm_form_users_allegro.prototype.load_specific_data = function(editing_user_id)
	{
		var self = this;
		var allegro_api = inConcert.app.getAllegroAPI();
		// Requests unicamente SI tenemos integracion con Allegro
		this.requests.push({
			request_function: function(success) {
				return allegro_api.loadAvailablesPeers( editing_user_id, success );
			},
			success: function(peers) {
				self.fill_peers(peers);
			}
		});
	};

	/*
		Aqui se definen los campos que no se validaran para allegro
	*/
	adm_form_users_allegro.prototype.can_validate_field = function(field_id)
	{
		var fields = {
			adm_form_user_id           : false, // validacion del largo
			adm_form_user_pwd          : false, // validacion del largo
			adm_form_user_initial_state: false  // validacion de requerido
		};
		if (fields.hasOwnProperty(field_id)) {
			return fields[field_id];
		}
		// si el campo a verificar si se valida, no existe dentro de la definicion, entonces se toma como que se debe de validar igual (como que fuse un true)
		return true;
	};

	/*
		Aqui invoco a la clase base para obtener la definicion de validacion, y agrego validacion correspondiente a Allegro
	*/
	adm_form_users_allegro.prototype.get_validation_def = function()
	{
		var definitions = adm_form_users_allegro.uber.get_validation_def.call(this); //invokes the base class method
		// por ahora no realizo validacion alguna, que sea exclusiva para allegro, pero lo dejo por si hay que hacer algo, mas que nada para tenerlo como referencia
		return definitions;
	};

	adm_form_users_allegro.prototype.show_interaction_app_panels = function()
	{
		adm_form_users_allegro.uber.show_interaction_app_panels.call(this); //invokes the base class method
		// allegro integration
		this.__show_hide_allegro_integration();
	};

	adm_form_users_allegro.prototype.show_role_panels = function()
	{
		adm_form_users_allegro.uber.show_role_panels.call( this ); //invokes the base class method
		// allegro integration
		this.__show_hide_allegro_integration();
	};

	adm_form_users_allegro.prototype.ready = function()
	{
		adm_form_users_allegro.uber.ready.call( this ); //invokes the base class method
		// muestra / oculta los controles de allegro
		this.__show_hide_allegro_integration();
	};

	adm_form_users_allegro.prototype.__show_hide_allegro_integration = function()
	{
		$("#adm_form_user_allegro_information").show();
		$("#adm_form_user_initial_state_section").hide();
	};

	adm_form_users_allegro.prototype.__get_server_error_description = function(err_type, err)
	{
		var response = { is_custom_message: true, message: undefined };
		if (err_type === "ALLEGRO_INTEGRATION_ERROR") {
			response.message = "<%=_T('Error updating information on Allegro integration...')%>";
			return response;
		}
		response = adm_form_users_allegro.uber.__get_server_error_description.call(this, err_type, err); //invokes the base class method
		return response;
	};
}());
