/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, -W014, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const FORMAT = inConcert.Utils.format;

	const can_exec_importation = function() {
		return inConcert.permissions.to_action("administration", "import_users")
		 	&& inConcert.permissions.to_action("administration", "saveuser");
	};

	//---------------------------------------------------------------------
	// form users importations class
	//
	function adm_form_users_importations (params)
	{
		this.owner = params.owner;
		this.m_allowed_csv_columns = null;
		this.m_csv_header = null;
		this.m_current_upload_info = null;
		this.m_upload_instance = null;
		this.m_importations_paginator = null;
		this.m_importations_failed_paginator = null;
		this.m_total_importations = 0;
	}

	if (typeof inConcert.adm_form_users_importations !== "function") {
		inConcert.adm_form_users_importations = adm_form_users_importations;
	}

	adm_form_users_importations.prototype.__initialize = function()
	{
		this.m_total_importations = 0;
		this.__load_allowed_csv_columns();
		this.__initialize_upload_ctrl();
		this.__load_importations_grid();
		// para que la primera vez muestre vacio.
		this.__show_detail_importation({});
	};

	adm_form_users_importations.prototype.__load_allowed_csv_columns = function()
	{
		var self = this;
		if (this.m_allowed_csv_columns) {
			this.__make_allowed_columns();
			return;
		}
		var importation_api = inConcert.app.getImportationAPI();
		importation_api.importGetAllowedHeaderInfo({},
			function(success, response)
			{
				if (success) {
					self.m_allowed_csv_columns = response;
					self.__make_allowed_columns();
					return;
				}
				self.show_status_message("<%=_T('Unable to retrieve information from CSV allowed columns ...')%>");
			}
		);
	};

	adm_form_users_importations.prototype.__load_importations_grid = function()
	{
		var self = this;
		this.m_importations_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.owner,
			row_id: "Id",
			display_name: "Name",
			auto_initialize: true,
			rows_per_page: 8,
			show_at_least: 7,
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Importations ...')%>",
			$custom_waiting_container: $("#adm_form_users_importation_popup_waiting"),
			order_info: {
				options: {
					order: "desc"
				},
				sort_value_fn: function(row) {
					return row.StartDate;
				}
			},
			ids: {
				paginator: "adm_form_users_importation_paginator",
				grid: "adm_form_users_importation_list",
				not_results_found: "adm_form_users_importation_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Importations found.')%>",
				can_remove       : "<%=_T('Do you want to delete Importation: {0} ?')%>"
			},
			columns: [
				{id: "_number", name: "", type: "counter"},
				{id: "Id"     , name: "<%=_T('Id')%>  "  },
				{id: "Name"   , name: "<%=_T('Name')%>"  },
				{id: "Status" , name: "<%=_T('Status')%>",
					type: "icon",
					image_type: "image_with_text",
					get_value: function(item) {
						return self.__get_grid_value(item, "Status");
					}
				},
				{id: "StartDate"   , name: "<%=_T('Start Date')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "StartDate");
					}
				},
				{id: "EndDate"   , name: "<%=_T('End Date')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "EndDate");
					}
				},
				{id: "User"              , name: "<%=_T('Owner')%>"   },
				{id: "NumberImportedRows", name: "<%=_T('Imported')%>"},
				{id: "NumberFailedRows"  , name: "<%=_T('Failed')%>"  },
				{id: "_action", name: "",
					system_action: {erase: true},
					custom_action: {
						detail: {
							name: "<%=_T('Show Detail')%>",
							execute: function($row /*, $owner*/) {
								var importation = $row.data("row_data");
								self.__show_detail_importation(importation);
							},
							class_name: "buttons-icons-view"
						}
					},
					is_enabled: function(action_id, importation) {
						if (action_id === "DELETE") {
							if (importation && importation.Status !== "RUNNING") {
								return 1;
							}
						}
						else if (action_id === "detail") {
							if (!importation || importation.Status !== "FINISHED" || importation.NumberFailedRows < 1) {
								return 0;
							}
							return 1;
						}
						return 0;
					}
				}
			],
			callbacks: {
				remove: function(row, callback) {
					self.__remove_importation(row, callback);
				},
				on_removed_row: function() {
					// dejo vacia la grilla de usuarios fallidos para la importacion que se ha eliminado
					self.__show_detail_importation({});
				},
				load_items: function(_, callback) {
					self.__load_items(callback);
				}
			}
		});
	};

	adm_form_users_importations.prototype.__initialize_upload_ctrl = function()
	{
		var self = this;

		this.m_upload_instance = new inConcert.RepositoryUpload({
			auto_initialize: true,
			repository_url: "/inconcert/apps/administration/imports",
			container_id: "adm_form_users_importation_file_panel_upload",
			must_validate_max_size: false,
			availables_extensions: {
				"csv": true
			},
			show_status_message_fn: function(message, type, time_ms) {
				var $custom_message_container = $("#adm_form_users_importation_file_panel_popup_message");
				self.show_status_message(message, type, time_ms, undefined, undefined, $custom_message_container);
			},
			on_uploading_fn: function(success, data) {
				self.on_uploading_fn(success, data);
			},
			on_uploaded_fn: function(success, data, callback) {
				self.on_uploaded(success, data, callback);
			},
			can_upload_file_fn: function(callback) {
				callback = callback || $.noop;
				self.can_upload_file(callback);
			}
		});
		this.m_current_upload_info = null;
	};

	adm_form_users_importations.prototype.__get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : "",
			class_name    : ""
		};

		var item_value = item[field_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "StartDate" || field_id === "EndDate") {
			if (value.original_value.length > 0) {
				let date_object = inConcert.Utils.getDateFromString(value.original_value);
				if (date_object instanceof Date) {
					value.value = inConcert.Utils.getStringFromDate(date_object, false, "full", "ymd", "-");
					value.search_value = value.value;
				}
			}
		}
		else if (field_id === "Status") {
			var statuses_def = {
				FINISHED: {
					column_class_name: "importationCompleted text-left",
					translated: "<%=_T('Finished')%>"
				},
				ERROR: {
					column_class_name: "importationError text-left",
					translated: "<%=_T('Error')%>"
				},
				RUNNING: {
					column_class_name: "importationRunning text-left",
					translated: "<%=_T('Running')%>"
				}
			};
			var status_def = statuses_def[item_value];
			if (status_def) {
				value.class_name = "userImportationStatus";
				value.column_class_name = status_def.column_class_name;
				value.value = status_def.translated;
				value.search_value = value.value;
			}
			else {
				value.value = item_value;
				value.search_value = item_value;
			}
		}

		return value;
	};

	adm_form_users_importations.prototype.makePasswordsCheckUI = function(passwordChecks)
	{
		if (typeof passwordChecks !== "object") {
			return [];
		}
		const makePasswordCheckUI = function(text) {
			return text;
		};
		let translatedPasswordChecks = [];
		//passwordLen always goes
		let passwordLenText = inConcert.Utils.format("<%=_T('Be between {0} and {1} characters long')%>", passwordChecks.LEN.range[0], passwordChecks.LEN.range[1]);
		translatedPasswordChecks.push(makePasswordCheckUI(passwordLenText));

		if(passwordChecks.LOWERS.range[0] > 0) {
			let minLowerCharsText = inConcert.Utils.format("<%=_T('Include at least {0} lowercase characters')%>", passwordChecks.LOWERS.range[0]);
			translatedPasswordChecks.push(makePasswordCheckUI(minLowerCharsText));
		}
		if(passwordChecks.UPPERS.range[0] > 0) {
			let minUpperCharsText = inConcert.Utils.format("<%=_T('Include at least {0} uppercase characters')%>", passwordChecks.UPPERS.range[0]);
			translatedPasswordChecks.push(makePasswordCheckUI(minUpperCharsText));
		}
		if(passwordChecks.NUMBERS.range[0] > 0){
			let minNumericCharsText = inConcert.Utils.format("<%=_T('Include at least {0} numeric characters')%>", passwordChecks.NUMBERS.range[0]);
			translatedPasswordChecks.push(makePasswordCheckUI(minNumericCharsText));
		}
		if(passwordChecks.SYMBOLS.range[0] > 0) {
			let minSpecialCharsText = inConcert.Utils.format("<%=_T('Include at least {0} special characters')%>", passwordChecks.SYMBOLS.range[0]);
			translatedPasswordChecks.push(makePasswordCheckUI(minSpecialCharsText));
		}
		if(passwordChecks.HISTORY.range[0] > 0) {
			let historyText = inConcert.Utils.format("<%=_T('Not have been used in the last {0} passwords')%>", passwordChecks.HISTORY.range[0]);
			translatedPasswordChecks.push(makePasswordCheckUI(historyText));
		}
		return translatedPasswordChecks;
	};
	adm_form_users_importations.prototype.__get_grid_failed_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : "",
			class_name    : ""
		};

		var item_value = item[field_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "errType") {
			// Estos codigos de error se generan en la funcion "ValidateUser" del controller "users.lua" de la App "agent"
			var translated_errs = {
				"INVALID REQUEST": function() {return "<%=_T('Invalid request.')%>";},
				"USER WITH CHARS NOT ALLOWED": function() {return "<%=_T('Only letters, numbers, dashes and dots are allowed in the user Id.')%>";},
				"USER OUT OF RANGE": function() {return FORMAT("<%=_T('Length of User Id must be between {0} and {1} characters.')%>", 8, 255);},
				"USER ALREADY EXISTS": function() {return "<%=_T('User id already taken')%>";},
				"INVALID FORMAT MAIL": function() {return "<%=_T('Invalid format e-mail')%>";},
				"MAIL ALREADY EXISTS": function() {return "<%=_T('E-mail is required in VCC with HelpDesk Integration.')%>";},
				"MAIL IS REQUIRED IN VCC WITH HELPDESK INTEGRATION": function() {return "<%=_T('E-mail is required in VCC with HelpDesk Integration.')%>";},
				"PASSWORD IS REQUIRED": function() {return "<%=_T('Password is required.')%>";},
				"PASSWORD OUT OF RANGE": function() {return FORMAT("<%=_T('Password length must be between {0} and {1} characters.')%>", 8, 15);},
				"MUST HAVE PERMISSIONS TO INTERACTION APPLICATION TO USE ACTIVE SYSTEM STATE": function() {return "<%=_T('Must have permissions to interaction application to use ACTIVE system state')%>";},
				"IF HAVE PERMISSION FOR INTERACTION APPLICATION, MUST HAVE SOME ASSIGNED CAMPAIGN": function() {return "<%=_T('if have permission for interaction application, must have some assigned campaign')%>";},
				"CAMPAIGN DOES NOT EXIST": function() {return "<%=_T('Campaign does not exist')%>";},
				"USER IDENTIFIER IN USE": function() {return "<%=_T('User id already taken')%>";},
				"NOT RESOURCES AVAILABLES": function() {return "<%=_T('Not resources availables')%>";},
				"WORKGROUP IDENTIFIER IN USE": function() {return "<%=_T('Workgroup identifier in use')%>";},
				"WORKGROUP DOES NOT EXIST": function() {return "<%=_T('Workgroup does not exist')%>";},
				"FILE IS REQUIRED": function(row) {
					return FORMAT("<%=_T('Field {0} is required')%>", row.csvFieldName || row.fieldId);
				},
				"VALUE NOT ALLOWED": function(row) {
					return FORMAT("<%=_T('Value {0} is not allowed in field {1}')%>", row.fieldValue, row.csvFieldName || row.fieldId);
				},
				"ALLEGRO_INTEGRATION_ERROR": function() {return "<%=_T('Allegro integration error.')%>";},
				"HELPDESK_INTEGRATION_ERROR": function() {return "<%=_T('HelpDesk integration error.')%>";}
			};
			var err_msg;
			if (typeof(item_value) === "string") {
				err_msg = (translated_errs[item_value.toUpperCase()] || $.noop)();
			}
			else {
				if (item_value.msg) {
					//Los errores de password vienen en un formato especial
					if (item_value.fieldId === "Password" && item_value.result === "VALUE NOT ALLOWED") {
						err_msg = "<%=_T('Invalid password')%>";
						if (typeof item_value.msg ==="object" && item_value.msg.result === "INVALID CHARACTERS IN PASSWORD") {
							err_msg = "<%=_T('The new password value must not contain reserved characters (#, &, ?, =).')%>";
						}
						else if (typeof item_value.fieldValue === "object") {
							let customErr = "";
							for (let pwdErr of this.makePasswordsCheckUI(item_value.fieldValue)) {
								customErr = (customErr.length === 0 ? ` - ${"<%=_T('Password must')%>"}: ` : "; ") + pwdErr;
							}
							if (customErr.length > 0) {
								err_msg = `${err_msg} - ${customErr}`;
							}
						}
					}
					else {
						err_msg = (translated_errs[item_value.msg.toUpperCase()] || $.noop)();
					}
				}
				else {
					var err_type = item_value.result.toUpperCase();
					err_msg = (translated_errs[err_type] || $.noop)(item_value);
				}
			}
			value.value = err_msg || "<%=_T('Unrecognized Error')%>";
			value.search_value = err_msg;
		}

		return value;
	};

	adm_form_users_importations.prototype.__show_detail_importation = function(importation)
	{
		var self = this;

		if (!importation) {
			return;
		}

		var failed_users = importation.DetailFailedRows || [];
		var load_items = function(callback) {
			callback = callback || $.noop;
			callback(true, failed_users);
		};

		$("#adm_form_users_importation_detail_id").val(importation.Id || "");
		$("#adm_form_users_importation_detail_name").val(importation.Name || "");

		this.m_importations_failed_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.owner,
			row_id: "line",
			display_name: "id",
			auto_initialize: true,
			rows_per_page: 30,
			show_at_least: 29,
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Failed Users ...')%>",
			$custom_waiting_container: $("#adm_form_users_importation_popup_waiting"),
			ids: {
				paginator: "adm_form_users_importation_detail_failed_paginator",
				grid: "adm_form_users_importation_detail_failed_list",
				not_results_found: "adm_form_users_importation_detail_failed_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Failed Users Importations found.')%>",
			},
			columns: [
				{id: "_number", name: "", type: "counter"     },
				{id: "id"     , name: "<%=_T('Id')%>  "       },
				{id: "line"   , name: "<%=_T('Number Line')%>"},
				{id: "errType" , name: "<%=_T('Error Type')%>",
					get_value: function(item) {
						return self.__get_grid_failed_value(item, "errType");
					}
				}
			],
			callbacks: {
				load_items: function(_, callback) {
					load_items(callback);
				},
				must_shown_loading_msg: function() {
					return failed_users.length > 0;
				}
			}
		});
	};

	adm_form_users_importations.prototype.show_status_message = function(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container)
	{
		$custom_message_container = $custom_message_container || $("#adm_form_users_importation_popup_message");
		this.owner.show_status_message(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container);
		// TODO: cambiar este hide automatico, el mismo deber de ser realizado por la clase base conf_form en la invocacion a la funcion "show_status_message". No estaba funcionando. Revisar!!!
		if (timeout_ms) {
			if (this.m_show_message_handle) {
				this.m_show_message_handle = null;
				window.clearTimeout(this.m_show_message_handle);
			}
			this.m_show_message_handle = window.setTimeout(function() {
				$custom_message_container.hide();
			}, timeout_ms);
		}
	};

	adm_form_users_importations.prototype.show_waiting = function(message, $waiting_container)
	{
		$waiting_container = $waiting_container || $("#adm_form_users_importation_popup_waiting");
		this.owner.show_waiting(message, $waiting_container);
	};

	adm_form_users_importations.prototype.hide_waiting = function()
	{
		this.owner.hide_waiting();
	};

	adm_form_users_importations.prototype.disable_control = function($control, disabled, is_required, is_disable_lbl, keep_value, must_select_blank_item)
	{
		this.owner.disable_control($control, disabled, is_required, is_disable_lbl, keep_value, must_select_blank_item);
	};

	adm_form_users_importations.prototype.on_uploading_fn = $.noop;

	adm_form_users_importations.prototype.__make_allowed_columns = function()
	{
		var $header = $("#adm_form_users_importation_file_panel_allowed_csv_columns");
		$header.html("");
		$.each(this.m_allowed_csv_columns || [], function(_, column) {
			var $column = $("<div class='importationHeaderItem badge'>" + column.CSVFieldName + "</div>");
			$column.attr("col_name", column.CSVFieldName.toUpperCase());
			if (column.required) {
				$column.addClass("required");
			}
			$header.append($column);
		});
		$("#adm_form_users_importation_file_panel_csv_header_columns").show();
	};

	adm_form_users_importations.prototype.__mark_csv_columns = function(list, type)
	{
		var $header = $("#adm_form_users_importation_file_panel_allowed_csv_columns");
		$.each(list || [], function(_, col_name) {
			var $column = $header.find("[col_name='" + col_name.toUpperCase() + "']");
			if ($column.length > 0) {
				$column.removeClass("error").removeClass("success");
				if (type === "SUCCESS") {
					$column.addClass("success");
				}
				else {
					$column.addClass("error");
				}
			}
		});
	};

	adm_form_users_importations.prototype.__ready_for_start_importation = function()
	{
		$("#adm_form_users_importation_file_panel_start_importation").prop("disabled", false);
	};

	adm_form_users_importations.prototype.__disable_start_importation = function()
	{
		$("#adm_form_users_importation_file_panel_start_importation").prop("disabled", true);
	};

	adm_form_users_importations.prototype.on_uploaded = function(success, file_info, callback)
	{
		callback = callback || $.noop;
		var self = this;

		if (!success) {
			this.m_current_upload_info = null;
			callback();
		}
		else {
			this.m_current_upload_info = file_info;
			var importation_api = inConcert.app.getImportationAPI();

			importation_api.importGetUploadedCSVHeader(file_info,
				function(success, response)
				{
					self.m_csv_header = response.csvHeader;
					self.__mark_csv_columns(self.m_csv_header, "SUCCESS");
					if (success) {
						self.__ready_for_start_importation();
					}
					else {
						response.info.timemout_ms = 10000;
						self.__show_error(response.info);
						self.__disable_start_importation();
					}
				}
			);
			callback();
		}
	};

	adm_form_users_importations.prototype.can_upload_file = function(callback)
	{
		callback = callback || $.noop;
		callback(true);
	};

	adm_form_users_importations.prototype.on_render = function()
	{
		this.__initialize();
	};

	adm_form_users_importations.prototype.dispose = function()
	{
		if (this.m_upload_instance) {
			this.m_upload_instance.dispose();
		}
		this.m_current_upload_info = null;
		this.m_csv_header = null;
		this.m_total_importations = 0;
	};

	adm_form_users_importations.prototype.$get_markup = function()
	{
		return $("".concat(

			"<div class='internalPopUpContainer usersImportationPopUp popUp940w formPopUp'>",
				"<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp'>",

					// waiting
					"<div id='adm_form_users_importation_popup_waiting' class='internalWaitingContainer' style='display:none;'>",
						"<div waiting_message='true' class='internalWaitingMessage'></div>",
						"<div class='internalWaitingAnimation'></div>",
						"<div class='internalWaitingOverlay'></div>",
					"</div>",

					// title
					"<div class='internalPopUpTitle'>",
						"<label>", "<%=_T('New Importation')%>", "</label>",
					"</div>",

					// content
					"<div class='internalPopUpMiddle pageFormContainer'>",
						
						// mensaje de error
						"<div id='adm_form_users_importation_popup_message' class='genericFormMessagesContainer messageError' hidden>",
							'<label class="messageText"></label>',
						"</div>",
						
						// Panel con grillas
						"<div id='adm_form_users_importation_lists_panel' class='usersImportationList'>",
							
							// Lista de importaciones
							"<div class='card no-card' id='adm_form_users_importation_list_container'>" ,
								/*"<div class='card-block'>" ,*/
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											//button importaciones	
											"<button id='adm_form_users_new_importation' class='pageButtonGeneric buttonColorBlue pageButtonNew float-left ml-0'> <i class='far fa-plus'> </i> <%=_T('New Importation')%> </button>",
										"</div>" ,
									"</div>" ,
									//"<hr />" ,
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											"<div class='card-header'>" , 
												"<h5 class='mb-0'><%=_T('Imported Users')%></h5>" ,
												"<i id='adm_form_users_importation_list_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>" ,
										"</div>" ,
									"</div>" , 

									// tabla importaciones
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											"<div id='adm_form_users_importation_container' class='destinationsTableContainer'>",
												"<div id='adm_form_users_importation_not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
												"<table id='adm_form_users_importation_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid'>",
													"<thead class='viewGridHeader'></thead>",
													"<tbody class='viewGridBody'></tbody>",
												"</table>",
												"<div class='formDataDisclaimer inlineMessagesContainer messageInfo destinationsDisclaimer'><%=_T('DISCLAIMER.')%></div>",
											"</div>",
											"<div id='adm_form_users_importation_paginator'></div>",
										"</div>" ,
									"</div>" , 

									"<hr />" ,

								/*"</div>" ,*/
							"</div>" ,

							// Detalle de la importacion
							"<div class='card no-card' id='adm_form_users_importation_detail_container'>" ,
								"<div class='card-block'>" ,
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											"<div class='card-header'>" , 
												"<h5><%=_T('Importation Detail')%></h5>" ,
											"</div>" ,
										"</div>" ,
									"</div>" , 
									"<div class='row'>" ,
										"<div class='col-6'>" ,
											"<div class='form-group'>" ,
												"<label for='adm_form_users_importation_detail_id'><%=_T('Id')%><%=_T(':')%></label>",
												"<input id='adm_form_users_importation_detail_id' class='form-control' type='text' disabled='true' />",
											"</div>" ,
										"</div>" ,
										"<div class='col-6'>" ,
											"<div class='form-group'>" ,
												"<label for='adm_form_users_importation_detail_name'><%=_T('Name')%><%=_T(':')%></label>",
												"<input id='adm_form_users_importation_detail_name' class='form-control' type='text' disabled='true' />",
											"</div>" ,
										"</div>" ,
									"</div>" , 
									"<hr />" ,
									// tabla detalle fallos importaciones
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											"<div class='card-header'>" , 
												"<h5 class='mb-0'><%=_T('Importation Fails Detail')%></h5>" ,
											"</div>" ,
										"</div>" ,
									"</div>" , 
									"<div class='row'>" ,
										"<div class='col-12'>" ,
											"<div id='adm_form_users_importation_detail_failed_container' class='destinationsTableContainer'>",
												"<div id='adm_form_users_importation_detail_failed_not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
												"<table id='adm_form_users_importation_detail_failed_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid'>",
													"<thead class='viewGridHeader'></thead>",
													"<tbody class='viewGridBody'></tbody>",
												"</table>",
												"<div class='formDataDisclaimer inlineMessagesContainer messageInfo destinationsDisclaimer'><%=_T('DISCLAIMER.')%></div>",
											"</div>",
											"<div id='adm_form_users_importation_detail_failed_paginator'></div>",
										"</div>" ,
									"</div>" , 
								"</div>" ,
							"</div>" ,
						"</div>",
					"</div>",
					// MODAL FOOTER BUTTONS
					"<div class='internalPopUpButtons'>",
						/* "<button id='adm_form_users_new_importation' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('New Importation')%>" , "</button>" , */
						"<button id='adm_form_users_importation_close' class='pageButtonGeneric buttonColorWhite float-right'>" , "<%=_T('Close')%>" , "</button>" ,
					"</div>",
				"</div>",
			"</div>",
			// INTERNAL POPUP con los controles de upload file
			this.__get_html_popup_upload_file()
		));
	};

	adm_form_users_importations.prototype.__get_html_popup_upload_file = function()
	{
		var html = "".concat(
			"<div id='adm_form_users_importation_file_panel' class='internalPopUpContainer popUp380w internalUserImportationPopUp formPopUp' hidden>",
				// "<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp'>",

					// waiting
					"<div id='adm_form_users_importation_file_panel_waiting' class='internalWaitingContainer' style='display:none;'>",
						"<div waiting_message='true' class='internalWaitingMessage'></div>",
						"<div class='internalWaitingAnimation'></div>",
						"<div class='internalWaitingOverlay'></div>",
					"</div>",

					// title no va en este caso
					// "<div class='internalPopUpTitle'>" + "<%=_T('New Importation')%>" + "</div>",

					// content
					"<div class='internalPopUpMiddle pageFormContainer'>",

						// mensaje de error
						"<div id='adm_form_users_importation_file_panel_popup_message' class='genericFormMessagesContainer messageError' hidden>",
							"<label class='messageText'></label>",
						"</div>",

						// Detalle de la importacion
						"<div class='card no-card' id='adm_form_users_importation_detail_container'>" ,
							"<div class='card-block'>" ,
								"<div class='row'>" ,
									"<div class='col-12'>" ,
										"<div class='form-group'>" ,
											"<label for='adm_form_users_importation_file_panel_name' class='required' ><%=_T('Name')%><%=_T(':')%></label>",
											"<input id='adm_form_users_importation_file_panel_name' class='form-control' placeholder='", "<%=_T('Enter name')%>...", "' type='text' maxlength='100' />",
											"<i id='adm_form_users_importation_file_panel_name_validation' class='formFieldValidation formFieldError' hidden></i>" ,
										"</div>" ,
									"</div>" ,
								"</div>" , 

								// Aca se genera de forma dinamica desde JAVASCRIPT el DOM correspondiente al Upload de archivos. (tiene row y col)
								"<div id='adm_form_users_importation_file_panel_upload' class='fileSelectionSection'></div>",
								
								//"<hr />" ,
							"</div>" ,
						"</div>" ,
						// Columnas permitidas
						"<div class='card no-card' id='adm_form_users_importation_file_panel_csv_header_columns' hidden>" ,
							"<div class='card-block'>" ,
								"<div class='row'>" ,
									"<div class='col-12'>" ,
										"<div class='form-group'>" ,
											"<label class='pl-0'><%=_T('Allowed CSV Columns')%><%=_T(':')%></label>",
											"<div id='adm_form_users_importation_file_panel_allowed_csv_columns' class='importationHeaders w-100'></div>",
										"</div>" ,
									"</div>" ,
								"</div>" ,
							"</div>" ,
						"</div>" ,
					"</div>",
					// MODAL FOOTER BUTTONS
					"<div class='internalPopUpButtons'>",
						"<button id='adm_form_users_importation_file_panel_back' class='pageButtonGeneric buttonColorWhite float-right'>" , "<%=_T('Close')%>" , "</button>" ,
						"<button id='adm_form_users_importation_file_panel_start_importation' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('Confirm')%>" , "</button>" ,
					"</div>",
				"</div>",
			"</div>"
		);
		return html;
	};

	adm_form_users_importations.prototype.set_events = function()
	{
		var self = this;

		$("#adm_form_users_new_importation")
			.off("click")
			.on("click", function() {
				self.__new_importation();
			});

		$("#adm_form_users_importation_close")
			.off("click")
			.on("click", function() {
				self.close_view();
			});

		$("#adm_form_users_importation_close_times")
			.off("click")
			.on("click", function() {
				self.close_view();
			});

		$("#adm_form_users_importation_file_panel_start_importation")
			.off("click")
			.on("click", function() {
				self.__start_importation();
			});

		$("#adm_form_users_importation_file_panel_back")
			.off("click")
			.on("click", function() {
				self.__show_importation_list_panel();
			});
	};

	adm_form_users_importations.prototype.__show_importation_panel = function()
	{
		// $("#adm_form_users_new_importation").hide();
		// $("#adm_form_users_importation_close").hide();
		// $("#adm_form_users_importation_close_times").hide();
		$("#adm_form_users_importation_file_panel_popup_message").hide();
		$("#adm_form_users_importation_file_panel").show();
		var $name = $("#adm_form_users_importation_file_panel_name");
		var validation_id = $name.attr("id") + "_validation";
		var $validation_name = $("#" + validation_id);
		if ($validation_name.length > 0) {
			$validation_name.hide();
		}
		$name.val("").focus();
	};

	adm_form_users_importations.prototype.__show_importation_list_panel = function()
	{
		$("#adm_form_users_importation_file_panel").hide();
		// $("#adm_form_users_new_importation").show();
		// $("#adm_form_users_importation_close").show();
		// $("#adm_form_users_importation_close_times").show();
	};

	adm_form_users_importations.prototype.close_view = function()
	{
		$("#adm_form_users_importation_file_panel_name").val("");
		this.owner.popup_hide();
	};

	adm_form_users_importations.prototype.__new_importation = function()
	{
		if (!can_exec_importation())
			return this.__show_error({code: 403}, $("#adm_form_users_importation_popup_message"));
		this.__initialize_upload_ctrl();
		this.__disable_start_importation();
		this.__make_allowed_columns();
		this.__show_importation_panel();
	};

	adm_form_users_importations.prototype.show_validation_error = function($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container)
	{
		$custom_message_container = $custom_message_container || $("#adm_form_users_importation_popup_message");
		this.owner.show_validation_error($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container);
	};

	adm_form_users_importations.prototype.__can_start_importation = function()
	{
		$("#adm_form_users_importation_file_panel").find(".formFieldValidation").hide();
		const result = {success: false};
		if (!this.m_current_upload_info) {
			result.error = {
				code: -1,
				type: "CSV FILE HAS NOT BEEN UPLOADED"
			};
			return result;
		}

		const $importation_name = $("#adm_form_users_importation_file_panel_name");
		const importation_info = {
			fileInfo: this.m_current_upload_info,
			importName: $importation_name.val()
		};
		const validation_result = inConcert.Utils.validate(importation_info.importName, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			const $custom_message_container = $("#adm_form_users_importation_file_panel_popup_message");
			this.show_validation_error($importation_name, "<%=_T('Name is required.')%>", undefined, undefined, undefined, $custom_message_container);
			return result;
		}

		// valido si los permisos han cambiado desde que tengo la pantalla abierta (en tiempo real)
		if (!can_exec_importation()) {
			result.error = {
				code: 403
			};
			return result;
		}

		// OK
		result.success = true;
		result.importation_info = importation_info;

		return result;
	};

	adm_form_users_importations.prototype.__show_error = function(current_err, $message_container)
	{
		this.hide_waiting();
		if (typeof current_err !== "object") {
			return;
		}
		var code_error = parseInt(current_err.code, 10);
		var msg;
		switch(code_error) {
			// Client side errors
			case -1:
				msg = "<%=_T('CSV File has not been uploaded')%>";
				break;

			// Server side errors
			case -100:
				msg = "<%=_T('Invalid request.')%>";
				break;

			case -101:
				msg = "<%=_T('CSV File does not exist')%>";
				break;

			case -102:
				var missing_fields = current_err.missingRequiredFields || [];
				var total_missing_fields = missing_fields.length;
				msg = total_missing_fields === 1 ?
					"<%=_T('Missing {0} required field in CSV File')%>" :
					"<%=_T('Missing {0} required fields in CSV File')%>";
				msg = FORMAT(msg, total_missing_fields);
				this.__mark_csv_columns(missing_fields, "ERROR");
				break;

			case -103:
				msg = "<%=_T('Unable to Import. There another import in progress, try later.')%>";
				break;

			case -104:
				msg = FORMAT("<%=_T('The import must not exceed {0} users. File contains {1} Users.')%>", current_err.maximumNumberUsersInImport, current_err.fileRows);
				break;

			case -105:
				msg = "<%=_T('Invalid CSV File')%>";
				break;

			case -106:
				msg = "<%=_T('Error validating campaigns/users relationship limits.')%>";
				if (current_err.type && current_err.type === "ERROR_CAMPS_USERS_EXCEEDS_LIMIT_ALLOWED") {
					const limitCampUsers = inConcert.app.getMaxConfigurationsAllowed("MaxCampaignsUsers");
					msg = FORMAT("<%=_T('The total number of users assigned to the campaigns exceeds the limit allowed {0}.')%>", limitCampUsers);
				}
				current_err.timemout_ms = 10000;
				break;

			case 403:
				msg = "<%=_T('You dont have permissions for this action.')%>";
				break;

			default:
				if (current_err.result === "forbidden") 
					msg = "<%=_T('You dont have permissions for this action.')%>";
				else
					msg = "<%=_T('Unable to Import.')%>";
		}

		const $custom_message_container = $message_container || $("#adm_form_users_importation_file_panel_popup_message");
		const timemout_ms = current_err.timemout_ms || 5000;
		this.show_status_message(msg, "Error", timemout_ms, undefined, undefined, $custom_message_container);
	};

	adm_form_users_importations.prototype.__start_importation = function()
	{
		var self = this;

		// Funcion que regresa a la vista cuando el start importation se ha realizado correctamente y el start se encuentra en progreso o ha finalizado.
		var go_back = function(msg, importation) {
			self.hide_waiting();
			self.show_status_message(msg, "Info", 5000);
			window.setTimeout(function() {
				self.__load_importations_grid();
				self.__show_detail_importation(importation);
				self.__show_importation_list_panel();
			}, 500);
		};

		var import_start = function(importation_info)
		{
			var $waiting_container = $("#adm_form_users_importation_file_panel_waiting");
			self.show_waiting("<%=_T('Please wait, Starting Importation....')%>", $waiting_container);
			self.__disable_start_importation();
			var importation_api = inConcert.app.getImportationAPI();
			importation_api.importStart(importation_info, function(success, importation)
			{
				self.m_total_importations++;
				if (success) {
					var msg = "<%=_T('Starting Importation.')%>";
					go_back(msg, importation);
					return;
				}
				var err = importation;
				try {err = JSON.parse(err);}
				catch(e) {}
				self.__show_error(err);
			});
		};

		var result = this.__can_start_importation();
		if (!result.success) {
			self.__show_error(result.error);
			return false;
		}
		import_start(result.importation_info);
	};

	adm_form_users_importations.prototype.__load_items = function(callback)
	{
		var self = this;

		callback = callback || $.noop;
		var importation_api = inConcert.app.getImportationAPI();
		importation_api.loadImportations(function(success, importations) {
			if (!success) {
				self.show_status_message("<%=_T('Error retrieving Importations.')%>", "Error", false);
				callback(false);
				return;
			}
			callback(true, importations);
		});
	};

	adm_form_users_importations.prototype.has_made_imports = function()
	{
		return this.m_total_importations > 0;
	};

	adm_form_users_importations.prototype.__remove_importation = function(importation, callback)
	{
		callback = callback || $.noop;
		var self = this;

		if (!importation) {
			throw("adm_form_users_importations.prototype.adm_form_users_importations. Undefined importation.");
		}

		var end = function(success) {
			var msg;
			if (success) {
				msg = FORMAT("<%=_T('Importation {0} was successfully deleted.')%>", importation.Name);
				self.show_status_message(msg, "Info", 5000);
				self.hide_waiting();
			}
			else {
				msg = FORMAT("<%=_T('Error deleting Importation {0}.')%>", importation.Name);
				self.show_status_message(msg, "Error", 5000);
				self.hide_waiting();
			}
			callback(success);
		};

		this.show_waiting(FORMAT("<%=_T('Deleting Importation {0}.')%>", importation.Name));
		// pequeno timeout para que pueda ser visualizado el waiting y no se realice un juego de pantalla
		window.setTimeout(function() {
			var params = {importationId: importation.Id };
			var importation_api = inConcert.app.getImportationAPI();
			importation_api.importDelete(params,
				function(success) {
					end(success);
				}
			);
		}, 500);
	};

}());
