/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form layout class
	function adm_form_vcc_layouts(params)
	{
		let success = adm_form_vcc_layouts.uber.constructor.call(this, params);
		if ( !success ) {
			return;
		}

		this.new_title      = "<%=_T('Create Window Design')%>";
		this.update_title   = "<%=_T('Edit Window Design {0}')%>";
		this.view_title     = "<%=_T('View Client Window Design {0}')%>";
		this.tagline        = "<%=_T('Tagline for Clients Windows Design')%>";

		this.html_file_name = "/inconcert/apps/administration/vccs/clients_design/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/vccs/clients_design/save/";
		this.get_uri        = "{0}/inconcert/apps/administration/vccs/clients_design/get/{1}";

		this.include_save_and_continue = false;
		this.m_all_current_layouts = null;

		this.m_layout_hnd = new inConcert.LayoutVccDesign({
			owner       : this,
			container_id: "adm_form_clients_layout",
			parent      : {frm: this, is_readonly: this.is_readonly}
		});

		return true;
	}

	inConcert.Utils.inherit(adm_form_vcc_layouts, inConcert.conf_form);

	if ( typeof inConcert.adm_form_vcc_layouts !== "function" ) {
		inConcert.adm_form_vcc_layouts = adm_form_vcc_layouts;
	}

	adm_form_vcc_layouts.prototype.initialize = function()
	{
		if (this.is_creating()) {
			this.m_layout_hnd.set_layout(null);
			//this.m_layout_hnd.initialize();
		}
	};

	adm_form_vcc_layouts.prototype.dispose = function()
	{
		this.m_all_current_layouts = null;
		this.m_layout_hnd.dispose();
	};

	adm_form_vcc_layouts.prototype.set_events = function()
	{
		adm_form_vcc_layouts.uber.set_events.call(this); //invokes the base class method
		this.m_layout_hnd.set_events();
	};

	adm_form_vcc_layouts.prototype.readonly = function()
	{
		adm_form_vcc_layouts.uber.readonly.call( this );
		this.m_layout_hnd.readonly();
	};

	adm_form_vcc_layouts.prototype.set = function(success, layout)
	{
		if (!success) {
			return;
		}
		this.m_layout_hnd.set_layout(layout);
	};

	adm_form_vcc_layouts.prototype.ready_for_save = function()
	{
		adm_form_vcc_layouts.uber.ready_for_save.call(this);  //invokes the base class method
		this.m_layout_hnd.set_readonly(false);
		this.m_layout_hnd.initialize();
		this.m_layout_hnd.set_availables_layouts_types(this.m_all_current_layouts);
		this.m_layout_hnd.edit();
	};

	adm_form_vcc_layouts.prototype.readonly = function()
	{
		this.m_layout_hnd.set_readonly(true);
		this.m_layout_hnd.initialize();
		this.m_layout_hnd.set_availables_layouts_types(this.m_all_current_layouts);
		this.m_layout_hnd.readonly();
		adm_form_vcc_layouts.uber.readonly.call(this); //invokes the base class method
	};

	adm_form_vcc_layouts.prototype.empty_controls = function()
	{
		this.m_layout_hnd.set_layout(null);
		this.m_layout_hnd.empty_controls();
	};

	adm_form_vcc_layouts.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;

		var get_error_description = function(err_type)
		{
			is_custom_message = true;
			let campaign_err = err.campaign || "";
			let mail_user_url_err = err.user || "";
			let mail_err = err.mail || "";

			if (( err.url || "" ).length > 0) {
				mail_user_url_err = mail_user_url_err + " - " + err.url;
			}

			let message;
			switch (err_type)
			{
				default:
					is_custom_message = false;
					message = "<%=_T('Unable to save Client Window Design. Server error')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				let error_type = err.result.toUpperCase();
				err = get_error_description(error_type);
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		adm_form_vcc_layouts.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	adm_form_vcc_layouts.prototype.save = function(callback)
	{
		let layout_info = this.m_layout_hnd.validate();
		if (!layout_info.success) {
			return;
		}
		let layout = {
			[layout_info.layout_id]: layout_info.layout,
			// Estos 2 atributos NO son necesarios server side, pero SI los necesito para visualizar info en la grilla.
			Id: layout_info.layout_id,
			Name: layout_info.layout_id
		};
		adm_form_vcc_layouts.uber.save.call(this, callback, layout); //invokes the base class method
	};

	adm_form_vcc_layouts.prototype.uri_get_entity = function()
	{
		if (this.is_new) {
			return "";
		}
		let name = this.$current_row.attr("row_id");
		return FORMAT(this.get_uri, inConcert.app.getBaseDomain(), name);
	};

	adm_form_vcc_layouts.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const base_uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/";

		this.requests = [
			{
				url: base_uri + "vccs/clients_design/search/",
				type: "POST",
				data: {columns: ["Id", "Name"]},
				success: function(layouts)
				{
					self.__set_all_current_layouts(layouts.rows);
				}
			}
		];

		adm_form_vcc_layouts.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	adm_form_vcc_layouts.prototype.__set_all_current_layouts = function(all_current_layouts)
	{
		this.m_all_current_layouts = all_current_layouts;
	};
}());

