/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";
	const FORMAT = inConcert.Utils.format;
	//---------------------------------------------------------------------
	// form workgroups class
	function adm_form_workgroups(params)
	{
		/* jshint validthis: true*/
		var success = adm_form_workgroups.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Create workgroup')%>";
		this.update_title = "<%=_T('Update workgroup {0}')%>";
		this.view_title = "<%=_T('View workgroup {0}')%>";
		this.tagline = "<%=_T('Tagline form Workgroup!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/workgroups/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/workgroups/save/";
		this.get_uri        = "/inconcert/apps/administration/workgroups/get/";

		this.m_grd_members_list = null;
		this.__initialize_grid();
		return true;
	}

	inConcert.Utils.inherit(adm_form_workgroups, inConcert.conf_form);

	if (typeof inConcert.adm_form_workgroups !== "function") {
		inConcert.adm_form_workgroups = adm_form_workgroups;
	}

	adm_form_workgroups.prototype.get = function()
	{
		var workgroup = {
			Name: $("#adm_form_workgroup_name").val(),
			Description: $("#adm_form_workgroup_desc").val()
		};
		var action = this.dashboard.get_current_action_obj();
		var id = action.view.row_id;
		if (!this.is_new) {
			workgroup[id] = this.$current_row.attr("row_id");
		}
		return workgroup;
	};

	const get_full_name  = function( {Name, FirstSurname} ) {
		const full_name = `${Name} ${FirstSurname}`;
		return {
			original_value: full_name,
			value         : full_name
		};
	};

	adm_form_workgroups.prototype.set = function(success, workgroup)
	{
		if (!success) {
			return;
		}
		$("#adm_form_workgroup_name").val(workgroup.Name);
		$("#adm_form_workgroup_desc").val(workgroup.Description);

		let rows = [];
		let row_data = this.$current_row.data("row_data") || {};
		$.each(row_data.Agents || [], function(_, data) {
			data.Role = "<%=_T('agent')%>";
			rows.push(data);
		});
		$.each(row_data.Supervisors || [], function(_, data) {
			data.Role = "<%=_T('supervisor')%>";
			rows.push(data);
		});
		this.m_grd_members_list.fill_rows($.noop, inConcert.Utils.sortArray(rows, "Name"));
		
	};

	// get user avatar
	const get_user_avatar = function(user)
		{
			const image_url = inConcert.app.getBaseDomain() + "/inconcert/apps/agent/users/get_avatar/" + user.Id;
			return { 
				class_name: "avatarContainer",
				src       : image_url
			};
		};

	adm_form_workgroups.prototype.__initialize_grid = function()
	{
		this.m_grd_members_list = new inConcert.conf_form_paginator({
			paginator_name: "MEMBERS_LIST",
			owner: this,
			frm: this,
			row_id: "id",
			display_name: "id",
			rows_per_page: 10,
			show_at_least: 9,
			auto_initialize: true,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			ids: {
				grid: "adm_form_workgroup_members_list",
				paginator: "adm_form_workgroup_members_list_paginator",
				not_results_found: "adm_form_workgroup_members_list_not_results_found"
			},
			strings: {
				not_results_found         : "<%=_T('Not found.')%>",
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "_avatar"    , name: "" , type: "image", class_name: "avatarColumnTitle", get_value: get_user_avatar},
				{id: "Id"         , name: "<%=_T('Id')%>"},
				{id: "FullName"   , name: "<%=_T('Full Name')%>", get_value: get_full_name},
				{id: "Role"       , name: "<%=_T('Role')%>"}
			],
			callbacks: {}
			
		});
	};

	adm_form_workgroups.prototype.empty_controls = function()
	{
		$("#adm_form_workgroup_name").val("").focus();
		$("#adm_form_workgroup_desc").val("");
	};

	adm_form_workgroups.prototype.err_save_messages = function(err)
	{
		var is_custom_message = false;

		if ( typeof err === "object" ) {
			var error_type = err.result.toUpperCase();
			if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
				$("#adm_form_workgroup_name").focus();
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				$("#adm_form_workgroup_desc").focus();
			}
			else if ( err.result === "ALLEGRO_INTEGRATION_ERROR" ) {
				err = "<%=_T('Error updating information on Allegro integration...')%>";
				$("#adm_form_workgroup_name").focus();
				is_custom_message = true;
			}else if (err.result === "ERROR_WORKGROUPS_EXCEEDS_LIMIT_ALLOWED"){
				const limit = inConcert.app.getMaxConfigurationsAllowed("MaxWorkgroups");
				err = FORMAT("<%=_T('Can not create workgroups, exceeds limit allowed {0}.')%>", limit);
				is_custom_message = true;
			}
		}

		adm_form_workgroups.uber.err_save_messages.call( this, err, is_custom_message ); //invokes the base class method
	};

	adm_form_workgroups.prototype.get_validation_def = function()
	{
		var definitions = [
			{
				control_id: "adm_form_workgroup_name",
				messages: {
					required: "<%=_T('Please enter the workgroup Name.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the workgroup Name.')%>"
				}
			},
			{
				control_id: "adm_form_workgroup_desc",
				messages: {
					required: "<%=_T('Please enter the workgroup description.')%>"
				}
			}
		];
		return definitions;
	};

	adm_form_workgroups.prototype.save = function(callback)
	{
		adm_form_workgroups.uber.save.call( this, callback );  //invokes the base class method
	};

	adm_form_workgroups.prototype.ready = function()
	{
		adm_form_workgroups.uber.ready.call( this );  //invokes the base class method

		var $name = $("#adm_form_workgroup_name");
		if (this.is_new) {
			this.m_grd_members_list.fill_rows($.noop, []);
		}
		else if (inConcert.app.has_allegro_integration()) {
			this.readonly_control($name, true);
			$("#adm_form_workgroup_desc").focus();
			return;
		}
		$name.focus();
	};

}());
