/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	var format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// skills usage handle class (instanced in frm user, accounts/campaign)
	//
	function adm_skills_handle( params )
	{
		if ( !params.input_ctrls || !params.input_ctrls.skill_value ) {
			throw("adm_skills_handle: Parameter params.input_ctrls is reqquired ");
		}

		this.container_id = params.container_id;
		this.parent_frm   = params.parent_frm;
		this.parent_type  = params.parent_type;
		this.title        = params.title || "";
		this.help         = params.help || "";
		this.input_ctrls  = params.input_ctrls;

		this.list_id      = this.container_id + "_list";
		this.message_id   = params.container_id + "_message_not_results_found";

		// estos retries son para el set de datos cuando se edita en el frm de accounts
		//  ya que el DOM de los skills quizas aun no se haya generado ( se genera en la respuesta del request )
		this.count_setting_retries = 0;
		this.max_setting_retries = 10;
		this.request_status = "NONE";

		this.combo_values = [
			{ name: "<%=_T('Low')%>"    , value : 1 , range: { from: 1, to: 3  } },
			{ name: "<%=_T('Medium')%>" , value : 5 , range: { from: 4, to: 7  } },
			{ name: "<%=_T('High')%>"   , value : 10, range: { from: 7, to: 10 } },
		];

		return true;
	}

	if ( typeof inConcert.adm_skills_handle !== "function" ) {
		inConcert.adm_skills_handle = adm_skills_handle;
	}

	adm_skills_handle.prototype.initialize = function()
	{
		this.build_container_html();
		this.set_events();
		this.load_data();
	};

	adm_skills_handle.prototype.load_data = function()
	{
		var self = this;
		this.request_status = "RUNNING";
		inConcert.app.loadSkills(function( success, skills )
		{
			if ( success ) {
				self.build_skills_html(skills);
				self.request_status = "COMPLETED";
				return;
			}
			self.request_status = "ERROR";
			// Pequeno timeout para que se pueda visualizar el mensaje de error, ya que este request se tuvo que generar desacoplado al handle de request
			// de los forms de las clases bases de "configuration"
			window.setTimeout(function() {
				var timeout_ms = false; //infinite
				self.parent_frm.show_status_message( "<%=_T('Error retrieving Skills list.')%>", "Error", timeout_ms);
			}, 3000);
		});
	};

	adm_skills_handle.prototype.fill_combos = function()
	{
		var self = this;
		$("#" + this.list_id).find("select[skill_value='true']").each(function(_, cbo) {
			self.parent_frm.fill_combo( $(cbo), true, self.combo_values, "value", "name", false );
		});
	};

	adm_skills_handle.prototype.build_container_html = function()
	{
		var help = this.help.length > 0 ? (this.help) : "";

		$("#" + this.container_id).html(
			'<div class="row">' +
				'<div class="col-12">' +
					'<div class="card-header">' +
						'<h5 class="mb-2">'+ this.title + ' <span class="fa fa-question-circle" title="' + help + '"></span></h5>' +
					'</div>' +
				'</div>' +
			'</div>' +
			'<div class="row">' +
				'<div class="col-12">' +
					'<ul id="' + this.list_id + '" class="viewList viewListAsGrid mb-3"></ul>' +
					'<div id="' + this.message_id + '" class="inlineMessagesContainer messageInfo" hidden></div>' +
				'</div>' +
			'</div>'
		);

		if(this.help.length > 0){
			$("h5 span.fa-question-circle").removeClass("d-none");
		} else {
			$("h5 span.fa-question-circle").addClass("d-none");
		}
	};

	adm_skills_handle.prototype.get_opt_id = function( skill_id )
	{
		var normalized_id = inConcert.Utils.getNormalizedId( skill_id );
		var opt_id = this.list_id + "_opt_skill_" + normalized_id;
		return opt_id;
	};

	adm_skills_handle.prototype.get_opt_input_ctrl_id = function( skill_id )
	{
		return "input_" + this.get_opt_id( skill_id );
	};

	adm_skills_handle.prototype.get_opt_required_id = function( skill_id )
	{
		return "required_" + this.get_opt_id( skill_id );
	};

	adm_skills_handle.prototype.get_input_ctrl_type = function()
	{
		return this.input_ctrls.skill_value;
	};

	adm_skills_handle.prototype.build_skills_html = function( skills )
	{
		var self = this;
		var sorted_skills = inConcert.Utils.sortArray(skills, null, null,
			function(skill) {
				return skill.Name? skill.NameLabel : skill.IdLabel;
			});

		var html_skills = "";
		var has_skills = sorted_skills.length > 0;
		// input de entrada ( textbox o combo )
		var type_ctrl_value = this.get_input_ctrl_type();

		let skill_is_required = self.input_ctrls.skill_required;

		if (has_skills) {
			this.hide_not_skills_found();

			let html_required_column = "";
			if (skill_is_required) {
				html_required_column = "<label class='viewListTitleText checkboxImageCol requiredColumn'>" + "<%=_T('Required')%>" + "</label>";
			}

			// header
			html_skills = html_skills +
				"<li class='viewListTitle'>" +
					"<span class='firstItemCheckbox'></span>" +
					"<label class='viewListTitleText nameColumn flex-grow-1 w-100'>"  + "<%=_T('Name')%>"        + "</label>" +
					"<label class='viewListTitleText descriptionColumn flex-grow-1 w-100'>" + "<%=_T('Description')%>" + "</label>" +
					"<label class='viewListTitleText valueColumn formItemInput100'>" + "<%=_T('Value')%>"       + "</label>" +
					html_required_column +
				"</li>";

			var get_ctrl_value = function( skill )
			{
				var default_value = skill.DefaultValue || 1;
				var html_actions = "";
				var dom_skill_value_id = self.get_opt_input_ctrl_id( skill.Id );

				if ( type_ctrl_value === "TEXTBOX" ) {
					html_actions = "<input id='" + dom_skill_value_id + "' class='formListField form-control' skill_value='true' type='text' default_value='" + default_value + "' disabled='true' />";
				}
				else if ( type_ctrl_value === "COMBO" ) {
					html_actions = "<select id='" + dom_skill_value_id + "' class='formListField form-control' skill_value='true' default_value='" + default_value + "' disabled='true'></select>";
				}
				else {
					throw("adm_skills_handle.build_skills_html: Unrecognized input control '" + type_ctrl_value + "'");
				}

				return html_actions;
			};

			var get_ctrl_required = function( skill )
			{
				var html_actions = "";
				// checkbox para marcar si es requerido el skills
				if (skill_is_required) {
					var required_id = self.get_opt_required_id( skill.Id );
					html_actions = "<div class='form-check form-check-only mt-0'>" +
										"<input class='formListCheck ml-0' id='" + required_id + "' type='checkbox' />" +
									"</div>";
				}
				return html_actions;
			};

			// lista de skills
			$.each(sorted_skills, function( _, skill )
			{
				var skill_id = self.get_opt_id(skill.Id);
				var skill_name = self.sanitize_value(skill.Name || skill.Id);
				var skill_desc = self.sanitize_value(skill.Description || "");

				html_skills = html_skills +
					"<li class='viewListItem' skill_id='" + skill.Id + "'>" +
						"<input id='" + skill_id + "' skill_id='" + skill.Id + "' class='formListCheck' selector='1' type='checkbox' />" +
						"<div class='viewListItemText skillText'>" +
							"<label for='" + skill_id + "'>" + skill_name + "</label>" +
						"</div>" +
						"<div class='viewListItemText skillText'>" +
							"<label for='" + skill_id + "'>" + skill_desc + "</label>" +
						"</div>" +
						"<div class='formItemInput100'>" +
							"<div skill_value_container='true' class='viewListItemSkillValue' hidden>" +
								get_ctrl_value( skill ) +
							"</div>" +
						"</div>" +
						"<div skill_value_container='true' class='viewListSkillRequired viewListItemText checkboxImageCol' hidden style='display: none;'>" +
								get_ctrl_required( skill ) +
						"</div>" +
					"</li>";
			});
		}
		else {
			this.show_not_skills_found();
		}

		$("#" + this.list_id).html( html_skills );
		if ( has_skills && type_ctrl_value === "COMBO" ) {
			this.fill_combos();
		}
	};

	adm_skills_handle.prototype.sanitize_value = function(value)
	{
		if (!this.parent_frm || !this.parent_frm.sanitize_value) {
			return value;
		}
		return this.parent_frm.sanitize_value(value);
	};

	adm_skills_handle.prototype.show_container = function()
	{
		$("#" + this.container_id).show();
	};

	adm_skills_handle.prototype.hide_container = function()
	{
		$("#" + this.container_id).hide();
	};

	adm_skills_handle.prototype.is_visible_container = function()
	{
		return $("#" + this.container_id).is(":visible");
	};

	adm_skills_handle.prototype.show_not_skills_found = function( message )
	{
		$("#" + this.message_id)
			.text( message || "<%=_T('No skills found.')%>" )
			.show();
	};

	adm_skills_handle.prototype.hide_not_skills_found = function()
	{
		$("#" + this.message_id).hide();
	};

	adm_skills_handle.prototype.unmark_all = function()
	{
		var self = this;

		$("#" + this.list_id + " input[selector='1'][type='checkbox']:checked").each(function( _, skill )
		{
			var $chk = $(skill);
			$chk.prop("checked", true); // primero lo marco ya que despues lo vuelvo a desmarcar en (*)
			var skill_id =  $chk.attr("skill_id");
			var $skill = $("#" + self.get_opt_input_ctrl_id( skill_id ));
			self.set_input_default_value( $skill, true /* force */ );
			$chk.trigger("click"); // (*) El trigger del evento lo desmarca
		});
	};

	adm_skills_handle.prototype.editable = function()
	{
		// editable chks de la lista
		$("#" + this.list_id + " input[type='checkbox']")
			.prop("disabled", false)
			.removeClass("readonly");

		// editable los divs contenedores de textos en la lista
		$("#" + this.list_id + " .viewListItemText.skillText")
			.removeClass("readonly");

		// editable los inputs con los valores de los skills en la lista
		$("#" + this.list_id + " .formListField.form-control")
			.prop("disabled", false)
			.removeClass("readonly");
	};

	adm_skills_handle.prototype.readonly = function()
	{
		var self = this;
		if ( this.request_status === "NONE" || this.request_status === "RUNNING" ) {
		//  Si el DOM de los skills quizas aun no se haya generado ( se genera en la respuesta del request )
			window.setTimeout(function() {
				self.readonly();
			}, 500);
			return;
		}
		else if ( this.request_status === "COMPLETED" ) {
			// readonly chks de la lista
			$("#" + this.list_id + " input[type='checkbox']")
				.prop("disabled", true)
				.addClass("readonly");

			// readonly los divs contenedores de textos en la lista
			$("#" + this.list_id + " .viewListItemText.skillText")
				.addClass("readonly");

			// readonly los inputs con los valores de los skills en la lista
			$("#" + this.list_id + " .formListField.form-control")
				.prop("disabled", true)
				.addClass("readonly");
		}
		// El otro estado es "ERROR" pero no hace nada con el, ya que el mensaje de error ya ha sido visualizado en la respuesta del request.
	};

	adm_skills_handle.prototype.set_input_default_value = function( $txt, force )
	{
		if ( !$txt || $txt.length === 0 ) {
			return;
		}
		if ( force || $txt.val().trim().length === 0 ) {
			var default_value = $txt.attr("default_value");
			default_value = parseInt( default_value, 10 );
			if ( this.get_input_ctrl_type() === "TEXTBOX" ) {
				$txt.val( default_value );
			}
			else {
				// combo
				$.each( this.combo_values, function( _, cbo_val ) {
					if ( default_value >= cbo_val.range.from && default_value <= cbo_val.range.to ) {
						default_value = cbo_val.value;
						return false; // break
					}
				});
				$txt.val( default_value );
			}
		}
	};

	adm_skills_handle.prototype.set_events = function()
	{
		var self = this;

		var event_selector = "input[selector='1'][type='checkbox']";
		$("#" + this.list_id)
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $chk = $(this);
				self.show_or_hide_skill_value( $chk );
				var checked = $chk.prop("checked");
				var disabled = self.parent_frm.is_readonly() ? true : !checked;
				if ( !disabled && checked ) {
					var $skill = $("#" + self.get_opt_input_ctrl_id( $chk.attr("skill_id") ));
					self.set_input_default_value( $skill );
					$skill.focus();
				}
			});
	};

	adm_skills_handle.prototype.get = function()
	{
		var self = this;
		var assigned_skills = [];

		$("#" + this.list_id + " input[selector='1'][type='checkbox']").each(function( _, skill )
		{
			var $chk = $(skill);
			var checked = $chk.prop("checked");
			if ( checked ) {
				var skill_id = $chk.attr("skill_id");
				var $skill = $("#" + self.get_opt_input_ctrl_id( skill_id ));
				var $chk_req = $("#" + self.get_opt_required_id( skill_id ));
				var assigned_skill = {
					skill:  skill_id,
					weight: $skill.val()
				};
				if ( $chk_req.length > 0 ) {
					assigned_skill.required = $chk_req.prop("checked");
				}
				assigned_skills.push( assigned_skill );
			}
		});

		return assigned_skills;
	};

	adm_skills_handle.prototype.set = function( assigned_skills )
	{
		var self = this;
		if ( !assigned_skills ) {
			return;
		}
		if ( this.request_status === "NONE" || this.request_status === "RUNNING" ) {
		// estos retries son para el set de datos cuando se edita en el frm de accounts
		//  ya que el DOM de los skills quizas aun no se haya generado ( se genera en la respuesta del request )
			if ( this.count_setting_retries <= this.max_setting_retries ) {
				this.count_setting_retries++;
				window.setTimeout(function() {
					self.set( assigned_skills );
				}, 500);
			}
			return;
		}
		else if ( this.request_status === "COMPLETED" ) {
			var get = $.isArray( assigned_skills ) ?
				function( data ) {
				// esta funcion sirve para setear los assigned_skills de la account ya que aca se guarda en un array
					return { id: data.skill, value: data.weight, required: data.required };
				}
				:
				function( data, _ ) {
				// esta funcion sirve para setear los assigned_skills del frm de users ya que aca se guarda en atributos
					return { id: _, value: data };
				}
			;

			var checked = true;
			$.each(assigned_skills, function( _, data )
			{
				var skill = get(data, _);
				var chk_id = self.get_opt_id(skill.id);
				var ctrl_id = self.get_opt_input_ctrl_id(skill.id);
				var $chk = $("#" + chk_id);
				$chk.prop("checked", checked);
				self.show_or_hide_skill_value( $chk );
				$("#" + ctrl_id).val(skill.value);
				var $chk_req = $("#" + self.get_opt_required_id(skill.id));
				if ( $chk_req.length > 0 ) {
					$chk_req.prop("checked", skill.required);
				}
			});
		}
		// El otro estado es "ERROR" pero no hace nada con el, ya que el mensaje de error ya ha sido visualizado en la respuesta del request.
	};

	adm_skills_handle.prototype.show_or_hide_skill_value = function( $chk )
	{
		if ( !$chk || $chk.length === 0 ) {
			return;
		}
		var checked = $chk.prop("checked");
		var $value_container = $chk.parent().find("[skill_value_container]");
		if ( checked ) {
			$value_container.show();
		}
		else {
			$value_container.hide();
		}
		var $skill = $("#" + this.get_opt_input_ctrl_id( $chk.attr("skill_id") ));
		var disabled = this.parent_frm.is_readonly() ? true : !checked;
		$skill.prop("disabled", disabled);
	};

	adm_skills_handle.prototype.validate = function()
	{
		var self = this;
		var response = { success: true };

		if ( !this.is_visible_container() ) {
			return response;
		}

		var validate_field = function( $skill, type, msg_err, options ) {
			// required validacion
			var value = $skill.val();
			var result = self.parent_frm.valid( value, type, options );
			if ( !result.success ) {
				result.$ctrl = $("#" + self.container_id); // este elemento es utilizado unicamente en la configuracion de accounts, ya que en el frm de usuario, no es necesario.
				result.message = msg_err;
				$skill.focus();
			}
			return result;
		};

		var type_ctrl_value = this.get_input_ctrl_type();
		var range = { between: { from: 1, to: 10} }; // range skill value validation

		$("#" + this.list_id + " input[selector='1'][type='checkbox']:checked").each(function( _, skill )
		{
			var skill_id = $(skill).attr("skill_id");
			var $skill = $("#" + self.get_opt_input_ctrl_id( skill_id ) );

			// required validacion
			response = validate_field( $skill, "required", format("<%=_T('{0} Skill value is required.')%>", skill_id) );
			if ( !response.success ) {
				return false; // break
			}
			if ( type_ctrl_value === "TEXTBOX" ) {
				// numeric validacion
				response = validate_field( $skill, "digits", format("<%=_T('{0} Skill value is not a number.')%>", skill_id) );
				if ( !response.success ) {
					return false; // break
				}
				// range validacion
				response = validate_field( $skill, "range", format("<%=_T('{0} Skill value out of range. Please enter Skill value between {1} and {2}. ')%>", skill_id, range.between.from, range.between.to ), range );
				if ( !response.success ) {
					return false; // break
				}
			}
		});

		return response;
	};
}());
