/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, -W014, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if ( !this.inConcert ) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

/**
 * @namespace
 * @name GoogleplayAccountPanel
 * @author Sebastián Moreno (smoreno@inconcertcc.com)
 */
(function () {

	'use strict';

	var format = inConcert.Utils.format;

	/**
	 * Creates a new Googleplay account panel instance.
	 *
	 * @constructor
	 * @constructor
	 * @memberOf GoogleplayAccountPanel
	 */
	function GoogleplayAccountPanel(params){
		this.prefix_id = "adm_form_campaign_accounts_googleplay_";
		GoogleplayAccountPanel.uber.constructor.call(this, params);

		this.panel_type = "googleplay";
		this.messages_class = null; // Googleplay no tiene mensajes automaticos ya que da problemas.
		this.container_id = '#adm_form_campaign_accounts_googleplay_panel';
		this.authorize_id = '#adm_form_campaign_accounts_googleplay_account_auth';
		this.authorize_img_id = '#adm_form_campaign_accounts_googleplay_account_auth_img';

		this.language = '#adm_form_campaign_accounts_googleplay_language';
		this.distribution = '#adm_form_campaign_accounts_googleplay_distribution';
		this.priority = '#adm_form_campaign_accounts_googleplay_priority';
		this.clientId = '#adm_form_campaign_accounts_googleplay_client_id';
		this.clientSecret = '#adm_form_campaign_accounts_googleplay_client_secret';
		this.packageName = '#adm_form_campaign_accounts_googleplay_package_name';
		this.username = '#adm_form_campaign_accounts_googleplay_display_name';

		this.attention_levels_container = '#adm_form_campaign_accounts_googleplay_attention_levels_config';
		this.ticket_management_container = '#adm_form_campaign_accounts_googleplay_tickets_mng_panel';
		this.contact_management_container = '#adm_form_campaign_accounts_googleplay_contact_mng_panel';
		this.thresholds_container = '#adm_form_campaign_accounts_googleplay_thresholds_panel';

		this.account_save_id = '#adm_form_campaign_accounts_save';

		this.tagline_new      = "<%=_T('New Googleplay account.')%>";
		this.tagline_edit     = "<%=_T('Edit Googleplay account {0}.')%>";
		this.tagline_readonly = "<%=_T('Googleplay account {0}.')%>";
	}

	inConcert.Utils.inherit( GoogleplayAccountPanel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.GoogleplayAccountPanel !== 'function' ) {
		inConcert.Administration.Campaigns.GoogleplayAccountPanel = GoogleplayAccountPanel;
	}

	/**
	 * Called when the form is being initialized
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.initialize = function(){
		GoogleplayAccountPanel.uber.initialize.call(this);

		// TO-DO: Find a prettier way to do this.
		$(this.account_save_id).prop('disabled', true);

		var $account_name = $( this.username );
		this.parent.frm.readonly_control( $account_name, true /* disabled */ );

		this.tabs.disable_tab( 'messages' );
	};

	GoogleplayAccountPanel.prototype.has_qna_maker       = function() {return true;};
	/*GoogleplayAccountPanel.prototype.has_flow_management = function() {return true;};
	GoogleplayAccountPanel.prototype.has_flow_inactivity = function() {return true;};*/

	/**
	 * Initializes thresholds management
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		GoogleplayAccountPanel.uber.initialize_thresholds.call( this,
			'GOOGLEPLAY',
			{
				parent    : this.parent,
				prefix_id : 'adm_form_campaign_accounts_googleplay_thresholds'
			}
		);
	};

	GoogleplayAccountPanel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		GoogleplayAccountPanel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "GOOGLEPLAY"                                                  ,
				parent       : this                                                    ,
				container_id : "adm_form_campaign_accounts_googleplay_qna_maker_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_googleplay_qna_maker_mng"
			}
		);
	};

	/**
	 * Initializes contact management
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.initialize_contact_mng = function(){
		// build contact management panel
		GoogleplayAccountPanel.uber.initialize_contact_mng.call(this,
			{
				type         : 'GOOGLEPLAY'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_googleplay_contact_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_googleplay_contact_mng'
			}
		);
	};

	GoogleplayAccountPanel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		GoogleplayAccountPanel.uber.initialize_flows_mng.call(
			this,
			{
				type         : 'googleplay_BOT'                                                    ,
				parent       : this                                                         ,
				container_id : 'adm_form_campaign_accounts_googleplay_flows_designer_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_googleplay_flows_designer_mng'
			}
		);
	};

	/**
	 * Initializes ticket management
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.initialize_ticket_emitter = function(){
		// build ticket management section
		GoogleplayAccountPanel.uber.initialize_ticket_emitter.call(this,
			{
				type         : 'GOOGLEPLAY'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_googleplay_tickets_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_googleplay_tickets_mng'
			}
		);
	};

	/**
	 * Toggle every control, used from init and edit mode.
	 *
	 * @param  {Boolean} on_or_off - Toggle or not.
	 */
	GoogleplayAccountPanel.prototype.__toggle_controls = function(on_or_off)
	{
		// Los divs que contienen el attr 'row_toggle', se ocultan/muestran cuando la app es autorizada!!
		$(this.container_id).find("[row_toggle]").toggle(on_or_off);
		$(this.account_save_id).prop('disabled', !on_or_off);
		$(this.authorize_id).val(!on_or_off ? "<%=_T('Authorize Application')%>" : "<%=_T('Re-Authorize Application')%>");
		$(this.authorize_img_id).toggleClass('formImageTrue', on_or_off).toggleClass('formImageFalse', !on_or_off);
	};

	/**
	 * Clears every control
	 *
	 * @todo This name is wrong, must be "clear controls" or something.
	 * @public
	 */
	GoogleplayAccountPanel.prototype.empty_controls = function() {
		GoogleplayAccountPanel.uber.empty_controls.call( this );
		this.__toggle_controls(false);
	};
	
	/**
	 * Called when the user enter to account edition
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.edit = function(account)
	{
		GoogleplayAccountPanel.uber.edit.call( this, account );
		if ( !this.current_account ) {
			return;
		}
		this.m_client_id = this.current_account.client_id;
		this.m_client_secret = this.current_account.client_secret;
		this.m_access_token = this.current_account.access_token;
		this.m_refresh_token = this.current_account.refresh_token;
		this.m_user_id = this.current_account.user_id;
		this.m_user_name = this.current_account.user_name;
		this.m_package_name = this.current_account.uid;
		
		$(this.clientId).val(this.current_account.client_id);
		$(this.clientSecret).val(this.current_account.client_secret);
		$(this.username).val(this.current_account.user_id);
		
		let $packageName = $( this.packageName );
		this.parent.frm.readonly_control( $packageName, true /* disabled */ );
		$packageName.val(this.current_account.uid);

		this.__toggle_controls(true);
		this.tabs.enable_tab( 'messages' );
	};

	/**
	 * Method to initialize tabs (basic & messages)
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.initialize_tabs = function() {
		GoogleplayAccountPanel.uber.initialize_tabs.call(this,
			{
				parent       : this                              ,
				frm          : this.parent.frm                   ,
				container_id : 'adm_form_campaign_accounts_tabs' ,
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('Googleplay Config')%>",
						tab_container_id : 'adm_form_campaign_accounts_googleplay_panel_tab_basic',
						selected         : true
					}
				}
			}
		);
	};

	GoogleplayAccountPanel.prototype.initialize_skills = function()
	{
		GoogleplayAccountPanel.uber.initialize_skills.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_googleplay_skills_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION'
			}
		);
	};

	GoogleplayAccountPanel.prototype.initialize_attention_levels = function()
	{
		GoogleplayAccountPanel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_googleplay_attention_levels_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION',
				proxy_type   : 'GOOGLEPLAY'
			}
		);
	};

	/**
	 * Validates Googleplay account fields.
	 *
	 * @todo Validate and save should be separated actions
	 *
	 * @public
	 *
	 * @return {Object}  result
	 * @return {Boolean} result.success
	 * @return {Boolean} result.is_new
	 *
	 * @return {Object}  [result.account]
	 * @return {String}  [result.account.id]
	 * @return {String}  [result.account.proxyType]
	 * @return {String}  [result.account.name]
	 * @return {String}  [result.account.CCAlgorithm]
	 *
	 * @return {String}  [result.error_message]
	 */
	GoogleplayAccountPanel.prototype.validate = function()
	{
		var is_new = this.is_new();

		var result = {
			success: false,
			is_new: is_new
		};

		this.hide_marked_errors();

		// PACKAGE NAME - validation
		let $packageName = $(this.packageName);
		if (!this.validate_field($packageName, "required", "<%=_T('Package name is required.')%>")) {
			return result;
		}
		if (!this.validate_field($packageName, "alpha", "<%=_T('Only letters, numbers, dashes and dots are allowed in the Account Package Name.')%>")) {
			return result;
		}

		var $distribution = $(this.distribution);
		if (!this.validate_field($distribution, 'required', "<%=_T('Distribution is required.')%>")) {
			return result;
		}

		// PRIORITY - validation
		var $priority = $(this.priority);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		if (!has_att_levels && $priority.val().trim().length > 0) {
			var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
			if (!this.validate_field($priority, 'range', format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
				return result;
			}
		}

		// LANGUAGE - validation
		var $language = $(this.language);
		if (!this.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
			return result;
		}

		// valido los skills ----------
		var result_skills = this.validate_skills();
		if ( !result_skills.success ) {
			this.show_validation_error( result_skills.$ctrl, result_skills.message );
			return result;
		}

		// valido los attention levels ----------
		var result_attention_levels = this.validate_attention_levels();
		if ( !result_attention_levels.success ) {
			this.show_validation_error(
				result_attention_levels.$ctrl,
				result_attention_levels.message,
				false, //timeout_ms
				true, //scrolling
				result_attention_levels.show_custom_button
			);
			return result;
		}

		// validate ticket configuration
		var result_ticket_emitter = this.validate_ticket_emitter();
		if (!result_ticket_emitter.success) {
			this.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
			return result;
		}

		// valido los contact mng ----------
		var result_contact_mng = this.validate_contact_mng();
		if ( !result_contact_mng.success ) {
			this.show_validation_error( result_contact_mng.$ctrl, result_contact_mng.error_message );
			return result;
		}

		// valido los flows mng ----------
		var result_flows_mng = this.validate_flows_mng();
		if ( !result_flows_mng.success ) {
			this.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
			return result;
		}

		// valido los qna maker mng ----------
		var result_qna_maker_mng = this.validate_qna_maker_mng();
		if ( !result_qna_maker_mng.success ) {
			this.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
			return result;
		}

		// valido los thresholds ----------
		var result_th = this.validate_thresholds();
		if ( !result_th.success ) {
			this.show_validation_error( result_th.$ctrl, result_th.error_message );
			return result;
		}

		// valido los messages de la account ----------
		var result_messages = this.validate_messages();
		if ( !result_messages.success ) {
			this.show_validation_error( undefined, result_messages.error_message );
			return result;
		}

		// all good, now save it.
		// TO-DO: This should be separeted
		var account = {};
		if (is_new) {
			this.m_package_name = $packageName.val();
			var campaign = this.parent.frm.get_entity_instance();
			// TO-DO: Why we need to create the account id client side?
			account.id = 'GOOGLEPLAY_' + campaign.Id + '_' + this.m_package_name;
		}
		else {
			account.id = this.current_account.id;
		}

		account.proxyType          = 'GOOGLEPLAY';
		account.name               = this.m_package_name;
		account.uid                = this.m_package_name;
		account.screen_name        = this.m_package_name;
		account.user_id		       = this.m_user_id;
		account.user_name	       = this.m_user_name;
		account.client_id		   = this.m_client_id;
		account.client_secret	   = this.m_client_secret;
		account.access_token	   = this.m_access_token;
		account.refresh_token	   = this.m_refresh_token;
		account.CCAlgorithm        = $distribution.val();
		account.language           = $language.val();
		account.TicketEmitter      = result_ticket_emitter.result;
		account.Skills             = this.skills_handle.get();
		account.Priority           = $priority.val();
		account.AttentionLevels    = this.attention_levels_handle.get();
		account.GUI                = this.get_gui_config();
		account.ActionOnAcdWaitTimeout = result_th.ActionOnAcdWaitTimeout;
		account.InactivityTimeout = result_th.InactivityTimeout;
		account.ignoreAgentAssignedFromReOpen = $(this.ignoreAgent).prop("checked")? "1" : "0";
		this.contact_mng_instance.attach_data(account);
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account);
		}

		result.account = account;
		result.success = true;

		return result;
	};

	/**
	 * Called when our Googleplay app is authorized.
	 *
	 * @param  {Object} data
	 * @param  {String} data.oauth_token
	 * @param  {String} data.oauth_token_secret
	 * @param  {String} data.oauth_verifier
	 * @param  {String} data.user_id
	 * @param  {String} data.screen_name
	 */
	GoogleplayAccountPanel.prototype.__on_app_authorized = function(data, callback = $.noop) {
		this.m_client_id = data.client_id;
		this.m_client_secret = data.client_secret;
		this.m_access_token = data.access_token;
		this.m_refresh_token = data.refresh_token || this.m_refresh_token;
		this.m_user_id = data.uid;
		this.m_user_name = data.name;
		$(this.username).val(this.m_user_id);
		this.__toggle_controls(true);
		this.tabs.enable_tab( 'messages' );
		callback(true);
	};

	/**
	 * Called once the panel is initialized, we can attach buttons events here.
	 *
	 * @public
	 */
	GoogleplayAccountPanel.prototype.set_events = function()
	{
		GoogleplayAccountPanel.uber.set_events.call( this );

		let self = this;

		$(this.authorize_id).unbind('click').bind('click', function() {
			if ( !self.parent.has_permission_to_save() ) {
				var message = "<%=_T('You do not have permissions for save account.')%>";
				self.parent.frm.show_status_message( message, 'Error', 5000 );
				return;
			}
			
			self.hide_marked_errors();
			
			let $clientId = $(self.clientId);
			let $clientSecret = $(self.clientSecret);
			if (!self.validate_field($clientId, "required", "<%=_T('Client id is required.')%>")) {
				return;
			}
			if (!self.validate_field($clientSecret, "required", "<%=_T('Client secret is required.')%>")) {
				return;
			}
			
			let campaign = self.parent.frm.get_entity_instance();
			let postback_data = JSON.stringify({ 
				campaignId : campaign.Id, 
				origin: window.location.origin,
				host : window.location.host
			});
			let parameters = '?&post_data=' + encodeURIComponent(postback_data) +
				"&client_id=" + encodeURIComponent($clientId.val()) +
				"&client_secret=" + encodeURIComponent($clientSecret.val());
			let href = inConcert.app.getBaseDomain() + '/inconcert/apps/oauth/googleplay/authorize_user/' + parameters;
			let window_options = "height=600,width=800,status=yes,toolbar=yes,menubar=no,location=yes";
			let authorization_window = window.open(href, "_blank", window_options);

			if (window.GoogleplayAccountPanel_listener) {
				window.removeEventListener("message", window.GoogleplayAccountPanel_listener);
				window.GoogleplayAccountPanel_listener = undefined;
			}

			let close_popup_window = function( can_add ) {
				let timeout_ms = can_add ? 1500 : undefined;
				window.setTimeout( function() {
					authorization_window.close();
					authorization_window = undefined;
				}, timeout_ms );
			};

			let listener = function(ev) {
				if (ev.origin.search( window.location.host ) > 0 && ev.data && ev.data.authorization === "GOOGLEPLAY") {
					self.__on_app_authorized(ev.data, close_popup_window);
					window.removeEventListener("message", window.GoogleplayAccountPanel_listener);
					window.GoogleplayAccountPanel_listener = undefined;
				}
			};

			window.GoogleplayAccountPanel_listener = listener;
			window.addEventListener("message", window.GoogleplayAccountPanel_listener, false);
		});
	};

	GoogleplayAccountPanel.prototype.editable = function()
	{
		GoogleplayAccountPanel.uber.editable.call( this );
		$( this.authorize_id ).show();
	};

	GoogleplayAccountPanel.prototype.readonly = function()
	{
		GoogleplayAccountPanel.uber.readonly.call( this );
		$( this.authorize_id ).hide();
	};

	GoogleplayAccountPanel.prototype.get_account_name = function()
	{
		return this.m_package_name;
	};

}());
