/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, -W014, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, Mustache, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

/**
 * @namespace
 * @name InstagramAccountPanel
 * @author Gaston Nicolas Bagnasco Rodriguez (nbagnasco@inconcertcc.com)
 */

(function () {

	'use strict';

	var format = inConcert.Utils.format;

	var INSTAGRAM_API_URL = 'https://graph.facebook.com/v15.0/';
	var page_title_template = ('').concat(
		"<li class='formListTitle'>",
			"<span class='firstItemCheckbox'></span>" +
			"<label class='formListTitleText nameColumn'>" + "<%=_T('Name')%>" + "</label>",
			"<label class='gridAction1ButtonCol'></label>",
		"</li>"
	);
	var page_node_template = ('').concat(
		"<li class='formListItem' >",
			'<input checkbutton id="adm_form_campaign_accounts_instagram_page_{{ id }}" page_id="{{ id }}" class="formListCheck" type="checkbox" checked/>',
			"<div class='formItemText'>",
				"<label for='adm_form_campaign_accounts_instagram_page_{{ id }}'>{{ name }}</label>",
			"</div>",
			'<button messenger_button page_id="{{ id }}" page_name="{{ name }}" id="adm_form_campaign_accounts_instagram_page_{{ id }}_messenger" class="buttonColorGray gridActionButton last" title="' + '<%=_T("Messenger bot")%>' + '" >',
				'<i class="far fa-robot"></i>',
				'<%=_T("Messenger bot")%>',
			'</button>',
		"</li>"
	);

	/**
	 * Creates a new Instagram account panel instance.
	 *
	 * @constructor
	 * @constructor
	 * @memberOf InstagramAccountPanel
	 */
	function InstagramAccountPanel( params ) {
		this.prefix_id = "adm_form_campaign_accounts_instagram_";
		InstagramAccountPanel.uber.constructor.call( this, params );

		this.panel_type = "instagram";
		this.messages_class = inConcert.adm_form_campaigns_instagram_account_messages;
		this.container_id = '#adm_form_campaign_accounts_instagram_panel';
		this.name = '#adm_form_campaign_accounts_instagram_name';
		this.authorize_id = '#adm_form_campaign_accounts_instagram_account_auth';
		this.authorize_img_id = '#adm_form_campaign_accounts_instagram_account_auth_img';

		this.language = '#adm_form_campaign_accounts_instagram_language';
		this.distribution = '#adm_form_campaign_accounts_instagram_distribution';
		this.priority = '#adm_form_campaign_accounts_instagram_priority';
		this.username = '#adm_form_campaign_accounts_instagram_display_name';

		this.check_profile = '#adm_form_campaign_accounts_instagram_check_profile';
		this.check_pages_list_container = '#adm_form_campaign_accounts_instagram_check_pages_list';
		this.check_pages_help_msg = '#adm_form_campaign_accounts_instagram_check_pages_help';
		this.check_pages_no_pages_msg = '#adm_form_campaign_accounts_instagram_check_pages_notFound';
		this.check_pages = '#adm_form_campaign_accounts_instagram_check_pages';

		this.m_pages = [];
		this.m_pages_ids = [];
		this.m_pages_by_name = {};

		this.attention_levels_container = '#adm_form_campaign_accounts_instagram_attention_levels_config';
		this.ticket_management_container = '#adm_form_campaign_accounts_instagram_tickets_mng_panel';
		this.contact_management_container = '#adm_form_campaign_accounts_instagram_contact_mng_panel';
		this.thresholds_container = '#adm_form_campaign_accounts_instagram_thresholds_panel';

		this.account_save_id  = '#adm_form_campaign_accounts_save';

		this.tagline_new      = "<%=_T('New Instagram account.')%>";
		this.tagline_edit     = "<%=_T('Edit Instagram account {0}.')%>";
		this.tagline_readonly = "<%=_T('Instagram account {0}.')%>";

		this.avoid_assign_out_of_hours_container   = "#" + this.prefix_id + "avoid_assign_out_of_hours_container";
		this.avoid_assign_out_of_hours        = "#" + this.prefix_id + "avoid_assign_out_of_hours";
	}

	inConcert.Utils.inherit( InstagramAccountPanel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.InstagramAccountPanel !== 'function' ) {
		inConcert.Administration.Campaigns.InstagramAccountPanel = InstagramAccountPanel;
	}

	InstagramAccountPanel.prototype.has_qna_maker = function() {return true;};

	InstagramAccountPanel.prototype.__process_api_result = function(result, callback) {
		var self = this;
		if (!result.paging || !result.paging.next) {
			callback(result);
			return;
		}
		$.ajax({
			url : result.paging.next,
			type : "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(res) {
				// 200 but error in the query
				if (res.error_code){
					callback();
					return;
				}
				$.each(res.data, function(_, element) {
					result.data.push(element);
				});
				res.data = result.data;
				self.__process_api_result(res, callback);
			},
			error: function(res) {
				var error;
				try {
					var result = JSON.parse(res.responseText);
					error = result.error && result.error.message || "";
				}
				catch(e) {
					error = "";
				}
				callback(false, error);
			}
		});
	};

	InstagramAccountPanel.prototype.__process_api_query = function(access_token, query, type, data, callback){
		var self = this;
		var url = INSTAGRAM_API_URL + query + (query.indexOf('?') > 0? '&' : '?') + 'access_token=' + access_token;

		// we use jquery ajax and not inConcert.invoke due to CORS (invoke adds inConcert headers and fails)
		$.ajax({
			url : url,
			type : type,
			dataType: "json",
			contentType: "application/json",
			data : data? JSON.stringify(data) : null,
			success: function(res) {
				// 200 but error in the query
				if (res.error_code){
					callback();
					return;
				}
				self.__process_api_result(res, callback);
			},
			error: function(res) {
				var error;
				try {
					var result = JSON.parse(res.responseText);
					error = result.error && result.error.message || "";
				}
				catch(e) {
					error = "";
				}
				callback(false, error);
			}
		});
	};

	InstagramAccountPanel.prototype.__get_user_pages = function( access_token, _, on_pages_retrieved )
	{
		this.__process_api_query( access_token, 'me/accounts?fields=id,name,access_token,instagram_business_account{id,name}&limit=25', 'GET', null, 
		function( pages, error ) {
			var data = pages && pages.data ? pages.data : undefined;
			on_pages_retrieved( data, error );
		});
	};

	InstagramAccountPanel.prototype.get_messenger_profile = function (access_token, callback)
	{
		var fields = 'ice_breakers';
		this.__process_api_query( access_token, 'me/messenger_profile/?fields=' + fields + "&platform=instagram", 'GET', null, function( result, error )
		{
			var data = false;
			if (result) {
				data = result.data && result.data.length && result.data[0] || {};
			}
			callback(data, error);
		});
	};

	InstagramAccountPanel.prototype.get_messenger_qr = function (access_token, size, ref, callback)
	{
		var data = {
			type : "standard",
			image_size : size || null,
			data : ref? { ref : ref } : null
		};
		this.__process_api_query( access_token, 'me/messenger_codes/', 'POST', data, function( result, error )
		{
			callback(result && result.uri || false, error);
		});
	};

	InstagramAccountPanel.prototype.update_messenger_profile = function (access_token, properties, callback)
	{
		this.__process_api_query( access_token, 'me/messenger_profile?platform=instagram', 'POST', properties, function( result, error )
		{
			callback(!!result, error);
		});
	};

	InstagramAccountPanel.prototype.delete_messenger_profile = function (access_token, properties, callback)
	{
		this.__process_api_query( access_token, 'me/messenger_profile?platform=instagram', 'DELETE', { fields : properties }, function( result, error )
		{
			callback(!!result, error);
		});
	};

	// TO-DO: Based on the previous scenario (beta) we show to the user a "Check user profile" check,
	// but at data level we use "onlyCheckPages", so for certain checks we need the reverse value, wat.
	InstagramAccountPanel.prototype.__get_reverse_checkbox_value = function(checkbox_node){
		return checkbox_node.prop('checked') ? '0' : '1';
	};

	// TO-DO: Based on the previous scenario (beta) we show to the user a "Check user profile" check,
	// but at data level we use "onlyCheckPages", so for certain checks we need the reverse value, wat.
	InstagramAccountPanel.prototype.__set_reverse_checkbox_value = function(checkbox_node, value){
		return checkbox_node.prop('checked', value === '0');
	};

	InstagramAccountPanel.prototype.__get_checked_pages = function()
	{
		var checked_pages_ids = [];
		var checked_pages_by_name = {};
		var checked_pages_by_fbId = {};

		this.m_pages.forEach(function(page){
			var $page_check = $('#adm_form_campaign_accounts_instagram_page_' + page.id);
			if ($page_check.prop('checked')){
				checked_pages_by_name[page.id] = page.name;
				checked_pages_by_fbId[page.id] = page.fb_id;
				checked_pages_ids.push(page.id.toString());
			}
		});

		return {
			by_id : checked_pages_ids,
			by_name : checked_pages_by_name,
			by_fb : checked_pages_by_fbId
		};
	};

	// TO-DO: Why we need to create the account id client side?
	InstagramAccountPanel.prototype.__build_account_id = function( campaign_id, uid ) {
		return format('INSTAGRAM_{0}_{1}', campaign_id, uid);
	};

	InstagramAccountPanel.prototype.__showMessengerPopup = function(pageId, pageName, accessToken) {
		var self = this;
		var pageData = this.m_pagesMessenger && this.m_pagesMessenger[pageId] || {};
		var popup = new inConcert.PageMessengerInstagramAdministration(self, pageId, pageName, accessToken, pageData, function(pageId, data) {
			self.__updateMessengerData(pageId, data);
		});
		popup.Show();
	};

	InstagramAccountPanel.prototype.__updateMessengerData = function(pageId, pageData) {
		this.m_pagesMessenger = this.m_pagesMessenger || {};
		this.m_pagesMessenger[pageId] = pageData;
	};

	InstagramAccountPanel.prototype.__build_pages_nodes = function()
	{
		var self = this;

		var $no_pages_msg = $( this.check_pages_no_pages_msg );
		var $message = $( this.check_pages_help_msg );
		// remove previous pages
		var $container = $( this.check_pages_list_container );
		$container.children().remove();

		var is_readonly = this.parent.is_readonly();

		this.__get_user_pages( this.m_access_token, this.m_user_uid, function( fbPages )
		{
			var message;
			
			var pages = [];
			$.each(fbPages, function(_, page) {
				if (!page.instagram_business_account) {
					return;
				}
				pages.push({
					id : page.instagram_business_account.id,
					fb_id : page.id,
					name : page.instagram_business_account.name,
					access_token : page.access_token
				});
			});

			if ( pages ) {
				if ( pages.length > 0 ){
					// update member, yeah, this sucks.
					self.m_pages = pages;

					$no_pages_msg.hide();
					$container.show();
					$container.append(page_title_template);
					
					pages.forEach(function( page ) {
						var $node = $( Mustache.render( page_node_template, {
							id   : page.id,
							name : page.name
						}));
						if (is_readonly) {
							$node.find("[messenger_button]").remove();
						}
						else {
							$node.find("[checkbutton]").change(function() {
								$("#adm_form_campaign_accounts_instagram_page_" + $(this).attr("page_id")  + "_messenger")
									.toggle($(this).is(":checked"));
							});
							$node.find("[messenger_button]")
								.data("access_token", page.access_token)
								.click(function() {
									var page =  $(this).attr("page_id");
									var pageName =  $(this).attr("page_name");
									var accessToken = $(this).data("access_token");
									if (!$("#adm_form_campaign_accounts_instagram_page_" + page).is(":checked")) {
										return;
									}
									self.__showMessengerPopup(page, pageName, accessToken);
								});
						}
						$container.append($node);
					});

					if ( self.current_account && self.current_account.pages ) {
						$container.find( '.formListCheck' ).prop( 'checked', function() {
							//check pages that we have selected in current account
							var $chk = $( this );
							self.parent.frm.readonly_control( $chk, is_readonly /* readonly */ );
							return _.contains( self.current_account.pages, $chk.attr( 'page_id' ) );
						}).change();
					}
				} else {
					$container.hide();
					$no_pages_msg.show();
				}
				message = "<%=_T('/ Select pages to be checked')%>";
			}

			$message.text( message || "<%=_T('Failed to retrieve user pages. Please retry later')%>" );
		});
	};

	/**
	 * Called when the form is being initialized
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.initialize = function()
	{
		InstagramAccountPanel.uber.initialize.call(this);

		// TO-DO: Find a prettier way to do this.
		$(this.account_save_id).prop('disabled', true);

		var $account_name = $( this.username );
		this.parent.frm.readonly_control( $account_name, true /* disabled */ );
	};

	/**
	 * Initializes thresholds management
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		InstagramAccountPanel.uber.initialize_thresholds.call( this,
			'INSTAGRAM',
			{
				parent    : this.parent,
				prefix_id : 'adm_form_campaign_accounts_instagram_thresholds'
			}
		);
	};

	/**
	 * Initializes contact management
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.initialize_contact_mng = function()
	{
		// build contact management panel
		InstagramAccountPanel.uber.initialize_contact_mng.call( this,
			{
				type         : 'INSTAGRAM',
				parent       : this,
				auto_add_type: 'BOTH', // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_instagram_contact_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_instagram_contact_mng'
			}
		);
	};

	InstagramAccountPanel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		InstagramAccountPanel.uber.initialize_flows_mng.call(
			this,
			{
				type         : 'INSTAGRAM'                                                    ,
				parent       : this                                                          ,
				container_id : 'adm_form_campaign_accounts_instagram_flows_designer_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_instagram_flows_designer_mng'
			}
		);
	};

	/**
	 * Initializes ticket management
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.initialize_ticket_emitter = function() {
		// build ticket management section
		InstagramAccountPanel.uber.initialize_ticket_emitter.call(this,
			{
				type         : 'INSTAGRAM',
				parent       : this,
				auto_add_type: 'BOTH', // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_instagram_tickets_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_instagram_tickets_mng'
			}
		);
	};

	InstagramAccountPanel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		InstagramAccountPanel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "INSTAGRAM"                                               ,
				parent       : this                                                      ,
				container_id : "adm_form_campaign_accounts_instagram_qna_maker_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_instagram_qna_maker_mng"
			}
		);
	};

	/**
	 * Toggle every control, used from init and edit mode.
	 *
	 * @param  {Boolean} on_or_off - Toggle or not.
	 */
	InstagramAccountPanel.prototype.__toggle_controls = function(on_or_off)
	{
		// Los divs que contienen el attr 'row_toggle', se ocultan/muestran cuando la app es autorizada!!
		$(this.container_id).find("[row_toggle]").toggle(on_or_off);
		$(this.account_save_id).prop('disabled', !on_or_off);
		$(this.authorize_id).text( !on_or_off ? "<%=_T('Connect with Facebook')%>" : "<%=_T('Re-connect with Facebook')%>" );
		// $(this.authorize_img_id).toggleClass('formImageTrue', on_or_off).toggleClass('formImageFalse', !on_or_off);
	};

	/**
	 * Clears every control
	 *
	 * @todo This name is wrong, must be "clear controls" or something.
	 * @public
	 */
	InstagramAccountPanel.prototype.empty_controls = function(){
		InstagramAccountPanel.uber.empty_controls.call( this );
		this.__toggle_controls(false);
	};

	/**
	 * Called when the user enter to account edition
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.edit = function(account)
	{
		InstagramAccountPanel.uber.edit.call( this, account );
		if ( !this.current_account ) {
			return;
		}
		this.m_name = this.current_account.userName;
		this.m_user_uid = this.current_account.userId;
		this.m_authorization_code = this.current_account.code;
		this.m_access_token = this.current_account.accessToken;
		this.m_pagesMessenger = this.current_account.pagesMessenger || {};
		$(this.username).val(this.current_account.userName);
		$(this.name).val(this.current_account.name);
		this.parent.frm.readonly_control($(this.name), true);
		this.__toggle_avoid_assign_out_of_hours();
		const avoid_assign_out_of_hours = this.current_account.avoidAssignOutOfSchedule === "1" ? true : false;
		$(this.avoid_assign_out_of_hours).prop("checked", avoid_assign_out_of_hours);
		this.__toggle_controls(true);
		this.__build_pages_nodes();
	};

	InstagramAccountPanel.prototype.get_account_name = function()
	{
		return this.current_account.name;
	};

	/**
	 * Method to initialize tabs (basic & messages)
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.initialize_tabs = function() {
		InstagramAccountPanel.uber.initialize_tabs.call(this,
			{
				parent       : this                              ,
				frm          : this.parent.frm                   ,
				container_id : 'adm_form_campaign_accounts_tabs' ,
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('Instagram Config')%>",
						tab_container_id : 'adm_form_campaign_accounts_instagram_panel_tab_basic',
						selected         : true
					},
					messages : {
						id               : "messages",
						name             : "<%=_T('Messages')%>",
						tab_container_id : "adm_form_campaign_accounts_instagram_panel_tab_messages" 
					}
				}
			}
		);
	};

	/**
	 * Initializes survey management
	 *
	 * @public
	 */
	 InstagramAccountPanel.prototype.initialize_survey_management = function()
	{
		InstagramAccountPanel.uber.initialize_survey_management.call(
			this,
			{
				type         : "INSTAGRAM",
				parent       : this,
				container_id : "adm_form_campaign_accounts_instagram_survey_panel",
				prefix_id    : "adm_form_campaign_accounts_instagram_survey"
			}
		);
	};

	InstagramAccountPanel.prototype.initialize_skills = function()
	{
		InstagramAccountPanel.uber.initialize_skills.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_instagram_skills_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION'
			}
		);
	};

	InstagramAccountPanel.prototype.initialize_attention_levels = function()
	{
		InstagramAccountPanel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_instagram_attention_levels_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION',
				proxy_type   : 'INSTAGRAM'
			}
		);
	};

	InstagramAccountPanel.prototype.__askForMessengerClean = function(callback) {
		callback = callback || $.noop;
		var self = this;
		
		function onConfirmation(mustSave, mustClean) {
			self.mustCleanMessenger = mustClean;
			callback(mustSave);
		}

		var removedPages = false;
		$.each(self.current_account && self.current_account.pages || [], function(_, pageId) {
			if (!$("#adm_form_campaign_accounts_instagram_page_" + pageId + ":checked").length) {
				removedPages = true;
				return false;
			}
		});
		
		if (!removedPages) {
			return onConfirmation(true, false);
		}
		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Some pages were removed. Would you like to restore messenger configuration?')%>",
			$.inconcertPopup.ButtonSets.BS_YESNOCANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "yes":
						return onConfirmation(true, true);
					case "no":
						return onConfirmation(true, false);
					default:
						return onConfirmation(false, false);
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	InstagramAccountPanel.prototype.before_saving = function(callback) {
		callback = callback || $.noop;
		var self = this;
		this.__askForMessengerClean(function(mustSave) {
			if (mustSave) {
				InstagramAccountPanel.uber.before_saving.call(self, callback);
			}
			else {
				callback(false);
			}
		});
	};

	InstagramAccountPanel.prototype.__toggle_avoid_assign_out_of_hours = function(distribution){
		distribution = distribution || $( this.distribution ).val();
		/* Si el algoritmo es circular, se muestra el check para no asignar fuera de hora.*/
		const can_show = distribution.toUpperCase() === "CIRCULAR";
		const $out_of_hours_container = $(this.avoid_assign_out_of_hours_container);
		const $out_of_hours = $(this.avoid_assign_out_of_hours);
		$out_of_hours_container[can_show? "show": "hide"]();
		$out_of_hours.prop("checked", false); //Por defecto es false
	};

	InstagramAccountPanel.prototype.on_change_distribution = function(value)
	{
		InstagramAccountPanel.uber.on_change_distribution.call(this, value);
		this.__toggle_avoid_assign_out_of_hours(value);
	};

	/**
	 * Validates Instagram account fields.
	 *
	 * @todo Validate and save should be separated actions
	 *
	 * @public
	 *
	 * @return {Object}  result
	 * @return {Boolean} result.success
	 * @return {Boolean} result.is_new
	 *
	 * @return {Object}  [result.account]
	 * @return {String}  [result.account.id]
	 * @return {String}  [result.account.proxyType]
	 * @return {String}  [result.account.name]
	 * @return {String}  [result.account.CCAlgorithm]
	 *
	 * @return {String}  [result.error_message]
	 */
	InstagramAccountPanel.prototype.validate = function()
	{
		var is_new = this.is_new();

		var result = {
			success: false,
			is_new: is_new
		};

		this.hide_marked_errors();
		// NAME - validation
		var $name = $(this.name);
		if (!this.validate_field($name, "required", "<%=_T('Account name is required.')%>")) {
			return result;
		}
		if (!this.validate_field($name, "id", "<%=_T('Only letters, numbers, dashes and dots are allowed in the Account Name.')%>")) {
			return result;
		}
		var name = $name.val();

		var $distribution = $(this.distribution);
		if (!this.validate_field($distribution, 'required', "<%=_T('Distribution is required.')%>")) {
			return result;
		}

		// PRIORITY - validation
		var $priority = $(this.priority);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		if (!has_att_levels && $priority.val().trim().length > 0) {
			var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
			if (!this.validate_field($priority, 'range', format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
				return result;
			}
		}

		// LANGUAGE - validation
		var $language = $(this.language);
		if (!this.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
			return result;
		}

		// valido los skills ----------
		var result_skills = this.validate_skills();
		if ( !result_skills.success ) {
			this.show_validation_error( result_skills.$ctrl, result_skills.message );
			return result;
		}

		// valido las paginas ----------
		var pages = this.__get_checked_pages();
		if (pages.by_id.length === 0) {
			this.show_validation_error( null, "<%=_T('You must check at least one page.')%>" );
			return result;
		}

		var pagesMessenger_d = this.m_pagesMessenger || {};
		var pagesMessenger = {};
		$.each(pages.by_id, function(_, pageId) {
			var pageMessenger = pagesMessenger_d[pageId] || undefined;
			if ($.isEmptyObject(pageMessenger) || !pageMessenger.processFlow) {
				pageMessenger = undefined;
			}
			pagesMessenger[pageId] = pageMessenger;
		});

		// valido los attention levels ----------
		var result_attention_levels = this.validate_attention_levels();
		if ( !result_attention_levels.success ) {
			this.show_validation_error(
				result_attention_levels.$ctrl,
				result_attention_levels.message,
				false, //timeout_ms
				true, //scrolling
				result_attention_levels.show_custom_button
			);
			return result;
		}

		// validate ticket configuration
		var result_ticket_emitter = this.validate_ticket_emitter();
		if (!result_ticket_emitter.success) {
			this.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
			return result;
		}

		// valido los contact mng ----------
		var result_contact_mng = this.validate_contact_mng();
		if ( !result_contact_mng.success ) {
			result.error_message = result_contact_mng.error_message;
			return result;
		}

		// valido los flows mng ----------
		var result_flows_mng = this.validate_flows_mng();
		if ( !result_flows_mng.success ) {
			this.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
			return result;
		}

		// valido los qna maker mng ----------
		var result_qna_maker_mng = this.validate_qna_maker_mng();
		if ( !result_qna_maker_mng.success ) {
			this.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
			return result;
		}

		// valido los thresholds ----------
		var result_th = this.validate_thresholds();
		if ( !result_th.success ) {
			this.show_validation_error( result_th.$ctrl, result_th.error_message );
			return result;
		}

		// valido los messages
		var result_messages = this.validate_messages();
		if (!result_messages.success) {
			this.show_validation_error(undefined, result_messages.error_message);
			return result;
		}

		// valido la configuracion de survey ----------
		var result_survey_management = this.validate_survey();
		if (!result_survey_management.success) {
			this.show_validation_error(result_survey_management.$ctrl, result_survey_management.error_message);
			return result;
		}

		// all good, now save it.
		// TO-DO: This should be separated
		//
		var campaign = this.parent.frm.get_entity_instance();
		var account = {
			id             : is_new ? this.__build_account_id(campaign.Id, name) : this.current_account.id,
			name           : name,
			uid            : is_new ? name : this.current_account.uid,
			proxyType      : 'INSTAGRAM',
			userId         : this.m_user_uid,
			userName       : this.m_name,
			CCAlgorithm    : $distribution.val(),
			language       : $language.val(),
			TicketEmitter  : result_ticket_emitter.result,
			Skills         : this.skills_handle.get(),
			Priority       : $priority.val(),
			AttentionLevels: this.attention_levels_handle.get(),
			GUI            : this.get_gui_config(),
			code           : this.m_authorization_code,
			accessToken    : this.m_access_token,
			onlyCheckPages : '1', //this.__get_reverse_checkbox_value($(this.check_profile)),
			pages          : pages.by_id,
			pagesNames     : pages.by_name,
			pagesMessenger : pagesMessenger,
			pagesFbIds     : pages.by_fb,
			authorized     : this.m_authorized,
			mustCleanMessenger : this.mustCleanMessenger,
			ActionOnAcdWaitTimeout : result_th.ActionOnAcdWaitTimeout,
			InactivityTimeout : result_th.InactivityTimeout,
			Survey             : result_survey_management.result,
			ignoreAgentAssignedFromReOpen : $(this.ignoreAgent).prop("checked")? "1" : "0",
			avoidAssignOutOfSchedule : $(this.avoid_assign_out_of_hours).prop("checked")? "1": "0"
		};

		this.contact_mng_instance.attach_data(account);
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account);
		}

		result.account = account;
		result.success = true;

		return result;
	};

	/**
	 * Called when our Instagram app is authorized.
	 *
	 * @param  {Object} data
	 * @param  {String} data.authorization_code
	 * @param  {String} data.access_token
	 * @param  {String} data.uid
	 * @param  {String} data.name
	 */
	InstagramAccountPanel.prototype.__on_app_authorized = function( data, callback = $.noop )
	{
		this.m_authorized = true;
		this.m_authorization_code = data.authorization_code;
		this.m_access_token = data.access_token;
		this.m_user_uid = data.uid;
		this.m_name = data.name;
		$( this.username ).val(data.name);
		this.__toggle_controls( true );
		this.__build_pages_nodes();
		callback(true);
	};

	InstagramAccountPanel.prototype.authorize = function()
	{
		var self = this;

		this.parent.frm.hide_messages();
		if ( !this.parent.has_permission_to_save() ) {
			var message = "<%=_T('You do not have permissions for save account.')%>";
			this.parent.frm.show_status_message( message, 'Error', 5000 );
			return;
		}

		var postback_data = JSON.stringify({ 
			origin : window.location.origin,
			host : window.location.host
		});
		var parameters = '?&post_data=' + encodeURIComponent( postback_data );
		var href = '/inconcert/apps/oauth/instagram/authorize_user/' + parameters;
		var window_options = 'height=600,width=800,status=yes,toolbar=yes,menubar=no,location=yes';
		var instagram_window = window.open( href, '_blank', window_options );

		if (window.adm_form_campaigns_section_accounts_instagram_message_listener) {
			window.removeEventListener("message", window.adm_form_campaigns_section_accounts_instagram_message_listener);
			window.adm_form_campaigns_section_accounts_instagram_message_listener = undefined;
		}
		
		var close_popup_window = function( can_add ) {
			var timeout_ms = can_add ? 1500 : undefined;
			window.setTimeout( function() {
				instagram_window.close();
				instagram_window = undefined;
			}, timeout_ms );
		};

		var listener = function( ev )
		{
			if (ev.origin.search( window.location.host ) > 0 && ev.data && ev.data.authorization === "INSTAGRAM")
			{
				self.__on_app_authorized( ev.data, close_popup_window );
				window.removeEventListener( 'message', window.adm_form_campaigns_section_accounts_instagram_message_listener );
				window.adm_form_campaigns_section_accounts_instagram_message_listener = undefined;
			}
		};

		window.adm_form_campaigns_section_accounts_instagram_message_listener = listener;
		window.addEventListener( 'message', window.adm_form_campaigns_section_accounts_instagram_message_listener, false );
	};

	/**
	 * Called once the panel is initialized, we can attach buttons events here.
	 *
	 * @public
	 */
	InstagramAccountPanel.prototype.set_events = function()
	{
		var self = this;
		InstagramAccountPanel.uber.set_events.call( this );

		$( this.authorize_id )
			.unbind('click')
			.bind('click', function() {
				self.authorize();
			});
	};

	InstagramAccountPanel.prototype.editable = function()
	{
		InstagramAccountPanel.uber.editable.call( this );
		$( this.authorize_id ).show();
	};

	InstagramAccountPanel.prototype.readonly = function()
	{
		InstagramAccountPanel.uber.readonly.call( this );
		$( this.authorize_id ).hide();
	};
	
	InstagramAccountPanel.prototype.on_saved = function(new_account, current_account, callback = $.noop)
	{
		InstagramAccountPanel.uber.on_saved.call(this, new_account, current_account, callback);

		var self = this;

		if ( new_account.Survey ) {
			var project_id = new_account.Survey.id;
			var token = new_account.Survey.Token;

			if ( !project_id ) { return; }

			var tweety = inConcert.app.getTweetyAPI();

			return tweety.set_authfilter( project_id, token,
				function( success )
				{
					if( !success ) {
						var error_message = "<%=_T('Error setting auth filter to survey.')%>";
						var timeout_ms = 3000;
						self.parent.frm.show_status_message( error_message, "Error", timeout_ms );
					}
				}
			);
		}
	};
}());
