/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, -W014, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if (!this.inConcert.Administration) {
	this.inConcert.Administration = {};
}

if (!this.inConcert.Administration.Campaigns) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	const format = inConcert.Utils.format;

	//------------------------------------------------------------------

	/**
	* Creates a new Mail account panel instance.
	*
	* @constructor
	* @memberOf mail_account_panel
	*/
	function MailAccountPanel( params )
	{
		this.prefix_id = "adm_form_campaign_accounts_mail_";
		var success = MailAccountPanel.uber.constructor.call( this, params );
		if ( !success ) {
			return;
		}
		this.panel_type         = "mail";
		this.container_id       = "#" + this.prefix_id + "panel";
		this.distribution       = "#" + this.prefix_id + "distribution";
		this.language           = "#" + this.prefix_id + "language";
		this.priority           = "#" + this.prefix_id + "priority";
		this.download_emails_out_of_hours = "#" + this.prefix_id + "download_emails_out_of_hours";
		this.signature          = "#" + this.prefix_id + "signature";
		this.font_family        = "#" + this.prefix_id + "font_family";
		this.font_size          = "#" + this.prefix_id + "font_size";
		this.font_color_enabled = "#" + this.prefix_id + "font_color_enabled";
		this.font_color         = "#" + this.prefix_id + "font_color";
		this.font_sample        = "#" + this.prefix_id + "font_sample";
		this.avoid_assign_out_of_hours_container   = "#" + this.prefix_id + "avoid_assign_out_of_hours_container";
		this.avoid_assign_out_of_hours        = "#" + this.prefix_id + "avoid_assign_out_of_hours";
		this.mail_hnd = new inConcert.Administration.MailAccountHnd({
			owner     : this, 
			parent    : params.section,
			prefix_id : this.prefix_id,
			validate_name_before_incomming_section: false
		});

		this.messages_class = inConcert.adm_form_campaigns_mail_account_messages;

		this.previous_font_color = null;

		return true;
	}

	inConcert.Utils.inherit( MailAccountPanel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.MailAccountPanel !== "function" ) {
		inConcert.Administration.Campaigns.MailAccountPanel = MailAccountPanel;
	}

	MailAccountPanel.prototype.initialize = function()
	{
		this.previous_font_color = null;
		var $font_color_enabled = $(this.font_color_enabled);
		$font_color_enabled.prop("checked", false);
		// Creo el color picker de spectrum
		this.parent.frm.create_color_pickers("adm_form_campaign_accounts_mail_apparance_container");

		var $font_sample = $(this.font_sample);
		this.parent.frm.readonly_control( $font_sample, true /* disabled */ );

		MailAccountPanel.uber.initialize.call( this );
		this.mail_hnd.initialize();
	};

	MailAccountPanel.prototype.has_qna_maker       = function() {return true;};
	MailAccountPanel.prototype.has_flow_management = function() {return true;};
	MailAccountPanel.prototype.has_flow_inactivity = function() {return true;};
	MailAccountPanel.prototype.get_flow_inactivity_time = function() { 
		return {
			maxHours : 15 * 24,
			disclaimer : "<%=_T('Interaction will be closed after configured inactivity time (max: 15 days).')%>",
			error : "<%=_T('Inactivity time must be a number and less than 15 days.')%>"
		};
	};

	MailAccountPanel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		MailAccountPanel.uber.initialize_thresholds.call(
			this,
			"MAIL",
			{
				parent    : this.parent,
				prefix_id : this.prefix_id + "thresholds"
			}
		);
	};

	MailAccountPanel.prototype.initialize_contact_mng = function()
	{
		// build contact management panel
		MailAccountPanel.uber.initialize_contact_mng.call(
			this,
			{
				type         : "MAIL"                                              ,
				parent       : this                                                ,
				auto_add_type: "BOTH"                                              , // incoming y outgoing
				container_id : this.prefix_id + "contact_mng_panel" ,
				prefix_id    : this.prefix_id + "contact_mng"
			}
		);
	};

	MailAccountPanel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		MailAccountPanel.uber.initialize_flows_mng.call(
			this,
			{
				type         : "MAIL"                                                    ,
				parent       : this                                                      ,
				container_id : "adm_form_campaign_accounts_mail_flows_designer_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_mail_flows_designer_mng"
			}
		);
	};

	MailAccountPanel.prototype.initialize_ticket_emitter = function()
	{
		MailAccountPanel.uber.initialize_ticket_emitter.call(
			this,
			{
				type         : "MAIL",
				parent       : this,
				container_id : this.prefix_id + "tickets_mng_panel",
				prefix_id    : this.prefix_id + "tickets_mng"
			}
		);
	};

	MailAccountPanel.prototype.initialize_survey_management = function()
	{
		MailAccountPanel.uber.initialize_survey_management.call(
			this,
			{
				type         : "MAIL",
				parent       : this,
				container_id : this.prefix_id + "survey_panel",
				prefix_id    : this.prefix_id + "survey"
			}
		);
	};

	MailAccountPanel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		MailAccountPanel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "MAIL"                                ,
				parent       : this                                  ,
				container_id : this.prefix_id + "qna_maker_mng_panel",
				prefix_id    : this.prefix_id + "qna_maker_mng"
			}
		);
	};

	MailAccountPanel.prototype.initialize_tabs = function()
	{
		// build tabs
		MailAccountPanel.uber.initialize_tabs.call(
			this,
			{
				parent       : this                              ,
				frm          : this.parent.frm                   ,
				container_id : "adm_form_campaign_accounts_tabs" ,
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('E-mail Config')%>"           ,
						tab_container_id : this.prefix_id + "panel_tab_basic",
						selected         : true
					},
					messages : {
						id               : "messages",
						name             : "<%=_T('Messages')%>",
						tab_container_id : this.prefix_id + "panel_tab_messages"
					}
				}
			}
		);
	};

	MailAccountPanel.prototype.initialize_skills = function()
	{
		MailAccountPanel.uber.initialize_skills.call(
			this,
			{
				container_id : "adm_form_campaign_accounts_mail_skills_config",
				parent_frm   : this.parent,
				parent_type  : "FORM-SECTION"
			}
		);
	};

	MailAccountPanel.prototype.initialize_attention_levels = function()
	{
		MailAccountPanel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : "adm_form_campaign_accounts_mail_attention_levels_config",
				parent_frm   : this.parent,
				parent_type  : "FORM-SECTION",
				proxy_type   : "MAIL"
			}
		);
	};

	MailAccountPanel.prototype.readonly = function()
	{
		MailAccountPanel.uber.readonly.call( this );
		this.mail_hnd.readonly();
		this.parent.frm.readonly_control( $( this.distribution ) , true /* disabled */ );
		this.__enable_font_color(false);
	};

	MailAccountPanel.prototype.show = function( account )
	{
		this.mail_hnd.set_account( account );
		MailAccountPanel.uber.show.call( this, account );
	};

	MailAccountPanel.prototype.get_account_name = function()
	{
		return this.mail_hnd.get_account_name();
	};

	MailAccountPanel.prototype.set_font_sample = function(font)
	{
		if (!font) {
			return;
		}
		var default_font = inConcert.app.getDefaultFont();
		var font_family = font.family || default_font.family.id;
		var font_size = font.size || default_font.size;
		var font_color = font.color || default_font.color;

		var $font_sample = $(this.font_sample);
		$font_sample.css("font-family", font_family);
		$font_sample.css("font-size", font_size + "px");
		$font_sample.css("color", font_color);
	};

	MailAccountPanel.prototype.set_font = function(acc)
	{
		var font = {};
		if (acc && typeof acc.Font === "object") {
			font.family = acc.Font.family;
			font.size = acc.Font.size;
			font.color = acc.Font.color;
		}
		$(this.font_family).val(font.family || "");
		$(this.font_size).val(font.size || "");

		var $font_color_enabled = $( this.font_color_enabled );
		var $font_color = $(this.font_color);
		if (font.color) {
			this.__enable_font_color(true);
			$font_color.spectrum("set", font.color);
			$font_color_enabled.prop("checked", true);
		}
		else {
			$font_color_enabled.prop("checked", false);
			this.__enable_font_color(false);
		}
		this.set_font_sample(font);
	};

	MailAccountPanel.prototype.edit = function()
	{
		var acc = this.mail_hnd.edit();
		MailAccountPanel.uber.edit.call( this, acc );
		this.set_signature(acc);
		this.set_font(acc);
		const download_emails_out_of_hours = !acc.hasOwnProperty("downloadEmailsOutOfHour") || acc.downloadEmailsOutOfHour === "1" ? true : false;
		$(this.download_emails_out_of_hours).prop("checked", download_emails_out_of_hours).trigger("change");

		const avoid_assign_emails_out_of_hours = acc.hasOwnProperty("avoidAssignOutOfSchedule") && acc.avoidAssignOutOfSchedule === "1" ? true : false;
		$(this.avoid_assign_out_of_hours).prop("checked", avoid_assign_emails_out_of_hours);
	};

	MailAccountPanel.prototype.empty_controls = function()
	{
		MailAccountPanel.uber.empty_controls.call( this );
		this.mail_hnd.empty_controls();
		this.set_font();
		$(this.download_emails_out_of_hours).prop("checked", true).trigger("change");
	};

	MailAccountPanel.prototype.set_events = function()
	{
		const self = this;

		MailAccountPanel.uber.set_events.call( this );
		this.mail_hnd.set_events();

		var $font_family = $( this.font_family );
		var $font_size = $( this.font_size );
		var $font_color = $( this.font_color );
		var $font_color_enabled = $( this.font_color_enabled );

		var __set_font_sample = function()
		{
			var color = $font_color_enabled.prop("checked") ? $font_color.val() : "";
			self.set_font_sample({
				family: $font_family.val(),
				size  : $font_size.val(),
				color : color
			});
		};

		// cambia el font family del sample.
		$font_family
			.off( "change" )
			.on({
				change: function() {
					__set_font_sample();
				}
			});

		// cambia el font size del sample.
		$font_size
			.off( "change" )
			.on({
				change: function() {
					__set_font_sample();
				}
			});

		// cambia el font color del sample.
		$font_color
			.off( "change" )
			.on({
				change: function() {
					__set_font_sample();
				}
			});

		// cambia enabled / disable el color picker y cambia el font color del sample.
		$font_color_enabled
			.off( "change" )
			.on({
				change: function() {
					var enable = $font_color_enabled.prop("checked");
					self.__enable_font_color(enable);
					__set_font_sample();
				}
			});

		// Si modifica el estado, debe actualizar el combo fuera de hora de la configuracion de flujos.
		$(this.download_emails_out_of_hours)
			.off("change")
			.on({
				change: function() {
					const $flows_process_out_of_schedule = $("#" + self.prefix_id + "flows_designer_mng_flows_process_out_of_schedule");
					const checked = $(this).prop("checked");
					if (checked) {
						self.parent.frm.disable_control($flows_process_out_of_schedule, false);
					}
					else {
						$flows_process_out_of_schedule.val("");
						self.parent.frm.disable_control($flows_process_out_of_schedule, true);
					}

					self.__toggle_avoid_assign_emails_out_of_hours(undefined, checked);
				}
			});
	};

	MailAccountPanel.prototype.__enable_font_color = function(enable)
	{
		if (this.parent.is_readonly()) {
			enable = false;
		}
		var $font_color = $( this.font_color );
		if (enable) {
			this.parent.frm.disable_control($font_color, false);
			if (this.previous_font_color) {
				$font_color.spectrum("set", this.previous_font_color);
			}
		}
		else {
			var color = $font_color.spectrum("get");
			this.previous_font_color = color.toHexString();
			var default_font = inConcert.app.getDefaultFont();
			$font_color.spectrum("set", default_font.color);
			this.parent.frm.disable_control($font_color, true);
		}
	};

	MailAccountPanel.prototype.on_show = function()
	{
		MailAccountPanel.uber.on_show.call(this);
		this.mail_hnd.on_show();
	};

	MailAccountPanel.prototype.on_saved = function(new_account, current_account, callback = $.noop)
	{
		MailAccountPanel.uber.on_saved.call(this, new_account, current_account, callback);

		var self = this;

		if ( new_account.Survey ) {
			var project_id = new_account.Survey.id;
			var token = new_account.Survey.Token;

			if ( !project_id ) { return; }

			var tweety = inConcert.app.getTweetyAPI();

			return tweety.set_authfilter( project_id, token,
				function( success )
				{
					if( !success ) {
						var error_message = "<%=_T('Error setting auth filter to survey.')%>";
						var timeout_ms = 3000;
						self.parent.frm.show_status_message( error_message, "Error", timeout_ms );
					}
				}
			);
		}
	};

	MailAccountPanel.prototype.select_tab_panel = function( panel_id, on_selected )
	{
		this.mail_hnd.select_tab_panel( panel_id, on_selected );
	};

	MailAccountPanel.prototype.get_signature = function()
	{
		var $signature = $(this.signature);
		var selected_signature = $signature.val();
		var signature = "";
		if ( selected_signature.length > 0 ) {
			signature = selected_signature;
		}
		return signature;
	};

	MailAccountPanel.prototype.set_signature = function( acc )
	{
		if (!acc) {
			return;
		}
		var $signature = $(this.signature);
		if ( acc.Signature && acc.Signature.length > 0 ) {
			$signature.val(acc.Signature);
		}
		else {
			$signature.val("");
		}
	};

	MailAccountPanel.prototype.__toggle_avoid_assign_emails_out_of_hours = function(distribution, download_emails_out_of_hours){
		distribution = distribution || $( this.distribution ).val();
		download_emails_out_of_hours = download_emails_out_of_hours || $(this.download_emails_out_of_hours).prop("checked");
		
		/* Si el algoritmo es circular y esta marcada el poder descargar correos fuera de hora,
			se muestra el check para no asignar fuera de hora.*/
		const can_show = distribution.toUpperCase() === "CIRCULAR" && download_emails_out_of_hours;
		const $out_of_hours_container = $(this.avoid_assign_out_of_hours_container);
		const $out_of_hours = $(this.avoid_assign_out_of_hours);
		$out_of_hours_container[can_show? "show": "hide"]();
		$out_of_hours.prop("checked", false); //Por defecto es false
	};

	MailAccountPanel.prototype.on_change_distribution = function(value)
	{
		MailAccountPanel.uber.on_change_distribution.call(this, value);
		this.__toggle_avoid_assign_emails_out_of_hours(value);
	};

	MailAccountPanel.prototype.validate = function()
	{
		var self = this;

		var $distribution = $( this.distribution );
		var $language = $(this.language);
		var $priority = $(this.priority);
		const $download_emails_out_of_hours = $(this.download_emails_out_of_hours);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		var result_survey_management;
		var result_ticket_emitter;

		var middle_validation = function()
		{
			var result = { success: false };
			// DISTRIBUTION - validation
			var callback = function() { self.mail_hnd.select_tab_panel( "incoming" ); };
			if ( !self.mail_hnd.validate_field( $distribution, "required", "<%=_T('Distribution is required.')%>", undefined, callback )) {
				return result;
			}

			// PRIORITY - validation
			if (!has_att_levels && $priority.val().trim().length > 0) {
				var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
				if (!self.validate_field($priority, "range", format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
					return result;
				}
			}

			result.success = true;
			return result;
		};

		var last_validation = function()
		{
			var result = { success: false };

			// LANGUAGE - validation
			if ( !self.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
				return result;
			}

		// 3.1 - valido los skills ----------
		//
			var result_skills = self.validate_skills();
			if ( !result_skills.success ) {
				self.show_validation_error( result_skills.$ctrl, result_skills.message );
				return result;
			}

		// 3.2 - valido los attention levels ----------
		//
			var result_attention_levels = self.validate_attention_levels();
			if ( !result_attention_levels.success ) {
				self.show_validation_error(
					result_attention_levels.$ctrl,
					result_attention_levels.message,
					false, //timeout_ms
					true, //scrolling
					result_attention_levels.show_custom_button
				);
				return result;
			}


		// 4 - valido la configuracion de tickets
		//
			result_ticket_emitter = self.validate_ticket_emitter();
			if ( !result_ticket_emitter.success ) {
				self.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
				return result;
			}

		// 5 - valido los contact mng ----------
		//
			var result_contact_mng = self.validate_contact_mng();
			if ( !result_contact_mng.success ) {
				self.show_validation_error( result_contact_mng.$ctrl, result_contact_mng.error_message );
				return result;
			}

		// 6 valido los flows mng ----------
		//
			var result_flows_mng = self.validate_flows_mng();
			if ( !result_flows_mng.success ) {
				self.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
				return result;
			}

		// 7 - valido la configuracion de survey
		//
			result_survey_management = self.validate_survey();
			if ( !result_survey_management.success ) {
				self.show_validation_error( result_survey_management.$ctrl, result_survey_management.error_message );
				return result;
			}

		// 7.1 valido los qna maker mng ----------
			var result_qna_maker_mng = self.validate_qna_maker_mng();
			if ( !result_qna_maker_mng.success ) {
				self.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
				return result;
			}

		// 8 - valido los thresholds ----------
		//
			var result_th = self.validate_thresholds();
			if ( !result_th.success ) {
				self.show_validation_error( result_th.$ctrl, result_th.error_message );
				return result;
			}
			result.ActionOnAcdWaitTimeout = result_th.ActionOnAcdWaitTimeout;
			result.InactivityTimeout = result_th.InactivityTimeout;

		// 9 - valido los messages de la account ----------
		//
			var result_messages = self.validate_messages();
			if ( !result_messages.success ) {
			// error en "messages" tab
				self.show_validation_error( undefined, result_messages.error_message );
				return result;
			}

			result.success = true;
			return result;
		};

		var account_info = this.mail_hnd.validate( last_validation, middle_validation );

		if ( !account_info.success ) {
			return account_info;
		}

		var font_color = ""; // si no ha definido color se alamacena un string vacio.
		if ($(this.font_color_enabled).prop("checked")) {
			var color = $(this.font_color).spectrum("get");
			font_color = color.toHexString();
		}

		account_info.account.CCAlgorithm     = $distribution.val();
		account_info.account.language        = $language.val();
		account_info.account.downloadEmailsOutOfHour = $download_emails_out_of_hours.prop("checked") ? "1" : "0";
		account_info.account.TicketEmitter   = result_ticket_emitter.result;
		account_info.account.Survey          = result_survey_management.result;
		account_info.account.Skills          = this.skills_handle.get();
		account_info.account.Priority        = $priority.val();
		account_info.account.Signature       = this.get_signature();
		account_info.account.AttentionLevels = this.attention_levels_handle.get();
		account_info.account.GUI             = this.get_gui_config();
		account_info.account.ignoreAgentAssignedFromReOpen = $(this.ignoreAgent).prop("checked")? "1" : "0";
		account_info.account.avoidAssignOutOfSchedule = $(this.avoid_assign_out_of_hours).prop("checked")? "1": "0";

		account_info.account.Font            = {
			family : $(this.font_family).val(),
			size   : $(this.font_size).val(),
			color  : font_color
		};

		this.contact_mng_instance.attach_data( account_info.account );
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account_info.account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account_info.account);
		}

		return account_info;
	};

}());
