/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, -W083, esversion: 11 */

/* global $, inConcert, CInconcertPlayer*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//"use strict";
	const format = inConcert.Utils.format;

	// Check file size. Do not upload files we know will fail.
	// 64 MB is what the cookbook uses. Needs a knob.
	//
	//
	// Tamanio maximo de los archivos a subir
	const MAX_FILE_UPLOAD_SIZE_MB = 5;
	const MAX_FILE_UPLOAD_SIZE_BYTES = MAX_FILE_UPLOAD_SIZE_MB * 1024 * 1024;

	const get_chk_bool_to_string = function( chk_id ) {
		var $chk = $( "#" + chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

/*
----------------------------------------------------------------------------------------------
  Clase BASE 'account_config'
----------------------------------------------------------------------------------------------
*/
	function account_config( params )
	{
		var self = this;

		this.parent = params.parent;
		this.frm = params.frm;

		// Este objeto contiene la configuracion de los archivos subidos.
		// el mismo es cargado de forma asincronica cuando se edita una configuracion.
		this.m_uploadedFiles = {};
		// Este objeto contiene la configuracion de los archivos subidos, antes de presionar el boton "save".
		// Si se presiona el boton cancel, se debe recorrer este objeto e ir eliminando del repo todos los archivos
		// que aqui se encuentren.
		this.recently_uploaded_files = {};
		// 
		this.m_player = null;

		this.create_fields(function(success) {
			self.on_created_fields(success);
		});
	}

	account_config.prototype.add_message       = $.noop;
	account_config.prototype.set               = $.noop;
	account_config.prototype.on_created_fields = $.noop;

	account_config.prototype.make_control_id = function( message ) { return this.type + "_" + message.id; };

	account_config.prototype.supportsCompositeFilesMessages = function()
	{
		const is_allegro_campaign = this.frm.is_allegro_campaign();
		if (!is_allegro_campaign) {
			// VCC/Campana CON Integracion con Allegro (NO soporta reproducciones compuestas)
			//
			return true;
		}
		return false;
	};

	// De momento no realiza nada, pero queda disponible por si se necesita realizar algo especifico en el Tab
	account_config.prototype.readonly = function()
	{
		var selector;
		if ( this.section_id ) {
			selector = " #" + this.section_id;
		}
		this.frm.readonly_all_crtls( selector );

		if (this.mail_mng) {
			$.each(this.mail_mng, function(_, mail_mng) {
				mail_mng.readonly();
			});
		}
	};

	account_config.prototype.editable = function()
	{
		var self = this;
		if ( !this.messages ) {
			return;
		}
		$.each( this.messages, function( i, message_config )
		{
			if ( typeof message_config.editable !== "function" ) {
				return true; // continue
			}
			var control_id = self.make_control_id( message_config );
			message_config.editable( message_config, control_id );
		});
	};

	/*
		Esta funcion carga la configuracion que se ha obtenido de la BD en los controles genericos.
	*/
	account_config.prototype.load_current_config = function()
	{
		const self = this;
		if ( !self.config || !self.messages ) {
			return;
		}
		let supportsCompositeFilesMessages = this.supportsCompositeFilesMessages();

		$.each( this.messages, function( i, message_config ) {
			var config = self.config[message_config.id];
			var control_id = self.make_control_id( message_config );
			var $field = $("#" + control_id);
			if (!$field) {
				return true; // continue
			}
			var $chk = $field.find("#chk" + control_id);
			$chk.prop("checked", config ? true : false);
			$chk.change();
			if (!config) {
				return true; // continue
			}

			if ( typeof message_config.set === "function" ) {
				message_config.set( config );
				return true; // continue
			}

			if(message_config && message_config.finalizeInteraction === "-1") {
				$( "#" + control_id + "_finalizeInteraction" ).prop("checked", true );
				$( "#" + control_id + "_finalizeInteraction" ).attr("disabled", true );
			}
			else {
				if (config && config.finalizeInteraction) {
					$( "#" + control_id + "_finalizeInteraction" ).prop( "checked", config.finalizeInteraction === "1" );
				}
			}

			if (config && config.type) {
				$field.find("#cbo" + control_id).val(config.type);
				$field.find("#cbo" + control_id + " option[value='" + config.type + "']").attr("selected", "selected").change();

				if (config.type.toLowerCase() === "file") {
					if (supportsCompositeFilesMessages) {
					// VCC/Campana SIN Integracion con Allegro (SI soporta reproducciones compuestas)
					//
						let grid_id = self.make_control_id(message_config);
						if (self.m_grdAudioMessages[grid_id]) {
							let items = [];
							if (config.isComposite) {
								(config.options || []).forEach(info => {
									let can_add_row = true;
									let item = info;
									item.RawData = {};
									if (item.MessageType === "audio") {
										item.RawData.RepositoryId = info.RepositoryId || info.fileInformation.id;
										item.RawData.Section = info.Section || info.fileInformation.section;
										item.RawData.ContentType = info.ContentType || info.fileInformation.contentType;
										item.RawData.Format = info.Format || info.fileInformation.format;
										item.RawData.Name = info.Name || info.fileInformation.fileName;
										item.RawData.Size = info.Size || info.fileInformation.fileSize;

										delete info.fileInformation;
									}
									else {
										if (!self.m_vcc_tts_config.has_configrued) {
											// Si el VCC no tiene configurado TTS, solo se visualizan los MessageType "audio" y "variable" de tipo "file".
											if (item.MessageType === "tts" || (item.MessageType === "variable" && info.Engine.toUpperCase() === "TTS")) {
												// No lo visualiza
												can_add_row = false;
											}
											else {
												// Si, lo visualiza
												can_add_row = true;
											}
										}
									}
									if (can_add_row) {
										items.push(item);
									}
								});
								self.m_grdAudioMessages[grid_id].fill_rows($.noop, items);
							}
						}
					}
					else {
					// VCC/Campana CON Integracion con Allegro (no soporta reproducciones compuestas)
					//
						// upload
						var upload_field_id = "txt" + control_id + "Upload";
						var $upload_field = $field.find("#" + upload_field_id);
						if (!$upload_field) {
							return true; // continue
						}
						self.m_uploadedFiles[upload_field_id] = config.fileInformation; // cargo en objeto la conf. de los archivos subidos.
						$upload_field.val(config.fileInformation.fileName);

						// download
						const download_field_id = "cmd" + control_id + "Download";
						const $file_download = $field.find("#" + download_field_id);
						if ($file_download) {
							$file_download.data("file_info", config.fileInformation);
						}
						$file_download.show();

						// player
						const dom_player_id = "player_" + control_id;
						const $player = $field.find("#" + dom_player_id);
						config.fileInformation.dom_player_id = dom_player_id;
						if ($player) {
							$player.data("file_info", config.fileInformation);
						}

						// play
						const $play = $("#cmd" + control_id + "Play");
						if ($play) {
							$play.data("file_info", config.fileInformation);
						}
						$play.show();
					}
				}
				else {
					if (config.message) {
						$field.find("#txt" + control_id).val(config.message);
					}
				}
			}
		});
	};

	/*
		Esta funcion es llamada cuando se edita la configuracion, y la misma termina llamando a las funciones correspondientes
		para que actualicen los controles en pantalla.
	*/
	account_config.prototype.set_current_config = function(config)
	{
		var self = this;
		self.config = config;
		self.load_current_config();
		if (typeof(self.load_custom_current_config) === "function") {
			self.load_custom_current_config();
		}
	};

	/*
		Esta funcion obtiene un mensaje, primero buscando en los datos guardados (cuando se esta editando) y sino en los mensajes builtin.
	*/
	account_config.prototype.get_message = function(message_id, option)
	{
		var self = this;
		var text = "";
		if (!message_id) {
			return text;
		}
		//primero busco en los mensajes guardados
		if (self.config) {
			var config = self.config[message_id];
			if (config && config.type && config.type.toLowerCase() === option) {
				text = config.message ? config.message : text;
			}
		}
		if (text.length > 0) {
			return text;
		}
		// luego en los builtin mensajes
		if (self.messages && option.toLowerCase() === "builtin") {
			for (var i = 0; i < self.messages.length; i++) {
				var message_config = self.messages[i];
				if (message_config.id === message_id) {
					text = message_config.builtInMessage ? message_config.builtInMessage : text;
					break;
				}
			}
		}
		return text;
	};

	account_config.prototype.delete_uploaded_file = function( upload_id, callback )
	{
		callback = callback || $.noop;

		if ( !upload_id ) {
			callback( false );
			return;
		}
		var uri = this.__make_delete_uri( upload_id );

		inConcert.invoke({
			url: uri,
			dataType: "json",
			contentType: "application/json",
			success: function( response ) {
				callback( response );
			},
			error: function( /* error */ ) {
				callback( false );
			}
		});
	};

	/*
		Esta funcion envia request al server para eliminar una lista de archivos subidos del repositorio.
	*/
	account_config.prototype.delete_uploaded_files = function( remove_list )
	{
		var self = this;
		if (!remove_list || remove_list.length === 0) {
			return;
		}
		var uri = this.__make_multi_delete_uri();

		inConcert.invoke({
			url: uri,
			type: "POST",
			data: JSON.stringify(remove_list),
			dataType: "json",
			contentType: "application/json",
			success: function(/*response*/) {},
			error: function(/*response, error*/) {
				self.show_status_message( "<%=_T('Error delete_uploaded_files.')%>", "Error", 5000 );
			}
		});
	};

	/*
		Esta funcion genera un array con los id de los archivos subidos que se deben eliminar, ya sea cuando
			se presiona el boton "save" o "cancel".
	*/
	account_config.prototype.delete_changed_uploaded_files = function( cancel )
	{
		var remove_list = [];
		var control_id;
		var campaign = this.frm.get_entity_instance() || {};

		if (cancel) {
			// LOS RECIENTES SUBIDOS (son los que se han subido antes de presionar el boton "save"----------------------------------
			// 1- Estos se eliminan de forma asincronica cuando es presionado el boton "CANCEL"
			for (control_id in this.recently_uploaded_files) {
				if (this.recently_uploaded_files.hasOwnProperty(control_id)) {
					var recently_uploaded_file = this.recently_uploaded_files[control_id];
					if (!recently_uploaded_file) {
						continue;
					}
					remove_list.push({
						id      : recently_uploaded_file.id,
						section : "1",
						campaign: campaign.Id
					});
					continue;
				}
			}
		}
		else {
			// LOS CURRENTS UPLOADS (cuando se edita) --------------------------------------------------
			// 1- Si presiono el boton "SAVE" y estoy editando tengo que remover del repo los actuales que se hayan cambiado x otro file-
			// 2- Si presiona el boton "CANCEL" y estoy editando no elimino los currents.
			// 3- Si no estoy editando, o sea es una nueva configuracion de la cuenta, no existen currents entonces no entra en loop.
			for (control_id in this.m_uploadedFiles) {
				if (this.m_uploadedFiles.hasOwnProperty(control_id)) {
					var current_uploaded_file = this.m_uploadedFiles[control_id];
					if (current_uploaded_file.changed !== true) {
						continue;
					}
					remove_list.push({
						id     : current_uploaded_file.id,
						section: "1",
						campaign: campaign.Id
					});
				}
			}
		}
		//llamo a funcion que llama URI que elimina los archivos x id del repo!
		this.delete_uploaded_files( remove_list );
	};

	/*
		Esta funcion muestra el mensaje de status "ok" o "error" cuando se ha subido un archivo.
	*/
	account_config.prototype.display_upload_file_message = function($upload_field, success)
	{
		if (!$upload_field) {
			return;
		}
		var control_id = $upload_field.attr("id");
		if (!control_id) {
			return;
		}
		var status_file_upload = $("#lbl" + control_id.substring(3) + "Status");
		if (!status_file_upload) {
			return;
		}
		var message;
		if (success) {
			message = "<%=_T('File uploaded successfully')%>";
			status_file_upload.removeClass("popUpFormSubMessageError").addClass("popUpFormSubMessageOk");
		}
		else {
			message = "<%=_T('An error ocurred uploading the file.')%>";
			status_file_upload.removeClass("popUpFormSubMessageOk").addClass("popUpFormSubMessageError");
		}
		status_file_upload.text(message).show();
		var cancel_button_id = "cmd" + control_id.substring(3) + "Cancel";
		$('#' + cancel_button_id).off('click').prop('disabled', true);
	};

	/*
		Esta funcion oculta el mensaje de status "ok" o "error" de un archivo subido.
	*/
	account_config.prototype.remove_upload_file_message = function($upload_field)
	{
		if (!$upload_field) {
			return;
		}
		var control_id = $upload_field.attr("id");
		if (!control_id) {
			return;
		}
		var status_file_upload = $("#lbl" + control_id.substring(3) + "Status");
		if (!status_file_upload) {
			return;
		}
		status_file_upload.text("").hide();
	};

	/*
		Funcion que setea eventos.
	*/
	account_config.prototype.set_events = function()
	{
		const self = this;
		//---------------------------------------
		// Esta funcion actualiza la info de los archivos recientes subidos. (Antes de presionar "save")
		var update_uploaded_file = function(control, changed) {
			if (!control) {
				return;
			}
			var control_id = "txt" + control.attr("id").substring(3) + "Upload";
			var $field = $("#" + control_id);
			if (!$field) {
				return;
			}
			self.remove_upload_file_message($field);
			if (self.m_uploadedFiles[control_id]) {
				self.m_uploadedFiles[control_id].changed = changed;
				if (!changed && self.m_uploadedFiles[control_id].fileName) {
					// si no ha cambidao dejo el archivo original (el que se levanto cuando se ha edito)
					$field.val(self.m_uploadedFiles[control_id].fileName);
				}
			}
			else {
				$field.val("");
			}
			// si existe en recientes lo elimino!!!!
			if (self.recently_uploaded_files[control_id]) {
				self.delete_uploaded_file(self.recently_uploaded_files[control_id].id);
				self.recently_uploaded_files[control_id] = undefined;
			}
		};

		//evento generico para los checkbox que visualiza/oculta los controles.
		$("#" + this.section_id).find(".formTitleCheckbox").change(function()
		{
			var $chk = $( this );
			var checked = $chk.prop( "checked" );

			var message = $chk.data( "message-data" );
			if ( message && typeof message.toggle === "function" ) {
				message.toggle( message, checked );
				return;
			}

			var $parent = $chk.parent().parent().parent().parent().parent();
			var $grand_pahter = $parent.parent().parent();
			var message_id = $grand_pahter.attr("id");
			var $file_selection_section = $grand_pahter.find('.fileSelectionSection');
			var $custom_message_section = $grand_pahter.find('.bigRow');
			var $selection_row = $grand_pahter.find('.selectionRow');
			var $selection_combo = $('#cbo' + message_id);
			var selected_value;

			if ( checked ) {
				if ( $selection_combo ) {
					$selection_row.show();
					selected_value = $selection_combo.val();
					if (selected_value && selected_value.toLowerCase() === "file") {
						update_uploaded_file($chk, false);
						$file_selection_section.show();
					}
					else {
						if (selected_value && selected_value.toLowerCase() === "builtin") {
							$custom_message_section.find('textarea').addClass('formTextarea-ro').attr("readonly", true).attr("disabled", true).focus();
						}
						else {
							$custom_message_section.find('textarea').removeClass('formTextarea-ro').attr("readonly", false).attr("disabled", false).focus();
						}
						$custom_message_section.show();
					}
				}
				else {
					$file_selection_section.show();
					$custom_message_section.show();
				}
			}
			else {
				selected_value = $selection_combo.val();
				if (selected_value && selected_value.toLowerCase() === "file") {
					update_uploaded_file($chk, true);
				}
				$file_selection_section.hide();
				$custom_message_section.hide();
				$selection_row.hide();
			}
		});

		//evento generico sobre los combos!!
		$("#" + this.section_id).find('.formComboInput').change(function()
		{
			var $combo = $(this);
			var selection = $combo.val();
			var $grand_pahter = $combo.parent().parent().parent();
			var $file_selection_section = $grand_pahter.find('.fileSelectionSection');
			var $custom_message_section = $grand_pahter.find('.bigRow');
			var message_id = $combo.attr("messageid");

			switch (selection)
			{
				case "builtin":
					update_uploaded_file($(this), true);
					$file_selection_section.hide();
					$custom_message_section.find('.formFieldValidation').hide();
					$custom_message_section.find('label')
						.text("<%=_T('Message')%>")
						.removeClass("required");

					$custom_message_section.find('textarea').addClass('formTextarea-ro').attr("readonly", true).attr("disabled", true).val(self.get_message(message_id, selection));
					$custom_message_section.show();
					break;

				case "custom":
					update_uploaded_file($(this), true);
					$file_selection_section.hide();
					$custom_message_section.find('.formFieldValidation').hide();

					var $label = $custom_message_section.find('label');
					$label.text("<%=_T('Message')%>");
					if ( !self.frm.is_readonly_current_section() ) {
						$label.addClass("required");
					}

					$custom_message_section.find('textarea').removeClass('formTextarea-ro').attr("readonly", false).attr("disabled", false).val(self.get_message(message_id, selection)).focus();
					$custom_message_section.show();
					break;

				case "file":
					update_uploaded_file($(this), false);
					$file_selection_section.show();
					$file_selection_section.find('.formFieldValidation').hide();
					$custom_message_section.hide();
					break;
			}
		});

		// inicia el uploader de files
		this.init_files_uploader();

		// download
		$("#" + this.section_id).find("[ctrl_type='download']")
			.off("click")
			.on("click", function() {
				const $this = $(this);
				const file_info = $this.data("file_info");
				file_info.control_msg_desc = $this.attr("control_msg_desc") || "";
				self.__download_file(file_info);
			});

		// play
		$("#" + this.section_id).find("[ctrl_type='play']")
			.off("click")
			.on("click", function() {
				const $play = $(this);
				const dom_owner_id = $play.attr("id");
				if ((self.m_player || {}).dom_owner_id === dom_owner_id) {
					self.destroy_players();
					self.toggle_player_button($play, true /*play action*/);
					return;
				}
				self.destroy_players();
				self.toggle_player_button($play, false /*stop action*/);
				const file_info = $play.data("file_info");
				self.initialize_player(file_info, function(success) {
					if (success) {
						self.m_player.dom_owner_id = dom_owner_id;
						self.m_player.instance.play();
					}
				});
			});

			// add_shared_file
			$("#" + this.section_id).find("[ctrl_type='new_shared_file']")
				.off("click")
				.on("click", function() {
					const $this = $(this);
					self.show_file_contextual_menu($this);
				}
			);
	};

	account_config.prototype.show_file_contextual_menu = function($parent)
	{
		let grid_id = $parent.attr("control_messageid");
		this.multi_messages[grid_id].show_file_contextual_menu($parent);
	};

	account_config.prototype.find_message = function(msg_id)
	{
		if (!msg_id) {
			return;
		}
		if (!this.messages) {
			return;
		}
		let msg_config;
		msg_id = msg_id.toLowerCase();
		for (let i = 0; i < this.messages.length; i++) {
			let message = this.messages[i];
			let current_msg_id = this.make_control_id(message).toLowerCase();
			if (current_msg_id === msg_id) {
				msg_config = message;
				break;
			}
		}
		return msg_config;
	};

	account_config.prototype.__download_file = function(file_info, callback = $.noop)
	{
		const self = this;
		var callback_fn = function(success, response) {
			callback(success, response);
		};
		let uri = this.__make_download_uri(file_info);
		$.fileDownload(uri, {
				httpMethod : "GET",
				cookieName : "download_started" // cookie exist in repository!!
			})
			.done(function() {
				callback_fn(true);
			})
			.fail(function(response) {
				const control_msg_desc = file_info.control_msg_desc || "";
				self.show_status_message(format("<%=_T('Error downloading audio: {0}')%>", control_msg_desc), "Error", 5000);
				callback_fn(false, response);
			}
		);
	};

	/*
		https://testubu.i6.inconcertcc.com/inconcert/apps/repository/download/?id=1F099DA7AFE67360746FFBE5D6825460&section=1
	*/
	account_config.prototype.__make_download_uri = function(file_info)
	{
		let uri = format(
			"{0}/inconcert/apps/repository/download/?id={1}&section={2}",
			inConcert.app.getBaseDomain(),
			encodeURIComponent(file_info.id),
			file_info.section /* section */
		);
		return uri;
	};

	account_config.prototype.__make_upload_uri = function( file )
	{
		var file_id = inConcert.Utils.generateGuid();
		var extension = inConcert.Utils.getFileExtension( file.name ) || "";

		var uri = format(
			"{0}/inconcert/apps/repository/new/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}",
			inConcert.app.getBaseDomain(),
			encodeURIComponent( file_id ),
			"1", /* section */
			file.type,
			extension, /* format */
			file.name
		);

		return {
			uri: uri,
			file_id: file_id
		};
	};

	account_config.prototype.__make_delete_uri = function( upload_id )
	{
		return format(
			"{0}/inconcert/apps/repository/erase/?id={1}&section={2}"   , /* uri         */
			inConcert.app.getBaseDomain()                                , /* domain      */
			encodeURIComponent( upload_id )                              , /* upload guid */
			"1"                                                           /* section     */);
	};

	account_config.prototype.__make_multi_delete_uri = function()
	{
		return inConcert.app.getBaseDomain() + "/inconcert/apps/repository/multipleserase/";
	};

	account_config.prototype.show_status_message = function(message, type, time_ms)
	{
		this.frm.show_status_message(message, type, time_ms);
	};

	account_config.prototype.is_readonly = function()
	{
		this.frm.is_readonly();
	};

	account_config.prototype.init_files_uploader = function()
	{
		var self = this;

		var waiting_time_ms = 5000;

		// The <input type="file"> button cannot be styled by css
		// So we have a another element (a div) which can be styled. In that element's click handler,
		// we forward the click event to the actual <input>
		// Note: to do that (the forwarding), we need the raw DOM element instead of the jQuery wrapped one
		// because $files_picker_ctrl.click() somehow causes a stack overflow (it seems to retrigger the
		// 'click' on the parent).
		// See: http://ericbidelman.tumblr.com/post/14636214755/making-file-inputs-a-pleasure-to-look-at
		//

		var $files_picker_ctrl = $( "#" + this.section_id ).find( ".butFormUploadFile > input[type='file']" );
		$files_picker_ctrl.parent().click(function() {
			var $file_picker_ctrl = $( this ).find( "input[type='file']" );
			$file_picker_ctrl[0].click();
			self.stop_players();
		});

		$files_picker_ctrl.change(function( event )
		{
			inConcert.logger.guiAction({action_name: "upload_file", window_id: window.id});

			var file = event.target.files[0];
			if ( !file ) {
				return;
			}

			//aca valida que la extension del archivo a subir sea de un archivo de audio.
			var extension = inConcert.Utils.getFileExtension( file.name ) || "";
			var availables_extensions = {
				"gsm": true,
				"mp3": true,
				"wav": true
			};
			if ( !availables_extensions[ extension.toLowerCase() ] ) {
				self.show_status_message( "<%=_T('Unsupported extension file. Only audio files (gsm, mp3, wav).')%>", "Error", waiting_time_ms );
				return;
			}
			// tamano maximo soportado en el upload
			if ( file.size > MAX_FILE_UPLOAD_SIZE_BYTES ) {
				self.show_status_message( format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", MAX_FILE_UPLOAD_SIZE_MB), "Error", waiting_time_ms );
				return;
			}

			var control_id = $( this ).attr("control_messageid");
			if ( !control_id ) {
				throw("Unexistent 'id' attribute on upload control!!");
			}

			// create tracking node
			var txt_id = "txt" + control_id + "Upload";
			var $file_name = $( "#" + txt_id );
			$file_name.val( file.name );
			//TODO: #9335 - revisar merge mcwarren con cany
			const $file_percentage = $( "#lbl" + control_id + "UploadPercentage" );
			$file_percentage.text( "0" + "%" ).show();

			const $file_download = $("#cmd" + control_id + "Download");
			$file_download.hide();

			const $play = $("#cmd" + control_id + "Play");
			$play.hide();

			self.player_hide(control_id);

			const uri_info = self.__make_upload_uri(file);

			var xhr = new XMLHttpRequest();
			xhr.open("POST", uri_info.uri, true);
			xhr.onload = function(e)
			{
				$file_name.val( file.name );
				if ( e.target.status !== 200 ) {
				// error
					$file_percentage.text( "" ).hide();
					self.show_status_message( format( "<%=_T('Error uploading file {0}.')%>", file.name ), "Error", waiting_time_ms );
					$files_picker_ctrl.val("");
					return;
				}
				// success
				//
				// si estoy editando, tengo que marcar el archivo existente como delete, para que se elimine al dar "save"
				var currentUploadedFiles = self.m_uploadedFiles[ txt_id ];
				if (currentUploadedFiles && currentUploadedFiles.id && currentUploadedFiles.id !== uri_info.file_id) {
					self.m_uploadedFiles[ txt_id ].changed = true;
				}
				// si existe en recientes lo elimino!!!!
				if ( self.recently_uploaded_files[ txt_id ] ) {
					self.delete_uploaded_file(self.recently_uploaded_files[ txt_id ].id);
				}
				// doy de alta en recientes!!!!
				self.recently_uploaded_files[ txt_id ] = {
					id         : uri_info.file_id,
					section    : "1",
					fileName   : file.name,
					fileSize   : file.size,
					contentType: file.type,
					format     : extension
				};

				$files_picker_ctrl.val("");

				window.setTimeout( function() {
					$file_percentage.text( "" ).hide();
					$file_download.show();
					$play.show();
					let fileInformation = self.recently_uploaded_files[txt_id];
					fileInformation.dom_player_id = "player_" + control_id;
					$play.data("file_info", fileInformation);
				}, waiting_time_ms );
			};

			// show the uploaded percentage
			xhr.upload.onprogress = function( e ) {
				var percentage = "";
				if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
					percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
				}
				$file_percentage.text( percentage );
			};

			xhr.send( file );
		});
	};

	account_config.prototype.init_audio_grid = function(grid_id, params)
	{
		if (!this.supportsCompositeFilesMessages()) {
		// VCC/Campana CON Integracion con Allegro (NO soporta reproducciones compuestas)
			return;
		}
		this.multi_messages = this.multi_messages || {};
		const self = this;
		this.multi_messages[grid_id] = new inConcert.adm_form_multi_messages(this, {
			grid_id : grid_id,
			isAllowedCustomVariables: false,
			ownerName: "accounts messages",
			can_include_account_messages_variable: function (accountsMessages, variableId) {
				//Si accountsMessages es una tabla, es porque debe terner los canal permitidos para la variable en proceso.
				if (accountsMessages && typeof accountsMessages === "object" && !accountsMessages[self.type]) {
					return false;
				}

				if (params?.can_include_variables) {
					return params.can_include_variables(accountsMessages, variableId);
				}

				return true;
			},
			make_description_variables: params?.make_description_variables || undefined,
			visibility: {
				AgentVisible          : false,
				AccountMessagesVisible: true,
				FlowDesignerVisible   : false
			}
		});
	};

	/*
		Funcion que genera el markup con los controles.
	*/
	account_config.prototype.create_fields = function(callback = $.noop)
	{
		const self = this;

		const $container = $( "#" + this.section_id + " .card-block" );
		$container.html("");

		if ( typeof(self.set_custom_controls) === "function" ) {
			self.set_custom_controls();
		}
		if ( !self.messages ) {
			callback(false);
			return;
		}

		// las opciones del combo, los tipos de mensajes (builtin, custom y file)
		const get_combo_type_opts = function( m )
		{
			let count = 0;

			if (m.hasBuiltIn) {count++;}
			if (m.allowCustom) {count++;}
			if (m.allowFileUpload) {count++;}

			return {
				count: count,
				html : ''.concat(
					!m.hasBuiltIn ?      '' : '<option value="builtin" selected>' + "<%=_T('Use Built-in Message')%>" + '</option>',
					!m.allowCustom ?     '' : '<option value="custom">'           + "<%=_T('Write Custom Message')%>" + '</option>',
					!m.allowFileUpload ? '' : '<option value="file">'             + "<%=_T('Upload File')%>"          + '</option>'
				)
			};
		};

		let supportsCompositeFilesMessages = this.supportsCompositeFilesMessages();

		// El contenido de los mensajes, si es un file upload, o si es custom
		const get_html_messages = function( m )
		{
			var control_id = self.make_control_id( m );

			if ( typeof m.markup === "function" ) {
				return m.markup( m, control_id );
			}

			var markup = '';
			if ( m.allowFileUpload ) {
				var get_player_html = function () {
					if (!self.must_shown_player()) {
						return "";
					}
					let markup = "" +
						"<div class='form-group d-flex align-items-end'>" +
							// el boton de play del reproductor
							"<button id='cmd" + control_id + "Play' class='pageButtonGeneric buttonColorWhite iconOnlyBtn mr-2' control_messageid='" + control_id + "' type='button' ctrl_type='play' title='" + "<%=_T('Play')%>" + "' control_msg_desc='" + m.description + "' hidden>" +
								"<i class='far fa-play-circle'></i>" +
							"</button>" +

							// el boton download
							"<button id='cmd" + control_id + "Download' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' control_messageid='" + control_id + "' type='button' ctrl_type='download' title='" + "<%=_T('Download')%>" + "' control_msg_desc='" + m.description + "' hidden>" +
								"<i class='far fa-download'></i>" +
							"</button>" +
							
							// el reproductor
							"<div id='player_" + control_id + "' class='clientPlayContainer' control_msg_desc='" + m.description + "' hidden></div>"+
						"</div>";
					return markup;
				};

				var get_simple_file_html = function () {
					return "" +
						"<div class='row mt-2'>" +
							"<div class='col-6'>" +
								"<div class='form-group d-flex'>" +	
									"<button class='pageButtonGeneric buttonColorWhite butFormUploadFile'>" +
										"<i class='far fa-upload mr-2'></i>" +
										"<%=_T('Upload File')%>" +
										"<input id='cmd" + control_id + "Upload' control_messageid='" + control_id + "' type='file' hidden accept='audio/*' />" +
									"</button>" +
								"</div>" +
							"</div>" +	
						"</div>" +	
						"<div class='row'>" +
							"<div class='col-6 d-flex flex-row'>" +	
								"<div class='form-group flex-grow-1'>" +
									"<label class='required'>" + "<%=_T('File')%>" + "<%=_T(':')%>" + "</label>" +
									"<input id='txt" + control_id + "Upload' class='form-control readonly' control_messageid='" + control_id + "' type='text' readonly='readonly' value='' />" +
									"<i id='txt" + control_id + "UploadValidation' class='formFieldValidation formFieldError' hidden></i>" +
									"<label id='lbl" + control_id + "UploadPercentage' class='uploadFilePercentageChip' hidden >100%</label>" +
								"</div>" +
								// download & player
								get_player_html() +
							"</div>" +
						"</div>";
				};

				var get_composite_file_html = function () {
					return "" +
						"<div class='row mt-2' id='" + control_id + "_neww_container_btns' control_messageid='" + control_id + "'>" +
							"<div class='col-12'>" +
								"<div class='card-header d-flex flex-row justify-content-between'>" +
									"<h5 class='mt-2'>" + "<%=_T('Message Components')%>" + "</h5>" +
									"<button ctrl_type='new_shared_file' control_messageid='" + control_id + "' class='pageButtonGeneric buttonColorWhite pageButtonNew mt-1 mb-4' >" +
										"<i class='far fa-plus'></i>" +
										"<%=_T('Add Component')%>" +
									"</button>" +
								"</div>" +
							"</div>" +
						"</div>" +
						"<div class='row'>" +
							"<div class='col-12'>" +
								"<div id='" + control_id + "not_results_found' class='inlineMessagesContainer messageInfo mt-0' style='display:none'></div>" +
								"<table id='" + control_id + "list' cellpadding='0' cellspacing='0' border='0' class='viewGrid mt-0 mb-3'>" +
									"<thead thead='true' class='viewGridHeader'></thead>" +
									"<tbody tbody='true' class='viewGridBody'></tbody>" +
								"</table>" +
								// El markup correspondiente a la tabla se genera dentro de la clase 'conf_form_paginator.js'
								"<div id='" + control_id + "grid_container'></div>" +
								"<div class='mb-2' id='" + control_id + "paginator'></div>" +
							"</div>" +
						"</div>";
				};

				markup = markup +
					"<div class='col-12 fileSelectionSection' style='display:none;'>" +
						(supportsCompositeFilesMessages ? get_composite_file_html() : get_simple_file_html()) +
						//get_simple_file_html() + get_composite_file_html()
					"</div>";
			}

			if ( m.finalizeInteraction !== undefined) {
				markup = markup +
					'<div class="col-6 siblingFileSelectionSection">' +
						'<div class="form-group">' +
							'<div class="form-check">' +
								'<label for="' + control_id + '_finalizeInteraction" class="form-check-label form-check-label-title">' +
									'<input id="' + control_id + '_finalizeInteraction" class="form-check-input" type="checkbox" />' +
									'<%=_T("Finalize Conversation")%>' +
								'</label>' +
							'</div>' +
						'</div>' +
					'</div>';
			}

			if ( m.allowCustom ) {
				//all the fields having the option to use a builtin message, default to that option, if not, default to custom empty message
				var defaultMessage = m.hasBuiltIn ? '<%=_T("Message")%>' : '<%=_T("Message")%>';
				markup = markup +
					'<div class="col-12 bigRow" style="display:' + (m.checked ? '' : 'none') + ';">' +
						'<div class="form-group">' +
							'<label class="required"> ' + defaultMessage + ' </label>' +
							'<textarea id="txt' + control_id + '" class="form-control formTextarea-ro rowsBased" rows="10" ' + (m.hasBuiltIn ? 'readonly="readonly"' : "") + '>' +
								self.get_message(m.id, "builtin") +
							'</textarea>' +
							'<i id="txt' + control_id + 'Validation" class="formFieldValidation formFieldError" hidden></i>' +
						'</div>' +
					'</div>';
			}

			var combo_type_opts = get_combo_type_opts( message ); // opciones del menu
			var can_show_combo_type = true;
			if (combo_type_opts.count <= 1) {
				can_show_combo_type = false;
			}

			return ''.concat(
				'<div class="col-12">',
					'<div class="row selectionRow mt-3" style="display:', (message.checked ? '' : 'none'), ';">', // contenedor general (prueba)
						'<div class="col-6" style="display:', (can_show_combo_type ? '' : 'none'), ';">',
							'<div class="form-group">',
								'<label class="required">', "<%=_T('Message Type')%>", "<%=_T(':')%>", '</label>',
								'<select id="cbo', control_id, '" class="form-control formComboInput" messageid="', message.id, '" >',
									combo_type_opts.html,
								'</select>',
							'</div>',
						'</div>',
						markup, // el markup contenedor de mensajes (builtin y custom)
					'</div>',
				'</div>');
		};
		
		if(self.tts_config_section && self.tts_config_section.voices) {
			var $tts_config_section_markup = $( ''.concat(
				'<div class="card" id="CALL_TTS_CONFIG">',
					'<div class="card-block">',
						'<div class="row">',
							'<div class="col-6">',
								'<div class="form-group">',
									'<label for="">', self.tts_config_section.description, '</label>',
									'<select id="ttsDefaultVoice" class="form-control formComboInput"></select>',
								'</div>',
							'</div>',
						'</div>',
					'</div>',
				'</div>'
			));
			$container.append( $tts_config_section_markup );
			self.frm.fill_combo( $("#ttsDefaultVoice"), true, self.tts_config_section.voices, "id", "name", true );
			$("#ttsDefaultVoice").val(self.tts_config_section.ttsDefaultVoice);
		}

		// loop que genera los mensajes para la account/canal
		for ( var i = 0; i < self.messages.length; i++ ) {
			var message =  self.messages[i];
			var control_id = self.make_control_id( message );
			var $markup = $( ''.concat(
				'<div id=', control_id, ' class="check-with-bottom-separator">',
					'<div class="card">',
						'<div class="card-block">',
							'<div class="col-12">',
								'<div class="form-group">',
									'<div class="form-check form-switch">',
										'<label for="chk', control_id, '" class="form-check-label form-check-label-title">',
											'<input id="chk', control_id, '" class="form-check-input formTitleCheckbox" type="checkbox" ', (message.checked ? 'checked' : ''), '/>',
											'<span></span>', message.description,
										'</label>',
									'</div>',
								'</div>',
							'</div>',
							get_html_messages( message ), // el markup contenedor de mensajes (builtin y custom)
						'</div>',
					'</div>',
				'</div>'
			));

			$markup.find( '#chk' + control_id ).data( 'message-data', message );
			$container.append( $markup );
			inConcert.Utils.sortOptions('cbo' + control_id);

			if ( typeof message.init === 'function' ) {
				message.init( message, control_id );
			}

			//init grid files de los mensajes de files composite (audios) (solo para vcc SIN integracion con Allegro)
			if (message.allowFileUpload && supportsCompositeFilesMessages) {
				this.init_audio_grid(control_id, {
					can_include_variables: message.can_include_variables || undefined,
					make_description_variables: message.make_description_variables || undefined
				});
			}
		}

		//seteo eventos
		this.set_events();

		callback(true);
	};

	/*
		Funcion muestra la cruz de error el los campos que han fallado.
	*/
	account_config.prototype.display_field_error = function($field)
	{
		var validation_id = $field.attr("id") + "Validation";
		var $field_validation = $("#" + validation_id);
		if ($field_validation) {
			$field_validation.show();
		}
		$field.focus();
	};

	/*
		Validates the specified configuration by the user and returns a hash with the message configuration for this proxyType
	*/
	account_config.prototype.validate = function()
	{
		const self = this;

		const error = function( $field, current_message, data ) 
		{
			self.display_field_error( $field );
			var error_message;
			switch ( typeof current_message.error_message ) {
				case "string":
					error_message = current_message.error_message;
				break;
				case "function":
					var data_type = data ? data.type : null;
					error_message = current_message.error_message( data_type, data );
				break;
			}
			return {
				error        : true,
				$ctrl        : $field,
				error_message: error_message
			};
		};

		var account_messages = {};
		if (typeof(self.on_validate) === "function") {
			var custom_validation = self.on_validate();
			if ( custom_validation && custom_validation.error ) {
				return custom_validation;
			}
			$.extend(true, account_messages, custom_validation);
		}

		//iterate over the generic fields stuff and then join with the custom stuff by proxy
		if (this.messages) {
			let supportsCompositeFilesMessages = this.supportsCompositeFilesMessages();

			for (var i = 0; i < self.messages.length; i++) {
				var message = {};

				var message_id = self.make_control_id( self.messages[i] );
				//message is ment to be used?
				if ($("#" + message_id).find('.form-check-input').prop('checked'))
				{
					if ( typeof self.messages[i].validate === "function" ) {
						message = self.messages[i].validate( self.messages[i] );
						if ( message.error ) {
							return error( message.error.$field, self.messages[i], message.error.data );
						}
					}
					else {
						var selection;
						if (self.messages[i].allowFileUpload) {
							selection = "file";
						}
						if (self.messages[i].allowCustom) {
							selection = "custom";
						}
						if (self.messages[i].hasBuiltIn) {
							selection = $("#" + message_id).find('.formComboInput').val();
						}
						message.id = self.messages[i].id;
						message.type = selection;
						var $field = $("#" + message_id + " .bigRow .form-control");

						switch ( selection.toLowerCase() ) 
						{
							case "custom":
								var customMessage = $field.val();
								if (customMessage.trim().length === 0) {
									return error( $field, self.messages[i], { type: "required" } );
								}
								var maxlength = self.messages[i].maxlength;
								if ( !isNaN( maxlength ) && customMessage.length > maxlength ) {
									return error( $field, self.messages[i], { type: "overflow", maxlength: maxlength } );
								}
								message.message = customMessage;
								break;

							case "file":
								var upload_fileId = "txt" + message_id + "Upload";

								if (!supportsCompositeFilesMessages) {
								// VCC/Campana CON Integracion con Allegro (NO soporta reproducciones compuestas)
								//
									var uploaded_file_information = undefined;
									if (self.recently_uploaded_files && self.recently_uploaded_files[upload_fileId]) {
										uploaded_file_information = self.recently_uploaded_files[upload_fileId];
									}
									if (!uploaded_file_information && self.m_uploadedFiles && self.m_uploadedFiles[upload_fileId]) {
										uploaded_file_information = self.m_uploadedFiles[upload_fileId];
									}
									if (!uploaded_file_information) {
										return error( $("#" + upload_fileId), self.messages[i] );
									}
									message.fileInformation = uploaded_file_information;
									message.fileInformation.isComposite = false;
								}
								else {
								// VCC/Campana SIN Integracion con Allegro (SI soporta reproducciones compuestas)
								//
									let grid_id = this.make_control_id(message);
							//let grid_id = format("{0}_{1}", this.type, (message.id || ""));
									if (!this.m_grdAudioMessages[grid_id]) {
										return error( $("#" + upload_fileId), self.messages[i] );
									}
	
									let items = this.m_grdAudioMessages[grid_id].get_items();
									if ((items || []).length === 0) {
										return error( $("#" + upload_fileId), self.messages[i] );
									}
	
									message.isComposite = true;
									message.options = [];

									items.forEach(item => {
										let item_info = item;
										let messageType = (item.MessageType || "").toLowerCase();
	
										if (messageType === "audio") {
											item_info.type = "file";
											item_info.fileInformation = {
												id         : item.RawData.RepositoryId,
												fileSize   : parseInt(item.RawData.Size, 10),
												contentType: item.RawData.ContentType,
												fileName   : item.RawData.Name,
												format     : inConcert.Utils.getFileExtension(item.RawData.Name) || "",
												section    : item.RawData.Section || "-1"
											};
										}
										else if (messageType === "variable") {
											item_info.type = item_info.Engine.toUpperCase() === "TTS" ? "custom" : "file";
											item_info.message = item_info.VariableId;
										}
										else if (messageType === "tts") {
											item_info.type = "custom";
										}
										else {
											throw("Unrecognized messageType " + messageType);
										}
										delete(item.RawData);
										message.options.push(item_info);
									});
								}
								break;

							default:
								message.message = $field.val();
								break;
						}
						
						if ( $("#" + message_id + '_finalizeInteraction').attr("disabled"))
						{
							message.finalizeInteraction = "-1";
						}
						else
						{
							message.finalizeInteraction = get_chk_bool_to_string( message_id + '_finalizeInteraction' );
						}
					}

					account_messages[ message.id ] = message;
				}
			}
		}

		return account_messages;
	};

	/*
		custom validation
	*/
	account_config.prototype.custom_validate = function(account_messages, messages)
	{
		var self = this;

		var error = function( $field, current_message ) 
		{
			self.display_field_error( $field );
			var error_message;
			switch ( typeof current_message.error_message ) {
				case "string":
					error_message = current_message.error_message;
				break;
				case "function":
					error_message = current_message.error_message( /*error type*/ );
				break;
			}
			return { error: true, $ctrl: $field, error_message: error_message };
		};

		account_messages = account_messages ===  undefined ? {} : account_messages;
		messages = messages !== undefined ? messages : self.get_custom_messages();

		for (var i = 0; i < messages.length; i++)
		{
			var message = messages[i];
			var $field = $("#" + message.control_id);

			if (message.is_parent) {
				if ($field.prop('checked') && message.dependentControls) {
					// invocacion recursiva
					account_messages = self.custom_validate(account_messages, message.dependentControls);
				}
			}
			else {
				var $parent = $("#" + message.parent_control_id);
				if (!$parent.prop("checked")) {
					continue;
				}
				if ($field.attr("type") !== "checkbox") {
					account_messages[message.id] = account_messages[message.id] === undefined ? {} : account_messages[message.id];

					var message_value;

					if (typeof(message.get_value) === "function") {
						message_value = message.get_value($field);
						if (!message_value) {
							return error( $field, messages[i] );
						}
					}
					else {
						message_value = $field.val() || "";
						if (message_value.trim().length === 0) {
							return error( $field, messages[i] );
						}
					}

					if (typeof(message.attach_custom_value) === "function") {
						message.attach_custom_value(account_messages[message.id]);
					}

					account_messages[message.id].id = message.id;
					account_messages[message.id].type = "custom";
					if (message.fieldName) {
						account_messages[message.id][message.fieldName] = message_value;
					}
					else {
						account_messages[message.id].message = message_value;
					}
				}
				else {
					if ($field.prop("checked")) {
						account_messages[message.id] = {
							id: message.id,
							type: "custom"
						};
					}
				}
			}
		}
		return account_messages;
	};

	/*
		load custom configuration
	*/
	account_config.prototype.load_custom_config = function( messages )
	{
		var self = this;
		var parentChecked = false;
		messages = messages !== undefined ? messages : self.get_custom_messages();

		for (var i = 0; i < messages.length; i++) {
			var message_config = messages[i];
			var $field = undefined;
			$field = $("#" + message_config.control_id);
			if (!$field) {
				continue;
			}
			var config = self.config[message_config.id];
			var checked = false;
			if ($field.attr("type") === "checkbox") {
				if (message_config.dependentControls) {
					checked = self.load_custom_config(message_config.dependentControls);
				}
				checked = config ? true : checked;
				$field.prop("checked", checked);
				$field.change();
			}
			else {
				var fieldName = message_config.fieldName ? message_config.fieldName : "message";
				checked = config && config.type && config.type.toLowerCase() === "custom" && config[fieldName];
				var value = checked ? config[ fieldName ] : "";

				if (typeof(message_config.set_value) === "function") {
					message_config.set_value($field, value, config);
				}
				else {
					$field.val( value );
				}
				checked = checked ? true : false;
			}
			if (!parentChecked) {
				parentChecked = checked;
			}
		}
		return parentChecked;
	};

	// Funcion invocada desde Accounts
	account_config.prototype.start_upload_images = function(callback)
	{
		callback = callback || $.noop;
		if (!this.mail_mng) {
			callback(true);
			return;
		}
		var total_upload = Object.keys(this.mail_mng).length;
		var count_upload = 0;

		$.each(this.mail_mng, function(_, mail_mng)
		{
			mail_mng.start_upload_images(function(success)
			{
				count_upload++;
				if (count_upload === total_upload) {
					callback(success);
				}
			});
		});
	};

	// Funcion invocada desde Accounts
	account_config.prototype.on_saved = function()
	{
		if (!this.mail_mng) {
			return;
		}
		$.each(this.mail_mng, function(_, mail_mng) {
			mail_mng.on_saved();
		});
	};

	// Funcion invocada desde Accounts
	account_config.prototype.on_cleanup = function()
	{
		if (!this.mail_mng) {
			return;
		}
		$.each(this.mail_mng, function(_, mail_mng) {
			mail_mng.cleanup_temp_repository_files();
		});
	};

	// Funcion invocada desde la clase MailManagement
	account_config.prototype.get_repository_relation = function()
	{
		if (!this.mail_mng) {
			return;
		}
		var relation = {campaign : this.parent.get_campaign_id()};
		if (this.parent.current_account) {
			relation.account   = this.parent.current_account.id;
			relation.proxyType = this.parent.current_account.proxyType;
		}
		return relation;
	};

	// Funcion invocada desde la clase adm_form_campaigns_section_accounts
	account_config.prototype.messages_cleanup = function(is_cancel)
	{
		this.delete_changed_uploaded_files(is_cancel);
		this.destroy_players();
	};

	account_config.prototype.must_shown_player = function()
	{
		return true;
	};

	account_config.prototype.toggle_player_button = function($play, is_play)
	{
		if ($play.length === 0) {
			return;
		}
		const $i = $play.find("i");
		$i.removeClass("fa-stop-circle fa-play-circle");

		if (is_play) {
			$play.attr("title", "<%=_T('Play')%>");
			$i.addClass("fa-play-circle");
		}
		else {
			$play.attr("title", "<%=_T('Stop')%>");
			$i.addClass("fa-stop-circle");
		}
	};

	/*
		Sample URI: "https://testubu.i6.inconcertcc.com/inconcert/apps/repository/download/?id=84072f21-5fde-4c2e-b4ce-04f2d94946d0&section=1&part=1";
	*/
	account_config.prototype.initialize_player = function(file_info, callback = $.noop)
	{
		const self = this;
		if (!this.must_shown_player()) {
			return;
		}
		this.m_player = this.m_player || {};
		this.stop_players();

		let dom_player_id = file_info.dom_player_id;
		let uri = this.__make_download_uri(file_info);

		this.m_player.instance = new CInconcertPlayer({
			containerId: dom_player_id,
			url        : uri,
			fileFormat : file_info.format,
			success: function() {
				// Callback cuando el audio se ha cargado correctamente.
				callback(true);
			},
			error: function() {
				// Callback cuando se produce un error en la carga del audio.
				const control_msg_desc = $("#" + file_info.dom_player_id).attr("control_msg_desc") || "";
				self.show_status_message(format("<%=_T('Error loading audio: {0}')%>", control_msg_desc), "Error", 5000);
				callback(false);
			}
		});
		self.m_player.instance.onEnded(function() {
			// Dejo el Callback como referencia por si tenemos que realizar algo cuando termina de ser reproducido el audio.
			const $play = $("#" + (self.m_player || {}).dom_owner_id);
			self.toggle_player_button($play, true /*play action*/);
		});
		self.m_player.instance.load();
	};

	account_config.prototype.stop_players = function()
	{
		if (!this.m_player) {
			return;
		}
		if (this.m_player.instance) {
			this.m_player.instance.stop();
		}
	};

	account_config.prototype.destroy_players = function()
	{
		if (!this.m_player) {
			return;
		}
		if (this.m_player.instance) {
			this.m_player.instance.destroy();
			const $play = $("#" + (this.m_player || {}).dom_owner_id);
			this.toggle_player_button($play, true /*play action*/);
		}
		this.m_player = null;
	};

	account_config.prototype.player_hide = function(/*control_id*/)
	{
		if (!this.m_player) {
			return;
		}
		const player = this.m_player.instance;
		if (!player) {
			return;
		}
		player.hide();
	};

/*
----------------------------------------------------------------------------------------------
  Clase - 'call_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function call_config(params)
	{
		this.m_vcc_tts_config = inConcert.app.get_vcc_tts_config();

		let ttsVoices;
		if (this.m_vcc_tts_config.has_configrued) {
			this.m_vcc_tts_config.ttsasrEngines.forEach(function(engine) {
				if((engine.providerType === "TTS" || engine.providerType === "TTS-ASR") && engine.voices && $.isArray(engine.voices) && engine.voices.length > 0) {
					ttsVoices = [];
					engine.voices.forEach(function(voice) {
						let voice_to_show = {
							id  : voice.id,
							name: voice.LanguageName + " - " + voice.Variant + " - " + voice.Gender + " - " + voice.VoiceName
						};
						ttsVoices.push(voice_to_show);
					});
				}
			});
		}

		this.type = "CALL";
		this.section_id = "adm_form_campaign_accounts_call_panel_tab_messages";
		this.tts_config_section = {
			description    : "<%=_T('Default TTS voice:')%>",
			voices         : ttsVoices,
			ttsDefaultVoice: (params.parent && params.parent.current_account && params.parent.current_account.ttsDefaultVoice)
		};

		const can_include_variables = function(_, variableId) {
			//Aca valido que no se permitan las variables para los mensajes de call
			switch (variableId) {
				case "{ESTIMATED_WAITING_TIME_HH_MM_SS}":
				case "{ESTIMATED_WAITING_TIME_HH_MM}":
				case "{ESTIMATED_WAITING_TIME_MM_SS}":
				case "{ESTIMATED_WAITING_TIME_HH}":
					return false;
				default:
					break;
			}
			return true;
		};

		const make_description_variables = function(variableId) {
			//Aca obtengo solo para call por el momento una breve descripcion de algunas variables. 
			switch (variableId) {
				case "{ESTIMATED_WAITING_TIME}":
					return "<div>" +  
					 			"<%=_T('Numeric value corresponding to the estimated wait time in seconds.')%><br>"+
								"<%=_T('Example')%>:<br>" +
								"<%=_T('If the timeout is 11 seconds, an attempt will be made to vocalize eleven.wav or 11.wav.')%><br>" +
								"<%=_T('If the timeout is 57 seconds, an attempt will be made to vocalize fifty.wav and.wav seven.wav or 57.wav.')%>" +
							"</div>";
				case "{ESTIMATED_WAITING_TIME_MM}":
					return "<div>" +  
					 			"<%=_T('Numeric value corresponding to the rounded estimated wait time in minutes.')%><br>"+
								"<%=_T('Example')%>:<br>" +
								"<%=_T('If the timeout is 30 seconds, an attempt will be made to vocalize un.wav or 01.wav.')%><br>" +
								"<%=_T('If the waiting time is 171 seconds (more than 2 minutes), an attempt will be made to vocalize three.wav or 03.wav.')%>" +
							"</div>";
				case "{ESTIMATED_WAITING_TIME_MM_SS_CALL}":
					return "<div>" +  
					 			"<%=_T('Text including the estimated time plus the unit of time it is sent in.')%> <br>"+
								"<%=_T('The logic is: if the time is less than one minute, the time is returned in seconds, if the value is greater than 1 minute, it is returned in rounded minutes.')%><br>"+
								"<%=_T('Example')%>:<br>" +
								"<%=_T('If the timeout is 57 seconds, an attempt will be made to vocalize fifty.wav and.wav seven.wav seconds.wav.')%><br>" +
								"<%=_T('If the waiting time is 200 seconds (more than 3 minutes), an attempt will be made to vocalize four.wav minutes.wav.')%><br>" +
							"</div>";
				default:
					break;
			}
		};

		this.messages = [
			{
				id : "waiting_for_agent",
				description : "<%=_T('Music on ACD')%>",
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Waiting for Agent is required.')%>";
				}
			},
			{
				id : "waiting_for_agent_with_ewt",
				description : "<%=_T('Waiting message on ACD')%>",
				builtInMessage: "<%=_T('Your queue number is {PLACE_ON_QUEUE}, with about {ESTIMATED_WAITING_TIME_MM} minutes of waiting time. Please hold on!')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts"     , id: "<%=_T('Your queue number is ')%>"},
							{type: "variable", id: "{PLACE_ON_QUEUE}"},
							{type: "tts"     , id: "<%=_T(', with about')%>"},
							{type: "variable", id: "{ESTIMATED_WAITING_TIME_MM}"},
							{type: "tts"     , id: "<%=_T(' minutes of waiting time. Please hold on!')%>"}
						]
					}
				],
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Waiting for Agent with Estimated Waiting Time is required.')%>";
				}
			},
			{
				id : "welcome",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, were searching for the best person to talk with you')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts"     , id: "<%=_T('Thanks for getting in touch with ')%>"},
							{type: "variable", id: "{GROUPNAME}"},
							{type: "tts"     , id: "<%=_T(', were searching for the best person to talk with you')%>"}
						]
					}
				],
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome Message is required.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				finalizeInteraction : "-1",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts", id: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>"}
						]
					}
				],
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Out of Hours Message is required.')%>";
				},
				editable: function( message, control_id ) {
					if(message.finalizeInteraction === "-1")
					{
						$( "#" + control_id + "_finalizeInteraction" ).prop("checked", true );
						$( "#" + control_id + "_finalizeInteraction" ).attr("disabled", true );
					}
				}
			},
			{
				id : "voiceMail",
				description : "<%=_T('Voice Mail')%>",
				builtInMessage: "<%=_T('We are unable to take your call, please leave a message and we will call you back.')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts", id: "<%=_T('We are unable to take your call, please leave a message and we will call you back.')%>"}
						]
					}
				],
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Voice Mail is required.')%>";
				}
			},
			{
				id : "virtualHold",
				description : "<%=_T('Virtual Hold')%>",
				builtInMessage: "<%=_T('Will be called back soon.')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts", id: "<%=_T('Will be called back soon.')%>"}
						]
					}
				],
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Virtual Hold is required.')%>";
				}
			},
			{
				id : "dialerCallback",
				description : "<%=_T('Dialer Callback')%>",
				builtInMessage: "<%=_T('Will be called back soon.')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts", id: "<%=_T('Will be called back soon.')%>"}
						]
					}
				],
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Dialer Callback is required.')%>";
				}
			},
			{
				id : "music_on_hold",
				description : "<%=_T('Music on hold')%>",
				allowFileUpload : true,
				checked: false,
				allowsVariables: false, // si el atributo no se encuentra o se define como undefined el mismo vale TRUE
				maxAudioFiles: 1,       // si el atributo no se encuentra o se define como undefined el tope es infinito
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Music on hold is required.')%>";
				}
			},
			{
				id : "transfer_failure",
				description : "<%=_T('Transfer failure')%>",
				builtInMessage: "<%=_T('The transfer to {AGENTNAME} has failed.')%>",
				builtInMessagesList: [
					{
						id: 1,
						default: true,
						msg: [
							{type: "tts"     , id: "<%=_T('The transfer to ')%>"},
							{type: "variable", id: "{AGENTNAME}"},
							{type: "tts"     , id: "<%=_T(' has failed.')%>"}
						]
					}
				],
				allowFileUpload : true,
				checked: false,
				can_include_variables: can_include_variables,
				make_description_variables: make_description_variables,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Transfer failure is required.')%>";
				}
			},
			{
				id : "voa",
				description : "<%=_T('VDN of Origin Announcements (VOA) Message.')%>",
				allowFileUpload : true,
				checked: false,
				allowsVariables: false,
				maxAudioFiles: 1,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('VOA Message is required.')%>";
				}
			}
		];
		call_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit( call_config, account_config );

	if ( typeof inConcert.adm_form_campaigns_call_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_call_account_messages = call_config;
	}

/*
----------------------------------------------------------------------------------------------
  Clase - 'call_allegro_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function call_allegro_config(params)
	{
/*
En allegro la carga del combo "cboMultimediaType"

[
  { "Name": "PLEASEWAIT_PROMPT"        , "DisplayValue": "Please wait ..."                                                 },
  { "Name": "MOH"                      , "DisplayValue": "Music on hold."                                                  },
  { "Name": "EXIT_PROMPT"              , "DisplayValue": "Thanks for calling, goodbye."                                    },
  { "Name": "IVR_PROMPT"               , "DisplayValue": "This is an interactive voice response audio."                    },
  { "Name": "VMONWAIT_PROMPT"          , "DisplayValue": "To leave a Voicemail please press 1"                             },
  { "Name": "OUTOFTIME_PROMPT"         , "DisplayValue": "We are currently out of business hours ..."                      },
  { "Name": "VOICEMAILOUTOFTIME_PROMPT", "DisplayValue": "We are currently out of business hours, you can leave a message" },
  { "Name": "WERESORRY_PROMPT"         , "DisplayValue": "We are sorry, we cannot process your call right now."            },
  { "Name": "RESCHEDULE_PROMPT"        , "DisplayValue": "We must reschedule your call"                                    },
  { "Name": "WELCOME_PROMPT"           , "DisplayValue": "Welcome to ..."                                                  },
  { "Name": "WAIT_PROMPT"              , "DisplayValue": "While you wait [play this music/message]"                        },
  { "Name": "YOUREINPOSITION_PROMPT"   , "DisplayValue": "You are in the position [number]."                               },
  { "Name": "VOICEMAIL_PROMPT"         , "DisplayValue": "You have [n] seconds to leave a message."                        },
  { "Name": "YOUMUSTWAIT_PROMPT"       , "DisplayValue": "You must wait [time] to be served."                              }
]
*/
		this.is_allegro_channel = true;
		this.type = "CALL";
		this.section_id = "adm_form_campaign_accounts_call_allegro_panel_tab_messages";
		this.messages = [
			{
				id : "PLEASEWAIT_PROMPT",
				description : "<%=_T('Please wait')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Please wait is required.')%>";
				}
			},
			{
				id : "MOH",
				description : "<%=_T('Music on hold')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Music on hold is required.')%>";
				}
			},
			{
				id : "EXIT_PROMPT",
				description : "<%=_T('Thanks for calling, goodbye')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Thanks for calling, goodbye is required.')%>";
				}
			},
			{
				id : "IVR_PROMPT",
				description : "<%=_T('This is an interactive voice response audio')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('This is an interactive voice response audio is required.')%>";
				}
			},
			{
				id : "VMONWAIT_PROMPT",
				description : "<%=_T('To leave a Voicemail please press 1')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('To leave a Voicemail please press 1 is required.')%>";
				}
			},
			{
				id : "OUTOFTIME_PROMPT",
				description : "<%=_T('We are currently out of business hours ...')%>",
				finalizeInteraction : "-1",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('We are currently out of business hours is required.')%>";
				},
				editable: function( message, control_id ) {
					if(message.finalizeInteraction === "-1")
					{
						$( "#" + control_id + "_finalizeInteraction" ).prop("checked", true );
						$( "#" + control_id + "_finalizeInteraction" ).attr("disabled", true );
					}
				}
			},
			{
				id : "VOICEMAILOUTOFTIME_PROMPT",
				description : "<%=_T('We are currently out of business hours, you can leave a message')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('We are currently out of business hours, you can leave a message is required.')%>";
				}
			},
			{
				id : "WERESORRY_PROMPT",
				description : "<%=_T('We are sorry, we cannot process your call right now')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('We are sorry, we cannot process your call right now is required.')%>";
				}
			},
			{
				id : "RESCHEDULE_PROMPT",
				description : "<%=_T('We must reschedule your call')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('We must reschedule your call is required.')%>";
				}
			},
			{
				id : "WELCOME_PROMPT",
				description : "<%=_T('Welcome to ...')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome to is required.')%>";
				}
			},
			{
				id : "WAIT_PROMPT",
				description : "<%=_T('While you wait [play this music/message]')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('While you wait [play this music/message] is required.')%>";
				}
			},
			{
				id : "YOUREINPOSITION_PROMPT",
				description : "<%=_T('You are in the position [number]')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('You are in the position [number] is required.')%>";
				}
			},
			{
				id : "VOICEMAIL_PROMPT",
				description : "<%=_T('You have [n] seconds to leave a message')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('You have [n] seconds to leave a message is required.')%>";
				}
			},
			{
				id : "YOUMUSTWAIT_PROMPT",
				description : "<%=_T('You must wait [time] to be served')%>",
				hasBuiltIn : false,
				allowCustom : false,
				allowFileUpload : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('You must wait [time] to be served is required.')%>";
				}
			}
		];
		call_allegro_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit( call_allegro_config, account_config );

	if ( typeof inConcert.adm_form_campaigns_call_allegro_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_call_allegro_account_messages = call_allegro_config;
	}

	call_allegro_config.prototype.__make_upload_uri = function( file )
	{
		var file_id = inConcert.Utils.generateGuid();
		var extension = inConcert.Utils.getFileExtension( file.name ) || "";
		var campaign = this.frm.get_entity_instance();

		var uri = format("{0}/inconcert/apps/allegro/repository/upload/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}&campaign={6}",
			inConcert.app.getBaseDomain(),
			encodeURIComponent( file_id ),
			"1", /* section */
			file.type,
			extension, /* format */
			file.name,
			campaign.Id
		);

		return {
			uri: uri,
			file_id: file_id
		};
	};

	call_allegro_config.prototype.__make_delete_uri = function( upload_id )
	{
		var campaign = this.frm.get_entity_instance();
		return format(
			"{0}/inconcert/apps/allegro/repository/remove/?id={1}&section={2}&campaign={3}", /* uri         */
			inConcert.app.getBaseDomain()                                                  , /* domain      */
			encodeURIComponent( upload_id )                                                , /* upload guid */
			"1"                                                                            , /* section     */
			campaign.Id                                                                      /* campaign id */
		);
	};

	call_allegro_config.prototype.__make_multi_delete_uri = function()
	{
		return inConcert.app.getBaseDomain() + '/inconcert/apps/allegro/repository/remove_all/';
	};

	call_allegro_config.prototype.__make_download_uri = $.noop;

	call_allegro_config.prototype.must_shown_player = function()
	{
		return false;
	};

/*
----------------------------------------------------------------------------------------------
  Clase - 'facebook_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function facebook_config(params)
	{
		var self = this;
		self.type = "FACEBOOK";
		self.section_id = "adm_form_campaign_accounts_facebook_panel_tab_messages";

		self.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to talk with you')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome Message is required.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				finalizeInteraction : "0",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Out of Hours Message is required.')%>";
				}
			},
			{
				id : "finalization_message",
				description : "<%=_T('Finalization Message')%>",
				markup: function( message, control_id ) {
					return self.finalization_markup( message, control_id );
				},
				init: function( message, control_id ) {
					self.finalization_init( message, control_id );
				},
				set: function( message ) {
					self.finalization_set( message );
				},
				validate: function( message ) {
					return self.finalization_validate( message );
				},
				error_message: function( error_type, data )
				{
					if ( data && typeof data.message === "string" ) {
						return data.message;
					}
					switch( error_type )
					{
						case "required-send-type":
							return "<%=_T('Send Type Finalization Message is required (Automatic or Automatic by Disposition).')%>";
				
						case "required-message":
							return "<%=_T('Finalization Message is required.')%>";
					}
					return "<%=_T('Finalization Message error.')%>";
				}
			}
		];
		facebook_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(facebook_config, account_config);

	if ( typeof inConcert.adm_form_campaigns_facebook_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_facebook_account_messages = facebook_config;
	}
	facebook_config.prototype.on_created_fields = function(success)
	{
		if (!success) {
			return;
		}

		$("#chkFACEBOOK_finalization_message").change(function()
		{
			if ($(this).prop("checked")) {
				$("#FACEBOOK_finalization_message_container").show();
			}
			else {
				$("#FACEBOOK_finalization_message_container").hide();
			}
		});
	};

	facebook_config.prototype.finalization_markup = function( message, control_id )
	{
		return ''.concat(
			'<div class="col-12 mt-3" id="', control_id, '_container" style="display:none">',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_send_type" class="required">', "<%=_T('Send Type')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_send_type" class="form-control formComboInput"></select>',
							'<i id="', control_id, '_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
						
						'</div>',
					'</div>',
				'</div>',

				'<div class="row bigRow">',
					'<div class="col-12">',
						'<div class="form-group">',
							'<label class="required">', '<%=_T("Message")%>' , '<%=_T(":")%>', '</label>',
							'<textarea id="', control_id, '_message" class="form-control rowsBased" rows="10"></textarea>',
							'<i id="', control_id, '_messageValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

			'</div>'
		);
	};

	facebook_config.prototype.finalization_init = function( message, control_id )
	{

		// fill combo send types
		const $send_type = $( "#" + control_id + "_send_type" );
		let send_list = [
			{ id: "AUTOMATIC"               , description: "<%=_T('Automatic')%>"                },
			{ id: "AUTOMATIC_BY_DISPOSITION", description: "<%=_T('Automatic by Disposition')%>" }
		];
		this.frm.fill_combo( $send_type, true, send_list, "id", "description" );

	};

	facebook_config.prototype.finalization_set = function( message )
	{
		let control_id = this.make_control_id( message );

		// send
		let send_type = message.sendType.type;
		$( "#" + control_id + "_send_type" ).val( send_type );
		// message
		$( "#" + control_id + "_message" ).val( message.body );
	};

	facebook_config.prototype.finalization_validate = function( message )
	{
		let current_message = { id: message.id, type: 'custom' };
		let control_id = this.make_control_id( message );

		// seteo y valido el tipo de send 
		let $send_type = $( '#' + control_id + '_send_type' );
		let send_type = $send_type.val();
		if ( send_type.trim().length === 0 ) {
			current_message.error = {
				$field : $send_type,
				data   : { type: 'required-send-type' }
			};
			return current_message;
		}

		current_message.sendType   = { type : send_type };

		// valida los requeridos
		var validate_req = function( $text, type ) {
			if ( $text.val().trim().length === 0 ) {
				return {
					$field : $text,
					data   : { type: type }
				};
			}
		};

		// valido el mensaje
		let $message = $( '#' + control_id + '_message' );
		current_message.error = validate_req( $message, "required-message" );
		if ( current_message.error ) {
			return current_message;
		}
		//Mantenemos el campo body porque compartimos codigo lua con canal mail
		current_message.body = $message.val();
		
		// is ok
		return current_message;
	};

/*
----------------------------------------------------------------------------------------------
  Clase - 'twitter_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
function twitter_config(params)
{
	var self = this;
	self.type = "TWITTER";
	self.section_id = "adm_form_campaign_accounts_twitter_panel_tab_messages";

	self.messages = [
		{
			id : "welcome_message",
			description : "<%=_T('Welcome Message')%>",
			builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to talk with you')%>",
			hasBuiltIn : true,
			allowCustom : true,
			checked: false,
			error_message: function( /*error_type*/ ) {
				return "<%=_T('Welcome Message is required.')%>";
			}
		},
		{
			id : "out_of_hours",
			description : "<%=_T('Out of Hours Message')%>",
			finalizeInteraction : "0",
			builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
			hasBuiltIn : true,
			allowCustom : true,
			checked: false,
			error_message: function( /*error_type*/ ) {
				return "<%=_T('Out of Hours Message is required.')%>";
			}
		},
		{
			id : "finalization_message",
			description : "<%=_T('Finalization Message')%>",
			markup: function( message, control_id ) {
				return self.finalization_markup( message, control_id );
			},
			init: function( message, control_id ) {
				self.finalization_init( message, control_id );
			},
			set: function( message ) {
				self.finalization_set( message );
			},
			validate: function( message ) {
				return self.finalization_validate( message );
			},
			error_message: function( error_type, data )
			{
				if ( data && typeof data.message === "string" ) {
					return data.message;
				}
				switch( error_type )
				{
					case "required-send-type":
						return "<%=_T('Send Type Finalization Message is required (Automatic or Automatic by Disposition).')%>";
			
					case "required-message":
						return "<%=_T('Finalization Message is required.')%>";
				}
				return "<%=_T('Finalization Message error.')%>";
			}
		}
	];
	twitter_config.uber.constructor.call(this, params);
}

inConcert.Utils.inherit(twitter_config, account_config);

if ( typeof inConcert.adm_form_campaigns_twitter_account_messages !== "function" ) {
	inConcert.adm_form_campaigns_twitter_account_messages = twitter_config;
}
twitter_config.prototype.on_created_fields = function(success)
{
	if (!success) {
		return;
	}

	$("#chkTWITTER_finalization_message").change(function()
	{
		if ($(this).prop("checked")) {
			$("#TWITTER_finalization_message_container").show();
		}
		else {
			$("#TWITTER_finalization_message_container").hide();
		}
	});
};

twitter_config.prototype.finalization_markup = function( message, control_id )
{
	return ''.concat(
		'<div class="col-12 mt-3" id="', control_id, '_container" style="display:none">',

			'<div class="row">',
				'<div class="col-6">',
					'<div class="form-group">',
						'<label for="', control_id, '_send_type" class="required">', "<%=_T('Send Type')%>" , "<%=_T(':')%>", '</label>',
						'<select id="', control_id, '_send_type" class="form-control formComboInput"></select>',
						'<i id="', control_id, '_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
					
					'</div>',
				'</div>',
			'</div>',

			'<div class="row bigRow">',
				'<div class="col-12">',
					'<div class="form-group">',
						'<label class="required">', "<%=_T('Message')%>" , "<%=_T(':')%>", '</label>',
						'<textarea id="', control_id, '_message" class="form-control rowsBased" rows="10"></textarea>',
						'<i id="', control_id, '_messageValidation" class="formFieldValidation formFieldError" hidden></i>',
					'</div>',
				'</div>',
			'</div>',

		'</div>'
	);
};

twitter_config.prototype.finalization_init = function( message, control_id )
{

	// fill combo send types
	const $send_type = $( "#" + control_id + "_send_type" );
	let send_list = [
		{ id: "AUTOMATIC"               , description: "<%=_T('Automatic')%>"                },
		{ id: "AUTOMATIC_BY_DISPOSITION", description: "<%=_T('Automatic by Disposition')%>" }
	];
	this.frm.fill_combo( $send_type, true, send_list, "id", "description" );

};

twitter_config.prototype.finalization_set = function( message )
{
	let control_id = this.make_control_id( message );

	// send
	let send_type = message.sendType.type;
	$( "#" + control_id + "_send_type" ).val( send_type );
	// message
	$( "#" + control_id + "_message" ).val( message.body );
};

twitter_config.prototype.finalization_validate = function( message )
{
	let current_message = { id: message.id, type: 'custom' };
	let control_id = this.make_control_id( message );

	// seteo y valido el tipo de send 
	let $send_type = $( '#' + control_id + '_send_type' );
	let send_type = $send_type.val();
	if ( send_type.trim().length === 0 ) {
		current_message.error = {
			$field : $send_type,
			data   : { type: 'required-send-type' }
		};
		return current_message;
	}

	current_message.sendType   = { type : send_type };

	// valida los requeridos
	var validate_req = function( $text, type ) {
		if ( $text.val().trim().length === 0 ) {
			return {
				$field : $text,
				data   : { type: type }
			};
		}
	};

	// valido el mensaje
	let $message = $( '#' + control_id + '_message' );
	current_message.error = validate_req( $message, "required-message" );
	if ( current_message.error ) {
		return current_message;
	}
	//Mantenemos el campo body porque compartimos codigo lua con canal mail
	current_message.body = $message.val();
	
	// is ok
	return current_message;
};
	
/*
----------------------------------------------------------------------------------------------
  Clase - 'instagram_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function instagram_config(params)
	{
		var self = this;
		self.type = "INSTAGRAM";
		self.section_id = "adm_form_campaign_accounts_instagram_panel_tab_messages";

		self.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to talk with you')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome Message is required.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				finalizeInteraction : "0",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Out of Hours Message is required.')%>";
				}
			},
			{
				id : "finalization_message",
				description : "<%=_T('Finalization Message')%>",
				markup: function( message, control_id ) {
					return self.finalization_markup( message, control_id );
				},
				init: function( message, control_id ) {
					self.finalization_init( message, control_id );
				},
				set: function( message ) {
					self.finalization_set( message );
				},
				validate: function( message ) {
					return self.finalization_validate( message );
				},
				error_message: function( error_type, data )
				{
					if ( data && typeof data.message === "string" ) {
						return data.message;
					}
					switch( error_type )
					{
						case "required-send-type":
							return "<%=_T('Send Type Finalization Message is required (Automatic or Automatic by Disposition).')%>";
				
						case "required-message":
							return "<%=_T('Finalization Message is required.')%>";
					}
					return "<%=_T('Finalization Message error.')%>";
				}
			}
		];
		instagram_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(instagram_config, account_config);

	if ( typeof inConcert.adm_form_campaigns_instagram_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_instagram_account_messages = instagram_config;
	}
	instagram_config.prototype.on_created_fields = function(success)
	{
		if (!success) {
			return;
		}

		$("#chkINSTAGRAM_finalization_message").change(function()
		{
			if ($(this).prop("checked")) {
				$("#INSTAGRAM_finalization_message_container").show();
			}
			else {
				$("#INSTAGRAM_finalization_message_container").hide();
			}
		});
	};

	instagram_config.prototype.finalization_markup = function( message, control_id )
	{
		return ''.concat(
			'<div class="col-12 mt-3" id="', control_id, '_container" style="display:none">',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_send_type" class="required">', "<%=_T('Send Type')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_send_type" class="form-control formComboInput"></select>',
							'<i id="', control_id, '_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
						
						'</div>',
					'</div>',
				'</div>',

				'<div class="row bigRow">',
					'<div class="col-12">',
						'<div class="form-group">',
							'<label class="required">', "<%=_T('Message')%>" , "<%=_T(':')%>", '</label>',
							'<textarea id="', control_id, '_message" class="form-control rowsBased" rows="10"></textarea>',
							'<i id="', control_id, '_messageValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

			'</div>'
		);
	};

	instagram_config.prototype.finalization_init = function( message, control_id )
	{

		// fill combo send types
		const $send_type = $( "#" + control_id + "_send_type" );
		let send_list = [
			{ id: "AUTOMATIC"               , description: "<%=_T('Automatic')%>"                },
			{ id: "AUTOMATIC_BY_DISPOSITION", description: "<%=_T('Automatic by Disposition')%>" }
		];
		this.frm.fill_combo( $send_type, true, send_list, "id", "description" );

	};

	instagram_config.prototype.finalization_set = function( message )
	{
		let control_id = this.make_control_id( message );

		// send
		let send_type = message.sendType.type;
		$( "#" + control_id + "_send_type" ).val( send_type );
		// message
		$( "#" + control_id + "_message" ).val( message.body );
	};

	instagram_config.prototype.finalization_validate = function( message )
	{
		let current_message = { id: message.id, type: 'custom' };
		let control_id = this.make_control_id( message );

		// seteo y valido el tipo de send 
		let $send_type = $( '#' + control_id + '_send_type' );
		let send_type = $send_type.val();
		if ( send_type.trim().length === 0 ) {
			current_message.error = {
				$field : $send_type,
				data   : { type: 'required-send-type' }
			};
			return current_message;
		}

		current_message.sendType   = { type : send_type };

		// valida los requeridos
		var validate_req = function( $text, type ) {
			if ( $text.val().trim().length === 0 ) {
				return {
					$field : $text,
					data   : { type: type }
				};
			}
		};

		// valido el mensaje
		let $message = $( '#' + control_id + '_message' );
		current_message.error = validate_req( $message, "required-message" );
		if ( current_message.error ) {
			return current_message;
		}
		//Mantenemos el campo body porque compartimos codigo lua con canal mail
		current_message.body = $message.val();
		
		// is ok
		return current_message;
	};
/*
----------------------------------------------------------------------------------------------
  Clase - 'youtube_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function youtube_config(params)
	{
		var self = this;
		self.type = "YOUTUBE";
		self.section_id = "adm_form_campaign_accounts_youtube_panel_tab_messages";

		self.messages = [
			{
				id : "private_message",
				description : "<%=_T('Private message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with us, we are going to answer as soon as possible.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked : false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Private message is required.')%>";
				}
			}
		];
		youtube_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(youtube_config, account_config);

/*
----------------------------------------------------------------------------------------------
  Clase - 'appstore_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function appstore_config(params)
	{
		var self = this;
		self.type = "APPSTORE";
		self.section_id = "adm_form_campaign_accounts_youtube_panel_tab_messages";

		self.messages = [
			{
				id : "private_message",
				description : "<%=_T('Private message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with us, we are going to answer as soon as possible.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked : false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Private message is required.')%>";
				}
			}
		];
		appstore_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(appstore_config, account_config);

/*
----------------------------------------------------------------------------------------------
  Clase - 'googleplay_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function googleplay_config(params)
	{
		var self = this;
		self.type = "GOOGLEPLAY";
		self.section_id = "adm_form_campaign_accounts_youtube_panel_tab_messages";

		self.messages = [
			{
				id : "private_message",
				description : "<%=_T('Private message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with us, we are going to answer as soon as possible.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked : false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Private message is required.')%>";
				}
			}
		];
		googleplay_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(googleplay_config, account_config);

/*
----------------------------------------------------------------------------------------------
  Clase - 'rcs_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function rcs_config(params)
	{
		var self = this;
		self.type = "RCS";
		self.section_id = "adm_form_campaign_accounts_youtube_panel_tab_messages";

		self.messages = [
			{
				id : "private_message",
				description : "<%=_T('Private message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with us, we are going to answer as soon as possible.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked : false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Private message is required.')%>";
				}
			}
		];
		rcs_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(rcs_config, account_config);

/*
----------------------------------------------------------------------------------------------
  Clase - 'im_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function im_config(params)
	{
		this.type = "IM";
		this.section_id = "adm_form_campaign_accounts_im_panel_tab_messages";
		this.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to chat with you.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "announce_chatting_agent",
				description : "<%=_T('Announce Chatting Agent')%>",
				builtInMessage: "<%=_T('You are now chatting with {AGENTNAME}.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "announce_agent_added",
				description : "<%=_T('Announce Agent Added')%>",
				builtInMessage: "<%=_T('{AGENTNAME} has joined the conversation.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "announce_agent_left",
				description : "<%=_T('Announce Agent Left')%>",
				builtInMessage: "<%=_T('Agent {AGENTNAME} has left the conversation.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "waiting_message",
				description : "<%=_T('Waiting Message')%>",
				builtInMessage: "<%=_T('Please wait, you are queued on place {PLACE_ON_QUEUE}.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "without_agent_message",
				description : "<%=_T('Without Agent Message')%>",
				builtInMessage: "<%=_T('Please wait, we are searching for the best person to chat with you.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "on_dangling_message",
				description : "<%=_T('On hanging up message')%>",
				builtInMessage: "<%=_T('Session closed. Please try again.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "conversation_timedout",
				description : "<%=_T('On Conversation Timeout')%>",
				builtInMessage: "<%=_T('Thanks for your time, if you have any other inquiries please contact us again!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			}
		];
		im_config.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( im_config, account_config );

	if ( typeof inConcert.adm_form_campaigns_im_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_im_account_messages = im_config;
	}

/*
----------------------------------------------------------------------------------------------
  Clase - 'mail_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function mail_config(params)
	{
		var self = this;

		this.type = "MAIL";
		this.section_id = "adm_form_campaign_messages_mail_panel";

		this.set_custom_controls = function() {
			//seteo eventos
			$("#mailRuleUseAutoResponse").change(function()
			{
				if ($(this).prop("checked")) {
					$("#mailAutoResponseFields").show();
					$("#mailRuleAutoResponseSubject").focus();
				}
				else {
					$("#mailAutoResponseFields").hide();
				}
			});

			$("#mailRuleOutOfHours").change(function()
			{
				if ($(this).prop("checked")) {
					$("#mailRuleOutOfHoursFields").show();
					$("#mailRuleOutOfHoursSubject").focus();
				}
				else {
					$("#mailRuleOutOfHoursFields").hide();
				}
			});

			$("#mailRuleFinalizationResponse").change(function()
			{
				if ($(this).prop("checked")) {
					$("#mailRuleFinalizationResponseFields").show();
					var $send_type = $("#mailRuleFinalizationSendType");
					if ($send_type.length > 0 && $send_type.is(":visible")) {
						$send_type.focus();
					}
					else {
						$("#mailRuleFinalizationResponseSubject").focus();
					}
				}
				else {
					$("#mailRuleFinalizationResponseFields").hide();
				}
			});
		};

		this.get_custom_messages = function()
		{
			return [
				{
					id: "rule_use_auto_response",
					control_id: "mailRuleUseAutoResponse",
					is_parent: true,
					dependentControls: [
						{
							id: "auto_response",
							control_id: "mailRuleAutoResponseSubject",
							parent_control_id: "mailRuleUseAutoResponse",
							is_parent: false,
							fieldName: "subject",
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Subject is required.')%>";
							}
						},
						{
							id: "auto_response",
							control_id: "mailRuleAutoResponseMessage",
							parent_control_id: "mailRuleUseAutoResponse",
							is_parent: false,
							fieldName: "body",
							get_value: function() {
								var result = {success: false};
								self.mail_mng.mailRuleUseAutoResponse.validate(function(validation_result) {
									result = validation_result;
								});
								if (!result.success) {
									return;
								}
								var body_parts = self.mail_mng.mailRuleUseAutoResponse.get_body_parts();
								return body_parts;
							},
							set_value: function(_, body, info) {
								if (info && info.contentType === "text/html") {
									self.mail_mng.mailRuleUseAutoResponse.set_body_parts(body);
								}
								else {
									self.mail_mng.mailRuleUseAutoResponse.set(body);
								}
							},
							attach_custom_value: function(message) {
								if (typeof(message) === "object") {
									message.contentType = "text/html";
									var images_list = self.mail_mng.mailRuleUseAutoResponse.get_repository_images_list();
									message.attachments = JSON.stringify(images_list);
								}
							},
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Message is required.')%>";
							}
						}
					]
				},
				{
					id: "rule_finalization_message",
					control_id: "mailRuleFinalizationResponse",
					is_parent: true,
					dependentControls: [
						{
							id: "finalization_message", // <-- this ends up in Cassandra
							control_id: "mailRuleFinalizationSendType",
							parent_control_id: "mailRuleFinalizationResponse",
							is_parent: false,
							get_value: function($field) {
								if (!$field || $field.length === 0) {
									return;
								}
								var value = $field.val() || "";
								if (value.trim().length === 0) {
									return;
								}
								return { type: value };
							},
							set_value: function($field, value) {
								if (!$field || $field.length === 0) {
									return;
								}
								if (typeof(value) === "object") {
									$field.val(value.type || "");
								}
								else {
									$field.val("");
								}
							},
							fieldName: "sendType", // <-- this ends up in Cassandra
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Send Type is required.')%>";
							}
						},
						{
							id: "finalization_message", // <-- this ends up in Cassandra
							control_id: "mailRuleFinalizationResponseSubject",
							parent_control_id: "mailRuleFinalizationResponse",
							is_parent: false,
							fieldName: "subject", // <-- this ends up in Cassandra
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Subject is required.')%>";
							}
						},
						{
							id: "finalization_message",
							control_id: "mailRuleFinalizationResponseMessage",
							parent_control_id: "mailRuleFinalizationResponse",
							is_parent: false,
							fieldName: "body", // <-- this ends up in Cassandra
							get_value: function() {
								var result = {success: false};
								self.mail_mng.mailRuleFinalizationResponse.validate(function(validation_result) {
									result = validation_result;
								});
								if (!result.success) {
									return;
								}
								var body_parts = self.mail_mng.mailRuleFinalizationResponse.get_body_parts();
								return body_parts;
							},
							set_value: function(_, body, info) {
								if (info && info.contentType === "text/html") {
									self.mail_mng.mailRuleFinalizationResponse.set_body_parts(body);
								}
								else {
									self.mail_mng.mailRuleFinalizationResponse.set(body);
								}
							},
							attach_custom_value: function(message) {
								if (typeof(message) === "object") {
									message.contentType = "text/html";
									var images_list = self.mail_mng.mailRuleFinalizationResponse.get_repository_images_list();
									message.attachments = JSON.stringify(images_list);
								}
							},
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Message is required.')%>";
							}
						}
					]
				},
				{
					id: "rule_out_of_hours",
					control_id: "mailRuleOutOfHours",
					is_parent: true,
					dependentControls: [
						{
							id: "out_of_hours",
							control_id: "mailRuleOutOfHoursSubject",
							parent_control_id: "mailRuleOutOfHours",
							is_parent: false,
							fieldName: "subject",
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Subject is required.')%>";
							}
						},
						{
							id: "out_of_hours",
							control_id: "mailRuleOutOfHoursMessage",
							parent_control_id: "mailRuleOutOfHours",
							is_parent: false,
							fieldName: "body",
							get_value: function() {
								var result = {success: false};
								self.mail_mng.mailRuleOutOfHours.validate(function(validation_result) {
									result = validation_result;
								});
								if (!result.success) {
									return;
								}
								var body_parts = self.mail_mng.mailRuleOutOfHours.get_body_parts();
								return body_parts;
							},
							set_value: function(_, body, info) {
								if (info && info.contentType === "text/html") {
									self.mail_mng.mailRuleOutOfHours.set_body_parts(body);
								}
								else {
									self.mail_mng.mailRuleOutOfHours.set(body);
								}
							},
							attach_custom_value: function(message) {
								if (typeof(message) === "object") {
									message.contentType = "text/html";
									var images_list = self.mail_mng.mailRuleOutOfHours.get_repository_images_list();
									message.attachments = JSON.stringify(images_list);
								}
							},
							error_message: function( /*error_type*/ ) {
								return "<%=_T('Message is required.')%>";
							}
						}
					]
				},
			];
		};
		this.on_validate = function() {
			return this.custom_validate();
		};
		this.load_custom_current_config = function() {
			this.load_custom_config();
		};

		mail_config.uber.constructor.call(this, params);

		this.mail_mng = {
			mailRuleUseAutoResponse: new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "mailRuleAutoResponseMessage",
					container: "mailRuleAutoResponseMessageContainer"
				}
			}),

			mailRuleFinalizationResponse: new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "mailRuleFinalizationResponseMessage",
					container: "mailRuleFinalizationResponseMessageContainer"
				}
			}),

			mailRuleOutOfHours: new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "mailRuleOutOfHoursMessage",
					container: "mailRuleOutOfHoursMessageContainer"
				}
			})
		};

		// fill combo send type
		var $send_type = $("#mailRuleFinalizationSendType");
		var send_list = [
			{ id: "AUTOMATIC"               , description: "<%=_T('Automatic')%>"                },
			{ id: "AUTOMATIC_BY_DISPOSITION", description: "<%=_T('Automatic by Disposition')%>" }
		];
		this.frm.fill_combo( $send_type, false, send_list, "id", "description" );
	}

	inConcert.Utils.inherit( mail_config, account_config );

	if ( typeof inConcert.adm_form_campaigns_mail_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_mail_account_messages = mail_config;
	}

/*
----------------------------------------------------------------------------------------------
  Clase - 'sms_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function sms_config(params)
	{
		this.type = "SMS";
		this.section_id = "adm_form_campaign_messages_sms_panel";
		this.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to talk with you')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "announce_chatting_agent",
				description : "<%=_T('Announce Chatting Agent')%>",
				builtInMessage: "<%=_T('You are now chatting with {AGENTNAME}')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "without_agent_message",
				description : "<%=_T('Without Agent Message')%>",
				builtInMessage: "<%=_T('Please wait, we are searching for the best person to chat with you.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "on_dangling_message",
				description : "<%=_T('On hanging up message')%>",
				builtInMessage: "<%=_T('Session closed. Please try again.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			},
			{
				id : "on_conversation_finished",
				description : "<%=_T('On Conversation Finished')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with us.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('TU MENSAJE DE ERROR.')%>";
				}
			}
		];
		sms_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(sms_config, account_config);

/*
----------------------------------------------------------------------------------------------
  Clase - 'whatsapp_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function whatsapp_config(params)
	{
		const self = this;
		this.type = "WHATSAPP";
		this.section_id = "adm_form_campaign_messages_whatsapp_panel";

		this.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to talk with you')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome Message is required.')%>";
				}
			},
			{
				id : "waiting_message",
				description : "<%=_T('Waiting Message')%>",
				builtInMessage: "<%=_T('Please wait, you are queued on place {PLACE_ON_QUEUE}')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Waiting Message is required.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				finalizeInteraction : "0",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Out of Hours Message is required.')%>";
				}
			},
			{
				id : "finalization_message",
				description : "<%=_T('Finalization Message')%>",
				markup: function( message, control_id ) {
					return self.finalization_markup( message, control_id );
				},
				init: function( message, control_id ) {
					self.finalization_init( message, control_id );
				},
				set: function( message ) {
					self.finalization_set( message );
				},
				validate: function( message ) {
					return self.finalization_validate( message );
				},
				error_message: function( error_type, data )
				{
					if ( data && typeof data.message === "string" ) {
						return data.message;
					}
					switch( error_type )
					{
						case "required-send-type":
							return "<%=_T('Send Type Finalization Message is required (Automatic or Automatic by Disposition).')%>";
				
						case "required-message":
							return "<%=_T('Finalization Message is required.')%>";
					}
					return "<%=_T('Finalization Message error.')%>";
				}
			}
		];

		whatsapp_config.uber.constructor.call(this, params);	
	}

	inConcert.Utils.inherit(whatsapp_config, account_config);

	if ( typeof inConcert.adm_form_campaigns_whatsapp_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_whatsapp_account_messages = whatsapp_config;
	}

	whatsapp_config.prototype.on_created_fields = function(success)
	{
		if (!success) {
			return;
		}

		$("#chkWHATSAPP_finalization_message").change(function()
		{
			if ($(this).prop("checked")) {
				$("#WHATSAPP_finalization_message_container").show();
			}
			else {
				$("#WHATSAPP_finalization_message_container").hide();
			}
		});
	};

	whatsapp_config.prototype.finalization_markup = function( message, control_id )
	{
		return ''.concat(
			'<div class="col-12 mt-3" id="', control_id, '_container" style="display:none">',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_send_type" class="required">', "<%=_T('Send Type')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_send_type" class="form-control formComboInput"></select>',
							'<i id="', control_id, '_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
						
						'</div>',
					'</div>',
				'</div>',

				'<div class="row bigRow">',
					'<div class="col-12">',
						'<div class="form-group">',
							'<label class="required">', "<%=_T('Message')%>" , "<%=_T(':')%>", '</label>',
							'<textarea id="', control_id, '_message" class="form-control rowsBased" rows="10"></textarea>',
							'<i id="', control_id, '_messageValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

			'</div>'
		);
	};

	whatsapp_config.prototype.finalization_init = function( message, control_id )
	{

		// fill combo send types
		const $send_type = $( "#" + control_id + "_send_type" );
		let send_list = [
			{ id: "AUTOMATIC"               , description: "<%=_T('Automatic')%>"                },
			{ id: "AUTOMATIC_BY_DISPOSITION", description: "<%=_T('Automatic by Disposition')%>" }
		];
		this.frm.fill_combo( $send_type, true, send_list, "id", "description" );

	};

	whatsapp_config.prototype.finalization_set = function( message )
	{
		let control_id = this.make_control_id( message );

		// send
		let send_type = message.sendType.type;
		$( "#" + control_id + "_send_type" ).val( send_type );
		// message
		$( "#" + control_id + "_message" ).val( message.body );
	};

	whatsapp_config.prototype.finalization_validate = function( message )
	{
		//let current_message = { id: message.id, type: 'custom', subject: '', contentType : "text\/html", attachments : [] };
		let current_message = { id: message.id, type: 'custom' };
		let control_id = this.make_control_id( message );

		// seteo y valido el tipo de send 
		let $send_type = $( '#' + control_id + '_send_type' );
		let send_type = $send_type.val();
		if ( send_type.trim().length === 0 ) {
			current_message.error = {
				$field : $send_type,
				data   : { type: 'required-send-type' }
			};
			return current_message;
		}

		current_message.sendType   = { type : send_type };

		// valida los requeridos
		var validate_req = function( $text, type ) {
			if ( $text.val().trim().length === 0 ) {
				return {
					$field : $text,
					data   : { type: type }
				};
			}
		};

		// valido el mensaje
		let $message = $( '#' + control_id + '_message' );
		current_message.error = validate_req( $message, "required-message" );
		if ( current_message.error ) {
			return current_message;
		}
		//Mantenemos el campo body porque compartimos codigo lua con canal mail
		current_message.body = $message.val();
		
		// is ok
		return current_message;
	};

/*
----------------------------------------------------------------------------------------------
  Clase - 'web_chat_config' ---> hereda de la clase 'account_config'!!!!!!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------
*/
	function web_chat_config(params)
	{
		var self = this;
		this.type = "CHAT";
		this.section_id = "adm_form_campaign_accounts_webchat_panel_tab_messages";

		this.mail_mng = null;

		this.messages = [
			{
				id : "welcome_message",
				description : "<%=_T('Welcome Message')%>",
				builtInMessage: "<%=_T('Thanks for getting in touch with {GROUPNAME}, we are searching for the best person to chat with you.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Welcome Message is required.')%>";
				}
			},
			{
				id : "finalization_message",
				description : "<%=_T('Finalization Message')%>",
				markup: function( message, control_id ) {
					return self.finalization_markup( message, control_id );
				},
				init: function( message, control_id ) {
					self.finalization_init( message, control_id );
				},
				editable: function( message, control_id ) {
					self.finalization_editable( message, control_id );
				},
				toggle: function( message, show ) {
					self.finalization_toggle( message, show );
				},
				set: function( message ) {
					self.finalization_set( message );
				},
				validate: function( message ) {
					return self.finalization_validate( message );
				},
				error_message: function( error_type, data )
				{
					if ( data && typeof data.message === "string" ) {
						return data.message;
					}
					switch( error_type )
					{
						case "required-send-type":
							return "<%=_T('Send Type Finalization Message is required (automatic or manual).')%>";

						case "required-manual-send-type":
							return "<%=_T('Manual Send Type Finalization Message is required.')%>";

						case "required-subject":
							return "<%=_T('Subject Finalization Message is required.')%>";

						case "required-body":
							return "<%=_T('Body Finalization Message is required.')%>";

						case "required-outgoing-mail":
							return "<%=_T('Outgoing Account is required.')%>";
					}
					return "<%=_T('Finalization Message error.')%>";
				}
			},
			{
				id : "out_of_hours",
				description : "<%=_T('Out of Hours Message')%>",
				finalizeInteraction : "-1",
				builtInMessage: "<%=_T('We are sorry, but there are no agents to attend you right now. Please contact us during our our operative hours. Thank you!')%>",
				hasBuiltIn : true,
				allowCustom : true,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Out of Hours Message is required.')%>";
				},
				editable: function( message, control_id ) {
					if(message.finalizeInteraction === "-1")
					{
						$( "#" + control_id + "_finalizeInteraction" ).prop("checked", true );
						$( "#" + control_id + "_finalizeInteraction" ).attr("disabled", true );
					}
				}
			},
			{
				id : "waiting_message",
				description : "<%=_T('Waiting Message')%>",
				builtInMessage: "<%=_T('Please wait, you are queued on place {PLACE_ON_QUEUE}')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Waiting Message is required.')%>";
				}
			},
			{
				id : "announce_chatting_agent",
				description : "<%=_T('Announce Chatting Agent')%>",
				builtInMessage: "<%=_T('You are now chatting with {AGENTNAME}')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Announce Chatting Agent is required.')%>";
				}
			},
			{
				id : "without_agent_message",
				description : "<%=_T('Without Agent Message')%>",
				builtInMessage: "<%=_T('Please wait, we are searching for the best person to chat with you.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('Without Agent Message is required.')%>";
				}
			},
			{
				id : "on_dangling_message",
				description : "<%=_T('On hanging up message')%>",
				builtInMessage: "<%=_T('Session closed. Please try again.')%>",
				hasBuiltIn : true,
				allowCustom : true,
				checked: false,
				error_message: function( /*error_type*/ ) {
					return "<%=_T('On hanging up message is required.')%>";
				}
			}
		];
		web_chat_config.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit( web_chat_config, account_config );

	if ( typeof inConcert.adm_form_campaigns_webchat_account_messages !== "function" ) {
		inConcert.adm_form_campaigns_webchat_account_messages = web_chat_config;
	}

	web_chat_config.prototype.on_created_fields = function(success)
	{
		if (!success) {
			return;
		}
		this.mail_mng = {
			finalization_message: new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "CHAT_finalization_message_body",
					container: "CHAT_finalization_message_body_container"
				}
			})
		};
	};

	web_chat_config.prototype.finalization_markup = function( message, control_id )
	{
		return ''.concat(
			'<div id="', control_id, '_container" style="display:none" class="col-12 mt-3">',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_send_type" class="required">', "<%=_T('Send Type')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_send_type" class="form-control formComboInput"></select>',
							'<i id="', control_id, '_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',

					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_manual_send_type">', "<%=_T('Control By')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_manual_send_type" class="form-control formComboInput" disabled="true"></select>',
							'<i id="', control_id, '_manual_send_typeValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label>', "<%=_T('BCC')%>" , "<%=_T(':')%>", '</label>',
							'<input id="', control_id, '_bcc" class="form-control" type="field" />',
							'<i id="', control_id, '_bccValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="row">',
					'<div class="col-12">',
						'<div class="form-group">',
							'<label class="required">', "<%=_T('Subject')%>" , "<%=_T(':')%>", '</label>',
							'<input id="', control_id, '_subject" class="form-control" type="field" />',
							'<i id="', control_id, '_subjectValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="row autoRow messagesTextEditorRow">',
					'<div class="col-12">',
						'<div class="form-group">',
							'<label class="required">', "<%=_T('Message')%>" , "<%=_T(':')%>", '</label>',
							'<i id="', control_id, '_bodyValidation" class="formFieldValidation formFieldError" hidden></i>',
							'<div id="', control_id, '_body_container">',
								'<textarea id="', control_id, '_body" class="form-control rowsBased" rows="10"></textarea>',
							'</div>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="row">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<label for="', control_id, '_outgoing_mail" class="required">', "<%=_T('Outgoing Account')%>" , "<%=_T(':')%>", '</label>',
							'<select id="', control_id, '_outgoing_mail" class="form-control formComboInput"></select>',
							'<i id="', control_id, '_outgoing_mailValidation" class="formFieldValidation formFieldError" hidden></i>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="row pt-2">',
					'<div class="col-6">',
						'<div class="form-group">',
							'<div class="form-check mt-0">' ,
								'<label for="' , control_id , '_transcription" class="form-check-label form-check-label-title">' ,
									'<input id="' , control_id , '_transcription" class="form-check-input" type="checkbox" />' ,
									'<%=_T("Include Transcription")%>' ,
								'</label>' ,
							'</div>' ,
						'</div>',
					'</div>',
				'</div>',

			'</div>'
		);
	};

	web_chat_config.prototype.finalization_init = function( message, control_id )
	{
		var self = this;

		// fill combo send types
		var $send_type = $( "#" + control_id + "_send_type" );
		var send_list = [
			{ id: "automatic" , description: "<%=_T('Automatic')%>" } ,
			{ id: "manual"    , description: "<%=_T('Manual')%>"    }
		];
		this.frm.fill_combo( $send_type, true, send_list, "id", "description" );

		// fill combo send manual types
		var $manual_send_type = $( "#" + control_id + "_manual_send_type" );
		var manual_list = [
			{ id: "agent"    , description: "<%=_T('Agent')%>"    } ,
			{ id: "customer" , description: "<%=_T('Customer')%>" }
		];
		this.frm.fill_combo( $manual_send_type, true, manual_list, "id", "description" );

		// fill combo outgoing mail address
		var $outgoing_mail = $( "#" + control_id + "_outgoing_mail" );
		var vcc_mail_list = this.frm.get_vcc_mail_addresses();
		this.frm.fill_combo( $outgoing_mail, true, vcc_mail_list, "uid", "uid" );

		// set events
		$send_type
			.off( "change" )
			.on( "change", function() {
				var disabled = true;
				var required;
				if ( $( this ).val() === "manual" ) {
					disabled = false;
					required = true;
				}
				self.frm.disable_control( $manual_send_type, disabled, required );
			});
	};

	web_chat_config.prototype.finalization_editable = function( message, control_id )
	{
		var $send_type = $( "#" + control_id + "_manual_send_type" );
		this.frm.disable_control( $send_type, true );
	};

	web_chat_config.prototype.finalization_toggle = function( message, show )
	{
		var control_id = this.make_control_id( message );
		var $container = $( '#' + control_id + '_container' );
		if ( show ) {
			$container.show();
			$( '#' + control_id + '_subject' ).focus();
		}
		else {
			$container.hide();
		}
	};

	web_chat_config.prototype.finalization_set = function( message )
	{
		var control_id = this.make_control_id( message );

		// send
		var send_type = message.automatic_send === "1" ? "automatic" : "manual";
		$( "#" + control_id + "_send_type" ).val( send_type );
		if ( send_type === "manual" ) {
			var manual_send_type = message.customer_control === "1" ? "customer" : "agent";
			$( "#" + control_id + "_manual_send_type" ).val( manual_send_type );
			$( "#" + control_id + "_send_type" ).trigger( "change" );
		}
		// bcc
		$( "#" + control_id + "_bcc" ).val( message.bcc );
		// subject
		$( "#" + control_id + "_subject" ).val( message.subject );
		// body
		$( "#" + control_id + "_body" ).val( message.body );
		var mail_mng = this.mail_mng[message.id];
		if (!mail_mng) {
			throw("web_chat_config.prototype.finalization_set - Message instance '" + message.id + "' does not exist.");
		}
		if ( message.contentType === "text/html" ) {
			mail_mng.set_body_parts(message.body);
		}
		else {
			mail_mng.set(message.body);
		}
		// outgoing mail
		$( "#" + control_id + "_outgoing_mail" ).val( message.outgoing_mail );
		// transcripcion
		$( "#" + control_id + "_transcription" ).prop( "checked", message.include_transcription === "1" );
	};

	web_chat_config.prototype.finalization_validate = function( message )
	{
		var current_message = { id: message.id, type: 'custom' };
		var control_id = this.make_control_id( message );

		// seteo y valido el tipo de send (manual/send) y si si es manual (customer/agent)
		var $send_type = $( '#' + control_id + '_send_type' );
		var send_type = $send_type.val();
		if ( send_type.trim().length === 0 ) {
			current_message.error = {
				$field : $send_type,
				data   : { type: 'required-send-type' }
			};
			return current_message;
		}

		current_message.automatic_send   = "0";
		current_message.agent_control    = "0";
		current_message.customer_control = "0";

		if ( send_type === "automatic" ) {
			current_message.automatic_send = "1";
		}
		else {
			var $manual_send_type = $( '#' + control_id + '_manual_send_type' );
			var manual_send_type = $manual_send_type.val();
			if ( manual_send_type.trim().length === 0 ) {
				current_message.error = {
					$field : $manual_send_type,
					data   : { type: 'required-manual-send-type' }
				};
				return current_message;
			}
			else if ( manual_send_type === "customer" ) {
				current_message.customer_control = "1";
			}
			else {
				current_message.agent_control    = "1";
			}
		}

		// valido el BCC (unicamente si ha escrito algo, No es requerido!!)
		var $bcc = $( '#' + control_id + '_bcc' );
		var bcc = $bcc.val().trim();

		if ( bcc.length > 0 ) {

			var mail_list = {
				bcc:  {
					addresses : bcc,
					get_error : function( result )
					{
						var message_error;
						var address = inConcert.Utils.truncateWord( result.address );
						switch( result.error_type )
						{
							case "invalid_mail":
								message_error = format( "<%=_T('BCC e-mail account must be a valid e-mail address: {0}.')%>", address );
								break;

							case "invalid_mail_list":
								message_error = format( "<%=_T('BCC e-mail account list must be a valid: {0}. Must use the separator ;. Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...')%>", address );

								break;
							default:
								throw "Invalid validation mail message type";
						}
						return message_error;
					}
				}
			};

			var err;
			var validation_error = function( result )
			{
				var email_type = mail_list[ result.type ];
				err = {
					$field :  $bcc,
					data   : { message: email_type.get_error( result ) }
				};
			};

			if ( !inConcert.Utils.validateEmails( mail_list, validation_error ) ) {
				current_message.error = err;
				return current_message;
			}
		}
		current_message.bcc = bcc;

		// valida los requeridos
		var validate_req = function( $text, type ) {
			if ( $text.val().trim().length === 0 ) {
				return {
					$field : $text,
					data   : { type: type }
				};
			}
		};

		// valido el subject
		var $subject = $( '#' + control_id + '_subject' );
		current_message.error = validate_req( $subject, "required-subject" );
		if ( current_message.error ) {
			return current_message;
		}
		current_message.subject = $subject.val();

		// valido el body
		var mail_mng = this.mail_mng[message.id];
		if (!mail_mng) {
			throw("web_chat_config.prototype.finalization_validate - Message instance '" + message.id + "' does not exist.");
		}

		var result = {success: false};
		mail_mng.validate(function(validation_result) {
			result = validation_result;
		});
		if (!result.success) {
			return result.message;
		}
		var body_parts = mail_mng.get_body_parts();
		current_message.body = body_parts;

		// content Type
		current_message.contentType = "text/html";
		// los attachments (las imagenes subidas al repo)
		var images_list = mail_mng.get_repository_images_list();
		current_message.attachments = JSON.stringify(images_list);

		// valido el outgoing e-mail address
		var $outgoing_mail = $( '#' + control_id + '_outgoing_mail' );
		current_message.error = validate_req( $outgoing_mail, "required-outgoing-mail" );
		if ( current_message.error ) {
			return current_message;
		}
		current_message.outgoing_mail = $outgoing_mail.val();

		// incluir la transcripcion
		current_message.include_transcription = get_chk_bool_to_string( control_id + '_transcription' );

		// is ok
		return current_message;
	};

}());
