/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 11 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//"use strict";
	const FORMAT = inConcert.Utils.format;

	let m_vcc_tts_config = null;
	let m_variables = null;

	const find_variable = function (variableId) {
		if (!m_variables || !variableId) {
			return;
		}
		for (let i = 0; i < m_variables.length; i++) {
			let variable = m_variables[i];
			if (variable.id === variableId) {
				return variable;
			}
		}
	};

	let m_variablesEngineTypes = {
		FILE: {description: "<%=_T('File (audio)')%>"  },
		TTS : {description: "<%=_T('Engine TTS/ASR')%>"}
	};

	const get_file_message_type_img = function(msg_type) {
		const src_def = {
			audio   : "/inconcert/apps/administration/images/audio.png",
			variable: "/inconcert/apps/administration/images/variable.png",
			tts     : "/inconcert/apps/administration/images/tts.png"
		};
		return src_def[msg_type] || "/inconcert/apps/agent/images/blank.png";
	};

	/*

	*/
	function base_audio_popup (params) {
		const self = this;
		this.popup_id = params.popup_id;
		params.callbacks = {
			on_close: function() {
				self.on_close();
			},
			on_save: function(item, create_and_continue, callback) {
				self.on_save(item, create_and_continue, callback);
			}
		};
		base_audio_popup.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(base_audio_popup, inConcert.conf_form_popup);

	base_audio_popup.prototype.set_events = function() {
		base_audio_popup.uber.set_events.call(this);
	};

	base_audio_popup.prototype.on_close = function() {
		this.owner.cleanup_edited_row(this.popup_id);
	};

	base_audio_popup.prototype.on_save = function(item, create_and_continue, callback = $.noop) {
		this.owner.add_row_to_grid(this.popup_id, item);
		let response;
		if (create_and_continue) {
			response = {};
			response.msg = "<%=_T('Has been added successfully.')%>";
		}
		callback(true, response);
	};

	/*

	*/
	function variable_audio_popup (params) {
		variable_audio_popup.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(variable_audio_popup, base_audio_popup);

	variable_audio_popup.prototype.get_data = function() {
		let item = variable_audio_popup.uber.get_data.call(this);

		let variable_engine;
		if (this.owner.variable_supports_tts(this.popup_id)) {
			variable_engine = $("#" + this.container_id).find("[adm_account_message_engine_variable_section] input[type='radio'][name='adm_account_message_engine_variable_value']:checked").val();
		}
		else {
			variable_engine = "FILE";
		}

		const $variable = $("#" + this.container_id + "_variable");

		item.Engine = variable_engine;
		item.MessageType = "variable";
		item.VariableType = "system";
		item.RawData = {};

		if (this.owner.is_enabled_custom_variables(this.popup_id)) {
			const is_selected_custom_variable = $("#" + this.container_id + "_variable_type_custom").prop("checked");
			if (is_selected_custom_variable) {
				const $custom_variable = $("#" + this.container_id + "_variable_custom");
				let variable_id = $custom_variable.val();
				item.VariableId = FORMAT("{{{0}}}", variable_id);
				item.__brackets_message = [];
				item.__brackets_message.push(variable_id);
				item.VariableType = "custom";
			}
		}

		if (item.VariableType === "system") {
			item.VariableId = $variable.val();
		}

		return item;
	};

	variable_audio_popup.prototype.edit = function(item) {
		variable_audio_popup.uber.edit.call(this, item);

		if (this.owner.variable_supports_tts(this.popup_id)) {
			if (item.Engine === "TTS") {
				const $engine_tts = $("#" + this.container_id + "_tts");
				$engine_tts.prop("checked", true);
			}
			else {
				const $engine_file = $("#" + this.container_id + "_file");
				$engine_file.prop("checked", true);
			}
		}
		else {
			this.set_default_values();
		}

		const $variable = $("#" + this.container_id + "_variable");
		$variable.val("").change();

		if (this.owner.is_enabled_custom_variables(this.popup_id)) {
			const $custom_variable = $("#" + this.container_id + "_variable_custom");
			$custom_variable.val("");
			$variable.val("");
			if (item.VariableType === "custom") {
				const $variable_type_custom = $("#" + this.container_id + "_variable_type_custom");
				$variable_type_custom.prop("checked", true).trigger("click");
				let array_parsed_value = inConcert.Utils.getBracketsValue(item.VariableId, {open: "{{", close: "}}"});
				if ((array_parsed_value || []).length === 1) {
					$custom_variable.val(array_parsed_value[0]);
				}
			}
			else {
				$variable.val(item.VariableId).change();;
			}
		}
		else {
			$variable.val(item.VariableId).change();;
		}

		$("#" + this.container_id + "_create").val("<%=_T('Update')%>");
		$("#" + this.container_id + "_create_and_continue").hide();
	};

	variable_audio_popup.prototype.ready = function() {
		this.set_default_values();
	};

	variable_audio_popup.prototype.set_default_values = function() {
		if (this.owner.variable_supports_tts(this.popup_id)) {
			const $engine_tts = $("#" + this.container_id + "_tts");
			$engine_tts.prop("checked", true);
		}
		else {
			const $engine_file = $("#" + this.container_id + "_file");
			$engine_file.prop("checked", true);
		}
		const $variable_system = $("#" + this.container_id + "_variable_type_system");
		$variable_system.prop("checked", true);
	};

	variable_audio_popup.prototype.get_html_middle = function() {
		const self = this;

		const get_html_tts_variable = function() {
			if (!self.owner.variable_supports_tts(self.popup_id)) {
				return "";
			}
			return "" +
				"<div id='" + self.container_id + "_variable_tts_container' class='form-radiobutton'>" +
					"<label for='" + self.container_id + "_tts' class='form-radiobutton-label'>" +
						"<input id='" + self.container_id + "_tts' type='radio' value='TTS' name='adm_account_message_engine_variable_value' class='form-radiobutton-input' />" +
						"<span>" + m_variablesEngineTypes.TTS.description + "</span>" +
					"</label>" +
				"</div>";
		};

		const get_html_custom_variable = function() {
			if (!self.owner.is_enabled_custom_variables(self.popup_id)) {
				return "";
			}
			return "" +
				"<div id='" + self.container_id + "_variable_custom_container' class='form-group' style='display:none;'>" +
					"<input id='" + self.container_id + "_variable_custom' type='text' class='form-control' />" +
					"<i id='" + self.container_id + "_variable_customValidation' mark_error='true' class='formFieldValidation formFieldError' hidden style='display:none'></i>" +
				"</div>";
		};

		const get_html_radiobutton_custom_variable = function() {
			if (!self.owner.is_enabled_custom_variables(self.popup_id)) {
				return "";
			}
			return "" +
				"<div id='" + self.container_id + "_variable_type_custom_container' class='form-radiobutton'>" +
					"<label for='" + self.container_id + "_variable_type_custom' class='form-radiobutton-label'>" +
						"<input id='" + self.container_id + "_variable_type_custom' type='radio' value='TTS' name='adm_account_message_engine_variable_type_value' class='form-radiobutton-input' />" +
						"<span>" + "<%=_T('Custom Variable')%>" + "</span>" +
					"</label>" +
				"</div>";
		};

		let html = "" +
			"<div class='card no-card'>" +
				"<div class='card-block'>" +
					"<div class='row' adm_account_message_engine_variable_type_section='true' style='display:" + (this.owner.is_enabled_custom_variables(this.popup_id) ? "" : "none") + ";'>" +
						"<div class='col-12'>" +
							"<div class='form-group'>" +
								"<div id='" + this.container_id + "_variable_type_system_container' class='form-radiobutton'>" +
									"<label for='" + this.container_id + "_variable_type_system' class='form-radiobutton-label'>" +
										"<input id='" + this.container_id + "_variable_type_system' type='radio' value='SYSTEM' name='adm_account_message_engine_variable_type_value' class='form-radiobutton-input' />" +
										"<span>" + "<%=_T('System Variable')%>" + "</span>" +
									"</label>" +
								"</div>" +
								get_html_radiobutton_custom_variable() +
							"</div>" +
						"</div>" +
					"</div>" +
					"<div class='row'>" +
						"<div class='col-12'>" +
							"<div class='form-group'>" +
								"<div id='" + this.container_id + "_variable_system_container' class='form-group'>" +
									"<select id='" + this.container_id + "_variable' class='form-control formComboInput'></select>" +
									"<i id='" + this.container_id + "_variableValidation' mark_error='true' class='formFieldValidation formFieldError' hidden style='display:none'></i>" +
								"</div>" +
								get_html_custom_variable() +
							"</div>" +
						"</div>" +
					"</div>" +
					"<div class='row' id='" + this.container_id + "_variable_description_container' style='display:none;'>" +
						"<div class='col-12'>" +
							"<div id='" + this.container_id + "_variable_description' class='inlineMessagesContainer messageInfo mt-0'>" +
							"</div>" +
						"</div>" +
					"</div>" +
					"<div class='row' adm_account_message_engine_variable_section='true'>" +
						"<div class='col-12'>" +
							"<div class='form-group'>" +
								get_html_tts_variable() +
								"<div id='" + this.container_id + "_variable_file_container' class='form-radiobutton'>" +
									"<label for='" + this.container_id + "_file' class='form-radiobutton-label'>" +
										"<input id='" + this.container_id + "_file' type='radio' value='FILE' name='adm_account_message_engine_variable_value' class='form-radiobutton-input' />" +
										"<span>" + m_variablesEngineTypes.FILE.description + "</span>" +
									"</label>" +
								"</div>" +
							"</div>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</div>";
		return html;
	};

	variable_audio_popup.prototype.validate = function() {
		const self = this;

		const showError = function ($ctrl, message) {
			self.display_field_error($ctrl);
			self.show_status_message(message, "Error", 5000);
		};

		const is_enabled_custom_variables = this.owner.is_enabled_custom_variables(this.popup_id);
		const $variable = $("#" + this.container_id + "_variable");
		let system_variable_value = $variable.val();
		let system_variable_error = "<%=_T('Variable type is required')%>";

		if (!is_enabled_custom_variables) {
			if (system_variable_value.trim().length === 0) {
				showError($variable, system_variable_error);
				return false;
			}
		}
		else {
			const is_selected_system_variable = $("#" + this.container_id + "_variable_type_system").prop("checked");
			if (is_selected_system_variable) {
				if (system_variable_value.trim().length === 0) {
					showError($variable, system_variable_error);
					return false;
				}
			}
			else {
				let system_variable_error = "<%=_T('Custom variable is required')%>";
				const $custom_variable = $("#" + this.container_id + "_variable_custom");
				let custom_variable_value = $custom_variable.val();
				if (custom_variable_value.trim().length === 0) {
					showError($custom_variable, system_variable_error);
					return false;
				}
			}
		}

		return true;
	};

	variable_audio_popup.prototype.load_data = function(callback = $.noop) {
		const self = this;

		const $variable = $("#" + this.container_id + "_variable");

		const load = function(success, variables) {
			if (!success) {
				throw("Unable to load loadVariablesForAccountMessages");
			}
			let customVariables = self.owner.m_customVariables;
			if ((customVariables || []).length > 0) {
				m_variables = variables.concat(customVariables);
			}
			else {
				m_variables = variables;
			}
			self.frm.fill_combo($variable, true, m_variables, "id", function(_, item) {
				return item.name;
			}, true);
			callback(true);
		};

		const ownerName = this.owner.get_owner_name();
		if (ownerName === "accounts messages") {
			inConcert.app.loadVariablesForAccountMessages(true, load, this.owner.can_include_account_messages_variable);
		}
		else {
			inConcert.app.loadVariablesForFlowDesigner(true, load);
		}
	};

	variable_audio_popup.prototype.empty_controls = function() {
		const $variable = $("#" + this.container_id + "_variable");
		$variable.val("").focus();
		$variable.change();
		$("#" + this.container_id + "_tts").prop("checked", true);
	};

	variable_audio_popup.prototype.set_events = function() {
		const self = this;
		variable_audio_popup.uber.set_events.call(this);

		const $variable_type_system = $("#" + this.container_id + "_variable_type_system");
		$variable_type_system.off("click")
			.on("click", function() {
				let option = $(this).val();
				self.toggle_variable_type(option);
			});

		const $variable_type_custom = $("#" + this.container_id + "_variable_type_custom");
		$variable_type_custom.off("click")
			.on("click", function() {
				let option = $(this).val();
				self.toggle_variable_type(option);
			});
		const $variable = $("#" + this.container_id + "_variable");
		$variable.off("change")
			.on("change", function() {
				const option = $(this).val();
				const $variable_description_container = $("#" + self.container_id + "_variable_description_container");
				const $variable_description = $("#" + self.container_id + "_variable_description");
				$variable_description_container.hide();
				$variable_description.html("");

				if (typeof self.owner.make_description_variables === "function" ) {
					const description = self.owner.make_description_variables((option || "").toUpperCase());
					if (description) {
						$variable_description.append($(description));
						$variable_description_container.show();
					}
				}
			});
	};

	variable_audio_popup.prototype.toggle_variable_type = function(option) {
		const $variable_system_container = $("#" + this.container_id + "_variable_system_container");
		const $custom_variable_container = $("#" + this.container_id + "_variable_custom_container");

		if ("SYSTEM" === option) {
			$variable_system_container.show();
			$custom_variable_container.hide();
		}
		else {
			$variable_system_container.hide();
			$custom_variable_container.show();
		}
	};

	/*

	*/
	function tts_audio_popup (params) {
		this.builtInMessagesList = inConcert.Utils.cloneArray(params.builtInMessagesList);
		tts_audio_popup.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(tts_audio_popup, base_audio_popup);

	tts_audio_popup.prototype.get_data = function() {
		let item = tts_audio_popup.uber.get_data.call(this);

		item.Name = item.Id;
		item.MessageType = "tts";
		item.MessageSubType = $("#" + this.container_id + "_msg_type").val();
		item.message = {};
		item.RawData = {};

		if (item.MessageSubType === "CUSTOM") {
			item.message = $("#" + this.container_id + "_custom_msg").val();
		}
		return item;
	};

	tts_audio_popup.prototype.get_html_middle = function() {
		let html = "" +
			"<div class='card'>" +
				"<div class='card-block'>" +
					"<div class='row'>" +
						"<div class='col-6'>" +
							"<div class='form-group'>" +
								"<label for='" + this.container_id + "_msg_type'>" + "<%=_T('Message Type')%>" + "<%=_T(':')%>" + "</label>" +
								"<select id='" + this.container_id + "_msg_type' class='form-control formComboInput'></select>" +
							"</div>" +
						"</div>" +
					"</div>" +
					"<div class='row' id='" + this.container_id + "_predefined_msg_type_container' style='display:none;'>" +
						"<div class='col-12'>" +
							"<div class='form-group d-flex flex-column'>" +
								"<label>" + "<%=_T('Visualization')%>" + "<%=_T(':')%>" + "</label>" +
								"<ul id='" + this.container_id + "_predefined_msg_list' class='viewList viewListAsGrid mt-1'></ul>" +
							"</div>" +
						"</div>" +
					"</div>" +
					"<div id='" + this.container_id + "_custom_msg_type_container' class='row autoRow messagesTextEditorRow' style='display:none;'>" +
						"<div class='col-12'>" +
							"<div class='form-group'>" +
								"<label class='required'>" + "<%=_T('Custom message')%>" + "<%=_T(':')%>" + "</label>" +
								"<textarea id='" + this.container_id + "_custom_msg' class='form-control rowsBased' rows='10' ></textarea>" +
								"<i id='" + this.container_id + "_custom_msgValidation' class='formFieldValidation formFieldError' hidden></i>" +
							"</div>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</div>";
		return html;
	};

	tts_audio_popup.prototype.ready = function() {
		const $msg_type = $("#" + this.container_id + "_msg_type");
		$msg_type.trigger("change");
	};

	tts_audio_popup.prototype.validate = function() {
		const $msg_type = $("#" + this.container_id + "_msg_type");
		const msg_type = $msg_type.val();

		if (msg_type === "CUSTOM") {
			const $custom_msg = $("#" + this.container_id + "_custom_msg");
			if ($custom_msg.val().trim().length === 0) {
				this.display_field_error($custom_msg);
				this.owner.show_status_message("<%=_T('Custom message is required')%>", "Error", 5000);
				return false;
			}
		}
		else if (msg_type === "PREDEFINED") {
			// sin validaciones
			return true;
		}
		else {
			throw("tts_audio_popup.prototype.validate - Unrecognized message type " + msg_type);
		}

		return true;
	};

	tts_audio_popup.prototype.load_data = function(callback = $.noop) {
		const $msg_type = $("#" + this.container_id + "_msg_type");
		let options = [{id: "CUSTOM", name: "<%=_T('Custom')%>"}];
		if ((this.builtInMessagesList || []).length > 0) {
			options.push({id: "PREDEFINED", name: "<%=_T('Predefined')%>"});
		}
		else {
			this.frm.readonly_control($msg_type, true);
		}
		this.frm.fill_combo($msg_type, false, options, "id", "name", true);
		callback(true);
	};

	tts_audio_popup.prototype.set_events = function() {
		const self = this;
		tts_audio_popup.uber.set_events.call(this);
		const $msg_type = $("#" + this.container_id + "_msg_type");
		$msg_type.off("change")
			.on("change", function() {
				self.on_changed_message_type();
			});
	};

	tts_audio_popup.prototype.on_changed_message_type = function() {
		const $custom_container = $("#" + this.container_id + "_custom_msg_type_container");
		const $predefined_container = $("#" + this.container_id + "_predefined_msg_type_container");
		const msg_type = $("#" + this.container_id + "_msg_type").val();

		if (msg_type === "CUSTOM") {
			$predefined_container.hide();
			$custom_container.show();
		}
		else if (msg_type === "PREDEFINED") {
			const $predefined_list = $("#" + this.container_id + "_predefined_msg_list");
			let predefined_message = this.get_predefined_message();
			let html = "";
			predefined_message.msg.forEach(message => {
				let src = get_file_message_type_img(message.type);
				html+= "" + FORMAT(
					"<li class='viewListItem'>" +
						"<img class='viewListIconImage' src='{0}' />" +
						"<label class='viewListItemText'>{1}</label>" +
					"</li>", src, message.id);
			});
			$predefined_list.html(html);
			$custom_container.hide();
			$predefined_container.show();
		}
		else {
			throw("tts_audio_popup.prototype.on_changed_message_type - Unrecognized message type " + msg_type);
		}
	};

	tts_audio_popup.prototype.get_predefined_message = function() {
		if (!this.builtInMessagesList) {
			return [];
		}
		let predefined_message;
		if (!this.builtInMessagesList) {
			return [];
		}
		for (let i = 0; i < this.builtInMessagesList.length; i++) {
			let builtInMessage = this.builtInMessagesList[i];
			if (builtInMessage.default) {
				predefined_message = builtInMessage;
				break;
			}
			else if (!predefined_message) {
				// me quedo con el primero por las dudas de que no hayan marcado uno como default.
				predefined_message = builtInMessage;
			}
		}
		return predefined_message || {};
	};

	tts_audio_popup.prototype.on_save = function(item, create_and_continue, callback = $.noop) {
		if (item.MessageSubType === "CUSTOM") {
			tts_audio_popup.uber.on_save.call(this, item, create_and_continue, callback);
			return;
		}

		let predefined_message = this.get_predefined_message();
		for (let i = 0; i < (predefined_message.msg || []).length; i++) {
			let message = predefined_message.msg[i];
			let uuid = inConcert.Utils.generateGuid();
			let row_info = {
				Id         : uuid,
				Name       : uuid,
				MessageType: message.type,
				message    : "",
				RawData    : {}
			};
			if (message.type === "tts") {
				row_info.message = message.id;
			}
			else {
				// variable
				row_info.VariableId = message.id;
				row_info.Engine = "TTS";
			}
			this.owner.add_row_to_grid(this.popup_id, row_info);
		}

		callback(true);
	};

	tts_audio_popup.prototype.edit = function(item) {
		tts_audio_popup.uber.edit.call(this, item);
		const $msg_type = $("#" + this.container_id + "_msg_type");
		this.frm.readonly_control($msg_type, true);
		const $msg = $("#" + this.container_id + "_custom_msg");
		$msg.val(item.message);
	};

	/*

	*/
	function multi_messages (owner, params)
	{
		const self = this;

		let grid_id = params.grid_id;
		this.owner = owner;
		this.owner.m_grdAudioMessages = this.owner.m_grdAudioMessages || {};
		this.m_customVariables = params.customVariables || [];
		this.m_isAllowedCustomVariables = params.isAllowedCustomVariables || false;
		this.m_ownerName = params.ownerName;

		this.m_sharedFileVisibility = {
			AgentVisible          : params.visibility.AgentVisible || false,
			AccountMessagesVisible: params.visibility.AccountMessagesVisible || false,
			FlowDesignerVisible   : params.visibility.FlowDesignerVisible || false
		};

		let callbacks = params.callbacks || {};
		this.params = {
			callbacks: {
				show_status_message: function(message, msg_type, timemout_ms) {
					if (callbacks.show_status_message) {
						callbacks.show_status_message(message, msg_type, timemout_ms);
						return;
					}
					self.owner.frm.show_status_message(message, msg_type, timemout_ms);
				}
			}
		};

		if (params.isTtsNotAllowed) {
			m_vcc_tts_config = null;
		}
		else {
			m_vcc_tts_config = inConcert.app.get_vcc_tts_config();
		}

		if (params.can_include_account_messages_variable) {
			this.can_include_account_messages_variable = params.can_include_account_messages_variable;
		}

		if (params.make_description_variables) {
			this.make_description_variables = params.make_description_variables;
		}

		const get_name = function(item) {
			if (item.MessageType === "variable") {
				let brackets;
				if (item.VariableType === "custom") {
					brackets = {open: "{{", close: "}}"};
				}
				else {
					brackets = {open: "{", close: "}"};
				}
				// nombre de la variable
				let variableName;
				let array_parsed_value = inConcert.Utils.getBracketsValue(item.VariableId, brackets);
				if ((array_parsed_value || []).length === 1) {
					variableName = array_parsed_value[0];
				}
				// engine de la variable
				let engineName;
				if (m_variablesEngineTypes[item.Engine]) {
					engineName = m_variablesEngineTypes[item.Engine].description;
				}
				// descripcion de la variable
				let description;
				if (item.VariableType === "system") {
					let variableDef = find_variable(item.VariableId);
					description = variableDef ? (variableDef.name + " |") : "";
				}
				// replace de los valores a visualizar
				let templateVariable = "{0} {1} | {2}";
				return FORMAT(templateVariable, (description || ""), (variableName || ""), (engineName || item.Engine));
			}
			else if (item.MessageType === "tts") {
				if (!item.MessageSubType || item.MessageSubType === "CUSTOM") {
					return item.message;
				}
				return "Unrecognized -> PREDEFINED";
			}
			return item.Name || "Unrecognized";
		};

		// is action enabled
		const is_action_enabled = function(action, row) {
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if (action.toLowerCase() === "play") {
				if ((row.MessageType || "").toLowerCase() !== "audio") {
					return -1;
				}
			}
			return 1;
		};

		this.owner.m_grdAudioMessages[grid_id] = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner.frm,
			row_id: "UUID",
			display_name: function(item) {
				return get_name(item);
			},
			sortable: true,
			rows_per_page: 20000,
			show_at_least: 19999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			auto_initialize: true,
			header_hidden: true,
			header_lables_inside_fields: false,
			markup_with_divs: false, // Con este flag la grilla se crea con DIV+UL+LI,etc en lugar de ser creada con TABLE+THEAD+TBODY,etc
			grid_actions: {
				without_label: true
			},
			ids: {
				grid_container: grid_id + "grid_container",
				paginator: grid_id + "paginator",
				grid: grid_id + "list",
				not_results_found: grid_id + "not_results_found",
				//popup0 (afdssssssssssssssssssssssssssssssssssssssss)
				container_new_btn: grid_id + "_neww_container_btns",
				new_item: grid_id + "_neww",
			},
			strings: {
				not_results_found: "<%=_T('There is no configuration associated with the message.')%>",
				title_new        : "<%=_T('Files Configuration')%>",
				button_create    : "<%=_T('Confirm')%>",
				can_remove       : "<%=_T('Do you want to delete: {0} ?')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "Image" , name: "", type: "image", cell_class_name: "messageTypeImg",
					get_value: function(item) {
						const get_default_src = function() {
							let src = get_file_message_type_img(item.MessageType);
							return src;
						};
						return {
							value: "",
							src  : get_default_src()
						};
					}
				},
				{id: "Name", name: "<%=_T('Name')%>", cell_class_name: "col", get_value: function(item) {
						let name = get_name(item);
						return {
							value: name, original_value: name
						};
					}
				},
				{
					id: "_action",
					name: "",
					cell_class_name: "col-2",
					custom_action: {
						play: {
							name: "<%=_T('Play Audio')%>",
							execute: function($row, $owner) {
								let row_data = $row.data("row_data");
								let file_info = {
									id     : row_data.RawData.RepositoryId,
									section: row_data.RawData.Section || "-1",
									format : row_data.RawData.Format || "wav"
								};
								$owner.data("file_info", file_info);
								self.owner.frm.dashboard.player_start($owner);
							},
							class_name: "buttons-icons-start"
						},
						edit: {
							name: "<%=_T('Edit')%>",
							execute: function($row, $owner) {
								self.owner.m_grdAudioMessages[grid_id].$set_edited_row($row);
								let item = $row.data("row_data");
								self.show_edit_audio_popup($owner, grid_id, item);
							},
							class_name: "buttons-icons-edit"
						}
					},
					system_action: {
						erase: true
					},
					is_enabled: is_action_enabled
				}
			],
			callbacks: {}
		});
	}

	if (typeof inConcert.adm_form_multi_messages !== "function") {
		inConcert.adm_form_multi_messages = multi_messages;
	}

	multi_messages.prototype.show_file_contextual_menu = function($parent)
	{
		const self = this;

		let message_id = $parent.attr("control_messageid");
		if (!message_id) {
			throw("account_config.prototype.show_file_contextual_menu - Invalid button attribute 'control_messageid'");
		}

		// menu contextual config.
		const menu_options = [];

		if (this.is_enabled_shared_files(message_id)) {
			// File config
			menu_options.push({
				id     : "file",
				name   : "<%=_T('File')%>",
				execute: function() {
					self.show_popup_shared_file($parent);
				}
			});
		}

		if (this.is_enabled_variables(message_id)) {
				// Variable config
			menu_options.push({
				id     : "variable",
				name   : "<%=_T('Variable')%>",
				execute: function() {
					let grid_id = $parent.attr("control_messageid"); // "CALL_waiting_for_agent", "CALL_waiting_for_agent_with_ewt", etc
					if (!grid_id) {
						throw("account_config.show_popup_variable_file - Unexistent 'control_messageid' !!");
					}
					self.show_popup_variable_file(grid_id);
				}
			});
		}
		if (this.is_enabled_tts()) {
			let is_supported = this.variable_supports_tts(message_id);
			if (is_supported) {
				// Message config
				menu_options.push({
					id     : "message",
					name   : "<%=_T('Message')%>",
					execute: function() {
						let grid_id = $parent.attr("control_messageid"); // "CALL_waiting_for_agent", "CALL_waiting_for_agent_with_ewt", etc
						if (!grid_id) {
							throw("account_config.show_popup_variable_file - Unexistent 'control_messageid' !!");
						}
						self.show_popup_tts_file(grid_id);
					}
				});
			}
		}

		this.owner.frm.toggle_contextual_menu($parent, menu_options);
	};

	multi_messages.prototype.show_popup_variable_file = function(grid_id, item)
	{
		if (this.m_variable_audio_popup) {
			this.m_variable_audio_popup = null;
		}
		let instance_info = {
			container_id: "adm_form_campaign_accounts_messages_variable_popup",
			owner       : this,
			popup_id    : grid_id,
			container_class : "formPopUp popUp520w",
			container_popup_middle_class : "pageFormContainer",
			title       : "<%=_T('Add Variable')%>",
			frm         : this.owner.frm,
			buttons_actions: {}
		};

		if (item) {
			instance_info.title = "<%=_T('Update Variable')%>";
			instance_info.buttons_actions.create = {caption: "<%=_T('Update')%>"};
		}

		this.m_variable_audio_popup = new variable_audio_popup(instance_info);

		if (item) {
			this.m_variable_audio_popup.edit(item);
		}
	};

	multi_messages.prototype.show_popup_shared_file = function($parent, grid_id, item)
	{
		const self = this;

		let control_id = grid_id || $parent.attr("control_messageid"); // "CALL_waiting_for_agent", "CALL_waiting_for_agent_with_ewt", etc
		if (!control_id) {
			throw("show_popup_shared_file - Unexistent 'control_messageid' !!");
		}

		let is_new = item === undefined;
		// Si el popup se abre para inlcuir un shared files mas.
		if (is_new) {
			// busco la definicion del mensaje
			let message_def = this.find_owner_message(control_id);
			if (message_def) {
				// valido que no se haya llegado a la maxima cantidad de shared files incluidos.
				if (typeof(message_def.maxAudioFiles) === "number") {
					let totalFiles = this.owner.m_grdAudioMessages[control_id].count_rows();
					if (message_def.maxAudioFiles !== -1 && totalFiles >= message_def.maxAudioFiles) {
						let message = message_def.maxAudioFiles === 1 ? "<%=_T('The message does not support more than {0} file.')%>" : "<%=_T('The message does not support more than {0} files.')%>";
						this.show_status_message(FORMAT(message, message_def.maxAudioFiles), "Info", 5000);
						return;
					}
				}
			}
		}

		this.m_sharedFilesPopupList = null;
		this.m_sharedFilesPopupList = new inConcert.SharedFilesPopupList({
			from_app            : "administration",
			owner               : this,
			popupId             : "adm_form_campaign_accounts_messages_shared_file_popup_container",
			search_files_type_id: "asterisk_audio",
			abm_files_type_id   : "asterisk_audio",
			visibility          : this.m_sharedFileVisibility,
			custom_class        : "internalPopUpContainer formPopUp flowDesignerPopUpContainer flowDesignerSharedFilesSelectorPopup",
			callbacks: {
				on_show: function() {},
				on_close: function(selected_file) {
					if (!selected_file) {
						// Si cerro el popup por cancelar
						self.cleanup_edited_row(control_id);
						return;
					}
					// Si cerro el popup por guardar
					let row_info = {
						Id         : selected_file.Id,
						Name       : selected_file.Name,
						MessageType: "audio",
						RawData    : selected_file
					};
					if (!is_new) {
						row_info.UUID = item.UUID;
					}
					self.add_row_to_grid(control_id, row_info);
				}
			}
		});

		this.m_sharedFilesPopupList.show({
			parentNode: $parent,
			option_id : "audio",
			can_create_new_file: true
		});
		if (item) {
			let file = inConcert.Utils.cloneObject(item);
			$.extend(true, file, item.RawData || {});
			delete file.RawData;
			this.m_sharedFilesPopupList.edit_file(file);
		}
	};

	multi_messages.prototype.add_row_to_grid = function(grid_id, item)
	{
		const self = this;
		item.UUID = item.UUID || inConcert.Utils.generateGuid();
		this.owner.m_grdAudioMessages[grid_id].add_or_update_row(item, null, null, function() {
			self.cleanup_edited_row(grid_id);
		});
	};

	multi_messages.prototype.cleanup_edited_row = function(grid_id)
	{
		this.owner.m_grdAudioMessages[grid_id].cleanup_edited_row();
	};

	multi_messages.prototype.show_popup_tts_file = function(grid_id, item)
	{
		if (this.m_tts_audio_popup) {
			this.m_tts_audio_popup = null;
		}
		let message_def = this.find_owner_message(grid_id);
		if (!message_def) {
			return;
		}
		let instance_info = {
			container_id                : "adm_form_campaign_accounts_messages_tts_popup",
			owner                       : this,
			popup_id                    : grid_id,
			title                       : "<%=_T('Add Message')%>",
			frm                         : this.owner.frm,
			container_class             : "formPopUp popUp720w",
			container_popup_middle_class: "pageFormContainer",
			builtInMessagesList         : message_def.builtInMessagesList,
			buttons_actions             : {
				create_and_continue: {allowed: false}
			}
		};
		if (item) {
			instance_info.title = "<%=_T('Update Message')%>";
			instance_info.buttons_actions.create = {caption: "<%=_T('Update')%>"};
		}
		this.m_tts_audio_popup = new tts_audio_popup(instance_info);
		if (item) {
			this.m_tts_audio_popup.edit(item);
		}
	};

	multi_messages.prototype.show_edit_audio_popup = function($owner, grid_id, item)
	{
		if (item.MessageType === "variable") {
			this.show_popup_variable_file(grid_id, item);
		}
		else if (item.MessageType === "tts") {
			this.show_popup_tts_file(grid_id, item);
		}
		else if (item.MessageType === "audio") {
			this.show_popup_shared_file($owner, grid_id, item);
		}
		else {
			throw("multi_messages.prototype.show_edit_audio_popup - unrecognized " + item.MessageType);
		}
	};

	multi_messages.prototype.is_enabled_shared_files = function(/*message_id*/) {
		return true;
	};

	multi_messages.prototype.is_enabled_variables = function(message_id) {
		let message_def = this.find_owner_message(message_id);
		if (!message_def) {
			return false;
		}
		// Algunos mensajes NO permiten variables (Ej: Musica en espera)
		if (message_def.allowsVariables || message_def.allowsVariables === undefined) {
			return true;
		}
		return false;
	};

	multi_messages.prototype.is_enabled_tts = function(/*message_id*/) {
		if (m_vcc_tts_config && m_vcc_tts_config.has_configrued) {
			return true;
		}
		return false;
	};

	multi_messages.prototype.is_enabled_custom_variables = function(message_id) {
		if (!this.is_enabled_variables(message_id)) {
			return false;
		}
		if (this.m_isAllowedCustomVariables) {
		 	return true;
		}
		return false;
	};

	multi_messages.prototype.fill_grid_rows = function(grid_id, items, callback = $.noop) {
		this.owner.m_grdAudioMessages[grid_id].fill_rows(callback, items);
	};

	multi_messages.prototype.get_items = function(grid_id) {
		let rows = this.owner.m_grdAudioMessages[grid_id].get_items();
		return rows;
	};

	multi_messages.prototype.get_owner_name = function() {
		return this.m_ownerName;
	};

	multi_messages.prototype.find_owner_message = function(message_id) {
		if (!this.owner.find_message) {
			return;
		}
		let message_def = this.owner.find_message(message_id);
		if (!message_def) {
			return;
		}
		return message_def;
	};

	multi_messages.prototype.variable_supports_tts  = function(message_id) {
		if (!this.is_enabled_variables(message_id)) {
			return false;
		}
		if (!this.is_enabled_tts(message_id)) {
			return false;
		}
		let message_def = this.find_owner_message(message_id);
		if (!message_def) {
			return false;
		}
		// Si tiene messages built in, quiere decir que permite TTS
		if (message_def.builtInMessagesList) {
			return true;
		}
		return false;
	};

	multi_messages.prototype.show_status_message = function(message, msg_type, timemout_ms) {
		this.params.callbacks.show_status_message(message, msg_type, timemout_ms);
	};
}());
