/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	var get_chk_bool_to_string  = function( chk_id ) {
		var $chk = $( chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

	//---------------------------------------------------------------------
	// qna_maker management
	function qna_maker_management(params)
	{
		params = params || {};
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.section = this.parent.parent;

		let prefix_id = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_qna_maker_form, etc

		this.current_qna_maker_mng = null;

		// container
		this.container_id = "#" + params.container_id;
		// enable/disable
		this.qna_maker_mng = "#" + prefix_id + "_qna_maker_mng";

		return true;
	}

	if (typeof inConcert.Administration.Campaigns.QnAMakerManagement !== "function") {
		inConcert.Administration.Campaigns.QnAMakerManagement = qna_maker_management;
	}

	qna_maker_management.prototype.has_configured_qna_maker_in_vcc = function()
	{
		if (!inConcert.app.has_configured_qna_maker_in_vcc()) {
			return false;
		}
		return true;
	};

	qna_maker_management.prototype.initialize = function()
	{
		this.current_qna_maker_mng = null;
		this.make_html();
		this.set_events();
		this.set_default_values();
	};

	qna_maker_management.prototype.validate = function()
	{
		const result = {success: true};
		return result;
	};

	qna_maker_management.prototype.get = function()
	{
		return {
			Enabled: get_chk_bool_to_string(this.qna_maker_mng)
		};
	};

	qna_maker_management.prototype.attach_data = function( acc )
	{
		if (!acc) {
			return;
		}
		acc.QnAMaker = this.get(true /* keep_old_data */);
		return acc;
	};

	qna_maker_management.prototype.update_current_qna_maker_mng = function()
	{
		this.current_qna_maker_mng = this.get();
	};

	qna_maker_management.prototype.set_ctrls_values = function(qna_maker_mng)
	{
		if (!qna_maker_mng) {
			return;
		}
	};

	qna_maker_management.prototype.has_configured_qna_maker_mng = function( acc )
	{
		if ( !acc ) {
			return false;
		}
		if (acc.QnAMaker && acc.QnAMaker.Enabled === "1") {
			return true;
		}
		return false;
	};

	qna_maker_management.prototype.set = function( acc )
	{
		if ( !acc ) {
			return;
		}
		this.current_qna_maker_mng = acc.QnAMaker;
		const $qna_maker_mng = $(this.qna_maker_mng);

		if (!this.has_configured_qna_maker_mng(acc)) {
			$qna_maker_mng.prop("checked", false);
			this.disable_qna_maker_mng(true);
			return false;
		}

		$qna_maker_mng.prop("checked", true);
		this.disable_qna_maker_mng(false);
		this.set_ctrls_values(this.current_qna_maker_mng);
	};

	qna_maker_management.prototype.disable_qna_maker_mng = function(/*disabled*/) 
	{
	};

	qna_maker_management.prototype.set_default_values = function()
	{
		$( this.qna_maker_mng ).prop( "checked", true );
		$( this.qna_maker_mng ).trigger( "click" );
	};

	qna_maker_management.prototype.editable = function()
	{
		const self = this;
		var ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.disable_control( $ctrl, ds, required );
		};
		const $qna_maker_mng = $( this.qna_maker_mng );
		ds_ctrl( $qna_maker_mng, false );
	};

	qna_maker_management.prototype.readonly = function()
	{
		const self = this;
		const ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.readonly_control( $ctrl, ds, required );
		};
		ds_ctrl( $( this.qna_maker_mng ), true /* disabled */ );
	};

	qna_maker_management.prototype.sanitize_id = function( id )
	{
		// remueve el # de jquery
		return id.substring( 1 );
	};

	qna_maker_management.prototype.make_html = function()
	{
		const qna_maker_mng_id = this.sanitize_id(this.qna_maker_mng);
		$( this.container_id ).html( "".concat(
		// titulo
			// "<h2 class='formSubTitle'>",
			// 	"<label class='subTitleText'>", "<%=_T('QnA Maker')%>", "</label>",
			// 	"<label class='subTitleHelp'>/", "<%=_T('Allows you to make queries against the knowledge base of QnA Maker.')%>", "</label>",
			// "</h2>",

		// qna_maker management ( Enabled / Disabled )
			"<div class='form-group'>",
				"<div class='form-check'>",
					"<label for='", qna_maker_mng_id, "' class='form-check-label form-check-label-title'>",
						"<input id='", qna_maker_mng_id, "' class='form-check-input' type='checkbox' />",
						"<%=_T('Enable automatic queries based of')%>", "&nbsp;<strong><%=_T('QnA Maker')%></strong>&nbsp;", "<%=_T('knowledge')%>",
					"</label>",
				"</div>",
			"</div>",

		// disclaimer
			// "<div style='display:none' class='tabDisclaimer formFieldsetMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>"
		));
	};

	qna_maker_management.prototype.set_events = function()
	{
		const self = this;

		if ( this.section.is_readonly() ) {
			return;
		}

		$(this.qna_maker_mng)
			.off("click")
			.on("click", function() {
				const disabled = !$(this).prop("checked");
				if (!disabled) {
					if (self.current_qna_maker_mng) {
						self.set_ctrls_values(self.current_qna_maker_mng);
					}
				}
				else {
				// actualizo la configuracion del current qna_maker_mng cuando estoy editando una account existente.
					self.update_current_qna_maker_mng();
				}
				self.disable_qna_maker_mng(disabled);
			}
		);
	};

}());
