/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 11 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if (!this.inConcert.Administration) {
	this.inConcert.Administration = {};
}

if (!this.inConcert.Administration.Campaigns) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	var format = inConcert.Utils.format;

	/**
	* SurveyManagement. It is a kind of mixin that is added in each account and deals with whether we use a survey or not.
	*/
	function SurveyManagement (params)
	{
		this.parent = params.parent;
		this.section = this.parent.parent;

		this.id_prefix = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_mail_survey, etc

		this.current_survey = null;
		this.survey_list = null;

		// container
		this.container_id = "#" + params.container_id; // i.e. adm_form_campaign_accounts_mail_survey_panel
	}

	if (typeof inConcert.Administration.Campaigns.SurveyManagement !== "function") {
		inConcert.Administration.Campaigns.SurveyManagement = SurveyManagement;
	}

	SurveyManagement.prototype.initialize = function()
	{
		this.current_survey = null;
		this.__make_html();
		this.__load_data();
		this.__set_events();
		this.__set_default_values();
	};

	SurveyManagement.prototype.editable = function()
	{
		var $use_survey = $("#" + this.id_prefix + "_use_survey" );
		var $survey = $("#" + this.id_prefix + "_survey" );
		var $expire_survey = $("#" + this.id_prefix + "_expire_survey" );

		this.section.disable_control( $use_survey, false );
		var checked = $use_survey.prop( "checked" );
		var disabled = !checked;

		this.section.disable_control( $survey, disabled );
		this.section.disable_control( $expire_survey, disabled );
	};

	SurveyManagement.prototype.readonly = function()
	{
		var $use_survey = $("#" + this.id_prefix + "_use_survey" );
		var $survey = $("#" + this.id_prefix + "_survey" );
		var $expire_survey = $("#" + this.id_prefix + "_expire_survey" );

		this.section.disable_control( $use_survey, true );
		this.section.disable_control( $survey, true );
		this.section.disable_control( $expire_survey, true );
	};

	/**
	* Builds the markup that is common to each account configuration.
	* Private.
	*/
	SurveyManagement.prototype.__make_html = function ()
	{
		var template =
			// "<h2 class='formSubTitle'>" +
			// 	"<label class='subTitleText'>" + "<%=_T('Survey Management')%>" + "</label>" +
			// 	"<label class='subTitleHelp'>/ " + "<%=_T('You must add a Survey, on WebDesigners administration page, to enable this section.')%>" + "</label>" +
			// "</h2>" +
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<div class='form-group mb-3 pb-1'>" +
						"<div class='form-check'>" +
							"<label for='{{ prefix }}_use_survey' class='form-check-label form-check-label-title'>" +
								"<input id='{{ prefix }}_use_survey' class='form-check-input' type='checkbox' />" +
								"<strong>" + "<%=_T('Add a survey from the Web Designer')%>" + "</strong>" + 
							"</label>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</div>" +
			"<div class='row'>" +
				"<div class='col-6'>" +
					"<div class='form-group'>" +
						"<label for='{{ prefix }}_survey' class='required'>" + "<%=_T('Survey')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='{{ prefix }}_survey' class='form-control'></select>" +
						"<i id='{{ prefix }}_survey_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='{{ prefix }}_expire_survey' class='required'>" + "<%=_T('Expire time (days)')%>" + "<%=_T(':')%> </label>" +
						"<input id='{{ prefix }}_expire_survey' class='form-control formFieldHour' type='text'>" +
						"<i id='{{ prefix }}_expire_survey_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
			"</div>";

		var markup = Mustache.render(template, {
			prefix: this.id_prefix
		});
		$(this.container_id).html(markup);
	};

	/**
	* Request the survey list and fills the combo.
	* Private.
	*/
	SurveyManagement.prototype.__load_data = function ()
	{
		var self = this;
		var tweety = inConcert.app.getTweetyAPI();
		
		tweety.load_survey_projects(function(status, survey_list){
			if(status) {
				self.__fill_data(survey_list);
			}
			else {
				var timeout_ms = false; //infinite
				self.section.frm.show_status_message("<%=_T('Error retrieving surveys')%>", "Error", timeout_ms);
			}
		});
	};

	/**
	* Fills the combo with an array of surveys.
	* Private.
	*/
	SurveyManagement.prototype.__fill_data = function (survey_list)
	{
		this.survey_list = survey_list;
		var combo = $("#" + this.id_prefix + "_survey" );
		if(survey_list.length > 0) {
			this.section.frm.fill_combo(combo, true, survey_list, "id", "description");
			this.__select_survey_in_combo();
		}
		else {
			this.section.disable_control(combo, true, true);
			var expireTime = $("#" + this.id_prefix + "_expire_survey" );
			this.section.disable_control( expireTime, true );
			var checkbox = $("#" + this.id_prefix + "_use_survey" );
			this.section.disable_control(checkbox, true, false);
		}
	};

	/**
	* Hooks events in our markup.
	* Private.
	*/
	SurveyManagement.prototype.__set_events = function()
	{
		var self = this;
		var $use_survey = $( "#" + this.id_prefix + "_use_survey" );
		$use_survey
			.unbind("click")
			.bind("click", function() {
				var status =  $use_survey.prop("checked");
				var $combo = $( "#" + self.id_prefix + "_survey" );
				self.section.disable_control( $combo, !status, true );
				var $expireTime = $("#" + self.id_prefix + "_expire_survey" );
				self.section.disable_control( $expireTime, !status, true );
				if ( !status ) {
					$combo.val( "" );
				}
			});
	};

	/**
	* Sets initial values in the form.
	* Private.
	*/
	SurveyManagement.prototype.__set_default_values = function ()
	{
		var checkbox_use_survey = $("#" + this.id_prefix + "_use_survey" );
		var combo = $("#" + this.id_prefix + "_survey");
		var expireTime = $("#" + this.id_prefix + "_expire_survey");
		if(!this.current_survey) {
			checkbox_use_survey.prop("checked", false);
			this.section.disable_control(combo, true, false);
			this.section.disable_control(expireTime, true, false);
		}
		else {
			this.section.disable_control(combo, false, true);
			this.section.disable_control(expireTime, false, true);
		}
	};

	/**
	* Sets the account instance we're working with.
	*/
	SurveyManagement.prototype.set_account = function (account)
	{
		if(!account) { return; }
		if(account.Survey && account.Survey.id) {
			this.current_survey = account.Survey;
		}
		else {
			this.current_survey = null;
		}
		this.__select_survey_in_combo();
		this.__set_survey_expire_time();
		this.__set_default_values();
	};

	/**
	* Retrieves the survey that was selected in the combo.
	* Private.
	*/
	SurveyManagement.prototype.__get_chosen_survey = function ()
	{
		var checkbox = $("#" + this.id_prefix + "_use_survey" );
		var enabled = checkbox.prop("checked");
		if(!enabled) {
			return;
		}
		var combo = $("#" + this.id_prefix + "_survey");
		return combo.val();
	};

	/**
	* Performs validations. This method is called before saving.
	*/
	SurveyManagement.prototype.validate = function ()
	{
		var survey = this.__get_chosen_survey();
		// survey disabled?
		if (typeof(survey) === "undefined") {
			return { success: true, result: {} };
		}
		// survey enabled, but no survey chosen?
		if(survey.length === 0) {
			const $err_ctrl = $("#" + this.id_prefix + "_survey" );
			return { success: false, error_message: "<%=_T('A survey is required.')%>", $ctrl: $err_ctrl };
		}
		// Expire time - validation
		var $expireTime = $("#" + this.id_prefix + "_expire_survey");
		if (!this.parent.validate_field( $expireTime, 'required', "<%=_T('Expire survey time is required.')%>", range )) {
			const $err_ctrl = $("#" + this.id_prefix + "_expire_survey" );
			return { success: false, error_message: "<%=_T('Expire survey time is required.')%>", $ctrl: $err_ctrl };
		}
		var range = { between: { from: 1, to: 30 } };
		if (!this.parent.validate_field( $expireTime, 'range', format("<%=_T('Please enter expire survey time value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
			const $err_ctrl = $("#" + this.id_prefix + "_expire_survey" );
			return { success: false, error_message: format("<%=_T('Please enter expire survey time value between {0} and {1}.')%>", range.between.from, range.between.to ), $ctrl: $err_ctrl };
		}
		var expireTimeValue =  $expireTime.val() * 24 * 60 * 60;
		// all is fine and dandy
		var token = this.current_survey ? this.current_survey.Token : inConcert.Utils.generateGuid();
		// get the url but strip the protocol
		var webdesigner_url = inConcert.app.getTweetyRuntimeUrlWithSubDomain().match("^http[s]?://(.+)")[1];
		var new_url = format("https://{0}/{1}", webdesigner_url, survey);

		return {
			success: true,
			result: {
				id: survey,
				Token: token,
				url: new_url,
				expirationTime: expireTimeValue
			}
		};
	};

	/**
	* After we have an account to work with and we have retrieved the list of possible surveys,
	* update the combo selection.
	*/
	SurveyManagement.prototype.__select_survey_in_combo = function()
	{
		if(this.current_survey) {
			$("#" + this.id_prefix + "_use_survey" ).prop("checked", true);
			var combo = $("#" + this.id_prefix + "_survey");
			this.section.disable_control(combo, false, true);
			combo.find("option[value='" + this.current_survey.id + "']").attr("selected", "selected");
		}
	};

	SurveyManagement.prototype.__set_survey_expire_time = function()
	{
		if(this.current_survey && this.current_survey.expirationTime && typeof(this.current_survey.expirationTime) === "number") {
			$("#" + this.id_prefix + "_use_survey" ).prop("checked", true);
			var $expire_time = $("#" + this.id_prefix + "_expire_survey");
			this.section.disable_control($expire_time, false, true);
			var expireTimeValue = this.current_survey.expirationTime / 60 / 60 / 24;
			$expire_time.val(expireTimeValue);
		}
	};

}());
