/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 11 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if (!this.inConcert.Administration) {
	this.inConcert.Administration = {};
}

if (!this.inConcert.Administration.Campaigns) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	/**
	TicketEmitterManagement. It is a kind of mixin that is added in each account. It deals with using ticket emitters or not.
	*/
	function TicketEmitterManagement( params )
	{
		this.parent = params.parent;
		this.section = this.parent.parent;

		this.id_prefix = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_contact_form, etc

		this.current_emitter = null;

		// container
		this.container_id = "#" + params.container_id;
	}

	if (typeof inConcert.Administration.Campaigns.TicketEmitterManagement !== "function") {
		inConcert.Administration.Campaigns.TicketEmitterManagement = TicketEmitterManagement;
	}

	TicketEmitterManagement.prototype.initialize = function()
	{
		this.current_emitter = null;
		this.__make_html();
		this.__load_data();
		this.__set_events();
		this.__set_default_values();
	};

	TicketEmitterManagement.prototype.editable = function()
	{
		var $use_tickets = $("#" + this.id_prefix + "_use_tickets" );
		var $automatic_emit = $("#" + this.id_prefix + "_automatic_emit" );
		var $ticket_emitter = $("#" + this.id_prefix + "_ticket_emitter" );

		this.section.disable_control( $use_tickets, false );
		var checked = $use_tickets.prop( "checked" );
		var disabled = !checked;

		this.section.disable_control( $automatic_emit, disabled );
		this.section.disable_control( $ticket_emitter, disabled );
	};

	TicketEmitterManagement.prototype.readonly = function()
	{
		var $use_tickets = $("#" + this.id_prefix + "_use_tickets" );
		var $automatic_emit = $("#" + this.id_prefix + "_automatic_emit" );
		var $ticket_emitter = $("#" + this.id_prefix + "_ticket_emitter" );

		this.section.disable_control( $use_tickets, true );
		this.section.disable_control( $automatic_emit, true );
		this.section.disable_control( $ticket_emitter, true );
	};

	/**
	Builds the markup that is common to each account configuration.
	Private.
	*/
	TicketEmitterManagement.prototype.__make_html = function ()
	{
		var template =
			// "<h2 class='formSubTitle'>" +
			// 	"<label class='subTitleText'>" + "<%=_T('Tickets Management')%>" + "</label>" +
			// 	"<label class='subTitleHelp'>/ " + "<%=_T('You must add a Ticket Emitter, on tickets administration page, to enable this section.')%>" + "</label>" +
			// "</h2>" +
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<div class='form-group mb-3 pb-1'>" +
						"<div class='form-check'>" +
							"<label for='{{ prefix }}_use_tickets' class='form-check-label form-check-label-title'>" +
								"<input id='{{ prefix }}_use_tickets' class='form-check-input' type='checkbox' />" +
								"<strong>" + "<%=_T('Enable Ticket Assignment')%>" + "</strong>" + 
							"</label>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</div>" +
			"<div class='row'>" +
				"<div class='col-6'>" +
					"<div class='form-group'>" +
						"<label for='{{ prefix }}_ticket_emitter' class='required'>" + "<%=_T('Ticket Emitter')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='{{ prefix }}_ticket_emitter' class='form-control'></select>" +
						"<i id='{{ prefix }}_ticket_emitter_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +

				"<div class='col-6'>" +
					"<div class='form-group'>" +
						"<div class='form-check'>" +
							"<label for='{{ prefix }}_automatic_emit' class='form-check-label form-check-label-title'>" +
								"<input id='{{ prefix }}_automatic_emit' class='form-check-input' type='checkbox' />" +
								"<%=_T('Assign automatically')%>" +
							"</label>" +
						"</div>" +
					"</div>" +
				"</div>" +
			"</div>" +
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<div class='inlineMessagesContainer messageInfo'>" +
						"<%=_T('To add an Ticket Emitter, please go to the Tickets Administration Page.')%>" +
					"</div>" +
				"</div>" +
			"</div>";

		var markup = Mustache.render(template, {
			prefix: this.id_prefix
		});
		$(this.container_id).html(markup);
	};

	/**
	Request the ticket emitters list and fills the combo.
	Private.
	*/
	TicketEmitterManagement.prototype.__load_data = function ()
	{
		var self = this;

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/tickets/list",
			type: "POST",
			data: {},
			contentType: "application/json",
			success: function(emitter_list /*, textStatus */)
			{
				self.__fill_data(emitter_list);
			},
			error: function( /*xhr, textStatus, errorThrown */)
			{
				var timeout_ms = false; //infinite
				self.section.frm.show_status_message("<%=_T('Error retrieving ticket configuration')%>", "Error", timeout_ms);
			}
		});
	};

	/**
	Fills the combo with an array of ticket emitters.
	Private.
	*/
	TicketEmitterManagement.prototype.__fill_data = function (emitter_list)
	{
		var combo = $("#" + this.id_prefix + "_ticket_emitter" );
		if(emitter_list.length > 0) {
			this.section.frm.fill_combo(combo, true, emitter_list, "id", "description");
			this.__select_emitter_in_combo();
		}
		else {
			this.section.disable_control(combo, true, true);
			var checkbox = $("#" + this.id_prefix + "_use_tickets" );
			this.section.disable_control(checkbox, true, false);

			var checkbox_automatic = $("#" + this.id_prefix + "_automatic_emit" );
			this.section.disable_control(checkbox_automatic, true, false);
		}
	};

	/**
	Hooks events in our markup.
	Private.
	*/
	TicketEmitterManagement.prototype.__set_events = function()
	{
		var self = this;
		var checkbox = $("#" + this.id_prefix + "_use_tickets" );

		checkbox
			.unbind("click")
			.bind("click", function() {
				var status = $(this).prop("checked");
				var combo = $("#" + self.id_prefix + "_ticket_emitter");
				self.section.disable_control(combo, !status, true);

				var checkbox_automatic = $("#" + self.id_prefix + "_automatic_emit" );
				self.section.disable_control(checkbox_automatic, !status, false);

				var cbo_previous_value;
				var automatic_previous_value;
				if ( !status ) {
					cbo_previous_value = combo.val();
					automatic_previous_value = checkbox_automatic.prop("checked") ? "1" : "0";
					combo.attr("previous_value", cbo_previous_value);
					checkbox_automatic.attr("previous_value", automatic_previous_value);

					combo.val( "" );
					checkbox_automatic.prop("checked", false);
				}
				else {
					cbo_previous_value = combo.attr("previous_value");
					if ( cbo_previous_value ) {
						combo.val( cbo_previous_value);
					}
					automatic_previous_value = checkbox_automatic.attr("previous_value");
					if ( automatic_previous_value ) {
						checkbox_automatic.prop("checked", automatic_previous_value === "1" ? true: false );
					}
				}
			});
	};

	/**
	Sets initial values in the form.
	Private.
	*/
	TicketEmitterManagement.prototype.__set_default_values = function ()
	{
		var checkbox_use_tickets = $("#" + this.id_prefix + "_use_tickets" );
		var checkbox_automatic = $("#" + this.id_prefix + "_automatic_emit" );
		var combo = $("#" + this.id_prefix + "_ticket_emitter");

		if(!this.current_emitter) {
			checkbox_use_tickets.prop("checked", false);
			this.section.disable_control(checkbox_automatic, true, false);
			this.section.disable_control(combo, true, false);
		}
		else {
			var status = (this.current_emitter && this.current_emitter.automatic);
			checkbox_automatic.prop("checked", status);
			this.section.disable_control(checkbox_automatic, false, false);
			this.section.disable_control(combo, false, true);
		}
	};

	/**
	Sets the account instance we're working with.
	*/
	TicketEmitterManagement.prototype.set_account = function (account)
	{
		if(!account) { return; }

		if(account.TicketEmitter && account.TicketEmitter.id) {
			this.current_emitter = account.TicketEmitter;
		}
		else {
			this.current_emitter = null;
		}

		this.__select_emitter_in_combo();
		this.__set_default_values();
	};

	/**
	Retrieves the emitter that was selected in the combo.
	Private.
	*/
	TicketEmitterManagement.prototype.__get_chosen_emitter = function ()
	{
		var checkbox = $("#" + this.id_prefix + "_use_tickets" );
		var enabled = checkbox.prop("checked");
		if(enabled) {
			var combo = $("#" + this.id_prefix + "_ticket_emitter");
			return combo.val();
		}
		return;
	};

	/**
	Performs validations. This method is called before saving.
	*/
	TicketEmitterManagement.prototype.validate = function ()
	{
		var emitter = this.__get_chosen_emitter();
		// tickets disabled?
		if (typeof(emitter) === "undefined") {
			return { success: true, result: {} };
		}
		// tickets enabled, but no emitter chosen?
		if( emitter.length === 0 ) {
			var $err_ctrl = $("#" + this.id_prefix + "_ticket_emitter");
			return { success: false, error_message: "<%=_T('A ticket emitter is required.')%>", $ctrl: $err_ctrl };
		}
		// all is fine and dandy
		var checkbox = $("#" + this.id_prefix + "_automatic_emit" );
		var automatic_status = checkbox.prop("checked");
		return {
			success: true,
			result: { id: emitter, automatic: automatic_status }
		};
	};

	/**
	After we have an account to work with and we have retrieved the list of possible emitters,
	update the combo selection.
	*/
	TicketEmitterManagement.prototype.__select_emitter_in_combo = function()
	{
		if(this.current_emitter) {
			$("#" + this.id_prefix + "_use_tickets" ).prop("checked", true);
			var combo = $("#" + this.id_prefix + "_ticket_emitter");
			this.section.disable_control(combo, false, true);
			combo.find("option[value='" + this.current_emitter.id + "']").attr("selected", "selected");
		}
	};

}());
