/* jshint smarttabs:true, quotmark:false, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	var FORMAT = inConcert.Utils.format;
	var REQUEST_TIMEOUT_MS = 1800000; // 30 minutes

	function API (params)
	{
		if(typeof(params) !== "object") {
			throw("API.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("API.Constructor - base uri is required");
		}
		this.uri = params.uri + "/";
	}

	if (typeof inConcert.ImportationAPI !== "function") {
		inConcert.ImportationAPI = API;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	API.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Get: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Delete: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data || {}),
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) { callback(true, response); },
			error       : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

/********************************************************************************************************
	IMPORT USERS
********************************************************************************************************/

	API.prototype.importGetAllowedHeaderInfo = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/get_allowed_header_info/
		 */
		callback = callback || $.noop;
		params = params || {};

		var uri = FORMAT("{0}imports/get_allowed_header_info/", this.uri);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.importGetUploadedCSVHeader = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/get_uploaded_csv_header/file_id
		 */
		callback = callback || $.noop;
		params = params || {};

		var uri = FORMAT("{0}imports/get_uploaded_csv_header/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.getAllowedSharedFilesHeader = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/get_allowed_header_shared_files/
		 */
		callback = callback || $.noop;
		params = params || {};

		var uri = FORMAT("{0}imports/get_allowed_header_shared_files/", this.uri);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.loadUploadedCSVtoJSON = function(params, callback = $.noop) {
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/load_csv_uploaded_to_json/file_id
		 */
		params = params || {};

		var uri = FORMAT("{0}imports/load_csv_uploaded_to_json/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.fileSharingLoadUploadedCSVtoJSON = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/shared_files/load_csv_uploaded_to_json/file_id
		 */
		params = params || {};

		var uri = FORMAT("{0}imports/shared_files/load_csv_uploaded_to_json/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.importStart = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/start/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}imports/start/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.importDelete = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/delete/d469528f-a521-4035-a9ea-36a346b83df8
		 */
		callback = callback || $.noop;
		params = params || {};

		var uri = FORMAT("{0}imports/delete/{1}",
			this.uri,
			encodeURIComponent(params.importationId || "")
		);

		var request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	/*
	
	*/
	API.prototype.loadImportations = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/imports/load_all/
		 */
		callback = callback || $.noop;
		var uri = FORMAT("{0}imports/load_all/", this.uri);

		var request_handler = this.__Get(uri, callback);
		return request_handler;
	};

}());
