/* jshint smarttabs:true, quotmark:double, esversion: 9 */
/* global $, inConcert, mxUtils */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	var FORMAT = inConcert.Utils.format;
	var REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function PhoneValidatorAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("PhoneValidatorAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("PhoneValidatorAPI.Constructor - base URI is required");
		}

		this.uri = params.uri + "/phone_validator";
	}

	if (typeof inConcert.PhoneValidatorAPI !== "function") {
		inConcert.PhoneValidatorAPI = PhoneValidatorAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	PhoneValidatorAPI.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("PhoneValidatorAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("PhoneValidatorAPI.prototype.__Get: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (response)    {callback(true, response);},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("PhoneValidatorAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("PhoneValidatorAPI.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("PhoneValidatorAPI.prototype.__Post: data variable must be object type.");
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("PhoneValidatorAPI.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("PhoneValidatorAPI.prototype.__Delete: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("PhoneValidatorAPI.prototype.__Delete: data variable must be object type.");
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

/********************************************************************************************************

********************************************************************************************************/

	/*
	
	*/
	PhoneValidatorAPI.prototype.getSchema = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/get/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get/", this.uri);

		var request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.saveArea = function(countryId, data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/save_area/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/save_area/{1}", this.uri, countryId);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.deleteArea = function(countryId, data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/delete_area/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete_area/{1}", this.uri, countryId);

		var request_handler = this.__Delete(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.deleteAreas = function(countryId, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/delete_areas/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete_areas/{1}", this.uri, countryId);

		var request_handler = this.__Delete(uri, callback, {});
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.saveRule = function(countryId, data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/save_rule/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/save_rule/{1}", this.uri, countryId);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.deleteRule = function(countryId, data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/delete_rule/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete_rule/{1}", this.uri, countryId);

		var request_handler = this.__Delete(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	PhoneValidatorAPI.prototype.deleteGlobalRules = function(countryId, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/administration/phone_validator/delete_global_rules/Global
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete_global_rules/{1}", this.uri, countryId);

		var request_handler = this.__Delete(uri, callback, {});
		return request_handler;
	};

	/*
	*/
	PhoneValidatorAPI.prototype.asyncCall = async function(method, ...args)
	{
		return inConcert.Utils.asyncCallObjectMethod(this, method, ...args);
	};


}());
