local LuaOrm = {   
	Date = require "LuaOrm.Date"  
} 
require "inconcert.translators" 
local lua_globaltimer = require "lua_globaltimer" 
local agentservice = require "apps.agent.agentservice"
local m_interactions = require "apps.agent.controllers.interactions"
local interactionmanip = require "interactionmanip"
require "AsyncTasks"
local utils = require "shared.apps.utils"
-- MODDEL
local agentModel = require "apps.agent.model"
local models = agentModel:GetModels()
local model_vccs = require "apps.agent.models.vccs"
local admCampaignsModel = require "apps.administration.models.campaigns"
-- PERMISSIONS
local permissions = require "shared.permissions.permissions"
local model_perm = require "shared.permissions.models.permissions"
-- LUIS
local luis = require "shared.cognitive_services.luis.luis"
local apiLuis = luis.GetAPI() -- rocka
-- QnA
local apiQnAMaker = require "qna_maker" -- rocka
-- RASA
local rasa = require "shared.rasa.rasa"
-- AUTFILTER
local authFilter = (require "authfilter")("agent")
-- TTS
local model_ttsasrengine = require "apps.administration.models.ttsasr_engines"
-- WIDGETS INTEGRATIONS
local widgetsIntegrationsAPI = require "apps.integrations.api"
--Audit
local audits = require "audits"
local adm_users = require "apps.administration.models.users"
--Helpers
local m_helpers = require "helpers"
--
local m_moduleLocation = select(2, ...)
-- Provisioning
local modelIceServers = require("apps.provisioning.models.ice_servers")
  
module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil

local TOP_INTERACTIONS = m_interactions.GetTopInteractions()
--#19060: La sincronizacion de usuarios desde ldap puede fallar si alguno de los atributos contiene datos binarios. Para evitarlo usamos cjson
local json = require "cjson"

local modelRestrictions = require "apps.agent.models.config_restrictions"
local redisSyncUsersWarningKey = "SyncUsersWarning:" 

local _T = function(t)
	return t
end
local function is_table (t)
	return type(t) == "table"
end
local function is_string (v)
	return type(v) == "string"
end
local function is_empty (t)
	return next(t) == nil
end

local function GetUserAuditData (fullUserId, keyspace, usersOverridePermissions, vcc)
	local admUserController = require "apps.administration.controllers.users"
	return admUserController.GetUserAuditData(fullUserId, keyspace, usersOverridePermissions, vcc)
end

local function GetServerInfo ()
	--Module - model System Configuration
	local admSystemModel = require "apps.administration.models.system"
	-- IR
	local ir = require "lua_iragentlink"
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local nodes = irLink:GetIRNodes()
--[[
	Código comentado y utilizado para simular Ambientes con MultiNodo.
]]--
--table.insert(nodes.nodes, {
-- 	port = "48000",
-- 	IP = "192.168.4.12" -- este es el test la supuesta ip del node actual que esta laburando.
-- })
	local isMultiNode = false
	if nodes and #(nodes.nodes) > 1 then
		isMultiNode = true
	end
	local info = {
		isMultiNode         = isMultiNode,
		nodesInfo           = nodes,
		OSDistro            = utils.GetOSDistro():lower(),
		maxAllowedRowsInES  = utils.GetElasticSearchInstance():GetMaxAllowedRowsInES(),
		systemConfiguration = admSystemModel.LoadAllSystemConfiguration()
	}
	return info
end

local function GetIRVariables (RID)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local ir = require "lua_iragentlink"
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local response = irLink:SendCommand({
		RID        = RID,
		proxyType  = "AGENT",
		agentAtVCC = utils.GetCurrentLoggedUser(),
		command    = "GetIRVariables"
	})
	if response.result ~= "OK" then
		LogWarning("[AGENT:GetIRVariables] Unable to retrieve IR Variables")
		return {}
	end
	--LogDebug("[AGENT:GetIRVariables] 1 - IR response.variables '%s'.", json.encode(response.variables))
	local variables = {}
	for _, variable in pairs(response.variables) do
		if variable.showIn and variable.showIn.agent then
			variable.originalName = variable.name
			variable.name = _T(variable.name)
			table.insert(variables, variable)
		end
	end
	--LogDebug("[AGENT:GetIRVariables] 2 - IR Filtered Variables '%s'.", json.encode(variables))
	return variables
end

local function GetLUISInfo ()
	local info = {
		--API_VERSION = luis.GetAPIVersion() -- ya no se usa
	}
	return info
end

local function GetRASAInfo ()
	local info = {
		API_VERSION = rasa.GetAPIVersion()
	}
	return info
end

local function GetWidgetsDefinitions (params)
	local widgetsDefinitions = widgetsIntegrationsAPI.GetAvailablesWidgetsDefinitions(params)
	return widgetsDefinitions
end

local function LoadVccConfig (vcc, keyspace)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local vccInfo = model_vccs.Get(vcc, keyspace)

	-- Load LUIS information
	local luisSubscriptionsKeys, luisTotalSubscriptions = apiLuis.LoadVCCSubscriptions(vcc)
	if luisTotalSubscriptions == 0 then
		--LogInfo("[AGENT:LoadVccConfig] VCC '%s' has no assigned Cognitive Services Subscriptions keys.", vcc)
	else
		--LogInfo("[AGENT:LoadVccConfig] VCC '%s' has '%d' assigned Cognitive Services Subscriptions keys.", vcc, luisTotalSubscriptions)

		local _, totalAzzureAccounts = apiLuis.LoadVCCAzureAccounts(vcc)
		local isAssignedAzureAccount = totalAzzureAccounts > 0

		--LogInfo("[AGENT:LoadVccConfig] VCC '%s' has %s assigned Cognitive Services Azzure Accounts.", vcc, isAssignedAzureAccount and "" or "NOT")

		vccInfo.MsCognitiveServices = {
			SubscriptionsKeys = luisSubscriptionsKeys,
			isAssignedAzureAccount = isAssignedAzureAccount
		}
	end

	-- Load QNA MAKER information
	local qnaMakerSubscriptionsKeys, qnaMakerTotalSubscriptions = apiQnAMaker.LoadVCCSubscriptions(vcc)
	if qnaMakerTotalSubscriptions > 0 then
		local success, answerRequestInfo = apiQnAMaker.GetAnswerPostInfo(vcc)
		if not success then
			answerRequestInfo = nil
		end
		vccInfo.QnAMaker = {
			SubscriptionsKeys = qnaMakerSubscriptionsKeys,
			AnswerRequestInfo = answerRequestInfo
		}
	end

	-- Load QNA MAKER information V2 fede
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc) -- fede
	if qnaMakerTotalSubscriptionsV2 > 0 then
		local successV2, answerRequestInfoV2 = apiQnAMaker.GetAnswerPostInfoV2(vcc)
		if not successV2 then
			answerRequestInfoV2 = nil
		end
		vccInfo.QnAMakerV2 = {
			SubscriptionsKeys = qnaMakerSubscriptionsKeysV2,
			AnswerRequestInfo = answerRequestInfoV2
		}
	else
		vccInfo.QnAMakerV2 = {
			TotalSubscriptions = qnaMakerTotalSubscriptionsV2,
			SubscriptionsKeys = {}
		}
	end

	local ttsasrEngines = model_ttsasrengine.GetTtsAsrEngines(vcc, keyspace)
	vccInfo.ttsasrEngines = ttsasrEngines

	-- AppsIntegrations
	if vccInfo.AppsIntegrations and type(vccInfo.AppsIntegrations) == "string" then
		vccInfo.AppsIntegrations = json.decode(vccInfo.AppsIntegrations)
	end

	-- Permissions (permisos del VCC sobre apps que se forzaran su estado (enabled/disabled) sin importar lo que se ha configurado en el rol o en override de usuarios)
	vccInfo.Permissions = utils.JSON_Decode(vccInfo.Permissions)

	--LogInfo("[AGENT:LoadVccConfig] VCC '%s' Config: '%s'.", vcc, json.encode(vccInfo))
	return vccInfo
end

--[[
	AppsIntegrations, value={"HelpDesk":{"Enabled":"1","Licenses":{"Interaction":"2","Supervisor":"3","Administration":"1"},"UserAuthentication":{"Password":"pwd","User":"user"},"Server":{"IP":"168.62.161.232","PORT":"2443"}}}, timestamp=1554232196629066)
]]--
local function SetUserLoggedInfo (vccInfo, loggedInfo)
	if not vccInfo or not loggedInfo then
		return
	end
	if vccInfo.AppsIntegrations and (loggedInfo.Licenses and loggedInfo.Licenses.AppsIntegrations) then
		if vccInfo.AppsIntegrations.HelpDesk and loggedInfo.Licenses.AppsIntegrations.HelpDesk then
			local licenses = vccInfo.AppsIntegrations.HelpDesk.Licenses or {}
			local helpDeskLicenses = loggedInfo.Licenses.AppsIntegrations.HelpDesk
			licenses.FullApp = {
				isAvailable = helpDeskLicenses.IsLicensed
			}
			licenses.Tickets = {
				isAvailable = helpDeskLicenses.IsLicensed
			}
			LogInfo("[AGENT:SetUserLoggedInfo] HelpDesk Info: '%s'", json.encode(licenses))
			vccInfo.AppsIntegrations.HelpDesk.Licenses = licenses
		end
	end
end

local function GetVCCInfo (vcc, keyspace, userLoggedInfo)
	local vccInfo = LoadVccConfig(vcc, keyspace)
	local passwordExpirationDays, passwordChecks
	local processMaxManualRate = 8

	if vccInfo.Security
		and vccInfo.Security.authentication
		and (
			vccInfo.Security.authentication.type == "BASIC"
			or vccInfo.Security.authentication.type == "MFA"
		)
	then
		local vccPasswordChecks = vccInfo.Security.authentication.passwordChecks
		passwordChecks = {
			LEN     = { range = {vccPasswordChecks.minLen, vccPasswordChecks.maxLen} },
			UPPERS  = { range = {vccPasswordChecks.minUpperChars}                    },
			LOWERS  = { range = {vccPasswordChecks.minLowerChars}                    },
			NUMBERS = { range = {vccPasswordChecks.minNumericChars}                  },
			SYMBOLS = { range = {vccPasswordChecks.minSpecialChars}                  },
			HISTORY = { range = {vccPasswordChecks.history}                          }
		}
		--
		if userLoggedInfo then
			local now = os.time()
			local passwordExpiration = tonumber(vccPasswordChecks.expirationDays) * 24 * 60 * 60
			local passwordExpirationDate = userLoggedInfo.PasswordChangeTimestamp and (tonumber(userLoggedInfo.PasswordChangeTimestamp) + passwordExpiration) or now
			-- Si estamos en periodo de notificacion de proxima expiracion de password, devolvemos el numero de dias que quedan para que expire
			local timeToExpiration = (passwordExpirationDate - now)
			local daysToExpiration = math.floor(timeToExpiration / (24 * 60 * 60))
			if tonumber(vccPasswordChecks.expirationNotificationDays) > 0 and daysToExpiration <= tonumber(vccPasswordChecks.expirationNotificationDays) then
				passwordExpirationDays = daysToExpiration
			end
		end
	end

	if vccInfo.TelephonyConfig and vccInfo.TelephonyConfig.ProcessMaxManualRate then
		local rateAsNumber = tonumber(vccInfo.TelephonyConfig.ProcessMaxManualRate)
		if rateAsNumber then 
			processMaxManualRate = rateAsNumber
		end
	end

	-- Solo envio al lado del cliente, la información del nombre de la compania, dirección, y datos relevantes.
	-- No se envia información, de licencias, y demas.
	local info = {
		Id                       = vccInfo.Id or "",
		Company                  = vccInfo.Company or "",
		CompanyAddress           = vccInfo.CompanyAddress or "",
		ContactEmail             = vccInfo.ContactEmail or "",
		ContactName              = vccInfo.ContactName or "",
		Description              = vccInfo.Description or "",
		Telephone1               = vccInfo.Telephone1 or "",
		Telephone2               = vccInfo.Telephone2 or "",
		AllegroIntegration       = vccInfo.AllegroIntegration,
		AppsIntegrations         = vccInfo.AppsIntegrations,
		InteractionNotifications = vccInfo.InteractionNotifications,
		MsCognitiveServices      = vccInfo.MsCognitiveServices,
		QnAMaker                 = vccInfo.QnAMaker,
		QnAMakerV2               = vccInfo.QnAMakerV2,
		Clients                  = vccInfo.Clients,
		ttsasrEngines            = vccInfo.ttsasrEngines,
		BotInfo                  = vccInfo.BotInfo,
		ScreenRecording          = vccInfo.ScreenRecording,
		VideoConfig              = vccInfo.VideoConfig,
		ConfigLimits             = vccInfo.ConfigLimits,
		DraftsConfig             = vccInfo.DraftsConfig,
		PhoneValidation          = vccInfo.PhoneValidation,
		Audit                    = vccInfo.Audit,
		Databases                = vccInfo.Databases,
		ProcessMaxManualRate     = processMaxManualRate,
		HasExternalUsersAuth     = authFilter:HasExternalAuth(vccInfo),
		ExternalUsersAuthType    = authFilter:ExternalAuthType(vccInfo),
		HasExternalUsersSync     = authFilter:HasExternalUsersSync(vccInfo),
		ExternalUsersSyncMethod  = authFilter:ExternalSyncMethod(vccInfo),
		passwordChecks           = passwordChecks,
		passwordExpirationDays   = passwordExpirationDays,
		Permissions              = vccInfo.Permissions
	}

	SetUserLoggedInfo(info, userLoggedInfo)
	return info
end

function HasConfiguredInteractionNotificationsInVCC (vcc, keyspace)
	local vccInfo = GetVCCInfo(vcc, keyspace)
	if not vccInfo then
		return false
	end
	if not vccInfo.InteractionNotifications then
		return false
	end
	if vccInfo.InteractionNotifications.Enabled ~= "1" then
		return false
	end
	return true
end

function GetChannelsLicences (vcc, ksAgent)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	if m_helpers.utils.HasLicenseSystem() then
		local options = utils.Get_m_helpers_agentservice_options()
		return m_helpers.agentservice.GetChannelLicenses(options, vcc)
	end
	local vccInfo = LoadVccConfig(vcc, ksAgent)
	local types = {
		CALL       = false,
		CHAT       = false,
		FACEBOOK   = false,
		INSTAGRAM  = false,
		YOUTUBE	   = false,
		APPSTORE   = false,
		GOOGLEPLAY = false,
		RCS        = false,
		IM         = false,
		MAIL       = false,
		SMS        = false,
		WHATSAPP   = false,
		TWITTER    = false,
		WEBCONTACT = false
	}
	for channel in pairs(types) do
		types[channel] = vccInfo[channel .. "_licensed"] == "1"
	end

	types.VOICEMAIL = types.CALL

	return types
end

function hasLicensesForChannel (channel, ksAgent, vcc)
	channel = (channel or ""):upper()
	local licensed = GetChannelsLicences(vcc, ksAgent)
	if licensed and licensed[channel] then
		return true
	end
	return false
end

---
-- Given a user, return all interactions queued to the campaigns the users belongs to, all the interactions assigned
-- to the user and the user campaign's counters.
--
local function GetCurrentInteractions (vcc, name, campaigns)
	LogInfo("[AGENT:GetCurrentInteractions] Get Current Interactions: '%s@%s'", name, vcc)

	local campaignCounters = {}
	local userAtVcc = name .. "@" .. vcc

	-- Fetch level ids and build a lookup map
	local user_attn_levels = models.Users.GetUserAttentionLevels(userAtVcc)
	for _, level in ipairs(user_attn_levels) do
		user_attn_levels[level] = level
	end

	-- everything on the user campaigns, filtered by attention level
	local params = {
		campaign_id = nil,
		start = 0,
		total = TOP_INTERACTIONS,
		filter = function(interaction)
			if not interaction then
				return false
			end
			if not interaction.attention_level or user_attn_levels[interaction.attention_level.id] then
				return true
			end
			return false
		end
	}

	for _, campaign in pairs(campaigns) do
		campaignCounters[campaign.Id] = models.Interactions.GetInteractionsSummary(campaign.Id, user_attn_levels)
	end

	-- everything for the user itself
	local ksInteractions = utils.GetDatasource():CassandraConnection("Interactions")
	local userInteractions = models.Interactions.GetUserInteractions(vcc, name, ksInteractions)
	m_interactions.CleanupInteractionsOnAgentStartup(vcc, name, userInteractions)
	local userOpenInteractions = models.Interactions.GetUserOpenInteractions(vcc, name, ksInteractions)
	local userAbandonedInteractions = models.Interactions.GetUserAbandonedInteractions(vcc, name, ksInteractions)
	local userQueuedInteractions = models.Interactions.GetUserQueuedInteractions(vcc, name, ksInteractions)
	local userDraftPendingInteractions = models.Interactions.GetDraftsInteractions(userAtVcc, ksInteractions)
	ksInteractions:Close()

	local userInteractionList = {}
	for _, interaction in pairs(userInteractions) do
		table.insert(userInteractionList, interaction)
	end
	for id, interaction in pairs(userOpenInteractions) do
		if not userInteractions[id] then
			table.insert(userInteractionList, interaction)
		end
	end
	for id, interaction in pairs(userAbandonedInteractions) do
		if not userInteractions[id] then
			table.insert(userInteractionList, interaction)
		end
	end
	for id, interaction in pairs(userQueuedInteractions) do
		if not userInteractions[id] then
			table.insert(userInteractionList, interaction)
		end
	end

	-- Se obtienen las interacciones pending 
	for id, interaction in pairs(userDraftPendingInteractions or {}) do
		if not userInteractions[id] then
			table.insert(userInteractionList, interaction)
		end
	end

	return userInteractionList, campaignCounters
end


--logs out the currently logged agent
function LogoutAgent (authUser)
	local authUser = authUser or inconcert.authUser
	local vcc = authUser.virtualContactCenter
	local name = authUser.userName
	local sessionId = authUser.authToken
	local kickout = authUser.kickout
	local loggedOut = agentservice.Logout(sessionId, name, vcc, kickout)
	if loggedOut == false then
		LogError("Failed logging out agent %s@%s on session %s", name, vcc, sessionId)
	end
end

--the reference is anyway dummy cause reads a local copy, but it lives on the module
--in order to avoid adding the servers everytime
local m_globaltimer

---
-- Calculates the time difference between the GlobalTime server and the date supplied by the client (an agent)
--
local function CalculateTimeDifference (clientDate)
	if not m_globaltimer then
		m_globaltimer = lua_globaltimer.LuaGlobalTimer(inconcert.environment.apps.agent.cluster)
		local timeServers = inconcert.environment.apps.agent.timeServers
		for _, server in pairs(timeServers or {}) do
			m_globaltimer:AddKnownServer(server.IP, server.port)
		end
	end
	local now = LuaOrm.Date(m_globaltimer:GetTime())
	return clientDate:interval(now)
end

local function SetChannelAddress (data)
	local vcc = inconcert.authUser.virtualContactCenter
	local user = inconcert.authUser.userName
	local res, encodedData = pcall(json.encode, data)
	if not res or not encodedData then
		LogError("Invalid data for setting channel addresses, user: '%s', vcc: '%s', data :'%s'", user, vcc, tostring(data))
		return false
	end
	LogInfo("Setting channel addresses - user: '%s', vcc: '%s', address, data: %s", user, vcc, encodedData)
	local clientType, addressId, resourceAddress = data.ClientType, data.address_id, data.resource_address
	if clientType == "CALL" then
		if data.phoneType == "external" then
			addressId = data.external_phone
			resourceAddress = data.phoneType:upper()
		else
			--addressId = ( (not data.phoneType or data.phoneType == "builtin") and "RTMP" or "SIP") .."/".. data.name
			addressId = "SIP/".. data.name
		end
	end
	local result = agentservice.SetChannelAddress(user, vcc, clientType, addressId, resourceAddress)
	if not result then
		LogError("Failed setting channel addresses, user: '%s', vcc: '%s', data: '%s'.", user, vcc, encodedData)
	end
	return result
end

--[[
	Obtiene desde authFilter:GetLoggedUserInfo informacion del usuario logueado.
	Retorna:
		userLoggedInfo = {
			Licenses = {
				AppsIntegrations = {
					HelpDesk = {
						IsLicensed = true
					}
				}
			}
		}
]]--
local function LoadUserLoggedInfo (userId, vcc, sessionId)
	return authFilter:GetLoggedUserInfo(userId, vcc, sessionId) or {}
end

function GetTranslatedApplicationsActions (keyspace, vcc)
	LogInfo("[AGENT:GetTranslatedApplicationsActions] Retrieve Translated Application Actions...")
	local _T = m_htmlRender.extension.makeTextTranslator()
	local applicationsActions = model_perm.GetApplicationsActions(keyspace, vcc)
	for _, app in pairs(applicationsActions) do
		if app.Actions then
			for _, action in pairs(app.Actions) do
				action.DescriptionLabel = _T(action.Description)
			end
		end
		app.NameLabel = _T(app.Name)
	end
	return applicationsActions
end

--[[
	Callback que se invoca cuando tenemos un warning de una campana de Allegro.
	El mismo termina ejecutando un request asincrono contra la app "administracion->campaigns", para que esta se encargue de sincronizar los usuarios.
]]--
local function CallbackOnAddWarningCampaign (vcc, warning, ksAgent)
	assert(type(vcc) == "string")
	assert(type(warning) == "table")
	assert(type(warning.campaignId) == "string")
	assert(type(warning.error) == "table")
	assert(ksAgent)
	local err = warning.error
	--LogDebug("[AgentStartup.CallbackOnAddWarningCampaign] ----> warning: %s", json.encode(warning))
	local totalErrMembers = #(err.members or {})
	if totalErrMembers == 0 then
		--LogDebug("[AgentStartup.CallbackOnAddWarningCampaign] 1 - NO must processing warning: err.errType: '%s', totalErrMembers: %s.", err.errType or "UNRECOGNIZED_ERROR", totalErrMembers)
		return false, err
	end
	-- obtengo el controller campaign de administracion para realizar validaciones
	local admCampaignsController = utils.GetControllerOthersApps("administration", {controllerName = "campaigns", modelName = nil})
	local musExecute, resp = admCampaignsController.MustExecuteAutomaticAllegroMembersSynchronize(vcc, warning.campaignId, ksAgent, warning)
	if not musExecute then
		local err = resp
		LogInfo("[AgentStartup.CallbackOnAddWarningCampaign] 2 - NO must processing warning. Err: %s", json.encode(resp))
		return false, err
	end
	-- ejecuto el request asincronico contra la app "administration" controller "campaigns" funcion "SynchronizeAllegroMembers"
	LogInfo("[AgentStartup.CallbackOnAddWarningCampaign] ===> Execute Async Synchronize Allegro Members. Campaign: %s, totalErrMembers: %s.", warning.campaignId, totalErrMembers)
	local postData = json.encode({
		campaignId = warning.campaignId,
		members    = err.members
	})
	utils.AsyncTasksWithAuthToken(
		"/inconcert/apps/administration/campaigns/synchronize_allegro_members/", -- url
		"SynchronizeAllegroMembers",                                             -- taskId
		"POST",                                                                  -- method
		nil,                                                                     -- headers
		postData,                                                                -- jsonData
		nil                                                                      -- cookies
	)
	return true
end

---
-- Processes the bootstrap for the client just logged in.
-- groups, settings, time difference and other stuff is handled here.
--
local function AgentStartup (utcClientDate, web)
	local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local username = inconcert.authUser.userName
	local sessionId = inconcert.authUser.authToken
	local sessionRejoin = inconcert.authUser.sessionRejoin
	local RID = web.vars.HTTP_X_ICC_RID

	LogDebug("")
	LogInfo("------------------------------------------------------------------------------------------------------------------------------------------------")
	LogInfo("--> [AgentStartup] START Bootstraping console for user '%s@%s' who claims to be on UTC %s and sessionid '%s, RID: '%s' - startTime: '%s'.", username, vcc, tostring(utcClientDate), sessionId, RID, mainProcessElapsedTime.startTime)
	LogInfo("------------------------------------------------------------------------------------------------------------------------------------------------")

	-- diferencia de fechas contra server GlobalTimer
	local timeDifference = CalculateTimeDifference(utcClientDate)

	--LogDebug("[AgentStartup] 1) AgentService -> Validating if user '%s@%s' is logged in.", username, vcc)

	if sessionRejoin then
		LogInfo("--> [AgentStartup] Rejoined sessionid '%s' for user '%s@%s' who claims to be on UTC %s", sessionId, username, vcc, tostring(utcClientDate))
		inconcert.authUser.sessionRejoin = nil
	else
		if agentservice.IsAgentLoggedIn(username, vcc) then
			LogWarning("[AgentStartup] User was already logged in %s@%s kickin' it out", username, vcc)
			if not agentservice.Logout("", username, vcc) then
				LogError("[AgentStartup] Failed kicking %s@%s out", username, vcc)
				return {result = false, reason = _T("Your account was already in use, unable to continue")}
			end
		end

		--LogDebug("[AgentStartup] 2) AgentService-> Starting session for user '%s@%s'.", username, vcc)

		local loggedIn, loginError = agentservice.Login(sessionId, username, vcc)
		if loggedIn == false then
			LogError("[AgentStartup] Failed logging in agent '%s@%s' on session '%s' - %s", username, vcc, sessionId, loginError)
			return {result = false, reason = _T("Unable to complete user login"), loginError = loginError }
		end
	end
	--LogDebug("[AgentStartup] 3) Loading User '%s@%s' from Data Base.", username, vcc)

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")

	local full_user_id = utils.AppendVcc(username, vcc)
	local user = models.Users.Get(full_user_id, nil, ksAgent)
	if not user then
		LogError("[AgentStartup] User '%s@%s' not found", username, vcc)
		ksAgent:Close()
		return {result = false, reason = _T("User not found")}
	end

	--retrieve the user settings, if non existent, initialize with a sensible default
	--LogDebug("[AgentStartup] 4) AgentService -> Set Channel Addresses for User '%s@%s'.", username, vcc)
	local settings = {
		InteractionClients = {},
		iceServers = {}
	}
	for addressType, address in pairs(user.addresses) do
		if sessionRejoin or SetChannelAddress(address) then
			settings.InteractionClients[addressType] = address
		end
	end

	-- #19796 Add the ice servers configuration
	settings.iceServers = modelIceServers.GetAllIceServers()

	-- retrieve the campaigns for this user and send them fully with the user role
	--LogDebug("[AgentStartup] 5) Loading Campaigns information for User '%s@%s' from Data Base.", username, vcc)

	local campaignRoles = models.Campaigns.GetUserCampaigns(username, vcc, ksAgent)
	local campaigns = {}
	for campaign_id, campaign_role in pairs(campaignRoles) do
		local campaign = models.Campaigns.Get(campaign_id, ksAgent)
		if campaign then
			campaigns[campaign_id] = campaign
			campaigns[campaign_id].role = campaign_role
			campaigns[campaign_id].status = true --when logging in the status is always active
		else
			LogError("[AgentStartup] The user '%s@%s' has been assigned the campaign '%s' and it does not exist", username, vcc, campaign_id)
		end
	end

	--LogDebug("[AgentStartup] 6) Loading Current Interactions for User '%s@%s'.", username, vcc)
	local userInteractions, campaignCounters = GetCurrentInteractions(vcc, username, campaigns)
	interactionmanip.NormalizeInteractions(userInteractions)
	-- Get rid of heavy properties, and precompute some of the stuff we lose.
	interactionmanip.StripUnwantedMembers(userInteractions)

	--retrieve total available states
	--LogDebug("[AgentStartup] 7) Loading Users States for User '%s@%s' from Data Base.", username, vcc)
	local availableStates = models.AgentStates.FindAll(vcc, ksAgent)
	for stateId, s in pairs(availableStates) do
		s.Id = stateId
		if s.Description then
			s.Description = _T(s.Description)
		end
		if s.ShortMessage then
			s.ShortMessage = _T(s.ShortMessage)
		end
	end

	--LogDebug("[AgentStartup] 8) Loading Users Permissions for User '%s@%s' from Data Base.", username, vcc)
	local loggedUserPermissions = model_perm.GetLoggedUserPermissions(ksAgent)

	--LogDebug("[AgentStartup] 10) Loading Applications Actions.")
	local applicationsActions = GetTranslatedApplicationsActions(ksAgent, vcc)

	local userTimezoneInfo = models.Timezones.Get(user.TimeZoneId, ksAgent)

	--
	local userLoggedInfo = LoadUserLoggedInfo(username, vcc, sessionId)
	userLoggedInfo.PasswordChangeTimestamp = user.PasswordChangeTimestamp
	local vccInfo = GetVCCInfo(vcc, ksAgent, userLoggedInfo)

	-------------------------------
	local campaignsWarnings = {
		statusAutoSyncAllegroMembers = "NONE",
		warnings = {}
	}

	local now = os.time()
	--[[
		Funcion encargada de invocar a la funcion generica que determina si una campana de Allegro debe estar en la lista de Warnings, ya que sus usuarios NO han sido sincronizados correctamente.
	]]--
	local function AddCampaignsWarning (campaign)
		utils.AddCampaignsWarning(
			vcc,
			campaign,
			campaignsWarnings.warnings,
			now,
			function(warning)
				-- realizo el request asincronico que sincroniza los members con allegro
				local success, result = CallbackOnAddWarningCampaign(vcc, warning, ksAgent)
				warning.error.members = {} -- para que no viajen todos los members como respuesta al cliente (frontend)
				if not success then
					local err = result
					if err.result == "UNABLE TO CONNECT WITH ALLEGRO"
						or err.result == "CURRENT ALLEGRO ERROR DOES NOT ALLOW SYNC OF MEMBERS"
					then
						-- los unicos errores que debo de marcar
						campaignsWarnings.statusAutoSyncAllegroMembers = err.result
						return
					end
				end
				-- si es NONE agrego el valor "RUNNING", solo si es la primera vez
				campaignsWarnings.statusAutoSyncAllegroMembers = campaignsWarnings.statusAutoSyncAllegroMembers == "NONE"
					and "RUNNING"
					or campaignsWarnings.statusAutoSyncAllegroMembers
			end,
			ksAgent
		)
	end
	-- recorro la lista de campanas del VCC.
	local internalCampaignId = ""
	local vccCampaigns = models.Campaigns.GetVCCCampaigns(vcc, ksAgent)
	for campaignIdWithoutVCC in pairs(vccCampaigns) do
		local campaign = admCampaignsModel.GetCampaign(vcc, campaignIdWithoutVCC, ksAgent, {"Id", "IsInternal", "UpdateMembersResult"})
		if campaign then
			if campaign.IsInternal == "1" then
				internalCampaignId = campaignIdWithoutVCC
			end
			-- invoco funcion para saber si tiene que estar la misma dentro de las campanas de Allegro en Warning
			AddCampaignsWarning(campaign)
		end
	end
	-- Campaigns Warnings
	LogInfo("[AgentStartup] Status of Automatically Sync Allegro Users Campaigns Is: '%s'", campaignsWarnings.statusAutoSyncAllegroMembers)
	--LogDebug("[AgentStartup] campaignsWarnings: %s", json.encode(campaignsWarnings))

	-- AgentMonitorService
	LogInfo("[AgentStartup] AgentMonitorService: Loading User '%s@%s' capture statuses.", username, vcc)
	local agentCaptureStatus = models.Users.GetUserCaptureStatisticsStatus(username, vcc)

	local redis = agentModel:GetRedisClient()
	local vccWarnings = redis:get(redisSyncUsersWarningKey .. vcc)
	--LogDebug("[AgentStartup] 11) Making Information for User '%s@%s'.", username, vcc)
	local response = {
		result                  = true,
		timeDifference          = timeDifference,
		settings                = settings,
		campaigns               = campaigns,
		groupPermissions        = loggedUserPermissions and loggedUserPermissions.groupPermissions,
		userPermissions         = loggedUserPermissions and loggedUserPermissions.userPermissions,
		user                    = username,
		domain                  = vcc,
		licensedAccountTypes    = GetChannelsLicences(vcc, ksAgent),
		agentCaptureStatisticsStatus = agentCaptureStatus,
		userLanguage            = user.LanguageId,
		userHasAvatar           = user.Avatar and true or false,
		userName                = user.Name,
		userLastName            = user.LastName,
		userFirstSurname        = user.FirstSurname,
		userLastSurname         = user.LastSurname,
		userEmail               = user.Mail,
		userDefaultCampaign     = user.DefaultCampaign,
		userDateFormat          = user.DateFormat,
		userCompanyInfo         = user.CompanyInformation or {},
		userCustomInfo          = user.CustomInformation or {},
		userClients             = user.Clients,
		userAccessToken         = authFilter:GetTokenFromUser(username, vcc),
		userIdleThreshold       = user.idleThreshold and tonumber(user.idleThreshold) or 2, -- el mismo valor por defecto que se encuentra en el JavaScript 'personalsettings.js' en la variable m_idleDefault
		userSoundNotifications  = user.soundNotifications == "1" and true or false,
		userVisualNotifications = user.visualNotifications == "1" and true or false,
		sessionRejoin           = sessionRejoin,
		userRole                = user.UserGroup,
		initialState            = user.initialState,
		country                 = user.countryCode,
		area                    = user.areaCode,
		timezone                = user.TimeZoneId,
		enterAsKeyToSend        = (user.EnterAsKeyToSend and user.EnterAsKeyToSend == "true") and true or false,
		userTimezoneInfo        = userTimezoneInfo,
		userIsFullSystemAccess  = utils.CurrentLoggedUserIsFullSystemAccess(),
		status                  = GetUserInitialState(vcc, ksAgent),
		availableStates         = availableStates,
		statusMessage           = user.StatusMessage,
		interactions            = userInteractions,
		campaignCounters        = campaignCounters,
		applicationsActions     = applicationsActions,
		vccInfo                 = vccInfo,
		luisInfo                = GetLUISInfo(),
		rasaInfo                = GetRASAInfo(),
		widgetsIntegrationsInfo = GetWidgetsDefinitions({
			htmlRender = m_htmlRender
		}),
		sessionData             = inconcert.authUser.sessionData,
		serverInfo              = GetServerInfo(),
		IRInfo                  = {
			variables = GetIRVariables(RID)
		},
		passwordExpirationDays  = vccInfo.passwordExpirationDays,
		internalCampaignId      = internalCampaignId,
		campaignsWarnings       = campaignsWarnings,
		vccWarnings             = vccWarnings
	}

	ksAgent:Close()

	local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.End(mainProcessElapsedTime.processId)
	LogInfo("--------------------------------------------------------------------------------------------------------")
	LogInfo("[AgentStartup] END Bootstraping console for user '%s@%s'. endTime: %s, ELAPSED TIME: '%s' seconds.", username, vcc, mainProcessElapsedTime.endTime, mainProcessElapsedTime.humanElapsedTime.formattedTime)
	LogInfo("--------------------------------------------------------------------------------------------------------\n")

	return response
end

-------------------------------------
--COMIENZA MAPPING
--Estas deberian bifurcarse segun header y content-type si se quiere extender json a xml
--------------------------------------

local function RefreshAccountStatusesHandler (web)
	LogInfo("[AGENT.RefreshAccountStatusesHandler] LOADING campaigns accounts statuses.")
	web.headers["Content-Type"] = "application/json"
	local campaignModel = utils.GetAgentModel("Campaigns")
	local campaigns = campaignModel.FindAll()
	local m_interactionsController = utils.GetAgentController({controllerName = "interactions", modelName = "Interactions"})
	local response = m_interactionsController.GetCampaignAccountsStatus(campaigns)
	local encodedResponse = json.encode(response)
	--LogDebug("[AGENT.RefreshAccountStatusesHandler] --> Campaigns accounts statuses: %s", encodedResponse)
	return encodedResponse
end

--given a field separated date, returns a LuaDate object
local function GetDateTimeFromTable (dateTable)
	return LuaOrm.Date(
		{
			year = tonumber(dateTable.year),
			month = tonumber(dateTable.month),
			day = tonumber(dateTable.day),
			hour = tonumber(dateTable.hours),
			min = tonumber(dateTable.minutes),
			sec = dateTable.seconds
		}
	)
end

---
-- Processes the initial load for an agent. It returns a json encoded table with the campaigns the agent is in,
-- the available interactions, the permissions, settings, available states and so on.
--
local function InitializeHandler (web)
	local request = web.request

	local utcClientDate = GetDateTimeFromTable(request.utcDate)
	if not utcClientDate then
		web.status = 400
		return "Invalid request"
	end

	return AgentStartup(utcClientDate, web)
end

local function ActivateAgentAddress (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user = inconcert.authUser.userName

	local res, channels = pcall(json.decode, web.input["post_data"])
	if not res or not channels then
		LogError("Activate agent address for user %s@%s received invalid channel", user, vcc)
		web.status = 400
		return "Invalid request"
	end

	for _, channel in pairs(channels) do
		LogInfo("Activating channel %s for user %s@%s", channel, user, vcc)
		local result = agentservice.ActivateAgentAddress(user, vcc, channel)
		if not result then
			web.status = 400
			return "Unable to activate address for channel "..channel
		end
	end
	web.status = 200
	return ""
end

local function AssignAddressHandler (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user = inconcert.authUser.userName
	local res, request = pcall(json.decode, web.input["post_data"])

	LogInfo("[AGENT:AssignAddressHandler] Assigning Agent '%s@%s' Address. Channel: '%s'.", user, vcc, (request or {}).channel)

	if not res or not request or not request.channel then
		LogError("[AGENT:AssignAddressHandler]  Assign agent address for user %s@%s received invalid channel", user, vcc)
		web.status = 400
		return "Invalid request"
	end

	local encodedAddress, err = agentservice.AssignAgentAddress(user, vcc, request.channel)
	if not encodedAddress then
		LogError("[AGENT:AssignAddressHandler]  Assign agent address for user %s@%s received invalid channel", user, vcc)
		web.status = 500
		return "Unable to assign address for channel " .. request.channel .. " - " .. err
	end

	-- OK
	web.status = 200
	LogInfo("[AGENT:AssignAddressHandler] Success Assigned Agent '%s@%s' Address. Channel: '%s'. Address Info: '%s'.", user, vcc, request.channel, encodedAddress)

	return encodedAddress
end

function GetUserInitialState (vcc, keyspace)
	return utils.HasAllegroIntegration(vcc, keyspace) and "NOACD" or "BREAK"
end

--save user settings
local function SaveSettings (web)
	web.headers["Content-Type"] = "application/json"

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local completeUserId = user_name .. "@" .. vcc
	local postedData = web.input["post_data"]

	LogInfo("[AGENT:SaveSettings] Save personal settings, posted data: %s", postedData)

	local res, settings = pcall(json.decode, postedData)
	if not res or not settings or not settings.personal then
		LogError("[AGENT:SaveSettings] Save personal setting, incorrect posted data!!! %s", postedData)
		return json.encode({result = false})
	end

	local initialState
	if utils.HasAllegroIntegration(vcc) then
		initialState = GetUserInitialState(vcc)
	else
		initialState = settings.personal.initialState or GetUserInitialState(vcc)
		if initialState:upper() == "ACTIVE" then
			local hasPermissionToAppInteraction = model_perm.HasPermissionToApplication( "interaction", user_name, inconcert.authUser.userGroup )
			if not hasPermissionToAppInteraction then
				LogError("User %s not has permission for change to ACTIVE initial state.", completeUserId)
				web.headers["Content-Type"] = "application/json"
				return json.encode({result = false, error = "NOT ALLOWED INITIAL AVAILABLE SYSTEM STATE"})
			end
		end
	end

	local userBeforeSaving = models.Users.Get(completeUserId)
	local userSkills = models.Users.GetAgentSkills (vcc, user_name) or {}
	userBeforeSaving.skills = {}
	for skillId, skillValue in pairs(userSkills) do
		local skill = {
				skill = skillId,
				weight = skillValue
		}
		table.insert(userBeforeSaving.skills, skill)
	end
	LogInfo("[AGENT:SaveSettings] Saving personal settings for user:%s@%s", user_name, vcc)

	local function RecursiveUpdate(newInfo, oldInfo)
		for name, value in pairs(newInfo) do
			if type(value) == "table" then
				if type(oldInfo[name]) == "table" then
					RecursiveUpdate(value, oldInfo[name])
				else
					oldInfo[name] = value
				end
			else
				oldInfo[name] = value
			end
		end
	end

	local newUserValues = settings.personal
	-- Obtengo el layout que han posteado para que el mismo NO se actualice en la funcion recursiva "RecursiveUpdate"
	local newLayouts = newUserValues.layouts or nil
	newUserValues.layouts = nil
	local currentUserValues = utils.DeepCopy(userBeforeSaving)

	-- Actualizo de forma recursiva los campos del Usuario (menos el layout)
	RecursiveUpdate(newUserValues, currentUserValues)

	-- Ahora actualizo el layout
	local clients = currentUserValues.Clients or {}
	clients.Layouts = newLayouts
	currentUserValues.Clients = clients

	-- Change to string the EnterAsKeyToSend value
	currentUserValues.EnterAsKeyToSend = currentUserValues.EnterAsKeyToSend and "true" or "false"

	-- Actualizo contra el controller de Usuarios
	local moduleAgentUsers = require "apps.agent.controllers.users"
	currentUserValues.newUser = false
	local success, err = moduleAgentUsers.UpdateUser(currentUserValues, nil, nil)
	if success then
		LogInfo("[AGENT:SaveSettings] Personal setting successfully saved.")
	else
		LogError("[AGENT:SaveSettings] Error saving personal data. Err: %s", json.encode(err or {}))
	end

	return json.encode({result = success, error = err})
end

local function RemoveVccsAudits (web)
	local vccs = model_vccs.GetAllVccs()
	if utils.TableIsEmpty(vccs) then
		return
	end
	audits.RemoveVccsAudits({
		vccs = vccs
	})
end

-- refresh external users
local function RefreshExternalUsersHandler (web)
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local vccs = model_vccs.GetAllVccs(ksAgent)
	local ksHistorical

	local vccsUsersOverridePermissions = {}
	local function GetUsersOverridePermissions (vcc)
		if not vccsUsersOverridePermissions[vcc] then
			vccsUsersOverridePermissions[vcc] = model_perm.GetUsersOverridePermissions(vcc, ksAgent)
		end
		return vccsUsersOverridePermissions[vcc]
	end

	--#19249: En redis agregamos flag para cada VCC en el que se haya superado el limite MaxCampaignsUsers
	local redis = agentModel:GetRedisClient()

	for vccId, vcc in pairs(vccs) do
		if authFilter:HasExternalUsersSync(vcc) then
			LogInfo("[AGENT:RefreshExternalUsersHandler] - start external users synchronization for vcc '%s'", vccId)
			--Obtenemos los usuarios desde el server externo
			local externalUsersSuccess, externalUsers, defaultValues = authFilter:GetExternalUsers(vcc)
			--En caso de errores al recuperar la informacion de usuarios externos no hacemos nada porque no queremos que se desactiven los usuarios en occ
			if externalUsersSuccess then
				--Obtenemos los usuarios actualmente existentes en el vcc
				local vccUsers = models.Users.GetVCCUsers(vccId, ksAgent)
				local deactivateNonExternalUsers = {}
				--En principio agregamos todos los usuarios existentes en el vcc a una tabla para desactivarlos
				for simpleUserId, vccUser in pairs(vccUsers) do
					local fullUserId = utils.AppendVcc(utils.GetId(simpleUserId), vccId)
					local existingUser = models.Users.Get(fullUserId, nil, ksAgent)
					existingUser.Status = "0"
					existingUser.StatusReason = "SYNC"
					existingUser.StatusTimestamp = tostring(os.time())
					existingUser.wasActive = tonumber(vccUser.Status) == 1

					--Mantener skills de usuarios aunque luego los desactive
					--Recuperar los skills del user
					local userSkills = models.Users.GetAgentSkills(vccId, simpleUserId, ksAgent) or {}
					existingUser.skills = {}
					for skillId, skillValue in pairs(userSkills) do
						local skill = {
							skill = skillId,
							weight = skillValue
						}
						table.insert(existingUser.skills, skill)
					end

					deactivateNonExternalUsers[simpleUserId] = existingUser
				end

				local campsUsersExceedsLimitAllowed = false
				-- recorremos la tabla de usuarios que vienen del server externo
				for externalUserId, externalUser in pairs(externalUsers) do
					local userAtVCC = externalUser.Id .. "@" .. externalUser.VCC
					local userExists = models.Users.UserExists(userAtVCC, ksAgent)
					local mustSaveUser = false
					if userExists then
						--Usuario existente ya en OCC, lo eliminamos de la lista de usuarios a desactivar
						externalUser.skills = deactivateNonExternalUsers[externalUser.Id].skills or {}
						deactivateNonExternalUsers[externalUser.Id] = nil
						--vemos si hay que actualizarlo comprobando si cambia alguno de los datos que me traigo del external server
						local oldUser = models.Users.Get(userAtVCC, nil, ksAgent)
						--en el caso del mail, si son distintos y el que tenemos es el de por defecto pisamos el mail actual
						local defaultUserMail = oldUser.Id.."@"..vccId..".com"
						local mustUpdateMail = oldUser.Mail ~= externalUser.Mail and oldUser.Mail == defaultUserMail or (not oldUser.Mail and externalUser.Mail)
						if not mustUpdateMail then
							externalUser.Mail = oldUser.Mail
						end
						mustSaveUser = oldUser.Name ~= externalUser.Name or oldUser.FirstSurname ~= externalUser.FirstSurname or mustUpdateMail or oldUser.Status == "0" or oldUser.UserGroup ~= externalUser.UserGroup
						if mustSaveUser then
							--En la actualizacion del usuario mantenemos todos sus datos actuales excepto Name y FirstSurname. El Mail tampoco lo tenemos en cuenta aqui porque ya lo hemos hecho mas arriba. El UserGroup tampoco porque nos pueden haber cambiado de grupo al usuario en el servidor LDAP
							for attribName, attrib in pairs(oldUser) do
								if attribName ~= "Name" and attribName ~= "FirstSurname" and attribName ~= "Mail" and attribName ~= "UserGroup" then
									externalUser[attribName] = attrib
								end
							end
						end
					else
						--Si no existe el usuario, aplicamos los atributos por defecto
						for attrName, attrValue in pairs(defaultValues) do
							externalUser[attrName] = defaultValues[attrName]
						end
						-- Y aplicamos los addresses por defecto
						-- addresses
						externalUser.addresses = {}
						externalUser.addresses.CHAT = {
							address_id = externalUser.Id .. "@" .. vccId,
							resource_address = "",
							resourceAddress = "",
							vcc = externalUser.VCC,
							alias =  externalUser.Name,
							username = externalUser.Id,
							address = externalUser.Id
						}
						externalUser.addresses.IM = {
							address_id = externalUser.Id .. "@" .. vccId,
							resource_address = "",
							resourceAddress = "",
							vcc = externalUser.VCC,
							alias =  externalUser.Name,
							username = externalUser.Id,
							address = externalUser.Id
						}
						if externalUser.Mail then
							externalUser.addresses.MAIL = {
								address_id = externalUser.Mail,
								resource_address = "",
								address = externalUser.Mail
							}
						end
						mustSaveUser = true
					end

					if mustSaveUser then
						local cont_campaigns_user = 0
						if not userExists then
							--#19249: Si voy a agregar un nuevo usuario, validar MaxCampaignsUsers
							cont_campaigns_user = externalUser.groups and #externalUser.groups
							if cont_campaigns_user > 0 then
								local success, error = modelRestrictions.ValidateConfigurationsLimits(vccId, "MaxCampaignsUsers", ksAgent, cont_campaigns_user)
								if not success then
									campsUsersExceedsLimitAllowed = true
									LogWarning("[AGENT:RefreshExternalUsersHandler] - Vcc '%s' users syncronization will stop while user campaigns exceeds allowed limit.", vccId)
									break --Exists externalUsers loop
								end
							end
						end

						LogInfo("[AGENT:RefreshExternalUsersHandler] - saving external user '%s':'%s'", externalUserId, json.encode(externalUser))
						if not deactivateNonExternalUsers[externalUser.Id] then
							externalUser.Status = "1"
						end
						local success, result = models.Users.Save(externalUser.Id, externalUser.VCC, externalUser, ksAgent)
						if not success then
							LogWarning("[AGENT:RefreshExternalUsersHandler] - error saving external user '%s':%s", externalUserId, result.result)
						end
						local usersOverridePermissions = GetUsersOverridePermissions(externalUser.VCC)
						local auditUserData = GetUserAuditData(userAtVCC, ksAgent, usersOverridePermissions, externalUser.VCC)
						if  auditUserData and not utils.TableIsEmpty(auditUserData) then
							if not ksHistorical then
								ksHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
							end
							audits.Create({
								vcc = externalUser.VCC,
								entity = "User",
								user = "system",
								fieldKey = utils.GetId(externalUser.Id),
								newFields = auditUserData,
								connection = ksHistorical
							})
						end
						-- #19249: Si he agregado un nuevo usuario, actualizar el contador actual de MaxCampaignsUsers
						if success and not userExists and cont_campaigns_user > 0 then
							modelRestrictions.UpdateRedisConfigTotals(vccId, {
								MaxCampaignsUsers = cont_campaigns_user
							}, ksAgent)
						end
					end
				end

				--deactivate non existing external users
				for simpleUserId, user in pairs(deactivateNonExternalUsers) do
					if user.wasActive then
						user.wasActive = nil
						LogInfo("[AGENT:RefreshExternalUsersHandler] - deactivating non existing external user '%s'", simpleUserId)
						local usersOverridePermissions = GetUsersOverridePermissions(user.VCC)
						local fullUserId = utils.AppendVcc(utils.GetId(simpleUserId), user.VCC)
						local oldAuditUserData = GetUserAuditData(fullUserId, ksAgent, usersOverridePermissions, user.VCC)
						local success, result = models.Users.Save(simpleUserId, user.VCC, user, ksAgent)
						if success and oldAuditUserData then
							local newAuditUserData = GetUserAuditData(fullUserId, ksAgent, usersOverridePermissions, user.VCC)
							if newAuditUserData then
								if not ksHistorical then
									ksHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
								end
								audits.Create({
									vcc = user.VCC,
									entity = "User",
									user = "system",
									fieldKey = utils.GetId(simpleUserId),
									newFields = newAuditUserData,
									oldFields = oldAuditUserData,
									connection = ksHistorical
								})
							end
						end
					end
				end

				local redisSyncUsersWarningKeyVcc = redisSyncUsersWarningKey .. vccId
				local vccAuthType = vcc and vcc.Security and vcc.Security.authentication and vcc.Security.authentication.type or ""
				local vccSyncMethod = vcc.Security[vccAuthType:lower()] and vcc.Security[vccAuthType:lower()]["sync_method"] or vccAuthType

				-- #19249: Seteo o elimino la notificacion de usuarios excedidos en el vcc
				if campsUsersExceedsLimitAllowed then
					redis:set(redisSyncUsersWarningKeyVcc, vccSyncMethod)
				else
					redis:del(redisSyncUsersWarningKeyVcc)
				end
			end
			LogInfo("[AGENT:RefreshExternalUsersHandler] - end external users synchronization for vcc '%s'", vccId)
		end
	end
	if ksHistorical then
		ksHistorical:Close()
	end
	ksAgent:Close()
end

-- refresh external group filters
local function RefreshExternalGroupFilters(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user = inconcert.authUser.userName

	LogInfo("[AGENT:RefreshExternalGroupFilters] user '%s@%s'.", user, vcc)
	
	local vccInfo = model_vccs.Get(vcc)
	if vccInfo.Security.authentication.type == "BASIC" or vccInfo.Security.authentication.type == "MFA" then
		return json.encode({})
	end 
	local authType = vccInfo.Security.authentication.type:lower()
	return json.encode(vccInfo.Security[authType].groupFilters)
end

function MapUris ( m, htmlRender )
	local Log = require("apps.agent.requestFilters").LogRequest
	local JsonValidatorReply = utils.JsonValidatorReply

	m:dispatch_post( Log .. JsonValidatorReply .. InitializeHandler, "/initialize/(.*)" )
	m:dispatch_post( Log .. permissions:CheckPermission( "settings", "change_personal_settings", SaveSettings ) , "/save_settings/(.*)" )
	m:dispatch_post( Log .. RefreshAccountStatusesHandler, "/refresh_account_statuses/(.*)" )
	m:dispatch_post( Log .. ActivateAgentAddress, "/activate_address/(.*)" )
	m:dispatch_post( Log .. AssignAddressHandler, "/assign_address/(.*)" )
	m:dispatch_post( Log .. RefreshExternalUsersHandler, "/refresh_external_users/(.*)" )
	m:dispatch_post( Log .. RefreshExternalGroupFilters, "/refresh_external_group_filters/(.*)" )
	m:dispatch_post( Log .. RemoveVccsAudits, "/remove_audits/(.*)" )

	m_htmlRender = htmlRender
end
