require "inconcert.translators"
local agentService = require "lua_agentserviceclient"
local model_perm = require("shared.permissions.models.permissions")
local m_moduleLocation = select(2, ...)
local utils = require "shared.apps.utils"
local utf8 = require "icu.utf8"
local jsonEncoder = require "LuaOrm.Serializers.Json"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_agentclient

function GetAgentClient ()
	if (not m_agentclient) then
		local host = inconcert.environment.apps.agent.agentservice.host
		local port = inconcert.environment.apps.agent.agentservice.port
		m_agentclient = agentService.LuaAgentServiceClient(inconcert.environment.apps.agent.cluster)
		m_agentclient:AddKnownAgentService(host, port)
	end
	return m_agentclient
end

--[[
 
]]--
function GetStatusOnCampaign (user, campaign, vcc)
	local userAtVCC = user.."@"..vcc
	local users = {}
	table.insert(users, user)
	local status = GetAgentClient():GetAgentsStatus(vcc, users)
	if status and status[userAtVCC] and status[userAtVCC].campaigns and status[userAtVCC].campaigns[campaign] then
		return status[userAtVCC].campaigns[campaign]
	end
	return false
end

--[[
 
]]--
function Login (sessionId, user, vcc)
	LogInfo("[AGENTSERVICE:Login] Logging in User '%s@%s', sessionId '%s'.", user, vcc, sessionId)
	local logged = GetAgentClient():AgentLogin(sessionId, user, vcc)
	LogInfo("[AGENTSERVICE:Login] User '%s@%s' with sessionId: '%s', is logged: '%s'.", user, vcc, sessionId, logged)
	return logged == "OK", logged
end

--[[
 
]]--
function Logout (sessionId, user, vcc, kickout)
	local kickoutData = "{}"
	if kickout then
		kickoutData = json.encode(kickout)
	end

	LogInfo("[AGENTSERVICE:Logout] User '%s@%s' with sessionId: '%s' and kickout info '%s'.", user, vcc, sessionId, kickoutData)
	--doing it forced under test as an easy way to cancel interactions and relogin must fix TODO
	return GetAgentClient():AgentLogout("", user, vcc, true, kickoutData)
end

--[[
 
]]--
function ChangeAgentState (sessionId, userId, vcc, newStatus)
	local fullUserId = utils.AppendVcc(userId, vcc)
	local currentStatus = GetAgentStatus(fullUserId)

	if type(currentStatus) == "table" and type(currentStatus.state) == "string" then
		currentStatus = utf8.upper(currentStatus.state)
	end

	local loggedUser = utils.GetCurrentLoggedUser()
	LogInfo("[AGENTSERVICE:ChangeAgentState] User '%s', change status '%s' to user '%s'.", loggedUser, newStatus, fullUserId)

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local modelAgentStates = require("apps.agent.models.states")
	local states = modelAgentStates.FindAll(vcc, keyspace)

	local response = {success = nil, errType = nil, httpCode = nil}
	local function ReturnError (errType, errMsg, httpCode)
		keyspace:Close()
		LogError("[AGENTSERVICE:ChangeAgentState] %s", errMsg)
		response.success = false
		response.errType = errType
		response.httpCode = httpCode or 500
		return response
	end

	if not states then
		local errMsg = "Unable to change state. Data Base States list is empty."
		return ReturnError("DATABASE_STATES_IS_EMPTY", errMsg)
	end
	local stateInfo = states[newStatus]
	if not stateInfo then
		local errMsg = string.format("Unable to change state. State '%s' does not exist.", newStatus)
		return ReturnError("STATE_DOES_NOT_EXIST", errMsg, 400)
	end

	-- Validate permission to interaction application for change to active state
	if utf8.upper(newStatus) == "ACTIVE" then
		local hasPermissionToAppInteraction
		if loggedUser:lower() == fullUserId:lower() then
			hasPermissionToAppInteraction = model_perm.HasPermissionToApplication("interaction", inconcert.authUser.userName, inconcert.authUser.userGroup, nil, nil, nil, keyspace)
		else
			-- Este require lo meti aca, ya que si lo defino arriba, falla cuando se realiza el mismo
			--  porque model.users tambien hace require a agentservice, y falla porque no esta aun inicalizado.
			local modelUsers = require("apps.agent.models.users")
			local userGroup = modelUsers.GetSpecificColumn(fullUserId, "UserGroup", keyspace)
			if not userGroup then
				local errMsg = string.format("Unable to change ACTIVE state. User '%s' does not exist.", fullUserId)
				return ReturnError("USER_DOES_NOT_EXIST", errMsg)
			end
			hasPermissionToAppInteraction = model_perm.HasPermissionToApplication("interaction", userId, userGroup, nil, nil, nil, keyspace)
		end

		if not hasPermissionToAppInteraction then
			local errMsg = string.format("Unable to change state. User '%s' not has permission for change to ACTIVE state.", fullUserId)
			return ReturnError("USER_NOT_HAS_PERMISSION", errMsg, 403)
		end
	else
		if tonumber(stateInfo.isBreak) == 1 then
			local controllerInteractions = require("apps.agent.controllers.interactions")
			local interactionsInfo = controllerInteractions.CountUserAssignedInteractionsByType(fullUserId, vcc, keyspace)
			if interactionsInfo.live > 0 then
				response.assignedLiveInteractions = interactionsInfo.live
				local errMsg = string.format("Unable change to '%s' state. User '%s' have '%s' live interactions assigned.", newStatus, fullUserId, interactionsInfo.live)
				return ReturnError("USER_HAS_ASSIGNED_LIVE_INTERACTIONS", errMsg)
			end
		end
	end

	local success = GetAgentClient():ChangeAgentState(sessionId, userId, vcc, newStatus)
	if not success then
		LogWarning("[AGENTSERVICE:ChangeAgentState] Unable to change '%s' state.", utf8.upper(newStatus))
		local errMsg = string.format("Unable to change '%s' state.", newStatus)
		return ReturnError("AGENT_SERVICE_ERROR", errMsg)
	end

	-- success
	keyspace:Close()

	LogInfo("[AGENTSERVICE:ChangeAgentState] Successfully changed state. The user '%s' has been changed from '%s' to '%s'.", fullUserId, currentStatus, newStatus)

	response.success = true
	return response
end

--[[
 
]]--
function SetChannelAddress (user, vcc, channel, address, resourceAddress)
	if not user or not vcc or not channel or not address or not resourceAddress then
		LogError("Invalid data, setting channel address - user: '%s', vcc: '%s', channel: '%s', address: '%s', resourceAddress: '%s'", 
			user, vcc, channel, address, resourceAddress)
		return
	end
	LogInfo("AgentService - setting channel address - user: '%s', vcc: '%s', channel: '%s', address: '%s', resourceAddress: '%s'", user, vcc, channel, address, resourceAddress)
	return GetAgentClient():SetAgentAddress(user, vcc, channel, address, resourceAddress)
end

function ActivateAgentAddress (user, vcc, channel)
	if not user or not vcc or not channel then
		LogError("Invalid data, On ready channel address - user: '%s', vcc: '%s', channel: '%s'", user, vcc, channel)
		return
	end
	LogInfo("AgentService - on ready channel address - user: '%s', vcc: '%s', channel: '%s'", user, vcc, channel)
	return GetAgentClient():ActivateAgentAddress(user, vcc, channel)
end

function AssignAgentAddress (user, vcc, channel)
	if not user or not vcc or not channel then
		LogError("Invalid data, Assign channel address - user: '%s', vcc: '%s', channel: '%s'", user, vcc, channel)
		return
	end
	LogInfo("AgentService - Assign channel address - user: '%s', vcc: '%s', channel: '%s'", user, vcc, channel)
	return GetAgentClient():AssignAgentAddress(user, vcc, channel)
end

--[[

]]--
function GetAgentsStatuses (agentList, vcc)
	--LogInfo("AgentService - GetAgentsStatuses - vcc: '%s', agentList: '%s'", vcc, json.encode(agentList))

	local queryTable = {}
	for k, v in pairs(agentList) do
		local simpleUserId = utils.SeparateVCC(v.Id)
		table.insert(queryTable, simpleUserId)
	end
	--LogInfo("AgentService - GetAgentsStatuses - vcc: '%s', agentList: '%s'", vcc, json.encode(queryTable))

	local agentStatus = GetAgentClient():GetAgentsStatus(vcc, queryTable)
	--LogInfo("AgentService - GetAgentsStatuses - vcc: '%s', agentStatus: '%s'", vcc, json.encode(agentStatus))
	return agentStatus
end

--[[

]]--
function GetAgentStatus (fullUserId)
	local simpleUserId, vccUser = utils.SeparateVCC(fullUserId)
	vccUser = vccUser or inconcert.authUser.virtualContactCenter
	local agents = {}
	table.insert(agents, simpleUserId)
	local agentsStatuses = GetAgentClient():GetAgentsStatus(vccUser, agents) or {}
	--LogDebug("-------> agentsStatuses: %s", json.encode(agentsStatuses))
	local agentStatus = agentsStatuses[fullUserId]
	--LogInfo("[AGENTSERVICE:GetAgentStatus] Agent '%s' Status: '%s'", fullUserId, agentStatus)
	return agentStatus
end

---
-- See documentation in lua_agentserviceclient
function GetAgentsToPassInteraction (interaction_id, interaction_type, campaign_name, vcc, exclude_agent_name)
	return GetAgentClient():GetAgentsToPassInteraction(interaction_id, interaction_type, campaign_name, vcc, exclude_agent_name)
end

--[[
 
]]--
function IsAgentLoggedIn (user, vcc)
	return GetAgentClient():IsAgentLoggedIn(user, vcc)
end

function AddAgentToCampaign (user, campaign, vcc, role)
	return GetAgentClient():AddAgentToCampaign(user, campaign, vcc, role)
end

function RemoveAgentFromCampaign(user, campaign, vcc)
	return GetAgentClient():RemoveAgentFromCampaign(user, campaign, vcc)
end

function DeleteCampaign (campaign, vcc)
	return GetAgentClient():DeleteCampaign(campaign, vcc)
end

---
-- Checks if a given agent can take an interaction or not. This is only used to know beforehand if we can proceed
-- with the (heavy) process of interaction assignment (see issue #5402).
-- The request_id is optional (but please, do provide a valid one).
--
function CanTakeInteraction (agent, vcc, interaction_type, interaction_id, campaign, request_id)
	--LogDebug("[AGENTSERVICE:CanTakeInteraction] agent '%s' vcc: '%s' interaction_type: '%s' interaction_id: '%s' campaign: '%s' request_id: '%s'.", agent, vcc, interaction_type, interaction_id, campaign, request_id)
	return GetAgentClient():CanTakeInteraction(agent, vcc, interaction_type, interaction_id, campaign, request_id)
end

--[[
	Return the available agents.
	@vcc        The vcc to retrieve agents
	@campign    The campaign to check if have agents
	@channel    The channels to check if have available slots (optional).
	@idAttLevel The attention level to check. i.e. if user belongs to it. If channel parameter not set then it will be discarded (optional).
	@return     Returns a table like this: {"result":"OK","availableAgents": numberOfAvailableAgents}
]]--
function GetAvailableAgents(vcc, campaign, channel, idAttLevel)
	local response
	local request_str
	local request = {}
	
	if not vcc or not campaign  then
		LogError("Invalid data params, On GetAvailableAgentsSlots. Invalid vcc or campaign: vcc: '%s', campaign: '%s'", vcc, campaign)
		return false, "Invalid params"
	end

	request.channels = {}
	if channel ~= nil and channel ~= '' then
		local attLevelValue = idAttLevel and idAttLevel or ""
		request.channels[channel] = {["attentionLevel"] = attLevelValue}
	end

	request.loadSkills    = false
	request.campaignAtVCC = string.lower(campaign .. "@" .. vcc)
	request_str           = json.encode(request)

	LogInfo("[AGENTSERVICE:GetAvailableAgents] Doing request: %s", request_str)
	response = GetAgentClient():GetAvailableSlots(request_str)

	if not response.result or response.result ~= "OK" then
		local reason = response.reason and response.reason or "Internal server error"
		return false, reason
	else
		local availableAgents = 0

		if next(response.availableAgents) ~= nil then
			for _, agent in pairs(response.availableAgents) do
				local isAvailable = false
				local avblAgent   = agent.agentAtVCC

				-- If request for channels
				if channel ~= nil and channel ~= '' then
					if next(agent.channels) ~= nil then
						-- Check available channels slots & attLevel
						for id, chData in pairs(agent.channels) do
							-- Channel Info
							if id == channel then
								if chData.slots > 0 then
									-- If request for attLevel
									if idAttLevel ~= nil and idAttLevel ~= '' then
										isAvailable = chData.attentionLevel
									else
										isAvailable = true
									end
								end
							end
						end
					end
				else
					isAvailable = true
				end

				if isAvailable then 
					availableAgents = availableAgents + 1;
				end
			end
		end

		return true, availableAgents
	end
end
