module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local apiQnAMaker = require "qna_maker" -- rocka

m_htmlRender = nil
m_appLocation = nil

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/agent/qna_maker/runtime/get_answer_info/

	Respuesta:
		Status:200 OK
		[]
]]--
local function GetAnswerInfo (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[AGENT.QNA_MAKER:GetAnswerInfo] User '%s' Retrieving QnA Maker Answer Info. VCC: '%s'.", userId, vcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[AGENT.QNA_MAKER:GetAnswerInfo] Error Retrieving QnA Maker Answer for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	local success, response = apiQnAMaker.GetAnswerPostInfo(vcc, info)
	if not success then
		web.status = 400
		LogError("[AGENT.QNA_MAKER:GetAnswerInfo] Error Retrieving QnA Maker Answer for vcc '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

local function GetAnswerInfoV2 (web) -- fede
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[AGENT.QNA_MAKER:GetAnswerInfoV2] User '%s' Retrieving QnA Maker Answer Info. VCC: '%s'.", userId, vcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[AGENT.QNA_MAKER:GetAnswerInfoV2] Error Retrieving QnA Maker Answer for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	local success, response = apiQnAMaker.GetAnswerPostInfoV2(vcc, info)
	if not success then
		web.status = 400
		LogError("[AGENT.QNA_MAKER:GetAnswerInfoV2] Error Retrieving QnA Maker Answer for vcc '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/agent/qna_maker/runtime/get_answer/

	Metodo: POST
		{
		  "question": "Emmita2"
		}
	Respuesta:
		Status:200 OK
		[]
]]--
local function GenerateAnswer (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[AGENT.QNA_MAKER:GenerateAnswer] User '%s' Retrieving QnA Maker Answer. VCC: '%s'.", userId, vcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[AGENT.QNA_MAKER:GenerateAnswer] Error Retrieving QnA Maker Answer for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	local success, response = apiQnAMaker.GenerateAnswer(vcc, info)
	if not success then
		web.status = 400
		LogError("[AGENT.QNA_MAKER:GenerateAnswer] Error Retrieving QnA Maker Answer for vcc '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

local function GenerateAnswerV2 (web) -- fede
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[AGENT.QNA_MAKER:GenerateAnswerV2] User '%s' Retrieving QnA Maker Answer. VCC: '%s'.", userId, vcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[AGENT.QNA_MAKER:GenerateAnswerV2] Error Retrieving QnA Maker Answer for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	local success, response = apiQnAMaker.GenerateAnswerV2(vcc, info)
	if not success then
		web.status = 400
		LogError("[AGENT.QNA_MAKER:GenerateAnswerV2] Error Retrieving QnA Maker Answer for vcc '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

--[[
****************************************************************
	MAPEO DE URLS
****************************************************************
]]--

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply

	local function HasPermission (fn)
		return replyFn .. LogRequest .. function(web, ...)
			-- 1 - Valido si el VCC tiene soporta QnA Maker
			local vcc = inconcert.authUser.virtualContactCenter
			local _, qnaMakerTotalSubscriptions = apiQnAMaker.LoadVCCSubscriptions(vcc)
			local _, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc) -- fede
			if qnaMakerTotalSubscriptions == 0 and qnaMakerTotalSubscriptionsV2 == 0 then -- fede
				LogError("[AGENT.QNA_MAKER:LoadVccConfig] VCC '%s' has no assigned QnA Maker Subscriptions keys.", vcc)
				web.status = "403"
				return json.encode({result = "forbidden"})
			end
			-- 2 - TODO: Validar si la cuenta de la campana soporta ??
			-- 3 - Ok 
			return fn(web, ...)
		end
	end

	local p = prefix
	-- RUNTIME
	module:dispatch_post(HasPermission(GetAnswerInfo) , p .. "/runtime/get_answer_info/")
	module:dispatch_post(HasPermission(GenerateAnswer), p .. "/runtime/generate_answer/")
	module:dispatch_post(HasPermission(GenerateAnswerV2), p .. "/runtime/generate_answer_v2/") -- fede

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
