local ir     = require "lua_iragentlink"
local utils  = require "shared.apps.utils"
local cosmo  = require "cosmo"
local models = require("apps.agent.model"):GetModels()

local m_moduleLocation = select(2, ...)

module(..., package.seeall, inconcert.orbit.application.Decorate)

--[[
	Sends a system email using a cluster-wide mail account via IR/EmailProxy
--]]
local function SendMessage (to, cc, bcc, subject, body, attachments, content_type)
	--DEPRECATED
	if true then
		return false
	end
	if not to then
		LogInfo("[AGENT.MAIL:SendMessage] Failure sending Isolated Mail. Mail destination is empty. TO: '%s'.", to)
		return false
	end
	local mailConfig = inconcert.environment.apps.agent.mailAccount

	local smtpUrl = string.format("%s://%s", mailConfig.secureConnection == "1" and "smtps" or "smtp", mailConfig.server)
	local smtpPort = "0"
	LogInfo("Sending Isolated Mail from: '%s', \nto: '%s', \ncc: '%s', \nbcc: '%s', \nsubject:'%s'\nbody:\n%s", mailConfig.address, to, cc, bcc, subject, body)

	local irLink = ir.IRAgentLink( inconcert.environment.apps.agent.cluster )
	local res = irLink:MailSendIsolated(
		mailConfig.user, 
		mailConfig.password, 
		smtpUrl, 
		smtpPort,
		mailConfig.secureConnection,
		mailConfig.encryption or "0",
		mailConfig.authentication or "0",
		mailConfig.name,
		mailConfig.address, 
		to, 
		subject, 
		body,
		(attachments and json.encode(attachments)) or "",
		cc or "",
		bcc or "",
		content_type
	)
	if res then
		LogInfo("[AGENT.MAIL:SendMessage] Successfully sending Isolated Mail.")
	else
		LogWarning("[AGENT.MAIL:SendMessage] Failure sending Isolated Mail.")
	end
	return res
end

function SendNewUserWelcomeMessage (userDestination)
	LogInfo("Sending welcome message to user %s from domain %s", userDestination.Id, userDestination.VCC)

	local body = cosmo.fill(
	[[
Hi, your i6 domain '$new_domain' is active! Please use the following credentials to login and create your attention groups:
<br>
<br>
<br>
	User:     <strong>$new_user_id</strong>
<br>
	$new_user_pwd
<br>
<br>
Please click to download the application: <a href=http://i6.inconcertcc.com/inconcert/apps/installer/pages/download.html>Download</a>
<br>
<br>
Or paste this link in your web browser: http://i6.inconcertcc.com/inconcert/apps/installer/pages/download.html
<br>
<br>
Thanks,<br>
<strong>The inConcert Team</strong>
<br>
	
	]], 
	{
		new_user_id = userDestination.Id .. "@" .. userDestination.VCC,
		new_user_pwd = userDestination.plainPassword and "Password: <strong>"..userDestination.plainPassword.plain.."</strong>" or "Talk with you administrator to obtain you password",
		new_domain = userDestination.VCC
	})
	local subject = cosmo.fill([[Your i6 registration for '$domain' is active]], {domain = userDestination.VCC})
	return SendMessage(userDestination.Mail, nil, nil, subject, body)
end

function SendNewUserAndInviteMessage (fromName, groupName, userDestination, customMessage)
	local userId = userDestination.Id .. "@" .. userDestination.VCC
	LogInfo("New user '%s' and inviting '%s' from user '%s' to group '%s'", userId, userDestination.Mail, fromName, groupName)

	if customMessage and #customMessage > 0 then
		customMessage = '<br><br>' .. '<i>' .. customMessage .. '</i>'
	end
	local body = cosmo.fill(
	[[
Hi, <strong>$inviting_user</strong> has invited you to join the group <strong>$invited_group</strong>!
<br>
<br>
<strong>Credentials</strong>
<br>
	User:     <strong>$inviting_new_user_id</strong>
<br>
$inviting_new_user_pwd
$user_invitation_message
<br>
<br>
Please click to download the application: <a href=http://i6.inconcertcc.com/inconcert/apps/installer/pages/download.html>Download</a>
<br>
<br>
Or paste this link in your web browser: http://i6.inconcertcc.com/inconcert/apps/installer/pages/download.html
<br>
<br>
Thanks,<br>
<strong>The inConcert Team</strong>
<br>

	]], 
	{
		inviting_user           = fromName, 
		user_invitation_message = customMessage, 
		invited_group           = groupName,
		inviting_new_user_id    = userId,
		inviting_new_user_pwd   = userDestination.plainPassword and "	Password: <strong>"..userDestination.plainPassword.plain.."</strong>" or "Talk with your administrator to obtain you password",
	})
	local subject = cosmo.fill([[You've been invited to take part of the group $invited_group]], {invited_group = groupName})
	return SendMessage(userDestination.Mail, nil, nil, subject, body)
end

function Print (web, htmlRender, interactionId, currentEvent)
	local userId = utils.GetCurrentLoggedUser()

	LogInfo("[MAIL:Print] User '%s' printing interaction '%s', event '%s'.", userId, interactionId, currentEvent)

	local _T, _L = htmlRender.extension.makeTextTranslator()
	local isAllowedPrint = "ALLOWED"

	local interaction = models.Interactions.GetInteraction(interactionId) or {}
	if utils.TableIsEmpty(interaction) then
		interaction = models.Interactions.GetFromArchive(interactionId) or {}
		if utils.TableIsEmpty(interaction) then
			LogError("[MAIL:Print] '%s' Interaction does not exist.", interactionId)
			isAllowedPrint = "NOT ALLOWED"
		end
	end

	if isAllowedPrint == "ALLOWED" then
		local agentInteractionsController = require "apps.agent.controllers.interactions"
		local hasPermissionInInteractionApp = agentInteractionsController.HasPermission("PRINT", interaction.campaign, "interaction")
		local hasPermissionInSupervisorApp = agentInteractionsController.HasPermission("PRINT", interaction.campaign, "supervisor")

		if not hasPermissionInInteractionApp and not hasPermissionInSupervisorApp then
			LogError("[MAIL:Print] You no has permission for mail print. App Interaction permission: '%s', App Supervisor permission: '%s.", hasPermissionInInteractionApp, hasPermissionInSupervisorApp)
			isAllowedPrint = "NOT ALLOWED"
		end
	end

	local mailEvent
	for eventIndex, event in ipairs(interaction.events or {}) do
		if tonumber(currentEvent) == tonumber(eventIndex) then
			mailEvent = event
			break
		end
	end
	if not mailEvent then
		LogError("[MAIL:Print] '%s' Event does not exist on '%s' interaction.", currentEvent, interactionId)
		isAllowedPrint = "NOT ALLOWED"
	end

	local ReplyString = function(value)
		return type(value) == "string" and value or ""
	end

	local CanPrint = function()
		return ReplyString(isAllowedPrint)
	end

	local GetEventDate = function()
		if not mailEvent then
			return ReplyString()
		end
		local formatted_date = utils.ConvertUTCDateToDate(mailEvent.timestampDouble, nil, "%Y/%m/%d %H:%M:%S", true)
		return ReplyString(formatted_date)
	end

	--[[
		Esta funcion obtiene los mails de una tabla y los retorna en un string, separados por ;
	]]--
	local ParseMailsAddresses = function(mails)
		if type(mails) ~= "table" then
			return ""
		end
		local parsed = ""
		local i = 0
		for mail in pairs(mails) do
			i = i + 1
			if i > 1 then
				parsed = parsed .. "; "
			end
			parsed = parsed .. mail
		end
		return parsed
	end

	--[[
		Esta funcion obtiene el FROM a imprimir.
		Si el creador del evento es el AGENT, muestra el id del mismo + la direccion de mail de la cuenta.
		Si el creador del evento es el CONTACT:
			1 - Si el evento del mail, tiene un contacto en el attr name.
				1.1 - Si el from es un string, concatena el nombre del contacto + la direccion de mail.
				1.2 - Si el from es una tabla, obtiene todos mails de la misma.
			2 - Si el evento del mail, NO tiene un contacto en al attr name, lo manda derecho al string.
	]]--
	local GetEventFrom = function()
		if not mailEvent then
			return ReplyString()
		end

		local from = mailEvent.from

		if mailEvent.creator == "agent" then
			local creatorWithoutVCC = utils.SeparateVCC(mailEvent.agent)
			from = creatorWithoutVCC .. " [" .. tostring(from) .. "]"

		elseif mailEvent.creator == "contact" then
			if mailEvent.name then
				if type(from) == "string" then
					from = mailEvent.name .. " [" .. from .. "]"
				elseif type(from) == "table" then
					from = ParseMailsAddresses(from)
				end
			end
		end

		return ReplyString(from)
	end

	--[[
		Esta funcion obtiene el TO a imprimir.
		Si es un string lo manda derecho.
		Si es una tabla, obtiene todos mails de la misma.
	]]--
	local GetEventTo = function()
		if not mailEvent then
			return ReplyString()
		end
		local to = mailEvent.to
		if type(to) == "table" then
			to = ParseMailsAddresses(to)
		end
		return ReplyString(to)
	end

	--[[
		Esta funcion obtiene el CC a imprimir
	]]--
	local GetEventCC = function()
		if not mailEvent then
			return ReplyString()
		end
		local cc = mailEvent.cc
		return ReplyString(cc)
	end

	--[[
		Esta funcion obtiene el BCC a imprimir
	]]--
	local GetEventBCC = function()
		if not mailEvent then
			return ReplyString()
		end
		local bcc = mailEvent.bcc
		return ReplyString(bcc)
	end

	--[[
		Esta funcion obtiene el SUBJECT a imprimir
	]]--
	local GetEventSubject = function()
		if not mailEvent then
			return ReplyString()
		end
		local subject = mailEvent.subject
		return ReplyString(subject)
	end

	--[[
		Esta funcion obtiene el BODY a imprimir, primero lo busca en el contentType "text/html" luego en "text/plain"

		Luego retorna un string con 3 campos separados con un separador ";".
		Estos campos nos indica si la impresion se puede realizar correctamente o no.
		El separador de campos es el caracter ";" y los 3 campos que retorna son los siguientes:

			1 - El primer campo retorna el valor 1 o 0. El valor 1 nos indica que la impresion se puede realizar.
			2 - El segundo campo nos retorna el valor del contentType del html a imprimir, y sus valores posibles son: "text/html" y "text/plain"
			3 - El tercer campo contiene el html a imprimir.

			Ej:
				ok = "1;text/plain;EL TEXTO A IMPRIMIR"
				error = "0;;"

		Nota:
			Esto se envia asi hacia el luapage "print.html" como un string con separadores, en lugar de una tabla, porque el cgilua, no permite enviar una tabla, sino un string.
	]]--
	local GetEventBody = function()

		local success = false
		local html, contentType

		local returnBody = function()
			local separator = ";"
			if not success then
				return "0" .. separator .. separator
			end
			return "1" .. separator .. contentType .. separator .. html
		end

		if CanPrint() ~= "ALLOWED" then
			-- No permitido
			return returnBody()
		end

		if not mailEvent then
			isAllowedPrint = "NOT ALLOWED"
			return ReplyString()
		end

		local function GetBodyPart(body, content_type)
			local content
			for _, part in ipairs(body or {}) do
				if part.type and part.type:lower() == content_type then
					LogInfo("[MAIL:Print] Retrieving html body part content_type '%s' in interaction '%s' event '%s'.", content_type, interactionId, currentEvent)
					content = part.content
					break
				end
			end
			return content
		end

		-- Obtengo el contenido, primero busco con contenType = "text/html", despues contenType = "text/plain"
		for _, content_type in ipairs({"text/html", "text/plain"}) do
			local content = GetBodyPart(mailEvent.body, content_type)
			if content then
				success = true
				html = content
				contentType = content_type
				break
			end
		end

		if not success then
			LogError("[MAIL:Print] Event '%s' does not exist in '%s' interaction.", currentEvent, interactionId)
			isAllowedPrint = "NOT ALLOWED"
		end

		return returnBody()
	end

	local env = {
		_T = _T,
		_L = _L,
		m_utils = utils,
		_MAIL = {
			CanPrint   = CanPrint,
			GetDate    = GetEventDate,
			GetFrom    = GetEventFrom,
			GetTo      = GetEventTo,
			GetCC      = GetEventCC,
			GetBCC     = GetEventBCC,
			GetSubject = GetEventSubject,
			GetBody    = GetEventBody
		}
	}
	return htmlRender:RenderPageLP("pages/print.html", web, env)
end


