module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_cachedJsonCountries
local m_cachedCountries
local m_datasourceZ

function GetName ()
	return "Countries"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function MakeIndexes (countries)
	local countriesAreas = {}
	for countryCode, countryData in pairs(countries) do
		--descarto los tombstomes
		if countryData and countryData.Id then
			--LogDebug("[AGENT.COUNTRIES:MakeIndexes] --> countryCode: '%s', countryName: '%s'", countryCode, countryData.Name)
			local country = {
				code = countryCode,
				name = countryData.Name,
				areasByCode = {},
				areasByName = {}
			}
			for areaCode, areaData in pairs(countryData.Areas or {}) do
				local areaName = areaData.name or areaData.id or "NO_NAME"
				country.areasByCode[areaCode] = country.areasByCode[areaCode] or {}
				table.insert(country.areasByCode[areaCode], areaName)
				country.areasByName[areaName] = country.areasByName[areaName] or {}
				table.insert(country.areasByName[areaName], areaCode)
				--LogDebug("[AGENT.COUNTRIES:MakeIndexes]    --> countryCode: '%s', areaName: '%s'", countryCode, areaName)
			end
			countriesAreas[countryCode] = country
		end
	end
	local jsonCountriesAreas = utils.JSON_Encode(countriesAreas)
	return countriesAreas, jsonCountriesAreas
end

function FindAll ()
	if not m_cachedCountries then
		--LogDebug("[AGENT.COUNTRIES:FindAll] A--> FROM DATABASE")
		local countries = utils.GetAllColumns {
			keyspace = "Provisioning",
			columnFamily = "Countries",
			startKey = "",
			endKey = "",
			rowCount = 10000
		}
		m_cachedCountries, m_cachedJsonCountries = MakeIndexes(countries)
	--else
	--	LogDebug("[AGENT.COUNTRIES:FindAll] b--> FROM CACHE")
	end
	--LogDebug("[AGENT.COUNTRIES:FindAll] --> m_cachedJsonCountries: %s", m_cachedJsonCountries)
	return m_cachedCountries, m_cachedJsonCountries
end
