module(..., package.seeall)

local m_datasource

function GetName()
	return "Settings"
end
	
function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end


function Get(user, vcc)
	local settings = {}
	local userid = user .. "@" .. vcc
	
	local keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	
	local userSettings = keyspace:GetSliceRange(userid, 
										   "Settings", 
										   "", { 
												column_names = {},
												slice_range = { 
															start = "", 
															finish = "", 
															reversed = true, 
															count = 200}
												}
											) 
	keyspace:Close()
	if (userSettings) then
		for k, v in ipairs(userSettings) do
			settings[v.name] = v.value
		end
	else
		return nil
	end
	return settings
end

--Saves settings for userid
function Save(user, vcc, settings)	
	local userid = user .. "@" .. vcc
	
	local keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	
	for k, v in pairs(settings) do
		if (type(v) ~= "table") then
			keyspace:InsertColumn(userid, 'Settings', k, tostring(v))
		else
			LogError("Only basic types allowed when saving settings, value %s was a table", k)
		end
	end
	keyspace:Close()
	return true
end

--Deletes the settings for the user
function Delete(user, vcc)
	local userid = user .. "@" .. vcc
	
	local keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	if (keyspace == nil) then
		error("Agent keyspace not found trying to delete user settings")
	end
	
	keyspace:GetSliceRange(userid, 
						   "Settings", 
						   "", { 
								column_names = {},
								slice_range = { 
											start = "", 
											finish = "", 
											reversed = true, 
											count = 200}
								}
							) 
    keyspace:Close()
end

