module(..., package.seeall)

local m_datasource

function GetName()
	return "TemporaryFiles"
end
	
function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

function GetFile(user, vcc, fileId)
	LogDebug("Retrieving temporary file %s from user %s@%s", fileId, user, vcc)
	local userAtVCC = user .. "@" .. vcc
	local keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	local column = keyspace:GetColumn(userAtVCC, "TemporaryFiles", fileId)
	if column then
		return column.value
	end
end

function SaveFile(user, vcc, fileId, blob, size)
	LogDebug("Saving temporary file %s with size %s from user %s@%s", fileId, size or tostring(#blob), user, vcc)
	local userAtVCC = user .. "@" .. vcc
	local keyspace = m_datasource.cassandra:CassandraConnection("Agent")
	keyspace:InsertColumnWithSize(userAtVCC, 'TemporaryFiles', fileId, blob, size or #blob)
	keyspace:Close()
	return true
end

--Deletes a particular fileId
function Delete(user, vcc, fileId, keyspace)
	LogDebug("Deleting file %s from user %s@%s", fileId, user, vcc)
	local userAtVCC = user .. "@" .. vcc
	local releaseKS = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		releaseKS = true
	end
	keyspace:RemoveColumn(userAtVCC, 'TemporaryFiles', fileId)
	if releaseKS then
		keyspace:Close()
	end
	return true
end

function DeleteAll (user, vcc, keyspace)
	LogDebug("Deleting all temporary files for user %s@%s", user, vcc)
	local userAtVCC = user .. "@" .. vcc

	local releaseKS = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		releaseKS = true
	end

	local filesColumns = keyspace:GetSliceRange(userAtVCC, "TemporaryFiles", "", {
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = true,
				count = 200
			}
		}
	)
	if filesColumns then
		for k, v in ipairs(filesColumns) do
			Delete(user, vcc, v.name, keyspace)
		end
	end

	if releaseKS then
		keyspace:Close()
	end

	return true
end

