module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource
local m_helpers = require "helpers"

function GetName ()
	return "VCCs"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function Get (vcc, keyspace, columns, skipLicense)
	assert(vcc)
	local vccInfo = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCs",
		key = vcc,
		columns = columns or {}
	} or {}
	if not skipLicense and m_helpers.utils.HasLicenseSystem() then
		local options = utils.Get_m_helpers_agentservice_options()
		vccInfo = m_helpers.agentservice.GetVccWithLicenseInfo(options, vccInfo)
	end
	return vccInfo
end

local function LoadSpecificsAccounts ( accounts, accountType, accountId )
	if not accounts then
		return
	end

	if type( accounts ) == "string" then
		accounts = json.decode( accounts )
	end

	if accountType then
		accountType = accountType:upper()
		local accountsByType = accounts[ accountType ]
		if accountId then
			accountId = accountId:upper()
			return accountsByType and accountsByType[ accountId ]
		else
			return accountsByType
		end
	else
		return accounts
	end
end

function GetAccounts ( vcc, accountType, accountId, keyspace )
	local accounts = utils.GetColumnValue {
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCs",
		key          = vcc,
		column       = "Accounts"
	}

	return LoadSpecificsAccounts( accounts, accountType, accountId )
end

function GetAllVccs(keyspace)
	local vccs = utils.GetAllColumns {
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCs",
		startKey = "",
		endKey = ""
	} or {}
	if m_helpers.utils.HasLicenseSystem() then
		local options = utils.Get_m_helpers_agentservice_options()
		for vccId, vcc in pairs(vccs) do
			vcc = m_helpers.agentservice.GetVccWithLicenseInfo(options, vcc)
		end
	end
	return vccs
end