/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}
 
(function () {

	"use strict";

	/**
		Class AbandonedInteractionsPanel.
		Subclass of Panel used for abandoned interactions.
	*/
	function AbandonedInteractionsPanel (id, parent_panel)
	{
		this.m_parent_panel = parent_panel;
		this.m_category = "abandoned";

		AbandonedInteractionsPanel.uber.constructor.call(this, {id: id, name: "<%=_T('Abandoned Conversations')%>"});

		let iframes_container_id, css_classes;
		if (inConcert.app.is_enable_docked_windows_interactions()) {
			// TODO: #9778 - CANY - Docked. Revisar !!
			iframes_container_id = "iFramesWindowsDailogsContainerForInteractionAppAbandonedPanel";
			css_classes = css_classes;
		}

		this.abandoned_search = new inConcert.GlobalSearch(
		{
			from_app              : "interaction",
			panel_name            : this.m_category,
			panel_name_text       : "<%=_T('Abandoned_Int')%>",
			queryBuilderObj       : parent_panel.queryBuilderObj,
			container_id          : "int_panel_abandoned",
			navigation            : this.m_parent_panel.get_navigation(), // la navegacion de conversations panels
			custom_search_url     : "/interactions/search_abandoned/",
			filter_by_campaign    : { type: "ASSIGNED_CAMPAIGNS" },
			callbacks             : {}, // TODO: implementar ?
			method                : "searchAbandonedInteractions",
			has_exact_match       : false,
			additional_columns    : {},
			iframes_container_id  : iframes_container_id, // Docked
			css_classes           : css_classes, // Docked
			facets                : {
				address : true,
				campaign: true, 
				account : true,
				contact_id : true,
				contact_name : true,
				conversation_type: true,
				text : true,
				date_from : true,
				date_to : true,
				ticket : true
			},
			interactions_per_page : 25,
			last_control          : true,
			agent_filter          : { agents : [inConcert.app.getCompleteId()], no_agent : true },
			messages              : {
				waiting: "<%=_T('Abandoned conversations search, please wait')%>"
			},
			permissions : { 
				global_search: (this.permissions || {}).abandoned // El permiso contra el panel de abandoned (de la navegacion, segun app interaction)
			}
		});
	}

	inConcert.Utils.inherit(AbandonedInteractionsPanel, inConcert.InteractionPanel);

	if (typeof inConcert.AbandonedInteractionsPanel !== "function") {
		inConcert.AbandonedInteractionsPanel = AbandonedInteractionsPanel;
	}

	AbandonedInteractionsPanel.prototype.reset = function() {
		this.loadAllInteractions();
	};

	AbandonedInteractionsPanel.prototype.show = function()
	{
		var new_search = true;
		this.apply_filter(new_search);
	};

	AbandonedInteractionsPanel.prototype.apply_filter = function(is_new_search, on_end = $.noop)
	{
		this.abandoned_search.search(is_new_search, on_end);
	};

	AbandonedInteractionsPanel.prototype.interactionBelongsToPanel = function(interaction) {
		return interaction.state === "ABANDONED";
	};

	AbandonedInteractionsPanel.prototype.updateInteractions = function(/*interactions*/) {
		return 0;
	};

	AbandonedInteractionsPanel.prototype.updateAssignedCounter = function(counter) {
		if ( this.m_parent_panel ) {
			this.m_parent_panel.update_counters( {
				abandoned : { agent : counter }
			});
		}
	};
}());
