/*jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50,
  quotmark: false
*/

/* global inConcert: true, log4javascript, console */

/*
Logger
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	/**
	Custom json layout that allows to log objects in addition to strings.
	*/
	function MyJsonLayout() {
		MyJsonLayout.uber.constructor.call(this);
	}

	inConcert.Utils.inherit(MyJsonLayout, log4javascript.Layout);

	MyJsonLayout.prototype.format = function(loggingEvent) {
		var logObject = {
			timestamp : loggingEvent.timeStampInSeconds,
			level : loggingEvent.level.name,
			data : {}
		};
		for (var i = 0; i < loggingEvent.messages.length; i++) {
			switch (typeof(loggingEvent.messages[i])) {
				case "string":
					if(!logObject.data.message) {
						logObject.data.message = "";
					}
					logObject.data.message += loggingEvent.messages[i] + "\n";
					break;
				case "object":
					for (var property in loggingEvent.messages[i]) {
						if (loggingEvent.messages[i].hasOwnProperty(property)) {
							logObject.data[property] = loggingEvent.messages[i][property];
						}
					}
				break;
			}
		}
		return JSON.stringify(logObject);
	};

	MyJsonLayout.prototype.ignoresThrowable = function() {
		return true;
	};

	MyJsonLayout.prototype.toString = function() {
		return "MyJsonLayout";
	};

	function CustomAppender ()
	{
		CustomAppender.uber.constructor.call(this);
	}

	inConcert.Utils.inherit(CustomAppender, log4javascript.Appender);

	CustomAppender.prototype.layout = new MyJsonLayout();

	CustomAppender.prototype.append = function(loggingEvent) {
		var formattedMessage = this.getLayout().format(loggingEvent);
		if (this.getLayout().ignoresThrowable()) {
			formattedMessage += loggingEvent.getThrowableStrRep();
		}
		switch (loggingEvent.level.name){
			case "DEBUG":
				console.debug(formattedMessage);
				break;
			case "INFO":
				console.info(formattedMessage);
				break;
			case "WARN":
				console.warn(formattedMessage);
				break;
			case "ERROR":
				console.error(formattedMessage);
				break;
			default:
				break;
		}

	};

	CustomAppender.prototype.toString = function() {
		return "CustomAppender";
	};
	
	CustomAppender.prototype.close = function() {
		//this.m_fileStream.close();
	};

	function GetLogger() {

		log4javascript.logLog.setQuietMode(true);

		var logger = log4javascript.getLogger();
		// TODO esto se deberia poder pisar/cambiar con la configuracion inicial que se le baja al usuario, tanto si se 
		// usa el ajax appender con su correspondiente URL como el threshold level de logging

		var appender = new CustomAppender();
		appender.setThreshold(log4javascript.Level.DEBUG);
		logger.addAppender(appender);

		logger.action = function(data) {
			logger.info(data);
		};

		logger.guiAction = function(data, interaction) {
			if (interaction === null) {
				interaction = inConcert.app.m_interactionManager.m_interactions[data.window_id];
			}
			if (interaction !== undefined) {
				data.id = interaction.id;
				data.type = interaction.type;
				data.campaign = interaction.campaign;
			} else {
				data.id = null;
				data.type = null;
				data.campaign = null;
			}
			data.action_type = "gui";
			logger.info(data);
		};

		logger.enableAjaxLog = function(base_domain_url) {
			// Log errors in our backend as soon as they occur.
			var url_log_endpoint = base_domain_url + "/inconcert/apps/logstorage/log_error";
			var ajaxAppender = new log4javascript.AjaxAppender(url_log_endpoint);
			ajaxAppender.setThreshold(log4javascript.Level.ERROR);
			ajaxAppender.setLayout( new MyJsonLayout() );
			this.addAppender(ajaxAppender);
		};

		return logger;
	}

	if (typeof inConcert.logger !== 'object') {
		//the storage is a singleton, so it's published already initialized
		inConcert.logger = GetLogger();
	}
}());
