/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:false, -W014, esversion: 11 */

/* global inConcert, RenderCaptcha, RemoveCaptchaJS */

/*
Authentication
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//constructor
	function Authenticator() {
		this.m_loginToken = null;
		this.m_applicationName = "agent";
		this.m_sessionId = null;
		this.m_user = null;
		this.m_password = null;
		this.m_urlBase = "/inconcert/apps/";
	}

	Authenticator.prototype.parseUser = function(user /*optional*/)
	{
		let response = {success: null, userName: null, VCC: null, code_err: null};

		user = user || this.m_user;
		const matched = user.match(/^(.+)@([^@]+)$/);

		if (!matched) {
			//invalid user non alfa
			response.success = false;
			response.code_err = -100;
			return response;
		}
		response.userName = RegExp.$1;
		response.VCC = RegExp.$2.trim();
		if (response.VCC.length === 0) {
			//invalid user syntax must be user@domain
			response.success = false;
			response.code_err = -200;
			return;
		}
		// is Ok
		response.success = true;
		response.userName = response.userName.toLowerCase();
		response.VCC = response.VCC.toLowerCase();

		return response;
	};

	//Logs out the current user invalidating the cookied token
	//TODO: change to params object
	Authenticator.prototype.logout = function(callback, callFrom) {
		var self = this;

		function doLogout(callFrom) {
			var data = {};
			if (callFrom) {
				data.callFrom = callFrom;
			}
			var url = self.m_urlBase + "agent/login/logout/";
			// Si se esta descargando la pagina, el XMLHttpRequest no funcionara en Chrome. Usamos sendBeacon en su lugar
			if(callFrom === "MainWindow-onWindowClosed" || callFrom === "MainWindow-onShutdown")
			{
				navigator.sendBeacon(url, JSON.stringify(data));
			}
			else
			{
				inConcert.invoke({
					type: 'POST',
					async: false,
					url: url,
					data: JSON.stringify(data),
					//dataType: "json",
					contentType: "application/json",
					success: function(response) {
						if(typeof(callback) === "function") {
							callback(response.kickout);
						}
					},
					error : function() {
						if(typeof(callback) === "function") {
							callback(false);
						}
					}
				});
			}
		}
		doLogout(callFrom);
	};

	//TODO: change to params object
	Authenticator.prototype.changePassword = function(user, oldPassword, newPassword, callback)
	{
		let response = this.parseUser(user);
		if(!response.success) {
			callback({code: response.code_err});
			return;
		}
		var url = this.m_urlBase + "login/changepassword/user";
		var request = {
			userName: response.userName,
			virtualContactCenter: response.VCC,
			currentPassword: oldPassword,
			newPassword: newPassword
		};
		inConcert.invoke({
			url: url,
			type: 'post',
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "json",
			success: function(data) {
				var result = {
					code: data.status === true ? 0 : -1,
					message: data.message
				};
				callback(result, data);
			},
			error: function(result, error) {
				callback(result, error);
			}
		});
	};

	Authenticator.prototype.getAuthToken = function(params)
	{
		//required params
		let callback = params.callback;
		
		const self = this;

		let response = this.parseUser();
		if(!response.success) {
			this.loginError({status: response.code_err});
			return;
		}

		let url = self.m_urlBase + "agent/login/getToken/";
		let urlParams = "username=" + encodeURIComponent(response.userName) + "&callcenter=" + response.VCC;

		inConcert.invoke({
			url: url,
			type: 'post',
			data: urlParams,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded",
			success: function(result) {
				self.m_loginToken = result.loginToken;
				self.m_encodeMode = result.encodeMode;
				self.m_info = result.info;
				self.m_userInfo = result.userInfo;
				callback(true, result);
			},
			error: function(result) {
				self.loginError(result, self);
			}
		});
	};

	/*
		Funcion que valida si se encuentra activo captcha en el vcc para el login.
	*/
	Authenticator.prototype.__get_captcha_config = function(info) {
		info = info || this.m_info;
		if (!info || !info.vcc) {
			return false;
		}
		if (!info.vcc.captcha || info.vcc.captcha.enabled !== "1") {
			return false;
		}
		if (info.must_auto_login) {
			return false;
		}
		return info.vcc.captcha;
	};

	/*
		Funcion que invoca a la funciones de render de captcha si es que el vcc tiene configurado esta opcion.
	*/
	Authenticator.prototype.login_with_captcha = function(info) {
		const self = this;
		let captcha_config = this.__get_captcha_config(info);
		if (!captcha_config) {
			// si el vcc no tiene configuracion de captcha.
			RemoveCaptchaJS();
			return false;
		}
		if (!captcha_config.publicKey) {
			// la config de captcha para el vcc tiene que tener la clave publica.
			RemoveCaptchaJS();
			this.loginError({status: -300});
			return true;
		}
		// funcion existente en "loginForm_common.js"
		RenderCaptcha(captcha_config, function(success, response) {
			if (!success) {
				// si la obtencion del captcha ha dado error.
				let err = response || {status: -301};
				self.loginError(err);
				return;
			}
			// realizamos el login con captcha
			self.loginUser({
				recaptchaToken: response
			});
		});
		return true;
	};

	//Retrieve auth token and vcc authentication info
	Authenticator.prototype.GetAuthToken = function(params) {
		//required params
		let userAtVCC = params.userAtVCC;
		//optional params
		let domain = params.domain || "";
		let callback = params.callback || $.noop;
		let mfaCallback = params.mfaCallback || $.noop;
		
		const self = this;
		this.m_user = userAtVCC.toLowerCase();
		this.m_callbackFn = callback;
		this.m_mfaCallbackFn = mfaCallback;
		this.m_urlBase = domain + "/inconcert/apps/";
		this.getAuthToken({
			callback : function(success, info) {
				self.m_callbackFn(success, info);
			}
		});
	};

	//Starts login handshaking process
	Authenticator.prototype.login = function(params) {
		//required params
		let userAtVCC = params.userAtVCC;
		let password = params.password;
		//optional params
		let domain = params.domain || "";
		let data = params.data || null;
		let autoLogin = params.autoLogin || false;
		let mustmustChangePassword = params.mustChangePassword || false;
		let newPassword = params.newPassword;
		let callback = params.callback || $.noop;
		let mfaCallback = params.mfaCallback || $.noop;
		
		const self = this;
		this.m_user = userAtVCC.toLowerCase();
		this.m_password = password;
		this.m_must_change_password = mustmustChangePassword;
		this.m_new_password = newPassword;
		this.m_callbackFn = callback;
		this.m_mfaCallbackFn = mfaCallback;
		this.m_urlBase = domain + "/inconcert/apps/";

		if(autoLogin) {
			this.getAuthToken({
				callback : function(success, info) {
					if (!success) {
						return;
					}
					info = info || {};
					// atributo utilizado para indicar que tiene que realizar un autologin, es decir que captcha no lo tengo en cuenta.
					info.must_auto_login = autoLogin;
					// invocamos para ver si realizamos el login con captcha
					if (!self.login_with_captcha(info)) {
						// este es el caso en el que el vcc no tiene captcha configurado, y realizamos el login comun.
						// tambien puede ser que el vcc si tenga configurado captcha, pero la invocacion se realizo mediante api (must_auto_login = true),
						// y se envia hacia el server el valor "webuser_with_auto_login"
						self.loginUser({
							authenticationForm : autoLogin ? "webuser_with_auto_login" : null,
							data : data
						});
					}
				}
			});
		}
		else {
			let info = params.info;
			info = info || {};
			// atributo utilizado para indicar que tiene que realizar un autologin, es decir que captcha no lo tengo en cuenta.
			info.must_auto_login = autoLogin;
			// invocamos para ver si realizamos el login con captcha
			if (!self.login_with_captcha(info)) {
				// este es el caso en el que el vcc no tiene captcha configurado, y realizamos el login comun.
				// tambien puede ser que el vcc si tenga configurado captcha, pero la invocacion se realizo mediante api (must_auto_login = true),
				// y se envia hacia el server el valor "webuser_with_auto_login"
				self.loginUser({
					authenticationForm : autoLogin ? "webuser_with_auto_login" : null,
					data : data
				});
			}
		}
	};

	Authenticator.prototype.encodeCredentials = function(params) {
		//required params
		let username = params.username.toLowerCase();
		let password = params.password;
		let token = params.token;
		
		switch(this.m_encodeMode) {
			case "plain":
				return encodeURIComponent(password);
			default:
				let textToEncode = username + password;
				let hash = inConcert.Utils.md5(inConcert.Utils.md5(textToEncode) + token);
				return hash;
		}
	};

	//Auxiliar function using in allegro integration to pre validate login
	Authenticator.prototype.validateAuthentication = function(params) {
		//optional params
		let mfaAuthData = params.mfaAuthData || null;
		let mfaFinalStepFn = params.mfaFinalStepFn || $.noop;
		let callback = params.callback || $.noop;
		
		const self = this;

		let response = this.parseUser();
		if(!response.success) {
			this.loginError({status: response.code_err});
			return;
		}

		var hash = this.encodeCredentials({
			username : response.userName,
			password : self.m_password,
			token : self.m_loginToken
		});

		let urlParams = "hash=" + hash + "&username=" + encodeURIComponent(response.userName) + "&loginToken=" + self.m_loginToken +
			"&callcenter=" + response.VCC + "&app=" + encodeURIComponent(self.m_applicationName);
		if (mfaAuthData) {
			urlParams = urlParams + "&mfa=" + encodeURIComponent(JSON.stringify(mfaAuthData));
		}

		var url = self.m_urlBase + "agent/login/validate_authentication/";
		inConcert.invoke({
			url: url,
			type: 'post',
			data: urlParams,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded",
			success: function(data) {
				if (data.mfa) {
					return self.m_mfaCallbackFn(data, mfaFinalStepFn);
				}
				if (data.status) {
					callback();
				}
				else {
					self.m_callbackFn({ code : -1, message : data.message });
				}
			},
			error: function(result) {
				self.loginError(result, self);
			}
		});
	};

	//Using a valid token and session ID tries to validate user using specified id and password
	Authenticator.prototype.loginUser = function(params)
	{
		//optional params
		let authenticationForm = params.authenticationForm || "webuser";
		let data = params.data || "{}";
		let recaptchaToken = params.recaptchaToken || null;
		let mfaAuthData = params.mfaAuthData || null;
		
		const self = this;

		let response = this.parseUser();
		if(!response.success) {
			this.loginError({status: response.code_err});
			return;
		}
		
		function mfaFinalStepFn(authData) {
			params.mfaAuthData = authData;
			self.loginUser(params);
		}

		function doLogin() {
			let hash = self.encodeCredentials({
				username : response.userName,
				password : self.m_password,
				token : self.m_loginToken
			});

			let jwtToken = inConcert.Utils.GetJwtToken(self.m_password, self.m_loginToken);

			let urlParams = "hash=" + hash +
				"&username=" + encodeURIComponent(response.userName) +
				"&loginToken=" + self.m_loginToken + 
				"&callcenter=" + response.VCC +
				"&app=" + encodeURIComponent(self.m_applicationName) +
				"&authenticationFrom=" + encodeURIComponent(authenticationForm) +
				"&jwtToken=" + jwtToken +
				"&data=" + encodeURIComponent(data);

			if(self.m_must_change_password || self.m_new_password) {
				let hashNew = inConcert.Utils.md5(response.userName.toLowerCase() + self.m_new_password);
				let jwtTokenNew = inConcert.Utils.GetJwtToken(self.m_new_password, self.m_loginToken);
				urlParams = urlParams + "&hashNew=" + hashNew + "&jwtTokenNew=" + jwtTokenNew;
			}

			// enviamos el token del captcha al server, para que este realice la validacion pertinente
			if (recaptchaToken) {
				urlParams = urlParams + "&recaptchaToken=" + encodeURIComponent(recaptchaToken);
			}
			if (mfaAuthData) {
				urlParams = urlParams + "&mfa=" + encodeURIComponent(JSON.stringify(mfaAuthData));
			}

			let url = self.m_urlBase + "agent/login/authenticate/";
			inConcert.invoke({
				url: url,
				type: 'post',
				data: urlParams,
				dataType: "json",
				contentType: "application/x-www-form-urlencoded",
				success: function(data) {
					if (data.mfa) {
						return self.m_mfaCallbackFn(data, mfaFinalStepFn);
					}
					self.authToken = data.authToken;
					if (self.m_callbackFn) {
						let result = {
							code: data.status === true ? 0 : -1,
							message: data.message,
							languageId: data.languageId,
							retry: data.retry,
							newPasswordError: data.message === "new password error" || false,
							newPasswordErrorDetails: data.additionalErrorInfo
						};
						//if has error then logout in allegro
						if (data.status) {
							self.m_callbackFn(result, data);
						}
						else {
							inConcert.AllegroAuth.logoutOnLoginError({
								callback : function() {
									self.m_callbackFn(result, data);
								}
							});
						}
					}
				},
				error: function(response) {
					//if has error then logout in allegro
					inConcert.AllegroAuth.logoutOnLoginError({
						callback : function() {
							self.loginError(response, self);
						}
					});
				}
			});
		}

		inConcert.AllegroAuth.login({
			userAtVCC : this.m_user,
			password : this.m_password,
			mfaAuthData : mfaAuthData,
			mfaFinalStepFn : mfaFinalStepFn,
			callback : function(ok, message) {
				if (ok) {
					doLogin();
				}
				else {
					self.m_callbackFn({ code : -1, message : message });
				}
			}
		});
	};

	//Handles every server-related login error and propagates the result to the requester
	//TODO hay que mover estos handler de lugar
	Authenticator.prototype.loginError = function(response) {
		var result = {};
		var self = this;
		result.code = response.status;
		switch (response.status) {
			case -100:
			case -200:
				result.message = "<%=_T('Unknown user, please specify your user as user@domain')%>";
				break;

			case 12007:
				result.message = "<%=_T('Unable to communicate with server, please verify server name.')%>";
				break;

			case 12029:
			case 0: //firefox esta enviando status cero en la desconexion
				result.message = "<%=_T('Unable to communicate with server, please check your network connection')%>";
				break;

			case 500: //los server errors, well...
				result.message = "<%=_T('An error has ocurred, please try again in a few minutes.')%>";
				break;

			case 403: //o me pisaron el login o reiniciaron el web server
				result.message = "<%=_T('Your credentials are no longer valid, please login again')%>";
				break;

		//reCaptcha errors -----------------------------------------------------
			case -300: 
				result.message = "<%=_T('Invalid reCaptcha configuration.')%>";
				break;

			case -301:
				result.message = "<%=_T('reCaptcha validation error.')%>";
				break;

			case -302:
				result.message = "<%=_T('Invalid reCaptcha configuration. Bad publick key.')%>";
				break;

			case -303:
				result.message = "<%=_T('Error intializing reCaptcha.')%>";
				break;

			case -304:
				result.message = "<%=_T('Internal reCaptcha error.')%>";
				break;

			case -305:
				result.message = "<%=_T('reCaptcha has expired.')%>";
				break;

			case -306:
				result.message = "<%=_T('reCaptcha connection error. Try again.')%>";
				break;

			case -307: 
				result.message = "<%=_T('reCaptcha needs to have an internet connection.')%>";
				break;
		//-------------- -----------------------------------------------------

			case 200:
				result.code = 0;
				result.message = "";
				break;

			default:
				result.message = "<%=_T('Server error ')%>" + (response.status || "");
		}

		if (self.m_callbackFn) {
			self.m_callbackFn(result);
		}
	};

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.InconcertAuth !== 'object') {
		inConcert.InconcertAuth = new Authenticator();
	}

}());

