/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:false, -W014, esversion: 11 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	const m_channels = {
		"CALL": {
			isSync: true
		},
		"CHAT": {
			isSync: true
		},
		"IM": {
			isSync: true
		},
		"FACEBOOK": {
			isSync: false
		},
		"INSTAGRAM": {
			isSync: false
		},
		"YOUTUBE": {
			isSync: false
		},
		"APPSTORE": {
			isSync: false
		},
		"GOOGLEPLAY": {
			isSync: false
		},
		"RCS": {
			isSync: false
		},
		"MAIL": {
			isSync: false
		},
		"SMS": {
			isSync: false
		},
		"TWITTER": {
			isSync: false
		},
		"VOICEMAIL": {
			isSync: true
		},
		"WEBCONTACT": {
			isSync: false
		},
		"WHATSAPP": {
			isSync: false
		}
	};

	// por defecto son 5 minutos
	var get_long_request_timeout_ms = function(time_min = 5) {
		return (time_min * 60) * 1000; // retorno en milisegundos
	};

	//Campaign constructor
	function Campaign(campaign, integrationConfig, app_id)
	{
		jQuery.extend(true, this, campaign);
		this.app_id = app_id || "agent"; //TODO: sacar esto y el parametro!!! cuando se elimine la parte de configuracion de campanas desde el agent.
		this.m_members = [];
		this.m_queuedInteractions = 0;

		if (integrationConfig) {
			this.m_integration = inConcert.IntegrationManager.bindApplication(integrationConfig);
		}
		this.att_levels_by_users = null;
		this.accounts_by_att_levels = null;
		this.refresh();

		this.EventsSuscribe();
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.Campaign !== 'function') {
		inConcert.Campaign = Campaign;
	}

	Campaign.prototype.EventsSuscribe = function()
	{
		var self = this;

		var campaign_id = this.Id;
		if ( !campaign_id.match( inConcert.app.idAtVCCRegExp ) ) {
			campaign_id = campaign_id + "@" + inConcert.app.getDomain();
		}

		//subscribe to added account
		var pub_ds_id = "/accounts/" + campaign_id + "/added";
		this.pubsub_account_added_handle = inConcert.app.messagebus.subscribe(
			pub_ds_id,
			function(account) {
				self.on_added_account(account);
			});

		//subscribe to updated account
		pub_ds_id = "/accounts/" + campaign_id + "/updated";
		this.pubsub_account_updated_handle = inConcert.app.messagebus.subscribe(
			pub_ds_id,
			function(account) {
				self.on_updated_account(account);
			});

		//subscribe to removed account
		pub_ds_id = "/accounts/" + campaign_id + "/removed";
		this.pubsub_account_removed_handle = inConcert.app.messagebus.subscribe(
			pub_ds_id,
			function(account_id) {
				self.on_removed_account(account_id);
			});

		//subscribe to disposition change
		pub_ds_id = "/dispositions/" + campaign_id + "/updated";
		this.pubsub_disposition_handle = inConcert.app.messagebus.subscribe(
			pub_ds_id,
			function(dispositions) {
				self.dispositionCodes = JSON.stringify(dispositions);
			});

		//subscribe to thresholds change
		var pub_th_id = "/thresholds/" + campaign_id + "/updated";
		this.pubsub_thresholds_handle = inConcert.app.messagebus.subscribe(
			pub_th_id,
			function(thresholds) {
				self.setThresholds( thresholds );
			});

		//subscribe to integration change
		var pub_cci_id = "/integration_cci/" + campaign_id + "/updated";
		this.pubsub_integration_handle = inConcert.app.messagebus.subscribe(
			pub_cci_id,
			function( integration ) {
				self.Integration = integration;
			});

		//subscribe to InteractionNotifications change
		var pub_interaction_notifications_id = "/interaction_notifications/" + campaign_id + "/updated";
		this.pubsub_interaction_notifications_handle = inConcert.app.messagebus.subscribe(
			pub_interaction_notifications_id,
			function( data ) {
				self.InteractionNotifications = data;
			});

		//subscribe to permissions change
		var pub_pm_id = "/permissions/" + campaign_id + "/updated";
		this.pubsub_permissions_handle = inConcert.app.messagebus.subscribe(
			pub_pm_id,
			function( permissions ) {
				self.setAllPermissions( permissions );
			});

		//subscribe to attention levels change
		var pub_als_id = "/update_attention_levels/" + campaign_id + "/updated";
		this.pubsub_attention_levels_handle = inConcert.app.messagebus.subscribe(
			pub_als_id,
			function(als) {
				self.AttentionLevels = {};
				self.__setAttentionLevels( als );
			});

		//subscribe to canned messages changed
		var pub_cm_id = "/canned_messages/" + campaign_id + "/updated";
		this.pubsub_canned_messages_handle = inConcert.app.messagebus.subscribe(
			pub_cm_id,
			function(differences)
			{
				$.each(differences, function(canned_id, difference)
				{
					switch(difference.type)
					{
						case "NEW":
							self.CannedMessages[canned_id] = difference.row;
						break;

						case "UPDATE":
							self.CannedMessages[canned_id] = difference.updated_row;
						break;

						case "DELETE":
							delete self.CannedMessages[canned_id];
						break;
					}
				});
			});

		//subscribe to labels changed
		var pub_lbl_id = "/update_labels/" + campaign_id + "/updated";
		this.pubsub_labels_handle = inConcert.app.messagebus.subscribe(
			pub_lbl_id,
			function(differences)
			{
				self.Labels = self.Labels || {};
				$.each(differences || {}, function(label_id, difference)
				{
					switch(difference.type)
					{
						case "NEW":
							self.Labels[label_id] = difference.row;
						break;

						case "UPDATE":
							self.Labels[label_id] = difference.updated_row;
						break;

						case "DELETE":
							delete self.Labels[label_id];
						break;
					}
				});
			});
	};

	Campaign.prototype.EventsUnSuscribe = function()
	{
		if (this.pubsub_account_added_handle) {
			//unsubscribe to added account
			inConcert.app.messagebus.unsubscribe(this.pubsub_account_added_handle);
		}
		if (this.pubsub_account_updated_handle) {
			//unsubscribe to updated account
			inConcert.app.messagebus.unsubscribe(this.pubsub_account_updated_handle);
		}
		if (this.pubsub_account_removed_handle) {
			//unsubscribe to removed account
			inConcert.app.messagebus.unsubscribe(this.pubsub_account_removed_handle);
		}
		if (this.pubsub_disposition_handle) {
			//unsubscribe to disposition change
			inConcert.app.messagebus.unsubscribe(this.pubsub_disposition_handle);
		}
		if (this.pubsub_thresholds_handle) {
			//unsubscribe to thresholds change
			inConcert.app.messagebus.unsubscribe(this.pubsub_thresholds_handle);
		}
		if (this.pubsub_canned_messages_handle) {
			//unsubscribe to canned messages change
			inConcert.app.messagebus.unsubscribe(this.pubsub_canned_messages_handle);
		}
		if (this.pubsub_labels_handle) {
			//unsubscribe to labels change
			inConcert.app.messagebus.unsubscribe(this.pubsub_labels_handle);
		}
		if (this.pubsub_integration_handle) {
			//unsubscribe to integration change
			inConcert.app.messagebus.unsubscribe(this.pubsub_integration_handle);
		}
		if (this.pubsub_interaction_notifications_handle) {
			//unsubscribe to Interaction Notifications change
			inConcert.app.messagebus.unsubscribe(this.pubsub_interaction_notifications_handle);
		}
		if (this.pubsub_permissions_handle) {
			//unsubscribe to permissions change
			inConcert.app.messagebus.unsubscribe(this.pubsub_permissions_handle);
		}
		if (this.pubsub_attention_levels_handle) {
			//unsubscribe to attention levels change change
			inConcert.app.messagebus.unsubscribe(this.pubsub_attention_levels_handle);
		}
	};

	Campaign.prototype.Dispose = function() { this.EventsUnSuscribe(); };

	Campaign.prototype.refresh = function()
	{
		this.IndexAttentionLevels();

		$.each(this.get_accounts() || [], function(_, account )
		{
			if (typeof account.Font === "string") {
				account.Font = JSON.parse(account.Font);
			}
		});
	};

	Campaign.prototype.getBaseURI = function( app_id ) {
		app_id = app_id || this.app_id;
		var apps_URIs = {
			agent: inConcert.app.getBaseUrl(),
			administration: inConcert.app.getBaseDomain() + "/inconcert/apps/administration"
		};
		var base_URI = apps_URIs[ app_id ];
		return base_URI;
	};

	Campaign.prototype.getAvatarURI = function() {
		var parsedCampaign = this.Id.match(new RegExp("(.+)@([^@]+)$"));
		var campaignWithoutVCC = parsedCampaign ? parsedCampaign[1] : this.Id;
		return "/campaigns/get_avatar/" + campaignWithoutVCC;
	};

	Campaign.prototype.summarizeCampaignInteractions = function() {
		var counters = this.interactionCounters;
		var totalQueued = 0;
		var totalUnassignedQueued = 0;
		var totalActive = 0;
		var totalWaiting = 0;
		var totalUnassignedAbandoned = 0;
		//this loop just accumulate totals by counter name
		for (var interactionType in counters) {
			if (counters.hasOwnProperty(interactionType)) {
				for (var counterName in counters[interactionType]) {
					if (counters[interactionType].hasOwnProperty(counterName)) {
						var total = parseInt(counters[interactionType][counterName], 10);
						if (!isNaN(total)) {
							switch (counterName) {
								case "Waiting":
									totalWaiting += total;
									break;
								case "Queued":
									totalQueued += total;
									break;
								case "UnassignedQueued":
									totalUnassignedQueued += total;
									break;
								case "Assigned":
									totalActive += total;
									break;
								case "UnassignedAbandoned":
									totalUnassignedAbandoned += total;
									break;
							}
						}
					}
				}
			}
		}
		return {
			totalActive              : totalActive,
			totalWaiting             : totalWaiting,
			totalQueued              : totalQueued,
			totalUnassignedQueued    : totalUnassignedQueued,
			totalUnassignedAbandoned : totalUnassignedAbandoned
		};
	};

	// TO-DO: do not use browser date
	Campaign.prototype.now = function() {
		let now = new Date();
		let offset = now.getTimezoneOffset() + this.timezoneOffset;
		now.setTime(now.getTime() + (offset * 60 * 1000));
		return now;
	};

	/**
	 * returns true/false if the campaign is on schedule or not
	 *
	 * @return {Boolean}
	 */
	Campaign.prototype.isOnSchedule = function() {
		let now = this.now();
		let scheduler = this.scheduler || {};
		let status = inConcert.Utils.isOnSchedule(now, scheduler);
		return status === "ok";
	};

	//returns true/false if the campaign has set up dialer
	Campaign.prototype.hasDialer = function() {
		//TODO: realizar validacion correspondiente cuando i6 tenga discador implementado.
		return false;
	};

	Campaign.prototype.get_attention_capability = function( callback )
	{
		callback = callback || $.noop;

		var url = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/channels/get_campaign/" + this.Id;
		inConcert.invoke({
			url: url,
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function( values ) {
				callback( true, values );
			},
			error: function( undefined /*response*/, error) {
				callback( false, error );
			}
		});
	};

	Campaign.prototype.save_attention_capability = function( params )
	{
		inConcert.logger.action({
			action_name : "save_attention_capability",
			campaign    : this.Id,
			action_type : "campaign"
		});

		var callback   = params.callback || $.noop;
		var capability = params.capability;
		capability.campaign_id = this.Id;

		var execute_callback = function( success, error )
		{
			if ( success ) {
				callback( success );
				return;
			}
			callback( success, error );
		};

		var uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/channels/save_campaign/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			data:        JSON.stringify( capability ),
			dataType:    "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success:     function( undefined /*response*/ ) { execute_callback( true );                                } ,
			error:       function( code, error )            { execute_callback( false, { code: code, error: error } ); }
		});
	};

	Campaign.prototype.getUserCampaignPermissions = function( user_id, callback )
	{
		inConcert.logger.action({action_name : "get_user_campaign_permissions", campaign : this.Id, action_type:"campaign"});
		callback = callback || $.noop;

		var url = this.getBaseURI() + "/campaigns/get_user_campaign_permissions/" + this.Id + "/" + user_id;
		inConcert.invoke({
			url: url,
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function(response) {
				callback(response);
			},
			error: function( undefined /*response*/, error) {
				callback(null, error);
			}
		});
	};

	Campaign.prototype.saveRolePermissions = function( roles_permissions, onSavedFn )
	{
		var campaign_id = this.Id;
		inConcert.logger.action({
			action_name : "save_roles_permissions_to_campaign"  ,
			campaign    : campaign_id                           ,
			action_type : "campaign"
		});

		onSavedFn = onSavedFn || $.noop;
		var execute_callback = function( undefined /*code*/, error_type )
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			onSavedFn( error );
		};

		inConcert.invoke({
			url         : this.getBaseURI() + "/campaigns/save_role_permissions/"                               ,
			type        : "POST"                                                                                ,
			data        : JSON.stringify( { campaignId: campaign_id, roles_permissions: roles_permissions   } ) ,
			dataType    : "json"                                                                                ,
			contentType : "application/json"                                                                    ,
			timeout    : get_long_request_timeout_ms(),
			success     : function( undefined /*response*/ )   { execute_callback();                        }    ,
			error       : function( code, error )              { execute_callback( code, error );           }
		});
	};

	Campaign.prototype.setAllPermissions = function( permissions )
	{
		this.permissions = permissions;
	};

	Campaign.prototype.getAllPermissions = function()
	{
		return this.permissions;
	};

	Campaign.prototype.getOverrideSpecificRolePermissions = function( role /* object */ )
	{
		if ( typeof role !== "object" ) {
			return;
		}
		var override_permissions = this.permissions;
		if ( !override_permissions || !override_permissions.groups ) {
			return null;
		}
		var domain = inConcert.app.getDomain();
		var role_id = role.Id + "@" + ( role.IsSystemGroup === "1" ? "system" : domain );
		var override = override_permissions.groups[ role_id ];
		if ( override && ( override.actions || override.apps ) ) {
			return override;
		}
		return null;
	};

	Campaign.prototype.getOverrideSpecificMemberPermissions = function( member_id /* string */ )
	{
		if ( typeof member_id !== "string" ) {
			return;
		}
		var override_permissions = this.permissions;
		if ( !override_permissions || !override_permissions.members ) {
			return null;
		}
		var override = override_permissions.members[ member_id ];
		if ( override && ( override.actions || override.apps ) ) {
			return override;
		}
		return null;
	};

	Campaign.prototype.saveDispositionCodes = function( dispositionCodes, onSavedFn )
	{
		inConcert.logger.action({
			action_name:	"save_disposition_codes_to_campaign",
			campaign:		this.Id,
			action_type:	"campaign"
		});
		onSavedFn = onSavedFn || $.noop;
		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			onSavedFn(error);
		};
		inConcert.invoke({
			url        : this.getBaseURI() + "/campaigns/save_disposition_codes/",
			type       : "POST",
			data       : JSON.stringify({ campaignId: this.Id, dispositionCodes: dispositionCodes }),
			dataType   : "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success    : function( undefined /*response*/ ) { execute_callback();            },
			error      : function(code, error)              { execute_callback(code, error); }
		});
	};

	Campaign.prototype.getDispositionCodes = function()
	{
		var dispositionCodes = this.dispositionCodes;
		if (dispositionCodes) {
			if (typeof(dispositionCodes) === "string") {
				dispositionCodes = JSON.parse(dispositionCodes);
			}
			if (dispositionCodes.length > 0) {
				return dispositionCodes;
			}
		}
		return null;
	};

	Campaign.prototype.getDefaultDispositionTreePathForAutoWrapup = function()
	{
		const dispositionCodes = this.getDispositionCodes();
		if (!dispositionCodes) {
			return null;
		}
		const fnGetTreePathFromDefaultCodeOrLast = function(codes) {
			let result = {
				founded : false,
				hierarchical : []
			};
			for (let index = 0; index < codes.length; index++) {
				const element = codes[index];
				if ((element.childs)) {
					result = fnGetTreePathFromDefaultCodeOrLast(element.childs);
				}
				else if(element.isDefaultForWrapUp){
					result.founded = true;
				}
				if (result.founded || ((index + 1) === codes.length)) {
					result.hierarchical.push(element.name);
					break;
				}
				result.hierarchical = [];
			}
			return result;
		};
		let treePath = fnGetTreePathFromDefaultCodeOrLast(dispositionCodes);
		return {
			name    : treePath.hierarchical[0] ,
			treePath: treePath.hierarchical.reverse()
		};
	};

	Campaign.prototype.hasDispositionCodes = function()
	{
		var dispositionCodes = this.getDispositionCodes();
		return dispositionCodes !== null;
	};

	Campaign.prototype.saveIntegrationData = function( integration, onSaved )
	{
		inConcert.logger.action({
			action_name:	"save_intergration_cci_to_campaign",
			campaign:		this.Id,
			action_type:	"campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function( undefined /*code*/, error_type )
		{
			var error;
			if ( error_type ) {
				error = error_type;
			}
			onSaved( error );
		};
		inConcert.invoke({
			url         : this.getBaseURI() + "/campaigns/save_integration_cci/",
			type        : "POST",
			data        : JSON.stringify({campaignId:	this.Id, integration: integration}),
			dataType    : "json",
			contentType : "application/json",
			timeout     : get_long_request_timeout_ms(),
			success     : function(undefined /*response*/ ) { execute_callback();            },
			error       : function( code, error )           { execute_callback(code, error); }
		});
	};

	Campaign.prototype.getIntegrationData = function()
	{
		let integration = this.Integration;

		switch (typeof integration)
		{
			case "string":
				integration = JSON.parse(integration);
				/* falls through */
			case "object":
				if ($.isEmptyObject(integration)) {
					return null;
				}
				return integration;
			default:
				return null;
		}
	};

	Campaign.prototype.hasCCIntegration = function()
	{
		const integration = this.getIntegrationData();
		if (!integration) {
			return false;
		}
		if (typeof integration.type !== "string") {
			return false;
		}
		const recognised_cc_integrations = {
			CTIPOPUP       : true,
			CTIALLEGROPOPUP: true,
			CTIPOPUPWEB    : true
		};
		if (!recognised_cc_integrations[integration.type.toUpperCase()]) {
			return false;
		}
		// Si tiene integracion con CC
		return true;
	};

	Campaign.prototype.hasIntegrationWithMarketingAndSales = function()
	{
		let mas_integration = this.getMasIntegrationData();
		if (!mas_integration || !mas_integration.Enable) {
			return false;
		}
		// Si tiene integracion con M&S
		return true;
	};

	Campaign.prototype.getMasIntegrationData = function()
	{
		if (!inConcert.app.has_mas_integration()) {
			return false;
		}
		const integration = this.getIntegrationData();
		if (!integration) {
			return false;
		}
		if (!integration.othersIntegrations) {
			return false;
		}
		if (!integration.othersIntegrations.MAS) {
			return false;
		}
		return integration.othersIntegrations.MAS;
	};

	Campaign.prototype.hasIntegrationWithHelpDesk = function()
	{
		let helpdesk_integration = this.getHelpDeskIntegrationData();
		if (!helpdesk_integration || !helpdesk_integration.Enable) {
			return false;
		}
		// Si tiene integracion con HelpDesk
		return true;
	};

	Campaign.prototype.getHelpDeskIntegrationData = function()
	{
		if (!inConcert.app.has_helpdesk_integration()) {
			return false;
		}
		const integration = this.getIntegrationData();
		if (!integration) {
			return false;
		}
		if (!integration.othersIntegrations) {
			return false;
		}
		if (!integration.othersIntegrations.HELPDESK) {
			return false;
		}
		return integration.othersIntegrations.HELPDESK;
	};

	Campaign.prototype.getWidgetIntegrationData = function(widget_id)
	{
		widget_id = (widget_id || "").toLowerCase();
		if (!inConcert.app.has_widget_integration(widget_id)) {
		 	return false;
		}
		const integration = this.getIntegrationData();
		if (!integration) {
			return false;
		}
		if (!integration.othersIntegrations) {
			return false;
		}
		if (!integration.othersIntegrations[widget_id]) {
			return false;
		}
		let widget_info = integration.othersIntegrations[widget_id];
		if (!widget_info) {
			return false;
		}
		return widget_info;
	};

	Campaign.prototype.hasIntegrationWithOccWidget = function(widget_id)
	{
		widget_id = (widget_id || "").toLowerCase();
		let widget_info = this.getWidgetIntegrationData(widget_id);
		if (!widget_info || !widget_info.IsWidget || (widget_info.IsWidget && !widget_info.Enable)) {
			return false;
		}
		// Si tiene integracion con el Widget
		return true;
	};

	Campaign.prototype.saveInteractionNotificationsInfo = function(data, onSaved)
	{
		inConcert.logger.action({
			action_name:	"save_interaction_notifications_to_campaign",
			campaign:		this.Id,
			action_type:	"campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function( undefined /*code*/, error_type )
		{
			var error;
			if ( error_type ) {
				error = error_type;
			}
			onSaved( error );
		};
		inConcert.invoke({
			url         : this.getBaseURI() + "/campaigns/save_interaction_notifications/",
			type        : "POST",
			data        : JSON.stringify({campaignId: this.Id, Data: data}),
			dataType    : "json",
			contentType : "application/json",
			timeout     : get_long_request_timeout_ms(),
			success     : function(undefined /*response*/ ) { execute_callback();            },
			error       : function( code, error )           { execute_callback(code, error); }
		});
	};

	Campaign.prototype.getInteractionNotificationsInfo = function()
	{
		switch (typeof this.InteractionNotifications)
		{
			case "string":
				this.InteractionNotifications = JSON.parse(this.InteractionNotifications);
				/* falls through */
			case "object":
				if ($.isEmptyObject(this.InteractionNotifications)) {
					return null;
				}
				return this.InteractionNotifications;
			default:
				return null;
		}
	};

	Campaign.prototype.hasConfiguredInteractionNotifications = function()
	{
		var data = this.this.InteractionNotifications;
		return data !== undefined && data !== null && !$.isEmptyObject(data);
	};

	Campaign.prototype.setThresholds = function( new_thresholds )
	{
		this.Thresholds = new_thresholds;
	};

	Campaign.prototype.getThresholds = function()
	{
		var thresholds = this.Thresholds;

		switch ( typeof thresholds )
		{
			case "object":
				return thresholds;

			case "string":
				return JSON.parse(thresholds);

			default:
				return undefined;
		}
	};

	Campaign.prototype.getThreshold = function( interaction_type, attr )
	{
		if (typeof interaction_type !== "string" || typeof attr !== "string") {
			return;
		}
		var thresholds = this.getThresholds();
		if (typeof thresholds !== "object" || typeof thresholds[ interaction_type ] !== "object") {
			return;
		}
		return thresholds[interaction_type][attr];
	};

	Campaign.prototype.CheckInteractionType = function( interaction_type )
	{
		if (typeof interaction_type !== "string") {
			return false;
		}
		var thresholds = this.getThresholds();
		if (typeof thresholds !== "object" || typeof thresholds[ interaction_type ] !== "object") {
			return false;
		}
		return true;
	};

	Campaign.prototype.saveThresholds = function( params )
	{
		var self = this;
		inConcert.logger.action({
			action_name : "save_thresholds_to_campaign",
			campaign    : this.Id,
			action_type : "campaign"
		});

		var thresholds = params.thresholds;
		var callback   = params.callback || $.noop;
		var account_is_created = params.account_is_created; // optional ( cuando la cuenta se esta creando )

		var execute_callback = function( success, error, thresholds)
		{
			if ( success ) {
				self.setThresholds( thresholds || {});
				callback( true );
				return;
			}
			callback( false, error );
		};

		var uri = this.getBaseURI() + "/campaigns/save_thresholds/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			data:        JSON.stringify( { campaignId: this.Id, thresholds: thresholds, account_is_created: account_is_created } ),
			dataType:    "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success:     function(response) { 
				execute_callback( true, undefined, response && response.thresholds);                                
			} ,
			error:       function( code, error )            { execute_callback( false, { code: code, error: error } ); }
		});
	};

	Campaign.prototype.getOEConfig = function()
	{
		if (typeof this.OEConfig === "string") {
			this.OEConfig = JSON.parse(this.OEConfig);
		}
		return this.OEConfig;
	};

	/*
		Returns the joined list of teams for the campaigns of an agent
	*/
	Campaign.prototype.getTeamMembers = function( callback, from_cache )
	{
		var self = this;

		inConcert.logger.action({action_name : "get_members", campaign : this.Id, action_type:"campaign"});
		callback = callback || $.noop;

		var execute_callback = function(error) {
			callback(self.m_members, error);
		};
		if ( from_cache && this.m_members.length > 0 ) {
			execute_callback();
			return;
		}
		inConcert.invoke({
			url: this.getBaseURI() + "/campaigns/get_members/",
			type: "POST",
			data: JSON.stringify({
				campaignId: this.Id
			}),
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function(response) {
				self.m_members = [];
				for (var member in response) {
					if (response.hasOwnProperty(member)) {
						var memberObj = new inConcert.TeamMember(response[member]);
						self.m_members.push(memberObj);
					}
				}
				execute_callback();
			},
			error: function( undefined /*response*/, error) {
				execute_callback(null, error.result);
			}
		});
	};

	Campaign.prototype.saveAccountMessages = function( account_id, messages, callback )
	{
		inConcert.logger.action({action_name : "save_account_messages", campaign : this.Id, action_type:"campaign"});
		callback = callback || $.noop;

		var url = this.getBaseURI() + "/campaigns/save_account_messages/";
		var request = {
			campaignId : this.Id    ,
			accountId  : account_id ,
			messages   : messages
		};

		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function(response) {
				if (response.result) {
					callback(); // success
				}
				else {
					var error = true;
					callback(error);
				}
			},
			error: function( undefined /*response*/, error) {
				callback(error);
			}
		});
	};

	Campaign.prototype.getAccountMessages = function( account_id, callback, app_id )
	{
		inConcert.logger.action({action_name : "get_account_messages", campaign : this.Id, action_type:"campaign"});
		callback = callback || $.noop;

		var url = this.getBaseURI( app_id ) + "/campaigns/get_account_messages/" + this.Id + "/" + account_id;
		inConcert.invoke({
			url: url,
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function( response ) {
				callback( response);
			},
			error: function( undefined /*response*/, error) {
				callback( null, error );
			}
		});
	};

	Campaign.prototype.serialize = function()
	{
		var include_attributes = {
			"isNew"             : true, // este attr se elimina en el server y no se guarda
			"Id"                : true,
			"Name"              : true,
			"Description"       : true,
			"ValidationTimeZone": true,
			"Thresholds"        : true
		};
		return inConcert.Utils.plainOldObject(this, include_attributes);
	};

	/*
		Saves the campaign for the first time, it's assumed the user has acquired the lock for this campaign id calling
		/campaign/exists previously
	*/
	Campaign.prototype.create = function(callbackFn)
	{
		var self = this;

		inConcert.logger.action({action_name : "create_campaign", campaign : this.Id, action_type:"campaign"});

		callbackFn = callbackFn || $.noop;
		var request = this.serialize();

		inConcert.invoke({
			url: this.getBaseURI() + "/campaigns/create/",
			type: "POST",
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function(response) {
				var campaign = response && response.campaign;
				var success;
				if (campaign) {
					//just extend locally the object using the created campaign with all it's default values
					$.extend(true, self, campaign);
					success = true;
				}
				else {
					campaign = response || "error";
					success = false;
				}
				callbackFn(success, campaign);
			},
			error : function( xhr, textStatus, undefined /*errorThrown*/ ) {
				var code = "error";
				if (xhr.status === 400) {
					textStatus = textStatus.toLowerCase();
					switch(textStatus) {
						case "already exists":
							code = "already_exists";
							break;
						case "used by another user":
							code = "used_by_another_user";
							break;
					}
				}
				var success = false;
				callbackFn(success, code);
			}
		});
	};

	/*
		Saves the campaign assuming it already exists, if non existant this function will fail
	*/
	Campaign.prototype.save = function(callbackFn) {
		var self = this;
		inConcert.logger.action({action_name : "save_campaign", campaign : self.Id, action_type:"campaign"});

		var url = this.getBaseURI() + "/campaigns/save/";
		var request = self.serialize();
		request.campaignId = request.Id;

		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(request),
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function() {
				if (typeof(callbackFn) === "function") {
					callbackFn(true);
				}
			},
			error: function() {
				if (typeof(callbackFn) === "function") {
					callbackFn(false);
				}
			}
		});
	};

	/*
		Saves the campaign assuming it already exists, if non existant this function will fail
	*/
	Campaign.prototype.save_basic = function(callbackFn, is_new) {
		var self = this;
		inConcert.logger.action({action_name : "create_campaign", campaign : this.Id, action_type:"campaign"});
		callbackFn = callbackFn || $.noop;
		var url = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/save/";
		var request = this.serialize();

		request.campaignId = request.Id;
		var encodedCampaign = JSON.stringify(request);

		inConcert.invoke({
			url: url,
			type: "POST",
			data: encodedCampaign,
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function(response)
			{
				var success;
				var campaign;
				if (is_new) {
					var saved_campaign = response && response.saved_campaign;
					if (saved_campaign) {
						saved_campaign = JSON.parse(saved_campaign);
						campaign = saved_campaign.campaign;
						//just extend locally the object using the created campaign with all it's default values
						jQuery.extend(true, self, campaign);
						success = true;

						campaign.allPermissionsOnCampaign = response.allPermissionsOnCampaign;
					}
					else {
						campaign = response || "error";
						success = false;
					}
				}
				else {
					success = true;
				}
				callbackFn(success, campaign);
			},
			error : function( undefined /*xhr*/, text_status)
			{
				var err;
				try {
					err = JSON.parse(text_status);
				}
				catch(e) {
					err = text_status || "error";
				}
				var success = false;
				callbackFn(success, err);
			}
		});
	};

	/*
		Returns campaign description, if doesn't have one returns campaign id
	*/
	Campaign.prototype.getDescription = function() {
		if (this.Name && this.Name.length > 0) {
			return this.Name;
		}
		else {
			var match = this.Id.match(new RegExp("([^@]+)@(.*)"));
			return match ? match[1] : this.Id;
		}
	};

	Campaign.prototype.saveScheduler = function(scheduler, callbackFn) {
		var self = this;
		inConcert.logger.action({action_name : "save_campaign_scheduler", campaign : this.Id, action_type:"campaign"});

		callbackFn = callbackFn || $.noop;

		inConcert.invoke({
			url: this.getBaseURI() + "/campaigns/save_scheduler/",
			type: "POST",
			timeout: get_long_request_timeout_ms(),
			data: JSON.stringify({
				campaignId : this.Id,
				scheduler : scheduler
			}),
			dataType: "json",
			contentType: "application/json",
			success: function( /*response*/ ) {
				self.scheduler = scheduler;
				callbackFn(true);
			},
			error : function( undefined /*code*/ , error) {
				callbackFn(false, error);
			}
		});
	};

	/*
		Deletes the current campaign, it's assumed the user must be administrator of the campaign in order to delete it
	*/
	Campaign.prototype.changeAccountState = function(account, accountEnabled, callback = $.noop) {
		const self = this;
		inConcert.logger.action({action_name: "change_account_state", campaign: self.Id, action_type: "campaign"});
		let request = {
			campaignId: self.Id,
			accountId : account.id,
			proxyType : account.proxyType,
			state     : accountEnabled
		};
		inConcert.invoke({
			url        : this.getBaseURI() + "/campaigns/change_account_state/",
			type       : "POST",
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function(response) {
				let changed = response && response.result;
				if (changed) {
					account.state = accountEnabled ? "1" : "0";
					self.setAccount(account);
				}
				callback(changed);
			},
			error: function(undefined /*response*/, error) {
				callback(false, error);
			}
		});
	};

	/*
		Deletes the current campaign, it's assumed the user must be administrator of the campaign in order to delete it
	*/
	Campaign.prototype.remove = function(callbackFn) {
		var self = this;
		inConcert.logger.action({action_name : "delete_campaign", campaign : self.Id, action_type:"campaign"});

		var url = this.getBaseURI() + "/campaigns/delete/";
		var request = {campaignId : self.Id};

		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function( /*response*/ ) {
				if (typeof(callbackFn) === "function") {
					callbackFn(true);
				}
			},
			error : function( /*xhr, textStatus, errorThrown*/ ) {
				if (typeof(callbackFn) === "function") {
					callbackFn(false);
				}
			}
		});
	};

	Campaign.prototype.getFacebookPages = function(accountId, callbackFn) {
		var self = this;
		inConcert.logger.action({action_name : "getFacebookPages", campaign : self.Id, action_type:"campaign"});

		var url = inConcert.app.getBaseUrl() + "/interactions/facebook/get_account_pages/";
		var request = {
			campaignId : self.Id,
			account : accountId
		};

		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function(response) {
				if (typeof(callbackFn) === "function") {
					callbackFn(response);
				}
			},
			error : function() {
				if (typeof(callbackFn) === "function") {
					callbackFn();
				}
			}
		});
	};

	Campaign.prototype.get_accounts = function() { return this.accounts; };

	Campaign.prototype.get_accounts_channel = function(proxyType, can_include = function() {return true;}) {
		proxyType = (proxyType || "").toUpperCase();
		let accounts = [];
		$.each(this.get_accounts() || [], function(_, account) {
			if ((account.proxyType || "").toUpperCase() === proxyType) {
				if (can_include(account)) {
					accounts.push(account);
				}
			}
		});
		return accounts;
	};

	Campaign.prototype.get_enabled_accounts_channel = function(proxyType) {
		return this.get_accounts_channel(proxyType, function(account) {
			if (!account) {
				return false;
			}
			if (account.state === "1") {
				return true;
			}
			return false;
		});
	};

	Campaign.prototype.get_channels = function() {
		let channels = {};
		$.each(this.get_accounts() || [], function(_, acc) {
			channels[acc.proxyType] = true;
		});
		return channels;
	};

	Campaign.prototype.__belongsToProxy = function(type, proxy) {
		return type === proxy || type === "VOICEMAIL" && proxy === "CALL";
	};

	Campaign.prototype.getAccount = function( account_id, proxy, findByName )
	{
		var self = this;
		var account;
		$.each(this.get_accounts() || [], function(_, acc)
		{
			if ( acc.id === account_id || (findByName && acc.name.toLowerCase() === account_id.toLowerCase()) ) {
				if ( proxy ) {
					if ( acc.proxyType === proxy || self.__belongsToProxy(proxy, acc.proxyType)) {
						account = acc;
						return false; // break
					}
				}
				else {
					account = acc;
					return false; // break
				}
			}
		});
		return account;
	};

	Campaign.prototype.setAccount = function(new_account)
	{
		var self = this;

		if (!new_account) {
			return;
		}
		var is_new = true;
		this.accounts = this.accounts || [];
		$.each(this.accounts, function( i, current_account )
		{
			if ( new_account.id === current_account.id ) {
				self.accounts[i] = new_account;
				is_new = false;
				return false; // break
			}
		});
		if (is_new) {
			this.accounts.push(new_account);
		}
		this.IndexAttentionLevels();
	};

	Campaign.prototype.getAccountDescription = function( account )
	{
		if ( !account ) {
			return "";
		}
		switch ( account.proxyType )
		{
			case "CALL":
			case "VOICEMAIL":
			case "IM":
			case "FACEBOOK":
			case "INSTAGRAM":
			case "YOUTUBE":
			case "APPSTORE":
			case "GOOGLEPLAY":
			case "RCS":
			case "MAIL":
			case "SMS":
			case "WHATSAPP":
			case "TWITTER":
				return account.name || "<%=_T('(noname)')%>";
			case "CHAT":
			case "WEBCONTACT":
				if ( typeof account.name === "string" && account.name.trim().length > 0 ) {
					return account.name;
				}
				var urls = account.url_regexp;
				if ( typeof account.url_regexp === "string" ) {
					urls = JSON.parse( account.url_regexp );
				}
				if ( $.isArray( urls ) && urls.length > 0 ) {
					return urls[ 0 ] + ( urls.length > 1 ? "..." : "" );
				}
				if ( !account.autochat ) {
					return "";
				}
				var autochats = typeof( account.autochat ) === "object" ? account.autochat : JSON.parse( account.autochat );
				if ( autochats.length > 0 ) {
					return autochats[ 0 ].url_regexp + ( autochats.length > 1 ? "..." : "" );
				}
				else {
					return "";
				}
			break;

			default:
				return account.id;
		}
	};

	Campaign.prototype.getAccountPages = function(account_id)
	{
		var account = this.getAccount(account_id);
		if (!account) {
			return;
		}
		if (account.proxyType !== "FACEBOOK" && account.proxyType !== "INSTAGRAM") {
			return;
		}
		if (typeof(account.pagesNames) !== "object") {
			account.pagesNames = JSON.parse(account.pagesNames);
		}
		return account.pagesNames;
	};

	Campaign.prototype.get_account_language = function( account_id )
	{
		var account = this.getAccount(account_id);
		if ( !account ) {
			return;
		}
		return account.language;
	};

	Campaign.prototype.get_account_channel = function(account_id)
	{
		const account = this.getAccount(account_id);
		if (!account) {
			return;
		}
		return account.proxyType;
	};

	Campaign.prototype.getPreferredAccounts = function( account ) {
		if ( typeof account !== "object" ||
				( typeof account.PreferredAccounts !== "string" && typeof account.PreferredAccounts !== "object") ) {
			return;
		}
		if ( typeof account.PreferredAccounts === "object" ) {
			return account.PreferredAccounts;
		}
		account.PreferredAccounts = JSON.parse( account.PreferredAccounts );
		return account.PreferredAccounts;
	};

	/*
		Changes the role for a specific user on this campaign, empty role removes user from the campaign
	*/
	Campaign.prototype.UpdateMembersList = function(members, callbackFn) {
		inConcert.logger.action({action_name : "update_members_list", campaign : this.Id, action_type:"campaign"});

		callbackFn = callbackFn || $.noop;
		inConcert.invoke(
		{
			url: this.getBaseURI() + "/campaigns/update_members_list/",
			timeout: get_long_request_timeout_ms(),
			type: "POST",
			data: JSON.stringify({
				campaignId: this.Id,
				members: members
			}),
			dataType: "json",
			contentType: "application/json",
			success: function( /*response*/ )
			{
				callbackFn(true);
			},
			error: function( undefined /*response*/, error)
			{
				inConcert.logger.error("Error changing member role: " + error.result);
				callbackFn(false, error);
			}
		});
	};

	/*
		Retrieves the full user list of the VCC with the corresponding role on the current campaign
	*/
	Campaign.prototype.getFullUserList = function(callbackFn)
	{
		inConcert.logger.action({action_name : "get_campaign_members", campaign : this.Id, action_type:"campaign"});
		callbackFn = callbackFn || $.noop;

		var url = this.getBaseURI() + "/campaigns/get_campaign_members/" + this.Id;
		inConcert.invoke({
			url: url,
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function(response) {
				callbackFn(response);
			},
			error: function( undefined /*response*/, error ) {
				callbackFn( false, error );
			}
		});
	};

	Campaign.prototype.availableSaveAccount = function( account, onValidate )
	{
		onValidate = onValidate || $.noop;

		inConcert.logger.action( {action_name : "is_available_save_account", campaign : this.Id, action_type:"campaign"} );
		var url = this.getBaseURI() + "/campaigns/is_available_save_account/";
		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify({
				campaignId : this.Id,
				account : account
			}),
			dataType: "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success: function( account ) {
				onValidate( true, account );
			},
			error: function( undefined /*response*/, error ) {
				inConcert.logger.error("(Campaign.prototype.availableSaveAccount) - Error validating saving account with error: " + error.result);
				onValidate( false, error );
			}
		});
	};

	/*
		Adds a new account to this campaign
	*/
	Campaign.prototype.addAccount = function(account, onAdded)
	{
		var self = this;
		onAdded = onAdded || $.noop;

		inConcert.logger.action({action_name : "add_campaign_account", campaign : this.Id, action_type:"campaign"});

		var uri = this.getBaseURI() + "/campaigns/add_account/";
		inConcert.invoke({
			url: uri,
			type: "POST",
			data: JSON.stringify({
				campaignId : this.Id,
				account : account
			}),
			timeout: get_long_request_timeout_ms(),
			dataType: "json",
			contentType: "application/json",
			success: function(account) {
				onAdded(true, account);
				// la invocacion a esta funcion debe ir despues del callback, sino los thresholds no se almacenan correctamente.
				self.on_added_account(account);
			},
			error: function( undefined /*response*/, error) {
				inConcert.logger.error("(Campaign.prototype.addAccount) - Error adding account with error: " + error.result);
				onAdded(false, error);
			}
		});
	};

	Campaign.prototype.on_added_account = function( account )
	{
		this.setAccount(account);
		var campaign_id = this.Id;
		// la publicacion la realizo en un timeout, asi la ejecucion se realiza asincronica y puede seguir el flujo del codigo de forma correcta.
		window.setTimeout(function() {
			inConcert.app.messagebus.publish_async("/application/campaign/" + campaign_id + "/account/added/", account);
		}, 500);
	};

	/*
		Updates an existing account
	*/
	Campaign.prototype.updateAccount = function(account, onUpdated)
	{
		var self = this;
		onUpdated = onUpdated || $.noop;

		inConcert.logger.action({action_name : "update_campaign_account", campaign : this.Id, action_type:"campaign"});

		var uri = this.getBaseURI() + "/campaigns/update_account/";
		inConcert.invoke({
			url: uri,
			type: "POST",
			data: JSON.stringify({
				campaignId : this.Id,
				account : account
			}),
			timeout: get_long_request_timeout_ms(),
			dataType: "json",
			contentType: "application/json",
			success: function( /*response*/ ) {
				self.on_updated_account(account);
				onUpdated(true);
			},
			error: function( undefined /*response*/, error) {
				inConcert.logger.error("Error updating account: " + error.result);
				onUpdated(false, error);
			}
		});
	};

	Campaign.prototype.on_updated_account = function( account )
	{
		this.setAccount(account);
		var campaign_id = this.Id;
		// la publicacion la realizo en un timeout, asi la ejecucion se realiza asincronica y puede seguir el flujo del codigo de forma correcta.
		window.setTimeout(function() {
			inConcert.app.messagebus.publish_async("/application/campaign/" + campaign_id + "/account/updated/", account);
		}, 500);
	};

	/*
		Deletes an account from this campaign
	*/
	Campaign.prototype.deleteAccount = function(accountId, proxyType, deleteData, callbackFn)
	{
		var self = this;
		callbackFn = callbackFn || $.noop;

		inConcert.logger.action({action_name : "remove_campaign_account", campaign : this.Id, action_type:"campaign"});

		var uri = this.getBaseURI() + "/campaigns/remove_account/";
		var request = {
			campaignId : this.Id,
			accountId : accountId,
			proxyType : proxyType,
			data : deleteData
		};
		inConcert.invoke({
			url: uri,
			type: "POST",
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			timeout: get_long_request_timeout_ms(),
			success: function( /*response*/ ) {
				self.on_removed_account(accountId);
				callbackFn(true);
			},
			error : function( undefined /*response*/, error) {
				callbackFn(false, error);
			}
		});
	};

	Campaign.prototype.beforeDeleteAccount = function (_, proxyType, callbackFn)
	{
		callbackFn = callbackFn || $.noop;

		function onConfirmation(mustDelete, deleteData) {
			callbackFn(mustDelete, deleteData);
		}

		switch(proxyType) {
			case "FACEBOOK":
				window.setTimeout(function() {
					$.inconcertPopup.Show(
						"<%=_T('Confirmation')%>",
						"<%=_T('Would you like to restore account pages messenger configuration?')%>",
						$.inconcertPopup.ButtonSets.BS_YESNOCANCEL,
						function(response) {
							switch(response.toLowerCase()) {
								case "yes":
									return onConfirmation(true, { mustCleanMessenger : true });
								case "no":
									return onConfirmation(true, { mustCleanMessenger : false });
								default:
									return onConfirmation(false);
							}
						},
						undefined,
						{
							popup_id        : "conf_inconcertPopup",
							container_class : "confirmationPopUp",
							overlay_class   : "confirmationOverlay"
						}
					);
				}, 50);
				break;
			default:
				onConfirmation(true);
				break;
		}
	};

	Campaign.prototype.on_removed_account = function( removed_account_id )
	{
		var self = this;
		var accounts = this.get_accounts() || [];
		$.each(accounts, function( i, account )
		{
			if (account.id === removed_account_id) {
				accounts.splice(i, 1);
				self.IndexAttentionLevels();
				return false; // break
			}
		});
		var campaign_id = this.Id;
		// la publicacion la realizo en un timeout, asi la ejecucion se realiza asincronica y puede seguir el flujo del codigo de forma correcta.
		window.setTimeout(function() {
			inConcert.app.messagebus.publish_async("/application/campaign/" + campaign_id + "/account/removed/", removed_account_id );
		}, 500);
	};

	/*
		Checks whether a campaign can solve a specific type of interaction
	*/
	Campaign.prototype.has_accounts_of_proxy = function(proxy, enabled, active)
	{
		if (!this.accounts || !proxy) {
			return false;
		}
		proxy = proxy.toUpperCase();
		if (proxy === "VOICEMAIL") {
			proxy = "CALL";
		}

		for (var i = 0; i < this.accounts.length; i++) {
			let account = this.accounts[i];
			if (this.accounts[i].proxyType.toUpperCase() === proxy &&
				(!enabled || (account.state && account.state !== "0") ) &&
				(!active || (typeof account.status === "object" && account.status.status === "OK") )
			) {
				return true;
			}
		}

		return false;
	};

	Campaign.prototype.has_sync_accounts = function()
	{
		if (!this.accounts) {
			return false;
		}

		for (let channel in m_channels) {
			if (m_channels.hasOwnProperty(channel)) {
				const channel_info = m_channels[channel];
				if (channel_info.isSync) {
					let has_sync_accounts = this.has_accounts_of_proxy(channel);
					if (has_sync_accounts) {
						return true;
					}
				}
			}
		}

		return false;
	};

	Campaign.prototype.has_async_accounts = function()
	{
		if (!this.accounts) {
			return false;
		}

		for (let channel in m_channels) {
			if (m_channels.hasOwnProperty(channel)) {
				const channel_info = m_channels[channel];
				if (!channel_info.isSync) {
					let has_async_accounts = this.has_accounts_of_proxy(channel);
					if (has_async_accounts) {
						return true;
					}
				}
			}
		}

		return false;
	};

	Campaign.prototype.countActiveAccountsByType = function(proxyType)
	{
		if (!this.accounts || !proxyType) {
			return 0;
		}
		var count = 0;
		$.each(this.accounts, function( undefined, account)
		{
			if ( account.proxyType === proxyType && account.state === "1" ) {
				if ( typeof account.status === "object" && account.status.status === "OK" ) {
					count++;
				}
			}
		});
		return count;
	};

	Campaign.prototype.saveCannedMessages = function(canned_messages, onSaved)
	{
		inConcert.logger.action({
			action_name:	"save_canned_messages_to_campaign",
			campaign:		this.Id,
			action_type:	"campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			onSaved(error);
		};
		var uri = this.getBaseURI() + "/campaigns/save_canned_messages/";
		inConcert.invoke({
			url:			uri,
			type:			"POST",
			data:			JSON.stringify({campaignId:	this.Id, canned_messages: canned_messages}),
			dataType:		"json",
			contentType:	"application/json",
			timeout    : get_long_request_timeout_ms(),
			success:		function( /*response*/ ) { execute_callback();            },
			error:			function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.deleteCannedMessages = function(canned_messages, onRemoved)
	{
		inConcert.logger.action({
			action_name:	"delete_canned_messages_to_campaign",
			campaign:		this.Id,
			action_type:	"campaign"
		});
		onRemoved = onRemoved || $.noop;
		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			onRemoved(error);
		};
		var uri = this.getBaseURI() + "/campaigns/delete_canned_messages/";
		inConcert.invoke({
			url:			uri,
			type:			"POST",
			data:			JSON.stringify({campaignId:	this.Id, canned_messages: canned_messages}),
			dataType:		"json",
			contentType:	"application/json",
			timeout    : get_long_request_timeout_ms(),
			success:		function( /*response*/ ) { execute_callback();            },
			error:			function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.getCannedMessages = function(fnCallback)
	{
		var self = this;
		fnCallback = fnCallback || $.noop;
		inConcert.logger.action({action_name : "get_campaign_canned_messages", campaign : this.Id, action_type:"campaign"});

		var clone = inConcert.Utils.cloneObject;

		if (!this.CannedMessages) {
			var uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/get_canned_messages/" + this.Id;
			inConcert.invoke({
				url: uri,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				timeout    : get_long_request_timeout_ms(),
				success: function(canned_messages) {
					self.CannedMessages = canned_messages;
					fnCallback( clone(canned_messages) );
				},
				error: function( undefined /*response*/, error) {
					self.CannedMessages = null;
					fnCallback(null, error.result);
				}
			});
		}
		else {
			var canned_messages = this.CannedMessages;
			if (typeof canned_messages === "string") {
				canned_messages = JSON.parse(canned_messages);
			}
			fnCallback( clone(canned_messages) );
		}
	};

	Campaign.prototype.hasCannedMessages = function()
	{
		var canned = this.CannedMessages;
		return canned !== undefined && canned !== null && !$.isEmptyObject(canned);
	};

	Campaign.prototype.__setAttentionLevels = function( attention_levels_update )
	{
		var self = this;
		this.AttentionLevels = this.AttentionLevels || {};
		$.each(attention_levels_update, function(_, att_level ) {
			self.AttentionLevels[att_level.id] = att_level;
		});
		this.IndexAttentionLevels();
	};

	Campaign.prototype.saveAttentionLevels = function(attention_levels_update, onSaved)
	{
		var self = this;
		inConcert.logger.action({
			action_name : "save_attention_levels_to_campaign",
			campaign    : this.Id,
			action_type : "campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			else {
				self.__setAttentionLevels(attention_levels_update);
			}
			onSaved(error);
		};
		var uri = this.getBaseURI() + "/campaigns/save_attention_levels/";
		inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify({campaignId: this.Id, attention_levels: attention_levels_update}),
			dataType    : "json",
			contentType : "application/json",
			timeout    : get_long_request_timeout_ms(),
			success     : function( /*response*/ ) { execute_callback();            },
			error       : function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.can_delete_attention_levels = function( attention_levels_to_remove )
	{
		var response = { success: true };

		if (!this.AttentionLevels) {
			response.success = false;
			response.result = "NO_CONFIGURED_ATTENTION_LEVELS";
			return response;
		}
		if (!$.isArray(attention_levels_to_remove)) {
			response.success = false;
			response.result = "NO_ATTENTION_LEVELS_TO_DELETE";
			return response;
		}

		// Valida que el attention level no se encuentre en uso en alguna account, seteado con distribution en true.
		// esta validacion tambien se realiza en el API server side. La dejo aca tambien para que no tenga que realiza un request al dope.
		var accounts = this.get_accounts();
		if ($.isArray(accounts)) {
			$.each(attention_levels_to_remove, function(_, att_level_id_to_remove )
			{
				$.each(accounts, function(_, account )
				{
					if ( !account.AttentionLevels ) {
						return true; // continue
					}
					$.each(account.AttentionLevels, function(_, assigned_att_level )
					{
						if ( assigned_att_level.distribution && att_level_id_to_remove === assigned_att_level.attention_level_id ) {
							response.success = false;
							response.result = "ATTENTION_LEVEL_IN_USE_IN_ACCOUNT";
							response.att_level_id = att_level_id_to_remove;
							response.account_id = account.id;
							return false; // break
						}
					});
					if ( !response.success ) {
						return false; // break
					}
				});
				if ( !response.success ) {
					return false; // break
				}
			});
		}

		return response;
	};

	Campaign.prototype.delete_attention_levels = function( attention_levels_to_remove, on_removed )
	{
		inConcert.logger.action({
			action_name : "delete_attention_levels_to_campaign",
			campaign    : this.Id,
			action_type : "campaign"
		});

		var self = this;
		on_removed = on_removed || $.noop;

		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			else {
				if ( self.AttentionLevels ) {
					$.each(attention_levels_to_remove, function(_, att_level_id ) {
						delete self.AttentionLevels[att_level_id];
					});
					self.IndexAttentionLevels();
				}
			}
			on_removed(error);
		};

		var result = this.can_delete_attention_levels(attention_levels_to_remove);
		if ( !result.success ) {
			inConcert.logger.error({
				action_name : "delete_attention_levels_to_campaign",
				campaign    : this.Id,
				action_type : "campaign",
				err_result  : result
			});
			execute_callback(undefined, result);
			return;
		}

		var uri = this.getBaseURI() + "/campaigns/delete_attention_levels/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			data:        JSON.stringify({campaignId: this.Id, attention_levels: attention_levels_to_remove}),
			dataType:    "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success:     function( /*response*/ ) { execute_callback();            },
			error:       function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.getAttentionLevels = function( callback )
	{
		var self = this;
		callback = callback || $.noop;

		var clone = inConcert.Utils.cloneObject;
		if (!this.AttentionLevels) {
			var uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/get_attention_levels/" + this.Id;
			inConcert.invoke({
				url: uri,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				timeout    : get_long_request_timeout_ms(),
				success: function(attention_levels) {
					self.AttentionLevels = attention_levels;
					callback( clone(attention_levels) );
				},
				error: function( undefined /*response*/, error) {
					self.AttentionLevels = null;
					callback(null, error.result);
				}
			});
		}
		else {
			var attention_levels = this.AttentionLevels;
			if (typeof attention_levels === "string") {
				attention_levels = JSON.parse(attention_levels);
			}
			callback( clone(attention_levels) );
		}
	};

	Campaign.prototype.getAttentionLevel = function( attention_level_id, findByName = false )
	{
		let attention_level;
		this.getAttentionLevels(function( attention_levels )
		{
			if (!attention_levels) {
				return;
			}
			if (attention_levels[attention_level_id]) {
				attention_level = attention_levels[attention_level_id];
				return;
			}
			if (findByName && !$.isEmptyObject(attention_levels)) {
				let found_al_id = Object.keys(attention_levels).find(
					al_id => attention_levels[al_id].name === attention_level_id);
				attention_level = found_al_id? attention_levels[found_al_id] : null;
			}
		});
		return attention_level;
	};

	Campaign.prototype.getAttentionLevelByName = function( attention_level_name )
	{
		if ( !attention_level_name ) {
			return;
		}
		var attention_level;
		this.getAttentionLevels(function( attention_levels )
		{
			if ( attention_levels ) {
				attention_level_name = attention_level_name.toLowerCase();
				$.each(attention_levels, function( _, al )
				{
					if ( attention_level_name === (al.name).toLowerCase() ) {
						attention_level = al;
						return false; // break
					}
				});
			}
		});
		return attention_level;
	};

	Campaign.prototype.getDistributionAttentionLevelFromAccount = function( account_id )
	{
		var self = this;
		var account = this.getAccount( account_id );
		if ( !account ) {
			return;
		}
		var distribution_al;
		$.each( account.AttentionLevels || {}, function( _, al )
		{
			if ( al.distribution ) {
				distribution_al = self.getAttentionLevel( al.attention_level_id );
				return false; // break
			}
		});
		return distribution_al;
	};

	Campaign.prototype.getUserAttentionLevelsInAccount = function( account_id, user_id )
	{
		var self = this;
		var account = this.getAccount( account_id );
		if ( !account ) {
			return [];
		}
		var als_user = [];
		$.each( account.AttentionLevels || {}, function(_, al)
		{
			var current_al = self.getAttentionLevel( al.attention_level_id );
			if ( current_al ) {
				$.each( current_al.users || {}, function( al_user_id )
				{
					if ( user_id === al_user_id ) {
						als_user.push(current_al);
					}
				});
			}
		});
		if (als_user.length > 1) {
			let distribution_level = this.getDistributionAttentionLevelFromAccount(account_id);
			als_user = als_user.sort(function(a, b) {
				if (a.id === distribution_level.id) {
					return -1;
				}
				else if (b.id === distribution_level.id) {
					return 1;
				}
				else {
					return a.name.localeCompare(b.name);
				}
			});
		}
		return als_user;
	};

	Campaign.prototype.IndexAttentionLevels = function()
	{
		var self = this;

		if ( !this.AttentionLevels ) {
			this.att_levels_by_users = null;
			this.accounts_by_att_levels = null;
		}
		else {
			if (typeof this.AttentionLevels === "string") {
				this.AttentionLevels = JSON.parse(this.AttentionLevels);
			}
			// Aca quedan reindexados/agrupados los attention levels por usuarios.
			// algo asi:
			/*
				att_levels_by_users = {
					member_1={ att_levels = { att_level_1 = true, att_level_2 = true } },
					member_1={ att_levels = { att_level_1 = true, att_level_2 = true } },
				}
			*/
			this.att_levels_by_users = {};
			$.each(this.AttentionLevels, function( att_level_id, att_level )
			{
				$.each(att_level.users || {}, function( user_id )
				{
					self.att_levels_by_users[user_id] = self.att_levels_by_users[user_id] || {
						att_levels: {}
					};
					self.att_levels_by_users[user_id].att_levels[att_level_id] = true;
				});
			});
		}

		// Aca quedan reindexados/agrupados las cuentas por attention levels.
		// algo asi:
		/*
			accounts_by_att_levels = {
				att_level_1={ accounts = { account_1 = true, account_2 = true } },
				att_level_2={ accounts = { account_1 = true, account_2 = true } },
			}
		*/
		this.accounts_by_att_levels = null;
		$.each(this.get_accounts() || [], function(_, account )
		{
			if ( !account.AttentionLevels ) {
				return true; // continue
			}
			self.accounts_by_att_levels = self.accounts_by_att_levels || {};
			if (typeof account.AttentionLevels === "string") {
				account.AttentionLevels = JSON.parse(account.AttentionLevels);
			}
			$.each(account.AttentionLevels, function(_, assigned_att_level )
			{
				var att_level_id = assigned_att_level.attention_level_id;
				self.accounts_by_att_levels[att_level_id] = self.accounts_by_att_levels[att_level_id] || {
					accounts: {}
				};
				self.accounts_by_att_levels[att_level_id].accounts[account.id] = true;
			});
		});
	};

	Campaign.prototype.getAttentionLevelsByUser = function( user_id )
	{
		if ( !user_id ) {
			return;
		}
		var att_levels_by_users = this.att_levels_by_users[ user_id ];
		if ( !att_levels_by_users ) {
			return;
		}
		return inConcert.Utils.cloneObject( att_levels_by_users );
	};

	Campaign.prototype.getAccountsByAttentionLevel = function( al_id )
	{
		if ( !al_id ) {
			return;
		}
		var accounts_by_att_levels = this.accounts_by_att_levels[ al_id ];
		if ( !accounts_by_att_levels ) {
			return;
		}
		return inConcert.Utils.cloneObject( accounts_by_att_levels );
	};

	Campaign.prototype.getAttentionLevelByUserAndAccount = function( user_id, account_id )
	{
		var self = this;

		if ( !user_id || !account_id ) {
			return;
		}
		var att_levels_by_user = this.getAttentionLevelsByUser(user_id);
		if ( !att_levels_by_user ) {
			return;
		}

		var attention_level;
		$.each(att_levels_by_user.att_levels, function( user_al_id )
		{
			var accounts_by_att_level = self.getAccountsByAttentionLevel(user_al_id);
			if ( accounts_by_att_level ) {
				if ( accounts_by_att_level.accounts[account_id] ) {
					attention_level = self.getAttentionLevel(user_al_id);
				}
				return false; // break
			}
		});

		return attention_level;
	};

	Campaign.prototype.getUsersInAttentionLevel = function( al_id )
	{
		var users = {};
		var attention_level = (this.AttentionLevels || {})[al_id] || {};

		$.each( attention_level.users || {}, function( user_id, user )
		{
			users[user_id] = user;
		});
		return users;
	};

	Campaign.prototype.userBelongsToAttentionLevel = function( user_id, al_id )
	{
		var att_levels_by_users = this.att_levels_by_users[ user_id ];
		if ( !att_levels_by_users ) {
			return false;
		}
		if ( att_levels_by_users.att_levels[ al_id ] ) {
			return true;
		}
		return false;
	};

	Campaign.prototype.userBelongsSomeAttentionLevel = function( user_id )
	{
		var att_levels_by_users = this.att_levels_by_users[ user_id ];
		if ( !att_levels_by_users ) {
			return false;
		}
		return true;
	};

	Campaign.prototype.has_attention_levels_configured_in_account = function( account_id )
	{
		var account = this.getAccount( account_id );
		if ( !account ) {
			return false;
		}
		var assigned_als = account.AttentionLevels;
		if ( !assigned_als ) {
			return false;
		}
		if ( typeof assigned_als === "string" ) {
			assigned_als = JSON.parse( assigned_als );
		}
		if ( $.isEmptyObject( assigned_als ) ) {
			return false;
		}
		return true;
	};

	Campaign.prototype.can_move_folder_mail_account = function(account_id)
	{
		var account = this.getAccount(account_id);
		if (!account) {
			return false;
		}
		if (account.proxyType !== "MAIL") {
			return false;
		}
		if (account.__protocol !== "IMAP") {
			return false;
		}
		if (parseInt((account.mustKeepFolder || "0"), 10) === 0) {
			return false;
		}
		return true;
	};

	Campaign.prototype.saveLabels = function(labels_update, onSaved)
	{
		var self = this;
		inConcert.logger.action({
			action_name : "save_campaign_labels",
			campaign    : this.Id,
			action_type : "campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function(undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			else {
				self.Labels = self.Labels || {};
				$.each(labels_update, function(_, att_level ) {
					self.Labels[att_level.id] = att_level;
				});
			}
			onSaved(error);
		};
		var uri = this.getBaseURI() + "/campaigns/save_labels/";
		inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify({campaignId: this.Id, labels: labels_update}),
			dataType    : "json",
			contentType : "application/json",
			timeout    : get_long_request_timeout_ms(),
			success     : function( /*response*/ ) { execute_callback();            },
			error       : function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.loadLabels = function( callback )
	{
		var self = this;
		callback = callback || $.noop;

		var clone = inConcert.Utils.cloneObject;
		if (!this.Labels) {
			var uri = this.getBaseURI() + "/campaigns/get_labels/" + this.Id;
			inConcert.invoke({
				url: uri,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				timeout    : get_long_request_timeout_ms(),
				success: function(_) {
					callback( clone(self.Labels) );
				},
				error: function(undefined /*response*/, error) {
					self.Labels = null;
					callback(null, error.result);
				}
			});
		}
		else {
			if (typeof this.Labels === "string") {
				this.Labels = JSON.parse(this.Labels);
			}
			callback( clone(this.Labels) );
		}
	};

	Campaign.prototype.loadCustomLabels = function( callback )
	{
		callback = callback || $.noop;

		this.loadLabels(function ( labels, err )
		{
			if (!labels) {
				callback(false, err);
				return;
			}
			var custom_labels = {};
			$.each(labels || {}, function( label_id, label )
			{
				if (label.system !== "1") {
					custom_labels[label_id] = label;
				}
			});
			callback(custom_labels);
		});
	};

	Campaign.prototype.loadSystemLabels = function( callback )
	{
		callback = callback || $.noop;

		this.loadLabels(function ( labels, err )
		{
			if (!labels) {
				callback(false, err);
				return;
			}
			var system_labels = {};
			$.each(labels || {}, function( label_id, label )
			{
				if (label.system === "1") {
					system_labels[label_id] = label;
				}
			});
			callback(system_labels);
		});
	};

	Campaign.prototype.can_delete_labels = function( labels_to_remove )
	{
		var response = { success: true };
		if (!this.Labels) {
			response.success = false;
			response.result = "NO_CONFIGURED_LABELS";
			return response;
		}
		if (!$.isArray(labels_to_remove)) {
			response.success = false;
			response.result = "NO_LABELS_TO_DELETE";
			return response;
		}
		return response;
	};

	Campaign.prototype.deleteLabels = function( labels_to_remove, on_removed )
	{
		inConcert.logger.action({
			action_name : "deleteLabels",
			campaign    : this.Id,
			action_type : "campaign"
		});

		var self = this;
		on_removed = on_removed || $.noop;

		var execute_callback = function( undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			else {
				if ( self.Labels ) {
					$.each(labels_to_remove, function(_, att_level_id ) {
						delete self.Labels[att_level_id];
					});
				}
			}
			on_removed(error);
		};

		var result = this.can_delete_labels(labels_to_remove);
		if ( !result.success ) {
			inConcert.logger.error({
				action_name : "deleteLabels",
				campaign    : this.Id,
				action_type : "campaign",
				err_result  : result
			});
			execute_callback(undefined, result);
			return;
		}

		var uri = this.getBaseURI() + "/campaigns/delete_labels/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			data:        JSON.stringify({campaignId: this.Id, labels: labels_to_remove}),
			dataType:    "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success:     function( /*response*/ ) { execute_callback();            },
			error:       function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.hasTicketConfiguredInAccount = function( account_id, proxy_type )
	{
		var account = this.getAccount( account_id, proxy_type );
		if ( !account ) {
			return false;
		}
		var ticket_emitter = account.TicketEmitter;
		if ( ticket_emitter === undefined ) {
			return false;
		}
		if ( ticket_emitter === null ) {
			return false;
		}
		if ( typeof ticket_emitter === "string" ) {
			ticket_emitter = JSON.parse( ticket_emitter );
		}
		if ( $.isEmptyObject( ticket_emitter ) ) {
			return false;
		}
		return true;
	};

	Campaign.prototype.has_signature_configured_in_account = function( account_id, proxy_type )
	{
		var account = this.getAccount( account_id, proxy_type );
		if ( !account ) {
			return false;
		}
		var signature_id = account.Signature;
		if ( typeof signature_id !== "string" ) {
			return false;
		}
		return signature_id.length > 0;
	};

	Campaign.prototype.has_font_configured_in_account = function( account_id, proxy_type )
	{
		var account = this.getAccount( account_id, proxy_type );
		if ( !account ) {
			return false;
		}
		var font = account.Font;
		if ( typeof font !== "object" ) {
			return false;
		}
		var is_configured_attr = function(attr) {
			return (attr || "").trim().length > 0;
		};
		return is_configured_attr(font.family) || is_configured_attr(font.size) || is_configured_attr(font.color);
	};

	Campaign.prototype.saveRules = function(rules_update, onSaved)
	{
		var self = this;
		inConcert.logger.action({
			action_name : "save_campaign_rules",
			campaign    : this.Id,
			action_type : "campaign"
		});
		onSaved = onSaved || $.noop;
		var execute_callback = function(undefined /*code*/, error_type)
		{
			var error;
			if (error_type) {
				error = error_type;
			}
			else {
				self.Rules = self.Rules || {};
				$.each(rules_update, function(_, rule ) {
					self.Rules[rule.id] = rule;
				});
			}
			onSaved(error);
		};
		var uri = this.getBaseURI() + "/campaigns/save_rules/";
		inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify({campaignId: this.Id, rules: rules_update}),
			dataType    : "json",
			contentType : "application/json",
			timeout    : get_long_request_timeout_ms(),
			success     : function( /*response*/ ) { execute_callback();            },
			error       : function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.loadRules = function( callback, force_update )
	{
		var self = this;
		callback = callback || $.noop;

		var clone = inConcert.Utils.cloneObject;
		if (!this.Rules || force_update) {
			var uri = this.getBaseURI() + "/campaigns/get_rules/" + this.Id;
			inConcert.invoke({
				url: uri,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				timeout    : get_long_request_timeout_ms(),
				success: function(rules) {
					self.Rules = rules;
					callback( clone(self.Rules) );
				},
				error: function(undefined /*response*/, error) {
					self.Rules = null;
					callback(null, error.result);
				}
			});
		}
		else {
			if (typeof this.Rules === "string") {
				this.Rules = JSON.parse(this.Rules);
			}
			callback( clone(this.Rules) );
		}
	};

	Campaign.prototype.can_delete_rules = function( rules_to_remove )
	{
		var response = { success: true };
		if (!this.Rules) {
			response.success = false;
			response.result = "NO_CONFIGURED_RULES";
			return response;
		}
		if (!$.isArray(rules_to_remove)) {
			response.success = false;
			response.result = "NO_RULES_TO_DELETE";
			return response;
		}
		return response;
	};

	Campaign.prototype.deleteRules = function( rules_to_remove, on_removed )
	{
		inConcert.logger.action({
			action_name : "deleteRules",
			campaign    : this.Id,
			action_type : "campaign"
		});

		var self = this;
		on_removed = on_removed || $.noop;

		var execute_callback = function( undefined /*code*/, error_type)
		{
			if (error_type) {
				return on_removed(error_type);
			}
			if ( self.Rules ) {
				$.each(rules_to_remove, function(_, rule_id ) {
					delete self.Rules[rule_id];
				});
			}
			return on_removed();
		};

		var result = this.can_delete_rules(rules_to_remove);
		if ( !result.success ) {
			inConcert.logger.error({
				action_name : "deleteRules",
				campaign    : this.Id,
				action_type : "campaign",
				err_result  : result
			});
			execute_callback(undefined, result);
			return;
		}

		var uri = this.getBaseURI() + "/campaigns/delete_rules/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			data:        JSON.stringify({campaignId: this.Id, rules: rules_to_remove}),
			dataType:    "json",
			contentType: "application/json",
			timeout    : get_long_request_timeout_ms(),
			success:     function( /*response*/ ) { execute_callback();            },
			error:       function(code, error)    { execute_callback(code, error); }
		});
	};

	Campaign.prototype.isAllegro = function() {
		var is_allegro = false;
		if (!inConcert.app.has_allegro_integration()) {
			return is_allegro;
		}
		if (!$.isArray(this.accounts)) {
			return is_allegro;
		}
		this.accounts.forEach(function(account)
		{
			if (inConcert.app.is_allegro_channel(account.proxyType)) {
				is_allegro = true;
				return false;
			}
		});
		return is_allegro;
	};

	Campaign.prototype.hasTelephonyChannel  = function() {
		let has_configured = false;
		if (!$.isArray(this.accounts)) {
			return has_configured;
		}
		this.accounts.forEach(account => {
			if (inConcert.app.is_call_channel(account.proxyType)) {
				has_configured = true;
				return false;
			}
		});
		return has_configured;
	};

	Campaign.prototype.getIncomingConfig = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account || !account.incomingConfig) {
			return null;
		}
		if (typeof account.incomingConfig === "string") {
			account.incomingConfig = JSON.parse(account.incomingConfig);
		}
		return account.incomingConfig;
	};

	Campaign.prototype.getOutgoingConfig = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account || !account.outgoingConfig) {
			return null;
		}
		if (typeof account.outgoingConfig === "string") {
			account.outgoingConfig = JSON.parse(account.outgoingConfig);
		}
		return account.outgoingConfig;
	};

	Campaign.prototype.isInboundAccount = function(account) {
		var incomingConfig = this.getIncomingConfig(account);
		switch (account.proxyType) {
			case "CALL":
				return incomingConfig && incomingConfig.enabled || false;
			default:
				return true;
		}
	};

	Campaign.prototype.isOutboundAccount = function(account) {
		var outgoingConfig = this.getOutgoingConfig(account);
		switch (account.proxyType) {
			case "CALL":
				return outgoingConfig && outgoingConfig.enabled || false;
			case "CHAT":
				return false;
			default:
				return true;
		}
	};

	Campaign.prototype.isInboundRedialEnabled = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account) {
			return false;
		}
		var incomingConfig = this.getIncomingConfig(account);
		return this.isInboundAccount(account) && incomingConfig.ContactCenter && incomingConfig.ContactCenter.CheckRedial;
	};

	Campaign.prototype.isCallbackEnabled = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account) {
			return false;
		}
		return parseInt(account.CallbackIsAllowed, 10) === 1;
	};

	Campaign.prototype.isCallbackEnabledForAbandoned = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account) {
			return false;
		}
		return this.isCallbackEnabled(account) && parseInt(account.LifeTimeAbandonedCalls, 10) > 0;
	};

	Campaign.prototype.isCallbackEnabledForAppointment= function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account) {
			return false;
		}
		return this.isCallbackEnabled(account) && parseInt(account.LifeTimeAppointmentCalls, 10) > 0;
	};

	Campaign.prototype.canCreateOutboundInteraction = function(account) {
		account = (account && typeof account === "object") ? account : this.getAccount(account);
		if (!account) {
			return false;
		}
		var outgoingConfig = this.getOutgoingConfig(account);
		switch (account.proxyType) {
			case "CALL":
				return this.isOutboundAccount(account) && outgoingConfig.ContactCenter && outgoingConfig.ContactCenter.AllowManualCalls;
			case "CHAT":
				return false;
			default:
				return true;
		}
	};

	Campaign.prototype.loadTemplateMessageOptions = function(accountId, page, cache, callback) {
		var account = this.getAccount(accountId);
		if (!account) {
			return callback(false, "account not found");
		}
		var uri = inConcert.app.getBaseDomain() + "/inconcert/apps/agent/campaigns/get_template_message_options/";
		inConcert.invoke({
			url:         uri,
			type:        "POST",
			timeout    : get_long_request_timeout_ms(),
			data:        JSON.stringify({
				proxyType : account.proxyType,
				campaignAtVCC : this.Id,
				account : accountId,
				page : page,
				cache : cache
			}),
			dataType:    "json",
			contentType: "application/json",
			success:     function(data) { 
				callback(data.options);
			},
			error:       function( code, error ) { 
				callback(false, { code: code, error: error });
			}
		});
	};

	Campaign.prototype.getAccountName = function(account) {
		if (!account) {
			return "";
		}
		switch(account.proxyType) {
			case "TWITTER":
				return account.screen_name;
			default:
				return account.name || "";
		}
	};

	Campaign.prototype.getWhatsappAccountIntegrationType = function(account) {
		return account? (account.integrationType || "WEB") : "";
	};

	Campaign.prototype.hasAccountTemplateMessageOptions = function(account/*, page*/) {
		if (!account) {
			return false;
		}
		switch(account.proxyType) {
			case "WHATSAPP":
				var integrationType = this.getWhatsappAccountIntegrationType(account);
				switch(integrationType) {
					case "WEB":
						return false;
					default:
						return account.HSMConfig && account.HSMConfig.Enabled === "1" && account.HSMConfig.TemplateMessages && Object.keys(account.HSMConfig.TemplateMessages).length;
				}
				break;
			default:
				return false;
		}
	};

	Campaign.prototype.accountIsOperational = function(account_id) {
		const accountResponse = {
			success: false,
			errorMessage: null
		};
		const accounts = this.get_accounts() || [];

		for (let ai = 0; ai < accounts.length; ai++) {
			const account = accounts[ai];
			if (account_id !== account.id) {
				continue;
			}
			if (account.state === "0" || !account.status || account.status.status !== "OK") {
				if (account.status && account.status.status !== "OK") {
					accountResponse.errorMessage = inConcert.app.getAccountStatusLabel(account.proxyType, account.status.status);
				}
			}
			else {
				accountResponse.success = true;
			}
			break;
		}

		return accountResponse;
	};

	Campaign.prototype.hasAccountWhatsappGroups = function(account) {
		const whatsappIntegrationsWithGroups = {
			WAVY: true
		};
		return whatsappIntegrationsWithGroups[this.getWhatsappAccountIntegrationType(account)] || false;
	};

	Campaign.prototype.isInternal = function() {
		return this.IsInternal === "1";
	};

}());
