/*jshint quotmark:false, esversion: 9*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	/**
	* Class CategorieUI
	*/
	function CategorieUI (options)
	{
		CategorieUI.uber.constructor.call(this);

		this.parent = options.parent;
		this.interaction_id = options.interaction_id;
		this.canReposition = true;
		this.m_conversations_navigation = options.conversations_navigation;

		this.ownerId = "windowCategoryContainer";
	}

	inConcert.Utils.inherit( CategorieUI, inConcert.PopupWindow );

	if ( typeof inConcert.CategorieUI !== "function" ) {
		inConcert.CategorieUI = CategorieUI;
	}

	CategorieUI.prototype.initialize = function ()
	{
		const $windowLabelContainer = $("#windowLabelContainer");
		$windowLabelContainer.remove();
		const node = $(this.createMarkup());
		const $body = $("body");
		$body.append(node);
		this.data.container = node;
		this.parent.show();
		this.setEvents();
		const $searchCategoryData = $("#searchCategoryData");
		$searchCategoryData.focus();
		let campaigns = inConcert.app.getCampaigns();
		let merged_campaigns_labels = {};
		const self = this;
		$.each(campaigns, function(_, campaign)
		{
			const campaign_id = campaign.Id;
			if(self.m_conversations_navigation.campaigns[campaign_id])
			{
				campaign.loadLabels(function(labels)
				{
					$.each(labels || [], function(_, label)
					{
						const cloned_label = inConcert.Utils.cloneObject(label);
						merged_campaigns_labels[label.name] = merged_campaigns_labels[label.name] || label;
						merged_campaigns_labels[label.name].campaigns = merged_campaigns_labels[label.name].campaigns || {};
						merged_campaigns_labels[label.name].campaigns[campaign_id] = cloned_label;
					});
				});
			}
		});

		self.add_campaigns_labels(merged_campaigns_labels, true /*cleanup*/);

		this.__clean_button_categorie();
		const $labels = $('#windowCategoryListLabels').children();
		$.each( $labels, function( _, label ){
			const $label = $(label);
			const $labelText = $label.text();
			if(self.m_conversations_navigation.isLabelSelected($labelText)){
				$label
					.addClass('selected')
					.children()
					.children()
					.children()
					.prop('checked',true);			
			}
		});
	};

	CategorieUI.prototype.close = function ()
	{
		this.data.container.remove();
		this.finalize();
	};

	CategorieUI.prototype.add_campaigns_labels = function( labels, cleanup )
	{
		if (!labels) {
			return;
		}
		const $labels = $('#windowCategoryListLabels');
		if (cleanup) {
			$labels.html('');
		}

		const sorted_labels = inConcert.Utils.hash_to_sorted_array(labels, function(label) { return label.name; }) || {};

		$.each(sorted_labels || {}, function( _, label )
		{
			const $already_exist = $labels.find('[label_id="' + label.name + '"]');
			if ( $already_exist.length > 0 ) {
				let current_campaigns = $already_exist.data('campaigns');
				current_campaigns = $.extend(current_campaigns, label.campaigns);
				$already_exist.data('campaigns', current_campaigns);
				return true; // continue
			}
			const random_id = inConcert.Utils.generateGuid();
			const color = label.color || "#FFF5EB";
			const $label = $(
				'<li class="categoryListItem">' +
					'<div class="form-check">' +
						'<label for="' + random_id + '" label_name="true" class="form-check-label form-check-label-title">' +
							'<input id="' + random_id + '" selector="true" class="form-check-input" type="checkbox" />' +
							'<div label_color="true" class="categoryLabelColor"></div>'+
							'<span></span>' +
						'</label>' +
					'</div>' +
				'</li>'
				// '<li class="navigationTagItem">' +
				// 	'<input id="' + random_id + '" selector="true" type="checkbox" class="tagItemCheck" />' + 
				// 	'<label for="' + random_id + '" label_name="true" class="tagItemText"></label>' +
				// '</li>'
				);
			$label.attr('label_id', label.name);
			$label.find('[selector="true"]').attr('label_id', label.name);
			$label.find('[label_name="true"] span').text(label.name);
			$label.find('[label_color="true"]').css({'backgroundColor': label.color || "#FFF5EB"});
			// $label.find('[label_name="true"]').text(label.name);
			$label.data('campaigns', label.campaigns);
			$labels.append($label);
			$label.find('[id="' + random_id + '"]').on('click', function()
			{
				const $this = $(this);
				// const $parent = $this.parent();
				const $parent = $this.parent().parent().parent();
				$parent.removeClass('selected');
				if ( $this.prop('checked') ) {
					$parent.addClass('selected');
				}
			});
		});
	};

	CategorieUI.prototype.createMarkup = function ()
	{
		return "".concat(
		"<div id='windowCategoryContainer' class='systemPopWindow windowSearchPopoverContainer windowContainer skinLight' style='top:auto !important;'>",
			/*"<div class='windowTop'>",
				"<div class='row'>",
					"<div class='col-12 d-flex flex-row align-items-center'>",
						"<span class='windowTitle mb-0'>", "<%=_T('Categories')%>", "</span>",
						"<button id='cleanCategoryDialog' class='pageButtonGeneric buttonColorBlack'>" + "<%=_T('Clean')%>" + "</button>",
					"</div>",
				"</div>",
				
			"</div>",
			*/
			"<div class='windowMiddle windowTicketsResultsContent'>",
				"<div class='row searchFilterRow'>",
					"<div class='col-12 pl-0 pr-0'>",
						"<div class='searchFilter'>",
							"<input  id='searchCategoryData' class='ticketSearchField filterField' type='text' placeholder='", "<%=_T('Search')%>...", "' />",
							'<button id="searchCategoryButton" class="buttonToggle buttonToggleOutlined search"><i class="far fa-search"></i></button>',
						"</div>",
					"</div>",
				"</div>",
				"<div class='row'>",
					"<div class='col-12'>",
						"<div id='windowCategoryList' class=''>",/*class interactionMenuBlind */
							"<ul id= 'windowCategoryListLabels' labels='true' class='list-unstyled mb-0'></ul>",/*se saco esta clase que no dejaba que se visualizara la ul, revisar. interactionNavigationTags */
							"<div class='noTagDisclaimer' hidden><%=_T('No tags created yet.')%></div>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='windowBottom'>",					
				"<button id='cleanCategoryDialog' class='pageButtonGeneric buttonColorWhite buttonSize pageButtonGenericHeight2 pageButtonGenericWidth2'>" + '<i class="far fa-eraser"></i>' + "</button>",
				"<button id='closeCategoryDialog' class='pageButtonGeneric buttonColorWhite toRight pageButtonGenericHeight2'>" + "<%=_T('Close')%>" + "</button>",
				"<button id='confirmCategoryDialog' class='pageButtonGeneric buttonColorOrange pageButtonNew ml-2 pageButtonGenericHeight2'>" + "<%=_T('Confirm')%>" + "</button>",
			"</div>",
		"</div>",
		);
	};

	CategorieUI.prototype.get_selected_labels = function()
	{	
		const $windowCategoryListLabels = $("#windowCategoryListLabels");
		const selected_labels = [];
		$windowCategoryListLabels
			.children()
			.each(( _, label ) => {
				const $label = $(label);
				if( $label.hasClass('selected') ){
					const $labelText = $label.text();
					selected_labels.push($labelText);
				}	  
		});
		return selected_labels;
	};
	CategorieUI.prototype.__clean_button_categorie = function()
	{	
		const self = this;
		const $cleanCategoryDialog = $("#cleanCategoryDialog")
			.off('click')
			.on('click',function (){
				self.m_conversations_navigation.clearSelectedLabels();
				const $windowCategoryListLabels = $("#windowCategoryListLabels");
				let hasToRefresh = false;
				$windowCategoryListLabels
					.children()
					.each(( _, label ) => {
						const $label = $(label);
						if( $label.hasClass('selected') ){
							$label.removeClass('selected');
							$label
								.children()
								.children()
								.children()
								.prop('checked',false);
							hasToRefresh = true;
						}	  
				});
				if(hasToRefresh) {
					self.m_conversations_navigation.callbacks[self.m_conversations_navigation.current_item].filter_label();
					const $selectedCategoriesCounter = self.parent.find("[selected_categories_counter='true']");
					const selectedLabels = self.m_conversations_navigation.get_selected_labels();
					if(selectedLabels.length > 0) 
					{
						self.parent.addClass("selectedItemsActive");
						$selectedCategoriesCounter.text(selectedLabels.length);
					}
					else
					{
						self.parent.removeClass("selectedItemsActive");
						$selectedCategoriesCounter.text("");
					}
				}
		});
		return $cleanCategoryDialog;
	};

	CategorieUI.prototype.setEvents = function ()
	{
		const self = this;
		const searchFn = self.__search.bind(this);
		$("#searchCategoryButton")
			.off('click')
			.on('click', searchFn);

		$("#searchCategoryData").keyup(function(evt){
			if(evt.keyCode === 13) {
				searchFn();
				return;
			}
		});
		$("#closeCategoryDialog")
			.off('click')
			.on('click', function() {
				/*const $activeInteractionSearchMessageInfo = $("#activeInteractionSearchMessageInfo");
				$activeInteractionSearchMessageInfo.show();
				$("#active_pill_0_label").hide();*/
				self.close();
		});
		const confirmButtonClick = $("#confirmCategoryDialog");
		confirmButtonClick
			.off('click')
			.on('click', function() {
				self.__filter_categorie();
				self.m_conversations_navigation.callbacks[self.m_conversations_navigation.current_item].filter_label();
			});
	};
	CategorieUI.prototype.__filter_categorie = function (){
		const self = this;
		const $windowCategoryListLabels = $("#windowCategoryListLabels");
				$windowCategoryListLabels
					.children()
					.each(( _, label ) => {
						const $label = $(label);
						const $labelText = $label.text();
						if( $label.hasClass('selected') ){
							self.m_conversations_navigation.addSelectedLabel($labelText);
						}
						else {
							self.m_conversations_navigation.delSelectedLabel($labelText);
						}
				});
			
		const $selectedCategoriesCounter = this.parent.find("[selected_categories_counter='true']");
		const selectedLabels = self.m_conversations_navigation.get_selected_labels();
		if(selectedLabels.length > 0) 
		{
			this.parent.addClass("selectedItemsActive");
			$selectedCategoriesCounter.text(Object.keys(selectedLabels).length);
		}
		else
		{
			this.parent.removeClass("selectedItemsActive");
			$selectedCategoriesCounter.text("");
		}
	};
	
	CategorieUI.prototype.__render_results = function (interactions)
	{
		const interCategories = [];
		interactions.forEach(function(interaction){
			const timeStamp = inConcert.Utils.formatDate(interaction.timestampDouble * 1000);
			const interCategorie = {
				type : interaction.type,
				timestamp: timeStamp,
				state: interaction.state,
				assignedAgents: interaction.assignedAgents,
				contactName: interaction.interactionData.contactName,
				campaign: interaction.campaign
			};
			interCategories.push(interCategorie);
		});
		const template =
			"<ul class='searchResultsList list-unstyled'>" +
			"	{{#.}}" + // for each element in the array
			"	<li class='searchResultsItem'>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Type')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo interactionType{{type}}'>{{type}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Date')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{timestamp}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('State')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo interactionState{{state}}'>{{state}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Assigned to')%>" + "<%=_T(':')%>" + "</label> <label  class='resultInfo'>{{assignedAgents}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Contact')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{contactName}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Campaign')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{campaign}}</label></div>" +
			"	</li>" +
			"	{{/.}}" +
			"	{{^.}}" + // this renders when the array is empty 
			"		<li class='noResultsAvailable'><span class='inlineMessagesContainer messageInfo d-flex align-items-center windowLabelNoResults'>" + "<%=_T('No matching conversation. Make sure you entered the whole ticket (including prefix and suffix).')%>" + "</span></li>" + 
			"	{{/.}}" +
			"</ul>";
			
		const markup = Mustache.render(template, interCategories);

		$(".windowTicketsResultsContent > div > div").html("").append(markup);

	};

	/**
	* Performs a search based on the value of the INPUT field
	*/
	CategorieUI.prototype.__search = function ()
	{
		let search_data = $("#searchCategoryData").val().trim();
		const $windowCategoryMessageEmpty = $("#windowCategoryMessageEmpty");
		const $windowCategoryListLabels = $("#windowCategoryListLabels");
		if ( search_data.length === 0 ) {
			$windowCategoryListLabels
				.children()
				.show();
			$windowCategoryMessageEmpty.hide();
			return;
		}
		if ( search_data.length > 0 ) {
			
			let countHideLabel = 0;
			$windowCategoryListLabels
				.children()
				.each(( _, label ) => {
					const $label = $(label);
					const $labelText = $label.text();
					if($labelText.indexOf(search_data) !== -1) {
						$label.show();
						}
					else {
						$label.hide();
						countHideLabel++;
					}
					
					 
			});
			if( countHideLabel  === $windowCategoryListLabels.children().length ){
				const messageEmpty = `<li id ="windowCategoryMessageEmpty" class='noResultsAvailable'><span class='inlineMessagesContainer messageInfo d-flex align-items-center windowLabelNoResults'><%=_T('There are no matching categories.')%></span></li>`;
				const $windowCategoryMessageEmpty= $("#windowCategoryMessageEmpty");
				if($windowCategoryMessageEmpty.length == 0){
					$windowCategoryListLabels.append(messageEmpty);
				}else{
					$windowCategoryMessageEmpty.show();
				}
			}else{
				$windowCategoryMessageEmpty.hide();
			}
			return;
		}
	};

}());