/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:single, esversion: 11 */

/* global $, inConcert */

/*
Handling of the overlaying popup with the comments section of each interaction
*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	var _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;
	var m_maxCharsOnComment = 500;

	/**
	Class CommentsPopup.
	*/
	function CommentsPopup ( params )
	{
		CommentsPopup.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + 'comment';
		this.ownerId = 'windowCommentContainer';
		this.canReposition = true;
		this.from_app = params ? params.from_app : 'interaction';
	}

	inConcert.Utils.inherit(CommentsPopup, inConcert.PopupWindow);

	if (typeof inConcert.CommentsPopup !== 'function') {
		inConcert.CommentsPopup = CommentsPopup;
	}

	CommentsPopup.prototype.canComment = function( interaction )
	{
		var response = {
			success: false
		};
		if ( !interaction ) {
			response.err_type = 'INTERACTION_NOT_EXIST';
			return response;
		}
		if ( interaction.isFinished() ) {
		// 1-no puede comentar porque la interaccion se encuentra archivada
			response.err_type = 'INTERACTION_IS_FINISHED';
			return response;
		}
		if ( !interaction.hasPermission( 'COMMENT', this.from_app ) ) {
		// 2-no puede comentar porque no tiene permisos para comentar en la app actual
			response.err_type = 'FORBIDDEN';
			return response;
		}
		if ( interaction.belongsToMe() && ( this.from_app || '' ).toLowerCase() === 'supervisor' ) {
		// 3-no puede comentar porque la interaccion me pertenece y me encuentro en la app 'supervisor'
			response.err_type = 'UNABLE_TO_COMMENT_MY_INTERACTIONS_ON_CURRENT_APP';
			return response;
		}

		// Congratulations !!! si puedes comentar
		response.success = true;
		return response;
	};

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	CommentsPopup.prototype.createMarkup = function( interaction ) {
		//In case there's a current node opened with a comment in this window, prune it
		$('#windowCommentContainer').remove();
		var pendingComment = interaction.pending_comment ? interaction.pending_comment : '';
		//create the markup everytime
		var $node = $(''.concat(
			'<div class="systemPopWindow windowContainer  windowCommentContainer popUp380w" style="display:none" id="windowCommentContainer">', 
				 '<div class="windowTop">',
				 	'<div  class="windowTitle windowComments">', 
					 	'<span id="windowCommentTitle" ><%=_T("Add Comments")%>', 
						'</span>',
						'<button id="closeInteractionComment" class="internalPopUpCloseButton">',
							'<i class="far fa-times">',
							'</i>',
						'</button>',
			
					'</div>',
				 '</div>',

				'<div id="commentWindowMiddle" class="windowMiddle">',
					'<div class="addCommentInputContainer">',
						//'<label for="newCommentText">', '<%=_T("Comment")%>', ':</label>',
						'<textarea id="newCommentText" class="addCommentInput form-control" placeholder="', '<%=_T("Comment")%>', ' ">', 
						pendingComment, '</textarea>',
						'<label class="commentClientCharacterCounter" id="characterCounterComment"></label>',
					'</div>',

					'<div class="addCommentButtonsContainer align-items-center justify-content-end">',
						'<input class="pageButtonGeneric buttonColorWhite ml-2" id="cancelInteractionComment" disabled type="button" value="', '<%=_T("Clean")%>', '"/>',
						'<input class="pageButtonGeneric buttonColorBlue" id="newInteractionComment" type="button" value="', '<%=_T("Add")%>', '"/>',
						//'<label class="commentClientCharacterCounter" id="characterCounterComment"></label>',
					'</div>',
				'</div>',
				'<div class="addCommentThreadContainer" style="display:none">',
					'<div class="addCommentThreadContentScroll"></div>',
				'</div>',
				//'<div class="windowBottom">',
					//'<input class="pageButtonGeneric buttonColorWhite" id="closeInteractionComment" type="button" value="', '<%=_T("Close")%>', '"/>',
				//'</div>',
			'</div>'));
		//finished interactions have no new comments, TODO
		if ( !this.canComment( interaction ).success ) {
			$node.find('.addCommentInputContainer, #newInteractionComment, #characterCounterComment, #cancelInteractionComment, #commentWindowMiddle').hide();
			$node.find( '#windowCommentTitle' ).text( _T('Comments' ) );
		}
		$('body').append($node);
		return $node;
	};

	/*******************************************************************
	 * Metodo que visualiza los comentarios e inicializa eventos de la ventana!
	 */
	CommentsPopup.prototype.initialize = function()
	{
		var interaction = this.data.interaction;
		this.data.container = this.createMarkup(interaction);
		this.parent_on_comment = this.data.callbacks.on_comment || $.noop;
		var threadContainer = this.data.container.find('.addCommentThreadContainer');
		this.fillComments(interaction, threadContainer);
		this.setEvents(interaction);
		this.interaction = interaction;
		$('#characterCounterComment').text(m_maxCharsOnComment);
	};

	CommentsPopup.prototype.scroll = function( callback )
	{
		callback = callback || $.noop;
		// animate scroll to bottom
		var $scroll = $('.addCommentThreadContentScroll');
		var animation = {scrollTop: 0};
		var scroll_height = $scroll.prop('scrollHeight');
		var height = $scroll.height();
		animation.scrollTop = scroll_height - height;
		var time_ms = 250;
		$scroll.animate(animation, time_ms, 'linear',
			function() {
				callback();
			});
	};

	CommentsPopup.prototype.ready = function()
	{
		var self = this;

		var focus = function() {
			if ( self.canComment( self.interaction ).success ) {
				$('#newCommentText').focus();
			}
			else {
				$('#cancelInteractionComment').focus();
			}
		};

		this.scroll( focus );
	};

	CommentsPopup.prototype.close = function(params) {
		var commentText = $('#newCommentText').val();
		if (commentText && commentText.length > 0) {
			this.interaction.pending_comment = commentText;
		}
		if (this.data && this.data.container) {
			this.data.container.remove();
		}
		this.finalize(params);
	};

	CommentsPopup.prototype.showMessage = function( err )
	{
		var message = {
			message     : '<%=_T("Unable to comment conversation.")%>',
			messageType : 'Error',
			timeout     : 5000
		};

		switch( ( err || '' ).toUpperCase() )
		{
			case 'FORBIDDEN':
				message.message = format('<%=_T("Unable to comment conversation. You have not permissions on {0} campaign.")%>', this.interaction.campaign);
				break;

			case 'INTERACTION_NOT_EXIST':
				message.message = '<%=_T("Unable to comment conversation. Conversation does not exist.")%>';
				break;

			case 'INTERACTION_IS_FINISHED':
				message.message = '<%=_T("Unable to comment archived conversations.")%>';
				break;

			case 'UNABLE_TO_COMMENT_MY_INTERACTIONS_ON_CURRENT_APP':
				message.message = '<%=_T("Unable to comment your conversations on the current application.")%>';
				break;

			case 'NOT_WRITE_IN_COMMENT':
				message.message = '<%=_T("Please enter the comment")%>';
				break;
		}

		inConcert.app.m_mainWindow.showBannerMessage( message, null, true );

		return message.message;
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	CommentsPopup.prototype.setEvents = function(interaction)
	{
		var self = this;
		//---------------
		//bind the new comment handler for this interaction
		$('#newInteractionComment')
			.unbind('click')
			.click(function() {
				var response = self.canComment( interaction );
				if ( !response.success ) {
					self.showMessage( response.err_type );
					return;
				}
				inConcert.logger.guiAction({action_name : 'add_comment', window_id: window.id}, interaction);
				var newCommentTextArea = document.getElementById('newCommentText');
				var commentText = newCommentTextArea.value;

				if (commentText && commentText.length > 0) {
					$('#newInteractionComment').prop('disabled', true);

					var comment_callback = function(success, error)
					{
						var params = {};
						if (success) {
							newCommentTextArea.value = '';
							self.interaction.pending_comment = undefined;
						}
						else {
							if (typeof error === 'string') {
								if ( error ) {
									params.errorMessage = self.showMessage( error );
								}
							}
						}
						self.parent_on_comment(success, error);
						self.close(params);
					};

					interaction.comment( commentText, comment_callback, self.from_app );
				}else {
					self.showMessage( 'NOT_WRITE_IN_COMMENT' );
				}
			});
		//---------------
		const $cancelInteractionComment = $('#cancelInteractionComment');
		$cancelInteractionComment.unbind('click').click(function() {
			inConcert.logger.guiAction({action_name : 'cancel_comment', window_id: window.id},interaction);
			document.getElementById('newCommentText').value = '';
			self.interaction.pending_comment = undefined;
			$('#characterCounterComment').text(m_maxCharsOnComment);
			$cancelInteractionComment.prop('disabled', true);
		});
		$('#closeInteractionComment').unbind('click').click(function() {
			inConcert.logger.guiAction({action_name : 'cancel_comment', window_id: window.id},interaction);
			document.getElementById('newCommentText').value = '';
			self.interaction.pending_comment = undefined;
			self.close();
		});
		//---------------
		const $newCommentText = $('#newCommentText');
		$newCommentText.unbind('keyup').bind('keyup', function( /*event*/ ) {
			var textArea = $(this);
			var currentChars = document.getElementById(textArea.attr('id')).value.length;
			const $cancelInteractionComment = $('#cancelInteractionComment');
			if(currentChars > 0) {
				$cancelInteractionComment.prop('disabled', false);
			}
			else {
				$cancelInteractionComment.prop('disabled', true);
			}
			var charsDifference = m_maxCharsOnComment - currentChars;
			var disabledTweetButton = charsDifference < 0;
			$('#characterCounterComment').text(charsDifference)[disabledTweetButton ? 'addClass' : 'removeClass']('commentClientCharacterCounterNegative');
			$('#newInteractionComment').prop('disabled', disabledTweetButton);
		});
	};

	/*******************************************************************
	 * Metodo que crea el markup de todos los comentarios y los renderiza
	 */
	CommentsPopup.prototype.fillComments = function(interaction, threadContainer) {
		var threadList = threadContainer.find('.addCommentThreadContentScroll');
		threadList.children().remove();
		//----------------------------------------
		// función que retorna el markup de un comentario
		var getMarkupFn = function(comment, index) {
			var getAvatarURI = function() {
				var memberId = inConcert.app.get_id(comment.author || '');
				return inConcert.app.getBaseUrl() + '/users/get_avatar/' + memberId;
			};
			return ''.concat(
				'<div id="', index, '" class="interactionCommentEvent">',
					'<div class="commentAutorAvatar">',
						'<object data="', getAvatarURI(), '" type="image/png" width="24" height="24">',
							'<img src="/inconcert/apps/agent/images/genericAvatar.png" width="24" height="24">',
						'</object>',
					'</div>',
					'<div class="commentInfoContainer">',
						'<div class="commentInfoTop">',
							'<label class="commentAutorName">', comment.authorName, '</label>',
							'<label class="commentMessageDate">', inConcert.Utils.timeAgo(comment.comment_date), '</label>',
						'</div>',
						'<label class="commentMessageText"></label>',
					'</div>',
				'</div>');
		};
		//----------------------------------------
		//if there are existing comments, build the list and show it
		var comments = interaction.getComments();
		if (comments && comments.length > 0) {
			$.each(comments, function(i, comment) {
				threadList.append(getMarkupFn(comment, i));
				var commentText = inConcert.Utils.nl2br(inConcert.Utils.escapeAttrib(comment.comment));
				commentText = inConcert.Utils.linkify(commentText, {exclude: {email: true}});
				threadList.find('#' + i + ' > .commentInfoContainer > .commentMessageText').html(commentText);
			});
			threadContainer.show();
		}
		else {
			threadContainer.hide();
		}
	};
}());
