/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const FORMAT = inConcert.Utils.format;

	//===================================================================================================
	//===================================================================================================
	/**
	Class conference base.
	@param params
	*/
	function conference_base ( params )
	{
		this.parent = params.parent;
		this.$tab_container = undefined;
		this.selected_item_id = null;
		this.jquery_txt_search_selector = "input[txt_search='true']";
		this.total_rows = 0;
		this.total_message_one  = "<%=_T('{0} row.')%>";  // el por defecto
		this.total_message_many = "<%=_T('{0} rows.')%>"; // el por defecto
		this.load_fn = null;
		this.is_loaded = false;
		this.m_has_allegro_integration = inConcert.app.has_allegro_integration();
	}

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.initialize = function()
	{
		var initializing = false;
		var result = this.is_enabled();
		if ( result.enabled ) {
			initializing = true;
		}
		else {
			this.disable( result.reason );
		}
		this.set_events();
		return initializing;
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.disable = function()
	{
		var $tab_container = this.$get_tab_container();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.hide_list( "<%=_T('Conference Disabled')%>" );
		$search.prop("disabled", true);
	};

	conference_base.prototype.enable = function()
	{
		var $tab_container = this.$get_tab_container();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.show_list();
		$search.prop("disabled", false).focus();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.is_enabled = function()
	{
		return { enabled : true };
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.is_visible = function()
	{
		return { visible : true };
	};

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_base.prototype.load = function( callback )
	{
		var interactions = this.parent.data.interactions;
		if ( !this.load_fn ) {
			throw "conference_base.load - this.load_fn is required!!";
		}
		callback = callback || $.noop;
		var interactions_ids = _.pluck( interactions , "id");
		this.load_fn( interactions_ids, callback );
	};

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_base.prototype.on_finish_load = function( callback )
	{
		callback = callback || $.noop;
		this.is_loaded = true;
		this.set_totals();
		if ( this.total_rows === 0 ) {
			this.disable();
		}
		else {
			this.enable();
		}
		this.parent.hide_waiting();
		callback();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.on_switch_tab = function()
	{
		this.parent.enable_disable_conference();
		var $tab_container = this.$get_tab_container();
		$tab_container.find(this.jquery_txt_search_selector).focus();
		//this.parent.setPosition();
	};

	/*******************************************************************
	* Metodo que xxx
	* @param id
	*/
	conference_base.prototype.get_radio_dom_id = function( id )
	{
		return this.prefix_radio_id + inConcert.Utils.getNormalizedId(id);
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.$get_tab_container = function()
	{
		if ( !this.$tab_container ) {
			this.$tab_container = this.parent.data.container.find(this.jquery_tab_selector);
		}
		return this.$tab_container;
	};

	conference_base.prototype.get_selected_item_id = function()
	{
		return this.selected_item_id;
	};

	conference_base.prototype.make_detail = function() { return false; };

	conference_base.prototype.show_detail = function()
	{
		var $detail_list = this.$get_tab_container().find("[detail_list='true']");
		if ( !$detail_list || $detail_list.length === 0 ) {
			return;
		}
		if ( !this.selected_item_id ) {
			$detail_list.html("");
			return;
		}
		var $item_container = $("#" + this.selected_item_id).parent();
		if ( !$item_container.is(":visible") ) {
			$detail_list.html("");
			return;
		}
		var item_data = $item_container.data("row");
		if ( !item_data ) {
			$detail_list.html("");
			return;
		}
		if ( !this.make_detail(item_data) ) {
			$detail_list.html("");
		}
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.on_selected_item = function( $new_selected_item /* radio buttom */ )
	{
		if ( this.selected_item_id !== null ) {
			$("#" + this.selected_item_id).parent().removeClass("selected");
		}
		this.selected_item_id = $new_selected_item.attr("for");
		const $conference_to = $("#" + this.selected_item_id);
		$conference_to.prop("checked", true).parent().addClass("selected");

		let conference_to = $conference_to.attr("conference_to");
		const $tab_container = this.$get_tab_container();
		$tab_container.find(this.jquery_txt_search_selector).val(conference_to);

		this.show_detail();
		this.parent.enable_conference();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.set_events = function()
	{
		var self = this;
		var $tab_container = this.$get_tab_container();

		// click en la row del item
		$tab_container
			.off("click", this.jquery_item_selector)
			.on("click", this.jquery_item_selector, function () {
				self.on_selected_item($(this));
			});
		// dbl click en la row del item
		$tab_container
			.off("dblclick", this.jquery_item_selector)
			.on("dblclick", this.jquery_item_selector, function () {
				const $this = $(this);
				self.parent.conference_conversations($this);
			});

		var search = function(text, $txt) {
			var callback = function() {
				$txt.prop("disabled", false);
				$txt.focus();
			};
			$txt.prop("disabled", true);
			self.filter(text, callback);
			window.clearTimeout(self.timeout);
			self.timeout = null;
		};

		// evento keyup del text de search
		$tab_container
			.off("keyup", this.jquery_txt_search_selector)
			.on("keyup", this.jquery_txt_search_selector, function(e)
			{
				var $txt = $(this);
				var text = $txt.val();
				if (self.timeout) {
					window.clearTimeout(self.timeout);
					self.timeout = null;
				}
				if (text.length === 0) {
					search(text, $txt);
				}
				else {
					if (e.keyCode === 13) {
						// enter
						search(text, $txt);
					}
					else if (e.keyCode === 27) {
						// escape
						search("", $txt);
					}
					else {
						self.timeout = window.setTimeout(function() {
							search(text, $txt);
						}, 400);
					}
				}
			});
	};

	/*******************************************************************
	* Metodo que xxx
	* @param word
	*/
	conference_base.prototype.make_search_regexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.set_totals = function()
	{
		var text = this.total_rows === 1 ? this.total_message_one : this.total_message_many;
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[total='true']").text(FORMAT(text, this.total_rows));
	};

	conference_base.prototype.filter = function(text, callback)
	{
		var self = this;
		callback = callback || $.noop;

		var $tab_container = this.$get_tab_container();
		var $list_container = $tab_container.find("[list_container='true']");
		var waiting_msg;

		if (text.trim().length === 0) {
		// si no hay texto buscado en el filtro, no filtro la row
			waiting_msg = "<%=_T('Removing search')%>";
			this.parent.show_waiting(waiting_msg);
			// pequeno timeout para meter feedback y que se pueda visualizar el waiting
			window.setTimeout(function() {
				$list_container.find(self.jquery_item_selector).show();
				self.show_list();
				callback();
			}, 10);
			return;
		}

		waiting_msg = FORMAT("<%=_T('Searching: {0}')%>", text);
		this.parent.show_waiting(waiting_msg);
		// pequeno timeout para meter feedback y que se pueda visualizar el waiting
		window.setTimeout(function() {
			var hidden_rows = 0;
			$list_container.find(self.jquery_item_selector).each(function(_, row)
			{
				var $row = $(row);
				var row_value = $row.attr("search_text");
				var search_regexp = self.make_search_regexp(text);
				if ( row_value.search(search_regexp) === -1 ) {
					$row.hide();
					hidden_rows++;
				}
				else {
					$row.show();
				}
			});

			if ( self.total_rows === hidden_rows ) {
				self.hide_list();
			}
			else {
			// muestra la lista
				self.show_list();
				self.show_detail();
			}
			callback();
		}, 10);
	};

	/*******************************************************************
	* Metodo que xxx
	* @param message
	*/
	conference_base.prototype.hide_list = function( message )
	{
		message = message || this.not_found_default_msg || "<%=_T('Not found.')%>";
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[data_container='true']").hide(); // TODO: ver si remuevo o lo dejo para cargar nuevamente la lista
		var $message_container = $tab_container.find("[not_found_container='true']");
		$message_container.find("span").text(message);
		$message_container.show();
		this.parent.disable_conference();
		this.parent.hide_waiting();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_base.prototype.show_list = function()
	{
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[data_container='true']").show();
		$tab_container.find("[not_found_container='true']").hide();
		this.parent.enable_disable_conference();
		this.parent.hide_waiting();
	};

	/*******************************************************************
	* Metodo que xxx
	* @param campaign_with_vcc
	*/
	conference_base.prototype.get_campaign_name = function( campaign_with_vcc )
	{
		campaign_with_vcc.match(inConcert.app.idAtVCCRegExp);
		return RegExp.$1;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param account
	*/
	conference_base.prototype.get_account_name = function( account )
	{
		var channel_name = this.parent.get_channel_name(account.proxyType);
		return "(" + channel_name + ") " + account.name;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param account
	*/
	conference_base.prototype.can_disable_conference = function()
	{
		return true;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class conference member hereda de conference_base.
	* @param params
	*/
	function conference_member ( params )
	{
		conference_member.uber.constructor.call(this, params);
		this.jquery_item_selector = "[member_item]";
		this.jquery_tab_selector = "[tab_container='members']";
		this.not_found_default_msg = "<%=_T('No users found.')%>";
		this.prefix_radio_id = "conference_member_";
		this.total_message_one  = "<%=_T('{0} user.')%>";
		this.total_message_many = "<%=_T('{0} users.')%>";
		this.load_fn = function( interactions, callback ) { inConcert.app.getInteractionManager().getMembersForTransfer( interactions, callback, this.parent.from_app ); };
	}

	inConcert.Utils.inherit( conference_member, conference_base );

	/*******************************************************************
	* Metodo que xxx
	* @param member
	*/
	conference_member.prototype.get_member_name = function( member )
	{
		var member_name = member.Id;
		var can_name = false;
		if (( member.Name || "" ).trim().length > 0) {
			member_name = member.Name;
			can_name = true;
		}
		if (( member.FirstSurname || "" ).trim().length > 0) {
			member_name = ( can_name ? ( member_name + " " ) : "" ) + member.FirstSurname;
		}
		return member_name;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param member
	*/
	conference_member.prototype.make_search_item_text = function( member )
	{
		return this.get_member_name(member) + " " + member.Id;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_member.prototype.load = function( callback )
	{
		callback = callback || $.noop;
		var self = this;

		var waiting_msg = "<%=_T('Loading users, wait moment please')%>";
		this.parent.show_waiting( waiting_msg );

		// invoco a la clase base
		conference_member.uber.load.call(
			this,
			// callback del request
			function( members )
			{
				if ( !members || members.length === 0 ) {
					self.on_finish_load( callback );
					self.hide_list();
					return;
				}

				var $data_container = self.$tab_container.find("[data_container='true']");
				var $list_container = $data_container.find("[list_container='true']");
				$list_container.detach();

				var sorted_members = inConcert.Utils.sortArray(members, "Name");
				self.total_rows = 0;
				// se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
				inConcert.Utils.foreach_timeout({
					rows : sorted_members,
					max_append_per_loop: 500,
					pause_between_loop_ms: 2,
					waiting_message : "<%=_T('Please wait, adding user, {0} of {1}.')%>",
					append_fn : function( member )
					{
						self.total_rows++;
						var member_dom_id = self.get_radio_dom_id( member.Id );
						var search_text = self.make_search_item_text( member );

						var $item = $("".concat(
							"<div member_item='true' for='", member_dom_id ,"' search_text='", search_text, "' class='targetListItem'>",
								"<input id='", member_dom_id ,"' conference_to='", member.Id ,"' selector='true' type='radio' name='member' class='targetItemCheck' />",
								"<div class='targetItemData'>",
									"<label for='", member_dom_id ,"' class='targetItemName'>", self.get_member_name( member ), "</label>",
									"<label class='targetItemInfo'>", member.Id, "</label>",
								"</div>",
							"</div>"
						));
						$item.data("row", member);
						$list_container.append($item);
					},
					waiting_fn : function(message) {
						self.parent.show_waiting(message);
					},
					complete_fn : function() {
						$data_container.prepend($list_container);
						self.on_finish_load( callback );
					}
				});
			}
		);
	};

	conference_member.prototype.is_enabled = function()
	{
		var self = this;

		var availables_channels = {
			CALL       : true,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};

		var response = { enabled : true };

		$.each(this.parent.data.interactions, function( _, interaction )
		{
			if ( !availables_channels[interaction.type] ) {
				response.enabled = false;
				response.reason = FORMAT("<%=_T('Conference disabled for {0} channel.')%>", self.parent.get_channel_name(interaction.type));
				return false; // break
			}
		});

		return response;
	};

	conference_member.prototype.get_conference_to = function()
	{
		var $tab_container = this.$get_tab_container();
		var $conference_to = $tab_container.find("div[list_container='true'] input[name='member']:checked");
		if ( $conference_to.length === 0 ) {
			return;
		}
		var conference_to = $conference_to.attr("conference_to");
		return {
			to   : "member",
			user : conference_to
		};
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class conference number hereda de conference_base.
	* @param params
	*/
	function conference_number ( params )
	{
		const self = this;
		conference_number.uber.constructor.call(this, params);
		this.jquery_item_selector = "[addresses_book_item]";
		this.jquery_tab_selector = "[tab_container='number']";
		this.not_found_default_msg = "<%=_T('No numbers found.')%>";
		this.prefix_radio_id = "conference_number_";
		this.total_message_one  = "<%=_T('{0} number.')%>";
		this.total_message_many = "<%=_T('{0} numbers.')%>";
	
		this.always_enabled = true;
		this.m_contacts_model = null;

		this.load_fn = function( _, callback )
		{
			if (!self.m_contacts_model) {
				const first_interaction = (self.parent.data.interactions || [])[0];
				if (!first_interaction) {
					throw "Interaction not found";
				}
				let channel = first_interaction.type;
				self.m_contacts_model = new inConcert.AddressesBookSelectorModel(channel);
			}
			self.m_contacts_model.search("", function(success) { 
				if (success) {
					let contacts = self.m_contacts_model.getContacts();
					callback(contacts);
					return;
				}
				callback([]);
			});
		};
	}

	inConcert.Utils.inherit( conference_number, conference_base );

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_number.prototype.load = function( callback = $.noop)
	{
		const self = this;

		let waiting_msg = "<%=_T('Loading Addresses Book, wait moment please')%>";
		this.parent.show_waiting( waiting_msg );

		// invoco a la clase base
		conference_number.uber.load.call(
			this,
			// callback del request
			function( addresses_book )
			{
				if ( !addresses_book || addresses_book.length === 0 ) {
					self.on_finish_load( callback );
					self.hide_list();
					return;
				}

				const $data_container = self.$tab_container.find("[data_container='true']");
				const $list_container = $data_container.find("[list_container='true']");
				$list_container.detach();

				let sorted_addresses_book = inConcert.Utils.sortArray(addresses_book, "Number");
				self.total_rows = 0;
				// se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
				inConcert.Utils.foreach_timeout({
					rows : sorted_addresses_book,
					max_append_per_loop: 500,
					pause_between_loop_ms: 2,
					waiting_message : "<%=_T('Please wait, adding Addresses Book, {0} of {1}.')%>",
					append_fn : function( address_book )
					{
						self.total_rows++;
						let address_book_dom_id = self.get_radio_dom_id( address_book.AddressId );
						let search_text = self.make_search_item_text( address_book );
						let full_phone_number = self.get_addresses_book_full_phone_number(address_book);

						const $item = $("".concat(
							"<div addresses_book_item='true' for='", address_book_dom_id ,"' search_text='", search_text, "' class='targetListItem'>",
								"<input id='", address_book_dom_id ,"' conference_to='", full_phone_number ,"' selector='true' type='radio' name='address_book' class='targetItemCheck' />",
								"<div class='targetItemData'>",
									"<label for='", address_book_dom_id ,"' class='targetItemName'>", self.get_addresses_book_name( address_book ), "</label>",
									"<label class='targetItemInfo'>", full_phone_number, "</label>",
								"</div>",
							"</div>"
						));
						if (address_book.Description) {
							$item.attr("title", address_book.Description);
						}
						$item.data("row", address_book);
						$list_container.append($item);
					},
					waiting_fn : function(message) {
						self.parent.show_waiting(message);
					},
					complete_fn : function() {
						$data_container.prepend($list_container);
						self.on_finish_load( callback );
					}
				});
			}
		);
	};

	conference_number.prototype.is_visible = function()
	{
		var self = this;

		var availables_channels = {
			CALL       : true ,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};

		var response = { visible : true };

		$.each(this.parent.data.interactions, function( _, interaction )
		{
			if ( !availables_channels[interaction.type] ) {
				response.visible = false;
				response.reason = FORMAT("<%=_T('Conference disabled for {0} channel.')%>", self.parent.get_channel_name(interaction.type) );
				return false; // break
			}
		});

		return response;
	};

	conference_number.prototype.is_enabled = function()
	{
		return { enabled : this.is_visible().visible };
	};

	conference_number.prototype.validatePhoneNumber = function(number) {
		// conference a number es solo para interacciones de CALL
		return inConcert.app.validatePhoneNumber(number, "CALL");
	};

	conference_number.prototype.get_conference_to = function()
	{
		var $tab_container = this.$get_tab_container();
		var conference_to = $tab_container.find(this.jquery_txt_search_selector).val();
		if (!this.validatePhoneNumber(conference_to)) {
			return { error : "<%=_T('Invalid phone number')%>" };
		}
		return { to: "number", number: conference_to };
	};

	conference_number.prototype.on_finish_load = function( callback )
	{
		callback = callback || $.noop;
		this.is_loaded = true;
		this.enable();
		this.parent.hide_waiting();
		callback();
	};

	conference_number.prototype.get_addresses_book_full_phone_number = function( address_book )
	{
		let phone_number = address_book.Number;
		if (address_book.Area) {
			phone_number = address_book.Area + phone_number;
		}
		if (address_book.Country) {
			phone_number = address_book.Country + phone_number;
			if (!this.m_has_allegro_integration) {
				phone_number = "+" + phone_number;
			}
		}
		return phone_number;
	};

	conference_number.prototype.get_addresses_book_name = function( address_book )
	{
		let name = address_book.Name || this.get_addresses_book_full_phone_number(address_book);
		return name;
	};

	conference_number.prototype.make_search_item_text = function( address_book )
	{
		return this.get_addresses_book_name(address_book) + " " + this.get_addresses_book_full_phone_number(address_book);
	};

	// El boton para conferencias a number, nunca tiene que quedar disabled, ya que no depende de la lista de addresses book.
	conference_number.prototype.can_disable_conference = function()
	{
		return false;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class conference campaign.
	* @param params
	*/
	function conference_campaign ( params )
	{
		conference_campaign.uber.constructor.call(this, params);
		var self = this;
		this.jquery_item_selector = "[account_item]";
		this.jquery_tab_selector = "[tab_container='campaigns']";
		this.not_found_default_msg = "<%=_T('No campaigns found.')%>";
		this.prefix_radio_id = "conference_account_";
		this.total_message_one  = "<%=_T('{0} account.')%>";
		this.total_message_many = "<%=_T('{0} accounts.')%>";
		this.load_fn = function( interactions, callback )
		{
			inConcert.app.getInteractionManager().getCampaignsAccountsForTransfer(
			{
				app_id       : self.parent.from_app,
				callback     : callback,
				interactions : interactions
			});
		};
	}

	inConcert.Utils.inherit( conference_campaign, conference_base );

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_campaign.prototype.is_enabled = function()
	{
		var self = this;

		var availables_channels = {
			CALL       : false,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};

		var response = { enabled : true };

		$.each(this.parent.data.interactions, function( _, interaction )
		{
			if ( !availables_channels[interaction.type] ) {
				response.enabled = false;
				response.reason = FORMAT("<%=_T('Conference disabled for {0} channel.')%>", self.parent.get_channel_name(interaction.type) );
				return false; // break
			}
		});

		return response;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param account
	* @param campaign_id
	*/
	conference_campaign.prototype.make_search_item_text = function( account, campaign_id  )
	{
		return this.get_campaign_name( campaign_id ) + this.get_account_name( account );
	};

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_campaign.prototype.load = function( callback )
	{
		callback = callback || $.noop;
		var self = this;

		var waiting_msg = "<%=_T('Loading campaigns, wait moment please')%>";
		this.parent.show_waiting( waiting_msg );

		// invoco a la clase base
		conference_campaign.uber.load.call(
			this,
			// callback del request
			function( campaigns )
			{
				if ( !campaigns || Object.keys(campaigns).length === 0 ) {
					self.on_finish_load( callback );
					self.hide_list();
					return;
				}
				var sorted_campaigns = inConcert.Utils.hash_to_sorted_array(campaigns,
					function(campaign) {
						return campaign.Id;
					}
				);
				self.total_rows = 0;
				var $data_container = self.$tab_container.find("[data_container='true']");
				var $list_container = $data_container.find("[list_container='true']");
				$list_container.detach();

				// se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
				inConcert.Utils.foreach_timeout({
					rows : sorted_campaigns,
					pause_between_loop_ms: 2,
					waiting_message : "<%=_T('Please wait, adding campaign, {0} of {1}.')%>",
					append_fn : function( campaign )
					{
						var campaign_id = campaign.Id;
						var sorted_accounts = inConcert.Utils.hash_to_sorted_array(campaign.accounts,
							function(account) {
								return self.get_account_name( account );
							}
						);
						$.each( sorted_accounts, function( _, account )
						{
							self.total_rows++;
							var account_dom_id = self.get_radio_dom_id( account.id );
							var search_text = self.make_search_item_text( account, campaign_id );

							var $item = $("".concat(
								"<div account_item='true' for='", account_dom_id ,"' search_text='", search_text, "' class='targetListItem'>",
									"<input id='", account_dom_id ,"' campaign_conference_to='", campaign_id ,"' account_conference_to='", account.id ,"' selector='true' type='radio' name='account' class='targetItemCheck' />",
									"<div class='targetItemData'>",
										"<label for='", account_dom_id ,"' class='targetItemName'>", self.get_campaign_name( campaign_id ), "</label>",
										"<label class='targetItemInfo'>", self.get_account_name( account ), "</label>",
										"<label class='targetItemInfo'>", account.info.attention_level_name || "", "</label>",
									"</div>",
								"</div>"
							));
							$item.data("row", account);
							$list_container.append($item);
						});
					},
					waiting_fn : function(message) {
						self.parent.show_waiting(message);
					},
					complete_fn : function() {
						$data_container.prepend($list_container);
						self.on_finish_load( callback );
					}
				});
			}
		);
	};

	conference_campaign.prototype.get_conference_to = function()
	{
		var $tab_container = this.$get_tab_container();
		var $conference_to = $tab_container.find("div[list_container='true'] input[name='account']:checked");
		if ( $conference_to.length === 0 ) {
			return;
		}
		var campaign_conference_to = $conference_to.attr("campaign_conference_to");
		var account_conference_to = $conference_to.attr("account_conference_to");
		return {
			to          : "campaign",
			campaign_id : campaign_conference_to,
			account_id  : account_conference_to
		};
	};


	//===================================================================================================
	//===================================================================================================
	/**
	Class conference attention level.
	* @param params
	*/
	function conference_att_level ( params )
	{
		conference_att_level.uber.constructor.call(this, params);
		var self = this;
		this.jquery_item_selector = "[att_level_item]";
		this.jquery_tab_selector = "[tab_container='attention_levels']";
		this.not_found_default_msg = "<%=_T('No attention levels found.')%>";
		this.prefix_radio_id = "conference_att_level_";
		this.total_message_one  = "<%=_T('{0} attention level.')%>";
		this.total_message_many = "<%=_T('{0} attention levels.')%>";
		this.load_fn = function( interactions, callback )
		{
			inConcert.app.getInteractionManager().getAccountsAttentionLevelsForTransfer(
			{
				app_id       : self.parent.from_app,
				callback     : callback,
				interactions : interactions
			});
		};
	}

	inConcert.Utils.inherit( conference_att_level, conference_base );

	/*******************************************************************
	* Metodo que xxx
	* @param att_level
	*/
	conference_att_level.prototype.get_att_level_name = function( att_level )
	{
		return att_level.name;
	};

	/*******************************************************************
	* Metodo que xxx
	* @param account
	* @param att_level
	*/
	conference_att_level.prototype.make_search_item_text = function( account, att_level  )
	{
		return this.get_account_name( account ) + this.get_att_level_name( att_level);
	};

	/*******************************************************************
	* Metodo que xxx
	* @param callback
	*/
	conference_att_level.prototype.load = function( callback )
	{
		callback = callback || $.noop;
		var self = this;

		var waiting_msg = "<%=_T('Loading attention levels, wait moment please')%>";
		this.parent.show_waiting( waiting_msg );

		// invoco a la clase base
		conference_att_level.uber.load.call(
			this,
			// callback del request
			function( campaigns )
			{
				if ( !campaigns || Object.keys(campaigns).length === 0 ) {
					self.on_finish_load( callback );
					self.hide_list();
					return;
				}
				var sorted_campaigns = inConcert.Utils.hash_to_sorted_array(campaigns,
					function(campaign) {
						return campaign.Id;
					}
				);
				self.total_rows = 0;

				var $data_container = self.$tab_container.find("[data_container='true']");
				var $list_container = $data_container.find("[list_container='true']");
				$list_container.detach();

				// se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
				inConcert.Utils.foreach_timeout({
					rows : sorted_campaigns,
					pause_between_loop_ms: 2,
					waiting_message : "<%=_T('Please wait, adding attention level, {0} of {1}.')%>",
					append_fn : function( campaign )
					{
						var campaign_id = campaign.Id;
						var sorted_accounts = inConcert.Utils.hash_to_sorted_array(campaign.accounts,
							function(account) {
								return self.get_account_name( account );
							}
						);
						$.each( sorted_accounts, function( _, account )
						{
							var sorted_att_levels = inConcert.Utils.hash_to_sorted_array(account.AttentionLevels,
								function(att_level) {
									return self.get_att_level_name( att_level );
								}
							);
							$.each( sorted_att_levels, function( _, att_level )
							{
								self.total_rows++;
								var att_level_id = att_level.attention_level_id;
								var att_level_dom_id = self.get_radio_dom_id(att_level_id);
								var account_name = self.get_account_name( account );
								var search_text = self.make_search_item_text( account, att_level );

								var $item = $("".concat(
									"<div att_level_item='true' for='", att_level_dom_id ,"' search_text='", search_text, "' class='targetListItem'>",
										"<input id='", att_level_dom_id ,"' campaign_conference_to='", campaign_id ,"' account_conference_to='", account.id ,"' att_level_conference_to='", att_level_id ,"' selector='true' type='radio' name='att_level' class='targetItemCheck' />",
										"<div class='targetItemData'>",
											"<label class='targetItemName'>", self.get_att_level_name( att_level ), "</label>",
											"<label for='", att_level_dom_id ,"' class='targetItemInfo'>", account_name , "</label>",
										"</div>",
									"</div>"
								));
								$item.data("row", att_level);
								$list_container.append($item);
							});
						});
					},
					waiting_fn : function(message) {
						self.parent.show_waiting(message);
					},
					complete_fn : function() {
						$data_container.prepend($list_container);
						self.on_finish_load( callback );
					}
				});
			}
		);
	};

	conference_att_level.prototype.is_enabled = function()
	{
		var self = this;

		var availables_channels = {
			CALL       : false,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};

		var response = { enabled : true };

		$.each(this.parent.data.interactions, function( _, interaction )
		{
			if ( !availables_channels[interaction.type] ) {
				response.enabled = false;
				response.reason = FORMAT("<%=_T('Conference disabled for {0} channel.')%>", self.parent.get_channel_name(interaction.type) );
				return false; // break
			}
		});

		return response;
	};

	conference_att_level.prototype.is_visible = function()
	{
		var self = this;

		var availables_channels = {
			CALL       : false,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};

		var response = { visible : true };

		$.each(this.parent.data.interactions, function( _, interaction )
		{
			if ( !availables_channels[interaction.type] ) {
				response.visible = false;
				response.reason = FORMAT("<%=_T('Conference disabled for {0} channel.')%>", self.parent.get_channel_name(interaction.type) );
				return false; // break
			}
		});

		return response;
	};

	conference_att_level.prototype.get_conference_to = function()
	{
		var $tab_container = this.$get_tab_container();
		var $conference_to = $tab_container.find("div[list_container='true'] input[name='att_level']:checked");
		if ( $conference_to.length === 0 ) {
			return;
		}
		var campaign_conference_to  = $conference_to.attr("campaign_conference_to");
		var account_conference_to   = $conference_to.attr("account_conference_to");
		var att_level_conference_to = $conference_to.attr("att_level_conference_to");
		return {
			to           : "attention_level",
			campaign_id  : campaign_conference_to,
			account_id   : account_conference_to,
			att_level_id : att_level_conference_to
		};
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class conference_popup.
	* @param params
	*/
	function conference_popup (params)
	{
		var self = this;
		conference_popup.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "conference";
		this.ownerId = "window_conference_popup_container";
		this.canReposition = true;

		if (!params) {
			throw "conference_popup - params is required";
		}
		if (!params.from_app) {
			throw "conference_popup - params is required.from_app";
		}

		this.from_app = params.from_app;
		this.conference_type = params.conference_type;
		this.current_container = null;
		this.count_loading_list = 0;
		this.m_is_conferencering = false;

		// instancias de los handles de las listas
		this.list_handle = {
			attention_levels : conference_att_level,
			campaigns        : conference_campaign,
			members          : conference_member,
			number           : conference_number
		};

		if (this.conference_type) {
			$.each(this.conference_type, function(conference_type) {
				if (!self.list_handle.hasOwnProperty(conference_type)) {
					throw "conference_popup - Unrecognized conference type " + conference_type;
				}
				self.list_handle[conference_type] = new self.list_handle[conference_type]({parent: self});
			});

			var handles_to_delete = {};
			$.each(this.list_handle, function(conference_type) {
				if (!self.conference_type.hasOwnProperty(conference_type)) {
					handles_to_delete[conference_type] = true;
				}
			});

			$.each(handles_to_delete, function(conference_type) {
				delete self.list_handle[conference_type];
			});
		}
		else {
			$.each(this.list_handle, function(id, conference_class) {
				self.list_handle[id] = new conference_class({parent: self});
			});
		}
	}

	inConcert.Utils.inherit(conference_popup, inConcert.PopupWindow);

	//----------------------------
	if (typeof(inConcert.conference_popup) !== "function") {
		inConcert.conference_popup = conference_popup;
	}

	/*******************************************************************
	*
	* @param text
	* @param type
	* @param closeDisabled
	* @param timeout_ms
	*/
	conference_popup.prototype.show_status_message = function(text, type, closeDisabled, timeout_ms )
	{
		var $section = $("#conference_message");
		$section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#conference_close_message").hide();
		}
		else {
			$("#conference_close_message").show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	conference_popup.prototype.hide_status_message = function() {
		$("#conference_message").hide();
	};

	/*******************************************************************
	*
	*/
	conference_popup.prototype.validate = function()
	{
		if ( !this.current_container ) {
			return false;
		}
		if (this.list_handle[this.current_container].always_enabled) {
			return true;
		}
		var selected_item_id = this.list_handle[this.current_container].get_selected_item_id();
		return selected_item_id !== null;
	};

	/*******************************************************************
	* 
	*/
	conference_popup.prototype.get_conference_to = function()
	{
		if ( !this.current_container ) {
			return;
		}
		var list_handle = this.list_handle[this.current_container];
		if ( !list_handle ) {
			return;
		}
		return list_handle.get_conference_to();
	};

	/*******************************************************************
	* 
	* @param message
	*/
	conference_popup.prototype.show_message_in_main_window = function( message )
	{
		inConcert.app.getMainWindow().showBannerMessage(message);
	};

	conference_popup.prototype.conference_execute = function(conference_to, dispositon_info, callback_end = $.noop)
	{
		const self = this;
		const $cancel = $("#conference_popup_cancel_interaction");

		this.disable_conference();
		$cancel.prop("disabled", true);

		var get_conflict_message = function(error, count_errors)
		{
			switch (error)
			{
				case "COULD_NOT_RESERVE_AGENT":
					return "<%=_T('Could not reserve the agent')%>";

				case "CHATSERVER_NOT_AVAILABLE":
					return "<%=_T('Chatserver not available')%>";

				case "FORBIDDEN":
					return FORMAT("<%=_T('You have not permissions to conference {0} conversations on campaign.')%>", count_errors);

				case "GENERIC_ERROR":
					return FORMAT("<%=_T('{0} failed putting conversations in conference, please try again later')%>", count_errors);

				case "GENERIC_WARNING":
					return "<%=_T('Warning, some conflicts have occurred putting conversations in conference, please contact your supervisor')%>";

				case "INTERACTION_CLOSED_OR_TAKEN":
					return "<%=_T('Unable to conference. The conversation was closed or taken')%>";

				case "INTERACTION_NOT_FOUND":
					return "<%=_T('Conversation not found')%>";

				case "INVALID_REQUEST":
					return "<%=_T('Invalid request please try again later')%>";

				case "TRANSFER_TYPE_NOT_SUPPORTED":
					return "<%=_T('Unsupported conference type')%>";

				case "CAN_NOT_TRANSFER_YOURSELF":
					return "<%=_T('Unable to conference conversation, cannot conference yourself')%>";

				case "MEMBER_DOES_NOT_EXIST":
					return "<%=_T('Unable to conference conversation, user does not exist')%>";

				case "MEMBER_IS_DISABLED":
					return "<%=_T('Unable to conference conversation, user is disabled')%>";

				case "MEMBER_NOT_HAS_PERMISSION_TO_INTERACTION_APPLICATION":
					return "<%=_T('Unable to conference conversation, user not has permission to Interaction Application')%>";

				case "MEMBER_NOT_AVAILABLE":
					return "<%=_T('Unable to conference conversation, user not available')%>";

				case "CHANNEL_DOES_NOT_SUPPORTED_TRANSFERS_TO_MEMBERS":
					return "<%=_T('Unable to conference conversation, the channel not supported conferences to users')%>";

				case "MEMBER_HAS_NOT_AVAILABLE_CAPABILITY_FOR_CHANNEL":
					return "<%=_T('Unable to conference conversation, user has not available capability for channel')%>";

				case "CONVERSATION_ALREADY_TRANSFERRING":
					return "<%=_T('Conversation is already conferencering')%>";

				case "ALLEGRO_MEMBER_DOES_NOT_HAVE_A_PHONE_CONFIGURED":
					return "<%=_T('Unable to conference conversation, user does not have a Phone configured')%>";

				default:
					return "<%=_T('Unable to conference conversation, please try again later')%>";
			}
		};

		var get_error_message = function(errors, total, callback)
		{
			var message = total > 1?
				FORMAT("<%=_T('Failed conference {0} {1} of {2}')%>", 
					errors.count_total, "<%=_T('conversations')%>", total) :
				"<%=_T('Failed to conference conversation')%>";
			if (errors.count_types > 1) {
				//multiple types: generic error
				return callback(message);
			}
			var type = errors.types[0];
			var raw = errors.types_raw[type] || {};
			let agentId = inConcert.Utils.getIdWithoutVCC(raw.agent);

			switch(type) {
				case "INTERACTION_OPEN":
					raw = raw.error_source;
					var openMessage;
					if (errors.count_total > 1) {
						openMessage = "<%=_T('there are open conversations with same clients in destination campaign')%>";
					}
					else {
						switch(raw.state) {
							case "WAITING":
								openMessage = "<%=_T('there is a conversation with same client in destination campaign waiting list')%>";
								break;
							case "QUEUED":
								openMessage = "<%=_T('there is a conversation with same client in destination campaign queue')%>";
								break;
							case "TAKEN":
								if (agentId === inConcert.app.getLoggedUserId()) {
									openMessage = "<%=_T('you have a conversation with same client in destination campaign')%>";
								}
								else {
									inConcert.app.getTeamMember(agentId, function(agent) {
										let displayName = (agent && agent.Name) ? agent.Name : agentId;
										message += ": " + FORMAT("<%=_T('there is a conversation with same client in destination campaign assigned to agent {0}')%>", displayName);
										callback(message);
									});
									return;
								}
								break;
							default:
								openMessage = "<%=_T('there is an open conversation with same client in destination campaign')%>";
								break;
						}
					}
					message += ": " + openMessage;
			}
			callback(message);
		};

		var cloned_interactions = inConcert.Utils.cloneArray(this.data.interactions) || [];
		var count_interactions = cloned_interactions.length;

		var callbacks = {
			onFinished : function(result)
			{
				self.m_is_conferencering = false;
				var message      = null;
				var message_type = "Info";
				var errors       = result.errors;
				var warnings     = result.warnings;

				if ( errors && errors.count_total > 0 ) {
				// con errores
					message_type = errors.count_total === count_interactions ? "Error" : "Alert";
					get_error_message(errors, count_interactions, function(message_err) {
						self.show_message_in_main_window({message: message_err, messageType: message_type, timeout: 5000});
						self.close();
					});
					callback_end();
					return;
				}
				else {
					if ( warnings && warnings.count_total > 0 ) {
					// con conflictos
						message_type = "Alert";
						if ( warnings.count_types === 1 ) {
							var warning_type = warnings.types.pop();
							message = get_conflict_message(warning_type, warnings.count_total);
						}
						else {
							message = get_conflict_message("GENERIC_WARNING", warnings.count_total);
						}
					}
				}

				if ( message === null ) {
				// aca entra si es success, es decir si no tuvo ningun error ni ningun warning
					if ( count_interactions <= 1 ) {
						// no muestra ningun mensaje si transfirio con exito y es una unica interaccion.
						self.on_conference_success();
						return;
					}
					message = count_interactions === 1?
							"<%=_T('Conversation conferencered successfully!')%>"
						:	"<%=_T('All conversations were conferencered successfully!')%>";
				}
				callback_end();
				self.on_conference_success( { message: message, messageType: message_type } );
			},
			onPartial: function(successfulNodes/*, unsuccessfulNodes*/)
			{
				if (successfulNodes) {
					$.each(successfulNodes, function(_, conferenceredInteraction) {
						var index_to_remove = -1;
						$.each(self.data.interactions, function(i, interaction) {
							if (conferenceredInteraction.id === interaction.id) {
								index_to_remove = i;
								return false; //break
							}
						});
						if (index_to_remove > -1) {
							self.data.interactions.splice(index_to_remove, 1);
						}
					});
				}
				callback_end();
			},
			onFailure: function(err_message)
			{
				self.on_conference_error(get_conflict_message(err_message));
				callback_end();
			}
		};

		var panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");
		var batchFunction = function( ints, to, conference_callback ) {
			return inConcert.app.getInteractionManager().batchConference(ints, to, conference_callback, self.from_app, dispositon_info);
		};
		var destination = [ conference_to ];
		var batchData = new panel.BatchData(batchFunction, cloned_interactions, destination, callbacks);
		panel.processBatchInteractions(batchData);
	};

	/*******************************************************************
	* Conference interactions and save the current conference code and comment.
	*/
	conference_popup.prototype.conference_conversations = function($parent_node)
	{
		const self = this;

		if (this.m_is_conferencering) {
			return;
		}
		this.m_is_conferencering = true;

		if (!this.validate()) {
			this.m_is_conferencering = false;
			this.show_status_message("<%=_T('Please select destination item to conference.')%>", "Error", undefined, 2500);
			return;
		}

		var conference_to = this.get_conference_to();
		if ( !conference_to || conference_to.error ) {
			this.m_is_conferencering = false;
			this.show_status_message(conference_to && conference_to.error || "<%=_T('Error in selected conference item.')%>", "Error", undefined, 2500);
			return;
		}

		if (conference_to.to !== "campaign") {
			// 1 - Aca realizamos las conferenceencias que NO son a campanas
			this.conference_execute(conference_to);
			return;
		}

		// Aca realizamos la conferenceencia a campanas
		const first_interaction = (this.data.interactions || [])[0];
		if (!first_interaction) {
			throw "Interaction not found";
		}

		const campaign = inConcert.app.getCampaign(first_interaction.campaign);
		if (!campaign) {
			throw "Campaign has been deleted";
		}

		//------------------------------------------------------------------------------------------
		// proxies asincronicos que admiten disposiciones de finalizacion al realizar conferencias x campana
		//   existe la misma validacion en:
		//             1 - (AGENT) /agent/cotrollers/interactions.lua - en la funcion para conferencias (proxiesThatSupportDispositionsWhenTransferring)
		//             2 - (IR)    /scripts/IR/chatscripts.lua        - en la funcion para conferencias (proxiesThatSupportDispositionsWhenTransferring)
		const proxies_that_support_dispositions_when_conferencering = {};

		if (!proxies_that_support_dispositions_when_conferencering[first_interaction.type] || !campaign.hasDispositionCodes()) {
			// 2 - Transferencia a campana SIN dispositon code
			this.conference_execute(conference_to);
			return;
		}

		// 3 - Transferencia a campana CON dispositon code. Abro el popup de dispositions para seleccionar uno.
		this.m_is_conferencering = false;
		const $cancel = $("#conference_popup_cancel_interaction");
		this.disable_conference();
		$cancel.prop("disabled", true);

		const dispositionCodes = campaign.getDispositionCodes();
		const dispositionCodesPopup = new inConcert.DispositionCodesPopup({from_app: this.from_app, action_type: "NOT_FINISH - GET_DISPOSITION"});

		dispositionCodesPopup.show({
			interactions:     [first_interaction],
			parentNode:       $parent_node,
			dispositionCodes: dispositionCodes,
			callbacks: {
				on_finish : function(disposition_info) {
					// Aca ejecuto la conferenceencia con el disposition seleccionado.
					self.m_is_conferencering = true;
					self.conference_execute(conference_to, disposition_info);
				},
				onClose: function() {
					if (self.m_is_conferencering === false) {
						self.enable_conference();
					}
				}
			}
		});
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	* @param tab_item
	*/
	conference_popup.prototype.switch_conference_container = function( tab_item )
	{
		var $tab_items = $("#conference_tab_items");
		var $tab_item = $tab_items.find("[tab_item='" + tab_item + "']");
		if ( this.current_container === tab_item ) {
			return;
		}
		this.current_container = tab_item;
		$tab_items.find("[tab_item]").removeClass("selected");
		$tab_item.addClass("selected");
		var $container = $("#window_conference_popup_container");
		$container.find("[tab_container]").hide();
		$container.find("[tab_container='" + tab_item + "']").show();
		this.list_handle[tab_item].on_switch_tab();
	};

	/*******************************************************************
	* Metodo que muestra u oculta el selector
	* @param tab_item
	*/
	conference_popup.prototype.toggle_conference_selector = function( tab_item )
	{
		var tab_handle = this.list_handle[tab_item];
		if (!tab_handle) {
			return;
		}
		var result = tab_handle.is_visible();
		$("#conference_tab_items")
			.find("[tab_item='" + tab_item + "']")
			.toggle(result.visible);
	};

	/*******************************************************************
	* Metodo xxx
	*/
	conference_popup.prototype.enable_conference = function()
	{
		$("#conference_popup_conference_conversation").prop("disabled", false);
		$("#conference_popup_cancel_interaction").prop("disabled", false);
	};

	conference_popup.prototype.disable_conference = function()
	{
		if (!this.can_disable_conference()) {
			return;
		}
		$("#conference_popup_conference_conversation").prop("disabled", true);
	};

	conference_popup.prototype.can_disable_conference = function()
	{
		if (!this.current_container) {
			return true;
		}
		const list_handle = this.list_handle[this.current_container];
		if (!list_handle) {
			return true;
		}
		return list_handle.can_disable_conference();
	};

	conference_popup.prototype.enable_disable_conference = function()
	{
		var list_handle = this.list_handle[this.current_container];
		var selected_item = list_handle.get_selected_item_id();
		var enabled = list_handle.always_enabled || false;

		if ( !enabled && selected_item ) {
			var $item = $("#" + selected_item);
			var $item_container = $item.parent();
			enabled = $item_container.is(":visible");
		}

		if ( enabled ) {
			this.enable_conference();
		}
		else {
			this.disable_conference();
		}
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	conference_popup.prototype.set_events = function()
	{
		var self = this;

		$("#conference_popup_conference_conversation")
			.bind("click", function() {
				const $this = $(this);
				self.conference_conversations($this);
			});

		var close_popup = function() { self.close(); };

		// evento que cierra el dialogo y no realiza cambios
		$("#conference_popup_cancel_interaction").bind("click", close_popup );
		// evento que cierra el dialogo y no realiza cambios
		$("#close_conference_popup").bind("click", this.close);

		$("#conference_close_message")
			.bind("click", function() {
				self.hide_status_message();
			});

		var event_selector = "[tab_item]";
		$("#conference_tab_items")
			.find(event_selector)
			.each(function() {
				var $this = $(this);
				var tab_item = $this.attr("tab_item");
				self.toggle_conference_selector( tab_item );
			})
			.off("click")
			.on("click", function () {
				var $this = $( this );
				var tab_item = $this.attr("tab_item");
				self.switch_conference_container( tab_item );
			}
		);
	};

	/*******************************************************************
	* Metodo que crea el markup contenedor de la ventana
	*/
	conference_popup.prototype.create_markup = function()
	{
		$("#window_conference_popup_container").remove();
		var $node = $("".concat(
			"<div id='window_conference_popup_container' class='systemPopWindow windowConferenceContainer windowContainer' hidden>",

				"<div id='conference_waiting' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

				"<div class='windowTop'>",
					"<div class='windowTitle windowConference'>",
						"<div class='windowConferenceTitleText'></div>",
						"<img id='close_conference_popup' src='/inconcert/apps/agent/images/blank.png' class='windowConferenceButClose' hidden />",
					"</div>",
				"</div>",

				"<div id='conference_message' class='windowContainerMessages messageError' hidden>",
					"<a id='conference_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					// conference options
					"<div id='conference_tab_items' class='windowTabsMenu'>",
						"<span tab_item='members' class='windowTabsMenuItem' hidden>", "<%=_T('Members')%>", "</span>",
						"<span tab_item='campaigns' class='windowTabsMenuItem' hidden>", "<%=_T('Campaigns')%>", "</span>",
						"<span tab_item='attention_levels' class='windowTabsMenuItem' hidden>", "<%=_T('Attention Levels')%>", "</span>",
						"<span tab_item='number' class='windowTabsMenuItem' hidden>", "<%=_T('Number')%>", "</span>",
					"</div>",

					// members list container
					"<div tab_container='members' class='windowTargetConferenceContainer' hidden>",
						"<input txt_search='true' class='windowTargetSearch' type='text' disabled='true'/>",
						"<div data_container='true' class='windowTargetConferenceControl' hidden>",
							"<div list_container='true' class='windowTargetConferenceScrollArea'></div>",
							"<div class='windowTargetFooter'>",
								"<div total='true' class='windowTargetTotal'></div>",
								"<div detail='true' class='windowTargetDetailContainer'>",
									"<div detail_list='true' class='windowTargetDetailInfo'></div>",
								"</div>",
							"</div>",
						"</div>",
						"<div not_found_container='true' class='col-12 px-0' hidden>",
							"<span class='inlineMessagesContainer messageInfo windowTargetNoResultsText'></span>",
						"</div>",
					"</div>",

					// campaigns list container
					"<div tab_container='campaigns' class='windowTargetConferenceContainer' hidden>",
						"<input txt_search='true' class='windowTargetSearch' type='text' disabled='true'/>",
						"<div data_container='true' class='windowTargetConferenceControl' hidden>",
							"<div list_container='true' class='windowTargetConferenceScrollArea'></div>",
							"<div class='windowTargetFooter'>",
								"<div detail='true' class='windowTargetDetailContainer'>",
									"<div detail_list='true' class='windowTargetDetailInfo'></div>",
								"</div>",
							"</div>",
						"</div>",
						"<div not_found_container='true' class='col-12 px-0' hidden>",
							"<span class='inlineMessagesContainer messageInfo windowTargetNoResultsText'></span>",
						"</div>",
					"</div>",

					// attention levels list container
					"<div tab_container='attention_levels' class='windowTargetConferenceContainer' hidden>",
						"<input txt_search='true' class='windowTargetSearch' type='text' disabled='true'/>",
						"<div data_container='true' class='windowTargetConferenceControl' hidden>",
							"<div list_container='true' class='windowTargetConferenceScrollArea'></div>",
							"<div class='windowTargetFooter'>",
								"<div detail='true' class='windowTargetDetailContainer'>",
									"<div detail_list='true' class='windowTargetDetailInfo'></div>",
								"</div>",
							"</div>",
						"</div>",
						"<div not_found_container='true' class='col-12 px-0' hidden>",
							"<span class='inlineMessagesContainer messageInfo windowTargetNoResultsText'></span>",
						"</div>",
					"</div>",

					// number container
					"<div tab_container='number' class='windowTargetNumberContainer' hidden>",
						"<input txt_search='true' class='windowTargetSearch' type='text' />",
						"<div class='windowTargetTitleContainer'>",
							"<span class='windowTargetTitle'>",
								"<%=_T('Addresses Book')%>",
							"</span>",
						"</div>",
						"<div data_container='true' class='windowTargetTransferControl' hidden>",
							"<div list_container='true' class='windowTargetTransferScrollArea'></div>",
							"<div class='windowTargetFooter'>",
								"<div detail='true' class='windowTargetDetailContainer'>",
									"<div detail_list='true' class='windowTargetDetailInfo'></div>",
								"</div>",
							"</div>",
						"</div>",
						"<div not_found_container='true' class='col-12 px-0' hidden>",
							"<span class='inlineMessagesContainer messageInfo d-block windowTargetNoResultsText'></span>",
						"</div>",
					"</div>",
				"</div>",

				"<div class='windowBottom'>",
				"<input id='conference_popup_cancel_interaction' class='pageButtonGeneric buttonColorWhite float-right' type='button' value='", "<%=_T('Cancel')%>", "'/>",
					"<input id='conference_popup_conference_conversation' class='pageButtonGeneric buttonColorBlue float-right' disabled='true' type='button' value='", "<%=_T('Conference')%>", "'/>",
				"</div>",

			"</div>"));
		$("body").append($node);
		return $node;
	};

	conference_popup.prototype.get_channel_name = function( type )
	{
		var channels = {
			CALL       : "<%=_T('Phone Call')%>"  ,
			CHAT       : "<%=_T('Web chat')%>"    ,
			FACEBOOK   : "<%=_T('Facebook')%>"    ,
			INSTAGRAM  : "<%=_T('Instagram')%>"   ,
			YOUTUBE    : "<%=_T('Youtube')%>"     ,
			APPSTORE   : "<%=_T('Appstore')%>"    ,
			GOOGLEPLAY : "<%=_T('Googleplay')%>"  ,
			RCS    	   : "<%=_T('Rcs')%>"     	  ,
			IM         : "<%=_T('Chat')%>"        ,
			MAIL       : "<%=_T('E-Mail')%>"      ,
			SMS        : "<%=_T('SMS')%>"         ,
			WHATSAPP   : "<%=_T('Whatsapp')%>"    ,
			TWITTER    : "<%=_T('Twitter')%>"     ,
			WEBCONTACT : "<%=_T('Web contact')%>" ,
			VOICEMAIL  : "<%=_T('Voice mail')%>"
		};
		return channels[type];
	};

	conference_popup.prototype.set_popup_title = function()
	{
		var popup_title;
		if (this.is_batch){
			var count_interactions = this.data.interactions.length;
			popup_title = FORMAT("<%=_T('Batch Conference - {0} conversations')%>", count_interactions);
		}
		else {
			var interaction_type = this.data.interactions[0].type;
			var channel_name = this.get_channel_name( interaction_type );
			if ( !channel_name ) {
				throw "No channel name found for the conversation of type " + interaction_type;
			}
			popup_title = FORMAT("<%=_T('{0} Conference')%>", channel_name);
		}
		$("#window_conference_popup_container .windowConferenceTitleText").text(popup_title);
	};

	conference_popup.prototype.on_conference_success = function( message_object )
	{
		if ( message_object ) {
			message_object.timeout = 5000;
			this.show_message_in_main_window( message_object );
		}
		this.close();
		this.parent_on_conference_success();
	};

	/*******************************************************************
	* Metodo que xxxx
	* @param message_err
	*/
	conference_popup.prototype.on_conference_error = function( message_err )
	{
		this.m_is_conferencering = false;
		this.enable_conference();
		this.show_message_in_main_window({ message: message_err, messageType : "Error", timeout : 5000 });
		this.parent_on_conference_error();
	};

	/*******************************************************************
	* Metodo que xxx
	* @param message
	*/
	conference_popup.prototype.show_waiting = function( message )
	{
		this.waiting_progress_count = this.waiting_progress_count || 0;
		if ( this.waiting_progress_count === 5 ) {
			this.waiting_progress_count = -1;
		}
		this.waiting_progress_count++;
		var progress_text = "";
		var progress = "...";
		for ( var i = 1; i <= this.waiting_progress_count; i++ ) {
			progress_text = progress_text + progress;
		}
		message = message || "<%=_T('Loading, wait moment please')%>";
		var $waiting = $("#conference_waiting");
		$waiting.find("[message='true']").text(message + " " + progress_text);
		$waiting.show();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	conference_popup.prototype.hide_waiting = function()
	{
		var count_loaded = 0;
		$.each( this.list_handle, function( _, instance )
		{
			var result = instance.is_enabled();
			if ( result.enabled && instance.is_loaded ) {
				count_loaded++;
			}
		});
		if ( this.count_loading_list === count_loaded ) {
			$("#conference_waiting").hide();
		}
		//this.setPosition();
	};

	/*******************************************************************
	* Metodo que visualiza el popup con los conference e inicializa eventos del popup!
	*/
	conference_popup.prototype.initialize = function()
	{
		var self = this;
		var interactions = this.data.interactions;
		if ( !interactions ) {
			throw "conference_popup.initialize - interactions is required!!";
		}
		this.is_batch = interactions.length > 1;
		this.data.container = this.create_markup();
		this.parent_on_conference_success = this.data.callbacks.on_conference_success || $.noop;
		this.parent_on_conference_error   = this.data.callbacks.on_conference_error   || $.noop;
		this.set_popup_title();
		// set events
		this.set_events();

		var pending_instances_load = [];
		// continuno con los request pendientes
		var on_load = function() {
			if ( pending_instances_load.length === 0 ) {
				return;
			}
			var instance = pending_instances_load.pop();
			if ( instance ) {
				instance.load( on_load );
			}
		};

		// inicializo las instancias de las diferentes listas
		$.each( this.list_handle, function( _, instance ) {
			if ( instance.initialize() ) {
				self.count_loading_list++;
				if (self.count_loading_list === 1) {
					instance.load( on_load );
				}
				else {
					// a los request los dejo como pendientes para que el siguiente request, recien mande a ejecutar, una vez que ha terminado el anterior.
					pending_instances_load.push( instance );
				}
			}
		});
		var conference_type = "members";
		if (this.conference_type) {
			for (var t in this.conference_type) {
				if (this.conference_type.hasOwnProperty(t)) {
					conference_type = t;
					break;
				}
			}
		}
		this.switch_conference_container(conference_type);
	};

	/*******************************************************************
	* Metodo que cierra el popup
	*/
	conference_popup.prototype.close = function() {
		this.data.container.remove();
		this.finalize();
	};
}());
