/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class ConversationsPanel.
	*/

	'use strict';

	const CSS_BODY_HELPDESK_CLASS = "interactionInboxHelpDesk";

	function ConversationsPanel (id)
	{
		const self = this;
		// ConversationsPanel.uber.constructor.call(this, id, "<%=_T('Conversations Panels')%>");

		this.conversations_panels = {
			assigned  : null,
			queued    : null,
			waiting   : null,
			abandoned : null,
			spam      : null
		};

		this.others_panels = {
			shared    : null
		};

		this.$assigned         = $('#panelAssignedInteractions');
		// el queued sale de la misma clase. TODO: Hacer lo mismo con los demas paneles.
		this.$waiting          = $('#int_panel_waiting');
		this.$abandoned        = $('#int_panel_abandoned');
		this.$spam             = $('#int_panel_spam');
		this.$global_search    = $('#int_panel_global_search');
		this.$search_tickets   = $('#int_panel_search_tickets');
		this.$categorie        = $('#int_panel_categorie');
		this.$helpdesk_tickets = $('#int_panel_integration_helpdesk_tickets');
		const $body             = $('body');

		const main = this.__get_main_window();

		const on_show_folder = function() {
			// Esta funcion es la encargada de reposicionar los paneles.
			main.repositionPanels();
		};

		// --------------------------------------------------------
		//QueryBuilder
		const $interaction_dashboard = $("#interaction_dashboard");

		const on_ready_to_refresh = function() {
			self.onReadyToRefresh();
		};	

		const on_rule_added = function() {
			self.queryBuilderCleanButton.prop("disabled", false);
		};

		const $queryBuilder = $interaction_dashboard.find('[query_builder]');
		const inconcert_querybuilder_config = {
			container: $queryBuilder,
			fromApp: "interaction",
			onReadyToRefresh: on_ready_to_refresh,
			onRuleAdded: on_rule_added,
			panels: [
				{
					panelId: "active",
					filters:["campaign", "type", "has_attachments", "initiative", "has_draft"]
				},
				{
					panelId: "live",
					filters:["campaign", "type", "has_attachments", "initiative"]
				},
				{
					panelId: "appointment",
					filters:["campaign", "type", "initiative"]
				},
				{
					panelId: "archive",
					filters:["campaign", "type", "businessdata", "account", "disposition_code", "contact_address", "start_date", "contact", "contact_name", "content", "ticket", "has_attachments", "initiative"]
				},
				{
					panelId: "queued",
					filters:["campaign", "type", "account", "contact_address", "start_date", "contact", "contact_name", "content", "ticket", "has_attachments", "initiative"]
				},
				{
					panelId: "waiting",
					filters:["campaign", "type", "account", "contact_address", "start_date", "contact", "contact_name", "content", "ticket", "has_attachments", "initiative"]
				},
				{
					panelId: "abandoned",
					filters:["campaign", "type", "account", "contact_address", "start_date", "contact", "contact_name", "content", "ticket", "initiative"]
				},
				{
					panelId: "integration_helpdesk_tickets",
					filters:["campaign"]
				},
				{
					panelId: "global_search",
					filters:["campaign", "type", "businessdata", "agent", "account", "disposition_code", "contact_address", "state", "start_date", "contact", "attention_level", "agentname", "contact_name", "content", "ticket", "has_attachments", "initiative", "has_draft", "id" /*interaction_id*/]
				},
				{
					panelId: "search_tickets",
					filters:["campaign", "type", "businessdata", "agent", "account", "disposition_code", "contact_address", "state", "start_date", "contact", "attention_level", "agentname", "contact_name", "content", "ticket", "has_attachments", "initiative"]
				},
				{
					panelId: "categorie",
					filters:["campaign", "type", "businessdata", "agent", "account", "disposition_code", "contact_address", "state", "start_date", "contact", "attention_level", "agentname", "contact_name", "content", "ticket", "has_attachments", "initiative"]
				}
			]
		};
		this.queryBuilderObj = new inConcert.inconcert_querybuilder(inconcert_querybuilder_config);
		this.queryBuilderRefressButton = $('#interactionQueryBuilder-btn-refresh');
		this.queryBuilderCleanButton = $('#interactionQueryBuilder-btn-clean-all');

		this.queryBuilderRefressButton.on('click', function() {
			self.onReadyToRefresh();
		});		

		this.queryBuilderCleanButton.on('click', function() {
			self.queryBuilderObj.cleanAll();
		});

		let iframes_container_id;
		if (inConcert.app.is_enable_docked_windows_interactions()) {
			// TODO: #9778 - CANY - Docked. Revisar !!
			iframes_container_id = "iFramesWindowsDailogsContainerForInteractionAppGlobalSearchPanel";
		}

		this.navigation = new inConcert.ConversationsNavigation(
		{
			from_app: 'interaction',
			queryBuilderObj: this.queryBuilderObj,
			container_id: 'int_conversations_navigation',
			global_search_container_id: 'int_panel_global_search', // debe existir en inboxmaster.html
			// TODO: #9778 - CANY - Docked.
			global_search_iframes_container_id: iframes_container_id,
			integration_helpdesk_container_id: 'int_panel_integration_helpdesk_tickets', // debe existir en inboxmaster.html
			// El valor del campo "id" debe coincidir con lo que se almacena en el IR para ES, en la funcion "store_in_elasticsearch" en la variable "data" del archivo (ir.lua)
			global_search_columns_sort : {
				creation_date                 : { order: 'desc', 'default': false, id: 'start_timestamp' },
				last_event_date_or_state_date : { order: 'desc', 'default': true , id: 'state_timestamp' },
				contact_name                  : { order: 'asc' , 'default': false, id: 'contact_name'    },
				owner                         : { order: 'asc' , 'default': false, id: 'agent'           }
			// este se ordena segun la traduccion asi que puede quedar mal ordenado
				//state    : { order: 'asc' , 'default': false, id: 'state'           }
			// A estos no los ordena correctamente, ya sea porque es una imagen o es un campo calculado cliente side.
				//type     : { order: 'asc' , 'default': false, id: 'type'            }
			},
			campaigns: inConcert.app.getCampaigns(),
			parent : this,
			global_search_filter_by_campaign: undefined,
			permissions: {
//				active       : 'int_view_active',
//				live         : 'int_view_live',
				archive      : 'int_view_archive',
				queued       : 'int_view_queued',
				waiting      : 'int_view_waiting',
				abandoned    : 'int_view_abandoned',
				appointment  : 'int_view_appointment',
				spam         : 'int_view_spam',
				global_search: 'int_view_global_search',
				search_tickets : 'int_view_search_tickets',
				bot_interactions : 'sup_monitoring_bot_interaction'
				// el permiso de la vista de tickets se obtiene de la misma clase "helpdesk_gui --> HelpDeskTickets"
			},
			callbacks: {
				active: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("active");
						// hide
						self.conversations_panels.queued.hide();
						self.$abandoned.hide();
						self.$waiting.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$assigned.show();
						self.conversations_panels.assigned.show_active();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.assigned.on_show_active();
						on_show_folder();
					},
					filter_label: function(filtered_labels) {
						self.conversations_panels.assigned.set_filter_labels(filtered_labels);
						self.conversations_panels.assigned.apply_filter();
					}
				},
				live: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("live");
						// hide
						self.conversations_panels.queued.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$assigned.show();
						self.conversations_panels.assigned.show_live();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.assigned.on_show_live();
						on_show_folder();
					},
					filter_label: function(filtered_labels) {
						self.conversations_panels.assigned.set_filter_labels(filtered_labels);
						self.conversations_panels.assigned.apply_filter();
					}
				},
				appointment: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("appointment");
						// hide
						self.conversations_panels.queued.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$assigned.show();
						self.conversations_panels.assigned.show_appointment();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.assigned.on_show_appointment();
						on_show_folder();
					},
					filter_label: function(filtered_labels) {
						self.conversations_panels.assigned.set_filter_labels(filtered_labels);
						self.conversations_panels.assigned.apply_filter();
					}
				},
				archive: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("archive");
						// hide
						self.conversations_panels.queued.hide();
						self.$abandoned.hide();
						self.$waiting.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$assigned.show();
						self.conversations_panels.assigned.show_archive();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.assigned.on_show_archive();
						on_show_folder();
					},
					filter_label: function(_) {
						self.conversations_panels.assigned.apply_filter();
					}
				},
				queued: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("queued");
						// hide
						self.$assigned.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.conversations_panels.queued.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.queued.on_show_folder();
						on_show_folder();
					},
					filter_label: function(_) {
						self.conversations_panels.queued.apply_filter();
					}
				},
				waiting: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("waiting");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$spam.hide();
						self.$abandoned.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$waiting.show();
						self.conversations_panels.waiting.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.waiting.on_show_folder();
						on_show_folder();
					},
					filter_label: function(_) {
						self.conversations_panels.waiting.apply_filter();
					}
				},
				abandoned: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("abandoned");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$spam.hide();
						self.$waiting.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$abandoned.show();
						self.conversations_panels.abandoned.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.conversations_panels.abandoned.on_show_folder();
						on_show_folder();
					},
					filter_label: function(_) {
						self.conversations_panels.abandoned.apply_filter();
					}
				},
				spam: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("spam");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$spam.show();
						self.conversations_panels.spam.refreshSpamQueue();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: on_show_folder,
					filter_label: $.noop
				},
				integration_helpdesk_tickets: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("integration_helpdesk_tickets");
						// hide
						self.conversations_panels.queued.hide();
						self.$assigned.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$spam.hide();
						self.$global_search.hide();
						self.$helpdesk_tickets.hide();
						self.others_panels.shared.hide();
						$("#interactionQueryBuilderContainer").hide();
						//show
						self.$helpdesk_tickets.show();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS)
							.addClass(CSS_BODY_HELPDESK_CLASS);
						if (self.navigation) {
							self.navigation.enter_tickets();
						}
						return true;
					},
					on_show_folder: on_show_folder,
					filter_label: $.noop
				},
				global_search: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("global_search");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$spam.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$global_search.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: function() {
						self.on_show_folder_global_search();
						on_show_folder();
					},
					filter_label: function(_) {
						self.queryBuilderObj.onReadyToRefresh();
					}
				},
				search_tickets: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("search_tickets");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$spam.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$search_tickets.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: on_show_folder,
					filter_label: $.noop
				},
				categorie: {
					show: function() {
						//queryBuilder
						self.queryBuilderObj.switchToPanel("categorie");
						// hide
						self.$assigned.hide();
						self.conversations_panels.queued.hide();
						self.$spam.hide();
						self.$waiting.hide();
						self.$abandoned.hide();
						self.$helpdesk_tickets.hide();
						//show
						self.$categorie.show();
						self.others_panels.shared.show();
						$("#interactionQueryBuilderContainer").hide();
						$body.removeClass(CSS_BODY_HELPDESK_CLASS);
						return true;
					},
					on_show_folder: on_show_folder,
					filter_label: $.noop
				}
			}
		});

		const parent_panel = this;
		this.conversations_panels.assigned  = new inConcert.AssignedInteractionsPanel('panelAssignedInteractions', parent_panel);
		this.conversations_panels.queued    = new inConcert.QueuedInteractionsPanel('panelQueuedInteractions', parent_panel);
		this.conversations_panels.waiting   = new inConcert.WaitingInteractionsPanel('int_panel_waiting', parent_panel);
		this.conversations_panels.abandoned = new inConcert.AbandonedInteractionsPanel('int_panel_abandoned', parent_panel);
		this.conversations_panels.spam      = new inConcert.SpamInteractionsPanel('int_panel_spam', parent_panel);

		ConversationsPanel.uber.constructor.call(this, id, "<%=_T('Conversations Panels')%>");
	}

	inConcert.Utils.inherit(ConversationsPanel, inConcert.Panel);

	if (typeof inConcert.ConversationsPanel !== 'function') {
		inConcert.ConversationsPanel = ConversationsPanel;
	}

	ConversationsPanel.prototype.initialize = function(callback = $.noop)
	{
		const self = this;

		const main = this.__get_main_window();
		main.showGenericWaiting("<%=_T('Loading conversations, wait moment please...')%>");

		var end = function() {
			self.navigation.select_default_folder_item();
			const timeout_ms = 1500; // le damos un tiempo al waiting para que la pagina quede bien cargada
			main.hideGenericWaiting(timeout_ms);
		};

		this.conversations_panels.assigned.initialize(function() {
			self.conversations_panels.queued.initialize(function() {
				// TODO: cuando se inicie el panel de SPAM hay que revisar si es necesario que tenga un respuesta de callback
				//self.conversations_panels.spam.initialize();
				end();
			});
		});

		callback(function() {
			self.others_panels.shared = self.__get_shared_panel();
		});
	};

	ConversationsPanel.prototype.can_reposition = function() { return true; };

	ConversationsPanel.prototype.enter = function() {
		if (this.navigation) {
			this.navigation.enter();
		}
	};

	ConversationsPanel.prototype.on_show_folder_global_search = function() {
		this.on_show_folder();
	};

	// TODO: #9778 - CANY - Docked.
			ConversationsPanel.prototype.get_panel_info = function(panel_id, interaction) {
				let info = {
					$panel             : null,
					iFrames_containers : {},
					navigation_panel_id: panel_id
				};
				if (panel_id === "active" || panel_id === "live" || panel_id === "archive" || panel_id === "appointment") {
					info.$panel = this.$assigned;
					info.iFrames_containers.active = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppActivePanel"),
						selected  : function() {
							if (interaction) {
								return (interaction.isAssigned() || interaction.isBeingComposed()) && !interaction.isLive();
							}
							return panel_id === "active";
						}()
					};
					info.iFrames_containers.live = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppLivePanel"),
						selected  : function() {
							if (interaction) {
								return (interaction.isAssigned() || interaction.isBeingComposed() || interaction.isWrappedUp() || interaction.isLocked()) && interaction.isLive();
							}
							return panel_id === "live";
						}()
					};
					info.iFrames_containers.archive = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppArchivedPanel"),
						selected  : function() {
							if (interaction) {
								return interaction.isArchived();
							}
							return panel_id === "archive";
						}()
					};
					info.iFrames_containers.appointment = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppCitesPanel"),
						selected  : function() {
							if (interaction) {
								return interaction.isAppointment();
							}
							return panel_id === "appointment";
						}()
					};
					return info;
				}
				else if (panel_id === "queued") {
					info.$panel = $("#panelQueuedInteractions");
					info.iFrames_containers.queued = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppQueuedPanel"),
						selected  : true
					};
					return info;
				}
				else if (panel_id === "abandoned") {
					info.$panel = $("#int_panel_abandoned");
					info.iFrames_containers.abandoned = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppAbandonedPanel"),
						selected  : true
					};
					return info;
				}
				else if (panel_id === "waiting") {
					info.$panel = $("#int_panel_waiting");
					info.iFrames_containers.abandoned = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppWaitingPanel"),
						selected  : true
					};
					return info;
				}
				else if (panel_id === "global_search") {
					info.$panel = $("#int_panel_global_search");
					info.iFrames_containers.abandoned = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionAppGlobalSearchPanel"),
						selected  : true
					};
					return info;
				}
				else if (panel_id === "spam") {
					info.$panel = this.$spam;
					info.iFrames_containers.spam = {
						$container: $("#iFramesWindowsDailogsContainerForInteractionApp"), // TODO: si algun dia se implementa hay que modificarlo.
						selected  : true
					};
					return info;
				}
				throw("error 111");
			};

			ConversationsPanel.prototype.get_interaction_panel_info = function(interaction, force_navigation_panel_id) {
				let navigation_panel_id = force_navigation_panel_id || this.navigation.get_panels_interaction_belongs_to(interaction);
				let info = this.get_panel_info(navigation_panel_id, interaction);
				return info;
			};
	// ------------------------------------------------------

	ConversationsPanel.prototype.__get_main_window = function() {
		return inConcert.app.getMainWindow();
	};

	ConversationsPanel.prototype.__get_shared_panel = function() {
		const main = this.__get_main_window();
		return main.getPanel('shared');
	};

	ConversationsPanel.prototype.get_childs_panel = function() { return this.conversations_panels; };

	ConversationsPanel.prototype.get_child_panel = function( panel_id )
	{
		var panel = this.conversations_panels[panel_id];
		if ( !panel ) {
			throw 'ConversationsPanel.get_conversation_panel - panel_id: "' + panel_id + '" unrecognized panel id';
		}
		return panel;
	};

	ConversationsPanel.prototype.refresh_panel = function(allowed_app_interaction)
	{
		if (!allowed_app_interaction) {
			return;
		}
		var selected_folder = this.navigation.get_selected_folder_id();
		if ( 'active' === selected_folder || 'live' === selected_folder || 'appointment' === selected_folder ) {
			this.conversations_panels.assigned.refreshPanel();
		}
		else if ( 'queued' === selected_folder ) {
			this.conversations_panels.queued.refreshPanel();
		}
		//this.conversations_panels.spam.refreshPanel();
		// el panel waiting no se refresca, es estatico
	};

	ConversationsPanel.prototype.refresh_permission_gui = function(allowed_app_interaction)
	{
		this.__refresh_navigation(allowed_app_interaction);
		this.__refresh_bulk_actions(allowed_app_interaction);
	};

	ConversationsPanel.prototype.__refresh_navigation = function(allowed_app_interaction)
	{
		if (!allowed_app_interaction) {
			return;
		}
		this.navigation.toggle_navigation_items();
	};

	ConversationsPanel.prototype.__refresh_bulk_actions = function(allowed_app_interaction)
	{
		if (!allowed_app_interaction) {
			return;
		}
		var assignedPanel = this.get_child_panel('assigned');
		if (assignedPanel) {
			assignedPanel.updateBatchActions();
		}
	};

	ConversationsPanel.prototype.reset = function()
	{
		this.conversations_panels.assigned.reset();
		this.conversations_panels.queued.reset();
		this.conversations_panels.waiting.reset();
		this.conversations_panels.abandoned.reset();
		this.conversations_panels.spam.reset();
	};

	ConversationsPanel.prototype.close_all_clients = function()
	{
		this.conversations_panels.assigned.closeAllClients();
		this.conversations_panels.queued.closeAllClients();
		this.conversations_panels.waiting.closeAllClients();
		this.conversations_panels.abandoned.closeAllClients();
		this.conversations_panels.spam.closeAllClients();
	};

	ConversationsPanel.prototype.switch_window_interaction = function( old_id, interaction )
	{
		if ( this.conversations_panels.assigned.switchWindowInteraction ) {
			this.conversations_panels.assigned.switchWindowInteraction(old_id, interaction);
		}
		if ( this.conversations_panels.queued.switchWindowInteraction ) {
			this.conversations_panels.queued.switchWindowInteraction(old_id, interaction);
		}
	};

	ConversationsPanel.prototype.update_counters = function( info )
	{
		this.navigation.update_counters( info );
	};

	ConversationsPanel.prototype.get_selected_labels = function(simple_item)
	{
		return this.navigation.get_selected_labels(simple_item);
	};

	ConversationsPanel.prototype.get_navigation = function()
	{
		return this.navigation;
	};

	ConversationsPanel.prototype.move_to_live_panel = function()
	{
		this.get_navigation().select_folder('live', true);
	};

	ConversationsPanel.prototype.move_to_active_panel = function()
	{
		this.get_navigation().select_folder('active');
	};

	ConversationsPanel.prototype.onReadyToRefresh = function()
	{
		switch(this.navigation.current_item) {
			case "global_search":
				if(this.navigation.global_search.queryBuilderObj.queryBuilderCurrentPanel === this.navigation.current_item) {
					this.navigation.global_search.search(false);
				}
				break;
			case "abandoned":
				if(this.conversations_panels[this.navigation.current_item].abandoned_search.queryBuilderObj.queryBuilderCurrentPanel === this.navigation.current_item) {
					this.conversations_panels[this.navigation.current_item].abandoned_search.search(false);
				}
				break;
			case "waiting":
				if(this.conversations_panels[this.navigation.current_item].waiting_search.queryBuilderObj.queryBuilderCurrentPanel === this.navigation.current_item) {
					this.conversations_panels[this.navigation.current_item].waiting_search.search(false);
				}
				break;
			case "archive":
				this.conversations_panels.assigned.searchArchive(false);
				break;
			case "queued":
				this.conversations_panels.queued.refresh_queued();
				break;
			case "active":
			case "live":
			case "appointment":
				this.conversations_panels.assigned.apply_filter();
				break;
		}
	};

	ConversationsPanel.prototype.ReloadHelpdeskTickets = function(callback = $.noop)
	{
		if (!this.navigation) {
			callback({success: false, reason: "there is no instance of the navigation class"});
			return;
		}
		this.navigation.ReloadHelpdeskTickets(callback);
	};
}());
