/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert, dhtmlXTreeObject*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const FORMAT = inConcert.Utils.format;
	const ZEROPAD = inConcert.Utils.zeropad;
	const ROOT_RESULT_ID = "0";
	const SON_BRANCH = 0;

	/**
	Class DispositionCodesPopup.
	*/
	function DispositionCodesPopup (params)
	{
		DispositionCodesPopup.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "disposition_codes";
		this.ownerId = "window_disposition_codes_popup_container";
		this.canReposition = true;
		this.maxCharsOnComment = 500;
		this.from_app = params.from_app;
		this.action_type = params.action_type; // para saber si tiene que finalizar la interaccion, o si tiene que retornar el valor seleccionado a quien ha instanciado el popup.
		this.loading_tree = false;
	}

	inConcert.Utils.inherit(DispositionCodesPopup, inConcert.PopupWindow);

	//----------------------------
	if (typeof(inConcert.DispositionCodesPopup) !== "function") {
		inConcert.DispositionCodesPopup = DispositionCodesPopup;
	}

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.displayStatusMessage = function(text, type, closeDisabled, timeout_ms ) {
		var section = $("#disposition_codes_message");
		section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#disposition_codes_close_message").hide();
		}
		else {
			$("#disposition_codes_close_message").show();
		}
	};

	/*******************************************************************
	 * funcion que oculta mensaje
	 */
	DispositionCodesPopup.prototype.hideStatusMessage = function() {
		$("#disposition_codes_message").hide();
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.show_waiting = function( message ) {
		var $waiting = $("#disposition_codes_waiting");
		$waiting.show().find("[message]").text(message);
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.hide_waiting = function() {
		$("#disposition_codes_waiting").hide();
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.getResultData = function(resultId) {
		resultId = resultId || this.treeResults.getSelectedItemId();
		return JSON.parse(this.treeResults.getUserData(resultId, "result"));
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.getRescheduleDate = function(utc) {
		var $datepicker = $("#disposition_code_popup_rescheduled_date");
		if ($datepicker.val().length === 0) {
			return {
				error:		true,
				message:	"<%=_T('Please enter rescheduling date.')%>"
			};
		}
		var time = $("#disposition_code_popup_rescheduled_time").val();
		if (time.length === 0) {
			return {
				error:		true,
				message:	"<%=_T('Please enter rescheduling time.')%>"
			};
		}
		var matchTime = time.match(/(\d\d):(\d\d)/);
		if (!matchTime || matchTime[1] > 23 ) {
			return {
				error:		true,
				message:	"<%=_T('Please enter hour between 0 and 23.')%>"
			};
		}
		if (!matchTime[2] || matchTime[2] > 59) {
			return {
				error:		true,
				message:	"<%=_T('Please enter minutes between 0 and 59.')%>"
			};
		}
		var date = $datepicker.datepicker("getDate");
		date.setHours(matchTime[1], matchTime[2]);
		// TODO: Se debe averiguar en que timezone esta el contacto, ya que el schedule se debe hacer con el timezone del contacto.
		if (utc) {
			return date.getUTCFullYear() + "/" + ZEROPAD(date.getUTCMonth() + 1) + "/" + ZEROPAD(date.getUTCDate()) + " " +
				ZEROPAD(date.getUTCHours()) + ":" + ZEROPAD(date.getUTCMinutes()) + ":00";
		}
		else {
			return date.getFullYear() + "/" + ZEROPAD(date.getMonth() + 1) + "/" + ZEROPAD(date.getDate()) + " " +
				ZEROPAD(date.getHours()) + ":" + ZEROPAD(date.getMinutes());
		}
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.validate = function() {
		var result = this.getResultData();
		var message;
		if (this.needReschedule(result)) {
			if ($("#disposition_code_popup_rescheduled_wait").is(":visible")) {
				message ="<%=_T('Please wait loading rescheduling list.')%>";
				this.displayStatusMessage(message, "Error", undefined, 2500);
				return false;
			}
			var rescheduleDate = this.getRescheduleDate();
			if (rescheduleDate.error) {
				this.displayStatusMessage(rescheduleDate.message, "Error", undefined, 2500);
				return false;
			}
			rescheduleDate = new Date(rescheduleDate);
			// TODO: Hay que revisar porque la validacion de fecha/hora podria estar mal si el contacto esta en un timezone diferente al agente.
			var now = new Date();
			if (rescheduleDate <= now) {
				message ="<%=_T('Date and time to reschedule must be greater than the current date and time.')%>";
				this.displayStatusMessage(message, "Error", undefined, 2500);
				return false;
			}
		}
		return true;
	};

	/*******************************************************************
	 *  TODO: (Ref #7571) Activar si en algun momento se realiza la parte de schedule de interacciones.
	 */
	DispositionCodesPopup.prototype.needReschedule = function(/*resultConfig*/)
	{
		// if (!resultConfig) {
			// throw("DispositionCodesPopup.prototype.needReschedule. Result reschedule is required.")
		// }
		// return resultConfig.needReschedule ? true : false;
		return false;
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.getDispositionCodeTreePath = function(itemId)
	{
		itemId = itemId || this.treeResults.getSelectedItemId();
		var dispositionCodeTreePath = [];
		while (itemId) {
			dispositionCodeTreePath.unshift(this.getResultData(itemId).name);
			itemId = this.treeResults.getParentId(itemId);
		}
		return dispositionCodeTreePath;
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.getResult = function()
	{
		const resultConfig = this.getResultData();
		let disposition = {
			name     : resultConfig.name,
			treePath : this.getDispositionCodeTreePath(),
			dialer : []
		};

		const first_interaction = this.data.interactions[0];
		const is_call = this.interactionIsCall(first_interaction);
		const is_voicemail = this.interactionIsVoicemail(first_interaction);

		if (is_call || is_voicemail) {
			disposition.name = resultConfig.code || resultConfig.name;
			disposition.dialer = first_interaction.interactionData && first_interaction.interactionData.dialer || [];
			disposition.dialer.contactName = first_interaction.interactionData && first_interaction.interactionData.contactName || "";
			disposition.dialer.contactCode = resultConfig.contactCode || 0;
		}

		if (this.needReschedule(resultConfig)) {
			const utc = true;
			disposition.rescheduleDate = this.getRescheduleDate(utc);
			const $rescheduleTo = $("#disposition_code_popup_rescheduled_selected > .rescheduleAssignedToItem .assignedToItemText");
			disposition.rescheduleTo = {
				type:	$rescheduleTo.attr("item_type"),
				id:		$rescheduleTo.attr("item_id")
			};
			if (is_call || is_voicemail) {
				disposition.alternativeNumber = $("#disposition_codes_alt_number").val();
			}
		}
		return disposition;
	};

	/*******************************************************************
	 * Finish interactions and save the current disposition code and comment.
	 */
	DispositionCodesPopup.prototype.finishInteractions = function() {
		var self = this;
		var $cmdFinish = $("#disposition_codes_popup_finish_interaction");
		if (!this.validate()) {
			$cmdFinish.prop("disabled", false);
			return;
		}

		var result = this.getResult();
		var comment = "";
		if (this.hasPermissionToComment()) {
			comment = $("#disposition_code_popup_comment").val();
		}

		// 1 - Aca se realiza accion de retornar la info del dispositon seleccionado a quien ha instanciado el popup de dispositions (ej: al transferir un mail a campana)
		if (this.action_type === "NOT_FINISH - GET_DISPOSITION") {
			let disposition_info = {
				disposition: result,
				comment: comment
			};
			this.parent_on_finish(disposition_info);
			this.close();
			return;
		}

		// 2 - Aca realiza la finalizacion de la(s) interaccion(es)
		var interactions = this.data.interactions;
		var count_interactions = interactions.length;
		var mainWindow = inConcert.app.getMainWindow();

		var panel = mainWindow.getPanel("AssignedInteractions");
		var batchFunction = function( ints, dispositionCode, new_comment, onArchived ) {
			return inConcert.app.getInteractionManager().batchArchive( ints, dispositionCode, new_comment, onArchived, self.from_app );
		};
		let args = [result, comment];

		var getConflictMessage = function(error, count_errors)
		{
			switch (error)
			{
				case "INVALID_REQUEST":
					return "<%=_T('Invalid request please try again later')%>";

				case "FORBIDDEN":
					return FORMAT("<%=_T('You have not permissions to finish {0} conversations on campaign.')%>", count_errors);

				case "FORBIDDEN_COMMENT":
					return FORMAT("<%=_T('{0} conversations have not been comment, because you have not permissions on campaign.')%>", count_errors);

				case "GENERIC_ERROR":
					return FORMAT("<%=_T('{0} failed finished conversations, please try again later')%>", count_errors);

				case "GENERIC_WARNING":
					return "<%=_T('Warning, some conflicts have occurred finished conversations, please contact your supervisor')%>";

				case "UNABLE_TO_ARCHIVE":
					return "<%=_T('Unable to finish conversation, please try again later')%>";

				default:
					return "<%=_T('Unable to finish conversation, please try again later')%>";
			}

		};

		var callbacks = {
			onFinished : function(result)
			{
				$("#close_disposition_codes_popup").trigger("click");

				var message      = null;
				var message_type = "Info";
				var timeout_ms   = 5000;
				var errors       = result.errors;
				var warnings     = result.warnings;

				if ( errors && errors.count_total > 0 ) {
				// con errores
					message_type = "Error";
					if (errors.count_types === 1) {
						var error_type = errors.types.pop();
						message = getConflictMessage(error_type, errors.count_total);
					}
					else {
						message = getConflictMessage("GENERIC_ERROR", errors.count_total);
					}
				}
				else {
					if ( warnings && warnings.count_total > 0 ) {
					// con conflictos
						message_type = "Alert";
						if ( warnings.count_types === 1 ) {
							var warning_type = warnings.types.pop();
							message = getConflictMessage(warning_type, warnings.count_total);
						}
						else {
							message = getConflictMessage("GENERIC_WARNING", warnings.count_total);
						}
					}
				}

				if ( message === null ) {
				// aca entra si es success, es decir si no tuvo ningun error ni ningun warning
					self.parent_on_finish(true);
					if ( count_interactions <= 1 ) {
						// no muestra nignun mensaje si finalizo con exito y es una unica interaccion.
						return;
					}
					message = "<%=_T('All conversations were finished successfully!')%>";
				}
				mainWindow.showBannerMessage({ message: message, messageType: message_type, timeout: timeout_ms });
				self.parent_on_finish(true);
			},
			onPartial: function( /* successfulNodes , unsuccessfulNodes */ ) {},
			onFailure: function(errorMessage)
			{
				let message = getConflictMessage(errorMessage);
				mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout : 5000 });
				self.parent_on_finish(false, message);
				self.close();
			}
		};

		var batchData = new panel.BatchData(batchFunction, interactions, args, callbacks);
		panel.processBatchInteractions(batchData);
	};

	/********************************************************************
	 * Esta funcion valida si se debe truncar los comentarios.
	 */
	DispositionCodesPopup.prototype.truncateComment = function()
	{
		var $txtComment = $("#disposition_code_popup_comment");
		var currentChars = document.getElementById($txtComment.attr("id")).value.length;
		var charsDifference = this.maxCharsOnComment - currentChars;
		var disabledFinishButton = charsDifference < 0;
		$("#disposition_codes_popup_finish_interaction").prop("disabled", disabledFinishButton);
		var $characterComment = $("#disposition_codes_popup_character_comment");
		$characterComment.text(charsDifference);
		if (disabledFinishButton) {
			$characterComment.addClass("commentDispositionCodeCharacterCounterNegative");
		}
		else {
			$characterComment.removeClass("commentDispositionCodeCharacterCounterNegative");
		}
	};

	/*******************************************************************
	 * Funcion que evalua si la interaccion tiene permisos para comentar
	 */
	DispositionCodesPopup.prototype.hasPermissionToComment = function()
	{
		if (this.data.permissionToComment !== undefined) {
			return this.data.permissionToComment;
		}
		this.data.permissionToComment = this.data.interactions[0].hasPermission( "COMMENT", this.from_app );
		return this.data.permissionToComment;
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	DispositionCodesPopup.prototype.setEvents = function() {
		var self = this;

		$("#disposition_codes_popup_finish_interaction")
			.bind("click", function() {
				if (self.loading_tree) {
					return;
				}
				$(this).prop("disabled", true);
				self.finishInteractions();
			});
		var closeDialog = function() {
			self.close();
		};
		// evento que cierra el dialogo y no realiza cambios
		$("#disposition_codes_popup_cancel_interaction").bind("click", closeDialog);
		// evento que cierra el dialogo y no realiza cambios
		$("#close_disposition_codes_popup").bind("click", closeDialog);

		// este evento lo que realiza es cerrar la lista desplegable si se ha dado click fuera de ella.
		// dentro de otro control del popup.
		$("#window_disposition_codes_popup_container")
			.bind("click", function(e) {
				var $listContainer = $("#disposition_code_popup_rescheduled_list");
				if (!$listContainer.is(":visible")) {
					return false;
				}
				e = e || event;
				var target = e.target || e.srcElement;
				if (!target) {
					return;
				}
				var listId = "disposition_code_popup_rescheduled_container";
				do {
					if (target.id === listId) {
					// El click se ha producido dentro del elemento, no se hace nada.
						return;
					}
					if (target.id === self.ownerId) {
					// El click se ha producido dentro del popup pero fuera de la lista. Cierro
						break;
					}
					target = target.parentNode;
				} while (target);
				// Se ha cliqueado fuera del elemento, se cierra la lista desplegable.
				$listContainer.hide();
			});

		// evento que visualiza u oculta la lista desplegable con la campana y los contactos a reagendar
		$("#disposition_code_popup_rescheduled_selected")
			.bind("click", function() {
				var $listContainer = $("#disposition_code_popup_rescheduled_list");
				var method = $listContainer.is(":visible") ? "hide" : "show";
				$listContainer[method]();
			});
		//este evento es producido cuando se selecciona un elemento en la lista desplegable
		$("#disposition_code_popup_rescheduled_list .rescheduleAssignedToItem")
			.off("click")
			.on("click", function() {
				var newSelectedMarkup = $(this).html();
				$("#disposition_code_popup_rescheduled_selected .rescheduleAssignedToItem")
					.html(newSelectedMarkup)
					.trigger("click");			// lo oculto
			});
		//
		$("#disposition_codes_close_message")
			.bind("click", function() {
				self.hideStatusMessage();
			});

		if (this.hasPermissionToComment()) {
			//
			$("#disposition_code_popup_comment")
				.bind("change", function() {
					self.truncateComment();
				})
				.bind("keyup", function() {
					self.truncateComment();
				});
			//
			// $("#disposition_code_popup_comment_title")
			// 	.bind("click", function() {
			// 		var $commentPanel = $("#disposition_code_popup_comment_panel");
			// 		if ($commentPanel.is(":visible")) {
			// 			$commentPanel.hide();
			// 			$(this).removeClass("subtitleUncollapsed").addClass("subtitleCollapsed");
			// 		}
			// 		else {
			// 			$commentPanel.show();
			// 			$(this).removeClass("subtitleCollapsed").addClass("subtitleUncollapsed");
			// 			$("#disposition_code_popup_comment").focus();
			// 		}
			// 		self.onChangePopupHeight(); // recalcula la posicion si cambia de tamano y el popup quedara fuera de ventana.
			// 	});
		}
	};

	/*******************************************************************
	 * add child result to tree
	 */
	DispositionCodesPopup.prototype.addChildResultToTree = function(result, resultId, parentId) {
		// The SELECT parameter remains selected the added element.
		this.treeResults.insertNewChild(parentId, resultId, result.name, 0, 0, 0, 0, "SELECT");
		this.treeResults.setUserData(resultId, "result", JSON.stringify(result));
	};

	/*******************************************************************
	 * add sibling result to tree
	 */
	DispositionCodesPopup.prototype.addSiblingResultToTree = function(result, resultId, parentId) {
		// The SELECT parameter remains selected the added element.
		this.treeResults.insertNewNext(parentId, resultId, result.name, 0, 0, 0, 0, "SELECT");
		this.treeResults.setUserData(resultId, "result", JSON.stringify(result));
	};

	/*******************************************************************
	 * fill branch the tree with all results
	 */
	DispositionCodesPopup.prototype.fill_branch = function(dispositionCodes, currentFinalizedCode, root_id, on_end)
	{
		/* Sample of the expected results on the json structure
		 *
			[
				{
					name: "No Compro",
					isGoal: false,
					needReschedule :true,
					dialerCode: "DIALER_CODE"
				},
				{
					name: "No compro porque ha fallecido!!",
					isGoal: false,
					changeContactState: "DEAD"
				},
				{
					name: "Compro",
					childs: [
						{
							name: "Efectivo",
							isGoal: true
						},
						{
							name: "Tarjeta",
							childs: [
								{name: "Visa",         isGoal: false},
								{name: "Master",       isGoal: false}
							]
						}
					]
				}
			]
		 */
		var self = this;

		on_end = on_end || $.noop;

		if (!dispositionCodes) {
			throw("Disposition codes configurations does not exit");
		}
		var currentDispositionCode;
		if (currentFinalizedCode) {
			currentDispositionCode = currentFinalizedCode.name;
		}
		var selectedId;

		var selectResultOnTree = function()
		{
			if ( root_id === ROOT_RESULT_ID ) {
				self.treeResults.closeAllItems(root_id);
			}
			var resultIds = self.treeResults.getSubItems(root_id).split(self.treeResults.dlmtr);
			if (!resultIds) {
				return;
			}
			selectedId = resultIds[0]; // selecciono el primer resultado/categoria del tree
			self.treeResults.selectItem(selectedId);							// selecciono item en tree
			self.onSelectResultInTree(self, selectedId, currentFinalizedCode);	// disparo evento selected
		};

		this.treeResults.deleteChildItems(root_id);
		// recursive function that adds results on the tree
		var timeout_counter = 0;
		var total_added = 0;
		this.pendings_branchs = this.pendings_branchs || {};

		var fill = function(dispositions, parentId)
		{
			if ( !self.opened ) {
				//console.log("1 fill dispositions: Me voy -> self.opened: " + self.opened);
				// si el popup esta cerrado, me voy
				self.loading_tree = false;
				on_end();
				return;
			}
			//console.log("fill dispositions: " + parentId);
			// Lo agrego dentro de la funcion foreach_timeout para que el navegador no quede colgado.
			// Como los timeout se tiran por cada invocacion a funcion recursiva del tree, tengo
			//   un contador "timeout_counter", que se va incrementando en cada invocacion a la funcion
			//   y va decrementando cuando el loop timeout culmina para cada invocacion.
			// Entonces con esto, cuando el contador quede en cero nuevamente, ahi ya termino la carga total del tree.
			timeout_counter++;
			inConcert.Utils.foreach_timeout(
			{
				rows : dispositions,
				//max_append_per_loop: 500,
				pause_between_loop_ms: 2,
				append_fn : function( result, added )
				{
					if ( !self.opened ) {
						//console.log("2 [append_fn] fill dispositions: Me voy -> self.opened: " + self.opened);
						// si el popup esta cerrado, me voy
						self.loading_tree = false;
						on_end();
						return { cancel: true };
					}
					total_added++;
					var result_id = inConcert.Utils.generateGuid();
					var first_child = added;
					if (first_child === SON_BRANCH) {
						self.addChildResultToTree(result, result_id, parentId);
					}
					else {
						self.addSiblingResultToTree(result, result_id, parentId);
					}
					//console.log("1 append_fn fill dispositions: " + result_id);
					parentId = result_id;
					// si tiene childs es categoria y sino es resultado
					if (result.childs) {
						// Agrego un unico hijo para que me aparezca la cruz colapsada.
						// luego si me expanden la rama, tengo que eliminar este temoral hijo, y agregar toda la rama hija.
						// Tambien agrego 'pendings_branchs', toda la rama de dispositions, para que luego se pueda cargar al expandir la misma
						var temporal_result_id = inConcert.Utils.generateGuid();
						var temporal_result = result.childs[0];
						self.addChildResultToTree(temporal_result, temporal_result_id, parentId);
						self.pendings_branchs[parentId] = {
							temporal_result_id: temporal_result_id,
							parentId: parentId,
							childs: result.childs
						};
					}
					else {
						// si es una interaccion reabierta me quedo con el id del tree del elemento buscado.
						if (currentDispositionCode && !selectedId) {
							if (currentDispositionCode === result.name) {
								selectedId = result_id;
							}
						}
					}
				},
				waiting_fn : function(/*message*/)
				{
					var waiting_message = FORMAT("<%=_T('Please wait, adding {0} dispositions.')%>", total_added);
					self.show_waiting( waiting_message );
					/*
						(#6097) Esta invocacion a la funcion se realiza para que ajuste la poscion del popup en la medida que se cargan los dispositions, sino realiza un juego en pantalla feo (habria que revisarlo con Murias porque hace ese juego de pantalla) por ahora queda asi
						Esto se da en la accion finish de los batch
					*/
					self.setPosition();
				},
				complete_fn : function() {
					//console.log("3 complete_fn - timeout_counter " + timeout_counter);
					if ( !self.opened ) {
						//console.log("3 [complete_fn] fill dispositions: Me voy -> self.opened: " + self.opened);
						// si el popup esta cerrado, me voy
						self.loading_tree = false;
						on_end();
						return;
					}
					timeout_counter--;
					//console.log("3.1 complete_fn - timeout_counter: " + timeout_counter);
					if ( timeout_counter <= 0 ) {
						//console.log("4 complete_fn - ENDDDDDDDDDDD ");
						// aca termino la carga total del tree
						self.loading_tree = false;
						selectResultOnTree();
						self.hide_waiting();
						on_end();
					}
				}
			});
		};

		var waiting_message = "<%=_T('Please wait, adding dispositions.')%>";
		this.show_waiting( waiting_message );
		this.loading_tree = true;
		fill(dispositionCodes, root_id); // start fill
	};

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	DispositionCodesPopup.prototype.createMarkup = function() {
		$("#window_disposition_codes_popup_container").remove();
		var node = $("".concat(
			"<div id='window_disposition_codes_popup_container' class='systemPopWindow windowDispositionCodesContainer internalPopUpContainer popUp520w' style='display:none;'>",//windowContainer

				"<div class='internalPopUp'>",

					"<div class='windowTop '>",
						"<div class='windowTitle windowDisposition internalPopUpTitle'>",
							"<div class='windowDispositionCodesTitleText'></div>",
							"<img id='close_disposition_codes_popup' src='/inconcert/apps/agent/images/blank.png' class='windowDispositionCodesButClose' hidden />",
						"</div>",
					"</div>",

					"<div id='disposition_codes_waiting' class='windowContainerMessages messageProcess' style='display:none;'>",
						"<div class='windowMessagesText'>",
							"<label message='true' class='windowMessagesTextAux'></label>",
						"</div>",
					"</div>",

					"<div id='disposition_codes_message' class='windowContainerMessages messageError' style='display:none;'>",
						"<a id='disposition_codes_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('close')%>", "</a>",
						"<div class='windowMessagesText'>",
							"<label class='windowMessagesTextAux'></label>",
						"</div>",
					"</div>",

					"<div class='windowMiddle internalPopUpMiddle'>",
						"<div class='dispositionCodesThreadContainer'>",
							"<div id='disposition_codes_tree_results' class='dispositionCodesThreadContentScrollArea'></div>",
						"</div>",
						"<div class='dispositionCodesRescheduledContainer' id='disposition_code_rescheduled_panel' style='display:none;'>",
							"<hr>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<h5>","<%=_T('Reschedule Information')%>","</h5>",
								"</div>",
							"</div>",
							"<div class='row windowDispositionCodesData'>",
								"<div class='col-12'>",
									"<label>", "<%=_T('Date and Time:')%>", "</label>",
									"<div class='d-flex flex-row'>",
										"<input id='disposition_code_popup_rescheduled_date' readonly type='text' class='dispositionCodesInput dispositionCodesDate form-control mr-1' />",
										"<input id='disposition_code_popup_rescheduled_time' type='text' class='dispositionCodesInput dispositionCodesTime form-control ml-1' />",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row windowDispositionCodesData'>",
								"<div class='col-12'>",
									"<label>", "<%=_T('To:')%>", "</label>",
								"</div>",
								"<div class='col-12'>",
									"<div id='disposition_code_popup_rescheduled_container' class='rescheduledAssignedToContainer'>",// destination container
										"<div id='disposition_code_popup_rescheduled_wait' class='dispositionCodesWaiting' style='display:none;'>",
											"<img src='/inconcert/apps/agent/images/blank.png' class='dispositionCodesWaitingAnima' />",
										"</div>",
										"<div id='disposition_code_popup_rescheduled_selected' class='rescheduleAssignedToSelected'></div>",// destination selected
										"<div id='disposition_code_popup_rescheduled_list' class='rescheduleAssignedToList' style='display:none;'></div>",// destination list
									"</div>",
								"</div>",
							"</div>",
							"<div id='disposition_codes_alt_panel' class='row windowDispositionCodesData' style='display:none;'>",
								"<div class='col-12'>",
									"<label>", "<%=_T('Alternative Phone:')%>", "</label>",
									"<input id='disposition_codes_alt_number' type='text' class='dispositionCodesInput dispositionCodesAlternativePhone form-control' maxlength='50' />",
								"</div>",
							"</div>",
						"</div>",

						"<div id='disposition_code_popup_comment_panel' class='dispositionCodesInputContainer'>",
							"<hr/>",
							// "<div id='disposition_code_popup_comment_title' class='dispositionCodesFormSubtitle subtitleCollapsed'>","<%=_T('Comment')%>","</div>",
							"<div class='dispositionCodesCommentContainer'>",
								//"<label for='newCommentText'>", "<%=_T('Comment')%>", ":</label>",
								"<textarea id='disposition_code_popup_comment' disabled='true' class='dispositionCodesInput form-control' placeholder='<%=_T('Comment')%>'></textarea>",
								"<label id='disposition_codes_popup_character_comment' class='commentDispositionCodeCharacterCounter' id='characterCounterComment' style='display:none;'></label>",
							"</div>",
						"</div>",

					"</div>",

					"<div class='windowBottom d-flex justify-content-end internalPopUpButtons'>",
						"<button id='disposition_codes_popup_cancel_interaction' class='pageButtonGeneric buttonColorWhite float-right' ><%=_T('Cancel')%></button>",
						"<button id='disposition_codes_popup_finish_interaction' class='pageButtonGeneric buttonColorBlue float-right' disabled='true' ><%=_T('Confirm')%></button>",
					"</div>",

				"</div>",

			"</div>"));
		$("body").append(node);
		return node;
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.onSelectResultInTree = function(self, undefined /* resultId */, currentFinalizedCode)
	{
		if ( this.loading_tree ) {
			return;
		}
		var result = self.getResultData();
		var $cmdFinish = $("#disposition_codes_popup_finish_interaction");
		var $txtComment = $("#disposition_code_popup_comment");
		var $panelRescheduled =  $("#disposition_code_rescheduled_panel");
		var $characterComment = $("#disposition_codes_popup_character_comment");
		if (result.childs) {
			$cmdFinish.prop("disabled", true);
			$txtComment.prop("disabled", true);
			$characterComment.hide();
			$panelRescheduled.hide();
		}
		else {
			$cmdFinish.prop("disabled", false);
			$txtComment.prop("disabled", false);
			$characterComment.show();
			if (currentFinalizedCode && currentFinalizedCode.comment && currentFinalizedCode.comment.length > 0) {
			// si es una interaccion que se finaliza por segunda vez o mas veces actualizo el comentario
				$txtComment.val(currentFinalizedCode.comment);
				// $("#disposition_code_popup_comment_title").trigger("click");
			}
			self.truncateComment(); // valida si se debe truncar los comentarios
			if (!this.needReschedule(result)) {
				$panelRescheduled.hide();
				return;
			}
			$panelRescheduled.show();
			if (currentFinalizedCode && currentFinalizedCode.rescheduleDate) {
			// si una interaccion que se finaliza por segunda vez o mas veces actualizo la fecha y hora
			//	y convierto de UTC al timezone local
				var parsedDate = currentFinalizedCode.rescheduleDate.match(/(\d\d\d\d)[\/\-](\d\d)[\/\-](\d\d) (\d\d):(\d\d)/);
				var rescheduleDate = new Date(
					Date.UTC(
						parsedDate[1],						//year
						parseInt(parsedDate[2], 10) - 1,	//month
						parsedDate[3],						//day
						parsedDate[4],						//hour
						parsedDate[5]						//minutes
					)
				);
				var stringTime = ZEROPAD(rescheduleDate.getHours()) + ":" + ZEROPAD(rescheduleDate.getMinutes());
				$("#disposition_code_popup_rescheduled_date").datepicker("setDate", rescheduleDate);
				$("#disposition_code_popup_rescheduled_time").val(stringTime);
			}

			var $panelAltNumber = $("#disposition_codes_alt_panel");
			if (self.data.interactions.every(self.interactionIsCall)){
				$panelAltNumber.show();
			}
			else {
				$panelAltNumber.hide();
			}

			self.setPosition(); // recalcula la posicion si cambia de tamano y el popup quedara fuera de ventana.
		}
	};

	/*******************************************************************
	 *
	 */
	DispositionCodesPopup.prototype.fillRescheduleToCombo = function() {
		var self = this;
		// esta funcion agrega los elementos a la lista desplegable
		var addItem = function(itemType, id, name, avatar, selected) {
			var markup =
				"<div class='rescheduleAssignedToItem'>" +
					"<img class='assignedToItemAvatar' src='" + inConcert.app.getBaseUrl() + avatar + "' />" +
					"<label item_id='" + id + "' item_type='" + itemType + "' class='assignedToItemText'>" + name + "</label>" +
				"</div>";
			$("#disposition_code_popup_rescheduled_list").append(markup); // agrego a la lista desplegable
			if (selected) {
				$("#disposition_code_popup_rescheduled_selected").append(markup); // agrego el seleccionado
			}
		};
		//agrego a la lista la campana
		var campaign = inConcert.app.getCampaign(this.data.interactions[0].campaign);
		if (!campaign) {
			throw("Campaign does not exist");
		}
		var $waiting = $("#disposition_code_popup_rescheduled_wait");
		$waiting.show();
		var disposition = self.isBatch ? null : self.data.interactions[0].getLastDispositionFromSteps();
		var selected;
		if (!disposition) {
		// si es la primera vez que se finaliza una interaccion, selecciono la campana
			selected = true;
		}
		else {
		//sino, valido si se reagendo a una campana y la selecciono.
			var to = disposition.rescheduleTo;
			selected = !to || to.type === "campaign";
		}
		addItem("campaign", campaign.Id, campaign.getDescription(), campaign.getAvatarURI(), selected);

		//agrego a la lista los members de la campana
		campaign.getTeamMembers(
			function(members, error) {
				if (!members) {
					$waiting.hide();
					var errorMessages = {
						"INVALID_REQUEST": "<%=_T('There was an error retrieving users list, please try again later.')%>"
					};
					var message = errorMessages[error] || errorMessages.INVALID_REQUEST;
					self.displayStatusMessage(message, "Error", undefined, 2500);
					return;
				}
				// sort list by member name
				var campaignMembers = inConcert.Utils.sortArray(members, "Name");
				// add members to combo
				for (var mi = 0; mi < campaignMembers.length; mi++) {
					var member = campaignMembers[mi];
					//valido si se reagendo a un member y lo selecciono.
					selected = to && to.type === "member" && to.id === member.Id;
					addItem("member", member.Id, member.Name, ("/users/get_avatar/" + member.Id), selected);
				}
				$waiting.hide();
			},
			true // from cache
		);
	};

	/*******************************************************************
	 * Metodo que visualiza el popup con los disposition codes e inicializa eventos de la ventana!
	 */
	DispositionCodesPopup.prototype.initialize = function()
	{
		var self = this;

		this.data.container = this.createMarkup();
		this.parent_on_finish = this.data.callbacks.on_finish || $.noop;
		this.data.threadContainer = this.data.container.find(".dispositionCodesThreadContentScrollArea");
		this.data.isBatch = this.data.interactions.length > 1;

		// popup title
		var popupTitle = function() {
			if (self.data.isBatch){
				return "<%=_T('Batch Disposition')%>";
			}
			var channelNames = {
				CALL       : "<%=_T('Call')%>"        ,
				CHAT       : "<%=_T('Web chat')%>"    ,
				FACEBOOK   : "<%=_T('Facebook')%>"    ,
				INSTAGRAM  : "<%=_T('Instagram')%>"   ,
				YOUTUBE	   : "<%=_T('Youtube')%>"     ,
				APPSTORE   : "<%=_T('Appstore')%>"    ,
				GOOGLEPLAY : "<%=_T('Googleplay')%>"  ,
				RCS	   	   : "<%=_T('Rcs')%>"     	  ,
				IM         : "<%=_T('Chat')%>"        ,
				MAIL       : "<%=_T('E-Mail')%>"      ,
				SMS        : "<%=_T('SMS')%>"         ,
				WHATSAPP   : "<%=_T('Whatsapp')%>"    ,
				TWITTER    : "<%=_T('Twitter')%>"     ,
				WEBCONTACT : "<%=_T('Web contact')%>" ,
				VOICEMAIL  : "<%=_T('Voice mail')%>"
			};
			var interactionType = self.data.interactions[0].type;
			var channelName = channelNames[interactionType];
			if (!channelName) {
				throw("No channel name found for the interaction of type " + interactionType);
			}
			return FORMAT("<%=_T('{0} Disposition')%>", channelName);
		}();
		$("#window_disposition_codes_popup_container .windowDispositionCodesTitleText").text(popupTitle);
		// numero maximo de caracteres en el comentario
		$("#disposition_codes_popup_character_comment").text(this.maxCharsOnComment);
		// initialize dhtmlx tree
		var image_paths = function() {
			var skin_names = {
				skinDark  : "dark",
				skinLight : "light",
				skinNew   : "new"
			};
			var current_skin = inConcert.app.getMainWindow().getCurrentSkin() || "";
			var skin_name = skin_names[current_skin] || skin_names.skinLight;
			return "/inconcert/apps/agent/images/" + skin_name + "/tree/2023/";
		}();
		this.treeResults = new dhtmlXTreeObject("disposition_codes_tree_results", "100%", "100%", 0);
		this.treeResults.setImagePath(image_paths);
		this.treeResults.setStdImages("csh_books/leaf.svg","csh_books/folderOpen.svg","csh_books/folderClosed.svg");
		this.treeResults.setImageArrays("plus", "plus2.svg","plus3.svg","plus4.svg","plus.svg","plus5.svg");
		this.treeResults.setImageArrays("minus", "minus2.svg","minus3.svg","minus4.svg","minus.svg","minus5.svg");
		this.treeResults.setLinesImages("line2.svg", "line3.svg","line4.svg","blank.svg","blank.svg","line1.svg");
		
		this.treeResults.enableHighlighting(true);

		// tree events
		this.treeResults.setOnDblClickHandler(function(result_id) {
			var result = self.getResultData(result_id);
			if (result.childs) {
				return false;
			}
			$("#disposition_codes_popup_finish_interaction").prop("disabled", false).trigger("click");
			return true;
		});
		this.treeResults.setOnClickHandler(function(resultId){
			self.onSelectResultInTree(self, resultId);
		});
		this.treeResults.setOnOpenStartHandler(function(result_id)
		{
			// al expandir la rama, busco la lista de json dentro de ramas pendientes, luego mando a cargar en el tree la rama
			var pending_branch = self.pendings_branchs && self.pendings_branchs[result_id] ? self.pendings_branchs[result_id] : null;
			if ( pending_branch ) {
				self.fill_branch(pending_branch.childs, null, pending_branch.parentId, function() {
					var childs = self.treeResults.getSubItems(result_id).split(self.treeResults.dlmtr);
					$.each(childs || [], function(_, child) {
						self.treeResults.closeItem(child);
					});
					self.pendings_branchs[result_id] = null;    // elimino la rama que ya se ha procesado y cargado en el tree
					self.treeResults.selectItem(result_id);     // selecciono item en tree
					self.onSelectResultInTree(self, result_id); // disparo evento selected
				});
			}
			return true;
		});

		// (Ref #6053) De momento queda comentado, si en algun momento se necesita agregar el tema de reschedule, hay que descomentarlo.
		// this.fillRescheduleToCombo();

		// initialize datepicker jquery plugin
		//   Los eventos "beforeShow" y "onClose" son utilizados para que no se cierre
		//		el popup cuando en el calendario se selecciona una fecha.
		//		Si no se hace esto, se cierra porque el markup del datepicker es generado a nivel
		//		del body y no dentro del contenedor del popup.
		$("#disposition_code_popup_rescheduled_date").datepicker({
			dateFormat:			"yy-mm-dd",
			minDate:			new Date(),
			monthNames:			[
									"<%=_T('January')%>",   "<%=_T('February')%>", "<%=_T('March')%>",    "<%=_T('April')%>",
									"<%=_T('May')%>",       "<%=_T('June')%>",     "<%=_T('July')%>",     "<%=_T('August')%>",
									"<%=_T('September')%>", "<%=_T('October')%>",  "<%=_T('November')%>", "<%=_T('December')%>"
								],
			monthNamesShort:	[
									"<%=_T('Jan')%>", "<%=_T('Feb')%>", "<%=_T('Mar')%>", "<%=_T('Apr')%>",
									"<%=_T('Maj')%>", "<%=_T('Jun')%>", "<%=_T('Jul')%>", "<%=_T('Aug')%>",
									"<%=_T('Sep')%>", "<%=_T('Okt')%>", "<%=_T('Nov')%>", "<%=_T('Dec')%>"
								],
			dayNames:			[
									"<%=_T('Sunday')%>",   "<%=_T('Monday')%>", "<%=_T('Tuesday')%>", "<%=_T('Wednesday')%>",
									"<%=_T('Thursday')%>", "<%=_T('Friday')%>", "<%=_T('Saturday')%>"
								],
			dayNamesShort:		["<%=_T('Sun')%>", "<%=_T('Mon')%>", "<%=_T('Tue')%>", "<%=_T('Wed')%>", "<%=_T('Thu')%>", "<%=_T('Fri')%>", "<%=_T('Sat')%>"],
			dayNamesMin:		["<%=_T('Su')%>", "<%=_T('Mo')%>", "<%=_T('Tu')%>", "<%=_T('We')%>", "<%=_T('Th')%>", "<%=_T('Fr')%>", "<%=_T('Sa')%>"],
			//events
			beforeShow:			function() { self.setClosedAutomatic(false); },
			onClose:			function() { self.setClosedAutomatic(true);  }
		});

		if (!this.hasPermissionToComment()) {
			// $("#disposition_code_popup_comment_title").hide();
			$("#disposition_code_popup_comment_panel").hide();
		}

		// initialize maskedinput jquery plugin
		$("#disposition_code_popup_rescheduled_time").mask("99:99");
		// set events
		this.setEvents();
		// Current finalized code apply if I have only one interaction
		var currentFinalizedCode = this.data.isBatch ? null : this.data.interactions[0].getLastDispositionFromSteps();
		// start fill tree with disposition codes
		this.fill_branch(this.data.dispositionCodes, currentFinalizedCode, ROOT_RESULT_ID, function() {
			self.setPosition();
		});
	};

	DispositionCodesPopup.prototype.interactionIsCall = function(interaction) {
		return interaction.type === "CALL";
	};

	DispositionCodesPopup.prototype.interactionIsVoicemail = function(interaction) {
		return interaction.type === "VOICEMAIL";
	};
 
	/*******************************************************************
	 * Metodo que cierra el popup
	 */
	DispositionCodesPopup.prototype.close = function()
	{
		var self = this;
		this.hide(
			function() {
				self.treeResults.deleteChildItems(ROOT_RESULT_ID);
				self.treeResults.destructor();
				self.tree = null;
				self.data.container.remove();
				self.finalize();
			}
		);
	};
}());
