/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, dhtmlXTreeObject*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const FORMAT = inConcert.Utils.format;
	const ROOT_RESULT_ID = "0";
	const SON_BRANCH = 0;

	/**
	Class GenericTreePopup.
	*/
	function GenericTreePopup ()
	{
		GenericTreePopup.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "disposition_codes";
		this.ownerId = "window_generic_tree_popup_container";
		this.canReposition = true;
		this.loading_tree = false;
	}

	inConcert.Utils.inherit(GenericTreePopup, inConcert.PopupWindow);

	//----------------------------
	if (typeof(inConcert.GenericTreePopup) !== "function") {
		inConcert.GenericTreePopup = GenericTreePopup;
	}

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.displayStatusMessage = function(text, type, closeDisabled, timeout_ms ) {
		var section = $("#disposition_codes_message");
		section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#disposition_codes_close_message").hide();
		}
		else {
			$("#disposition_codes_close_message").show();
		}
	};

	/*******************************************************************
	 * funcion que oculta mensaje
	 */
	GenericTreePopup.prototype.hideStatusMessage = function() {
		$("#disposition_codes_message").hide();
	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.show_waiting = function( /*message*/ ) {

	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.hide_waiting = function() {

	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.getResultData = function(resultId) {
		resultId = resultId || this.treeResults.getSelectedItemId();
		return JSON.parse(this.treeResults.getUserData(resultId, "result"));
	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.validate = function() {
		return true;
	};

	/*******************************************************************
	 *fv
	 */
	GenericTreePopup.prototype.getDispositionCodeTreePath = function(itemId)
	{
		itemId = itemId || this.treeResults.getSelectedItemId();
		var dispositionCodeTreePath = [];
		while (itemId) {
			dispositionCodeTreePath.unshift(this.getResultData(itemId).name);
			itemId = this.treeResults.getParentId(itemId);
		}
		return dispositionCodeTreePath;
	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.getResult = function()
	{
		const resultConfig = this.getResultData();
		let disposition = {
			name     : resultConfig.name,
			treePath : this.getDispositionCodeTreePath()
		};
		return disposition;
	};

	/*******************************************************************
	 * Finish return the current selection
	 */
	GenericTreePopup.prototype.finishPopup = function() {
		var $cmdFinish = $("#disposition_codes_popup_finish");
		if (!this.validate()) {
			$cmdFinish.prop("disabled", false);
			return;
		}

		var result = this.getResult();

		let disposition_info = {
			disposition: result
		};
		this.parent_on_finish(disposition_info);
		this.close();
		return;
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	GenericTreePopup.prototype.setEvents = function() {
		var self = this;

		$("#disposition_codes_popup_finish")
			.bind("click", function() {
				if (self.loading_tree) {
					return;
				}
				$(this).prop("disabled", true);
				self.finishPopup();
			});
		var closeDialog = function() {
			self.close();
		};
		// evento que cierra el dialogo y no realiza cambios
		$("#disposition_codes_popup_cancel").bind("click", closeDialog);
		// evento que cierra el dialogo y no realiza cambios
		$("#close_disposition_codes_popup").bind("click", closeDialog);

		//
		$("#disposition_codes_close_message")
			.bind("click", function() {
				self.hideStatusMessage();
			});

	};

	/*******************************************************************
	 * add child result to tree
	 */
	GenericTreePopup.prototype.addChildResultToTree = function(result, resultId, parentId) {
		// The SELECT parameter remains selected the added element.
		this.treeResults.insertNewChild(parentId, resultId, result.name, 0, 0, 0, 0, "SELECT");
		this.treeResults.setUserData(resultId, "result", JSON.stringify(result));
	};

	/*******************************************************************
	 * add sibling result to tree
	 */
	GenericTreePopup.prototype.addSiblingResultToTree = function(result, resultId, parentId) {
		// The SELECT parameter remains selected the added element.
		this.treeResults.insertNewNext(parentId, resultId, result.name, 0, 0, 0, 0, "SELECT");
		this.treeResults.setUserData(resultId, "result", JSON.stringify(result));
	};

	/*******************************************************************
	 * fill branch the tree with all results
	 */
	GenericTreePopup.prototype.fill_branch = function(dispositionCodes, root_id, on_end)
	{
		/* Sample of the expected results on the json structure
		 *
			[
				{
					name: "Ventas"
				},
				{
					name: "Compras"
				},
				{
					name: "Marketing",
					childs: [
						{
							name: "Banca"
						},
						{
							name: "Seguros",
							childs: [
								{name: "Auto"},
								{name: "Hogar"}
							]
						}
					]
				}
			]
		 */
		var self = this;

		on_end = on_end || $.noop;

		if (!dispositionCodes) {
			throw("Disposition codes configurations does not exit");
		}
		var currentDispositionCode;
		var selectedId;

		var selectResultOnTree = function()
		{
			if ( root_id === ROOT_RESULT_ID ) {
				self.treeResults.closeAllItems(root_id);
			}
			var resultIds = self.treeResults.getSubItems(root_id).split(self.treeResults.dlmtr);
			if (!resultIds) {
				return;
			}
			selectedId = resultIds[0]; // selecciono el primer resultado/categoria del tree
			self.treeResults.selectItem(selectedId);							// selecciono item en tree
			self.onSelectResultInTree(self, selectedId);	// disparo evento selected
		};

		this.treeResults.deleteChildItems(root_id);
		// recursive function that adds results on the tree
		var timeout_counter = 0;
		var total_added = 0;
		this.pendings_branchs = this.pendings_branchs || {};

		var fill = function(dispositions, parentId)
		{
			if ( !self.opened ) {
				//console.log("1 fill dispositions: Me voy -> self.opened: " + self.opened);
				// si el popup esta cerrado, me voy
				self.loading_tree = false;
				on_end();
				return;
			}
			//console.log("fill dispositions: " + parentId);
			// Lo agrego dentro de la funcion foreach_timeout para que el navegador no quede colgado.
			// Como los timeout se tiran por cada invocacion a funcion recursiva del tree, tengo
			//   un contador "timeout_counter", que se va incrementando en cada invocacion a la funcion
			//   y va decrementando cuando el loop timeout culmina para cada invocacion.
			// Entonces con esto, cuando el contador quede en cero nuevamente, ahi ya termino la carga total del tree.
			timeout_counter++;
			inConcert.Utils.foreach_timeout(
			{
				rows : dispositions,
				//max_append_per_loop: 500,
				pause_between_loop_ms: 2,
				append_fn : function( result, added )
				{
					if ( !self.opened ) {
						//console.log("2 [append_fn] fill dispositions: Me voy -> self.opened: " + self.opened);
						// si el popup esta cerrado, me voy
						self.loading_tree = false;
						on_end();
						return { cancel: true };
					}
					total_added++;
					var result_id = inConcert.Utils.generateGuid();
					var first_child = added;
					if (first_child === SON_BRANCH) {
						self.addChildResultToTree(result, result_id, parentId);
					}
					else {
						self.addSiblingResultToTree(result, result_id, parentId);
					}
					//console.log("1 append_fn fill dispositions: " + result_id);
					parentId = result_id;
					// si tiene childs es categoria y sino es resultado
					if (result.childs) {
						// Agrego un unico hijo para que me aparezca la cruz colapsada.
						// luego si me expanden la rama, tengo que eliminar este temoral hijo, y agregar toda la rama hija.
						// Tambien agrego 'pendings_branchs', toda la rama de dispositions, para que luego se pueda cargar al expandir la misma
						var temporal_result_id = inConcert.Utils.generateGuid();
						var temporal_result = result.childs[0];
						self.addChildResultToTree(temporal_result, temporal_result_id, parentId);
						self.pendings_branchs[parentId] = {
							temporal_result_id: temporal_result_id,
							parentId: parentId,
							childs: result.childs
						};
					}
					else {
						// si es una interaccion reabierta me quedo con el id del tree del elemento buscado.
						if (currentDispositionCode && !selectedId) {
							if (currentDispositionCode === result.name) {
								selectedId = result_id;
							}
						}
					}
				},
				waiting_fn : function(/*message*/)
				{
					var waiting_message = FORMAT("<%=_T('Please wait, adding {0} elements.')%>", total_added);
					self.show_waiting( waiting_message );
					/*
						(#6097) Esta invocacion a la funcion se realiza para que ajuste la poscion del popup en la medida que se cargan los dispositions, sino realiza un juego en pantalla feo (habria que revisarlo con Murias porque hace ese juego de pantalla) por ahora queda asi
					*/
					self.setPosition();
				},
				complete_fn : function() {
					//console.log("3 complete_fn - timeout_counter " + timeout_counter);
					if ( !self.opened ) {
						//console.log("3 [complete_fn] fill dispositions: Me voy -> self.opened: " + self.opened);
						// si el popup esta cerrado, me voy
						self.loading_tree = false;
						on_end();
						return;
					}
					timeout_counter--;
					//console.log("3.1 complete_fn - timeout_counter: " + timeout_counter);
					if ( timeout_counter <= 0 ) {
						//console.log("4 complete_fn - ENDDDDDDDDDDD ");
						// aca termino la carga total del tree
						self.loading_tree = false;
						selectResultOnTree();
						self.hide_waiting();
						on_end();
					}
				}
			});
		};

		var waiting_message = "<%=_T('Please wait, adding elements.')%>";
		this.show_waiting( waiting_message );
		this.loading_tree = true;
		fill(dispositionCodes, root_id); // start fill
	};

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	GenericTreePopup.prototype.createMarkup = function() {
		$("#window_disposition_codes_popup_container").remove();
		var node = $("".concat(
			"<div id='window_disposition_codes_popup_container' class='systemPopWindow windowDispositionCodesContainer windowContainer' style='display:none;'>",

				"<div class='windowTop'>",
					"<div class='windowTitle windowDisposition'>",
						"<div class='windowDispositionCodesTitleText'></div>",
						"<img id='close_disposition_codes_popup' src='/inconcert/apps/agent/images/blank.png' class='windowDispositionCodesButClose' hidden />",
					"</div>",
				"</div>",

				"<div id='disposition_codes_waiting' class='windowContainerMessages messageProcess' style='display:none;'>",
					"<div class='windowMessagesText'>",
						"<label message='true' class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div id='disposition_codes_message' class='windowContainerMessages messageError' style='display:none;'>",
					"<a id='disposition_codes_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div class='dispositionCodesThreadContainer'>",
						"<div id='disposition_codes_tree_results' class='dispositionCodesThreadContentScrollArea'></div>",
					"</div>",
				"</div>",
				
				"<div class='windowBottom internalPopUpButtons'>",
					"<button id='disposition_codes_popup_cancel' class='pageButtonGeneric buttonColorWhite' >", "<%=_T('Cancel')%>", "</button>",
					"<button id='disposition_codes_popup_finish' class='pageButtonGeneric buttonColorBlack' disabled>", "<%=_T('Confirm')%>", "</button>",
				"</div>",

			"</div>"));
		$("body").append(node);
		return node;
	};

	/*******************************************************************
	 *
	 */
	GenericTreePopup.prototype.onSelectResultInTree = function(self, undefined /* resultId */)
	{
		if ( this.loading_tree ) {
			return;
		}
		var result = self.getResultData();
		var $cmdFinish = $("#disposition_codes_popup_finish");
		if (result.childs) {
			$cmdFinish.prop("disabled", true);
		}
		else {
			$cmdFinish.prop("disabled", false);

			self.setPosition(); // recalcula la posicion si cambia de tamano y el popup quedara fuera de ventana.
		}
	};

	/*******************************************************************
	 * Metodo que visualiza el popup con los disposition codes e inicializa eventos de la ventana!
	 */
	GenericTreePopup.prototype.initialize = function()
	{
		var self = this;

		this.data.container = this.createMarkup();
		this.parent_on_finish = this.data.callbacks.on_finish || $.noop;
		this.data.threadContainer = this.data.container.find(".dispositionCodesThreadContentScrollArea");

		// popup title
		var popupTitle = function() {
			if(self.data.title){
				return self.data.title;
			}
			return "<%=_T('Select an element')%>";
		}();
		$("#window_disposition_codes_popup_container .windowDispositionCodesTitleText").text(popupTitle);
		// initialize dhtmlx tree
		// initialize tree
		var image_paths = function() {
			var skin_names = {
				skinDark  : "dark",
				skinLight : "light",
				skinNew   : "new"
			};
			var current_skin = inConcert.app.getMainWindow().getCurrentSkin() || "";
			var skin_name = skin_names[current_skin] || skin_names.skinLight;
			return "/inconcert/apps/agent/images/" + skin_name + "/tree/29/";
		}();
		this.treeResults = new dhtmlXTreeObject("disposition_codes_tree_results", "100%", "100%", 0);
		this.treeResults.setImagePath(image_paths);
		this.treeResults.setStdImages("csh_books/leaf.png","csh_books/folderOpen.png","csh_books/folderClosed.png");
		this.treeResults.setImageArrays("plus", "plus2.png","plus3.png","plus4.png","plus.png","plus5.png");
		this.treeResults.setImageArrays("minus", "minus2.png","minus3.png","minus4.png","minus.png","minus5.png");
		this.treeResults.setLinesImages("line2.png", "line3.png","line4.png","blank.png","blank.png","line1.png");
		this.treeResults.enableHighlighting(true);

		// tree events
		this.treeResults.setOnDblClickHandler(function(result_id) {
			var result = self.getResultData(result_id);
			if (result.childs) {
				return false;
			}
			$("#disposition_codes_popup_finish").prop("disabled", false).trigger("click");
			return true;
		});
		this.treeResults.setOnClickHandler(function(resultId){
			self.onSelectResultInTree(self, resultId);
		});
		this.treeResults.setOnOpenStartHandler(function(result_id)
		{
			// al expandir la rama, busco la lista de json dentro de ramas pendientes, luego mando a cargar en el tree la rama
			var pending_branch = self.pendings_branchs && self.pendings_branchs[result_id] ? self.pendings_branchs[result_id] : null;
			if ( pending_branch ) {
				self.fill_branch(pending_branch.childs, pending_branch.parentId, function() {
					var childs = self.treeResults.getSubItems(result_id).split(self.treeResults.dlmtr);
					$.each(childs || [], function(_, child) {
						self.treeResults.closeItem(child);
					});
					self.pendings_branchs[result_id] = null;    // elimino la rama que ya se ha procesado y cargado en el tree
					self.treeResults.selectItem(result_id);     // selecciono item en tree
					self.onSelectResultInTree(self, result_id); // disparo evento selected
				});
			}
			return true;
		});

		// set events
		this.setEvents();
		// start fill tree with disposition codes
		this.fill_branch(this.data.dispositionCodes, ROOT_RESULT_ID, function() {
			self.setPosition();
		});
	};
 
	/*******************************************************************
	 * Metodo que cierra el popup
	 */
	GenericTreePopup.prototype.close = function()
	{
		var self = this;
		this.hide(
			function() {
				self.treeResults.deleteChildItems(ROOT_RESULT_ID);
				self.treeResults.destructor();
				self.tree = null;
				self.data.container.remove();
				self.finalize();
			}
		);
	};
}());
