/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, -W083, esversion: 11 */

/* global inConcert, _ , EmojiPicker, CInconcertPlayer*/


/*
	Handling GUI events to interact with instagram client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var m_maxCharsOnTheWall = 1000;
	var m_maxCharsOnTheComment = 8000;
	var FORMAT = inConcert.Utils.format;

	var SCROLL_TO = inConcert.Utils.scroll_to;
	var TIME_SELECTED_COMMENT_MS = 5000;

	var LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	var LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";
	var LNK_ORIGIN_MARK_AS_READ_CLASS   = "originActionsMarkAsRead";
	var LNK_ORIGIN_MARK_AS_UNREAD_CLASS = "originActionsMarkAsUnread";
	
	var LNK_LIKE_CLASS   = "actionsLike";
	var LNK_UNLIKE_CLASS = "actionsUnlike";

	var CLASS_DELIVERY_SENDING    = "deliverySending";
	var CLASS_DELIVERY_SUCCESS    = "deliveryOk";
	var CLASS_DELIVERY_FAILED     = "deliveryFail";
	var CLASS_DELIVERY_DISCARDED  = "deliveryDiscarded";
	var CLASS_DELIVERY_DISCARDING = "deliveryDiscarding";

	/**
	Class InstagramClientWindow.
	Instagram Client Handler
	@param interaction
	@param options
	*/
	function InstagramClientWindow (interaction, options)
	{
		var self = this;

		//relate the interaction to the view
		this.m_maxCompressedComments = 20;
		this.m_interaction = interaction;
		this.hash_events = {};
		this.update_hash_events_return_diff();
		this.setView("client");
		this.setHorizontalResizingLimits(480);

		var subtype = interaction && interaction.interactionData && interaction.interactionData.subtype;

		if (subtype === "PageInboxNewMessage") {
			//this flag must be set up before the contructor call because is needed in the markup build
			this.m_privateMessages = true;
		}
		else if (subtype === "NewPostComment" || subtype === "MentionComment") {
			this.m_postComment = true;
		}
		else if (subtype === "MentionPost") {
			this.m_mention = true;
			$("#instagramCommentButton").val("<%=_T('COMMENT')%>");
		}
		else {
			throw "invalid subtype";
		}

		var success = InstagramClientWindow.uber.constructor.call( this, options );
		if (!success) {
			return;
		}

		if (this.m_privateMessages) {
			this.setView("mail_client");
			this.replyByMail();
		}

		$("#instagramClientCharacterCounter").text(m_maxCharsOnTheWall);
		this.has_created_main_thread = false;
		this.can_show_all_comments = false;
		this.are_showing_all_messages = false;
		this.loading_all_comments = false;

		if (options.readonly || options.monitor) {
			//cannot comment anything on this state
			$('.instagramThreadCommentControlContainer').hide();
			//cannot reply in wall
			$("#replyInWall").hide();
		}
		
		$("#postCommentText").off('change').on('change', function() {
			$("#instagramCommentButton").prop("disabled", !$(this).val());
		}).change();

		$("#instagramCommentButton").click(function() {
			self.comment_current_post();
		});

		$("#replyInWall").click(function() {
			self.reply_in_wall();
		});

		//load avatar for the contact
		var avatar_uri = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
		$('#contactFromAvatar').attr("data", avatar_uri);
		$("#instagramWallText").keyup(function() {
			self.onCharCountChanged($(this));
		});

		$("#postCommentText").keypress(function() {
			var comment = $(this).val();
			if (comment.length >= m_maxCharsOnTheComment) {
				return false;
			}
		}).bind("paste", null, function() {
			var self = $(this);
			var previousValue = self.val();
			window.setTimeout(function() {
				var actualValue = self.val();
				if (actualValue.length >= m_maxCharsOnTheComment) {
					self.val(previousValue);
				}
			}, 1);
		});

		$("#instagramGroup").text(this.m_interaction.campaign);
		//instagram cboAccount shows the page name in case this is a message for the page
		var page = this.m_interaction.interactionData.page;
		if (page) {
			var accountId = this.m_interaction.getAccount();
			var account;
			var campaignAccounts = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "INSTAGRAM");
			for (var i = 0; i < campaignAccounts.length; i++) {
				if (campaignAccounts[i].id === accountId) {
					account = campaignAccounts[i];
					break;
				}
			}
			if (account) {
				var campaign = inConcert.app.getCampaign(this.m_interaction.campaign);
				if (campaign) {
					var pagesNames = campaign.getAccountPages(accountId);
					if (pagesNames) {
						$("#cboAccounts option[value='" + this.m_interaction.account + "']").text(pagesNames[page]+" ("+account.name+")");
					}
				}
			}
		}

		this.was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		var pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( this.was_being_composed || pending_origin_post ) {
			this.reply_in_wall( pending_origin_post );
			$("#mainContainer").addClass("instagramStartOutboundInteraction");
			if (!this.m_interaction.contact) {
				this.on_account_changed();
			}
		}
		else {
			var events = this.m_interaction.getEventList();
			if (events && events.length) {
				$("#instagramDate").text(this.m_interaction.getLocaleDateString(0));
				$("#instagramFrom").text(this.m_interaction.getSource(0));
			}
			var campaignAddresses = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "INSTAGRAM");
			for (var j = 0; j < campaignAddresses.length; j++) {
				if (campaignAddresses[j].id === this.m_interaction.getAccount()) {
					this.campaignAddress = campaignAddresses[j];
				}
			}
		}
		this.m_timer = window.setInterval(function() {
			self.onTimer();
		}, 1000);
	}

	inConcert.Utils.inherit( InstagramClientWindow, inConcert.ClientWindow );

	InstagramClientWindow.prototype.initialize = function()
	{
		InstagramClientWindow.uber.initialize.call( this );
	};

	InstagramClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	InstagramClientWindow.prototype.clearInput = function($input) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	InstagramClientWindow.prototype.focusInput = function($input) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	InstagramClientWindow.prototype.enableInput = function(enabled) {
		var $input = this.$input();
		$input.prop("disabled", !enabled);
		this.$emojiText($input).prop("contenteditable", enabled);
		$(".emoji-picker-button").toggleClass("emoji-picker-button-disabled", !enabled);
	};
	
	InstagramClientWindow.prototype.$input = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#instagramWallText");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#instagramMailText");
			}
			else {
				return $("#postCommentText");
			}
		}
	};
	
	InstagramClientWindow.prototype.$attachmentPicker = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#sendFilePost");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#sendFilePrivate");
			}
			else {
				return $("#sendFileComment");
			}
		}
	};
	
	InstagramClientWindow.prototype.$recordStart = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#recordAudioStartButtonPost");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#recordAudioStartButtonPrivate");
			}
			else {
				return $("#recordAudioStartButtonComment");
			}
		}
	};
	
	InstagramClientWindow.prototype.$inputContainer = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#instagramWallMessage");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#instagramPrivateMessage");
			}
			else {
				return $("#instagramComment");
			}
		}
	};

	InstagramClientWindow.prototype.$emojiText = function($input) {
		return $input.siblings(".emoji-wysiwyg-editor");
	};

	InstagramClientWindow.prototype.$emojiInput = function() {
		return this.$emojiText(this.$input());
	};
	
	InstagramClientWindow.prototype.appendToInput = function($input, text) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
	};
	
	InstagramClientWindow.prototype.setInputText = function($input, text) {
		this.clearInput($input);
		this.appendToInput($input, text);
		this.focusInput($input);
		$input.change();
	};

	InstagramClientWindow.prototype.getWindowDefaultTitle = function()
	{
		return "<%=_T('Instagram Window')%>";
	};

	InstagramClientWindow.prototype.eventsSuscribe = function()
	{
		InstagramClientWindow.uber.eventsSuscribe.call( this );
		var self = this;

		if ( this.m_interaction && this.m_interaction.id ) {
			// failure ( POST, COMMENT, PRIVATE MESSAGE )
			var pub_instagram_failure_id =  "/instagram/" + this.m_interaction.id + "/failure";
			this.pubsub_instagram_failure_messages_handle = inConcert.app.messagebus.subscribe(
				pub_instagram_failure_id,
				function( event, type ) {
					self.show_proxy_failure( event, type );
				}
			);
		}
	};

	InstagramClientWindow.prototype.eventsUnsuscribe = function()
	{
		InstagramClientWindow.uber.eventsUnsuscribe.call( this );

		if ( this.pubsub_instagram_failure_messages_handle ) {
			inConcert.app.messagebus.unsubscribe( this.pubsub_instagram_failure_messages_handle );
		}
	};

	InstagramClientWindow.prototype.show_proxy_failure = function( event /*, type*/ )
	{
		var short_msg = this.m_interaction ? this.m_interaction.get_error_message( event.error_code ) : "<%=_T('Instagram error.')%>";
		var long_msg = short_msg + ". " + "<%=_T('Please try again later.')%>";
		this.hideStatusMessage();
		this.displayStatusMessage( long_msg, "Error", true, 5000 );
		this.delivery_failed_message( event, short_msg );
	};
	
	/*
		Like/unlike an event
	*/
	InstagramClientWindow.prototype.like = function( like, event_index, callback )
	{
		var self = this;
		callback = callback || $.noop;

		var message;
		if ( like ) {
			message = "<%=_T('Please wait, liking.')%>";
		}
		else {
			message = "<%=_T('Please wait, unliking.')%>";
		}
		this.displayStatusMessage( message, "Process", false );

		this.m_interaction.like(
			like,
			event_index,
			function( success, message ) {
				callback( success );
				self.hideStatusMessage();
				if (!success) {
					self.displayStatusMessage( message, "Error" );
				}
			}
		);
	};
	
	InstagramClientWindow.prototype.buildPrivateReply = function($reply_link) {
		let self = this;
		if ( !self.canExecuteInteractionAction() ) {
			return "can not private reply";
		}
		let event_index = $reply_link.data("event_index");
		let name = $reply_link.data("user_name");
		let $comment = $reply_link.parent();

		if ( $comment.hasClass( "unread" ) ) {
			self.mark_as_read(
				true,
				event_index,
				function( success ) {
					self.on_readed_event( success, $comment );
				}
			);
		}

		self.setPrivateReply( event_index, name );
	};

	InstagramClientWindow.prototype.setEventsHandlers = function()
	{
		const self = this;

		InstagramClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if ( !this.m_interaction.isBeingComposed() ) {

			$("#mark_all_read_unread").click( function()
			{
				self.markAllReadUnread();
			});
		}

		var $container = $("#thread_container");

		// reply comment link
		var event_selector = "[reply_comment_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $reply_link = $( this );
				var message_id = $reply_link.data("message_id");
				var name = $reply_link.data("user_name");
				var $comment = $reply_link.parent();

				if ( $comment.hasClass( "unread" ) ) {
					var event_index = $reply_link.data("event_index");
					self.mark_as_read(
						true,
						event_index,
						function( success ) {
							self.on_readed_event( success, $comment );
						}
					);
				}

				self.setReplyToComment( message_id, name );
			});

		event_selector = "[private_reply_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				self.buildPrivateReply($(this));
			});
						
		// mark as read/unread comment link
		event_selector = "[mark_as_read_unread]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				var $mark_read_unread = $( this );

				if ( $mark_read_unread.prop( "disabled" ) ) {
					return;
				}

				var $comment = $mark_read_unread.parent();
				if ( $comment.hasClass( "selected" ) ) {
					return;
				}
				if ($comment.hasClass('instagramMainInfoContainer')) {
					$comment =  $comment.parent().parent();
				}
				var event_index = $mark_read_unread.data( "event_index" );
				var new_state = $mark_read_unread.data( "event_unread" ) === "unread";
				self.mark_as_read(
					new_state,
					event_index,
					function( success ) {
						self.on_readed_event( success, $comment );
					}
				);
			});

		// mark as read/unread comment link
		event_selector = "[like_unlike]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $like_unlike = $( this );

				if ( $like_unlike.prop( "disabled" ) ) {
					return;
				}

				var event_index = $like_unlike.data( "event_index" );
				var new_state = $like_unlike.data( "event_like" ) === "like";
				self.like(
					new_state,
					event_index,
					function( success ) {
						self.on_liked_event( success, new_state, $like_unlike.parent() );
					}
				);
			});

		// resend failed comment link
		var on_resend = function( success, message )
		{
			if ( !success ) {
				// Estos son errores genericos. Los errores de instagram se procesan en "InstagramClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> InstagramClientWindow.clientUpdate"
		};

		event_selector = "[resend_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $resend = $( this );
				var $comment = $resend.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T('Resending...')%>", "Process", true);
				self.delivery_sending_message( $comment, "<%=_T('Resending')%>" );

				if ( self.m_interaction.isPrivateMessage() ) {
					self.m_interaction.newPrivateMessage(
						fake_event_id,
						failed_event.page,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
				else {
					self.m_interaction.commentPost(
						fake_event_id,
						failed_event.postId,
						failed_event.commentId,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
			});

		// discard failed comment link
		event_selector = "[discard_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $discard = $( this );
				var $comment = $discard.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				if ( !fake_event_id ) {
					return;
				}
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T('Discarding...')%>", "Process", true);
				self.delivery_discarding_message( $comment );

				self.m_interaction.discardFailedEvent( fake_event_id,
					function( success )
					{
						self.hideStatusMessage();
						if ( success ) {
							self.delivery_discarded_message( $comment );
							return;
						}
						self.displayStatusMessage( FORMAT( "<%=_T('Error discarding message {0}')%>", fake_event_id), "Error", true, 5000 );
						self.delivery_failed_message( failed_event );
					}
				);
			});

		// show / hide message error !!!
		var timeout_handle;
		var show_error = function( $error, can_show ) {
			var $err_msg_container = $error.parent().find( ".deliveryFailReasonContainer" );
			if ( can_show ) {
				window.clearTimeout( timeout_handle );
				var show_error_timeout_ms = 200;
				timeout_handle = window.setTimeout( function() { $err_msg_container.show(); }, show_error_timeout_ms );
			}
			else {
				window.clearTimeout( timeout_handle );
				$err_msg_container.hide();
			}
		};

		event_selector = ".deliveryMessageContainer." + CLASS_DELIVERY_FAILED;
		$container
			.off("mouseenter", event_selector)
			.off("mouseleave", event_selector)
			.on("mouseenter", event_selector, function () { show_error( $( this ), true  ); })
			.on("mouseleave", event_selector, function () { show_error( $( this ), false ); });

			/* Gestionar contacto si:
		 1- Esta habilitado la administracion de contactos
		 2- Se puede visualizar los addresses
		*/
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);

		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) && visualize_address) {
			//Setup contact manager popup when user click in the contact
			event_selector = ".subEventAuthor";
			$container
				.off("click", event_selector)
				.on("click", event_selector, function ()
				{
					var event_index = $( this ).data( "event_index" );
					if ( !event_index ) {
						return;
					}
					var event = self.m_interaction.getEvent( event_index );
					if ( !event || event.creator !== "contact" ) {
						return;
					}

					var address_normalized = inConcert.app.getContactManager().normalizeContactAddress(self.m_interaction.getRawAddress( event_index ), "Instagram");

					var contact_window = new inConcert.ContactWindow({
						from_app    : self.from_app,
						interaction : self.m_interaction,
						name        : self.m_interaction.getUserName( event ),
						avatar      : self.m_interaction.getEventUserAvatarURI( event ),
						addresses   : [ address_normalized ]
					});
					contact_window.show();
				});
		}
	};

	InstagramClientWindow.prototype.getScrollBasicInfo = function()
	{
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	InstagramClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return FORMAT( "<%=_T('{0} Unread')%>", this.m_interaction.getUnreadEventCount() );
	};

	InstagramClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	InstagramClientWindow.prototype.invalidInteraction = function() {
		InstagramClientWindow.uber.invalidInteraction.call( this );
		$(".interactionActionsButtons").children().hide();
		$(".instagramCommentControl").hide();
		$(".instagramThreadCommentControlContainer").hide();
	};

	InstagramClientWindow.prototype.onInteractionGone = function(interaction) {
		InstagramClientWindow.uber.onInteractionGone.call( this, interaction );
		$("#postCommentText").prop("disabled", true);
		$("#instagramComment").prop("disabled", true);
		$("#instagramWallText").prop("disabled", true);
		$("#instagramWallButtonPost").prop("disabled", true);
	};

	InstagramClientWindow.prototype.__setExternalMessage = function(message)
	{
		const self = this;

		if (!this.m_interaction) {
			return;
		}
		message = message || "";

		let $writing_area;
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		let afterSetFn = $.noop;

		if ( was_being_composed || pending_origin_post ) {
			$writing_area = $("#instagramWallText");
			afterSetFn = function() {
				self.onCharCountChanged($writing_area);
			};
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				$writing_area = $("#instagramMailText");
			}
			else {
				$writing_area = $("#postCommentText");
			}
		}
		this.appendToInput($writing_area, message);
		this.focusInput($writing_area);
		afterSetFn();
	};

	InstagramClientWindow.prototype.setCannedMessage = function(canned)
	{
		this.__setExternalMessage(canned.message);
	};

	InstagramClientWindow.prototype.setQnAMakerAnswer = function(answer)
	{
		this.__setExternalMessage(answer.answer);
	};

	InstagramClientWindow.prototype.on_account_changed = function()
	{
		InstagramClientWindow.uber.on_account_changed.call( this );

		$("#instagramAvailablePages").unbind("change");

		if ( !this.m_interaction.contact && this.m_interaction.initiative === "OUTBOUND" ) {
			$("#instagramAvailablePages").parent().hide();
			$("#instagramAvailablePages").children().remove();
			var accountId = this.m_interaction.getAccount();
			var account;
			var campaignAccounts = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "INSTAGRAM");
			for (var i = 0; i < campaignAccounts.length; i++) {
				if (campaignAccounts[i].id === accountId) {
					account = campaignAccounts[i];
					break;
				}
			}

			if (account) {
				var campaign = inConcert.app.getCampaign(this.m_interaction.campaign);
				if (campaign) {
					var pagesNames = campaign.getAccountPages(accountId);
					if (pagesNames) {
						for (var page in pagesNames) {
							if (pagesNames.hasOwnProperty(page)) {
								$("#instagramAvailablePages").
									append($("<option></option>").
										attr("value",page).
										text(pagesNames[page]));
							}
						}

						$("#instagramAvailablePages").val(" ");
						$("#instagramAvailablePages").parent().show();
						$("#instagramAvailablePages").bind("change", function() {
							var page = $(this).val();
							var pageName = $("#instagramAvailablePages option[value='" + page + "']").text();
							var helperMessage = FORMAT("<%=_T('Post Something On {0}s Page')%>", pageName);
							$("#instagramWallMessage > .instagramInteractionControlTitle").text(helperMessage);
						}).change();
					}
				}
			}
		}
	};

	InstagramClientWindow.prototype.onCharCountChanged = function(textArea) {
		var currentChars = document.getElementById(textArea.attr("id")).value.length;
		var charsDifference = m_maxCharsOnTheWall - currentChars;
		var disabledWallButton = charsDifference < 0;
		$("#instagramClientCharacterCounter").text(charsDifference)[disabledWallButton ? "addClass" : "removeClass"]("instagramClientCharacterCounterNegative");
		$("#instagramWallButtonPost").prop("disabled", disabledWallButton);
	};

	InstagramClientWindow.prototype.setView = function( view_id, pending_event )
	{
		var class_name;
		var $comment = $("#instagramComment");
		var $wall = $("#instagramWallMessage");
		var $private_message = $("#instagramPrivateMessage");

		switch( view_id )
		{
			case "client" :
				class_name = "instagramClientContainer";
				//hide
				$private_message.hide();
				$wall.hide();
				// show
				$comment.show();
				break;

			case "wall_client" :
				class_name = "instagramWallClientContainer";
				var $wall_err_container = $("#instagramWallError");
				$wall_err_container.hide();

				if ( pending_event ) {
					var state_class = "";
					var state = pending_event.state || "";

					switch( state ) 
					{
						case "DELIVERED":
							// NO HAGO NADA!!! ( CLASS_DELIVERY_DELIVERED )
						break;
						case "DISCARDED":
							state_class = CLASS_DELIVERY_DISCARDED;
						break;
						case "DISCARDING":
							state_class = CLASS_DELIVERY_DISCARDING;
						break;
						case "FAILED":
							var short_msg = this.m_interaction.get_error_message( pending_event.error_code );
							$wall_err_container.show();
							$wall_err_container.find("#instagramWallErrorMessage").text(short_msg);
							state_class = CLASS_DELIVERY_FAILED;
						break;
						case "SENDING":
							state_class = CLASS_DELIVERY_SENDING;
						break;
						default:
							throw "Unrecognized pending event '" + state + "' state";
					}

					class_name += " " + state_class;
				}
				//hide
				$comment.hide();
				$private_message.hide();
				// show
				$wall.show();
				break;

			case "mail_client" :
				class_name = "instagramMailClientContainer";
				//hide
				$comment.hide();
				$wall.hide();
				// show
				$private_message.show();
				break;

			default:
				throw "InstagramClientWindow.prototype.setView. Unrecognized view_id: '" + view_id + "'";
		}

		var initiative = this.m_interaction.initiative;

		$("#mainContainer")
			.removeClass()
			.addClass( class_name )
			.addClass( initiative === "OUTBOUND" ? "instagramOutboundClientContainer" : "instagramInboundClientContainer" );
	};

	InstagramClientWindow.prototype.reply_in_wall = function( pending_origin_post )
	{
		var self = this;

		this.setView("wall_client", pending_origin_post);
		$("#replyInWall").prop("disabled", true);

		var text = "";
		var $wall_text = $("#instagramWallText");
		if ( pending_origin_post ) {
			text = pending_origin_post.message;
			$wall_text.trigger("click");
		}

		document.getElementById("instagramWallText").value = text;

		$wall_text.focus();
		$("#instagramClientCharacterCounter").text(m_maxCharsOnTheWall);

		var on_send = function( success, message )
		{
			if ( !success ) {
			// Estos son errores genericos. Los errores de instagram se procesan en "InstagramClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> InstagramClientWindow.clientUpdate"
		};

		$("#instagramWallButtonPost")
			.unbind('click')
			.bind('click', function()
			{
				if (!self.canExecuteInteractionAction()) {
					return;
				}
				self.displayStatusMessage("<%=_T('Posting...')%>", "Process", true);
				var message = document.getElementById("instagramWallText").value;
				var destination;
				var destinationType = "CONTACT";
				// Instagram doens't let us write wall post on friend's walls, just log error if
				// someone try to do it.
				if (self.m_interaction.initiative === "OUTBOUND" && self.m_interaction.contact) {
					inConcert.logger.error("Instagram doesn't allow to write post on friend's walls");
				}
				var destinationName;
				var page = $.trim($("#instagramAvailablePages").val());
				//first part of the "and" = user didn't select anything: default to profile
				//second part user selected the profile
				if (page !== "" && page !== self.m_interaction.account) {
					destination = page;
					destinationName = $("#instagramAvailablePages option[value='" + page + "']").text();
					destinationType = "PAGE";
				}
				else {
					destination = $("#cboAccounts").val();
					destinationType = "CAMPAIGN";
					var campaign = inConcert.app.getCampaign(self.m_interaction.campaign);
					if (campaign) {
						destinationName = campaign.Name;
					}
				}

				var fake_event_id; // tiene que ser pasado como parametro en undefined
				pending_origin_post = self.m_interaction.get_pending_origin_post();
				if ( !pending_origin_post ) {
					self.m_interaction.newPost( fake_event_id, destination, message, destinationType, destinationName, on_send );
				}
				else {
					// resend
					fake_event_id = pending_origin_post.fake_event_id;
					self.m_interaction._newPost( fake_event_id, destination, message, on_send );
				}
			}
		);
	};

	/*
	Esta funcion actualiza la informacion del usuario (id, name, avatar), de los eventos pendings
	En las acciones del lado del cliente (delivery_discarding_message y delivery_sending_message)
	Esta funcion no es incovada en las respuestas del delivery

	Es necesaria cuando el evento fallido lo genero un agente diferente al que se encuentra logueado.
	*/
	InstagramClientWindow.prototype.delivery_update_user_info = function( $pending )
	{
		var $author = $pending.find( ".subEventAuthor" );
		var author_id = $author.data( "event_user_id" );

		if ( typeof author_id !== "string" ) {
			return;
		}
		var logged_user_id = inConcert.app.getCompleteId();
		if ( author_id === logged_user_id ) {
			return;
		}

		$pending.find( ".subEventAvatar object" ).attr( "data", inConcert.app.getLoggedUserAvatar() );
		$author.data( "event_user_id", logged_user_id );
		$author.text( inConcert.app.getLoggedUserName() );
	};

	InstagramClientWindow.prototype.delivery_success_message = function( $success, success_event )
	{
		var $thread_container = $("#thread_container");

		var $siblings, $last_message, last_message_id;
		var parent_id = success_event.message.parent_id;
		var is_reply = _.isString( parent_id );

		// obtengo el ultimo mensaje del hilo al cual corresponde el evento SUCCESS
		if ( is_reply ) {
			// si es reply, lo obtengo de su hilo
			$siblings = $thread_container.find("[data-parent_id=" + parent_id +"]");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}
		else {
			// si no es un reply, lo obtengo del hilo principal
			$siblings = $thread_container.find(".instagramThreadSubEvent");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}

		var fake_event_id = $success.attr( "data-id" );
		var previous_state = $success.attr( "previous_state" );

		var move_last = ( previous_state === "failed" && fake_event_id !== last_message_id );
		if ( move_last ) {
		// si habia fallado y el ultimo mensaje del hilo al cual pertenece no es el mismo evento,
		// le realizao detach ya que se tiene que agrega al final de su thread ( como ultimo mensaje )
		// (*)
			$success.removeAttr( "previous_state" );
			$success.detach();
		}

		$success
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.attr( "data-id", success_event.message.id )
			.attr( "data-time", success_event.creationDate.getTime() );

		var $date = $success.find( ".subEventDate" );
		$date.text( inConcert.Utils.userFormatDate(success_event.creationDate, false, true));

		var $delivery_message_container = $success.find( ".deliveryMessageContainer" );
		var $lbl_message = $success.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Delivered')%>" );

		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.show();

		var $err_msg_container = $success.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $success.find( ".deliveryFailActions" );
		$failed_actions.hide();

		if ( move_last ) {
		// como hemos explicado arriba en (*), debemos mover el mensaje a la ultima posicion del hilo al cual pertenece!!!
			if ( is_reply ) {
			// si es reply tiene que validar si tiene hermanos o no, para saber donde ubicarlo!!
				if ( $siblings.length === 0 ) {
					$thread_container.find("[data-id='" + parent_id + "']").after( $success );
				}
				else {
					$siblings.last().after( $success );
				}
			}
			else {
			// si no es reply derecho al final
				$thread_container.append( $success );
			}

			var scroll_data = this.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $success;
			SCROLL_TO( scroll_data );
		}

		window.setTimeout(function()
		{
			$success.removeClass( CLASS_DELIVERY_SUCCESS );
			$delivery_message_container.removeClass( CLASS_DELIVERY_SUCCESS ).hide();
		}, TIME_SELECTED_COMMENT_MS);
	};

	InstagramClientWindow.prototype.delivery_sending_message = function( $pending, message )
	{
		var has_failed = $pending.hasClass( CLASS_DELIVERY_FAILED );
		if ( has_failed ) {
			$pending.attr( "previous_state", "failed");
		}

		$pending
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( message || "<%=_T('Sending')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $pending.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $pending );
	};

	InstagramClientWindow.prototype.delivery_failed_message = function( failed_event, err_msg )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}

		$pending
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Failed')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		var $err_msg = $err_msg_container.find( ".deliveryFailReasonTxt" );
		if ( $err_msg.text().length > 0 || err_msg ) {
			if ( err_msg ) {
				$err_msg.text( err_msg );
			}
		}

		if ( this.isEditable() ) {
			var $failed_actions = $pending.find( ".deliveryFailActions" );
			$failed_actions.show();
		}
	};

	InstagramClientWindow.prototype.delivery_discarding_message = function( $discarding )
	{
		$discarding
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING );

		var $delivery_message_container = $discarding.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Discarding')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING )
			.show();

		var $err_msg_container = $discarding.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarding.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $discarding );
	};

	InstagramClientWindow.prototype.delivery_discarded_message = function( $discarded )
	{
		$discarded
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED );

		var $delivery_message_container = $discarded.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Discarded')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED )
			.show();

		var $err_msg_container = $discarded.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarded.find( ".deliveryFailActions" );
		$failed_actions.hide();
	};

	InstagramClientWindow.prototype.delivery_remove = function( failed_event )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}
		$pending.fadeOut( 1000, function(){ 
			$pending.remove();
		});
	};

	InstagramClientWindow.prototype.on_private_message_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			this.clearInput($("#instagramMailText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( _, can_scroll )
			{
				if ( !can_scroll ) {
					return;
				}
				var scroll_data = self.getScrollBasicInfo();
				scroll_data.destination = "bottom";
				SCROLL_TO( scroll_data );
			};
			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};

	InstagramClientWindow.prototype.replyByMail = function() 
	{
		var self = this;
		this.setView("mail_client");
		$("#replyInWall").prop("disabled", false);
		var helperMessage = "<%=_T('Send a private message')%>";
		$("#instagramPrivateMessage > .instagramInteractionControlTitle").text(helperMessage);
		var $txt = $("#instagramMailText");
		this.clearInput($txt);
		
		$txt.off('change').on('change', function() {
			$("#instagramMailButtonSend").prop("disabled", !$(this).val());
		}).change();

		$("#instagramMailButtonSend").unbind('click').bind('click', function() {
			if (!self.canExecuteInteractionAction()) {
				return;
			}
			self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
			
			//mark all as read
			if ($("#mark_all_read_unread").val() === self.MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			
			var text = $txt.val();
			var message = { text : text };
			var event = self.get_current_event();
			if ( !event ) {
				return;
			}
			var fake_event_id = self.m_interaction.newPrivateMessage(
				undefined, // fake_event_id
				self.m_interaction.interactionData.page,
				message,
				function( success, message, error ) {
					if ( success ) {
					// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
						return;
					}
					if ( error.agent_err ) {
					// estos son errores que se producen en controller interactions del lado del agente (ejemplo sin licencia para el canal).
					// Los errores de delivery, llegan desde el IR en la funcion "eventsSuscribe"
						event.fake_event_id = fake_event_id;
						self.hideStatusMessage();
						self.displayStatusMessage( message, "Error", true, 5000 );
						self.delivery_remove( event );
					}
				}
			);
			self.add_new_outgoing_fake_event( fake_event_id, message );
		});
	};

	InstagramClientWindow.prototype.replyByMailAction = function(callback = $.noop) {
		const self = this;
		if (!self.canExecuteInteractionAction()) {
			return callback(false, "can not send private message");
		}
		self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
		
		//mark all as read
		if ($("#mark_all_read_unread").val() === self.MARK_ALL_READ_BTN_TEXT) {
			$("#mark_all_read_unread").click();
		}
		
		var text = $("#instagramMailText").val();
		var message = { text : text };
		var event = self.get_current_event();
		if ( !event ) {
			return callback(false, "event not found");
		}
		var fake_event_id = self.m_interaction.newPrivateMessage(
			undefined, // fake_event_id
			self.m_interaction.interactionData.page,
			message,
			function( success, message, error ) {
				if ( success ) {
				// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
					return callback(true);
				}
				if ( error.agent_err ) {
				// estos son errores que se producen en controller interactions del lado del agente (ejemplo sin licencia para el canal).
				// Los errores de delivery, llegan desde el IR en la funcion "eventsSuscribe"
					event.fake_event_id = fake_event_id;
					self.hideStatusMessage();
					self.displayStatusMessage( message, "Error", true, 5000 );
					self.delivery_remove( event );
				}
				callback(false, message || "error sending message");
			}
		);
		self.add_new_outgoing_fake_event( fake_event_id, message );
	};
	
	/*
		fake_event_id
		text
		parent_comment - opcional 
		creation_date  - opcional
		state          - opcional
		agent          - opcional
	*/
	InstagramClientWindow.prototype.make_fake_event = function( fake_data )
	{
		var self = this;

		var get_parent = function()
		{
			var origin_event = self.get_current_event();
			if ( !origin_event ) {
				return "";
			}
			return origin_event.message.id;
		};
		var creation_date = fake_data.creation_date || new Date(); // si no viene la fecha, toma now
		var contact_form = $("#contactID").text();

		return {
			outgoing_pending : true,
			state            : fake_data.state || "SENDING",
			agent            : fake_data.agent || inConcert.app.getCompleteId(),
			agentName        : contact_form,
			creationDate     : creation_date,
			parent_id        : get_parent(),
			message : {
				id           : fake_data.fake_event_id,
				created_time : creation_date.toString(),
				parent_id    : fake_data.parent_comment,
				text      	 : fake_data.message.text || "",
				from : {
					id   : this.m_interaction.getAccount(),
					name : contact_form
				}
			}
		};
	};

	InstagramClientWindow.prototype.add_new_outgoing_fake_event = function( fake_event_id, message, parent_comment /* opcional */ )
	{
		var self = this;
		var $txt;
		var scroll_data = this.getScrollBasicInfo();
		scroll_data.destination = "bottom";
		scroll_data.callback = function() {
			self.focusInput($txt);
		};

		if ( this.m_interaction.isPrivateMessage() ) {
			$txt = $("#instagramMailText");
		}
		else {
			$txt = $("#postCommentText");
			if ( parent_comment ) {
				scroll_data.destination = "item_position";
				scroll_data.type = "without_animation";
			}
		}
		this.clearInput($txt);

		var fake_event = this.make_fake_event({
			fake_event_id  : fake_event_id,
			message        : message,
			parent_comment : parent_comment
		});

		this.add_new_message_stub( fake_event, function( $comment )
		{
			if ( scroll_data.destination === "item_position" ) {
				scroll_data.$item = $comment;
			}
			SCROLL_TO( scroll_data );
		});
	};

	InstagramClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	InstagramClientWindow.prototype.realtime_add_new_incoming_message = function ( $comment, new_event )
	{
		if ( !$comment || $comment.length === 0 ) {
			return false;
		}

		var can_scroll = false;
		var $container = $("#thread_container");
		var parent_comment = new_event.message.parent_id;
		var is_reply = _.isString( parent_comment );
		var $siblings = is_reply ? 
						$container.find(".replyMessage[data-parent_id='" + parent_comment +"']") :
						$siblings = $container.find(".topLevelMessage");

		if ( $siblings.length === 0 ) {
		// si es el primer evento del hilo principal o del hilo de una respuesta no tiene hermanos, lo inserta primero en la lista
			if ( is_reply ) {
				$container.find("[data-id='" + parent_comment +"']").after( $comment );
			}
			else {
				$container.append( $comment );
			}
		}
		else {
		// Si tiene hermanos, recorre el hilo al cual pertenece, buscando entre sus hermanos el primero que tenga una fecha/hora menor a la del current event.
		// La busqueda es realizada comenzando desde el ultimo ( ya que estan ordenado x fecha ).
			var added = false;
			var new_event_time = new_event.creationDate.getTime();

			for ( var i = $siblings.length - 1; i >= 0; i -- ) {
				if ( !$siblings[ i ] ) {
					continue;
				}
				var $sibling = $( $siblings[ i ] );
				var sibling_time = $sibling.data( "time" );
				if ( sibling_time <= new_event_time ) {
				// si la fecha del hermano es menor o igual al nuevo evento, inserto este nuevo evento despues del hermano
					$sibling.after( $comment );
					added = true;
					break;
				}
			}

			if ( !added ) {
			// si no existe un hermano menor tiene que agregarlo si o si al nodo
				if ( is_reply ) {
					$siblings.last().after( $comment );
				}
				else {
					$container.append( $comment );
				}
			}
		}

		delete new_event.is_new;
		return can_scroll;
	};

	InstagramClientWindow.prototype.add_new_message_stub = function ( new_event, callback, sorted )
	{
		var self = this;
		callback = callback || $.noop;
		var stub = new_event;
		this.m_interaction.getEvent( 0 ).children.push( stub );
		var event_index = new_event.__index || this.m_interaction.getEventCount();
		var $container = $("#thread_container");

		//
		this.insert_comment_in_dom(
			stub,
			event_index,
			function( $comment )
			{
				var can_scroll = true;
				if ( new_event.is_new && sorted ) {
				// aca se insertan los evento entrantes cuando la pagina es chequeada por el proxy
					can_scroll = self.realtime_add_new_incoming_message( $comment, new_event );
				}
				else {
				// aca se insertan los mensajes de la carga inicial
					var parent_comment = new_event.message.parent_id;
					var postId = self.m_interaction.interactionData.postId;
					if ( _.isString( parent_comment ) && parent_comment !== postId) {
						var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

						if ( $siblings.length === 0 ) {
							$container.find("[data-id='" + parent_comment +"']").after( $comment );
						}
						else {
							$siblings.last().after( $comment );
						}
					}
					else {
						$container.append( $comment );
					}
				}

				$comment.addClass( "new" );
				window.setTimeout( function() {
					$comment.removeClass( "new" );
				}, TIME_SELECTED_COMMENT_MS );

				callback( $comment, can_scroll );
			}
		);
	};

	InstagramClientWindow.prototype.setReplyToComment = function( commentId, name )
	{
		var self = this;
		this.m_reply_to_comment = commentId;
		$(".instagramInteractionControlTitle").show().text( "<%=_T('Reply to')%>" + "<%=_T(':')%>" + " " + name );
		$("#instagramCommentButton").val( "<%=_T('SEND REPLY')%>" );
		$("#mainContainer").addClass("commentReply");
		self.focusInput($("#postCommentText"));

		$("#cancelReplyToComment").show().unbind('click').bind('click', function() {
			$( this ).hide();
			$(".instagramInteractionControlTitle").hide();
			$("#instagramCommentButton").val( "<%=_T('COMMENT')%>" );
			$("#mainContainer").removeClass("commentReply");
			self.focusInput($("#postCommentText"));
			self.m_reply_to_comment = null;
		});
	};

	InstagramClientWindow.prototype.setPrivateReply = function( index, name )
	{
		var self = this;
		this.m_private_reply_index = index;
		$("#instagramComment").hide();
		$("#instagramPrivateMessage").show().find(".instagramInteractionControlTitle").show().text( "<%=_T([[Private message to]])%>" + "<%=_T([[:]])%>" + " " + name );
		$("#instagramCommentButton").val( "<%=_T([[SEND MESSAGE]])%>" );
		$("#mainContainer").addClass("commentReply");
		if (this.m_legazyPost) {
			$("#instagram_solve_container").show();
		}
		self.focusInput($("#postCommentText"));

		var $txt = $("#instagramMailText");
		$txt.off('change').on('change', function() {
			$("#instagramMailButtonSend").prop("disabled", !$(this).val());
		}).change();
		
		$("#instagramMailButtonSend").unbind('click').bind('click', function() {
			self.comment_current_post(function(success) {
				if(success) {
					$("#cancelPrivateReplyToComment").hide();
					$("#instagramComment").show();
					$("#instagramPrivateMessage").hide().find(".instagramInteractionControlTitle").hide();
					$("#instagramCommentButton").val( "<%=_T([[COMMENT]])%>" );
					$("#mainContainer").removeClass("commentReply");
					self.focusInput($("#postCommentText"));
					if (self.m_legazyPost) {
						$("#instagram_solve_container").hide();
					}	
				}
			});
		});		

		$("#cancelPrivateReplyToComment").show().unbind('click').bind('click', function() {
			$( this ).hide();
			$("#instagramComment").show();
			$("#instagramPrivateMessage").hide().find(".instagramInteractionControlTitle").hide();
			$("#instagramCommentButton").val( "<%=_T([[COMMENT]])%>" );
			$("#mainContainer").removeClass("commentReply");
			self.focusInput($("#postCommentText"));
			self.m_private_reply_index = null;
			if (self.m_legazyPost) {
				$("#instagram_solve_container").hide();
			}
		});
	};

	InstagramClientWindow.prototype.getMessageColapsedComments = function( increment_count )
	{
		var count = this.m_interaction.getEventList().length + ( increment_count || 0 );
		return FORMAT( "<%=_T('View all {0} comments')%>", count );
	};

	InstagramClientWindow.prototype.updateMessageColapsedComments = function()
	{
		var $message = $('.instagramThreadSubEventSummary');
		//if the hidden markup is visible, update it with + 1 comment count
		if ( $message.is(':visible') ) {
			var message = this.getMessageColapsedComments();
			$message.find('a').text( message );
		}
	};

	InstagramClientWindow.prototype.on_comment_post_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			self.clearInput($("#postCommentText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
		// aca actualizo en pantalla el evento pending/sending.
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( $comment, can_scroll )
			{
				if ( can_scroll ) {
					var exist_comment_in_dom = $comment && $comment.length > 0;

					var scroll_data = self.getScrollBasicInfo();
					scroll_data.callback = function() {
						if ( !exist_comment_in_dom ) {
							return;
						}
					};

					if ( exist_comment_in_dom) {
						scroll_data.destination = "item_position";
						scroll_data.type = "without_animation";
						scroll_data.$item = $comment;
					}
					else {
						scroll_data.destination = "bottom";
					}

					SCROLL_TO( scroll_data );
				}

				if ( !incoming_realtime ) {
					$("#cancelReplyToComment").trigger("click");
				}
			};

			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};

	InstagramClientWindow.prototype.comment_current_post = function(callback = $.noop)
	{
		var self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		let isPrivateReply = this.m_private_reply_index && this.m_private_reply_index > 0;

		var text = isPrivateReply ? $("#instagramMailText").val() : $("#postCommentText").val();
		this.focusInput($("#postCommentText"));
		if (text && text.length > 0) {
			var message = { text : text };
			this.hideStatusMessage();
			this.displayStatusMessage("<%=_T('Sending comment...')%>", "Process", true);
			var event = this.get_current_event();
			if ( !event ) {
				return;
			}
			
			//mark all as read
			if ($("#mark_all_read_unread").val() === this.MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			
			if (isPrivateReply) {
				return this.private_reply(this.m_private_reply_index, { text : text }, callback);
			}

			//reply to comment, or reply to post
			//IMPORTANT: for nested comments to work properly, the page in Instagram
			//must be configured with the option "Allow replies to comments on my Page"
			//otherwise it won't work

			var postId = this.m_interaction.interactionData.postId;
			var commentId = this.m_reply_to_comment || this.m_interaction.interactionData.commentId || "";

			var fake_event_id = this.m_interaction.commentPost(
				undefined, // fake event id - tiene que ser pasado como parametro en undefined
				postId,
				commentId,
				message,
				function( success, message ) {
					if ( success ) {
					// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
						return callback(true);
					}
					event.fake_event_id = fake_event_id;
					event.fake_event_id = fake_event_id;
					self.hideStatusMessage();
					self.displayStatusMessage( message, "Error", true, 5000 );
					self.delivery_remove( event );
					callback(false, message || "error sending message");
				}
			);
			this.add_new_outgoing_fake_event( fake_event_id, message, commentId);
		}
	};

	InstagramClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let typeClass = 'instagramEventType' + (this.m_privateMessages? 'PrivateMessage' : 'Wall');
		let photoClass = properties.event.message.type ? "photoPost": "";
		let readClass = "relatedInteractionUnread";
		let message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessage(properties.sorted_index));
		let message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		let author_name = this.m_interaction.getUserName(properties.event);
		let genericAvatar = this.getGenericAvatar();
		let avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div unread_event="true" class="relatedInteraction ', readClass, ' ', typeClass, ' ', photoClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png" width=“36px" height="36px">',
						'<img src="', genericAvatar, '" width="36px" height="36px"/>',
					'</object>',
				'</div>',
				'<div class="interactionInfoContainer">',
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i class="far fa-chevron-right"></i>',
				'</a>',
			'</div>'));
	};
	
	InstagramClientWindow.prototype.__getButtonMarkup = function(button)
	{
		return FORMAT("".concat(
			"<button class='instagramButton'>",
				"<label class='instagramButtonLabel'>",
					"{0}",
				"</label>",
			"</button>"
		), button.title);
	};
	
	InstagramClientWindow.prototype.__getGalleryElementMarkup = function(element)
	{
		var self = this;
		var imageHtml = element.image_url? FORMAT("".concat(
			"<div class='instagramGalleryElementImageContainer'>",
				"<img class='instagramGalleryElementImage' src='{0}'>",
			"</div>"), element.image_url) : "";
		var buttonsHtml = "";
		$.each(element.buttons || [], function(_, button) {
			buttonsHtml += self.__getButtonMarkup(button);
		});
		var galleryelementMarkupSubtitle = (!element.subtitle || element.subtitle === "") ? "" : FORMAT("".concat(
			"<label class='instagramGalleryElementSubtitle'>",
				"{0}",
			"</label>"
		), element.subtitle);
		var galleryelementMarkupButtons = buttonsHtml === "" ? "" : FORMAT("".concat(
				"<div class='instagramGalleryButtons'>",
					"{0}",
				"</div>"
		), buttonsHtml);

		return FORMAT("".concat(
			"<div class='instagramGalleryElement'>",
				"{2}",
				"<label class='instagramGalleryElementTitle'>",
					"{0}",
				"</label>",
				"{1}",
				"{3}",
			"</div>"
		), element.title, galleryelementMarkupSubtitle || "", imageHtml, galleryelementMarkupButtons);
	};
	
	InstagramClientWindow.prototype.__getListElementMarkup = function(element)
	{
		var self = this;
		var imageHtml = element.image_url? FORMAT("".concat(
			"<div class='instagramListElementImageContainer'>",
				"<img class='instagramListElementImage' src='{0}'>",
			"</div>"), element.image_url) : "";
		var buttonsHtml = "";
		$.each(element.buttons || [], function(_, button) {
			buttonsHtml += self.__getButtonMarkup(button);
		});
		return FORMAT("".concat(
			"<div class='instagramListElement'>",
				"<label class='instagramListElementTitle'>",
					"{0}",
				"</label>",
				"<label class='instagramListElementSubtitle'>",
					"{1}",
				"</label>",
				"<div class='instagramListButtons'>",
					"{3}",
				"</div>",
				"{2}",
			"</div>"
		), element.title, element.subtitle || "", imageHtml, buttonsHtml);
	};
	
	InstagramClientWindow.prototype.__getImageMarkup = function(link) 
	{
		return link? FORMAT("<img class='imagePostedContainer' src='{0}'>", link) : "";
	};
	
	InstagramClientWindow.prototype.__getLinkMarkup = function(type, link, linkName, imageLink) 
	{
		return link? FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>",
			(type === "video"? "instagramVideoLink" : "instagramkLink") + (imageLink? "Image" : ""), 
			link, 
			this.__getImageMarkup(imageLink) || linkName || link) : "";
	};
	
	InstagramClientWindow.prototype.__getAudioMarkup = function(link, mimeType)
	{
		return "".concat(
			"<div class='instagramMedia instagramAudio' file_type='audio' mime_type='", mimeType || "audio/mp3", "'>",
				"<button class='instagramMediaPlay'>",
					"<i class='buttons-icons-start'></i>",
				"</button>",
				"<button class='instagramMediaPause' hidden>",
					"<i class='buttons-icons-voz-hold'></i>",
				"</button>",
				"<button class='instagramMediaStop' hidden>",
					"<i class='buttons-icons-stop'></i>",
				"</button>",
				//En instagram los attachments no los subimos al repo, por lo que la URL es la nativa. 
				//Al ser un dominio distinto al nuestro, el navegador no hace el download sino el open en un nuevo tab
				//Por tanto no mostramos el botón de download
				"<a class='instagramMediaDownload' href='", link, "' target='_blank' hidden>",
					"<i class='buttons-icons-download'></i>",
				"</a>",
			"</div>");
	};
	
	InstagramClientWindow.prototype.__getVideoMarkup = function(link, mimeType)
	{
		return "".concat(
			"<div class='instagramMedia instagramVideo' file_type='video' mime_type='", mimeType || "video/mp4", "'>",
				"<button class='instagramMediaPlay'>",
					"<i class='far fa-play-circle'></i>",
					"<label><%=_T([[Watch video]])%></label>",
				"</button>",
				//En instagram los attachments no los subimos al repo, por lo que la URL es la nativa.
				//Al ser un dominio distinto al nuestro, el navegador no hace el download sino el open en un nuevo tab
				//Por tanto no mostramos el botón de download
				"<a class='instagramMediaDownload' href='", link, "' target='_blank' hidden>",
					"<i class='buttons-icons-download'></i>",
				"</a>",
			"</div>");
	};
	
	InstagramClientWindow.prototype.__getFileMarkup = function(file) {
		let url = file.id? (inConcert.app.getBaseDomain() + "/inconcert/apps/repository/download/?id=" + 
			encodeURIComponent(file.id) + "&section=-1&part=-1") : file.url;
		let previewUrl = file.previewFileId? (inConcert.app.getBaseDomain() + "/inconcert/apps/repository/download/?id=" + 
			encodeURIComponent(file.previewFileId) + "&section=-1&part=-1") : file.previewUrl;
		//2020-09-14 DEPRECATED: fix some days before build error (remove after some years)
		file.fileType = file.fileType || file.type.split("/")[0];
		switch(file.fileType) {
			case "image":
				return this.__getImageMarkup(url);
			case "video":
				return file.id? this.__getVideoMarkup(url, file.type) :
					this.__getLinkMarkup("video", url, file.name, previewUrl);
			case "audio":
				return file.id? this.__getAudioMarkup(url, file.type) :
					this.__getLinkMarkup("video", url, file.name, previewUrl);
			default:
				return this.__getLinkMarkup("file", url, file.name, previewUrl);
		}
	};

	InstagramClientWindow.prototype.getAttachMarkup = function( event )
	{
		var self = this;
		// retorna el markup correspondiente a imagenes, etc.
		// tambien retorna un disclaimer si tiene archivos adjuntos.

		var response = {
			disclaimer : "",
			media      : ""
		};

		if (!event) {
			return response;
		}

		let files = this.m_interaction.getFiles(event);
		if (files) {
			$.each(files, function(_, file) {
				response.media += self.__getFileMarkup(file);
			});
		}
		else {
			//direct messages shares link
			var share_data;
			if (event.message && event.message.shares && event.message.shares.data && $.isArray(event.message.shares.data) && event.message.shares.data.length && event.message.shares.data[0]) {
				share_data = event.message.shares.data[0];
			}
			if (share_data && share_data.link) {
				response.media += FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>", "instagramLink", share_data.link, share_data.name || share_data.description || "<%=_T('Link')%>");
			}
		}

		var attachment = event.message && event.message.attachment || event.message.attachments;
		if ( attachment ) {
			if ( $.isPlainObject(attachment)) {
				attachment = [attachment];
			}
			if ( $.isArray(attachment) ) {
				$.each( attachment, function( _, current_attach ) {
					if ( typeof current_attach !== "object" ) {
						return true; // continue
					}
					
					var content = {};
					if (current_attach.image_data && current_attach.image_data.url) {
						content.type = "photo";
						content.picture = current_attach.image_data.url;
					}
					else if (current_attach.video_data && current_attach.video_data.url) {
						content.type = "video";
						content.picture = current_attach.video_data.preview_url;
						content.link = current_attach.video_data.url;
					}
					else if (current_attach.file_url) {
						content.type = "file";
						content.link = current_attach.file_url;
						content.linkName = current_attach.name;
					}
					else if (current_attach.type) {
						switch(current_attach.type) {
							case "photo":
								content.type = "photo";
								content.picture = current_attach.media && current_attach.media.image && 
									current_attach.media.image.src || "";
								break;
							case "sticker":
								content.type = "sticker";
								content.picture = current_attach.media && current_attach.media.image && 
									current_attach.media.image.src || "";
								break;
							case "image":
								content.type = "photo";
								content.picture = current_attach.payload.url;
								break;
							case "file":
								content.type = "file";
								content.link = current_attach.payload.url;
								var parts = content.link.split("/");
								content.linkName = parts[parts.length - 1].replace(/\?.*/g, "");
								break;
							case "location":
								content.type = "file";
								content.link = inConcert.Utils.getUrlParameter(current_attach.url, "u") ||
									current_attach.url;
								content.linkName = "<%=_T('My location')%>";
								break;
							case "template":
								var payload = current_attach.payload || {};
								switch(payload.template_type) {
									case "button":
										var buttons = "<div class='instagramButtonsContainer'>";
										$.each(payload.buttons, function(_, button) {
											buttons += self.__getButtonMarkup(button);
										});
										buttons += "</div>";
										response.media += buttons;
										break;
									case "generic":
										var generic_elements = FORMAT("<div class='instagramGalleryContainer {0}Gallery'>",
											payload.image_aspect_ratio);
										$.each(payload.elements, function(_, element) {
											generic_elements += self.__getGalleryElementMarkup(element);
										});
										generic_elements += "</div>";
										response.media += generic_elements;
										break;
									case "list":
										var list = "<div class='instagramListContainer'>";
										$.each(payload.elements, function(_, element) {
											list += self.__getListElementMarkup(element);
										});
										list += "</div>";
										response.media += list;
										if (payload.buttons && payload.buttons.length) {
											var btns = "<div class='instagramListBottomButtonsContainer'>";
											$.each(payload.buttons, function(_, button) {
												btns += self.__getButtonMarkup(button);
											});
											btns += "</div>";
											response.media += btns;
										}
										break;
								}
								break;
							case "video":
								response.media += current_attach.payload && current_attach.payload.url ? self.__getVideoMarkup(current_attach.payload.url, "video/mp4") :
									FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>", "instagramLink", current_attach.payload && current_attach.payload.url || current_attach.url, current_attach.linkName || "<%=_T('Link')%>");
								break;
							case "audio":
								response.media += current_attach.payload && current_attach.payload.url ? self.__getAudioMarkup(current_attach.payload.url, "video/mp4") :
									FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>", "instagramLink", current_attach.payload && current_attach.payload.url || current_attach.url, current_attach.linkName || "<%=_T('Link')%>");
								break;
							default:
								content.type = current_attach.type.indexOf("video") >= 0? "video" : "link";
								content.link = current_attach.payload && current_attach.payload.url || current_attach.url;
								content.picture = current_attach.media && current_attach.media.image && 
									current_attach.media.image.src || "";
								break;
						}
					}

					if (content.picture) {
						var pictureHtml = "{0}";
						if (content.link) {
							pictureHtml = FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>",
								content.type === "video"? "instagramVideoLinkImage" : "instagramLinkImage", content.link, "{0}");
						}
						response.media += FORMAT(pictureHtml,
							FORMAT("<div class='instagramGalleryImageContainer'><div class='cropImage'><img class='imagePostedContainer' src='{0}'></div></div>", content.picture));
					}
					else if (content.link) {
						response.media += FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>",
							"instagramLink", content.link, content.linkName || "<%=_T('Link')%>");
					}
				});
			}
		}
		
		//quickreplies
		if (event.message && event.message && event.message.quick_replies && event.message.quick_replies.length) {
			var quickReplies = "<div class='instagramQuickrepliesContainer'>";
			$.each(event.message.quick_replies, function(_, quickReply) {
				var text = "";
				var icon = "";
				switch(quickReply.content_type) {
					case "location":
						text = "<%=_T('Location')%>";
						icon = "/inconcert/apps/agent/images/locationIcon.PNG";
						break;
					case "text":
						text = quickReply.title;
						icon = quickReply.image_url || "";
						break;
				}
				var iconHtml = icon? FORMAT("".concat(
						"<i class='instagramQuickreplyIcon'>",
							"<img class='instagramQuickreplyIconImage' src='{0}'>",
						"</i>"), icon) : "";
				quickReplies += FORMAT("".concat(
					"<button class='instagramQuickreply'>",
						"{1}",
						"<label class='instagramQuickreplyLabel'>",
							"{0}",
						"</label>",
					"</button>"
				), text, iconHtml);
			});
			quickReplies += "</div>";
			response.media += quickReplies;
		}

		return response;
	};

	InstagramClientWindow.prototype.isReadEvent = function( event, event_index )
	{
		if ( typeof event !== "object" ) {
			throw "InstagramClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};

	InstagramClientWindow.prototype.insert_comment_in_dom = function( event, event_index, callback = $.noop )
	{
		const self = this;

		event_index = event_index || -1;
		var failed_event;

		if ( event.outgoing_pending && event.timestamp ) {
			if ( event.state === "FAILED" ) {
				failed_event = $.extend( {}, event );
			}
			event = this.make_fake_event({
				fake_event_id  : event.fake_event_id,
				message        : event.message,
				parent_comment : event.parent_id,
				creation_date  : event.creationDate,
				state          : event.state,
				agent          : event.agentAtVCC || event.agent
			});
		}
		var id = event.message.id;
		var parent_id = event.message.parent_id;

		var author_name = this.m_interaction.getUserName( event );
		var author_id   = this.m_interaction.get_user_event_id( event );
		var attach_html = this.getAttachMarkup( event );
		var story_html = this.getStoryMarkup(event);
		var message_text = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib( this.m_interaction.getMessageText(event.message) ), { exclude: { email: true } } );
		var message_class = (event.message.webhookType === "REFERRAL" || event.message.webhookType === "OPTIN")? "referralMessage" : "";

		var postId = this.m_interaction.get_origin_message_id();
		var has_parent = _.isString( parent_id ) && parent_id !== postId;
		var html_reply = '';

		if (this.isEditable()) {
			if ( event.outgoing_pending !== true && this.m_mention ) {
				html_reply = FORMAT(
					''.concat(
						'<a reply_comment_link class="threadActionsReply" href="javascript:void(0);"',
								' data-event_index="{0}" data-message_id="{1}" data-user_name="{2}">',
							"<%=_T('Reply')%>",
						'</a>'
					),
					event_index,						// data-event_index
					has_parent? parent_id : id,			// data-message_id
					author_name							// data-user_name
				);
			}
		}

		var class_container, attr_parent_id;
		if ( has_parent ) {
			class_container = "replyMessage";
			attr_parent_id = ' data-parent_id="' + parent_id + '" ';
		}
		else {
			class_container = "topLevelMessage";
			attr_parent_id = '';
		}

		if (event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}
		if(this.isMessageLiked(event_index)) {
			class_container += " liked";
		}
		if(this.isMessageDeleted(event_index)) {
			class_container += " deleted";
		}

		var read_unread_lnk = this.get_html_read_unread_link( event, event_index, true );
		var private_reply_lnk = this.get_html_private_reply_link( event, event_index, true );
		var like_unlike_lnk = this.get_html_like_unlike_link( event, event_index );
		var like_unlike_from_client = this.get_html_like_unlike_from_client(event);
		var delete_from_client = this.get_html_deleted_message_from_client(event);
		class_container += read_unread_lnk.parent_class;
		class_container += like_unlike_lnk.parent_class;

		var attr_id =  ' data-id="' + id + '" ';
		var attr_index =  ' data-event_index="' + event_index + '" ';
		var time = ' data-time="' + event.creationDate.getTime() + '" ';

		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI(event) || genericAvatar;

		var $comment = $( ''.concat(
			'<div class="instagramThreadSubEvent ', class_container, '" ', attr_id, attr_parent_id, attr_index, time, '>',

				'<div class="subEventMainContainer">',
					'<div class="subEventAvatar">',
						'<object data="', avatar, '" type="image/png" width="24" height="24">',
							'<img src="', genericAvatar, '" width="24" height="24"/>',
						'</object>',
					'</div>',

					'<div class="subEventMessageContainer">',
						'<a href="javascript:void(0);" class="subEventAuthor" data-event_index="', event_index , '" data-event_user_id="', author_id , '">', author_name, '</a>',
						'<label event_date="true" class="subEventDate">', inConcert.Utils.userFormatDate(event.creationDate, false, true), '</label>',
						story_html,
						'<label message_text="true" class="subEventMessage ', message_class , '">', message_text, '</label>',
						attach_html.media,
					'</div>',
					'<div class="subEventMessageRightContainer">',
						'<div class="subEventMessageRightTopContainer">',
							//'<label event_date="true" class="subEventDate">', inConcert.Utils.userFormatDate(event.creationDate, false, true), '</label>',
							/*
								SE OCULTA EL BOTON DE 3 PUNTITOS POR MEDIO DE CSS SI NO HAY NINGUNA OPCION EN EL DROPDOWN: 
									masterClientInstagram.css  -> .subEventActionsContainer:empty ~ .butInteractionGeneric
							*/
							'<div class="dropdown">',
								'<div class="subEventActionsContainer dropdown-menu">',
									html_reply,
									read_unread_lnk.html,
									private_reply_lnk.html,
									attach_html.disclaimer,
									delete_from_client,
								'</div>',
								'<button class="butInteractionGeneric butShowMoreActions" type="button" data-toggle="dropdown" aria-expanded="false">',
									//'<i class="fas fa-ellipsis-v"></i>',
								'</button>',
							'</div>',
						'</div>',
						'<div class="subEventMessageRightBottomContainer">',
							like_unlike_lnk.html,
							like_unlike_from_client,
						'</div>',
					'</div>',
				'</div>',
				'<div class="deliveryFailActions" hidden>',
					'<a discard_message class="threadActionsDiscardFailed pageButtonGeneric buttonColorWhite" href="javascript:void(0);">', "<%=_T('Discard')%>", '</a>',
					'<a resend_message class="threadActionsResendFailed pageButtonGeneric buttonColorBlue" href="javascript:void(0);">', "<%=_T('Resend')%>", '</a>',
				'</div>',

				// label de estado
				'<div class="deliveryMessageContainer" hidden >',
					'<label class="deliveryMessageText" ></label>',
				'</div>',

				// tooltip - mensaje de por que fallo
				'<div class="deliveryFailReasonContainer" hidden>',
					'<label class="deliveryFailReasonTxt"></label>',
				'</div>',
			'</div>'
		));
		
		this.__processMediaPlayers($comment);

		if (event.creator === "contact") {
			const $message_text = $comment.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		// inserta in dom
		callback( $comment );

		if ( event.outgoing_pending ) {
			var state = event.state || "";
			switch( state ) 
			{
				case "DELIVERED":
					// NO HAGO NADA!!! ( delivery_success_message )
				break;
				case "DISCARDED":
					this.delivery_discarded_message( $comment );
				break;
				case "DISCARDING":
					this.delivery_discarding_message( $comment );
				break;
				case "FAILED":
					var short_msg = this.m_interaction.get_error_message( failed_event.error_code );
					this.delivery_failed_message( failed_event, short_msg );
				break;
				case "SENDING":
					this.delivery_sending_message( $comment );
				break;
				default:
					throw "Unrecognized pending event '" + state + "' state";
			}
		}
	};

	// Reorder events so that nested comments are placed right after the original comment
	// But maintaining the chronological order between comments in the same scope
	InstagramClientWindow.prototype.reorder_nested_events = function ()
	{
		//TODO: Revisar esta funcion, que una linda y hermosa bosta!!
		//		ya lo era, y ahora aun mas!!! Despues que este aprobado el issue de ACK, revisar y realizar refactoring de la misma Ellio!!
		//

		var pending_events = {};
		var count_pending_events = this.m_interaction.count_total_pending_events();

		if ( count_pending_events > 0 ) {

			pending_events = inConcert.Utils.cloneObject( this.m_interaction.get_pending_events() );

			// Es necesario, agregar algunos fake atributos los eventos pending ( creationDate, outgoing_pending, parent_id )
			//
			// creationDate: a partir del timestamp, necesito crear un objeto Date, para que pueda ser ordenado el hash de pending, y para que lo pueda insertar en orden en la lista posta de eventos.
			// outgoing_pending: es necsario para diferenciar los eventos pending de los que no son pending en la lista unificada de sorted_events, para su posterior utilizacion.
			// parent_id: se necesita este atributo para que puedan ser ordenados en su propio hilo de comments.

			$.each( pending_events, function( _, pending_event )
			{
				pending_event.creationDate = new Date( parseInt( pending_event.timestamp, 10 ) * 1000 );
				pending_event.outgoing_pending = true;
				if ( pending_event.reply ) {
					pending_event.parent_id = pending_event.destination;
				}
			});

			// si tengo mas de un evento pending, mando ordenar
			if ( count_pending_events > 1 ) {
				// Ordena el hash por fecha de creacion
				pending_events = inConcert.Utils.sorted_hash( 
					// hash desordenado
					pending_events,

					// funcion que retorna el valor por el cual realizar el orden de los pendings ( fecha de creacion )
					function( pending_event ) { return pending_event.creationDate; },

					// funcion que genera el hash ordenado
					function( target_hash, pending_event ) { target_hash[ pending_event.Id ] = pending_event; return target_hash; }
				);
			}
		}

		var replies_by_top_level = {};
		var events = this.m_interaction.getEventList() || [];
		var i, e, parent_id;
		//first iterate over all events and group each one by parent
		for ( i = 0; i < events.length; i++ ) {
			e = events[i];
			parent_id = e.message.parent_id;
			if ( parent_id ) {
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push(e);
			}
		}

		// agrego los pending de replies a la lista de replies
		var added_pending_events = {};
		$.each( pending_events, function( pending_id, pending_event )
		{
			if ( pending_event.reply ) {
				var parent_id = pending_event.parent_id;
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push( pending_event );
				added_pending_events[ pending_id ] = true;
			}
		});

		// ahora reordeno la lista de replies x fecha
		$.each( replies_by_top_level, function( parent_id, list )
		{
			replies_by_top_level[ parent_id ] = inConcert.Utils.sortArray( list, "creationDate" );
		});
		// hago clean up de los pendings que ya se han agregado
		$.each( added_pending_events, function( pending_id )
		{
			delete pending_events[ pending_id ];
		});


		// esta funcion agrega los pending a nivel de post original, ordenados por fecha
		var add_sorted_pending_events = function( list, current_event, compare_fn )
		{
			var added_pending_events = {};
			$.each( pending_events, function( pending_id, pending_event )
			{
				if ( compare_fn( pending_event, current_event ) ) {
					list.push( pending_event );
					added_pending_events[ pending_id ] = true;
				}
			});

			$.each( added_pending_events, function( pending_id )
			{
				delete pending_events[ pending_id ];
			});
		};

		var sort_pending_fn = function( pending_event, current_event ) { return pending_event.creationDate <= current_event.creationDate; };
		var sorted_events = [];
		var id;
		//second, push each event in the final array
		//if the event has replies, the push each one after it
		for (i = 0; i < events.length; i++) {
			e = events[i];
			id = e.message.id;
			parent_id = e.message.parent_id;

			if ( !parent_id ) {
				add_sorted_pending_events(
					sorted_events,
					e,
					sort_pending_fn
				);
				sorted_events.push(e);
				if (replies_by_top_level[id]) {
					sorted_events = sorted_events.concat(replies_by_top_level[id]);
					delete replies_by_top_level[id];
				}
			}
		}

		// x ulitmo tengo que agregar los pendings a nivel post original que no se han agregado!!
		add_sorted_pending_events(
			sorted_events,
			undefined,
			function() { return true; }
		);

		//these comments does not have any parent
		//we put them at the last place
		for ( id in replies_by_top_level ) {
			if ( replies_by_top_level.hasOwnProperty(id) ) {
				sorted_events = sorted_events.concat(replies_by_top_level[id]);
			}
		}

		return sorted_events;
	};

	InstagramClientWindow.prototype.canShowRelatedSections = function()
	{
		return this.m_interaction.getUnreadEventCount() > 0;
	};

	InstagramClientWindow.prototype.closeRelated = function( original_index )
	{
		var self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, self.m_currentEvent ) ) {
					var id = current_event.message.id;
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
			if ( self.m_interaction.isPrivateMessage() ) {
				self.focusInput($("#instagramMailText"));
			}
			else {
				self.focusInput($("#postCommentText"));
			}
		};

		if ( !this.can_show_all_comments ) {
			this.can_show_all_comments = original_index === undefined ? false : true;
		}

		InstagramClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( current_event === false || !current_event.message || !current_event.message.id ) {
				on_ready();
				return;
			}
			var id = current_event.message.id;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	InstagramClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( $comment.hasClass( "instagramThreadOrigin" ) ) {
		// origin link class
			class_mark_as_read = LNK_ORIGIN_MARK_AS_READ_CLASS;
			class_mark_as_unread = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T('Mark as unread')%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T('Mark as read')%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	InstagramClientWindow.prototype.on_liked_event = function( success, liked, $object )
	{
		if ( !success || typeof $object !== "object" || $object.length === 0 ) {
			return;
		}
		
		$object
			.toggleClass("liked", liked)
			.find("[like_unlike]")
				.toggleClass(LNK_UNLIKE_CLASS, liked)
				.toggleClass(LNK_LIKE_CLASS, !liked)
				.data("event_like", liked? "unlike" : "like")
				//.html(liked? "<%=_T('Unlike')%>" : "<%=_T('Like')%>");//
				.html(liked? "" : "");
		
	};

	InstagramClientWindow.prototype.get_html_read_unread_link = function( event, event_index, is_thread )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		var lnk_mark_as_read_class = LNK_THREAD_MARK_AS_READ_CLASS;
		var lnk_mark_as_unread_class = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( !is_thread ) {
			lnk_mark_as_read_class = LNK_ORIGIN_MARK_AS_READ_CLASS;
			lnk_mark_as_unread_class = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		if ( event.outgoing_pending === true || event.creator === 'agent' ) {
			return response;
		}

		if ( !this.isReadEvent( event, event_index ) ) {
			response.parent_class = ' unread';
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_read_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as read')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'unread'
				);
			}
		}
		else {
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_unread_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as unread')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'read'
				);
			}
		}

		return response;
	};

	InstagramClientWindow.prototype.get_html_like_unlike_link = function( _ /*event*/, event_index )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		// Habilitar like_unlike solo para mensajes del cliente (para comment no esta soportado)
		event_index = this.m_interaction.get_sorted_index_by_original_index(event_index);
		if (event_index === false || !this.m_privateMessages || this.isMessageDeleted(event_index)){
			return response;
		}

		var liked = this.m_interaction.isLiked(event_index);
		if (liked) {
			response.parent_class = " liked";
		}
		if ( this.isEditable() && this.m_interaction.events[event_index].creator === "contact" && this.m_interaction.events[event_index].message.from.id) {
			response.html = FORMAT(
				''.concat(
					'<a like_unlike class="', liked? LNK_UNLIKE_CLASS : LNK_LIKE_CLASS, '" href="javascript:void(0);"',
							' data-event_index="{0}" data-event_like="{1}">',
							liked? "" : "",
						//liked? "<%=_T('Unlike')%>" : "<%=_T('Like')%>",//
					'</a>'
				),
				event_index,  // data-event_index
				liked? "unlike" : "like"
			);
		}

		return response;
	};

	InstagramClientWindow.prototype.get_html_private_reply_link = function( event, event_index )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		if ( event.outgoing_pending === true || event.creator === 'agent' || !this.m_interaction.canPrivateReply(event_index)) {
			return response;
		}

		var id = event.message.id;
		var parent_id = event.message.parent_id;

		var author_name = this.m_interaction.getUserName( event );
		var postId = this.m_interaction.get_origin_message_id();
		var has_parent = _.isString( parent_id ) && parent_id !== postId;

		if ( this.isEditable() ) {
			response.html = FORMAT(
				''.concat(
					'<a private_reply_link class="threadActionsPrivateReply" href="javascript:void(0);"',
							' data-event_index="{0}" data-message_id="{1}" data-user_name="{2}" title="', 
							'<%=_T("Reply user by a private message")%>' ,'">',
						"<%=_T([[Private]])%>",
					'</a>'
				),
				event_index,						// data-event_index
				has_parent? parent_id : id,			// data-message_id
				author_name	
			);
		}

		return response;
	};

	InstagramClientWindow.prototype.openCurrentEvent = function(callback = $.noop)
	{
		const self = this;

		var execute_callback = function( success ) {
			self.hideStatusMessage();
			callback( success );
			if (self.__loadDraftMessage) {
				self.__loadDraftMessage = false;
				self.loadDraftMessage();
			}
		};

		if ( this.are_showing_all_messages || this.loading_all_comments ) {
			execute_callback( false );
			return;
		}

		var $container = $("#thread_container");

		$container.find(".instagramThreadOrigin").remove();
		$container.find(".instagramThreadSubEventSummary").remove();
		$container.find(".instagramThreadSubEvent").remove();

		var firstEventIndex = 0;
		var current_event = this.m_interaction.getEvent( firstEventIndex );
		if ( !current_event || !current_event.message || current_event.message.pending ) {
			execute_callback( false );
			return;
		}

		var getMessageMarkup = function() {
			var escAttrib = inConcert.Utils.escapeAttrib;
			var linkify = inConcert.Utils.linkify;
			var messageMarkup = "";
			var content = self.m_interaction.getMessageContent( firstEventIndex );
			//add story markup
			messageMarkup += self.getStoryMarkup( current_event );
			//add label
			messageMarkup += '<label message_text="true">' + linkify(escAttrib(content.text), {exclude: {email: true}}) + '</label>';
			//add pictures
			var picturesMarkup = '';
			$.each(content.media, function(_, media) {
				// #16481: Unificar criterios entre los media de un post y de una story
				switch(media.type) {
					case "IMAGE":
						picturesMarkup += FORMAT("<div class='instagramHistoryImageContainer'><div class='cropImage'><img class='imagePostedContainer' src='{0}'></div></div>", media.link || media.picture);
						break;
					case "VIDEO":
						picturesMarkup += self.__getVideoMarkup(media.link, "video/mp4");
						break;				
				}					
			});
			if (content.media && content.media.length > 1) {
				messageMarkup += '<div class="instagramCarousel">' + picturesMarkup + '</div>';
			}
			else {
				messageMarkup += picturesMarkup;
			}
			
			return messageMarkup;
		};

		var event_id = current_event.message.id;
		var date_time = new Date( current_event.creationDate.toString() );
		var author_name = this.m_interaction.getUserName( current_event );
		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI( current_event ) || genericAvatar;
		var read_unread_lnk = this.get_html_read_unread_link( current_event, firstEventIndex, false );
		var private_reply_lnk = this.get_html_private_reply_link( current_event, firstEventIndex, false );
		var like_unlike_lnk = this.get_html_like_unlike_link( current_event, firstEventIndex );
		var delete_from_client = this.get_html_deleted_message_from_client(current_event);
		var message_class = (current_event.message.webhookType === "REFERRAL" || current_event.message.webhookType === "OPTIN")? "referralMessage" : "";

		var class_container = "instagramThreadOrigin instagramEventType" +
			( this.m_interaction.isPrivateMessage() ? 'PrivateMessage' : 'Wall' ) +
			read_unread_lnk.parent_class +
			like_unlike_lnk.parent_class;

		if (current_event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (current_event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}
		if(this.isMessageDeleted(firstEventIndex)) {
			class_container += " deleted";
		}

		var $root_node = $( ''.concat(
			'<div class="', class_container, '" data-id="', event_id , '">',
				'<div class="instagramThreadInfoContainer">',
					'<div class="instagramThreadEventTypeIcon" style="display:none"></div>',
					'<div class="instagramThreadOriginAvatar">',
						'<object data="', avatar, '" type="image/png">',
							'<img src="', genericAvatar, '"/>',
						'</object>',
					'</div>',
					'<div class="instagramMainInfoContainer">',
						'<a href="javascript:void(0);" class="originAuthor">', author_name, '</a>',
						'<label event_date="true" class="originDate">', inConcert.Utils.userFormatDate(date_time, false, true), '</label>',
						'<div class="originMessage ', message_class, '">', getMessageMarkup(), '</div>',
					'</div>',
					'<div class="instagramRightInfoContainer">',
						'<div class="instagramRightTopInfoContainer">',
							//'<label event_date="true" class="originDate">', inConcert.Utils.userFormatDate(date_time, false, true), '</label>',
							/*
								SE OCULTA EL BOTON DE 3 PUNTITOS POR MEDIO DE CSS SI NO HAY NINGUNA OPCION EN EL DROPDOWN: 
									masterClientInstagram.css  -> .subEventActionsContainer:empty ~ .butInteractionGeneric
							*/
							'<div class="dropdown">',
								'<div class="originActionsContainer dropdown-menu">',
									delete_from_client,
									read_unread_lnk.html,
									private_reply_lnk.html,
								'</div>',
								'<button class="butInteractionGeneric butShowMoreActions" type="button" data-toggle="dropdown" aria-expanded="false">',
									//'<i class="fas fa-ellipsis-v"></i>',
								'</button>',
							'</div>',
						'</div>',
						'<div class="instagramRightBottomInfoContainer">',
							like_unlike_lnk.html,
						'</div>',
					'</div>',
				'</div>',
			'</div>')
		);

		var $thread_container = $container.find( ".instagramThreadCommentControlContainer" );
		$thread_container.before( $root_node );

		// this window
		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) ) {
			$('.originAuthor').unbind('click').bind('click', function() {
				let avatar_uri = inConcert.app.getBaseUrl() + self.m_interaction.getAvatarURI();
				var contact_window = new inConcert.ContactWindow({
					from_app    : self.from_app,
					interaction : self.m_interaction,
					name        : author_name,
					avatar      : avatar_uri,
					addresses   : current_event.addresses
				});
				contact_window.show();
			});
		}

		var events = this.reorder_nested_events();

		$thread_container.show();

		////
		var hiddenComments = Math.max( events.length - 1 - self.m_maxCompressedComments, 0 );
		var base = self.can_show_all_comments ? 0 : hiddenComments;

		// esta funcion carga el detalle en el header de POST/PM original.
		// la misma es invocada, desde la funcion 'append_messages' una vez que ha finalizado con la carga de todos los comentarios del POST/PM
		//
		var append_header_detail = function()
		{
			if ( !self.can_show_all_comments ) {
				if ( hiddenComments > 0 ) {
					var increment_count = -1; // aca se debe decrementar en uno ya que el post original, no se debe de contar
					var message = self.getMessageColapsedComments( increment_count );
					var $hidden_node = $(
						'<div class="instagramThreadSubEventSummary">' +
							'<a href="javascript:void(0);">' + message + '</a>' +
						'</div>'
					);
					$root_node.after( $hidden_node );
					$hidden_node.find('a')
						.unbind('click').click(function() {
							self.can_show_all_comments = true;
							self.openCurrentEvent( function( success )
							{
								if ( !success ) {
									return;
								}
								var scroll_data = self.getScrollBasicInfo();
								scroll_data.destination = "bottom";
								SCROLL_TO( scroll_data );
								self.hideStatusMessage();
							});
						});
				}
			}

			self.are_showing_all_messages = base === 0;
			self.loading_all_comments = false;

			var attach_html = self.getAttachMarkup( current_event );
			//if current message has an attachment, show disclaimer
			if ( attach_html.disclaimer ) {
				$(".instagramThreadOrigin").append($( attach_html.disclaimer ));
			}
			// if current message has an media
			if ( attach_html.media ) {
				$(".instagramThreadOrigin .originMessage").after($( attach_html.media ));
			}

			self.__processMediaPlayers($(".instagramThreadOrigin"));

			self.has_created_main_thread = true;
			self.m_currentEvent = 0;
			execute_callback( true );
		};

		// a la carga de los mensajes la agrego en un timeout, para que se pueda visualizar el mensaje de waiting.
		// ya que sino el mismo no se visualiza si tiene muchos mensajes y comienza a agregar en el DOM.

		// esta funcion muestra un mensaje informativo en el cual va inidicando la cantidad de mensajes que se van cargando.
		//
		var count_events = events.length;
		var show_loading_message = function( count_added )
		{
			var count_comments  = count_events - 1; // se le debe de restar uno, ya que dentro de los eventos tambien esta el POST original, y en el mensaje se de de informar los comments agregados
			count_added = count_added > count_comments ? count_comments : count_added;
			var message;
			if ( count_added === undefined ) {
				message = "<%=_T('Please wait, loading messages.')%>";
			}
			else {
				message = FORMAT( "<%=_T('Please wait, loading messages, {0} of {1}.')%>", count_added, count_comments );
			}
			self.displayStatusMessage( message, "Process", false );
		};

		// Esta funcion es una funcion recursiva y es la encargada de cargar los mensajes (eventos) en el markup
		// La misma se ejecuta dentro de un timeout minimo, y se va cargando de a unos 50 elementos, luego se invoca nuevamente hasta terminar la carga.
		// Se ejecuta dentro de un timeout, para que el navegador no quede colgado, en el loop y en la carga del DOM, si es que tiene muchos mensajes.
		// Tambien es ejecutada dentro del timeout, porque sino el mensaje de info de la funcion 'show_loading_message', no se visualiza en pantalla cuando comienza el loop.
		// Una vez que termina la carga de los mensajes, invoca a la funcion que carga el detalle en el POST/PM original.
		//
		var max_append_per_loop = 50;
		var pause_between_loop_ms = 10;
		var added = base + 1;

		var append_messages = function( count_added )
		{
			show_loading_message( count_added );

			// timeout que contiene la funcion que carga los mensajes.
			window.setTimeout( function() 
			{
				var added_in_loop = 0;

				for ( var i = added; i < count_events; i++ )
				{
					var current_children_event = events[i];

					if(current_children_event.creator === "system"){ //no mostrar mensajes de sistema en el agente (bienvenida, fuera de tiempo, finalizar)
						added++;
						continue;
					}
					
					self.insert_comment_in_dom(
						current_children_event,
						current_children_event.__index,
						function ($comment)
						{
							var parent_comment = current_children_event.message.parent_id;
							var postId = self.m_interaction.get_origin_message_id();
							if ( _.isString( parent_comment ) && parent_comment !== postId) {
								var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

								if ( $siblings.length === 0 ) {
									$container.find("[data-id='" + parent_comment +"']").after( $comment );
								}
								else {
									$siblings.last().after( $comment );
								}
							}
							else {
								$container.append( $comment );
							}
						}
					);

					added_in_loop++;
					added++;
					if ( added_in_loop === max_append_per_loop ) {
						break;
					}
				}

				if ( added === count_events ) {
					// FIN. muestro mensaje de que se ha completado la carga, cargo los mensajes en el DOM e invoco a la funcion que carga los datos del detalle.
					// el Append al DOM lo tengo que realizar en un timeout, porque sino el ultimo mensaje de info de que se ha cargado correctamente, no es visualizado.
					show_loading_message( added );
					window.setTimeout(function() {
						append_header_detail();
					}, pause_between_loop_ms);
				}
				else {
					// si aun quedan mensajes por cargar invoca la misma funcion recursiva
					append_messages( added );
				}

			}, pause_between_loop_ms);
		};

		// aca comienza la carga de los mensajes
		this.loading_all_comments = true;
		append_messages();
	};

	InstagramClientWindow.prototype.update_hash_events_return_diff = function () 
	{
		var self = this;
		var new_events = [];
		if ( !this.m_interaction || this.m_interaction.isBeingComposed() || !$.isArray( this.m_interaction.events ) ) {
			return new_events;
		}
		var events = this.reorder_nested_events();
		$.each( events || [], function( i, event )
		{
			if ( !event || !event.message || !event.message.id ) {
				return true; // continue
			}
			var evt_id = event.message.id.toUpperCase();
			if ( !self.hash_events[ evt_id ] ) {
				if ( i > 0 ) {
					event.is_new = true;
				}
				new_events.push( event );
				self.hash_events[ evt_id ] = true;
			}
		});
		return new_events;
	};

	InstagramClientWindow.prototype.append_new_events = function () 
	{
		var self = this;
		var new_events = this.update_hash_events_return_diff();
		if ( new_events.length === 0 ) {
			return;
		}

		var append = function()
		{
			$.each( new_events, function( _, new_event )
			{
				var sorted = false;
				if ( new_event.creator === "contact" ) {
					// Siempre que el nuevo evento sea generado x un contacto,
					//    lo tengo que agregar ordenado x fecha en el DOM.
					// Los eventos del agente no se ordenan, ya que los mismo ya fueron agregado en el "delivery sending" y actualizados en el "delivery success/failed"
					sorted = true;
				}
				if ( self.m_interaction.isPrivateMessage() ) {
					self.on_private_message_created( new_event, sorted );
				}
				else {
					self.on_comment_post_created( new_event, sorted );
				}
			});
		};

		if ( this.has_created_main_thread ) {
			append();
		}
		else {
			this.openCurrentEvent();
		}
	};

	InstagramClientWindow.prototype.clientUpdate = function () 
	{
		if (!this.m_interaction.isValid()) {
			return;
		}
		if ( this.was_being_composed && !this.m_interaction.isBeingComposed() && ( this.m_interaction.events || [] ).length > 0 ) {
			this.close();
			return;
		}

		$("#postCommentText").prop("disabled", false);
		$("#instagramComment").prop("disabled", false);
		$("#instagramWallText").prop("disabled", false);
		$("#instagramWallButtonPost").prop("disabled", false);

		this.append_new_events();
		this.set_mark_all_unread_read_button_caption();
		this.markLikeUnlikeEvents();
		this.markDeletedEvents();
	};

	InstagramClientWindow.prototype.change_client_state = function( new_state )
	{
		new_state = InstagramClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#instagram_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		return new_state;
	};
	
	
	InstagramClientWindow.prototype.firstLoadDraftMessage = function() {
		this.__loadDraftMessage = true;
	};
	
	InstagramClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		let self = this;
		window.setTimeout(function() {
			self.__setExternalMessage(message.text);
			callback(true);
		}, 1000);
	};

	InstagramClientWindow.prototype.__sendDraftTextMessage = function(_, callback) {
		if (this.m_interaction.properties.is_private_message) {
			this.replyByMailAction(callback);
		} else {
			this.comment_current_post(callback);
		}
	};

	InstagramClientWindow.prototype.__processMediaPlayers = function($message) {
		let self = this;
		$message.find(".instagramMedia").each(function() {
			self.__processMediaPlayer($message, $(this));
		});
	};
	
	InstagramClientWindow.prototype.__processMediaPlayer = function($message, $container) {
		let self = this;
		let player = null;
		let $play = $container.find(".instagramMediaPlay");
		let $pause = $container.find(".instagramMediaPause");
		let $stop = $container.find(".instagramMediaStop");
		let $download = $container.find(".instagramMediaDownload");
		let content = {
			fileType : $container.attr("file_type"),
			contentType : $container.attr("mime_type"),
			link : $download.attr("href")
		};
		$play.click(function() {
			if (!player) {
				let guid = inConcert.Utils.generateGuid();
				$message.find("[message_text]").after("".concat(
					"<div id='", guid, "' class='", content.fileType, "InstagramPlayer'>",
					"</div>"
				));
				$play.prop("disabled", true);
				player = new CInconcertPlayer({
					containerId : guid,
					url : content.link,
					fileFormat : inConcert.Utils.getFileExtensionFromMimeType(content.contentType),
					fileType : content.fileType,
					contentType : content.contentType,
					success : function() {
						player.play();
						if (content.fileType === "video") {
							//Ocultamos el contenedor de botones en lugar de ocultar el boton play, ya que el boton de download no se esta mostrando y en este punto quedaria una 'caja' vacia
							$container.hide();
						}
						else {
							$play.prop("disabled", false);
							$play.hide();
							$pause.show();
							$stop.show();
						}
					},
					error : function() {
						self.displayStatusMessage("<%=_T([[Error playing audio]])%>", "Error", false, 2000);
					}
				});
				player.onEnded(function() {
					if (content.fileType !== "video") {
						$play.show();
						$pause.hide();
						$stop.hide();
					}
				});
				player.load();
				return;
			}
			player.play();
			$play.hide();
			$pause.show();
			$stop.show();
		});
		if (content.fileType !== "video") {
			$stop.click(function() {
				player.stop();
				$play.show();
				$pause.hide();
				$stop.hide();
			});
			$pause.click(function() {
				player.pause();
				$play.show();
				$pause.hide();
			});
		}
	};

	InstagramClientWindow.prototype.hasAttachment = function() {
		return true;
	};

	InstagramClientWindow.prototype.getAttachmentOptions = function() {
		let options = InstagramClientWindow.uber.getAttachmentOptions.call(this);
		options.image.description = false;
		options.video.description = false;
		options.audio.description = false;
		options.file.description = false;
		options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
		options.video.type.extensions = [ "mp4", "3gpp" ];
		options.audio.type.extensions = [ "aac", "amr", "mp3", "ogg" ];
		options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx",
			"7z", "csv", "eml", "ics", "key", "log", "neon", "numbers", "odt", "pages", "pps", "ppsx", 
			"txt", "vcf", "xml", "yml", "yaml" ];
		//TODO: Por el momento no hay manera adecuada de enviar en un mensaje privado algo distinto a una imagen. Dejamos ocultas las opciones correspondientes
		if (!this.m_interaction.isPrivateMessage() || true) {
			options.video.disabled = true;
			options.audio.disabled = true;
			options.file.disabled = true;
		}
		return options;
	};

	InstagramClientWindow.prototype.hasDragAndDropFiles = function() {
		return true;
	};

	InstagramClientWindow.prototype.getDragAndDropControl = function() {
		return this.$emojiInput();
	};

	InstagramClientWindow.prototype.hasAudioRecord = function() {
		return false;
		//TODO: Volver a dejar este codigo cuando se pueda enviar audio en un DM de instagram
		//return this.m_interaction.isPrivateMessage();
	};

	InstagramClientWindow.prototype.onAudioRecordStarted = function() {
		this.$inputContainer().hide();
	};

	InstagramClientWindow.prototype.onAudioRecordStopped = function() {
		this.$inputContainer().show();
	};
	
	InstagramClientWindow.prototype.processAttachmentData = function(data) {
		data.commentId = this.m_reply_to_comment || this.m_interaction.interactionData.commentId || null;
		return data;
	};

	InstagramClientWindow.prototype.markLikeUnlikeEvents = function () 
	{
		var likes = this.m_interaction.interactionData.likes;
		var events = this.m_interaction.getEventList();
		if (events && events.length && likes) {
			for (var event = 0; event < events.length; event++) {
				this.markLikeUnlikeEvent(events[event]);
			}
		}
	};

	InstagramClientWindow.prototype.markLikeUnlikeEvent = function (event) {
		var messageId = event.message.id;
		var $message = $("[data-id='" + messageId + "']");
		if(this.isMessageLiked(event.__index)){
			$message.addClass("liked");
			if($message.find("[clientLikesThis]").length === 0) {
				$message.append( this.get_html_like_unlike_from_client(event) );
			}
		}
		else {
			$message.removeClass("liked");
			$message.find("[clientLikesThis]").remove();
		}
	};

	InstagramClientWindow.prototype.get_html_like_unlike_from_client = function (event) {
		if(event.creator === "agent" && this.isMessageLiked(event.__index)) {
			//return "<label clientLikesThis>❤️</label>";
			return "<div clientLikesThis class='heart-like-container'><i class='buttons-icons-heart'></i></div>";
		}
		return "";
	};

	InstagramClientWindow.prototype.markDeletedEvents = function () 
	{
		var deletedMessages = this.m_interaction.interactionData.deletedMessages;
		var events = this.m_interaction.getEventList();
		if (events && events.length && deletedMessages) {
			for (var event = 0; event < events.length; event++) {
				this.markDeletedEvent(events[event]);
			}
		}
	};

	InstagramClientWindow.prototype.markDeletedEvent = function (event) {
		var messageId = event.message.id;
		var $message = $("[data-id='" + messageId + "']");
		if(this.isMessageDeleted(event.__index)){
			$message.addClass("deleted");
			if($message.find("[clientDeletedThis]").length === 0) {
				$message.append( this.get_html_deleted_message_from_client(event) );
			}
			$message.find("[like_unlike]").remove();
		}
		else {
			$message.removeClass("deleted");
			$message.find("[clientDeletedThis]").remove();
		}
	};

	InstagramClientWindow.prototype.get_html_deleted_message_from_client = function (event) {
		if(event.creator === "contact" && this.isMessageDeleted(event.__index)) {
			return "<div clientDeletedThis class='deliveryMessageContainer deliveryDiscarded'><label class='deliveryMessageText'><%=_T('Deleted')%></label></div>";
		}
		return "";
	};

	InstagramClientWindow.prototype.isMessageLiked = function (index) {
		return this.m_interaction.events && this.m_interaction.events[index] && this.m_interaction.events[index].message.id && this.m_interaction.interactionData.likes && this.m_interaction.interactionData.likes[this.m_interaction.events[index].message.id];
	};

	InstagramClientWindow.prototype.isMessageDeleted = function (index) {
		return this.m_interaction.events && this.m_interaction.events[index] && this.m_interaction.events[index].message.id && this.m_interaction.interactionData.deletedMessages && this.m_interaction.interactionData.deletedMessages[this.m_interaction.events[index].message.id];
	};

	InstagramClientWindow.prototype.private_reply = function(index, message, callback = $.noop)
	{
		var self = this;
		this.displayStatusMessage("<%=_T([[Sending private message...]])%>", "Process", true);
		this.m_interaction.privateReply(index, message, function( success, message, error ) {
			self.hideStatusMessage();
			if (success) {
				callback(true);
			}
			else {
				self.displayStatusMessage( message, "Error", true, 5000 );
				callback(false, message || "error sending message");
			}
			if (success || error && error.result === "ALREADY_REPLIED") {
				$(FORMAT("[private_reply_link][data-event_index={0}]", index)).remove();
				$("#cancelReplyToComment").click();
				self.clearInput($("#postCommentText"));
			}
		});
	};	

	InstagramClientWindow.prototype.getStoryMarkup = function( event )
	{
		var self = this;
		// retorna el markup correspondiente a la historia cuando el evento es una respuesta a una historia.

		var response = "";

		if(!(event && event.message && event.message.reply_to && event.message.reply_to.story && event.message.reply_to.story.url && event.message.reply_to.story.type)) {
			return response;
		}

		response += "<div class='instagramHistoryContainer'><label message_text='true'><%=_T('Answer to the Story:')%></label></div>";
		switch(event.message.reply_to.story.type) {
			case "image":
				response += FORMAT("<div class='instagramHistoryImageContainer'><div class='cropImage'><img class='imagePostedContainer' src='{0}'></div></div>", event.message.reply_to.story.url);
				break;
			case "video":
				response += self.__getVideoMarkup(event.message.reply_to.story.url, "video/mp4");
				break;
		}

		return response;
	};	
	
	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'buttonColorGray emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function()
		{
			var client = new InstagramClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};


		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
