/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name InternalChatInteraction
 */
(function () {

	"use strict";

	/**
	 * Class ChatInteraction.
	 * Subclass of Interaction
	 *
	 * @param {Object} rawInteraction
	 */
	function ChatInteraction (rawInteraction)
	{
		ChatInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
	}

	inConcert.Utils.inherit(ChatInteraction, inConcert.interactions.base);

	if (typeof inConcert.interactions.chat !== "function") {
		inConcert.interactions.chat = ChatInteraction;
	}

	//sends a message on the current conversation
	ChatInteraction.prototype.sendMessage = function(/*message*/) {};

	//called by the chat client when a message is received for this conversation
	ChatInteraction.prototype.onMessageReceived = function(/*user, message*/) {};

	ChatInteraction.prototype.isLive = function() {
		return true;
	};

	ChatInteraction.prototype.getExtendedInfo = function( max_length ) {
		let internal_consultation_flag = this.interactionData && this.interactionData.internal_consultation_flag && this.state === "CREATED";
		if (internal_consultation_flag) {
			return inConcert.Utils.truncateWord( this.interactionData.internal_consultation_question, max_length );
		} else {
			var extended_info = this.interactionData.firstMessage ? inConcert.Utils.html2text(this.interactionData.firstMessage): "";
			return inConcert.Utils.truncateWord( extended_info, max_length );
		}
	};

	ChatInteraction.prototype.getMessageTime = function(msg) {
		if (!msg || !msg.msgtime) {
			return "";
		}
		var parsedValue = parseInt(msg.msgtime, 10) * 1000;
		if(isNaN(parsedValue)) {
			return "";
		}
		var msgTime = new Date(parsedValue);
		var zeropad = inConcert.Utils.zeropad;

		return zeropad(msgTime.getHours()) + ":" + zeropad(msgTime.getMinutes()) + ":" + zeropad(msgTime.getSeconds());
	};

	//implemented only in webchat
	ChatInteraction.prototype.getTransferrerIds = function ()
	{
		return [];
	};

	ChatInteraction.prototype.canHangup = function() {
		return this.state === "TAKEN";
	};

	ChatInteraction.prototype.hangup = function(callback = $.noop) {
		if (!this.canHangup()) {
			return callback(false, "can not hangup");
		}
		let session = this.getSession();
		if (!session || !session.close()) {
			return callback(false, "conversation is closed");
		}
		//this code is for showing the spinner
		this.pending = true;
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode(this);
		callback(true);
	};
	
	ChatInteraction.prototype.notifyNewMessage = function(message) {
		this.__fireNotification("NewChatMessage", message.originalMessage);
	};

	ChatInteraction.prototype.hasAttachments = function () {
		if (this.interactionData && this.interactionData.repositoryAttachments && Object.keys(this.interactionData.repositoryAttachments).length > 0) {
			return true;
		}

		return false;
	};

}());
