/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name IMInteraction
 */
(function () {

	"use strict";

	/**
	Class IMInteraction.
	Subclass of ChatInteraction
	*/
	function IMInteraction (rawInteraction)
	{
		IMInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
	}

	inConcert.Utils.inherit(IMInteraction, inConcert.interactions.chat);

	//creates a conversation request for a pending conversation
	IMInteraction.prototype.createConversation = function(address, callback) {
		var self = this;
		if (self.state !== "PENDING") {
			throw("Invalid conversation state for creating conversation");
		}
		inConcert.logger.action({action_name : "start_im_conversation", id : this.id, type : this.type,
			campaign : self.campaign, protocol : address.protocol, buddy : address.buddy, action_type : "conversation"});

		var url = this.m_manager.getBaseUrl() + "/interactions/im/create_outgoing/";
		var request = {
			id : self.id,
			parentInteractionId : this.parentInteractionId,
			address : {
				protocol : address.protocol,
				buddy : address.buddy
			},
			campaign : self.campaign,
			contact : self.contact
		};
		inConcert.invoke({
			type: "POST",
			url: url,
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			success: function () {
				//200 OK
				if (typeof(callback) === "function") {
					callback(true, "OK");
				}
			},
			// "error" contains a description of the error and a reason,
			// could be shown to the user
			error: function(_, error) {
				inConcert.logger.action({action_name : "start_im_conversation", error_message : error.result });
				if (typeof(callback) !== "function") {
					return;
				}
				var message;
				switch (error.result) {
					case "CHATSERVER_NOT_AVAILABLE":
						message = "<%=_T('Chatserver not available')%>";
						break;

					case "INTERACTION_ALREADY_EXISTS":
						message = "<%=_T('There is an active conversation with this buddy')%>";
						break;

					case "AGENT_UNAVAILABLE":
						message = "<%=_T('Agent is currently unavailable')%>";
						break;

					case "FAIL_CREATING_OUTGOING":
						message = "<%=_T('There was an error while creating the conversation, please try again later')%>";
						break;

					case "FORBIDDEN":
						message = "<%=_T('You not have permission for create outgoing IM conversation.')%>";
						break;

					case "NOT_LICENSED":
						message = "<%=_T('Unable to create outgoing IM conversation. Not has licensed for the IM channel.')%>";
						break;

					//Hay mas casos?
					default:
						message = "<%=_T('Failed to start chat, please try again in a few seconds...')%>";
				}
				callback(false, message); //not successful
			}
		});
	};

	IMInteraction.prototype.getAddress = function() {
		var protocol = this.interactionData.protocol || "";
		return protocol + "|" + this.contactAddress;
	};

	IMInteraction.prototype.getDisplayAddress = function() {
		return this.contactAddress;
	};

	IMInteraction.prototype.compatibleAccount = function(account) {
		if (!account || typeof(this.completeAddress) !== "object") {
			return true;
		}
		return account.protocol === this.completeAddress.protocol;
	};

	/**
	* IM conversations don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	IMInteraction.prototype.getEventList = function()
	{
		return null;
	};

	IMInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	IMInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	IMInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};

	if (typeof inConcert.interactions.im !== "function") {
		inConcert.interactions.im = IMInteraction;
	}
}());
