
/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name InternalCallInteraction
 */
(function () {

	"use strict";
	
	/**
	 * Class InternalCallInteraction.
	 * Subclass of TelephonyInteraction
	 *
	 * @param {Object} rawInteraction
	 */
	function InternalCallInteraction (rawInteraction)
	{
		InternalCallInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
		this.properties = this.properties || {};
		this.properties.event_count = 0;
	}

	inConcert.Utils.inherit(InternalCallInteraction, inConcert.interactions.telephony);

	if (typeof inConcert.interactions.internal_call !== "function") {
		inConcert.interactions.internal_call = InternalCallInteraction;
	}

	InternalCallInteraction.prototype.canHangup = function() {
		return true;
	};

	InternalCallInteraction.prototype.canInflateInteraction = function ()
	{
		return true;
	};

	/**
	* Internal chats don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	InternalCallInteraction.prototype.getEventList = function()
	{
		return null;
	};

	InternalCallInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	InternalCallInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	InternalCallInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};

	InternalCallInteraction.prototype.isInternal = function() {
		return true;
	};

	InternalCallInteraction.prototype.isLocal = function() {
		return true;
	};	

	InternalCallInteraction.prototype.hasPermission = function(action, app) {
		if(app === "interaction") {
			return ["FINISH", "HANGUP", "LISTEN_RECORD", "DOWNLOAD_RECORD"].includes(action);
		}
		else if(app === "supervisor") {
			return ["LISTEN_RECORD", "DOWNLOAD_RECORD", "SPY_CALL_LISTEN"].includes(action);
		}
	};

	InternalCallInteraction.prototype.getDisplayAddress = function()
	{
		return this.contactAddress || "";
	};

	InternalCallInteraction.prototype.getAvatarURI = function()
	{
		let loggedUserId = inConcert.app.getLoggedUserId();
		let teamMember = loggedUserId === this.invitedAgent ? this.inviterAgent : this.invitedAgent;
		return "/users/get_avatar/" + teamMember;
	};

	InternalCallInteraction.prototype.getClientName = function()
	{
		let loggedUserId = inConcert.app.getLoggedUserId();
		return loggedUserId === this.invitedAgent ? this.contactName : this.teamMemberName;
	};

	InternalCallInteraction.prototype.getDisplayName = function()
	{
		return this.contactName || "";
	};
}());



