
/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name InternalChatInteraction
 */
(function () {

	"use strict";
	
	/**
	 * Class InternalChatInteraction.
	 * Subclass of ChatInteraction
	 *
	 * @param {Object} rawInteraction
	 */
	function InternalChatInteraction (rawInteraction)
	{
		InternalChatInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
		this.properties = { event_count : 0 };
	}

	inConcert.Utils.inherit(InternalChatInteraction, inConcert.interactions.chat);

	if (typeof inConcert.interactions.internal_chat !== "function") {
		inConcert.interactions.internal_chat = InternalChatInteraction;
	}

	InternalChatInteraction.prototype.canHangup = function() {
		return true;
	};

	/**
	* Internal chats don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	InternalChatInteraction.prototype.getEventList = function()
	{
		return null;
	};

	InternalChatInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	InternalChatInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	InternalChatInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};

	InternalChatInteraction.prototype.isInternal = function() {
		return true;
	};

	InternalChatInteraction.prototype.isMonitorable = function()
	{
		return false;
	};

	InternalChatInteraction.prototype.isTaken = function()
	{
		return true;
	};

	InternalChatInteraction.prototype.belongsToMe = function()
	{
		return true;
	};

	InternalChatInteraction.prototype.getClientName = function()
	{
		if (this.interactionData) {
			if (this.interactionData.contactName) {
				return this.interactionData.contactName;
			}
			if (this.interactionData.data && this.interactionData.data.display_name) {
				return this.interactionData.data.display_name;
			}
		}
		if (this.contactObject && this.contactObject.Name) {
			return this.contactObject.Name;
		}
		return "";
	};
}());



