/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 11 */

/* global $, inConcert, console*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class InteractionPanel.
		Panel subclass.
	*/

	'use strict';
	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	function InteractionPanel ( params )
	{
		InteractionPanel.uber.constructor.call(this, params.id, params.name);
		this.m_cached_bulk_actions_HTML = null;
		this.m_filteredLabels = undefined;
		var $container = this.m_handler.find('.boxContentScrollArea');
		this.m_interaction_gui = new inConcert.interaction_gui( inConcert.attention_actions,
			{
				from_app   : 'interaction',
				$container : $container,
				owner      : this,
				callbacks  : {
					get_timestamp_type: function() {
						if (params.callback && params.callback.get_timestamp_type) {
							return params.callback.get_timestamp_type();
						}
						return; // interaction gui va a tomar el por default.
					},
					can_open_client_window: function( _, callback ) {
						/* Por el momento aca no se realiza ninguna validacion si la interaccion se puede abrir o no.*/
						callback = callback || $.noop;
						callback(true);
					}
				}
			}
		);
		this.m_listContainer = this.m_handler.find('.boxContentScrollArea');
		this.m_interactions = {}; //all the interactions currently belonging in this panel
		this.facets_by_name = null;
		this.mapped_accounts_by_desc = {};

		this.proxies_desc = {
			CALL       : "<%=_T('Phone')%>",
			CHAT       : "<%=_T('Web Chat')%>",
			IM         : "<%=_T('IM')%>",
			FACEBOOK   : "<%=_T('Facebook')%>",
			INSTAGRAM  : "<%=_T('Instagram')%>",
			YOUTUBE    : "<%=_T('Youtube')%>",
			APPSTORE   : "<%=_T('Appstore')%>",
			GOOGLEPLAY : "<%=_T('Googleplay')%>",
			RCS	       : "<%=_T('Rcs')%>",
			MAIL       : "<%=_T('E-mail')%>",
			SMS        : "<%=_T('SMS')%>",
			WHATSAPP   : "<%=_T('Whatsapp')%>",
			TWITTER    : "<%=_T('Twitter')%>",
			WEBCONTACT : "<%=_T('Contact Form')%>",
			VOICEMAIL  : "<%=_T('Voice Mail')%>"
		};
	}

	inConcert.Utils.inherit(InteractionPanel, inConcert.Panel);

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.InteractionPanel !== 'function') {
		inConcert.InteractionPanel = InteractionPanel;
	}

	InteractionPanel.prototype.initialize = function( callback ) { this.loadAllInteractions(callback); };

	InteractionPanel.prototype.interactionBelongsToPanel = function(/*interaction*/) { return false; };

	InteractionPanel.prototype.show = function( callback ) { this.on_show( callback ); };

	InteractionPanel.prototype.on_show = function( callback ) {
		callback = callback || $.noop;
		callback();
	};

	InteractionPanel.prototype.hide = function( callback ) { this.on_hide( callback ); };
	InteractionPanel.prototype.on_hide = function( callback ) {
		callback = callback || $.noop;
		callback();
	};

	InteractionPanel.prototype.startingSearch = $.noop;

	InteractionPanel.prototype.endSearch = $.noop;

	InteractionPanel.prototype.set_events = function()
	{
		InteractionPanel.uber.set_events.call( this );
		const self = this;
		const $select_all = this.$getChkSelectAllConversations();
		if ($select_all) {
			// select/unselect all interactions button
			$select_all.unbind('click')
				.bind('click', function() {
					self.selectAllInteractions();
					self.updateBatchButtonsStatus();
				});
		}
	};

	//Loads all interactions
	InteractionPanel.prototype.loadAllInteractions = function(callback)
	{
		var self = this;
		callback = callback || $.noop;

		this.startingSearch();
		var interactions = inConcert.app.getInteractionManager().getInteractions();

		inConcert.Utils.foreach_timeout({
			rows : interactions,
			pause_between_loop_ms: 30,
			append_fn : function( interaction ) {
				if (self.interactionBelongsToPanel(interaction)) {
					self.addInteraction(interaction);
				}
			},
			complete_fn : function() {
				callback();
				self.endSearch();
			}
		});
	};

	//Selects as active an interaction from the panel by its id
	InteractionPanel.prototype.activateInteraction = $.noop;

	InteractionPanel.prototype.$findInteraction = function( interaction )
	{
		if ( !interaction ) {
			return;
		}
		var selector = '[original_int_id="' + interaction.id + '"]';
		var $interaction = this.m_listContainer.find( selector );
		return $interaction;
	};

	InteractionPanel.prototype.calculateInteractionClass = function(interaction, $node) {
		this.m_interaction_gui.calculate_interaction_class(interaction, $node);
	};

	InteractionPanel.prototype.get_prefix = function( /*interaction*/ ) {
		throw 'InteractionPanel.prototype.get_prefix. It must be implemented in derived class.';
	};
 
	InteractionPanel.prototype.allowed_bulk_actions = function(/*interaction*/) {
		return inConcert.permissions.to_action_on_any_campaign('interaction', 'int_bulk_actions');
	};

	InteractionPanel.prototype.$build_interaction_node = function(interaction, config)
	{
		config = config || {};
		let parmams = {
			prefix: this.get_prefix(interaction),
			allowed_bulk_actions: this.allowed_bulk_actions(interaction),
			columns: config.columns
		};
		const template_tpl = null;
		const $node = this.m_interaction_gui.$build_interaction_node(interaction, template_tpl, parmams);
		return $node;
	};

	InteractionPanel.prototype.updateLabelsAndDisposition = function(interaction, $node) {
		this.m_interaction_gui.update_labels_and_disposition(interaction, $node);
	};

	//Refresh node data for an interaction
	InteractionPanel.prototype.updateInteractionData = function(interaction, $node)
	{
		this.m_interaction_gui.update_interaction_data(interaction, $node);
	};

	//Retrieves interaction html node given an interaction object
	InteractionPanel.prototype.getInteractionNode = function( interaction )
	{
		if ( this.m_interactions[ interaction.id ] ) {
		// las que no son archive
			return this.m_interactions[ interaction.id ];
		}
		// las archive
		var $nodes = this.m_handler.find( '.boxContentScrollArea' ).find( '[original_int_id]' );
		for ( var i = 0; i < $nodes.length; i++ ) {
			var $node = $( $nodes[ i ] );
			var current = $node.data( 'interaction' );
			if ( current && current.id === interaction.id ) {
				return $node;
			}
		}
	};

	//Refresh interactions timers or other time-changing-oriented data
	InteractionPanel.prototype.updateInteractionTime = function(interaction, $node)
	{
		this.m_interaction_gui.update_interaction_time(interaction, $node);
	};

	//Refresh interactions timers or other time-changing-oriented data
	InteractionPanel.prototype.refresh = function() {
		var panel = this;
		this.m_handler.find('.boxContentScrollArea').children().each(function() {
			var interactionNode = $(this);
			//a node should always have an interaction associated, but if html mangling has taken place, consider it
			var interaction = interactionNode.data('interaction');
			if (interaction) {
				panel.updateInteractionTime(interaction, interactionNode);
			}
		});
	};

	InteractionPanel.prototype.isInSomeGroupInteractions = function( interaction )
	{
		if ( !interaction ) {
			return false;
		}
		var $interaction = this.m_listContainer.find( '[original_int_id="' + interaction.id + '"]' );
		if ( $interaction.length === 0 ) {
			return false;
		}
		var belongs_to_group = $interaction.attr( 'belongs_to_group' );
		if ( !belongs_to_group ) {
			return false;
		}
		return true;
	};

	// implementada en las clases derivadas
	InteractionPanel.prototype.attachInteractionToGroup = function( /*groupId, interaction*/ ) {};

	InteractionPanel.prototype.detachInteractionOfGroup = function( $interaction, $group )
	{
		if ( !$interaction || $interaction.length === 0 ) {
			return;
		}
		$interaction.detach();

		var int_id = $interaction.attr( 'original_int_id' );
		var interaction = inConcert.app.getInteractionManager().findInteraction( int_id );

		if ( $group ) {
		// lo inserto antes del grupo para que quede en la misma posicion
			$group.before( $interaction );
			return;
		}
		else {
			$group = this.$getGroupInteractions( interaction );
			if ( $group && $group.length > 0 ) {
			// lo inserto antes del grupo para que quede en la misma posicion
				$group.before( $interaction );
				return;
			}
		}
		// si no existe un grupo lo inserto directo en panel ( aca no deberia llegar nunca, pero por las dudas para que la interaccion no se pierda)
		this.m_listContainer.append( $interaction );
	};

	InteractionPanel.prototype.removeGroupInteractions = function( interaction )
	{
		var $group = this.$getGroupInteractions( interaction );
		if ( !$group || $group.length === 0 ) {
		// si el grupo no existe en el dom, me voy
			return;
		}
		var children = $group.children();
		if ( !children || children.length === 0 ) {
			// si no quedan interacciones en el grupo, lo elimino del panel
			$group.remove();
			return;
		}

		if ( children.length === 1 ) {
		// si queda una sola interaccion en el grupo:
		//   1 la saco del grupo,
		//   2 la inserto directo en el panel
		//   3 elimino el grupo del panel
			var $last_interaction = $( children[ 0 ] );
			this.detachInteractionOfGroup( $last_interaction, $group );
			$group.remove();
			return;
		}
	};

	InteractionPanel.prototype.sortInteractionOnGroup = function( fi /* first interaction */, si /* second interaction */ )
	{
		if ( fi.type === si.type ) {
		// a igual tipo de interacciones, comparo por fecha
			return this.sortingFunction(fi, si);
		}
		// cuanto mayor peso tenga el tipo de interaccion, se ubica abajo.
		// los webcontact, van debajo de todo
		var interactions_weight = {
			"WEBCONTACT": 100
			// "MAIL": 50   // sample
		};
		var fi_weight = interactions_weight[ fi.type ] || 0;
		var si_weight = interactions_weight[ si.type ] || 0;

		if ( fi_weight === si_weight ) {
		// a igual peso entre diferentes tipos de interacciones, comparo por fecha
			return this.sortingFunction(fi, si);
		}
		// aca ordeno segun el peso del tipo de interaccion
		return fi_weight > si_weight;
	};

	InteractionPanel.prototype.$getGroupInteractions = function( interaction )
	{
		if ( !interaction || !interaction.getGroupId ) {
			return false;
		}
		var group_interaction_id = interaction.getGroupId();
		return this.m_listContainer.find( '[group_interaction_id="' + group_interaction_id + '"]' );
	};

	InteractionPanel.prototype.insertInteractionInGroup = function( interaction, $new_interaction )
	{
		var self = this;
		var belongsToSomeGroup = interaction.belongsToSomeGroup();
		if ( !belongsToSomeGroup ) {
			return false;
		}

		var $group = this.$getGroupInteractions( interaction );

		if ( !$group ) {
			return false;
		}

		var response = { new_group: false, $group: false, sort_group: false };
		var group_interaction_id = interaction.getGroupId();
		var $interactions_group = this.m_listContainer.find( '[belongs_to_group="' + group_interaction_id + '"]' );
		if ( $interactions_group.length === 0 ) {
		// si aun no existe ninguna interaccion en el grupo.
		// inserto la primer interaccion del grupo directo en el panel
			return false;
		}
		else if ( $interactions_group.length === 1 ) {
		// si ya existe una interaccion en el panel que pertenece al grupo:
		// creo el grupo y lo inserto al panel
		//	quito la interaccion existente en el panel y la agrego al grupo.
		// despues procedo a agrega al grupo la segunda interaccion (la en curso)
			group_interaction_id = interaction.getGroupId();
			$group = $(''.concat(
				'<div group_interaction_id="', group_interaction_id, '" class="asociatedInteractionsGroup"></div>'
			));
			$group.data( 'interaction', interaction ); // data from the first interaction are used to sort the group node list
			response.new_group = true;

			$.each( $interactions_group, function( undefined, dom_it ) {
				var $it = $( dom_it );
				$it.detach();
				$it.attr( 'belongs_to_group', group_interaction_id );
				$group.append( $it );
			});
		}

		var inserted = false;
		var group_interactions = $group.children();
		$.each( group_interactions, function( i, group_interaction )
		{
			var $group_interaction = $( group_interaction );
			var current_interaction = $group_interaction.data('interaction');
			if ( self.sortInteractionOnGroup( current_interaction, interaction )) {
				$group_interaction.before( $new_interaction );
				if ( i === 0 ) {
					$group.data( 'interaction', interaction ); // data from the first interaction are used to sort the group node list
					response.sort_group = true;
				}
				inserted = true;
				return false; // break
			}
		});

		if ( !inserted ) {
			$group.append( $new_interaction );
		}

		response.$group = $group;
		return response;
	};

	// This function inserts the interaction or group interactions in the list panel (queued, archived, assigned)
	InteractionPanel.prototype.insertSortedInteractionInPanelList = function( interaction, $new_interaction, insert_in_list_fn )
	{
		var group_response = this.insertInteractionInGroup( interaction, $new_interaction );
		if ( !group_response ) {
		// insert a simple new interaction in list panel
			insert_in_list_fn( $new_interaction );
		}
		else if ( group_response.new_group ) {
		// insert a new group interaction in list panel
			insert_in_list_fn( group_response.$group );
		}
		// else if ( group_response.sort_group ) {
		// // TODO: reordenar grupo en la lista, con los datos de la primer interaccion.
		// }
	};

	InteractionPanel.prototype.sortingFunction = function( first_interaction, second_interaction )
	{
		var fi_time = first_interaction.currentTimeRaw();
		var si_time = second_interaction.currentTimeRaw();
		if (first_interaction.isAppointment() && second_interaction.isAppointment()) {
		return fi_time > si_time;
		}
		return fi_time < si_time;
	};

	InteractionPanel.prototype.sortedInsertion = function( interaction, $new_interaction )
	{
		var self = this;

		if ( !this.interactionPassesFilter( interaction ) ) {
			return false;
		}

		var insert_in_list_fn = function( $new_node )
		{
			var inserted = false;
			var children = self.m_listContainer.children();
			var currentSize = children.length;
			for (var i = 0; i < currentSize; i++) {
				var it = $(children[i]).data('interaction');
				if ( self.sortingFunction(it, interaction) ) {
					$(children[i]).before( $new_node );
					inserted = true;
					break;
				}
			}

			if (!inserted) {
				self.m_listContainer.append( $new_node );
			}
		};

		// inserts the interaction or group interactions in the list panel
		this.insertSortedInteractionInPanelList( interaction, $new_interaction, insert_in_list_fn );

		if ( this.onNodeInserted ) {
			this.onNodeInserted( $new_interaction, interaction );
			var $bulk_actions_container = $('#' + this.get_batch_actions_prefix() +'container_selected_finish');
			this.show_or_hide_bulk_actions_container(false,$bulk_actions_container);
		}
		// agrego la row al DOM
		return true;
	};

	InteractionPanel.prototype.onInteractionUpdated = function(interaction, callback = $.noop)
	{
		if (!interaction || !interaction.id || !this.m_windows) {
			return;
		}
		var currentWindow = this.m_windows[interaction.id];
		var view;
		if (currentWindow) {
			view = currentWindow.getWindow().view;
		}
		else if (interaction.isTelephonyInteraction()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient('CALL');
			if (phone && phone.attendingInteraction(interaction)) {
				view = phone.getView();
			}
		}

		if (!view || typeof(view.onInteractionUpdated) === 'undefined') {
			return;
		}
		if(typeof(view.onInteractionUpdated) !== 'function') {
			inConcert.logger.warn({
				method:         'InteractionPanel.onInteractionUpdated',
				interaction_id: interaction.id,
				message:        'onInteractionUpdated is not a function. Is a ' + typeof(view.onInteractionUpdated)
			});
		}
		view.onInteractionUpdated(interaction, callback);
	};

	InteractionPanel.prototype.notify_desktop_new_interactions = $.noop;
	InteractionPanel.prototype.notify_desktop_new_events_interactions = $.noop;
	InteractionPanel.prototype.remove_interaction_desktop_notifications = $.noop;

	//Update a list of interactions
	//Returns how many interaction where added to the panel
	InteractionPanel.prototype.updateInteractions = function(interactions)
	{
		if (interactions.length === 0) {
			return 0;
		}

		var new_incomming_interactions = [];

		for (var i = 0; i < interactions.length; i++) {
			var interaction = interactions[i];
			var $node = this.m_interactions[interaction.id];
			if (!interaction.deleted && this.interactionBelongsToPanel(interaction)) {
				if ($node) {
					this.updateInteractionData(interaction, $node);
				}
				else if (interaction.isBotInternalConsultation()) {
					if (!this.m_interactions[interaction.id]) {
						var $new_interaction = this.$build_interaction_node(interaction);
						if ( $new_interaction.length > 0 ) {
							this.updateInteractionData(interaction, $new_interaction);
							this.m_interactions[interaction.id] = $new_interaction;
						}	
					}
				}
				else {
					var is_added = this.addInteraction(interaction);
					if (is_added) {
						if (interaction.initiative !== 'OUTBOUND' || 
							$.inArray(interaction.state, ['TRANSFER LOCKED', 'CONFERENCE LOCKED']) >= 0
						) {
							new_incomming_interactions.push(interaction);
						}
					}
				}
			}
			else {
				this.removeInteraction(interaction);
			}
		}

		this.updateBatchActions();

		return {
			new_incomming_interactions: new_incomming_interactions
		};
	};

	//Adds an interaction to the panel
	InteractionPanel.prototype.addInteraction = function( interaction )
	{
		if (this.m_interactions[interaction.id]) {
			return false;
		}
		var $new_interaction = this.$build_interaction_node(interaction);
		if ( $new_interaction.length === 0 ) {
			return false;
		}
		interaction.expanded = true;
		inConcert.app.getMainWindow().updateUnreadCounters();
		//set data
		this.updateInteractionData(interaction, $new_interaction);
		this.m_interactions[interaction.id] = $new_interaction;
		this.onInteractionAdded(interaction, $new_interaction);
		return true;
	};

	InteractionPanel.prototype.removeClientWindow = function(interaction)
	{
		if (!this.m_windows[interaction.id]) {
			return;
		}
		// TODO: en this.m_windows tengo que tener cosas que me permitan cerrar de una
		// No asumir que es algo que tiene una property 'view' y 'view.onInteractionGone'
		var currentWindow = this.m_windows[interaction.id].getWindow();
		if (currentWindow && currentWindow.view && currentWindow.view.onInteractionGone) {
			currentWindow.view.onInteractionGone(interaction);
		}
	};

	InteractionPanel.prototype.onInteractionRemoved = function(interaction)
	{
		if (!interaction) {
			return;
		}
		this.disposeMenuActions();
		this.removeGroupInteractions(interaction);
		this.removeClientWindow(interaction);
	};

	InteractionPanel.prototype.onInteractionAdded = $.noop;

	InteractionPanel.prototype.hideMenuActions = function()
	{
		if (this.m_interaction_gui) {
			this.m_interaction_gui.hide_menu_actions();
		}
	};

	InteractionPanel.prototype.disposeMenuActions = function()
	{
		if (this.m_interaction_gui) {
			this.m_interaction_gui.dispose_menu_actions();
		}
	};

	InteractionPanel.prototype.removeInteraction = function( interaction )
	{
		if ( !interaction ) {
			return;
		}
		var $interaction = this.$findInteraction( interaction );
		if ( $interaction.length > 0 ) {
			$interaction.remove();
		}
		if ( this.m_interactions.hasOwnProperty( interaction.id ) ) {
			delete this.m_interactions[ interaction.id ];
		}
		this.onInteractionRemoved( interaction );
	};

	InteractionPanel.prototype.closeAllClients = function()
	{
		for (var interactionID in this.m_windows) {
			if (this.m_windows.hasOwnProperty(interactionID)) {
				var sessionWindow = this.m_windows[interactionID];
				if (!sessionWindow || !sessionWindow.getWindow()) {
					return;
				}
				var clientWindow = sessionWindow.getWindow();
				// this check is added because every time you reload
				// the browser tab, an exception is thrown
				if (clientWindow && clientWindow.close){
					clientWindow.close();
				}
			}
		}
	};

	//Overrides the Panel's method
	InteractionPanel.prototype.openInteractionClient = function(interaction, node, options)
	{
		// Si la interaccion (webcontact), tiene una interaccion que lo esta resolviendo (pending)
		//   no abre el cliente de webcontact nuevamente
		options = options || {};
		const on_window_open = options.on_window_open || $.noop;
		if ( typeof interaction.hasRelationshipWithPendingInteraction === "function" &&
			interaction.hasRelationshipWithPendingInteraction()
			)
		{
			on_window_open(false, interaction);
			return false;
		}
		const hasOpened = InteractionPanel.uber.openInteractionClient.call(this, interaction, node, options); //invokes the base class method
		if (!hasOpened) {
			on_window_open(false, interaction);
			return false;
		}
		//update node
		if (!node) {
			node = this.m_interactions[interaction.id];
		}
		//refresh interaction state on the taskbar
		this.calculateInteractionClass(interaction, node);
		//on_window_open(true, interaction);
		return true;
	};

	InteractionPanel.prototype.updateInteractionNode = function(interaction) {
		var node = this.m_interactions[interaction.id];
		this.calculateInteractionClass(interaction, node);
	};

	//Overrides the panel's method
	InteractionPanel.prototype.onLocalInteractionUpdate = function(interaction, refreshAvatar)
	{
		InteractionPanel.uber.onLocalInteractionUpdate.call(this, interaction, refreshAvatar); //invokes the base class method

		var $node = this.getInteractionNode(interaction);
		if ( $node && $node.length > 0) {
			//an interaction can be triggering the onread event but not being on a panel
			this.updateInteractionData(interaction, $node);
			if (refreshAvatar) {
				var avatarURI = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
				$node.find('[int_avatar] > img').attr('src', avatarURI);
			}
		}
	};

	//triggered from each client when the panel is closed
	InteractionPanel.prototype.onClientClosed = function(interaction)
	{
		console.debug("[InteractionPanel.prototype.onClientClosed] --> interaction: " + (interaction || {}).id);
		InteractionPanel.uber.onClientClosed.call(this, interaction); //invokes the base class method
		if(interaction) {
			var node = this.getInteractionNode(interaction);
			this.calculateInteractionClass(interaction, node);
		}
	};

	InteractionPanel.prototype.$getChkSelectAllConversations = function()
	{
		return;
	};

	/**
	 * If there is no interaction checked or at least one interaction is checked,
	 * I need to check all if all interactions are checked, I do the reverse.
	 *
	 * @todo This name is kinda wrong, if it does the reverse too, should be called "toggle" or something
	 * @private
	 */
	InteractionPanel.prototype.selectAllInteractions = function()
	{
		const $select_all = this.$getChkSelectAllConversations();
		if ($select_all) {
			const mark_all = $select_all.prop('checked');
			if ( this.m_interaction_gui.batch_select_all_interactions( mark_all ) ) {
				this.setBatchButtonsEvents();
			}
			this.updateBatchActions();
		}
	};

	/**
	 *Get selected nodes from the interactions panel.
	 *
	 * @private
	 * @return {Object} result    - The result with the totals.
	 * @return {Array} result.all - All nodes selected.
	 * @return {Array} result.interactions - All interactions selected
	 */
	InteractionPanel.prototype.getSelectedInteractions = function()
	{
		return this.m_interaction_gui.get_batch_selected_interactions();
	};

	InteractionPanel.prototype.has_permission_to_bulk_actions = function( selected_interactions )
	{
		var has_permission = true;
		selected_interactions = selected_interactions || this.getSelectedInteractions();
		var interactions = selected_interactions.interactions || {};
		var campaigns = {};

		// obtiene las diferentes campanas de las interacciones
		$.each( interactions, function( undefined, interaction ) {
			campaigns[ interaction.campaign ] = true;
		});

		// recorro y valido si tiene permisos de bulk action, y si al menos una campana, no tiene permisos, retorna false
		$.each( campaigns, function( campaign_id ) {
			has_permission = inConcert.permissions.to_action(
				'interaction',
				'int_bulk_actions',
				undefined /* category */,
				campaign_id, true /* no log */
			);
			if ( !has_permission ) {
				return false; // break
			}
		});

		return has_permission;
	};

	InteractionPanel.prototype.__setClickBatchButton = function($button)
	{
		var self = this;
		if (!$button || $button.length === 0) {
			return;
		}
		$button
			.unbind('click')
			.bind('click', function(e)
			{
				if ( !self.has_permission_to_bulk_actions() ) {
					var message = "<%=_T('You have not permissions to bulk actions.')%>";
					inConcert.app.getMainWindow().showBannerMessage({ message : message, messageType : "Error", timeout : 5000});
					return;
				}
				var action = $(this).attr('batch_action').toLowerCase();
				// if action type is finish, this will call finishSelectedNodes
				self[action + 'SelectedNodes'](self.updateBatchButtonsStatus);
				// if action type is transfer, mantain first menu
				if (action === 'transfer') {				
					e.stopPropagation();
				}
			});
	};

	/**
	 * Given an interactionId will return the corresponding jquery node
	 *
	 * @private
	 * @param  {String} interactionId - Interaction Identificator
	 * @return {Object}               - The jQuery node
	 */
	 InteractionPanel.prototype.__getInboxInteractionNode = function(interactionId, prefix_interaction = "assigned_interaction_") {
		return this.__getInteractionNode(interactionId, prefix_interaction);
	};

	InteractionPanel.prototype.__getInteractionNode = function( interactionId, prefix ) {
		return $('#' + prefix + inConcert.Utils.getNormalizedId(interactionId));
	};

	/**
	 * Triggered when the agent select one interaction from the checkbox
	 *
	 * @private
	 * @param  {Object} interaction - The selected interaction node.
	 * @param  {Object} node        - The jQuery object for the selected interaction node.
	 */
	 InteractionPanel.prototype.onNodeSelected = function(interaction, node){
		// Selected to batch is true if the checkbox is selected.
		interaction.selectedToBatch = node.find('[int_batch_selector]').prop('checked') ? true : false;
		this.calculateInteractionClass(interaction, node);
		const count_checked_interactions = this.count_checked_interactions();
		const $bulk_actions_container_selected = $("#" + this.get_batch_actions_prefix() + "selected_number");
		$bulk_actions_container_selected.text(count_checked_interactions);
		
		if (interaction.selectedToBatch) {
			node.removeClass("interactionSelectToBatch").addClass("interactionSelectToBatch");
		}else{
			node.removeClass("interactionSelectToBatch");
		}
		
		if (count_checked_interactions === 0) {
			const $bulk_actions_container = $('#' + this.get_batch_actions_prefix() + 'container_selected_finish');
			this.show_or_hide_bulk_actions_container(false,$bulk_actions_container);
			const $select_all = this.$getChkSelectAllConversations();
			$select_all.prop('checked', false);
			return;
		}
		// update buttons events and status
		this.setBatchButtonsEvents();
		this.updateBatchButtonsStatus();
	};

	/**
	 * Will label all selected interactions on the inbox, assuming previous validation.
	 */
	 InteractionPanel.prototype.labelSelectedNodes = function()
	 {
		 var self = this;
		 var interactions = this.getSelectedInteractions().interactions;
 
		 var $batch_label = this.m_handler.find('[bulk_actions_container="true"]').find('[batch_action="label"]');
		 var labelPopup = new inConcert.LabelsPopup({ from_app: 'interaction' });
		 labelPopup.show({
			 interactions: interactions,
			 campaign: interactions[0].getCampaign(),
			 parentNode: $batch_label,
			 container: $('.boxContebatchLabelntOptionsArea'),
			 callbacks: {
				 onOpen: function() {
					 interactions.forEach(function(interaction){
						 var $interaction = self.$findInteraction( interaction );
						 if ( $interaction.length > 0 ) {
							 $interaction.addClass('selectedToTag');
						 }
					 });
				 },
				 onClose: function() {
					 interactions.forEach(function(interaction){
						 var $interaction = self.$findInteraction( interaction );
						 if ( $interaction.length > 0 ) {
							 $interaction.removeClass('selectedToTag');
						 }
					 });
				 }
			 }
		 });
	 };
 
	 /**
	  * Will transfer to an agent all selected interactions on the inbox, assuming previous validation.
	  *
	  * @param  {Function} callback
	  */
	InteractionPanel.prototype.transferSelectedNodes = function(callback = $.noop)
	 {
		 const self = this;
 
		 if (this.m_submenu_actions_popup) {
			 this.m_submenu_actions_popup.finalize(function() {
				 self.m_submenu_actions_popup = null;
			 });
			 return;
		 }
		 let interactions = this.getSelectedInteractions().interactions;
 
		 let popup_callbacks = {
			 transfer: {
				 on_transfer_success : $.noop,
				 on_transfer_error : function() {
					 interactions.forEach(function(interaction){
						 let $interaction = self.$findInteraction( interaction );
						 if ( $interaction.length > 0 ) {
							 self.updateInteractionNode( interaction );
						 }
					 });
				 },
				 onOpen: function() {
					 interactions.forEach(function(interaction){
						 let $interaction = self.$findInteraction( interaction );
						 if ( $interaction.length > 0 ) {
							 $interaction.addClass('selectedToTransfer');
						 }
					 });
				 },
				 onClose: function() {
					 interactions.forEach(function(interaction){
						 let $interaction = self.$findInteraction( interaction );
						 if ( $interaction.length > 0 ) {
							 $interaction.removeClass('selectedToTransfer');
						 }
					 });
				 }
			 }
		 };
		 // instancia la clase "interaction_gui" para cargar el submenu de transfer ("member", "campaign", "att level" y "number")
		 let interaction_gui = new inConcert.interaction_gui(inConcert.attention_actions, {
			 from_app : 'interaction',
			 owner    : this,
			 callbacks: popup_callbacks,
			 custom_allowed_actions: {
				 transfer: true
			 }
		 });
		 let $batch_transfer = this.m_handler.find('[bulk_actions_container="true"]').find('[batch_action="transfer"]');
		 interaction_gui.show_submenu_actions(interactions, {
			 $parent: $batch_transfer,
			 action_type: 'Transfer',
			 align: null, // para que no quede alineado a la izquierda, ya que los submenus se alinean siempre a la izquierda.
			 submenuNewPosition : true,
			 callbacks: {
				 on_open_submenu: function(submenu_actions_popup) {
					 self.m_submenu_actions_popup = submenu_actions_popup;
				 },
				 on_close_submenu: function() {
					 window.setTimeout(function() {
						 self.m_submenu_actions_popup = null;
					 });
				 }
			 }
		 });
	 };

	 /**
	 * Will finish all selected interactions on the inbox, assuming previous validation.
	 *
	 *
	 * @param  {Function} onInteractionsFinished - Callback
	 */
	InteractionPanel.prototype.finishSelectedNodes = function(onInteractionsFinished)
	{
		var self = this;
		var interactions = this.getSelectedInteractions().interactions;

		var campaign = inConcert.app.getCampaign(interactions[0].campaign);
		if (!campaign) {
			this.m_main.showBannerMessage({ message : "<%=_T('Campaign does not exist')%>", messageType:'Error', timeout : 4000});
			return;
		}

		var dispositionCodes = campaign.getDispositionCodes();
		if (dispositionCodes) {

			var $batch_finish = this.m_handler.find('[bulk_actions_container="true"]').find('[batch_action="finish"]');
		
			var dispositionCodesPopup = new inConcert.DispositionCodesPopup( { from_app: 'interaction' } );
			dispositionCodesPopup.show({
				interactions:		interactions,
				dispositionCodes:	dispositionCodes,
				parentNode: $batch_finish,
				container: $('.boxContentOptionsArea'),
				callbacks: {
					// when popup close, remove selection on every node.
					onClose: function() {
						interactions.forEach(function(interaction){
							self.__getInboxInteractionNode(interaction.id).removeClass('selectedToTag interactionWaiting');
						});
					}
				}
			});
		}
		else {
			var batchFunction = inConcert.app.getInteractionManager().batchArchive;
			var args = [ '','' ];
			var callbacks = {
				onFinished: function(result) {
					result = self.__extractBatchMessageInformation(result, 'finish');
					self.m_main.showBannerMessage({
						message:     result.message      ,
						messageType: result.message_type ,
						timeout:     5000
					});
				},
				onPartial: function(successfulNodes){
					if (successfulNodes){
						successfulNodes.forEach(function(interaction){
							// remove nodes if they finished successfuly.
							self.__getInboxInteractionNode(interaction.id).remove();
							self.onInteractionRemoved(interaction);
						});
					}
				},
				onFailure: function(errorMessage){
					self.m_main.showBannerMessage({ message : errorMessage, messageType : 'Error', timeout : 5000 });
					interactions.forEach(function(interaction){
						self.__getInboxInteractionNode(interaction.id).removeClass('interactionWaiting');
					});
				}
			};

			var batchData = new self.BatchData(batchFunction, interactions, args, callbacks);
			self.processBatchInteractions(batchData, function(){
				if (typeof(onInteractionsFinished) === 'function'){
					onInteractionsFinished.apply(self);
				}
			});
		}
	};

	/**
		Class Batch Data Constructor
		@param batchFunction
		@param interactions
		@param args
		@param callbacks
		@param chunkSize
	*/
	InteractionPanel.prototype.BatchData = function(batchFunction, interactions, args, callbacks, chunkSize) {
		if ( !batchFunction ){
			throw 'Batch Data Constructor - You must specify a batch function to call.';
		}

		if ( !interactions ){
			throw 'Batch Data Constructor - There are not interactions to process';
		}

		// set chunk size to 20 as default if isn't passed as argument.
		if ( !chunkSize || typeof(chunkSize) !== 'number' ){
			this.chunkSize = 20;
		}
		else{
			this.chunkSize = chunkSize;
		}

		// set default callbacks
		this.callbacks = callbacks || {};
		if (typeof(callbacks.onPartial) !== 'function'){
			this.callbacks.onPartial = $.noop;
		}
		if (typeof(callbacks.onFailure) !== 'function'){
			this.callbacks.onFailure = $.noop;
		}
		if (typeof(callbacks.onFinished) !== 'function'){
			this.callbacks.onFinished = $.noop;
		}
		this.batchFunction = batchFunction;
		this.batchInteractions = interactions;
		this.batchArgs = args;
	};

	 	/**
	 * Given a batch result, will extract the message information.
	 *
	 * @private
	 * @param  {Object} batchResult          - The batch result
	 * @param  {String} action               - The batch action (finish, requeue and so on)
	 * @return {Object} result
	 * @return {String} result.message       - The message per se
	 * @return {String} result.message_type  - Info or Error message type
	 */
	InteractionPanel.prototype.__extractBatchMessageInformation = function(batchResult, action){
		var result = {
			message : '',
			message_type : ''
		};
		var errors = batchResult.errors;
		var warnings = batchResult.warnings;
		var display_action = _T(action || 'action');

		var getConflictMessage = function(error, count_errors)
		{
			var msg;
			switch (error) {
				case 'INVALID_REQUEST':
					msg = "<%=_T('Invalid request please try again later')%>";
					break;

				case 'FORBIDDEN':
					msg = format("<%=_T('You have not permissions to {0} {1} conversations on campaign.')%>", display_action, count_errors);
					break;

				case 'FORBIDDEN_COMMENT':
					msg = format("<%=_T('{0} conversations have not been comment, because you have not permissions on campaign.')%>", count_errors);
				break;
				case 'UNABLE_TO_TAKE':
					msg = format("<%=_T('Some conversations ({0}) could not be taken, please try again later.')%>", count_errors);
					break;
				default: // 'GENERIC_ERROR':
					msg = format("<%=_T('{0} failed to {1} conversations, please try again later')%>", count_errors, display_action);
					break;
			}
			return msg;
		};

		// the batch contains errors
		if (errors && errors.count_total > 0) {
			result.message_type = 'Error';
			if (errors.count_types === 1) {
				var error_type = errors.types.pop();
				result.message = getConflictMessage(error_type, errors.count_total);
			}
			else {
				result.message = getConflictMessage('GENERIC_ERROR', errors.count_total);
			}
		}
		else {
			result.message_type = 'Info';
			if ( warnings && warnings.count_total > 0) {
				if (warnings.count_types === 1) {
					var warning_type = warnings.types.pop();
					result.message = getConflictMessage(warning_type, warnings.count_total);
				}
				else {
					result.message = getConflictMessage('GENERIC_WARNING', warnings.count_total);
				}
				result.message = format("<%=_T('All conversations were {0} successfully! {1}')%>", display_action, result.message) ;
			}
			else {
				result.message = format("<%=_T('All conversations were {0} successfully!')%>", display_action);
			}
		}

		return result;
	};

	 	/**
	 * Add waiting animation for all interactions
	 *
	 * @private
	 * @param  {Array} interactions   - A list of interactions.
	 * @param  {Boolean} are_pending  - If the interactions are in pending state or not
	 */
	InteractionPanel.prototype.mark_interactions_as_pending = function(interactions, are_pending) {
		// TO-DO: Check this, it's necessary?
		if ( !_.isArray(interactions) ) {
			return;
		}
		const self = this;
		let interactionManager = inConcert.app.getInteractionManager();

		interactions.forEach(function(interaction) {
			if (interaction) {
				let current_interaction;
				if (!interaction.type) {
					current_interaction = interactionManager.findInteraction(interaction.id);
				}
				else {
					current_interaction = interaction;
				}
				if (current_interaction){
					current_interaction.pending = are_pending;
					let prefix_interaction = self.get_prefix(current_interaction);
					let $node = self.__getInboxInteractionNode(interaction.id, prefix_interaction);
					if ($node && $node.length > 0) {
						self.calculateInteractionClass(current_interaction, $node);
					}
				}
			}
		});
	};

	/**
	 * Process a bunch of interactions by chunk
	 *
	 * @public
	 *
	 * @param  {Object}    batchData
	 *		   {Boolean}   batchData.containsChunk
	 *		   {Array}     batchData.batchInteractions       - The interactions to be processed.
	 *         {Object}    batchData.callbacks               - An object containing callbacks to be called.
	 *         {Object}    batchData.callbacks.onFinished    - Called when all interactions are processed.
	 *         {Object}    batchData.callbacks.onPartial     - Called when a chunk of interactions is completed.
	 *         {Object}    batchData.callbacks.onFailure     - Called when the process fails.
	 *         {Object}    batchData.batchArgs               - Extra args for this batch.
	 *
	 * @param  {Function} onBatchFinished - Callback
	 */
	InteractionPanel.prototype.processBatchInteractions = function(batchData, onBatchFinished) {
		var self = this;
		var interactionsChunk = [];

		// this is done to know if we already make a chunk from this batch data
		batchData.containsChunk = false;

		// All interactions start as pending
		self.mark_interactions_as_pending(batchData.batchInteractions, true);

		// This two maps will old the warnings and errors for every
		// batch response
		var interactions_warnings = {}, interactions_errors = {};

		/**
		 * Adds a conflict to the corresponding list depending
		 * on the conflict type
		 *
		 * @private
		 * @param  {Object} result
		 * @param  {String} conflict_type
		 * @param  {Object} raw_info
		 */
		var __add_conflict = function(result, conflict_type, raw_info) {
			if (conflict_type === 'error') {
				interactions_errors[result] = interactions_errors[result] || {count: 0};
				interactions_errors[result].count++;
				interactions_errors[result].raw_info = raw_info;
			}
			else {
				interactions_warnings[result] = interactions_warnings[result] || {count: 0};
				interactions_warnings[result].count++;
				interactions_warnings[result].raw_info = raw_info;
			}
		};

		/**
		 * Will check for any conflicts inside a batch response, if there are
		 *
		 * @private
		 * @param  {Array} conflicts_list - The conflict list
		 * @param  {String} conflict_type - The conflict type
		 */
		var __check_batch_chunk_for_conflicts = function( conflicts_list, conflict_type ) {
			// TO-DO: Check this, it's necessary?
			if (!$.isArray(conflicts_list) || conflicts_list.length === 0) {
				return;
			}

			conflicts_list.forEach(function( conflict_node ) {
				var result;
				switch (conflict_type){
					case 'error':
						result = conflict_node.result;
						break;
					case 'warning':
						if (conflict_node.warning){
							result = conflict_node.warning;
						}
						else {
							// successfully, is not a conflict
							return true; // continue in loop
						}
						break;
					default:
						break;
				}

				if (typeof result === 'string') {
					result = result.toUpperCase();
				}
				else {
					result = 'GENERIC_WARNING';
				}
				__add_conflict(result, conflict_type, conflict_node.raw_info);
			});
		};

		/**
		 * Will count the total of conflicts in a batch response
		 * for a determinated conflict type.
		 *
		 * @private
		 * @param  {String} conflict_type - The conflict type (eg. error, warning)
		 * @return {Object} result        - An object with the result
		 */
		var __count_conflicts = function(conflict_type) {
			var conflicts_list;
			if (conflict_type === 'warning') {
				conflicts_list = interactions_warnings;
			}
			else {
				conflicts_list = interactions_errors;
			}

			var result = {
				types: [],
				types_raw: {},
				count_types: 0,
				count_total: 0
			};

			_.each(conflicts_list, function(conflict, conflict_type){
				result.types.push(conflict_type);
				result.types_raw[conflict_type] = conflict.raw_info;
				result.count_types++;
				result.count_total = result.count_total + conflict.count;
			});

			return result;
		};

		var cloned_batch_interactions = inConcert.Utils.cloneArray(batchData.batchInteractions);
		/**
		 * Will process a new chunk of interactions
		 *
		 * @private
		 */
		var __start_new_chunk = function() {
			// get the first chunk, this will modify the array
			interactionsChunk = batchData.batchInteractions.splice(0, batchData.chunkSize);
			if (interactionsChunk.length === 0){
				// no more interactions to process, call onFinished with all interactions processed.
				batchData.callbacks.onFinished({
					errors   : __count_conflicts( 'error'   ),
					warnings : __count_conflicts( 'warning' )
				});
				if (typeof(onBatchFinished) === 'function'){
					onBatchFinished();
				}
				return;
			}

			// make sure we don't have any interactions in the argument list
			if (batchData.containsChunk){
				batchData.batchArgs.shift();
			}

			// put as first argument the new interactions chunk
			batchData.batchArgs.unshift(interactionsChunk);
			batchData.containsChunk = true;

			// call the batch function passed as argument with the custom arguments
			batchData.batchFunction.apply(null, batchData.batchArgs);
		};

		// Push to the args list a custom callback for every state of the ajax request
		batchData.batchArgs.push(function(ok, successfulNodes, unsuccessfulNodes, errorMessage) {
			if (ok && successfulNodes && unsuccessfulNodes) {
				self.mark_interactions_as_pending(unsuccessfulNodes, false);

				// go and check if there are any errors or warnings inside this
				// batch response.
				__check_batch_chunk_for_conflicts(successfulNodes, 'warning');
				__check_batch_chunk_for_conflicts(unsuccessfulNodes, 'error');

				// callback for partial chunk processing
				batchData.callbacks.onPartial(successfulNodes, unsuccessfulNodes);

				// start a new chunk of interactions
				__start_new_chunk();
			}
			else{
				self.mark_interactions_as_pending(cloned_batch_interactions, false);
				batchData.callbacks.onFailure(errorMessage);
				// finish processing if something went wrong.
				if (typeof(onBatchFinished) === 'function'){
					return onBatchFinished();
				}
			}
		});

		// Start processing interactions
		__start_new_chunk();
	};

	// triggered when the user clicks on the interaction checkbox
	// will update the batch action buttons status
	InteractionPanel.prototype.setBatchButtonsEvents = function()
	{
		var self = this;

		if (!this.m_has_bulk_actions) {
			return;
		}

		var $bulk_actions = this.m_handler.find('#' + this.get_batch_actions_prefix() + 'container_selected_finish');
		if ($bulk_actions.length === 0) {
			return;
		}

		const $select_all = this.$getChkSelectAllConversations();
		var allowed_bulk_actions = this.allowed_bulk_actions();
		if (!allowed_bulk_actions) {
			if (this.is_enable_bulk_actions_container($bulk_actions)) {
				this.m_interaction_gui.hide_batch_selectors();
				if ($select_all) {
					if ($select_all.prop('checked')) {
						$select_all.trigger('click');
					}
					else {
						$select_all.prop('checked', true);
						$select_all.trigger('click');
					}
				}
				this.show_or_hide_bulk_actions_container(false,$bulk_actions);
			}
			if (!this.m_cached_bulk_actions_HTML) {
				this.m_cached_bulk_actions_HTML = $bulk_actions.html();
				$bulk_actions.html("");
			}
			return;
		}

		if (this.m_cached_bulk_actions_HTML) {
			$bulk_actions.html(this.m_cached_bulk_actions_HTML);
			this.m_cached_bulk_actions_HTML = null;
		}
		if (this.is_enable_bulk_actions_container($bulk_actions)) {
			return;
		}
		this.show_or_hide_bulk_actions_container(true,$bulk_actions);
		this.m_interaction_gui.show_batch_selectors();

		// se asigna el evento click a cada uno de los botones del batch
		$.each($bulk_actions.find('.batchActionButton'), function(_, button) {
			self.__setClickBatchButton($(button));
		});
	};

	InteractionPanel.prototype.$get_total_interactions = function()
	{
		return this.m_handler.find('.boxContentScrollArea').find('[original_int_id]');
	};

	InteractionPanel.prototype.count_checked_interactions = function()
	{
		const $totalInteractions = this.$get_total_interactions();
		return $totalInteractions.find('[int_batch_selector]:checked').length;
	};

	InteractionPanel.prototype.get_batch_actions_prefix = function()
	{
		return 'interaction_assigned_';
	};

	InteractionPanel.prototype.$get_panel_container_buttons = function()
	{
		return $("#panelAssignedContainerButtons");
	};

	InteractionPanel.prototype.is_enable_bulk_actions_container = function($container)
	{
		const isPanelQueued = this.m_category === 'queued';
		if (isPanelQueued) {// para el panel de encolados se habilita o deshabilita.
			const $interaction_bulk_actions_menu = $("#" + this.get_batch_actions_prefix() + "bulk_actions_menu");
			return $interaction_bulk_actions_menu.prop('disabled');
		}
		return $container.is(':visible');
	};

	InteractionPanel.prototype.show_or_hide_bulk_actions_container = function(isShow, $container)
	{
		const isPanelQueued = this.m_category === 'queued';
		if (isPanelQueued) {// para el panel de encolados se habilita o deshabilita.
			const $interaction_bulk_actions_menu = $("#" + this.get_batch_actions_prefix() + "bulk_actions_menu");
			$interaction_bulk_actions_menu.prop('disabled', !isShow);
			return;
		}
		$container[ isShow ? "show" : "hide" ]();
	};

	InteractionPanel.prototype.updateBatchActions = function()
	{
		if (!this.m_has_bulk_actions) {
			return;
		}
		const $bulk_actions_container = $('#' + this.get_batch_actions_prefix() + 'container_selected_finish');
		const count_checked_interactions = this.count_checked_interactions();
		let $bulk_actions_container_selected = $("#"+ this.get_batch_actions_prefix() +"selected_number");
		$bulk_actions_container_selected.text(count_checked_interactions);
		if (count_checked_interactions === 0) {
			this.show_or_hide_bulk_actions_container(false,$bulk_actions_container);
			return;
		}
		this.show_or_hide_bulk_actions_container(true,$bulk_actions_container);
		this.setBatchButtonsEvents();
		this.updateBatchButtonsStatus();
	};

	/**
	 * Given an batch action name, will validate if can be done with the selected nodes.
	 *
	 * @todo Break down this function into smaller ones.
	 *
	 * @private
	 * @param  {String}  actionType              - An action to validate (finish, requeue, and so on)
	 * @param  {Object}	 selected_interactions
	 * @return {Object}  result
	 * @return {Boolean} result.passesValidation - Indicates if passes validation or not
	 * @return {String}  result.message          - Error message, if any, empty otherwise.
	 */
	InteractionPanel.prototype.validateBatchAction = function( actionType, selected_interactions )
	{
		if (!this.m_has_bulk_actions) {
			return;
		}
		// An object to save the validation status and the message
		// initialize status with error, but if all validations pass, result will be ok
		var result = { passesValidation: false, message: '' };

		selected_interactions = selected_interactions || this.getSelectedInteractions();
		if (selected_interactions.interactions.length === 0){
			return result;
		}
//
		if ( !this.has_permission_to_bulk_actions( selected_interactions ) ) {
			result.message = "<%=_T('You have not permissions to bulk actions.')%>";
			return result;
		}

		var allNodesAreFromTheSameChannel = function() {
			var channel = selected_interactions.interactions[0].type;
			return selected_interactions.interactions.every(function(node){
				return node.type === channel;
			});
		};

		var allNodesAreOrArentAllegro = function() {
			var allegro = selected_interactions.interactions[0].isAllegro();
			return selected_interactions.interactions.every(function(node){
				return node.isAllegro() === allegro;
			});
		};

		var allNodesAreFromTheSameCampaign = function() {
			var has_permission = selected_interactions.interactions.some(function(interaction)
			{
				var hasPermission = interaction.hasPermission( actionType );
				if (!hasPermission) {
					return false;
				}
				return true;
			});
			if ( !has_permission ) {
				return false;
			}
			var campaignId = selected_interactions.interactions[0].campaign;
			return selected_interactions.interactions.every(function(node){
				return node.campaign === campaignId;
			});
		};

		var allInteractionsArePendingOrInternal = function(){
			return selected_interactions.interactions.every(function(interaction){
				return interaction.state === 'PENDING' || interaction.isInternal();
			});
		};

		var allInteractionsInWrapup = function(){
			return selected_interactions.interactions.every(function(interaction){
				var is_wrapped_up = interaction.isWrappedUp();
				return is_wrapped_up;
			});
		};

		var allInteractionsInQueued = function(){
			return selected_interactions.interactions.every(function(interaction){
				var is_wrapped_up = interaction.isQueued();
				return is_wrapped_up;
			});
		};

		var isMaxOneCall  = function(){
			const result = selected_interactions.interactions.filter(interaction => (interaction.isTelephonyInteraction() || interaction.isAllegro()));
			//Si existen calls. No se puede tomar|transferir multiples llamadas.
			return result.length <= 1;
		};

		var atLeastOneInWrapup = function(){
			return selected_interactions.interactions.some(function(interaction){
				var is_wrapped_up = interaction.isWrappedUp();
				return is_wrapped_up;
			});
		};

		var allInteractionsAreLive = function(){
			return selected_interactions.interactions.every(function(interaction){
				var is_active = interaction.isLive();
				return is_active;
			});
		};

		var atLeastOneIsLive = function(){
			return selected_interactions.interactions.some(function(interaction){
				var is_active = interaction.isLive();
				return is_active;
			});
		};

		var atLeastOneIsPending = function(){
			return selected_interactions.interactions.some(function(node){
				return node.state === 'PENDING';
			});
		};

		var atLeastOneHasPendingLabel = function(){
			return selected_interactions.interactions.some(function(interaction){
				return interaction.hasPendingLabel();
			});
		};

		var atLeastOneHasContactConflict = function(){
			return selected_interactions.interactions.some(function(interaction){
				return interaction.hasContactConflict();
			});
		};

		var atLeastOneIsInternal = function(){
			return selected_interactions.interactions.some(function(interaction){
				return interaction.isInternal();
			});
		};

		var atLeastOneHasPermission = function() {
			return selected_interactions.interactions.some(function(interaction)
			{
				var hasPermission = interaction.hasPermission( actionType );
				if (!hasPermission) {
					return false;
				}
				var hasAction = interaction.hasIntegrationOverrideActionPopupShow(actionType);
				if (hasAction) {
					return false;
				}
				return true;
			});
		};

		switch (actionType) {
			case 'FINISH':
				if (allInteractionsAreLive() && !allInteractionsInWrapup()){
					result.message = "<%=_T('To finish live conversations, you first need to hang them!')%>";
					return result;
				}
				if (!atLeastOneHasPermission()) {
					result.message = "<%=_T('You have not permissions to finish conversations.')%>";
					return result;
				}
				if (atLeastOneHasContactConflict()) {
					result.message = "<%=_T('You cannot finish conversations with contact conflict!')%>";
					return result;
				}
				// validate if all interactions selected are or arent allegro
				if(!allNodesAreOrArentAllegro()){
					result.message = "<%=_T('You can not finish phone calls and another interactions at same time.')%>";
					return result;
				}
				/* falls through */

			case 'TRANSFER':

				if (!allNodesAreFromTheSameChannel()) {
					result.message = "<%=_T('You can not transfer conversations from different channels!')%>";
					return result;
				}

				if (!atLeastOneHasPermission()) {
					result.message = "<%=_T('You have not permissions to transfer conversations.')%>";
					return result;
				}
				if (atLeastOneHasPendingLabel()){
					result.message = format("<%=_T('You can not {0} conversations labeled as pending!')%>", actionType);
					return result;
				}
				if ( actionType !== 'FINISH' ) {
					if ( atLeastOneIsLive() ) {
						if ( atLeastOneInWrapup() ) {
							result.message = "<%=_T('Conversations in wrap-up can not be transfered.')%>";
							return result;
						}
					}

					if (allInteractionsInQueued() && !isMaxOneCall()) {
						result.message = "<%=_T('You can only transfer one call conversation.')%>";
						return result;
					}
				}

				// this validation only applies to finish and transfer actions.
				// you can (of course) label an interaction already labeled as pending.
				/* falls through */
			case 'LABEL':
				if (!atLeastOneHasPermission()) {
					result.message = "<%=_T('You have not permissions to label conversations.')%>";
					return result;
				}
				// validate if all interactions are from the same campaign.
				if (!allNodesAreFromTheSameCampaign()){
					result.message = format("<%=_T('You can not {0} conversations from different campaigns!')%>", actionType);
					return result;
				}
				// if some interaction selected is pending.
				if(atLeastOneIsPending()){
					result.message = format("<%=_T('You can not {0} pending conversations!')%>", actionType);
					return result;
				}
				// validate if there is no internal chat selected.
				if(atLeastOneIsInternal()){
					result.message = format("<%=_T('Action {0} disabled for internal chats!')%>", actionType);
					return result;
				}
				break;

			case 'CANCEL':
				// validate if all interactions selected are outbound or internal chats.
				if(!allInteractionsArePendingOrInternal()){
					result.message = "<%=_T('You can only cancel pending or internal conversations!')%>";
					return result;
				}
				if (atLeastOneHasPendingLabel()){
					result.message = "<%=_T('You can not cancel conversations labeled as pending!')%>";
					return result;
				}
				break;

			case 'REQUEUE':
				// validate if all interactions selected are not outbound.
				if(atLeastOneIsPending()){
					result.message = "<%=_T('You can not requeue pending conversations!')%>";
					return result;
				}
				if (atLeastOneHasPendingLabel()){
					result.message = "<%=_T('You can not requeue conversations labeled as pending!')%>";
					return result;
				}
				// validate if there is no internal chat selected.
				if(atLeastOneIsInternal()){
					result.message = "<%=_T('You can not requeue internal chats!')%>";
					return result;
				}
				if (!atLeastOneHasPermission()) {
					result.message = "<%=_T('You have not permissions to requeue conversations.')%>";
					return result;
				}
				if ( atLeastOneIsLive() ) {
					if ( atLeastOneInWrapup() ) {
						result.message = "<%=_T('Conversations in wrap-up can not be requeued.')%>";
						return result;
					}
				}
				// validate if all interactions selected are or arent allegro
				if(!allNodesAreOrArentAllegro()){
					result.message = "<%=_T('You can not requeue phone calls and another interactions at same time.')%>";
					return result;
				}
				break;
			case 'TAKE':
				if (!atLeastOneHasPermission()) {
					result.message = "<%=_T('You have not permissions to take conversations.')%>";
					return result;
				}

				if (!allInteractionsInQueued()) {
					result.message = "<%=_T('Conversations that are not queued cannot be taken.')%>";
					return result;
				}
				if (!isMaxOneCall()) {
					result.message = "<%=_T('You can only take one call conversation.')%>";
					return result;
				}

				if(!allNodesAreOrArentAllegro()){
					result.message = "<%=_T('You can not take phone calls and another interactions at same time.')%>";
					return result;
				}

				break;
			default :
				// unknown validation
				result.message = format("<%=_T('Unknown action: {0}')%>", actionType);
				return result;
		}
		// all validation passed
		result.passesValidation = true;
		return result;
	};

	InteractionPanel.prototype.getBatchButtons = function()
	{
		return [
			{ actionType : 'FINISH'  , selector : '#batchActionFinish'   },
			{ actionType : 'CANCEL'  , selector : '#batchActionCancel'   },
			{ actionType : 'REQUEUE' , selector : '#batchActionRequeue'  },
			{ actionType : 'LABEL'   , selector : '#batchActionLabel'    },
			{ actionType : 'TRANSFER', selector : '#batchActionTransfer' },
		];
	};

	// triggered when the user clicks on the interaction checkbox
	// will update the batch action buttons status
	InteractionPanel.prototype.updateBatchButtonsStatus = function()
	{
		var self = this;

		if (!this.m_has_bulk_actions) {
			return;
		}

		var $bulk_actions = this.m_handler.find('#'+ this.get_batch_actions_prefix() +'container_selected_finish');
		if ($bulk_actions.length === 0 ) {
			return;
		}
		// select/unselect all interactions checkbox
		var $totalInteractions = this.$get_total_interactions();
		var count_total_interactions = $totalInteractions.length;
		var count_checked_interactions = $totalInteractions.find('[int_batch_selector]:checked').length;

		// if there is no interaction on the list, disable and uncheck the checkbox
		const $chk_select_all = this.$getChkSelectAllConversations();
		if ($chk_select_all) {
			if ( count_total_interactions === 0 ){
				$chk_select_all.prop('disabled', true).prop('checked', false);
			}
			else{
				// make sure if all interactions are selected one by one to check the checkbox
				$chk_select_all
					.prop('disabled', false)
					.prop('checked', count_checked_interactions === count_total_interactions ? true : false);
			}
		}

		// I will disable double click on the checkbox container,
		// if I didn't, interaction client will be opened.

		var $bulk_actions_container = this.$get_panel_container_buttons();
		if ($bulk_actions_container.length === 0) {
			return;
		}
		$bulk_actions_container.find('.selectionTriggerContainer').dblclick(function(e){
			e.preventDefault();
		});

		var atLeastOneChecked = count_checked_interactions > 0;
		// At least one interaction must be selected to make buttons enabled.
		var $bulk_action = $bulk_actions_container.find('.batchActionButton');
		if (!atLeastOneChecked){
			$bulk_action
				.prop('disabled', true)
				.removeClass('disabledInfo');
			return;
		}

		// enable batch action buttons
		$bulk_action.prop('disabled', false);

		var batchButtons = this.getBatchButtons();

		var selected_interactions = this.getSelectedInteractions();
		const $interaction_bulk_actions_menu = $("#" + this.get_batch_actions_prefix() + "bulk_actions_menu");
		const $interaction_bulk_actions_menu_options = $("#" + this.get_batch_actions_prefix() + "bulk_actions_menu_options");
		// validate every button status, display a message if the action is disabled
		let countHidePanelList = 0;
		batchButtons.forEach(function(currentButton)
		{
			var $btn_action = $interaction_bulk_actions_menu_options.find(currentButton.selector);
			if ( $btn_action.length > 0 ) {
				var buttonValidation = self.validateBatchAction( currentButton.actionType, selected_interactions );
				if (buttonValidation.passesValidation){
					var action = inConcert.Utils.capitaliseFirstLetter(currentButton.actionType);
					$btn_action
						.removeClass('disabledInfo')
						.attr('tooltip', format("<%=_T('{0} {1} conversations')%>", _T(action), count_checked_interactions));
					$btn_action.show();
					self.__setClickBatchButton($btn_action);
				}
				else{
					$btn_action
						.attr('tooltip', buttonValidation.message)
						.addClass('disabledInfo')
						.unbind('click')
						.inconcert_tooltip();
					$btn_action.hide();
					countHidePanelList++;
				}
			}
		});
		if (batchButtons.length === countHidePanelList) {
			$interaction_bulk_actions_menu.prop("disabled", true);
		}
		else {
			$interaction_bulk_actions_menu.prop("disabled", false);
		}
		/*
			Como al menu de acciones se visualiza mediante boostrap, se debe agregar este evento, para que realice un trigger del eveno "click" al body
				para que todo el comportamiento funcione como siempre.
			Ej: para que otros menues abiertos con la clase "popup_windows.js" se puedan cerrar de forma automatica 
		*/
		$interaction_bulk_actions_menu.off("click").on("click", function() {
			let is_visible = $interaction_bulk_actions_menu_options.is(':visible');
			window.setTimeout(function() {
				if (is_visible) {
					$("body").trigger("click");
				}
			});
		});

		if (count_checked_interactions === 0) {
			this.show_or_hide_bulk_actions_container(false,$bulk_actions_container);
		}
	};

	InteractionPanel.prototype.__get_face_by_name = function(name)
	{
		if (!this.facets_by_name) {
			return '';
		}
		return this.facets_by_name[name.toLowerCase()];
	};

	InteractionPanel.prototype.__make_faces_by_name = function(facets)
	{
		var self = this;
		this.facets_by_name = {};
		var facet_labels = [];
		$.each(facets, function(_, facet) {
			self.facets_by_name[facet.label.toLowerCase()] = facet.value;
			facet_labels.push({ value : facet.label, label : facet.label });
		});
		return facet_labels;
	};

	/*
		Mapea los accounts Id por la descripcion de las cuentas y lo asigna a un atributo de la clase.
		Retorna un array con los accounts descriptions.
	*/
	InteractionPanel.prototype.index_campaign_accouts_by_desc = function()
	{
		var self = this;
		this.mapped_accounts_by_desc = {};

		var index_accouts_by_desc = function(campaign)
		{
			var ids = [];
			var campaign_id = campaign.Id.match('([^@]+)@(.+)')[1];
			$.each(campaign.accounts || {}, function( _, account) {
				var account_desc = campaign.getAccountDescription( account );
				account_desc = campaign_id + ' ' + account_desc + ' (' + (self.proxies_desc[ account.proxyType.toUpperCase()] || account.proxyType) + ')';
				ids.push(account_desc);
				self.mapped_accounts_by_desc[account_desc] = account.id;
			});
			return ids;
		};

		var accounts_ids = [];
		var campaigns = inConcert.app.getCampaigns();
		for (var i = 0; i < campaigns.length; i++) {
			var campaign_accounts_ids = index_accouts_by_desc(campaigns[i]);
			accounts_ids = accounts_ids.concat(campaign_accounts_ids);
		}

		return accounts_ids;
	};
}());
