/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert, _, moment */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name TwitterInteraction
 * @author Sebastián Moreno (smoreno@inconcertcc.com)
 */
(function () {

	"use strict";
	var unesc_attr = inConcert.Utils.unescapeAttrib;
	var format     = inConcert.Utils.format;

	function TwitterInteraction (rawInteraction)
	{
		TwitterInteraction.uber.constructor.call(this, rawInteraction);
	}

	inConcert.Utils.inherit(TwitterInteraction, inConcert.interactions.base);

	TwitterInteraction.prototype.inflate = function (callback)
	{
		var self = this;
		TwitterInteraction.uber.inflate.call(this, function(data) {
			if(data) {
				self.sortEvents();
				callback(data);
			}
		});
	};

	TwitterInteraction.prototype.SanitizeMessage = function(message = "") {
		return inConcert.Utils.twitterProcessMessage(message).sanitizedText;
	};

	TwitterInteraction.prototype.getAddress = function(_) {
		return this.getAccount() + "|" + this.campaign + "|" + this.getAccount();
	};

	TwitterInteraction.prototype.getDisplayAddress = function() {
		return this.getAddress();
	};

	TwitterInteraction.prototype.getDisplayContactAddress = function() {
		let contactAddress = this.contactAddress;
		if (!contactAddress) {
			return "";
		}
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.campaign, null, true);
		if(!visualize_address)
		{
			return "<%=_T('Hidden Address')%>"
		}
		return "@" + contactAddress;
	};

	TwitterInteraction.prototype.getDateObject = function(index) {
		index = index || 0;
		const event = this.getEvent(index);
		if (!event) {
			return "";
		}
		let date_value;
		if (event.created_timestamp) {
			date_value = parseInt(event.created_timestamp, 10);
		}
		else {
			date_value = event.created_at;
		}
		return this.convertDateStringToDate(date_value);
	};

	/*
		twitter_date: (format) Sat Dec 07 21:39:11 +0000 2013
		Se utiliza la libreria "moment" para realizar el parseo de este tipo de fechas y solucionar problemas de compatibilidad con IE

		Referencias
			issue: http://redmine.inconcert/issues/7104
			info :
					- https://stackoverflow.com/questions/6549223/javascript-code-to-display-twitter-created-at-as-xxxx-ago
					- http://momentjs.com/
					- http://momentjs.com/docs/
	*/
	TwitterInteraction.prototype.convertDateStringToDate = function( twitter_date )
	{
		var string_date;

		if (inConcert.Utils.browserDetect().name === "IE") {
			var moment_date = moment(twitter_date, "dd MMM DD HH:mm:ss ZZ YYYY");
			moment_date._isUTC = true;
			string_date = moment_date.format("YYYY-MM-DD HH:mm:ss");
		}
		else {
			string_date = twitter_date;
		}

		// Se invoca a la clase Base Interaction
		var date_time = TwitterInteraction.uber.convertDateStringToDate.call(this, string_date);
		return date_time;
	};

	TwitterInteraction.prototype.getSource = function(index) {
		this.assert_inflated();

		var event = this.events[index];
		if (event.agent) {
			return event.agent;
		}
		else {
			return this.getUserName( event );
		}
	};

	TwitterInteraction.prototype.getEventUser = function(index) {
		this.assert_inflated();
		return this.events[index].user || this.events[index].sender;
	};

	TwitterInteraction.prototype.getEventUserId = function(index) {
		var user = this.getEventUser(index);
		return user && user.id_str || null;
	};

	TwitterInteraction.prototype.isDirectMessage = function(index) {
		if (index === undefined) {
			return this.interactionData.subtype === "DIRECTMESSAGE";
		}
		this.assert_inflated();
		var event = this.events[index || 0];
		//2016-11-07: Old messages has not direct_message flag
		return (event.direct_message || !event.source) && true || false;
	};

	TwitterInteraction.prototype.isRetweeted = function(index) {
		this.assert_inflated();
		var event = this.events[index || 0];
		return event.retweeted || event.retweeted_status;
	};

	TwitterInteraction.prototype.isQuoted = function(index) {
		if (index === undefined) {
			return this.interactionData.subtype === "QUOTED";
		}
		this.assert_inflated();
		var event = this.events[index || 0];
		return event.quoted_status;
	};

	TwitterInteraction.prototype.isDeleted = function(index) {
		this.assert_inflated();
		var event = this.events[index || 0];
		var id = event.id_str;
		return this.tweetIsDeleted(id);
	};

	TwitterInteraction.prototype.tweetIsDeleted = function(id) {
		return this.interactionData.deletedMessages && this.interactionData.deletedMessages[id] || false;
	};

	TwitterInteraction.prototype.getEventUserAvatarURI = function(event) {
		if (typeof(event) !== "object") {
			return "";
		}

		if ( event.creator === "agent" ) {
			var user_id = event.agent;
			if ( !user_id ) {
				user_id = inConcert.app.getCompleteId();
			}
			if ( user_id.match(inConcert.app.idAtVCCRegExp) ) {
				user_id = RegExp.$1;
			}
			return inConcert.app.getBaseUrl() + "/users/get_avatar/" + user_id;
		}
		else if ( event.creator === "bot" ) {
			return "/inconcert/apps/agent/images/botAvatar.png";
		}
		else if (event.avatarFileId) {
			return inConcert.app.getBaseDomain() + "/inconcert/apps/repository/download/?id=" +
				encodeURIComponent(event.avatarFileId) + "&section=-1&part=-1";
		}
		else {
			let user = event.sender ? event.sender : event.user;
			if (typeof(user) === "object" && user.profile_image_url) {
				return user.profile_image_url.replace(/http:|https:/g, location.protocol);
			}
			return "";
		}
	};

	TwitterInteraction.prototype.eventsAreFromTheSameClient = function() {
		this.assert_inflated();

		var events_from_the_same_client = true;
		var last_event_client = null;

		this.events.forEach(function(current_event){
			var current_event_client = _.findWhere(current_event.addresses, {"source" : "from"}).address.displayable;
			// first iteration
			if (!last_event_client){
				last_event_client = current_event_client;
			} else {
				if (current_event_client !== last_event_client){
					events_from_the_same_client = false;
				}
			}
		});
		return events_from_the_same_client;
	};

	TwitterInteraction.prototype.getEventScreenName = function(eventIndex) {
		this.assert_inflated();

		if (eventIndex === undefined || !this.events || this.events.length === 0) {
			return "";
		}
		var event = this.events[eventIndex];
		if (!event) {
			return "";
		}
		var user = event.user ? event.user : event.sender;
		if (!user) {
			return "";
		}
		return user.screen_name;
	};

	TwitterInteraction.prototype.getMessage = function(eventIndex) {
		// IB:
		var result;
		if(eventIndex > 0) {
			this.assert_inflated();

			if (!this.events || this.events.length === 0) {
				return "";
			}
			var event = this.events[eventIndex];
			if (!event) {
				return "";
			}
			result = event.text;
		}
		else {
			// Use what we've set in the initial load
			result = this.properties.first_message_text;
		}

		return unesc_attr(result);
	};

	TwitterInteraction.prototype.getEventShortDescription = function(sorted_index) {
		return this.getMessage(sorted_index);
	};

	//returns the conversation contact or member display text
	TwitterInteraction.prototype.getUserName = function( event )
	{
		this.assert_inflated();

		if ( typeof( event ) !== "object" ) {
			return "";
		}

		var user = event.sender ? event.sender : event.user;
		var template = "{0} ({1})";
		if ( !event.agent ) {
			if ( !user ) {
				return "";
			}
			let screen_name = "@" + user.screen_name;
			const visualize_address = inConcert.app.canVisualizeAddress(null, this.campaign, null, true);
			if(event.creator === "contact" && !visualize_address)
			{
				screen_name = "<%=_T('Hidden Address')%>"
			}
			return format( template, user.name, screen_name );
		}
		else {
			var agent_name = this.get_agent_name( event );
			if ( !user ) {
				return agent_name;
			}
			return format( template, agent_name, "@" + user.screen_name );
		}
	};

	//This function doesn't assume a true/false response.
	//This version uses http status codes
	TwitterInteraction.prototype.post = function(uri, request, callback) {
		inConcert.invoke({
			type: "POST",
			url: uri,
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			success: function() {
				if (callback) {
					callback(true); //successfull
				}
			},
			error: function(_, error) {
				inConcert.logger.action({action_name : "twitter_post", error_message : error.result });
				if (typeof(callback) !== "function") {
					return;
				}
				var message;
				switch (error.result)
				{
					case "PROXY_ERROR":
						message = "<%=_T('Unable to send your tweet now: ')%>" + error.reason;
						break;

					case "NOT_LICENSED":
						message = "<%=_T('Unable to send tweet. Not has licensed for the Twitter channel.')%>";
						break;

					case "INTERNAL_ERROR":
						message = "<%=_T('There was an error while sending the tweet, please try again later')%>";
						break;

					//direct message from tweet

					case "REOPEN_FAIL":
						message = "<%=_T('There was an error while reopening direct message interaction, please check your attention capability.')%>";
						break;
						
					case "POST_CLOSED":
						message = "<%=_T('Unable to send tweet. Related post was closed or deleted.')%>";
						break;

					case "CREATE_INTERACTION":
						message = "<%=_T('There was an error while creating direct message interaction, please check your attention capability.')%>";
						break;

					case "THREAD_OPEN":
						if (error.agent) {
							var agent = inConcert.Utils.getIdWithoutVCC(error.agent);
							if (agent === inConcert.app.getLoggedUserId()) {
								message = "<%=_T('You already have an assigned interaction with this user.')%>";
							}
							else {
								message = format("<%=_T('There is an open interaction with this user assigned to agent {0}.')%>",
									agent);
							}
						}
						else {
							var state = error.state || "";
							switch(state) {
								case "WAITING":
									state = "<%=_T('Waiting')%>";
									break;
								case "QUEUED":
									state = "<%=_T('Queued')%>";
									break;
							}
							message = format("<%=_T('There is an open interaction with this user in state {0}.')%>", 
								state);
						}
						break;

					default:
						message = "<%=_T('There was an error while sending the tweet, please try again later')%>";
				}
				callback(false, message); //not successful
			}
		});
	};

	/**
	We return the first tweet's text of this interaction as 'extended info'.
	@param max_length
	*/
	TwitterInteraction.prototype.getExtendedInfo = function( max_length )
	{
		var extended_info = this.getMessage(0) || "";
		return inConcert.Utils.truncateWord( extended_info, max_length );
	};

	TwitterInteraction.prototype.newPost = function(parameters) {
		if (this.state !== "PENDING") {
			this[parameters.method](parameters);
		}
		else {
			inConcert.logger.action({action_name : "create_twitter", id : this.id, type : this.type,
						campaign : this.getCampaign(), destination : this.contactAddress, action_type : "conversation"});
			var url = this.m_manager.getBaseUrl() + "/interactions/twitter/create_outgoing/";
			var request = {
				id: this.id,
				parentInteractionId : this.parentInteractionId,
				campaign: this.getCampaign(),
				account: this.getAccount(),
				destination: this.contactAddress || this.completeAddress.User
			};
			inConcert.invoke({
				type: "POST",
				url: url,
				data: JSON.stringify(request),
				dataType: "json",
				contentType: "application/json",
				success: function() {
					this[parameters.method](parameters);
				},
				error: function(_, error) {
					inConcert.logger.action({action_name : "create_twitter", error_message : error.result });
					if (typeof(parameters.callback) !== "function") {
						return;
					}

					var message;
					switch (error.result) {
						case "CHATSERVER_NOT_AVAILABLE":
							message = "<%=_T('Chatserver not available')%>";
							break;

						case "NOT_LICENSED":
							message = "<%=_T('Unable to send tweet. Not has licensed for the Twitter channel.')%>";
							break;

						case "INTERACTION_ALREADY_EXISTS":
							message = "<%=_T('There was an error while creating the conversation, please try again later')%>";
							break;

						case "AGENT_UNAVAILABLE":
							message = "<%=_T('There was an error while creating the conversation, please try again later')%>";
							break;

						case "FAIL_CREATING_OUTGOING":
							message = "<%=_T('There was an error while creating the conversation, please try again later')%>";
							break;

						case "INTERACTION_CANCELED_BEFORE_TAKING":
							message = "<%=_T('The conversation was cancelled')%>";
							break;

						case "FORBIDDEN":
							message = "<%=_T('You not have permission for create outgoing tweet conversation.')%>";
							break;

						default:
							message = "<%=_T('Unable to update status, please verify how many tweets you have assigned.')%>";
					}
					parameters.callback(false, message, "FATAL");
				}
			});
		}
	};

	TwitterInteraction.prototype.tweet = function(parameters) {
		let sanitizedMessage = this.SanitizeMessage(parameters.status);

		inConcert.logger.action({action_name: "twitter_tweet", id: this.id, type: this.type, campaign: this.campaign,
			message: parameters.status, sanitizedMessage: sanitizedMessage, account: this.account, action_type: "conversation"});

		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/tweet/";
		let request = {
			interactionID : this.id,
			campaignId    : this.campaign,
			status        : sanitizedMessage,
			attachment_url: parameters.attachment_url,
			account       : this.account
		};
		this.post(url, request, parameters.callback);
	};

	TwitterInteraction.prototype.replyTweet = function(parameters) {
		let sanitizedMessage = this.SanitizeMessage(parameters.status);

		inConcert.logger.action({action_name : "twitter_reply_tweet", id : this.id, type : this.type,
			campaign : this.campaign, message: parameters.status, sanitizedMessage: sanitizedMessage, account : this.account, action_type : "conversation"});

		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/reply_tweet/";
		let request = {
			interactionID         : this.id,
			campaignId            : this.campaign,
			status                : sanitizedMessage,
			replyTo               : parameters.tweet_id,
			exclude_reply_user_ids: parameters.exclude_reply_user_ids,
			account               : this.account
		};
		this.post(url, request, parameters.callback);
	};

	/**
	event_count is computed on the server. It does not include automatic responses.
	*/
	TwitterInteraction.prototype.getEventCount = function(){
		// IB:
		return this.properties.event_count;
	};

	TwitterInteraction.prototype.getReadEventsCount = function() {
		var self = this;
		if (!this.readers){
			return 0;
		}
		var user = inConcert.app.getCompleteId();
		return _.reject(this.readers[user], function(index){
			return self.getEvent(index).creator === "automatic";
		}).length;
	};

	/*
		Retweets a specific tweet id and relates it to the current conversation
	*/
	TwitterInteraction.prototype.reTweet = function(parameters) {
		inConcert.logger.action({action_name : "twitter_retweet", id : this.id, type : this.type,
			campaign : this.campaign, tweetId : parameters.tweet_id, account : this.account, action_type : "conversation"});

		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/retweet/";
		let request = {
			interactionID: this.id,
			campaignId   : this.campaign,
			tweet_id     : parameters.tweet_id,
			account      : this.account
		};
		this.post(url, request, parameters.callback);
	};

	TwitterInteraction.prototype.sendDirectMessageFromTweet = function(parameters) {
		this.assert_inflated();
		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/send_dm_from_tweet/";
		let targetUserId = this.getEventUserId(parameters.index);
		if (!targetUserId) {
			return parameters.callback(false, "<%=_T('User id not found')%>");
		}
		let sanitizedMessage = this.SanitizeMessage(parameters.message);
		let request = {
			interactionID : this.id,
			campaignId    : this.campaign,
			target_user   : parameters.targetUser,
			target_user_id: targetUserId,
			message       : sanitizedMessage,
			account       : this.account,
			tweet         : parameters.index !== undefined? this.events[parameters.index] : null
		};
		this.post(url, request, parameters.callback);
	};

	TwitterInteraction.prototype.sendDirectMessage = function(parameters) {
		inConcert.logger.action({action_name : "twitter_direct_message", id : this.id, type : this.type,
			campaign : this.campaign, targetUser : parameters.targetUser, account : this.account, action_type : "conversation"});

		if (parameters.index !== undefined && !this.isDirectMessage(parameters.index)) {
			return this.sendDirectMessageFromTweet(parameters);
		}

		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/send_dm/";
		let targetUserId = this.getEventUserId(parameters.index);
		if (!targetUserId) {
			return parameters.callback(false, "<%=_T('User id not found')%>");
		}

		let sanitizedMessage = this.SanitizeMessage(parameters.message);
		let request = {
			interactionID : this.id,
			campaignId    : this.campaign,
			target_user   : parameters.targetUser,
			target_user_id: targetUserId,
			message       : sanitizedMessage,
			account       : this.account
		};
		this.post(url, request, parameters.callback);
	};

	TwitterInteraction.prototype.__sort_by_creation_date = function() {
		var self = this;
		if (!this.events || !this.events.length || !this.interactionData) {
			return;
		}
		//first sort by date
		TwitterInteraction.uber.__sort_by_creation_date.call(this);
		//for pure dm interactions it's ok
		if (this.interactionData.subtype === "DIRECTMESSAGE") {
			$.each(this.events, function(index, event) {
				if (index > 0) {
					event.topLevelMessage = true;
				}
			});
			return;
		}
		//for tweets, sort by reply
		//direct messages are grouped by user
		var first = null;
		var dms = {}, dms_sorted = [];
		var replies = {}, replies_sorted = [], replies_threads = {};
		var badTweets = [];

		$.each(this.events, function(index, event) {
			//first
			if (!first) {
				first = event;
				return;
			}
			//dm
			if (self.isDirectMessage(index)) {
				var threadId = event.creator === "contact"? event.sender_id_str : event.recipient_id_str;
				if (threadId) {
					if (!dms[threadId]) {
						dms[threadId] = [];
						dms_sorted.push(dms[threadId]);
						event.topLevelMessage = true;
					}
					dms[threadId].push(event);
				}
				else {
					badTweets.push(event);
				}
				return;
			}
			//reply
			var replyId = event.in_reply_to_status_id_str || event.quoted_status_id_str;
			if (replyId) {
				//first tweet reply
				if (replyId === first.id_str) {
					replies[event.id_str] = [];
					replies[event.id_str].push(event);
					replies_threads[event.id_str] = replies[event.id_str];
					replies_sorted.push(replies[event.id_str]);
					event.topLevelMessage = true;
				}
				//replies for a reply
				else {
					if (replies_threads[replyId]) {
						replies_threads[replyId].push(event);
						replies_threads[event.id_str] = replies_threads[replyId];
					}
					else {
						badTweets.push(event);
					}
				}
			}
			else {
				badTweets.push(event);
			}
		});

		this.events = [];
		this.events.push(first);
		$.each(replies_sorted, function(_, thread) {
			$.each(thread, function(_, event) {
				self.events.push(event);
			});
		});
		$.each(dms_sorted, function(_, thread) {
			$.each(thread, function(_, event) {
				self.events.push(event);
			});
		});
		$.each(badTweets, function(_, event) {
			self.events.push(event);
		});
	};

	TwitterInteraction.prototype.getReplyReceipments = function(eventIndex) {
		var self = this;
		var result = {
			sender : {},
			mentions : []
		};
		var event = this.events[eventIndex];
		if (!event) {
			return result;
		}
		var accountName = this.getAccountDisplayName();
		result.sender = {
			id : event.user.id_str,
			name : event.user.name,
			user : event.user.screen_name,
			avatar : this.getEventUserAvatarURI(event)
		};
		$.each(event.entities && event.entities.user_mentions || [], function(_, mention) {
			if (mention.screen_name === accountName || mention.screen_name === result.sender.user) {
				return;
			}
			result.mentions.push({
				id : mention.id_str,
				name : mention.name,
				user : mention.screen_name,
				avatar : self.getEventUserAvatarURI(event)
			});
		});
		return result;
	};

	TwitterInteraction.prototype.isLiked = function (index) {
		let event = this.getEvent(index);
		let likes = this.interactionData && this.interactionData.likes || {};
		return event && event.id_str && likes[event.id_str] || false;
	};

	TwitterInteraction.prototype.like = function(like, event_index, callback) {
		let self = this;
		callback = callback || $.noop;
		let fake_event_id = inConcert.Utils.generateGuid();

		inConcert.logger.action({
			action_name : "twitter_like",
			id : this.id,
			type : this.type,
			campaign : this.campaign,
			fake_event_id: fake_event_id
		});

		let event = this.getEvent(event_index);
		if (!event || !event.id_str) {
			inConcert.logger.action({ action_name : "twitter_like", error_message : "Event not found" });
			return callback(false);
		}

		let url = this.m_manager.getBaseUrl() + "/interactions/twitter/like/";
		let request = {
			fake_event_id : fake_event_id,
			interactionId : this.id,
			campaign      : this.campaign,
			account       : this.account,
			objectId      : event.id_str,
			like          : like
		};
		inConcert.invoke({
			type: "POST",
			url: url,
			data: JSON.stringify(request),
			dataType: "json",
			contentType: "application/json",
			success: function() {
				self.interactionData.likes = self.interactionData.likes || {};
				self.interactionData.likes[event.id_str] = like;
				callback(true);
			},
			error: function(_, error){
				inConcert.logger.action({ action_name : "twitter_like", error_message : error.result });
				callback(false);
			}
		});

		return fake_event_id;
	};

	TwitterInteraction.prototype.getEventUserlIdByChannel = function(index) {
		return this.getEventScreenName(index);
	};

	if (typeof inConcert.interactions.twitter !== "function") {
		inConcert.interactions.twitter = TwitterInteraction;
	}
}());
