
/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name VoicemailInteraction
 */
(function () {

	"use strict";
	
	const format = inConcert.Utils.format;

	/**
	 * Class VoicemailInteraction.
	 * Subclass for voicemail conversations.
	 *
	 * @param {Object} rawInteraction
	 */
	function VoicemailInteraction (rawInteraction)
	{
		VoicemailInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
	}

	inConcert.Utils.inherit(VoicemailInteraction, inConcert.interactions.telephony);

	if (typeof inConcert.interactions.voicemail !== "function") {
		inConcert.interactions.voicemail = VoicemailInteraction;
	}

	VoicemailInteraction.prototype.isRecordEnabled = function() {
		return false;
	};

	VoicemailInteraction.prototype.isStopRecordEnabled = function() {
		return false;
	};

	VoicemailInteraction.prototype.isLive = function() {
		return false;
	};

	VoicemailInteraction.prototype.isRead = function( /*event_index*/ ) {
		return this.read;
	};

	VoicemailInteraction.prototype.getVoicemailUrl = function() {
		if (inConcert.app.has_allegro_integration()) {
			return format(
				"{0}/inconcert/apps/allegro/repository/stream/?id={1}&section={2}&part={3}",
				inConcert.app.getBaseDomain(),
				this.id,
				-1,
				1
			);
		}
		else if (this.interactionData.voicemailRecord) {
			return format(
				"{0}/inconcert/apps/repository/download/?id={1}{2}&section={3}&part={4}",
				inConcert.app.getBaseDomain(),
				this.interactionData.voicemailRecord.id,
				this.hasVideo() ? "_video" : "",
				this.interactionData.voicemailRecord.section,
				this.interactionData.voicemailRecord.part
			);
		}
		// por defecto
		return "";
	};
}());

