/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name WebContactInteraction
 */
(function () {

	"use strict";

	/**
	 * Class WebContactInteraction.
	 * Subclass of Interaction
	 * @param {Object} rawInteraction
	 */
	function WebContactInteraction ( rawInteraction ) {
		WebContactInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
		this.m_pending_relationship_interaction = null; // client side pending relationship conversation
	}

	inConcert.Utils.inherit( WebContactInteraction, inConcert.interactions.base );

	WebContactInteraction.prototype.dispose = function() {
		WebContactInteraction.uber.dispose.call( this );

		this.removeRelationshipPendingInteraction();
	};

	WebContactInteraction.prototype.getAddress = function()	{
		// TO-DO: Check if returning the first address is ok
		if(this.interactionData && this.interactionData.contactAddresses && this.interactionData.contactAddresses.length > 0){
			return this.interactionData.contactAddresses[0].address;
		}
		return this.contactAddress;
	};

	WebContactInteraction.prototype.getAddressType = function()	{
		// TO-DO: Check if returning the first address type is ok
		if(this.interactionData && this.interactionData.contactAddresses && this.interactionData.contactAddresses.length > 0){
			// Tweety sends this address all in uppercase, I don't really know how to handle this type.
			return inConcert.Utils.capitaliseFirstLetter(this.interactionData.contactAddresses[0].type);
		}
	};

	WebContactInteraction.prototype.getDisplayAddress = function() {
		return this.getAddress();
	};
	
	WebContactInteraction.prototype.__getReportDefaultField = function()
	{
		return "contactform";
	};

	WebContactInteraction.prototype.addRelationshipPendingInteraction = function( interaction )	{
		// client side pending relationship, no deberia persistir en el server
		if ( typeof interaction !== "object" ) {
			return;
		}
		this.m_pending_relationship_interaction = {
			id      : interaction.id      ,
			proxy   : interaction.type    ,
			account : interaction.account
		};
		interaction.addRelationshipPendingAppointment( this );
	};

	WebContactInteraction.prototype.removeRelationshipPendingInteraction = function() {
		// client side pending relationship, no deberia persistir en el server
		if ( !this.hasRelationshipWithPendingInteraction() ) {
			return;
		}
		var interaction_id = this.m_pending_relationship_interaction.id;
		this.m_pending_relationship_interaction = null;
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( this );

		var im = inConcert.app.getInteractionManager();
		var interaction = im.findInteraction( interaction_id );
		if ( interaction ) {
			interaction.removeRelationshipPendingAppointment();
		}
	};

	WebContactInteraction.prototype.getRelationshipPendingInteraction = function() {
		// client side pending relationship, no deberia persistir en el server
		return this.m_pending_relationship_interaction;
	};

	WebContactInteraction.prototype.hasRelationshipWithPendingInteraction = function() {
		return this.m_pending_relationship_interaction !== null;
	};

	WebContactInteraction.prototype.getExtendedInfo = function( max_length )
	{
		var extended_info = "";
		var fields = this.getReportFields();

		$.each( fields || {}, function( i, field )
		{
			var separator    = ( i > 0 ) ? " ; " : "";
			var display_name = field.displayName ? ( field.displayName + " : " ) : "";
			var value        = field.value ? field.value  : "";

			extended_info    = extended_info.concat( separator, display_name, value );
		});

		return inConcert.Utils.truncateWord( extended_info, max_length );
	};

	WebContactInteraction.prototype.eventsAreFromTheSameClient = function() {
		return true;
	};


	/**
	* Webcontacts don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	WebContactInteraction.prototype.getEventList = function()
	{
		return null;
	};

	WebContactInteraction.prototype.eventsAreFromTheSameClient = function() {
		return true;
	};

	WebContactInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	WebContactInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	WebContactInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};

	if (typeof inConcert.interactions.webcontact !== "function") {
		inConcert.interactions.webcontact = WebContactInteraction;
	}
}());




