/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50 */

/* global $, inConcert */

/*
Handling of the overlaying popup with the labels section of each interaction
*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var format = inConcert.Utils.format;

	/* ////////////////////////////////////////////////////////////////////////
	Funciones genericas utilizadas por las clases de "LabelsPopup" y "LabelsManagement"

	*/ ////////////////////////////////////////////////////////////////////////

	/*******************************************************************
	 * funcion que recibe un hash y a partir de el retorna un array ordenado por su clave
	 */
	var sortHash = function(hash) {
		var keys = [];
		for (var key in hash) {
			if (hash.hasOwnProperty(key)) {
				keys.push(key);
			}
		}
		return keys.sort();
	};

	/* ////////////////////////////////////////////////////////////////////////
	Clase que administra los labels:
		- Obtiene el markup de los labels para los owner (quien va a mostrar el label).
		- Acciones sobre las etiquetas mostradas en los owner.
	*/ ////////////////////////////////////////////////////////////////////////

	function LabelsManagement(data) {
		this.data = data;
	}

	if (typeof(inConcert.LabelsManagement) !== "function") {
		inConcert.LabelsManagement = LabelsManagement;
	}

	/*******************************************************************
	 * Esta funcion recibe los labels pertenecientes a la interaccion y los ordena segun su peso
	 * primero las pendientes, luego las custom y por ultimo las de integracion
	 */
	LabelsManagement.prototype.setData = function(data) { this.data = data; };

	/*******************************************************************
	 * Esta funcion recibe los labels pertenecientes a la interaccion y los ordena segun su peso
	 * primero las pendientes, luego las custom y por ultimo las de integracion
	 */
	LabelsManagement.prototype.sortInteractionOwnerLabels = function(interactionLabels)
	{
		if (!interactionLabels) {
			return;
		}
		var orderedInteractionLabels = {};
		var sortedList = sortHash(interactionLabels);
		//recorro el array ordenado
		for (var i = 0; i < sortedList.length; i++) {
			var key = sortedList[i]; //obtengo la key
			var interactionLabel = interactionLabels[key]; //a partir de la key lo busco en el hash
			orderedInteractionLabels[key] = interactionLabel;
		}
		return orderedInteractionLabels;
	};

	/*******************************************************************
	*Parameters:
	*		name				required		etiqueta a mostrar en el label.
			className			optional		clase a aplicar utilizada para los labels externos.
			key					optional		es la clave que se agrega en attr "key" del markup,
												y se utiliza para identificar el label en cuestion.
			tooltip				optional		es el tooltip a mostrar.
	*/
	LabelsManagement.prototype.getSingleLabelMarkup = function(lbl)
	{
		if (!lbl) {
			throw("[LabelsManagement.getSingleLabelMarkup] lbl is required.");
		}

		var className = function()
		{
			var class_name = lbl.className;
			if (class_name) {
				// por ejemplo para el caso de dispositions que tiene sus propias clases
				return class_name;
			}
			class_name = "interactionCustomTag";
			if ( lbl.removed ) {
				class_name = class_name + " interactionRemovedTag";
			}
			return class_name;
		}();

		var key = (lbl.key || inConcert.Utils.generateGuid()).toUpperCase();
		var name = lbl.name || key;
		var tooltip = lbl.tooltip === "" ? "" : lbl.tooltip || name;

		const styleColor = lbl.color? " style='background-color:"+ lbl.color +";'":"";

		return "".concat(
			"<div key='", key, "' tooltip='", tooltip, "' class='interactionTagItem ", className, "'"+ styleColor +">",
				"<div class='interactionTagText'>", name, "</div>",
			"</div>");
	};

	/*******************************************************************
	 * Esta funcion retorna el markup de los labels para la interaccion de quien ha abierto el dialogo
	 * (el inbox o un cliente en particular "mail", "twitter", etc)
	 */
	LabelsManagement.prototype.getOwnerLabelMarkup = function(interaction)
	{
		var self = this;
		if (!interaction) {
			return '';
		}
		var campaign_labels = {};
		var campaign = inConcert.app.getCampaign(interaction.campaign);
		if (campaign) {
			campaign.loadCustomLabels(function(camp_labels, error)
			{
				if (error) {
					return;
				}
				campaign_labels = camp_labels;
			});
		}
		var labelMarkup = "";
		var buildMarkup = function(interaction_labels)
		{
			$.each(interaction_labels, function(key, int_label)
			{
				var removed = !campaign_labels.hasOwnProperty(key.toUpperCase());
				var camp_label = campaign_labels[key.toUpperCase()] || {};
				var label_name = camp_label.name || int_label.name;
				var label_color = camp_label.color || int_label.color || "#FFF5EB";
				var label_tooltip = label_name;

				labelMarkup += self.getSingleLabelMarkup({
					name    : label_name,
					key     : key,
					tooltip : label_tooltip,
					removed : removed,
					color   : label_color
				});
			});
		};
		var labels = interaction.getLabels();
		var orderedInteractionLabels = self.sortInteractionOwnerLabels(labels);
		if (!orderedInteractionLabels) {
			return labelMarkup;
		}
		buildMarkup(orderedInteractionLabels);
		return labelMarkup;
	};

	/*******************************************************************
	 * Es el evento click de la cruz de eliminar un label de quien ha abierto el dialogo
	 * (el inbox o un cliente en particular "mail", "twitter", etc)
	 */
	LabelsManagement.prototype.deleteLabelOnInteractions = function(buttonDelete, interactions)
	{
		var self = this;
		if (!buttonDelete || !interactions) {
			return;
		}

		var mainWindow = inConcert.app.getMainWindow();
		var panel = mainWindow.getPanel("AssignedInteractions");
		var interactionManager = inConcert.app.getInteractionManager();

		var tagInfo = buttonDelete.parent();
		tagInfo.addClass("interactionDisabledTag");
		var labelId = tagInfo.attr("key");
		var labels = {};
		labels[labelId] = {
			id: labelId
		};

		var batchFunction = function( ints, lbls, onLabelsDeleted ) {
			return interactionManager.batchDeleteLabels( ints, lbls, onLabelsDeleted, self.from_app );
		};
		var args = [labels];
		var callbacks = {
			onFinished: function(){
				panel.setBatchButtonsEvents();
				panel.updateBatchButtonsStatus();
			},
			onPartial : function(successfulInteractions, unsuccessfulInteractions)
			{
				if (unsuccessfulInteractions && unsuccessfulInteractions.length > 0)
				{
					var errorMessage = format(
						"<%=_T('Failed to delete label on {0} {1}')%>",
						unsuccessfulInteractions.length,
						unsuccessfulInteractions.length === 1 ? "<%=_T('conversation')%>" : "<%=_T('conversations')%>"
					);
					inConcert.app.getMainWindow().showBannerMessage({message : errorMessage, messageType : "Error", timeout : 5000});
				}
				if (successfulInteractions){
					successfulInteractions.forEach(function(interaction){
						// change labels for current 'in memory' interactions.
						var interactions = interactionManager.getInteractions();
						if (interactions[interaction.id]){
							interactions[interaction.id].labels = interaction.current_labels;
						}
					});
				}
			},
			onFailure: function( err_type ) {
				err_type = ( err_type || "" ).toUpperCase();
				if ( err_type === "FORBIDDEN" ) {
					inConcert.app.getMainWindow().showBannerMessage({ message : "<%=_T('You have not permissions to delete label conversations.')%>", messageType : "Error", timeout : 5000});
				}
				else {
					self.displayStatusMessage("<%=_T('Error deleting labels on conversation')%>", "Error", undefined, 5000);
				}
				tagInfo.removeClass("interactionDisabledTag");
			}
		};
		var batchData = new panel.BatchData(batchFunction, interactions, args, callbacks);
		panel.processBatchInteractions(batchData);
	};


	/* ////////////////////////////////////////////////////////////////////////
	Clase que muestra el dialogo de labels:
		- Hace un show con labels del sistema y de la campana.
		- Agrega/Elimina labels a la campana.
		- Asignar/Elimina labels a las interacciones.
	*/ ////////////////////////////////////////////////////////////////////////

	function LabelsPopup( params ) {
		LabelsPopup.uber.constructor.call(this);
		this.labels = null;
		this.clickEventName = this.clickEventName + "label";
		this.ownerId = "windowLabelContainer";
		this.canReposition = true;
		this.labelManagement = new LabelsManagement();
		this.from_app = params ? params.from_app : "interaction";
	}

	inConcert.Utils.inherit(LabelsPopup, inConcert.PopupWindow);

	if (typeof(inConcert.LabelsPopup) !== "function") {
		inConcert.LabelsPopup = LabelsPopup;
	}

	/*******************************************************************
	 * funcion retorna puntero a la instancia del objeto 'LabelsManagement'
	 */
	LabelsPopup.prototype.getLabelManagementObject = function(){
		return this.labelManagement;
	};

	/*******************************************************************
	 * funcion que retorna el markup de un Label
	 */
	LabelsPopup.prototype.getLabelMarkup = function(label, removed)
	{
		var classes = 'interactionCustomLabel';
		if ( removed ) {
			classes = classes + " interactionRemovedLabel";
		}
		var label_id = label.id.toUpperCase();
		var ctrl_id = (this.from_app || 'unrecognized') + '_lbl_' + label_id;
		return ''.concat(
			'<div key="', label_id, '" class="interactionLabel ', classes , '">',
				'<input id="', ctrl_id, '" type="checkbox" class="selectedLabel" />',
				'<div label_color="true" class="assignLabelColor" style="background-color:', label.color || "#FFF5EB" ,'"></div>',
				'<label for="', ctrl_id, '" class="labelMessageText"></label>',
			'</div>');
	};

	/*******************************************************************
	 * funcion que muestra mensaje de error
	 */
	LabelsPopup.prototype.displayStatusMessage = function(text, type, closeDisabled, timeout) {
		var section = $("#labelErrorMessageSection");
		section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		section.show().find('.messageText').text(text);
		if (timeout) {
			window.setTimeout(function() {
				section.hide();
			}, timeout);
		}
		if (closeDisabled) {
			$("#labelCloseErrorHandler").hide();
		}
		else {
			$("#labelCloseErrorHandler").show();
		}
	};

	/*******************************************************************
	 * funcion que oculta mensaje de error
	 */
	LabelsPopup.prototype.hideStatusMessage = function() {
		$("#labelErrorMessageSection").hide();
	};

	/*******************************************************************
	 * funcion que agrega el markup del label en la lista
	 */
	LabelsPopup.prototype.addLabelMarkup = function(label, $container, selected, removed)
	{
		if (!label) {
			return false;
		}
		var label_id = label.id.toUpperCase();
		var jqueryExpression = '[key="' + label_id + '"]';
		this.labels = this.labels || {};
		if (!this.labels[label_id]) {
			this.labels[label_id] = label;
			$container.append(this.getLabelMarkup(label, removed));
			var removed_span = removed ?  "<span>" + "<%=_T('deleted')%>" + "</span>" : "";
			var label_name = label.name || label_id;
			$container.find(jqueryExpression + " > .labelMessageText").html( label_name + removed_span);
		}
		if (selected !== undefined) {
			$container.find(jqueryExpression + " > .selectedLabel").prop("checked", selected);
		}
		return true;
	};

	/*******************************************************************
	 * funcion que marca el label (checkbox)
	 */
	LabelsPopup.prototype.markLabelMarkup = function(label, $container, selected)
	{
		if (!label) {
			return;
		}
		var label_id = label.id.toUpperCase();
		var jqueryExpression = '[key="' + label_id + '"]';
		$container.find(jqueryExpression + ' > .selectedLabel').prop("checked", selected);
	};

	/*******************************************************************
	 * funcion que envia request para asignar labels en la interaccion
	 */
	LabelsPopup.prototype.assignLabelsToInteraction = function()
	{
		var self = this;

		var forbiddenMessage = "<%=_T('You have not permissions to label conversation.')%>";
		if ( !this.has_permission_to_save()) {
			inConcert.app.getMainWindow().showBannerMessage({ message : forbiddenMessage, messageType : "Error", timeout : 5000});
			this.close();
			return;
		}

		var current_interactions_labels = {};
		$.each( this.data.interactions, function(_, interaction) {
			$.each(interaction.getLabels() || {}, function(label_id) {
				current_interactions_labels[label_id.toUpperCase()] = interaction.id;
			});
		});

		var assign_labels = {}, remove_labels = {};
		// selecciono los labels que se van a asignar a la interaccion
		$.each($(".interactionLabel"), function(_, htmlLabel)
		{
			var $label = $(htmlLabel);
			var is_selected = $label.find(".selectedLabel").prop("checked");
			var label_id = $label.attr("key").toUpperCase();
			if (is_selected) {
				if (!current_interactions_labels[label_id]) {
					assign_labels[label_id] = {
						id  : label_id,
						name: $label.find(".labelMessageText").text().trim()
					};
				}
			}
			else {
				if (current_interactions_labels[label_id]) {
					remove_labels[label_id] = {
						id  : label_id,
						name: $label.find(".labelMessageText").text().trim()
					};
				}
			}
		});

		var mainWindow = inConcert.app.getMainWindow();
		var panel = mainWindow.getPanel("AssignedInteractions");
		var interactionManager = inConcert.app.getInteractionManager();

		var batchFunction = function( ints, labels, currentLabels, onInteractionsLabeled ) {
			return interactionManager.batchUpdateLabels( ints, labels, currentLabels, onInteractionsLabeled, self.from_app );
		};
		var args = [assign_labels, remove_labels];
		var callbacks = {
			onFinished: function(){
				panel.setBatchButtonsEvents();
				panel.updateBatchButtonsStatus();
				self.parent_on_label(true);
			},
			onPartial : function(successfulInteractions, unsuccessfulInteractions)
			{
				if (unsuccessfulInteractions && unsuccessfulInteractions.length > 0) {
					var errorMessage = format(
						"<%=_T('{0} {1} failed assigned label')%>",
						unsuccessfulInteractions.length,
						unsuccessfulInteractions.length === 1 ? "<%=_T('conversation')%>" : "<%=_T('conversations')%>"
					);
					inConcert.app.getMainWindow().showBannerMessage({ message : errorMessage, messageType : "Error", timeout : 5000});
				}
				if (successfulInteractions){
					successfulInteractions.forEach(function(interaction){
						// change labels for current 'in memory' interactions.
						var interactions = interactionManager.getInteractions();
						if (interactions[interaction.id]){
							interactions[interaction.id].labels = interaction.current_labels;
						}
					});
				}
			},
			onFailure: function( err_type ) {
				err_type = ( err_type || "" ).toUpperCase();
				if ( err_type === "FORBIDDEN" ) {
					inConcert.app.getMainWindow().showBannerMessage({message: forbiddenMessage, messageType: "Error", timeout: 5000});
				}
				else {
					self.displayStatusMessage("<%=_T('Error assign labels on conversation')%>", "Error", undefined, 5000);
				}
				self.parent_on_label(false);
			}
		};

		var batchData = new panel.BatchData(batchFunction, this.data.interactions, args, callbacks);
		panel.processBatchInteractions(batchData, this.close());
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	LabelsPopup.prototype.setEvents = function()
	{
		var self = this;

		//---------------
		// evento que realiza la asignacion del label
		$("#assignInteractionLabel").unbind('click').bind('click', function() {
			self.assignLabelsToInteraction();
		});

		var closeDialog = function() {
			self.close();
		};
		// evento que cierra el dialogo y no realiza cambios
		$("#cancelInteractionLabel").unbind('click').bind('click', closeDialog);
		// evento que cierra el dialogo y no realiza cambios
		$("#closeLabelDialog").unbind('click').bind('click', closeDialog);
		//---------------
		// evento que cierra el mensaje de error
		$("#labelCloseErrorHandler").unbind('click').bind('click', self.hideStatusMessage);
		//---------------
		// evento (LIVE) que se ejecuta cuando se chequea/deschequeda el checkbox
		//  por el momento solo aplica al label del sistema INTEGRATION
		$("#windowLabelContainer").off('change', '.selectedLabel').on('change', '.selectedLabel', function() {
			var checkbox = $(this);
			var personalizedContainer = checkbox.parent().find(".windowPersonalizedLabelFieldContainer");
			if (checkbox.prop("checked")) {
				personalizedContainer.show();
				personalizedContainer.find(".windowPersonalizedLabelField").focus();
			}
			else {
				personalizedContainer.hide();
			}
			$("#assignInteractionLabel").prop("disabled", false);
		});
	};

	/*******************************************************************
	* Marca los labels que esten asignados en la interaccion
	*/
	LabelsPopup.prototype.markInteractionLabels = function(campaign_labels)
	{
		campaign_labels = campaign_labels || {};
		var $container = this.data.threadContainer.find('.addLabelThreadContentScroll');

		for (var i = 0; i < this.data.interactions.length; i++) {
			var interaction_labels = this.data.interactions[i].getLabels();
			if (!interaction_labels){
				continue;
			}
			for (var key in interaction_labels) {
				if (interaction_labels.hasOwnProperty(key)) {
					var camp_label = campaign_labels[key.toUpperCase()];
					this.markLabelMarkup(camp_label, $container, true);
				}
			}
		}
	};

	/*******************************************************************
	 * Metodo que crea el markup de todos los Labels de la campana y los renderiza
	 */
	LabelsPopup.prototype.fill_campaign_labels = function(labels)
	{
		var $container = this.data.threadContainer.find('.addLabelThreadContentScroll');
		if (!$container) { // Related to issue: http://redmine.inconcert/issues/2402
			inConcert.logger.error({
				result: "LabelsPopup.prototype.fill_campaign_labels - Failed to find container: addLabelThreadContentScroll",
				container: this.data.threadContainer,
				container_type: typeof(this.data.threadContainer),
				self_data : this.data,
				self_data_type : typeof(this.data)
			});
			this.displayStatusMessage("<%=_T('Error loading labels')%>", "Error", undefined, 3000);
			return;
		}
		if (labels) {
			this.labels = this.labels ? this.labels : {};
			// a partir de un hash desordenado, obtengo un array ordenado alfabeticamente por la clave
			var sortedList = sortHash(labels);
			var i = 0;
			//recorro el array ordenado
			while (i < sortedList.length) {
				var key = sortedList[i]; //obtengo la key
				var label = labels[key]; //a partir de la key lo busco en el hash
				if (label && label.id) {
					this.addLabelMarkup(label, $container);
				}
				++i;
			}
		}
	};

	/*******************************************************************
	 * Agrega los labels de la interaccion que han sido removidos de la campana.
	 */
	LabelsPopup.prototype.addRemovedInteractionLabels = function(campaign_labels)
	{
		var self = this;
		var interaction = this.data.interactions[0];
		if (!interaction) {
			return;
		}
		var interaction_labels = interaction.getLabels();
		if (!interaction_labels){
			return;
		}
		var $container = this.data.threadContainer.find('.addLabelThreadContentScroll');

		var removed_interaction_labels, key;
		for (key in interaction_labels) {
			if (interaction_labels.hasOwnProperty(key)) {
				var interaction_label = interaction_labels[key];
				key = key.toUpperCase();
				if ( self.m_cached_campaign_system_labels && self.m_cached_campaign_system_labels[key] ) {
					continue;
				}
				if ( !campaign_labels[key] ) {
					removed_interaction_labels = removed_interaction_labels || {};
					removed_interaction_labels[key] = interaction_label;
				}
			}
		}

		if (removed_interaction_labels) {
			// a partir de un hash desordenado, obtengo un array ordenado alfabeticamente por la clave
			var sortedList = sortHash(removed_interaction_labels);
			var i = 0;
			//recorro el array ordenado
			while (i < sortedList.length) {
				key = sortedList[i]; //obtengo la key
				var label = removed_interaction_labels[key]; //a partir de la key lo busco en el hash
				var selected = true, removed = true;
				this.addLabelMarkup(label, $container, selected, removed);
				++i;
			}
		}
	};

	/*******************************************************************
	* Get campaign labels
	*/
	LabelsPopup.prototype.loadCampaignLabels = function()
	{
		var campaign = inConcert.app.getCampaign(this.data.campaign);
		if (!campaign) {
			return;
		}
		var self = this;
		campaign.loadSystemLabels(function(campaign_system_labels, error)
		{
			if (error) {
				self.displayStatusMessage("<%=_T('Error loading system campaign labels')%>", "Error", undefined, 3000);
				return;
			}
			self.m_cached_campaign_system_labels = campaign_system_labels;
		});

		// desde la app supervisor se visualizan todos los labels, los del sistema y los custom.
		var load_method_name = this.from_app === "supervisor" ? "loadLabels" : "loadCustomLabels";
		campaign[load_method_name](function(campaign_custom_labels, error)
		{
			if (error) {
				self.displayStatusMessage("<%=_T('Error loading custom campaign labels')%>", "Error", undefined, 3000);
				return;
			}
			self.fill_campaign_labels(campaign_custom_labels);
			self.markInteractionLabels(campaign_custom_labels);
			self.addRemovedInteractionLabels(campaign_custom_labels);
			self.toggle_not_configured_labels();
		});
	};

	/*******************************************************************
	 *
	 */
	LabelsPopup.prototype.toggle_not_configured_labels = function()
	{
		var $message = this.data.threadContainer.find('#label_no_configured');
		var $container = this.data.threadContainer.find('.addLabelThreadContentScroll');
		var $custom_labels = $container.find('.interactionCustomLabel');
		if ($custom_labels.length > 0) {
			$container.show();
			$message.hide();
		}
		else {
			$container.hide();
			$message.show();
		}
	};

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	LabelsPopup.prototype.createMarkup = function()
	{
		$("#windowLabelContainer").remove();


		//-------------------------------
		var $node = $(''.concat(
			"<div id='windowLabelContainer' class='internalPopUpContainer popUp380w formPopUp' hidden>",
				"<div class='internalPopUpOverlay'></div>" ,
				"<div class='internalPopUp popUp380w'>",
					// title
					"<div class='internalPopUpTitle'>",
						"<span>", "<%=_T('Assign Labels')%>", "</span>",
						/*"<button id='closeLabelDialog' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",*/
					"</div>",

					// content
					"<div class='internalPopUpMiddle pageFormContainer'>" ,
						// mensaje de error
						"<div id='labelErrorMessageSection' class='genericFormMessagesContainer messageError' style='display:none;'>",
							"<a id='labelCloseErrorHandler' class='messageClose' href='javascript:void(0);'>", "<%=_T('close')%>", "</a>",
							"<div class='messageContent'>",
								"<label class='messageText'></label>",
							"</div>",
						"</div>",

						// content
						"<div class='card no-card'>",
							"<div class='card-block'>",
								"<div class='row addLabelThreadContainer'>",
									"<div class='col-12 pl-0 pr-0'>",
										// '<div class="addLabelThreadContentWaiting" hidden>',
										// 	'<img src="/inconcert/apps/agent/images/blank.png" class="addLabelThreadContentWaitingIco" />',
										// 	'<div class="addLabelThreadContentWaitingOverlay"></div>',
										// '</div>',

										"<div class='addLabelThreadContentScroll'></div>",		
									"</div>",

									"<div id='label_no_configured' class='col-12' style='display:none'>",
										"<span class='inlineMessagesContainer messageInfo d-block windowLabelNoResults'>", "<%=_T('No configured labels')%>", "</span>",										
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",

					// MODAL FOOTER BUTTONS
					"<div class='internalPopUpButtons'>",
						"<button id='assignInteractionLabel' class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Apply')%>" + "</button>",
						"<button id='cancelInteractionLabel' class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>",
					"</div>",
					

				"</div>",
			"</div>"
			));
		$("body").append($node);

		return $node;
	};

	LabelsPopup.prototype.has_permission_to_save = function()
	{
		var self = this;
		if ( !this.data || !this.data.interactions ) {
			return false;
		}

		var has_permission = true;
		// recorro y valido si tiene permisos de label, y si al menos una interaccion, no tiene permisos, retorna false
		$.each( this.data.interactions, function( _, interaction ) {
			if ( !interaction.hasPermission( "LABEL", self.from_app )) {
				has_permission = false;
				return false; // break
			}
		});

		return has_permission;
	};

	/*******************************************************************
	 * Metodo que visualiza dialogo con Labels e inicializa eventos de la ventana!
	 */
	LabelsPopup.prototype.initialize = function(data)
	{
		this.labels = null;
		this.data.container = this.createMarkup();
		this.parent_on_label = this.data.callbacks.on_label || $.noop;
		this.data.threadContainer = this.data.container.find('.addLabelThreadContainer');
		this.labelManagement.setData(data);
		this.loadCampaignLabels();
		this.setEvents();
	};

	/*******************************************************************
	 * Metodo que cierra el dialogo con los labels
	 */
	LabelsPopup.prototype.close = function()
	{
		inConcert.logger.guiAction({action_name : "close_label_dialog", window_id: window.id});
		if (this.data && this.data.container) {
			this.data.container.remove();
		}
		if (!this.labelManagement) {
			return;
		}
		this.finalize();
	};
}());
