/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	function TeamMember(member) {
		jQuery.extend(true, this, member);
	}

	//Start a chat interaction with a team member
	TeamMember.prototype.chat = $.noop;

	// True if member is online
	//
	// Note: Empty state is considered offline,
	//       this happens if the user never accessed the application.
	TeamMember.prototype.isOnline = function() {
		return this.status && this.status.state !== "OFFLINE" && this.status.state !== "";
	};

	// retorna el nombre y el apellido
	TeamMember.prototype.get_name_and_surname = function() {
		return this.Name + (this.FirstSurname ? " " + this.FirstSurname : "");
	};

	// retorna el nombre completo ( primer y segundo nombre y primer y segundo apellido)
	TeamMember.prototype.get_full_name = function() {
		return this.Name +
			(this.LastName     ? " " + this.LastName     : "") +
			(this.FirstSurname ? " " + this.FirstSurname : "") +
			(this.LastSurname  ? " " + this.LastSurname  : "");
	};

	TeamMember.prototype.getLastLoggedInDate = function() {
		var statusDate = parseInt(this.status.lastStatusChangeTime * 1000, 10);
		var exactTimeInState, timeInState;
		if ( statusDate ) {
			exactTimeInState = inConcert.Utils.getTimeDifference(new Date(statusDate));
			timeInState      = inConcert.Utils.timeAgo(new Date(statusDate));
		}
		return {
			exactTimeInState : exactTimeInState || "<%=_T('Never')%>",
			timeInState: timeInState || "<%=_T('Never')%>",
			statusDate: statusDate
		};
	};

	TeamMember.prototype.getLastDateInCampaignStatus = function() {
		var statusDate = parseInt(this.lastTimeInCampaign * 1000, 10);
		var timeInState;
		if ( statusDate ) {
			timeInState = inConcert.Utils.timeAgo(new Date(statusDate));
		}
		return timeInState || "<%=_T('Never')%>";
	};

	TeamMember.prototype.getArchive = function(filter, onArchiveLoadedFn) {
		inConcert.logger.action({action_name : "get_member_archive", member : this.Id, action_type:"campaign"});
		var privateFilter = $.extend(true, {}, filter);
		privateFilter.query += ' agent:"' + this.Id + '"';
		inConcert.app.getInteractionManager().searchArchiveInteractions(privateFilter, onArchiveLoadedFn);
	};

	TeamMember.prototype.getAvatarURI = function() {
		return '' + inConcert.app.getBaseUrl() + "/users/get_avatar/" + this.Id + '';
	};

	TeamMember.prototype.getAttentionLevels = function(sort = false) {
		if (this.AttentionLevels && sort) {
			this.AttentionLevels = this.AttentionLevels.sort((a, b) => 
				a.distribution? -1 : b.distribution? 1 : a.name.localeCompare(b.name));
		}
		return this.AttentionLevels || [];
	};

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.TeamMember !== 'function') {
		inConcert.TeamMember = TeamMember;
	}
}());
