/*jshint bitwise:false, browser:true, curly:false, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true */
/*jshint quotmark:false, -W014, esversion: 11*/
/*global _, inConcert, CInconcertPlayer, $, Mustache*/

this.inConcert = this.inConcert || {};

(function () {
	"use strict";

	const format = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;
	// Estas dimensiones deben de coincidir con lo que se ha definido en "aplication.js" funcion "openLiveTelephoneWindow"
	const DEFAULT_WINDOW_WIDTH_PX = 950;
	const MIN_CALL_DURATION_SECONDS_TO_SHOW_PARTS_DISCLAIMER = 60;	//1 minute
	const MIN_SECONDS_FROM_CALL_END_TO_SHOW_PARTS_DISCLAIMER = 300;	//5 minutes

	//**** Global Phone Window ****
	
	/** Class PhoneClientWindow.
	@param interaction
	@param options
	*/
	function PhoneClientWindow(interaction, options) {
		this.m_confirmButton = "#confirmButton";
		this.m_cancelButton = "#cancelButton";
		this.m_interaction = interaction;
		PhoneClientWindow.uber.constructor.call(this, options);
		document.title = options.title || "Web Phone";
		this.m_phoneClient = inConcert.app.getInteractionManager().getChannelClient('CALL');
		this.disableCampaignAccountsControls();
	}

	inConcert.Utils.inherit(PhoneClientWindow, inConcert.ClientWindow);

	/** Window size without sidebar
	*/
	PhoneClientWindow.prototype.getWindowSizeOnSidebarShown = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};

	/** Window size with sidebar
	*/
	PhoneClientWindow.prototype.getWindowSizeOnSidebarHidden = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};
	
	/** Set control button state
	* @param  {jquery}		$button		-Required. Button to set state.
	* @param  {object}		config		-Required. Button config.
	*/
	PhoneClientWindow.prototype.__setControlButtonState = function($button, config) {
		$button
			.prop("disabled", config.disabled || false)
			.removeClass($button.prop("actionClass") || "")
			.addClass(config["class"] || "")
			.prop("actionClass", config["class"] || "")
			.attr("title", config.label)
			.html("<i></i>");
	};

	/** Set control buttons state
	* @param  {object}		confirm		-Required. Confirm button config.
	* @param  {object}		cancel		-Required. Cancel button config.
	*/
	PhoneClientWindow.prototype.setControlButtonsStates = function(confirm, cancel) {
		this.__setControlButtonState($(this.m_confirmButton), confirm);
		this.__setControlButtonState($(this.m_cancelButton), cancel);
	};

	PhoneClientWindow.prototype.getPhoneStatus = function() {
		return this.m_phoneClient?.status();
	};

	/** Show error message
	* @param  {string}		message			-Required. message.
	*/
	PhoneClientWindow.prototype.showError = function(message) {
		this.displayStatusMessage(message, "Error", false, 10000);
	};

	/** Show alert message
	* @param  {string}		message			-Required. message.
	*/
	PhoneClientWindow.prototype.showAlert = function(message, timeout_ms) {
		timeout_ms = timeout_ms || 1000;
		this.displayStatusMessage(message, "Alert", false, timeout_ms);
	};

	/** Show alert message
	* @param  {string}		message			-Required. message.
	*/
	PhoneClientWindow.prototype.showInfo = function(message, timeout_ms) {
		timeout_ms = timeout_ms || 1000;
		this.displayStatusMessage(message, "Info", false, timeout_ms);
	};

	/** Hide error message
	*/
	PhoneClientWindow.prototype.hideError = function() {
		this.hideStatusMessage();
	};
	
	/** Set window interaction
	* @param  {object}		interaction		-Optional. Interaction to set.
	*/
	PhoneClientWindow.prototype.setInteraction = function(interaction) {
		console.log("[PhoneClientWindow.setInteraction] this.m_interaction.id: " + this.m_interaction?.id +
			", this.m_interaction.state:" + this.m_interaction?.state +
			", param-> interaction.id: " + interaction?.id +
			", param-> interaction.state:" + interaction?.state
		);
		this.m_interaction = interaction;
		if (interaction) {
			interaction.setView(this);
		}
	};
	
	/** Get window interaction
	* @return  {object}					-Window interaction.
	*/
	PhoneClientWindow.prototype.getInteraction = function() {
		return this.m_interaction;
	};
	
	/** Window has interaction
	* @param  {object}		interaction		-Optional. Interaction to check.
	* @return {boolean}						-Window has interaction.
	*/
	PhoneClientWindow.prototype.hasInteraction = function(interaction) {
		return interaction && this.m_interaction && this.m_interaction.id === interaction.id;
	};

	/** Updates client view
	* @param  {object}		interaction		-Window interaction.
	*/
	PhoneClientWindow.prototype.clientUpdate = function(interaction, event_name) {
		let currentInteraction = interaction ? interaction : this.m_interaction;
		//console.log("[PhoneClientWindow.clientUpdate] currentInteraction: " + currentInteraction + ", event_name: " + event_name);
		if (!currentInteraction || currentInteraction.state === "CANCELED" || currentInteraction.state === "ABANDONED") {
			return;
		} 
		if (!this.m_initialized) {
			this.initialize();
		}
		this.updateContactDisplay(currentInteraction);
		if (!currentInteraction.isBeingComposed()) {
			this.disableCampaignAccountsControls();
		}
		this.setTitle();
		this.updateControls(currentInteraction, event_name);
		this.updateToolbarButtons();
	};

	/** Updates client view controls
	* @param  {object}		interaction		-Window interaction.
	*/
	PhoneClientWindow.prototype.updateControls = $.noop;

	/** Loads interaction in window
	* @param  {object}		interaction		-Window interaction.
	* @return {object}						-Window handler.
	*/
	PhoneClientWindow.prototype.switchTo = function(interaction) {
		// only switch if no existent one or existing is different
		try {
			if (interaction && !this.hasInteraction(interaction)) {
				this.setInteraction(interaction);
				this.initialize();
			}
			this.hideError();
			this.clientUpdate();
			this.show();
			return window.getWindowHandle();
		} catch (error) {
			console.log(error);
		}
	};
	
	/** Class EmptyCallClientWindow.
	@param interaction
	*/
	function EmptyCallClientWindow(interaction) {
		
		EmptyCallClientWindow.uber.constructor.call(this, undefined, {
			title   : "<%=_T('Call')%>",
			queued	: interaction && interaction.isQueued(),
			waiting : interaction && interaction.isWaiting()
		});

		$("body").addClass("callClientEmpty");
		this.switchTo(interaction);
	}

	inConcert.Utils.inherit(EmptyCallClientWindow, PhoneClientWindow);

	//****	Player Window (archive) ****
	
	/** Class PlayerClientWindow.
	@param interaction
	@param options
	*/
	function PlayerClientWindow(interaction) {
		const self = this;
		
		let archive_header_tpl_allegro = ''.concat(
			'<tr>',
				'<th><%=_T("Agent")%></th>',
				'<th><%=_T("Section")%></th>',
				'<th><%=_T("Part")%></th>',
				'<th><%=_T("Format")%></th>',
				'<th class="gridColumnActions"></th>',
			'</tr>'
		);
			
		let archive_header_tpl_occ = ''.concat(
			'<tr>',
				'<th><%=_T("Type")%></th>',
				'<th><%=_T("Part")%></th>',
				'<th><%=_T("Format")%></th>',
				'<th class="gridColumnActions"></th>',
			'</tr>'
		);
			
		let archive_header_tpl = inConcert.app.has_allegro_integration() ? 
			archive_header_tpl_allegro : archive_header_tpl_occ;
		
		let archive_row_tpl_allegro = ''.concat(
				'<tr section="{{ section }}" part="{{ part }}" agent="{{ agent }}" format="{{ format }}" class="{{ classes }}" >' +
					'<td>{{ agentText }}</td>' +
					'<td>{{ sectionText }}</td>' +
					'<td>{{ part }}</td>' +
					'<td>{{ format }}</td>' +
					'<td>' +
						'{0}' +
						'{1}' +
					'</td>' +
				'</tr>'
			);
			
		let archive_row_tpl_occ = ''.concat(
				'<tr part="{{ part }}" agent="{{ agent }}" file_type="{{ file_type }}" format="{{ format }}" class="{{ classes }}" >' +
					'<td>{{ agentText }}</td>' +
					'<td>{{ part }}</td>' +
					'<td>{{ format_prefix }}<%=_T(":")%> {{ format }}</td>' +
					'<td>' +
						'{0}' +
						'{1}' +
					'</td>' +
				'</tr>'
			);
			
		let archive_row_tpl = inConcert.app.has_allegro_integration() ? archive_row_tpl_allegro : archive_row_tpl_occ;
		
		self.templates = {
			archive_row_tpl : archive_row_tpl,
			archive_row_tpl_play : ''.concat(
				'<button type="button" action="play" class="buttonColorGray gridPlayerButton gridButPlay" title="<%=_T("Play")%>">' +
					'<i></i>' +
				'</button>'
			),
			archive_row_tpl_download : ''.concat(
				'<button type="button" action="download" class="buttonColorGray gridPlayerButton gridButDownload" title="<%=_T("Download")%>">' +
					'<i></i>' +
				'</button>'
			)
		};
		
		PlayerClientWindow.uber.constructor.call(self, undefined, {
			title   : "inConcert Player",
			archive : true
		});
		$("body").addClass("callClientPlayer");
		$(".playerPartsGridHeader").append(archive_header_tpl);
		this.switchTo(interaction);
		interaction.inflate(function() {
			interaction.getRecords(function(success, records) {
				if (!success) {
					self.displayStatusMessage("<%=_T('Error getting records')%>", "Error", false, 2000);
					return;
				}
				self.setMediaPlayer(records);
			});
		});
	}
	
	inConcert.Utils.inherit(PlayerClientWindow, PhoneClientWindow);

	/** Set media player
	* @param  {object}		sections		-Record sections.
	*/
	PlayerClientWindow.prototype.setMediaPlayer = function(sections) {
		const self = this;

		function setEmptyPlayer() {
			$("#noRecordsFoundMessage").show();
		}

		const $partContainer = $("#partContainer");
		$partContainer.show();
		
		if ($.isEmptyObject(sections)) {
			setEmptyPlayer();
			return;
		}
		
		const now = new Date();
		const now_seconds = now.getTime() / 1000;
		if(this.m_interaction.timestampDouble - this.m_interaction.startTimestamp >= MIN_CALL_DURATION_SECONDS_TO_SHOW_PARTS_DISCLAIMER && now_seconds - this.m_interaction.timestampDouble <= MIN_SECONDS_FROM_CALL_END_TO_SHOW_PARTS_DISCLAIMER) {
			$partContainer.prepend("<div class='inlineMessagesContainer messageWarning' style='margin-bottom:0'><strong><%=_T('Attention')%></strong> <%=_T('Recording files may not be available yet to start playback')%>.</div>");
		}

		let html = "";
		let listen = this.m_interaction.hasPermission("LISTEN_RECORD", this.from_app) && this.templates.archive_row_tpl_play || "";
		let download = this.m_interaction.hasPermission("DOWNLOAD_RECORD", this.from_app) && this.templates.archive_row_tpl_download || "";
		let attentionTime = !inConcert.app.has_allegro_integration() ? this.m_interaction.getAttentionTime() : 0;
		let agentNames = {};
		
		function getAgentName(agentAtVCC, callback) {
			if (!agentAtVCC) {
				return callback("");
			}
			let agentId = inConcert.Utils.getIdWithoutVCC(agentAtVCC);
			if (agentNames[agentId]) {
				return callback(agentNames[agentId]);
			}
			else if (agentId === inConcert.app.getLoggedUserId()) {
				let agentName = inConcert.app.getLoggedUserName();
				agentNames[agentId] = agentName;
				callback(agentName);
			}
			else {
				inConcert.app.getTeamMember(agentId, function(member) {
					let agentName = inConcert.app.getMemberName(member);
					agentNames[agentId] = agentName;
					callback(agentName);
				}, true);
			}
		}
		
		function getAgent(section) {
			if (inConcert.app.has_allegro_integration()) {
				return section.AgentName;
			}
			let agent = "";
			switch(section.Type) {
				case "Agent":
					agent = section.agent;
					break;
				case "TransferConsultation":
				case "ConferenceConsultation":
				case "Conference":
				case "ConferenceWithoutClient":
					agent = section.from;
					break;
				case "External":
					agent = "number";
					break;
				case "Bot":
					agent = "bot";
			}
			return inConcert.Utils.getIdWithoutVCC(agent);
		}
		
		function getAgentText(section, callback) {
			if (inConcert.app.has_allegro_integration()) {
				getAgentName(section.AgentId, function(agentName) {
					agentName = (!agentName || agentName.trim() === "")? section.agentName : agentName;
					section.AgentName = agentName;
					callback(agentName);
				});
				return;
			}
			//first get all names
			inConcert.Utils.asyncIterate(["agent", "from", "to"], function(key, callback) {
				getAgentName(section[key], function(agentName) {
					section[key + "Name"] = agentName;
					callback();
				});
			}, function() {
				let toName = section.toName || section.number;
				switch(section.Type) {
					case "Agent":
						return callback(format("<%=_T('Agent {0}')%>", section.agentName));
					case "TransferConsultation":
					case "ConferenceConsultation":
						return callback(format("<%=_T('Consultation from {0} to {1}')%>", 
							section.fromName, toName));
					case "Conference":
					case "ConferenceWithoutClient":
						if(section.conferenceParticipants) {
							let conferenceMsg = "Conference with {0}";
							let conferenceWithoutClientMsg = "Conference (after contact hangup) with {0}";
							let msg = section.Type === "Conference" ? conferenceMsg : conferenceWithoutClientMsg;
							let participantsStr = "";
							const ConferenceParticipants = function(agentName) {
								participantsStr += agentName;
							};
							for(const participant in section.conferenceParticipants) {
								if (section.conferenceParticipants.hasOwnProperty(participant)) {
									if(participantsStr.length) {
										participantsStr += ", ";
									}
									if(section.conferenceParticipants[participant].agent) {
										getAgentName(section.conferenceParticipants[participant].agent, ConferenceParticipants);
									}
									else if(section.conferenceParticipants[participant].number) {
										participantsStr += section.conferenceParticipants[participant].number;
									}
								}
							}
							return callback(format(_T(msg), participantsStr));
						}
						else {
							let conferenceMsg = "Conference with {0} and {1}";
							let conferenceWithoutClientMsg = "Conference with {0} and {1} (after contact hangup)";
							let msg = section.Type === "Conference" ? conferenceMsg : conferenceWithoutClientMsg;
							return callback(format(_T(msg), section.fromName, toName));
						}
						break;
					case "External":
						return callback(format("<%=_T('Contact and {0} (without agent)')%>", toName));
					case "Bot":
						return callback("<%=_T('Bot')%>");
					default:
						return callback("<%=_T('Record')%>");
				}
			});
		}

		inConcert.Utils.asyncIterate(sections, function(section, callback, i) {
			let previousSection = i > 0? sections[i -1].Section : null;
			let nextSection = i < sections.length - 1? sections[i + 1] : null;
			let currentSection = section.Section;
			let classes = "";
			if (previousSection !== currentSection) {
				classes += " firstPart";
				if (inConcert.app.has_allegro_integration()) {
					attentionTime += section.AttentionTime;
				}
			}
			if (nextSection !== currentSection) {
				classes += " lastPart";
			}
			if (!classes) {
				classes = "middlePart";
			}
			
			getAgentText(section, function(agentText) {
				html += format(Mustache.render(self.templates.archive_row_tpl, {
					agent : getAgent(section),
					section : section.Section,
					agentText : agentText,
					sectionText : section.Section,
					part : section.Part,
					format : section.Format,
					format_prefix : section.Video? "<%=_T('Video')%>" : "<%=_T('Audio')%>",
					listen : listen,
					download : download,
					classes : classes,
					file_type : section.Video? (section.Mode === "screen_recording" ? "screen_recording" : "video") : null
				}), listen, download);
				callback();
			});
		});

		$partContainer
				.find("tbody")
				.html(html)
				.find("button")
				.click(function() {
					let $row = $(this).parent().parent();
					let section = $row.attr("section");
					let part = $row.attr("part");
					let agent = $row.attr("agent");
					let format = $row.attr("format");
					let fileType = $row.attr("file_type");
					let action = $(this).attr("action");
					switch(action) {
						case "download":
							return self.downloadRecord(agent, section, part, format, fileType);
						case "play":
							return self.playRecord(agent, section, part, format, fileType);
					}
				}
		);

		if (this.m_interaction && this.m_interaction.__last_step && this.m_interaction.__last_step.timestampDouble) {
			let endDate = inConcert.Utils.userFormatDate(this.m_interaction.__last_step.timestampDouble * 1000);
			$("#phoneCallEndDate").find(".phoneCallDataValue").text(endDate);
		}

		$("#phoneCallAttentionTime").find(".phoneCallDataValue").text(inConcert.Utils.formatSecondsAsTime(attentionTime, {
			with_zero_hour: false,
			with_padding  : false
		}));

		if (this.m_interaction && self.m_interaction.interactionData && this.m_interaction.interactionData.DNIS) {
			$("#phoneCallDnis").find(".phoneCallDataValue").text(this.m_interaction.interactionData.DNIS);
		}
		else {
			$("#phoneCallDnis").hide();
		}
		
		$("#phoneDataContainer").show();
	};

	/** Open record download popup
	* @param  {string}		agent			-Record agent.
	* @param  {string}		section			-Record section.
	* @param  {string}		part			-Record part.
	* @param  {string}		fileFormat		-Record format.
	*/
	PlayerClientWindow.prototype.downloadRecord = function(agent, section, part, fileFormat, fileType) {
		let url = this.getRecordUrl(section, part, true, fileType) + "/" + 
			this.getRecordFileName(agent, section, part, fileFormat);
		window.open(url);
	};
	
	/** Plays record
	* @param  {string}		agent			-Record agent.
	* @param  {string}		section			-Record section.
	* @param  {string}		part			-Record part.
	* @param  {string}		fileFormat		-Record format.
	*/
	PlayerClientWindow.prototype.playRecord = function(agent, section, part, fileFormat, fileType) {
		const self = this;

		let url = self.getRecordUrl(section, part, false, fileType) + "&range=true";
		let player = new CInconcertPlayer({
			containerId : "mainPlayerContainer",
			url : url,
			fileType : fileType,
			fileFormat : fileFormat,
			success : function() {
				player.play();
			},
			error : function() {
				self.displayStatusMessage("<%=_T('Error playing record')%>", "Error", false, 2000);
			}
		});

		player.load();
		self.m_player = player;

		$("#partContainer")
				.find("tbody tr")
				.removeClass("selected")
				.filter(format("[section={0}][part={1}]", section || 1, part))
				.addClass("selected");
	};
	
	/** Get record filename
	* @param  {string}		agent			-Record agent.
	* @param  {string}		section			-Record section.
	* @param  {string}		part			-Record part.
	* @param  {string}		fileFormat		-Record format.
	*/
	PlayerClientWindow.prototype.getRecordFileName = function(agent, section, part, fileFormat) {
		let campaign = this.m_interaction.campaign && this.m_interaction.campaign.split("@")[0] || "";
		let datetime_aux = inConcert.Utils.convertLocalDateToTimezoneDate(new Date(this.m_interaction.startTimestamp* 1000) ,inConcert.app.m_user_timezone_object);
		let datetime= inConcert.Utils.getStringFromDate(datetime_aux, false, "full_datetime", "yyyy-mm-dd" ,"-");

		let date = datetime.split(" ")[0];
		let time = datetime.split(" ")[1].split(".")[0];
		let fileType = fileFormat;

		let filename = format("{0}_{1}_{2}_{3}_{4}_{5}.{6}",
			campaign, agent, date, time, section, part, fileType);
		return filename;
	};
	
	/** Get record url.
	* @param  {string}		section			-Record section.
	* @param  {string}		part			-Record part.
	* @param  {boolean}		download		-Indicates if url is for download or play.
	*/
	PlayerClientWindow.prototype.getRecordUrl = function(section, part, isDownload, fileType) {
		const urlBase = inConcert.app.getBaseDomain();
		if (inConcert.app.has_allegro_integration()) {
			// allegro
			if (isDownload) {
				return `${urlBase}/inconcert/apps/allegro/repository/download/?id=${this.m_interaction.id}&section=${section}&part=${part}`;
			}
			return `${urlBase}/inconcert/apps/allegro/repository/stream/?id=${this.m_interaction.id}&section=${section}&part=${part}`;
		}
		// nativo
		return urlBase +
			"/inconcert/apps/repository/download/?id=" +
			this.m_interaction.id +
			(fileType === "video"? "_video" : (fileType === "screen_recording" ? "_screen_recording" : "")) +
			"&section=" + part +
			"&part=" + part;
	};

	//****	Voicemail Player Window (archive) ****

	/** Class VoicemailPlayerClientWindow.
	@param interaction
	*/
	function VoicemailPlayerClientWindow (interaction) {
		VoicemailPlayerClientWindow.uber.constructor.call(this, undefined, {
				title   : "inConcert Player",
				archive : interaction.state !== "TAKEN"
		});
		$('body')
			.addClass('callClientPlayer')
			.addClass('callClientVoicemailPlayer');
		this.switchTo(interaction);
		this.setMediaPlayer();
		if (interaction.isQueued()) {
			$(".clientStateFlap").text("<%=_T('Queued')%>");
		}
	}

	inConcert.Utils.inherit(VoicemailPlayerClientWindow, PhoneClientWindow);
	
	/** Initialices media player.
	*/
	VoicemailPlayerClientWindow.prototype.setMediaPlayer = function() {
		const self = this;
		let url = this.getVoicemailUrl();
		let player = new CInconcertPlayer({
			containerId : "mainPlayerContainer",
			url : url,
			fileType : this.m_interaction.hasVideo()? "video" : null,
			fileFormat : this.m_interaction.hasVideo()? "webm" : "mp3",
			error : function() {
				self.displayStatusMessage("<%=_T('Error playing record')%>", "Error", false, 2000);
			}
		});
		player.load();
		this.m_player = player;

		let div = document.getElementById("downloadDivVoicemail");
		let button = document.getElementById("downloadButtonVoicemail");
		div.style.display = "block";
		button.href = url;
	};
	
	/** Get voicemail url.
	*/
	VoicemailPlayerClientWindow.prototype.getVoicemailUrl = function() {
		return this.m_interaction.getVoicemailUrl();
	};	//****	Monitor Player Window (supervisor) ****
	
	/** Class MonitorClientWindow.
	@param interaction
	*/
	function MonitorClientWindow(interaction) {
		this.m_monitoring = null;

		const self = this;

		// self.m_interaction set on switchTo
		MonitorClientWindow.uber.constructor.call(self, undefined, {
			title   : "<%=_T('Call Supervisor')%>",
			monitor : true
		});

		_.extend(self, {
			m_confirmButtonStates: {
				LISTEN: { label: "<%=_T('MONITOR')%>", 'class': '' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			},
			m_cancelButtonStates: {
				STOP_LISTEN: { label: "<%=_T('STOP MONITORING')%>", 'class': '' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			}
		});
		
		self.m_monitorModes = [
			{
				id : "listen",
				label : "<%=_T('Listen')%>",
				status : "<%=_T('Listening conversation')%>"
			},
			{
				id : "whisper",
				label : "<%=_T('Talk with agent')%>",
				status : "<%=_T('Talking with agent')%>"
			},
			{
				id : "conference",
				label : "<%=_T('Talk with both')%>",
				status : "<%=_T('Talking with both')%>"
			}
		];

		$("body").addClass("callClientPlayer");

		self.switchTo(interaction);
		self.loadOwnerInformation();

		self.m_interaction.inflate(function() {
			// setup monitoring timer, checks interaction state
			self.m_monitorTimer = window.setInterval(function() {
				self.m_interaction.inflate(function() {
					self.checkInteraction();
				});
			}, 3000);
			self.initializeMonitoring();
		});
	}

	inConcert.Utils.inherit(MonitorClientWindow, PhoneClientWindow);
	
	MonitorClientWindow.prototype.__setStatusText = function(text) {
		$("#phoneMonitorStatus")
			.text(text || "")
			.toggle(!!text);
	};

	/** Window size without sidebar
	*/
	MonitorClientWindow.prototype.getWindowSizeOnSidebarShown = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};

	/** Window size with sidebar
	*/
	MonitorClientWindow.prototype.getWindowSizeOnSidebarHidden = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};
	
	/** Invalidates window.
	* @param  {string}		msg				-Error message.
	* @param  {string}		msgType			-Error message type. Default: Alert.
	*/
	MonitorClientWindow.prototype.invalidateMonitoring = function(msg, msgType) {
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		$(this.m_confirmButton).off("click");
		$(this.m_cancelButton).off("click");
		if (this.m_monitorTimer) {
			window.clearInterval(this.m_monitorTimer);
			this.m_monitorTimer = null;
		}
		if (msg) {
			this.displayStatusMessage(msg, msgType || "Alert", true);
		}
		this.stopMonitoring();
	};
	
	/** Checks if interaction status is valid to monitor interaction.
	* @return  {boolean}						-Status is valid.
	*/
	MonitorClientWindow.prototype.checkInteraction = function() {
		if (!this.canMonitor()) {
			let message = this.m_interaction && (this.m_interaction.isFinished() || this.m_interaction.isWrappedUp())?
				"<%=_T('Call ends')%>" : "<%=_T('You can not monitor interaction')%>";
			this.invalidateMonitoring(message, "Alert");
		}
	};
	
	MonitorClientWindow.prototype.canMonitor = function() {
		return this.m_interaction && this.m_interaction.isTaken() && 
			this.m_interaction.hasPermission("SPY_CALL_LISTEN", this.from_app);
	};
	
	MonitorClientWindow.prototype.initializeMonitoring = function() {
		let self = this;
		if (!self.canMonitor()) {
			return self.invalidateMonitoring();
		}
		self.setControlButtonsStates(self.m_confirmButtonStates.LISTEN, self.m_cancelButtonStates.NOTHING);
		$(self.m_confirmButton)
			.off("click")
			.on("click", function() {
				self.showMonitoringOptions();
			});
		$(self.m_cancelButton).off("click");
	};
	
	MonitorClientWindow.prototype.__makeMonitoringOption = function(option) {
		let self = this;
		let $option = $("".concat(
			"<div option='" + option.id + "'  class='interactionMenuItem " + option.id + "AttachmentOption'>",
				option.label,
			"</div>"
		));
		$option.data("option", option);
		$option.click(function() {
			let option = $(this).data("option");
			window.setTimeout(function() {
				self.disposeMonitoringOptions();
				self.startMonitoring(option);
			}, 50);
		});
		return $option;
	};
	
	MonitorClientWindow.prototype.__showMonitoringOptions = function() {
		let self = this;
		if (!self.canMonitor()) {
			return self.invalidateMonitoring("<%=_T('You can not monitor interaction')%>", "Error");
		}
		const menu_id = "monitoring_menu_actions";
		$("#" + menu_id).remove();
		let $menu = $('<div id="' + menu_id + '" class="interactionMenuBlind" hidden></div>');
		$.each(self.m_monitorModes, function(_, option) {
			$menu.append(self.__makeMonitoringOption(option));
		});

		$('body').append($menu);
		this.monitoringMenu = new inConcert.PopupWindow({ownerId: menu_id});
		this.monitoringMenu.show(
		{
			ownerId       : menu_id,
			eventName     : menu_id,
			canReposition : true,
			container     : $menu,
			parentNode    : $(self.m_confirmButton),
			align         : "LEFT",
			callbacks: {
				onOpen: function() {
					$('#' + menu_id).show();
				},
				onClose: function( onClose ) {
					onClose = onClose || $.noop;
					if (self.monitoringMenu) {
						self.monitoringMenu.hide(function() {
							onClose();
						});
					}
					else {
						onClose();
					}
				}
			}
		});
	};

	MonitorClientWindow.prototype.showMonitoringOptions = function() {
		if ( this.monitoringMenu && this.monitoringMenu.isVisible() ) {
			this.disposeMonitoringOptions();
		}
		else {
			this.__showMonitoringOptions();
		}
	};

	MonitorClientWindow.prototype.disposeMonitoringOptions = function() {
		let self = this;
		if (this.monitoringMenu) {
			this.monitoringMenu.finalize(function() {
				self.monitoringMenu = null;
			});
		}
	};
	
	MonitorClientWindow.prototype.__makeMonitoringData = function(option) {
		return {
			type : option.id
		};
	};
	
	MonitorClientWindow.prototype.startMonitoring = function(option) {
		const self = this;
		if (!self.canMonitor()) {
			return self.invalidateMonitoring("<%=_T('You can not monitor interaction')%>", "Error");
		}
		self.hideError();
		self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);
		$(self.m_confirmButton).off("click");
		$("#callClientMonitoringLoading").show();
		let data = self.__makeMonitoringData(option);
		inConcert.app.getInteractionManager().startMonitoringInteraction(self.m_interaction, data,
			function(success, errorMessage, alreadyClosed) {
				if (success) {
					self.m_monitoring = option;
					self.__waitForAnswerMonitoring();
				}
				else {
					$("#callClientMonitoringLoading").hide();
					if (alreadyClosed) {
						self.invalidateMonitoring(errorMessage, "Alert");
					} 
					else {
						self.showError( errorMessage );
						self.initializeMonitoring();
					}
					return;
				}
			}
		);
	};
	
	MonitorClientWindow.prototype.__waitForAnswerMonitoring = function() {
		let self = this;
		inConcert.Utils.polling({
			interval : 500,
			timeout : 60 * 1000,
			onPolling : function() {
				if (!self.m_monitoring) {
					return false;
				}
				let status = self.m_phoneClient?.status();
				if (status === "OFFERING") {
					self.__answerMonitoring();
					return false;
				}
			},
			onTimeout : function() {
				self.invalidateMonitoring("<%=_T('Timeout answering call.')%>", "Error");
			}
		});
	};
	
	MonitorClientWindow.prototype.__answerMonitoring = function() {
		let self = this;
		self.m_phoneClient?.answer(self.m_interaction, function(ok) {
			$("#callClientMonitoringLoading").hide();
			if (!ok) {
				return self.invalidateMonitoring("<%=_T('Error answering call.')%>", "Error");
			}
			self.__setStatusText(self.m_monitoring.status);
			self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.STOP_LISTEN);
			$(self.m_cancelButton)
				.off("click")
				.on("click", function() {
					self.stopMonitoring(true);
				});
		});
	};
	
	/** Stops monitoring.
	*/
	MonitorClientWindow.prototype.stopMonitoring = function(fromButton) {
		if (!this.m_monitoring) {
			return;
		}
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		$(this.m_cancelButton).off("click");
		$("#callClientMonitoringLoading").show();
		inConcert.app.getInteractionManager().stopMonitoringInteraction(this.m_interaction);
		let phoneStatus = this.m_phoneClient?.status();
		switch(phoneStatus) {
			case "OFFERING":
				this.m_phoneClient?.reject(this.m_interaction);
				break;
			case "ACTIVE":
				this.m_phoneClient?.hangup(this.m_interaction);
				break;
		}
		$("#callClientMonitoringLoading").hide();
		this.__setStatusText();
		this.m_monitoring = null;
		if (fromButton) {
			this.initializeMonitoring();
		}
	};
	
	/** Stops monitoring on window closed.
	*/
	MonitorClientWindow.prototype.onWindowClosed = function() {
		this.stopMonitoring();
		MonitorClientWindow.uber.onWindowClosed.call(this);
	};
	
	//****	Allegro Monitor Player Window (supervisor) ****
	
	/** Class MonitorClientWindow.
	@param interaction
	*/
	function AllegroMonitorClientWindow(interaction) {
		this.m_monitoring = false;

		const self = this;

		// self.m_interaction set on switchTo
		AllegroMonitorClientWindow.uber.constructor.call(self, undefined, {
			title   : "<%=_T('Call Supervisor')%>",
			monitor : true
		});

		_.extend(self, {
			m_confirmButtonStates: {
				LISTEN: { label: "<%=_T('LISTEN')%>", 'class': '' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			},
			m_cancelButtonStates: {
				STOP_LISTEN: { label: "<%=_T('STOP LISTEN')%>", 'class': '' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			}
		});

		$("body").addClass("callClientPlayer");

		self.switchTo(interaction);
		self.loadOwnerInformation();

		if (self.checkInteraction()) {
			// setup monitoring timer, checks interaction state
			self.m_monitorTimer = window.setInterval(function() {
				self.m_interaction.inflate(function() {
					self.checkInteraction();
				});
			}, 3000);
			self.startMonitoring();
		}
	}

	inConcert.Utils.inherit(AllegroMonitorClientWindow, PhoneClientWindow);


	/** Window size without sidebar
	*/
	AllegroMonitorClientWindow.prototype.getWindowSizeOnSidebarShown = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};

	/** Window size with sidebar
	*/
	AllegroMonitorClientWindow.prototype.getWindowSizeOnSidebarHidden = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};
	
	/** Invalidates window.
	* @param  {string}		msg				-Error message.
	* @param  {string}		msgType			-Error message type. Default: Alert.
	*/
	AllegroMonitorClientWindow.prototype.invalidateWindow = function(msg, msgType) {
		if (this.m_invalid) {
			return;
		}
		if (this.m_player) {
			this.m_player.destroy();
		}
		$("#callClientMonitoring").hide();
		$("#callClientMonitoringLoading").hide();
		this.m_invalid = true;
		window.clearInterval(this.m_monitorTimer);
		this.displayStatusMessage(msg, msgType || "Alert", true);
	};
	
	/** Checks if interaction status is valid to monitor interaction.
	* @return  {boolean}						-Status is valid.
	*/
	AllegroMonitorClientWindow.prototype.checkInteraction = function() {
		const self = this;
		let isClosed = function() {
			return _.contains(['FINISHED', 'CANCELED', 'WRAPUP'], self.m_interaction.state);
		};
		if (isClosed()) {
			self.invalidateWindow("<%=_T('Call ends')%>");
			self.stopMonitoring();
			return false;
		}
		return true;
	};
	
	/** Starts monitor process.
	*/
	AllegroMonitorClientWindow.prototype.startMonitoring = function() {
		const self = this;

		self.hideError();
		self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);

		inConcert.app.getInteractionManager().startMonitoringInteraction (
			self.m_interaction,
			null,
			function(success, errorMessage, alreadyClosed) {
				if (!success) {
					if (alreadyClosed) {
						self.invalidateWindow(errorMessage, "Alert");
					} else {
						self.showError( errorMessage );
					}
					return;
				}
				self.m_monitoring = true;

				if (!self.m_interaction || !self.m_interaction.hasPermission("SPY_CALL_LISTEN", self.from_app)) {
					self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);
					return;
				}

				self.setControlButtonsStates(self.m_confirmButtonStates.LISTEN, self.m_cancelButtonStates.NOTHING);
				$(self.m_confirmButton)
					.off("click")
					.on("click", function() {
						self.startListening();
					});

				$(self.m_cancelButton)
					.off("click")
					.on("click", function() {
						self.stopListening();
					});
			}
		);
	};
	
	/** Stops monitoring.
	*/
	AllegroMonitorClientWindow.prototype.stopMonitoring = function() {
		inConcert.app.getInteractionManager().stopMonitoringInteraction(this.m_interaction);
		if (this.m_monitoring) {
			this.m_monitoring = false;
			this.stopListening();
			this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		}
	};
	
	/** Stops monitoring on window closed.
	*/
	AllegroMonitorClientWindow.prototype.onWindowClosed = function() {
		this.stopMonitoring();
		AllegroMonitorClientWindow.uber.onWindowClosed.call(this);
	};
	
	/** Gets spy url.
	* @param  {function}		callback	-Callback function.
	* @return {string}						-Spy url.
	*/
	AllegroMonitorClientWindow.prototype.getSpyUrl = function(callback) {
		const self = this;
		if (self.m_interaction.interactionData && self.m_interaction.interactionData.asteriskServerIP && 
				self.m_interaction.interactionData.asteriskServerIP !== ""
		) {
			var ip = self.m_interaction.interactionData.asteriskServerIP;
			let url = format(
				inConcert.app.getBaseDomain() + "/inconcert/apps/mediaserver/action/startStream/live/{0}&{1}",
				self.m_interaction.id, ip);
			callback(url);
		}
		else {
			inConcert.app.getAllegroAPI().getAllegroAsteriskIp(function( success, ip ) {
				if (!success) {
					return callback(false);
				}
				let url = format(
					inConcert.app.getBaseDomain() + "/inconcert/apps/mediaserver/action/startStream/live/{0}&{1}",
					self.m_interaction.id, ip);
				callback(url);
			});
		}
	};

	/** Starts monitor streaming.
	*/
	AllegroMonitorClientWindow.prototype.startListening = function() {
		const self = this;
		let $loading = $("#callClientMonitoringLoading");
		$loading.show();
		self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);
		self.getSpyUrl(function(url) {
			if (!url) {
				$loading.hide();
				self.invalidateWindow("<%=_T('Error playing record')%>", "Error");
				self.stopMonitoring();
				return;
			}
			let player = new CInconcertPlayer({
				containerId : "callClientMonitoring",
				url : url,
				fileFormat : "mp3",
				isLive : true,
				success: function() {
					$loading.hide();
					player.hide();
					self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.STOP_LISTEN);
				},
				error: function() {
					//wait for session close
					//allegro returns 200 when call close but without length then ends with an error
					window.setTimeout(function() {
						$loading.hide();
						self.invalidateWindow("<%=_T('Error playing record')%>", "Error");
						self.stopMonitoring();
					}, 5000);
				}
			});
			player.load();
			player.hide();
			self.m_player = player;
		});
	};

	/** Stops monitor streaming.
	*/
	AllegroMonitorClientWindow.prototype.stopListening = function() {
		this.setControlButtonsStates(this.m_confirmButtonStates.LISTEN, this.m_cancelButtonStates.NOTHING);
		if (this.m_player) {
			this.m_player.destroy();
		}
		this.close();
	};
	
	//****	Allegro Voicemail Monitor Player Window (supervisor) ****

	/** Class VoicemailMonitorClientWindow.
	@param interaction
	*/
	function VoicemailAllegroMonitorClientWindow(interaction)
	{
		this.m_monitoring = false;
		VoicemailAllegroMonitorClientWindow.uber.constructor.call(this, undefined, {
			title   : "<%=_T('Call Supervisor')%>",
			monitor : true
		});
		$('body').addClass('callClientPlayer');
		$("#attentionContainer").hide();
		this.switchTo(interaction);
		this.loadOwnerInformation();
		this.setMediaPlayer();
	}

	inConcert.Utils.inherit(VoicemailAllegroMonitorClientWindow, PhoneClientWindow);

	VoicemailAllegroMonitorClientWindow.prototype.setMediaPlayer = VoicemailPlayerClientWindow.prototype.setMediaPlayer;

	VoicemailAllegroMonitorClientWindow.prototype.getVoicemailUrl = VoicemailPlayerClientWindow.prototype.getVoicemailUrl;
	
	//****	Live Client Window (Agent) ****
	
	/** Class LiveClientWindow.
	@param interaction
	@param options
	*/
	function LiveClientWindow(interaction, options)
	{
		const self = this;

		console.log("[LiveClientWindow] START CREATING instance of class LiveClientWindow. interaction: " + this.m_interaction?.id || "NONE");

		// Flag utilizado para determinar si la ventana se debe minimizar al dar click fuera del area de la misma.
		// Cuando la ventana se esta creando se encuentra apagado.
		this.m_enabledMinimizeWhenOutside = false;

		const customInteractionToolbarButtons = {
			Conference: { 
				id    : "conferenceInteraction",
				method: function($button) { self.conferenceInteraction($button); }
			},
			Schedule: { 
				id    : "scheduleInteraction",
				method: function($button) { self.scheduleInteraction($button); }
			}
		};

		LiveClientWindow.uber.constructor.call(self, interaction, _.extend(options, {
			hideOnClose : true,
			customInteractionToolbarButtons: customInteractionToolbarButtons
		}));

		_.extend(this, {
			m_lastDialedNumber  : "",  // for redial
			m_lastDialedCampaign: "",  // for redial
			m_lastDialedContact: null, // for redial
			m_confirmButtonStates: {
				CALL: { label: "<%=_T('MAKE CALL')%>", 'class': 'butBigCall' },
				ANSWER: { label: "<%=_T('ANSWER')%>", 'class': 'butBigAnswer' },
				CONFIRM_TRANSFER: { label: "<%=_T('CONFIRM')%>", 'class': 'butBigOkTransfer' },
				CONFIRM_CONFERENCE: { label: "<%=_T('CONFIRM')%>", 'class': 'butBigOkConference' },
				CONFIRM_PREVIEW: { label: "<%=_T('ACCEPT')%>", 'class': 'butBigOkPreview' },
				REPLAY_VOICEMAIL: { label: "<%=_T('REPLAY')%>", 'class': 'butBigReplay' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			},
			m_cancelButtonStates: {
				HANGUP: { label: "<%=_T('HANG UP')%>", 'class': 'butBigHangUp' },
				WRAPUP: { label: "<%=_T('FINISH')%>", 'class': 'butBigWrapUp' },
				CANCEL_OUTBOUND: { label: "<%=_T('HANG UP')%>", 'class': 'butBigCancel' },
				CANCEL_TRANSFER: { label: "<%=_T('CANCEL')%>", 'class': 'butBigCancel' },
				CANCEL_CONFERENCE: { label: "<%=_T('CANCEL')%>", 'class': 'butBigCancel' },
				CANCEL_PREVIEW: { label: "<%=_T('REJECT')%>", 'class': 'butBigCancel' },
				CLOSE_VOICEMAIL: { label: "<%=_T('CLOSE')%>", 'class': 'butBigClose' },
				NOTHING: { label: "<%=_T('')%>", 'class': '', disabled: true }
			},
			m_actionButtons: {
				REDIAL: {
					id: "#redialButton",
					execute: function() {
						self.redial();
					}
				},
				HOLD: {
					id: "#holdButton",
					execute: function() {
						self.hold();
					}
				},
				UNHOLD: {
					id: "#unholdButton",
					execute: function() {
						self.unhold();
					}
				},
				MUTE: {
					id: "#muteButton",
					execute: function() {
						self.mute();
					}
				},
				UNMUTE: {
					id: "#unmuteButton",
					execute: function() {
						self.unmute();
					}
				},
				_SWITCH: {
					id: "#switchButton",
					execute: function() {
						self._switch();
					}
				},
				UNSWITCH: {
					id: "#unswitchButton",
					execute: function() {
						self.unswitch();
					}
				},				
				RECORD: {
					id: "#recordButton",
					execute: function() {
						self.record();
					}
				},
				STOP_RECORD: {
					id: "#stopRecordButton",
					execute: function() {
						self.stopRecord();
					}
				},
				SHOW_NUMERIC_KEYBOARD: {
					id: "#showNumericKeyboard",
					execute: function() {
						self.showNumericKeyboard();
					}
				},
				HIDE_NUMERIC_KEYBOARD: {
					id: "#hideNumericKeyboard",
					execute: function() {
						self.hideNumericKeyboard();
					}
				}
			},
			m_states : {
				CONNECTED: {
					bodyClassName: "phoneConnected",
					waiting: {
						mustShow: false
					}
				},
				ERROR: {
					bodyClassName: "phoneError",
					waiting: {
						mustShow: false
					}
				},
				DIALING: {
					bodyClassName: "phoneDialing",
					waiting: {
						mustShow: true,
						mustShowAnimation: true
					}
				},
				OFFERING: {
					bodyClassName: "phoneOffering",
					waiting: {
						mustShow: false
					}
				},
				PREVIEW: {
					bodyClassName: "phoneOffering",
					waiting: {
						mustShow: false
					}
				},
				REJECTED: {
					bodyClassName: "phoneCanceled",
					waiting: {
						mustShow: true,
						mustShowAnimation: false
					}
				},
				CANCELLED: {
					bodyClassName: "phoneCanceled",
					waiting: {
						mustShow: true,
						mustShowAnimation: false
					}
				},
				ACTIVE:	{
					bodyClassName: "callTalking",
					waiting: {
						mustShow: false
					}
				},
				WRAPUP: {
					bodyClassName: "callWrapup",
					waiting: {
						mustShow: false
					}
				},
				"TRANSFER TRYING": {
					bodyClassName: "callTransferDialing",
					waiting: {
						mustShow: true,
						mustShowAnimation: true,
						small: true
					}
				},
				"TRANSFER TAKEN": {
					bodyClassName: "callTransfer",
					waiting: {
						mustShow: false
					}
				},
				"INCOMING TRANSFER": {
					bodyClassName: "interactionTransfered",
					waiting: {
						mustShow: false
					}
				},
				"INCOMING TRANSFER TAKEN": {
					bodyClassName: "callTransfer",
					waiting: {
						mustShow: false
					}
				},
				"CONFERENCE TRYING": {
					bodyClassName: "callConference",
					waiting: {
						mustShow: false
					}
				},
				"CONFERENCE TAKEN": {
					bodyClassName: "callConference",
					waiting: {
						mustShow: false
					}
				},
				"INCOMING CONFERENCE": {
					bodyClassName: "interactionTransfered",
					waiting: {
						mustShow: false
					}
				},
				"INCOMING CONFERENCE TAKEN": {
					bodyClassName: "callConference",
					waiting: {
						mustShow: false
					}
				},
				"CONFERENCE": {
					bodyClassName: "callConference",
					waiting: {
						mustShow: false
					}
				}
			}
		});

		this.setEvents();
		this.setCachedHTMLInteractionToolbarButtons();

		const setView = function(/*windowPhone*/) {
			console.log(`[LiveClientWindow.prototype.constructor] 1-> setView - START. this.m_interaction: ${self.m_interaction}`);
			if (!self.m_interaction) {
				self.m_phoneClient.setView(null);
				// TODO: self.close(); ?
				return;
			}
			if (self.m_interaction.isGone()) {
				console.log("[LiveClientWindow.prototype.constructor] 1.2-> Interaction isGoned. interactionId: " + self.m_interaction.id + ", state: " + self.m_interaction.state + ", invoke this.m_phoneClient.setView(null)");
				self.m_phoneClient?.setView(null);
				self.close();
				return;
			}
			console.log("[LiveClientWindow.prototype.constructor] 1.3-> OpenWindows interactionId: " + self.m_interaction.id + ", state: " + self.m_interaction.state + ", invoke this.m_phoneClient.setView(this)");
			self.m_phoneClient?.setView(self);
		};
		//
		_.defer(_.bind(setView, self, self));
		console.log("[LiveClientWindow] END CREATED instance of class LiveClientWindow. interaction: " + this.m_interaction?.id || "NONE");
	}

	inConcert.Utils.inherit(LiveClientWindow, PhoneClientWindow);

	LiveClientWindow.prototype.initialize = function(options)
	{
		LiveClientWindow.uber.initialize.call(this, options);
		
		if (this.m_interaction && this.m_interaction.state === "PENDING") {
			this.updateActiveCallDialingAddress(this.m_interaction);
		}
	};

	/** Window size without sidebar
	*/
	LiveClientWindow.prototype.getWindowSizeOnSidebarShown = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};

	/** Window size with sidebar
	*/
	LiveClientWindow.prototype.getWindowSizeOnSidebarHidden = function() {
		return { width : DEFAULT_WINDOW_WIDTH_PX, height : null };
	};
	
	/** Avoid to close when cci is enabled
	*/
	LiveClientWindow.prototype.canCloseWindow = function(params)
	{
		const phoneStatus = this.m_phoneClient?.status();
		const myInt = (this.m_interaction || {});
		console.info("[LiveClientWindow.prototype.canCloseWindow]  1 -> Validating can close. interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
		if (!this.m_interaction) {
			console.info("[LiveClientWindow.prototype.canCloseWindow]  2 -> YES canClose - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
			return true;
		}
		if (params && params.interactive) {
			if (this.hasAnyIntegration()) {
				const interactionFound = inConcert.app.getInteractionManager().getInteractions()[myInt.id];
				if (interactionFound) {
					if (_.contains(['CANCELED', 'PENDING', 'ABANDONED',], this.m_interaction.state)) {
						console.info("[LiveClientWindow.prototype.canCloseWindow]  3 -> YES canClose 'hasAnyIntegration' - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
						return true;
					}
					console.info("[LiveClientWindow.prototype.canCloseWindow]  4 -> NO canClose 'hasAnyIntegration' - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
					return false;
				}
				else {
					console.info("[LiveClientWindow.prototype.canCloseWindow]  5 -> YES canClose 'hasAnyIntegration' - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus + ". Interaction NOT FOUND in Assigned Panel !!");
				}
			}
			if (this.m_interaction.isInternal() && this.m_interaction.state !== "CANCELED") {
				console.info("[LiveClientWindow.prototype.canCloseWindow]  6 -> NO canClose 'isInternal && not Canceled' - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
				return false;
			}
		}
		console.info("[LiveClientWindow.prototype.canCloseWindow]  7 -> YES canClose - interaction.id: " + myInt.id + ", state: " + myInt.state + ", phoneStatus: " + phoneStatus);
		return true;
	};

	LiveClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
			// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			transfer     : true,
			schedule     : true,
			conference   : true,
			queue        : true
			// estas NO tiene que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	LiveClientWindow.prototype.conferenceInteraction = function($button)
	{
		const cloned_interaction = inConcert.Utils.cloneObject(this.m_interaction);
		const params = {
			positionNode: $button
		};
		this.m_interaction_gui.execute_action("Conference", cloned_interaction, params);
	};

	LiveClientWindow.prototype.scheduleInteraction = function($button)
	{
		const cloned_interaction = inConcert.Utils.cloneObject(this.m_interaction);
		const params = {
			positionNode: $button
		};
		this.m_interaction_gui.execute_action("Schedule", cloned_interaction, params);
	};

	/** Clean and cancel pending interactions on closed
	*/
	LiveClientWindow.prototype.onWindowClosed = function() {
		const myInt = {};
		if (this.m_interaction) {
			myInt.id = this.m_interaction.id;
			myInt.state = this.m_interaction.state;
			myInt.initiative = this.m_interaction.initiative;
		}
		const phoneStatus = this.m_phoneClient ? this.m_phoneClient.status() : 'WITHOUT INSTANCE PHONE';
		console.log("[LiveClientWindow.prototype.onWindowClosed] -> interactionId: '" + myInt.id + "', state: '" + myInt.state + "', phoneStatus: '"  + phoneStatus + "', invoke this.m_phoneClient.setView(null)");
		if (this.m_interaction) {
			this.m_interaction.opened = false;
			this.m_interaction.setView(null);
			if ($.inArray(this.m_interaction.state, ["PENDING"]) >= 0) {
				// la invocación de la función cancelInteraction se debe efectuar unicamente para interacciones "pending" ya que sino el comportamiento del telefono es inconsistente.
				this.cancelInteraction();
			}
			else {
				inConcert.app.m_mainWindow.updateInteractions([this.m_interaction]);
			}
		}
		LiveClientWindow.uber.onWindowClosed.call(this);
		if (this.m_phoneClient) {
			this.m_phoneClient.onWindowClosed(myInt);
			this.m_phoneClient.setView(null);
		}
		this.is_closed = true;
	};

	/** Cancel interaction
	*/
	LiveClientWindow.prototype.cancelInteraction = function(event_name) {
		console.log("[LiveClientWindow.prototype.cancelInteraction] 1-> interactionId: " + (this.m_interaction || {}).id + ", event_name: " + event_name);
		let isCancelled = false;
		$("#contact_window_cancel_main").click();
		if (this.m_interaction) {
			console.log("[LiveClientWindow.prototype.cancelInteraction] 2-> interactionId: " + (this.m_interaction || {}).id + ", in_redial: " + this.m_interaction.in_redial + ", in_appointment: " + this.m_interaction.in_appointment);
			if (!this.m_interaction.in_redial && !this.m_interaction.in_appointment) {
				console.log("[LiveClientWindow.prototype.cancelInteraction] 3-> interactionId: " + (this.m_interaction || {}).id + ", voy por el cancel");
				this.m_interaction.event_name = event_name;
				isCancelled = inConcert.app.getInteractionManager().cancelInteraction(this.m_interaction);
				this.onInteractionGone(this.m_interaction);
			}
		}
		return isCancelled;
	};

	/** Clean interaction when is gone
	*/
	LiveClientWindow.prototype.onInteractionGone = function(interaction, showMessage) {
		const self = this;
		const invokeId = inConcert.Utils.generateGuid();
		if (!this.m_interaction) {
			console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  --> NO Gone -> this.m_interaction = undefined.");
			return;
		}
		if (interaction && (interaction.in_redial || interaction.in_appointment)) {
			console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  -> NO Gone interaction '" + interaction.id + "', interaction.in_redial: "  + interaction.in_redial + "', interaction.in_appointment: " + interaction.in_appointment);
			return;
		}

		if (!inConcert.app.has_allegro_integration()) {
			/* nativo:
				cuando la interaccion es una saliente manual, debemos dejar el estado del agente como se encontraba antes de realizar la llamada (tema: conflictos con las entrantes en el SipPhone).
			*/
			if (interaction.initiative === "OUTBOUND" &&
					this.m_oldUserStatus &&
					this.m_oldUserStatus !== "NOACD" &&
					inConcert.app.getCurrentStatus() === "NOACD"
				)
			{
				inConcert.app.execChangeUserState(this.m_oldUserStatus);
				this.m_oldUserStatus = null;
			}
		}

		let phoneStatus = self.m_phoneClient?.status();
		//event_name === REJECTED, close with timeout
		const interactionEventName = (interaction || {}).event_name;
		console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  1 .-> interaction '" + interaction.id + "', phoneStatus: "  + phoneStatus + "', interaction.event_name: " + interactionEventName);
		if (phoneStatus === "DIALING") {
			console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  1.1 .-> NO Gone interaction '" + interaction.id);
			return;
		}
		const is_rejected = interactionEventName === "REJECTED" ? true: false;

		const closeWindow = function() {
			const isGoneInteraction = self.m_interaction.isGone();
			console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  2 .-> START Closing Window interaction '" + interaction.id + ", isGoned: " + isGoneInteraction);
			self._maximized = false;
			if (!self.isMinimized()) {
				self.restore();
			}
			LiveClientWindow.uber.onInteractionGone.apply(self, [interaction, showMessage]);
			self.m_phoneClient?.setInteraction(null);
			self.m_interaction = null;
			self.__updateWindow("CANCELLED");
			self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);
			self.hideSideBar();
			if (is_rejected) {
				let text = self.displayStatusMessage("<%=_T('The conversation was canceled')%>...", "Info", true, null, null, {
					concatenateValue: true,
					messageSeparator: "..."
				});
				inConcert.app.m_mainWindow.showBannerMessage({message: text, messageType: "Info", timeout: 5000 }, null, true);
			}
			////close window
			let close_timeout_ms = 0;
			let close_retry = 0;
			let max_close_retry = 5;
			if (inConcert.app.has_allegro_baragent()) {
				close_timeout_ms = 100;
				max_close_retry = 30; // close_timeout_ms * max_close_retry = 3 segundos
			}
			let timeout_hnd = null;
			const mustCloseWindow = function(phoneStatus) {
				if (!inConcert.app.has_allegro_baragent()) {
					//YES close
					return {mustClose: true, reason: "NO HAS ALLEGRO BARAGENT", phoneStatus: phoneStatus};
				}
				if (self.m_interaction) {
					const isGoneInteraction = self.m_interaction.isGone();
					console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  -----> mustCloseWindow -> self.m_interaction '" + self.m_interaction.id + "', state: "  + self.m_interaction.state + "', phoneStatus: " + phoneStatus);
					if (!isGoneInteraction) {
						//NO close
						return {mustClose: false, reason: "UNABLE TO CLOSE WINDOW. INTERACTION NOT IS GONE.", state: self.m_interaction.state, phoneStatus: phoneStatus};
					}
				}
				if (phoneStatus !== "OFFERING") {
					//YES close
					return {mustClose: true, reason: "OFFERING !== phoneStatus: " + phoneStatus, phoneStatus: phoneStatus};
				}
				if (!self.m_phoneClient?._interaction) {
					//YES close
					return {mustClose: true, reason: "NOT self.m_phoneClient._interaction", phoneStatus: phoneStatus};
				}
				if (self.m_phoneClient?._interaction.id !== interaction.id) {
					//YES close
					return {mustClose: true, reason: "CLOSE WINDOW IN OFFERING PHONE STATUS", phoneStatus: phoneStatus};
				}
				//NO close
				return {mustClose: false, reason: "UNABLE TO CLOSE WINDOW IN OFFERING PHONE STATUS", phoneStatus: phoneStatus};
			};
			let validateCloseWindow;
			validateCloseWindow = function() {
				close_retry++;
				console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  2.1 -----> validateCloseWindow -> interaction '" + interaction.id + "', interactionState: "  + interaction.state +
					"', phoneStatus: " + phoneStatus + ", close_retry: " + close_retry + ", max_close_retry: " + max_close_retry + ", close_timeout_ms: " + close_timeout_ms);
				if (
					self.m_interaction?.isTaken() ||
					self.m_interaction?.isWrappedUp() ||
					close_retry > max_close_retry
				) {
					let phoneStatus = self.m_phoneClient?.status();
					console.warn("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".    2.2 -----> UNABLE to close window in " + phoneStatus + " phone status. interaction '" + interaction.id + "', interactionState: "  + interaction.state);
					window.clearTimeout(timeout_hnd);
					return;
				}
				timeout_hnd = window.setTimeout(function() {
					let phoneStatus = self.m_phoneClient?.status();
					let result = mustCloseWindow(phoneStatus);
					console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  2.3 --->  interaction '" + interaction.id + "', result: " + JSON.stringify(result));
					if (result.mustClose) {
						console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  2.3.1 -----> window.close() interaction '" + interaction.id + "', interactionState: "  + interaction.state + "', phoneStatus: " + phoneStatus);
						window.clearTimeout(timeout_hnd);
						self.close();
					}
					else {
						console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  2.3.2 ----->  WAIT Still can't close the phone ... interaction '" + interaction.id + "', interactionState: "  + interaction.state + "', phoneStatus: " + phoneStatus);
						validateCloseWindow();
					}
				}, close_timeout_ms);
			};
			validateCloseWindow();
		};

		if (interaction) {
			if (this.m_interaction.id !== interaction.id) {
				return;
			}
			const isGoneInteraction = this.m_interaction.isGone();
			if (isGoneInteraction) {
				closeWindow();
				return;
			}
			const isQueuedInteraction = interaction.isQueued();
			if (interaction.state === "PENDING" || isQueuedInteraction) {
				console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  NO Gone -> interaction '" + interaction.id + "', state: " + interaction.state + " isQueued: " + isQueuedInteraction + ", phoneStatus: "  + phoneStatus + "', interaction.event_name: " + interactionEventName);
				return;
			}
		}
		else {
			console.debug("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  -> interaction = undefined");
		}

		console.info("[LiveClientWindow.onInteractionGone] invokeId: " + invokeId + ".  Invoke BaseClass interaction '" + interaction.id + "', state: " + interaction.state + ", phoneStatus: "  + phoneStatus + "', interaction.event_name: " + interactionEventName);
		LiveClientWindow.uber.onInteractionGone.apply(this, [interaction, showMessage]);
	};

	/** Play an error sound
	*/
	LiveClientWindow.prototype.playErrorSound = function() {
		$('#beeptone').get(0).load();
		$('#beeptone').get(0).play();
		return false;
	};

	/** Switches between tabs (contacts, keyboard...)
	*/
	LiveClientWindow.prototype.switchClientControlView = function($clientControlView, $switchButton, targetView) {
		// 1- update selected button
		const $selectedButton = $('#viewTabs').find('.selected');
		if ( $selectedButton[0] !== $switchButton[0]) {
		// update only if it's not the same
			$selectedButton.removeClass('selected');
			$switchButton.addClass('selected');
		}
		// 2- update view
		const $previousView = $('#conversationControlsContainer').find('.callClientControlCurrent');
		$previousView.removeClass('callClientControlCurrent').hide();
		$clientControlView.addClass('callClientControlCurrent');

		if (targetView !== "keypad" && !this.m_interaction.isBeingComposed()) {
			$clientControlView.hide();
		}
		else {
			$clientControlView.show();
		}

		$("#digitsDisplay").focus(); // keep focus in the display
	};
	
	/** Enable tabs switches
	*/
	LiveClientWindow.prototype.enableClientControlSwitchers = function() {
		$('#switchButtonMembers').prop('disabled', false);
		$('#switchButtonKeypad, #switchButtonFavorites, #switchButtonContacts, #switchButtonAddressesBook').removeAttr('disabled');
	};
	
	/** Disable tabs switches
	*/
	LiveClientWindow.prototype.disableClientControlSwitchers = function() {
		var buttons = $('#switchButtonKeypad, #switchButtonMembers, #switchButtonFavorites, #switchButtonContacts, #switchButtonAddressesBook');
		buttons.prop('disabled', true);
	};

	LiveClientWindow.prototype.colapseToolbarActions = function(action = "colapse") {
		const $uncollapsedButtonsActions = $("#uncollapsedButtonsActions");
		const $collapsedButtonsActions = $("#collapsedButtonsActions");
		if (!$uncollapsedButtonsActions || !$collapsedButtonsActions) {
			return;
		}
		if (action.toLowerCase() === "colapse") {
			let html = $uncollapsedButtonsActions.html() || "";
			if (html.length > 0) {
				$uncollapsedButtonsActions.html("");
				$uncollapsedButtonsActions.hide();
				$collapsedButtonsActions.html(html);
				$collapsedButtonsActions.show();
			}
		}
		else {
			// Muevo de forma dinamica el markup de la toolbar
			let html = $collapsedButtonsActions.html() || "";
			if (html.length > 0) {
				$collapsedButtonsActions.html("");
				$collapsedButtonsActions.hide();
				$uncollapsedButtonsActions.html(html);
				$uncollapsedButtonsActions.show();
			}
		}
		// Se bindean los botones genericos de la toolbar de la clase base "ClientWindow"
		this.bindGenericToolbarActionsButtons();
	};

	/** Toggle keyboard
	*/
	LiveClientWindow.prototype.toggleKeyboard = function(is_visible) {
		const $keyboard = $("#keyboardContainer");
		const $digitsDisplay = $("#digitsDisplay");
		const $digitsClearDisplay = $("#digitsClearDisplay");
		const $viewTabs = $("#viewTabs");
		const $contactsContainer = $("#contactsContainer");
		const $body = $("body");

		const can_visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		const isOutgoing = this.m_interaction.isOutgoing();
		const isBeingComposed = this.m_interaction.isBeingComposed();
		if (is_visible) {
			$keyboard.show();
			$viewTabs.show();
			if (isBeingComposed) {
				$contactsContainer.show();
			}
			else {
				$contactsContainer.hide();
			}
			$body.removeClass("hidingNumericKeyboard");
			this.colapseToolbarActions("colapse");
			$digitsDisplay.show();
			$digitsClearDisplay.show();

			if (!can_visualize_address && !(isOutgoing && isBeingComposed)) {
				$digitsDisplay.hide();
				$digitsClearDisplay.hide();
			}
		}
		else {
			$keyboard.hide();
			$digitsDisplay.hide();
			$digitsClearDisplay.hide();
			$viewTabs.hide();
			$contactsContainer.hide();
			$body.removeClass("hidingNumericKeyboard").addClass("hidingNumericKeyboard");
			this.colapseToolbarActions("uncolapse");
		}
	};

	/** Gets campaign auto answer time (negative for no autoanswer)
	 * Auto Answer Time Value is in milliseconds
	*/
	LiveClientWindow.prototype.getAutoAnswer = function(interaction)
	{
		interaction = interaction || this.m_interaction;
		if (!interaction) {
			return -1;
		}
		if (interaction.isDialer() || (interaction.isVirtualhold() && interaction.state === "LOCKED")) {
			return 0;
		}
		const campaignId = interaction.campaign ? interaction.campaign : $("#cboCampaigns").val();
		if (!campaignId) {
			return -1;
		}
		const campaign = inConcert.app.getCampaign(campaignId);
		if (!campaign) {
			return -1;
		}
	
		const accounts_channel = campaign.get_accounts_channel("CALL");
		const first_account_channel = (accounts_channel || [])[0];
		if (!first_account_channel) {
			return -1;
		}
	
		const incomingConfig = campaign.getIncomingConfig(first_account_channel);
		if (!incomingConfig) {
			return -1;
		}
	
		if (incomingConfig.AutoAnswerTime === null || incomingConfig.AutoAnswerTime === undefined || isNaN(incomingConfig.AutoAnswerTime)) {
			return -1;
		}
		let autoAnswerTimeMs = Number(incomingConfig.AutoAnswerTime);
		return autoAnswerTimeMs;
	};

	/** Validates phone number
	*/
	LiveClientWindow.prototype.validatePhoneNumber = function(number) {
		return inConcert.app.validatePhoneNumber(number, "CALL");
	};

	/** Enables action button
	* @param  {object}		button		-Button to enable or disable
	* @param  {boolean}		enabled		-Button is enabled
	*/
	LiveClientWindow.prototype.enableActionButton = function(button, enabled) {
		$(button.id).toggle(!!enabled);
	};

	/** Enables action buttons
	* @param  {object}		enabledButtons		-Enabled buttons
	*/
	LiveClientWindow.prototype.enableActionButtons = function(enabledButtons = {}) {
		const self = this;
		$.each(this.m_actionButtons, function(buttonId, button) {
			self.enableActionButton(button, !!enabledButtons[buttonId]);
		});
	};

	/** Sets phone status classname.
	* @param  {string}		phoneStatus		-Phone status.
	*/
	LiveClientWindow.prototype.__updateWindow = function(phoneStatus) {
		const $body = $("body");
		let current_state_config;

		$.each(this.m_states, function(state, state_config) {
			let is_status = state === phoneStatus;
			$body.toggleClass(state_config.bodyClassName, is_status);

			if (is_status && !current_state_config) {
				current_state_config = state_config;
			}
		});

		/*
			Visualiza u oculta el waiting del area del telefono, dependiendo del estado.
		*/
		const $waiting = $("#phoneWaiting");
		if (current_state_config) {
			const waiting_info = current_state_config.waiting;
			if (waiting_info) {
				if (waiting_info.mustShow) {
					const $waiting_animation = $waiting.find("[animation]");
					/* El reloj que da vueltas del waiting */
					if (waiting_info.mustShowAnimation) {
						$waiting_animation.show();
					}
					else {
						$waiting_animation.hide();
					}
					// Un waiting mas pequeno, que deja visible algunas acciones.
					const small_class = "smallPhoneWaiting";
					if (waiting_info.small) {
						if (!$waiting.hasClass(small_class)) {
							$waiting.addClass(small_class);
						}
					}
					else {
						$waiting.removeClass(small_class);
					}
					$waiting.show();
					return;
				}
			}
		}

		// Oculta el Waiting
		$waiting.hide();
	};

	/** Sets phone status classname.
	* @param  {string}		label		-Phone status label.
	*/
	LiveClientWindow.prototype.__setStatusLabel = function(label) {
		$("#callClientControlStatus").val(label);
	};
	
	/** Sets view as voicemail
	*/
	LiveClientWindow.prototype.__makeVoicemail = function() {
		//already build
		if ($('body').hasClass("callClientPlayer")) {
			return;
		}
		$("#conversationControlsContainer").hide();
		$('body').addClass("callClientPlayer");
		$("#mainPlayerContainer").show();
		this.setMediaPlayer();
	};
	
	/** Enables active call buttons
	*/
	LiveClientWindow.prototype.__enableActiveCallActionButtons = function(interaction) {
		const canShownNumericKeyboard = this.canShownNumericKeyboard(interaction);
		
		const canHold = interaction.canHold(this.from_app);
		const canRecord = interaction.canRecord(this.from_app);
		const canSwitch = interaction.canSwitchConversation(this.from_app);
		
		const inHold = interaction.isOnHold();
		const muted = interaction.isMuted();
		const recording = interaction.isRecording();
		const switched = interaction.isSwitched();

		this.enableActionButtons({
			HOLD                 : canHold && !inHold,
			UNHOLD               : canHold && inHold,
			MUTE                 : !muted,
			UNMUTE               : muted,
			_SWITCH              : canSwitch && !switched,
			UNSWITCH             : canSwitch && switched,			
			RECORD               : canRecord && !recording,
			STOP_RECORD          : canRecord && recording,
			SHOW_NUMERIC_KEYBOARD: canShownNumericKeyboard,
			HIDE_NUMERIC_KEYBOARD: !canShownNumericKeyboard
		});
	};
	
	/** Updates dialing address
	*/
	LiveClientWindow.prototype.updateDialingAddress = function(text, contact, can_visualize_address) {
		if ( typeof(text) === "string" && text.length > 0) {
			$('#digitsDisplay').val(text);
			if (can_visualize_address !== false ) {
				$('.butControlDisplayClear').show();
			}
			
		} else {
			$('#digitsDisplay').val("");
			$('.butControlDisplayClear').hide();
		}
		if (contact) {
			$("#digitsDisplay").data("contact", contact);
		} else {
			$("#digitsDisplay").removeData("contact");
		}
	};
	
	/** Updates active call dialing address
	*/
	LiveClientWindow.prototype.updateActiveCallDialingAddress = function(interaction) {
		let phoneNumber = "";
		if (interaction.completeAddress && interaction.completeAddress.DialedNumber) {
			phoneNumber = interaction.completeAddress.DialedNumber;
		} else if (interaction.contactAddress) {
			phoneNumber = interaction.contactAddress;
		} else if (interaction.completeAddress && interaction.completeAddress.Number) {
			phoneNumber = interaction.completeAddress.Number;
		}
		const $digitsDisplay = $('#digitsDisplay');
		if (phoneNumber && $digitsDisplay.val().indexOf(phoneNumber) !== 0) {
			//Validar si se puede mostrar los addresses
			const can_visualize_address = inConcert.app.canVisualizeAddress(null, interaction.campaign, null, true);
			this.updateDialingAddress(phoneNumber, undefined);

			if (!can_visualize_address) {
				$digitsDisplay.hide();
				$('#digitsClearDisplay').hide();
			}
		}
	};

	/** Updates client view controls
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.updateControls = function(interaction, event_name)
	{
		const self = this;
		const belongsToMe = interaction.belongsToMe();
		let phoneStatus = this.m_phoneClient?.status();

		console.info(format("[LiveClientWindow.updateControls] START Update Phone Window Ctrls - Interaction Id: {0}, state: {1}, phoneStatus: {2}, belongsToMe: {3}, event_name: {4}.",
			interaction.id,
			interaction.state,
			phoneStatus,
			belongsToMe,
			event_name
		));

		if (phoneStatus === "CONNECTED" && interaction.state === "WRAPUP" && belongsToMe) {
			// Opened a WRAPUP interaction, set the phone as such
			this.m_phoneClient?.interactionWrapup();
		} 
		else if (phoneStatus === "WRAPUP" && interaction.state === "TAKEN" && belongsToMe) {
			// Opened a TAKEN interaction, set the phone as such
			this.m_phoneClient?.interactionTaken();
		}

		if (phoneStatus === "WRAPUP" && (interaction.state === "QUEUED" || !belongsToMe)) {
			/*
				close queued interaction

				Ref Issue #15445: PhoneClientWindow con ventanas que NO se cerraban.

				Esta invocacion debe realizarse asincronica en un timeout, ya que sino quedaba en un loop infinito, cuando es Nativo,
					ya que cuando la interaccion se remueve se invoca a la funcion onByed del phoneclient que termina invocando nuevamente
					a la funcion "remove interaction" del interactionmanager y ahi continuaba de forma indefinidia ya que la interaccion nunca se llegaba a borrar.
			*/
			inConcert.Utils.zeroTimeout(function() {
				self.m_phoneClient?.interactionFinished(interaction);
			});
			return;
		}

		if (phoneStatus === "OFFERING" && interaction.state === "TRANSFER LOCKED") {
			phoneStatus = "INCOMING TRANSFER";
		}
		else if (phoneStatus === "OFFERING" && interaction.state === "CONFERENCE LOCKED") {
			phoneStatus = "INCOMING CONFERENCE";
		}
		else if (phoneStatus === "ACTIVE" && interaction.state === "TRANSFER TAKEN") {
			phoneStatus = "INCOMING TRANSFER TAKEN";
		}
		else if (phoneStatus === "ACTIVE" && interaction.state === "CONFERENCE TAKEN") {
			phoneStatus = "INCOMING CONFERENCE TAKEN";
		}
		else if (interaction.state === "CONFERENCE") {
			phoneStatus = "CONFERENCE";
		}

		this.__updateWindow(phoneStatus);
		if (phoneStatus === "CONNECTED") {
			// Si el telefono se encuentra en estado CONNECTED y la interaccion de tipo live ha desaparecido (ABANDONED, CANCELED, FINISHED), debo cancelar.
			//
			let isGoneInteraction = interaction.isGone();
			if (isGoneInteraction) {
				console.info("[LiveClientWindow.updateControls] (GONE) Finished interaction popup will be closed - Id " + interaction.id + ", state " + interaction.state);
				this.cancelInteraction(event_name);
				this.showOrHideSwitchButtonNumericKeyboard();
				return;
			}
		}
		else {
			this.disableCampaignAccountsControls();
			this.disableClientControlSwitchers();
		}
		if (!inConcert.app.has_allegro_baragent() &&
			_.contains(["OFFERING", "INCOMING TRANSFER", "INCOMING CONFERENCE"], self.m_phoneClient?.status())) 
		{
			// Auto answer (if set)
			// El valor del answer time viene en milisegundos.
			const autoAnswerTimeMs = this.getAutoAnswer(interaction);
			if (autoAnswerTimeMs >= 0) {
				const currentId = interaction.id;
				if (this.m_autoAnswerTimeoutHnd) {
					window.clearTimeout(this.m_autoAnswerTimeoutHnd);
					this.m_autoAnswerTimeoutHnd = null;
				}
				this.m_autoAnswerTimeoutHnd = window.setTimeout(function() {
					const phoneStatus = self.m_phoneClient?.status();
					const interactionId = self.m_interaction && self.m_interaction.id;
					if (_.contains(["OFFERING", "INCOMING TRANSFER", "INCOMING CONFERENCE"], phoneStatus) && 
						interactionId === currentId) 
					{
						if (self.m_interaction && self.m_interaction.isPreview()) {
							self.m_phoneClient?.acceptPreview(self.m_interaction);
						}
						else {
							self.startAnswer(true);
						}
					}
					self.m_autoAnswerTimeoutHnd = null;
				}, autoAnswerTimeMs);
			}
		}
		switch(phoneStatus) {
			case "CONNECTED":
				this.__setConnectedControls(interaction);
				break;
			case "ERROR":
				this.__setErrorControls(interaction);
				break;
			case "DIALING":
				this.__setDialingControls(interaction);
				break;
			case "OFFERING":
				this.__setOfferingControls(interaction);
				break;
			case "PREVIEW":
				this.__setPreviewControls(interaction);
				break;
			case "REJECTED":
				this.__setRejectedControls(interaction);
				break;
			case "CANCELLED":
				this.__setCanceledControls(interaction);
				break;
			case "ACTIVE":
				this.__setActiveControls(interaction, event_name);
				break;
			case "WRAPUP":
				this.__setWrapupControls(interaction);
				break;
			case "TRANSFER TRYING":
				this.__setTransferTryingControls(interaction);
				break;
			case "TRANSFER TAKEN":
				this.__setTransferTakenControls(interaction);
				break;
			case "INCOMING TRANSFER":
				this.__setIncomingTransferControls(interaction);
				break;
			case "INCOMING TRANSFER TAKEN":
				this.__setIncomingTransferTakenControls(interaction);
				break;
			case "CONFERENCE TRYING":
				this.__setConferenceTryingControls(interaction);
				break;
			case "CONFERENCE TAKEN":
				this.__setConferenceTakenControls(interaction);
				break;
			case "INCOMING CONFERENCE":
				this.__setIncomingConferenceControls(interaction);
				break;
			case "INCOMING CONFERENCE TAKEN":
				this.__setIncomingConferenceTakenControls(interaction);
				break;
			case "CONFERENCE":
				this.__setConferenceControls(interaction);
				break;
		}

		this.show_attention_time(interaction, phoneStatus);
		this.showConferenceInfo(interaction);
		this.showOrHideSwitchButtonNumericKeyboard();
	};
	
	LiveClientWindow.prototype.show_attention_time = function (interaction, phoneStatus) {
		const self = this;
		const contains = _.contains(["CONFERENCE TRYING", "TRANSFER TRYING", "CONFERENCE TAKEN", "INCOMING CONFERENCE TAKEN", "INCOMING TRANSFER TAKEN", "CONFERENCE", "TRANSFER TAKEN"], phoneStatus);
		const $callDuration = $("#callClientDuration");
		if (contains || (phoneStatus === "ACTIVE" && (interaction.state === "TAKEN" || interaction.state === "TRANSFER TRYING" || interaction.state === "CONFERENCE TRYING"))) {
			if (self.m_att_timer) {
				return;
			}
			const initialTimestamp = (new Date()).getTime();
			self.m_att_timer = setInterval(function () {
				$callDuration.text(inConcert.Utils.getTimeDifference(initialTimestamp));
			}, 500);
			$callDuration.show();
		} else {
			$callDuration.hide();
			if (self.m_att_timer) {
				clearInterval(self.m_att_timer);
				delete self.m_att_timer;
			}
		}
	};

	/**
	 * Funcion que evalua si los usuarios conectados en la conferencia deben visualizarse. 
	*/
	LiveClientWindow.prototype.canShowConferenceInfo = function(interaction)
	{
		if (!interaction) {
			// No puede visualizar la conferencia.
			return false;
		}
		if (inConcert.app.has_allegro_integration()) {
			// No puede visualizar la conferencia.
			// Solo permitido para VCCs que NO tengan integracion con Allegro.
			return false;
		}
		if (interaction.isConferenceTaken() || interaction.isInConference()) {
			// ---> SI puede visualizar la conferencia
			return true;
		}

		return false;
	};

	LiveClientWindow.prototype.showBtnConfirmToAppointment = function(interaction)
	{
		if (interaction && interaction.isAppointment() && !interaction.isAppointmentActive()) {
			this.__setControlButtonState($(this.m_confirmButton), this.m_confirmButtonStates.NOTHING); 
		}
	};

	/**
	 * Funcion que visualiza informacion de los agentes y contactos que integran de la conferencia.
	*/
	LiveClientWindow.prototype.showConferenceInfo = function(interaction)
	{
		const $conference_info = $("#callConferenceUsersContainer");
		// 1 - Valido si debe o no visualizar la misma
		if (!this.canShowConferenceInfo(interaction)) {
			$conference_info.find("[users_list]").html("");
			$conference_info.hide();
			return;
		}

		// Ahora si procedo a visualizar informacion de la conferencia.
		//

		// funcion que crea el markup de un item
		let $createItemMarkup = function(userInfo) {
			let $userItem = $(
				'<li>' +
					'<img src="' + (userInfo.avatar ? userInfo.avatar : userInfo.generic_avatar) + '" type="image/png" width="16px" height="16px"/>' +
					'<span>' + userInfo.name + '</span>' +
					'<input type="hidden" id="' + userInfo.id + '"' +
				'</li>'
			);
			return $userItem;
		};

		const $user_list = $conference_info.find("[users_list]");

		let $createCallConferenceMarkup = function(participants) {
			$user_list.empty();
			let $callConferenceMarkup = $(
				'<button id="callConferenceUsersButton" class="callConferenceUsersButton">' +
				'+' + participants +
				'</button>' +
				'<ul id="callConferenceDropdown" class="callConferenceDropdown">' +
				'</ul>'
			);
			$user_list.append($callConferenceMarkup);
			let $callConferenceUsersButton = $("#callConferenceUsersButton");
			$callConferenceUsersButton.show();
			const target = document.querySelector('#callConferenceUsersButton');
			document.addEventListener('click', (event) => {
				const withinBoundaries = event.composedPath().includes(target);
				if(withinBoundaries) {
					$("#callConferenceDropdown").show();
				}
				else {
					$("#callConferenceDropdown").hide();
				}
			});
		};

		let me_id = inConcert.app.getLoggedUserFullId();

		// funcion que agrega agentes al markup de la conferencia.
		var addAgent = function (user_id, legId) {
			if (me_id === user_id) {
				let $conference_user = $createItemMarkup({
					avatar        : inConcert.app.getLoggedUserAvatar(),
					generic_avatar: inConcert.app.getUserGenericAvatar(),
					name          : "<%=_T('Me')%>",
					id            : legId
				});
				if($("#callConferenceDropdown").find("#" + legId).length === 0) {
					//Me has to appear at the end
					$("#callConferenceDropdown").append($conference_user);
				}
			}
			else {
				inConcert.app.getTeamMember(user_id, function(member = {}) {
					let user_name = member.name || user_id;
					let user_avatar = member.avatar;

					let $conference_user = $createItemMarkup({
						avatar        : user_avatar,
						generic_avatar: inConcert.app.getUserGenericAvatar(),
						name          : format("<%=_T('Agent: {0}')%>", user_name),
						id            : legId
					});
					if($("#callConferenceDropdown").find("#" + legId).length === 0) {
						//Me has to appear at the end. This goes in the beginning
						$("#callConferenceDropdown").prepend($conference_user);
					}
				}, true);
			}
		};

		// funcion que agrega contactos al markup de la conferencia.
		var addContact = function (number, legId) {
			let generic_avatar = inConcert.app.getContactGenericAvatar();
			inConcert.app.getContactManager().getContactByAddress({Type: "PHONE", Number: number}, function(contact) {
				let conference_contact = {
					generic_avatar: generic_avatar
				};
				if (typeof contact === "object") {
					conference_contact.avatar = contact.avatar;
					conference_contact.name = contact.Name;
				}
				conference_contact.avatar = conference_contact.avatar || conference_contact.generic_avatar;
				conference_contact.name = format("<%=_T('Contact: {0}')%>", conference_contact.name || number);
				conference_contact.id = legId;
				let $conference_contact = $createItemMarkup(conference_contact);
				if($("#callConferenceDropdown").find("#" + legId).length === 0) {
					//Me has to appear at the end. This goes in the beginning
					$("#callConferenceDropdown").prepend($conference_contact);
				}
			});
		};

		let conferenceParticipants = interaction.getLegsBy(function(leg) {
			return leg && leg.status === "BRIDGED" && !leg.contactLeg && leg.agent !== me_id;
		});
		if(Object.keys(conferenceParticipants).length > 0) {
			$createCallConferenceMarkup(Object.keys(conferenceParticipants).length + 1); //+ 1 == me_id
			for(const leg in conferenceParticipants) {
				if (conferenceParticipants.hasOwnProperty(leg)) {
					if(conferenceParticipants[leg].agent) {
						addAgent(conferenceParticipants[leg].agent, leg);
					}
					else if(conferenceParticipants[leg].number) {
						addContact(conferenceParticipants[leg].number, leg);
					}					
				}
			}
			
			let myLeg = interaction.getLegsBy(function(leg) {
				return leg && leg.status === "BRIDGED" && !leg.contactLeg && leg.agent === me_id;
			});
			let myLegId = "";
			if(Object.keys(myLeg).length === 1) {
				myLegId = Object.keys(myLeg)[0];
			}
			addAgent(me_id, myLegId);

			$("#callConferenceDropdown").css({left: $("#callConferenceUsersButton").position().left});		
		}

		// Visualizo el mensaje y el contenedor.
		$conference_info.show();
	};

	/** Set controls in CONNECTED state (phone bussy)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setConnectedControls = function(interaction) {
		this.enableActionButtons({ 
			REDIAL : this.m_lastDialedNumber.length !== 0
		});
		this.setControlButtonsStates(this.m_confirmButtonStates.CALL, this.m_cancelButtonStates.NOTHING);
		this.__setStatusLabel("<%=_T('Online')%>");
		this.showBtnConfirmToAppointment(interaction);
	};

	/** Set controls in ERROR state (phone with errros)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setErrorControls = function(/*interaction*/) {
		this.enableActionButtons();
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		this.__setStatusLabel("<%=_T('Error')%>");
	};
   
	/** Set controls in DIALING state (phone dialing an outgoing call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setDialingControls = function(interaction) {
		this.enableActionButtons();
		// valores por defecto
		const defaultButtonsStatuses = {
			confirm: this.m_confirmButtonStates.NOTHING,
			cancel : this.m_cancelButtonStates.CANCEL_OUTBOUND
		};
		//
		const phoneStatus = this.m_phoneClient?.status();
		const sipPhoneState = this.m_phoneClient?.getPhoneState();
		const statusesButtonsBySipPhoneState = {
			// cuando aun no comenzo a discar los botones tienen que estar inhabilitados.
			connected: {
				eval: () => {
					return {
						confirm: this.m_confirmButtonStates.NOTHING,
						cancel : this.m_confirmButtonStates.NOTHING
					};
				}
			}
		};
		//
		const buttonStates = statusesButtonsBySipPhoneState[sipPhoneState]?.eval() || defaultButtonsStatuses;
		console.log(`[LiveClientWindow.prototype.__setDialingControls] --> id: ${interaction.id}, state: ${interaction.state}, phoneStatus: ${phoneStatus}, sipPhoneState: ${sipPhoneState}, buttonStates: ${JSON.stringify(buttonStates)}`);
		//
		this.setControlButtonsStates(
			buttonStates.confirm,
			buttonStates.cancel
		);
		this.__setStatusLabel("<%=_T('Dialing')%>");
		//this.hideNumericKeyboard();
		this.change_client_state("connected");
	};

	/** Set controls in OFFERING state (phone ringing with an inbound call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setOfferingControls = function(interaction) {
		if (!interaction) {
			return;
		}
		//console.info(`[LiveClientWindow.prototype.__setOfferingControls] --> id: ${interaction.id}, state: ${interaction.state}, phoneStatus: ${this.m_phoneClient?.status()}`);
		this.enableActionButtons();
		if (interaction.isInternal()) {
			this.setControlButtonsStates(this.m_confirmButtonStates.ANSWER, this.m_cancelButtonStates.HANGUP);
			$('#callClientControlStatus').val("<%=_T('Internal call')%>");
		}
		else if (interaction.isInboundCall()) {
			if (interaction.type === "VOICEMAIL") {
				this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
				this.__makeVoicemail();
			}
			else {
				if (!interaction.isLocked()) {
					return;
				}
				/*
					La accion de answer debe estar disponible solo si el telefono se encuentra locked,
						ya que sino el telefono queda con acciones incompatibles, ejemplo si la intearccion se esta finalizando quedan botones accesibles que no deberían.
				*/
				this.setControlButtonsStates(!this.answering?
					this.m_confirmButtonStates.ANSWER : this.m_confirmButtonStates.NOTHING,
					this.m_cancelButtonStates.NOTHING);
				$('#callClientControlStatus').val("<%=_T('Incoming call')%>");
			}
		}
		else {
			if (interaction.isPreview()) {
				this.__setStatusLabel("<%=_T('Dialer call')%>");
				this.setControlButtonsStates(this.m_confirmButtonStates.CONFIRM_PREVIEW, this.m_cancelButtonStates.CANCEL_PREVIEW);
			} else if (interaction.isDialer() || inConcert.app.has_allegro_baragent()) {
				this.__setStatusLabel("<%=_T('Dialer call')%>");
				this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
			} else {
				this.setControlButtonsStates(!this.answering?
					this.m_confirmButtonStates.ANSWER : this.m_confirmButtonStates.NOTHING,
					this.m_cancelButtonStates.NOTHING);
				$('#callClientControlStatus').val("<%=_T('Incoming call')%>");
			}
		}
		this.hideNumericKeyboard();
		this.change_client_state("connected");
	};

	/** Set controls in PREVIEW state (phone in preview)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setPreviewControls = function(interaction) {
		this.enableActionButtons();
		if (interaction.isPreview()) {
			this.__setStatusLabel("<%=_T('Dialer Call (Preview)')%>");
			this.setControlButtonsStates(this.m_confirmButtonStates.CONFIRM_PREVIEW, this.m_cancelButtonStates.CANCEL_PREVIEW);
		} else {
			this.__setStatusLabel("<%=_T('Dialer Call (Trying)')%>");
			this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		}
		this.hideNumericKeyboard();
		this.change_client_state("connected");
	};

	/** Set controls in REJECTED state (phone call is rejected)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setRejectedControls = function(/*interaction*/) {
		this.enableActionButtons();
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		this.__setStatusLabel("<%=_T('Rejected')%>");
	};

	/** Set controls in CANCELLED state (phone call is canceled)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setCanceledControls = function(/*interaction*/) {
		this.enableActionButtons();
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		this.__setStatusLabel("<%=_T('Canceled')%>");
	};

	/** Set controls in ACTIVE state (phone with an active call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setActiveControls = function(interaction, event_name) {
		if (!interaction) {
			return;
		}
		//console.info(`[LiveClientWindow.prototype.__setActiveControls] --> id: ${interaction.id}, state: ${interaction.state}, phoneStatus: ${this.m_phoneClient?.status()}`);
		if (interaction.type === "VOICEMAIL") {
			this.updateActiveCallDialingAddress(interaction);
			this.change_client_state("connected");
			this.enableActionButtons();
			this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING,
				interaction.hasPermission("HANGUP", this.from_app)?
				this.m_cancelButtonStates.CLOSE_VOICEMAIL : this.m_cancelButtonStates.NOTHING);
			this.__makeVoicemail();
		}
		else {
			if (!interaction.isAssigned()) { // No actualizo si no es TAKEN o TRANSFER TAKEN
				return;
			}
			/*
				La acciones deben estar disponible solo si el telefono se encuentra TAKEN,
					ya que sino el telefono queda con acciones incompatibles, ejemplo si la intearccion se esta finalizando quedan botones accesibles que no deberían.
			*/
			this.updateActiveCallDialingAddress(interaction);
			this.change_client_state("connected");
			this.__enableActiveCallActionButtons(interaction);
			let cancelButtonState = interaction.hasPermission("HANGUP", this.from_app)? this.m_cancelButtonStates.HANGUP : this.m_cancelButtonStates.NOTHING;
			this.setControlButtonsStates(
				this.m_confirmButtonStates.NOTHING,
				cancelButtonState
			);
			const inHold = interaction.isOnHold();
			if (event_name === "ACCEPTED") {
				this.hideNumericKeyboard();
				this.updateLayout("LiveClientWindow.__setActiveControls - event_name " + event_name); // actualizo los Tabs, cuando estoy creando una interaccion saliente.
			}
			$('#callClientControlStatus').val(inHold? "<%=_T('Talking (Hold)')%>" : "<%=_T('Talking')%>");
		}
	};

	/** Set controls in WRAPUP state (phone call in wrapup)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setWrapupControls = function(interaction) {
		this.enableActionButtons({
			REDIAL : !interaction.manualState && interaction.hasPermission("REDIAL", this.from_app)
		});
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.NOTHING);
		this.__setStatusLabel("<%=_T('Wrapup')%>");
		if (interaction.type === "CALL") {
			this.hideNumericKeyboard();
		}
		this.change_client_state("wrapping_up");
		this.had_wrapup = true;
	};

	/** Set controls in TRANSFER TRYING state (phone call active trying a transfer)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setTransferTryingControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.CANCEL_TRANSFER);
		this.__setStatusLabel("<%=_T('Dialing (Transferring)')%>");
		//this.hideNumericKeyboard();
	};

	/** Set controls in TRANSFER TAKEN state (phone call active with a taken transfer call, as the transferer)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setTransferTakenControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		this.setControlButtonsStates(this.m_confirmButtonStates.CONFIRM_TRANSFER, this.m_cancelButtonStates.CANCEL_TRANSFER);
		if (interaction.isSwitched()) {
			this.__setStatusLabel("<%=_T('Talking (Switched)')%>");
		}
		else {
			this.__setStatusLabel("<%=_T('Talking (Transferring)')%>");
		}
		// this.showOrHideNumericKeyboard(interaction);
		// // this.toggleKeyboard(interaction.hasPermission("SEND_DIGITS", this.from_app));
	};

	/** Set controls in INCOMING TRANSFER state (phone ringing with an incoming transfer call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setIncomingTransferControls = function(/*interaction*/) {
		this.enableActionButtons();
		$('#callClientControlStatus').val("<%=_T('Incoming transfer')%>");
		this.setControlButtonsStates(!this.answering?
			this.m_confirmButtonStates.ANSWER : this.m_confirmButtonStates.NOTHING, 
			this.m_cancelButtonStates.NOTHING);
		this.hideNumericKeyboard();
		this.change_client_state("connected");
		
		/* TODO: add transfer controls to html and css
		$('#clientContainerTransfer').show(); THIS SHOULD BE AS A CSS RULE WITH interactionTransfered CLASS
		$('#transferDate').text("<%=_T('is transferring to you')%>");

		var step = self.m_interaction.steps[self.m_interaction.steps.length-1];
		var matches = step.from.match(new RegExp("([^@]+)@(.*)"));
		var nameFrom = matches ? matches[1] : step.from;
		var avatarFrom = inConcert.app.getBaseUrl() + "/users/get_avatar/" + (matches ? matches[1] : step.from);
		$('#transferMemberName').text(nameFrom);
		$("#transferMemberAvatar > img").attr("src", avatarFrom);
		 */
	};

	/** Set controls in INCOMING TRANSFER TAKEN state (phone call active with a taken transfer call, as the destination)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setIncomingTransferTakenControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.CANCEL_TRANSFER);
		if (interaction.isOnHold()) {
			this.__setStatusLabel("<%=_T('Talking (Incoming transfer on hold)')%>");
		}
		else {
			this.__setStatusLabel("<%=_T('Talking (Incoming transfer)')%>");
		}
		//this.showOrHideNumericKeyboard(interaction);
	};

	/** Set controls in CONFERENCE TRYING state (phone call active trying a conference)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setConferenceTryingControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		let statusLabel = this.getConferenceStatusLabel(interaction);
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.CANCEL_CONFERENCE);
		statusLabel = statusLabel || (inConcert.app.has_allegro_baragent() ? "<%=_T('Talking (Making conference)')%>" : "<%=_T('Dialing (Making conference)')%>");
		this.__setStatusLabel(statusLabel);
	};

	/** Set controls in CONFERENCE TAKEN state (phone call active with a taken conference call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setConferenceTakenControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		let statusLabel = this.getConferenceStatusLabel(interaction);
		this.setControlButtonsStates(statusLabel ? this.m_confirmButtonStates.NOTHING : this.m_confirmButtonStates.CONFIRM_CONFERENCE, this.m_cancelButtonStates.CANCEL_TRANSFER);
		statusLabel = statusLabel || (interaction.isSwitched() ? "<%=_T('Talking (Switched)')%>" : "<%=_T('Talking (Making conference)')%>");
		this.__setStatusLabel(statusLabel);
	};

	/** Set controls in INCOMING CONFERENCE state (phone ringing with an incoming conference call)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setIncomingConferenceControls = function(/*interaction*/) {
		this.enableActionButtons();
		$('#callClientControlStatus').val("<%=_T('Incoming conference')%>");
		this.setControlButtonsStates(!this.answering?
			this.m_confirmButtonStates.ANSWER : this.m_confirmButtonStates.NOTHING, 
			this.m_cancelButtonStates.NOTHING);
		this.hideNumericKeyboard();
		this.change_client_state("connected");
	};

	/** Set controls in INCOMING CONFERENCE TAKEN state (phone call active with a taken conference call, as the destination)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setIncomingConferenceTakenControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, this.m_cancelButtonStates.CANCEL_CONFERENCE);
		if (interaction.isOnHold()) {
			this.__setStatusLabel("<%=_T('Talking (Incoming conference on hold)')%>");
		}
		else {
			this.__setStatusLabel("<%=_T('Talking (Incoming conference)')%>");
		}
		//this.showOrHideNumericKeyboard(interaction);
	};

	/** Set controls in CONFERENCE state (phone call active with a conference)
	* @param  {object}		interaction		-Window interaction.
	*/
	LiveClientWindow.prototype.__setConferenceControls = function(interaction) {
		this.__enableActiveCallActionButtons(interaction);
		let cancelButtonState = interaction.hasPermission("HANGUP", this.from_app)? 
			this.m_cancelButtonStates.HANGUP : this.m_cancelButtonStates.NOTHING;
		this.setControlButtonsStates(this.m_confirmButtonStates.NOTHING, cancelButtonState);
		if (interaction.isInConferenceWithoutClient()) {
			this.__setStatusLabel("<%=_T('Conference (Contact hangup)')%>");
		}
		else {
			this.__setStatusLabel("<%=_T('Conference')%>");
		}
		//this.hideNumericKeyboard();
	};

	/** Runs confirm action
	*/
	LiveClientWindow.prototype.runConfirm = function() {
		const self = this;
		let phoneStatus = self.m_phoneClient?.status();
		switch(phoneStatus) {
			case "OFFERING":
				if (self.m_interaction && self.m_interaction.isPreview()) {
					self.m_phoneClient?.acceptPreview(self.m_interaction);
				}
				else {
					self.startAnswer();
				}
				break;
			case "PREVIEW":
				self.m_phoneClient?.acceptPreview(self.m_interaction);
				break;
			case "ACTIVE":
				if (self.m_interaction && self.m_interaction.type === "VOICEMAIL") {
					self.m_phoneClient?.replayVoicemail(self.m_interaction);
				}
				break;
			case "TRANSFER TAKEN":
				self.commitAttendedTransfer();
				break;
			case "CONFERENCE TAKEN":
				self.commitAttendedConference();
				break;
			default:
				if (self.prepareToDial()) {
					self.startDial();
				}
				break;
		}
	};

	/** Runs cancel action
	*/
	LiveClientWindow.prototype.runCancel = function() {
		const self = this;
		let phoneStatus = self.m_phoneClient?.status();
		switch(phoneStatus) {
			case "OFFERING":
				if (self.m_interaction && self.m_interaction.isPreview()) {
					self.m_phoneClient?.cancelPreview(self.m_interaction);
				}
				else {
					self.m_phoneClient?.reject(self.m_interaction);
				}
				break;
			case "PREVIEW":
				self.m_phoneClient?.cancelPreview(self.m_interaction);
				break;
			case "DIALING":
			case "ACTIVE":
				self.m_phoneClient?.hangup(self.m_interaction);
				break;
			case "TRANSFER TRYING":
			case "TRANSFER TAKEN":
			case "INCOMING TRANSFER TAKEN":
				self.cancelAttendedTransfer();
				break;
			case "CONFERENCE TRYING":
			case "CONFERENCE TAKEN":
			case "INCOMING CONFERENCE TAKEN":
				self.cancelAttendedConference();
				break;
			default:
				if (self.prepareToDial()) {
					self.startDial();
				}
				break;
		}
		$("#digitsDisplay").focus(); // keep focus in the display
	};

	/** Answers a call
	*/
	LiveClientWindow.prototype.startAnswer = function(autoAnswer = false) {
		const self = this;
		let currentId = self.m_interaction.id;
		self.disableClientControlSwitchers();
		self.setControlButtonsStates(self.m_confirmButtonStates.NOTHING, self.m_cancelButtonStates.NOTHING);
		self.answering = true;
		self.m_phoneClient?.answer(self.m_interaction, function(ok) {
			if (!ok) {
				self.answering = false;
				if (!autoAnswer) {
					self.showError("<%=_T('Error answering call.')%>");
				}
				self.clientUpdate();
				return;
			}
			//Hay casos bordes con allegro en que el answer falla pero devuelve que fue bien
			window.setTimeout(function() {
				self.answering = false;
				let phoneStatus = self.m_phoneClient?.status();
				let interactionId = self.m_interaction && self.m_interaction.id;
				if (phoneStatus === "OFFERING" && interactionId === currentId) {
					if (!autoAnswer) {
						self.showError("<%=_T('Error answering call.')%>");
					}
					self.clientUpdate();
				}
			}, 3000);
		});
	};

	/** Takes phone client ownership
	*/
	LiveClientWindow.prototype.takePhoneOwnership = function() {
		return this.m_phoneClient?.restoreView();
	};
	
	/** Prepare phone to dial
	*/
	LiveClientWindow.prototype.prepareToDial = function() {
		if ($("#digitsDisplay").val().trim().length === 0) {
			$("#digitsDisplay").focus();
			return this.playErrorSound();
		}
		if (this.m_interaction && this.m_interaction.state === 'WRAPUP') {
			this.displayStatusMessage("<%=_T('Current conversation is in wrapup')%>", 'Alert', false, 2000);
			return this.playErrorSound();
		}
		
		let phoneStatus = this.m_phoneClient?.status();
		if (phoneStatus !== "CONNECTED") {
			return this.playErrorSound();
		}

		var currentState = inConcert.app.getStateObject(inConcert.app.getCurrentStatus());
		if (currentState && currentState.isBreak === '1' ) { // don't ask if you don't wanna know the answer
			this.displayStatusMessage("<%=_T('Must be Available in order to dial')%>", 'Alert', false, 2000);
			return this.playErrorSound();
		}

		return this.takePhoneOwnership();
	};

	/** Dial
	*/
	LiveClientWindow.prototype.startDial = function(callback = $.noop)
	{
		const has_allegro_integration = inConcert.app.has_allegro_integration();
		/*
		*/
		const startDialCallback = (success, resp) => {
			if (!success) {
				// error
				const errType = resp;
				this.showError(_T(errType));
				return callback(false, errType);
			}
			// ok
			return callback(true, resp);
		};

		let fullPhoneNumber = inConcert.Utils.cleanPhoneNumber($("#digitsDisplay").val());
		if (fullPhoneNumber.length === 0) {
			return startDialCallback(false, "Phone number is empty");
		}

		//Sacar char '+' allegro
		let phoneNumber = fullPhoneNumber;
		if (has_allegro_integration && fullPhoneNumber[0] === "+") {
			phoneNumber = fullPhoneNumber.substring(1);
		}

		if (!this.validatePhoneNumber(phoneNumber)) {
			return startDialCallback(false, "Invalid phone number");
		}

		this.disableClientControlSwitchers();
		const contact = $("#digitsDisplay").data("contact") || null;
		if (!this.m_interaction) {
			const panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");
			const response = panel.createOutgoingInteraction("CALL", contact, "", inConcert.app.getCampaigns(), undefined, window.getWindowHandle());
			if (!response.success) {
				if (response.conflicts) {
					this.displayStatusMessage(
						response.conflicts.message,
						"Error",
						false,
						5000);
				}
				return startDialCallback(false, response.conflicts.message);
			}
			const interaction = response.interaction;
			this.m_phoneClient?.setInteraction(interaction);
			this.setInteraction(interaction);
			if (!this.m_interaction) {
				return startDialCallback(false, "Interaction not found");
			}
		}
		else if (contact) {
			this.m_interaction.setContactObject(contact);
		}

		const dialingCampaign = $("#cboCampaigns").val();
		this.m_interaction.setCampaign(dialingCampaign); // update interaction campaign

		let accountId;
		if (!has_allegro_integration) {
			accountId = $("#cboAccounts").val();
		}

		if (!this.canExecuteInteractionAction()) {
			return startDialCallback(false, "Can not make call");
		}

		this.m_interaction.contactAddress = fullPhoneNumber;
		this.m_lastDialedNumber = this.m_interaction.contactAddress;
		this.m_lastDialedCampaign = dialingCampaign;
		this.m_lastDialedContact = this.m_interaction.contactObject;

		// START DIAL function
		const startDial = ()=> {
			this.m_interaction.dial({
				destination     : phoneNumber,
				accountId       : accountId, // solo para telefonia nativa de i6
				attentionLevelId: this.getSelectedAttentionLevel(),
				callback        : startDialCallback
			});
		};

		// TODO: #16835 ACOMODAR por el ultimo reopen del 30/06/2023
		// if (has_allegro_integration) {
		if (true) {
			// allegro
			startDial();
			return;
		}
		/*	nativo:
			cuando realizamos una saliente manual, previamente debemos dejar el estado en NOACD, para que no haya conflictos con las entrantes en el SipPhone.
		*/
		this.m_oldUserStatus = inConcert.app.getCurrentStatus();
		inConcert.app.execChangeUserState("NOACD", undefined, (/*resp*/)=> {
			startDial();
		});
	};

	/** Redial
	*/
	LiveClientWindow.prototype.redial = function() {
		let phoneStatus = this.m_phoneClient?.status();
		if (phoneStatus === "WRAPUP") {
			if (this.m_interaction.isPhoneValidToRedial()) {
				this.disableClientControlSwitchers();
				let self = this;
				this.m_phoneClient?.redial(this.m_interaction, function(success, error) {
					if (success) {
						self.toggleKeyboard(true); // This leaves the hangup button below the overlay
					} else {
						self.showError(error);
					}
					self.enableClientControlSwitchers();
				});
			}
			else {
				this.showError("<%=_T('The number to redial is invalid or hidden.')%>");
			}
		}
		else {
			const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
			this.updateDialingAddress(this.m_lastDialedNumber, undefined, visualize_address);
			$("#cboCampaigns").val(this.m_lastDialedCampaign);
			if (this.m_lastDialedContact) {
				$("#digitsDisplay").data("contact", this.m_lastDialedContact);
			} else {
				$("#digitsDisplay").removeData("contact");
			}
			$(this.m_confirmButton).click();
		}
	};

	/** Cancel attended transfer
	*/
	LiveClientWindow.prototype.cancelAttendedTransfer = function() {
		if (inConcert.app.has_allegro_baragent()) {
			this.m_phoneClient?.cancelTransfer(this.m_interaction);
		}
		else {
			const self = this;
			self.m_interaction.cancelAttendedTransfer(function(success, error) {
				if (!success) {
					self.showError(error);
				}
				self.clientUpdate();
			});
		}
	};

	/** Commit attended transfer
	*/
	LiveClientWindow.prototype.commitAttendedTransfer = function() {
		if (inConcert.app.has_allegro_baragent()) {
			this.m_phoneClient?.confirmTransfer(this.m_interaction);
		}
		else {
			const self = this;
			self.m_interaction.commitAttendedTransfer(function(success, error) {
				//TODO: action must change phone_client state
				if (!success) {
					self.showError(error);
				}
				self.clientUpdate();
			});
		}
	};

	/** Cancel attended conference
	*/
	LiveClientWindow.prototype.cancelAttendedConference = function() {
		const self = this;
		this.m_phoneClient?.cancelConference(this.m_interaction, function(success, error) {
				if (!success) {
					self.showError(error);
				}
				self.clientUpdate();
			});
	};

	/** Commit attended conference
	*/
	LiveClientWindow.prototype.commitAttendedConference = function() {
		const self = this;
		this.m_phoneClient?.confirmConference(this.m_interaction, function(success, error) {
				//TODO: action must change phone_client state
				if (!success) {
					self.showError(error);
				}
				self.clientUpdate();
			});
	};
	
	/** Hold
	*/
	LiveClientWindow.prototype.hold = function() {
		const self = this;
		if (!self.m_interaction || self.m_interaction.isOnHold()) {
			self.clientUpdate();		
			return;
		}
		let $buttons = $(self.m_actionButtons.HOLD.id + ", " + self.m_actionButtons.UNHOLD.id);
		$buttons.prop("disabled", true);	
		self.m_phoneClient.hold(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error holding call.')%>");
			}
			self.clientUpdate();
		});
	};
	
	/** Unhold
	*/
	LiveClientWindow.prototype.unhold = function() {
		const self = this;
		if (!self.m_interaction || !self.m_interaction.isOnHold()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons.HOLD.id + ", " + self.m_actionButtons.UNHOLD.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?.unhold(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error unholding call.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Mute
	*/
	LiveClientWindow.prototype.mute = function() {
		const self = this;
		if (!self.m_interaction || self.m_interaction.isMuted()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons.MUTE.id + ", " + self.m_actionButtons.UNMUTE.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?.mute(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error muting call.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Unmute
	*/
	LiveClientWindow.prototype.unmute = function() {
		const self = this;
		if (!self.m_interaction || !self.m_interaction.isMuted()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons.MUTE.id + ", " + self.m_actionButtons.UNMUTE.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?.unmute(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error unmuting call.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Switch
	*/
	LiveClientWindow.prototype._switch = function() {
		const self = this;
		if (!self.m_interaction || !self.m_interaction.canSwitchConversation()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons._SWITCH.id + ", " + self.m_actionButtons.UNSWITCH.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?._switch(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error switching call.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Unswitch
	*/
	LiveClientWindow.prototype.unswitch = function() {
		const self = this;
		if (!self.m_interaction || !self.m_interaction.canSwitchConversation()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons._SWITCH.id + ", " + self.m_actionButtons.UNSWITCH.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?.unswitch(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error unswitching call.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Record
	*/
	LiveClientWindow.prototype.record = function() {
		const self = this;
		if (!self.m_interaction || self.m_interaction.isRecording()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons.RECORD.id + ", " + self.m_actionButtons.STOP_RECORD.id);
		$buttons.prop("disabled", true);	
		self.m_phoneClient?.record(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error starting record.')%>");
			}
			self.clientUpdate();
		});
	};

	/** Stop Record
	*/
	LiveClientWindow.prototype.stopRecord = function() {
		const self = this;
		if (!self.m_interaction || !self.m_interaction.isRecording()) {
			self.clientUpdate();
			return;
		}
		let $buttons = $(self.m_actionButtons.RECORD.id + ", " + self.m_actionButtons.STOP_RECORD.id);
		$buttons.prop("disabled", true);
		self.m_phoneClient?.stopRecord(self.m_interaction, function(ok) {
			$buttons.prop("disabled", false);
			if (!ok) {
				self.showError("<%=_T('Error stopping record.')%>");
			}
			self.clientUpdate();
		});
	};

	/** show Numeric Keyboard
	*/
	LiveClientWindow.prototype.showNumericKeyboard = function() {
		const is_visible = true;
		this.toggleKeyboard(is_visible);
		const $hideNumericKeyboard = $(this.m_actionButtons.HIDE_NUMERIC_KEYBOARD.id);
		$hideNumericKeyboard.show();
		const $showNumericKeyboard = $(this.m_actionButtons.SHOW_NUMERIC_KEYBOARD.id);
		$showNumericKeyboard.hide();
	};

	/** hide Numeric Keyboard
	*/
	LiveClientWindow.prototype.hideNumericKeyboard = function() {
		const is_visible = false;
		this.toggleKeyboard(is_visible);

		let inHold = false;
		let hasPermissionToSendDigits = false;
		const interaction = this.getInteraction();
		if (interaction) {
			inHold = interaction.isOnHold();
			hasPermissionToSendDigits = interaction.hasPermission("SEND_DIGITS", this.from_app);
		}

		const showSwithButtons = hasPermissionToSendDigits && !inHold;
		const $showNumericKeyboard = $(this.m_actionButtons.SHOW_NUMERIC_KEYBOARD.id);
		const $hideNumericKeyboard = $(this.m_actionButtons.HIDE_NUMERIC_KEYBOARD.id);

		if (showSwithButtons) {
			$showNumericKeyboard.show();
			$hideNumericKeyboard.hide();
		}
		else {
			$showNumericKeyboard.hide();
			$hideNumericKeyboard.hide();
		}
	};

	LiveClientWindow.prototype.showOrHideNumericKeyboard = function(interaction) {
		const canShown = this.canShownNumericKeyboard(interaction);
		if (!canShown) {
			this.hideNumericKeyboard();
			return;
		}
		this.showNumericKeyboard();
	};

	LiveClientWindow.prototype.showOrHideSwitchButtonNumericKeyboard = function(interaction) {
		const $buttonShowNumericKeyboard = $(this.m_actionButtons.SHOW_NUMERIC_KEYBOARD.id);
		const $buttonHideNumericKeyboard = $(this.m_actionButtons.HIDE_NUMERIC_KEYBOARD.id);
			
		const canShown = this.canShownNumericKeyboard(interaction);
		if (!canShown) {
			$buttonShowNumericKeyboard.hide();
			$buttonHideNumericKeyboard.hide();
			return;
		}
		if (this.isVisibleNumericKeyboard()) {
			$buttonHideNumericKeyboard.show();
			$buttonShowNumericKeyboard.hide();
		}
		else {
			$buttonShowNumericKeyboard.show();
			$buttonHideNumericKeyboard.hide();
		}
	};

	LiveClientWindow.prototype.isVisibleNumericKeyboard = function() {
		const $keyboard = $("#keyboardContainer");
		const is_visible_keyboard = $keyboard.is(":visible");
		return is_visible_keyboard;
	};

	LiveClientWindow.prototype.canShownNumericKeyboard = function(interaction) {
		interaction = interaction || this.getInteraction();

		if (!interaction) {
			return false;
		}

		if (interaction.type !== "CALL") {
			return false;
		}

		if (!interaction.isAssigned()) {
			return false;
		}

		if (!interaction.belongsToMe()) {
			return false;
		}

		const hasPermissionToSendDigits = interaction.hasPermission("SEND_DIGITS", this.from_app);
		if (!hasPermissionToSendDigits) {
			return false;
		}

		return true;
	};

	/** Pad key pressed
	*/
	LiveClientWindow.prototype.onKeyPressed = function(key) {
		let phoneStatus = this.m_phoneClient?.status();
		const self = this;
		if (_.contains(["CONNECTED", "ACTIVE", "TRANSFER TAKEN", "INCOMING TRANSFER TAKEN", 
			"CONFERENCE TAKEN", "INCOMING CONFERENCE TAKEN", "CONFERENCE"], phoneStatus))
		{
			const visualize_address = inConcert.app.canVisualizeAddress(null, self.m_interaction.campaign, null, true);
			const isOutgoing = self.m_interaction.isOutgoing();
			const isBeingComposed = self.m_interaction.isBeingComposed();
			let can_show_address = visualize_address || (isOutgoing && isBeingComposed);
			this.updateDialingAddress( $("#digitsDisplay").val() + key, undefined, can_show_address); // update this way in order to clear contact data in case it exists
			if (phoneStatus !== "CONNECTED") {
				const self = this;
				self.m_phoneClient?.sendDigit(self.m_interaction, key, function(ok) {
					if (!ok) {
						self.showError("<%=_T('Error sending digit.')%>");
					}
				});
			}
		}
	};

	/** Bind events
	*/
	LiveClientWindow.prototype.setEvents = function()
	{
		const self = this;

		const $digitsDisplay = $("#digitsDisplay");

		//binding of the keypad
		$(".butCallKeyboard").click(function() {
			const digit = $(this).attr("digit");
			self.onKeyPressed(digit);
		});
		//confirm button
		$("#confirmButton").click(function() {
			self.runConfirm();
		});
		//cancel button
		$("#cancelButton").click(function() {
			self.runCancel();
		});
		//clear number display
		$(".butControlDisplayClear")
			.unbind("click")
			.bind("click", function(){
				const phoneStatus = self.m_phoneClient?.status();
				if (phoneStatus === "ACTIVE" || phoneStatus === "DIALING") {
					return;
				}
				else {
					self.updateDialingAddress("");
					$digitsDisplay.focus();
				}
			});

		let withPrecondition = function(checkPrecondition, function2call, context) {
			context = context || this;
			return function() {
				if ( checkPrecondition.apply(context, arguments) ) {
					return function2call.apply(context, arguments);
				}
			};
		};

		let preconditionHasFocus = _.partial(withPrecondition, function() {
			let activeElement = document.activeElement;
			if (activeElement && activeElement.nodeName &&
					(activeElement.nodeName.toUpperCase() === "BODY" ||
						(activeElement.nodeName.toUpperCase() !== "BODY" && activeElement.id === "digitsDisplay"))) {
				return true;
			}
			return false;
		});

		//digits
		const $body = $("body");
		$body.on('keydown', preconditionHasFocus(function(event) {
				let phoneStatus = self.m_phoneClient?.status();
				if (phoneStatus === "ACTIVE") {
					if (event.keyCode === 8 || event.keyCode === 46) { // backspace or delete
						event.preventDefault();
					}
				}
				$digitsDisplay.focus();
			}))
			.on('keypress', preconditionHasFocus(function(event) {
				if (event.keyCode === 13) {
					return self.prepareToDial() && self.startDial();
				}

				let displayTextLength = $digitsDisplay.val().length;
				let charDigit = String.fromCharCode(event.keyCode);

				// NO permitimos el caracter + si:
					//		1 - Ya existe al menos un caracter ingresado en el Display.
				// SI permitimos el caracter + si:
					//		1 - NO existe ningunb caracter ingresado en el Display (o sea es el primer carater ingresado).

				const digitsAllowed = displayTextLength > 0 ? "1234567890ABCD*#": "1234567890ABCD*#+";
				if (digitsAllowed.indexOf(charDigit) === -1) {
					return event.preventDefault();
				}

				$("#keyBoardButton" + event.keyCode).addClass("keyBoardButtonPressed");
				$digitsDisplay.focus();

				let phoneStatus = self.m_phoneClient?.status();
				if (_.contains(["ACTIVE", "TRANSFER TAKEN", "INCOMING TRANSFER TAKEN", 
					"CONFERENCE TAKEN", "INCOMING CONFERENCE TAKEN", "CONFERENCE"], phoneStatus))
				{
					let currentDigits = $digitsDisplay.val();
					self.m_phoneClient?.sendDigit(self.m_interaction, charDigit, function(ok) {
						if (ok) {
							currentDigits += charDigit;
						}
						else {
							self.showError("<%=_T('Error sending digit.')%>");
						}
						$digitsDisplay.val(currentDigits);
					});
					return false;
				}
			}))
			.on('keyup', preconditionHasFocus(function() {
				$("#keyboardContainer button").removeClass("keyBoardButtonPressed");
				let displayTextLength = $digitsDisplay.val().length;

				let phoneStatus = self.m_phoneClient?.status();
				const visualize_address = inConcert.app.canVisualizeAddress(null, self.m_interaction.campaign, null, true);
				if ( displayTextLength > 0 && phoneStatus !== "ACTIVE" && phoneStatus !== "DIALING") {
					if (visualize_address) {
						$(".butControlDisplayClear").show();
					}
				} else {
					$(".butControlDisplayClear").hide();
				}
			}));

		//tabs switch
		$('#conversationControlsContainer').find("[tab_client_ctrl_switcher]")
			.unbind('click')
			.bind('click', function() {
				const $selectedButton = $(this);
				const targetView = ($selectedButton.attr('tab_client_ctrl_switcher') || "").toLowerCase();

				if ( self.m_contactsModel ) {
					self.m_contactsModel.cancelSearch();
				}

				$('#directorySearchField').val("");

				$body.removeClass("activeKeyboardTab activeMembersTab activeFavoritesTab activeContactsTab activeAddressBookTab");

				switch (targetView) {
					case "keypad":
						// just make the keypad visible and leave
						self.switchClientControlView($('#keyboardContainer'), $selectedButton, targetView);
						$body.addClass("activeKeyboardTab");
						return;
					case "members":
						// create model for team members
						let interaction = self.m_interaction && self.m_interaction.state !== "PENDING" ? self.m_interaction : null;
						self.m_contactsModel = new inConcert.TeamMemberSelectorModel("phone", interaction);
						$body.addClass("activeMembersTab");
						break;
					case "favorites":
						// create model for favorite contacts
						self.m_contactsModel = new inConcert.FavoritesSelectorModel("phone");
						$body.addClass("activeFavoritesTab");
						break;
					case "contacts":
						// create model for contacts in the current campaign
						self.m_contactsModel = new inConcert.ContactSelectorModel("phone", null, function() {
							return $("#cboCampaigns").val();
						});
						$body.addClass("activeContactsTab");
						break;
					case "addresses_book":
						// create model for addresses_book
						self.m_contactsModel = new inConcert.AddressesBookSelectorModel("call"); // tiene que ir el proxy ya que la clase resuelve el tipo de interaccion.
						$body.addClass("activeAddressBookTab");
						break;
				}

				// clear contact list
				$('#controlDirectoryScroll').children().remove();
				// make contact list visible and propper button selected
				self.switchClientControlView($('#contactsContainer'), $selectedButton, targetView);
				// perform search
				self.startSearch(false);
			}
		);

		let searchTimer = null;
		let doStartSearch = function(invokeServer) {
			clearTimeout(searchTimer);

			if ( self.m_contactsModel ) {
				self.m_contactsModel.cancelSearch();
			}

			let fromCache = !invokeServer; // useCache === !invokeServer
			if (self.m_contactsModel.mustAddAllRowsClientSide()) {
				fromCache = true;
			}

			self.startSearch(fromCache);
		};

		// search contacts on keyup
		$('#directorySearchField').unbind('keyup').bind('keyup', function(event) {
			// If user press ctrl/tab/command/arrows don't do anything
			let filteredControlKeys = inConcert.Utils.getControlCharacters();
			if (filteredControlKeys[event.keyCode]) {
				return;
			}
			// if user press Enter trigger the search immediately
			if (event.keyCode === 13) {
				doStartSearch(true); // invoke server
				return;
			}
			clearTimeout(searchTimer);
			searchTimer = setTimeout(doStartSearch, 300); // this is the only one who uses cache
		});

		// search contacts on click
		$('#butDirectorySearch').unbind('click').bind('click', function() {
			doStartSearch(true); // invoke server
		});

		// biendo los eventos de forma dinamica.
		const $buttonsActions = $("#uncollapsedButtonsActions, #collapsedButtonsActions");

		for (let id in this.m_actionButtons) {
			if (this.m_actionButtons.hasOwnProperty(id)) {
				const action = this.m_actionButtons[id];
				if (action.id) {
					if (action.execute) {
						const event_selector = action.id;
						$buttonsActions
							.off("click", event_selector)
							.on("click", event_selector, function() {
								action.execute();
							}
						);
					}
				}
			}
		}
	};

	/** Start contacts search
	*/
	LiveClientWindow.prototype.startSearch = function(fromCache) {
		const self = this;

		self.m_lastFilter = $('#directorySearchField').val();

		// locally bound variables to check for timeouts and cancellations
		let filter         = self.m_lastFilter;
		let searchComplete = false;
		let monitorTimer   = null;

		let onSearchComplete = function(statusOk) {
			if (monitorTimer) {
				clearTimeout(monitorTimer);
			}
			searchComplete = !searchComplete; // should set it to true
			// re-enable search controls
			$('#directorySearchField').removeAttr('disabled');
			$('#butDirectorySearch').removeAttr('disabled');
			// re-enable switch buttons
			self.enableClientControlSwitchers();
			// hide animation
			$('#controlDirectorySearchWaiting').hide();
			// use results
			let ok = (statusOk && searchComplete);
			self.updateContactsDirectory(ok);
			$('#directorySearchField').focus();
		};

		let searchFunction = function() {
			if (self.m_searching) {
			// a search is being executed, cancel it and try again in 10 ms
				self.cancelSearch();
				setTimeout(searchFunction, 10);
				return;
			}

			self.m_searching = true;
			self.m_cancelSearch = false;

			if (!self.m_contactsModel) {
			// this function should always be called with contactsModel set
				throw "startSearch called but contactsModel is not set";
			}

			// make sure controls are hidden
			$('#controlDirectoryScroll').show();
			$('#controlDirectorySearchMessages').hide();

			// and only the 'waiting' box is shown
			$('#controlDirectorySearchWaiting').show();

			// avoid lots of searchs at the same time
			self.disableClientControlSwitchers();
			$('#butDirectorySearch').prop('disabled', true);
			$('#directorySearchField').prop('disabled', true);

			if (fromCache) {
			// filter from cache when the user types in the search box
				self.m_contactsModel.filterFromCache(filter, onSearchComplete);
			} else {
			// perform proper search
				self.m_contactsModel.search(filter, onSearchComplete);
				// check search timeout
				monitorTimer = setTimeout(function() {
					monitorTimer = null;
					if (!searchComplete) { // locally bound, applies for this search only
						onSearchComplete(false);
					}
				}, 10000);
			}
		};

		setTimeout(searchFunction, 0);
	};

	/** Cancel contacts search
	*/
	LiveClientWindow.prototype.cancelSearch = function() {
		this.m_cancelSearch = true;
		this.m_contactsModel.cancelSearch();
	};

	/** Updates contacts list
	*/
	LiveClientWindow.prototype.updateContactsDirectory = function(statusOk) {
		const self = this;
		let searchAborted = !statusOk;
		//inConcert.logger.action({action_name:'updateContactsDirectory'});

		let searchCanceled = function() {
			let canceled = self.m_cancelSearch;
			if (canceled) {
				searchCanceled = function() {
					return true;
				};
			}
			return canceled;
		};

		// create markup for one item
		let $createItemMarkup = function(contactAvatar, contactName, contactAddress, description) {
			let $contactItem = $(
				'<div class="directoryItem">' +
				(contactAvatar ? '<img src="' + contactAvatar + '" class="directoryItemAvatar"/>' : '') +
				'<div class="directoryItemInfoContainer">' +
					(contactName ? '<label class="directoryItemName">' + contactName + '</label>' : '') +
					'<label class="directoryItemAccount">' + contactAddress + '</label>' +
				'</div>' +
				'</div>'
			);
			if (description) {
				$contactItem.attr("title", description);
			}
			return $contactItem;
		};

		let contactCount = 0;
		let contactsDiv = $('#controlDirectoryScroll');
		contactsDiv.children().remove();
		contactsDiv.unbind('scroll');

		let onContactClicked = function() {
			let contact = $(this).data("contact");
			let contactAddress = self.m_contactsModel.getContactAddress(contact);
			self.updateDialingAddress(contactAddress, contact);
		};

		let onContactDoubleClicked = function() {
			let contact = $(this).data("contact");
			let contactAddress = self.m_contactsModel.getContactAddress(contact);
			self.updateDialingAddress(contactAddress, contact);
			if (self.prepareToDial()) {
				self.startDial();
			}
		};

		//Validar si se puede mostrar los addresses
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		// this function actually appends a contact to the list
		let appendContact = function(contact) {
			let $contactItem = $createItemMarkup(
				self.m_contactsModel.getAvatar(contact),
				self.m_contactsModel.getDisplayName(contact),
				visualize_address? self.m_contactsModel.getContactAddress(contact): "<%=_T('Hidden Address')%>",
				self.m_contactsModel.getContactDescription(contact)
			);
			$contactItem.data("contact", contact);
			if(visualize_address){
				$contactItem.bind('dblclick.selected', onContactDoubleClicked);
				$contactItem.bind('click', onContactClicked);
			}
			contactsDiv.append($contactItem);
		};

		let mustAddAllRowsClientSide = this.m_contactsModel.mustAddAllRowsClientSide();

		// appends the next contact to the list
		//and keeps looking for more
		let appendNextContact = function(ok, contact) {
			if ( contactCount === 0 && self.m_contactsModel.getContacts().length === 0 ) {
				$('#controlDirectorySearchMessages').show();
				$('#controlDirectoryScroll').hide();
			}
			if ( searchCanceled() || ok === false ) {
				self.m_searching = false;
				return;
			}

			appendContact(contact);
			contactCount++;

			// harcoded to 5 elements because of the size of the panel
			// we could calculate it using the height of the scroll
			//area and the height of an item
			if(mustAddAllRowsClientSide || contactCount < 5) {
				self.m_contactsModel.getNextContact(appendNextContact);
				if (mustAddAllRowsClientSide) {
					self.m_searching = false;
				}
			} else {
			// done with the search
				self.m_searching = false;
			}
		};

		if( searchAborted ) { // search canceled or timed-out
			if (self.m_cancelSearch === false) { // cancel flag is not set so it timed-out
				$('#controlDirectorySearchMessages').show();
				$('#controlDirectoryScroll').hide();
			}
			self.m_searching = false;
			return;
		}

		contactsDiv.show(); // hidden on startSearch()

		// initialize the contact iterator
		this.m_contactsModel.getNextContact(appendNextContact);

		if (!mustAddAllRowsClientSide) {
			// load more contacts on scroll
			contactsDiv.bind('scroll', function() {
				let scroll = this;
				let scrollTop = scroll.scrollTop;
				let scrollHeight = scroll.scrollHeight;
				let clientHeight = scroll.clientHeight;
				let scrolledPercentage = Math.round((scrollTop / (scrollHeight - clientHeight)) * 100);
				if (scrolledPercentage >= 80 && self.m_contactsModel) {
					contactCount = 0;
					self.m_searching = true; // will probably invoke the server
					self.m_contactsModel.getNextContact(appendNextContact);
				}
			});
		}
	};

	/** Set draft message
	*/
	LiveClientWindow.prototype.__addDraftMessage = function(_message, callback) {
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		const isOutgoing = this.m_interaction.isOutgoing();
		const isBeingComposed = this.m_interaction.isBeingComposed();
		if (!visualize_address && !(isOutgoing && isBeingComposed)) {
			const $digitsDisplay = $("#digitsDisplay");
			$digitsDisplay.hide();
		}
		window.setTimeout(function() {
			callback(true);
		}, 50);
	};

	/** Send draft message
	*/
	LiveClientWindow.prototype.__sendDraftMessage = function(_message, callback) {
		this.startDial(function(status, message) {
			callback(status, message);
		});
	};
	
	/** TODO: FIRE ME!!!
	*/
	LiveClientWindow.prototype.checkAllegroPopup = function(popup)
	{
		let id = this.m_interaction && this.m_interaction.id || "";
		console.info("[LiveClientWindow.prototype.checkAllegroPopup] interaction.id: " + id + ", popup: " + popup);
		if (!this.m_cci_application) {
			inConcert.logger.debug("checkAllegroPopup: No existe una instancia de la app CCI Application for interaction " + id);
			return;
		}

		let domIDs = this.m_cci_application.getDOMIds();
		const $sandBox = $("#" + domIDs.iframe_sandbox);
		let sandBoxSrc = $sandBox.attr("src") || "";

		if (!sandBoxSrc) {
			inConcert.logger.debug("checkAllegroPopup: Popup iframe not open yet for interaction " + id);
			return;
		}

		if (sandBoxSrc !== "about:blank") {
			if (popup.type === "TWEETY" && id && sandBoxSrc.indexOf(id) < 0 && sandBoxSrc.indexOf(encodeURIComponent(id)) < 0 && !this.m_interaction.getRedialProcess()) {
				inConcert.logger.error("checkAllegroPopup: Popup open with another url " + sandBoxSrc + " for interaction " + id);
				$sandBox.attr("src", "about:blank");
			}
			else {
				inConcert.logger.debug("checkAllegroPopup: Popup already open for interaction " + id);
				return;
			}
		}

		this.enableIntegrations(null, /* Interaction */
			true,                     /* Forzar*/
			function(enabled) {
				if (enabled) {
					inConcert.logger.debug("checkAllegroPopup: Popup enabled for interaction " + id);
				}
				else {
					inConcert.logger.debug("checkAllegroPopup: Popup disabled for interaction " + id);
				}
			}
		);
	};

	LiveClientWindow.prototype.canMinimizeWhenOutside = function(target)
	{
		if (!LiveClientWindow.uber.canMinimizeWhenOutside.call(this, target)) {
			return false;
		}

		if (this.m_enabledMinimizeWhenOutside) {
			return true;
		}
		if (this.m_interaction && this.m_interaction.hasVideo()) {
			this.m_enabledMinimizeWhenOutside = true;
		}
		return false;
	};

	/*
		Una vez que ha culminado la carga de la sidebar, con el GoldenLayout habilita:
			1 - El movimiento de la ventana, realizando drag desde la title bar.
			2 - El minimizar al dar click fuera del area de la ventana.
	*/
	LiveClientWindow.prototype.onSidebarIntialisedLayout = function()
	{
		LiveClientWindow.uber.onSidebarIntialisedLayout.call(this);
		this.setDraggable(true);
	};

	//Mostrar boton de minimizar cuando se haya ejecutado el evento initialised del GoldenLayout en el sidebar
	LiveClientWindow.prototype.showMinimizeButton = function()
	{
		this.setMinimizable(true);
	};

	/*
	*/
	LiveClientWindow.prototype.canAutomaticCloseWindow = function (/*params*/) {
		const myInt = this.m_interaction || {};
		const interactionState = (myInt.state || "").toUpperCase();
		console.info("[LiveClientWindow.prototype.canAutomaticCloseWindow]  1-> validate can automatic close phone window - interaction.id: " + myInt.id + ", state: " + interactionState + ", phoneStatus: " + this.m_phoneClient?.status());
		if (!this.m_interaction ) {
			return false;
		}
		if (interactionState === 'CANCELED' || interactionState === 'ABANDONED') {
			console.info("[LiveClientWindow.prototype.canAutomaticCloseWindow]  2-> YES MUST automatic close phone window - interaction.id: " + myInt.id + ", state: " + interactionState + ", phoneStatus: " + this.m_phoneClient?.status());
			return true;
		}
		return false;
	};

	/*
	*/
	LiveClientWindow.prototype.onWindowIsReady = function(interaction = {}) {
		console.log("[LiveClientWindow.prototype.onWindowIsReady] interaction: " + interaction.id + ", state: " + interaction.state + ", initiative: " + interaction.initiative + ", phoneStatus: " + this.getPhoneStatus());
		if (!this.m_phoneClient) {
			console.error("[LiveClientWindow.prototype.onWindowIsReady] NOT PHONE CLIENT FOUND !!");
			return;
		}
		this.m_phoneClient.onWindowIsReady(interaction);
	};

	LiveClientWindow.prototype.setMediaPlayer = VoicemailPlayerClientWindow.prototype.setMediaPlayer;

	LiveClientWindow.prototype.getVoicemailUrl = VoicemailPlayerClientWindow.prototype.getVoicemailUrl;

	LiveClientWindow.prototype.getConferenceStatusLabel = function(interaction) {
		if(["CONFERENCE", "CONFERENCE LOCKED"].includes(interaction.state) && interaction.__last_step && interaction.__last_step.transferType === "BLIND CONFERENCE") {
			let bridgedLegs = interaction.getLegsBy(function(leg) {
				return leg && leg.status === "BRIDGED";
			});
			let callingLeg = interaction.getLegBy(function(leg) {
				return leg && leg.status === "CALLING";
			});
			if(Object.keys(bridgedLegs).length >= 2 && callingLeg) {
				return format("<%=_T('Conference (Adding {0})')%>", callingLeg.number || inConcert.Utils.getIdWithoutVCC(callingLeg.agent));
			}			
		}
	};

	//****	 Initialize window ****
	$(document).ready(function() {
		window.hook_environment();
		// TODO: #9778 - CANY - Docked.
		let options = {from_app: window.from_app, panel_name: window.panel_name};
		let voicemail = window.interaction && window.interaction.type === "VOICEMAIL";
		let taken = window.interaction && window.interaction.isTaken();
		let wrapup = window.interaction && window.interaction.isWrappedUp();
		let finished = window.interaction && window.interaction.isFinished();
		let abandoned = window.interaction && window.interaction.isAbandoned();
		let belongsToMe = window.interaction && window.interaction.belongsToMe();
		let isLocked = window.interaction && window.interaction.isLocked();

		console.log("[phone_client_window] ====> $(document).ready: " + window.interaction?.id || "NONE");

		let WindowToCreate;
		if (inConcert.app.has_allegro_integration()) {
			WindowToCreate = window.archive && (voicemail && VoicemailPlayerClientWindow || PlayerClientWindow) ||
				window.queued && window.from_app === "supervisor" && (voicemail && VoicemailPlayerClientWindow || PlayerClientWindow) ||
				window.monitor && (voicemail && VoicemailAllegroMonitorClientWindow || AllegroMonitorClientWindow) ||
				(voicemail && VoicemailPlayerClientWindow) || ((window.from_app === "supervisor" && !belongsToMe) && ((wrapup && PlayerClientWindow) || ( isLocked && EmptyCallClientWindow))) ||
				LiveClientWindow;
		}
		else {
			//get client window class
			if (voicemail) {
				//1. all voicemails are processed with player
				WindowToCreate = VoicemailPlayerClientWindow;
			}
			else if (!window.interaction || (window.from_app !== "supervisor" && belongsToMe && !finished && !abandoned)) {
				//2. phone when interaction belongs to me and is not finished
				WindowToCreate = LiveClientWindow;
			}
			else if (finished || abandoned || (wrapup && (window.from_app === "supervisor" || !belongsToMe))) {
				//2. call records: archive or supervisor when is wrapup
				WindowToCreate = PlayerClientWindow;
			}
			else if (window.monitor && taken) {
				//3. monitor
				WindowToCreate = MonitorClientWindow;
			}
			else {
				WindowToCreate = EmptyCallClientWindow;
			}
		}

		const client = new WindowToCreate(window.interaction, options);

		window.onReady(client);
		if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
			window.view.apply(client);
		}

		window.view = client;
	});
}());
