/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const format = inConcert.Utils.format;
	const TIMEOUT_MS = 5000;

	//===================================================================================================
	//===================================================================================================
	/**.
	Class schedule_popup.
	* @param params
	*/
	function schedule_popup ( params )
	{
		schedule_popup.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "schedule";
		this.ownerId = "window_schedule_popup_container";
		this.canReposition = true;
		this.from_app = params.from_app;
		this.$interaction = params.$interaction;
	}

	inConcert.Utils.inherit(schedule_popup, inConcert.PopupWindow);

	//----------------------------
	if (typeof(inConcert.schedule_popup) !== "function") {
		inConcert.schedule_popup = schedule_popup;
	}

	/*******************************************************************
	*
	* @param text
	* @param type
	* @param closeDisabled
	* @param timeout_ms
	*/
	schedule_popup.prototype.show_status_message = function(text, type, closeDisabled, timeout_ms )
	{
		var $section = $("#schedule_message");
		$section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#schedule_close_message").hide();
		}
		else {
			$("#schedule_close_message").show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	schedule_popup.prototype.hide_status_message = function() {
		$("#schedule_message").hide();
	};

	/*******************************************************************
	* @param data
	*/
	schedule_popup.prototype.validate = function(data)
	{
		var interaction = this.data.interactions[0];

		if (!interaction.hasPermission( "SCHEDULE", this.from_app )) {
			this.show_status_message("<%=_T('You not has permission for scheduled.')%>", "Error", undefined, TIMEOUT_MS);
			return false;
		}

		if (!interaction.isScheduleEnabled()) {
			this.show_status_message("<%=_T('Scheduled option has not configured on campaign.')%>", "Error", undefined, TIMEOUT_MS);
			return false;
		}

		if (!data.number || !data.number.match(/^[\+ \- \(\)0-9_.*#]+$/)) {
			this.show_status_message("<%=_T('Must enter a number.')%>", "Error", undefined, TIMEOUT_MS);
			return false;
		}
		switch(data.type) {
			case "DATE":
				if (!data.date) {
					this.show_status_message("<%=_T('Must enter a date.')%>", "Error", undefined, TIMEOUT_MS);
					return false;
				}
				if (data.date < new Date()) {
					this.show_status_message("<%=_T('Date invalid.')%>", "Error", undefined, TIMEOUT_MS);
					return false;
				}
				break;
		}

		// success
		return true;
	};

	/*******************************************************************
	* 
	* @param message
	*/
	schedule_popup.prototype.show_message_in_main_window = function( message )
	{
		inConcert.app.getMainWindow().showBannerMessage(message);
	};

	/*******************************************************************
	* schedule interactions
	*/
	schedule_popup.prototype.schedule = function()
	{
		var self = this;
		this.hide_status_message();
		var data = {
			type   : this.__getScheduleType(),
			number : this.__getScheduleNumber(),
			overridedNumber : $("#schedule_form_number").val() === "ANOTHER",
			date   : this.__getScheduleDate()
		};
		if ( !this.validate(data) ) {
			return;
		}
		data.number = data.number.replace(/[^0-9*#]/g, "");

		var interaction = this.data.interactions[0];
		this.before_schedule();

		interaction.schedule(data, function(ok, error) {
			if (ok) {
				var message_type = "Info";
				var message = "<%=_T('Conversation was scheduled successfully!')%>";
				self.on_schedule_success({ message: message, messageType: message_type });
			}
			else {
				self.on_schedule_error(error || "<%=_T('Error scheduling conversation.')%>");
			}
		});
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	schedule_popup.prototype.set_events = function()
	{
		var self = this;

		$("#schedule_popup_schedule_conversation")
			.bind("click", function() {
				self.schedule();
			});

		var close_popup = function() { self.close(); };

		// evento que cierra el dialogo y no realiza cambios
		$("#schedule_popup_cancel_conversation").bind("click", close_popup );
		// evento que cierra el dialogo y no realiza cambios
		$("#close_schedule_popup").bind("click", this.close);

		$("#schedule_close_message")
			.bind("click", function() {
				self.hide_status_message();
			});
			
		$("[name=schedule_form_type]")
			.click(function() {
				$("#schedule_form_date_container").toggle(self.__getScheduleType() === "DATE");
			})
			.filter("#schedule_form_type_now")
			.prop("checked", true)
			.click();
	
		$("#schedule_form_number").change(function() {
			var number = $(this).val();
			$("#schedule_form_number_another").toggle(number === "ANOTHER");
		});
		
		$("#schedule_form_number_another").keypress(function (e) {
			var charDigit = String.fromCharCode(e.keyCode);
			if ( "1234567890ABCD*#".indexOf(charDigit) === -1 ) {
				return e.preventDefault();
			}
		});

		$('#schedule_form_date').datetimepicker({
			format:	"Y-m-d H:i",
			step: 15,
			defaultDate: new Date(),
			defaultTime: new Date(),
			minDateTime: new Date()
		});
	};

	schedule_popup.prototype.__getScheduleType = function() {
		return $("[name=schedule_form_type]:checked").val();
	};

	schedule_popup.prototype.__getScheduleNumber = function() {
		var number = $("#schedule_form_number").val();
		return number === "ANOTHER"? $("#schedule_form_number_another").val() : number;
	};

	schedule_popup.prototype.__getScheduleDate = function() {
		return $("#schedule_form_date").datetimepicker('getValue');
	};

	schedule_popup.prototype.loadAddresses = function() {
		var self = this;
		self.show_waiting();
		//por ahora esto no funciona en batch
		var interaction = this.data.interactions[0];
		interaction.loadContactAddresses(function(addresses) {
			var phones = {};
			var phones_a = [];
			var lastAddress = interaction.getAddress();
			if (lastAddress) {
				phones[lastAddress] = true;
				phones_a.push(lastAddress);
			}
			$.each(addresses, function(_, address) {
				if (address.type === "PHONE") {
					var phone = address.address;
					if (!phones[phone]) {
						phones[phone] = true;
						phones_a.push(phone);
					}
				}
			});
			var options = "";
			const visualize_address = inConcert.app.canVisualizeAddress(null, interaction.campaign, null, true);
			if(visualize_address)
			{
				$.each(phones_a, function(_, phone) {
					options += format("<option value='{0}'>{0}</option>", phone);
				});
			}
			options += "<option value='ANOTHER'><%=_T('Another')%></option>";
			$("#schedule_form_number")
				.html(options)
				.val(lastAddress || "")
				.change();
			self.hide_waiting();
		});
	};

	/*******************************************************************
	* Metodo que crea el markup contenedor de la ventana
	*/
	schedule_popup.prototype.create_markup = function()
	{
		$("#window_schedule_popup_container").remove();
		var $node = $("".concat(
			"<div id='window_schedule_popup_container' class='systemPopWindow windowScheduleContainer windowContainer' hidden>",

				"<div id='schedule_waiting' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

				"<div class='windowTop'>",
					"<div class='windowTitle windowSchedule'>",
						"<div class='windowScheduleTitleText'></div>",
						"<img id='close_schedule_popup' src='/inconcert/apps/agent/images/blank.png' class='windowScheduleButClose' hidden />",
					"</div>",

				"</div>",

				"<div id='schedule_message' class='windowContainerMessages messageError' hidden>",
					"<a id='schedule_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div class='card'>",
						"<div class='card-header'>",
							"<h4 class='mb-2 mt-0'><%=_T('Schedule to:')%></h4>",
						"</div>",
						"<div class='card-block'>",
							"<div class='row pb-2'>",
								"<div class='col-12'>",
									"<div class='form-group d-flex mb-0'>",
										"<div class='form-radiobutton'>",
											"<label class='d-flex align-items-center pl-0' for='schedule_form_type_now'>",
												"<input class='form-radiobutton-input' id='schedule_form_type_now' type='radio' name='schedule_form_type' value='NOW'>",
												"<span><%=_T('Now')%></span>",
											"</label>",
										"</div>",
										"<div class='form-radiobutton'>",
											"<label class='d-flex align-items-center' for='schedule_form_type_date'>",
												"<input class='form-radiobutton-input' id='schedule_form_type_date' type='radio' name='schedule_form_type' value='DATE'>",
												"<span><%=_T('Date')%></span>",
											"</label>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='form-group mb-0' id ='schedule_form_date_container'>",
										"<label class='' for='schedule_form_date'><%=_T('Date')%>:</label>",
										"<input class='form-control hasDatepicker' id='schedule_form_date' type='text'>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='form-group mb-0' id ='schedule_form_number_container'>",
										"<label class='' for='schedule_form_number'><%=_T('Number:')%></label>",
										"<select class='form-control' id='schedule_form_number'></select>",
										"<input class='form-control mt-3' id='schedule_form_number_another' type='text'>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",
				
				"<div class='windowBottom'>",
					"<input id='schedule_popup_cancel_conversation' class='pageButtonGeneric buttonColorWhite' type='button' value='", "<%=_T('CANCEL')%>", "'/>",
					"<input id='schedule_popup_schedule_conversation' class='pageButtonGeneric buttonColorBlue' type='button' value='", "<%=_T('CONFIRM')%>", "'/>",
				"</div>",

			"</div>"));
		$("body").append($node);
		return $node;
	};

	schedule_popup.prototype.get_channel_name = function( type )
	{
		var channels = {
			CALL       : "<%=_T('Phone Call')%>"  ,
			CHAT       : "<%=_T('Web chat')%>"    ,
			FACEBOOK   : "<%=_T('Facebook')%>"    ,
			INSTAGRAM  : "<%=_T('Instagram')%>"   ,
			YOUTUBE	   : "<%=_T('Youtube')%>"     ,
			APPSTORE   : "<%=_T('Appstore')%>"    ,
			GOOGLEPLAY : "<%=_T('Googleplay')%>"  ,
			RCS		   : "<%=_T('Rcs')%>"     	  ,
			IM         : "<%=_T('Chat')%>"        ,
			MAIL       : "<%=_T('E-Mail')%>"      ,
			SMS        : "<%=_T('SMS')%>"         ,
			WHATSAPP   : "<%=_T('Whatsapp')%>"    ,
			TWITTER    : "<%=_T('Twitter')%>"     ,
			WEBCONTACT : "<%=_T('Web contact')%>" ,
			VOICEMAIL  : "<%=_T('Voice mail')%>"
		};
		return channels[type];
	};

	schedule_popup.prototype.set_popup_title = function()
	{
		var popup_title;
		if (this.is_batch){
			var count_interactions = this.data.interactions.length;
			popup_title = format("<%=_T('Batch Schedule - {0} conversations')%>", count_interactions);
		}
		else {
			var interaction_type = this.data.interactions[0].type;
			var channel_name = this.get_channel_name( interaction_type );
			if ( !channel_name ) {
				throw "No channel name found for the conversation of type " + interaction_type;
			}
			popup_title = format("<%=_T('{0} Schedule')%>", channel_name);
		}
		$("#window_schedule_popup_container .windowScheduleTitleText").text(popup_title);
	};

	schedule_popup.prototype.before_schedule = function()
	{
		this.show_waiting("<%=_T('Scheduling, wait moment please')%>");
		this.parent_before_schedule();
	};

	schedule_popup.prototype.on_schedule_success = function( message_object )
	{
		var self = this;
		if ( message_object ) {
			message_object.timeout = TIMEOUT_MS;
			this.show_message_in_main_window( message_object );
		}
		this.parent_on_schedule_success(function() {
			self.hide_waiting();
			self.close();
		});
	};

	/*******************************************************************
	* Metodo que xxxx
	* @param message_err
	*/
	schedule_popup.prototype.on_schedule_error = function( message_err )
	{
		this.close();
		this.show_message_in_main_window({ message: message_err, messageType : "Error", timeout : TIMEOUT_MS });
		this.parent_on_schedule_error();
	};
	
		/*******************************************************************
	* Metodo que xxx
	* @param message
	*/
	schedule_popup.prototype.show_waiting = function( message )
	{
		message = message || "<%=_T('Loading, wait moment please')%>";
		var $waiting = $("#schedule_waiting");
		$waiting.find("[message='true']").text(message);
		$waiting.show();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	schedule_popup.prototype.hide_waiting = function()
	{
		$("#schedule_waiting").hide();
		this.setPosition();
	};

	/*******************************************************************
	* Metodo que visualiza el popup con los schedule e inicializa eventos del popup!
	*/
	schedule_popup.prototype.initialize = function()
	{
		var interactions = this.data.interactions;
		if ( !interactions ) {
			throw "schedule_popup.initialize - interactions is required!!";
		}
		this.is_batch = interactions.length > 1;
		this.from_state = interactions[0].state;
		this.data.container = this.create_markup();
		this.parent_before_schedule = this.data.callbacks.before_schedule || $.noop;
		this.parent_on_schedule_success = this.data.callbacks.on_schedule_success ||
			function(callback) {
				callback();
			};
		this.parent_on_schedule_error   = this.data.callbacks.on_schedule_error   || $.noop;
		this.set_popup_title();
		// set events
		this.set_events();
		this.loadAddresses();
	};

	/*******************************************************************
	* Metodo que cierra el popup
	*/
	schedule_popup.prototype.close = function() {
		this.data.container.remove();
		this.finalize();
	};

}());
