/* jshint smarttabs:true, quotmark:double, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	let FORMAT = inConcert.Utils.format;
	let REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function TicketsAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("TicketsAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("TicketsAPI.Constructor - base URI is required");
		}

		this.path = params.path;
		this.uri = params.uri + "/tickets";
	}

	if (typeof inConcert.TicketsAPI !== "function") {
		inConcert.TicketsAPI = TicketsAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	TicketsAPI.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("TicketsAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("TicketsAPI.prototype.__Get: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (response)    {callback(true, response);},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	TicketsAPI.prototype.__Post = function(uri, callback, data, contentType, dataType)
	{
		if (typeof(uri) !== "string") {
			throw("TicketsAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("TicketsAPI.prototype.__Post: callback variable must be function type.");
		}
		contentType = contentType || "application/json";
		dataType = dataType !== undefined? dataType : "json";

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : contentType === "application/json"? JSON.stringify(data) : data,
			dataType    : dataType,
			contentType : contentType,
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

/********************************************************************************************************

********************************************************************************************************/
	
	/*
	
	*/
	TicketsAPI.prototype.search = function(request, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/agent/tickets/search
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/search", this.uri);

		var request_handler = this.__Post(uri, callback, {
			filter : {
				tickets : $.isArray(request.tickets)? request.tickets : [request.tickets || ""]
			},
			interaction_id : request.interaction_id || null
		});
		return request_handler;
	};
	
	/*
	
	*/
	TicketsAPI.prototype.emit = function(request, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/agent/tickets/emit/<id>
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/emit/{1}", this.uri, encodeURIComponent(request.interaction_id));

		var request_handler = this.__Post(uri, function(status, result) {
			if (status) {
				if (result.status) {
					callback(true, result.ticket);
				}
				else {
					callback(false, result.code);
				}
			}
			else {
				callback(status, result);
			}
		}, {});
		return request_handler;
	};

}());
