 /* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, esversion: 11 */

/* global inConcert, _, Mustache, EmojiPicker */

/*
	Handling GUI events to interact with Twitter client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	const m_maxCharsOnTweet = 280;
	const m_maxCharsOnDirectMessage = 2000;
	let m_maxChars = m_maxCharsOnTweet;
	const format = inConcert.Utils.format;

	const SCROLL_TO = inConcert.Utils.scroll_to;
	const TIME_SELECTED_COMMENT_MS = 5000;

	const LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	const LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";

	/**
	Class TwitterClientWindow.
	Twitter Client Handler
	@param interaction
	@param options
	*/
	function TwitterClientWindow (interaction, options)
	{
		const self = this;
		this.m_templates = inConcert.templates.twitter_client_window;

		//relate the interaction to the view
		this.m_interaction = interaction;
		this.set_enable_twitter_actions(options);

		this.setHorizontalResizingLimits(480);
		this.setView("twitterClientContainer");
		TwitterClientWindow.uber.constructor.call(this, options);

		this.hideWallText();

		$("#twitterClientCharacterCounter").text(m_maxChars);
		$('#twitterWallButtonPostCancel').click(function() {
			if (self.m_interaction.isOutgoing() && self.m_interaction.getEventCount() === 0) {
				self.close();
			}
			else {
				self.hideWallText();
			}
		});

		var $writing_area = this.$get_writing_area_node();
		$writing_area.change(function( /* event */ ) {
			self.__updateCharsCounter();
		});

		if (this.m_interaction.isOutgoing()) {
			this.__build_outgoing_client();
		}
		else {
			$('#cmdDirectMessage').hide();
			$('#cmdMention').hide();
			$('.actionButtonsSeparator').hide();
		}
	}

	inConcert.Utils.inherit(TwitterClientWindow, inConcert.ClientWindow);

	TwitterClientWindow.prototype.getWindowDefaultTitle = function() {
		return "<%=_T('Twitter Window')%>";
	};
	
	TwitterClientWindow.prototype.setTitle = function( window_title )
	{
		if (window_title || !this.m_interaction || this.m_interaction.isBeingComposed()) {
			return TwitterClientWindow.uber.setTitle.call( this, window_title );
		}
		var name = this.m_interaction.getClientName();
		var address = "@" + this.m_interaction.getHumanReadableAddress();
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if (name) {
			if (address && address !== name && visualize_address) {
				name += " (" + address + ")";
			}
		}
		else {
			name =  !visualize_address? "<%=_T('Hidden Address')%>" : address;
		}
		return TwitterClientWindow.uber.setTitle.call( this, name );
	};

	TwitterClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return format( "<%=_T('{0} Unread')%>", this.m_interaction.getUnreadEventCount() );
	};

	TwitterClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	TwitterClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	TwitterClientWindow.prototype.canShowRelatedSections = function()
	{
		return TwitterClientWindow.uber.canShowRelatedSections.call( this ) && this.m_interaction.getUnreadEventCount() > 0;
	};

	TwitterClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	TwitterClientWindow.prototype.clientUpdate = function () 
	{
		this.set_mark_all_unread_read_button_caption();
	};

	TwitterClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T('Mark as unread')%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T('Mark as read')%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	TwitterClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let readClass = "relatedInteractionUnread";
		let message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessage(properties.sorted_index));
		let message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		let author_name = this.m_interaction.getUserName(properties.event);
		let genericAvatar = this.getGenericAvatar();
		let avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div unread_event="true" class="relatedInteraction ', readClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png">',
						'<img src="', genericAvatar, '" width="36px" height="36px"/>',
					'</object>',
				'</div>',
				'<div class="interactionInfoContainer">',
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i class="far fa-chevron-right"></i>',
				'</a>',
			'</div>'));
	};

	TwitterClientWindow.prototype.closeRelated = function( original_index )
	{
		const self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, current_event.__index || self.m_currentEvent ) ) {
					var id = current_event.id_str;
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
		};

		if ( !this.can_show_all_comments ) {
			this.can_show_all_comments = original_index === undefined ? false : true;
		}

		TwitterClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( !current_event || !current_event.id_str ) {
				on_ready();
				return;
			}
			var id = current_event.id_str;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	TwitterClientWindow.prototype.getScrollBasicInfo = function()
	{
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	// TO-DO: Why the need of this?
	TwitterClientWindow.prototype.invalidInteraction = function() {
		TwitterClientWindow.uber.invalidInteraction.call( this ); //invokes the base class method
		$(".interactionActionsButtons").children().hide();
	};

	TwitterClientWindow.prototype.__build_outgoing_client = function()
	{
		const self = this;

		var genericAvatar = this.getGenericAvatar();
		var avatarURI = inConcert.app.getBaseUrl() + this.m_interaction.getAvatarURI() || genericAvatar;

		if (this.m_interaction.getEventCount() === 0) {

			$('#cmdDirectMessage').hide();
			self.update_header_contact_info();

			var screenName = this.m_interaction.completeAddress.User;

			// its an interaction directed to someone
			if (screenName){

				// by default we mention this user
				$('#cmdMention').click(function(){
					$(this).prop("disabled", true);
					$('#cmdDirectMessage').prop("disabled", false);
				});
				this.buildTweet(screenName);
				this.setTitle(format("<%=_T('{0} - New mention')%>", screenName));

				// go and try to get relationship
				// to know if we can dm this user
				inConcert.app.getContactManager().twitterGetRelationship(screenName, this.m_interaction.account, this.m_interaction.campaign, function(relationship) {
					if (relationship && relationship.source.can_dm) {
						$('#cmdDirectMessage').show().click(function() {
							$(this).prop("disabled", true);
							$('#cmdMention').prop("disabled", false);
							self.setTitle(format("<%=_T('{0} - New direct message')%>", screenName));
							self.buildDirectMessage(screenName);
						});
					}
				});
			} else {
				this.setTitle("<%=_T('New tweet')%>");
				this.buildTweet();
				$('#cmdDirectMessage').hide();
				$('#cmdMention').hide();
				$("#twitterWallButtonPostCancel").attr("title", "<%=_T('CANCEL')%>");
				$('.actionButtonsSeparator').hide();
			}

			$("#twitterWallButtonPostCancel").attr("title", "<%=_T('CANCEL')%>");
		}
		else {
			$('#cmdDirectMessage').hide();
			$('#cmdMention').hide();
			$('.actionButtonsSeparator').hide();
			var campaign = this.getCampaign(this.m_interaction.campaign);
			$("#contactID").text(campaign.getDescription());
			$("#contactFromAvatar").attr("data", avatarURI);
		}
	};

	TwitterClientWindow.prototype.set_enable_twitter_actions = function(options)
	{
		if (options) {
			this.m_disableTwitterActions = options.readonly || options.monitor;
			if (this.m_disableTwitterActions) {
				return;
			}
		}

		if (!this.campaignIsOperational().success) {
			this.m_disableTwitterActions = true;
			return;
		}

		if (!this.m_interaction) {
			this.m_disableTwitterActions = true;
			return;
		}

		if (!this.m_interaction.accountIsOperational().success) {
			this.m_disableTwitterActions = true;
			return;
		}

		if (this.m_interaction.isFinished()) {
			this.m_disableTwitterActions = true;
			return;
		}

		this.m_disableTwitterActions = false;
	};

	/**
	 * Called from base class when the account changes is status
	 * @return {[type]} [description]
	 */
	TwitterClientWindow.prototype.on_account_changed = function()
	{
		TwitterClientWindow.uber.on_account_changed.call( this );

		this.set_enable_twitter_actions();
		// TO-DO: Update events buttons
	};

	TwitterClientWindow.prototype.onInteractionUpdated = function(interaction, callback)
	{
		const self = this;
		callback = callback || $.noop;
		TwitterClientWindow.uber.onInteractionUpdated.call( this, interaction,
			function(success) {
				self.set_enable_twitter_actions();
				self.openCurrentEvent();
				callback(success);
			}
		);
	};

	/**
	 * Check if a text contains a valid URL.
	 *
	 * @param  {String} text
	 * @return {Boolean}
	 */
	TwitterClientWindow.prototype.__text_contains_url = function(text){
		return text.match(inConcert.Utils.getUrlRegexp());
	};

	/**
	 * Get the current writing text area for tweet/replying.
	 *
	 * @private
	 * @return {Object} jQuery text area node
	 */
	TwitterClientWindow.prototype.$get_writing_area_node = function(){
		return $("#twitterWallText");
	};

	/*

	*/
	TwitterClientWindow.prototype.$get_writing_text = function(){
		let originalText = this.$get_writing_area_node().val();
		let textInfo = inConcert.Utils.twitterProcessMessage(originalText);
		return textInfo;
	};

	TwitterClientWindow.prototype.$emojiInput = function() {
		return this.$get_writing_area_node().siblings(".emoji-wysiwyg-editor");
	};

	TwitterClientWindow.prototype.clearInput = function() {
		var $input = this.$get_writing_area_node();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	TwitterClientWindow.prototype.appendToInput = function(text) {
		var $input = this.$get_writing_area_node();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
	};
	
	TwitterClientWindow.prototype.setInputText = function(text) {
		this.clearInput();
		this.appendToInput(text);
		this.focusInput();
		this.__updateCharsCounter();
	};

	TwitterClientWindow.prototype.focusInput = function() {
		var $input = this.$get_writing_area_node();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	/**
	 * Update chars counter, will also lookup for URLs
	 * and make the appropriate calculus.
	 *
	 * Also, for performance reasons, it's called in a
	 * throttled mode (because its invoked from a keyup event).
	 *
	 * @return {Function} - The throttled function
	 */
	TwitterClientWindow.prototype.__updateCharsCounter = _.throttle(function() {
		// 1 - obtengo informacion del area de escritura (texto original, texto sanitizado, menciones, urls, etc)
		let textInfo = this.$get_writing_text();

		// 2 - valido si el mensaje tiene URLs
		let textMatchsURLs = textInfo.urls.length > 0;
		$('#twitterClientURLMessage').toggle(textMatchsURLs);

		// 3 - valido el largo total del mensaje (sanitizado)
		let charsDifference = m_maxChars - textInfo.totalSanitizedChars;
		let disabledTweetButton;
		if (textInfo.hasMentions) {
			disabledTweetButton = true;
		}
		else {
			disabledTweetButton = textInfo.totalSanitizedChars === 0 || charsDifference < 0;
		}
		$("#twitterClientCharacterCounter")
			.text(charsDifference)
			.toggleClass("twitterClientCharacterCounterNegative", disabledTweetButton);
		$("#twitterWallButtonPost").prop("disabled", disabledTweetButton);

		// 4 - mentions - valido si tiene el mensaje tiene menciones.
		const $footerMessage = $("#twitterClientFooterMessage");
		if (textInfo.hasMentions) {
			// visualizo el mensaje en el footer
			let message = textInfo.mentions.length === 1 ? "<%=_T('Has included 1 mention, to send the message you must delete.')%>" :
				format("<%=_T('There are {0} mentions included, to send the message you must delete them.')%>", textInfo.mentions.length);
			$footerMessage.text(message).show();
		}
		else {
			// oculto el mensaje
			$footerMessage.hide();
		}
	}, 100);

	TwitterClientWindow.prototype.onInteractionGone = function(interaction) {
		TwitterClientWindow.uber.onInteractionGone.call( this, interaction ); //invokes the base class method
		$('body').addClass('clientReadOnly');
		$(".twitterThreadActions").hide();
	};

	TwitterClientWindow.prototype.__setExternalMessage = function(message = "")
	{
		this.appendToInput(message);
		this.focusInput();
		this.__updateCharsCounter();
	};

	// TO-DO: What wall means? We are mixing facebook terminology?
	TwitterClientWindow.prototype.hideWallText = function() {
		this.clearInput();
		$("#twitterClientCharacterCounter").text(m_maxChars);
		this.setView("twitterClientContainer");
		$('#twitterWallButtonPost').unbind("click");
		this.toggleSendMessage(false);
	};

	TwitterClientWindow.prototype.likeObject = function(index, callback = $.noop) {
		let self = this;
		let like = !self.m_interaction.isLiked(index);
		self.hideStatusMessage();
		self.displayStatusMessage(like? "<%=_T('Liking tweet...')%>" : "<%=_T('Unliking tweet...')%>", "Process", true);
		self.m_interaction.like(like, index, function(status) {
			self.hideStatusMessage();
			if (status) {
				$(like? ".threadActionsLike" : ".threadActionsUnlike")
					.removeClass(like? "threadActionsLike" : "threadActionsUnlike")
					.addClass(like? "threadActionsUnlike" : "threadActionsLike")
					.attr("title", like? "<%=_T('Unlike')%>" : "<%=_T('Like')%>");
				callback(true);
			}
			else {
				self.displayStatusMessage(like? "<%=_T('Error liking tweet')%>" : "<%=_T('Error unliking tweet')%>", 
					"Error");
				callback(false, like? "error liking tweet" : "error unliking tweet");
			}
		});
	};

	/*
	  Post the current action to the server
	 */
	TwitterClientWindow.prototype.post = function(displayMessage, parameters) {
		this.hideStatusMessage();
		if (!this.get_current_event() && !this.m_interaction.isOutgoing()) {
			return;
		}
		this.displayStatusMessage(displayMessage, "Process", true);
		this.m_interaction.newPost(parameters);
	};

	TwitterClientWindow.prototype.sendTweet = function()
	{
		const self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		let textInfo = this.$get_writing_text();
		// TO-DO: This is possible?
		if (textInfo.sanitizedText.length === 0) {
			return;
		}
		const executeFn = function() {
			self.post("<%=_T('Sending Tweet...')%>", {
				method: "tweet",
				status: textInfo.originalText,
				sanitizedText: textInfo.sanitizedText,
				callback: function(success, message, errorType) {
					self.hideStatusMessage();
					if (success) {
						self.hideWallText();
					}
					else {
						if (errorType === "FATAL") {
							$("#twitterWallButtonPost").prop("disabled", true);
							$("#cmdDirectMessage").prop("disabled", true);
							$("#cmdMention").prop("disabled", true);
							$(".threadActionsReply").prop("disabled", true);
							$(".threadActionsRetweet").prop("disabled", true);
							$(".threadActionsDirectMessage").prop("disabled", true);
							$(".threadActionsLike").prop("disabled", true);
							$(".threadActionsUnlike").prop("disabled", true);
						}
						self.displayStatusMessage(message, "Error");
					}
				}
			});
		};
		// ejecuto el post, previamente validando si el mensaje tiene menciones y solicitando confirmacion.
		this.executePostValidateMentions(textInfo, executeFn);
	};

	TwitterClientWindow.prototype.executePostValidateMentions = function(textInfo, executeFn = $.noop) {
		if (textInfo.hasMentions) {
			const message = textInfo.mentions.length === 1 ? "<%=_T('Unable to send the message. The message contains 1 mention.')%>" :
				format("<%=_T('Unable to send the message. The message contains {0} mentions.')%>", textInfo.mentions.length);
			this.displayStatusMessage(message, "Info", true, 5000);
			return;
		}
		executeFn();
	};

	TwitterClientWindow.prototype.sendReplyTweet = function(eventIndex, callback = $.noop) {
		const self = this;
		if (!this.canExecuteInteractionAction()) {
			return callback(false, "can not reply tweet");
		}
		let textInfo = this.$get_writing_text();
		// TO-DO: This is possible?
		if (textInfo.sanitizedText.length === 0) {
			return callback(false, "must enter a text");
		}
		var event = this.m_interaction.getEvent(eventIndex);
		if (!event) {
			throw 'sendReplyTweet - event not found';
		}
		var exclude_reply_user_ids = null;
		if (this.excludedReplies && !$.isEmptyObject(this.excludedReplies)) {
			exclude_reply_user_ids = "";
			$.each(this.excludedReplies, function(userId) {
				exclude_reply_user_ids += (exclude_reply_user_ids? "," : "") + userId;
			});
		}
		const executeFn = function() {
			self.post("<%=_T('Sending Reply Tweet...')%>", {
				method: "replyTweet",
				tweet_id: event.id_str,
				status: textInfo.originalText,
				sanitizedText: textInfo.sanitizedText,
				exclude_reply_user_ids : exclude_reply_user_ids,
				callback: function(success, message) {
					self.hideStatusMessage();
					if (success) {
						self.hideWallText();
						callback(true);
					} else {
						self.displayStatusMessage(message, "Error");
						callback(false, message);
					}
				}
			});
		};
		// ejecuto el post, previamente validando si el mensaje tiene menciones y solicitando confirmacion.
		this.executePostValidateMentions(textInfo, executeFn);
	};

	TwitterClientWindow.prototype.sendRetweet = function(eventIndex, callback = $.noop) {
		const self = this;
		if (!self.canExecuteInteractionAction()) {
			return callback(false, "can not retweet");
		}
		var event = this.m_interaction.getEvent(eventIndex);
		if (!event) {
			throw 'sendRetweet - event not found';
		}
		let textInfo = this.$get_writing_text();

		var parameters;
		if (textInfo.sanitizedText) {
			var userName = event.user ? event.user.screen_name : event.sender.name;
			var tweetId = event.id_str;
			var url = format("https://twitter.com/{0}/status/{1}", 
					encodeURIComponent(userName), encodeURIComponent(tweetId));
			parameters = {
				method: "tweet",
				status: textInfo.originalText,
				sanitizedText: textInfo.sanitizedText,
				attachment_url: url
			};
		}
		else {
			parameters = {
				method: "reTweet",
				tweet_id: event.id_str
			};
		}
		parameters.callback = function(success, message) {
			self.hideStatusMessage();
			if (success) {
				self.hideWallText();
				callback(true);
			}
			else {
				self.displayStatusMessage(message, "Error");
				callback(false, message);
			}
		};

		this.post("<%=_T('Sending Retweet...')%>", parameters);
	};

	/**
	 * Will send a Direct Message to the server.
	 *
	 * @param  {String} screen_name
	 * @param index
	 */
	TwitterClientWindow.prototype.sendDirectMessage = function(index, callback = $.noop) {
		const self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		let textInfo = this.$get_writing_text();
		// TO-DO: This is possible?
		if (textInfo.sanitizedText.length === 0) {
			return;
		}
		var screen_name = $("#twitterWallButtonPost").data("screen_name");
		const executeFn = function() {
			self.post("<%=_T('Sending Direct Message...')%>", {
				method: "sendDirectMessage",
				targetUser: screen_name,
				message: textInfo.originalText,
				sanitizedText: textInfo.sanitizedText,
				index: index,
				callback: function(success, message) {
					self.hideStatusMessage();
					if (success) {
						self.hideWallText();
						callback(true);
					}
					else {
						self.displayStatusMessage(message, "Error");
						callback(false, message);
					}
				}
			});
		};
		// ejecuto el post, previamente validando si el mensaje tiene menciones y solicitando confirmacion.
		this.executePostValidateMentions(textInfo, executeFn);
	};

	/**
	 * Enable controls for DM
	 *
	 * @param  {String} screen_name - User to send the direct message
	 * @param index
	 */
	TwitterClientWindow.prototype.buildDirectMessage = function(screen_name, index) {
		m_maxChars = m_maxCharsOnDirectMessage;
		$("#twitterClientCharacterCounter").text(m_maxChars);
		const self = this;
		$("#twitterWallButtonPost").attr("title", "<%=_T('SEND')%>")
			.data("screen_name", screen_name)
			.unbind("click")
			.bind("click", function() {
				self.sendDirectMessage(index);
			});
		this.setView("twitterMailClientContainer");
		this.$get_writing_area_node().removeAttr("readonly");
		this.clearInput();
		this.focusInput();
		this.__updateCharsCounter();
		this.toggleSendMessage(true);
		this.m_sendMode = {
			mode : "DIRECTMESSAGE",
			index : index
		};
	};

	/**
	 * Enable controls for normal tweet
	 *
	 * @param {String} screen_name
	 */
	TwitterClientWindow.prototype.buildTweet = function(screen_name) {
		m_maxChars = m_maxCharsOnTweet;
		$("#twitterClientCharacterCounter").text(m_maxChars);
		const self = this;
		$("#twitterWallButtonPost").attr("title", "<%=_T('TWEET')%>")
			.unbind("click")
			.bind("click", function() {
				self.sendTweet();
			});
		this.setView("twitterReplyClientContainer");
		var message = screen_name ? format('@{0} ', screen_name) : "";
		this.$get_writing_area_node().removeAttr("readonly");
		this.clearInput();
		this.appendToInput(message);
		this.focusInput();
		this.__updateCharsCounter();
		this.m_sendMode = {
			mode : "TWEET"
		};
	};

	/**
	 * Enable controls for tweet replying
	 *
	 * @param  {Number} event_index - Event index
	 */
	TwitterClientWindow.prototype.buildReplyTweet = function( event_index )
	{
		const self = this;

		m_maxChars = m_maxCharsOnTweet;
		$("#twitterClientCharacterCounter").text(m_maxChars);
		$("#twitterWallButtonPost").attr("title", "<%=_T('REPLY')%>")
			.unbind("click")
			.bind("click", function() {
				self.sendReplyTweet( event_index );
			});
		this.setView("twitterRetweetClientContainer");
		this.$get_writing_area_node().removeAttr("readonly");
		this.clearInput();
		this.focusInput();
		this.toggleSendMessage(true);
		$('#twitterClientURLMessage').hide();
		this.__updateCharsCounter();

		this.excludedReplies = {};
		let sender_user = this.m_interaction.getEventScreenName(event_index);
		let recipients = this.m_interaction.getReplyReceipments(event_index);

		var getUsersDestination = function() {
			let users_destination = "@" + sender_user;
			$.each(recipients.mentions || [], function(_, mention) {
				if (!self.excludedReplies[mention.id]) {
					users_destination = users_destination + " @" + mention.user;
				}
			});
			return users_destination;
		};

		$("#responseSection")
			.addClass("replyTweet")
			.attr("title", "<%=_T('select who receives your response.')%>")
			.html(Mustache.render(
				this.m_templates.response,
				{
					label            : "<%=_T('Reply to')%>",
					users_destination: getUsersDestination()
				})
			)
			.off("click")
			.on("click", function() {
				self.__showSelectRepliesPopup(event_index, function() {
					const users_destination = getUsersDestination();
					$("#reply_users_destination").text(users_destination);
				});
			});
		this.m_sendMode = {
			mode : "TWEET",
			index : event_index
		};
	};

	TwitterClientWindow.prototype.__showSelectRepliesPopup = function (event_index, callback = $.noop) {
		if ($(".selectRecipientsPopup").length) {
			return;
		}
		const self = this;
		var $popup = $(self.m_templates.select_replies);
		var recipients = self.m_interaction.getReplyReceipments(event_index);
		$popup.find("[sender]").append(Mustache.render(self.m_templates.select_replies_reply, recipients.sender));
		var $mentions = [];
		var all = true;
		$.each(recipients.mentions, function(_, mention) {
			mention.mention = true;
			var checked = !self.excludedReplies[mention.id];
			all = all && checked;
			var $mention = $(Mustache.render(self.m_templates.select_replies_reply, mention));
			$mention.find("[selectMention]")
				.prop("checked", checked)
				.change(function() {
					$popup.find("[selectAllMentions]")
						.prop("checked", $popup.find("[selectMention]:not(:checked)").length === 0);
				});
			$mentions.push($mention);
		});
		if ($mentions.length) {
			$popup.find("[mentionsList]").append($mentions);
			$popup.find("[selectAllMentions]")
				.prop("checked", all)
				.change(function() {
					$popup.find("[selectMention]").prop("checked", $(this).is(":checked"));
				});
		}
		else {
			$popup.find("[mentionsList]").hide();
			$popup.find("[allMentionsContainer]").hide();
		}
			
		$popup.find("[confirm_button]").click(function() {
			self.excludedReplies = {};
			$popup.find("[selectMention]:not(:checked)").each(function() {
				self.excludedReplies[$(this).attr("selectMention")] = true;
			});
			$popup.remove();
			callback();
		});
		$popup.find("[close_button]").click(function() {
			$popup.remove();
		});
			
		$("body").append($popup);
	};

	/**
	 * Enable controls for retweeting
	 *
	 * @param  {Number} event_index - Event index
	 */
	TwitterClientWindow.prototype.buildRetweet = function( event_index ) {
		m_maxChars = m_maxCharsOnTweet;
		$("#twitterClientCharacterCounter").text(m_maxChars);
		const self = this;
		$("#twitterWallButtonPost").attr("title", "<%=_T('RETWEET')%>")
			.unbind("click")
			.bind("click", function() {
				self.sendRetweet( event_index );
			});
		this.setView("twitterRetweetClientContainer");
		this.toggleSendMessage(true);
		this.__updateCharsCounter();
		var sanitizedText = inConcert.Utils.escapeAttrib(this.m_interaction.getMessage(this.m_currentEvent));
		var user_name = this.m_interaction.getEventScreenName(this.m_currentEvent);

		$("#responseSection")
			.removeClass("replyTweet")
			.attr("title", "")
			.html(Mustache.render(
				this.m_templates.retweet,
				{
					name : user_name,
					text : sanitizedText
				})
			)
			.off("click");
		this.m_sendMode = {
			mode : "RETWEET",
			index : event_index
		};
	};
	
	TwitterClientWindow.prototype.buildIndexDirectMessage = function(index, event) {
		event = event || this.m_interaction.getEvent(index);
		if (event) {
			this.buildDirectMessage(event.user.screen_name || this.m_interaction.contactAddress, index);
		}
	};

	/**
	 * Load event buttons markup
	 *
	 * @todo Move markup things to a mustache template
	 * @todo Split this function
	 *
	 * @param  {Object} properties
	 * @param  {Number} properties.index
	 * @param  {Object} properties.event
	 * @param  {jquery Object} $parent
	 * @param  {String} suffix
	 */
	TwitterClientWindow.prototype.load_buttons_markup = function( properties, $parent, suffix )
	{
		const self = this;

		var contactManager = inConcert.app.getContactManager();
		var interactionAccountName = this.m_interaction.getAccountDisplayName();
		var isDirectMessage = this.m_interaction.isDirectMessage(properties.index);
		var isDeleted = this.m_interaction.isDeleted(properties.index);
		var isLiked = this.m_interaction.isLiked(properties.index);
		var user = this.m_interaction.getEventUser(properties.index);
		var classSuffix = suffix ? suffix : "";

		var buttons = {
			threadActionsDirectMessage : {
				description : "",
				tooltip:  "<%=_T('Direct Message')%>",
				visible: function (from = "threadmain") {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (from !== "relationship") {
						return false;
					}
					return true;
				},
				relationship : function(relation) {
					return relation.source.can_dm;
				},
				action : function(index) {
					self.buildIndexDirectMessage(index, properties.event);
				}
			},
			threadActionsLike : {
				description : "",
				tooltip: "<%=_T('Like')%>",
				inParent :  true,
				visible: function () {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (isDirectMessage || isDeleted || isLiked) {
						return false;
					}
					const has_permission_to_likes = inConcert.permissions.to_action( 'interaction', 'int_vizualize_action_likes_twitter', null, self.m_interaction.campaign, null, true );
					if (!has_permission_to_likes) {
						return false;
					}
					return properties.event.creator === "contact";
				},
				action : function(index) {
					self.likeObject(index);
				}
			},
			threadActionsUnlike : {
				description :"",
				tooltip: "<%=_T('Unlike')%>",
				inParent :  true,
				visible: function () {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (isDirectMessage || isDeleted || !isLiked) {
						return false;
					}
					const has_permission_to_likes = inConcert.permissions.to_action( 'interaction', 'int_vizualize_action_likes_twitter', null, self.m_interaction.campaign, null, true );
					if (!has_permission_to_likes) {
						return false;
					}
					return properties.event.creator === "contact";
				},
				action : function(index) {
					self.likeObject(index);
				}
			},
			threadActionsReply : {
				description : "",
				tooltip : "<%=_T('Reply')%>",
				visible: function () {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (isDirectMessage || isDeleted) {
						return false;
					}
					//First message can not replied if is outbound
					if (properties.index > 0) {
						return true;
					}
					return properties.event.creator === "contact";
				},
				action : function(index) {
					self.buildReplyTweet(index);
				}
			},
			threadActionsRetweet : {
				description : "",
				tooltip : "<%=_T('Retweet')%>",
				visible: function () {
					var visible = false;

					if (self.m_disableTwitterActions) {
						return false;
					}

					if (isDeleted) {
						return false;
					}
					
					// we need to look for already retweeted tweets from inConcert,
					// if we found some in other events, we don't allow retweet those
					// because it will fail, this only happen with "no retweeted" tweets.
					//
					// TO-DO: This method is kinda of a trick because I don't
					// have the updated data for this events (retweet data)
					// we need to count every retweet from our plattform and retweets outside
					// are out of our scope, more info: https://dev.twitter.com/discussions/3606
					var already_retweeted = _.some(self.m_interaction.getEventList(), function(e /*, index */ )
					{
						var current_text = (e.text || '').toUpperCase();
						var event_text = (properties.event.text || '').toUpperCase();

						return (current_text.indexOf('RT') > -1) &&
							(e.user.screen_name === interactionAccountName) && 
							(current_text.indexOf( event_text ) > -1);
					});

					// when retweets are from outside inconcert (external)
					// have this fields
					if (properties.event.retweeted_status){
						// so we make sure we are not allowing RT from an outside RTed
						if (!already_retweeted && !properties.event.retweeted && properties.event.retweeted_status.user.screen_name !== interactionAccountName){
							visible = true;
						}
					} else {
						// if we are dealing with a retweet from inConcert
						if (!already_retweeted && !properties.event.retweeted	&& !isDirectMessage && user.screen_name !== interactionAccountName){
							visible = true;
						}
					}

					return visible;
				},
				action : function(index) {
					self.buildRetweet(index);
				}
			},
			threadActionsFollow : {
				description : "<%=_T('Follow')%>",
				tooltip: "",
				visible: function (from = "threadmain") {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (from !== "relationship") {
						return false;
					}
					return true;
				},
				relationship : function(relation) {
					return !relation.target.followed_by;
				},
				inParent :  true,
				insertAfterElement: "eventAuthor", // class
				action : function(_, buttonNode) {
					// TO-DO: Refactor with Unfollow
					if (!self.canExecuteInteractionAction()) {
						return;
					}
					self.displayStatusMessage(format("<%=_T('Following {0}, please wait..')%>", user.screen_name), "Process", true);
					const contactID = properties.event && properties.event.addresses[0] && properties.event.addresses[0].contact_id;
					const followers = properties.event && properties.event.followers_count;
					contactManager.twitterFollowUser(user.screen_name, self.m_interaction.account, self.m_interaction.campaign, function(followedOk, message) {
						if (followedOk) {
							self.displayStatusMessage(format("<%=_T('Success following {0}!')%>", user.screen_name), "Info");
							//Change the <a> class to the follow one and change the button binding too
							var currentClass = buttonNode.attr("class");
							currentClass = currentClass.replace("threadActionsFollow", "threadActionsUnfollow");
							buttonNode.removeClass().addClass(currentClass).text(buttons.threadActionsUnfollow.description);
							buttonNode.data("button", buttons.threadActionsUnfollow);
						} else {
							self.displayStatusMessage(message, "Error");
						}
					}, contactID, followers);
				}
			},
			threadActionsUnfollow : {
				description : "<%=_T('Unfollow')%>",
				tooltip: "",
				visible: function (from = "threadmain") {
					if (self.m_disableTwitterActions) {
						return false;
					}
					if (from !== "relationship") {
						return false;
					}
					return true;
				},
				relationship : function(relation) {
					return relation.target.followed_by;
				},
				inParent :  true,
				insertAfterElement: "eventAuthor", // class
				action : function(_, buttonNode) {
					// TO-DO: Refactor with Follow
					if (!self.canExecuteInteractionAction()) {
						return;
					}
					self.displayStatusMessage(format("<%=_T('Unfollowing {0}, please wait..')%>", user.screen_name), "Process", true);
					
					const contactID = properties.event && properties.event.addresses[0] && properties.event.addresses[0].contact_id;
					const followers = properties.event && properties.event.followers_count;
					contactManager.twitterUnfollowUser(user.screen_name, self.m_interaction.account, self.m_interaction.campaign, function(unfollowedOk, message) {
						if (unfollowedOk) {
							self.displayStatusMessage(format("<%=_T('Success unfollowing {0}!')%>", user.screen_name), "Info");
							//Change the <a> class to the follow one and change the button binding too
							var currentClass = buttonNode.attr("class");
							currentClass = currentClass.replace("threadActionsUnfollow", "threadActionsFollow");
							buttonNode.removeClass().addClass(currentClass).text(buttons.threadActionsFollow.description);
							buttonNode.data("button", buttons.threadActionsFollow);
						} else {
							self.displayStatusMessage(message, "Error");
						}
					}, contactID, followers);
				}
			}
		};

		var genericButtonFn = function() {
			var index = parseInt($(this).attr("index"), 10);
			var event = self.m_interaction.getEvent(index);
			if (!event || event.__index === undefined) {
				throw "TwitterClientWindow.prototype.setEventsHandlers - event.__index is required";
			}
			var $button = $(this);
			var button = $button.data("button");
			var $object = $button.parent().parent();
			
			if ( $object.hasClass( "unread" ) ) {
				self.mark_as_read(
					true,
					event.__index,
					function( success ) {
						self.on_readed_event( success, $object );
					}
				);
			}
			button.action(index, $button);
		};

		_.each(buttons, function(button, buttonId){
			if (button.visible()) {
				var $button = $(Mustache.render(self.m_templates.button, {
					class_name : buttonId + classSuffix,
					index      : properties.index,
					description: button.description,
					tooltip    : button.tooltip
				}));
				$button.data("button", button);
				$button.unbind("click").bind("click", genericButtonFn);
				$parent.append( $button );
			}
		});

		//queda por fuera para hacerla facil
		var append_read_unread_button = function() {
			if ((properties.event.creator || "").toLowerCase() !== "contact") {
				return;
			}
			const isRead = self.isReadEvent( properties.event, properties.event.__index || properties.index );
			const $readUnread = $(format(''.concat(
				'<a mark_as_read_unread class="{0}" href="javascript:void(0);"',
					' data-event_index="{1}" data-event_unread="{2}">',
					'{3}',
				'</a>'
				),
				isRead? LNK_THREAD_MARK_AS_UNREAD_CLASS : LNK_THREAD_MARK_AS_READ_CLASS,
				properties.index,
				isRead? "read" : "unread",
				isRead? "<%=_T('Mark as unread')%>" : "<%=_T('Mark as read')%>"
			));
			$readUnread
				.appendTo($parent)
				.click(function() {
					const $mark_read_unread = $(this);

					if ( $mark_read_unread.prop( "disabled" ) ) {
						return;
					}

					const $comment = $mark_read_unread.parent().parent();
					if ( $comment.hasClass( "selected" ) ) {
						return;
					}

					const event_index = $mark_read_unread.data( "event_index" );
					const new_state = $mark_read_unread.data( "event_unread" ) === "unread";
					const event = self.m_interaction.getEvent(event_index);
					if (!event || event.__index === undefined) {
						throw "TwitterClientWindow.prototype.setEventsHandlers - event.__index is required";
					}
					self.mark_as_read(
						new_state,
						event.__index,
						function( success ) {
							self.on_readed_event( success, $comment );
						}
					);
				}
			);
		};

		if (user.screen_name !== interactionAccountName) {
			//now try to turn on all the buttons that depend on the relationship with the user who originated it
			contactManager.twitterGetRelationship(
				user.screen_name,
				this.m_interaction.account,
				this.m_interaction.campaign,
				function( relationship ) {
					if (relationship) {
						_.each(buttons, function(button, buttonId){
							if (button.visible("relationship")) {
								if ( typeof(button.relationship) === "function" ) {
									if (button.relationship(relationship)) {
										var $button = $(Mustache.render(self.m_templates.button, {
											class_name : buttonId + classSuffix,
											index      : properties.index,
											description: button.description,
											tooltip    : button.tooltip
										}));
										$button.data("button", button);
										if (button.inParent) {
											const $parentOfParent = $parent.parent();
											if (!button.insertAfterElement) {
												$parentOfParent.prepend($button);
											}
											else {
												const $afterElement = $parentOfParent.find("." + button.insertAfterElement);
												$button.insertAfter($afterElement);
											}
										} else {
											$parent.append($button);
										}
										$button.click(genericButtonFn);
									}
								}
							}
						});
					}
					append_read_unread_button();
				}
			);
		}
		else {
			append_read_unread_button();
		}
	};

	/**
	 * Based on the event type, will return the according class.
	 * Also, an empty class string can be returned.
	 *
	 * @param  {Number} eventIndex
	 * @return {String}
	 */
	TwitterClientWindow.prototype.getTwitterTypeClass = function(eventIndex) {
		var classes = "";
		if (this.m_interaction.isDeleted(eventIndex)) {
			classes += " deletedTweet";
		}
		if (this.m_interaction.isDirectMessage(eventIndex)) {
			return classes + " twitterDM";
		}
		if (this.m_interaction.isRetweeted(eventIndex)) {
			return classes + " twitterRT";
		}
		if (this.m_interaction.isQuoted(eventIndex)) {
			return classes + " twitterQuoted";
		}
		return classes;
	};

	/**
	 * Based on the event sender, will return the according class.
	 *
	 * @param  {Number} eventIndex
	 * @return {String}
	 */
	TwitterClientWindow.prototype.getTwitterSenderClass = function(eventIndex) {
		if (this.m_interaction.isAgentEvent(eventIndex)) {
			return "agentEvent";
		}
		else if (this.m_interaction.isContactEventIndex(eventIndex)) {
			return "clientEvent";
		}
		return "";
	};

	/**
	 * Based on the event type, will return the according class.
	 * Also, an empty class string can be returned.
	 *
	 * @param  {Object} event
	 * @param  {Number} eventIndex
	 * @return {String}
	 */
	TwitterClientWindow.prototype.getTwitterThreadClass = function(event, eventIndex) {
		var classes = "";
		if (eventIndex === 0) {
			classes = "twitterThreadOrigin";
		}
		else {
			classes = "twitterThreadSubEvent " + (event.topLevelMessage? "topLevelMessage" : "replyMessage");
		}
		return classes;
	};

	TwitterClientWindow.prototype.isReadEvent = function( event, event_index )
	{
		if ( typeof event !== "object" ) {
			throw "TwitterClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index && event_index !== 0 ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};
	
	TwitterClientWindow.prototype.getTwitterReadUnreadClass = function(event, eventIndex) {
		if (!this.isReadEvent(event, eventIndex)) {
			return "unread";
		}
		else {
			return "";
		}
	};

	TwitterClientWindow.prototype.__makeTweetText = function(event) {
		const self = this;
		var text = event.text || "";
		var text_a = Array.from(text);

		function validateIndexes(indexes) {
			return $.isArray(indexes) && indexes.length === 2 && indexes[1] <= text_a.length;
		}

		function replaceText(indexes, html) {
			text_a[indexes[0]] = html;
			for (var i = indexes[0] + 1; i < indexes[1]; i++) {
				text_a[i] = "";
			}
		}

		var entities = event.entities || {};
		//mentions
		$.each(entities.user_mentions || [], function(_, mention) {
			if (!mention.screen_name || !validateIndexes(mention.indices)) {
				return;
			}
			var mentionHtml = format("<a class='mentionTag' title='{0}' mention_user_id='{1}'>{2}</a>",
				mention.name || mention.screen_name, 
				mention.id_str || "undefined", 
				"@" + mention.screen_name);
			replaceText(mention.indices, mentionHtml);
		});
		//hashtags
		$.each(entities.hashtags || [], function(_, hashtag) {
			if (!hashtag.text || !validateIndexes(hashtag.indices)) {
				return;
			}
			var hashtagHtml = format("<a class='hashtagTag' title='{0}'>{1}</a>",
				hashtag.text,
				"#" + hashtag.text);
			replaceText(hashtag.indices, hashtagHtml);
		});
		//urls
		$.each(entities.urls || [], function(_, url) {
			if (!url.url || !validateIndexes(url.indices)) {
				return;
			}
			//check if is mention url
			if (event.quoted_status && event.quoted_status_id_str && url.expanded_url && 
					url.expanded_url.indexOf(event.quoted_status_id_str) >= 0
			) {
				replaceText(url.indices, "");
				return;
			}

			var urlHtml = format("<a class='urlTag' title='{0}' href='{1}' target='_blank'>{2}</a>",
				url.expanded_url || url.url,
				url.expanded_url || url.url,
				url.display_url || url.url);
			replaceText(url.indices, urlHtml);
		});
		//media
		$.each(entities.media || [], function(_, media) {
			if (!media.url || !validateIndexes(media.indices)) {
				return;
			}
			//si no hay url del medio no lo puedo mostrar. Dejo la url.
			var mediaHtml = "";
			var mediaUrl = self.__makeMediaUrl(media);
			if (!mediaUrl.url || (media.type !== "photo" && media.type !== "video" && media.type !== "animated_gif")) {
				mediaHtml = format("<a class='urlTag' title='{0}' href='{1}' target='_blank'>{2}</a>",
				mediaUrl.view,
				mediaUrl.view,
				mediaUrl.view);
			}
			replaceText(media.indices, mediaHtml);
		});

		var newText = "";
		$.each(text_a, function(_, textPart) {
			newText += textPart;
		});
		return newText;
	};

	TwitterClientWindow.prototype.__makeMediaUrl = function(media, event) {
		let mediaUrl = {
			url : media.media_url_https || media.media_url,
			view: media.expanded_url || media.url
		};
		if (media.repositoryFileId) {
			mediaUrl.url = inConcert.app.getBaseDomain() + "/inconcert/apps/repository/download/?id=" + encodeURIComponent(media.repositoryFileId) + "&section=-1&part=-1";
		}
		else if (event && event.direct_message){
			mediaUrl.url = format("{0}/interactions/twitter/get_content_media/?url={1}&accountId={2}&campaignAtVCC={3}",
				inConcert.app.getBaseUrl(),
				encodeURIComponent(mediaUrl.url),
				encodeURIComponent(this.m_interaction.account),
				encodeURIComponent(this.m_interaction.campaign)
			);
		}
		return mediaUrl;
	};

	TwitterClientWindow.prototype.__makeEventsAltTextMedia = function ($container) {
		const $btn_alt_text = $container.find("[btn_alt_text='true']");
		const self = this;
		$btn_alt_text
			.off('click')
			.on('click', function () {
				if (self.m_popup_alt_text) {
					self.m_popup_alt_text.finalize();
				}
				const alt_text_ui = $(this).data("ext_alt_text_uid");
				const popup_id = alt_text_ui;
				self.m_popup_alt_text = new inConcert.PopupWindow({ ownerId: popup_id });
				self.m_popup_alt_text.show({
					ownerId: popup_id,
					eventName: popup_id,
					canReposition: true,
					container: $(this).parent().find("#" + popup_id),
					parentNode: $(this)
				});
			});
			$btn_alt_text.parent().find("[close_button]").click(function () {
			if (self.m_popup_alt_text) {
				self.m_popup_alt_text.finalize();
			}
		});
	};

	TwitterClientWindow.prototype.__makeAltTextMedia = function(media)
	{
		const is_ext_alt_text = media.ext_alt_text && media.ext_alt_text !=="";
		let guid = "alt_text_media_"+ media.id_str + "_" + inConcert.Utils.generateGuid();
		const htmlAltText = is_ext_alt_text? ''.concat(
			'<div id="ext_alt_text_container" class="tw_alt_text_container">',
				'<button btn_alt_text="true" data-ext_alt_text_uid="',guid,'"type="button" class="pageButtonRounded buttonColorWhite">ALT</button>',
				'<div id="',guid,'" class="tw_event_alt_text_media" hidden>',
					'<div class="internalPopUp">',
						'<div class="internalPopUpTitle">',
							'<%=_T("Image description")%>',
							'<button close_button class="closeButton" type="button"><i class="far fa-times"></i></button>',
						'</div>',
						'<label ext_alt_text="true" class="">',
						 media.ext_alt_text ,
						'</label>',
					'</div>',
				'</div>',
			'</div>'):"";
			return htmlAltText;
	};

	TwitterClientWindow.prototype.__makeTweetMedia = function(event) {
		const self = this;
		var mediaHtml = "";
		$.each(event.entities && event.entities.media || [], function(_, media) {
			var mediaUrl = self.__makeMediaUrl(media, event);
			if (!mediaUrl.view && !mediaUrl.url) {
				return;
			}
			const htmlAltText = self.__makeAltTextMedia(media);

			mediaHtml += "<div class='eventMediaContainer'>" ;

			switch(media.type) {
				//images - gif - video
				case "photo":
					mediaHtml += format("<a class='eventImage' title='{0}' href='{1}' target='_blank'><img src='{2}'></a>", 
						 mediaUrl.view,
						mediaUrl.view,
						mediaUrl.url);
					break;
				case "animated_gif":
				case "video":
					mediaHtml += format("<a class='eventVideo' title='{0}'><video controlslist='nofullscreen nodownload' controls><source src='{1}' /></video></a>",
						mediaUrl.view,
						mediaUrl.url);
					break;
			}

			mediaHtml += htmlAltText + "</div>";
		});
		return mediaHtml;
	};

	TwitterClientWindow.prototype.__makeTweetQuoted = function(event) {
		if (!event.quoted_status) {
			return "";
		}
		var genericAvatar = this.getGenericAvatar();
		return Mustache.render(this.m_templates.quoted, {
			genericAvatar: genericAvatar,
			avatar       : this.m_interaction.getEventUserAvatarURI( event.quoted_status ) || genericAvatar,
			author       : this.m_interaction.getUserName( event.quoted_status ),
			text         : this.__makeTweetText( event.quoted_status ),
			media        : this.__makeTweetMedia( event.quoted_status)
		});
	};

	TwitterClientWindow.prototype.__makeTweetLocation = function(event) {
		if (!event.place) {
			return "";
		}
		return format("<div class='eventMessage eventLocation'><p class='eventMessageLocationText'><%=_T('From_location')%> <a class='urlTag' href='https://twitter.com/places/{0}' target='_blank'>{1} </a> · {2}</p></div>", event.place.id, event.place.full_name, event.source);
	};


	/**
	 * Given an event and a event index will return a jquery node.
	 *
	 * @param  {Object}        event
	 * @param  {Number}        event_index
	 * @return {jquery Object} $markup_node
	 */
	TwitterClientWindow.prototype.$get_event_markup = function( event, event_index )
	{
		const self = this;

		var dateTime = this.m_interaction.getDateObject(event_index);

		var classes = this.getTwitterTypeClass( event_index ) + " " + 
			this.getTwitterSenderClass( event_index ) + " " + 
			this.getTwitterThreadClass( event, event_index ) + " " +
			this.getTwitterReadUnreadClass( event, event.__index || event_index);

		var genericAvatar = this.getGenericAvatar();
		let message_text = this.__makeTweetText( event );

		var $markup_node = $(Mustache.render(this.m_templates.tweet, {
			id                        : event.id_str || "",
			event_class               : classes,
			genericAvatar             : genericAvatar,
			avatar                    : this.m_interaction.getEventUserAvatarURI( event ) || genericAvatar,
			author                    : this.m_interaction.getUserName( event ),
			contact_management_enabled: this.m_interaction.accountHasContactManagementEnabled( this.from_app ),
			text                      : message_text,
			media                     : this.__makeTweetMedia( event ),
			quoted                    : this.__makeTweetQuoted( event ),
			location				  : this.__makeTweetLocation( event ),
			date                      : inConcert.Utils.userFormatDate(dateTime, false, true)
		}));


		self.__makeEventsAltTextMedia($markup_node, event.id_str);

		if (event.creator === "contact") {
			const $message_text = $markup_node.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		var $actions = $markup_node.find('.twitterThreadActions');
		this.load_buttons_markup({
				from: "VIEW_EVENT",
				index: event_index,
				event: event
			},
			$actions
		);
		return $markup_node;
	};

	TwitterClientWindow.prototype.markAllReadUnread = function(callback = $.noop) {
		const self = this;
		const mark_as_read = $("#mark_all_read_unread").val() === self. MARK_ALL_READ_BTN_TEXT;
		let events_to_mark = [];

		if ( !mark_as_read ) {
			$.each( self.m_interaction.getEventList() || [],
				function( _, event )
				{
					if ( event.creator === "contact" ) {
						let original_index = event.__index;
						if ( typeof original_index === undefined ) {
							throw "TwitterClientWindow.prototype.setEventsHandlers - event.__index is required";
						}
						// en el array de eventos a marcar como "unread" tiene que ir el indice posta del evento, segun se encuentra en el server.
						events_to_mark.push( original_index );
					}
				});
		}

		self.mark_as_read(
			mark_as_read,
			events_to_mark,
			function( success ) {
				if ( !success ) {
					callback(false);
					return;
				}
				callback(true);
				let all_events = self.m_interaction.getEventList();
				$.each( all_events || [], function( _, event )
				{
					if ( event.creator !== "contact" || !event.id_str ) {
						return true; // continue
					}
					let id = event.id_str;
					let $message = $("#thread_container").find("[data-id='" + id + "']");
					if ( !$message || $message.length === 0 ) {
						return true; // continue
					}
					self.on_readed_event( true /* success */, $message, mark_as_read );
				});
			}
		);
	};

	TwitterClientWindow.prototype.setEventsHandlers = function()
	{
		const self = this;

		TwitterClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if ( !this.m_interaction.isBeingComposed() ) {

			$("#mark_all_read_unread").click( function()
			{
				self.markAllReadUnread();
			});
		}
	};

	/**
	 * Will build every tweet event
	 *
	 * @todo This name is wrong, for twitter we are going to build every event now in
	 * the same $container, this is inherited from client_window.
	 * @param callback
	 *
	 */
	TwitterClientWindow.prototype.openCurrentEvent = function( callback )
	{
		const self = this;
		
		this.updateNavigationButtons();

		var $container = $('.clientThreadContainer');

		var events = this.m_interaction.getEventList();
		var lastNewEvent = null;

		if ($container.children().length > 0) {
			events.forEach(function(currentEvent) {
				if (currentEvent.id_str && !$container.find(format("[data-id={0}]", currentEvent.id_str)).length) {
					lastNewEvent = currentEvent;
				}
			});
		}

		var contact_management_enabled = this.m_interaction.accountHasContactManagementEnabled( this.from_app );
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		events.forEach(function(currentEvent, index){
			// we are not interested in automatic events
			if (currentEvent.creator !== "automatic") {
				var $event_node = self.$get_event_markup(currentEvent, index);
				var $current_event = $container.find(format("[data-id={0}]", currentEvent.id_str));
				if ($current_event.length) {
					if ($current_event.html() !== $event_node.html()) {
						$current_event.replaceWith($event_node);
					}
					else {
						$event_node = $current_event;
					}
				}
				else {
					$container.append($event_node);
				}

				// initialize contact information popup
				if (contact_management_enabled && visualize_address){
					$event_node.find('.eventAuthor')
						.unbind('click')
						.bind('click', function() {
							var name = currentEvent.user ? currentEvent.user.name : currentEvent.sender.name;
							var genericAvatar = self.getGenericAvatar();
							var avatarURI = self.m_interaction.getEventUserAvatarURI(currentEvent) || genericAvatar;
							var address = _.findWhere(currentEvent.addresses, { "source" : "from" }).address;

							var contactWindow = new inConcert.ContactWindow({
								from_app: self.from_app,
								interaction: self.m_interaction,
								name : name,
								avatar : avatarURI,
								addresses : [ address ]
							});
							contactWindow.show();
						}
					);
				}
			}
		});

		if (lastNewEvent) {
			var $message = $container.find( "[data-id='" + lastNewEvent.id_str + "']" );
			if ($message.length) {
				var scroll_data = self.getScrollBasicInfo();
				scroll_data.destination = "item_position";
				scroll_data.$item = $message;
				scroll_data.callback = $.noop;
				SCROLL_TO( scroll_data );
			}
		}

		TwitterClientWindow.uber.openCurrentEvent.call( this, callback );
	};

	TwitterClientWindow.prototype.setView = function(className) {
		$("#mainContainer").
			removeClass().
			addClass(className).
			addClass(this.m_interaction.isOutgoing() ? "twitterOutboundClientContainer" : "twitterInboundClientContainer");
	};

	TwitterClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		let self = this;
		if (!message.replyEvent && message.replyEvent !== 0) {
			var events = this.m_interaction.getEventList();
			var lastContactEvent = null;

			events.forEach(function(currentEvent) {
				if (currentEvent.creator && currentEvent.creator === "contact") {
					lastContactEvent = currentEvent;
				}
			});
			message.replyEvent = lastContactEvent.__index;
		}
		this.hideWallText();
		if (message.privateMessage) {
			const selector = format(".threadActionsDirectMessage[index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length > 0) {
				this.buildIndexDirectMessage(message.replyEvent);
			}
			else {
				return callback(false, "can not send private message");
			}
		}
		else if (message.replyTweet) {
			const selector = format(".threadActionsReply[index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length > 0) {
				this.buildReplyTweet(message.replyEvent);
			}
			else {
				return callback(false, "can not reply tweet");
			}
		}
		else if (message.retweet) {
			const selector = format(".threadActionsRetweet[index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length > 0) {
				this.buildReplyTweet(message.replyEvent);
			}
			else {
				return callback(false, "can not reply tweet");
			}
		}
		else if (message.like || message.unlike) {
			const selector = format(".threadActionsLike[index={0}], .threadActionsUnlike[index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length === 0) {
				return callback(false, format("can not {0} tweet", message.like? "like" : "unlike"));
			}
		}
		else {
			return callback(false, "unsuported message");
		}
		window.setTimeout(function() {
			self.setInputText(message.text || "");
			callback(true);
		}, 1000);
	};

	TwitterClientWindow.prototype.__sendDraftTextMessage = function(message, callback) {
		if (message.privateMessage) {
			this.sendDirectMessage(message.replyEvent, callback);
		}
		else if (message.replyTweet) {
			this.sendReplyTweet(message.replyEvent, callback);
		}
		else if (message.retweet) {
			this.sendRetweet(message.replyEvent, callback);
		}
		else if (message.like) {
			if (this.m_interaction.isLiked(message.replyEvent)) {
				callback(true);
			}
			else {
				this.likeObject(message.replyEvent, callback);
			}
		}
		else if (message.unlike) {
			if (!this.m_interaction.isLiked(message.replyEvent)) {
				callback(true);
			}
			else {
				this.likeObject(message.replyEvent, callback);
			}
		}
		else {
			callback(false, "unsuported message");
		}
	};

	TwitterClientWindow.prototype.hasAttachment = function() {
		return true;
	};

	TwitterClientWindow.prototype.getAttachmentOptions = function() {
		let options = TwitterClientWindow.uber.getAttachmentOptions.call(this);
		options.audio.disabled = true;
		options.video.disabled = true;
		options.file.disabled = true;
		options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
		return options;
	};

	TwitterClientWindow.prototype.hasDragAndDropFiles = function() {
		return true;
	};

	TwitterClientWindow.prototype.getDragAndDropControl = function() {
		return this.$emojiInput();
	};

	TwitterClientWindow.prototype.hasAudioRecord = function() {
		return false;
	};

	TwitterClientWindow.prototype.onAudioRecordStarted = function() {
		$("#sendMessageContainer").hide();
	};

	TwitterClientWindow.prototype.onAudioRecordStopped = function() {
		$("#sendMessageContainer").show();
	};
	
	TwitterClientWindow.prototype.processAttachmentData = function(data) {
		if (!this.m_interaction.isDirectMessage()) {
			data.mode = this.m_sendMode.mode;
			switch(this.m_sendMode.mode) {
				case "DIRECTMESSAGE": {
					data.targetUser = $("#twitterWallButtonPost").data("screen_name");
					data.targetUserId = this.m_interaction.getEventUserId(this.m_sendMode.index);
					data.tweet = JSON.stringify(this.m_interaction.events[this.m_sendMode.index]);
					break;
				}
				default: {
					let exclude_reply_user_ids = null;
					if (this.excludedReplies && !$.isEmptyObject(this.excludedReplies)) {
						exclude_reply_user_ids = "";
						$.each(this.excludedReplies, function(userId) {
							exclude_reply_user_ids += (exclude_reply_user_ids? "," : "") + userId;
						});
					}
					data.exclude_reply_user_ids = exclude_reply_user_ids;
					let event = this.m_interaction.getEvent(this.m_sendMode.index);
					if (this.m_sendMode.mode === "RETWEET") {
						data.attachment_url = format("https://twitter.com/{0}/status/{1}",
							encodeURIComponent(event.user ? event.user.screen_name : event.sender.name),
							encodeURIComponent(event.id_str));
					}
					else {
						data.replyTo = event.id_str;
					}
				}
			}
		}
		return data;
	};

	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'buttonColorGray emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function() {
			var client = new TwitterClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
