/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, esversion: 11 */

/* global inConcert, _ , EmojiPicker, _, CInconcertPlayer*/


/*
	Handling GUI events to interact with whatsapp client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

///////////////////Class Definition

	var m_maxCharsPerMessage = 5000; //real limit is 65536 but...
	var FORMAT = inConcert.Utils.format;

	var SCROLL_TO = inConcert.Utils.scroll_to;
	var TIME_SELECTED_COMMENT_MS = 5000;

	var LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	var LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";
	var LNK_ORIGIN_MARK_AS_READ_CLASS   = "originActionsMarkAsRead";
	var LNK_ORIGIN_MARK_AS_UNREAD_CLASS = "originActionsMarkAsUnread";

	/**
	Class WhatsappClientWindow.
	Whatsapp Client Handler
	@param interaction
	@param options
	*/
	function WhatsappClientWindow (interaction, options) {
		this.m_interaction = interaction;
		this.setHorizontalResizingLimits(480);

		var success = WhatsappClientWindow.uber.constructor.call( this, options );
		if (!success) {
			return;
		}
		
		if (options.readonly || options.monitor) {
			this.toggleSendMessage(false);
		}
		else {
			this.bindSendMessage();
		}

		//load avatar for the contact
		var avatar_uri = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
		$('#contactFromAvatar').attr("data", avatar_uri);

		//initiative class
		$("#mainContainer").addClass(interaction.initiative === "OUTBOUND" ? 
			"whatsappOutboundClientContainer" : "whatsappInboundClientContainer" );

		//Send disclaimer
		if ( inConcert.app.getSendKeyStatus() ) {
			$("#disclaimerMessageSend strong").text("\"<%=_T('Enter Key')%>\"");
		}
	}

	inConcert.Utils.inherit( WhatsappClientWindow, inConcert.ClientWindow );

///////////////////Client Methods

	WhatsappClientWindow.prototype.initialize = function( options )
	{
		WhatsappClientWindow.uber.initialize.call( this, options );
		
		if (this.m_interaction.state === "PENDING" && this.m_interaction && this.m_interaction.contactAddress) {
			$("#setAddressPhone").val(this.m_interaction.contactAddress);
		}

		this.createSpellChecker();
	};

	WhatsappClientWindow.prototype.dispose = function() {
		WhatsappClientWindow.uber.dispose.call( this);
		this.destroySpellChecker();
	};

	WhatsappClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	WhatsappClientWindow.prototype.getWindowDefaultTitle = function() {
		return "<%=_T('Whatsapp Window')%>";
	};
	
	WhatsappClientWindow.prototype.setTitle = function( window_title )
	{
		if (window_title || !this.m_interaction || this.m_interaction.isBeingComposed()) {
			return WhatsappClientWindow.uber.setTitle.call( this, window_title );
		}
		var name = this.m_interaction.getClientName();
		var address = this.m_interaction.getHumanReadableAddress();
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if (name) {
			if (address && address !== name && visualize_address) {
				name += " (" + address + ")";
			}
		}
		else {
			name = !visualize_address? "<%=_T('Hidden Address')%>" : address;
		}
		if (this.m_interaction.isGroup()) {
			name = "<%=_T('Group')%>: " + name;
		}
		return WhatsappClientWindow.uber.setTitle.call( this, name );
	};

	WhatsappClientWindow.prototype.getScrollBasicInfo = function() {
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	WhatsappClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return FORMAT( "<%=_T('{0} Unread')%>", this.m_interaction.getUnreadEventCount() );
	};

	WhatsappClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	WhatsappClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	WhatsappClientWindow.prototype.canShowRelatedSections = function()
	{
		return this.m_interaction.getUnreadEventCount() > 0;
	};

	WhatsappClientWindow.prototype.closeRelated = function( original_index )
	{
		var self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, self.m_currentEvent ) ) {
					var id = current_event.message.id; //TODO
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
			self.focusSendMessageText();
		};

		WhatsappClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( current_event === false || !current_event.message || !current_event.message.id ) {
				on_ready();
				return;
			}
			var id = current_event.message.id;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	WhatsappClientWindow.prototype.invalidInteraction = function() {
		WhatsappClientWindow.uber.invalidInteraction.call( this );
		this.toggleSendMessage(false);
	};

	WhatsappClientWindow.prototype.onInteractionGone = function(interaction) {
		WhatsappClientWindow.uber.onInteractionGone.call( this, interaction );
		this.enableSendMessageButton(false);
		this.enableSendMessageText(false);
	};

	WhatsappClientWindow.prototype.setEventsHandlers = function() {
		const self = this;

		WhatsappClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if ( !this.m_interaction.isBeingComposed() ) {
			$("#mark_all_read_unread").click( function(){
				self.markAllReadUnread();
			});
		}
		else {
			const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
			// Si los addresses no se deben mostrar, no agrego el evento de busqueda.
			if(visualize_address){
			const $setAddressPhone = $("#setAddressPhone");
			$setAddressPhone.off("keyup")
				.on("keyup", function(evt) {
					self.search_contact_keypress(evt, $setAddressPhone);
				})
				.off("click")
				.on("click", function() {
					window.setTimeout(function() {
						let value = $setAddressPhone.val().trim();
						self.search_contact(value, $setAddressPhone);
					});
				}
			);}
		}
		var $container = $("#thread_container");

		// mark as read/unread comment link
		var event_selector = "[mark_as_read_unread]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function () {
				var $mark_read_unread = $( this );

				if ( $mark_read_unread.prop( "disabled" ) ) {
					return;
				}

				var $comment = $mark_read_unread.parent();
				if ( $comment.hasClass( "selected" ) ) {
					return;
				}

				var event_index = $mark_read_unread.data( "event_index" );
				var new_state = $mark_read_unread.data( "event_unread" ) === "unread";
				self.mark_as_read(new_state, event_index, function( success ) {
					self.on_readed_event( success, $comment );
				});
			});
			
		event_selector = "[forward_event]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function () {
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				self.startForward($(this).data( "event_index" ));
			});

		event_selector = "[show_more_selector_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function() {
				let message_id = $(this).attr("show_more_selector_link")
				$("#" + message_id + "_whatsappListMessagesContainer").find("[show_more_selector]").toggle();
				let see_more_button_text = $(this).text() === "<%=_T('See more')%>" ? "<%=_T('See less')%>" : "<%=_T('See more')%>";
				$(this).text(see_more_button_text);
			});			

		/* Gestionar contacto si:
		 1- Esta habilitado la administracion de contactos
		 2- Se pueden visualizar los addresses
		*/
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);

		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) && visualize_address) {
			//Setup contact manager popup when user click in the contact
			event_selector = ".whatsappEventAuthor";
			$container
				.off("click", event_selector)
				.on("click", event_selector, function () {
					var event_index = $( this ).data( "event_index" );
					if ( !event_index && event_index !== 0 ) {
						return;
					}
					var event = self.m_interaction.getEvent( event_index );
					if ( !event || event.creator !== "contact" ) {
						return;
					}

					var address_normalized = inConcert.app.getContactManager().normalizeContactAddress(self.m_interaction.getRawAddress( event_index ), "Phone");

					var contact_window = new inConcert.ContactWindow({
						from_app    : self.from_app,
						interaction : self.m_interaction,
						name        : self.m_interaction.getUserName( event ),
						avatar      : self.m_interaction.getEventUserAvatarURI( event ),
						addresses   : [ address_normalized ]
					});
					contact_window.show();
				});
		}

		var language_timeout = null;
		$('#cboLanguage').change(function()
		{
			if ( language_timeout ) {
				window.clearTimeout(language_timeout);
				language_timeout = null;
			}
			var $cbo = $(this);
			language_timeout = window.setTimeout(function()
			{
				window.clearTimeout(language_timeout);
				language_timeout = null;
				var new_language = $cbo.val();
				self.ChangeSpellcheckLanguage(new_language);
			}, 1000);
		});

		$('#wpButtonSpellcheck').
		off("click")
		.on("click", function () {
			if ( !self.m_spellchecker) {
				return;
			}
			var $spellcheck_feedback = $('#wpSpellcheckFeedback');
			self.m_spellchecker.check(null, function (words) {
				if (_.isArray(words[0]) && words[0].length > 0) {
					$spellcheck_feedback.hide();
				}
				else {
					$spellcheck_feedback.show();
					window.setTimeout(function () {
						$spellcheck_feedback.hide();
					}, 3000);
				}
			});
		});
	};

	WhatsappClientWindow.prototype.onForwardSelectorChange = function() {
		WhatsappClientWindow.uber.onForwardSelectorChange.call(this);
		$("#sendFile").hide();
	};

	WhatsappClientWindow.prototype.stopForward = function() {
		WhatsappClientWindow.uber.stopForward.call(this);
		$("#sendFile").show();
	};

	WhatsappClientWindow.prototype.clientUpdate = function () {
		if (!this.m_interaction.isValid()) {
			return;
		}
		if ( this.was_being_composed && !this.m_interaction.isBeingComposed() && ( this.m_interaction.events || [] ).length > 0 ) {
			this.close();
			return;
		}

		this.enableSendMessageText(true);
		this.__updateCharsCounter();
		this.set_mark_all_unread_read_button_caption();
		this.updateContactDisplay(this.m_interaction);
		this.update_header_contact_info();
	};

	WhatsappClientWindow.prototype.change_client_state = function( new_state ) {
		new_state = WhatsappClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#whatsapp_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		if (this.m_interaction.isBotInternalConsultation()) {
			$solve_container.hide();
		}

		return new_state;
	};

/*
-----------
Send Message
-----------

TODO: (REFACTORING) Revisar todas estas funciones referentes a "send message", las mismas se repiten con el mismo codigo en los clientes "webchat", "whatsapp", "sms", "rcs".

Seria lo mejor moverlas a la clase "clientWindow" asi no repetimos codigo. Aunque la clase client window, tambien la utiliza "mail" el "phone" etc, y estas funciones no son utiles ahi, habria que reviarlo.

Quizas lo mejor es crear un clase a parte, algo asi tipo "clientSendMessage" y quien la necesite que la instancie. VER!!!

*/
	WhatsappClientWindow.prototype.$sendMessageText = function() {
		return this.$sendMessageContainer().find("[sendMessageText]");
	};

	WhatsappClientWindow.prototype.$emojiText = function($input) {
		return $input.siblings(".emoji-wysiwyg-editor");
	};

	WhatsappClientWindow.prototype.$sendMessageEmojiText = function() {
		return this.$emojiText(this.$sendMessageText());
	};

	WhatsappClientWindow.prototype.clearSendMessageText = function() {
		var $input = this.$sendMessageText();
		var $input_emoji = this.$sendMessageEmojiText();
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	WhatsappClientWindow.prototype.focusSendMessageText = function() {
		var $input = this.$sendMessageText();
		var $input_emoji = this.$sendMessageEmojiText();
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	WhatsappClientWindow.prototype.enableSendMessageText = function(enabled) {
		if (this.textDisabled) {
			enabled = false;
		}
		var $input = this.$sendMessageText();
		$input.prop("disabled", !enabled);
		this.$sendMessageEmojiText().prop("contenteditable", enabled);
		$(".emoji-picker-button").toggleClass("emoji-picker-button-disabled", !enabled);
		$("#setAddressPhone").prop("disabled", !enabled);
	};

	WhatsappClientWindow.prototype.appendToSendMessageText = function(text) {
		var $input = this.$sendMessageText();
		var $input_emoji = this.$sendMessageEmojiText();
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
		this.__updateCharsCounter();
	};
	
	WhatsappClientWindow.prototype.setSendMessageText = function(text) {
		this.clearSendMessageText();
		this.appendToSendMessageText(text);
	};

	WhatsappClientWindow.prototype.getSendMessageText = function() {
		return this.$sendMessageText().val();
	};

	WhatsappClientWindow.prototype.$sendMessageButton = function() {
		return this.$sendMessageContainer().find("[sendMessageButton]");
	};

	WhatsappClientWindow.prototype.enableSendMessageButton = function(enabled) {
		if (this.textDisabled) {
			enabled = false;
		}
		this.$sendMessageButton().prop("disabled", !enabled);
	};

	WhatsappClientWindow.prototype.$sendMessageLimit = function() {
		return this.$sendMessageContainer().find("[sendMessageLimit]");
	};

	WhatsappClientWindow.prototype.setSendMessageLimit = function(value) {
		this.$sendMessageLimit()
			.text(value)
			.toggleClass("whatsappClientCharacterCounterNegative", value < 0);
	};

	WhatsappClientWindow.prototype.bindSendMessage = function() {
		var self = this;
		const $emojiEditor = $("#whatsappClientWrite .emoji-wysiwyg-editor");

		//container
		self.toggleSendMessage(true);

		// Enable send with enter key
		if ($emojiEditor && inConcert.app.getSendKeyStatus())
		{
			$emojiEditor.unbind('keypress').bind('keypress', function(event) {
				if (event.which === 13 && !event.shiftKey) {
					self.sendMessage();
				}
			});
		}

		//button
		self.$sendMessageButton().click(function() {
			self.sendMessage();
		});

		//text
		self.$sendMessageText().change(function() {
			self.__updateCharsCounter();
			self.Spellcheck();
		});
		self.focusSendMessageText();

		//limit
		self.__updateCharsCounter();

		//address
		if (self.m_interaction.isBeingComposed()) {
			self.bindAddressSelector();
		}
	};

	WhatsappClientWindow.prototype.setChatTypes = function() {
		this.m_chatTypesAccount = this.m_interaction.account;
		var chatTypes = [
			{ id : "user", name : "<%=_T('User')%>" }
		];
		if (this.m_interaction && this.m_interaction.hasGroups()) {
			chatTypes.push({ id : "group", name : "<%=_T('Group')%>" });
		}
		var html = "";
		$.each(chatTypes, function(_, type) {
			html += FORMAT("<option value='{0}'>{1}</option>", type.id, type.name);
		});
		$("#setAddressType")
			.html(html)
			.change();
	};

	WhatsappClientWindow.prototype.updateAddressSelector = function() {
		if (this.m_chatTypesAccount === this.m_interaction.account) {
			return;
		}
		this.setChatTypes();
	};

	WhatsappClientWindow.prototype.bindAddressSelector = function() {
		var self = this;
		$("#setAddressType").change(function() {
			self.selectChatType($(this).val());
		});
		self.setChatTypes();

		$("#mainContainer").addClass("whatsappNewMessageClientContainer");
		$("#whatsapp_solve_container").find("[set_address_container=type]").show();
		$("#setAddressPhone").focus();
	};

	WhatsappClientWindow.prototype.selectChatType = function(chatType) {
		$("#whatsapp_solve_container").find("[chat_type_container]")
			.hide()
			.filter(FORMAT("[chat_type_container={0}]", chatType))
				.show();
		switch(chatType) {
			case "user":
				this.selectUserChat();
				break;
			case "group":
				this.selectGroupChat();
				break;
		}
	};

	WhatsappClientWindow.prototype.selectUserChat = $.noop;

	WhatsappClientWindow.prototype.selectGroupChat = function() {
		var self = this;
		var $groups = $("#setAddressGroup");
		$groups.html("");
		self.displayStatusMessage("<%=_T('Getting groups')%>...", "Process", true);
		self.m_interaction.listGroups(function(groups) {
			if (groups) {
				self.hideStatusMessage();
				var html = "";
				$.each(groups, function(_, group) {
					html += FORMAT("<option value='{0}'>{1}</option>", group.id, group.subject);
				});
				$groups.html(html);
			}
			else {
				self.displayStatusMessage("<%=_T('Error getting groups')%>", "Error", false, 2000);
			}
		});
	};

	WhatsappClientWindow.prototype.__updateCharsCounter = _.throttle(function() {
		var currentText = this.$sendMessageText().val();
		var currentChars = currentText && currentText.length || 0;
		var remainingChars = m_maxCharsPerMessage - currentChars;
		this.setSendMessageLimit(remainingChars);
		this.enableSendMessageButton(currentChars > 0 && remainingChars >= 0);
	}, 100);

	WhatsappClientWindow.prototype.validatePhoneNumber = function(number) {
		return inConcert.app.validatePhoneNumber(number, "WHATSAPP");
	};

	WhatsappClientWindow.prototype.__getAndValidatePhoneNumber = function() {
		var phoneNumber = $("#setAddressPhone").val();
		if (!this.validatePhoneNumber(phoneNumber)) {
			this.displayStatusMessage( "<%=_T('Invalid phone number')%>", "Error", true, 5000 );
			return false;
		}
		phoneNumber = inConcert.Utils.cleanPhoneNumber(phoneNumber);
		return phoneNumber;
	};

	WhatsappClientWindow.prototype.__getAddressToSendMessage = function() {
		var chatType = $("#setAddressType").val();
		var address = {
			chatType : chatType
		};
		switch(chatType) {
			case "user":
				var phoneNumber = this.__getAndValidatePhoneNumber();
				if (!phoneNumber) {
					return false;
				}
				address.address = phoneNumber;
				break;
			case "group":
				var group = $("#setAddressGroup").val();
				if (!group ) {
					return false;
				}
				address.wappId = group;
				address.address = group;
				address.chatName = $("#setAddressGroup option:selected").text();
				break;
		}
		return address;
	};

	WhatsappClientWindow.prototype.getAddressToSendTemplateMessage = function() {
		if (!this.m_interaction.isBeingComposed()) {
			return null;
		}
		return this.__getAddressToSendMessage() || { error : true };
	};
	
	WhatsappClientWindow.prototype.sendMessage = function(callback = $.noop) {
		var self = this;
		if (self.__sending) {
			return callback(false, "already sending another message");
		}
		var text = self.getSendMessageText();
		if (!text) {
			return callback(false, "must enter a message text");
		}
		var message = {
			text : text
		};

		function beforeSend() {
			self.__sending = true;
			self.clearSendMessageText();
			self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
			self.enableSendMessageButton(false);
		}

		function processResult(success, message) {
			if ( success ) {
				self.clearSendMessageText();
				self.hideStatusMessage();
			}
			else {
				self.enableSendMessageButton(true);
				self.displayStatusMessage(message, "Error", true, 5000);
			}
			self.__sending = false;
			callback(success, message);
		}

		if (self.m_interaction.isBeingComposed()) {
			var address = self.__getAddressToSendMessage();
			if (!address) {
				return callback(false, "Must enter an address");
			}
			beforeSend();
			self.m_interaction.createAndSendMessage({
				attention_level_id : self.getSelectedAttentionLevel(),
				address : address,
				message : message,
				callback : processResult
			});
		}
		else {
			//mark all as read
			if ($("#mark_all_read_unread").val() === this.MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			beforeSend();
			self.m_interaction.sendMessage(undefined, message, processResult);
		}
	};

	WhatsappClientWindow.prototype.SendMessageInternal = function(text, add_to_qna, callback = $.noop) {

		var self = this;
		if (self.__sending) {
			return callback(false, "already sending another message");
		}

		if (!text) {
			return callback(false, "must enter a message text");
		}
		var message = {
			text : text,
            add_to_qna : add_to_qna
		};

		function beforeSend() {
			self.__sending = true;
			self.clearSendMessageText();
			self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
		}

		function processResult(success, message) {

			if ( success ) {
				self.clearSendMessageText();
				self.hideStatusMessage();
			}
			else {
				self.displayStatusMessage(message, "Error", true, 5000);
			}
			self.__sending = false;
			callback(success, message);
		}

		beforeSend();
		self.m_interaction.sendInternalMessage(undefined, message, processResult);
	};

///////////////////Events

	WhatsappClientWindow.prototype.isReadEvent = function( event, event_index ) {
		if ( typeof event !== "object" ) {
			throw "WhatsappClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};

	WhatsappClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( $comment.hasClass( "whatsappThreadOrigin" ) ) {
		// origin link class
			class_mark_as_read = LNK_ORIGIN_MARK_AS_READ_CLASS;
			class_mark_as_unread = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T('Mark as unread')%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T('Mark as read')%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	WhatsappClientWindow.prototype.get_html_read_unread_link = function( event, event_index, is_thread ) {
		var response = {
			html         : '',
			parent_class : ''
		};

		var lnk_mark_as_read_class = LNK_THREAD_MARK_AS_READ_CLASS;
		var lnk_mark_as_unread_class = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( !is_thread ) {
			lnk_mark_as_read_class = LNK_ORIGIN_MARK_AS_READ_CLASS;
			lnk_mark_as_unread_class = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		if ( event.outgoing_pending === true || event.creator === 'agent' ) {
			return response;
		}

		if ( !this.isReadEvent( event, event_index ) ) {
			response.parent_class = ' unread';
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_read_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as read')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'unread'
				);
			}
		}
		else {
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_unread_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as unread')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'read'
				);
			}
		}

		return response;
	};
	
	WhatsappClientWindow.prototype.makeEventForward = function(_, event_index, is_thread) {
		if (!this.isEditable() || !this.m_interaction.canForward()) {
			return;
		}
		return {
			html : FORMAT(
				"".concat(
					'<a forward_event message_control class="{0}" href="javascript:void(0);"',
							' data-event_index="{1}">',
						"<%=_T('Forward')%>",
					'</a>'
				),
				is_thread? "threadActionsForward" : "actionsForward",
				event_index
			)
		};
	};
	
	WhatsappClientWindow.prototype.makeEventActions = function(event, event_index, is_thread) {
		let actions = [];
		actions.push(this.makeEventForward(event, event_index, is_thread));
		actions.push(this.get_html_read_unread_link(event, event_index, is_thread));
		return actions.reduce(function(content, action) {
			content.html += action && action.html || "";
			content.parent_class += action && action.parent_class || "";
			return content;
		}, { html : "", parent_class : "" });
	};
	
	WhatsappClientWindow.prototype.makeMessageStatusMarkupTooltip = function(status) {
		var integrationType = this.m_interaction.getIntegrationType();
		var chatType = this.m_interaction.getChatType();
		switch(integrationType) {
			case "WAVY":
				switch(status) {
					case "SENT":
						switch(chatType) {
							case "group":
								return "<%=_T('Group messages have only Sent status.')%>";
						}
						break;
				}
				break;
		}
		return "";
	};

	WhatsappClientWindow.prototype.makeMessageStatusMarkup = function(status, reason) {
		var text;
		var className;
		var showReason = false;
		switch(status) {
			case "QUEUED":
				text = "<%=_T('Queued')%>";
				className = "deliveryQueued";
				break;
			case "SENDING":
				text = "<%=_T('Sending')%>";
				className = "deliverySending";
				break;
			case "FAIL":
				text = "<%=_T('Failed')%>";
				className = "deliveryFail";
				showReason = true;
				break;
			case "SENT":
				text = "<%=_T('Delivered')%>";
				className = "deliveryDelivered";
				break;
			case "RECEIVED":
				text = "<%=_T('Received')%>";
				className = "deliveryReceived";
				break;
			case "READ":
				text = "<%=_T('Read')%>";
				className = "deliveryRead";
				break;
			default:
				return null;
		}
		switch(reason) {
			case "chat not found":
				reason = "<%=_T('Only can send messages to users with an existent whatsapp chat session.')%>";
				break;
			case "error creating chat":
				reason = "<%=_T('Error starting chat. Check phone is valid and has a whatsapp account.')%>";
				break;
		}
		var tooltip = this.makeMessageStatusMarkupTooltip(status, reason);
		var $status = $(''.concat(
			'<div class="deliveryMessageContainerWrapper ', className, '" title="' , tooltip,'" >',
				//'<div class="deliveryMessageContainer ', className, '" title="', tooltip ,'" >',
				'<div class="deliveryMessageContainer ','" title="', text ,'" >',
					'<label class="deliveryMessageText" >', text ,'</label>',
				'</div>',
			'</div>',
			'<div class="deliveryFailReasonContainer" hidden>',
				'<label class="deliveryFailReasonTxt">', reason || "", '</label>',
			'</div>'
		));
		if (showReason && reason) {
			$status.filter(".deliveryMessageContainer").hover(
				function() {
					$(this).siblings(".deliveryFailReasonContainer").show();
				},
				function() {
					$(this).siblings(".deliveryFailReasonContainer").hide();
				}
			);
		}
		return $status;
	};
	
	WhatsappClientWindow.prototype.__makeMediaPlayer = function($message, $container, content) {
		var self = this;
		var player = null;
		var $play = $container.find(".whatsappMediaPlay");
		var $pause = $container.find(".whatsappMediaPause");
		var $stop = $container.find(".whatsappMediaStop");
		$play.click(function() {
			if (!player) {
				var guid = inConcert.Utils.generateGuid();
				$message.find(".whatsappEventMessage").after("".concat(
					"<div id='", guid, "' class='", content.fileType, "WhatsappPlayer'>",
					"</div>"
				));
				$play.prop("disabled", true);
				player = new CInconcertPlayer({
					containerId : guid,
					url : content.link,
					fileFormat : content.fileExtension,
					fileType : content.fileType,
					success : function() {
						player.play();
						if (content.fileType === "video") {
							$play.hide();
						}
						else {
							$play.prop("disabled", false);
							$play.hide();
							$pause.show();
							$stop.show();
						}
					},
					error : function() {
						self.displayStatusMessage("<%=_T('Error playing audio')%>", "Error", false, 2000);
					}
				});
				player.onEnded(function() {
					if (content.fileType !== "video") {
						$play.show();
						$pause.hide();
						$stop.hide();
					}
				});
				player.load();
				return;
			}
			player.play();
			$play.hide();
			$pause.show();
			$stop.show();
		});
		if (content.fileType !== "video") {
			$stop.click(function() {
				player.stop();
				$play.show();
				$pause.hide();
				$stop.hide();
			});
			$pause.click(function() {
				player.pause();
				$play.show();
				$pause.hide();
			});
		}
	};
	
	WhatsappClientWindow.prototype.__makeContactAddressMarkup = function(address, addressTypesActions) {
		let self = this;
		if (address.type === "Address") {
			return; //TODO
		}
		if (!addressTypesActions[address.type]) {
			addressTypesActions[address.type] = self.getContactAddressActions(address.type);
		}
		let actions = addressTypesActions[address.type];
		let $address = $("".concat(
			"<div class='whatsappContactAddress whatsappContactAddress", address.type, "'>",
				"<label class='whatsappContactAddressValue'>",
					address.address,
				"</label>",
				"<div contactAddressActions class='whatsappContactAddressActions'>",
				"</div>",
			"</div>"
		));
		let $actions = $address.find("[contactAddressActions]");
		$.each(actions, function(_, action) {
			let $action = $("".concat(
				"<button class='buttonColorGray butContactInteraction", action.type.to_camel(), "' ",
					"title='", action.label, "'>",
					"<i></i>",
				"</button>"
			));
			$action
				.data("address", address)
				.data("action", action)
				.click(function() {
					let address = $(this).data("address");
					let action = $(this).data("action");
					let $button  = $(this);
					$button.prop("disabled", true);
					self.startContactAddressInteraction(address.type, action.type, address.address, null, function(status, err) {
						$button.prop("disabled", false);
						if (!status) {
							self.displayStatusMessage(err, "Error", true, 5000);
						}
					});
				});
			$actions.append($action);
		});
		return $address;
	};
	
	WhatsappClientWindow.prototype.__makeContactUrlMarkup = function(url) {
		let $url = $("".concat(
			"<a class='whatsappContactUrl' href='", url.url, "' target='_blank'>",
				url.url,
			"</a>"
		));
		return $url;
	};
	
	WhatsappClientWindow.prototype.__makeContactMarkup = function(contact) {
		let self = this;
		//contact
		let $contact = $("".concat(
			"<div class='whatsappContact'>",
				"<label class='whatsappContactName'>",
					contact.name || "<%=_T('Unknown')%>",
				"</label>",
				"<div class='whatsappContactAddressesContainer'>",
					"<label class='whatsappContactAddressesTitle'>",
						"<%=_T('Addresses')%>:",
					"</label>",
					"<div contactAddresses class='whatsappContactAddresses'>",
					"</div>",
				"</div>",
				"<div class='whatsappContactUrlsContainer'>",
					"<label class='whatsappContactUrlsTitle'>",
						"<%=_T('Urls')%>:",
					"</label>",
					"<div contactUrls class='whatsappContactUrls'>",
					"</div>",
				"</div>",
			"</div>"
		));
		//addresses
		let $addresses = $contact.find("[contactAddresses]");
		let addressTypesActions = {};
		$.each(contact.addresses || [], function(_, address) {
			$addresses.append(self.__makeContactAddressMarkup(address, addressTypesActions));
		});
		if ($addresses.is(":empty")) {
			$addresses.parent().hide();
		}
		//urls
		let $urls = $contact.find("[contactUrls]");
		$.each(contact.urls || [], function(_, url) {
			$urls.append(self.__makeContactUrlMarkup(url));
		});
		if ($urls.is(":empty")) {
			$urls.parent().hide();
		}

		return $contact;
	};

	WhatsappClientWindow.prototype.__makeMediaMarkup = function($message, content, isQuoted) {
		var self = this;
		var quotedClass = "";
		var $media;
		switch(content.type) {
			case "geo":
				var geoLink = "https://maps.google.com/maps?q=" + content.latitude + "%2C" + content.longitude + 
					"&amp;z=17&amp;hl=" + inConcert.app.m_userLanguage;
				$media = $("".concat(
					"<div class='whatsappMedia whatsappGeo'>",
						"<a class='' href='", geoLink, "' target='_blank'>",
							"<i class='far fa-map-marker-alt'></i>",
							"<%=_T('See location')%>",
						"</a>",
					"</div>"
				));
				$message.find(".whatsappEventMessage").after($media);
				quotedClass = "whatsappQuotedGeo";
				break;
			case "contact":
				$media = $("".concat(
					"<div class='whatsappContactsList'>",
					"</div>"
				));
				$.each(content.contacts, function(_, contact) {
					$media.append(self.__makeContactMarkup(contact));
				});
				$message.find(".whatsappEventMessage").after($media);
				quotedClass = "whatsappQuotedContacts";
				break;
			case "attachment":
				if (content.mediaFail) {
					var $mediaFail = $("".concat(
						"<div class='whatsappMediaFail'>",
							"<%=_T('Attachment not available')%>",
						"</div>"
					));
					$message.find(".whatsappEventMessage").after($mediaFail);
					quotedClass = "whatsappQuotedMediaFail";
				}
				else {
					switch(content.fileType) {
						case "image":
							$media = $("".concat(
								"<img class='imagePostedContainer' src='", content.link, "'>"
							));
							if (content.sticker) {
								$media.addClass("stickerImage");
							}
							$message.find(".whatsappEventMessage").after($media);
							quotedClass = "whatsappQuotedImage";
							break;
						case "video":
							$media = $("".concat(
								"<div class='whatsappMedia whatsappVideo'>",
									"<button class='whatsappMediaPlay'>",
										"<i class='fas fa-play'></i>",
									"</button>",
									"<a class='whatsappMediaDownload' href='", content.link, "' target='_blank'>",
										"<i class='far fa-download'></i>",
									"</a>",
								"</div>"
							));
							this.__makeMediaPlayer($message, $media, content);
							$message.find(".whatsappEventMessage").after($media);
							quotedClass = "whatsappQuotedVideo";
							break;
						case "audio":
							$media = $("".concat(
								"<div class='whatsappMedia whatsappAudio'>",
									"<div>",
										"<button class='whatsappMediaPlay'>",
											"<i class='fas fa-play'></i>",
										"</button>",
										"<button class='whatsappMediaPause' hidden>",
											"<i class='fas fa-pause'></i>",
										"</button>",
										"<button class='whatsappMediaStop' hidden>",
											"<i class='fas fa-stop'></i>",
										"</button>",
									"</div>",
									"<a class='whatsappMediaDownload' href='", content.link, "' target='_blank'>",
										"<i class='far fa-download'></i>",
									"</a>",
								"</div>"
							));
							this.__makeMediaPlayer($message, $media, content);
							$message.find(".whatsappEventMessage")
								.text("")
								.after($media);
							quotedClass = "whatsappQuotedAudio";
							break;
						case "file":
							$media = $("".concat(
								"<div class='filesharingContainer'>",
									"<i class='far fa-paperclip'></i>",
									"<label>", content.fileName ,"</label>",
									"<a class='whatsappMediaLink whatsappFile' href='", content.link, "' target='_blank'>",
										"<i class='far fa-download'></i>",
									"</a>",
								"</div>"
							));
							$message.find(".whatsappEventMessage")
								.text("")
								.after($media);
							quotedClass = "whatsappQuotedFile";
							break;
					}
				}
		}
		if (isQuoted && quotedClass) {
			$message.addClass(quotedClass);
		}
	};

	WhatsappClientWindow.prototype.makeEventMarkup = function(event, index)
	{
		const self = this;

		let isFirst = index === 0;
		let id = event.message.id;
		let authorName = this.m_interaction.getUserName( event );
		let authorNumber = this.m_interaction.getUserNumber( event );
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		/*if (authorNumber && authorNumber !== authorName) {
			authorName = FORMAT("{0} ({1})", authorName, (!visualize_address && event.creator === "contact")? "<%=_T('Hidden Address')%>": authorNumber);
		}*/
		if (authorNumber && authorNumber !== authorName) {
			authorName = FORMAT("{0}", authorName);
		}
		let authorId  = this.m_interaction.get_user_event_id( event );
		let genericAvatar = this.getGenericAvatar();
		let avatar = this.m_interaction.getEventUserAvatarURI(event) || genericAvatar;
		let eventDate = this.m_interaction.getDateObject(index);
		let status = this.m_interaction.getMessageStatus(event);
		let forwarded = this.m_interaction.getForwarded(index);
		if (index === 0 && this.m_interaction.isBatchReply()) {
			status = null;
		}
		let content = this.m_interaction.getMessageContent(event);
		let message_text = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib(content.text), { exclude: { email: true } } );
		let class_container = isFirst? 
			"whatsappThreadOrigin " :
			"whatsappThreadSubEvent ";

		switch(event.creator) {
			case "contact":
				class_container += " contactEvent";
				break;
			case "bot":
				class_container += " botEvent";
				break;
			default:
				class_container += " agentEvent";
				break;
		}
		
		if (forwarded) {
			class_container += " forwardedEvent";
		}

		let actions = this.makeEventActions(event, index, !isFirst);
		class_container += actions.parent_class;

		let attr_id =  ' data-id="' + id + '" ';
		let attr_index =  ' data-event_index="' + index + '" ';
		let time = ' data-time="' + eventDate.getTime() + '" ';

		let $message = $( ''.concat(
			'<div class="', class_container, '" ', attr_id, attr_index, time, '>',
				
				forwarded? FORMAT(''.concat(
					'<div class="forwardedEventMark" title="', '<%=_T("Forwarded by agent {0}")%>', '">', 
					'<%=_T("Forwarded")%>', '</div>'), inConcert.Utils.getIdWithoutVCC(forwarded.agent)) : "",
				
				'<div class="whatsappEventAvatar">',
					'<object data="', avatar, '" type="image/png" width="24" height="24">',
						'<img src="', genericAvatar, '" width="24" height="24"/>',
					'</object>',
				'</div>',

				'<div class="whatsappEventMessageContainer">',
					'<a href="javascript:void(0);" class="whatsappEventAuthor" data-event_index="', index , '" data-event_user_id="', authorId , '">', authorName, '</a>',
					'<label message_text="true" class="whatsappEventMessage">', message_text, '</label>',
					content.media ? content.media : "",
				'</div>',

				'<label class="whatsappEventDate">', inConcert.Utils.userFormatDate(eventDate, false, true), '</label>',

				actions.html,
			'</div>'
		));

		this.__makeMediaMarkup($message, content);

		if (content.quoted) {
			let quotedContext = content.quoted;
			let quotedText = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib(quotedContext.text), { exclude: { email: true } } );
			let $quoted = $(''.concat(
				'<div class="whatsappQuotedMessage">',
					'<span class="whatsappQuotedReference">',
						'<%=_T("Quoted message")%>',
					'</span>',
					'<label class="whatsappEventMessage">', quotedText, '</label>',
				'</div>'
			));
			this.__makeMediaMarkup($quoted, quotedContext, true);
			$message.find(".whatsappEventMessage").before($quoted);
		}

		if (event.creator === "contact") {
			const $message_text = $message.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		if (status) {
			$message
				.data("status", status)
				.addClass("status" + status)
				.prepend(this.makeMessageStatusMarkup(status, this.m_interaction.getMessageStatusReason(event)));
		}

		return $message;
	};

	WhatsappClientWindow.prototype.openCurrentEvent = function( callback ) {
		callback = callback || $.noop;
		var self = this;

		var execute_callback = function( success ) {
			self.hideStatusMessage();
			callback( success );
		};

		self.displayStatusMessage( "<%=_T('Please wait, loading messages.')%>", "Process", false );
		var max_append_per_loop = 50;
		var pause_between_loop_ms = 10;
		var index = 0;
		var events = self.m_interaction.events || [];
		var $container = $("#thread_container");
		$container.html("");

		(function addEvents() {
			var added = 0;
			while (index < events.length && added < max_append_per_loop) {
				var $message = self.makeEventMarkup(events[index], index);
				$container.append($message);
				added++;
				index++;
			}
			if (index === events.length) {
				return execute_callback(true);
			}
			window.setTimeout(addEvents, pause_between_loop_ms);
		})();
	};

	WhatsappClientWindow.prototype.__updateEvent = function(index, event) {
		var id = event.message && event.message.id || null;
		if (!id) {
			return;
		}
		var $container = $( "#thread_container" );
		var $currentMessage = $container.find( "[data-id='" + id + "']" );
		if ($currentMessage.length) {
			var status = this.m_interaction.getMessageStatus(event);
			if (!status || status === $currentMessage.data("status")) {
				return;
			}
		}
		var $message = this.makeEventMarkup(event, index);
		if ($currentMessage.length) {
			$currentMessage.replaceWith($message);
		}
		else {
			$container.append($message);
			var scroll_data = this.getScrollBasicInfo();
			scroll_data.destination = "bottom";
			SCROLL_TO( scroll_data );
		}
	};

	WhatsappClientWindow.prototype.__updateEvents = function() {
		var self = this;
		$.each(self.m_interaction && self.m_interaction.events || [], function(index, event) {
			self.__updateEvent(index, event);
		});
	};

	WhatsappClientWindow.prototype.onInteractionUpdated = function(interaction, callback) {
		callback = callback || $.noop;
		var self = this;
		WhatsappClientWindow.uber.onInteractionUpdated.call(this, interaction, function(status) {
			if (self.m_interaction.isBeingComposed()) {
				self.updateAddressSelector();
			}
			else {
				$("#mainContainer").removeClass("whatsappNewMessageClientContainer");
				$("#whatsapp_solve_container").find("[set_address_container]").hide();
			}
			self.__updateEvents();
			callback(status);
		});
	};

	WhatsappClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let readClass = "relatedInteractionUnread";
		let message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessageText(properties.event));
		let message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		let author_name = this.m_interaction.getUserName(properties.event);
		let genericAvatar = this.getGenericAvatar();
		let avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div unread_event="true" class="relatedInteraction ', readClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png" width="48px" height="48px">',
						'<img src="', genericAvatar, '" width="48px" height="48px"/>',
					'</object>',
				'</div>',
				'<div class="interactionInfoContainer">',
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i  class="far fa-chevron-right"></i>',
				'</a>',
			'</div>'));
	};

	WhatsappClientWindow.prototype.hasAttachment = function() {
		return true;
	};

	WhatsappClientWindow.prototype.getAttachmentOptions = function() {
		var integrationType = this.m_interaction? this.m_interaction.getIntegrationType() : "";
		var options = WhatsappClientWindow.uber.getAttachmentOptions.call(this);
		options.file.description = false;
		options.audio.description = false;
		switch(integrationType) {
			case "WAVY":
				options.video.disabled = true;
				options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
				options.audio.type.extensions = [ "aac", "am3", "mp3", "ogg" ];
				options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx" ];
				break;
			case "INFOBIP":
				options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
				options.video.type.extensions = [ "mp4", "3gpp" ];
				options.audio.type.extensions = [ "aac", "amr", "mp3", "ogg", "m4a", "opus" ];
				options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx" ];
				break;
			case "SMOOCH":
				options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx",
					"7z", "csv", "eml", "ics", "key", "log", "neon", "numbers", "odt", "pages", "pps", "ppsx", 
					"txt", "vcf", "xml", "yml", "yaml" ];
				break;
			case "BUSINESS":
				options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
				options.video.type.extensions = [ "mp4", "3gpp" ];
				options.audio.type.extensions = [ "aac", "amr", "mp3", "ogg" ];
				options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx",
					"7z", "csv", "eml", "ics", "key", "log", "neon", "numbers", "odt", "pages", "pps", "ppsx", 
					"txt", "vcf", "xml", "yml", "yaml" ];
				break;
		}
		return options;
	};

	WhatsappClientWindow.prototype.getAttachmentSizeLimitMb = function() {
		var integrationType = this.m_interaction? this.m_interaction.getIntegrationType() : "";
		switch(integrationType) {
			case "WEB":
				return 3;
			default:
				return WhatsappClientWindow.uber.getAttachmentSizeLimitMb.call(this);
		}
	};

	WhatsappClientWindow.prototype.hasDragAndDropFiles = function() {
		return true;
	};

	WhatsappClientWindow.prototype.getDragAndDropControl = function() {
		return this.$sendMessageEmojiText();
	};

	WhatsappClientWindow.prototype.hasAudioRecord = function() {
		return true;
	};

	WhatsappClientWindow.prototype.onAudioRecordStarted = function() {
		this.textDisabled = true;
		this.enableSendMessageText(false);
		this.enableSendMessageButton(false);
	};

	WhatsappClientWindow.prototype.onAudioRecordStopped = function() {
		this.textDisabled = false;
		this.enableSendMessageText(true);
		this.__updateCharsCounter();
	};

	WhatsappClientWindow.prototype.hasTemplateMessages = function() {
		return true;
	};

	WhatsappClientWindow.prototype.appendTemplateMessagesPopup = function($popup) {
		$("#whatsappClientWrite").append($popup);
	};

	WhatsappClientWindow.prototype.iterateEventsToForward = function(index) {
		let self = this;
		$("#thread_container").children().each(function() {
			let $event = $(this);
			let event_index = $event.data("event_index");
			self.enableEventForward($event, event_index, event_index === index);
		});
	};

	WhatsappClientWindow.prototype.addForwardSelector = function($event, $selector) {
		$event.prepend($selector);
	};

	WhatsappClientWindow.prototype.markForwardSelector = function($selector, selected) {
		$selector.parent().parent().toggleClass("forwardSelected", selected);
	};

	WhatsappClientWindow.prototype.getForwardMessagePopupAddressLabel = function() {
		return "<%=_T('Number')%>";
	};

	WhatsappClientWindow.prototype.validateForwardMessageAddress = function(phoneNumber) {
		if (typeof(phoneNumber) === "object") {
			phoneNumber = phoneNumber.SanitizedId;
		}
		if (!this.validatePhoneNumber(phoneNumber)) {
			this.displayStatusMessage( "<%=_T('Invalid phone number')%>", "Error", true, 5000 );
			return false;
		}
		phoneNumber = inConcert.Utils.cleanPhoneNumber(phoneNumber);
		return {
			chatType : "user",
			address : phoneNumber
		};
	};

	WhatsappClientWindow.prototype.getForwardMessageAddressType = function() {
		return "Phone";
	};
	
	WhatsappClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		this.setSendMessageText(message.text || "");
		callback(true);
	};
	
	WhatsappClientWindow.prototype.__sendDraftTextMessage = function(_, callback) {
		this.sendMessage(callback);
	};

	WhatsappClientWindow.prototype.ChangeSpellcheckLanguage = function( new_language )
	{
		if( !this.m_spellchecker ) {
			return;
		}
		this.m_spellchecker.config.lang = new_language;
		this.m_spellchecker.webservice.config.lang = new_language;
		this.m_spellchecker.webservice.defaultConfig.data.lang = new_language;

		$('#incorrect-word-list').hide();
		this.Spellcheck();
	};

	WhatsappClientWindow.prototype.HasToSpellcheck = function ()
	{
		if(!this.m_spellchecker) {
			return false;
		}

		var current_text = this.getSendMessageText();
		return current_text.trim() !== '';
	};

	WhatsappClientWindow.prototype.Spellcheck = function ()
	{
		
		if(this.HasToSpellcheck()) {
			if (this.m_spellchecker_timeout) {
				this.m_spellchecker_timeout = null;
			}
			const self = this;
			this.m_spellchecker_timeout = setTimeout(() => {
				if (self.m_spellchecker) {
					self.m_spellchecker.check();
				}
			}, 500);
		}
		else {
			$('#incorrect-word-list').find('.spellchecker-incorrectwords').empty().parent().hide();
		}
	};

	WhatsappClientWindow.prototype.createSpellChecker = function () {
		if ( !$.hasOwnProperty("SpellChecker")) {
			return false;
		}
		const self = this;
		this.destroySpellChecker();
		const $body = $("body");
		const validate_spellchecker_permissions = function()
		{
			// add to dictionary permission
			const allowed_add_to_dictionary = self.hasPermissionToAction( 'SPELL_CHECKER_ADD_WORD' );
			const $add_dictionary = $body.find('.ignore-forever');
			if ( allowed_add_to_dictionary ) {
				$add_dictionary.show();
			}
			else {
				$add_dictionary.hide();
			}
		};

		const vcc = inConcert.app.get_vcc_id();
		const language = this.get_language();
		const $text = this.$sendMessageText();
		this.m_spellchecker = new $.SpellChecker($text, {
			lang: language,
			parser: 'text',
			webservice: {
				path: inConcert.app.getBaseDomain() + '/inconcert/apps/spellchecker/spellcheck/' + vcc,
				driver: 'pspell'
			},
			suggestBox: {
				position: 'above'
			},
			incorrectWords: {
				container: '#incorrectWordList'
			}
		});

		var $incorrect_words_list = $('#incorrectWordList');
		var $incorrect_words = $incorrect_words_list.find('.spellchecker-incorrectwords');
		// Bind spellchecker handler functions
		this.m_spellchecker.on('check.success', function () {
			$incorrect_words.empty().parent().hide();
		});
		
		this.m_spellchecker.on('replace.word', function () {
			// if there are no more words to be replaced, hide the container
			if ($incorrect_words.children().length === 0) {
				$incorrect_words_list.hide();
			}
			const unicodeToImageText = window.emojiPicker.unicodeToImage($text.val());
			const $emoji = self.$sendMessageEmojiText();
			$emoji.html(unicodeToImageText);
			$emoji.change();
			self.__updateCharsCounter();
		});

		const $spellcheck_feedback = $('#wpSpellcheckFeedback');
		this.m_spellchecker.on('check.fail', function () {
			if ($text.val().trim() !=="") {
				$incorrect_words_list.show();
				$spellcheck_feedback.hide();
				validate_spellchecker_permissions();
				return;
			}
			$incorrect_words.empty().parent().hide();
		});

		return true;
	};

	WhatsappClientWindow.prototype.destroySpellChecker = function()
	{
		if ( !this.m_spellchecker ) {
			return;
		}
		this.m_spellchecker.destroy();
		this.m_spellchecker = null;
	};

///////////////////Iframe Constructor

	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function()
		{
			var client = new WhatsappClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
