/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50,
  quotmark: false, -W083, esversion: 11
*/

/* global inConcert, Mustache, _, console, EventEmitter, CollectGarbage */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables..

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const dialog_container_tpl = '<div id="dialog_container_{{ id }}"></div>';
	const iframe_tpl = '<iframe id="dialog_{{ id }}" src="{{ url }}"></iframe>';

	/*

	*/
	const m_analyticsInfo = {};
	const AnalyticsSend = (info = {}) => {
		// Si NO existe el objeto "window.gtag" el SPNIPPET de GoogleAnalytics NO se ha incluido en el HTML.
		if (!window.gtag)
			return;
		m_analyticsInfo.action_id     = info.action_id     || m_analyticsInfo.action_id;
		m_analyticsInfo.entity_id     = info.entity_id     || m_analyticsInfo.entity_id;
		m_analyticsInfo.entity_type   = info.entity_type   || m_analyticsInfo.entity_type;
		m_analyticsInfo.item_id       = info.item_id       || m_analyticsInfo.item_id;
		m_analyticsInfo.sub_action_id = info.sub_action_id || m_analyticsInfo.sub_action_id;
		m_analyticsInfo.page_title    = info.page_title    || m_analyticsInfo.page_title;
		m_analyticsInfo.window_name   = info.window_name   || m_analyticsInfo.window_name;
		if (info.customInfo) {
			m_analyticsInfo.customInfo = info.customInfo;
		}
		inConcert.Utils.AnalyticsSend(m_analyticsInfo);
	};

	//--------------------------------------------
	//
	function DialogHandle (_, window, dialog_ui) {
		// client will be set at a later time
		this.client = null;
		this.closed = false;
		this.window = window;
		this.dialog_ui = dialog_ui;
		return this;
	}

	inConcert.Utils.inherit(DialogHandle, EventEmitter);

	DialogHandle.prototype.isValid = function() {
		return !!this.client;
	};

	DialogHandle.prototype.activate = function() {
		// TODO: usar el focus del dialogo?
		if (this.dialog_ui) {
			const $parent = this.dialog_ui.parent();
			const $titlebar = $parent.find(".ui-dialog-titlebar");
			$titlebar.trigger("click");
		}
	};

	/**
	This hack is to keep the rest of the code working.
	*/
	DialogHandle.prototype.getWindow = function() {
		return {
			view: this.client
		};
	};

	/**
	Returns the actual 'window' object.
	*/
	DialogHandle.prototype.getNativeWindow = function() {
		return this.window;
	};

	DialogHandle.prototype.isClosed = function() {
		return this.closed;
	};

	/**
	 * @brief Closes the client instance.
	 *
	 * @note This function does not actually "close" the dialog because the
	 * dialog itself is closed by sending a message to dialog-ui. The HTML
	 * elements involved in this dialog are also removed somewhere else.
	 */
	DialogHandle.prototype.close = function() {
		//console.debug("DialogHandle.prototype.close - START");
		if (this.isClosed()) {
			// should not happen again, code is not reentering anymore
			//console.warn("DialogHandle.prototype.close", "already closed");
			//console.debug("[DialogHandle.prototype.close] END 1==> Unable to close window - already closed");
			return;
		}
		if (!this.client) {
			//console.debug("[DialogHandle.prototype.close] END 2==> Unable to close window - current window is not ready !!");
			return false;
		}
		const closingParams = {
			interactive: !this.closeFunctionCalled
		};
		this.closeFunctionCalled = false;
		const closingResult = this.client.onWindowClosing(closingParams);
		if (closingResult === false) {
			//console.debug("[DialogHandle.prototype.close] END 3==> Unable to close window - onWindowClosing = false");
			return false;
		}
		if (closingResult === 'hide') {
			//console.debug("[DialogHandle.prototype.close] END 4==> Unable to close window - onWindowClosing = hide");
			const hiddenData = this.client.onWindowHidden();
			this.emit("hide", hiddenData);
			return false;
		}
		this.closed = true;
		const closeData = this.client.onWindowClosed();
		//console.debug("[DialogHandle.prototype.close] END 5==> Close. closeData: " + JSON.stringify(closeData || {}));
		this.emit("close", closeData);
		this.dialog_ui = null;
		this.client = null;
		this.window = null;
	};

	DialogHandle.prototype.setVisible = function(_) {
		// TODO: usar el focus del dialogo?
		if (this.dialog_ui) {
			this.dialog_ui.parent().find(".ui-dialog-titlebar").trigger("click");
		}
	};

	//--------------------------------------------
	//
	function DialogManager() {
		var self = this;

		//windows are indexed by its id
		this.m_openedWindows = [];
		//array of iframes sorted in a LRU fashion
		this.m_sortedIframes = [];
		this.m_userPresent = true;
		this.m_focusGained = true;

		//only for the main window
		if (window.location.pathname === "/inconcert/apps/agent/") {
			var resetTimer = _.debounce(function () {
				self.setIdleThreshold(inConcert.Storage.get("idleThreshold"));
			}, 100);

			$(document).mousemove(resetTimer);
			$(document).keypress(resetTimer);
		}
	}

	DialogManager.prototype.setIdleThreshold = function(minutes) {
		if (this.idleTimeout) {
			clearTimeout(this.idleTimeout);
			this.idleTimeout = null;
		}
		var self = this;
		if (typeof(minutes) === "number") {
			this.idleTimeout = setTimeout(function () {
				self.stateIdle = true;
				if (self.onUserIdle) {
					self.onUserIdle();
				}
			}, minutes * 60000);
		}
		if (this.stateIdle === true) {
			this.stateIdle = false;
			if (this.onUserPresent) {
				this.onUserPresent();
			}
		}
	};

	DialogManager.prototype.applicationHasFocus = function() {
		return window.document.hasFocus();
	};

	DialogManager.prototype.isUserPresent = function() {
		return this.m_userPresent;
	};

	// this function is called everytime a window gains focus
	DialogManager.prototype.onWindowActivated = function() {
		if (!this.m_focusGained) {
			this.m_focusGained = true;
			if (typeof(this.onApplicationGainFocus) === "function") {
				this.onApplicationGainFocus();
			}
		}
	};

	DialogManager.prototype.__get_dialog_options = function(parameters) {
		// size and position options
		var width = parameters.size && parameters.size.width ? parameters.size.width : 400;
		var height = parameters.size && parameters.size.height ? parameters.size.height : 600;

		// boundaries
		var min_width = parameters.size && parameters.size.min_width ? parameters.size.min_width : 0;
		var max_width = parameters.size && parameters.size.max_width ? parameters.size.max_width : 0;

		var min_height = parameters.size && parameters.size.min_height ? parameters.size.min_height : 0;
		var max_height = parameters.size && parameters.size.max_height ? parameters.size.max_height : 0;

		// offsets
		var left = parameters.offset && parameters.offset.left ? parameters.offset.left : (screen.width / 2) - (width / 2);
		var top = parameters.offset && parameters.offset.top ? parameters.offset.top : (screen.height / 2) - (height / 2);

		var resizable = true;
		if ( parameters && typeof( parameters.resizable ) === "boolean" ) {
			resizable = parameters.resizable;
		}
		return {
			position: { my: top.toString(), at: left.toString(), of: parameters.of ? parameters.of : "body" },
			width : width,
			height : height,
			minWidth : min_width,
			maxWidth : max_width,
			minHeight : min_height,
			maxHeight : max_height,
			title : parameters.title ? parameters.title : "",
			closeOnEscape: false,
			closeText: "<%=_T('Close Window')%>",
			resizable: resizable
		};
	};

	DialogManager.prototype.__get_dialog_extended_options = function(parameters)
	{
		let maximizable = true;
		let minimizable = true;
		if (parameters) {
			if (typeof(parameters.maximizable) === "boolean") {
				maximizable = parameters.maximizable;
			}
			if (typeof(parameters.minimizable) === "boolean") {
				minimizable = parameters.minimizable;
			}
		}
		return {
			"maximizable"     : maximizable,
			"closable"        : true,
			"minimizable"     : minimizable,
			"minimizeLocation": "left",
			"collapsable"     : false,
			"titlebar"        : false
		};
	};

	// this function is called everytime a window loses focus
	DialogManager.prototype.onWindowDeactivated = function() {
		if (this.m_focusGained && !this.applicationHasFocus()) {
			this.m_focusGained = false;
			if (typeof(this.onApplicationLoseFocus) === "function") {
				this.onApplicationLoseFocus();
			}
		}
	};

	// given an iframe move it to the end of the list and correct all z-indexes according to their position in the list
	DialogManager.prototype.correctZIndex = _.throttle(function($iframe) {
		this.m_sortedIframes = _.filter(this.m_sortedIframes, function(e) { return e !== $iframe[0]; });
		this.m_sortedIframes.push($iframe[0]);
		_.each(this.m_sortedIframes, function(e, i) {
			var zIndex = 1000 + i * 10;
			$(e).css("z-index", zIndex);
			//this __dialog is the jquery-ui dialog
			//it's set when the iframe is pushed onto the list
			if (e.__dialog) {
				e.__dialog.parent().css("z-index", zIndex - 1);
			}
		});
	}, 100);

	/**
	 * Creates a new window dialog with the following parameters
	 *
	 * @param  {Object}  parameters           - Window options.
	 * @param  {Object}  parameters.size      - (height, width) - defaults to 400x600 if unespecified
	 * @param  {Boolean} parameters.visible   - true if unespecified
	 * @param  {String}  parameters.title     - Window title
	 */
	DialogManager.prototype.createDialog = function(node, parameters) {
		var self = this;
		var options = self.__get_dialog_options(parameters);
		return node.dialog(options);
	};

	$.fn.purgeFrame = function() {
		var deferred;
		var browserName = inConcert.Utils.browserDetect().name;
		if (browserName === "IE" || browserName === "EDGE") {
			deferred = purge(this);
		} else {
			this.remove();
			deferred = $.Deferred();
			deferred.resolve();
		}
		return deferred;
	};

	function purge($frame) {
		var sem = $frame.length, deferred = $.Deferred();
		$frame.on("load", function() {
			var frame = this;
			frame.contentWindow.document.innerHTML = '';
			sem -= 1;
			if (sem <= 0) {
				$frame[0].contentWindow.document.write('');
				$frame[0].contentWindow.close();
				$frame.remove();
				if( typeof CollectGarbage === "function") {
					CollectGarbage();
				}
				deferred.resolve();
			}
		});
		$frame.attr('src', 'about:blank');
		if ($frame.length === 0) {
			deferred.resolve();
		}
		return deferred.promise();
	}

	/**
	 * Creates a new window (iframe) dialog with the following parameters
	 *
	 * @param  {Object}  parameters           - Window options.
	 * @param  {Object}  parameters.page      - The page to be opened
	 * @param  {Object}  parameters.url       - The url to the page
	 * @param  {Object}  parameters.size      - (height, width) - defaults to 400x600 if unespecified
	 * @param  {String}  parameters.windowID  - The windowID (could be the interaction id too)
	 * @param  {Boolean} parameters.visible   - true if unespecified
	 * @param  {String}  parameters.title     - Window title
	 */

	DialogManager.prototype.createWindow = function(parameters)
	{
		const self = this;

		console.log("[DialogManager.createWindow] ==> Start creating Window. parameters.windowID: " + parameters.windowID);

		// google analytics
		const analyticsParams = {
			entity_id  : parameters.windowID,
			page_title : parameters.title,
			window_name: parameters.page,
			action_id  : "window_open"
		};
		AnalyticsSend(analyticsParams);

		var set_iframe_position = function(iframe, ui_dialog) {
			// // Logueo en la consola del Navegador las posiciones actuales del Dialog UI
			// let dialog_ui_position = ui_dialog.dialog("option", "position");
			// log_dialog_positions(dialog_ui_position, "SET iFrame Positions");

			var dialog_container = ui_dialog.parent();
			var dialog_position = dialog_container.position();
			var dialog_width = parseInt(dialog_container.width(), 10);
			var dialog_height = parseInt(dialog_container.height(), 10);

			// TO:DO: Get this position querying the jquery object
			var title_header = 38; //ACÁ SE SETEA EL ALTO DE LAS VENTANAS

			// set new position and size
			iframe.css({
				left : dialog_position.left,
				top : dialog_position.top + title_header,
				width : dialog_width,
				height : dialog_height - title_header,
				position : "absolute"
			});
		};

		let url = parameters.url ? parameters.url : "/inconcert/apps/agent/pages/" + parameters.page + ".<%=_L%>.html";
		let iframe_html = Mustache.render(iframe_tpl, {id : parameters.windowID, url : url});
		let iframe_node = $(iframe_html);
		let dialog_container_html = Mustache.render(dialog_container_tpl, {id : parameters.windowID});
		let dialog_container = $(dialog_container_html);

		if (typeof(parameters.onComplete) === "function") {
			iframe_node.load(parameters.onComplete);
		}

		var set_events = function(is_enabled) {
			dialog_container.off("clickoutside");
			if (!is_enabled) {
				return;
			}
			//minimize all windows if clicked somewhere
			dialog_container.on("clickoutside", function(e) {
				//console.log("[DialogManager.createWindow] clickoutside");
				if ((e.target.tagName || "").toUpperCase() === "HTML") {
					// cuando se mueve la ventana al top del main principal
					return;
				}
				const target = e.target;
				const $target = $(target);
				// cuando esta minimizado
				const is_fixed = $target.is("#dialog-extend-fixed-container *");
				if (is_fixed) {
					return;
				}
				// otro
				const is_dialog = $target.is(".ui-dialog *");
				if (is_dialog) {
					return;
				}
				// otro
				const is_icon = $target.is(".ui-icon *");
				if (is_icon) {
					return;
				}
				// click en la cruz para realizar el resize
				const is_resize_arrow = $target.hasClass("ui-resizable");
				if (is_resize_arrow) {
					return;
				}
				// ahora si realizo las validaciones para minimizar las ventanas
				for (let i = 0; i < (self.m_openedWindows || []).length; i++) {
					const e = self.m_openedWindows[i];
					const constructor_parameters = e ? e.constructor_parameters : undefined;
					if (self.canMinimizeWhenOutside(constructor_parameters)) {

						// Realiza validacion contra la ventana del canal
						if (window_handle && window_handle.client && !window_handle.client.canMinimizeWhenOutside(target)) {
							continue;
						}

						/*
						Aca se valida que sea el mismo dialogo.
						#18807 Se daba en ocasiones que el window_handle, no era el mismo del m_openedWindows
						*/
						const window_handle_dialog_id = window_handle && window_handle.dialog_ui && window_handle.dialog_ui.length > 0 && window_handle.dialog_ui[0].id;
						const opened_window_dialog_id = e && e.dialog && e.dialog.length > 0 && e.dialog[0].id;
						if (window_handle_dialog_id && opened_window_dialog_id && window_handle_dialog_id !== opened_window_dialog_id) {
							continue;
						}
						const state = dialogExtend(e.dialog, "state");
						if (state !== "minimized") {
							dialogExtend(e.dialog, "minimize");
						}
					}
				}
			});
		};

		// bind events to the dialog container for resize and dragging.
		dialog_container.bind("dialogopen dialogfocus", function() {
			//console.log("[DialogManager.createWindow] dialogopen dialogfocus");
			set_iframe_position(iframe_node, dialog_container);
			self.correctZIndex(iframe_node);
		});

		dialog_container.bind( "dialogresizestart dialogdragstart", function( /*event, ui*/ ) {
			//console.log("[DialogManager.createWindow] dialogresizestart dialogdragstart");
			//
			// Cuando el usuario comienza a realizar el resize de la pantalla, tenemos que apagar el evento "clickoutside" para que el mismo NO se ejecute,
			// luego de soltar el boton del mouse. En el evento "dialogresizestop dialogdragstop" tenemos que volver a bindear el evento "clickoutside"
			//
			set_events(false);
			set_iframe_position(iframe_node, dialog_container, "dialogdragstart");
			iframe_node.hide();
			self.correctZIndex(iframe_node);

			if (window_handle && window_handle.client) {
				window_handle.client.onResizeStart();
			}
		});

		dialog_container.bind( "dialogresizestop dialogdragstop", function( /*event, ui*/ ) {
			//console.log("[DialogManager.createWindow] dialogresizestop dialogdragstop");
			set_iframe_position(iframe_node, dialog_container, "dialogdragstop");
			iframe_node.show();
			self.correctZIndex(iframe_node);

			if (window_handle && window_handle.client) {
				window_handle.client.onResizeStop();
			}

			//
			// Una vez que ha terminado de realizar resize de la pantalla, tenemos que bindear nuevamente el evento "clickoutside".
			// Esto se debe de realizar en un timeout para que el stop del resize se realice antes que el bindeo del "clickoutside"
			//
			window.setTimeout(function() {
				set_events(true);
			});
		});

		var window_handle;
		dialog_container.bind("dialogbeforeclose", function(e) {
			//console.log("[DialogManager.createWindow] dialogbeforeclose");
			if (window_handle.close() === false) {
				e.stopPropagation();
				return false;
			}
		});

		// append iframe to dialog container
		let $iframeWindowsContainer;
		if (typeof(parameters.id_iframe_windows_dialog_container) === "string") {
			$iframeWindowsContainer = $("#" + parameters.id_iframe_windows_dialog_container);
		}
		else {
			$iframeWindowsContainer = $("#iFramesWindowsDailogsContainerForGenericsApps");
		}
		if ($iframeWindowsContainer.length === 0) {
			throw("window_dialog error - DOM iframe container id is required: " + $iframeWindowsContainer.attr("id"));
		}
		$iframeWindowsContainer.append(iframe_node);

		// add dialog container with the iframe to the body
		dialog_container.appendTo("body");

		var allowed_restore_button = parameters.allowed_restore_button;
		var previous_state; // minimized, maximized, normal
		var my_dialog;

		/*
			#8183, #8192 jQueryUI last version.
			Funcion encargada de cachear las posiciones actuales del dialogo en modo normal (antes de minimizar o maximizar)
		*/
		var cache_dialog_positions = function(/*method*/) {
			// Cacheo las posiciones actuales
			const state = dialogExtend(dialog_container, "state");
			if (state.toLowerCase() !== "normal") {
				return;
			}
			//console.log("[DialogManager.createWindow] Caching dialog positions. Current state: '" + state + "'");
			dialog_container.cached_dialog_ui_normal_positions = null;
			dialog_container.cached_dialog_ui_normal_positions = dialog_container.dialog("option", "position");
			// Logueo en la consola del Navegador las posiciones actuales del Dialog UI
			//log_dialog_positions(dialog_container.cached_dialog_ui_normal_positions, method);
		};
		/*
			#8183, #8192 jQueryUI last version.
			Funcion encargada de restaurar desde cache las posiciones del dialogo a modo normal (cuando estabamos en los estados minimizar o maximizar)
		*/
		var restore_dialog_positions_from_cache = function() {
			if (!dialog_container.cached_dialog_ui_normal_positions) {
				return;
			}
			//console.log("[DialogManager.createWindow] Restoring dialog positions from cache.");
			// 1 - Ahora reposiciono el Dialogo con las coordenadas que hemos cacheado antes de minimizar o maximizar el dialogo.
			dialog_container.dialog("option", "position", dialog_container.cached_dialog_ui_normal_positions);
			// 2 - Logueo las posiciones del dialogo luego de restaurar. En si las posiciones que ha retornado jQueryUI
			////let restored_dialog_ui_position = dialog_container.dialog("option", "position");
			////log_dialog_positions(restored_dialog_ui_position, "Restored Dialog Positions from Cache");
			// 3 - Elimino el cache de posiciones
			dialog_container.cached_dialog_ui_normal_positions = null;
		};

		// custom iframe dialog-extend options
		var iframeDialogOptions = {
			beforeCollapse: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.beforeCollapse windowID " + parameters.windowID);
			},
			beforeMaximize: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.beforeMaximize windowID " + parameters.windowID);
				// Cacheo las posiciones actuales, antes de maximizar
				cache_dialog_positions("beforeMaximize");
			},
			beforeMinimize: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.beforeMinimize windowID " + parameters.windowID);
				// Cacheo las posiciones actuales, antes de minimizar
				cache_dialog_positions("beforeMinimize");
			},
			beforeRestore: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.beforeRestore windowID " + parameters.windowID);
			},
			collapse: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.collapse windowID " + parameters.windowID);
				iframe_node.hide();
			},
			maximize: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.maximize windowID " + parameters.windowID);
				set_iframe_position(iframe_node, dialog_container);
				iframe_node.show();
				self.correctZIndex(iframe_node);
				iframe_node.addClass( "clientMaximized" );
				if ( window_handle && window_handle.client ) {
					window_handle.client._maximized = true;
					window_handle.client.onMaximize();
					if ( allowed_restore_button === false ) {
						previous_state = "maximized";
						window_handle.client.hideRestoreButton();
					}
				}
				// // google analytics
				// analyticsParams.action_id = "window_maximize";
				// AnalyticsSend(analyticsParams);
			},
			minimize: function() {
				//console.log("[DialogManager.createWindow] iframeDialogOptions.minimize windowID " + parameters.windowID);
				if ( window_handle && window_handle.client ) {
					set_iframe_position(iframe_node, dialog_container);
					iframe_node.hide();
					self.correctZIndex(iframe_node);
					iframe_node.removeClass( "clientMaximized" );
					window_handle.client.onMinimize();
					if ( allowed_restore_button === false ) {
						previous_state = "minimized";
						window_handle.client.hideRestoreButton();
					}
				}
				// // google analytics
				// analyticsParams.action_id = "window_minimize";
				// AnalyticsSend(analyticsParams);
			},
			restore: function() {
				// Restauro las posiciones del dialogo que se cachearon antes de minimizar o maximizar
				// A esto se tiene que ejecutar aca, y no en el evento "beforeRestore" ya que sino despues se posiciona en un lugar incorrecto.
				restore_dialog_positions_from_cache();

				if (window_handle && window_handle.client) {
					if ( iframe_node.hasClass("clientMaximized") && !window_handle.client._showing ) {
						window_handle.client._maximized = false;
					}
					window_handle.client._showing = false;
				}

				//console.log("[DialogManager.createWindow] iframeDialogOptions.restore windowID " + parameters.windowID);

				set_iframe_position(iframe_node, dialog_container, "restore");
				iframe_node.show();
				self.correctZIndex(iframe_node);
				iframe_node.removeClass( "clientMaximized" );

				// // google analytics
				// analyticsParams.action_id = "window_restore";
				// AnalyticsSend(analyticsParams);

				if ( window_handle && window_handle.client ) {
					window_handle.client.onRestore();
				}
				// (3) Si han creado minimizada la ventana, este evento es ejecutado, cuando se ha invocado el evento "show",
				//   entonces tengo que realizar de forma efectiva el "minimize", y dejar el flag de create_minimized en false
				//   ya que este flag es utilizado unicamente cuando se ha creado la ventana. FIN
				if (parameters.create_minimized === true || parameters.create_maximized === true) {
					window.setTimeout(function(){
						if ( parameters.create_minimized ) {
							dialogExtend(my_dialog, "minimize");
						}
						else if ( parameters.create_maximized ) {
							dialogExtend(my_dialog, "maximize");
						}
						parameters.create_minimized = false;
						parameters.create_maximized = false;
						window_handle.create_minimized = false;
						window_handle.create_maximized = false;
						if (window_handle && window_handle.client) {
							window_handle.client.create_minimized = false;
							window_handle.client.create_maximized = false;
						}
					});
				}
				else {
					// dbl click de la barra titulo que hace un restore, lo tengo que tratar aca para el caso de que no esta permitido el restore (solo minimizada o maximizada)
					window.setTimeout(function(){
						if (!window_handle || !window_handle.client) {
							return;
						}
						if (allowed_restore_button === false) {
							if ( previous_state === "maximized" ) {
								dialogExtend(my_dialog, "minimize");
							}
							else {
								dialogExtend(my_dialog, "maximize");
							}
						}
						else if (window_handle.client._maximized) {
							dialogExtend(my_dialog, "maximize");
						}
					});
				}
			}
		};

		var defaultDialogOptions = this.__get_dialog_extended_options(parameters);
		if ( defaultDialogOptions.maximizable ) {
			iframeDialogOptions.dblclick = "maximize";
		}

		var dialogExtendOptions = $.extend(true, defaultDialogOptions, iframeDialogOptions);

		// the dialog widget
		var options = this.__get_dialog_options(parameters);
		my_dialog = dialog_container.dialog(options).dialogExtend(dialogExtendOptions);

		if (parameters.visible === false) {
			dialogExtend(my_dialog, "hide");
		}

		// Inhabilita el move de la ventana, cuando la misma es instanciada.
		// Ej: Se ha utilizado al iniciar el "phone" de tipo "live"
		if (parameters.draggable === false) {
			my_dialog.dialog("option", "draggable", false);
		}
	
		/*
		if (parameters.create_minimized === true || parameters.create_maximized === true) {
			// (1) si la tengo que crear minimizada, primero le realizo "hide" ya que sino aparece una ventana vacia en pantalla,
			// mientras el iframe se carga; despues continuo en (2)
			my_dialog(my_dialog, "hide");
		}
		*/

		// An instance of a Client (ChatClient, PhoneClient, MailClient, etc)
		var client;

		// This function is called by each client
		// obj_inconcert and new_window corresponds to 'inConcert' and 'window in the child so
		// we need to pass some 'singletons' over there.
		iframe_node[0].contentWindow.hook_environment = function(/*obj_inconcert, new_window*/)
		{
			var new_window = iframe_node[0].contentWindow;
			var obj_inconcert = new_window.inConcert;

			obj_inconcert.app = inConcert.app;
			obj_inconcert.permissions = inConcert.permissions;
			obj_inconcert.logger = inConcert.logger;
			if (parameters.title) {
				new_window.document.title = parameters.title;
			}

			var currentSkin = self.getCurrentSkin();
			$("body", new_window.document).removeClass("skinDark skinLight skinNew skinOCCNew").addClass(currentSkin);

			// This is a hack to have the dialog available at Dialog.prototype.setTitle
			// it's a chicken and egg problem
			// TODO: Fixme later
			new_window.dialog_ui = my_dialog;
		};
		let console_methods = ["log","info","debug","error"];
		for (let i in console_methods) {
			if (console_methods.hasOwnProperty(i)) {
				let method = console_methods[i];
				iframe_node[0].contentWindow.console[method]=function(){
					console[method].apply(null,arguments);
				};
			}
		}

		iframe_node[0].contentWindow.addEventListener("message",function(event){
			if (event.data.type === "log-message"){
				if(event.data.method){
					window.console[event.data.method].apply(null,JSON.parse(event.data.arguments));
				}else{
					window.console.log.apply(null,JSON.parse(event.data.arguments));
				}
			}
		}, true);
		iframe_node[0].contentWindow.onReady = function(client_instance)
		{
			set_events(true);
			client_instance.dialog_ui = my_dialog;
			window_handle.client = _.extend(client_instance,
			{
				create_minimized: parameters.create_minimized,
				create_maximized: parameters.create_maximized,
				// Inject the close() function in order to have more control over it
				close: function() {
					try {
						// console.debug("[DialogManager.prototype.createWindow]  1 - iframe onReady - close callback -> START");
						if ( !window_handle.isClosed() ) {
							// console.debug("[DialogManager.prototype.createWindow]  2 - iframe onReady - close callback -> invoke close window");
							window_handle.closeFunctionCalled = true;
							my_dialog.dialog("close");
						}
						// else {
						// 	console.debug("[DialogManager.prototype.createWindow]  3 - iframe onReady - close callback -> window already closed");
						// }
					}
					catch(e) {
						throw e;
					}
				}
			});

			if (parameters.visible !== false) {
				if (parameters.create_minimized || parameters.create_maximized) {
					// (2) si se tiene que crear minimizada, le realizo "show", ya que previamente se ha realizado "hide",
					//   para que no aparezca la ventana vacia, mientras se carga el iframe.
					//  Este show, dispara el evento "resize", y aca tengo que realizar el minimize de la ventana. (3)
					dialogExtend(my_dialog, "show");
				}
				else {
					dialogExtend(my_dialog, "restore");
				}
			}

			if (typeof(parameters.on_ready) === "function") {
				// TODO: #9778 - CANY - Docked.
				parameters.on_ready(window_handle.client, window_handle);
			}
			// TODO: #9778 - CANY - Docked.
			window.setTimeout(function() {
				window_handle.emit("ready");
			});
		};

		iframe_node[0].contentWindow.getWindowHandle = function() {
			return window_handle;
		};

		iframe_node[0].contentWindow.adjust_iframe_position = function () {
			//console.log("[DialogManager.createWindow] contentWindow.adjust_iframe_position");
			set_iframe_position(iframe_node, dialog_container);
			self.correctZIndex(iframe_node);
		};

		// pass dynamic parameters to the window
		// TODO, esto esta peligrosisimo. Se esta tirando cualquier cosa adentro de la ventana
		// y esas cosas pueden dejar anclados objetos y no se van por GC
		_.each(parameters, function(_, key){
			iframe_node[0].contentWindow[key] = parameters[key];
		});

		// push the window to member
		this.m_openedWindows.push({iframe: iframe_node[0], dialog: dialog_container, constructor_parameters: parameters});
		iframe_node[0].__dialog = dialog_container;
		this.m_sortedIframes.push(iframe_node[0]);

		// this is how interaction panels interact with this window. Through a handle.
		window_handle = new DialogHandle(client, iframe_node[0].contentWindow, my_dialog, analyticsParams);
		window_handle.on("close", function()
		{
			self.m_openedWindows = _.filter(self.m_openedWindows, function(e) { return e.iframe !== iframe_node[0]; });
			self.m_sortedIframes = _.filter(self.m_sortedIframes, function(e) { return e !== iframe_node[0]; });
			// It is important to remove this two nodes
			const $iframes = $("#cc_integration_sandbox_iframe",
				"#mas_integration_iframe",
				"#helpdesk_integration_sandbox_iframe",
				iframe_node.contents());
			
			$iframes.purgeFrame().then(function() {
				iframe_node.purgeFrame();
				my_dialog.remove();
			});

			if (typeof(parameters.on_close) === "function") {
				parameters.on_close(parameters.windowID);
			}
		});
		window_handle.on("hide", function()
		{
			const $iframes = $("#cc_integration_sandbox_iframe",
				"#mas_integration_iframe",
				"#helpdesk_integration_sandbox_iframe",
				iframe_node.contents());

			$iframes.purgeFrame();

			if (iframe_node[0].id === "cc_integration_sandbox_iframe"){
				purge(iframe_node[0]);
			}
		});
		window_handle.create_minimized = parameters.create_minimized;
		window_handle.create_maximized = parameters.create_maximized;
		return window_handle;
	};

	DialogManager.prototype.canMinimizeWhenOutside = function(constructor_parameters) {
		if (constructor_parameters) {
			if (constructor_parameters.can_minimize_when_outside === false) {
				return false;
			}
		}
		// por defecto siempre se minimiza
		return true;
	};

	DialogManager.prototype.getCurrentSkin = function() {
	// De momento va a existir un unico skin, asi que lo comentamos y dejamos el skinNew
		//var currentSkin = inConcert.Storage.get("Skin", "string");
		//return currentSkin || "skinNew";
		return "skinOCCNew";
	};

	DialogManager.prototype.setSkin = function(newSkin) {
		inConcert.Storage.put("Skin", newSkin);
		var self = this;
		this.m_openedWindows.forEach(function(e, index){
			var current_window = e.iframe;
			// TODO: esto es solo para Air, corregirlo
			if (current_window.window && !current_window.stage.nativeWindow.closed) {
				self.applySkin(current_window.window);
			}
			else {
				self.m_openedWindows.splice(index, 1);
			}
		});
	};

	DialogManager.prototype.applySkin = function(w, skin) {
		skin = skin || this.getCurrentSkin();
		$("body", w.document).removeClass("skinDark skinLight skinNew skinOCCNew").addClass(skin);
	};

	/**
	 * [Dialog description]
	 * @param {[type]} handlers [description]
	 * @param {[type]} settings [description]
	 */
	function Dialog(/*handlers, settings*/)
	{
		var self = this;

		this.__window_id = inConcert.Utils.generateGuid();
		inConcert.app.messagebus.publish("/windows/change_state", "create", this);

		window.onunload = function ()
		{
			// LEAK: we have to remove all this stuff or else we have a leak.
			delete window.onunload;
			delete window.onclose;
			delete window.dialog_ui;
			delete window.view;
			delete window.interaction;
			delete window.adjust_iframe_position;
			delete window.taskbar; // I'm *this* close to get rid of this
			delete window.onComplete;
			delete window.onReady;
			delete window.getWindowHandle;
			delete window.hook_environment;
		};
		window.onload  = function () { self.onload();  };
		window.onfocus = function () { self.onfocus(); };

		$(window).click(function() {
			if (window.adjust_iframe_position) {
				//console.log("[Dialog] $(window).click");
				window.adjust_iframe_position();
			}
		});

		// Inhabilita la combinacion de teclas "Ctrl+S" en cada dialogo/ventana que se abra.
		$(document).ready(function() {
			inConcert.Utils.disableCtrlSHotKeys();
			self.validateAutomaticCloseWindow();
		});
	}

	if (typeof inConcert.Dialog !== "function") {
		inConcert.Dialog = Dialog;
		inConcert.DialogManager = new DialogManager();
	}

	/*
		Ref #7115
		----------
		Si jQuery no implementa la funcion "$.noop", asignamos una funcion vacia.
		A esto lo realizamos asi porque la version que utilizamos de jQuery en todos lados es la 1.9.1 y esta version SI implementa esta funcion,
		pero la version de jQuery que utiliza el "frm Change Avatar" es la 1.3.2
		Esta version no implementa esa funcion, ergo se realiza esta pisada.

		TODO: Si en algun momento se actualiza la version de jQuery del "frm Change Avatar" a una mas nueva, este codigo debe de cambiar.
	*/
	Dialog.prototype.onload  = $.noop || function() {};
	Dialog.prototype.onfocus = $.noop || function() {};
	Dialog.prototype.center  = $.noop || function() {};

	Dialog.prototype.AnalyticsSend  = function(info) {
		AnalyticsSend(info);
	};

	Dialog.prototype.setTitle = function(title)
	{
		var $widget = window.dialog_ui.dialog("widget");
		var $title = $widget.find(".ui-dialog-title");
		$title.html(title);
	};

	Dialog.prototype.hideRestoreButton = function()
	{
		var $widget = window.dialog_ui.dialog("widget");
		var $titlebar = $widget.find(".ui-dialog-titlebar");
		var $restore = $titlebar.find(".ui-dialog-titlebar-restore");
		$restore.hide();
	};

	Dialog.prototype.addClass = function( class_name ) {
		window.dialog_ui.dialog("option", "dialogClass", class_name);
	};

	Dialog.prototype.removeClass = function() {
		window.dialog_ui.dialog("option", "dialogClass", "");
	};

	Dialog.prototype.isMinimized = function() {
		const state = dialogExtend(window.dialog_ui, "state");
		return state === "minimized";
	};

	Dialog.prototype.isMaximized = function() {
		const state = dialogExtend(window.dialog_ui, "state");
		return state === "maximized";
	};

	Dialog.prototype.state = function() {
		const state = dialogExtend(window.dialog_ui, "state");
		return state;
	};

	Dialog.prototype.setBounds = function(/*screenBounds*/) {};

	Dialog.prototype.onApplicationExit = function(/*event*/) {
		if (this.onShutdown) {
			this.onShutdown();
		}
	};

	Dialog.prototype.bringToFront = function(/*event*/) {
		window.focus();
	};

	Dialog.prototype.show = function(/*event*/) {
		var state = this.state();
		if (state !== "maximized") {
			this._showing = true;
			dialogExtend(window.dialog_ui, "show");
			// TODO: Revisar (caso golden layout / sidebar / telefono del cliente)
			if (state !== "normal") {
				this.activate();
			}
		}
		return this;
	};

	Dialog.prototype.minimize = function(/*event*/) {
		dialogExtend(window.dialog_ui, "minimize");
		return this;
	};

	Dialog.prototype.maximize = function(/*event*/) {
		dialogExtend(window.dialog_ui, "maximize");
		return this;
	};

	Dialog.prototype.restore = function(/*event*/) {
		dialogExtend(window.dialog_ui, "restore");
		return this;
	};

	Dialog.prototype.activate = function(/*event*/) {
		const state = dialogExtend(window.dialog_ui, "state");
		if (state === "minimized") {
			dialogExtend(window.dialog_ui, "restore");
		}
		else if (state === "normal") {
			this.show();
		}
		// en el maximized no deber realizar nada!!
		return this;
	};

	Dialog.prototype.hide = function(/*event*/) {
		dialogExtend(window.dialog_ui, "hide");
		_.defer( _.bind(this.onWindowHidden, this) );
		return this;
	};

	/*
		Ref Issue #15445: PhoneClientWindow con ventanas que NO se cerraban.
		Funcion encargada de invocar la funcion 'canAutomaticCloseWindow' de las clases hijas para determinar si la ventana se tiene que cerrar de forma automatica.
		Ej:
			LiveClientWindow
				Cuando la interaccion se abandona, ya comenzo la instancia, pero aun no se encuentra ready el telefono. (casos de borde)
	*/
	Dialog.prototype.validateAutomaticCloseWindow = function () {
		const self = this;
		console.log("Dialog.prototype.validateAutomaticCloseWindow - 1 validating ...");
		if (this.canAutomaticCloseWindow()) {
			console.log("Dialog.prototype.validateAutomaticCloseWindow - 2 *------------------> CLOSE AUTOMATICALLY WINDOW !!!");
			self.close();
			return true;
		}
		return false;
	};

	Dialog.prototype.canAutomaticCloseWindow = function (/*params*/) {
		//console.log("Dialog.prototype.canAutomaticCloseWindow - false");
		return false;
	};

	Dialog.prototype.canCloseWindow = function (/*params*/) {
		return true;
	};

	// return false to cancel closing, return 'hide' to hide the window instead
	Dialog.prototype.onWindowClosing = function (params) {
		if (this.canCloseWindow(params)) {
			return true;
		}
		return false;
	};

	Dialog.prototype.onWindowHidden = function() {
		inConcert.app.messagebus.publish("/windows/change_state", "hide", this);
		return;
	};

	Dialog.prototype.onWindowClosed = function() {
		inConcert.app.messagebus.publish("/windows/change_state", "close", this);
		// // google analytics
		// this.AnalyticsSend({
		// 	action_id: "window_closed"
		// });
		return;
	};

	Dialog.prototype.closeAllPopups = function()
	{
		// Al ejecutar el trigger "click" del "body", ejecutamos los eventos para que se cierren todos los popups abiertos en las ventanas.
		$("body").trigger("click");
	};

	Dialog.prototype.onMaximize = function(/*event*/) {
		// #8183 jQueryUI last version. Como la ultima version jQueryUI posiciona en una ubicación incorrecta el TitleBar, cuando se maximiza el dialgo.
		// Tenemos que ajustar las posiciones del mismo para que se visualicen correctamente.
		const $widget = window.dialog_ui.dialog("widget");
		$widget.css({left: 0, top: 0});
		//-----------
		inConcert.app.messagebus.publish("/windows/change_state", "maximize", this);
		this.closeAllPopups();
	};

	Dialog.prototype.onMinimize = function(/*event*/) {
		inConcert.app.messagebus.publish("/windows/change_state", "minimize", this);
		this.closeAllPopups();
	};

	Dialog.prototype.onRestore = function(/*event*/) {
		inConcert.app.messagebus.publish("/windows/change_state", "restore", this);
		this.closeAllPopups();
	};

	Dialog.prototype.onResizeStart = function(/*event*/) {
		inConcert.app.messagebus.publish("/windows/change_state", "resize_start", this);
		this.closeAllPopups();
	};

	Dialog.prototype.onResizeStop = function(/*event*/) {
		inConcert.app.messagebus.publish("/windows/change_state", "resize_stop", this);
	};

	Dialog.prototype.setHorizontalResizingLimits = function(minimumSize, maximumSize)
	{
		if (window.dialog_ui) {
			//console.log("[Dialog.setHorizontalResizingLimits] minimumSize: '" + minimumSize + "', maximumSize: '" + maximumSize + "'");
			if (minimumSize) {
				window.dialog_ui.dialog("option", "minWidth", minimumSize);
			}
			if (maximumSize) {
				window.dialog_ui.dialog("option", "maxWidth", maximumSize);
			}
			window.adjust_iframe_position();
		}
	};

	Dialog.prototype.setVerticalResizingLimits = function(minimumSize, maximumSize)
	{
		if (window.dialog_ui) {
			//console.log("[Dialog.setVerticalResizingLimits] minimumSize: '" + minimumSize + "', maximumSize: '" + maximumSize + "'");
			if (minimumSize) {
				window.dialog_ui.dialog("option", "minHeight", minimumSize);
			}
			if (maximumSize) {
				window.dialog_ui.dialog("option", "maxHeight", maximumSize);
			}
			window.adjust_iframe_position();
		}
	};

	Dialog.prototype.setWidth = function(width)
	{
		if (window.dialog_ui) {
			//console.log("[Dialog.setWidth] width: '" + width + "'");
			window.dialog_ui.dialog("option", "width", width);
			window.adjust_iframe_position();
		}
	};

	Dialog.prototype.notifyTaskbar = function(/*priority*/){};

	Dialog.prototype.onDisplayStateChange = function(/*event*/){};

	/*
		Funcion que retorna si la ventana se debe minimizar al dar click fuera del area de la current ventana.
		Por defecto es true.
		Se puede implementar en las clases hijas que lo necesiten.
		Ej:
			En el telefono de tipo "live" > "LiveClientWindow.prototype.canMinimizeWhenOutside"
	*/
	Dialog.prototype.canMinimizeWhenOutside = function()
	{
		return true;
	};

	/*
		Funcion que habilita o inhabilita el movimiento de la ventana.
	*/
	Dialog.prototype.setDraggable = function(enabled = true)
	{
		if (window.dialog_ui) {
			window.dialog_ui.dialog("option", "draggable", enabled);
		}
	};

	/*
		Funcion que habilita o inhabilita el minimizar (boton) de la ventana.
	*/
	Dialog.prototype.setMinimizable = function(enabled = true)
	{
		if (window.dialog_ui) {
			dialogExtend(window.dialog_ui, {minimizable: enabled});
		}
	};

	/*
		Las ejecuciones contra el dialogo se deben realizar en un try catch ya que jQueryUI en ocasiones puede tirar una excepcion si la ventana aun no esta ready,
			y esto corta el flujo del programa.
	*/
	const dialogExtend = (dialogUI, params)=> {
		try {
			//console.debug(`window_dialog 1 ---> dialogExtend('${params}') EXECUTE`);
			const response = dialogUI.dialogExtend(params);
			//console.debug(`window_dialog 2 ---> dialogExtend('${params}') RESPONSE: ${response}`);
			return response;
		} catch (error) {
			console.warn(`window_dialog 3---> dialogExtend('${params}') CAPTURED jQueryUI - ${error}`);
			return;
		}
	};
}());
