module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local model_perm = require "shared.permissions.models.permissions"
local models = require("apps.agent.model"):GetModels()
local proxy = require "apps.allegro.proxy"

local function HasBaragent (web, username)
	web.headers["Content-Type"] = "application/json"

	local agent, vcc
	if username and username ~= "" then
		agent, vcc = utils.SeparateVCC(username)
	else
		vcc = inconcert.authUser.virtualContactCenter
		agent = inconcert.authUser.userName
		username = agent .. "@" .. vcc
	end

	--check if has integration
	if not proxy.HasIntegration(vcc) then
		--reason = "vcc has not integration"
		return json.encode({ baragent = false})
	end

	--check if has agent permissions
	local user = models.Users.Get(username)
	if not user or not user.VCC then
		--reason = "user not found"
		return json.encode({ baragent = false})
	end

	--check if has address
	if not user.addresses or not user.addresses.CALL then
		--reason = "user has not address" 
		return json.encode({ baragent = false})
	end

	local authenticated = true
	if not inconcert.authUser then
		authenticated = false
		inconcert.authUser = { virtualContactCenter = vcc }
	end

	local result
	if model_perm.HasPermissionToApplication("interaction", username, user.UserGroup) then
		result = json.encode({ baragent = true })
	else
		--reason = "user is not in interaction app"
		result = json.encode({ baragent = false })
	end

	if not authenticated then
		inconcert.authUser = nil
	end

	return result
end

local function LoginAgentFromToken (web)
	web.headers["Content-Type"] = "application/json"
	local data_s = web.input["post_data"]
	local success, data = pcall(json.decode, data_s)
	if not success or not data.token then
		return json.encode({ status = false, reason = "Invalid data" })
	end
	local ok, message = proxy.generic.LoginAgentFromToken(data.token)
	return json.encode(ok and { status = true } or { status = false, reason = message })
end

function MapUris (module, htmlRender, prefix)
	module:dispatch_get(HasBaragent, "/login/has_baragent/(.*)")
	module:dispatch_post(LoginAgentFromToken, "/agent/login_agent_from_token/")
end
