local utils = require "shared.apps.utils"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"

function LoadPaginatedContacts (encodedInfo)
	local startProcessTime = os.time()
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContacts] Loading Paginated Contacts.")
	local success, data = proxy.outboundEngine.LoadContacts(encodedInfo)
	local elapsedTimeSec = utils.GetElapsedTimeSec(startProcessTime)
	if success and data then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContacts] Successfully loaded %d of %s Contacts. ELAPSED Time '%s' seconds.", #data.rows, data.total_rows, elapsedTimeSec)
		--LogDebug("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContacts] --> Successfully loaded Contacts: %s", json.encode(data))
		--utils.CreateFile("/tmp/allegro_page_contacts_from_luaorm.json", json.encode(data))
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_CONTACTS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContacts] Error loading paginated Contacts. Err: %s. ELAPSED Time '%s' seconds.", json.encode(errType), elapsedTimeSec)
	return false, errType
end

function LoadPaginatedContactsToMigrate (encodedInfo)
	local startProcessTime = os.time()
	--LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContactsToMigrate] Loading Paginated Contacts to Migrate.")
	local success, data = proxy.outboundEngine.LoadContactsToMigrate(encodedInfo)
	local elapsedTimeSec = utils.GetElapsedTimeSec(startProcessTime)
	if success and data then
		--LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContactsToMigrate] Successfully loaded %d of %s Paginated Contacts to Migrate. ELAPSED Time '%s' seconds.", #data.rows, data.total_rows, elapsedTimeSec)
		--LogDebug("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContactsToMigrate] --> Paginated Contacts to Migrate: %s", json.encode(data))
		--utils.CreateFile("/tmp/allegro_page_contacts_from_sql.json", json.encode(data))
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_CONTACTS_TO_MIGRATE"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadPaginatedContactsToMigrate] Error loading Paginated Contacts to Migrate. Err: %s. ELAPSED Time '%s' seconds.", json.encode(errType), elapsedTimeSec)
	return false, errType
end

-- el vccId no es requerido
function GetContact (contactId, vccId)
	assert(contactId)
	--LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:GetContact] Retrieving Contact: '%s', vccId: %s.", contactId, vccId)
	local response, err = proxy.outboundEngine.GetContact(contactId, vccId)
	--LogDebug("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:GetContact] --> Allegro Contact Data: '%s', .", json.encode(response or {}))
	if response then
		local contact = response.contact
		contact.Batches = response.batches
		contact.History = response.history
		contact.NameValues = response.nameValues
		contact.EMails = response.emails
		contact.Skills = response.skills
		contact.ScheduledCalls = response.scheduledCalls
		--LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:GetContact] Successfully retrieving %s Contact. Info: %s", contactId, contact)
		--LogDebug("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:GetContact] --> Successfully ContactId: %s, ContactInfo: %s.", contactId, json.encode(contact))
		return true, contact
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_CONTACT",
		contactId = contactId
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:GetContact] Error retrieving %s Contact. Err: %s", contactId, json.encode(errType))
	return false, errType
end

function LoadContactsBatchFromProcess (processId)

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadContactsBatchFromProcess] Loading Contact Batches from process '%s'.", processId)

	local response, err = proxy.outboundEngine.LoadContactsBatchFromProcess(processId)

	if response then
		local contactsBatch = response.contactsBatch
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadContactsBatchFromProcess] Successfully Loaded Contact Batches from process '%s'. Info: %s", processId, contactsBatch)
		return true, contactsBatch
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_CONTACTS_BATCH"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_CONTACTS:LoadContactsBatchFromProcess] Error Loading Contacts Batch from process '%s'. Err: %s", processId, json.encode(errType))
	return false, errType
end