module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"
local allegroRepository = require "apps.allegro.controllers.repository"

function LoadPaginatedExports (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadPaginatedExports] Loading Exports.")

	local success, data = proxy.outboundEngine.LoadExports(encodedInfo)
	if success and data then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadPaginatedExports] Successfully loaded %d of %s Exports.", #data.rows, data.total_rows)
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_EXPORTS"
	}
	return false, errType
end

function StartExportation (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:StartExportation] Starting Exportation.", encodedInfo)
	local info = json.decode(encodedInfo)
	-------------------------------------
	-- Se le agrega el "logged user" a la informacion que se va a postear a 5.5, para que en el campo "ContactExport.User" quede el usuario
	-- que ha realizado la accion en i6 y no el usuario "integrationuser"
	info.__user = inconcert.authUser.userName
	encodedInfo = json.encode(info)
	-------------------------------------
	local success, data = proxy.outboundEngine.StartExportation(encodedInfo)
	--LogDebug("---> StartExportation success: %s, data: %s", success, data)
	if success then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:StartExportation] Successfully Started Exportation.")
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_STARTING_EXPORTATION"
	}
	return false, errType
end

function DeleteExport (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:DeleteExport] Deleting Exports.", encodedInfo)

	local success, data = proxy.outboundEngine.DeleteExport(encodedInfo)
	--LogDebug("---> DeleteExport success: %s, data: %s", success, data)
	if success then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:DeleteExport] Successfully deleted %s Exports.", encodedInfo)
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_DELETING_EXPORTS"
	}
	return false, errType
end

function DownloadExport (exportationId, fileName)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:DownloadExport] Downloading Failed Contacts on ExportationId: '%s', fileName: '%s'.", exportationId, fileName)

	local vcc = inconcert.authUser.virtualContactCenter
	return allegroRepository.OutboundEngineExportationDownload(exportationId, vcc, fileName)
end

function CancelExport (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:CancelExport] Canceling Exports.", encodedInfo)

	local success, data = proxy.outboundEngine.CancelExport(encodedInfo)
	--LogDebug("---> CancelExport success: %s, data: %s", success, data)
	if success then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:CancelExport] Successfully cancelled %s Exports.", encodedInfo)
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_CANCELING_EXPORTS"
	}
	return false, errType
end

function LoadPaginatedExportLayouts (encodedInfo)

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadPaginatedExportLayouts] Loading Export Layout.")

	local success, data = proxy.outboundEngine.LoadPaginatedExportLayouts(encodedInfo)
	if success and data then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadPaginatedExportLayouts] Successfully loaded %d of %s Paginanted Export Layout.", #data.rows, data.total_rows)
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_PAGINATED_EXPORT_LAYOUTS"
	}
	return false, errType
end

function SaveExportLayout (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:SaveExportLayout] Saving Export Layout.", encodedInfo)

	local success, data = proxy.outboundEngine.SaveExportLayout(encodedInfo)
	--LogDebug("---> SaveExportLayout success: %s, data: %s", success, data)
	if success then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:SaveExportLayout] Successfully saved %s Export Layout.", encodedInfo)
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_SAVING_EXPORT_LAYOUT"
	}
	return false, errType
end

function DeleteExportLayout (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:DeleteExportLayout] Deleting Export Layout.", encodedInfo)

	local success, data = proxy.outboundEngine.DeleteExportLayout(encodedInfo)
	--LogDebug("---> DeleteExportLayout success: %s, data: %s", success, data)
	if success then
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:DeleteExportLayout] Successfully deleted %s Export Layout.", encodedInfo)
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_DELETING_EXPORT_LAYOUT"
	}
	return false, errType
end

function GetExportLayout (layoutId)

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:GetExportLayout] Retrieving Export Layout: '%s'.", layoutId)

	local response, err = proxy.outboundEngine.GetExportLayout(layoutId)

	if response then
		local layout = response.layout
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:GetExportLayout] Successfully retrieving %s Export Layout. Info: %s", layoutId, layout)
		return true, layout
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_LAYOUT"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:GetExportLayout] Error retrieving %s Export Layout. Err: %s", layoutId, json.encode(errType))
	return false, errType
end

function LoadExportLayouts ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadExportLayouts] Loading All Export Layouts.")

	local response, err = proxy.outboundEngine.LoadExportLayouts()

	if response then
		local layouts = response.layouts
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadExportLayouts] Successfully Loaded All Export Layouts. Info: %s", layouts)
		return true, layouts
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_LAYOUT"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_EXPORTS:LoadExportLayouts] Error Loading All Export Layouts. Err: %s", json.encode(errType))
	return false, errType
end
