module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"

function LoadEngineServices ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineServices] Loading Engine Services.")

	local response, err = proxy.outboundEngine.LoadEngineServices()

	if response then
		local services = response.services
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineServices] Successfully Loaded Engine Services Info: %s", services)
		return true, services
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_ENGINE_SERVICES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineServices] Error loading Engine Services. Info: %s", json.encode(errType))
	return false, errType
end

function LoadEngineTypes ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineTypes] Loading Engine Types.")

	local response, err = proxy.outboundEngine.LoadEngineTypes()

	if response then
		local types = response.types
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineTypes] Successfully Loaded Engine Types Info: %s", types)
		return true, types
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_ENGINE_TYPES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadEngineTypes] Error loading Engine Types. Info: %s", json.encode(errType))
	return false, errType
end

function LoadResponsesByChannel (channel)

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadResponsesByChannel] Loading Rules Responses by Channel %s.", channel)

	local response, err = proxy.outboundEngine.LoadResponsesByChannel(channel)

	if response then
		local responsesByChannel = response.responsesByChannel
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadResponsesByChannel] Successfully Loaded Rules Responses by Channel %s Info: %s", channel, responsesByChannel)
		return true, responsesByChannel
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_RULES_RESPONSES_CHANNEL"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadResponsesByChannel] Error loading Rules Responses by Channel %s. Info: %s", channel, json.encode(errType))
	return false, errType
end

function LoadBatchesByCampaign (campaign)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadBatchesByCampaign] Loading Batches by Campaign %s.", campaign)

	local response, err = proxy.outboundEngine.LoadBatchesByCampaign(campaign)

	if response then
		local batchesByCampaign = response.batchesByCampaign
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadBatchesByCampaign] Successfully Loaded Batches by Campaign %s Info: %s", campaign, batchesByCampaign)
		return true, batchesByCampaign
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_BATCHES_CAMPAIGN"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadBatchesByCampaign] Error loading Batches by Campaign %s. Info: %s", campaign, json.encode(errType))
	return false, errType
end


function LoadDestinations ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadDestinations] Loading Rules Destinations.")

	local response, err = proxy.outboundEngine.LoadDestinations()

	if response then
		local destinations = response.destinations
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadDestinations] Successfully Loaded Rules Destinations Info: %s", destinations)
		return true, destinations
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_RULES_DESTINATIONS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadDestinations] Error loading Rules Destinations. Info: %s", json.encode(errType))
	return false, errType
end

function LoadImportations ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportations] Loading Importations.")

	local response, err = proxy.outboundEngine.LoadImportations()

	if response then
		local importations = response.importations
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportations] Successfully Loaded Importations Info: %s", importations)
		return true, importations
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORTATIONS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportations] Error loading Importations. Info: %s", json.encode(errType))
	return false, errType
end

function LoadImportFieldsTypes ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportFieldsTypes] Loading Import Format Fields Types.")

	local response, err = proxy.outboundEngine.LoadImportFieldsTypes()

	if response then
		local fieldsTypes = response.fieldsTypes
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportFieldsTypes] Successfully Loaded Import Format Fields Types Info: %s", fieldsTypes)
		return true, fieldsTypes
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_FORMAT_FIELDS_TYPES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadImportFieldsTypes] Error loading Import Format Fields Types. Info: %s", json.encode(errType))
	return false, errType
end

function LoadScheduledDateFormats ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadScheduledDateFormats] Loading Import Scheduled Date Formats.")

	local response, err = proxy.outboundEngine.LoadScheduledDateFormats()

	if response then
		local dateFormats = response.dateFormats
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadScheduledDateFormats] Successfully Loaded Import Scheduled Date Formats Info: %s", dateFormats)
		return true, dateFormats
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_SCHEDULED_DATE_FORMATS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadScheduledDateFormats] Error loading Import Scheduled Date Formats. Info: %s", json.encode(errType))
	return false, errType
end

function LoadCountryList ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountryList] Loading Import Countries.")

	local response, err = proxy.outboundEngine.LoadCountryList()

	if response then
		local countries = response.countries
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountryList] Successfully Loaded Import countries Info: %s", countries)
		return true, countries
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_COUNTRIES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountryList] Error loading Import countries. Info: %s", json.encode(errType))
	return false, errType
end

function LoadContactDestinations ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactDestinations] Loading Import Contact Destinations.")

	local response, err = proxy.outboundEngine.LoadContactDestinations()

	if response then
		local destinations = response.destinations
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactDestinations] Successfully Loaded Import Contact Destinations Info: %s", destinations)
		return true, destinations
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_CONTACT_DESTINATIONS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactDestinations] Error loading Import Contact Destinations. Info: %s", json.encode(errType))
	return false, errType
end

function LoadContactMailDestinations ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactMailDestinations] Loading Import Contact Mail Destinations.")

	local response, err = proxy.outboundEngine.LoadContactMailDestinations()

	if response then
		local mailDestinations = response.mailDestinations
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactMailDestinations] Successfully Loaded Import Contact Mail Destinations Info: %s", mailDestinations)
		return true, mailDestinations
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_CONTACT_MAIL_DESTINATIONS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactMailDestinations] Error loading Import Contact Mail Destinations. Info: %s", json.encode(errType))
	return false, errType
end

function LoadContactAddressDestinations ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactAddressDestinations] Loading Import Contact Address Destinations.")

	local response, err = proxy.outboundEngine.LoadContactAddressDestinations()

	if response then
		local addressDestinations = response.addressDestinations
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactAddressDestinations] Successfully Loaded Import Contact Address Destinations Info: %s", addressDestinations)
		return true, addressDestinations
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_CONTACT_ADDRESS_DESTINATIONS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadContactAddressDestinations] Error loading Import Contact Address Destinations. Info: %s", json.encode(errType))
	return false, errType
end

function LoadCustomColumns ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCustomColumns] Loading Import Custom Columns.")

	local response, err = proxy.outboundEngine.LoadCustomColumns()

	if response then
		local customColumns = response.customColumns
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCustomColumns] Successfully Loaded Import Custom Columns Info: %s", customColumns)
		return true, customColumns
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_CUSTOM_COLUMNS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCustomColumns] Error loading Import Custom Columns. Info: %s", json.encode(errType))
	return false, errType
end

function LoadReservedWords ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadReservedWords] Loading Import Reserved Words.")

	local response, err = proxy.outboundEngine.LoadReservedWords()

	if response then
		local reservedWords = response.reservedWords
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadReservedWords] Successfully Loaded Import Reserved Words Info: %s", reservedWords)
		return true, reservedWords
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_IMPORT_RESERVED_WORDS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadReservedWords] Error loading Import Reserved Words. Info: %s", json.encode(errType))
	return false, errType
end

local m_cachedTimezones = nil
function LoadTimeZones ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadTimeZones] Loading Timezones.")

	if m_cachedTimezones then
		LogDebug("------------------> [ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadTimeZones] Retorna los Timezones Cacheados.")
		return true, m_cachedTimezones
	end

	local response, err = proxy.outboundEngine.LoadTimeZones()

	if response then
		local timezones = response.timezones
		m_cachedTimezones = timezones
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadTimeZones] Successfully Loaded Timezones Info: %s", timezones)
		return true, timezones
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_TIMEZONES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadTimeZones] Error loading Timezones. Info: %s", json.encode(errType))
	return false, errType
end

local m_cachedCountries = nil
function LoadCountries ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountries] Loading Countries.")

	if m_cachedCountries then
		LogDebug("------------------> [ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountries] Retorna las Countries Cacheadas.")
		return true, m_cachedCountries
	end

	local response, err = proxy.outboundEngine.LoadCountries()

	if response then
		local countries = response.countries
		m_cachedCountries = countries
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountries] Successfully Loaded Countries Info: %s", countries)
		return true, countries
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_COUNTRIES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadCountries] Error loading Countries. Info: %s", json.encode(errType))
	return false, errType
end

local m_cachedAreas = nil
function LoadAreas ()

	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadAreas] Loading Areas.")

	if m_cachedAreas then
		LogDebug("------------------> [ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadAreas] Retorna las Areas Cacheadas.")
		return true, m_cachedAreas
	end

	local response, err = proxy.outboundEngine.LoadAreas()

	if response then
		local areas = response.areas
		m_cachedAreas = areas
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadAreas] Successfully Loaded Areas Info: %s", areas)
		return true, areas
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_AREAS"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadAreas] Error loading Areas. Info: %s", json.encode(errType))
	return false, errType
end

function LoadExportLayoutColumnTypes ()
	LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadExportLayoutColumnTypes] Loading Export Columns Types.")

	local response, err = proxy.outboundEngine.LoadExportLayoutColumnTypes()

	if response then
		local columnsTypes = response.columnsTypes
		LogInfo("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadExportLayoutColumnTypes] Successfully Loaded Export Columns Types Info: %s", columnsTypes)
		return true, columnsTypes
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_SYSTEM_EXPORT_COLUMNS_TYPES"
	}
	LogError("[ALLEGRO.OUTBOUNDENGINE_SYSTEM:LoadExportLayoutColumnTypes] Error loading Export Columns Types. Info: %s", json.encode(errType))
	return false, errType
end
