local ctrls_prov_vccs = require "apps.provisioning.controllers.vccs"
local model_prov_vccs = require "apps.provisioning.models.vccs"
local templates = require "apps.allegro.templates"

module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil

local function CreateUser (vccId, user)
	if not user then
		return
	end
	local isNew = true
	LogInfo("[ALLEGRO.PROVISIONING.VCCS:CreateUser] Creating User '%s' in vcc '%s' .", user.Id, vccId)
	local omnichannelUser = templates.MakeOmnichannelUser(vccId, user, isNew)
	if not omnichannelUser then
		web.status = 500
		return "allegro provisioning error, making omnichannel user template"
	end
	local success, errMsg = ctrls_prov_vccs.CreateFirstUserInVCC(omnichannelUser)
	if not success then -- error
		LogWarning("[ALLEGRO.PROVISIONING.VCCS:CreateUser] Error creating user '%s' in vcc '%s'. --> '%s'", user.Id, vccId, errMsg)
	end
end

local function CreateApiUser (vccId, apiUsers)
	if not apiUsers then
		LogWarning("[ALLEGRO.PROVISIONING.VCCS:CreateApiUser] Unable to create API users")
		return
	end
	LogInfo("[ALLEGRO.PROVISIONING.VCCS:CreateApiUser] Creating API Users in vcc '%s' .", vccId)
	local success, errMsg = ctrls_prov_vccs.CreateFirstApiUsersInVCC(apiUsers)
	if not success then -- error
		LogWarning("[ALLEGRO.PROVISIONING.VCCS:CreateApiUser] Error creating API users")
	end
end

local function CreateSuperUser (vccId, superUser)
	if not superUser then
		return
	end
	local isNew = true
	LogInfo("[ALLEGRO.PROVISIONING.VCCS:CreateSuperUser] Creating SUPER User '%s' in vcc '%s' .", superUser.Id, vccId)
	local omnichannelUser = templates.MakeOmnichannelUser(vccId, superUser, isNew)
	if not omnichannelUser then
		web.status = 500
		return "allegro provisioning error, making omnichannel super user template"
	end
	local success, errMsg = ctrls_prov_vccs.SuperUser().CreateSuperUserInVCC(omnichannelUser)
	if not success then -- error
		LogWarning("[ALLEGRO.PROVISIONING.VCCS:CreateSuperUser] Error creating super user '%s' in vcc '%s'. --> '%s'", user.Id, vccId, errMsg)
	end
end

--[[
	5.5 Posted information:

{"VCCServices":[],"Description":"","RootUser":{"LanguageId":"en","ValidationOrigin":0,"Country":"","Id":"a","VccGroup":"system","VCC":"a","Name":"a","Password":"4124bc0a9335c27f086f24ba207a4912","TimeZoneId":"UTC","State":"ENABLED","IsSystemUser":0,"UserGroup":"administrator"},"ContactName":"","VCCLicenses":[{"LicenseId":"IVR_ENABLED","Total":"1","VCC":"a"},{"LicenseId":"AGENTES_LICENSED","Total":"0","VCC":"a"},{"LicenseId":"DESIGNERS_LICENSED","Total":"0","VCC":"a"},{"LicenseId":"SUPERVISOR_LICENSED","Total":"0","VCC":"a"},{"LicenseId":"IVR_PORTS","Total":"0","VCC":"a"},{"LicenseId":"DIALER_PORTS","Total":"0","VCC":"a"},{"LicenseId":"RECORDING_PORTS","Total":"0","VCC":"a"},{"LicenseId":"SCRIPTING_ENGINES","Total":"0","VCC":"a"},{"LicenseId":"ADMINISTRATOR_LICENSED","Total":"0","VCC":"a"},{"LicenseId":"MULTIMEDIA_ENABLED","Total":"1","VCC":"a"},{"LicenseId":"DIALER_ENABLED","Total":"1","VCC":"a"},{"LicenseId":"RECORDING_ENABLED","Total":"1","VCC":"a"},{"LicenseId":"RECORDING_MODE","Total":"1","VCC":"a"},{"LicenseId":"CC_ENABLED","Total":"1","VCC":"a"},{"LicenseId":"CC_LICENSE_MODE","Total":"1","VCC":"a"}],"VCCIpPeers":[],"VCCTrunks":[],"VCCSystemServiceLimits":[{"ServiceId":"DATABASE_192_168_24_205","ServiceLimitId":"contacts_in_batchs","Limit":1000000,"VCC":"a"}],"CompanyAddress":"","VCCCarriers":[],"Id":"a","Telephone2":"","Telephone1":"","Company":"a","ContactEmail":"","VCCDNISs":[],"IsSystemVCC":0}

--]]

local function SaveVCC (web)
	local posted_data = web.input["post_data"]
	local success, allegro_vcc = pcall(json.decode, posted_data)

	LogInfo("[ALLEGRO.PROVISIONING.VCCS:SaveVCC] Saving VCC. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, posted_data)

	if not success or not allegro_vcc or type(allegro_vcc.Id) ~= "string" then
		web.status = 400
		return "Invalid request"
	end

	local vcc_id = allegro_vcc.Id
	local success, omnichannel_vcc = model_prov_vccs.Get(vcc_id)

	if not success then
		omnichannel_vcc = {}
	end

	local is_new_vcc
	omnichannel_vcc, is_new_vcc = templates.MakeOmnichannelVcc( allegro_vcc, omnichannel_vcc )
	if not omnichannel_vcc then
		web.status = 500
		local err = is_new_vcc 
		return err
	end

	-- TODO: accedo derecho al controller de provisioning, deberia cambiarlo para que realice un request
	local success, err_msg, firstApiUsers, firstSuperUser = ctrls_prov_vccs.ExternalSaveVCC(omnichannel_vcc)
	if not success then -- error
		LogWarning("[ALLEGRO.PROVISIONING.VCCS:SaveVCC] Error Updating VCC '%s'. --> '%s'", vcc_id, err_msg)
		web.status = 500
		return success
	end

	LogInfo("[ALLEGRO.PROVISIONING.VCCS:SaveVCC] '%s' %s successfully.", vcc_id, (is_new_vcc and " Added" or " Modified"))

	if is_new_vcc then
		CreateUser( vcc_id, allegro_vcc.AllegroProxyUser ) -- crea el usuario allegro proxy
		CreateUser( vcc_id, allegro_vcc.RootUser ) -- crea el root user definido en la interfaz de 5.5 cuando se creo el VCC
		LogInfo("[ALLEGRO.PROVISIONING.VCCS:SaveVCC] Created root user for new vcc '%s'. root user--> '%s'", vcc_id, json.encode(allegro_vcc.RootUser))
		-- creo los usuarios de API
		CreateApiUser(vcc_id, firstApiUsers)
		-- creo el primer Super User
		CreateSuperUser(vcc_id, firstSuperUser)
	else
		-- Si el VCC, se esta modificando y llega el usuario en "AllegroProxyUser", quiere decir
		--  que el VCC previamente no tenia integracion con "Omnichannel" y ahora si, entonces tengo que crear el usuario de integracion con "Omnichannel"
		if allegro_vcc.AllegroProxyUser then
			CreateUser( vcc_id, allegro_vcc.AllegroProxyUser ) -- crea el usuario allegro proxy
		end
		-- idem al comentario anterior
		if allegro_vcc.RootUser then
		-- crea el root user definido en la interfaz de 5.5 cuando se creo el VCC
		-- aunque realmente desde 5.5 ha seleccionado el primer usuario administrador de BD, ya que no sabe si realmente cual es el primer usuario administrador
			CreateUser( vcc_id, allegro_vcc.RootUser )
			LogInfo("[ALLEGRO.PROVISIONING.VCCS:SaveVCC] Created root user for actual vcc '%s'. root user--> '%s'", vcc_id, json.encode(allegro_vcc.RootUser))
		end
	end

	web.status = 200
	return ""
end

--[[
	NO SE ESTA UTILIZANDO!!
	Hoy Esta funcion NO se utiliza.
	La misma debería ser invocada desde Allegro, pero el código para invocarla se encuentra comentado en Allegro.
	Igual la dejo como referencia !!
]]--
-- local function DeleteVCC (web)
-- 	local postedData = web.input["post_data"]
-- 	LogInfo("[ALLEGRO.PROVISIONING.VCCS:DeleteVCC] Deleting VCC. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)
-- 	local success, vccInfo = pcall(json.decode, postedData)
-- 	if not success or not vcc_info or type(vccInfo.Id) ~= "string" then
-- 		web.status = 400
-- 		return "Invalid request"
-- 	end
-- 	local result = ctrls_prov_vccs.DeleteVCC(vccInfo.Id)
-- 	if not result.success then
-- 		web.status = 400
-- 		return "Invalid request"
-- 	end
-- 	web.status = 200
-- 	return ""
-- end

function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	module:dispatch_post( LogRequest .. SaveVCC   , prefix .. "/save/(.*)"   )
--	module:dispatch_post( LogRequest .. DeleteVCC , prefix .. "/delete/(.*)" )
	m_htmlRender = htmlRender
end

----
function GetVCC (vccId)
	LogInfo("[ALLEGRO.PROVISIONING.VCCS:GetVCC] Retrieving allegro VCC '%s' information.", vccId)
	local proxy = require "apps.allegro.proxy"
	local response = proxy.provisioning.GetVCC(vccId)
	if response and response.status == true then
		local vccInfo = response.vccInfo
		--LogDebug("[ALLEGRO.PROVISIONING.VCCS:GetVCC] Successfully retrieving %s VCC. Info: %s", vccId, json.encode(vccInfo))
		return true, vccInfo
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_VCC"
	}
	LogError("[ALLEGRO.PROVISIONING.VCCS:GetVCC] Error retrieving %s VCC. Info: %s", vccId, errType)
	return false, errType
end
