module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"

local function GetVideoStreamingServiceIp(web, vcc)
	local success, response = proxy.soaIntegration.GetVideoStreamingServiceIp(vcc)
	if not success then
		LogError("[SOAINTEGRATION:GetVideoStreamingServiceIp] Error retrieving %s ip - %s.", vcc, response)
		web.status = 500
	end
	return response
end

local function GetVideoStreamingSessionId(web, vcc)
	local success, response = proxy.soaIntegration.GetVideoStreamingSessionId(vcc)
	if not success then
		LogError("[SOAINTEGRATION:GetVideoStreamingSessionId] Error retrieving %s session - %s.", vcc, response)
		web.status = 500
	end
	return response
end

local function RecognizePhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.RecognizePhone(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:RecognizePhone] Error recognizing phone %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function ManualCallBack(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.ManualCallBack(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:ManualCallBack] Error running manual callback %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function DialerCallBack(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.DialerCallBack(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:DialerCallBack] Error running dialer callback %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function ExecuteFlowState(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.ExecuteFlowState(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:ExecuteFlowState] Error executing flow state %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function AddContactsToCampaign(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.AddContactsToCampaign(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:AddContactsToCampaign] Error adding contacts to campaign %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function ReloadCampaign(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.soaIntegration.ReloadCampaign(encodedData)
	if not success then
		LogError("[SOAINTEGRATION:ReloadCampaign] Error reloading campaign %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

function MapUris (module, htmlRender, prefix)
	module:dispatch_get(GetVideoStreamingServiceIp, prefix .. "/get_videostreaming_serviceip/(.*)")
	module:dispatch_get(GetVideoStreamingSessionId, prefix .. "/get_videostreaming_sessionid/(.*)")
	
	module:dispatch_post(RecognizePhone, prefix .. "/recognize_phone/([^.]*)")
	module:dispatch_post(ManualCallBack, prefix .. "/manual_callback/([^.]*)")
	module:dispatch_post(DialerCallBack, prefix .. "/dialer_callback/([^.]*)")
	module:dispatch_post(DialerCallBack, prefix .. "/dialer_callback_to_date/([^.]*)")
	module:dispatch_post(ExecuteFlowState, prefix .. "/execute_flow_state/([^.]*)")
	module:dispatch_post(AddContactsToCampaign, prefix .. "/add_contacts_to_campaign/([^.]*)")
	module:dispatch_post(ReloadCampaign, prefix .. "/reload/reload_campaign/([^.]*)")
end
