local allegro = require "apps.allegro.proxy"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function HasIntegration ( vcc_id ) return allegro.HasIntegration(vcc_id) end

--[[
  Guarda workgroups en 5.5
]]--
function Save ( workgroups_omnichannel, vcc_id )

	assert(type(workgroups_omnichannel) == "table")

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.WORKGROUPS:Save] Error saving allegro workgroups. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local encoded_workgroups_omnichannel = json.encode(workgroups_omnichannel)
	LogInfo("[ALLEGRO.WORKGROUPS:Save] encoded workgroups: %s", encoded_workgroups_omnichannel)

	local success, allegro_err = allegro.Post(
		vcc_id,
		encoded_workgroups_omnichannel,
		"administration/workgroups/save/" -- la URL de 5.5
	)

	local err
	if not success then
		err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_err }
		LogWarning("[ALLEGRO.WORKGROUPS:Save] Error saving allegro workgroups: %s. Err: %s.", encoded_workgroups_omnichannel, json.encode(err))
	end

	return success, err
end

--[[
  Elimina los workgroups en 5.5
]]--
function Delete ( workgroups_omnichannel, vcc_id )

	assert(type(vcc_id) == "string")

	local encoded_workgroups_omnichannel = json.encode(workgroups_omnichannel)

	LogInfo("[ALLEGRO.WORKGROUPS:Delete] Deleting allegro workgroups '%s'", encoded_workgroups_omnichannel)

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.WORKGROUPS:Delete] Error deleting allegro workgroups. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local success, allegro_err = allegro.Post(
		vcc_id,
		encoded_workgroups_omnichannel,
		"administration/workgroups/delete/" -- la URL de 5.5
	)

	local err
	if not success then
		err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_err }
		if allegro_err.result == "NOT_FOUND" then
			err.type = "NOT_A_ERROR"
		end
		LogWarning("---> [ALLEGRO.WORKGROUPS:Delete] Error deleting allegro workgroups: %s. Err: %s.", encoded_workgroups_omnichannel, json.encode(err))
	end

	return success, err
end
