module(..., package.seeall)
local utils = require "shared.apps.utils"
local adm_camp_controller = require("apps.administration.controllers.campaigns")

local m_datasource

function GetName ()
	return "Campaigns"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function LoadAllegroIntegrationInfo ( campaign )
	return campaign.AllegroIntegration or {}
end

local function __GetAccounts (campaignIdWithVCC, keyspace, columns)
	assert(campaignIdWithVCC, "campaignIdWithVCC id is required")

	local vcc = inconcert.authUser.virtualContactCenter
	campaignIdWithVCC = utils.AppendVcc(utils.GetId(campaignIdWithVCC), vcc)

	local params = {
		keyspace     = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		key          = campaignIdWithVCC,
		columns      = columns
	}

	local accounts = utils.GetAllColumns( params ) or {}

	-- tiene que ir un array por eso lo paso asi!!
	local accounts_list = {}
	for _, account in pairs(accounts) do
		table.insert(accounts_list, account)
	end

	return accounts_list
end

local function __GetCampaignSchedulerSpecialDays (campaignIdWithVCC, keyspace)
	local specialDays = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignSchedulersSpecialDays",
		key = campaignIdWithVCC
	} or {}
	return specialDays
end

local function __GetScheduler (campaignIdWithVCC, keyspace)
	local closeKeyspace = false
	if not keyspace then
		closeKeyspace = true
		keyspace = utils.GetDatasource():CassandraConnection("Agent")
	end
	local schedulerColumns = keyspace:GetSliceRange(campaignIdWithVCC,
		"CampaignSchedulers",
		"", {
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = true,
				count = 300
			}
		}
	)
	local activationScheduler = {}
	if schedulerColumns then
		activationScheduler.scheduleDays = {}
		for k, v in pairs(schedulerColumns) do
			local scheduler = {}
			local day, startDate, endDate = string.match(v.name, "(.*)_(.*)_(.*)")
			scheduler.startTime = startDate
			scheduler.endTime = endDate
			scheduler.dayOfWeek = day
			table.insert(activationScheduler.scheduleDays, scheduler)
		end
		-- Aca cargo los special days
		local specialDays = __GetCampaignSchedulerSpecialDays(campaignIdWithVCC, keyspace)
		if specialDays then
			activationScheduler.scheduleSpecialDays = specialDays.scheduleSpecialDays
			activationScheduler.scheduleSpecialDayAllYearss = specialDays.scheduleSpecialDayAllYearss
		end
	end
	if closeKeyspace then
		keyspace:Close()
	end
	return activationScheduler
end

local function __GetThresholds (campaignIdWithVCC, keyspace)
	local closeKeyspace = false
	if not keyspace then
		closeKeyspace = true
		keyspace = utils.GetDatasource():CassandraConnection("Agent")
	end

	local all_thresholds = {}
	local channels = {"CALL", "CHAT", "FACEBOOK", "INSTAGRAM", "YOUTUBE", "APPSTORE", "GOOGLEPLAY", "RCS", "IM", "MAIL", "SMS", "WHATSAPP", "TWITTER", "WEBCONTACT"}
	for i, channel in ipairs(channels) do
		local threshold_key = string.format("%s:%s", channel, campaignIdWithVCC)
		local thresholds_by_channel = keyspace:GetSliceRange(
			threshold_key, 
			"CampaignThresholds",
			"", {
				column_names = {},
				slice_range = {
					start = "",
					finish = "",
					reversed = true,
					count = 300
				}
			}
		)
		for i, column in pairs(thresholds_by_channel) do
			local th_name  = column.name
			local th_value = column.value
			all_thresholds[channel] = all_thresholds[channel] or {}
			all_thresholds[channel][th_name] = tonumber(th_value)
		end
	end

	if closeKeyspace then
		keyspace:Close()
	end
	return all_thresholds
end

local function __GetMembers (campaignIdWithVCC, keyspace)
	-- Voy contra el controller de campanas, sino tengo que duplicar todo el codigo de esa funcion aca, y no vale la pena.
	return adm_camp_controller.LoadMembers( campaignIdWithVCC, keyspace )
end

local function __MakeAccountMessageId ( campaign_id, accountId ) return ( accountId or "" ).. "_" .. ( campaign_id or "" ) end

function __GetAccountMessages ( campaign_id, accountId, keyspace )
	local key = __MakeAccountMessageId( campaign_id, accountId )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAccountMessages",
		key = key
	}
end

local campaign__index = function(campaign, attribute)
	if attribute == "dispositions" then
		local dispositions = getmetatable(campaign).dispositions
		if not dispositions then
			return
		end
		dispositions = json.decode(dispositions)
		rawset(campaign, "dispositions", dispositions)
		return dispositions
	end
	return nil
end

local function __IsAllegroAccount (account)
	return account and account.proxyType == "CALL"
end

function IsAllegroCampaign (vcc, campaignIdWithVCC, keyspace, accounts)
	if not utils.HasAllegroIntegration(vcc) then
		return false
	end
	accounts = accounts or __GetAccounts(campaignIdWithVCC, keyspace)
	for _, account in pairs(accounts) do
		if __IsAllegroAccount(account) then
			return true
		end
	end
	return false
end

function CampaignExists (campaign_at_vcc, keyspace)
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaign_at_vcc,
		column = "Id"
	})
end

function GetCampaign (vcc, simple_camp_id, keyspace, mustLoadMembers)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local campaignIdWithVCC = utils.AppendVcc(utils.GetId(simple_camp_id), vcc)

	local campaign = utils.GetAllColumns({
		keyspace     = keyspace or "Agent",
		columnFamily = "Campaigns",
		key          = campaignIdWithVCC
	})

	if not campaign then
		return
	end

	-- Los datos que deben viajar siempre
	campaign.AllegroIntegration = LoadAllegroIntegrationInfo( campaign )
	campaign.accounts           = __GetAccounts( campaignIdWithVCC, keyspace )
	campaign.scheduler          = __GetScheduler( campaignIdWithVCC, keyspace )
	campaign.Thresholds         = __GetThresholds( campaignIdWithVCC, keyspace )
	if mustLoadMembers then
		campaign.Members        = __GetMembers( campaignIdWithVCC, keyspace )
	end

	local accountsMessages
	for _, account in ipairs(campaign.accounts or {}) do
		accountsMessages = __GetAccountMessages( campaignIdWithVCC, account.id, keyspace )
		break
	end
	campaign.AccountMessages = accountsMessages

	-- Los datos que deben de viajar unicamente para campanas de call allegro integration
	if IsAllegroCampaign (vcc, campaignIdWithVCC, keyspace, campaign.accounts) then
		setmetatable(campaign, {dispositions = campaign.dispositionCodes, __index = campaign__index} )
	else
		campaign.dispositionCodes = nil
	end

	return campaign
end
