require "json"
require "socket" 
local http = require "socket.http" 
local ltn12 = require "ltn12" 

module(..., package.seeall, inconcert.orbit.application.Decorate)

local function __BuildCompletePath (url)
	return "/inconcert/apps/" .. url
end

local function __BuildCompleteUrl (url, ip, port)
	return string.format("http://%s:%s%s", ip, port, __BuildCompletePath(url))
end

function Get (url, headers, cookie, ip, port)
	local uri = __BuildCompleteUrl(url, ip, port)
	LogDebug("")
	LogDebug("-- GET start --")
	LogDebug("GET URI: %s", uri)
	headers = headers or {}
	if cookie and string.len(cookie) then
		headers["Cookie"] = cookie
	end
	LogDebug("GET -> headers; %s", json.encode(headers))
	local response_body = {}
	local result, code, header = http.request{
		url = uri,
		method = "GET",
		headers = headers,
		sink = ltn12.sink.table(response_body)
	}
	local response = table.concat(response_body, "")
	LogDebug("GET-> result: '%s', code: '%s', header: '%s'.", result, code, json.encode(header))
	LogDebug("GET-> response length: '%s'", #(response or ""))
	LogDebug("-- GET end --")
	LogDebug("")
	if code == 200 then
		return true, response
	end
	return false, response, code
end

local function Request (ip, port, uri, requestHeaders)
	local code, status, headers, request
	local result, message = pcall(http.open, ip, port)

	if (result == true) then
		request = message
		result, message = pcall(request.sendrequestline, request, "GET", uri)
		if result == true then
			request:sendheaders(requestHeaders)
			result, message, status = pcall(request.receivestatusline, request)
			if (result == true) then
				code = message
				while (code == 100) do 
					headers = request:receiveheaders()
					code, status = request:receivestatusline()
				end
				headers = request:receiveheaders()
			else
				code = 504
			end
		else
			code = 500
		end
	else
		code = 500
	end

	if not headers then
		headers = { ["Content-Length"] = 0 }
	end

	if (result == false) then
		status = "HTTP/1.1 " .. code .. " " .. message[1]
	end

	return result, code, headers, status, request
end

function StreamGet (url, headers, cookie, ip, port)
	local uri = __BuildCompletePath(url)
	LogDebug("")
	LogDebug("-- STREAM GET start --")
	LogDebug("STREAM GET URI: %s", uri)
	headers = headers or {}
	if cookie and string.len(cookie) then
		headers["Cookie"] = cookie
	end
	LogDebug("STREAM GET -> headers; %s", json.encode(headers))
	local result, code, header, status, request = Request(ip, port, uri, headers)
	LogDebug("STREAM GET-> result: '%s', code: '%s', header: '%s'.", status, code, json.encode(header))
	LogDebug("-- STREAM GET end --")
	LogDebug("")
	if code == 200 then
		return true, request, code, headers
	end
	return false, status, code, headers
end

function Post (url, body, headers, cookie, ip, port, timeout_sec)
	local uri = __BuildCompleteUrl(url, ip, port)

	LogInfo("")
	LogInfo("-- POST start --")
	LogInfo("POST URI: " .. tostring(uri))

	headers = headers or {}
	headers["content-length"] = headers["content-length"] or (body and string.len(body) or 0)

	if cookie and string.len(cookie) then
		headers["Cookie"] = cookie
	end

	local response_body = {}
	LogInfo("POST -> headers: %s", json.encode(headers))
	--LogDebug("POST -> body: %s", json.encode(body))
	if timeout_sec then
		http.TIMEOUT = timeout_sec
	end
	--LogDebug("POST -> http.TIMEOUT (sec) : '%s'", http.TIMEOUT)

	local result, code, header = http.request{
		url = uri,
		method = "POST",
		headers = headers,
		source = ltn12.source.string(body),
		sink = ltn12.sink.table(response_body)
	}

	local response = table.concat(response_body, "")
	LogInfo("POST-> result: '%s', code: '%s', header: '%s'.", result, code, json.encode(header))
	LogInfo("POST-> response: '%s'", response)
	LogInfo("-- POST end --")
	LogInfo("")

	if code == 200 then
		return true, response
	end
	return false, response, code
end
