/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 6  */

/* global inConcert, BaragentController*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
    this.inConcert = {};
}

(function () {

	//constructor
	function BaragentPolling() {
		this.m_pollingTime = 1000; //TODO: Configurame
		this.m_baController = new BaragentController();
		this.m_lastStatus = null;
		this.m_interval = null;
		this.m_running = false;
		this.m_observers = {};
		this.m_poolingFailures = 0;
		this.m_maxConsecutiveFailures = 10;
		this.callbackPolling = $.noop;
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.BaragentPolling !== 'object') {
		inConcert.BaragentPolling = new BaragentPolling();
	}

	function processPolling(self, status) {
		$.each(self.m_observers, function(_, observer) {
			if (observer) {
				observer.instance[observer.method](status);
			}
		});
	}

	function onPolling(self) {
		if (self.m_running) {
			return;
		}
		self.m_running = true;
		self.m_baController.GetBarAgentStatus
		(
			function(status) {
				try {
					//if is equal than last then return
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 1 -", status: status});
					if ( !status || !status.data || !status.data.status) {
						//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 2"});
						self.m_running = false;
						self.callbackPolling(false);
						return;
					}
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 3"});
					status = JSON.parse(status.data.status);
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 4"});
					if (status.onChangeData === true) {
						//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 5"});
						inConcert.logger.info({ message : "[baragentPolling][GetBarAgentStatus] onChangeData = true. descarto los datos obtenidos" });
						self.m_running = false;
						self.callbackPolling(false);
						return;
					}
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 6"});
					self.m_poolingFailures = 0;
					if (!status.activeInteractions.length) {
						//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 7"});
						status.interaction = {};
					}
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 8"});
					self.m_lastStatus = status;
					processPolling(self, self.m_lastStatus);
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 9"});
					self.callbackPolling(true, status);
					//console.log({ message : "[BaragentPolling.prototype.start --> onPolling] 10"});
				}
				catch(e) {
					self.callbackPolling(false);
					inConcert.logger.warn({ message : "[BaragentPolling.prototype.start --> onPolling] Error processing baragent polling", error : e.message });
				}
				self.m_running = false;
			},
			function() {
				self.m_poolingFailures++;
				var message = "Server error processing baragent polling. Is " + self.m_poolingFailures + " consecutive failure.";
				if (self.m_poolingFailures >= self.m_maxConsecutiveFailures) {
					self.callbackPolling(false);
					message += " User will be logged out.";
					inConcert.logger.warn({ message : message });
					self.logoutAgent();
					return;
				}
				else {
					message += " Will retry in " + self.m_pollingTime + " ms.";
					inConcert.logger.warn({ message : message });
				}
				self.callbackPolling(false);
				self.m_running = false;
			}
		);
	}

	BaragentPolling.prototype.logoutAgent = function() {
		inConcert.app.logout(function(kickout) {
			var logoutPopupMsg = "<%=_T('Telephony components are down')%>";
			if(kickout && kickout.UserID === "SYSTEM")
			{
				logoutPopupMsg = "<%=_T('Max strikes reached')%>";
			}
			$.inconcertPopup.Show(
				logoutPopupMsg,
				"<%=_T('Press OK to retry login.')%>",
				$.inconcertPopup.ButtonSets.BS_OK,
				function(/*response*/) {
					window.location = "/inconcert/apps/agent/login/";
				},
				undefined,
				{
					popup_id        : "agent_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		},"baragentPolling-onPolling");
	};

	BaragentPolling.prototype.start = function(callbackPolling = $.noop) {
		if (!inConcert.app.has_allegro_baragent()) {
			return;
		}
		var self = this;
		this.callbackPolling = callbackPolling;
		if (!this.m_interval) {
			this.m_interval = window.setInterval(function() {
				onPolling(self);
			}, this.m_pollingTime);
		}
	};

	BaragentPolling.prototype.stop = function() {
		if (this.m_interval) {
			window.clearInterval(this.m_interval);
			this.m_interval = null;
		}
	};

	BaragentPolling.prototype.enabled = function() {
		return this.m_interval !== null;
	};

	BaragentPolling.prototype.addObserver = function(id, instance, method) {
		this.m_observers[id] = { instance : instance, method : method };
	};

	BaragentPolling.prototype.removeObserver = function(id) {
		this.m_observers[id] = null;
	};

	// El parametro "account" queda comentado porque no se realiza nada con este parametro dentro de la funcion.
	// TODO: revisar si es correcto que quede, o se puede eliminar el mismo.
	BaragentPolling.prototype.getAccountStatus = function(/*account*/) {
		if (!this.m_lastStatus) {
			return "CONNECTING";
		}
		return { status : this.m_lastStatus.agent && this.m_lastStatus.agent.id? "OK" : "FAIL" };
	};

	BaragentPolling.prototype.hasInteraction = function() {
		return this.m_lastStatus && this.m_lastStatus.interaction && this.m_lastStatus.interaction.id;
	};

	BaragentPolling.prototype.hasThisInteraction = function(id) {
		return this.m_lastStatus && this.m_lastStatus.interaction && this.m_lastStatus.interaction.id === id;
	};

	BaragentPolling.prototype.isRedial = function() {
		let redial = false;
		$.each(this.m_lastStatus && this.m_lastStatus.activeInteractions || [], function(_, interaction) {
			if (interaction && interaction.phoneState === "REDIAL") {
				redial = true;
				return false;
			}
		});
		if (!redial) {
			let interactions = inConcert.app.m_interactionManager.getInteractions();
			$.each(interactions, function(_, interaction) {
				if (interaction.type === "CALL" && (interaction.state === "LOCKED" || interaction.state === "TAKEN")) {
					let view = interaction.getView();
					if (view && view.had_wrapup) {
						redial = true;
						console.info("Redial detected from view");
					}
					else if (interaction.getRedialProcess()) {
						redial = true;
						console.info("Redial detected from process");
					}
				}
			});
		}
		return redial;
	};

	BaragentPolling.prototype.isRedialInteraction = function(interaction) {
		return this.isRedial() && interaction.id === this.m_lastStatus.interaction.id;
	};

	BaragentPolling.prototype.getStatusInteraction = function() {
		return this.m_lastStatus && this.m_lastStatus.interaction?
			this.m_lastStatus.interaction : null;
	};

	BaragentPolling.prototype.getPopupInfo = function() {
		return this.m_lastStatus && this.m_lastStatus.interaction && this.m_lastStatus.interaction.popup || null;
	};

	BaragentPolling.prototype.getAuthToken = function() {
		return this.m_lastStatus && this.m_lastStatus.agent && this.m_lastStatus.agent.token || null;
	};

}());
