local utils = require "shared.apps.utils"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local function __Search (vcc, encodedInfo, url, contentType)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local allegro = require "apps.allegro.proxy"
	return allegro.Post(vcc, encodedInfo, url, contentType)
end

function Search (url, encodedInfo, vcc, contentType)
	local info = utils.JSON_Decode(encodedInfo)
	vcc = vcc or info.vcc or inconcert.authUser.virtualContactCenter
	--LogDebug("[ALLEGRO.SHARED_VIEW:Search] Searching - vcc: '%s', url: '%s', info: '%s'.", vcc, url, encodedInfo)
	local success, data = __Search(vcc, encodedInfo, url, contentType)
	if not success or not data then
		-- Error
		local err = data or {}
		LogError("[ALLEGRO.SHARED_VIEW:Search] Error Searching. Err: '%s'.", json.encode(err))
		return false, err
	end
	--------------------
	-- OK
	--------------------
	local pageRows = data.paginated["0"]
	--LogDebug("[ALLEGRO.SHARED_VIEW:Search] Successfully Searched %d rows.", #pageRows)
	--LogDebug("[ALLEGRO.SHARED_VIEW:Search] Successfully Searched -> data: %s.", json.encode(data))

	local allegroPaginated = data.paginated.__config -- valores que llegan desde allegro
	local numberOfPages = tonumber(allegroPaginated.numberOfPages)
	--LogDebug("[ALLEGRO.SHARED_VIEW:Search] Successfully Searched -> allegroPaginated: %s.", json.encode(allegroPaginated))
	--[[
		Funcion que retorna el total de rows exactos, cuando es la ultima pagina.
		Esto ya que en Allegro el LUA.ORM, nunca informa exactamente el total de rows, sino que hace multiplos del total de las paginas
			ej: el ORM si tiene 4 paginas de 16 rows, va a retornar como total 64 rows sin importar si en la ultima pagina alcanza al total de rows para completar los 64 rows.
	]]--
	local function GetTotalRows ()
		local totalRows = allegroPaginated.numberOfRecords
		if not info
			or not info.numberPage
			or not info.limit
		then
			--1 - aca no deberia entrar nunca ya que allegro deberia retornar estos valores, pero pos si las moscas nos cubrimos.
			--LogDebug("[ALLEGRO.SHARED_VIEW:Search] (GetTotalRows) 1-  totalRows: %s.", totalRows)
			return totalRows
		end
		if info.numberPage ~= numberOfPages then
			--2 - si no es la ultima pagina, retornamos el total que nos pasa allegro ya que no nos ocasiona problemas.
			--LogDebug("[ALLEGRO.SHARED_VIEW:Search] (GetTotalRows) 2-  totalRows: %s.", totalRows)
			return totalRows
		end
		local totalPageRows = #pageRows
		if totalPageRows == info.limit then
			--3 - si la ultima pagina y la pagina esta completa retornamos el total que nos retorna allegro.
			--LogDebug("[ALLEGRO.SHARED_VIEW:Search] (GetTotalRows) 3-  totalRows: %s.", totalRows)
			return totalRows
		end
		--4 - Ahora si debemos calcular el total de rows, si es la ultima pagina y el total de rows no alcanza al total de rows soportados por la pagina.
		totalRows = totalRows - (info.limit - totalPageRows)
		--LogDebug("[ALLEGRO.SHARED_VIEW:Search] (GetTotalRows) 4-  totalRows: %s.", totalRows)
		return totalRows
	end
	------------------
	local response = {
		lastRow    = allegroPaginated.lastRow or "",
		numberPage = info.numberPage,
		rows       = pageRows,
		total_rows = GetTotalRows(),
		totalPages = numberOfPages
	}
	--LogDebug("[ALLEGRO.SHARED_VIEW:Search] Successfully Searched --->  response: %s.", json.encode(response))
	return true, response
end
