local utils = require "shared.apps.utils"
local m_helpers = require "helpers"

module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil

local function __MakeCreatedBy (from_app)
	return {
		version = "allegro",
		fromApp = from_app or "provisioning"
	}
end

local function __Num (value, default_value)
	value = tonumber( value )
	if value then
		return value
	end
	return default_value
end

--[[
RECEIVES "allegro_user": 
	{
		UserGroup        = "administrator",
		ValidationOrigin = 0,
		Country          = "",
		Id               = "u1",
		VccGroup         = "system",
		LanguageId       = "en",
		Name             = "u1",
		TimeZoneId       = "UTC",
		IsSystemUser     = 0,
		State            = "ENABLED",
		countryCode      = "598",
		areaCode         =  "",
		Password         = "a4cb5cf0c26c130189bc9536bd787515",
		VCC              = "vcc1",
		-- optional
		Agent            = {
			Email = "cany@inconcertcc.com"
		}
	}
------
RETURNS "omnichannel_user":
	{
	-- VALORES QUE MAPEAN
		newUser        = true, -- este no mapea
		Id             = "u1",
		Name           = "u1"
		FirstSurname   = "u1",
		TimeZoneId     = "UYT",
		Mail           = "MAIL_u1@CHANGEME.COM",
		UserGroup      = "administrator",
		Enabled        = "1",
		Password       = "a4cb5cf0c26c130189bc9536bd787515",
		LanguageId     = "en",
		countryCode      = "598",
		areaCode         =  "",

	-- JSON con la informacion que se ha posteado desde Allegro
		AllegroRawInfo = {...}
	}
--]]

function MakeOmnichannelUser (vcc_id, allegro_user, is_new, from_app)
	if not vcc_id or not allegro_user then
		LogWarning("[ALLEGRO.TEMPLATES:MakeOmnichannelUser] Error creating template omnichannel user in vcc: '%s', allegro info: %s.", vcc_id, json.encode(allegro_user or {}))
		return false
	end

	local created_by
	if not is_new then
		is_new = false
	else
		is_new = true
		created_by = __MakeCreatedBy(from_app)
	end

	local function __AddUserCampaign (userCampaigns, campaignId)
		local omnichannelUserCampaign = {
			id = utils.AppendVcc(campaignId, vcc_id),
			role = "agent",
			invite = true
		}
		table.insert(userCampaigns, omnichannelUserCampaign)
	end

	local function __GetUserCampaigns ()
		local campaigns = allegro_user.UserCampaigns
		if not campaigns then
			return
		end
		local userCampaigns = {}
		for _, campaign in pairs(campaigns) do
			__AddUserCampaign(userCampaigns, campaign.CampaignId)
		end
		return userCampaigns
	end

	local function __GetUserSkills ()
		local allegroSkills = allegro_user.Agent and allegro_user.Agent.AgentSkills
		if not allegroSkills then
			return
		end
		local omnichannelSkills = {}
		for _, allegroSkill in pairs(allegroSkills) do
			local omnichannelSkill = {
				skill = allegroSkill.SkillName,
				weight = tostring(allegroSkill.SkillValue)
			}
			table.insert(omnichannelSkills, omnichannelSkill)
		end
		return omnichannelSkills
	end

	local function __GetAllegroIntegrationInfo ()
		if not allegro_user.Agent then
			return
		end
		local allegroIntegration = {
			peer = {
				service    = allegro_user.Agent.TelephonyService, --"TELEPHONY_192_168_24_236"
				peer       = allegro_user.Agent.TelephonePeer, --"2002"
				technology = allegro_user.Agent.Technology --"ASTERISK,SIP"
			}
		}
		return allegroIntegration
	end

	local mail, workgroupId, defaultCampaign
	if allegro_user.Agent then
		mail = allegro_user.Agent.Email
		workgroupId = allegro_user.Agent.WorkgroupId and tostring(allegro_user.Agent.WorkgroupId)
		defaultCampaign = allegro_user.Agent.DefaultCampaign and tostring(allegro_user.Agent.DefaultCampaign)
	end

	-- Aca valida si tiene que agregar la "default campaign" a la lista de " user campaigns"
	local mustAddDefaultCampaignToList = false
	local userCampaigns = __GetUserCampaigns()

	if defaultCampaign then
		local alreadyExist = false
		userCampaigns = userCampaigns or {}
		-- Si la campana "default campaign", NO existe en la lista, la tengo que agregar.
		local defaultCampaignWithVCC = utils.AppendVcc(utils.GetId(defaultCampaign), vcc_id)
		defaultCampaignWithVCC = defaultCampaignWithVCC:lower()
		for _, campaignInfo in ipairs(userCampaigns) do
			if campaignInfo.id:lower() == defaultCampaignWithVCC then
				alreadyExist = true
				break
			end
		end
		if not alreadyExist then
			mustAddDefaultCampaignToList = true
		end
	end

	if mustAddDefaultCampaignToList then
		__AddUserCampaign(userCampaigns, defaultCampaign)
	end
	-----------------------------
	LogDebug("[ALLEGRO.TEMPLATES:MakeOmnichannelUser] 1 allegro_user: '%s'.", json.encode(allegro_user))

	-- Country
	local countryCode
	if not allegro_user.Country or allegro_user.Country == "" then
		countryCode = "598"
	else
		countryCode = tostring(allegro_user.Country)
	end
	-- Area
	local areaCode
	if allegro_user.areaCode then
		areaCode = tostring(allegro_user.areaCode)
	end

	local omnichannelUser = {
		newUser            = is_new,
		Id                 = utils.AppendVcc(allegro_user.Id, vcc_id),
		Name               = allegro_user.Name or allegro_user.Id,
		FirstSurname       = allegro_user.LastName or allegro_user.Name or allegro_user.Id,
		TimeZoneId         = allegro_user.TimeZoneId or "UYT",
		countryCode        = countryCode,
		areaCode           = areaCode,
		Mail               = mail,
		WorkgroupId        = workgroupId,
		DefaultCampaign    = defaultCampaign,
		IsSystemUser       = allegro_user.IsSystemUser or "0",
		UserGroup          = allegro_user.UserGroup,
		Enabled            = allegro_user.State == "ENABLED" and "1" or "0",
		Password           = allegro_user.Password,
		LanguageId         = allegro_user.LanguageId,
		CreatedBy          = created_by,
		groups             = userCampaigns,
		skills             = __GetUserSkills(),
		AllegroIntegration = __GetAllegroIntegrationInfo()
	}

	LogDebug("[ALLEGRO.TEMPLATES:MakeOmnichannelUser] 2 omnichannelUser: '%s'.", json.encode(omnichannelUser))
	return omnichannelUser
end

--[[
RECEIVES "allegro_vcc": 
	{
		Id           = "INCONCERT",
		Description  = "VCC de prueba",
		Company      = "Inconcert"
		Telephone1   = "11111111",
		Telephone2   = "UYT",
		ContactName  = "Nombre de contacto",
		ContactEmail = "contactmail@mail.com"
	}
------
RETURNS "omnichannel_vcc":
	{
	-- VALORES QUE MAPEAN
		Id           = "INCONCERT",
		Description  = "VCC de prueba",
		Company      = "Inconcert"
		Telephone1   = "11111111",
		Telephone2   = "UYT",
		ContactName  = "Nombre de contacto",
		ContactEmail = "contactmail@mail.com",

	-- VALORES QUE HAY AGREGARLES UN DEFAULT
		FreeSecondsAfterInteraction = "0",
		OnlineUsers                 = "100",
		IsSystemVCC                 = "0",
		WEBCONTACT_licensed         = "1",
		CALL_licensed               = "1",
		CHAT_licensed               = "1",
		MAIL_licensed               = "1",
		TWITTER_licensed            = "1",
		FACEBOOK_licensed           = "1",
		CustomMetricsLimit          = "0",
		CustomFiltersLimit          = "0",
		ExportSpaceQuota            = "0",

	-- JSON con la informacion que se ha posteado desde Allegro
		AllegroRawInfo = {...}
	}
--]]

function MakeOmnichannelVcc ( allegro_vcc, omnichannel_vcc )
	if not allegro_vcc then
		LogWarning("[ALLEGRO.TEMPLATES:MakeOmnichannelVcc] Error creating template omnichannel vcc.")
		return false, "INCOMPLETE_INFORMATION"
	end

	omnichannel_vcc = omnichannel_vcc or {}
	local is_new = utils.TableIsEmpty(omnichannel_vcc)

	if not is_new then
		LogInfo("[ALLEGRO.TEMPLATES:MakeOmnichannelVcc] Validating has integration in VCC '%s', AllegroIntegration: '%s'.", allegro_vcc.Id, omnichannel_vcc.AllegroIntegration)

		local allegroIntegration
		if type(omnichannel_vcc.AllegroIntegration) == "table" then
			allegroIntegration = omnichannel_vcc.AllegroIntegration
		else
			local success
			success, allegroIntegration = pcall(json.decode, omnichannel_vcc.AllegroIntegration)
			if not success then
				LogError("[ALLEGRO.TEMPLATES:MakeOmnichannelVcc] Unable to create or update '%s' VCC. Decode Error", allegro_vcc.Id)
				return false, "VCC_DECODE_ERROR_ALLEGRO_INTEGRATION"
			end
		end

		if not allegroIntegration.enabled then
			LogError("[ALLEGRO.TEMPLATES:MakeOmnichannelVcc] Unable to create or update '%s' VCC. This VCC already exist without Allegro Integration", allegro_vcc.Id)
			return false, "VCC_ALREADY_EXISTS_WITHOUT_ALLEGRO_INTEGRATION"
		end
	end

	-- Esta funcion calcula la cantidad de usuarios que pueden haber logueados al mismo tiempo en i6.
	--  Se calcula a partir de las licencias que llegan desde 5.5 en la tabla "VCCLicenses"
	--   y se calcula segun la cantidad de licencias que se hayan otorgado a los roles "administrator" + "agent" + "supervisor".
	--
	local function __CalculateOnlineUsers()

		local allowed_license_types = { ADMINISTRATOR_LICENSED = true, AGENTES_LICENSED = true, SUPERVISOR_LICENSED = true }
		local online_users = 0

		for _, license in ipairs(allegro_vcc.VCCLicenses) do
			if allowed_license_types[ license.LicenseId ] then
				online_users = online_users + (tonumber(license.Total) or 0)
			end
		end

		return tostring(online_users)
	end

	-- 1 valores que llegan desde Allegro
	omnichannel_vcc.Id                          = allegro_vcc.Id
	omnichannel_vcc.Description                 = allegro_vcc.Description
	omnichannel_vcc.Company                     = allegro_vcc.Company
	omnichannel_vcc.CompanyAddress              = allegro_vcc.CompanyAddress
	omnichannel_vcc.Telephone1                  = allegro_vcc.Telephone1
	omnichannel_vcc.Telephone2                  = allegro_vcc.Telephone2
	omnichannel_vcc.ContactName                 = allegro_vcc.ContactName
	omnichannel_vcc.ContactEmail                = allegro_vcc.ContactEmail
	omnichannel_vcc.AllegroIntegration          = json.encode( allegro_vcc.AllegroIntegration or { enabled = true })
	omnichannel_vcc.OnlineUsers                 = __CalculateOnlineUsers()

	-- 2 valores que NO llegan desde Allegro y hay que meterles un default
	omnichannel_vcc.FreeSecondsAfterInteraction = omnichannel_vcc.FreeSecondsAfterInteraction or "0"
	omnichannel_vcc.IsSystemVCC                 = "0"
	-- Licenses Channels
	omnichannel_vcc.APPSTORE_licensed           = omnichannel_vcc.APPSTORE_licensed           or "0"
	omnichannel_vcc.CALL_licensed               = omnichannel_vcc.CALL_licensed               and m_helpers.utils.HasLicenseSystem() and "0" or "1"
	omnichannel_vcc.CHAT_licensed               = omnichannel_vcc.CHAT_licensed               or "0"
	omnichannel_vcc.FACEBOOK_licensed           = omnichannel_vcc.FACEBOOK_licensed           or "0"
	omnichannel_vcc.GOOGLEPLAY_licensed         = omnichannel_vcc.GOOGLEPLAY_licensed         or "0"
	omnichannel_vcc.INSTAGRAM_licensed          = omnichannel_vcc.INSTAGRAM_licensed          or "0"
	omnichannel_vcc.MAIL_licensed               = omnichannel_vcc.MAIL_licensed               or "0"
	omnichannel_vcc.RCS_licensed            	= omnichannel_vcc.RCS_licensed            	  or "0"
	omnichannel_vcc.SMS_licensed                = omnichannel_vcc.SMS_licensed                or "0"
	omnichannel_vcc.TWITTER_licensed            = omnichannel_vcc.TWITTER_licensed            or "0"
	omnichannel_vcc.WEBCONTACT_licensed         = omnichannel_vcc.WEBCONTACT_licensed         or "0"
	omnichannel_vcc.WHATSAPP_licensed           = omnichannel_vcc.WHATSAPP_licensed           or "0"
	omnichannel_vcc.YOUTUBE_licensed            = omnichannel_vcc.YOUTUBE_licensed            or "0"
	-- Licenses Metrics, Filters, Quotas
	omnichannel_vcc.CustomMetricsLimit          = omnichannel_vcc.CustomMetricsLimit          or "0"
	omnichannel_vcc.CustomFiltersLimit          = omnichannel_vcc.CustomFiltersLimit          or "0"
	omnichannel_vcc.ExportSpaceQuota            = omnichannel_vcc.ExportSpaceQuota            or "0"
	omnichannel_vcc.SharedFilesSpaceQuota       = omnichannel_vcc.SharedFilesSpaceQuota       or "0"

	return omnichannel_vcc, is_new
end

--[[
RECEIVES "allegro_scheduler": 
{
  "ScheduleDays": [
    {
      "StartTime": "00:00:00",
      "EndTime": "00:10:00",
      "DayOfWeek": 2,
      "Enabled": 1
    }
  ],
  "ScheduleSpecialDays": [],
  "ScheduleSpecialDayAllYearss": [],
  "Id": "nuevosch1",
  "OutboundProcessId": "OE_c10"
}
------
RETURNS "omnichannel_scheduler":
{
  "scheduleSpecialDays": [],
  "scheduleDays": [
    {
      "endTime": "1970-01-01 00:10:00",
      "startTime": "1970-01-01 00:00:00",
      "enabled": 1,
      "dayOfWeek": 2
    }
  ],
  "scheduleSpecialDayAllYearss": []
}
--]]

function MakeOmnichannelScheduler ( allegro_scheduler )

	if not allegro_scheduler then
		LogWarning("[ALLEGRO.TEMPLATES:MakeOmnichannelScheduler] Error creating template omnichannel scheduler, allegro info: %s.", json.encode(allegro_scheduler or {}))
		return false
	end

	local omnichannel_scheduler = { scheduleDays = {}, scheduleSpecialDays = {}, scheduleSpecialDayAllYearss = {} }

	local function __FullDate( hour )
		-- aca llega una hora con este formato: "00:00:00" y retorno la fecha "1970-01-01 00:00:00"
		return  "1970-01-01 " .. hour
	end

	-- 1 - los schedule days
	for _, day in ipairs( allegro_scheduler.ScheduleDays ) do
		local sch_day = {
			startTime = day.StartTime,
			endTime   = day.EndTime,
			dayOfWeek = __Num( day.DayOfWeek ),
			enabled   = 1
		}
		table.insert( omnichannel_scheduler.scheduleDays, sch_day )
	end

	-- 2 - los schedule special days
	for _, day in ipairs( allegro_scheduler.ScheduleSpecialDays ) do
		local dayRanges = {}
		for _, range in ipairs( day.ScheduleSpecialDayRanges ) do
			local schRange = {
				startTime = range.StartTime,
				endTime   = range.EndTime,
				day       = range.Day,
				month     = range.Month,
				year      = range.Year
			}
			table.insert( dayRanges, schRange )
		end

		local sch_day = {
			day = day.Day,
			month = day.Month,
			year = day.Year,
			description = day.Description,
			scheduleSpecialDayRanges = dayRanges,
			overrides = day.Overrides
		}
		table.insert( omnichannel_scheduler.scheduleSpecialDays, sch_day )
	end

	-- 3 - los schedule special day all year
	for _, day in ipairs( allegro_scheduler.ScheduleSpecialDayAllYearss ) do
		local sch_day = {
			day = day.Day,
			month = day.Month,
			year = day.Year,
			description = day.Description,
			overrides = day.Overrides
		}
		table.insert( omnichannel_scheduler.scheduleSpecialDayAllYearss, sch_day )
	end

	return omnichannel_scheduler
end

--[[
	Funcion encargada de eliminar datos de la Campana del JSON que se envia en el POST hacia Allegro.
]]
function SanitizeOmnichannelCampaign ( omnichannelCampaign )
	if not omnichannelCampaign then
		LogWarning("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] Error Sanitizing omnichannel campaign.")
		return
	end
	LogInfo("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] START + Sanitize Allegro campaign: %s", omnichannelCampaign.Id)
	--LogDebug("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] ANTES -> omnichannelCampaign: %s", json.encode(omnichannelCampaign))
	------------------
	-- 1. Sanitizo las cuentas (solo dejo las de CALL)
	local allegroChannels = {CALL = true, VOICEMAIL = true}
	local isAllegroCampaign = false
	for _, account in pairs(omnichannelCampaign.accounts or {}) do
		local proxyType = (account.proxyType or ""):upper()
		if allegroChannels[proxyType] then
			LogDebug("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign]  + accountId: %s -> isAllegroCampaign: true.", account.id)
			isAllegroCampaign = true
			break
		else
			LogDebug("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign]  - accountId: %s -> isAllegroCampaign: false.", account.id)
		end
	end
	--
	if not isAllegroCampaign then
		-- NO se deben de enviar cuentas "accounts" si no es una campana de Allegro, es decir no tiene cuentas de CALL o VOICEMAIL.
		LogInfo("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] Sanitized - Remove Accounts in campaign: %s", omnichannelCampaign.Id)
		omnichannelCampaign.accounts = {}
	end
	-------------------
	-- 2. Sanitizo las Integraciones
	if omnichannelCampaign.Integration then
		-- Estas son las integraciones permitidas en 5.5
		local allowedIntegration = {CTIALLEGROPOPUP = true, CTIPOPUP = true, CTIPOPUPWEB = true}
		if allowedIntegration[omnichannelCampaign.Integration.type] then
			-- En i6 la integración con 5.5 se almacena con la clave CTIALLEGROPOPUP, pero en 5.5 se debe almacenar con la clave CTIPOPUP issue: http://redmine.inconcert/issues/7871
			local integrationType = omnichannelCampaign.Integration.type
			if integrationType == "CTIPOPUP" or integrationType == "CTIALLEGROPOPUP" then
				if integrationType == "CTIALLEGROPOPUP" then
					omnichannelCampaign.Integration.type = "CTIPOPUP"
					LogInfo("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] Sanitized - Change Integration type '%s' for 'CTIPOPUP' is not allowed in Allegro in campaign: %s", integrationType, omnichannelCampaign.Id)
				end
			end
			-- Siempre hay que realizar un sanitize y obtener el valor,
			--  esto ya que para los casos de los atributos de integracion de tipo context, se alamcena de la siguiente forma "params.elvalor"
			--  No se él porqué se almacena así, entiendo que deberia cambiar y almacenarlo de alguna forma diferente.
			-- TODO: Ver cambiar la forma de almacenar client side en la config cci integration de la campana para los context!!
			local hasChangedParamsValues = false
			for k, v in pairs(omnichannelCampaign.Integration.popup.parameters) do
				local hasAPoint = v:match("%.")
				if hasAPoint then
					local parameterValue = v:match(".*%.([^%.]*)$")
					omnichannelCampaign.Integration.popup.parameters[k] = parameterValue
					hasChangedParamsValues = true
				end
			end
			if hasChangedParamsValues then
				LogInfo("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] Sanitized - Change params values in Integration type '%s' for 'CTIPOPUP' is not allowed in Allegro in campaign: %s", integrationType, omnichannelCampaign.Id)
			end
		else
			-- Si el tipo de integración de i6 NO esta permitido en 5.5 el mismo NO debe de ir en el request hacia 5.5
			--  Esto es el caso en que en i6 se haya configurado una integración de WebDesigner de i6, esto en i6 se alamacena con la clave CTIPOPUP, pero a 5.5 no tiene que ir.
			omnichannelCampaign.Integration = nil
		end
	end
	-----------------------
	-- OK: retorno la campana sanitizada.
	LogInfo("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] END - Sanitize Allegro campaign: %s", omnichannelCampaign.Id)
	--LogDebug("[ALLEGRO.TEMPLATES:SanitizeOmnichannelCampaign] DESPUES -> omnichannelCampaign: %s", json.encode(omnichannelCampaign))
end
