/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert, _, Mustache, EmojiPicker, inConcertLanguages */

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementCreate) {
	inConcert.batchManagementCreate = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.batch_management;

	function CreatePopup (account)
	{
		this.account = account;
	}
	
	if (typeof inConcert.batchManagementCreate.base !== "function") {
		inConcert.batchManagementCreate.base = CreatePopup;
	}
	
	CreatePopup.prototype.HasId = function() {
		return true;
	};
	
	CreatePopup.prototype.HasScheduledDate = function() {
		return false;
	};
	
	CreatePopup.prototype.HasBackdatedTime = function() {
		return false;
	};
	
	CreatePopup.prototype.__getScheduleDateLimits = function() {
		return null;
	};
	
	CreatePopup.prototype.__getBackdatedTimeLimits = function() {
		return null;
	};
	
	CreatePopup.prototype.__getTextLimit = function() {
		return null;
	};
	
	CreatePopup.prototype.__getAvailableContentTypes = function() {
		return null;
	};
	
	CreatePopup.prototype.__getMaxFileSizeMB = function() {
		return 0;
	};
	
	CreatePopup.prototype.__getClassName = function() {
		return "";
	};
	
	CreatePopup.prototype.__isScheduled = function(data) {
		return false;
	};
	
	CreatePopup.prototype.__addDataAsync = function(data, callback) {
		callback(true);
	};
	
	CreatePopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.__loadInitialData(function(err) {
			if (err) {
				inConcert.app.m_mainWindow.showBannerMessage({ message: err, messageType: "Error", timeout: 5000 });
				callback();
				return;
			}
			self.Close(function() {
				self.__show(callback);
			});
		});
	};
	
	CreatePopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#outgoingBatchesCreatePopup").remove();
				callback();
			});
		}
		else {
			$("#outgoingBatchesCreatePopup").remove();
			callback();
		}
	};
	
	CreatePopup.prototype.Cancel = function(callback) {
		callback();
	};
	
	CreatePopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.create_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.__makeTabs();
		self.__showText();
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "outgoingBatchesCreatePopup",
			eventName     : "outgoingBatchesCreatePopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#outgoingBatchesCreatePopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	CreatePopup.prototype.__makeDateLimit = function(definition, now) {
		if (!definition) {
			return null;
		}
		var value = (definition.up? 1 : -1) * definition.value;
		var date = new Date(now);
		date.setMilliseconds(0);
		date.setSeconds(0);
		switch(definition.unity) {
			case "minutes":
				date.setMinutes(date.getMinutes() + value);
				break;
			case "hours":
				date.setHours(date.getHours() + value);
				break;
			case "days":
				date.setDate(date.getDate() + value);
				break;
			case "months":
				date.setMonth(date.getMonth() + value);
				break;
			case "years":
				date.setFullYear(date.getFullYear() + value);
				break;
		}
		return date;
	};
	
	CreatePopup.prototype.__makeDatetime = function($section, limits) {
		limits = limits || {};
		limits["default"] = limits["default"] || new Date();
		
		$section.find("[date]")
			.datepicker({
				dateFormat: "yy-mm-dd",
				minDate: limits.min, 
				maxDate: limits.max
			})
			.datepicker("setDate", limits["default"]);
			
		$section.find("[hour]")
			.timeEntry({
				spinnerImage : "",
				show24Hours : true
			})
			.val(inConcert.Utils.zeropad(limits["default"].getHours()) + 
				":" + inConcert.Utils.zeropad(limits["default"].getMinutes()));
	};
	
	CreatePopup.prototype.__makeScheduledDate = function($body) {
		var $scheduled = $body.find("[schedule_date]");
		if (!this.HasScheduledDate()) {
			$scheduled.remove();
			return false;
		}
		this.__makeDatetime($scheduled, this.__getScheduleDateLimits());
		return true;
	};
	
	CreatePopup.prototype.__makeBackdatedTime = function($body) {
		var $backdated = $body.find("[backdated_time]");
		if (!this.HasBackdatedTime()) {
			$backdated.remove();
			return false;
		}
		this.__makeDatetime($backdated, this.__getBackdatedTimeLimits());
		return true;
	};
	
	CreatePopup.prototype.__getDatetime = function($section) {
		if (!$section.find("input[type=checkbox]").prop("checked")) {
			return null;
		}
		var date = $section.find("[date]").val();
		var hours = $section.find("[hour]").val() || "00:00";
		return new Date(date + "T" + hours);
	};
	
	CreatePopup.prototype.__getScheduledDate = function() {
		return this.__getDatetime($("#outgoingBatchesCreatePopup").find("[schedule_date]"));
	};
	
	CreatePopup.prototype.__getBackdatedTime = function() {
		return this.__getDatetime($("#outgoingBatchesCreatePopup").find("[backdated_time]"));
	};
	
	CreatePopup.prototype.__makeAdvancedOptions = function($body) {
		var $options = $body.find("[advanced_options]");
		$options.append(templates.create_popup_advanced);
		
		var options = this.__makeScheduledDate($body);
		options = this.__makeBackdatedTime($body) || options;
		
		if (!options) {
			$options.hide();
			return;
		}
		$options.find("input[type=checkbox]")
			.change(function() {
				var selected = $(this).prop("checked");
				$(this).parent()
					.toggleClass("disabled", !selected)
					.find("input[type=text], select")
					.prop("disabled", !selected);
			})
			.change();
	};
	
	CreatePopup.prototype.__updateImagePreview = _.throttle(function() {
		var url = null;
		if ($("#batch_create_content_image_link").is(":checked")) {
			url = $("#batch_create_content_image_link_text").val();
		}
		else if ($("#batch_create_content_image_disk").is(":checked")) {
			url = this.__uploadedImage;
		}
		$("#batch_create_content_image_preview").css("background-image", url? 
			format("url(\"{0}\")", url) : "none");
	}, 100);
	
	CreatePopup.prototype.__makeImageUploader = function($uploader) {
		var self = this;
		$uploader.parent().click(function() {
			if (self.processing) {
				return;
			}
			$uploader[0].click();
		});
		$uploader.change(function( event ) {
			if (self.processing) {
				return;
			}
			var file = event.target.files[0];
			if ( !file ) {
				return;
			}
			var reader = new FileReader();
			reader.onload = function (e) {
				self.__uploadedImage = e.target.result;
				self.__updateImagePreview();
            };
			reader.readAsDataURL(file);
		});
	};
	
	CreatePopup.prototype.__makeId = function($body) {
		var self = this;
		var $idContainer = $body.find("[batch_id_container]");
		if (!self.HasId()) {
			$idContainer.remove();
			return;
		}
	};
	
	CreatePopup.prototype.__makeContent = function($body) {
		var self = this;
		var $content = $body.find("[content_container]");
		var content = this.__getAvailableContentTypes();
		if (!content) {
			$content.hide();
			return;
		}
		$content.append(templates.create_popup_content);
		//link
		if (!content.link) {
			$content.find("[content=link]").remove();
		}
		//image
		if (content.image) {
			if (content.image.link) {
				$content.find("[image_link_text]")
					.change(function() {
						self.__updateImagePreview();
					});
			}
			else {
				$content.find("[image_content=link]").remove();
			}
			if (content.image.disk) {
				this.__makeImageUploader($content.find("#batch_create_content_image_disk_upload"));
			}
			else {
				$content.find("[image_content=disk]").remove();
			}
			$content.find("[image_selector]")
				.click(function() {
					var $parent = $(this).parent();
					$parent
						.siblings("[image_content]")
						.find("[image_content_data]")
						.hide();
					$parent
						.find("[image_content_data]")
						.show();
					self.__updateImagePreview();
				})
				.first()
				.click();
		}
		else {
			$content.find("[content=image]").remove();
		}
		//selector
		$content.find("[content_selector]")
			.click(function() {
				$content
					.find("[content_data]")
					.hide()
					.filter(format("[content_data={0}]", $(this).attr("content_selector")))
					.show();
			})
			.first()
			.click();
	};
	
	CreatePopup.prototype.__makeAddressesUploader = function($uploader) {
		var self = this;
		$uploader.parent().click(function() {
			if (self.processing) {
				return;
			}
			$uploader[0].click();
		});
		$uploader.change(function( event ) {
			if (self.processing) {
				return;
			}
			var file = event.target.files[0];
			if ( !file ) {
				return;
			}
			$("#batch_create_addresses_filename").text(file.name);
			var reader = new FileReader();
			reader.onload = function (e) {
				self.__uploadedAddresses = e.target.result;
            };
			reader.readAsText(file);
		});
	};
	
	CreatePopup.prototype.__makeAddresses = function($body) {
		const $bodyContainer = $body.filter("#outgoingBatchesCreatePopupBodyContainer");
		const $content = $bodyContainer.find("[tab_id='address']");
		$content.append(templates.create_popup_addresses);
		$content.find("[address_source_selector]")
			.click(function() {
				const source = $(this).attr("address_source_selector");
				const $parent = $(this).closest("[batch_address_content]");
				$parent
					.find("[address_source_data]")
					.hide()
					.filter(format("[address_source_data='{0}']", source))
					.show();
			})
			.first()
			.click();
		$content.find("#batch_create_include_headers").change(function() {
			$("#batch_create_address_column_container, #batch_create_reporting_group_container, #batch_create_address_agent_data_container")
				.toggle($(this).is(":checked"));
		});
		// $body.find("[brackets_icon]")
		// 	.attr("title", "<%=_T('Can use {{ }} to send addresses customized messages.')%>")
		// 	.show();
		this.__makeAddressesUploader($content.find("#batch_create_addresses_file"));
		this.__makeAddressesAgentData($content.find("#batch_create_address_agent_data_container"));
	};
	
	CreatePopup.prototype.__makeAddressesAgentData = function($container) {
		$container.find("#batch_create_address_agent_data_add").click(function() {
			var $item = $(templates.address_agent_data);
			$item.find("[agent_data_delete]").click(function() {
				$(this)
					.parent().parent().parent()
					.remove();
				var $list = $("#batch_create_address_agent_data_list");
				$list.toggle($list.children().length > 1);
			});
			$("#batch_create_address_agent_data_list")
				.append($item)
				.show();
		});
	};
	
	CreatePopup.prototype.__getTitle = function() {
		return "<%=_T('Create Batch on')%>";
	};
	
	CreatePopup.prototype.__makeText = function($body) {
		var self = this;
		$body.find("[batch_text]")
			.change(function() {
				self.__onTextChange();
			});
	};
	
	CreatePopup.prototype.__showText = function() {
		this.__makeEmojiPicker();
		this.__toggleTemplateMessage(this.isTemplateMessageRequired());
		this.__onTextChange();
	};
	
	CreatePopup.prototype.__makeAfterIdContent = function() {
		if (!this.isTemplateMessageAvailable()) {
			return "";
		}
		return templates.create_popup_template_selector;
	};
	
	CreatePopup.prototype.isTemplateMessageAvailable = function() {
		var campaign = inConcert.app.getCampaign(this.account.campaign);
		var account = inConcert.app.getBatchesManager().GetAccountConfiguration(
			this.account.campaign, this.account.account);
		return campaign && campaign.hasAccountTemplateMessageOptions(account) || false;
	};
	
	CreatePopup.prototype.isTemplateMessageRequired = function() {
		return false;
	};
	
	CreatePopup.prototype.__makeTemplate = function($body) {
		var self = this;
		if (!self.isTemplateMessageAvailable()) {
			return;
		}
		$body.find("[batch_header_text]").after(templates.create_popup_template_header_text);
		$body.find("[batch_text]").after(templates.create_popup_template_text);
		
		$body.find("#batch_create_template_language").change(function() {
			self.__loadTemplateMessageText();
		});
			
		$body.find("#batch_create_template_name").change(function() {
			var groupId = $("#batch_create_template_group").val();
			var messageId = $(this).val();
			var message = (groupId && messageId)? self.templateMessageOptions[groupId].messages[messageId] : {};
			var languages_a = [];
			$.each(message.languages || {}, function(id, language) {
				languages_a.push({
					id : id,
					label : inConcertLanguages.getLanguageLabel(id),
					text : language.text
				});
			});
			languages_a.sort(function(a, b) {
				return a.label.localeCompare(b.label);
			});
			var $languages = [];
			$.each(languages_a, function(_, language) {
				var $language = $(format("<option value='{0}'>{1}</option>",
					language.id, language.label));
				$language.attr("title", language.text);
				$languages.push($language);
			});
			if ($languages.length === 0 || $languages.length > 1) {
				var $language = $("<option value='USER'><%=_T('User Language')%></option>");
				$language.attr("title", "<%=_T('User language is used to select which message send')%>");
				$languages.unshift($language);
			}
			$("#batch_create_template_language")
				.html("")
				.append($languages)
				.change()
				.parent()
					.toggle($languages.length > 1);
		});
			
		$body.find("#batch_create_template_group")
			.change(function() {
				var groupId = $(this).val();
				var messages = groupId && self.templateMessageOptions[groupId].messages || {};
				var messages_a = [];
				$.each(messages, function(_, message) {
					messages_a.push({
						id : message.id,
						name : message.name || message.id,
						text : self.getDefaultTemplateMessageLanguage(message).text
					});

				});
				messages_a.sort(function(a, b) {
					return a.name.localeCompare(b.name);
				});
				var $messages = [];
				$.each(messages_a, function(_, message) {
					var $message = $(format("<option value='{0}'>{1}</option>",
						message.id, message.name));
					$message.attr("title", message.text);
					$messages.push($message);
				});
				$("#batch_create_template_name")
					.html("")
					.append($messages)
					.change();
			});
	};
	
	var defaultTexts = [
		"en",
		"en_US"
	];
	CreatePopup.prototype.getDefaultTemplateMessageLanguage = function(message) {
		if (message.defaultText) {
			return { text : message.defaultText };
		}
		var defaultLanguage = null;
		$.each(defaultTexts, function(_, id) {
			if (message.languages[id]) {
				defaultLanguage = message.languages[id];
				return false;
			}
		});
		if (!defaultLanguage) {
			var keys = Object.keys(message.languages);
			keys.sort();
			defaultLanguage = message.languages[keys[0]];
		}
		return defaultLanguage;
	};
	
	CreatePopup.prototype.__loadTemplateMessageText = function() {
		var self = this;
		var groupId = $("#batch_create_template_group").val();
		var messageId = $("#batch_create_template_name").val();
		var message = (groupId && messageId)? self.templateMessageOptions[groupId].messages[messageId] : null;
		if (!message) {
			return;
		}
		var languageId = $("#batch_create_template_language").val();
		let language = languageId === "USER"? self.getDefaultTemplateMessageLanguage(message) :
			message.languages[languageId];
		if (!language) {
			return;
		}

		//caso ICC_CLOUD_API: los buttons estan en los components
		languageId = languageId === "USER" ? inConcert.app.getLoggedUserLanguage() : languageId;
		let header_text;
		let example_header_text;
		message.hsm_buttons = [];
		for (const language in message.languages) {
			if (Object.hasOwn(message.languages, language)) {
				if(languageId === language && message.languages[language].components) {
					for (const component in message.languages[language].components) {
						if (Object.hasOwn(message.languages[language].components, component)) {
							if(message.languages[language].components[component]) {
								if(message.languages[language].components[component] && message.languages[language].components[component].type === "HEADER" && message.languages[language].components[component].format === "TEXT" && message.languages[language].components[component].header_param) {
									header_text = message.languages[language].components[component].text;
									example_header_text = message.languages[language].components[component].header_param;
								}
								else if(message.languages[language].components[component] && message.languages[language].components[component].type === "BUTTONS") {
									for (let i = 0; i < message.languages[language].components[component].buttons.length; i++) {										
										let button = {};									
										button.type = message.languages[language].components[component].buttons[i].type === "QUICK_REPLY" ? "quick_reply" : "call_to_action";
										if(button.type === "call_to_action" && message.languages[language].components[component].buttons[i].type === "URL" && message.languages[language].urlButtonParam) {
											button.title = message.languages[language].components[component].buttons[i].text;
											button.url = message.languages[language].components[component].buttons[i].url;
											button.urlButtonParam = true;
											button.otp_type = message.languages[language].components[component].buttons[i].otp_type;
										}
										button.payload = message.languages[language].components[component].buttons[i].text;
										message.hsm_buttons.push(button);
									}
								}
							}
						}
					}
				}
			}
		}

		this.textTemplate = new inConcert.TextTemplate({
			header_control : $("#batch_create_template_header_text_container"),
			header_text : header_text,
			example_header_text : example_header_text,
			control : $("#batch_create_template_text_container"), 
			content : language.text,
			placeholders : language.params || message.params,
			messages : {
				"required" : "<%=_T('Item value is required')%>",
				"resource" : "<%=_T('Resource must be a valid URL resource')%>"
			},
			button_control: $("#batch_create_template_message_button"),
			hsm_buttons: message.hsm_buttons
		});
	};
	
	CreatePopup.prototype.__toggleTemplateMessage = function(enabled) {
		var self = this;
		if (!enabled || self.templateMessagesLoaded) {
			self.templateMode = enabled;
			$("#outgoingBatchesCreatePopup").toggleClass("withTemplate", enabled);
			return;
		}
		self.__loadTemplateMessages(function(status, err) {
			if (!status) {
				self.Message(err, "error", 5000);
				return;
			}
			if (!self.templateMessageOptions_a.length) {
				self.Message("<%=_T('Has not configured template messages')%>", "error", 5000);
				return;
			}
			var $groups = [];
			$.each(self.templateMessageOptions_a, function(_, group) {
				$groups.push($(format("<option value='{0}'>{1}</option>",
					group.groupId, group.groupName)));
			});
			$("#batch_create_template_group")
				.append($groups)
				.change();
			if ($groups.length < 2) {
				$("#batch_create_template_group").parent().hide();
			}
			$("#outgoingBatchesCreatePopup").addClass("withTemplate");
			self.templateMode = true;
			self.templateMessagesLoaded = true;
		});
	};
	
	CreatePopup.prototype.__loadTemplateMessages = function(callback, initial) {
		var self = this;
		if (initial && !self.isTemplateMessageRequired()) {
			return callback(true);
		}
		if (self.templateMessageOptions) {
			return callback(true);
		}
		var campaign = inConcert.app.getCampaign(self.account.campaign);
		campaign.loadTemplateMessageOptions(self.account.account, self.account.page, true, 
			function(options) {
				if (!options) {
					return callback(false, "<%=_T('Error loading template messages')%>");
				}
				self.templateMessageOptions = {};
				self.templateMessageOptions_a = [];
				$.each(options, function(_, option) {
					if (!option.available) {
						return;
					}
					var group;
					if (self.templateMessageOptions[option.groupId]) {
						group = self.templateMessageOptions[option.groupId];
					}
					else {
						group = {
							groupId : option.groupId,
							groupName : option.groupName || option.groupId,
							messages : {}
						};
						self.templateMessageOptions[option.groupId] = group;
						self.templateMessageOptions_a.push(group);
					}
					group.messages[option.id] = option;
				});
				self.templateMessageOptions_a.sort(function(a, b) {
					return a.groupName.localeCompare(b.groupName);
				});
				callback(true);
			});
	};
	
	CreatePopup.prototype.__loadInitialData = function(callback) {
		var self = this;
		self.__loadTemplateMessages(function(_, err) {
			callback(err);
		}, true);
	};
	
	CreatePopup.prototype.__make = function() {
		var self = this;
		var manager = inConcert.app.getBatchesManager();
		var $body = $(Mustache.render(templates.create_popup, {
			title : self.__getTitle(),
			name : manager.GetAccountName(self.account.campaign, self.account.account, self.account.page),
			after_id : self.__makeAfterIdContent()
		}));
		// $body.find("#batchesCreatePopupButtonClose")
		// .off("click")
		// .on("click", function() {
		// 	self.Cancel(function() {
		// 		self.Close();
		// 	});
		// });
		//cancel
		$body.find("[cancel]").click(function() {
			self.Cancel(function() {
				self.Close();
			});
		});
		//update
		$body.find("[create]").click(function() {
			self.Create();
		});
		self.__makeId($body);
		self.__makeText($body);
		self.__makeTemplate($body);
		self.__makeContent($body);
		self.__makeAdvancedOptions($body);
		self.__makeAddresses($body);
		return $body;
	};
	
	CreatePopup.prototype.__makeTabs = function() {
		this.tabs = new inConcert.conf_form_tab({
			parent       : this.activity,
			container_id : "outgoingBatchesCreatePopup_tabs",
			tabs_list    : {
				addresses : {
					id               : "addresses",
					name             : "<%=_T('Addresses')%>",
					tab_container_id : "outgoingBatchesCreatePopup_tabs_addresses",
					initial_state    : "enabled",
					selected         : true,
				},
				message : {
					id               : "message",
					name             : "<%=_T('Message')%>",
					tab_container_id : "outgoingBatchesCreatePopup_tabs_message",
					initial_state    : "enabled"
				}
			}
		});
		this.tabs.show_tabs_container();
	};
	
	CreatePopup.prototype.__makeEmojiPicker = function() {
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: "#batchesCreatePopup_textarea",
			assetsPath: "/inconcert/lib/emoji-picker/lib/img/",
			popupButtonClasses: "buttonColorGray emoji-picker-button",
			norealTime: false,
			onPaste: function(pasted) {
				return pasted;
			}
		});
		window.emojiPicker.discover();
	};
	
	CreatePopup.prototype.__validateDatetime = function(date, limits) {
		return !isNaN(date.getTime()) &&
				(!limits.max || limits.max >= date) &&
				(!limits.min || limits.min <= date);
	};

	CreatePopup.prototype.__setTextDisclaimer = function(disclaimer) {
		$("#outgoingBatchesCreatePopup").find("[text_disclaimer]")
			.text(disclaimer || "")
			.toggle(!!disclaimer);
	};

	CreatePopup.prototype.__getText = function() {
		return $("#outgoingBatchesCreatePopup").find("[batch_text]").val() || "";
	};

	CreatePopup.prototype.__showTextLimit = function(/*length, limit*/) {
		if (this.templateMode) {
			$("#outgoingBatchesCreatePopup").find("[text_limit]").hide();
			return;
		}
		var length = this.__getText().length;
		var limit = this.__getTextLimit();
		if (!limit) {
			return;
		}
		
		var remaining = limit - length;
		$("#outgoingBatchesCreatePopup").find("[text_limit]")
			.toggleClass("neutral", remaining === 0)
			.toggleClass("positive", remaining > 0)
			.toggleClass("negative", remaining < 0)
			.text(Math.abs(remaining))
			.show();
		var text = this.__getAndValidateText();
		if (text.error && length !== 0) {
			this.Message(text, "error", 2000);
			$("#ConfirmButtonBatch").prop("disabled", true);
		}else {
			$("#ConfirmButtonBatch").prop("disabled", false);
		}
	};

	CreatePopup.prototype.__onTextChange = _.throttle(function() {
		this.__showTextLimit();
	}, 100);

	CreatePopup.prototype.Message = function(message, type, timeout) {
		var self = this;
		if (self.messageTimeout) {
			window.clearTimeout(self.messageTimeout);
			self.messageTimeout = null;
		}
		var text = message && (message.text || message.error) || message || "";
		var field = message && message.field || "";
		var $message = $("#outgoingBatchesCreatePopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
		if (field) {
			$("#" + field + "_validation").show();
		}
		if (timeout) {
			self.messageTimeout = window.setTimeout(function() {
				self.Message();
				self.messageTimeout = null;
			}, timeout);
		}
	};
	
	CreatePopup.prototype.Loading = function(loading) {
		$("#outgoingBatchesCreatePopup").find("[loading]").toggle(!!loading);
	};
	
	CreatePopup.prototype.__addId = function(data) {
		if (!this.HasId()) {
			return;
		}
		var id = $("#batch_create_id").val();
		if (!id) {
			return { text : "<%=_T('Must enter an id.')%>", field : "batch_create_id" };
		}
		if (!inConcert.Utils.validate(id, "id" ).success) {
			return { 
				text : "<%=_T('Only letters, numbers, dashes and dots are allowed in the id.')%>", 
				field : "batch_create_id" 
			};
		}
		data.id = id;
	};
	
	CreatePopup.prototype.__getAndValidateText = function() {
		var limit = this.__getTextLimit();
		var text = this.__getText();
		var length = text.length;
		if (!length) {
			return { error : "<%=_T('Must enter a message.')%>", field : "batch_create_text" };
		}
		if (limit && length > limit) {
			return { error : format("<%=_T('Message length must be less or equal than {0}.')%>", limit), field : "batch_create_text" };
		}
		return text;
	};
	
	CreatePopup.prototype.__addText = function(data) {
		if (this.templateMode) {
			return this.__addTemplate(data);
		}
		var text = this.__getAndValidateText();
		if (text.error) {
			return text;
		}
		data.text = text;
	};
	
	CreatePopup.prototype.__addTemplate = function(data) {
		var group = $("#batch_create_template_group").val();
		if (!group) {
			return { text : "<%=_T('Must select a template group.')%>", field : "batch_create_template_group" };
		}
		var name = $("#batch_create_template_name").val();
		if (!name) {
			return { text : "<%=_T('Must select a template name.')%>", field : "batch_create_template_name" };
		}
		var language = $("#batch_create_template_language").val();
		if (!language) {
			return { text : "<%=_T('Must select a template language.')%>", field : "batch_create_template_language" };
		}
		var textData = this.textTemplate.get(true);
		if (textData.error) {
			return { text : textData.error.message };
		}
		data.msgtype = "template";
		data.template = {
			group : group,
			id : name,
			language : language,
			header_value_param : textData.header_value_param,
			parameters : textData.values,
			text : textData.template_text
		};

		var message = (group && name)? this.templateMessageOptions[group].messages[name] : null;
		if (message && message.hsm_buttons)
			data.template.hsm_buttons = message.hsm_buttons;

		if (message && message.mediaUrl)
			data.template.mediaUrl = message.mediaUrl;
		
		data.text = textData.text;
	};
	
	CreatePopup.prototype.__addScheduleDate = function(data) {
		
	};
	
	CreatePopup.prototype.__addBackdatedTime = function(data) {
		
	};
	
	CreatePopup.prototype.__addAdvancedOptions = function(data) {
		return this.__addScheduleDate(data) ||
			   this.__addBackdatedTime(data);
	};
	
	CreatePopup.prototype.__addLinkToData = function(data, link, isImage) {
		
	};
	
	CreatePopup.prototype.__addLink = function(data) {
		if (!$("#batch_create_content_link").is(":checked")) {
			return;
		}
		var link = $("#batch_create_content_link_text").val();
		if (!link) {
			return "<%=_T('Must enter a link.')%>";
		}
		this.__addLinkToData(data, link);
	};
	
	CreatePopup.prototype.__addImageFromLink = function(data) {
		var link = $("#batch_create_content_image_link_text").val();
		if (!link) {
			return "<%=_T('Must enter a link.')%>";
		}
		this.__addLinkToData(data, link, true);
	};
	
	CreatePopup.prototype.__addImageFromDisk = function(data) {
		var files = $("#batch_create_content_image_disk_upload")[0].files;
		var file = files && files[0];
		if (!file) {
			return "<%=_T('Must select an image.')%>";
		}
		if ( file.size > this.__getMaxFileSizeMB() * 1024 * 1024 ) {
			return format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", this.__getMaxFileSizeMB());
		}
		data.file = file;
	};
	
	CreatePopup.prototype.__addImage = function(data) {
		if (!$("#batch_create_content_image").is(":checked")) {
			return;
		}
		if ($("#batch_create_content_image_link").is(":checked")) {
			return this.__addImageFromLink(data);
		}
		if ($("#batch_create_content_image_disk").is(":checked")) {
			return this.__addImageFromDisk(data);
		}
	};
	
	CreatePopup.prototype.__addContent = function(data) {
		return this.__addLink(data) ||
			   this.__addImage(data);
	};
	
	CreatePopup.prototype.__getAndValidateAddress = function(address_t) {
		return address_t;
	};
	
	CreatePopup.prototype.__getSeparator = function($separator) {
		var separator = $separator.val();
		switch(separator) {
			case "TAB":
				separator = "\t";
				break;
			case "LINE_BREAK":
				separator = "\n";
				break;
		}
		return separator;
	};
	
	CreatePopup.prototype.__getAddressHeaders = function(address_s, columnsSeparator, addressColumn, nameColumn) {
		var headers = address_s.split(columnsSeparator);
		var addressFound = false;
		var nameFound = false;
		$.each(headers, function(_, header) {
			if (addressColumn === header) {
				addressFound = true;
			}
			if (nameColumn && nameColumn === header) {
				nameFound = true;
			}
		});
		if (!addressFound) {
			return { __error : "<%=_T('Address column not found in headers.')%>" };
		}
		if (nameColumn && !nameFound) {
			return { __error : "<%=_T('Name column not found in headers.')%>" };
		}
		return headers;
	};
	
	CreatePopup.prototype.__getAddress = function(address_s, headers, columnsSeparator, addressColumn, nameColumn, reportingGroup, agentData) {
		if (!headers) {
			return { address : address_s };
		}
		var address_a = address_s.split(columnsSeparator);
		if (address_a.length !== headers.length) {
			return { __error : format("<%=_T('Address {0} has different columns number than headers.')%>", address_s) };
		}
		var address = {
			data : {
				data : {},
				reportingGroup : reportingGroup,
				agentData : agentData? {} : null
			}
		};
		$.each(address_a, function(index, value) {
			var name = headers[index];
			if (!name) {
				return;
			}
			if (name === addressColumn) {
				address.address = value;
			}
			else if (nameColumn && name === nameColumn) {
				address.data.name = value;
			}
			address.data.data[name] = value;
			if (agentData && agentData[name]) {
				address.data.agentData[agentData[name]] = value;
			}
		});
		if (!address.address) {
			return { __error : format("<%=_T('Address not found in {0}.')%>", address_s) };
		}
		return address;
	};
	
	CreatePopup.prototype.__addAddresses = function(data) {
		var self = this;
		var rowsSeparator = self.__getSeparator($("#batch_create_rows_separator"));
		var columnsSeparator = self.__getSeparator($("#batch_create_columns_separator"));
		var includeHeaders = $("#batch_create_include_headers").is(":checked");
		var addressColumn = includeHeaders? $("#batch_create_address_column").val() : null;
		var nameColumn = includeHeaders? $("#batch_create_name_column").val() : null;
		var reportingGroup = includeHeaders? $("#batch_create_reporting_group").val() : null;
		var agentData = null;
		var error = null;

		if (includeHeaders) {
			if (!addressColumn) {
				return "<%=_T('Must enter an address column name.')%>";
			}
			agentData = {};
			error = null;
			$("#batch_create_address_agent_data_list")
				.children("[agent_data_item]")
				.each(function() {
					var column = $(this).find("[agent_data_column]").val();
					if (!column) {
						return;
					}
					var name = $(this).find("[agent_data_name]").val();
					if (!name) {
						error = "<%=_T('Must enter an agent data item name.')%>";
						return false;
					}
					agentData[column] = name;
				});
			if (error) {
				return error;
			}
		}
		
		var text;
		if ($("#batch_create_addresses_file_selector").prop("checked")) {
			text = self.__uploadedAddresses;
			if (!text) {
				return "<%=_T('Must upload an addresses file.')%>";
			}
		}
		else {
			text = $("#batch_create_addresses_text").val();
			if (!text) {
				return "<%=_T('Must enter an addresses list.')%>";
			}
		}
		text = text.trim();
		var addresses_s = text.split(rowsSeparator);
		var addresses_d = {};
		var addresses = [];
		error = null;
		var headers = null;
		$.each(addresses_s, function(index, address_s) {
			address_s = address_s.trim();
			if (includeHeaders && index === 0) {
				headers = self.__getAddressHeaders(address_s, columnsSeparator, addressColumn, nameColumn);
				if (headers.__error) {
					error = headers.__error;
					return false;
				}
				return;
			}
			if (!address_s) {
				return;
			}
			var address = self.__getAddress(address_s, headers, columnsSeparator, addressColumn, nameColumn, reportingGroup, agentData);
			if (address) {
				if (address.__error) {
					error = address.__error;
					return false;
				}
				address.address = self.__getAndValidateAddress(address.address);
				if (!address.address) {
					error = format("<%=_T('Invalid address: {0}.')%>", address_s);
					return false;
				}
				if (addresses_d[address.address]) {
					error = format("<%=_T('Duplicated address: {0}.')%>", address.address);
					return false;
				}
				addresses_d[address.address] = true;
				addresses.push(address);
			}
			else {
				error = format("<%=_T('Invalid address: {0}.')%>", address_s);
				return false;
			}
		});
		if (error) {
			return error;
		}
		if (!addresses.length) {
			return "<%=_T('Must enter an addresses list.')%>";
		}
		data.addresses = addresses;
	};
	
	CreatePopup.prototype.__getData = function() {
		var data = {};
		var errorTab = "message";
		var error = this.__addId(data) ||
					this.__addText(data) ||
					this.__addContent(data) ||
					this.__addAdvancedOptions(data);
		if (!error) {
			error = this.__addAddresses(data);
			errorTab = "addresses";
		}
			
		if (error) {
			this.tabs.switch_tab(errorTab);
			this.Message(error, "error", 5000);
			return null;
		}
		return data;
	};
	
	CreatePopup.prototype.Create = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		$("#outgoingBatchesCreatePopup").find(".formFieldValidationError").hide();
		var data = self.__getData();
		if (!data) {
			return;
		}
		self.Message();
		$("#outgoingBatchesCreatePopup").find("button").prop("disabled", true);
		self.processing = true;
		self.Loading(true);
		
		var onEnd = function() {
			self.processing = false;
			self.Loading(false);
			$("#outgoingBatchesCreatePopup").find("button").prop("disabled", false);
		};
		
		self.__addDataAsync(data, function(status) {
			if (!status) {
				onEnd();
				return;
			}
			inConcert.app.getBatchesManager().CreateBatch(self.account, data, self.__isScheduled(data), function(status, message) {
				onEnd();
				if (status) {
					return self.Close();
				}
				self.Message(message, "error", 5000);
			});
		});
	};

}());