/*jshint quotmark:false*/
/*global $, inConcert, _, Mustache, EmojiPicker*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementEdit) {
	inConcert.batchManagementEdit = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var templates = inConcert.templates.batch_management;

	function EditPopup (batch)
	{
		this.batch = batch;
	}
	
	if (typeof inConcert.batchManagementEdit.base !== "function") {
		inConcert.batchManagementEdit.base = EditPopup;
	}
	
	EditPopup.prototype.__getClassName = function() {
		return "";
	};
	
	EditPopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	
	EditPopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#outgoingBatchesEditPopup").remove();
				callback();
			});
		}
		else {
			$("#outgoingBatchesEditPopup").remove();
			callback();
		}
	};
	
	EditPopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.edit_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.__makeEmojiPicker();
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "outgoingBatchesEditPopup",
			eventName     : "outgoingBatchesEditPopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#outgoingBatchesEditPopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	EditPopup.prototype.__make = function() {
		var self = this;
		var manager = inConcert.app.getBatchesManager();
		var $body = $(Mustache.render(templates.edit_popup, {
			name : manager.GetAccountName(self.batch.campaign, self.batch.account, self.batch.page)
		}));
		//text
		$body.find("[batch_text]")
			.val(self.__getOriginalText())
			.change(function() {
				self.__onTextChange();
			});
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//update
		$body.find("[update]").click(function() {
			self.Update();
		});
		
		return $body;
	};
	
	EditPopup.prototype.__makeEmojiPicker = function() {
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: "#outgoingBatchesEditPopup_textarea",
			assetsPath: "/inconcert/lib/emoji-picker/lib/img/",
			popupButtonClasses: "buttonColorGray emoji-picker-button",
			norealTime: false,
			onPaste: function(pasted) {
				return pasted;
			}
		});
		window.emojiPicker.discover();
	};
	
	EditPopup.prototype.__getText = function() {
		return $("#outgoingBatchesEditPopup").find("[batch_text]").val();
	};
	
	EditPopup.prototype.__getOriginalText = function() {
		return this.batch.GetText();
	};
	
	EditPopup.prototype.__onTextChange = _.throttle(function() {
		var text = this.__getText();
		$("#outgoingBatchesEditPopup").find("[update]").prop("disabled", !text || text === this.__getOriginalText());
	}, 100);
	
	EditPopup.prototype.Message = function(text, type) {
		var $message = $("#outgoingBatchesEditPopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	EditPopup.prototype.Loading = function(loading) {
		$("#outgoingBatchesEditPopup").find("[loading]").toggle(!!loading);
	};
	
	EditPopup.prototype.__makeData = function() {
		return {};
	};
	
	EditPopup.prototype.Update = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		self.Message();
		self.processing = true;
		self.Loading(true);
		$("#outgoingBatchesEditPopup").find("button").prop("disabled", true);
		
		var data = { 
			batch : self.__makeData()
		};
		
		self.batch.RunAction("EDIT", function(status, message) {
			self.processing = false;
			self.Loading(false);
			$("#outgoingBatchesEditPopup").find("button").prop("disabled", false);
			if (status) {
				return self.Close();
			}
			self.Message(message, "error");
		}, data, true);
	};

}());