/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.batchManagementList) {
	inConcert.batchManagementList = {};
}
(function () {
/*jshint validthis:true*/
"use strict";
	let format = inConcert.Utils.format;
	let templates = inConcert.templates.batch_management;

	// Defines a generic object with its common behaviour()
	// constructor
	function BatchesList ()
	{
		const self = this;
		this.columns = {
			type : {
				header	: { type : "text",	className : "columnType",			label : "<%=_T('Type')%>"			},
				row		: { type : "icon",	className : "columnType",			iconClass : "interactionType"		}
			},
			account_name : {
				header	: { type : "text",	className : "columnAccount",		label : "<%=_T('Account/Page')%>"	},
				row		: { type : "text",	className : "columnAccount",		textClass : "interactionAccount"	},
				value : function(batch) {
					let account = self.accounts[batch.campaign + ":" + batch.account + ":" + (batch.page || "")];
					if (account) {
						return account.name;
					}
					return batch.GetAccountName();
				}
			},
			campaign : {
				header	: { type : "text",	className : "columnCampaign",		label : "<%=_T('Campaign')%>"		},
				row		: { type : "text",	className : "columnCampaign",		textClass : "interactionGroup"		},
				value : function(batch) {
					return inConcert.Utils.getIdWithoutVCC(batch.campaign);
				}
			},
			id : {
				header	: { type : "text",	className : "columnId",		label : "<%=_T('Id')%>"		},
				row		: { type : "abstract", className : "columnId",	textClass : "interactionId"	},
				value : function(batch) {
					return batch.GetIdString();
				}
			},
			state : {
				header	: { type : "text",	className : "columnState",		label : "<%=_T('State')%>"		},
				row		: { type : "abstract", className : "columnState",	textClass : "interactionId"	},
				value : function(batch) {
					return batch.translatedState || batch.state;
				}
			},
			content : {
				header	: { type : "text",	className : "columnAbstract",		label : "<%=_T('Content')%>"		},
				row		: { type : "abstract", className : "columnAbstract",	textClass : "interactionAbstract"	},
				value : function(batch) {
					return batch.GetText();
				}
			},
			attributes : {
				header	: { type : "text",	className : "columnAttributes",		label : "<%=_T('Attributes')%>"		},
				row		: { type : "abstract", className : "columnAttributes",	textClass : "interactionAttributes"	},
				value : function(batch) {
					return batch.GetAttributes();
				}
			},
			publish_timestamp : {
				header	: { type : "text",	className : "columnDate",			label : "<%=_T('Publish Date')%>"	},
				row		: { type : "date",	className : "creationDate",			textClass : "interactionDate"		},
				sort	: true
			},
			last_message_timestamp : {
				header	: { type : "text",	className : "columnDate",			label : "<%=_T('Last Message')%>"	},
				row		: { type : "date",	className : "stateDate",			textClass : "interactionDate"		},
				sort	: true
			}
		};
		this.columns_order = [
			"type",
			"account_name",
			"campaign",
			"id",
			"state",
			"content",
			"attributes",
			"publish_timestamp",
			"last_message_timestamp"
		];
		this.default_sort = { column : "publish_timestamp", asc : false };
		this.batch_actions = [];
		this.current_batch_actions = [];
		this.accounts = {};
		this.query = {};
		this.sort = null;
		this.current_batches = {};
		this.MAX_ALLOWED_ROWS_IN_ES = inConcert.app.getServerInfo().maxAllowedRowsInES; // Este valor se obtiene del LUA "/apps/agent/shared/utils.lua", funcion "GetMaxAllowedRowsInES"
		this.rowsPerPage = 20;
		this.rowsPerPage = this.rowsPerPage > this.MAX_ALLOWED_ROWS_IN_ES ? this.MAX_ALLOWED_ROWS_IN_ES : this.rowsPerPage;
		this.paginationId = "outgoingBatchesPagination";
	}
	
	if (typeof inConcert.batchManagementList.base !== "function") {
		inConcert.batchManagementList.base = BatchesList;
	}

	BatchesList.prototype.__getPermission = function() {
		throw "Abstract method";
	};
	
	BatchesList.prototype.GetView = function() {
		throw "Abstract method";
	};

	BatchesList.prototype.RetrieveType = function() {
		throw "Abstract method";
	};

	BatchesList.prototype.GetName = function() {
		throw "Abstract method";
	};
	
	BatchesList.prototype.GetDescription = function() {
		throw "Abstract method";
	};
	
	BatchesList.prototype.GetClassName = function() {
		throw "Abstract method";
	};
	
	BatchesList.prototype.__batchStateValid = function(batch) {
		const tabSelectedId = inConcert.batch_management.getSelectedTabId();
		if (tabSelectedId === "ACTIVE") {
			return batch.IsActive() || batch.IsPaused() || batch.IsProcessing();
		}
		else if (tabSelectedId === "ARCHIVE") {
			return batch.IsClosed() || batch.IsDeleted();
		}
		throw(`BatchesList.prototype.__batchStateValid - unrecognized category '${tabSelectedId}'`);
	};

	BatchesList.prototype.GetRowsPerPage = function() {
		return this.rowsPerPage;
	};

	BatchesList.prototype.HasPermission = function(campaign) {
		return inConcert.app.getBatchesManager().HasPermission(campaign, this.__getPermission());
	};

	BatchesList.prototype.HasDisabledBatchActions = function() {
		return false;
	};

	BatchesList.prototype.HasBatchActions = function() {
		return this.current_batch_actions.length > 0;
	};
	
	BatchesList.prototype.HasActions = function() {
		return true;
	};

	BatchesList.prototype.HasPagination = function() {
		return true;
	};

	BatchesList.prototype.__loadAccount = function(accountData) {
		const self = this;
		const $account = $(Mustache.render(templates.menu_account_item, accountData));
		$account.click(function() {
			if (self.IsProcessing()) {
				return;
			}
			const $this = $(this);
			$this
				.addClass("selected")
				.siblings()
				.removeClass("selected");
			self.selectedAccount = {
				campaign: $this.attr("campaign"),
				account : $this.attr("account"),
				page    : $this.attr("page") || null,
				refresh : true
			};
			self.selectedBatch = null;
			self.Search();
		});
		return $account;
	};

	BatchesList.prototype.__getAvailableChannels = function() {
		return null;
	};

	BatchesList.prototype.__getViewAccounts = function() {
		let list = inConcert.app.getBatchesManager().GetAccounts(true, this.__getPermission());
		let channels = this.__getAvailableChannels();
		if (channels) {
			let original = list;
			list = [];
			$.each(original, function(_, account) {
				if (channels[account.type]) {
					list.push(account);
				}
			});
		}
		return list;
	};

	BatchesList.prototype.__loadAccounts = function() {
		let self = this;
		let accounts = this.__getViewAccounts();
		self.accounts = {};
		let $accounts = [];
		$.each(accounts, function(_, account) {
			self.accounts[account.campaign + ":" + account.account + ":" + account.page] = account;
			$accounts.push(self.__loadAccount(account));
		});
		$("#outgoingBatchMenu")
			.find(format("[category={0}]", self.GetView()))
			.find("[accountsList]")
			.html("")
			.append($accounts);
	};
	
	BatchesList.prototype.__movePage = function(page) {
		if (this.IsProcessing()) {
			return;
		}
		this.page = page;
		this.Search(null, true);
	};
	
	BatchesList.prototype.__loadPagination = function() {
		const self = this;
		this.page = 1;
		const $pagination = $(templates.pagination());
		$pagination
			.find("[first]")
			.click(function() {
				self.__movePage(1);
			});
		$pagination
			.find("[prev]")
			.click(function() {
				self.__movePage(self.page - 1);
			});
		$pagination
			.find("[next]")
			.click(function() {
				self.__movePage(self.page + 1);
			});
		$pagination
			.find("[last]")
			.click(function() {
				let page;
				let rowsPerPage = self.GetRowsPerPage();
				if (self.total > self.MAX_ALLOWED_ROWS_IN_ES) {
					// Si ha llegado al maximo de rows permitidos por ES, me quedo con la pagina previa
					page = self.MAX_ALLOWED_ROWS_IN_ES / rowsPerPage;
				}
				else {
					page = self.total / rowsPerPage;
				}
				page = Math.floor(page) < page ? Math.floor(page) + 1 : page;
				self.__movePage(page);
			});
		return $pagination;
	};

	BatchesList.prototype.__updatePagination = function() {
		let rowsPerPage = this.GetRowsPerPage();
		let from = (this.page - 1) * rowsPerPage + 1;
		let to = Math.min(from + rowsPerPage - 1, this.total);

		let disableMorePages = false;
		if (this.total  === 0 || this.total  <= rowsPerPage) {
			// Si no tiene registros o el total de registros es menor o igual a la cantidad de registros por pagina, desactivo la navegacion para obtener mas Rows.
			disableMorePages = true;
		}
		else if ((this.page * rowsPerPage) >= this.MAX_ALLOWED_ROWS_IN_ES) {
			// Si llego al maximo de Rows permitidas desactivo la navegacion para obtener mas Rows.
			disableMorePages = true;
		}
		else {
			let lastPage =this.total / rowsPerPage;
			lastPage = Math.floor(lastPage) < lastPage ? Math.floor(lastPage) + 1 : lastPage;
			if (this.page === lastPage) {
				// Si estoy en la ultima pagina desactivo la navegacion para obtener mas Rows.
				disableMorePages = true;
			}
		}

		let $pagination = $("#" + this.paginationId);
		$pagination
			.find("[pagination_label]")
			.html(format("<%=_T('{0} - {1} of {2}')%>", from, to, this.total));
		$pagination
			.find("[first]")
			.prop("disabled", from === 1);
		$pagination
			.find("[prev]")
			.prop("disabled", from === 1);
		$pagination
			.find("[next]")
			.prop("disabled", disableMorePages);
		$pagination
			.find("[last]")
			.prop("disabled", disableMorePages);
	};
	
	BatchesList.prototype.__canRunAction = function(action) {
		if (!action) {
			return false;
		}
		if (this.query.campaign) {
			return inConcert.permissions.to_action("batch_management", action.permission, null, this.query.campaign);
		}
		return inConcert.permissions.to_action_on_any_campaign("batch_management", action.permission);
	};

	BatchesList.prototype.__updateBatchActions = function() {
		const $rows = $("#outgoingBatchesList").find("input[row_selector]:checked");
		const $actions = $("#outgoingBatchesActions_actions_menu_options").find("a[action]");
		this.__createButtonActions();
		$.each(this.current_batch_actions, function(_, action) {
			const hide = $rows.length === 0 || $rows.filter(format("[{0}=false]", action)).length > 0;
			$actions
				.filter(format("[action={0}]", action))[hide?"hide":"show"]();
		});
	};

	/*
		(*) Las acciones se tienen que ocultar ya que las mismas se encuentran en el menu contextual de acciones por batch.
 			Por el funcionamiento del codigo el boton $action tiene que existir en el DOM.
	*/
	BatchesList.prototype.__loadBatchAction = function(actionName) {
		const self = this;
		const action = inConcert.batchManagementActions.all[actionName];
		if (!this.__canRunAction(action)) {
			return null;
		}
		this.current_batch_actions.push(actionName);
		const $action = $(Mustache.render(templates.batch_action_button, {
			action : actionName,
			name : action.name,
			description : action.description,
			iconClassName : actionName.toLowerCase() + "Batch"
		}));
		$action.hide(); // (*) ver comentario de la funcion
		$action.click(function() {
			self.__runBatchAction($(this).attr("action"));
		});
		return $action;
	};

	BatchesList.prototype.__loadBatchActions = function () {
		const self = this;
		let $actions = [];
		this.current_batch_actions = [];
		if (!this.HasDisabledBatchActions()) {
			$.each(this.batch_actions, function (_, action) {
				$actions.push(self.__loadBatchAction(action));
			});
		}
		const $pagination = $("#outgoingBatchesPagination");
		$pagination.html("").hide();
		if (this.HasPagination()) {
			$pagination.append(this.__loadPagination());
			$pagination.show();
		}
		else {
			this.pagination = null;
		}
		$("#outgoingBatchesActions_actions_menu_options")
			.html("")
			.append($actions);
	};

	BatchesList.prototype.__runBatchAction = function(actionName) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		self.processing = true;
		self.__waiting(true);
		let batches = [];
		$("#outgoingBatchesList")
			.find("input[row_selector]:checked")
			.each(function() {
				batches.push(self.current_batches[$(this).attr("batch_id")]);
			});
		inConcert.app.getBatchesManager().RunBatchAction(batches, actionName, function(status, result) {
			self.processing = false;
			self.__waiting(false);
			if (status) {
				if (result !== "ABORTED" && result.unsuccessful_nodes.length) {
					inConcert.app.m_mainWindow.showBannerMessage({
						message: "<%=_T('Action failed for some batches.')%>", messageType: "Error", timeout: 5000 
					});
				}
			}
			else {
				inConcert.app.m_mainWindow.showBannerMessage({ message: result, messageType: "Error", timeout: 5000 });
			}
		}, null, true);
	};

	BatchesList.prototype.destroy_action_menu = function() {
		const self = this;
		if (!this.m_action_menu) {
			return;
		}
		this.m_action_menu.finalize(function() {
			self.m_action_menu = null;
		});
	};

	BatchesList.prototype.toggle_action_menu = function($parent) {
		const self = this;
		let isPaused = true;
		let isReactivate = true;
		
		let custom_menu_options = [
			{
				id     : "pause",
				name   : "<%=_T('Pause')%>",
				execute: function() {
					self.__runBatchAction("PAUSE");
				}
			},
			{
				id     : "resume",
				name   : "<%=_T('Resume')%>",
				execute: function() {
					self.__runBatchAction("RESUME");
				}
			},
			{
				id     : "set attributes",
				name   : "<%=_T('Set Attributes')%>",
				execute: function() {
					self.__runBatchAction("SET_ATTRIBUTES");
				}
			},
			{
				id     : "close",
				name   : "<%=_T('Close')%>",
				execute: function() {
					
					self.__runBatchAction("CLOSE");
				}
			}
		];
		if (isPaused) {
			custom_menu_options = custom_menu_options.filter(option => option.id !== 'pause');
		}
		if(isReactivate) {
			custom_menu_options = custom_menu_options.filter(option => option.id !== 'resume');
		}
		if (this.m_action_menu && this.m_action_menu.isVisible()) {
			this.destroy_action_menu();
		}
		else {
			this.show_action_menu($parent, custom_menu_options);
		}
	};

	BatchesList.prototype.can_show_action_menu = function(can_show_action_menu) {
		can_show_action_menu = can_show_action_menu || function() {return true;};
		return can_show_action_menu();
	};

	BatchesList.prototype.show_action_menu = function($parent, menu_options, can_show_action_menu, callback = $.noop) {
		const self = this;

		if (!this.can_show_action_menu(can_show_action_menu)) {
			callback();
			return;
		}
		let $rows = $("#outgoingBatchesList").find("input[row_selector]:checked");
		if ($rows.length == 0) {
			$("#outgoingBatchesContentMenuAction").hide();
			this.destroy_action_menu();
		}else{
			const menu_id =  "outgoingBatchesContentMenuAction";
			$('#' + menu_id).remove();
			const $menu = $(''.concat(
				'<div id="' + menu_id + '" class="interactionMenuBlind snackBarActionsDropup" hidden></div>'
			));

			$.each(menu_options, function(_, option) {
				const $option = $("".concat(
					"<div option='" + option.id + "' class='interactionMenuItem '>",
						option.name,
					"</div>"
				));
				$option.click(function() {
					window.setTimeout(function() {
						option.execute($menu);
						self.destroy_action_menu();
					}, 50);
				});
				$menu.append($option);
			});

			const fnPosition = function() {
				const $btnAction = $parent;
				const btnActionOffset = $btnAction.offset();
				const dropdownLeft = btnActionOffset.left - ($menu.outerWidth() - $btnAction.outerWidth());
				$menu.css({left: dropdownLeft });
			};

			$('body').append($menu);
			this.m_action_menu = new inConcert.PopupWindow({ownerId: menu_id});
			this.m_action_menu.show(
			{
				ownerId       : menu_id,
				eventName     : menu_id,
				canReposition : false,
				container     : $menu,
				parentNode    : $parent,
				callbacks: {
					onOpen: function() {
						self.m_action_menu.setPosition();
						fnPosition();
					},
				}
	//			align         : "LEFT"
			});
			callback();
			const $outgoingBatchesStats = $("#outgoingBatchesStats");
			$(window).on('resize', function() {
				if ($menu.is(':visible') && self.m_action_menu && $outgoingBatchesStats.is(":hidden")) {
						self.m_action_menu.setPosition();
						fnPosition();
				}
			});
		}
	};

	BatchesList.prototype.__createButtonActions = function() {
		const self = this;
		const tabSelectedId = inConcert.batch_management.getSelectedTabId();
		const $outgoingBatchesActions = $("#outgoingBatchesActions");
		$outgoingBatchesActions.hide();
		if (tabSelectedId === "ACTIVE") {
			const $rows = $("#outgoingBatchesList").find("input[row_selector]:checked");
			const $rowsnumber = $("#outgoingBatchesActions_selected_number");
			$rowsnumber.html($rows.length);
			if ($rows.length > 0) {
				$outgoingBatchesActions.show();
				$("#outgoingBatchesActions_actions_menu")
				.off("click")
				.on("click", function() {
					const $this = $(this);
					self.toggle_action_menu($this);
				});

				$("#outgoingBatchesActions_actions_cancel")
				.off("click")
				.on("click", function() {
					$("#batchManagementBatchSelect").prop("checked", true).click();
				});
			}
		}
	};
	
	BatchesList.prototype.__loadHeaderColumn = function(name, header, sort) {
		const self = this;
		let content = "";
		switch(header.type) {
			case "text":
				content = Mustache.render(templates.list_header_column_text, { label : header.label });
				break;
			case "icon":
				content = templates.list_header_column_icon;
				break;
			case "selector-all":
				content = templates.batch_action_select_all;
				break;
		}
		const className = header.className;
		const $column = $(Mustache.render(templates.list_header_column, { name : name, className : className, content : content }));
		if (sort) {
			$column
				.addClass("columnSorteable")
				.click(function() {
					if (self.IsProcessing()) {
						return;
					}
					self.__setSort(name, !$(this).hasClass("columnSortAsc"));
					self.Search();
				});
		}
		if (header.type === "selector-all") {
			const $selectAll = $column.find("[select-all='true']");
			$selectAll.change(function() {
				$("#outgoingBatchesList")
					.find("input[row_selector]")
					.prop("checked", $(this).prop("checked"));
				self.__updateBatchActions();
			});
		}
		return $column;
	};
	
	BatchesList.prototype.__loadHeaders = function() {
		const self = this;
		const $header = $("#outgoingBatchesListHeader");
		let $columns = [];
		if (this.HasBatchActions()) {
			$columns.push(this.__loadHeaderColumn("", { type : "selector-all", className : "columnBatch" }));
		}
		$.each(this.columns_order, function(_, columnName) {
			$columns.push(self.__loadHeaderColumn(columnName, self.columns[columnName].header, self.columns[columnName].sort));
		});
		if (this.HasActions()) {
			$columns.push(this.__loadHeaderColumn("actions", { type : "icon", className : "columnActions" }));
		}
		$header
			.html("")
			.append($columns);
	};

	BatchesList.prototype.__message = function(message) {
		if (message) {
			$("#outgoingBatchesMessages")
				.html(message)
				.show();
			// $('#outgoingBatchesListHeaderArea').hide();
			$('#outgoingBatchesList').hide();
			$('#outgoingBatchesBatchActions').hide();
			$('#outgoingBatchesMessagesInfo').hide();
		}
		else {
			$("#outgoingBatchesMessages")
				.html("")
				.hide();
			// $('#outgoingBatchesListHeaderArea').show();
			$('#outgoingBatchesList').show();
			$('#outgoingBatchesBatchActions').show();
			$('#outgoingBatchesMessagesInfo').show();
		}
	};

	BatchesList.prototype.__waiting = function(waiting) {
		$("#outgoingBatchesWaiting").toggle(!!waiting);
	};
	
	BatchesList.prototype.__clear = function() {
		$("#outgoingBatchesList").html("");
		$("#outgoingBatchesBatchSelect").prop("checked", false);
		this.__message();
	};
	
	BatchesList.prototype.__getActionsMenuId = function() {
		return "batch_management_list_menu_actions";
	};

	BatchesList.prototype.__runSingleBatchAction = function(batchId, actionName) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		self.processing = true;
		let $waiting = $("#outgoingBatchesList").find(format("div[batch_waiting][batch_id='{0}']", batchId));
		let batch = self.current_batches[batchId];
		$waiting.show();
		batch.RunAction(actionName, function(status, error) {
			$waiting.hide();
			self.processing = false;
			if (!status) {
				inConcert.app.m_mainWindow.showBannerMessage({ message: error, messageType: "Error", timeout: 5000 });
			}
		});
	};
	
	BatchesList.prototype.__loadSingleBatchAction = function(batchId, action) {
		let self = this;
		let $action = $(Mustache.render(templates.batch_actions_menu_item, { 
			id : batchId, 
			action : action.id,
			name : action.name,
			tooltip : action.description,
			className : action.id.toLowerCase() + "Batch"
		}));
		$action.click(function() {
			if (self.actionsMenu) {
				self.actionsMenu.finalize(function() {
					self.actionsMenu = null;
				});
			}
			self.__runSingleBatchAction($(this).attr("batch_id"), $(this).attr("action"));
		});
		return $action;
	};
	
	BatchesList.prototype.__loadSingleBatchActions = function(batchId) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		function show() {
			let $button = $("#outgoingBatchesList").find(format("input[batch_actions][batch_id='{0}']", batchId));
			let menuId = self.__getActionsMenuId();
			let batch = self.current_batches[batchId];
			let $menu = $(Mustache.render(templates.batch_actions_menu, { id : menuId }));
			$.each(batch.GetActions(), function(_, action) {
				$menu.append(self.__loadSingleBatchAction(batchId, action));
			});
			$("#" + menuId).remove();
			$("body").append($menu);
			self.actionsMenu = new inConcert.PopupWindow();
			self.actionsMenu.show({
				ownerId      : menuId,
				eventName    : menuId,
				canReposition: true,
				container    : $menu,
				parentNode   : $button,
				batchId      : batchId,
				callbacks: {
					onOpen: function() {
						$('#' + menuId).show();
					},
					onClose: function( callback = $.noop ) {
						if (self.actionsMenu) {
							self.actionsMenu.hide(function() {
								callback();
							});
						}
						else {
							callback();
						}
					}
				}
			});
		}
		if (self.actionsMenu && self.actionsMenu.isVisible()) {
			self.actionsMenu.finalize(function() {
				let data = self.actionsMenu.getData();
				self.actionsMenu = null;
				if ( data && data.batchId !== batchId ) {
					show();
				}
			});
		}
		else {
			show();
		}
	};
	
	BatchesList.prototype.__loadBatchColumn = function(name, column, batch) {
		let row = column.row;
		let value = column.value? column.value(batch) : batch[name];
		let content = "";
		let className = row.className;
		switch(column.row.type) {
			case "date":
				value = inConcert.Utils.formatDate(value * 1000);
				className = "columnDate " + className;
				content = Mustache.render(templates.list_column_text, { className : row.textClass, text : value });
				break;
			case "text":
				content = Mustache.render(templates.list_column_text, { className : row.textClass, text : value });
				break;
			case "abstract":
				content = Mustache.render(templates.list_column_abstract, { className : row.textClass, text : value });
				break;
			case "icon":
				content = templates.list_column_icon;
		}
		if (this.sort && this.sort.column === name) {
			className += " columnSorted";
		}
		return Mustache.render(templates.list_column, { name : name, className : className, content : content });
	};
	BatchesList.prototype.UpdateBatches = function() {
		let self = this;
		if ($.isEmptyObject(self.current_batches)) {
			return;
		}
		let batches = [];
		$.each(self.current_batches, function(_, batch) {
			batches.push(batch);
		});
		self.__clear();
		self.__loadBatches(batches, true);
		if (self.selectedBatch) {
			self.__selectStatistics(self.__makeBatchStatistics());
		}
		self.__updateBatchActions();
	};

	BatchesList.prototype.__loadBatch = function(batch, hasBatch, hasActions) {
		let self = this;
		let batchId = batch.GetId();
		self.current_batches[batchId] = batch;
		let $columns = [];
		if (hasBatch) {
			let $select = $(Mustache.render(templates.list_row_selector, { id : batchId }));
			$select.change(function() {
				$("#outgoingBatchesBatchSelect").prop("checked", false);
				self.__updateBatchActions();
			});
			$.each(self.current_batch_actions, function(_, action) {
				$select
					.find("input[row_selector]")
					.attr(action, batch.CanRunAction(action));
			});
			$columns.push($select);
		}
		$.each(this.columns_order, function(_, columnName) {
			$columns.push($(self.__loadBatchColumn(columnName, self.columns[columnName], batch)));
		});
		if (hasActions && batch.HasActions()) {
			let $actions = $(Mustache.render(templates.list_column_actions, { id : batchId }));
			$columns.push($actions);
			$actions.click(function() {
				self.__loadSingleBatchActions($(this).attr("batch_id"));
			});
		}
		let className = batchId === self.selectedBatch? "selected " : "";
		let iconTooltip = "";
		$.each(batch.GetNativeProperties(), function(_, property) {
			className += property.className? ((className? " " : "") + property.className + "Batch") : "";
			iconTooltip += property.tooltip? ((iconTooltip? " " : "") + property.tooltip) : "";
		});
		let $batch = $(Mustache.render(templates.list_row, { id : batchId, type : batch.type, className : className }));
		$batch.append($columns);
		$batch.click(function(e) {
			let $target = $(e.target);
			if ($target.attr("row_selector") || $target.attr("batch_actions")) {
				return;
			}
			$(this)
				.addClass("selected")
				.siblings()
				.removeClass("selected");
			self.selectedBatch = $(this).attr("batch_id");
			const $outgoingBatchesStats = $("#outgoingBatchesStats");
			if ( $outgoingBatchesStats.children().length > 0 && !$outgoingBatchesStats.is(":hidden")) {
				this.statistics = self.__makeBatchStatistics();
				this.statistics.Load();
			}else {
				self.__selectStatistics(self.__makeBatchStatistics());
			}
		});
		$batch.find("[column=type]").attr("title", iconTooltip);
		return $batch;
	};

	BatchesList.prototype.__loadBatches = function(batches, noMessage) {
		let self = this;
		self.current_batches = {};
		if (batches.length === 0) {
			if (!noMessage) {
				self.__message("<%=_T('No batches found')%>");
			}
			return;
		}
		let $container = $("#outgoingBatchesList");
		let elements = [];
		let hasBatch = self.HasBatchActions();
		let hasActions = self.HasActions();
		$.each(batches, function(_, batch) {
			if (self.__batchStateValid(batch)) {
				elements.push(self.__loadBatch(batch, hasBatch, hasActions));
			}
		});
		$container.append(elements);
		const uniqueRow = {};
		$container.contents().each(function() {
			const row = $(this).attr("batch_id");
			if (uniqueRow[row]) {
				$(this).remove();
			}
			else {
				uniqueRow[row] = true;
			}
		});
	};
	
	BatchesList.prototype.__loadSearch = function() {
		let self = this;
		$("#outgoingBatchesRefresh")
			.off("click")
			.on("click", function() {
				if (self.selectedAccount) {
					self.selectedAccount.refresh = true;
				}
				self.Search();
			});
		$("#outgoingBatchesSearch")
			.val("")
			.off("keypress")
			.on("keypress", function(e) {
				if(e.which === 13) {
					self.Search();
				}
			});
	};
	
	BatchesList.prototype.__setSort = function(column, asc) {
		let $headers = $("#outgoingBatchesListHeader").children();
		$headers
			.removeClass("columnSortAsc")
			.removeClass("columnSortDesc");
		if (column) {
			this.sort = { column : column, asc : asc };
			$headers
				.filter(format("[column={0}]", column))
				.addClass(asc? "columnSortAsc" : "columnSortDesc");
		}
		else {
			this.sort = null;
		}
	}; 
	
	BatchesList.prototype.IsProcessing = function() {
		return !!this.processing;
	};

	BatchesList.prototype.Search = function(callback, pagination, tabSelected) {
		callback = callback || $.noop;
		const self = this;
		if (this.IsProcessing()) {
			return callback();
		}
		this.processing = true;
		//sort
		let sort = null;
		if (this.sort) {
			let condition = {};
			condition[this.sort.column] = { order : this.sort.asc? "asc" : "desc" };
			sort = [condition];
		}

		let request = {
			view : this.RetrieveType(),
			query: this.query,
			sort : sort,
			rows : this.GetRowsPerPage()
		};
		// contenido
		request.query.content = $("#outgoingBatchesSearch").val() || null;
		this.query.content = request.query.content;

		const listItems = $("#outgoingBatchAccountList");
		const $itemSelected = listItems.find(".selected");

		if ($itemSelected.length > 0) {
			this.selectedAccount = {
				campaign: $itemSelected.attr("campaign"),
				account : $itemSelected.attr("account"),
				page    : $itemSelected.attr("page") || null,
				content : this.query.content,
				refresh : true
			};
			request.query = this.selectedAccount;
		}
		//selected account
		if (this.selectedAccount && !this.selectedBatch) {
			this.__addAccountConditions(request);
		}
		//pagination
		if (self.HasPagination() && pagination) {
			request.from = (this.page - 1) * request.rows;
		}
		else {
			this.page = 1;
			request.from = 0;
		}
		this.__clear();
		this.__waiting(true);
		inConcert.app.getBatchesManager().SearchBatches(request, function(data, error) {
			self.__waiting(false);
			self.processing = false;
			if (data) {
				self.__loadBatches(data.batches);
				self.total = data.total;
			}
			else {
				let message = error && error.reason || "<%=_T('Unexpected error searching batches')%>";
				self.__message(message);
				self.total = 0;
			}
			if (self.HasPagination()) {
				self.__updatePagination();
			}
			if (request.account_statistics && !tabSelected && $itemSelected.length > 0) {
				let statistics = data && data.account_statistics? self.__makeAccountStatistics(data.account_statistics) : null;
				self.__selectStatistics(statistics, undefined, true);
			}
			else {
				self.__selectStatistics(self.__makeBatchStatistics(), undefined, true);
			}
			self.__updateBatchActions();
			callback();
		});
	};

	BatchesList.prototype.__addAccountConditions = function(request) {
		request.query.campaign = this.selectedAccount.campaign || null;
		request.query.account = this.selectedAccount.account;
		request.query.page = this.selectedAccount.page || null;
		if (this.selectedAccount.refresh) {
			request.account_statistics = true;
			this.selectedAccount.refresh = false;
		}
	};
	
	BatchesList.prototype.SelectStatsSlice = function(slice, callback = $.noop) {
		if (this.statistics) {
			this.statistics.SelectSlice(slice, callback);
		}
		else {
			callback();
		}
	};

	BatchesList.prototype.__selectStatistics = function(statistics, beforeSearch, hideDetailPanel) {
		const $outgoingBatchesStats = $("#outgoingBatchesStats");
		const $buttonCreate = $("#outgoingBatchButtonCreate");
		const $main = $("#batchManagementMain");
		if (!beforeSearch) {
			$buttonCreate.show();
			const hasStatistics = !!statistics && !hideDetailPanel;
			$outgoingBatchesStats.toggle(hasStatistics);
			$main.toggleClass("splitedScenario", hasStatistics);
		}
		if (statistics) {
			this.statistics = statistics;
			this.statistics.Load();
		}
		else {
			this.statistics = null;
			$outgoingBatchesStats.html("");
			$buttonCreate.hide();
		}
	};

	BatchesList.prototype.__makeAccountStatistics = function(data) {
		let selected = this.selectedAccount;
		let account = this.accounts[selected.campaign + ":" + selected.account + ":" + (selected.page || "")];
		if (!account) {
			return null;
		}
		selected.statistics = data;
		return inConcert.app.getBatchesManager().BuildAccountStatistics({
			accountName	: account.name,
			type		: account.type,
			campaign	: account.campaign,
			account		: account.account,
			page		: account.page || "",
			statistics	: data
		});
	};

	BatchesList.prototype.__makeBatchStatistics = function() {
		let batch = this.current_batches[this.selectedBatch || ""];
		if (!batch) {
			this.selectedBatch = null;
			if (this.selectedAccount && this.selectedAccount.statistics) {
				return this.__makeAccountStatistics(this.selectedAccount.statistics);
			}
			return null;
		}
		let account = this.accounts[batch.campaign + ":" + batch.account + ":" + (batch.page || "")];
		return inConcert.app.getBatchesManager().BuildBatchStatistics({
			accountName : account.name,
			batch		: batch
		});
	};

	BatchesList.prototype.Load = function(callback, selectedAccount) {
		callback = callback || $.noop;
		this.__loadAccounts();
		this.__loadBatchActions();
		this.__loadHeaders();
		this.__loadSearch();
		this.query = {};
		this.selectedBatch = null;
		this.selectedAccount = selectedAccount || null;
		this.__selectStatistics(null, false);
		if (selectedAccount) {
			$("#outgoingBatchMenu").find("[accountslist]")
				.find(format("[campaign='{0}'][account='{1}'][page='{2}']",
					selectedAccount.campaign, selectedAccount.account, selectedAccount.page))
				.addClass("selected")
				.siblings()
				.removeClass("selected");
		}
		this.__setSort(this.default_sort.column, this.default_sort.asc);
		const tabSelected = true;
		this.Search(callback, null, tabSelected);
		this.__createButtonActions();
	};
}());