/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementReschedule) {
	inConcert.batchManagementReschedule = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var templates = inConcert.templates.batch_management;

	function ReschedulePopup (batch)
	{
		this.batch = batch;
	}
	
	if (typeof inConcert.batchManagementReschedule.base !== "function") {
		inConcert.batchManagementReschedule.base = ReschedulePopup;
	}
	
	ReschedulePopup.prototype.__getClassName = function() {
		return "";
	};
	
	ReschedulePopup.prototype.__getScheduleDateLimits = function() {
		return null;
	};
	
	ReschedulePopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	
	ReschedulePopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#outgoingBatchesReschedulePopup").remove();
				callback();
			});
		}
		else {
			$("#outgoingBatchesReschedulePopup").remove();
			callback();
		}
	};
	
	ReschedulePopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.reschedule_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "outgoingBatchesReschedulePopup",
			eventName     : "outgoingBatchesReschedulePopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#outgoingBatchesReschedulePopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	ReschedulePopup.prototype.__make = function() {
		var self = this;
		var manager = inConcert.app.getBatchesManager();
		var $body = $(Mustache.render(templates.reschedule_popup, {
			name : manager.GetAccountName(self.batch.campaign, self.batch.account, self.batch.page)
		}));
		//control
		self.__makeScheduledDate($body);
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//update
		$body.find("[update]").click(function() {
			self.Update();
		});
		
		return $body;
	};
	
	ReschedulePopup.prototype.__makeScheduledDate = function($body) {
		var $scheduled = $body.filter("[schedule_date]");
		this.__makeDatetime($scheduled, this.__getScheduleDateLimits());
		return true;
	};
	
	ReschedulePopup.prototype.__makeDatetime = function($section, limits) {
		limits = limits || {};
		limits["default"] = limits["default"] || new Date();
		
		$section.find("[date]")
			.datepicker({
				dateFormat: "yy-mm-dd",
				minDate: limits.min, 
				maxDate: limits.max
			})
			.datepicker("setDate", limits["default"]);
			
		$section.find("[hour]")
			.timeEntry({
				spinnerImage : "",
				show24Hours : true
			})
			.val(inConcert.Utils.zeropad(limits["default"].getHours()) + 
				":" + inConcert.Utils.zeropad(limits["default"].getMinutes()));
	};
	
	ReschedulePopup.prototype.__makeDateLimit = function(definition, now) {
		if (!definition) {
			return null;
		}
		var value = (definition.up? 1 : -1) * definition.value;
		var date = new Date(now);
		date.setMilliseconds(0);
		date.setSeconds(0);
		switch(definition.unity) {
			case "minutes":
				date.setMinutes(date.getMinutes() + value);
				break;
			case "hours":
				date.setHours(date.getHours() + value);
				break;
			case "days":
				date.setDate(date.getDate() + value);
				break;
			case "months":
				date.setMonth(date.getMonth() + value);
				break;
			case "years":
				date.setFullYear(date.getFullYear() + value);
				break;
		}
		return date;
	};
	
	ReschedulePopup.prototype.__getDatetime = function($section) {
		var date = $section.find("[date]").val();
		var hours = $section.find("[hour]").val() || "00:00";
		return new Date(date + "T" + hours);
	};
	
	ReschedulePopup.prototype.__getScheduledDate = function() {
		return this.__getDatetime($("#outgoingBatchesReschedulePopup").find("[schedule_date]"));
	};
	
	ReschedulePopup.prototype.__getOriginalScheduleDate = function() {
		return this.batch.GetScheduledDate();
	};
	
	ReschedulePopup.prototype.__validateDatetime = function(date, limits) {
		return !isNaN(date.getTime()) &&
				(!limits.max || limits.max >= date) &&
				(!limits.min || limits.min <= date);
	};
	
	ReschedulePopup.prototype.Message = function(text, type) {
		var $message = $("#outgoingBatchesReschedulePopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	ReschedulePopup.prototype.Loading = function(loading) {
		$("#outgoingBatchesReschedulePopup").find("[loading]").toggle(!!loading);
	};
	
	ReschedulePopup.prototype.__makeData = function() {
		return {};
	};
	
	ReschedulePopup.prototype.Update = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		self.Message();
		self.processing = true;
		self.Loading(true);
		$("#outgoingBatchesReschedulePopup").find("button").prop("disabled", true);
		
		var data = { 
			batch : self.__makeData()
		};
		if (!data.batch) {
			return;
		}
		
		self.batch.RunAction("EDIT", function(status, message) {
			self.processing = false;
			self.Loading(false);
			$("#outgoingBatchesReschedulePopup").find("button").prop("disabled", false);
			if (status) {
				return self.Close();
			}
			self.Message(message, "error");
		}, data, true);
	};

}());