/*jshint quotmark:false*/
/*global $, inConcert, _, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementEdit) {
	inConcert.batchManagementEdit = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var templates = inConcert.templates.batch_management;

	function SetAttributesPopup (batch)
	{
		if ($.isArray(batch) && batch.length === 1) {
			batch = batch[0];
		}
		this.batch = batch;
	}
	
	if (typeof inConcert.batchManagementSetAttributes !== "function") {
		inConcert.batchManagementSetAttributes = SetAttributesPopup;
	}
	
	SetAttributesPopup.prototype.__getClassName = function() {
		var type = "";
		if ($.isArray(this.batch)) {
			$.each(this.batch, function(_, batch) {
				if (type === "") {
					type = batch.type;
				}
				else if (type !== batch.type) {
					type = "";
					return false;
				}
			});
		}
		else {
			type = this.batch.type;
		}
		switch(type) {
			case "SMS":
				return "smsBatch";
			case "RCS":
				return "rcsBatch";
			case "WHATSAPP":
				return "whatsappBatch";
			case "MAIL":
				return "mailBatch";
			default:
				return "multipleBatch";
		}
	};
	
	SetAttributesPopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	SetAttributesPopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#outgoingBatchesSetAttributesPopup").remove();
				callback();
			});
		}
		else {
			$("#outgoingBatchesSetAttributesPopup").remove();
			callback();
		}
	};
	
	SetAttributesPopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.set_attributes_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "outgoingBatchesSetAttributesPopup",
			eventName     : "outgoingBatchesSetAttributesPopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#outgoingBatchesSetAttributesPopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	SetAttributesPopup.prototype.__make = function() {
		var self = this;
		var manager = inConcert.app.getBatchesManager();
		var $body = $(Mustache.render(templates.set_attributes_popup, {
			name : $.isArray(this.batch)? 
				"<%=_T('multiple batches')%>" : 
				manager.GetAccountName(self.batch.campaign, self.batch.account, self.batch.page)
		}));
		//attributes
		$body.find("[batch_attributes]")
			.val(self.__getOriginalText())
			.change(function() {
				self.__onTextChange();
			});
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//update
		$body.find("[update]").click(function() {
			const $buttonParent = $("#outgoingBatchesButtonAction");
			self.Update();
			$buttonParent.prop("disabled", true);
		});
		$body.find("#batchesPopupSetAttributesButtonClose")
		.off("click")
		.on("click", function() {
			self.Close();
		});
		
		return $body;
	};
	
	SetAttributesPopup.prototype.__getText = function() {
		return $("#outgoingBatchesSetAttributesPopup").find("[batch_attributes]").val() || "";
	};
	
	SetAttributesPopup.prototype.__getOriginalText = function() {
		return !$.isArray(this.batch) && this.batch.GetAttributes() || "";
	};
	
	SetAttributesPopup.prototype.__onTextChange = _.throttle(function() {
		
	}, 100);
	
	SetAttributesPopup.prototype.Message = function(text, type) {
		var $message = $("#outgoingBatchesSetAttributesPopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	SetAttributesPopup.prototype.Loading = function(loading) {
		$("#outgoingBatchesSetAttributesPopup").find("[loading]").toggle(!!loading);
	};
	
	SetAttributesPopup.prototype.__makeData = function() {
		return {
			attributes : this.__getText()
		};
	};
	
	SetAttributesPopup.prototype.Update = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		self.Message();
		self.processing = true;
		self.Loading(true);
		$("#outgoingBatchesSetAttributesPopup").find("button").prop("disabled", true);
		
		var data = {
			properties : self.__makeData(),
			action : "SET_ATTRIBUTES"
		};
		
		function processResult(status, message) {
			self.processing = false;
			self.Loading(false);
			$("#outgoingBatchesSetAttributesPopup").find("button").prop("disabled", false);
			if (status) {
				return self.Close();
			}
			self.Message(message, "error");
		}
		
		if ($.isArray(self.batch)) {
			var batchesManager = inConcert.app.getBatchesManager();
			batchesManager.RunBatchAction(self.batch, "SET_ATTRIBUTES", processResult, data);
		}
		else {
			self.batch.RunAction("SET_ATTRIBUTES", processResult, data, true);
		}
	};

}());