/*jshint quotmark:false*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.outgoingBatchesStatistics) {
	inConcert.outgoingBatchesStatistics = {};
}
if (!inConcert.outgoingBatchesStatistics.batch) {
	inConcert.outgoingBatchesStatistics.batch = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	function Statistics (definition, slices, defaultSlice, rawData)
	{
		Statistics.uber.constructor.call(this, definition, slices, defaultSlice, rawData);
	}
	
	inConcert.Utils.inherit(Statistics, inConcert.outgoingBatchesStatistics.base);
	if (typeof inConcert.outgoingBatchesStatistics.batch.base !== "function") {
		inConcert.outgoingBatchesStatistics.batch.base = Statistics;
	}
	
	Statistics.prototype.HasAddresses = function() {
		return this.batch.HasAddresses();
	};
	
	var availableAddressesStatus = {
		"TOTAL": {
			name: "<%=_T('Total')%>",
			order: 1
		},
		"QUEUED": {
			name: "<%=_T('Queued')%>",
			order: 2
		},
		"SENDING": {
			name: "<%=_T('Sending')%>",
			order: 3
		},
		"SENT": {
			name: "<%=_T('Sent')%>",
			order: 4
		},
		"RECEIVED": {
			name: "<%=_T('Delivered')%>",
			order: 5
		},
		"READ": {
			name: "<%=_T('Read')%>",
			order: 6
		},
		"ANSWERED": {
			name: "<%=_T('Answered')%>",
			order: 7
		},
		"FAIL": {
			name: "<%=_T('Fail')%>",
			order: 8
		},
		"CANCELED": {
			name: "<%=_T('Canceled')%>",
			order: 9
		}
	};
	
	Statistics.prototype.__getAvailableAddressesStatus = function() {
		return availableAddressesStatus;
	};
	
	Statistics.prototype.__sortAddressesStatus = function(addresses) {
		var available = this.__getAvailableAddressesStatus();
		var addresses_a = [];
		var total = 0;
		$.each(addresses, function(status, value) {
			var availableStatus = available[status] || {};
			addresses_a.push({
				id : status,
				name : availableStatus.name || status,
				order : availableStatus.order || 100,
				value : value
			});
			if (status === "TOTAL") {
				total = parseInt(value);
			}
		});
		$.each(addresses_a, function(_, address) {
			if (address.id !== "TOTAL") {
				address.percentage = Math.round(parseInt(address.value) * 100 / total);
			}
		});
		addresses_a.sort(function(a, b) {
			if (a.order !== b.order) {
				return a.order - b.order;
			}
			return a.name.localeCompare(b.name);
		});
		return addresses_a;
	};
	
	Statistics.prototype.GetAddressesStatus = function(sorted) {
		var addresses = this.batch.GetAddressesStatus();
		return sorted? this.__sortAddressesStatus(addresses) : addresses;
	};
	
	Statistics.prototype.DownloadAddressesStatus = function() {
		this.batch.DownloadAddressesStatus();
	};
	
	Statistics.prototype.GetContent = function(callback) {
		callback = callback || $.noop;
		callback(this.batch.GetContent());
	};
	
	Statistics.prototype.GetAccountData = function() {
		return {
			type		: this.batch.type,
			campaign	: this.batch.campaign,
			account		: this.batch.account,
			page		: this.batch.page || ""
		};
	};
	
	Statistics.prototype.GetData = function() {
		return this.batch.statistics || {};
	};

}());