/*jshint quotmark:false*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;

	function Slice (name)
	{
		this.name = name;
	}
	
	Slice.prototype.GetName = function() {
		return this.name;
	};
	
	Slice.prototype.GetData = function(data, now) {
		throw "not implemented";
	};
	
	//Total
	function TotalSlice() {
		TotalSlice.uber.constructor.call(this, "<%=_T('Total')%>");
	}
	inConcert.Utils.inherit(TotalSlice, Slice);
	
	TotalSlice.prototype.GetData = function(data) {
		return {
			total : data.total || {}
		};
	};
	
	//last days, hours, etc.
	function LastIntervalsSlice(key, name, number) {
		LastIntervalsSlice.uber.constructor.call(this, format(name, number));
		this.number = number;
		this.key = key;
	}

	inConcert.Utils.inherit(LastIntervalsSlice, Slice);
	
	LastIntervalsSlice.prototype.__getLastValuesAndTotal = function(data, from) {
		var result = { values : [], total : {} };
		var sliceData = data[this.key] || [];
		for (var i = 0; i < this.number; i++) {
			var index = sliceData.length - from - i - 1;
			if (index < 0) {
				break;
			}
			var interval = sliceData[index];
			interval.label = inConcert.Utils.getStringFromDate(
				new Date(interval.timestamp * 1000), true, "day-short-word-month");
			interval.shortLabel = inConcert.Utils.getStringFromDate(
				new Date(interval.timestamp * 1000), true, "day-month");
		
			if (!interval.indicators) {
				interval.indicators = {};
				$.each(interval.hours || [], function(_, hour) {
					$.each(hour.indicators || {}, function(id, value) {
						interval.indicators[id] = (interval.indicators[id] || 0) + value;
					});
				});
			}
		
			result.values.unshift(interval);
			$.each(interval.indicators, function(indicator, value) {
				result.total[indicator] = (result.total[indicator] || 0) + value;
			});
		}
		return result;
	};
	
	LastIntervalsSlice.prototype.GetData = function(data) {
		var current = this.__getLastValuesAndTotal(data, 1);
		var last = this.__getLastValuesAndTotal(data, this.number + 1);
		return {
			total : current.total, 
			lastTotal : last.total, 
			values : current.values 
		};
	};
	
	//today or yesterday
	function LastDaySlice(name, number) {
		LastDaySlice.uber.constructor.call(this, name);
		this.number = number;
	}
	inConcert.Utils.inherit(LastDaySlice, Slice);
	
	LastDaySlice.prototype.GetData = function(data, now) {
		//check if today statistics were updated
		var update = data.timestamp? new Date(data.timestamp * 1000) : null;
		if (!update ||
			update.getFullYear() !== now.getFullYear() ||
			update.getMonth() !== now.getMonth() ||
			update.getDate() !== now.getDate()
		) {
			return {
				total : {},
				lastTotal : {},
				values : []
			};
		}
		//get day
		var days = data.days || [];
		var index = days.length - this.number - 1;
		if (index < 0) {
			return {
				total : {},
				lastTotal : {},
				values : []
			};
		}
		var day = days[index];
		//process result
		var result = {
			total : {},
			lastTotal : index > 0? days[index - 1].indicators : {},
			values : day.hours || []
		};
		$.each(result.values, function(_, hour) {
			hour.label = inConcert.Utils.zeropad(hour.hour) + ":00"; 
			$.each(hour.indicators || {}, function(indicator, value) {
				result.total[indicator] = (result.total[indicator] || 0) + value;
			});
		});
		return result;
	};
	
	if (!inConcert.outgoingBatchesStatisticsSlices) {
		inConcert.outgoingBatchesStatisticsSlices = {
			total : new TotalSlice(),
			last7days : new LastIntervalsSlice("days", "<%=_T('Last {0} days')%>", 7),
			last28days : new LastIntervalsSlice("days", "<%=_T('Last {0} days')%>", 28),
			today : new LastDaySlice("<%=_T('Today')%>", 0),
			yesterday : new LastDaySlice("<%=_T('Yesterday')%>", 1)
		};
	}

}());