
/*jshint quotmark:false, esversion: 11*/
/*global inConcert */

// methods in a closure to avoid creating global variables.
if ( !this.inConcert ) {
	this.inConcert = {};
}
if (!inConcert.templates) {
	inConcert.templates = {};
}

(function () {
	'use strict';

	const FORMAT = inConcert.Utils.format;

	const templates = {
		  //search bar
		  search_bar : ''.concat(
			  '<div search_bar class="batchManagementSearchArea assignedInteractionSearch"',
			  '</div>'),
		  //menu
		  menu_category_item : ''.concat(
			  '<li category="{{ id }}" category_selector="true" class="batchCategoryItem {{ className }}">',
				  '<div id="outgoingBatchTitleIcon" category_selector="true" class="categoryItemTitle" title="{{ description }}">',
					  '<i category_selector="true" class="fad categoryItemIcon"></i>',
					  '<label category_selector="true" class="categoryItemText">{{ name }}</label>',
				  '</div>',
				  '<ul id="outgoingBatchAccountList" accountsList class="batchAccountList" style="display:block"></ul>',
			  '</li>'),
		  menu_account_item : ''.concat(
			  '<li campaign="{{ campaign }}" account="{{ account }}" page="{{ page }}" class="batchAccountItem {{ className }}">',
				  '<label title="{{ name }}" class="listItemText">{{ name }}</label>',
			  '</li>'),
		  //list header
		  list_header_column : ''.concat(
			  '<div column="{{ name }}" class="interactionListColumn interactionColumnTitle {{ className }}">',
				  '{{{ content }}}',
			  '</div>'),
		  list_header_column_text : ''.concat(
			  '<div class="interactionColumnTitleText">',
				  '{{ label }}',
			  '</div>'),
		  list_header_column_icon : ''.concat(
			  '<i class="interactionColumnTitleIco"></i>'),
		  //list row
		  list_row : ''.concat(
			  '<div batch_id="{{ id }}" class="interactionListRow interactionOutbound interactionType{{ type }} {{ className }}">',
				  '<div batch_waiting batch_id="{{ id }}" class="interactionWaitingContainer" style="display:none">',
					  '<i class="interactionWaitingAnima"></i>',
					  '<div class="interactionWaitingOverlay"></div>',
				  '</div>',
			  '</div>'),
		  list_row_selector : ''.concat(
			  '<div class="interactionListColumn columnBatch">',
				  '<div class="selectionTriggerContainer">',
					  '<input batch_id="{{ id }}" row_selector="true" class="selectionTriggerCheck" type="checkbox">',
				  '</div>',
			  '</div>'),
		  list_column : ''.concat(
			  '<div column="{{ name }}" class="interactionListColumn {{ className }}">',
				  '{{{ content }}}',
			  '</div>'),
		  list_column_text : ''.concat(
			  '<label class="{{ className }}">{{ text }}</label>'),
		  list_column_abstract : ''.concat(
			  '<div class="{{ className }}" title="{{ text }}">',
				  '{{ text }}',
			  '</div>'),
		  list_column_icon : ''.concat(
			  '<i class="interactionType"></i>'),
		  list_column_actions : ''.concat(
			  '<input batch_actions="true" batch_id="{{ id }}" type="button" value="..." class="menuActionButton buttonColorGray" style="display: inline-block;">'),
		  //batch actions
		  batch_action_select_all : ''.concat(
			  '<input id="batchManagementBatchSelect" select-all="true" class="selectAllTriggerCheck" type="checkbox"/>'),
		  batch_action_select_all_search : ''.concat(
			  '<input class="selectAllTriggerCheck" type="checkbox"/>'),
		  batch_action_button : ''.concat(
			  '<a action="{{ action }}" class="batchActionButton buttonColorGray"  title="{{ description }}" style="display: none;" >', 
				  '{{ name }}', 
				'</a>'),
		  pagination: function() {
			  return ''.concat(
				  '<div id="outgoingBatchesPagination" class="interactionListPager">',
					  '<div class="listPager">',
						  '<div class="pagerTextsContainer">',
							  '<label pagination_label class="pagerText"></label>',
							  '<label id="snPostsPaginationDisclaimer" class="pagerDisclaimer">' +
								  FORMAT('<%=_T("The Panel allows you to display up to {0} rows. If you need to view more rows, you must perform a more specific search.")%>', inConcert.batch_management.maxAllowedRows()),
							  '</label>',
						  '</div>',	
						  '<div class="pagerButtonsContainer">',
							  '<button first class="pg-button butPagerFirst"></button>',
							  '<button prev class="pg-button butPagerPrev"></button>',
							  '<button next class="pg-button butPagerNext"></button>',
							  '<button last class="pg-button butPagerLast"></button>',
						  '</div>',
					  '</div>',
				  '</div>'
			  );
		  },
		  batch_action_pagination_tweet_search : ''.concat(
			  '<div class="interactionListPager">',
				  '<div class="listPager">',
					  '<label pagination_label class="pagerText"></label>',
				  '</div>',
			  '</div>'),
		  //menu actions
		  batch_actions_menu : ''.concat(
			  '<div id="{{ id }}" class="interactionMenuBlind">',
			  '</div>'
		  ),
		  batch_actions_menu_item : ''.concat(
			  '<div batch_id="{{ id }}" action="{{ action }}" class="interactionMenuItem {{ className }}" title="{{ tooltip }}" >',
				  '{{ name }}',
			  '</div>'
		  ),
		  //create batch
		  create_popup_container: ''.concat(
			  '<div id="outgoingBatchesCreatePopup" class="batchesPopup batchesCreatePopup {{ className }}" style="z-index: 999 !important;"></div>'
		  ),
		  create_popup: ''.concat(
			// HEADER
			  '<div class="batchesPopupTitle">',
				  '<label class="batchesPopupTitleText">',
					  '{{ title }} {{ name }}',
				  '</label>',
				//   '<button id="batchesCreatePopupButtonClose" statsbuttonClose class="batchesPopupCloseButton" >',
				// 		"<i class='far fa-times'></i>",
				// 	'</button>',
			  '</div>',
			 // BODY
	         '<div id="outgoingBatchesCreatePopupBodyContainer" class="batchesPopupBodyContainer">',
			    '<div message_container class="popupMessage" style="display:none">',
				  '<label message_text>',
				  '</label>',
			    '</div>',
			    '<div loading class="popupLoading" style="display:none">',
				  '<i class="popupWaitingAnimation"></i>',
				  '<div class="popupWaitingOverlay"></div>',
			    '</div>',
				'<div id="outgoingBatchesCreatePopup_tabs" class="formFatherFieldset"></div>',
				'<div id="outgoingBatchesCreatePopup_tabs_message" tab_id="message" class="card-block" hidden >',
					'<div class="row">',
						'<div batch_id_container class="col-6">',
							'<div class="form-group">',
								'<label class="required">',
									'<%=_T("Id")%>:',
								'</label>',
								'<i id="batch_create_id_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
								'<input id="batch_create_id" type="text" class="form-control" placeholder="<%=_T([[Enter the id]])%><%=_T([[...]])%>">',
							'</div>',
						'</div>',
						'{{{ after_id }}}',
					'</div>',
					'<div class="row" id="batchesCreatePopup_headertextcontainer">',
						'<div class="col-12">',
							'<div class="form-group" style="display:none;">',
								'<label batch_header_text class="">',
									'<%=_T("Header")%>:',
								'</label>',
								'<i brackets_icon="" class="formFieldBracketsIcon" style="display:none;"></i>',
								'<i id="batch_create_header_text_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
							'</div>',
							'<div text_limit class="textLimit" style="display:none"></div>',
						'</div>',
					'</div>',
					'<div class="row" id="batchesCreatePopup_textcontainer">',
						'<div class="col-12">',
							'<div class="form-group">',
								'<label class="">',
									'<%=_T("Message")%>:',
									'<span class="fa fa-question-circle" title="<%=_T([[You can use &#123;&#123;&nbsp;&#125;&#125; to send custom messages per address. For example, if the header title of the name column is Fullname, the message would read &#123;&#123; FullName &#125;&#125;.]])%>"></span>',
								'</label>',
								'<i id="batch_create_text_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
								'<textarea batch_text id="batchesCreatePopup_textarea" class="form-control message-textarea"></textarea>',
								'<i brackets_icon="" class="formFieldBracketsIcon" style="display:none;"></i>',
							'</div>',
							'<div text_limit class="textLimit" style="display:none"></div>',
						'</div>',
					'</div>',
					'<div content_container class="batchContent"></div>',
					'<div advanced_options class="advancedOptions"></div>',
				'</div>',
				'<div id="outgoingBatchesCreatePopup_tabs_addresses" tab_id="address" class="card-block"></div>',
			'</div>',
			  // FOOTER
			  '<div class="editActions d-flex justify-content-end">',
				  '<label character_counter class="characterCounter">',
				  '</label>',
				  '<button cancel class="pageButtonGeneric buttonColorWhite mr-0" >',
					  '<%=_T("Cancel")%>',
				  '</button>',
				  '<button id="ConfirmButtonBatch" create class="pageButtonGeneric buttonColorBlue" >',
					  '<%=_T("Confirm")%>',
				  '</button>',
			'</div>'
		  ),
		  create_popup_template_selector: ''.concat(
			  '<div id="batch_create_template_group_container" class="batchCreateTemplateGroupContainer col-6">',
				'<div class="form-group">',
					'<label class="required">',
						'<%=_T("Template Group")%>:',
					'</label>',
					'<select id="batch_create_template_group" class="form-control">',
					'</select>',
					'<i id="batch_create_template_group_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
				'</div>',
			  '</div>',
			  '<div id="batch_create_template_name_container" class="batchCreateTemplateNameContainer col-6">',
			  	'<div class="form-group">',
					'<label class="required">',
						'<%=_T("Template Name")%>:',
					'</label>',
					'<select id="batch_create_template_name" class="form-control">',
					'</select>',
					'<i id="batch_create_template_name_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
				'</div>',
			  '</div>',
			  '<div id="batch_create_template_language_container" class="batchCreateTemplateLanguageContainer col-6">',
				'<div class="form-group">',
					'<label class="required">',
					'<%=_T("Language")%>:',
					'</label>',
					'<select id="batch_create_template_language" class="form-control">',
					'</select>',
					'<i id="batch_create_template_language_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
				'</div>',
			  '</div>'
		  ),
		  create_popup_template_header_text: ''.concat(
		      "<div id='batch_create_template_header_text_container' class='batchTemplateMessageText'>",
		      "</div>",		  
		  ),
		  create_popup_template_text: ''.concat(
			  "<div id='batch_create_template_text_container' class='batchTemplateMessageText'>",
			  "</div>",
			  "<div style='display:none;'>",
			  	"<label class='windowFormLabel'>",
			  		"<%=_T('Resource')%>:",
			  	"</label>",
			  	"<div id='batch_create_template_message_button' class='batchTemplateMessageText'>",
			  	"</div>",
			  "</div>",			  
		  ),
		  create_popup_subject: ''.concat(
			  '<div batch_subject_container class="col-12">',
				  '<div class="form-group">',
					  '<label class="required">',
						  '<%=_T("Subject")%>:',
					  '</label>',
					  '<input id="batch_create_subject" type="text" class="form-control" placeholder="<%=_T([[Enter the subject]])%><%=_T([[...]])%>" />',
					  '<i brackets_icon="" class="formFieldBracketsIcon" style="display:none;"></i>',
					  '<i id="batch_create_subject_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
				  '</div>',
			  '</div>'
		  ),
		  create_popup_attachments: ''.concat(
			  '<div batch_attachments_container class="col-12">',
				  '<div class="form-group">',
					  '<label class="">',
						  '<%=_T("Attachments")%>:',
					  '</label>',
					  '<div class="d-flex flex-row">',
							'<button id="batch_add_attachment_button" class="pageButtonGeneric buttonColorWhite pageButtonNew float-none ml-0" >',
								'<i class="far fa-plus"></i>',
								'<%=_T("Add attachment")%>',
							'</button>',
							'<input type="file" id="batch_add_attachment_input" style="display:none;">',
						'</div>',
						'<div id="batch_attachments" class="batchCreateAttachments">',
						'</div>',
					  '<i id="batch_attachments_validation" class="popUpFormFieldValidation formFieldValidationError" style="display: none;"></i>',
				  '</div>',
			  '</div>'
		  ),
		  create_popup_attachment: ''.concat(
			  '<div batch_attachment="{{ id }}" class="batchCreateAttachmentContainer">',
				  '<label class="batchCreateAttachmentLabel">',
					  '{{ name }}',
				  '</label>',
				  '<i batch_attachment_delete class="far fa-times"></i>',
			  '</div>'
		  ),
		  //create batch
		  create_popup_content: ''.concat(
			  '<label class="batchContentTitle">',
				  '<%=_T("Content:")%>',
			  '</label>',
			  '<div content="nothing" class="batchContentItem">',
				  '<input  content_selector="nothing" type="radio" class="batchContentRadio" id="batch_create_content_nothing" name="batch_create_content_selector">',
				  '<label class="batchContentLabel" for="batch_create_content_nothing">',
					  '<%=_T("Just Text")%>',
				  '</label>',
			  '</div>',
			  '<div content="link" class="batchContentItem">',
				  '<input  content_selector="link" type="radio" class="batchContentRadio" id="batch_create_content_link" name="batch_create_content_selector">',
				  '<label class="batchContentLabel" for="batch_create_content_link">',
					  '<%=_T("Url link")%>',
				  '</label>',
			  '</div>',
			  '<div content="image" class="batchContentItem">',
				  '<input content_selector="image" type="radio" class="batchContentRadio" id="batch_create_content_image" name="batch_create_content_selector">',
				  '<label class="batchContentLabel" for="batch_create_content_image">',
					  '<%=_T("Image")%>',
				  '</label>',
			  '</div>',
			  '<div content_data="link" class="batchContentItemAux batchContentUrl">',
				  '<input link_text type="text" id="batch_create_content_link_text" class="batchContentField">',
			  '</div>',
			   '<div content_data="image" class="batchContentItemAux">',
				   '<div class="batchContentItemContent">',
					   '<div image_content="link" class="batchImageItem">',
						   '<label class="batchContentAuxLabel" for="batch_create_content_image_link">',
							   '<%=_T("From link:")%>',
						   '</label>',
						   '<input image_selector="link" type="radio" class="batchContentRadio" id="batch_create_content_image_link" name="batch_create_image_selector">',
						   '<input image_content_data image_link_text type="text" id="batch_create_content_image_link_text" class="batchContentField">',
					   '</div>',
					   '<div image_content="disk" class="batchImageItem">',
						   '<label class="batchContentAuxLabel" for="batch_create_content_image_disk">',
							   '<%=_T("Upload an image:")%>',
						   '</label>',
						   '<input image_selector="disk" type="radio" class="batchContentRadio" id="batch_create_content_image_disk" name="batch_create_image_selector">',
						   '<div image_content_data class="buttonColorGray batchImageUploadFile">',
							   '<%=_T("Select image...")%>',
							   '<input id="batch_create_content_image_disk_upload" type="file" hidden accept="image/*" />',
						   '</div>',
					   '</div>',
					   '<div class="batchImageItem">',
						   '<div image_preview class="imagePreview">',
							   '<label class="batchContentAuxLabel">',
								   '<%=_T("Preview:")%>',
							   '</label>',
							   '<div class="batchContentPreview" id="batch_create_content_image_preview">',
							   '</div>',
						   '</div>',
					   '</div>',
				   '</div>',
			   '</div>'
		  ),
		  create_popup_advanced: ''.concat(
			  '<div schedule_date class="advancedOption scheduleDate">',
				  '<label class="advancedOptionLabel" for="batch_create_schedule_date">',
					  '<%=_T("Schedule Date:")%>',
				  '</label>',
				  '<input type="checkbox" class="advancedOptionCheck" id="batch_create_schedule_date">',
				  '<input date type="text" class="advancedOptionField datePicker">',
				  '<input hour type="text" class="advancedOptionField hourPicker">',
			  '</div>',
			  '<div backdated_time class="advancedOption previousDate">',
				  '<label class="advancedOptionLabel" for="batch_create_backdated_time">',
					  '<%=_T("Assign a previous Date:")%>',
				  '</label>',
				  '<input type="checkbox" class="advancedOptionCheck" id="batch_create_backdated_time">',
				  '<input date type="text" class="advancedOptionField datePicker">',
				  '<input hour type="text" class="advancedOptionField hourPicker">',
			  '</div>'
		  ),
		  create_popup_addresses: ''.concat(
			  '<div class="batchAddressesContent" batch_address_content="true">',
				  '<div class="row">',
					  '<div class="col-12 d-flex flex-column">',
						  '<div class="form-group">',
							  '<div address_source="file" class="form-radiobutton">',
								  '<label class="form-radiobutton-label" for="batch_create_addresses_file_selector">',
									  '<input address_source_selector="file" placeholder="<%=_T("File name")%>..." type="radio" class="form-radiobutton-input" id="batch_create_addresses_file_selector" name="batch_create_addresses">',
									  '<span>',
										  '<%=_T("Upload file")%>',
									  '</span>',
								  '</label>',
							  '</div>',
							  '<div address_source="text" class="form-radiobutton">',
								  '<label class="form-radiobutton-label" for="batch_create_addresses_text_selector">',
									  '<input address_source_selector="text" type="radio" class="form-radiobutton-input" id="batch_create_addresses_text_selector" name="batch_create_addresses">',
									  '<span>',
										  '<%=_T("Use address list")%>',
									  '</span>',
								  '</label>',
							  '</div>',
						  '</div>',
					  '</div>',
				  '</div>',
				//   '<hr/>',
				  '<div address_data class="">',
					  '<div class="">',
						  '<div address_source_data="file" class="row">',
							  '<div class="col-6 d-flex flex-row">',
								  '<div class="form-group flex-grow-1">',
									  '<label class="required">',
										  '<%=_T("File")%><%=_T(":")%>',
									  '</label>',
									  '<div id="batch_create_addresses_filename" class="form-control">',
									  '</div>',
								  '</div>',
								  '<div class="form-group d-flex align-items-end">',
							  		'<div class="pageButtonGeneric buttonColorWhite iconOnlyBtn">',
							  			'<i class="far fa-upload" title="<%=_T("Upload file")%>"></i>',
							  			'<input id="batch_create_addresses_file" type="file" hidden accept="*" />',
						  			'</div>',
							  	  '</div>',
							  '</div>',
						  '</div>',
						  '<div address_source_data="text" class="row">',
							   '<div class="col-12">',
								  '<div class="form-group">',
									  '<label class="required">',
										  '<%=_T("List")%><%=_T(":")%>',
										  '<span class="fa fa-question-circle" title="<%=_T([[Paste or write the list with the phone numbers of the recipients.]])%>"></span>',
									  '</label>',
									  '<textarea id="batch_create_addresses_text" placeholder="<%=_T([[Paste or write]])%>..." class="form-control">',
									  '</textarea>',
								  '</div>',
							  '</div>',
						  '</div>',
						  '<hr/>',
						'<div class="row">',
							'<div class="col-12">',
							  '<div class="card-header">',
								  '<h5>',
									  '<%=_T("Separators")%>',
									  '<span class="fa fa-question-circle" title="<%=_T("Specify how data is separated into rows and columns.")%>"></span>',
								  '</h5>',
							  '</div>',
						  	'</div>',
					  	'</div>',
						  '<div class="row">',
							  '<div class="col-6">',
								  '<div class="form-group">',
									  '<label>',
										  '<%=_T("For rows:")%>',
									  '</label>',
									  '<select id="batch_create_rows_separator" class="form-control">',
										  '<option value=";">;</option>',
										  '<option value=",">,</option>',
										  '<option value=" "><%=_T("White space")%></option>',
										  '<option value="TAB"><%=_T("Tab")%></option>',
										  '<option value="LINE_BREAK" selected><%=_T("Line break")%></option>',
									  '</select>',
								  '</div>',
							  '</div>',
							   '<div class="col-6">',
								  '<div class="form-group">',
									  '<label>',
										  '<%=_T("For columns:")%>',
									  '</label>',
									  '<select id="batch_create_columns_separator" class="form-control">', 
										  '<option value=";">;</option>',
										  '<option value=",">,</option>',
										  '<option value=" "><%=_T("White space")%></option>',
										  '<option value="TAB"><%=_T("Tab")%></option>',
										  '<option value="LINE_BREAK"><%=_T("Line break")%></option>',
									  '</select>',
								  '</div>',
							  '</div>',
						  '</div>',
						  '<hr/>',
						  '<div address_data class="row">',
						  		'<div class="col-12 d-flex flex-row justify-content-between header-form-switch">',
								  	'<div class="card-header">',
								  		'<h5>',
									  		'<%=_T("This file have headers")%>',
											  '<span class="fa fa-question-circle" title="<%=_T([[By checking this checklist, you will be able to use the header columns as variables in the message ({{FullName}} for example, where the header title is FullName). The file must include the headings (without accents, spaces or symbols)]])%>"></span>',
								  		'</h5>',
							  		'</div>',
									'<div class="form-group">',
									  '<div class="form-check form-switch">',
										  '<label class="form-check-label">',
											  '<input type="checkbox" class="form-check-input" id="batch_create_include_headers">',
										  		'<span></span>',
											'</label>',
									  '</div>',
								  	'</div>',
							  	'</div>',
						  '</div>',
						  '<div id="batch_create_address_column_container" class="row" style="display:none;">',
							  '<div class="col-6">',
								  '<div class="form-group">',
									  '<label class="required">',
										  '<%=_T("Address Column:")%>',
									  '</label>',
									  '<input type="text" class="form-control" placeholder="<%=_T("Enter the value")%><%=_T("...")%>" id="batch_create_address_column" title="',
										  '<%=_T("Name of the column that has messages address.")%>',
									  '">',
								  '</div>',
							  '</div>',
							  '<div class="col-6">',
								  '<div class="form-group">',
									  '<label>',
										  '<%=_T("Name Column:")%>',
									  '</label>',
									  '<input type="text" class="form-control" placeholder="<%=_T("Enter the value")%><%=_T("...")%>" id="batch_create_name_column" title="',
										  '<%=_T("Name of the column that has messages contact name.")%>',
									  '">',
								  '</div>',
							  '</div>',
							  '<div id="batch_create_reporting_group_container" class="col-6" style="display:none;">',
								  '<div class="form-group">',
									  '<label>',
										  '<%=_T("Reporting group:")%>',
										  '<i class="fa fa-question-circle" title="',
										  "<%=_T([[It allows to group the custom fields to report later. This grouping will allow you to differentiate one custom value from others. All additional data entered will be available to display in custom reports.]])%>",
									  '"></i>',
									  '</label>',
									  '<input type="text" class="form-control" placeholder="<%=_T("Enter the value")%><%=_T("...")%>" id="batch_create_reporting_group" />',
								  '</div>',
							  '</div>',
						  '</div>',
						  '<hr />',
						  '<div id="batch_create_address_agent_data_container" class="row" style="display:none;">',
							  '<div class="col-12">',
								  '<div class="card-header d-flex flex-row justify-content-between mb-0">',
									 '<div>',
										'<h5 class="mb-0">',
										  '<%=_T("Agent Data")%>',
										  '<span class="fa fa-question-circle" title="<%=_T("Define columns from the contact import that will be displayed to the agent that processes customer responses")%>"></span>',
									  	'</h5>',
									  '</div>',
								  	  
									  '<button id="batch_create_address_agent_data_add" class="btn buttonIconAndText">', 
										  '<span class="far fa-plus mr-0"></span>',
									  '</button>',
								  '</div>',
							  '</div>',
							  '<div class="agentDataListContainer col-12">',
								  '<ul id="batch_create_address_agent_data_list" class="formList newParameterList mr-0" style="display:none;">',
								  '</ul>',
							  '</div>',
						  '</div>',
					  '</div>',
				  '</div>',
			  '</div>'
		  ),
		  address_agent_data: ''.concat(
			  	'<li agent_data_item class="row">',
				  	'<div class="col-6">',
						'<div class="form-group">',
							'<label>',
								'<%=_T("Column")%><%=_T(":")%>',
							'</label>',
							'<input agent_data_column type="text" placeholder="<%=_T("Enter the value")%><%=_T("...")%>" class="form-control">',
						'</div>',
					'</div>',
					'<div class="col-6 d-flex flex-row">',
						'<div class="form-group flex-grow-1">',
							'<label>',
								'<%=_T("Name to show")%><%=_T(":")%>',
							'</label>',
							'<input agent_data_name type="text" placeholder="<%=_T("Enter the value")%><%=_T("...")%>" class="form-control">',
						'</div>',
						'<div class="form-group align-self-end">',
							'<button agent_data_delete class="pageButtonGeneric buttonColorWhite iconOnlyBtn">',
								'<i class="far fa-trash-alt"></i>',
							'</button>',
						'</div>',
					'</div>',
				'</li>',
		  ),
		    //add new addresses to batch
			add_addresses_popup_container: ''.concat(
				'<div id="outgoingBatchesAddressesPopup" class="batchesPopup batchesCreatePopup {{ className }}"></div>'
			),
			add_addresses_popup: ''.concat(
				// HEADER
				  '<div class="batchesPopupTitle">',
					  '<label class="batchesPopupTitleText">',
						  '{{ title }} > {{ type }} > {{ batchId }}',
					  '</label>',
					  '<button id="batchesAddressesPopupButtonClose" statsbuttonClose class="batchesPopupCloseButton" >',
							"<i class='far fa-times'></i>",
						'</button>',
				  '</div>',
				 // BODY
				 '<div id="batchesAddressesPopupBodyContainer" class="batchesPopupBodyContainer">',
					'<div message_container class="popupMessage" style="display:none">',
					  '<label message_text>',
					  '</label>',
					'</div>',
					'<div loading class="popupLoading" style="display:none">',
					  '<i class="popupWaitingAnimation"></i>',
					  '<div class="popupWaitingOverlay"></div>',
					'</div>',
					'<div id="batchesAddressesPopupBodyAdd" class="card-block"></div>',
				'</div>',
				  // FOOTER
				  '<div class="editActions d-flex justify-content-end">',
					  '<label character_counter class="characterCounter">',
					  '</label>',
					  '<button cancel class="pageButtonGeneric buttonColorWhite mr-0" >',
						  '<%=_T("Cancel")%>',
					  '</button>',
					  '<button id="ConfirmButtonBatch" create class="pageButtonGeneric buttonColorBlue" >',
						  '<%=_T("Confirm")%>',
					  '</button>',
				'</div>'
			  ),
		  //set attributes batch
		  set_attributes_popup_container: ''.concat(
			  '<div id="outgoingBatchesSetAttributesPopup" class="batchesPopup batchesSetAttributesPopup {{ className }}">',
			  '</div>'
		  ),
		  set_attributes_popup: ''.concat(
			  '<div class="batchesPopupTitle">',
				  '<label class="batchesPopupTitleText">',
					  '<%=_T("Set Attributes on")%> {{ name }}',
				  '</label>',
				  //'<button id="batchesPopupSetAttributesButtonClose" statsbuttonClose class="batchesPopupCloseButton" >',
						//"<i class='far fa-times'></i>",
					//'</button>',
			  '</div>',
			  '<div message_container class="popupMessage" style="display:none">',
				  '<label message_text>',
				  '</label>',
			  '</div>',
			  '<div loading class="popupLoading" style="display:none">',
				  '<i class="popupWaitingAnimation"></i>',
				  '<div class="popupWaitingOverlay"></div>',
			  '</div>',
			  '<div class="card">',
				  '<div class="card-block">',
					  '<div class="row">',
						  '<div class="col-12">',
							  '<div class="form-group">',
								  '<label for="batchesSetAttributesPopup_textarea">',
									  '<%=_T("Attributes")%><%=_T(":")%>',
								  '</label>',
								  '<textarea batch_attributes id="batchesSetAttributesPopup_textarea" class="form-control">',
								  '</textarea>',
							  '</div>',
						  '</div>',
					  '</div>',
				  '</div>',
			  '</div>',
			  '<div class="editActions d-flex justify-content-end">',
					'<button cancel class="pageButtonGeneric buttonColorWhite mr-0" >',
						'<%=_T("Cancel")%>',
					'</button>',
					'<button update class="pageButtonGeneric buttonColorBlue" >',
						'<%=_T("Update")%>',
					'</button>',
			  '</div>'
		  ),
		  //edit batch
		  edit_popup_container: ''.concat(
			  '<div id="outgoingBatchesEditPopup" class="batchesEditPopup {{ className }}">',
			  '</div>'
		  ),
		  edit_popup: ''.concat(
			  '<div class="batchesPopupTitle">',
				  '<label class="batchesPopupTitleText">',
					  '<%=_T("Edit Batch on")%> {{ name }}',
				  '</label>',
			  '</div>',
			  '<div message_container class="popupMessage" style="display:none">',
				  '<label message_text>',
				  '</label>',
			  '</div>',
			  '<div loading class="popupLoading" style="display:none">',
				  '<i class="popupWaitingAnimation"></i>',
				  '<div class="popupWaitingOverlay"></div>',
			  '</div>',
			  '<div class="textContainer">',
				  '<textarea batch_text id="outgoingBatchesEditPopup_textarea">',
				  '</textarea>',
			  '</div>',
			  '<div class="editActions">',
				  '<button update class="createBatchButton buttonColorBlack"  disabled>',
					  '<%=_T("Update")%>',
				  '</button>',
				  '<button cancel class="cancelBatchButton buttonColorLightblue" >',
					  '<%=_T("Cancel")%>',
				  '</button>',
				  '<label character_counter class="characterCounter">',
				  '</label>',
			  '</div>'
		  ),
		  //reschedule batch
		  reschedule_popup_container: ''.concat(
			  '<div id="outgoingBatchesReschedulePopup" class="batchesReschedulePopup {{ className }}">',
			  '</div>'
		  ),
		  reschedule_popup: ''.concat(
			  '<div class="batchesPopupTitle">',
				  '<label class="batchesPopupTitleText">',
					  '<%=_T("Reschedule Batch on")%> {{ name }}',
				  '</label>',
			  '</div>',
			  '<div message_container class="popupMessage" style="display:none">',
				  '<label message_text>',
				  '</label>',
			  '</div>',
			  '<div loading class="popupLoading" style="display:none">',
				  '<i class="popupWaitingAnimation"></i>',
				  '<div class="popupWaitingOverlay"></div>',
			  '</div>',
			  '<div schedule_date class="advancedOption scheduleDate">',
				  '<label class="advancedOptionLabel" for="batch_create_schedule_date">',
					  '<%=_T("Schedule Date:")%>',
				  '</label>',
				  '<input date type="text" class="advancedOptionField datePicker">',
				  '<input hour type="text" class="advancedOptionField hourPicker">',
			  '</div>',
			  '<div class="editActions">',
				  '<button update class="createBatchButton buttonColorBlack" >',
					  '<%=_T("Update")%>',
				  '</button>',
				  '<button cancel class="cancelBatchButton buttonColorLightblue" >',
					  '<%=_T("Cancel")%>',
				  '</button>',
				  '<label character_counter class="characterCounter">',
				  '</label>',
			  '</div>'
		  ),
		  //statistics view
		  statistics_preview : ''.concat(
			  '<div class="statsTitleContainer">',
				  '<label batch_name class="statsTitleText"><%=_T("Detail")%></label>',
				  	'<button id="outgoingBatchesStatsTitle" statsbuttonClose class="statsCloseButton" >',
						"<i class='far fa-times'></i>",
					'</button>',
			  '</div>',
			  '<div stats_body class="statsContentContainer ">',
					'<div class="statsRange card" >',
							/*'<div class="card-block">',*/
								'<div class="row align-items-end">',
										/*'<div class="col-6">',
											'<div class="form-group mb-0">',
												'<label><%=_T("Stats Range:")%></label>',
												'<select range class="form-control">',
												'</select>',
											'</div>',
										'</div>',*/
										'<div class="col-6 d-flex justify-content-start align-items-center">',
											'<i class="far fa-signal-3">',
											'</i>',
											'<label class="selectLabel"><%=_T("Stats:")%></label>',
										'</div>',
										/*'<div class="col-6 d-flex justify-content-end">',
											'<button view_details class="pageButtonGeneric buttonColorWhite" >',
												'<%=_T("Details")%>',
											'</button>',
										'</div>',*/

										'<div class="col-6 d-flex justify-content-end pl-0">',
											'<select range class="form-control">',
											'</select>',
											'<button view_details class="pageButtonGeneric buttonColorWhite p-maximize" >',
											'<i class="far fa-expand mr-0">',
											'</i>',
											'</button>',
										'</div>',

									'<div class="row w-100">',
										'<div class="col-12 px-0">',
											'<div list class="statsContent" >',
											'</div>',
										'</div>',
									'</div>',
								'</div>',
							/*'</div>',*/
					'</div>',

					/*'<div list class="statsContent card" >',
					'</div>',*/

					'<div class="row statsContentLastUpdate">',
						'<div class="col-12 d-flex justify-content-end">',
							'<div class="form-group ">',
								'<label><%=_T("Last Update:")%></label>',
								'<label class="pl-1" update_date></label>',
							'</div>',
						'</div>',
					'</div>',

					'<div class="statsBottom " >',
						'<div class="card-block">',
							'<div class="row">',
								/*'<div class="col-6">',
									'<div class="form-group mb-0 mt-2">',
										'<label><%=_T("Last Update:")%></label>',
										'<label class="pl-1" update_date></label>',
									'</div>',
								'</div>',*/
								'<div class="col-12 d-flex justify-content-end mt-2">',
									'<button id="outgoingBatchesStatsButton" addresses_status_details class="pageButtonGeneric buttonColorWhite" >',
										'<%=_T("Download list")%>',
									'</button>',
								'</div>',
							'</div>',
						'</div>',
				  '</div>',

			  '</div>'
		  ),
		  statistics_popup_container: ''.concat(
			  '<div id="outgoingBatchesStatsPopup" class="batchesPopup statsPopup {{ className }}">',
			  '</div>'
		  ),
		  statistics_popup: ''.concat(
			  '<div stats_body class="statsContentContainer">',
				  '<div class="batchesPopupTitle">',
					  '<label class="batchesPopupTitleText">',
						  '{{ name }} <%=_T("Details")%> ',
					  '</label>',
					  '<button id="snPopupButtonClose" bmPopupButtonClose class="batchesPopupCloseButton">',
						"<i class='far fa-times'></i>",
					'</button>',
				  '</div>',
				  '<div class="statsBodyContent">',
					  '<div class="card">',
						  '<div class="card-block">',
							  '<div class="row">',
								  '<div class="col-6">',
									  '<div class="form-group">',
										  '<label>',
											  '<%=_T("Stats Range:")%>',
										  '</label>',
										  '<select range class="form-control">',	
										  '</select>',
									  '</div>',
								  '</div>',
							  '</div>',
							  '<hr/>',
							  '<div class="row">',
								  '<div class="col-12">',
									  '<div list class="statsContent">',
									  '</div>',
								  '</div>',
							  '</div>',
						  '</div>',
					  '</div>',
				  '</div>',
				  '<div class="editActions d-flex justify-content-between">',
					  '<div class="statsUpdateDate">',
						  '<label class="statsUpdateDateTitle"><%=_T("Last Update:")%></label>',
						  '<label update_date class="statsUpdateDateValue"></label>',
					  '</div>',
					  '<button close_popup class="pageButtonGeneric buttonColorWhite">',
						  '<%=_T("Close")%>',
					  '</button>',
				  '</div>',
			  '</div>'
		  ),
		  statistics_batch_content : ''.concat(
			  '<div batch_content class="statsBatchContent card mt-0 px-0">',
			 		'<div class="statsBatchContentTitle d-flex justify-content-start align-items-center">',
							'<i class="far fa-poll-h"></i>',
							'<label class="selectLabel"><%=_T("General")%>:</label>',
					'</div>',
			  '</div>'
		  ),
		  statistics_batch_quoted_content : ''.concat(
			  '<div batch_content class="statsBatchQuotedContent">',
				  '<label class="statsBatchQuotedUsername">',
					  '{{ name }}',
				  '</label>',
				  '<div class="statsBatchQuotedAvatar">',
					  '<img src="{{ avatar }}">',
				  '</div>',
			  '</div>'
		  ),
		  statistics_batch_text : ''.concat(
			  '<div batch_text class="statsBatchText truncated">',
				'<label title="{{ text }}">',
				'</label>',
				  '<a view_more href="#"><%=_T("View more")%></a>',
			  '</div>'
		  ),
		  statistics_batch_image : ''.concat(
			  '<div class="statsBatchImage ">',
				  '<img class="archivedBatchImage" src="{{ image }}">',
				  '<label class="archivedBatchName">{{ name }}</label>',
				  '<label class="archivedBatchDescription">{{ description }}</label>',
				  '<label class="archivedBatchCaption">{{ caption }}</label>',
			  '</div>'
		  ),
		  statistics_batch_image_link : ''.concat(
			  '<div class="statsBatchImage ">',
				  '<a href="{{ link }}">',
					  '<img class="archivedBatchImage" src="{{ image }}">',
					  '<label class="archivedBatchName">{{ name }}</label>',
					  '<label class="archivedBatchDescription">{{ description }}</label>',
					  '<label class="archivedBatchCaption">{{ caption }}</label>',
				  '</a>',
			  '</div>'
		  ),
		  statistics_batch_video : ''.concat(
			  '<div class="statsBatchImage video ">',
				  '<a href="{{ link }}">',
					  '<img class="archivedBatchImage" src="{{ image }}">',
					  '<label class="archivedBatchName">{{ name }}</label>',
					  '<label class="archivedBatchDescription">{{ description }}</label>',
					  '<label class="archivedBatchCaption">{{ caption }}</label>',
				  '</a>',
			  '</div>'
		  ),
		  statistics_batch_mail : ''.concat(
			  '<div class="statsBatchMailContent card mt-0">',
			 	 '<div class="statsBatchMailContentTitle d-flex justify-content-start align-items-center">',
					'<i class="far fa-poll-h">',
					'</i>',
			  		'<label class="selectLabel"><%=_T("General")%>:</label>',
		  		'</div>',
				  '<div class="statsBatchMailContentIdContainer">',
					  '<label class="statsBatchMailContentIdTitle">',
						  '<%=_T("Id")%>:',
					  '</label>',
					  '<label class="statsBatchMailContentIdText">',
						  '{{ id }}',
					  '</label>',
				  '</div>',
				  '<div class="statsBatchMailContentSubjectContainer">',
					  '<label class="statsBatchMailContentSubjectTitle">',
						  '<%=_T("Subject")%>:',
					  '</label>',
					  '<label class="statsBatchMailContentSubjectText">',
						  '{{ subject }}',
					  '</label>',
				  '</div>',
				  '<div class="statsBatchMailContentBody">',
					  '<label class="statsBatchMailContentBodyTitle">',
						  '<%=_T("Message")%>:',
					  '</label>',
					  '<iframe id="mail_batch_iframe_{{ id }}" class="statsBatchMailContentBodyIframe" ', 
						  'src="/inconcert/apps/agent/pages/mailBody.<%=_L%>.html" ', 
						  'width="100%" height="100%" frameborder="0" scrolling="yes" style="font-family: "Roboto Condensed", sans-serif;">',
					  '</iframe>',
				  '</div>',
			  '</div>'
		  ),
		  statistics_batch_whatsapp_cloud_api_template_paused : ''.concat(
			'<div class=" pausedBatchContent card mt-0">',
				'<div class=" pausedBatchContentTitle  d-flex justify-content-start align-items-center">',
					'<i class="far fa-exclamation-triangle">',
					'</i>',
					'<label class="selectLabel"><%=_T("Paused batch")%>:</label>',
				'</div>',
				'<div class=" pausedBatchContentBody">',
					'<label class=" pausedBatchContentSubjectText">',
						'<%=_T("WhatsApp has temporarily paused the batch due to a possible violation of the terms of service. It will be resumed once the template is adjusted and approved by them.")%>',
					'</label>',
					'<div class=" pausedBatchContentIdContainer">',
						'<label class=" pausedBatchContentIdTitle">',
							'<%=_T("Template")%>:',
						'</label>',
						'<label class=" pausedBatchContentIdText">',
							'{{ template_id }}',
							' ({{ template_language }})',
						'</label>',
					'</div>',
				'</div>',
			'</div>'
		  ),
		  statistics_batch_generic_content_account : ''.concat(
			'<div class=" statsBatchGenericContent card mt-0">',
				'<div class=" statsBatchGenericContentTitle  d-flex justify-content-start align-items-center">',
				  '<i class="far fa-poll-h">',
				  '</i>',
					'<label class="selectLabel"><%=_T("General")%>:</label>',
				'</div>',
				'<div class=" statsBatchGenericContentIdContainer">',
					'<label class=" statsBatchGenericContentIdTitle">',
						'<%=_T("Id")%>:',
					'</label>',
					'<label class=" statsBatchGenericContentIdText">',
						'{{ id }}',
					'</label>',
				'</div>',
				'{{#subject}}<div class=" statsBatchGenericContentSubjectContainer">',
					'<label class="statsBatchGenericAccountContentSubjectTitle">',
						'<%=_T("Subject")%>:',
					'</label>',
					'<label class=" statsBatchGenericContentSubjectText">',
						'{{ subject }}',
					'</label>',
				'</div>{{/subject}}',
				'{{#message}}<div class=" statsBatchGenericContentBody">',
					'<label class=" statsBatchGenericContentBodyTitle">',
						'<%=_T("Message")%>:',
					'</label>',
					'<div class=" statsBatchGenericContentSubjectText">{{ message }}</div>',
				'</div>{{/message}}',
			'</div>'
		),
		  statistics_item : ''.concat(
			  '<div class="statsItem">',
				  '<div statistic="{{ id }}" class="statsItemData">',
					  '<div class="row align-items-center">',
						  '<div class="col">',
							  '<i class="statsItemIcon {{ iconClass }}" title="{{ description }}">',
							  '</i>',
							  '<label name class="statsItemName" title="{{ description }}">',
								  '{{ name }}',
							  '</label>',
						  '</div>',
						  /*'<div class="col-4 text-right">',
							  '<label total class="statsItemTotal" title="{{ totalDescription }}">',
								  '{{ total }}',
							  '</label>',
							  '<label trend class="statsItemTrend">',
							  '</label>',
						  '</div>',
						  '<div class="col-4" line_chart class="statsLineChart">',
						  '</div>',*/
					  '</div>',

					  '<div class="row d-flex justify-content-start align-items-center">',
							'<div class="col-6 text-left">',
								'<label total class="statsItemTotal" title="{{ totalDescription }}">',
									'{{ total }}',
								'</label>',
								'<label trend class="statsItemTrend">',
								'</label>',
							'</div>',
							'<div class="col statsLineChart" line_chart>',
						    '</div>',
					  '</div>',
					  '<button view_details class="viewStatsDetailsButton buttonToggle border-0" >',
						  '<i class="far fa-chevron-down"></i>',
					  '</button>',
				  '</div>',
				  '<div details class="statsItemDetails" style="display:none;">',
					  '<div sub_list class="statsSubContent">',
					  '</div>',
					  '<div compare_chart>',
					  '</div>',
					  '<div multiline_chart class="statsMultiLineChart">',
					  '</div>',
				  '</div>',
			  '</div>'

					  /*'<div class="row">',
						  '<div class="col-12">',
							  '<button view_details class="viewStatsDetailsButton buttonColorLightblue" >',
								  '<i class="buttons-icons-zoomin"></i>',
							  '</button>',
						  '</div>',
					  '</div>',

					  '<button view_details class="viewStatsDetailsButton buttonToggle" >',
						  '<i class="far fa-chevron-down"></i>',
					  '</button>',
				  '</div>',
				  '<div details class="statsItemDetails" style="display:none;">',
					  '<div sub_list class="statsSubContent">',
					  '</div>',
					  '<div compare_chart>',
					  '</div>',
					  '<div multiline_chart class="statsMultiLineChart">',
					  '</div>',
				  '</div>',
			  '</div>'*/
		  ),
		  //addresses status
		  addresses_status_container : ''.concat(
			  '<div class="batchesAddressStatusContainer card">',
				  '<div class="batchesAddressStatusHeader d-flex justify-content-start align-items-center">',
				  		'<i class="fas fa-equals">','</i>',
					  '<label class="batchesAddressStatusTitle">{{ title }}</label>',
				  '</div>',
				  '<div addresses_status_body class="batchesAddressStatusBody">',
				  '</div>',
			  '</div>'
		  ),
		  addresses_status_state : ''.concat(
			  '<div class="batchesAddressStatusState d-flex justify-content-start">',
				  '<label class="batchesAddressStatusStateName">{{ name }}: </label>',
				  '<label class="batchesAddressStatusStateValue ">',
					  '{{ value }}',
					  '{{ #percentage }} ({{ percentage }}%){{ /percentage }}',
				  '</label>',
			  '</div>'
		  )
	  };
  
	  if ( !inConcert.templates.batch_management ) {
		  inConcert.templates.batch_management = templates;
	  }
  }());
  
  