local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local helpers = require "helpers"
m_htmlRender = nil
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"

local LOG_CONTEXT = "campaign_analytics:contacts"

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	m:dispatch_get(utils.JsonReply..LogRequest..LastContacts, prefix.."/last_contacts/(.+)")
end

local function OeMethod(web, method, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local mockIndicators = inconcert.environment.apps.campaign_analytics.mock_indicators
	if mockIndicators then
		request.mockIndicators = true
	end
	local requestStr = json.encode(request)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local client = m_outboundEngineClient.GetClient()
	local result = client[method](client, requestStr)
	if result.result ~= "OK" then
		web.status = 500
		LogError("[%s:%s] %s %s - %s", LOG_CONTEXT, method, userId, requestStr, json.encode(result))
	end
	return result
end

local LAST_CONTACTS_FIELDS = {
	"LastContactDate",
	"LastAddress",
	"BatchId",
	"LastInteractionCode",
	"LastLowLevelCode",
	"LastAgent",
	"State",
	"StateCause",
	"ScheduleDate",
	"Name",
	"Id",
}

local function MakeLastContactsFilter(campaignId, query)
	local filter = {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		skillId = query.skill or "",
		batchId = query.batch or "",
		size = query.size or 100,
		sortTerms = { { name = "LastContactDate", order = "desc"  } },
		fields = LAST_CONTACTS_FIELDS
	}
	return filter
end

function LastContacts(web, campaignId)
	return OeMethod(web, "SearchContacts", MakeLastContactsFilter(campaignId, web.input or {}))
end
