/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	const CONTAINER_ID = "campaign_analytics_dashboard";
	const MODULE_NAME = "icc-campaign-analytics";

	class CampaignAnalyticsApp {
		constructor() {
			this.loaded = false;
		}

		$container = () => $(`#${CONTAINER_ID}`);

		can_show = () => inConcert.permissions.to_application("campaign_analytics");

		on_leave = () => {
			this.$container().hide();
			$("#inboxMainContainerTop").show();
			this.notify_leave();
		}

		notify_leave = () => {
			if (!this.loaded) return;

			this.module.leave();
		}

		notify_focus = () => {
			this.module.focus();
		}

		render = async (callback = $.noop) => {
			try {
				if (this.loaded)
					this.notify_focus();
				else
					await this.load_module();
				$("#inboxMainContainerTop").hide();
				this.$container().show();
			} catch(err) {
				inConcert.app.m_mainWindow.showBannerMessage({
					message: err,
					timeout: 5000,
				});
			}
			callback();
		}

		load_module = async () => {
			this.module = document.createElement(MODULE_NAME);
			this.module.setAttribute("data-icc-dev-component",true);
			this.$container()[0].appendChild(this.module);
			await this.wait_for_module();
			this.bootstrap();
			this.loaded = true;
		};

		wait_for_module = async () => new Promise((resolve, _reject) => {
			const interval = window.setInterval(() => {
				if (!this.module.bootstrap)
					return;

				clearInterval(interval);
				resolve();
			});
		});

		bootstrap = () => this.module.bootstrap();
	}

	inConcert.CampaignAnalyticsApp = CampaignAnalyticsApp;
}());