module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local apiLuis = require("shared.cognitive_services.luis.luis").GetAPI() -- rocka

m_htmlRender = nil
m_appLocation = nil

-------------------------------------------------------
-- APPS
-------------------------------------------------------

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/load_versions/3a080c31-a44d-4b67-b586-ca51b0e15446/0

	Metodo: GET

		Parametros:
			1) App ID : d469528f-a521-4035-a9ea-36a346b83df8
			2) encodedPaginated: {page: 0, rows_per_page: 10};
				Atributo "page" = (-1 / 0 o N) - El numero de row en la cual tiene que comenzar el paginado. (-1 es sin paginado).
				Atributo "rows_per_page" = cantidad de filas por pagina.
				Si viene un string vacío, en lugar del json, no tiene paginado.

	Respuesta:
		Todos las versiones de la app, o los de la pagina solicitada.
]]--
local function LoadAppVersions (web, appId, encodedPaginated)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAppVersions] User '%s' Loading cognitive services. APPLICATION VERSIONS on appId '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, vcc, encodedPaginated)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, paginated = pcall(json.decode, encodedPaginated)
	if not ok then
		paginated = ""
	end
	local success, response = apiLuis:LoadAppVersions(appId, paginated, vcc)

	if success then
		local encodedAppVersions = response
		local appVersions = json.decode(encodedAppVersions)
		local total = utils.GetRowCount(appVersions.value or appVersions)

		LogInfo("[COGNITIVE_SERVICES:LoadAppVersions] Loaded %d APPLICATION VERSIONS on appId '%s' for vcc '%s'.", total, appId, vcc)
		return encodedAppVersions
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAppVersions] Error loading cognitive services. APPLICATION VERSIONS on appId '%s' for vcc: '%s'. Error: '%s'", appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/get_version/3a080c31-a44d-4b67-b586-ca51b0e15446/0.1

	Metodo: GET
		Parametros:
			1) App Id: 3a080c31-a44d-4b67-b586-ca51b0e15446
			2) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		Status:200 OK
			{
				"version": "0.1",
				"createdDateTime": "2017-01-31T16:38:06Z",
				"lastModifiedDateTime": "2017-01-31T18:04:35Z",
				"lastTrainedDateTime": "2017-01-31T17:05:25Z",
				"lastPublishedDateTime": "2017-01-31T17:05:37Z",
				"endpointUrl": "DummyUrl",
					"assignedEndpointKey": {
					"SubscriptionKey": "",
					"SubscriptionName": ""
				},
				"externalApiKeys": {},
				"intentsCount": 3,
				"entitiesCount": 2,
				"endpointHitsCount": 0
			}
]]--
local function GetAppVersion (web, appId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetAppVersion] User '%s' Retrieving cognitive services 'APPLICATION VERSION' on app '%s' versionId '%s'.", userId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

--[[
TODO: Probar el API contra el propio de esta API URI de get app version.
El codigo que esta comentado y dejar de realizar la (**) PISADA de abajo

]]--
	-- local success, response = apiLuis.GetAppVersion(appId, versionId)
	-- if success then
		-- return response
	-- else
		-- web.status = 400
		-- LogError("[COGNITIVE_SERVICES:GetAppVersion] Error Retrieving cognitive services 'APPLICATION VERSION' on app '%s' versionId '%s'. Error: '%s'", appId, versionId, json.encode(response))
		-- return {result = "NOT_FOUND", cognitiveServiceError = response}
	-- end

-- (**) PISADA - INICIO
	local paginated = ""
	local versions = LoadAppVersions({}, appId, paginated)
	if type(versions) == "table" then
		-- Si viene una TABLE es un ERROR, ya que es lo que retorna la FUNCION/URL
		web.status = 400
		local err = versions
		LogError("[COGNITIVE_SERVICES:GetAppVersion] Error Retrieving cognitive services 'APPLICATION VERSION' on app '%s' versionId '%s'. Error: '%s'", appId, versionId, json.encode(err))
		return err
	end

	-- Si viene un STRING es OK, idem comentario al del Error.
	versions = json.decode(versions)
	local version
	for _, v in ipairs(versions.value or versions) do
		if v.version == versionId or v.label == versionId then
			version = v
			break
		end
	end

	if not version then
		LogWarning("[COGNITIVE_SERVICES:GetAppVersion] Error Retrieving cognitive services 'APPLICATION VERSION' on app '%s' versionId '%s'. Error: Version NOT FOUND.", appId, versionId)
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	version.lastDeployedDateTime = versions.lastDeployedDateTime 
	version.endpointHitsCount = versions.endpointHitsCount 
	version.modelId = versions.modelId 
	version.intentsCount = versions.intentsCount 
	version.intents = versions.intents 
	version.entitiesCount = versions.entitiesCount 
	version.entities = versions.entities 
	version.utterancesCount = versions.utterancesCount 
	version.utterances = versions.utterances 

-- (**) PISADA - FIN
	return json.encode(version)
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/get/17c545c3-98cd-4e80-8201-15e8d24c4234

	Metodo: GET
		Parametros:
			1) App Id: 17c545c3-98cd-4e80-8201-15e8d24c4234

	Respuesta:
		Status:200 OK

		{"id":"17c545c3-98cd-4e80-8201-15e8d24c4234","name":"App 2","description":"Rocio 2","culture":"es-es","usageScenario":"","domain":"","versionsCount":1,"createdDateTime":"2017-05-16T13:44:12Z","endpoints":{},"endpointHitsCount":0}

]]--
local function GetApp (web, appId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetApp] User '%s' Retrieving cognitive services app '%s'.", userId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local function AddInfoAppVersion(encodedApp)
		local app = json.decode(encodedApp)
		local activeVersion = app.activeVersion
		local api_version = app.api_version

		local activeVersionInfo = GetAppVersion ({}, appId, activeVersion, vcc)
		if type(activeVersionInfo) == "table" then
			LogInfo("[COGNITIVE_SERVICES:GetApp] Unable to Add Active Version to App '%s'.", appId)
			return encodedApp
		end
		activeVersionInfo = json.decode(activeVersionInfo)
		app.activeVersionInfo = activeVersionInfo
		encodedApp = json.encode(app)
		return encodedApp
	end

	-- 1 - Lo busco contra la BD
	local encodedApp = apiLuis.GetAppFromDataBase(vcc, appId)
	LogDebug("1 [COGNITIVE_SERVICES:GetApp] From DATABASE appId: '%s', app: '%s'.", appId, encodedApp)
	if encodedApp then
		local encodedApp = AddInfoAppVersion(encodedApp)
		return encodedApp
	end

	-- 2 - Si no se encuentra en la BD de inconcert, se lo pide al API de LUIS.
	local success, response = apiLuis:GetApp(appId, vcc)
	LogDebug("2 [COGNITIVE_SERVICES:GetApp] From API appId: '%s', success: '%s' app: '%s'.", appId, success, response)
	if success then
		local encodedApp = AddInfoAppVersion(response)
		return encodedApp
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:GetApp] Error Retrieving cognitive services app: '%s' for vcc: '%s'. Error: '%s'", appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/load_all/

	Metodo: POST
		Parametros:
			{}

	Respuesta:
		Status:200 OK
			{
			  "rows": {
				"6749b29c-9065-4b98-990e-174efa9effbc": {
				  "id": "6749b29c-9065-4b98-990e-174efa9effbc",
				  "culture": "es-es",
				  "description": "Application2 Rocio",
				  "name": "Application2"
				},
				"d469528f-a521-4035-a9ea-36a346b83df8": {
				  "ID": "App 1",
				  "Culture": "es-es",
				  "Name": "App 1",
				  "Description": "Rocio 1"
				}
			  },
			  "total_rows": 2,
			  "last": ""
			}
]]--

local function LoadApps (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[COGNITIVE_SERVICES:LoadApps] 1 User '%s' Loading all apps for vcc '%s'. postData: '%s'", userId, vcc, web.input["post_data"])

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local apps, totalUnfilteredRows = apiLuis.LoadApps(vcc)
	local totalAdded = 0
	local filteredRows = {}
	local postData = web.request

	LogInfo("[COGNITIVE_SERVICES:LoadApps] 2 - Filtering rows")

	for appId, app in pairs(apps) do
		app.Id = appId
		app.Name = app.name or appId
		if utils.MatchFilter(app, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, app)
		end
	end

	LogInfo("[COGNITIVE_SERVICES:LoadApps] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	postData.__order_attr_name = "name"
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[COGNITIVE_SERVICES:LoadApps] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/add/

	Metodo: POST
		Data:
			{"name":"App EMMA & ROCIO","culture":"es-es"}

	Respuesta:
		Status:200 OK
		{"appId":"70bffde2-1369-49ac-a200-77f0fef50b22"}
]]--
local function AddApp (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:AddApp] Error Adding cognitive services app for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name or not info.culture then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddApp] Error Adding cognitive services app for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:AddApp] User '%s' Adding cognitive services app.", userId)

	local success, response = apiLuis:AddApp(info, vcc)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {appId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddApp] Error Adding cognitive services app for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end


--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/import/

	Metodo: POST
		Data:
			{"name":"App EMMA & ROCIO","culture":"es-es"}

	Respuesta:
		Status:200 OK
		{"appId":"70bffde2-1369-49ac-a200-77f0fef50b22"}
]]--
local function ImportApp (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:ImportApp] Error importing cognitive services app for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name or not info.culture then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:ImportApp] Error importing cognitive services app for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:ImportApp] User '%s' importing cognitive services app.", userId)

	local success, response = apiLuis:ImportApp(vcc,info, web.GET.optionalName)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {appId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:ImportApp] Error importing cognitive services app for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/rename/

	Metodo: PUT

		Data:
		{
			"id": "string", -- requerido
			"name": "string", -- requerido
			"description": "string", -- optional
		}

	Respuesta:
		Status:200 OK
		[]
]]--
local function RenameApp (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:RenameApp] Error Renaming cognitive services app for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.id or not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameApp] Error Renaming cognitive services app for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:RenameApp] User '%s' Renaming cognitive services app.", userId)

	local success, response = apiLuis:RenameApp(info, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameApp] Error Renaming cognitive services app for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/delete/

	Metodo: POST
		Data:
			["48b866be-f747-45bb-8a80-332a5cf6a0f8", "f23db312-1ff6-47d5-be25-f6a739daf5ee", ...]

	Respuesta:
		Status:200 OK
		[]
]]--
local function DeleteApps (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local appsToDelete = web.request

	LogInfo("[COGNITIVE_SERVICES:DeleteApps] User '%s' Deleting cognitive services %d apps '%s'.", userId, #appsToDelete, web.input["post_data"])

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response

	for _, appId in ipairs(appsToDelete) do
		success, response = apiLuis:DeleteApp(appId, vcc)
		if not success then
			web.status = 400
			LogError("[COGNITIVE_SERVICES:DeleteApps] Error Deleting cognitive services app: '%s' for vcc: '%s'. Error: '%s'", appId, vcc, json.encode(response))
			return {result = "NOT_FOUND", cognitiveServiceError = response}
		end
	end

	return response -- success
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/publish/

	Metodo: POST
		Data:
			{
				"appId": "3a080c31-a44d-4b67-b586-ca51b0e1544",
				-- opcionales
				"versionId": "0.1",
				"isStaging": false,
				"subscriptionKey": "d24518b086a248c4ae64afd6e3eb475c"
			}

	Respuesta:
		Status:200 OK
			{
				"versionId": null,
				"isStaging": false,
				"endpointUrl": "https://westus.api.cognitive.microsoft.com/luis/v2.0/apps/3a080c31-a44d-4b67-b586-ca51b0e15446",
				"endpointRegion": "westus",
				"publishedDateTime": "2017-05-18T14:46:45Z"
			}

NOTA:
	Si en los parametros no llega el atributo "subscriptionKey", solo se publicará la aplicación, si ya tiene asociada una subscriptionKey.

]]--
local function PublishApp (web)
	--LogDebug("[COGNITIVE_SERVICES:PublishApp] post_data: %s", web.input["post_data"])

	local vcc = inconcert.authUser.virtualContactCenter

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:PublishApp] Error Publish cognitive services app for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.subscriptionKey then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:PublishApp] Error Publish cognitive services app for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[COGNITIVE_SERVICES:PublishApp] User '%s' Publish cognitive services Info '%s' on vcc '%s'.", userId, web.input["post_data"], vcc)

	info.timezone = info.timezone or "WET" -- 00

	local publishInfo = {
		appId           = info.appId,
		subscriptionKey = info.subscriptionKey,
		versionId       = info.versionId,
		isStaging       = info.isStaging or false,
		verbose         = info.verbose,
		timezone        = info.timezone,
		language        = info.language
	}
	local success, response = apiLuis:PublishApp(info.appId, publishInfo, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:PublishApp] Error Publish cognitive services app '%s' versionId '%s'. Error: '%s'", info.appId, info.versionId, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/get_train/3a080c31-a44d-4b67-b586-ca51b0e15446/0.1

	Metodo: GET
		Parametros:
			1) App Id: 3a080c31-a44d-4b67-b586-ca51b0e15446
			2) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		Status:200 OK
			[
			  {
				"modelId": "55356f76-40e6-4ba1-bc56-7a4dc2802aad",
				"details": {
				  "statusId": 0,
				  "status": "Success",
				  "exampleCount": 24,
				  "trainingDateTime": "2017-05-11T16:40:31Z"
				}
			  },
			  {
				"modelId": "b2834af5-19b2-4d9a-a00f-ae1fc539198a",
				"details": {
				  "statusId": 0,
				  "status": "Success",
				  "exampleCount": 24,
				  "trainingDateTime": "2017-05-11T16:35:08Z"
				}
			  },
			  {
				"modelId": "6618e7fd-19dd-4303-a1aa-af5fc20151da",
				"details": {
				  "statusId": 0,
				  "status": "Success",
				  "exampleCount": 24,
				  "trainingDateTime": "2017-05-11T16:35:08Z"
				}
			  }
			]


**********************************************
ATENCION!!!
-----------
	Esta funcion NO debe invocarse, sin que previamente se haya realizado un TRAIN (POST) de la APP.
	Ya que sino los scores de los utterances NO cambian.
	Ref #8900
**********************************************
]]--
local function GetTrainApp (web, appId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetTrainApp] User '%s' Retrieving cognitive services TRAIN app '%s' versionId '%s'.", userId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:GetTrainApp(appId, versionId)
	if success then
		return response
	else
		--[[
			#7167 cuando NO es un error retorno un code 202 (Accepted), ya que sino del lado del cliente lo ve como error, y no es un error.
			El API de LUIS en la respuesta no me identifica el tipo de mensaje, entonces tengo que realizarlo así.
			https://es.wikipedia.org/wiki/Anexo:C%C3%B3digos_de_estado_HTTP)

			TODO: en algún momento poder dirimir la respuesta, si es que LUIS retorna un valor para poder indentificar el mismo.
		]]--
		web.status = 202 -- Accepted
--		web.status = 400
		LogWarning("[COGNITIVE_SERVICES:GetTrainApp] Error Retrieving cognitive services TRAIN app '%s' versionId '%s'. Error: '%s'", appId, versionId, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/train/

	Metodo: POST
		Data:
			{
				"appId":"3a080c31-a44d-4b67-b586-ca51b0e15446",
				"versionId":"0.1" -- opcional (por defecto 0.1)
			}

	Respuesta:
		Status:200 OK
			null
]]--
local function TrainApp (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local api_version 

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:TrainApp] Error cognitive services TRAINING app for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:TrainApp] Error cognitive services TRAINING app for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	-- Estados posibles los cuales retorna el API en el Train / GetTrain
	local statuses = {
		["0"] = {
			name = "Success",
			action = "TRAINED"
		},
		["1"] = {
			name = "Fail",
			action = "ERROR"
		},
		["2"] = {
			name = "UpToDate",
			action = "TRAINED"
		},
		["3"] = {
			name = "InProgress",
			action = "RETRY"
		},
		["9"] = {
			name = "Queued",
			action = "RETRY"
		}
	}

	local function ValidateTrainStatus (trainResponse)
		assert(trainResponse)

		trainResponse = trainResponse[1]
		assert(trainResponse)

		api_version = trainResponse.api_version
		info.operationLocation = trainResponse.operationLocation

		trainResponse = type(trainResponse) == "string" and json.decode(trainResponse) or trainResponse

		LogInfo("[COGNITIVE_SERVICES:TrainApp] Validating TrainStatus trainResponse: '%s'.", json.encode(trainResponse))

		assert(trainResponse.statusId)

		local isOk = false
		local mustRetry = false

		local currentAction = statuses[tostring(trainResponse.statusId)]
		if not currentAction then
			LogError("[COGNITIVE_SERVICES:TrainApp] Unrecognized train status: '%s'. Inconcert Implemented Actions: '%s'.", json.encode(trainResponse), json.encode(statuses))
			isOk = false
			mustRetry = false
		else
			LogInfo("[COGNITIVE_SERVICES:TrainApp] App Id: '%s', trainResponse: '%s', Action: '%s'.", info.appId, trainResponse, currentAction.action)
			if currentAction.action == "TRAINED" then
				isOk = true

			elseif currentAction.action == "ERROR" then
				isOk = false
				mustRetry = false

			elseif currentAction.action == "RETRY" then
				-- Tengo que reintentar
				isOk = false
				mustRetry = true
			end
		end

		LogInfo("[COGNITIVE_SERVICES:TrainApp] Validating TrainStatus Result - isOk: '%s', mustRetry: '%s', trainResponse: %s.", isOk, mustRetry, json.encode(trainResponse))
		return isOk, mustRetry, trainResponse
	end

	LogInfo("[COGNITIVE_SERVICES:TrainApp] User '%s' cognitive services TRAINING app '%s' versionId '%s'.", userId, info.appId, info.versionId)

	local success, firstTrainResponse
	
	if info.operationLocation then
		success, firstTrainResponse = apiLuis:GetTrainStatus(info, vcc, info.operationLocation) 
	else
		success, firstTrainResponse = apiLuis:TrainApp(info, vcc) 
	end

	if success and firstTrainResponse then
		-- Luego de publicar x primera vez la App tengo que realizar un TrainApp de la misma ya que sino el Publish posterior falla.
		-- Estimo que pueder ser un falla del API de LUIS.
		-- #7169
		-- TODO: ver de quitar si es que se soluciona el problema del API
		--

		local isOk, mustRetry, trainResponse = ValidateTrainStatus(firstTrainResponse)
		if isOk then
			LogInfo("[COGNITIVE_SERVICES:TrainApp] User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, encodedTrainResponse)
			return trainResponse or {}
		end
		if not mustRetry then
			LogError("[COGNITIVE_SERVICES:TrainApp] Error User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, encodedTrainResponse)
			return trainResponse or {}
		end

		-- Voy por los reintentos
		require "socket"
		local isOk = false
		local mustRetry = false
		local timeoutSec = 5
		local topRetries = 5
		local retry = 1

		-- Si el Status del Train de la app se encuentra "InProgress" o "Queued" reintenta el TrainApp N veces con un lapso de 1 segundo entre reintentos.
		local trainStatus = {statusId = "1", name = "Fail", modelId = ""}

		while retry <= topRetries do
			
			LogInfo("[COGNITIVE_SERVICES:TrainApp] Retry ... waiting %d seconds", timeoutSec)
			socket.sleep(timeoutSec) -- espero antes de comenzar el loop de reintentos

			LogInfo("[COGNITIVE_SERVICES:TrainApp] Executing POST Train for completed. Retry '%d' of '%d'.", retry, topRetries)

			local success, trainResponse

			if api_version == "2022-10-01-preview" then  
				success, trainResponse = apiLuis:GetTrainStatus(info, vcc, info.operationLocation) 
			else
				success, trainResponse = apiLuis:TrainApp(info, vcc)
			end
			
			if success then
				success, mustRetry, trainResponse = ValidateTrainStatus(trainResponse)
				trainStatus = trainResponse

				if success then
					LogInfo("[COGNITIVE_SERVICES:TrainApp] Succes cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, json.encode(trainResponse))
					isOk = true
				else
					if not mustRetry then
						LogError("[COGNITIVE_SERVICES:TrainApp] Error User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, json.encode(trainResponse))
						isOk = false
					end
				end
			else
				LogError("[COGNITIVE_SERVICES:TrainApp] Error User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, json.encode(trainResponse or {}))
				mustRetry = true
			end

			if isOk then
				break
			end
			if not mustRetry then
				break
			end

			retry = retry + 1
		end

		if isOk then
			LogInfo("[COGNITIVE_SERVICES:TrainApp] User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, json.encode(trainStatus))
			return trainStatus
		elseif mustRetry then
			LogError("[COGNITIVE_SERVICES:TrainApp] Error User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Timeout '%s'.", userId, info.appId, info.versionId, json.encode(trainStatus))
			return trainStatus
		else
			LogError("[COGNITIVE_SERVICES:TrainApp] Error User '%s' cognitive services TRAINING app '%s' versionId '%s'. Trained Status '%s'.", userId, info.appId, info.versionId, json.encode(trainStatus))
			return trainStatus
		end
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:TrainApp] Error TRAINING cognitive services app '%s' versionId '%s' for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, firstTrainResponse)
		return {result = "NOT_FOUND", cognitiveServiceError = firstTrainResponse}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/load_cultures/

	Metodo: GET
		Parametros:
			...

	Respuesta:
		Status:200 OK
		[
			{
				"name": "English",
				"code": "en-us"
			},
			{
				"name": "Chinese",
				"code": "zh-cn"
			},
			..
			..
		]
]]--
local function LoadAppCultures (web, api_version)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAppCultures] User '%s' Loading cognitive services app cultures. Api Version: '%s'", userId, api_version)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadAppCultures(vcc, api_version)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAppCultures] Error Loading cognitive services app cultures for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

local function LoadAppCulturesForIntents (web, api_version)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAppCulturesForIntents] User '%s' Loading cognitive services app cultures. Api Version: '%s'", userId, api_version)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadAppCulturesForIntents(vcc, api_version)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAppCulturesForIntents] Error Loading cognitive services app cultures for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

local function LoadAvailableApiVersions (web) 
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAvailableApiVersions] User '%s' Loading available api versions.", userId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadAvailableApiVersions(vcc) 
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAvailableApiVersions] Error Loading available api versions for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/suggested_utterances/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/el utterance/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) UtteranceId Name : El utterance name
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[]
]]--
local function DeleteAppSuggestedUtterance (web, appId, utteranceName, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppSuggestedUtterance] User '%s' deleting cognitive services UTTERANCE NAME: '%s' on app: '%s' versionId: '%s'.", userId, utteranceName, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis.DeleteAppSuggestedUtterance(appId, utteranceName, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppSuggestedUtterance] Error deleting cognitive services UTTERANCE NAME: '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", utteranceName, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

-------------------------------------------------------
-- INTENTS - APPS
-------------------------------------------------------

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/load_all/d469528f-a521-4035-a9ea-36a346b83df8/0/0.1

	Metodo: GET

		Parametros:
			1) App ID : d469528f-a521-4035-a9ea-36a346b83df8
			2) encodedPaginated: {page: 0, rows_per_page: 10};
				Atributo "page" = (-1 / 0 o N) - El numero de row en la cual tiene que comenzar el paginado. (-1 es sin paginado).
				Atributo "rows_per_page" = cantidad de filas por pagina.
				Si viene un string vacío, en lugar del json, no tiene paginado.
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[{"id":"d04834af-1e1b-44f1-884e-e454e7cda020","name":"INTENTS1","typeId":0,"readableType":"Intent Classifier"},{"id":"a09596b6-9fdf-4037-8c93-ef3fddf0be05","name":"INTENTS2","typeId":0,"readableType":"Intent Classifier"},{"id":"47a359af-2ce8-45db-a7f0-ff3fc6e4367c","name":"None","typeId":0,"readableType":"Intent Classifier"}]

		Todos los registros, o los de la pagina solicitada.
]]--
local function LoadAllAppIntents (web, appId, encodedPaginated, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAllAppIntents] User '%s' Loading cognitive services. All APP INTENTS on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, paginated = pcall(json.decode, encodedPaginated)
	if not ok then
		paginated = ""
	end
	local success, response = apiLuis:LoadAllAppIntents(appId, versionId, paginated, vcc)

	if success then
		local encodedAppIntents = response
		local appIntents = json.decode(encodedAppIntents)
		local total = utils.GetRowCount(appIntents)

		LogInfo("[COGNITIVE_SERVICES:LoadAllAppIntents] Loaded %d APP INTENTS on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)

		local function lowerName(intent)
			return intent and intent.name:lower() or "" 
		end

		-- 1 - Obtengo el index en el array del INTENT NONE
		local indexNone
		for index, intent in ipairs(appIntents) do
			if lowerName(intent) == "none" then
				indexNone = index
				break
			end
		end
		-- 2 - Remuevo el INTENT NONE del array y me quedo con el, para luego insertarlo en la primer posicion
		local intentNone
		if indexNone then
			intentNone = table.remove(appIntents, indexNone)
		end
		-- 3 - Ordeno el array alfabetico (sin el INTENT NONE)
		table.sort(appIntents, function (intentA, intentB)
			local aName = lowerName(intentA)
			local bName = lowerName(intentB)
			return aName < bName
		end)
		-- 4 - Se inserta en el array ordenado, el INTENT NONE en la primer posicion
		if intentNone then
			table.insert(appIntents, 1, intentNone)
		end

		encodedAppIntents = json.encode(appIntents)

		return encodedAppIntents
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAllAppIntents] Error loading cognitive services. All APP INTENTS on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/get/d469528f-a521-4035-a9ea-36a346b83df8/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: GET

		Parametros:
			1) App ID    : d469528f-a521-4035-a9ea-36a346b83df8
			2) Intent ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		{"id":"d04834af-1e1b-44f1-884e-e454e7cda020","name":"INTENTS1","typeId":0,"readableType":"Intent Classifier"}
]]--
local function GetAppIntent (web, appId, intentId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetAppIntent] User '%s' getting cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s'.", userId, intentId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:GetAppIntent(appId, intentId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:GetAppIntent] Error getting cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", intentId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/create/

	Metodo: POST

		DATA:
			{
				"appId":"d469528f-a521-4035-a9ea-36a346b83df8",
				"name":"Intents 2.0",
				"versionId":"0.1" -- opcional (por defecto 0.1)
			}

	Respuesta:
		{"intentId":"bdf01c0a-67c9-4b39-9ecf-f7185d0120b7"}
]]--
local function CreateAppIntent (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreateAppIntent] Error Creating cognitive services APP INTENTS for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppIntent] Error Creating cognitive services APP INTENTS for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreateAppIntent] User '%s' creating cognitive services APP INTENTS on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	local success, response = apiLuis:CreateAppIntent(info, vcc)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {intentId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppIntent] Error creating cognitive services APP INTENTS on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/rename/d469528f-a521-4035-a9ea-36a346b83df8/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: PUT

		Parametros:
			1) App ID    : d469528f-a521-4035-a9ea-36a346b83df8
			2) Intent ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

		Data:
		{
			"name": "string" -- requerido
		}

	Respuesta:
		[]
]]--
local function RenameAppIntent (web, appId, intentId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:RenameAppIntent] Error renaming cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Request Information '%s'", intentId, appId, versionId, vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppIntent] Error renaming cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Posted Information '%s'", intentId, appId, vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:RenameAppIntent] User '%s' renaming cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s'.", userId, intentId, appId, versionId)

	info.appId = appId
	info.intentId = intentId
	info.versionId = versionId

	local success, response = apiLuis:RenameAppIntent(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppIntent] Error renaming cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", intentId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/delete/d469528f-a521-4035-a9ea-36a346b83df8/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : d469528f-a521-4035-a9ea-36a346b83df8
			2) Intent ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[]
]]--
local function DeleteAppIntent (web, appId, intentId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppIntent] User '%s' deleting cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s'.", userId, intentId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppIntent(appId, intentId, versionId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppIntent] Error deleting cognitive services APP INTENTS '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", intentId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/utterances/load_all/d469528f-a521-4035-a9ea-36a346b83df8/INTENT_NAME/0.1

	Metodo: GET

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Intent Name : Intent Name 1
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
	[
		{"entityPredictions":[],"intentPredictions":[],"tokenizedText":["1","hola","emma","como","estas","?"],"id":-8010780,"text":"1 hola emma como estas ?","intentLabel":"INTENTS2","entityLabels":[{"entityName":"ENTITY2","endTokenIndex":4,"startTokenIndex":4}]},

		{"entityPredictions":[],"intentPredictions":[],"tokenizedText":["6","hola","rocio","como","estas","?"],"id":-8001776,"text":"6 hola rocio como estas ?","intentLabel":"INTENTS1","entityLabels":[{"entityName":"ENTITY2","endTokenIndex":1,"startTokenIndex":1}]},
		..
		..
	]
]]--
local function LoadUtterancesByAppIntent (web, appId, intentName, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadUtterancesByAppIntent] User '%s' getting cognitive services APP UTTERANCES INTENTS '%s' on app: '%s' versionId: '%s'.", userId, intentName, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadUtterancesByAppIntent(appId, intentName, versionId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadUtterancesByAppIntent] Error getting cognitive services APP UTTERANCES INTENTS '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", intentName, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/utterances/suggested/load_all/d469528f-a521-4035-a9ea-36a346b83df8/INTENT_ID/0.1

	Metodo: GET

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Intent Id : 4d4f891a-dd4a-480d-b15f-b78e77189023
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[
			{
				"text": "hello, world!",
				"tokenizedText": [
					"hello",
					",",
					"world",
					"!"
				],
				"intentPredictions": [
					{
						"name": "None",
						"score": 0.63
					},
					{
						"name": "GetWeather",
						"score": 0
					},
					{
						"name": "BookFlight",
						"score": 0
					}
				],
				"entityPredictions": []
			}
		]
]]--
local function LoadSuggestedUtterancesByAppIntent (web, appId, intentId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadSuggestedUtterancesByAppIntent] User '%s' getting cognitive services APP SUGGESTED UTTERANCES INTENTS '%s' on app: '%s' versionId: '%s'.", userId, intentId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadSuggestedUtterancesByAppIntent(appId, versionId, intentId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadSuggestedUtterancesByAppIntent] Error getting cognitive services APP SUGGESTED UTTERANCES INTENTS '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", intentName, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/utterances/add/

	Metodo: POST

		DATA:
			{
			-- Requeridos -----
			  "appId": "d469528f-a521-4035-a9ea-36a346b83df8",
			  "utterance": "1 Hola Me puede informar el saldo ?",
			  "intentName": "IntentsName1",

			-- Opcional -----
			  "entityLabels": [
				{
				  "entityName": "EntityName1",
				-- Inicio y final de la palabra "puede"
				  "startCharIndex": 3,
				  "endCharIndex": 7
				}
			  ]
			}

	Respuesta:
		{"UtteranceText":"1 hola me puede informar el saldo ?","ExampleId":-8038796}
]]--
local function AddUtteranceAppIntent (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:AddUtteranceAppIntent] Error Adding cognitive services UTTERANCE TO INTENT on vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.utterance or not info.intentName then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddUtteranceAppIntent] Error Adding cognitive services UTTERANCE TO INTENT on vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:AddUtteranceAppIntent] User '%s' Adding cognitive services UTTERANCE TO INTENT '%s' on app: '%s' versionId: '%s'.", userId, info.intentName, info.appId, info.versionId)

	local success, response = apiLuis:AddUtteranceAppIntent(info, vcc)
	require "socket"

	if success then
		response = table.concat(response)
		response = json.decode(response)
		
		local TIMEOUT_SEC = 5

		socket.sleep(TIMEOUT_SEC)

		local success, utterance = apiLuis:GetUtteranceByAppIntent(info.appId, info.intentName, response.ExampleId, info.versionId, vcc)

		if not success then
			web.status = utterance.code
			LogError("[COGNITIVE_SERVICES:AddUtteranceAppIntent] Error Adding cognitive services UTTERANCE TO INTENT '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.intentName, info.appId, info.versionId, vcc, json.encode(utterance))
			return {result = "NOT_FOUND", cognitiveServiceError = utterance}
		end
		local encodedUtterance = json.encode(utterance)
		--LogDebug("3 [AddUtteranceAppIntent] (GetUtteranceByAppIntent) encodedUtterance: %s", encodedUtterance)

		return encodedUtterance
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddUtteranceAppIntent] Error Adding cognitive services UTTERANCE TO INTENT '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.intentName, info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/intents/utterances/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/-8010780/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) UtteranceId ID : -8010780 (Example ID)
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[]
]]--
local function DeleteAppUtterance (web, appId, utteranceId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppUtterance] User '%s' deleting cognitive services UTTERANCE ID: '%s' on app: '%s' versionId: '%s'.", userId, utteranceId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppUtterance(appId, utteranceId, versionId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppUtterance] Error deleting cognitive services UTTERANCE ID: '%s' on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", utteranceId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

-------------------------------------------------------
-- ENTITIES - APPS (MACHINE LEARNED, COMPOSITE, HIERARCHICAL)
-------------------------------------------------------

--[[
	Esta función realiza 3 request para obtener las 3 listas por separado.

	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/load_all/d469528f-a521-4035-a9ea-36a346b83df8/0/0.1

	Metodo: GET

		Parametros:
			1) App ID : d469528f-a521-4035-a9ea-36a346b83df8
			2) encodedPaginated: {page: 0, rows_per_page: 10};
				Atributo "page" = (-1 / 0 o N) - El numero de row en la cual tiene que comenzar el paginado. (-1 es sin paginado).
				Atributo "rows_per_page" = cantidad de filas por pagina.
				Si viene un string vacío, en lugar del json, no tiene paginado.
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		Todos los registros, o los de la pagina solicitada.
]]--
local function LoadAllAppEntities (web, appId, encodedPaginated, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] User '%s' Loading cognitive services. All APP ENTITIES (MACHINE LEARNED, COMPOSITE, HIERARCHICAL) on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, paginated = pcall(json.decode, encodedPaginated)
	if not ok then
		paginated = ""
	end

	-------------------------------------------------------------------------------------------------------------------------
	-- 1 - Obtengo las Entidades MACHINE LEARNED
	-------------------------------------------------------------------------------------------------------------------------
	LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] User '%s' Loading MACHINE LEARNED ENTITIES on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)
	local successSimple, responseSimple = apiLuis:LoadAllAppEntitiesMachineLearned(appId, versionId, paginated, vcc)

	local appSimpleEntities
	if successSimple then
		local encodedAppSimpleEntities = responseSimple
		appSimpleEntities = json.decode(encodedAppSimpleEntities)
		local total = utils.GetRowCount(appSimpleEntities)

		LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] Loaded %d APP MACHINE LEARNED ENTITIES on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAllAppEntities] Error loading cognitive services. All APP MACHINE LEARNED ENTITIES on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(responseSimple))
		return {result = "NOT_FOUND", cognitiveServiceError = responseSimple}
	end

	local appAllEntities = {}
	local function Union (list)
		for _, entitiy in ipairs(list or {}) do
			table.insert(appAllEntities, entitiy)
		end
	end
	
	Union(appSimpleEntities)

	if versionId ~= "" and versionId ~= nil then 

		-------------------------------------------------------------------------------------------------------------------------
		-- 2 - Obtengo las Entidades COMPOSITE
		-------------------------------------------------------------------------------------------------------------------------
		LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] User '%s' Loading COMPOSITE ENTITIES on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)
		local successComposite, responseComposite = apiLuis:LoadAllAppEntitiesComposite(appId, versionId, paginated, vcc) 

		local appCompositeEntities
		if successComposite then
			local encodedAppCompoisteEntities = responseComposite
			appCompositeEntities = json.decode(encodedAppCompoisteEntities)
			local total = utils.GetRowCount(appCompositeEntities)

			LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] Loaded %d APP COMPOSITE ENTITIES on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
		else
			web.status = 400
			LogError("[COGNITIVE_SERVICES:LoadAllAppEntities] Error loading cognitive services. All APP COMPOSITE ENTITIES on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(responseComposite))
			return {result = "NOT_FOUND", cognitiveServiceError = responseComposite}
		end

		-------------------------------------------------------------------------------------------------------------------------
		-- 3 - Obtengo las Entidades HIERARCHICAL
		-------------------------------------------------------------------------------------------------------------------------
		LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] User '%s' Loading HIERARCHICAL ENTITIES on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)
		local successHierarchical, responseHierarchical = apiLuis:LoadAllAppEntitiesHierarchical(appId, versionId, paginated, vcc) 

		local appHierarchicalEntities
		if successHierarchical then
			local encodedAppHierarchicalEntities = responseHierarchical
			appHierarchicalEntities = json.decode(encodedAppHierarchicalEntities)
			local total = utils.GetRowCount(appHierarchicalEntities)

			LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] Loaded %d APP HIERARCHICAL ENTITIES on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
		else
			web.status = 400
			LogError("[COGNITIVE_SERVICES:LoadAllAppEntities] Error loading cognitive services. All APP HIERARCHICAL ENTITIES on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(responseHierarchical))
			return {result = "NOT_FOUND", cognitiveServiceError = responseHierarchical}
		end

		-------------------------------------------------------------------------------------------------------------------------
		-- 4 - Obtengo las Entidades PREBUILT
		-------------------------------------------------------------------------------------------------------------------------
		LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] User '%s' Loading PREBUILT ENTITIES on appId '%s' versionId: '%s' for vcc '%s'. Paginated info: '%s'.", userId, appId, versionId, vcc, encodedPaginated)
		local successPrebuilt, responsePrebuilt = apiLuis:LoadAllAppEntitiesPrebuilt(appId, versionId, paginated, vcc) 

		local appPrebuiltEntities
		if successPrebuilt then
			local encodedAppPrebuiltEntities = responsePrebuilt
			appPrebuiltEntities = json.decode(encodedAppPrebuiltEntities)
			local total = utils.GetRowCount(appPrebuiltEntities)

			LogInfo("[COGNITIVE_SERVICES:LoadAllAppEntities] Loaded %d APP PREBUILT ENTITIES on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
		else
			web.status = 400
			LogError("[COGNITIVE_SERVICES:LoadAllAppEntities] Error loading cognitive services. All APP PREBUILT ENTITIES on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(responsePrebuilt))
			return {result = "NOT_FOUND", cognitiveServiceError = responsePrebuilt}
		end

		-------------------------------------------------------------------------------------------------------------------------
		-- 5 - Realizo una UNION de los 4 tipos de entidades (MACHINE LEARNED, COMPOSITE, HIERARCHICAL, PREBUILT)
		-------------------------------------------------------------------------------------------------------------------------
		
		Union(appCompositeEntities)
		Union(appHierarchicalEntities)
		Union(appPrebuiltEntities)
	end
		
	return json.encode(appAllEntities)
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/machine_learned/get/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: GET

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		{
		  "id": "3d73462d-98f0-4dcd-8d30-fab9e65f2e73",
		  "name": "DayOfWeek",
		  "typeId": 1,
		  "readableType": "Entity Extractor"
		}
]]--
local function GetAppEntityMachineLearned (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetAppEntityMachineLearned] User '%s' getting cognitive services APP MACHINE LEARNED ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:GetAppEntityMachineLearned(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:GetAppEntityMachineLearned] Error getting cognitive services APP MACHINE LEARNED ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/machine_learned/create/

	Metodo: POST
		DATA:
			{
				"appId":"f23db312-1ff6-47d5-be25-f6a739daf5ee",
				"name":"Entity6",
				"versionId":"0.1" -- opcional (por defecto 0.1)
			}

	Respuesta:
		{"entityId":"734783da-ca2f-4fac-893f-394eec461b83"}
]]--
local function CreateAppEntityMachineLearned (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreateAppEntityMachineLearned] Error Creating cognitive services APP MACHINE LEARNED ENTITIES for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityMachineLearned] Error Creating cognitive services APP MACHINE LEARNED NTITIES for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreateAppEntityMachineLearned] User '%s' creating cognitive services APP ENTITIES on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	local success, response = apiLuis:CreateAppEntityMachineLearned(info, vcc)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		local result = {entityId = newId}
		local success, encodedEntity = apiLuis:GetAppEntityMachineLearned(info.appId, result.entityId, info.versionId)
		if success then
			result.entity = json.decode(encodedEntity)
		end
		return result
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityMachineLearned] Error creating cognitive services APP MACHINE LEARNED ENTITIES on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/machine_learned/create_child/

	Metodo: POST
		DATA:
			{
				"appId":"f23db312-1ff6-47d5-be25-f6a739daf5ee",
				"name":"Entity6",
				"versionId":"0.1" -- opcional (por defecto 0.1)
			}

	Respuesta:
		{"entityId":"734783da-ca2f-4fac-893f-394eec461b83"}
]]--
local function CreateAppEntityChildMachineLearned (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreateAppEntityChildMachineLearned] Error Creating cognitive services APP MACHINE LEARNED ENTITY CHILD for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name or not info.entityId then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityChildMachineLearned] Error Creating cognitive services APP MACHINE LEARNED ENTITY CHILD for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreateAppEntityChildMachineLearned] User '%s' creating cognitive services APP ENTITY CHILD on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	local success, response = apiLuis:CreateAppEntityChildMachineLearned(info)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {entityId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityChildMachineLearned] Error creating cognitive services APP MACHINE LEARNED ENTITY CHILD on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/machine_learned/rename/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: PUT

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

		DATA:
		{
			"name": "string", -- requerido
		}
]]--
local function RenameAppEntityMachineLearned (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:RenameAppEntityMachineLearned] Error Updating cognitive services APP MACHINE LEARNED ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Request Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityMachineLearned] Error Updating cognitive services APP MACHINE LEARNED ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Posted Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:RenameAppEntityMachineLearned] User '%s' Updating cognitive services APP MACHINE LEARNED ENTITIES '%s' on app: '%s' versionId: '%s'.", userId, entityId, appId, versionId)

	info.appId = appId
	info.entityId = entityId
	info.versionId = versionId

	local success, response = apiLuis:RenameAppEntityMachineLearned(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityMachineLearned] Error Updating cognitive services APP MACHINE LEARNED ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", entityId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/machine_learned/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)
]]--
local function DeleteAppEntityMachineLearned (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppEntityMachineLearned] User '%s' deleting cognitive services APP MACHINE LEARNED ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppEntityMachineLearned(appId, entityId, versionId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppEntityMachineLearned] Error deleting cognitive services APP MACHINE LEARNED ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/composite/get/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: GET

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		{
		  "id": "3d73462d-98f0-4dcd-8d30-fab9e65f2e73",
		  "name": "DayOfWeek",
		  "typeId": 1,
		  "readableType": "Entity Extractor"
		}
]]--
local function GetAppEntityComposite (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetAppEntityComposite] User '%s' getting cognitive services APP COMPOSITE ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = apiLuis:GetAppEntityComposite(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:GetAppEntityComposite] Error getting cognitive services APP COMPOSITE ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/composite/create/

	Metodo: POST
		DATA:
			{
				"appId":"a13fda11-cf4b-4db4-8133-b7d4c2c8bf01",
				"versionId":"0.1" -- opcional (por defecto 0.1)
				"entity": {
					"name": "Reservation",
					"children": [ "Location::To", "datetime" ]
				}
			}

	Respuesta:
		{"entityId":"734783da-ca2f-4fac-893f-394eec461b83"}
]]--
local function CreateAppEntityComposite (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreateAppEntityComposite] Error Creating cognitive services APP COMPOSITE ENTITIES for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name or not info.children or #info.children < 1 then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityComposite] Error Creating cognitive services APP COMPOSITE NTITIES for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreateAppEntityComposite] User '%s' creating cognitive services APP COMPOSITE ENTITIES on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	info.entity = {
		name = info.name,
		children = info.children
	}
	info.name = nil
	info.children = nil

	local success, response = apiLuis:CreateAppEntityComposite(info)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {entityId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityComposite] Error creating cognitive services APP COMPOSITE ENTITIES on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end


--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/composite/rename/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: PUT

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

		DATA:
		{
			entity: {
				"name": "string", -- requerido
				"children": [ "Location::To", "datetime" ]
			}
		}
]]--
local function RenameAppEntityComposite (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:RenameAppEntityComposite] Error Updating cognitive services APP COMPOSITE ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Request Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name or not info.children or #info.children < 1 then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityComposite] Error Updating cognitive services APP COMPOSITE ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Posted Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:RenameAppEntityComposite] User '%s' Updating cognitive services APP COMPOSITE ENTITIES '%s' on app: '%s' versionId: '%s'.", userId, entityId, appId, versionId)

	info.appId = appId
	info.entityId = entityId
	info.versionId = versionId
	info.entity = {
		name = info.name,
		children = info.children
	}
	info.name = nil
	info.children = nil

	local success, response = apiLuis:RenameAppEntityComposite(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityComposite] Error Updating cognitive services APP COMPOSITE ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", entityId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/composite/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)
]]--
local function DeleteAppEntityComposite (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppEntityComposite] User '%s' deleting cognitive services APP COMPOSITE ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppEntityComposite(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppEntityComposite] Error deleting cognitive services APP COMPOSITE ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/hierarchical/get/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: GET

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		{
		  "id": "3d73462d-98f0-4dcd-8d30-fab9e65f2e73",
		  "name": "DayOfWeek",
		  "typeId": 1,
		  "readableType": "Entity Extractor"
		}
]]--
local function GetAppEntityHierarchical (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:GetAppEntityHierarchical] User '%s' getting cognitive services APP HIERARCHICAL ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:GetAppEntityHierarchical(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:GetAppEntityHierarchical] Error getting cognitive services APP HIERARCHICAL ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/hierarchical/create/

	Metodo: POST
		DATA:
			{
				"appId":"a13fda11-cf4b-4db4-8133-b7d4c2c8bf01",
				"versionId":"0.1" -- opcional (por defecto 0.1)
				"entity": {
					"name": "Reservation",
					"children": [ "Location::To", "datetime" ]
				}
			}

	Respuesta:
		{"entityId":"734783da-ca2f-4fac-893f-394eec461b83"}
]]--
local function CreateAppEntityHierarchical (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreateAppEntityHierarchical] Error Creating cognitive services APP HIERARCHICAL ENTITIES for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name or not info.children or #info.children < 1 then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityHierarchical] Error Creating cognitive services APP HIERARCHICAL NTITIES for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreateAppEntityHierarchical] User '%s' creating cognitive services APP HIERARCHICAL ENTITIES on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	info.entity = {
		name = info.name,
		children = info.children
	}
	info.name = nil
	info.children = nil

	local success, response = apiLuis:CreateAppEntityHierarchical(info)
	if success then
		-- Elimino caracteres del Id
		local newId = utils.SanitizeId(response)
		return {entityId = newId}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreateAppEntityHierarchical] Error creating cognitive services APP HIERARCHICAL ENTITIES on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/hierarchical/rename/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: PUT

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)

		DATA:
		{
			entity: {
				"name": "string", -- requerido
				"children": [ "Location::To", "datetime" ]
			}
		}
]]--
local function RenameAppEntityHierarchical (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:RenameAppEntityHierarchical] Error Updating cognitive services APP HIERARCHICAL ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Request Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name or not info.children or #info.children < 1 then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityHierarchical] Error Updating cognitive services APP HIERARCHICAL ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Posted Information '%s'", entityId, appId, versionId, vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:RenameAppEntityHierarchical] User '%s' Updating cognitive services APP HIERARCHICAL ENTITIES '%s' on app: '%s' versionId: '%s'.", userId, entityId, appId, versionId)

	info.appId = appId
	info.entityId = entityId
	info.versionId = versionId
	info.entity = {
		name = info.name,
		children = info.children
	}
	info.name = nil
	info.children = nil

	local success, response = apiLuis:RenameAppEntityHierarchical(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:RenameAppEntityHierarchical] Error Updating cognitive services APP HIERARCHICAL ENTITIES '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", entityId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/hierarchical/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)
]]--
local function DeleteAppEntityHierarchical (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppEntityHierarchical] User '%s' deleting cognitive services APP HIERARCHICAL ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppEntityHierarchical(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppEntityHierarchical] Error deleting cognitive services APP HIERARCHICAL ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/prebuilt/add/

	Metodo: POST
		DATA:
			{
				"appId":"a13fda11-cf4b-4db4-8133-b7d4c2c8bf01",
				"versionId":"0.1" -- opcional (por defecto 0.1)
				"entity": {
					"list": [ "datetimeV2", "..." ]
				}
			}

	Respuesta:
		[
			{
				"id": "a18eaa87-ecd9-4a67-a951-e70cf5b87c7e",
				"name": "datetimeV2",
				"typeId": 2,
				"readableType": "Prebuilt Entity Extractor"
			}
		]
]]--
local function AddAppEntityPrebuilt (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:AddAppEntityPrebuilt] Error Adding cognitive services APP PREBUILT ENTITIES for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.list or #info.list < 1 then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddAppEntityPrebuilt] Error Adding cognitive services APP PREBUILT NTITIES for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:AddAppEntityPrebuilt] User '%s' Adding cognitive services APP PREBUILT ENTITIES on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	info.entity = {
		list = info.list
	}
	info.list = nil

	local success, response = apiLuis:AddAppEntityPrebuilt(info)
	if success then
		-- -- Elimino caracteres del Id
		-- local newId = utils.SanitizeId(response)
		-- return {entityId = newId}
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AddAppEntityPrebuilt] Error Adding cognitive services APP PREBUILT ENTITIES on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/entities/hierarchical/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/7b1d9a5f-2804-42a1-9b58-03b07788477a/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) Entity ID : 7b1d9a5f-2804-42a1-9b58-03b07788477a
			3) Version ID : 0.1 (opcional - por defecto 0.1)
]]--
local function DeleteAppEntityPrebuilt (web, appId, entityId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeleteAppEntityPrebuilt] User '%s' deleting cognitive services APP PREBUILT ENTITIES '%s' versionId: '%s' on app: '%s'.", userId, entityId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeleteAppEntityPrebuilt(appId, entityId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeleteAppEntityPrebuilt] Error deleting cognitive services APP PREBUILT ENTITIES '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", entityId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/load_cultures/

	Metodo: GET
		Parametros:
			...

	Respuesta:
		Status:200 OK
		[
			{
				"name": "English",
				"code": "en-us"
			},
			{
				"name": "Chinese",
				"code": "zh-cn"
			},
			..
			..
		]
]]--
local function LoadAvailablePrebuiltEntityList (web, appId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAvailablePrebuiltEntityList] User '%s' Loading cognitive services Available Prebuilt list. App: '%s', Version: '%s'.", userId, appId, versionId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:LoadAvailablePrebuiltEntityList(appId, versionId, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAvailablePrebuiltEntityList] Error Loading cognitive services Available Prebuilt list. App: '%s', Version: '%s'. Error: '%s'", appId, versionId, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

-------------------------------------------------------
-- FEATURES - APPS
-------------------------------------------------------

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/features/load_all/d469528f-a521-4035-a9ea-36a346b83df8/0/0.1

	Metodo: GET

		Parametros:
			1) App ID : d469528f-a521-4035-a9ea-36a346b83df8
			2) encodedPaginated: {page: 0, rows_per_page: 10};
				Atributo "page" = (-1 / 0 o N) - El numero de row en la cual tiene que comenzar el paginado. (-1 es sin paginado).
				Atributo "rows_per_page" = cantidad de filas por pagina.
				Si viene un string vacío, en lugar del json, no tiene paginado.
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		{
		  "phraselistFeatures": [
			{
			  "id": 91640,
			  "name": "moneda",
			  "phrases": "pesos,dolares,euros,libras,dólares,yen",
			  "isExchangeable": true,
			  "isActive": true
			}
		  ],
		  "patternFeatures": []
		}

		Puede regresar la totalidad o la pagina solicitada.
]]--
local function LoadAllAppFeatures (web, appId, encodedPaginated, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAllAppFeatures] User '%s' Loading cognitive services. APP FEATURES on appId '%s' versionId: '%s' for vcc '%s'. Page skip to row: '%s'.", userId, appId, versionId, vcc, encodedPaginated)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, paginated = pcall(json.decode, encodedPaginated)
	if not ok then
		paginated = ""
	end
	local success, response = apiLuis:LoadAllAppFeatures(appId, versionId, paginated, vcc)

	if success then
		local encodedAppFeatures = response
		local appFeatures = json.decode(encodedAppFeatures)
		local total = utils.GetRowCount(appFeatures)

		LogInfo("[COGNITIVE_SERVICES:LoadAllAppFeatures] Loaded %d APP FEATURES on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
		return encodedAppFeatures
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAllAppFeatures] Error loading cognitive services. APP FEATURES on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/features/phraselist/load_all/d469528f-a521-4035-a9ea-36a346b83df8/0/0.1

	Metodo: GET

		Parametros:
			1) App ID : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) encodedPaginated: {page: 0, rows_per_page: 10};
				Atributo "page" = (-1 / 0 o N) - El numero de row en la cual tiene que comenzar el paginado. (-1 es sin paginado).
				Atributo "rows_per_page" = cantidad de filas por pagina.
				Si viene un string vacío, en lugar del json, no tiene paginado.
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		[
			{"id":91640,"name":"moneda","phrases":"pesos,dolares,euros,libras,dólares,yen","isExchangeable":true,"isActive":true},
			..
		]

		Puede regresar la totalidad o la pagina solicitada.
]]--
local function LoadAllAppFeaturesPhraselist (web, appId, encodedPaginated, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:LoadAllAppFeaturesPhraselist] User '%s' Loading cognitive services. FEATURES PHRASELIST on appId '%s' versionId: '%s' for vcc '%s'. Page skip to row: '%s'.", userId, appId, versionId, vcc, encodedPaginated)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, paginated = pcall(json.decode, encodedPaginated)
	if not ok then
		paginated = ""
	end
	local success, response = apiLuis:LoadAllAppFeaturesPhraselist(appId, versionId, paginated)

	if success then
		local encodedAppFeatures = response
		local appFeatures = json.decode(encodedAppFeatures)
		local total = utils.GetRowCount(appFeatures)

		LogInfo("[COGNITIVE_SERVICES:LoadAllAppFeaturesPhraselist] Loaded %d FEATURES PHRASELIST on appId '%s' versionId: '%s' for vcc '%s'.", total, appId, versionId, vcc)
		return encodedAppFeatures
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:LoadAllAppFeaturesPhraselist] Error loading cognitive services. APP FEATURES PHRASELIST on appId '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/features/phraselist/create/

	Metodo: POST
		DATA:
			{
				"appId":"d469528f-a521-4035-a9ea-36a346b83df",
				"name": "Mi Lista 1",
				"phrases": "rocio,ayelen,emma,ellio",
				"isExchangeable": true,
				"versionId":"0.1" -- opcional (por defecto 0.1)
			}

	Respuesta:
		{"phraseId":"114170"}
]]--
local function CreatePhraselist (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:CreatePhraselist] Error Creating cognitive services FEATURES PHRASELIST for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.appId or not info.name or not info.phrases then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreatePhraselist] Error Creating cognitive services FEATURES PHRASELIST for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:CreatePhraselist] User '%s' creating cognitive services FEATURES PHRASELIST on app: '%s' versionId: '%s'.", userId, info.appId, info.versionId)

	local success, response = apiLuis:CreatePhraselist(info)
	if success then
		return {phraseId = response}
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:CreatePhraselist] Error creating cognitive services FEATURES PHRASELIST on app: '%s' versionId: '%s', for vcc: '%s'. Error: '%s'", info.appId, info.versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/features/phraselist/update/d469528f-a521-4035-a9ea-36a346b83df8/113023/0.1

	Metodo: PUT

		Parametros:
			1) App ID    : d469528f-a521-4035-a9ea-36a346b83df8
			2) Phrase list ID : 113023
			3) Version ID : 0.1 (opcional - por defecto 0.1)

		DATA:
			{
				"name": "Mi Lista 1",
				"phrases": "rocio,ayelen,emma,ellio",
				"isActive": true/false (opcional - por defecto es true),
				"isExchangeable": true
			}
	Respuesta:
		Status:200 OK
		[]
]]--
local function UpdatePhraselist (web, appId, phraseListId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:UpdatePhraselist] Error Updating cognitive services FEATURES PHRASELIST '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Request Information '%s'", phraseListId, appId, versionId, vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:UpdatePhraselist] Error Updating cognitive services FEATURES PHRASELIST '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Invalid Posted Information '%s'", phraseListId, appId, versionId, vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:UpdatePhraselist] User '%s' Updating cognitive services FEATURES PHRASELIST '%s' on app: '%s' versionId: '%s'.", userId, phraseListId, appId, versionId)

	info.appId = appId
	info.phraseListId = phraseListId
	info.versionId = versionId

	local success, response = apiLuis:UpdatePhraselist(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:UpdatePhraselist] Error Updating cognitive services FEATURES PHRASELIST '%s' on app: '%s' versionId: '%s' for vcc: '%s'. Error: '%s'", phraseListId, appId, versionId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
http://i6.inconcert.cc/inconcert/apps/cognitive_services/cognitive_services/apps/features/phraselist/delete/d469528f-a521-4035-a9ea-36a346b83df8/113023/0.1

	Metodo: DELETE

		Parametros:
			1) App ID    : f23db312-1ff6-47d5-be25-f6a739daf5ee
			2) PhraseList ID : 1823
			3) Version ID : 0.1 (opcional - por defecto 0.1)

	Respuesta:
		Status:200 OK
		[]
]]--
local function DeletePhraselist (web, appId, phraseListId, versionId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[COGNITIVE_SERVICES:DeletePhraselist] User '%s' deleting cognitive services FEATURES PHRASELIST '%s' versionId: '%s' on app: '%s'.", userId, phraseListId, versionId, appId)

	local success, response = utils.IsAvailableNLUInVCC("LUIS", vcc)
	if not success then
		web.status = 400
		return {result = "NOT_AVAILABLE_NLU", err = response}
	end

	local success, response = apiLuis:DeletePhraselist(appId, phraseListId, versionId)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:DeletePhraselist] Error deleting cognitive services FEATURES PHRASELIST '%s' versionId: '%s' on app: '%s', for vcc: '%s'. Error: '%s'", phraseListId, versionId, appId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[COGNITIVE_SERVICES:Render] Render form Cognitive Services with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local sections = {
		{id = "application"      , file = "frm_cognitive_services_content_application.html"      },
		{id = "intents"          , file = "frm_cognitive_services_content_intents.html"          },
		{id = "entities"         , file = "frm_cognitive_services_content_entities.html"         },
		{id = "features_phrases" , file = "frm_cognitive_services_content_features_phrases.html" },
		{id = "train_and_publish", file = "frm_cognitive_services_content_train_and_publish.html"}
	}
	local page_dir = "pages/"
	local html_content = ""
	for _, section in ipairs(sections) do
		local html_file = page_dir .. section.file
		LogDebug("[COGNITIVE_SERVICES:Render] Open form File '%s' of Section: '%s'.", section.file, section.id)
		local html_section = m_htmlRender:RenderPageLP(html_file, web, env)
		if html_section then
			html_content = html_content .. 
				"<div form_section='" .. section.id .. "' class='pageFormContainerAux' hidden>" ..
					html_section ..
				"</div>"
		else
			LogError("HTML Cognitive Services section does not exist: %s", html_file)
		end
	end

	return html_content
end

--[[
****************************************************************
	MAPEO DE URLS
****************************************************************
]]--

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local perm_view  = "view_cognitive_services"
	local perm_save  = "save_cognitive_services"
	local perm_erase = "delete_cognitive_services"
	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		actionId = actionId or perm_save
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("cognitive_services", actionId, fn)
	end

	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- APPS
	module:dispatch_post  (HasPermission(LoadApps, perm_view, replyValFn)              , p .. "/apps/load_all/"                                                 )
	module:dispatch_get   (HasPermission(GetApp, perm_view)                            , p .. "/apps/get/(.*)"                                                  )
	module:dispatch_post  (HasPermission(AddApp)                                       , p .. "/apps/add/"                                                      )
    module:dispatch_post  (HasPermission(ImportApp, perm_save)                         , p .. "/apps/import"                                                    )
	module:dispatch_put   (HasPermission(RenameApp)                                    , p .. "/apps/rename/"                                                   )
	module:dispatch_post  (HasPermission(DeleteApps, perm_erase, replyValFn)           , p .. "/apps/delete/"                                                   )
	module:dispatch_post  (HasPermission(PublishApp)                                   , p .. "/apps/publish/"                                                  )
	module:dispatch_post  (HasPermission(TrainApp)                                     , p .. "/apps/train/"                                                    )
	module:dispatch_get   (HasPermission(GetAppVersion, perm_view)                     , p .. "/apps/get_version/([^/]+)/(.*)"                                  )
	module:dispatch_get   (HasPermission(LoadAppVersions, perm_view)                   , p .. "/apps/load_versions/([^/]*)/(.*)"                                )
	module:dispatch_get   (HasPermission(LoadAppCultures, perm_view)                   , p .. "/apps/load_cultures/(.*)"                                        ) 
	module:dispatch_get   (HasPermission(LoadAppCulturesForIntents, perm_view)         , p .. "/apps/load_cultures_for_intents/(.*)"                            )
	module:dispatch_delete(HasPermission(DeleteAppSuggestedUtterance)                  , p .. "/apps/suggested_utterances/delete/([^/]*)/([^/]*)/(.*)"          )
	module:dispatch_get   (HasPermission(LoadAvailableApiVersions, perm_view)          , p .. "/apps/load_available_api_versions/"                              ) 
	
	--[[
		**********************************************
		ATENCION!!!
		-----------
			Esta URL/Funcion NO debe invocarse, sin que previamente se haya realizado un TRAIN (POST) de la APP.
			Ya que sino los scores de los utterances NO cambian.
			Ref #8900

			https://westus.dev.cognitive.microsoft.com/docs/services/5890b47c39e2bb17b84a55ff/operations/5890b47c39e2bb052c5b9c46
				train - Get version training status
				Gets the training status of all models (intents and entities) for the specified LUIS app. You must call the train API to train the LUIS app before you call this API to get training status.

				Select the testing console in the region where you created your resource:
		**********************************************
	]]--
	module:dispatch_get   (HasPermission(GetTrainApp, perm_view)                       , p .. "/apps/get_train/([^/]+)/(.*)"                                    )

	-- APPS - ENTITIES MACHINE LEARNED, COMPOSITE, HIERARCHICAL
	module:dispatch_get   (HasPermission(LoadAllAppEntities, perm_view)                , p .. "/apps/entities/load_all/([^/]*)/([^/]+)/(.*)"                    )

	-- APPS - ENTITIES MACHINE LEARNED
	module:dispatch_get   (HasPermission(GetAppEntityMachineLearned, perm_view)        , p .. "/apps/entities/machine_learned/get/([^/]*)/([^/]*)/(.*)"         )
	module:dispatch_post  (HasPermission(CreateAppEntityMachineLearned)                , p .. "/apps/entities/machine_learned/create/"                          )
	module:dispatch_put   (HasPermission(RenameAppEntityMachineLearned)                , p .. "/apps/entities/machine_learned/rename/([^/]*)/([^/]*)/(.*)"      )
	module:dispatch_delete(HasPermission(DeleteAppEntityMachineLearned)                , p .. "/apps/entities/machine_learned/delete/([^/]*)/([^/]*)/(.*)"      )
	module:dispatch_post  (HasPermission(CreateAppEntityChildMachineLearned)           , p .. "/apps/entities/machine_learned/create_child/"                    )

	-- APPS - ENTITIES COMPOSITE
	module:dispatch_get   (HasPermission(GetAppEntityComposite, perm_view)             , p .. "/apps/entities/composite/get/([^/]*)/([^/]*)/(.*)"               )
	module:dispatch_post  (HasPermission(CreateAppEntityComposite)                     , p .. "/apps/entities/composite/create/"                                )
	module:dispatch_put   (HasPermission(RenameAppEntityComposite)                     , p .. "/apps/entities/composite/rename/([^/]*)/([^/]*)/(.*)"            )
	module:dispatch_delete(HasPermission(DeleteAppEntityComposite)                     , p .. "/apps/entities/composite/delete/([^/]*)/([^/]*)/(.*)"            )

	-- APPS - ENTITIES HIERARCHICAL
	module:dispatch_get   (HasPermission(GetAppEntityHierarchical, perm_view)          , p .. "/apps/entities/hierarchical/get/([^/]*)/([^/]*)/(.*)"            )
	module:dispatch_post  (HasPermission(CreateAppEntityHierarchical)                  , p .. "/apps/entities/hierarchical/create/"                             )
	module:dispatch_put   (HasPermission(RenameAppEntityHierarchical)                  , p .. "/apps/entities/hierarchical/rename/([^/]*)/([^/]*)/(.*)"         )
	module:dispatch_delete(HasPermission(DeleteAppEntityHierarchical)                  , p .. "/apps/entities/hierarchical/delete/([^/]*)/([^/]*)/(.*)"         )

	-- APPS - ENTITIES PREBUILT
	module:dispatch_post  (HasPermission(AddAppEntityPrebuilt)                         , p .. "/apps/entities/prebuilt/add/"                                    )
	module:dispatch_delete(HasPermission(DeleteAppEntityPrebuilt)                      , p .. "/apps/entities/prebuilt/delete/([^/]*)/([^/]*)/(.*)"             )
	module:dispatch_get   (HasPermission(LoadAvailablePrebuiltEntityList, perm_view)   , p .. "/apps/entities/prebuilt/load_available/([^/]*)/(.*)"             )

	-- APPS - INTENTS
	module:dispatch_get   (HasPermission(LoadAllAppIntents, perm_view)                 , p .. "/apps/intents/load_all/([^/]*)/([^/]+)/(.*)"                     )
	module:dispatch_get   (HasPermission(GetAppIntent, perm_view)                      , p .. "/apps/intents/get/([^/]*)/([^/]*)/(.*)"                          )
	module:dispatch_post  (HasPermission(CreateAppIntent)                              , p .. "/apps/intents/create/"                                           )
	module:dispatch_put   (HasPermission(RenameAppIntent)                              , p .. "/apps/intents/rename/([^/]*)/([^/]*)/(.*)"                       )
	module:dispatch_delete(HasPermission(DeleteAppIntent)                              , p .. "/apps/intents/delete/([^/]*)/([^/]*)/(.*)"                       )

	-- APPS - INTENTS / UTTERANCES
	module:dispatch_get   (HasPermission(LoadUtterancesByAppIntent, perm_view)         , p .. "/apps/intents/utterances/load_all/([^/]*)/([^/]*)/(.*)"          )
	module:dispatch_post  (HasPermission(AddUtteranceAppIntent)                        , p .. "/apps/intents/utterances/add/"                                   )
	module:dispatch_delete(HasPermission(DeleteAppUtterance)                           , p .. "/apps/intents/utterances/delete/([^/]*)/([^/]*)/(.*)"            )
	module:dispatch_get   (HasPermission(LoadSuggestedUtterancesByAppIntent, perm_view), p .. "/apps/intents/utterances/suggested/load_all/([^/]*)/([^/]*)/(.*)")

	-- APPS - FEATURES
	module:dispatch_get   (HasPermission(LoadAllAppFeatures, perm_view)                , p .. "/apps/features/load_all/([^/]+)/([^/]+)/(.*)"                    )
	module:dispatch_get   (HasPermission(LoadAllAppFeaturesPhraselist, perm_view)      , p .. "/apps/features/phraselist/load_all/([^/]+)/([^/]+)/(.*)"         )
	module:dispatch_post  (HasPermission(CreatePhraselist)                             , p .. "/apps/features/phraselist/create/"                               )
	module:dispatch_put   (HasPermission(UpdatePhraselist)                             , p .. "/apps/features/phraselist/update/([^/]*)/([^/]*)/(.*)"           )
	module:dispatch_delete(HasPermission(DeletePhraselist)                             , p .. "/apps/features/phraselist/delete/([^/]*)/([^/]*)/(.*)"           )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
