/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	var format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form cognitive_services class - train and publish section
	//
	function adm_form_cognitive_services_section_train_and_publish(params)
	{
		var success = adm_form_cognitive_services_section_train_and_publish.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_action_id = "view_cognitive_services";  // view action id de permisos
		this.save_action_id  = { create: "save_cognitive_services", update: "save_cognitive_services" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos
		this.trained_list = [];
		this.app_assigned_key = false;

		return true;
	}

	inConcert.Utils.inherit(adm_form_cognitive_services_section_train_and_publish, inConcert.conf_form_section);

	if ( typeof inConcert.adm_form_cognitive_services_section_train_and_publish !== "function" ) {
		inConcert.adm_form_cognitive_services_section_train_and_publish = adm_form_cognitive_services_section_train_and_publish;
	}

	adm_form_cognitive_services_section_train_and_publish.prototype.dispose = function()
	{
		this.trained_list = [];
		this.app_assigned_key = false;
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.set = function(cognitive_service)
	{
		var self = this;

		if (!cognitive_service) {
			return;
		}

		this.__enable_or_disabled_publish_action();
		var info = cognitive_service.endpoints ? cognitive_service.endpoints.PRODUCTION : null;


		if (cognitive_service.api_version && cognitive_service.api_version === "2022-10-01-preview") {
			var $endpoint_key_parent = $("#adm_form_cognitive_services_train_and_publish_endpoint_key").parent();
			$endpoint_key_parent.hide();

			var $language_parent = $("#adm_form_cognitive_services_train_and_publish_info_language").parent();
			$language_parent.show();

			var $language = $("#adm_form_cognitive_services_train_and_publish_info_language");
			$language.prop("checked", cognitive_service.inconcertInfo ? cognitive_service.inconcertInfo.language : "en-us");

		} else {
			this.__get_assigned_endpoint_key(function(assigned_endpoint_key)
			{
				var $endpoint_key = $("#adm_form_cognitive_services_train_and_publish_endpoint_key");
				$endpoint_key.val(assigned_endpoint_key);

				if (cognitive_service.inconcertInfo) {

					var $endpoint_key_parent = $("#adm_form_cognitive_services_train_and_publish_endpoint_key").parent();
					$endpoint_key_parent.show();

					var $language_parent = $("#adm_form_cognitive_services_train_and_publish_info_language").parent();
					$language_parent.hide();

					var $verbose = $("#adm_form_cognitive_services_train_and_publish_info_verbose");
					$verbose.prop("checked", cognitive_service.inconcertInfo.verbose);

					var $timezone = $("#adm_form_cognitive_services_train_and_publish_info_timezone");
					$timezone.val(cognitive_service.inconcertInfo.timezone);
				}

				self.__set_published_info(info);
			});
		}

	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__make_train_app_result = function(result)
	{
		if (typeof result !== "object" || !result) {
			return;
		}
		var statuses = {};

		statuses["0"] = {
			name  : "Success",
			action: "TRAINED",
			msg   : "<%=_T('App {0} was successfully trained.')%>"
		};
		statuses["1"] = {
			name  : "Fail",
			action: "ERROR",
			msg   : {
			// failureReason
				"FewLabels": "<%=_T('Error Training App ({0}).')%>" + " " + format("<%=_T('The Intent ({0}) no has Utterances.')%>", result.__intentName || result.modelId)
			}
		};
		statuses["2"] = {
			name  : "UpToDate",
			action: "TRAINED",
			msg   : "<%=_T('App {0} was successfully updated trained.')%>"
		};
		statuses["3"] = {
			name  : "InProgress",
			action: "InProgress",
			msg: "<%=_T('In Progress Training App {0}.')%>"
		};
		statuses["9"] = {
			name  : "Queued",
			action: "ERROR",
			msg   : "<%=_T('Queued Training App {0}.')%>"
		};

		var status_id = result.statusId.toString();
		var current_status = statuses[status_id];
		return current_status;
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__all_trained_success = function()
	{
		var self = this;
		// Si el request es Ok valida si todos los TRAINED son ok
		var count_success_trained = 0;
		$.each(this.trained_list || [], function(_, trained)
		{
			var current_status = self.__make_train_app_result(trained);
			if (current_status) {
				if (current_status.action === "TRAINED") {
					count_success_trained++;
					return true; //continue
				}
				else {
					var app_id = self.frm.get_entity_id();
					console.warn("Cognitive Sevices: No Trained App " + app_id + ", " + JSON.stringify(trained));
				}
			}
		});

		if (count_success_trained > 0) {
			return count_success_trained === this.trained_list.length;
		}
		return false;
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__is_trained_app = function(callback = $.noop)
	{
		// Valida si todos los TRAINED son ok
		var is_ok = this.__all_trained_success();
		if (is_ok) {
			// Todos trained fueron OK
			callback(true);
			return;
		}
		// Si alguno de los Train son Fail, InProgress, Queued, etc
		this.trained_list = [];
		callback(false);
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__set_caption_train_button = function(caption_id)
	{
		var captions = { TRAIN: "<%=_T('Train')%>", TRAIN_AGAIN: "<%=_T('Train again')%>" };
		var is_ok = this.__all_trained_success();
		if (!is_ok) {
			caption_id = "TRAIN";
		}
		var caption = captions[caption_id];
		var $train = $("#adm_form_cognitive_services_train_and_publish_action_train");
		$train.text(caption);
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__enable_publish_action = function()
	{
		var $endpoint_key = $("#adm_form_cognitive_services_train_and_publish_endpoint_key");
		this.frm.readonly_control($endpoint_key, false, true);

		var $verbose = $("#adm_form_cognitive_services_train_and_publish_info_verbose");
		this.frm.readonly_control($verbose, false);

		var $language = $("#adm_form_cognitive_services_train_and_publish_info_language");
		this.frm.readonly_control($language, false);

		var $timezone = $("#adm_form_cognitive_services_train_and_publish_info_timezone");
		this.frm.readonly_control($timezone, false);

		var $train = $("#adm_form_cognitive_services_train_and_publish_action_train");
		$train.removeClass("buttonColorLightblue buttonColorBlue").addClass("buttonColorLightblue");
		this.__set_caption_train_button("TRAIN_AGAIN");

		var $publish = $("#adm_form_cognitive_services_train_and_publish_action_publish");
		$publish.show();
		$publish.attr("disabled", false);

		var $publish_msg = $("#adm_form_cognitive_services_train_and_publish_msg");
		$publish_msg.hide();

		var $publish_info = $("#adm_form_cognitive_services_train_and_publish_info");
		$publish_info.show();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__disable_publish_action = function()
	{
		var $endpoint_key = $("#adm_form_cognitive_services_train_and_publish_endpoint_key");
		this.frm.readonly_control($endpoint_key, true);

		var $verbose = $("#adm_form_cognitive_services_train_and_publish_info_verbose");
		this.frm.readonly_control($verbose, true);

		var $timezone = $("#adm_form_cognitive_services_train_and_publish_info_timezone");
		this.frm.readonly_control($timezone, true);

		var $language = $("#adm_form_cognitive_services_train_and_publish_info_language");
		this.frm.readonly_control($language, true);

		var $train = $("#adm_form_cognitive_services_train_and_publish_action_train");
		$train.removeClass("buttonColorLightblue buttonColorBlue").addClass("buttonColorBlue");
		this.__set_caption_train_button("TRAIN");

		var $publish = $("#adm_form_cognitive_services_train_and_publish_action_publish");
		$publish.hide();
		$publish.attr("disabled", true);

		var $publish_msg = $("#adm_form_cognitive_services_train_and_publish_msg");
		$publish_msg.show();

		var $publish_info = $("#adm_form_cognitive_services_train_and_publish_info");
		$publish_info.hide();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__enable_or_disabled_publish_action = function(callback = $.noop)
	{
		const self = this;

		this.__is_trained_app(function(is_trained) {
			if (is_trained) {
				self.__enable_publish_action();
			}
			else {
				self.__disable_publish_action();
			}
			callback();
		});
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__set_published_info = function(info)
	{
		if (!info) {
			return;
		}

		var $date = $("#adm_form_cognitive_services_train_and_publish_info_date");
		var published_date = inConcert.Utils.getStringFromDate(inConcert.Utils.parseDateTimeString(info.publishedDateTime || info.lastUpdatedDateTime), false, "date-short-year-short-hour");
		$date.val(published_date);

		var $version = $("#adm_form_cognitive_services_train_and_publish_info_version");
		$version.val(info.versionId || "0.1");

		this.__set_caption_train_button("TRAIN_AGAIN");
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.attach_load_request = function(request)
	{
		var self = this;

		if (!$.isArray(request)) {
			return;
		}

		// endpoints key
		var luisApi = inConcert.app.getLuisAPI();
		request.push(
			{
				request_function: function(success) {
					return luisApi.getVCCSubscriptionsKeys(success);
				},
				success: function(list) {
					var $endpoint_key = $("#adm_form_cognitive_services_train_and_publish_endpoint_key");
					var is_blank_first_item = true;
					var attr_id = "subscriptionKey";
					var attr_name = "subscriptionName";
					var sorted = true;
					self.frm.fill_combo($endpoint_key, is_blank_first_item, list, attr_id, attr_name, sorted);
				}
			}
		);

		// timezone
		request.push(
			{
				request_function: function(success) {
					return inConcert.app.loadTimezones(success);
				},
				success: function(response) {
					var $timezone = $("#adm_form_cognitive_services_train_and_publish_info_timezone");
					var is_blank_first_item = false;
					self.frm.fill_timezones(response, $timezone, is_blank_first_item); // funcion definida en clase base "conf_form"
				}
			}
		);
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.before_show = function(callback = $.noop)
	{
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		this.__enable_or_disabled_publish_action(function() {
			callback();
		});
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.on_show = function()
	{
		adm_form_cognitive_services_section_train_and_publish.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.ready = function()
	{
		var $train = $("#adm_form_cognitive_services_train_and_publish_action_train");
		$train.attr("disabled", false);

		var $version = $("#adm_form_cognitive_services_train_and_publish_info_version");
		var $date = $("#adm_form_cognitive_services_train_and_publish_info_date");

		this.frm.readonly_control($version, true /* disabled */);
		this.frm.readonly_control($date, true /* disabled */);

		var $cultures = $("#adm_form_cognitive_services_cultures_for_intents");
		var $options = $cultures.find("option");
		const $language = $("#adm_form_cognitive_services_train_and_publish_info_language");
		$language.empty();
		$language.append($options.clone());
		
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.set_events = function()
	{
		var self = this;

		var $train = $("#adm_form_cognitive_services_train_and_publish_action_train");
		$train
			.off("click")
			.on("click", function(){
				self.training_retries = 1;
				self.__train_app();
			});

		var $publish = $("#adm_form_cognitive_services_train_and_publish_action_publish");
		$publish
			.off("click")
			.on("click", function(){
				self.__publish_app();
			});
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__update_app_version_info = function()
	{
		var app_section = this.frm.get_section_def("application");
		app_section.frm.update_app_version_info();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__update_intents_info = function()
	{
		var app_section = this.frm.get_section_def("intents");
		app_section.frm.external_update_intents_info();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__on_success_trained = function()
	{
		this.__update_app_version_info();
		this.__update_intents_info();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__on_success_published = function()
	{
		this.__update_app_version_info();
		this.__update_intents_info();
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__train_app = function(params)
	{
		var self = this;

		var can_train_app = function() {
			return true;
		};

		if (!can_train_app()) {
			return;
		}

		var app_name = this.frm.get_entity_display_name();
		
		if (params) {
			this.frm.dashboard.show_waiting(format("<%=_T('Get Training App Status {0}.')%>", app_name));
			self.training_retries++;
		} else {
			this.frm.dashboard.show_waiting(format("<%=_T('Training App {0}.')%>", app_name));
			var app_id = this.frm.get_entity_id();
			params = {appId: app_id};
		}

		var luisApi = inConcert.app.getLuisAPI();
		this.disabled_all_sub_menus(true);

		luisApi.trainApp(params,
			function(success, train_info)
			{
				var msg, type;
				var is_error = true;
				var need_retry = false;

				if (success) {
					if (train_info) {
						var current_status = self.__make_train_app_result(train_info);
						if (current_status) {
							if (current_status.action === "TRAINED") {
								is_error = false;
								msg = current_status.msg || "<%=_T('App {0} was successfully trained.')%>";
							}
							else if (current_status.action === "InProgress") {
								is_error = false;
								need_retry = true;
								msg = current_status.msg || "<%=_T('In Progress Training App {0}.')%>";
							}
							else {
								is_error = true;
								if (typeof(current_status.msg) === "string") {
									msg = current_status.msg;
								}
								else if (typeof(current_status.msg) === "object") {
									if (train_info.details && train_info.details.failureReason) {
										msg = current_status.msg[train_info.details.failureReason];
									} else {
										msg = "<%=_T('You have not met the recommended minimum number of 10 utterances in your project before training.')%>";
									}								
								}
							}
						}
					}
				}
				else {
					self.frm.dashboard.hide_waiting();
				}

				if (need_retry && self.training_retries < 5) {
					params.operationLocation = train_info.operationLocation; 
					self.__train_app(params);
				} else {

					self.__on_success_trained();

					if (!is_error) {
						self.trained_list.push(train_info);
						self.__enable_publish_action();
						msg = msg || "<%=_T('App {0} was successfully trained.')%>";
						type = "Info";
					}
					else {
						self.trained_list = [];
						self.__disable_publish_action();
						msg = msg || "<%=_T('Error Training App {0}.')%>";
						type = "Error";
					}

					self.frm.enable_all_sections();
					msg = format(msg, app_name);
					self.frm.show_status_message(msg, type, false);
				}
					
			}
		);
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__get_timezone_data = function()
	{
		var $timezone = $("#adm_form_cognitive_services_train_and_publish_info_timezone option:selected");

		var timezone_data = $timezone.data("timezone");
		if (!timezone_data) {
			return;
		}
		return timezone_data;
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__get_assigned_endpoint_key = function(callback)
	{
		var cognitive_service = this.frm.get_entity();
		var info = cognitive_service.inconcertInfo ? cognitive_service.inconcertInfo : null;

		if (info && info.subscriptionKey) {
			this.app_assigned_key = info.subscriptionKey;
		}

		callback(this.app_assigned_key);
	};

	adm_form_cognitive_services_section_train_and_publish.prototype.__publish_app = function()
	{
		var self = this;

		var cognitive_service = this.frm.get_entity();

		var $endpoint_key = $("#adm_form_cognitive_services_train_and_publish_endpoint_key");

		if (cognitive_service && cognitive_service.api_version !== "2022-10-01-preview") {
			var can_publish_app = function() {
				return self.frm.validate_field($endpoint_key, "required", "<%=_T('Unable to Publish App. Endpoint Key is required.')%>");
			};
	
			if (!can_publish_app()) {
				return;
			}
		}
		
		this.disabled_all_sub_menus(true);

		var publish_app = function(success)
		{
			if (!success) {
				self.frm.enable_all_sections();
				return;
			}

			var app_id = self.frm.get_entity_id();
			var selected_endpoint_key = $endpoint_key.val();

			var params = {appId: app_id, subscriptionKey: selected_endpoint_key};

			var timezone_data = self.__get_timezone_data();
			if (timezone_data) {
				params.timezone = timezone_data.Id;
			}

			var $language = $("#adm_form_cognitive_services_train_and_publish_info_language");
			params.language = $language.val();

			if (cognitive_service && cognitive_service.api_version === "2022-10-01-preview" && (params.language === undefined || params.language === "")) {
				return self.frm.validate_field($language, "required", "<%=_T('Language is required.')%>");
			}
			
			var $verbose = $("#adm_form_cognitive_services_train_and_publish_info_verbose");
			params.verbose = $verbose.prop("checked");

			var app_name = self.frm.get_entity_display_name();
			self.frm.dashboard.show_waiting(format("<%=_T('Publishing App {0}.')%>", app_name));

			var luisApi = inConcert.app.getLuisAPI();
			luisApi.publishApp(params,
				function(success, result)
				{
					var msg, type;
					if (success) {
						self.__set_published_info(result);
						msg = format("<%=_T('App {0} was successfully Published.')%>", app_name);
						type = "Info";
						self.__on_success_published();
					}
					else {
						self.frm.dashboard.hide_waiting();
						msg = format("<%=_T('Error Publishing App {0}.')%>", app_name);
						type = "Error";
					}
					self.frm.enable_all_sections();
					self.frm.show_status_message(msg, type, false);
				}
			);
		};

		publish_app(true);
	};
}());
