/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	const FORMAT = inConcert.Utils.format;
	const POPUP_ID_ENTITIES = 0;
	const ERROR_TIMEOUT_MS = 5000;
	let API_VERSION;

	//---------------------------------------------------------------------
	// form rasa class - entities section
	function adm_form_rasa_section_entities(params)
	{
		const success = adm_form_rasa_section_entities.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		API_VERSION = inConcert.app.get_rasa_api_version();

		const self = this;

		this.view_action_id  = "view_cognitive_services"; // view action id de permisos
		this.save_action_id  = "save_cognitive_services"; // save action id de permisos
		this.erase_action_id = "save_cognitive_services"; // delete action id de permisos

		this.is_new = null;
		this.current_item = null;
		this.loading = false;

		this.id_prefix = "adm_form_rasa_entities";
		// entity popup
		this.id_btn_new = this.id_prefix + "_new";
		this.id_new_entity_popup = this.id_prefix + "_new_popup";

		var must_show_button = function(/*row*/)
		{
			let show = true;
			return show;
		};

		this.form_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			row_id: "id",
			display_name: "name",
			rows_per_page: 11,
			show_at_least: 10,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			canEdit: function(row) {
				return must_show_button(row);
			},
			ids: {
				paginator: this.id_prefix + "_paginator",
				grid: this.id_prefix + "_list",
				not_results_found: this.id_prefix + "_not_results_found",
				//popup0 (new entity) (POPUP_ID_ENTITIES)
				new_item: this.id_btn_new,
				popup: this.id_new_entity_popup,
				popup_msg: this.id_new_entity_popup + "_message",
				popup_create: this.id_new_entity_popup + "_create",
				popup_create_and_continue: this.id_new_entity_popup + "_create_and_continue",
				popup_title: this.id_new_entity_popup + "_title",
				popup_close: this.id_new_entity_popup + "_close",
				popup_input_id: this.id_new_entity_popup + "_name",
				popup_waiting: this.id_new_entity_popup + "_waiting"
			},
			strings: {
			// popup0 (new entity) (POPUP_ID_ENTITIES)
				not_results_found         : "<%=_T('No Entities found.')%>",
				can_remove                : "<%=_T('Do you want to delete Entity: {0} ?')%>",
				title_new                 : "<%=_T('New Entity')%>",
				title_update              : "<%=_T('Update Entity')%>",
				button_create             : "<%=_T('Create')%>",
				button_create_and_continue: "<%=_T('Create & Continue')%>"
			},
			columns: [
				{id: "_number", name: "", type: "counter"},
				{id: "name"   , name: "<%=_T('Name')%>"  },
				{id: "createdDateTime", name: "<%=_T('Creation Date')%>", type: "timestampSec"},
				{
					id: "_action",
					name: "",
					system_action: {
						edit: {
							must_show: function(row) {
								return must_show_button(row);
							}
						},
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.save(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row, current_popup) {
					self.on_open_popup(row, current_popup);
				},
				remove: function(row, callback = $.noop) {
					self.remove(row, callback);
				},
				on_closed_popup: function(current_popup) {
					self.on_closed_popup(current_popup);
				}
			}
		});

		let simple_children_info = {
			enabled: false
		};

		let get_type1_children_info = function() {
			return simple_children_info;
		};

		this.m_entities_types = {
			// MACHINE LEARNED
			"1": {
				id: 1,
				name: function() {
					return "<%=_T('Simple')%>";
				},
				help: function() {
					if (API_VERSION === "2.0") {
						return "<%=_T('HELP_COGNITIVE_SERVICES_SIMPLE_ENTITY')%>";
					}
					return "<%=_T('HELP_COGNITIVE_SERVICES_MACHINE_LEARNED_ENTITY')%>";
				},
				can_update_memory_info_on_closed_poup: function() {
					return false;
				},
				children_info: get_type1_children_info(),
				api_rasa_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "RasaAPI" -> "/shared/rasa/scripts/rasa_api.js"
					create: "createAppEntity",
					rename: "renameAppEntity",
					erase : "deleteAppEntity"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: true
			}
		};

		return true;
	}

	inConcert.Utils.inherit(adm_form_rasa_section_entities, inConcert.conf_form_section);

	if (typeof inConcert.adm_form_rasa_section_entities !== "function") {
		inConcert.adm_form_rasa_section_entities = adm_form_rasa_section_entities;
	}

	adm_form_rasa_section_entities.prototype.get_grid_value = function(item, field_id)
	{
		let value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		let item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "typeId") {
			let entity_type_def = this.__get_entity_type_def(value.original_value);
			value.value = entity_type_def ? (typeof entity_type_def.name === "function" ? entity_type_def.name() : entity_type_def.name) : "No reconocido";
			value.search_value = value.value;
		}

		return value;
	};

	adm_form_rasa_section_entities.prototype.__create_entity_type_container = function(info)
	{
		if (!info) {
			return;
		}
		info.instance = new info.class_name({
			owner    : this,
			prefix_id: info.jquery_prefix_id
		});
		return true;
	};

	adm_form_rasa_section_entities.prototype.__get_entity_type_def = function(type = "")
	{
		type = type.toString();
		const entity_type_def = this.m_entities_types[type];
		return entity_type_def;
	};

	adm_form_rasa_section_entities.prototype.build_header_custom_buttons = function()
	{
		let html_buttons = 
		"<button id='" + this.id_btn_new + "' class='pageButtonGeneric buttonColorBlue pageButtonNew'>" +
			"<i class='fal fa-plus'></i>" +
			"<%=_T('New Entity')%>" +
		"</button>";
		adm_form_rasa_section_entities.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	adm_form_rasa_section_entities.prototype.remove = function(entity, callback = $.noop)
	{
		const self = this;

		if (!entity) {
			throw("adm_form_rasa_section_entities.prototype.remove. Undefined Entity.");
		}

		var end = function(success, result) {
			let msg;
			if (success) {
				msg = FORMAT("<%=_T('Entity {0} was successfully deleted.')%>", entity.name);
				self.__update_app_version_info({
						// Se le resta 1 ya que aun no se ha eliminado la row ... pero sucedera !!
						entitiesCount: self.form_paginator.count_rows() - 1
					},
					function() {
						self.frm.enable_all_sections();
						self.frm.show_status_message(msg, "Info", ERROR_TIMEOUT_MS);
						self.frm.dashboard.hide_waiting();
					}
				);
			}
			else {
				let server_err_msg = rasaApi.getRasaErrorMsg(result);
				msg = FORMAT("<%=_T('Error deleting Entity {0}.')%>", entity.name) + " " + server_err_msg;

				self.frm.enable_all_sections();
				self.frm.show_status_message(msg, "Error", ERROR_TIMEOUT_MS);
				self.frm.dashboard.hide_waiting();
			}
			callback(success);
		};

		this.frm.dashboard.show_waiting(FORMAT("<%=_T('Deleting Entity {0}.')%>", entity.name));
		this.disabled_all_sub_menus(true);

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			end(false);
			return;
		}

		const params = {appId: app_id, entityId: entity.id};
		const rasaApi = inConcert.app.getRasaAPI();
		const entity_type_def = this.__get_entity_type_def(entity.typeId);
		const api_rasa_methods = entity_type_def.api_rasa_methods;

		/*
			Metodos ERASE
				MachineLearned: "deleteAppEntity"
		*/
		rasaApi[api_rasa_methods.erase](params,
			function(success, result) {
				end(success, result);
			}
		);
	};

	adm_form_rasa_section_entities.prototype.set_events = function()
	{
		this.form_paginator.set_events();
	};

	adm_form_rasa_section_entities.prototype.__show_or_hide_loading_msg = function(message)
	{
		if (this.is_loading()) {
			this.frm.dashboard.show_waiting(message || "<%=_T('Loading Entities.')%>");
		}
		else {
			this.frm.dashboard.hide_waiting();
		}
	};

	adm_form_rasa_section_entities.prototype.load_entities = function(callback = $.noop)
	{
		const self = this;

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		this.loading = true;
		this.__show_or_hide_loading_msg();

		const params = {appId: app_id};

		const rasaApi = inConcert.app.getRasaAPI();
		rasaApi.loadAllEntities(params,
			function(success, entities) {
				self.loading = false;
				if (!success) {
					self.frm.show_status_message("<%=_T('Error retrieving Entities.')%>", "Error", false);
					callback(false);
					return;
				}
				self.__update_app_version_info({
						entitiesCount: (entities || []).length
					},
					function() {
						callback(true, entities);
					}
				);
			}
		);
	};

	adm_form_rasa_section_entities.prototype.__init_entities_paginator = function()
	{
		const self = this;
		this.load_entities(function(success, entities) {
			if (!success) {
				return;
			}
			self.form_paginator.initialize($.noop, entities);
		});
	};

	adm_form_rasa_section_entities.prototype.initialize = function()
	{
		this.__init_entities_paginator();

		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				if (entity_type.children_info && typeof(entity_type.children_info.create) === "function") {
					if (entity_type.children_info.create()) {
						entity_type.children_info.instance.initialize();
					}
				}
			}
		}
	};

	adm_form_rasa_section_entities.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_rasa_section_entities.prototype.on_show = function()
	{
		adm_form_rasa_section_entities.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		this.set_events();
	};

	adm_form_rasa_section_entities.prototype.on_hide = function()
	{
		adm_form_rasa_section_entities.uber.on_hide.call( this );
		this.form_paginator.hide_popup();
	};

	adm_form_rasa_section_entities.prototype.__load_entity_popup_info = function(callback = $.noop)
	{
		let total_entities_type = Object.keys(this.m_entities_types || {}).length;
		let i = 0;

		var on_load = function() {
			i++;
			if (i === total_entities_type) {
				callback();
			}
		};

		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				entity_type.load(on_load);
			}
		}
	};

	adm_form_rasa_section_entities.prototype.on_open_popup = function(item, current_popup)
	{
		if (!current_popup || current_popup === POPUP_ID_ENTITIES) {
			this.__on_open_popup_entity(item);
		}
		else {
			throw("adm_form_rasa_section_entities.prototype.on_open_popup - Unrecognized popup " + current_popup);
		}
	};

	adm_form_rasa_section_entities.prototype.__on_open_popup_entity = function(item)
	{
		const self = this;

		this.is_new = item ? false: true;
		this.current_item = item;

		this.__load_entity_popup_info(function() {
			if (self.is_new) {
				// nuevo
				self.clean_up_controls();
				return;
			}
			// editando
			const $name = $("#" + self.id_new_entity_popup + "_name");
			$name.val(item.name).focus();
		});
	};

	adm_form_rasa_section_entities.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.is_new = null;
		this.current_item = null;
	};

	adm_form_rasa_section_entities.prototype.hide_all_errors = function()
	{
		this.form_paginator.hide_popup_msg();
	};

	adm_form_rasa_section_entities.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.form_paginator.show_popup_msg(err_info.msg, "Error", ERROR_TIMEOUT_MS);
	};

	adm_form_rasa_section_entities.prototype.validate_info_to_save = function(/*current_popup*/)
	{

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to save Entity.')%>"};
		let item = {};

		const $name = $("#" + this.id_new_entity_popup + "_name");
		//const $type = $("#" + this.id_new_entity_popup + "_type");

		item.name = $name.val();
		item.typeId = 1;
		//item.typeId = $type.val();

		let validation_result = inConcert.Utils.validate(item.name, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Name is required.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}
		validation_result = inConcert.Utils.validate(item.name, "id");
		if (!validation_result.success) {
			// valida los caracteres permitidos
			response.msg = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	adm_form_rasa_section_entities.prototype.clean_up_controls = function()
	{
		const $name = $("#" + this.id_new_entity_popup + "_name");
		$name.val("").focus();
	};

	adm_form_rasa_section_entities.prototype.__update_app_version_info = function(info, callback = $.noop)
	{
		const app_section = this.frm.get_section_def("application");
		app_section.frm.update_app_version_info(info, callback);
	};

	adm_form_rasa_section_entities.prototype.save = function(callback, must_create_and_contiune, current_popup)
	{
		const self = this;
		callback = callback || $.noop;

		let response = this.validate_info_to_save(current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		let item = response.item;
		const rasaApi = inConcert.app.getRasaAPI();
		let params = {
			appId: app_id,
			name : item.name
		};

		this.disabled_all_sub_menus(true);

		const entity_type_def = this.__get_entity_type_def(item.typeId);
		const api_rasa_methods = entity_type_def.api_rasa_methods;
		const $full_screen_waiting = $("#genericWaiting");

		if (current_popup === 1) {
			this.is_new = true;
		}

		if (this.is_new) {
			let waiting_msg, success_msg, error_msg;
			waiting_msg = FORMAT("<%=_T('Creating Entities {0}.')%>", item.name);
			success_msg = FORMAT("<%=_T('Entity {0} was successfully created.')%>", item.name);
			error_msg   = FORMAT("<%=_T('Error creating Entity {0}.')%>", item.name);

			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);
			/*
				Metodos CREATE
					MachineLearned: "createAppEntity"
			*/
			rasaApi[api_rasa_methods.create](params,
				function(success, result)
				{
					let msg_response = {msg: "", type: ""};
					if (success) {
						msg_response.msg = success_msg;
						msg_response.type = "Info";
						let must_add_many = false;

						if (result.entity) {
							item = result.entity;
						}
						else {
							item.id = result.entityId;
						}
						callback(true, item, msg_response, must_add_many);

						self.clean_up_controls();
						self.__update_app_version_info({
								entitiesCount: self.form_paginator.count_rows()
							},
							function() {
								self.frm.dashboard.hide_waiting($full_screen_waiting);
								self.frm.enable_all_sections();
							}
						);
					}
					else {
						self.frm.dashboard.hide_waiting($full_screen_waiting);
						self.frm.enable_all_sections();
						let server_err_msg = rasaApi.getRasaErrorMsg(result);
						msg_response.msg = error_msg + server_err_msg;
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
		else {
			let waiting_msg = FORMAT("<%=_T('Updating Entities {0}.')%>", item.name);
			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);

			params.entityId = this.current_item.id;
			/*
				Metodos RENAME
					MachineLearned: "renameAppEntitySimple"
			*/
			rasaApi[api_rasa_methods.rename](params,
				function(success, result)
				{
					let msg_response = {msg: "", type: ""};
					self.frm.dashboard.hide_waiting($full_screen_waiting);
					self.frm.enable_all_sections();
					if (success) {
						$.extend(true, self.current_item, item);
						msg_response.msg = FORMAT("<%=_T('Entity {0} was successfully updated.')%>", item.name);
						msg_response.type = "Info";
						callback(true, self.current_item, msg_response);
						self.clean_up_controls();
					}
					else {
						let server_err_msg = rasaApi.getRasaErrorMsg(result);
						msg_response.msg = FORMAT("<%=_T('Error updating Entity {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
	};

	adm_form_rasa_section_entities.prototype.get_current_entities = function()
	{
		return this.form_paginator.get_items();
	};
}());
