/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form rasa_bots class - train and publish section
	//
	function adm_form_rasa_section_train_and_publish(params)
	{
		const success = adm_form_rasa_section_train_and_publish.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_action_id = "view_cognitive_services"; // view action id de permisos
		this.save_action_id  = { create: "save_cognitive_services", update: "save_cognitive_services" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos

		return true;
	}

	inConcert.Utils.inherit(adm_form_rasa_section_train_and_publish, inConcert.conf_form_section);

	if ( typeof inConcert.adm_form_rasa_section_train_and_publish !== "function" ) {
		inConcert.adm_form_rasa_section_train_and_publish = adm_form_rasa_section_train_and_publish;
	}

	adm_form_rasa_section_train_and_publish.prototype.dispose = function()
	{
	};

	adm_form_rasa_section_train_and_publish.prototype.set = function(rasa_bot)
	{
		if (!rasa_bot) {
			return;
		}
		if (!$.isArray(rasa_bot.trainInfo) || rasa_bot.trainInfo.length === 0) {
			return;
		}
		let last_train_steps = rasa_bot.trainInfo[rasa_bot.trainInfo.length - 1];
		if (!$.isArray(last_train_steps) || last_train_steps.length === 0) {
			return;
		}
		let last_train_step = last_train_steps[last_train_steps.length - 1];
		let current_status = this.__make_train_app_result(last_train_step);
		if (current_status) {
			this.__update_last_train_info(current_status);
		}
	};

	adm_form_rasa_section_train_and_publish.prototype.__make_train_app_result = function(result)
	{
		const rasaApi = inConcert.app.getRasaAPI();
		return rasaApi.makeTrainAppResult(result);
	};

	adm_form_rasa_section_train_and_publish.prototype.attach_load_request = function(request)
	{
		if (!$.isArray(request)) {
			return;
		}
	};

	adm_form_rasa_section_train_and_publish.prototype.before_show = function(callback = $.noop)
	{
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		callback();
	};

	adm_form_rasa_section_train_and_publish.prototype.on_show = function()
	{
		adm_form_rasa_section_train_and_publish.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_rasa_section_train_and_publish.prototype.ready = function()
	{
		const $train = $("#adm_form_rasa_train_run");
		$train.attr("disabled", false);

		const $last_train_status_not_trained = $("#adm_form_rasa_train_status_not_trained_name");
		this.frm.readonly_control($last_train_status_not_trained, true /* disabled */);

		const $last_train_status = $("#adm_form_rasa_train_status_name");
		this.frm.readonly_control($last_train_status, true /* disabled */);

		const $last_train_date = $("#adm_form_rasa_train_status_date");
		this.frm.readonly_control($last_train_date, true /* disabled */);

		const $last_file_name = $("#adm_form_rasa_train_status_file_name");
		this.frm.readonly_control($last_file_name, true /* disabled */);
	};

	adm_form_rasa_section_train_and_publish.prototype.set_events = function()
	{
		var self = this;

		var $train = $("#adm_form_rasa_train_run");
		$train
			.off("click")
			.on("click", function(){
				self.__train_app();
			});
	};

	adm_form_rasa_section_train_and_publish.prototype.__update_app_version_info = function(app)
	{
		var app_section = this.frm.get_section_def("application");
		app_section.frm.update_app_version_info(app);
	};

	adm_form_rasa_section_train_and_publish.prototype.__update_intents_info = function()
	{
		var app_section = this.frm.get_section_def("intents");
		app_section.frm.external_update_intents_info();
	};

	adm_form_rasa_section_train_and_publish.prototype.__update_last_train_info = function(current_status)
	{
		if (!current_status) {
			return;
		}

		var format_date = function(time_sec) {
			let time_ms = time_sec * 1000;
			return inConcert.Utils.formatDate(time_ms, "date-short-year-short-hour");
		};
		const $last_train_status = $("#adm_form_rasa_train_status_name");
		const $last_train_date = $("#adm_form_rasa_train_status_date");

		const train_info = current_status.raw_status;
		$last_train_status.val(current_status.desc);
		$last_train_date.val(format_date(train_info.startTrainDateTime));

		if (train_info.appStatus) {
			const $last_file_name = $("#adm_form_rasa_train_status_file_name");
			let result = inConcert.Utils.parseDirectoryAndFileName(train_info.appStatus.model_file);
			let file_name = result && result.file_name ? result.file_name : train_info.appStatus.model_file || "";
			$last_file_name.val(file_name);
		}
		$("#adm_form_rasa_train_status_not_trained_container").hide();
		$("#adm_form_rasa_train_status_container").show();
	};

	adm_form_rasa_section_train_and_publish.prototype.__on_success_trained = function(current_status, app)
	{
		this.__update_last_train_info(current_status);
		this.__update_app_version_info(app);
		this.__update_intents_info();
	};

	adm_form_rasa_section_train_and_publish.prototype.__on_error_train = function(current_status)
	{
		this.__update_last_train_info(current_status);
	};

	adm_form_rasa_section_train_and_publish.prototype.__train_app = function()
	{
		const self = this;

		var can_train_app = function() {
			return true;
		};

		if (!can_train_app()) {
			return;
		}

		this.frm.dashboard.disable_back_button(true);
		var app_name = this.frm.get_entity_display_name();
		this.frm.dashboard.show_waiting(format("<%=_T('Training App {0}.')%>", app_name));

		var app_id = this.frm.get_entity_id();
		var params = {appId: app_id};
		const rasaApi = inConcert.app.getRasaAPI();
		this.disabled_all_sub_menus(true);

		rasaApi.trainApp(params,
			function(success, train_info)
			{
				let msg, type;
				let is_error = true;
				let current_status;
				if (success) {
					if (train_info) {
						let train_status = train_info.trainStatus;
						current_status = self.__make_train_app_result(train_status);
						if (current_status) {
							if (current_status.action === "TRAINED") {
								const app_section = self.frm.get_section_def("application");
								let app = app_section.frm.get_entity();
								if (app) {
									app.lastTrainedDateTime = train_status.startTrainDateTime;
									app_section.frm.set_entity(app);
								}
								is_error = false;
								msg = current_status.msg || "<%=_T('App {0} was successfully trained.')%>";
							}
							else {
								is_error = true;
								if (typeof(current_status.msg) === "string") {
									msg = current_status.msg;
								}
								else if (typeof(current_status.msg) === "object") {
									msg = current_status.msg[train_status.details.failureReason];
								}
							}
						}
						self.__on_success_trained(current_status, train_info.app);
					}
				}
				else {
					self.frm.dashboard.hide_waiting();
					current_status = self.__make_train_app_result(train_info.rasaError);
					if (current_status) {
						if (typeof(current_status.msg) === "string") {
							msg = current_status.msg;
						}
						else if (typeof(current_status.msg) === "object") {
							msg = current_status.msg[train_info.details.failureReason];
						}
					}
					self.__on_error_train(current_status);
				}
				if (!is_error) {
					msg = msg || "<%=_T('App {0} was successfully trained.')%>";
					type = "Info";
				}
				else {
					msg = msg || "<%=_T('Error Training App {0}.')%>";
					type = "Error";
				}
				self.frm.enable_all_sections();
				self.frm.dashboard.show_back_button();
				msg = format(msg, app_name);
				self.frm.show_status_message(msg, type, false);
			}
		);
	};
}());
