/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const format = inConcert.Utils.format;

	var must_init_full_screen = function (dashboard) {
		const frm = dashboard?.view?.form;
		if (frm && frm.has_sections() && frm.current_section && !frm.is_new) {
			return false;
		}

		return true;
	};
	//------------------------------------------------------
	// LUIS BOT navigation config
	//
	var bot_luis_config = function(/*base_uri*/)
	{
		let nlu_type = " 'LUIS'";

		var get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				var err_messages = {
					"ALLEGRO_INTEGRATION_ERROR" : "<%=_T('Unable to delete {0} Cognitive Service. Allegro integration error.')%>",
					"FORBIDDEN"                 : "<%=_T('Unable to delete {0} Cognitive Service. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} Cognitive Services.')%>";
				message = format(message, name);
			}
			return message;
		};

		var luisAPI = inConcert.app.getLuisAPI();

		return {
			id: "cognitive_services",
			name: "<%=_T('Cognitive Services')%>",
			can_show: function() {
				return inConcert.app.is_enabled_nlu();
			},
			permissions: {
				view: [
					"view_cognitive_services"
				],
				save: "save_cognitive_services" ,
				erase: "delete_cognitive_services",
				permission_type: "vcc"
			},
			class_name: "itemCampaign",
			i_class_name: "far fa-user-robot",
			view: {
				render: "/inconcert/apps/cognitive_services/pages/view_cognitive_services_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Cognitive Services')%>" + nlu_type,
				tagline: "<%=_T('Tagline for Cognitive Services')%>",
				new_button_name: "<%=_T('Create cognitive service')%>",
				row_id: "id", // sale de aca (*)
				display_name: "name",
				change_nav_sections: true,
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				messages: {
					erase: "<%=_T('Do you want to delete Cognitive Services {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase: function(app_id, _, callback) {
						var apps = [];
						apps.push(app_id);
						return luisAPI.deleteApps(apps, callback);
					},
					search: function(callback/*, filter*/) {
						return luisAPI.loadApps(callback);
					}
				},
				source: ["id", "name", "description"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "name"       , name: "<%=_T('Name')%>"       },
					{id: "id"         , name: "<%=_T('App Id')%>"     },
					{id: "description", name: "<%=_T('Description')%>"},
					{id: "_action"    , name: "", system_action: {edit: true, erase: true}}
				],
				filter: {fields: {"name": {control: null}, "id": {control: null}, "description": {control: null}}},
				realtime: null,
				must_init_full_screen: must_init_full_screen
			},
			form: {instance: inConcert.adm_form_cognitive_services}
		};
	};

	//------------------------------------------------------
	// RASA BOT navigation config
	//
	var bot_rasa_config = function(/*base_uri*/)
	{
		const rasaAPI = inConcert.app.getRasaAPI();

		var get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				let default_err_msg = "<%=_T('Error deleting {0} Cognitive Services.')%>";
				let err_messages = {
					"ALLEGRO_INTEGRATION_ERROR" : "<%=_T('Unable to delete {0} Cognitive Service. Allegro integration error.')%>",
					"FORBIDDEN"                 : "<%=_T('Unable to delete {0} Cognitive Service. You dont have permission.')%>"
				};
				let err_type = err;
				if (typeof err === "object") {
					err_type = err.result || "";
				}
				message = err_messages[err_type.toUpperCase()];
				if (!message) {
					message = rasaAPI.getRasaErrorMsg(err);
				}
				message = format(message || default_err_msg, name);
			}
			return message;
		};

		let server_info = inConcert.app.getServerInfo();

		var get_value = function(row, type)
		{
			let resp = {};

			switch(type)
			{
				case "state":
					let nodes_pending_to_update;
					let last_train_step = rasaAPI.getLastTrainInfo(row);

					if (last_train_step) {
						let current_status_id = last_train_step.statusId;
						let is_fail = current_status_id === "FAIL TRAIN";
						if (!last_train_step.isRunning && !is_fail) {
							if (server_info && (server_info.nodesInfo && (server_info.nodesInfo.nodes || []).length > 1)) {
								nodes_pending_to_update = (row.publications || []).length;
							}
						}
					}

					let current_status = rasaAPI.makeTrainAppResult(last_train_step);
					if (nodes_pending_to_update && nodes_pending_to_update > 0) {
						current_status = rasaAPI.makeTrainAppResult({statusId: "PARTIALLY TRAIN - SOME NODES REMAIN TO UPDATE"});
					}
					current_status = current_status || rasaAPI.makeTrainAppResult({statusId: "NEVER TRAINED"});

					const remove_dot = function(text) {
						return text && text.charAt(text.length - 1) === '.' ? text.slice(0, -1): text;
					};
					let msg = remove_dot(current_status.desc);

					resp.value = msg;
					resp.class_name = current_status.class_name;
					resp.original_value = row.trainInfo || [];
				break;
			}

			return resp;
		};

		var get_tooltip = function(row, type)
		{
			let tooltip = "";

			switch(type)
			{
				case "state":
					let nodes_msg;
					let nodes_pending_to_update;
					let last_train_step = rasaAPI.getLastTrainInfo(row);

					if (last_train_step) {
						let current_status_id = last_train_step.statusId;
						let is_fail = current_status_id === "FAIL TRAIN";
						if (!last_train_step.isRunning && !is_fail) {
							if (server_info) {
								if (server_info.nodesInfo && (server_info.nodesInfo.nodes || []).length > 1) {
									nodes_pending_to_update = (row.publications || []).length;
									if (nodes_pending_to_update === 0) {
										nodes_msg = "<%=_T('All nodes updated')%>";
									}
									else if (nodes_pending_to_update === 1) {
										nodes_msg = format("<%=_T('{0} node remain to be updated.')%>", nodes_pending_to_update);
									}
									else {
										nodes_msg = format("<%=_T('{0} nodes remain to be updated.')%>", nodes_pending_to_update);
									}
								}
							}
						}
						else {
							if (!is_fail) {
								let time_ago = inConcert.Utils.timeAgo(last_train_step.startTrainDateTime * 1000);
								nodes_msg = time_ago;
							}
						}
					}

					let current_status = rasaAPI.makeTrainAppResult(last_train_step);
					if (nodes_pending_to_update && nodes_pending_to_update > 0) {
						current_status = rasaAPI.makeTrainAppResult({statusId: "PARTIALLY TRAIN - SOME NODES REMAIN TO UPDATE"});
					}
					current_status = current_status || rasaAPI.makeTrainAppResult({statusId: "NEVER TRAINED"});

					const remove_dot = function(text) {
						return text && text.charAt(text.length - 1) === '.' ? text.slice(0, -1): text;
					};
					let msg = remove_dot(current_status.desc);
					if (nodes_msg) {
						msg = format("{0} - {1}", msg, remove_dot(nodes_msg));
					}

					tooltip = msg || "";
				break;
			}

			return tooltip;
		};

		// is action enabled
		var is_action_enabled = function(action, row)
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */

			let last_train_step = rasaAPI.getLastTrainInfo(row);
			if (last_train_step) {
				switch(action.toLowerCase())
				{
					case "edit":
						if (last_train_step.isRunning) {
							return 0;
						}
						return 1;
	
					case "delete":
						if (last_train_step.isRunning) {
							return 0;
						}
						return 1;
				}
			}
			return 1;
		};

		// columnas en comun
		let columns = [
			{id: "_number"    , name: "", type: "counter"     },
			{id: "name"       , name: "<%=_T('Name')%>"       },
			{id: "id"         , name: "<%=_T('App Id')%>"     },
			{id: "description", name: "<%=_T('Description')%>"},
			{id: "State"      , name: "<%=_T('State')%>", type:"chip", get_value: function(row) {
				return get_value(row, "state");
			}, get_tooltip: function(row) {
				return get_tooltip(row, "state");
			}},
			{id: "imported"   , name: "<%=_T('Imported')%>"    , type: "boolean", class_name: "booleanColumnTitle", get_tooltip: function(item) {
				return item.imported? "<%=_T('Yes')%>": "<%=_T('No')%>";
			}},
		];

		// columnas con acciones
		columns.push({id: "_action"    , name: "", system_action: {
				edit: true, erase: true
			},
			is_enabled: is_action_enabled
		});
	
		return {
			id: "cognitive_services",
			name: "<%=_T('Cognitive Services')%>",
			can_show: function() {
				return inConcert.app.is_enabled_nlu();
			},
			permissions: {
				view: [
					"view_cognitive_services"
				],
				save: "save_cognitive_services" ,
				erase: "delete_cognitive_services",
				permission_type: "vcc"
			},
			class_name: "itemCampaign",
			i_class_name: "far fa-brain",
			view: {
				render: "/inconcert/apps/cognitive_services/pages/view_cognitive_services_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Cognitive Services')%>",
				tagline: "<%=_T('Tagline for Cognitive Services')%>",
				new_button_name: "<%=_T('Create cognitive service')%>",
				row_id: "id", // sale de aca (*)
				display_name: "name",
				change_nav_sections: true,
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				messages: {
					erase: "<%=_T('Do you want to delete Cognitive Services {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase: function(app_id, _, callback) {
						let apps = [];
						apps.push(app_id);
						return rasaAPI.deleteApps(apps, callback);
					},
					search: function(callback/*, filter*/) {
						return rasaAPI.loadApps(callback);
					}
				},
				source: ["id", "name", "description", "imported", "trainInfo", "publications"],
				columns: columns,
				filter: {fields: {"name": {control: null}, "id": {control: null}, "description": {control: null}}},
				realtime: null,
				must_init_full_screen: must_init_full_screen
			},
			form: {instance: inConcert.adm_form_rasa}
		};
	};
	
//------------------------------------------------------
	// cognitive_services navigation config
	// BOT (LUIS or RASA)
	var cognitiveservices_config = function(base_uri)
	{
		let vcc_info = inConcert.app.get_vcc_info();
		if (!vcc_info) {
			return false;
		}
		let bot_info = vcc_info.BotInfo;
		if (!bot_info || vcc_info.BotInfo.Type.toUpperCase() === "LUIS") {
			return bot_luis_config(base_uri);
		}
		else if (vcc_info.BotInfo.Type.toUpperCase() === "RASA") {
			return bot_rasa_config(base_uri);
		}
		throw("Unrecognized both " + vcc_info.BotInfo.Type);
	};

	//------------------------------------------------------
	// get all navigation config
	const get_navigation_config = function()
	{
		return {
            cognitive                : cognitiveservices_config(),
		};
	};

	function cognitive_services_config() {}

	if (typeof inConcert.cognitive_services_config !== "function") {
		inConcert.cognitive_services_config = cognitive_services_config;
	}

	cognitive_services_config.prototype.get_app_config = function()
	{
		return {
			id         : "cognitive_services",
			name       : "<%=_T('Cognitive Services')%>",
			selected   : false,
			class_name : "appAdministration",
			main_html  : "/inconcert/apps/cognitive_services/pages/conf_cognitive_services_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category("cognitive_services", "view")
				&& inConcert.app.is_enabled_nlu();
			},
			css : [
				{ id: "adm_messages_css"                            , href: "/inconcert/apps/administration/css/administrationRules.css#grunt-cache-bust"                       },
                { id: "skin_occ_internal_forms_css"                 , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"          },
                { id: "skin_occ_adm_messenger_css"                 	, href: "/inconcert/apps/administration/css/skin-2021/administrationPageMessenger.css#grunt-cache-bust"     },
                { id: "skin_occ_master_buttons_css"                 , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                            },
				{ id: "skin_occ_inbox_master_css"                   , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                              },
                { id: "skin_occ_messages_tooltips_css"              , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"               },
				{ id: "skin_occ_adm_master_css"                     , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"            },
				{ id: "skin_occ_inbox_master_popups_css"            , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                        },
				{ id: "skin_occ_master_client_window_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                       },
				{ id: "skin_occ_custom_visual_search_css"           , href: "/inconcert/apps/agent/css/skin-2021/customVisualSearch.css#grunt-cache-bust"                       }
			]
		};
	};
}());
