-- configuration application

local orbit = require "orbit"
require "inconcert.orbit.application"
require("inconcert.translators")
local json = require "json"
local cosmo = require "cosmo"
local lfs = require "lfs"
require "webHandler.path"

local m_appLocation = select(2, ...)

local m_requireArgs = { ... }

local http = require "socket.http"
local ssl = ssl
local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", function()
	return require ("ssl.https")
end)

-- TODO:
-- ver si los permisos los configuro as� tambien o vienen como par�metro en la url del auth_request
-- hacer una p�gina para manejar cuando un usuario NO da permiso


module(..., package.seeall, orbit.new, inconcert.orbit.application.Decorate)
local m_htmlRender = require"inconcert.render".new("configuration")

--App status control table for the appReadyFilter
--It validates the configuration received from the config server
app_status = {
	actualStatus = "ready",
        
	CheckStatus = function ()
		return "ready"
	end
}

-- 
-- This function gets called when the web server initializes this application (once per VM)
function Initialize(config)
	LogInfo("apps.configuration: Starting application Configuration")
	return true
end

--scripts and pages

local function makeTextTranslator()
	local translator = inconcert.translators.configuration
	return function(...)
		return translator:Text(...)
	end, i18nlib.LCID[translator.language]
end

function GetApplicationTitle()
	local _T = makeTextTranslator()
	return _T("configuration")
end

local htmlRender = require"inconcert.render".new("configuration", {
	MainTitle = _M.GetApplicationTitle,
	Title = function() return _T("configuration") end,
	model = nil,
	makeTextTranslator = makeTextTranslator
})

--paginas
local function render_pages(web, name, versionCode, languageCode)
	if (languageCode ~= "") then
		name = name .. "." .. versionCode
	else
		languageCode = versionCode
	end
	inconcert.authUser = { languageNumber = tonumber(languageCode) }
	local _T, _L = makeTextTranslator()
	local env = { _T = _T, _L = _L }
	return htmlRender:RenderPageLP("pages/" .. name .. ".html", web, env)
end

--scripts
function render_script(web, name, versionCode, languageCode)
	if (languageCode ~= "") then
		name = name .. "." .. versionCode
	else
		languageCode = versionCode
	end
	inconcert.authUser = { languageNumber = tonumber(languageCode) }
	local _T, _L = makeTextTranslator()
	local env = { _T = _T, _L = _L }
	return htmlRender:ScriptLp("scripts/" .. name .. ".js", web, env)
end

_M:dispatch_get(render_script, "/scripts/([^.]+)%.([^.]+)%.?([^.]*)%.js")
_M:dispatch_get(render_pages, "/pages/([^.]+)%.([^.]+)%.?([^.]*)%.html")

function serve_file(app_module, cache)
	return function (web)
		local filename = string.sub(web.path_info, 2, #web.path_info)
		if not cache then
			app_module.response.DontCache(web)
		end
		return _M:serve_static(web, m_appLocation.."/"..filename)
	end
end

--
--  styles and resources
--  NOTA: ESTO SE PROCESA EN XAVANTE
_M:dispatch_get(_M:serve_file(), "/images/.+", "/styles/.+", "/css/.+", ".+%.mp3")
