/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert*/
/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	function conf_api() { /* the constructor */ }

	/* -------------------------------------------------------------------------
	 * Dashboard API
	 */
	conf_api.prototype.load_content_page = function( params )
	{
		if (typeof params !== "object" || typeof params.URI !== "string") {
			throw "Configuration API - Failed to render page. Parameter URI is required";
		}
		var callback = params.callback || $.noop;
		var html_URI = inConcert.app.getBaseDomain() + params.URI;

		inConcert.invoke({
			url         : html_URI,
			type        : "GET",
			contentType : "text/html",
			success     : function(content) { callback(true /*success*/, content); },
			error       : function(xhr, error) { throw "Configuration API - Failed to render page: '" + html_URI + "'"; }
		});
	};

	if ( typeof inConcert.conf_api !== "function" ) {
		inConcert.conf_api = conf_api;
	}

	/* -------------------------------------------------------------------------
	 * View API
	 */

	conf_api.prototype.search = function( params )
	{
		var uri      = params.uri;
		var callback = params.callback;
		var filter   = JSON.stringify( params.filter );
		var request  = params.request || {};

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : filter,
			dataType    : "json",
			contentType : "application/json",
			timeout     : request.timeout, // no requerido
			success     : function( response ) { callback( true, response ); },
			error       : function( xhr, text_status ) { callback( false, text_status ); }
		});

		return request_handler;
	};

	conf_api.prototype.erase = function( params )
	{
		var uri                = params.uri;
		var callback           = params.callback;
		var entities_to_delete = JSON.stringify( params.entities_to_delete || [] );
		var request            = params.request || {};

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : entities_to_delete,
			dataType    : "json",
			contentType : "application/json",
			timeout     : request.timeout, // no requerido
			success: function() {
				callback(true);
			},
			error: function(xhr, text_status) {
				var err;
				try {
					err = JSON.parse(text_status);
					err = err.result;
				}
				catch(e) {
					err = text_status;
				}
				callback(false, err);
			}
		});

		return request_handler;
	};


	/* -------------------------------------------------------------------------
	 * Form API
	 */

	conf_api.prototype.save = function( params )
	{
		var uri      = params.uri;
		var callback = params.callback;
		var data     = JSON.stringify( params.data );
		var request = params.request || {};

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : data,
			dataType    : "json",
			contentType : "application/json",
			timeout     : request.timeout, // no requerido
			success: function( response ) {
				callback( true /* success */, response );
			},
			error: function(xhr, text_status) {
				var err;
				try {
					err = JSON.parse(text_status);
					err = err.result;
				}
				catch(e) {
					err = text_status;
				}
				callback(false /* not a success */ , err);
			}
		});

		return request_handler;
	};

	conf_api.prototype.get_entity = function( params )
	{
		var uri = params.uri;
		var callback = params.callback || $.noop;
		var request  = params.request || {};

		inConcert.invoke({
			url: uri,
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			timeout: request.timeout, // no requerido
			success: function(entity) {
				callback(true, entity);
			},
			error: function(xhr, text_status) {
				var err;
				try {
					err = JSON.parse(text_status);
					err = err.result;
				}
				catch(e) {
					err = text_status;
				}
				callback(false, err);
			}
		});
	};
}());
