/*jshint quotmark:false, -W014, esversion: 11*/
/*global $, inConcert */
/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	/*jslint validthis:true */
	"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form basic class
	function conf_form( params )
	{
		params = params || {};
		this.dashboard = params.dashboard;
		this.action = params.action;
		this.is_new = params.is_new || false;
		this.cached_content = null;
		this.$current_row = null;
		// TODO: esa pagina no deberia estar en configuration?
		this.html_file_name = "/inconcert/apps/administration/pages/frm_empty_content.html#grunt-cache-bust";
		this.instance_config = null;
		this.validation = new inConcert.conf_form_validation( { form: this } );
		this.include_save_and_continue = true;
		this.is_read_only = false;
		this.sections = undefined;
		this.current_section = null;
		this.events_to_suscribe = null;
		this.permissions_type = "vcc"; // vcc or campaign ( default is vcc )
		this.entity = null;
		this.classes_name_frm_container = null;
		this.not_show_required_message = false;
		this.timeout_handle_waiting = null;
		this.$waiting_container = null;
		this.$autocomplete_list = {};
		return true;
	}

	if ( typeof inConcert.conf_form !== "function" ) {
		inConcert.conf_form = conf_form;
	}

	conf_form.prototype.get_entity_instance  = function()           { return this.entity;           };
	conf_form.prototype.set_entity_instance  = function( entity )   { this.entity = entity;         };
	conf_form.prototype.is_readonly          = function()           { return this.is_read_only;     };
	conf_form.prototype.set_readonly         = function( readonly ) { this.is_read_only = readonly; };
	conf_form.prototype.get_permissions_type = function()           { return this.permissions_type; };
	conf_form.prototype.is_creating          = function()           { return this.is_new;           };
	conf_form.prototype.set_new_flag         = function(is_new)     { this.is_new = is_new;         };
	conf_form.prototype.set                  = $.noop;
	conf_form.prototype.get                  = $.noop;
	conf_form.prototype.empty_controls       = $.noop;
	conf_form.prototype.apply_initial_config = $.noop;

	conf_form.prototype.initialize = function()
	{
		if ( this.classes_name_frm_container ) {
			this.add_css_class( this.classes_name_frm_container );
		}
		if (this.has_sections()) {
			$.each(this.sections, function(_, section)
			{
				if (!section.frm) {
					return true; // continue
				}
				section.frm.init();
			});
		}
	};

	conf_form.prototype.dispose = function()
	{
		this.__destroy_all_autocomplete();
		if (this.has_sections()) {
			$.each(this.sections, function(_, section)
			{
				if (!section.frm) {
					return true; // continue
				}
				section.frm.dispose();
			});
		}
		this.remove_css_class();
		this.popup_dispose();
		this.current_section = null;
		this.set_edit_mode(false);
		this.dashboard.disabled_all_menus(false);
		this.onReadySections(false);
	};

	conf_form.prototype.validate_entity = function()
	{
		const errors = [];
		if (this.has_sections()) {
			$.each(this.sections, function(_, section)
			{
				if (!section.frm) {
					return true; // continue
				}
				const result = section.frm.validate_entity();
				if (!result.success) {
					errors.push(result);
				}
			});
		}
		if (errors.length > 0) {
			const errMessages = errors.map(err => (err.errMsg || "")).join("<br>");
			if (errMessages.length > 0) {
				$.inconcertPopup.Show(
					"<%=_T('Update Problems.')%>",
					errMessages,
					$.inconcertPopup.ButtonSets.BS_OK,
					function(/*response*/) {},
					undefined,
					{
						popup_id        : "agent_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			}
		}
		return errors;
	};

	conf_form.prototype.on_scroll = function()
	{
		this.__close_all_autocomplete();
	};

	conf_form.prototype.uri_get_entity = function()
	{
		if (!this.get_uri) {
			return;
		}
		if (this.is_new) {
			return "";
		}
		if (!this.$current_row) {
			return "";
		}
		const name = this.$current_row.attr("row_id");
		return inConcert.app.getBaseDomain() + this.get_uri + encodeURIComponent(name);
	};

	conf_form.prototype.get_entity_id = function()
	{
		if (!this.$current_row) {
			if (this.__entity_id) {
				var entity = this.get_entity_instance();
				if (entity) {
					return entity[this.__entity_id];
				}
			}
			return null;
		}
		return this.$current_row.attr("row_id") || null;
	};

	conf_form.prototype.get_entity_display_name = function()
	{
		var display_name;
		if (this.$current_row) {
			display_name = this.$current_row.attr("row_name") || this.$current_row.attr("row_id");
		}
		if (!display_name) {
			display_name = this.get_entity_id();
		}
		display_name = inConcert.app.get_id(display_name || "");
		return this.sanitize_value(display_name);
	};

	conf_form.prototype.$get_dom_element_by_id = function( id )
	{
		return this.dashboard.$get_dom_element_by_id( id );
	};

	conf_form.prototype.show_waiting = function( message, $waiting_container )
	{
		const self = this;

		var show_waiting = function() {
			self.$waiting_container = $waiting_container || null;
			self.dashboard.show_waiting( message, $waiting_container );
		};

		if (this.$waiting_container) {
			this.hide_waiting();
		}
		show_waiting();
	};

	conf_form.prototype.hide_waiting = function()
	{
		const self = this;
		if (!this.dashboard) {
			return;
		}
		if (this.timeout_handle_waiting) {
			window.clearTimeout(this.timeout_handle_waiting);
			this.timeout_handle_waiting = null;
		}
		if (this.is_loading_sections()) {
			this.timeout_handle_waiting = window.setTimeout(function() {
				self.hide_waiting();
			}, 500);
			return;
		}
		this.dashboard.hide_waiting(this.$waiting_container);
		this.$waiting_container = null;
	};

	/*
		Funcion que obtiene el contenedor de los mensajes de Info, Error, segun si se debe de visualizar en un formulario o en una vista.
		Esto se realiza a partir del nuevo skin del 2021, ya que el contenedor del mensaje ha cambiado de lugar.
		Dejo de ser generico y estar en un unico lugar en el DOM; ahora existe uno diferente para que cada vista (view o frm) en el DOM.
	*/
	conf_form.prototype.__$skin_message_container = function($custom_message_container)
	{
		if ($custom_message_container) {
			return $custom_message_container;
		}
		let skin = inConcert.app.get_skin_name_of_current_app();
		if (skin !== "skinOCCNew") {
			return $custom_message_container;
		}
		const $form_message_container = this.$get_dom_element_by_id("form_message");
		if ($form_message_container.length === 0) {
			return $custom_message_container;
		}
		return $form_message_container;
	};

	conf_form.prototype.show_status_message = function(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes)
	{
		const close = false;
		scrolling = typeof(scrolling) === "boolean" ? scrolling : true; // default is true
		this.hide_waiting();
		$custom_message_container = this.__$skin_message_container($custom_message_container);
		this.dashboard.show_status_message(message, type, timeout_ms, close, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes);
	};

	conf_form.prototype.hide_status_message = function(timeout_ms, callback, $custom_message_container)
	{
		$custom_message_container = this.__$skin_message_container($custom_message_container);
		this.dashboard.hide_status_message(timeout_ms, callback, $custom_message_container);
	};

	conf_form.prototype.show_status_message_in_view = function(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes)
	{
		$custom_message_container = $custom_message_container || this.$get_dom_element_by_id("view_message");
		this.dashboard.show_status_message(message, type, timeout_ms, close, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes);
	};

	conf_form.prototype.hide_status_message_in_view = function(timeout_ms, callback, $custom_message_container)
	{
		$custom_message_container = $custom_message_container || this.$get_dom_element_by_id("view_message");
		this.dashboard.hide_status_message(timeout_ms, callback, $custom_message_container);
	};

	conf_form.prototype.show_status_message_in_both = function(message, type, timeout_ms, scrolling, show_custom_button, $frm_custom_message_container, $view_custom_message_container, callbacks, wrap_classes_view, wrap_classes_form)
	{
		// el formulario
		$frm_custom_message_container = this.__$skin_message_container($frm_custom_message_container);
		this.show_status_message(message, type, timeout_ms, scrolling, show_custom_button, $frm_custom_message_container, callbacks, wrap_classes_form);
		// la vista
		$view_custom_message_container = $view_custom_message_container || this.$get_dom_element_by_id("view_message");
		this.show_status_message_in_view(message, type, timeout_ms, scrolling, show_custom_button, $view_custom_message_container, callbacks, wrap_classes_view);
	};

	conf_form.prototype.hide_status_message_in_both = function(timeout_ms, callback, $frm_custom_message_container, $view_custom_message_container)
	{
		// el formulario
		$frm_custom_message_container = this.__$skin_message_container($frm_custom_message_container);
		this.dashboard.hide_status_message(timeout_ms, callback, $frm_custom_message_container);
		// la vista
		$view_custom_message_container = $view_custom_message_container || this.$get_dom_element_by_id("view_message");
		this.dashboard.hide_status_message(timeout_ms, callback, $view_custom_message_container);
	};

	conf_form.prototype.hide_messages = function( timeout_ms /* no requerido */, callback /* no requerido */, $custom_message_container /* no requerido */ )
	{
		const self = this;
		callback = callback || $.noop;
		this.hide_status_message(timeout_ms,
			function() {
				self.hide_waiting();
				const $errors = self.$get_dom_element_by_id("form_data_container .formFieldValidation");
				$errors.hide();
				callback();
			},
			$custom_message_container
		);
	};

	conf_form.prototype.get_frm_section = function( section_id )
	{
		if ( !this.sections || typeof section_id !== "string" ) {
			return;
		}
		return this.sections[ section_id ].frm;
	};

	conf_form.prototype.show_or_hide_required_message = function(must_show)
	{
		// header required message
		const $required_message = this.$get_dom_element_by_id("form_container .requiredFormMessage");
		must_show = must_show === undefined ? true : must_show;
		if ( this.not_show_required_message || !must_show || this.is_readonly() ) {
			$required_message.hide();
		}
		else {
			$required_message.show();
		}
	};

	conf_form.prototype.popup_render = function( params )
	{
		const self = this;
		this.hide_messages();
		let $view_popup = this.$get_dom_element_by_id("view_popup");
		let html = this.popup_html(params);
		$view_popup.html(html);
		this.popup_on_render(params);
		this.popup_set_events(params);
		this.popup_load_data(params, function() {
			self.popup_show(params);
		});
	};

	conf_form.prototype.popup_dispose = $.noop;

	conf_form.prototype.popup_html = function()
	{
		return ""; // esta funcion se tiene que implementar en la clase hija
	};

	conf_form.prototype.popup_on_render = $.noop;

	conf_form.prototype.popup_load_data = function(_, callback = $.noop)
	{
		callback();
	};

	conf_form.prototype.popup_set_events = $.noop;

	conf_form.prototype.popup_show = function(params)
	{
		this.hide_messages();
		this.popup_hide_errors();
		if (params && params.super_popup) {
			$("body").removeClass("popupDisplay").addClass("popupDisplay");
		}
		var $view_popup = this.$get_dom_element_by_id("view_popup");
		$view_popup.show();
	};

	conf_form.prototype.popup_hide = function()
	{
		var $view_popup = this.$get_dom_element_by_id("view_popup");
		$view_popup.hide();
		$("body").removeClass("popupDisplay");
	};

	conf_form.prototype.popup_hide_errors = function()
	{
		var $view_popup = this.$get_dom_element_by_id( "view_popup" );
		$view_popup.find(".messageError").hide();
		$view_popup.find(".formFieldError").hide();
	};

	conf_form.prototype.popup_show_error = function() {
		const self = this;
		this.popup_hide_errors();
		window.setTimeout(function() {
			self.popup_hide_errors();
		}, 3000);
	};

	conf_form.prototype.err_save_messages = function( err, is_custom_message )
	{
		if (err === "skip_message") {
			return;
		}
		const type = "Error";
		const default_message = "<%=_T('Server error. Unable to save.')%>";
		const timeout_ms = false; //infinite
		let message;
		if ( is_custom_message ) {
			message = err || default_message;
		}
		else {
			let err_messages = {
				FORBIDDEN              : "<%=_T('NOT HAS PERMISSIONS TO UNABLE SAVE')%>",
				INVALID_REQUEST        : "<%=_T('Unable to save. Invalid request.')%>",
				MUST_HAVE_A_DESCRIPTION: "<%=_T('Unable to save. Description taken, choose another one.')%>",
				MUST_HAVE_A_NAME       : "<%=_T('Unable to save. Name already taken, choose another one.')%>",
				OVERFLOW_NAME          : "<%=_T('Unable to save. Name exceeds the allowed length.')%>"
			};
			err_messages["ALREADY EXISTS"]     = err_messages.MUST_HAVE_A_NAME;
			err_messages.MUST_HAVE_UNIQUE_NAME = err_messages.MUST_HAVE_A_NAME;
			if (typeof err === "object") {
				const res = (err.result || "").toUpperCase();
				message = err_messages[ res ]
					|| err.reason
					|| default_message;
			}
			else {
				message = err ? err_messages[ err.toUpperCase() ] : err;
				message = message || default_message;
			}
		}
		this.show_status_message( message, type, timeout_ms );
	};

	conf_form.prototype.get_validation_def = function() { return []; };

	conf_form.prototype.execute_batch_action = function(action, callback = $.noop) {
		const self = this;
		let selected_grid_items = this.get_selected_grid_items();

		if (!selected_grid_items || selected_grid_items.length < 1) {
			let message = "<%=_T('You must select at least one item.')%>";
			this.show_status_message_in_view(message, "Info", 3000);
			return;
		}

		this.show_waiting("<%=_T('Please wait')%>" + "...");

		const response_fn = function(success, result) {
			result = result || {};
			if (success) {
				self.grid_refresh();
				if (result.message !== "") {
					let message = result.message || "<%=_T('Selected items have been processed.')%>";
					self.show_status_message_in_view(message, "Info", 3000);
				}
			}
			else {
				self.hide_waiting();
				if (result.message !== "") {
					let message = result.message || "<%=_T('Error executing batch action.')%>";
					self.show_status_message_in_view(message, "Error", 3000);
				}
				if (result.must_refresh_grid) {
					self.grid_refresh();
				}
			}
			callback(success, result);
		};

		if (typeof(action) === "function") {
			let execute_fn = function() {
				action(selected_grid_items, response_fn);
			};
			execute_fn();
			return;
		}
		else if (typeof(action) === "string") {
			action = action.toLowerCase();
			if (action === "delete") {
				this.execute_batch_delete_action(selected_grid_items, function(success, result) {
					result = result || {};
					if (!result.message) {
						if (result.message !== "") {
							result.message = (success ? "<%=_T('Selected items have been removed.')%>" : "<%=_T('Error executing delete batch action.')%>");
						}
					}
					response_fn(success, result);
				});
			}
			else {
				throw("conf_form.prototype.execute_batch_action - Unrecognized batch action: " + action);
			}
		}
		else {
			throw("conf_form.prototype.execute_batch_action - Unrecognized batch action: " + action);
		}
		
	};

	conf_form.prototype.execute_batch_delete_action  = function(selected_grid_items, callback = $.noop) {
		if (!this.delete_uri) {
			callback(false, {});
			return;
		}
		let files_to_delete = [];
		selected_grid_items.forEach(item => {
			files_to_delete.push(item.Id);
		});
		const api = this.dashboard.get_api();
		api.erase({
			uri               : inConcert.app.getBaseDomain() + this.delete_uri,
			entities_to_delete: files_to_delete,
			callback          : callback
		});
	};

	conf_form.prototype.get_selected_grid_items = function() {
		return this.dashboard.view.get_selected_items();
	};

	conf_form.prototype.grid_refresh = function(callback = $.noop) {
		this.dashboard.grid_refresh(undefined, callback);
	};

	conf_form.prototype.get_batch_menu_options = function() {
		const self = this;
		let system_options = [
			{
				id     : "delete",
				name   : "<%=_T('Delete')%>",
				execute: function() {
					let selected_grid_items = self.get_selected_grid_items();
					if (!selected_grid_items || selected_grid_items.length < 1) {
						let message = "<%=_T('You must select at least one item.')%>";
						self.show_status_message(message, "Info", 3000);
						return;
					}
					let message = FORMAT("<%=_T('You want to delete the {0} selected items.')%>", selected_grid_items.length);
					self.confirm(message, function(success) {
						if (success) {
							self.execute_batch_action("delete");
						}
					});
				}
			}
		];
		return system_options;
	};

	// TODO: Implementar en clase hija (ej: adm_form_shared_files.js)
	conf_form.prototype.get_new_menu_options = $.noop;

	conf_form.prototype.validate = function(save_callback, validation_def, tiemout_ms, $custom_message_container)
	{
		const self = this;

		this.show_waiting("<%=_T('Please wait, validating information')%>");

		save_callback = save_callback || $.noop;
		validation_def = validation_def || this.get_validation_def();

		var validation_callback = function(success) {
			if (!success) {
				self.hide_waiting();
			}
			save_callback(success);
		};

		return this.validation.start(validation_def, validation_callback, tiemout_ms, $custom_message_container);
	};

	conf_form.prototype.valid = function( value, type, options )
	{
		return this.validation.valid( value, type, options );
	};

	conf_form.prototype.validate_field = function( $field, type, message, options, callback_err, timeout_ms, $custom_message_container )
	{
		return this.validation.validate_field( $field, type, message, options, callback_err, timeout_ms, $custom_message_container );
	};

	conf_form.prototype.show_validation_error = function( $field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container, focus )
	{
		return this.validation.show_validation_error( $field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container, focus );
	};

	conf_form.prototype.mark_ctrl_error = function($field)
	{
		this.validation.mark_ctrl_error($field);
	};

	conf_form.prototype.oncreated_wizard = function()
	{
		this.apply_initial_config();
	};

	conf_form.prototype.datepicker_validate_date_and_range = function($from, result)
	{
		const reutrnErr = ($crtl, msg) => {
			result.success = false;
			result.msg = msg;
			result.message = result.msg;
			result.$ctrl = $crtl;
			return result;
		};
		const options = {
			dateFormat: "YYYY-MM-DD"
		};
		// start date
		let start_date = $from.val();
		let validation_result = inConcert.Utils.validate(start_date, "date", options);
		if (!validation_result.success) {
			return reutrnErr(
				$from,
				FORMAT("<%=_T('The from date an invalid date. Allowed format {0}.')%>", options.dateFormat)
			);
		}
		// enddate
		const datepick_to_id = $from.attr("datepicker-rel");
		const $to = $( "#" + datepick_to_id );
		if ($to.length > 0) {
			let start_date_obj = validation_result.dateObject;
			let end_date = $to.val();
			if (end_date.trim().length > 0) {
				validation_result = inConcert.Utils.validate(end_date, "date", options);
				if (!validation_result.success) {
					return reutrnErr(
						$to,
						FORMAT("<%=_T('The to date value an invalid date. Allowed format {0}.')%>", options.dateFormat)
					);
				}
				let end_date_obj = validation_result.dateObject;
				if (start_date_obj > end_date_obj) {
					return reutrnErr(
						$to,
						"<%=_T('The from date cannot be greater than the to date.')%>"
					);
				}
			}
		}
		// ok
		result.success = true;
	};

	conf_form.prototype.datepicker_set_date = function($dtp, date)
	{
		let current_date;
		if (typeof date === "undefined") {
			current_date = new Date(); // now
		}
		else if (date === "") {
			current_date = ""; // vacio
		}
		else {
			let parsed_date = date.match(/(\d\d\d\d)[\/\-](\d\d)[\/\-](\d\d)/);
			current_date = new Date(
				Date.UTC(
					parsed_date[1],                   //year
					parseInt(parsed_date[2], 10) - 1, //month
					parsed_date[3],                   //day
					parsed_date[4] || "12",           //hour
					parsed_date[5] || "00"            //minutes
				)
			);
		}
		$dtp.datepicker("setDate", current_date);
	};

	/*
		este rango es agregado unicamente a los dtp pares (from/to)
	*/
	conf_form.prototype.datepicker_set_range = function($current_dtp, selected_date)
	{
		const datepick_type = $current_dtp.attr( "datepicker-type" );
		const datepick_relation_id = $current_dtp.attr( "datepicker-rel" );
		const $dtp_rel = $( "#" + datepick_relation_id );
		if ( datepick_type === "from" ) {
			$dtp_rel.datepicker( "option", "minDate", selected_date );
		}
		else {
			$dtp_rel.datepicker( "option", "maxDate", selected_date );
		}
	};

	conf_form.prototype.create_datepickers = function(container_id, disabled = false)
	{
	// para crear de forma automatica los controles dtp, tiene que tener en el markup los siguientes attrs:
	//        datepicker='pair' datepicker-type='to' datepicker-rel=
	//      ( mirar ejemplo en reports/schedule )
	//
		const self = this;

		if ( !container_id ) {
			return;
		}
		const $container = $( "#" + container_id );
		const $dates = $container.find( "[datepicker]" );
		const date_format = "yy-mm-dd";

		// este rango es agregado unicamente a los dtp pares (from/to)
		const set_initial_date = function( $current_dtp ) {
			let start_date_increment = $current_dtp.attr( "start-date-increment" );
			if ( start_date_increment ) {
				start_date_increment = parseInt( start_date_increment, 10 );
				if ( !isNaN( start_date_increment ) ) {
					let now = new Date();
					now.setDate( now.getDate() + start_date_increment );
					let start_date = $.datepicker.formatDate( date_format, now );
					$current_dtp.datepicker( "setDate", start_date );
					return;
				}
			}
			let date = $current_dtp.val();
			if (date) {
				date = new Date(date);
			}
			else {
				date = new Date();
			}
			const current_date = $.datepicker.formatDate( date_format, date );
			$current_dtp.datepicker( "setDate", current_date );
		};

		$.each( $dates, function( _, date )
		{
			const $date = $( date );
			let date_value = $date.val();
			if (date_value) {
				date_value = new Date(date_value);
			}
			else {
				date_value = new Date();
			}
			const current_date = $.datepicker.formatDate( date_format, date_value );

			$date.datepicker("destroy");
			$date.datepicker({
				dateFormat : date_format
			});

			set_initial_date( $date );

			const datepick = $date.attr("datepicker");
			if ( datepick === "pair" ) {
				self.datepicker_set_range( $date, current_date );
				$date.datepicker("option", "onClose", function( selected_date )
				{
					const $this = $( this );
					self.datepicker_set_range( $this, selected_date );
				});
			}
			self.disable_control($date, disabled);
		});
	};

	conf_form.prototype.create_hours = function( container_id )
	{
	// para crear de forma automatica los controles hour, tiene que tener en el markup el siguiente attr:
	//        hour='true'
	//      ( mirar ejemplo en reportes/schedule )
	//
		if ( !container_id ) {
			return;
		}
		const $container = $( "#" + container_id );
		const $hours = $container.find( "[hour]" );
		$.each( $hours, function( _, hour )
		{
			const $hour = $( hour );
			$hour.mask( "99:99" );
			const hour_default_value = $hour.attr( "hour-default-value" );
			if ( hour_default_value ) {
				$hour.val( hour_default_value );
			}
		});
	};

	conf_form.prototype.create_spinners = function( container_id )
	{
	// para crear de forma automatica los controles spinner, tienen que tener en el markup los siguientes attrs:
	//        spinner='true' spinner-min-value='1' spinner-max-value='2' spinner-step='1' spinner-overflow='true' spinner-initial-state='disabled'
	//      ( mirar ejemplo en reportes/schedule )
	//        si define spinner-overflow='true', el spinner cuando llega al limite, comienza desde el principio o desde el final
	//
		const self = this;
		if ( !container_id ) {
			return;
		}
		const $container = $( "#" + container_id );
		const $spinners = $container.find( "[spinner]" );

		const get_value = function( $spinner, attr ) {
			const value = parseInt( $spinner.attr( attr ), 10 );
			return isNaN( value ) ? undefined : value;
		};

		const is_valid = function( value, min_value, max_value ) {
			value = parseInt( value, 10 );
			if ( isNaN( value ) ) {
				return false;
			}
			if ( value < min_value || value > max_value ) {
				return false;
			}
			return true;
		};

		const on_keypress = function( event, $spinner, min_value, max_value )
		{
			if ( self.is_readonly() ) {
				event.preventDefault();
				return false;
			}
			if (event.keyCode === 46 || event.keyCode === 8 || event.keyCode === 9 || event.keyCode === 27 || event.keyCode === 13 ||
				(event.keyCode === 65 && event.ctrlKey === true) || (event.keyCode >= 35 && event.keyCode <= 39)) {
				// empty block
			} else {
				if (event.shiftKey || (event.keyCode < 48 || event.keyCode > 57)) {
					event.preventDefault();
					return false;
				}
			}
			const new_char = String.fromCharCode( event.keyCode );
			const value = $spinner.val() + new_char;
			return is_valid( value, min_value, max_value);
		};

		$.each( $spinners, function( _, spinner )
		{
			const $spinner  = $( spinner );
			const min_value = get_value( $spinner, "spinner-min-value" );
			const max_value = get_value( $spinner, "spinner-max-value" );
			const step      = get_value( $spinner, "spinner-step" );

			// crea el spinner de jquery ui
			$spinner.spinner({
				step : step,
				start: min_value
			});

			// valores maximo y minimo
			$spinner.spinner( "option", "min", min_value );
			$spinner.spinner( "option", "max", max_value );

			// initial state
			if ( self.is_readonly() ) {
				$spinner.spinner( "disable" );
			}
			else {
				var initial_state = $spinner.attr( "spinner-initial-state" );
				if ( ( initial_state || "" ).toLowerCase() === "disabled" ) {
					self.disable_control( $spinner, true, false );
				}
				else {
					$spinner.spinner( "enable" );
				}
			}

			// events
			$spinner
				.off( "keypress" )
				.on( "keypress" , function( event ) {
					return on_keypress( event, $spinner, min_value, max_value );
				})
				.off( "paste" )
				.on( "paste" , function() {
					var pre_value = $spinner.val();
					window.setTimeout( function() {
						var new_value = $spinner.val();
						if ( !is_valid( new_value, min_value, max_value) ) {
							$spinner.val( pre_value );
						}
						else {
							new_value = parseInt( new_value, 10 );
							$spinner.val( new_value );
						}
					});
				});
		});
	};

	conf_form.prototype.create_color_pickers = function( container_id )
	{
	// para crear de forma automatica los controles COLOR PICKER de SCPECTRUM, tiene que tener en el markup el siguiente attr:
	//       color-picker='true'
	//      ( mirar ejemplo en campaign/mail accounts )
	//
	//     Referencia: http://bgrins.github.io/spectrum/
	//
		if ( !container_id ) {
			return;
		}
		const $container = $( "#" + container_id );
		const $colors_pickers = $container.find("[color-picker]");
		$.each($colors_pickers, function(_, color)
		{
			const $color = $(color);
			$color.spectrum({
				disabled             : true,
				showPaletteOnly      : true,
				togglePaletteOnly    : true,
				showInput            : true,
				togglePaletteMoreText: "<%=_T('More')%>",
				togglePaletteLessText: "<%=_T('Less')%>",
				chooseText           : "<%=_T('Select')%>",
				cancelText           : "<%=_T('Cancel')%>",
				palette: [
					["#000"   ,"#444"   ,"#666"   ,"#999"   ,"#ccc"   ,"#eee"   ,"#f3f3f3","#fff"   ],
					["#f00"   ,"#f90"   ,"#ff0"   ,"#0f0"   ,"#0ff"   ,"#00f"   ,"#90f"   ,"#f0f"   ],
					["#f4cccc","#fce5cd","#fff2cc","#d9ead3","#d0e0e3","#cfe2f3","#d9d2e9","#ead1dc"],
					["#ea9999","#f9cb9c","#ffe599","#b6d7a8","#a2c4c9","#9fc5e8","#b4a7d6","#d5a6bd"],
					["#e06666","#f6b26b","#ffd966","#93c47d","#76a5af","#6fa8dc","#8e7cc3","#c27ba0"],
					["#c00"   ,"#e69138","#f1c232","#6aa84f","#45818e","#3d85c6","#674ea7","#a64d79"],
					["#900"   ,"#b45f06","#bf9000","#38761d","#134f5c","#0b5394","#351c75","#741b47"],
					["#600"   ,"#783f04","#7f6000","#274e13","#0c343d","#073763","#20124d","#4c1130"]
				]
			});
		});
	};

	conf_form.prototype.create_sliders = function( container_id )
	{
	// para crear de forma automatica los controles slider, tienen que tener en el markup los siguientes attrs:
	//        slider='true' slider-min-value='1' slider-max-value='2' slider-step='1' slider-initial-value='true' slider-initial-state='disabled'
	//      ( mirar ejemplo en outbound_engine/processes )
		const self = this;

		if ( !container_id ) {
			return;
		}
		const $container = $( "#" + container_id );
		const $sliders = $container.find( "[slider]" );

		const get_value = function( $slider, attr, is_number = true) {
			let value = $slider.attr( attr );
			if (!is_number) {
				return value;
			}
			value = parseInt( value , 10 );
			return isNaN( value ) ? undefined : value;
		};

		const set_display_value = function ($ctrl, value, display_format) {
			if (display_format) {
				value = FORMAT(display_format, value);
			}
			$ctrl.text(value);
		};

		const set_display_values = function ($slider_display_value, current_value, $slider_display_remaining_value, display_format, max_value)
		{
			// asigno el valor restante (max value - current value)
			if (max_value && $slider_display_remaining_value && $slider_display_remaining_value.length > 0) {
				const remaining_value = (max_value - current_value);
				set_display_value($slider_display_remaining_value, remaining_value, display_format);
			}
			// asigno el valor actual
			set_display_value($slider_display_value, current_value, display_format);
		};

		$.each( $sliders, function( _, slider ) {
			const $slider                         = $( slider );
			const range                           = get_value( $slider, "slider-range", false );
			const min_value                       = get_value( $slider, "slider-min-value" );
			const max_value                       = get_value( $slider, "slider-max-value" );
			const step                            = get_value( $slider, "slider-step" );
			const initial_value                   = get_value( $slider, "slider-initial-value" );
			const dom_display_value_format        = get_value( $slider, "slider-display-value-format", false);
			// display value
			const dom_display_value_id            = get_value( $slider, "slider-display-value-id", false);
			const $slider_display_value           = $(`#${dom_display_value_id}`);
			// display reamining value
			const dom_display_value_remaining_id  = get_value( $slider, "slider-display-value-remaining-id", false);
			const $slider_display_remaining_value = $(`#${dom_display_value_remaining_id}`);
	
			const slider_generic_event_fn = ( _, ui ) => {
				set_display_values(
					$slider_display_value,
					ui.value,
					$slider_display_remaining_value,
					dom_display_value_format,
					max_value
				);
			};
			// crea el slider de jquery ui
			$slider.slider({
				range: range,
				min  : min_value,
				max  : max_value,
				value: initial_value,
				step : step,
				create: function() {
					slider_generic_event_fn( _, {
						value: $(this).slider("value")
					});
				},
				slide : slider_generic_event_fn,
				change: slider_generic_event_fn
			});

			// initial state
			if ( self.is_readonly() ) {
				$slider.slider( "disable" );
			}
			else {
				const initial_state = $slider.attr( "slider-initial-state" );
				if ( ( initial_state || "" ).toLowerCase() === "disabled" ) {
					self.disable_control( $slider, true, false );
				}
				else {
					$slider.slider( "enable" );
				}
			}

			// custom events
			// TODO: Implement ?
		});
	};

	conf_form.prototype.combo_add_value = function($combo, value_id, description, selected, data_id, data)
	{
		return inConcert.Utils.combo_add_value($combo, value_id, description, selected, data_id, data);
	};

	conf_form.prototype.combo_update_value = function($combo, new_value_id, new_description, old_value_id, data_id, data)
	{
		if (new_value_id === undefined) {
			return false;
		}
		if (typeof new_description === "function") {
			new_description = new_description();
		}
		else {
			new_description = new_description || new_value_id;
		}

		var $option = $combo.find("option[value='" + old_value_id + "']");
		$option.attr("value", new_value_id)
			.html(_T(new_description))
			.attr("selected", "selected");

		if (data) {
			data_id = data_id || "item";
			$option.data(data_id, data);
		}
	};

	conf_form.prototype.combo_remove_value = function($combo, value_id)
	{
		if (value_id === undefined) {
			return false;
		}
		$combo.find("option[value='" + value_id + "']").remove();
		$combo.val("").trigger("change");
		return true;
	};

	conf_form.prototype.$combo_get_selected_item = function($combo)
	{
		return inConcert.Utils.$combo_get_selected_item($combo);
	};

	conf_form.prototype.combo_get_selected_item_text = function($combo)
	{
		return inConcert.Utils.combo_get_selected_item_text($combo);
	};

	conf_form.prototype.combo_get_selected_item_data = function($combo, item_data_name = "item")
	{
		return inConcert.Utils.combo_get_selected_item_data($combo, item_data_name);
	};

	/*
		retorna:
			- Id del item seleccionado.
			- Name del item seleccionado.
			- Objeto data del item seleccionado.
			- Objeto jQuery del item seleccionado.
	*/
	conf_form.prototype.combo_get_selected_item_info = function($combo, item_data_name = "item")
	{
		const item = {
			id   : $combo.val() || "",
			name : this.combo_get_selected_item_text($combo) || "",
			data : this.combo_get_selected_item_data($combo, item_data_name),
			$item: this.$combo_get_selected_item($combo)
		};
		return item;
	};

	/*
		A partir de un $control retorna su par $label
	*/
	conf_form.prototype.__$get_label = function($control)
	{
		const control_id = $control.attr("id");
		const $parent = $control.parent();
		let $label = $parent.find("div[for='" + control_id + "']");
		if ( $label.length === 0 ) {
			$label = $parent.find("label[for='" + control_id + "']");
		}
		return $label;
	};

	conf_form.prototype.disable_control = function( $control, disabled, is_required, is_disable_lbl, keep_value, must_select_blank_item )
	{
		const self = this;
		is_disable_lbl = is_disable_lbl === undefined ? true : is_disable_lbl;
		is_required = is_required === undefined ? false : is_required;

		if (typeof $control !== "object") {
			return;
		}

		const disable = function()
		{
			const is_spinner = $control.attr( "spinner" );
			if ( ( is_spinner || "" ).toLowerCase() === "true" ) {
			// spinner control
				$control.spinner( disabled ? "disable" : "enable" );
				return;
			}
			const is_color_picker = $control.attr( "color-picker" );
			if ( ( is_color_picker || "" ).toLowerCase() === "true" ) {
			// color picker control
				$control.spectrum(disabled ? "disable" : "enable");
				return;
			}
			const is_slider = $control.attr( "slider" );
			if ( ( is_slider || "" ).toLowerCase() === "true" ) {
			// slider control
				$control.slider( disabled ? "disable" : "enable" );
				return;
			}
			const is_datepicker = $control.attr( "datepicker" );
			if ( is_datepicker ) {
			// datepicker control
				$control.datepicker( disabled ? "disable" : "enable" );
				return;
			}
			// simple control
			$control.prop( "disabled", disabled );
		};

		const must_include_required_class = function() {
			if (!is_required) {
				return false;
			}
			if (self.is_readonly()) {
				return false;
			}
			let ctrl_type = ($control.attr("type") || "").toLowerCase();
			if (ctrl_type === "checkbox" || ctrl_type === "radio") {
				return false;
			}
			let custom_required = ($control.attr("custom-required") || "").toLowerCase();
			if (custom_required === "never-required") {
				return false;
			}
			return true;
		};

		const $label = this.__$get_label( $control );
		disable();
		if ( disabled ) {
			// se queda con el valor anterior
			if ( !keep_value ) {
				$control.val( "" );
			}
			$label.removeClass("formLabel-rq");
			$label.removeClass("required");
			if ( is_disable_lbl ) {
				$label.addClass("formLabel-ds").prop("disabled", true);
				//$label.prop("disabled", true);
			}

			// si cambia al estado "disabled" y previamente habia dado un error ( por ej: la cruz indicadora de requerido ), la oculta
			var $validation = $( "#" + $control.attr( "id" ) + "_validation" );
			if ( $validation.length > 0 && $validation.is( ":visible" ) ) {
				this.hide_messages();
			}
		}
		else {
			if ( must_select_blank_item ) {
				$control.val( "" );
			}
			// si el control era readonly tengo que eliminar las clases de este tipo
			$control.removeClass("readonly");
			$label.removeClass("formLabel-ds readonly").prop("disabled", false);
			if ( must_include_required_class() ) {
				$label.addClass("formLabel-rq");
				$label.addClass("required");
			}
		}
	};

	conf_form.prototype.readonly_control = function( $control, readonly, is_required )
	{
		is_required = is_required === undefined ? false : is_required;
		if (typeof $control !== "object") {
			return;
		}
		const $label = this.__$get_label( $control );
		if ( readonly ) {
			$control.prop("disabled", true).addClass("readonly");
			$label.removeClass("formLabel-rq").addClass("formLabel-ds readonly").prop("disabled", true);
			$label.removeClass("required");
		}
		else {
			$control.prop("disabled", false).removeClass("readonly");
			$label.removeClass("formLabel-ds formLabel-rq readonly").prop("disabled", false);
			$label.removeClass("required");
			if ( is_required && !this.is_readonly() ) {
				$label.addClass("formLabel-rq");
				$label.addClass("required");
			}
		}
	};

	conf_form.prototype.disable_label = function( $label, disabled, is_required )
	{
		is_required = is_required === undefined ? false : is_required;
		if (typeof $label !== "object") {
			return;
		}
		if ( disabled ) {
			$label.removeClass("formLabel-rq").addClass("formLabel-ds").prop("disabled", true);
			$label.removeClass("required");
		}
		else {
			$label.removeClass("formLabel-ds").prop("disabled", false);
			if ( is_required && !this.is_readonly() ) {
				$label.addClass("formLabel-rq");
				$label.addClass("required");
			}
		}
	};

	conf_form.prototype.readonly_label = function( $label, disabled, is_required )
	{
		is_required = is_required === undefined ? false : is_required;
		if (typeof $label !== "object") {
			return;
		}
		if ( disabled ) {
			$label.removeClass("formLabel-rq").addClass("formLabel-ds readonly").prop("disabled", true);
			$label.removeClass("required");
		}
		else {
			$label.removeClass("formLabel-ds readonly").prop("disabled", false);
			if ( is_required && !this.is_readonly()  ) {
				$label.addClass("formLabel-rq");
				$label.addClass("required");
			}
		}
	};

	conf_form.prototype.required_label = function( $control, is_required )
	{
		is_required = is_required === undefined ? false : is_required;
		if ( typeof $control !== "object" ) {
			return;
		}
		var $label = this.__$get_label( $control );
		if ($label.length === 0 ) {
			return;
		}
		$label.removeClass( "formLabel-rq" );
		$label.removeClass("required");
		if ( is_required ) {
			$label.addClass("formLabel-rq");
			$label.addClass("required");
		}
	};

	conf_form.prototype.show_waiting_combo = function( $cbo )
	{
		if ( !$cbo ) {
			return;
		}
		var $parent = $cbo.parent();
		var $label = $parent.find("label[for='" + $cbo.attr("id") + "']");

		var is_required = false;
		if ( $label.length > 0 ) {
			is_required = $label.hasClass( "formLabel-rq" );
			is_required = $label.hasClass( "required" );
		}

		if ( is_required ) {
			$label.removeClass( "formLabel-rq" );
			$label.removeClass("required");
		}

		var is_disabled = $cbo.prop( "disabled" );
		$cbo
			.prop( "lbl-is-required", is_required )
			.prop( "before-disabled-state", is_disabled )
			.prop( "disabled", true )
			.removeClass( "formControlLoading" )
			.addClass( "formControlLoading" );

	};

	conf_form.prototype.hide_waiting_combo = function( $cbo )
	{
		if ( !$cbo ) {
			return;
		}
		var is_disabled = $cbo.prop( "before-disabled-state" );
		var is_required = $cbo.prop( "lbl-is-required" );

		$cbo
			.prop( "disabled", is_disabled )
			.removeClass( "formControlLoading" );

		if ( is_required ) {
			var $parent = $cbo.parent();
			var $label = $parent.find("label[for='" + $cbo.attr("id") + "']");
			if ( $label.length > 0 ) {
				$label.addClass( "formLabel-rq" );
				$label.addClass("required");
			}
		}
	};

	conf_form.prototype.set_numeric_event = function(jquery_selector)
	{
		if (typeof jquery_selector !== "string") {
			return;
		}
		var $control = $(jquery_selector);
		if (typeof $control !== "object") {
			return;
		}
		var is_numeric = function (e)
		{
			var key = e.keyCode;
			return ((key >= 48 && key <= 57) || (key >= 96 && key <= 105)) && !(e.altKey || e.shiftKey || e.ctrlKey);
		};

		$control
			.off("paste")
			.on("paste", function(e)
			{
				var text = e.originalEvent.clipboardData.getData("text/plain");
				for (var i = 0; i < text.length; ++i)
				{
					if (text[i] < "0" || text[i] > "9") {
						return false; // parseInt is not good enough
					}
				}
			})
			.off("keydown")
			.on("keydown", function(e)
			{
				var key = e.keyCode;
				if (key === 40 || key === 38) {
				// allow to increment or decrement using arrow keys
					var current_value = $(this).val().length > 0 ? parseInt($(this).val(), 10) : 0;
					var sum = (key === 40 && current_value > 1) ? -1 : (key === 38 && current_value < 99) ? 1 : 0;
					$(this).val(current_value + sum);
					return false;
				}
				return is_numeric(e) || key === 8 || key === 39 || key === 37;
			})
			.off("keypress")
			.on("keypress", function(e)
			{
				return is_numeric(e);
			});
	};

	conf_form.prototype.save_execute = function( after_save, entity, hash_id, customCallback )
	{
		const self = this;
		const view = this.dashboard.get_current_action_obj().view;
		const entity_name = entity[view.display_name];
		/*
		*/
		const callback = customCallback
			|| function( success, response ) {
				if ( success ) {
					if (self.is_new && typeof response.idMapping === "object") {
					// si es nuevo tengo que obtener el id que me genero
						$.each(response.idMapping, function(name, new_id) {
							if (entity_name === name) {
								entity[view.row_id] = new_id;
								return false; // Sale del loop, es un break!!
							}
						});
					}
					self.hide_waiting();
					self.on_saved(success, entity, after_save);
					return;
				}
				// error
				const err = response;
				self.on_saved(success, entity, after_save, err);
				self.hide_waiting();
			};
		let data = {};
		if (hash_id) 
			data[hash_id] = entity;
		else
			data = entity;
		this.dashboard.get_api().save({
			uri     : inConcert.app.getBaseDomain() + this.get_save_uri(),
			data    : data,
			callback: callback,
			request : view.request
		});
	};

	conf_form.prototype.get_save_uri = function()
	{
		if (!this.save_uri) throw new Error("this.save_uri is required");
		if (typeof this.save_uri === "string") return this.save_uri;
		if (typeof this.save_uri === "object") {
			if (this.is_new) {
				if (this.save_uri.new)
					return this.save_uri.new;
				throw new Error("this.save_uri.new is required");
			}
			else {
				if (this.save_uri.update)
					return this.save_uri.update;
				throw new Error("this.save_uri.update is required");
			}
		}
		throw new Error("this.save_uri type is not allowed");
	};

	conf_form.prototype.get_hash_id = function(entity)
	{
		const action = this.dashboard.get_current_action_obj();
		return this.is_new? entity[action.view.display_name] : entity[action.view.row_id];
	};

	conf_form.prototype.make_save_waiting_msg = function(entity)
	{
		entity = entity || this.get();
		const action = this.dashboard.get_current_action_obj();
		const entity_name = entity[action.view.display_name] || "";

		let waiting_message = this.is_new ?
			FORMAT("<%=_T('Please wait, saving {0}')%>", entity_name):
			FORMAT("<%=_T('Please wait, updating {0}')%>", entity_name);

		return {
			hash_id: this.get_hash_id(entity),
			waiting_message: waiting_message
		};
	};

	conf_form.prototype.can_save = function(entity)
	{
		return true;
	};
	
	conf_form.prototype.save = function( after_save, entity)
	{
		const self = this;

		var save = function( success ) {
			if (!success) {
				return;
			}
			entity = entity || self.get();
			entity.isNew = self.is_new ? true: false;

			if(!self.can_save(entity)){
				return;
			}
			const resp = self.make_save_waiting_msg(entity);
			self.show_waiting(resp.waiting_message);
			self.save_execute(after_save, entity, resp.hash_id);
		};

		self.validate(save);
	};

	conf_form.prototype.on_saved = function(success, entity, after_save, err, callback, not_show_grid_message)
	{
		const self = this;

		callback = callback || $.noop;

		this.dashboard.hide_back_button();

		if (!success) {
			this.err_save_messages(err);
			this.dashboard.show_back_button();
			callback(false);
			return;
		}

		if (typeof after_save === "function") {
			after_save(entity);
		}
		else {
			const entity_id = this.get_entity_id();
			let message = "<%=_T('Was successfully added.')%>";
			if (entity_id !== null && this.has_sections() && this.count_enabled_sections(entity_id) === 0) {
			// Si el frm es con secciones (ej campaign, basic, etc) y no tiene permisos para ninguna seccion,
			// tengo que cerrar el frm y regresar a la grilla.
			// Este es el supuesto caso en que solo podria tener permisos para crear campaigns, pero NO para modificar.
				this.close("saved");
			}
			else {
				let message_type = "Info";
				this.show_status_message(message, message_type);

				switch(after_save)
				{
					case "SAVE_AND_CONTINUE":
					// vacia los controles y deja pronto para agregar una nueva entidad.
						this.empty_controls();
					break;

					case "SAVE_AND_NOT_EXIT":
					// no cierra el formulario, deja todo pronto para poder seguir editando la misma entidad.
						this.ready_for_update();
					break;

					case "SAVE_AND_NOT_EXIT_AND_REFRESH_VIEW":
					// refresca la vista y no cierra el formulario, lo deja pronto para editar.
						this.grid_refresh(function(success) {
							self.set_entity_instance( entity );
							self.ready_for_update();
							callback(success);
						});

					return;
//					break;

					case "SAVE_WITHOUT_ACTION":
					// Guarda y no realiza ninguna accion.
					break;

					case "SAVE_AND_REFRESH_VIEW":
					// cierra el formulario, refresca la vista.
						this.close("saved");
						this.grid_refresh();
						callback(true);
					return;

					default:
					// default "save & quit"
						if (!this.action.only_frm) {
							this.close("saved");
						}
						else {
							message_type = "Info";
							message = this.update_ok_message || "<%=_T('Was successfully updated.')%>";
							this.show_status_message(message, message_type);
						}
				}
			}
		}
		if (!this.action.only_frm) {
			// actualizo la grilla de la vista
			this.dashboard.view.update_view(entity, this.$current_row, not_show_grid_message);
		}
		this.dashboard.show_back_button();
		callback(true);
	};

	conf_form.prototype.no_close = function() {
		if (this.instance_config && typeof(this.instance_config.show_view_readonly) === "boolean")
			this.instance_config.show_view_readonly = true;
		this.AnalyticsSend();
	};

	conf_form.prototype.can_exit = function()
	{
		if ( this.action && this.action.only_frm ) {
			return false;
		}
		return true;
	};

	conf_form.prototype.close = function( action )
	{
		this.set_edit_mode(false);
		if ( !this.can_exit() ) {
			this.no_close();
			return;
		}
		this.on_close( action );
		this.hide_messages();
		var $form_container = this.$get_dom_element_by_id("form_container");
		$form_container
			.removeClass()
			.addClass("pageContainer")
			.hide();
		this.$get_dom_element_by_id("view_container").show();
		this.dashboard.view.on_show();
	};

	conf_form.prototype.on_close = function( action )
	{
		const conf = this.instance_config;
		if (conf) {
			if ( action === "saved" && typeof conf.callback_save === "function" ) {
				conf.callback_save( true );
			}
			else if ( action === "cancel" && typeof conf.callback_cancel === "function" ) {
				conf.callback_cancel();
			}
		}
		this.dispose();
	};

	conf_form.prototype.disable_footer_buttons = function(disabled)
	{
		var $save = this.$get_dom_element_by_id("form_button_save");
		var $continue = this.$get_dom_element_by_id("form_button_save_and_continue");
		var $cancel = this.$get_dom_element_by_id("form_button_cancel");

		$save.prop("disabled", disabled);
		$continue.prop("disabled", disabled);
		$cancel.prop("disabled", disabled);
	};

	conf_form.prototype.ready_for_create = function()
	{
		const self = this;

		const $save = this.$get_dom_element_by_id( "form_button_save" );

		if ( this.has_sections() ) {
			this.show_sections();
			const default_section = this.get_default_section();
			if ( default_section ) {
				this.select_section( default_section );
			}
		}

		$save.attr("value", "<%=_T('Create')%>");

		const $save_and_continue = this.$get_dom_element_by_id( "form_button_save_and_continue" );
		if (this.include_save_and_continue && !(this.action && this.action.only_frm)) {
			$save_and_continue
				.prop( "disabled", false )
				.show()
				.off( "click" )
				.on( "click", function() {
					const after_save = "SAVE_AND_CONTINUE";
					self.save( after_save );
			});
		}
		else {
			$save_and_continue
				.prop( "disabled", true )
				.hide()
				.unbind( "click" );
		}
	};

	conf_form.prototype.ready_for_update = function()
	{
		this.is_new = false;
		this.set_readonly(false);
		this.set_title();

		const $save = this.$get_dom_element_by_id("form_button_save");
		let update_caption = "<%=_T('Update')%>";
		if (this.action.form && this.action.form.save_button) {
			update_caption = this.action.form.save_button.update_caption || update_caption;
		}
		$save.attr("value", update_caption);
		this.$get_dom_element_by_id("form_button_save_and_continue")
			.hide()
			.unbind("click");
		if (this.has_sections()) {
//			this.set_events();
			this.disabled_all_sub_menus(false);
			this.show_sections();
		}
		this.validate_entity();
	};

	/*
		Esta funcion agrega evento al ctrl parent de controles relacionados, para que deje enabled/disabled sus controles hijos relacionados.
	*/

	conf_form.prototype.set_related_events = function( $container )
	{
		const self = this;

		var __change_owner_state = function( self, $owner, related_info, default_value )
		{
			var is_owner_checked = $owner.prop("checked");
			var is_owner_disabled = $owner.is(":disabled") === true;
			var $related = $("#" + related_info.related_id);
			var disabled_related, is_required_related, related_previous_value;
			var focus_in_related = true;

			if ( is_owner_checked ) {
				disabled_related = is_owner_disabled || !related_info.disabled;
				is_required_related = true;
				related_previous_value = $related.attr("previous_value");
				if ( typeof related_previous_value === "string" ) {
					$related.val(related_previous_value);
				}
				else {
					if ( typeof default_value === "string" && $related.val().trim().length === 0 ) {
						$related.val( default_value );
					}
				}
			}
			else {
				disabled_related = is_owner_disabled || related_info.disabled;
				is_required_related = false;
				related_previous_value = $related.val();
				if ( typeof related_previous_value === "string" && related_previous_value.trim().length > 0 ) {
					$related.attr("previous_value", related_previous_value);
				}
				$related.val("");
			}
			if ( disabled_related ) {
				is_required_related = false;
				focus_in_related = false;
			}
			else {
				is_required_related = typeof(related_info.required) === "boolean" ? related_info.required : true;
				focus_in_related = typeof(related_info.focus) === "boolean" ? related_info.focus : true;
			}

			self.disable_control($related, disabled_related, is_required_related);

			if (focus_in_related) {
				window.setTimeout(function() {
					$related.focus();
				});
			}
		};

		$container.find("[type='checkbox'][related]")
			.off("change.related_ctrls")
			.on("change.related_ctrls", function() {
				var $owner = $(this);
				var related_ctrls = self.__parse_related_ctrls($owner);
				$.each(related_ctrls, function(related_id, related_info) {
					related_info.related_id = related_id;
					__change_owner_state(self, $owner, related_info);
				});
			});
	};

	/*
		Parsea el atributo "related" del control owner de controles relacionados.
		El valor del attr es un JSON array.--> <input ... related='["ctrl_id1","ctrl_id2"]' />

		En el markup la apertura y cierre del attr "related" debe ser con comillas simples y los valores deben estar entre comillas dobles, sino el parse falla. (JSON.parse)
	*/
	conf_form.prototype.__parse_related_ctrls = function( $ctrl_owner )
	{
		try {
			var encoded_related_ctrls = $ctrl_owner.attr("related");
			var related_ctrls = JSON.parse(encoded_related_ctrls);
			return related_ctrls;
		}
		catch(e) {
			throw("conf_form.prototype.__parse_related_ctrls - related attr is a invalid json array for ctrl: " + $ctrl_owner.attr("id"));
		}
	};

	/*
		Esta funcion deja disbled los controles relacionados dependiente del ctrl owner.
	*/
	conf_form.prototype.related_empty_disabled = function($ctrl_owner)
	{
		var related_ctrls = this.__parse_related_ctrls($ctrl_owner);
		$.each(related_ctrls, function(related_id) {
			var $related = $("#" + related_id);
			$related.prop("previous_value", "").val("");
			$ctrl_owner.prop("checked", false).trigger("change.related_ctrls");
		});
	};

	conf_form.prototype.realtime_update = $.noop;

	conf_form.prototype.load_events_suscribe = function()
	{
		// esta funcion es implementada en la clase hija, y se definen los eventos realtime (ejemplo adm_form_campaigns.js)
		this.events_to_suscribe = {};
	};

	conf_form.prototype.event_suscribe = function( app_id, event_id, config )
	{
		this.dashboard.realtime.event_suscribe( app_id, event_id, config );
	};

	conf_form.prototype.event_unsuscribe = function( app_id, event_id )
	{
		this.dashboard.realtime.event_unsuscribe( app_id, event_id );
	};

	conf_form.prototype.events_suscribe = function()
	{
		const self = this;
		var app_id = this.dashboard.current_app;
		this.load_events_suscribe();
		var events_to_suscribe = this.events_to_suscribe || {};
		$.each( events_to_suscribe, function( event_id, config ) {
			self.event_suscribe( app_id, event_id, config );
		});
	};

	conf_form.prototype.events_unsuscribe = function()
	{
		const self = this;
		var app_id = this.dashboard.current_app;
		var events_to_suscribe = this.events_to_suscribe || {};
		$.each( events_to_suscribe, function( event_id ) {
			self.event_unsuscribe(app_id, event_id);
		});
	};

	conf_form.prototype.get_default_section = function()
	{
		if (!this.has_sections()) {
			return;
		}
		var default_action;
		$.each( this.sections, function(section_id, section)
		{
			if (section.selected) {
			// en cuanto encuentro la default, salgo del loop
				default_action = section_id;
				return false; // break
			}
		});
		return default_action;
	};

	conf_form.prototype.ready_for_save = function()
	{
		const self = this;
		this.set_readonly(false);

		this.show_or_hide_required_message(true);

		var $save = this.$get_dom_element_by_id( "form_button_save" );
		$save
			.prop( "disabled", false)
			.show()
			.off( "click" )
			.on( "click", function() {
				self.save();
			});
		this.set_events();
		this.events_suscribe();
		if ( this.is_new ) {
			this.ready_for_create();
		}
		else {
			this.ready_for_update();
		}
		this.editable_buttons();
		this.apply_initial_config();
		this.hide_waiting();
		this.set_edit_mode(true);
	};

	conf_form.prototype.unable_for_save = function(err, callback = $.noop)
	{
		const $save = this.$get_dom_element_by_id("form_button_save");
		const $save_continue = this.$get_dom_element_by_id("form_button_save_and_continue");

		$save.prop("disabled", true)
			.unbind("click");
		$save_continue.prop("disabled", true)
			.unbind("click");

		if ( this.action && this.action.only_frm ) {
			$save.attr("value", "<%=_T('Update')%>");
			$save_continue.hide();
		}
		this.readonly_all_crtls();
		this.set_events();
		this.hide_waiting();
		if (this.has_sections()) {
			this.dashboard.show_back_button();
		}

		callback();
	};

	conf_form.prototype.editable_all_crtls = function( specific_selector /* opcional */ )
	{
		var selector = "form_data_container";

		if ( specific_selector ) {
		// solo los controles de la seccion
			selector = selector + specific_selector;
		}

		var $conf_form_data_container = this.$get_dom_element_by_id( selector ); // por defecto el container generico de todos los frms

		// editable all textbox
		$conf_form_data_container.find(".form-group:not(.customEditable) input:not([force_ds='true'])")
			.prop("disabled", false)
			.removeClass("readonly");

		// editable all combos
		$conf_form_data_container.find(".form-group:not(.customEditable) select:not([force_ds='true'])")
			.prop("disabled", false)
			.removeClass("readonly");

		// editable all textarea
		$conf_form_data_container.find(".form-group:not(.customEditable) textarea:not([force_ds='true'])")
			.prop("disabled", false)
			.removeClass("readonly");

		// remove all required class
		var $lbls = $conf_form_data_container.find(".form-group:not(.customEditable) .formLabel:not([force_ds='true'])");
		$.each( $lbls, function( _, lbl ) {
			var $lbl = $( lbl );
			var old_classes = $lbl.attr( "old_classes" );
			if ( typeof old_classes === "string" ) {
				$lbl.removeClass().addClass( old_classes );
				$lbl.removeAttr( "old_classes" );
			}
		});
	};

	conf_form.prototype.readonly_all_crtls = function( specific_selector /* opcional */ )
	{
		var selector = "form_data_container";

		if ( specific_selector ) {
		// solo los controles de la seccion
			selector = selector + specific_selector;
		}

		var $conf_form_data_container = this.$get_dom_element_by_id( selector ); // por defecto el container generico de todos los frms

		// todo los contenedores de controles (label + input), (label + select), (label + textarea), etc
		var $all_form_data = $conf_form_data_container.find(".form-group:not(.customEditable)");

		// readonly all textbox, chks, radio
		var $inputs = $all_form_data.find("input:not([force_ds='true'])");
		$inputs.prop("disabled", true).removeClass("readonly").addClass("readonly");

		// disabled all combos
		var $combos = $all_form_data.find("select:not([force_ds='true'])");
		$combos.prop("disabled", true).removeClass("readonly").addClass("readonly");

		// readonly all textarea
		var $textareas = $all_form_data.find("textarea:not([force_ds='true'])");
		$textareas.prop("disabled", true).removeClass("readonly").addClass("readonly");

		// remove all required class
		var $lbls = $all_form_data.find(".formLabel:not([force_ds='true'])");
		$.each( $lbls, function( _, lbl ) {
			var $lbl = $( lbl );
			var old_classes = $lbl.attr("class");
			if ( typeof old_classes === "string" ) {
				$lbl.attr( "old_classes", old_classes );
			}
			$lbl.removeClass("formLabel-rq").addClass("formLabel-ds readonly");
			$lbl.removeClass("required");
		});

		this.set_edit_mode(false);
	};

	conf_form.prototype.disable_all_crtls = function( specific_selector /* opcional */ )
	{
		var selector = "form_data_container";

		if ( specific_selector ) {
		// solo los controles de la seccion
			selector = selector + specific_selector;
		}

		var $conf_form_data_container = this.$get_dom_element_by_id( selector ); // por defecto el container generico de todos los frms

		// todo los contendedores de controles (label + input), (label + select), (label + textarea), etc
		var $all_form_data = $conf_form_data_container.find(".form-group:not(.customEditable)");

		// readonly all textbox, chks, radio
		var $inputs = $all_form_data.find("input:not([force_ds='true'])");
		$inputs.prop("disabled", true);

		// disabled all combos
		var $combos = $all_form_data.find("select:not([force_ds='true'])");
		$combos.prop("disabled", true);

		// readonly all textarea
		var $textareas = $all_form_data.find("textarea:not([force_ds='true'])");
		$textareas.prop("disabled", true);

		// remove all required class
		var $lbls = $all_form_data.find(".formLabel:not([force_ds='true'])");
		$.each( $lbls, function( _, lbl ) {
			var $lbl = $( lbl );
			var old_classes = $lbl.attr("class");
			if ( typeof old_classes === "string" ) {
				$lbl.attr( "old_classes", old_classes );
			}
			$lbl.removeClass("formLabel-rq").addClass("formLabel-ds");
			$lbl.removeClass("required");
		});
	};

	conf_form.prototype.editable_buttons = function()
	{
		// hide save and save&continue buttons
		this.$get_dom_element_by_id("form_button_save").show();

		// header required message
		this.show_or_hide_required_message(true);

		var $cancel = this.$get_dom_element_by_id("form_button_cancel");
		if ( this.action.only_frm ) {
			// show button
			$cancel.show();
		}
		var cancel_label, cancel_initial_state;
		if ( this.action.form && this.action.form.cancel_button ) {
			cancel_label = this.action.form.cancel_button.caption;
			cancel_initial_state = this.action.form.cancel_button.initial_state;
		}
		// change label of the cancel button
		var disabled = ( cancel_initial_state || "" ).toLowerCase() === "disabled";
		$cancel
			.prop( "disabled", disabled )
			.attr("value", cancel_label || "<%=_T('Cancel')%>");
	};

	conf_form.prototype.readonly_buttons = function()
	{
		// hide save and save&continue buttons
		this.$get_dom_element_by_id("form_button_save").hide();

		this.$get_dom_element_by_id("form_button_save_and_continue").hide();

		// header required message
		this.show_or_hide_required_message(false);

		var $cancel = this.$get_dom_element_by_id("form_button_cancel");
		if ( this.action.only_frm ) {
			// hide button
			$cancel.hide();
		}
		// change label of the cancel button
		$cancel.attr("value", "<%=_T('Close')%>").prop("disabled", false);
	};

	conf_form.prototype.readonly = function()
	{
		this.set_readonly(true);
		this.readonly_buttons();
		this.set_events();

		if ( this.has_sections() ) {
			this.disabled_all_sub_menus( false );
			this.show_sections();
		}
		else {
			this.readonly_all_crtls();
		}
		this.dashboard.disabled_all_menus(false);
		this.apply_initial_config();
		this.hide_waiting();
	};

	conf_form.prototype.disabled_all_sub_menus = function(disabled)
	{
		this.dashboard.disabled_all_sub_menus(disabled);
	};

	conf_form.prototype.mark_sub_menu_option = function(section_id)
	{
		this.dashboard.mark_sub_menu_option(section_id);
	};

	conf_form.prototype.__can_show_section = function( section_id )
	{
		var section = this.get_section_def(section_id);
		if (!section) {
			return false;
		}
		if (!section.can_show(section)) {
			return false;
		}
		return section; // retorna la seccion por si se quiere realizar algo con ella (viene a ser como que retorne un TRUE)
	};

	conf_form.prototype.show_sections = function(callback = $.noop)
	{
		const self = this;
		if (!this.has_sections()) {
			callback();
			return;
		}

		let selected_action = this.current_section;

		$.each( this.sections, function( section_id, section )
		{
			if ( !section.frm ) {
				return true; // continue
			}

			if ( !self.__can_show_section(section_id) ) {
				self.hide_section(section_id);
				return true; // continue
			}

			if ( !selected_action && section.selected ) {
				selected_action = section_id;
			}

			self.show_section( section_id );

			if (!self.is_creating()) {
				var entity = self.get_entity_instance();
				section.frm.set( entity );
				if ( section.frm.is_readonly() ) {
					section.frm.readonly();
				}
				else {
					section.frm.editable();
				}
			}
		});

		if ( selected_action ) {
			this.select_section( selected_action );
		}
		this.dashboard.show_back_button();
		this.onReadySections(true);
		callback();
	};

	conf_form.prototype.has_permission_to_save = function()
	{
		var no_log = true;
		var has_permission_to_save = this.dashboard.permissions.to_action(this.dashboard.current_app, this.action.permissions.save, "SAVE", undefined, no_log);
		var readonly = !has_permission_to_save;
		this.set_readonly( readonly );
		return has_permission_to_save;
	};

	conf_form.prototype.has_permission_to_delete = function()
	{
		var no_log = true;
		var has_permission_to_delete = this.dashboard.permissions.to_action(this.dashboard.current_app, this.action.permissions.erase, "DELETE", undefined, no_log);
		var readonly = !has_permission_to_delete;
		this.set_readonly( readonly );
		return has_permission_to_delete;
	};

	conf_form.prototype.fill_timezones = function(timezones, $combo, is_blank_first_item)
	{
		const self = this;
		if (typeof $combo !== "object") {
			return;
		}
		if (!$.isArray(this.sorted_timezones))
		{
			var calculate_timezone = function(timezone)
			{
				var minutes = parseInt(timezone.TimeZoneMinutes, 10);
				var timezone_minutes = parseInt(timezone.TimeZone, 10) * 60;
				if (timezone_minutes < 0) {
					minutes = -minutes;
				}
				return timezone_minutes + minutes;
			};
			this.sorted_timezones = inConcert.Utils.hash_to_sorted_array(timezones, calculate_timezone);
		}

		$combo.html("");
		is_blank_first_item = typeof(is_blank_first_item) === "boolean" ? is_blank_first_item : true;
		if (is_blank_first_item) {
			this.combo_add_value($combo, "", "", true);
		}

		var zeropad = inConcert.Utils.zeropad;
		$.each(this.sorted_timezones, function( _, timezone )
		{
			var offset = parseInt(timezone.TimeZone, 10);
			var sign = offset > 0 ? "+" : ( offset < 0 ? "-" : "" );
			var description = sign + zeropad(timezone.TimeZone) + ":" +
				zeropad(timezone.TimeZoneMinutes) + " - " + timezone.Description;

			self.combo_add_value($combo, timezone.Id, description, false, "timezone", timezone);
		});

		if (!is_blank_first_item) {
			// Si el primer item del combo, no es un item en blanco, dejo seleccionado por defecto el TIMEZONE "WET", que es el de hora 0.
			$combo.val("WET");
		}
	};

	conf_form.prototype.fill_campaigns = function($combo, is_blank_first_item, campaigns)
	{
		const self = this;
		if (typeof $combo !== "object") {
			return;
		}
		campaigns = campaigns || inConcert.app.getCampaigns();

		$combo.html("");
		is_blank_first_item = typeof(is_blank_first_item) === "boolean" ? is_blank_first_item : true;
		if (is_blank_first_item) {
			this.combo_add_value($combo, "", "", true);
		}

		campaigns.forEach(campaign => {
			self.combo_add_value($combo, campaign.Id, campaign.Name, false, "campaign", campaign);
		});
	};

	conf_form.prototype.fill_combo = function($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive, selected_value, tooltip)
	{
		id_is_case_sensitive = id_is_case_sensitive || false;
		return inConcert.Utils.fill_combo($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive, selected_value, tooltip);
	};

	conf_form.prototype.hide_combo_items = function($combo, must_hide)
	{
		must_hide = must_hide || function() {return false;};
		if (typeof $combo !== "object" || $combo.length === 0) {
			return;
		}

		var $items =$combo.find("option");
		$items.show();
		$.each($items, function(_, item) {
			var $item = $(item);
			var value = $item.attr("value");
			if (must_hide(value)) {
				$item.hide();
			}
		});
	};

	conf_form.prototype.load_data = function(callback_complete_requests, entity_id)
	{
		const self = this;
		var callback = callback_complete_requests || $.noop; //ready for save OR readonly

		if (!$.isArray(this.requests) || this.requests.length === 0) {
			callback();
			return;
		}

		var requests_length = this.requests.length;

		var cancel_running_request = function()
		{
			$.each(self.requests, function( _, request )
			{
				if (request.running) {
					request.running.abort();
					request.running = null;
				}
			});
		};

		var request_callback = function(success, response, req_info)
		{
			if (success) {
				if (typeof req_info === "object" && typeof req_info.success === "function") {
					req_info.success(response);
				}
				requests_length--;
				if (requests_length === 0) {
					callback();
				}
				req_info.running = null;
			}
			else {
				cancel_running_request();
				let err = req_info;
				self.unable_for_save(err, function() {
					if (typeof err === "object" && typeof err.error === "function") {
						err.error( response );
					}
					else {
						var message = "<%=_T('Server error. Unable to retrieve information.')%>";
						var timeout_ms = false; //infinite
						self.show_status_message(message, "Error", timeout_ms);
					}
				});
			}
		};

		this.show_waiting("<%=_T('Please wait, retrieving information')%>");
		$.each(this.requests, function( i, req_info )
		{
			if (typeof req_info.request_function === "function") {
				req_info.running = req_info.request_function(
					function(success, response) {
						request_callback(success, response, req_info);
					},
					entity_id
				);
				return;
			}
			else {
				var request = {
					url         : req_info.url,
					dataType    : req_info.dataType || "json",
					contentType : req_info.contentType || "application/json",
					success: function(response) {
						request_callback(true, response, req_info);
					},
					error: function( /*xhr, text_status, error_thrown*/ ) {
						request_callback(false, i, req_info);
					}
				};

				if (!req_info.type) {
					request.type = "GET";
				}
				else {
					request.type = req_info.type;
					if (req_info.type.toUpperCase() === "POST") {
						request.data = JSON.stringify(req_info.data);
					}
				}

				req_info.running = inConcert.invoke(request); // request!!!!
			}
		});
	};

	conf_form.prototype.build_header_custom_buttons = function(html_buttons, callback = $.noop)
	{
	/*
	 * Ejemplo de modo de uso en las clases hijas:
	 *  adm_form_skills.prototype.build_header_custom_buttons = function() {
	 *     var html_buttons = "<input type='button' value='xxxx' />";
	 *     adm_form_skills.uber.build_header_custom_buttons.apply(this, [html_buttons]); //invokes the base class method
	 *  };
	*/

		var hide_or_show_method = null;

		if ( typeof this.current_section === "string" ) {
		// si el frm tiene secciones tiene que validar contra la seccion.
			if ( this.is_readonly_current_section() ) {
				hide_or_show_method = "hide";
				html_buttons = "";
			}
		}
		else {
		// si el frm NO tiene secciones, valida derecho contra la frm.
			if ( this.is_readonly() ) {
				hide_or_show_method = "hide";
				html_buttons = "";
			}
		}

		if ( hide_or_show_method === null && html_buttons ) {
			hide_or_show_method = "show";
		}
		else {
			hide_or_show_method = "hide";
			html_buttons = "";
		}
		this.$get_dom_element_by_id( "form_custom_buttons" ).html( html_buttons )[ hide_or_show_method ]();

		callback();
	};

	conf_form.prototype.destroy_header_custom_buttons = function()
	{
		this.$get_dom_element_by_id("form_custom_buttons")
			.html("")
			.hide();
	};

	conf_form.prototype.has_sections = function()
	{
		return typeof this.sections === "object";
	};

	conf_form.prototype.is_readonly_current_section = function()
	{
		if ( !this.has_sections() ) {
			return true;
		}
		var section = this.sections[ this.current_section ];
		if ( !section || !section.frm ) {
			return true;
		}
		return section.frm.is_readonly();
	};

	conf_form.prototype.count_enabled_sections = function( entity_id )
	{
		var count = 0;
		if ( !this.has_sections() ) {
			return count;
		}

		$.each( this.sections, function( _, section )
		{
			if ( !section.can_show( section, entity_id ) ) {
			// si no tiene permisos sobre la seccion, sigo buscando
				return true; // continue
			}
			count++;
		});

		return count;
	};

	conf_form.prototype.change_title_section = function( section_id, sub_title){
		section_id = section_id || this.current_section;
		if (typeof section_id !== "string" || !this.has_sections()) {
			return;
		}

		const action_obj = this.dashboard.get_current_action_obj();
		const change_nav_sections = action_obj && action_obj.view && action_obj.view.change_nav_sections;
		if (change_nav_sections) {
			/*
			TODO Esta validacion 'change_nav_sections' es para cambios nuevos, definidos por UX/UI en las secciones.
			Por el momento para navegaciones seleccionadas.
			Eliminar cuando se haya agregado en todos los FORM que usen secciones.

			*/
			const section = this.get_section_def(section_id);
			if (section) {
				let form_title = section.name || "";
				if (sub_title) {
					form_title = `${form_title} <span>${sub_title}</span>`;
				}
				this.$get_dom_element_by_id("form_title").html(form_title);
				if (section.i_class_name) {
					const $item_img = this.$get_dom_element_by_id("form_item_img");
					$item_img.removeClass().addClass(section.i_class_name || "");
				}
			}
		}
	};

	conf_form.prototype.show_section = function( section_id )
	{
		if (typeof section_id !== "string") {
			return;
		}
		this.dashboard.show_sub_menu_option( section_id );
		this.$get_dom_element_by_id("form_data_container > [form_section]").hide();
		this.$get_dom_element_by_id("form_data_container > [form_section='" + section_id + "']").show();

		inConcert.Utils.inline_manual_update();
		this.set_edit_mode(false);
	};

	conf_form.prototype.hide_section = function( section_id )
	{
		if (typeof section_id !== "string") {
			return;
		}
		this.dashboard.hide_sub_menu_option( section_id );
		this.$get_dom_element_by_id("form_data_container > [form_section='" + section_id + "']").hide();
	};

	conf_form.prototype.select_section = function( section_id )
	{
		if (typeof section_id !== "string") {
			return;
		}
		var section;
		if ( typeof this.current_section === "string" ) {
			section = this.sections[this.current_section];
			if (section.frm) {
				section.frm.on_hide();
			}
		}
		this.show_section( section_id );
		this.dashboard.mark_sub_menu_option( section_id );
		section = this.sections[ section_id ];
		if ( section.frm ) {
			this.current_section = section_id;
			this.hide_status_message();
			section.frm.before_show(function() {
				section.frm.on_show();
				section.frm.ready();
			});
		}
	};

	conf_form.prototype.create_sections = function(frm)
	{
		const self = this;
		frm = frm || this;
		if (!this.has_sections()) {
			return;
		}
		$.each(this.sections, function(section_id, section)
		{
			if (section.frm) {
				return true; // continue
			}
			var entity_id;
			var entity = self.get_entity_instance();
			if (entity !== null && entity !== undefined) {
				entity_id = entity.Id;
			}
			var instance = section.instance( section_id, entity_id );
			if (instance) {
				section.frm = new instance({frm: frm, section_id: section_id});
			}
		});
	};

	conf_form.prototype.set_events = function()
	{
		if (this.has_sections()) {
		// set section events
			$.each(this.sections, function( _, section )
			{
				if (!section.frm) {
					return true; // continue
				}
				section.frm.set_events();
			});
		}
		// agrego los eventos a las relaciones de los controles pares chk + txt, etc
		const $container = this.$get_dom_element_by_id("form_data_container");
		this.set_related_events( $container );
	};

	conf_form.prototype.set_event_scroll = function(on_scroll = $.noop)
	{
		this.$get_dom_element_by_id("form_ctrls_container")
			.off("scroll")
			.on("scroll", function(e) {
				e.stopPropagation();
				//Para ocultar el datepicker si se scrollea
				$(document).mousedown();
				on_scroll();
			});
	};

	conf_form.prototype.get_section_def = function( section_id )
	{
		if ( !this.has_sections() ) {
			return;
		}
		return this.sections[ section_id ];
	};
	
	conf_form.prototype.can_save_entity = function(entity) 
	{
		return true;
	};

	conf_form.prototype.ready = function()
	{
		const self = this;

		var allowed_save = this.has_permission_to_save(); // check permission to save
		this.set_entity_instance( null );
//		this.set_events();
		this.set_event_scroll();

		if (this.has_sections() && this.dashboard.allow_disable_all_navigation()) {
			this.disabled_all_sub_menus(true);
		}

		this.initialize();

		var edit = function() {
			var row_id;
			var name = "";
			if (self.$current_row) {
				row_id = self.$current_row.attr("row_id");
				name = self.$current_row.attr("row_name");
			}

			self.load_data(function()
				{
					var waiting_message = FORMAT("<%=_T('Please wait, retrieving {0}')%>", name);
					self.show_waiting(waiting_message);

					var callback = function(success, response)
					{
						if ( success ) {
	// TODO: #4636 - Este set no tiene que ir aca!!
	//   Tiene que ir donde se encuentra comentado, despues del if y el else.
	//   Esto va de la mano con el comentario de "adm_form_channels.prototype.set"
							self.set_entity_instance( response );
							self.set(true, response);
							if (allowed_save && self.can_save_entity(response)) {
								self.ready_for_save();
							}
							else {
								self.readonly();
							}
	//						self.set(true, response);
						}
						else {
							let err = response;
							self.unable_for_save(err, function() {
								var message = "<%=_T('Server error. Unable to retrieve information.')%>";
								if (typeof err === "object" && typeof err.result === "string") {
									if ((err.result.toUpperCase() === "FORBIDDEN")) {
										message = "<%=_T('Unable to retrieve information. You dont have permissions for this action.')%>";
									}
								}
								var timeout_ms = false; //infinite
								self.show_status_message(message, "Error", timeout_ms);
								self.set(false);
							});
						}
					};

					var action_obj = self.dashboard.get_current_action_obj();
					var view_obj = action_obj.view;
					if (action_obj.api && action_obj.api.get) {
						action_obj.api.get(callback);
					}
					else {
						const uri = self.uri_get_entity();
						if (uri === undefined) {
							let row_data = self.$current_row.data("row_data");
							callback(true, row_data);
							return;
						}
						const api = self.dashboard.get_api();
						api.get_entity({
							uri     : uri,
							callback: callback,
							request : view_obj.request
						});
					}
				},
				row_id
			);
		};
		var view = edit;

		this.$get_dom_element_by_id("form_button_cancel")
			.attr("value", "<%=_T('Cancel')%>")
			.off("click")
			.on("click", function() {
				self.close("cancel");
			});

		if (allowed_save) {
			// if has permission to save
			if (this.is_new) {
				this.load_data(function() {
					self.ready_for_save();
				});
			}
			else {
				edit();
			}
		}
		else {
			// if not has permission to save (readonly)
			view();
		}
		this.dashboard.initial_full_screen();
		this.set_title_img();
		this.set_title();
		this.set_sub_title();
		this.set_tagline(this.tagline);
		this.build_header_custom_buttons();
		this.build_footer_custom_buttons();
		this.onReadySections(true);
		this.dashboard.on_form_ready();
	};

	conf_form.prototype.onReadySections = function(showing)
	{
		if (!this.has_sections()) {
			return;
		}
		const action_obj = this.dashboard.get_current_action_obj();
		const change_nav_sections = action_obj && action_obj.view && action_obj.view.change_nav_sections;
		if (!change_nav_sections) {
			return;
		}
		/*
			TODO Esta validacion 'change_nav_sections' es para cambios nuevos, definidos por UX/UI en las secciones.
			Por el momento para navegaciones seleccionadas.
			Eliminar cuando se haya agregado en todos los FORMs que usen secciones.
		*/
		const $app_menu_container = this.$get_dom_element_by_id("applications_menu_container");
		const $section_title = this.$get_dom_element_by_id("form_section_title");
		const $menu_items = this.$get_dom_element_by_id("application_actions");
		const $current_action = this.dashboard.$get_menu_action();
		$current_action.removeClass("optionSubmenuActive");
		$menu_items.removeClass("applicationOptionsSubmenuActive");
		if (showing) {
			this.$get_dom_element_by_id("form_tagline").hide();
			$section_title.hide();
			$app_menu_container.hide();
			$menu_items.addClass("applicationOptionsSubmenuActive");
			$current_action.addClass("optionSubmenuActive");
			return;
		}
		$section_title.show();
		$app_menu_container.show();
		this.change_title_section();
	};

	conf_form.prototype.set_title_img = function()
	{
		const $item_img = this.$get_dom_element_by_id("form_item_img");
		if ($item_img.length === 0) {
			return;
		}
		this.dashboard.set_title_img($item_img);
	};

	conf_form.prototype.set_title = function(custom_title)
	{
		let title = custom_title;
		this.$get_dom_element_by_id("form_title").removeClass("hasEntityTitle");
		if (!title) {
			const allowed_save = this.has_permission_to_save(); // check permission to save;
			if (allowed_save && this.is_new) {
				title = this.new_title;
			}
			else {
				const entity_name = this.get_entity_display_name();
				const template_title = allowed_save ? this.update_title : this.view_title;
				title = FORMAT(template_title || "{0}", ("<span>" + entity_name + "</span>"));
				this.$get_dom_element_by_id("form_title").addClass("hasEntityTitle");
			}
		}
		this.$get_dom_element_by_id("form_title").html(title);
	};

	conf_form.prototype.get_title = function()
	{
		return this.$get_dom_element_by_id("form_title").text();
	};

	conf_form.prototype.set_sub_title = function(sub_title)
	{
		sub_title = sub_title || "";
		this.$get_dom_element_by_id("form_section_title").text(sub_title);
	};

	conf_form.prototype.get_sub_title = function()
	{
		return this.$get_dom_element_by_id("form_section_title").text();
	};

	conf_form.prototype.set_tagline = function(tagline)
	{
		tagline = ( tagline || this.tagline ) || "";
		var $tagline = this.$get_dom_element_by_id("form_tagline");
		if ( tagline.length === 0 ) {
			$tagline.hide();
		}
		else {
			$tagline.show();
		}
		$tagline.html( tagline );
	};

	conf_form.prototype.get_html_content = function(render_callback)
	{
		render_callback = render_callback || $.noop;
		var api = this.dashboard.get_api();
		api.load_content_page({
			URI: this.html_file_name,
			callback: render_callback
		});
	};

	conf_form.prototype.render = function($current_row, instance_config)
	{
		const self = this;
		this.instance_config = instance_config;
		this.dashboard.view.on_hide();
		this.hide_messages();
		if (this.action.only_frm) {
			this.is_new = false;
		}
		else {
			this.is_new = !$current_row;
		}
		this.$current_row = $current_row;
		if (typeof this.action.form.render === "function") {
			this.action.form.render();
		}
		else {
			const render = function(success, content)
			{
				if (success && typeof content === "string") {
					const $view = self.$get_dom_element_by_id("view_container");
					if (instance_config && instance_config.split_screen) {
						$view.show();
					}
					else {
						$view.hide();
					}
					self.$get_dom_element_by_id("form_data_container").html(content);
					self.$get_dom_element_by_id("form_container").show();
					self.ready();
					self.cached_content = content;
				}
			};
			const content = this.cached_content;
			if (content) {
				render(true, content);
			}
			else {
				this.get_html_content(render);
			}
		}
		this.AnalyticsSend();                   // google analytics
		inConcert.Utils.inline_manual_update(); // inline manual
	};

	conf_form.prototype.AnalyticsSend = function(info = {})
	{
		// basic info
		const analyticsInfo = {
			//app_id       : este valor sale de confdashboard,
			entity_id    : info.entity_id     || null,
			entity_type  : info.entity_type   || "form",
			page_title   : info.page_title    || this.get_title(), // TODO ver si podemos sacar el titulo
			sub_action_id: info.sub_action_id || null
		};
		// new o update
		if (this.is_new) {
			analyticsInfo.action_id = info.action_id || "new";
		}
		else {
			if (info.action_id) {
				analyticsInfo.action_id = info.action_id;
			}
			else {
				if (this.instance_config
					&& typeof(this.instance_config.show_view_readonly) === "boolean"
				) {
					analyticsInfo.action_id = this.instance_config.show_view_readonly ? "edit_readonly" : "edit";
				}
				else {
					analyticsInfo.action_id = info.action_id || (this.has_permission_to_save() ? "edit" : "edit_readonly");
				}
			}
			analyticsInfo.item_id = info.entity_id || this.get_entity_id();
		}
		// el path viene se envia en un array
		analyticsInfo.source_path = [
			{name: "action_id", value: analyticsInfo.action_id}, // "/main/administration/campaigns/edit"
			{name: "item_id"  , value: analyticsInfo.item_id  }  // "/main/administration/campaigns/edit/camp1@vcc1"
		].concat(info.source_path);
		// custom information
		info.customInfo = {...{}, ...info.customInfo || {}};
		// invoco a la funcion de dashboard
		this.dashboard.AnalyticsSend(analyticsInfo);
	};

	conf_form.prototype.add_css_class = function(class_name)
	{
		var $form_container = this.$get_dom_element_by_id("form_container");
		if ( !$form_container ) {
			return;
		}
		$form_container.removeClass( class_name ).addClass( class_name );
	};

	conf_form.prototype.remove_css_class = function( class_name /* optional */ )
	{
		var $form_container = this.$get_dom_element_by_id("form_container");
		if ( !$form_container ) {
			return;
		}
		if ( class_name ) {
			$form_container.removeClass( class_name );
		}
		else {
			// en el caso de que el parametro "class_name" venga undefined, tiene que remover todas las clases y dejar la clase por defecto -> "pageContainer"
			$form_container.removeClass();
			this.add_css_class("pageContainer");
		}
	};

	conf_form.prototype.is_loading_sections = function()
	{
		if ( !this.has_sections() ) {
			return false;
		}
		var is_loading_section = false;
		$.each(this.sections, function( _, section )
		{
			if (!section.frm) {
				return true; // continue
			}
			if (section.frm.loading) {
				is_loading_section = true;
				return false; // break
			}
		});
		return is_loading_section;
	};

	conf_form.prototype.make_autocomplete = function(
		$autocomplete,
		source_list,
		change_event = $.noop,
		control_type = "SELECT",
		maxItems = null,
		clearSearch = false
	) {
		const self = this;

		$autocomplete.data("source_list", source_list);
		//$autocomplete.val("");
		$autocomplete.attr("current_value", "");
		this.autocomplete_change_control_type($autocomplete, control_type);
		let value = "";

		const on_change = function (_, item) {
			let current_value, current_label;
			if (item && item.item) {
				current_value = item.item.value;
				current_label = item.item.label;
			}
			else {
				const current_text = $autocomplete.val();
				const found_item = self.autocomplete_find_item_by_label($autocomplete, current_text);
				if (found_item) {
					current_label = found_item.label;
					current_value = found_item.value;
				}
				else {
				 	current_value = current_text;
				}
			}
			const $this = $(this);
			if (current_label) {
				$this.val(current_label);
			}
			else {
				$this.val(current_value);
			}
			window.setTimeout(function() {
				if (current_label) {
					$autocomplete.val(current_label);
				}
				else {
					$autocomplete.val(current_value);
				}
				$autocomplete.attr("current_value", current_value);
				change_event($autocomplete, current_value, current_label);
				self.autocomplete_custom_change_callback();
			});
			value = current_value;
		};

		const autocomplete_id = $autocomplete.prop("id");
		const is_created = this.__is_created_autocomplete($autocomplete);
		if (is_created) {
			this.__destroy_autocomplete($autocomplete);
			delete this.$autocomplete_list[autocomplete_id];
		}
		$autocomplete.css("cursor", "pointer");

		const sourceByMaxItems = function (request, response) {
			let matchingItems = [];
			const matcher = new RegExp("^" + $.ui.autocomplete.escapeRegex(request.term), "i");
			for (let i = 0; (i < source_list.length) && (matchingItems.length < maxItems); i++) {
				const source_item = source_list[i];
				if (matcher.test(typeof source_item === "object" ? source_item.label : source_item)) {
					matchingItems.push(source_item);
				}
			}
			response(matchingItems);
		};

		if (typeof maxItems === "number") {
			$autocomplete.attr("title", FORMAT("<%=_T('Exists a total of {0} elements.')%>", source_list.length));
		}

		$autocomplete.autocomplete({
			source: (typeof maxItems === "number") ? sourceByMaxItems : source_list,
			minLength: 0,
			select: on_change,
			change: on_change
		});
		$autocomplete.off("change").on("change", on_change);
		$autocomplete.off("click").on("click", function () {
			if ($autocomplete.autocomplete("widget").is(":visible")) {
				$autocomplete.autocomplete("search", "zzzzzzz");
			}
			else {
				$autocomplete.autocomplete("search", clearSearch ? "" : $autocomplete.val());
			}
		});
		this.$autocomplete_list[autocomplete_id] = $autocomplete;
		$autocomplete.attr("autocomplete", "do-not-autofill");
	};

	conf_form.prototype.autocomplete_change_control_type = function($autocomplete, control_type = "SELECT")
	{
		$autocomplete.attr("control_type", control_type.toUpperCase());
	};

	conf_form.prototype.autocomplete_find_item_by_label = function($autocomplete, find_label)
	{
		if (!find_label) {
			return;
		}
		find_label = find_label.toLowerCase();
		let found_item;
		const source_list = $autocomplete.data("source_list");

		for (let i = 0; i < source_list.length; i++) {
			let item = source_list[i];
			if (typeof(item) === "object") {
				if (item.label.toLowerCase() === find_label) {
					found_item = item;
					break;
				}
			}
			else if (item.toLowerCase() === find_label) {
				found_item = {
					label: item,
					value: item
				};
				break;
			}
		}

		return found_item;
	};

	/*
		Funcion de callback que debe implementar quien instancie el "autocomplete" y tenga otros "autocomplete" relacionados.
		Caso en outboundengie->process->management->filter: campos field y value.
	*/
	conf_form.prototype.autocomplete_custom_change_callback = $.noop;

	conf_form.prototype.autocomplete_get_value = function($autocomplete)
	{
		let item = {};
		item.label = $autocomplete.val();
		item.value = $autocomplete.attr("current_value");
		if (!item.value) {
			item.value = item.label;
		}
		return item;
	};

	conf_form.prototype.autocomplete_set_value = function($autocomplete, current_value)
	{
		if (!current_value) {
			$autocomplete.val("");
			$autocomplete.attr("current_value", "");
			$autocomplete.trigger("change");
			return;
		}

		let value, label;
		const source_list = $autocomplete.data("source_list");

		for (let i = 0; i < source_list.length; i++) {
			let item = source_list[i];

			if (typeof(item) === "object") {
				if (item.value.toLowerCase() === current_value.toLowerCase()) {
					value = item.value;
					label = item.label;
					break;
				}
			}
			else {
				value = current_value;
				label = current_value;
				break;
			}
		}

		if (!value) {
			value = current_value;
			label = current_value;
		}

		$autocomplete.val(label);
		$autocomplete.attr("current_value", value);
		$autocomplete.trigger("change");
	};

	conf_form.prototype.__is_created_autocomplete = function($autocomplete)
	{
		if (!$autocomplete || $autocomplete.length === 0) {
			return false;
		}
		return $autocomplete.hasClass("ui-autocomplete-input");
	};

	conf_form.prototype.__close_all_autocomplete = function()
	{
		const self = this;
		$.each(this.$autocomplete_list, function(_, $autocomplete) {
			var is_created = self.__is_created_autocomplete($autocomplete);
			if (is_created) {
				$autocomplete.autocomplete("close");
			}
		});
	};

	conf_form.prototype.__destroy_all_autocomplete = function()
	{
		const self = this;
		$.each(this.$autocomplete_list, function(_, $autocomplete) {
			self.__destroy_autocomplete($autocomplete);
		});
		this.$autocomplete_list = {};
	};

	conf_form.prototype.__destroy_autocomplete = function($autocomplete)
	{
		var is_created = this.__is_created_autocomplete($autocomplete);
		if (!is_created) {
			return;
		}
		$autocomplete.autocomplete("close");
		$autocomplete.autocomplete("destroy");
		$autocomplete.off();
	};

	conf_form.prototype.show_header_system_actions = function()
	{
		const $actions = this.$get_dom_element_by_id("form_buttons_actions");
		$actions.show();
	};

	conf_form.prototype.hide_header_system_actions = function()
	{
		const $actions = this.$get_dom_element_by_id("form_buttons_actions");
		$actions.hide();
	};

	conf_form.prototype.show_header_custom_actions = function()
	{
		const $actions = this.$get_dom_element_by_id("form_custom_buttons");
		$actions.show();
	};

	conf_form.prototype.hide_header_custom_actions = function()
	{
		const $actions = this.$get_dom_element_by_id("form_custom_buttons");
		$actions.hide();
	};

	conf_form.prototype.show_custom_actions = function()
	{
		const $custom_actions = this.$get_dom_element_by_id("form_buttons_custom_actions");
		$custom_actions.show();
	};

	conf_form.prototype.hide_custom_actions = function()
	{
		const $actions = this.$get_dom_element_by_id("form_buttons_custom_actions");
		$actions.hide();
	};

	conf_form.prototype.reopen_view = function(config)
	{
		this.dashboard.reopen_view(config);
	};

	conf_form.prototype.build_footer_custom_buttons = function( html_buttons )
	{
	/*
	 * Ejemplo de modo de uso en las clases hijas:
	 *  adm_form_qna_maker.prototype.build_footer_custom_buttons = function() {
	 *     var html_buttons = "<input type='button' value='xxxx' />";
	 *     adm_form_skills.uber.build_footer_custom_buttons.apply(this, [html_buttons]); //invokes the base class method
	 *  };
	*/

		let hide_or_show_method = null;

		if (typeof this.current_section === "string") {
		// si el frm tiene secciones tiene que validar contra la seccion.
			if (this.is_readonly_current_section()) {
				hide_or_show_method = "hide";
				html_buttons = "";
			}
		}
		else {
		// si el frm NO tiene secciones, valida derecho contra la frm.
			if (this.is_readonly()) {
				hide_or_show_method = "hide";
				html_buttons = "";
			}
		}

		if (hide_or_show_method === null && html_buttons) {
			hide_or_show_method = "show";
		}
		else {
			hide_or_show_method = "hide";
			html_buttons = "";
		}
		this.$get_dom_element_by_id("form_buttons_custom_actions").html(html_buttons)[hide_or_show_method]();
	};

	conf_form.prototype.sanitize_value = function(value)
	{
		return this.dashboard.sanitize_value(value);
	};

	conf_form.prototype.escape = function(value)
	{
		return this.dashboard.escape(value);
	};

	conf_form.prototype.convert_chk_bool_to_string = function(chk_id)
	{
		var sanitized_id = (chk_id || "").replace("#", "");
		const $chk = $("#" + sanitized_id);
		if ($chk.length === 0) {
			return undefined;
		}
		const checked = $chk.prop("checked");
		return Number(checked).toString();
	};

	conf_form.prototype.show_popup_message = function(info)
	{
		if (!info) {
			return;
		}
		if (!info.messages) {
			return;
		}
		// accion a efectuar luego de cerrar el popup.
		let close_action = info.close_action || "NONE";
		// el titulo
		const $title = this.$get_dom_element_by_id("form_popup_title");
		$title.text(info.messages.title);
		// el mensaje
		const $message = this.$get_dom_element_by_id("form_popup_message");
		$message.text(info.messages.message);
		// el detalle
		if (info.messages.detail) {
			// el mensaje del detalle
			const $detail = this.$get_dom_element_by_id("form_popup_detail");
			$detail.text(info.messages.detail);
			// el container del detalle
			const $detail_container = this.$get_dom_element_by_id("form_popup_detail_container");
			$detail_container.show();
		}
		// el container del popup
		const $container = this.$get_dom_element_by_id("form_popup_container");
		// definicion de clases (css) por tipo de mensaje
		const css_classes_by_message_type = {
			"error": ["errorPopup"],
			"info" : ["infoPopup"],
			"warn" : ["warningPopup"]
		};
		// primero remuevo del contenedor todas las clases (css) de los diferentes tipos de mensajes
		for (let type in css_classes_by_message_type) {
			if (css_classes_by_message_type.hasOwnProperty(type)) {
				let css_classes = css_classes_by_message_type[type];
				for (let i = 0; i < css_classes.length; i++) {
					let class_name = css_classes[i];
					$container.removeClass(class_name);
				}
			}
		}
		// segundo agrego al contenedor todas las clases del tipo de mensaje a visualizar
		let message_type = (info.messages.type || "").toLowerCase();
		let css_classes = css_classes_by_message_type[message_type];
		for (let i = 0; i < (css_classes || []).length; i++) {
			let class_name = css_classes[i];
			$container.addClass(class_name);
		}
		// oculto el waiting
		this.hide_waiting();
		// visualizo el popup
		$container.show();
		// hide popup message
		this.set_popup_message_events(close_action);
	};

	conf_form.prototype.hide_popup_message = function(close_action)
	{
		const self = this;
		// el container del detalle
		const $detail_container = this.$get_dom_element_by_id("form_popup_detail_container");
		$detail_container.hide();
		// el container del popup
		const $container = this.$get_dom_element_by_id("form_popup_container");
		$container.hide();
		if (close_action) {
			const allowed_actions = {
				"CLOSE FORM AND NOT REFRESH VIEW": function() {
					self.close();
				},
				"CLOSE FORM AND REFRESH VIEW": function() {
					self.close();
					self.grid_refresh();
				},
				"NONE": $.noop
			};
			if (!allowed_actions[close_action]) {
				throw("conf_form.prototype.hide_popup_message - Unrecognized close_action " + close_action);
			}
			allowed_actions[close_action]();
		}
	};

	conf_form.prototype.set_popup_message_events = function(close_action)
	{
		const self = this;
		// hide popup message
		const $popup_close = this.$get_dom_element_by_id("form_popup_close");
		$popup_close
			.off("click")
			.on("click", function() {
				self.hide_popup_message(close_action);
			}
		);
	};

	conf_form.prototype.confirm = function(message, callback = $.noop)
	{
		this.dashboard.confirm(message, callback);
	};

	conf_form.prototype.get_current_section_id = function()
	{
		return this.current_section;
	};

	conf_form.prototype.toggle_contextual_menu = function($parent, menu_options, can_show_contextual_menu, callback) {
		this.dashboard.view.toggle_contextual_menu($parent, menu_options, can_show_contextual_menu, callback);
	};

	conf_form.prototype.set_edit_mode = function (isEditMode) {
		this.dashboard.view.set_edit_mode(isEditMode);
		this.change_title_section();
	};

	conf_form.prototype.can_jump_to_other_menunav = function () {
		if (this.dashboard.view.isEditMode()) {
			return false;
		}
		return true;
	};

	conf_form.prototype.before_jump_to_other_menunav = function () {
		this.close("cancel");
	};

	conf_form.prototype.fill_combo_app_languages = function($app_language, is_blank_first_item = true)
	{
		const app_languages = [
			{ id: "en", name: "<%=_T('English')%>" },
			{ id: "es", name: "<%=_T('Spanish')%>" },
			{ id: "fr", name: "<%=_T('French')%>" },
			{ id: "pt", name: "<%=_T('Portuguese')%>" }
		];
		this.fill_combo($app_language, is_blank_first_item, app_languages, "id", "name");
	};

	conf_form.prototype.fill_countries = function( countries, $combo )
	{
		const self = this;
		/*
		*/
		const build_sorted_countries = ( countries_map ) => {
			if ($.isArray(self.sorted_countries)) {
				return self.sorted_countries;
			}
			self.sorted_countries = [];
			$.each(countries_map, function(country_id, country) {
				self.sorted_countries.push( { code: country_id, name: _T(country.name) });
			});
			return inConcert.Utils.sortArray(self.sorted_countries, "name");
		};

		$combo.html("");
		this.combo_add_value($combo, "", "", true);
		const sorted_countries = build_sorted_countries( countries );

		$.each( sorted_countries, function( _, country ) {
			self.combo_add_value($combo, country.code, country.name, false);
		});
	};

	conf_form.prototype.fill_combo_areas = function( country_code, $area, is_blank_first_item )
	{
		if (!this.telephone_codes)
			this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();

		const areas = this.telephone_codes.getAreasByCode( country_code );
		if ( !areas ) {
			$area.html("");
			return;
		}
		let blank_first_item_fn = false;
		if (is_blank_first_item) {
			/*
				En la lista de "areas" hay que manejar diferente al primer elemento "blank" del combo.
				Esto porque en TODOS los combos que utilizamos el elemento "blank" tiene como "value" del <option> el valor "" (vacio).
				Esto funciona bien en todos lados, menos con los "code areas", ya que aca existe un elemento con Id "" que es el "Fixed Lines".
				Esto hace que el combo tenga 2 items con el mismo Id, y aqui el error.
				Para solucionar esto, al elemento "blank" de este combo le ingresamos una palabra clave "__BLANK" que se asignara en el atributo "value" del <option>
			*/
			blank_first_item_fn = function() {
				return {
					id: "__BLANK",
					name: ""
				};
			};
		}
		// cargamos el combo
		this.fill_combo( $area, blank_first_item_fn, areas, "code", "name" );
	};

	conf_form.prototype.__get_selected_area = function($ctrl)
	{
		const area = this.combo_get_selected_item_data($ctrl);
		if (area) return area.code.toString();
		return null;
	};

	conf_form.prototype.get_current_app_id = function()
	{
		return this.dashboard.get_current_app_id();
	};

	conf_form.prototype.get_current_view_id = function()
	{
		return this.dashboard.get_current_view_id();
	};

}());
