/*jshint quotmark:double, esversion: 11*/
/*global $, inConcert */
/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	/*jslint validthis:true */
	"use strict";

	//---------------------------------------------------------------------
	// form basic class
	function conf_form_blacklists (params = {})
	{
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.frm = params.frm;
		this.prefix_id = params.prefix_id;
		// container
		this.container_id = "#" + params.container_id;
		// other info
		this.m_title = params.title || "<%=_T('Blacklists')%>";
		this.m_help = params.help || "<%=_T('You can select the blacklists of phone numbers that you will not be able to call.')%>";
		this.m_assignedBlacklists = null;
		this.m_addressType = params.addressType || "PHONE";
		this.m_grid = null;
		//ok
		return true;
	}

	if (typeof inConcert.conf_form_blacklists !== "function") {
		inConcert.conf_form_blacklists = conf_form_blacklists;
	}

	conf_form_blacklists.prototype.initialize = function()
	{
		this.m_assignedBlacklists = null;
		this.make_html();
		this.init_grid();
		this.set_events();
		this.load_data();
	};

	conf_form_blacklists.prototype.get = function()
	{
	 	const selectedItems = this.m_grid.get_selected_items();
	 	const blacklists = {};
		const enabled = selectedItems.length > 0;
		if (enabled) {
			selectedItems.forEach(item => {
				let scopeId = `${item.VCC}:${item.ID}`.toLowerCase();
				let isGlobal = item.VCC.toLowerCase() === "system";
				blacklists[scopeId] = {isGlobal: isGlobal};
			});
		}
		return {
			enabled: enabled,
			scopes : blacklists
		};
	};

	conf_form_blacklists.prototype.set = function(item)
	{
		this.m_assignedBlacklists = null;
		if (!item || !item.outgoingConfig) {
			return;
		}
		this.m_assignedBlacklists = item.outgoingConfig.Blacklists || null;
	};

	conf_form_blacklists.prototype.attach_data = function( item )
	{
		if (!item) {
			return;
		}
		item.outgoingConfig.Blacklists = this.get(true /* keep_old_data */);
		return item;
	};

	conf_form_blacklists.prototype.validate = function()
	{
		return {success: true};
	};

	conf_form_blacklists.prototype.editable = function()
	{
		const self = this;
		const ds_ctrl = function( $ctrl, ds, required ) {
			self.frm.disable_control( $ctrl, ds, required );
		};
		const $hsm_mng = $( this.hsm_mng );
		ds_ctrl( $hsm_mng, false );
	};

	conf_form_blacklists.prototype.readonly = function()
	{
		const self = this;
		const ds_ctrl = function( $ctrl, ds, required ) {
			self.frm.readonly_control( $ctrl, ds, required );
		};
		ds_ctrl( $( this.hsm_mng ), true /* disabled */ );
	};

	conf_form_blacklists.prototype.set_events = function()
	{
		if ( this.frm.is_readonly() ) {
			return;
		}
	 	this.m_grid.set_events();
	};

	conf_form_blacklists.prototype.load_data = function(callback = $.noop)
	{
		const self = this;
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.blackListGetAll({AddressType: this.m_addressType},
			function(success, list) {
				if (!success) {
					self.frm.show_status_message("<%=_T('Unable to load backlists.')%>", "Error", 5000);
					callback();
					return;
				}
				self.m_grid.initialize($.noop, list);
				callback();
			}
		);
	};

	conf_form_blacklists.prototype.get_scope_id = function(scopeId)
	{
		scopeId = scopeId.toLowerCase();
		const index = scopeId.indexOf(":");
		if (index < 1) {
			return scopeId;
		}
		return scopeId.substring(index + 1);
	};

	conf_form_blacklists.prototype.init_grid = function()
	{
		const self = this;

		const get_grid_value = function(item, field_id) {
			let value = {
				original_value: "",
				value         : "",
				search_value  : "",
				class_name    : ""
			};
			let item_value = item[field_id];
			if (item_value === undefined) {
				item_value = "";
			}
			value.original_value = item_value;
			value.value = item_value;
			value.search_value = item_value;
			if (field_id === "VCC") {
				value.value = item_value.toUpperCase() === "SYSTEM" ? "<%=_T('yes')%>" : "<%=_T('no')%>";
				value.search_value = value.value;
			}
			return value;
		};

		this.m_grid = new inConcert.conf_form_paginator({
			paginator_name: "BLACKLIST",
			owner: this,
			frm: this.frm,
			row_id: "ID",
			display_name: "ID",
			rows_per_page: 2500,
			show_at_least: 2499,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			order_info: {
				options: {
					order: "asc"
				},
				sort_value_fn: function(row) {
					return row.ID;
				}
			},
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading blacklists ...')%>",
			ids: {
				grid: this.prefix_id + "_list",
				paginator: this.prefix_id + "_paginator",
				not_results_found: this.prefix_id + "_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No blacklists found.')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "_select"    , name: "", type: "row_selector",
					is_checked: function(item) {
						let currentItemId = self.get_scope_id(item.ID);
						let scopes = self.m_assignedBlacklists ? self.m_assignedBlacklists.scopes : {};
						for (let scopeId in scopes) {
							if (scopes.hasOwnProperty(scopeId)) {
								let assignedId = self.get_scope_id(scopeId);
								if (assignedId === currentItemId) {
									return true;
								}
							}
						}
						return false;
					}
				},
				{id: "ID"         , name: "<%=_T('Name')%>"        },
				{id: "Description", name: "<%=_T('Description')%>" },
				{id: "VCC"        , name: "<%=_T('From System')%>",
					get_value: function(item) {
						return get_grid_value(item, "VCC");
					}
				}
			],
			callbacks: {}
		});
	};

	conf_form_blacklists.prototype.make_html = function()
	{
		$( this.container_id ).html(`
			<div class="card">
				<div class="card-block">
					<div class="row">
						<div class="col-12">
							<div class="card-header">
								<h5 class="mb-0">${this.m_title}</h5>
								<p>${this.m_help}</p>
							</div>
						</div>
					</div>
					<div class="row">
						<div class="col-12">
							<div id="${this.prefix_id}_list_container" class="checkbox-table-container">
								<div id="${this.prefix_id}_not_results_found" class="inlineMessagesContainer messageInfo mt-0" style="display:none"></div>
								<div class="rolesActionsListTableContainer">
									<table id="${this.prefix_id}_list" cellpadding="0" cellspacing="0" border="0" class="viewGrid mt-0">
										<thead class="viewGridHeader"></thead>
										<tbody class="viewGridBody"></tbody>
									</table>
								</div>
							</div>
							<div id="${this.prefix_id}_paginator"></div>
						</div>
					</div>
				</div>
			</div>
		`);
	};
}());
