/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	/*jshint validthis:true*/
	"use strict";

	const FORMAT = inConcert.Utils.format;
	//---------------------------------------------------------------------
	//
	function conf_form_paginator(params)
	{
		if (typeof(params) !== "object") {
			throw("conf_form_paginator. params is undefined.");
		}
		if (typeof(params.frm) !== "object") {
			throw("conf_form_paginator. params.frm is undefined.");
		}
		if (typeof(params.owner) !== "object") {
			throw("conf_form_paginator. params.owner is undefined.");
		}

		const self = this;

		this.frm   = params.frm;
		this.owner = params.owner;

		this.canEdit = function(row) {
			if (!self.columns) {
				return false;
			}
			let can_edit = false;
			for (let _ in self.columns) {
				if (self.columns.hasOwnProperty(_)) {
					let column = self.columns[_];
					if (column.system_action && column.system_action.edit) {
						can_edit = true;
						break;
					}
				}
			}
			if (!can_edit) {
				return false;
			}
			if (typeof(params.canEdit) !== "function") {
				return true;
			}
			return params.canEdit(row);
		};

		this.has_counter_column = false;
		this.current_popup_number = null;

		// usar instancia de inconcert popup window
		this.use_instance_of_inconcert_popup = params.use_instance_of_inconcert_popup;

		// paginated
		this.paginator         = null;
		this.paginator_name    = params.paginator_name;
		this.paginator_type    = params.paginator_type || "ALL_ROWS_SERVER_SIDE"; // ALL_ROWS_SERVER_SIDE o ALL_ROWS_CLIENT_SIDE
		this.row_id            = params.row_id || false;
		this.display_name      = params.display_name;
		this.no_merge_row_data = params.no_merge_row_data || false;

		// grid
		this.columns      = params.columns;
		this.hash_columns = this.__make_columns_to_hash();
		this.$edited_row  = null;
		this.markup_with_divs = params.markup_with_divs || false;
		//actions definition
		this.grid_actions = params.grid_actions || false;
		// header
		this.header_hidden = params.header_hidden || false;
		this.header_lables_inside_fields = params.header_lables_inside_fields || false;
		// ids
		this.id_grid_container            = params.ids.grid_container || false;
		this.id_paginator                 = params.ids.paginator;
		this.id_grid                      = params.ids.grid;
		this.id_not_results_found         = params.ids.not_results_found;
		this.id_container_new_btn         = params.ids.container_new_btn;
		this.id_delete_all                = params.ids.delete_all;
		// ids popup0
		this.id_new_item                  = params.ids.new_item;
		this.id_disclaimer                = params.ids.disclaimer;
		this.id_popup                     = params.ids.popup;
		this.id_popup_msg                 = params.ids.popup_msg;
		this.id_popup_create              = params.ids.popup_create;
		this.id_popup_create_and_continue = params.ids.popup_create_and_continue;
		this.id_popup_title               = params.ids.popup_title;
		this.id_popup_close               = params.ids.popup_close;
		this.id_popup_input               = params.ids.popup_input_id;
		this.id_popup_waiting             = params.ids.popup_waiting;
		// ids popup1
		this.id_new_item1                 = params.ids.new_item1;
		this.id_popup1                    = params.ids.popup1;
		this.id_popup_msg1                = params.ids.popup1_msg;
		this.id_popup_create1             = params.ids.popup1_create;
		this.id_popup_create_and_continue1= params.ids.popup1_create_and_continue;
		this.id_popup_title1              = params.ids.popup1_title;
		this.id_popup_close1              = params.ids.popup1_close;
		this.id_popup_input1              = params.ids.popup1_input_id;
		this.id_popup_waiting1            = params.ids.popup1_waiting;
		// ids search
		this.id_search_builtin_panel      = params.ids.search_builtin_panel;
		this.id_search_container          = params.ids.search_container;
		this.id_search_text               = params.ids.search_text;
		this.id_search_reset              = params.ids.search_reset;
		this.id_search_apply              = params.ids.search_apply;
		// popup
		this.popup_is_always_visible      = params.popup_is_always_visible || false;
		this.popup_must_show_messages_in_frm = params.popup_must_show_messages_in_frm || false;
		// html
		this.m_html = params.html || false;
		// others
		this.not_hide_third_button        = params.not_hide_third_button || false;
		this.loading_waiting_msg          = params.loading_waiting_msg || false;
		this.$custom_waiting_container    = params.$custom_waiting_container;
		this.add_with_enter               = typeof(params.add_with_enter) === "boolean" ? params.add_with_enter : true;
		// funcion que calcula por cual atributo se va a ordenar el array cuando se cree a partir de un hash (opcional)
		this.order_info                   = params.order_info || undefined;

		// tipos de labels del pie de pagina
		this.paged_text_format_types = {
			many     : {text: "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"     }, // 1-10 de muchos (el por defecto si es que no llega como parametro cuando se ha instanciado la clase)
			row_count: {text: "<%=_T('{page_row_start}-{page_row_end} of {row_count}')%>"}  // 1-10 de 5000
		};
		// el pie de pagina con el cual se ha instanciado o sino el por defecto
		let paged_text_format_type = this.paged_text_format_types[params.paged_text_format] || this.paged_text_format_types.many;
		// paginator
		this.paginator_info = {
			rows_per_page    : params.rows_per_page || 3, // default
			show_at_least    : params.show_at_least || 2, // default
			without_paginator: params.without_paginator || false,
			paged_text_format: paged_text_format_type.text,
			sortable         : params.sortable || false,
			on_sorted: function() {
				(params.on_sorted || $.noop)();
			}
		};

		this.strings = {
			// string popup0
			title_new                  : params.strings.title_new,
			title_update               : params.strings.title_update,
			title_readonly             : params.strings.title_readonly,
			not_results_found          : params.strings.not_results_found,
			can_remove                 : params.strings.can_remove,
			button_create              : params.strings.button_create,
			button_create_and_continue : params.strings.button_create_and_continue,
			button_update              : params.strings.button_update,
			button_close               : params.strings.button_close,
			can_remove_all             : params.strings.can_remove_all,
			// string popup1
			title_new1                 : params.strings.title_new1,
			title_update1              : params.strings.title_update1,
			not_results_found1         : params.strings.not_results_found1,
			can_remove1                : params.strings.can_remove1,
			button_create1             : params.strings.button_create1,
			button_create_and_continue1: params.strings.button_create_and_continue1,
			button_update1             : params.strings.button_update1,
			button_close1              : params.strings.button_close1
		};

		let callbacks = params.callbacks || {};
		this.callbacks = {
			save: callbacks.save || function(callback, must_create_and_contiune, current_popup) {
				if (self.owner.save) {
					self.owner.save(callback, must_create_and_contiune, current_popup);
				}
				else {
					callback(false);
				}
			},
			edit: callbacks.edit || function(row, current_popup) {
				if (self.owner.on_open_popup) {
					self.owner.on_open_popup(row, current_popup);
				}
			},
			on_added_row: callbacks.on_added_row || $.noop,
			on_updated_row: callbacks.on_updated_row || $.noop,

			remove: callbacks.remove || function(_, callback) {
				callback(true);
			},
			on_removed_row: callbacks.on_removed_row || $.noop,
			on_closed_popup: callbacks.on_closed_popup || function(current_popup) {
				if (self.owner.on_closed_popup) {
					self.owner.on_closed_popup(current_popup);
				}
			},
			load_items: callbacks.load_items || function(page, callback) {
				if (self.owner.load_items) {
					self.owner.load_items(page, callback, self.paginator_name);
				}
				else {
					callback(false);
				}
			},
			on_loaded_items: callbacks.on_loaded_items || function(callback) {
				self.__show_or_hide_loading_msg(false);
				callback();
			},
			show_or_hide_loading_msg: callbacks.show_or_hide_loading_msg || function(must_show, loading_waiting_msg) {
				if (must_show) {
					if (self.callbacks.must_shown_loading_msg()) {
						self.frm.show_waiting(loading_waiting_msg, self.$custom_waiting_container);
					}
				}
				else {
					// Pequeno timeout de 1 segundo, solo para dar efecto y que se visualice el waiting, por algun momento, ya que si la carga de la grilla es instantanea, se realiza un juego de pantalla, muy feo.
					window.setTimeout(function() {
						self.frm.hide_waiting();
					}, 1000);
				}
			},
			must_shown_loading_msg: callbacks.must_shown_loading_msg || function() {
				return true;
			},
			custom_validation_cell_value: callbacks.custom_validation_cell_value || function(_, callback) {
				callback(true);
			},
			on_show_popup_msg: callbacks.on_show_popup_msg || $.noop,
			on_hide_popup_msg: callbacks.on_hide_popup_msg || $.noop,
			//
			on_selected_row: callbacks.on_selected_row || $.noop,
			on_selected_all_row: callbacks.on_selected_all_row || $.noop
		};

		this.m_rowsErrorMarkClasses = ["Info", "Error"];

		this.validate = {
			required: false,
			selected: false,
			messages: {}
		};

		if (params.validate) {
			this.validate.get_message_container = params.validate.get_message_container || $.noop;
			if (params.validate.required) {
				this.validate.required = params.validate.required;
				this.validate.messages.required = params.validate.required_msg;
			}
			if (params.validate.selected) {
				this.validate.selected = params.validate.selected;
				this.validate.messages.selected = params.validate.selected_msg;
			}
		}

		// sample: contextual
		this.new_actions = params.new_actions;

		if (params.auto_initialize) {
			this.initialize();
			this.set_events();
		}

		this.search_text = null;
		this.m_filtered = false;

		return true;
	}

	if (typeof inConcert.conf_form_paginator !== "function") {
		inConcert.conf_form_paginator = conf_form_paginator;
	}

	conf_form_paginator.prototype.__make_columns_to_hash = function()
	{
		let hash_columns = {};
		(this.columns || []).forEach(column => {
			if (column.id) {
				hash_columns[column.id] = column;
			}
		});
		return hash_columns;
	};

	conf_form_paginator.prototype.get_columns = function(format_type = "hash")
	{
		return format_type === "hash" ? this.hash_columns : this.columns;
	};

	conf_form_paginator.prototype.get_column = function(column_id)
	{
		if (!column_id || !this.hash_columns) {
			return;
		}
		return this.hash_columns[column_id];
	};

	conf_form_paginator.prototype.start_validation = function()
	{
		const self = this;

		const return_and_show_error = function(message, err_type) {
			const $grid = $("#" + self.id_grid);
			const $custom_message_container = self.validate.get_message_container();
			self.frm.show_validation_error($grid, message, undefined, undefined, undefined, $custom_message_container);
			return {
				success : false,
				err_type: err_type,
				message : message
			};
		};

		let msg;
		let total_rows = this.count_rows();

		// 1 - validacion de requeridos
		if (this.validate.required) {
			let default_required_msg = "<%=_T('You have to add at least one row')%>";
			if (this.validate.required === true) {
				if (total_rows === 0) {
					msg = this.validate.messages.required || default_required_msg;
					return return_and_show_error(msg, "IS REQUIRED");
				}
			}
			else if (typeof(this.validate.required) === "function") {
				let is_ok = this.validate.required();
				if (!is_ok) {
					msg = this.validate.messages.required || default_required_msg;
					return return_and_show_error(msg, "IS REQUIRED");
				}
			}
		}
		// 2 - validacion de seleccionados
		if (total_rows > 0) {
			if (this.validate.selected) {
				let default_selected_msg = "<%=_T('You have to select at least one row')%>";
				if (this.validate.selected === true) {
					let total_selected = this.get_selected_items().length;
					if (total_selected === 0) {
						msg = this.validate.messages.selected || default_selected_msg;
						return return_and_show_error(msg, "HAVE TO SELECT AT LEAST ONE");
					}
				}
				else if (typeof(this.validate.selected) === "function") {
					let is_ok = this.validate.selected();
					if (!is_ok) {
						msg = this.validate.messages.selected || default_selected_msg;
						return return_and_show_error(msg, "HAVE TO SELECT AT LEAST ONE");
					}
				}
			}
		}

		return {success: true};
	};

	conf_form_paginator.prototype.get_validation_message_err = function(type)
	{
		var msg_err;
		type = (type || "").toLowerCase();
		if (type === "required") {
			msg_err = this.validate.messages.required;
		}
		return msg_err;
	};

	conf_form_paginator.prototype.dispose = function()
	{
		this.search_text = null;
		this.m_filtered = false;
		var $txt_search = $("#" + this.id_search_text);
		$txt_search.val("");
	};

	conf_form_paginator.prototype.show_not_results_found = function(/*message*/)
	{
		const caption = this.__eval_id_by_popup("strings.not_results_found");
		$("#" + this.id_not_results_found)
			.text(caption)
			.show();
		$("#" + this.id_grid).hide();
		$("#" + this.id_paginator).hide();
	};

	conf_form_paginator.prototype.hide_not_results_found = function()
	{
		const paginator_info = this.get_paginator_info();
		$("#" + this.id_grid).show();
		if (paginator_info.without_paginator) {
			$("#" + this.id_paginator).hide();
		}
		else {
			$("#" + this.id_paginator).show();
		}
		$("#" + this.id_not_results_found).hide();
	};

	conf_form_paginator.prototype.show_or_hide_not_results_found = function(callback)
	{
		var self = this;
		callback = callback || $.noop;

		var show_or_hide;
		var top_retries = 10;
		var count_retries = 0;

		show_or_hide = function()
		{
			if (count_retries === top_retries) {
				return;
			}
			if (!self.paginator) {
				window.setTimeout(function() {
					show_or_hide();
					count_retries++;
				}, 100);
				return;
			}
			var page_count = self.paginator.total_rows_in_page(1);
			if (page_count > 0) {
				self.hide_not_results_found();
			}
			else {
				self.show_not_results_found();
			}
			callback();
		};

		show_or_hide();
	};

	conf_form_paginator.prototype.on_removed_row = function($row, callback = $.noop)
	{
		const self = this;
		const row_data = self.get_row_data($row);

		// primero obtengo los hermanos, asi despues puedo actualizar el contador
		const $siblings = $row.siblings();

		// column selector
		let column_selector = this.__jquery_column_selector("counter");

		// agrega un efecto a la row antes de eliminarla
		$row.fadeOut("slow", function()
		{
			self.paginator.remove($row);
			// Actualizo la celda contador de los rows hermanos, que contengan un numero superior
			if ($siblings.length > 0) {
				const counter = parseInt($row.find(column_selector).text(), 10);

				$.each($siblings, function(_, sibling)
				{
					let $sibling_row = $(sibling);
					let $sibling_counter = $sibling_row.find(column_selector);
					let sibling_counter = parseInt($sibling_counter.text(), 10);
					if (sibling_counter > counter) {
						sibling_counter--;
						$sibling_counter.text(sibling_counter);
						let item = $sibling_row.data("row_data");
						if (item) {
							item.__number = sibling_counter;
							self.__update_row_data($row, item);
						}
					}
				});
			}
			// muestra u oculta el mensaje de "no resultados"
			self.show_or_hide_not_results_found(function() {
				callback(true);
			});
			// habilita o inhabilita el boton delete all
			self.__enable_disable_delete_all();
			// invoca al callback de la clase que me ha instanciado.
			self.callbacks.on_removed_row(row_data);
		});
	};

	conf_form_paginator.prototype.remove = function(item, $row, callback = $.noop)
	{
		const self = this;
		$row = $row || this.$get_edited_row();

		this.callbacks.remove(item, function(success) {
			if (!success) {
				callback(false);
				return;
			}
			self.on_removed_row($row, callback);
		});
	};

	conf_form_paginator.prototype.__enable_disable_delete_all = function()
	{
		if (!this.id_delete_all) {
			return;
		}
		const $delete_all = $("#" + this.id_delete_all);
		if ($delete_all.length === 0) {
			return;
		}
		const total_rows = this.count_rows();
		if (total_rows === 0) {
			$delete_all.prop("disabled", true);
		}
		else {
			$delete_all.prop("disabled", false);
		}
	};

	conf_form_paginator.prototype.on_added_row = function(item, callback = $.noop)
	{
		// habilita o inhabilita el boton delete all
		this.__enable_disable_delete_all();
		// invoca al callback de la clase que me ha instanciado.
		this.callbacks.on_added_row(item);
		callback();
	};

	conf_form_paginator.prototype.on_updated_row = function(item, callback = $.noop)
	{
		// invoca al callback de la clase que me ha instanciado.
		this.callbacks.on_updated_row(item);
		callback();
	};

	conf_form_paginator.prototype.remove_dom_row_by_id = function(item_id, callback = $.noop)
	{
		let $row = this.$find_row(item_id);
		if (!$row) {
			callback(false);
			return;
		}
		let item = this.get_row_data($row);
		this.remove(item, $row, callback);
	};

	conf_form_paginator.prototype.__get_display_name = function(item)
	{
		if (!item) {
			return "";
		}
		let display_name = "";
		if (typeof(this.display_name) === "string") {
			display_name = item[this.display_name];
		}
		else if (typeof(this.display_name) === "function") {
			display_name = this.display_name(item);
		}
		display_name = this.escape(display_name);
		return display_name || "";
	};

	conf_form_paginator.prototype.confirm = function(message, callback = $.noop)
	{
		this.frm.confirm(message, callback);
	};

	conf_form_paginator.prototype.$tbody = function()
	{
		const $grid = $("#" + this.id_grid);
		let selector = this.markup_with_divs ? "[tbody='true']" : "tbody";
		const $tbody = $grid.find(selector);
		return $tbody;
	};

	conf_form_paginator.prototype.$thead = function()
	{
		const $grid = $("#" + this.id_grid);
		let selector = this.markup_with_divs ? "[thead='true']" : "thead";
		const $thead = $grid.find(selector);
		return $thead;
	};

	conf_form_paginator.prototype.get_markup_definition_by_type = function()
	{
		if (!this.m_markup_by_type) {
			this.m_markup_by_type = {
				div: {
					container: function(info) {
						let grid_dom_id = info.grid_dom_id || inConcert.Utils.generateGuid();
						return "" +
							"<div id='" + grid_dom_id + "' table='true' class='" + (info.grid_class || "") + "'>" +
								"<div thead='true' class='" + (info.thead_class || "") + "'></div>" +
								"<div tbody='true' class='" + (info.tbody_class || "") + "'></div>" +
							"</div>";
					},
					head: {
						row: "ul",
						column: "li"
					},
					body: {
						row: "ul",
						column: "li"
					}
				},
				table: {
					container: function(info) {
						let grid_dom_id = info.grid_dom_id || inConcert.Utils.generateGuid();
						return "" +
							"<table id='" + grid_dom_id + "' cellpadding='0' cellspacing='0' border='0' class='" + (info.grid_class || "") + "'>" +
								"<thead class='" + (info.thead_class || "") + "'></thead>" +
								"<tbody class='" + (info.tbody_class || "") + "'></tbody>" +
							"</table>";
					},
					head: {
						row: "tr",
						column: "th"
					},
					body: {
						row: "tr",
						column: "td"
					}
				}
			};
		}
		let type = this.markup_with_divs ? "div" : "table";
		return this.m_markup_by_type[type];
	};

	conf_form_paginator.prototype.set_events = function()
	{
		const self = this;

		var remove = function($row)
		{
			const item = $row.data("row_data");
			let caption = self.__eval_id_by_popup("strings.can_remove");
			caption = caption || "<%=_T('Do you want to delete: {0} ?')%>";
			let display_name = self.__get_display_name(item);
			const message = FORMAT(caption, display_name);

			self.confirm(message, function(success) {
				if (success) {
					self.remove(item, $row);
				}
			});
		};

		var edit = function($row) {
			const item = $row.data("row_data");
			if (!self.canEdit(item)) {
				return;
			}
			self.$edited_row = $row;
			self.show_popup();
		};

		const $tbody = this.$tbody();
		let rows_event_selector = this.__jquery_row_selector();

		// doble click sobre la row edita la misma
		$tbody
			.off("dblclick", rows_event_selector)
			.on("dblclick", rows_event_selector, function(e) {
				e.stopPropagation();
				edit($(this));
			});

		// botones system (edit, delete) y botones customs
		let event_selector = rows_event_selector + " button[action]";
		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				const $button = $(this);
				const $column = $button.parent();
				const column_id = $column.attr("column_id");
				const column_def = self.get_column(column_id);
				let $row;
				if (column_def.grouper) {
					$row = $column.parent().parent();
				}
				else {
					$row = $column.parent();
				}
				const action_id = $button.attr("action");
				const action_type = $button.attr("action_type");

				if (action_type === "system") {
				// acciones del sistema
					if (action_id === "delete") {
					// boton delete de la lista de parametros
						remove($row);
					}
					else if (action_id === "edit") {
					// boton edit de la lista de parametros
						edit($row);
					}
					else {
						throw("conf_form_paginator.prototype.set_events. Unrecognized system action id: '" + action_id + "'");
					}
				}
				else {
				// acciones custom
					const columns = self.columns;
					let executed = false;
					for (let key in columns || []) {
						if (columns.hasOwnProperty(key)) {
							let column = columns[key];
							if ((column.id === "_action" || column.type === "actions") && column.custom_action && column.custom_action.hasOwnProperty(action_id)) {
								var custom_action = column.custom_action[action_id];
								if (typeof(custom_action.execute === "function")) {
									custom_action.execute($row, $button);
									executed = true;
								}
							}
						}
					}
					if (!executed) {
						throw("conf_form_paginator.prototype.set_events. Unrecognized custom action id: '" + action_id + "'");
					}
				}
			});

		//--------------------------------------------------------------------------
		// eventos focusin y focusout de los controles editables (textbox y combo)
		//--------------------------------------------------------------------------

		const editable_focusin = function($ctrl) {
			const value = $ctrl.val();
			$ctrl.attr("old_value", value);
			//console.debug("conf_form_paginator. 1 - focusin - column_id: " + $ctrl.parent().attr("column_id") + ", value: " + value);
		};
		const update_row = function($ctrl, value) {
			const ctrl_type = $ctrl.attr("ctrl_type");
			const $column = $ctrl.parent();
			const column_id = $column.attr("column_id");
			const column_def = self.get_column(column_id);
			let $row;
			if (column_def.grouper) {
				$row = $column.parent().parent();
			}
			else {
				$row = $column.parent();
			}
			const row_data = self.get_row_data($row);

			// Si ha modificado el valor de la celda/columna que corresponde al name "display_name" que se ha definido en la clase,
			//   tengo que actualizar el atributo "row_name" de la row con el nuevo valor.
			if (typeof(self.display_name) === "string") {
				if (self.display_name === column_id) {
					let current_row_name = $row.attr("row_name");
					if (current_row_name !== value) {
						$row.attr("row_name", value);
					}
				}
			}

			//console.debug("conf_form_paginator. 2 - focusout - column_id: " + column_id + ", value: " + value);

			self.__validate_cell_value(ctrl_type, $ctrl, value, column_id, row_data, column_def,
				function(success, message) {
					if (!success) {
						if (message) {
							self.show_status_message(message, "Error", 5000);
						}
						return;
					}
					let column_name = column_def.get_column_name ? column_def.get_column_name(row_data) : column_id;
					row_data[column_name] = value;
					self.__update_row_data($row, row_data);
				}
			);
		};
		var editable_focusout = function($ctrl) {
			//console.debug("conf_form_paginator. 2 - focusout);
			const value = $ctrl.val();
			update_row($ctrl, value);
		};
		var editable_chk_click = function($ctrl) {
			//console.debug("editable_click. 2 - focusout);
			const value = $ctrl.prop("checked") ? "1" : "0";
			update_row($ctrl, value);
		};
		//--------------------------------------------

		// celdas editables (textbox)
		let column_selector = this.__jquery_column_selector("cell_input");
		let textbox_event_selector = rows_event_selector + " " + column_selector + " [ctrl_type='textbox']";
		event_selector = textbox_event_selector;
		$tbody
			.off("focusin", event_selector)
			.on("focusin", event_selector, function() {editable_focusin($(this));})
			.off("focusout", event_selector)
			.on("focusout", event_selector, function() {editable_focusout($(this));});

		// celdas editables (combo)
		column_selector = this.__jquery_column_selector("cell_combo");
		let combo_event_selector = rows_event_selector + " " + column_selector + " [ctrl_type='combo']";
		event_selector = combo_event_selector;
		$tbody
			.off("focusin", event_selector)
			.on("focusin", event_selector, function() {editable_focusin($(this));})
			.off("focusout", event_selector)
			.on("focusout", event_selector, function() {editable_focusout($(this));});

		// celdas editables (checkbox)
		column_selector = this.__jquery_column_selector("cell_checkbox");
		let chk_event_selector = rows_event_selector + " " + column_selector + " [ctrl_type='checkbox']";
		event_selector = chk_event_selector;
		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function() {editable_chk_click($(this));});

		// celdas selectors de cada row
		column_selector = this.__jquery_column_selector("row_selector");
		event_selector = rows_event_selector + " " + column_selector + " [type='checkbox']";
		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.__toggle_checkbox_select_all_rows();
				const $chk = $(this);
				const is_selected = $chk.prop("checked");
				self.callbacks.on_selected_row($chk, is_selected);
			});

		if (this.id_delete_all) {
			const $delete_all = $("#" + this.id_delete_all);
			$delete_all
				.off("click")
				.on("click", function() {
					const message = self.strings.can_remove_all;
					self.confirm(message, function(success) {
						if (success) {
							self.remove_all_rows();
						}
					});
				});
		}

		// header
		const $thead = this.$thead();

		let col_event_selector = "[header_column_type]";
		// click sobre las columnas del header
		$thead
			.off("click", col_event_selector)
			.on("click", col_event_selector, function(e) {
				e.stopPropagation();
				const $this = $(this);
				const header_column_type = $this.attr("header_column_type");
				if (header_column_type === "row_selector") {
				// click sobre el boton select all
					const selected = $this.prop("checked");
					self.__toggle_select_rows(selected);
				}
			});

		// popup & search events
		this.__set_events_popup();
		this.__set_events_search();
	};

	conf_form_paginator.prototype.select_row = function(row_id, is_selected)
	{
		const $row = this.$find_row(row_id);
		if (!$row || $row.length === 0) {
			return false;
		}
		const $row_selector = $row.find("[column_type='row_selector']");
		if (!$row_selector || $row_selector.length === 0) {
			return false;
		}
		const $checkbox = $row_selector.find("input[type='checkbox']");
		const current_state = $checkbox.prop("checked");
		if (current_state === is_selected) {
			return false;
		}
		$checkbox.prop("checked", is_selected);
		//this.__toggle_select_rows(is_selected);
		return true;
	};

	conf_form_paginator.prototype.__toggle_select_rows = function(selected)
	{
		const is_filtered = this.__is_filtered();
		const $rows = this.$get_rows();

		$.each($rows, function(_, row) {
			const $row = $(row);
			const $row_selector = $row.find("[column_type='row_selector']");
			const $checkbox = $row_selector.find("input[type='checkbox']");
			if (!is_filtered || $checkbox.is(":visible")) {
				$checkbox.prop("checked", selected);
			}
		});

		this.callbacks.on_selected_all_row(selected);
	};

	conf_form_paginator.prototype.$get_rows = function()
	{
		if (!this.paginator) {
			return;
		}
		return this.paginator.$get_rows();
	};

	conf_form_paginator.prototype.__toggle_checkbox_select_all_rows = function()
	{
		if (!this.m_has_row_selector) {
			return;
		}
		if (this.m_toggle_select_all_rows_status === "RUNNING") {
			return;
		}
		this.m_toggle_select_all_rows_status = "RUNNING";

		let selected_all = true;
		const $rows = this.$get_rows();

		$.each($rows, function(_, row) {
			const $row = $(row);
			const $row_selector = $row.find("[column_type='row_selector']");
			const $checkbox = $row_selector.find("input[type='checkbox']");
			const is_selected = $checkbox.prop("checked");
			if (!is_selected) {
				selected_all = false;
				return false; // break
			}
		});

		const $thead = this.$thead();
		const $select_all = $thead.find("[header_column_type]");
		$select_all.prop("checked", selected_all);

		this.m_toggle_select_all_rows_status = "COMPLETED";
	};

	conf_form_paginator.prototype.remove_all_rows = function()
	{
		const self = this;
		const $rows = this.$get_rows();
		$.each($rows, function(_, row) {
			const $row = $(row);
			const row_data = self.get_row_data($row);
			self.remove(row_data, $row);
		});
	};

	conf_form_paginator.prototype.show_status_message = function(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks)
	{
		this.frm.show_status_message(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks);
	};

	conf_form_paginator.prototype.__validate_cell_value = function(ctrl_type, $ctrl, value, column_id, row_data, column_def, callback = $.noop)
	{
		const self = this;
		window.setTimeout(function()
		{
			let old_value;
			if (ctrl_type === "textbox") {
				old_value = $ctrl.attr("old_value");
			}
			// validacion de requeridos
			if ($.isArray(column_def.validation)) {
				for (let i = 0; i < column_def.validation.length; i++) {
					let validation = column_def.validation[i];
					if (validation.required) {
						if (value.trim().length === 0) {
							if (old_value) {
								$ctrl.val(old_value);
							}
							$ctrl.focus();
							const message = validation.message;
							callback(false, message);
							return;
						}
					}
				}
			}
			// custom validations
			const info = {
				ctrl_type: ctrl_type,
				$ctrl    : $ctrl,
				value    : value,
				old_value: old_value,
				column_id: column_id,
				row_data : row_data
			};
			self.callbacks.custom_validation_cell_value(info,
				function(success, action, message) {
					if (!success) {
						if (ctrl_type === "textbox") {
							if (action === "rollback") {
								$ctrl.val(old_value);
							}
						}
					}
					callback(success, message);
				}
			);
		});
	};

	conf_form_paginator.prototype.__has_search = function()
	{
		return this.id_search_text ? true : false;
	};

	conf_form_paginator.prototype.__is_builtin_search = function()
	{
		return this.id_search_builtin_panel ? true : false;
	};

	conf_form_paginator.prototype.__set_events_search = function()
	{
		const self = this;

		if (!this.__has_search()) {
			return;
		}

		var trigger_search = function() {
			self.__search();
		};

		let $panel;
		if (this.__is_builtin_search()) {
			$panel = $("#" + this.id_search_builtin_panel);
		}
		else {
			//conf_form_custom_buttons
			$panel = this.frm.dashboard.$get_dom_element_by_id("form_custom_buttons");
		}

		let event_selector = "[id='" + this.id_search_text + "']";
		$panel
			.off("keyup", event_selector)
			.on("keyup", event_selector, function(e) {
				// If user press ctrl/tab/command/arrows don't do anything
				const filtered_control_keys = inConcert.Utils.getControlCharacters();
				if (filtered_control_keys[e.keyCode]) {
					return;
				}
				// if user press Enter trigger the search inmediatly
				if (e.keyCode === 13) {
					trigger_search();
					return;
				}
			});

		event_selector = "[id='" + this.id_search_apply + "']";
		$panel
			.off("click", event_selector)
			.on("click", event_selector, function() {
				trigger_search();
			});

		event_selector = "[id='" + this.id_search_reset + "']";
		$panel
			.off("click", event_selector)
			.on("click", event_selector, function() {
				const $txt_search = $("#" + self.id_search_text);
				$txt_search.val("");
				trigger_search();
			});
	};

	conf_form_paginator.prototype.__is_filtered = function()
	{
		return this.m_filtered;
	};

	// La busqueda se realiza client side
	conf_form_paginator.prototype.__search = function()
	{
		const self = this;

		const $search_container = $("#" + this.id_search_container);
		const $txt_search       = $("#" + this.id_search_text);
		const $btn_reset        = $("#" + this.id_search_reset);
		const $grid             = $("#" + this.id_grid);
		const $paginator        = $("#" + this.id_paginator);

		$search_container.removeClass("twoButtonsFilter");

		let row_selector = this.__jquery_row_selector();

		this.search_text = $txt_search.val().trim();
		if (this.search_text.length === 0) {
			$btn_reset.hide();
		// si no hay texto buscado en el filtro, no filtro las row
			$paginator.show(); // muestro la botonera del paginado
			this.hide_not_results_found();
			let $rows = $grid.find(row_selector);
			$rows.hide();
			this.jump_to_first_page();
			this.m_filtered = false;
			return;
		}

		$search_container.addClass("twoButtonsFilter");
		$btn_reset.show();

		let total_rows = 0;
		let hidden_rows = 0;

		$grid.find(row_selector).each(function(_, row) {
			total_rows++;
			const $row = $(row);
			const row_value = $row.attr("search_text");
			const search_regexp = self.__make_search_regexp(self.search_text);
			if (row_value.search(search_regexp) === -1) {
				$row.hide();
				hidden_rows++;
			}
			else {
				$row.show();
			}
		});
		if (total_rows === hidden_rows) {
			this.show_not_results_found();
		}
		else {
			this.hide_not_results_found();
		}

		this.m_filtered = true;
		$paginator.hide(); // oculto la botonera del paginado
	};

	conf_form_paginator.prototype.__make_search_regexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	conf_form_paginator.prototype.__set_current_popup = function(current_popup = null)
	{
		this.current_popup_number = current_popup;
	};

	conf_form_paginator.prototype.__get_current_popup = function()
	{
		const popup_number = (!this.current_popup_number || this.current_popup_number === 0) ? "" : this.current_popup_number.toString();
		return popup_number;
	};

	conf_form_paginator.prototype.__eval_id_by_popup = function(prefix_id)
	{
		const popup_number = this.__get_current_popup();
		const id = eval("this." + prefix_id + popup_number);
		return id;
	};

	conf_form_paginator.prototype.update_popup_info = function(info)
	{
		const $update = $("#" + this.id_popup_create);
		$update.val(info.caption_btn_update || "<%=_T('Update')%>");
	};

	conf_form_paginator.prototype.get_button_html_type = function()
	{
		const defualt = "INPUT";
		if (!this.m_html || !this.m_html.popup || !this.m_html.popup.actions) {
			return defualt;
		}
		if (typeof this.m_html.popup.actions.html_type !== "string") {
			return defualt;
		}
		return this.m_html.popup.actions.html_type.toUpperCase();
	};

	conf_form_paginator.prototype.show_popup = function(current_popup = 0)
	{
		this.__set_current_popup(current_popup);

		const id_popup = this.__eval_id_by_popup("id_popup");
		const $popup = $("#" + id_popup);

		this.frm.hide_messages();
		this.hide_popup_msg();

		$popup.find(".formFieldValidation").hide();
		$popup.show();

		var row_data;
		var title = "";
		const id_popup_create = this.__eval_id_by_popup("id_popup_create");
		const id_popup_create_and_continue = this.__eval_id_by_popup("id_popup_create_and_continue");

		const $create = $("#" + id_popup_create);
		const $create_and_continue = $("#" + id_popup_create_and_continue);
		// obtenemos el tipo de html utilizado para los botones de acciones <INPUT> o <BUTTON>.
		//  dependido de esto va a ser el metodo "method_name" de jQuery que invocaremos "val()" o "text()"
		const html_button_type = this.get_button_html_type();
		const method_name = html_button_type === "BUTTON" ? "text" : "val";

		if (!this.$edited_row) {
		// si presione el boton para crear uno nuevo
			const create_caption = this.__eval_id_by_popup("strings.button_create");
			$create[method_name](create_caption || "<%=_T('Create')%>").show();

			const create_and_continue_caption = this.__eval_id_by_popup("strings.button_create_and_continue");
			$create_and_continue
				[method_name](create_and_continue_caption || "<%=_T('Confirm & continue')%>")
				.show();

			title = this.__eval_id_by_popup("strings.title_new");
		}
		else {
		// si presione el boton para editar
			const update_caption = this.__eval_id_by_popup("strings.button_update");
			$create[method_name](update_caption || "<%=_T('Update')%>").show();
			if (!this.not_hide_third_button) {
				$create_and_continue.hide();
			}
			title = this.__eval_id_by_popup("strings.title_update");
			row_data = this.$edited_row.data("row_data");
		}

		if (this.frm.is_readonly()) {
			title = this.__eval_id_by_popup("strings.title_readonly") || title;
			$create.hide();
			$create_and_continue.hide();
		}

		this.set_popup_title(title);

		const id_popup_close = this.__eval_id_by_popup("id_popup_close");
		const close_caption = this.__eval_id_by_popup("strings.button_close");
		$("#" + id_popup_close)[method_name](close_caption || "<%=_T('Close')%>");

		const id_new_item = this.__eval_id_by_popup("id_new_item");
		const $new_item = $("#" + id_new_item);
		$new_item.prop("disabled", true);

		this.disable_actions(false);
		this.disable_grid_actions(true);

		if (this.popup_instance) {
			this.popup_instance.show();
		}
		if (this.callbacks.edit) {
			this.callbacks.edit(row_data, current_popup);
		}
	};

	conf_form_paginator.prototype.set_popup_title = function(title = "...")
	{
		const id_popup_title = this.__eval_id_by_popup("id_popup_title");
		const $title = $("#" + id_popup_title);
		const $popup_title_text = $title.find("[popup_title_text]");
		if ($popup_title_text.length > 0) {
			$popup_title_text.text(title);
			return;
		}
		$title.text(title);
	};

	conf_form_paginator.prototype.$get_edited_row = function()
	{
		return this.$edited_row;
	};

	conf_form_paginator.prototype.$set_edited_row = function($edited_row)
	{
		this.$edited_row = $edited_row;
	};

	conf_form_paginator.prototype.cleanup_edited_row = function()
	{
		this.$edited_row = null;
	};

	conf_form_paginator.prototype.set_first_row = function()
	{
		this.cleanup_edited_row();
		const $tbody = this.$tbody();
		let selector = this.__jquery_row_selector();
		selector = selector + ":first";
		const $first_row = $tbody.find(selector);
		this.$set_edited_row($first_row);
	};

	conf_form_paginator.prototype.can_hide_popup = function()
	{
		if (this.popup_is_always_visible) {
			return false;
		}
		return true;
	};

	conf_form_paginator.prototype.hide_popup = function(params = {})
	{
		if (!this.can_hide_popup()) {
			this.disable_grid_actions(false);
			this.cleanup_edited_row();
			this.callbacks.edit();
			return;
		}
		let must_invoke_callback = true;
		if (params.must_invoke_callback === false) {
			must_invoke_callback = false;
		}
		const current_popup = this.__get_current_popup();
		const id_popup = this.__eval_id_by_popup("id_popup");
		$("#" + id_popup).hide();

		const id_new_item = this.__eval_id_by_popup("id_new_item");
		const $new_item = $("#" + id_new_item);
		$new_item.prop("disabled", false);

		this.disable_grid_actions(false);
		this.cleanup_edited_row();

		if (this.popup_instance) {
			this.popup_instance.hide();
		}
		if (must_invoke_callback) {
			this.callbacks.on_closed_popup(current_popup);
		}
		this.__set_current_popup(null);
	};

	conf_form_paginator.prototype.refresh_page = function(on_refresh = $.noop)
	{
		this.paginator.refresh_page(on_refresh);
	};

	conf_form_paginator.prototype.jump_to_first_page = function()
	{
		this.paginator.__jump_to_page(1, null, true);
	};

	conf_form_paginator.prototype.add_or_update_row = function(item, no_refresh_page, force_add, on_added = $.noop)
	{
		const self = this;
		let callback;

		if (force_add || !this.$edited_row) {
			callback = function($item) {
				let count_append = self.__append_row($item);
				if (count_append && !no_refresh_page) {
					let paginator_info = self.get_paginator_info();
					if (!paginator_info.without_paginator) {
						// tiene que tener paginado para refrescar, y el paginado tiene que ser serverside
						self.refresh_page();
					}
				}
				self.hide_not_results_found();
				on_added();
				self.on_added_row(item);
				return count_append;
			};
			this.__make_row(item, null, callback);
			return;
		}
		this.update_row(item, callback);
	};

	conf_form_paginator.prototype.update_row = function(item, on_updated, info)
	{
		on_updated = on_updated || $.noop;
		const self = this;
		const callback = function($item) {
			let result = self.__update_row($item, info);
			on_updated();
			self.on_updated_row(item);
			return result;
		};
		this.__make_row(item, null, callback);
	};

	conf_form_paginator.prototype.refresh_row = function(row_id)
	{
		let row_data = this.find_row_data(row_id);
		if (row_data) {
			this.update_row(row_data);
		}
	};

	conf_form_paginator.prototype.refresh_all_client_side_rows = function()
	{
		const self = this;
		let row_id_attr_name = this.row_id;
		if (!row_id_attr_name) {
			return;
		}
		let rows = this.get_items();
		rows.forEach(row => {
			let row_id = row[row_id_attr_name];
			self.refresh_row(row_id);
		});
	};

	conf_form_paginator.prototype.__has_popup = function()
	{
		return this.id_popup ? true : false;
	};

	conf_form_paginator.prototype.__new_aciton = function($parent, popup_number)
	{
		this.cleanup_edited_row();
		let contextual_menu = this.new_actions && this.new_actions.contextual_menu ? this.new_actions.contextual_menu : null;
		if (contextual_menu && contextual_menu.get_options) {
			let menu_options = contextual_menu.get_options();
			this.frm.toggle_contextual_menu($parent, menu_options);
			return;
		}
		this.show_popup(popup_number);
	};

	conf_form_paginator.prototype.__set_events_popup = function()
	{
		const self = this;

		if (!this.__has_popup()) {
			return;
		}

		// new message event
		let $new_btn_container;
		if (this.id_container_new_btn) {
			$new_btn_container = $("#" + this.id_container_new_btn);
		}
		else {
			$new_btn_container = this.frm.dashboard.$get_dom_element_by_id("form_custom_buttons"); //conf_form_custom_buttons
		}

		var set_events = function(popup_number)
		{
			self.__set_current_popup(popup_number); // Tengo que asignar el popup number de forma temporal para que se asignen los eventos!! Despues lo quito y dejo en null (**)

			const id_popup = self.__eval_id_by_popup("id_popup");
			const $popup = $("#" + id_popup);

			if ($popup.length === 0) {
				return;
			}

			const id_new_item = self.__eval_id_by_popup("id_new_item");
			const event_selector = "#" + id_new_item;
			$new_btn_container
				.off("click", event_selector)
				.on("click", event_selector, function() {
					const $this = $(this);
					self.__new_aciton($this, popup_number);
				});

			const id_popup_create_and_continue = self.__eval_id_by_popup("id_popup_create_and_continue");
			if (id_popup_create_and_continue) {
				$("#" + id_popup_create_and_continue)
					.off("click")
					.on("click", function() {
						const must_create_and_contiune = true;
						self.callbacks.save(function(success, item, response) {
								if (success) {
									self.add_or_update_row(item, must_create_and_contiune);
									if (response) {
										self.show_popup_msg(response.msg, response.type, 5000, response);
									}
								}
								else {
									if (response) {
										self.show_popup_msg(response.msg, response.type, 5000, response);
									}
								}
							},
							must_create_and_contiune,
							popup_number
						);
					}
				);
			}

			const id_popup_create = self.__eval_id_by_popup("id_popup_create");
			if (id_popup_create) {
				$("#" + id_popup_create)
					.off("click")
					.on("click", function() {
						const must_create_and_contiune = false;
						self.callbacks.save(function(success, item, response, must_add_many, force_close_ms, msg_timeout_ms) {
								if (success) {
									if (must_add_many) {
										let items = item;
										items.forEach(one_item => {
											self.add_or_update_row(one_item, must_create_and_contiune);
										});
									}
									else {
										self.add_or_update_row(item, must_create_and_contiune);
									}
									self.hide_popup();
									if (response) {
										self.show_status_message(response.msg, response.type, 5000, undefined, undefined, response.$msg_container);
									}
								}
								else {
									if (response) {
										msg_timeout_ms = msg_timeout_ms || false;
										self.show_popup_msg(response.msg, response.type, msg_timeout_ms, response);
									}
								}
								if (force_close_ms) {
									self.disable_actions(true);
									window.setTimeout(function() {
										self.hide_popup();
									}, force_close_ms);
								}
							},
							must_create_and_contiune,
							popup_number
						);
					}
				);
			}

			const id_popup_close = self.__eval_id_by_popup("id_popup_close");
			if (id_popup_close) {
				const $close = $("#" + id_popup_close);
				let popup_number = self.__get_current_popup();
				$close.attr("popup_number", popup_number);
				$close.off("click")
					.on("click", function() {
						let popup_number = $close.attr("popup_number");
						self.__set_current_popup(popup_number);
						self.hide_popup();
					}
				);
			}

			const id_popup_title = self.__eval_id_by_popup("id_popup_title");
			if (id_popup_title) {
				const $popup_title_close = $("#" + id_popup_title).find("[popup_close]");
				$popup_title_close.off("click")
					.on("click", function() {
						self.hide_popup();
					}
				);
			}

			const id_popup_input = self.__eval_id_by_popup("id_popup_input");
			if (id_popup_input && id_popup_create_and_continue && self.add_with_enter) {
				// el Enter en el control Id, ejecuta el create & conitune
				$("#" + id_popup_input)
					.off("keyup")
					.on("keyup", function(e) {
						if (e.keyCode === 13) {
							$("#" + id_popup_create_and_continue).trigger("click");
							return;
						}
					}
				);
			}
		};

		for (let i = 0; i <= 1; i++) {
			set_events(i);
		}
		this.__set_current_popup(null); // (**) Despues de que se han asignado los eventos, lo dejo en null
	};

	conf_form_paginator.prototype.load_items = function(page, callback)
	{
		if (this.loading_waiting_msg) {
			this.__show_or_hide_loading_msg(true);
		}
		this.callbacks.load_items(page, callback);
	};

	conf_form_paginator.prototype.__on_loaded_items = function(callback = $.noop)
	{
		const self = this;
		window.setTimeout(function() {
			self.callbacks.on_loaded_items(callback);
		});
	};

	conf_form_paginator.prototype.markup_row = function()
	{
		const markup = this.get_markup_definition_by_type();
		return markup.body.row;
	};

	conf_form_paginator.prototype.__destroy_paginator = function()
	{
		const $tbody = this.$tbody();
		let markup_row = this.markup_row();
		$tbody.find(markup_row).remove();
		if (!this.paginator) {
			return;
		}
		this.paginator = this.paginator.destroy();
	};

	conf_form_paginator.prototype.__has_row_id = function()
	{
		return this.row_id !== false;
	};

	conf_form_paginator.prototype.__make_row = function(row, row_count, on_row_created)
	{
		on_row_created = on_row_created || $.noop;

		if (row_count === null) {
			const $tbody = this.$tbody();
			let markup_row = this.markup_row();
			row_count = $tbody.find(markup_row).length + 1;
		}

		var is_created = false;
		if (!this.__has_row_id() && !row.hasOwnProperty("id")) {
			row.id = inConcert.Utils.generateGuid();
		}
		var $new_rows = this.__$get_rows_html(row, row_count); // es un array de jquery rows html
		if ($new_rows && $new_rows.length > 0) {
			if (on_row_created($new_rows)) {
				is_created = true;
			}
		}
		return is_created;
	};

	conf_form_paginator.prototype.get_row_data = function($row) {
		return $row.data("row_data");
	};

	conf_form_paginator.prototype.find_row_data = function(row_id) {
		var $row = this.$find_row(row_id);
		if (!$row || $row.length === 0) {
			return false;
		}
		return $row.data("row_data");
	};

	conf_form_paginator.prototype.$find_row = function(row_id) {
		row_id = this.escape(row_id);
		const $tbody = this.$tbody();
		let selector = this.__jquery_row_selector("row_id", row_id);
		const $current_row = $tbody.find(selector);
		return $current_row;
	};

	conf_form_paginator.prototype.$find_row_by_name = function(row_name) {
		row_name = this.escape(row_name);
		row_name = row_name.toLowerCase();
		const $tbody = this.$tbody();
		let selector = this.__jquery_row_selector("row_name", row_name);
		const $current_row = $tbody.find(selector);
		return $current_row;
	};

	conf_form_paginator.prototype.__append_row = function($list) {
		if (!$.isArray($list) || $list.length === 0) {
			return false;
		}
		var $row = $list[0];
		var new_row_id = $row.attr("row_id");
		var $current_row =  this.$find_row(new_row_id);
		if ($current_row && $current_row.length > 0) {
			return false;
		}
		const $tbody = this.$tbody();
		let markup_row = this.markup_row();
		var row_count = $tbody.find(markup_row).length + 1;
		this.paginator.append($row);
		this.cleanup_edited_row();
		return row_count;
	};

	conf_form_paginator.prototype.__update_row = function($list, info) {
		if (!$.isArray($list) || $list.length === 0) {
			return false;
		}
		let $row = $list[0];
		let $current_row;
		if (info && info.find_type === "row_name") {
			let row_name = $row.attr("row_name");
			$current_row =  this.$find_row_by_name(row_name);
		}
		else {
			let row_id = $row.attr("row_id");
			$current_row =  this.$find_row(row_id);
		}
		if (!$current_row || $current_row.length === 0) {
			return false;
		}
		// column selector
		let column_selector = this.__jquery_column_selector("counter");
		let counter = $current_row.find(column_selector).text();
		$row.find(column_selector).text(counter);
		let item = $row.data("row_data");
		item.__number = counter;
		this.__update_row_data($row, item);

		$current_row.html("");
		let new_html = $row.html();
		$current_row.html(new_html);
		$current_row.attr("row_id", $row.attr("row_id"));

		this.__update_row_data($current_row, item);
		// el clean up del row - se realiza al cerrar el popup
		return true;
	};

	conf_form_paginator.prototype.__update_column_counter = function()
	{
		const self = this;

		if (!this.has_counter_column) {
			return;
		}

		let counter = 0;
		const $grid = $("#" + this.id_grid);
		let row_selector = this.__jquery_row_selector();
		let column_selector = this.__jquery_column_selector("counter");

		$grid.find(row_selector).each(function(_, row) {
			counter++;
			const $row = $(row);
			$row.find(column_selector).text(counter);
			let item = $row.data("row_data");
			item.__number = counter;
			self.__update_row_data($row, item);
		});
	};

	conf_form_paginator.prototype.count_rows = function()
	{
		const $tbody = this.$tbody();
		let markup_row = this.markup_row();
		let row_count = $tbody.find(markup_row).length;
		return row_count;
	};

	conf_form_paginator.prototype.__init_paginator = function (rows, callback = $.noop) {
		const self = this;

		rows = rows || [];
		this.__destroy_paginator();
		var row_count = this.count_rows() + 1;

		var $items = [];

		let order_info = this.order_info;
		let sorted_rows;
		if (order_info) {
			sorted_rows = inConcert.Utils.sortArray(rows, null, order_info.options.order, order_info.sort_value_fn);
		}
		else {
			sorted_rows = rows;
		}

		const $container = $("#" + this.id_paginator);
		const $tbody = this.$tbody();
		let paginator_info = this.get_paginator_info();

		// Cargar en modo async, para cuando sea un volumen grande de rows. Se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
		if (this.paginator_type === "ALL_ROWS_CLIENT_SIDE_ASYNC") {
			const init_paginator = function () {
				self.paginator = jQuery.Paginator.init({
					pages_container: $container,
					data_container: $tbody,
					previous: "",
					next: "",
					show_at_least: paginator_info.show_at_least,
					rows_per_page: paginator_info.rows_per_page,
					items: $items,
					text_format: paginator_info.paged_text_format,
					translated_many: "<%=_T('many')%>",
					on_changed_page: $.noop
				});

				// habilita o inhabilita el boton delete all
				self.__enable_disable_delete_all();
				self.__toggle_checkbox_select_all_rows();
				self.paginator.no_more_pages();
				self.__on_loaded_items(function () {
					callback();
					// on_done();
				});
				//callback();
			};

			inConcert.Utils.foreach_timeout({
				rows: sorted_rows,
				append_fn: function (row) {
					var is_created = self.__make_row(row, row_count, function ($new_rows) {
						// $new_rows es un array de jquery rows html
						$items = $items.concat($new_rows);
						return true;
					});
					if (is_created) {
						row_count++;
						self.on_added_row(row);
					}
					else {
						self.on_updated_row(row);
					}
				},
				waiting_message : "<%=_T('Please wait, adding row, {0} of {1}.')%>",
				waiting_fn : function( message ) {
					self.frm.show_waiting(message);
				},
				complete_fn: function () {
					init_paginator();
				}
			});
		}
		else {
			const make_row_fn = function() {
				return function ($new_rows) {
					// $new_rows es un array de jquery rows html
					$items = $items.concat($new_rows);
					return true;
				};
			};

			for (let _ in sorted_rows) {
				if (sorted_rows.hasOwnProperty(_)) {
					let row = sorted_rows[_];
					let is_created = self.__make_row(row, row_count, make_row_fn());
					if (is_created) {
						row_count++;
						self.on_added_row(row);
					}
				}
			}

			// $.each(sorted_rows, function (_, row) {
			// 	var is_created = self.__make_row(row, row_count, function ($new_rows) {
			// 		// $new_rows es un array de jquery rows html
			// 		$items = $items.concat($new_rows);
			// 		return true;
			// 	});
			// 	if (is_created) {
			// 		row_count++;
			// 		self.on_added_row(row);
			// 	}
			// });

			if (this.paginator_type === "ALL_ROWS_SERVER_SIDE") {
				this.paginator = $.Paginator.init(
					{
						pages_container: $container,
						data_container: $tbody,
						previous: "",
						next: "",
						show_at_least: paginator_info.show_at_least,
						rows_per_page: paginator_info.rows_per_page,
						rows_number_per_record: 1,
						items: $items,
						row_count: row_count,
						text_format: paginator_info.paged_text_format,
						translated_many: "<%=_T('many')%>",

						callback: function (_, need_data, on_done) {
							if (!need_data) {
								callback();
								return on_done();
							}

							// Esta funcion se ejecuta en un timeout de forma recursiva hasta que el paginador este listo.
							var ready_paginator_fn;

							ready_paginator_fn = function () {
								window.setTimeout(function () {
									if (!self.paginator) {
										ready_paginator_fn();
										return;
									}
									var current_page_number = self.paginator.get_current_page_number();
									current_page_number++;

									self.load_items(current_page_number, function (success, list) {
										var cloned_list;
										if ($.isArray(list)) {
											cloned_list = inConcert.Utils.cloneArray(list);
										}
										else {
											let order_information = order_info || {};
											cloned_list = inConcert.Utils.hash_to_sorted_array(
												list || {},
												order_information.sort_value_fn,
												order_information.filter,
												order_information.options
											);
										}

										if (!success || !cloned_list || cloned_list.length === 0) {
											self.paginator.no_more_pages();
											self.__on_loaded_items(function () {
												callback();
												on_done();
											});
											return;
										}

										$.each(cloned_list, function (_, row) {
											var is_created = self.__make_row(row, row_count, function ($list) { return self.__append_row($list); });
											if (is_created) {
												row_count++;
											}
										});

										self.__on_loaded_items(function () {
											callback();
											on_done();
										});
									});
								});
							};

							ready_paginator_fn();
						}
					});
			}
			else { // ALL_ROWS_CLIENT_SIDE
				this.paginator = $.Paginator.init({
					pages_container: $container,
					data_container: $tbody,
					previous: "",
					next: "",
					show_at_least: paginator_info.show_at_least,
					rows_per_page: paginator_info.rows_per_page,
					items: $items,
					text_format: paginator_info.paged_text_format,
					translated_many: "<%=_T('many')%>",
					on_changed_page: $.noop
				});

				// habilita o inhabilita el boton delete all
				this.__enable_disable_delete_all();
				this.__toggle_checkbox_select_all_rows();
				this.paginator.no_more_pages();
				this.__on_loaded_items(function () {
					callback();
					//				on_done();
				});
				//callback();
			}
		}
	};

	conf_form_paginator.prototype.get_paginator_info = function()
	{
		return this.paginator_info;
	};

	conf_form_paginator.prototype.set_footer_total_rows = function(total_rows)
	{
		this.paginator.set_total_rows(total_rows);
	};

	conf_form_paginator.prototype.update_footer_total_text = function()
	{
		this.paginator.update_footer_total_text();
	};

	conf_form_paginator.prototype.__show_or_hide_loading_msg = function(must_show)
	{
		this.callbacks.show_or_hide_loading_msg(must_show, this.loading_waiting_msg);
	};

	conf_form_paginator.prototype.__$get_header_columns = function(columns)
	{
		const self = this;

		if ( !$.isArray( columns ) || columns.length === 0 ) {
			return "";
		}

		// "<tr></tr>" o "<div></div>"
		const markup = this.get_markup_definition_by_type();
		let columns_markup = FORMAT("<{0}></{0}>", markup.head.row);
		let $columns = $(columns_markup);

		for (let key in columns) {
			if (columns.hasOwnProperty(key)) {
				let column = columns[key];
				if ( column.colspan ) {
					break;
				}
				let column_name = column.name || "";
				let class_name;
				if ( column.system_action || column.custom_action ) {
					class_name = (column.class_name || "gridColumnActions");
				}
				else {
					class_name = (column.class_name || "");
				}
				let header_column_html;
				let col_type = column.type || "";
				if (col_type === "row_selector") {
					self.m_has_row_selector = true;
					header_column_html = "<{0}>" + "<input class='formCheckbox' type='checkbox' header_column_type='" + col_type + "' />" + "</{0}>";
				}
				else {
					header_column_html = "<{0}" + " class='" + class_name + "'><span header_column_type='" + col_type + "'>" + column_name + "</span>" + "</{0}>";
				}
				// reemplazo los {0} por th o div
				header_column_html = FORMAT(header_column_html, markup.head.column);
				$columns.append($(header_column_html));
			}
		}

		return $columns;
	};

	conf_form_paginator.prototype.__$get_rows_html = function(item, row_count)
	{
		if (item === undefined) {
			return;
		}
		let columns = this.columns;

		if (!$.isArray(columns) || columns.length === 0) {
			return;
		}

		let attr_id = "id";
		if (this.__has_row_id()) {
			attr_id = this.row_id;
		}
		let row_id = item[attr_id];
		row_id = row_id || FORMAT("{0}", inConcert.Utils.generateGuid());
		let escaped_row_id = this.escape(row_id);

		let display_name = this.__get_display_name(item);

		// "<tr></tr>" o "<div></div>"
		const markup = this.get_markup_definition_by_type();
		let row_markup = FORMAT("<{0}></{0}>", markup.body.row);
		let $row = $(row_markup);

		$row.attr("row_id", escaped_row_id);
		$row.attr("row_name", display_name.toLowerCase());
		$row.attr("row_type", "main");
		$row.addClass(this.__get_row_classes(item, $row));

		let $rows = [];
		$rows.push($row);

		let $main_row = $rows[0];
		this.__$fill_row_columns_content($main_row, item, columns, row_count);

		return $rows;
	};

	conf_form_paginator.prototype.__get_row_classes = function(/*item, $row*/)
	{
		return "";
	};

	conf_form_paginator.prototype.__create_header_columns = function(columns)
	{
		const $columns = this.__$get_header_columns(columns);
		const $thead = this.$thead();
		$thead.html($columns);
		if (this.header_hidden) {
			$thead.hide();
		}
	};

	conf_form_paginator.prototype.sanitize_value = function(value)
	{
		if (!this.frm || !this.frm.sanitize_value) {
			return value;
		}
		return this.frm.sanitize_value(value);
	};

	conf_form_paginator.prototype.escape = function(value)
	{
		if (!this.frm || !this.frm.escape) {
			return value;
		}
		return this.frm.escape(value);
	};

	conf_form_paginator.prototype.show_rows_only_with_errors = function(only_errors)
	{
		const $rows = this.$get_rows();
		if ($rows.length === 0) {
			return;
		}
		$.each($rows, function(_, row) {
			const $row = $(row);
			if (only_errors) {
				let with_error = $row.attr("with_error");
				if (with_error === "true") {
					$row.show();
				}
				else {
					$row.hide();
				}
			}
			else {
				$row.show();
			}
		});
	};

	conf_form_paginator.prototype.mark_row_error = function(row_info)
	{
		if (!row_info) {
			return;
		}
		const $row =  this.$find_row(row_info.row_id);
		if (!$row || $row.length === 0) {
			return;
		}
		const classes_to_rmv = this.m_rowsErrorMarkClasses.join(" ");
		const msg_type = row_info.msgType || "Error";

		$row.removeClass(classes_to_rmv)
			.addClass(msg_type)
			.attr("with_error", "true")
			.show();

		const $error_mark = $row.find("[column_type='row_error_mark']");
		if ($error_mark.length > 0) {
			$error_mark.removeClass(classes_to_rmv)
				.addClass(msg_type)
				.show();
		}

		const $error_msg = $row.find("[column_type='row_error_msg']");
		if ($error_msg.length > 0) {
			$error_msg.removeClass(classes_to_rmv)
				.addClass(msg_type)
				.show();
			const $error_msg_container = $error_msg.find("[error_msg_container='true']");
			if ($error_msg_container.length > 0) {
				$error_msg_container.text(row_info.msg_err || "Error");
			}
		}
	};

	conf_form_paginator.prototype.hide_all_mark_rows_errors = function()
	{
		const $rows = this.$get_rows();
		if ($rows.length === 0) {
			return;
		}
		const classes_to_rmv = this.m_rowsErrorMarkClasses.join(" ");
		$rows.removeClass(classes_to_rmv);

		$.each($rows, function(_, row) {
			const $row = $(row);
			$row.removeAttr("with_error");
			const $error_mark = $row.find("[column_type='row_error_mark']");
			if ($error_mark.length > 0) {
				$error_mark.removeClass(classes_to_rmv).hide();
			}
			const $error_msg = $row.find("[column_type='row_error_msg']");
			if ($error_msg.length > 0) {
				$error_msg.removeClass(classes_to_rmv)
					.hide();
				const $error_msg_container = $error_msg.find("[error_msg_container='true']");
				if ($error_msg_container.length > 0) {
					$error_msg_container.text("");
				}
			}
		});
	};

	// esta funcion es la encargada de cargar los TD de los rows con sus datos.
	conf_form_paginator.prototype.__$fill_row_columns_content = function($row, item, columns, row_count)
	{
		const self = this;

		let search_text = "";
		let is_readonly_frm = this.frm.is_readonly();
		let disabled_attr = "disabled='true'";

		const make_html_header_lables_inside_fields = function(column) {
			if (!self.header_lables_inside_fields) {
				return "";
			}
			if (!column || !column.name) {
				return "";
			}
			return FORMAT("<label cell_lbl='true'>{0}</label>", column.name);
		};

		const get_cell_class_name = function(column) {
			if (!column || !column.cell_class_name) {
				return "";
			}
			return column.cell_class_name;
		};

		// La definicion del markup dependiendo de si usa <TABLE..> o <DIV..> para la creacion de la grilla.
		const markup = this.get_markup_definition_by_type();
		let $html_col_grouped = null;

		const is_checked_system_fn = function(column) {
			return function(row) {
				if (!row || !column.id || !row.hasOwnProperty(column.id)) {
					return false;
				}
				let check_val = row[column.id];
				switch (typeof(check_val)) {
					case "boolean":
						return check_val;
					case "string":
						return parseInt(check_val, 10) === 1 ? true: false;
					case "number":
						return check_val === 1 ? true: false;
				}
				return false;
			};
		};

		for (let key in columns) {
			if (columns.hasOwnProperty(key)) {
				let column = columns[key];
				let $html_col = null;
				let column_type = column.type || "";
				let cell_class_name = get_cell_class_name(column);
				if (column_type === "counter" && typeof row_count === "number") {
					self.has_counter_column = true;
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" + row_count + "</{0}>",
						markup.body.column
					));
					item.__number = row_count;
				}
				else if (column.type === "row_error_mark") {
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "' style='display:none;'>" +
							"<i icon_type='true'></i>" +
						"</{0}>",
						markup.body.column
					));
				}
				else if (column.type === "row_error_msg") {
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "' style='display:none;'>" +
							"<div error_msg_container='true'></div>" +
						"</{0}>",
						markup.body.column
					));
				}
				else if (column_type === "row_selector" || column_type === "cell_checkbox") {
					let is_checked_fn = column.is_checked || is_checked_system_fn(column);
					let checked_attr = is_checked_fn(item) ? "checked" : "";
					let html_header_inside = make_html_header_lables_inside_fields(column);
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" +
							"<input ctrl_type='checkbox' class='formCheckbox' type='checkbox' " + checked_attr + " " + (is_readonly_frm ? disabled_attr : "") + " />" +
							html_header_inside +
						"</{0}>",
						markup.body.column
					));
				}
				else if (column_type === "cell_input") {
					let value;
					if (column.get_value) {
						let input_value = column.get_value(item);
						value = input_value ? input_value.value : "";
					}
					else {
						value = typeof item[column.id] !== "undefined" ? item[column.id] : "";
					}
					let disable = "";
					if (is_readonly_frm) {
						disable = disabled_attr;
					}
					else {
						const is_disable_editable_cell = (column.is_disable_editable_cell || $.noop)(item);
						if (is_disable_editable_cell) {
							disable = disabled_attr;
						}
					}
					let html_header_inside = make_html_header_lables_inside_fields(column);
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" +
							html_header_inside +
							"<input ctrl_type='textbox' class='formListField form-control' type='text' value='" + value + "' " + disable + " />" +
						"</{0}>",
						markup.body.column
					));
				}
				else if (column_type === "cell_combo") {
					let value, combo_info;
					if (column.get_value) {
						let input_value = column.get_value(item);
						if (input_value) {
							value = input_value.value;
							combo_info = input_value.combo_info;
						}
						value = value || "";
					}
					else {
						value = typeof item[column.id] !== "undefined" ? item[column.id] : "";
					}
					let disable = "";
					if (is_readonly_frm) {
						disable = disabled_attr;
					}
					else {
						const is_disable_editable_cell = (column.is_disable_editable_cell || $.noop)(item);
						if (is_disable_editable_cell) {
							disable = disabled_attr;
						}
					}
					let html_header_inside = make_html_header_lables_inside_fields(column);
					$html_col = $(FORMAT(
						"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" +
							html_header_inside +
							"<select ctrl_type='combo' class='form-control' " + disable + "></select>" +
						"</{0}>",
						markup.body.column
					));
					if (combo_info) {
						const $cbo = $html_col.find("[ctrl_type='combo']");
						self.frm.fill_combo($cbo,
							combo_info.is_blank_first_item,
							combo_info.list,
							combo_info.field_id,
							combo_info.field_name,
							combo_info.sorted,
							null, //can_add_item
							null, //data_id
							null, // id_is_case_sensitive
							combo_info.selected_value
						);
					}
				}
				else if (column.system_action || column.custom_action) {
					$html_col = $(self.__get_row_action_html(item, column));
				}
				else {
					let original_value, cell_value, class_name, column_class_name, search_value, image_src;

					// get value
					if (typeof column.get_value === "function") {
					// lo obtiene de una funcion
						let response = column.get_value(item);
						original_value = response.original_value;
						search_value = response.search_value;
						cell_value = response.value || "";
						class_name = response.class_name || "";
						column_class_name = response.column_class_name !== undefined ? response.column_class_name : "";
						if (column_type === "image") {
							image_src = response.src || "";
						}
					}
					else {
					// lo obtiene directo de los datos
						original_value = typeof item[column.id] !== "undefined" ? item[column.id] : "";
						cell_value = original_value;
						search_value = original_value;
					}

					// Sanitizo el valor a visualizar en la grilla (elimino markup html, etc)
					let tooltip = cell_value;
					let must_sanitize_value = column.sanitize === false ? false : true;
					if (must_sanitize_value) {
						cell_value = self.sanitize_value(cell_value);
					}
					search_value = self.sanitize_value(search_value); // la busqueda siempre se tiene que sanitizar

					// El tooltip
					let tooltip_attr = "";
					if (!column.without_tooltip) {
						// get_tooltip value
						if (typeof column.get_tooltip === "function" ) {
						// lo obtiene de una funcion
							tooltip = column.get_tooltip(item);
						}
						tooltip = inConcert.Utils.htmlBodyToPlainText(tooltip);
						tooltip_attr = FORMAT(" title='{0}' ", tooltip);
					}

					switch (column_type) {
						case "icon":
							let icon_cell_value = "<i class='" + class_name + "'></i>";
							if (column.image_type === "image_with_text") {
								cell_value = icon_cell_value + cell_value;
							}
							else {
								cell_value = icon_cell_value;
							}
						break;

						case "image":
							cell_value = FORMAT("<img src='{0}' class='{1}' />", image_src, class_name);
							// cell_value = "<audio controls preload='none'>" +
							// 				"<source src='" + image_src + "' type='audio/wav' />" + //'" type="" + self.contentType + "" />" +
							// 			"</audio>";
						break;

						case "boolean":
							if ( cell_value === true || parseInt( cell_value, 10 ) === 1 ) {
								class_name = "fa fa-check-circle";
							}
							else {
								class_name = "fa fa-times-circle";
							}
							cell_value = "<i class='" + class_name + "'></i>";
						break;

						case "date":
							let date_config = column.date_config || {};
							let date_object = inConcert.Utils.getDateFromString(cell_value);
							if (date_object instanceof Date) {
								cell_value = inConcert.Utils.getStringFromDate(date_object, date_config.utc, date_config.type_date, date_config.format_date, date_config.separator_date);
							}
						break;

						case "timestampSec":
							if (!isNaN(Number(cell_value))) {
								let time_ms = Number(cell_value) * 1000;
								cell_value = inConcert.Utils.formatDate(time_ms, "date-short-year-short-hour");
							}
						break;

						default:
							if (column_type !== "") {
								throw("conf_form_paginator.__$fill_row_columns_content - Unrecognized column type " + column_type);
							}
					}

					if (column_class_name) {
						cell_class_name = cell_class_name + (cell_class_name ? " " : "") + column_class_name;
					}

					$html_col = $(FORMAT(
						"<{0} " + tooltip_attr + " column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'></{0}>",
						markup.body.column
					));

					original_value = original_value || "";
					if ( typeof column.get_tooltip === "function" ) {
						let tooltip = column.get_tooltip(item);
						tooltip = inConcert.Utils.htmlBodyToPlainText(tooltip);
						$html_col.attr("title", tooltip);
					}

					let html_header_inside = make_html_header_lables_inside_fields(column);
					if (html_header_inside) {
						cell_value = html_header_inside + cell_value;
					}

					$html_col.html(cell_value);
					$html_col.data( "original_value", original_value );

					if (!column.no_search && search_value) {
						search_text = search_text + search_value;
					}
				}
				// Agrego custom atributos a la celda.
				if (typeof column.add_customs_attrs === "function") {
					column.add_customs_attrs($html_col, column, item);
				}

				if ($html_col) {
					if (column.grouper) {
						if (column.grouper.start) {
							let class_name = column.grouper.class_name || "";
							let col_grouped_name = column.grouper.name || "true";
							$html_col_grouped = $(FORMAT("<div col_grouped='{0}' class='{1}'></div>", col_grouped_name, class_name));
							$html_col_grouped.append($html_col);
						}
						else if (column.grouper.middle) {
							$html_col_grouped.append($html_col);
						}
						else if (column.grouper.end) {
							$html_col_grouped.append($html_col);
							$row.append($html_col_grouped);
							$html_col_grouped = null;
						}
						else {
							if (!column.grouper.start && !column.grouper.end && !column.grouper.middle) {
								throw("conf_form_paginator.prototype.__$fill_row_columns_content - column.grouper.start or column.grouper.end column.grouper.middle is required");
							}
						}
					}
					else {
						if ($html_col_grouped) {
							$html_col_grouped.append($html_col);
						}
						else {
							$row.append($html_col);
						}
					}
				}
			}
		}

		if ($html_col_grouped) {
			throw("conf_form_paginator.prototype.__$fill_row_columns_content - Missing html end gruper");
		}

		//search_text = this.escape(search_text);
		$row.attr("search_text", search_text);
		this.__update_row_data($row, item);

		return $row;
	};

	conf_form_paginator.prototype.__update_row_data = function($current_row, entity)
	{
		let current_entity;
		if (this.no_merge_row_data) {
			current_entity = entity;
		}
		else {
			current_entity = $current_row.data("row_data") || {};
			current_entity = $.extend(current_entity, entity);
		}
		$current_row.data("row_data", current_entity);
		return current_entity;
	};

	conf_form_paginator.prototype.get_items = function(must_return_a_map, field_id)
	{
		field_id = field_id || this.row_id;
		let $rows = this.$get_rows();
		let items = [];
		$.each($rows, function(_, row) {
			let $row = $(row);
			let item = $row.data("row_data");
			let cloned_item = inConcert.Utils.cloneObject(item);
			delete cloned_item.__number;
			items.push(cloned_item);
		});
		if (!must_return_a_map || !field_id) {
			return items;
		}
		let map_items = inConcert.Utils.arrayToMap(items || [],
			function(row) {
				return row;
			},
			function(row) {
				// el ID
				return (row[field_id] || "").toLowerCase();
			}
		);
		return map_items;
	};

	conf_form_paginator.prototype.unmark_all_rows = function()
	{
		var $rows = this.$get_rows();
		$rows.removeClass("selected");
	};

	conf_form_paginator.prototype.mark_row = function($row)
	{
		$row.addClass("selected");
	};

	conf_form_paginator.prototype.show_actions_rows = function()
	{
		const $rows = this.$get_rows();
		$rows.find("[column_type='actions'] > [action]").show();
	};

	conf_form_paginator.prototype.hide_actions_rows = function()
	{
		const $rows = this.$get_rows();
		if ($rows) {
			$rows.find("[column_type='actions'] > [action]").hide();	
		}
	};

	conf_form_paginator.prototype.unmark_row = function($row)
	{
		$row.removeClass("selected");
	};

	conf_form_paginator.prototype.get_selected_items = function()
	{
		let $rows = this.$get_rows();
		let selected_items = [];
		$.each($rows, function(_, row)
		{
			let $row = $(row);
			let $row_selector = $row.find("[column_type='row_selector']");
			let $checkbox = $row_selector.find("input[type='checkbox']");
			if ($checkbox.prop("checked")) {
				let item = $row.data("row_data");
				let cloned_item = inConcert.Utils.cloneObject(item);
				selected_items.push(cloned_item);
			}
		});
		return selected_items;
	};

	conf_form_paginator.prototype.get_header_search_html = function()
	{
		let value = this.search_text || "";
		return "".concat(
			"<div id='", this.id_search_container, "' class='pageViewFilter frmPaginatorFilter'>",
				"<input id='", this.id_search_text, "' class='filterField' type='text' placeholder='<%=_T([[Search]])%> ...' value='", value, "' />",
				"<button id='", this.id_search_reset, "' class='buttonToggle reset' type='button' style='display:none;'>",
					"<i class='far fa-times'></i>",
				"</button>",
				"<button id='", this.id_search_apply, "' class='buttonToggle buttonToggleOutlined search' type='button'>",
					"<i class='far fa-search'></i>",
				"</button>",
			"</div>");
	};

	conf_form_paginator.prototype.on_header_created = function()
	{
		const $search = $("#" + this.id_search_apply);
		const $reset = $("#" + this.id_search_reset);
		const $text = $("#" + this.id_search_text);
		$search.show();
		$reset.hide();
		$text.val("").focus();
	};

	/*
	 * Esta funcion genera el markup necesario para las acciones de cada row de la grilla
	 */
	conf_form_paginator.prototype.__get_row_action_html = function(row, column)
	{
		const self = this;

		if (!row || !column) {
			return "";
		}
		const get_button_html = function( action_id, label, is_enabled, additional_class, is_system, tooltip, action_btn_separator)
		{
			/*
			 * Valores del parametro is_enabled:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if (is_enabled === -1) {
				return "";
			}
			let disabled_attr = (is_enabled === 1 ? "" : "disabled='true'");
			let button_classes = "buttonColorGray gridActionButton";
			let i_classes = additional_class;
			let action_type = is_system ? "system" : "custom";
			tooltip = tooltip || "";

			if (self.grid_actions && self.grid_actions.without_label === true) {
				label = "";
			}
			let button_dom_id = inConcert.Utils.generateGuid();
			let button_html =
				"<button id='" + button_dom_id + "' type='button' " + disabled_attr + " action='" + action_id + "' action_type='" + action_type + "' title='" + tooltip + "' class='" + button_classes + "'>" +
					"<i class='" + i_classes + "'></i>" +
					label +
				"</button>";

				if (action_btn_separator) {
					button_html = button_html + "<span class='action-buttons-separator'></span>";
				}

			return button_html;
		};

		var is_enabled_fn = column.is_enabled || function() {return 1;/*Enable x defecto*/};

		var must_show_button = function(info) {
			if (typeof(info) !== "object") {
				return true;
			}
			if (typeof(info.must_show) !== "function") {
				return true;
			}
			return info.must_show(row);
		};

		let button_class_name, caption_button, tooltip;
		let html_edit_button = "";
		let html_delete_button = "";
		let is_readonly_frm = this.frm.is_readonly();

		if (column.system_action) {
			// edit button
			if (column.system_action.edit) {
				if (must_show_button(column.system_action.edit)) {
					if (is_readonly_frm) {
						caption_button = "<%=_T('View')%>";
						tooltip = caption_button;
						button_class_name = "buttons-icons-view";
					}
					else {
						caption_button = "<%=_T('Edit')%>";
						tooltip = caption_button;
						button_class_name = "buttons-icons-edit";
					}
					if (typeof(column.system_action.edit) === "object") {
						caption_button = column.system_action.edit.label || caption_button;
						tooltip = column.system_action.edit.tooltip || caption_button;
						button_class_name = column.system_action.edit.class_name || button_class_name;
					}
					var is_enabled_edit = is_enabled_fn("EDIT", row);
					html_edit_button = get_button_html( "edit", caption_button, is_enabled_edit, button_class_name, true, tooltip );
				}
			}

			// delete button
			if (column.system_action.erase) {
				if (!is_readonly_frm && must_show_button(column.system_action.erase)) {
					caption_button = "<%=_T('Delete')%>";
					tooltip = caption_button;
					button_class_name = "buttons-icons-delete";
					if (typeof(column.system_action.erase) === "object") {
						caption_button = column.system_action.erase.label || caption_button;
						tooltip = column.system_action.erase.tooltip || caption_button;
						button_class_name = column.system_action.erase.class_name || button_class_name;
					}
					var is_enabled_delete = is_enabled_fn("DELETE", row);
					html_delete_button = get_button_html( "delete", caption_button, is_enabled_delete, button_class_name, true, tooltip );
				}
			}
		}

		// custom buttons
		var html_custom_buttons = "";
		if (typeof column.custom_action === "object") {
			this.grid_custom_action = column.custom_action;
			$.each( column.custom_action, function( custom_action_id, custom_action )
			{
				var is_enabled_custom = is_enabled_fn(custom_action_id, row);
				let name = custom_action.name || "";
				tooltip = custom_action.tooltip || name;
				button_class_name = custom_action.class_name || undefined;
				let action_btn_separator = custom_action.action_separator;
				html_custom_buttons = html_custom_buttons + get_button_html( custom_action_id, name, is_enabled_custom, button_class_name, false, tooltip, action_btn_separator);
			});
		}

		// <td> o <li>
		const markup = this.get_markup_definition_by_type();
		let html_action =
			"<{0} column_id='" + column.id + "' column_type='actions'>" +
				html_custom_buttons +
				"<span class='action-buttons-separator'></span>" +
				html_edit_button    +
				html_delete_button  +
			"</{0}>";

		return FORMAT(html_action, markup.body.column);
	};

	conf_form_paginator.prototype.show_popup_waiting = function(msg)
	{
		const id_popup_waiting = this.__eval_id_by_popup("id_popup_waiting");
		const $waiting = $("#" + id_popup_waiting);
		const $msg = $waiting.find("[waiting_message]");
		$msg.text(msg || "<%=_T('Please wait ...')%>");
		$waiting.show();
	};

	conf_form_paginator.prototype.hide_popup_waiting = function()
	{
		const id_popup_waiting = this.__eval_id_by_popup("id_popup_waiting");
		const $waiting = $("#" + id_popup_waiting);
		$waiting.hide();
	};

	conf_form_paginator.prototype.show_popup_msg = function(msg, type, timeout_ms, response_object)
	{
		const self = this;

		timeout_ms = timeout_ms || false;
		if (this.popup_must_show_messages_in_frm) {
			// 1- visualizo el mensaje en el formulario
			this.show_status_message(msg, type, timeout_ms);
			return;
		}
		// 2- visualizo el mensaje en el popup
		const class_type = type === "Info" ? "messageInfo" : "messageError";
		const id_popup_msg = this.__eval_id_by_popup("id_popup_msg");

		const $msg_container = $("#" + id_popup_msg);
		$msg_container
			.removeClass("messageError messageInfo")
			.addClass(class_type)
			.show();
		const $message = $msg_container.find(".messageText");
		if ($message.length > 0) {
			$message.text(msg || "");
		}
		else {
			$msg_container.text(msg || "");
		}

		this.callbacks.on_show_popup_msg(msg, type, response_object);

		if (typeof(timeout_ms) === "number") {
			window.setTimeout(function() {
				self.hide_popup_msg();
			}, timeout_ms);
		}
	};

	conf_form_paginator.prototype.hide_popup_msg = function()
	{
		const id_popup_msg = this.__eval_id_by_popup("id_popup_msg");
		$("#" + id_popup_msg).hide();
		const id_popup = this.__eval_id_by_popup("id_popup");
		$("#" + id_popup).find(".formFieldValidation").hide();
		this.callbacks.on_hide_popup_msg();
	};

	conf_form_paginator.prototype.disable_actions = function(disable)
	{
		disable = disable === undefined ? true : disable;
		const id_popup_create = this.__eval_id_by_popup("id_popup_create");
		$("#" + id_popup_create).prop("disabled", disable);
		const id_popup_create_and_continue = this.__eval_id_by_popup("id_popup_create_and_continue");
		$("#" + id_popup_create_and_continue).prop("disabled", disable);
	};

	/*
		Deshabilita o Habilita las acciones de TODAS las rows de la grilla
	*/
	conf_form_paginator.prototype.disable_grid_actions = function(disable)
	{
		const self = this;

		disable = disable === undefined ? true : disable;

		/*
			Cuando la accion es habilitar acciones de la grilla, se deben evaluar contra su definición de columnas y estados por row, para saber si la misma puede estar activa.

			Ej: en outbound config, procesos, la funcion que decide si la accion esta habilitada es la funcion "__grd_batches_row_status_actions".

				this.form_paginator_batches = new inConcert.conf_form_paginator(
					....
					columns: [
						..
						{id: "_rigth_actions", type: "actions", name: "",
							system_action: {edit: true, erase: true},
							custom_action: {
								..
							},
							is_enabled: function(action_id, batch) {
								return self.__grd_batches_row_status_actions(action_id, batch);
							}
						}
				)
		
			La validacion de esta funcion es solo valida cuando estamos Edtiando un row.
		*/
		const IsEnabledAction = function(action_id, row_data, action_type = "")
		{
			const $current_row = self.$get_edited_row();
			if (!$current_row) {
				// siempre que se este creando un row se evaluara cuando se inserte la nueva row
				return true;
			}

			// cuando editamos
			let is_enabled = true;
			action_type = action_type.toLowerCase();

			// recorro la definicion de las columanas
			for (let i = 0; i < self.columns.length; i++) {
				let column = self.columns[i];
				if (column.type === "actions" && typeof(column.is_enabled) === "function") {
					let actions;
					if (action_type === "system") {
						if (column.system_action) {
							actions = column.system_action;
						}
					}
					else if (action_type === "custom") {
						if (column.custom_action) {
							actions = column.custom_action;
						}
					}
					else {
						throw("conf_form_paginator.prototype.disable_grid_actions - Unrecognized button action_type " + action_type);
					}

					// busca la accion a evaluar dentro de la definicion de las columnas (system o custom)
					if (actions) {
						for (let column_aciton_id in actions) {
							if (actions.hasOwnProperty(column_aciton_id)) {
								if (column_aciton_id === action_id || (column_aciton_id === "erase" && action_id === "delete")) {
									// si es la misma accion, evaluo contra la definicion.
									is_enabled = column.is_enabled(action_id, row_data);
									break;
								}
							}
						}
					}
				}
			}

			return is_enabled;
		};

		const $rows = this.$get_rows();
		$.each($rows, function(_, row) {
			const $row = $(row);
			const $actions = $row.find("button[action]");	
			$.each($actions, function(_, action) {
				const $action = $(action);
				if (disable) {
					// desactivado
					$action.prop("disabled", true);
				}
				else {
					// activado
					let row_data = $row.data("row_data");
					let action_id = $action.attr("action");
					let action_type = $action.attr("action_type");
					let is_enabled = IsEnabledAction(action_id, row_data, action_type);
					if (is_enabled) {
						$action.prop("disabled", false);
					}
					else {
						$action.prop("disabled", true);
					}
				}
			});
		});
	};

	/*
		Return: "tr[row_id]" or ul[row_id] / tr[row_id='algo'] ...
	*/
	conf_form_paginator.prototype.__jquery_row_selector = function(name, value)
	{
		name = name || "row_id";
		const markup = this.get_markup_definition_by_type();
		if (value) {
			return FORMAT("{0}[{1}='{2}']", markup.body.row, name, value);
		}
		return FORMAT("{0}[{1}]", markup.body.row, name);
	};

	/*
		Return: "td[column_type]" or li[column_type] / td[column_type='algo'] ...
	*/
	conf_form_paginator.prototype.__jquery_column_selector = function(with_value)
	{
		const markup = this.get_markup_definition_by_type();
		if (with_value) {
			return FORMAT("{0}[column_type='{1}']", markup.body.column, with_value);
		}
		return FORMAT("{0}[column_type]", markup.body.column);
	};

	conf_form_paginator.prototype.__sortable = function()
	{
		const self = this;
		const is_readonly_frm = this.frm.is_readonly();
		if (is_readonly_frm) {
			return;
		}
		const paginator_info = this.get_paginator_info();
		if (!paginator_info.sortable) {
			return;
		}
		const $tbody = this.$tbody();
		let selector = this.__jquery_row_selector();
		$tbody.sortable({
			cursor: "move",
			items : selector,
			out: function(/*e, ui*/) {
				self.__update_column_counter();
				(paginator_info.on_sorted || $.noop)();
			}
		});
	};

	conf_form_paginator.prototype.show_disclaimer = function(msg)
	{
		const id_disclaimer = this.__eval_id_by_popup("id_disclaimer");
		const $disclaimer = $("#" + id_disclaimer);
		$disclaimer.text(msg);
		$disclaimer.show();
	};

	conf_form_paginator.prototype.hide_disclaimer = function()
	{
		const id_disclaimer = this.__eval_id_by_popup("id_disclaimer");
		const $disclaimer = $("#" + id_disclaimer);
		$disclaimer.hide();
	};

	conf_form_paginator.prototype.build_grid_markup = function()
	{
		if (!this.id_grid_container) {
			return;
		}
		const $grid_container = $("#" + this.id_grid_container);
		if ($grid_container.length === 0) {
			throw("conf_form_paginator.prototype.build_grid_markup - grid DOM container does not exist.");
		}
		const markup = this.get_markup_definition_by_type();
		if (markup) {
			let info = {
				grid_dom_id: this.id_grid,
				grid_class : "viewGrid",
				thead_class: "viewGridHeader",
				tbody_class: "viewGridBody"
			};
			let html_grid = markup.container(info);
			$grid_container.html(html_grid);
		}
	};

	conf_form_paginator.prototype.initialize = function(callback, list = null)
	{
		const self = this;

		this.build_grid_markup();
		this.__create_builtin_html();
		this.__create_header_columns(this.columns);
		this.fill_rows(callback, list);

		const id_new_item = this.__eval_id_by_popup("id_new_item");
		const $new_item = $("#" + id_new_item);

		const id_disclaimer = this.__eval_id_by_popup("id_disclaimer");
		const $disclaimer = $("#" + id_disclaimer);

		const is_readonly_frm = this.frm.is_readonly();
		if (is_readonly_frm) {
			$new_item.hide();
			$disclaimer.hide();
		}
		else {
			$new_item.show();
			$disclaimer.show();
		}

		if (this.__has_popup()) {
			if (this.use_instance_of_inconcert_popup) {
				this.popup_instance = new inConcert.PopupWindow(
				{
					ownerId: this.id_popup, // container id
					eventName: this.id_popup, // evento click personalizado
					callbacks: {
						onClose: function(){
							self.hide_popup();
						}
					}
				});
			}
		}

		if (this.popup_is_always_visible) {
			this.show_popup();
		}
	};

	conf_form_paginator.prototype.__create_builtin_html = function()
	{
		if (!this.__is_builtin_search()) {
			return;
		}
		const $search_container = $("#" + this.id_search_container);
		if ($search_container.length > 0) {
			return;
		}
		const $search_builtin_panel = $("#" + this.id_search_builtin_panel);
		let html = this.get_header_search_html();
		$search_builtin_panel.html(html);
	};

	conf_form_paginator.prototype.fill_rows = function(callback, list = null)
	{
		const self = this;
		callback = callback || $.noop;

		this.__init_paginator(list, function() {
			self.show_or_hide_not_results_found(function()
			{
				self.__sortable();
				callback(true);
			});
		});
	};

	conf_form_paginator.prototype.editable = function()
	{
		if (this.id_delete_all) {
			const $delete_all = $("#" + this.id_delete_all);
			if ($delete_all.length > 0) {
				$delete_all.show();
			}
		}
	};

	conf_form_paginator.prototype.readonly = function()
	{
		if (this.id_delete_all) {
			const $delete_all = $("#" + this.id_delete_all);
			if ($delete_all.length > 0) {
				$delete_all.hide();
			}
		}
	};

}());
