/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert */
/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	/*jslint validthis:true */
	"use strict";

//	const _T = inConcert.i18n.getString;
//	const FORMAT = inConcert.Utils.format;

	/*
		form popup basic class
	*/
	function conf_form_popup (params) {
		this.owner = params.owner;
		this.container_id = params.container_id;
		this.container_class = params.container_class;
		this.container_popup_middle_class = params.container_popup_middle_class;
		this.popup_id = params.popup_id;
		this.title = params.title || "...";
		this.buttons_actions = params.buttons_actions || {create_and_continue: {allowed: true}};
		this.frm = params.frm;
		this.current_item = null;

		let callbacks = params.callbacks || {};
		this.callbacks = {
			on_close: callbacks.on_close || $.noop,
			on_save: callbacks.on_save || function(callback) {
				callback(true);
			}
		};

		this.init();
	}

	if ( typeof inConcert.conf_form_popup !== "function" ) {
		inConcert.conf_form_popup = conf_form_popup;
	}

	conf_form_popup.prototype.get_html_middle = $.noop;
	conf_form_popup.prototype.ready           = $.noop;
	conf_form_popup.prototype.empty_controls  = $.noop;

	conf_form_popup.prototype.get_data = function() {
		if (this.current_item) {
			let current_item = inConcert.Utils.cloneObject(this.current_item);
			return current_item;
		}
		let id = inConcert.Utils.generateGuid();
		let new_item = {Id: id};
		return new_item;
	};

	conf_form_popup.prototype.validate = function() {
		return true;
	};

	conf_form_popup.prototype.edit = function(item) {
		this.current_item = item;
	};

	conf_form_popup.prototype.load_data = function(callback = $.noop) {
		callback(true);
	};

	conf_form_popup.prototype.init = function() {
		const self = this;
		let $popup_html = this.$get_html_popup();
		$("body").append($popup_html);
		this.set_events();
		$popup_html.show();

		this.show_waiting();
		this.load_data(function() {
			self.ready();
			self.hide_waiting();
		});
	};

	conf_form_popup.prototype.show_waiting = function(msg) {
		const $waiting = $("#" + this.container_id + "_waiting");
		if (msg) {
			$waiting.find("[waiting_message]").text(msg);
		}
		$waiting.show();
	};

	conf_form_popup.prototype.hide_waiting = function() {
		const $waiting = $("#" + this.container_id + "_waiting");
		$waiting.hide();
	};

	conf_form_popup.prototype.dispose = function() {
		this.close_popup();
	};

	conf_form_popup.prototype.set_events = function() {
		const self = this;

		const $body = $("body");
		let event_selector = "[id='" + this.container_id + "_close']";
		$body.off("click", event_selector)
			.on("click", event_selector, function() {
				self.close_popup();
			});

		event_selector = "[id='" + this.container_id + "_close_times']";
		$body.off("click", event_selector)
			.on("click", event_selector, function() {
				self.close_popup();
			});

		event_selector = "[id='" + this.container_id + "_create']";
		$body.off("click", event_selector)
			.on("click", event_selector, function() {
				self.save();
			});

		event_selector = "[id='" + this.container_id + "_create_and_continue']";
		$body.off("click", event_selector)
			.on("click", event_selector, function() {
				self.save(true);
			});
	};

	conf_form_popup.prototype.close_popup = function() {
		const $popup_html = $("#" + this.container_id);
		$popup_html.remove();
		this.callbacks.on_close();
	};

	conf_form_popup.prototype.save = function(create_and_continue) {
		const self = this;

		let data = this.get_data();
		$("#" + this.container_id).find("[mark_error]").hide();
		if (!this.validate()) {
			return false;
		}
		this.show_waiting();
		this.callbacks.on_save(data, create_and_continue, function(success, response) {
				self.hide_waiting();
				if (success) {
					if (create_and_continue) {
						if (response) {
							self.show_status_message(response.msg, response.type || "Info", 5000);
							self.empty_controls();
						}
					}
					else {
						self.close_popup();
					}
				}
				else {
					if (response) {
						self.show_status_message(response.msg, "Error", 5000);
					}
				}
			}
		);
	};

	conf_form_popup.prototype.show_status_message = function(message, type, timeout_ms) {
		const $custom_message_container = $("#" + this.container_id + "_message");
		this.frm.show_status_message(message, type, timeout_ms, undefined, undefined, $custom_message_container);
	};

	conf_form_popup.prototype.$get_html_popup = function() {
		let ba = this.buttons_actions;

		let html_create_and_continue = "";
		if (ba.create_and_continue && ba.create_and_continue.allowed) {
			html_create_and_continue = "<button id='" + this.container_id + "_create_and_continue'  class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Confirm & continue')%>" + "</button>";							
		}

		let html = "" +
			"<div id='" + this.container_id + "' class='internalPopUpContainer "+ this.container_class +"' style='display:none; z-index: 119 !important;'>" +
				"<div class='internalPopUpOverlay'></div>" +
				"<div class='internalPopUp'>" +
					// waiting
					"<div id='" + this.container_id + "_waiting' class='internalWaitingContainer popupWaitingContainer' style='display:none;'>" +
						"<div waiting_message='true' class='internalWaitingMessage'>" + "<%=_T('Please wait.')%>" + "</div>" +
						"<div class='internalWaitingAnimation'></div>" +
						"<div class='internalWaitingOverlay'></div>" +
					"</div>" +
					// title
					"<div class='internalPopUpTitle'>" + 
						"<span id='" + this.container_id + "_title'>" + this.title + "</span>" + 
						"<button id='" + this.container_id + "_close_times' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>"+
					"</div>" +
					"<div class='internalPopUpMiddle "+ this.container_popup_middle_class +"'>" +
						"<div id='" + this.container_id + "_message' class='genericFormMessagesContainer messageError' style='display:none;'>" +
							"<label class='messageText'></label>" +
						"</div>" +
						this.get_html_middle() +
					"</div>" +
					// MODAL FOOTER BUTTONS
					'<div class="internalPopUpButtons">' +
						"<button id='" + this.container_id + "_create'  class='pageButtonGeneric buttonColorBlue float-right'>" + (ba.create ? ba.create.caption : "<%=_T('Confirm')%>") + "</button>" +
						html_create_and_continue +
						"<button id='" + this.container_id + "_close'  class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>" +
					'</div>' +
				"</div>" +
			"</div>";
		return $(html);
	};

	/*
		Funcion muestra la cruz de error el los campos que han fallado.
	*/
	conf_form_popup.prototype.display_field_error = function($field)
	{
		const validation_id = $field.attr("id") + "Validation";
		const $field_validation = $("#" + validation_id);
		if ($field_validation) {
			$field_validation.show();
		}
		$field.focus();
	};
}());
