module(..., package.seeall, inconcert.orbit.application.Decorate)

local PM = require("provisioningmanager")
local utils = require "shared.apps.utils"
local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", function()
	return require ("ssl.https")
end)
local ltn12 = require("ltn12")


m_htmlRender = nil
m_appLocation = nil

function GetSystemConfiguration (elementId)
	assert(elementId)
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, systemConfiguration = pcall(PM.GetSystemConfiguration, ksProv, elementId)
	ksProv:Close()
	if not success then
		local errMsg = systemConfiguration
		LogWarning("[provisioning:models.vccs.GetSystemConfiguration] Error loading System Configuration for element '%s'.", elementId, errMsg)
		return false
	end
	return true, systemConfiguration
end


local function List (web, typeFlow, category)
	local success, info = GetSystemConfiguration("API_INTEGRATIONS_ICC")
	if not success then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Obtaining system configuration")
		web.status = 500
		return false
	end
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Obtaining system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(info))
	local encodedInfo = info.APIIntegrationsDefaultConfiguration
	if type(encodedInfo) ~= "string" then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Decoding system configuration")
		web.status = 500
		return false
	end
	local decodedInfo = json.decode(encodedInfo)
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Decoding system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(infoTable))
	
	local baseUri = decodedInfo.Templates.base ..[[/templates/occ/list/%s/%s/%s?lang=%s]]
	local verb = "GET"
	local apikey = decodedInfo.Templates.apikey
	
	--Obtenemos la versión de inconcert ejecutando sobre el SO y buscando el match
	local handle = io.popen("dpkg -s inconcert | grep '^Version:'")
	local version = handle:read("*a"):match "%d+.%d+.%d+"
	handle:close()

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Listing for version %s and type %s, user %s, vcc %s", version, typeFlow, userId, vcc)
	
	local uri = string.format(baseUri,version,typeFlow,category,web.GET.lang)
	
	if web.GET.page then
		uri = uri ..string.format("&page=%s", web.GET.page)
	end	

	if web.GET.sort then
		uri = uri ..string.format("&sort=%s", web.GET.sort)
	end	

	if web.GET.filter then
		uri = uri ..string.format("&filter=%s", web.GET.filter)
	end	


	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)
	
	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function ListCategories (web, typeFlow)
	local success, info = GetSystemConfiguration("API_INTEGRATIONS_ICC")
	if not success then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Obtaining system configuration")
		web.status = 500
		return false
	end
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Obtaining system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(info))
	local encodedInfo = info.APIIntegrationsDefaultConfiguration
	if type(encodedInfo) ~= "string" then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Decoding system configuration")
		web.status = 500
		return false
	end
	local decodedInfo = json.decode(encodedInfo)
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Decoding system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(infoTable))
	
	local baseUri = decodedInfo.Templates.base ..[[/templates/occ/categories/%s/%s?lang=%s]]
	local verb = "GET"
	local apikey = decodedInfo.Templates.apikey
	
	--Obtenemos la versión de inconcert ejecutando sobre el SO y buscando el match
	local handle = io.popen("dpkg -s inconcert | grep '^Version:'")
	local version = handle:read("*a"):match "%d+.%d+.%d+"
	handle:close()

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Listing for version %s and type %s, user %s, vcc %s", version, typeFlow, userId, vcc)
	
	local uri = string.format(baseUri,version,typeFlow,web.GET.lang)
	
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)
	
	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:LIST] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function download (web, idPackage)
	local success, info = GetSystemConfiguration("API_INTEGRATIONS_ICC")
	if not success then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Obtaining system configuration")
		web.status = 500
		return false
	end
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Obtaining system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(info))
	local encodedInfo = info.APIIntegrationsDefaultConfiguration
	if type(encodedInfo) ~= "string" then
		LogError("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Decoding system configuration")
		web.status = 500
		return false
	end
	local decodedInfo = json.decode(encodedInfo)
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Decoding system configuration %s, with status %s, and value %s", "API_INTEGRATIONS_ICC", success, json.encode(infoTable))
	

	local baseUri = decodedInfo.Templates.base ..[[/templates/occ/%s]]
	local verb = "GET"
	local apikey = decodedInfo.Templates.apikey

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Download package %s, user %s, vcc %s", idPackage, userId, vcc)
	
	local uri = string.format(baseUri,idPackage)
	
	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)
	
	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW_TEMPLATES:DOWNLOAD] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end



-- local function ImportProcessFlow (web)
-- 	local vcc = inconcert.authUser.virtualContactCenter
-- 	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
-- 	LogInfo("[FLOW_DESIGNER.PROCESS_FLOW:ImportProcessFlow] Upload Flows to be imported, userId:  %s", userId)
-- 	--LogDebug("[FLOW_DESIGNER.PROCESS_FLOW:ImportProcessFlow] Upload Flows to be imported, post data:  %s", web.input["post_data"])
-- 	local status, result = ValidateFlowToBeImported(web.request)
-- 	--LogDebug("[FLOW_DESIGNER.PROCESS_FLOW:ImportProcessFlow] Uploaded Flows to be imported, status: %s, result: %s", status, json.encode(result))
-- 	web.status = status
-- 	return result
-- end

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_get (utils.JsonReply ..LogRequest ..List, prefix.."/list/(.+)/(.+)")
	module:dispatch_get (utils.JsonReply ..LogRequest ..ListCategories, prefix.."/categories/(.+)")
	module:dispatch_get (utils.JsonReply ..LogRequest ..download, prefix.."/download/(.+)")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
