module(..., package.seeall)
local utils = require "shared.apps.utils"
local agent_model = require("apps.agent.model"):GetModels()
local m_datasource

function GetName ()
	return "ProcessFlowClasses"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetFlowClassess (vcc, classId, keyspace)
	local classes = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClasses",
		superColumns = true,
		key = vcc,
		columns = classId and {classId} or {}
	}
	classes = classes or {}
	if classId then
		local classData = classes[classId] or nil
		if not classData and vcc ~= "system" then
			classData = GetFlowClassess("system", classId, keyspace)
		end
		if classData and vcc == "system" then
			classData.System = true
		end
		return classData
	else
		if vcc == "system" then
			for _, class in pairs(classes) do
				class.System = true
			end
		else
			local systemClasses = GetFlowClassess ("system", classId, keyspace)
			for id, class in pairs(systemClasses) do
				classes[id] = class
			end
		end
		return classes
	end
end

function GetFlowClassTypes (vcc, typeId, keyspace)
	local types = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClassTypes",
		superColumns = true,
		key = vcc,
		columns = typeId and {typeId} or {}
	}
	if typeId then
		return types and types[typeId] or {}
	else
		return types or {}
	end
end

function SaveFlowClassess (vcc, classes, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClasses",
		superColumns = true,
		rows = {
			[vcc] = classes
		}
	}
end

function SaveFlowClassTypes (vcc, types, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClassTypes",
		superColumns = true,
		rows = {
			[vcc] = types
		}
	}
end

function RemoveFlowClassess (vcc, classes_to_remove, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClasses",
		superColumns = true,
		rows = {
			[vcc] = classes_to_remove
		}
	}
end

function RemoveFlowClassTypes (vcc, types_to_remove, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClassTypes",
		superColumns = true,
		rows = {
			[vcc] = types_to_remove
		}
	}
end

function LoadFlowClassesByType (vcc, typeId, keyspace)
	assert(vcc)
	assert(typeId)

	local classes = utils.GetAllColumns({
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlowClasses",
		superColumns = true,
		key = vcc,
		columnCount = 5000,
		columns = {}
	}) or {}

	local classesByType = {}
	for _, flowClass in pairs(classes) do
		if flowClass.Type == typeId then
			table.insert(classesByType, flowClass)
		end
	end

	return classesByType
end

