local cjson = require "cjson"

local function makeFields (fields, aux)
	local code = "{"
	for _, field in pairs(fields) do
		if field.type == "array" then
			--2 posibilidades: 
			--	el field.value es el nombre de una variable, que a su vez es un array de objetos
			--		-> Pasamos como value el nombre de la variable
			--	el field.value es directamente un json pasado a string
			--		-> Pasamos como value un string encerrado entre comilla simple

			local validJson, fieldValue = pcall(function() return cjson.decode(field.value) end)
			if validJson then		
				field.value = "'" .. field.value .. "'"
			end
		end

		code = code .. string.format("\n\t{ name = '%s', value = %s, type = '%s' },", field.name, field.value, field.type)
	end
	code = code .. "\n}"
	return code
end

local function MakeExecutable (stateName, activityId, data, aux)
	if aux.empty(data.vcc) or aux.empty(data.methodUri) or aux.empty(data.methodType) then
		return aux.invalidParameterActivity(stateName, activityId, "invalid data")
	end
	
	--Default timeout
	if aux.empty(data.timeout) then
		data.timeout = 120
	end

	--run method
	local text = [[
local _fields = %s
local _result = ExecuteApiHelpDesk('%s', '%s', '%s', _fields, %s)
]]
	code = makeFields(data.fields, aux)
	text = string.format(text, code, data.vcc, data.methodUri, data.methodType, data.timeout)
	text = text .. '\nlogTrace("DEBUG", string.format("Processing activity (end) result: %s", cjson.encode(_result)))'
	--process result
	if not aux.empty(data.result) then
		text = text .. string.format('\n\n%s = _result', data.result)
	end
	return 200, text
end

return {
	MakeExecutable = MakeExecutable
}
