local cjson = require "cjson"

local TIMEOUT_MIN = 5
local TIMEOUT_MAX = 1800

local RETRIES_MIN = 1
local RETRIES_MAX = 10

local function MakeExecutable (stateName, activityId, data, aux)
	if aux.empty(data.message) or aux.empty(data.timeoutRetry) or aux.empty(data.retries) then
		return aux.invalidParameterActivity(stateName, activityId, "invalid data")
	end
	--validate limits client in code
	local timeout_n = tonumber(data.timeoutRetry)
	if timeout_n and (timeout_n < TIMEOUT_MIN or timeout_n > TIMEOUT_MAX) then
		return aux.invalidParameterActivity(stateName, activityId, "invalid timeout")
	end
	local retries_n = tonumber(data.retries)
	if retries_n and (retries_n < RETRIES_MIN or retries_n > RETRIES_MAX) then
		return aux.invalidParameterActivity(stateName, activityId, "invalid retries")
	end
	local text = ""
	--process message
	local message = data.message
	if type(message) == "table" then
		message = cjson.encode(cjson.encode(message))
		text = text .. string.format('local _message = cjson.decode(%s)', message)
		if data.message.optionsVariable then
			text = text .. '\n_message.optionsVariable = nil'
			text = text .. '\n_message.optionsTemplate = nil'
			text = text .. '\n_message.options = {}'
			text = text .. string.format('\nlocal _options = %s', data.message.optionsVariable)
			text = text .. '\nfor _, _option in pairs(_options or {}) do'
				text = text .. string.format('\nlocal _element = cjson.decode(%s)\t', cjson.encode(cjson.encode(data.message.optionsTemplate)))
				text = text .. '\nsetTemporalVariable("element", _option)'
				text = text .. '\nprocessBrackets(_element)'
				text = text .. '\ntable.insert(_message.options, _element)'
			text = text .. '\nend'
			text = text .. '\nremoveTemporalVariable("element")'
		end
		text = text .. '\nprocessBrackets(_message)'
		if message.writingWait then
			text = text .. string.format('\n_message.writingWait = %s', message.writingWait)
		end
	else
		text = text .. string.format('local _message = %s', message)
	end
	--error message
	local errorMessage = data.errorMessage
	if errorMessage then
		if type(errorMessage) == "table" then
			errorMessage = cjson.encode(cjson.encode(errorMessage))
			text = text .. string.format('local _errorMessage = cjson.decode(%s)', errorMessage)
			text = text .. '\nprocessBrackets(_errorMessage)'
		else
			text = text .. string.format('local _errorMessage = %s', errorMessage)
		end
	end
	-- validate limits in runtime
	text = text .. string.format("\nassert(%s >= %d and %s <= %d, 'invalid timeout value')", 
		data.timeoutRetry, TIMEOUT_MIN, data.timeoutRetry, TIMEOUT_MAX)
	text = text .. string.format("\nassert(%s >= %d and %s <= %d, 'invalid retries value')", 
		data.retries, RETRIES_MIN, data.retries, RETRIES_MAX)
	--run method
	text = text .. string.format(
		"\nSendAndWaitMessage(__interactionId, _message, { ".. 
			"timeout = %s, " .. 
			"retries = %s, ".. 
			"errorMessage = _errorMessage " .. 
		"}, __vccName)", 
		data.timeoutRetry, data.retries)
	return 200, text
end

return {
	MakeExecutable = MakeExecutable
}
