
local function InvalidParameterVariable (stateName, activityId, variableName, aux, reason)
	return aux.invalidParameterActivity(stateName, activityId, "variable " .. variableName .. " " .. reason)
end

local function MakeExpression (stateName, activityId, variableName, aux, expression)
	if expression == "" then
		return InvalidParameterVariable(stateName, activityId, variableName, aux, "empty value")
	end
	return 200, expression
end

local function MakeObject (stateName, activityId, variableName, aux, object)
	local text = "{ "
	for index, data in pairs(object or {}) do
		if not data.name or not data.value then
			return InvalidParameterVariable(stateName, activityId, variableName, aux, "invalid object value")
		end
		local key = string.format('["%s"]', data.name)
		local status, value = MakeExpression(stateName, activityId, variableName .. key, aux, data.value)
		if not status then
			return status, value
		end
		text = text .. (index > 1 and ", " or "") .. key .. " = " .. value
	end
	text = text .. " }"
	return 200, text
end

local function MakeArray (stateName, activityId, variableName, aux, array)
	local text = "{ "
	for index, data in pairs(array or {}) do
		if not data.value then
			return InvalidParameterVariable(stateName, activityId, variableName, aux, "invalid array value")
		end
		local status, value = MakeExpression(stateName, activityId, variableName .. "[" .. index .. "]", aux, data.value)
		if not status then
			return status, value
		end
		text = text .. (index > 1 and ", " or "") .. value
	end
	text = text .. " }"
	return 200, text
end

local function MakeVariable (stateName, activityId, variable, aux)
	if not variable.name or not variable.type then
		return aux.invalidParameterActivity(stateName, activityId, "invalid variable definition")
	end
	local text = variable.name .. " = "
	local status, value
	if variable.type == "expression" then
		status, value = MakeExpression(stateName, activityId, variable.name, aux, variable.value)
	elseif variable.type == "object" then
		status, value = MakeObject(stateName, activityId, variable.name, aux, variable.entries)
	elseif variable.type == "array" then
		status, value = MakeArray(stateName, activityId, variable.name, aux, variable.entries)
	else
		return aux.invalidParameterActivity(stateName, activityId, string.format("invalid variable %s type %s", 
			variable.name, variable.type))
	end
	if status ~= 200 then
		return status, value
	end
	text = text .. value .. "\n"
	text = text .. 'logTrace("DEBUG", string.format("Variable ' .. aux.scapeText(variable.name) .. ' assigned with value %s", '
	text = text .. 'type(' .. variable.name .. ') == "table" and json.encode(' .. variable.name .. ') or tostring(' .. variable.name .. ')))'
	return 200, text
end

local function MakeExecutable (stateName, activityId, data, aux)
	local text = ""
	for index, variable in pairs(data.variables or {}) do
		local status, variableText = MakeVariable(stateName, activityId, variable, aux)
		if status ~= 200 then
			return status, variableText
		end
		text = text .. (index > 1 and "\n" or "") .. variableText
	end
	text = text .. "\nlogTrace('DEBUG', 'Processing activity (end)')"
	return 200, text
end

return {
	MakeExecutable = MakeExecutable
}
