
local function MakeExecutable (name, draw, classes, datasources, stateProcessor, activitiesProcessors, aux)
	local process = {
		states = {},
		classes = classes,
		datasources = datasources
	}
	local model = draw and draw.main and draw.main.model
	if not model then
		return aux.invalidParameter("main model not found")
	end
	--initial node
	local initialConnection = aux.getConnection(model, "initial")
	if not initialConnection then
		return aux.invalidParameter("initial condition not found")
	end
	local initialNodeId = initialConnection.to
	process.condition = initialConnection.data and initialConnection.data.event
	if not initialNodeId or not aux.getNode(model, initialNodeId) or not process.condition then
		return aux.invalidParameter("initial condition invalid")
	end
	--states
	for stateId, state in pairs(model) do
		if stateId ~= "initial" then
			--process state
			local status, stateData = stateProcessor.MakeExecutable(name, stateId, draw, activitiesProcessors, aux)
			if status ~= 200 then
				return status, stateData
			end
			--process transitions
			local transitions = {}
			for index, connection in pairs(aux.getConnections(model, stateId)) do
				local connectionData = connection.data or {}
				--process transition conditions
				local status, conditionData = aux.makeConditions(connectionData.conditions)
				if status ~= 200 then
					return status, conditionData
				end
				local event = connectionData.event or "NoEvent"
				local label = aux.empty(connectionData.description) and (event .. "_" .. index) or connectionData.description
				table.insert(transitions, {
					index = index,
					event = event,
					condition = conditionData,
					to = connection.to,
					label = label
				})
			end
			stateData.initial = initialNodeId == stateId
			stateData.transitions = transitions
			process.states[stateId] = stateData
		end
	end
	--add transitions to name
	for _, state in pairs(process.states) do
		for _, transition in pairs(state.transitions) do
			transition.to_label = process.states[transition.to].label
		end
	end
	return 200, process
end

return {
	MakeExecutable = MakeExecutable
}
