/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	let templates = inConcert.templates.process_flows;
	const format = inConcert.Utils.format;

	function adm_form_process_flows(params)
	{
		var success = adm_form_process_flows.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		return true;
	}

	inConcert.Utils.inherit(adm_form_process_flows, inConcert.conf_form);

	if (typeof inConcert.adm_form_process_flows !== "function") {
		inConcert.adm_form_process_flows = adm_form_process_flows;
	}

	adm_form_process_flows.prototype.render = function( $row )
	{
		//console.log("RENDER")
		var self = this;
		if ( !$row ) {
			// new project
			//return this.popup_render( { type: "new" } );
			//return this.popup_render( { type: "downloading" } );
			return self.show_generator();
		}
		if ( !this.has_permission_to_save() ) {
			return;
		}
		var params = {flowId: $row.attr("row_id")};
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		self.show_waiting("<%=_T('Please wait')%>" + "...");
		
		// edit project
		flowAPI.getFlow(params,
			function(success, flow) {
				if (!success) {
					inConcert.app.m_mainWindow.showBannerMessage({ message: "<%=_T('Error getting flow.')%>", timeout: 5000 });
					return;
				}
				self.show_designer( flow );
			});
	};

	adm_form_process_flows.prototype.popup_show = function( params )
	{
		//debugger;
		adm_form_process_flows.uber.popup_show.call( this, params );
	};

	adm_form_process_flows.prototype.popup_html = function( params )
	{
		switch(params.type) {
			case "new":
				return templates.new_popup;
			case "clone":
				return templates.clone_popup;
			case "downloading":
				return templates.template_wizard_template_popup;
			case "configure_template":
				return templates.template_wizard_popup_configuration;
			default:
				return "";
		}
	};

	adm_form_process_flows.prototype.popup_load_data = function( params, callback )
	{
		let self = this;
		const has_allegro_integration = inConcert.app.has_allegro_integration();
		switch(params.type) {
			case "new":
				let types = {};

				inConcert.adm_process_flow_types.forEach(flow_info => {
					let channel = flow_info.channel;
					let can_include = false;
					if (inConcert.app.hasLicensesForChannel(channel)) {
						if (!has_allegro_integration) {
							can_include = true;
						}
						else {
							can_include = flow_info.is_enabled_for_allegro;
						}
					}
					if (can_include) {
						types[flow_info.type] = flow_info.desc;
					}
				});

				this.fill_combo( 
					$("#adm_form_process_flow_type"),
					false                          , /* blank first element */
					types,
					null                          , /* attr_id             */
					null                          , /* attr_name           */
					true                            /* sorted              */
				);
				break;
			case "configure_template":
				let languages = {};
				
				self.templateSelected.packages.forEach((p)=>{
					languages[p.id] = p.lang; 
				});

				this.fill_combo( 
					$("#adm_form_process_flow_package"),
					false                          , /* blank first element */
					languages,
					null                          , /* attr_id             */
					null                          , /* attr_name           */
					true                            /* sorted              */
				);
				break;
		}
		callback();
	};
	
	adm_form_process_flows.prototype.__makeImportUploader = function() {
		var self = this;
		var MAX_FILE_UPLOAD_SIZE_MB = 50;
		var MAX_FILE_UPLOAD_SIZE_BYTES = MAX_FILE_UPLOAD_SIZE_MB * 1024 * 1024;
		
		var show_error = function(message ) {
			$("#adm_form_process_flow_new_message").text( message ).show();
		};
		
		var hide_error = function() {
			$("#adm_form_process_flow_new_message").hide();
		};
		
		var $files_picker_ctrl = $( "#adm_form_process_flow_import_file_upload");
		$files_picker_ctrl.parent().click(function() {
			var $file_picker_ctrl = $( this ).find( "input[type='file']" );
			$file_picker_ctrl[0].click();
		});

		$files_picker_ctrl.change(function( event ) {
			inConcert.logger.guiAction({action_name: "upload_file", window_id: window.id});

			var file = event.target.files[0];
			if ( !file ) {
				return;
			}

			//aca valida que la extension del archivo a subir sea de un archivo de audio.
			var extension = inConcert.Utils.getFileExtension( file.name ) || "";
			var availables_extensions = {
				"json": true
			};
			if ( !availables_extensions[ extension.toLowerCase() ] ) {
				show_error("<%=_T('Unsupported extension file. Only json.')%>");
				return;
			}
			// tamano maximo soportado en el upload
			if ( file.size > MAX_FILE_UPLOAD_SIZE_BYTES ) {
				show_error(format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", MAX_FILE_UPLOAD_SIZE_MB));
				return;
			}

			// create tracking node
			var $file_name = $( "#adm_form_process_flow_import_file");
			$file_name.val( file.name );

			var $file_percentage = $( "#adm_form_process_flow_import_file_upload_percentage" );
			$file_percentage.text( "0" + "%" ).show();
			
			var flowAPI = inConcert.app.getFlowDesignerAPI();
			self.show_waiting("<%=_T('Please wait')%>" + "...");

			flowAPI.importFlow(file,
				function(status, data) {
					$file_name.val( file.name );
					if (!status) {
					// error
						$file_percentage.text( "" ).hide();
						show_error(format( "<%=_T('Error uploading file {0}.')%>", file.name ));
						$files_picker_ctrl.val("");
						self.hide_waiting();
						self.__removeImportedFlow();
						return;
					}
					// success
					hide_error();
					
					self.__processImportedFlow(data);

					$files_picker_ctrl.val("");
					$file_percentage.text( "" ).hide();
					self.hide_waiting();
				},
				function(percentage) {
					$file_percentage.text( percentage );
				});
		});
	};
	
	adm_form_process_flows.prototype.__processImportedFlow = function(data) {
		this.importedFlow = data;
		$("#adm_form_process_flow_name").val(data.flow.Name);
		$("#adm_form_process_flow_description").val(data.flow.Description);
		$("#adm_form_process_flow_version").val(data.flow.Version?data.flow.Version:"1");
		$("#adm_form_process_flow_type")
			.val(data.flow.Type)
			.prop("disabled", true);
	};
	
	adm_form_process_flows.prototype.__removeImportedFlow = function() {
		this.importedFlow = null;
		$("#adm_form_process_flow_type").prop("disabled", false);
	};

	adm_form_process_flows.prototype.popup_set_events = function( params )
	{
		var self = this;
		switch(params.type) {
			case "new":
				$("#adm_form_process_flow_open")
					.off("click")
					.on("click", function() {
						self.show_waiting("<%=_T('Please wait')%>" + "...");
						self.new_flow({from:"new"});
					});

				$("#adm_form_process_flow_cancel")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});

				$("#adm_form_process_flow_cancel_times")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
					
				$("#adm_form_process_flow_import")
					.off("change")
					.on("change", function() {
						$("#adm_form_process_flow_import_file_container").toggle($(this).is(":checked"));
						if ($(this).is(":checked")) {
							if (self.importedFlow) {
								self.__processImportedFlow(self.importedFlow);
							}
						}
						else {
							self.__removeImportedFlow();
						}
					});

					
				this.__makeImportUploader();
					
				break;
			case "clone":
				$("#adm_form_process_flow_open")
					.off("click")
					.on("click", function() {
						self.clone_flow(params.source_id);
					});

				$("#adm_form_process_flow_cancel")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});

				$("#adm_form_process_flow_cancel_times")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
					
				break;
			case "configure_template":
				$("#adm_form_process_flow_open")
					.off("click")
					.on("click", function() {
						self.new_flow({from:"template"});
					});

				$("#adm_form_process_flow_cancel")
					.off("click")
					.on("click", function() {
						self.templateSelected = undefined;
						self.popup_hide();
					});

				$("#adm_form_process_flow_cancel_times")
					.off("click")
					.on("click", function() {
						self.templateSelected = undefined;
						self.popup_hide();
					});
				break;
		}
	};

	adm_form_process_flows.prototype.__validate_new_flow_server = function(data, callback)
	{
		callback = callback || $.noop;

		var params = {};
		params[data.Id] = data;
		var flowAPI = inConcert.app.getFlowDesignerAPI();

		flowAPI.validateFlow(params,
			function(success, error) {
				if (!success) {
					callback(false, error && error.result, error);
					return;
				}
				callback(true);
			});
	};

	adm_form_process_flows.prototype.validate_new_flow = function(callback, opt)
	{
		var self = this;
		this.popup_hide_errors();

		let show_error = function( $control, message )
		{
			$("#adm_form_process_flow_new_message").text( message ).show();
			if ( $control ) {
				let validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};

		let $name = $("#adm_form_process_flow_name");
		let $description = $("#adm_form_process_flow_description");
		let $type = (opt.from === "new" ? $("#adm_form_process_flow_type"): $("#adm_form_process_flow_generator_type"));
		let $lang = $("#adm_form_process_flow_package");
		let $version = $("#adm_form_process_flow_version");

		let data = {
			Id : inConcert.Utils.generateGuid(),
			Name : $name.val(),
			Description : $description.val(),
			Type : $type.val(),
			Version: $version.val()?$version.val():"2"
		};


		// requerido
		if (!data.Name) {
			show_error( $name, "<%=_T('Must enter flow name.')%>" );
			return callback(false);
		}
		if (!inConcert.Utils.isValid(data.Name, "id")) {
			show_error( $name, "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>" );
			return callback(false);
		}
		if (!data.Description) {
			show_error( $description, "<%=_T('Must enter flow description.')%>" );
			return callback(false);
		}
		if (!data.Type) {
			show_error( $type, "<%=_T('Must enter flow type.')%>" );
			return callback(false);
		}
		
		if (self.templateSelected && opt.from === "template"){
			data.package = $lang.val();
			if(!data.package){
				show_error( $type, "<%=_T('Must select language.')%>" );
				return callback(false);
			}
		}

		let imported = false;
		if ($("#adm_form_process_flow_import").is(":checked")) {
			if (!this.importedFlow) {
				show_error( $("#adm_form_process_flow_import_file"), "<%=_T('Must upload a file.')%>" );
				return callback(false);
			}
			imported = true;
			//console.log(this.importedFlow)
			$.each(this.importedFlow.flow, function(key, value) {
				data[key] = data[key] || value;
			});
			if (!this.importedFlow.override) {
				data.Id = this.importedFlow.flow.Id;
			}
		}

		
		function confirmOverride() {
			$.inconcertPopup.Show(
				"<%=_T('Confirmation')%>",
				"<%=_T('A flow with same name already exists. Do you want to override it?')%>",
				$.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					callback(response.toLowerCase() === "yes"? data : false);
				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		}

		this.__validate_new_flow_server(data, function(status, error, errorData) {
			if (!status) {
				switch(error) {
					case "NAME_ALREADY_EXISTS":
						if (imported && errorData && errorData.flowId) {
							data.Id = typeof(errorData.flowId) === "string" ? errorData.flowId : errorData.flowId.flowId;
							return confirmOverride();
						}
						show_error( $name, "<%=_T('Name must be unique.')%>" );
						break;
					default:
						show_error( null, "<%=_T('Error validating fields.')%>" );
						break;
				}
				return callback(false);
			}
			if (imported && self.importedFlow.override === data.Name) {
				data.Id = self.importedFlow.flow.Id;
				return confirmOverride();
			}
			// validation successufully
			if(opt.from ==="new"){
				callback(data);
			}else{
				self.wizard_template_install(data, callback);	
				
			}
			
		});
	};

	adm_form_process_flows.prototype.new_flow = function(opt)
	{
		var self = this;
		this.validate_new_flow(function(data) {
			if (!data) {
				self.hide_waiting();
				return;
			}
			self.show_designer(data,(opt.from === "template"));
		},opt);
	};

	adm_form_process_flows.prototype.show_designer = function (data, autosave)
	{
		const self = this;
		let designer = new inConcert.adm_process_flow_designer({
			frm: this,
			data : data,
			close : function() {
				self.dashboard.view.refresh();
			}
		});
		designer.Show();
		if(autosave){
			designer.__save();
		}
		this.popup_hide();
		self.hide_waiting();
	};

	adm_form_process_flows.prototype.clone_flow = function(flowId)
	{
		var self = this;
		this.popup_hide_errors();

		var show_error = function( $control, message )
		{
			$("#adm_form_process_flow_new_message").text( message ).show();
			if ( $control ) {
				var validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};

		var $name = $("#adm_form_process_flow_name");
		var name = $name.val();

		// requerido
		if (!name) {
			show_error( $name, "<%=_T('Must enter flow name.')%>" );
			return;
		}
		if (!inConcert.Utils.isValid(name, "id")) {
			show_error( $name, "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>" );
			return;
		}

		inConcert.app.getFlowDesignerAPI().cloneFlow(flowId, name, function(status, error) {
			if (!status) {
				switch(error && error.result) {
					case "NAME_ALREADY_EXISTS":
						show_error( $name, "<%=_T('Name must be unique.')%>" );
						break;
					default:
						show_error( null, "<%=_T('Error cloning flow.')%>" );
						break;
				}
				return;
			}
			self.dashboard.view.refresh();
			self.popup_hide();
		});
	};

	/***
	 * 
	 * 	TEMPLATES SECTION
	 * 
	 */

	 adm_form_process_flows.prototype.show_generator = function ()
	 {
		// console.log(this)
		 let generator = new inConcert.adm_process_flow_generator({
			 frm: this,
			 close : function() {
				 //self.dashboard.view.refresh();
			 }
		 });
		 generator.Show();
		 this.popup_hide();
	 };

	/*
	 * Obtiene las templates disponibles para determinada versión y canal entregadas en el 
	 * options: 
	 * ej: {v: "0.0.1", type:"whatsapp"}
	 */
	adm_form_process_flows.prototype.get_templates_options = function(options,callback){
		this.templateSelected = undefined;
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		options.lang = inConcert.app.getLoggedUserLanguage();
		flowAPI.listTemplates(options,callback);
	};

	/*
	 * Obtiene las categorías disponibles para determinada versión y canal entregadas en el 
	 * options: 
	 * ej: {v: "0.0.1", type:"whatsapp"}
	 */
		adm_form_process_flows.prototype.get_categories_options = function(options,callback){
			this.templateSelected = undefined;
			var flowAPI = inConcert.app.getFlowDesignerAPI();
			options.lang = inConcert.app.getLoggedUserLanguage();
			flowAPI.list_categories(options,callback);
		};
	/*
	 * Descarga un template concreto indicado por su uuid
	 * ej: {v: "0.0.1", type:"whatsapp"}
	 */
	adm_form_process_flows.prototype.download_template = function(options,callback){
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.download_template(options,callback);
	};

	adm_form_process_flows.prototype.print_templates_library_pagination = function(library){
		let self = this;
		if(!self.library_pagination_offset){
			self.library_pagination_offset = 1;
		}

		//console.log("PRINT PAGINATION", library)
		if(library[0].templates.pages > 1){
			$("#flow_designer_view_generator_paginator_container").html(Mustache.render(templates.generator_pagination_content, {
				offset: self.library_pagination_offset || 1,
				pages: library[0].templates.pages || 1
			}));
			if (self.library_pagination_offset < library[0].templates.pages){
				//quedan paginas por arriba
				$("#flow_designer_view_paginator_next").removeClass("pg-button-disabled");
				$("#flow_designer_view_paginator_next").off("click").on("click", function(){
					self.library_pagination_offset++;
					self.download_templates_catalog();
				});
			}
			if (self.library_pagination_offset > 1){
				//quedan paginas por abajo
				$("#flow_designer_view_paginator_prev").removeClass("pg-button-disabled");
				$("#flow_designer_view_paginator_prev").off("click").on("click", function(){
					self.library_pagination_offset--;
					self.download_templates_catalog();
				});
			}
		}else{
			$("#flow_designer_view_generator_paginator_container").html("");
		}
		
	};

	/***
	 * Limpia las opciones de librería para preveer una selección equivocada
	 * 
	 */
	adm_form_process_flows.prototype.lock_templates_library = function(){
		$("#adm_form_process_flow_generator_catalog").html(templates.template_list_lock);
	};

	/***
	 * Pinta las opciones de una librería en el contenedor de opciones
	 */
	adm_form_process_flows.prototype.print_templates_library = function(library){	
		let processed = "";
		let self = this;
		if(library.length === 0){
			//$("#adm_form_process_flow_import_template_container").html(templates.template_list_no_result)
			$("#adm_form_process_flow_generator_catalog").html(templates.template_list_no_result);
		}else{
			let setClickEvents = function(library){
				library.forEach((category)=>{
					category.id = category.name.replaceAll(" ", "_");
					category.templates.data.forEach((template)=>{

						$(`#process_flow_template_overview_${category.id}_${template.id}_button`).click(function(){
							
							self.templateSelected = template;
							self.popup_render({type:"configure_template"});
						});
					});
				});
				
			};
			library.forEach((category) => {
				category.id = category.name.replaceAll(" ", "_");
				processed += Mustache.render(templates.template_overview_category, category);
				//$("#adm_form_process_flow_import_template_container").html(processed)
				$("#adm_form_process_flow_generator_catalog").html(processed);
	
				let templatesProcessed = "";
				category.templates.data.forEach((template) => {
					
					template.category = category.id;
					templatesProcessed += Mustache.render(templates.template_overview_item, template);
					$(`#${category.id}_templates_container`).html(templatesProcessed);
					
					templatesProcessed = $(`#${category.id}_templates_container`).html();
				});
				//processed = $("#adm_form_process_flow_import_template_container").html()	
				processed = $("#adm_form_process_flow_generator_catalog").html();	
			});
			setClickEvents(library);
		}

		self.print_templates_library_pagination(library);
		
	};

	adm_form_process_flows.prototype.wizard_template_install = function(data, callback){
		const self = this;
		
		//Controla los diferentes pasos que hay y se encarga de pintarlos sobre el wizard.
		const buffer = (()=>{
			let installBuffer = [];
			let uninstallBuffer = [];
			const errors_management = {
				show_error: function(target, message){
					$(`#adm_form_process_flow_wizard_step_error_${target}`).text( message ).show();
				},
				clear_errors: function(){
	
				}
			};

			const __print_steps = function(uninstallMode){
				let steps_html = "";
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				buffer.forEach((step)=>{
					steps_html += Mustache.render(templates.template_wizard_template_popup_step, step);
					$("#adm_for_process_flow_wizard_steps_container > .card > .card-block").html(steps_html);
					$(`#adm_form_process_flow_wizard_step_${step.action} > p`).removeClass();
					switch (step.status) {
						case "loading": 
							$(`#adm_form_process_flow_wizard_step_${step.action} > p`).addClass("adm_for_process_flow_wizard_step_loading");
							break;
						case "success":
							$(`#adm_form_process_flow_wizard_step_${step.action} > p`).addClass("adm_for_process_flow_wizard_step_success");
							break;
						case "error":
							$(`#adm_form_process_flow_wizard_step_${step.action} > p`).addClass("adm_for_process_flow_wizard_step_error");
							errors_management.show_error(step.action, step.error);
							break;
						default:
							$(`#adm_form_process_flow_wizard_step_${step.action} > p`).addClass("adm_for_process_flow_wizard_step_pending");
						
					}
					steps_html = $("#adm_for_process_flow_wizard_steps_container > .card > .card-block").html();
				});
			};

			const add = function(step, uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				if(!buffer.find((stepB)=>{return stepB.action === step.action;})){
					buffer.push(step);
				}
				
				__print_steps(uninstallMode);
			};
			const changeStatus = function(action, newStatus, uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				let target = buffer.find((s)=>{
					return s.action === action;
				});
				target.status = newStatus;
				__print_steps(uninstallMode);
			};
			const get = function(action, uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				return buffer.find((s)=>{
					return s.action === action;
				});
			};
			const setError = function(action, error, uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				let target = buffer.find((s)=>{
					return s.action === action;
				});
				target.status = "error";
				target.error = error;
				__print_steps(uninstallMode);
			};

			const getNext = function(uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
						
				return buffer.find((s)=>{
					return s.status === "pending";
				});
			};

			const finish = function(action, uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				buffer.find((s)=>{return s.action === action;}).status = "success";
			};

			const reset = function(uninstallMode){
				let buffer = (uninstallMode?uninstallBuffer:installBuffer);
				for(let i = 0; i < buffer.length; i++){
					buffer[i].status = "pending";
				}
				//console.log("RESET", buffer)
				__print_steps(buffer);
			};
			const clear = function(uninstallMode){
				if (uninstallMode){
					uninstallBuffer = [];
					//console.log("U", uninstallMode)
				}else{
					installBuffer = [];
					//console.log("I", installBuffer)
				}
				__print_steps(uninstallMode);
			};

			let context = {};
			const setContext = function(key, value){
				context[key] = value;
			};
			const getContext = function(key){
				return context[key];
			};

			return {add,changeStatus,get, setError, getNext, finish,reset, clear, setContext, getContext};
		})();
		

		const steps = {
			luis: {
				install:{
					action:"luis",
					description:"<%=_T('Installing Cognitive Model.')%>",
					status: "pending",
					step_logic: function(callback, {download}){
						download.luis.name = data.Name;
						//download.luis.description = `Luis App for ${data.Name} flow`;
						download.luis.description = data.Description;					
						let apiLuis = inConcert.app.getLuisAPI();						
						apiLuis.importApp(download.luis, function(success, appLuis){
							//console.log("IMPORT", success, appLuis)
							//Debemos sustituir los ids del servicio cognitivo del flujo, por el nuevo ID de luis		
							if(!success){
								if(appLuis.cognitiveServiceError){
									callback(success, appLuis.cognitiveServiceError.response.error.message);
								}else{
									callback(success,"<%=_T('Error Importing template.')%>");
								}
								
							}else{
								buffer.add(steps.luis.uninstall,true);
								buffer.setContext("appLuis",appLuis);
								callback(success,{appLuis, download});
							}
						});
					}
				},
				train:{
					action:"training",
					description:"<%=_T('Training Cognitive Model.')%>",
					status: "pending",
					step_logic: function(callback, {appLuis, download}){
						/*  LANZAMOS ERROR PARA PROBAR EL UNINSTALL */
						// callback(false,"ERROR DEBUG")
						// return
						let apiLuis = inConcert.app.getLuisAPI();		
						apiLuis.trainApp(appLuis, function(success, trainedInfo){
							if(!success){
								callback(success, "<%=_T('Error Training template.')%>");
							}else{
								
								const _parse_train_response = function(result){
									if (typeof result !== "object" || !result) {
										return;
									}
									let statuses = {};
							
									statuses["0"] = {
										name  : "Success",
										action: "TRAINED",
										msg   : "<%=_T('App {0} was successfully trained.')%>"
									};
									statuses["1"] = {
										name  : "Fail",
										action: "ERROR",
										msg   : {
										// failureReason
											"FewLabels": "<%=_T('Error Training App ({0}).')%>" + " " + format("<%=_T('The Intent ({0}) no has Utterances.')%>", result.__intentName || result.modelId)
										}
									};
									statuses["2"] = {
										name  : "UpToDate",
										action: "TRAINED",
										msg   : "<%=_T('App {0} was successfully updated trained.')%>"
									};
									statuses["3"] = {
										name  : "InProgress",
										action: "InProgress",
										msg: "<%=_T('In Progress Training App {0}.')%>"
									};
									statuses["9"] = {
										name  : "Queued",
										action: "ERROR",
										msg   : "<%=_T('Queued Training App {0}.')%>"
									};
							
									let status_id = result.statusId.toString();
									let current_status = statuses[status_id];
									return current_status;
								};

								//Parseamos la respuesta del entrenamiento
								let response_parsed = _parse_train_response(trainedInfo);
								if (response_parsed) {
									if (response_parsed.action === "TRAINED") {
										callback(true, {appLuis, download});
										
									}
									else {
										let errorMessage = "";
										if (typeof(response_parsed.msg) === "string") {
											errorMessage = response_parsed.msg;
										}
										else if (typeof(response_parsed.msg) === "object") {
											errorMessage = response_parsed.msg;
										}
										callback(false, errorMessage);
									}
								}
							
							}
						});
					}
				},
				publish:{
					action:"publish",
					description:"<%=_T('Publishing Cognitive Model.')%>",
					status: "pending",
					step_logic: function(callback, {appLuis, download}){
						
						let apiLuis = inConcert.app.getLuisAPI();

						// //Para publicar la aplicación, necesitamos seleccionar la clave de suscripción
						// //y la zona horaria
						const _publish = function({endpoint, timezone, flag}){
							appLuis.subscriptionKey = endpoint;
							appLuis.timezone = timezone;
							appLuis.verbose = flag;

							apiLuis.publishApp(appLuis, function(success){
								if(success){
									callback(success,{appLuis, download});
								}
								else {
									callback(success,"<%=_T('Error Publishing App.')%>");
								}
							});
						};

						const _validate_form = function(){

							let validEndpoint = self.validate_field($("#adm_form_process_flow_wizard_step_form_publish_luis_endpoint"), "required", "<%=_T('Unable to Publish App. Endpoint Key is required.')%>");
							let validTimeZone = self.validate_field($("#adm_form_process_flow_wizard_step_form_publish_luis_timezone"), "required", "<%=_T('Unable to Publish App. TimeZone is required.')%>");
							let validated = validEndpoint && validTimeZone;
							let values = {
								endpoint:$("#adm_form_process_flow_wizard_step_form_publish_luis_endpoint").val(),
								timezone: $("#adm_form_process_flow_wizard_step_form_publish_luis_timezone").val(),
								flag: $("#adm_form_process_flow_wizard_step_form_publish_luis_verbose").prop("checked")
							};
							
							return {validated, values};
						};
						const _set_form_events = function(){
							$("#adm_form_process_flow_wizard_step_form_publish_luis_confirm").click(function(){
								let {validated, values} = _validate_form();
								if(validated){
									$("#adm_form_process_flow_wizard_step_form_publish").hide();
									_publish(values);
								}
							});
						};

						const _load_publish_cognitive_form = function(endpoints, timezones){
							
							$("#adm_form_process_flow_wizard_step_form_publish").html(templates.template_wizard_template_popup_step_form_publish_luis_app);
							$("#adm_form_process_flow_wizard_step_form_publish").show();
						
							const $combo_timezone = $("#adm_form_process_flow_wizard_step_form_publish_luis_timezone");
							self.fill_timezones(timezones, $combo_timezone);

							 const $select_component = $("#adm_form_process_flow_wizard_step_form_publish_luis_endpoint");
							 const is_blank_first_item = true;
							 const attr_id = "subscriptionKey";
							 const attr_name = "subscriptionName";
							 var sorted = true;
							 self.fill_combo($select_component, is_blank_first_item, endpoints, attr_id, attr_name, sorted);
						
							_set_form_events();
						};


	
						apiLuis.getVCCSubscriptionsKeys(function(success, list){
              success = true;
							inConcert.app.loadTimezones(function(success, timezones){
                success = true;
                _load_publish_cognitive_form(list, timezones);
							});
						});

						
					}
				},
				link:{
					action:"linking",
					description:"<%=_T('Linking Cognitive Application to process.')%>",
					status: "pending",
					step_logic: function(callback, {appLuis, download}){
						const _recursiveSearch = function(obj, prop, rec_callback){							
							if(typeof obj === "object"){
								for(let k in obj){
									if(obj.hasOwnProperty(k)){
										if(k === prop && obj[k] !== ""){
											//console.log("ENCONTRADA REFERENCIA CON VALOR", k, prop, obj[k])
											//El target ya no es un string con el antiguo id de aplicación, ahora es:
											//	{[appId]:[appName]} en algunos casos y 
											// 	[appId] en otros
											if(typeof obj[k] === "string"){
												rec_callback(obj[k], "");
											}else{
												if(typeof obj[k] === "object"){
													rec_callback(Object.keys(obj[k])[0], obj[k][Object.keys(obj[k])[0]]);
												}
											}
											
											return;
										}
										if(typeof obj[k] === "object"){						
											_recursiveSearch(obj[k], prop, rec_callback);
										}
									}
								}
							}
						};
						const _recursiveChange = function(obj, prop,oldId, newId, oldName, newName){
							//console.log("ENTRAMOS A SUSTITUIR: ",oldValue, "POR",newValue)
							if(typeof obj === "object"){
								for(let k in obj){
									if(obj.hasOwnProperty(k)){
										
										if(k === prop && obj[k] === oldId){
											//console.log("SUSTITUIMOS EN", obj)
											//console.log("PRE", obj[k])
											obj[k] = newId;
											//console.log("POST", obj[k])
										}
										if(k === prop && obj[k][oldId]){
											//console.log("PRE", obj[k])
											let aux = {};
											aux[newId] = newName;
											obj[k] = aux;
											//console.log("POST", obj[k])
										}
										if(typeof obj[k] === "object"){
											_recursiveChange(obj[k],prop,oldId,newId, oldName, newName);
										}else{
											if(typeof obj[k] === "string"){
												if(obj[k].indexOf(oldId)>0){
													//console.log("STRING ENCONTRADO CON REFERENCIA AL ANTIGUO ID", obj[k], k, obj)
													obj[k] = obj[k].replaceAll(oldId, newId);
													//obj[k] = obj[k].replaceAll(oldName, newName);
												}
											}
										}
									}
								}
							}
						};
						let firstTime = true;
						_recursiveSearch(data, "cognitiveServicesApp", function(_old_id, _old_name){
							if(firstTime){
								//console.log("RECIBIMOS VALORES ANTIGUOS:", _old_id, _old_name);
								//console.log("VAMOS A SUSTITUIR POR:", appLuis.appId, data.Name);
								_recursiveChange(data, "cognitiveServicesApp",_old_id, appLuis.appId,_old_name, data.Name);
								callback(true,{appLuis, download});
							}
							firstTime = false;
						});				
						
						
						
					}
				},
				uninstall:{
					action:"uninstall_luis",
					description:"<%=_T('Uninstalling Luis Application.')%>",
					status: "pending",
					step_logic: function(callback){
						let appLuis = buffer.getContext("appLuis");
						let apiLuis = inConcert.app.getLuisAPI();
						apiLuis.deleteApps([appLuis.appId],function(success){
							
							if(!success){
								callback(success, "<%=_T('Error uninstalling Luis Application.')%>");
							}else{
								callback(true);
							}
						});
						//console.log("Uninstalling Luis");
						
					}
				}
			},
			qna:{
				init:{
					action:"qna",
					description:"<%=_T('Downloading QnA information.')%>",
					status: "pending",
					step_logic: function(callback, {download}){
						//Debemos preguntar si despues de la instalación se desea publicar QnA
						const continue_installation = function(publishAfter){
							const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
							qnAMakerAPI.getVccKnowledgeBase(function(success, response){
								
								if(!success){
									if(response.cognitiveServiceError){
										if(response.cognitiveServiceError.code === 404 && response.cognitiveServiceError.errType === "inconcert"){
											//No existe una base, por lo que hay que crearla
											buffer.add(steps.qna.createKb);
										}else{
											callback(false,"<%=_T('Error getting QnA Information.')%>");
											return;
										}
									}else{
										//console.log("ERROR")
										callback(false,"<%=_T('Error getting QnA Information.')%>");
										return;
									}
									
								}
								//Añadimos el paso de importación de la informacion qna
								buffer.add(steps.qna.addToKb);
								//Si el usuario quiere publicar despues de la instalación, añadimos ese paso al wizard
								if(publishAfter){
									buffer.add(steps.qna.publish);
								}
								//Añadimos el paso del lincado de datos de las metadata de qna
								buffer.add(steps.qna.link);

								callback(true, {download, kb:response});
							});
						};
						const _set_events = function(){
							$("#template_wizard_template_popup_step_form_publish_qna_confirm").click(function(){
								$("#adm_form_process_flow_wizard_step_form_qna").hide();
								continue_installation($("#template_wizard_template_popup_step_form_publish_qna_check").prop("checked"));
							});
						};
						const _print_form = function(){
							$("#adm_form_process_flow_wizard_step_form_qna").html(templates.template_wizard_template_popup_step_form_publish_qna);
							$("#adm_form_process_flow_wizard_step_form_qna").show();
							_set_events();
						};
						//callback(true)
						_print_form();
					}
				},
				createKb:{
					action:"createKb",
					description:"<%=_T('Creating Knowledge Database.')%>",
					status: "pending",
					step_logic: function(callback, {download}){
						//Debemos preguntar al usuario por el nombre de la base de datos
						const _createKB = function(name){
							const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
							qnAMakerAPI.createKnowledgeBase({name}, function(success, response){
								if(!success){
									callback(success, "<%=_T('Error creating Knowledge Base.')%>");
								}else{
									callback(success, {download, kb:{inconcertInfo:response, qnaDocuments:[]}});
								}
							});
						};

						const _validate = function(){
							let valid = self.validate_field($("#adm_form_process_flow_wizard_step_form_create_KB_name"), "required", "<%=_T('Unable to create Knowledge Base. Name is required.')%>");
							return {valid,name: $("#adm_form_process_flow_wizard_step_form_create_KB_name").val()};
						};

						const _set_events = function(){
							//console.log("SET EVENTS",$("#adm_form_process_flow_wizard_step_form_create_KB_confirm"))
							$("#adm_form_process_flow_wizard_step_form_create_KB_confirm").click(function(){
								
								let {valid, name} = _validate();
								if(valid){
									$("#adm_form_process_flow_wizard_step_form_createKb").hide();
									_createKB(name);
								}
							});
						};

						const _print_form = function(){
							$("#adm_form_process_flow_wizard_step_form_createKb").html(templates.template_wizard_template_popup_step_form_create_KB);
							$("#adm_form_process_flow_wizard_step_form_createKb").show();

							_set_events();
						};

						_print_form();
						
					}
				},
				addToKb:{
					action:"addToKb",
					description:"<%=_T('Adding knowledge to QnA Maker.')%>",
					status: "pending",
					step_logic: function(callback, {download, kb}){
						const _deserializeMetadata = function(metadata){
							if(metadata){
								let metas = metadata.split(","); //(asumimos que cuando venga con mas de una metadata, la exportación viene separada por comas)
								let deserialized = [];
								metas.forEach((meta_serialized) =>{
									let meta_tupla = meta_serialized.split(":");
									if(meta_tupla[0] !== ""){
										deserialized.push({
											name: meta_tupla[0],
											value: meta_tupla[1]
										});
									}
								});
	
								return deserialized;
							}else{
								return [];
							}
							
						};
						const _deserialize_tsv = function(serialized){
							let rows = serialized.split("\n");
							let headers = rows[0].split("\t");

							let deserialized = [];
							for(let i = 1; i < rows.length; i++){
								let element = {};
								let values = rows[i].split("\t");
								for(let a = 0; a < headers.length; a++){
									element[headers[a]] = values[a].replaceAll("\\n","\n");
								}
								deserialized.push(element);
							}

							let dictionary = {};
							for(let i = 0; i < deserialized.length; i++){
								if(!dictionary[deserialized[i].QnaId]){
									dictionary[deserialized[i].QnaId] = {
										answer: deserialized[i].Answer,
										questions: [deserialized[i].Question],
										metadata: _deserializeMetadata(deserialized[i].Metadata)
									};
									dictionary[deserialized[i].QnaId].metadata.push({name:"flow", value:data.Name});
								}else{
									dictionary[deserialized[i].QnaId].questions.push(deserialized[i].Question);
								}
							}

							let target = [];
							for(let id in dictionary){
                				if(dictionary.hasOwnProperty(id)){
									target.push(dictionary[id]);
                				}
							}
							return target;
						};

						//console.log(download.qna, kb.inconcertInfo.kbId)
						let deserialize_qna = _deserialize_tsv(download.qna);
						let dataQna = {
							add:deserialize_qna,
							delete:[],
							update:[],
							isNew: false,
							kbId:kb.inconcertInfo.kbId
						};
						buffer.setContext("QnAInfo", dataQna);
						const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
						qnAMakerAPI.updateKnowledgeBase(kb.inconcertInfo.kbId,dataQna, function(success, response){
							if(!success){
								callback(success, "<%=_T('Error adding Knowledge information to DataBase.')%>");
							}else{
								buffer.add(steps.qna.uninstall,true);
								buffer.setContext("KBUpdateResponse", response);
								callback(success,{download,kb,updateStatus:response});
							}
							
						});
					}
				},
				publish:{
					action:"publish_qna",
					description:"<%=_T('Publishing QnA knowledgement.')%>",
					status: "pending",
					step_logic: function(callback, {download,kb,updateStatus}){
						// callback(false)//forzamos el error
						// return;
						const qnAMakerAPI = inConcert.app.getQnAMakerAPI();

						const _publish = function(){
							qnAMakerAPI.publishKnowledgeBase(kb.inconcertInfo.kbId, null, function(success){
							
								if(!success){
									callback(success, "<%=_T('Error Publishing QnA knowledgement.')%>");
								}else{
									callback(true, {download,kb, updateStatus});
								}							
							});
						};

						let interval = setInterval(function(){
							qnAMakerAPI.getKnowledgeBaseOperationDetail(updateStatus.operationId, function(success, detail){
								if(!success){
									callback(success, "<%=_T('Error obtaining Qna Status')%>");
								}else{
									//console.log(detail)
									if(detail.operationState === "Succeeded"){
										_publish();
										clearInterval(interval);
									}
								}
								
								
							});
						},5000);
						

						
					}
				},
				link:{
					action:"linkQna",
					description:"<%=_T('Linking QnA information.')%>",
					status: "pending",
					step_logic: function(callback){
						/*  LANZAMOS ERROR PARA PROBAR EL UNINSTALL */
						// callback(false,"ERROR DEBUG")
						// return


						//El objetivo de este paso es añadir a todas las actividades qna
						//El metadato creado para este proceso que es flow:{nombre_flow}

						const flowName = data.Name;
						//Debemos buscar todos los elementos que tengan type: execute_qna
						//En caso de que no tengan metadata, esta debe ser:
						/*
						metadata:{operationType:'AND', filters:[{name:flow,value:{nombreflow}}]}
						*/
						/*
						Si si que tiene metadata, simplemente hay que añadir a los filtros el nuevo metadata
						*/
						let _searchInData = function(block){
							if(typeof block === "object"){
								let isTarget = false;
								for(let k in block){
									if(k === "type" && block[k] === "execute_qna"){
										//Es el objeto que buscamos
										isTarget = true;
									}else{
										if(typeof block[k] === "object"){
											_searchInData(block[k]);
										}
									}
								}
								if(isTarget){
									if(!block.metadata){
										block.metadata = {
											operationType: "AND",
											filters:[{name:"flow", value:flowName}]
										};
									}else{
										block.metadata.filters.push({name:"flow",value:flowName});
									}
								}
							}
						};

						let encode_html = function(raw){							
							 return raw.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;");
						};

						let _buildXML = function(block){
							if(typeof block === "object"){
								let isTarget = false;
								for(let k in block){
									if(k==="xml"){
										isTarget = true;
									}else{
										if(typeof block[k] === "object"){
											_buildXML(block[k]);
										}
									}
								}
								if(isTarget){
									let virtualElement = document.createElement("div");
									virtualElement.innerHTML = block.xml;
									let celdas = virtualElement.getElementsByTagName("mxcell");
									for(let i = 0; i < celdas.length; i++){
										let validJSON = true;
										let celdaValue = {};
										try {
											celdaValue = JSON.parse(celdas[i].getAttribute("value"));
										} catch (error) {
											validJSON = false;
										}
										if(validJSON && celdaValue && celdaValue.type){
											if(celdaValue.type === "execute_qna"){
												let oldValue =  encode_html(JSON.stringify(celdaValue));
												if(!celdaValue.metadata){
													celdaValue.metadata = {
														operationType: "AND",
														filters:[{name:"flow", value:flowName.toLowerCase()}]
													};
												}else{
													celdaValue.metadata.filters.push({name:"flow",value:`"${flowName.toLowerCase()}"`});
												}
												let newValue = encode_html(JSON.stringify(celdaValue));
												block.xml = block.xml.replace(oldValue,newValue);

											}
										}
										
									}
								}
							}
						};
						_searchInData(data);
						_buildXML(data);
						callback(true);
					}
				},
				uninstall:{
					action:"uninstall_qna",
					description:"<%=_T('Uninstalling QnA Knowledgement.')%>",
					status: "pending",
					step_logic: function(callback){
						
						let QnAInfo = buffer.getContext("QnAInfo");
						
						const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
						
						let lastOperationResponse = buffer.getContext("KBUpdateResponse");
						
						let _StartUninstall = function(){

							qnAMakerAPI.getVccKnowledgeBase(function(success, infoKB){
								if(success){
									let ToRemove = [];
									infoKB.qnaDocuments.forEach((question) =>{
										//console.log(question, data.Name)
										if(question.metadata){
											
											if(question.metadata.find((meta)=>{ return meta.name === "flow" && meta.value === data.Name.toLowerCase();})){
												ToRemove.push(question.id);
											}
										}
									});
									//console.log("TO REMOVE", ToRemove)
									QnAInfo.add = [];
									QnAInfo.delete = ToRemove;
									qnAMakerAPI.updateKnowledgeBase(QnAInfo.kbId,QnAInfo, function(success, response){
										if(!success){
											callback(success, "<%=_T('Error removing Knowledge information to DataBase.')%>");
										}else{
	
											let interval = setInterval(function(){
												qnAMakerAPI.getKnowledgeBaseOperationDetail(response.operationId, function(success, detail){
													if(!success){
														callback(success, "<%=_T('Error obtaining Qna Status')%>");
													}else{
														//console.log(detail)								
														if(detail.operationState === "Succeeded"){
															callback(success);
															clearInterval(interval);
														}
													}
													
													
												});
											},5000);
	
	
											
										}
											
									});
								}else{
									callback(success, "<%=_T('Error removing Knowledge information to DataBase.')%>");
								}
							});
						};
						
						
						let interval = setInterval(function(){
							qnAMakerAPI.getKnowledgeBaseOperationDetail(lastOperationResponse.operationId, function(success, detail){
								//console.log("STATUS UNINSTALL QNA:",success,detail)
								if(!success){
									callback(success, "<%=_T('Error obtaining Qna Status')%>");
								}else{
									//console.log(detail)								
									if(detail.operationState === "Succeeded"){
										_StartUninstall();
										clearInterval(interval);
									}
								}
								
								
							});
						},5000);



					}
				}
				
			}
		};
		const initStep = {
			action:"downloading",
			description:"<%=_T('Downloading template data.')%>",
			status: "pending",
			step_logic: function(callback){
				self.download_template({packageId: data.package}, function(success, download){
					
					if(!success){
						callback(success);
					}
					//Seteamos la data del process en la data para el designer
					$.each(download.process, function(key, value) {
							data[key] = data[key] || value;
					});
					//data.Id = download.process.Id;	
					data.Id = inConcert.Utils.generateGuid();
					
					//TODO Debemos pintar los pasos faltantes en el wizard
					if(download.luis){
						for(let k in steps.luis){
							if(k !== "uninstall"){
								
								buffer.add(steps.luis[k]);
							}							
						}	
					}

					if(download.qna){
						buffer.add(steps.qna.init);
					}
					//Se retorna el estado y los parametros que pueda necesitar el siguiente paso
					callback(success, {download});
				});
			}
		};
	

		//Orquesta cada uno de los pasos de instalación del wizard
		const orchestator = ((finalCallback, buffer)=>{
			
			
			let play = function(lastParams){
				
				let nextStep = buffer.getNext();
				//console.log("NEXT STEP INSTALL", nextStep)
				if(nextStep){
					buffer.changeStatus(nextStep.action,"loading");
					try{
						nextStep.step_logic(function(success, data){
							//console.log("CALLBACK LOCAL", success, data)
							if(success){
								//console.log("STEP SUCCESS INSTALL", nextStep)
								buffer.changeStatus(nextStep.action, "success");
								play(data);
							}else{
								//console.log("STEP ERROR", nextStep)
								buffer.setError(nextStep.action, data);
								_show_cancel_options();
							}
						}, lastParams);
					}catch(e){
						console.error(e);
						buffer.setError(nextStep.action, e);
						_show_cancel_options();
					}
					
				}else{
					//console.log("NO HAY MAS PASOS; FIN!")
					self.popup_hide();
					finalCallback(data);
				}
				
			
			};
			let uninstall_context = {};
			let uninstall_step = function(lastParams){
				let nextStep = buffer.getNext(true);
				//console.log("NEXT STEP UNINSTALL", nextStep);
				if(nextStep){

					buffer.changeStatus(nextStep.action,"loading",true);
					try{
						nextStep.step_logic(function(success, data){
							if(success){
								//console.log("STEP SUCCESS UNINSTALL", nextStep)
								buffer.changeStatus(nextStep.action, "success",true);
								uninstall_step(data);
							}else{
								buffer.setError(nextStep.action, data,true);
							}
						}, lastParams);
					}catch(e){
						buffer.setError(nextStep.action, e,true);
					}
					
				}else{				
					uninstall_context.finalCallback(true);
				}
			};
			let uninstall = function(callback_u){
				uninstall_context.finalCallback = callback_u;
				uninstall_step();
			};
			let reset = function(){
				buffer.reset();//Reiniciamos los pasos del wizard
				buffer.reset(true);
				buffer.clear(true);//Limpiamos los pasos de desinstalacion					
				buffer.clear();//Limpiamos todos los pasos de instalacion
				buffer.add(initStep);
				
			};
			return {play, uninstall, reset};
		})(callback, buffer);


		let _set_cancel_events = function(){
			//Aspa
			$("#adm_form_process_flow_wizard_cancel").off("click").on("click",function(){
				_hide_cancel_options();
				orchestator.uninstall(function(success){
					if(success){
						self.popup_hide();
					}
				});
				
			});

			//Reintentar
			$("#adm_for_process_flow_wizard_steps_retry").off("click").on("click",function(){
				_hide_cancel_options();
				orchestator.uninstall(function(success){
					//console.log("UNINSTALL FINISHED", success)
					if(success){
						//console.log("RESET AND PLAY...")
						orchestator.reset();
						orchestator.play();
					}
				});
			});

			//Cancelar
			$("#adm_for_process_flow_wizard_steps_cancel").off("click").on("click",function(){
				_hide_cancel_options();
				orchestator.uninstall(function(success){
					if(success){
						self.popup_hide();
					}
				});
			});
		};
		let _show_cancel_options = function(){
			$("#adm_form_process_flow_wizard_cancel").show();
			$("#adm_for_process_flow_wizard_steps_actions").show();
			_set_cancel_events();
		};
		let _hide_cancel_options = function(){
			$("#adm_form_process_flow_wizard_cancel").hide();
			$("#adm_for_process_flow_wizard_steps_actions").hide();
		};


		let async_install = async function(){
			//Renderizamos el popup del wizard
			self.popup_hide();
			self.popup_render( { type: "downloading" } );
			
			buffer.add(initStep);
		
			orchestator.play();
			
		};
		async_install(data);
		
	};

	adm_form_process_flows.prototype.load_generator_type_select = function(){
		//console.log("LOADING")
		const has_allegro_integration = inConcert.app.has_allegro_integration();
		let types = {};

		inConcert.adm_process_flow_types.forEach(flow_info => {
			//console.log(flow_info)
			let channel = flow_info.channel;
			let can_include = false;
			if (inConcert.app.hasLicensesForChannel(channel)) {
				if (!has_allegro_integration) {
					can_include = true;
				}
				else {
					can_include = flow_info.is_enabled_for_allegro;
				}
			}
			if (can_include) {
				types[flow_info.type] = flow_info.desc;
			}
		});

		//console.log(types)

		this.fill_combo( 
			$("#adm_form_process_flow_generator_type"),
			false                          , /* blank first element */
			types,
			null                          , /* attr_id             */
			null                          , /* attr_name           */
			true                            /* sorted              */
		);
	};

	adm_form_process_flows.prototype.load_generator_category_select = function(callback){
		let self = this;
		let types = {all:"<%=_T('All')%>"};

		self.get_categories_options({type:$("#adm_form_process_flow_generator_type")[0].value}, function(success, library){
			if (!success) {
				console.error("Error obtaining categories!");
				return;
			}

			library.forEach((category)=>{
				types[category.id] = category.name;
			});


			self.fill_combo( 
				$("#adm_form_process_flow_generator_category"),
				false                          , /* blank first element */
				types,
				null                          , /* attr_id             */
				null                          , /* attr_name           */
				false                            /* sorted              */
			);

			$("#adm_form_process_flow_generator_category")[0].value = "all";

			callback();

		});

		
		
	};


	adm_form_process_flows.prototype.load_generator_orderby_select = function(){
		//console.log("LOADING")
		let types = {
			updated: "<%=_T('Most recent')%>",
			download: "<%=_T('Most downloaded')%>"
		};

		this.fill_combo( 
			$("#adm_form_process_flow_generator_orderby"),
			false                          , /* blank first element */
			types,
			null                          , /* attr_id             */
			null                          , /* attr_name           */
			true                            /* sorted              */
		);

		$("#adm_form_process_flow_generator_orderby")[0].value = "download";
	};

	adm_form_process_flows.prototype.load_generator_events = function(){
		let self = this;
		$("#adm_form_process_flow_generator_type")
		.off("change")
		.on("change", function(){
			$("#adm_form_process_flow_generator_filter_text")[0].value = "";
			self.library_pagination_offset = 1;
			self.lock_templates_library();
			self.load_generator_category_select(function(){
				self.download_templates_catalog();
			});
			
			//Debemos reiniciar la categoría seleccionada
		});

		$("#adm_form_process_flow_generator_category")
		.off("change")
		.on("change", function(){
			$("#adm_form_process_flow_generator_filter_text")[0].value = "";
			self.library_pagination_offset = 1;
			self.download_templates_catalog();
		});

		$("#adm_form_process_flow_generator_orderby")
		.off("change")
		.on("change", function(){
			self.download_templates_catalog();
		});

		$("#adm_form_process_flow_generator_filter_button")
		.off("click")
		.on("click", function(){
			self.library_pagination_offset = 1;
			self.download_templates_catalog();
			
		});
	};

	adm_form_process_flows.prototype.download_templates_catalog = function(){
		let self = this;
		self.lock_templates_library();
		self.get_templates_options({
			type:$("#adm_form_process_flow_generator_type")[0].value, 
			category:$("#adm_form_process_flow_generator_category")[0].value,
			sort:$("#adm_form_process_flow_generator_orderby")[0].value,
			filter: ($("#adm_form_process_flow_generator_filter_text")[0].value ? $("#adm_form_process_flow_generator_filter_text")[0].value: undefined),
			page: self.library_pagination_offset || 1
		}, function(success, library){
			if (!success) {
				console.error("Error obtaining templates!");
				return;
			}
			self.print_templates_library(library);
		});
	};

	adm_form_process_flows.prototype.__make_generator_draw = function(){
		let self = this;
		$("#flow_designer_view_generator_page_content").html(templates.generator_content);

		self.load_generator_type_select();
		self.load_generator_orderby_select();
		self.load_generator_category_select(function(){
			self.download_templates_catalog();
		});		
		self.load_generator_events();
	};
}());
