/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	var templates = inConcert.templates.process_flows;
	const FLOW_ID = "execute_helpdesk";

	function adm_process_flow_activity(editor, node) {
		var self = this;
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		
		this.frm = editor.main.frm;

		this.helpDeskApiDefinition = {
			id: "helpDeskApiDefinition",
			name: "<%=_T('Method')%>",
			description: "<%=_T('Method')%>",
			vcc: undefined,
			sort: true,
			required: true,
			selectedMethod: undefined,
			form_group: true,
			labelClass: "required",
			required: false,
			change: function () {
				var reThis = $(this);
				$.inconcertPopup.Show(
					"<%=_T('Confirmation')%>",
					"<%=_T('All existing fields will be deleted. Confirm method change?')%>",
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function (response) {
						if (response.toLowerCase() === "yes") {
							self.__selectHelpDeskApiMethod(reThis.val());
							const $inputs = self.$get("span[class=ic_variable_icon_container");
							self.__addAutocompleteToInput($inputs);
						}
						else {
							if (self.helpDeskApiDefinition.selectedMethod) {
								reThis.val(self.helpDeskApiDefinition.selectedMethod.id);
							}
							return false;
						}
					},
					undefined,
					{
						popup_id: "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class: "confirmationOverlay"
					}
				);
			}
		};

		this.selectedMethodParameters = {
			id: "selectedMethodParameters",
			name: "<%=_T('Parameter')%>",
			description: "<%=_T('Parameter')%>",
			required: false,
			not_extended: true,
			options: undefined
		};

		this.resultDefinition = {
			id: "result",
			name: "<%=_T('Result')%>",
			description: "<%=_T('Variable name to store result (object with HelpDesk response).')%>",
			form_group: true,
			controlContainerClass: "position-relative"
		};

		this.focusControl = this.helpDeskApiDefinition;
	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[FLOW_ID] !== "function") {
		inConcert.adm_process_flow_activities[FLOW_ID] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = FLOW_ID;
	adm_process_flow_activity.prototype.name = "<%=_T('Executes HelpDesk API')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Executes HelpDesk API')%>";
	adm_process_flow_activity.prototype.group = "middleware";
	adm_process_flow_activity.prototype.image = "ico-execute-helpdesk.svg";

	adm_process_flow_activity.prototype.__isFullScreen = function () {
		return true;
	};

	adm_process_flow_activity.prototype.__load = function (callback) {
		const self = this;

		self.editor.main.getHelpDeskApi(function (success, response) {
			if (success && response.success) {
				const vccInfo = inConcert.app.get_vcc_info();
				self.helpDeskApiDefinition.vcc = vccInfo.Id;

				var helpDeskApiMethods = [];
				var responseObj = JSON.parse(response.data);
				helpDeskApiMethods = getFormattedAPI(responseObj.paths, responseObj.components);

				helpDeskApiMethods.sort(function (a, b) {
					return a.name.localeCompare(b.name);
				});
				self.helpDeskApiDefinition.options = helpDeskApiMethods;

				if (self.data.selectedMethod) {
					var methods = self.helpDeskApiDefinition.options.filter(function (method) {
						return method.id === self.data.selectedMethod;
					});
					if (methods && Array.isArray(methods) && methods.length === 1) {
						self.helpDeskApiDefinition.selectedMethod = methods[0];
					}
					else {
						self.helpDeskApiDefinition.selectedMethod = helpDeskApiMethods[0];
					}
				}
				else {
					self.helpDeskApiDefinition.selectedMethod = helpDeskApiMethods[0];
				}

				if (self.helpDeskApiDefinition.selectedMethod && self.helpDeskApiDefinition.selectedMethod.parameters) {
					self.selectedMethodParameters.options = self.helpDeskApiDefinition.selectedMethod.parameters;
				}
			}
			else {
				let alertText = inConcert.Utils.format("API HelpDesk error: {0}", response.data);
				$.inconcertPopup.Show(
					"<%=_T('API HelpDesk error')%>",
					alertText,
					$.inconcertPopup.ButtonSets.BS_OK,
					undefined,
					undefined,
					{
						popup_id: "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class: "confirmationOverlay"
					}
				);
			}
			callback();
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver1300w flowDesignerGlobalModal botActivity_execute_helpdesk";
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		const self = this;
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__makeField = function (field) {
		field = field || {};
		var self = this;
		var field_uid = inConcert.Utils.generateGuid();
		var $field = $(Mustache.render(templates.helpdesk_field, { guid: field_uid, id: field_uid, hidden: true }));

		if (field.name) {
			$field.find("[field_name]").val(field.name);
		}
		if (field.required) {
			$field.find("[field_required]").prop("checked", true);
		}
		if (field.type) {
			if (Array.isArray(field.type)) {
				$field.find("[field_type]").val("array");
			}
			else {
				$field.find("[field_type]").val(field.type);
			}

			switch ($field.find("[field_type]").val()) {
				case "array":
					$field.find("[variable_icon]").attr("title", "<%=_T('Enter an array of objects variable or an expression resulting in an array of JSON objects, as in the example')%>");
					break;
				case "integer":
					$field.find("[variable_icon]").attr("title", "<%=_T('Enter a variable or an expression resulting in an integer number')%>");
					break;
				case "number":
					$field.find("[variable_icon]").attr("title", "<%=_T('Enter a variable or an expression resulting in an integer or decimal number')%>");
					break;
				case "string":
					$field.find("[variable_icon]").attr("title", "<%=_T('Enter a variable or an expression enclosed in double quotes')%>");
					break;
				case "boolean":
					$field.find("[variable_icon]").attr("title", "<%=_T('Enter a variable or an expression resulting in a boolean value')%>");
					break;
			}
		}
		else if (self.selectedMethodParameters.options.length > 0) {
			$field.find("[field_type]").val(self.selectedMethodParameters.options[0].type);
		}
		else {
			$field.find("[field_type]").val("");
			$field.find("[field_value]")[0].disabled = true;
		}

		var fieldValue = "";
		if (self.data.fields) {
			var fields = self.data.fields.filter(function (pField) {
				return pField.name === field.name;
			});
			if (fields && Array.isArray(fields) && fields.length === 1) {
				fieldValue = fields[0].value;
			}
		}
		$field.find("[field_value]").val(fieldValue);

		return $field;
	};

	adm_process_flow_activity.prototype.__makeFields = function () {
		var self = this;
		var $fields = $(templates.helpdesk_fields);
		var $fieldsList = $fields.find("[fields]");
		var $list = [];

		$.each(self.selectedMethodParameters.options, function (_, field) {
			$list.push(self.__makeField(field));
		});
		$fieldsList.append($list);
		return $fields;
	};

	adm_process_flow_activity.prototype.__removeFields = function () {
		var $fieldsList = this.$get("[field]");
		for (var i = $fieldsList.length - 1; i >= 0; i--) {
			$fieldsList[i].remove();
		}
	};

	adm_process_flow_activity.prototype.__makeApiDocContent = function () {
		var $apiContainer = $(templates.helpdesk_api_doc_content);
		if (this.helpDeskApiDefinition.selectedMethod) {
			this.__showSelectedMethodRequest($apiContainer);
			this.__showSelectedMethodResponse($apiContainer);
		}
		return $apiContainer;
	};

	adm_process_flow_activity.prototype.__makeContent = function () {
		const $content = adm_process_flow_activity.uber.__makeContent.call(this);

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeDivFlex(
					  [
						this.__makeDivFlex(
						  [
							this.__makeComboFlex(
							  this.helpDeskApiDefinition,
							  this.data.selectedMethod
							),
							this.__makeFields(),
		  
							this.__makeInputFlex(
							  this.resultDefinition,
							  this.data[this.resultDefinition.id]
							),
						  ],
						  { class: "col-6 col-md-7" }
						),
		  
						this.__makeApiDocContent(),
					  ],
					  { class: "row" }
					),
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );
		  
		return $content;
	};

	adm_process_flow_activity.prototype.__makeBody = function () {
		var $popup = adm_process_flow_activity.uber.__makeBody.call(this);
		// var $content = $(templates.helpdesk_main_content);
		// $popup.find("[content]")
		// 	.removeAttr("content")
		// 	.append($content)
		// 	.append(this.__makeApiDocContent());
		return $popup;
	};

	adm_process_flow_activity.prototype.__clearErrors = function () {
		adm_process_flow_activity.uber.__clearErrors.call(this);
		this.$get("[entry_validation]").hide();
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function ($field, data, fields_d) {
		fields_d = fields_d || {};

		var field = {
			name: $field.find("[field_name]").val() || undefined,
			value: $field.find("[field_value]").val() || undefined,
			type: $field.find("[field_type]").val() || undefined,
			required: $field.find("[field_required]").prop("checked") || false,
		};
		//validation
		var error = null;
		if (!field.value && field.required) {
			error = "<%=_T('Must enter a value.')%>";
		}

		if (error) {
			data.error = {
				message: error,
				$control: $field
			};
		}

		//Para metodos POST y fields de tipo string, se agrega el valor por defecto "" en su caso. Para fields de tipo array se agrega valor por defecto []
		if (data.methodType.toUpperCase() === "POST") {
			if (!field.value) {
				if (field.type === "string") {
					field.value = "\"\"";
				}
				else if (field.type === "array") {
					field.value = "[]";
				}
			}
		}

		if (field.value !== undefined) {
			data.fields.push(field);
			fields_d[field.name] = true;
		}

	};

	adm_process_flow_activity.prototype.__clearErrors = function () {
		adm_process_flow_activity.uber.__clearErrors.call(this);
		this.$get("[validation='result']").hide();
	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function (data) {
		var self = this;
		data.fields = [];
		var fields_d = {};
		self.$get("[field]").each(function () {
			self.__getAndValidateField($(this), data, fields_d);
			if (data.error) {
				return false;
			}
		});
	};

	adm_process_flow_activity.prototype.__getAndValidate = function () {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		if (!this.helpDeskApiDefinition.selectedMethod) {
			data.error = {
				message: "<%=_T('Must select a method.')%>"
			};
		}

		
		if (!$(this.$get("[control='result']")).val().match(/^[_a-z]\w*$/g)) {
			data.error = {
				message: "<%=_T('Invalid Variable name')%>"
			};
			this.$get("[validation='result']").show();
		}

		if (!data.error) {
			data.vcc = this.helpDeskApiDefinition.vcc;
			data.selectedMethod = this.helpDeskApiDefinition.selectedMethod.id;
			data.methodUri = this.helpDeskApiDefinition.selectedMethod.uri;
			data.methodType = this.helpDeskApiDefinition.selectedMethod.type;
			this.__addValue(data, this.resultDefinition);
			this.__getAndValidateFields(data);
		}
		return data;
	};

	adm_process_flow_activity.prototype.__selectHelpDeskApiMethod = function (methodId) {
		var self = this;
		if (this.helpDeskApiDefinition.options) {
			var methods = this.helpDeskApiDefinition.options.filter(function (method) {
				return method.id === methodId;
			});

			if (methods && Array.isArray(methods) && methods.length === 1) {
				self.helpDeskApiDefinition.selectedMethod = methods[0];
				self.selectedMethodParameters.options = self.helpDeskApiDefinition.selectedMethod.parameters;
				self.__removeFields();
				var $fields = self.__makeFields();
				var $fieldsList = this.$get("[fields]");
				$fieldsList.append($fields[1].childNodes[0].childNodes);
				self.__showSelectedMethodRequest();
				self.__showSelectedMethodResponse();
			}
		}
	};

	adm_process_flow_activity.prototype.__showSelectedMethodRequest = function ($container) {
		if (this.helpDeskApiDefinition.selectedMethod) {
			var requestTextExample = this.helpDeskApiDefinition.selectedMethod.type.toUpperCase() + " " + this.helpDeskApiDefinition.selectedMethod.uri;
			if (this.helpDeskApiDefinition.selectedMethod.parameters.length > 0) {
				if (this.helpDeskApiDefinition.selectedMethod.type.toUpperCase() === "GET") {
					var firstParameter = true;
					this.helpDeskApiDefinition.selectedMethod.parameters.forEach(function (parameter) {
						if (firstParameter) {
							requestTextExample += "\n?";
						}
						else {
							requestTextExample += "\n&";
						}

						requestTextExample += parameter.id + "=";
						switch (parameter.type) {
							case "integer":
								requestTextExample += "0";
								break;
							case "number":
								requestTextExample += "0.0";
								break;
							case "string":
								requestTextExample += "string";
								break;
							case "boolean":
								requestTextExample += "true";
								break;
						}
						firstParameter = false;
					});
				}
				else if (this.helpDeskApiDefinition.selectedMethod.type.toUpperCase() === "POST") {
					var requestTextExampleObj = getPostParameters(this.helpDeskApiDefinition.selectedMethod.parameters);
					requestTextExample += "\n";
					requestTextExample += JSON.stringify(requestTextExampleObj, null, "\t");
				}
			}

			if ($container) {
				$container.find("[text_request]").val(requestTextExample);
			}
			else {
				this.$get("[text_request]").val(requestTextExample);
			}
		}
	};

	adm_process_flow_activity.prototype.__showSelectedMethodResponse = function ($container) {
		var responseText = "";
		responseText = JSON.stringify(this.helpDeskApiDefinition.selectedMethod.response, null, "\t");
		if ($container) {
			$container.find("[text_response]").val(responseText);
		}
		else {
			this.$get("[text_response]").val(responseText);
		}
	};

	function getPostParameters(parameters) {
		var parameterObj = {};
		parameters.forEach(function (parameter) {
			parameterObj[parameter.id] = getPostParameter(parameter);
		});
		return parameterObj;
	}

	function getPostParameter(parameter) {
		var retParameter;
		if (Array.isArray(parameter.type)) {
			retParameter = [];
			var retParameterObj = getPostParameters(parameter.type);
			retParameter.push(retParameterObj);
		}
		else {
			switch (parameter.type) {
				case "integer":
					retParameter = 0;
					break;
				case "number":
					retParameter = 0.0;
					break;
				case "string":
					retParameter = "string";
					break;
				case "boolean":
					retParameter = true;
					break;
			}
		}
		return retParameter;
	}

	function getFormattedAPI(schema, components) {
		var solvedRefObj = resolveJSONref(schema, components);
		var formattedAPI = [];
		$.each(solvedRefObj, function (_, srcMethod) {
			var dstMethod = {};
			dstMethod.type = Object.keys(srcMethod)[0];
			dstMethod.id = srcMethod[dstMethod.type].operationId;
			dstMethod.name = srcMethod[dstMethod.type].summary;
			dstMethod.uri = _;
			dstMethod.parameters = [];
			dstMethod.response = {};
			dstMethod.response.success = "boolean";
			if (dstMethod.type.toUpperCase() === "GET") {
				$.each(srcMethod[dstMethod.type].parameters, function (_, srcParameter) {
					var dstParameter = {
						id: srcParameter.name,
						name: srcParameter.name,
						type: srcParameter.schema.type,
						required: srcParameter.required
					};
					dstMethod.parameters.push(dstParameter);
				});
			}
			else if (dstMethod.type.toUpperCase() === "POST") {
				var postParameters;
				if (srcMethod[dstMethod.type].requestBody.content["application/json"].schema.type === "array") {
					postParameters = getProperties(srcMethod[dstMethod.type].requestBody.content["application/json"].schema.items.$ref.properties);
				}
				else {
					postParameters = getProperties(srcMethod[dstMethod.type].requestBody.content["application/json"].schema.$ref.properties);
				}
				$.each(postParameters, function (_, srcParameter) {
					if (Array.isArray(srcParameter)) {
						var arr = [];
						$.each(srcParameter[0], function (_, arrParameter) {
							var dstArrParameter = {
								id: _,
								name: _,
								type: arrParameter,
								required: false
							};
							arr.push(dstArrParameter);
						});
						srcParameter = arr;
					}
					var dstParameter = {
						id: _,
						name: _,
						type: srcParameter,
						required: false
					};
					dstMethod.parameters.push(dstParameter);
				});
			}
			else {
				return;	//API method ommited
			}
			if (srcMethod[dstMethod.type].responses["200"].content["application/json"].schema.type === "array") {
				dstMethod.response.data = [];
				dstMethod.response.data.push(getProperties(srcMethod[dstMethod.type].responses["200"].content["application/json"].schema.items.$ref.properties));
			}
			else {
				dstMethod.response.data = getProperties(srcMethod[dstMethod.type].responses["200"].content["application/json"].schema.$ref.properties);
			}

			formattedAPI.push(dstMethod);
		});
		return formattedAPI;
	}

	function getProperties(properties) {
		var retProperties = {};
		$.each(properties, function getProperty(_, property) {
			//Si la property es Success, la omitimos, porque ya la incluimos en el objeto response generio
			if (_ !== "Success") {
				//Si la property no tiene type, es por que es un conjunto de properties
				if (!property.type) {
					retProperties[_] = getProperties(property.$ref.properties);
				}
				//Si property.type es un array, devolvemos el conjunto de properties como un array
				else if (property.type === "array") {
					retProperties[_] = [];
					retProperties[_].push(getProperties(property.items.$ref.properties));
				}
				//Para property.type simples (integer, string, boolean) devolvemos directamente el type
				else {
					retProperties[_] = property.type;
				}
			}
		});
		return retProperties;
	}

	function resolveJSONref(schema, components) {
		var obj = JSON.parse(JSON.stringify(schema), function (key, value) {
			if (key === "$ref") {
				if (JSON.stringify(components.schemas[value.replace("#/components/schemas/", "")]).includes("$ref")) {
					return resolveJSONref(components.schemas[value.replace("#/components/schemas/", "")], components);
				}
				else {
					return components.schemas[value.replace("#/components/schemas/", "")];
				}
			}
			else {
				return value;
			}
		});
		return obj;
	}

	adm_process_flow_activity.prototype.htmlBody = function (data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_execute_helpdesk`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Executes HelpDesk API')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});


		// en algún lado el usuario debe decidir si quiere un puerto por defecto
		let puerto_por_defecto = true;

		// agrego un puerto por defecto si es necesario
		if (puerto_por_defecto) {
			let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
			let conditions = [];

			// agrego las condiciones para los errores
			conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Timeout\"" });
			conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"ChannelClosed\"" });
			conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

			ports.push({
				direction : "out", // in/out
				type : "option", // error/success/option
				option_label : data.result?data.result:"<%=_T('Default output')%>", // error/success/option
				channels: "1", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", //<%=_T('Default output')%>
					conditions: conditions,
					port_id: portId
				},
				intern_value: undefined
			});
		}
		
		// agrego una opción para los errores
		let unificar_errores = true;
		
		if (unificar_errores) {

			let existe = node?node.children.find(c => c.id.includes("_" + "Error")):undefined;
			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" +"Error");
			ports.push({
				direction : "out", // in/out
				type : "error", // error/success/option
				option_label : "Error", // error/success/option
				channels: "1", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", //Error
					conditions: [
						{ field: data.result + ".eventname", comparator: "==", value: "\""+ "Error" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "Timeout" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "ChannelClosed" +"\"" }
					],
					port_id: portId
				}
			});

		} else {

			let errores_admitidos = ["Error","Timeout","ChannelClosed"];
			errores_admitidos.forEach(error_type => {
				let existe = node?node.children.find(c => c.id.includes("_" + error_type)):undefined;
				let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" + error_type);
				ports.push({
					direction : "out", // in/out
					type : "error", // error/success/option
					option_label : error_type, // error/success/option
					channels: "1", // 0, 1 o n
					id: portId,  // le asigno un id por defecto
					preset_value: {
						description: "", //error_type
						conditions: [
							{ field: data.result + ".eventname", comparator: "==", value: "\""+ error_type +"\"" }
						],
						port_id: portId
					}
				});
			});

		}

		return ports;
	};

}());
