/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FLOW_ID = "execute_qna";
	const templates = inConcert.templates.process_flows;

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		this.frm = editor.main.frm;

		this.messageDefinition = {
			id: "message",
			name: "<%=_T('Message')%>",
			description: "<%=_T('Message to be sent to QnA Maker API.')%>",
			placeholder: "<%=_T('Enter message')%>...",
			required: true,
			variable: true,
			span: true,
			not_extended: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			controlContainerClass: "position-relative",
			form_group: true	
		};

		this.resultDefinition = {
			id: "result",
			name: "<%=_T('Result')%>",
			placeholder: "<%=_T('Enter result')%>...",
			description: "<%=_T('Variable name to store result (object with QnA Maker response).')%>",
			controlContainerClass: "position-relative",
			form_group: true
		};

		this.metadataFilterOperationType = {
			id: "operationType"
		};

		this.focusControl = this.messageDefinition;
	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[FLOW_ID] !== "function") {
		inConcert.adm_process_flow_activities[FLOW_ID] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = FLOW_ID;
	adm_process_flow_activity.prototype.name = "<%=_T('Execute QnA Maker API')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Executes a QnA Maker API method.')%>";
	adm_process_flow_activity.prototype.group = "middleware";
	adm_process_flow_activity.prototype.image = "ico-execute-qna-maker.svg";

	adm_process_flow_activity.prototype.__getMetadataFilterOperationType = function () {
		let isOr = true; // default value is OR
		if (!this.data || !this.data.metadata) {
			return isOr;
		}
		let operationType = this.data.metadata[this.metadataFilterOperationType.operationType];
		if (typeof (operationType) === "boolean") {
			return operationType;
		}
		return isOr;
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver640w flowDesignerGlobalModal botActivity_execute_qna popOverFixedHeight";
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		const self = this;
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__makeContent = function () { 
		const $content = adm_process_flow_activity.uber.__makeContent.call(this);

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.messageDefinition,
							this.data[this.messageDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeMetadata(),
					this.__makeDivFlex(
						[
						  this.__makeDivFlex(
							this.__makeInputFlex(
								this.resultDefinition,
								this.data[this.resultDefinition.id]
							),
							{ class: "col-12" }
						  ),
						],
						{ class: "row" }
					),
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );

		return $content;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function () {
		let data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		if (!data.error) {
			this.__getAndValidateFields(data);
		}
		this.__addValue(data, this.messageDefinition);
		this.__addValue(data, this.resultDefinition);
		return data;
	};

	adm_process_flow_activity.prototype.__makeField = function (field) {
		field = field || {};
		var self = this;
		var $field = $(Mustache.render(templates.qna_maker_metadata_filter_field, { guid: inConcert.Utils.generateGuid(), hidden: true }));

		$field.find("[field_name]").val(field.name || "");
		$field.find("[field_value]").val(field.value || "");
		$field.find("[field_hidden]").prop("checked", field.hidden || false);

		$field.find("[add]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $newRow = self.__makeField();
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
		});
		$field.find("[remove]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[field_name]").focus();
				}
				else {
					$row.next().find("[field_name]").focus();
				}
			}
			else {
				var $newRow = self.__makeField();
				$row.after($newRow);
				$newRow.find("[field_name]").focus();
			}
			$row.remove();
		});
		return $field;
	};

	adm_process_flow_activity.prototype.__makeMetadata = function () {
		const self = this;
		let filters = [];
		let is_or_operation_type = false;

		if (this.data.metadata) {
			filters = this.data.metadata.filters || [];
			is_or_operation_type = (this.data.metadata.operationType || "OR").toUpperCase() === "OR" ? true : false;
		}

		const $metadata = $(templates.qna_maker_metadata);
		const $filtersList = $metadata.find("[fields]");
		const $list = [];

		$.each(filters, function (_, field) {
			$list.push(self.__makeField(field));
		});
		if (!$list.length) {
			$list.push(self.__makeField());
		}
		$filtersList.append($list);

		const $operation_type = $metadata.find("[id='adm_flow_qna_maker_metadata_operation_type']");
		$operation_type.prop("checked", is_or_operation_type);

		return $metadata;
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function ($field, data) {
		let field = {
			name: $field.find("[field_name]").val() || undefined,
			value: $field.find("[field_value]").val() || undefined
		};
		//validation
		let error = null;
		if (field.name || field.value) {
			if (!field.name) {
				error = "<%=_T('Must enter a name.')%>";
			}
			else if (!field.value) {
				error = "<%=_T('Must enter a value.')%>";
			}
			let name = inConcert.Utils.htmlBodyToPlainText(field.name);
			if (name.trim().length === 0) {
				error = "<%=_T('Invalid name.')%>";
			}
			let value = inConcert.Utils.htmlBodyToPlainText(field.value);
			if (value.trim().length === 0) {
				error = "<%=_T('Invalid value.')%>";
			}
		}
		else {
			return;
		}
		if (error) {
			data.error = {
				message: error,
				$control: $field
			};
		}

		const $operation_type = $("#adm_flow_qna_maker_metadata_operation_type");
		data.metadata = data.metadata || {
			filters: [],
			operationType: $operation_type.prop("checked") ? "OR" : "AND"
		};

		data.metadata.filters.push(field);
	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function (data) {
		const self = this;
		this.$get("[field]").each(function () {
			self.__getAndValidateField($(this), data);
			if (data.error) {
				return false;
			}
		});
	};
	// agregado por Erick Gonzalez
	adm_process_flow_activity.prototype.htmlBody = function (data) {

	// agregado por Erick Gonzalez
	adm_process_flow_activity.prototype.htmlBody = function (data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_execute_qna`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Execute QnA Maker API')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});


		// en algún lado el usuario debe decidir si quiere un puerto por defecto
		let puerto_por_defecto = true;

		// agrego un puerto por defecto si es necesario
		if (puerto_por_defecto) {
			let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
			ports.push({
				direction : "out", // in/out
				type : "option", // error/success/option
				option_label : data.result?data.result:"<%=_T('Default output')%>", // error/success/option
				channels: "n", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: undefined,
				intern_value: undefined
			});
		}
		
		return ports;
	};

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Execute QnA Maker API')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function (data, node) {


		var ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction: "in", // in/out
			type: "", // error/success/option
			option_label: "Input" // error/success/option
		});


		let existe = node ? node.children.find(c => c.id.includes("_defaultOut")) : undefined;
		let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_defaultOut");
		ports.push({
			direction: "out", // in/out
			type: "option", // error/success/option
			option_label: "<%=_T('Salida')%>", // error/success/option
			channels: "1", // 1 o n
			id: portId // le asigno un id por defecto
		});
		existe = node ? node.children.find(c => c.id.includes("_defaultFailed")) : undefined;
		let portIdFailed = (existe?existe.id:inConcert.Utils.generateGuid() + "_defaultFailed");
		ports.push({
			direction: "out", // in/out
			type: "error", // error/success/option
			option_label: "<%=_T('Fallido')%>", // error/success/option
			channels: "1", // 1 o n
			id: portIdFailed // le asigno un id por defecto
		});


		return ports;
	};
}());
