/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var id = "logger";

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		this.editor = editor;
		this.frm = editor.main.frm;

		this.levelDefinition = {
			id : "level",
			name : "<%=_T('Level')%>",
			description : "<%=_T('Log level.')%>",
			required : true,
			not_extended : true,
			form_group: true,
			options : [
				{ id : "DEBUG" },
				{ id : "INFO" },
				{ id : "WARNING" },
				{ id : "ERROR" }
			]
		};

		this.expressionDefinition = {
			id : "expression",
			name : "<%=_T('Expression')%>",
			description : "<%=_T('Expression to be logged (can be a string, a number or an object).')%>",
			required : true,
			not_extended: true,
			variable: true,
			controlContainerClass: "position-relative",
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.focusControl = this.writingDefinition;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Logger')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Grants to log an expression or variable.')%>";
	adm_process_flow_activity.prototype.group = "variables";
	adm_process_flow_activity.prototype.image = "ico-logger.svg";
	
	adm_process_flow_activity.prototype.dataStyle = function(data) {
		return id + "_" + (data.level || "DEBUG");
	};
	
	adm_process_flow_activity.prototype.dataStyles = function() {
		return [
			{ id : id + "_DEBUG", imageName : "ico-logger-debug.svg" },
			{ id : id + "_INFO", imageName : "ico-logger-info.svg" },
			{ id : id + "_WARNING", imageName : "ico-logger-warning.svg" },
			{ id : id + "_ERROR", imageName : "ico-logger-error.svg" }
		];
	};

	adm_process_flow_activity.prototype.__getClassName = function() { 
		return "popOver380w flowDesignerGlobalModal botActivity_logger";
	};

	adm_process_flow_activity.prototype.GetActivityDeclaredVariables = function(/*data*/) {
		return [];
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		const self = this;
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__makeContent = function() { 
		var $content = adm_process_flow_activity.uber.__makeContent.call(this);

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeRowFlex(
					  [
						this.__makeComboFlex(
						  this.levelDefinition,
						  this.data[this.levelDefinition.id]
						),
						this.__makeInputFlex(
						  this.expressionDefinition,
						  this.data[this.expressionDefinition.id]
						),
					  ],
					  { class: "col-12" }
					),
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );		  

		return $content;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function() {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		this.__addValue(data, this.expressionDefinition);
		this.__addValue(data, this.levelDefinition);
		return data;
	};

	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_logger`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Logger')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "option", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puerto único por defecto
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option/none
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: {
				description: "",
				conditions: [],
				port_id: portId
			},
			intern_value: undefined
		});

		return ports;
	};

}());
