/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	// var format = inConcert.Utils.format;

	var id = "menu";

	/*
	 * BASE
	 */

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		this.frm = editor.main.frm;

		this.errorMessageOptions = {
			title: "<%=_T('Error message')%>",
			disclaimer: "<%=_T('Sent when client enter an invalid option or timeout expired.')%>",
			channels: { CALL: true }
		};
		if (!this.errorMessageOptions.channels[editor.main.data.Type]) {
			this.errorMessageOptions = null;
		}

		this.retriesDefinition = {
			id: "retries",
			name: "<%=_T('Number of retries')%>",
			description: "<%=_T('Number of times retry sending message and waiting for client reply.')%>",
			required: true,
			not_extended: true,
			fieldClass: "Short",
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 1,
					max: 10
				}
			},
			defaultValue: 1,
			variable: true,
			controlContainerClass: "position-relative",
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.timeoutRetryDefinition = {
			id: "timeoutRetry",
			name: "<%=_T('Timeout (seconds)')%>",
			description: "<%=_T('Number of seconds waiting to client reply after to send message.')%>",
			required: true,
			not_extended: true,
			fieldClass: "Short",
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 5,
					max: 1800
				}
			},
			variable: true,
			defaultValue: 10,
			controlContainerClass: "position-relative",
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.resultDefinition = {
			id: "result",
			name: "<%=_T('Result')%>",
			required : true, 
			validate : {
				type : "id",
				message : "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>"
			},
			description: "<%=_T('Variable name to store result: object { status : boolean, event : object|nil with message event, eventname : NewChatMessage|ChannelClosed|Timeout indicating event name (ChannelClosed if client closes channel while is waiting to reply and Timeout if timeouts waiting message) }.')%>",
			form_group: true,
			labelClass: "required"
		};

		this.type = editor.main.data.Type;
		this.messageId = "message";
		this.messagesEditor = new inConcert.adm_process_flow_messages[this.type.toLowerCase()](this);

		this.focusControl = this.messageDefinition;
	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Menu')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Send a list of options to be selected by client.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.channels = {
		"CALL": true
	};
	adm_process_flow_activity.prototype.image = "ico-menu.svg";

	adm_process_flow_activity.prototype.dataStyle = function () {
		return "message_menu";
	};

	adm_process_flow_activity.prototype.dataStyles = function () {
		return [
			{
				id: "message_menu",
				imageName: "ico-menu.svg"
			}
		];
	};

	adm_process_flow_activity.prototype.GetMigrationData = function (data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetMigrationData(messages, type, {}, data);
	};

	adm_process_flow_activity.prototype.__hasKeyboardEvents = function () {
		return false;
	};

	adm_process_flow_activity.prototype.__isFullScreen = function () {
		return true;
	};

	adm_process_flow_activity.prototype.__makeFormContent = function ($content) {
		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  this.__makeInputFlex(
					this.retriesDefinition,
					this.data[this.retriesDefinition.id]
				  ),
				  { class: "col-6" }
				),
				this.__makeDivFlex(
				  this.__makeInputFlex(
					this.timeoutRetryDefinition,
					this.data[this.timeoutRetryDefinition.id]
				  ),
				  { class: "col-6" }
				),
			  ],
			  { class: "row" }
			),
		  
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  this.__makeInputFlex(
					this.resultDefinition,
					this.data[this.resultDefinition.id]
				  ),
				  { class: "col-12" }
				),
			  ],
			  { class: "row" }
			),
		  
			this.__makeErrorMessage(),
			/////////////////////////////////////////////////////////////////////////////////////////////
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeComboFlex(
					  this.voiceDefinition,
					  this.data.message && this.data.message[this.voiceDefinition.id]
					),
				  ],
				  { class: "col-12" }
				),
			  ],
			  { class: "row" }
			),
		  
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeComboFlex(
					  this.recognitionEngineDefinition,
					  this.data.message &&
						this.data.message[this.recognitionEngineDefinition.id]
					),
				  ],
				  { class: "col-6" }
				),
				this.__makeDivFlex(
				  [
					this.__makeComboFlex(
					  this.RecognitionLanguageDefinition,
					  this.data.message &&
						this.data.message[this.RecognitionLanguageDefinition.id]
					),
				  ],
				  { class: "col-6" }
				),
			  ],
			  { class: "row" }
			),
		  
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeInput(
					  this.voiceDetectOnPlaySilDurDefinition,
					  this.data.message &&
						this.data.message[this.voiceDetectOnPlaySilDurDefinition.id]
					),
				  ],
				  { class: "col-6 col-lg-4" }
				),
		  
				this.__makeDivFlex(
				  [
					this.__makeInput(
					  this.voiceDetectOnPlayMinDurDefinition,
					  this.data.message &&
						this.data.message[this.voiceDetectOnPlayMinDurDefinition.id]
					),
				  ],
				  { class: "col-6 col-lg-4" }
				),
		  
				this.__makeDivFlex(
				  [
					this.__makeInput(
					  this.voiceDetectOnPlayMaxDurDefinition,
					  this.data.message &&
						this.data.message[this.voiceDetectOnPlayMaxDurDefinition.id]
					),
				  ],
				  { class: "col-6 col-lg-4" }
				),
			  ],
			  { class: "row" }
			)
		  );		  
		  
		return $content;
	};

	adm_process_flow_activity.prototype.__makeContent = function () {
		let $content = this.__makeFormContent(adm_process_flow_activity.uber.__makeContent.call(this));
		return this.messagesEditor.Make($content, this.data[this.messageId], {
			types: ["menu"]
		});
	};

	adm_process_flow_activity.prototype.__makeErrorMessage = function () {
		if (!this.errorMessageOptions) {
			return null;
		}
		this.errorMessageEditor = new inConcert.adm_process_flow_messages_embeed(
			this.messagesEditor, this.errorMessageOptions);

		let errorMessageObj = this.data.errorMessage;
		if (errorMessageObj && errorMessageObj.msgtype && !this.errorMessageOptions.message_types.find(msgtype => msgtype.id === errorMessageObj.msgtype)) {
			errorMessageObj = {};
		}
		//Guardo este objeto para poder referenciarlo en el AfterSelectType
		this.errorMessageObj = this.errorMessageEditor.Make(errorMessageObj || {});
		return this.errorMessageObj;
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function () {
		if (this.errorMessageOptions) {
			this.errorMessageEditor.AfterAppendBody();
		}
		this.messagesEditor.AfterAppendBody();
	};

	adm_process_flow_activity.prototype.__getAndValidateForm = function (data) {
		this.__addValue(data, this.retriesDefinition);
		this.__addValue(data, this.timeoutRetryDefinition);
		this.__addValue(data, this.resultDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateAsync = function (data, callback) {
		let self = this;
		self.__getAndValidateForm(data);
		self.__getAndValidateMessage(data, function () {
			self.__getAndValidateErrorMessage(data, function () {
				callback(data);
			});
		});
	};

	adm_process_flow_activity.prototype.__getAndValidateMessage = function (data, callback) {
		this.messagesEditor.AddValue(data, this.messageId, function () {
			callback(data);
		});
	};

	adm_process_flow_activity.prototype.__getAndValidateErrorMessage = function (data, callback) {
		if (!this.errorMessageEditor) {
			return callback(data);
		}
		data.errorMessage = {};
		this.errorMessageEditor.AddValue(data.errorMessage);
		if (this.getAndValidateErrorMessage && typeof this.getAndValidateErrorMessage === "function") {
			this.getAndValidateErrorMessage(data.errorMessage);
		}
		this.errorMessageEditor.AddValueAsync(data.errorMessage, function () {
			data.error = data.error || data.errorMessage.error;
			if ($.isEmptyObject(data.errorMessage)) {
				data.errorMessage = null;
			}
			callback();
		});
	};

	/*
	 * CALL
	 */

	function adm_process_flow_activity_call(editor, node) {
		let self = this;
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);

		// this.ttsEngine;
		// this.ttsVoices;
		// this.recognitionLanguages;
		this.recognitionEngines = [];

		this.errorMessageOptions.message_types = [
			{
				id: "attachment",
				name: "<%=_T('Attachment')%>"
			}
		];

		this.voiceDefinition = {
			id: "voiceId",
			name: "<%=_T('Voice')%>",
			description: "<%=_T('TTS voice to use.')%>",
			not_extended: false,
			fieldClass: "Short",
			emptyLabel: "<%=_T('')%>",
			form_group: true
		};

		this.recognitionEngineDefinition = {
			id: "asrEngine",
			name: "<%=_T('Recognition Engine')%>",
			description: "<%=_T('Recognition Engine.')%>",
			not_extended: false,
			fieldClass: "Short",
			emptyLabel: "<%=_T('DTMF digits')%>",
			form_group: true,
			change: function () {
				let currentRecognitionEngineValue = self.__getValue(self.recognitionEngineDefinition) || "";
				self.__toggleControl(self.RecognitionLanguageDefinition, currentRecognitionEngineValue !== "");
				self.RecognitionLanguageDefinition.required = (currentRecognitionEngineValue !== "");
				self.__toggleBargein(currentRecognitionEngineValue === "speechmatic" && self.ExistsAttachmentSelectedOption(), currentRecognitionEngineValue === "speechmatic");
				if (self.messagesEditor && self.messagesEditor.typeEditor && self.messagesEditor.typeEditor.__toggleOptionsAsrRecognize && $.isFunction(self.messagesEditor.typeEditor.__toggleOptionsAsrRecognize)) {
					self.messagesEditor.typeEditor.__toggleOptionsAsrRecognize(currentRecognitionEngineValue === "");
				}
			}
		};

		this.RecognitionLanguageDefinition = {
			id: "recognitionLanguage",
			name: "<%=_T('Recognition Language')%>",
			description: "<%=_T('Recognition Language to use.')%>",
			not_extended: false,
			fieldClass: "Short",
			sort: true,
			emptyLabel: "<%=_T('')%>",
			form_group: true
		};

		this.voiceDetectOnPlayMinDurDefinition = {
			id: "mindur",
			name: "<%=_T('Min voice duration (ms)')%>",
			description: "<%=_T('Minimum voice duration to start voice detection while playback (milliseconds).')%>",
			required: true,
			not_extended: true,
			fieldClass: "Short",
			form_group: true,
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 100,
					max: 1000
				}
			},
			defaultValue: 100
		};

		this.voiceDetectOnPlayMaxDurDefinition = {
			id: "maxdur",
			name: "<%=_T('Max voice duration (ms)')%>",
			description: "<%=_T('Maximum voice duration to end voice detection while playback (milliseconds).')%>",
			required: true,
			not_extended: true,
			fieldClass: "Short",
			form_group: true,
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 1200,
					max: 30000
				}
			},
			defaultValue: 30000
		};

		this.voiceDetectOnPlaySilDurDefinition = {
			id: "sildur",
			name: "<%=_T('Silence duration (ms)')%>",
			description: "<%=_T('Silence duration to end voice detection (milliseconds).')%>",
			required: true,
			not_extended: true,
			fieldClass: "Short",
			form_group: true,
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 500,
					max: 5000
				}
			},
			defaultValue: 1000
		};
	}

	inConcert.Utils.inherit(adm_process_flow_activity_call, adm_process_flow_activity);
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);

	adm_process_flow_activity_call.prototype.__load = function (callback) {
		const self = this;

		this.editor.main.LoadTtsAsrEngines(function (success, ttsasrEngines) {
			self.ttsasrEngines = success ? ttsasrEngines : undefined;
			if (self.ttsasrEngines && $.isArray(self.ttsasrEngines)) {
				self.ttsasrEngines.forEach(function (engine) {
					if ((engine.providerType === "TTS" || engine.providerType === "TTS-ASR") && engine.voices && $.isArray(engine.voices) && engine.voices.length > 0) {
						self.ttsEngine = engine.engineId;
						self.ttsVoices = [];
						engine.voices.forEach(function (voice) {
							let voice_to_show = {
								id: voice.id,
								name: voice.LanguageName + " - " + voice.Variant + " - " + voice.Gender + " - " + voice.VoiceName
							};
							self.ttsVoices.push(voice_to_show);
						});
					}
					if ((engine.providerType === "ASR" || engine.providerType === "TTS-ASR") && engine.grammars && $.isArray(engine.grammars) && engine.grammars.length > 0) {
						self.recognitionEngines.push({ id: engine.engineId, name: engine.engineDesc });
						if (engine.recognitionLanguages && $.isArray(engine.recognitionLanguages) && engine.recognitionLanguages.length > 0) {
							self.recognitionLanguages = [];
							engine.recognitionLanguages.forEach(function (recognitionLanguage) {
								let recognitionLanguage_to_show = {
									id: recognitionLanguage.id,
									name: recognitionLanguage.LanguageName
								};
								self.recognitionLanguages.push(recognitionLanguage_to_show);
							});
						}
					}
				});

				self.isTtsEnabled = (self.ttsVoices && $.isArray(self.ttsVoices) && self.ttsVoices.length > 0);
				self.isAsrEnabled = (self.recognitionLanguages && $.isArray(self.recognitionLanguages) && self.recognitionLanguages.length > 0);

				if (self.isTtsEnabled) {
					self.errorMessageOptions.message_types.push({ id: "text", name: "<%=_T('Text')%>" });
				}

				self.voiceDefinition.options = self.ttsVoices;
				self.recognitionEngineDefinition.options = self.recognitionEngines;
				self.RecognitionLanguageDefinition.options = self.recognitionLanguages || [];
			}
			callback();
		});
	};

	adm_process_flow_activity_call.prototype.__getClassName = function() { 
		return "popOver1300w flowDesignerGlobalModal tabsPopOver botActivity_menu";
	};

	adm_process_flow_activity.prototype.GetActivityDeclaredVariables = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetActivityDeclaredVariables(type, data, editor);

	};

	adm_process_flow_activity_call.prototype.__afterAppendBody = function () {
		adm_process_flow_activity_call.uber.__afterAppendBody.call(this);

		if (this.isTtsEnabled) {
			this.AfterSelectType();
		}
		else {
			this.__hideControl(this.voiceDefinition);
		}

		if (this.isAsrEnabled) {
			this.recognitionEngineDefinition.change();
		}
		else {
			this.__hideControl(this.recognitionEngineDefinition);
			this.__hideControl(this.RecognitionLanguageDefinition);
			this.__toggleBargein(false);
		}

		const $inputs = this.$get("span[class=ic_variable_icon_container");
		this.__addAutocompleteToInput($inputs);

	};

	adm_process_flow_activity_call.prototype.__getAndValidateFormMessage = function (message) {
		//var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		this.__addValue(message, this.voiceDefinition);
		this.__addValue(message, this.recognitionEngineDefinition);
		this.__addValue(message, this.RecognitionLanguageDefinition);
		this.__addValue(message, this.voiceDetectOnPlaySilDurDefinition);
		this.__addValue(message, this.voiceDetectOnPlayMinDurDefinition);
		this.__addValue(message, this.voiceDetectOnPlayMaxDurDefinition);
		message.ttsEngine = this.ttsEngine;
		let currentRecognitionEngineValue = this.__getValue(this.recognitionEngineDefinition);

		if (this.isAsrEnabled && currentRecognitionEngineValue === "speechmatic") {
			message.promptOptions = {};
			let sildur = this.__getValue(this.voiceDetectOnPlaySilDurDefinition);
			if (sildur) {
				message.promptOptions.sildur = sildur;
			}
			if(this.ExistsAttachmentSelectedOption()) {
				let mindur = this.__getValue(this.voiceDetectOnPlayMinDurDefinition);
				if (mindur) {
					message.promptOptions.mindur = mindur;
				}
				let maxdur = this.__getValue(this.voiceDetectOnPlayMaxDurDefinition);
				if (maxdur) {
					message.promptOptions.maxdur = maxdur;
				}
			}
		}
	};

	adm_process_flow_activity_call.prototype.getAndValidateErrorMessage = function (errorMessage) {
		if (errorMessage.msgtype === "text" && this.isTtsEnabled) {
			errorMessage.voiceId = this.__getValue(this.voiceDefinition);
		}
	};

	adm_process_flow_activity_call.prototype.AfterSelectType = function () {
		let existTextOption = false;
		// Si la opcion variable esta checked, es posible que alguna de las opciones indicadas en la variable sea de texto, pero como no lo sabemos, dejamos habilitado el combo de voces
		if (this.$get("[menu_type='variable']").prop("checked")) {
			existTextOption = true;
		}
		else {
			for (let menuOption of this.$get("[control='msgtype']")) {
				if (menuOption.value === "text") {
					if (menuOption === this.errorMessageObj.find("[control='msgtype']")[0]) {
						existTextOption = this.errorMessageObj.find(".formCheckbox").prop("checked");
					}
					else {
						existTextOption = true;
					}
				}
			}
		}

		if (existTextOption) {
			this.__showControl(this.voiceDefinition);
		}
		else {
			this.__hideControl(this.voiceDefinition);
		}

		let currentRecognitionEngineValue = this.__getValue(this.recognitionEngineDefinition);
		this.__toggleBargein(currentRecognitionEngineValue === "speechmatic" && this.ExistsAttachmentSelectedOption(), currentRecognitionEngineValue === "speechmatic");
	};

	adm_process_flow_activity_call.prototype.ExistsAttachmentSelectedOption = function () {
		// Si la opcion variable esta checked, es posible que alguna de las opciones indicadas en la variable sea de attachment, pero como no lo sabemos, devolvemos true
		if (this.$get("[menu_type='variable']").prop("checked")) {
			return true;
		}
		else {
			for (let menuOption of this.$get(".flowDesignerMessagesMenuOption.selected")) {
				let msgTypeObj = menuOption.querySelector("[control='msgtype']");
				if (msgTypeObj.value === "attachment") {
					return true;
				}
			}
		}
		return false;
	};

	adm_process_flow_activity_call.prototype.__toggleBargein = function (show, showSilDur) {
		var objBargeinRow = this.$get("[form_subtitle='bargein_subtitle_id']").parent();
		objBargeinRow.toggle(show);
		this.$get("#bargein_disclaimer_id").toggle(show);

		if (show) {
			this.__showControl(this.voiceDetectOnPlayMinDurDefinition);
			this.__showControl(this.voiceDetectOnPlayMaxDurDefinition);
		} else {
			this.__hideControl(this.voiceDetectOnPlayMinDurDefinition);
			this.__hideControl(this.voiceDetectOnPlayMaxDurDefinition);
		}
		if(showSilDur) {
			this.__showControl(this.voiceDetectOnPlaySilDurDefinition);
		}
		else {
			this.__hideControl(this.voiceDetectOnPlaySilDurDefinition);
		}
	};

	adm_process_flow_activity.prototype.htmlBody = function (data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${id}`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Menu')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function (data, node) {
		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "option", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puerto único por defecto
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option
			option_label : data.result?data.result:"<%=_T('Default output')%>", // error/success/option
			channels: "n", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: undefined, // undefined para que permita al usuario crear sus propias condiciones
			intern_value: undefined
		});
		
		return ports;
	};
}());
