/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flows;
	var id = "notify_data";

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		this.focusFirst = true; 
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Agent Data')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Add or update data to be used by agent integration.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.image = "ico-notify_data.svg";
	adm_process_flow_activity.prototype.deprecated = "<%=_T('Must use Manage Process Data activity.')%>"; //2020-06-16

	adm_process_flow_activity.prototype.__makeField = function(field) {
		field = field || {};
		var self = this;
		var $field = $(Mustache.render(templates.report_field, { guid : inConcert.Utils.generateGuid(), hidden : true }));

		$field.find("[field_name]").val(field.name || "");
		$field.find("[field_value]").val(field.value || "");
		$field.find("[field_hidden]").prop("checked", field.hidden || false);

		$field.find("[add]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent();
			var $newRow = self.__makeField();
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
		});
		$field.find("[remove]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent();
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[field_name]").focus();
				}
				else {
					$row.next().find("[field_name]").focus();
				}
			}
			else {
				var $newRow = self.__makeField();
				$row.after($newRow);
				$newRow.find("[field_name]").focus();
			}
			$row.remove();
		});
		return $field;
	};

	adm_process_flow_activity.prototype.__makeFields = function() {
		var self = this;
		var fields = self.data.fields || [];
		var $fields = $(templates.report_fields);
		var $fieldsList = $fields.find("[fields]");
		var $list = [];
		
		$.each(fields, function(_, field) {
			$list.push(self.__makeField(field));
		});
		if (!$list.length) {
			$list.push(self.__makeField());
		}
		$fieldsList.append($list);
		return $fields;
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		var $content = adm_process_flow_activity.uber.__makeContent.call(this);
		$content.unshift(this.__makeFields());
		return $content;
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function($field, data, fields_d) {
		fields_d = fields_d || {};
		var field = {
			name : $field.find("[field_name]").val() || undefined,
			value : $field.find("[field_value]").val() || undefined,
			hidden : $field.find("[field_hidden]").is(":checked")
		};
		//validation
		var error = null;
		if (!field.name) {
			error = "<%=_T('Must enter a name.')%>";
		}
		else if (fields_d[field.name]) {
			error = "<%=_T('Name must be unique.')%>";
		}
		else if (!field.value) {
			error = "<%=_T('Must enter a value.')%>";
		}
		if (error) {
			data.error = {
				message : error,
				$control : $field
			};
		}
		data.fields.push(field);
		fields_d[field.name] = true;
	};

	adm_process_flow_activity.prototype.__clearErrors = function() {
		adm_process_flow_activity.uber.__clearErrors.call(this);
		this.$get("[entry_validation]").hide();
	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function(data) {
		var self = this;
		data.fields = [];
		var fields_d = {};
		self.$get("[field]").each(function() {
			self.__getAndValidateField($(this), data, fields_d);
			if (data.error) {
				return false;
			}
		});
		if (!data.error && !data.fields.length) {
			data.error = {
				message : "<%=_T('Must define almost one field.')%>"
			};
		}
	};

	adm_process_flow_activity.prototype.__getAndValidate = function() {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		if (!data.error) {
			this.__getAndValidateFields(data);
		}
		return data;
	};

}());
