/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, ace, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	var id = "send_and_wait_chat_message";

/*
 * BASE
 */

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		
		var self = this;
		self.drawVersion = editor.main.__getVersion();

		this.errorMessageOptions = {
			title : "<%=_T('Error message')%>",
			disclaimer : "<%=_T('Sent when client enter an invalid option or timeout expired.')%>",
			channels : { CALL : true }
						   
			  
		};

		this.errorMessageOptions.channels = {
			CALL : true, 
			CHAT : true, 
			FACEBOOK_BOT : true,
			INSTAGRAM_BOT : true,
			TWITTER_BOT : true,
			//MAIL : true,
			//SMS : true,
			WHATSAPP : true 
		};
        
		if (!this.errorMessageOptions.channels[editor.main.data.Type]) {
			this.errorMessageOptions = null;
		} else {
            if(editor.main.data.Type !== "CALL")
            {
                this.errorMessageOptions.message_types = self.errorMessageOptions.message_types || [];
                this.errorMessageOptions.message_types.push({id : "text", name : "<%=_T('Text')%>"});
            }
        }
  
		this.output_options = [];
        this.interactive_buttons = [];

		if (self.drawVersion === "2") {

            this.quick_reply_name_prefix = "Quickreply: "; // no se debe traducir, porque sino genera un error cuando editan el flujo usando usaurios con lenguajes diferentes

			this.retriesDefinition = {
				id : "retries",
				name : "<%=_T('Number of retries')%>",
				description: "<%=_T('Number of times retry sending message and waiting for client reply.')%>",
				not_extended : true,
				required : true,
				fieldClass : "col_4",
				options : this.__getRange(1, 10),
				emptyLabel : "<%=_T('Nothing')%>",
				defaultValue: 1,
                form_group: true
			};
			
			this.timeoutRetryDefinition = {
				id : "timeoutRetry",
				name : "<%=_T('Timeout (seconds)')%>",
				description: "<%=_T('Number of seconds waiting to client reply after to send message.')%>",
				not_extended : true,
				required : true,
				fieldClass : "col_4",
				options : this.__getRange(5, 1800),
                validate : {
					range : {
						type : "integer",
						allowVariables : true,
						min : 5,
						max : 1800
					}
				},
				emptyLabel : "<%=_T('Nothing')%>",
				defaultValue: 10,
                form_group: true
			};
			
			this.cognitiveServicesDefinition = { 
				id : "cognitiveServicesApp",
				name : "<%=_T('Select Cognitive Services App')%>",
				description : "<%=_T('Select Cognitive Services App')%>",
				not_extended : false,
                required : true,
				emptyLabel : "<%=_T('')%>",
				change: function () {
					self.__selectCognitiveService($(this).val());
				},
                form_group: true
			};

			this.scoringDefinition = {
				id         : "luis_scoring",
				name       : "<%=_T('Scoring')%>"
			};

            this.useCognitiveServicesDefinition = {
                id: "useCognitiveServicesApp",
                name: "<%=_T('Use Cognitive Services Api')%>",
                description: "<%=_T('Use Cognitive Services')%>",
                not_extended: true,
                form_group: false,
                envolving_label_class: "form-check-label form-check-label-title",
                envolving_label: true,
                without_label: true,
                controlContainerClass: "form-check mt-0",
                checkBoxClass: "form-check-input",
                change: function () {
                    self.__changeUseCognitiveServices($(this).prop("checked"), true);
                }
            };

            this.validationDefinition = {
				id : "validation_type",
				name : "<%=_T('Validation')%>",
				description: "<%=_T('Validation')%>",
				not_extended : true,
				required : false,
                options : [
                    { id: "only-alphanumeric", name: "<%=_T('Only alphanumeric')%>" },
                    { id: "only-letters", name: "<%=_T('Only letters')%>" },
                    { id: "only-numbers", name: "<%=_T('Only numbers')%>" },
                    //{ id: "phone", name: "<%=_T('Phone')%>" },
                    { id: "email", name: "<%=_T('E-mail')%>" },
                    { id: "name-and-surname", name: "<%=_T('Name and Surname')%>" },
                    { id: "custom", name: "<%=_T('Custom')%>" }
                ],
                change : function() {
                    self.__changeValidationDefinition($(this).val());
                },
                form_group: true
			};

            this.resultDefinition = {
                id : "result",
                name : "<%=_T('Result')%>",
                placeholder : "<%=_T('Enter result')%>...",
                required : true, 
                validate : {
                    type : "id",
                    message : "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>"
                },
                description : "<%=_T('Variable name to store result: object { status : boolean, event : object|nil with message event, eventname : NewChatMessage|ChannelClosed|Timeout indicating event name (ChannelClosed if client closes channel while is waiting to reply and Timeout if timeouts waiting message) }.')%>",
                form_group: true
            };
			
		} else {

			this.retriesDefinition = {
				id : "retries",
				name : "<%=_T('Number of retries')%>",
				description : "<%=_T('Number of times retry sending message and waiting for client reply.')%>",
				required : true,
				not_extended : true,
				fieldClass : "col_4",
				validate : {
					range : {
						type : "integer",
						allowVariables : true,
						min : 1,
						max : 10
					}
				},
				//variable : true,
				defaultValue : 1,
                form_group: true
			};

			this.timeoutRetryDefinition = {
				id : "timeoutRetry",
				name : "<%=_T('Timeout (seconds)')%>",
				description : "<%=_T('Number of seconds waiting to client reply after to send message.')%>",
				required : true,
				not_extended : true,
				fieldClass : "col_4",
				validate : {
					range : {
						type : "integer",
						allowVariables : true,
						min : 5,
						max : 1800
					}
				},
				//variable : true,
				defaultValue : 10,
                form_group: true
			};
			
			this.cognitiveServicesDefinition = {
				id : "cognitiveServicesApp",
				name : "<%=_T('Cognitive Services App')%>",
				description : "<%=_T('Cognitive Services App.')%>",
				not_extended : false,
				fieldClass : "",
				emptyLabel : "<%=_T('')%>",
                form_group: true
			};

            this.resultDefinition = {
                id : "result",
                name : "<%=_T('Result')%>",
                placeholder : "<%=_T('Enter result')%>...",
                required : true, 
                validate : {
                    type : "id",
                    message : "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>"
                },
                description : "<%=_T('Variable name to store result: object { status : boolean, event : object|nil with message event, eventname : NewChatMessage|ChannelClosed|Timeout indicating event name (ChannelClosed if client closes channel while is waiting to reply and Timeout if timeouts waiting message) }.')%>",
                form_group: true
            };

		}
		
		this.type = editor.main.data.Type;
		this.messageId = "message";
		this.messagesEditor = new inConcert.adm_process_flow_messages[this.type.toLowerCase()](this);
		
		this.focusControl = this.messageDefinition;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

    adm_process_flow_activity.prototype.__changeUseCognitiveServices = function(checked, fromControl) {

        if (this.drawVersion === "2") {
            if (checked) { // muestro el control
                this.cognitiveServicesDefinition.required = true;
				let cognitiveServicesDefinitionValue = this.__getValue(this.cognitiveServicesDefinition);

                if (cognitiveServicesDefinitionValue === "" || cognitiveServicesDefinitionValue === null || cognitiveServicesDefinitionValue === " ") {
                    this.__setValue(this.cognitiveServicesDefinition, undefined);
                }

                this.__showControl(this.cognitiveServicesDefinition);
                $("#luis_scoring_slider_title").show();
                $("#luis_scoring_slider_control").show();
                $("#adm_flow_cognitive_services_card_header").show();
                $("#cognitive_services_card_header").show();                
            } else { //oculto el control
                this.cognitiveServicesDefinition.required = false;
                this.__hideControl(this.cognitiveServicesDefinition);
                this.__setValue(this.cognitiveServicesDefinition, undefined);
                $("#luis_scoring_slider_title").hide();
                $("#luis_scoring_slider_control").hide();
                $("#adm_flow_cognitive_services_card_header").hide();
                $("#cognitive_services_card_header").hide();

                if(fromControl) {
                    this.__selectCognitiveService("");
                }
            }
        }   
	};

    adm_process_flow_activity.prototype.__changeValidationDefinition = function(value, delayed) {
        const self = this;
        if (delayed) {
            setTimeout(function() { 
                if (self.__changeValidationDefinitionAction) {
                    self.__changeValidationDefinitionAction(value);
                }                
            }, 250);
        } else {
            self.__changeValidationDefinitionAction(value);
        }
	};

    adm_process_flow_activity.prototype.__changeValidationDefinitionAction = function(value) {

        let existMinDigits = false;
        
        if (this.minDigitsDefinition) {
            let minDigits = this.__getValue(this.minDigitsDefinition);
            if (minDigits && minDigits !== "1") {                
                existMinDigits = true;
            }
        }

        this.__toggleControl(this.validationDefinition, !existMinDigits);

        if (this.drawVersion === "2") {
            $("#adm_flow_validation_custom_disclaimer").show();

            if((value === "" || value === undefined || existMinDigits) && this.data && this.data.message) {
                this.data.message.validation = undefined;
            }

            if (existMinDigits) {
                $("#adm_flow_validation_custom_disclaimer").hide();
            } else if (value === "only-alphanumeric") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The variable must contain only alphanumeric characters')%>");
            } else if (value === "only-letters") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The variable must contain only letters')%>");
            } else if (value === "only-numbers") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The variable must contain only numbers')%>");
            } else if (value === "phone") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The Phone variable to be validated must contain a phone number')%>");
            } else if (value === "email") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The e-mail variable to be validated must have a valid e-mail format')%>");
            } else if (value === "name-and-surname") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The Name variable to be validated must contain the First Name and Last Name fields')%>");
            } else if (value === "custom") {
                $("#adm_flow_validation_custom_disclaimer").find("[disclaimer]").text("<%=_T('The Custom variable validation must be set in the code block')%>");
            } else {
                $("#adm_flow_validation_custom_disclaimer").hide();
            }

            if (value === "custom" && !existMinDigits) {
                $(".flowDesignerSendAndWaitValidation").show();
            } else {
                $(".flowDesignerSendAndWaitValidation").hide();
            }
        }
	};

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Send message and wait reply')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Send a chat message and wait for client reply.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.channels = { 
		"CALL" : true, 
		"CHAT" : true, 
		"FACEBOOK_BOT" : true,
		"INSTAGRAM_BOT" : true,
		"TWITTER_BOT" : true,
		"MAIL" : true,
		"SMS" : true,
		"WHATSAPP" : true
	};
	adm_process_flow_activity.prototype.image = "ico-send-wait.svg";

	adm_process_flow_activity.prototype.GetMigrationData = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetMigrationData(messages, type, {}, data);
	};

	adm_process_flow_activity.prototype.GetActivityDeclaredVariables = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetActivityDeclaredVariables(type, data, editor);
	};
	
	adm_process_flow_activity.prototype.dataStyle = function(data) {
		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			return id + "_" + type;
		}
		return null;
	};
	
	adm_process_flow_activity.prototype.dataStyles = function(type) {
		var types = inConcert.adm_process_flow_messages_type[type.toLowerCase() + "_sorted"] || [];
		var styles = [];
		$.each(types, function(_, type) {
			styles.push({
				id : id + "_" + type,
				imageName : format("ico-send-wait-{0}-message.svg", type)
			});
		});
		["image", "video", "audio", "file"].forEach(function(fileType) {
			styles.push({
				id : id + "_" + fileType,
				imageName : format("ico-send-wait-{0}-message.svg", fileType)
			});
		});
		return styles;
	};
	
	adm_process_flow_activity.prototype.__hasKeyboardEvents = function() {
		return false;
	};
	
	adm_process_flow_activity.prototype.__isFullScreen = function() {
		return false;
	};

	// Acá se arma el "markup" de los formularios_
	adm_process_flow_activity.prototype.__makeFormContent = function($content) {

		if (this.drawVersion === "2" && this.type === "CALL") {

			this.cognitiveServicesDefinition.fieldClass = "mb_10"; 
			this.scoringDefinition.fieldClass = "col_6"; 

			if (!this.data.message) {
				this.data.message = {};
			}

            $content.unshift(this.__makeScoring(this.data[this.scoringDefinition.id]));
            $content.unshift(this.__makeRowFlex(this.__makeComboFlex(this.cognitiveServicesDefinition, this.data[this.cognitiveServicesDefinition.id])));
            $content.unshift($(inConcert.templates.process_flows.cognitive_services_card_header));

            $content.unshift(
                this.__makeHiddenDivFlex(
                  [
                    this.__makeDivFlex(
                      this.__makeDivFlex(
                        this.__makeCheckbox(
                          this.useCognitiveServicesDefinition,
                          this.data[this.useCognitiveServicesDefinition.id]
                        ),
                        { class: "form-group" }
                      ),{ class: "col-6" }
                    )
                  ],{ class: "row" }
                )
            );

			// Campo resultado caso CALL
            $content.push(this.__makeDivFlex(this.__makeDivFlex(this.__makeInputFlex(this.resultDefinition, this.data[this.resultDefinition.id]),{ class: "col-12" }),{ class: "row" }));

            this.validationDefinition.options = [
                { id: "custom", name: "<%=_T('Custom')%>" }
            ];

            $content.push(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.minDigitsDefinition, this.data.message[this.minDigitsDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.maxDigitsDefinition, this.data.message[this.maxDigitsDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.retriesDefinition, this.data[this.retriesDefinition.id])
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            );

            $content.push(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.timeoutRetryDefinition, this.data[this.timeoutRetryDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.endDigitDefinition, this.data.message[this.endDigitDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.validationDefinition, this.data[this.validationDefinition.id])
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            );

            $content.push(this.__makeValidationEditor(this.data.validation_type === "custom"));

            $content.push(Mustache.render(inConcert.templates.process_flows.send_and_wait_validation, {
                hidden: ((this.data.validation_type || (this.data.minDigits && this.data.minDigits !== "1"))?false:true)
            }));

            this.__changeValidationDefinition(this.data.validation_type, true);

		} else if (this.drawVersion === "2" && this.type !== "CALL") { 
			
			this.cognitiveServicesDefinition.fieldClass = "mb_10"; 
			this.scoringDefinition.fieldClass = "col_6"; 

			if (!this.data.message) {
				this.data.message = {};
			}

            $content.unshift(this.__makeDivFlex(this.__makeDivFlex(this.__makeInputFlex(this.resultDefinition, this.data[this.resultDefinition.id]),{ class: "col-12" }),{ class: "row" }));
			
            $content.push(
                this.__makeDivFlex(
                  [
                    (this.type !== "MAIL"?(this.__makeDivFlex(this.__makeComboFlex(this.validationDefinition,this.data[this.validationDefinition.id]),{ class: "col-4" })):undefined),
                    this.__makeDivFlex(this.__makeInputFlex(this.retriesDefinition,this.data[this.retriesDefinition.id]),{ class: "col-4" }),
                    this.__makeDivFlex(this.__makeInputFlex(this.timeoutRetryDefinition,this.data[this.timeoutRetryDefinition.id]),{ class: "col-4" }),
                  ],{ class: "row" }
                )
            );

            $content.push(this.__makeValidationEditor(this.data.validation_type === "custom"));

            $content.push(Mustache.render(inConcert.templates.process_flows.send_and_wait_validation, {
                hidden: (this.data.validation_type?false:true)
            }));

            // Cognitive Services
            $content.push(
                this.__makeHiddenDivFlex(
                  [
                    this.__makeDivFlex(
                      this.__makeDivFlex(
                        this.__makeCheckbox(
                          this.useCognitiveServicesDefinition,
                          this.data[this.useCognitiveServicesDefinition.id]
                        ),
                        { class: "form-group" }
                      ),
                      { class: "col-6" }
                    )
                  ],
                  { class: "row" }
                )
            );

            $content.push($(inConcert.templates.process_flows.cognitive_services_card_header));

            $content.push( 
                this.__makeDivFlex(
                  [
                    this.__makeDivFlex(this.__makeComboFlex(this.cognitiveServicesDefinition,this.data[this.cognitiveServicesDefinition.id]),{ class: "col-12" })
                  ],{ class: "row" }
                )
            );

            $content.push(this.__makeScoring(this.data[this.scoringDefinition.id]));

			$content.push(this.__makeErrorMessage());	
            
            this.__changeValidationDefinition(this.data.validation_type, true);

		} else {

			this.cognitiveServicesDefinition.fieldClass = "mb_10";
			
            if (!this.data.message) {
				this.data.message = {};
			}

            $content.unshift(this.__makeDivFlex(this.__makeDivFlex(this.__makeInputFlex(this.resultDefinition, this.data[this.resultDefinition.id]),{ class: "col-12" }),{ class: "row" }));
		
            $content.push(
                this.__makeDivFlex(
                  [
                    this.__makeDivFlex(this.__makeInputFlex(this.retriesDefinition,this.data[this.retriesDefinition.id]),{ class: "col-4" }),
                    this.__makeDivFlex(this.__makeInputFlex(this.timeoutRetryDefinition,this.data[this.timeoutRetryDefinition.id]),{ class: "col-4" }),
                  ],{ class: "row" }
                )
            );

			
            $content.push( 
                this.__makeDivFlex(
                  [
                    this.__makeDivFlex(this.__makeComboFlex(this.cognitiveServicesDefinition,this.data[this.cognitiveServicesDefinition.id]),{ class: "col-12" })
                  ],{ class: "row" }
                )
            );

            if (this.type === "CALL") {
                $content.push(
                    this.__makeDivFlex([
                        this.__makeDivFlex(
                            this.__makeComboFlex(this.maxDigitsDefinition, this.data[this.maxDigitsDefinition.id])
                        ,{ class: "col-4" }),
                        this.__makeDivFlex(
                            this.__makeComboFlex(this.endDigitDefinition, this.data[this.endDigitDefinition.id])
                        ,{ class: "col-4" })
                    ]
                    ,{ class: "row" })
                );
            }

			$content.unshift(this.__makeValidationEditor(true));
			$content.push(this.__makeErrorMessage());
		}
		
		return $content;
	};
	
	adm_process_flow_activity.prototype.__makeScoring = function(scoring) {
		
		const $metadata = $(inConcert.templates.process_flows.menu_outputs_scoring);

		let $slider_value = $metadata.find(".ui-slider-scoring-value");
		$metadata.find("[control=global_scoring]").slider({
			range: "min",
			min: 0,
			max: 1,
			value: scoring?scoring:0.50,
			step: 0.01,
			create: function() {
				$slider_value.text(Math.floor(($(this).slider("value")) * 100) + "%");
				$metadata.find(".ui-slider-scoring-input-value").val($(this).slider("value"));
			},
			slide: function(_, ui) {
				$slider_value.text(Math.floor((ui.value) * 100) + "%");
				$metadata.find(".ui-slider-scoring-input-value").val(ui.value);
			}
		});
		
		return $metadata;
	};

	adm_process_flow_activity.prototype.__load = function(callback) {
		let self = this;
		self.editor.main.LoadBotApps(function(services) {
			self.cognitiveServicesDefinition.options = services;
			if ((self.cognitiveServicesDefinition.options.length === 0) || 
				(self.cognitiveServicesDefinition.options.length > 0 && self.cognitiveServicesDefinition.options[0].id !== "")) {
				self.cognitiveServicesDefinition.options.unshift({ id: "", name: "" });
			}
			callback();

            if (self.drawVersion === "2") {
                if (self.data[self.cognitiveServicesDefinition.id] === null || self.data[self.cognitiveServicesDefinition.id] === undefined || (self.data[self.cognitiveServicesDefinition.id] + "").length === 0) {
                    self.__changeUseCognitiveServices(false, false);
                } else {
                    self.__setValue(self.useCognitiveServicesDefinition, true);
                    self.__changeUseCognitiveServices(true, false);
                }
            }
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver640w flowDesignerGlobalModal tabsPopOver botActivity_send_and_wait_call";
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		let $content = this.__makeFormContent(adm_process_flow_activity.uber.__makeContent.call(this));

		if (this.drawVersion === "2") {
			let $outputs;
			let $error_message;

			if (this.type === "CALL" || this.type === "CHAT" || this.type === "FACEBOOK_BOT" || this.type === "INSTAGRAM_BOT" || 
                this.type === "TWITTER_BOT" || this.type === "SMS" || this.type === "WHATSAPP" || this.type === "MAIL") {
				this.__processing(true);
				$outputs = this.__makeOutputs();
			}

			if (this.type === "CALL") {
				$error_message = this.__makeErrorMessage();
			}

			return this.messagesEditor.Make($content, this.data[this.messageId], {}, $outputs, $error_message);
		} else {
            let $error_message;

            if (this.type === "CALL") {
				$error_message = this.__makeErrorMessage();
			}

			return this.messagesEditor.Make($content, this.data[this.messageId], {}, undefined, $error_message);
		}
		
	};
	
	adm_process_flow_activity.prototype.__makeErrorMessage = function() {

		if (!this.errorMessageOptions) {
			return null;
		}
		this.errorMessageEditor = new inConcert.adm_process_flow_messages_embeed(
			this.messagesEditor, this.errorMessageOptions);
			
		let errorMessageObj = this.data.errorMessage;
		if (errorMessageObj && errorMessageObj.msgtype && !this.errorMessageOptions.message_types.find(msgtype => msgtype.id === errorMessageObj.msgtype)) {
			errorMessageObj = {};
		}
		return this.errorMessageEditor.Make(errorMessageObj || {});
	};

	adm_process_flow_activity.prototype.__makeValidationEditor = function(visible) {
		let self = this;
		let currentValue = self.data && self.data.message && self.data.message.validation || "";
		let $container = $(templates.validation_custom_disclaimer, {});
		self.validationEditor = ace.edit($container.find("[validation_code]")[0]);
		self.validationEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		self.validationEditor.renderer.setScrollMargin(8, 8);
		self.validationEditor.getSession().setValue(currentValue);

        if (!visible) {
            $container.toggle(false);
        }

		return $container;
	};
	
	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		if (this.errorMessageOptions) {
			this.errorMessageEditor.AfterAppendBody();
		}
		this.messagesEditor.AfterAppendBody();
	};
	
	adm_process_flow_activity.prototype.__getAndValidateForm = function(data) {
		this.__addValue(data, this.retriesDefinition);
		this.__addValue(data, this.timeoutRetryDefinition);
		this.__addValue(data, this.resultDefinition);
		this.__addValue(data, this.cognitiveServicesDefinition);

        if (this.drawVersion === "2") {
            this.__addValue(data, this.validationDefinition);
        }
		
	};
	
	adm_process_flow_activity.prototype.__getAndValidateAsync = function(data, callback) {
		let self = this;		
		self.__getAndValidateMessage(data, function() {
            self.__getAndValidateForm(data);
			self.__getAndValidateErrorMessage(data, function() {
				callback(data);
			});
		});
	};
	
	adm_process_flow_activity.prototype.__getAndValidateMessage = function(data, callback) {
		let self = this;
		this.messagesEditor.AddValue(data, this.messageId, function() {
			data.message = data.message || {};
			self.__addValue(data.message, self.cognitiveServicesDefinition);
			callback(data);
		});
	};
	
	adm_process_flow_activity.prototype.__getAndValidateErrorMessage = function(data, callback) {
		if (!this.errorMessageEditor) {
			return callback(data);
		}
		data.errorMessage = {};
		this.errorMessageEditor.AddValue(data.errorMessage);
		if(this.getAndValidateErrorMessage && typeof this.getAndValidateErrorMessage === "function") {
			this.getAndValidateErrorMessage(data.errorMessage);
		}
		this.errorMessageEditor.AddValueAsync(data.errorMessage, function() {
			data.error = data.error || data.errorMessage.error;
			if ($.isEmptyObject(data.errorMessage)) {
				data.errorMessage = null;
			}
			callback();
		});
		
	};
	
	adm_process_flow_activity.prototype.__getAndValidateFormMessage = function(message) {

		if (this.drawVersion === "2") {

			if (this.type === "CALL") {
				this.__addValue(message, this.minDigitsDefinition);
			} 

            let validation_type = this.__getValue(this.validationDefinition);

			// agrego un código para validar el minDigits dado que el backend no lo soporta aún
			if (this.type === "CALL" && message && message.minDigits && parseInt(message.minDigits) > 1) {
                message.validation = "return #message.text >= " + message.minDigits;
			} else if (this.type === "CALL" && message && message.minDigits && parseInt(message.minDigits) <= 1) {
                message.validation = "return #message.text >= " + message.minDigits;

                if (!validation_type || validation_type === "") {
                    message.validation = null;
                } else {
                    if(this.validationEditor) {
                        message.validation = this.validationEditor.getValue() || null;			
                    
                        $.each(this.validationEditor.getSession().getAnnotations(), function(_, annotation) {
                            if (annotation.type === "error") {
                                message.error = "<%=_T('Invalid lua code.')%>";
                                return false;
                            }
                        });
                    }
                }

			} else if (( this.type === "CHAT" || 
                this.type === "FACEBOOK_BOT" || 
                this.type === "INSTAGRAM_BOT" || 
                this.type === "TWITTER_BOT" || 
                //this.type === "MAIL" || 
                this.type === "SMS" || 
                this.type === "WHATSAPP" ) && validation_type !== "custom" && validation_type !== "" && validation_type !== undefined) {

                if (validation_type === "only-alphanumeric") {
                    message.validation = "local str = message.text\n\rif str == nil then return nil end\n\r  if (type(str) ~= 'string') then\n\r    error(\"Expected string\")\n\r    return nil\n\r  end\n\r  \n\r  if(str:match(\"%W\")) then\n\r	return nil, \"not alphanumeric string\"\n\r  end\n\r\n\r  return true";
                } else if (validation_type === "only-letters") {
                    message.validation = "local str = message.text\n\rif str == nil then return nil end\n\r  if (type(str) ~= 'string') then\n\r    error(\"Expected string\")\n\r    return nil\n\r  end\n\r  \n\r  if(str:match(\"%A\")) then\n\r	return nil, \"not only letters\"\n\r  end\n\r\n\r  return true";
                } else if (validation_type === "only-numbers") {
                    message.validation = "local str = message.text\nif str == nil then return nil end\n\r  if (type(str) ~= 'string') then\n\r    error(\"Expected string\")\n\r    return nil\n\r  end\n\r  \n\r  local amount = tonumber(str)\n\r  if amount == nil then\n\r       return nil, \"not number\"\n\r  end\n\r\n\r  return true";
                } else if (validation_type === "phone") {
                    message.validation = "local str = message.text\nif str == nil then return nil end\n\r  if (type(str) ~= 'string') then\n\r    error(\"Expected string\")\n\r    return nil\n\r  end\n\r  \n\r  local amount = tonumber(str)\n\r  if amount == nil then\n\r       return nil, \"not number\"\n\r  end\n\r\n\r  return true";
                } else if (validation_type === "email") {
                    message.validation = "local str = message.text\nif str == nil then return nil end\n  if (type(str) ~= 'string') then\n\r\nerror(\"Expected string\")\n\r\nreturn nil\n  end\n  local lastAt = str:find(\"[^%@]+$\")\n  local localPart = str:sub(1, (lastAt - 2))\n  local domainPart = str:sub(lastAt, #str)\n  if localPart == nil then\n\r\nreturn nil, \"Local name is invalid\"\n  end\n  if domainPart == nil then\n\r\nreturn nil, \"Domain is invalid\"\n  end\n  if #localPart > 64 then\n\r\nreturn nil, \"Local name must be less than 64 characters\"\n  end\n  if #domainPart > 253 then\n\r\nreturn nil, \"Domain must be less than 253 characters\"\n  end\n  if lastAt >= 65 then\n\r\nreturn nil, \"Invalid @ symbol usage\"\n  end\n  local quotes = localPart:find(\"[\\\"]\")\n  if type(quotes) == 'number' and quotes > 1 then\n\r\nreturn nil, \"Invalid usage of quotes\"\n  end\n  if localPart:find(\"%@+\") and quotes == nil then\n\r\nreturn nil, \"Invalid @ symbol usage in local part\"\n  end\n  if not domainPart:find(\"%.\") then\n\r\nreturn nil, \"No TLD found in domain\"\n  end\n  if domainPart:find(\"%.%.\") then\n\r\nreturn nil, \"Too many periods in domain\"\n  end\n  if localPart:find(\"%.%.\") then\n\r\nreturn nil, \"Too many periods in local part\"\n  end\n  if not str:match('[%w]*[%p]*%@+[%w]*[%.]?[%w]*') then\n\r\nreturn nil, \"E-mail pattern test failed\"\n  end\n  return true";
                } else if (validation_type === "name-and-surname") {
                    message.validation = "local str = message.text\n\rlocal sep = \" \"\n\rlocal contador = 0\n\r\nfor str2 in string.gmatch(str, \"([^\"..sep..\"]+)\") do\n\r    contador = contador + 1\n\rend\n\r\nif contador ~= 2 and contador ~= 3 then\n\r	return nil, \"not 2 words\"\n\rend\n\r\n\r  return true";
                } else {
                    if(this.validationEditor) {
                        message.validation = this.validationEditor.getValue() || null;			
                    
                        $.each(this.validationEditor.getSession().getAnnotations(), function(_, annotation) {
                            if (annotation.type === "error") {
                                message.error = "<%=_T('Invalid lua code.')%>";
                                return false;
                            }
                        });
                    }
                }
                
            } else if (( this.type === "CHAT" || 
                this.type === "FACEBOOK_BOT" || 
                this.type === "INSTAGRAM_BOT" || 
                this.type === "TWITTER_BOT" || 
                //this.type === "MAIL" || 
                this.type === "SMS" || 
                this.type === "WHATSAPP" ) && (validation_type === "" || validation_type === undefined)) {
                    message.validation = undefined;                    
            } else {
                if(this.validationEditor) {
					message.validation = this.validationEditor.getValue() || null;			
				    
					$.each(this.validationEditor.getSession().getAnnotations(), function(_, annotation) {
						if (annotation.type === "error") {
							message.error = "<%=_T('Invalid lua code.')%>";
							return false;
						}
					});
				}
			}
		} else {
			message.validation = this.validationEditor.getValue() || null;
			$.each(this.validationEditor.getSession().getAnnotations(), function(_, annotation) {
				if (annotation.type === "error") {
					message.error = "<%=_T('Invalid lua code.')%>";
					return false;
				}
			});
		}

	};
	
/*
 * CALL
 */

	function adm_process_flow_activity_call(editor, node) {
		let self = this;
		
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);
		
		this.ttsEngine;
		this.ttsVoices;
		this.recognitionLanguages;
		this.asrGrammars;
		this.recognitionEngines = [];
		
		this.errorMessageOptions.message_types = [
			{
				id : "attachment",
				name : "<%=_T('Attachment')%>"
			}
		];

		this.voiceDefinition = {
			id : "voiceId",
			name : "<%=_T('Voice')%>",
			description : "<%=_T('TTS voice to use.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
		
		this.recognitionEngineDefinition = {
			id : "asrEngine",
			name : "<%=_T('Recognition Engine')%>",
			description : "<%=_T('Recognition Engine.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('DTMF digits')%>",
			change : function() {
				let currentRecognitionEngineValue = self.__getValue(self.recognitionEngineDefinition) || "";
				let currentGrammar = self.__getValue(self.GrammarDefinition);
				self.__toggleControl(self.GrammarDefinition, currentRecognitionEngineValue !== "");
				self.GrammarDefinition.required = (currentRecognitionEngineValue !== "");
				self.__toggleControl(self.dtmfTerminateTimeoutDefinition, currentRecognitionEngineValue === "unimrcp");
				self.__toggleControl(self.RecognitionLanguageDefinition, currentRecognitionEngineValue !== "");
				self.RecognitionLanguageDefinition.required = (currentRecognitionEngineValue !== "");
				self.__toggleBargein(currentRecognitionEngineValue === "speechmatic", currentRecognitionEngineValue === "speechmatic");
				self.__toggleRecognizers(currentRecognitionEngineValue === "speechmatic");
				self.__toggleControl(self.cognitiveServicesDefinition, currentRecognitionEngineValue !== "" && currentGrammar === "transcribe");

				if (self.drawVersion === "2") {
					
					if (currentRecognitionEngineValue !== null && currentGrammar === "transcribe") {
						let cognitiveServicesAppValue = self.__getValue(self.cognitiveServicesDefinition);
                        if (cognitiveServicesAppValue !== "" && cognitiveServicesAppValue !== null) { // hay app de luis seleccionada
                            self.__setValue(self.useCognitiveServicesDefinition, true);
                            self.__changeUseCognitiveServices(true, false);
                            self.__toggleControl(self.useCognitiveServicesDefinition, true);
                        } else {
                            self.__setValue(self.useCognitiveServicesDefinition, false);
							self.__changeUseCognitiveServices(false, false);
                            self.__toggleControl(self.useCognitiveServicesDefinition, true);
                        }
					} else {
                        self.__setValue(self.useCognitiveServicesDefinition, false);
                        self.__changeUseCognitiveServices(false, true);
                        self.__toggleControl(self.useCognitiveServicesDefinition, false);
					}
				}
			},
            form_group: true
		};
		
		this.GrammarDefinition = {
			id : "grammarId",
			name : "<%=_T('Grammar')%>",
			description : "<%=_T('Grammar.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('')%>",
			change : function() {
				let currentGrammar = self.__getValue(self.GrammarDefinition);
				self.__toggleControl(self.cognitiveServicesDefinition, currentGrammar === "transcribe");

				if (self.drawVersion === "2") {
                    if (currentGrammar === "transcribe") {
                        self.__changeUseCognitiveServices(false, false);
                        self.__toggleControl(self.useCognitiveServicesDefinition, true);
                    } else {
                        self.__setValue(self.useCognitiveServicesDefinition, false);
                        self.__changeUseCognitiveServices(false, true);
                        self.__toggleControl(self.useCognitiveServicesDefinition, false);
                    }
				} else {
                    if (currentGrammar === "transcribe") {
                        self.__toggleControl(self.cognitiveServicesDefinition, true);
                    } else {
                        self.__toggleControl(self.cognitiveServicesDefinition, false);
                    }
				}
			},
            form_group: true
		};

		if (this.drawVersion === "2") {
			
			this.dtmfTerminateTimeoutDefinition = {
				id : "dtmfTerminateTimeout",
				name : "<%=_T('DTMF terminate timeout (seconds)')%>",
				description : "<%=_T('DTMF terminate timeout (seconds)')%>",
				not_extended : true,
				fieldClass : "Short",
				options : this.__getRange(1, 99),
                form_group: true 
			};
			
			this.minDigitsDefinition = {
				id : "minDigits",
				name : "<%=_T('Min Digits')%>",
				description : "<%=_T('Min number of digits to be enter by user.')%>",
				not_extended : true,
				required : true,
				fieldClass : "col_4",
				options : this.__getRange(1, 30),
				emptyLabel : "<%=_T('Nothing')%>",
				change: function () {
					self.__selectMinDigits($(this).val());
				},
				defaultValue: 1,
                form_group: true
			};

			this.maxDigitsDefinition = {
				id : "maxDigits",
				name : "<%=_T('Max Digits')%>",
				description : "<%=_T('Max number of digits to be enter by user.')%>",
				not_extended : true,
				required : true,
				fieldClass : "col_4",
				options : this.__getRange(1, 30),
				emptyLabel : "<%=_T('Nothing')%>",
				defaultValue: 1,
                form_group: true
			};

			this.endDigitDefinition = {
				id : "endDigit",
				name : "<%=_T('End Digit')%>",
				description : "<%=_T('Digit to end capture.')%>",
				not_extended : true,
				fieldClass : "col_4",
				options : this.__getPhoneDigitsOptions(),
				emptyLabel : "<%=_T('Nothing')%>",
                form_group: true
			};
			
		} else {
			
			this.dtmfTerminateTimeoutDefinition = {
				id : "dtmfTerminateTimeout",
				name : "<%=_T('DTMF terminate timeout (seconds)')%>",
				description : "<%=_T('DTMF terminate timeout (seconds)')%>",
				not_extended : true,
				fieldClass : "Short",
				validate : {
					range : {
						type : "integer",
						allowVariables : true,
						min : 2,
						max : 1800
					}
				},
				//variable : true,
                form_group: true
			};
			
            this.maxDigitsDefinition = {
				id : "maxDigits",
				name : "<%=_T('Max Digits')%>",
				description : "<%=_T('Max number of digits to be enter by user.')%>",
				not_extended : true,
				fieldClass : "col_4",
				validate : {
					range : {
						type : "integer",
						allowVariables : true,
						min : 1,
						max : 20
					}
				},
				//variable : true,
                form_group: true
			};

			this.endDigitDefinition = {
				id : "endDigit",
				name : "<%=_T('End Digit')%>",
				description : "<%=_T('Digit to end capture.')%>",
				not_extended : true,
				fieldClass : "col_4",
				options : this.__getPhoneDigitsOptions(),
				emptyLabel : "<%=_T('Nothing')%>",
                form_group: true
			};
		}

		this.RecognitionLanguageDefinition = {
			id : "recognitionLanguage",
			name : "<%=_T('Recognition Language')%>",
			description : "<%=_T('Recognition Language to use.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
		
		this.voiceDetectOnPlayMinDurDefinition = {
			id : "mindur",
			name : "<%=_T('Min voice duration (ms)')%>",
			description : "<%=_T('Minimum voice duration to start voice detection while playback (milliseconds).')%>",
			required : true,
			not_extended : true,
			fieldClass : "Short",
			validate : {
				range : {
					type : "integer",
					allowVariables : true,
					min : 100,
					max : 1000
				}
			},
			defaultValue : 250,
            form_group: true
		};
		
		this.voiceDetectOnPlayMaxDurDefinition = {
			id : "maxdur",
			name : "<%=_T('Max voice duration (ms)')%>",
			description : "<%=_T('Maximum voice duration to end voice detection while playback (milliseconds).')%>",
			required : true,
			not_extended : true,
			fieldClass : "Short",
			validate : {
				range : {
					type : "integer",
					allowVariables : true,
					min : 1200,
					max : 30000
				}
			},
			defaultValue : 5000,
            form_group: true
		};
		
		this.voiceDetectOnPlaySilDurDefinition = {
			id : "sildur",
			name : "<%=_T('Silence duration (ms)')%>",
			description : "<%=_T('Silence duration to end voice detection (milliseconds).')%>",
			required : true,
			not_extended : true,
			fieldClass : "Short",
			validate : {
				range : {
					type : "integer",
					allowVariables : true,
					min : 500,
					max : 5000
				}
			},
			defaultValue : 500,
            form_group: true
		};
		
		this.enableTimeRecognizerDefinition = {
			id : "enableTimeRecognizer",
			name : "<%=_T('Date/Time')%>",
			description : "<%=_T('Enable time recognizer.')%>",
			not_extended : true,
			change : function() {
				self.timeRecognizerDimensionDefinition.required = $("[control='enableTimeRecognizer']").is(":checked");
				self.__toggleControl(self.timeRecognizerDimensionDefinition, $("[control='enableTimeRecognizer']").is(":checked"));
			},
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.timeRecognizerDimensionDefinition = {
			id : "timeRecognizerDimension",
			name : "<%=_T('Time part')%>",
			description : "<%=_T('Part of the date/time to be recognized.')%>",
			options : this.__getTimeRecognizerDimensionOptions(),
			not_extended : true,
			fieldClass : "Short",
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
		
		this.enableNumberRecognizerDefinition = {
			id : "enableNumberRecognizer",
			name : "<%=_T('Number')%>",
			description : "<%=_T('Enable number recognizer.')%>",
			not_extended : true,
			change : function() {
				self.numberRecognizerIntegerDigitsDefinition.required = $("[control='enableNumberRecognizer']").is(":checked");
				self.numberRecognizerDecimalDigitsDefinition.required = $("[control='enableNumberRecognizer']").is(":checked");
				self.__toggleControl(self.numberRecognizerIntegerDigitsDefinition, $("[control='enableNumberRecognizer']").is(":checked"));
				self.__toggleControl(self.numberRecognizerDecimalDigitsDefinition, $("[control='enableNumberRecognizer']").is(":checked"));
			},
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.numberRecognizerIntegerDigitsDefinition = {
			id : "numberRecognizerIntegerDigits",
			name : "<%=_T('Integer digits')%>",
			description : "<%=_T('Min digits to recognize integer part of number.')%>",
			options : this.__getNumberRecognizerDigitsOptions(),
			not_extended : true,
			fieldClass : "Short",
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
		
		this.numberRecognizerDecimalDigitsDefinition = {
			id : "numberRecognizerDecimal",
			name : "<%=_T('Decimal digits')%>",
			description : "<%=_T('Min digits to recognize decimal part of number.')%>",
			options : this.__getNumberRecognizerDigitsOptions(),
			not_extended : true,
			fieldClass : "Short",
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
		
		this.enableOrdinalRecognizerDefinition = {
			id : "enableOrdinalRecognizer",
			name : "<%=_T('Ordinal')%>",
			description : "<%=_T('Enable ordinal recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableMoneyRecognizerDefinition = {
			id : "enableMoneyRecognizer",
			name : "<%=_T('Money')%>",
			description : "<%=_T('Enable money recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableCreditCardRecognizerDefinition = {
			id : "enableCreditCardRecognizer",
			name : "<%=_T('Credit Card')%>",
			description : "<%=_T('Enable credit card number recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enablePhoneRecognizerDefinition = {
			id : "enablePhoneRecognizer",
			name : "<%=_T('Phone')%>",
			description : "<%=_T('Enable phone number recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableDistanceRecognizerDefinition = {
			id : "enableDistanceRecognizer",
			name : "<%=_T('Distance')%>",
			description : "<%=_T('Enable distance recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableDurationRecognizerDefinition = {
			id : "enableDurationRecognizer",
			name : "<%=_T('Duration')%>",
			description : "<%=_T('Enable duration recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableQuantityRecognizerDefinition = {
			id : "enableQuantityRecognizer",
			name : "<%=_T('Quantity')%>",
			description : "<%=_T('Enable quantity recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableTemperatureRecognizerDefinition = {
			id : "enableTemperatureRecognizer",
			name : "<%=_T('Temperature')%>",
			description : "<%=_T('Enable temperature recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableVolumeRecognizerDefinition = {
			id : "enableVolumeRecognizer",
			name : "<%=_T('Volume')%>",
			description : "<%=_T('Enable volume recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableUrlRecognizerDefinition = {
			id : "enableUrlRecognizer",
			name : "<%=_T('URL')%>",
			description : "<%=_T('Enable URL recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.enableEmailRecognizerDefinition = {
			id : "enableEmailRecognizer",
			name : "<%=_T('E-mail')%>",
			description : "<%=_T('Enable e-mail recognizer.')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
	}

	inConcert.Utils.inherit( adm_process_flow_activity_call, adm_process_flow_activity );
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);
	
	adm_process_flow_activity_call.prototype.__load = function(callback) {
		const self = this;

		this.editor.main.LoadTtsAsrEngines(function(success, ttsasrEngines) {
			self.ttsasrEngines = success ? ttsasrEngines : undefined;
			if(self.ttsasrEngines && $.isArray(self.ttsasrEngines)) {
				self.ttsasrEngines.forEach(function(engine) {
					if((engine.providerType === "TTS" || engine.providerType === "TTS-ASR") && engine.voices && $.isArray(engine.voices) && engine.voices.length > 0) {
						self.ttsEngine = engine.engineId;
						self.ttsVoices = [];
						engine.voices.forEach(function(voice) {
							let voice_to_show = {
								id : voice.id,
								name : voice.LanguageName + " - " + voice.Variant + " - " + voice.Gender + " - " + voice.VoiceName
							};
							self.ttsVoices.push(voice_to_show);
						});
					}
					if(engine.providerType === "ASR" || engine.providerType === "TTS-ASR") {
						self.recognitionEngines.push({id : engine.engineId, name: engine.engineDesc});
						self.asrGrammars = [];
						if(engine.grammars && $.isArray(engine.grammars) && engine.grammars.length > 0) {
							engine.grammars.forEach(function(grammar) {
								let grammar_to_show = {
									id : grammar.id,
									name : grammar.id + " (" + grammar.type + ")",
									grammar : grammar.grammar
								};
								self.asrGrammars.push(grammar_to_show);
							});
						}
						
						if(engine.recognitionLanguages && $.isArray(engine.recognitionLanguages) && engine.recognitionLanguages.length > 0) {
							self.recognitionLanguages = [];
							engine.recognitionLanguages.forEach(function(recognitionLanguage) {
								let recognitionLanguage_to_show = {
									id : recognitionLanguage.id,
									name : recognitionLanguage.LanguageName
								};
								self.recognitionLanguages.push(recognitionLanguage_to_show);
							});
						}
					}
				});

				self.isTtsEnabled = (self.ttsVoices && $.isArray(self.ttsVoices) && self.ttsVoices.length > 0);
				self.isAsrEnabled = (self.asrGrammars && $.isArray(self.asrGrammars) && self.asrGrammars.length > 0);
				
				if(self.isTtsEnabled)
				{
					self.errorMessageOptions.message_types.push({id : "text", name : "<%=_T('Text')%>"});
				}
				
				self.voiceDefinition.options = self.ttsVoices;
				self.recognitionEngineDefinition.options = self.recognitionEngines;
				self.GrammarDefinition.options = self.asrGrammars;
				self.RecognitionLanguageDefinition.options = self.recognitionLanguages || [];
			}

			self.editor.main.LoadBotApps(function(services) {
				self.cognitiveServicesDefinition.options = services;
				if ((self.cognitiveServicesDefinition.options.length === 0) || 
					(self.cognitiveServicesDefinition.options.length > 0 && self.cognitiveServicesDefinition.options[0].id !== "")) {
					self.cognitiveServicesDefinition.options.unshift({ id: "", name: "" });
				}
				callback();
			});
		});
	};

	adm_process_flow_activity.prototype.__makeEntityRecognizersFields = function() {
        var $fields = $(Mustache.render("<div></div>", {}));
        var $controls = [];

        $controls.push($(templates.entity_recognizer_fields));

		$controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableOrdinalRecognizerDefinition, this.data.message && this.data.message[this.enableOrdinalRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableMoneyRecognizerDefinition, this.data.message && this.data.message[this.enableMoneyRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableCreditCardRecognizerDefinition, this.data.message && this.data.message[this.enableCreditCardRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );

        $controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enablePhoneRecognizerDefinition, this.data.message && this.data.message[this.enablePhoneRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableDistanceRecognizerDefinition, this.data.message && this.data.message[this.enableDistanceRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableDurationRecognizerDefinition, this.data.message && this.data.message[this.enableDurationRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );

        $controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableTimeRecognizerDefinition, this.data.message && this.data.message[this.enableTimeRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-6" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.timeRecognizerDimensionDefinition, this.data.message && this.data.message[this.timeRecognizerDimensionDefinition.id])
                    ,{ class: "col-6" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );
        
        $controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableNumberRecognizerDefinition, this.data.message && this.data.message[this.enableNumberRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeComboFlex(this.numberRecognizerIntegerDigitsDefinition, this.data.message && this.data.message[this.numberRecognizerIntegerDigitsDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeComboFlex(this.numberRecognizerDecimalDigitsDefinition, this.data.message && this.data.message[this.numberRecognizerDecimalDigitsDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );

        $controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableQuantityRecognizerDefinition, this.data.message && this.data.message[this.enableQuantityRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableTemperatureRecognizerDefinition, this.data.message && this.data.message[this.enableTemperatureRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableVolumeRecognizerDefinition, this.data.message && this.data.message[this.enableVolumeRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );

        $controls.push( 
            this.__makeDivFlex(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableUrlRecognizerDefinition, this.data.message && this.data.message[this.enableUrlRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-6" }),
                    this.__makeDivFlex(
                        this.__makeDivFlex(
                            this.__makeCheckboxFlex(this.enableEmailRecognizerDefinition, this.data.message && this.data.message[this.enableEmailRecognizerDefinition.id])
                        ,{ class: "form-group" })
                    ,{ class: "col-6" })
                ]
                ,{ class: "row" })
            ,{ class: "flowDesignerCheckboxGroup flowDesignerEntityRecognizerFieldsContainerId" })
        );

        $fields.append($controls);
        
        return $fields;
	};
	
	adm_process_flow_activity_call.prototype.__makeFormContent = function($content) {
		$content = adm_process_flow_activity_call.uber.__makeFormContent.call(this, $content);
		//We use splice to keep cognitiveServicesDefinition the last one

		if (this.drawVersion === "2") {

			$content.unshift(this.__makeEntityRecognizersFields());
			
            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlaySilDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlaySilDurDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlayMinDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlayMinDurDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlayMaxDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlayMaxDurDefinition.id])
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.dtmfTerminateTimeoutDefinition, this.data.message && this.data.message[this.dtmfTerminateTimeoutDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.GrammarDefinition, this.data.message && this.data.message[this.GrammarDefinition.id])
                    ,{ class: "col-6" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.RecognitionLanguageDefinition, this.data.message && this.data.message[this.RecognitionLanguageDefinition.id])
                    ,{ class: "col-6" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.recognitionEngineDefinition, this.data.message && this.data.message[this.recognitionEngineDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.voiceDefinition, this.data.message && this.data.message[this.voiceDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );

		} else {

            $content.unshift(this.__makeEntityRecognizersFields());
			
            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlaySilDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlaySilDurDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlayMinDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlayMinDurDefinition.id])
                    ,{ class: "col-4" }),
                    this.__makeDivFlex(
                        this.__makeInputFlex(this.voiceDetectOnPlayMaxDurDefinition, this.data.message && this.data.message[this.voiceDetectOnPlayMaxDurDefinition.id])
                    ,{ class: "col-4" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.dtmfTerminateTimeoutDefinition, this.data.message && this.data.message[this.dtmfTerminateTimeoutDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.GrammarDefinition, this.data.message && this.data.message[this.GrammarDefinition.id])
                    ,{ class: "col-6" }),
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.RecognitionLanguageDefinition, this.data.message && this.data.message[this.RecognitionLanguageDefinition.id])
                    ,{ class: "col-6" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.recognitionEngineDefinition, this.data.message && this.data.message[this.recognitionEngineDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );

            $content.unshift(
                this.__makeDivFlex([
                    this.__makeDivFlex(
                        this.__makeComboFlex(this.voiceDefinition, this.data.message && this.data.message[this.voiceDefinition.id])
                    ,{ class: "col-12" })
                ]
                ,{ class: "row" })
            );
		
		}

		return $content;
	};
	
	adm_process_flow_activity_call.prototype.__afterAppendBody = function() {
		adm_process_flow_activity_call.uber.__afterAppendBody.call(this);
		
		if(this.isTtsEnabled)
		{
			this.AfterSelectType();
		}
		else
		{
			this.__hideControl(this.voiceDefinition);
		}
		
		if(this.isAsrEnabled)
		{
			this.enableTimeRecognizerDefinition.change();
			this.enableNumberRecognizerDefinition.change();
			this.recognitionEngineDefinition.change();
		}
		else
		{
			this.__hideControl(this.recognitionEngineDefinition);
			this.__hideControl(this.dtmfTerminateTimeoutDefinition);
			this.__hideControl(this.GrammarDefinition);
			this.__hideControl(this.RecognitionLanguageDefinition);
			this.__toggleBargein(false);
			this.__toggleRecognizers(false);
			this.__hideControl(this.cognitiveServicesDefinition);

			if (this.drawVersion === "2") {
                this.__setValue(this.useCognitiveServicesDefinition, false);
                this.__changeUseCognitiveServices(false, true);
                this.__toggleControl(this.useCognitiveServicesDefinition, false);
			}
		}
	};
	
	adm_process_flow_activity_call.prototype.__getAndValidateFormMessage = function(message) {
		adm_process_flow_activity_call.uber.__getAndValidateFormMessage.call(this, message);

		if (this.drawVersion === "2") {
			this.__addValue(message, this.minDigitsDefinition);
		}

		this.__addValue(message, this.maxDigitsDefinition);
		this.__addValue(message, this.endDigitDefinition);
		this.__addValue(message, this.voiceDefinition);
		this.__addValue(message, this.recognitionEngineDefinition);
		this.__addValue(message, this.GrammarDefinition);
		this.__addValue(message, this.RecognitionLanguageDefinition);
		this.__addValue(message, this.voiceDetectOnPlayMinDurDefinition);
		this.__addValue(message, this.voiceDetectOnPlayMaxDurDefinition);
		this.__addValue(message, this.voiceDetectOnPlaySilDurDefinition);
		this.__addValue(message, this.dtmfTerminateTimeoutDefinition);
		
		this.__addValue(message, this.enableTimeRecognizerDefinition);
		this.__addValue(message, this.timeRecognizerDimensionDefinition);
		this.__addValue(message, this.enableNumberRecognizerDefinition);
		this.__addValue(message, this.numberRecognizerIntegerDigitsDefinition);
		this.__addValue(message, this.numberRecognizerDecimalDigitsDefinition);
		this.__addValue(message, this.enableOrdinalRecognizerDefinition);
		this.__addValue(message, this.enableMoneyRecognizerDefinition);
		this.__addValue(message, this.enableCreditCardRecognizerDefinition);
		this.__addValue(message, this.enablePhoneRecognizerDefinition);
		this.__addValue(message, this.enableDistanceRecognizerDefinition);
		this.__addValue(message, this.enableDurationRecognizerDefinition);
		this.__addValue(message, this.enableQuantityRecognizerDefinition);
		this.__addValue(message, this.enableTemperatureRecognizerDefinition);
		this.__addValue(message, this.enableVolumeRecognizerDefinition);
		this.__addValue(message, this.enableUrlRecognizerDefinition);
		this.__addValue(message, this.enableEmailRecognizerDefinition);

		message.ttsEngine = this.ttsEngine;
		let currentRecognitionEngineValue = this.__getValue(this.recognitionEngineDefinition);
		let currentMessageType = this.messagesEditor.currentType;
		if(this.isAsrEnabled && currentRecognitionEngineValue === "unimrcp") {
			message.promptOptions = {};
				
			let dtt = this.__getValue(this.dtmfTerminateTimeoutDefinition);
			if(dtt) {
				message.promptOptions.dtt = dtt;
			}
			
			//El dttc (DTMF terminate characters) lo seteamos a partir del endDigitDefinition
			let dttc = this.__getValue(this.endDigitDefinition);
			if(dttc){
				message.promptOptions.dttc = dttc;
			}
			
			let spl = this.__getValue(this.RecognitionLanguageDefinition);
			if(spl){
				message.promptOptions.spl = spl;
			}
		}
		else if(this.isAsrEnabled && currentRecognitionEngineValue === "speechmatic") { 
			message.promptOptions = {};
			let sildur = this.__getValue(this.voiceDetectOnPlaySilDurDefinition);
			if(sildur){
				message.promptOptions.sildur = sildur;
			}
			//Opciones aplicables solo a attachment
			if(currentMessageType !== "text") {
				let mindur = this.__getValue(this.voiceDetectOnPlayMinDurDefinition);
				if(mindur){
					message.promptOptions.mindur = mindur;
				}
				let maxdur = this.__getValue(this.voiceDetectOnPlayMaxDurDefinition);
				if(maxdur){
					if(maxdur === this.voiceDetectOnPlayMaxDurDefinition.validate.range.max) {
						maxdur = "-1";	//No limited
					}
					message.promptOptions.maxdur = maxdur;
				}
			}

			let enabledRecognizers = "0";
			message.promptOptions.dimensions = {};
			if(this.__getValue(this.enableTimeRecognizerDefinition)) {
				message.promptOptions.dimensions["time"] = this.__getValue(this.timeRecognizerDimensionDefinition);
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableNumberRecognizerDefinition)) {
				message.promptOptions.dimensions["number"] = this.__getValue(this.numberRecognizerIntegerDigitsDefinition) + "." + this.__getValue(this.numberRecognizerDecimalDigitsDefinition);
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableOrdinalRecognizerDefinition)) {
				message.promptOptions.dimensions["ordinal"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableMoneyRecognizerDefinition)) {
				message.promptOptions.dimensions["amount-of-money"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableCreditCardRecognizerDefinition)) {
				message.promptOptions.dimensions["credit-card-number"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enablePhoneRecognizerDefinition)) {
				message.promptOptions.dimensions["phone-number"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableDistanceRecognizerDefinition)) {
				message.promptOptions.dimensions["distance"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableDurationRecognizerDefinition)) {
				message.promptOptions.dimensions["duration"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableQuantityRecognizerDefinition)) {
				message.promptOptions.dimensions["quantity"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableTemperatureRecognizerDefinition)) {
				message.promptOptions.dimensions["temperature"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableVolumeRecognizerDefinition)) {
				message.promptOptions.dimensions["volume"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableUrlRecognizerDefinition)) {
				message.promptOptions.dimensions["url"] = "";
				enabledRecognizers = "1";
			}
			if(this.__getValue(this.enableEmailRecognizerDefinition)) {
				message.promptOptions.dimensions["email"] = "";
				enabledRecognizers = "1";
			}
			message.promptOptions.enabledRecognizers = enabledRecognizers;
			
		}
	};
	
	adm_process_flow_activity_call.prototype.getAndValidateErrorMessage = function(errorMessage) {
		if(errorMessage.msgtype === "text" && this.isTtsEnabled) {
			errorMessage.voiceId = this.__getValue(this.voiceDefinition);
		}
	};
	
	adm_process_flow_activity_call.prototype.AfterSelectType = function() {
		let currentMessageType = this.messagesEditor.currentType;
		let errorMessageActive = $("[embeed_message_toggle]").is(":checked");
		let currentErrorMessageType = $("[control='msgtype']").val();
		if(currentMessageType === "text" || (errorMessageActive && currentErrorMessageType === "text"))
		{
			this.__showControl(this.voiceDefinition);
		}
		else {
			this.__hideControl(this.voiceDefinition);
		}
		let currentRecognitionEngineValue = this.__getValue(this.recognitionEngineDefinition) || "";
		this.__toggleBargein(currentRecognitionEngineValue === "speechmatic", currentRecognitionEngineValue === "speechmatic");
	};
	
	adm_process_flow_activity_call.prototype.__toggleBargein = function(show, showSilDur)
	{
 		var objBargeinRow = $("[form_subtitle='bargein_subtitle_id']").parent();
		objBargeinRow.toggle(show);
		$("#bargein_disclaimer_id").toggle(show);
		this.voiceDetectOnPlayMinDurDefinition.required = show;
		this.voiceDetectOnPlayMaxDurDefinition.required = show;
		this.voiceDetectOnPlaySilDurDefinition.required = show || showSilDur;

		if (show) {
			this.__showControl(this.voiceDetectOnPlayMinDurDefinition);
			this.__showControl(this.voiceDetectOnPlayMaxDurDefinition);
		} else {
			this.__hideControl(this.voiceDetectOnPlayMinDurDefinition);
			this.__hideControl(this.voiceDetectOnPlayMaxDurDefinition);
		}
		if(showSilDur) {
			this.__showControl(this.voiceDetectOnPlaySilDurDefinition);
		}
		else {
			this.__hideControl(this.voiceDetectOnPlaySilDurDefinition);
		}
	};
	
	adm_process_flow_activity_call.prototype.__getTimeRecognizerDimensionOptions = function() {
		return [ 
				{
					id : "year",
					name : "<%=_T('Year')%>",
					disclaimer : "<%=_T('Recognizes a specific year. Examples: 2021. Next year.')%>"
				},
				{
					id : "month",
					name : "<%=_T('Month')%>",
					disclaimer : "<%=_T('Recognizes a month of a specific year. Example: June 2021. Next month.')%>"
				},
				{
					id : "day",
					name : "<%=_T('Day')%>",
					disclaimer : "<%=_T('Recognizes a full date: Day, month and year. Examples: June 22, 2021. Tomorrow. Yesterday.')%>"
				},
				{
					id : "hour",
					name : "<%=_T('Hour')%>",
					disclaimer : "<%=_T('Recognizes an hour. Examples: At 7 PM.')%>"
				},
				{
					id : "minute",
					name : "<%=_T('Minute')%>",
					disclaimer : "<%=_T('Recognizes an hour and minute. Example: Half past 7. In two hours.')%>"
				},
		];
	};
	
	adm_process_flow_activity_call.prototype.__getNumberRecognizerDigitsOptions = function() {
		return [ "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" ];
	};
	
	adm_process_flow_activity_call.prototype.__toggleRecognizers = function(show)
	{
		$("#flowDesignerEntityRecognizerFieldsContainerTitle").toggle(show);
		$(".flowDesignerEntityRecognizerFieldsContainerId").toggle(show);
		$("#entity_recognizers_disclaimer_id").toggle(show);
	};

    adm_process_flow_activity.prototype.__getAndValidateOutputTitles = function(data) {

		var self = this;
		let outputNameList = [];
		let validated = true;

		self.$get("[output_field]").each(function() {
			let outputName = $(this).find('[field_name]').val();

			if (outputName && outputName !== "")
				outputNameList.push(outputName);
		});

		const orderedOutputNames = outputNameList.sort(); 

		self.$get("[output_field]").each(function(_) {

			var namesDuplicated = [];
			for (var i = 0; i < orderedOutputNames.length - 1; i++) {
				if (orderedOutputNames[i + 1] === orderedOutputNames[i]) {
					namesDuplicated.push(orderedOutputNames[i]);
				}
			}
			if (namesDuplicated.length > 0) {
				data.error = {
					message : "<%=_T('Output names must be different from each other')%>"
				};	
				validated = false;
				return false;
			}
		});
		
		return validated;
	};

	adm_process_flow_activity.prototype.htmlBody = function(data) {

		let image = "";

		if (data) {
			let type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Send message')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		let ports = [];
		let all_outputs = { ports: [] };
		let acumulative_outputs = { ports: [] };

		// genero una lista con todos los puertos
		this.__generate_all_port_labels(data, all_outputs);

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});

		if (data.message.voiceId !== undefined) { // es un IVR
		
			// agrego puertos
			ports.push(...this.__generate_ports(data, node, all_outputs, acumulative_outputs)); // debe ir siempre primero

			// agrego un puerto por defecto
			ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

			// agrego una opción para los errores
			ports.push(...this.__generate_error_ports(data, node, true));

		} else { // es un CHAT
		
			// agrego puertos
			ports.push(...this.__generate_ports(data, node, all_outputs, acumulative_outputs)); // debe ir siempre primero

			// agrego puertos para los botones
			ports.push(...this.__generate_button_ports(data, node, acumulative_outputs));

			// agrego puertos para las opciones 
			ports.push(...this.__generate_options_ports(data, node, acumulative_outputs));

			// agrego puertos para los quickreplies
			//ports.push(...this.__generate_quickreplies_ports(data, node, acumulative_outputs));

			// agrego un puerto por defecto
			ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

			// agrego una opción para los errores
			ports.push(...this.__generate_error_ports(data, node, true));
		}

		return ports;
	};

	adm_process_flow_activity.prototype.__generate_all_port_labels = function(data, all_outputs) {
		
        if (!data.ports) {
            data.ports = [];
        }

		let ports = [];
		
		data.ports.forEach(p => {

			// agrego outputs
			if (p.outputs.length > 0) {
				p.outputs.forEach(i => {
					ports.push(i);
				});
			} 
			
			// agrego buttons
			if (data.message && data.message.buttons && data.message.buttons.length > 0) {
				data.message.buttons.forEach(b => {	
					ports.push(b.message?b.message:b.title);	
				});
			}

			// agrego options
			if (data.message && data.message.options && data.message.options.length > 0) {
				data.message.options.forEach(b => {				
					if (b && b.buttons && b.buttons.length > 0) {
						b.buttons.forEach(b1 => {			
							ports.push(b1.message?b1.message:b1.title);								
						});
					}
				});
			}

			// agrego quickreplies
			if (data.message && data.message.quickreplies && data.message.quickreplies.length > 0) {
				data.message.quickreplies.forEach(b => {	
					ports.push(b.title);						
				});
			}
		});

		all_outputs.ports.push(...ports.filter(function(value, index, self) {
			return self.indexOf(value) === index;
		}));
	};

	adm_process_flow_activity.prototype.__generate_ports = function(data, node, all_outputs, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }

		let ports = [];
		
		if (data.ports && data.ports.length > 0) {

			data.ports.forEach(p => {

				if (p.outputs.length > 0) {

					let existe = node?node.children.find(c => c.id === p.port_id):undefined;
				
					let conditions = [];

					// agrego las condiciones para los errores
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Timeout\"" });
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"ChannelClosed\"" });
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

					let contador = 0;

					p.outputs.forEach(i => {
						contador++;

						// incluyo aquí todos los ports que voy agregando
						acumulative_outputs.ports.push(i);

						let begin_parenths = "(";

                        if(data["cognitiveServicesApp"] && data["cognitiveServicesApp"].length > 0) { // hay app de nlu
                            begin_parenths = "((";
                        }

						conditions.push({ 
							begin_parenths: (contador === 1?begin_parenths:undefined),
							logic : (contador === 1?"and":"or"), //logic, //conditions.length > 0?"and":undefined, 
							field: "decodeUnicodeEscape(" + data.result + ".event.message.text)",
							comparator: "==", 
							value: "decodeUnicodeEscape(\""+ i +"\")",
							end_parenths: (contador === p.outputs.length?")":undefined)
						});
					
					});
					
					if(data["cognitiveServicesApp"] && data["cognitiveServicesApp"].length > 0) { // hay app de nlu

						// agrego una condición para validar el caso de NLU
						conditions.push({ begin_parenths: "(", logic : "or", field: data.result + ".event.nludata.topScoringIntent", comparator: "~=", value: "nil" });

						// agrego condición para el scoring ! todavía no agregué el control web para definir el scoring !
						conditions.push({ begin_parenths: "(", logic : "and", field: data.result + ".event.nludata.topScoringIntent.score", comparator: ">=", value: data.luis_scoring });

						// si el puerto incluye el intent None, entonces agrego una validación para que no entre aquí ningún texto quickreply u otro que pertenezca a otro puerto
						if (p.outputs.find(i => i === "None")) {							
							all_outputs.ports.forEach(i_none => {
								conditions.push({ 
									logic : "and",
									field: "decodeUnicodeEscape(" + data.result + ".event.message.text)",
									comparator: "~=", 
									value: "decodeUnicodeEscape(\""+ i_none +"\")"
								});
							});
						}

						// prosigo
						let contador = 0;

						p.outputs.forEach(i => {

							contador++;

							conditions.push({ 
								begin_parenths: (contador === 1?"(":undefined),
								logic : (contador === 1?"and":"or"), //logic, //conditions.length > 0?"and":undefined, 
								field: data.result + ".event.nludata.topScoringIntent.intent",
								comparator: "==", 
								value: "\""+ i +"\"" ,
								end_parenths: (contador === p.outputs.length?"))))":undefined)
							});
						});
					}

					ports.push({
						direction : "out", // in/out/none
						type : "option", // error/success/option
						option_label : p.name, // error/success/option
						channels: "1", // 0, 1 o n
						id: (existe?existe.id:p.port_id),
						preset_value: {
							description: "", //p.name
							conditions: conditions
						}
					});
				}

			});
		}
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_button_ports = function(data, node, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		if (data.message && data.message.buttons && data.message.buttons.length > 0) {
			data.message.buttons.forEach(b => {

				if (!acumulative_outputs.ports.find(c => c === (b.message?b.message:b.title))) { // si no existe el botón dentro del arreglo de puertos actual

					// incluyo aquí todos los ports que voy agregando
					acumulative_outputs.ports.push((b.message?b.message:b.title));

					let existe = node?node.children.find(c => c.id === b.port_id):undefined;	
					let portId = (existe?existe.id:b.port_id);

					ports.push({
						direction : "none", // in/out/none
						type : "option", // error/success/option
						option_label : b.message?b.message:b.title, // error/success/option
						channels: "0", // 0, 1 o n
						id: portId,
						preset_value: {
							port_id: portId
						}
					});
				}	
			});
		}
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_options_ports = function(data, node, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		if (data.message && data.message.options && data.message.options.length > 0) {
			data.message.options.forEach(b => {
			
				if (b && b.buttons && b.buttons.length > 0) {
					b.buttons.forEach(b1 => {
		
						if (!acumulative_outputs.ports.find(c => c === (b1.message?b1.message:b1.title))) { // si no existe el botón dentro del arreglo de puertos actual
							
							// incluyo aquí todos los ports que voy agregando
							acumulative_outputs.ports.push((b1.message?b1.message:b1.title));

							let existe = node?node.children.find(c => c.id === b1.port_id):undefined;	
							let portId = (existe?existe.id:b1.port_id);
							ports.push({
								direction : "none", // in/out/none
								type : "option", // error/success/option
								option_label : b1.message?b1.message:b1.title, // error/success/option
								channels: "0", // 0, 1 o n
								id: portId,
								preset_value: {
									port_id: portId
								}
							});
						}
							
					});
				}
			});
		}

		return ports;
	};

	adm_process_flow_activity.prototype.__generate_default_port = function(data, node, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		let conditions = [];
		
		// agrego las condiciones para los errores
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Timeout\"" });
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"ChannelClosed\"" });
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

		if (acumulative_outputs.ports.length > 0) {

			let contador = 0;

			acumulative_outputs.ports.forEach(i => {

				contador++;

				conditions.push({ 
					begin_parenths: (contador === 1?"(":undefined),
					logic : ("and"), //logic, //conditions.length > 0?"and":undefined, 
					field: "decodeUnicodeEscape(" + data.result + ".event.message.text)",
					comparator: "~=", 
					value: "decodeUnicodeEscape(\""+ i +"\")" ,
					end_parenths: (contador === acumulative_outputs.ports.length?")":undefined)
				});

			});
			
			if(data["cognitiveServicesApp"] && data["cognitiveServicesApp"].length > 0) { // hay app de nlu

				// agrego una condición para validar el caso de NLU
				conditions.push({ begin_parenths: "((", logic : "and", field: data.result + ".event.nludata.topScoringIntent", comparator: "~=", value: "nil" });

				// agrego condición para el scoring ! todavía no agregué el control web para definir el scoring !
				conditions.push({ begin_parenths: "(", logic : "and", field: data.result + ".event.nludata.topScoringIntent.score", comparator: "<", value: data.luis_scoring });

				let contador_defaultB = 0;

				acumulative_outputs.ports.forEach(i => {

					contador_defaultB++;

					conditions.push({ 
						begin_parenths: (contador_defaultB === 1?"(":undefined),
						logic : (contador_defaultB === 1?"or":"and"), //logic, //conditions.length > 0?"and":undefined, 
						field: data.result + ".event.nludata.topScoringIntent.intent",
						comparator: "~=", 
						value: "\""+ i +"\"" ,
						end_parenths: (contador_defaultB === acumulative_outputs.ports.length?")))":undefined)
					});

				});

                // agrego una salida para cuando el topScoringIntent llega vacío a pesar de tener configurado NLU
                conditions.push({ begin_parenths: "", logic : "or", field: data.result + ".event.nludata.topScoringIntent", comparator: "==", value: "nil", end_parenths: ")" });
			}
			
		}

		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: (existe?existe.id:inConcert.Utils.generateGuid() + "_default"),  // le asigno un id por defecto
			preset_value: {
				description: "", //<%=_T('Default output')%>
				conditions: conditions
			}
		});
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_error_ports = function(data, node, unificar_errores) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		if (unificar_errores) {

			let existe = node?node.children.find(c => c.id.includes("_" + "Error")):undefined;

			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_Error");

			ports.push({
				direction : "out", // in/out
				type : "error", // error/success/option
				option_label : "Error", // error/success/option
				channels: "1", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", // Error
					conditions: [
						{ field: data.result + ".eventname", comparator: "==", value: "\""+ "Error" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "Timeout" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "ChannelClosed" +"\"" }
					],
					port_id: portId
				}
			});

		} else {

			let errores_admitidos = ["Error","Timeout","ChannelClosed"];
			errores_admitidos.forEach(error_type => {
				let existe = node?node.children.find(c => c.id.includes("_" + error_type)):undefined;
				let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" + error_type);
				ports.push({
					direction : "out", // in/out
					type : "error", // error/success/option
					option_label : error_type, // error/success/option
					channels: "1", // 0, 1 o n
					id: portId,  // le asigno un id por defecto
					preset_value: {
						description: "", //error_type
						conditions: [
							{ field: data.result + ".eventname", comparator: "==", value: "\""+ error_type +"\"" }
						],
						port_id: portId
					}
				});
			});

		}

		return ports;
	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function(data, flexible) { 

		const self = this;
		this.$get("[output_field]").each(function() {

			self.__getAndValidateField($(this), data);

			if (data.error && !flexible) {
				return false;
			}
		});

		// valido el slide scoring 
		this.__addValue(data, this.scoringDefinition); 
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function($field, data) {

		data.ports = data.ports || [];

		let field = {
			name: $field.find("[field_name]").val() || undefined,
			port_id: $field.attr("template_option") || undefined,
			outputs: []
		};

		//validation
		let error = null;
		if (field.name) {
			if (!field.name) {
				error = "<%=_T('Must enter a name.')%>";
			}
			let name = inConcert.Utils.htmlBodyToPlainText(field.name);
			if (name.trim().length === 0) {
				error = "<%=_T('Invalid name.')%>";
			}
		} 
		else {
			error = "<%=_T('Must enter a name.')%>";
		}

		$field.find("[ic-data-bind=option]").each(function() {
			let $field_i = $(this);

			let value = $field_i.text();
			let is_template = $field_i.attr("ic-role");

			if (is_template !== "template" && is_template !== "template-qr") {
				field.outputs.push(value);
			}
		});

        $field.find("[ic-data-bind=option-qr]").each(function() {
			let $field_i = $(this);

			let value = $field_i.text();
			let is_template = $field_i.attr("ic-role");

			if (is_template !== "template" && is_template !== "template-qr") { 
				field.outputs.push(value);
			}
		});

		if (error) {
			data.error = {
				message : error,
				$control: $field
			};
		}

        // aquí estoy intentando ver si con un prefijo en el nombre de la respuesta rápida puedo manejar este bloqueo
        if(field.name && field.name.startsWith(this.quick_reply_name_prefix)) {
            field.isQuickReply = true;
        }

		data.ports.push(field);

	};

	adm_process_flow_activity.prototype.__selectMinDigits = function (digit) {
		let self = this;
		self.maxDigitsDefinition.options = self.__getRange(digit, 30);

		let maxDigits = self.__getValue(self.maxDigitsDefinition);

		if (parseInt(maxDigits) > parseInt(digit)) {
			digit = maxDigits;
		}

		self.__replaceControl(self.maxDigitsDefinition, self.__makeComboFlex(self.maxDigitsDefinition, digit));

        self.__changeValidationDefinition(self.__getValue(self.validationDefinition), false);
	};

	adm_process_flow_activity.prototype.__makeOutputs = function () {

		const self = this;
		let ports = [];

		if (self.data.ports) {
			ports = self.data.ports || [];			
		}

		const $outputs = $(inConcert.templates.process_flows.menu_outputs);

		$outputs.find("[add_new_output]").click(function () {

			$outputs.find("[messages_output_option_form]").append(self.MakeOutputOptionForm(function(result) {
				
				if (result.option_text !== null && result.option_text !== undefined && result.option_text !== "") {

					let existe = self.output_options?self.output_options.find(c => c.name.toUpperCase() === result.option_text.toUpperCase()):undefined;

					if (!existe) {
						self.output_options.push({ name: result.option_text });

						setTimeout(function() { 
							const $metadata = $(inConcert.templates.process_flows.menu_outputs);
							self.loadOptions($metadata, function() {});
						}, 100);
					}		
				}
			}));
			
		});

        $outputs.find("[close_message]").click(function(){
            $outputs.find("[disclaimer]").remove();
        });

		const $optionsList = $outputs.find("[output_fields]");
		const $list = [];

        // agrego fields para las quickreplies
        if (self.data && self.data.message && self.data.message.quickreplies) {
            self.data.message.quickreplies.forEach(b => {

                let existe = ports?ports.find(c => c.port_id === b.port_id):undefined;

                if (!existe) {
                    let new_field_qr = {
                        name: self.quick_reply_name_prefix + b.title,
                        outputs: [],
                        port_id: b.port_id,
                        isQuickReply: true
                    };
                    $list.push(self.__makeField($outputs, new_field_qr));
                }
            });
        }

		$.each(ports, function(_, field) {
			$list.push(self.__makeField($outputs, field));
		});

		if (!$list.length) {
			$list.push(self.__makeField($outputs));
		}

		$optionsList.append($list);

        let cognitiveServicesApp = "";
		if (self.data && self.data.message && self.data.message.cognitiveServicesApp && self.data.message.cognitiveServicesApp !== "") { //cognitiveServicesApp
			cognitiveServicesApp = self.data.message.cognitiveServicesApp;
		}

        self.__loadIntents(cognitiveServicesApp, function() {
            setTimeout(function() {
                self.loadOptions($outputs, function() {});
            }, 100);
        });

        $outputs.find("[add_new_port]").click(function() {
			self.__disableNextKeyboardEvent();
			let $row = $(".flowDesignerMenuMetadataFieldsList").children(":last");
			let $newRow = self.__makeField($outputs);
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
			// actualizo el evento de drag and drop
			self.updateDragListener($outputs);
		});

		return $outputs;
	};

	adm_process_flow_activity.prototype.MakeOutputOptionForm = function (callback) {
		var self = this;
		self.$button = $(templates.messages_output);

		//text
		this.optionTextDefinition = {
			id: "option_text",
			name: "<%=_T('Text')%>",
			placeholder : "<%=_T('Enter the text')%>...",
			required: true,
			maxLength: 150,
            form_group: true
		};

		var $controls = [];
		$controls.push(this.__makeRowFlex(this.__makeInputFlex(this.optionTextDefinition, this.data[this.optionTextDefinition.id])));

		self.SelfOverlay(true);

		self.$button.find("[hyperlink_action=save]").click(function () {
			self.__saveEdit(callback);
		});
		self.$button.find("[hyperlink_action=cancel]").click(function () {
			self.__cancelEdit();
		});

		self.$button.find("[hyperlink_edit_form]").append($controls);
        $("[messages_output_option_form]").show();
		return self.$button;
	};

	adm_process_flow_activity.prototype.__saveEdit = function (callback) {

		let valor = {};

		this.__addValue(valor, this.optionTextDefinition, this.$button);

        if (!valor.error) {
            callback(valor);

            $("[messages_output_option_form]").hide();
            this.$button.remove();
            this.SelfOverlay(false);
        } else {
            return this.__showError(valor.error, this.$button);
        }
	};

	adm_process_flow_activity.prototype.__cancelEdit = function () {
        $("[messages_output_option_form]").hide();
		this.$button.remove();
		this.SelfOverlay(false);
	};

	adm_process_flow_activity.prototype.__loadIntents = function(app_id, callback) {

		let self = this;

        self.output_options = [];
        self.data.ports = self.data.ports || [];

        // agrego puertos para las quickrepies
        self.data.ports.forEach(element => {
            if (element.name && element.name.startsWith(self.quick_reply_name_prefix)) {
                self.output_options.push({
                    name: element.name.substring(self.quick_reply_name_prefix.length),
                    type: self.quick_reply_name_prefix
                });
            }
        });

        // agrego outputs para los botones interactivos de whatsapp
        if (self.data && self.data.message && self.data.message.interactive && self.data.message.interactive.type === "list" 
            && self.data.message.interactive.action && self.data.message.interactive.action.sections) {
                self.data.message.interactive.action.sections.forEach(section => {
                    if (section.rows) {
                        section.rows.forEach(row => {
                            self.output_options.push({
                                name: row.title
                            });
                        });
                    }
                });
        }

        if (self.data && self.data.message && self.data.message.interactive && self.data.message.interactive.type === "button" 
            && self.data.message.interactive.action && self.data.message.interactive.action.buttons) {
                self.data.message.interactive.action.buttons.forEach(button => {
                    self.output_options.push({
                        name: (button.title?button.title:button.reply.title)
                    });
                });
        }
        
        self.output_options.push(...self.interactive_buttons);

        if (app_id === "" || app_id === null || app_id === undefined) {        
            callback();
        } else {
            self.editor.main.LoadBotLuisIntents(app_id, function(intents) { 
                self.output_options.push(...intents);  
                callback();
            });
        }            
	};

	adm_process_flow_activity.prototype.__selectCognitiveService = function (app) {
		let self = this;

		self.__loadIntents(app, function() {
            setTimeout(function() { 
                const $metadata = $(inConcert.templates.process_flows.menu_outputs);
                self.loadOptions($metadata, function() {});
            }, 100);
        });		
	};

	adm_process_flow_activity.prototype.__makeField = function($outputs, field) {

		field = field || {};
		let self = this;
		let $field = $(Mustache.render(inConcert.templates.process_flows.menu_outputs_field, { 
			guid : field.port_id?field.port_id:inConcert.Utils.generateGuid(), 
			hidden : true,
            placeholder: "<%=_T('Enter the name of the output port')%>",
            isQuickReply: field.isQuickReply
		}));

        if ($(".flowDesignerOutputPortsContainer").length === 0 || field.name !== undefined) {
            $field.find("[field_name]").val(field.name || "Port " + ($(".flowDesignerOutputPortsContainer").length + 1));
            $field.find("[field_name_label]").html(field.name || "Port " + ($(".flowDesignerOutputPortsContainer").length + 1));
        }

        // si no es una quickreply, se agrego el botón de eliminar
        if (!field.isQuickReply) {
            $field.find("[remove]").click(function() {
                
                self.__disableNextKeyboardEvent();
                let $row = $(this).parent();
                $row = $row.parent();
                $row = $row.parent();
                $row = $row.parent();
                let $siblings = $row.siblings();

                // acualizo la definición de los puertos

                let current_data = self.__getAndValidate(true); 
                self.data.ports = current_data.ports;

                // elimino el puerto actual en caso de que exista
                for (let index = 0; index < self.data.ports.length; index++) {
                    const p = self.data.ports[index];
                    
                    if ($field.attr("template_option") === p.port_id) {
                        self.data.ports.splice(index, 1);
                        break;
                    }
                }
                
                if ($siblings.length) {
                    if ($row.prev().length) {
                        $row.prev().find("[field_name]").focus();
                    }
                    else {
                        $row.next().find("[field_name]").focus();
                    }
                }
                else {
                    var $newRow = self.__makeField($outputs);
                    $row.after($newRow);
                    $newRow.find("[field_name]").focus();
                }
                $row.remove();

                // actualizo las opciones
                self.loadOptions($outputs, function() {});
                
            });
        } else {
            // oculto el botón de eliminar
            $field.find("[remove]").hide();
        }
            
		return $field;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function(flexible) { 
		
		const data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		const drawVersion = this.editor.main.__getVersion(); 

		if (drawVersion === "2") {

			if (!data.error) {
				this.__getAndValidateFields(data, flexible);
                this.__getAndValidateOutputTitles(data);
			}
			this.__addValue(data, this.scoringDefinition);	
		}

		return data;
	};

    adm_process_flow_activity.prototype.deleteQuickReplyPort = function(port_id) 
    {   
        const self = this;

        if (self.drawVersion === "2") {
            let current_data = self.__getAndValidate(true); 
            self.data.ports = current_data.ports;

            // elimino el puerto actual en caso de que exista
            for (let index = 0; index < self.data.ports.length; index++) {
                const p = self.data.ports[index];
                
                if (port_id === p.port_id) {

                    // elimino el output que sea predefinido para quickreply, dado que solo debe tener 1
                    for (let index2 = 0; index2 < p.outputs.length; index2++) {
                        const name = p.outputs[index2];

                        if (name === p.name.substring(self.quick_reply_name_prefix.length)) {
                            // elimino el output antiguo de la lista de outputs del port
                            p.outputs.splice(index2, 1);
                            // también elimino el output de output_options
                            self.output_options = self.output_options.filter(function(value, _, _1){ 
                                return value.name !== p.name.substring(self.quick_reply_name_prefix.length);
                            });

                            break;
                        }
                    }

                    self.data.ports.splice(index, 1);
                    break;
                }
            }

            $("#rd_menu_option_selector_receiver").children("ul").each(function () {
                $(this).children("li").each(function () {
                    if ($(this).attr("template_option") === port_id) {
                        $(this).remove();                    
                    }
                });
            });

            // actualizo las opciones
            setTimeout(function() {
                self.loadOptions($(inConcert.templates.process_flows.menu_outputs), function() {});
            }, 100);
        }
    };

    adm_process_flow_activity.prototype.saveEditQuickReplyPort = function(port_id, actual_title, title, type, update)
    {   
        const self = this;

        if (self.drawVersion === "2") {
            if (!title) {
                title = type;
            }

            let current_data = self.__getAndValidate(true); 
            self.data.ports = current_data.ports;
			
            // agrego fields para las quickreplies
            let existe = self.data.ports?self.data.ports.find(c => c.port_id === port_id):undefined;
            const $outputs = $("#rd_menu_option_selector_receiver");

            if (!existe) {

                let field = {
                    name: self.quick_reply_name_prefix + title,
                    port_id: port_id,
                    outputs: [],
                    isQuickReply: true
                };

				if (self.data.ports) {
                	self.data.ports.push(field);
				}

                const $optionsList = $outputs.find("[output_fields]");
                const $list = [];

                let new_field_qr = {
                    name: self.quick_reply_name_prefix + title,
                    outputs: [],
                    port_id: port_id,
                    isQuickReply: true
                };

                $list.push(self.__makeField($outputs, new_field_qr));				
                $optionsList.append($list);
            } else {
                // si existe debo actualizar el nombre
                existe.name = self.quick_reply_name_prefix + title;
                existe.actual_title = actual_title;

                $("#rd_menu_option_selector_receiver").children("ul").each(function () {
                    $(this).children("li").each(function () {
                        if ($(this).attr("template_option") === port_id) {
                            $(this).find("[field_name]").val(self.quick_reply_name_prefix + title);
                            $(this).find("[field_name_label]").html(self.quick_reply_name_prefix + title);
                        }
                    });
                });
            }

			if (update) {
				// actualizo las opciones
				setTimeout(function() {
					self.loadOptions($outputs, function() {});
				}, 100);
			}
        }
    };

	adm_process_flow_activity.prototype.saveEditHsmPorts = function(buttons)
    {   
        const self = this;
        if (self.drawVersion === "2") {

			let current_data = self.__getAndValidate(true); 
            self.data.ports = current_data.ports;

			$.each(self.data.ports, function (_, port) {
				if (port.name && port.name.startsWith(self.quick_reply_name_prefix)) {
					let existe = buttons.find(c => self.quick_reply_name_prefix + c.title === port.name);
					if (!existe) {
						self.deleteQuickReplyPort(port.port_id);
					}				
				}
			});
			// agrego los nuevos
			$.each(buttons, function (_, btn) {
				let existe = false;
				if (self.data.ports) {
					existe = self.data.ports.find(c => c.name === self.quick_reply_name_prefix + btn.title);
				}
				// Issue: 20549. Se crean puertos unicamente para los botones que no sean de tipo call to action.
				if (!existe && btn.type === "quick_reply") {
					self.saveEditQuickReplyPort(btn.guid, btn.title, btn.title, btn.type, false);
				}
			});

			setTimeout(function() {
				const $outputs = $("#rd_menu_option_selector_receiver");
				self.loadOptions($outputs, function() {});
			}, 100);
        }
    };

    adm_process_flow_activity.prototype.saveEditInteractiveButton = function(data) 
    {   
        const self = this;

        if (data.error) {
            return false;
        }

        if (self.drawVersion === "2") {

            let current_data = self.__getAndValidate(true); 
            self.data.ports = current_data.ports || [];
            let existe = false;

            if (!data.title_original) {
                data.title_original = data.title;
            }

            for (let i = 0; i < self.data.ports.length; i++) { // busco dentro de los puertos existentes
                self.data.ports[i].outputs = self.data.ports[i].outputs || {};
                for (let i2 = 0; i2 < self.data.ports[i].outputs.length; i2++) {
                    if (self.data.ports[i].outputs[i2].toUpperCase() === data.title_original.toUpperCase()) {
                        self.data.ports[i].outputs[i2] = data.title;
                        existe = true;
                    }
                }
            }

            if (self.data && self.data.message && self.data.message.interactive && self.data.message.interactive.type === "button" 
                && self.data.message.interactive.action && self.data.message.interactive.action.buttons) {                        
                    for (let x = 0; x < self.data.message.interactive.action.buttons.length; x++) {
                        if (self.data.message.interactive.action.buttons[x].reply.title.toUpperCase() === data.title_original.toUpperCase()) {
                            self.data.message.interactive.action.buttons[x].reply.title = data.title;
                            existe = true;
                        }
                    }
            }

            if (self.data && self.data.message && self.data.message.interactive && self.data.message.interactive.type === "list" 
                && self.data.message.interactive.action && self.data.message.interactive.action.sections) {
                    for (let y = 0; y < self.data.message.interactive.action.sections.length; y++) {
                        for (let y2 = 0; y2 < self.data.message.interactive.action.sections[y].rows.length; y2++) {
                            if (self.data.message.interactive.action.sections[y].rows[y2].title.toUpperCase() === data.title_original.toUpperCase()) {
                                self.data.message.interactive.action.sections[y].rows[y2].title = data.title;
                                existe = true;
                            }                                
                        }
                    }
            }
            
            for (let z = 0; z < self.interactive_buttons.length; z++) {
                if (self.interactive_buttons[z].name.toUpperCase() === data.title_original.toUpperCase()) {
                    self.interactive_buttons[z].name = data.title;
                    existe = true;
                }
            }

            if (!existe) {
                self.data.ports.forEach(port => {
                    port.outputs = port.outputs || {};
                    port.outputs.forEach(output => {
                        if (output.toUpperCase() === data.title.toUpperCase()) {
                            existe = true;
                        }
                    });          
                });

                if (!existe) {
                    self.interactive_buttons.push({ name: data.title });
                }
            }

            let app = self.__getValue(self.cognitiveServicesDefinition);
            self.__loadIntents(app, function() {
                setTimeout(function() { 
                    const $metadata = $(inConcert.templates.process_flows.menu_outputs);
                    self.loadOptions($metadata, function() {});
                }, 100);
            });
        }

    };

	adm_process_flow_activity.prototype.loadOptions = function($metadata, callback) 
	{
		const self = this;

		try {
			if (self.data && self.data.ports) {
				self.data.ports.forEach(p => {
					if (p.outputs) {
						p.outputs.forEach(o => {
							let existe = self.output_options?self.output_options.find(c => c.name.toUpperCase() === o.toUpperCase()):undefined;
							if (!existe) {
								self.output_options.push({ name: o });
							}
						});
					} else {
                        p.outputs = [];
                    }

                    // busco si tiene una pastilla por defecto, si no la tiene la agrego
                    if (p.name && p.name.startsWith(self.quick_reply_name_prefix)) { // significa que es una QR

                        // primero elimino el output que sea predefinido para quickreply, dado que solo debe tener 1
                        for (let index = 0; index < p.outputs.length; index++) {
                            const actual_title = p.outputs[index];
                            if (actual_title === p.actual_title) {
                                // elimino el output antiguo de la lista de outputs del port
                                p.outputs.splice(index, 1);
                                // también elimino el output de output_options
                                self.output_options = self.output_options.filter(function(value, _, _1){ 
                                    return value.name !== p.actual_title;
                                });

                                break;
                            }
                        }

                        let existe = p.outputs.find(c => c === p.name.substring(self.quick_reply_name_prefix.length));
                        if (!existe) {
                            p.outputs.push(p.name.substring(self.quick_reply_name_prefix.length));
                            self.output_options.push(
                                { 
                                    name: p.name.substring(self.quick_reply_name_prefix.length),
                                    type: self.quick_reply_name_prefix
                                }
                            );
                        }
                    }

				});
			}

			let context = {
				listOptionsAvail    : $("#rd_menu_available_option_list"),
				listOptionsInUse    : $metadata.find("[ic-data-bind=options]")
			};

			this.context = $.extend(context, this.context);

			function renderSummary (callback) {

				self.setupOptionsSelector({
					callback  : function() {
						self.updateDragListener($metadata);
						callback(true);
					},
					update : function() {

						context.listOptionsAvail
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).addClass("sorting-initialize");
								context.listOptionsAvail.sortable("refresh");
							});

						context.listOptionsInUse
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
                                if ($(this).attr("ic-data-bind") !== "option-qr") {
                                    $(this).addClass("sorting-initialize");
                                    context.listOptionsInUse.sortable("refresh");
                                }
							});
					}
				});
			} 

			renderSummary(function(success, errorThrown) {
				if (success) {
					callback(true, errorThrown);

				} else {
					callback(false, errorThrown);
				}
			});
		} finally {
			self.__processing(false);
		}
	
	};

	adm_process_flow_activity.prototype.setupOptionsSelector = function(params) {

		let self = this;
		let constructor_callback = params.callback || $.noop;
		let update               = params.update   || $.noop;
		let $list_container      = $("#rd_menu_output_selector").children("[ic-role='list-container']");
		let $options_available   = $list_container.find("[ic-role='options-available']");

		// limpio los options existentes antes de hacer todo esto
		let template = $options_available.children("[ic-role='template']");
		let template_qr = $options_available.children("[ic-role='template-qr']");
		$options_available.empty().append(template);
		$options_available.append(template_qr);
		
		// también limpio los puertos existentes
		$("#rd_menu_option_selector_receiver").children("ul").each(function () {
			$(this).children("li").each(function () {
				let $options_receive = $(this).find("[ic-data-bind='options']");
				let template = $options_receive.children("[ic-role='template']");
				let template_qr = $options_receive.children("[ic-role='template-qr']");
				$options_receive.empty().append(template);
				$options_receive.append(template_qr);
			});
		});

		// agrego los options a los puertos
		inConcert.Utils.foreach_timeout({
			rows : self.output_options,
			max_append_per_loop: 2,
			pause_between_loop_ms: 10,
			append_fn: function( item )
			{
				let existe = false;

				if (self.data.ports) {

					self.data.ports.forEach(p => {
						if (p.outputs) {
							p.outputs.forEach(i => {

								if (i === item.name) {
									existe = true;

									$("#rd_menu_option_selector_receiver").children("ul").each(function () {

										$(this).children("li").each(function () {

											if ($(this).attr("template_option") === p.port_id) {

												let existe_option = false;

												if (!existe_option) {
													let $options_receive = $(this).find("[ic-data-bind='options']");
                                                    let $template_html_receive;

                                                    if (item.type === self.quick_reply_name_prefix) {
                                                        $template_html_receive = $("<div>").append(
                                                            $options_receive.children("[ic-role='template-qr']")
                                                                .clone()
                                                                .show()
                                                                .removeAttr("ic-role")
                                                                .attr("ic-data-bind", "option-qr")
                                                        ).html();
                                                    } else {
                                                        $template_html_receive = $("<div>").append(
                                                            $options_receive.children("[ic-role='template']")
                                                                .clone()
                                                                .show()
                                                                .removeAttr("ic-role")
                                                                .attr("ic-data-bind", "option")
                                                        ).html();
                                                    }

                                                    let item_html = $template_html_receive.replace("{name}", item.name);
                                                    let item_result = $(item_html).appendTo($options_receive);

													return item_result;
												} else {
													return;
												}
												
											}
										});
									});
								}
							});
						}
					});

				}

				if (!existe) {

					let existe_option = false;

					if (!existe_option) { 

                        let $template_html_available;

                        if (item.type === self.quick_reply_name_prefix) { // le desactivo el drag and drop
                            $template_html_available = $("<div>").append(
                                $options_available.children("[ic-role='template-qr']")
                                    .clone()
                                    .show()
                                    .removeAttr("ic-role")
                                    .attr("ic-data-bind", "option-qr")
                                ).html();
                        } else {
                            $template_html_available = $("<div>").append(
                                $options_available.children("[ic-role='template']")
                                    .clone()
                                    .show()
                                    .removeAttr("ic-role")
                                    .attr("ic-data-bind", "option")
                                ).html();
                        }

                        let item_html = $template_html_available.replace("{name}", item.name);	
                        let item_result = $(item_html).appendTo($options_available);

						return item_result;
					} else {
						return;
					}
					
				}
			},
			complete_fn: function()
			{
				constructor_callback();
				update();
			}
		});

		return {
			reset: function() {}
		};		
	};

	adm_process_flow_activity.prototype.updateDragListener = function($metadata) {

		let context = {
			listOptionsAvail    : $("#rd_menu_available_option_list"), 
			listOptionsInUse    : $metadata.find("[ic-data-bind=options]")
		};

		context.listOptionsAvail.sortable({
			cursor: "move",
			connectWith: ".ui-options-receive",
			items: ".sorting-initialize",
			receive: function( _ ) {}
		});

		context.listOptionsInUse.sortable({
			cursor: "move",
			connectWith: "#" + context.listOptionsAvail.uniqueId().attr("id"),
			items: ".sorting-initialize",
			receive: function( _ ) {}
		});
		
	};
	
}());
