/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;

	var id = "send_chat_message";
	
/*
 * BASE
 */

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		
		this.resultDefinition = { 
			id : "result",
			name : "<%=_T('Result')%>",
			placeholder : "<%=_T('Enter result')%>...",
			description : "<%=_T('Variable name to store result (boolean value indicating success).')%>",
            required: true,
            hidden: true
		};
		
		this.type = editor.main.data.Type;
		this.messageId = "message";
		this.messagesEditor = new inConcert.adm_process_flow_messages[this.type.toLowerCase()](this);
		
		this.focusFirst = true;
        //this.without_buttons = true;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Send message')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Send a message.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.channels = {
		"CALL" : true, 
		"CHAT" : true, 
		"FACEBOOK_BOT" : true,
		"INSTAGRAM_BOT" : true,
		"TWITTER_BOT" : true,
		"MAIL" : true,
		"SMS" : true,
		"WHATSAPP" : true
	};
	adm_process_flow_activity.prototype.image = "ico-send-chat.svg";

	adm_process_flow_activity.prototype.GetMigrationData = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetMigrationData(messages, type, {}, data);
	};
	
	adm_process_flow_activity.prototype.dataStyle = function(data) {
		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			return id + "_" + type;
		}
		return null;
	};
	
	adm_process_flow_activity.prototype.dataStyles = function(type) {
		var types = inConcert.adm_process_flow_messages_type[type.toLowerCase() + "_sorted"] || [];
		var styles = [];
		$.each(types, function(_, type) {
			styles.push({
				id : id + "_" + type,
				imageName : format("ico-send-{0}-message.svg", type)
			});
		});
		["image", "video", "audio", "file"].forEach(function(fileType) {
			styles.push({
				id : id + "_" + fileType,
				imageName : format("ico-send-{0}-message.svg", fileType)
			});
		});
		return styles;
	};
	
	adm_process_flow_activity.prototype.__hasKeyboardEvents = function() {
		return false;
	};
	
	adm_process_flow_activity.prototype.__isFullScreen = function() {
		return false;
	};

	adm_process_flow_activity.prototype.__makeFormContent = function($content) { 

        if (!this.data.result) {
            this.data.result = inConcert.Utils.generateGuid();
        }

        $content.unshift(
            this.__makeDivFlex(this.__makeDivFlex(
                this.__makeDivFlex(this.__makeInputFlex(this.resultDefinition, this.data[this.resultDefinition.id]), { class: "form-group" }),{ class: "col-12" }
            ),
            { class: "row d-none" })
        );

        if (this.type === "MAIL" || this.type === "WHATSAPP" || this.type === "TWITTER_BOT" || this.type === "SMS" || this.type === "INSTAGRAM_BOT") {
            this.hide_configuration_tab = true;
        }

		return $content;
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		let $content = this.__makeFormContent(adm_process_flow_activity.uber.__makeContent.call(this));	
		return this.messagesEditor.Make($content, this.data[this.messageId], undefined, undefined, undefined, this.hide_configuration_tab);
	};
	
	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		this.messagesEditor.AfterAppendBody();
	};
	
	adm_process_flow_activity.prototype.__getAndValidateForm = function(data) {
		this.__addValue(data, this.resultDefinition);
	};
	
	adm_process_flow_activity.prototype.__getAndValidateAsync = function(data, callback) {
		this.__getAndValidateForm(data);
		this.messagesEditor.AddValue(data, this.messageId, function() {
			callback(data);
		});
	};
	
	adm_process_flow_activity.prototype.__getAndValidateFormMessage = function(message) {
		
	};
	
/*
 * CALL
 */

	function adm_process_flow_activity_call(editor, node) {
		let self = this;
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);

		this.ttsVoices;
		
		this.asyncDefinition = {
			id : "async",
			name : "<%=_T('Async')%>",
			description : "<%=_T('Start playing and continue execution or wait until the end.')%>",
			not_extended : true,
			change : function() {
				self.__selectAsync($(this).is(":checked"));
			},
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.loopDefinition = {
			id : "loop",
			name : "<%=_T('Loop')%>",
			description : "<%=_T('Loop playing audio (only for async mode).')%>",
			not_extended : true,
            form_group: false,
            envolving_label_class: "form-check-label form-check-label-title",
            envolving_label: true,
            without_label: true,
            controlContainerClass: "form-check mt-0",
            checkBoxClass: "form-check-input"
		};
		
		this.EndDigitDefinition = {
			id : "endDigit",
			name : "<%=_T('End Digit')%>",
			description : "<%=_T('Digit to end capture.')%>",
			not_extended : true,
			fieldClass : "Short",
			options : this.__getPhoneDigitsOptions(),
			emptyLabel : "<%=_T('Nothing')%>",
            form_group: true
		};

		this.voiceDefinition = {
			id : "voiceId",
			name : "<%=_T('Voice')%>",
			description : "<%=_T('TTS voice to use.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};
	}

	inConcert.Utils.inherit( adm_process_flow_activity_call, adm_process_flow_activity );
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);

	adm_process_flow_activity_call.prototype.__load = function(callback) {
		const self = this;

		this.editor.main.LoadTtsAsrEngines(function(success, ttsasrEngines) {
			self.ttsasrEngines = success ? ttsasrEngines : undefined;
			if(self.ttsasrEngines && $.isArray(self.ttsasrEngines)) {
				self.ttsasrEngines.forEach(function(engine) {
					if((engine.providerType === "TTS" || engine.providerType === "TTS-ASR") && engine.voices && $.isArray(engine.voices) && engine.voices.length > 0) {
						self.ttsVoices = [];
						engine.voices.forEach(function(voice) {
							let voice_to_show = {
								id : voice.id,
								name : voice.LanguageName + " - " + voice.Variant + " - " + voice.Gender + " - " + voice.VoiceName
							};
							self.ttsVoices.push(voice_to_show);
						});
					}
				});
				
				self.isTtsEnabled = (self.ttsVoices && $.isArray(self.ttsVoices) && self.ttsVoices.length > 0);
				
				self.voiceDefinition.options = self.ttsVoices;
			}
			callback();
		});
	};
	
	adm_process_flow_activity.prototype.__getClassName = function() {
    return "popOver640w flowDesignerGlobalModal tabsPopOver botActivity_send_call";
	};

	adm_process_flow_activity_call.prototype.__makeFormContent = function($content) { 
		$content = adm_process_flow_activity_call.uber.__makeFormContent.call(this, $content);


        $content.push(
            this.__makeDivFlex(
              this.__makeDivFlex(this.__makeComboFlex(this.voiceDefinition, this.data.message && this.data.message[this.voiceDefinition.id]),{ class: "col-12" }),{ class: "row" }
            )
        );

        $content.push(
            this.__makeDivFlex(
              this.__makeDivFlex(this.__makeComboFlex(this.EndDigitDefinition, this.data.message && this.data.message[this.EndDigitDefinition.id]),{ class: "col-12" }),{ class: "row" }
            )
        );

        $content.unshift(
            this.__makeDivFlex([
                this.__makeDivFlex(
                    this.__makeDivFlex(
                        this.__makeCheckboxFlex(this.asyncDefinition, this.data.message && this.data.message[this.asyncDefinition.id])
                    ,{ class: "form-group" })
                ,{ class: "col-6" }),
                this.__makeDivFlex(
                    this.__makeDivFlex(
                        this.__makeCheckboxFlex(this.loopDefinition, this.data.message && this.data.message[this.loopDefinition.id])
                    ,{ class: "form-group" })
                ,{ class: "col-6" })
            ]
            ,{ class: "row" })
        );

		return $content;
	};
	
	adm_process_flow_activity_call.prototype.__selectAsync = function(async) {
		this.__disableEnableControl(this.loopDefinition, !async);
		if (!async) {
			this.__setValue(this.loopDefinition, false);
		}
	};
	
	adm_process_flow_activity_call.prototype.__afterAppendBody = function() {
		adm_process_flow_activity_call.uber.__afterAppendBody.call(this);
		this.__selectAsync(this.data.message && this.data.message[this.asyncDefinition.id]);
		
		if(this.isTtsEnabled)
		{
			this.AfterSelectType();
		}
		else
		{
			this.__hideControl(this.voiceDefinition);
		}
	};
	
	adm_process_flow_activity_call.prototype.__getAndValidateFormMessage = function(message) {
		adm_process_flow_activity_call.uber.__getAndValidateFormMessage.call(this, message);
		this.__addValue(message, this.asyncDefinition);
		this.__addValue(message, this.loopDefinition);
		this.__addValue(message, this.EndDigitDefinition);
		this.__addValue(message, this.voiceDefinition);
		if(this.isTtsEnabled && this.messagesEditor.currentType === "text") {
			message.promptOptions = {};
			//El i (digits to allow the TTS to be interrupted with) lo seteamos a partir del EndDigitDefinition
			let i = this.__getValue(this.EndDigitDefinition);
			if(i){
				message.promptOptions.i = i;
			}
		}
	};
	
	adm_process_flow_activity_call.prototype.AfterSelectType = function() {
		let currentMessageType = this.messagesEditor.currentType;
		if(currentMessageType === "text")
		{
			this.__showControl(this.voiceDefinition);
		}
		else {
			this.__hideControl(this.voiceDefinition);
		}
	};

	adm_process_flow_activity.prototype.htmlBody = function(data) {

		let image = "";

		if (data) {
			let type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Send message')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) { 
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});


		if (data.message.voiceId !== undefined) { // es un IVR
		
			// agrego puerto único por defecto
			let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
            const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
			ports.push({
				direction : "out", // in/out
				type : "option", // error/success/option
				option_label : "<%=_T('Default output')%>", // error/success/option
				channels: "n", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", //<%=_T('Default output')%>
					conditions: [],
					port_id: portId
				},
				intern_value: undefined
			});

		} else { // es un CHAT
			
			// agrego puerto único por defecto
			let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
			const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
			ports.push({
				direction : "out", // in/out
				type : "option", // error/success/option
				option_label : "<%=_T('Default output')%>", // error/success/option
				channels: "n", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", //<%=_T('Default output')%>
					conditions: [],
					port_id: portId
				},
				intern_value: undefined
			});
		}

		return ports;
	};

}());
