/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const id = "set_automatic_message";
	
/*
	* BASE
*/
	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		const self = this;

		this.messageTypeDefinition = {
			id : "messageType",
			name : "<%=_T('Type')%>",
			description : "<%=_T('Type of message to be overrided.')%>",
			required : true,
			form_group: true,
			with_label:true,
			labelClass: "required",
			options : [
				{
					id : "waiting_for_agent",
					name : "<%=_T('Music on ACD')%>",
					channels : { CALL : true },
					allowText : false
				},
				{
					id : "waiting_for_agent_with_ewt",
					name : "<%=_T('Waiting message on ACD')%>",
					channels : { CALL : true },
					allowText : true
				},
				{
					id : "voiceMail",
					name : "<%=_T('Voice Mail')%>",
					channels : { CALL : true },
					allowText : true
				},
				{
					id : "music_on_hold",
					name : "<%=_T('Music on hold')%>",
					channels : { CALL : true },
					allowText : false,
					// configuracion para hacer override de los valores soportados en ABM de multiples shared files (variables, audios)
					multiSharedFiles: {
						allowsVariables: false, // si el atributo no se encuentra o se define como undefined el mismo vale TRUE
						maxAudioFiles  : 1      // si el atributo no se encuentra o se define como undefined el tope es infinito
					}
				},
				{
					id : "transfer_failure",
					name : "<%=_T('Transfer failure')%>",
					channels : { CALL : true },
					allowText : true
				},
				{
					id : "without_agent_message",
					name : "<%=_T('Without Agent Message')%>",
					channels : { SMS : true }
				},
				{
					id : "on_dangling_message",
					name : "<%=_T('On hanging up message')%>",
					channels : { SMS : true }
				},
				{
					id : "on_conversation_finished",
					name : "<%=_T('On Conversation Finished')%>",
					channels : { SMS : true }
				},
				{
					id : "finalization_message",
					name : "<%=_T('Finalization Message')%>",
					channels : { WHATSAPP : true, CHAT : true }
				},
				{
					id : "waiting_message",
					name : "<%=_T('Waiting Message')%>",
					channels : { CHAT : true }
				},
				{
					id : "announce_chatting_agent",
					name : "<%=_T('Announce Chatting Agent')%>",
					channels : { CHAT : true }
				},
				{
					id : "without_agent_message",
					name : "<%=_T('Without Agent Message')%>",
					channels : { CHAT : true }
				},
				{
					id : "on_dangling_message",
					name : "<%=_T('On hanging up message')%>",
					channels : { CHAT : true }
				}
			].filter(function(value) {
				return !value.channels || value.channels[editor.main.data.Type] || false;
			}),
			change: function() {
				let selectedOption = $(".formCombo option:selected").val();
				let currentMessageDefinition = self.messageTypeDefinition.options.find(obj => {
					return obj.id === selectedOption;
				});
				if (currentMessageDefinition && currentMessageDefinition.allowText) {
					//Esta opción SI permite texto
					$(".textMessage").show();
				}
				else {
					//Esta opción NO permite texto
					self.messagesEditor.__selectType("attachment");
					$(".textMessage").hide();
				}
				if (currentMessageDefinition && self.messagesEditor.typeEditor && self.messagesEditor.typeEditor.setMultiSharedFilesMessage) {
					// se realiza override de los valores de multiples shared files
					self.messagesEditor.typeEditor.setMultiSharedFilesMessage(currentMessageDefinition.multiSharedFiles);
				}
			}
		};

		this.messageTypes = null;
		switch(editor.main.data.Type) {
			case "CHAT":
			case "WHATSAPP":
				this.messageTypes = ["text"];
			break;
		}

		this.type = editor.main.data.Type;
		this.messageId = "message";
		this.messagesEditor = new inConcert.adm_process_flow_messages[this.type.toLowerCase()](this);
		
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Automatic Message')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Override campaign automatic messages.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.channels = {
		"CALL" : true
	};
	adm_process_flow_activity.prototype.image = "ico-automatic-message.svg";

	adm_process_flow_activity.prototype.GetMigrationData = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetMigrationData(messages, type, {}, data);
	};

	adm_process_flow_activity.prototype.__hasKeyboardEvents = function() {
		return false;
	};

	adm_process_flow_activity.prototype.__isFullScreen = function() {
		return true;
	};

	adm_process_flow_activity.prototype.__makeTopContent = function() {
		let $content = [];
		$content.unshift(this.__makeRowFlex(
			this.__makeComboFlex(this.messageTypeDefinition, this.data[this.messageTypeDefinition.id]))
		);

		return $content;
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		let $content = this.__makeFormContent(adm_process_flow_activity.uber.__makeContent.call(this));	
		let $top = this.__makeTopContent();
		let $tabs = this.messagesEditor.Make($content, this.data[this.messageId], {
			types : this.messageTypes,
			$top : $top
		});
		$tabs.filter("#tabs-message").hide();
		return $tabs;
	};

	adm_process_flow_activity.prototype.__getClassName = function() { 
		return "popOver640w flowDesignerGlobalModal botActivity_automatic_message popOverFixedHeight";
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		this.messagesEditor.AfterAppendBody();
		this.messageTypeDefinition.change();
	};

	adm_process_flow_activity.prototype.__getAndValidateForm = function(data) {
		this.__addValue(data, this.messageTypeDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateAsync = function(data, callback) {
		this.__getAndValidateForm(data);
		this.messagesEditor.AddValue(data, this.messageId, function() {
			callback(data);
		});
	};

/*
 * CALL
 */

	function adm_process_flow_activity_call(editor, node) {
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);
	}

	inConcert.Utils.inherit( adm_process_flow_activity_call, adm_process_flow_activity );
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);

	adm_process_flow_activity_call.prototype.__makeFormContent = function($content) {
		return $content;
	};

	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Automatic Message')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function (data, node) { 


		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction: "in", // in/out
			type: "option", // error/success/option
			option_label: "Input" // error/success/option
		});

		// agrego puerto único por defecto
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option/none
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: {
				description: "",
				conditions: [],
				port_id: portId
			},
			intern_value: undefined
		});

		return ports;
	};

}());
