/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, ace, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	var id = "sql";

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		var self = this;
		this.frm = editor.main.frm;
		editor.main.LoadVariablesSet();

		this.datasourceDefinition = {
			id: "datasource",
			name: "<%=_T('Datasource')%>",
			description: "<%=_T('Query database connection.')%>",
			required: true,
			not_extended: true,
			form_group: true
		};

		this.resultDefinition = {
			id: "result",
			name: "<%=_T('Result')%>",
			placeholder: "<%=_T('Enter variable for result')%>...",
			description: "<%=_T('Variable name to store result ({ status : boolean, data : query result|number of modified rows|error message }).')%>",
			with_label: true,
			form_group: true,
		};

		this.autocompleteVariableDefinition = {
			id: "autocompleteVariable",
			placeholder: "<%=_T('Search Variable')%>...",
			description: "<%=_T('Select an existing variable.')%>",
			with_label: false,
			with_dots: false,
			required: false,
			not_extended: true,
			form_group: true,
			options : Array.from(editor.main.variables),
			change : function () {
				self.addVariableToCodeInput();
			}
		};

	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('SQL')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Run a SQL query.')%>";
	adm_process_flow_activity.prototype.group = "middleware";
	adm_process_flow_activity.prototype.image = "ico-sql.svg";

	adm_process_flow_activity.prototype.__hasKeyboardEvents = function () {
		return false;
	};

	adm_process_flow_activity.prototype.__load = function (callback) {
		var self = this;
		self.editor.main.GetDatasources(function (datasources) {
			var datasources_a = [];
			var selectedDatasources = self.editor.main.data.Datasources || {};
			$.each(datasources, function (dsId, dsData) {
				if (!selectedDatasources[dsId]) {
					return;
				}
				var dsType = inConcert.flowDesignerDatasourcesTypes[dsData.Type];
				if (!dsType || dsType.type !== "sql") {
					return;
				}
				datasources_a.push({
					id: dsId,
					name: format("{0} - {1}", dsType.name, dsData.Name),
					disclaimer: format("<%=_T('Server: {0}:{1}. Username: {2}. Database: {3}. Description: {4}')%>",
						dsData.Server, dsData.Port, dsData.Username, dsData.Database || dsData.DatabaseText, dsData.Description || "")
				});
			});
			datasources_a.sort(function (a, b) {
				return a.name.localeCompare(b.name);
			});
			self.datasourceDefinition.options = datasources_a;
			callback();
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver640w flowDesignerGlobalModal botActivity_sql";
	};

	adm_process_flow_activity.prototype.__makeCodeEditor = function () {
		var self = this;
		var $container = $(Mustache.render(templates.code_block, {
			brackets: "<%=_T('You can use {{ }} to enter variables inside the query. Quotes are auto escaped.')%>",
		}));
		self.codeEditor = ace.edit($container.find("[code_block]")[0]);
		self.codeEditor.getSession().setMode(format("ace{0}mode{0}sql", "/"));
		self.codeEditor.renderer.setScrollMargin(8, 8);
		self.codeEditor.getSession().setValue(self.data.code || "");
		return $container;
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		this.codeEditor.focus();
		this.__makeAutocompleteVariableDropDownInput();
	};

	adm_process_flow_activity.prototype.__makeAutocompleteVariableDropDownInput = function () {
		const $autocompleteInput = this.$get("[control=autocompleteVariable]");
		$autocompleteInput.siblings("label").remove();
		$autocompleteInput.select2( {
			language: {
				noResults: function(){
					return "<%=_T('No results found.')%>";
				}
			},
			placeholder: "VARIABLES",
			templateSelection: function() {
				return "VARIABLES";
			  }
		});
	};

	// adm_process_flow_activity.prototype.__makeAutocompleteVariableDropDownInput = function () {
	// 	const self = this;
	// 	const $autocompleteInput = $("[control=autocompleteVariable]");
	// 	$autocompleteInput.siblings("label").remove()
	// 	$autocompleteInput.select2();

	// 	this.$get("#processFlowAutocompleteVariableButton").click( function () {
	// 		const $autocompleteCol = $("[control=autocompleteVariable]").closest(".col-autocompletecombo");
	// 		$autocompleteCol.toggle();
	// 	});

	// 	self.__hideControl(self.autocompleteVariableDefinition);
	// };

	adm_process_flow_activity.prototype.addVariableToCodeInput = function () {
		const self = this;
		const codeEditor = self.codeEditor;
		const $selectedVariable = self.$get("[control=autocompleteVariable] option:selected");
		
		const actualValue = codeEditor.getValue();
		const variableValue = $selectedVariable.text();
		const valueWithVariable = `${actualValue} {{ ${variableValue} }}`;
		codeEditor.setValue(valueWithVariable);
		self.$get("[control=autocompleteVariable]").val(null);
	};

	adm_process_flow_activity.prototype.__makeContent = function () {
		var $content = adm_process_flow_activity.uber.__makeContent.call(this);
	  
		$content.unshift(
		  this.__makeDivFlex(
			[
			  this.__makeDivFlex(
				[
				  this.__makeDivFlex(
					[
					  this.__makeDivFlex(
						this.__makeComboFlex(
						  this.datasourceDefinition,
						  this.data[this.datasourceDefinition.id]
						),
						{ class: "col-12" }
					  ),
					],
					{ class: "row" }
				  ),
				  this.__makeDivFlex(
					[
					  this.__makeDivFlex(this.__makeCodeEditor(), {
						class: "col-12",
					  }),
					],
					{ class: "row" }
				  ),
				  this.__makeDivFlex(
					[
					  this.__makeDivFlex(
						this.__makeComboFlex(
						  this.autocompleteVariableDefinition,
						  this.data[this.autocompleteVariableDefinition.id]
						),
						{ class: "col-6 col-autocompletecombo" }
					  ),
					],
					{ class: "row" }
				  ),
				  this.__makeDivFlex(
					[
					  this.__makeDivFlex(
						this.__makeInputFlex(
						  this.resultDefinition,
						  this.data[this.resultDefinition.id]
						),
						{ class: "col-12" }
					  ),
					],
					{ class: "row" }
				  ),
				],
				{ class: "card-block" }
			  ),
			],
			{ class: "card no-card" }
		  )
		);
	  
		return $content;
	};
	  
		
	adm_process_flow_activity.prototype.__makeBody = function () {
		var $popup = adm_process_flow_activity.uber.__makeBody.call(this);
		// var $content = $(templates.code_block_content);
		// $popup.find("[content]")
		// 	.removeAttr("content")
		// 	.append($content);
		return $popup;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function () {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		this.__addValue(data, this.resultDefinition);
		this.__addValue(data, this.datasourceDefinition);
		data.code = this.codeEditor.getValue();
		if (!data.code) {
			data.error = "<%=_T('Must enter a code.')%>";
		}
		else {
			$.each(this.codeEditor.getSession().getAnnotations(), function (_, annotation) {
				if (annotation.type === "error") {
					data.error = "<%=_T('Invalid sql code.')%>";
					return false;
				}
			});
		}
		data.__brackets_code = this.__getBracketsValue(data.code);
		return data;
	};

	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_sql`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('SQL')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "option", // error/success/option
			option_label : "Input" // error/success/option
		});

		// en algún lado el usuario debe decidir si quiere un puerto por defecto
		let puerto_por_defecto = true;

		// agrego un puerto por defecto si es necesario
		if (puerto_por_defecto) {
			let existe = node?node.children.find(c => c.id.includes("_default")):undefined;

			ports.push({
				direction : "out", // in/out
				type : "option", // error/success/option
				option_label : "<%=_T('Default output')%>", // error/success/option
				channels: "n", // 0, 1 o n
				id: (existe?existe.id:inConcert.Utils.generateGuid() + "_default"),  // le asigno un id por defecto
				preset_value: undefined, // undefined para que permita al usuario crear sus propias condiciones 
				intern_value: undefined
			});
		}
		
		return ports;
	};

}());
