/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, ace */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;

	function adm_process_flow_class_editor_base() {
		adm_process_flow_class_editor_base.uber.constructor.call(this);
	}

	inConcert.Utils.inherit( adm_process_flow_class_editor_base, inConcert.adm_process_flow_popup );

	if (typeof inConcert.adm_process_flow_class_editor_base !== "function") {
		inConcert.adm_process_flow_class_editor_base = adm_process_flow_class_editor_base;
	}
	
	adm_process_flow_class_editor_base.prototype.__hasKeyboardEvents = function() {
		return false;
	};
	
	adm_process_flow_class_editor_base.prototype.IsReadonly = function() {
		return false;
	};
	
	adm_process_flow_class_editor_base.prototype.__updateTooltip = function() {
		
	};
	
	adm_process_flow_class_editor_base.prototype.__make = function() {
		var self = this;
		var $container = $(Mustache.render(templates.editor_container, {
			className : self.__getClassName()
		}));
		$container.append(self.__makeContent());
		return $container;
	};
	
	adm_process_flow_class_editor_base.prototype.__makeDescription = function() {
		var self = this;
		self.descriptionDefinition.text = self.data[self.descriptionDefinition.id] || "";
		var $description = $(Mustache.render(templates.editor_description, self.descriptionDefinition));
		$description.find("[description]")
			.prop("readonly", !!self.IsReadonly())
			.change(function() {
				self.__updateTooltip();
			});
		
		return $description;
	};
	
	adm_process_flow_class_editor_base.prototype.__getDescription = function() {
		return this.$get("[description]").val() || "";
	};
	
	adm_process_flow_class_editor_base.prototype.__makeExamples = function() {
		var self = this;
		var $examples = $(Mustache.render(templates.editor_examples, self.examplesDefinition));
		this.examplesCodeEditor = ace.edit($examples.find("[examples]")[0]);
		this.examplesCodeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		this.examplesCodeEditor.getSession().setValue(self.data[self.examplesDefinition.id] || "");
		this.examplesCodeEditor.setReadOnly(!!self.IsReadonly());
		
		return $examples;
	};
	
	adm_process_flow_class_editor_base.prototype.__getExamples = function() {
		return this.examplesCodeEditor?.getValue() || this.data.examples || "";
	};
	
	adm_process_flow_class_editor_base.prototype.__makeAction = function(action) {
		var self = this;
		action.withText = !action.withoutText;
		var $action = $(Mustache.render(templates.class_action, action));
		if (!action.skipEvent) {
			$action.click(function() {
				self.RunAction($(this).attr("class_action_button"));
			});
		}
		return $action;
	};
	
	adm_process_flow_class_editor_base.prototype.__makeActions = function() {
		var self = this;
		self.actions_d = {};
		var $actions = [];
		var readonly = this.IsReadonly();
		$.each(self.GetActions(), function(_, action) {
			if (readonly || action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}
			self.actions_d[action.id] = action;
			$actions.push(self.__makeAction(action));
		});
		return $actions;
	};
	
	adm_process_flow_class_editor_base.prototype.RunAction = function(actionId) {
		var self = this;
		if (!self.actions_d[actionId]) {
			return;
		}
		var action = self.actions_d[actionId];
		if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
			return;
		}
		if (action.confirmation) {
			$.inconcertPopup.Show(
				"<%=_T('Confirmation')%>",
				action.confirmation,
				$.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() === "yes") {
						self["__" + actionId]();
					}
				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "popUp380w confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		}
		else {
			self["__" + actionId]();
		}
	};
	
	adm_process_flow_class_editor_base.prototype.Show = function() {
		if (!this.$body) {
			this.$body = this.__make();
		}
		if (!this.$actions) {
			this.$actions = this.__makeActions();
		}

		const $container = this.$getElement("form_container");
		$container.toggleClass("readonlyClass", !!this.IsReadonly())
			.append(this.$body);

		if (this.focusControl) {
			this.__focusControl(this.focusControl);
		}
		else if (this.focusFirst) {
			this.$get("input, select, textarea")
				.first()
				.focus();
		}
		this.__afterAppendBody();
		this.__set_events();
	};

	adm_process_flow_class_editor_base.prototype.__set_events = function() {

		const self = this;

		const $inputs = this.$get("input, select, textarea");
		$inputs
		.off("keyup")
		.on("keyup", function() {
			self.toggle_global_buttons();
		});
	};

	adm_process_flow_class_editor_base.prototype.Close = function() {
		this.$body.detach();
	};
	
	adm_process_flow_class_editor_base.prototype.toggle_global_buttons = function() {
	};

	adm_process_flow_class_editor_base.prototype.__showError = function(error) {
		if (typeof error === "string") {
			error = { message : error };
		}
		adm_process_flow_class_editor_base.uber.__showError.call(this, error);
		inConcert.app.m_mainWindow.showBannerMessage({ message: error.message, messageType : "Error", timeout: 5000 });
	};

	adm_process_flow_class_editor_base.prototype.$container = function() {
		return this.$body || adm_process_flow_class_editor_base.uber.$container.call(this);
	};
	
	adm_process_flow_class_editor_base.prototype.GetActions = function() {
		return [];
	};
	
	adm_process_flow_class_editor_base.prototype.Rename = function(name) {
		
	};

	adm_process_flow_class_editor_base.prototype.reloadData = function(skipSelect) {
	};

}());
