/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	const actions = [
		{
			id : "edit",
			description : "<%=_T('Rename')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-pencil",
			skipEvent: true
		},
		{
			id : "clone",
			description : "<%=_T('Clone')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-copy",
			confirmation: "<%=_T('Do you want to clone attribute?')%>",
			skipEvent: true
		},
		{
			id : "delete",
			description : "<%=_T('Delete')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-trash-alt",
			confirmation: "<%=_T('Do you want to delete attribute?')%>",
			skipEvent: true
		},
		{
			id : "confirm",
			description : "<%=_T('Confirm')%>",
			action_icon_class : "far fa-check-circle",
			hidden: true,
			skipEvent: true
		},
	];

	function adm_process_flow_attribute_editor(classEditor, id, data) {
		adm_process_flow_attribute_editor.uber.constructor.call(this);
		this.classEditor = classEditor;
		this.id = id;
		this.data = data;
		
		this.descriptionDefinition = {
			id : "description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Attribute description.')%>"
		};
		this.examplesDefinition = {
			id : "examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Attribute extended description and usage examples.')%>"
		};
		
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_attribute_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_attribute_editor !== "function") {
		inConcert.adm_process_flow_attribute_editor = adm_process_flow_attribute_editor;
	}
	
	adm_process_flow_attribute_editor.prototype.__getClassName = function() {
		return "attributeEditorContainer";
	};
	
	adm_process_flow_attribute_editor.prototype.IsReadonly = function() {
		return this.classEditor.IsReadonly();
	};
	
	adm_process_flow_attribute_editor.prototype.__makeValue = function() {
		var self = this;
		var $container = $(Mustache.render(templates.editor_value_container, {
			name : "<%=_T('Value')%>",
			description : "<%=_T('Attribute initial value.')%>"
		}));
		$container.find("[attribute_value]")
			.val(self.data.value || "")
			.prop("readonly", self.IsReadonly())
			.change(function() {
				self.__updateTooltip();
			});
		return $container;
	};
	
	adm_process_flow_attribute_editor.prototype.__makeContent = function() {
		var $content = [];
		$content.push(this.__makeValue());
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_attribute_editor.prototype.__getValue = function(data, errors) {
		data.value = this.$get("[attribute_value]").val() || "";
	};
	
	adm_process_flow_attribute_editor.prototype.GetData = function(errors) {
		var data = {
			name : this.data.name,
			description : this.__getDescription(),
			examples : this.__getExamples()
		};
		this.__getValue(data, errors);
		if (data.error) {
			if (errors) {
				this.__showError(data.error);
				this.classEditor.main.toolbar.SelectAttribute(this.classEditor.data.Id, this.id, true);
				throw data.error;
			}
			else {
				data.error = undefined;
			}
		}
		return data;
	};
	
	adm_process_flow_attribute_editor.prototype.$GetActions = function() {
		return this.__makeActions();
	};

	adm_process_flow_attribute_editor.prototype.GetActions = function() {
		return actions;
	};
	
	adm_process_flow_attribute_editor.prototype.__clone = function() {
		var data = inConcert.Utils.cloneObject(this.GetData(false));
		var popup = new inConcert.adm_process_flow_duplicate_attribute_popup(this.classEditor.main, this.classEditor, data);
		popup.Show();
	};
	
	adm_process_flow_attribute_editor.prototype.__delete = function() {
		this.classEditor.DeleteAttribute({
			class_id : this.classEditor.data.Id,
			id : this.id
		});
	};

	adm_process_flow_attribute_editor.prototype.UpdateTitle = function(name) {
		this.classEditor.main.$element("classes_editor_name").html(name);
	};

	adm_process_flow_attribute_editor.prototype.Rename = function(name) {
		this.data.name = name;
		this.UpdateTitle(name);
		this.classEditor.main.toolbar.UpdateAttribute({
			class_id : this.classEditor.data.Id,
			id : this.id,
			name : this.data.name,
			method : true,
			editor : this
		});
		this.classEditor.main.toolbar.MarkSelectedClass(this.classEditor.data.Id);
	};
	
	adm_process_flow_attribute_editor.prototype.__updateTooltip = function() {
		this.classEditor.UpdateAttributeTooltip({
			class_id : this.classEditor.data.Id,
			id : this.id
		});
	};

	adm_process_flow_attribute_editor.prototype.__afterAppendBody = function() {
		this.UpdateTitle(this.data.name || "");
		this.isInitialized = true;
	};

	adm_process_flow_attribute_editor.prototype.__edit = function()
	{
		// Rename
		const $container = this.classEditor.main.$element(`toolbar_class_members=${this.classEditor.data.Id}`);
		this.readonly_name($container.find(`[toolbar_attribute="${this.id}"]`), false);
	};

	adm_process_flow_attribute_editor.prototype.readonly_name = function($item, readonly)
	{
		const $name_edit = $item.find("[toolbar_attribute_edit_name]");
		const $name_readonly = $item.find("[toolbar_attribute_readonly_name]");
		const $action_confirm = $item.find("[class_action_button='confirm']");
		const toggle_actions = function() {
			$.each($item.find("[class_action_button]"), function(_, element) {
				const $element = $(element);
				const action =  $element.attr("class_action_button");
				switch (action) {
					case "confirm":
						$element[readonly? "hide": "show"]();
						break;
					default:
						$element[readonly? "show": "hide"]();
						break;
				}
			});
		};
		toggle_actions();
		if (readonly) {
			this.classEditor.main.form.readonly_control($name_edit, true);
			$name_edit.hide();
			$name_readonly.show();
			$name_edit.val($name_readonly.html());
			$action_confirm.hide();
			return;
		}

		this.classEditor.main.form.readonly_control($name_edit, false);
		$name_edit.show();
		$name_edit.focus();
		$action_confirm.show();
		$name_readonly.hide();
	};

	adm_process_flow_attribute_editor.prototype.__confirm = function()
	{
		/*validate*/
		const $container = this.classEditor.main.$element(`toolbar_class_members=${this.classEditor.data.Id}`);
		const $attribute = $container.find(`[toolbar_attribute="${this.id}"]`);
		const $name = $attribute.find("input[type='text']");
		this.__clearErrors();
		const name_val = $name.val();

		let validation_result = this.classEditor.main.form.valid(name_val, "variable");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid attribute name.')%>");
		}

		validation_result = this.classEditor.main.form.valid(name_val, "required");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid attribute name.')%>");
		}
		
		const error = this.classEditor.ValidatePropertyName(name_val, this.id);
		if(error){
			return this.__showError(error);
		}

		this.Rename(name_val);
		this.readonly_name($attribute, true);
		this.toggle_global_buttons();
	};

	adm_process_flow_attribute_editor.prototype.HasUnsavedChanges = function() {
		return this.classEditor.HasUnsavedChanges();
	};

	adm_process_flow_attribute_editor.prototype.Cancel = function() {
		return this.classEditor.Cancel();
	};

	adm_process_flow_attribute_editor.prototype.toggle_global_buttons = function() {
		this.classEditor.toggle_global_buttons();
	};

	adm_process_flow_attribute_editor.prototype.$getElement = function(elem) {
		return this.classEditor.$getElement(elem);
	};

}());
