/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	const actions = [
		{
			id : "edit",
			description : "<%=_T('Rename')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-pencil",
		},
		{
			id : "clone",
			description : "<%=_T('Clone')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-copy",
			confirmation: "<%=_T('Do you want to clone class?')%>"
		},
		{
			id : "delete",
			description : "<%=_T('Delete')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-trash-alt",
			confirmation: "<%=_T('Do you want to delete class?')%>"
		},
		{
			id : "confirm",
			description : "<%=_T('Confirm')%>",
			action_icon_class : "far fa-check-circle",
			hidden: true,
		},
	];
	
	var reservedWords = {
		"new" : true
	};

	function adm_process_flow_class_editor(main, data) {
		adm_process_flow_class_editor.uber.constructor.call(this);
		this.main = main;
		this.data = data;
		this.data.Editor = this.data.Editor || {
			classConstructor : {
				parameters : [],
				code : "",
				description : "",
				examples : ""
			},
			methods : {},
			attributes : {}
		};
		if (!this.data.Editor.classConstructor && this.data.Editor.constructor) {
			this.data.Editor.classConstructor = this.data.Editor.constructor;
		}
		if ($.isArray(this.data.Editor.methods)) {
			this.data.Editor.methods = {};
		}
		if ($.isArray(this.data.Editor.attributes)) {
			this.data.Editor.attributes = {};
		}
		
		this.descriptionDefinition = {
			id : "Description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Class description.')%>"
		};
		this.examplesDefinition = {
			id : "Examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Class extended description and usage examples.')%>"
		};
	}

	inConcert.Utils.inherit( adm_process_flow_class_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_class_editor !== "function") {
		inConcert.adm_process_flow_class_editor = adm_process_flow_class_editor;
	}
	
	adm_process_flow_class_editor.prototype.__getClassName = function() {
		return "classEditorContainer";
	};
	
	adm_process_flow_class_editor.prototype.IsSystem = function() {
		return !!this.data.System;
	};
	
	adm_process_flow_class_editor.prototype.IsReadonly = function() {
		return this.main.readonly || this.IsSystem();
	};
	
	adm_process_flow_class_editor.prototype.__makeContent = function() {
		var $content = [];
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_class_editor.prototype.GetAttributes = function() {
		var self = this;
		var attributes = [];
		$.each(this.data.Editor.attributes, function(id, attribute) {
			attributes.push({
				class_id : self.data.Id,
				id : id,
				name : attribute.name,
				description : self.__makeAttributeTooltip(id)
			});
		});
		attributes.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});
		return attributes;
	};
	
	adm_process_flow_class_editor.prototype.GetMethods = function() {
		var self = this;
		var methods = [];
		$.each(this.data.Editor.methods, function(id, method) {
			methods.push({
				class_id : self.data.Id,
				id : id,
				name : method.name,
				description : self.__makeMethodTooltip(id),
				method : true
			});
		});
		methods.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});
		methods.unshift({
			class_id : self.data.Id,
			id : "constructor",
			name : "new",
			description : self.__makeMethodTooltip("constructor"),
			constructor : true
		});
		return methods;
	};
	
	adm_process_flow_class_editor.prototype.GetAttributeEditor = function(attributeId) {
		const attribute = this.data.Editor.attributes[attributeId];
		if (!attribute) {
			return null;
		}
		if (!attribute.editor || !attribute.editor.isInitialized) {
			attribute.editor = new inConcert.adm_process_flow_attribute_editor(this, attributeId, attribute);
		}
		return attribute.editor;
	};
	
	adm_process_flow_class_editor.prototype.GetMethodEditor = function(methodId) {
		const method = methodId === "constructor"? this.data.Editor.classConstructor : this.data.Editor.methods[methodId];
		if (!method) {
			return null;
		}
		if (!method.editor || !method.editor.isInitialized) {
			method.editor = new inConcert.adm_process_flow_method_editor(this, methodId, method);
		}
		return method.editor;
	};
	
	adm_process_flow_class_editor.prototype.__getEditorData = function(data, errors) {
		return data.editor?.isInitialized ? data.editor.GetData(errors) : data;
	};
	
	adm_process_flow_class_editor.prototype.GetData = function(errors) {
		var self = this;
		if (!self.$body) {
			var data = {};
			$.each(self.data, function(k, v) {
				if (v !== undefined && v !== null) {
					data[k] = v;
				}
			});
			return data;
		}
		try {
			//self data
			var data = {
				Id : self.data.Id,
				Name : self.data.Name,
				Description : self.__getDescription(),
				Examples : self.__getExamples(),
				Editor : {
					methods : {},
					attributes : {}
				}
			};
			if (data.error) {
				if (errors) {
					this.__showError(data.error);
					this.main.toolbar.SelectClass(this.data.Id, true);
					throw data.error;
				}
				else {
					data.error = undefined;
				}
			}
			//attributes
			$.each(self.data.Editor.attributes, function(attributeId, attributeData) {
				data.Editor.attributes[attributeId] = self.__getEditorData(attributeData, errors);
			});
			//constructor
			data.Editor.classConstructor = self.__getEditorData(self.data.Editor.classConstructor, errors);
			//methods
			$.each(self.data.Editor.methods, function(methodId, methodData) {
				data.Editor.methods[methodId] = self.__getEditorData(methodData, errors);
			});
			
			return data;
		}
		catch(e) {
			return null;
		}
	};
	
	adm_process_flow_class_editor.prototype.HasUnsavedChanges = function() {
		if (this.IsReadonly()) {
			return false;
		}

		if (this.IsNew()) {
			return true;
		}

		let currData = inConcert.Utils.plainOldObject(this.GetData(false), undefined, {
			editor: true
		}, true, true);
		let savedData = inConcert.Utils.plainOldObject(this.savedData, undefined, {
			editor: true
		}, true, true);
		return !_.isEqual(currData, savedData);
	};
	
	adm_process_flow_class_editor.prototype.IsNew = function() {
		return !this.savedData;
	};
	
	adm_process_flow_class_editor.prototype.SetSavedData = function(data) {
		this.savedData = inConcert.Utils.cloneObject(data);
	};
	
	adm_process_flow_class_editor.prototype.ClearErrors = function() {
		var self = this;
		if (!self.$body) {
			return;
		}
		self.__clearErrors();
		//attributes
		$.each(self.data.Editor.attributes, function(_, attributeData) {
			if (attributeData.editor) {
				attributeData.editor.__clearErrors();
			}
		});
		//constructor
		if (self.data.Editor.classConstructor.editor) {
			self.data.Editor.classConstructor.editor.__clearErrors();
		}
		//methods
		$.each(self.data.Editor.methods, function(_, methodData) {
			if (methodData.editor) {
				methodData.editor.__clearErrors();
			}
		});
	};
	
	adm_process_flow_class_editor.prototype.AddAttribute = function(id, data) {
		this.data.Editor.attributes[id] = data;
		this.main.toolbar.AddAttribute({
			class_id : this.data.Id,
			id : id,
			name : data.name,
			description : this.__makeAttributeTooltip(id)
		});
	};
	
	adm_process_flow_class_editor.prototype.AddMethod = function(id, data) {
		this.data.Editor.methods[id] = data;
		this.main.toolbar.AddMethod({
			class_id : this.data.Id,
			id : id,
			name : data.name,
			description : this.__makeMethodTooltip(id),
			method : true
		});
	};
	
	adm_process_flow_class_editor.prototype.ValidatePropertyName = function(name, id) {
		//check reserved words
		if (reservedWords[name]) {
			return "<%=_T('This name is a reserved word.')%>";
		}
		var found = false;
		//check attributes
		$.each(this.data.Editor.attributes, function(attributeId, attribute) {
			if (attribute.name === name && attributeId !== id) {
				found = true;
				return false;
			}
		});
		if (found) {
			return "<%=_T('Already exists an attribute with same name.')%>";
		}
		//check methods
		$.each(this.data.Editor.methods, function(methodId, method) {
			if (method.name === name && methodId !== id) {
				found = true;
				return false;
			}
		});
		if (found) {
			return "<%=_T('Already exists a method with same name.')%>";
		}
	};

	adm_process_flow_class_editor.prototype.$GetActions = function() {
		return this.__makeActions();
	};
	
	adm_process_flow_class_editor.prototype.GetActions = function() {
		return actions;
	};
	
	adm_process_flow_class_editor.prototype.__clone = function() {
		var data = inConcert.Utils.cloneObject(this.GetData(false));
		var popup = new inConcert.adm_process_flow_duplicate_class_popup(this.main, data);
		popup.Show();
	};
	
	adm_process_flow_class_editor.prototype.__delete = function() {
		this.main.DeleteClass(this.data.Id);
		this.main.toolbar.DeleteClassesMembers(this.data.Id);
	};
	
	adm_process_flow_class_editor.prototype.__rename = function() {
		var popup = new inConcert.adm_process_flow_rename_class_popup(this.main, this);
		popup.Show();
	};
	
	adm_process_flow_class_editor.prototype.Rename = function(name) {
		this.data.Name = name;
		this.main.UpdateTitle(name);
		this.main.toolbar.UpdateClass(this);
	};
	
	adm_process_flow_class_editor.prototype.DeleteAttribute = function(attribute) {
		var self = this;
		const attributeId = attribute.id;
		var oldAttributes = self.data.Editor.attributes;
		self.data.Editor.attributes = {};
		let firstData;
		$.each(oldAttributes, function(id, data) {
			if (!firstData && id !== attributeId) {
				firstData = data;
			}
			if (id !== attributeId) {
				self.data.Editor.attributes[id] = data;
			}
		});
		this.main.toolbar.RemoveAttribute(attribute);

		if (firstData) {
			this.main.toolbar.SelectAttribute(attribute.class_id, firstData.editor.id, true);
		}
		else{
			this.main.$element("toolbar_class_members_general").click();
		}
		this.toggle_global_buttons();
	};
	
	adm_process_flow_class_editor.prototype.DeleteMethod = function(method) {
		var self = this;
		var oldMethods = self.data.Editor.methods;
		const methodId = method.id
		self.data.Editor.methods = {};
		let firstData;
		$.each(oldMethods, function(id, data) {
			if (!firstData && id !== methodId) {
				firstData = data;
			}
			if (id !== methodId) {
				self.data.Editor.methods[id] = data;
			}
		});
		this.main.toolbar.RemoveMethod(method);
		if (firstData) {
			this.main.toolbar.SelectMethod(method.class_id, firstData.editor.id, true);
		}
		else{
			this.main.$element("toolbar_class_members_general").click();
		}
		this.toggle_global_buttons();
	};
	
	adm_process_flow_class_editor.prototype.MakeTooltip = function() {
		return this.$container()? this.__getDescription() : this.data.Description;
	};
	
	adm_process_flow_class_editor.prototype.__updateTooltip = function() {
		this.main.toolbar.UpdateClassTooltip(this.data.Id, this.MakeTooltip());
	};
	
	adm_process_flow_class_editor.prototype.__makeParametersTooltip = function(data) {
		var snippet = "";
		$.each(data.parameters || [], function(_, parameter) {
			snippet += (snippet? ", " : "") + parameter.name;
		});
		return snippet;
	};
	
	adm_process_flow_class_editor.prototype.__makeMethodTooltip = function(methodId) {
		var method = methodId === "constructor"? this.data.Editor.classConstructor : this.data.Editor.methods[methodId];
		var data = method.editor?.isInitialized? method.editor.GetData() : method;
		
		var snippet;
		if (methodId === "constructor") {
			snippet = format("local myObject = classHelpers.{0}.new({1})", this.data.Name, 
				this.__makeParametersTooltip(data));
		}
		else {
			snippet = format("myObject:{0}({1})", data.name, this.__makeParametersTooltip(data));
		}
		var tooltip = format("{0}\n{1}", snippet, data.description);
		return tooltip;
	};
	
	adm_process_flow_class_editor.prototype.UpdateMethodTooltip = function(method) {
		const methodId = method.id;
		this.main.toolbar.UpdateMethodTooltip(method, this.__makeMethodTooltip(methodId));
	};
	
	adm_process_flow_class_editor.prototype.__makeAttributeTooltip = function(attributeId) {
		var attribute = this.data.Editor.attributes[attributeId];
		var data = attribute.editor?.isInitialized? attribute.editor.GetData() : attribute;
		var snippet = format("myObject.{0}", data.name);
		var defaultValue = format("<%=_T('Default value: {0}')%>", data.value || "nil");
		var tooltip = format("{0}\n{1}\n{2}", snippet, defaultValue, data.description);
		return tooltip;
	};
	
	adm_process_flow_class_editor.prototype.UpdateAttributeTooltip = function(attribute) {
		const attributeId = attribute.id;
		this.main.toolbar.UpdateAttributeTooltip(attribute, this.__makeAttributeTooltip(attributeId));
	};

	adm_process_flow_class_editor.prototype.__edit = function()
	{
		// Rename
		this.readonly_name(this.main.$element(`toolbar_class=${this.data.Id}`), false);
	};

	adm_process_flow_class_editor.prototype.readonly_name = function($class, readonly)
	{
		const $name_edit = $class.find("[toolbar_class_edit_name]");
		const $name_readonly = $class.find("[toolbar_class_readonly_name]");
		const $action_confirm = $class.find("[class_action_button='confirm']");
		const toggle_actions = function() {
			$.each($class.find("[class_action_button]"), function(_, element) {
				const $element = $(element);
				const action =  $element.attr("class_action_button");
				switch (action) {
					case "confirm":
						$element[readonly? "hide": "show"]();
						break;
					default:
						$element[readonly? "show": "hide"]();
						break;
				}
			});
		};
		toggle_actions();
		if (readonly) {
			this.main.form.readonly_control($name_edit, true);
			$name_edit.hide();
			$name_readonly.show();
			$name_edit.val($name_readonly.html());
			$action_confirm.hide();
			return;
		}

		this.main.form.readonly_control($name_edit, false);
		$name_edit.show();
		$name_edit.focus();
		$action_confirm.show();
		$name_readonly.hide();
	};

	adm_process_flow_class_editor.prototype.__confirm = function()
	{
		/*validate*/
		const $class = this.main.$element(`toolbar_class=${this.data.Id}`);
		const $name = $class.find("input[type='text']");
		this.ClearErrors();
		const name_val = $name.val();

		let validation_result = this.main.form.valid(name_val, "variable");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid class name.')%>");
		}

		validation_result = this.main.form.valid(name_val, "required");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid class name.')%>");
		}
		
		const classId = this.main.GetClassFromName(name_val);
		if (classId && classId !== this.data.Id) {
			return this.__showError("<%=_T('Already exists a class with same name.')%>");
		}

		this.Rename(name_val);
		this.readonly_name($class, true);
		this.toggle_global_buttons();
	};

	adm_process_flow_class_editor.prototype.__afterAppendBody = function() {
		this.main.UpdateTitle(this.data.Name || this.data.name || "");
		this.main.$element("classes_editor_name").html("<%=_T('General')%>");
		this.main.$element("classes_editor_system_name")[this.IsSystem()? "show":"hide"]();
		const classId = this.data.Id;
		const $class_members_container = (this.main.toolbar.GetClassesMembers(classId)).$body;
		$class_members_container.find(".selected").removeClass("selected");
		const $general = $class_members_container.find("[toolbar_class_members_general]");
		$general.addClass("selected");
		$general.parent().addClass("selected");
		$class_members_container.show();
		this.main.$element("tool_bar_list_class_members").html($class_members_container);
		this.main.toolbar.set_events_members();
		if (this.IsNew()) {
			this.toggle_global_buttons();
		}
	};

	adm_process_flow_class_editor.prototype.reloadData = function(skipSelect) {
		
		
		if (this.savedData?.Name) {
			this.Rename(this.savedData.Name);
		}
		
		this.$body = this.__make();
		this.main.$element("class_editor_container").remove();
		if (this.savedData) {
			this.data = inConcert.Utils.cloneObject(this.savedData);
		}

		//Elimino la data de members
		this.main.toolbar.DeleteClassesMembers(this.data.Id);
		this.main.toolbar.Make_Class_Members(this.data.Id);
		if (!skipSelect) {
			this.main.toolbar.SelectClass(this.data.Id);
		}
	};

	adm_process_flow_class_editor.prototype.Cancel = function() {
		this.reloadData();
		this.toggle_global_buttons();
	}; 

	adm_process_flow_class_editor.prototype.toggle_global_buttons = function() {
		this.main.toggle_global_buttons();
	};

	adm_process_flow_class_editor.prototype.$getElement = function(elem) {
		return this.main.$element(elem);
	};

}());
