/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, ace */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	const actions = [
		{
			id : "edit",
			description : "<%=_T('Rename')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-pencil",
			skipEvent: true
		},
		{
			id : "clone",
			description : "<%=_T('Clone')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-copy",
			confirmation: "<%=_T('Do you want to clone method?')%>",
			skipEvent: true
		},
		{
			id : "delete",
			description : "<%=_T('Delete')%>",
			permission: "save_flow_classes",
			action_icon_class : "far fa-trash-alt",
			confirmation: "<%=_T('Do you want to delete method?')%>",
			skipEvent: true
		},
		{
			id : "confirm",
			description : "<%=_T('Confirm')%>",
			action_icon_class : "far fa-check-circle",
			hidden: true,
			skipEvent: true
		},
	];
	
	var reservedWords = {
		"this" : true,
		"self" : true
	};

	function adm_process_flow_method_editor(classEditor, id, data) {
		adm_process_flow_method_editor.uber.constructor.call(this);
		this.classEditor = classEditor;
		this.id = id;
		this.data = data;
		this.constructor = id === "constructor";
		
		this.descriptionDefinition = {
			id : "description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Method description.')%>"
		};
		this.examplesDefinition = {
			id : "examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Method extended description and usage examples.')%>"
		};
	}

	inConcert.Utils.inherit( adm_process_flow_method_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_method_editor !== "function") {
		inConcert.adm_process_flow_method_editor = adm_process_flow_method_editor;
	}
	
	adm_process_flow_method_editor.prototype.__getClassName = function() {
		return "methodEditorContainer";
	};
	
	adm_process_flow_method_editor.prototype.IsReadonly = function() {
		return this.classEditor.IsReadonly();
	};
	
	adm_process_flow_method_editor.prototype.__makeParameter = function(parameter, readonly) {
		var self = this;
		parameter = parameter || { name : "" };
		var $parameter = $(Mustache.render(templates.editor_param, {
			name : parameter.name,
			id : inConcert.Utils.generateGuid(),
			readonly : !!readonly
		}));
		if (readonly) {
			$parameter.find("button, input").remove();
		}
		else {
			$parameter.find("[param_edit]").click(function() {
				self.__editParameter($(this).attr("param_edit"));
			});
			$parameter.find("[param_delete]").click(function() {
				self.__deleteParameter($(this).attr("param_delete"));
				self.classEditor.main.toggle_global_buttons();
			});
			$parameter.find("[param_confirm]").click(function() {
				self.__confirmParameter($(this).attr("param_confirm"));
				self.classEditor.main.toggle_global_buttons();
			});
			$parameter.find("[param_cancel]").click(function() {
				self.__cancelParameter($(this).attr("param_cancel"));
			});
			$parameter.dblclick(function() {
				if ($(this).hasClass("modeView")) {
					self.__editParameter($(this).attr("param"));
				}
			});
		}
		
		return $parameter;
	};
	
	adm_process_flow_method_editor.prototype.__addParameter = function() {
		var $parameter = this.__makeParameter();
		this.$get("[parameters_list]").append($parameter);
		this.__editParameter($parameter.attr("param"));
	};
	
	adm_process_flow_method_editor.prototype.__getParameter = function(id) {
		return this.$get(format("[param='{0}']", id));
	};
	
	adm_process_flow_method_editor.prototype.__editParameter = function(id) {
		var $parameter = this.__getParameter(id);
		$parameter
			.removeClass("modeView")
			.addClass("modeEdit")
			.find("[param_value]")
			.val($parameter.attr("param_name"))
			.focus();
	};
	
	adm_process_flow_method_editor.prototype.__cancelParameter = function(id) {
		var $parameter = this.__getParameter(id);
		if (!$parameter.attr("param_name")) {
			$parameter.remove();
			return;
		}
		$parameter
			.removeClass("modeEdit")
			.addClass("modeView");
	};
	
	adm_process_flow_method_editor.prototype.__validateParameter = function($parameter, edit, errors) {
		var data = {
			name : edit? $parameter.find("[param_value]").val() : $parameter.attr("param_name")
		};
		if (!errors) {
			return data;
		}
		if (!data.name) {
			data.error = "<%=_T('Parameter name is required.')%>";
		}
		if (!inConcert.Utils.isValid(data.name, "variable")) {
			data.error = "<%=_T('Invalid parameter name.')%>";
		}
		else if (reservedWords[data.name]) {
			data.error = "<%=_T('This name is a reserved word.')%>";
		}
		else {
			var $nameParameter = this.$get(format("[param_name='{0}']", data.name));
			if ($nameParameter.length && $nameParameter.attr("param") !== $parameter.attr("param")) {
				data.error = "<%=_T('Already exists a parameter with same name.')%>";
			}
		}
		return data;
	};
	
	adm_process_flow_method_editor.prototype.__confirmParameter = function(id) {
		var $parameter = this.__getParameter(id);
		var data = this.__validateParameter($parameter, true, true);
		if (data.error) {
			inConcert.app.m_mainWindow.showBannerMessage({ message: data.error, messageType : "Error", timeout: 5000 });
			return;
		}
		$parameter.replaceWith(this.__makeParameter(data));
		this.__updateTooltip();
	};
	
	adm_process_flow_method_editor.prototype.__deleteParameter = function(id) {
		this.__getParameter(id).remove();
		this.__updateTooltip();
	};
	
	adm_process_flow_method_editor.prototype.__makeParameters = function(readonly) {
		var self = this;
		var $container = $(Mustache.render(templates.editor_params_container, {
			name : "<%=_T('Parameters')%>",
			description : "<%=_T('Method parameters.')%>"
		}));
		var $list = $container.find("[parameters_list]");
		if (!readonly) {
			$list.sortable({
				cursor : "move"
			});
		}
		$.each(self.data.parameters || [], function(_, parameter) {
			$list.append(self.__makeParameter(parameter, readonly));
		});
		if (readonly) {
			$container.find("[parameters_add]").remove();
		}
		else {
			$container.find("[parameters_add]").click(function() {
				self.__addParameter();
			});
		}
		return $container;
	};
	
	adm_process_flow_method_editor.prototype.__makeCode = function(readonly) {
		var $container = $(Mustache.render(templates.editor_code_container, {
			name : "<%=_T('Code')%>",
			description : "<%=_T('Method code.')%>"
		}));
		this.codeEditor = ace.edit($container.find("[code_block]")[0]);
		this.codeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		this.codeEditor.getSession().setValue(this.data.code || "");
		this.codeEditor.setReadOnly(readonly);
		
		return $container;
	};
	
	adm_process_flow_method_editor.prototype.__afterAppendBody = function() {
		this.codeEditor.focus();
		this.UpdateTitle(this.data.name || "");
		this.isInitialized = true;
	};
	
	adm_process_flow_method_editor.prototype.__makeContent = function() {
		var readonly = !!this.IsReadonly();
		var $content = [];
		$content.push(this.__makeParameters(readonly));
		$content.push(this.__makeCode(readonly));
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_method_editor.prototype.__getParameters = function(data, errors) {
		var self = this;
		data.parameters = [];
		self.$get("[param]").each(function() {
			var paramData = self.__validateParameter($(this), false, errors);
			if (paramData.error) {
				data.error = paramData.error;
				return false;
			}
			data.parameters.push(paramData);
		});
	};
	
	adm_process_flow_method_editor.prototype.__getCode = function(data, errors) {
		data.code = this.codeEditor?.getValue() || this.data.code;
		if (!errors || !this.codeEditor) {
			return;
		}
		$.each(this.codeEditor.getSession().getAnnotations(), function(_, annotation) {
			if (annotation.type === "error") {
				data.error = "<%=_T('Invalid lua code.')%>";
				return false;
			}
		});
	};
	
	adm_process_flow_method_editor.prototype.GetData = function(errors) {
		var data = {
			description : this.__getDescription(),
			examples : this.__getExamples()
		};
		if (!this.constructor) {
			data.name = this.data.name;
		}
		this.__getParameters(data, errors);
		this.__getCode(data, errors);
		if (data.error) {
			if (errors) {
				this.__showError(data.error);
				this.classEditor.main.toolbar.SelectMethod(this.classEditor.data.Id, this.id, true);
				throw data.error;
			}
			else {
				data.error = undefined;
			}
		}
		return data;
	};
	
	adm_process_flow_method_editor.prototype.GetActions = function() {
		return this.constructor? [] : actions;
	};
	
	adm_process_flow_method_editor.prototype.__clone = function() {
		var data = inConcert.Utils.cloneObject(this.GetData(false));
		var popup = new inConcert.adm_process_flow_duplicate_method_popup(this.classEditor.main, this.classEditor, data);
		popup.Show();
	};
	
	adm_process_flow_method_editor.prototype.__delete = function() {
		this.classEditor.DeleteMethod({
			class_id : this.classEditor.data.Id,
			id : this.id
		});
	};
	
	adm_process_flow_method_editor.prototype.Rename = function(name) {
		this.data.name = name;
		this.UpdateTitle(name);
		this.classEditor.main.toolbar.UpdateMethod({
			class_id : this.classEditor.data.Id,
			id : this.id,
			name : this.data.name,
			method : true
		});
	};
	
	adm_process_flow_method_editor.prototype.__updateTooltip = function() {
		this.classEditor.UpdateMethodTooltip({
			class_id : this.classEditor.data.Id,
			id : this.id
		});
	};

	adm_process_flow_method_editor.prototype.UpdateTitle = function(name) {

		if (!name && this.constructor) {
			name = "<%=_T('Constructor')%>";
		}
		
		this.classEditor.main.$element("classes_editor_name").html(name || "");
	};

	adm_process_flow_method_editor.prototype.__edit = function()
	{
		// Rename
		const $container = this.classEditor.main.$element(`toolbar_class_members=${this.classEditor.data.Id}`);
		this.readonly_name($container.find(`[toolbar_method="${this.id}"]`), false);
	};

	adm_process_flow_method_editor.prototype.$GetActions = function() {
		return this.__makeActions();
	};

	adm_process_flow_method_editor.prototype.readonly_name = function($item, readonly)
	{
		const $name_edit = $item.find("[toolbar_method_edit_name]");
		const $name_readonly = $item.find("[toolbar_method_readonly_name]");
		const $action_confirm = $item.find("[class_action_button='confirm']");
		const toggle_actions = function() {
			$.each($item.find("[class_action_button]"), function(_, element) {
				const $element = $(element);
				const action =  $element.attr("class_action_button");
				switch (action) {
					case "confirm":
						$element[readonly? "hide": "show"]();
						break;
					default:
						$element[readonly? "show": "hide"]();
						break;
				}
			});
		};
		toggle_actions();
		if (readonly) {
			this.classEditor.main.form.readonly_control($name_edit, true);
			$name_edit.hide();
			$name_readonly.show();
			$name_edit.val($name_readonly.html());
			$action_confirm.hide();
			return;
		}

		this.classEditor.main.form.readonly_control($name_edit, false);
		$name_edit.show();
		$name_edit.focus();
		$action_confirm.show();
		$name_readonly.hide();
	};

	adm_process_flow_method_editor.prototype.__confirm = function()
	{
		/*validate*/
		const $container = this.classEditor.main.$element(`toolbar_class_members=${this.classEditor.data.Id}`);
		const $method = $container.find(`[toolbar_method="${this.id}"]`);
		const $name = $method.find("input[type='text']");
		this.__clearErrors();
		const name_val = $name.val();

		let validation_result = this.classEditor.main.form.valid(name_val, "variable");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid method name.')%>");
		}

		validation_result = this.classEditor.main.form.valid(name_val, "required");
		if (!validation_result || !validation_result.success) {
			return this.__showError("<%=_T('Invalid method name.')%>");
		}
		
		const error = this.classEditor.ValidatePropertyName(name_val, this.id);
		if(error){
			return this.__showError(error);
		}

		this.Rename(name_val);
		this.readonly_name($method, true);
		this.toggle_global_buttons();
	};

	adm_process_flow_method_editor.prototype.HasUnsavedChanges = function() {
		return this.classEditor.HasUnsavedChanges();
	};

	adm_process_flow_method_editor.prototype.Cancel = function() {
		return this.classEditor.Cancel();
	};

	adm_process_flow_method_editor.prototype.toggle_global_buttons = function() {
		this.classEditor.toggle_global_buttons();
	};

	adm_process_flow_method_editor.prototype.$getElement = function(elem) {
		return this.classEditor.$getElement(elem);
	};

}());
