/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flow_classes;
	var format = inConcert.Utils.format;

	function adm_process_flow_classes_toolbar(main) {
		this.main = main;
		this.__make();
	}

	if (typeof inConcert.adm_process_flow_classes_toolbar !== "function") {
		inConcert.adm_process_flow_classes_toolbar = adm_process_flow_classes_toolbar;
	}
	
	adm_process_flow_classes_toolbar.prototype.__make = function() {
		this.classes_members = {};
	};
	
	adm_process_flow_classes_toolbar.prototype.__addSorted = function($classes, $item, name) {
		var itemName = $item.attr(name);
		var $before = $item;
		$classes.each(function() {
			if ($(this).attr(name).localeCompare(itemName) > 0) {
				return false;
			}
			$before = $(this);
		});
		if ($before === $item) {
			$classes.first().before($item);
		}
		else {
			$before.after($item);
		}
	};
	
	adm_process_flow_classes_toolbar.prototype.MarkSelectedClass = function(classId, force)
	{
		if (!force && this.main?.currentEditor?.data?.Id !== classId) {
			return;
		}
		const $class = this.main.$element("toolbar_class=" + classId);
		$class
			.removeClass("selected")
			.addClass("selected")
			.removeClass("childSelected");
		
		$class.siblings("[toolbar_class]")
			.removeClass("childSelected")
			.removeClass("selected")

		$class.parent()
			.scrollTo($class);
	};

	adm_process_flow_classes_toolbar.prototype.SelectClass = function(classId) {
		
		this.MarkSelectedClass(classId, true);
		this.main.ShowClassEditor(classId);
	};
	
	adm_process_flow_classes_toolbar.prototype.SelectAttribute = function(classId, attributeId, selected) {
		const $toolbar_class_members =  this.main.$element(`toolbar_class_members="${classId}"`);
		const $attribute = $toolbar_class_members.find(`[toolbar_attribute="${attributeId}"]`);
		const $attributes_list = $toolbar_class_members.find("[attributes_list]");
		if (!$attributes_list.is(":visible")) {
			$attributes_list.show();
		}
		$toolbar_class_members.find(".selected").removeClass("selected");
		$attribute
			.toggleClass("selected", selected)
			.parent()
			.scrollTo($attribute);

			$toolbar_class_members.find("[toolbar_class_members_head='attributes']").parent().addClass("selected");

		this.main.ShowAttributeEditor(classId, attributeId);
	};
	
	adm_process_flow_classes_toolbar.prototype.SelectMethod = function(classId, methodId, selected) {
		const $toolbar_class_members =  this.main.$element(`toolbar_class_members="${classId}"`);
		const $method = $toolbar_class_members.find(`[toolbar_method="${methodId}"]`);
		const $methods_list = $toolbar_class_members.find("[methods_list]");
		
		if (!$methods_list.is(":visible")) {
			$methods_list.show();
		}
		$toolbar_class_members.find(".selected").removeClass("selected");
		$method
			.toggleClass("selected", selected)
			.parent()
			.scrollTo($method);
		
		$toolbar_class_members.find("[toolbar_class_members_head='methods']").parent().addClass("selected");

		this.main.ShowMethodEditor(classId, methodId);
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeAttribute = function(attribute) {
		const $attribute = $(Mustache.render(templates.toolbar_attribute, attribute));
		return $attribute;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeAttributes = function(classEditor) {
		var self = this;
		var $attributes = [];
		const attributes = classEditor.GetAttributes();
		$.each(attributes, function(_, attribute) {
			const $attribute = self.__makeAttribute(attribute);
			self.AddAttributeActions($attribute, attribute);
			$attributes.push($attribute);
		});
		return $attributes;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeMethod = function(method) {
		const $method = $(Mustache.render(templates.toolbar_method, method));
		return $method;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeMethods = function(classEditor) {
		const self = this;
		const $methods = [];
		const methods = classEditor.GetMethods();
		$.each(methods, function(_, method) {
			const $method = self.__makeMethod(method);
			self.AddMethodActions($method, method);
			$methods.push($method);
		});
		return $methods;
	};
	
	adm_process_flow_classes_toolbar.prototype.__insertClass = function($item, sorted) {
		const $container = this.main.$element("tool_bar_list_class");
		const $classes = $container.children("[toolbar_class]");
		if (sorted || !$classes.length) {
			$container.append($item);
		}
		else {
			this.__addSorted($classes, $item, "class_name");
		}
	};
	
	adm_process_flow_classes_toolbar.prototype.AddClass = function(classEditor, sorted) {
		const self = this;
		const readonly = classEditor.IsReadonly();
		
		const $item = $(Mustache.render(templates.toolbar_class, {
			id : classEditor.data.Id,
			name : classEditor.data.Name,
			description : classEditor.MakeTooltip(),
			readonly : !!readonly,
			system : classEditor.IsSystem()
		}));
		
		$item.click(function(e) {
			const classId = $(this).attr("toolbar_class");
			let currEditor = self.main?.currentEditor?.classEditor || self.main?.currentEditor;
			
			const $target = $(e.target);
			if ($target.closest("input").length || currEditor?.data?.Id === classId) {
				return;
			}
			
			const fnCallback = function() {
				self.SelectClass(classId);
				self.main.toggle_global_buttons();
			};

			//Si para el que voy a saltar tiene cambios
			if (self.main?.form && currEditor?.HasUnsavedChanges && currEditor.HasUnsavedChanges()) {
				self.main?.form?.confirm("<%=_T('Any unsaved changes will be lost!')%>", function (success) {
					if (success) {

						if (currEditor.IsNew()) {
							//Elimino el item
							self.main.removeClass(currEditor.data.Id);
						}else {
							//Deshago los cambios.
							currEditor.reloadData(true);
						}
						fnCallback();
					}
				}, "<%=_T('Are you sure you want to leave?')%>");
				return;
			}

			fnCallback();
		});
		
		this.__insertClass($item, sorted);
		this.__AddClassActions($item, classEditor);
		this.Make_Class_Members(classEditor.data.Id);
		this.MarkSelectedClass(classEditor.data.Id);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveClass = function(classId) {
		this.main.$element(format("toolbar_class={0}", classId)).remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateClass = function(classEditor) {
		this.RemoveClass(classEditor.data.Id);
		this.AddClass(classEditor);
	};
	
	adm_process_flow_classes_toolbar.prototype.AddAttribute = function(attribute) {
		const $attribute = this.__makeAttribute(attribute);
		const $container_class_members =  this.main.$element(`toolbar_class_members="${attribute.class_id}"`);
		$container_class_members.removeClass("withoutAttributes");
		const $list = $container_class_members.find("[attributes_list]");
		const $attributes = $list.children("[toolbar_attribute]");
		if (!$attributes.length) {
			$list.prepend($attribute);
		}
		else {
			this.__addSorted($attributes, $attribute, "attribute_name");
		}
		this.SelectAttribute(attribute.class_id, attribute.id, true);
		this.main.toggle_global_buttons();
		this.AddAttributeActions($attribute, attribute);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveAttribute = function(attribute) {
		const attributeId = attribute.id;
		const $container = this.main.$element(`toolbar_class_members='${attribute.class_id}'`);
		$container.toggleClass("withoutAttributes", $container.find("[toolbar_attribute]").length <= 1);
		const $list = $container.find("[attributes_list]");
		$list
			.find(format("[toolbar_attribute={0}]", attributeId))
			.remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateAttribute = function(attribute) {
		this.RemoveAttribute(attribute);
		this.AddAttribute(attribute);
	};
	
	adm_process_flow_classes_toolbar.prototype.AddMethod = function(method) {
		const $method = this.__makeMethod(method);
		
		const $list = this.main.$element(`toolbar_class_members="${method.class_id}"`).find("[methods_list]");
		const $methods = $list.children("[method_name]");
		if (!$methods.length) {
			$list
				.find("[toolbar_method=constructor]")
				.after($method);
		}
		else {
			this.__addSorted($methods, $method, "method_name");
		}
		this.SelectMethod(method.class_id, method.id, true);
		this.main.toggle_global_buttons();
		this.AddMethodActions($method, method);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveMethod = function(method) {
		this.main.$element(`toolbar_class_members='${method.class_id}'`).find(`[toolbar_method="${method.id}"]`).remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateMethod = function(method) {
		this.RemoveMethod(method);
		this.AddMethod(method);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateClassTooltip = function(classId, tooltip) {
		this.main.$element(format("toolbar_class={0}", classId))
			.find("[class_selector_title]")
			.attr("title", tooltip);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateMethodTooltip = function(method, tooltip) {
		const methodId = method.id;
		this.main.$element(`toolbar_class_members='${method.class_id}'`).find(`[toolbar_method="${methodId}"]`)
			.attr("title", tooltip);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateAttributeTooltip = function(attribute, tooltip) {
		const attributeId = attribute.id;
		this.main.$element(`toolbar_class_members='${attribute.class_id}'`).find(`[toolbar_method="${attributeId}"]`)
			.attr("title", tooltip);
	};

	adm_process_flow_classes_toolbar.prototype.__AddClassActions = function($item, editor) {
		const $class_actions_container = $item.find("[editor_actions]");
		$class_actions_container.html("");
		const $actions = editor.$GetActions();
		$item
		.off("clickoutside")
		.on("clickoutside", function (e) {
			const $this = $(this);
			if ($this.find("[toolbar_class_readonly_name]").is(":visible")) {
				return;
			}
			editor.readonly_name($this, true);
		});	

		$class_actions_container.append($actions);
	};

	adm_process_flow_classes_toolbar.prototype.GetClassesMembers = function(class_id) {
		return this.classes_members[class_id];
	};

	adm_process_flow_classes_toolbar.prototype.DeleteClassesMembers = function(class_id) {
		if(!this.classes_members || !this.classes_members[class_id]){
			return;
		}
		delete this.classes_members[class_id];
	};

	adm_process_flow_classes_toolbar.prototype.set_events_members = function() {
		const self = this;

		const $members_general = this.main.$element("toolbar_class_members_general");
		$members_general
		.off("click")
		.on("click", function() {
			self.SelectClass($(this).attr("class_id"));
		});

		this.main.$element("toolbar_method")
		.off("clickoutside")
		.on("clickoutside", function (e) {
			const $this = $(this);
			if ($this.find("[toolbar_method_readonly_name]").is(":visible")) {
				return;
			}
			const methodid = $this.attr("toolbar_method");
			const classId = $this.attr("class_id");
			const classEditor = self.main.GetClassEditor(classId);
			const editor = classEditor.GetMethodEditor(methodid);
			if (editor) {
				editor.readonly_name($this, true);
			}
			
		});	

		this.main.$element("toolbar_attribute")
		.off("clickoutside")
		.on("clickoutside", function (e) {
			const $this = $(this);
			if ($this.find("[toolbar_attribute_readonly_name]").is(":visible")) {
				return;
			}
			const attrid = $this.attr("toolbar_attribute");
			const classId = $this.attr("class_id");
			const classEditor = self.main.GetClassEditor(classId);
			const editor = classEditor.GetAttributeEditor(attrid);
			if (editor) {
				editor.readonly_name($this, true);
			}
		});	

		this.main.$element("toolbar_class_members")
		.off("click")
		.on("click", "[toolbar_method], [toolbar_attribute]", function(e) {
			const $target = $(e.target);
			if ($target.is("input")) {
				return;
			}
			const $this = $(this);
			const executeAction = function(){
				let action_button = "";
				if ($target.is("button")) {
					action_button = $target.attr("class_action_button");
				}
				else if ($target.closest("button").length) {
					action_button = $target.parent().attr("class_action_button");
				}
				if (action_button === "") {
					return;
				}
				self.main.currentEditor.RunAction(action_button);
			};

			if ($this.attr("toolbar_method")) {
				 self.SelectMethod($(this).attr("class_id"), $(this).attr("toolbar_method"));
				 executeAction();
				 return;
			}
			
			if($this.attr("toolbar_attribute")){
				self.SelectAttribute($(this).attr("class_id"), $(this).attr("toolbar_attribute"));
				executeAction();
			}

		});

		this.main.$element("add_attribute")
		.off("click")
		.on("click", function() {
			self.main.NewAttribute($(this).attr("add_attribute"));
		});

		this.main.$element("add_method")
		.off("click")
		.on("click", function() {
			self.main.NewMethod($(this).attr("add_method"));
		});

		this.main.$element("toolbar_class_members_head")
		.off("click")
		.on("click", function() {
			const $this = $(this);
			const member_type = $(this).attr("toolbar_class_members_head");
			self.toggle_class_members_list(member_type, $this);			
		});

	};

	adm_process_flow_classes_toolbar.prototype.toggle_class_members_list = function(member_type, $member_head) {
		const $list = this.main.$element(`${member_type}_list`);
		const is_visible = $list.is(":visible");
		$member_head.parent().removeClass("showingSubMenu");
		if (is_visible) {
			$list.hide();
			return;
		}
		$list.show();
		$member_head.parent().addClass("showingSubMenu");
	};

	adm_process_flow_classes_toolbar.prototype.Make_Class_Members = function(class_id) {
		if (this.classes_members[class_id]?.$body) {
			return;
		}
		const $render_toolbar_class_members = $(Mustache.render(templates.toolbar_class_members, {
			id : class_id
		}));
		
		const classEditor = this.main.GetClassEditor(class_id);
		const readonly = classEditor.IsReadonly();

		$render_toolbar_class_members.find("[attributes_list]").prepend(this.__makeAttributes(classEditor));
		$render_toolbar_class_members.find("[methods_list]").prepend(this.__makeMethods(classEditor));

		if (readonly) {
			$render_toolbar_class_members.find("[add_attribute]").remove();
			$render_toolbar_class_members.find("[add_method]").remove();
		}

		this.classes_members[class_id] = {
			$body : $render_toolbar_class_members
		};
	};

	adm_process_flow_classes_toolbar.prototype.AddAttributeActions = function($item, attribute) {
		const $actions_container = $item.find("[attribute_editor_actions]");
		$actions_container.html("");
		const classId = attribute.class_id;
		const classEditor = attribute.classEditor || this.main.GetClassEditor(classId);
		if (classEditor.IsReadonly()) {
			return;
		}

		const editor = classEditor.GetAttributeEditor(attribute.id);		
		$actions_container.append(editor.$GetActions());
		return $item;
	};

	adm_process_flow_classes_toolbar.prototype.AddMethodActions = function($item, method) {
		const $actions_container = $item.find("[method_editor_actions]");
		$actions_container.html("");
		const classId = method.class_id;
		const classEditor = method.classEditor || this.main.GetClassEditor(classId);
		if (classEditor.IsReadonly()) {
			return;
		}

		const editor = classEditor.GetMethodEditor(method.id);
		$actions_container.append(editor.$GetActions());
		return $item;
	};

}());
