/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_datasources;
	
	var actions = [
		{
			id : "editDS",
			name : "<%=_T('Edit')%>",
			description : "<%=_T('Edit')%>",
			permission : "save_flow_datasources",
		},
	];
	
	function adm_process_flow_datasource_editor(main, data, isSaved) {
		adm_process_flow_datasource_editor.uber.constructor.call(this, main);
		this.data = data;
		if (isSaved) {
			this.SetSavedData(data);
		}
		
		var self = this;
		
		this.descriptionDefinition = {
			id : "Description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Datasource description.')%>"
		};
		
		this.typeDefinition = {
			id : "Type",
			name : "<%=_T('Type')%>",
			description : "<%=_T('Datasource type.')%>",
			required : true,
			not_extended : true,
			options : inConcert.flowDesignerDatasourcesTypesSorted,
			change : function() {
				self.UpdateType();
				self.main.__toggle_global_buttons();
			},
			form_group: true,
		};

		this.serverDefinition = {
			id : "Server",
			name : "<%=_T('Server')%>",
			description : "<%=_T('Server IP or domain.')%>",
			required : true,
			not_extended : true,
			form_group: true,
		};

		this.portDefinition = {
			id : "Port",
			name : "<%=_T('Port')%>",
			description : "<%=_T('Server port.')%>",
			required : true,
			not_extended : true,
			validate : {
				range : {
					type : "integer",
					allowVariables : true,
					min : 0
				}
			},
			form_group: true,
		};

		this.usernameDefinition = {
			id : "Username",
			name : "<%=_T('User')%>",
			description : "<%=_T('User name.')%>",
			required : true,
			not_extended : true,
			form_group: true,
		};

		this.passwordDefinition = {
			id : "Password",
			name : "<%=_T('Password')%>",
			description : "<%=_T('User password.')%>",
			emptyValue : " ",
			required : this.IsNew(),
			type: this.IsNew()? "text" : "password",
			icon_eye: this.IsNew()? "far fa-eye" : "far fa-eye-slash",
			not_extended : true,
			form_group: true,
		};

		this.databaseDefinition = {
			id : "Database",
			name : "<%=_T('Database')%>",
			description : "<%=_T('Database')%>",
			required : true,
			not_extended : true,
			form_group: true,
			change: function() {
				self.main.__toggle_global_buttons();
			},
			options: inConcert.flowDesignerBuiltinDatabases || []
		};

		this.databaseDefinitionText = {
			id : "DatabaseText",
			name : "<%=_T('Database')%>",
			description : "<%=_T('Database')%>",
			required : true,
			not_extended : true,
			form_group: true,
		};
		
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_datasource_editor, inConcert.adm_process_flow_datasource_editor_base );

	if (typeof inConcert.adm_process_flow_datasource_editor !== "function") {
		inConcert.adm_process_flow_datasource_editor = adm_process_flow_datasource_editor;
	}
	
	adm_process_flow_datasource_editor.prototype.__getClassName = function() {
		return "datasourceEditorContainer";
	};

	adm_process_flow_datasource_editor.prototype.GetDatabaseDefinition = function() {
		return this.databaseDefinition;
	};

	adm_process_flow_datasource_editor.prototype.SetDatabaseDefinition = function(text) {
		this.databaseDefinition = text;
	};
	
	adm_process_flow_datasource_editor.prototype.GetTitle = function() {
		return format("<%=_T('{0}')%>", this.data.Name);
	};
	
	adm_process_flow_datasource_editor.prototype.IsReadonly = function() {
		return (this.main && this.main.readonly) || this.data.System;
	};

	adm_process_flow_datasource_editor.prototype.__makePassword = function() {
		const self = this;
		self.passwordDefinition.text = self.data[self.passwordDefinition.id] || "";
		const $editor_password = $(Mustache.render(templates.editor_password, self.passwordDefinition));
		$editor_password.find("[dsTogglePassword='true']").
		off("click").
		on("click", function() {
			const $password = $editor_password.find("[control='Password']");
			const type = $password.attr("type") === "password" ? "text" : "password";
			$(this).removeClass().addClass(type === "password"? "far fa-eye-slash" : "far fa-eye");
			$password.attr("type", type);
			$(this).attr("visible", type === "text"? "true":"false");
		});

		return $editor_password;
	};
	
	adm_process_flow_datasource_editor.prototype.MakePropertiesContent = function($content) {
		  $content.push(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
					this.__makeCombo(this.typeDefinition, this.data[this.typeDefinition.id]),
					{ class: "col-3" }
				  ),
				this.__makeDivFlex(
				  this.__makeInputFlex(
					this.serverDefinition,
					this.data[this.serverDefinition.id]
				  ),
				  { class: "col-3 pl-0" }
				),
				this.__makeDivFlex(
					this.__makeInputFlex(
					  this.portDefinition,
					  this.data[this.portDefinition.id]
					),
					{ class: "col-3 pl-0" }
				),
				this.__makeDivFlex(
					this.__makeInputFlex(
					  this.usernameDefinition,
					  this.data[this.usernameDefinition.id]
					),
					{ class: "col-3 pl-0" }
				),
				this.__makePassword(),
				this.__makeDivFlex(
					  this.__makeInputFlex(
						this.databaseDefinitionText,
						this.data[this.databaseDefinitionText.id]
					  ),
					  { class: "col-3 pl-0" }
				),
				this.__makeDivFlex(
					this.__makeCombo(
						this.databaseDefinition,
						this.data[this.databaseDefinition.id]
					 ),
					  { class: "col-3 pl-0"}
				),
				this.__makeDescription() || ""
			  ],
			  { class: "row" }
			)
		  );
	};
	
	adm_process_flow_datasource_editor.prototype.__makeContent = function() {
		var $content = [];
		this.MakePropertiesContent($content);
		return $content;
	};

	adm_process_flow_datasource_editor.prototype.__afterAppendBody = function() {
		this.SelectType();
		const is_readonly = !this.IsNew() && (this.IsReadonly() || !this.is_mode_edit);
		this.__readonly_controls(is_readonly);
		this.main.$element("action_button='editDS'")[this.is_mode_edit || this.IsNew()? "hide":"show"]();

		$.each(this.main.$element("dstogglepassword"), function(_, element) {
			const $element = $(element);
			if ($element.attr("visible")) {
				$element.click();
			}
		});
		
	};

	adm_process_flow_datasource_editor.prototype.__readonly_controls = function(is_readonly) {
		const $container = this.main.$element("form_container");
		const self = this;
		const $controls = $container.find("[control]");
		$container.removeClass("readonlyDatasource");

		if (is_readonly) {
			$container.addClass("readonlyDatasource");
		}

		$.each( $controls, function(_, element) {
			const $element = $(element);
			$element.attr("placeholder", is_readonly && $element.val() === ""? "<%=_T('There are no records')%>": "");
			self.main.form.readonly_control($element, is_readonly, true);
		});
	};
	
	adm_process_flow_datasource_editor.prototype.SelectType = function() {
		var type = inConcert.flowDesignerDatasourcesTypes[this.data.Type];
		if (!type) {
			return;
		}
		switch(type.type) {
			case "sql":
				this.__selectSqlType(type);
				break;
		}
		this.currentType = this.data.Type;
	};

	adm_process_flow_datasource_editor.prototype.__afterSelectSqlType = function(type) {
		//No show controls for id builtin
		if(type.id === "builtin")
		{
			this.__hideControl(this.serverDefinition);
			this.__setControlRequired(this.serverDefinition);
			this.__setValue(this.serverDefinition, "");

			this.__hideControl(this.portDefinition);
			this.__setControlRequired(this.portDefinition);
			this.__setValue(this.serverDefinition, "");
			
			this.__hideControl(this.usernameDefinition);
			this.__setControlRequired(this.usernameDefinition);
			this.__setValue(this.usernameDefinition, "");

			this.__hideControl(this.passwordDefinition);
			this.__setControlRequired(this.passwordDefinition);
			this.__setValue(this.passwordDefinition, "");

			// this.databaseDefinition.options = inConcert.flowDesignerBuiltinDatabases || [];
			// this.__replaceControl(this.databaseDefinition,
			// 	this.__makeCombo(this.databaseDefinition, this.data[this.databaseDefinition.id]));
			this.__showControl(this.databaseDefinition);
			this.__hideControl(this.databaseDefinitionText);

		}
		else{
			// this.__replaceControl(this.databaseDefinition,
			// 	this.__makeInput(this.databaseDefinition, this.data[this.databaseDefinition.id]));
			this.__showControl(this.databaseDefinitionText);
			this.__hideControl(this.databaseDefinition);


			this.__setControlRequired(this.serverDefinition, true);
			this.__setControlRequired(this.portDefinition, true);
			this.__setControlRequired(this.usernameDefinition, true);
			this.__setControlRequired(this.passwordDefinition, this.IsNew() || !!this.is_mode_edit);
		}
	};
	
	adm_process_flow_datasource_editor.prototype.__selectSqlType = function (type) {
		//default port
		if (type.defaultPort && (this.currentType || !this.data[this.portDefinition.id])) {
			this.__setValue(this.portDefinition, type.defaultPort);
		}
		//controls
		this.__showControl(this.serverDefinition);
		this.__showControl(this.portDefinition);
		this.__showControl(this.usernameDefinition);
		this.__showControl(this.passwordDefinition);
		// this.__showControl(this.databaseDefinition);

		this.__afterSelectSqlType(type);
	};
	
	adm_process_flow_datasource_editor.prototype.__addTypeSqlValues = function(type, data) {
		if (type.id === "builtin") {
			this.__addValue(data, this.databaseDefinition);
		}			
		else {
			this.__addValue(data, this.databaseDefinitionText);
		}
		this.__addValue(data, this.passwordDefinition);
		this.__addValue(data, this.usernameDefinition);
		this.__addValue(data, this.portDefinition);
		this.__addValue(data, this.serverDefinition);
	};
	
	adm_process_flow_datasource_editor.prototype.GetProperties = function(data) {
		this.__addValue(data, this.typeDefinition);
		var type = inConcert.flowDesignerDatasourcesTypes[data.Type];
		if (type) {
			switch(type.type) {
				case "sql":
					this.__addTypeSqlValues(type, data);
			}
		}
	};
	
	adm_process_flow_datasource_editor.prototype.GetData = function(errors) {
		var self = this;
		if (!self.$body) {
			var data = {};
			$.each(self.data, function(k, v) {
				if (v !== undefined && v !== null) {
					data[k] = v;
				}
			});
			return data;
		}
		try {
			let data = {
				Id : self.data.Id,
				Name : self.data.Name,
				Description : self.__getDescription()
			};
			this.GetProperties(data);
			
			if (data.error) {
				if (errors) {
					this.__showError(data.error);
					this.main.toolbar.SelectDatasource(this.data.Id, true);
					throw data.error;
				}
				else {
					data.error = undefined;
				}
			}
			if (!data[this.passwordDefinition.id]) {
				delete data[this.passwordDefinition.id];
			}
			
			return data;
		}
		catch(e) {
			return null;
		}
	};
	
	adm_process_flow_datasource_editor.prototype.HasUnsavedChanges = function() {
		
		let savedData;
		let currData;
		if (this.savedData) {
			savedData = inConcert.Utils.cloneObject(this.savedData);
			currData = this.GetData(false);
			if (savedData.Type === "builtin" && currData.Type === "builtin"){
				//No comparo ni Username ni password
				savedData.Password = "";
				currData.Password = "";
				savedData.Username = "";
				currData.Username = "";
			}
		}
		return !savedData || !_.isEqual(currData, savedData);
	};
	
	adm_process_flow_datasource_editor.prototype.IsNew = function() {
		return !this.savedData;
	};
	
	adm_process_flow_datasource_editor.prototype.SetSavedData = function(data) {
		this.savedData = JSON.parse(JSON.stringify(data));
	};
	
	adm_process_flow_datasource_editor.prototype.ClearErrors = function() {
		var self = this;
		if (!self.$body) {
			return;
		}
		self.__clearErrors();
	};
	
	adm_process_flow_datasource_editor.prototype.GetActions = function() {
		return actions;
	};
	
	adm_process_flow_datasource_editor.prototype.__duplicate = function() {
		var data = JSON.parse(JSON.stringify(this.GetData(false)));
		var popup = new inConcert.adm_process_flow_duplicate_datasource_popup(this.main, data);
		popup.Show();
	};
	
	adm_process_flow_datasource_editor.prototype.__delete = function() {
		this.main.DeleteDatasource(this.data.Id);
	};
	
	adm_process_flow_datasource_editor.prototype.Rename = function(name) {
		this.data.Name = name;
		this.main.UpdateTitle(this);
		this.main.toolbar.UpdateDatasource(this);
	};
	
	adm_process_flow_datasource_editor.prototype.MakeTooltip = function() {
		return this.$container()? this.__getDescription() : this.data.Description;
	};
	
	adm_process_flow_datasource_editor.prototype.__updateTooltip = function() {
		if (this.main) {
			this.main.toolbar.UpdateDatasourceTooltip(this.data.Id, this.MakeTooltip());
		}
	};
	
	adm_process_flow_datasource_editor.prototype.UpdateType = function() {
		this.data.Type = this.__getValue(this.typeDefinition);
		if (this.main) {
			this.main.UpdateTitle(this);
			this.main.toolbar.UpdateDatasourceType(this.data.Id, this.data.Type);
		}
		this.SelectType();
	};

	adm_process_flow_datasource_editor.prototype.__editDS = function() {
		this.is_mode_edit = true;
		this.main.toolbar.SelectDatasource(this.data.Id, true);
	};
	
}());
