/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	
	function adm_process_flow_import_datasources_popup(main) {
		adm_process_flow_import_datasources_popup.uber.constructor.call(this);
		this.main = main;
		this.showEditTitleButton = false;
		
		var self = this;
		
		this.fileDefinition = {
			id : "File",
			name : "<%=_T('File')%>",
			description : "<%=_T('File to be imported.')%>",
			required : true,
			validate : {
				extension : {
					types : {
						json : true
					},
					message : "<%=_T('Unsupported extension file. Only json.')%>"
				}
			}
		};
		
		this.datasourcesDefinition = {
			id : "Datasources",
			name : "<%=_T('Datasources')%>",
			description : "<%=_T('Datasources to be exported.')%>",
			sort : true,
			selectAll : true,
			required : true
		};
	}

	inConcert.Utils.inherit( adm_process_flow_import_datasources_popup, inConcert.adm_process_flow_popup );

	if (typeof inConcert.adm_process_flow_import_datasources_popup !== "function") {
		inConcert.adm_process_flow_import_datasources_popup = adm_process_flow_import_datasources_popup;
	}

	adm_process_flow_import_datasources_popup.prototype.__getClassName = function() {
		// return "flowDesignerImportDatasourcesPopup hybridDesignContainer";
		return "popUp380w flowDesignerGlobalModal flowDesignerOverlayModal";
	};

	adm_process_flow_import_datasources_popup.prototype.__getTitle = function() {
		return "<%=_T('Import Datasources')%>";
	};

	adm_process_flow_import_datasources_popup.prototype.__makeContent = function() {
		var self = this;
		var $content = [];
		$content.push(
			/*
			self.__makeRowFlex(self.__makeUpload(self.fileDefinition, function(file, onReady, onProgress, onEnd) {
			self.__uploadFile(file, onReady, onProgress, onEnd);
			}))
			*/
			self.__makeDivFlex(
				[
				  self.__makeDivFlex(
					[
					  self.__makeRowFlex([
						self.__makeDivFlex(
						  [
							self.__makeRowFlex(
								self.__makeUpload(self.fileDefinition, function(file, onReady, onProgress, onEnd) {
									self.__uploadFile(file, onReady, onProgress, onEnd);
								})
							)
						  ],
						  { class: "form-group" }
						),
					  ]),
					],
					{ class: "card-block" }
				  ),
				],
				{ class: "card no-card datasourcesPopupImport" }
			)
		);
		return $content;
	};
	
	adm_process_flow_import_datasources_popup.prototype.__makeActions = function() {
		var $actions = adm_process_flow_import_datasources_popup.uber.__makeActions.call(this);
		$actions.filter("[action=save]").prop("disabled", true);
		return $actions;
	};
	
	adm_process_flow_import_datasources_popup.prototype.__uploadFile = function(file, onReady, onProgress, onEnd) {
		var self = this;
		if (self.$datasources) {
			self.$datasources.remove();
			self.$datasources = null;
		}
		self.$get("[action=save]").prop("disabled", true);
		onReady();
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.importFlowDatasources(file, function(status, data) {
			var message = "";
			if (!status) {
				switch(data && data.result) {
					case "SYSTEM_DATASOURCE":
						message = "<%=_T('A system datasource has same name.')%>";
						break;
					default :
						message = "<%=_T('Error loading file.')%>";
						break;
				}
			}
			onEnd(status, message);
			if (status) {
				self.__loadDatasources(data.datasources || {});
			}
		}, onProgress);
	};
	
	adm_process_flow_import_datasources_popup.prototype.__loadDatasources = function(datasources) {
		var self = this;
		if ($.isEmptyObject(datasources)) {
			self.__showError("<%=_T('There are no datasources to be imported.')%>");
			return;
		}
		self.datasources = datasources;
		self.datasourcesDefinition.options = [];
		$.each(datasources, function(_, flowDs) {
			self.datasourcesDefinition.options.push({
				id : flowDs.data.Id,
				name : flowDs.data.Name
			});
		});
		self.$datasources = self.__makeCheckboxList(self.datasourcesDefinition);
		self.__addContent(self.$datasources);
		self.$get("[action=save]").prop("disabled", false);
	};

	adm_process_flow_import_datasources_popup.prototype.__getAndValidate = function() {
		var data = {};
		this.__addValue(data, this.datasourcesDefinition);
		return data;
	};

	adm_process_flow_import_datasources_popup.prototype.__save = function(data, callback) {
		var self = this;
		var datasources = [];
		var overrideDatasources = "";
		$.each(data[self.datasourcesDefinition.id], function(dsId) {
			var flowDs = self.datasources[dsId];
			//existing id with different name
			if (flowDs.existingId && flowDs.existingId !== flowDs.data.Name) {
				flowDs.data.Id = inConcert.Utils.generateGuid();
			}
			//existing name (will be overrided)
			if (flowDs.existingName) {
				flowDs.data.Id = flowDs.existingName;
				overrideDatasources = overrideDatasources + (overrideDatasources? ", " : "") + flowDs.data.Name;
			}
			//add ds
			datasources.push(flowDs.data);
		});
		
		function doImport() {
			datasources.sort(function(a, b) {
				return a.Name.localeCompare(b.Name);
			});
			self.main.ImportDatasources(datasources, function(status) {
				callback(status, status? "<%=_T('Error importing datasources.')%>" : "");
			});
		}
		
		if (!overrideDatasources) {
			return doImport();
		}
		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			format("<%=_T('These datasources already exists and will be overrided: {0}.')%>", overrideDatasources),
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				switch(response.toLowerCase()) {
					case "yes":
						return doImport();
				}
				callback(false);
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

}());
