/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_datasources;
	
	var types =  [
		{
			id			: "sqlserver",
			type		: "sql",
			name		: "<%=_T('SQL Server')%>",
			//description	: "<%=_T('A SQL Server database.')%>",
			defaultPort : 1433
		},
		{
			id			: "mysql",
			type		: "sql",
			name		: "<%=_T('MySQL')%>",
			//description	: "<%=_T('A MySQL database.')%>",
			defaultPort : 3306
		},
		{
			id			: "builtin",
			type		: "sql",
			name		: "<%=_T('Built In')%>",
		}
	];
	
	if (!inConcert.flowDesignerDatasourcesTypesSorted) {
		inConcert.flowDesignerDatasourcesTypesSorted = types.map(function(type) {
			type.disclaimer = type.description;
			return type;
		});
	}
	
	if (!inConcert.flowDesignerDatasourcesTypes) {
		inConcert.flowDesignerDatasourcesTypes = types.reduce(function(types_d, type) {
			types_d[type.id] = type;
			return types_d;
		}, {});
	}
	
	if (!inConcert.flowDesignerDatasourcesTypesLabels) {
		inConcert.flowDesignerDatasourcesTypesLabels = types.reduce(function(types_d, type) {
			types_d[type.id] = type.name;
			return types_d;
		}, {});
	}

	function adm_process_flow_datasources_editor(flowDesigner, form) {
		if(flowDesigner) {
			this.flowDesigner = flowDesigner;
			this.currentEditor = flowDesigner.currentEditor;
		}

		this.form = form || this.flowDesigner?.frm;
	}

	if (typeof inConcert.adm_process_flow_datasources_editor !== "function") {
		inConcert.adm_process_flow_datasources_editor = adm_process_flow_datasources_editor;
	}
	
	var globalActions = [
		{
			id : "save",
			name : "<%=_T('Save')%>",
			description : "<%=_T('Save Datasources')%>",
			permission : "save_flow_datasources",
			disabled: true
		},
		{
			id : "export",
			name : "<%=_T('Export')%>",
			description : "<%=_T('Export Datasources')%>",
			permission : "export_flow_datasources",
		},
		{
			id : "import",
			name : "<%=_T('Import')%>",
			description : "<%=_T('Import Datasources')%>",
			permission : "save_flow_datasources",
		},
		{
			id : "cancel",
			name : "<%=_T('Undo')%>",
			description : "<%=_T('Undo changes')%>",
			hidden: true
		}
	];

	adm_process_flow_datasources_editor.prototype.$element = function(name, $parent) {
		$parent = $parent || this.$container;
		return name? $parent.find(format("[{0}]", name)) : $parent;
	};
	
	adm_process_flow_datasources_editor.prototype.__loadDatasources = function(callback) {
		callback = callback || $.noop;
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getDatasources(function(success, datasources) {
			if (!success) {
				var message = "<%=_T('Error loading datasources.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				return callback(false);
			}
			var datasources_a = [];
			$.each(datasources, function(_, dsData) {
				datasources_a.push(dsData);
			});
			datasources_a.sort(function(a, b) {
				return a.Name.localeCompare(b.Name);
			});
			callback(datasources_a);
		});
	};

	adm_process_flow_datasources_editor.prototype.__loadBuiltinDatabases = function(callback) {
		callback = callback || $.noop;
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getBuiltinDatabases(function(success, databases) {
			if (!success) {
				var message = "<%=_T('Error loading databases.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				return callback(false);
			}
			inConcert.flowDesignerBuiltinDatabases = databases;
			callback(true);
		});
	};
	
	adm_process_flow_datasources_editor.prototype.AddDatasource = function(dsData, sorted, select, isSaved) {
		var datasourceEditor =  new inConcert.adm_process_flow_datasource_editor(this, dsData, isSaved);
		if (select) {
			datasourceEditor.is_mode_edit = datasourceEditor.IsNew()? true: false;
		}
		this.datasources[dsData.Id] = datasourceEditor;
		this.toolbar.AddDatasource(datasourceEditor, sorted);
		if (select) {
			this.toolbar.SelectDatasource(dsData.Id, true);
		}
		this.__new_datasource_tooltip();
		return datasourceEditor;
	};
	
	adm_process_flow_datasources_editor.prototype.__make = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.readonly = !inConcert.permissions.to_action("flow_designer", "save_flow_datasources");
		self.$container = $(templates.base);
		self.$element("close_button").click(function() {
			self.Close();
		});
		self.$element("minimize_button").click(function() {
			self.Minimize();
		});

		self.__loadBuiltinDatabases(function(success){
			self.__loadDatasources(function(datasources) {
				if (!datasources) {
					return callback();
				}
				self.__makeGlobalActions();
				self.__makeToolbar();
				self.datasources = {};
				$.each(datasources, function(_, dsData) {
					dsData.Executable = undefined;

					/*
					Se agrega este fix pq en 14669 agregaron una nuevo campo 'DatabaseText' para la base datos que no son builtin
						pero no se tuvo encuenta, los datasources que ya podian existir.
						Esto generaba que cuando se creaban los markup de basedatos, quedaran nulos.
						Se acomoda de esta forma pq ya anda en produccion a la fecha. 
					*/

					if (dsData.Type !== "builtin" &&  dsData.hasOwnProperty('Database')) {
						dsData.DatabaseText = dsData.Database;
						delete dsData.Database;
					}
					self.AddDatasource(dsData, true, false, true);
				});

				//select first
				self.$element("datasource_name").first().click();
				callback();
			});
		});
	};
	
	adm_process_flow_datasources_editor.prototype.__makeGlobalAction = function(action) {
		var self = this;
		var $action = $(Mustache.render(templates.action_button, action));
		$action.click(function() {
			self["__" + $(this).attr("action_button")]();
		});
		$action.prop("disabled", action.disabled);
		$action[action.hidden?"hide":"show"]();
		return $action;
	};
	
	adm_process_flow_datasources_editor.prototype.__makeGlobalActions = function() {
		var self = this;
		//global
		var $actions = [];
		$.each(globalActions, function(_, action) {
			if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}
			action.withText = !action.withoutText;
			$actions.push(self.__makeGlobalAction(action));
		});
		self.$element("global_actions").append($actions);

		//new datasources.
		const $new_datasource = self.$element("action_button='new_datasource'");
		const has_permission = inConcert.permissions.to_action("flow_designer", "save_flow_datasources");
		if (has_permission) {
			$new_datasource.
			off("click").
			on("click", function() {
				self["__new_datasource"]();
			});
		}
		else{
			$new_datasource.remove();
		}
	};
	
	adm_process_flow_datasources_editor.prototype.__makeToolbar = function() {
		this.toolbar = new inConcert.adm_process_flow_datasources_toolbar(this);
	};

	adm_process_flow_datasources_editor.prototype.__new_datasource_tooltip = function () {
		const $editor_container = this.$element("editor_container");
		const $new_datasource = $editor_container.find("[action_button='new_datasource']");
		if ($new_datasource) {
			const has_ds = $editor_container.find("[toolbar_datasource]").length > 0;
			$editor_container.inconcert_tooltip({
				bind_selector:"[action_button='new_datasource']",
				message: has_ds ? "<%=_T('New Datasource')%>" : "<%=_T('Create your first data source from this button.')%>",
			});
		}
	};
	
	adm_process_flow_datasources_editor.prototype.Show = function(callback, onClose) {
		callback = callback || $.noop;
		var self = this;
		self.onClose = onClose || $.noop;
		
		function show() {
			$("#flow_designer_dashboard").append(self.$container);
			self.__new_datasource_tooltip();
			callback();
		}
		
		if (self.$container) {
			return show();
		}
		self.__make(function() {
			show();
		});
	};
	
	adm_process_flow_datasources_editor.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		
		function close() {
			if (self.$container) {
				self.$container.remove();
				self.$container = null;
				// self.currentEditor.main.__keepDesignerScreenMode();
			}
			self.onClose();
			callback();
		}
		
		if (!self.__hasUnsavedChanges()) {
			return close();
		}

		this.form?.confirm("<%=_T('Any unsaved changes will be lost!')%>", function (success) {
			if (success) {
				close();
			}
		}, "<%=_T('Are you sure you want to leave?')%>");
	};

	adm_process_flow_datasources_editor.prototype.__process = function(process) {
		this.processing = !!process;
		this.$element("processing").toggle(this.processing);
	};

	adm_process_flow_datasources_editor.prototype.__isProcessing = function() {
		return !!this.processing;
	};

	adm_process_flow_datasources_editor.prototype.__toggle_global_buttons = function() {
		if (this.readonly) {
			return;
		}

		this.$element("action_button='save'").prop("disabled", !this.__hasUnsavedChanges());
		this.$element("action_button='cancel'")[this.currentEditor?.HasUnsavedChanges()?"show":"hide"]();
	};

	adm_process_flow_datasources_editor.prototype.__save = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.__isProcessing()) {
			return callback(false);
		}
		//clear errors
		inConcert.app.m_mainWindow.hideBannerMessage();
		$.each(self.datasources, function(_, datasourceEditor) {
			datasourceEditor.ClearErrors();
		});
		//get data
		var data = {};
		var newDatasources = [];
		var error = false;
		$.each(self.datasources, function(dsId, datasourceEditor) {
			if (datasourceEditor.IsReadonly()) {
				return;
			}
			var dsData = datasourceEditor.GetData(true);
			if (!dsData) {
				error = true;
				return false;
			}
			data[dsId] = dsData;
			if (datasourceEditor.IsNew()) {
				newDatasources.push(datasourceEditor.data.Id);
			}
		});
		if (error) {
			return callback(false);
		}
		//save
		self.__process(true);
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.saveDatasources(data, function(success, error) {
			self.__process(false);
			if (success) {
				$.each(self.datasources, function(dsId, datasourceEditor) {
					if (data[dsId]) {
						datasourceEditor.SetSavedData(data[dsId]);
					}
				});
				if (self.flowDesigner) {
					self.flowDesigner.AddDatasources(newDatasources);
					self.flowDesigner.SetDatasources(data);
				}
				self.__toggle_global_buttons();
				if (self.currentEditor && self.toolbar) {
					self.currentEditor.is_mode_edit = false;
					self.toolbar.SelectDatasource(self.currentEditor.data.Id, true);
				}
				var message = "<%=_T('Datasources saved successfully.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Ok", timeout: 1500 });
			}
			else {
				var message;
				switch(error && error.result) {
					case "FORBIDDEN":
						message = "<%=_T('Unable to save process flow datasources. You dont have permission.')%>";
						break;
					default:
						message = "<%=_T('Error saving datasources.')%>";
						break;
				}
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(success);
		});
	};
	
	adm_process_flow_datasources_editor.prototype.__removeDatasource = function(dsId) {
		var self = this;
		self.toolbar.RemoveDatasource(dsId);
		var oldDatasources = self.datasources;
		self.datasources = {};
		$.each(oldDatasources, function(id, editor) {
			if (id !== dsId) {
				self.datasources[id] = editor;
			}
		});
		if (self.flowDesigner) {
			self.flowDesigner.RemoveDatasource(dsId);
		}
		self.__showEditor();
	};
	
	adm_process_flow_datasources_editor.prototype.DeleteDatasource = function(dsId) {
		var self = this;
		if (self.__isProcessing()) {
			return;
		}

		const dsIdCurrentEditor = this.currentEditor?.data?.Id;
		const callback_success = function () {
			self.__process(false);
			self.__removeDatasource(dsId);
			var message = "<%=_T('Datasource deleted successfully.')%>";
			inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType: "Ok", timeout: 1500 });
			self.__new_datasource_tooltip();
			self.__toggle_global_buttons();

			//Si se elimina el ds seleccionado, selecciono el primero disponible.
			if (dsId === dsIdCurrentEditor) {
				const $datasources = self.$element("toolbar_datasource");
				const $first = $datasources?.first();
				if ($first) {
					const id = $first.attr("toolbar_datasource");
					self.toolbar.SelectDatasource(id, true);
				}
			}
			else{
				self.toolbar.SelectDatasource(dsIdCurrentEditor, true);
			}
		};

		if (this.currentEditor?.IsNew()) {
			return callback_success();
		}

		self.__process(true);
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.deleteDatasources([ dsId ], function(success, error) {
			if (success) {
				callback_success();
			}
			else {
				self.__process(false);
				var message;
				switch(error && error.result) {
					case "FORBIDDEN":
						message = "<%=_T('Unable to delete process flow datasource. You dont have permission.')%>";
						break;
					case "DATASOURCE_IN_USE":
						var flows_s = "";
						$.each(error.flows || [], function(_, flowData) {
							flows_s += (flows_s? ", " : "") + flowData.name;
						});
						message = format("<%=_T('Datasource is currently in use by these flows: {0}.')%>", flows_s);
						break;
					default:
						message = "<%=_T('Error deleting datasource.')%>";
						break;
				}
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				return;
			}
		});
	};
	
	adm_process_flow_datasources_editor.prototype.__hasUnsavedChanges = function() {
		var has = false;
		$.each(this.datasources, function(_, datasourceEditor) {
			has = has || datasourceEditor.HasUnsavedChanges();
			if (has) {
				return false;
			}
		});
		return has;
	};
	
	adm_process_flow_datasources_editor.prototype.__export = function() {
		var self = this;

		function doExport() {
			var popup = new inConcert.adm_process_flow_export_datasources_popup(self);
			popup.Show();
		}
		
		if (self.__isProcessing()) {
			return;
		}

		//save validation
		if (!self.__hasUnsavedChanges()) {
			return doExport();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before export datasources.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doExport();
							}
						});
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	adm_process_flow_datasources_editor.prototype.ExportDatasources = function(datasources, callback) {
		inConcert.app.getFlowDesignerAPI().exportFlowDatasources(datasources, function() {
			callback(true);
		});
	};
	
	adm_process_flow_datasources_editor.prototype.__import = function() {
		var self = this;

		function doImport() {
			var popup = new inConcert.adm_process_flow_import_datasources_popup(self);
			popup.Show();
		}
		
		if (self.__isProcessing()) {
			return;
		}

		//save validation
		if (!self.__hasUnsavedChanges()) {
			return doImport();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before import datasources.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doImport();
							}
						});
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	adm_process_flow_datasources_editor.prototype.ImportDatasources = function(datasources, callback) {
		var self = this;
		let added = false;
		$.each(datasources, function(_, flowDs) {
			if (self.datasources[flowDs.Id]) {
				self.__removeDatasource(flowDs.Id);
			}
			added = true;
			self.AddDatasource(flowDs, false, true);
		});

		if (added) {
			this.__new_datasource_tooltip();
		}
		callback(true);
	};
	
	adm_process_flow_datasources_editor.prototype.__new_datasource = function() {
		const self = this;
		function doCreate() {
			var popup = new inConcert.adm_process_flow_new_datasource_popup(self);
			popup.Show();
		}
		
		if (self.__isProcessing()) {
			return;
		}

		//save validation
		if (!self.__hasUnsavedChanges()) {
			return doCreate();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before create datasources.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doCreate();
							}
						});
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);

	};

	adm_process_flow_datasources_editor.prototype.__cancel = function() {
		var self = this;
		
		if (self.__isProcessing()) {
			return;
		}
		if (this.currentEditor) {
			this.currentEditor.Cancel();
		}
	};
	
	adm_process_flow_datasources_editor.prototype.__showEditor = function(editor) {
		if (this.currentEditor) {
			this.currentEditor.Close();
		}
		this.currentEditor = editor || null;
		if (this.currentEditor) {
			this.currentEditor.Show();
			this.__setTitle(this.currentEditor.GetTitle());
		}
		else {
			this.__setTitle("");
		}
	};
	
	adm_process_flow_datasources_editor.prototype.__setTitle = function(title) {
		this.$element("context_name").text(title);
	};
	
	adm_process_flow_datasources_editor.prototype.UpdateTitle = function(editor) {
		if (this.currentEditor === editor) {
			this.__setTitle(this.currentEditor.GetTitle());
		}
	};
	
	adm_process_flow_datasources_editor.prototype.ShowDatasourceEditor = function(dsId) {
		this.__showEditor(this.datasources[dsId]);
	};

	adm_process_flow_datasources_editor.prototype.GetDatasourceFromName = function(name) {
		var dsId;
		$.each(this.datasources, function(id, datasourceEditor) {
			if (datasourceEditor.data.Name === name) {
				dsId = id;
				return false;
			}
		});
		return dsId;
	};

}());