/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flow_datasources;
	var format = inConcert.Utils.format;
	function adm_process_flow_datasources_toolbar(main) {
		this.main = main;
		this.__make();
	}

	if (typeof inConcert.adm_process_flow_datasources_toolbar !== "function") {
		inConcert.adm_process_flow_datasources_toolbar = adm_process_flow_datasources_toolbar;
	}
	
	adm_process_flow_datasources_toolbar.prototype.__make = function() {
		
	};
	
	adm_process_flow_datasources_toolbar.prototype.__addSorted = function($datasources, $item, name) {
		var itemName = $item.attr(name);
		var $before = $item;
		$datasources.each(function() {
			if ($(this).attr(name).localeCompare(itemName) > 0) {
				return false;
			}
			$before = $(this);
		});
		if ($before === $item) {
			$datasources.first().before($item);
		}
		else {
			$before.after($item);
		}
	};
	
	adm_process_flow_datasources_toolbar.prototype.SelectDatasource = function(dsId, selected) {
		
		var $ds = this.main.$element("toolbar_datasource=" + dsId);
		
		$ds
			.addClass("selected")
			.toggleClass("extended", selected)
			.removeClass("childSelected");
		
		$ds.siblings("[toolbar_datasource]")
			.removeClass("childSelected")
			.removeClass("selected")
			.removeClass("extended");
		
		$ds.find(".selected")
			.removeClass("selected");

		$ds.parent()
			.scrollTo($ds);
		this.main.ShowDatasourceEditor(dsId);
	};
	
	adm_process_flow_datasources_toolbar.prototype.__insertDatasource = function($item, sorted) {
		var $container = this.main.$element("tool_bar_list");
		var $datasources = $container.children("[toolbar_datasource]");
		if (sorted || !$datasources.length) {
			$container.append($item);
		}
		else {
			this.__addSorted($datasources, $item, "datasource_name");
		}
	};

	adm_process_flow_datasources_toolbar.prototype.__ds_action_edit = function(_, $datasource)
	{
		// Rename
		this.__readonly_ds_name($datasource, false);
	};

	adm_process_flow_datasources_toolbar.prototype.__readonly_ds_name = function($datasource, readonly)
	{
		const $name_edit = $datasource.find("[toolbar_ds_edit_name]");
		const $name_readonly = $datasource.find("[toolbar_ds_readonly_name]");
		const $action_confirm = $datasource.find("[ds_action_button='confirm']");
		const toggle_actions = function() {
			$.each($datasource.find("[ds_action_button]"), function(_, element) {
				const $element = $(element);
				const action =  $element.attr("ds_action_button");
				switch (action) {
					case "confirm":
						$element[readonly? "hide": "show"]();
						break;
					default:
						$element[readonly? "show": "hide"]();
						break;
				}
			});
		};
		toggle_actions();
		$datasource.removeClass("onFocus");
		if (readonly) {
			this.main.form.readonly_control($name_edit, true);
			$name_edit.hide();
			$name_readonly.show();
			$name_edit.val($name_readonly.html());
			$action_confirm.hide();
			return;
		}

		this.main.form.readonly_control($name_edit, false);
		$name_edit.show();
		$name_edit.focus();
		$action_confirm.show();
		$name_readonly.hide();
		$datasource.addClass("onFocus");
	};

	adm_process_flow_datasources_toolbar.prototype.__ds_action_confirm = function(datasourceEditor, $datasource)
	{
		//validate
		const $name = $datasource.find("input[type='text']");
		datasourceEditor.ClearErrors();
		const name_val = $name.val();

		let validation_result = this.main.form.valid(name_val, "variable");
		if (!validation_result || !validation_result.success) {
			return datasourceEditor.__showError("<%=_T('Invalid datasource name.')%>");
		}

		validation_result = this.main.form.valid(name_val, "required");
		if (!validation_result || !validation_result.success) {
			return datasourceEditor.__showError("<%=_T('Invalid datasource name.')%>");
		}
		
		const dsId = this.main.GetDatasourceFromName(name_val);
		if (dsId && dsId !== datasourceEditor.data.Id) {
			return datasourceEditor.__showError("<%=_T('Already exists a datasource with same name.')%>");
		}

		datasourceEditor.Rename(name_val);
		this.__readonly_ds_name($datasource, true);
		this.main.__toggle_global_buttons();
	};

	adm_process_flow_datasources_toolbar.prototype.__ds_action_delete = function(datasourceEditor)
	{
		this.main.form.confirm("<%=_T('Do you want to delete datasource?')%>", function(success) {
			if (success) {
				//Validate
				datasourceEditor.__delete();
			}
		});
	};

	adm_process_flow_datasources_toolbar.prototype.__ds_action_clone = function(datasourceEditor)
	{
		this.main.form.confirm("<%=_T('Do you want to clone datasource?')%>", function(success) {
			if (success) {
				datasourceEditor.__duplicate();
			}
		});
	};

	adm_process_flow_datasources_toolbar.prototype.__makeDataSourceAction = function(action, $datasource, datasourceEditor) {
		var self = this;
		var $action = $(Mustache.render(templates.datasource_action, action));
		$action.click(function() {
			self["__ds_action_" + action.id](datasourceEditor, $datasource);
		});
		return $action;
	};

	adm_process_flow_datasources_toolbar.prototype.__insertDatasourceActions = function($item, datasourceEditor) {
		const $datasource_actions = $item.find("[datasource_actions]");
		$datasource_actions.html("");
		const self = this;
		var $actions = [];
		const has_datasources = Object.keys(this.main?.datasources || []).length > 0;
		const datasource_actions = [
			{
				id : "edit",
				description : "<%=_T('Rename')%>",
				permission: "save_flow_datasources",
				action_icon_class : "far fa-pencil",
				hidden: has_datasources? false:true,
			},
			{
				id : "clone",
				description : "<%=_T('Clone')%>",
				permission: "save_flow_datasources",
				action_icon_class : "far fa-copy",
				hidden: has_datasources? false:true,
			},
			{
				id : "delete",
				description : "<%=_T('Delete')%>",
				permission: "delete_flow_datasources",
				action_icon_class : "far fa-trash-alt",
				hidden: has_datasources? false:true,
			},
			{
				id : "confirm",
				description : "<%=_T('Confirm')%>",
				action_icon_class : "far fa-check-circle",
				hidden: has_datasources? true: false,
			},
		];
	
		$.each(datasource_actions, function(_, action) {
			if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}

			$actions.push(self.__makeDataSourceAction(action, $item, datasourceEditor));
		});
		$datasource_actions.append($actions);
	};
	
	adm_process_flow_datasources_toolbar.prototype.AddDatasource = function(datasourceEditor, sorted) {
		var self = this;
		var readonly = datasourceEditor.IsReadonly();
		
		var $item = $(Mustache.render(templates.toolbar_datasource, {
			id : datasourceEditor.data.Id,
			name : datasourceEditor.data.Name,
			type : datasourceEditor.data.Type,
			typeLabel : inConcert.flowDesignerDatasourcesTypesLabels[datasourceEditor.data.Type] || datasourceEditor.data.Type,
			description : datasourceEditor.MakeTooltip()
		}));

		$item.
		off("click").
		on("click", function(e) {
			var $target = $(e.target);
			const datasource = $(this).attr("toolbar_datasource");
			const currEditor = self.main?.currentEditor;
			if ($target.closest("[datasource_actions]").length || currEditor?.data?.Id === datasource) {
				return;
			}

			const fnCallback = function() {
				if (currEditor) {
					currEditor.is_mode_edit = false;
				}
				
				self.SelectDatasource(datasource);
				self.main.__toggle_global_buttons();	
			};
			

			//Si para el que voy a saltar tiene cambios
			if (currEditor?.HasUnsavedChanges && currEditor.HasUnsavedChanges() && self.main?.form) {
				self.main?.form?.confirm("<%=_T('Any unsaved changes will be lost!')%>", function (success) {
					if (success) {

						if (currEditor.IsNew()) {
							//Elimino el item
							self.main.__removeDatasource(currEditor.data.Id);
						}
						else {
							//Deshago los cambios.
							currEditor.reloadData(true);
						}
						fnCallback();
					}
				}, "<%=_T('Are you sure you want to leave?')%>");
				return;
			}

			fnCallback();
		})
		
		self.__insertDatasource($item, sorted);
		if (!readonly) {
			self.__insertDatasourceActions($item, datasourceEditor);

			$item
			.off("clickoutside")
			.on("clickoutside", function (e) {
				const $this = $(this);
				if ($this.find("[toolbar_ds_readonly_name]").is(":visible")) {
					return;
				}
				self.__readonly_ds_name($this, true);
			});	
		}
	};
	
	adm_process_flow_datasources_toolbar.prototype.RemoveDatasource = function(dsId) {
		this.main.$element(format("toolbar_datasource={0}", dsId)).remove();
	};
	
	adm_process_flow_datasources_toolbar.prototype.UpdateDatasource = function(datasourceEditor) {
		this.RemoveDatasource(datasourceEditor.data.Id);
		this.AddDatasource(datasourceEditor);
	};
	
	adm_process_flow_datasources_toolbar.prototype.UpdateDatasourceTooltip = function(dsId, tooltip) {
		this.main.$element(format("toolbar_datasource={0}", dsId))
			.find("[datasource_selector_title]")
			.attr("title", tooltip);
	};
	
	adm_process_flow_datasources_toolbar.prototype.UpdateDatasourceType = function(dsId, type) {
		this.main.$element(format("toolbar_datasource={0}", dsId))
			.find("[datasource_icon]")
			.attr("title", inConcert.flowDesignerDatasourcesTypesLabels[type] || type)
			.attr("class", format("far fa-database mr-2 {0}DatasourceIcon", type));
	};

}());
