/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	function adm_process_flow_main_editor(main, actionsBar, data) {
		this.id = "main";
		data = data || {};
		adm_process_flow_main_editor.uber.constructor.call(this, main, actionsBar, data.xml);
	}

	inConcert.Utils.inherit( adm_process_flow_main_editor, inConcert.adm_process_flow_editor );

	if (typeof inConcert.adm_process_flow_main_editor !== "function") {
		inConcert.adm_process_flow_main_editor = adm_process_flow_main_editor;
	}

	adm_process_flow_main_editor.prototype.__getClassName = function() {
		return "mainEditor";
	};

	adm_process_flow_main_editor.prototype.__loadStyles = function() {
		adm_process_flow_main_editor.uber.__loadStyles.call(this);
		this.__addNodeStyle("stateNode", "ico-state.svg");
	};

	adm_process_flow_main_editor.prototype.__getNodeTooltip = function(node) {
		var stateData = this.main.GetStateData(node.id);
		return stateData.description || "";
	};

	adm_process_flow_main_editor.prototype.__getNodeLabel = function(node) {
		var stateData = this.main.GetStateData(node.id);
		return stateData.name || "";
	};

	adm_process_flow_main_editor.prototype.__getConnectorLabel = function(node) {
		var data = this.GetNodeData(node);
		if (data) {
			if (data.description && data.description !== "<%=_T('New transition')%>" && data.description !== "<%=_T('Edit transition')%>") {
				return data.description;
			}
			if (data.event) {
				var events = inConcert.adm_process_flow_events[this.main.data.Type];
				return events[data.event]? events[data.event].name : data.event;
			}
		}
		return "";
	};

	adm_process_flow_main_editor.prototype.__editNode = function(node) { 
		
		const stateDraw = this.main.ShowState(node.id);
		
		if (this.editor && this.editor.graph) {
			this.editor.graph.getModel().processing = false;
		}

		let existingTab = false;
		$("[navigation_tab]").each(function(){
			if ($(this).attr("navigation_tab") === node.id) {
				existingTab = true;
				return
			}
		});

		if (existingTab) return; 

		const state = {
			id : stateDraw.data.id,
			name : stateDraw.data.name,
			description : stateDraw.data.description
		}
		const $state = this.main.__makeStateTab(state);
		
		this.main.__appendStateTab($state);
	};

	adm_process_flow_main_editor.prototype.__resetNodeAnimations = function(node) {
		const self = this;
		if (!node) return;

		var state = self.graph.view.getState(node);

		self.__editInternalTransaction(function () {
			if (node.edge) {
				try {
					state.shape.node.getElementsByTagName('path')[0].setAttribute('visibility', 'hidden');
					state.shape.node.getElementsByTagName('path')[1].removeAttribute('class', 'flow-animation');
					self.graph.setCellStyles("strokeColor", '#39F', [node]);
					self.graph.cellsOrdered([node]);
				}
				catch(err) {
					console.log(node, err);
				}
			}  else {
				var html = self.graph.getModel().getStyle(node);
				var $content = $(`<div>${html}</div>`);
					try {
						$content.children().removeClass("path-card-disabled");
					}
					catch (er) {
						console.log();
					}
				self.graph.getModel().setStyle(node, $content.html());
			}
		});
	};

	adm_process_flow_main_editor.prototype.__reanimateEdge = function(node) {
		const self = this;
		var state = self.graph.view.getState(node);
		if (!node || !node.edge || !state || !state.shape) return;
		const color = '#39F';
		self.__editInternalTransaction(function () {			
			if (node.target && 	node.target.parent && node.target.parent.value && 
				JSON.stringify(node.target.parent.value).includes('close_channel')) 
				return
			
			state.shape.node.getElementsByTagName("path")[0].removeAttribute("visibility");
			state.shape.node.getElementsByTagName("path")[0].setAttribute("stroke-width", "2");
			state.shape.node.getElementsByTagName("path")[0].setAttribute("stroke", color);
			state.shape.node.getElementsByTagName("path")[1].setAttribute("stroke", color);
			state.shape.node.getElementsByTagName("path")[1].setAttribute("class", "flow-animation");
			// state.shape.node.getElementsByTagName("path")[2].setAttribute("stroke", color);
		});
	}

	adm_process_flow_main_editor.prototype.__animateEdge = function(node, color, animate) {
		const self = this;
		var state = self.graph.view.getState(node);
		if (!node) return;

		self.__editInternalTransaction(function () {
			if (node.edge) {
				try {
					state.shape.node.getElementsByTagName('path')[0].setAttribute('visibility', 'hidden');
					state.shape.node.getElementsByTagName('path')[0].setAttribute('stroke-width', '2');
					state.shape.node.getElementsByTagName('path')[1].removeAttribute('class', 'flow-animation');
					self.graph.setCellStyles("strokeColor", color, [node]);	
					if (animate) {
						self.graph.setCellStyles("strokeColor", color, [node]);
						self.graph.cellsOrdered([node]);
					}
				}
				catch(err) {
					console.log();
				}
			} else {
				var html = self.graph.getModel().getStyle(node);
				var $content = $(`<div>${html}</div>`);
					try {
						$content.children().removeClass("path-card-disabled");
					}
					catch (er) {
						console.log();
					}
				self.graph.getModel().setStyle(node, $content.html());
			}
		});
	};

	adm_process_flow_main_editor.prototype.__makeConnectorPopup = function(connector) {
		return new inConcert.adm_process_flow_connection_main_popup(this, connector, this.main.data.Type);
	};

	adm_process_flow_main_editor.prototype.AddState = function(state) {
		var self = this;
		self.__editTransaction(function(parent) {

			const drawVersion = self.main.__getVersion();

			if (drawVersion === "2") {
				let htmlBody = Mustache.render(inConcert.templates.process_flows.state_node_template, {
					image: `custom_icon_state`, 
					description: state.description
				});
				let ports = []; // aún no le vamos a poner puertos a los estados

				let x = (self.graph.view.translate.x * -1) + 550; 
				let y = (self.graph.view.translate.y * -1) + 300;

				let node = self.__addNode(parent, null, "stateNode", null, x, y, 150, null, htmlBody, ports);
				state.id = node.id;
			} else {
				let node = self.__addNode(parent, null, "stateNode", null);
				state.id = node.id;
			}

		});
	};

	adm_process_flow_main_editor.prototype.UpdateState = function(id, data) {
		var self = this;
		var node = self.GetNodeById(id);

		if (node) {
			self.UpdateNodeLabel(node, data);
		}
	};

	adm_process_flow_main_editor.prototype.__new_state = function() {
		this.main.NewState();
	};

	adm_process_flow_main_editor.prototype.__properties = function() {
		if (this.CountSelectedNodes().nodes !== 1) {
			return;
		}
		var stateId = this.selectedNodes[0].id;
		this.main.EditStateProperties(stateId);
	};

	adm_process_flow_main_editor.prototype.__duplicate = function() {
		if (this.CountSelectedNodes().nodes !== 1) {
			return;
		}
		var stateId = this.selectedNodes[0].id;
		this.main.DuplicateState(stateId);
	};

	adm_process_flow_main_editor.prototype.__delete = function() {
		var self = this;
		if (!self.CountSelectedNodes().total) {
			return;
		}
		$.each(self.selectedNodes, function(_, node) {
			self.main.DeleteState(node.id);
		});
		self.editor.execute("delete");
	};

	adm_process_flow_main_editor.prototype.__getNodePrefix = function() {
		return "state_";
	};

}());
