/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const format = inConcert.Utils.format;

	function adm_process_flow_state_editor(data, main, actionsBar) {
		this.data = data;
		this.id = data.id;
		adm_process_flow_state_editor.uber.constructor.call(this, main, actionsBar, data && data.xml || null);
	}

	inConcert.Utils.inherit( adm_process_flow_state_editor, inConcert.adm_process_flow_editor );

	if (typeof inConcert.adm_process_flow_state_editor !== "function") {
		inConcert.adm_process_flow_state_editor = adm_process_flow_state_editor;
	}

	adm_process_flow_state_editor.prototype.GetAndValidateData = function(notValidate) {
		var data = adm_process_flow_state_editor.uber.GetAndValidateData.call(this, notValidate);
		if (data) {
			data.migration = this.__getMigrationData(data.model);
			// data.declaredVariables = this.__loadDeclaredVariables(data.editor);
			$.each(this.data, function(k, v) {
				data[k] = data[k] || v;
			});
			
			if (data.model.initial.connections.length == 0) {
				inConcert.app.m_mainWindow.showBannerMessage({ message: format(
						"<%=_T('The state {0} must contain at least one activity linked to the initial state')%>",
						data.name
					), messageType : "Error", timeout: 1500 });
				return null;
			}
		}
		return data;
	};

	adm_process_flow_state_editor.prototype.__getActivityMigrationData = function(activityData) { 
		if (activityData.data) {
			let activity = this.main.toolbar.activities[activityData.data.type];
			let migrationData = activity? activity.prototype.GetMigrationData(activityData.data, this) : null;
			return !$.isEmptyObject(migrationData)? migrationData : null;
		} else {
			return null;
		}
	};

	adm_process_flow_state_editor.prototype.__getMigrationData = function(model) {
		var self = this;
		var data = {};
		$.each(model, function(activityId, activity) {
			if (activityId === "initial") {
				return;
			}
			data[activityId] = self.__getActivityMigrationData(activity) || undefined;
		});
		return data;
	};

	adm_process_flow_state_editor.prototype.__getActivityDeclaredVariables = function(activityData) { 
		if (activityData.data) {
			let activity = this.main.toolbar.activities[activityData.data.type];
			let declaredVariables = activity ? activity.prototype.GetActivityDeclaredVariables(activityData.data, this) : [];
			return declaredVariables;
		} else {
			return [];
		}
	};

	adm_process_flow_state_editor.prototype.__loadDeclaredVariables = function() {
		var self = this;
		const model = self.data.model;

		$.each(model, function(activityId, activity) {
			if (activityId !== "initial") {
				self.__getActivityDeclaredVariables(activity);

			}
		});
	};

	adm_process_flow_state_editor.prototype.__getClassName = function() {
		return "stateEditor";
	};

	adm_process_flow_state_editor.prototype.__hasUndoManager = function() {
		return true;
	};

	adm_process_flow_state_editor.prototype.__loadStyles = function() {
		adm_process_flow_state_editor.uber.__loadStyles.call(this);
		var self = this;
		var type = self.main.data.Type;
		$.each(this.main.toolbar.activities, function(id, activity) {
			self.__addNodeStyle(id, activity.prototype.image);
			if (activity.prototype.dataStyles) {
				$.each(activity.prototype.dataStyles(type), function(_, style) {
					self.__addNodeStyle(style.id, style.imageName);
				});
			}
		});
	};

	adm_process_flow_state_editor.prototype.__getNodeTooltip = function(node) {
		return this.__getNodeLabel(node);
	};

	adm_process_flow_state_editor.prototype.__getNodeLabel = function(node) {
		let data = this.GetNodeData(node);
		if (data && data.description) {
			return data.description;
		}
		
		if(data) { 
			let activity = this.main.toolbar.activities[data.type || ""];
			if (activity) {
				return activity.prototype.name;
			}
		}

		return "";
	};

	adm_process_flow_state_editor.prototype.__editNode = function(node) {
		var self = this;
		let data = this.GetNodeData(node);
		if (data) { 
			let activity = this.main.toolbar.activities[data.type];
			let popup = new activity(this, node);
			popup.Show();
		} else {
            if (self.editor && self.editor.graph) {
                self.editor.graph.getModel().processing = false;
            }
		}
	};

	adm_process_flow_state_editor.prototype.NewActivity = function(activity) {
		if (!activity) {
			return;
		}
		var popup = new activity(this);
		popup.Show();
	};

	adm_process_flow_state_editor.prototype.UpdateData = function(data) {
		var self = this;
		$.each(data, function(k, v) {
			self.data[k] = v;
		});
	};

	adm_process_flow_state_editor.prototype.GetName = function() {
		return this.data.name;
	};

	adm_process_flow_state_editor.prototype.__properties = function() {
		if (this.CountSelectedNodes().nodes !== 1) {
			return;
		}
		this.__editNode(this.selectedNodes[0]);
	};

	adm_process_flow_state_editor.prototype.__setCopiedNodes = function() {
		this.main.SetCopiedActivities(this.__copySelectedNodes());
	};

	adm_process_flow_state_editor.prototype.__getCopiedNodes = function() {
		return this.main.GetCopiedActivities();
	};

	adm_process_flow_state_editor.prototype.__delete = function() {
		var self = this;
		if (!self.CountSelectedNodes().total) {
			return;
		}
		// self.main.__deleteVariablesOnDeleteActivity(self.selectedNodes);
		self.editor.execute("delete");
	};


	adm_process_flow_state_editor.prototype.__getNodePrefix = function() {
		return "activity_";
	};

	adm_process_flow_state_editor.prototype.AddOrUpdateActivity = function(node, data) {

		var self = this;
		let dataStyle = null;
		
		const drawVersion = this.main.__getVersion(); 

		if (drawVersion === "2") {

			if (data && data.type) {

				let activity = self.main.toolbar.activities[data.type];

				let htmlBody = null;
				let ports = [];
				if (activity.prototype.htmlBody) {
					htmlBody = activity.prototype.htmlBody(data);
				}
				if (activity.prototype.ports) {
					ports = activity.prototype.ports(data, node); 
				}

				if (activity.prototype.dataStyle) {
					dataStyle = activity.prototype.dataStyle(data);
				}

				if (node) {
					self.__editTransaction(function() {
						self.SetNodeData(node, data, ports); 
						
						if (dataStyle) {
							self.__setNodeStyle(node, dataStyle);
						}
					});

				}
				else {
					const dropCoords = self.main.GetDropCoords();
					if (dropCoords && dropCoords.yCoord !== null)
						self.__addNode(null, null, dataStyle? dataStyle : data.type, data, dropCoords.xCoord, dropCoords.yCoord, null, null, htmlBody, ports);
					else
						self.__addNode(null, null, dataStyle? dataStyle : data.type, data, null, null, null, null, htmlBody, ports);
				}
				
			} else {
				
				// es initial

				let node_for_update = node.children.find(c => c.value === "PortOut_1");

				if (node_for_update) {

					let geo_node = self.graph.getModel().getGeometry(node_for_update);
					geo_node.y = 0.23;				
					geo_node.x = 1;
					geo_node.width = 14;
					geo_node.height = 18;
					geo_node.offset = new mxPoint(-8, 8);

					node_for_update.setGeometry(geo_node);
					self.graph.refresh();						
				}
			}	

		} else {

			let activity = self.main.toolbar.activities[data.type];

			if (activity.prototype.dataStyle) {
				dataStyle = activity.prototype.dataStyle(data);
			}
			if (node) {
				self.__editTransaction(function() {
					self.SetNodeData(node, data);
					if (dataStyle) {
						self.__setNodeStyle(node, dataStyle);
					}
				});
			}
			else {
				self.__addNode(null, null, dataStyle? dataStyle : data.type, data);
			}

		}

		self.main.SetDropCoords(null,null);

	};

	adm_process_flow_state_editor.prototype.__loadDrawNodeFromXml = function(node) { 

		let data = this.GetNodeData(node);

		if (data) {
			let activity = this.main.toolbar.activities[data.type];
			if (activity && activity.prototype && activity.prototype.dataStyle) {
				let dataStyle = activity.prototype.dataStyle(data);
				if (dataStyle) {
					this.__setNodeStyle(node, dataStyle);
				}
			}
		}
		
	};

	adm_process_flow_state_editor.prototype.RunMigration = function(migrationData) {
		var self = this;
		$.each(self.data.migration || {}, function(activityId) {
			var node = self.GetNodeById(activityId.replace(self.__getNodePrefix(), ""));
			var data = self.GetNodeData(node);
			var activity = self.main.toolbar.activities[data.type];
			if (activity) {
				activity.prototype.MigrateData(data, migrationData);
				self.SetNodeData(node, data);
			}
		});
		self.data.migration = undefined;
	};

}());
