/* jshint smarttabs:true, quotmark:double, esversion: 9 */
/* global $, inConcert, mxUtils */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	var FORMAT = inConcert.Utils.format;
	var REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function FlowDesignerAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("FlowDesignerAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("FlowDesignerAPI.Constructor - base URI is required");
		}

		this.uri = params.uri + "/process_flows";
		this.uri_classes = params.uri + "/process_flow_classes";
		this.uri_datasources = params.uri + "/process_flow_datasources";
		this.cached_cultures = null;
		this.uri_apps_notifications = params.uri_admin + "/apps_notifications/search/";
		this.uri_helpdesk_api_info = params.uri_admin + "/apps_notifications/helpdesk_api_info/";

		this.uri_templates = params.uri + "/process_flow_templates";
	}

	if (typeof inConcert.FlowDesignerAPI !== "function") {
		inConcert.FlowDesignerAPI = FlowDesignerAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	FlowDesignerAPI.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("FlowDesignerAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("FlowDesignerAPI.prototype.__Get: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (response)    {callback(true, response);},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("FlowDesignerAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("FlowDesignerAPI.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("FlowDesignerAPI.prototype.__Post: data variable must be object type.");
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

/********************************************************************************************************

********************************************************************************************************/
	/*
	
	*/
	FlowDesignerAPI.prototype.getFlow = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/get/F2089C37B7C6B13E58D6A9315ED91702
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get/{1}",
			this.uri,
			encodeURIComponent(params.flowId || "")
		);

		var request_handler = this.__Get(uri, function(success, flow) {
			if (success) {
				callback(true, flow);
				return;
			}
			// error
			callback(false, flow);
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.saveFlow = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/save/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/save/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.cloneFlow = function(flowId, newName, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/clone/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/clone/", this.uri);

		var request_handler = this.__Post(uri, callback, {
			Id : flowId,
			Name : newName
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.deleteFlows = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/delete/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.searchFlows = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/search/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/search/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*

	*/
	FlowDesignerAPI.prototype.createTestSession = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/create_test_session/
		 */
		callback = callback || $.noop;
		
		var uri = FORMAT("{0}/create_test_session/", this.uri);
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*

	*/
	FlowDesignerAPI.prototype.closeTestBot = function(data, callback)
	{
		/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/close_test_bot/
			*/
		callback = callback || $.noop;
		
		var uri = FORMAT("{0}/close_test_bot/", this.uri);
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.loadFlowsByType = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/load_by_type/CHAT
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/load_by_type/{1}",
			this.uri,
			encodeURIComponent(params.proxyType || "")
		);

		var request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.validateFlow = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/validate/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/validate/", this.uri);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.exportFlow = function(flowId, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/export/
		 */
		callback = callback || $.noop;
		var uri = FORMAT("{0}/export/{1}", this.uri, flowId);
		window.open(uri, "_blank");
		callback();
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.importFlow = function(file, callback, onProgress)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/import/
		 */
		callback = callback || $.noop;
		onProgress = onProgress || $.noop;
		var uri = FORMAT("{0}/import/", this.uri);
		var xhr = new XMLHttpRequest();
		xhr.open("POST", uri, true);
		xhr.onload = function(e)  {
			var data = null;
			try {
				data = JSON.parse(e.target.responseText);
				for(let state in data.flow.Draw.states)
				{
					for(let activity in data.flow.Draw.states[state].model)
					{
						if(data.flow.Draw.states[state].model[activity].data && data.flow.Draw.states[state].model[activity].data.type && data.flow.Draw.states[state].model[activity].data.type === "execute_helpdesk")
						{
							data.flow.Draw.states[state].model[activity].data.vcc = inConcert.app.get_vcc_info().Id;
						}
					}
					let oldXmlStr = data.flow.Draw.states[state].xml;
					let oldXmlObj = mxUtils.parseXml(oldXmlStr);
					for(let node in oldXmlObj.childNodes[0].childNodes[0].childNodes)
					{
						if(oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes && oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes.value && oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes.value.value && typeof(oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes.value.value) === "string")
						{
							let jsonNode = JSON.parse(oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes.value.value);
							if(jsonNode && jsonNode.type && jsonNode.type === "execute_helpdesk")
							{
								jsonNode.vcc = inConcert.app.get_vcc_info().Id;
								oldXmlObj.childNodes[0].childNodes[0].childNodes[node].attributes.value.value = JSON.stringify(jsonNode);
							}
						}
					}
					data.flow.Draw.states[state].xml = mxUtils.getXml(oldXmlObj);
				}
			}
			catch(_) {
				
			}
			callback(e.target.status === 200, data);
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function( e ) {
			var percentage = "";
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
			}
			onProgress(percentage);
		};

		xhr.send( file );
	};

/********************************************************************************************************

********************************************************************************************************/
	/*
	
	*/
	FlowDesignerAPI.prototype.getTrace = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/get_trace/F2089C37B7C6B13E58D6A9315ED91702
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_trace/{1}",
			this.uri,
			encodeURIComponent(params.flowId || "")
		);

		var request_handler = this.__Get(uri, function(success, trace) {
			if (success) {
				callback(true, trace);
				return;
			}
			// error
			callback(false, trace);
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.getEvent = function(session, callback) 
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows/get_event/F2089C37B7C6B13E58D6A9315ED91702
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_event/{1}",
			this.uri,
			encodeURIComponent(session || "")
		);

		var request_handler = this.__Get(uri, function(success, trace) {
			if (success) {
				callback(true, trace.events);
				return;
			}
			// error
			callback(false, trace.events);
		});
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.getClasses = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/get_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_all/", this.uri_classes);

		var request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.saveClasses = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/save/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/save/", this.uri_classes);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.deleteClasses = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/delete/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete/", this.uri_classes);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.getClass = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/get/F2089C37B7C6B13E58D6A9315ED91702
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get/{1}",
			this.uri,
			encodeURIComponent(params.classId || "")
		);

		var request_handler = this.__Get(uri, function(success, flowClass) {
			if (success) {
				callback(true, flowClass);
				return;
			}
			// error
			callback(false, flowClass);
		});
		return request_handler;
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.exportFlowClasses = function(classes, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/export/
		 */
		callback = callback || $.noop;
		var uri = FORMAT("{0}/export/{1}", this.uri_classes, encodeURIComponent(JSON.stringify(classes)));
		window.open(uri, "_blank");
		callback();
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.importFlowClasses = function(file, callback, onProgress)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_classes/import/
		 */
		callback = callback || $.noop;
		onProgress = onProgress || $.noop;
		var uri = FORMAT("{0}/import/", this.uri_classes);
		var xhr = new XMLHttpRequest();
		xhr.open("POST", uri, true);
		xhr.onload = function(e)  {
			var data = null;
			try {
				data = JSON.parse(e.target.responseText);
			}
			catch(_) {
				
			}
			callback(e.target.status === 200, data);
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function( e ) {
			var percentage = "";
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
			}
			onProgress(percentage);
		};

		xhr.send( file );
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.getDatasources = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/get_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_all/", this.uri_datasources);

		var request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/

	FlowDesignerAPI.prototype.getBuiltinDatabases = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/get_builtin_databases/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_builtin_databases/", this.uri_datasources);

		var request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/

	FlowDesignerAPI.prototype.saveDatasources = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/save/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/save/", this.uri_datasources);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	FlowDesignerAPI.prototype.deleteDatasources = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/delete/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/delete/", this.uri_datasources);

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.getDatasource = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/get/F2089C37B7C6B13E58D6A9315ED91702
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get/{1}",
			this.uri,
			encodeURIComponent(params.datasourceId || "")
		);

		var request_handler = this.__Get(uri, function(success, flowDatasource) {
			if (success) {
				callback(true, flowDatasource);
				return;
			}
			// error
			callback(false, flowDatasource);
		});
		return request_handler;
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.exportFlowDatasources = function(datasources, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/export/
		 */
		callback = callback || $.noop;
		var uri = FORMAT("{0}/export/{1}", this.uri_datasources, encodeURIComponent(JSON.stringify(datasources)));
		window.open(uri, "_blank");
		callback();
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.importFlowDatasources = function(file, callback, onProgress)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flow_datasources/import/
		 */
		callback = callback || $.noop;
		onProgress = onProgress || $.noop;
		var uri = FORMAT("{0}/import/", this.uri_datasources);
		var xhr = new XMLHttpRequest();
		xhr.open("POST", uri, true);
		xhr.onload = function(e)  {
			var data = null;
			try {
				data = JSON.parse(e.target.responseText);
			}
			catch(_) {
				
			}
			callback(e.target.status === 200, data);
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function( e ) {
			var percentage = "";
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
			}
			onProgress(percentage);
		};

		xhr.send( file );
	};
	
	/*
	
	*/
	FlowDesignerAPI.prototype.getAppsNotifications = function(callback)
	{
		/*
		 * sample URI:
		 *   http://clstest2.i6.inconcertcc.com/inconcert/apps/administration/apps_notifications/search/
		 */
		callback = callback || $.noop;
		var uri = this.uri_apps_notifications;
		var data = {};
		
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;			
	};	
	
	/*
	
	*/
	FlowDesignerAPI.prototype.getAppIntegrationProperties = function(app, callback)
	{
		/*
		 * sample URI:
		 *   https://webapp01.inconcertcc.com/api/custom-property/get_custom_properties/
		 */
		callback = callback || $.noop;
		//Si la App dispone de los datos para recuperar las properties notificables
		if(app.data.Api && app.data.Api.Url && app.data.AccessToken && app.data.AccessToken.Token)
		{
			//TODO: Agregar para cada App los datos a enviar en el body del request
			const dataMAS = {
					customProperty : {
						filters : [
							{
								column : "Scope",
								value : "contact"
							}
						]
					}
				};

			var uri = app.data.Api.Url;
			var method = "";
			const headers = new Headers();
			var data = {};
			//TODO: Agregar case para cada App, personalizando uri, method, headers y data
			switch (app.id)
			{
				case "MarketingAndSales":
					uri += "/custom-property/get_custom_properties/";
					method = "POST";
					headers.append("authorization", app.data.AccessToken.Token);    
					data = dataMAS;
				break;
			}
			
			//var request_handler = this.__Post(uri, callback, data);
			//return request_handler;		
			
			const init = {
				method: method,
				headers
			};
			
			fetch(uri, init)
			.then((response) => {
				return response.json(); 
			})
			.then((data) => {
				callback(true, data);
			})
			.catch((e) => {
				callback(false, e);
			});		
		}
		else
		{
			callback(false);
		}
	};		
	
	/*
	
	*/
	FlowDesignerAPI.prototype.getHelpDeskApi = function(callback)
	{
		/*
		 * sample URI:
		 *   http://clstest2.i6.inconcertcc.com/inconcert/apps/administration/apps_notifications/search/
		 */
		callback = callback || $.noop;
		var uri = this.uri_helpdesk_api_info;
		var data = {};
		
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;	
	};
	
	
	/*########################### TEMPLATES #######################
	/*

		Obtiene la lista de templates disponibles

	*/
	/*
	
	*/
	FlowDesignerAPI.prototype.listTemplates = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows_templates/list/whatsapp/all?lang=es
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/list/{1}/{2}?lang={3}",
			this.uri_templates,
			encodeURIComponent(params.type || "webchat"),
			encodeURIComponent(params.category || "all"),
			encodeURIComponent(params.lang || "es")
		);

		if(params.page){
			uri += `&page=${params.page}`;
		}

		if(params.sort){
			uri += `&sort=${params.sort}`;
		}

		if(params.filter){
			uri += `&filter=${params.filter}`;
		}
		
		var request_handler = this.__Get(uri, function(success, templates) {
			if (success) {
				callback(true, templates);
				return;
			}
			// error
			callback(false, templates);
		});
		return request_handler;
	};

	FlowDesignerAPI.prototype.download_template = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows_templates/download/{uuid}
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/download/{1}",
			this.uri_templates,
			encodeURIComponent(params.packageId || "")
		);

		var request_handler = this.__Get(uri, function(success, templates) {
			if (success) {
				callback(true, templates);
				return;
			}
			// error
			callback(false, templates);
		});
		return request_handler;
	};

	FlowDesignerAPI.prototype.list_categories = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/process_flows_templates/categories/whatsapp
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/categories/{1}?lang={2}",
			this.uri_templates,
			encodeURIComponent(params.type || "webchat"),
			encodeURIComponent(params.lang || "es")
		);

		var request_handler = this.__Get(uri, function(success, templates) {
			if (success) {
				callback(true, templates);
				return;
			}
			// error
			callback(false, templates);
		});
		return request_handler;
	};

	FlowDesignerAPI.prototype.loadLlmProfiles = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/flow_designer/get_llm_profiles/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/get_llm_profiles/",
			this.uri
		);

		var request_handler = this.__Get(uri, function(success, profiles) {
			if (success) {
				callback(true, profiles);
				return;
			}
			// error
			callback(false, profiles);
		});
		return request_handler;
	};

}());
