/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, Mustache, mxClient */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;
	const templates = inConcert.templates.process_flows;

	//types

	let flowTypes = [
		{channel: "CALL"     , desc: "<%=_T('Call')%>"         , type: "CALL"         , is_enabled_for_allegro: false},
		{channel: "CHAT"     , desc: "<%=_T('Web Chat Bot')%>" , type: "CHAT"         , is_enabled_for_allegro: true },
		{channel: "FACEBOOK" , desc: "<%=_T('Facebook Bot')%>" , type: "FACEBOOK_BOT" , is_enabled_for_allegro: true },
		{channel: "INSTAGRAM", desc: "<%=_T('Instagram Bot')%>", type: "INSTAGRAM_BOT", is_enabled_for_allegro: true },
		{channel: "MAIL"     , desc: "<%=_T('E-mail')%>"       , type: "MAIL"         , is_enabled_for_allegro: true },
		{channel: "SMS"      , desc: "<%=_T('Sms')%>"          , type: "SMS"          , is_enabled_for_allegro: true },
		{channel: "TWITTER"  , desc: "<%=_T('Twitter Bot')%>"  , type: "TWITTER_BOT"  , is_enabled_for_allegro: true },
		{channel: "WHATSAPP" , desc: "<%=_T('Whatsapp')%>"     , type: "WHATSAPP"     , is_enabled_for_allegro: true }
	];

	if (!inConcert.adm_process_flow_types) {
		inConcert.adm_process_flow_types = flowTypes;
	}

	//events
	let genericEvents = {
		"NoEvent" : {
			name : "<%=_T('No event')%>",
			description : "<%=_T('For transitions without event.')%>",
			disclaimer : "<%=_T('Context variable __event = { name : NoEvent }')%>"
		}
	};

	let flowEvents = {
		"CALL" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When call session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming }')%>"
			},
			"Answered" : {
				name : "<%=_T('Answered')%>",
				description : "<%=_T('When conversation was answered.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : Answered }')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When call session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"CHAT" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When chat session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming }')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client chat message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When chat session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"FACEBOOK_BOT" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When chat session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewIncoming,<br>source : MESSAGE|REFERRAL(links, adds, code)|OPTIN(send to messenger)|START_BUTTON,<br>message : postback message<br>}')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client chat message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When chat session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"INSTAGRAM_BOT" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When chat session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewIncoming,<br>source : ICE_BREAKER,<br>message : payload message<br>}')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client chat message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When chat session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"TWITTER_BOT" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When chat session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming,<br>message : { text : message text, ... }<br> }')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When chat session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"MAIL" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When e-mail session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming,<br>message : { text : message text, ... }<br> }')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client e-mail arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When e-mail session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"SMS" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When sms session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming,<br>message : { text : message text, ... }<br> }')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When sms session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		},
		"WHATSAPP" : {
			"NewIncoming" : {
				name : "<%=_T('New incoming')%>",
				description : "<%=_T('When whatsapp session is created.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : NewIncoming,<br>message : { text : message text, ... }<br> }')%>"
			},
			"NewChatMessage" : {
				name : "<%=_T('New chat message')%>",
				description : "<%=_T('When a client message arrives.')%>",
				disclaimer : "<%=_T('Context variable __event = {<br>name : NewChatMessage,<br>message : { text : message text, ... }<br>}')%>"
			},
			"ChannelClosed" : {
				name : "<%=_T('Channel closed')%>",
				description : "<%=_T('When whatsapp session ends.')%>",
				disclaimer : "<%=_T('Context variable __event = { name : ChannelClosed }')%>"
			}
		}
	};

	$.each(flowEvents, function(_, channelEvents) {
		$.each(genericEvents, function(id, event) {
			channelEvents[id] = channelEvents[id] || event;
		});
	});

	if (!inConcert.adm_process_flow_events) {
		inConcert.adm_process_flow_events = flowEvents;
	}

	var globalActions = [
		// {
		// 	id : "save",
		// 	name : "<%=_T('Save')%>",
		// 	description : "<%=_T('Save process')%>",
		// 	permission : "save_flows"
		// },
		{
			id : "properties",
			name : "<%=_T('Configuration')%>",
			description : "<%=_T('Process configuration')%>"
		},
		{
			id : "classes_editor",
			name : "<%=_T('Classes')%>",
			description : "<%=_T('Declare classes to be used in code block activity.')%>",
			permission : "view_flow_classes"
		},
		{
			id : "datasources_editor",
			name : "<%=_T('Datasources')%>",
			description : "<%=_T('Setup datasources to be used by the flow.')%>",
			permission : "view_flow_datasources"
		},
		{
			id : "print",
			name : "<%=_T('Print')%>",
			description : "<%=_T('Print draw.')%>"
		},
		{
			id : "export",
			name : "<%=_T('Export')%>",
			description : "<%=_T('Export process')%>",
			permission : "export_flows"
		}
	];

	function adm_process_flow_designer(properties) {
		this.data = properties.data;
		this.data.Draw = this.data.Draw || {};
		this.subVersion = this.data.subVersion || 0;
		// this.declaredVariables = new Set();
		this.loadResultFromData = true;
		this.waba_templates = [];
		// this.LoadDeclaredVariables();
		this.variables = new Set();

		if (!this.data.Classes || $.isArray(this.data.Classes)) {
			this.data.Classes = {};
		}
		if (!this.data.Datasources || $.isArray(this.data.Datasources)) {
			this.data.Datasources = {};
		}
		this.__onClose = properties.close || $.noop;
		this.frm = properties.frm;
	}

	if (typeof inConcert.adm_process_flow_designer !== "function") {
		inConcert.adm_process_flow_designer = adm_process_flow_designer;
	}

	adm_process_flow_designer.prototype.LoadVariablesSet = function() {
		const self = this;
		self.variables = new Set();

		const states = self.stateDraws;

		Object.keys(states).forEach(function(key) {
			let state = states[key];
			let activities = state.graph.getModel().cells;
			self.GetVariablesFromStateActivities(activities);
		  });
	};

	adm_process_flow_designer.prototype.GetVariablesFromStateActivities = function(activities) {
		const self = this;

		Object.keys(activities).forEach(function(key) {
			let activity = activities[key];

			if(activity.value && self.__isJson(activity.value)) {
				let data = $.parseJSON(activity.value);
				let type = data.type;
				if (type !== "variable") {
					self.AddVariableFromResult(data);
				} else {
					self.AddVariablesFromVariableActivity(data);
				}
			}
		  });
	};

	adm_process_flow_designer.prototype.AddVariablesFromVariableActivity = function(data) {
		const self = this;	
		if (data.hasOwnProperty("variables")) {
			const variables = data.variables;
			
			for (let key in variables) {
			  if (variables.hasOwnProperty(key)) {
				let entry = variables[key];
				let name = entry.hasOwnProperty("name") ? entry.name : null;
				self.variables.add(name);
			  }
			}
		}
	};

	adm_process_flow_designer.prototype.AddVariableFromResult = function(data) {
		const self = this;	
		let resultVariable = data.hasOwnProperty("result") ? data.result : null;
		if (resultVariable) {
			self.variables.add(resultVariable);
		}
	};

	adm_process_flow_designer.prototype.$element = function(name, $parent) {
		$parent = $parent || $("#flowDesignerContainer");
		return name? $parent.find(FORMAT("[{0}]", name)) : $parent;
	};

	adm_process_flow_designer.prototype.__makeGlobalAction = function(action) {
		var self = this;
		var $action = $(Mustache.render(templates.action_button, action));
		$action.click(function() {
			self["__" + $(this).attr("action_button")]();
		});
		return $action;
	};

	adm_process_flow_designer.prototype.__makeGlobalActions = function() {
		var self = this;
		var $actions = [];
		$.each(globalActions, function(_, action) {
			if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}
			action.withText = !action.withoutText;
			$actions.push(self.__makeGlobalAction(action));
		});
		return $actions;
	};

	adm_process_flow_designer.prototype.__makeState = function(state, $container) {
		const self = this;
		
		//add editor
		this.stateDraws[state.id] = new inConcert.adm_process_flow_state_editor(state, this, this.stateActionsBar);

		//add tab
		const $tab = self.__makeStateTab(state, $container);
		

		return $tab;
	};

	// adm_process_flow_designer.prototype.__makeScrollTabsButtons = function($container) {
	// 	const $left = $(Mustache.render(templates.navigation_tab_scroll, {direction: 'left'}));
	// 	const $right = $(Mustache.render(templates.navigation_tab_scroll, {direction: 'right'}));

	// 	$container.find("[navigation_bar]").prepend($left);
	// 	$container.find("[navigation_bar]").append($right);
	// 	return $container;
	// }

	adm_process_flow_designer.prototype.__set_draw_offset = function(viewingMain) {
		const self = this;
		self.currentEditor.__set_draw_offset(viewingMain);
	};

	adm_process_flow_designer.prototype.__makeStateTab = function(state) {
		const self = this;
		const closeButton = "<div close_tab class='closeBtn'><i class='far fa-times'></i></div>";

		const $tab = $(Mustache.render(templates.navigation_tab, {
			id : state.id,
			name : state.name,
			description : state.description,
			className : "stateTab"
		}));

		$tab.append(closeButton);
		$tab.addClass("selected");
		
		$tab.click(function() {
			self.onClickStateTab(this, $("#flowDesignerContainer"));
			const tabId = $tab.attr("navigation_tab");
			self.ShowState(tabId);
			self.$element("new_state").hide();
			self.currentEditor.__set_draw_offset();

			// $nextStateTab = $(this).next();
			// const nextStateId = $nextStateTab.attr("navigation_tab");
			// if (!nextStateId) 
			// 	$("[scroll_right]").hide();
			// else 
			// 	$("[scroll_right]").show();			
			// $("[scroll_left]").show();
		});

		$tab.find("[close_tab]").click(function() {
			self.__showMain();	
			$(this).parent().remove();
			self.reagrupateTabs();
			// $("[scroll_left]").hide();
			// if (self.$element("navigation_tab").length <= 1)
			// 	$("[scroll_right]").hide();	
		});

		return $tab;
	};

	adm_process_flow_designer.prototype.reagrupateTabs = function() {
		const $tabs = $("[navigation_bar_items]").children();
		if ( !$tabs ) {
			return;
		}
		const $visibleNavBar = $("[navigation_bar_items]");
		const $invisibleNavBarTabs = $(".flowDesignerInvisibleTabList").children();
		const windowWidth = $(window).width();
		const widthPerTab = 130; //$tabs[0].width();
		const tabsWidth = $tabs.length * widthPerTab + 32; // El width de un tab es 130, el width del button con el kebab icon es 32

		const spareWidth = windowWidth - tabsWidth;
		if ( windowWidth > tabsWidth ) {
			const tabsToBeShownCount = Math.ceil(spareWidth / widthPerTab);
			if (tabsToBeShownCount > 0) {
				for (let i = 0; i < tabsToBeShownCount; i++) {
					let removedTab = $invisibleNavBarTabs.toArray().pop();
					$visibleNavBar.append(removedTab);
				}
			}
		}
	};

	adm_process_flow_designer.prototype.UpdateState = function(id, data) {

		if (!this.stateDraws[id]) {
			return;
		}

		//update data
		this.stateDraws[id].UpdateData(data);

		//update draw
		this.mainDraw.UpdateState(id, data);

		//update tab
		this.__appendStateTab(
			this.$element("navigation_tab=" + id)
				.attr("tab_name", data.name)
				.text(data.name)
				.attr("title", data.description)
				.detach());
	};
	
	adm_process_flow_designer.prototype.__appendStateTab = function($tab) {
		var name = $tab.attr("tab_name");
		var $tabNext = null;
		this.$element("navigation_tab").each(function(index) {
			if (index === 0) {
				$tabNext = $(this);
			}
			else {
				if ($(this).attr("tab_name").localeCompare(name) < 0) {
					$tabNext = $(this);
				}
				else {
					return false;
				}
			}
		});
		$tabNext.after($tab);
	};

	adm_process_flow_designer.prototype.__toggleDisableStateTabs = function(startTest) {
		let tabs = this.$element("navigation_tab");

		$.each(tabs, function(index) {
			if (startTest) {
				$(`div[navigation_tab="${tabs[index].attributes[0].value}"]`).css("pointer-events", "none");
				$(`div[navigation_tab="${tabs[index].attributes[0].value}"]`).addClass("disabled");
			} 
			else {
				$(`div[navigation_tab="${tabs[index].attributes[0].value}"]`).css("pointer-events", "auto");
				$(`div[navigation_tab="${tabs[index].attributes[0].value}"]`).removeClass("disabled");
			}
		});
	};

	adm_process_flow_designer.prototype.__getVersion = function() {
		const myData = this.data;
		return myData.Version || null;
	};

	adm_process_flow_designer.prototype.__getSubVersion = function() {
		const myData = this.data;
		return myData.subVersion || 0;
	};

	adm_process_flow_designer.prototype.__getDrawType = function() {
		const myData = this.data;
		return myData.Type || null;
	};

	adm_process_flow_designer.prototype.getCookieValue = function(name) {
		return document.cookie.match("(^|;)\\s*" + name + "\\s*=\\s*([^;]+)")?.pop() || "";
	};

	adm_process_flow_designer.prototype.AddState = function(data) {
		//add node
		this.mainDraw.AddState(data);
		//add state
		var $state = this.__makeState(data);
		this.__appendStateTab($state);
		//update label
		this.mainDraw.UpdateState(data.id, data);
		//make draw
		this.ShowState(data.id);
		this.__showMain();
	};

	adm_process_flow_designer.prototype.DeleteState = function(id) {
		var self = this;

		//remove editor
		var draws = this.stateDraws;
		this.stateDraws = {};
		//const stateNodes = Object.values(draws[id].data.model);
		// self.__deleteVariablesOnDeleteState(stateNodes);

		$.each(draws, function(drawId, draw) {
			if (drawId !== id) {
				self.stateDraws[drawId] = draw;
			}
		});

		//remove tab
		this.$element("navigation_tab=" + id).remove();
	};

	adm_process_flow_designer.prototype.__deleteVariablesOnDeleteActivity = function(selectedNodes) {
		const self = this;
		selectedNodes.forEach(function(node) {
			if (self.__isJson(node.value)) {
				let nodeProperties = JSON.parse(node.value);
				self.declaredVariables.delete(nodeProperties.result);
			}
		});
	};

	adm_process_flow_designer.prototype.__deleteVariablesOnDeleteState = function(stateNodes) {
		const self = this;
		stateNodes.forEach(function(node) {
			if (node.data) {
				let variable = node.data.result;
				if (variable) {
					self.declaredVariables.delete(variable);
				}
			}
		});
	};

	adm_process_flow_designer.prototype.__isJson = function(str) {
		try {
			JSON.parse(str);
		} catch (e) {
			return false;
		}
		return true;
	};

	adm_process_flow_designer.prototype.NewState = function() {
		var popup = new inConcert.adm_process_flow_state_popup(this);
		popup.showEditTitleButton = false;
		popup.Show();
	};

	adm_process_flow_designer.prototype.DuplicateState = function(id) {
		var state = this.stateDraws[id];
		if (!state) {
			return;
		}
		var data = state.GetAndValidateData();
		if (!data) {
			return;
		}
		var popup = new inConcert.adm_process_flow_duplicate_state_popup(this, data);
		popup.Show();
	};

	adm_process_flow_designer.prototype.EditStateProperties = function(id) {
		var state = this.stateDraws[id];
		if (!state) {
			return;
		}
		var popup = new inConcert.adm_process_flow_state_popup(this, state.data);
		popup.showEditTitleButton = false;
		popup.Show();
	};

	adm_process_flow_designer.prototype.NewActivity = function(activity) {
		if (this.currentEditor && this.currentEditor.graph && !this.currentEditor.graph.getModel().processing) {
			this.currentEditor.graph.getModel().processing = true;
			this.currentEditor.NewActivity(activity);	
		}		
	};

	adm_process_flow_designer.prototype.onClickStateTab = function($tab, $container)
	{
		const self = this;
		const $myTab = $tab;
		/*
			1) Cuando seleccionan una App que se encuentra en la navegacion de Apps ocultas, tengo que quitar el TAB de la lista de la navegacion de Apps ocultas,
				luego tengo que insertarla en la lista de TABs visibles y tengo que quitar la última TAB visible e insertarla en la lista de la navegacion de Apps ocultas.
			Esto se realiza asi, ya que el Plugin NO realiza este paso, y si no hacemos esto, no sabemos que App oculta ha seleccioando.
		*/
		// Obtengo el contenedor de TABs invisibles.
		const $invisible_container = $container.find(".okayNav__nav--invisible");
		// Valido si el mismo se encuentra abierto.
		const is_opened_invisible_container = $invisible_container.hasClass("nav-open");
		if (is_opened_invisible_container) {
			// Si se encuentra abierto, selecciono de la lista de TABs invisibles, el que han presionado.
			// const $selected_invisible_app = $invisible_container.find("[navigation_tab]");
			const $selected_invisible_app = $myTab;
			// if ($selected_invisible_app.length > 0) {
			if ($selected_invisible_app) {
				// Ahora obtengo el contenedor de TABs visibles.
				const $visible_container = $container.find(".okayNav__nav--visible");
				// Obtengo el ultimo TAB visible.
				const $last_visible_tab_app = $visible_container.find("[navigation_tab]:last");
				// Ahora realizo el switch de TABs entre las 2 listas de TABs (Invisible y Visible)
				$invisible_container.append($last_visible_tab_app);
				$visible_container.append($selected_invisible_app);
			}
		}
		// 2) Cuando seleccionan una App que se encuentra en la navegacion de Apps ocultas, tengo que cerrar de forma manual la navegacion, ya que el Plugin NO realiza esto !!!
		self.stateTabsNavigation.okayNav("closeInvisibleNav");
	};

	adm_process_flow_designer.prototype.__makeStates = function($container) {
		var self = this;
		var $tabs = [];

		//main
		this.mainDraw = new inConcert.adm_process_flow_main_editor(this, 
			new inConcert.adm_process_flow_main_actions_bar(this), this.data.Draw.main);

		var $mainTab = $(Mustache.render(templates.navigation_tab, {
			id : "main",
			name : "<%=_T('Main')%>",
			description : "<%=_T('Main')%>",
			className : "mainTab"
		}));
		$mainTab.click(function() {
			// self.onClickStateTab(this, $container);
			self.__showMain();
			// $("[scroll_left]").hide();
			// $("[scroll_right]").show();
			self.__set_draw_offset(true);
		});
		$tabs.push($mainTab);

		//states
		this.stateDraws = {};
		this.stateActionsBar = new inConcert.adm_process_flow_state_actions_bar(this);
		var sortedStates = [];
		$.each(this.data.Draw.states || {}, function(_, state) {
			sortedStates.push(state);
		});
		sortedStates.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});
		
		$.each(sortedStates || [], function(_, state) {
			$tabs.push(self.__makeState(state, $container));
		});

		return $tabs;
	};

	adm_process_flow_designer.prototype.__makeToolbar = function() {
		this.toolbar = new inConcert.adm_process_flow_toolbar(this);
		return this.toolbar.Make();
	};

	adm_process_flow_designer.prototype.__makeTrace = function($container, $bottomBar) {
		this.trace = new inConcert.adm_process_flow_trace(this);
		return this.trace.Make($container, $bottomBar);
	};

	adm_process_flow_designer.prototype.GetDropCoords = function() {
		return this.dropCoords;
	};
	
	adm_process_flow_designer.prototype.SetDropCoords = function(x,y) {
		this.dropCoords = {xCoord: x, yCoord: y};
	};

	adm_process_flow_designer.prototype.GetPreviousNode = function() {
		return this.previousNode;
	};
	
	adm_process_flow_designer.prototype.SetPreviousNode = function(node) {
		this.previousNode = node;
	};
	
	adm_process_flow_designer.prototype.GetPreviousPortIn = function() {
		return this.previousPortIn;
	};
	
	adm_process_flow_designer.prototype.SetPreviousPortIn = function(node) {
		this.previousPortIn = node;
	};

	adm_process_flow_designer.prototype.GetPreviousPortOut = function() {
		return this.previousPortOut;
	};
	
	adm_process_flow_designer.prototype.SetPreviousPortOut = function(node) {
		this.previousPortOut = node;
	};

	adm_process_flow_designer.prototype.GetTraceNodeList = function() {
		return this.traceNodeList;
	};

	adm_process_flow_designer.prototype.SetTraceNodeList = function(nodeList) {
		this.traceNodeList = nodeList;
	};

	adm_process_flow_designer.prototype.GetTestSession = function() {
		return this.testSession;
	};

	adm_process_flow_designer.prototype.SetTestSession = function(session) {
		this.testSession = session;
	};

	adm_process_flow_designer.prototype.GetLastState = function() {
		return this.lastState;
	};

	adm_process_flow_designer.prototype.SetTLastState = function(state) {
		this.lastState = state;
	};

	adm_process_flow_designer.prototype.GetLastStateNode = function() {
		return this.lastStateNode;
	};

	adm_process_flow_designer.prototype.SetLastStateNode = function(node) {
		this.lastStateNode = node;
	};

	adm_process_flow_designer.prototype.GetStateTransitionNode = function() {
		return this.stateTransitionNode;
	};

	adm_process_flow_designer.prototype.SetStateTransitionNode = function(node) {
		this.stateTransitionNode = node;
	};

	adm_process_flow_designer.prototype.GetLastStateTransition = function() {
		return this.lastStateTransition;
	};

	adm_process_flow_designer.prototype.SetLastStateTransition = function(transition) {
		this.lastStateTransition = transition;
	};

	adm_process_flow_designer.prototype.GetLastLastStateTransition = function() {
		return this.lastLastStateTransition;
	};

	adm_process_flow_designer.prototype.SetLastLastStateTransition = function(transition) {
		this.lastLastStateTransition = transition;
	};

	adm_process_flow_designer.prototype.GetTransitionNode = function() {
		return this.transitionNode;
	};

	adm_process_flow_designer.prototype.SetTransitionNode = function(node) {
		this.transitionNode = node;
	};

	adm_process_flow_designer.prototype.GetPortOutId = function() {
		return this.portOutId;
	};

	adm_process_flow_designer.prototype.SetPortOutId = function(index) {
		this.portOutId = index;
	};

	adm_process_flow_designer.prototype.GetPreviousPortOut = function() {
		return this.previousPortOut;
	};

	adm_process_flow_designer.prototype.SetPreviousPortOut = function(portOut) {
		this.previousPortOut = portOut;
	};

	adm_process_flow_designer.prototype.GetInitialTransition = function() {
		return this.initialTransition;
	};

	adm_process_flow_designer.prototype.SetInitialTransition = function(node) {
		this.initialTransition = node;
	};

	adm_process_flow_designer.prototype.GetEditorTimeoutReanimate = function() {
		return this.editorTimeout;
	};

	adm_process_flow_designer.prototype.SetEditorTimeoutReanimate = function(timeout) {
		this.editorTimeout = timeout;
	};

	adm_process_flow_designer.prototype.GetMainTimeoutReanimate = function() {
		return this.mainTimeout;
	};

	adm_process_flow_designer.prototype.SetMainTimeoutReanimate = function(timeout) {
		this.mainTimeout = timeout;
	};

	adm_process_flow_designer.prototype.GetPreviousZoomValue = function() {
		return this.previousZoomValue;
	};

	adm_process_flow_designer.prototype.SetPreviousZoomValue = function(value) {
		this.previousZoomValue = value;
	};

	adm_process_flow_designer.prototype.SetRunningTest = function(running) {
		this.runningTest = running;
	};

	adm_process_flow_designer.prototype.GetRunningTest = function() {
		return this.runningTest;
	};

	adm_process_flow_designer.prototype.LoadDeclaredVariables = function() {
		const self = this;
		const stateDraws = self.stateDraws;

		$.each(stateDraws, function(_, state) {
			state.__loadDeclaredVariables();
		});
		self.loadResultFromData = false;
		const declaredVariablesList = Array.from(self.declaredVariables);
		return declaredVariablesList;
	};

	adm_process_flow_designer.prototype.DeleteVariableFromSet = function() {
		const self = this;
		const stateDraws = self.stateDraws;

		$.each(stateDraws, function(_, state) {
			state.__loadDeclaredVariables();
		});
		self.loadResultFromData = false;
		const declaredVariablesList = Array.from(self.declaredVariables);
		return declaredVariablesList;
	};

	adm_process_flow_designer.prototype.ReanimateNode = function() {
		const node = this.GetTransitionNode();
		const stateTransition = this.GetStateTransitionNode();

		let editorTimeoutReanimate = window.setTimeout(() => {
			this.currentEditor.__reanimateEdge(node);
		}, 50);
		this.SetEditorTimeoutReanimate(editorTimeoutReanimate);
		let mainTimeoutReanimate = window.setTimeout(() => {
			this.mainDraw.__reanimateEdge(stateTransition);
		}, 50);
		this.SetMainTimeoutReanimate(mainTimeoutReanimate);
	};

	adm_process_flow_designer.prototype.StopReanimateNode = function() {
		clearTimeout(this.GetMainTimeoutReanimate());
		clearTimeout(this.GetEditorTimeoutReanimate());
	};

	adm_process_flow_designer.prototype.__handleEventTrace = function(trace) { 
		const self = this;
		let action;
		let actualState;
		let lastState = self.GetLastState();
		let type;
		let activityID;
		let node;
		let transitionNode;
		let previousNode = self.GetPreviousNode();
		let lookingForward = false;
		let mainDrawNodes = Object.values(self.mainDraw.graph.model.cells);
		let stateNode;
		let lastStateNode = self.GetLastStateNode();

		self.GetStatesList();

		$.each(trace, function(index , event) {
			if (!event) {
				return;
			}
			
			action = trace[index].action;
			if ((trace[index].customEvent ||  action.activityType) && 
			((trace[index].customEvent === "finished") || 
			((action.activityType === "close_channel") && trace[index].message.includes("end")))) 
			{
				self.trace.__testStopActions();
				return;
			}

			actualState = self.GetStateFromName(trace[index].state);
			activityID = action.id; 
			type = action.type;

			// INICIO coloreo de nodos de estado
			self.ShowState(actualState);

			stateNode = mainDrawNodes.find(cell => cell.id === actualState);
			
			if (stateNode !== lastStateNode && (lastStateNode !== undefined)){
				if (lastStateNode.value && lastStateNode.value !== undefined && lastStateNode.edges !== null) {
					self.mainDraw.__animateEdge(lastStateNode, "#39F", false);
					if (lastStateNode.edges.count > 1) {
						self.mainDraw.__animateEdge(lastStateNode.edges[0].source.edges[0], "#39F", false);
					}
					else {
						self.mainDraw.__animateEdge(lastStateNode.edges[0], "#39F", false);
					}
				}
			}

			self.mainDraw.__animateEdge(stateNode, "#39F", false);

			if (stateNode.edges !== null) {
				if (stateNode.edges.count > 1) {
					self.mainDraw.__animateEdge(stateNode.edges[0].source.edges[0], "#39F", true);
				}
				else {
					self.mainDraw.__animateEdge(stateNode.edges[0], "#39F", true);
				}
			}

			lastStateNode = stateNode;
			// FIN coloreo de nodos de estado

			// INICIO coloreo de nodos de actividades y transisiones dentro de estados
			if (previousNode) {
				self.currentEditor.__animateEdge(previousNode, "#39F" , false);
			}

			if (type === "activity" && action.type !== "") {
				if (lookingForward) {
					// INICIO coloreo de transisiones
					node = self.currentEditor.GetNodeById(activityID);
					transitionNode = self.currentEditor.GetTransitionByNodes(previousNode.id, node.id, true);	
					previousNode = transitionNode;
					lookingForward = false;
					self.currentEditor.__animateEdge(transitionNode, "#39F", true);
					self.currentEditor.graph.scrollCellToVisible(node, true);
					self.currentEditor.__animateEdge(node, "#39F", false);
					nodeList.push(node);
					nodeList.push(transitionNode);
					// FIN coloreo de transisiones
				}
				else {
					node = self.currentEditor.GetNodeById(activityID);
					previousNode = node;
					self.currentEditor.graph.scrollCellToVisible(node, true);
					self.currentEditor.__animateEdge(node, "#39F", false);
					nodeList.push(node);
				}
			}
			else if (type === "transition") {
				lookingForward = true;
			}
			lastState = actualState;
			// FIN coloreo de nodos de actividades y transisiones dentro de estados
		});

		self.SetLastStateNode(lastStateNode);
		self.SetTraceNodeList(nodeList);
		self.SetTLastState(lastState);

		if (transitionNode) {
			self.SetPreviousNode(transitionNode);
		}
		else if (node) {
			self.SetPreviousNode(node);
		}
		return trace;
	};

	var nodeList = [];

	adm_process_flow_designer.prototype.__handleEventTraceV2 = function(trace) { 
		const self = this;
		let mainDrawNodes = Object.values(self.mainDraw.graph.model.cells);
		let previousNode = self.GetPreviousNode();
		let lastState = self.GetLastState();
		let lastStateNode = self.GetLastStateNode();
		let lastStateTransition = self.GetLastStateTransition();
		let lastLastStateNode = self.GetLastLastStateTransition();
		let transitionNode = self.GetTransitionNode();
		let portOut = self.GetPreviousPortOut();
		let portOutID = self.GetPortOutId();
		let previousPortOut;
		let action;
		let actualState;
		let type;
		let activityID;
		let node;
		let stateNode;
		let portIn;
		let color;

		$.each(trace, function(index , event) {
			if (!event) {
				return;
			}

			action = trace[index].action;
			if ((trace[index].customEvent ||  action.activityType) && ((trace[index].customEvent === "finished") || ((action.activityType === "close_channel") && trace[index].message.includes("end")))) {
				const session = self.trace.GetTestSession();
				self.trace.__ToggleEventsTracePooling(session);
				return;
			}
			
			actualState = self.GetStateFromName(trace[index].state);
			activityID = action.id; 
			type = action.type;
			if (type === "activity" && action.type !== "") {
				node = self.currentEditor.GetNodeById(activityID);
			}
			self.ShowState(actualState);
			// self.__makeState(actualState);
			// INICIO coloreo de nodos de estado
			stateNode = mainDrawNodes.find(cell => cell.id === actualState);
			
			if (lastStateNode && (stateNode !== lastStateNode)) {
				lastStateTransition = self.GetStateTransitionNode();
				if (lastStateTransition) {
					self.mainDraw.__animateEdge(lastStateTransition, "#39F", false); // camino que tomó en el main editor
				}
			}

			if (stateNode && (stateNode !== lastStateNode)) {
				let stateTransition;
				if (lastStateNode) {
					stateTransition = stateNode.edges.find(edge => edge.target.id === stateNode.id && (edge.source.id === lastStateNode.id));
				}
				else {
					stateTransition = stateNode.edges.find(edge => edge.target.id === stateNode.id);
				}
				self.mainDraw.__animateEdge(stateTransition, "#39F", true);
				self.SetStateTransitionNode(stateTransition);
			}

			lastLastStateNode = lastStateNode;
			lastStateNode = stateNode;

			// FIN coloreo de nodos de estado

			//INICIO coloreo de nodos de actividades y transisiones dentro de estados

			if (previousNode && previousPortOut && (node !== previousNode)) {
				portIn = previousNode.children.find(child => child.value === "PortIn");
				let transition = self.currentEditor.GetTransitionByNodes(previousPortOut.id, portIn.id, true);
				self.currentEditor.__resetNodeAnimations(previousNode); // actividad
				self.currentEditor.__resetNodeAnimations(portIn); 
				self.currentEditor.__resetNodeAnimations(previousPortOut);
				self.currentEditor.__resetNodeAnimations(transition);
			}

			if (type === "transition" && action.type !== "" && trace[index].message.includes("TRUE")) {
				portOutID = action.port_id;
				if (!portOutID) {
					const fromNode = self.currentEditor.GetNodeById(action.activity);
					portOutID = fromNode.children.find(child => child.value.includes("PortOut")).id;
				}
				self.SetPortOutId(portOutID);
			}

			if (type === "activity" && action.type !== "") {
				node = self.currentEditor.GetNodeById(activityID);
				portIn = node.children.find(child => child.value === "PortIn");
				if (portOutID) {
					portOut = self.currentEditor.GetNodeById(portOutID);
				}
				if (!portOut) {
					portOut = node.children.find(child => child.value.includes("PortOut"));
				}
				let transition = self.currentEditor.GetTransitionByNodes(portOut.id, portIn.id, true);
				if (node.value && JSON.stringify(node.value).includes("close_channel")) { // si la actividad actual es el cierre de la interaccion
					color = "#666"; // rojo (fin)
					self.currentEditor.__resetNodeAnimations(portIn); // puerto de entrada
					self.currentEditor.__resetNodeAnimations(portOut); // puerto de salida
				} 
				else {
					color = "#39F"; // naranja (activa)
					self.currentEditor.__animateEdge(portIn, color, false, "port"); // puerto de entrada
					self.currentEditor.__animateEdge(portOut, color, false, "port"); // puerto de salida
				}
				
				self.currentEditor.__animateEdge(node, color, false, "activity"); // actividad 
				self.currentEditor.__animateEdge(transition, color, true, "transition"); // transicion

				self.currentEditor.graph.scrollCellToVisible(node, true);
				previousNode = node;
				transitionNode = transition;
				previousPortOut = portOut;
			}
		});
		// FIN coloreo de nodos de actividades y transisiones dentro de estados
		self.SetPreviousPortOut(portOut);
		self.SetPortOutId(portOutID);
		self.SetTransitionNode(transitionNode);
		self.SetLastLastStateTransition(lastLastStateNode);
		self.SetLastStateTransition(lastStateTransition);
		self.SetLastStateNode(lastStateNode);
		self.SetTraceNodeList(nodeList);
		self.SetTLastState(lastState);
		self.SetPreviousNode(node);
		
		return trace;
	};

	adm_process_flow_designer.prototype.isNumeric = function(str) {
		return !isNaN(str);
	};

	adm_process_flow_designer.prototype.__make = function() {
		var self = this;
		var drawVersion = self.__getVersion();
		self.SetPreviousZoomValue(100);

		var $container = $(Mustache.render(templates.flow_base, {
			type : this.data.Type.toLowerCase(),
			title : "<%=_T('Process flow')%>",
			name : this.data.Name,
			description : this.data.Description,
			translated_type : flowTypes.find(item => item.type === this.data.Type.toUpperCase())?.desc
		}));
		
		const $tabs = this.__makeStates($container);

		$container.find("[global_actions]").append(this.__makeGlobalActions());
		$container.find("[navigation_bar_items]").append($tabs);
		$container.find("[tool_bar_list]").append(this.__makeToolbar());
		$container.find("#flowDesignerCloseScrollInstructonsDisclaimer").click(function(){
			$container.find("#flowDesignerScrollInstructonsDisclaimer").remove();
		});

		const $navigationBar = $container.find("#adm_flow_navigation_bar");
		this.stateTabsNavigation = $navigationBar.okayNav(
			{
				parent             : "",                         // will call nav"s parent() by default
				toggle_icon_class  : "okayNav__menu-toggle",
				toggle_icon_content: "<span /><span /><span />",
				align_right        : true,                       // If false, the menu and the kebab icon will be on the left
				swipe_enabled      : true,                       // If true, you'll be able to swipe left/right to open the navigation
				threshold          : 50,                         // Nav will auto open/close if swiped >= this many percent
				resize_delay	   : 10,
				beforeOpen         : $.noop,                     // Will trigger before the nav gets opened
				afterOpen          : $.noop,                     // Will trigger after the nav gets opened
				beforeClose        : $.noop,                     // Will trigger before the nav gets closed
				afterClose         : $.noop,                     // Will trigger after the nav gets closed
				itemHidden         : $.noop,                     // Will trigger after an item moves to the hidden navigation
				itemDisplayed      : $.noop                      // Will trigger after an item moves to the visible navigation
			}
		);

		$navigationBar.find(".okayNav__nav--invisible").addClass("flowDesignerInvisibleTabList");

		$container.find("[trace_container]").append(this.__makeTrace(
			$container.find("[trace_container]"), $container.find("[bottom_bar]")));

			$container.find("[tool_bar_title]").click(function(e) {
			var $toolbar = $(this).parent();
			$toolbar.find("[toolbar_category]").removeClass("selected");
			var minimized = $toolbar.hasClass("minimized");
			if (minimized) {
				$toolbar.removeClass("minimized");
			}
			else {
				var $target = $(e.target);
				if ($target.is("[tool_bar_toggle]")) {
					$toolbar.addClass("minimized");
				}
			}
		});
		
		// $container = self.__makeScrollTabsButtons($container);

		this.$chat_test = $(templates.draw_chat_test);

		$container.find("#flowDesignerShowEventTrace").click(function() {
			$("#flowDesignerShowEventTrace").css("pointer-events", "none");
			self.trace.TestStart();				
		});

		// $container.find("[scroll_left]").click(function() {
		// 	const stateId = self.currentEditor.id;
		// 	let $previousStateTab;
		// 	$("[navigation_tab]").each(function(){
		// 		if ($(this).attr("navigation_tab") === stateId) {
		// 			$previousStateTab = $(this).prev();
		// 			return
		// 		}
		// 	});
		// 	const previousStateId = $previousStateTab.attr("navigation_tab");
		// 	if (previousStateId === "main") {
		// 		self.__showMain();	
		// 		$container.find("[scroll_left]").hide();
		// 	}
		// 	else
		// 	{
		// 		self.ShowState(previousStateId);
		// 		self.$element("new_state").hide();
		// 	}
		// 	$container.find("[scroll_right]").show();
			
		// });

		// $container.find("[scroll_right]").click(function() {
		// 	const stateId = self.currentEditor.id;
		// 	let $nextStateTab;
		// 	let $nextNextStateTab;
		// 	$("[navigation_tab]").each(function(){
		// 		if ($(this).attr("navigation_tab") === stateId) {
		// 			$nextStateTab = $(this).next();
		// 			$nextNextStateTab = $(this).next().next();
		// 			return
		// 		}
		// 	});
		// 	const nextStateId = $nextStateTab.attr("navigation_tab");
		// 	if (!$nextNextStateTab.attr("navigation_tab")) $container.find("[scroll_right]").hide();
		// 	self.ShowState(nextStateId);
		// 	self.$element("new_state").hide();
		// 	$container.find("[scroll_left]").show();
		// });

		$container.find("[undo]").click(function() {
			self.currentEditor.__undo();
		});
		
		$container.find("[redo]").click(function() {
			self.currentEditor.__redo();
		});

		$container.find("[zoom_actual]").click(function() {
			let isMain = false;
			if (self.currentEditor.id === "main") {
				isMain = true;
			}
			self.currentEditor.__zoom_actual();
			self.__set_draw_offset(isMain);
			self.__resetScrollPosition();
		});

		$container.find("[zoom_in]").click(function() {
			self.ZoomButtonsAction("in");
		});	

		$container.find("[zoom_out]").click(function() {
			self.ZoomButtonsAction("out");
		});	

		$container.find("[adm_flow_main_zoom_input]").on("keypress", function (e) {
			if (e.which !== 13) {
				return;
			}
 // SI SE PRESIONA CUALQUIER OTRA TECLA QUE NO SEA ENTER -> RETURN
			let zoomValue = parseInt($("#adm_flow_main_zoom_input").val());
			if (!self.isNumeric(zoomValue)) {
				$("#adm_flow_main_zoom_input").val(self.GetPreviousZoomValue());
				return;
			}
			const zoom = zoomValue/100;
			self.currentEditor.__zoomTo(zoom);
			self.SetPreviousZoomValue(zoom);		
		});	

		$container.find("[new_state]").click(function() {
			self.NewState();				
		});	
	
		$container.find("[flow_save]").click(function() {
			self.__save();				
		});	
 
		$container.find("[expand_button]").click(function() {
			$("body").toggleClass("flowDesignerFullScreen");

			var $expand_collapse_toggle_class = $(this).find("i");
			var expanded_view = $(this).find("i").hasClass("fa-expand");
			self.isDesignerFullScreenMode = expanded_view;
			if (expanded_view) {
				$expand_collapse_toggle_class.removeClass("fa-expand");
				$expand_collapse_toggle_class.addClass("fa-compress");
			}
			else {
				$expand_collapse_toggle_class.removeClass("fa-compress");
				$expand_collapse_toggle_class.addClass("fa-expand");
			}
		});	

		$container.find("[close_button]").click(function() {
			self.$chat_test.html("");
			self.currentEditor.graph.popupMenuHandler.hideMenu();
			self.__exitDesignerFullScreenMode();
			document.cookie = "dontShowVariablesForm=; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/inconcert/apps/agent;";
			
			
			// CLEAR ALL INTERVALS
			const interval_id = window.setInterval(function(){}, Number.MAX_SAFE_INTEGER);
			for (let i = 1; i < interval_id; i++) {
			window.clearInterval(i);
			}
			if (self.trace.GetOpenTestFlag()) {
				if (drawVersion === "2") {
					self.__clearTraceEvents();
				}
				else {
					self.__clearTraceEventsV1();
				}
				nodeList = [];
				self.SetTraceNodeList([]);
				self.cognitiveServices = null;
				self.__disposeKeyboardEvents();
				self.$element().remove();
				self.frm.$get_dom_element_by_id("grid_scroll").show();
				self.__onClose();
			} else {
				inConcert.app.m_mainWindow.hideBannerMessage();
				self.Close();
			}			
		});	

		return $container;
	};

	adm_process_flow_designer.prototype.__exitDesignerFullScreenMode = function() {
		$("body").removeClass("flowDesignerFullScreen");
	};

	adm_process_flow_designer.prototype.__keepDesignerScreenMode = function() {
		const $expand_collapse_toggle_class = $(this).find("i");
		if (this.isDesignerFullScreenMode) {
			$expand_collapse_toggle_class.removeClass("fa-compress");
			$expand_collapse_toggle_class.addClass("fa-expand");
			$("body").addClass("flowDesignerFullScreen");
		}
		else {
			$expand_collapse_toggle_class.removeClass("fa-expand");
			$expand_collapse_toggle_class.addClass("fa-compress");
			$("body").removeClass("flowDesignerFullScreen");
		}
	};

	adm_process_flow_designer.prototype.ZoomButtonsAction = function(zoomDirection) {
		const self = this;
		let newZoomValue;
		let zoomValue = parseInt($("#adm_flow_main_zoom_input").val());
		if (zoomDirection === "out") {
			newZoomValue = zoomValue - 10;
			self.currentEditor.__zoom_out();
		}
		if (zoomDirection === "in") {
			newZoomValue = zoomValue + 10; 
			self.currentEditor.__zoom_in();
		}
		$("#adm_flow_main_zoom_input").val(newZoomValue);
		self.SetPreviousZoomValue(newZoomValue);
	};
	
	adm_process_flow_designer.prototype.GetSandboxCampaign = function(callback = $.noop) {
		const self = this;
		// const accountId = "sandbox_chat";
		if (self.sandboxCampaign) {
			return callback(self.sandboxCampaign);
		}

		inConcert.invoke({
			url: inConcert.app.getAdministrationUrl() + "/campaigns/get/sandbox",
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(campaign)	{
				self.sandboxCampaign = campaign;
				callback(campaign);
			},
			error: function(_, _1) {
				const message = "<%=_T('Error loading sandbox campaign.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				callback(null);
			}
		});
	};

	adm_process_flow_designer.prototype.GetSandboxWebchatInformation= function(callback = $.noop) {
		const self = this;
		const vcc = inConcert.app.get_vcc_id();
		const campaignAtVCC = "sandbox@" + vcc;
		const accountId = `CHAT_${campaignAtVCC}_sandbox_chat`;
    	const url = `${inConcert.app.getWebchatUrl().replace("/v3", "")}` + "/resolve_account/" + campaignAtVCC+"/"+accountId;

		if (self.sandboxCampaign) {
			return callback(self.sandboxCampaign);
		}

		inConcert.invoke({
			// url: inConcert.app.getAdministrationUrl() + "/campaigns/get/sandbox",
			url: url,
			type: "GET",
			dataType: "jsonp",
			contentType: "application/json",
			success: function(campaign)	{
				self.sandboxCampaign = campaign;
				callback(campaign);
			},
			error: function(_, _1) {
				const message = "<%=_T('Error loading sandbox campaign.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				callback(null);
			}
		});
	};

	adm_process_flow_designer.prototype.GetEvent = function(session, callback) {
		callback = callback || $.noop;
		
		var flowAPI = inConcert.app.getFlowDesignerAPI();

		flowAPI.getEvent(session,
			function(success, trace) {
				if (!success) {
					inConcert.app.m_mainWindow.showBannerMessage({ message: "", messageType : "Error", timeout: 5000 });
					callback(false);
					return;
				}
				callback(trace);
			});
	};

	adm_process_flow_designer.prototype.__disposeKeyboardEvents = function() {
		$.each(this.keyboardEvents || {}, function(event, handler) {
			$(document).off(event, handler);
		});
		this.keyboardEvents = null;
	};

	adm_process_flow_designer.prototype.__bindKeyboardEvents = function() {
		var self = this;

		function runAction(action) {
			if (self.__isProcessing() || self.IsPopupVisible() || !self.currentEditor) {
				return;
			}
			self.currentEditor.actionsBar.RunAction(action);
		}

		this.keyboardEvents = {
			"keyup.process_flow_main" : function(e) {
				//delete
				if (e.keyCode === 46) {
					return runAction("delete");
				}
				//ctrl +
				if (e.ctrlKey) {
					//c
					if (e.keyCode === 67) {
						return runAction("copy");
					}
					//v
					if (e.keyCode === 86) {
						return runAction("paste");
					}
					//z
					if (e.keyCode === 90) {
						return runAction("undo");
					}
					//y
					if (e.keyCode === 89) {
						return runAction("redo");
					}
				}
			}
		};

		$.each(this.keyboardEvents || {}, function(event, handler) {
			$(document).on(event, handler);
		});
	};

	adm_process_flow_designer.prototype.__scrollWithArrows = function() {
		var self = this;
		let dx = 0;
		let dy = 0;

		$(document).on("keydown.process_flow_main", function (e) {
			e = e || window.event;
			if (e.keyCode === 38 ) { // UP
				dy = -10;
				self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x, self.currentEditor.graph.view.translate.y + dy);
			} else if (e.keyCode === 40 ) { // DOWN
				dy = 10;
				self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x, self.currentEditor.graph.view.translate.y + dy);
			} else if (e.keyCode === 37 ) { // LEFT
				dx = -10;
				self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x + dx, self.currentEditor.graph.view.translate.y);
			} else if (e.keyCode === 39)  {
				dx = 10;
				self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x + dx, self.currentEditor.graph.view.translate.y);
			}
		}); 
	};

	adm_process_flow_designer.prototype.__scrollWithMouseWheel = function(up, shift) {
		var self = this;
		let dx = 0;
		let dy = 0;

		if (!up && !shift) { // UP
			dy = -10;
			self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x, self.currentEditor.graph.view.translate.y + dy);
		} else if (up && !shift) { // DOWN
			dy = 10;
			self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x, self.currentEditor.graph.view.translate.y + dy);
		} 
		else if (up && shift) { // LEFT
			dx = -10;
			self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x + dx, self.currentEditor.graph.view.translate.y);
		} else if (!up && shift)  {
			dx = 10;
			self.currentEditor.graph.view.setTranslate(self.currentEditor.graph.view.translate.x + dx, self.currentEditor.graph.view.translate.y);
		}
	};

	adm_process_flow_designer.prototype.__bindMouseEvents = function() {
		const container = this.$element("draw_container")[0];
		container.onwheel = (event) => {
			if (!this.currentEditor) {
				return;
			}

			event.preventDefault();
			const up = event.deltaY < 0;
			if (event.ctrlKey) {
				this.currentEditor.__onWheelZoom(up);
			} else {
				this.__scrollWithMouseWheel(up, event.shiftKey);
			}
		};
	};

	adm_process_flow_designer.prototype.Show = function() {
		var self = this;
		self.Close(true);
		if (!mxClient.isBrowserSupported()) {
			inConcert.app.m_mainWindow.showBannerMessage({ message: "<%=_T('Browser not supported.')%>", timeout: 5000 });
			return;
		}
		var $container = self.__make();
		$("#flow_designer_dashboard").append($container);
		self.$element("editor_container").append(this.$chat_test);
		
		self.frm.$get_dom_element_by_id("grid_scroll").hide();

		if(self.data.Type !== "CHAT") {
			$("#flowDesignerShowEventTrace").hide();
		}

		self.__bindKeyboardEvents();
		self.__bindMouseEvents();

		self.__initializeEditors(function() {
			self.__showMain();
			self.savedData = JSON.stringify(self.__makeData(true));

			self.__processMigration(function() {
				self.__processClassesMigration(function() {
					self.__processDatasourcesMigration(function() {
						// const subVersion = self.__getSubVersion();
						// if (subVersion !== self.subVersion) {
							self.__renderNodesHtml();
						// }
					});
				});
			});

			self.windowResizeInterval = window.setInterval(function(){
				self.onWindowResize();
			}, 50);
		});
	};

	adm_process_flow_designer.prototype.__initializeEditors = function(callback) {
		const self = this;
		inConcert.app.getMainWindow().showGenericWaiting("<%=_T('Loading draw...')%>");
		inConcert.Utils.asyncIterate(
			self.stateDraws,
			(_state, callback, stateId) => inConcert.Utils.zeroTimeout(() => {
				self.ShowState(stateId);
				callback();
			}),
			() => {
				callback();
			}
		);
	};

	adm_process_flow_designer.prototype.onWindowResize = function() {
		const $tabs = $("[navigation_bar_items]").children();
		if ( !$tabs ) {
			return;
		}
		const $invisibleNavBar = $(".flowDesignerInvisibleTabList");
		const windowWidth = $(window).width();
		const widthPerTab = 130; //$tabs[0].width();
		const tabsWidth = $tabs.length * widthPerTab + 32; // El width de un tab es 130, el width del button con el kebab icon es 32

		const spareWidth = tabsWidth - windowWidth;
		if ( windowWidth < tabsWidth ) {
			const tabsToBeHidenCount = Math.ceil(spareWidth / widthPerTab);
			if (tabsToBeHidenCount > 0) {
				for (let i = 0; i < tabsToBeHidenCount; i++) {
					let removedTab = $tabs.toArray().pop();
					$invisibleNavBar.append(removedTab);
				  }
			}
		}
	};

	adm_process_flow_designer.prototype.__hasUnsavedChanges = function() {
		return JSON.stringify(this.__makeData(true)) !== this.savedData;
	};

	adm_process_flow_designer.prototype.Close = function(noEvent) {
		var self = this;
		var drawVersion = self.__getVersion();
		const close_cross_icon = true;
		
		const close = function() {
			if (drawVersion === "2") {
				self.__clearTraceEvents();
			}
			else {
				self.__clearTraceEventsV1();
			}
			clearInterval(self.windowResizeInterval);
			nodeList = [];
			self.SetTraceNodeList([]);
			self.cognitiveServices = null;
			self.__disposeKeyboardEvents();
			self.$element().remove();
			self.frm.$get_dom_element_by_id("grid_scroll").show();
			if (!noEvent) {
				self.__onClose();
			}
			inConcert.app.getMainWindow().hideGenericWaiting();
		};

		//confirmation
		if (noEvent || !this.__hasUnsavedChanges()) {
			return close();
		}
		$.inconcertPopup.Show(
			"<%=_T('Exit')%>",
			"<%=_T('Do you want to save changes?')%>",
			$.inconcertPopup.ButtonSets.BS_SAVEEXIT,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								close();
							}
						});
					case "close":
						return close();
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp popUp260w flowDesignerGlobalModal",
				overlay_class   : "confirmationOverlay"
			},
			close_cross_icon
		);
	};

	adm_process_flow_designer.prototype.__process = function(process) {
		this.processing = !!process;
		this.$element("processing").toggle(this.processing);
	};

	adm_process_flow_designer.prototype.__isProcessing = function() {
		return !!this.processing;
	};

	adm_process_flow_designer.prototype.__makeDraw = function(notValidate) {
		var self = this;
		//main draw
		var main = this.mainDraw.GetAndValidateData(notValidate);
		if (!main) {
			this.__showMain();
			return null;
		}
		//states
		var error = false;
		var states = {};
		$.each(this.stateDraws, function(id, stateDraw) {
			var state = stateDraw.GetAndValidateData(notValidate);
			if (!state) {
				self.ShowState(id);
				error = true;
				return false;
			}
			states[id] = state;
		});
		if (error) {
			return null;
		}
		return {
			main : main,
			states : states
		};
	};

	adm_process_flow_designer.prototype.__makeData = function(clone) {
		var data;
		if (clone) {
			data = JSON.parse(JSON.stringify(this.data));
		}
		else {
			data = this.data;
		}
		data.Draw = this.__makeDraw(clone);
		if (!data.Draw) {
			return null;
		}
		return data;
	};

	adm_process_flow_designer.prototype.__saveAction = function(data, callback =  $.noop)
	{
		const self = this;
		if (this.trace.GetOpenTestFlag()) {
			return callback(false, "<%=_T('Must close the test before saving the flow.')%>");
		}
		const flowAPI = inConcert.app.getFlowDesignerAPI();
		data.subVersion = self.subVersion + 1;
		self.subVersion = data.subVersion;

		let flowId = typeof(data.Id) === "object" ? data.Id.flowId : data.Id;

		flowAPI.saveFlow({[flowId]: data}, function(success, error) {
			if (success) {
				callback(true);
				return;
			}
			let message = "<%=_T('Error saving flow.')%>";
			switch(error && error.result) {
				case "FORBIDDEN":
					message = "<%=_T('Unable to save process flow. You dont have permission.')%>";
					break;

				case "INVALID_PROCESS":
					message = self.__sendToMainDrawOnMissingTransition();
					break;

				case "ONLY_ALLOWS_ONE_WHATSAPP_ACCOUNT":
					message = FORMAT("<%=_T('The process can only have only one WhatsApp account between the different activities. You have {0} WhatsApp accounts configured.')%>", error.totalError);
					break;

				case "UNABLE_TO_CHANGE_OR_REMOVE_CONFIGURED_ACCOUNTS_FLOW_IN_USE":
					let firstConflict;
					for (let id in error.dataError) {
						if (error.dataError.hasOwnProperty(id)) {
							firstConflict = error.dataError[id];
							break;
						}
					}
					message = FORMAT("<%=_T('Unable to configure accounts in some activities. The process is in use different account {0} of campaign {1}.')%>", firstConflict.account_name, inConcert.app.get_id(firstConflict.campaign_id));
					break;

				case "UNABLE_TO_ASSIGN_FLOW_ACCOUNT_HAS_ASSIGNED_OTHER_FLOW_ON_SCHEDULE":
				case "UNABLE_TO_ASSIGN_FLOW_ACCOUNT_HAS_ASSIGNED_OTHER_FLOW_OUT_OF_SCHEDULE":
					message = "<%=_T('Unable to configure campaign account. The selected account has another process associated with it.')%>";
					break;
			}
			callback(false, message);
		});
	};

	adm_process_flow_designer.prototype.__sendToMainDrawOnMissingTransition = function() {
		this.__showMain();
		return "<%=_T('Must set an initial state transition.')%>";
	};

	adm_process_flow_designer.prototype.__clearTraceEvents = function() {
		const self = this;
		if (!self.stateDraws) {
			return;
		}

		const draws = Object.values(self.stateDraws);
		const mainNodes = self.mainDraw.graph.model.cells;
		let initialTransition = self.GetInitialTransition();
		let editor;

		$.each(draws, function(index) {
			editor = draws[index];
			const editorNodes = editor.graph.model.cells;
			if (!editorNodes) {
				return;
			}

			$.each(editorNodes, function(index) {
				if (editorNodes[index].id === "initial") {
					if (editorNodes[index].children[0].edges) {
						initialTransition = editorNodes[index].children[0].edges[0];
					}
				}
				else if (!(editorNodes[index].target && 
						editorNodes[index].target.parent && 
						editorNodes[index].target.parent.value &&
						JSON.stringify(editorNodes[index].target.parent.value).includes("close_channel"))) {
							editor.__resetNodeAnimations(editorNodes[index]);
						}

				if (editorNodes[index].value && JSON.stringify(editorNodes[index].value).includes("close_channel")) {
					if (editorNodes[index].children[0].edges) { //coloreo todos las transisiones que entran a la actividad close_channel
						let transitionsToCloseChat = editorNodes[index].children[0].edges;
						$.each(transitionsToCloseChat, function(index) {
							// editor.__animateEdge(transitionsToCloseChat[index], "#39F", false, "transition");
							editor.__animateEdge(transitionsToCloseChat[index], "#666", false, "transition");
						});
					}
				}
					
				});
				editor.__animateEdge(initialTransition, "#00A02D", false, "transition");
				self.SetInitialTransition(undefined);
		});

		$.each(mainNodes, function(index) {
			if (mainNodes[index].id === "initial")  {
				if (mainNodes[index].children[0].edges) {
					initialTransition = mainNodes[index].children[0].edges[0];
				}
			}

			if (mainNodes[index].edges) {
				self.mainDraw.__resetNodeAnimations(mainNodes[index].edges[0]);
				$.each(mainNodes[index].edges, function(index2) {
					self.mainDraw.__resetNodeAnimations(mainNodes[index].edges[index2]);
				});
			}
		});
		self.mainDraw.__animateEdge(initialTransition, "#00A02D", false);

		self.savedData = JSON.stringify(self.__makeData(true));

	};

	adm_process_flow_designer.prototype.__clearTraceEventsV1 = function() {
		const self = this;
		if (!self.stateDraws) {
			return;
		}

		const draws = Object.values(self.stateDraws);
		const mainNodes = self.mainDraw.graph.model.cells;
		let editor;

		$.each(draws, function(index/*, draw*/) {
			editor = draws[index];
			const editorNodes = editor.graph.model.cells;
			if (!editorNodes) {
				return;
			}

			
			$.each(editorNodes, function(index) {
				editor.__resetNodeAnimations(editorNodes[index]);
			});
		});

		$.each(mainNodes, function(index) {
			self.mainDraw.__resetNodeAnimations(mainNodes[index]);
		});

		self.savedData = JSON.stringify(self.__makeData(true));
	};

	adm_process_flow_designer.prototype.__makeAllGrey = function() {
		const self = this;
		const draws = Object.values(self.stateDraws);
		const mainNodes = self.mainDraw.graph.model.cells;
		let initialTransition = self.GetInitialTransition();
		let editor;

		$.each(draws, function(index) {
			editor = draws[index];
			const editorNodes = editor.graph.model.cells;
			if (!editorNodes) {
				return;
			}

			
			$.each(editorNodes, function(index) {
				if (editorNodes[index].id === "initial") {
					initialTransition = editorNodes[index].children[0].edges[0];
					editor.__animateEdge(editorNodes[index], "#00A02D", false, "activity");
				}
				editor.__setAllGray(editorNodes[index]);
			});
			editor.__animateEdge(initialTransition, "#00A02D", false, "transition");
			self.SetInitialTransition(undefined);
		});

		$.each(mainNodes, function(index) {
			if (mainNodes[index].id === "initial") {
				initialTransition = mainNodes[index].children[0].edges[0];
			}

			if (initialTransition !== mainNodes[index]) {	
				if (mainNodes[index].edges) {
					$.each(mainNodes[index].edges, function(index2) {
						self.mainDraw.__setAllGray(mainNodes[index].edges[index2]);
					});
				}
			}

			if (initialTransition === mainNodes[index]) {
				self.SetInitialTransition(undefined);
			}
		});
	};

	adm_process_flow_designer.prototype.__makeAllGreyV1 = function() {
		const self = this;
		const draws = Object.values(self.stateDraws);
		const mainNodes = self.mainDraw.graph.model.cells;
		let editor;

		$.each(draws, function(index/*, draw*/) {
			editor = draws[index];
			const editorNodes = editor.graph.model.cells;
			if (!editorNodes) {
				return;
			}
			$.each(editorNodes, function(index) {
				if (editorNodes[index].source && editorNodes[index].source.id === "initial") {
					editor.__animateEdge(editorNodes[index], "#00A02D", false, false);
				}
				else {
					editor.__setAllGray(editorNodes[index]);
				}
			});
		});

		$.each(mainNodes, function(index) {
			// si el nodo es un puerto de entrada entonces tiene una transition asociada
			if (mainNodes[index].edges) {
				if (mainNodes[index].source && mainNodes[index].source.id === "initial") {
					editor.__animateEdge(mainNodes[index], "#00A02D", false, false);
				}
				else {
					editor.__setAllGray(mainNodes[index]);
				}
			}
		});
	};

	adm_process_flow_designer.prototype.__renderNodesHtml = function(callback = $.noop) {
		const self = this;		
		inConcert.app.getMainWindow().showGenericWaiting("<%=_T('Loading draw...')%>");
		inConcert.Utils.asyncIterate(
			self.stateDraws,
			(_state, callback, stateId) => inConcert.Utils.zeroTimeout(() => {
				self.__renderStateNodesHtml(stateId, callback);
			}),
			() => {
				inConcert.app.getMainWindow().hideGenericWaiting();
				self.savedData = JSON.stringify(self.__makeData(true));
				callback();
			}
		);
	};

	adm_process_flow_designer.prototype.__renderStateNodesHtml = function(stateId, callback = $.noop) {
		const self = this;		
		const editor = self.stateDraws[stateId];

		inConcert.Utils.asyncIterate(
			editor.graph.model.cells || [],
			(node, callback) => inConcert.Utils.zeroTimeout(() => {
				self.__renderStateNodeHtml(editor, node, callback);
			}),
			() => {
				callback();
			}
		);
	};

	adm_process_flow_designer.prototype.__renderStateNodeHtml = function(editor, node, callback = $.noop) {
		if (!node ||
			!node.value ||
			node.edge ||
			node.id === "initial" ||
			JSON.stringify(node.value).substring(0,5).includes("Port")
		) {
			return callback();
		}

		editor.__undoFreeScope(
			() => editor.AddOrUpdateActivity(node, editor.GetNodeData(node), true)
		);
		callback();
	};

	adm_process_flow_designer.prototype.__animateInitialEdges = function() {
		const self = this;
		const draws = Object.values(self.stateDraws);
		const mainNodes = self.mainDraw.graph.model.cells;
		let editor;

		$.each(draws, function(index/*, draw*/) {
			editor = draws[index];
			const editorNodes = editor.graph.model.cells;
			if (!editorNodes) {
				return;
			}
			$.each(editorNodes, function(index) {
				if (editorNodes[index].id === "initial") {
					editor.__animateEdge(editorNodes[index].children[0].edges[0], "#00A02D", false, false);
					editor.__resetNodeAnimations(editorNodes[index].children[0].edges[0].target);	
				}
			});
		});

		$.each(mainNodes, function(index) {
			if (mainNodes[index].id === "initial") {
				self.mainDraw.__animateEdge(mainNodes[index].children[0].edges[0], "#00A02D", false, false);
			}
		});
	};

	adm_process_flow_designer.prototype.__save = function(callback) { 
		callback = callback || $.noop;
		var self = this;
		var drawVersion = self.__getVersion(); 

		if (this.__isProcessing()) {
			return callback(false);
		}

		var data = this.__makeData();
		if (!data) {
			return callback(false);
		}
		if (drawVersion === "2") {
			self.__clearTraceEvents();
		}
		else {
			self.__clearTraceEventsV1();
		}
		this.__process(true);
		this.__saveAction(data, function(status, message) {
			if (status) {
				self.savedData = JSON.stringify(data);
				message = "<%=_T('Process saved successfully.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Ok", timeout: 1500 });
			}
			else {
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, timeout: 5000 });
			}
			self.__process(false);
			callback(status);
		});
	};

	adm_process_flow_designer.prototype.__export = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (this.__isProcessing()) {
			return callback(false);
		}

		function doExport() {
			inConcert.app.getFlowDesignerAPI().exportFlow(self.data.Id, function() {
				callback(true);
			});
		}

		//save validation
		if (!this.__hasUnsavedChanges()) {
			return doExport();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before export process.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doExport();
							}
							else {
								callback(false);
							}
						});
				}
				callback(false);
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp popUp260w flowDesignerGlobalModal",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_process_flow_designer.prototype.__properties = function(callback) {
		callback = callback || $.noop;
		if (this.__isProcessing()) {
			return callback(false);
		}
		var popup = new inConcert.adm_process_flow_configuration_popup(this, this.data);
		popup.Show(function() {
			callback(true);
		});
	};

	adm_process_flow_designer.prototype.__validateProperties = function(properties, callback)
	{
		callback = callback || $.noop;

		var params = {};
		params[properties.Id] = properties;
		var flowAPI = inConcert.app.getFlowDesignerAPI();

		flowAPI.validateFlow(params,
			function(success, error) {
				if (!success) {
					callback(false, error && error.result);
					return;
				}
				callback(true);
			});
	};

	adm_process_flow_designer.prototype.UpdateProperties = function(properties, callback, notValidate) {
		callback = callback || $.noop;
		var self = this;
		properties.Id = this.data.Id;
		
		function doUpdate(status, error) {
			if (status) {
				$.each(properties, function(k, v) {
					self.data[k] = v;
				});
			}
			self.$element("flow_title").attr("title", self.data.Description);
			self.$element("flow_name").text(self.data.Name);
			callback(status, error);
		}
		
		if (notValidate) {
			doUpdate(true);
		}
		else {
			this.__validateProperties(properties, doUpdate);
		}
	};

	adm_process_flow_designer.prototype.__showEditor = function(editor) {
		const self = this;
		if (this.currentEditor) {
			if (this.currentEditor === editor) {
				return;
			}
			this.currentEditor.Close();
		}
		this.currentEditor = editor;
		this.currentEditor.Show();
		this.$element("navigation_tab")
			.removeClass("selected")
			.filter(FORMAT("[navigation_tab={0}]", this.currentEditor.id))
			.addClass("selected");

		const $activityImg = $("[drag_activity]");
			$activityImg.each(function(){
				let img = $(this);
				img.src = "https://media.geeksforgeeks.org/wp-content/uploads/20190529122828/bs21.png";
				self.currentEditor.addSidebarIcon(
					$(this),
					"Drag it",
					$(this),
					$(this).attr("drag_activity"));
			});
		self.__resetScrollPosition();
	};

	adm_process_flow_designer.prototype.__resetScrollPosition = function() {
		if (this.currentEditor.id === "main") {
			this.currentEditor.x = 0;
			this.currentEditor.y = 0;
		}
		else {
			this.currentEditor.x = 368;
			this.currentEditor.y = 0;
		}
	};

	adm_process_flow_designer.prototype.__showMain = function() {
		// this.stateTabsNavigation.okayNav("openInvisibleNav");
		this.$element("tool_bar").hide();
		this.$element("draw_container").removeClass("withToolbar");
		this.$element("new_state").show();
		// this.$element("scroll_left").hide();
		this.$element("undo_redo").hide();
		this.__showEditor(this.mainDraw);
		this.__set_draw_offset(true);
	};

	adm_process_flow_designer.prototype.ShowState = function(stateId) {
		var stateDraw = this.stateDraws[stateId];
		if (stateDraw) {
			// this.stateTabsNavigation.okayNav("openInvisibleNav");
			this.$element("tool_bar").show();
			this.$element("draw_container").addClass("withToolbar");
			this.$element("new_state").hide();
			// this.$element("scroll_left").show();
			this.$element("undo_redo").show();
			this.__showEditor(stateDraw);
		}
		this.currentEditor.__set_draw_offset();

		return stateDraw;
	};

	adm_process_flow_designer.prototype.GetStateFromName = function(name) {
		var stateId;
		$.each(this.stateDraws, function(id, draw) {
			if (draw.GetName() === name) {
				stateId = id;
				return false;
			}
		});
		return stateId;
	};
	
	adm_process_flow_designer.prototype.GetStatesList = function() {
		var states = [];
		$.each(this.stateDraws, function(id, draw) {
			states.push({
				id : "state_" + id,
				name : draw.GetName()
			});
		});
		return states;
	};
	
	adm_process_flow_designer.prototype.GetStateName = function(id) {
		id = id? id.replace("state_", "") : "";
		return this.stateDraws[id] && this.stateDraws[id].GetName() || "";
	};

	adm_process_flow_designer.prototype.GetStateData = function(id) {
		return this.stateDraws[id] && this.stateDraws[id].data || {};
	};

	adm_process_flow_designer.prototype.__print = function(callback) {
		callback = callback || $.noop;
		this.currentEditor.Print(callback);
	};

	adm_process_flow_designer.prototype.SetCopiedActivities = function(activities) {
		this.copiedActivities = activities;
	};

	adm_process_flow_designer.prototype.GetCopiedActivities = function() {
		return this.copiedActivities;
	};

	adm_process_flow_designer.prototype.IsPopupVisible = function() {
		return !!$("#flowDesignerPopup:visible").length;
	};

	adm_process_flow_designer.prototype.GetBotType = function()
	{
		let vccInfo = inConcert.app.get_vcc_info();
		if (!vccInfo) {
			return "";
		}
		let botInfo = vccInfo.BotInfo;
		if (!botInfo || vccInfo.BotInfo.Type.toUpperCase() === "LUIS") {
			return "LUIS";
		}
		else if (vccInfo.BotInfo.Type.toUpperCase() === "RASA") {
			return "RASA";
		}
		throw("Unrecognized both " + vccInfo.BotInfo.Type);
	};

	adm_process_flow_designer.prototype.LoadBotApps = function(callback = $.noop) {
		const self = this;
		if (!inConcert.app.is_enabled_nlu()) {
			return callback([]);
		}
		if (this.cognitiveServices) {
			return callback(this.cognitiveServices);
		}
		const botType = this.GetBotType();
		let botApi;
		if (botType === "LUIS") {
			botApi = inConcert.app.getLuisAPI();
		}
		else if (botType === "RASA") {
			botApi = inConcert.app.getRasaAPI();
		}
		else {
			throw("Unrecognized both " + botType);
		}
		botApi.loadApps(function(status, apps) {
			if (status) {
				self.cognitiveServices = apps;
			}
			else {
				let message = FORMAT("<%=_T('Error loading BOT {0} apps.')%>", botType);
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(self.cognitiveServices || []);
		}, true);
	};

	adm_process_flow_designer.prototype.LoadLlmProfiles = function(callback = $.noop) {
		const self = this;
	
		let botApi;
		botApi = inConcert.app.getFlowDesignerAPI();

		botApi.loadLlmProfiles(function(status, apps) {
			if (status) {
				self.llmProfiles = apps;
			}
			else {
				let message = "<%=_T('Error loading LLM Profiles.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(self.llmProfiles || []);
		}, true);
	};

	adm_process_flow_designer.prototype.LoadBotLuisIntents = function(app_id, callback = $.noop) {
	
		const self = this;
		if (!inConcert.app.is_enabled_nlu()) {
			return callback([]);
		}
		/*if (this.cognitiveServices) {
			return callback(this.cognitiveServices);
		}*/
		const botType = this.GetBotType();
		let botApi;
		if (botType === "LUIS") {
			botApi = inConcert.app.getLuisAPI();
		}
		else if (botType === "RASA") {
			botApi = inConcert.app.getRasaAPI();
		}
		else {
			throw("Unrecognized both " + botType);
		}

		const params = {appId: app_id, paginated: {page: 1, rows_per_page: 100}};

		botApi.loadAllIntents(params, function(status, intents) {
			if (status) {
				self.cognitiveIntents = intents;
			}
			else {
				let message = FORMAT("<%=_T('Error loading BOT {0} entities.')%>", botType);
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(self.cognitiveIntents || []);
		});

	};

	adm_process_flow_designer.prototype.__processMigrationTask = function(taskId, taskData, callback = $.noop) {
		let constructor = inConcert.adm_process_flow_migrations[taskId];
		if (!constructor) {
			return callback(false, FORMAT("<%=_T('Migration task {0} not found.')%>", taskId));
		}
		let task = new constructor(this, taskData, true /* overlay */);
		task.Process(function(status, data) {
			callback(status, data);
		});
	};

	adm_process_flow_designer.prototype.__processMigration = function(callback = $.noop) {
		const self = this;
		if (!self.data.Migration) {
			return callback();
		}
		var orderedTasks = [];
		$.each(this.data.Migration, function(taskId, taskData) {
			orderedTasks.push({ id : taskId, data : taskData });
		});
		var migrationData = {};
		(function runTask(i) {
			if (i >= orderedTasks.length) {
				return self.__runMigration(migrationData, callback);
			}
			self.__processMigrationTask(orderedTasks[i].id, orderedTasks[i].data, function(status, data) {
				if (status) {
					migrationData[orderedTasks[i].id] = data;
					runTask(i + 1);
				}
				else {
					inConcert.app.m_mainWindow.showBannerMessage({ message: data, messageType : "Error", timeout: 5000 });
					self.Close();
				}
			});
		})(0);
	};

	adm_process_flow_designer.prototype.__runMigration = function(data, callback) {
		callback = callback || $.noop;
		if (!$.isEmptyObject(data)) {
			$.each(this.stateDraws, function(_, stateDraw) {
				stateDraw.RunMigration(data);
			});
		}
		this.data.Migration = undefined;
		callback();
	};
	
	adm_process_flow_designer.prototype.__processClassesMigration = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.data.ClassesMigration) {
			return callback();
		}
		var classesMigration = self.data.ClassesMigration;
		self.data.ClassesMigration = undefined;
		self.data.Classes = {};
		var classes = [];
		var overrideClasses = "";
		var overrideClasses_d = {};
		$.each(classesMigration, function(_, flowClass) {
			//existing id with different name
			if (flowClass.existingId && flowClass.existingId !== flowClass.data.Name) {
				flowClass.data.Id = inConcert.Utils.generateGuid();
			}
			//existing name (will be overrided)
			if (flowClass.existingName) {
				flowClass.data.Id = flowClass.existingName;
				overrideClasses += (overrideClasses? ", " : "") + flowClass.data.Name;
				overrideClasses_d[flowClass.data.Id] = true;
			}
			//add class
			self.data.Classes[flowClass.data.Id] = true;
			classes.push(flowClass.data);
		});
		
		function doImport(override) {
			var classesToUpdate = {};
			$.each(classes, function(_, classData) {
				if (!classData.System && (!overrideClasses_d[classData.Id] || override)) {
					classesToUpdate[classData.Id] = classData;
				}
			});
			if ($.isEmptyObject(classesToUpdate)) {
				return callback();
			}
			self.__process(true);
			var flowAPI = inConcert.app.getFlowDesignerAPI();
			flowAPI.saveClasses(classesToUpdate, function(success, error) {
				self.__process(false);
				if (!success) {
					var message;
					switch(error && error.result) {
						case "FORBIDDEN":
							message = "<%=_T('Unable to save process flow classes. You dont have permission.')%>";
							break;
						default:
							message = "<%=_T('Error saving classes.')%>";
							break;
					}
					inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				}
				callback();
			});
		}
		
		if (!overrideClasses) {
			return doImport();
		}
		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			FORMAT("<%=_T('These classes already exists: {0}. Do you want to override them?')%>", overrideClasses),
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				doImport(response.toLowerCase() === "yes");
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopupClasses",
				container_class : "confirmationPopUp popUp260w flowDesignerGlobalModal",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	adm_process_flow_designer.prototype.__migrateDatasources = function(datasources, callback) {
		var self = this;
		if ($.isEmptyObject(datasources)) {
			return callback();
		}
		var keys = Object.keys(datasources);
		var migratedDatasources = {};
		self.__process(true);
		(function migrateDatasource(index) {
			if (index < keys.length) {
				var id = keys[index];
				var datasource = datasources[id];
				self.__migrateDatasource(datasource, function(status, migratedDatasource) {
					if (status) {
						migratedDatasources[id] = migratedDatasource;
						window.setTimeout(function() {
							migrateDatasource(index + 1);
						}, 100);
					}
					else {
						self.__process(false);
						self.Close(true);
					}
				});
			}
			else {
				var flowAPI = inConcert.app.getFlowDesignerAPI();
				flowAPI.saveDatasources(migratedDatasources, function(success, error) {
					self.__process(false);
					if (!success) {
						var message;
						switch(error && error.result) {
							case "FORBIDDEN":
								message = "<%=_T('Unable to save process flow datasources. You dont have permission.')%>";
								break;
							default:
								message = "<%=_T('Error saving datasources.')%>";
								break;
						}
						inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
					}
					callback();
				});
			}
		})(0);
	};
	
	adm_process_flow_designer.prototype.__migrateDatasource = function(datasource, callback) {
		var popup = new inConcert.adm_process_flow_datasources_migration(this, datasource, callback, true /* overlay */);
		popup.Show();
	};
	
	adm_process_flow_designer.prototype.__processDatasourcesMigration = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.data.DatasourcesMigration) {
			return callback();
		}
		var datasourcesMigration = self.data.DatasourcesMigration;
		self.data.DatasourcesMigration = undefined;
		self.data.Datasources = {};
		var datasources = [];
		var overrideDatasources = "";
		var overrideDatasources_d = {};
		$.each(datasourcesMigration, function(_, flowDs) {
			//existing id with different name
			if (flowDs.existingId && flowDs.existingId !== flowDs.data.Name) {
				flowDs.data.Id = inConcert.Utils.generateGuid();
			}
			//existing name (will be overrided)
			if (flowDs.existingName) {
				flowDs.data.Id = flowDs.existingName;
				overrideDatasources += (overrideDatasources? ", " : "") + flowDs.data.Name;
				overrideDatasources_d[flowDs.data.Id] = true;
			}
			//add ds
			self.data.Datasources[flowDs.data.Id] = true;
			datasources.push(flowDs.data);
		});
		
		function doImport(override) {
			var datasourcesToUpdate = {};
			$.each(datasources, function(_, dsData) {
				if (!overrideDatasources_d[dsData.Id] || override) {
					datasourcesToUpdate[dsData.Id] = dsData;
				}
			});
			self.__migrateDatasources(datasourcesToUpdate, callback);
		}
		
		if (!overrideDatasources) {
			return doImport();
		}
		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			FORMAT("<%=_T('These datasources already exists: {0}. Do you want to override them?')%>", overrideDatasources),
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				doImport(response.toLowerCase() === "yes");
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopupDatasources",
				container_class : "confirmationPopUp popUp260w flowDesignerGlobalModal",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_process_flow_designer.prototype.GetTrace = function(callback) {
		callback = callback || $.noop;

		var flowAPI = inConcert.app.getFlowDesignerAPI();

		flowAPI.getTrace({ flowId : this.data.Id },
			function(success, trace) {
				if (!success) {
					var message = "<%=_T('Error loading process execution trace.')%>";
					inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
					callback(false);
					return;
				}
				callback(trace);
			});
	};

	adm_process_flow_designer.prototype.CreateTestSession = function(callback) {
		callback = callback || $.noop;
		var self = this;

		var flowAPI = inConcert.app.getFlowDesignerAPI();
		var sessionId = inConcert.Utils.generateGuid();
		var data = {id: sessionId, processId: this.data.Id};

		flowAPI.createTestSession( data ,
			function(success) {
				if (!success) {
					var message = "<%=_T('Error creating test session.')%>";
					inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
					callback(false);
					return;
				}
				self.SetTestSession(data.id);
				callback(success, data.id);
			});
	};

	adm_process_flow_designer.prototype.CloseTestBot = function(session, callback) {
		callback = callback || $.noop;

		var flowAPI = inConcert.app.getFlowDesignerAPI();
		var data = {id: session, channel: "CHAT"};

		flowAPI.closeTestBot( data );
	};
	
	adm_process_flow_designer.prototype.ShowStateEvent = function(from, to) {
		//console.log('state event');
		this.__showMain();
		this.currentEditor.SelectTransitionByNodes(from, to);
	};
	
	adm_process_flow_designer.prototype.ShowStateActivity = function(stateId, activityId) {
		//console.log('state activity');
		this.ShowState(stateId);
		this.currentEditor.SelectNodeById(activityId);
	};
	
	adm_process_flow_designer.prototype.ShowStateTransition = function(stateId, from, to) {
		//console.log('state transition');
		this.ShowState(stateId);
		this.currentEditor.SelectTransitionByNodes(from, to);
	};

	adm_process_flow_designer.prototype.__classes_editor = function(callback) {
		callback = callback || $.noop;
		if (this.__isProcessing()) {
			return callback(false);
		}
		var popup = new inConcert.adm_process_flow_classes_popup(this, this.data);
		popup.Show(function() {
			callback(true);
		});
	};

	adm_process_flow_designer.prototype.ShowClassesEditor = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.__isProcessing()) {
			return callback(false);
		}
		self.__process(true);
		self.classesEditor = self.classesEditor || new inConcert.adm_process_flow_classes_editor(self);
		$("#flowDesignerContainer").hide();
		self.__exitDesignerFullScreenMode();
		self.classesEditor.Show(
			function() {
				self.__process(false);
				callback(true);
			},
			function() {
				$("#flowDesignerContainer").show();
			}
		);
	};
	
	adm_process_flow_designer.prototype.AddClasses = function(classes) {
		var self = this;
		$.each(classes, function(_, classId) {
			self.data.Classes[classId] = true;
		});
	};
	
	adm_process_flow_designer.prototype.RemoveClass = function(classId) {
		var self = this;
		var oldClasses = self.data.Classes;
		self.data.Classes = {};
		$.each(oldClasses, function(selectedClassId) {
			if (selectedClassId !== classId) {
				self.data.Classes[selectedClassId] = true;
			}
		});
		if (self.classes) {
			oldClasses = self.classes;
			self.classes = {};
			$.each(oldClasses, function(selectedClassId, classData) {
				if (selectedClassId !== classId) {
					self.classes[selectedClassId] = classData;
				}
			});
		}
	};

	adm_process_flow_designer.prototype.GetClasses = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (this.classes) {
			return callback(this.classes);
		}
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getClasses(function(success, classes) {
			if (success) {
				self.classes = classes;
			}
			else {
				var message = "<%=_T('Error loading classes.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(self.classes || {});
		});
	};

	adm_process_flow_designer.prototype.SetClasses = function(classes) {
		var self = this;
		var current = self.classes || {};
		self.classes = classes;
		$.each(current, function(id, classData) {
			if (!self.classes[id] && classData.System) {
				self.classes[id] = classData;
			}
		});
	};

	adm_process_flow_designer.prototype.__datasources_editor = function(callback) {
		callback = callback || $.noop;
		if (this.__isProcessing()) {
			return callback(false);
		}
		var popup = new inConcert.adm_process_flow_datasources_popup(this, this.data);
		popup.Show(function() {
			callback(true);
		});
	};

	adm_process_flow_designer.prototype.ShowDatasourcesEditor = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.__isProcessing()) {
			return callback(false);
		}
		self.__process(true);
		self.datasourcesEditor = self.datasourcesEditor || new inConcert.adm_process_flow_datasources_editor(self);
		$("#flowDesignerContainer").hide();
		self.__exitDesignerFullScreenMode();
		self.datasourcesEditor.Show(
			function() {
				self.__process(false);
				callback(true);
			},
			function() {
				$("#flowDesignerContainer").show();
			}
		);
	};
	
	adm_process_flow_designer.prototype.AddDatasources = function(datasources) {
		var self = this;
		$.each(datasources, function(_, datasourceId) {
			self.data.Datasources[datasourceId] = true;
		});
	};
	
	adm_process_flow_designer.prototype.RemoveDatasource = function(datasourceId) {
		var self = this;
		var oldDatasources = self.data.Datasources;
		self.data.Datasources = {};
		$.each(oldDatasources, function(selectedDatasourceId) {
			if (selectedDatasourceId !== datasourceId) {
				self.data.Datasources[selectedDatasourceId] = true;
			}
		});
		if (self.datasources) {
			oldDatasources = self.datasources;
			self.datasources = {};
			$.each(oldDatasources, function(selectedDatasourceId, dsData) {
				if (selectedDatasourceId !== datasourceId) {
					self.datasources[selectedDatasourceId] = dsData;
				}
			});
		}
	};

	adm_process_flow_designer.prototype.GetDatasources = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (this.datasources) {
			return callback(this.datasources);
		}
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getDatasources(function(success, datasources) {
			if (success) {
				self.datasources = datasources;
			}
			else {
				var message = "<%=_T('Error loading datasources.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(self.datasources || {});
		});
	};

	adm_process_flow_designer.prototype.SetDatasources = function(datasources) {
		this.datasources = datasources;
	};

	adm_process_flow_designer.prototype.getAppsNotifications = function(callback) {
		callback = callback || $.noop;

		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getAppsNotifications(function(success, response) {
			callback(success, response || {});
		});
	};
	
	adm_process_flow_designer.prototype.getAppIntegrationProperties = function(app, callback) {
		callback = callback || $.noop;

		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getAppIntegrationProperties(app, function(success, response) {
			callback(success, response || {});
		});
	};

	adm_process_flow_designer.prototype.getHelpDeskApi = function(callback) {
		callback = callback || $.noop;

		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getHelpDeskApi(function(success, response) {
			callback(success, response || {});
		});
	};
	
	adm_process_flow_designer.prototype.LoadTelephonyCodes = function(callback = $.noop) {
		let self = this;
		if (self.telephonyCodes) {
			return callback(true);
		}
		let codesApi = inConcert.app.getInstanceTelephonyCodes();
		codesApi.loadCountries(function(status, countries) {
			if (!status) {
				let message = "<%=_T('Error loading telephony codes.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			countries = status? countries : {};
			let countries_a = Object.keys(countries)
				.map(function(key) {
					let country = countries[key];
					return { 
						id : country.code, 
						name : FORMAT("{0} ({1})", country.name, country.code)
					};
				})
				.sort(function(a, b) {
					return a.name.localeCompare(b.name);
				});
			self.telephonyCodes = {
				api : codesApi,
				countries_a : countries_a,
				areas_a : {}
			};
			callback(true);
		});
	};
	
	adm_process_flow_designer.prototype.GetTelephonyCountryCodes = function() {
		return this.telephonyCodes.countries_a;
	};
	
	adm_process_flow_designer.prototype.GetTelephonyAreaCodes = function(countryCode) {
		if (!countryCode) {
			return [];
		}
		if (!this.telephonyCodes.areas_a[countryCode]) {
			this.telephonyCodes.areas_a[countryCode] = (this.telephonyCodes.api.getAreasByCode(countryCode) || [])
				.map(function(area) {
					return { id : area.code, name : area.name };
				});
		}
		return this.telephonyCodes.areas_a[countryCode];
	};

	adm_process_flow_designer.prototype.LoadTtsAsrEngines = function(callback = $.noop) {
		if (this.ttsAsrEngines) {
			return callback(this.ttsAsrEngines);
		}

		const uri = FORMAT("{0}/ttsasr_engines/list/", inConcert.app.getAdministrationUrl());

		const request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: 300000,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				if (result.toLowerCase() === "success") {
					callback(true, list.ttsasr_engines);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (_, error) {callback(false, error);}
		});
		return request_handler;
	};

	adm_process_flow_designer.prototype.GetFlowsStates = function(callback = $.noop) {
		const flowAPI = inConcert.app.getFlowDesignerAPI();
		const params = {proxyType: this.data.Type};

		flowAPI.loadFlowsByType(params,
			function(success, flows) {
				let flowsStates = [];
				if (!success) {
					callback(flowsStates);
				}
				let sorted_flows = inConcert.Utils.sortArray(flows, "Name");
				sorted_flows.forEach(function(flow) {

					function GetStates(states) {
						let sorted_states = inConcert.Utils.sortArray(states, "name");
						let _states = [];

						$.each(sorted_states, function(_, state) {
							_states.push({
								id : "state_" + state.id,
								name : state.name
							});
						});
						return _states;
					}
					flowsStates.push({
						id : flow.Id,
						name : flow.Name,
						childs : GetStates(flow.Draw.states)
					});
				});
				callback(flowsStates);
			}
		);
	};

	/*
		Catalogo
	*/
	function adm_process_flow_generator(properties) {
		this.__onClose = properties.close || $.noop;
		this.frm = properties.frm;
		
	}

	if (typeof inConcert.adm_process_flow_generator !== "function") {
		inConcert.adm_process_flow_generator = adm_process_flow_generator;
	}

	adm_process_flow_generator.prototype.Show = function() {
		var self = this;
		
		self.$container = self.__make();
		self.frm.$get_dom_element_by_id("container").append(self.$container);
		self.frm.$get_dom_element_by_id("view_container").hide();

		self.frm.__make_generator_draw();
	};


	adm_process_flow_generator.prototype.__make = function() {
		var self = this;

		let $container = $(Mustache.render(templates.generator_base, {
			
		}));
		
		
		$container.find("[close_button]").off("click").on("click", function() {
			self.Close();
		});
		$container.find("[from_scratch]").off("click").on("click",function(){
			self.frm.popup_render({type:"new"});
		});

		return $container;
	};

	adm_process_flow_generator.prototype.Close = function(noEvent) {
		var self = this;

		self.$container.remove();
		self.frm.$get_dom_element_by_id("view_container").show();
		if (!noEvent) {
			self.__onClose();
		}
		
	};
}());
