/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

if (!this.inConcert.adm_process_flow_migrations) {
	this.inConcert.adm_process_flow_migrations = {};
}

(function () {
	//"use strict";

	let id = "sharedFiles";
	const format = inConcert.Utils.format;

	function adm_process_flow_migration(main, data, overlay) {
		adm_process_flow_migration.uber.constructor.call(this, main, data, overlay);
        this.overlay = overlay;
	}

	inConcert.Utils.inherit( adm_process_flow_migration, inConcert.adm_process_flow_migrations.base );

	if (typeof inConcert.adm_process_flow_migrations[id] !== "function") {
		inConcert.adm_process_flow_migrations[id] = adm_process_flow_migration;
	}

	adm_process_flow_migration.prototype.id = id;

	adm_process_flow_migration.prototype.__getName = function() {
		return "<%=_T('Shared Files')%>";
	};

	adm_process_flow_migration.prototype.__checkTask = function(callback) {
		const self = this;
		inConcert.Utils.asyncReduce(self.data,
			function(errors, file, onStepEnd) {
				self.__processFile(errors, file, onStepEnd);
			},
			function(errors) {
				if (!errors.length) {
					callback();
				}
				else {
					let fileNames = inConcert.Utils.arrayToText(errors, function(file) { return file.Name; });
					callback(false, format("<%=_T('Error importing the shared files: {0}. Must create or import them.')%>", fileNames));
				}
			}, []
		);
	};

	adm_process_flow_migration.prototype.__processFile = function(errors, sharedFile, onStepEnd) {
		const self = this;
		const sharedFilesAPI = inConcert.app.getSharedFilesAPI();
		sharedFilesAPI.getFile({fileId: sharedFile.Id}, function(exist) {
			// exist en true, si el archivo shared file ya existe en la BD, entonces no tengo que crearlo.
			if (exist) {
				// el archivo ya existe, is ok, no hago nada
				onStepEnd(errors);
				return;
			}
			// ahora procedo a crear el archvio en shared files y en el repo
			let respositoryFileInfo = sharedFile.RespositoryFileInfo;
			delete sharedFile.RespositoryFileInfo;
			// creo el archivo shared file en la BD
			sharedFilesAPI.saveFile(sharedFile, function(success, err) {
				if (success) {
					if (respositoryFileInfo) {
						// creo el archivo en el repositorio
						self.__processRepositoryFile(respositoryFileInfo, function(success, err) {
							if (!success) {
								console.warn("Error creating repository file. " + respositoryFileInfo.id);
								errors.push(sharedFile);
							}
							onStepEnd(errors);
						});
					}
					else {
						console.warn("Flow malformed. Missing attribute 'RepositoryFileInfo'.");
						onStepEnd(errors);
					}
				}
				else {
					// error al crear el archivo en el repo.
					console.warn("Error creating shared file. " + sharedFile.Name);
					errors.push(sharedFile);
					onStepEnd(errors);
				}
			});
		});
	};

}());
