/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W014, esversion: 11 */

/* global $, inConcert, Mustache, EmojiPicker */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const format = inConcert.Utils.format;
	const templates = inConcert.templates.process_flows;

	function adm_process_flow_popup() {
		this.m_timeout_handle_hide_message = null;
	}

	if (typeof inConcert.adm_process_flow_popup !== "function") {
		inConcert.adm_process_flow_popup = adm_process_flow_popup;
	}

	adm_process_flow_popup.prototype.__getPopupId = function() {
		return "flowDesignerPopup";
	};

	adm_process_flow_popup.prototype.__getClassName = function() {
		return "";
	};

	adm_process_flow_popup.prototype.__isFullScreen = function() {
		return false;
	};
	
	adm_process_flow_popup.prototype.__getTitle = function() {
		return "";
	};

	adm_process_flow_popup.prototype.__getSubtitle = function() {
		return "";
	};


	adm_process_flow_popup.prototype.__makeContent = function() {
		return "";
	};

	adm_process_flow_popup.prototype.__getAndValidate = function() {
		return {};
	};

	adm_process_flow_popup.prototype.__getAndValidateAsync = function(data, callback) {
		callback(data);
	};

	adm_process_flow_popup.prototype.__save = function(_, callback) {
		callback(true);
	};

	adm_process_flow_popup.prototype.__updateDeclaredVariablesList = function(data) {
		const self = this;
		if (!self.editor || !self.editor.main) {
			return;
		}
		let oldResulValue = self.editor.main.lastActivityResultValue;
		if (data.result !== oldResulValue) {
			self.editor.main.declaredVariables.delete(oldResulValue);
			self.editor.main.declaredVariables.add(data.result);
		}
	};

	adm_process_flow_popup.prototype.__afterSave = $.noop;

	adm_process_flow_popup.prototype.__cancel = function(callback) {
		callback();
	};

	adm_process_flow_popup.prototype.__hasKeyboardEvents = function() {
		return true;
	};

	adm_process_flow_popup.prototype.__disableEnableControl = function(properties, disabled, $container) {
		($container || this.$container()).find(format("[control={0}]", properties.id)).prop("disabled", !!disabled);
		($container || this.$container()).find(format("[control_label={0}]", properties.id))
			.toggleClass("formLabel-ds", !!disabled);
	};

	adm_process_flow_popup.prototype.__readonlyControl = function(properties, readonly, $container) {
		this.__disableEnableControl(properties, readonly, $container);
		($container || this.$container()).find(format("[control_label={0}]", properties.id))
			.toggleClass("readonly", !!readonly);
	};

	adm_process_flow_popup.prototype.__makeControl = function(properties, value) {
		var self = this;
		properties.with_label = !properties.without_label;
		properties.with_dots = !properties.inverse_order;
		properties.extended = !properties.not_extended;
		properties.enabled_disabled_description = properties.enabled_disabled &&
			properties.enabled_disabled.description || "";
		properties.controlFieldClass = properties.fieldClass? ("flowDesignerControl" + properties.fieldClass) : null;
		if (!properties.checkBoxClass) properties.checkBoxClass = "formCheckbox";
		if (!properties.variableIconClass) properties.variableIconClass = "formFieldVariableIcon";	
		var $control = $(Mustache.render(templates.popup_control, properties));
		if (properties.with_label && properties.inverse_order) {
			$control.find("[control]").after($control.find("[control_label]").detach());
		}
		if (properties.enabled_disabled) {
			$control.find("[control_enabled_disabled]")
				.prop("checked", value !== properties.enabled_disabled.value)
				.change(function() {
					var $control = $(this).parent().parent().parent().parent().parent();
					var enabled = self.__isEnabledDisabledControlActive(properties, $control);
					self.__readonlyControl(properties, !enabled, $control, true);
					if (enabled) {
						if (self.__getValue(properties, $control) === properties.enabled_disabled.text) {
							self.__setValue(properties, "", $control);
						}
					}
					else {
						self.__setValue(properties, properties.enabled_disabled.text || "", $control);
					}
					if (properties.enabled_disabled.event) {
						properties.enabled_disabled.event();
					}
				})
				.change();
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__valueOrDefault = function(properties, value) {
		if (inConcert.Utils.isNothing(value) && !inConcert.Utils.isNothing(properties.defaultValue)) {
			return properties.defaultValue;
		}
		else {
			return value;
		}
	};
	
	let childList = [];

	adm_process_flow_popup.prototype.__makeChildren = function(properties, $result) {
		if (!properties.children) return;
		// const children = properties.children;
		$.each(childList, function(_, child) {
			$result.find("[parent]").append(child);
		}
		);
		return $result;
	};


	adm_process_flow_popup.prototype.__makeDiv = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.div_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.div_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);

		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeLabel = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.label_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.label_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);
		}
			
		return $resultWithChildren || $result;
	};
	
	adm_process_flow_popup.prototype.__makeSpan = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.span_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.span_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);
		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeIcon = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.icon_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.icon_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);
		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeButton = function(properties) { 
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.button_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.button_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);
		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeButtonIconAndText = function(properties) { 
		properties.guid = inConcert.Utils.generateGuid();
		const $result = Mustache.render(templates.button_icon_and_text_popup, properties);

		return $result;
	};

	adm_process_flow_popup.prototype.__makeSelectList = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.select_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.select_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);

		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeInput2 = function(properties) {
		properties.guid = inConcert.Utils.generateGuid();
		let $result;
		let $resultWithChildren;
		if (properties.isChildren) {
			$result = Mustache.render(templates.input_popup, properties);
			childList.push($result);
		}
		else {
			$result = $(Mustache.render(templates.input_popup, properties));
			$resultWithChildren = this.__makeChildren(properties, $result);

		}
			
		return $resultWithChildren || $result;
	};

	adm_process_flow_popup.prototype.__makeInput = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_text, properties);
		properties.control_type = "input";
		var $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeInputFlex = function(properties, value) { 
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_text_flex, properties);
		properties.control_type = "input";
		let $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeButtonTextIconFlex = function(properties, value) { 
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.button_icon_and_text_popup, properties);
		properties.control_type = "input";
		let $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeSeparator = function(properties) { 
		return Mustache.render(templates.popup_separator, properties);
	};

	adm_process_flow_popup.prototype.__makeScoringSlider = function() {
		const $slider = $(Mustache.render(templates.dialer_batch_scoring_slider, {
			guid : this.guid
		}));

		return $slider;
	};

	adm_process_flow_popup.prototype.__makePassword = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_password, properties);
		properties.control_type = "password";
		var $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeTextarea = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_textarea, properties);
		properties.control_type = "textarea";
		var $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
	};

    adm_process_flow_popup.prototype.__makeTextareaFlex = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_textarea_flex, properties);
		properties.control_type = "textarea";

		let $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		return $control;
    };
	adm_process_flow_popup.prototype.__discoverEmojiPickers = function($container) {

        if (!$container) {
            if (this.emojiPickerEmbeed) {
                return;
            }
            this.emojiPickerEmbeed = new EmojiPicker({
                emojiable_selector: ".emojipickerTextarea",
                assetsPath: "/inconcert/lib/emoji-picker/lib/img/",
                popupButtonClasses: "buttonColorTransparent emoji-picker-button flowDesignerControlEmojiPicker",
                norealTime: false,
                onPaste: function(pasted) {
                    return pasted;
                }
            });
            this.emojiPickerEmbeed.discover();
        } else {

            if ($container && $container.find(".emoji-wysiwyg-editor").length > 0) {
                return;
            }
            /*if (this.emojiPicker) {
                return;
            }*/

            var container_id = $($container.find(".textareaControl")[0]).attr("id");

            this.emojiPicker = new EmojiPicker({
                emojiable_selector: "#" + container_id + " .emojipickerTextarea",
                assetsPath: "/inconcert/lib/emoji-picker/lib/img/",
                popupButtonClasses: "buttonColorTransparent emoji-picker-button flowDesignerControlEmojiPicker",
                norealTime: false,
                onPaste: function(pasted) {
                    return pasted;
                }
            });
            this.emojiPicker.discover();
        }
        
	};
	

	adm_process_flow_popup.prototype.__makeCheckbox = function(properties, value, checkboxList) {
		value = this.__valueOrDefault(properties, value);
		var $control;
		properties.guid = inConcert.Utils.generateGuid();
		if (checkboxList) {
			$control = $(Mustache.render(templates.popup_control_check_item, properties));
		}
		else {
			properties.control = Mustache.render(templates.popup_control_check, properties);
			properties.control_type = "checkbox";
			$control = this.__makeControl(properties, value);
		}
		if (properties.change) {
			$control.find("[control]").change(properties.change);
		}
		if (value) {
			$control.find("[control]").prop("checked", !!value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeCheckboxFlex = function(properties, value, checkboxList) { 
		value = this.__valueOrDefault(properties, value);
		let $control;
		properties.guid = inConcert.Utils.generateGuid();
		if (checkboxList) {
			$control = $(Mustache.render(templates.popup_control_check_item_flex, properties));
		}
		else {
			properties.control = Mustache.render(templates.popup_control_check, properties);
			properties.control_type = "checkbox";
			$control = this.__makeControl(properties, value);
		}
		if (properties.change) {
			$control.find("[control]").change(properties.change);
		}
		if (value) {
			$control.find("[control]").prop("checked", !!value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeCheckboxList = function(properties, values) {
		var self = this;
		values = values || {};
		properties.guid = inConcert.Utils.generateGuid();
		var $control = $(Mustache.render(templates.popup_control_checklist, properties));
		var $list = $control.find("[checklist]");
		if (properties.options) {
			var options = JSON.parse(JSON.stringify(properties.options));
			if (!$.isArray(options)) {
				var options_d = options;
				options = [];
				$.each(options_d, function(id, data) {
					if (typeof data === "string") {
						data = { name : data };
					}
					data.id = id;
					options.push(data);
				});
			}
			if (properties.sort) {
				options.sort(properties.sort !== true? properties.sort : function(a, b) {
					return a.name.localeCompare(b.name);
				});
			}
			$.each(options, function(_, option) {
				option = properties.map? properties.map(option) : option;
				if (typeof option === "string") {
					option = { id : option, name : option };
				}
				var checked = !!values[option.id];
				option.id = properties.id + "_" + option.id;
				$list.append(self.__makeCheckbox(option, checked, true));
			});
		} 
		if (properties.selectAll) {
			// $control.find("[select_all]").change(function() {
			// 	self.$get(format("[control={0}]", $(this).attr("select_all")))
			// 		.find("input[type='checkbox']")
			// 		.prop("checked", $(this).is(":checked"));
			// });
			$control.find("[select_all]").change(function() {
				self.$get("[checkbox_item]")
					.prop("checked", $(this).is(":checked"));
			});
			$control.find("[control]").change(function() {
				if (!$(this).is(":checked")) {
					var id = $(this).attr("control");
					var id_a = id.split("_");
					id = id.replace("_" + id_a[id_a.length - 1]);
					self.$get(format("[select_all={0}]", id)).prop("checked", false);
				}
			});
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeRadio = function(properties, value, radioList) {
		value = this.__valueOrDefault(properties, value);
		var $control;
		properties.guid = inConcert.Utils.generateGuid();
		if (radioList) {
			$control = $(Mustache.render(templates.popup_control_radiobutton_item, properties));
		}
		else {
			properties.control = Mustache.render(templates.popup_control_radiobutton, properties);
			properties.control_type = "radiobutton";
			$control = this.__makeControl(properties, value);
		}
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.prop("checked", !!value);
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeRadioButtonList = function(properties, value) {
		var self = this;
		value = this.__valueOrDefault(properties, value);
		properties.guid = inConcert.Utils.generateGuid();
		var $control = $(Mustache.render(templates.popup_control_radiobuttonlist, properties));
		var $list = $control.find("[radiobuttonlist]");
		if (properties.options) {
			var options = JSON.parse(JSON.stringify(properties.options));
			if (!$.isArray(options)) {
				var options_d = options;
				options = [];
				$.each(options_d, function(id, data) {
					if (typeof data === "string") {
						data = { name : data };
					}
					data.id = id;
					options.push(data);
				});
			}
			if (properties.sort) {
				options.sort(properties.sort !== true? properties.sort : function(a, b) {
					return a.name.localeCompare(b.name);
				});
			}
			$.each(options, function(_, option) {
				option = properties.map? properties.map(option) : option;
				if (typeof option === "string") {
					option = { id : option, name : option };
				}
				var checked = option.id === value;
				option.group = properties.id;
				option.id = properties.id + "_" + option.id;
				$list.append(self.__makeRadio(option, checked, true));
			});
		}
		return $control;
	};

	adm_process_flow_popup.prototype.__makeBooleanRadioButton = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		properties.options = [
			{ id : "true", name : properties.checkedName },
			{ id : "false", name : properties.uncheckedName }
		];
		return this.__makeRadioButtonList(properties, value? "true" : "false");
	};

	adm_process_flow_popup.prototype.__makeCombo = function(properties, value) {
		value = this.__valueOrDefault(properties, value);
		var options_s = "";
		if (!properties.required) {
			options_s += format("<option value='{0}'>{1}</option>",
				properties.emptyValue || "", properties.emptyLabel || "");
		}
		var optionsDisclaimer = false;
		if (properties.options) {
			var options = JSON.parse(JSON.stringify(properties.options));
			if (!$.isArray(options)) {
				var options_d = options;
				options = [];
				$.each(options_d, function(id, data) {
					if (typeof data === "string") {
						data = { name : data };
					}
					data.id = id;
					options.push(data);
				});
			}
			if (properties.sort) {
				options.sort(properties.sort !== true? properties.sort : function(a, b) {
					return a.name.localeCompare(b.name);
				});
			}
			$.each(options, function(_, option) {
				option = properties.map? properties.map(option) : option;
				if (typeof option === "string") {
					option = { id : option, name : option };
				}
				options_s += format("<option value='{0}' title='{1}' disclaimer='{3}'>{2}</option>",
					option.id, option.description || "", option.name || option.id, option.disclaimer || "");
				optionsDisclaimer = optionsDisclaimer || !!option.disclaimer;
			});
		}
		properties.options_list = options_s;
		properties.guid = inConcert.Utils.generateGuid();
		if (optionsDisclaimer && !properties.disclaimer) {
			properties.disclaimer = true;
		}
		properties.control = Mustache.render(templates.popup_control_combo, properties);
		properties.options_list = null;
		properties.control_type = "combo";
		var $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		if (optionsDisclaimer) {
			var loadDisclaimer = function($control) {
				var disclaimer = $control.find(":selected").attr("disclaimer") || "";
				if (!$control.val() && properties.disclaimer !== true) {
					disclaimer = properties.disclaimer;
				}
				$control.siblings("[disclaimer]")
					.html(disclaimer.replace(/\n/g, "<br>"))
					.toggle(!!disclaimer);
			};

			loadDisclaimer($control.find("[control]").change(function() {
				loadDisclaimer($(this));
			}));
		}

		return $control;
	};

	adm_process_flow_popup.prototype.__makeComboFlex = function(properties, value) { 
		value = this.__valueOrDefault(properties, value);
		let options_s = "";
		if (!properties.required) {
			options_s += format("<option value='{0}'>{1}</option>", 
				properties.emptyValue || "", properties.emptyLabel || "");
		}
		let optionsDisclaimer = false;
		if (properties.options) {
			let options = JSON.parse(JSON.stringify(properties.options));
			if (!$.isArray(options)) {
				let options_d = options;
				options = [];
				$.each(options_d, function(id, data) {
					if (typeof data === "string") {
						data = { name : data };
					}
					data.id = id;
					options.push(data);
				});
			}
			if (properties.sort) {
				options.sort(properties.sort !== true? properties.sort : function(a, b) {
					return a.name.localeCompare(b.name);
				});
			}
			$.each(options, function(_, option) {
				option = properties.map? properties.map(option) : option;
				if (typeof option === "string") {
					option = { id : option, name : option };
				}
				options_s += format("<option value='{0}' title='{1}' disclaimer='{3}'>{2}</option>", 
					option.id, option.description || "", option.name || option.id, option.disclaimer || "");
				optionsDisclaimer = optionsDisclaimer || !!option.disclaimer;
			});
		}
		properties.options_list = options_s;
		properties.guid = inConcert.Utils.generateGuid();
		if (optionsDisclaimer && !properties.disclaimer) {
			properties.disclaimer = true;
		}
		if (!properties.comboClass) properties.comboClass = "formCheckbox";	
		properties.control = Mustache.render(templates.popup_control_combo_flex, properties);
		properties.options_list = null;
		properties.control_type = "combo";
		let $control = this.__makeControl(properties, value);
		if (properties.change) {
			$control.find("[control]")
				.change(properties.change);
		}
		if (value) {
			$control.find("[control]")
				.val(value);
		}
		if (optionsDisclaimer) {
			let loadDisclaimer = function($control) {
				let disclaimer = $control.find(":selected").attr("disclaimer") || "";
				if (!$control.val() && properties.disclaimer !== true) {
					disclaimer = properties.disclaimer;
				}
				$control.siblings("[disclaimer]")
					.html(disclaimer.replace(/\n/g, "<br>"))
					.toggle(!!disclaimer);
			};

			loadDisclaimer($control.find("[control]").change(function() {
				loadDisclaimer($(this));
			}));
		}

		return $control;
	};

	adm_process_flow_popup.prototype.__getPhoneDigitsOptions = function() {
		return [ "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "#" ];
	};

	adm_process_flow_popup.prototype.__getRange = function(min, max) {
		let arreglo = [];

		if (min === max) {
			arreglo.push(min.toString());
		} else {
			for (let index = min; index <= max; index++) {
				arreglo.push(index.toString());			
			}
		}
		
		return arreglo;
	};

	adm_process_flow_popup.prototype.__getGenderOptions = function() {
		return [ "female", "male" ];
	};

	adm_process_flow_popup.prototype.__getProsodyVolumeOptions = function() {
		return [ "silent", "x-soft", "soft", "medium", "load", "x-loud" ];
	};

	adm_process_flow_popup.prototype.__getProsodyRateOptions = function() {
		return [ "x-slow", "slow", "medium", "fast", "x-fast" ];
	};

	adm_process_flow_popup.prototype.__getBargeInOptions = function() {
		return [ {id : 0, name : "not allowed"}, {id:1, name: "allowed"} ];
	};

	adm_process_flow_popup.prototype.__getRecognitionModeOptions = function() {
		return [ "normal", "hotword" ];
	};

	adm_process_flow_popup.prototype.__getURIencodedResultsOptions = function() {
		return [ {id : 0, name : "not encode"}, {id:1, name: "encode"} ];
	};

	adm_process_flow_popup.prototype.__getStartInputTimersOptions = function() {
		return [ {id : 0, name : "no"}, {id:1, name: "yes (start with recognize)"}, {id: 2, name: "auto (start when prompt is finished)"} ];
	};

	adm_process_flow_popup.prototype.__getControl = function(properties, $container) {
		return ($container || this.$container()).find(format("[control={0}]", properties.id));
	};

	adm_process_flow_popup.prototype.__getControlContainer = function(properties, $container) {
		return this.__getControl(properties, $container).parent();
	};

	adm_process_flow_popup.prototype.__getControlLabel = function(properties, $container) {
		return this.__getControl(properties, $container).siblings("[control_label]");
	};

	adm_process_flow_popup.prototype.__getControlGuid = function(properties, $container) {
		return this.__getControl(properties, $container)[0].id;
	};

	adm_process_flow_popup.prototype.__getControlContainerGuid = function(properties, $container) {
		return this.__getControlContainer(properties, $container)[0].id;
	};

	adm_process_flow_popup.prototype.__replaceControl = function(properties, $control, $container) {
		var $currentControl = this.__getControl(properties, $container);
		$currentControl.parent().replaceWith($control);
	};

	adm_process_flow_popup.prototype.__toggleControl = function(properties, show, $container) {
		var $control = this.__getControl(properties, $container).parent().parent();
		$control
			.toggle(!!show)
			.attr("hidden_control", show? "false" : "true");
		var $row = $control.parent();
		if ($row.is("[form_row]")) {
			$row.toggle($row.children("[hidden_control!=true]").length > 0);
		}
	};

	adm_process_flow_popup.prototype.__showControl = function(properties, $container) {
		this.__toggleControl(properties, true, $container);
	};

	adm_process_flow_popup.prototype.__hideControl = function(properties, $container) {
		this.__toggleControl(properties, false, $container);
	};

	adm_process_flow_popup.prototype.__toggleControlRow = function(properties, show, $container) {
		this.__getControl(properties, $container).parent().parent().parent().parent().toggle(!!show);
	};

	adm_process_flow_popup.prototype.__showControlRow = function(properties, $container) {
		this.__toggleControlRow(properties, true, $container);
	};

	adm_process_flow_popup.prototype.__hideControlRow = function(properties, $container) {
		this.__toggleControlRow(properties, false, $container);
	};

	adm_process_flow_popup.prototype.__setControlRequired = function(properties, required, $container) {
		properties.required = !!required;
		this.__getControl(properties, $container).siblings("[control_label]")
			.toggleClass("formLabel-rq", properties.required);
	};

	adm_process_flow_popup.prototype.__getControlEnabledDisabled = function(properties, $container) {
		return this.__getControl(properties, $container).parent().parent().find("[control_enabled_disabled]");
	};

	adm_process_flow_popup.prototype.__isEnabledDisabledControlActive = function(properties, $container) {
        var $enabledDisabled = this.__getControlEnabledDisabled(properties, $container);
		return $enabledDisabled.attr("control_enabled_disabled") !== "true" || $enabledDisabled.is(":checked");
	};

	adm_process_flow_popup.prototype.__toggleEnabledDisabledControl = function(properties, show, $container) {
		this.__getControlEnabledDisabled(properties, $container)
			.toggle(show)
			.attr("control_enabled_disabled", show? "true" : "false")
			.change();
	};

	adm_process_flow_popup.prototype.__showEnabledDisabledControl = function(properties, $container) {
		this.__toggleEnabledDisabledControl(properties, true, $container);
	};

	adm_process_flow_popup.prototype.__hideEnabledDisabledControl = function(properties, $container) {
		this.__toggleEnabledDisabledControl(properties, false, $container);
	};

	adm_process_flow_popup.prototype.__makeUpload = function (properties, upload) {
		var self = this;
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_upload, properties);
		properties.control_type = "uploader";
		var $control = this.__makeControl(properties);
		$control.find("[upload_button]").click(function () {
			self.$get(format("[upload_input={0}]", $(this).attr("upload_button")))[0].click();
		});
		$control.find("[upload_input]").change(function () {
			self.__clearErrors();
			var file = event.target.files[0];
			if (!file) {
				return;
			}
			var validation = properties.validate || {};
			var extension = inConcert.Utils.getFileExtension(file.name) || "";
			if (validation.extension && !validation.extension.types[extension]) {
				self.__showError({
					control: properties.id,
					message: validation.extension.message || "<%=_T('Unsuported extension.')%>"
				});
				return;
			}
			var maxMb = validation.size || 5;
			if (file.size > maxMb * 1024 * 1024) {
				self.__showError({
					control: properties.id,
					message: format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", maxMb)
				});
				return;
			}
			self.$get(format("[control={0}]", properties.id)).val(file.name);

			upload(file,
				function () {
					self.$get(format("[upload_percentage={0}]", properties.id)).text("0" + "%").show();
				},
				function (percentage) {
					self.$get(format("[upload_percentage={0}]", properties.id)).text(percentage);
				},
				function (status, error) {
					self.$get(format("[upload_percentage={0}]", properties.id)).val("");
					self.$get(format("[upload_percentage={0}]", properties.id)).text("").hide();
					if (!status) {
						self.__showError({
							control: properties.id,
							message: error
						});
					}
				}
			);
		});

		return $control;
	};

    adm_process_flow_popup.prototype.__makeUploadGeo = function(properties, upload) {
     let self = this;
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.popup_control_upload_geo, properties);
		properties.control_type = "uploader";
		let $control = this.__makeControl(properties);
		$control.find("[upload_button]").click(function() {
			self.$get(format("[upload_input={0}]", $(this).attr("upload_button")))[0].click();
		});
		$control.find("[upload_input]").change(function() {
			self.__clearErrors();
			let file = event.target.files[0];
			if ( !file ) {
				return;
			}
			let validation = properties.validate || {};
			let extension = inConcert.Utils.getFileExtension( file.name ) || "";
			if (validation.extension && !validation.extension.types[extension]) {
				self.__showError({
					control : properties.id,
					message : validation.extension.message || "<%=_T('Unsuported extension.')%>"
				});
				return;
			}
			let maxMb = validation.size || 5;
			if ( file.size > maxMb * 1024 * 1024 ) {
				self.__showError({
					control : properties.id,
					message : format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", maxMb)
				});
				return;
			}
			self.$get(format("[control={0}]", properties.id)).val(file.name);

			upload(file,
				function() {
					self.$get(format("[upload_percentage={0}]", properties.id)).text("0" + "%").show();
				},
				function(percentage) {
					self.$get(format("[upload_percentage={0}]", properties.id)).text(percentage);
				},
				function(status, error) {
					self.$get(format("[upload_percentage={0}]", properties.id)).val("");
					self.$get(format("[upload_percentage={0}]", properties.id)).text( "" ).hide();
					if (!status) {
						self.__showError({
							control : properties.id,
							message : error
						});
					}
				}
			);
		});

		return $control;
								
	};	

	adm_process_flow_popup.prototype.__makeDisclaimerFlex = function($controls, properties) {
		var $row = $(Mustache.render(templates.popup_disclaimer_flex, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeHiddenDivFlex = function($controls, properties) { 
		properties = properties || {};
		properties.hidden = true;
		return this.__makeDivFlex($controls, properties);
	};

	adm_process_flow_popup.prototype.__makeRemoveButton = function($controls, properties) {
		var $button = $(Mustache.render(templates.popup_remove_button, properties || {}));
		$button.append($controls);
		return $button;
	};

	adm_process_flow_popup.prototype.__makeDivFlex = function($controls, properties) {
		var $row = $(Mustache.render(templates.popup_div_flex, properties || {}));
		$row.append($controls);
		return $row;
	};
	adm_process_flow_popup.prototype.__makeRow = function($controls, properties) {
		var $row = $(Mustache.render(templates.popup_row, properties || {}));
		$row.append($controls);
		return $row;
	};
	adm_process_flow_popup.prototype.__makeRowFlex = function($controls, properties) {
		// if(!properties) properties = { class: "row" };
		if(!properties) properties = { class: "col-12" };
		let $row = $(Mustache.render(templates.popup_row_flex, properties || {}));
		$row.append($row.find("[form_row]").append($controls));
		return $row;
	};

    adm_process_flow_popup.prototype.__makeRowFlex2 = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_flex2, properties || {}));
		$row.append($controls);
		return $row;
    };
    
	adm_process_flow_popup.prototype.__makeRowMailEditor = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_mail_editor, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRow50 = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_50, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRow49 = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_49, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRow33 = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_33, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRow34 = function($controls, properties) { 
		let $row = $(Mustache.render(templates.popup_row_34, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRowGroup = function($controls, properties) {
		let $row = $(Mustache.render(templates.popup_row_group, properties || {}));
		$row.append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeRowGroupCheckBoxs = function($controls, properties) {
		let $row = $(Mustache.render(templates.popup_row_group_checkboxs, properties || {}));
		$row.children("[ic-role='form-row']").append($controls);
		return $row;
	};

	adm_process_flow_popup.prototype.__makeHiddenRow = function($controls, properties) {
		properties = properties || {};
		properties.hidden = true;
		return this.__makeRow($controls, properties);
	};

	adm_process_flow_popup.prototype.__makeHiddenRowFlex = function($controls, properties) { 
		properties = properties || {};
		properties.hidden = true;
		return this.__makeRowFlex($controls, properties);
	};

	adm_process_flow_popup.prototype.__makeSubtitle = function(id, name, description) {
		return $(Mustache.render(templates.popup_form_subtitle, {
			id : id,
			name : name,
			description : description
		}));
	};

	adm_process_flow_popup.prototype.__getChecklistValue = function(id, $control) {
		var values = {};
		$control.find("input[type=checkbox]").each(function() {
			if ($(this).is(":checked") && ($(this)[0].name !== "select_all")) {
				values[$(this).attr("control").replace(id + "_", "")] = true;
			}
		});
		return values;
	};

	adm_process_flow_popup.prototype.__setChecklistValue = function(id, values, $control) {
		$control.find("input[type=checkbox]").each(function() {
			$(this).prop("checked", !!values[$(this).attr("control").replace(id + "_", "")]);
		});
	};

	adm_process_flow_popup.prototype.__getRadiobuttonListValue = function(id, $control) {
		let $selected = $control.find("input[type=radio]:checked");
		if ($selected.length) {
			let selected = $selected.attr("control").replace(id + "_", "");
			switch(selected) {
				case "true":
					return true;
				case "false":
					return false;
				default:
					return selected;
			}
		}
		else {
			return null;
		}
	};

	adm_process_flow_popup.prototype.__getComboLabel = function(properties, $container) {
		let $control = this.__getControl(properties, $container);
		return $control.find("option:selected").text() || "";
	};

	adm_process_flow_popup.prototype.__setEmojiTextArea = function (properties, $container, val) {
		var $control = this.__getControl(properties, $container);
		$control.siblings(".emoji-wysiwyg-editor").html(val);
	};

	adm_process_flow_popup.prototype.__getEmojiHtmlTextArea = function (properties, $container) {
		var $control = this.__getControl(properties, $container);
		return $control.siblings(".emoji-wysiwyg-editor").html();
	};

	adm_process_flow_popup.prototype.__getValue = function(properties, $container) {
		var $control = this.__getControl(properties, $container);
		var value;
		if (properties.enabled_disabled && !this.__isEnabledDisabledControlActive(properties, $container)) {
			return properties.enabled_disabled.value;
		}
		if ($control.is("input[type=checkbox]") || $control.is("input[type=radio]")) {
			value = $control.is(":checked");
		}
		else if ($control.is("[checklist]")) {
			value = this.__getChecklistValue(properties.id, $control);
		}
		else if ($control.is("[radiobuttonlist]")) {
			value = this.__getRadiobuttonListValue(properties.id, $control);
		}
		else {
			value = (properties.hyperlinks && properties.emojipicker)? $control.siblings(".emoji-wysiwyg-editor").html() :$control.val();
		}
		if (properties.emptyValue && properties.emptyValue === value) {
			return null;
		}
		return value;
	};

	adm_process_flow_popup.prototype.__setValue = function(properties, value, $container) {
		var $control = this.__getControl(properties, $container);
		if ($control.is("input[type=checkbox]")) {
			$control.prop("checked", !!value);
		}
		else if ($control.is("[checklist]")) {
			this.__setChecklistValue(properties.id, value, $control);
		}
		else {
			$control.val(value);
		}
		return value;
	};

	adm_process_flow_popup.prototype.__addValue = function(data, properties, $container) {
		if (data.error) {
			return;
		}
		var value = this.__getValue(properties, $container);
		if ((!value || $.isEmptyObject(value)) && properties.required) {
			data.error = {
				$container : $container,
				control : properties.id,
				message : (typeof properties.required === "string")? properties.required :
					format("<%=_T('Field {0} is required.')%>", properties.name || properties.id)
			};
			return;
		}
		if (properties.validate) {
			var error;
			//object
			if ($.isPlainObject(properties.validate)) {
				//first method
				if (properties.validate.method) {
					error = properties.validate.method(value);
				}
				//then types
				if (!error) {
					var types = properties.validate.types || [properties.validate.type];
					var message = properties.validate.message;
					$.each(types, function(_, type) {
						if (!type) {
							return;
						}
						if ($.isPlainObject(type)) {
							message = type.message;
							type = type.type;
						}
						if (!inConcert.Utils.isValid(value, type)) {
							error = message;
							return false;
						}
					});
				}
				//then ranges
				if (!error && properties.validate.range) {
					var range = properties.validate.range;
					var value_n = range.type === "integer"? parseInt(value) : parseFloat(value);
					if (isNaN(value_n)) {
						if (!range.allowVariables) {
							error = format(range.type === "integer"?
								"<%=_T('{0} value must be an integer.')%>" : "<%=_T('{0} value must be a number.')%>",
								properties.name);
						}
					}
					else {
						if (range.min !== undefined && value_n < range.min) {
							error = format("<%=_T('{0} value must be greater than {1}.')%>", properties.name, range.min);
						}
						else if (range.max !== undefined && value_n > range.max) {
							error = format("<%=_T('{0} value must be less than {1}.')%>", properties.name, range.max);
						}
					}
				}
			}
			//function
			else {
				error = properties.validate(value);
			}
			if (error) {
				data.error = {
					$container : $container,
					control : properties.id,
					message : error
				};
				return;
			}
		}

		data[properties.id] = value;

		if (properties.brackets) {
			let bracketsValue =  this.__getBracketsValue(value);
			data["__brackets_" + properties.id] = bracketsValue;
		}
	};

	adm_process_flow_popup.prototype.__getBracketsValue = function(value) {
		return inConcert.Utils.getBracketsValue(value);
	};

	adm_process_flow_popup.prototype.__focusControl = function(control) {
		var $control = this.$get(format("[control={0}]", $.isPlainObject(control)? control.id : control));
		if ($control.length) {
			$control.focus();
		}
	};

	adm_process_flow_popup.prototype.__addContent = function($content) {
		this.$get("[content]").append($content);
	};

	adm_process_flow_popup.prototype.__addActions = function($actions) {
		this.$get("[actions]").append($actions);
	};

	adm_process_flow_popup.prototype.__makeActions = function() {
		var self = this;
		var $actions = $(Mustache.render(templates.popup_actions, { hasCloseButton: !self.__hasCloseButton() }));
		$actions.filter("[action=save]").click(function() {
			self.Save();
		});
		$actions.filter("[action=cancel]").click(function() {
			self.__cancel(function() {
				self.Close();
			});
		});
		
		return $actions;
	};

	adm_process_flow_popup.prototype.__hasCloseButton = function() {
		return false;
	};

	adm_process_flow_popup.prototype.__getContainerPopup = function () {
		return "";
	};
	
	adm_process_flow_popup.prototype.__makeBody = function() {
		var self = this;
		let newOrEdit = self.isNew ? "<%=_T('New')%>: " : "<%=_T('Edit')%>: ";
		if (self.isNewHidden) {
            newOrEdit = "";
        }					   
		var className = self.__getClassName();
		if (self.__isFullScreen()) {
			className += " fullScreenPopUp";
		}

		let editTitleButton = true;
		
		if (self.showEditTitleButton === false) 
			editTitleButton = false;
		
		var $popup = $(Mustache.render(templates.popup_container, {
			id: self.__getPopupId(),
			className : className,
			title : self.__getTitle(),
			newOrEdit: newOrEdit,
			showEditTitleButton: editTitleButton
		}));
		if (self.__hasCloseButton()) {
			$popup.find("[close_button_icon]").click(function() {
				self.__cancel(function() {
					self.Close();
				});
			});
			$popup.find("[close_button]").remove();
		}
		else {
			$popup.find("[close_button]").remove();
			$popup.find("[close_button_icon]").remove();
		}

		$popup.find("#flow_designer_activity_description_edit").click(function(){
			$popup.find("[activity_description]").hide();
			$popup.find("#flow_designer_activity_description_edit").hide();
			$("#activity_description_input").val($("#activity_title").text());
			$popup.find("[activity_description_edit]").show();
			$("#activity_description_input").focus();
			$("#flow_designer_internal_popup_title").off("mousedown");
		});

		$popup.find("#flow_designer_activity_description_edit_cancel").click(function(){
			$popup.find("[activity_description]").show();
			$popup.find("#flow_designer_activity_description_edit").show();
			$popup.find("[activity_description_edit]").hide();
			self.___dragDropPopup();
		});

		$popup.find("#flow_designer_activity_description_edit_save").click(function(){
			$popup.find("#activity_title").html($("#activity_description_input").val());
			$popup.find("[activity_description]").show();
			$popup.find("#flow_designer_activity_description_edit").show();
			$popup.find("[activity_description_edit]").hide();
			self.___dragDropPopup();
		});
		return $popup;
	};

	adm_process_flow_popup.prototype.__make = function() {
		var self = this;
		var $popup = self.__makeBody();
		$popup.find("[content]").append(self.__makeContent());

        let actions = self.__makeActions();
        $popup.find("[actions]").append(actions);

        if (!actions) {
            $popup.find("[actions]").hide();
        }
		return $popup;
	};

	adm_process_flow_popup.prototype.__afterAppendBody = $.noop;

	adm_process_flow_popup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $popup = this.__make();
		$("body").append($popup);
		this.popup = new inConcert.PopupWindow();
		this.popup.show({
			ownerId         : self.__getPopupId(),
			eventName       : self.__getPopupId(),
			canReposition   : true,
			closeOnFocusLost: false,
			overlay         : true,
			container       : $popup,
			inlineStyleValues: {
				["z-index"]: (self.overlay ? 300 : 119)
			},
			callbacks: {
				onOpen: function() {
					self.$container().show();
                    if ($("#activity_description_input").val() === "") {
                        $("#activity_description_input").val($("#activity_title").text());
						$(document).off("keydown.process_flow_main");
                    }
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							$(document).off("mousemove.process_flow_popup").off("mouseup.process_flow_popup");
							if (self.editor && self.editor.main) {
                                self.editor.main.__scrollWithArrows();
                            }
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		this.__bindKeyboardEvents();
		if (this.focusControl) {
			this.__focusControl(this.focusControl);
		}
		else if (this.focusFirst) {
			this.$get("input, select, textarea")
				.first()
				.focus();
		}
		this.___dragDropPopup();
		this.__afterAppendBody();
		callback();
	};

	adm_process_flow_popup.prototype.___dragDropPopup = function() {
		let isDragging = false;
		let offsetX, offsetY;
		const inBoxMainHeaderHeight = $("#inboxMainHeader").height();
		const $popup_draggable = $("#flow_designer_internal_popup_draggable");
		const isDraggeable = $popup_draggable.attr("isDraggable");

		if (isDraggeable) {
			$("#flow_designer_internal_popup_title").mousedown(function(e) {
				const modalOffset = $popup_draggable.offset();
				offsetX = e.clientX - modalOffset.left;
				offsetY = e.clientY - modalOffset.top;

				isDragging = true;
				e.preventDefault();
			});

			// El bindeo de eventos debió ser sobre $(document) para que los cálculos fueran correctos
			$(document)
				.off("mousemove.process_flow_popup")
				.on("mousemove.process_flow_popup", function handleMouseMove(e) {
					if (isDragging) {
						const newX = e.clientX - offsetX;
						const newY = e.clientY - offsetY - inBoxMainHeaderHeight;

						// Obtener los límites de la pantalla
						const maxX = window.innerWidth - $popup_draggable.width();
						const maxY = window.innerHeight - $popup_draggable.height();

						// Verificar los límites antes de actualizar la posición
						const constrainedX = Math.max(0, Math.min(newX, maxX));
						const constrainedY = Math.max(0, Math.min(newY, maxY));

						$popup_draggable.css({
							"left": constrainedX + "px",
							"top": constrainedY + "px",
							"right": "auto",
							"bottom": "auto",
							"margin": 0
						});
					}
				})
				.off("mouseup.process_flow_popup")
				.on("mouseup.process_flow_popup", function handleMouseUp() {
					isDragging = false;
				});
		}
	};

	adm_process_flow_popup.prototype.__show_with_container = function(callback, $container) {
		callback = callback || $.noop;
		var self = this;
		var $popup = this.__make();
		$("body").append($popup);
		this.popup = new inConcert.PopupWindow();
		this.popup.show({
			ownerId         : self.__getPopupId(),
			eventName       : self.__getPopupId(),
			canReposition   : true,
			closeOnFocusLost: true,
			overlay         : false,
            parentNode      : $container,
			container       : $popup,
			callbacks: {
				onOpen: function() {
					self.$container().show();
                    if ($("#activity_description_input").val() === "") {
                        $("#activity_description_input").val($("#activity_title").text());
                    }
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		this.__bindKeyboardEvents();
		if (this.focusControl) {
			this.__focusControl(this.focusControl);
		}
		else if (this.focusFirst) {
			this.$get("input, select, textarea")
				.first()
				.focus();
		}
		this.__afterAppendBody();
		callback();
	};

	adm_process_flow_popup.prototype.__load = function(callback) {
		callback();
	};

	adm_process_flow_popup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__processing(true);
			self.__load(function() {
				self.__show(callback);
				self.__processing(false);
				if (self.editor && self.editor.graph) {
					self.editor.graph.getModel().processing = false;
				}
			});
		});
	};

    adm_process_flow_popup.prototype.ShowWithContainer = function(callback, $container) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__load(function() {
				self.__show_with_container(callback, $container);
			});
		});
	};    

	adm_process_flow_popup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.IsProcessing()) {
			return;
		}
		self.__disposeKeyboardEvents();
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				self.$container().remove();
				callback();
			});
		}
		else {
			self.$container().remove();
			callback();
		}
	};

	adm_process_flow_popup.prototype.__onEnter = function() {
		if (this.$get(":focus").is("textarea")) {
			return;
		}
		this.Save();
	};

	adm_process_flow_popup.prototype.__onEscape = function() {
		this.Close();
	};

	adm_process_flow_popup.prototype.__disableNextKeyboardEvent = function() {
		var self = this;
		self.omitNextKeyboardEvent = true;
		if (self.omitNextKeyboardEvent_t) {
			window.clearTimeout(self.omitNextKeyboardEvent_t);
		}
		self.omitNextKeyboardEvent_t = window.setTimeout(function() {
			self.omitNextKeyboardEvent = false;
		}, 1000);
	};

	adm_process_flow_popup.prototype.__bindKeyboardEvents = function() {
		var self = this;

		if (!self.__hasKeyboardEvents()) {
			return;
		}

		this.keyboardEvents = {
			"keyup.process_flow_popup" : function(e) {
				if (self.omitNextKeyboardEvent) {
					self.omitNextKeyboardEvent = false;
					return;
				}
				if (e.keyCode === 13) {
					return self.__onEnter();
				}
				if (e.keyCode === 27) {
					return self.__onEscape();
				}
			}
		};

		$.each(this.keyboardEvents || {}, function(event, handler) {
			$(document).on(event, handler);
		});
	};

	adm_process_flow_popup.prototype.__disposeKeyboardEvents = function() {
		$.each(this.keyboardEvents || {}, function(event, handler) {
			$(document).off(event, handler);
		});
		this.keyboardEvents = null;
	};

	adm_process_flow_popup.prototype.IsProcessing = function() {
		return !!this.processing;
	};

	adm_process_flow_popup.prototype.__processing = function(processing) {
		this.processing = !!processing;
		this.$container().find("[loading]").toggle(this.processing);
	};

	adm_process_flow_popup.prototype.__clearErrors = function() {
		this.hideStatusMessage();
		const $container = this.$container();
		$container.find("[validation]").hide();
		$container.find("[message_text]").text("");
	};

	adm_process_flow_popup.prototype.__showError = function(error, $container, timeout_ms) {
		const self = this;

		this.hideStatusMessage();

		if (typeof error === "string") {
			error = { message : error };
		}

		$container = $container || this.$container();
		var $message = $container.find("[message_container]").first();
		$message
			.show()
			.find("[message_text]")
			.text(error.message);

		$message.parent().scrollTop(0);

		var $validation;
		if (error.control) {
			$validation = (error.$container || $container).find(format("[validation={0}]", error.control));
		}
		else if (error.$control) {
			$validation = error.$control.find("[validation]");
		}
		if ($validation) {
			$validation
				.attr("title", error.message)
				.show();
		}

		if (timeout_ms) {
			this.m_timeout_handle_hide_message = window.setTimeout(function() {
				self.hideStatusMessage();
			}, timeout_ms);
		}
	};

	adm_process_flow_popup.prototype.displayStatusMessage = function(msg_error) {
		this.__showError(msg_error);
	};

	adm_process_flow_popup.prototype.hideStatusMessage = function() {
		if (this.m_timeout_handle_hide_message) {
			window.clearTimeout(this.m_timeout_handle_hide_message);
			this.m_timeout_handle_hide_message = null;
		}
		const $container = this.$container();
		$container.find("[message_container]").hide();
	};

	adm_process_flow_popup.prototype.Save = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.IsProcessing()) {
			return callback(false);
		}
		self.__clearErrors();
		var data = self.__getAndValidate();
		if (data.error) {
			self.__showError(data.error);
			return callback(false);
		}
		self.__processing(true);
		self.__getAndValidateAsync(data, function(data) {
			if (data.error) {
				self.__processing(false);
				self.__showError(data.error);
				return callback(false);
			}
			self.__save(data, function(status, error) {
				self.__processing(false);
				// self.__updateDeclaredVariablesList(data);
				if (status) {
					self.Close(function() {
						self.__afterSave();
						callback(true);
					});
				}
				else {
					if (error) {
						self.__showError(error);
					}
					callback(false);
				}
			});
		});
	};

	adm_process_flow_popup.prototype.ScrollTo = function($element) {
		this.$get("[content]").scrollTo($element);
	};

	adm_process_flow_popup.prototype.SelfOverlay = function(enabled) {
		this.$get("[self_overlay]").toggle(!!enabled);
	};

	adm_process_flow_popup.prototype.$container = function() {
		const $container = $("#" + this.__getPopupId());
		return $container;
	};

	adm_process_flow_popup.prototype.$get = function(query) {
		const $container = this.$container();
		const $element = $container.find(query);
		return $element;
	};

	adm_process_flow_popup.prototype.$get_actions = function() {
		return this.$get("[actions]");
	};

	adm_process_flow_popup.prototype.$get_save_action = function() {
		let $save = this.$get("[actions] [action='save']");
		return $save;
	};

	adm_process_flow_popup.prototype.__makeButtonAdd = function(properties, value) {
		properties.guid = inConcert.Utils.generateGuid();
		properties.control = Mustache.render(templates.button_add, properties);
		properties.control_type = "input";
		var $control = this.__makeControl(properties, value);
		if (properties.click) {
			$control.find("button")
				.click(properties.click);
		}
		return $control;
	};
}());
