local json = require "cjson"
local helpDeskLogin = require "apps.helpdesk.login"
local m_request = require "apps.helpdesk.request"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

--[[
	Ejemplos de la URL que reenderiza la FullApp.
		https://168.62.161.232:2443/ichelpdeskdesarrollo/awsfullappocc.aspx?felipe2@collegeocc,4B24B1765FBEBBD58F155396A7C64502,es
		https://168.62.161.232:2443/ichelpdeskdesarrollo/awsfullappocc.aspx?{USER_ID},{USER_TOKEN}

	Esta función se encarga de armar la URL donde se visualizará toda la App, se le pasa un UserId y su Token.
	Esta URL es retornada al lado del cliente para que pueda ser utilizada en iFrames.
]]--

local m_fullAppURI = "awsfullappocc.aspx?{USER_ID},{USER_TOKEN}"

local function MakeFullAppURI (fullUserId, config)
	assert(fullUserId)
	assert(config)

	LogInfo("[HELPDESK.APPS:MakeFullAppURI] Making FullApp URL, User: '%s'. config: '%s'", fullUserId, json.encode(config))

	local success, userToken = helpDeskLogin.GetUserToken(fullUserId, config)
	if not success then
		local err = userToken
		LogError("[HELPDESK.APPS:MakeFullAppURI] Error Making FullApp URL, User: '%s' Err: '%s'.", fullUserId, err)
		return false, err
	end

	LogInfo("[HELPDESK.APPS:MakeFullAppURI] Making FullApp URL, User: '%s' Token: '%s'\nurl base: '%s'.", fullUserId, userToken, m_fullAppURI)

	--[[ 
		Ahora a la URL le reemplazo los valores que correspondan, {USER_ID}, {USER_TOKEN}
	]]--
	-- 1 - {USER_ID}
	local suffixURL = m_fullAppURI:gsub("{USER_ID}", fullUserId)
	-- 2 - {USER_TOKEN}
	suffixURL = suffixURL:gsub("{USER_TOKEN}", userToken)

	-- Por último obtengo la full URL
	local fullURI = m_request.MakeFullURL(suffixURL, config.Server)

	LogInfo("[HELPDESK.APPS:MakeFullAppURI] FullApp URL: '%s'.", fullURI)
	return true, fullURI
end

function FullAppURI (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	web.headers["Content-Type"] = "application/json"
	LogInfo("[HELPDESK.APPS.FullAppURI] Retrieving Full App URL for 'HelpDesk' App Integration. User '%s'.", fullUserId)

	local enabled, vccHelpDeskConfig = utils.HasHelpDeskIntegration(vcc)
	if not enabled then
		local err = {result = "ERROR RETRIEVING USER TOKEN"}
		LogError("[HELPDESK.APPS.FullAppURI] Error Retrieving Full App URL for 'HelpDesk' App Integration. User '%s'. Err: VCC '%s' has not configured Integration.", fullUserId, vcc)
		web.status = 400
		return json.encode(err)
	end

	local success, uri = MakeFullAppURI(fullUserId, vccHelpDeskConfig)
	if not success then
		local err = {result = "ERROR RETRIEVING FULL APP URL", rawError = uri}
		LogError("[HELPDESK.APPS.FullAppURI] Error Retrieving Full App URL for 'HelpDesk' App Integration. User '%s'. Err: '%s'", fullUserId, json.encode(err))
		web.status = 400
		return json.encode(err)
	end
	--LogDebug("uri: %s", uri)
	local info = {
		uri = uri
	}
	return json.encode(info)
end

function MapUris (module, htmlRender, prefix, appLocation)
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest

	module:dispatch_get(Log .. permissions:CheckPermission("apps_integrations", "helpdesk_full_access", FullAppURI), p .. "/get_uri/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
