local json = require "cjson"
local helpDeskLogin = require "apps.helpdesk.login"
local m_request = require "apps.helpdesk.request"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

--[[
	Ejemplos de la URL que reenderiza la sección de Tickets.
		https://168.62.161.232:2443/ichelpdeskdesarrollo/awsticketsocc.aspx?felipe2@collegeocc,4B24B1765FBEBBD58F155396A7C64502,es
		https://168.62.161.232:2443/ichelpdeskdesarrollo/awsticketsocc.aspx?{USER_ID},{USER_TOKEN}

	Esta función se encarga de armar la URL donde se visualizarán los tickets, se le pasa un UserId y su Token.
	Esta URL es retorna al lado del cliente para que pueda ser utilizada en iFrames.
]]--

local m_ticketsURI = "awsticketsocc.aspx?{USER_ID},{USER_TOKEN}"         -- URL utilizada para listar los tickets dentro de la APP Interaction.
local m_addTicketsURI = "addiccticket.aspx?logged_user={USER_ID}&token={USER_TOKEN}" -- URL utilizada para agregar tickets dentro de los clientes (en los tabs de la sidebar)

local function MakeTicketsURI (urlBase, fullUserId, config)
	assert(fullUserId)
	assert(config)

	LogInfo("[HELPDESK.TICKETS:MakeTicketsURI] Making Tickets URL, User: '%s'. config: '%s'", fullUserId, json.encode(config))

	local success, userToken = helpDeskLogin.GetUserToken(fullUserId, config)
	if not success then
		local err = userToken
		LogError("[HELPDESK.TICKETS:MakeTicketsURI] Error Making Tickets URL, User: '%s' Err: '%s'.", fullUserId, err)
		return false, err
	end

	LogInfo("[HELPDESK.TICKETS:MakeTicketsURI] Making Tickets URL, User: '%s' Token: '%s'\nurl base: '%s'.", fullUserId, userToken, urlBase)

	--[[ 
		Ahora a la URL le reemplazo los valores que correspondan, {USER_ID}, {USER_TOKEN}
	]]--
	-- 1 - {USER_ID}
	local suffixURL = urlBase:gsub("{USER_ID}", fullUserId)
	-- 2 - {USER_TOKEN}
	suffixURL = suffixURL:gsub("{USER_TOKEN}", userToken)

	-- Por último obtengo la full URL
	local fullURI = m_request.MakeFullURL(suffixURL, config.Server)

	LogInfo("[HELPDESK.TICKETS:MakeTicketsURI] Tickets URL: '%s'.", fullURI)
	return true, fullURI
end

function GetTicketsURL (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	web.headers["Content-Type"] = "application/json"
	LogInfo("[HELPDESK.TICKETS.GetTicketsURL] Retrieving Tickets URL for 'HelpDesk' App Integration. User '%s'.", fullUserId)

	local enabled, vccHelpDeskConfig = utils.HasHelpDeskIntegration(vcc)
	if not enabled then
		local err = {result = "ERROR RETRIEVING USER TOKEN"}
		LogError("[HELPDESK.TICKETS.GetTicketsURL] Error Retrieving Tickets URL for 'HelpDesk' App Integration. User '%s'. Err: VCC '%s' has not configured Integration.", fullUserId, vcc)
		web.status = 400
		return json.encode(err)
	end

	local success, uri = MakeTicketsURI(m_ticketsURI, fullUserId, vccHelpDeskConfig)
	if not success then
		local err = {result = "ERROR RETRIEVING TICKETS URL", rawError = uri}
		LogError("[HELPDESK.TICKETS.GetTicketsURL] Error Retrieving Tickets URL for 'HelpDesk' App Integration. User '%s'. Err: '%s'", fullUserId, json.encode(err))
		web.status = 400
		return json.encode(err)
	end
	--LogDebug("uri: %s", uri)
	local info = {
		uri = uri
	}
	return json.encode(info)
end

function GetAddTicketsURL (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	web.headers["Content-Type"] = "application/json"
	LogInfo("[HELPDESK.TICKETS.GetAddTicketsURL] Retrieving Add Tickets URL for 'HelpDesk' App Integration. User '%s'.", fullUserId)

	local enabled, vccHelpDeskConfig = utils.HasHelpDeskIntegration(vcc)
	if not enabled then
		local err = {result = "ERROR RETRIEVING USER TOKEN"}
		LogError("[HELPDESK.TICKETS.GetAddTicketsURL] Error Retrieving Add Tickets URL for 'HelpDesk' App Integration. User '%s'. Err: VCC '%s' has not configured Integration.", fullUserId, vcc)
		web.status = 400
		return json.encode(err)
	end

	local success, uri = MakeTicketsURI(m_addTicketsURI, fullUserId, vccHelpDeskConfig)
	if not success then
		local err = {result = "ERROR RETRIEVING ADD TICKETS URL", rawError = uri}
		LogError("[HELPDESK.TICKETS.GetAddTicketsURL] Error Retrieving Add Tickets URL for 'HelpDesk' App Integration. User '%s'. Err: '%s'", fullUserId, json.encode(err))
		web.status = 400
		return json.encode(err)
	end
	--LogDebug("uri: %s", uri)
	local info = {
		uri = uri
	}
	return json.encode(info)
end

-- -- function RenderTickets (web)
-- -- LogDebug("--------------> 1 RenderTickets")
	-- -- local params = {}
	-- -- local html = m_htmlRender:RenderPageLP("TICKETS_REDIRECT.HTML", web, params)
-- -- LogDebug("--------------> 2 RenderTickets - html: '%s'", html)
	-- -- return html
-- -- end

-- -- function RenderTickets (web)
-- -- LogDebug("--------------> 1.1 RenderTickets")
	-- -- local params = {}
	-- -- local html =[[
-- -- <html>
	-- -- <body>
-- -- Rocio Ayelén de Mello Aquino !!!!!!!!!!!
	-- -- </body>
-- -- </html>
	-- -- ]]
-- -- LogDebug("--------------> 2.1 RenderTickets - html: '%s'", html)
	-- -- return html
-- -- end

-- function RenderTickets (web)
-- LogDebug("--------------> a RenderTickets")
	-- local params = {}

	-- local vcc = inconcert.authUser.virtualContactCenter
	-- local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	-- local enabled, vccHelpDeskConfig = utils.HasHelpDeskIntegration(vcc)
	-- local success, uri = MakeTicketsURI(fullUserId, vccHelpDeskConfig)

	-- local success, html = m_request.GetTickets(uri)
-- LogDebug("--------------> b RenderTickets - success: '%s', html: '%s'", success, html)
-- -- web.headers["Content-Type"] = "application/json"
-- LogDebug("-----1---------> Content-Type: '%s'", web.headers["Content-Type"])
	-- web.headers["Content-Type"] = "text/html; charset=UTF-8"
-- LogDebug("-----2---------> Content-Type: '%s'", web.headers["Content-Type"])


	-- return html
-- end

function MapUris (module, htmlRender, prefix, appLocation)
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest

	module:dispatch_get(Log .. permissions:CheckPermission("apps_integrations", "helpdesk_tickets_in_app_interaction", GetTicketsURL), p .. "/get_uri/"    )
	module:dispatch_get(Log .. GetAddTicketsURL                                                                                      , p .. "/add/get_uri/")
--module:dispatch_get(RenderTickets, p .. "/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
