/*jshint bitwise:true, browser:true, curly:false, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false */
/*global $, inConcert*/

this.inConcert = this.inConcert || {};
this.inConcert.helpdesk = this.inConcert.helpdesk || {};

(function () {
	"use strict";

	const _T = inConcert.i18n.getString;
	const HELPDESK_APP_ID                = "apps_integrations";
	const HELPDESK_ACTION_FULL_ACCESS_ID = "helpdesk_full_access";
	const HELPDESK_ACTION_TICKETS_ID     = "helpdesk_tickets_in_app_interaction";

	let m_current_helpdesk_view = null;

	const can_show = function(app_id, action_id) {
		const is_enabled_helpdesk = inConcert.app.has_helpdesk_integration();
		if (!is_enabled_helpdesk) {
			return false;
		}
		if (!app_id || !action_id) {
			return false;
		}
		const has_permission = inConcert.permissions.to_action(app_id, action_id);
		if (!has_permission) {
			return false;
		}
		return true;
	};

	function HelpDeskBaseGUI (params) {
		this.container_id = params.container_id;
		this.render_status = null;
		this.server_err_info = null;
		this.m_hndReload = null;
		this.$dashboard = null;
		if (this.can_show()) {
			this.__create_markup();
		}
	}

	HelpDeskBaseGUI.prototype.get_container_id = function() {
		return this.container_id;
	};

	HelpDeskBaseGUI.prototype.__create_markup = function(callback = $.noop) {
		if (this.$dashboard) {
			callback({success: true, reason: "already created markup", name: this.name});
			return;
		}
		const markup = "".concat(
			"<div waiting_container='true' hidden>",
				"<div class='helpDeskContentListWaiting'>",
					"<div class='helpDeskListWaitingMessage'>", _T("Loading, wait moment please"), "</div>",
					"<div class='helpDeskListWaitingAnimation'></div>",
					"<div class='helpDeskListWaitingOverlay'></div>",
				"</div>",
			"</div>",
			"<div error_container='true' hidden>",
				"<div class='helpDeskContentErrorContainer'>",
					"<div class='helpDeskListErrorMessage'>",
						"<label error_msg='true' class='TU_CLASE_RCR'></label>",
						"<button load_retry='true' class='helpDeskListRetryButton' type='button' tooltip='", _T("Retry"), "'>",
							"<i class='TU_CLASE_RCR'></i>",
							_T("Retry"),
						"</button>",
					"</div>",
					"<div class='helpDeskListErrorIcon'></div>",
					"<div class='helpDeskListErrorOverlay'></div>",
				"</div>",
			"</div>",
			"<div view_container='true' hidden>",
				"<iframe view_frm='true' scrolling='yes' frameborder='0' allowtransparency='true' style='position: absolute;left: 0px;width: 100%;height: 100%;top: 0px;bottom: 0px;'></iframe>",
			"</div>"
		);
		const $markup = $(markup);
		this.$dashboard = $("#" + this.container_id).html($markup);
		this.__render_integration(false, callback);
	};

	HelpDeskBaseGUI.prototype.__on_finish_render_integration = function(render_status, callback = $.noop)
	{
		this.render_status = render_status;
		m_current_helpdesk_view = this.name;
		callback();
	};

	HelpDeskBaseGUI.prototype.__render_integration = function(must_reload, callback = $.noop)
	{
		const self = this;

		const log_preffix = "-----> [" + this.name + ":__render_integration] ";
		console.info(log_preffix + "ENTER - must_reload: " + must_reload);

		if (this.render_status === "RENDERING") {
			console.info(log_preffix + "return already rendering.");
			callback({success: false, reason: "already created markup", name: this.name});
			return;
		}
		const is_enabled_helpdesk = inConcert.app.has_helpdesk_integration();
		if (!is_enabled_helpdesk) {
			console.info(log_preffix + "disabled.");
			callback({success: false, reason: "disabled helpdesk integration", name: this.name});
			return;
		}
		if (!this.has_license()) {
			console.info(log_preffix + "has not license.");
			callback({success: false, reason: "has not license for helpdesk integration", name: this.name});
			return;
		}

		const $waiting_container = this.$dashboard.find("[waiting_container]");
		const $view_container = this.$dashboard.find("[view_container]");
		const $error_container = this.$dashboard.find("[error_container]");
		const $error_msg = $error_container.find("[error_msg]");

		const show_waiting = function() {
			console.info(log_preffix + "-> show_waiting");
			self.render_status = "RENDERING";
			$error_container.hide();
			$view_container.hide();
			$waiting_container.show();
		};
		const show_error = function(err_info) {
			console.info(log_preffix + "-> show_error");
			self.__on_finish_render_integration("ERROR", function () {
				$waiting_container.hide();
				$view_container.hide();
				const err_msg = self.__get_error_msg(err_info);
				$error_msg.text(err_msg);
				$error_container.show();
				callback({success: false, reason: err_msg, name: self.name});
			});
		};
		const show_integration = function() {
			console.info(log_preffix + "-> show_integration");
			self.__on_finish_render_integration("SUCCESS", function () {
				$waiting_container.hide();
				$error_container.hide();
				$view_container.show();
				callback({success: true, name: self.name});
			});
		};

		show_waiting();

		const $retry = $error_container.find("[load_retry]");
		$retry.off("click")
			.on("click", function() {
				self.__render_integration(false, callback);
			});

		const $iframe = $view_container.find("[view_frm]");
		const current_uri = $iframe.attr("src");

		const set_uri_to_iframe = function(uri) {
			console.info(log_preffix + "-> set_uri_to_iframe");
			$iframe.attr("src", uri);
			// El show de la seccion del iFrame se visualiza despues de un timeout de N segundos, ya que la carga del helpdesk
			// puede demorar, y no tenemos feedback de su carga.
			// TODO: revisar si es posible obtener feedback de la carga del iframe.
			const IFRAME_LOADING_TIMEOUT_MS = 5000;
			window.setTimeout(function() {
				show_integration();
			}, IFRAME_LOADING_TIMEOUT_MS);
		};

		if (must_reload && current_uri && current_uri !== "") {
			console.info(log_preffix + "clean iframe.src. setTimeout 1000");
			$iframe.attr("src", "");
			window.setTimeout(function() {
				set_uri_to_iframe(current_uri);
			}, 1000);
		}
		else {
			console.info(log_preffix + "retrieving uri");
			this.get_helpdesk_render_uri(function(success, helpdesk_info) {
				console.info(log_preffix + "retrieved uri, success " + success);
				if (!success) {
					show_error(helpdesk_info);
					return;
				}
				const uri = helpdesk_info.uri;
				set_uri_to_iframe(uri);
			});
		}
	};

	HelpDeskBaseGUI.prototype.__get_error_msg = function(err_info)
	{
		const default_msg = _T("There was an error, please try again later.");
		let msg_error = default_msg;
		if (err_info) {
			this.server_err_info = err_info;
			if (err_info.code === 403) {
				msg_error = _T("You not has permission for this view.");
			}
		}
		return msg_error;
	};

	HelpDeskBaseGUI.prototype.get_helpdesk_render_uri = function()
	{
		// Se debe implementar en las clases hijas
		throw("HelpDeskBaseGUI.prototype.get_helpdesk_render_uri. TODO: Implement !!");
	};

	HelpDeskBaseGUI.prototype.has_license = function()
	{
		// Se debe implementar en las clases hijas
		throw("HelpDeskBaseGUI.prototype.has_license. TODO: Implement !!");
	};

	HelpDeskBaseGUI.prototype.can_show = function() {
		return can_show(this.permissions.app_id, this.permissions.action_id);
	};

	HelpDeskBaseGUI.prototype.can_show_related_view = function() {
		return can_show(this.permissions.app_id, this.permissions.related_action_id);
	};

	HelpDeskBaseGUI.prototype.show = function() {
		if (!this.$dashboard) {
			return;
		}
		if (this.css_body_helpdesk_class) {
			$("body").removeClass(this.css_body_helpdesk_class)
				.addClass(this.css_body_helpdesk_class);
		}
		return this.$dashboard.show();
	};

	HelpDeskBaseGUI.prototype.hide = function() {
		if (!this.$dashboard) {
			return;
		}
		if (this.css_body_helpdesk_class) {
			$("body").removeClass(this.css_body_helpdesk_class);
		}
		return this.$dashboard.hide();
	};

	HelpDeskBaseGUI.prototype.render = function(callback = $.noop) {
		this.show();
		if (!this.$dashboard && this.can_show()) {
			this.__create_markup(callback);
			return;
		}
		this.__reload(callback);
	};

	HelpDeskBaseGUI.prototype.reload = function(callback = $.noop) {
		const log_preffix = "-----> [" + this.name + ":reload] ";
		console.info(log_preffix + "ENTER");
		if (this.m_hndReload) window.clearTimeout(this.m_hndReload);
		this.m_hndReload = window.setTimeout(() => {
			if (!this.$dashboard) {
				console.info(log_preffix + "Unable to reload has not integration - $dashboard html object does not exist");
				callback({success: false, reason: "has not integration", panel: this.name});
				return;
			}
			if (!this.can_show()) {
				console.info(log_preffix + "Unable to reload - not has permission");
				callback({success: false, reason: "unable to show", name: this.name});
				return;
			}
			console.info(log_preffix + "START");
			this.__reload(true, callback); // force_reload
		});
	};

	HelpDeskBaseGUI.prototype.__must_reload = function() {
		if (this.render_status === "RENDERING" || this.render_status === "ERROR") {
			return false;
		}
		if (m_current_helpdesk_view === this.name) {
			return false;
		}
		if (this.server_err_info && this.server_err_info.code === 403) {
			return false;
		}
		return true;
	};

	HelpDeskBaseGUI.prototype.__reload = function(force_reload, callback = $.noop) {
		if (force_reload) {
			this.__render_integration(true, callback);
			return;
		}
		if (!this.__must_reload()) {
			return;
		}
		this.__render_integration(true, callback);
	};

	HelpDeskBaseGUI.prototype.on_leave = function() {
		this.hide();
	};

	HelpDeskBaseGUI.prototype.get_permissions = function() {
		return this.permissions;
	};

	//-----------------------------------------------------------------------------
	//---------------------------------------------------------------------
	// HelpDesk Dashboard Integration
	//
	function HelpDeskDashboard (params)
	{
		this.permissions = {
			app_id           : HELPDESK_APP_ID,
			action_id        : HELPDESK_ACTION_FULL_ACCESS_ID,
			related_action_id: HELPDESK_ACTION_TICKETS_ID
		};
		this.css_body_helpdesk_class = "appHelpDeskIntegration";
		this.name = "HELPDESK Full App";

		HelpDeskDashboard.uber.constructor.call(this, params);
	}
	inConcert.Utils.inherit(HelpDeskDashboard, HelpDeskBaseGUI);
	if (typeof inConcert.helpdesk.dashboard !== "object") {
		inConcert.helpdesk.dashboard = HelpDeskDashboard;
	}

	HelpDeskDashboard.prototype.get_helpdesk_render_uri = function(callback = $.noop)
	{
		const helpdesk_api = inConcert.app.getHelpDeskAPI();
		helpdesk_api.getFullAppURI(function(success, uri_info) {
			callback(success, uri_info);
		});
	};

	HelpDeskDashboard.prototype.has_license = function()
	{
		return inConcert.app.has_license_for_helpdesk_full_app();
	};

	//---------------------------------------------------------------------
	// HelpDesk Tickets Integration
	//
	function HelpDeskTickets (params)
	{
		this.permissions = {
			app_id           : HELPDESK_APP_ID,
			action_id        : HELPDESK_ACTION_TICKETS_ID,
			related_action_id: HELPDESK_ACTION_FULL_ACCESS_ID
		};
		this.css_body_helpdesk_class = null;
		this.name = "HELPDESK Tickets";

		HelpDeskTickets.uber.constructor.call(this, params);
	}
	inConcert.Utils.inherit(HelpDeskTickets, HelpDeskBaseGUI);
	if (typeof inConcert.helpdesk.tickets !== "object") {
		inConcert.helpdesk.tickets = HelpDeskTickets;
	}

	HelpDeskTickets.prototype.get_helpdesk_render_uri = function(callback = $.noop)
	{
		const helpdesk_api = inConcert.app.getHelpDeskAPI();
		helpdesk_api.getTicketsURI(function(success, uri_info) {
			callback(success, uri_info);
		});
	};

	HelpDeskTickets.prototype.has_license = function()
	{
		return inConcert.app.has_license_for_helpdesk_tickets();
	};

	HelpDeskTickets.prototype.can_show = function() {
		const is_mobile = inConcert.app.isMobile();
		if (is_mobile) {
			return false;
		}
		return HelpDeskTickets.uber.can_show.call(this); //invokes the base class method
	};

}());
