local json = require "cjson"
local utils = require "shared.apps.utils"
local m_permissions = require "shared.permissions.permissions"
local m_widgetsApi = {}
local m_models

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

local INCONCERT_ENV_MODE = (os.getenv("INCONCERT_ENV_MODE") or "PRODUCTION"):upper()

function GetEnvironmentVariable ()
	--LogDebug("[INTEGRATIONS.integrations:GetEnvironmentVariable] ===> INCONCERT_ENV_MODE: '%s'", INCONCERT_ENV_MODE)
	return INCONCERT_ENV_MODE
end

function ValidateByEnvironmentVariable ()
	-- if true then
	-- 	return false
	-- end
	local isAllowed = INCONCERT_ENV_MODE == "DEVELOPMENT"
	if isAllowed then
		LogInfo("[INTEGRATIONS.integrations:ValidateByEnvironmentVariable] ===> Widgets are all available in DEVELOPMENT mode. Environment variable: INCONCERT_ENV_MODE")
	end
	return isAllowed
end

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.integrations.model"
		model:BuildModels()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function GetInteraction (interactionId)
	assert(interactionId)
	local interaction = GetModel("Interactions").GetInteraction(interactionId)
	if interaction then
		-- OK
		return interaction
	end
	-- ERROR
	return nil
end

function MapUris (module, htmlRender, prefix, appLocation)
	m_htmlRender = htmlRender
	m_appLocation = nil --appLocation
end

function MapWidgetsURIs (module, widgetsControllers, widgetsControllerPath, widgetDir)
	--LogDebug("[INTEGRATIONS.INTEGRATIONS:MapWidgetsURIs] b--------------------------------------------------------->  widgetDir: %s", widgetDir)

	-- LogDebug("[INTEGRATIONS.INTEGRATIONS:MapWidgetsURIs] ----------------------------------------------------------> widgetsControllers '%s'.", widgetsControllers)
	-- LogDebug("[INTEGRATIONS.INTEGRATIONS:MapWidgetsURIs] ----------------------------------------------------------> widgetsControllerPath '%s'.", widgetsControllerPath)
	-- LogDebug("[INTEGRATIONS.INTEGRATIONS:MapWidgetsURIs] ----------------------------------------------------------> widgetDir '%s'.", widgetDir)

	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local responseFn = utils.JsonReply .. LogRequest

	local widgetBaseUrl = string.format("/%s/", widgetDir)

	for key, controllerDef in pairs(widgetsControllers) do
		--requires y mapeo funciones
		--
		local controllerName = widgetsControllerPath .. "." .. key --"apps.integrations.controllers.integrations"
		--LogDebug("------------> controllerName %s", controllerName)
		local controller = require(controllerName)
		if type(controller.GetMapUris) == "function" then
			local widgetEntityBaseUrl = widgetBaseUrl .. key:lower()
			local definitions = controller.GetMapUris()
			for _, def in ipairs(definitions or {}) do
				local responseFn = utils.JsonReply .. LogRequest

				if def.noValidatePermissions == true then
					-- SIN validacion de permisos en las URLs
					responseFn =  responseFn .. def.fn
				else
					-- CON validacion de permisos en las URLs
					responseFn = responseFn .. m_permissions:CheckAppPermission("interaction", nil, def.fn)
				end
				-- la URLs
				local url = widgetEntityBaseUrl .. def.url
				--LogDebug("[INTEGRATIONS.INTEGRATIONS:MapWidgetsURIs]      b.2----------------------------------------------------------> url '%s'.", url)
				if (def.methodType or ""):upper() == "GET" then
					-- GET
					module:dispatch_get(responseFn, url)
				else
					-- POST
					module:dispatch_post(responseFn, url)
				end
			end
		end
	end
end

function Initialize (urlParameters)
	local vcc = inconcert.authUser.virtualContactCenter
	local simpleUserId = inconcert.authUser.userName
	local widgetId = urlParameters.widget_id:lower() -- "example" "sample"
	local widgetApiModuleName = string.format("apps.integrations.controllers.widgets.%s.api", widgetId)
	local interactionId = urlParameters.interaction_id

	--LogDebug("[INTEGRATIONS:integrations.Initialize] 111111--------------------------------> widgetApiModuleName: '%s'.", widgetApiModuleName)
	m_widgetsApi[widgetId] = require(widgetApiModuleName)
	--LogDebug("[INTEGRATIONS:integrations.Initialize] 2222222--------------------------------.")

	local user = GetModel("Users").Get(vcc, simpleUserId)
	--LogDebug("[INTEGRATIONS:integrations.Initialize] 333333--------------------------------> user: '%s'.", json.encode(user))

	local vccWidgetConfig = GetModel("VCCs").GetWidgetConfig(vcc, widgetId) or {}
	--LogDebug("[INTEGRATIONS:integrations.Initialize] 444444 --------------------------------> vcc: '%s', widgetId: '%s', vccWidgetConfig: '%s'.", vcc, widgetId, json.encode(vccWidgetConfig))

	local interactionData = {
		id = interactionId
	}
	local interaction = GetInteraction(interactionId)
	-- LogDebug("[INTEGRATIONS:integrations.Initialize] 555555 --------------------------------> vcc: '%s', interactionId: '%s', interaction: '%s'.", vcc, interactionId, json.encode(interaction))
	if interaction then
		-- si queremos enviar toda la interaccion
		--interactionData = interaction
		-- datos especificos
		interactionData.id = interaction.id
		interactionData.campaign = interaction.campaign
		interactionData.account = interaction.account
		interactionData.initiative = interaction.initiative
		interactionData.type = interaction.type
		-- TODO: Attention levels ?
	end

	local initialData = {
		vcc = {
			id = vcc,
			widgetConfig = vccWidgetConfig
		},
		interaction = interactionData,
		user = {
			LanguageId         = user.LanguageId,
			Name               = user.Name,
			LastName           = user.LastName,
			FirstSurname       = user.FirstSurname,
			LastSurname        = user.LastSurname,
			Mail               = user.Mail,
			DefaultCampaign    = user.DefaultCampaign,
			CompanyInformation = user.CompanyInformation or {},
			CustomInformation  = user.CustomInformation or {},
			UserGroup          = user.UserGroup,
			initialState       = user.initialState,
			countryCode        = user.countryCode,
			areaCode           = user.areaCode,
			TimeZoneId         = user.TimeZoneId
		},
		widgetExtraData = {
			data = m_widgetsApi[widgetId].Initialize(urlParameters)
		}
	}
	--LogDebug("[INTEGRATIONS:integrations.Initialize] ------------------------------->>> initialData: %s", json.encode(initialData))

	return initialData
end

function IsAllowedWidgetInVcc (widgetId)
	local vcc = inconcert.authUser.virtualContactCenter
	if vcc:lower() == "system" then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInVcc] Widgets are NOT AVAILABLE for the SYSTEM VCC.")
		return false, "WIDGET NOT ALLOWED FOR VCC"
	end

	if ValidateByEnvironmentVariable() then
		return true, ""
	end

	local vccWidgetConfig = GetModel("VCCs").GetWidgetConfig(vcc, widgetId)
	if not vccWidgetConfig then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInVcc] Widget '%s' not configured in VCC '%s'", widgetId, vcc)
		return false, "WIDGET NOT ALLOWED FOR VCC"
	end
	if tonumber(vccWidgetConfig.Enabled or "0") ~= 1 then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInVcc] Widget '%s' is disabled in VCC '%s'", widgetId, vcc)
		return false, "WIDGET NOT ALLOWED FOR VCC"
	end
	-- ok
	LogInfo("[INTEGRATIONS.integrations:IsAllowedWidgetInVcc] Widget '%s' is AVAILABLE in VCC '%s'", widgetId, vcc)
	return true, "IS ALLOWED FOR VCC"
end

function IsAllowedWidgetInCampaign (widgetId, interactionId)
	assert(widgetId)
	assert(interactionId)

	widgetId = widgetId:lower()
	if ValidateByEnvironmentVariable() then
		return true, ""
	end

	local interaction = GetInteraction(interactionId)
	if not interaction then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] 1 - Widget '%s' validation error. Interaction does not exist '%s'", widgetId, interactionId)
		return false, "WIDGET NOT ALLOWED FOR CAMPAIGN"
	end

	local campaignId = interaction.campaign
	local encodedCampaignIntegrations = GetModel("Campaigns").GetSpecificColumn(campaignId, "Integration")
	local campaignIntegrations = utils.JSON_Decode(encodedCampaignIntegrations)

	--LogDebug("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] Campign: '%s', Integrations: '%s'", campaignId, encodedCampaignIntegrations)

	if not campaignIntegrations then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] 2-  Widget '%s' is disabled in CAMPAIGN '%s'. Campaign has not configured Integrations.", widgetId, campaignId)
		return false, "WIDGET NOT ALLOWED FOR CAMPAIGN"
	end

	if not campaignIntegrations.othersIntegrations then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] 3 - Widget '%s' is disabled in CAMPAIGN '%s'. Campaign has not configured othersIntegrations.", widgetId, campaignId)
		return false, "WIDGET NOT ALLOWED FOR CAMPAIGN"
	end

	local campaignWidgetConfig = campaignIntegrations.othersIntegrations[widgetId]
	if not campaignWidgetConfig then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] 4 - Widget '%s' is disabled in CAMPAIGN '%s'. Campaign has not configured Current Widget Integration.", widgetId, campaignId)
		return false, "WIDGET NOT ALLOWED FOR CAMPAIGN"
	end

	if not campaignWidgetConfig.Enable then
		LogWarning("[INTEGRATIONS.integrations:IsAllowedWidgetInCampaign] 5 - Widget '%s' is disabled in CAMPAIGN '%s'.", widgetId, campaignId)
		return false, "WIDGET NOT ALLOWED FOR CAMPAIGN"
	end

	-- OK
	return true, campaignId
end

local function HasPermissionToApplication (appId, fullCampaignId, keyspace)
	assert(appId)

	local vcc = inconcert.authUser.virtualContactCenter
	local simpleUserId = inconcert.authUser.userName
	local fullUserId = utils.AppendVcc(simpleUserId, vcc)
	local roleId = inconcert.authUser.userGroup

	-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
	local hasPermissionToApp = m_permissions.HasPermissionToApplication(
		appId,
		fullUserId,
		roleId,
		fullCampaignId,
		keyspace
	)

	-- LogDebug("[INTEGRATIONS.integrations:HasPermissionToApplication]---> %s TIENE PERMISOS sobre la App '%s'. fullUserId: '%s', roleId: '%s', fullCampaignId: '%s",
	-- 	(hasPermissionToApp and "1 - SI" or "2 - NO"),
	-- 	appId,
	-- 	fullUserId,
	-- 	roleId,
	-- 	fullCampaignId
	-- )

	if not hasPermissionToApp then
		-- NO tiene permisos sobre la App
		LogWarning("[INTEGRATIONS.integrations:HasPermissionToApplication] --> Can't access widgets. User '%s' has not permission to App '%s'.", appId, fullUserId)
		return false, "USER HAS NOT PERMISSION TO APP"
	end

	-- OK - tiene permisos sobre la App
	return true
end

function HasPermissionToInteractionApp (fullCampaignId, keyspace)
	local success, err = HasPermissionToApplication("interaction", fullCampaignId, keyspace)
	if not success then
		-- NO tiene permisos sobre la App Interaction
		return false, "USER HAS NOT PERMISSION TO INTERACTION APP"
	end
	-- OK - tiene permisos sobre la App Interaction
	return true 
end
