module(..., package.seeall)
local utils = require "shared.apps.utils"
local m_datasource

function GetName ()
	return "Campaigns"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function CampaignExists ( campaign_at_vcc, keyspace )
	local vcc = inconcert.authUser.virtualContactCenter
	campaign_at_vcc = utils.AppendVcc(utils.GetId(campaign_at_vcc), vcc)
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaign_at_vcc,
		column = "Id"
	})
end

function GetSpecificColumn ( campaign_at_vcc, specific_column, keyspace )
	if not specific_column then
		return
	end
	local vcc = inconcert.authUser.virtualContactCenter
	campaign_at_vcc = utils.AppendVcc(utils.GetId(campaign_at_vcc), vcc)
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaign_at_vcc,
		column = specific_column
	})
end

function GetVCCCampaigns (vcc, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCCampaigns",
		key = vcc
	} or {}
end

function GetCampaign (vcc, Id, keyspace, columns)
	return utils.GetAllColumns {
		keyspace		= keyspace or "Agent",
		columnFamily	= "Campaigns",
		key				= utils.AppendVcc(Id, vcc),
		columns			= columns
	} or {}
end

function GetCampaignAccount (campaignIdWithVcc, accountId, keyspace, columns)
	assert(campaignIdWithVcc, "campaignIdWithVcc id is required")
	assert(accountId, "campaignIdWithVcc id is required")

	local vcc = inconcert.authUser.virtualContactCenter
	campaignIdWithVcc = utils.AppendVcc(utils.GetId(campaignIdWithVcc), vcc)

	local accounts = GetCampaignAccounts(campaignIdWithVcc, keyspace, columns)
	return accounts[accountId]
end

function GetUserCampaigns (vcc, userId, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaigns",
		key = utils.AppendVcc(userId, vcc)
	} or {}
end

function GetCampaignMembers (vcc, campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignMembers",
		key = utils.AppendVcc(campaign_id, vcc)
	} or {}
end

-- rules
function GetRules (campaign_id, keyspace)
	local rules = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignRules",
		key = campaign_id,
		superColumns = true
	} or {}
	for rule_id, rule in pairs(rules) do
		local rule_error = m_datasource.redis:get("RuleError:" .. campaign_id .. ":" .. rule_id)
		if rule_error and rule_error ~= "" then
			rule.error = json.decode(rule_error)
		end
	end
	return rules
end

function GetAttentionLevels (campaign_id, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAttentionLevels",
		key = campaign_id
	} or {}
end

function GetAttentionLevelsByUser (user_id, keyspace)
	local user_id_without_vcc = utils.GetId( user_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local full_user_id = utils.AppendVcc(user_id_without_vcc, vcc )

	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AttentionLevelsByUser",
		key = full_user_id
	} or {}
end

function IsUserInAttentionLevels (user_id, keyspace)
	local user_id_without_vcc = utils.GetId( user_id )
	local vcc = inconcert.authUser.virtualContactCenter
	local full_user_id = utils.AppendVcc(user_id_without_vcc, vcc )
	local als_by_user = GetAttentionLevelsByUser( full_user_id, keyspace )
	local has_als = not utils.TableIsEmpty(als_by_user)
	return has_als
end

function GetAccountMessages ( campaign_id, accountId, keyspace )
	local key = MakeAccountMessageId( campaign_id, accountId )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAccountMessages",
		key = key
	} or {}
end

function GetCampaignAccountsMessages (campaignId, campaignAccounts, keyspace)
	local accountsMessages = {}
	for _, account in pairs(campaignAccounts or {}) do
		accountsMessages[account.id] = GetAccountMessages(campaignId, account.id, keyspace)
	end
	return accountsMessages
end

function UserIsMember (campaign_id, userId, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local res, column = pcall(keyspace.GetColumn, keyspace, campaign_id, "CampaignMembers", userId)
	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		return column and column.value
	end
end
