/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false */
/*global, inConcert, $*/

/*
	Clase que se instancia la clase del archivo (main.js)
	
	Esta clase se encarga de:
		Realizar request (invocacion AJAX) hacia las URLs declaradas en cada controller del Widget.


	Expone en JavaScript el objeto "window.occIntegrations.widget.widget"
*/

window.occIntegrations = window.occIntegrations || {};
window.occIntegrations.widget = window.occIntegrations.widget || {};

(function () {
	"use strict";
	const FORMAT = inConcert.Utils.format;

	// -------------------------------------------------
	const REQUEST_TIMEOUT_MS = 1800000; // 30 minutes

	function API (params)
	{
		if(typeof(params) !== "object") {
			throw("API.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("API.Constructor - base uri is required");
		}
		let widget_id = window.occIntegrations.widgetId;
		this.uri = params.uri + "/" + widget_id + "/";
	}

	if (typeof window.occIntegrations.widget.api !== "function") {
		window.occIntegrations.widget.api = API;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	*/
	API.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Get: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	API.prototype.getRequestTimeout = function() { return REQUEST_TIMEOUT_MS; };

	/********************************************************************************************************

	********************************************************************************************************/
		API.prototype.saveSampleUser = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/users/save
			*/
			let uri = FORMAT("{0}users/save/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};

		API.prototype.removeSampleUser = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/users/delete
			*/
			let uri = FORMAT("{0}users/delete/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};

		API.prototype.getSampleUser = function(user_id, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/users/get/emma
			*/
			let uri = FORMAT("{0}users/get/{1}", this.uri, user_id);
	
			let request_handler = this.__Get(uri, function(success, list) {
				if (success) {
					callback(true, list);
					return;
				}
				// error
				callback(false, list);
			});
			return request_handler;
		};

		API.prototype.listSampleUsers = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/users/list_all
			*/
			let uri = FORMAT("{0}users/list_all/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};

		// -------------------------------------------------------------
		// -------------------------------------------------------------

		API.prototype.saveSampleOther = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/others/save
			*/
			let uri = FORMAT("{0}others/save/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};

		API.prototype.removeSampleOther = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/others/delete
			*/
			let uri = FORMAT("{0}others/delete/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};
		//--
		API.prototype.getSampleOther = function(user_id, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/others/get/emma
			*/
			let uri = FORMAT("{0}others/get/{1}", this.uri, user_id);
	
			let request_handler = this.__Get(uri, function(success, list) {
				if (success) {
					callback(true, list);
					return;
				}
				// error
				callback(false, list);
			});
			return request_handler;
		};

		API.prototype.listSampleOthers = function(data, callback = $.noop)
		{
			/*
			* sample URI:
			*   http://i6.inconcert.cc/inconcert/apps/integrations/sample/others/list_all
			*/
			let uri = FORMAT("{0}others/list_all/", this.uri);
			let request_handler = this.__Post(uri, callback, data);
			return request_handler;
		};
}());