/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false */
/*global $ */

/*
	Clase que se instancia en el "ready" del HTML del iFrame.
	En este mismo archivo debajo de todo.

	La misma se encarga del manejo de la logica del Widget (iFrame):
		asignacion de eventos
		invocacion al API propia de la Integración (api.js)
		comunicacion con el Agent vía “Agent API”.
		acceso al DOM.

	Expone en JavaScript el objeto "window.occIntegrations.widget.main"
*/

window.occIntegrations = window.occIntegrations || {};
window.occIntegrations.widget = window.occIntegrations.widget || {};

(function () {
//	"use strict";
//	const FORMAT = inConcert.Utils.format;

	function main () {
		this.m_master = window.occIntegrations.master;
		this.m_integrationUrl = this.m_master.get_integrations_url(); // "https://testubu.i6.inconcertcc.com/inconcert/apps/integrations"
		this.m_icApi = this.m_master.api_agent_get_instance();
		this.initialize();
	}

	main.prototype.initialize = function()
	{
		this.set_events();
		this.api = new window.occIntegrations.widget.api({
			uri: this.m_integrationUrl
		});
		// Ejemplos: Borrar!!
			// parametros en el SRC del iFrame
			$("#iFrameParameters").text(JSON.stringify(window.occIntegrations.urlParameters));
			// carga inicial
			$("#initialData").text(JSON.stringify(window.occIntegrations.initialData));
			// agent api - interaction data (segun el interaction id recibido por parametro en la src del iframe)
			$("#idGetInteraction").val(window.occIntegrations.urlParameters.interaction_id);
			$("#submitGetInteraction").trigger("click"); // ejecuto el metodo del API
	};

	main.prototype.set_events = function()
	{
		const self = this;

		$("#entity_selector").change(function() {
			let val = $(this).val();
			if (val === "users") {
				$("#user_section").show();
				$("#other_section").hide();
			}
			else {
				$("#other_section").show();
				$("#user_section").hide();
			}
		});

		const $msg = $("#msg");
		const $msgTitle = $("#msg-title");
		const $msgContainer = $("#msg-container");
		const show_msg = function(msg) {
			$msg.text(msg || "....").show();
			$msgTitle.text("Success").show();
			$msgContainer.show();
		};
		const hide_msg = function() {
			$msg.hide();
			$msgContainer.hide();
		};

		const $user_id = $("#sample_users_id");
		const $user_name = $("#sample_users_name");

		$("#sample_users_get")
			.off("click")
			.on("click", function() {
				hide_msg();
				let user_id = $user_id.val();
				$user_name.val("");
				self.api.getSampleUser(user_id, function(success, response) {
					if (!success) {
						let err = JSON.stringify(response);
						show_msg(err);
						$msgTitle.text("Retrieving Error");
						return;
					}
					let user = response;
					$user_name.val(user.Name);
					let ok = JSON.stringify(response);
					show_msg(ok);
					$msgTitle.text("Success Retrieved");
				});
			});

		$("#sample_users_list_all")
			.off("click")
			.on("click", function() {
				hide_msg();
				let data = {};
				self.api.listSampleUsers(data, function(success, response) {
					if (!success) {
						let err = JSON.stringify(response);
						show_msg(err);
						$msgTitle.text("Error - List All");
						return;
					}
					let list = JSON.stringify(response);
					show_msg(list);
					$msgTitle.text("Success List All");
				});
			});

		$("#sample_users_save")
			.off("click")
			.on("click", function() {
				hide_msg();
				let user = {
					Id: $user_id.val(),
					Name: $user_name.val()
				};
				self.api.saveSampleUser(user,
					function(success, response) {
						if (!success) {
							let err = JSON.stringify(response);
							show_msg(err);
							$msgTitle.text("Saving Error");
							return;
						}
						$user_id.val("");
						$user_name.val("");
						let ok = JSON.stringify(response);
						show_msg(ok);
						$msgTitle.text("Success Saved");
					}
				);
			});

		$("#sample_users_delete")
			.off("click")
			.on("click", function() {
				hide_msg();
				let user = {
					Id: $user_id.val()
				};
				self.api.removeSampleUser(user,
					function(success, response) {
						if (!success) {
							let err = JSON.stringify(response);
							show_msg(err);
							$msgTitle.text("Deleting Error");
							return;
						}
						$user_id.val("");
						$user_name.val("");
						let ok = JSON.stringify(response);
						show_msg(ok);
						$msgTitle.text("Success Deleted");
					}
				);
			});

		//----------------------
		const $others_id = $("#sample_others_id");
		const $others_name = $("#sample_others_name");

		$("#sample_others_get")
			.off("click")
			.on("click", function() {
				hide_msg();
				let others_id = $others_id.val();
				$others_name.val("");
				self.api.getSampleOther(others_id, function(success, response) {
					if (!success) {
						let err = JSON.stringify(response);
						show_msg(err);
						$msgTitle.text("Retrieving Error");
						return;
					}
					let user = response;
					$others_name.val(user.Name);
					let ok = JSON.stringify(response);
					show_msg(ok);
					$msgTitle.text("Success Retrieved");
				});
			});

		$("#sample_others_list_all")
			.off("click")
			.on("click", function() {
				hide_msg();
				let data = {};
				self.api.listSampleOthers(data, function(success, response) {
					if (!success) {
						let err = JSON.stringify(response);
						show_msg(err);
						$msgTitle.text("Error - List All");
						return;
					}
					let list = JSON.stringify(response);
					show_msg(list);
					$msgTitle.text("Success List All");
				});
			});

		$("#sample_others_save")
			.off("click")
			.on("click", function() {
				hide_msg();
				let user = {
					Id: $others_id.val(),
					Name: $others_name.val()
				};
				self.api.saveSampleOther(user,
					function(success, response) {
						if (!success) {
							let err = JSON.stringify(response);
							show_msg(err);
							$msgTitle.text("Saving Error");
							return;
						}
						$others_id.val("");
						$others_name.val("");
						let ok = JSON.stringify(response);
						show_msg(ok);
						$msgTitle.text("Success Saved");
					}
				);
			});

		$("#sample_others_delete")
			.off("click")
			.on("click", function() {
				hide_msg();
				let user = {
					Id: $others_id.val()
				};
				self.api.removeSampleOther(user,
					function(success, response) {
						if (!success) {
							let err = JSON.stringify(response);
							show_msg(err);
							$msgTitle.text("Deleting Error");
							return;
						}
						$others_id.val("");
						$others_name.val("");
						let ok = JSON.stringify(response);
						show_msg(ok);
						$msgTitle.text("Success Deleted");
					}
				);
			});

		// Seteo eventos del API AGENT
		this.set_events_agent_api();
	};

	main.prototype.set_events_agent_api = function()
	{
		const self = this;
		// ================================= EVENTOS DEL AGENT API  ===========================================

		$("#submitSetInteractionId").click(function() {
			self.m_icApi.SetInteractionId($("#idSetInteractionId").val());
			$("#resultSetInteractionId").val("OK");
		});
	
		$("#submitOffEvent").click(function() {
			self.m_icApi.Off($("#eventNameOff").val());
			$("#resulteventNameOff").val("OK");
		});
	
		// ================================= USER ===========================================
		
		$("#submitSuscribeUserLoggedIn").click(function() {
			self.m_icApi.UserLoggedIn(function(result) {
				$("#resultUserLoggedIn").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeUserLoggedOut").click(function() {
			self.m_icApi.UserLoggedOut(function(result) {
				$("#resultUserLoggedOut").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeUserStateChange").click(function() {
			self.m_icApi.UserStateChanged(function(result) {
				$("#resultUserStateChange").val(JSON.stringify(result));
			});
		});
	
		$("#submitEnableAutomaticPoller").click(function() {
			self.m_icApi.EnableAutomaticPoller({}, function(result) {
				$("#resultEnableAutomaticPoller").val(JSON.stringify(result));
			});
		});
		
		$("#submitDisableAutomaticPoller").click(function() {
			self.m_icApi.DisableAutomaticPoller({}, function(result) {
				$("#resultDisableAutomaticPoller").val(JSON.stringify(result));
			});
		});
		
		$("#submitFirePoller").click(function() {
			var request = {
				sync :	$("#syncFirePoller").is(":checked")? true : false,
				force :	$("#forceFirePoller").is(":checked")? true : false,
				returnData :	$("#returnDataFirePoller").is(":checked")? true : false
			};
			
			self.m_icApi.FirePoller(request, function(result) {
				$("#resultFirePoller").val(JSON.stringify(result));
			});
		});
	
		$("#submitFireChatPoller").click(function() {
			var request = {
				sync :	$("#syncFireChatPoller").is(":checked")? true : false,
				force :	$("#forceFireChatPoller").is(":checked")? true : false,
				returnData :	$("#returnDataFireChatPoller").is(":checked")? true : false
			};
			
			self.m_icApi.FireChatPoller(request, function(result) {
				$("#resultFireChatPoller").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetLoggedUser").click(function() {
			self.m_icApi.GetLoggedUser(function(result) {
				$("#resultGetLoggedUser").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetLoggedUserState").click(function() {
			self.m_icApi.GetLoggedUserState(function(result) {
				$("#resultGetLoggedUserState").val(result);
			});
		});
			
		$("#submitListUserStates").click(function() {
			self.m_icApi.ListUserStates(function(result) {
				$("#resultListUserStates").val(JSON.stringify(result));
			});
		});
		
		$("#submitChangeUserState").click(function() {
			self.m_icApi.ChangeUserState($("#stateChangeUserState").val(), function(result) {
				$("#resultChangeUserState").show();
				$("#resultChangeUserState").text(JSON.stringify(result));
			});
		});
		
		$("#submitGetLoggedUserDefaultCampaign").click(function() {
			self.m_icApi.GetLoggedUserDefaultCampaign(function(result) {
				$("#resultGetLoggedUserDefaultCampaign").val(result);
			});
		});
	
		// ================================= CAMPAIGN ===========================================
		
		$("#submitGetCampaigns").click(function() {
			//var type = {type : "CHAT" };
			//var type= ["CHAT"];
			var request = {type : "CHAT"};
			self.m_icApi.GetCampaigns(request, function(result) {
				$("#msg-resultGetCampaigns").show();
				$("#resultGetCampaigns").text(JSON.stringify(result));
			});
		});
		
		$("#submitGetCampaign").click(function() {
			self.m_icApi.GetCampaign($("#campaignGetCampaign").val(), function(result) {
				$("#resultGetCampaign").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetCampaignAccount").click(function() {
			self.m_icApi.GetCampaignAccount($("#campaignGetCampaignAccount").val(), $("#accountGetCampaignAccount").val(), function(result) {
				$("#resultGetCampaignAccount").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetCampaignDispositions").click(function() {
			self.m_icApi.GetCampaignDispositions($("#campaignGetCampaignDispositions").val(), function(result) {
				$("#resultGetCampaignDispositions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetCampaignLabels").click(function() {
			self.m_icApi.GetCampaignLabels($("#campaignGetCampaignLabels").val(), function(result) {
				$("#resultGetCampaignLabels").val(JSON.stringify(result));
			});
		});
	
		// ================================= GET CONTACT DATA ===========================================
		
		$("#submitGetContact").click(function() {
			var request = {
				id : $("#idGetContact").val()
			};
			self.m_icApi.GetContact(request, function(result) {
				$("#msg-resultGetContact").show();
				$("#resultGetContact").text(JSON.stringify(result));
			});
		});
		
		$("#submitGetContactByExternalId").click(function() {
			var request = {
				id : $("#idGetContactByExternalId").val()
			};
			self.m_icApi.GetContactByExternalId(request, function(result) {
				$("#resultGetContactByExternalId").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetContactByAddress").click(function() {
			var request = {
				address : $("#addressGetContactByAddress").val(),
				addressType : $("#addressTypeGetContactByAddress").val()
			};
			self.m_icApi.GetContactByAddress(request, function(result) {
				$("#resultGetContactByAddress").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionContact").click(function() {
			var request = {
				id : $("#idGetInteractionContact").val()
			};
			self.m_icApi.GetInteractionContact(request, function(result) {
				$("#resultGetInteractionContact").text(JSON.stringify(result));
			});
		});
	
		$("#submitGetContactInteractionsHistory").click(function() {
			var request = {
				contactId : $("#contactIdGetContactInteractionsHistory").val()
			};
			self.m_icApi.GetContactInteractionsHistory(request, function(result) {
				$("#resultGetContactInteractionsHistory").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetInteractionContactInteractionsHistory").click(function() {
			var request = {
				id : $("#interactionIdGetInteractionContactInteractionsHistory").val()
			};
			self.m_icApi.GetInteractionContactInteractionsHistory(request, function(result) {
				$("#resultGetInteractionContactInteractionsHistory").val(JSON.stringify(result));
			});
		});
		// ================================= INTERACTION EVENTS ===========================================
		
		$("#submitSuscribeInteractionAssigned").click(function() {
			self.m_icApi.InteractionAssigned(function(result) {
				$("#resultInteractionAssigned").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionRemovedFromAgent").click(function() {
			self.m_icApi.InteractionRemovedFromAgent(function(result) {
				$("#resultInteractionRemovedFromAgent").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionRequeued").click(function() {
			self.m_icApi.InteractionRequeued(function(result) {
				$("#resultInteractionRequeued").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionTransferred").click(function() {
			self.m_icApi.InteractionTransferred(function(result) {
				$("#resultInteractionTransferred").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionArchived").click(function() {
			self.m_icApi.InteractionArchived(function(result) {
				$("#resultInteractionArchived").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionCanceled").click(function() {
			self.m_icApi.InteractionCanceled(function(result) {
				$("#resultInteractionCanceled").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeInteractionInWrapup").click(function() {
			self.m_icApi.InteractionInWrapup(function(result) {
				$("#resultInteractionInWrapup").val(JSON.stringify(result));
			});
		});
		
		$("#submitSuscribeNewInteractionMessages").click(function() {
			self.m_icApi.NewInteractionMessages(function(result) {
				$("#resultNewInteractionMessages").val(JSON.stringify(result));
			});
		});
		
		// ================================= SEARCH INTERACTIONS ===========================================
		
		$("#submitGetTopQueuedInteractions").click(function() {
			self.m_icApi.GetQueuedInteractions({ full : true }, function(result) {
				$("#resultGetTopQueuedInteractions").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetTopQueuedInteractionsInMemory").click(function() {
			self.m_icApi.GetQueuedInteractions({ full : true, fromMemory : true }, function(result) {
				$("#resultGetTopQueuedInteractionsInMemory").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetAssignedInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetAssignedInteractions").val() || null,
				campaign : $("#campaignGetAssignedInteractions").val() || null,
				account : $("#accountGetAssignedInteractions").val() || null
			};
			self.m_icApi.GetAssignedInteractions(request, function(result) {
				$("#resultGetAssignedInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetAppointmentInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetAppointmentInteractions").val() || null,
				campaign : $("#campaignGetAppointmentInteractions").val() || null
			};
			self.m_icApi.GetAppointmentInteractions(request, function(result) {
				$("#resultGetAppointmentInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetWaitingInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetWaitingInteractions").val() || null,
				campaign : $("#campaignGetWaitingInteractions").val() || null,
				account : $("#accountGetWaitingInteractions").val() || null,
				address : $("#addressGetWaitingInteractions").val() || null,
				contactId : $("#contactIdGetWaitingInteractions").val() || null,
				contactExternalId : $("#contactExternalIdGetWaitingInteractions").val() || null,
				contactName : $("#contactNameGetWaitingInteractions").val() || null,
				text : $("#textGetWaitingInteractions").val() || null,
				ticket : $("#ticketGetWaitingInteractions").val() || null,
				dateFrom : $("#dateFromGetWaitingInteractions").val() || null,
				dateTo : $("#dateToGetWaitingInteractions").val() || null,
				exact : $("#exactGetWaitingInteractions").is(":checked"),
				rows : $("#rowsGetWaitingInteractions").val() || null,
				last : $("#lastGetWaitingInteractions").val() || null
			};
			self.m_icApi.GetWaitingInteractions(request, function(result) {
				$("#resultGetWaitingInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetQueuedInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetQueuedInteractions").val() || null,
				campaign : $("#campaignGetQueuedInteractions").val() || null,
				account : $("#accountGetQueuedInteractions").val() || null,
				address : $("#addressGetQueuedInteractions").val() || null,
				contactId : $("#contactIdGetQueuedInteractions").val() || null,
				contactExternalId : $("#contactExternalIdGetQueuedInteractions").val() || null,
				contactName : $("#contactNameGetQueuedInteractions").val() || null,
				text : $("#textGetQueuedInteractions").val() || null,
				ticket : $("#ticketGetQueuedInteractions").val() || null,
				dateFrom : $("#dateFromGetQueuedInteractions").val() || null,
				dateTo : $("#dateToGetQueuedInteractions").val() || null,
				exact : $("#exactGetQueuedInteractions").is(":checked"),
				rows : $("#rowsGetQueuedInteractions").val() || null,
				last : $("#lastGetQueuedInteractions").val() || null
			};
			self.m_icApi.GetQueuedInteractions(request, function(result) {
				$("#resultGetQueuedInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetArchiveInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetArchiveInteractions").val() || null,
				campaign : $("#campaignGetArchiveInteractions").val() || null,
				account : $("#accountGetArchiveInteractions").val() || null,
				address : $("#addressGetArchiveInteractions").val() || null,
				contactId : $("#contactIdGetArchiveInteractions").val() || null,
				contactExternalId : $("#contactExternalIdGetArchiveInteractions").val() || null,
				contactName : $("#contactNameGetArchiveInteractions").val() || null,
				text : $("#textGetArchiveInteractions").val() || null,
				ticket : $("#ticketGetArchiveInteractions").val() || null,
				dateFrom : $("#dateFromGetArchiveInteractions").val() || null,
				dateTo : $("#dateToGetArchiveInteractions").val() || null,
				disposition : $("#dispositionGetArchiveInteractions").val() || null,
				exact : $("#exactGetArchiveInteractions").is(":checked"),
				rows : $("#rowsGetArchiveInteractions").val() || null,
				last : $("#lastGetArchiveInteractions").val() || null
			};
			self.m_icApi.GetArchiveInteractions(request, function(result) {
				$("#resultGetArchiveInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetAbandonedInteractions").click(function() {
			var request = { 
				full : true,
				type : $("#typeGetAbandonedInteractions").val() || null,
				campaign : $("#campaignGetAbandonedInteractions").val() || null,
				account : $("#accountGetAbandonedInteractions").val() || null,
				address : $("#addressGetAbandonedInteractions").val() || null,
				contactId : $("#contactIdGetAbandonedInteractions").val() || null,
				contactExternalId : $("#contactExternalIdGetAbandonedInteractions").val() || null,
				contactName : $("#contactNameGetAbandonedInteractions").val() || null,
				text : $("#textGetAbandonedInteractions").val() || null,
				ticket : $("#ticketGetAbandonedInteractions").val() || null,
				dateFrom : $("#dateFromGetAbandonedInteractions").val() || null,
				dateTo : $("#dateToGetAbandonedInteractions").val() || null,
				disposition : $("#dispositionGetAbandonedInteractions").val() || null,
				exact : $("#exactGetAbandonedInteractions").is(":checked"),
				rows : $("#rowsGetAbandonedInteractions").val() || null,
				last : $("#lastGetAbandonedInteractions").val() || null
			};
			self.m_icApi.GetAbandonedInteractions(request, function(result) {
				$("#resultGetAbandonedInteractions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetGlobalSearchInteractions").click(function() {
			var request = { 
				full : true,
				agent : $("#agentGetGlobalSearchInteractions").val() || null,
				state : $("#stateGetGlobalSearchInteractions").val() || null,
				type : $("#typeGetGlobalSearchInteractions").val() || null,
				campaign : $("#campaignGetGlobalSearchInteractions").val() || null,
				account : $("#accountGetGlobalSearchInteractions").val() || null,
				address : $("#addressGetGlobalSearchInteractions").val() || null,
				contactId : $("#contactIdGetGlobalSearchInteractions").val() || null,
				contactExternalId : $("#contactExternalIdGetGlobalSearchInteractions").val() || null,
				contactName : $("#contactNameGetGlobalSearchInteractions").val() || null,
				text : $("#textGetGlobalSearchInteractions").val() || null,
				ticket : $("#ticketGetGlobalSearchInteractions").val() || null,
				dateFrom : $("#dateFromGetGlobalSearchInteractions").val() || null,
				dateTo : $("#dateToGetGlobalSearchInteractions").val() || null,
				disposition : $("#dispositionGetGlobalSearchInteractions").val() || null,
				exact : $("#exactGetGlobalSearchInteractions").is(":checked"),
				rows : $("#rowsGetGlobalSearchInteractions").val() || null,
				last : $("#lastGetGlobalSearchInteractions").val() || null
			};
			self.m_icApi.GlobalSearchInteractions(request, function(result) {
				$("#resultGetGlobalSearchInteractions").val(JSON.stringify(result));
			});
		});
	
		// ================================= GET INTERACTION DATA ===========================================
		
		$("#submitGetInteraction").click(function() {
			self.m_icApi.GetInteraction($("#idGetInteraction").val(), function(result) {
				$("#resultGetInteraction").text(JSON.stringify(result));
				$("#msg-resultGetInteraction").show();
			});
		});
		
		$("#submitGetAssignedInteractionByAddress").click(function() {
			var request = {
				type : $("#typeGetAssignedInteractionByAddress").val(),
				campaign : $("#campaignGetAssignedInteractionByAddress").val(),
				account : $("#accountGetAssignedInteractionByAddress").val(),
				address : $("#addressGetAssignedInteractionByAddress").val()
			};
			self.m_icApi.GetAssignedInteractionByAddress(request, function(result) {
				$("#resultGetAssignedInteractionByAddress").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetAssignedInteractionByContact").click(function() {
			var request = {
				type : $("#typeGetAssignedInteractionByContact").val(),
				campaign : $("#campaignGetAssignedInteractionByContact").val(),
				account : $("#accountGetAssignedInteractionByContact").val(),
				contactId : $("#contactGetAssignedInteractionByContact").val(),
				isExternal : $("#isExternalGetAssignedInteractionByContact").is(":checked")
			};
			self.m_icApi.GetAssignedInteractionByContact(request, function(result) {
				$("#resultGetAssignedInteractionByContact").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionDispositions").click(function() {
			self.m_icApi.GetInteractionDispositions($("#idGetInteractionDispositions").val(), function(result) {
				$("#resultGetInteractionDispositions").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionReportingData").click(function() {
			var request = {
				id : $("#idGetInteractionReportingData").val(),
				scope : $("#scopeGetInteractionReportingData").val()
			};
			self.m_icApi.GetInteractionReportingDataFields(request, function(result) {
				$("#resultGetInteractionReportingData").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionReportingField").click(function() {
			var request = {
				id : $("#idGetInteractionReportingField").val(),
				scope : $("#scopeGetInteractionReportingField").val(),
				field : $("#fieldGetInteractionReportingField").val()
			};
			self.m_icApi.GetInteractionReportingDataField(request, function(result) {
				$("#resultGetInteractionReportingField").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionEvents").click(function() {
			self.m_icApi.GetInteractionEvents($("#idGetInteractionEvents").val(), function(result) {
				$("#resultGetInteractionEvents").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionUnreadEvents").click(function() {
			self.m_icApi.GetInteractionUnreadEvents($("#idGetInteractionUnreadEvents").val(), function(result) {
				$("#resultGetInteractionUnreadEvents").val(JSON.stringify(result));
			});
		});
	
		// ================================= INTERACTION ACTIONS ===========================================
		
		$("#submitTakeNextQueued").click(function() {
			var request = {
				type : $("#typeTakeNextQueued").val(),
				campaign : $("#campaignTakeNextQueued").val()
			};
			self.m_icApi.TakeNextQueuedInteraction(request, function(result) {
				$("#resultTakeNextQueued").val(JSON.stringify(result));
			});
		});
		
		$("#submitTakeQueued").click(function() {
			var request = {
				id : $("#idTakeQueued").val()
			};
			self.m_icApi.TakeQueuedInteraction(request, function(result) {
				$("#resultTakeQueued").val(JSON.stringify(result));
			});
		});
		
		$("#submitRequeue").click(function() {
			var request = {
				id : $("#idRequeue").val()
			};
			self.m_icApi.RequeueInteraction(request, function(result) {
				$("#resultRequeue").val(JSON.stringify(result));
			});
		});
		
		$("#submitArchive").click(function() {
			var request = {
				id : $("#idArchive").val(),
				comment : $("#commentArchive").val() || null
			};
			var disposition = $("#dispositionArchive").val();
			var dispositionTree = $("#dispositionTreeArchive").val();
			if (disposition) {
				if(dispositionTree) {
					request.disposition = {
						name : disposition,
						treePath : JSON.parse(dispositionTree)
					};
				}
			}
			if(request.disposition)
			{
				request.disposition.contactName = $("#contactNameArchive").val() || "";
				request.disposition.scheduleDate = new Date($("#dispositionRescheduledToDate").val());
				request.disposition.sameAgent = $("#dispositionRescheduleToSameAgent").is(":checked") || null;
				request.disposition.countryId = $("#dispositionRescheduledToPhoneCountry").val() || "";
				request.disposition.areaId = $("#dispositionRescheduledToPhoneArea").val() || "";
				request.disposition.zip = "-1";
				request.disposition.number = $("#dispositionRescheduledToPhoneNumber").val() || null;
				request.disposition.extension = "";
			}
			self.m_icApi.ArchiveInteraction(request, function(result) {
				$("#resultArchive").val(JSON.stringify(result));
			});
		});
		
		$("#submitReopen").click(function() {
			var request = {
				id : $("#idReopen").val()
			};
			self.m_icApi.ReopenInteraction(request, function(result) {
				$("#resultReopen").val(JSON.stringify(result));
			});
		});		
		
		$("#submitCancel").click(function() {
			var request = {
				id : $("#idCancel").val()
			};
			self.m_icApi.CancelInteraction(request, function(result) {
				$("#resultCancel").val(JSON.stringify(result));
			});
		});
		
		$("#submitComment").click(function() {
			var request = {
				id : $("#idComment").val(),
				comment : $("#commentComment").val()
			};
			self.m_icApi.CommentInteraction(request, function(result) {
				$("#resultComment").val(JSON.stringify(result));
			});
		});
		
		$("#submitSearchTicket").click(function() {
			var request = {
				ticket : $("#ticketSearchTicket").val()
			};
			self.m_icApi.SearchTicket(request, function(result) {
				$("#resultSearchTicket").val(JSON.stringify(result));
			});
		});
		
		$("#submitEmitInteractionTicket").click(function() {
			var request = {
				id : $("#idEmitInteractionTicket").val()
			};
			self.m_icApi.EmitInteractionTicket(request, function(result) {
				$("#resultEmitInteractionTicket").val(JSON.stringify(result));
			});
		});
		
		$("#submitUpdateInteractionLabels").click(function() {
			var request = {
				id : $("#idUpdateInteractionLabels").val(),
				addedLabels : JSON.parse($("#addedLabelsUpdateInteractionLabels").val() || "[]"),
				removedLabels : JSON.parse($("#removedLabelsUpdateInteractionLabels").val() || "[]")
			};
			self.m_icApi.UpdateInteractionLabels(request, function(result) {
				$("#resultUpdateInteractionLabels").val(JSON.stringify(result));
			});
		});
		
		$("#submitReadInteractionEvents").click(function() {
			var request = {
				id : $("#idReadInteractionEvents").val(),
				events : JSON.parse($("#eventsReadInteractionEvents").val() || "[]")
			};
			self.m_icApi.ReadInteractionEvents(request, function(result) {
				$("#resultReadInteractionEvents").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnreadInteractionEvents").click(function() {
			var request = {
				id : $("#idUnreadInteractionEvents").val(),
				events : JSON.parse($("#eventsUnreadInteractionEvents").val() || "[]")
			};
			self.m_icApi.UnreadInteractionEvents(request, function(result) {
				$("#resultUnreadInteractionEvents").val(JSON.stringify(result));
			});
		});
		
		$("#submitForwardInteractionMessages").click(function() {
			var vAddress = {type : "user", address : $("#addressForwardInteractionMessages").val()};
			var request = {
				id : $("#idForwardInteractionMessages").val(),
				address : vAddress,
				events : JSON.parse($("#eventsForwardInteractionMessages").val() || "[]")
			};
			self.m_icApi.ForwardInteractionMessages(request, function(result) {
				$("#resultForwardInteractionMessages").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionMembersToTransfer").click(function() {
			var request = {
				id : $("#idGetInteractionMembersToTransfer").val()
			};
			self.m_icApi.GetInteractionMembersToTransfer(request, function(result) {
				$("#resultGetInteractionMembersToTransfer").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionCampaignsAndAccountsToTransfer").click(function() {
			var request = {
				id : $("#idGetInteractionCampaignsAndAccountsToTransfer").val()
			};
			self.m_icApi.GetInteractionCampaignsAndAccountsToTransfer(request, function(result) {
				$("#resultGetInteractionCampaignsAndAccountsToTransfer").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionAttentionLevelsToTransfer").click(function() {
			var request = {
				id : $("#idGetInteractionAttentionLevelsToTransfer").val()
			};
			self.m_icApi.GetInteractionAttentionLevelsToTransfer(request, function(result) {
				$("#resultGetInteractionAttentionLevelsToTransfer").val(JSON.stringify(result));
			});
		});
		
		$("#submitTransferInteractionToMember").click(function() {
			var request = {
				id : $("#idTransferInteractionToMember").val(),
				member : $("#memberTransferInteractionToMember").val()
			};
			self.m_icApi.TransferInteractionToMember(request, function(result) {
				$("#resultTransferInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitTransferInteractionToNumber").click(function() {
			var request = {
				id : $("#idTransferInteractionToNumber").val(),
				number : $("#numberTransferInteractionToNumber").val()
			};
			self.m_icApi.TransferInteractionToNumber(request, function(result) {
				$("#resultTransferInteractionToNumber").val(JSON.stringify(result));
			});
		});
		
		$("#submitTransferInteractionToCampaign").click(function() {
			var request = {
				id : $("#idTransferInteractionToCampaign").val(),
				campaign : $("#campaignTransferInteractionToCampaign").val(),
				account : $("#accountTransferInteractionToCampaign").val()
			};
			self.m_icApi.TransferInteractionToCampaign(request, function(result) {
				$("#resultTransferInteractionToCampaign").val(JSON.stringify(result));
			});
		});
		
		$("#submitTransferInteractionToAttentionLevel").click(function() {
			var request = {
				id : $("#idTransferInteractionToAttentionLevel").val(),
				attentionLevel : $("#attentionLevelTransferInteractionToAttentionLevel").val()
			};
			self.m_icApi.TransferInteractionToAttentionLevel(request, function(result) {
				$("#resultTransferInteractionToAttentionLevel").val(JSON.stringify(result));
			});
		});
		
		$("#submitConferenceInteractionToMember").click(function() {
			var request = {
				id : $("#idConferenceInteractionToMember").val(),
				member : $("#memberConferenceInteractionToMember").val()
			};
			self.m_icApi.ConferenceInteractionToMember(request, function(result) {
				$("#resultConferenceInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitConferenceInteractionToNumber").click(function() {
			var request = {
				id : $("#idConferenceInteractionToNumber").val(),
				number : $("#numberConferenceInteractionToNumber").val()
			};
			self.m_icApi.ConferenceInteractionToNumber(request, function(result) {
				$("#resultConferenceInteractionToNumber").val(JSON.stringify(result));
			});
		});
		
		
		$("#submitScheduleInteraction").click(function() {
			var request = {
				id : $("#idScheduleInteraction").val(),
				date : $("#dateScheduleInteraction").val(),
				number : $("#numberScheduleInteraction").val()
			};
			self.m_icApi.ScheduleInteraction(request, function(result) {
				$("#resultScheduleInteraction").val(JSON.stringify(result));
			});
		});
	
		// ================================= INTERACTION MESSAGES ===========================================
		
		$("#submitCreateOutgoing").click(function() {
			var request = {
				type : $("#typeCreateOutgoing").val() || null,
				campaign : $("#campaignCreateOutgoing").val() || null,
				account : $("#accountCreateOutgoing").val() || null,
				address : $("#addressCreateOutgoing").val() || null,
				addressType : $("#addressTypeCreateOutgoing").val() || null,
				contactId : $("#contactIdCreateOutgoing").val() || null,
				isExternal : $("#isExternalCreateOutgoing").is(":checked"),
				autosend : $("#autosendCreateOutgoing").is(":checked"),
				message : self.m_icApi.MakeTextMessage({ text : $("#messageCreateOutgoing").val() })
			};
			self.m_icApi.CreateOutgoingInteraction(request, function(result) {
				$("#resultCreateOutgoing").val(JSON.stringify(result));
			});
		});
		
		$("#submitCreateOutgoingMAIL").click(function() {
			var request = {
				type : $("#typeCreateOutgoingMAIL").val() || null,
				campaign : $("#campaignCreateOutgoingMAIL").val() || null,
				account : $("#accountCreateOutgoingMAIL").val() || null,
				address : $("#addressCreateOutgoingMAIL").val() || null,
				addressType : $("#addressTypeCreateOutgoingMAIL").val() || null,
				contactId : $("#contactIdCreateOutgoingMAIL").val() || null,
				isExternal : $("#isExternalCreateOutgoingMAIL").is(":checked"),
				autosend : $("#autosendCreateOutgoingMAIL").is(":checked"),
				message : self.m_icApi.MakeMailMessage({ body : $("#messageCreateOutgoingMAIL").val(), to : $("#addressCreateOutgoingMAIL").val(), subject : $("#subjectCreateOutgoingMAIL").val() })
			};
			self.m_icApi.CreateOutgoingInteraction(request, function(result) {
				$("#resultCreateOutgoingMAIL").val(JSON.stringify(result));
			});
		});
		
		$("#submitCreateOutgoingFromHSM").click(function() {
			var hsmRequest = {
				group : $("#requestgroupCreateOutgoingFromHSM").val(),
				id : $("#requestidCreateOutgoingFromHSM").val(),
				language : $("#requestlanguageCreateOutgoingFromHSM").val(),
				parameters: JSON.parse($("#requestParameterCreateOutgoingFromHSM").val()),
				hsm_buttons : JSON.parse($("#requestButtonsCreateOutgoingFromHSM").val())
				};
			
			var request = {
				type : $("#typeCreateOutgoingFromHSM").val() || null,
				campaign : $("#campaignCreateOutgoingFromHSM").val() || null,
				account : $("#accountCreateOutgoingFromHSM").val() || null,
				address : $("#addressCreateOutgoingFromHSM").val() || null,
				addressType : $("#addressTypeCreateOutgoingFromHSM").val() || null,
				contactId : $("#contactIdCreateOutgoingFromHSM").val() || null,
				autosend : $("#autosendCreateOutgoingFromHSM").is(":checked"),
				message : self.m_icApi.MakeTemplateMessage(hsmRequest)
			};
			self.m_icApi.CreateOutgoingInteraction(request, function(result) {
				$("#resultCreateOutgoingFromHSM").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetAccountTemplateMessageOptions").click(function() {
			var request = {
				campaign : $("#campaignGetAccountTemplateMessageOptions").val() || null,
				account : $("#accountGetAccountTemplateMessageOptions").val() || null
			};
			self.m_icApi.GetAccountTemplateMessageOptions(request, function(result) {
				$("#resultGetAccountTemplateMessageOptions").val(JSON.stringify(result));
			});
		});
	
		$("#submitGetInteractionTemplateMessageOptions").click(function() {
			var interactionId = $("#IdGetInteractionTemplateMessageOptions").val();
			self.m_icApi.GetInteractionTemplateMessageOptions(interactionId, function(result) {
				$("#resultGetInteractionTemplateMessageOptions").val(JSON.stringify(result));
			});
		});
		
		$("#submitOpen").click(function() {
			var request = {
				id : $("#idOpen").val()
			};
			self.m_icApi.OpenInteractionClient(request, function(result) {
				$("#resultOpen").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendMessage").click(function() {
			var request = {
				id : $("#idSendMessage").val(),
				autosend : $("#autosendSendMessage").is(":checked"),
				reopen : $("#reopenSendMessage").is(":checked"),
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendMessage").val() })
			};
			self.m_icApi.SendMessageToInteraction(request, function(result) {
				$("#resultSendMessage").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendAttachment").click(function() {
			$("#attachmentSendAttachment")[0].click();
		});
		$("#attachmentSendAttachment").change(function(event) {
			if (!event.target.files || !event.target.files.length) {
				return;
			}
			
			var file = event.target.files[0];
			var fileId = $("#idSendAttachment").val();
			var section = "-1";
			var request = {id:fileId, section:section, file:file};
			self.m_icApi.UploadFile(request, function(status, data) { 
				console.log("Status: " + status + " Data: " + data); 
				if(status) {
					var request = {
						id : $("#idSendAttachment").val(),
						autosend : $("#autosendSendAttachment").is(":checked"),
						message : self.m_icApi.MakeAttachmentMessage({ 
							type : $("#typeSendAttachment").val(),
							id : data.id,
							text : $("#messageSendAttachment").val()
						})
					};
					self.m_icApi.SendMessageToInteraction(request, function(result) {
						$("#resultSendAttachment").val(JSON.stringify(result));
					});
				}
			});
		});
		
		$("#submitSendSharedFile").click(function() {
			var request = {
				id : $("#idSendSharedFile").val(),
				autosend : $("#autosendSendSharedFile").is(":checked"),
				message : self.m_icApi.MakeSharedFileMessage({ 
					type : $("#typeSendSharedFile").val(),
					id : $("#fileSendSharedFile").val(),
					text : $("#messageSendSharedFile").val()
				})
			};
			self.m_icApi.SendMessageToInteraction(request, function(result) {
				$("#resultSendSharedFile").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetSharedFiles").click(function() {
			var request = {
				type : $("#typeGetSharedFiles").val() || null,
				name : $("#nameGetSharedFiles").val() || null,
				extension : $("#extensionGetSharedFiles").val() || null,
				maxSize : $("#maxSizeGetSharedFiles").val() || null
			};
			self.m_icApi.GetSharedFiles(request, function(result) {
				$("#resultGetSharedFiles").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendMessageToAddress").click(function() {
			var request = {
				type : $("#typeSendMessageToAddress").val() || null,
				campaign : $("#campaignSendMessageToAddress").val() || null,
				account : $("#accountSendMessageToAddress").val() || null,
				address : $("#addressSendMessageToAddress").val() || null,
				addressType : $("#addressTypeSendMessageToAddress").val() || null,
				contactId : $("#contactIdSendMessageToAddress").val() || null,
				isExternal : $("#isExternalSendMessageToAddress").is(":checked"),
				createIfNotFound : $("#createIfNotFoundSendMessageToAddress").is(":checked"),
				autosend : $("#autosendSendMessageToAddress").is(":checked"),
				reopen : $("#reopenSendMessageToAddress").is(":checked"),
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendMessageToAddress").val() })
			};
			self.m_icApi.SendMessageToAddress(request, function(result) {
				$("#resultSendMessageToAddress").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendMessageToContact").click(function() {
			var request = {
				type : $("#typeSendMessageToContact").val() || null,
				campaign : $("#campaignSendMessageToContact").val() || null,
				account : $("#accountSendMessageToContact").val() || null,
				address : $("#addressSendMessageToContact").val() || null,
				addressType : $("#addressTypeSendMessageToContact").val() || null,
				contactId : $("#contactIdSendMessageToContact").val() || null,
				isExternal : $("#isExternalSendMessageToContact").is(":checked"),
				createIfNotFound : $("#createIfNotFoundSendMessageToContact").is(":checked"),
				autosend : $("#autosendSendMessageToContact").is(":checked"),
				reopen : $("#reopenSendMessageToContact").is(":checked"),
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendMessageToContact").val() })
			};
			self.m_icApi.SendMessageToContact(request, function(result) {
				$("#resultSendMessageToContact").val(JSON.stringify(result));
			});
		});
	
		// ================================= CHAT METHODS ===========================================
		$("#submitHangupChat").click(function() {
			var request ={id:$("#idHangupChat").val()};
			
			self.m_icApi.HangupInteraction(request, function(result) {
				$("#resultHangupChat").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetChatInteractionMessages").click(function() {
			self.m_icApi.GetChatInteractionMessages($("#idGetChatInteractionMessages").val(), function(result) {
				$("#resultGetChatInteractionMessages").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetChatBrowsingHistory").click(function() {
			self.m_icApi.GetChatBrowsingHistory($("#idGetChatBrowsingHistory").val(), function(result) {
				$("#resultGetChatBrowsingHistory").val(JSON.stringify(result));
			});
		});
		
		$("#submitSetInteractionFinalizationMessage").click(function() {
			var request = {
				id : $("#idSetInteractionFinalizationMessage").val() || null,
				address : $("#addressSetInteractionFinalizationMessage").val() || null,
				send : $("#sendSetInteractionFinalizationMessage").is(":checked")
			};
			self.m_icApi.SetInteractionFinalizationMessage(request, function(result) {
				$("#resultSetInteractionFinalizationMessage").val(JSON.stringify(result));
			});
		});
	
		// ================================= TWITTER METHODS ===========================================
		
		$("#submitSendTwitterPrivateMessageToInteraction").click(function() {
			var request = {
				id : $("#idSendTwitterPrivateMessageToInteraction").val() || null,
				event : $("#eventSendTwitterPrivateMessageToInteraction").val() || null,
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendTwitterPrivateMessageToInteraction").val() }),
				autosend : $("#autosendSendTwitterPrivateMessageToInteraction").is(":checked"),
				reopen : $("#reopenSendTwitterPrivateMessageToInteraction").is(":checked")
			};
			self.m_icApi.SendTwitterPrivateMessageToInteraction(request, function(result) {
				$("#resultSendTwitterPrivateMessageToInteraction").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendTwitterTweetReplyMessageToInteraction").click(function() {
			var request = {
				id : $("#idSendTwitterTweetReplyMessageToInteraction").val() || null,
				event : $("#eventSendTwitterTweetReplyMessageToInteraction").val() || null,
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendTwitterTweetReplyMessageToInteraction").val() }),
				autosend : $("#autosendSendTwitterTweetReplyMessageToInteraction").is(":checked"),
				reopen : $("#reopenSendTwitterTweetReplyMessageToInteraction").is(":checked")
			};
			self.m_icApi.SendTwitterTweetReplyMessageToInteraction(request, function(result) {
				$("#resultSendTwitterTweetReplyMessageToInteraction").val(JSON.stringify(result));
			});
		});
		
		$("#submitRetweetTwitterMessage").click(function() {
			var request = {
				id : $("#idRetweetTwitterMessage").val() || null,
				event : $("#eventRetweetTwitterMessage").val() || null,
				message : self.m_icApi.MakeTextMessage({ text : $("#messageRetweetTwitterMessage").val() }),
				autosend : $("#autosendRetweetTwitterMessage").is(":checked"),
				reopen : $("#reopenRetweetTwitterMessage").is(":checked")
			};
			self.m_icApi.RetweetTwitterMessage(request, function(result) {
				$("#resultRetweetTwitterMessage").val(JSON.stringify(result));
			});
		});
		
		$("#submitLikeTwitterMessage").click(function() {
			var request = {
				id : $("#idLikeTwitterMessage").val() || null,
				event : $("#eventLikeTwitterMessage").val() || null,
				reopen : $("#reopenLikeTwitterMessage").is(":checked")
			};
			self.m_icApi.LikeTwitterMessage(request, function(result) {
				$("#resultLikeTwitterMessage").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnlikeTwitterMessage").click(function() {
			var request = {
				id : $("#idUnlikeTwitterMessage").val() || null,
				event : $("#eventUnlikeTwitterMessage").val() || null,
				reopen : $("#reopenUnlikeTwitterMessage").is(":checked")
			};
			self.m_icApi.UnlikeTwitterMessage(request, function(result) {
				$("#resultUnlikeTwitterMessage").val(JSON.stringify(result));
			});
		});
	
		// ================================= FACEBOOK METHODS ===========================================
		
		$("#submitSendFacebookCommentReplyMessageToInteraction").click(function() {
			var request = {
				id : $("#idSendFacebookCommentReplyMessageToInteraction").val() || null,
				event : $("#eventSendFacebookCommentReplyMessageToInteraction").val() || null,
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendFacebookCommentReplyMessageToInteraction").val() }),
				autosend : $("#autosendSendFacebookCommentReplyMessageToInteraction").is(":checked"),
				reopen : $("#reopenSendFacebookCommentReplyMessageToInteraction").is(":checked")
			};
			self.m_icApi.SendFacebookCommentReplyMessageToInteraction(request, function(result) {
				$("#resultSendFacebookCommentReplyMessageToInteraction").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendFacebookPrivateReplyMessageToInteraction").click(function() {
			var request = {
				id : $("#idSendFacebookPrivateReplyMessageToInteraction").val() || null,
				event : $("#eventSendFacebookPrivateReplyMessageToInteraction").val() || null,
				message : self.m_icApi.MakeTextMessage({ text : $("#messageSendFacebookPrivateReplyMessageToInteraction").val() }),
				autosend : $("#autosendSendFacebookPrivateReplyMessageToInteraction").is(":checked"),
				reopen : $("#reopenSendFacebookPrivateReplyMessageToInteraction").is(":checked")
			};
			self.m_icApi.SendFacebookPrivateReplyMessageToInteraction(request, function(result) {
				$("#resultSendFacebookPrivateReplyMessageToInteraction").val(JSON.stringify(result));
			});
		});
		
		$("#submitLikeFacebookMessage").click(function() {
			var request = {
				id : $("#idLikeFacebookMessage").val() || null,
				event : $("#eventLikeFacebookMessage").val() || null,
				reopen : $("#reopenLikeFacebookMessage").is(":checked")
			};
			self.m_icApi.LikeFacebookMessage(request, function(result) {
				$("#resultLikeFacebookMessage").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnlikeFacebookMessage").click(function() {
			var request = {
				id : $("#idUnlikeFacebookMessage").val() || null,
				event : $("#eventUnlikeFacebookMessage").val() || null,
				reopen : $("#reopenUnlikeFacebookMessage").is(":checked")
			};
			self.m_icApi.UnlikeFacebookMessage(request, function(result) {
				$("#resultUnlikeFacebookMessage").val(JSON.stringify(result));
			});
		});
	
		// ================================= MAIL METHODS ===========================================
		
		$("#submitReplyInteractionMail").click(function() {
			var request = {
				id : $("#idReplyInteractionMail").val() || null,
				event : $("#eventReplyInteractionMail").val() || null,
				message : self.m_icApi.MakeMailMessage({ 
					body : $("#bodyReplyInteractionMail").val(),
					to : $("#toReplyInteractionMail").val(),
					cc : $("#ccReplyInteractionMail").val(),
					bcc : $("#bccReplyInteractionMail").val()
				}),
				autosend : $("#autosendReplyInteractionMail").is(":checked"),
				reopen : $("#reopenReplyInteractionMail").is(":checked")
			};
			self.m_icApi.ReplyInteractionMail(request, function(result) {
				$("#resultReplyInteractionMail").val(JSON.stringify(result));
			});
		});
		
		$("#submitReplyAllInteractionMail").click(function() {
			var request = {
				id : $("#idReplyAllInteractionMail").val() || null,
				event : $("#eventReplyAllInteractionMail").val() || null,
				message : self.m_icApi.MakeMailMessage({ 
					body : $("#bodyReplyAllInteractionMail").val(),
					to : $("#toReplyAllInteractionMail").val(),
					cc : $("#ccReplyAllInteractionMail").val(),
					bcc : $("#bccReplyAllInteractionMail").val()
				}),
				autosend : $("#autosendReplyAllInteractionMail").is(":checked"),
				reopen : $("#reopenReplyAllInteractionMail").is(":checked")
			};
			self.m_icApi.ReplyAllInteractionMail(request, function(result) {
				$("#resultReplyAllInteractionMail").val(JSON.stringify(result));
			});
		});
		
		$("#submitForwardInteractionMail").click(function() {
			var request = {
				id : $("#idForwardInteractionMail").val() || null,
				event : $("#eventForwardInteractionMail").val() || null,
				message : self.m_icApi.MakeMailMessage({ 
					body : $("#bodyForwardInteractionMail").val(),
					to : $("#toForwardInteractionMail").val(),
					cc : $("#ccForwardInteractionMail").val(),
					bcc : $("#bccForwardInteractionMail").val()
				}),
				autosend : $("#autosendForwardInteractionMail").is(":checked"),
				reopen : $("#reopenForwardInteractionMail").is(":checked")
			};
			self.m_icApi.ForwardInteractionMail(request, function(result) {
				$("#resultForwardInteractionMail").val(JSON.stringify(result));
			});
		});
		
		$("#submitPrintInteractionMail").click(function() {
			var request = {
				id : $("#idPrintInteractionMail").val() || null,
				event : $("#eventPrintInteractionMail").val() || null,
				reopen : $("#reopenPrintInteractionMail").is(":checked")
			};
			self.m_icApi.PrintInteractionMail(request, function(result) {
				$("#resultPrintInteractionMail").val(JSON.stringify(result));
			});
		});
	
		// ================================= PHONE EVENTS ===========================================
		
		$("#submitCallTaken").click(function() {
			self.m_icApi.CallTaken(function(result) {
				$("#resultCallTaken").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallOnRecording").click(function() {
			self.m_icApi.CallOnRecording(function(result) {
				$("#resultCallOnRecording").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallOnHold").click(function() {
			self.m_icApi.CallOnHold(function(result) {
				$("#resultCallOnHold").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallOnMute").click(function() {
			self.m_icApi.CallOnMute(function(result) {
				$("#resultCallOnMute").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallOnSwitch").click(function() {
			self.m_icApi.CallOnSwitch(function(result) {
				$("#resultCallOnSwitch").val(JSON.stringify(result));
			});
		});
		
		
		$("#submitCallOnPreview").click(function() {
			self.m_icApi.CallOnPreview(function(result) {
				$("#resultCallOnPreview").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallRedialing").click(function() {
			self.m_icApi.CallRedialing(function(result) {
				$("#resultCallRedialing").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallRedialAccepted").click(function() {
			self.m_icApi.CallRedialAccepted(function(result) {
				$("#resultCallRedialAccepted").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallRedialCanceled").click(function() {
			self.m_icApi.CallRedialCanceled(function(result) {
				$("#resultCallRedialCanceled").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallTransferring").click(function() {
			self.m_icApi.CallTransferring(function(result) {
				$("#resultCallTransferring").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallTransferConnected").click(function() {
			self.m_icApi.CallTransferConnected(function(result) {
				$("#resultCallTransferConnected").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallOnConference").click(function() {
			self.m_icApi.CallOnConference(function(result) {
				$("#resultCallOnConference").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallConferenceConsultationTrying").click(function() {
			self.m_icApi.CallConferenceConsultationTrying(function(result) {
				$("#resultCallConferenceConsultationTrying").val(JSON.stringify(result));
			});
		});
		
		$("#submitCallConferenceConsultationConnected").click(function() {
			self.m_icApi.CallConferenceConsultationConnected(function(result) {
				$("#resultCallConferenceConsultationConnected").val(JSON.stringify(result));
			});
		});
	
		// ================================= PHONE ===========================================
		
		$("#submitGetPhoneState").click(function() {
			self.m_icApi.GetPhoneState(function(result) {
				$("#resultGetPhoneState").val(result);
			});
		});
		
		$("#submitAnswerCall").click(function() {
			self.m_icApi.AnswerCall({ id : $("#idAnswerCall").val() }, function(result) {
				$("#resultAnswerCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitHangupCall").click(function() {
			self.m_icApi.HangupInteraction({ id : $("#idHangupCall").val() }, function(result) {
				$("#resultHangupCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitHoldCall").click(function() {
			self.m_icApi.HoldCall({ id : $("#idHoldCall").val() }, function(result) {
				$("#resultHoldCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnholdCall").click(function() {
			self.m_icApi.UnholdCall({ id : $("#idUnholdCall").val() }, function(result) {
				$("#resultUnholdCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitMuteCall").click(function() {
			self.m_icApi.MuteCall({ id : $("#idMuteCall").val() }, function(result) {
				$("#resultMuteCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnmuteCall").click(function() {
			self.m_icApi.UnmuteCall({ id : $("#idUnmuteCall").val() }, function(result) {
				$("#resultUnmuteCall").val(JSON.stringify(result));
			});
		});
	
		$("#submitSwitchCall").click(function() {
			self.m_icApi.SwitchCall({ id : $("#idSwitchCall").val() }, function(result) {
				$("#resultSwitchCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitUnswitchCall").click(function() {
			self.m_icApi.UnswitchCall({ id : $("#idSwitchCall").val() }, function(result) {
				$("#resultUnswitchCall").val(JSON.stringify(result));
			});
		});
		
		
		$("#submitRecordCall").click(function() {
			self.m_icApi.StartCallRecord({ id : $("#idRecordCall").val() }, function(result) {
				$("#resultRecordCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitStopRecordCall").click(function() {
			self.m_icApi.StopCallRecord({ id : $("#idStopRecordCall").val() }, function(result) {
				$("#resultStopRecordCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitGetInteractionRecords").click(function() {
			self.m_icApi.GetInteractionRecords($("#idGetInteractionRecords").val(), function(result) {
				$("#resultGetInteractionRecords").val(JSON.stringify(result));
			});
		});
				
		$("#submitAcceptPreviewCall").click(function() {
			self.m_icApi.AcceptCallPreview({ id : $("#idAcceptPreviewCall").val() }, function(result) {
				$("#resultAcceptPreviewCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitCancelPreviewCall").click(function() {
			self.m_icApi.CancelCallPreview({ id : $("#idCancelPreviewCall").val() }, function(result) {
				$("#resultCancelPreviewCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitTransCALLInteractionToMember").click(function() {
			var request = {
				id : $("#idTransCALLInteractionToMember").val(),
				member : $("#memberTransCALLInteractionToMember").val()
			};
			self.m_icApi.TransferInteractionToMember(request, function(result) {
				$("#resultTransCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitConfirmTransCALLInteractionToMember").click(function() {
			self.m_icApi.ConfirmCallTransfer({ id : $("#idConfirmTransCALLInteractionToMember").val() }, function(result) {
				$("#resultConfirmTransCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitCancelTransCALLInteractionToMember").click(function() {
			self.m_icApi.CancelCallTransfer({ id : $("#idCancelTransCALLInteractionToMember").val() }, function(result) {
				$("#resultCancelTransCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitConfirmTransferCall").click(function() {
			self.m_icApi.ConfirmCallTransfer({ id : $("#idConfirmTransferCall").val() }, function(result) {
				$("#resultConfirmTransferCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitCancelTransferCall").click(function() {
			self.m_icApi.CancelCallTransfer({ id : $("#idCancelTransferCall").val() }, function(result) {
				$("#resultCancelTransferCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitConfirmConferenceCall").click(function() {
			self.m_icApi.ConfirmCallConference({ id : $("#idConfirmConferenceCall").val() }, function(result) {
				$("#resultConfirmConferenceCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitCancelConferenceCall").click(function() {
			self.m_icApi.CancelCallConference({ id : $("#idCancelConferenceCall").val() }, function(result) {
				$("#resultCancelConferenceCall").val(JSON.stringify(result));
			});
		});
	
		
		$("#submitConfCALLInteractionToMember").click(function() {
			var request = {
				id : $("#idConfCALLInteractionToMember").val(),
				member : $("#memberConfCALLInteractionToMember").val()
			};
			self.m_icApi.ConferenceInteractionToMember(request, function(result) {
				$("#resultConfCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitConfirmConfCALLInteractionToMember").click(function() {
			self.m_icApi.ConfirmCallConference({ id : $("#idConfirmConfCALLInteractionToMember").val() }, function(result) {
				$("#resultConfirmConfCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		$("#submitCancelConfCALLInteractionToMember").click(function() {
			self.m_icApi.CancelCallConference({ id : $("#idCancelConfCALLInteractionToMember").val() }, function(result) {
				$("#resultCancelConfCALLInteractionToMember").val(JSON.stringify(result));
			});
		});
		
		
		$("#submitRedialCall").click(function() {
			self.m_icApi.RedialCall({ id : $("#idRedialCall").val() }, function(result) {
				$("#resultRedialCall").val(JSON.stringify(result));
			});
		});
		
		$("#submitSendDigitCall").click(function() {
			self.m_icApi.SendCallDigit({ id : $("#idSendDigitCall").val(), digit : $("#digitSendDigitCall").val() }, function(result) {
				$("#resultSendDigitCall").val(JSON.stringify(result));
			});
		});
	};

	//-------------------------
	$(document).ready(function() {
		window.occIntegrations.widget.main = new main();
	});
}());