local ir = require "lua_iragentlink"
local utils = require "shared.apps.utils"
require "LuaOrm.Date"
local m_helpers = require "helpers"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_channelsQueued = {
	CALL= {
		{
			redisKey = "InboundAbandonedInteractionsByCampaignAccount",
			state = "ABANDONED"
		}, -- clave en redis para queued de call en estado Abandoned
		{
			redisKey = "OutboundOpenInteractionsByCampaignAccount",
			state = "APPOINTMENT"
		}  -- clave en redis para queued de call en estado Appointment
	},
	VOICEMAIL = {
		{
			redisKey = "InboundQueuedInteractionsByCampaignAccount"
		}  -- clave de redis para queued de voicemails
	}
}

local function CloseAccountQueuedInteractionsByKey(redis, irLink, account, key)
	local accountId = account.accountId
	local campaignId = account.campaignId
	local proxyType = account.data.proxyType
	local redisKey = string.format(key.redisKey .. ":%s:%s", campaignId, accountId)
	if not redis:exists(redisKey) then return end

	local info = {
		proxyType     = proxyType,
		campaignAtVCC = campaignId,
		account       = accountId,
		otherInfo     = {
			redisKey        = key.redisKey,
			interactionSate = key.state
		}
	}
	local encodedInfo = json.encode(info)
	LogInfo("[JOB.CLEANUP_INTERACTOINS:CloseQueuedInteractionsByChannels] Processing Info %s.",
			encodedInfo)
	local result = irLink:CloseInactiveQueuedInteractions(info)
	if result.result == "OK" then
		LogInfo("[JOB.CLEANUP_INTERACTOINS:CloseQueuedInteractionsByChannels] (END) Processing Info %s",
				encodedInfo)
	else
		LogError("[JOB.CLEANUP_INTERACTOINS:CloseQueuedInteractionsByChannels] ERROR Processing Infot %s, Error: %s",
				 encodedInfo, result.reason)
	end
end

local function CloseAccountQueuedInteractions(redis, irLink, account)
	local proxyType = account.data.proxyType
	local keys = m_channelsQueued[proxyType] or {}
	for _, key in pairs(keys) do
		CloseAccountQueuedInteractionsByKey(redis, irLink, account, key)
	end
end

local function GetAllAccounts()
	local cassandra = inconcert.environment.apps.jobs.cassandra
	local keyspace = cassandra:CassandraConnection("Agent")
	local accountsByCampaign = utils.GetAllColumns({
		keyspace = keyspace,
		superColumns = true,
		columnFamily = "CampaignAccounts",
		startKey = "",
		endKey = ""
	}) or {}
	keyspace:Close()

	local accounts = {}
	for campaignId, campaignAccounts in pairs(accountsByCampaign) do
		for accountId, account in pairs(campaignAccounts) do
			table.insert(accounts, {
				campaignId = campaignId,
				accountId = accountId,
				data = account
			})
		end
	end
	return accounts
end

local function CloseQueuedInteractions (web)
	LogInfo("[JOB.CLEANUP_INTERACTOINS:CloseQueuedInteractions] Running task")
	local redis = inconcert.environment.apps.jobs.redis
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local accounts = GetAllAccounts()
	for _, account in pairs(accounts) do
		CloseAccountQueuedInteractions(redis, irLink, account)
	end

	LogInfo("[JOB.CLEANUP_INTERACTOINS:CloseQueuedInteractions] (END) Running task")
end



------------------------------------------------------------------
-- Aqui ejecutamos los eventos contra EL IR->Interacciones
--   Ej: la finalizacion de CALL(i6) en estado wrapup que se haya configurado un auto finish despues de un tiempo determinado.
--
local REDIS_KEY_AFTER_INTERACTIONS_WORKS = "InteractionsAfterConversationWorks"

local function GetAllInteractionsAfterConversationWorks ()
	local redis = inconcert.environment.apps.jobs.redis
	local allInteractions = redis:hgetall(REDIS_KEY_AFTER_INTERACTIONS_WORKS)

	--LogDebug("[JOB.CLEANUP_INTERACTOINS:GetAllInteractionsAfterConversationWorks] --> allInteractions: %s", #allInteractions)
	local interactionsAfterConversationWorks = {}
 
	for key, encodedInteractionInfo in pairs(allInteractions or {}) do

		--LogDebug("[JOB.CLEANUP_INTERACTOINS:GetAllInteractionsAfterConversationWorks] ----> _:'%s' - encodedInteractionInfo: %s", _, encodedInteractionInfo)

		local interactionInfo = utils.JSON_Decode(encodedInteractionInfo)
		if interactionInfo then
			interactionsAfterConversationWorks[interactionInfo.interactionID] = interactionInfo
		else
			LogError("[JOB.CLEANUP_INTERACTOINS:GetAllInteractionsAfterConversationWorks] Unable to decode key '%s'.", key)
		end
	end

	--LogDebug("JOB.CLEANUP_INTERACTOINS:GetAllInteractionsAfterConversationWorks] key: '%s' ----> interactionsAfterConversationWorks %s", REDIS_KEY_AFTER_INTERACTIONS_WORKS, utils.JSON_Encode(interactionsAfterConversationWorks))

	return interactionsAfterConversationWorks
end

local function FireInteractionsEvents (web)
	--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] (START) Running task")

	local redis = inconcert.environment.apps.jobs.redis
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local HTTP_X_ICC_RID = web.HTTP_X_ICC_RID

	--utils.EndRealtimeJob(redis, "FireInteractionsEvents")
	local autoStart = false
	if not utils.CanRunRealtimeJob(redis, "FireInteractionsEvents", 15, autoStart) then
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] No task")
		return
	end

	utils.StartRealtimeJob(redis, "FireInteractionsEvents")

	local interactionsData = GetAllInteractionsAfterConversationWorks()
	local nowTime = LuaOrm.Date():time()

	local function CanFinish (data)
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] Current State: %s", json.encode(data.state))

		-- 1 - Primero se evalua que ya NO exista una ejecucion de finalizacion automatica para la interaccion.
		if data.state.name == "FINISHED" then
			-- 1.1 - Si existe una ejecucion de finalizacion previa, valida que el tiempo desde que comenzo la misma NO haya superado el tiempo limite del timeout (data.state.timeoutSec)
			--		(2 minutos) este valor viene dado desde el IR.
			---
			local stateTime = LuaOrm.Date(data.state.timestamp):time()
			--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] Current State Finished stateTime %s", stateTime)
			local stateElapsedTimeSec = nowTime - stateTime
			--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] stateElapsedTimeSec %s, data.state.timeoutSec %s", stateElapsedTimeSec, data.state.timeoutSec)
			if stateElapsedTimeSec <= data.state.timeoutSec then
				-- NO - Si el tiempo del tiemout NO ha expirado, cancelo y espero a ver si la misma finaliza.
				return false
			else
				-- SI el tiempo del timeout SI ha expirado, continuo con las demas validaciones para que se envie nuevamente a procesar el finish automatico de la interaccion.
				LogInfo("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] Current Finish has expired. stateElapsedTimeSec %s, state timeoutSec %s, interactionData: %s", stateElapsedTimeSec, data.state.timeoutSec, json.encode(data))
			end
		end

		-- 2 - Segundo valido si el tiempo de la interaccion en estado wrapup ha expirado (segun la configuracion de la cuenta/campana)
		local wrapupTime = LuaOrm.Date(data.timestamp):time()
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] wrapupTime %s", wrapupTime)
		local elapsedTimeSec = nowTime - wrapupTime
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] elapsedTimeSec %s", elapsedTimeSec)
		local afterInteractionWorkSec = data.afterInteractionWorkMs / 1000
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] afterInteractionWorkSec %s", afterInteractionWorkSec)
		if elapsedTimeSec <= afterInteractionWorkSec then
			return false
		end

		return true, {elapsedTimeSec = elapsedTimeSec}
	end

	local interactionsOutofRangeDate
	for interactionId, data in pairs(interactionsData or {}) do
		local success, result = CanFinish(data)
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] afterInteractionWorkSec %s", afterInteractionWorkSec)
		if success then
			interactionsOutofRangeDate = interactionsOutofRangeDate or {}
			local channel = data.channel:upper()
			interactionsOutofRangeDate[channel] = interactionsOutofRangeDate[channel] or {}
			interactionsOutofRangeDate[channel][interactionId] = data
			LogInfo("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] Add Interactions Event. Channel %s interactionId %s, elapsedTimeSec %s", channel, interactionId, result.elapsedTimeSec)
		end
	end

	if interactionsOutofRangeDate then
		for channel, interactions in pairs(interactionsOutofRangeDate) do
			LogInfo("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] Send Command 'FireInteractionsEvents' to IR - channel '%s', interactions ", channel, interactions)
			local response = irLink:SendCommand({
				interactions = interactions,
				RID          = HTTP_X_ICC_RID,
				proxyType    = channel,
				command      = "FireInteractionsEvents"
			})
		end
	end

	utils.EndRealtimeJob(redis, "FireInteractionsEvents")

	--LogDebug("[JOB.CLEANUP_INTERACTOINS:FireInteractionsEvents] (END) Runned task")
end


local function LoadAllAsyncChannelsWithInactivityTimeoutThresholds(fnCallback)
	local cassandra = inconcert.environment.apps.jobs.cassandra
	local keyspace = cassandra:CassandraConnection("Agent")
	local syncChannels = {CHAT = true, CALL = true, IM = true}

	--Busco las columnas InactivityTimeout de todos los CampaignThresholds
	local campaignThresholdsList = utils.GetAllColumns({
		keyspace     = keyspace or "Agent",
		columnFamily = "CampaignThresholds",
		startKey = "",
		endKey = "",
		columns      = {"InactivityTimeout"}
	}) or {}

	keyspace:Close()
	--LogDebug("[JOB.CLEANUP_INTERACTOINS:GetAllAsyncChannelsWithInactivityTimeoutThresholds] has campaign thresholds '%s'", campaignThresholdsList ~= nil)

	for key, value in pairs(campaignThresholdsList) do
		local inactivityTimeoutVal = value and tonumber(value.InactivityTimeout or -1) > 0 and tonumber(value.InactivityTimeout) or nil
		local splitKey = m_helpers.utils.SplitString(key, ":")
		local channel = splitKey[1]
		local fullCampaignId = splitKey[2]

		-- Si el canal tiene el campo InactivityTimeout y el canal no es sincronico 
		if inactivityTimeoutVal and not syncChannels[channel] then
			fnCallback(channel, fullCampaignId, inactivityTimeoutVal)
		end
	end
end

--[[
	Esta funcion procesa todas las interacciones asignadas a un agente que esten inactivas (No se reciben ni envian mensaje)
	Se envian al IR para ejecutar la accion configurada. 
]]--
local function ProcessInactiveInteractions (web)
	LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] (START) Running task, time %s", LuaOrm.Date():time())
	local redis = inconcert.environment.apps.jobs.redis
	local interactionByChannel
	--[[
		Las interacciones se agregan/actualizan a esta key de redis, cuando:
			+ Son asignadas (Estado "TAKEN" o "TRANSFER TAKEN").
			+ Se agrega un nuevo evento.
		Cada vez que se actualiza se modifica el timestamp del ultimo evento o estado asignado.
	]]--

	--Buscar las interacciones inactivas
	local ProcessAssignedInteraction = function (channel, fullCampaignId, inactivityTimeoutVal)
		local now = LuaOrm.Date()
		--Tiempo actual menos tiempo de inactividad es el limite.
		-- Si el timestamp en redis de la interaccion es menos del limite, se considera inactiva 
		local inactiveTimeStamp = now:add(unpack({ 0, 0, 0, 0, 0, -inactivityTimeoutVal})):time()
		local redis_key = "UpdatedAssignedInteractionsByTypeAndCampaign:" .. channel .. ":" .. fullCampaignId

		--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] Search assigned inactive Interactions in redis %s, for channel %s, fullCampaignId %s, inactiveTimeout %s seg - inactiveTimestamp %s", 
		--redis_key, channel, fullCampaignId, inactivityTimeoutVal, inactiveTimeStamp)

		--Busco por rango de score (timestamp).
		local interactions = redis:zrangebyscore(redis_key, "-inf", inactiveTimeStamp) or {}
		
		for _, interactionId in ipairs(interactions) do
			interactionByChannel = interactionByChannel or {}
			interactionByChannel[channel] = interactionByChannel[channel] or {}
			table.insert( interactionByChannel[channel], interactionId )
		end

		--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] Search assigned inactive Interactions for channel %s, fullCampaignId %s - %s", channel, fullCampaignId, json.encode(interactions or {}))
	end
	
	LoadAllAsyncChannelsWithInactivityTimeoutThresholds(ProcessAssignedInteraction)
	--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] InteractionByChannel %s", json.encode(interactionByChannel or {}))
	if interactionByChannel then
		local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
		for proxy, interactions in pairs(interactionByChannel) do
			--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] Sending to IR interactions, channel %s", proxy)
			local response = irLink:SendCommand({
				RID           = web.vars.HTTP_X_ICC_RID,
				interactions = interactions,
				proxyType    = proxy,
				command      = "ProcessInactiveInteractions"
			})
			--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] Sending to IR interactions, response %s", json.encode(response or {}))
		end
	end
	
	LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessInactiveInteractions] (END) Running task, time %s", LuaOrm.Date():time())
end

--[[
	Esta funcion procesa todas las interacciones que no se permitieron asignar por estar fuera de hora la campanna.
	Se envian al IR para asignarlas. 
]]--
local function ProcessUnassignedInteractionsOutOfHours (web)
	local startProcessTime = LuaOrm.Date():time()
	LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessUnassignedInteractionsOutOfHours] (START) Running task, time %s seconds.", utils.GetElapsedTimeSec(startProcessTime))
	local redis = inconcert.environment.apps.jobs.redis

	local allInteractions = redis and redis:hgetall("OutOfHoursUnAssignedInteractions") or {}
	--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessUnassignedInteractionsOutOfHours] interactions before sending to IR %s", json.encode(allInteractions))
	if next(allInteractions) then
		local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
		local response = irLink:SendCommand({
			RID           = web.vars.HTTP_X_ICC_RID,
			command = "ProcessUnassignedInteractionsOutOfHours",
			proxyType = "AGENT"
		})
		allInteractions = redis:hgetall("OutOfHoursUnAssignedInteractions") or {}
		--LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessUnassignedInteractionsOutOfHours] (response: %s) interactions after sending to IR %s", json.encode(response), json.encode(allInteractions))
	end

	LogDebug("[JOB.CLEANUP_INTERACTOINS:ProcessUnassignedInteractionsOutOfHours] (END) Running task, Elapsed time %s seconds.", utils.GetElapsedTimeSec(startProcessTime))
end

function MapUris(module, prefix)
	-- close queued interactions
	module:dispatch_post(CloseQueuedInteractions, prefix .. "/close_queued_interactions/(.*)")
	module:dispatch_get (CloseQueuedInteractions, prefix .. "/close_queued_interactions/(.*)")
	-- fire interactions events (finish automatically wrapped call interactions)
	module:dispatch_post(FireInteractionsEvents , prefix .. "/fire_interactions_events/(.*)" )
	module:dispatch_get (FireInteractionsEvents , prefix .. "/fire_interactions_events/(.*)" )
	-- Execute a configured action (close, queue, transfer), for the inactive interactions
	module:dispatch_post(ProcessInactiveInteractions , prefix .. "/process_inactive_interactions/(.*)" )
	module:dispatch_get (ProcessInactiveInteractions , prefix .. "/process_inactive_interactions/(.*)" )
	-- process unassigned interactions out of hours  
	module:dispatch_post(ProcessUnassignedInteractionsOutOfHours , prefix .. "/process_unassigned_interactions_outofhours/(.*)" )
	module:dispatch_get (ProcessUnassignedInteractionsOutOfHours , prefix .. "/process_unassigned_interactions_outofhours/(.*)" )
	
end