require("orbit")
require "inconcert.orbit.application"
require("inconcert.translators")

local jsonEncoder = require "LuaOrm.Serializers.Json"

local LuaConfigurationControl = require "LuaConfigurationControl"
local cassandra = require "lua_cassandra"

require "luakeystoreclient"
local WrapRedis = require "wrapredis"

local m_appLocation = select(2, ...)
assert(m_appLocation, "should be running under inconcertWebHandler")

module(..., package.seeall, orbit.new, inconcert.orbit.application.Decorate)

_M.model = nil
local model = require "apps.agent.model"

local m_jobsModules = {
	cleanup_interactions = {module = nil},
	process_flows        = {module = nil},
	resources	         = {module = nil},
	nodes                = {module = nil},
	repair_interactions  = {module = nil},
	cleanup_databases	 = {module = nil}
}
--
for name, info in pairs(m_jobsModules) do
	info.module = relativeRequire(name, ...)
end

local configControl = nil
local m_helpers = require "helpers"
local m_hasRepositoryService = nil

local function HasRepositoryService()
	if m_hasRepositoryService == nil then
		m_hasRepositoryService = m_helpers.utils.HasRepositoryService()
	end
	return m_hasRepositoryService
end

-- funcion que es llamada cuando el webserver se inicializa.
function Initialize()
	LogInfo("Starting application 'jobs'.")

	configControl = LuaConfigurationControl.Client {
		type = "JOBS",
		address = inconcert.environment.apps.oauth.config_server.ip,
		port = inconcert.environment.apps.oauth.config_server.port,
		node_address = inconcert.environment.apps.oauth.config_server.myIp,
		node_port = inconcert.environment.apps.oauth.config_server.myPort,
	}

	if HasRepositoryService() then
		configControl:AddServiceDependency("Repository")
	end

	configControl:AddServiceDependency("InteractionRouter")
	configControl:TriggerNodeReady()

	return true
end

--App status control table for the appReadyFilter
--It validates the configuration received from the config server
app_status = {
	actualStatus = "waiting",

	CheckDependencies = function (newConfig, checkList)
		for _, dep in ipairs (checkList) do
			if newConfig[dep] == nil or newConfig:DiffState(dep) == "deleted" then
				LogError("Dependency '%s' is not met", dep)
				app_status.actualStatus = "invalid"
				error("Unmet dependency "..dep)
			end
		end
	end,

	--Actually this is the only function called from the appReadyFilter
	CheckStatus = function ()
		local hasChanged, newConfig = configControl:ConfigurationHasChanged()
		if not hasChanged and app_status.actualStatus ~= "invalid" then
			return app_status.actualStatus
		end

		inconcert.environment.apps.jobs = {}
		newConfig = newConfig or app_status.current
		if HasRepositoryService() then
			app_status.CheckDependencies(newConfig, {"redis", "agentredis", "cassandra", "irLinks", "repository"})

			local nodes = {}
			for key, value, state in newConfig["repository"]:childs() do
				if state ~= "deleted" then
					LogInfo("Connecting to repositoryservice on %s:%s", value.ip, tostring(value.port))
					table.insert(nodes, { ip = value.ip, port = value.port })
				end
			end
			inconcert.environment.apps.jobs.repsrvnodes = next(nodes) and nodes or nil
		else
			app_status.CheckDependencies(newConfig, {"redis", "agentredis", "cassandra", "irLinks"})
		end

		app_status.current = newConfig

		--Configure Redis
		inconcert.environment.apps.jobs.redis = WrapRedis(newConfig["redis"]["host"], tonumber(newConfig["redis"]["port"]), newConfig["redis"]["pwd"])
		--Configure Agent Redis
		inconcert.environment.apps.jobs.agentredis = WrapRedis(newConfig["agentredis"]["host"], tonumber(newConfig["agentredis"]["port"]), newConfig["agentredis"]["pwd"])
		--Configure Cassandra
		for key, value, state in newConfig["cassandra"]:childs() do
			if state ~= "deleted" then
				LogInfo("Connecting to cassandra on %s:%s", value.host, tostring(value.port))
				cassandra.seed(value.host, value.port)
				cassandra.ip = value.host
				cassandra.port = value.port
			end
		end
		inconcert.environment.apps.jobs.cassandra = cassandra
		
		--Configure IR links
		if newConfig:DiffState("irLinks") == "updated" or newConfig:DiffState("irLinks") == "added" then
			local links = {}
			for key, value, state in newConfig["irLinks"]:childs() do
				if state ~= "deleted" then
					inconcert.environment.apps.jobs.irLink = value
				end
			end
		end

		--Mysql
		inconcert.environment.apps.jobs.mysql = newConfig["mysql"]
		app_status.actualStatus = "ready"
		return "ready"
	end
}

-- Mapeo las URIS de los modulos.
--
for name, info in pairs(m_jobsModules) do
	info.module.MapUris(_M,  "")
end
