local ir = require "lua_iragentlink"
local utils = require "shared.apps.utils"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local function AssignedPeerTimeout(irLink, vcc, interactionID)
	LogInfo("AssignedPeerTimeout VCC %s interaction %s", vcc, interactionID)
	irLink:AssignedPeerTimeout({
		proxyType = "CALL",
		VCC = vcc,
		interactionID = interactionID
	})
end

local function GetAllVccs()
	local cassandra = inconcert.environment.apps.jobs.cassandra
	local keyspace = cassandra:CassandraConnection("Agent")
	local vccs = utils.GetAllColumns {
		keyspace     = keyspace,
		columnFamily = "VCCs",
		startKey = "",
		endKey = ""
	} or {}
	keyspace:Close()
	return vccs
end

local function FireVccAssignedPeersTimeoutEvents(redis, irLink, vcc)
	local key = string.format("VccMembersWithAssignedPeers:client:%s", vcc)
	local entries = redis:zrangebyscore(key, "-inf", os.time())
	for _, entry in pairs(entries) do
		AssignedPeerTimeout(irLink, vcc, entry)
		redis:zrem(key, entry)
	end
end

local function FireAssignedPeersTimeoutEvents(web)
	local redis = inconcert.environment.apps.jobs.redis
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	if not utils.CanRunRealtimeJob(redis, "FireAssignedPeersTimeoutEvents", 15) then
		return
	end

	local vccs = GetAllVccs()
	for vcc, _ in pairs(vccs) do
		FireVccAssignedPeersTimeoutEvents(redis, irLink, vcc)
	end
	utils.EndRealtimeJob(redis, "FireAssignedPeersTimeoutEvents")
end

function MapUris(module, prefix)
	module:dispatch_post(FireAssignedPeersTimeoutEvents, prefix.."/assigned_peers_timeout_events/(.*)")
	module:dispatch_get(FireAssignedPeersTimeoutEvents, prefix.."/assigned_peers_timeout_events/(.*)")
end