local temp = os.tmpname();
local ostype = temp:sub(0, 1) == "/" and "linux" or "windows";
os.remove(temp);

if (ostype ~= "linux") then
	require "luarocks.require"
end

local Redis = require "redis"
local crypto = require "crypto"
local json = require "json"
local base64 = require "base64"

local args = require ('lapp') [[
AddKnownPublicKey parameters
	-k,--peerValidationPrivateKey (string) 	Path of the peer validation private key
	-n,--nodeId (string)				The ID of the node that is going to be registered
	-c,--servicePublicKey (string)		Path of the public key that is going to be registered
	-r,--redisIP (string) 				The redis IP
	-p,--redisPort (default 6377)	 	The redis port
	-f,--forceUpdate (default no)		Force the update of the key without human intervention
]]

local pwdRedis= os.getenv("REDIS_PWD")
local redis = Redis.connect(args.redisIP, args.redisPort, pwdRedis)

local kpriv = assert(crypto.pkey.read(args.peerValidationPrivateKey, true))

local f = io.open (args.servicePublicKey, "rb")
kpub = f:read("*a")
f:close()

local sig = assert(crypto.sign('sha256', kpub, kpriv))

local result = {
	key = kpub,
	signature = base64.encode (sig)
}

result = json.encode(result)

local res = redis:hsetnx("KeyStore::PublicKeys", args.nodeId, result)

if not res then

	if args.forceUpdate ~= "yes" then
		print "There is a public key already defined for this nodeId. overwrite? (Y/N)"
		local answer = string.upper(io.read())
		while not (answer == "Y" or answer == "N") do
			answer = string.upper(io.read())
		end
		if answer == "N" then
			return
		end
	end
	
	redis:hset("KeyStore::PublicKeys", args.nodeId, result)
end

local ok, err = pcall (function()
	redis:save()
end)

if redis:hget("KeyStore::PublicKeys", args.nodeId) == result then 
	print "Public key added successfully"
else
	error ("Something went wrong: "..err)
end
