local temp = os.tmpname();
local ostype = temp:sub(0, 1) == "/" and "linux" or "windows";
os.remove(temp);

if (ostype ~= "linux") then
	require "luarocks.require"
end

require "crypto"
require "json"
require "base64"
require "uuid"

local args = require ('lapp') [[
AddKnownPublicKey parameters
	-k,--privateKeyDestination (string) Path to store the new private key
	-q,--publicKeyDestination (string)	Path to store the new public key
	-c,--createCopyTo (default "") 		Path to store the keys protected by the passphrase (recommended)
	-p,--passphrase (default "")		Passphrase to store the private key
	-r,--restoreCopy (default "")		Path to the archive wich contains the protected copy of the keys to restore them
]]

if args.createCopyTo ~= '""' then
	assert (#args.passphrase >= 7, "passphrase must be longer than 6 characters")
end

function readFile (path)
	local f = io.open (path, "rb")
	local content = f:read("*a")
	f:close()
	return content
end

function writeFile (path, content)
	local f = io.open (path, "wb")
	f:write (content)
	f:close()
end

function EnableWriting(path)
	if ostype == "linux" then
		--Se setean los permisos de lectura, escritura y ejecucion
		os.execute ("chmod 700 "..path)
	end
end

function DisableWriting(path)
	if ostype == "linux" then
		--Se setean los permisos de solo lectura
		os.execute ("chmod 400 "..path)
	end
end

if args.restoreCopy ~= '""' then
	print "Restoring keys"
	
	local info = json.decode (readFile (args.restoreCopy))
	
	local passKey = crypto.digest("sha256", args.passphrase, true)
	local initVector = base64.decode(info.initVector)
	
	local private = crypto.decrypt ("aes256", base64.decode(info.privateKey), passKey, initVector)
	local public = crypto.decrypt ("aes256", base64.decode(info.publicKey), passKey, initVector)
	
	assert (private, "Incorrect passphrase")
	assert (public, "Incorrect passphrase")
	
	EnableWriting (args.privateKeyDestination)
	EnableWriting (args.publicKeyDestination)
	
	writeFile (args.privateKeyDestination, private)
	writeFile (args.publicKeyDestination, public)
	
	DisableWriting (args.privateKeyDestination)
	DisableWriting (args.publicKeyDestination)
	
	return
end

print "Generating new keys"

local keys = assert(crypto.pkey.generate('rsa', 2048), "no keys generated")

keys:write(args.publicKeyDestination, args.privateKeyDestination)

local kpub = assert(crypto.pkey.read(args.publicKeyDestination))
local kpriv = assert(crypto.pkey.read(args.privateKeyDestination, true))

DisableWriting (args.privateKeyDestination)
DisableWriting (args.publicKeyDestination)

print "Key pair were generated"

if args.createCopyTo ~= '""' then
	print "Creating protected copy"
	
	local passKey = crypto.digest("sha256", args.passphrase, true)
	local initVector = crypto.hmac.digest("md5", uuid.new(), passKey, true)

	local private = crypto.encrypt ("aes256", readFile(args.privateKeyDestination), passKey, initVector)
	local public = crypto.encrypt ("aes256", readFile(args.publicKeyDestination), passKey, initVector)

	local f = io.open (args.createCopyTo, "wb")
	f:write (json.encode {
		privateKey = base64.encode(private),
		publicKey = base64.encode(public),
		initVector = base64.encode(initVector)
	})
	f:close()
	
	print ("Protected copy was created to ".. args.createCopyTo)
end

