module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local apiQnAMaker = require "qna_maker" -- rocka

m_htmlRender = nil
m_appLocation = nil

-------------------------------------------------------
-- KNOWLEDGEBASE
-------------------------------------------------------

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get/17c545c3-98cd-4e80-8201-15e8d24c4234

	Metodo: GET
		Parametros:
			1) KbId: 17c545c3-98cd-4e80-8201-15e8d24c4234

	Respuesta:
		Status:200 OK
			{
				"qnaDocuments": []
			}
]]--
local function GetKnowledgeBase (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBase] User '%s' Retrieving QnA Maker KnowledgeBase '%s'.", userId, kbId)

	local success, response = apiQnAMaker.DownloadKnowledgebase(kbId, vcc)
	--LogDebug("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBase] From API kbId: '%s', success: '%s' KnowledgeBase: '%s'.", kbId, success, response)
	if success then
		return response
	else
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBase] Error Retrieving QnA Maker KnowledgeBase: '%s' for vcc: '%s'. Error: '%s'", kbId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get_detail/17c545c3-98cd-4e80-8201-15e8d24c4234

	Metodo: GET
		Parametros:
			1) KbId: 17c545c3-98cd-4e80-8201-15e8d24c4234

	Respuesta:
		Status:200 OK
			{
			  "id": "9d091697-fb8c-4ed5-9ac0-35bf8273bfff",
			  "hostName": "https://myqnamakerbot.azurewebsites.net",
			  "lastAccessedTimestamp": "2018-03-16T10:59:46Z",
			  "lastChangedTimestamp": "2018-03-16T10:58:10Z",
			  "lastPublishedTimestamp": "2018-03-16T10:59:56Z",
			  "name": "My QnA Maker Bot",
			  "userId": "03a4f4ce-30a6-4ec6-b436-02bcdf6153e1",
			  "urls": [
				"https://docs.microsoft.com/en-in/azure/cognitive-services/qnamaker/faqs",
				"https://docs.microsoft.com/en-us/bot-framework/resources-bot-framework-faq"
			  ],
			  "sources": [
				"Custom Editorial",
				"SurfaceManual.pdf"
			  ]
			}
]]--
local function GetKnowledgeBaseDetail (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseDetail] User '%s' Retrieving QnA Maker '%s' KnowledgeBase Detail.", userId, kbId)

	local success, response = apiQnAMaker.GetKnowledgebaseDetail(kbId, vcc)
	--LogDebug("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseDetail] From API kbId: '%s', success: '%s', '%s' KnowledgeBase Detail.", kbId, success, response)
	if success then
		return response
	else
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseDetail] Error Retrieving QnA Maker '%s' KnowledgeBase Detail for vcc: '%s'. Error: '%s'", kbId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/vcc/get/

	Metodo: GET
		Parametros:

	Respuesta:
		Status:200 OK
			{
				"qnaDocuments": []
			}
]]--
local function GetVccKnowledgeBase (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:GetVccKnowledgeBase] User '%s' Retrieving QnA Maker VCC '%s' KnowledgeBase.", userId, vcc)

	local success, response = apiQnAMaker.GetVccKnowledgebase(vcc)
	--LogDebug("[KNOWLEDGE_BASE.QNA_MAKER:GetVccKnowledgeBase] From API VCC '%s', success: '%s' KnowledgeBase: '%s'.", vcc, success, response)
	if success then
		return response
	else
		web.status = response and response.code or 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:GetVccKnowledgeBase] Error Retrieving QnA Maker VCC '%s' KnowledgeBase. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/create/

	Metodo: POST
		Data:
			{"name":"KnowledgeBase EMMA & ROCIO"}

	Respuesta:
		Status:200 OK
			{
				"lastActionTimestamp": "2019-06-26T12:41:41Z",
				"resourceLocation": "/knowledgebases/c0a43c12-54ac-4b09-ba76-403d4ab820b7",
				"operationState": "Succeeded",
				"createdTimestamp": "2019-06-26T12:41:38Z",
				"kbId": "c0a43c12-54ac-4b09-ba76-403d4ab820b7",
				"userId": "b78b146162334d4f9d665c014b8d72cf",
				"operationId": "17668972-9c2d-4e2b-89e4-61a04817859c"
			}
]]--
local function CreateKnowledgeBase (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:CreateKnowledgeBase] Error Creating QnA Maker KnowledgeBase for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.name then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:CreateKnowledgeBase] Error Creating QnA Maker KnowledgeBase for vcc: '%s'. Invalid Posted Information '%s'", vcc, web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	local list, total = apiQnAMaker.LoadKnowledgebasesFromCassandra(vcc)
	if total > 0 then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:CreateKnowledgeBase] Error Creating QnA Maker KnowledgeBase for vcc: '%s'. %d database already exists. Only 1 database is allowed per vcc.", vcc, total)
		return {result = "ALREADY EXIST", code = 400}
	end

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:CreateKnowledgeBase] User '%s' Creating QnA Maker KnowledgeBase.", userId)

	local success, response = apiQnAMaker.CreateAndWaitKnowledgeBase(info, vcc)
	if success then
		return response
	else
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:CreateKnowledgeBase] Error Creating QnA Maker KnowledgeBase for vcc: '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/delete/48b866be-f747-45bb-8a80-332a5cf6a0f8

	Metodo: DELETE

	Respuesta:
		Status:200 OK
		[]
]]--
local function DeleteKnowledgeBase (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:DeleteKnowledgeBase] User '%s' Deleting QnA Maker Knowledgebase '%s'.", userId, kbId)

	local success, response = apiQnAMaker.DeleteKnowledgeBase(kbId, vcc)
	if not success then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:DeleteKnowledgeBase] Error Deleting QnA Maker Knowledgebase '%s' for vcc '%s'. Error: '%s'", kbId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/update/48b866be-f747-45bb-8a80-332a5cf6a0f8

	Metodo: PUT
		{
		  "name": "Emmita2",
		  "add": [
			{
			  "answer": "no te interesa !!!",
			  "questions": [
				"cual es tu nombre?",
				"nombre"
			  ]
			}
		  ]
		}
	Respuesta:
		Status:200 OK
		[]
]]--
local function UpdateKnowledgeBase (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:UpdateKnowledgeBase] User '%s' Updating QnA Maker Knowledgebase '%s'.", userId, kbId)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:UpdateKnowledgeBase] Error Updating QnA Maker KnowledgeBase for vcc: '%s'. Invalid Request Information '%s'", vcc, web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	info.kbId = kbId
	local success, response = apiQnAMaker.UpdateKnowledgeBase(info, vcc)

	if not success then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:UpdateKnowledgeBase] Error Updating QnA Maker Knowledgebase '%s' for vcc '%s'. Error: '%s'", kbId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get_operation_detail/17c545c3-98cd-4e80-8201-15e8d24c4234

	Metodo: GET
		Parametros:
			1) operationId: 17c545c3-98cd-4e80-8201-15e8d24c4234

	Respuesta:
		Status:200 OK
			{
				"operationState": "Succeeded",
				"createdTimestamp": "2019-06-26T01:53:10Z",
				"lastActionTimestamp": "2019-06-26T01:53:22Z",
				"resourceLocation": "/knowledgebases/7bd37259-4571-4766-b911-f1f2c4f393bf",
				"userId": "b78b146162334d4f9d665c014b8d72cf",
				"operationId": "ec577e37-f592-46ba-a83a-21f79fef31dc"
			}
]]--
local function GetKnowledgeBaseOperationDetail (web, operationId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseOperationDetail] User '%s' Retrieving QnA Maker KnowledgeBase '%s' Operation Detail.", userId, operationId)

	local success, response = apiQnAMaker.GetOperationDetails(operationId, vcc)
	--LogDebug("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseOperationDetail] From API operationId: '%s', success: '%s' KnowledgeBase: '%s' Operation Detail.", operationId, success, response)
	if success then
		return response
	else
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:GetKnowledgeBaseOperationDetail] Error Retrieving QnA Maker KnowledgeBase: '%s' Operation Detail for vcc: '%s'. Error: '%s'", operationId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/update/48b866be-f747-45bb-8a80-332a5cf6a0f8

	Metodo: PUT
		{
		  "name": "Emmita2",
		  "add": [
			{
			  "answer": "no te interesa !!!",
			  "questions": [
				"cual es tu nombre?",
				"nombre"
			  ]
			}
		  ]
		}
	Respuesta:
		Status:200 OK
		[]
]]--
local function PublishKnowledgeBase (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:PublishKnowledgeBase] User '%s' Publishing QnA Maker Knowledgebase '%s'.", userId, kbId)

	local success, response = apiQnAMaker.PublishKnowledgeBase(kbId, vcc)

	if not success then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:PublishKnowledgeBase] Error Publishing QnA Maker Knowledgebase '%s' for vcc '%s'. Error: '%s'", kbId, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
	-- success (retorna el operationDetail de la kbId)
	return type(response) == "table" and response or {}
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/delete_occ/

	Metodo: DELETE

	Respuesta:
		Status:200 OK
		[]
]]--
local function RemoveKnowledgebaseOnlyCassandra (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:RemoveKnowledgebaseOnlyCassandra] User '%s' Removing QnA Maker Knowledgebase from Cassandra.", userId)

	local success, response = apiQnAMaker.RemoveKnowledgebaseOnlyCassandra(vcc)
	if not success then
		web.status = 400
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:RemoveKnowledgebaseOnlyCassandra] Error Removing QnA Maker Knowledgebase from Cassandra for vcc '%s'. Error: '%s'", vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end

	return response -- success
end

function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[KNOWLEDGE_BASE.QNA_MAKER:Render] Render form QnA Maker with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_qna_maker_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[KNOWLEDGE_BASE.QNA_MAKER:Render] HTML QnA Maker does not exist: %s", html_file)
	end
	return content
end

--[[
****************************************************************
	MAPEO DE URLS
****************************************************************
]]--

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local permissionView  = "view_qna_maker"
	local permissionSave  = "save_qna_maker"
	local permissionErase = "delete_qna_maker"

	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		actionId = actionId or permissionSave
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("knowledge_base", actionId, fn)
	end

	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- KNOWLEDGEBASES
	module:dispatch_get   (HasPermission(GetKnowledgeBase, permissionView)      , p .. "/knowledgebases/get/(.*)"                 )
	module:dispatch_get   (HasPermission(GetKnowledgeBaseDetail, permissionView), p .. "/knowledgebases/get_detail/(.*)"          )
	module:dispatch_get   (HasPermission(GetVccKnowledgeBase, permissionView)   , p .. "/knowledgebases/vcc/get/"                 )
	module:dispatch_post  (HasPermission(CreateKnowledgeBase)                   , p .. "/knowledgebases/create/"                  )
	module:dispatch_delete(HasPermission(DeleteKnowledgeBase, permissionErase)  , p .. "/knowledgebases/delete/(.*)"              )
	module:dispatch_put   (HasPermission(UpdateKnowledgeBase)                   , p .. "/knowledgebases/update/(.*)"              )
	module:dispatch_get   (HasPermission(GetKnowledgeBaseOperationDetail)       , p .. "/knowledgebases/get_operation_detail/(.*)")
	module:dispatch_post  (HasPermission(PublishKnowledgeBase)                  , p .. "/knowledgebases/publish/(.*)"             )
	module:dispatch_delete(HasPermission(RemoveKnowledgebaseOnlyCassandra)      , p .. "/knowledgebases/remove_occ/"              )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
