-------------------------------------------------------------------------------
-- Xavante configuration file.
--
-- The configuration defines the Xavante environment
-- Commented fields are optional and assume default values
--
-- Xavante defines virtualhosts for each site running with Xavante.
-- Each virtualhost can define the handlers for specific files extensions.
-- Xavante currently offers a fileHandler and a CGILuaHandler.
--
-- Xavante configuration can be redefined on the default file structure by the
-- the use of LUA_PATH, see more details in the online documentation.
--
-- Authors: Javier Guerra and Andre Carregal
-- Copyright (c) 2004-2006 Kepler Project
---
-- $Id: config.lua,v 1.4 2007/11/08 22:47:45 carregal Exp $
------------------------------------------------------------------------------
require "xavante.filehandler"
require "xavante.urlhandler"

local wsapi = require "wsapi"
wsapi.inconcertWebHandler = require "wsapi.inconcertWebHandler"

require "inconcert.appMounter"


local Mount = inconcert.appMounter.Mount
inconcert.appMounter.SetMountPrefix("/apps/")

-- Define here where Xavante HTTP documents scripts are located
local webDir = XAVANTE_WEB

local rules = {
	["default"] = {
		Mount{
			match = "^/inconcert/apps/logstorage/.*$",
			package = "logstorage",
			name = "logstorage",
			prefix = "/inconcert/apps/logstorage",
			webDir = webDir,
		},
		{ -- cgiluahandler example
			match = {"%.lp$", "%.lp/.*$", "%.lua$", "%.lua/.*$" },
			with = wsapi.inconcertWebHandler.makeRawSapiHandler(webDir)
		},
		{ -- filehandler example
			match = ".",
			with = xavante.filehandler,
			params = {baseDir = webDir}
		},
	}
}

inconcert = {
	authUser = {
		userName = "burguer",
		virtualContactCenter = "inconcert"
	}
}

return rules