module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local JSON_Decode = utils.JSON_Decode;

local Map = utils.Map;
local Reduce = utils.Reduce;



local SEC = 1000
local MIN = SEC * 60
local HOUR = MIN * 60
local DAY = HOUR * 24
local FIVEMIN = 5 * MIN;
local  WEBRTC_STATISTICS_KEY_PREFIX = "AgentMonitorService::AgentWebRTCStats";
local  SPEEDTEST_RESULTS_KEY_PREFIX = "AgentMonitorService::AgentSpeedTest";
local  BROWSER_HEALTH_CAPTURES_KEY_PREFIX = "AgentMonitorService::BrowserHealth";
local  CAPTURE_STATISTICS_STATUS_PREFIX = "AgentMonitorService::CaptureStatisticsStatus";

local ivalues = {
    second = SEC,
    minute = MIN,
    fivem = FIVEMIN,
    hour = HOUR,
    day = DAY
}


function js2unixts(ts)
    return math.floor(ts / 1000);
end

function ts2str(ts)
    return os.date("%Y-%m-%d %H:%M:%S", js2unixts(ts))
end

function slot(ts, period, slots)
    local svalue = ts - (math.floor(ts / period) * period)
    local ssize = period / slots
    return math.floor(svalue / ssize)
end

-- Debe ser exactamente igual a la del AgentMonitorService

function tsslot(ts, period)
	if period == "hour" then
		return slot(ts, HOUR, 120) 
	elseif period == "day" then
		return slot(ts, DAY, 120) 
	elseif period == "fivem" then
		return slot(ts,FIVEMIN,120*5) -- 500ms
	else
		assert(false)
	end
end



function best_interval(from, to, chunk_type)
    if chunk_type == "tabs.info" then
        return "hour"
    else
        if to - from >= DAY then
            return "hour"
        elseif to - from >= HOUR then
            return "hour"
        else
            return "fivem"
        end
    end
end

function keyfloor(ts, interval)
    local date = os.date("*t", math.floor(ts / 1000))
    if interval == "date" then
        return os.time({ year = date.year, month = date.month, day = date.day }) * 1000
    elseif interval == "hour" then
        return os.time({ year = date.year, month = date.month, day = date.day, hour = date.hour }) * 1000
    elseif interval == "fivem" then
        local hourf = os.time({ year = date.year, month = date.month, day = date.day, hour = date.hour }) * 1000
        local minutef = math.floor((ts % HOUR) / FIVEMIN) * FIVEMIN;
        return hourf + minutef
    elseif interval == "minute" then
        return os.time({ year = date.year, month = date.month, day = date.day, hour = date.hour, min = date.min }) * 1000
    end
end

function rkey(ts, interval)
    if interval == "day" then
        return os.date("%Y%m%d", js2unixts(ts))
    elseif interval == "hour" then
        return os.date("%Y%m%d%H", js2unixts(ts))
    elseif interval == "fivem" then
        local d = os.date("%Y%m%d%H", js2unixts(ts))

        local sliceNum = math.floor((ts % HOUR) / FIVEMIN)
        return d .. string.format("-slice-%02d", sliceNum)
    elseif interval == "minute" then
        return os.date("%Y%m%d%H%M", js2unixts(ts))
    end
end

function rediskeys(from, to, chunk_type, interval)
    local basekey = chunk_type .. ":" .. interval .. ":"
    local keys = {}
    for t = keyfloor(from, interval), to, ivalues[interval] do
        local k = rkey(t, interval)
        keys[k] = basekey .. k
    end
    return keys
end

function calcHistogramValue(hentry, chunk_type, value)
    if chunk_type == "ram.info" then
        local t = hentry.t + value.v
        local c = hentry.c + 1
        local v = math.floor(t / c)
        return { t = t, v = v, c = c }
    elseif chunk_type == "chrome.memory" then
        hentry.c = hentry.c + 1
        hentry.tused = hentry.tused + value.used
        hentry.tlimit = hentry.tlimit + value.limit
        hentry.ttotal = hentry.ttotal + value.total
        hentry.used = hentry.tused / hentry.c
        hentry.limit = hentry.tlimit / hentry.c
        hentry.total = hentry.ttotal / hentry.c
        return hentry;

    elseif chunk_type == "cpu.info" then
        local c = hentry.c + 1
        local v = hentry.v or {}
        for cnum, core in ipairs(value.v) do
            v[cnum] = v[cnum] or { t = 0 }
            v[cnum].t = v[cnum].t + core.v
            v[cnum].v = math.floor(v[cnum].t / c);
        end
        return { v = v, c = c }
    elseif chunk_type == "http.ping" then
        for snum, site in pairs(value) do
            hentry[snum]= hentry[snum] or {t=0;v=0;c=0}
            hentry[snum].c =  hentry[snum].c + 1
            hentry[snum].t = hentry[snum].t + site.v
            hentry[snum].v = hentry[snum].t / hentry[snum].c
        end
        return hentry
    elseif chunk_type == "webrtc.received" or chunk_type == "webrtc.sent" then
        for skey,svalue in pairs(value.v) do
            hentry.c = hentry.c + 1
            hentry.v[skey].t = hentry.v[skey].t + svalue.v
            hentry.v[skey].v = hentry.v[skey].t / hentry.c
        end
        return hentry
    elseif chunk_type == "tabs.info" then
        local hmap = {}
        for _,v in ipairs(hentry) do
            hmap[v.url]=true
        end
        for _,v in ipairs(value) do
            if not hmap[v.url] then
                table.insert(hentry,v)
            end
        end
        return hentry
    end
    assert(false);
end

function histogramDefaultValue(chunk_type)
    if chunk_type == "ram.info" then
        return { t = 0, v = 0, c = 0 }
    elseif chunk_type == "chrome.memory" then
        return {c = 0,
                total = 0,
                limit = 0,
                used = 0,
                ttotal = 0,
                tlimit = 0,
                tused =0}
    elseif chunk_type == "cpu.info" then
        return { v = {}, c = 0 }
    elseif chunk_type == "http.ping" then
        return { }
    elseif chunk_type == "webrtc.received" then
        return { c = 0, v= {kbitsPerSec={v=0;t=0},
                            packetsPerSec={v=0;t=0},
                            packetsLost={v=0;t=0},
                            jitter= {v=0;t=0}}}

    elseif chunk_type == "webrtc.sent" then
        return { c = 0, v= {kbitsPerSec={v=0;t=0},
                            packetsPerSec={v=0;t=0}}}
    elseif chunk_type == "tabs.info" then
        return {}
    end
    assert(false);
end


local function hst2chunk(chunk_type, value)
    if chunk_type == "ram.info" then
        return { used = value.v }
    elseif chunk_type == "chrome.memory" then 
        return {jsHeapSize = {total = value.total,used =value.used,limit=value.limit}}      
    elseif chunk_type == "cpu.info" then
        local usages = {}
        for _, core in ipairs(value.v) do
            table.insert(usages, core.v)
        end
        return { usage = usages }
    elseif chunk_type == "http.ping" then
        local res = {}
        for sitename,sitedata in pairs(value) do
            table.insert(res,{name=sitename,value=sitedata.v})
        end
        return res;
    elseif chunk_type == "webrtc.received" or chunk_type == "webrtc.sent" then
        local res = {}
        for skey,svalue in pairs(value.v) do
            res[skey]=svalue.v
        end
        return res;
    elseif chunk_type == "tabs.info" then
        return value
    end
end
function queryAgentStats(agentId, tsfrom, tsto)
    local vcc = inconcert.authUser.virtualContactCenter;
    local agent_at_vcc = agentId .. "@" .. vcc
    local redis = utils.GetRedisClient();
    local res = {}
    for _, stats in ipairs({
       { chunk_type = "ram.info" },
       { chunk_type = "cpu.info" },
       { chunk_type = "http.ping" },
       { chunk_type = "webrtc.received" },
       { chunk_type = "webrtc.sent"},
       { chunk_type = "tabs.info"},
       { chunk_type = "chrome.memory"}
    }) do
        local interval = best_interval(tsfrom, tsto,stats.chunk_type)
        local chunk_type = stats.chunk_type;
        local redis_data = {}
        local keys = rediskeys(tsfrom, tsto, chunk_type, interval)
        for fkey, key in pairs(keys) do
            local data = redis:hget("AgentMonitorService::CaptureData::" ..agent_at_vcc ,key)
            if data then
                redis_data[fkey] = JSON_Decode(data)
            end
        end
        local hst = {}
        local slot_gap_tolerance = 5
        for ts = tsfrom, tsto, (tsto - tsfrom) / 120 do
            local s = tsslot(math.floor(ts) - keyfloor(ts, interval), interval)
            local rediskey = rkey(keyfloor(ts, interval), interval)
            local v = (redis_data[rediskey] or {})[tostring(s)]
            if not v then
                local n = s
                while not v and n > s - slot_gap_tolerance do
                    v = (redis_data[rediskey] or {})[tostring(n)]
                    n = n - 1
                end
                n = s;
                while not v and n < s + slot_gap_tolerance do
                    v = (redis_data[rediskey] or {})[tostring(n)]
                    n = n + 1
                end
            end
            v = v or histogramDefaultValue(chunk_type)
            hst[ts] = hst[ts] or histogramDefaultValue(chunk_type);
            local vf = calcHistogramValue(hst[ts], chunk_type, v);
            hst[ts] = vf
        end
        for k, v in pairs(hst) do
            local ckvalue = hst2chunk(chunk_type, v)
            if ckvalue then
                table.insert(res, { chunk_type, { value = ckvalue, ts = math.floor(k) } })
            end
        end
    end
    table.sort(res,
        function(a, b)
            return a[2].ts > b[2].ts
        end
    )
    return res;
end
