module(..., package.seeall, inconcert.orbit.application.Decorate)
local cjson = require "cjson"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroContacts = require "apps.allegro.controllers.outboundengine_contacts"
local Fullcontact = require "fullcontact"

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

--[[
	Esta función se encarga de traducir la información que llega desde la BD de 5.5.
	Esto se realiza así, ya que en 5.5 se traduce en los campos calculados del model del ORM, y como el usuario que se encuentra logueado
	es "integrationuser" y su idioma es en inglés, la información siempre llega en este idioma.
	Entonces acá se debe de proceder a realizar la traducción del campo según el idioma del Usuario de i6.
]]--
local function Translate (list, sourceAttr, targetAttr)
	if not list then
		return
	end
	sourceAttr = sourceAttr or "DisplayName"
	targetAttr = targetAttr or "DisplayName"
	_T = m_htmlRender.extension.makeTextTranslator()
	for _, row in pairs(list) do
		row[targetAttr] = _T(row[sourceAttr] or "")
	end
end

local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_CONTACTS:__Search] Loading Contacts in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data

	if hasAllegroIntegration then
		success, data = allegroContacts.LoadPaginatedContacts(encodedInfo)
	else
		local info = web.request
		info.listName = "contacts"
		ExecuteFullcontactMethod(function()
			success, data = utils.ExecuteESQueryForConfigurationViews(info, Fullcontact.Dialer, "SearchContacts")
		end)
	end

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_CONTACTS:__Search] Loaded Contacts. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_CONTACTS:__Search] Error loading Contacts. Err: %s", json.encode(err))

	return err
end

--[[

Return
{
  "VCC": 76,
  "ID": "13B9EE18F8CE4279A5702BBB29932CCE",
  "Name": "Nombre_4",
  "Category": "",
  "Campaign": "",
  "AccountOfficer": "",
  "AccountGroup": "",
  "LastManagementResult": null,
  "LastUpdate": "eee",
  "VIP": 0,
  "IsClient": 0,
  "ImageVIP": {
    "name": "VIP_CONTACT_FALSE",
    "value": ""
  },
  "ImageIsClient": {
    "name": "ISCLIENT_CONTACT_FALSE",
    "value": ""
  },
  "ContactImportPhones": [
    {
      "ContactVCC": 76,
      "ContactID": "13B9EE18F8CE4279A5702BBB29932CCE",
      "Type": "HOME",
      "Country": 598,
      "Area": 1,
      "Zip": -1,
      "Number": "27000000",
      "Extension": "",
      "Status": "ENABLED",
      "ImportName": "Telefono",
      "DisplayStatus": "Enabled",
      "DisplayType": "Home"
    }
  ]
}
]]--
local function __Get (web, contactId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_CONTACTS:__Get] Retrieving Contact %s, user %s, vcc %s%s", contactId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, contact
	if hasAllegroIntegration then
		success, contact = allegroContacts.GetContact(contactId)
		if success and contact then
			-- los phones
			Translate(contact.ContactImportPhones, "Type", "Type")
			Translate(contact.ContactImportPhones, "Status", "Status")
			-- los batches
			Translate(contact.OutboundProcessContacts, "DisplayStatus", "DisplayStatus")
			Translate(contact.OutboundProcessContacts, "DisplayCause", "DisplayCause")
			Translate(contact.OutboundProcessContacts, "DisplayNextDestination", "DisplayNextDestination")
			-- los history
			Translate(contact.ContactResultDetails, "DisplayResult", "DisplayResult")
			Translate(contact.ContactResultDetails, "DisplayStatus", "DisplayStatus")
		end
	else
		ExecuteFullcontactMethod(function()
			success, contact = Fullcontact.Dialer:GetContact(contactId, vcc)
		end)
		if success and contact then
			-- los phones
			Translate(contact.Addresses, "Kind", "DisplayKind")
			Translate(contact.Addresses, "Status", "DisplayStatus")
			-- el proceso y el batch al cual pertenece
			if contact.OutboundProcessId and contact.BatchId then
				local outboundEngineAPI = utils.GetOEApi() -- Obtengo el API del "outbound_engine".
				local exist, process = outboundEngineAPI.native.processes.GetProcessAndBatches(contact.OutboundProcessId, vcc)
				if exist then
					local _T = m_htmlRender.extension.makeTextTranslator()
					-- informacion del proceso
					contact.ProcessInfo = {
						id           = process.id,
						state        = process.state,
						displayState = _T(process.state),
						batch        = nil -- el valor se carga debajo cuando se busca el batch
					}
					-- busco el batch en el array
					for _, batch in ipairs(process.batches or {}) do
						if (batch.id and contact.BatchId )
							and batch.id:lower() == contact.BatchId:lower()
						then
							batch.displayState = _T(batch.state)
							contact.ProcessInfo.batch = batch
							break
						end
					end
				end
			end
		end
	end

	if not success then
		web.status = 404
		return {result = "NOT_FOUND"}
	end

	local encodedContact = cjson.encode(contact) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
	--LogDebug("[OUTBOUNDENGINE_CONTACTS:__Get] retrieve Contact '%s', data: %s", contactId, encodedContact)

	return encodedContact
end

local function __LoadContactsBatchFromProcess (web, processId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_CONTACTS:__LoadContactsBatchFromProcess] Loading Contacts Batch From Process %s, user %s, vcc %s", processId, userId, vcc)

	local success, contactsBatch = allegroContacts.LoadContactsBatchFromProcess(processId)
	if success and contactsBatch then
		return json.encode(contactsBatch)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __Render (web)
	_M.response.DontCache(web)

	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_CONTACTS:__Render] Render form Contacts for user %s,", userId)

	local htmlFile = m_appLocation .. "/pages/frm_contacts_content.html"
	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[OUTBOUNDENGINE_CONTACTS:__Render] HTML Contacts does not exist: %s", htmlFile)
	end

	return content
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local permView  = "view_contacts"
	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix

	-- RENDER FRM
	module:dispatch_get(__Render, p .. "/frm/render/")

	-- CONTACTS
	module:dispatch_post(fn .. __Search, p .. "/search/")
	module:dispatch_get (HasPermission(__Get, permView), p .. "/get/(.+)")

	-- CONTACTS BATCH
	module:dispatch_get (utils.JsonReply .. __LoadContactsBatchFromProcess   , p .. "/batch/load/([^.]*)")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
