module(..., package.seeall, inconcert.orbit.application.Decorate)
local cjson = require "cjson"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroProcesses = require "apps.allegro.controllers.outboundengine_processes"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"

local function GetModel (entityName)
	assert(type(entityName) == "string")
	return utils.GetOEModel(entityName)
end

local function MakeAllegroOtherInfo (httpVars, action)
	-- LogDebug(" 1- httpVars %s", httpVars)
	-- LogDebug(" 2- httpVars.HTTP_X_ICC_RID '%s'", httpVars.HTTP_X_ICC_RID)
	-- LogDebug(" 3- httpVars.HTTP_X_REAL_IP '%s'", httpVars.HTTP_X_REAL_IP)
	local allegroOtherInfo = {}
	if not httpVars then
		return allegroOtherInfo
	end
	allegroOtherInfo.remoteServerIP = httpVars.HTTP_X_REAL_IP
	allegroOtherInfo.action = action
	return allegroOtherInfo
end

--[[
	Esta función se encarga de traducir la información que llega desde la BD de 5.5.
	Esto se realiza así, ya que en 5.5 se traduce en los campos calculados del model del ORM, y como el usuario que se encuentra logueado
	es "integrationuser" y su idioma es en inglés, la información siempre llega en este idioma.
	Entonces acá se debe de proceder a realizar la traducción del campo según el idioma del Usuario de i6.
]]--
local function Translate (list, sourceAttr, targetAttr)
	if not list then
		return
	end
	sourceAttr = sourceAttr or "DisplayName"
	targetAttr = targetAttr or "DisplayName"
	_T = m_htmlRender.extension.makeTextTranslator()
	for _, row in pairs(list) do
		row[targetAttr] = _T(row[sourceAttr] or "")
	end
end

--------------------------------------
-- En esta tabla se encuentran los metodos del outbound processes de VCCs SIN Allegro.
local m_outboundProcessesNative = {}

m_outboundProcessesNative.AreAllVccProcessesReady = function(self, info)
	return m_outboundEngineClient.processes:AreAllVccProcessesReady(info)
end

m_outboundProcessesNative.LoadProcesses = function(self, ksOE, vccId)
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	return GetModel("OutboundProcesses").LoadProcesses(vcc, ksOE)
end

m_outboundProcessesNative.LoadProcessesAndBatches = function(self, ksOE, vccId)
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	return GetModel("OutboundProcesses").LoadProcessesAndBatches(vcc, ksOE)
end

m_outboundProcessesNative.GetProcess = function(self, processId, ksOE, vccId)
	assert(processId)
	processId = processId:lower()
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local process = GetModel("OutboundProcesses").GetProcess(vcc, processId, ksOE)
	if utils.TableIsEmpty(process) then
		local err = {reult = "DOES NOT EXIST"}
		return false, err
	end
	return true, process
end

m_outboundProcessesNative.GetProcessAndBatches = function(self, processId, vccId, ksOE)
	assert(processId)
	processId = processId:lower()
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local process = GetModel("OutboundProcesses").GetProcessAndBatches(vcc, processId, ksOE)
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:GetProcessAndBatches] ----> processId: %s vcc: %s, process: %s", processId, vcc, utils.JSON_Encode(process))
	if utils.TableIsEmpty(process) then
		local err = {reult = "DOES NOT EXIST"}
		return false, err
	end
	return true, process
end

m_outboundProcessesNative.LoadBatchesOfProcess = function(self, processId, vccId, ksOE)
	assert(processId)
	processId = processId:lower()
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local batches = GetModel("OutboundProcesses").LoadBatchesOfProcess(vcc, processId, ksOE)
	return true, batches
end

m_outboundProcessesNative.GetRunningBatchActions = function(self, processId, vccId)
	assert(processId)
	processId = processId:lower()
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local runningBatchActions = GetModel("OutboundProcesses").GetRunningBatchActions(processId, vcc)
	return true, runningBatchActions
end

m_outboundProcessesNative.GetStatusCause = function(self, processId, vccId)
	assert(processId)
	processId = processId:lower()
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	return GetModel("OutboundProcesses").GetStatusCause(processId, vcc)
end

m_outboundProcessesNative.GetProcessStatus = function(processId)
	assert(processId)
	processId = processId:lower()
	local vcc = inconcert.authUser.virtualContactCenter
	local status = GetModel("OutboundProcesses").GetProcessStatus(vcc, processId)
	if not status then
		local err = {reult = "DOES NOT EXIST"}
		return false, err
	end
	return true, status
end

m_outboundProcessesNative.RemoveCampaignOEConfig = function(self, vcc, campaignId, ksAgent)
	assert(vcc)
	assert(campaignId)
	-- en nil el tercer parametro elimina por completo la relacion (columna OEConfig) en campaigns.
	GetModel("Campaigns").UpdateCampaignOEConfig(vcc, campaignId, nil, ksAgent)
end

m_outboundProcessesNative.UpdateCampaignOEConfig = function(self, vcc, process, dataBaseProcessCampaignAndAccounts, ksAgent)
	assert(process.configuration)
	assert(process.configuration.campaign)
	assert(process.configuration.channels)
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:UpdateCampaignOEConfig] --> process '%s', dataBaseProcessCampaignAndAccounts: %s", json.encode(process.configuration), json.encode(dataBaseProcessCampaignAndAccounts))
	if dataBaseProcessCampaignAndAccounts then
		if process.configuration.campaign and dataBaseProcessCampaignAndAccounts.campaign then
			if process.configuration.campaign:lower() ~= dataBaseProcessCampaignAndAccounts.campaign:lower() then
				local oldCampaignId = dataBaseProcessCampaignAndAccounts.campaign
				self:RemoveCampaignOEConfig(vcc, oldCampaignId, ksAgent)
			end
		end
	end
	local OEConfig = {
		processId = process.id,
		campaign = process.configuration.campaign,
		channels = {}
	}
	for channelId, channelConfig in pairs(process.configuration.channels) do
		OEConfig.channels[channelId] = {
			account = channelConfig.account
		}
	end
	GetModel("Campaigns").UpdateCampaignOEConfig(vcc, process.configuration.campaign, OEConfig, ksAgent)
	return true
end

m_outboundProcessesNative.SaveProcess = function(self, process, notMustValidate, ksAgent, ksProv, ksOE)
	assert(process)
	local vcc = process.__vcc or inconcert.authUser.virtualContactCenter
	LogDebug("[OUTBOUNDENGINE_PROCESSES.m_outboundProcessesNative:SaveProcess] --> Saving OE process: '%s', vcc: '%s'.", process.id, vcc)
	local success
	if not notMustValidate then
		success, process = self:ValidateProcessToSave(process, ksAgent, ksProv)
		if not success then
			local err = process
			return false, err
		end
	end
	-- atributo utilizado exclusivamente desde las migraciones; como la misma se realiza desde un vcc "system", el atributo debe de viajar segun el vcc que se se este migrando.
	-- el mismo NO debe de ser almacenado.
	process.__vcc = nil
	local success, dataBaseProcessCampaignAndAccounts = self.GetProcessCampaignAndAccounts(process.id, ksOE, vcc)
	local success, data = GetModel("OutboundProcesses").SaveProcess(vcc, process, ksOE)
	if not success then
		local err = data
		return false, err
	end
	self:UpdateCampaignOEConfig(vcc, process, dataBaseProcessCampaignAndAccounts, ksAgent)
	return true, data
end

m_outboundProcessesNative.DeleteProcess = function(self, processId, vccId, ksOE, ksAgent)
	assert(processId)
	processId = processId:lower()
	-- valido si el proceso y sus batches asociados se pueden eliminar
	local success, currentProcess = self:CanDeleteProcess(processId, vccId, ksOE, ksAgent)
	if not success then
		local err = currentProcess
		return false, err
	end
	-- elimino los batches del proceso
	for _, batch in pairs(currentProcess.batches or {}) do
		local batchName = batch.configuration.name
		local success, result = m_outboundEngineClient.batches:Destroy(processId, batchName)
		if not success then
			return false, {type = "UNABLE TO DELETE PROCESS - BATCH IN USE", processId = processId, batchName = batchName, info = result}
		end
	end
	-- elimino el proceso
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local success, err = GetModel("OutboundProcesses").DeleteProcess(vcc, processId, ksOE)
	if not success then
		return false, err
	end
	-- elimino de la campana la referencia al proceso
	self:RemoveCampaignOEConfig(vcc, currentProcess.configuration.campaign, ksAgent)
	return true
end

m_outboundProcessesNative.IsProcessAssignedToCallAccount = function(self, processId, account)
	if not account.incomingConfig then return false end

	if type(account.incomingConfig) == "string" then
		account.incomingConfig = json.decode(account.incomingConfig)
	end
	if not account.incomingConfig.enabled or not account.incomingConfig.ContactCenter then
		return false
	end

	local contactCenter = account.incomingConfig.ContactCenter
	return contactCenter.DialerCallbackOnTimeoutProcess == processId or
		contactCenter.VirtualholdCallbackProcess == processId
end

m_outboundProcessesNative.IsProcessAssignedToAccount = function(self, processId, account)
	if account.proxyType == "CALL" then return self:IsProcessAssignedToCallAccount(processId, account) end
	return false
end

m_outboundProcessesNative.IsProcessAssignedToAccounts = function(self, processId, vcc, ksAgent)
	local vccCampaignsAccounts = GetModel("Campaigns").GetVccCampaignsAccounts(vcc, ksAgent)
	for campaignId, campaignAccounts in pairs(vccCampaignsAccounts) do
		for accountId, account in pairs(campaignAccounts) do
			if self:IsProcessAssignedToAccount(processId, account) then
				return true, { campaignId = campaignId, accountId = accountId, accountName = account.name }
			end
		end
	end
	return false
end

m_outboundProcessesNative.CanDeleteProcess = function(self, processId, vccId, ksOE, ksAgent)
	assert(processId)
	processId = processId:lower()
	local exist, currentProcess = self:GetProcessAndBatches(processId, vccId, ksOE)
	if not exist then
		return false, {type = "PROCESS DOES NOT EXIST", processId = processId}
	end
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	local isRunning, result = m_outboundEngineClient.processes:IsRunning(processId, vcc)
	if isRunning then
		return false, {type = "PROCESS IS RUNNING", processId = processId, info = result}
	end
	-- valido si se pueden eliminar todos los batches del proceso.
	for _, batch in pairs(currentProcess.batches or {}) do
		local batchName = batch.configuration.name
		local batchInfo = {processId = processId, batchName = batchName}
		local success, result = m_outboundEngineClient.batches:CanDestroy(batchInfo)
		if not success then
			return false, {type = "UNABLE TO DELETE BATCH", processId = processId, batchName = batchName, info = result}
		end
	end
	-- valido contra cuentas
	local assigned, result = self:IsProcessAssignedToAccounts(processId, vcc, ksAgent)
	if assigned then
		return false, {
			type = "PROCESS IS ASSIGNED TO ACCOUNT", processId = processId, accountName = result.accountName, info = result
		}
	end
	return true, currentProcess
end

m_outboundProcessesNative.ValidateProcessToSave = function(self, process, ksAgent, ksProv, ksOE)
	assert(type(process) == "table")
	assert(type(process.id) == "string")
	assert(process.id:len() > 0)
	-- el atributo "__vcc", es utilizado desde las migraciones. dicho atributo no sera almacenado
	local vcc = process.__vcc or inconcert.authUser.virtualContactCenter
	process.vcc = vcc
	process.id = utils.AppendVcc(utils.GetId(process.id), vcc):lower()
	local exist, currentProcess = self:GetProcess(process.id, ksOE, vcc)
	local isNewProcess = process.isNew
	local isRunning
	local fullCampaignId = utils.AppendVcc(utils.GetId(process.configuration.campaign), vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:m_outboundProcessesNative.ValidateProcessToSave]--> Validating OE Process to save -> processId: '%s', fullCampaignId: '%s', vcc: '%s', ", process.id, fullCampaignId, vcc)
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:m_outboundProcessesNative.ValidateProcessToSave] ===> 1.1 - process: %s", json.encode(process))

	-- valido que la campana posteda exista en la BD.
	local campaignExist = GetModel("Campaigns").GetSpecificColumn(fullCampaignId, "Id", ksAgent, vcc)
	if not campaignExist then
		return false, {type = "UNABLE TO UPDATE PROCESS - CAMPAIGN DOES NOT EXIST", processId = process.id, campaignId = fullCampaignId}
	end
	-- valida que la campana tenga cuentas.
	local campaignAccounts = GetModel("Campaigns").GetCampaignAccounts(fullCampaignId, ksAgent, nil, vcc)
	if utils.TableIsEmpty(campaignAccounts) then
		return false, {type = "UNABLE TO UPDATE PROCESS - CAMPAIGN HAS NOT ACCOUNTS", processId = process.id, campaignId = fullCampaignId}
	end
	-- valida las cuentas
	for channel, channelInfo in pairs(process.configuration.channels or {}) do
		local accountInfo = campaignAccounts[channelInfo.account]
		if not accountInfo then
			return false, {type = "UNABLE TO UPDATE PROCESS - ACCOUNT DOES NOT EXIST", processId = process.id, campaignId = fullCampaignId, accountId = channelInfo.account}
		end
		local systemController = require "apps.outbound_engine.controllers.system"
		local success, accountErrType = systemController.ValidateIncludeAccount(accountInfo, ksProv)
		if not success then
			return false, {type = "UNABLE TO UPDATE PROCESS - ACCOUNT IS INVALID", accountErrType = accountErrType, processId = process.id, campaignId = fullCampaignId, accountId = channelInfo.account}
		end
	end
	-- validaciones de procesos nuevos o actualizaciones
	if isNewProcess then
	-- creando un nuevo proceso
		if exist then
			return false, {type = "PROCESS ALREADY EXIST", processId = process.id}
		end
		-- sanitize values
		process.state = "STOPPED"
	else
	-- actualizando un proceso
		if not exist then
			return false, {type = "PROCESS DOES NOT EXIST", processId = process.id}
		end
		local result
		isRunning, result = m_outboundEngineClient.processes:IsRunning(process.id, vcc)
		-- if isRunning then
		-- 	return false, {type = "PROCESS IS RUNNING", processId = process.id, info = result}
		-- end
	end
	process.isNew = nil
	local currentProcessCampaignId = (fullCampaignId or ""):lower()
	local vccProcesses = self:LoadProcesses(ksOE, vcc)
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:m_outboundProcessesNative.ValidateProcessToSave] ===> 1.2 - vccProcesses: %s", json.encode(vccProcesses))
	for vccProcessId, vccProcess in pairs(vccProcesses or {}) do
		if vccProcess.configuration then
			vccProcessId = vccProcessId:lower()
			local vccProcessCampaignId = (vccProcess.configuration.campaign or ""):lower()
			-- valido que la campana no se encuentre utilizada en otro proceso.
			if currentProcessCampaignId == vccProcessCampaignId then
				if isNewProcess then
					return false, {type = "UNABLE TO CREATE PROCESS - CAMPAIGN IS ASSIGNED IN ANOTHER PROCESS", processId = process.id, campaignId = currentProcessCampaignId, otherProcessId = vccProcessId}
				else
					if vccProcessId ~= process.id then
						return false, {type = "UNABLE TO UPDATE PROCESS - CAMPAIGN IS ASSIGNED IN ANOTHER PROCESS", processId = process.id, campaignId = currentProcessCampaignId, otherProcessId = vccProcessId}
					end
				end
			end
			-- valida que NO se pueda cambiar la campana de un proceso que se encuentre running.
			if not isNewProcess then
				if isRunning then
					if vccProcessId == process.id then
						if currentProcessCampaignId ~= vccProcessCampaignId then
							return false, {type = "UNABLE TO UPDATE PROCESS - CAN NOT CHANGE CAMPAIGN IF PROCESS IS RUNNING", processId = process.id, campaignId = currentProcessCampaignId, vccProcessCampaignId = vccProcessCampaignId}
						end
					end
				end
			end
		end
	end
	-- ok
	return true, process
end

m_outboundProcessesNative.GetProcessCampaignAndAccounts = function(processId, ksOE, vcc)
	assert(processId)
	local exist, process = m_outboundProcessesNative:GetProcess(processId, ksOE, vcc)
	if not exist then
		local err = process
		return false, err
	end
	if not process.configuration then
		local err = {reult = "INVALID PROCESS CONFIGURATION"}
		return false, err
	end
	return true, {
		campaign = process.configuration.campaign,
		channels = process.configuration.channels
	}
end

---------------------------------------

local function ListProcesses (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[OUTBOUNDENGINE_PROCESSES:ListProcesses] Loading Outbound Processes in '%s' vcc.", vcc)

	local processes = m_outboundProcessesNative:LoadProcesses()
	local processIds = {}
	for processId, _ in pairs(processes) do
		table.insert(processIds, processId)
	end
	return json.encode(processIds)
end

local function FilterProcessesByUserCampaigns (result)
	-- Only store the processes that belongs to a user campaigns
	if next(result) ~= nil then
		local keyspace      = utils.GetDatasource():CassandraConnection( "Agent" )
		local hasPermission = permissions.IsPermissionEnabledForLoggedUser("administration", "admin_restricted", keyspace)

		LogInfo("[OUTBOUNDENGINE_PROCESSES:FilterProcessesByUserCampaigns] Checking if outbounding process belongs to any user campaign")

		if hasPermission and not inconcert.authUser.api then
			local vcc           = inconcert.authUser.virtualContactCenter
			local userCampaigns = GetModel("Campaigns").GetUserCampaigns(vcc, inconcert.authUser.userName, keyspace)
			for id, row in pairs(result) do
				local belongs = false
				for campaignId in pairs(userCampaigns) do
					if row.configuration.campaign == campaignId then
						belongs = true
						break
					end
				end
				if not belongs then
					LogInfo("[OUTBOUNDENGINE_PROCESSES:FilterProcessesByUserCampaigns] Process: %s not belongs to a user campaign", id)
					result[id] = nil
				end
			end
		end
		keyspace:Close()
	end
end

local function SearchProcesses (web)
	_T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] Loading Outbound Processes in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data

	if hasAllegroIntegration then
		success, data = allegroProcesses.LoadPaginatedProcesses(encodedInfo)
		if success and data then
			for _, row in pairs(data.rows or {}) do
				row.DisplayActiveRule = _T(row.DisplayActiveRule)
			end
			return data
		end
	else
		local info = web.request
		local rows = m_outboundProcessesNative:LoadProcesses()

		FilterProcessesByUserCampaigns(rows)
		LogInfo("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] 1 - Filtering rows")
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] ===> 1.1 - rows: %s", json.encode(rows))

		local totalUnfilteredRows = 0
		local totalAdded = 0
		local filteredRows = {}

		for id, row in pairs(rows) do
			totalUnfilteredRows = totalUnfilteredRows + 1
			--row.Id = row.id
			--row.Description = row.Description or row.Id
			if utils.MatchFilter(row, info) then
				totalAdded = totalAdded + 1
				local _, runningBatchActions = m_outboundProcessesNative:GetRunningBatchActions(id, vcc)
				row.runningBatchActions = runningBatchActions and runningBatchActions.actions or {}
				row.statusCause = m_outboundProcessesNative:GetStatusCause(id, vcc)
				table.insert(filteredRows, row)
			end
		end

		--LogInfo("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] 2 - Sorting rows %d", #filteredRows)
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] ===> 2.1 - filteredRows: %s", json.encode(filteredRows or {}))

		--include only rows that belong to logged user vcc
		local response = {
			rows = filteredRows,
			last = "",
			total_rows = totalAdded,
			total_unfiltered_rows = totalUnfilteredRows
		}
		local responseParams = {
			MakeSortingValue = function(row)
				assert(row)
				return tostring(row.description or row.id):lower()
			end
		}
		data = utils.MakeSearchResponse(response, info, responseParams)
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] ===> 3 - data: %s", json.encode(data or {}))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:SearchProcesses] Error loading Outbound Processes. Err: %s", json.encode(err))

	return err
end

--[[

Return

]]--
local function GetProcess (web, processId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:GetProcess] Retrieving Process %s, user %s, vcc %s%s", processId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, process

	if hasAllegroIntegration then
		success, process = allegroProcesses.GetProcess(processId)
	else
		success, process = m_outboundProcessesNative:GetProcessAndBatches(processId)
	end

	if success and process then
		return json.encode(process)
	end

	LogWarning("[OUTBOUNDENGINE_PROCESSES:GetProcess] Unable to retrive process '%s' of vcc '%s'. Process does not exist.", processId, vcc)
	web.status = 404
	return {result = "NOT_FOUND"}
end

local function DeleteProcess (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "DELETE")
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:DeleteProcess] Deleting Process in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local disabledPermission = permissions.IsPermissionEnabledForLoggedUser("administration", "admin_restricted", nil, ksAgent)

	if disabledPermission and not inconcert.authUser.api then
		ksAgent:Close()
		web.status = 403
		LogError("[OUTBOUNDENGINE_PROCESSES:DeleteProcess] Unable to delete process. You don't have permissions.")
		return {type = "FORBIDDEN"}
	end

	local processToDelete = json.decode(encodedInfo)
	local success, data
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, ksAgent)
	local processId = processToDelete[1]

	if hasAllegroIntegration then
		local processInfo = {
			processId = processId
		}
		local encodedInfo = json.encode(processInfo)
		success, data = allegroProcesses.DeleteProcess(encodedInfo, allegroOtherInfo)
	else
		local ksOE = utils.GetDatasource():CassandraConnection("OutboundEngine")
		success, data = m_outboundProcessesNative:DeleteProcess(processId, nil, ksOE, ksAgent)
		ksOE:Close()
	end
	ksAgent:Close()

	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:DeleteProcess] Success deleted Process %s.", processId)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:DeleteProcess] Error deleting Process %s. Err: %s", processId, json.encode(err))

	return err
end

local function StartProcess (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:StartProcess] Starting Process in '%s' vccc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data

	if hasAllegroIntegration then
		success, data = allegroProcesses.StartProcess(encodedInfo)
	else
		local process = json.decode(encodedInfo)
		success, data = m_outboundEngineClient.processes:Start(process.process_id, vcc)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:StartProcess] Success Started Process %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:StartProcess] Error Starting Process %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function StopProcess (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:StopProcess] Stopping Process in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data

	if hasAllegroIntegration then
		success, data = allegroProcesses.StopProcess(encodedInfo)
	else
		local process = json.decode(encodedInfo)
		success, data = m_outboundEngineClient.processes:Stop(process.process_id, vcc)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:StopProcess] Success Stopped Process %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:StopProcess] Error Stopping Process %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function SaveProcess (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "SAVE")
	local vcc              = inconcert.authUser.virtualContactCenter
	local encodedInfo      = web.input["post_data"]
	local postData         = json.decode(web.input.post_data)
	local isNew            = postData.isNew

	LogInfo("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Saving Process in '%s' vcc%s.", vcc, utils.MakeLogAllegroVccMsg(vcc))
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Saving Process in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasPermission = permissions.IsPermissionEnabledForLoggedUser("administration", "admin_restricted")

	if hasPermission and isNew and not inconcert.authUser.api then
		web.status = 403
		LogError("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Unable to save process. You don't have permissions.")
		return {result = "FORBIDDEN"}
	end

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data

	if hasAllegroIntegration then
		success, data = allegroProcesses.SaveProcess(encodedInfo, allegroOtherInfo)
	else
		local process = json.decode(encodedInfo)
		success, data = m_outboundProcessesNative:SaveProcess(process)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Success Saved Process.")
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Success Saved Process Info %s.", encodedInfo)
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Error Saving Process Err: %s", json.encode(err))
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:SaveProcess] Error Saving Process Info %s. ", encodedInfo)
	return err
end

--[[

Return

]]--
local function __LoadAvailableSchedulers (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadAvailableSchedulers] Available Schedulers Outbound Processes, user %s, vcc %s", userId, vcc)

	local success, availableSchedulers = allegroProcesses.LoadAvailableSchedulers()
	if success and availableSchedulers then
		return json.encode(availableSchedulers)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __GetScheduler (web, schedulerId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__GetScheduler] Loading Scheduler: %s, Outbound Processes, user %s, vcc %s", schedulerId, userId, vcc)

	local success, scheduler = allegroProcesses.GetScheduler(schedulerId)
	if success and scheduler then
		return json.encode(scheduler)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __SaveScheduler (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "UPDATE_SCHEDULER")
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__SaveScheduler] User '%s' Saving Scheduler: '%s'.", userId, encodedInfo)

	local success, response = allegroProcesses.SaveScheduler(encodedInfo, allegroOtherInfo)
	if success then
		return response
	else
		local err = response
		web.status = 400
		LogError("[OUTBOUNDENGINE_PROCESSES:__SaveScheduler] Error Saving Scheduler: '%s'. Error: '%s'", encodedInfo, json.encode(err))
		return err
	end
end

--[[

Return

]]--
local function __LoadAdministrationProcesses (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadAdministrationProcesses] --> Loading Processes from Administration Application. user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		-- Invoco a la funcion de procesos de la app "administration".
		local admResources = require "apps.allegro.controllers.resources"
		return admResources.LoadProcesses(web)
	else
		return m_outboundProcessesNative:LoadProcesses()
	end
end

--[[

Return

]]--
local function __LoadBatchFilters (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadBatchFilters] Loading Batch Filters Outbound Processes, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		local success, filters = allegroProcesses.LoadBatchFilters()
		if success and filters then
			return json.encode(filters)
		end
	else
		local filters = GetModel("OutboundProcessesFiltersBatches").LoadFilters(vcc)
		return json.encode(filters)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadBatchContactFields (web, processId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadBatchContactFields] Loading Batch Contact Fields for Process: %s, Outbound Processes, user %s, vcc %s%s", processId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		local success, fields = allegroProcesses.LoadBatchContactFields()
		if success and fields then
			local jsonData = json.encode(fields)
			--LogDebug("[OUTBOUNDENGINE_PROCESSES:__LoadBatchContactFields] --> jsonData: %s", jsonData)
			return jsonData
		end
	else
		_T = m_htmlRender.extension.makeTextTranslator()
		local keyspace = utils.GetDatasource():CassandraConnection("Agent")
		local data = m_outboundEngineClient.queries.data:LoadBatchContactFields(_T, vcc, keyspace)
		keyspace:Close()
		local jsonData = json.encode(data)
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:__LoadBatchContactFields] --> jsonData: %s", jsonData)
		return jsonData
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __SaveNativeBatchFilter (vcc, filter, ksOE)
	assert(type(vcc) == "string")
	assert(type(filter) == "table")
	local modelFiltersBatches = GetModel("OutboundProcessesFiltersBatches")
	local currentFilter = modelFiltersBatches.GetFilter(vcc, filter.id, ksOE)
	local success, response = modelFiltersBatches.SaveFilter(vcc, filter, ksOE)
	if success then
		-- ok
		return true, {
			action = utils.TableIsEmpty(currentFilter) and "CREATED FILTER" or "UPDATED FILTER"
		}
	end
	-- err
	return false, response
end

local function __SaveBatchFilter (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__SaveBatchFilter] User '%s' Saving Batch Filter: '%s'. vcc %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, response
	if hasAllegroIntegration then
		success, response = allegroProcesses.SaveBatchFilter(encodedInfo)
	else
		local filter = json.decode(encodedInfo)
		success, response = __SaveNativeBatchFilter(vcc, filter)
	end
	if success then
		-- ok
		return response
	end
	-- error
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__SaveBatchFilter] Error Saving Batch Filter: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __DeleteBatchFilter (web, filterId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__DeleteBatchFilter] User '%s' Deleting Batch Filter ID: '%s'. vcc %s%s", userId, filterId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, response

	if hasAllegroIntegration then
		local encodedInfo = json.encode({filterId = filterId})
		success, response = allegroProcesses.DeleteBatchFilter(encodedInfo)
		if success then
			return response
		end
	else
		success, response = GetModel("OutboundProcessesFiltersBatches").DeleteFilter(vcc, filterId)
		if success then
			return {}
		end
	end

	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__DeleteBatchFilter] Error Deleting Batch Filter ID: '%s'. Error: '%s'", filterId, json.encode(response))
	return err
end

local function LoadProcessInfo (processId, response, vccId, ksOE)
	assert(processId)
	require("socket").sleep(3) -- Pequena pausa necesaria para que el OE cree en cassandra el batch.
	-- Necesito retornar los batches del proceso para que se actualicen en la grilla.
	local _, batches = m_outboundProcessesNative:LoadBatchesOfProcess(processId, vccId, ksOE)
	response.batches = batches
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:LoadProcessInfo] 1- Loading Process Info of processId: '%s'. batches: '%s'", processId, json.encode(response.batches or {}))
	-- 
	local _, runningBatchActions = m_outboundProcessesNative:GetRunningBatchActions(processId, vccId)
	--LogDebug("[OUTBOUNDENGINE_PROCESSES:LoadProcessInfo] 2- Loading Process Info of processId: '%s'. runningBatchActions: '%s'", processId, json.encode(response.runningBatchActions or {}))
	response.runningBatchActions = runningBatchActions
end

local function __CreateBatch (web, queryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__CreateBatch] User '%s' Creating contacts batch. queryId %s. Info: %s. Vcc: %s%s", userId, queryId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, response
	if hasAllegroIntegration then
		success, response = allegroProcesses.CreateBatch(queryId, encodedInfo)
		if success then
			-- OK
			return true
		end
	else
		local newBatch = json.decode(encodedInfo)
		success, response = m_outboundEngineClient.batches:Create(vcc, newBatch)
		if success then
			-- OK
			LoadProcessInfo(newBatch.processId, response)
			return response
		end
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__CreateBatch] Error Creating contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

--[[

]]--
local function __UpdateBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__UpdateBatch] User '%s' Updating contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	local batch = json.decode(encodedInfo)
	local successExecutedActionWaitSec = nil
	local waitForCompleted = true
	local ksOE = utils.GetDatasource():CassandraConnection("OutboundEngine")
	local success, response = m_outboundEngineClient.batches:Update(batch, vcc, ksOE, successExecutedActionWaitSec, waitForCompleted)
	ksOE:Close()

	if success then
		-- OK
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__UpdateBatch] Error Updating contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __UpdateBatchByQuery (web, queryId, processId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__UpdateBatchByQuery] User '%s' Updating contacts batch. queryId '%s', processId '%s'. vcc: %s%s, Info: %s.", userId, queryId, processId, vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == true, "Method Not Supported for VCC Without Allegro Integration")

	local success, response = allegroProcesses.UpdateBatchByQuery(queryId, processId, encodedInfo)
	if success then
		-- OK
		return success
	end
	-- Error
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__UpdateBatchByQuery] Error Updating contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __UpdateBatchByAction (web, batchId, processId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__UpdateBatchByAction] User '%s' Updating contacts batch. batchId '%s', processId '%s'.Vcc: %s%s. Info: %s.", userId, batchId, processId, vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == true, "Method Not Supported for VCC Without Allegro Integration")

	local success, response = allegroProcesses.UpdateBatchByAction(batchId, processId, encodedInfo)
	if success then
		-- OK
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__UpdateBatchByAction] Error Updating contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __DeleteBatch (web, processId, batchId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__DeleteBatch] User '%s' Deleting contacts batch. processId %s, batchId %s. Vcc: %s%s", userId, processId, batchId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, response

	if hasAllegroIntegration then
		local encodedInfo = json.encode({processId = processId, batchId = batchId})
		success, response = allegroProcesses.DeleteBatch(encodedInfo)
	else
		success, response = m_outboundEngineClient.batches:Destroy(processId, batchId)
	end
	if success then
		-- OK
		return true
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__DeleteBatch] Error Deleting contacts batch. processId %s, batchId %s Error: '%s'", processId, batchId, json.encode(response))
	return err
end

local function __RecycleContactsBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__RecycleContactsBatch] User '%s' Recyle contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	local rawInfo = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:RecycleContacts(vcc, rawInfo)
	if success then
		-- OK
		LoadProcessInfo(rawInfo.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__RecycleContactsBatch] Error Recyle contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __SetEngineTypeContacts (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__SetEngineTypeContacts] User '%s' Set Engine Type contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	local rawInfo = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:SetEngineTypeContacts(vcc, rawInfo)
	if success then
		-- OK
		LoadProcessInfo(rawInfo.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__SetEngineTypeContacts] Error Set Engine Type contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __ChangePriorityContactsBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__ChangePriorityContactsBatch] User '%s' Change priority contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	local rawInfo = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:ChangePriorityContacts(vcc, rawInfo)
	if success then
		-- OK
		LoadProcessInfo(rawInfo.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__ChangePriorityContactsBatch] Error Change priority contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __LoadBatchContactsStatus (web, processId, batchId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadBatchContactsStatus] Loading Batch '%s' Contact Status for Process: %s, Outbound Processes, user %s, vcc %s%s", batchId, processId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if hasAllegroIntegration then
		local success, status = allegroProcesses.LoadBatchContactsStatus(processId, batchId)
		if success and status then
			Translate(status, "status", "status")
			return cjson.encode(status) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		processId = utils.AppendVcc(utils.GetId(processId), vcc)
		local success, contactsStatuses = m_outboundEngineClient.batches:GetContactsStatuses(processId, batchId)
		--LogDebug("[OUTBOUNDENGINE_PROCESSES:__LoadBatchContactsStatus]    ----> contactsStatuses: %s.", json.encode(contactsStatuses or {}))
		if success then
			return contactsStatuses
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

-- ONLY ALLEGRO
local function __RunBatchOrContactsQuery (web, rowsPerPage, source)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchOrContactsQuery] Execute Contact Query in vcc %s%s. Posted Info: '%s', rowsPerPage: '%s', source: '%s'.", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo, rowsPerPage, source)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == true, "Method Not Supported for VCC Without Allegro Integration")

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroProcesses.RunBatchOrContactsQuery(encodedInfo, rowsPerPage, source)

	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchOrContactsQuery] Success Executed Contact Query. Source: '%s' info: %s.", source, encodedInfo)
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:__RunBatchOrContactsQuery] Error Execute Contact Query. Source: '%s', info %s. Err: %s", source, encodedInfo, json.encode(err))

	return err
end

-- ONLY VCC NATIVE
local function __RunBatchQuery (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchQuery] Execute Contact Query in vcc %s%s. Posted Info: '%s',", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	web.headers["Content-Type"] = "application/json"

	local rawInfo = json.decode(encodedInfo)
	rawInfo.source = rawInfo.source or "UNRECOGNIZED"
	local success, query = m_outboundEngineClient.queries.queries:MakeQuery(vcc, rawInfo)
	if not success then
		-- Error
		local err = query
		web.status = 500
		LogError("[OUTBOUNDENGINE_PROCESSES:__RunBatchQuery] Error Execute Contact Query. Source: '%s', info %s. Err: %s", rawInfo.source, encodedInfo, json.encode(err))
		return err
	end

	local success, data = m_outboundEngineClient.batches:SearchContacts(vcc, query, rawInfo.rows_per_page, rawInfo.includedStates)
	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchQuery] Success Executed Contact Query. Source: '%s', info: %s.", rawInfo.source, encodedInfo)
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:__RunBatchQuery] Error Execute Contact Query. Source: '%s', info %s. Err: %s", rawInfo.source, encodedInfo, json.encode(err))
	return err
end

-- ONLY VCC NATIVE
local function __RunBatchSummary (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

--[[
SUMMARY
state campo, cuando creamos el contacto state free
--]]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchSummary] Execute Batch Summary Contacts in vcc %s%s. Posted Info: '%s',", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")

	web.headers["Content-Type"] = "application/json"

	local rawInfo = json.decode(encodedInfo)
	rawInfo.source = rawInfo.source or "UNRECOGNIZED"
	local success, query = m_outboundEngineClient.queries.queries:MakeQuery(vcc, rawInfo)
	if not success then
		-- Error
		local err = query
		web.status = 500
		LogError("[OUTBOUNDENGINE_PROCESSES:__RunBatchSummary] Error Execute Summary Contact Query. Source: '%s', info %s. Err: %s", rawInfo.source, encodedInfo, json.encode(err))
		return err
	end

	local success, data = m_outboundEngineClient.batches:SummaryContacts(vcc, query)
	if success then
		LogInfo("[OUTBOUNDENGINE_PROCESSES:__RunBatchSummary] Success Executed Batch Summary Contacts. Source: '%s' info: %s.", rawInfo.source, encodedInfo)
--data.summary = json.decode('{"SCHEDULED":184845,"ACTIVE":4,"FINISHED":116385,"CANCELED":317637,"PENDING":2771916,"PROCESS":81}}')
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_PROCESSES:__RunBatchSummary] Error Execute Batch Summary Contacts. Source: '%s', info %s. Err: %s", rawInfo.source, encodedInfo, json.encode(err))
	return err
end

local function __PauseBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__PauseBatch] User '%s' Pausing contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")
	local batch = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:Pause(vcc, batch)
	if success then
		-- OK
		LoadProcessInfo(batch.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__PauseBatch] Error Pausing contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __ResumeBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__ResumeBatch] User '%s' Resuming contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")
	local batch = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:Resume(vcc, batch)
	if success then
		-- OK
		LoadProcessInfo(batch.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__ResumeBatch] Error Resuming contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __CancelBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__CancelBatch] User '%s' Canceling contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")
	local batch = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:Cancel(vcc, batch)
	if success then
		-- OK
		LoadProcessInfo(batch.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__CancelBatch] Error Canceling contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

local function __RemoveBatch (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_PROCESSES:__RemoveBatch] User '%s' Removing contacts batch. Info: %s. Vcc: %s%s", userId, encodedInfo, vcc, utils.MakeLogAllegroVccMsg(vcc))
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == false, "Method Not Supported for VCC With Allegro Integration")
	local batch = json.decode(encodedInfo)
	local success, response = m_outboundEngineClient.batches:Remove(vcc, batch)
	if success then
		-- OK
		LoadProcessInfo(batch.processId, response)
		return response
	end
	-- ERROR
	local err = response
	web.status = 400
	LogError("[OUTBOUNDENGINE_PROCESSES:__RemoveBatch] Error Removing contacts batch: '%s'. Error: '%s'", encodedInfo, json.encode(response))
	return err
end

--[[

Return

]]--
local function __LoadContactsFromQuery (web, queryId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local start = web.input["start"]
	local count = web.input["count"]
	local requestId = web.input["requestid"]

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__LoadContactsFromQuery] Loading Contacts From Query, user %s, vcc %s%s. Query: '%s', Start: '%s', Count: '%s', requestId: '%s'.", userId, vcc, utils.MakeLogAllegroVccMsg(vcc), queryId, start, count, requestId)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	assert(hasAllegroIntegration == true, "Method Not Supported for VCC Without Allegro Integration")

	local success, contacts = allegroProcesses.LoadContactsFromQuery(queryId, start, count, requestId)
	if success and contacts then
		return json.encode(contacts)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __Render (web)
	_M.response.DontCache(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local htmlFileName = hasAllegroIntegration and "frm_processes_content_allegro" or "frm_processes_content"

	LogInfo("[OUTBOUNDENGINE_PROCESSES:__Render] Render form Outbound Processes . Vcc '%s'%s", htmlFileName, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local fullHtmlFile = m_appLocation .. string.format("/pages/%s.html", htmlFileName)
	local env = {_T = m_htmlRender.extension.makeTextTranslator()}
	local content = m_htmlRender:RenderPageLP(fullHtmlFile, web, env)
	if not content then
		LogError("[OUTBOUNDENGINE_PROCESSES:__Render] HTML Outbound Processes does not exist: %s", fullHtmlFile)
	end

	return content
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local permView  = "view_outbound_process"
	local replyFn = utils.JsonReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix

	-- RENDER FRM
	module:dispatch_get   (__Render, p .. "/frm/render/")

	-- OUTBOUND PROCESSES
	module:dispatch_get   (replyFn .. ListProcesses, p .."/list/"                                   )
	module:dispatch_post  (fn .. SearchProcesses, p .."/search/"                                    )
	module:dispatch_get   (HasPermission(GetProcess   , permView                 ), p .. "/get/(.+)")
	module:dispatch_post  (HasPermission(DeleteProcess, "delete_outbound_process"), p .. "/delete/" )
	module:dispatch_post  (HasPermission(StartProcess , "save_outbound_process"  ), p .. "/start/"  )
	module:dispatch_post  (HasPermission(StopProcess  , "save_outbound_process"  ), p .. "/stop/"   )
	module:dispatch_post  (HasPermission(SaveProcess  , "save_outbound_process"  ), p .. "/save/"   )

	-- OUTBOUND PROCESSES RULES
	module:dispatch_get   (replyFn .. __LoadAdministrationProcesses, p .. "/administration/load_all/"                       )

	-- OUTBOUND PROCESSES SCHEDULER
	module:dispatch_get   (replyFn .. __LoadAvailableSchedulers    , p .. "/schedulers/available/load/"                     )
	module:dispatch_get   (replyFn .. __GetScheduler               , p .. "/schedulers/get/([^/]*)"                         )
	module:dispatch_post  (replyFn .. __SaveScheduler              , p .. "/schedulers/save/"                               )

	-- OUTBOUND PROCESSES BATCHES
	module:dispatch_get   (replyFn .. __LoadBatchFilters           , p .. "/batch/filters/load/"                            )
	module:dispatch_get   (replyFn .. __LoadBatchContactFields     , p .. "/batch/fields/load/(.+)"                         )
	module:dispatch_post  (replyFn .. __SaveBatchFilter            , p .. "/batch/filters/save/"                            )
	module:dispatch_delete(replyFn .. __DeleteBatchFilter          , p .. "/batch/filters/delete/(.*)"                      )

	-- OUTBOUND PROCESSES CONTACTS BATCHES (ABM)
	module:dispatch_post  (replyFn .. __CreateBatch                , p .. "/batch/contacts/create/([^/]*)"                  )
	module:dispatch_post  (replyFn .. __UpdateBatch                , p .. "/batch/contacts/update/"                         )
	module:dispatch_post  (replyFn .. __UpdateBatchByQuery         , p .. "/batch/contacts/update/by_query/([^/]*)/([^/]*)" )
	module:dispatch_post  (replyFn .. __UpdateBatchByAction        , p .. "/batch/contacts/update/by_action/([^/]*)/([^/]*)")
	module:dispatch_delete(replyFn .. __DeleteBatch                , p .. "/batch/contacts/delete/([^/]*)/([^/]*)"          )
	module:dispatch_get   (replyFn .. __LoadBatchContactsStatus    , p .. "/batch/contacts/status/([^/]*)/([^/]*)"          )

	-- OUTBOUND PROCESSES CONTACTS BATCHES (ABM) (NATIVO)
	module:dispatch_post  (replyFn .. __PauseBatch                 , p .. "/batch/pause/"                                   )
	module:dispatch_post  (replyFn .. __ResumeBatch                , p .. "/batch/resume/"                                  )
	module:dispatch_post  (replyFn .. __CancelBatch                , p .. "/batch/cancel/"                                  )
	module:dispatch_post  (replyFn .. __RemoveBatch                , p .. "/batch/remove/"                                  )
	module:dispatch_post  (replyFn .. __RecycleContactsBatch       , p .. "/batch/contacts/recycle/"                        )
	module:dispatch_post  (replyFn .. __ChangePriorityContactsBatch, p .. "/batch/contacts/change_priority/"                )
	module:dispatch_post  (replyFn .. __SetEngineTypeContacts      , p .. "/batch/contacts/set_engine_type/"                )

	-- OUTBOUND PROCESSES BATCHES PAGINATED CONTACTS (ALLEGRO)
	module:dispatch_post  (fn .. __RunBatchOrContactsQuery         , p .. "/batch_or_contacts/query/run/([^/]*)/([^/]*)"    )
	module:dispatch_get   (replyFn .. __LoadContactsFromQuery      , p .. "/batch/contacts/load/fromquery/([^/]*)/([^/]*)"  )

	-- OUTBOUND PROCESSES BATCHES PAGINATED CONTACTS (NATIVO)
	module:dispatch_post  (fn .. __RunBatchQuery                   , p .. "/batch/contacts/query/execute/"                  )
	module:dispatch_post  (fn .. __RunBatchSummary                 , p .. "/batch/contacts/summary/execute/"                )

	m_htmlRender  = htmlRender
	m_appLocation = appLocation
end

------------------------------------------------------------------------------------------------------------------------------------
--[[
	API.
		A los metodos del API se accede desde:
			1- (OCC Api SDK)                         \agent\inconcert\api\controllers\outbound_engine_process.lua
			2- (ANALYTICS)                           \agent\inconcert\apps\campaign_analytics\controllers\process.lua
			3- (OUTBOUNDENGINE)                      \agent\inconcert\apps\outbound_engine\outboundengineclient.lua
			4- (PROVISIONING-MIGRATIONS ALLEGRO VCC) \agent\inconcert\apps\provisioning\controllers\migrations\oeProcesses.lua
]]--

local function WaitForCompletedParameter (pWaitForCompleted)
	local waitForCompleted = true -- valor x defecto
	if type(pWaitForCompleted) == "boolean" then
		waitForCompleted = pWaitForCompleted
	end
	return waitForCompleted
end

function GetApi ()
	return {
		native = {
			processes = {
			-- procesos
				AreAllVccProcessesReady = function(vccId)
					return m_outboundProcessesNative:AreAllVccProcessesReady({vcc = vccId})
				end,
				GetProcessCampaignAndAccounts = m_outboundProcessesNative.GetProcessCampaignAndAccounts,
				GetProcessStatus              = m_outboundProcessesNative.GetProcessStatus,
				GetProcess = function(processId, vccId, ksOE)
					return m_outboundProcessesNative:GetProcess(processId, vccId, ksOE)
				end,
				GetProcessAndBatches = function(processId, vccId, ksOE)
					return m_outboundProcessesNative:GetProcessAndBatches(processId, vccId, ksOE)
				end,
				LoadProcesses = function(ksOE, vccId)
					return m_outboundProcessesNative:LoadProcesses(ksOE, vccId)
				end,
				LoadProcessesAndBatches = function(ksOE, vccId)
					return m_outboundProcessesNative:LoadProcessesAndBatches(ksOE, vccId)
				end,
				DeleteProcess = function(processId, vccId, ksOE, ksAgent)
					return m_outboundProcessesNative:DeleteProcess(processId, vccId, ksOE, ksAgent)
				end,
				SaveProcess = function(process, notMustValidate, ksAgent, ksProv, ksOE)
					-- TODO: quien invoque a esta funcion tiene que pasar como parametros los 2 keyspaces
					return m_outboundProcessesNative:SaveProcess(process, notMustValidate, ksAgent, ksProv, ksOE)
				end,
			-- batches
				batches = {
					Create = function (vcc, newBatch, ksOE, pWaitForCompleted)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local success, response = m_outboundEngineClient.batches:Create(vcc, newBatch, ksOE, waitForCompleted)
						if success then
							--LoadProcessInfo(newBatch.processId, response, vcc, ksOE)
							return true, response
						end
						return false, response
					end,
					AddContacts = function (vcc, currentBatch, ksOE, pWaitForCompleted)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local success, response = m_outboundEngineClient.batches:AddContacts(vcc, currentBatch, ksOE, waitForCompleted)
						if success then
							--LoadProcessInfo(currentBatch.processId, response, vcc, ksOE)
							return true, response
						end
						return false, response
					end,
					Destroy = function (vcc, processId, batchName, ksOE, pWaitForCompleted)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						return m_outboundEngineClient.batches:Destroy(processId, batchName, vcc, waitForCompleted, ksOE)
					end,
					Cancel = function (vcc, processId, batchName, contacts, ksOE, pWaitForCompleted, successExecutedActionWaitSec)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local batchInfo = {processId = processId, batchId = batchName, contacts = contacts}
						return m_outboundEngineClient.batches:Cancel(vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
					end,
					Finish = function (vcc, processId, batchName, contacts, ksOE, pWaitForCompleted, successExecutedActionWaitSec)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local batchInfo = {processId = processId, batchId = batchName, contacts = contacts}
						return m_outboundEngineClient.batches:Finish(vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
					end,
					Remove = function (vcc, processId, batchName, contacts, ksOE, pWaitForCompleted, successExecutedActionWaitSec)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local batchInfo = {processId = processId, batchId = batchName, contacts = contacts}
						return m_outboundEngineClient.batches:Remove(vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
					end,
					Pause = function (vcc, processId, batchName, contacts, ksOE, pWaitForCompleted, successExecutedActionWaitSec)
						local waitForCompleted = WaitForCompletedParameter(pWaitForCompleted)
						local batchInfo = {processId = processId, batchId = batchName, contacts = contacts}
						return m_outboundEngineClient.batches:Pause(vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
					end,
					filters = {
						Load = function(vcc, ksOE)
							return GetModel("OutboundProcessesFiltersBatches").LoadFilters(vcc, filter) -- respuesta filters
						end,
						Save = function (vcc, filter, ksOE)
							return __SaveNativeBatchFilter(vcc, filter, ksOE) -- respuesta-> success, response
						end,
						Delete = function (vcc, filterId, ksOE)
							return GetModel("OutboundProcessesFiltersBatches").DeleteFilter(vcc, filterId, ksOE) -- respuesta true/false
						end,
						LoadBatchContactFields = function (vcc, ksAgent)
							return m_outboundEngineClient.queries.data:LoadBatchContactFields(nil, vcc, ksAgent)
						end
					}
				}
			}
		}
		-- allegro ?
	}
end
