local m_models
module(..., package.seeall, inconcert.orbit.application.Decorate)
m_models = require("apps.outbound_engine.model"):GetModels()
local cjson = require "cjson"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroSystem = require "apps.allegro.controllers.outboundengine_system"
local Fullcontact = require "fullcontact"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

local function GetModel()
	if not m_models or not m_models.Campaigns then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.outbound_engine.model"
		model.CreateModelTable()
		m_models = model:GetModels()
	end
	return m_models
end

--[[
	Esta función se encarga de traducir la información que llega desde la BD de 5.5.
	Esto se realiza así, ya que en 5.5 se traduce en los campos calculados del model del ORM, y como el usuario que se encuentra logueado
	es "integrationuser" y su idioma es en inglés, la información siempre llega en este idioma.
	Entonces acá se debe de proceder a realizar la traducción del campo según el idioma del Usuario de i6.
]]--
local function Translate (list, sourceAttr, targetAttr)
	if not list then
		return
	end
	sourceAttr = sourceAttr or "DisplayName"
	targetAttr = targetAttr or "DisplayName"
	_T = m_htmlRender.extension.makeTextTranslator()
	for _, row in pairs(list) do
		local sourceValue = row[sourceAttr]
		local translatedSourceValue = _T(row[sourceAttr] or "")
		row[targetAttr] = translatedSourceValue
		--LogDebug("Translated %s -> %s TO-> %s", targetAttr, sourceValue, translatedSourceValue)
	end
end

--[[

Return

]]--
local function __LoadEngineServices (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadEngineServices] Loading Engine Services, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, services = allegroSystem.LoadEngineServices()
		if success and services then
			--utils.CreateFile("/tmp/systemDictionary/engineServices.json", cjson.encode(services))
			Translate(services)
			return cjson.encode(services) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		assert(false, "NOT SUPPORTED FOR VCC WITHOUT ALLEGRO INTEGRATION")
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadEngineTypes (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadEngineTypes] Loading Engine Types, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, types = allegroSystem.LoadEngineTypes()
		if success and types then
			--utils.CreateFile("/tmp/systemDictionary/engineTypes.json", cjson.encode(types))
			Translate(types, "Name")
			return cjson.encode(types) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadEngineTypes()
		end)
		if success and data.data then
			local list = data.data
			Translate(list, "Name")
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

]]--
local function LoadNativeResponsesByChannel (channel, mustAlsoReturnEncoded)
	mustAlsoReturnEncoded = mustAlsoReturnEncoded
	LogInfo("[OUTBOUNDENGINE_SYSTEM:LoadNativeResponsesByChannel] Loading Native Rules Responses by Channel %s", channel)
	local success, data
	ExecuteFullcontactMethod(function()
		success, data = Fullcontact.Dialer.Data:LoadResponsesByChannel(channel)
	end)
	local encodedList, list
	if success and data.data then
		-- ok
		list = data.data
		Translate(list, "name", "name")
		if mustAlsoReturnEncoded then
			encodedList = cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
		success = true
	else
		-- error
		success = false
	end
	return success, list, encodedList
end

--[[

Return

]]--
local function __LoadResponsesByChannel (web, channel)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadResponsesByChannel] Loading Rules Responses by Channel %s, user %s,vcc %s%s", channel, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))
	if hasAllegroIntegration then
		local success, responsesByChannel = allegroSystem.LoadResponsesByChannel(channel)
		if success and responsesByChannel then
			-- OK
			Translate(responsesByChannel)
			return cjson.encode(responsesByChannel) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, list, encodedList = LoadNativeResponsesByChannel(channel, true)
		if success then
			-- OK
			return encodedList
		end
	end
	-- ERROR
	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadDestinations (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadDestinations] Loading Rules Destinations, user %s, vcc %s %s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, destinations = allegroSystem.LoadDestinations()
		if success and destinations then
			--utils.CreateFile("/tmp/systemDictionary/destinations.json", cjson.encode(destinations))
			Translate(destinations, "Name")
			return cjson.encode(destinations) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadDestinations()
		end)
		if success and data.data then
			local list = data.data
			Translate(list, "Name")
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadAllowedRulesChannels (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadAllowedRulesChannels] Loading Allowed Rules Channels, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		assert(false, "NOT SUPPORTED FOR VCC WITH ALLEGRO INTEGRATION")
	else
		_T = m_htmlRender.extension.makeTextTranslator()
		local allowedChannels = utils.GetOEProcessesSystemChannels(_T)
		--TODO: Los KINDS los puedo obtener aca, asi no hago el request client side. "__LoadDestinations" (client side "__update_kinds_by_channels")
		return cjson.encode(allowedChannels)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[
]]--
local function __LoadRulesSystemActions (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadRulesSystemActions] Loading Rules System Actions, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		assert(false, "NOT SUPPORTED FOR VCC WITH ALLEGRO INTEGRATION")
	else
		_T = m_htmlRender.extension.makeTextTranslator()
		local systemActions = utils.GetOEProcessesSystemRulesActions(_T)
		return cjson.encode(systemActions)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadImportations (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadImportations] Loading Importations, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, importations = allegroSystem.LoadImportations()
	if success and importations then
		Translate(importations)
		return cjson.encode(importations) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

function LoadImportFieldsTypesByTelI6 (returnType)
	returnType = returnType or "NORMAL"
	local success, data
	ExecuteFullcontactMethod(function()
		success, data = Fullcontact.Dialer.Data:LoadImportFieldsTypes()
	end)
	if success and data.data then
		local list = data.data
		Translate(list)
		if returnType ~= "INDEXED LIST" then
			return true, list
		end
		local indexedList = {}
		for _, row in ipairs(list) do
			indexedList[tostring(row.Value)] = row
		end
		--LogDebug("[OUTBOUNDENGINE_SYSTEM:LoadImportFieldsTypesByTelI6] ----> indexedList: %s", json.encode(indexedList))
		return true, indexedList
	end
	return false
end

--[[

Return

]]--
local function __LoadImportFieldsTypes (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadImportFieldsTypes] Loading Import Format Fields Types, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, fieldsTypes = allegroSystem.LoadImportFieldsTypes()
		if success and fieldsTypes then
			--utils.CreateFile("/tmp/systemDictionary/fieldsTypes.json", cjson.encode(fieldsTypes))
			Translate(fieldsTypes)
			return cjson.encode(fieldsTypes) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, list = LoadImportFieldsTypesByTelI6()
		if success then
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadScheduledDateFormats (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadScheduledDateFormats] Loading Import Scheduled Date Formats, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, dateFormats = allegroSystem.LoadScheduledDateFormats()
		if success and dateFormats then
			--utils.CreateFile("/tmp/systemDictionary/schedulesDateFormats.json", cjson.encode(dateFormats))
			Translate(dateFormats)
			return cjson.encode(dateFormats) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end	
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadScheduledDateFormats()
		end)
		if success and data.data then
			local list = data.data
			Translate(list)
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadCountryList (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadCountryList] Loading Import Countries, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, countriesList = allegroSystem.LoadCountryList()
		if success and countriesList then
			--utils.CreateFile("/tmp/systemDictionary/countriesList.json", cjson.encode(countriesList))
			Translate(countriesList)
			return cjson.encode(countriesList) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		LogWarning("[OUTBOUNDENGINE_SYSTEM:__LoadCountryList] Imports countries disabled for vcc %s%s. This information is retrieved when you log in.", vcc, utils.MakeLogAllegroVccMsg(vcc))
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadContactDestinations (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadContactDestinations] Loading Import Contact Destinations, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, destinations = allegroSystem.LoadContactDestinations()
		if success and destinations then
			--utils.CreateFile("/tmp/systemDictionary/contactDestinations.json", cjson.encode(destinations))
			Translate(destinations, "Name")
			return cjson.encode(destinations) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadContactDestinations()
		end)
		if success and data.data then
			local list = data.data
			Translate(list, "Name")
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadContactMailDestinations (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadContactMailDestinations] Loading Import Contact Mail Destinations, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, mailDestinations = allegroSystem.LoadContactMailDestinations()
		if success and mailDestinations then
			--utils.CreateFile("/tmp/systemDictionary/mailDestinations.json", cjson.encode(mailDestinations))
			Translate(mailDestinations, "Name")
			return cjson.encode(mailDestinations) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadContactMailDestinations()
		end)
		if success and data.data then
			local list = data.data
			Translate(list, "Name")
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadContactAddressDestinations (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadContactAddressDestinations] Loading Import Contact Address Destinations, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, addressDestinations = allegroSystem.LoadContactAddressDestinations()
		if success and addressDestinations then
			--utils.CreateFile("/tmp/systemDictionary/addressDestinations.json", cjson.encode(addressDestinations))
			Translate(addressDestinations, "Name")
			return cjson.encode(addressDestinations) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadContactAddressDestinations()
		end)
		if success and data.data then
			local list = data.data
			Translate(list, "Name")
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadCustomColumns (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadCustomColumns] Loading Import Custom Columns, user %s,vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, customColumns = allegroSystem.LoadCustomColumns()
		if success and customColumns then
			--utils.CreateFile("/tmp/systemDictionary/customColumns.json", cjson.encode(customColumns))
			Translate(customColumns)
			return cjson.encode(customColumns) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end	
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadCustomColumns()
		end)
		if success and data.data then
			local list = data.data
			Translate(list)
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadReservedWords (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadReservedWords] Loading Import Reserved Words, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, reservedWords = allegroSystem.LoadReservedWords()
		if success and reservedWords then
			--utils.CreateFile("/tmp/systemDictionary/reserverdWords.json", cjson.encode(reservedWords))
			Translate(reservedWords)
			return cjson.encode(reservedWords) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadReservedWords()
		end)
		if success and data.data then
			local list = data.data
			Translate(list)
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadTimeZones (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadTimeZones] Loading Timezones, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, timezones = allegroSystem.LoadTimeZones()
		if success and timezones then
			--utils.CreateFile("/tmp/systemDictionary/timezones.json", cjson.encode(timezones))
			Translate(timezones)
			return cjson.encode(timezones) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer.Data:LoadTimeZones()
		end)
		if success and data.data then
			local list = data.data
			Translate(list)
			return cjson.encode(list) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadCountries (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadCountries] Loading Countries, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, countries = allegroSystem.LoadCountries()
		if success and countries then
			--utils.CreateFile("/tmp/systemDictionary/countries.json", cjson.encode(countries))
			Translate(countries, "Name")
			return cjson.encode(countries) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		LogWarning("[OUTBOUNDENGINE_SYSTEM:__LoadCountries] Countries disabled for vcc %s%s. This information is retrieved when you log in.", vcc, utils.MakeLogAllegroVccMsg(vcc))
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadAreas (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadAreas] Loading Areas, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	if hasAllegroIntegration then
		local success, areas = allegroSystem.LoadAreas()
		if success and areas then
			--utils.CreateFile("/tmp/systemDictionary/areas.json", cjson.encode(areas))
			Translate(areas)
			return cjson.encode(areas) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
		end
	else
		LogWarning("[OUTBOUNDENGINE_SYSTEM:__LoadAreas] Loading areas disabled for vcc %s%s. This information is retrieved when you log in.", vcc, utils.MakeLogAllegroVccMsg(vcc))
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadBatchesByCampaign (web, campaign)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	campaign = utils.GetId(campaign)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadBatchesByCampaign] Loading Batches by Campaign %s, user %s, vcc %s", campaign, userId, vcc)

	local success, batchesByCampaign = allegroSystem.LoadBatchesByCampaign(campaign)
	if success and batchesByCampaign then
		Translate(batchesByCampaign)
		return cjson.encode(batchesByCampaign) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadCampaignMembers (web, campaignId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadCampaignMembers] Loading Campaign '%s' Members. User %s", campaignId, userId)

	local campaignMembers = GetModel().Campaigns.GetCampaignMembers(vcc, campaignId)
	local members = {}

	for fullUserId in pairs(campaignMembers) do
		local userId = utils.GetId(fullUserId)
		table.insert(members, {
			UserId = userId
		})
	end
 
	return cjson.encode(members)
end

--[[
	Funcion generica para validar si cualquier tipo de cuenta puede o no ser incluida.
	Ejemplo que la cuenta SI se encuentre "active"
]]--
local function MustIncludeAccount (account)
	if not account then
		LogInfo("[OUTBOUNDENGINE_SYSTEM:MustIncludeAccount] Unable to include account. Account is invalid.")
		return false, "INVALID ACCOUNT"
	end
	if tonumber(account.state or "0") ~= 1 then
		LogInfo("[OUTBOUNDENGINE_SYSTEM:MustIncludeAccount] Unable to include account '%s'. Account is disabled.", account.id)
		return false, "ACCOUNT IS DISABLED"
	end
	-- ok (include)
	return true
end

--[[
	Funcion de cuentas de CALL para validar si los tipos de cuenta de CALL pueden ser incluidas.
	Ejemplo que tenga "packages" salientes.
]]--
local function MustIncludeCallAccount (callAccount, ksAgent, ksProv)
	local success, err = MustIncludeAccount(callAccount)
	if not success then
		return false, err
	end
	if utils.TableIsEmpty(callAccount.packages) then
		LogWarning("[OUTBOUNDENGINE_SYSTEM:MustIncludeCallAccount] Unable to include Call account '%s'. Account has not packages.", callAccount.id)
		return false, "ACCOUNT HAS NOT PACKAGES"
	end
	-- valido que al menos tenga un paquete saliente
	local hasOutboundPackages = false
	for packageId in pairs(callAccount.packages) do
		local success, package = GetModel().Packages.Get(packageId, ksProv)
		LogDebug("[OUTBOUNDENGINE_SYSTEM:MustIncludeCallAccount] Call account '%s'. PackageInfo: %s.", callAccount.id, json.encode(package or {}))
		if not success then
			LogInfo("[OUTBOUNDENGINE_SYSTEM:MustIncludeCallAccount] Attention Package '%s' of Call Account '%s' does not exist in Data Base.", packageId, callAccount.id)
		else
			local ioMode = (package.ioMode or ""):upper()
			if ioMode == "O" or ioMode == "I/O" then
				-- Si tiene al menos un paquete saliente.
				hasOutboundPackages = true
				break
			end
		end
	end
	if not hasOutboundPackages then
		LogWarning("[OUTBOUNDENGINE_SYSTEM:MustIncludeCallAccount] Unable to include Call account '%s'. Account has not outbound packages.", callAccount.id)
		return false, "ACCOUNT HAS NOT OUTBOUND PACKAGES"
	end
	-- ok (include)
	return true
end

--[[
	funcion invocada desde la validacion de procesos a guardar "controller.processes"
]]--
function ValidateIncludeAccount (account, ksAgent, ksProv)
	assert(type(account) == "table")
	if account.proxyType == "CALL" then
		return MustIncludeCallAccount(account, ksAgent, ksProv)
	end
	return MustIncludeAccount(account)
end

--[[

	funcion que verifica si el permiso de restringir campañas esta activo, solo filtre a las que pertenece el usuario

]]--
local function  ValidateBelongsToCampaign (vcc, campaignId, keyspace)
	local hasPermission = permissions.IsPermissionEnabledForLoggedUser("administration", "admin_restricted", nil, keyspace)
    
    if hasPermission and not inconcert.authUser.api then
		local userId           = inconcert.authUser.userName.."@"..vcc
		local isCampaignMember = GetModel().Campaigns.UserIsMember(campaignId, userId, keyspace)
			
		if not isCampaignMember then
			LogError("[OUTBOUNDENGINE_SYSTEM:ValidateBelongsCampaign] User %s  not belongs to campaign '%s'. Discarding it ...", userId, campaignId)
			return false
		end
	end

	return true
end

--[[

Return

]]--
local function __LoadCampaignsAndAccountsByChannels (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	--[[
		TODO: Para habilitar mas canales salientes en el OE, hay que cambiar a true, o agregar mas canales a la lista.
			Tambien hay que habilitar en el JavaScript ""form_outbound_engine_processes.js" "m_supportedChannels"
	]]--
	local supportedChannels = {
		CALL     = {enabled = true , MustIncludeAccount = MustIncludeCallAccount},
		MAIL     = {enabled = false, MustIncludeAccount = MustIncludeAccount    },
		SMS      = {enabled = false, MustIncludeAccount = MustIncludeAccount    },
		WHATSAPP = {enabled = false, MustIncludeAccount = MustIncludeAccount    }
	}
	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadCampaignsAndAccountsByChannels] Loading Campaigns by Supported Channels. %s", json.encode(supportedChannels))
	local ksAgent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
	local vccCampaignsAccountsByChannels = GetModel().Campaigns.GetVccCampaignsAccountsByChannels(vcc, ksAgent)
	local campaigns = {}

	for campaignId, campaignInfo in pairs(vccCampaignsAccountsByChannels) do
		if ValidateBelongsToCampaign(vcc, campaignId, ksAgent) then
			local isInternalCampaign = GetModel().Campaigns.GetSpecificColumn(campaignId, "IsInternal", ksAgent)
			if not isInternalCampaign then
				for channelId, info in pairs(supportedChannels) do
					if info.enabled then
						local campaignChannel = campaignInfo.channels[channelId]
						if #(campaignChannel or {}) > 0 then
							for _, account in ipairs(campaignChannel) do
								if info.MustIncludeAccount(account, ksAgent, ksProv) then
									if not campaigns[campaignId] then
										local campaignName = GetModel().Campaigns.GetSpecificColumn(campaignId, "Name", ksAgent)
										local encodedOEConfig = GetModel().Campaigns.GetSpecificColumn(campaignId, "OEConfig", ksAgent)
										local OEConfig = utils.JSON_Decode(encodedOEConfig)
										local processId = utils.GetId((OEConfig or {}).processId or "")
										campaigns[campaignId] = {Id = campaignId, Name = campaignName, channels = {}, ProcessId = processId}
									end
									campaigns[campaignId].channels = campaigns[campaignId].channels or {}
									campaigns[campaignId].channels[channelId] = campaigns[campaignId].channels[channelId] or {}
									table.insert(campaigns[campaignId].channels[channelId], {id = account.id, name = account.name or account.id})
								end
							end
						end
					end
				end
			end
		end
	end

	ksAgent:Close()
	ksProv:Close()
	LogDebug("[OUTBOUNDENGINE_SYSTEM.__LoadCampaignsAndAccountsByChannels] campaigns: %s", json.encode(campaigns))
	return cjson.encode(campaigns)
end

--[[

Return

]]--
local function __LoadSkills (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadSkills] Loading Skills. User %s", userId)

	local skills = GetModel().Skills.GetSkills(vcc)
	local encodedSkills = cjson.encode(skills)

	LogDebug("[OUTBOUNDENGINE_SYSTEM:__LoadSkills] Loaded Skills: %s", encodedSkills)

	return encodedSkills
end

--[[

Return

]]--
local function __LoadExportLayoutColumnTypes (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_SYSTEM:__LoadExportLayoutColumnTypes] Loading Export Columns Types, user %s, vcc %s", userId, vcc)

	local success, columnsTypes = allegroSystem.LoadExportLayoutColumnTypes()
	if success and columnsTypes then
		Translate(columnsTypes)
		return cjson.encode(columnsTypes) -- las traducciones se deben de encodear con cjson, para que no den problemas las comillas simples.
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, p, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local jsonReply = utils.JsonReply .. LogRequest

	-- URIS que van contra allegro
	module:dispatch_get (jsonReply .. __LoadEngineServices                , p .. "/engine/services/load/"               )
	module:dispatch_get (jsonReply .. __LoadEngineTypes                   , p .. "/engine/types/load/"                  )
	module:dispatch_get (jsonReply .. __LoadResponsesByChannel            , p .. "/rules/responsesby/channel/([^.]*)"   )
	module:dispatch_get (jsonReply .. __LoadDestinations                  , p .. "/rules/destinations/load/"            )
	module:dispatch_get (jsonReply .. __LoadImportFieldsTypes             , p .. "/imports/fieldstypes/load/"           )
	module:dispatch_get (jsonReply .. __LoadScheduledDateFormats          , p .. "/imports/scheduleformat/load/"        )
	module:dispatch_get (jsonReply .. __LoadCountryList                   , p .. "/imports/countries/load/"             )
	module:dispatch_get (jsonReply .. __LoadContactDestinations           , p .. "/imports/destinations/load/"          )
	module:dispatch_get (jsonReply .. __LoadContactMailDestinations       , p .. "/imports/maildestinations/load/"      )
	module:dispatch_get (jsonReply .. __LoadContactAddressDestinations    , p .. "/imports/addressdestinations/load/"   )
	module:dispatch_get (jsonReply .. __LoadCustomColumns                 , p .. "/imports/customcolumns/load/"         )
	module:dispatch_get (jsonReply .. __LoadReservedWords                 , p .. "/imports/reservedwords/load/"         )
	module:dispatch_get (jsonReply .. __LoadTimeZones                     , p .. "/timezones/load/"                     )
	module:dispatch_get (jsonReply .. __LoadCountries                     , p .. "/countries/load/"                     )
	module:dispatch_get (jsonReply .. __LoadAreas                         , p .. "/areas/load/"                         )
	module:dispatch_get (jsonReply .. __LoadBatchesByCampaign             , p .. "/batches_by_campaign/load/([^.]*)"    )
	module:dispatch_get (jsonReply .. __LoadImportations                  , p .. "/importations/load/"                  )
	module:dispatch_get (jsonReply .. __LoadExportLayoutColumnTypes       , p .. "/exports/columnstypes/load/"          )

	-- URIS que van contra i6
	module:dispatch_get (jsonReply .. __LoadCampaignMembers               , p .. "/campaign/members/load/([^.]*)"       )
	module:dispatch_get (jsonReply .. __LoadCampaignsAndAccountsByChannels, p .. "/campaigns/accounts/by_channels/load/")
	module:dispatch_get (jsonReply .. __LoadSkills                        , p .. "/skills/load/"                        )
	module:dispatch_get (jsonReply .. __LoadAllowedRulesChannels          , p .. "/rules/channels/load/"                )
	module:dispatch_get (jsonReply .. __LoadRulesSystemActions            , p .. "/rules/system_actions/load/"          )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end


------------------------------------------------------------------------------------------------------------------------------------
--[[
	API.
		A los metodos del API se accede desde:
			1- (OUTBOUNDENGINE)                      \agent\inconcert\apps\outbound_engine\outboundengineclient.lua
]]--

function GetApi ()
	return {
		native = {
			AreNodesReady = function()
				return m_outboundEngineClient.nodes:AreNodesReady()
			end,
			LoadNativeResponsesByChannel = function (channel, mustAlsoReturnEncoded)
				return LoadNativeResponsesByChannel(channel, mustAlsoReturnEncoded)
			end
		}
		-- allegro ?
	}
end
