module(..., package.seeall, inconcert.orbit.application.Decorate)
local cjson = require "cjson"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
local Fullcontact = require "fullcontact"

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

local function __Get (web, contactId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_WEBDESIGNER:__Get] Retrieving Contact %s, user %s, vcc %s",
			contactId, userId, vcc)

	local success, contact
	ExecuteFullcontactMethod(function()
		success, contact = Fullcontact.Dialer:GetContact(contactId, vcc)
	end)

	if not success then
		web.status = 404
		return {result = "NOT_FOUND"}
	end

	local encodedContact = cjson.encode(contact)
	--LogDebug("[OUTBOUNDENGINE_WEBDESIGNER:__Get] retrieve Contact '%s', data: %s", contactId, encodedContact)
	return encodedContact
end

local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_WEBDESIGNER:__Search] Loading Contacts in '%s'. Posted Info: '%s'",
			vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local queryInfo = web.request
	queryInfo.vcc = vcc
	local success, data = m_outboundEngineClient.batches:RawSearchContacts(queryInfo)

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_WEBDESIGNER:__Search] Loaded Contacts. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_WEBDESIGNER:__Search] Error loading Contacts. Err: %s", json.encode(err))

	return err
end

local function __ValidatePhone (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_WEBDESIGNER:__ValidatePhone] Validating phone in '%s'. Posted Info: '%s'",
			vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local queryInfo = web.request

	local success, result
	ExecuteFullcontactMethod(function()
		success, result = Fullcontact.Dialer:ValidatePhone(
			queryInfo.number, queryInfo.country, queryInfo.area, vcc, true)
	end)

	if not success then
		web.status = 401
	end

	return result
end

local function __SaveContact (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_WEBDESIGNER:__SaveContact] Saving contact in '%s'. Posted Info: '%s'",
			vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local queryInfo = web.request
	local success, data = m_outboundEngineClient.batches:StoreContacts(vcc, { contacts = { queryInfo.contact } })

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_WEBDESIGNER:__Search] Loaded Contacts. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_WEBDESIGNER:__SaveContact] Error Saving contact. Err: %s", json.encode(err))

	return err
end

local function __RescheduleContact (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_WEBDESIGNER:__RescheduleContact] Saving contact in '%s'. Posted Info: '%s'",
			vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local queryInfo = web.request
	queryInfo.vcc = vcc
	if queryInfo.campaignId then
		queryInfo.campaignId = utils.AppendVcc(queryInfo.campaignId, vcc)
	end
	local success, data = m_outboundEngineClient.batches:RescheduleContact(queryInfo)

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_WEBDESIGNER:__Search] Loaded Contacts. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_WEBDESIGNER:__RescheduleContact] Error Saving contact. Err: %s", json.encode(err))

	return err
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local permView  = "view_contacts"
	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix

	-- CONTACTS
	module:dispatch_get (replyFn .. LogRequest .. __Get, p .. "/get/(.+)")
	module:dispatch_post(fn .. __Search, p .. "/search/")
	module:dispatch_post(fn .. __ValidatePhone, p .. "/validate_phone/")
	module:dispatch_post(fn .. __SaveContact, p .. "/save_contact/")
	module:dispatch_post(fn .. __RescheduleContact, p .. "/reschedule_contact/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
