module(..., package.seeall)
local utils = require "shared.apps.utils"
local m_datasource

function GetName ()
	return "Campaigns"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetSpecificColumn (campaignAtVcc, specificColumn, keyspace, vccId)
	if not specificColumn then
		return
	end
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	campaignAtVcc = utils.AppendVcc(utils.GetId(campaignAtVcc), vcc)
	return utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "Campaigns",
		key = campaignAtVcc,
		column = specificColumn
	})
end

function GetCampaignMembers (vcc, campaign_id, keyspace)
	local simpleCampId = utils.GetId(campaign_id)
	local fullCampId = utils.AppendVcc(simpleCampId, vcc)

	local members = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignMembers",
		key = fullCampId
	} or {}
	--LogDebug("[GetCampaignMembers] Campaign '%s' members: '%s'", fullCampId, json.encode(members))
	return members
end

function GetVCCCampaigns (vcc, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCCampaigns",
		key = vcc
	} or {}
end

function GetAllVCCsAccounts (keyspace)
	local params = {
		keyspace = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		startKey = "",
		endKey = ""
	}
	return utils.GetAllColumns(params) or {}
end

function GetVccCampaignsAccounts (vcc, keyspace)
	assert(vcc, "vcc is required")
	vcc = vcc:lower()
	local allAccounts = GetAllVCCsAccounts(keyspace)
	local vccCampaignsAccounts = {}
	for campaignId, campaignAccounts in pairs(allAccounts) do
		local _, campaignVcc = utils.SeparateVCC(campaignId)
		if campaignVcc and (campaignVcc:lower() == vcc) then
			vccCampaignsAccounts[campaignId] = campaignAccounts
		end
	end
	return vccCampaignsAccounts
end

function GetVccCampaignsAccountsByChannels (vcc, keyspace)
	assert(vcc, "vcc is required")
	vcc = vcc:lower()
	local vccCampaignsAccounts = GetVccCampaignsAccounts(vcc, keyspace)
	local vccCampaignsAccountsByChannels = {}
	for campaignId, campaignAccounts in pairs(vccCampaignsAccounts) do
		vccCampaignsAccountsByChannels[campaignId] = vccCampaignsAccountsByChannels[campaignId] or {channels = {}}
		for accountId, account in pairs(campaignAccounts) do
			local channelId = account.proxyType:upper()
			vccCampaignsAccountsByChannels[campaignId].channels[channelId] = vccCampaignsAccountsByChannels[campaignId].channels[channelId] or {}
			table.insert(vccCampaignsAccountsByChannels[campaignId].channels[channelId], account)
		end
	end
	--LogDebug("[OUTBOUND_ENGINE.MODELS.CAMPAIGNS] vccCampaignsAccountsByChannels: %s", json.encode(vccCampaignsAccountsByChannels))
	return vccCampaignsAccountsByChannels
end

function UpdateCampaignOEConfig (vcc, campaignId, OEConfig, keyspace)
	assert(vcc, "vcc is required")
	local campaignIdWithVcc = utils.AppendVcc(utils.GetId(campaignId), vcc):lower()
	--LogDebug("[OUTBOUND_ENGINE.MODELS.UpdateCampaignOEConfig] 1--> campaignIdWithVcc: %s, --> OEConfig: %s", campaignIdWithVcc, json.encode(OEConfig or {}))
	if not OEConfig then
		LogDebug("[OUTBOUND_ENGINE.MODELS.UpdateCampaignOEConfig] 2--> BORRA campaignIdWithVcc: %s, --> OEConfig", campaignIdWithVcc)
		utils.RemoveColumns {
			keyspace = keyspace or "Agent",
			columnFamily = "Campaigns",
			rows = {
				[campaignIdWithVcc] = {"OEConfig"}
			}
		}
	else
		--LogDebug("[OUTBOUND_ENGINE.MODELS.UpdateCampaignOEConfig] 3--> ACTUALIZA campaignIdWithVcc: %s, --> OEConfig", campaignIdWithVcc)
		utils.SaveRows {
			keyspace = keyspace or "Agent",
			columnFamily = "Campaigns",
			rows = {
				[campaignIdWithVcc] = {OEConfig = OEConfig}
			}
		}
	end
	return true
end

function GetCampaignAccounts (campaignIdWithVcc, keyspace, columns, vccId)
	assert(campaignIdWithVcc, "campaignIdWithVcc id is required")
	local vcc = vccId or inconcert.authUser.virtualContactCenter
	campaignIdWithVcc = utils.AppendVcc(utils.GetId(campaignIdWithVcc), vcc)
	local params = {
		keyspace = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		key = campaignIdWithVcc,
		columns = columns
	}
	local campaignAccounts = utils.GetAllColumns( params ) or {}
	--LogDebug("[OUTBOUND_ENGINE.MODELS.GetCampaignAccounts] campaignIdWithVcc: %s, --> Accounts: %s", campaignIdWithVcc, json.encode(campaignAccounts))
	return campaignAccounts
end

function GetUserCampaigns (vcc, userId, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaigns",
		key = utils.AppendVcc(userId, vcc)
	} or {}
end

function UserIsMember (campaign_id, userId, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		closeKeyspace = true
	end

	local res, column = pcall(keyspace.GetColumn, keyspace, campaign_id, "CampaignMembers", userId)
	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		return column and column.value
	end
end
