require "lua_outboundengineclient"
local json = require "json"
local utils = require "shared.apps.utils"
local m_queries = require "apps.outbound_engine.outboundengineclient_queries"
local outboundEngineModel = require "apps.outbound_engine.model"
local m_helpers = require "helpers"
module(..., package.seeall)

local m_clientInstance = nil
local m_batches = {}
local m_processes = {}
local m_nodes = {}

LogDebug = LogDebug or function(s) print(tostring(s)) end
LogInfo = LogInfo or function(s) print(tostring(s)) end
LogWarning = LogWarning or function(s) print(tostring(s)) end

local function Sleep (timeSeconds)
	local DEFAULT_SLEEP_PROCESS_SEC = 5
	utils.Sleep(timeSeconds or DEFAULT_SLEEP_PROCESS_SEC)
	return DEFAULT_SLEEP_PROCESS_SEC
end
-----
local function GetClient ()
	if not m_clientInstance then
		local envCluster = {}
		if inconcert.environment.apps.agent.cluster then
			envCluster = inconcert.environment.apps.agent.cluster
		elseif inconcert.environment.apps.outbound_engine and inconcert.environment.apps.outbound_engine.cluster then
			envCluster = inconcert.environment.apps.outbound_engine.cluster
		end
		--LogDebug("[OUTBOUNDENGINE.CLIENT:GetClient] 1 - Creating Instance - cluster " .. envCluster)
		m_clientInstance = lua_outboundengineclient.LuaOutboundEngineClient(envCluster, "webhandler")
	end
	local oeAddresses = inconcert.environment.apps.agent and inconcert.environment.apps.agent.outboundengineservice or {}
	--LogDebug("[OUTBOUNDENGINE.CLIENT:GetClient] 1 - oeAddresses:  " .. json.encode(oeAddresses))
	for _, address in pairs(oeAddresses) do
		--LogDebug("[OUTBOUNDENGINE.CLIENT:GetClient] Add Known OutboundEngine Client. Cluster. IP: '" .. address.host .. "', Port: '" .. address.port .. "'.")
		m_clientInstance:AddKnownOutboundEngine(address.host, address.port)
	end
	--LogDebug("[OUTBOUNDENGINE.CLIENT:GetClient] 2 - Created Instance " .. tostring(m_clientInstance))
	return m_clientInstance
end
--[[
	Funcion que incluye el Usuario Admin como atributo al parametro que se le enviara al OE Client.
]]--
local function IncludeAdminUser (info)
	if type(info) ~= "table" then
		return
	end
	if not inconcert or not inconcert.authUser then
		return
	end
	info.admin = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:IncludeAdminUser] Include admin user to OE Client info.admin: '%s." , info.admin)
end

--[[
]]--
local function MakeError (err, oeErr)
	assert(type(err) == "table")
	local reason
	if type(oeErr) == "table" then
		if type(oeErr.reason) == "string" then
			reason = oeErr.reason
		elseif type(oeErr.result) == "string" then
			reason = oeErr.result
		elseif type(oeErr.reason) == "table" then
			reason = oeErr.reason.result
		end
	end
	err.reason = reason or "DEFAULT ERROR"
	err.subType = err.reason
	err.rawError = oeError
	return err
end

-----------------------------------------------------------------------------
-- BATCHES
-----------------------------------------------------------------------------

m_batches.MakeBatchId = function (self, processId, batchName)
	assert(type(processId) == "string")
	assert(type(batchName) == "string")
	batchName = utils.GetId(batchName)
	local batchId = string.format("%s:%s", batchName, processId):lower()
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.MakeBatchId] ------> Making BatchId --> '%s' batchName: '%s'", batchId, batchName)
	return batchId, batchName
end

--[[
]]--
m_batches.CanUpdate = function (self, batch, vcc, ksOE)
	assert(type(batch) =="table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanUpdate] 1--> Validate Can Update Batch %s of process %s", batch.batchId, batch.processId)
	-- 1. valida que el proceso exista.
	local outboundEngineAPI = utils.GetOEApi() -- Obtengo el API del "outbound_engine".
	local exist, process = outboundEngineAPI.native.processes.GetProcessAndBatches(batch.processId, vcc, ksOE)
	if not exist then
		return false, {type = "PROCESS_NOT_FOUND", processId = batch.processId, batchId = batch.batchId}
	end
	return true
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanDestroy = function (self, batch, ksAgent)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanDestroy] 1--> Validate Can Destroy Batch")
	return true
end

--[[
	Funcion utilizada en las migraciones de Vcc Allegro a VCC Nativo.
		dicha funcion se encarga de validar que el proceso que se esta ejecutando del OE Client haya culminado.
]]--
m_batches.IsCompletedAction = function (self, vcc, processId, maxRetries, sleepTimeSec)
	assert(vcc)
	assert(processId)
	maxRetries = maxRetries or 500 -- para que no quede en un loop infinito
	sleepTimeSec = sleepTimeSec or 1
	local retry = 0
	local isCompleted = false
	local reason = ""
	while true do
		retry = retry + 1
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.IsCompletedAction] -> Validating if is completed OE Client batch action. Retry %d of %d." , retry, maxRetries)
		if retry > maxRetries then
			-- si paso el tope maximo de intentos, corto para que no quede un loop infinito.
			isCompleted = false
			reason = "OVERFLOW"
			break
		end
		local success, runningBatchActions = m_processes:GetRunningBatchActions(processId, vcc)
		if not success then
			-- si el OEClient ha tirado una excepcion
			isCompleted = false
			reason = "OE CLIENT EXCEPTION"
			break
		end
		if #runningBatchActions.actions == 0 then
			-- si el el proceso no tiene acciones, corto y retorno OK
			isCompleted = true
			reason = "COMPLETED"
			break
		end
		-- el proceso aun no ha terminado, tiene acciones pendientes, tengo que realizar una pausa y volver a consultar si ha culminado.
		Sleep(sleepTimeSec) -- Pequena pausa necesaria para que el OE haga sus magias
	end
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.IsCompletedAction]  -> Validation OE Client actions Result. isCompleted: %s, reason: %s" , isCompleted, reason)
	return isCompleted, reason
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanCreate = function (self, vcc, newBatch, ksOE)
	assert(type(vcc) == "string")
	assert(type(newBatch) == "table")
	assert(type(newBatch.processId) == "string")
	assert(type(newBatch.configuration) == "table")
	assert(type(newBatch.configuration.batchName) == "string")
	assert(type(newBatch.query) == "table")
	local errReason
	local processId = m_processes:MakeProcessId(newBatch.processId, vcc)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.CanCreate] 1--> Validate Can Create Batch")
	-- 1. valida que el proceso exista.
	local outboundEngineAPI = utils.GetOEApi() -- Obtengo el API del "outbound_engine".
	local exist = outboundEngineAPI.native.processes.GetProcessAndBatches(processId, vcc, ksOE)
	if not exist then
		local err = {type = "PROCESS_NOT_FOUND", processId = processId}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanCreate] 1.1--> Error Creating Batch --> err: %s", json.encode(err))
		return false, err
	end
	newBatch.query.source = newBatch.source
	newBatch.query.processId = newBatch.processId
	newBatch.query.excludeContacted = newBatch.excludeContacted
	newBatch.query.contactedDate = newBatch.contactedDate
	local success, query = m_queries.queries:MakeQuery(vcc, newBatch.query)
	if not success then
		-- error
		errReason = (query or "")
		local err = {type = "UNABLE TO CREATE BATCH", reason = errReason, subType = errReason:upper()}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanCreate] 2--> Error Creating Batch --> err: %s", json.encode(err))
		return false, err
	end
	-- ok
	local batch = {
		processId     = processId,
		configuration = newBatch.configuration,
		scope         = query
	}
	batch.configuration.name = utils.GetId(batch.configuration.batchName):lower()
	batch.configuration.batchName = nil
	batch.configuration.batchPriority = tonumber(batch.configuration.batchPriority)
	batch.configuration.basePriority = tonumber(batch.configuration.basePriority)
	local percentage = tonumber(batch.configuration.percentage)
	if percentage and (percentage >= 0 and percentage <= 100) then
		batch.configuration.percentage = percentage
	else
		batch.configuration.percentage = nil
	end
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.CanCreate] 2--> Success Validation Can Create BatchInfo: %s", json.encode(batch))
	return true, batch
end

--[[
]]--
m_batches.Create = function (self, vcc, newBatch, ksOE, waitForCompleted)
	assert(type(vcc) == "string")
	assert(type(newBatch) == "table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Create] 1--> Creating Batch --> vcc: '%s', waitForCompleted: '%s', encodedBatch: ", vcc, waitForCompleted, json.encode(newBatch))
	local success, result = self:CanCreate(vcc, newBatch, ksOE)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Create] 1.1--> Error Creating Batch --> err: " .. json.encode(err))
		return false, err
	end
	newBatch = result
	IncludeAdminUser(result)
	local encodedBatch = json.encode(newBatch)
	local result = GetClient():CreateBatch(encodedBatch)
	LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.Create] GetClient():CreateBatch - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Create] 2--> Success Created Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la creacion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, newBatch.processId)
			end
			return true, result
		end
	end
	-- error
	result = result or {}
	local err = {type = "UNABLE TO CREATE BATCH", reason = result.reason or "", subType = result.result} -- type = BATCH_ALREADY_EXISTS el valor que puede retornar el OE Client.
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Create] 2--> Error Creating Batch --> err reason: %s. rawErr: %s", result.reason, json.encode(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanAddContacts = function (self, vcc, currentBatch, ksOE)
	assert(type(vcc) == "string")
	assert(type(currentBatch) == "table")
	assert(type(currentBatch.processId) == "string")
	assert(type(currentBatch.configuration) == "table")
	assert(type(currentBatch.configuration.batchName) == "string")
	assert(type(currentBatch.query) == "table")
	local errReason
	local processId = m_processes:MakeProcessId(currentBatch.processId, vcc)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.CanAddContacts] 1--> Validate Can AddContacts Batch")
	-- 1. valida que el proceso exista.
	local outboundEngineAPI = utils.GetOEApi() -- Obtengo el API del "outbound_engine".
	local exist = outboundEngineAPI.native.processes.GetProcessAndBatches(processId, vcc, ksOE)
	if not exist then
		local err = {type = "PROCESS_NOT_FOUND", processId = processId}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanAddContacts] 1.1--> Error Adding Contacts Batch --> err: %s", json.encode(err))
		return false, err
	end
	currentBatch.query.source = currentBatch.source
	currentBatch.query.processId = currentBatch.processId
	currentBatch.query.excludeContacted = currentBatch.excludeContacted
	currentBatch.query.contactedDate = currentBatch.contactedDate
	local success, query = m_queries.queries:MakeQuery(vcc, currentBatch.query)
	if not success then
		-- error
		errReason = (query or "")
		local err = {type = "UNABLE TO ADD CONTACTS BATCH", reason = errReason, subType = errReason:upper()}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanAddContacts] 2--> Error Adding Contacts Batch --> err: %s", json.encode(err))
		return false, err
	end
	-- ok
	local batch = {
		processId     = processId,
		batchId       = utils.GetId(currentBatch.configuration.batchName):lower(),
		configuration = {}, --currentBatch.configuration,
		scope         = query
	}
	-- batch.configuration.batchName = nil

--	LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.CanAddContacts] 2--> Success Validation Can AddContacts BatchInfo: %s", json.encode(batch))
	return true, batch
end

--[[
]]--
m_batches.AddContacts = function (self, vcc, currentBatch, ksOE, waitForCompleted)
	assert(type(vcc) == "string")
	assert(type(currentBatch) == "table")
	-- para determinar el tiempo de ejecucion
	local _, starElapsedTime = m_helpers.utils.ElapsedTime.Start()
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.AddContacts] 1--> Adding Contacts to Batch --> encodedBatch: " .. tostring(json.encode(currentBatch)))
	local totalContacts = #(currentBatch.query and currentBatch.query.contacts or {})
	local success, result = self:CanAddContacts(vcc, currentBatch, ksOE)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.AddContacts] 1.1--> Error Adding Contacts to Batch --> err: " .. json.encode(err))
		return false, err
	end
	currentBatch = result
	IncludeAdminUser(result)
	local encodedBatch = json.encode(currentBatch)
	local result = GetClient():AddContacts(encodedBatch)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.AddContacts] GetClient():AddContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la creacion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, currentBatch.processId)
			end
			-- fin del tiempo de ejecucion
			local _, endElapsedTime = m_helpers.utils.ElapsedTime.End(starElapsedTime.processId)
			local elapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(endElapsedTime.elapsedTimeSec)
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.AddContacts] 2--> Success AddContacts Batch. totalContacts: %s, elapsedTime: %s.",
				totalContacts,
				elapsedTimeDetail.formattedTime
			)
			return true, result
		end
	end
	-- error
	result = result or {}
	local err = {type = "UNABLE TO ADD CONTACTS BATCH", reason = result.reason or "", subType = result.result} -- type = BATCH_ALREADY_EXISTS el valor que puede retornar el OE Client.
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.AddContacts] 2--> Error Adding Contacts to Batch --> err: %s", result.reason)
	return false, err
end

--[[
	OutboundProcessBatchStatistics::ddd:rocio@vcc1::State::Total
		con el batch id full con el <batch_name>::<process_id@vcc>
		El batchId  NO tiene que ser CaseSensitive.
		El processId SI tiene que ser CaseSensitive.
]]--
m_batches.MakeKeyBatchStatistics = function (self, processId, batchId)
	assert(processId)
	assert(batchId)
	processId = processId:lower()
	batchId = batchId:lower()
	local key = ("OutboundProcessBatchStatistics::%s:%s::State::Total"):format(batchId, processId:lower())
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.MakeKeyBatchStatistics] ==> key: %s", key)
	return key
end

--[[

]]--
m_batches.GetContactsStatuses = function (self, processId, batchId)
	assert(processId)
	assert(batchId)
	processId = processId:lower()
	batchId = batchId:lower()
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.GetContactsStatuses] 1--> Get Batch Contacts statuses --> processId: %s, batchId: %s", processId, batchId)
	local key = self:MakeKeyBatchStatistics(processId, batchId)
	local oeRedis = outboundEngineModel:GetOutboundEngineRedisClient()
	local contactsStatuses = oeRedis:hgetall(key)
	----------------------------------------------------------------
	-- CODIGO DE PRUEBA PARA FORAZAR DATOS EN LOS ESTADOS
	----------------------------------------------------------------
	-- for statusName in pairs(contactsStatuses) do
	-- 	oeRedis:hdel(key, statusName)
	-- end
	-- for i, statusName in ipairs({"ACTIVE", "CANCELED", "FINISHED", "PAUSED", "SCHEDULED", "FREE", "PROCESS", "PENDING"}) do
	-- 	oeRedis:hset(key, statusName, i)
	-- end
	----------------------------------------------------------------
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.GetContactsStatuses] 1.1--> processId: %s, batchId: %s, redisKey: '%s' statuses: %s", processId, key, batchId, utils.JSON_Encode(contactsStatuses))
	local success = true
	if success then
		-- LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.GetContactsStatuses] 2--> Process: %s, batch: %s. Contact Statuses: %s", processId, batchId, utils.JSON_Encode(contactsStatuses))
		return true, contactsStatuses
	end
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.GetContactsStatuses] Unable to retrieve contacts statuses of Process: %s, batch: %s ", processId, batchId)
	return false, {result = "UNABLE TO RETRIEVE BATCH - CONTACT STATUSES"}
end

--[[

]]--
m_batches.GetTotalIndicators = function (self, processId, batchId)
	assert(processId)
	assert(batchId)
	processId = processId:lower()
	batchId = batchId:lower()
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.GetTotalIndicators] 1--> Get Batch Contacts statuses --> processId: %s, batchId: %s", processId, batchId)
	local info = {processId = processId, batchId = batchId}
	IncludeAdminUser(info)
	local request = json.encode(info)
	local result = GetClient():GetBatchTotalIndicators(request)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.GetTotalIndicators] GetClient():GetTotalIndicators - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.GetTotalIndicators] 2--> Success getting indicators Batch.")
			return true, result.indicators
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO RETRIEVE BATCH INDICATORS"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.GetTotalIndicators] 2--> Error getting indicators Batch --> err: " .. tostring(err))
	return false, err
end

--[[
]]--
m_batches.Update = function (self, batch, vcc, ksOE, successExecutedActionWaitSec, waitForCompleted)
	assert(type(batch) == "table")
	local batchId, batchName = self:MakeBatchId(batch.processId, batch.batchId)
	batch.batchId = batchId
	IncludeAdminUser(batch)
	local encodedBatch = json.encode(batch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Update] 1--> Updating Batch --> encodedBatch: " .. tostring(encodedBatch))
	local success, result = self:CanUpdate(batch, vcc, ksOE)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Update] 1.1--> Error Updating Batch --> err: " .. json.encode(err))
		return false, err
	end
	local result = GetClient():UpdateBatch(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Update] GetClient():UpdateBatch - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, batch.processId)
			end
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Update] 2--> Success Updated Batch.")
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchName, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO UPDATE BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Update] 2--> Error Updating Batch --> err: " .. tostring(err))
	return false, err
end

--[[
]]--
m_batches.Destroy = function (self, processId, batchName, vcc, waitForCompleted, ksOE)
	assert(processId)
	assert(batchName)
	local batchId = self:MakeBatchId(processId, batchName)
	local batch = {processId = processId, batchId = batchId}
	IncludeAdminUser(batch)
	local encodedBatch = json.encode(batch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Destroy] 1--> Destroy Batch --> encodedBatch: " .. tostring(encodedBatch))
	local success, result = self:CanDestroy(batch)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Destroy] 1.1--> Error Destroy Batch --> err: " .. json.encode(err))
		return false, err
	end
	local result = GetClient():DestroyBatch(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Destroy] GetClient():DestroyBatch - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Destroy] 2--> Success Destroyed Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, processId)
			end
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO DESTROY BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Destroy] 2--> Error Destroy Batch --> err: %s", utils.JSON_Encode(err))
	return false, err
end

--[[
]]--
m_batches.MakeActionInfo = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	assert(type(info.processId) == "string")
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.MakeActionInfo] 1-> Making Action Contacts Batch. vcc: '%s', info: '%s'.", vcc, utils.JSON_Encode(info))
	local processId = m_processes:MakeProcessId(info.processId, vcc)
	local batch = {
		processId = processId,
		scope = {
			contacts = info.contacts or nil -- no es requerido, utilizado desde la migracion
		}
	}
	if info.batchId then
		local batchId = self:MakeBatchId(processId, info.batchId)
		batch.scope.batchId = batchId
	else
		local errReason, success, conditions
		if info.query then
			success, conditions = m_queries.queries:MakeQueryConditions(info.query, vcc)
			if not success then
				-- error
				errReason = (conditions or "")
				local err = {type = "UNABLE TO MAKE ACTION CONTACTS CONTACTS BATCH", reason = errReason, subType = errReason:upper()}
				LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.MakeActionInfo] 2--> ERROR - Making Action Contacts Batch --> err: " .. json.encode(err))
				return false, err
			end
		end
		batch.configuration = {}
		batch.scope.conditions = conditions
		batch.scope.excludeContacted = info.excludeContacted
		batch.scope.contactedDate = info.contactedDate
	end
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.MakeActionInfo] 3-> OK - Making Action Contacts Batch. vcc: '%s', batch info: '%s'.", vcc, utils.JSON_Encode(batch))
	return true, batch
end

--[[
]]--
m_batches.GetBatch = function (self, vcc, processId, batchName, mustIncludeContactStatuses, ksOE)
	assert(vcc)
	assert(processId)
	assert(batchName)
	processId = processId:lower()
	batchName = batchName:lower()
	local batch = utils.GetOEModel("OutboundProcessBatches").GetBatch(vcc, processId, batchName, ksOE)
	if utils.TableIsEmpty(batch) then
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.GetBatch] Batch does not exist. vcc: '%s', processId: '%s', batchName: '%s'.", vcc, processId, batchName)
		return false
	end
	if mustIncludeContactStatuses then
		local _, contactsStatuses = self:GetContactsStatuses(processId, batchName)
		batch.contactsStatuses = contactsStatuses or {}
	else
		batch.contactsStatuses = {}
	end
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.GetBatch] vcc: '%s', processId: '%s', batchName: '%s', batch: '%s'.", vcc, processId, batchName, utils.JSON_Encode(batch))
	return true, batch
end

--[[
	-- En el caso de las migraciones no es necesario que el Batch retorne actualizado
		successExecutedActionWaitSec: realiza una pausa personalizada, si el parametro no llega (=nil) el sleep se realiza con el valor por defecto da la funcion Sleep DEFAULT_SLEEP_PROCESS_SEC
]]--
m_batches.ReturnSuccessfullyExecutedAction = function (self, processId, batchName, vcc, ksOE, successExecutedActionWaitSec)
	local batch = {}
	local executedSleepTimeSec
	if batchName then
		executedSleepTimeSec = Sleep(successExecutedActionWaitSec) -- Pequena pausa necesaria para que el OE actualice el batch en cassandra y en redis.
		local mustIncludeContactStatuses = true
		local success, updatedBatch = self:GetBatch(vcc, processId, batchName, mustIncludeContactStatuses, ksOE)
		if success then
			batch = updatedBatch
			--batch.state = "CREATING"
		end
	end
	LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.ReturnSuccessfullyExecutedAction] processId: '%s', batchName: '%s', Sleep(%s), batch: '%s'", processId, batchName, executedSleepTimeSec, utils.JSON_Encode(batch))
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanPause = function (self, vcc, batchInfo)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanPause] 1--> Validate Can Pause Batch")
	local success, batch = self:MakeActionInfo(vcc, batchInfo)
	if not success then
		-- error
		local err = {type = "UNABLE TO PAUSE BATCH", reason = batch.reason}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanPause] 2--> Error Pause Batch --> err: " .. json.encode(err))
		return false, err
	end
	-- ok
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanPause] 2--> Success Can Pause Batch")
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.Pause = function (self, vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Pause] 1--> Pause Batch --> encodedBatch: %s", utils.JSON_Encode(batchInfo))
	local success, batch = self:CanPause(vcc, batchInfo)
	if not success then
		local err = batch
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Pause] 1.1--> Error Pause Batch --> err: " .. json.encode(err))
		return false, err
	end
	IncludeAdminUser(batch)
	local encodedBatch = utils.JSON_Encode(batch)
	local result = GetClient():PauseContacts(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Pause] GetClient():PauseContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Pause] 2--> Success Paused Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, batch.processId)
			end
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchInfo.batchId, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO PAUSE BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Pause] 2--> Error Pause Pause --> err: " .. tostring(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanResume = function (self, vcc, batchInfo)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanResume] 1--> Validate Can Resume Batch")
	local success, batch = self:MakeActionInfo(vcc, batchInfo)
	if not success then
		-- error
		local err = {type = "UNABLE TO PAUSE BATCH", reason = batch.reason}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanResume] 2--> Error Resume Batch --> err: " .. json.encode(err))
		return false, err
	end
	-- ok
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanResume] 2--> Success Can Resume Batch")
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.Resume = function (self, vcc, batchInfo, ksOE, successExecutedActionWaitSec)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Resume] 1--> Resume Batch --> encodedBatch: %s", utils.JSON_Encode(batchInfo))
	local success, batch = self:CanResume(vcc, batchInfo)
	if not success then
		local err = batch
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Resume] 1.1--> Error Resume Batch --> err: " .. json.encode(err))
		return false, err
	end
	IncludeAdminUser(batch)
	local encodedBatch = utils.JSON_Encode(batch)
	local result = GetClient():ResumeContacts(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Resume] GetClient():ResumeContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Resume] 2--> Success Resumed Batch.")
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchInfo.batchId, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO RESUME BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Resume] 2--> Error Resume Batch --> err: " .. tostring(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanCancel = function (self, vcc, batchInfo)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanCancel] 1--> Validate Can Resume Batch")
	local success, batch = self:MakeActionInfo(vcc, batchInfo)
	if not success then
		-- error
		local err = {type = "UNABLE TO CANCEL BATCH", reason = batch.reason}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanCancel] 2--> Error Cancel Batch --> err: " .. json.encode(err))
		return false, err
	end
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.Cancel = function (self, vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Cancel] 1--> Cancel Batch --> encodedBatch: %s", utils.JSON_Encode(batchInfo))
	local success, batch = self:CanCancel(vcc, batchInfo)
	if not success then
		local err = batch
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Cancel] 1.1--> Error Cancel Batch --> err: " .. json.encode(err))
		return false, err
	end
	IncludeAdminUser(batch)
	local encodedBatch = utils.JSON_Encode(batch)
	local result = GetClient():CancelContacts(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Cancel] GetClient():CancelContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Cancel] 2--> Success Canceled Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, batch.processId)
			end
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchInfo.batchId, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO CANCEL BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Cancel] 2--> Error Cancel Batch --> err: " .. tostring(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanFinish = function (self, vcc, batchInfo)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanFinish] 1--> Validate Can Resume Batch")
	local success, batch = self:MakeActionInfo(vcc, batchInfo)
	if not success then
		-- error
		local err = {type = "UNABLE TO FINISH BATCH", reason = batch.reason}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanFinish] 2--> Error Finish Batch --> err: " .. json.encode(err))
		return false, err
	end
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.Finish = function (self, vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Finish] 1--> Finish Batch --> encodedBatch: %s", utils.JSON_Encode(batchInfo))
	local success, batch = self:CanFinish(vcc, batchInfo)
	if not success then
		local err = batch
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Finish] 1.1--> Error Finish Batch --> err: " .. json.encode(err))
		return false, err
	end
	IncludeAdminUser(batch)
	local encodedBatch = utils.JSON_Encode(batch)
	local result = GetClient():FinishContacts(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Finish] GetClient():FinishContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Finish] 2--> Success Finished Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan terminado.
				local isCompleted = self:IsCompletedAction(vcc, batch.processId)
			end
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchInfo.batchId, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO FINISH BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Finish] 2--> Error Finish Batch --> err: " .. tostring(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanRemove = function (self, vcc, batchInfo)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanRemove] 1--> Validate Can Resume Batch")
	local success, batch = self:MakeActionInfo(vcc, batchInfo)
	if not success then
		-- error
		local err = {type = "UNABLE TO REMOVE BATCH", reason = batch.reason}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanRemove] 2--> Error Remove Batch --> err: " .. json.encode(err))
		return false, err
	end
	return true, batch
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.Remove = function (self, vcc, batchInfo, waitForCompleted, ksOE, successExecutedActionWaitSec)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Remove] 1--> Remove Batch --> encodedBatch: %s", utils.JSON_Encode(batchInfo))
	local success, batch = self:CanRemove(vcc, batchInfo)
	if not success then
		local err = batch
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Remove] 1.1--> Error Remove Batch --> err: " .. json.encode(err))
		return false, err
	end
	IncludeAdminUser(batch)
	local encodedBatch = utils.JSON_Encode(batch)
	local result = GetClient():RemoveContacts(encodedBatch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Remove] GetClient():RemoveContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.Remove] 2--> Success Removed Batch. %s", waitForCompleted and "Wait a moment until the batch actions have completed !!" or "")
			if waitForCompleted then
				-- utilizado desde las migraciones, ya que como la ejecucion puede ser inmediata hay que esperar un determinado tiempo para que las acciones del OE Client sobre los batches que hayan termiando.
				local isCompleted = self:IsCompletedAction(vcc, batch.processId)
			end
			return self:ReturnSuccessfullyExecutedAction(batch.processId, batchInfo.batchId, vcc, ksOE, successExecutedActionWaitSec)
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO REMOVE BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.Remove] 2--> Error Remove Batch --> err: " .. tostring(err))
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanSearchContacts = function (self, batch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanSearchContacts] 1--> Validate Can Search Contacts Batch")
	return true
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.SearchContacts = function (self, vcc, queryInfo, rowsPerPage, includedStates)
	assert(type(vcc) == "string")
	assert(type(queryInfo) == "table")
	assert(type(queryInfo.conditions) == "table")
	queryInfo.vcc = vcc
	queryInfo.size = rowsPerPage
	if type(includedStates) == "table" then
		queryInfo.includedStates = includedStates
	end
	local encodedQueryInfo = json.encode(queryInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SearchContacts] 1--> Searching Contacts Batch --> encodedBatch: " .. tostring(encodedQueryInfo))
	local success, result = self:CanSearchContacts(queryInfo)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SearchContacts] 1.1--> Error Searching Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	local result = GetClient():SearchContacts(encodedQueryInfo)
	--LogDebug("result: %s", result)
	--LogDebug(json.encode(result))
	if result then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SearchContacts] 2--> Success Searched Contacts Batch")
		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.SearchContacts] 2.1--> result: %s", json.encode(result or {}))
		return true, result
	end
	-- error
	local err = MakeError({type = "UNABLE TO SEARCH CONTACTS BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SearchContacts] 3--> Error Searching Contacts Batch --> err: " .. tostring(err))
	return false, err
end

--[[
	Para los reportes por ejemplo
]]--
m_batches.RawSearchContacts = function(self, queryInfo)
	local encodedQueryInfo = json.encode(queryInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RawSearchContacts] 1--> " .. tostring(encodedQueryInfo))
	local result = GetClient():SearchContacts(encodedQueryInfo)
	--LogDebug("result: %s", result)
	--LogDebug(json.encode(result))
	if result then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RawSearchContacts] 2--> Success Searched Contacts")
		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.RawSearchContacts] 2.1--> result: %s", json.encode(result or {}))
		return true, result
	end
	-- error
	local err = MakeError({type = "UNABLE TO SEARCH CONTACTS"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.RawSearchContacts] 3--> Error Searching Contacts --> err: " .. tostring(err))
	return false, err
end

--[[
	Obtiene un contacto
]]--
m_batches.GetContact = function(self, vcc, contactId)
	local encodedQueryInfo = json.encode({
		vcc = vcc,
		contactId = contactId
	})
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.GetContact] 1--> " .. tostring(encodedQueryInfo))
	local result = GetClient():GetContact(encodedQueryInfo)
	--LogDebug("result: %s", result)
	--LogDebug(json.encode(result))
	if result then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.GetContact] 2--> Success Loaded Contact")
		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.RawSearchContacts] 2.1--> result: %s", json.encode(result or {}))
		return true, result
	end
	-- error
	local err = MakeError({type = "UNABLE TO LOAD CONTACTS"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.GetContact] 3--> Error Loading Contact --> err: " .. tostring(err))
	return false, err
end

-----------------------------------------------------------------------------
-- BATCHES - CONTACTS
------------------------------------------------------------------------------

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanSummaryContacts = function (self, batch)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanSummaryContacts] 1--> Validate Can Summary Contacts Batch")
	return true
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.SummaryContacts = function (self, vcc, queryInfo)
	assert(type(vcc) == "string")
	assert(type(queryInfo) == "table")
	assert(type(queryInfo.conditions) == "table")
	queryInfo.vcc = vcc
	local encodedQueryInfo = json.encode(queryInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SummaryContacts] 1--> Summary Contacts Batch --> encodedBatch: " .. tostring(encodedQueryInfo))
	local success, result = self:CanSummaryContacts(queryInfo)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SummaryContacts] 1.1--> Error Summary Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	local result = GetClient():SummaryContacts(encodedQueryInfo)
	-- LogDebug("result: %s", result)
	-- LogDebug(json.encode(result))
	if result then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SummaryContacts] 2--> Success Searched Contacts Batch")
		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.SummaryContacts] 2.1--> result: %s", json.encode(result or {}))
		return true, result
	end
	-- error
	local err = MakeError({type = "UNABLE TO SUMMARY CONTACTS"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SummaryContacts] 3--> Error Summary Contacts Batch --> err: " .. tostring(err))
	return false, err
end

-----------------------------------------------------------------------------
-- BATCHES - CONTACTS
------------------------------------------------------------------------------

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanRecycleContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	assert(type(info) == "table")
	assert(type(info.processId) == "string")
	assert(type(info.query) == "table")
	assert(type(info.configuration) == "table")
	assert(type(info.configuration.restartRetries) == "boolean")
	if (info.configuration.scheduleDate) then
		assert(info.configuration.scheduleHour, "hour is required for activation scheduler")
	end
	local errReason
	local success, conditions = m_queries.queries:MakeQueryConditions(info.query, vcc)
	if not success then
		-- error
		errReason = (conditions or "")
		local err = {type = "UNABLE TO RECYCLE CONTACTS CONTACTS BATCH", reason = errReason, subType = errReason:upper()}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanRecycleContacts] 2--> Error Recycle Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	local scheduleDate
	if info.configuration.scheduleDate then
		local scheduleHour = info.configuration.scheduleHour
		if scheduleHour then
			scheduleHour = scheduleHour .. ":00"
		end
		scheduleDate = utils.ConvertDateToUTCDate(info.configuration.scheduleDate, scheduleHour, "%Y-%m-%d %H:%M:%S")
		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_batches.CanRecycleContacts]  --> scheduleDate: " .. scheduleDate)
	end
	local data = {
		processId = m_processes:MakeProcessId(info.processId, vcc),
		configuration = {
			restartRetries = info.configuration.restartRetries or false,
			agent          = utils.GetId(info.configuration.agent),
			scheduleDate   = scheduleDate
		},
		scope = {
			conditions = conditions,
			excludeContacted = info.excludeContacted,
			contactedDate = info.contactedDate
		}
	}
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanRecycleContacts] 2--> Success Validation Can Recycled BatchInfo: %s", json.encode(data))
	return true, data
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.RecycleContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RecycleContacts] 1--> Recycle Contacts Batch --> encodedBatch: " .. tostring(json.encode(info)))
	local success, result = self:CanRecycleContacts(vcc, info)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.RecycleContacts] 1.1--> Error Recycle Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	info = result
	IncludeAdminUser(info)
	local encodedInfo = json.encode(info)
	local result = GetClient():RecycleContacts(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RecycleContacts] GetClient():RecycleContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RecycleContacts] 2--> Success Recycled Contacts Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO RECYCLE BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.RecycleContacts] 2--> Error Recycle Contacts Batch --> err: %s", result.reason)
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanChangePriorityContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	assert(type(info) == "table")
	assert(type(info.processId) == "string")
	assert(type(info.query) == "table")
	assert(type(info.configuration) == "table")
	assert(info.configuration.batchPriority)
	local errReason
	local success, conditions = m_queries.queries:MakeQueryConditions(info.query, vcc)
	if not success then
		-- error
		errReason = (conditions or "")
		local err = {type = "UNABLE TO CHANGE PRIORITY CONTACTS BATCH", reason = errReason, subType = errReason:upper()}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanChangePriorityContacts] 2--> Error Change Priority Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	local data = {
		processId = m_processes:MakeProcessId(info.processId, vcc),
		priority = tonumber(info.configuration.batchPriority),
		scope = {
			conditions = conditions,
			excludeContacted = info.excludeContacted,
			contactedDate = info.contactedDate
		}
	}
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanChangePriorityContacts] 2--> Success Validation Can Changed Priority BatchInfo: %s", json.encode(data))
	return true, data
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.ChangePriorityContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.ChangePriorityContacts] 1--> Change Priority Contacts Batch --> encodedBatch: " .. tostring(json.encode(info)))
	local success, result = self:CanChangePriorityContacts(vcc, info)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.ChangePriorityContacts] 1.1--> Error Change Priority Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	info = result
	IncludeAdminUser(info)
	local encodedInfo = json.encode(info)
	local result = GetClient():ChangePriorityContacts(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.ChangePriorityContacts] GetClient():ChangePriorityContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.ChangePriorityContacts] 2--> Success Change Priority Contacts Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO CHANGE PRIORITY CONTACTS BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.ChangePriorityContacts] 2--> Error Change Priority Contacts Batch --> err: %s", result.reason)
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanSetEngineTypeContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	assert(type(info) == "table")
	assert(type(info.processId) == "string")
	assert(type(info.query) == "table")
	assert(type(info.configuration) == "table")
	local errReason
	local success, conditions = m_queries.queries:MakeQueryConditions(info.query, vcc)
	if not success then
		-- error
		errReason = (conditions or "")
		local err = {type = "UNABLE TO SET ENGINE TYPE CONTACTS CONTACTS BATCH", reason = errReason, subType = errReason:upper()}
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.CanSetEngineTypeContacts] 2--> Error CSet Engine Type Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	local data = {
		processId = m_processes:MakeProcessId(info.processId, vcc),
		engineType = tostring(info.configuration.engineType):upper(), -- PROGRESSIVE or ""
		scope = {
			conditions = conditions,
			excludeContacted = info.excludeContacted,
			contactedDate = info.contactedDate
		}
	}
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanSetEngineTypeContacts] 2--> Success Validation Can Changed Priority BatchInfo: %s", json.encode(data))
	return true, data
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.SetEngineTypeContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SetEngineTypeContacts] 1--> Set Engine Type Contacts Batch --> encodedBatch: " .. tostring(json.encode(info)))
	local success, result = self:CanSetEngineTypeContacts(vcc, info)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SetEngineTypeContacts] 1.1--> Error Set Engine Type Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	info = result
	IncludeAdminUser(info)
	local encodedInfo = json.encode(info)
	local result = GetClient():SetEngineTypeContacts(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SetEngineTypeContacts] GetClient():SetEngineTypeContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.SetEngineTypeContacts] 2--> Success Set Engine Type Contacts Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO SET TYPE CONTACTS BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.SetEngineTypeContacts] 2--> Error Set Engine Type Contacts Batch --> err: %s", result.reason)
	return false, err
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.CanStoreContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	assert(type(info) == "table")
	assert(type(info.contacts) == "table")
	local data = {
		vcc = vcc,
		contacts = info.contacts,
	}
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.CanStoreContacts] 2--> Success Validation Can Changed Priority BatchInfo: %s", json.encode(data))
	return true, data
end

--[[
	Las funciones de validaciones quedan por si hay que realizar alguna validacion en los controllers.
	Ahora siempre esta retornando (success = true).
]]--
m_batches.StoreContacts = function (self, vcc, info)
	assert(type(vcc) == "string")
	assert(type(info) == "table")
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.StoreContacts] 1--> Save Contacts Batch --> encodedBatch: " .. tostring(json.encode(info)))
	local success, result = self:CanStoreContacts(vcc, info)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.StoreContacts] 1.1--> Error Save Contacts Batch --> err: " .. json.encode(err))
		return false, err
	end
	info = result
	IncludeAdminUser(info)
	local encodedInfo = json.encode(info)
	local result = GetClient():StoreContacts(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.StoreContacts] GetClient():StoreContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.StoreContacts] 2--> Success Save Contacts Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO SAVE CONTACTS BATCH"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.StoreContacts] 2--> Error Save Contacts Batch --> err: %s", result.reason)
	return false, err
end

--[[
]]--
m_batches.RescheduleContact = function (self, info)
	assert(type(info) == "table")
	IncludeAdminUser(info)
	local encodedInfo = json.encode(info)
	local result = GetClient():RescheduleContact(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RescheduleContact] GetClient():RescheduleContact - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_batches.RescheduleContact] 2--> Success Rescheduling Contact Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO RESCHEDULE CONTACT"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_batches.RescheduleContact] 2--> Error Rescheduling Contact Batch --> err: %s", result.reason)
	return false, err
end

-----------------------------------------------------------------------------
-- PROCESSES
-----------------------------------------------------------------------------

--[[
]]--
m_processes.MakeProcessId = function (self, processId, vcc)
	assert(type(processId) == "string")
	assert(type(vcc) == "string")
	processId = utils.AppendVcc(utils.GetId(processId), vcc):lower()
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.MakeProcessId] --> Making ProcessId --> %s", processId)
	return processId
end

--[[
]]--
m_processes.MakeProcess = function (self, processId, vcc)
	assert(type(processId) == "string")
	assert(type(vcc) == "string")
	processId = self:MakeProcessId(processId, vcc)
	local process = {processId = processId}
	IncludeAdminUser(process)
	local encodedProcess = json.encode(process)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.MakeProcess] --> Making Process --> encodedProcess: " .. tostring(encodedProcess))
	return process, encodedProcess
end

--[[
]]--
m_processes.CanStart = function (self, processId, vcc)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.CanStart] 1--> Validate Can Start Process")
	local isRunning, result = m_processes:IsRunning(processId, vcc)
	if not isRunning then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.CanStart] 2--> Success Can Start Process")
		return true
	end
	-- error
	local err = {type = "UNABLE TO START PROCESS", reason = result}
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.CanStart] 2--> Error Start Process --> err: " .. json.encode(err))
	return false, err
end

--[[
]]--
m_processes.CanStop = function (self, processId, vcc)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.CanStop] 1--> Validate Can StarStopt Process")
	local isRunning, result = m_processes:IsRunning(processId, vcc)
	if isRunning then
		-- ok
		--LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.CanStop] 2--> Success Can Stop Process")
		return true
	end
	-- error
	local err = {type = "UNABLE TO STOP PROCESS", reason = result}
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.CanStop] 2--> Error Stop Process --> err: " .. json.encode(err))
	return false, err
end

--[[
]]--
m_processes.Start = function (self, processId, vcc)
	local process, encodedProcess = self:MakeProcess(processId, vcc)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.Start] Starting Process --> encodedProcess: " .. tostring(encodedProcess))
	local success, result = self:CanStart(process.processId, vcc)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.Start] 1.1--> Error Start Process --> err: " .. json.encode(err))
		return false, err
	end
	local success, result = GetClient():StartProcess(encodedProcess)
	--LogDebug(json.encode(success))
	if success then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.Start] Success Started Process")
		return true, result
	end
	-- error
	local err = result
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.Start] Error Starting Process --> err: " .. tostring(err))
	return false, err
end

--[[
]]--
m_processes.Stop = function (self, processId, vcc)
	local process, encodedProcess = self:MakeProcess(processId, vcc)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.Stop] 1--> Stopping Process --> encodedProcess: " .. tostring(encodedProcess))
	local success, result = self:CanStop(process.processId, vcc)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.Stop] 1.1--> Error Stop Process --> err: " .. json.encode(err))
		return false, err
	end
	local success, result = GetClient():StopProcess(encodedProcess)
	--LogDebug(json.encode(success))
	if success then
		-- ok
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.Start] Success Stopped Process")
		return true, result
	end
	-- error
	local err = result
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.Start] Error Stopping Process --> err: " .. tostring(err))
	return false, err
end

--[[
]]--
m_processes.IsRunning = function (self, processId, vcc)
	local process, encodedProcess = self:MakeProcess(processId, vcc)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.IsRunning] Validating if the process is running --> encodedProcess: " .. tostring(encodedProcess))
	local success, status = self:GetStatus(process.processId)
	--local isRunning = status == "RUNNING"
	if success then
		local runningStatuses = {
			STARTING = true,
			STARTED  = true
		}
		local isRunning = runningStatuses[status]
		if isRunning then
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.IsRunning] Process Is Running: " .. tostring(processId))
			return true, "PROCESS IS RUNNING"
		end
	end
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.IsRunning] Process Is Not Running: " .. tostring(processId))
	return false, "PROCESS NOT IS RUNNING"
end

--[[
]]--
m_processes.GetStatus = function (self, processId)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.GetStatus] 1--> Get process status --> processId: " .. tostring(processId))
	--local processesController = require "apps.outbound_engine.controllers.processes"
	--local success = true --  GetClient():GetProcessStatus(encodedProcess)
	--local status = "ACTIVE" -- ACTIVE -- RUNNING

	local outboundEngineAPI = utils.GetOEApi() -- Obtengo el API del "outbound_engine".
	local success, status = outboundEngineAPI.native.processes.GetProcessStatus(processId)
	if success then
		LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.GetStatus] 2--> Process " .. tostring(processId) .. " status " .. tostring(status))
		return true, status:upper()
	end
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.GetStatus] 3--> -Unable to retrieve status of Process: " .. tostring(processId))
	return false, status
end

--[[
]]--
m_processes.GetRunningBatchActions = function (self, processId, vcc, isAdmin)
	assert(processId)
	assert(vcc)
	processId = processId:lower()
	processId = self:MakeProcessId(processId, vcc)
	isAdmin = type(isAdmin) == "boolean" and isAdmin or true
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.GetRunningBatchActions] 1-- > Loading Running Batch Actions in process: '%s', isAdmin: '%s'." , processId, isAdmin)
	local processInfo = {processId = processId, isAdmin = isAdmin}
	IncludeAdminUser(processInfo)
	local result = GetClient():GetRunningBatchActions(json.encode(processInfo))
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.GetRunningBatchActions] GetClient():GetRunningBatchActions - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			---- ok
			--LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.GetRunningBatchActions] 2--> Success Loaded Running Batchs Actions.")
			return true, result
		end
	end
	-- error
	result = result or {}
	-- error
	local err = result
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.GetRunningBatchActions] Error Loading Running Batchs Actions --> err: %s", json.encode(err))
	return false, err
end

--[[
]]--
m_processes.GetStatusCause = function (self, processId, vcc)
	assert(processId)
	assert(vcc)
	processId = processId:lower()
	processId = self:MakeProcessId(processId, vcc)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.GetStatusCause] 1-- > Loading Status Cause in process: '%s'." , processId)
	local processInfo = { processId = processId }
	IncludeAdminUser(processInfo)
	local result = GetClient():GetStatusCause(json.encode(processInfo))
	return result
end

--[[
]]--
m_processes.CanReloadContacts = function (self, processId, vcc)
	--LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.CanReloadContacts] 1--> Validate Can Reload process contacts")
	return true
end

--[[
]]--
m_processes.ReloadContacts = function (self, info)
	assert(type(info) == "table")
	-- IncludeAdminUser(info)
	local encodedInfo = utils.JSON_Encode(info)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] 1--> Reloading Contacts of Process --> info: %s", encodedInfo)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] 1--> Reloading %d contacts of Process: '%s'.", #((info.scope or {}).contacts or {}), info.processId)
	local success, result = self:CanReloadContacts(info)
	if not success then
		local err = result
		LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] 1.1--> Error Reloading Contacts of Process --> err: %s", utils.JSON_Encode(err))
		return false, err
	end
	local result = GetClient():ReloadContacts(encodedInfo)
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] GetClient():ReloadContacts - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] 2--> Success Reload Contacts Batch.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "UNABLE TO RELOAD CONTACTS"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.ReloadContacts] 2--> Error Reloading Contacts Batch --> err: %s", result.reason)
	return false, err
end

--[[
]]--
m_processes.AreAllVccProcessesReady = function (self, info)
	assert(type(info) == "table")
	assert(type(info.vcc) == "string")
	IncludeAdminUser(info)
	local result = GetClient():AreAllVccProcessesReady(utils.JSON_Encode(info))
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.AreAllVccProcessesReady] GetClient():AreAllVccProcessesReady - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_processes.AreAllVccProcessesReady] 1--> Success: Are All Vcc Prcoesses Ready.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "NOT ARE ALL VCC PRCOESSES READY"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_processes.AreAllVccProcessesReady] 2--> Error: NOT Are All Vcc Prcoesses Ready --> err: %s", result.reason)
	return false, err
end

-----------------------------------------------------------------------------
-- NODES
-----------------------------------------------------------------------------

--[[
]]--
m_nodes.AreNodesReady = function (self)
	local result = GetClient():AreNodesReady()
	LogInfo("[OUTBOUNDENGINE.CLIENT:m_nodes.AreNodesReady] GetClient():AreNodesReady - result: %s", utils.JSON_Encode(result))
	if type(result) == "table" then
		if (result.result or ""):upper() == "OK" then
			-- ok
			LogInfo("[OUTBOUNDENGINE.CLIENT:m_nodes.AreNodesReady] 2--> Success: All Nodes Are Ready.")
			return true, result
		end
	end
	-- error
	local err = MakeError({type = "NO ALL NODES ARE READY"}, result)
	LogWarning("[OUTBOUNDENGINE.CLIENT:m_nodes.AreNodesReady] 2--> Error: Not All Nodes Are Ready. --> err: %s", result.reason)
	return false, err
end

return {
	batches   = m_batches,
	nodes     = m_nodes,
	processes = m_processes,
	queries   = m_queries,
	GetClient = GetClient
}