/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form blacklist imports class
	function form_outbound_engine_blacklist_imports(params)
	{
		const self = this;
		const success = form_outbound_engine_blacklist_imports.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('New Import')%>";
		this.update_title = "<%=_T('Update Blacklist Importations {0}')%>";
		this.view_title = "<%=_T('View Blacklist Imports {0}')%>";
		this.tagline = "<%=_T('Tagline form Import!!!')%>";
	
		this.html_file_name = "/inconcert/apps/outbound_engine/blacklist/frm/render/";
		this.get_uri =        "/inconcert/apps/outbound_engine/blacklist/get/";

		this.form_paginator_past = null;
		this.__new_grids();

		this.m_csv_result = null;

		this.import_started_timestamp = null;
		this.is_uploaded_file = false;

		this.upload_instance = new inConcert.RepositoryUpload({
			container_id: "out_form_blacklist_import_upload",
			must_validate_max_size: false,
			availables_extensions: {
				"csv": true
			},
			show_status_message_fn: function(message, type, time_ms) {
				self.show_status_message(message, type, time_ms);
			},
			on_uploading_fn: function(success, data) {
				self.on_uploading_fn(success, data);
			},
			on_uploaded_fn: function(success, data, callback) {
				self.on_uploaded(success, data, callback);
			},
			can_upload_file_fn: function(callback) {
				callback = callback || $.noop;
				self.can_upload_file(callback);
			}
		});

		return true;
	}

	inConcert.Utils.inherit(form_outbound_engine_blacklist_imports, inConcert.conf_form);

	if (typeof inConcert.form_outbound_engine_blacklist_imports !== "function") {
		inConcert.form_outbound_engine_blacklist_imports = form_outbound_engine_blacklist_imports;
	}

	form_outbound_engine_blacklist_imports.prototype.initialize = function()
	{
		this.upload_instance.initialize();
		this.__load_allowed_csv_columns();
		this.is_uploaded_file = false;
		this.m_csv_result = null;
	};

	form_outbound_engine_blacklist_imports.prototype.dispose = function()
	{
		form_outbound_engine_blacklist_imports.uber.dispose.call(this); //invokes the base class method
		this.upload_instance.dispose();
		this.is_uploaded_file = false;
		this.m_csv_result = null;
		this.m_mustExecuteGoBackToGridAfterSaved = false;
	};

	form_outbound_engine_blacklist_imports.prototype.get = function()
	{
		return  {
			IsUpdate   : this.is_new ? false: true,
			ID         : $("#out_form_blacklist_import_id").val(),
			AddressType: $("#out_form_blacklist_import_address_type").val(),
			FileInfo   : this.upload_instance.get_info_uploaded_files(),
			Description: $("#out_form_blacklist_import_desc").val()
		};
	};

	form_outbound_engine_blacklist_imports.prototype.set = function(success, importation)
	{
		this.form_paginator_past.initialize();
		if (!success) {
			return;
		}
		$("#out_form_blacklist_import_id").val(importation.ID);
		$("#out_form_blacklist_import_address_type").val(importation.AddressType);
		$("#out_form_blacklist_import_desc").val(importation.Description);
	};

	form_outbound_engine_blacklist_imports.prototype.can_upload_file = function(callback =  $.noop)
	{
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.hasSomeImportationRunning(function(success, response) {
			let err;
			if (!success) {
				err = "<%=_T('Unable to Upload File.')%>";
				callback(false, err);
				return;
			}
			if (response.isRunning) {
				err = "<%=_T('Unable to Upload File. Exists another importation in process. please try again later.')%>";
				callback(false, err);
				return;
			}
			callback(true);
		});
	};

	form_outbound_engine_blacklist_imports.prototype.on_uploading_fn = function()
	{
		this.disable_footer_buttons(true);
	};

	form_outbound_engine_blacklist_imports.prototype.on_uploaded = function(success, fileInfo, callback = $.noop)
	{
		const self = this;
		this.disable_footer_buttons(false);
		if (!success) {
			callback();
			return;
		}
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.blackListImportGetUploadedCSVHeader({fileId: fileInfo.fileId}, function(success, response) {
			self.m_csv_result = response;
			self.__mark_csv_columns(self.m_csv_result.csvHeader, "SUCCESS");
			if (!success) {
				response.info.timemout_ms = 10000;
				self.__show_error(self.m_csv_result.info);
			}
		});
		callback();
	};

	form_outbound_engine_blacklist_imports.prototype.__new_grids = function()
	{
		const self = this;

		// Instancia de la grilla para VCC SIN Integracion con Allegro.
		this.form_paginator_past = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_IMPORT_PAST",
			owner: this,
			frm: this,
			row_id: "DetailId",
			display_name: "Date",
			rows_per_page: 10,
			show_at_least: 9,
			ids: {
				grid: "out_form_blacklist_import_past_list",
				paginator: "out_form_blacklist_import_past_paginator",
				not_results_found: "out_form_blacklist_import_past_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Past found.')%>"
			},
			columns: [
				{id: "_number"        , name: "", type: "counter"         },
				{
					id       : "StartDate",
					name     : "<%=_T('Date')%>",
					type     : "timestampSec",
					date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
				},
				{id: "UserId"     , name: "<%=_T('User')%>"           },
				{id: "Added"    , name: "<%=_T('New')%>"            },
				{id: "Updated", name: "<%=_T('Updated')%>"        },
				{id: "Errors", name: "<%=_T('Invalid')%>"        },
				{id: "_action", name: "",
					is_enabled: function(action_id, importation) {
						if (action_id === "download") {
							if (!importation || (importation.Errors || 0) === 0) {
								return 0;
							}
							return 1;
						}
						return 0;
					},
					custom_action: {
						download: {
							execute: function($row /*, $owner*/) {
								const importation = $row.data("row_data");
								self.__download_failed_contacts(importation);
							},
							class_name: "far fa-download"
						}
					}
				}
			]
		});
	};

	form_outbound_engine_blacklist_imports.prototype.__download_failed_contacts = function(importation)
	{
		const self = this;
		if (!importation) {
			return;
		}
		let info = {};
		if ((importation.Errors || 0) === 0) {
			return;
		}
		info.ImportId = importation.ImportId;
		info.ImportDetailId = importation.DetailId;
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.blackListImportDownloadFailedAddresses(info, function(success/*, err*/) {
			if (!success) {
				self.show_status_message( "<%=_T('Unable to Download Failed Contacts.')%>", "Error", 5000 );
			}
		});
	};

	/*
		Retorna los datos de la grilla de Importaciones anteriores.
	*/
	form_outbound_engine_blacklist_imports.prototype.load_items = function(_, callback = $.noop)
	{
		let importation = this.get_entity_instance();
		let list = importation.Detail;
		callback(true, list);
	};

	form_outbound_engine_blacklist_imports.prototype.load_data = function(callback_complete_requests)
	{
		form_outbound_engine_blacklist_imports.uber.load_data.call( this, callback_complete_requests ); //invokes the base class method
		this.__fill_address_type();
	};

	form_outbound_engine_blacklist_imports.prototype.__fill_address_type = function()
	{
		const $combo = $("#out_form_blacklist_import_address_type");
		let list = [
			{Id: "PHONE", Name: "<%=_T('Phone')%>"}
		];
		this.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Name" /* attr_name */, true);
	};

	form_outbound_engine_blacklist_imports.prototype.set_events = function()
	{

		this.form_paginator_past.set_events();
	};

	form_outbound_engine_blacklist_imports.prototype.ready = function()
	{
		const $id = $("#out_form_blacklist_import_id");
		const $addressType = $("#out_form_blacklist_import_address_type");
		const $past_list_container = $("#out_form_blacklist_import_past_list_container");
		if ( this.is_new ) {
			this.readonly_control($addressType, false /* disabled */, true /* is_required */);
			this.readonly_control($id, false /* disabled */, true /* is_required */);
			$id.focus();
			$past_list_container.hide();
		}
		else {
			this.readonly_control($id, true /* disabled */ );
			this.readonly_control($addressType, true /* disabled */ );
			$past_list_container.show();
		}
		form_outbound_engine_blacklist_imports.uber.ready.call(this); //invokes the base class method
	};

	form_outbound_engine_blacklist_imports.prototype.__update_buttons = function()
	{
		const $save = this.$get_dom_element_by_id("form_button_save");
		const $save_and_continue = this.$get_dom_element_by_id("form_button_save_and_continue");
		$save.attr("value", "<%=_T('Start')%>");
		$save_and_continue.hide();
	};

	form_outbound_engine_blacklist_imports.prototype.ready_for_update = function()
	{
		form_outbound_engine_blacklist_imports.uber.ready_for_update.call( this ); //invokes the base class method
		this.__update_buttons();
	};

	form_outbound_engine_blacklist_imports.prototype.ready_for_save = function()
	{
		form_outbound_engine_blacklist_imports.uber.ready_for_save.call( this ); //invokes the base class method
		this.__update_buttons();
	};

	form_outbound_engine_blacklist_imports.prototype.get_validation_def = function()
	{
		const self = this;
		const definitions = [
			{
				control_id: "out_form_blacklist_import_id",
				messages: {
					required: "<%=_T('Please enter the Importation Name.')%>",
					id      : "<%=_T('Only letters, numbers, dashes and dots are allowed in the Importation Name.')%>"
				}
			},
			{
				control_id: "out_form_blacklist_import_address_type",
				messages: {
					required: "<%=_T('Please select the Importation Type.')%>"
				}
			},
			{
				control_id: this.upload_instance.get_dom_upload_file_btn_id(),
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							const response = {
								success: true
							};
							const uploaded_file = self.upload_instance.get_info_uploaded_files();
							if (!uploaded_file) {
								response.success = false;
								response.message = "<%=_T('You must Upload a Addresses file.')%>";
								return response;
							}
							if (self.m_csv_result.result.toUpperCase() !== "OK") {
								response.success = false;
								response.message = self.__make_csv_header_error(self.m_csv_result.info);
								return response;
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};

	form_outbound_engine_blacklist_imports.prototype.save_execute = function(after_save, importation /*, hash_id */)
	{
		const self = this;

		this.show_waiting( "<%=_T('Please wait, Starting Importation....')%>" );

		let timeout_hnd = null;
		const clear_timeout = function () {
			if (timeout_hnd) {
				window.clearTimeout(timeout_hnd);
				timeout_hnd = null;
			}
		};

		this.m_mustExecuteGoBackToGridAfterSaved = true;
		// Funcion que regresa a la vista cuando el upload se ha realizado correctamente y el start se encuentra en progreso o ha finalizado.
		const go_back_grid = function(msg) {
			clear_timeout();
			self.import_started_timestamp = new Date();
			self.hide_waiting();
			if (msg) self.show_status_message(msg, "Info", 5000);
			const not_show_grid_message = true;
			self.on_saved(true, importation, after_save, null, null, not_show_grid_message);
		};

		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		const import_start = function()
		{
			clear_timeout();
			// Si el upload de file y sus validaciones no han dado error, regreso a la vista.
			//  El regreso se realiza en un timeout, ya que la importacion comienza a correr en el server de 5.5 y puedo durar varios minutos.
			//  Y suponemos que si pasaron N segundos la misma ha comenzado correctamente.
			const backFn = function() {
				// Si se ha realizado correctamente el upload de un server al otro, debo limpiar la informacion de los uploads de la instancia de los upload.
				after_save = !self.is_new ? "SAVE_AND_REFRESH_VIEW" : after_save;
				go_back_grid();
			};
			backFn();

			outbound_engine_api.blackListImportStart(importation, function(success, response)
			{
				if (success) {
					// Si se ha realizado correctamente el upload de un server al otro, debo limpiar la informacion de los uploads de la instancia de los upload.
					self.upload_instance.destroy_data();
					after_save = !self.is_new ? "SAVE_AND_REFRESH_VIEW" : after_save;
					let msg = "<%=_T('The importation has been successful.')%>";
					go_back_grid(msg);
					return;
				}
				clear_timeout();
				self.hide_waiting();
				let err = response;
				let current_err;
				// allegro
				try {current_err = JSON.parse(err.err);} catch(e) {}
				let code_error;
				if (current_err) {
					code_error = parseInt(current_err.result, 10);
				}
				else {
					// tel i6
					code_error = err.code;
				}
				let msg = "<%=_T('Unable to Import.')%>";
				switch(code_error) {
					case -1:
						//importation failed/not started
						msg = "<%=_T('Importation failed to start, please contact system administration')%>";
						break;

					case -100:
						//error in format definition
						msg = "<%=_T('The selected format does not match the file headers')%>";
						break;

					case -104:
						//...
						msg = FORMAT("<%=_T('The import must not exceed {0} contacts. The current import contains {1} contacts.')%>", err.maximumNumberRowsInImport, err.currentRows);
						break;

					case -105:
						//...
						msg = "<%=_T('The CSV file you uploaded has been deleted. You must upload it again.')%>";
						break;

					case -106:
						//...
						if (typeof err.result === "object") {
							if (err.result.status === "BREAK IMPORT") {
								msg = FORMAT("<%=_T('The import has been aborted. The first {0} contacts have been unsuccessful.')%>", err.result.MAXIMUM_FIRST_FAILED_CONTACTS_FOR_BREAK_IMPORT);
							}
						}
						break;

					case -107:
						//...
						msg = "<%=_T('Unable to complete action. There is another importation in progress.')%>";
							break;

					case -200:
						//the file wasn't correctly uploaded
						msg = "<%=_T('The file was not correctly uploaded')%>";
						break;

					case -300:
						//format not found
						msg = "<%=_T('The format definition was not found on the database.')%>";
						break;

					case -400:
						//format not found
						msg = "<%=_T('Error opening file.')%>";
						if (current_err.message) {
							msg += current_err.message;
						}
						break;

					case -500:
						//Exists another importation in process
						msg = "<%=_T('Exists another importation in process. please try again later.')%>";
						break;

					case "UNSUPPORTED ADDRESS TYPE":
						// tipo de address no soportada
						self.m_mustExecuteGoBackToGridAfterSaved = false;
						msg = "<%=_T('Unsupported address type.')%>";
						self.show_validation_error($("#out_form_blacklist_import_address_type"), msg);
						return;

					case "UNABLE TO CREATE IMPORTATION - ALREADY EXIST":
						// La importacion con este nombre ya existe.
						self.m_mustExecuteGoBackToGridAfterSaved = false;
						msg = "<%=_T('Importation Name is already in use.')%>";
						self.show_validation_error($("#out_form_blacklist_import_id"), msg);
						return;
				}
				// si es tel nativa, muestra el error luego de refrescar la vista, en ambos lugares (formulario y vista)
				self.dashboard.view.refresh(null, function() {
					window.setTimeout(() => {
						self.show_status_message_in_both(msg, "Error", 5000);
					});
				});
			});
		};

		if (this.is_uploaded_file) {
			// El archivo fue subido previamente hacia 5.5, entonces ejecuto la importacion.
			import_start();
			return;
		}

		// a - Primero realizo el Upload del archivo desde el server de i6 hacia el server de 5.5.
		outbound_engine_api.importUploadContactsFile(importation, function(success, response)
		{
			if (success) {
				self.is_uploaded_file = true;
				// Despues de subir el archivo tengo que comenzar la importacion.
				import_start();
			}
			else {
				self.hide_waiting();
				let err = response;
				let current_err;
				try { current_err = JSON.parse(err.err); }
				catch(e) {}
				let code_error;
				if (current_err) {
					code_error = parseInt(current_err.result, 10);
				}
				if (code_error === -10) {
					self.show_validation_error($("#out_form_blacklist_import_id"), "<%=_T('Importation Name is already in use.')%>");
				}
				else if (code_error === -500) {
					self.show_validation_error($("#out_form_blacklist_import_id"), "<%=_T('Exists another importation in process. please try again later.')%>");
				}
				else {
					self.show_status_message("<%=_T('Unable to Upload File.')%>", "Error", 5000);
				}
			}
		});
	};

	form_outbound_engine_blacklist_imports.prototype.on_saved = function(success, importation, after_save, err, callback, not_show_grid_message)
	{
		const self = this;
		// Un timeout para que el mensaje del frm se visualice por un tiempo determinado antes de ser invocada la funcion
		window.setTimeout(function() {
			if (self.m_mustExecuteGoBackToGridAfterSaved) {
				self.upload_instance.destroy_data();
				form_outbound_engine_blacklist_imports.uber.on_saved.call(self, success, importation, after_save, err, callback, not_show_grid_message); //invokes the base class method
			}
		}, 3000);
	};

	form_outbound_engine_blacklist_imports.prototype.__make_allowed_columns = function(allowed_csv_columns)
	{
		const $header = $("#out_form_blacklist_import_allowed_csv_columns");
		$header.html("");
		$.each(allowed_csv_columns || [], function(_, column) {
			if (!column.IsHidden) {
				const $column = $("<div class='importationHeaderItem badge'>" + column.DisplayName + "</div>");
				$column.attr("col_name", column.Name.toUpperCase());
				if (column.IsRequired) {
					$column.addClass("required");
				}
				$header.append($column);
			}
		});
		$("#out_form_blacklist_import_csv_header_columns").show();
	};

	form_outbound_engine_blacklist_imports.prototype.__mark_csv_columns = function(list, type)
	{
		const $header = $("#out_form_blacklist_import_allowed_csv_columns");
		$.each(list || [], function(_, col_name) {
			const $column = $header.find("[col_name='" + col_name.toUpperCase() + "']");
			if ($column.length > 0) {
				$column.removeClass("error").removeClass("success");
				if (type === "SUCCESS") {
					$column.addClass("success");
				}
				else {
					$column.addClass("error");
				}
			}
		});
	};

	form_outbound_engine_blacklist_imports.prototype.__load_allowed_csv_columns = function()
	{
		const self = this;
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.blackListImportGetAllowedHeaderInfo({},
			function(success, response) {
				if (success) {
					self.__make_allowed_columns(response);
					return;
				}
				self.show_status_message("<%=_T('Unable to retrieve information from CSV allowed columns ...')%>", "Error", 500);
			}
		);
	};

	form_outbound_engine_blacklist_imports.prototype.__make_csv_header_error = function(current_err)
	{
		const code_error = parseInt(current_err.code, 10);
		let msg;
		switch(code_error) {
			case -101:
				msg = "<%=_T('CSV File does not exist')%>";
				break;

			case -102:
				const missing_fields = current_err.missingRequiredFields || [];
				const total_missing_fields = missing_fields.length;
				msg = total_missing_fields === 1 ?
					"<%=_T('Missing {0} required field in CSV File')%>" :
					"<%=_T('Missing {0} required fields in CSV File')%>";
				msg = FORMAT(msg, total_missing_fields);
				this.__mark_csv_columns(missing_fields, "ERROR");
				break;

			case -105:
				msg = "<%=_T('Invalid CSV File')%>";
				break;

			default:
				msg = "<%=_T('Unable to Import.')%>";
		}
		return msg;
	};

	form_outbound_engine_blacklist_imports.prototype.__show_error = function(current_err)
	{
		this.hide_waiting();
		if (!current_err) {
			return;
		}
		let msg = this.__make_csv_header_error(current_err);
		const timemout_ms = current_err.timemout_ms || 5000;
		this.show_status_message(msg, "Error", timemout_ms);
	};
}());
