/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, moment */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	//---------------------------------------------------------------------
	// form contacts class
	function form_outbound_engine_contacts(params)
	{
		let success = form_outbound_engine_contacts.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.view_title = "<%=_T('View Contacts {0}')%>";
		this.tagline = "<%=_T('Tagline form Contact!!!')%>";

		this.html_file_name = "/inconcert/apps/outbound_engine/contacts/frm/render/";
		this.get_uri =        "/inconcert/apps/outbound_engine/contacts/get/";

		this.rows_per_page = 10;
		this.show_at_least = this.rows_per_page - 1;
		this.form_paginator_phones = null;
		this.form_paginator_batches = null;
		this.form_paginator_history = null;
		this.__new_grids();

		return true;
	}

	inConcert.Utils.inherit(form_outbound_engine_contacts, inConcert.conf_form);

	if (typeof inConcert.form_outbound_engine_contacts !== "function") {
		inConcert.form_outbound_engine_contacts = form_outbound_engine_contacts;
	}

	form_outbound_engine_contacts.prototype.initialize = function()
	{
		const $container = $("#out_form_contact_container");
		const $ctrls_for_vcc_teli6 = $container.find("[for-vcc='teli6']");
		const $ctrls_for_vcc_allegro = $container.find("[for-vcc='allegro']");

		if (inConcert.app.has_allegro_integration()) {
			$ctrls_for_vcc_teli6.hide();
			$ctrls_for_vcc_allegro.show();
		}
		else {
			$ctrls_for_vcc_allegro.hide();
			$ctrls_for_vcc_teli6.show();
		}
	};

	form_outbound_engine_contacts.prototype.set = function(success, contact)
	{
		if (!success) {
			return;
		}
		$("#out_form_contact_id").val(inConcert.app.get_id(contact.ID || contact.Id));
		$("#out_form_contact_name").val(contact.Name);
		$("#out_form_contact_category").val(contact.Category);
		$("#out_form_contact_campaign").val(contact.Campaign);
		$("#out_form_contact_account_officer").val(contact.AccountOfficer);
		$("#out_form_contact_external_id").val(contact.ExternalId);
		$("#out_form_contact_account_group").val(contact.AccountGroup);
		$("#out_form_contact_vip").prop("checked", contact.VIP);
		$("#out_form_contact_client").prop("checked", contact.IsClient);
		const utcDate = new Date(`${contact.LastUpdate} UTC`);
		$("#out_form_contact_last_update").val(moment(utcDate).format("YYYY-MM-DD HH:mm:ss"));
		$("#out_form_contact_last_result").val(contact.LastManagementResult);
		if (inConcert.app.has_allegro_integration()) {
			this.form_paginator_phones.initialize($.noop, contact.ContactImportPhones);
			this.form_paginator_batches.initialize($.noop, contact.Batches);
			this.form_paginator_history.initialize($.noop, contact.History);
		}
		else {
			let phone_addresses = [];
			(contact.Addresses || []).forEach(address => {
				if ((address.Type || "").toLowerCase() === "phone") {
					phone_addresses.push(address);
				}
			});
			this.form_paginator_phones.initialize($.noop, phone_addresses);
			const processInfo = contact.ProcessInfo;
			if (processInfo) {
				$("#out_form_contact_occ_process_id").val(inConcert.app.get_id(processInfo.id));
				$("#out_form_contact_occ_process_state").val(processInfo.displayState);
				$("#out_form_contact_occ_batch_id").val(inConcert.app.get_id(processInfo?.batch.configuration.name));
				$("#out_form_contact_occ_batch_state").val(processInfo?.batch.displayState);
			}
		}
	};

	form_outbound_engine_contacts.prototype.__new_grids = function()
	{
		let columns_type = inConcert.app.has_allegro_integration() ? "allegro" : "teli6";
		let columns = {
			phones: {
				allegro: [
					{id: "_number"      , name: "", type: "counter"   },
					{id: "Type"         , name: "<%=_T('Type')%>"     },
					{id: "Country"      , name: "<%=_T('Country')%>"  },
					{id: "Area"         , name: "<%=_T('Area')%>"     },
					{id: "Number"       , name: "<%=_T('Number')%>"   },
					{id: "Extension"    , name: "<%=_T('Extension')%>"},
					{id: "Status"       , name: "<%=_T('Status')%>"   }
				],
				teli6: [
					{id: "_number"      , name: "", type: "counter"   },
					{id: "DisplayKind"  , name: "<%=_T('Type')%>"     },
					{id: "Country"      , name: "<%=_T('Country')%>"  },
					{id: "Area"         , name: "<%=_T('Area')%>"     },
					{id: "Number"       , name: "<%=_T('Number')%>"   },
					{id: "DisplayStatus", name: "<%=_T('Status')%>"   }
				]
			}
		};
		this.form_paginator_phones = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_PHONES",
			owner: this,
			frm: this,
			row_id: "id",
			display_name: "Type",
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			ids: {
				grid: "out_form_contact_phones_list",
				paginator: "out_form_contact_phones_paginator",
				not_results_found: "out_form_contact_phones_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Phones found.')%>"
			},
			columns: columns.phones[columns_type]
		});
		if (!inConcert.app.has_allegro_integration()) {
			return;
		}
		this.form_paginator_batches = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_BATCHES",
			owner: this,
			frm: this,
			row_id: "id",
			display_name: "BatchId",
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			ids: {
				grid: "out_form_contact_batches_list",
				paginator: "out_form_contact_batches_paginator",
				not_results_found: "out_form_contact_batches_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Batches found.')%>"
			},
			columns: [
				{id: "_number"                    , name: "", type: "counter"                     },
				{id: "BatchId"                    , name: "<%=_T('Batch Name')%>"                 },
				{id: "TimeZonePreviousContactDate", name: "<%=_T('Previous Contact Date')%>",
					type: "date", date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
				},
				{id: "DisplayStatus"              , name: "<%=_T('Status')%>"                     },
				{id: "PreviousContactResult"      , name: "<%=_T('Previous Contact Result')%>"    },
				{id: "InProcess"                  , name: "<%=_T('In Process')%>", type: "boolean"},
				{id: "DisplayCause"               , name: "<%=_T('Cause')%>"                      },
				{id: "TimeZoneScheduledDate"      , name: "<%=_T('Scheduled Date')%>",
					type: "date", date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
				},
				{id: "DisplayNextDestination"     , name: "<%=_T('Next Destination')%>"           }
			]
		});
		this.form_paginator_history = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_HISTORY",
			owner: this,
			frm: this,
			row_id: "id",
			display_name: "Type",
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			ids: {
				grid: "out_form_contact_history_list",
				paginator: "out_form_contact_history_paginator",
				not_results_found: "out_form_contact_history_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No History found.')%>"
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"                     },
				{id: "OutboundProcessId" , name: "<%=_T('Process Name')%>"               },
				{id: "BatchId"           , name: "<%=_T('Batch Name')%>"                 },
				{id: "TimeZoneDate"      , name: "<%=_T('Date')%>",
					type: "date", date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
				},
				{id: "Campaign"          , name: "<%=_T('Campaign')%>"                   },
				{id: "DestinationAddress", name: "<%=_T('Destination')%>"                },
				{id: "DisplayResult"     , name: "<%=_T('Result')%>"                     },
				{id: "DisplayStatus"     , name: "<%=_T('Status')%>"                     },
				{id: "ImportationId"     , name: "<%=_T('Importation Name')%>"           },
				{id: "TimeZoneNextDate"  , name: "<%=_T('Next Date')%>",
					type: "date", date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
				},
				{id: "IsReschedule"      , name: "<%=_T('Reschedule')%>", type: "boolean"}
			]
		});
	};
}());
