/* jshint smarttabs:true, quotmark:false, esversion: 11 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 1800000; // 30 minutes

	function API (params)
	{
		if(typeof(params) !== "object") {
			throw("API.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("API.Constructor - base uri is required");
		}
		this.uri = params.uri + "/";
		this.cached_engine_types = null;
		this.cached_rules_responses_by_channel = null;
		this.cached_rules_destinations = null;
		this.cached_imports_fields_types = null;
		this.cached_imports_scheduled_formats = null;
		this.cached_imports_countries = null;
		this.cached_imports_destinations = null;
		this.cached_imports_mail_destinations = null;
		this.cached_imports_address_destinations = null;
		this.cached_imports_custom_columns = null;
		this.cached_imports_reserverd_words = null;
		this.cached_timezones = null;
		this.cached_countries = null;
		this.cached_areas = null;
		this.cached_exports_columns_types = null;
		this.cached_csv_allowed_header = null;
		// Este atributo contiene un GUID que se enviara en los diferentes request contra 5.5 (los que lo necesiten), para que caigan en la misma Lua VirtualMachine.
		this.m_current_contacts_request_id = null;
	}

	if (typeof inConcert.OutboundEngineAPI !== "function") {
		inConcert.OutboundEngineAPI = API;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	API.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Get: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Delete: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data || {}),
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) { callback(true, response); },
			error       : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Put = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Put: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Put: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Put: data variable must be object type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "PUT",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error  ); }
		});
		return request_handler;
	};

	API.prototype.getRequestTimeout = function() { return REQUEST_TIMEOUT_MS; };

	/*
	*/
	API.prototype.asyncCall = async function(method, ...args)
	{
		return inConcert.Utils.asyncCallObjectMethod(this, method, ...args);
	};

/********************************************************************************************************
	CONTACTS
********************************************************************************************************/

	API.prototype.contactsBatchFromProcess = function(process_id, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/contacts/batch/load/PROCESS_ID
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}contacts/batch/load/{1}", this.uri, process_id);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	EXPORTS
********************************************************************************************************/

	API.prototype.exportStart = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/exports/start/
		 */
		let uri = FORMAT("{0}exports/start/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.exportCancel = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/exports/cancel/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}exports/cancel/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.exportDownloadExportation = function(exportation, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/exports/download/exportation/?exportationId=ID1
		 */

		let uri = FORMAT("{0}exports/download/exportation/?exportationId={1}&fileName={2}",
			this.uri,
			encodeURIComponent(exportation.export_id),
			encodeURIComponent(exportation.file_name)
		);

		$.fileDownload(uri,
			{
				httpMethod : "GET",
				cookieName : "download_started" // cookie exist in repository!!
			})
			.done(function() {
				callback(true);
			})
			.fail(function(response) {
				callback(false, response);
			});
	};

/********************************************************************************************************
	IMPORT
********************************************************************************************************/

	API.prototype.hasSomeImportationRunning = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/has_some_importation_running/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/has_some_importation_running/", this.uri);

		let request_handler = this.__Get(uri, function(success, response) {
			if (success) {
				callback(true, response);
				return;
			}
			// error
			var err = response;
			callback(false, err);
		});
		return request_handler;
	};

	API.prototype.importUploadContactsFile = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/upload/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/upload/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.importStart = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/start/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/start/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.importCancel = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/cancel/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/cancel/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.importDepurate = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/depurate/
		 */

		let uri = FORMAT("{0}imports/depurate/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	// API.prototype.importBlacklistRelate = function(data, callback = $.noop)
	// {
	// 	/*
	// 	 * sample URI:
	// 	 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/blacklist/relate/
	// 	 */

	// 	let uri = FORMAT("{0}imports/blacklist/relate/", this.uri);
	// 	let request_handler = this.__Post(uri, callback, data);
	// 	return request_handler;
	// };

	API.prototype.importDownloadFailedContacts = function(importation, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/download/failed_contacts/?importid=ID1&update=1
		 */
		let uri = FORMAT("{0}imports/download/failed_contacts/?importid={1}&update={2}",
			this.uri,
			encodeURIComponent(importation.ImportId),
			encodeURIComponent(importation.ImportDetailId)
		);

		$.fileDownload(uri, {
				httpMethod : "GET",
				cookieName : "download_started" // cookie exist in repository!!
			})
			.done(function() {
				callback(true);
			})
			.fail(function(response) {
				callback(false, response);
			}
		);
	};

	API.prototype.importGetUploadedCSVHeader = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/get_uploaded_csv_header/file_id
		 */
		params = params || {};
		let uri = FORMAT("{0}imports/get_uploaded_csv_header/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	IMPORT FORMAT
********************************************************************************************************/
	API.prototype.importFormatsLoadAll = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/formats/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/formats/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.importFormatGet = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/formats/get/FORMAT_ID
		 */
		callback = callback || $.noop;
		params = params || {};

		let uri = FORMAT("{0}imports/formats/get/{1}",
			this.uri,
			encodeURIComponent(params.formatId || "")
		);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.importFormatSave = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/imports/formats/save/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}imports/formats/save/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	PROCESSES
********************************************************************************************************/

	API.prototype.processesList = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/processes/list/
		*/
		const uri = FORMAT("{0}processes/list/", this.uri);
		const request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.processSave = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/save/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/save/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processStart = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/start/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/start/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processStop = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/stop/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/stop/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processLoadAvailableSchedulers = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/schedulers/available/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/schedulers/available/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.processGetScheduler = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/schedulers/get/SCH_ID
		 */
		callback = callback || $.noop;
		params = params || {};

		let uri = FORMAT("{0}processes/schedulers/get/{1}",
			this.uri,
			encodeURIComponent(params.schedulerId || "")
		);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.processSaveScheduler = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/schedulers/save/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/schedulers/save/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processLoadAdministrationProcessess = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/administration/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/administration/load_all/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.processLoadBatchFilters = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/filters/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/batch/filters/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.processSaveFilter = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/filters/save/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}processes/batch/filters/save/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processDeleteFilter = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/filters/delete/FILTER_ID
		 */

		params = params || {};

		let uri = FORMAT("{0}processes/batch/filters/delete/{1}",
			this.uri,
			encodeURIComponent(params.filterId || "")
		);

		let request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	API.prototype.processCreateBatch = function(params, data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/create/QUERY_ID
		 */
		params = params || {};
		let query_id = inConcert.app.has_allegro_integration() ? params.queryId : "NONE";
		query_id = query_id || "";
		let uri = FORMAT("{0}processes/batch/contacts/create/{1}",
			this.uri,
			encodeURIComponent(query_id)
		);

		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	// SOLO VCCs NATIVOS
	API.prototype.processUpdateBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/update/
		 */
		let uri = FORMAT("{0}processes/batch/contacts/update/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	// SOLO VCCs con Allegro
	API.prototype.processUpdateBatchByQuery = function(params, data, callback =  $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/update/by_query/QUERY_ID/PROCESS_ID
		 */
		params = params || {};

		let uri = FORMAT("{0}processes/batch/contacts/update/by_query/{1}/{2}",
			this.uri,
			encodeURIComponent(params.queryId || ""),
			encodeURIComponent(params.processId || "")
		);

		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processUpdateBatchByAction = function(params, data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/update/by_action/BATCH_ID/PROCESS_ID
		 */
		params = params || {};

		let uri = FORMAT("{0}processes/batch/contacts/update/by_action/{1}/{2}",
			this.uri,
			encodeURIComponent(params.batchId || ""),
			encodeURIComponent(params.processId || "")
		);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processDeleteBatch = function(params, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/delete/PROCESS_ID/BATCH_ID
		 */

		callback = callback || $.noop;
		params = params || {};

		let uri = FORMAT("{0}processes/batch/contacts/delete/{1}/{2}",
			this.uri,
			encodeURIComponent(params.processId || ""),
			encodeURIComponent(params.batchId || "")
		);

		let request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	API.prototype.processPauseBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/pause/
		 */
		let uri = FORMAT("{0}processes/batch/pause/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processResumeBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/resume/
		 */
		let uri = FORMAT("{0}processes/batch/resume/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processCancelBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/cancel/
		 */
		let uri = FORMAT("{0}processes/batch/cancel/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processRemoveBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/remove/
		 */
		let uri = FORMAT("{0}processes/batch/remove/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processRecyleContactsBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/recycle/
		 */
		let uri = FORMAT("{0}processes/batch/contacts/recycle/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processeSetEngineTypeContactsBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/set_engine_type/
		 */
		let uri = FORMAT("{0}processes/batch/contacts/set_engine_type/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processChangePriorityContactsBatch = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/change_priority/
		 */
		let uri = FORMAT("{0}processes/batch/contacts/change_priority/", this.uri);
		data = this.__addRequestIdToPostData(data, this.m_current_contacts_request_id);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.processLoadBatchFields = function(process_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/fields/load/PROCESS_ID
		 */
		process_id = process_id || "";

		let uri = FORMAT("{0}processes/batch/fields/load/{1}", this.uri, process_id);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.__addRequestIdToPostData = function(data, request_id)
	{
		if (!data) {
			return;
		}
		if (!request_id) {
			return data;
		}
		data.__requestId = request_id;
		return data;
	};

	// SOLO VCCs con ALLEGRO
	API.prototype.processRunBatchOrContactsQuery = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch_or_contacts/query/run/100/batch_query o contact_query
		 */
		params = params || {};

		let uri = FORMAT("{0}processes/batch_or_contacts/query/run/{1}/{2}",
			this.uri,
			encodeURIComponent(params.rows_per_page || ""),
			encodeURIComponent(params.source || "")
		);

		// Este atributo contiene un GUID que se enviara en los diferentes request contra 5.5 (que lo necesiten), para que caigan en la misma Lua VirtualMachine.
		this.m_current_contacts_request_id = inConcert.Utils.generateGuid();
		params.query = this.__addRequestIdToPostData(params.query, this.m_current_contacts_request_id);

		let request_handler = this.__Post(uri, callback, params.query);
		return request_handler;
	};

	// SOLO VCCs con ALLEGRO
	API.prototype.processLoadContactsFromQuery = function(query_id, params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/load/fromquery/QUERY_ID/?posStart=1&count=1&requestid=GUID
		 */
		query_id = query_id || "";

		let uri = FORMAT("{0}processes/batch/contacts/load/fromquery/{1}/?start={2}&count={3}&requestid={4}",
			this.uri,
			query_id,
			params.start,
			params.count,
			this.m_current_contacts_request_id
		);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.processLoadBatchContactStatus = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/status/PROCESS_ID/BATCH_ID
		 */
		params = params || {};
		let uri = FORMAT("{0}processes/batch/contacts/status/{1}/{2}",
			this.uri,
			encodeURIComponent(params.processId || ""),
			encodeURIComponent(params.batchId || "")
		);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	// SOLO VCCs NATIVOS
	API.prototype.processRunBatchContactsQuery = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/query/execute/
		 */
		params = params || {};
		let uri = FORMAT("{0}processes/batch/contacts/query/execute/", this.uri);
		let request_handler = this.__Post(uri, callback, params.query);
		return request_handler;
	};

	// SOLO VCCs NATIVOS
	API.prototype.processRunBatchContactsSummary = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/processes/batch/contacts/summary/execute/
		 */
		params = params || {};
		let uri = FORMAT("{0}processes/batch/contacts/summary/execute/", this.uri);
		let request_handler = this.__Post(uri, callback, params.query);
		return request_handler;
	};

/********************************************************************************************************
	SYSTEM
********************************************************************************************************/

	API.prototype.systemEngineServicesLoadAll = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/engine/services/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}system/engine/services/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemEngineTypesLoadAll = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/engine/types/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_engine_types) {
			callback(true, this.cached_engine_types);
			return;
		}

		let uri = FORMAT("{0}system/engine/types/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_engine_types = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemRulesLoadResponsesByChannel = function(callback = $.noop, channel = "CALL")
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/rules/responsesby/channel/CALL
		 */
		const self = this;
		if (this.cached_rules_responses_by_channel) {
			if (this.cached_rules_responses_by_channel[channel]) {
				callback(true, this.cached_rules_responses_by_channel[channel]);
				return;
			}
		}
		let uri = FORMAT("{0}system/rules/responsesby/channel/{1}", this.uri, channel);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_rules_responses_by_channel = self.cached_rules_responses_by_channel || {};
				self.cached_rules_responses_by_channel[channel] = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemRulesLoadDestinations = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/rules/destinations/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_rules_destinations) {
			callback(true, this.cached_rules_destinations);
			return;
		}

		let uri = FORMAT("{0}system/rules/destinations/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_rules_destinations = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemRulesLoadChannels = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/rules/channels/load/
		 */
		const self = this;

		if (this.cached_rules_channels) {
			callback(true, this.cached_rules_channels);
			return;
		}

		let uri = FORMAT("{0}system/rules/channels/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_rules_channels = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemRulesSystemActions = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/rules/system_actions/load/
		 */
		const self = this;

		if (this.cached_rules_system_actions) {
			callback(true, this.cached_rules_system_actions);
			return;
		}

		let uri = FORMAT("{0}system/rules/system_actions/load/", this.uri);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_rules_system_actions = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadCampaignMembers = function(campaign_id, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/campaign/members/load/CAMPAIGN_ID
		 */
		callback = callback || $.noop;
		campaign_id = campaign_id || "";

		let uri = FORMAT("{0}system/campaign/members/load/{1}", this.uri, campaign_id);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadCampaignsAccountsByChannels = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/campaigns/accounts/by_channels/load/
		*/
		let uri = FORMAT("{0}system/campaigns/accounts/by_channels/load/", this.uri);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsFieldsTypes = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/fieldstypes/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_fields_types) {
			callback(true, this.cached_imports_fields_types);
			return;
		}

		let uri = FORMAT("{0}system/imports/fieldstypes/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_fields_types = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsScheduledFormat = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/scheduleformat/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_scheduled_formats) {
			callback(true, this.cached_imports_scheduled_formats);
			return;
		}

		let uri = FORMAT("{0}system/imports/scheduleformat/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_scheduled_formats = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsCountries = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/countries/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_countries) {
			callback(true, this.cached_imports_countries);
			return;
		}

		let uri = FORMAT("{0}system/imports/countries/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_countries = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsDestinations = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/destinations/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_destinations) {
			callback(true, this.cached_imports_destinations);
			return;
		}

		let uri = FORMAT("{0}system/imports/destinations/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_destinations = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsMailDestinations = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/maildestinations/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_mail_destinations) {
			callback(true, this.cached_imports_mail_destinations);
			return;
		}

		let uri = FORMAT("{0}system/imports/maildestinations/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_mail_destinations = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsAddressDestinations = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/addressdestinations/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_address_destinations) {
			callback(true, this.cached_imports_address_destinations);
			return;
		}

		let uri = FORMAT("{0}system/imports/addressdestinations/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_address_destinations = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsCustomColumns = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/customcolumns/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_custom_columns) {
			callback(true, this.cached_imports_custom_columns);
			return;
		}

		let uri = FORMAT("{0}system/imports/customcolumns/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_custom_columns = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportsReservedWords = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/imports/reservedwords/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_imports_reserverd_words) {
			callback(true, this.cached_imports_reserverd_words);
			return;
		}

		let uri = FORMAT("{0}system/imports/reservedwords/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_imports_reserverd_words = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadTimeZones = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/timezones/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_timezones) {
			callback(true, this.cached_timezones);
			return;
		}

		let uri = FORMAT("{0}system/timezones/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_timezones = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadCountries = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/countries/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_countries) {
			callback(true, this.cached_countries);
			return;
		}

		let uri = FORMAT("{0}system/countries/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_countries = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadAreas = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/areas/load/
		 */
		callback = callback || $.noop;
		const self = this;

		if (this.cached_areas) {
			callback(true, this.cached_areas);
			return;
		}

		let uri = FORMAT("{0}system/areas/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_areas = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadSkills = function(callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/skills/load/
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}system/skills/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadBatchesByCampaign = function(campaign_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/batches_by_campaign/load/
		 */
		let uri = FORMAT("{0}system/batches_by_campaign/load/{1}", this.uri, campaign_id);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadImportations = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/importations/load/
		 */
		let uri = FORMAT("{0}system/importations/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	EXPORT LAYOUT
********************************************************************************************************/

	API.prototype.exportLayoutsLoadAll = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/exports/layout/load_all/
		 */
		let uri = FORMAT("{0}exports/layout/load_all/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.systemLoadExportColumnsTypes = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/system/exports/columnstypes/load/
		 */
		const self = this;

		if (this.cached_exports_columns_types) {
			callback(true, this.cached_exports_columns_types);
			return;
		}

		let uri = FORMAT("{0}system/exports/columnstypes/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_exports_columns_types = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.exportLayoutSave = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/exports/layout//save/
		 */
		let uri = FORMAT("{0}exports/layout/save/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/********************************************************************************************************
		BLACKLILST
	********************************************************************************************************/

	API.prototype.blackListImportStart = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/blacklist/start/
		 */
		let uri = FORMAT("{0}blacklist/start/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.blackListImportDownloadFailedAddresses = function(importation, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/blacklist/download/failed_addresses/?importid=ID1&update=1
		 */
		let uri = FORMAT("{0}blacklist/download/failed_addresses/?importid={1}&update={2}",
			this.uri,
			encodeURIComponent(importation.ImportId),
			encodeURIComponent(importation.ImportDetailId)
		);
		$.fileDownload(uri, {
				httpMethod : "GET",
				cookieName : "download_started" // cookie exist in repository!!
			})
			.done(function() {
				callback(true);
			})
			.fail(function(response) {
				callback(false, response);
			}
		);
	};

	API.prototype.blackListImportGetAllowedHeaderInfo = function(params, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/outbound_engine/blacklist/get_allowed_header_info/
		*/
		const self = this;
		params = params || {};
		if (this.cached_csv_allowed_header) {
			callback(true, this.cached_csv_allowed_header);
			return;
		}
		const uri = FORMAT("{0}blacklist/csv/get_allowed_header_info/", this.uri);
		const request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				self.cached_csv_allowed_header = result;
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	API.prototype.blackListImportGetUploadedCSVHeader = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/outbound_engine/blacklist/get_uploaded_csv_header/file_id
		 */
		params = params || {};
		let uri = FORMAT("{0}blacklist/get_uploaded_csv_header/{1}",
			this.uri,
			encodeURIComponent(params.fileId || "")
		);
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.blackListGetAll = function(params, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/outbound_engine/blacklist/list_all/address_type
		*/
		params = params || {};
		const uri = FORMAT("{0}blacklist/list_all/{1}", this.uri, params.AddressType);
		const request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};

	/********************************************************************************************************
		AGENT OE
	********************************************************************************************************/

	API.prototype.agentOeUrl = function(path)
	{
		return `${inConcert.app.getBaseUrl()}/outbound_engine/${path}`;
	};

	API.prototype.GetCampaigns = function(callback = $.noop)
	{
		return this.__Get(this.agentOeUrl('get_campaigns/'), callback);
	};

	API.prototype.GetCampaignBatches = function({ campaignId }, callback = $.noop)
	{
		return this.__Get(this.agentOeUrl(`get_batches/${encodeURIComponent(campaignId)}`), callback);
	};

	API.prototype.GetCampaignBatch = function({ campaignId, batchId }, callback = $.noop)
	{
		return this.__Get(this.agentOeUrl(
			`get_batch/${encodeURIComponent(campaignId)}/${encodeURIComponent(batchId)}`
			), callback);
	};

	API.prototype.GetOutboundContact = function({ contactId }, callback = $.noop)
	{
		return this.__Get(this.agentOeUrl(`get_contact/${encodeURIComponent(contactId)}`), callback);
	};

	API.prototype.GetOutboundContactByName = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('get_contact_by_name/'), callback, data);
	};

	API.prototype.GetOutboundContactByAddress = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('get_contact_by_address/'), callback, data);
	};

	API.prototype.SearchOutboundContacts = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('search_contacts/'), callback, data);
	};

	API.prototype.GetOutboundContactResults = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('get_contact_results/'), callback, data);
	};

	API.prototype.StoreOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('store_contact/'), callback, data);
	};

	API.prototype.SetOutboundContactAddresses = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('set_contact_addresses/'), callback, data);
	};

	API.prototype.SetOutboundContactValues = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('set_contact_values/'), callback, data);
	};

	API.prototype.SetOutboundContactSkills = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('set_contact_skills/'), callback, data);
	};

	API.prototype.RemoveOutboundContactAddress = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('remove_contact_address/'), callback, data);
	};

	API.prototype.EnableOutboundContactAddress = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('enable_contact_address/'), callback, data);
	};

	API.prototype.DisableOutboundContactAddress = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('disable_contact_address/'), callback, data);
	};

	API.prototype.RescheduleOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('reschedule_contact/'), callback, data);
	};

	API.prototype.AddContactToOutboundBatch = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('add_contact/'), callback, data);
	};

	API.prototype.PauseOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('pause_contact/'), callback, data);
	};

	API.prototype.ResumeOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('resume_contact/'), callback, data);
	};

	API.prototype.CancelOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('cancel_contact/'), callback, data);
	};

	API.prototype.FinishOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('finish_contact/'), callback, data);
	};

	API.prototype.RemoveOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('remove_contact/'), callback, data);
	};

	API.prototype.RecycleOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('recycle_contact/'), callback, data);
	};

	API.prototype.ChangePriorityOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('change_contact_priority/'), callback, data);
	};

	API.prototype.SetAgentOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('set_agent_contact/'), callback, data);
	};

	API.prototype.SetEngineTypeOutboundContact = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('set_contact_engine_type/'), callback, data);
	};

	API.prototype.GetActionStatus = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('action_status/'), callback, data);
	};

	API.prototype.ValidateAddress = function(data, callback = $.noop)
	{
		return this.__Post(this.agentOeUrl('validate_address/'), callback, data);
	};
}());
