module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local apiLuis = require("shared.cognitive_services.luis.luis").GetAPI() -- rocka

m_htmlRender = nil
m_appLocation = nil

-------------------------------------------------------
-- SUBSCRIPTIONS
-------------------------------------------------------

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/provisioning/cognitive_services/subscriptions/assign_to_vcc/

	Metodo: POST

		DATA:
			{"subscriptionKey": "f5c41ac5c06b46d6a697c076f4f3923c", "vcc": "allegro4"}

	Respuesta:
		Status:200 OK
		{"result":"OK"}
]]--
local function AssignSubscriptionKeyToVCC (web)
	local currentVcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, currentVcc)

	local ok, info = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[COGNITIVE_SERVICES:AssignSubscriptionKeyToVCC] Error Assign Subscription key To VCC. Invalid Request Information '%s'", web.input["post_data"])
		web.status = 400
		return {result = "INVALID_REQUEST", code = 400}
	end

	if not info.vcc or not info.subscriptionKey then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AssignSubscriptionKeyToVCC] Error Assign Subscription key To VCC. Invalid Request Information '%s'", web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:AssignSubscriptionKeyToVCC] User '%s' Assign Subscription key: '%s' To VCC : '%s'.", userId, info.subscriptionKey, info.vcc)

	local success, response = apiLuis.AssignSubscriptionKeyToVCC(info)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:AssignSubscriptionKeyToVCC] Error Assign Subscription key: '%s' To VCC : '%s'. Error: '%s'", info.subscriptionKey, info.vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
	Ejemplo URL:
		http://i6.inconcert.cc/inconcert/apps/provisioning/cognitive_services/subscriptions/unassign_of_vcc/allegro4/f5c41ac5c06b46d6a697c076f4f3923c

	Metodo: DELETE

		Parametros:
			1) VCC : allegro4
			2) Subscription Key: f5c41ac5c06b46d6a697c076f4f3923c

	Respuesta:
		Status:200 OK
		{"result":"OK"}
]]--
local function UnAssignSubscriptionKeyOfVCC (web, vcc, subscriptionKey)
	local currentVcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, currentVcc)

	if not vcc or not subscriptionKey then
		web.status = 400
		LogError("[COGNITIVE_SERVICES:UnAssignSubscriptionKeyOfVCC] Error Unassign Subscription key of VCC. Invalid Request Information '%s'", web.input["post_data"])
		return {result = "INVALID_REQUEST", code = 400}
	end

	LogInfo("[COGNITIVE_SERVICES:UnAssignSubscriptionKeyOfVCC] User '%s' Unassign Subscription key: '%s' of VCC : '%s'.", userId, subscriptionKey, vcc)

	local success, response = apiLuis:UnAssignSubscriptionKeyOfVCC(vcc, subscriptionKey)
	if success then
		return response
	else
		web.status = 400
		LogError("[COGNITIVE_SERVICES:UnAssignSubscriptionKeyOfVCC] Error Unassign Subscription key: '%s' of VCC : '%s'. Error: '%s'", subscriptionKey, vcc, json.encode(response))
		return {result = "NOT_FOUND", cognitiveServiceError = response}
	end
end

--[[
****************************************************************
	MAPEO DE URLS
****************************************************************
]]--

function MapUris (module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local function HasPermission (fn)
		-- return utils.JsonReply .. LogRequest ..
		--	permissions:CheckPermission("provisioning", "save_ms_cognitive_services", fn)

		-- ESTA VALIDACION FALSA DE PERMISOS, SE DEBE ELIMINAR, CUANDO SE AGREGUEN LOS PERMISOS CORRESPONDIENTES AL INITIAL DATA + MIGRATION.
		-- DESPUES DEBEN DE QUEDAR LAS LINEAS QUE HAN QUEDADO COMENTADAS ARRIBA.
		function FalseCheckPermission (app_id, action, functionToCall)
			return function(web, ...)
				return functionToCall(web, ...)
			end
		end
		return utils.JsonReply .. LogRequest .. FalseCheckPermission("provisioning", "save_ms_cognitive_services", fn)
	end

	-- SUBSCRIPTIONS provisioning
	module:dispatch_post  (HasPermission(AssignSubscriptionKeyToVCC)  , prefix .. "/subscriptions/assign_to_vcc/"              )
	module:dispatch_delete(HasPermission(UnAssignSubscriptionKeyOfVCC), prefix .. "/subscriptions/unassign_of_vcc/([^/]+)/(.*)")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
