local utils = require "shared.apps.utils"
local m_provisioningModel = require "apps.provisioning.model"
local m_vccsModel = require "apps.provisioning.models.vccs"
local Fullcontact = require "fullcontact"
local m_helpers = require "helpers"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local LOG_PREFIX = "M I G R A T I O N ==> "

----------------------
local m_migration    = {}
local m_allegro      = {}
local m_fullcontact  = {}
local m_processFlows = {}
local m_running = {
	EXPIRE_SEC = 300 -- La key del current process de migracion expira a los 5 minutos
}

--[[---------------------------------------
	DEFINITIONS - MODULES TO MIGRATE

	En el orden en que se encuentre en el array es el orden en que se ejecutara.
-----------------------------------------]]--
local m_migrationsConfig = {
	-- VCC & Campaigns ---------------------------------------------------------------
	{
		id                  = "vccs", -- es el nombre del archivo al cual se le realizara require "/migrations/vccs.lua"
		description         = "VCC Module Migration",
		instance            = nil,
		steps               = {},
		enabled             = true, -- puedo encender o apagar la migracion de este modulo
		order               = 1,
		stepGroupId         = "STEP 1"
	},
	--[[
		peers -> DE MOMENTO INHABILITADA YA QUE ACCEDE A ALLEGRO
	]]--
			-- {
			-- 	id                  = "peers",
			-- 	description         = "Peers Module Migration",
			-- 	instance            = nil,
			-- 	steps               = {},
			-- 	enabled             = false, -- TODO: VER SI HAY QUE IMPLEMENTAR (por el momento esta desactivado) !!!!!!!!!!!!!
			-- 	order               = 222
			-- },
	{
		id                  = "campaigns", -- es el nombre del archivo al cual se le realizara require en el loop "/migrations/campaigns.lua"
		description         = "Campaigns Module Migration",
		instance            = nil,
		steps               = {},
		enabled             = true, -- puedo encender o apagar la migracion de este modulo
		order               = 2,
		stepGroupId         = "STEP 1"
	},
	-- OE Import Formats ---------------------------------------------------------------
	{
		id                  = "oeImportFormats", -- es el nombre del archivo al cual se le realizara require en el loop "/migrations/oeImportFormats.lua"
		description         = "OE Import Formats Module Migration",
		instance            = nil,
		steps               = {},
		enabled             = true, -- puedo encender o apagar la migracion de este modulo
		order               = 3,
		stepGroupId         = "STEP 1"
	},
	-- OE Processes ---------------------------------------------------------------
	{
		id                  = "oeProcesses", -- es el nombre del archivo al cual se le realizara require en el loop "/migrations/oeProcesses.lua"
		description         = "OE Processes Module Migration",
		instance            = nil,
		steps               = {},
		enabled             = true, -- puedo encender o apagar la migracion de este modulo
		order               = 4,
		stepGroupId         = "STEP 1"
	},
	-- OE Importations & Contactos ---------------------------------------------------------------
	{
		id                  = "oeImportContacts", -- es el nombre del archivo al cual se le realizara require en el loop"/migrations/oeImportContacts.lua"
		description         = "OE Importations & Contacts Module Migration",
		instance            = nil,
		steps               = {},
		enabled             = true, -- puedo encender o apagar la migracion de este modulo
		order               = 5,
		stepGroupId         = "STEP 2"
	}
}
for _, config in ipairs(m_migrationsConfig) do
	--if config.enabled then
		-- 1 - realizo require al modulo de la migracion
		local modulePath = "apps.provisioning.controllers.migrations." .. config.id
		config.instance = require(modulePath)
		config.instance.Init({
			id = config.id
		})
		-- 2 - obtengo los steps del modulo de la migracion
		for _, step in ipairs(config.instance.GetSteps()) do
			table.insert(config.steps, step)
		end
	--end
end

--[[

]]--
local function GetStepByModule (moduleId, stepId)
	assert(moduleId)
	assert(stepId)
	local currentModule
	for i, config in ipairs(m_migrationsConfig) do
		if config.id:upper() == moduleId:upper() then
			currentModule = config
			break
		end
	end
	assert(currentModule)
	local stepInstance
	for _, step in ipairs(currentModule.instance.GetSteps()) do
		if step.id:upper() == stepId:upper() then
			stepInstance = step
			break 
		end
	end
	assert(stepInstance)
	return stepInstance
end

--[[
	filtra y ordena los modulos
]]--
local function LoadSortedMigrationsModules (asc, filterByStepGroupId)
	assert(type(asc) == "boolean")
	filterByStepGroupId = (filterByStepGroupId or ""):upper()
	local sortedMigrationsConfig = {}
	-- 1- filtra segun el step group id
	for _, config in ipairs(m_migrationsConfig) do
		if filterByStepGroupId == "" or filterByStepGroupId == config.stepGroupId:upper() then
			table.insert(sortedMigrationsConfig, config)
		end
	end
	-- 2- ordena el array de modulos asc/desc
	table.sort(sortedMigrationsConfig, function(a, b)
		assert(type(a.order) == "number")
		assert(type(b.order) == "number")
		if asc then
			return a.order < b.order
		end
		return a.order > b.order
	end)
	-- ok
	return sortedMigrationsConfig
end

-----------------------------------------
-- MIGRATION
-----------------------------------------
--[[

]]--
m_migration.Start = function(self, info)
	local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start] + START Migration Allegro VCC to Native VCC. info '%s', startTime: '%s'.", utils.JSON_Encode(info), mainProcessElapsedTime.startTime)

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local ksProv  = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, result, allegroVccInfo = m_migration:CanStartMigration({
		vccId             = info.vcc,
		ksAgent           = ksAgent,
		ksProv            = ksProv,
		step              = info.step,
		contactsMigration = info.contactsMigration
	})
	if not success then
		ksAgent:Close()
		ksProv:Close()
		local err = result
		-- No puedo comenzar la migracion, dejo en HDD el registro del mismo
		m_running.Start({vcc = info.vcc, err = err, errType = "UNABLE TO START MIGRATION", migrationStep = info.step}, "MIGRATION") -- creo en redis el start del proceso de migracion
		m_running.End()
		return false, err
	end
	-- comienzo el proceso de migracion --------------------
	utils.CollectAllMemory()

	local occVccInfo                 = result
	info.occVccInfo                  = occVccInfo
	info.allegroVccInfo              = allegroVccInfo
	info.vccId                       = info.vcc
	info.allegroInfo                 = nil
	info.ksAgent                     = ksAgent
	info.ksProv                      = ksProv
	info.ksOE                        = utils.GetDatasource():CassandraConnection("OutboundEngine")
	info.allegroMigrationModule      = m_allegro
	info.runningModule               = m_running
	info.fullcontactMigrationModule  = m_fullcontact
	info.processFlowsMigrationModule = m_processFlows
	info.GetStepByModule             = GetStepByModule

	-- NORMAL o "CONTINUE AND REPAIR FAILED MIGRATION"
	local migrationType = info.contactsMigration and  info.contactsMigration.migrationType:upper() --para el step2
	local executionType = info.contactsMigration and info.contactsMigration.executionType:upper() or "NORMAL" --para el step2

	-- creo el proceso en el running
	local migrationId, step2UseSameDirectory, step2UseSameStoredKey
	if info.step == "STEP 1" then
		GetStepByModule("oeImportContacts", "importations").CleanUpStore(info, true) -- debo eliminar los datos de temporales de cassandra del step 2 del mismo VCC. Esto en el caso en que se haga rollback, etc.
	elseif info.step == "STEP 2" then
		local lastMigration = occVccInfo.AllegroIntegration.LastMigration
		migrationId = lastMigration.migrationId
		--[[
			La asignacion de los procesos y batches en memoria, que usaran al asignar contactos a los batches se realiza unicamente para:
				cuando se esta CONTINUANDO una migracion del step2 porque el mismo habia tenido una caida/stop del webhandler.
				de los 3 tipos de ejecuciones de CONTINUE por caida, solo se asigna este valor cuando:
					 se estaban migrando todos los contactos "ALL CONTACTS OF VCC"
					 se estaban migrando todos los contactos que SI pertenecen a los lotes de los procesos  "CONTACTS OF SOME PROCESS BATCHES"

			Para la opcion "CONTACTS NOT IN ANY BATCH PROCESSES" (cuando se migran todos los contactos que NO pertenezcan a ningun lote)
				NO es necesaria esta asignacion, ya que estos contactos NO se asiganaran a ningun lote.
		]]--
		if executionType == "CONTINUE AND REPAIR FAILED MIGRATION"
			and (migrationType == "ALL CONTACTS OF VCC" or migrationType == "CONTACTS OF SOME PROCESS BATCHES")
		then
			step2UseSameDirectory = lastMigration.currentMigrationDir
			step2UseSameStoredKey = lastMigration.tempStoredKey
			if lastMigration.contactsMigration then
				-- si es un step2 con accion de continuar por caida/stop del webhandler, debo de dejar la lista de procesos y batches se han obtenido de los contactos que ya se han migrado.
				-- esto es asi para luego cuando llegue a la migracion de batches (asignacion de contactos a los lotes), obtenga los contactos que ya fueron migrados.
				local importModule = GetStepByModule("oeImportContacts", "importations")
				importModule.SetMigratedContactsInfo(lastMigration.contactsMigration.processAndBatches)
			end
		end
	else
		error("Unrecognized migration step: " .. tostring(info.step))
	end
	-- creo en redis el start del proceso de migracion
	local _, runningInfo = m_running.Start({
			vcc                 = info.vcc,
			migrationStep       = info.step,
			migrationId         = migrationId,           -- opcional (solo para el step 2)
			currentMigrationDir = step2UseSameDirectory, -- opcional (solo para el step 2)
			tempStoredKey       = step2UseSameStoredKey  -- opcional (solo para el step 2)
		},
		"MIGRATION"
	)

	local mustUpdateLastMigration = true
	if info.step == "STEP 2" then
		-- para comenzar la migracion del STEP 2, tengo que dejar el VCC con Integracion con Allegro, ya que sino en el AllegroProxy fallan los request contra Allegro.
		local success = m_migration:ChangeVccToAllegroWithRetries(info, occVccInfo)
		if not success then
			ksAgent:Close()
			ksProv:Close()
			info.ksOE:Close()
			-- No puedo comenzar la migracion, dejo en HDD el registro del mismo
			m_running.Start({vcc = info.vcc, err = "UNABLE TO CONNECT WIHT ALLEGRO", errType = "UNABLE TO START MIGRATION", }, "MIGRATION") -- creo en redis el start del proceso de migracion
			m_running.End()
			return false, err
		end
		-- si va a continuar una migracion anterior por falla, no debo iniciar el LastMigration como nuevo
		if executionType == "CONTINUE AND REPAIR FAILED MIGRATION" then
			mustUpdateLastMigration = false
		end
	end

	-- en cualquier ejecucion NORMAL el LastMigration step se debe iniciar en cero en cassandra.
	--   en caso de CONTINUE por caida del webhandler se debe de utilizar el actual que existe en cassandra; es decir tiene que agarrar el de la caida.
	if mustUpdateLastMigration then
		-- marco en cassandra el comienzo de la migracion.
		runningInfo = runningInfo or {}
		occVccInfo.AllegroIntegration.LastMigration = {
			type                = "MIGRATION",
			status              = "RUNNING",
			migrationStep       = info.step,
			startTime           = os.time(),
			migrationId         = runningInfo.migrationId,
			currentMigrationDir = m_running.currentMigrationDir,
			tempStoredKey       = m_running.tempStoredKey
		}
		m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
	end

	-- filtra y ordena los modulos de migracion de menor a mayor segun el atributo "order"
	local asc = true
	local sortedMigrationsConfig = LoadSortedMigrationsModules(asc, info.step)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start]  Total Migration MODULES '%s' in STEP %s.", #sortedMigrationsConfig, info.step)

	for i, config in ipairs(sortedMigrationsConfig) do
		if config.enabled then
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start]  %s -  Init Module: '%s'.", i, config.description)
			local trace = {
				id = config.id,
				description = config.description,
				steps = {}
			}
			for j, step in ipairs(config.steps) do
				LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start]      %s.%s -  Init Step: '%s'.", i, j, step.description)
				local stepTrace = {
					id          = step.id,
					description = step.description,
					steps       = {}, -- estos steps se cargan dentro del modulo
					success     = nil,
					result      = nil
				}
				table.insert(trace.steps, stepTrace)
				m_running.Update(trace) -- inserto en redis el step del modulo de migracion
				----
				-- ACA comienza la ejecucion de la migracion del STEP
				step.Start(info)
			end
		end
	end
	-- TODO: realizar la actualizacion si esta todo ok + rollback en caso de error. ?
	if true then
	--if false then
		local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.End(mainProcessElapsedTime.processId)
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start] - END Migration Allegro VCC to Native VCC. endTime: %s, elapsedTime: %s.", mainProcessElapsedTime.endTime, mainProcessElapsedTime.humanElapsedTime.formattedTime)
		-- marco en cassandra el fin con exito de la migracion
		occVccInfo.AllegroIntegration.LastMigration.status = "ENDED"
		occVccInfo.AllegroIntegration.LastMigration.endTime = os.time()

		occVccInfo.AllegroIntegration.enabled = false
		occVccInfo.AllegroIntegration.Migrated = occVccInfo.AllegroIntegration.Migrated or {}

		mainProcessElapsedTime.elapsedTime = mainProcessElapsedTime.humanElapsedTime.formattedTime
		mainProcessElapsedTime.humanElapsedTime = nil
		local migrationInfo = {
			timeInfo = mainProcessElapsedTime
		}
		table.insert(occVccInfo.AllegroIntegration.Migrated, migrationInfo)
		occVccInfo.AllegroIntegration = utils.JSON_Encode(occVccInfo.AllegroIntegration)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start] -------> occVccInfo.AllegroIntegration: %s.", occVccInfo.AllegroIntegration)
		m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
	else
		-- TODO: rollback ?
	end
	-- dd
	GetStepByModule("oeImportContacts", "importations").CleanUpStore(info, false)
	info.ksAgent:Close()
	info.ksProv:Close()
	info.ksOE:Close()
	m_running.End() -- elimino de redis el proceso de migracion
	utils.CollectAllMemory()
	return true, {}
end

--[[

]]--
m_migration.Rollback = function(self, info)
	local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Rollback] Rollback Migrated VCC. Native to Allegro. info '%s', startTime: '%s'.", utils.JSON_Encode(info), mainProcessElapsedTime.startTime)

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local ksProv  = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, result = m_migration:CanStartRollback(info.vcc, ksAgent, ksProv)
	if not success then
		local err = result
		ksAgent:Close()
		ksProv:Close()
		-- No puedo comenzar el rollback, dejo en HDD el registro del mismo
		m_running.Start({vcc = info.vcc, err = err, errType = "UNABLE TO START ROLLBACK"}, "ROLLBACK") -- creo en redis el start del proceso de rollback
		m_running.End()
		return false, err
	end
	-- comienzo el proceso de rollback --------------------
	utils.CollectAllMemory()

	-- creo el proceso en el running
	m_running.Start({vcc = info.vcc}, "ROLLBACK") -- creo en redis el start del proceso de rollback

	local occVccInfo                 = result
	info.occVccInfo                  = occVccInfo
	info.vccId                       = info.vcc
	info.allegroInfo                 = nil
	info.ksAgent                     = ksAgent
	info.ksProv                      = ksProv
	info.ksOE                        = utils.GetDatasource():CassandraConnection("OutboundEngine")
	info.allegroMigrationModule      = m_allegro
	info.runningModule               = m_running
	info.fullcontactMigrationModule  = m_fullcontact
	info.processFlowsMigrationModule = m_processFlows

	GetStepByModule("oeImportContacts", "importations").CleanUpStore(info, true) -- debo eliminar los datos de temporales de cassandra del step 2 del mismo VCC. Esto en el caso en que se haga rollback, etc.

	-- ordena los modulos de migracion de mayor a menor segun el atributo "order"
	local asc = false
	local sortedMigrationsConfig = LoadSortedMigrationsModules(asc)

	-- El orden a recorrer las actualizaciones debe ser en orden inverso a la migracion (de mayor a menor)
	for i, config in ipairs(sortedMigrationsConfig) do
		if config.enabled then
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Rollback]  %s - Init Module: '%s'.", i, config.description)
			local trace = {
				id = config.id,
				description = config.description,
				steps = {}
			}
			-- El orden a recorrer los steps debe ser en orden inverso a la migracion (de mayor a menor)
			local j = #config.steps
			while j > 0 do
				local step = config.steps[j]
				LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Start]      %s.%s -  Init Step: '%s'.", i, j, step.description)
				local stepTrace = {
					id          = step.id,
					description = step.description,
					steps       = {}, -- estos steps se cargan dentro del modulo
					success     = nil,
					result      = nil
				}
				m_running.Update(trace) -- inserto en redis el step del modulo de rollback
				stepTrace.success, stepTrace.result = step.Rollback(info)
				j = j - 1
			end
		end
	end
	-- TODO: realizar la actualizacion si esta todo ok + rollback en caso de error.
	if true then
	--if false then
		local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.End(mainProcessElapsedTime.processId)
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.Rollback] - END Rollback Migrated VCC. Native to Allegro. endTime: %s, elapsedTime: %s.", mainProcessElapsedTime.endTime, mainProcessElapsedTime.humanElapsedTime.formattedTime)

		-- marco en cassandra el fin del rollback.
		occVccInfo.AllegroIntegration.LastMigration = occVccInfo.AllegroIntegration.LastMigration or {}
		occVccInfo.AllegroIntegration.LastMigration.status = "ROLLBACKED"
		occVccInfo.AllegroIntegration.LastMigration.endTime = os.time()

		occVccInfo.AllegroIntegration.enabled = true
		occVccInfo.AllegroIntegration.Rollback = occVccInfo.AllegroIntegration.Rollback or {}
		mainProcessElapsedTime.elapsedTime = mainProcessElapsedTime.humanElapsedTime.formattedTime
		mainProcessElapsedTime.humanElapsedTime = nil
		local rollbackInfo = {
			timeInfo = mainProcessElapsedTime
		}
		table.insert(occVccInfo.AllegroIntegration.Rollback, rollbackInfo)
		occVccInfo.AllegroIntegration = utils.JSON_Encode(occVccInfo.AllegroIntegration)
		m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
	else
		-- TODO: rollback
	end
	-- ok
	info.ksAgent:Close()
	info.ksProv:Close()
	info.ksOE:Close()
	m_running.End() -- elimino de redis el proceso de rollback
	utils.CollectAllMemory()
	return true, {}
end

--[[

]]--
m_migration.GetOccVCC = function(self, ksAgent, vcc)
	local success, occVccInfo = m_vccsModel.GetVCC(ksAgent, vcc)
	if not success or utils.TableIsEmpty(occVccInfo) then
		local err = {errType = "OCC VCC DOES NOT EXIST", vccId = vcc, info = occVccInfo}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.GetOccVCC] ---> Get OCC Vcc: '%s' success: false, err: %s.", vcc, json.encode(err))
		return false, err
	end
	return true, occVccInfo
end

--[[

]]--
m_migration.GetAllegroVCC = function(self, vcc)
	local allegroVccModule = utils.GetControllerOthersApps("allegro", {controllerName = "provisioning_vccs", modelName = nil})
	local success, result = allegroVccModule.GetVCC(vcc)
	if not success then
		local err = result
		return false, err
	end
	return true, result
end

--[[
	Para una migracion STEP 2) SOLO PARA ESTE STEP!!!

		Para obtener informacion del Servidor de Allegro, realiza request hacia Allegro via el AllegroProxy.
		El AllegroProxy antes de realizar el request, valida que el VCC tenga integracion con Allegro (CF: VCCs[elVCC].AllegroIntegration.enabled = true).
		Cando se realizo el STEP 1 de la migracion, este valor ha sido modificado a (CF: VCCs[elVCC].AllegroIntegration.enabled = false), entonces el AllegroProxy falla al enviar los request a Allegro.

		Para solucionar esto debo cambiar este valor nuevamente a (CF: VCCs[elVCC].AllegroIntegration.enabled == true), para que se puedan realizar los request a Allegro nuevamente.
		Cuando el STEP 2 de la migracion finaliza, este valor vuele a quedar en (CF: VCCs[elVCC].AllegroIntegration.enabled == false).

		A esta actualizacion la realiza en un loop con sleep esperando a que el AllegroProxy se entere que este valor ha cambiado.

		Si despues del ultimo intento (el numero 5) el AllegroProxy sigue sin poder realizar request contra Allegro, cancelo y dejo el atributo nuevamente con el estado en el cual habia quedado en el STEP 1 de la migracion.

]]--
m_migration.GetAllegroVCCWhithRetries = function(self, info, occVccInfo)
	local success, allegroVccInfo
	local maxRetry = 200
	local retry = 1
	while retry <= maxRetry do
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.GetAllegroVCCWhithRetries] Retrieve Allegro VCC '%s'. retry %s of %d.", info.vccId, retry, maxRetry)
		if not occVccInfo.AllegroIntegration.enabled then
			-- actualizo el flag del VCC para que el AllegroProxy pueda realizar request contra Allegro
			occVccInfo.AllegroIntegration.enabled = true
			m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
		end
		require("socket").sleep(5) -- Pequena pausa necesaria para que el AllegroProxy obtenga la informacion actualizada de la CF VCCs
		-- obtengo informacion del VCC de Allegro
		success, allegroVccInfo = m_migration:GetAllegroVCC(info.vccId)
		if success then
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.GetAllegroVCCWhithRetries] OK Retrieved Allegro VCC '%s'. retry %s of %d.", info.vccId, retry, maxRetry)
			break
		end
		retry = retry + 1
	end
	return success, allegroVccInfo
end

--[[
	Para una migracion STEP 2) SOLO PARA ESTE STEP!!!

	Esta funcion realiza lo mismo que la funcion 'GetAllegroVCCWhithRetries' por eso simplemente se declara con otro nombre.
	Esto porque en la funcion que migra STEP 2, tiene que cambiar el atributo de Cassandra del VCC AllegroIntegration a true para que se puedan realizar request al server de Allegro para obtener contactos.
]]--
m_migration.ChangeVccToAllegroWithRetries = m_migration.GetAllegroVCCWhithRetries

--[[
	Realizar validaciones:
		1 - Que no exista otra migracion/rollback en ejecucion.
		2 - Que el vcc exista en la BD de Occ.
		3 - Que sea un vcc que tenga integracion con allegro.
		4 - Que si el vcc de allegro ya no fue migrado anteriormente.
		5 - Que exista comunicacion entre ambos Webhandlers OCC->Allegro.
		6 - Que los indices de ES existan para el VCC (dialer, blacklist)
		7 - Que no existan usuarios con alguna sesion iniciada en el VCC (authFilter)
]]--
m_migration.CanStartMigration = function(self, info)
	assert(info)
	assert(info.vccId)
	assert(info.ksAgent)
	assert(info.ksProv)
	assert(info.step)
	local vccId = info.vccId
	local ksAgent = info.ksAgent

	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CanStartMigration] Validate Start Migration Allegro VCC to Native VCC. vccId '%s'. stepGroupId: %s", vccId, info.step)

	-- 0.5. valida que todos los nodos del OE se encuentren listos (el servicio  outboundengine se encuentre arriba y ok)
	local oeSystemAPI = utils.GetOEApi("system") -- Obtengo el API del "outbound_engine".
	local isReadyAllOENodes, resp = oeSystemAPI.native.AreNodesReady()
	if not isReadyAllOENodes then
		local err = resp
		return false, {errType = "ALL NODES OF OUTBOUND ENGINE ARE IS NOT READY", vccId = vccId, err = err}
	end

	-- 1. valido si existe otra migracion/rollback en running
	local isRunning, currentMigration = m_running.IsRunning()
	if isRunning then
		return false, {errType = "THERE IS ANOTHER MIGRATION IN PROGRESS", vccId = vccId, currentMigration = currentMigration}
	end
	-- 2. valido que el vcc exista en la BD de Occ
	local success, occVccInfo = m_migration:GetOccVCC(ksAgent, vccId)
	if not success then
		local err = occVccInfo
		return false, err
	end
	occVccInfo.AllegroIntegration = utils.JSON_Decode(occVccInfo.AllegroIntegration)
	-- 3. valido que sea un vcc que tenga integracion con allegro
	if not occVccInfo.AllegroIntegration then
		return false, {errType = "VCC DOES NOT HAVE INTEGRATION WITH ALLEGRO", vccId = vccId}
	end
	--3.1 valido si el VCC puede ser almacenado (provisioningmanager, licencias, etc) ej: por el error "The company is invalid"
	local success, encodedErr = m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
	if not success then
		return false, {errType = encodedErr, vccId = vccId}
	end
	--
	local successRetrievedAllegroVcc, allegroVccInfo
	local step2MigrationType -- "ALL CONTACTS OF VCC", "CONTACTS OF SOME PROCESS BATCHES", "CONTACTS NOT IN ANY BATCH PROCESSES"
	local step2ExecutionType -- -- "NORMAL", "CONTINUE AND REPAIR FAILED MIGRATION"

	if info.step == "STEP 1" then
	-- STEP 1 validations
	--
		-- 4. valido si el vcc de allegro ya no fue migrado anteriormente.
		if not occVccInfo.AllegroIntegration.enabled then
			return false, {errType = "VCC HAS ALREADY BEEN MIGRATED", vccId = vccId}
		end
		-- 5. valido si se hizo una migracion anterior y si la misma culmino correctamente; si la misma ha fallado,
			-- ejemplo por una caida del webhandler o porque lo detuvieron, tiene que correr un rollback para solucionar los datos migrados hasta el momento
		local lastMigration = occVccInfo.AllegroIntegration.LastMigration
		if lastMigration and (lastMigration.status ~= "ENDED" and lastMigration.status ~= "ROLLBACKED") then
			return false, {errType = "PREVIOUS MIGRATION ENDED WITH FAILURES. YOU HAVE TO RUN A ROLLBACK TO FIX BUG", vccId = vccId}
		end
		-- obtengo informacion del VCC de Allegro
		successRetrievedAllegroVcc, allegroVccInfo = m_migration:GetAllegroVCC(vccId)
	elseif info.step == "STEP 2" then
	-- STEP 2 validations
		assert(type(info.contactsMigration) == "table")
		assert(type(info.contactsMigration.migrationType) == "string")
		assert(type(info.contactsMigration.executionType) == "string")
		step2MigrationType = info.contactsMigration.migrationType:upper()
		step2ExecutionType = info.contactsMigration.executionType:upper()

		-- valido que TODOS los procesos del VCC en el OE se encuentren ready.
		local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
		local isReadyAllVccProcesses resp = apiOeProcesses.AreAllVccProcessesReady(vccId)
		if isReadyAllVccProcesses then
			local err = resp
			return false, {errType = "ALL PROCESSES IN OUTBOUND ENGINE ARE IS NOT READY", vccId = vccId, err = err}
		end

		-- deben coincidir con los KEY del archivo de consola "migrate_allegro_vcc.lua"
		-- y con el archivo de Allegro (5.5) \Web\inConcert\apps\omnichannelintegration\outboundengine\migrations.lua funcion "MakeWhereConditionsForProcessesAndBatches"
		local allowedTypes = {
			["ALL CONTACTS OF VCC"]                 = true, -- obtiene todos los contactos del VCC en Allegro
			["CONTACTS OF SOME PROCESS BATCHES"]    = true, -- obtiene los contactos de algunos procesos y batches del VCC en Allegro
			["CONTACTS NOT IN ANY BATCH PROCESSES"] = true  -- obtiene los contactos que no se encuentren en ningun proceso/batches del VCC en Allegro
		}
		if not allowedTypes[step2MigrationType] then
			return false, {errType = "INVALID CONTACT MIGRATION TYPE - STEP 2", vccId = vccId, migrationType = step2MigrationType, executionType = step2ExecutionType}
		end
		-- si es una migracion por partes de Procesos o Batches debe entrar en el IF.
		--   es decir si no ha seleccionado migrar todos los contactos del VCC.
		--
		if step2MigrationType == "CONTACTS OF SOME PROCESS BATCHES" or step2MigrationType == "CONTACTS NOT IN ANY BATCH PROCESSES" then
			local nativeProcesses = apiOeProcesses.LoadProcessesAndBatches(info.ksOE, info.vccId)
			if utils.TableIsEmpty(nativeProcesses) then
				-- Si no ha migrado previamente ningun proceso ni batch, no puede ejecutar estas acciones
				return false, {errType = "UNABLE TO EXECUTE CONTACT MIGRATION TYPE - STEP 2. NO BATCH HAS BEEN MIGRATED IN STEP 1 OF THE MIGRATION.", vccId = vccId, migrationType = step2MigrationType, executionType = step2ExecutionType}
			end
			-- recorro los procesos de Allegro que se migraron a OCC y me los quedo en un array y en un map.
			--  los mismos seran utilizados en las validaciones.
			local allegroProcessesAndBatches, allegroProcessesAndBatchesMap
			-- estos mapas son utilizados para validar que no se migren mas de una vez los contactos de un mismo batch
			local nativeProcessesAndBatchesMap = {}
			local previousMigratedProcessesAndBatches = {}

			for fullProcessId, nativeProcess in pairs(nativeProcesses) do
				-- mapeo de OCC (nativo) -----------------------------------------
				local processId = utils.GetId(fullProcessId):lower()
				nativeProcessesAndBatchesMap[processId] = nativeProcessesAndBatchesMap[processId] or {batchesMap = {}}
				for _, batch in pairs(nativeProcess.batches or {}) do
					local batchName = batch.configuration.name:lower()
					nativeProcessesAndBatchesMap[processId].batchesMap[batchName] = batch
				end
	
				-- mapeo de Allegro -----------------------------------------
				local allegroOeProcess = (nativeProcess.Allegro or {}).OeProcess
				if allegroOeProcess then
					-- para el array de procesos y batches
					local allegroProcessId = allegroOeProcess.Id:lower()
					local allegroProcess = {
						processId = allegroOeProcess.Id,
						batches   = {}
					}
					-- para el map de batches
					local allegroBatchesMap = {}
					for _, allegroBatch in ipairs(allegroOeProcess.OutboundProcessBatchs or {}) do
						local allegroBatchId = allegroBatch.BatchId:lower()
						if nativeProcessesAndBatchesMap[allegroProcessId] then
							local nativeBatch = nativeProcessesAndBatchesMap[allegroProcessId].batchesMap[allegroBatchId]
							if nativeBatch and nativeBatch.Allegro and nativeBatch.Allegro.isMigratedContacts then
								-- aca nos quedamos con los procesos y batch que ya fueron migrados.
								previousMigratedProcessesAndBatches[allegroProcessId] = previousMigratedProcessesAndBatches[allegroProcessId] or {}
								previousMigratedProcessesAndBatches[allegroProcessId][allegroBatchId] = true
							end
						end
						-- el map de batches
						allegroBatchesMap[allegroBatchId] = allegroBatch.BatchId
						-- el array de batches
						table.insert(allegroProcess.batches, {
							batchId = allegroBatch.BatchId
						})
					end
					if #allegroProcess.batches > 0 then
						-- el map de procesos y su map de batches
						allegroProcessesAndBatchesMap = allegroProcessesAndBatchesMap or {}
						allegroProcessesAndBatchesMap[allegroOeProcess.Id:lower()] = {
							processId = allegroOeProcess.Id,
							batches = allegroBatchesMap
						}
						-- el array de procesos y su array de batches
						allegroProcessesAndBatches = allegroProcessesAndBatches or {
							processes = {}
						}
						table.insert(allegroProcessesAndBatches.processes, allegroProcess)
					end
				end
			end
			--LogDebug("1 - nativeProcessesAndBatchesMap: %s", json.encode(nativeProcessesAndBatchesMap or {}))
			--LogDebug("2 - previousMigratedProcessesAndBatches: %s", json.encode(previousMigratedProcessesAndBatches or {}))
			if step2MigrationType == "CONTACTS OF SOME PROCESS BATCHES" then
				-- aca realizo validaciones cuando se van migrar los contactos de algunos procesos/batches.
				--
				assert(info.contactsMigration.fileName)
				local success, jsonData = utils.ReadFile(info.contactsMigration.fileName)
				--LogDebug("--------> process and batches to migrate by file: %s", jsonData)
				if not success then
					-- El archivo que contiene los procesos y batches migrados NO existe en HDD
					return false, {errType = "CONTACT MIGRATION FILE DOES NOT EXIST- STEP 2", vccId = vccId, fileName = info.contactsMigration.fileName}
				end
				local migratedAllegroProcesses = json.decode(jsonData)
				if #(migratedAllegroProcesses.processes or {}) == 0 then
					-- El archivo debe contener al menos un proceso
					return false, {errType = "MUST INCLUDE AT LEAST ONE PROCESS AND ONE BATCH- STEP 2", vccId = vccId}
				end
				-- ahora realiza validaciones contra la estructura del JSON que contiene los procesos y batches.
				local alreadyMigrated = {}
				for _, process in ipairs(migratedAllegroProcesses.processes or {}) do
					if type(process) ~= "table" then
						-- El proceso debe ser una tabla
						return false, {errType = "PROCESS THAT HAS BEEN INCLUDED DOES NOT HAVE AN INVALID FORMAT- STEP 2", vccId = vccId, process = process}
					end
					local processId = (process.processId or ""):lower()
					if not allegroProcessesAndBatchesMap[processId] then
						-- el proceso que ha incluido en el archivo, NO se ha migrado
						return false, {errType = "PROCESS THAT HAS BEEN INCLUDED HAS NOT BEEN MIGRATED- STEP 2", vccId = vccId, processId = process.processId}
					end
					if #(process.batches or {}) == 0 then
						-- el proceso que ha incluido en el archivo, NO incluye ningun batch
						return false, {errType = "PROCESS THAT HAS BEEN INCLUDED DOES NOT HAVE BATCHES- STEP 2", vccId = vccId, processId = process.processId}
					end
					for _, batch in pairs(process.batches) do
						if type(batch) ~= "table" then
							-- el batch debe ser una tabla
							return false, {errType = "BATCH THAT HAS BEEN INCLUDED DOES NOT HAVE AN INVALID FORMAT- STEP 2", vccId = vccId, processId = process.processId, batch = batch}
						end
						local batchId = (batch.batchId or ""):lower()
						if not allegroProcessesAndBatchesMap[processId].batches[batchId] then
							-- el batch que ha incluido en el archivo, NO se ha migrado
							return false, {errType = "BATCH YOU HAVE INCLUDED HAS NOT BEEN MIGRATED- STEP 2", vccId = vccId, processId = process.processId, batchId = batchId}
						end
						if previousMigratedProcessesAndBatches[processId] and
							previousMigratedProcessesAndBatches[processId][batchId] then
								table.insert(alreadyMigrated, {
									processId = processId,
									batchId   = batchId
								})
						end
					end
				end
				--
				LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CanStartMigration] Validate Start Migration Allegro VCC to Native VCC. vccId '%s'. stepGroupId: %s. ==> Total alreadyMigrated: %s", vccId, info.step, #alreadyMigrated)
				if step2ExecutionType == "NORMAL" or step2ExecutionType == nil then
					if #alreadyMigrated > 0 then
						--  NO deja migrar nuevamente los contactos de los batches. Si alguno de los batches que quiere migrar, ya fueron migrados (sus contactos) falla
						return false, {errType = "UNABLE PERFORM MIGRATION SINCE IT CONTAINS MIGRATED BATCHES IN THE PROCESSES- STEP 2", vccId = vccId, alreadyMigrated = alreadyMigrated}
					end
				end
				-- OK
				info.contactsMigration.processAndBatches = migratedAllegroProcesses
			else
				-- aca realizo validaciones cuando se van migrar los contactos que NO se encuentran en ninguno de los procesos/batches.
				--
				if not allegroProcessesAndBatches then
					-- Si no ha migrado previamente ningun proceso ni batch, no puede ejecutar estas acciones
					return false, {errType = "UNABLE TO EXECUTE CONTACT MIGRATION TYPE - STEP 2. HAS NOT MIGRATED ALLEGRO PROCESS.", vccId = vccId, migrationType = step2MigrationType, executionType = step2ExecutionType}
				end
				-- OK
				info.contactsMigration.processAndBatches = allegroProcessesAndBatches
			end
			info.contactsMigration.processAndBatches = info.contactsMigration.processAndBatches or {}
			--LogDebug("---> info.contactsMigration.processAndBatches: %s", json.encode(info.contactsMigration.processAndBatches))
			local importModule = GetStepByModule("oeImportContacts", "importations")
			importModule.SetMigratedContactsInfo(info.contactsMigration.processAndBatches)

-- 			-- ESO SIRVE cuando estamos desarrollando y solo queremos probar la validacion (sin ejecutar la migracion)
-- return false, {errType = "errrrrrrrrrrrrrrrrrrrrrrror 1111111111111111", vccId = vccId}
		end

		-- 3- valida si es un vcc que tenga integracion con allegro
		-- 4- valida si el vcc no fue migrado previamente
		local lastMigration = occVccInfo.AllegroIntegration.LastMigration
		if occVccInfo.AllegroIntegration.enabled then
			if not lastMigration then
				return false, "VCC HAS NOT BEEN MIGRATED"
			end
			if lastMigration.status == "ROLLBACKED" then
				-- Se revirtió la migración anterior. Debe ejecutar el paso 1 de la migración y luego puede ejecutar el paso 2.
				return false, {errType = "THE PREVIOUS MIGRATION WAS ROLLBACKED. YOU MUST RUN STEP 1 OF THE MIGRATION AND THEN YOU CAN RUN STEP 2.", vccId = vccId}
			end
			-- TIENE UNA MIGRACIÓN ANTERIOR QUE TERMINO CON FALLAS. SE QUE EJECUTARA EL ROLLBACK CON LA OPCIÓN DE REPARAR LOS DATOS MIGRADOS.
			if step2ExecutionType == "NORMAL" or step2ExecutionType == nil then
				return false, {errType = "a-ATTENTION: STEP OF THE VCC MIGRATION HAS FAILED. I KNOW THAT THE ROLLBACK WILL BE EXECUTED WITH THE OPTION TO REPAIR THE MIGRATED DATA. lastMigration.status: " .. lastMigration.status, vccId = vccId}
			end
		end
		-- 5- valido si se hizo una migracion anterior y si la misma culmino correctamente; si la misma ha fallado,
			-- ejemplo por una caida del webhandler o porque lo detuvieron, tiene que correr un rollback para solucionar los datos migrados hasta el momento
		if step2ExecutionType == "NORMAL" or step2ExecutionType == nil then
			if lastMigration and (lastMigration.status ~= "ENDED") then
				-- TIENE UNA MIGRACIÓN ANTERIOR QUE TERMINO CON FALLAS. SE QUE EJECUTARA EL ROLLBACK CON LA OPCIÓN DE REPARAR LOS DATOS MIGRADOS.
				return false, {errType = "b-ATTENTION: STEP 1 OF THE VCC MIGRATION HAS FAILED. I KNOW THAT THE ROLLBACK WILL BE EXECUTED WITH THE OPTION TO REPAIR THE MIGRATED DATA. lastMigration.status: " .. lastMigration.status, vccId = vccId}
			end
		end
		--[[
			Para obtener informacion del Servidor de Allegro, realiza request hacia Allegro via el AllegroProxy.
			El AllegroProxy antes de realizar el request, valida que el VCC tenga integracion con Allegro (CF: VCCs[elVCC].AllegroIntegration.enabled = true).
			Cando se realizo el STEP 1 de la migracion, este valor ha sido modificado a (CF: VCCs[elVCC].AllegroIntegration.enabled = false), entonces el AllegroProxy falla al enviar los request a Allegro.

			Para solucionar esto debo cambiar este valor nuevamente a (CF: VCCs[elVCC].AllegroIntegration.enabled == true), para que se puedan realizar los request a Allegro nuevamente.
			Cuando el STEP 2 de la migracion finaliza, este valor vuele a quedar en (CF: VCCs[elVCC].AllegroIntegration.enabled == false).

			A esta actualizacion la realiza en un loop con sleep esperando a que el AllegroProxy se entere que este valor ha cambiado.

			Si despues del ultimo intento, el AllegroProxy sigue sin poder realizar request contra Allegro, cancelo y dejo el atributo nuevamente con el estado en el cual habia quedado en el STEP 1 de la migracion.
		]]--
		successRetrievedAllegroVcc, allegroVccInfo = m_migration:GetAllegroVCCWhithRetries(info, occVccInfo)
		if not successRetrievedAllegroVcc then
			-- si falla el ultimo intento contra el AllegroProxy debo dejar en cassandra el VCC como migrado (segun lo dejo el STEP 1 de migracion)
			LogError(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CanStartMigration] ERROR Retrieving Allegro VCC '%s'.", vccId)
			occVccInfo.AllegroIntegration.enabled = false
			m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
		end
	else
		return false, {errType = "UNSUPPORTED MIGRATION STEP: " .. tostring(info.step), vccId = vccId}
	end
	-- 6. valido exista comunicacion entre ambos Webhandlers OCC->Allegro.
	if not successRetrievedAllegroVcc then
		local err = allegroVccInfo
		return false, {errType = "UNABLE TO ESTABLISH A CONNECTION BETWEEN WEBHANDLERS OF OCC AND ALLEGRO", vccId = vccId, err = err}
	end
	-- 7. valido exista comunicacion entre ambos Webhandlers OCC->Allegro.
	if not allegroVccInfo.ContactImportVCC or type(tonumber(allegroVccInfo.ContactImportVCC.ID)) ~= "number" then
		if info.step == "STEP 2" then
			-- si no tiene comunicacion con AllegroProxy debo dejar en cassandra el VCC como migrado (segun lo dejo el STEP 1 de migracion)
			occVccInfo.AllegroIntegration.enabled = false
			m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
		end
		return false, {errType = "ALLEGRO VCC DOES NOT HAS VCC NUMBER", vccId = vccId, contactImportVCC = allegroVccInfo.ContactImportVCC or "invalid"}
	end
	-- 8. valido contra ES que se haya creado el indice de DIALER y BLACKLIST
	local success, err = m_migration:ESValidate(info, vccId)
	if not success then
		if info.step == "STEP 2" then
			-- si la validacion de indices de ES ha fallado debo dejar en cassandra el VCC como migrado (segun lo dejo el STEP 1 de migracion)
			occVccInfo.AllegroIntegration.enabled = false
			m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
		end
		return false, err
	end
	-- 9. valido contra la rocka authFilter que no existan usuarios loggedIn en el VCC.
	local success, err = m_migration:ValidateLoggedUsers(vccId)
	if not success then
		if info.step == "STEP 2" then
			-- si no puede comenzar la migracion del STEP 2) porque existe algun usuario con una sesion activa debo dejar en cassandra el VCC como migrado (segun lo dejo el STEP 1 de migracion)
			occVccInfo.AllegroIntegration.enabled = false
			m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
		end
		return false, err
	end
	-- 10. sanitize OCC vcc info (AppsIntegrations)
	if info.step == "STEP 2" then
		occVccInfo.AllegroIntegration.enabled = false
		m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, occVccInfo, info.ksProv)
	end
	occVccInfo.AppsIntegrations = utils.JSON_Decode(occVccInfo.AppsIntegrations)

	-- -- ESO SIRVE cuando estamos desarrollando y solo queremos probar la validacion (sin ejecutar la migracion)
	-- if true then
	-- 	return false, {errType = "errrrrrrrrrrrrrrrrrrrrrrror 22222222222", vccId = vccId}
	-- end

	-- ok . puede comenzar la migracion.
	return true, occVccInfo, allegroVccInfo
end

--[[

]]--
m_migration.ValidateLoggedUsers = function(self, vccId)
	local authFilter = (require "authfilter")("agent")
	local currentVCCSessions = authFilter:GetAllSessions(vccId, nil, false) -- el Tercer parametro en "false" dice que NO debe excluir los SuperUsuarios.
	if not utils.TableIsEmpty(currentVCCSessions) then
		local loggedUsers = {}
		for _, loggedUser in pairs(currentVCCSessions) do
			table.insert(loggedUsers, loggedUser.username)
		end
		return false, {errType = "THERE ARE USERS LOGGED IN IN THE VCC", vccId = vccId, totalLoggedUsers = #loggedUsers, loggedUsers = loggedUsers}
	end
	-- ok
	return true
end

--[[
	Crea los indices y Templates de ES para DIALER y BLACKLIST
]]--
m_migration.CreateESIndexesForVCC = function(self, ksAgent, vccId)
	local indexes = "contacts_dialer|blacklist_dialer"
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CreateESIndexesForVCC] Creating ES indexes: '%s' for vcc: '%s'.", indexes, vccId)
	m_vccsModel.VCC().CreateESIndexesForVCC(vccId, indexes, ksAgent)
end

--[[
	Realizar validaciones contra los indices de ES:
		1 - Que exista el indice DIALER
		2 - Que exista el indice BLACKLIST
]]--
m_migration.ESValidate = function(self, info, vccId)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.ESValidate] Validate ES Indexes (DIALER & BLACKLIST) vcc: '%s'.", vccId)
	-- 1. creo los indices de ES Dialer y BLACKLIST para el VCC.
	self:CreateESIndexesForVCC(info.ksAgent, vccId)
	-- 2. valido que exista el indice de ES Dialer para el VCC.
	local queryESParams = {
		filter = {},
		startRow = "",
		rowsPerPage = 1,
		vcc = vccId,
		sort = {{Name = {order = "asc"}}}
	}
	local success, response
	m_fullcontact:Initialize(function()
		success, response = Fullcontact.Dialer:SearchContacts(queryESParams)
	end)
	if not success then
		local indexName = Fullcontact.Dialer:GetESIndexName()
		return false, {errType = "DIALER ELASTIC SEARCH INDEX FOR VCC DOES NOT EXIST", vccId = vccId, indexName = indexName, err = response}
	end
	-- 3. valido que exista el indice de ES BLACKLIST
	local blackListAPI = Fullcontact.Dialer:GetBlackListApi()
	success, response = blackListAPI:SearchAddresses(queryESParams)
	if not success then
		local indexName = blackListAPI:GetESIndexName()
		return false, {errType = "BLACKLIST ELASTIC SEARCH INDEX DOES NOT EXIST", vccId = vccId, indexName = indexName, err = response}
	end
	-- ok
	return true
end

--[[
	TODO: Realizar validaciones
		1 - Que el vcc sea un vcc de allegro.
		2 - Si existe otra migracion/rollback en running
		3 - Que no existan usuarios con alguna sesion iniciada en el VCC (authFilter)
]]--
m_migration.CanStartRollback = function(self, vccId, ksAgent, ksProv)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CanStartRollback] Validate Start Rollback Migration Allegro VCC to Native VCC. vccId '%s'.", vccId)
	-- 0.5 - Que el vcc sea un vcc de allegro.
	local success, occVccInfo = m_migration:GetOccVCC(ksAgent, vccId)
	if not success then
		local err = occVccInfo
		return false, err
	end
	--1 valido si el VCC puede ser almacenado (provisioningmanager, licencias, etc) ej: por el error "The company is invalid"
	local success, encodedErr = m_vccsModel.UpdateVCC(ksAgent, vccId, occVccInfo, ksProv)
	if not success then
		return false, {errType = encodedErr, vccId = vccId}
	end
	occVccInfo.AllegroIntegration = utils.JSON_Decode(occVccInfo.AllegroIntegration)
	if not occVccInfo.AllegroIntegration then
		return false, {errType = "VCC DOES NOT HAVE INTEGRATION WITH ALLEGRO", vccId = vccId}
	end
	if occVccInfo.AllegroIntegration.enabled then
		-- 1.1. valido si se hizo una migracion anterior y si la misma culmino correctamente; si la misma ha fallado,
		-- ejemplo por una caida del webhandler o porque lo detuvieron, tiene que correr un rollback para solucionar los datos migrados hasta el momento
		local lastMigration = occVccInfo.AllegroIntegration.LastMigration
		if lastMigration and (lastMigration.status ~= "ENDED" and lastMigration.status ~= "ROLLBACKED") then
			-- 1.4. la migracion anterior ha fallado: PREVIOUS MIGRATION ENDED WITH FAILURES. YOU HAVE TO RUN A ROLLBACK TO FIX BUG
			--  entonces debo dejar el VCC como que la migracion ha culminado correctamente entonces puedo ejecutar correctamente el rollback para que solucione los datos que quedaron
			--   migrados a medias.
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CanStartRollback] PREVIOUS MIGRATION ENDED WITH FAILURES. vccId '%s'. The forced rollback will be executed to correct migration data.", vccId)

			occVccInfo.AllegroIntegration.enabled = false
			m_vccsModel.UpdateVCC(ksAgent, vccId, occVccInfo, ksProv)
		else
			-- 1.2. si es un vcc que tiene integracion debe fallar.
			if occVccInfo.AllegroIntegration.Rollback then
				-- 1.3. si al vcc ya se le ha realizado rollback
				return false, {errType = "VCC HAS ALREADY BEEN ROLLBACKED MIGRATION", vccId = vccId}
			end
			-- si el vcc nunca ha sido migrado
			return false, {errType = "VCC ALREADY HAS INTEGRATION WITH ALLEGRO", vccId = vccId}
		end
	end
	-- 2. valido si existe otra migracion/rollback en running
	local isRunning, currentMigration = m_running.IsRunning()
	if isRunning then
		return false, {errType = "THERE IS ANOTHER MIGRATION IN PROGRESS", currentMigration = currentMigration}
	end
	-- 3. valido contra la rocka authFilter que no existan usuarios loggedIn en el VCC.
	local success, err = m_migration:ValidateLoggedUsers(vccId)
	if not success then
		return false, err
	end
	-- ok
	return true, occVccInfo
end

-----------------------------------------
-- RUNNING
-----------------------------------------

--[[
	Atencion si esta clave se modifica, debe de ser modificada tambien en:
		repo   : lua_authfilter
		archivo: authfilter.lua
		funcion: m_migration.MakeKey
]]--
m_running.MakeKey = function()
	local key = "Migration:Allegro:VCC:Status"
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:MakeKey]------------> key: '%s'", key)
	return key
end

--[[

]]--
m_running.Get = function()
	local key = m_running.MakeKey()
	local GetRedisValue = function()
		local redis = m_provisioningModel.GetRedisClient()
		local encodedCurrentRunning = redis:get(key)
		local runningInfo = utils.JSON_Decode(encodedCurrentRunning)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Get]------------> key: '%s', encodedCurrentRunning: '%s'", key, encodedCurrentRunning)
		return runningInfo, encodedCurrentRunning
	end
	local success, runningInfo, encodedCurrentRunning = pcall(GetRedisValue)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Get]-----pcall-------> success: %s, key: '%s', runningInfo: %s, encodedCurrentRunning: '%s'", success, key, runningInfo, encodedCurrentRunning)
	if not success then
		return {}, json.encode({})
	end
	-- ok
	return runningInfo, encodedCurrentRunning
end

--[[

]]--
m_running.Set = function(runningInfo)
	assert(type(runningInfo) == "table")
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Set] 1 ....... runningInfo: %s", json.encode(runningInfo))
	assert(type(runningInfo.vcc) == "string")

	local encodedRunningInfo
	local key = m_running.MakeKey()
	local SetRedisValue = function()
		local redis = m_provisioningModel.GetRedisClient()
		if runningInfo.status == "RUNNING" then
			--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Set] 2 ....... runningInfo.status: RUNNING action: %s", runningInfo.action)
			encodedRunningInfo = utils.JSON_Encode(runningInfo)
			redis:set(key, utils.JSON_Encode(runningInfo))
			redis:expire(key, m_running.EXPIRE_SEC)
			m_running.CreateFile(runningInfo, encodedRunningInfo) -- almaceno en hdd el estado de la migracion
		else
			--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Set] 3 ....... runningInfo.status: ", runningInfo.status)
			redis:del(key)
			local runningInfo = m_running.Get()
			if not utils.TableIsEmpty(runningInfo) then
				LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Set] Unable to Set. running key '%s'", key)
				return false
			end
		end
	end
	local success = pcall(SetRedisValue)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Set]-----pcall-------> success: %s, key: '%s', runningInfo: %s, encodedCurrentRunning: '%s'", success, key, runningInfo, json.encode(runningInfo))
	if not succcess then
		return false
	end
	return key, runningInfo, encodedRunningInfo
end

--[[
	Crea un archivo json con los steps de la migracion o el rollback.
	Ej: /tmp/inconcert/migration_result_vcc_emmatel2_utc_start_date_2023_04_18_13_53.json
]]--
m_running.mustCreateFile = true
m_running.CreateFile = function(runningInfo, encodedRunningInfo)
	if not m_running.mustCreateFile then
		return false
	end
	return utils.CreateFile(runningInfo.fullFileName, encodedRunningInfo)
end

--[[
	Crea el nombre del archivo de resultados json con los steps de la migracion o el rollback.
	Ej: /tmp/inconcert/migrations/migration_result_vcc_emmatel2_utc_start_date_2023_04_18_13_53_id_xxxxxxxxxxxxxxxxxxxxxxxx.json
]]--
m_running.MakeFileName = function(runningInfo)
	if not m_running.mustCreateFile then
		return false
	end
	if not runningInfo.vcc or not runningInfo.startDate or not runningInfo.processType then
		return false
	end
	--[[
		directorio          : /tmp/inconcert/migrations/
		archivo de migracion: migration_result_vcc_emmatel2_utc_start_date_2023_04_18_13_53_id_xxxxxxxxxx.json
		archivo de rollback : rollback_result_vcc_emmatel2_utc_start_date_2023_04_18_13_53_id_xxxxxxxxxxx.json
	]]--
	local fullFileName = ("%s/result.json"):format(m_running.currentMigrationDir):lower()
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:MakeFileName]------------>  fullFileName: '%s'", fullFileName)
	return fullFileName
end

--[[

]]--
m_running.baseDir = "/tmp/inconcert/migrations"
m_running.currentMigrationDir = nil -- este cambia segun la migracion en curso.
m_running.tempStoredKey = nil       -- este cambia segun la migracion en curso.

m_running.CreateTemporalDirs = function(runningInfo)
	-- 1- creo el directorio base de todas las migraciones
	local baseDir = m_running.baseDir
	utils.CreateDirectory(baseDir)
	-- 2- creo el directorio del current process (migracion o rollback) + el id
	--   el directorio viene dado por parametro, unicamente cuando estamos realizando un CONTINUE del step 2 por caida/stop del webhandler
	if runningInfo.currentMigrationDir then
		-- para el caso del STEP 2 - cuando corren con la opcion CONTINUE failed migration
		m_running.currentMigrationDir = runningInfo.currentMigrationDir
		m_running.tempStoredKey       = runningInfo.tempStoredKey
		--LogDebug("1 - Already exist. m_running.tempStoredKey:  %s, m_running.currentMigrationDir: %s", m_running.tempStoredKey, m_running.currentMigrationDir)
	else
		local sanitizedStepName = not runningInfo.migrationStep
			and ""
			or (runningInfo.migrationStep):gsub(" ","")
		require "LuaOrm.Date"
		local formattedStartDate = LuaOrm.Date(runningInfo.startDate):toString("%Y_%m_%d_%H_%M_%S")
		m_running.formattedStartDate = formattedStartDate

		-- prefix key de cassandra, CF TempVccAllegroMigrationOutProcessesBatches
		m_running.tempStoredKey = ("%s_id_%s"):format(
			formattedStartDate,
			runningInfo.migrationId
		):lower()

		m_running.currentMigrationDir = ("%s/%s_%s_result_vcc_%s_utc_start_date_%s"):format(
			m_running.baseDir,
			runningInfo.processType,
			sanitizedStepName,
			runningInfo.vcc,
			m_running.tempStoredKey
		):lower()
		--LogDebug("2 - Creating. m_running.tempStoredKey:  %s, m_running.currentMigrationDir: %s", m_running.tempStoredKey, m_running.currentMigrationDir)
	end

	utils.CreateDirectory(m_running.currentMigrationDir)
	-- ok
	return {
		baseDir             = baseDir,
		currentMigrationDir = m_running.currentMigrationDir,
		tempStoredKey       = tempStoredKey                  -- prefix key de cassandra, CF TempVccAllegroMigrationOutProcessesBatches
	}
end

--[[

]]--
m_running.Start = function(info, processType)
	assert(info)
	assert(info.vcc)
	assert(processType)
	info.migrationId = info.migrationId or utils.Uuid(true)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Start]------------>  info: '%s'", json.encode(info))
	local runningInfo = {
		action              = "START",
		migrationId         = info.migrationId,
		processType         = processType, -- "MIGRATION" "ROLLBACK"
		startDate           = os.time(os.date("!*t")), -- current date in UTC
		status              = "RUNNING",
		steps               = {},
		migrationStep       = info.migrationStep, -- "STEP 1" "STEP 2"
		vcc                 = info.vcc,
		err                 = info.err,
		errType             = info.errType,
		currentMigrationDir = info.currentMigrationDir, -- directorio temporal del current migration
		tempStoredKey       = info.tempStoredKey        -- prefix key de cassandra, CF TempVccAllegroMigrationOutProcessesBatches
	}
	runningInfo.tempDirs = m_running.CreateTemporalDirs(runningInfo)
	runningInfo.fullFileName = m_running.MakeFileName(runningInfo)
	return m_running.Set(runningInfo)
end

--[[

]]--
m_running.Update = function(newModuleStep)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Update] 1------------>  newModuleStep: '%s'", newModuleStep)
	local runningInfo = m_running.Get()
	if utils.TableIsEmpty(runningInfo) then
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Update] 2 ....... ")
		return false
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Update] 3------------")
	runningInfo.action = "UPDATING"
	runningInfo.lastUpdateDate = os.time(os.date("!*t")) -- current date in UTC
	local canAdd = true
	for i, currentStep in ipairs(runningInfo.steps or {}) do
		if currentStep.id == newModuleStep.id then
			--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Update] 4 ------Update step currentStep.id: %s", currentStep.id)
			newModuleStep.lastUpdateDate = runningInfo.lastUpdateDate
			runningInfo.steps[i] = newModuleStep
			canAdd = false
			break
		end
	end
	if canAdd then
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Update] 5 ------Add step newModuleStep.id: %s", newModuleStep.id)
		newModuleStep.startDate = runningInfo.lastUpdateDate
		table.insert(runningInfo.steps, newModuleStep)
	end
	--steps
	return m_running.Set(runningInfo)
end

--[[

]]--
m_running.UpdateStepProcess = function(migrationModule, stepProcess)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 1------------>  migrationModule: '%s', stepProcess: '%s'", migrationModule, stepProcess)
	assert(migrationModule)
	assert(stepProcess)
	local runningInfo = m_running.Get()
	if utils.TableIsEmpty(runningInfo) then
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 2 ....... ")
		return false
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 3------------")
	runningInfo.action = "UPDATING"
	runningInfo.lastUpdateDate = os.time(os.date("!*t")) -- current date in UTC
	runningInfo.stepProcess = stepProcess
	-- local canAdd = true
	-- for i, currentStep in ipairs(runningInfo.steps or {}) do
	-- 	if currentStep.id == migrationModule.migrationId then
	-- 		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 4 ------Update step currentStep.id: %s", currentStep.id)
	-- 		currentStep.lastUpdateDate = runningInfo.lastUpdateDate
	-- 		for j, stepModule in ipairs(currentStep.steps or {}) do
	-- 			if stepModule.id == migrationModule.id then
	-- 				--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 5 ------Add step stepModule.id: %s", stepModule.id)
	-- 				stepModule.steps = stepModule.steps or {}
	-- 				stepModule.lastUpdateDate = runningInfo.lastUpdateDate
	-- 				--
	-- 				local canAdd = true
	-- 				for j, currentStepProcess in ipairs(stepModule.steps or {}) do
	-- 					if currentStepProcess.stepProcessId == stepProcess.stepProcessId then
	-- 						--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 6 ------Update step currentStepProcess.id: %s", currentStepProcess.id)
	-- 						stepProcess.lastUpdateDate = runningInfo.lastUpdateDate
	-- 						stepModule[j] = stepProcess
	-- 						canAdd = false
	-- 						break
	-- 					end
	-- 				end
	-- 				if canAdd then
	-- 					--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateStepProcess] 7 ------Add step stepProcess.id: %s", stepProcess.id)
	-- 					stepProcess.startDate = runningInfo.lastUpdateDate
	-- 					table.insert(stepModule.steps, stepProcess)
	-- 				end
	-- 				break
	-- 			end
	-- 		end
	-- 	end
	-- end
	--steps
	return m_running.Set(runningInfo)
end

--[[

]]--
m_running.End = function()
	local runningInfo = m_running.Get()
	--LogDebug(LOG_PREFIX .. "[runningInfo.MIGRATIONS.running:End] 1--->  ending")
	if utils.TableIsEmpty(runningInfo) then
		--LogDebug(LOG_PREFIX .. "[runningInfo.MIGRATIONS.running:End] 2--->  ending")
		return false
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:End] 3--->  ending")
	runningInfo.status = "ENDING"
	runningInfo.endDate = os.time(os.date("!*t")) -- current date in UTC
	--LogDebug(LOG_PREFIX .. "[runningInfo.MIGRATIONS.running:End] 4 -->  ended - All Info: %s", json.encode(runningInfo))
	m_running.Set(runningInfo)
	return true
end

--[[

]]--
m_running.IsRunning = function()
	--m_running.End()
	local runningInfo, encodedCurrentRunning = m_running.Get()
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:IsRunning] 1------------> encodedCurrentRunning: '%s'", encodedCurrentRunning or "NADA")
	if utils.TableIsEmpty(runningInfo) then
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:IsRunning] 2------------")
		return false
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:IsRunning] 3------------")
	return true, runningInfo
end

--[[
	Actualiza la ultima traza del LastStepMigration (SOLO PARA EL STEP 2 - paginado de contactos y ejecuciones contra el OE->addcontact, OE->PauseContacts, etc)
		solo para este step2 de migracion ya que esos pasos pueden demorar muchos minutos hasta horas.
]]--
m_running.UpdateLastMigrationStep2InVcc = function(self, info, stepProcess)
	assert(type(info) == "table")
	assert(type(stepProcess) == "table")
	local lastMigration = info.occVccInfo.AllegroIntegration.LastMigration or {}
	local currentStepProcess = lastMigration.currentStepProcess or {}
	for name, value in pairs(stepProcess) do
		currentStepProcess[name] = value
	end
	currentStepProcess.time = os.time()
	lastMigration.currentStepProcess = currentStepProcess
	lastMigration.contactsMigration = info.contactsMigration
	local runningInfo = m_running.Get()
	if utils.TableIsEmpty(runningInfo) then
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateLastMigrationStep2InVcc] Attention - Running info is empty. lastMigration: %s", json.encode(lastMigration))
	end
	lastMigration.migrationId = runningInfo.migrationId
	-- informacion de los procesos y lotes que persisten en HDD para luego ser usados en el STEP2, en el caso de una caida/stop del Webhandler. con esta informacion va a poder asignar contactos a los procesos->lotes en al accion CONTINUE.
	local importModule = GetStepByModule("oeImportContacts", "importations")
	local _, processAndBatches = importModule:GetMigratedContactsInfo()
	lastMigration.contactsMigration.processAndBatches = processAndBatches
	--
	info.occVccInfo.AllegroIntegration.LastMigration = lastMigration
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:UpdateLastMigrationStep2InVcc] -------------> lastMigration: %s", json.encode(lastMigration))
	m_vccsModel.UpdateVCC(info.ksAgent, info.vccId, info.occVccInfo, info.ksProv)
end

--[[
	redis-cli -p 6377 del Migration:Allegro:VCC:Status

	De momento esta funcion NO se utiliza
]]--
m_running.Delete = function()
	local key = m_running.MakeKey()
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.running:Delete] Deleting redis key: '%s'.", key)
	local redis = m_provisioningModel.GetRedisClient()
	redis:del(key)
end

-----------------------------------------
-- ALLEGRO
-----------------------------------------
--[[

]]--
m_allegro.GetAllegroInfo = function(self, vccId, ksAgent)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro.GetAllegroInfo] Get Allegro VCC Info. vccId '%s'.", vccId)
	--1. obtengo otra informacion del vcc contra allegro (procesos).
	local hasAllegroIntegration = utils.HasAllegroIntegration(vccId, ksAgent)
	if not hasAllegroIntegration then
		return false
	end
	local oeProcessesStep = GetStepByModule("oeProcesses", "processes")
	local success, process = oeProcessesStep.LoadAllegroProcesses({vccId = vccId})
	if not success then
		-- TODO: Visualizar error o cancelar ??
		return false
	end
	local allegroInfo = {
		oeProcesses = {
			success = success,
			process = process
		}
	}
	-- ok
	return true, allegroInfo
end

--[[
	Funcion genercia, que obtiene todos los rows obteniendolos de pagina en pagina.
	Parametros:
		info     :  es la info que se envia para comenazar las migraciones de las diferentes entidades.
		otherInfo: contiene informacion relevante a la vista de Allegro de la cual se quieren obtener todas las paginas.
			- moduleInstance        = require del controller de la app (Ej: app "outbound_engine" controller "outboundengine_contacts" que termina yendo contra la app "allegro", que a su vez realiza request contra el webhandler de Allegro)
			- moduleName            = nombre del controller que se visualizara en el log
			- paginatedFunctionName = nombre de la funcion de paginado del controller "moduleInstance" que se ejecutara contra la app "oe->allegro"
			- listName              = nombre de la funcion del controller que se visualizara en el log
			- callback              = callback que se invocara cuando se haya obtenido cada pagina.
			- queryInfo             = informacion (parametros) del post que se realizara para ejecutar el paginado cuando se invoca a la funcion "paginatedFunctionName".
				- last              = no se utiliza, pero queda como referencia.
				- limit             = registros por pagina (rows per page)
				- startNumberPage   = numero de pagina a consultar.
				- query             = el filtro por el cual se buscara (segun definicion en la vista de allegro). los valores siempre van en ""
				- columns           = columnas que retornara la consulta (no requerido).
]]--
m_allegro.LoadAllRowsByPaging = function(self, info, otherInfo)
	assert(info)
	assert(otherInfo)
	assert(otherInfo.currentStepModule)
	assert(otherInfo.moduleInstance)
	assert(otherInfo.moduleName)
	assert(otherInfo.paginatedFunctionName)
	assert(otherInfo.queryInfo)
	assert(otherInfo.listName)
	assert(otherInfo.callback)

	-- TODO: Iterar topeando para que no quede en loop infinito por posible error.

	local moduleInstance = otherInfo.moduleInstance
	local moduleName = otherInfo.moduleName
	local paginatedFunctionName = otherInfo.paginatedFunctionName
	local queryInfo = otherInfo.queryInfo
	local listName = otherInfo.listName
	local callback = otherInfo.callback
	local logEachPage = otherInfo.logEachPage or 1
	local moduleAndPagedFn = string.format("%s.%s(..)", moduleName, paginatedFunctionName)
	local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()

	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro.LoadAllRowsByPaging] --> START LOAD ALL ROWS by paging of moduleAndPagedFn: '%s' listName '%s'. vccId '%s'. startTime: %s. logEachPage: %s", moduleAndPagedFn, paginatedFunctionName, listName, info.vccId, mainProcessElapsedTime.startTime, logEachPage)

	local runningInfo = m_running.Get()
	if utils.TableIsEmpty(runningInfo) then
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro:LoadAllRowsByPaging] Attention - Running info is empty.")
	end

	local requestInfo = {
		vcc         = info.vccId,
		vccNumber   = info.allegroVccInfo.ContactImportVCC.ID,
		migrationId = runningInfo.migrationId,
		last        = queryInfo.last or "",
		limit       = queryInfo.limit or 100,
		numberPage  = queryInfo.startNumberPage or 1,
		query       = queryInfo.query
	}
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro.LoadAllRowsByPaging]     ------>  vccId '%s', requestInfo: '%s'", info.vccId, json.encode(requestInfo))

	local stepProcessId = ("%s.%s"):format(otherInfo.listName, otherInfo.paginatedFunctionName) -- Sample: CONTACTS.LoadPaginatedContacts
	m_running.UpdateStepProcess(otherInfo.currentStepModule, {
		stepProcessId = stepProcessId,
		action = "start retrieving all page from allegro",
		mainProcessElapsedTime = mainProcessElapsedTime
	})

	local _, endTimeGroupPage
	local invokePageNumber = 0
	local totalRetrievedRows = 0
	local allPagesElapsedTimeSec = 0

	while true do
		-- LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro.LoadAllRowsByPaging]          --> START Loading Page -> moduleAndPagedFn: '%s'.'%s', vcc: '%s', numberPage: '%s'. startTime: %s - WAIT MOMENT PLEASE !!!!!!!!!!!",
		-- 	moduleAndPagedFn, listName, info.vccId, requestInfo.numberPage, pageProcessElapsedTime.startTime)

		-- m_running.UpdateStepProcess(otherInfo.currentStepModule, {
		-- 	stepProcessId = stepProcessId,
		-- 	action = "start retrieving one page from allegro",
		-- 	requestInfo = requestInfo,
		-- 	pageProcessElapsedTime = pageProcessElapsedTime
		-- })

		local _, startPageEndProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()
		if not endTimeGroupPage then
			endTimeGroupPage = startPageEndProcessElapsedTime
		end

		-- Ejemplo de la ejecucion de la siguiente linea --> outboundengine_contacts.LoadPaginatedContacts(json.encode(requestInfo))
		local success, data = moduleInstance[paginatedFunctionName](json.encode(requestInfo)) -- REALIZA UN REQUEST CONTRA ALLEGRO PARA OBTENER LA PAGINA !!!

		local _, endPageEndProcessElapsedTime = m_helpers.utils.ElapsedTime.End(startPageEndProcessElapsedTime.processId)
		endTimeGroupPage.elapsedTimeSec = (endTimeGroupPage.elapsedTimeSec or 0) + endPageEndProcessElapsedTime.elapsedTimeSec

		local rows = data.rows or {}
		local lastRow = data.lastRow
		local totalRowsAllPages = data.total_rows
		local totalRows = #rows
		totalRetrievedRows = totalRetrievedRows + totalRows
	
		invokePageNumber = invokePageNumber + 1
		local mustNewElapsedTime = invokePageNumber == logEachPage
		if mustNewElapsedTime then
			local endPage = requestInfo.numberPage
			local fromPage = (endPage - logEachPage) + 1
			local elapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(endTimeGroupPage.elapsedTimeSec)
			local averageTimeSecPerPage = logEachPage == 1 and "" or string.format("- Average TIME per page: %s", endTimeGroupPage.elapsedTimeSec / logEachPage)

			allPagesElapsedTimeSec = allPagesElapsedTimeSec + endTimeGroupPage.elapsedTimeSec
			local allPagesElapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(allPagesElapsedTimeSec)

			LogInfo(LOG_PREFIX .. "[..LoadAllRowsByPaging        ] (ALLEGRO)--> ALL PAGES TIME: %s -> DETAIL: fromPage: '%s' - endPage: '%s'-> %s of %s totalRows. lastRow: '%s' -> TIME: %s %s.\nRetrieved '%s'. vcc: '%s', success: '%s'.",
				allPagesElapsedTimeDetail.formattedTime,
				fromPage,
				endPage,
				totalRetrievedRows,
				totalRowsAllPages,
				lastRow,
				elapsedTimeDetail.formattedTime,
				averageTimeSecPerPage,
				listName,
				info.vccId,
				success
			)

			--totalRetrievedRows = 0
			invokePageNumber = 0
			endTimeGroupPage = nil
		end

		if not success then
			-- error
			local err = data
			m_running.UpdateStepProcess(otherInfo.currentStepModule, {
				stepProcessId = stepProcessId,
				action = "error retrieving one page from allegro",
				requestInfo = requestInfo,
				pageProcessElapsedTime = pageProcessElapsedTime,
				err = err
			})
			callback(false, err, nil, stepProcessId, requestInfo.numberPage, lastRow) -- como es un error notifico a quien me ha invocado y corto la obtencion de paginas
			break
		end
		-- ok
		local isLastPage = totalRows < requestInfo.limit
		-- envio los datos de la pagina para que pueda ser procesada
		local resp = callback(true, isLastPage, rows, stepProcessId, requestInfo.numberPage, lastRow)

		if isLastPage or resp.mustForceBreak then
			local _, mainProcessElapsedTime = m_helpers.utils.ElapsedTime.End(mainProcessElapsedTime.processId)
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.allegro.LoadAllRowsByPaging] --> END LOAD ALL ROWS by paging of moduleName: '%s' listName '%s'. vccId '%s'. endTime: %s, elapsedTime: %s. isLastPage: '%s', mustForceBreak: '%s'.",
				moduleName, listName, info.vccId, mainProcessElapsedTime.endTime, mainProcessElapsedTime.humanElapsedTime.formattedTime, isLastPage, resp.mustForceBreak)
			m_running.UpdateStepProcess(otherInfo.currentStepModule, {
				stepProcessId = stepProcessId,
				action = "end retrieving all page from allegro",
				mainProcessElapsedTime = mainProcessElapsedTime
			})
			break
		end
		requestInfo.numberPage = requestInfo.numberPage + 1
		requestInfo.last = lastRow
	end
end

-----------------------------------------
-- FULLCONTACT
-----------------------------------------

--[[

]]--
m_fullcontact.Initialize = function(self, callback)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.Initialize] Initialize fullcontact method..")
	local provisioning = require "apps.provisioning"
	provisioning.InitializeFullcontact(callback)
end

--[[

]]--
m_fullcontact.GetAllegroMigrationsApi = function(apiName)
	return Fullcontact.Dialer.GetAllegroMigrationsApi(apiName)
end

-----------------
-- FULLCONTACT - CONTACTS
----------------

m_fullcontact.contacts = {}

--[[
	funcion invocada desde el migration/oeImportContacts -> CreateNativeContacts
]]--
m_fullcontact.contacts.Migrate = function(self, vcc, allegroContacts, ksOE, ksAgent)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.contacts.Migrate] Execute fullcontact method 'Migrate'. vcc: %s, allegroContacts: %s", vcc, allegroContacts)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("Contacts"):Migrate(vcc, allegroContacts, ksOE, ksAgent)
	end)
	if not success then
		-- error
		local err = result
		return false, err
	end
	-- ok
	local contactsByBatches = result
	return true, contactsByBatches
end

--[[

]]--
m_fullcontact.contacts.Rollback = function(self, vcc, ksOE, ksAgent)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.contacts.Rollback] Execute fullcontact method 'Rollback'. vcc: %s.", vcc)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("Contacts"):Rollback(vcc, ksOE, ksAgent)
	end)
	return success, result
end

-----------------
-- FULLCONTACT - IMPORTS
----------------

m_fullcontact.imports = {}

--[[
	funcion invocada desde el migration/oeImportContacts -> CreateNativeContacts
]]--
m_fullcontact.imports.Migrate = function(self, vcc, allegroImports, ksOE)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.imports.Migrate] Execute fullcontact method 'Migrate'. vcc: %s, allegroImports: %s", vcc, allegroImports)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("Imports"):Migrate(vcc, allegroImports, ksOE)
	end)
	if not success then
		-- error
		local err = result
		return false, err
	end
	-- ok
	return true
end

--[[

]]--
m_fullcontact.imports.Rollback = function(self, vcc, ksOE)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.imports.Rollback] Execute fullcontact method 'RollbackImports'. vcc: %s.", vcc)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("Imports"):Rollback(vcc, ksOE)
	end)
	return success, result
end

-----------------
-- FULLCONTACT - IMPORT FORMATS
----------------

m_fullcontact.importFormats = {}

--[[
	funcion invocada desde el migration/oeImportContacts -> CreateNativeContacts
]]--
m_fullcontact.importFormats.Migrate = function(self, vcc, allegroImportFormats, ksOE)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.importFormats.Migrate] Execute fullcontact method 'Migrate'. vcc: %s, allegroImportFormats: %s", vcc, allegroImportFormats)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("ImportFormats"):Migrate(vcc, allegroImportFormats, ksOE)
	end)
	if not success then
		-- error
		local err = result
		return false, err
	end
	-- ok
	return true
end

--[[

]]--
m_fullcontact.importFormats.Rollback = function(self, vcc, ksOE)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.fullcontact.importFormats.Rollback] Execute fullcontact method 'RollbackImports'. vcc: %s.", vcc)
	local success, result
	m_fullcontact:Initialize(function()
		success, result = m_fullcontact.GetAllegroMigrationsApi("ImportFormats"):Rollback(vcc, ksOE)
	end)
	return success, result
end

----------------------------------------------------------------
-- PROCESS FLOWS
----------------------------------------------------------------
m_processFlows.m_cachedProcessFlows = nil

--[[

]]--
m_processFlows.LoadProcessFlows = function(self, info)
	if m_processFlows.m_cachedProcessFlows then
		return true, m_processFlows.m_cachedProcessFlows
	end
	local _, modelProcessFlows = utils.GetControllerOthersApps("flow_designer", {controllerName = "process_flows", modelName = "ProcessFlows"})
	local flows = modelProcessFlows.LoadFlowsByType(info.vccId, "CALL", info.ksAgent)
	if #flows == 0 then
		local err = {errType = "NOT CALL PROCESS FLOWS IN VCC", vcc = info.vccId}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.LoadProcessFlows] --> vcc: %s Unable to load process flows. err: '%s'.", info.vccId, json.encode(err))
		return false, err
	end
	m_processFlows.m_cachedProcessFlows = flows
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.LoadProcessFlows] --> vcc: %s Loaded %s Call Process Flows.", info.vccId, #flows)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.LoadProcessFlows] ----> vcc: %s Loaded %s Call Process Flows. Data: %s", info.vccId, #flows, utils.JSON_Encode(flows))
	-- ok
	return true, flows
end

--[[

]]--
m_processFlows.FindProcessFlowByName = function(self, flowName, info)
	assert(type(flowName) == "string")
	assert(type(info) == "table")
	-- obtengo de cassandra los flows existentes
	local success, flows = self:LoadProcessFlows(info)
	if not success then
		local err = flows
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.FindProcessFlowByName] --> Unable to find process flow name '%s'. err: '%s'.", flowName, json.encode(err))
		return false, err
	end
	-- ahora busco por nombre
	local currentFlow
	for _, flowInfo in ipairs(flows) do
		if (flowInfo.Name or "") == flowName then
			currentFlow = flowInfo
			break
		end
	end
	if not currentFlow then
		local err = {errType = "CALL PROCESS FLOW DOES NOT EXIST IN VCC", vcc = info.vccId, flowName = flowName}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.FindProcessFlowByName] --> Unable to find process flow name '%s'. err: '%s'.", flowName, json.encode(err))
		return false, err
	end
	-- ok lo encontro
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.FindProcessFlowByName] --> Call Process flow '%s' has been found.", flowName)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.processFlows.FindProcessFlowByName] --> Call Process flow '%s' - Data: %s", flowName, json.encode(currentFlow))
	return true, currentFlow
end

------------------------------------------
--
-- Exported functions
--
return {
	CanStartMigration = m_migration.CanStartMigration,
	CanStartRollback  = m_migration.CanStartRollback,
	IsRunning         = m_running.IsRunning,
	Start             = m_migration.Start,
	Rollback          = m_migration.Rollback,
	GetAllegroInfo    = m_allegro.GetAllegroInfo
}
