local utils = require "shared.apps.utils"
local m_helpers = require "helpers"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local LOG_PREFIX = "M I G R A T I O N ==> "
local m_logEachPage = 20 -- utilizado para que el log en el inConcertWebhandler.log se realice cada N paginas
--------------------------------------------------------------------------------------
local m_migration = {}
local m_migrationsSteps = {}

--------------------------------------------------------------------------------------
-- STEP OUTBOUND ENGINE - IMPORTATIONS & CONTACTS IMPORTS ------
--------------------------------------------------------------------------------------

local m_oeImportContactsStep = {
	id = "importations",
	migrationId = nil, -- es el id de la definicion del archivo "/migrations/init.lua" tabla m_migrationsConfig[0,...].id
	description = "OE Import Contacts Step Module Migration"
}

local m_dev = {
	enabled = false, -- cuando estoy desarrollando y quiero que me traiga solo N contactos en el paginado de contactos
	-- todos estos atributos se aplicaran si el atributo "enabled" se encuentra en true
	updateContacts = true,
	logContactsOfEachPage = false, -- para que loguee en el inConcertWebhandler.log cada uno de los contactos de las paginas.
	queryInfo = {
		limit           = 500, -- la cantidad de rows por pagina
		--last            = "Nombre_0059824049951",     -- para que comience en tal pagina (a partir de un ultimo registro de la pagina anterior)
		--startNumberPage = 91  -- idem al anterior pero DEPRECATED porque solo sirve para el paginado tradicional sin KEYSET
	},
	numberOfPagesToRetrieve = -1, -- -1 es igual a todas las paginas
	paginatedFunctionName = "LoadPaginatedContactsToMigrate"
	--paginatedFunctionName = "LoadPaginatedContacts"
}

--[[--------------------------
	Alamacenamiento temporal en Cassandra CF "TempVccAllegroMigrationOutProcessesBatches"
		se almacena de forma temporal los contactos que pertenecen a los batches y en que estados se encuentran.
----------------------------]]

--[[

]]--
m_tempStore = {
	-- para que almacene en HDD los ids de contactos segun los lotes a los cuales pertenece. Si esta en FALSE, los almacena en memoria (deprecated metodo, que hacia caer el webhandler)
	isEnabled = true,
	saveEachPage = 20, -- unos 10.000 contactos (saveEachPage * otherInfo.queryInfo.limit de la funcion LoadAllegroContacts)
	currentPage = 0,
	migratedContacts = {}
}

--[[

]]--
m_tempStore.GetMigratedContactsInfo = function()
	return true, m_tempStore.migratedContacts
end

--[[

]]--
m_tempStore.SetMigratedContactsInfo = function(migratedContacts)
	migratedContacts = migratedContacts or {}
	--LogDebug("m_tempStore.SetMigratedContactsInfo ---> migratedContacts: %s", json.encode(migratedContacts))
	m_tempStore.migratedContacts = migratedContacts
end

--[[

]]--
m_tempStore.CleanUp = function(info, mustCleanUpAll)
	m_tempStore.migratedContacts = {}
	-- elimino de cassandra los datos temporales.
	if mustCleanUpAll then
		utils.RemoveColumns {
			keyspace = info.ksOE,
			columnFamily = "TempVccAllegroMigrationOutProcessesBatches",
			rows = {info.vccId:lower()}
		}
	end
end

--[[

]]--
m_tempStore.Save = function(self, info, forceSave)
	if not self.isEnabled then
		return false
	end
	forceSave = forceSave and true or false

	assert(info)
	assert(info.runningModule)
	assert(info.runningModule.currentMigrationDir)
	assert(info.runningModule.tempStoredKey)

	if not info.migratedContactsByBatches then
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Save] No contacts batches information..")
		return false
	end

	local currentPage = self.currentPage
	if not forceSave then
		self.currentPage = self.currentPage + 1
		if self.currentPage < self.saveEachPage then
			return false
		end
		self.currentPage = 0
	end

	local tempStoredKey = info.runningModule.tempStoredKey -- "2023_07_31_17_20_59_id_aa3de5ede2264f14b7b9cdbf22aca510"
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Save] --> Saving contacts info in STORE. currentPage %d of %d Pages. tempStoredKey: %s", currentPage, self.saveEachPage, tempStoredKey)

	for processBatchId, batchInfo in pairs(info.migratedContactsByBatches) do
		local tempStoredKeyProcessId = ("%s_process:%s"):format(tempStoredKey, batchInfo.ProcessId):lower()
		local tempStoredKeyProcessIdAndBatchId = ("%s_batch:%s"):format(tempStoredKeyProcessId, batchInfo.BatchId):lower()
		--
		local processId = batchInfo.ProcessId:lower()
		local processInfo = self:FindProcess(processId)
		local batchNumber = processInfo and #(processInfo.batches or {}) or 0
		batchNumber = tostring(batchNumber + 1)
		local fullKey = ("%s_number:%s"):format(tempStoredKeyProcessIdAndBatchId, batchNumber):lower()
		-- almaceno en cassandra los datos temporales.
		utils.SaveRows {
			keyspace = info.ksOE,
			columnFamily = "TempVccAllegroMigrationOutProcessesBatches",
			rows = {
				[info.vccId:lower()] = {
					[fullKey] = json.encode(batchInfo)
				}
			}
		}
		if not processInfo then
			processInfo = {
				processId = processId,
				batches   = {}
			}
		end
		table.insert(processInfo.batches, {
			batchId     = batchInfo.BatchId:lower(),
			batchNumber = batchNumber,
			fullKey     = fullKey
		})
		-- update
		self:UpdateProcess(processId, processInfo)
	end
	info.migratedContactsByBatches = nil
	return true
end

--[[

]]--
m_tempStore.FindProcess = function(self, processId)
	if not self.migratedContacts[processId] then
		return
	end
	return self.migratedContacts[processId]
end

--[[

]]--
m_tempStore.UpdateProcess = function(self, processId, savedInfo)
	self.migratedContacts = self.migratedContacts or {}
	self.migratedContacts[processId] = savedInfo
end

--[[

]]--
m_oeImportContactsStep.LoadAllegroContacts = function(info, callback)
	local moduleName = "outboundengine_contacts"
	local moduleInstance = utils.GetControllerOthersApps("allegro", {controllerName = moduleName, modelName = nil})
	local otherInfo = {
		listName              = "CONTACTS",
		callback              = callback,
		moduleInstance        = moduleInstance,
		moduleName            = moduleName,
		currentStepModule     = m_oeImportContactsStep,
		paginatedFunctionName = "LoadPaginatedContactsToMigrate", -- (MAS RAPIDO) NO utiliza ORM en Allegro
		--paginatedFunctionName = "LoadPaginatedContacts",        -- (MAS LENTO ) SI utiliza ORM en Allegro
		queryInfo             = {
			last       = "",
			limit      = 500,
			numberPage = 1,
			query      = info.contactsMigration -- aca va el query que se ejecutara contra Allegro (select in o select not in) -- ALL CONTACTS OF VCC, CONTACTS OF SOME PROCESS BATCHES, 
		},
		logEachPage = m_logEachPage
	}
	-- cuando estoy desarrollando solo me debe de traer N contactos y cortar el paginado
	if m_dev.enabled then
		otherInfo.paginatedFunctionName     = m_dev.paginatedFunctionName
		otherInfo.queryInfo.limit           = m_dev.queryInfo.limit
		otherInfo.queryInfo.last            = m_dev.queryInfo.last
		otherInfo.queryInfo.startNumberPage = m_dev.queryInfo.startNumberPage
	else
		otherInfo.queryInfo.startNumberPage = info.startNumberPage or otherInfo.queryInfo.numberPage
		otherInfo.queryInfo.last            = info.lastRow or otherInfo.queryInfo.last
	end
	-- datos utilizados si el webhandler se cae/stop, cuando se ejecuta una migracion en modo continue.
	local stepProcess = {
		name     = "CONTACTS PAGINATION",
		fromPage = otherInfo.queryInfo.startNumberPage or otherInfo.queryInfo.numberPage,              -- ej: (la primera pagina - desde el row 1)
		endPage  =  m_logEachPage, -- ej: (la primera pagina - hasta el row 20)
		lastRow  = otherInfo.queryInfo.last
	}
	info.runningModule:UpdateLastMigrationStep2InVcc(info, stepProcess)
	--
	info.allegroMigrationModule:LoadAllRowsByPaging(info, otherInfo)
end

--[[
	funcion que se encarga de saber si puede o no comenzar la migracion de contactos STEP2.
	en una ejecucion normal, siempre se puede.
	en una ejecucion por caida/stop del Webhandler, debe realizar la validacion de que este punto de migracion ya no se haya ejecutado por completo, antes de la caida.
]]--
m_oeImportContactsStep.CanStartMigration = function(info)
	assert(type(info.contactsMigration.executionType) == "string")
	local executionType = info.contactsMigration.executionType:upper()
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CanStartMigration] Validate Start STEP CONTACTS Migration. VCC: '%s', executionType: '%s'.", info.vccId, executionType)

	if executionType == "NORMAL" then
		return "MIGRATE CONTACTS & ASSIGN CONTACTS TO BATCHES"
	elseif executionType == "CONTINUE AND REPAIR FAILED MIGRATION" then
		local lastMigration = info.occVccInfo.AllegroIntegration.LastMigration
		assert(lastMigration)
		assert(lastMigration.currentStepProcess)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CanStartMigration] Validate Start STEP CONTACTS Migration. VCC: '%s', currentStepProcess: '%s'.", info.vccId, json.encode(lastMigration.currentStepProcess or {}))
		local continueInStepProcess = lastMigration.currentStepProcess.name:upper()
		if continueInStepProcess == "CONTACTS PAGINATION" then
			info.startNumberPage = lastMigration.currentStepProcess.fromPage or 1
			info.lastRow = lastMigration.currentStepProcess.lastRow or 1
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CanStartMigration] Contacts step will be run. The recovery will be run from page '%s' for VCC: '%s'.", info.startNumberPage, info.vccId)
			return "MIGRATE CONTACTS & ASSIGN CONTACTS TO BATCHES"
		elseif continueInStepProcess == "ASSIGN CONTACTS TO BATCHES" then
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CanStartMigration] Assign Contacts to Batches step will be run. Recovery will be run for VCC: '%s'", info.startNumberPage, info.vccId)
			return "ASSIGN CONTACTS TO BATCHES"
		end
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CanStartMigration] SKIP Step Contacts. Recovery should continue in '%s' for VCC: '%s'.", continueInStepProcess, info.vccId, executionType)
		return false
	end
	LogError("Unrecognized executionType: " .. executionType)
	return false
end

--[[
	Al salir de la funcion en el parametro "info" se ha inyectado la tabla "migratedContactsByBatches".
	La misma contiene informacion de los contactos y los batches, que sera utilizada en la migracion de OE Process y Batches (archivo oeProcesses)
]]--
m_oeImportContactsStep.Start = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Start] Start Migration OE Contacts And Importations for vcc '%s'.", info.vccId)

	-----------------------------------------------------------------------
	-- A - Migracion de las importaciones (linea, header, detalle)
	local resultsList = {}
	-- 0.5 realizo validaciones para determinar si puede migrar el step (ejemplo si es una ejecucion por falla)
	local success = m_oeImportContactsStep.CanStartMigration(info)
	if not success then
		return false, resultsList
	end
	-- 1 - creo el step de running para las importaciones
	local runningInfo = {
		stepProcessId = "migrate oe importations"
	}
	info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)

	-- NORMAL o "CONTINUE AND REPAIR FAILED MIGRATION"
	if success == "MIGRATE CONTACTS & ASSIGN CONTACTS TO BATCHES" then
		local executionType = info.contactsMigration and info.contactsMigration.executionType:upper() or "NORMAL"
		-- 2 - obtengo las importaciones desde allegro (las lineas de detalle)
		--      para el caso de una ejecucion por una caida/stop del webhandler con la acccion CONTINUE, las lineas de detalle no las tiene que traer ya se han migrado.
		if not m_dev.enabled and executionType == "NORMAL" then
			local success, importations = m_oeImportContactsStep.LoadAllegroImportations(info)
			if not success then
				-- 2.1 error, debo actualizar el step de running
				runningInfo.success = false
				runningInfo.errType = "UNABLE TO LOAD OE IMPORTATIONS"
				runningInfo.err = importations
				info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
				return false
			end
			-- 3 - actualizo el step de running antes de comenzar la migracion de las importaciones
			local allegroImports = importations.rows or {}
			runningInfo.total = #allegroImports
			info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
			-- 4 - comienzo la migracion de las importaciones
			local success, result = m_oeImportContactsStep.UpdateImportations(allegroImports, info)
			if success then
				-- 4.1 OK, debo actualizar el step de running
				runningInfo.success = true
				runningInfo.result = result
			else
				-- 4.2 error, debo actualizar el step de running
				runningInfo.success = false
				runningInfo.errType = "UNABLE TO CREATE IMPORTATIONS"
				runningInfo.err = result
			end
			-- 5 - actualizo el proceso de running
			info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
		end
	
		-----------------------------------------------------------------------
		--B - Migracion de los contactos de la importacion
		local allegroOEContactsController = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_contacts", modelName = nil})
		local execGarbageCollectorEveryPages = m_tempStore.saveEachPage -- (execGarbageCollectorEveryPages x 500 contactos por pagina cada 10.000 contactos)
		local execGarbageCollectorInPageNumber = execGarbageCollectorEveryPages
	
		local _, endTimeGroupPage
		local invokePageNumber = 0
		local totalRowsInPageGroup = 0
		local allPagesElapsedTimeSec = 0
	
		--[[
			Este callback recibe los N contactos de la pagina actual de Allegro (con todos los datos de las tablas relacionadas).
			Por ultimo los procesa invocando los metodos pertinentes de la Rocka "fullcontact" para que esta cree los contactos segun el template y los almacene en ES.
		]]--
		local function CallbackOnLoadedPage (success, isLastPage, rows, paginatedStepProcessId, pageNumber, lastRow)
			rows = rows or {}
			local totalRows = #rows
			totalRowsInPageGroup = totalRowsInPageGroup + totalRows
	
			-- la primera pagina
			-- if not endTimeGroupPage then
			-- 	_, endTimeGroupPage = m_helpers.utils.ElapsedTime.Start()
			-- end
	
			-- LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Start] CallbackOnLoadedPage vcc '%s', success: '%s', isLastPage: '%s', pageNumber: %s', lastRow: '%s', totalRows: '%s' paginatedStepProcessId '%s'. Garbage collector exec in page: %d",
			-- 	info.vccId, success, isLastPage, pageNumber, lastRow, totalRows, paginatedStepProcessId, execGarbageCollectorInPageNumber)
	
			if execGarbageCollectorInPageNumber == pageNumber then
				utils.CollectAllMemory()
				execGarbageCollectorInPageNumber = execGarbageCollectorInPageNumber + execGarbageCollectorEveryPages
			end
	
			-- 1 - definicion del proceso running
			local runningInfo = {
				stepProcessId = "migrate allegro contacts to native (ES)"
			}
			--
			if not success then
				-- 1.1 error, debo actualizar el step de running
				runningInfo.success = false
				runningInfo.errType = "UNABLE TO LOAD PAGE CONTACTS IMPORTS"
				runningInfo.err = isLastPage
				info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
				--
				return {
					success = false,
					mustForceBreak = false
				}
			end
			-- DEBUG DEV- logs de cada row
			if m_dev.enabled and m_dev.logContactsOfEachPage then
				for i, contactRow in ipairs(rows or {}) do
					--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Start]       - %d of page %d =====> contactRow.ID '%s'.", i, pageNumber, contactRow.ID)
					LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Start]       - %d of page %d =====> contactRow: '%s'.", i, pageNumber, json.encode(contactRow))
				end
			end
	
			-- 2 - ACA actualizo contra la FullContact->ES y creo el step de running para la importacion de contactos (la pagina)
			if not m_dev.enabled or (m_dev.enabled and m_dev.updateContacts) then
				runningInfo.total = totalRows
	
				--------------------------------------------------------
				-- 3 - comienzo la migracion de la pagina de contactos.
				local _, startPageEndProcessElapsedTime = m_helpers.utils.ElapsedTime.Start()
				if not endTimeGroupPage then
					endTimeGroupPage = startPageEndProcessElapsedTime
				end
	
				local success, result = m_oeImportContactsStep.UpdateContacts(rows, info)
	
				local _, endPageEndProcessElapsedTime = m_helpers.utils.ElapsedTime.End(startPageEndProcessElapsedTime.processId)
				endTimeGroupPage.elapsedTimeSec = (endTimeGroupPage.elapsedTimeSec or 0) + endPageEndProcessElapsedTime.elapsedTimeSec
				--------------------------------------------------------
	
				invokePageNumber = invokePageNumber + 1
				if invokePageNumber == m_logEachPage then
					-- Actualizo en Cassandra - VCC.AllegroIntegration.LastMigration la ultima pagina procesada, obtenida desde Allegro e insertada en ES.
					local stepProcess = {
						name     = "CONTACTS PAGINATION",
						fromPage = pageNumber + 1,               -- ej: (la segunda pagina - desde el row 11)
						endPage  = (pageNumber + m_logEachPage), -- ej: (la segunda pagina - hasta el row 20)
						lastRow  = lastRow
					}
					info.runningModule:UpdateLastMigrationStep2InVcc(info, stepProcess)
					-----
					local endPage = pageNumber
					local fromPage = (endPage - m_logEachPage) + 1
					local elapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(endTimeGroupPage.elapsedTimeSec)
					local averageTimeSecPerPage = m_logEachPage == 1 and "" or string.format("- Average TIME per page: %s", endTimeGroupPage.elapsedTimeSec / m_logEachPage)
	
					allPagesElapsedTimeSec = allPagesElapsedTimeSec + endTimeGroupPage.elapsedTimeSec
					local allPagesElapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(allPagesElapsedTimeSec)
	
					LogInfo(LOG_PREFIX .. "[...Start.CallbackOnLoadedPage] (OCC)    --> ALL PAGES TIME: %s -> DETAIL: fromPage: '%s' - endPage: '%s'-> %s totalRows (lastRow: %s)       -> TIME: %s %s.\nUpdated in ES vcc: %s -  Garbage collector executed in page: %s",
						allPagesElapsedTimeDetail.formattedTime,
						fromPage,
						endPage,
						totalRowsInPageGroup,
						lastRow,
						elapsedTimeDetail.formattedTime,
						averageTimeSecPerPage,
						info.vccId,
						execGarbageCollectorInPageNumber
					)
					runningInfo.pagedInfo = {
						fromPage  = fromPage,
						endPage   = endPage,
						totalRows = totalRowsInPageGroup
					}
					info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
	
					totalRowsInPageGroup = 0
					invokePageNumber = 0
					endTimeGroupPage = nil
				end
			end
			-- cuando estoy desarrollando y solo me trae N contactos, corto de forma forzada el paginado
			if m_dev.enabled and m_dev.numberOfPagesToRetrieve == pageNumber then
				return {
					success = true,
					mustForceBreak = true
				}
			end
			-- ok
			return {
				success = true,
				mustForceBreak = false
			}
		end
		-- 3 - ACA comienza la obtencion de los contactos por paginas de N contactos contra el SQL de Allegro.
		--		En el CallbackOnLoadedPage se procesan los N contactos que se han obtenidos y se insertan en ES invocando a las funciones pertinentes de la fullcontact.
		m_oeImportContactsStep.LoadAllegroContacts(info, CallbackOnLoadedPage)
		--
		m_tempStore:Save(info, true) -- cuando ha termiando de obtener todas las paginas, tengo que almacenar la ultima en HDD, para su utilizacion cuando se asignan los contactos a los procesos del OE
	end
	-- limpio la memoria
	utils.CollectAllMemory()
	-- 4 - Asigno contactos a los procesos del OE
	if success == "ASSIGN CONTACTS TO BATCHES" or success == "MIGRATE CONTACTS & ASSIGN CONTACTS TO BATCHES" then
		m_oeImportContactsStep.AssignContactsToBatches(info, resultsList)
	end
	-- ok
	return true, resultsList
end

--[[
	Funcion invocada por cada batch/pagina de contactos de occ.
]]--
m_oeImportContactsStep.UpdateContacts = function(allegroContacts, info)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.UpdateContacts] Adding Native OE contacts from Allegro Contacts: '%s', vcc: '%s'.", #allegroContacts, info.vccId)
	-- 1- dfafdadfadfadfadfasdfa
	local success, result = m_oeImportContactsStep.CreateNativeContacts(allegroContacts, info)
	if not success then
		local err = result
		return false, err
	end
	return true, {}
end

--[[
	En info.migratedContactsByBatches retorna los contactos que se asignaran al batch correspondiente cuando se crea el Batch ejecutando el metodo CreateBatch del OEClient.
		Esto se realiza por cada pagina resultante del paginado de contactos de Allegro.
		Se invoca a esta funcion "CreateNativeContacts" y los contactos que se incluiran al crear, pausar, finalizar y cancelar los batches, se ira acumulando en "migratedContactsByBatches"
]]--
m_oeImportContactsStep.CreateNativeContacts = function(allegroContacts, info)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CreateNativeContacts] Creating Native Contacts: '%s', vcc: '%s'.", #allegroContacts, info.vccId)
	local success, result = info.fullcontactMigrationModule.contacts:Migrate(info.vccId, allegroContacts, info.ksOE, info.ksAgent)
	if not success then
		local err = result
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CreateNativeContacts] Error creating Contacts. Err: %s", json.encode(err))
		return false, err
	end
	-- ok
	--[[
		Ahora debemos agregar los contactos que seran asignados al lote cuando se cree.
		Tambien debemos agregar los contactos por estado (pause, canceled, finished), asi luego se procedera a cambiar los estados de los batches/contactos.
	]]--

	local contactsByBatches = result
	info.migratedContactsByBatches = info.migratedContactsByBatches or {}
	for processBatchId, batchInfo in pairs(contactsByBatches) do
		if not info.migratedContactsByBatches[processBatchId] then
			-- "CreateBatch": Aca agregamos  todos los contactos cuando se creara el batch. Si el batch no existe en la lista.
			info.migratedContactsByBatches[processBatchId] = batchInfo
		else
			-- "CreateBatch": aca agregamos los contactos 1 a 1 si es que no existen.
			for contactId, contactInfo in pairs(batchInfo.Contacts) do
				if not info.migratedContactsByBatches[processBatchId].Contacts[contactId] then
					info.migratedContactsByBatches[processBatchId].Contacts[contactId] = contactInfo
				end
			end
			-- "PauseContacts, FinishContacts, CancelContacts"
			for contactState, contactsByState in pairs(batchInfo.ContactsByStates) do
				if not info.migratedContactsByBatches[processBatchId].ContactsByStates[contactState] then
					info.migratedContactsByBatches[processBatchId].ContactsByStates[contactState] = contactsByState
				else
					for contactId, contactInfo in pairs(contactsByState.Contacts) do
						if not info.migratedContactsByBatches[processBatchId].ContactsByStates[contactState].Contacts[contactId] then
							info.migratedContactsByBatches[processBatchId].ContactsByStates[contactState].Contacts[contactId] = contactInfo
						end
					end
				end
			end
		end
	end
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CreateNativeContacts] Success created.")
	-- LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.CreateNativeContacts] Success created. ----> migratedContactsByBatches: %s", json.encode(info.migratedContactsByBatches))

	--
	m_tempStore:Save(info)

	return true
end

--[[
	
]]--
m_oeImportContactsStep.AssignContactsToBatches = function(info, results)
	local oeProcessesModule = info.GetStepByModule("oeProcesses", "processes")
	local _, bacthesInfo = oeProcessesModule.AssignContactsToBatches(info, results)
end

----------------------------
-- Rollback
----------------------------

--[[

]]--
m_oeImportContactsStep.Rollback = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Rollback] Rollback Migration OE Import Contacts for vcc '%s'.", info.vccId)
	-- A - realizo rollback de todos los contactos
	--
	-- 1 - creo el step de running
	local runningInfo = {
		stepProcessId = "rollback contacts"
	}
	info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
	-- 2 - realizo el rollback de los contactos
	local success, result = info.fullcontactMigrationModule.contacts:Rollback(info.vccId, info.ksOE, info.ksAgent)
	if not success then
		-- 2.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO ROLLBACK CONTACTS"
		runningInfo.err = result
		info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
	end
	--
	-- B - realizo rollback de las importaciones (header, detalle, etc)
	--
	-- 3 - creo el step de running
	local runningInfo = {
		stepProcessId = "rollback importations"
	}
	info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
	-- 4 - realizo el rollback de las importaciones
	local success, result = info.fullcontactMigrationModule.imports:Rollback(info.vccId, info.ksOE)
	if not success then
		-- 2.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO ROLLBACK IMPORTATIONS"
		runningInfo.err = result
		info.runningModule.UpdateStepProcess(m_oeImportContactsStep, runningInfo)
	end
	-- ok
	return true
end

--
table.insert(m_migrationsSteps, m_oeImportContactsStep)  -- agrego step al array

--------------------------------------------------------------------------------------
-- IMPORTACIONES (LINEAS HEADER Y DETALLE)
--------------------------------------------------------------------------------------
--[[

]]--
m_oeImportContactsStep.LoadAllegroImportations = function(info)
	local moduleAllegroImport = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_imports", modelName = nil})
	local requestInfo = {
		vcc = info.vccId,
		last = "",
limit = 1000, -- TODO: paginar
		numberPage = 1,
		query = {ID = "", ["ContactImportDetail.Date"] = "", Status = "", ["ContactImportDetail.ImportUser"] = ""},
		columns = {"DisplayStatus","ID","DBProvider","Updates"}
	}
	local success, data = moduleAllegroImport.LoadPaginatedImports(json.encode(requestInfo))
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.LoadAllegroImportations] --> vcc: %s Allegro OE Importations request. success: '%s'.", info.vccId, success)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.LoadAllegroImportations] --> vcc: %s Allegro OE Importations request. data: '%s'.", info.vccId, utils.JSON_Encode(data))
	if not success then
		local err = data
		return false, err
	end
	-- ok
	return true, data
end

--[[
]]--
m_oeImportContactsStep.UpdateImportations = function(allegroImports, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.UpdateImportations] Adding Native OE imports from Allegro Imports: '%s', vcc: '%s'.", #allegroImports, info.vccId)
	-- 1-
	local success, result = info.fullcontactMigrationModule.imports:Migrate(info.vccId, allegroImports, info.ksOE)
	if not success then
		local err = result
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.UpdateImportations] Error creating Native Imports. Err: %s", json.encode(err))
		return false, err
	end
	local result = {}
	return true, result
end

--[[
]]--
m_oeImportContactsStep.GetMigratedContactsInfo = function()
	return m_tempStore.GetMigratedContactsInfo()
end

--[[
]]--
m_oeImportContactsStep.SetMigratedContactsInfo = function(migratedContacts)
	return m_tempStore.SetMigratedContactsInfo(migratedContacts)
end

--[[

]]--
m_oeImportContactsStep.CleanUpStore = function(info, mustCleanUpAll)
	m_tempStore.CleanUp(info, mustCleanUpAll)
end

--[[

]]--
m_migration.GetSteps = function()
	return m_migrationsSteps
end

--[[

]]--
m_migration.Init = function(config)
	for _, step in ipairs(m_migration.GetSteps()) do
		step.migrationId = config.id
	end
end

------------------------------------------
--
-- Exported functions
--
return {
	Init                    = m_migration.Init,
	GetSteps                = m_migration.GetSteps,
	GetMigratedContactsInfo = m_oeImportContactsStep.GetMigratedContactsInfo,
	SetMigratedContactsInfo = m_oeImportContactsStep.SetMigratedContactsInfo,
	CleanUpStore            = m_oeImportContactsStep.CleanUpStore
}
