local utils = require "shared.apps.utils"
local modelPackages = require("apps.provisioning.models.packages")
local m_permissions = require "shared.permissions.permissions"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

local function Render (web)
	_M.response.DontCache(web)
	local env = {_T = m_htmlRender.extension.makeTextTranslator()}
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING:Render] Render form PACKAGES with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local htmlFile = m_appLocation .. "/pages/" .. "frm_package_content.html"

	local htmlContent = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not htmlContent then
		assert(htmlContent, "[PROVISIONING:Render] HTML PACKAGES does not exist: %s", htmlFile)
	end

	return htmlContent
end

local function ValidateAndMakeInfoToUpdate (packageId, packageInfo)
	assert(packageId)
	assert(packageInfo)

	LogInfo("[PROVISIONING.PACKAGES:ValidateAndMakeInfoToUpdate] Validating PACKAGE '%s' info to updated. Info %s", packageId, packageInfo)

	local exist, currentPackageInfo = modelPackages.Get(packageId)

	-- 1 - clean up valores que llegan del lado del cliente
	local isNew = packageInfo.isNew

	if exist then
		-- Si estoy creando el PACKAGE y el mismo ya exsite en la BD, fallo!!
		if isNew then
			return false, "MUST_HAVE_UNIQUE_NAME"
		end
	end

	-- atributos que NO se deben almacenar en la CF "Packages"
	packageInfo.isNew = nil
	packageInfo.VCC = nil -- El vcc se alamcena desde el ABM de VCCs.

	return true, packageInfo
end

local function SavePackage (web)
	local postedData = web.input["post_data"]
	local success, did_info = pcall(json.decode, postedData)

	LogInfo("[PROVISIONING.PACKAGES:SavePackage] Saving PACKAGE. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, rows = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.PACKAGES:SavePackage] Error Updating PACKAGE. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local rowId, rowInfo = next(rows)
	--LogDebug("--> rowId %s", rowId)
	--LogDebug("--> rowInfo %s", rowInfo)

	local success, rowInfo = ValidateAndMakeInfoToUpdate(rowId, rowInfo)
	if not success then
		local errMsg = rowInfo
		LogError("[PROVISIONING.PACKAGES:SavePackage] Error Validating PACKAGE '%s'. Err: %s", rowId, errMsg)
		web.status = 500
		result.result = errMsg
		return result
	end

	local success, err = modelPackages.Save(rowId, rowInfo)
	if not success then
		LogError("[PROVISIONING.PACKAGES:SavePackage] Error Updating PACKAGE '%s'. Err: %s", rowId, err)
		web.status = 500
		result.result = err.code -- Este código de error llega desde la Rocka "provisioningmanager"
		result.rawData = err.rawData
		return result
	end

	LogInfo("[PROVISIONING.PACKAGES:SavePackage] '%s' saved successfully.", rowId)

	web.status = 200
	result.result = "OK"
	return result
end

local function DeletePackage (web)
	local postedData = web.input["post_data"]
	local success, trunkInfo = pcall(json.decode, postedData)

	LogInfo("[PROVISIONING.PACKAGES:DeletePackage] Deleting Package. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, packages = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.PACKAGES:DeletePackage] Error Deleting Package. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local packageId = packages[1]
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")

	local success, vccInfo = modelPackages.Get(packageId, ksProv)
	if not success then
		ksProv:Close()
		LogError("[PROVISIONING.PACKAGES:DeletePackage] Error Deleting Package %s. Not Found.", packageId)
		web.status = 404
		result.result = "Package DOES NOT EXIST"
		return result
	end

	local response = modelPackages.Delete(packageId, ksProv)
	if not response.success then
		LogError("[PROVISIONING.PACKAGES:DeletePackage] Error Deleting Package %s. Error: %s.", packageId, response.errMsg or "")
		web.status = 400
		result.result = response.errType -- Este código de error llega desde la Rocka "provisioningmanager"
		result.relatedId = response.relatedId
		return result
	end

	LogInfo("[PROVISIONING.PACKAGES:DeletePackage] Package '%s' deleting successfully.", packageId)

	ksProv:Close()
	web.status = 200
	result.result = "OK"

	return result
end

local function GetPackage (web, packageId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.PACKAGES:GetPackage] Retrieving Package '%s'. User %s.", packageId, userId)

	local success, packageInfo = modelPackages.Get(packageId)
	if not success or not packageInfo then
		LogError("[PROVISIONING.PACKAGES:GetPackage] Error Retrieving Package %s. Not Found.", packageId)
		web.status = 404
		local result = {}
		result.result = "NOT FOUND"
		return result
	end

	web.status = 200
	return packageInfo
end

local function Search (web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	local postData = web.request
	local startTime = os.time()

	LogInfo("[PROVISIONING.PACKAGES:Search] 1 - (START) Loading Packages. Posted Info: '%s'", encodedInfo)

	local keyspace = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, packages = modelPackages.ListAll(keyspace)

	if not success then
		keyspace:Close()
		local err = packages
		LogWarning("[PROVISIONING.PACKAGES:Search] Error loading Packages. Err'%s'.", err)
		web.status = 500
		return {result = "SERVER ERROR", err = err}
	end

	--packages = utils.FilterEntities(packages, {IsSystemVCC = "0"})

	local totalUnfilteredRows = utils.GetRowCount(packages)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[PROVISIONING.PACKAGES:SEARCH] 2 - Filtering rows")

	for id, parckage in pairs(packages) do
		local parckageId = utils.GetId(id)
		parckage.Id = parckageId
		parckage.Name = parckage.Name or parckageId
		if utils.MatchFilter(parckage, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, parckage)
		end
	end

	keyspace:Close()

	LogInfo("[PROVISIONING.PACKAGES:SEARCH] 3 - Sorting rows")
	--include only rows that belong to logged user packages
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[PROVISIONING.PACKAGES:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. m_permissions:CheckPermission("provisioning", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- Package i6
	module:dispatch_post(fn .. Search, p .. "/search/")
	module:dispatch_get (HasPermission(GetPackage   , "view_packages"     ), p .. "/get/(.+)"   )
	module:dispatch_post(HasPermission(DeletePackage, "delete_package", fn), p .. "/delete/(.*)")
	module:dispatch_post(HasPermission(SavePackage  , "save_package", fn  ), p .. "/save/"      )


	m_htmlRender = htmlRender
	m_appLocation = appLocation
end