local utils = require "shared.apps.utils"
local modelPeers = require("apps.provisioning.models.peers")
local m_permissions = require "shared.permissions.permissions"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

local function SeparatePeer (addressId)
	assert(type(addressId) == "string")
	local peerId, tech
	local separatorPosition = addressId:find("/")
	if separatorPosition then
		tech = addressId:sub(1, separatorPosition - 1)
		peerId = addressId:sub(separatorPosition + 1)
	end
	--LogDebug("[PROVISIONING.PEERS:SeparatePeer] Separed Info - >tech: '%s', peerId: '%s'", tech, peerId)
	return tech, peerId
end

local function Render (web)
	_M.response.DontCache(web)
	local env = {_T = m_htmlRender.extension.makeTextTranslator()}
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.PEERS:Render] Render form PEER with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local htmlFile = m_appLocation .. "/pages/" .. "frm_peer_content.html"

	local htmlContent = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not htmlContent then
		assert(htmlContent, "[PROVISIONING.PEERS:Render] HTML PEER does not exist: %s", htmlFile)
	end

	return htmlContent
end

local function ValidateAndMakeInfoToUpdate (id, info)
	LogInfo("[PROVISIONING.PEERS:ValidateAndMakeInfoToUpdate] Validating PEERS '%s' info to updated. Info %s", id, info)

	if id then
		local exist, currentInfo = modelPeers.Get(id)
		-- 1 - clean up valores que llegan del lado del cliente
		local isNew = info.isNew

		if exist then
			-- Si estoy creando la Entidad y el mismo ya exsite en la BD, fallo!!
			if isNew then
				return false, "MUST_HAVE_UNIQUE_NAME"
			end
		end
	end
	-- host id
	if not info.hostId or info.hostId == "" then
		return false, "HOST_ID_IS_REQUIRED"
	end
	-- password
	if not info.password or info.password == "" then
		return false, "PASSWORD_IS_REQUIRED"
	end

	info.address_id = nil
 	info.isNew = nil

	return true, info
end

-- function ImportAllegroPeer (allegroPeer)
-- 	LogInfo("[PROVISIONING.PEERS:ImportAllegroPeer] Importing allegroPeer: '%s'.", utils.JSON_Encode(allegroPeer))
-- 	local peerId = allegroPeer.PeerName
-- 	local success, rowInfo = ValidateAndMakeInfoToUpdate(peerId, allegroPeer)
-- 	local err = {}
-- 	if not success then
-- 		local errMsg = rowInfo
-- 		LogError("[PROVISIONING.PEERS:ImportAllegroPeer] Error Validating Peer '%s'. Err: %s", peerId, errMsg)
-- 		err.result = errMsg
-- 		return false, err
-- 	end
-- 	local id = rowInfo.name
-- 	local success, resp = modelPeers.Save(peerId, nil, rowInfo)
-- 	if not success then
-- 		LogError("[PROVISIONING.PEERS:ImportAllegroPeer] Error Updating Peer '%s'. Err: %s", rowId, resp)
-- 		err.result = resp.code -- Este código de error llega desde la Rocka "provisioningmanager"
-- 		err.rawData = resp.rawData
-- 		return false, result
-- 	end
-- 	-- ok
-- 	return true
-- end

local function SavePeer (web)
	local postedData = web.input["post_data"]

	LogInfo("[PROVISIONING.PEERS:SavePeer] Saving Peer. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, rows = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.PEERS:SavePeer] Error Updating Peer. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local rowId, rowInfo = next(rows)
	-- LogDebug("--> rowId %s", rowId)
	-- LogDebug("--> rowInfo %s", rowInfo)
	local tech, peerId = SeparatePeer(rowId)

	local success, rowInfo = ValidateAndMakeInfoToUpdate(peerId, rowInfo)
	if not success then
		local errMsg = rowInfo
		LogError("[PROVISIONING.PEERS:SavePeer] Error Validating Peer '%s'. Err: %s", rowId, errMsg)
		web.status = 500
		result.result = errMsg
		return result
	end

	local id = rowInfo.name
	local success, err = modelPeers.Save(id, nil, rowInfo)
	if not success then
		LogError("[PROVISIONING.PEERS:SavePeer] Error Updating Peer '%s'. Err: %s", rowId, err)
		web.status = 500
		result.result = err.code -- Este código de error llega desde la Rocka "provisioningmanager"
		result.rawData = err.rawData
		return result
	end

	LogInfo("[PROVISIONING.PEERS:SavePeer] '%s' saved successfully.", rowId)

	web.status = 200
	result.result = "OK"
	return result
end

local function ValidateAndMakeRangeInfo (info)
	assert(info)

	LogInfo("[PROVISIONING.PEERS:ValidateAndMakeRangeInfo] Validating Range PEERS info to updated. Info %s", info)

	-- Range validation -------
	local rangeInfo = info.range
	if type(rangeInfo) ~= "table" then
		return false, "RANGE_DOES_NOT_EXIST"
	end
	rangeInfo.peer_start = tonumber(rangeInfo.peer_start)
	if not rangeInfo.peer_start then
		return false, "RANGE_START_IS_REQUIRED"
	end
	rangeInfo.peer_count = tonumber(rangeInfo.peer_count)
	if not rangeInfo.peer_count then
		return false, "RANGE_TOTAL_IS_REQUIRED"
	end

	-- Peer validation -------
	local peer = info.peer
	local success, peerInfo = ValidateAndMakeInfoToUpdate(nil, peer)
	if not success then
		local err = peerInfo
		return false, err
	end

	-- Ok
	return true, peerInfo, rangeInfo
end

local function ExecuteRangePeers (web, action)
	local postedData = web.input["post_data"]

	LogInfo("[PROVISIONING.PEERS:ExecuteRangePeers] Executing Peer Range. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, rowInfo = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.PEERS:ExecuteRangePeers] Error Executing Peer Range. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	--LogDebug("--> rowInfo %s", rowInfo)

	local success, peerInfo, rangeInfo = ValidateAndMakeRangeInfo(rowInfo)
	if not success then
		local errMsg = peerInfo
		LogError("[PROVISIONING.PEERS:ExecuteRangePeers] Error Validating AddiExecutingng Peer Range. Err: %s", errMsg)
		web.status = 500
		result.result = errMsg
		return result
	end

	local id = peerInfo.name
	local success, err = modelPeers.ExecuteRangePeers(id, nil, peerInfo, rangeInfo, action)
	if not success then
		LogError("[PROVISIONING.PEERS:ExecuteRangePeers] Error Executing Peer Range. Err: %s", err)
		web.status = 500
		result.result = err.code -- Este código de error llega desde la Rocka "provisioningmanager"
		result.rawData = err.rawData
		return result
	end

	-- ok o partial --
	local resp = err
	result.code = resp.code
	result.rawData = resp.rawData
	result.result = "OK"

	web.status = 200
	return result
end

local function AddPeerRange (web)
	LogInfo("[PROVISIONING.PEERS:AddPeerRange] Adding Peer Range. User %s@%s.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)
	local result = ExecuteRangePeers(web, "ADD PEERS RANGE")
	if result.result == "OK" then
		if result.code == "ALL PEERS ADDED" then
			LogInfo("[PROVISIONING.PEERS:AddPeerRange] All Peers Successfully Added.")
		else
			LogWarning("[PROVISIONING.PEERS:AddPeerRange] Not all Peers have been Added. Err: %s", json.encode(result))
		end
	end
	return result
end

local function UpdatePeerRange (web)
	LogInfo("[PROVISIONING.PEERS:UpdatePeerRange] Updating Peer Range. User %s@%s.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)
	local result = ExecuteRangePeers(web, "UPDATE PEERS RANGE")
	if result.result == "OK" then
		if result.code == "ALL PEERS UPDATED" then
			LogInfo("[PROVISIONING.PEERS:UpdatePeerRange] All Peers Successfully Updated.")
		else
			LogWarning("[PROVISIONING.PEERS:UpdatePeerRange] Not all Peers have been Updated. Err: %s", json.encode(result))
		end
	end
	return result
end

local function DeletePeer (web)
	local postedData = web.input["post_data"]
	local success, peerInfo = pcall(json.decode, postedData)

	LogInfo("[PROVISIONING.PEERS:DeletePeer] Deleting Peer. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, peers = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.CARRIERS:DeletePeer] Error Deleting PEER. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local addressId = peers[1]
	local tech, peerId = SeparatePeer(addressId)
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")

	local success, vccInfo = modelPeers.Get(peerId, tech, ksProv)
	if not success then
		ksProv:Close()
		LogError("[PROVISIONING.CARRIERS:DeletePeer] Error Deleting PEER %s. Not Found.", addressId)
		web.status = 404
		result.result = "PEER DOES NOT EXIST"
		return result
	end

	local response = modelPeers.Delete(peerId, tech, ksProv)
	if not response.success then
		LogError("[PROVISIONING.PEERS:DeletePeer] Error Deleting PEER %s. Error: %s.", addressId, response.errMsg or "")
		web.status = 400
		result.result = response.errType -- Este código de error llega desde la Rocka "provisioningmanager"
		result.relatedId = response.relatedId
		return result
	end

	LogInfo("[PROVISIONING.PEERS:DeletePeer] Peer '%s' deleting successfully.", addressId)

	ksProv:Close()
	web.status = 200
	result.result = "OK"

	return result
end

local function GetPeer (web, addressId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.PEERS:GetPeer] Retrieving PEER '%s' tech: '%s'. User %s.", addressId, tech, userId)

	local tech, peerId = SeparatePeer(addressId)

	local withCustomFields = true
	local success, peerInfo = modelPeers.Get(peerId, tech)
	if not success then
		LogError("[PROVISIONING.PEERS:GetPeer] Error Retrieving PEER %s tech %s. Not Found.", peerId, tech)
		web.status = 404
		result.result = "NOT FOUND"
		return result
	end

	web.status = 200
	return peerInfo
end

local function Search (web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	local postData = web.request
	local startTime = os.time()

	LogInfo("[PROVISIONING.PEERS:Search] 1 - (START) Loading Peers. Posted Info: '%s'", encodedInfo)

	local keyspace = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, peers = modelPeers.ListAll(keyspace)

	if not success then
		keyspace:Close()
		local err = peers
		LogWarning("[PROVISIONING.PEERS:Search] Error loading Peers. Err'%s'.", err)
		web.status = 500
		return {result = "SERVER ERROR", err = err}
	end

	--peers = utils.FilterEntities(peers, {IsSystemVCC = "0"})

	local totalUnfilteredRows = utils.GetRowCount(peers)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[PROVISIONING.PEERS:SEARCH] 2 - Filtering rows")

	for id, peer in pairs(peers) do
		local peerId = utils.GetId(id)
		peer.Id = peerId
		peer.Name = peer.Name or peerId
		if utils.MatchFilter(peer, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, peer)
		end
	end

	keyspace:Close()

	LogInfo("[PROVISIONING.PEERS:SEARCH] 3 - Sorting rows")
	--include only rows that belong to logged user peers
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[PROVISIONING.PEERS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

--[[
]]--
local function LoadPeers (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.HOSTS.:LoadPeers] Loading Peers, user %s, vcc %s", userId, vcc)

	local success, hosts = modelPeers.ListAll()
	if not success then
		local err = hosts
		LogWarning("[PROVISIONING.HOSTS:LoadPeers] Error loading Peers. Err'%s'.", err)
		web.status = 500
		return {result = "SERVER ERROR", err = err}
	end

	-- OK
	web.status = 200
	return hosts
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. m_permissions:CheckPermission("provisioning", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- PEERs i6
	module:dispatch_post(fn .. Search                                     , p .. "/search/"      )
	module:dispatch_get (HasPermission(GetPeer        , "view_peers"     ), p .. "/get/(.+)"     )
	module:dispatch_post(HasPermission(DeletePeer     , "delete_peer", fn), p .. "/delete/(.*)"  )
	module:dispatch_post(HasPermission(SavePeer       , "save_peer", fn  ), p .. "/save/"        )
	module:dispatch_post(HasPermission(AddPeerRange   , "save_peer", fn  ), p .. "/add_range/"   )
	module:dispatch_post(HasPermission(UpdatePeerRange, "save_peer", fn  ), p .. "/update_range/")
	module:dispatch_get (replyFn .. LoadPeers                             , p .. "/load/"        )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end