local utils = require "shared.apps.utils"
local modelTrunksGroups = require("apps.provisioning.models.trunksgroups")
local m_permissions = require "shared.permissions.permissions"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

local function Render (web)
	_M.response.DontCache(web)
	local env = {_T = m_htmlRender.extension.makeTextTranslator()}
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.TRUNKGROUPS:Render] Render form TRUNK GROUP with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local htmlFile = m_appLocation .. "/pages/" .. "frm_trunk_group_content.html"

	local htmlContent = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not htmlContent then
		assert(htmlContent, "[PROVISIONING.TRUNKGROUPS:Render] HTML TRUNK GROUP does not exist: %s", htmlFile)
	end

	return htmlContent
end

local function ValidateAndMakeInfoToUpdate (id, info)
	assert(id)
	assert(info)

	LogInfo("[PROVISIONING.TRUNKGROUPS:ValidateAndMakeInfoToUpdate] Validating TRUNK '%s' info to updated. Info %s", id, info)

	local exist, currentInfo = modelTrunksGroups.Get(id)

	-- 1 - clean up valores que llegan del lado del cliente
	local isNew = info.isNew

	if exist then
		-- Si estoy creando la Entidad y el mismo ya exsite en la BD, fallo!!
		if isNew then
			return false, "MUST_HAVE_UNIQUE_NAME"
		end
	end

	info.isNew = nil

	return true, info
end

local function SaveTrunkGroup (web)
	local postedData = web.input["post_data"]

	LogInfo("[PROVISIONING.TRUNKSGROPUS:SaveTrunkGroup] Saving Trunk Group. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, rows = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.TRUNKSGROPUS:SaveTrunkGroup] Error Updating Trunk Group. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local rowId, rowInfo = next(rows)
	--LogDebug("--> rowId %s", rowId)
	--LogDebug("--> rowInfo %s", rowInfo)

	local success, rowInfo = ValidateAndMakeInfoToUpdate(rowId, rowInfo)
	if not success then
		local errMsg = rowInfo
		LogError("[PROVISIONING.TRUNKSGROPUS:SaveTrunkGroup] Error Validating Trunk Group '%s'. Err: %s", rowId, errMsg)
		web.status = 500
		result.result = errMsg
		return result
	end

	local success, err = modelTrunksGroups.Save(rowId, rowInfo)
	if not success then
		LogError("[PROVISIONING.TRUNKSGROPUS:SaveTrunkGroup] Error Updating Trunk Group '%s'. Err: %s", rowId, err)
		web.status = 500
		result.result = err.code -- Este código de error llega desde la Rocka "provisioningmanager"
		result.rawData = err.rawData
		return result
	end

	LogInfo("[PROVISIONING.TRUNKSGROPUS:SaveTrunkGroup] '%s' saved successfully.", rowId)

	web.status = 200
	result.result = "OK"
	return result
end

local function DeleteTrunkGroup (web)
	local postedData = web.input["post_data"]
	local success, trunkInfo = pcall(json.decode, postedData)

	LogInfo("[PROVISIONING.TRUNKSGROPUS:DeleteTrunkGroup] Deleting Trunk Group. User %s@%s posted data '%s'.", inconcert.authUser.userName, inconcert.authUser.virtualContactCenter, postedData)

	local result = {}

	local success, trunksgroups = pcall(json.decode, postedData)
	if not success then
		LogError("[PROVISIONING.TRUNKGROUPS:DeleteTrunkGroup] Error Deleting TRUNK Group. Invalid Request.")
		web.status = 400
		result.result = "INVALID REQUEST"
		return result
	end

	local trunkGroupId = trunksgroups[1]
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")

	local success, vccInfo = modelTrunksGroups.Get(trunkGroupId, ksProv)
	if not success then
		ksProv:Close()
		LogError("[PROVISIONING.TRUNKGROUPS:DeleteTrunkGroup] Error Deleting TRUNK GROUP %s. Not Found.", trunkGroupId)
		web.status = 404
		result.result = "TRUNK GROUP DOES NOT EXIST"
		return result
	end

	local response = modelTrunksGroups.Delete(trunkGroupId, ksProv)
	if not response.success then
		LogError("[PROVISIONING.TRUNKSGROPUS:DeleteTrunkGroup] Error Deleting TRUNK GROUP %s. Error: %s.", trunkGroupId, response.errMsg or "")
		web.status = 400
		result.result = response.errType -- Este código de error llega desde la Rocka "provisioningmanager"
		result.relatedId = response.relatedId
		return result
	end

	LogInfo("[PROVISIONING.TRUNKSGROPUS:DeleteTrunkGroup] Trunk Group '%s' deleting successfully.", trunkGroupId)

	ksProv:Close()
	web.status = 200
	result.result = "OK"

	return result
end

local function GetTrunkGroup (web, trunkGroupId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.TRUNKSGROPUS:GetTrunkGroup] Retrieving TRUNK GROUP '%s'. User %s.", trunkGroupId, userId)

	local success, trunkGroupInfo = modelTrunksGroups.Get(trunkGroupId)
	if not success then
		LogError("[PROVISIONING.TRUNKSGROPUS:GetTrunkGroup] Error Retrieving TRUNK GROUP %s. Not Found.", trunkGroupId)
		web.status = 404
		local result = {}
		result.result = "NOT FOUND"
		return result
	end

	web.status = 200
	return trunkGroupInfo
end

local function Search (web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	local postData = web.request
	local startTime = os.time()

	LogInfo("[PROVISIONING.TRUNKSGROPUS:Search] 1 - (START) Loading Trunks Group. Posted Info: '%s'", encodedInfo)

	local keyspace = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, trunksgroups = modelTrunksGroups.ListAll(keyspace)

	if not success then
		keyspace:Close()
		local err = trunksgroups
		LogWarning("[PROVISIONING.TRUNKSGROPUS:Search] Error loading Trunks. Err'%s'.", err)
		web.status = 500
		return {result = "SERVER ERROR", err = err}
	end

	--trunksgroups = utils.FilterEntities(trunksgroups, {IsSystemVCC = "0"})

	local totalUnfilteredRows = utils.GetRowCount(trunksgroups)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[PROVISIONING.TRUNKSGROPUS:SEARCH] 2 - Filtering rows")

	for id, trunkGroup in pairs(trunksgroups) do
		local trunkGroupId = utils.GetId(id)
		trunkGroup.Id = trunkGroupId
		trunkGroup.Name = trunkGroup.Name or trunkGroupId
		if utils.MatchFilter(trunkGroup, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, trunkGroup)
		end
	end

	keyspace:Close()

	LogInfo("[PROVISIONING.TRUNKSGROPUS:SEARCH] 3 - Sorting rows")
	--include only rows that belong to logged user trunksgroups
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[PROVISIONING.TRUNKSGROPUS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

--[[
]]--
local function LoadTrunksGroups (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[PROVISIONING.TRUNKSGROPUS:LoadTrunksGroups] Loading Trunks Groups, user %s, vcc %s", userId, vcc)

	local success, trunksGroups = modelTrunksGroups.ListAll()
	if not success then
		local err = trunksGroups
		LogWarning("[PROVISIONING.TRUNKSGROPUS:LoadTrunksGroups] Error loading Trunks Groups. Err'%s'.", err)
		web.status = 500
		return {result = "SERVER ERROR", err = err}
	end

	-- OK
	web.status = 200
	return trunksGroups
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. m_permissions:CheckPermission("provisioning", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- TRUNK GROUP i6
	module:dispatch_post(fn .. Search, p .. "/search/")
	module:dispatch_get (replyFn .. LoadTrunksGroups, p .. "/load/")
	module:dispatch_get (HasPermission(GetTrunkGroup   , "view_trunks_groups"    ), p .. "/get/(.+)"   )
	module:dispatch_post(HasPermission(DeleteTrunkGroup, "delete_trunk_group", fn), p .. "/delete/(.*)")
	module:dispatch_post(HasPermission(SaveTrunkGroup  , "save_trunk_group", fn  ), p .. "/save/"      )


	m_htmlRender = htmlRender
	m_appLocation = appLocation
end