module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "DIDs"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, dids = pcall(PM.ListDIDs, ksProv)
	--LogDebug("[provisioning:models.dids.ListAll]--> success: '%s', DIDs: %s", success, json.encode(dids or {}))

	if not success then
		local errMsg = dids
		LogWarning("[provisioning:models.dids.ListAll] Error retrieving dids  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	dids = dids or {}

	return success, dids
end

function Get (didId, ksProv)
	LogDebug("[provisioning:models.dids.Get] Retrieving DID: '%s'.", didId)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentDID = pcall(PM.GetDID, ksProv, didId)
	LogDebug("[provisioning:models.dids.Get] Retrieved DID '%s' -> didId: '%s', currentDID: '%s'.", didId, success, json.encode(currentDID or {}))

	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentDID) then
		local errMsg = "DID DOES NOT EXIST"
		--LogWarning("[provisioning:models.dids.Get] Error retrieving DID '%s'. DID does not exist.'", didId)
		return false, errMsg
	end

	currentDID.Id = currentDID.Id or didId
	return true, currentDID
end

function Save (didId, didInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end

	-- Clean Up ---
	didInfo.isNew = nil

	local success, err = PM.UpdateDID(ksProv, didId, didInfo)
	if not success then
		LogWarning("[provisioning:models.dids.Save] Error saving dids '%s'. --> '%s'", didId, json.encode(err or {}))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function Delete (didId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	local success, result = pcall(PM.DeleteDID, ksProv, didId)
	if not success or not result.success then
		LogWarning("[provisioning:models.dids.Delete] Error deleting dids '%s' --> '%s'", didId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end
