module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "Peers"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetDefaultTech ()
	return "SIP"
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, peers = pcall(PM.ListPeers, ksProv)
	peers = peers or {}
	--LogDebug("[provisioning:models.peers.ListAll]--> success: '%s', Peers: %s", success, json.encode(peers))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = peers
		LogWarning("[provisioning:models.peers.ListAll] Error retrieving peers  -> '%s'", errMsg)
	end

	return success, peers
end

function GetFreeVCCPeers (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, freeVccPeers = pcall(PM.GetFreeVCCPeers, ksProv)
	freeVccPeers = freeVccPeers or {}

	--LogDebug("[provisioning:models.packages.GetFreeVCCPeers]--> success: '%s', freeVccPeers: %s", success, json.encode(freeVccPeers))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = freeVccPeers
		LogWarning("[provisioning:models.packages.GetFreeVCCPeers] Error retrieving freeVccPeers  -> '%s'", errMsg)
	end

	return success, freeVccPeers
end

function GetVCCPeers (currentVcc, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, vccPeers = pcall(PM.GetVCCPeers, ksProv, currentVcc)
	vccPeers = vccPeers or {}

	--LogDebug("[provisioning:models.packages.GetVCCPeers]--> success: '%s', Vcc: '%s' vccPeers: %s", success, currentVcc, json.encode(vccPeers))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = vccPeers
		LogWarning("[provisioning:models.packages.GetVCCPeers] Error retrieving Vcc: '%s' vccPeers  -> '%s'", currentVcc, errMsg)
	end

	return success, vccPeers
end

function Get (peerId, tech, ksProv)
	LogDebug("[provisioning:models.peers.Get] Retrieving PEER: '%s', tech:'%s'.", peerId, tech)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end
	tech = tech or GetDefaultTech()

	local success, currentPeer = pcall(PM.GetPeer, ksProv, tech, peerId)
	--LogDebug("[provisioning:models.peers.Get] Retrieved PEER '%s' -> peerId: '%s', tech:'%s', currentPeer: '%s'.", peerId, tech, success, json.encode(currentPeer or {}))

	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentPeer) then
		local errMsg = "PEER DOES NOT EXIST"
		--LogWarning("[provisioning:models.peers.Get] Error retrieving PEER '%s'. PEER does not exist.'", peerId)
		return false, errMsg
	end

	currentPeer.Id = currentPeer.Id or trunkGroupId

	return success, currentPeer
end

function Save (peerId, tech, peerInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	tech = tech or GetDefaultTech()

	local success, err = PM.UpdatePeer(ksProv, tech, peerId, peerInfo)
	if not success then
		LogWarning("[provisioning:models.peers.Save] Error saving peers '%s', tech: '%s'. --> '%s'", peerId, tech, json.encode(err))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function ExecuteRangePeers (peerId, tech, peerInfo, rangeInfo, action, ksProv)
	action = action or "ADD PEERS RANGE"

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	tech = tech or GetDefaultTech()

	local success, response
	if "ADD PEERS RANGE" == action then
		success, response = PM.AddPeerRange(ksProv, tech, rangeInfo.peer_start, rangeInfo.peer_count, peerInfo)
	elseif "UPDATE PEERS RANGE" then
		success, response = PM.UpdatePeerRange(ksProv, tech, rangeInfo.peer_start, rangeInfo.peer_count, peerInfo)
	else
		assert(false, "ExecuteRangePeers - Unrecognized action " .. tostring(action))
	end
	--LogDebug("[provisioning:models.peers.ExecuteRangePeers] success: '%s', response: '%s'", success, json.encode(response or {}))

	if not success then
		local err = response
		LogWarning("[provisioning:models.peers.ExecuteRangePeers] Error executing peers range '%s', tech: '%s'. --> '%s'", peerId, tech, json.encode(err))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, response
end

function Delete (peerId, tech, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	tech = tech or GetDefaultTech()

	local success, result = pcall(PM.DeletePeer, ksProv, tech, peerId)
	if not success or not result.success then
		LogWarning("[provisioning:models.peers.Delete] Error deleting peers '%s', tech: '%s'. --> '%s'", peerId, tech, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end
