module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "TtsAsr Engines"
end

function GetDefinition (datasource)
	m_datasource = datasource
	PM.SetRedis(datasource.redis)
	return _M
end

function ListTtsAsrVccEngines (vccId, ksAgent)
	local releaseConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseConn = true
	end

	local success, ttsasrEngines = pcall(PM.ListTtsAsrVccEngines, vccId, ksAgent)
	ttsasrEngines = ttsasrEngines or {}
	if releaseConn then
		ksAgent:Close()
	end
	--LogDebug("[provisioning:models.ttsasrEngines.ListTtsAsrVccEngines]--> success: '%s', ttsasrEngines: %s", success, json.encode(ttsasrEngines))

	if not success then
		local errMsg = ttsasrEngines
		LogWarning("[provisioning:models.ttsasrEngines.ListTtsAsrVccEngines] Error retrieving ttsasrEngines  -> '%s'", errMsg)
	end

	return success, ttsasrEngines
end

function ListProvisioningAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, ttsasrEngines = pcall(PM.ListTtsAsrProvisioningEngines, ksProv)
	ttsasrEngines = ttsasrEngines or {}
	if releaseConn then
		ksProv:Close()
	end
	--LogDebug("[provisioning:models.ttsasrEngines.ListProvisioningAll]--> success: '%s', ttsasrEngines: %s", success, json.encode(ttsasrEngines))

	if not success then
		local errMsg = ttsasrEngines
		LogWarning("[provisioning:models.ttsasrEngines.ListProvisioningAll] Error retrieving ttsasrEngines  -> '%s'", errMsg)
	end

	return success, ttsasrEngines
end

function ListProvisionedTtsEngines (ksProv, ksAgent)
	local releaseProvConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseProvConn = true
	end
	local releaseAgentConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseAgentConn = true
	end

	local success, ttsEngines = pcall(PM.ListProvisionedTtsEngines, ksProv, ksAgent)
	ttsEngines = ttsEngines or {}
	if releaseProvConn then
		ksProv:Close()
	end
	if releaseAgentConn then
		ksAgent:Close()
	end
	--LogDebug("[provisioning:models.ListProvisionedTtsEngines]--> success: '%s', ttsEngines: %s", success, json.encode(ttsEngines))

	if not success then
		local errMsg = ttsEngines
		LogWarning("[provisioning:models.ListProvisionedTtsEngines] Error retrieving ttsEngines  -> '%s'", errMsg)
	end

	return success, ttsEngines
end

function ListProvisionedAsrEngines (ksProv, ksAgent)
	local releaseProvConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseProvConn = true
	end
	local releaseAgentConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseAgentConn = true
	end

	local success, asrEngines = pcall(PM.ListProvisionedAsrEngines, ksProv, ksAgent)
	asrEngines = asrEngines or {}
	if releaseProvConn then
		ksProv:Close()
	end
	if releaseAgentConn then
		ksAgent:Close()
	end
	--LogDebug("[provisioning:models.ListProvisionedAsrEngines]--> success: '%s', asrEngines: %s", success, json.encode(asrEngines))

	if not success then
		local errMsg = asrEngines
		LogWarning("[provisioning:models.ListProvisionedAsrEngines] Error retrieving asrEngines  -> '%s'", errMsg)
	end

	return success, asrEngines
end

function ListTtsAsrProviders (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, ttsasrProviders = pcall(PM.ListTtsAsrProviders, ksProv)
	ttsasrProviders = ttsasrProviders or {}
	if releaseConn then
		ksProv:Close()
	end
	--LogDebug("[provisioning:models.ttsasrProviders.ListProviders]--> success: '%s', ttsasrProviders: %s", success, json.encode(ttsasrProviders))

	if not success then
		local errMsg = ttsasrProviders
		LogWarning("[provisioning:models.ttsasrProviders.ListProviders] Error retrieving ttsasrProviders  -> '%s'", errMsg)
	end

	return success, ttsasrProviders
end

function ListTtsAsrProviderTypes (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, ttsasrProviderTypes = pcall(PM.ListTtsAsrProviderTypes, ksProv)
	ttsasrProviderTypes = ttsasrProviderTypes or {}
	if releaseConn then
		ksProv:Close()
	end
	--LogDebug("[provisioning:models.ttsasrProviderTypes.ListProviders]--> success: '%s', ttsasrProviderTypes: %s", success, json.encode(ttsasrProviderTypes))

	if not success then
		local errMsg = ttsasrProviderTypes
		LogWarning("[provisioning:models.ttsasrProviderTypes.ListProviders] Error retrieving ttsasrProviderTypes  -> '%s'", errMsg)
	end

	return success, ttsasrProviderTypes
end

function Get (ttsAsrEngineId, ksProv)
	--LogDebug("[provisioning:models.ttsasrEngines.Get] Retrieving TTS/ASR Engines: '%s'.", ttsAsrEngineId)

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentTtsAsrEngine = pcall(PM.GetTtsAsrEngine, ksProv, ttsAsrEngineId)
	--LogDebug("[provisioning:models.ttsasrEngines.Get] Retrieved TTS/ASR Engines '%s' -> ttsAsrEngineId: '%s', currentTtsAsrEngine: '%s'.", ttsAsrEngineId, success, json.encode(currentTtsAsrEngine or {}))

	if not success then
		local errMsg = currentTtsAsrEngine
		LogWarning("[provisioning:models.ttsasrEngines.Get] Error retrieving TTS/ASR Engines '%s' -> '%s'", ttsAsrEngineId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	if utils.TableIsEmpty(currentTtsAsrEngine) then
		success = false
		local errMsg = "TTS/ASR ENGINE DOES NOT EXIST"
		--LogWarning("[provisioning:models.ttsasrEngines.Get] Error retrieving TTS/ASR Engines '%s'. TTS/ASR Engine does not exist.'", ttsAsrEngineId)
		return false, errMsg
	end

	return success, currentTtsAsrEngine
end

function Save (ttsAsrEngineId, ttsasrEngineInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	-- Clean Up ---
	ttsasrEngineInfo.isNew = nil

	if ttsasrEngineInfo.uniMrcpData then
		ttsasrEngineInfo.uniMrcpData = nil
	end
	
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local success, errMsg = PM.UpdateTtsAsrEngine(ksProv, ksAgent, ttsAsrEngineId, ttsasrEngineInfo)
	if not success then
		LogWarning("[provisioning:models.ttsasrEngines.Save] Error saving TTS/ASR Engines '%s' -> '%s'", ttsAsrEngineId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, errMsg
end

function Delete (ttsAsrEngineId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local success, result = pcall(PM.DeleteTtsAsrEngine, ksProv, ksAgent, ttsAsrEngineId)
	if not success or not result.success then
		LogWarning("[provisioning:models.ttsasrEngines.Delete] Error deleting TTS/ASR Engines '%s' -> '%s'.", ttsAsrEngineId, json.encode(result))
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end

function GetTtsAsrProfilesInUse (ttsAsrEngineId, ksAgent)
	local releaseConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseConn = true
	end

	local success, ttsAsrProfilesInUse = pcall(PM.GetTtsAsrProfilesInUse, ksAgent, ttsAsrEngineId)

	if releaseConn then
		ksAgent:Close()
	end

	return success, ttsAsrProfilesInUse
end

function GetVCCsByTtsAsrEngine (ttsAsrEngineId, ksAgent)
	local releaseConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseConn = true
	end

	local err, vcc = pcall(PM.GetVCCsByTtsAsrEngine, ksAgent, ttsAsrEngineId)

	if releaseConn then
		ksAgent:Close()
	end

	return vcc
end

function ListTtsAsrEnginesInUse(ksAgent, ksProv)
	local releaseAgentConn = false
	if not ksAgent then
		ksAgent = utils.GetDatasource():CassandraConnection("Agent")
		releaseAgentConn = true
	end
	local releaseProvConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseProvConn = true
	end
	
	local success, ttsAsrEnginesInUse = pcall(PM.ListTtsAsrEnginesInUse, ksAgent, ksProv)

	if releaseAgentConn then
		ksAgent:Close()
	end
	if releaseProvConn then
		ksProv:Close()
	end

	return success, ttsAsrEnginesInUse
end