/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	//const FORMAT = inConcert.Utils.format;

	const STATUS_LABELS = {
		RUNNING: "<%=_T([[Running]])%>",
		TIMEOUT: "<%=_T([[Timeout]])%>",
		MACHINE: "<%=_T([[Answering Machine]])%>",
		HUMAN: "<%=_T([[Human]])%>",
		NOTSURE: "<%=_T([[Not Sure]])%>",
		HANGUP: "<%=_T([[Hangup]])%>",
		NONE: "<%=_T([[Fail]])%>",
	};

	const CAUSE_LABELS = {
		TOOLONG: "TOOLONG - <%=_T([[Total Time]])%>",
		INITIALSILENCE: "INITIALSILENCE - <%=_T([[Silence Duration - Initial Silence]])%>",
		HUMAN: "HUMAN - <%=_T([[Silence Duration - After Greeting Silence]])%>",
		LONGGREETING: "LONGGREETING - <%=_T([[Voice Duration - Greeting]])%>",
		MAXWORDLENGTH: "MAXWORDLENGTH - <%=_T([[Word Length - max length of a single word]])%>",
		MAXWORDS: "MAXWORDS - <%=_T([[Word Count - maximum number of words]])%>",
		ERROR: "ERROR - <%=_T([[Error processing call]])%>",
	};

	const TEST_TIMEOUT_SECONDS = 120;

	//---------------------------------------------------------------------
	// Amd Management
	function ProvisioningAmdMng(params)
	{
		if (!params) {
			throw("ProvisioningAmdMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningAmdMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningAmdMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningAmdMng - 'params.container_id' is required");
		}
		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;

		this.default_values = {
			initialSilence: 2500,
			greeting: 1500,
			afterGreetingSilence: 800,
			totalAnalysisTime: 5000,
			miniumWordLength: 100,
			betweenWordSilence: 50,
			maximumNumberOfWords: 3,
			silenceThreshold: 256,
			maximumWordLength: 5000,
		};

		return true;
	}

	if (typeof inConcert.ProvisioningAmdMng !== "function") {
		inConcert.ProvisioningAmdMng = ProvisioningAmdMng;
	}

	ProvisioningAmdMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__set_events();
		this.__set_values();
		this.__enable_disable_setting_session(false);
	};

	ProvisioningAmdMng.prototype.__make_popup_html = function()
	{
		const prefix = this.prefix;
		const html = `
<div class="card-header d-flex flex-row justify-content-between align-items-center mb-3">
	<h5 class="mb-0"><%=_T([[Answering Machine Detection (AMD)]])%></h5>
	<button id="${prefix}btn_reset_params" title="<%=_T('Reset to default values.')%>" class="pageButtonGeneric buttonColorWhite ml-auto mb-0 mr-2"><%=_T([[Reset Values]])%></button>
	<div class="btn-group btnDropDownContainer">
		<button settings_params="true" id="${prefix}btn_config_by_trunk" class="form-control mr-1" data-toggle="dropdown" title="<%=_T([[Set parameter values by selecting a trunk.]])%>">
			<%=_T([[Parameters according to trunk]])%> 
		</button>
		<div id="${prefix}btn_config_by_trunk_menu" class="dropdown-menu btnDropDownMoreActions"></div>
	</div>
	<div class="btn-group">
	<button id="${prefix}test_results_button" class="pageButtonGeneric buttonColorPrimary ml-1" data-toggle="dropdown">
		<%=_T([[Start test]])%>
	</button>
	<div class="dropdown-menu dropdown-menu-right btnDropDownMoreActions AMD-test-dropdown">
		<div amd_test="true"  class="dropdownMenuBodyContainer">
			<div class="row">
				<div class="col-12">
					<div class="form-group">
						<label for="${prefix}test_results_number" class="required">
							<%=_T([[Enter area code and telephone]])%><%=_T([[:]])%>
						</label>
						<input id="${prefix}test_results_number" class="form-control" type="text"
							placeholder="<%=_T([[Enter the value…]])%>"
						/>
						<i id="${prefix}test_results_number_validation" class="formFieldValidation formFieldError" hidden></i>
					</div>
				</div>
			</div>
			<div id="${prefix}test_results_not_found" class="inlineMessagesContainer messageInfo mt-0">
				<%=_T('No AMD tests found.')%>
			</div>
			<div id="${prefix}test_results_list_container" hidden>
				<h6><%=_T([[Last results]])%></h6>
				<ul id="${prefix}test_results_list" class="resultsAMDList">
				</ul>
			</div>
		</div>
		<div class="dropdownMenuButtonsContainer">
			<button id="${prefix}btn_test_close" class="pageButtonGeneric buttonColorWhite"><%=_T([[Cancel]])%></button>
			<button id="${prefix}btn_test_confirm" class="pageButtonGeneric buttonColorBlue"><%=_T([[Start]])%></button>
		</div>
	</div>
	</div>
</div>
<div class="card-block">
	<div class="row">
		<div class="col-12">
			<div class="form-group">
				<div class="form-check">
					<label for="${prefix}enabled" class="form-check-label form-check-label-title">
						<input id="${prefix}enabled" class="form-check-input" type="checkbox"/>
						<%=_T([[Add AMD Settings]])%>
					</label>
				</div>
			</div>
		</div>
	</div>
	<div class="row">
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}initial_silence" class="required">
					<%=_T([[Maximum length of initial silence]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Maximum duration of the initial silence before the greeting. If passed, result is detection as MACHINE')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}initial_silence" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}initial_silence_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}greeting" class="required">
					<%=_T([[Maximum length of a greeting]])%>:
					<i class="fa fa-question-circle" title="<%=_T('If passed, result is detection as MACHINE')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}greeting" class="form-control" type="text" 
					   placeholder="<%=_T([[Enter the value…]])%>" 
				/>
				<i id="${prefix}greeting_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>			
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}after_greeting_silence" class="required">
					<%=_T([[Silence after greeting]])%>:
					<i class="fa fa-question-circle" title="<%=_T('If exceeded, result is detection as HUMAN')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}after_greeting_silence" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}after_greeting_silence_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>			
	</div>
	<div class="row">
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}total_analysis_time" class="required">
					<%=_T([[Total analysis time]])%>:
					<i class="fa fa-question-circle" title="<%=_T('It is the maximum time allowed for the algorithm to decide if the audio represents a HUMAN or a MACHINE')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}total_analysis_time" class="form-control" type="text"
					placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}total_analysis_time_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}minimum_word_length" class="required">
					<%=_T([[Minimum word length]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Minimum duration of the voice to be considered as a word')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}minimum_word_length" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}minimum_word_length_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}maximum_word_length" class="required">
					<%=_T([[Maximum word length]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Maximum length of a word to accept. If it is exceeded, then the result is detection as a MACHINE')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}maximum_word_length" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}maximum_word_length_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>			
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}maximum_number_of_words" class="required">
					<%=_T([[Maximum number of words]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Maximum number of words in a greeting. If exceeded, result is detection as a MACHINE')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}maximum_number_of_words" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}maximum_number_of_words_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
	</div>
	<div class="row">
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}between_word_silence" class="required">
					<%=_T([[Silence between words]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Minimum duration of silence after a word to consider the audio that follows as a new word')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}between_word_silence" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}between_word_silence_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}silence_threshold" class="required">
					<%=_T([[Silence threshold]])%>:
					<i class="fa fa-question-circle" title="<%=_T('Average noise level from 0 to 32767 which, if not exceeded, should be considered silence')%>"></i>
				</label>
				<input settings_params="true" id="${prefix}silence_threshold" class="form-control" type="text"
					   placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}silence_threshold_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>		
	</div>
</div>
</div>
		`;
		$(`#${this.container_id}`).html(html); 
	};

	ProvisioningAmdMng.prototype.dispose = function()
	{
		delete this.allTrunksAmd;
	};

	ProvisioningAmdMng.prototype.get = function(entity)
	{
		if (!$(`#${this.prefix}enabled`).is(":checked")){
			delete entity.amdParams;
			return;
		}

		entity.amdParams = {};

		const get_field = (key, name) =>
			entity.amdParams[key] =
				$(`#${this.prefix}${name}`).val() || this.default_values[key] || "";

		get_field("initialSilence", "initial_silence");
		get_field("greeting", "greeting");
		get_field("afterGreetingSilence", "after_greeting_silence");
		get_field("totalAnalysisTime", "total_analysis_time");
		get_field("miniumWordLength", "minimum_word_length");
		get_field("betweenWordSilence", "between_word_silence");
		get_field("maximumNumberOfWords", "maximum_number_of_words");
		get_field("silenceThreshold", "silence_threshold");
		get_field("maximumWordLength", "maximum_word_length");
	};

	ProvisioningAmdMng.prototype.__set_values = function(params)
	{
		params = params || {};
		const set_field = (key, name) =>
			$(`#${this.prefix}${name}`).val(params[key] || "");

		set_field("initialSilence", "initial_silence");
		set_field("greeting", "greeting");
		set_field("afterGreetingSilence", "after_greeting_silence");
		set_field("totalAnalysisTime", "total_analysis_time");
		set_field("miniumWordLength", "minimum_word_length");
		set_field("betweenWordSilence", "between_word_silence");
		set_field("maximumNumberOfWords", "maximum_number_of_words");
		set_field("silenceThreshold", "silence_threshold");
		set_field("maximumWordLength", "maximum_word_length");
	};

	ProvisioningAmdMng.prototype.set = function(entity = null)
	{
		this.currentEntity = entity;
		let params = entity?.amdParams;
		if (typeof(params) === 'string') params = JSON.parse(params);

		this.__set_values(params);

		if (entity)
			this.__load_test_results();

		$(`#${this.prefix}enabled`).prop("checked", !!entity?.amdParams);
		this.__enable_disable_setting_session(!!entity?.amdParams);
	};

	ProvisioningAmdMng.prototype.setAllTrunksAmd = function(allTrunksAmd)
	{
		let trunksfiltered = this.fill_trunks_parameters(allTrunksAmd); 
		this.allTrunksAmd = inConcert.Utils.arrayToMap(trunksfiltered,
			function(trunk) {
				return trunk;
			},
			function(trunk) {
				return trunk.id;
			}
		);
	};

	ProvisioningAmdMng.prototype.fill_trunks_parameters = function(allTrunksAmd)
	{
		const trunks_parameter = allTrunksAmd || [];
		const $trunk_parameters_menu = $(`#${this.prefix}btn_config_by_trunk_menu`);
		$trunk_parameters_menu.html("");
		const self = this;
		let trunks_filtered = [];
		$.each(trunks_parameter, function(_, item) {
			if (item.amdParams) {
				const $item = $(("").concat(
					'<a id=',`${self.prefix}btn_config_by_trunk_menu_item`, 
					' trunk_id="',item.id ,'" class="dropdown-item batchActionButton buttonColorGray" type="butto">',
						item.id,
					'</a>'
					));
				$trunk_parameters_menu.append($item);
				trunks_filtered.push(item);
			}
		});
		if (!trunks_filtered.length) {
			$(`#${this.prefix}btn_config_by_trunk`).prop("disabled", true);
		}
		return trunks_filtered;
	};

	ProvisioningAmdMng.prototype.empty_controls = function()
	{
		this.__set_values();
		this.__clean_test_results();
	};

	ProvisioningAmdMng.prototype.__set_default_values = function()
	{
		this.__set_values( this.default_values);
		const $trunk_parameters = $(`#${this.prefix}btn_config_by_trunk`);
		$trunk_parameters.text("<%=_T('Parameters according to trunk')%>");
		$trunk_parameters.attr("selected_trunk", "");
	};

	ProvisioningAmdMng.prototype.get_validation_def = function()
	{
		const self = this;
		const can_validate_adm = function() {
			return $(`#${self.prefix}enabled`).prop("checked");
		};
		return [
			{
				control_id: `${this.prefix}initial_silence`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd initial silence.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 100, to: 10000},
						message: "<%=_T('Amd initial silence. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}greeting`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd greeting.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 500, to: 10000},
						message: "<%=_T('Amd greeting. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}after_greeting_silence`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd after greeting silence.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 200, to: 5000},
						message: "<%=_T('Amd after greeting silence. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}total_analysis_time`,
				messages: {
					required: "<%=_T('Please enter amd total analysis time.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 2000, to: 15000},
						message: "<%=_T('Amd total analysis time. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}minimum_word_length`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd minimum word length.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 20, to: 500},
						message: "<%=_T('Amd minimum word length. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}between_word_silence`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd between word silence.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 10, to: 1000},
						message: "<%=_T('Amd between word silence. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}maximum_number_of_words`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd maximum number of words.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 1, to: 20},
						message: "<%=_T('Amd maximum number of words. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}silence_threshold`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd silence threshold.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 0, to: 32767},
						message: "<%=_T('Amd silence threshold. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}maximum_word_length`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter amd maximum word length.')%>",
					digits: "<%=_T('Only numeric values are allowed in amd parameters.')%>",
					range: {
						type   : "range",
						between: {from: 1000, to: 15000},
						message: "<%=_T('Amd maximum word length. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
		];
	};

	ProvisioningAmdMng.prototype.__clean_test_results = function()
	{
		const $results_not_found = $(`#${this.prefix}test_results_not_found`);
		const $results_list_container = $(`#${this.prefix}test_results_list_container`);
		
		$results_not_found.show();
		$results_list_container.find(`#${this.prefix}test_results_list`).html("");
		$results_list_container.hide();
	};

	ProvisioningAmdMng.prototype.__make_markup_list_result = function(results)
	{
		let not_found_result = !results.length;
		if (not_found_result) {
			this.__clean_test_results();
			return;
		}
		const $results_list_container = $(`#${this.prefix}test_results_list_container`);
		const $results_list = $results_list_container.find(`#${this.prefix}test_results_list`);
		$results_list.html("");

		let template = "".concat(
			"<li>",
				"<div class='resultsAMDListRow'>",
	  				"<p><label><%=_T('Status')%>:</label><span>{{statusDescription}}</span></p><i id='{{prefix}}test_result_status_question' class='fa fa-question-circle'></i>",
				"</div>",
				"<div class='resultsAMDListRow'>",
	  				"<p><label><%=_T('Date')%>:</label><span>{{date}}</span></p>",
				"</div>",
				"<div class='resultsAMDListRow'>",
	  				"<p><label><%=_T('Number')%>:</label><span>{{number}}</span></p>",
				"</div>",
				"<div class='resultsAMDListRow'>",
	  				"<p><label><%=_T('Description')%>:</label><span>{{description}}</span></p>",
				"</div>",
			"</li>"
		);
		
		results.forEach(elem => {
			const $result = $(Mustache.render(template, {
				prefix: this.prefix,
				date : elem.date,
				number : elem.number,
				statusDescription : elem.statusDescription,
				description: elem.description
			}));
			let $test_result_status_question =  $result.find(`#${this.prefix}test_result_status_question`);
			$test_result_status_question.inconcert_tooltip({
				className: 'bigTooltip',
				alignment: 'right-bottom',
				["remove-delay"]: 100000, 
				message: function() {
					const amdParams = elem.amdParams || {};
					let template = "".concat(
						"<div class='toolTipContainer'>",
							"<h6><%=_T('Configured parameters')%></h6>",
							"<ul class='resultsAMDList'>",
								"<li>",
									"<p><label><%=_T('Maximum length of initial silence')%></label><span>{{initial_silence}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Maximum length of a greeting')%></label><span>{{greeting}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Silence after greeting')%></label><span>{{after_greeting_silence}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Total analysis time')%></label><span>{{total_analysis_time}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Minimum word length')%></label><span>{{minimum_word_length}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Maximum word length')%>:</label><span>{{maximum_word_length}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Maximum number of words')%></label><span>{{maximum_number_of_words}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Silence between words')%></label><span>{{between_word_silence}}</span></p>",
								"</li>",
								"<li>",
									"<p><label><%=_T('Silence threshold')%></label><span>{{silence_threshold}}</span></p>",
								"</li>",
							"</ul>",
						"</div>"
					);
					const tooltip = Mustache.render(template, {
						initial_silence: amdParams.initialSilence,
						greeting : amdParams.greeting,
						after_greeting_silence : amdParams.afterGreetingSilence,
						total_analysis_time : amdParams.totalAnalysisTime,
						minimum_word_length: amdParams.miniumWordLength,
						maximum_word_length: amdParams.maximumWordLength,
						maximum_number_of_words: amdParams.maximumNumberOfWords,
						between_word_silence: amdParams.betweenWordSilence,
						silence_threshold: amdParams.silenceThreshold,
					});
					
					return tooltip;
				}
			});
			$results_list.append($result);
		});
		$results_list_container.show();
		const $results_not_found = $(`#${this.prefix}test_results_not_found`);
		$results_not_found.hide(); 
	};

	ProvisioningAmdMng.prototype.__make_result = function(result, now)
	{
		let {
			timestamp,
			number,
			status,
			cause
		} = result;
		if (status === "RUNNING" && now - timestamp > TEST_TIMEOUT_SECONDS)
			status = "TIMEOUT";
		return {
			...result,
			UUID : inConcert.Utils.generateGuid(),
			date: inConcert.Utils.timeAgo(timestamp * 1000),
			number,
			status,
			statusDescription: STATUS_LABELS[status] || status,
			description: cause ? (CAUSE_LABELS[cause] || cause) : '-',
		};
	};

	ProvisioningAmdMng.prototype.__load_test_results = async function()
	{
		try {
			const api = inConcert.app.getProvisioningAPI();
			let results = await api.asyncCall("trunkAmdResults", this.currentEntity.id);
			const now = (new Date()).getTime() / 1000;
			results = results.map((result) => this.__make_result(result, now));
			this.__make_markup_list_result(results);
			if (results.some((result) => result.status === "RUNNING"))
				window.setTimeout(() => this.__load_test_results(), 1000);
		} catch(error) {
			console.error(error);
		}
	};

	ProvisioningAmdMng.prototype.__validate_test_number = function(number)
	{
		let validation_result = inConcert.Utils.validate( number, "required");
		if ( !validation_result.success ) {
			const message = "<%=_T('Phone number is required.')%>";
			this.owner.show_status_message(message, "Error", 5000);
			return false;
		}

		validation_result = inConcert.Utils.validate( number, "number");
		if ( !validation_result.success ) {
			const message = "<%=_T('The field value is not a valid phone number.')%>";
			this.owner.show_status_message(message, "Error", 5000);
			return false;
		}

		return true;
	};

	ProvisioningAmdMng.prototype.__execute_test = function($number)
	{
		const number = $number.val();
		const self = this;
		if (this.__validate_test_number(number)) {
			const callback = async function(success) {
				if (success) {
					self.owner.show_waiting("<%=_T('Please wait ...')%>");
					await self.__start_test(number);
					self.owner.hide_waiting();
				}
			};
			this.owner.validation.start(this.get_validation_def(), callback, 6000);
		}
		else{
			$number.focus();
		}
	};

	ProvisioningAmdMng.prototype.__start_test = async function(number)
	{
		const data = {
			number: `+${number}`
		};
		this.get(data);
		const $button = $(`#${this.prefix}test_results_button`);
		const $button_confirm = $(`#${this.prefix}btn_test_confirm`);
		$button.prop("disabled", true);
		$button_confirm.prop("disabled", true);
		try {
			const api = inConcert.app.getProvisioningAPI();
			await api.asyncCall("trunkAmdStartTest", this.currentEntity.id, data);
			await this.__load_test_results();
		} catch(_error) {}
		$button.prop("disabled", false);
		$button_confirm.prop("disabled", false);
	};

	ProvisioningAmdMng.prototype.__enable_disable_setting_session = function(enable)
	{
		const $results_button = $(`#${this.prefix}test_results_button`);
		$results_button.prop("disabled",!enable);

		const $btn_reset_params = $(`#${this.prefix}btn_reset_params`);
		$btn_reset_params.prop("disabled",!enable);
		
		$container = $(`#${this.container_id}`);
		$container.find('[settings_params="true"]').each(function(_, setting_value)
		{
			$(setting_value).prop("disabled",!enable);
		});

		if (this.allTrunksAmd && !Object.keys(this.allTrunksAmd).length) {
			const $btn_trunk_parameters = $(`#${this.prefix}btn_config_by_trunk`);
			$btn_trunk_parameters.prop("disabled", true);
		}

	};

	ProvisioningAmdMng.prototype.__are_all_parameters_empty = function()
	{
		const is_empty = (name) =>
		$(`#${this.prefix}${name}`).val().trim() === "";

	return 	is_empty("initial_silence") && 
			is_empty("greeting") && 
			is_empty("after_greeting_silence") && 
			is_empty("total_analysis_time") && 
			is_empty("minimum_word_length") && 
			is_empty("between_word_silence") && 
			is_empty("maximum_number_of_words") && 
			is_empty("silence_threshold") && 
			is_empty("maximum_word_length");

	};

	ProvisioningAmdMng.prototype.__set_parameters_events = function()
	{
		const self = this;
		const event_fn = function() {
			const $trunk_parameters = $(`#${self.prefix}btn_config_by_trunk`);
			const selected_trunk = $trunk_parameters.attr("selected_trunk");
			if (selected_trunk && selected_trunk !== "") {
				$trunk_parameters.text("<%=_T('Parameters according to trunk')%>");
				$trunk_parameters.attr("selected_trunk", "");
			}
		};

		const field_events_fn = {
			keyup: event_fn,
			page: event_fn, 
			change: event_fn
		};

		const apply_event = (name) =>{
			$(`#${self.prefix}${name}`)
			.off("keyup", "page", "change" )
			.on(field_events_fn);
		};

		apply_event("initial_silence");
		apply_event("greeting");
		apply_event("after_greeting_silence");
		apply_event("total_analysis_time");
		apply_event("minimum_word_length");
		apply_event("between_word_silence");
		apply_event("maximum_number_of_words");
		apply_event("silence_threshold");
		apply_event("maximum_word_length");

	};

	ProvisioningAmdMng.prototype.on_scroll = function () {
		//Si se scrollea se oculta popup de pruebas si esta visible y no existe prueba en ejecucion.
		const $test_results_button = $(`#${this.prefix}test_results_button`);
		if (
			$test_results_button.attr("aria-expanded") === "true" &&
			!$test_results_button.prop("disabled")
		) {
			$(`#${this.prefix}btn_test_close`).click();
		}
	};

	ProvisioningAmdMng.prototype.__set_events = function()
	{
		const self = this;
		const $test_results_button = $(`#${this.prefix}test_results_button`);
		$test_results_button.parent()
		.off("hide.bs.dropdown")
		.on("hide.bs.dropdown", function(e) {
			if (e.clickEvent && e.clickEvent.target.id !== `${self.prefix}btn_test_close` ) {
				e.preventDefault();
			}
		});

		const $btn_init_test = $(`#${this.prefix}btn_test_confirm`);
		$btn_init_test
		.off("click")
		.on("click", function() {
			const $number = $(`#${self.prefix}test_results_number`);
			self.__execute_test($number);
		});

		const $btn_enable = $(`#${this.prefix}enabled`);
		$btn_enable.off("change")
			.on("change", function() {
				const isChecked = $(this).prop("checked");
				self.__enable_disable_setting_session(isChecked);
				if (isChecked && self.__are_all_parameters_empty()) {
					self.__set_default_values();
				}
			});
		
		const $btn_reset_params = $(`#${this.prefix}btn_reset_params`);
		$btn_reset_params
		.off("click")
		.on("click", function() {
			self.__set_default_values();
		});

		const $btn_config_by_trunk_menu = $(`#${this.prefix}btn_config_by_trunk_menu`);
		$btn_config_by_trunk_menu
			.off("click")
			.on("click", `#${this.prefix}btn_config_by_trunk_menu_item`,function() {
				const trunk_id = $(this).attr("trunk_id");
				const adm_by_id = self.allTrunksAmd && self.allTrunksAmd[trunk_id] && self.allTrunksAmd[trunk_id].amdParams;
				const params = typeof adm_by_id == "string"? JSON.parse(adm_by_id): adm_by_id;
				if (params) { 
					self.__set_values(params);
					const $trunk_parameters = $(`#${self.prefix}btn_config_by_trunk`);
					$trunk_parameters.text("<%=_T('Configuration')%>" + ": "+ trunk_id);
					$trunk_parameters.attr("selected_trunk", trunk_id);
				}
			});

		this.__set_parameters_events();
	};
}());
