/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	//---------------------------------------------------------------------
	// Fails Detection Management
	function ProvisioningFailsDetectionMng(params)
	{
		if (!params) {
			throw("ProvisioningFailsDetectionMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningFailsDetectionMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningFailsDetectionMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningFailsDetectionMng - 'params.container_id' is required");
		}
		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;

		this.default_values = {
			ConsecutiveFailsThreshold: 500,
			StepSize: 500,
			StepFailsRate: 95,
			Qualify: false,
			QualifyFrequency: 30,
		};

		return true;
	}

	if (typeof inConcert.ProvisioningFailsDetectionMng !== "function") {
		inConcert.ProvisioningFailsDetectionMng = ProvisioningFailsDetectionMng;
	}

	ProvisioningFailsDetectionMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__set_events();
		this.__set_values();
		this.__enable_disable_setting_session(false);
	};

	ProvisioningFailsDetectionMng.prototype.__make_popup_html = function()
	{
		const prefix = this.prefix;
		const html = `
<div class="card-header d-flex flex-row justify-content-between align-items-center mb-3">
	<h5 class="mb-0"><%=_T([[Trunk Fails Detection]])%></h5>
	<button
		id="${prefix}btn_reset_params"
		title="<%=_T('Reset to default values.')%>"
		class="pageButtonGeneric buttonColorWhite buttonReset ml-auto mb-0"
	>
		<%=_T([[Reset Values]])%>
	</button>
	<button
		id="${prefix}btn_reactivate_params"
		title="<%=_T('Activate all channels and remove fails.')%>"
		class="pageButtonGeneric buttonColorBlue buttonReset ml-2"
	>
		<%=_T([[Activate Channels]])%>
	</button>
</div>
<div class="card-block">
	<div class="row">
		<div class="col-12">
			<div class="form-group">
				<div class="form-check">
					<label for="${prefix}enabled" class="form-check-label form-check-label-title">
						<input id="${prefix}enabled" class="form-check-input" type="checkbox"/>
						<%=_T([[Fails Detection Enabled]])%>
					</label>
				</div>
			</div>
		</div>
	</div>
	<div class="row">
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}consecutive_fails_threshold" class="required">
					<%=_T([[Down Trunk Consecutive Fails]])%>:
					<i 
						class="fa fa-question-circle"
						title="<%=_T('Number of consecutive fails to mark trunk as down. Down trunks capacity is downgraded to 1 channel.')%>"
					>
					</i>
				</label>
				<input
					settings_params="true"
					id="${prefix}consecutive_fails_threshold"
					class="form-control"
					type="text"
					placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}consecutive_fails_threshold_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}step_size" class="required">
					<%=_T([[Intermittent Trunk Results Window]])%>:
					<i
						class="fa fa-question-circle"
						title="<%=_T('Number of results considered to detect an intermittent trunk. Intermittent trunks capacity is downgraded by 25%.')%>"
					>
					</i>
				</label>
				<input 
					settings_params="true"
					id="${prefix}step_size"
					class="form-control"
					type="text"
					placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}step_size_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}step_fails_rate" class="required">
					<%=_T([[Intermittent Trunk Fails %]])%>:
					<i
						class="fa fa-question-circle"
						title="<%=_T('Percentage of window results that are fail to detect an intermittent trunk. Intermittent trunks capacity is downgraded by 25%.')%>"
					>
					</i>
				</label>
				<input 
					settings_params="true"
					id="${prefix}step_fails_rate"
					class="form-control"
					type="text"
					placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i id="${prefix}step_fails_rate_validation" class="formFieldValidation formFieldError" hidden></i>
			</div>
		</div>
	</div>
	<div class="row">
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<div class="form-check mt-4">
					<label for="${prefix}qualify" class="form-check-label form-check-label-title">
						<input id="${prefix}qualify" class="form-check-input" type="checkbox"/>
						<%=_T([[Qualify]])%>
					</label>
				</div>
			</div>
		</div>
		<div class="col-6 col-xl-3">
			<div class="form-group">
				<label for="${prefix}qualify_frequency" class="required">
					<%=_T([[Qualify Frequency]])%>:
					<i
						class="fa fa-question-circle"
						title="<%=_T('Qualify monitor frequency')%>"
					>
					</i>
				</label>
				<input
					id="${prefix}qualify_frequency"
					class="form-control"
					type="text" 
					placeholder="<%=_T([[Enter the value…]])%>"
				/>
				<i
					id="${prefix}qualify_frequency_validation"
					class="formFieldValidation formFieldError"
					hidden
				>
				</i>
			</div>
		</div>
	</div>
</div>
		`;
		$(`#${this.container_id}`).html(html);
	};

	ProvisioningFailsDetectionMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ProvisioningFailsDetectionMng.prototype.get = function(entity)
	{
		if (!$(`#${this.prefix}enabled`).is(":checked")){
			delete entity.failsDetectionParams;
			return;
		}

		entity.failsDetectionParams = {};

		const get_field = (key, name) =>
			entity.failsDetectionParams[key] =
				parseInt($(`#${this.prefix}${name}`).val() || this.default_values[key] || "0", 10);

		get_field("ConsecutiveFailsThreshold", "consecutive_fails_threshold");
		get_field("StepSize", "step_size");
		get_field("StepFailsRate", "step_fails_rate");
		if ($(`#${this.prefix}qualify`).is(":checked")) {
			entity.failsDetectionParams.Qualify = true;
			get_field("QualifyFrequency", "qualify_frequency");
		}
	};

	ProvisioningFailsDetectionMng.prototype.__set_values = function(params)
	{
		params = params || {};
		const set_field = (key, name) =>
			$(`#${this.prefix}${name}`).val(params[key] || this.default_values[key]);

		set_field("ConsecutiveFailsThreshold", "consecutive_fails_threshold");
		set_field("StepSize", "step_size");
		set_field("StepFailsRate", "step_fails_rate");
		$(`#${this.prefix}qualify`).prop("checked", params.Qualify || this.default_values.Qualify);
		set_field("QualifyFrequency", "qualify_frequency");
	};

	ProvisioningFailsDetectionMng.prototype.set = function(entity = null)
	{
		this.currentEntity = entity;
		let params = entity?.failsDetectionParams;
		if (typeof(params) === 'string') params = JSON.parse(params);

		this.__set_values(params);

		$(`#${this.prefix}enabled`).prop("checked", !!entity?.failsDetectionParams);
		this.__enable_disable_setting_session(!!entity?.failsDetectionParams);
	};

	ProvisioningFailsDetectionMng.prototype.empty_controls = function()
	{
		this.__set_values();
	};

	ProvisioningFailsDetectionMng.prototype.__set_default_values = function()
	{
		this.__set_values( this.default_values);
		const $trunk_parameters = $(`#${this.prefix}trunk_parameters`);
		$trunk_parameters.val("");
	};

	ProvisioningFailsDetectionMng.prototype.__reactivate = function()
	{
		const api = inConcert.app.getProvisioningAPI();
		api.trunkResetFailsMonitor(this.currentEntity.id, (status) => {
			if (status)
				this.owner.show_status_message("<%=_T('Channels activated')%>", "Info", 5000);
			else
				this.owner.show_status_message("<%=_T('Error activating channels')%>", "Error", 5000);
		});
	};

	ProvisioningFailsDetectionMng.prototype.get_validation_def = function()
	{
		const self = this;
		const can_validate_adm = function() {
			return $(`#${self.prefix}enabled`).prop("checked");
		};
		const can_validate_qualify_frequency = function() {
			return can_validate_adm() && $(`#${self.prefix}qualify`).prop("checked");
		};

		return [
			{
				control_id: `${this.prefix}consecutive_fails_threshold`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter Down Trunk Consecutive Fails.')%>",
					range: {
						type   : "range",
						between: {from: 25, to: 500},
						message: "<%=_T('Down Trunk Consecutive Fails. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}step_size`,
				can_validate: can_validate_adm,
				messages: {
					required: "<%=_T('Please enter Intermittent Trunk Results Window.')%>",
					range: {
						type   : "range",
						between: {from: 25, to: 200},
						message: "<%=_T('Intermittent Trunk Results Window. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}step_fails_rate`,
				messages: {
					required: "<%=_T('Please enter Intermittent Trunk Fails %.')%>",
					range: {
						type   : "range",
						between: {from: 85, to: 99},
						message: "<%=_T('Intermittent Trunk Fails %. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: `${this.prefix}qualify_frequency`,
				can_validate: can_validate_qualify_frequency,
				messages: {
					required: "<%=_T('Please enter qualify monitor frequency.')%>",
					number: "<%=_T('Only numbers are allowed in the qualify monitor frequency.')%>"
				}
			},
		];
	};

	ProvisioningFailsDetectionMng.prototype.__enable_disable_setting_session = function(enable)
	{
		const $btn_reset_params = $(`#${this.prefix}btn_reset_params`);
		$btn_reset_params.prop("disabled",!enable);

		const $btn_reactivate_params = $(`#${this.prefix}btn_reactivate_params`);
		$btn_reactivate_params.prop("disabled",!enable);
		
		$container = $(`#${this.container_id}`);
		$container.find('[settings_params="true"]').each(function(_, setting_value)
		{
			$(setting_value).prop("disabled",!enable);
		});

		$(`#${this.prefix}qualify_frequency`).prop("disabled", !enable ||
			!$(`#${this.prefix}qualify`).prop("checked"));
	};

	ProvisioningFailsDetectionMng.prototype.__set_events = function()
	{
		const self = this;
		const $btn_enable = $(`#${this.prefix}enabled`);
		$btn_enable.off("change")
			.on("change", function() {
				self.__set_values();
				const isChecked = $(this).prop("checked");
				self.__enable_disable_setting_session(isChecked);
				if (!isChecked) {
					const $trunk_parameters = $(`#${self.prefix}trunk_parameters`);
					$trunk_parameters.val("");
				}
			});

		const $btn_qualify = $(`#${this.prefix}qualify`);
		$btn_qualify.off("change")
			.on("change", function() {
				self.__enable_disable_setting_session($btn_enable.prop("checked"));
			});
		
		const $btn_reset_params = $(`#${this.prefix}btn_reset_params`);
		$btn_reset_params
		.off("click")
		.on("click", function() {
			self.__set_default_values();
		});
		
		const $btn_reactivate_params = $(`#${this.prefix}btn_reactivate_params`);
		$btn_reactivate_params
		.off("click")
		.on("click", function() {
			self.__reactivate();
		});
	};
}());
