/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form dialplan processors class
	function form_provisioning_dialplan_processors(params)
	{
		const success = form_provisioning_dialplan_processors.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.include_save_and_continue = false;

		this.new_title = "<%=_T('New Dialplan Processor')%>";
		this.update_title = "<%=_T('Update Dialplan Processor {0}')%>";
		this.view_title = "<%=_T('View Dialplan Processor {0}')%>";
		this.tagline = "<%=_T('Tagline form Dialplan Processor!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/dialplan_processors/frm/render/";
		this.save_uri =       "/inconcert/apps/provisioning/dialplan_processors/save/";
		this.get_uri =        "/inconcert/apps/provisioning/dialplan_processors/get/";

		this.code_mng = new inConcert.ProvisioningDialplanProcessorMng({
			owner       : this,
			prefix      : "prov_form_dialplan_processor",
			container_id: "prov_form_dialplan_processor_panel"
		});

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_dialplan_processors, inConcert.conf_form);

	if (typeof inConcert.form_provisioning_dialplan_processors !== "function") {
		inConcert.form_provisioning_dialplan_processors = form_provisioning_dialplan_processors;
	}

	form_provisioning_dialplan_processors.prototype.initialize = function()
	{
		this.code_mng.initialize();
	};

	form_provisioning_dialplan_processors.prototype.get = function()
	{
		let processor = {
			Id         : $("#prov_form_dialplan_processor_id").val(),
			description: $("#prov_form_dialplan_processor_desc").val(),
			code : this.code_mng.get()
		};

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			processor[id] = this.$current_row.attr("row_id");
		}
		return processor;
	};

	form_provisioning_dialplan_processors.prototype.set = function(success, processor)
	{
		if (!success) {
			return;
		}
		$("#prov_form_dialplan_processor_id").val(processor.Id);
		$("#prov_form_dialplan_processor_desc").val(processor.description);
		this.code_mng.set(processor.Id, processor.code);
	};

	form_provisioning_dialplan_processors.prototype.empty_controls = function()
	{
		$("#prov_form_dialplan_processor_id").val("").focus();
		$("#prov_form_dialplan_processor_desc").val("");
	};

	form_provisioning_dialplan_processors.prototype.err_save_messages = function(err, is_custom_message)
	{
		var get_error_description = function(err_type, raw_data = {})
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
			// app provisioning errors
				case "ID_IS_REQUIRED":
					message = "<%=_T('Id is required.')%>";
				break;

				case "DESCRIPTION_IS_REQUIRED":
					message = "<%=_T('Description is required.')%>";
				break;

				case "CODE_IS_REQUIRED":
					message = "<%=_T('Diaplan Code is required.')%>";
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				const error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#prov_form_dialplan_processor_id").focus();
				}
				else {
					err = get_error_description(error_type, err.rawData);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		form_provisioning_dialplan_processors.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	form_provisioning_dialplan_processors.prototype.get_validation_def = function()
	{
		const self = this;

		const definitions = [
			{
				control_id: "prov_form_dialplan_processor_id",
				messages: {
					required: "<%=_T('Please enter the Dialplan Id.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Dialplan Id.')%>"
				}
			},
			{
				control_id: "prov_form_dialplan_processor_desc",
				messages: {
					required: "<%=_T('Please enter the Dialplan Description.')%>"
				}
			},
			{
				control_id: this.code_mng.get_dom_id(),
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							return self.code_mng.validate();
						}
					}
				}
			}
		];
		return definitions;
	};

	form_provisioning_dialplan_processors.prototype.ready = function()
	{
		form_provisioning_dialplan_processors.uber.ready.call(this); //invokes the base class method
		this.only_modified();
	};

	form_provisioning_dialplan_processors.prototype.only_modified = function()
	{
		const $id = $("#prov_form_dialplan_processor_id");
		if (this.is_new) {
			this.readonly_control($id, false /* disabled */ , true /* is_required */);
			$id.focus();
			this.code_mng.set(); // tengo que iniciarlo en vacio
		}
		else {
			this.readonly_control($id, true /* disabled */);
			$("#prov_form_dialplan_processor_desc").focus();
		}
	};

	form_provisioning_dialplan_processors.prototype.set_events = function()
	{
		form_provisioning_dialplan_processors.uber.set_events.call(this); //invokes the base class method
		// schedulers
		this.code_mng.set_events();
	};

	form_provisioning_dialplan_processors.prototype.load_data = function(callback_complete_requests)
	{
		let processorId = !this.is_new? this.$current_row.attr("row_id") : null;
		
		this.requests = this.code_mng.load_data(processorId);
		form_provisioning_dialplan_processors.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	form_provisioning_dialplan_processors.prototype.save_execute = function(after_save, entity, hash_id)
	{
		form_provisioning_dialplan_processors.uber.save_execute.call(this, 
			this.is_new? after_save : "SAVE_AND_NOT_EXIT_AND_REFRESH_VIEW", entity, hash_id);
	};

}());