/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	//---------------------------------------------------------------------
	// form channels class
	function form_provisioning_environment( params )
	{
		const success = form_provisioning_environment.uber.constructor.call( this, params );
		if ( !success ) {
			return;
		}
		this.view_title     = "<%=_T('Environment')%>";
		this.html_file_name = "/inconcert/apps/provisioning/environment/frm/render/";
		this.get_uri        = "/inconcert/apps/provisioning/environment/get/";
		this.container_id   = "#prov_form_environment_hotfixes";
		this.prefix_id      = "prov_form_environment_hotfixes";
		this.m_hotfixes     = null;
		return true;
	}

	inConcert.Utils.inherit( form_provisioning_environment, inConcert.conf_form );

	if ( typeof inConcert.form_provisioning_environment !== "function" ) {
		inConcert.form_provisioning_environment = form_provisioning_environment;
	}

	form_provisioning_environment.prototype.set = function( success, environment)
	{
		if (!success || !environment) {
			return;
		}
		const $inst_id = $("#prov_form_environment_installation_id");
		const $inst_creation_date = $("#prov_form_environment_installation_creation_date");
		$inst_id.val((environment.Installation && environment.Installation.ID) || "");
		$inst_creation_date.val((environment.Installation && environment.Installation.CreationDate) || "");
		this.__fill_grid(environment.Hotfixes || {});
		return true;
	};

	form_provisioning_environment.prototype.show_not_hotfixes_found = function( message )
	{
		$("#prov_form_environment_hotfixes_not_results_found")
			.text( message || "<%=_T('No hotfixes found.')%>" )
			.show();
	};

	form_provisioning_environment.prototype.apply_initial_config = function()
	{
		this.hide_header_system_actions();
		this.set_title(this.view_title);
	};

	form_provisioning_environment.prototype.hide_not_hotfixes_found = function()
	{
		$("#prov_form_environment_hotfixes_not_results_found").hide();
	};

	form_provisioning_environment.prototype.fill_hotfixes = function( hotfixes )
	{
		const $hotfixes = $("#prov_form_environment_hotfixes");
		$hotfixes.html("" +
			"<li class='viewListTitle'>" +
				"<span class='firstItemIcon'></span>" +
				"<label head='true' class='viewListTitleText'><%=_T('Id')%></label>" +
				"<label head='true' class='viewListTitleText'><%=_T('HostName')%></label>" +
				"<label head='true' class='viewListTitleText'><%=_T('Description')%></label>" +
				"<label head='true' class='viewListTitleText'><%=_T('Creation Date')%></label>" +
			"</li>"
		);
		let existsAvailable = false;
		$.each(hotfixes, function( _, hotfix ) {
			existsAvailable = true;
			const $hotfix = $(
				"<li class='viewListItem formListItem'>" +
					"<div class='viewListItemText'>" +
						"<label>" + hotfix.HotfixId + "</label>" +
					"</div>" +
					"<div class='viewListItemText'>" +
						"<label>" + hotfix.hostName + "</label>" +
					"</div>" +
					"<div class='viewListItemText'>" +
						"<label>" + hotfix.Description + "</label>" +
					"</div>" +
					"<div class='viewListItemText'>" +
						"<label>" + hotfix.timeStamp + "</label>" +
					"</div>" +
				"</li>");
			$hotfixes.append( $hotfix );
		});
		if (existsAvailable) {
			this.hide_not_hotfixes_found();
		}
		else{
			this.show_not_hotfixes_found();
			$hotfixes.html("");
		}
	};

	form_provisioning_environment.prototype.ready = function()
	{
		form_provisioning_environment.uber.ready.call( this );
		this.readonly();
		
	};

	form_provisioning_environment.prototype.make_html = function()
	{
		$(this.container_id).html("".concat(
			"<div class='row align-items-center'>",
				"<div class='col-6'>",
					"<div class='form-group mb-0'>",
						"<div class='card-header'>",
							"<h5><%=_T('Installed hotfixes')%></h5>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			// grid
			"<div id='", this.prefix_id , "_grid_container'>",
				// Table
				"<div class='row mt-2'>",
					"<div class='col-12'>",
						"<table id='", this.prefix_id , "_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid mt-0'>",
							"<thead class='viewGridHeader'></thead>",
							"<tbody class='viewGridBody'></tbody>",
						"</table>",
						"<div id='", this.prefix_id , "_not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
						"<div id='", this.prefix_id , "_paginator'></div>",
					"</div>",
				"</div>",
			"</div>",
			// disclaimer
			"<div class='row'>" ,
					"<div class='col-12'>" ,
						"<div style='display:none' class='inlineMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>",
					"</div>",
			"</div>"
		));
	};

	form_provisioning_environment.prototype.initialize = function()
	{
		this.make_html();
		this.__initialize_grid();
	};

	form_provisioning_environment.prototype.__initialize_grid = function()
	{
		this.m_hotfixes = new inConcert.conf_form_paginator({
			paginator_name: "HOTFIXES_LIST",
			owner: this,
			frm: this,
			auto_initialize: true,
			row_id: "HotfixId",
			display_name: "HotfixId",
			rows_per_page: 15,
			show_at_least: 14,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			add_with_enter: false,
			ids: {
				grid: this.prefix_id + "_list",
				paginator: this.prefix_id + "_paginator",
				not_results_found: this.prefix_id + "_not_results_found"
			},
			strings: {
				not_results_found         : "<%=_T('No hotfixes found.')%>",
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "HotfixId"   , name: "<%=_T('Id')%>"},
				{id: "hostName"   , name: "<%=_T('HostName')%>"},
				{id: "Description", name: "<%=_T('Description')%>"},
				{id: "timeStamp"  , name: "<%=_T('Creation Date')%>"},
				{
					id: "_action",
					name: "",
					system_action: {
						view: true
					}
				}
			],
		});
	};

	form_provisioning_environment.prototype.__fill_grid = function(list)
	{
		list = list || {};
		this.m_hotfixes.fill_rows($.noop, list);
	};

	form_provisioning_environment.prototype.uri_get_entity = function()
	{
		return inConcert.app.getBaseDomain() + this.get_uri;
	};

	form_provisioning_environment.prototype.no_close = function()
	{
		this.readonly();
	};
}());
