/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form gateways class - basic section
	function form_provisioning_gateways_section_basic( params )
	{
		const success = form_provisioning_gateways_section_basic.uber.constructor.call( this, params );
		if (!success) {
			return;
		}
		this.view_action_id = "view_gateways";  // view action id de permisos
		this.save_action_id  = { create: "save_gateway", update: "save_gateway" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos
		this.initialize();
	
		return true;
	}

	inConcert.Utils.inherit( form_provisioning_gateways_section_basic, inConcert.conf_form_section );

	if ( typeof inConcert.form_provisioning_gateways_section_basic !== "function" ) {
		inConcert.form_provisioning_gateways_section_basic = form_provisioning_gateways_section_basic;
	}

	form_provisioning_gateways_section_basic.prototype.initialize = function()
	{
		this.set_events();
	};

	form_provisioning_gateways_section_basic.prototype.set_events = function()
	{
		$("#form_provisioning_gateway_add_configuration_section")
			.off("click")
			.on("click", () => this.show_section_popup());
		$("#form_provisioning_gateway_add_configuration_section_popup_create")
			.off("click")
			.on("click", () => this.add_section());
		$("#form_provisioning_gateway_add_configuration_section_popup_close, " +
		  "#form_provisioning_gateway_add_configuration_section_popup_cancel")
			.off("click")
			.on("click", () => this.hide_section_popup());
		$("#form_provisioning_gateway_pjsip")
			.off("change")
			.on("change", () => this.toggle_pjsip());
		
		this.sections = {};
		this.toggle_pjsip();
	};

	form_provisioning_gateways_section_basic.prototype.get = function(gateway = {})
	{
		gateway.configuration = gateway.configuration || {};
		if (this.frm.is_new) {
			gateway.configuration.isNew = true;
			gateway.configuration.Id   = $("#form_provisioning_gateway_name").val();
			gateway.configuration.Name = $("#form_provisioning_gateway_name").val();
		}
		gateway.configuration.Description = $( "#form_provisioning_gateway_desc").val();
		gateway.configuration.Domain = $("#form_provisioning_gateway_domain").val();
		gateway.configuration.Port = $("#form_provisioning_gateway_port").val();
		gateway.configuration.Pjsip = $("#form_provisioning_gateway_pjsip").is(":checked") ? "1" : "0";
		gateway.configuration.AmiUser = $("#form_provisioning_gateway_ami_user").val();
		gateway.configuration.AmiPassword = $("#form_provisioning_gateway_ami_password").val();
		gateway.configuration.AmiPort = $("#form_provisioning_gateway_ami_port").val();
		gateway.configuration.Sections = this.get_sections();
		return gateway;
	};

	form_provisioning_gateways_section_basic.prototype.set = function(gateway)
	{
		this.initialize();

		$( "#form_provisioning_gateway_name" ).val( gateway.configuration.Name );
		$( "#form_provisioning_gateway_desc" ).focus().val( gateway.configuration.Description );
		$( "#form_provisioning_gateway_domain" ).val( gateway.configuration.Domain );
		$( "#form_provisioning_gateway_port" ).val( gateway.configuration.Port );
		$( "#form_provisioning_gateway_pjsip" ).prop("checked", gateway.configuration.Pjsip === "1" );
		$( "#form_provisioning_gateway_ami_user" ).val( gateway.configuration.AmiUser );
		$( "#form_provisioning_gateway_ami_password" ).val( gateway.configuration.AmiPassword );
		$( "#form_provisioning_gateway_ami_port" ).val( gateway.configuration.AmiPort );
		this.set_sections(gateway.configuration.Sections || {});
		this.toggle_pjsip();
	};

	form_provisioning_gateways_section_basic.prototype.refresh = function(gateway)
	{
		const refreshedSections = gateway.configuration.Sections;
		Object.keys(this.sections).forEach((sectionId) => {
			this.sections[sectionId].initialize();
			this.sections[sectionId].set(refreshedSections[sectionId]?.Attributes || []);
		});
	};

	form_provisioning_gateways_section_basic.prototype.get_validation_def = function(definitions = [])
	{
		definitions.push(
			{
				control_id: "form_provisioning_gateway_name",
				messages: {
					required: "<%=_T('Please enter the gateway Name.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the gateway Name.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_desc",
				messages: {
					required: "<%=_T('Please enter the gateway description.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_domain",
				messages: {
					required: "<%=_T('Please enter the gateway domain or ip.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_port",
				messages: {
					required: "<%=_T('Please enter the gateway asterisk port.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_ami_user",
				messages: {
					required: "<%=_T('Please enter the gateway AMI user.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_ami_password",
				messages: {
					required: "<%=_T('Please enter the gateway AMI password.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_ami_port",
				messages: {
					required: "<%=_T('Please enter the gateway AMI port.')%>"
				}
			},
			{
				control_id: "form_provisioning_gateway_add_configuration_section",
				messages: {
					custom: {
						type : "custom",
						valid: () => this.validate_sections()
					}
				}
			}
		);
	};

	form_provisioning_gateways_section_basic.prototype.attach_load_request = function( _request )
	{
		
	};

	form_provisioning_gateways_section_basic.prototype.editable = function()
	{
		form_provisioning_gateways_section_basic.uber.editable.call( this );
		this.frm.editable_buttons();
	};

	form_provisioning_gateways_section_basic.prototype.readonly = function()
	{
		form_provisioning_gateways_section_basic.uber.readonly.call( this );
		this.frm.readonly_buttons();
	};

	form_provisioning_gateways_section_basic.prototype.readonly_gateway_name = function()
	{
		var $name = $("#form_provisioning_gateway_name");
		var $desc = $("#form_provisioning_gateway_desc");
		if ( this.frm.is_new ) {
			this.frm.readonly_control( $name, false /* disabled */ , true /* is_required */ );
			$name.focus();
		}
		else {
			this.frm.readonly_control( $name, true /* disabled */ );
			$desc.focus();
		}
	};

	form_provisioning_gateways_section_basic.prototype.ready = function()
	{
		this.readonly_gateway_name();
	};

	form_provisioning_gateways_section_basic.prototype.on_show = function()
	{
		form_provisioning_gateways_section_basic.uber.on_show.call(this, {show: false});
		const is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	form_provisioning_gateways_section_basic.prototype.remove_section = function(sectionId)
	{
		if (!this.sections[sectionId]) return;

		$(`#form_provisioning_gateways_section_basic_attributes_${sectionId}`).remove();
		delete this.sections[sectionId];
	};

	form_provisioning_gateways_section_basic.prototype.toggle_pjsip = function()
	{
		if ($("#form_provisioning_gateway_pjsip").is(":checked"))
			this.enable_pjsip();
		else
			this.disable_pjsip();
	};

	form_provisioning_gateways_section_basic.prototype.enable_pjsip = function()
	{
		$("#form_provisioning_gateway_add_configuration_sections_title").text(
			"<%=_T('Global pjsip configuration (pjsip.conf)')%>");
		$("#form_provisioning_gateway_add_configuration_section").toggle(!this.is_readonly());
		this.remove_section("general");
	};

	form_provisioning_gateways_section_basic.prototype.disable_pjsip = function()
	{
		$("#form_provisioning_gateway_add_configuration_sections_title").text(
			"<%=_T('Global sip configuration (sip.conf)')%>");
		$("#form_provisioning_gateway_add_configuration_section").hide();
		Object.keys(this.sections)
			.filter((sectionId) => sectionId !== "general")
			.forEach((sectionId) => this.remove_section(sectionId));
		if (!this.sections.general)
			this.set_section({ Name: "general", Attributes: [] });
	};

	form_provisioning_gateways_section_basic.prototype.has_multiple_sections = function()
	{
		return $("#form_provisioning_gateway_pjsip").is(":checked");
	};

	form_provisioning_gateways_section_basic.prototype.set_sections = function(sections)
	{
		Object.keys(sections).sort().forEach((sectionId) =>
			this.set_section(sections[sectionId]));
	};

	form_provisioning_gateways_section_basic.prototype.set_section = function(section)
	{
		const multipleSections = this.has_multiple_sections();
		const $section = $(`
			<div
				id="form_provisioning_gateways_section_basic_attributes_${section.Name}"
				class="card-block">
			</div>
		`);
		$("#form_provisioning_gateway_add_configuration_sections").append($section);
		this.sections[section.Name] = new inConcert.ProvisioningGatewayConfigTableMng({
			title		 : section.Name,
			owner        : this,
			prefix       : `form_provisioning_gateways_carrier_trunks_edition_${section.Name}`,
			container_id : `form_provisioning_gateways_section_basic_attributes_${section.Name}`,
			footer_actions_scope : "form_provisioning_gateway_add_configuration_sections",
			footer_actions_popup: "form_provisioning_gateway_add_configuration_footer_actions",
			collapsable	 : true,
			allowContext : true,
			collapsed    : multipleSections,
			readonly	 : this.is_readonly(),
			tooltip		 : FORMAT("<%=_T('Configuration file section {0}')%>", `[${section.Name}]`),
			on_remove	 : multipleSections ? (
				() => {
					$section.remove();
					delete this.sections[section.Name];
					inConcert.ProvisioningGatewayConfigTableMng.prototype.toggle_bottom_actions({
						footer_actions_scope : "form_provisioning_gateway_add_configuration_sections",
						footer_actions_popup: "form_provisioning_gateway_add_configuration_footer_actions"
					})
				}) : null
		});
		this.sections[section.Name].initialize();
		this.sections[section.Name].set(section.Attributes);
	};

	form_provisioning_gateways_section_basic.prototype.get_sections = function()
	{
		return Object.keys(this.sections).reduce((sections, sectionId) => ({
			...sections,
			[sectionId]: {
				Name: sectionId,
				Attributes: this.sections[sectionId].get()
			}
		}), {});
	};

	form_provisioning_gateways_section_basic.prototype.validate_sections = function()
	{
		let result = { success: true };
		Object.keys(this.sections).some((sectionId) => {
			result = this.sections[sectionId].validate();
			return !result.success;
		});
		return result;
	};

	form_provisioning_gateways_section_basic.prototype.show_section_popup = function()
	{
		$("#form_provisioning_gateway_add_configuration_section_popup_name").val("");
		$("#form_provisioning_gateway_add_configuration_section_popup_message").hide();
		$("#form_provisioning_gateway_add_configuration_section_popup").show();
	};

	form_provisioning_gateways_section_basic.prototype.hide_section_popup = function()
	{
		$("#form_provisioning_gateway_add_configuration_section_popup").hide();
	};

	form_provisioning_gateways_section_basic.prototype.add_section = function()
	{
		const section = this.validate_section_to_add();
		if (!section)
			return;

		this.set_section(section);
		this.hide_section_popup();
	};

	form_provisioning_gateways_section_basic.prototype.validate_section_to_add = function()
	{
		const show_error = function(message_error, $control)
		{
			$("#form_provisioning_gateway_add_configuration_section_popup_message")
				.text(message_error)
				.show();
			if (!$control) return;

			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		};

		const $name = $("#form_provisioning_gateway_add_configuration_section_popup_name");
		const section = {
			Name: $name.val(),
			Attributes: []
		}

		if (section.Name === "general" && this.has_multiple_sections())
			return show_error("<%=_T('Pjsip has not general section.')%>", $name);

		if (!inConcert.Utils.validate( section.Name, "required" ).success)
			return show_error("<%=_T('Name is required.')%>", $name);

		if (!inConcert.Utils.validate( section.Name, "id" ).success)
			return show_error(
				"<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>", $name);

		if ($(`#form_provisioning_gateways_section_basic_attributes_${section.Name}`).length)
			return show_error(
				"<%=_T('Name must be unique.')%>", $name);

		return section;
	};
}());
