/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form packages class
	function form_provisioning_packages(params)
	{
		const success = form_provisioning_packages.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title      = "<%=_T('New Package')%>";
		this.update_title   = "<%=_T('Update Package {0}')%>";
		this.view_title     = "<%=_T('View Package {0}')%>";
		this.tagline        = "<%=_T('Tagline form Package!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/packages/frm/render/";
		this.save_uri       =  "/inconcert/apps/provisioning/packages/save/";
		this.get_uri        = "/inconcert/apps/provisioning/packages/get/";

		this.dialplan_call_types = inConcert.app.getProvisioningAPI().getCallTypes();

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_packages, inConcert.conf_form);

	if (typeof inConcert.form_provisioning_packages !== "function") {
		inConcert.form_provisioning_packages = form_provisioning_packages;
	}

	form_provisioning_packages.prototype.get = function()
	{
		let pack = {
			Id         : $("#prov_form_package_id").val(),
			description: $("#prov_form_package_desc").val(),
			capacity   : $("#prov_form_package_capacity").val(),
			ioMode     : $("#prov_form_package_io_mode").val()
		};

		pack.DID = $("#prov_form_package_did").val();

		if (pack.ioMode === "O" || pack.ioMode === "I/O") {
			pack.outgoingRoute = $("#prov_form_package_outgoing_route").val();

			const callerIDIsDID = $("#prov_form_package_outgoing_route_callerIDIsDID").prop("checked");
			if (callerIDIsDID) {
				pack.callerIDIsDID = true;
			}
			const is_prefix_enabled = $("#prov_form_package_outgoing_route_prefix_enabled").prop("checked");
			if (is_prefix_enabled) {
				pack.prefix = $("#prov_form_package_outgoing_route_prefix").val();
				pack.replaceCarrierPrefix = $("#prov_form_package_outgoing_route_prefix_replace_carrier_prefix").prop("checked");
			}
		}

		pack.dialplanProcessors = this.dialplan_call_types.reduce(function(processors, callType) {
			let processorId = $("#prov_form_package_dialplan_" + callType.id).val();
			if (processorId) {
				processors[callType.id] = { processor : processorId };
			}
			return processors;
		}, {});

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			pack[id] = this.$current_row.attr("row_id");
			let currentPackage = this.get_entity_instance();
			if (currentPackage) {
				pack.VCC = currentPackage.VCC;
			}
		}
		return pack;
	};

	form_provisioning_packages.prototype.set = function(success, pack)
	{
		if (!success) {
			return;
		}
		$("#prov_form_package_id").val(pack.Id);
		$("#prov_form_package_desc").val(pack.description);
		$("#prov_form_package_capacity").val(pack.capacity);
		$("#prov_form_package_io_mode").val(pack.ioMode);
		$("#prov_form_package_did").val(pack.DID);
		$("#prov_form_package_outgoing_route").val(pack.outgoingRoute || "");

		if (pack.callerIDIsDID === "true") {
			$("#prov_form_package_outgoing_route_callerIDIsDID").prop("checked", true);
		}

		if (pack.prefix) {
			$("#prov_form_package_outgoing_route_prefix_enabled").prop("checked", true);
			$("#prov_form_package_outgoing_route_prefix").val(pack.prefix);
			if (pack.replaceCarrierPrefix === "true") {
				$("#prov_form_package_outgoing_route_prefix_replace_carrier_prefix").prop("checked", true);
			}
		}
		
		$.each(pack.dialplanProcessors || {}, function(callType, processor) {
			$("#prov_form_package_dialplan_" + callType).val(processor.processor);
		});

		this.toggle_io_mode();
	};

	form_provisioning_packages.prototype.empty_controls = function()
	{
		$("#prov_form_package_id").val("").focus();
		$("#prov_form_package_desc").val("");
		$("#prov_form_package_capacity").val("");
		$("#prov_form_package_io_mode").val("");
		$("#prov_form_package_did").val("");
		$("#prov_form_package_outgoing_route").val("");

		this.toggle_io_mode();
	};

	form_provisioning_packages.prototype.err_save_messages = function(err, is_custom_message)
	{
		const self = this;

		var get_error_description = function(err_type, raw_data = {})
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
			// COMMON ERRORS
				case "PACKAGE MUST SPECIFY A DID":
					message = "<%=_T('Package must specify an existing DID.')%>";
				break;

				case "DID DOES NOT EXIST":
					message = "<%=_T('DID does not exist.')%>";
				break;

				case "INVALID IOMODE VALUES":
					message = "<%=_T('Invalid I/O Mode Values.')%>";
				break;

			// OUTBOUND ERRORS
				case "OUTBOUND ERROR - OUTBOUND PACKAGE MUST SPECIFY AN EXISTING ROUTE":
					message = "<%=_T('Outgoing package error. The package must specify an existing route.')%>";
				break;

				case "OUTBOUND ERROR - ROUTE DOES NOT EXIST":
					message = "<%=_T('Outgoing package error. Route does not exist.')%>";
				break;

				case "OUTBOUND ERROR - CANNOT SET 'CALLERIDISDID' BECAUSE ROUTE USES TRUNKS FROM MORE THAN ONE CARRIER":
					message = FORMAT("<%=_T('Outgoing package error. Cannot set callerIDIsDID, because route {0} uses trunks from more than one carrier.')%>", (raw_data.outgoingRoute || ""));
				break;

				case "OUTBOUND ERROR - CANNOT SET 'CALLERIDISDID' BECAUSE ROUTE USES TRUNKS FROM CARRIER WHILE DID BELONGS TO CARRIER":
					message = FORMAT("<%=_T('Outgoing package error. Cannot set callerIDIsDID, because route {0} uses trunks from carrier {1}, while DID {2} belongs to carrier {3}.')%>",
						(raw_data.outgoingRoute || ""),
						(raw_data.monoCarrier || ""),
						(raw_data.did || ""),
						(raw_data.didCarrier || "")
					);
				break;
			
			// INBOUND ERRORS
				case "INBOUND ERROR - DID NEEDS A DNIS TO BE USED AS AN INBOUND NUMBER":
					message = FORMAT("<%=_T('Incoming package error. DID {0} needs a DNIS to be used as an inbound number.')%>",
						(raw_data.did || "")
					);
				break;

				case "INBOUND ERROR - DID HAS DNIS WHICH IS ALREADY IN USE ON PACKAGE":
					message = FORMAT("<%=_T('Incoming package error. DID {0} has DNIS {1}, which is already in use on Incoming package {2}.')%>",
						(raw_data.did || ""),
						(raw_data.dnis || ""),
						(raw_data.package || "")
					);
				break;

				case "I/O ERROR - UNABLE TO CHANGE PACKAGE FROM OUTGOING TO INBOUND MODE. IT ASSOCIATED TO CAMPAIGNS OF VCC":
					let campaigns = (raw_data.campaigns || []);
					let vcc = (raw_data.vcc || "");

					let campaigns_msg = "";
					for (let i = 0; i < campaigns.length; i++) {
						let campaign_id = campaigns[i];
						if (campaigns_msg) {
							campaigns_msg = campaigns_msg + ", ";
						}
						campaigns_msg = campaigns_msg + campaign_id;
					}

					self.show_popup_message({
						messages: {
							title  : "<%=_T('Unable to change Package from Outgoing to Inbound mode.')%>",
							message: FORMAT("<%=_T('It associated to {0} campaigns of {1} Contact Center')%>", campaigns.length, vcc),
							detail : campaigns_msg,
							type   : "error"
						}
					});

				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				const error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#prov_form_package_id").focus();
				}
				else {
					err = get_error_description(error_type, err.rawData);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}
	
		if (err) {
			form_provisioning_packages.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
		}
	};

	form_provisioning_packages.prototype.get_validation_def = function()
	{
		var can_validate_outgoing_info = function() {
			const $io_mode = $("#prov_form_package_io_mode");
			const io_mode = $io_mode.val();
			if (io_mode === "I") {
				return false;
			}
			return true;
		};

		const definitions = [
			{
				control_id: "prov_form_package_id",
				messages: {
					required: "<%=_T('Please enter the Package Id.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Package Id.')%>"
				}
			},
			{
				control_id: "prov_form_package_desc",
				messages: {
					required: "<%=_T('Please enter the Package Description.')%>"
				}
			},
			{
				control_id: "prov_form_package_capacity",
				messages: {
					required: "<%=_T('Please enter the Package Capacity.')%>",
					range: {
						type   : "range",
						between: {from: 0, to: 5000},
						message: "<%=_T('Package Capacity allowed values between {0} to {1}.')%>"
					}
				}
			},
			{
				control_id: "prov_form_package_io_mode",
				messages: {
					required: "<%=_T('Please select I/O Mode.')%>"
				}
			},
			{
				control_id: "prov_form_package_did",
				messages: {
					required: "<%=_T('Please select DID.')%>"
				}
			},
			{
				control_id: "prov_form_package_outgoing_route",
				can_validate: function() {
					return can_validate_outgoing_info();
				},
				messages: {
					required: "<%=_T('Please select Outgoing Route.')%>"
				}
			},
			{
				control_id: "prov_form_package_outgoing_route_prefix",
				can_validate: function() {
					let can_validate = can_validate_outgoing_info();
					if (!can_validate) {
						return false;
					}
					can_validate = $("#prov_form_package_outgoing_route_prefix_enabled").prop("checked");
					if (!can_validate) {
						return false;
					}
					return true;
				},
				messages: {
					required: "<%=_T('Please enter Package Prefix.')%>",
					range: {
						type   : "range",
						between: {from: 1, to: 99999},
						message: "<%=_T('Package Prefix allowed values between {0} to {1}.')%>"
					}
				}
			}
		];
		return definitions;
	};

	form_provisioning_packages.prototype.ready = function()
	{
		form_provisioning_packages.uber.ready.call(this); //invokes the base class method
		this.only_modified();
	};

	form_provisioning_packages.prototype.only_modified = function()
	{
		const $id = $("#prov_form_package_id");
		if (this.is_new) {
			this.readonly_control($id, false /* disabled */ , true /* is_required */);
			$id.focus();
		}
		else {
			this.readonly_control($id, true /* disabled */);
			$("#prov_form_package_desc").focus();
		}
		// i/o mode
		this.toggle_io_mode();
	};

	form_provisioning_packages.prototype.toggle_io_mode = function()
	{
		const $outgoing_panel = $("#prov_form_package_outgoing_panel");
		const $outgoing_info = $("#prov_form_package_outgoing_info");
		const $io_mode = $("#prov_form_package_io_mode");
		const io_mode = $io_mode.val();
		const $outgoing_route = $("#prov_form_package_outgoing_route");
		const $outgoing_route_callerIDIsDID = $("#prov_form_package_outgoing_route_callerIDIsDID");

		this.toggle_outgoing_route_prefix();

		if (io_mode === "O" || io_mode === "I/O") {
		// O or I/O mode
			$outgoing_panel.show();
			$outgoing_info.hide();
			this.disable_control($outgoing_route, false, true);
			this.disable_control($outgoing_route_callerIDIsDID, false);
			return;
		}
		// I mode
		$outgoing_panel.hide();
		$outgoing_info.show();
		this.disable_control($outgoing_route, true);
		this.disable_control($outgoing_route_callerIDIsDID, true);
	};

	form_provisioning_packages.prototype.set_events = function()
	{
		const self = this;
		form_provisioning_packages.uber.set_events.call(this); //invokes the base class method

		$("#prov_form_package_io_mode").off("change")
			.on("change", function() {
				self.toggle_io_mode();
			});

		$("#prov_form_package_outgoing_route_prefix_enabled").off("change")
			.on("change", function() {
				self.toggle_outgoing_route_prefix();
			});
	};

	form_provisioning_packages.prototype.toggle_outgoing_route_prefix = function()
	{
		const $outgoing_route_prefix_enabled = $("#prov_form_package_outgoing_route_prefix_enabled");
		const $outgoing_route_prefix = $("#prov_form_package_outgoing_route_prefix");
		const $outgoing_route_prefix_replace_carrier_prefix = $("#prov_form_package_outgoing_route_prefix_replace_carrier_prefix");

		const $io_mode = $("#prov_form_package_io_mode");
		const io_mode = $io_mode.val();

		if (io_mode === "I" || io_mode === "") {
			this.disable_control($outgoing_route_prefix_enabled, true);
			this.disable_control($outgoing_route_prefix, true);
			this.disable_control($outgoing_route_prefix_replace_carrier_prefix, true);
			return;
		}

		this.disable_control($outgoing_route_prefix_enabled, false);
		const is_prefix_enabled = $outgoing_route_prefix_enabled.prop("checked");
		if (is_prefix_enabled) {
			this.disable_control($outgoing_route_prefix, false, true);
			this.disable_control($outgoing_route_prefix_replace_carrier_prefix, false);
			return;
		}
		this.disable_control($outgoing_route_prefix, true);
		this.disable_control($outgoing_route_prefix_replace_carrier_prefix, true);
	};

	form_provisioning_packages.prototype.ready_for_update = function()
	{
		form_provisioning_packages.uber.ready_for_update.call(this);
	};

	form_provisioning_packages.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const provisioning_api = inConcert.app.getProvisioningAPI();

		this.__fill_io_mode([
			{id: "I"   , desc: "<%=_T('I Mode')%>"   },
			{id: "O"   , desc: "<%=_T('O Mode')%>"   },
			{id: "I/O" , desc: "<%=_T('I/O Mode')%>" }
		]);

		this.requests = [
			// DIDs
			{
				request_function: function(success) {
					return provisioning_api.didsLoadAll(success);
				},
				success: function(list) {
					self.__fill_dids(list);
				}
			},
			// Outgoing Route
			{
				request_function: function(success) {
					return provisioning_api.routesLoadAll(success);
				},
				success: function(list) {
					self.__fill_outgoing_route(list);
				}
			},
			// Dialplan processors
			{
				request_function: function(success) {
					return provisioning_api.dialplanProcessorsLoadAll(success);
				},
				success: function(list) {
					self.__fill_dialplan_processors(list);
				}
			}
		];

		form_provisioning_packages.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	form_provisioning_packages.prototype.__fill_io_mode = function(list)
	{
		const $combo = $("#prov_form_package_io_mode");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "desc" /* attr_name */, false /* sorted */, undefined /*can_add_item*/, "trunk_type", /* data_id */);
	};

	form_provisioning_packages.prototype.__fill_dids = function(list)
	{
		const $combo = $("#prov_form_package_did");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "did", /* data_id */);
	};

	form_provisioning_packages.prototype.__fill_outgoing_route = function(list)
	{
		const $combo = $("#prov_form_package_outgoing_route");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "outgoing_route", /* data_id */);
	};

	form_provisioning_packages.prototype.__fill_dialplan_processors = function(dialplanProcessors)
	{
		const self = this;
		$("#prov_form_package_dialplan_panel").append(
			self.dialplan_call_types.map(function(callType) {
				return self.__makeCallTypeDialplanProcessors(callType, dialplanProcessors);
			})
		);
	};

	form_provisioning_packages.prototype.__makeCallTypeDialplanProcessors = function(callType, dialplanProcessors)
	{
		let $row = $(FORMAT("".concat(
			"<div class='col-lg-3'>",
				"<div class='form-group'>",
					"<label for='prov_form_package_dialplan_{0}'>",
						"{1}:",
						"<span class='fa fa-question-circle' title='", callType.description ,"'></span>",
					"</label>",
					"<select id='prov_form_package_dialplan_{0}' class='form-control' placeholder='<%=_T('Select the code…')%>'></select>",
					"<i id='prov_form_package_dialplan_{0}_validation' class='formFieldValidation formFieldError' hidden></i>",
//					"<div class='formDataDisclaimer explanation'>",
//						"{2}",
//					"</div>",
				"</div>",
			"</div>"
		), callType.id, callType.name, callType.description));
		this.fill_combo($row.find("select"), true, dialplanProcessors, "Id", "Id");
		return $row;
	};
	
}());