/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form routes class
	function form_provisioning_routes(params)
	{
		const success = form_provisioning_routes.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('New Route')%>";
		this.update_title = "<%=_T('Update Route {0}')%>";
		this.view_title = "<%=_T('View Route {0}')%>";
		this.tagline = "<%=_T('Tagline form Route!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/routes/frm/render/";
		this.save_uri =       "/inconcert/apps/provisioning/routes/save/";
		this.get_uri =        "/inconcert/apps/provisioning/routes/get/";

		this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();
		this.countries_by_code = null;
		this.sorted_countries = null;

		this.m_trunks_list = null;

		this.schedulers_mng = new inConcert.ProvisioningRouteSchedulersMng({
			owner       : this,
			prefix      : "prov_form_route",
			container_id: "prov_form_route_panel"
		});

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_routes, inConcert.conf_form);

	if (typeof inConcert.form_provisioning_routes !== "function") {
		inConcert.form_provisioning_routes = form_provisioning_routes;
	}

	form_provisioning_routes.prototype.initialize = function()
	{
		this.schedulers_mng.initialize();
	};

	form_provisioning_routes.prototype.get = function()
	{
		let route = {
			Id         : $("#prov_form_route_id").val(),
			description: $("#prov_form_route_desc").val(),
			schedulers : this.schedulers_mng.get()
		};

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			route[id] = this.$current_row.attr("row_id");
		}
		return route;
	};

	form_provisioning_routes.prototype.set = function(success, route)
	{
		if (!success) {
			return;
		}
		$("#prov_form_route_id").val(route.Id);
		$("#prov_form_route_desc").val(route.description);
		// tech specific data
		this.schedulers_mng.set(route);
	};

	form_provisioning_routes.prototype.empty_controls = function()
	{
		$("#prov_form_route_id").val("").focus();
		$("#prov_form_route_desc").val("");
	};

	form_provisioning_routes.prototype.err_save_messages = function(err, is_custom_message)
	{
		var get_error_description = function(err_type, raw_data = {})
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
			// rocka provisioningmanager errors
				case "SLOT FROM DATE IS NO IN TME FORMAT HH:MM":
					message = FORMAT("<%=_T('Slot beginning ({0}) is not in time format HH:MM.')%>", raw_data.fromDate);
				break;

				case "SLOT END DATE IS NO IN TME FORMAT HH:MM":
					message = FORMAT("<%=_T('Slot end ({0}) is not in time format HH:MM.')%>", raw_data.endDate);
				break;

				case "SLOT END IS MINOR FROM DATE":
					message = FORMAT("<%=_T('Slot ends at {0} before it begins at {1}.')%>", raw_data.endDate, raw_data.fromDate);
				break;

				case "DOW IS NOT A VALID DAY OF THE WEEK":
					message = FORMAT("<%=_T('{0} is not a valid day of the week.')%>", raw_data.dow);
				break;

				case "OVERLAP WITH AN EXISTING SLOT":
					message = "<%=_T('Overlap with an existing slot.')%>";
				break;

				case "TRUNKS PRIORITY LIST IS REQUIRED":
					message = "<%=_T('Trunks priority list is required.')%>";
				break;

				case "PARAMETER DOW MUST BE A TABLE WITH DAY VALUES NUMBERS FROM 1 TO 7":
					message = "<%=_T('Parameter days must be a list with day values (numbers from 1 to 7).')%>";
				break;

			// app provisioning errors
				case "ID_IS_REQUIRED":
					message = "<%=_T('Id is required.')%>";
				break;

				case "DESCRIPTION_IS_REQUIRED":
					message = "<%=_T('Description is required.')%>";
				break;

				case "SCHEDULERS_IS_REQUIRED":
					message = "<%=_T('Must add at least one Schedule.')%>";
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				const error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#prov_form_route_id").focus();
				}
				else {
					err = get_error_description(error_type, err.rawData);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		form_provisioning_routes.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	form_provisioning_routes.prototype.get_validation_def = function()
	{
		const self = this;

		const definitions = [
			{
				control_id: "prov_form_route_id",
				messages: {
					required: "<%=_T('Please enter the Route Id.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Route Id.')%>"
				}
			},
			{
				control_id: "prov_form_route_desc",
				messages: {
					required: "<%=_T('Please enter the Route Description.')%>"
				}
			},
			{
				control_id: this.schedulers_mng.get_grid_dom_id(),
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							return self.schedulers_mng.start_grid_validation();
						}
					}
				}
			}
		];
		return definitions;
	};

	form_provisioning_routes.prototype.ready = function()
	{
		form_provisioning_routes.uber.ready.call(this); //invokes the base class method
		this.only_modified();
	};

	form_provisioning_routes.prototype.only_modified = function()
	{
		const $id = $("#prov_form_route_id");
		if (this.is_new) {
			this.readonly_control($id, false /* disabled */ , true /* is_required */);
			$id.focus();
			this.schedulers_mng.set(); // tengo que iniciarlo en vacio
		}
		else {
			this.readonly_control($id, true /* disabled */);
			$("#prov_form_route_desc").focus();
		}
	};

	form_provisioning_routes.prototype.set_events = function()
	{
		form_provisioning_routes.uber.set_events.call(this); //invokes the base class method
		// schedulers
		this.schedulers_mng.set_events();
	};

	form_provisioning_routes.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const provisioning_api = inConcert.app.getProvisioningAPI();

		this.requests = [
			// Countries
			{
				request_function: function(success) {
					return self.telephone_codes.loadCountries(success);
				},
				success: function(countries) {
					self.__fill_countries(countries);
				}
			},
			// Routes Trunks Priority List
			{
				request_function: function(success) {
					return provisioning_api.routesLoadTrunksPriorityList(success);
				},
				success: function(list) {
					self.__fill_trunks_priority_list(list);
				}
			},
			// Trunks
			{
				request_function: function(success) {
					return provisioning_api.trunksLoadAll(success);
				},
				success: function(list) {
					self.__set_trunks(list);
				}
			},
		];

		form_provisioning_routes.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	form_provisioning_routes.prototype.__set_trunks = function(list = {})
	{
		this.m_trunks_list = list;
	};

	form_provisioning_routes.prototype.get_trunks = function()
	{
		return this.m_trunks_list;
	};

	form_provisioning_routes.prototype.__fill_countries = function(countries)
	{
		const self = this;

		var build_sorted_countries = function( countries_map ) {
			if ($.isArray(self.sorted_countries)) {
				return self.sorted_countries;
			}
			self.sorted_countries = [];
			self.countries_by_code = {};
			$.each(countries_map, function(country_id, country) {
				let translated_name = _T(country.name);
				self.sorted_countries.push({code: country_id, name: translated_name});
				country.translated_name = translated_name;
				self.countries_by_code[country_id] = country;
			});
			return inConcert.Utils.sortArray(self.sorted_countries, "name");
		};

		build_sorted_countries( countries );

		this.schedulers_mng.fill_countries();
	};

	form_provisioning_routes.prototype.__fill_trunks_priority_list = function(list)
	{
		// Ordena el hash por Nombre
		let sorted_list = inConcert.Utils.sorted_hash(
			// hash desordenado
			list,
			// funcion que retorna el valor por el cual realizar el orden
			function( item ) { return item.Name; },
			// funcion que genera el hash ordenado
			function( target_hash, item ) { target_hash[ item.Id ] = item; return target_hash; }
		);

		this.schedulers_mng.fill_trunks_priority_list(sorted_list);
	};

	form_provisioning_routes.prototype.get_countries = function()
	{
		return this.sorted_countries;
	};

	form_provisioning_routes.prototype.get_telephone_codes = function()
	{
		return this.telephone_codes;
	};

	form_provisioning_routes.prototype.get_country_info = function(country_code)
	{
		if (!this.countries_by_code) {
			return;
		}
		return this.countries_by_code[country_code];
	};

}());