/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form trunk groups class
	function form_provisioning_trunks_groups(params)
	{
		const success = form_provisioning_trunks_groups.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('New Trunk Group')%>";
		this.update_title = "<%=_T('Update Trunk Group {0}')%>";
		this.view_title = "<%=_T('View Trunk Group {0}')%>";
		this.tagline = "<%=_T('Tagline form Trunk!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/trunksgroups/frm/render/";
		this.save_uri =       "/inconcert/apps/provisioning/trunksgroups/save/";
		this.get_uri =        "/inconcert/apps/provisioning/trunksgroups/get/";

		this.m_grid_trunks_assign = null;
		this.m_all_trunks = null;
		this.m_carriers_with_trunks = null;

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_trunks_groups, inConcert.conf_form);

	if (typeof inConcert.form_provisioning_trunks_groups !== "function") {
		inConcert.form_provisioning_trunks_groups = form_provisioning_trunks_groups;
	}

	form_provisioning_trunks_groups.prototype.initialize = function()
	{
		this.__init_grid_trunks_assign();
	};

	form_provisioning_trunks_groups.prototype.get = function()
	{
		let trunk_group = {
			id     : $("#prov_form_trunk_group_id").val(),
			carrier: $("#prov_form_trunk_group_carrier").val()
		};
		// optionals
		const desc = $("#prov_form_trunk_group_description").val();
		if (desc.length > 0) {
			trunk_group.description = desc;
		}

		trunk_group.trunkList = this.__get_assigned_trunks();

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			trunk_group[id] = this.$current_row.attr("row_id");
		}
		return trunk_group;
	};

	form_provisioning_trunks_groups.prototype.set = function(success, trunk_group)
	{
		if (!success) {
			return;
		}
		$("#prov_form_trunk_group_id").val(trunk_group.id);
		$("#prov_form_trunk_group_carrier").val(trunk_group.carrier);
		// optionals
		if (trunk_group.description) {
			$("#prov_form_trunk_group_description").val(trunk_group.description);
		}
		// assigned trunks
		this.toggle_trunks();
	};

	form_provisioning_trunks_groups.prototype.empty_controls = function()
	{
		$("#prov_form_trunk_group_id").val("").focus();
		$("#prov_form_trunk_group_carrier").val("");
		// optionals
		$("#prov_form_trunk_group_description").val("");
		// assigned trunks
		this.m_grid_trunks_assign.remove_all_rows();
	};

	form_provisioning_trunks_groups.prototype.err_save_messages = function(err, is_custom_message)
	{
		var get_error_description = function(err_type, raw_data = {})
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
				case "CANNOT CHANGE TRUNK GROUP CARRIER WHILE DID IS STILL ASSOCIATED WITH IT":
					message = FORMAT("<%=_T('Cannot change trunk group {0} carrier while DID {1} is still associated with it.')%>",
						(raw_data.trunkGroup || ""),
						(raw_data.did || "")
					);
				break;
				case "CANNOT CHANGE TRUNK GROUP CARRIER WHILE TRUNK IS STILL ASSOCIATED WITH IT":
					message = FORMAT("<%=_T('Cannot change trunk group {0} carrier while trunk {1} is still associated with it.')%>",
						(raw_data.trunkGroup || ""),
						(raw_data.trunkID || "")
					);
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				const error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#prov_form_trunk_group_id").focus();
				}
				else {
					err = get_error_description(error_type, err.rawData);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		form_provisioning_trunks_groups.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	form_provisioning_trunks_groups.prototype.get_validation_def = function()
	{
		const self = this;

		const definitions = [
			{
				control_id: "prov_form_trunk_group_id",
				messages: {
					required: "<%=_T('Please enter the Trunk Group Id.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_group_carrier",
				messages: {
					required: "<%=_T('Please select Carrier.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_group_assign_paginator",
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {
								success: true
							};
							let result = self.m_grid_trunks_assign.start_validation();
							if (!result.success) {
								response.success = false;
								response.message = "<%=_T('You have to assign at least one trunk.')%>";
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};

	form_provisioning_trunks_groups.prototype.ready = function()
	{
		form_provisioning_trunks_groups.uber.ready.call(this); //invokes the base class method
		this.only_modified();
	};

	form_provisioning_trunks_groups.prototype.only_modified = function()
	{
		const $id = $("#prov_form_trunk_group_id");
		if (this.is_new) {
			this.__init_grid();
			this.readonly_control($id, false /* disabled */ , true /* is_required */);
			$id.focus();
		}
		else {
			this.readonly_control($id, true /* disabled */);
			$("#prov_form_trunk_group_carrier").focus();
		}
		this.toggle_trunks();
	};

	form_provisioning_trunks_groups.prototype.set_events = function()
	{
		const self = this;
		form_provisioning_trunks_groups.uber.set_events.call(this); //invokes the base class method

		$("#prov_form_trunk_group_carrier")
			.off("change")
			.on("change", function() {
				self.toggle_trunks();
			});
	};

	/*
		Esta funcion carga la grilla con los trunks disponibles.
		Es decir:
			1 - Los filtra segun el carrier seleccionado en el combo.
			2 - Despues agrega los carriers que no se hayan asignado a ningun otro trunk group (los libres), o si estoy modificando, el selected del propio trunk group.
	*/
	form_provisioning_trunks_groups.prototype.toggle_trunks = function()
	{
		let list = [];
		const has_trunks = inConcert.Utils.hasMapItems(this.m_all_trunks);

		if (has_trunks) {
			// Si existen carriers con trunks.
			const has_carriers_with_trunks = inConcert.Utils.hasMapItems(this.m_carriers_with_trunks);
			if (has_carriers_with_trunks) {
				// el current trunk group
				let current_trunk_group = ($("#prov_form_trunk_group_id").val() || "").toLowerCase();
				// el current carrier
				let carrier = $("#prov_form_trunk_group_carrier").val();
				// obtengo la lista de trunks para el carrier
				let carrier_with_trunks = this.m_carriers_with_trunks[carrier] || {};
				// recorro la lista de trunks del carrier
				for (let trunk_id in  carrier_with_trunks) {
					if (carrier_with_trunks.hasOwnProperty(trunk_id)) {
						// obtengo el trunk asignado
						let assigned_trunk_group = (carrier_with_trunks[trunk_id] || "").toLowerCase();
						// si el valor del trunk asignado no existe o = "", quiere decir que el trunk no ha sido asignado a ningun otro trunk group, entonces debo incluirlo en la grilla.
						// si no estoy creando uno nuevo y el assigend trunk group === a el current trunk group (el que estoy editando), debo agregarlo ya que es un selected.
						if (!assigned_trunk_group || (!this.is_creating() && assigned_trunk_group === current_trunk_group)) {
							let trunk_info = this.m_all_trunks[trunk_id];
							if (trunk_info) {
								list.push(trunk_info);
							}
						}
					}
				}
			}
		}

		this.__init_grid(list);
	};

	form_provisioning_trunks_groups.prototype.__init_grid = function(list = [])
	{
	 	this.m_grid_trunks_assign.fill_rows($.noop, list);
	};

	form_provisioning_trunks_groups.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const provisioning_api = inConcert.app.getProvisioningAPI();

		this.requests = [
			// Carriers
			{
				request_function: function(success) {
					return provisioning_api.carriersLoadAll(success);
				},
				success: function(list) {
					self.__fill_carriers(list);
				}
			},
			// Trunks all
			{
				request_function: function(success) {
					return provisioning_api.trunksLoadAll(success);
				},
				success: function(list) {
					self.__set_all_trunks(list);
				}
			},
			// Carriers with trunks
			{
				request_function: function(success) {
					return provisioning_api.carriersLoadWithTrunks(success);
				},
				success: function(list) {
					self.__set_carriers_with_trunks(list);
				}
			}
		];

		form_provisioning_trunks_groups.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	form_provisioning_trunks_groups.prototype.__fill_carriers = function(list)
	{
		const $combo = $("#prov_form_trunk_group_carrier");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "carrier", /* data_id */);
	};

	form_provisioning_trunks_groups.prototype.__set_all_trunks = function(list)
	{
		this.m_all_trunks = list;
	};

	form_provisioning_trunks_groups.prototype.__set_carriers_with_trunks = function(list)
	{
		this.m_carriers_with_trunks = list;
	};

	form_provisioning_trunks_groups.prototype.__init_grid_trunks_assign = function()
	{
		const self = this;

		const is_selected_trunk = function(trunk) {
			const trunk_group = self.get_entity_instance();
			if (!trunk_group || !trunk_group.trunkList) {
				return false;
			}
			const current_trunk_id = (trunk.id || "");
			const is_selected = trunk_group.trunkList.hasOwnProperty(current_trunk_id);
			return is_selected;
		};

		this.m_grid_trunks_assign = new inConcert.conf_form_paginator({
			owner: this,
			frm: this,
			auto_initialize: true,
			row_id: "id",
			display_name: "id",
			rows_per_page: 2000,
			show_at_least: 1999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Trunks ...')%>",
			ids: {
				paginator: "prov_form_trunk_group_assign_paginator",
				grid: "prov_form_trunk_group_assign_list",
				not_results_found: "prov_form_trunk_group_assign_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Trunks found.')%>"
			},
			columns: [
				{id: "_select", name: "", type: "row_selector",
					is_checked: function(trunk) {
						return is_selected_trunk(trunk);
					}
				},
				{id: "id"     , name: "<%=_T('Trunk Id')%>"  },
				{id: "hostId" , name: "<%=_T('Host Id')%>"   },
				{id: "codecs" , name: "<%=_T('Codecs')%>"    },
				{id: "tech"   , name: "<%=_T('Technology')%>"},
				{
					id: "_action",
					name: "",
					system_action: {}
				}
			],
			callbacks: {},
			validate: {
				required: true,
				selected: true
			}
		});
	};

	form_provisioning_trunks_groups.prototype.__get_assigned_trunks = function()
	{
		const selected_trunks = this.m_grid_trunks_assign.get_selected_items();
		let trunks_to_include = [];

		for (let i = 0; i < selected_trunks.length; i++) {
			let selected_trunk = selected_trunks[i];
			trunks_to_include.push({
				TrunkId : selected_trunk.id
			});
		}

		return trunks_to_include;
	};

}());