/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const FORMAT = inConcert.Utils.format;

	const m_peer_types = [
		{ id : "agent", name : "<%=_T('Agent IP Phones')%>" },
		{ id : "client", name : "<%=_T('Client IP Phones')%>" }
	];

	//---------------------------------------------------------------------
	// form vcc class - teli6 peers & packages section
	function form_provisioning_vccs_section_omnichannel_telephony( params )
	{
		const self = this;
		const success = form_provisioning_vccs_section_omnichannel_telephony.uber.constructor.call( this, params );
		if (!success) {
			return;
		}

		this.view_action_id = "view_vccs";  // view action id de permisos
		this.save_action_id  = {create: "save_vcc", update: "save_vcc"}; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos

		// tabs (agent_peers o client_peers)
		const peers_tabs = m_peer_types.reduce((acc, type, index) => {
			const tabId = `${type.id}_peers`;
			return Object.assign(acc, {
				[tabId] : {
					id              : tabId,
					name            : type.name,
					tab_container_id: `${type.id}_prov_form_vcc_omnichannel_telephony_peers_tab_container`,
					selected        : index === 0,
					on_show         : function() {
						self.on_show_tab(tabId);
					}
				}
			});
		}, {});

		// Instancia la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : "prov_form_vcc_omnichannel_telephony_tabs",
			must_intialize: false,
			// al principio se agregan los tabs (agent_peers o client_peers)
			tabs_list   : Object.assign(peers_tabs, {
				packages: {
					id              : "packages",
					name            : "<%=_T('Packages')%>",
					tab_container_id: "prov_form_vcc_omnichannel_telephony_packages_tab_container",
					on_show         : function() {
						self.on_show_tab("packages");
					}
				},
				ttsasr: {
					id              : "ttsasr",
					name            : "<%=_T('TTS/ASR')%>",
					tab_container_id: "prov_form_vcc_omnichannel_telephony_ttsasr_container"
				}
			})
		});

		this.m_grid_peers_assign = {};
		this.m_peers_mng = {};
		this.m_grid_packages_assign = null;

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_vccs_section_omnichannel_telephony, inConcert.conf_form_section);

	if ( typeof inConcert.form_provisioning_vccs_section_omnichannel_telephony !== "function" ) {
		inConcert.form_provisioning_vccs_section_omnichannel_telephony = form_provisioning_vccs_section_omnichannel_telephony;
	}

	form_provisioning_vccs_section_omnichannel_telephony.prototype.init = function()
	{
		m_peer_types.forEach(type => this.__init_grid_peers_assign(type.id));
		this.__init_grid_packages_assign();
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__grid_peers = function(type)
	{
		if (!type) {
			throw("form_provisioning_vccs_section_omnichannel_telephony.prototype.__grid_peers - type is required");
		}
		const grid = this.m_grid_peers_assign[type];
		if (!grid) {
			throw("form_provisioning_vccs_section_omnichannel_telephony.prototype.__grid_peers - unrecognized grid type" + type);
		}
		return grid;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_peers_grid = function(list = [])
	{
		m_peer_types.forEach(type => {
			const peerList = list.filter(peer => !peer.Assigned || peer.Assigned === type.id);
			let grid = this.__grid_peers(type.id);
			grid.fill_rows($.noop, peerList);
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__reload_peers = function() {
		inConcert.app.getProvisioningAPI().vccPeersLoadAll(this.frm.get_entity_id(), (success, list) => success && this.fill_peers_grid(list));
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_packages_grid = function(list = [])
	{
		this.m_grid_packages_assign.fill_rows($.noop, list);
		if (list.length === 0) {
			$("#prov_form_vcc_omnichannel_telephony_packages_close").text("<%=_T('Close')%>");
		}
	};
	
	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_TtsEngines = function(engines = {})
	{
		this.fill_combo(
			$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine"),
			true,          // is_blank_first_item
			engines,       // list
			"id",          // attr_id
			"description", // attr_name
			true,          // sorted
			undefined,     // can_add_item
			"id"           // data_id
		);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_AsrEngines = function(engines = {})
	{
		this.fill_combo(
			$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine"), 
			true,          // is_blank_first_item
			engines,       // list
			"id",          // attr_id
			"description", // attr_name
			true,          // sorted
			undefined,     // can_add_item
			"id"           // data_id
		);
	};
	
	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_TtsProfiles = function(profiles = {})
	{
		this.fill_combo(
			$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile"), 
			true,          // is_blank_first_item
			profiles,      // list
			"id",          // attr_id
			"description", // attr_name
			true,          // sorted
			undefined,     // can_add_item
			"id"           // data_id
		);
	};
	
	form_provisioning_vccs_section_omnichannel_telephony.prototype.fill_AsrProfiles = function(profiles = {})
	{
		this.fill_combo(
			$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile"), 
			true,          // is_blank_first_item
			profiles,      // list
			"id",          // attr_id
			"description", // attr_name
			true,          // sorted
			undefined,     // can_add_item
			"id"           // data_id
		);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__init_grid_peers_assign = function(type)
	{
		const self = this;
		let base_prefix = `${type}_prov_form_vcc_omnichannel_telephony_peers`;
		let assign_prefix_id = `${base_prefix}_assign`;
		let assign_popup_prefix_id = `${assign_prefix_id}_popup`;
		let create_prefix_id = `${base_prefix}_create`;
		let create_popup_prefix_id = `${create_prefix_id}_popup`;

		this.m_grid_peers_assign[type] = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			auto_initialize: true,
			row_id: "Id",
			display_name: "Id",
			rows_per_page: 10,
			show_at_least: 9,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading IP Phones ...')%>",
			not_hide_third_button: true,
			order_info: {
				options: {
					order: "asc"
				},
				sort_value_fn: function(row) {
					return Number(row.Name);
				}
			},
			ids: {
				// paginator
				paginator: `${assign_prefix_id}_paginator`,
				grid: `${assign_prefix_id}_list`,
				not_results_found: `${assign_prefix_id}_not_results_found`,
				// search
				search_builtin_panel: `${assign_prefix_id}_search_panel`,
				search_container    : `${assign_prefix_id}_search_container`,
				search_text         : `${assign_prefix_id}_search_text`,
				search_reset        : `${assign_prefix_id}_reset_text`,
				search_apply        : `${assign_prefix_id}_apply_text`,

				// header actions
				container_new_btn: `${assign_prefix_id}_actions_container`,

				// popup0 (asignacion y desasignacion de peers en batch)
				new_item: `${assign_prefix_id}_select`,
				popup: assign_popup_prefix_id,
				popup_msg: `${assign_popup_prefix_id}_message`,
				popup_create: `${assign_popup_prefix_id}_create`,
				popup_create_and_continue: `${assign_popup_prefix_id}_create_and_continue`,
				popup_title: `${assign_popup_prefix_id}_title`,
				popup_close: `${assign_popup_prefix_id}_close`,
				popup_input_id: `${assign_popup_prefix_id}_question`,
				popup_waiting: `${assign_popup_prefix_id}_waiting`,

				// //popup1 (crear nuevos peers)
				new_item1: `${assign_prefix_id}_create`,
				popup1: create_popup_prefix_id,
				popup1_msg: `${create_popup_prefix_id}_message`,
				popup1_create: `${create_popup_prefix_id}_create`,
				popup1_create_and_continue: `${create_popup_prefix_id}_create_and_continue`,
				popup1_title: `${create_popup_prefix_id}_title`,
				popup1_close: `${create_popup_prefix_id}_close`,
				popup1_input_id: `${create_popup_prefix_id}_question`,
				popup1_waiting: `${create_popup_prefix_id}_waiting`
			},
			strings: {
				not_results_found: "<%=_T('No IP Phones were found or not exist free IP Phones.')%>",
				// popup0 (asignacion y desasignacion de peers en batch)
				title_new                  : "<%=_T('Assign IP Phones')%>",
				title_update               : "<%=_T('Unassign IP Phones')%>",
				button_create              : "<%=_T('Confirm')%>",
				button_create_and_continue : "<%=_T('Confirm & continue')%>",
				button_update              : "<%=_T('Confirm')%>",

				//popup1 (crear nuevos peers)
//				not_results_found1         : "<%=_T('No Prebuilt Entities found.')%>",
//				can_remove1                : "<%=_T('Do you want to delete Prebuilt Entity: {0} ?')%>",
				title_new1                 : "<%=_T('Create Phones')%>",
				button_create_and_continue1: "<%=_T('Confirm & continue')%>",
				button_update1             : "<%=_T('Confirm')%>"
			},
			columns: [
				{id: "_select", name: "", type: "row_selector" , cell_class_name: "checkboxColumn",
					is_checked: function(peer) {
						return peer.Assigned === type;
					}
				},
				{id: "Tech"  , name: "<%=_T('Technology')%>"  },
				{id: "Name"  , name: "<%=_T('IP Phone')%>"    },
				{id: "HostId", name: "<%=_T('Host Id')%>"     },
				{
					id: "_action",
					name: "",
					system_action: {}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					if (current_popup === 0) {
						self.assign_or_unassign_peers(type, must_create_and_contiune, current_popup, callback);
					}
					else {
						self.create_peers(type, must_create_and_contiune, current_popup, callback);
					}
				},
				edit: function(_, current_popup) {
					if (current_popup === 0) {
						self.on_open_popup_assign_peers(type);
					}
					else {
						self.on_open_popup_create_peers(type);
					}
				},
				on_loaded_items: function(/*callback*/) {
					self.update_grouped_detail_of_peers(type);
				},
				on_selected_row: function(/*$chk, is_selected*/) {
					self.update_grouped_detail_of_peers(type);
				},
				on_selected_all_row: function(/*is_selected*/) {
					self.update_grouped_detail_of_peers(type);
				}
			}
		});

		this.m_peers_mng[type] = new inConcert.ProvisioningPeersMng({
			frm         : this.frm,
			prefix      : `${base_prefix}_create_popup`,
			container_id: `${base_prefix}_create_popup_all_data_panel`
		});
		this.m_peers_mng[type].initialize();
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__init_grid_packages_assign = function()
	{
		var is_selected = function(pkg) {
			if (pkg.IsAssigned) {
				return true;
			}
			return false;
		};

		this.m_grid_packages_assign = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			auto_initialize: true,
			row_id: "Id",
			display_name: "Id",
			rows_per_page: 2000,
			show_at_least: 1999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Packages ...')%>",
			ids: {
				paginator: "prov_form_vcc_omnichannel_telephony_packages_assign_paginator",
				grid: "prov_form_vcc_omnichannel_telephony_packages_assign_list",
				not_results_found: "prov_form_vcc_omnichannel_telephony_packages_assign_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No packages were found or not exist free packages.')%>"
			},
			columns: [
				{id: "_select", name: "", type: "row_selector", cell_class_name: "checkboxColumn",
					is_checked: function(pkg) {
						return is_selected(pkg);
					}
				},
				{id: "Id", name: "<%=_T('Package Id')%>" },
				{
					id: "_action",
					name: "",
					system_action: {}
				}
			],
			callbacks: {}
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.set_events = function() {
		const self = this;
		form_provisioning_vccs_section_omnichannel_telephony.uber.set_events.call(this); //invokes the base class method

		// PEERS
		m_peer_types.forEach(type => {
			const $unselect = $(`#${type.id}_prov_form_vcc_omnichannel_telephony_peers_assign_unselect`);
			$unselect
				.off("click")
				.on("click", function() {
					/*
						El tener 2 acciones desde el header de grilla, sobre el mismo popup no esta contemplado en la clase base de la grilla.
						Por este motivo, hago esta pisada aca, para poder levantar el popup de Asignar/Desasignar peers.
						TODO: Revisar de agregar este codigo en la misma clase "conf_form_paginator".
					*/
					let grid = self.__grid_peers(type.id);
					grid.set_first_row();
					grid.show_popup(0);
				});

			// add peers popups
			self.m_peers_mng[type.id].set_events();
		});

		// PACKAGES
		//

		// TTS/ASR
		//
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine")
			.change(function() {
				// Load TTS Profiles combo
				var selectedEngineId = $( this ).val();
				var selectedEngineProfiles = [];
				self.ttsEngines.forEach(function(engine) {
					if (engine.id === selectedEngineId) {
						selectedEngineProfiles = engine.profiles;
					}
				});
				self.fill_TtsProfiles(selectedEngineProfiles);
				$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").change();
			});

		$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine")
			.change(function() {
				// Load ASR Profiles combo
				var selectedEngineId = $( this ).val();
				var selectedEngineProfiles = [];
				self.asrEngines.forEach(function(engine) {
					if (engine.id === selectedEngineId) {
						selectedEngineProfiles = engine.profiles;
					}
				});
				self.fill_AsrProfiles(selectedEngineProfiles);
				$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").change();
			});

		$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile")
			.change(function() {
				if (self._stopPropagating) {
					return;
				}
				var selectedEngineId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").val();
				var selectedProfileId = $( this ).val();
				if (selectedProfileId) {
					self.ttsEngines.forEach(function(engine) {
						if (engine.id === selectedEngineId) {
							engine.profiles.forEach(function(profile) {
								if (profile.id === selectedProfileId) {
									if (profile.providerType === "TTS-ASR") {
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine"), true);
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile"), true);
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").val(engine.id);
										self._stopPropagating = true;
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").change();
										self._stopPropagating = false;
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").val(profile.id);
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text(profile.providerType);
									}
									else {
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine"), false);
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile"), false);
										//Si en ASR tenia un profile TTS-ASR, reinicializamos tanto el asrEngine como el asrProfile
										if ($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text() === "TTS-ASR"){
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").val("");
											self._stopPropagating = true;
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").change();
											self._stopPropagating = false;
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").val("");
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text("");
										}
									}
									$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text(profile.providerType);
								}
							});
						}
					});
				}
				else {
					self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine"), false);
					self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile"), false);
					if ($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text() === "TTS-ASR") {
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").val("");
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").val("");
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text("");
					}
					$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text("");
				}
			});

		$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile")
			.change(function() {
				if (self._stopPropagating) {
					return;
				}
				var selectedEngineId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").val();
				var selectedProfileId = $( this ).val();
				if (selectedProfileId) {
					self.asrEngines.forEach(function(engine) {
						if (engine.id === selectedEngineId) {
							engine.profiles.forEach(function(profile) {
								if (profile.id === selectedProfileId) {
									if (profile.providerType === "TTS-ASR") {
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine"), true);
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile"), true);
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").val(engine.id);
										self._stopPropagating = true;
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").change();
										self._stopPropagating = false;
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").val(profile.id);
										$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text(profile.providerType);
									}
									else {
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine"), false);
										self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile"), false);
										//Si en TTS tenia un profile TTS-ASR, reinicializamos tanto el ttsEngine como el ttsProfile
										if ($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text() === "TTS-ASR"){
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").val("");
											self._stopPropagating = true;
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").change();
											self._stopPropagating = false;
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").val("");
											$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text("");
										}
									}
									$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text(profile.providerType);
								}
							});
						}
					});
				}
				else {
					self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine"), false);
					self.disable_control($("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile"), false);
					if ($("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text() === "TTS-ASR") {
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").val("");
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").val("");
						$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text("");
					}
					$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text("");
				}
			});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.attach_load_request = function(request)
	{
		const self = this;

		if (!$.isArray(request)) {
			return;
		}

		const provisioning_api = inConcert.app.getProvisioningAPI();
		request.push(
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.vccPeersLoadAll(vcc_id, success);
				},
				success: function(response) {
					self.fill_peers_grid(response);
				}
			},
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.vccPackagesLoadAll(vcc_id, success);
				},
				success: function(response) {
					self.fill_packages_grid(response);
				}
			},
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.ttsasrEnginesVCCLoad(vcc_id, success);
				},
				success: function(response) {
					self.ttsasrEnginesVCC = response;
				}
			},
			{
				request_function: function(success) {
					return provisioning_api.ttsasrEnginesProvisioningLoadAll(success);
				},
				success: function(response) {
					self.ttsasrEnginesProvisioning = response;
				}
			},
			{
				request_function: function(success) {
					return provisioning_api.enginesLoadAllProvisionedTtsProfiles(success);
				},
				success: function(response) {
					self.ttsEngines = response;
					self.fill_TtsEngines(response);
				}
			},
			{
				request_function: function(success) {
					return provisioning_api.enginesLoadAllProvisionedAsrProfiles(success);
				},
				success: function(response) {
					self.asrEngines = response;
					self.fill_AsrEngines(response);
				}
			}
		);

		m_peer_types.forEach(type => {
			let peers_request = this.m_peers_mng[type.id].load_data();
			for (let i = 0; i < peers_request.length; i++) {
				request.push(peers_request[i]);
			}
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.editable = function()
	{
		form_provisioning_vccs_section_omnichannel_telephony.uber.editable.call( this );
		this.frm.editable_buttons();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();

		const force_new = true;
		m_peer_types.forEach(type => {
			this.m_peers_mng[type.id].ready(force_new); //only_modified();
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.readonly = function()
	{
		form_provisioning_vccs_section_omnichannel_telephony.uber.readonly.call( this );
		this.frm.readonly_buttons();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();

		const force_new = true;
		m_peer_types.forEach(type => {
			this.m_peers_mng[type.id].ready(force_new); //only_modified();
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.on_show = function()
	{
		form_provisioning_vccs_section_omnichannel_telephony.uber.on_show.call(this, {show: false});
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").change(); //Para el caso de que tengamos un profile TTS-ASR seleccionado, deshabilitar la parte de asr
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__get_assigned_peers = function(type)
	{
		const selected_peers = this.__grid_peers(type).get_selected_items();
		let peers_to_include = [];

		for (let i = 0; i < selected_peers.length; i++) {
			let selected_peer = selected_peers[i];
			peers_to_include.push({Id: selected_peer.Id});
		}

		return peers_to_include;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__get_assigned_packages = function()
	{
		const selected_packages = this.m_grid_packages_assign.get_selected_items();
		let packages_to_include = [];

		for (let i = 0; i < selected_packages.length; i++) {
			let selected_package = selected_packages[i];
			packages_to_include.push({
				Id: selected_package.Id
			});
		}

		return packages_to_include;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__make_warning_detail = function(warnings, attr_id, detail = "")
	{
		for (let i = 0; i < (warnings || []).length; i++) {
			let warning = warnings[i];
			let item = warning.rawData[attr_id];

			if (warning.code !== "PACKAGE IS ASSIGNED TO SOME VCC CAMPAIGNS") {
				if (detail) {
					detail = detail + ", ";
				}
				detail = detail + item;
			}
			else {
				let i = 0;
				let campaigns = "";
				for (let campaign_id in (warning.rawData.campaigns || {})) {
					if (warning.rawData.campaigns.hasOwnProperty(campaign_id)) {
						i++;
						if (campaigns) {
							campaigns = campaigns + ", ";
						}
						campaigns = campaigns + campaign_id;
					}
				}
				if (campaigns) {
					let campaigns_detail;
					if (i > 1) {
						campaigns_detail = FORMAT("<%=_T('{0} is assigned the campaigns: {1}')%>", item, campaigns);
					}
					else {
						campaigns_detail = FORMAT("<%=_T('{0} is assigned the campaign: {1}')%>", item, campaigns);
					}
					if (detail) {
						detail = detail + "\n\n";
					}
					detail = detail + campaigns_detail;
				}
			}
		}

		return detail;
	};

	/*
		Funcion que realiza rollback de los estados de los checkboxes que hayan fallado su actualizacion en las grillas de Client y Agent Peers y en tambien la de Packages.
		Ej:
			Si se habia marcado un Peer para asignarlo al VCC y el mismo no se pudo asignar, en checkbox vuelve al estado original.
	*/
	form_provisioning_vccs_section_omnichannel_telephony.prototype.__grid_rollback_conflicts = function(grid_obj, warnings, attr_id, is_selected)
	{
		for (let i = 0; i < (warnings || []).length; i++) {
			let warning = warnings[i];
			let item_id = warning.rawData[attr_id];
			grid_obj.select_row(item_id, is_selected);
		}
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.peers_update_vcc = function(type, callback = $.noop)
	{
		const self = this;

		const data = {
			vcc: this.frm.get_entity_id(),
			assignationType: type
		};

		// asigno datos
		data.peers = this.__get_assigned_peers(type);

		this.show_waiting("<%=_T('Please wait, Updating Vcc IP Phones Information ...')%>");

		const provisioning_api = inConcert.app.getProvisioningAPI();
		// Se ejecuta en un pequeno timeout ya que la ejecucion puede ser muy rapida!! asi tenemos feedback
		window.setTimeout(function() {
			provisioning_api.vccPeersUpdate(data,
				function(success, response)
				{
					self.hide_waiting();
					let message, msg_type;
					if (success) {
						if (response.code === "ALL PEERS UPDATED") {
							message = "<%=_T('Vcc IP Phones has been successfully updated.')%>";
							msg_type = "Info";
						}
						else { // PARTIAL UPDATED PEERS
							// muestra mensaje en un popup con los conflictos de los peers que no se han podido asignar o quitar.
							let warning_list = response.warningList || {};
							let detail = self.__make_warning_detail(warning_list.assign, "peerID");
							detail = self.__make_warning_detail(warning_list.remove, "peerID", detail);
							self.frm.show_popup_message({
								messages: {
									title  : "<%=_T('Not all IP Phones have been updated.')%>",
									message: "<%=_T('The IP Phones that could not be updated are')%>" + "<%=_T(':')%>",
									detail : detail,
									type   : "warn"
								}
							});
							// realiza rollback en la grilla de los peers (check/uncheck) que no se han podido asignar o quitar.
							const grid_obj = self.__grid_peers(type);
							self.__grid_rollback_conflicts(grid_obj, warning_list.assign, "peerID", false);
							self.__grid_rollback_conflicts(grid_obj, warning_list.remove, "peerID", true);
							// invoco al callback para que cancele la ejecucion de los Requests encadenados
							callback(false);
							return;
						}
					}
					else {
						message = "<%=_T('Error updating Vcc IP Phones.')%>";
						msg_type = "Error";
						self.show_status_message(message, msg_type, 5000);
					}
					// invoco al callback para que continue o no con los Requests encadenados
					callback(success, {
						msg_type: msg_type,
						message : message
					});
				}
			);
		}, 500);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.packages_update_vcc = function(callback = $.noop)
	{
		const self = this;

		const data = {
			vcc: this.frm.get_entity_id()
		};

		// asigno datos
		data.packages = this.__get_assigned_packages();

		this.show_waiting("<%=_T('Please wait, Updating Vcc Packages Information ...')%>");

		const provisioning_api = inConcert.app.getProvisioningAPI();
		// Se ejecuta en un pequeno timeout ya que la ejecucion puede ser muy rapida!! asi tenemos feedback
		window.setTimeout(function() {
			provisioning_api.vccPackagesUpdate(data,
				function(success, response)
				{
					self.hide_waiting();
					let message, msg_type;
					if (success) {
						if (response.code === "ALL PACKAGES UPDATED") {
							message = "<%=_T('Vcc Packages has been successfully updated.')%>";
							msg_type = "Info";
						}
						else { // PARTIAL UPDATED PACKAGES
							// muestra mensaje en un popup con los conflictos de los packages que no se han podido asignar o quitar.
							let warning_list = response.warningList || {};
							let detail = self.__make_warning_detail(warning_list.assign, "packageID");
							detail = self.__make_warning_detail(warning_list.remove, "packageID", detail);
							self.frm.show_popup_message({
								messages: {
									title  : "<%=_T('Not all Packages have been updated.')%>",
									message: "<%=_T('The Packages that could not be updated are')%>" + "<%=_T(':')%>",
									detail : detail,
									type   : "warn"
								}
							});
							// realiza rollback en la grilla de los packages (check/uncheck) que no se han podido asignar o quitar.
							const grid_obj = self.m_grid_packages_assign;
							self.__grid_rollback_conflicts(grid_obj, warning_list.assign, "packageID", false);
							self.__grid_rollback_conflicts(grid_obj, warning_list.remove, "packageID", true);
							// invoco al callback para que cancele la ejecucion de los Requests encadenados
							callback(false);
							return;
						}
					}
					else {
						message = "<%=_T('Error updating Vcc Packages.')%>";
						msg_type = "Error";
						self.show_status_message(message, msg_type, 5000);
					}
					// invoco al callback para que continue o no con los Requests encadenados
					callback(success, {
						msg_type: msg_type,
						message : message
					});
				}
			);
		}, 500);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.ttsasr_update_vcc = function(callback = $.noop)
	{
		const self = this;

		const data = {
			vcc: this.frm.get_entity_id()
		};

		// asigno datos
		let ttsEngineId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").val();
		let ttsProfileId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").val();
		let asrEngineId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").val();
		let asrProfileId = $("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").val();
		let confirmation_message = "<%=_T('{0} custom settings for vcc {1} will be lost')%>";
		let provider_type = "";
		let bShowConfirm = false;

		if (
			(this.ttsasrEnginesVCC.ttsEngine && this.ttsasrEnginesVCC.ttsEngine !== ttsEngineId) || 
			(this.ttsasrEnginesVCC.ttsProfile && this.ttsasrEnginesVCC.ttsProfile !== ttsProfileId)) {
				provider_type = $("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_provider_type").text();
				bShowConfirm = true;
		}
		if (provider_type !== "TTS-ASR") {
			if (
				(this.ttsasrEnginesVCC.asrEngine && this.ttsasrEnginesVCC.asrEngine !== asrEngineId) ||
				(this.ttsasrEnginesVCC.asrProfile && this.ttsasrEnginesVCC.asrProfile !== asrProfileId)) {
				provider_type += (provider_type === "" ? "" : "-") + $("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_provider_type").text();
				bShowConfirm = true;
			}
		}

		if (bShowConfirm && provider_type === "") {
			provider_type = "TTS-ASR";
		}
		confirmation_message = FORMAT(confirmation_message, provider_type, data.vcc);

		const execute_save = function() {
			let ttsasrEngines = [];
			let message, msg_type;

			if (ttsEngineId && !ttsProfileId) {
				message = "<%=_T('Select a TTS Profile.')%>";
				msg_type = "Error";
				self.show_status_message(message, msg_type, 5000);
				// invoco al callback para que cancele la ejecucion de los Requests encadenados
				callback(false);
				return;
			}
			if (asrEngineId && !asrProfileId) {
				message = "<%=_T('Select an ASR Profile.')%>";
				msg_type = "Error";
				self.show_status_message(message, msg_type, 5000);
				// invoco al callback para que cancele la ejecucion de los Requests encadenados
				callback(false);
				return;
			}
			let bSavedTtsAsrProfile = false;
			let ttsEngine = self.ttsasrEnginesProvisioning[ttsEngineId];
			if (ttsEngine){
				ttsEngine.profiles.some(function(profile) {
					if (profile.id === ttsProfileId) {
						bSavedTtsAsrProfile = profile.provider_type === "TTS-ASR";
						let grammars = profile.provider_type === "TTS-ASR" ? profile.provider.grammars : [];
						let recognitionLanguages = profile.provider_type === "TTS-ASR" ? profile.provider.recognitionLanguages : [];
						let ttsasrEngine = {
							Id : ttsEngine.id + "_" + profile.id,
							engineId : ttsEngine.id,
							engineDesc : ttsEngine.description,
							profileId: profile.id,
							profileDesc: profile.description,
							providerId: profile.provider.id,
							providerDesc: profile.provider.name,
							providerType: profile.provider_type,
							endpoint: profile.endpoint,
							voices : profile.provider.voices,
							grammars: grammars,
							recognitionLanguages: recognitionLanguages
						};
						ttsasrEngines.push(ttsasrEngine);
						return true;
					}
				});
				data.ttsEngine = ttsEngineId;
				data.ttsProfile = ttsProfileId;
			}
			else {
				data.ttsEngine = "";
				data.ttsProfile = "";
			}
			//Si se selecciono un profile con provider_type = "TTS-ASR", ya no guardo el asrEngine
			if (!bSavedTtsAsrProfile){
				let asrEngine = self.ttsasrEnginesProvisioning[asrEngineId];
				if (asrEngine){
					asrEngine.profiles.some(function(profile) {
						if (profile.id === asrProfileId) {
							let voices = profile.provider_type === "TTS-ASR" ? profile.provider.voices : [];
							let ttsasrEngine = {
								Id : asrEngine.id + "_" + profile.id,
								engineId : asrEngine.id,
								engineDesc : asrEngine.description,
								profileId: profile.id,
								profileDesc: profile.description,
								providerId: profile.provider.id,
								providerDesc: profile.provider.name,
								providerType: profile.provider_type,
								endpoint: profile.endpoint,
								voices : voices,
								grammars: profile.provider.grammars,
								recognitionLanguages: profile.provider.recognitionLanguages
							};
							ttsasrEngines.push(ttsasrEngine);
							return true;
						}
					});
					data.asrEngine = asrEngineId;
					data.asrProfile = asrProfileId;
				}
				else {
					data.asrEngine = "";
					data.asrProfile = "";
				}
			}
			else {
				data.asrEngine = "";
				data.asrProfile = "";
			}

			data.ttsasrEngines = ttsasrEngines;
			self.show_waiting("<%=_T('Please wait, Updating Vcc TTS/ASR configuration Information ...')%>");

			const provisioning_api = inConcert.app.getProvisioningAPI();
			// Se ejecuta en un pequeno timeout ya que la ejecucion puede ser muy rapida!! asi tenemos feedback
			window.setTimeout(function() {
				provisioning_api.vccTtsAsrUpdate(data,
					function(success)
					{
						self.hide_waiting();
						let message, msg_type;
						if (success) {
							message = "<%=_T('Vcc TTS/ASR configuration has been successfully updated.')%>";
							msg_type = "Info";
							self.ttsasrEnginesVCC.ttsEngine = ttsEngineId;
							self.ttsasrEnginesVCC.ttsProfile = ttsProfileId;
							self.ttsasrEnginesVCC.asrEngine = asrEngineId;
							self.ttsasrEnginesVCC.asrProfile = asrProfileId;
						}
						else {
							message = "<%=_T('Error updating Vcc TTS/ASR configuration.')%>";
							msg_type = "Error";
							self.show_status_message(message, msg_type, 5000);
						}
						// invoco al callback para que continue o no con los Requests encadenados
						callback(success, {
							msg_type: msg_type,
							message : message
						});
					}
				);
			}, 500);
		};

		if (bShowConfirm) {
			// Si los datos seleccionados han cambiado respecto de los que se cargaron, mostrar mensaje de advertencia
			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				confirmation_message,
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						// invoco al callback para que cancele la ejecucion de los Requests encadenados
						callback(false);
						return;
					}
					execute_save();
				},
				undefined,
				{
					popup_id       : "conf_inconcertPopup",
					container_class: "confirmationPopUp",
					overlay_class  : "confirmationOverlay"
				}
			);
		}
		else {
			execute_save();
		}
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.on_show_tab = function(/*tab_id*/)
	{
		//TODO: hacer algo si llega a ser necesario!!
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.set = function()
	{
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_engine").focus().val(this.ttsasrEnginesVCC.ttsEngine).change();
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_tts_profile").focus().val(this.ttsasrEnginesVCC.ttsProfile);
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_engine").focus().val(this.ttsasrEnginesVCC.asrEngine).change();
		$("#prov_form_vcc_omnichannel_telephony_ttsasr_asr_profile").focus().val(this.ttsasrEnginesVCC.asrProfile).change();

		m_peer_types.forEach(type => {
			this.m_peers_mng[type.id].set(true, {});
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.assign_or_unassign_peers = function(type, _, current_popup, callback = $.noop)
	{
		let response = this.validate_assign_or_unassign_peers(type, current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		let msg_response = {
			msg : "",
			type: "Info"
		};

		let grid = this.__grid_peers(type);
		let total_numbers = response.item.total_numbers + (response.item.first_number - 1);
		let template_peer_id = "{0}_{1}";
		let total_applied = 0;
		let total_failed = 0;
		let action_type  = !grid.$get_edited_row() ? "ASSIGNING" : "UNASSIGNIG";
		let is_assigning = action_type === "ASSIGNING" ? true: false;

		for (let i = response.item.first_number; i <= total_numbers; i++) {
			let peer_id = FORMAT(template_peer_id, "SIP", i);
			let is_ok = grid.select_row(peer_id, is_assigning);
			if (is_ok) {
				total_applied++;
			}
			else {
				total_failed++;
			}
		}

		let messages_by_action = {
			ASSIGNING: {
				ALL    : "<%=_T('All IP Phones have been selected.')%>",
				PARTIAL: "<%=_T('{0} IP Phones have been selected, {1} could not be selected.')%>",
				NONE   : "<%=_T('No IP Phones have been selected.')%>"
			},
			UNASSIGNIG: {
				ALL    : "<%=_T('All IP Phones have been unselected.')%>",
				PARTIAL: "<%=_T('{0} IP Phones have been unselected, {1} could not be unselected.')%>",
				NONE   : "<%=_T('No IP Phones have been unselected.')%>"
			}
		};

		if (total_applied > 0 && total_failed === 0) {
			msg_response.msg = messages_by_action[action_type].ALL;
			let must_add_many = false;
			callback(false, response.item, msg_response, must_add_many, 3000);
		}
		else if (total_applied > 0 && total_failed > 0) {
			msg_response.msg = FORMAT(messages_by_action[action_type].PARTIAL, total_applied, total_failed);
			let must_add_many = false;
			let msg_timeout_ms = 5000;
			callback(false, response.item, msg_response, must_add_many, null, msg_timeout_ms);
		}
		else {
			msg_response.msg = messages_by_action[action_type].NONE;
			msg_response.type = "Error";
			let must_add_many = false;
			let msg_timeout_ms = 5000;
			callback(false, response.item, msg_response, must_add_many, null, msg_timeout_ms);
		}

		this.update_grouped_detail_of_peers(type);
		this.clean_up_controls_assign_or_unassign_peer(type);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.validate_assign_or_unassign_peers = function(type)
	{
		let prefix = `${type}_prov_form_vcc_omnichannel_telephony_peers_assign`;

		this.hide_popup_peers_all_errors(type);
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Assign Peers.')%>"};

		let item = {};
		// first_number validation -----
		//
		// Required validation
		const $first_number = $("#" + prefix + "_popup_first_number");
		let first_number = $first_number.val();
		let validation_result = inConcert.Utils.validate(first_number, "required");
		if (!validation_result.success) {
			// valida que el first_number tenga al menos un caracter
			response.msg = "<%=_T('First phone number is required.')%>";
			response.$ctrl = $first_number;
			this.show_popup_peers_ctrl_err(type, response);
			return response;
		}

		validation_result = inConcert.Utils.validate(first_number, "digits");
		if (!validation_result.success) {
			// valida que el first_number sea un numero
			response.msg = "<%=_T('First phone number is not a number.')%>";
			response.$ctrl = $first_number;
			this.show_popup_peers_ctrl_err(type, response);
			return response;
		}

		// popup_total_numbers validation -----
		//
		// Required validation
		const $total_numbers = $("#" + prefix + "_popup_total_numbers");
		let total_numbers = $total_numbers.val();
		validation_result = inConcert.Utils.validate(total_numbers, "required");
		if (!validation_result.success) {
			// valida que el popup_total_numbers tenga al menos un caracter
			response.msg = "<%=_T('Number of phones is required.')%>";
			response.$ctrl = $total_numbers;
			this.show_popup_peers_ctrl_err(type, response);
			return response;
		}

		validation_result = inConcert.Utils.validate(total_numbers, "digits");
		if (!validation_result.success) {
			// valida que el total_numbers sea un numero
			response.msg = "<%=_T('Number of phones is not a number.')%>";
			response.$ctrl = $total_numbers;
			this.show_popup_peers_ctrl_err(type, response);
			return response;
		}

		item.first_number = Number(first_number);
		item.total_numbers = Number(total_numbers);

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.hide_popup_peers_all_errors = function(type)
	{
		this.__grid_peers(type).hide_popup_msg();
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.show_popup_peers_ctrl_err = function(type, err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.__grid_peers(type).show_popup_msg(err_info.msg, "Error");
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.clean_up_controls_assign_or_unassign_peer = function(type)
	{
		let prefix = `${type}_prov_form_vcc_omnichannel_telephony_peers_assign`;
		const $first_number = $("#" + prefix + "_popup_first_number");
		const $popup_total_numbers = $("#" + prefix + "_popup_total_numbers");
		$first_number.val("").focus();
		$popup_total_numbers.val("");
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.__make_grouped_detail_of_peers = function(selected_peers)
	{
		if (!selected_peers) {
			return "";
		}
		let sorted_peers = inConcert.Utils.sortArray(selected_peers, null, "asc", function(row) {
			return Number(row.Name);
		});
		let detail_grouped_peers = "";
		let first_grouped_peer = null;
		let last_grouped_peer = null;

		let make = function() {
			if (!first_grouped_peer) {
				return;
			}
			if (detail_grouped_peers.length > 0) {
				detail_grouped_peers = detail_grouped_peers + " ";
			}
			detail_grouped_peers = detail_grouped_peers + first_grouped_peer.toString();

			if (first_grouped_peer < last_grouped_peer) {
				detail_grouped_peers = detail_grouped_peers + "-" + last_grouped_peer.toString();
			}
		};

		for (let i = 0; i < sorted_peers.length; i++) {
			let selected_peer = sorted_peers[i];
			let peer_number = Number(selected_peer.Name);

			if (first_grouped_peer === null) {
				first_grouped_peer = peer_number;
				last_grouped_peer = peer_number;
			}
			else {
				let next_grouped_peer = last_grouped_peer + 1;
				if (next_grouped_peer === peer_number) {
					last_grouped_peer = peer_number;
				}
				else if (next_grouped_peer < peer_number) {
					make();
					first_grouped_peer = peer_number;
					last_grouped_peer = peer_number;
				}
			}
		}

		make();

		return detail_grouped_peers;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.update_grouped_detail_of_peers = function(type)
	{
		let prefix_id = `${type}_prov_form_vcc_omnichannel_telephony_peers_assign`;

		// pagina - detalle
		const $page_detail_assigned = $("#" + prefix_id + "_detail_assigned");
		// popup - detalle
		const $popup_detail_assigned = $("#" + prefix_id + "_popup_detail_assigned");

		let selected_peers = this.__grid_peers(type).get_selected_items();
		let total_selected = (selected_peers || []).length;
		let popup_total_msg, page_total_msg;

		if (total_selected > 0) {
			let template = "<strong>{0}</strong>";
			let formatted_total = FORMAT(template, total_selected);
			popup_total_msg = FORMAT("<%=_T('He currently has a total of {0} selected phones.')%>", formatted_total);
			page_total_msg = FORMAT("<%=_T('Total selected phones: {0}.')%>", formatted_total);

			let detail_grouped_peers = this.__make_grouped_detail_of_peers(selected_peers);
			let formatted_detail = FORMAT(template, detail_grouped_peers);
			let detail_msg = FORMAT("<%=_T('Detail: {0}')%>", formatted_detail);

			// pagina - detalle
			$page_detail_assigned.attr("title", detail_grouped_peers).html(detail_msg).show();
			// popup - detalle
			$popup_detail_assigned.attr("title", detail_grouped_peers).html(detail_msg).show();
		}
		else {
			popup_total_msg = "<%=_T('You currently have no selected phones.')%>";
			page_total_msg = popup_total_msg;
			// pagina - detalle
			$page_detail_assigned.removeAttr("title").html("").hide();
			// popup - detalle
			$popup_detail_assigned.removeAttr("title").html("").hide();
		}

		//pagina - total
		const $total_assigned = $("#" + prefix_id + "_total_assigned");
		$total_assigned.html(page_total_msg);
		//popup - total
		const $popup_total_assigned = $("#" + prefix_id + "_popup_total_assigned");
		$popup_total_assigned.html(popup_total_msg);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.on_open_popup_assign_peers = function(type)
	{
		let prefix_id = `${type}_prov_form_vcc_omnichannel_telephony_peers_assign`;
		$("#" + prefix_id + "_popup_total_numbers").val("");
		const $first_number = $("#" + prefix_id + "_popup_first_number");
		$first_number.val("").focus();

		this.update_grouped_detail_of_peers(type);
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.on_open_popup_create_peers = function(type)
	{
		this.m_peers_mng[type].empty_controls();
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.update_grid_peers = function(type, arr_not_added)
	{
		const self = this;

		const peers_mng = this.m_peers_mng[type];
		let peer = peers_mng.get(true);
		let peers_range = peers_mng.get_peers_range();

		let peer_start = parseInt(peers_range.peer_start, 10);
		let peer_end = peer_start + (parseInt(peers_range.peer_count, 10));

		let not_added = inConcert.Utils.arrayToMap(arr_not_added || [], 
			function(_) {
				return true;
			},
			function(value) {
				return parseInt(value, 10);
			}
		);

		let have_been_added = false;
		var add_new_peers_to_grids = function (peer) {
			m_peer_types.forEach(peer_type => {
				self.m_grid_peers_assign[peer_type.id].add_or_update_row(peer);
				have_been_added = true;
			});
		};

		for (let i = peer_start; i < peer_end; i++) {
			if (!not_added[i]) {
				let name = i.toString();
				let peer_to_add = {
					Id: peer.tech + "_" + name,
					Name: name,
					Tech: peer.tech,
					HostId: peer.hostId,
					Assigned: type
				};
				// agrego en los grids (agent peers y contact peers)
				add_new_peers_to_grids(peer_to_add);
			}
		}

		return have_been_added;
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.create_peers = function(type, must_create_and_contiune, _, callback = $.noop)
	{
		const self = this;

		const peers_mng = this.m_peers_mng[type];
		const grid = this.__grid_peers(type);

		grid.show_popup_waiting("<%=_T('Please wait ...')%>");

		var validation_callback = function(success) {
			grid.hide_popup_waiting();
			if (!success) {
				callback(false);
				return;
			}
			let force_new = true;
			let peers = peers_mng.get(force_new);
			let have_been_added = false;

			peers_mng.save_execute(peers, force_new,
				function(success, response)
				{
					let err = !success ? response: undefined;
					if (success && response) {
						if (response.code === "ALL PEERS ADDED") {
							have_been_added = self.update_grid_peers(type);
							let msg = "<%=_T('All IP Phones have been successfully added.')%>";
							if (!must_create_and_contiune) {
								self.frm.show_status_message(msg, "Info", 5000);
								grid.hide_popup();
							}
							else {
								grid.show_popup_msg(msg, "Info", 5000);
								peers_mng.empty_controls();
							}
						}
						else if (response.code === "PARTIAL ADDED PEERS") {
							let detail;
							let not_added = response.rawData ? response.rawData.notAdded : [];
							for (let i = 0; i < not_added.length; i++) {
								let peer = not_added[i];
								detail = detail || "";
								if (detail) {
									detail = detail + ", ";
								}
								detail = detail + peer;
							}
							self.frm.show_popup_message({
								messages: {
									title  : "<%=_T('Not all IP Phones have been added.')%>",
									message: "<%=_T('The IP Phones that could not be added are')%>" + "<%=_T(':')%>",
									detail : detail,
									type   : "warn"
								}
							});
							have_been_added = self.update_grid_peers(type, response.rawData.notAdded);
						}
					}
					else {
						let err_info = peers_mng.err_save_messages(err);
						grid.show_popup_msg(err_info.err, "Error", 5000);
					}

					callback(success, response);

					// Despues de agregar los peers en la BD (CF Peers) y luego dejarlos aregarlos en la grilla grilla y dejarlos marcados, tengo que enviar a actualizar el VCC.
					if (have_been_added) {
						self.save_execute();
					}
				}
			);
		};

		const $custom_message_container = $("#" + type + "_prov_form_vcc_omnichannel_telephony_peers_create_popup_message");
		let validation_def = peers_mng.get_validation_def();
		let timeout_ms = 3000;
		this.frm.validate(validation_callback, validation_def, timeout_ms, $custom_message_container);
	};

	/*
		Funcion que realiza un Requests (encadenados) por cada entidad/TAB.
		Los va encolando a los Request y ejecutando en la medida que el Request anterior de Ok.
	*/
	form_provisioning_vccs_section_omnichannel_telephony.prototype.start_update_all_tabs = function()
	{
		const self = this;

		this.hide_messages();

		// 1 - realizo validaciones client side y luego ejecuto el request contra los TTS (Tab - TTS/ASR)
		//      Es nescesario realizar la validacion primero sobre el tab de TTS/ASR ya que tiene validaciones client side y confirmaciones.
		//
		this.ttsasr_update_vcc(function(success/*, info_tts*/) {
			if (!success) {
				self.main_tabs.switch_tab("ttsasr");
				return;
			}
			// 2 - realizo request contra los peers de tipo agent (Tab - Telefonos IP Agente)
			self.peers_update_vcc("agent", function(success/*, info_peers_agent*/) {
				if (!success) {
					self.main_tabs.switch_tab("agent_peers");
					return;
				}
				// 3 - realizo request contra los peers de tipo clients (Tab - Telefonos IP Cliente)
				self.peers_update_vcc("client", function(success/*, info_peers_client*/) {
					// 3.1 - tengo que relizar actualizar ambas grillas de peers (agentes y clientes)
					self.__reload_peers();
					if (!success) {
						self.main_tabs.switch_tab("client_peers");
						return;
					}
					// 4 - realizo request contra los paquetes
					self.packages_update_vcc(function(success/*, info_packages*/) {
						if (!success) {
							self.main_tabs.switch_tab("packages");
							return;
						}
						let message = "<%=_T('Telephony Resources of VCC has been successfully updated.')%>";
						let msg_type = "Info";
						self.show_status_message(message, msg_type, 5000);
					});
				});
			});
		});
	};

	form_provisioning_vccs_section_omnichannel_telephony.prototype.save_execute = function()
	{
		this.start_update_all_tabs();
	};

}());
