/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

	
/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// configs Pool Management
	function ProvisioningGatewayConfigTableMng(params)
	{
		if (!params) {
			throw("ProvisioningGatewayConfigTableMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningGatewayConfigTableMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningGatewayConfigTableMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningGatewayConfigTableMng - 'params.container_id' is required");
		}
		this.title = params.title || "<%=_T('Configured Attributes')%>";
		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;
		this.footer_actions_scope = params.footer_actions_scope;
		this.footer_actions_popup = params.footer_actions_popup;
		this.collapsable = params.collapsable || false;
		this.collapsed = params.collapsed || false;
		this.readonly = params.readonly || false;
		this.allowContext = params.allowContext || false;
		this.tooltip = params.tooltip || "<%=_T('Asterisk configuration attributes.')%>";
		this.on_remove = params.on_remove;
		return true;
	}
	
	if (typeof inConcert.ProvisioningGatewayConfigTableMng !== "function") {
		inConcert.ProvisioningGatewayConfigTableMng = ProvisioningGatewayConfigTableMng;
	}
	
	ProvisioningGatewayConfigTableMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.set_events();
		this.toggle_bottom_actions();
	};
		
	ProvisioningGatewayConfigTableMng.prototype.__make_popup_html = function()
	{
		const prefix = this.prefix;
		const html = `
		<div
			id="${prefix}container"
			class="row accordeon-row ${this.collapsable ? "collapsable" : ""}"
		>
			<div class="col-6">
				<div id="${prefix}title" class="card-header gateways-card-header" hidden>
					${this.on_remove ? `
						<input
							bottom_action_selector
							id="${prefix}selector"
							class="form-check-input"
							type="checkbox"
						/>
					` : ""}
					<h5 class="mb-0">
						<span>${this.title}</span>
					</h5>
					<i
						class="fa fa-question-circle ml-1"
						title="${this.tooltip}"
					>
					</i>
				</div>
				<div class="card-block">
				<div id="${prefix}no_attributes_message" class="inlineMessagesContainer messageInfo" style="">
					<%=_T([[No attributes created]])%>
				</div>
					<div id="${prefix}row_container" class="row">
					</div>
				</div>
			</div>
			<div class="col-6">
				<div id="${prefix}attributes_title" class="card-header gateways-card-header">
					<h5 class="mb-0">
						<span id="${prefix}attributes_title_text">
						</span>
						<i
							class="fa fa-question-circle"
							title="<%=_T([[You can add multiple atributes line by line.]])%>"
						>
						</i>
					</h5>
				</div>
				<div class="card-block gateways-card-block">
					<div class="row h-100">
						<div class="col-12">
							<div class="form-group">
								<textarea
									id="${prefix}configuration"
									class="form-control"
									placeholder="<%=_T([[Bulk add the 'name=value']])%>"
									${this.readonly ? "readonly" : ""}
								></textarea>
							</div>
						</div>
					</div>
				</div>
				<div class="gateways-card-footer">
					${!this.readonly ? `
						<button
							id="${prefix}btn_add"
							class="pageButtonGeneric buttonColorBlue"
						>
							<%=_T([[Add]])%>
						</button>
					` : ""}
				</div>
			</div>
		</div>
		`;
		$(`#${this.container_id}`).html(html);
	};
	
	ProvisioningGatewayConfigTableMng.prototype.set_events = function()
	{
		if (this.collapsable) {
			$(`#${this.prefix}title, #${this.prefix}attributes_title`)
				.off("click")
				.on("click", () => {
					this.collapsed = !this.collapsed;
					this.toggle_collapsed();
				});
			this.toggle_collapsed();
		}

		if (this.on_remove) {
			$(`#${this.prefix}selector`)
				.off("click")
				.on("click", (e) => {
					e.stopPropagation();
					this.toggle_bottom_actions();
				})
				.off("bottom_action_remove")
				.on("bottom_action_remove", () => this.on_remove());
		}

		if (!this.readonly) {
			$(`#${this.prefix}btn_add`)
				.off("click")
				.on("click", () => this.add_attributes());
		}
	};
	
	ProvisioningGatewayConfigTableMng.prototype.dispose = function()
	{
	};

	ProvisioningGatewayConfigTableMng.prototype.get = function()
	{
		return this.get_attributes();
	};
	
	ProvisioningGatewayConfigTableMng.prototype.set = function(attributes)
	{
		attributes = attributes || [];
		if (typeof(attributes) === "string") attributes = JSON.parse(attributes);
		$(`#${this.prefix}row_container`)
			.html("")
			.append(attributes.map((attribute) => this.make_attribute(attribute)));
		this.toggle_empty();
	};

	ProvisioningGatewayConfigTableMng.prototype.validate = function()
	{
		const attributes = this.get_attributes();
		if (!attributes.every(({ name }) => inConcert.Utils.validate( name, "required").success))
			return { message: "<%=_T('Atrributes names are required.')%>" };

		if (!this.allowContext && attributes.some(({ name }) => name === "context"))
			return { message: "<%=_T('Cannot assign context as an attribute.')%>" };

		return { success: true };
	};

	ProvisioningGatewayConfigTableMng.prototype.get_validation_def = function(definitions = [])
	{
		definitions.push({
			control_id: `${this.prefix}title`,
			messages: {
				custom: {
					type : "custom",
					valid: () => this.validate()
				}
			}
		});
	};
	
	ProvisioningGatewayConfigTableMng.prototype.get_attributes = function()
	{
		return $(`#${this.prefix}row_container`).find("[attribute]").toArray().map((element) => {
			const $element = $(element);
			return {
				name: $element.find("[attribute_name]").val().trim().toLowerCase(),
				value: $element.find("[attribute_value]").val().trim(),
			};
		});
	};

	ProvisioningGatewayConfigTableMng.prototype.toggle_collapsed = function()
	{
		if (!this.collapsable) return;

		$(`#${this.prefix}container`).toggleClass("collapsed", this.collapsed);
		$(`#${this.prefix}attributes_title_text`).text(this.collapsed
			? FORMAT("<%=_T('Attributes ({0})')%>",
				$(`#${this.prefix}row_container`).find("[attribute]").length)
			: "<%=_T('Add Attributes')%>");
	};

	ProvisioningGatewayConfigTableMng.prototype.toggle_empty = function()
	{
		const empty = $(`#${this.prefix}row_container`).find("[attribute]").length === 0;
		$(`#${this.prefix}no_attributes_message`).toggle(empty);
	};

	ProvisioningGatewayConfigTableMng.prototype.add_attributes = function()
	{
		const lines = ($(`#${this.prefix}configuration`).val() || "").split("\n");
		$(`#${this.prefix}row_container`).append(
			lines.map((line) => this.add_attribute_by_line(line)));
		this.toggle_collapsed();
		this.toggle_empty();
		$(`#${this.prefix}configuration`).val("");
	};

	ProvisioningGatewayConfigTableMng.prototype.add_attribute_by_line = function(line)
	{
		line = line.trim();
		if (!line) return;

		const parts = line.split("=");
		if (parts.length !== 2) {
			this.owner.show_status_message(
				FORMAT("<%=_T('Invalid attribute: {0}. Must be name=value.')%>", line), "Error", 5000);
			return;
		}
		const name = parts[0].trim().toLowerCase();
		const value = parts[1].trim();
		if (!name) {
			this.owner.show_status_message(
				FORMAT("<%=_T('Invalid attribute: {0}. Must be name=value.')%>", line), "Error", 5000);
			return;
		}

		if (this.exists_attribute({ name, value })) return;

		return this.make_attribute({ name, value });
	};

	ProvisioningGatewayConfigTableMng.prototype.exists_attribute = function({ name, value })
	{
		return $(`#${this.prefix}row_container`).find("[attribute]").toArray().some((attribute) => {
			const $attribute = $(attribute);
			return $attribute.find("[attribute_name]").val() === name &&
				$attribute.find("[attribute_value]").val() === value;
		});
	};

	ProvisioningGatewayConfigTableMng.prototype.make_attribute = function({name, value})
	{
		const $attribute = $(`
			<div attribute class="col-12">
				<div class="formListRowContainer">
					${!this.readonly ? `
						<input bottom_action_selector class="form-check-input" type="checkbox" />
					` : ""}
					<input
						attribute_name
						class="form-control w-auto flex-grow-1"
						type="text"
						placeholder="<%=_T([[Enter the name]])%>"
						${this.readonly ? "readonly" : ""}
					/>
					<input
						attribute_value
						class="form-control w-auto flex-grow-1"
						type="text"
						placeholder="<%=_T([[Enter the value]])%>"
						${this.readonly ? "readonly" : ""}
					/>
				</div>
			</div>
		`);
		$attribute.find("[bottom_action_selector]")
			.on("click", () => this.toggle_bottom_actions())
			.on("bottom_action_remove", () => {
				$attribute.remove();
				this.toggle_empty();
				this.toggle_collapsed();
			});
		$attribute.find("[attribute_name]").val(name);
		$attribute.find("[attribute_value]").val(value);
		return $attribute;
	};

	ProvisioningGatewayConfigTableMng.prototype.toggle_bottom_actions = function(self)
	{
		self = self || this;
		const $footer = $(`#${self.footer_actions_popup}`);
		const $selected = $(`#${self.footer_actions_scope}`).find("[bottom_action_selector]:checked");
		if ($selected.length === 0) {
			$footer.hide();
			return;
		}

		const onclose = () => {
			$selected.prop("checked", false);
			$footer.hide();
		};

		$footer.find("[footer_counter]").text($selected.length);
		$footer.find("[footer_cancel]")
			.off("click")
			.on("click", () => onclose());
		$footer.find("[footer_delete]")
			.off("click")
			.on("click", () => {
				onclose();
				$selected.trigger("bottom_action_remove");
			});
		$footer.show();
	};

}());