/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Profile Management
	function ProvisioningLlmprofilesMng(params)
	{
		if (!params) {
			throw("ProvisioningLlmprofilesMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningLlmprofilesMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningLlmprofilesMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningLlmprofilesMng - 'params.container_id' is required");
		}

		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;

		this.m_grid_profiles = null;
		
		return true;
	}

	if (typeof inConcert.ProvisioningLlmprofilesMng !== "function") {
		inConcert.ProvisioningLlmprofilesMng = ProvisioningLlmprofilesMng;
	}

	ProvisioningLlmprofilesMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__init_profiles();
		this.__init_grids();
	};

	ProvisioningLlmprofilesMng.prototype.is_readonly = function()
	{
		return false;
	};

	ProvisioningLlmprofilesMng.prototype.get_grid_dom_id = function()
	{
		return this.prefix + "profiles_list";
	};

	ProvisioningLlmprofilesMng.prototype.__make_popup_html = function()
	{
		let translated_placeholder_profile_id = inConcert.Utils.escapeAttrib('<%=_T("AVOID_GENERAL_OR_DEFAULT_NAMES")%>');

		let html = '' +
			'<div id="' + this.prefix + 'new_container">' +
				'<div class="card-header d-flex flex-row justify-content-between">' +
					'<i id="' + this.prefix + 'new_list_validation" class="formFieldValidation formFieldError" hidden></i>' +				

                    '<div class="col-6 mt-2 p-0">' +
                        '<div class="form-group">' +
                            '<div class="form-group">' +
                                '<label for="' + this.prefix + '_copilot_profile" class=""><%=_T([[Select an LLM profile to enable inConcert Copilot]])%>:</label>' +
                                '<i class="fa fa-question-circle" title="<%=_T([[Enable assistance capabilities using artificial intelligence]])%>"></i>' +
                                '<select id="' + this.prefix + '_copilot_profile" class="form-control"></select>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                    '<div class="col-6">' +
                        '<button id="' + this.prefix + 'new" class="pageButtonGeneric buttonColorWhite pageButtonNew">' +
                            '<i class="fal fa-plus"></i>' +
                            '<%=_T([[New Profile]])%>' +
                        '</button>' +
                    '</div>' +
                '</div>' +
            '</div>' +

			'<div class="card-block">' +
				'<div id="' + this.prefix + 'container" class="ttsasrTableContainer">' +
					'<div id="' + this.prefix + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none;"></div>' +
					'<table id="' + this.get_grid_dom_id() + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
						'<thead class="viewGridHeader"></thead>' +
						'<tbody class="viewGridBody"></tbody>' +
					'</table>' +
				'</div>' +
				'<div id="' + this.prefix + 'paginator"></div>' +
			'</div>' +
            
			'<div id="' + this.prefix + 'popup" class="internalPopUpContainer profilesPopUpContainer" style="display:none;">' +
				'<div class="internalPopUpOverlay"></div>' +
				'<div class="internalPopUp">' +
					'<div id="' + this.prefix + 'popup_waiting" class="internalWaitingContainer" style="display:none;">' +
						'<div waiting_message="true" class="internalWaitingMessage"></div>' +
						'<div class="internalWaitingAnimation"></div>' +
						'<div class="internalWaitingOverlay"></div>' +
					'</div>' +
					'<div id="' + this.prefix + 'popup_title" class="internalPopUpTitle">' +
						'<span popup_title_text></span>' +
						'<button popup_close class="internalPopUpCloseButton"><i class="far fa-times"></i></button>' +
					'</div>' +
					'<div class="internalPopUpMiddle">' +
						'<div id="' + this.prefix + 'popup_message" class="genericFormMessagesContainer messageError popup" style="display:none;">' +
							'<label class="messageText"></label>' +
						'</div>' +
						'<div>' +
							'<div class="card-block">' +
								'<div class="row">' +
									'<div class="col-6">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_profile_id" class="required"><%=_T([[Name]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_profile_id" class="form-control" type="text" placeholder="' + translated_placeholder_profile_id + '" </input>' +
											'<i id="' + this.prefix + 'popup_profile_id_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-6">' +	
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_description" class="required"><%=_T([[Description]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_description" class="form-control" type="text" placeholder="<%=_T([[Enter the description]])%>..."></input>' +
											'<i id="' + this.prefix + 'popup_description_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div class="row">' +
									'<div class="col-4">' +
										'<div engineType="unimrcp" class="form-group">' +
											'<label for="' + this.prefix + 'popup_provider" class="required"><%=_T([[Provider]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_provider" class="form-control" placeholder="<%=_T([[Select the provider]])%>..."></select>' +
											'<i id="' + this.prefix + 'popup_provider_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
                                    '<div class="col-8">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_domain" class="required"><%=_T([[Domain]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_domain" class="form-control" type="text" placeholder="<%=_T([[Enter the value]])%>..."></input>' +
											'<i id="' + this.prefix + 'popup_domain_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
                                '<div class="row">' +
                                    '<div class="col-4">' +
                                        '<div class="form-group">' +
                                            '<label for="' + this.prefix + 'popup_auth_header_name" class="required"><%=_T([[Auth Header Name]])%><%=_T([[:]])%></label>' +
                                            '<input id="' + this.prefix + 'popup_auth_header_name" class="form-control" type="text" placeholder="<%=_T([[Enter the value]])%>..."></input>' +
                                            '<i id="' + this.prefix + 'popup_auth_header_name_validation" class="formFieldValidation formFieldError" hidden></i>' +
                                        '</div>' +
                                    '</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_auth_header_value" class="required"><%=_T([[Auth Header Value]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_auth_header_value" class="form-control" type="password" placeholder="<%=_T([[Enter the value]])%>..."></input>' +
											'<i id="' + this.prefix + 'popup_auth_header_value_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
                                    '<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_model" class="required"><%=_T([[Model]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_model" class="form-control" type="text" placeholder="<%=_T([[Enter the value]])%>..."></input>' +
											'<i id="' + this.prefix + 'popup_model_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_timeout" class="required"><%=_T([[Timeout (Seconds)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_timeout" class="form-control" type="number" step="1" placeholder="<%=_T([[Enter the value]])%>..."></input>' +
											'<i id="' + this.prefix + 'popup_timeout_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
                                    '<div class="col-8">' +
										'<div class="form-group">' +
											'<div class="form-check form-switch">' +
												'<label for="' + this.prefix + 'popup_enabled" class="form-check-label">' +
													'<input id="' + this.prefix + 'popup_enabled" class="form-check-input" type="checkbox"></input>' +
													'<span>' +
														'<%=_T([[Enabled]])%>' +
													'</span>' +
												'</label>' +
												'<i id="' + this.prefix + 'popup_enabled_validation" class="formFieldValidation formFieldError" hidden></i>' +
											'</div>' +
										'</div>' +
									'</div>' +
								'</div>' +								
							'</div>' +
						'</div>' +
					'<div>' +
					'<div class="internalPopUpButtons">' +
						'<input id="' + this.prefix + 'popup_close" class="pageButtonGeneric buttonColorWhite float-right" type="button" value="<%=_T([[Close]])%>" />' +
						'<input id="' + this.prefix + 'popup_create" class="pageButtonGeneric buttonColorBlue float-right" type="button" value="<%=_T([[Create]])%>" />' +
					'</div>' +
				'</div>' +
			'</div>';

		const $container = $("#" + this.container_id);
		$container.html(html);
	};

	ProvisioningLlmprofilesMng.prototype.__init_grids = function()
	{
		this.__init_grid_profiles();
	};

	ProvisioningLlmprofilesMng.prototype.__init_grid_profiles = function()
	{
		const self = this;

		var must_show_button = function(row)
		{
			return !self.isUsedByVCC(row.id);
		};

		this.m_grid_profiles = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner,
			row_id: "id",
			display_name: "id",
			rows_per_page: 10,
			show_at_least: 9,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Profiles ...')%>",
			ids: {
				paginator: this.prefix + "paginator",
				grid: this.get_grid_dom_id(),
				not_results_found: this.prefix + "not_results_found",
				container_new_btn: this.prefix + "new_container",
				new_item: this.prefix + "new",
				popup: this.prefix + "popup",
				popup_msg: this.prefix + "popup_message",
				popup_create: this.prefix + "popup_create",
				popup_create_and_continue: this.prefix + "popup_create_and_continue",
				popup_title: this.prefix + "popup_title",
				popup_close: this.prefix + "popup_close",
				popup_input_id: this.prefix + "popup_question",
				popup_waiting: this.prefix + "popup_waiting"
			},
			strings: {
				not_results_found: "<%=_T('No Profiles found.')%>",
				can_remove       : "<%=_T('Do you want to delete Profile: {0} ?')%>"
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"       },
				{id: "id"                , name: "<%=_T('Profile Name')%>" },
				{id: "description"       , name: "<%=_T('Description')%>"  },
				{id: "provider"          , name: "<%=_T('Provider')%>"     },
				{id: "domain"            , name: "<%=_T('Domain')%>"       },
			// El resto de columnas de datos quedan ocultas
				{
					id: "_action",
					name: "",
					system_action: {
						edit: true,
						erase: /*true*/{
							must_show: function(row) {
								return must_show_button(row);
							}
						}
					}
				}
			],
			filter: {
				fields: {
					"id"               : {control: null},
					"description"      : {control: null},
					"provider"         : {control: null}
				}
			},
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_row(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row_data) {
					self.on_open_popup(row_data);
				},
				on_removed_row: function(/*row, callback = $.noop*/) {
                    self.update_copilot_profiles();
                },
 				on_closed_popup: function() {
 					self.on_closed_popup();
 				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one Profile.')%>",
				required: true
			}
		});
	};

	ProvisioningLlmprofilesMng.prototype.start_grid_validation = function()
	{
		let response = {
			success: true
		};
		let success = this.m_grid_profiles.start_validation().success;
		if (!success) {
			response.success = false;
			response.message = this.m_grid_profiles.get_validation_message_err("required");
		}
		return response;
	};

	ProvisioningLlmprofilesMng.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_profile = null;
		this.m_current_profile = null;
	};

	ProvisioningLlmprofilesMng.prototype.add_or_update_row = function(callback/*, _, current_popup*/)
	{
		callback = callback || $.noop;

		const self = this;

		this.validate_profile_to_add_or_update(function(response)
		{
			if (!response.success) {
				callback(false);
				return;
			}
			let msg_response = {
				msg : "",
				type: "Info"
			};
	
			if (self.m_is_new_profile) {
				msg_response.msg = "<%=_T('Profile was successfully added.')%>";
				let must_add_many = false;
				callback(true, response.item, msg_response, must_add_many);
			}
			else {
				msg_response.msg = "<%=_T('Profile was successfully updated.')%>";
				self.m_current_profile = inConcert.Utils.cloneObject(response.item);
				callback(true, self.m_current_profile, msg_response);
			}

            self.update_copilot_profiles();
	
			self.clean_up_controls_profile();
		});
	};

	ProvisioningLlmprofilesMng.prototype.clean_up_controls_profile = function()
	{
		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		const $description = $("#" + this.prefix + "popup_description");
		const $provider = $("#" + this.prefix + "popup_provider");
		this.owner.readonly_control($profile_id, false, true);
		$profile_id.val("").focus();
		$description.val("");
		$provider.change();
    
        const $domain = $("#" + this.prefix + "popup_domain");
        const $auth_header_name = $("#" + this.prefix + "popup_auth_header_name");
        const $auth_header_value = $("#" + this.prefix + "popup_auth_header_value");
        const $model = $("#" + this.prefix + "popup_model");
        const $timeout = $("#" + this.prefix + "popup_timeout");
        const $enabled = $("#" + this.prefix + "popup_enabled");
                
        $domain.val("");
        $auth_header_name.val("");
        $auth_header_value.val("");
        $model.val("");
        $timeout.val(60);
        $enabled.prop("checked", true);
	};

	ProvisioningLlmprofilesMng.prototype.on_open_popup = function(item)
	{
		const self = this;
		const $title = $("#" + this.prefix + "popup_title").find("[popup_title_text]");
		this.m_is_new_profile = item ? false: true;
		this.m_current_profile = item || null;

		this.clean_up_controls_profile();

		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		const $description = $("#" + this.prefix + "popup_description");
		const $provider = $("#" + this.prefix + "popup_provider");

		const $combo = $("#" + this.prefix + "popup_provider");
		$combo.html("");
		let sorted_providers = [{ id: "", name: "" }, { id: "openai", name: "OpenAI" }];

		$.each(sorted_providers, function(_, provider) {
			self.combo_add_value($combo, provider.id, provider.name, false);
		});

		if (this.m_is_new_profile) {
			$title.text("<%=_T('New Profile')%>");
			this.owner.readonly_control($profile_id, false);
		}
		else {
            $title.text("<%=_T('Update Profile')%>");
			$profile_id.val(item.id);
			$description.val(item.description);
			$provider.val(item.provider);
			this.owner.readonly_control($profile_id, true);
		}

		$provider.change();

        const $domain = $("#" + this.prefix + "popup_domain");
        const $auth_header_name = $("#" + this.prefix + "popup_auth_header_name");
        const $auth_header_value = $("#" + this.prefix + "popup_auth_header_value");
        const $model = $("#" + this.prefix + "popup_model");
        const $timeout = $("#" + this.prefix + "popup_timeout");
        const $enabled = $("#" + this.prefix + "popup_enabled");

        if (this.m_is_new_profile) {
            $domain.val("");
            $auth_header_name.val("");
            $auth_header_value.val("");
            $model.val("");
            $timeout.val(60);
            $enabled.prop("checked", true);
            return;
        }
        // editando       
        $domain.val(item.domain);
        $auth_header_name.val(item.auth_header_name);
        $auth_header_value.val(item.auth_header_value);
        $model.val(item.model);
        $timeout.val(item.timeout);
        $enabled.prop("checked", item.enabled === "1" ? true : false);
	};

    ProvisioningLlmprofilesMng.prototype.combo_add_value = function($combo, value_id, description, selected, data_id, data, id_is_case_sensitive = false, tooltip)
    {
        if (value_id === undefined) {
            return false;
        }
        let name;
        if (typeof description === "function") {
            name = description();
        }
        else {
            description = (typeof description === undefined || description === null) ? value_id : description;
        }

        if (id_is_case_sensitive) {
            value_id = value_id.toLowerCase();
        }

        const _T = inConcert.i18n.getString;
        const $option = $("<option></option>").attr("value", value_id).html(_T(description));
        if (typeof tooltip === "string") {
            $option.attr("title", tooltip);
        }
        if (selected) {
            $option.attr("selected", "selected");
        }
        if (data) {
            data_id = data_id || "item";
            $option.data(data_id, data);
        }
        $combo.append($option);
        return $option;
    },

    ProvisioningLlmprofilesMng.prototype.update_copilot_profiles = function()
    {
        let self = this;

        let current_profile = $("#" + self.prefix + "_copilot_profile").val();

        let $InconcertCopilotOptions = "<option></option>";
        let profiles = this.m_grid_profiles.get_items();
        profiles.forEach(profile => {
            $InconcertCopilotOptions += FORMAT("<option value='{0}'>{1}</option>", profile.id, profile.id);
        });
        $("#" + self.prefix + "_copilot_profile")
            .html("")
            .append($InconcertCopilotOptions);
        
        $("#" + self.prefix + "_copilot_profile").val(current_profile);
    };

	ProvisioningLlmprofilesMng.prototype.validate_profile_to_add_or_update = function(callback = $.noop)
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Profile.')%>"};

		let item = {};
		let is_new = false;
		if (this.m_current_profile) {
			item = this.m_current_profile;
		}
		else {
			is_new = true;
		}

		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		let profile_id = $profile_id.val();

		const $description = $("#" + this.prefix + "popup_description");
		let description = $description.val();

		let current_all_profile = this.m_grid_profiles.get_items();
		let current_all_profile_map = inConcert.Utils.arrayToMap(current_all_profile || [],
			function(profile) {
				return profile;
			},
			function(profile) {
				// el ID
				return profile.id;
			}
		);

		let validation_result;
		if (is_new) {

			validation_result = inConcert.Utils.validate(profile_id, "required");
			if (!validation_result.success) {
				// valida que el profile_id tenga al menos un caracter
				response.msg = "<%=_T('Profile Name is required.')%>";
				response.$ctrl = $profile_id;
				this.show_ctrl_err(response);
				return callback(response);
			}

			// Already exist id validation
			var show_already_id_error = function() {
				response.msg = "<%=_T('Profile already exist with that name.')%>";
				response.$ctrl = $profile_id;
				self.show_ctrl_err(response);
				return response;
			};
			const already_id = current_all_profile_map[profile_id];
			if (already_id) {
				return callback(show_already_id_error());
			}
		}
        
		validation_result = inConcert.Utils.validate(description, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Description is required.')%>";
			response.$ctrl = $description;
			this.show_ctrl_err(response);
			return callback(response);
		}

		var profile = {};

		const $provider = $("#" + this.prefix + "popup_provider");
		let provider = $provider.val();

		// Required validation
		validation_result = inConcert.Utils.validate(provider, "required");
		if (!validation_result.success) {
			// valida que el provider tenga al menos un caracter
			response.msg = "<%=_T('Provider is required.')%>";
			response.$ctrl = $provider;
			this.show_ctrl_err(response);
			return callback(response);
		}
    
        const $domain = $("#" + this.prefix + "popup_domain");
        let domain = $domain.val();

        const $auth_header_name = $("#" + this.prefix + "popup_auth_header_name");
        let auth_header_name = $auth_header_name.val();

        const $auth_header_value = $("#" + this.prefix + "popup_auth_header_value");
        let auth_header_value = $auth_header_value.val();

        const $model = $("#" + this.prefix + "popup_model");
        let model = $model.val();

        const $timeout = $("#" + this.prefix + "popup_timeout");
        let timeout = $timeout.val();

        const timeout_range = {between: {from: 10, to: 300}};
        validation_result = inConcert.Utils.validate(timeout, "range", timeout_range);
        if (!validation_result.success) {
			response.msg = FORMAT('<%=_T("Timeout must be a number between {0} and {1}.")%>', timeout_range.between.from, timeout_range.between.to);
            response.$ctrl = $timeout;
			this.show_ctrl_err(response);
			return callback(response);
		}

        const $enabled = $("#" + this.prefix + "popup_enabled");
        let enabled = $enabled.prop("checked") ? "1" : "0";

        validation_result = inConcert.Utils.validate(domain, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Domain is required.')%>";
			response.$ctrl = $domain;
			this.show_ctrl_err(response);
			return callback(response);
		}

        validation_result = inConcert.Utils.validate(auth_header_name, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Auth Header Name is required.')%>";
			response.$ctrl = $auth_header_name;
			this.show_ctrl_err(response);
			return callback(response);
		}

        validation_result = inConcert.Utils.validate(auth_header_value, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Auth Header Value is required.')%>";
			response.$ctrl = $auth_header_value;
			this.show_ctrl_err(response);
			return callback(response);
		}

        validation_result = inConcert.Utils.validate(model, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Model is required.')%>";
			response.$ctrl = $model;
			this.show_ctrl_err(response);
			return callback(response);
		}

        validation_result = inConcert.Utils.validate(timeout, "required");
		if (!validation_result.success) {
			response.msg = "<%=_T('Timeout is required.')%>";
			response.$ctrl = $timeout;
			this.show_ctrl_err(response);
			return callback(response);
		}

        profile = {
            id : profile_id,
            description : description,
            provider : provider,
            domain : domain,
            auth_header_name : auth_header_name,
            auth_header_value : auth_header_value,
            model : model,
            timeout : timeout,
            enabled : enabled
        };
		
		const row_profile = self.make_row_profile(item, profile);
		response.success = true;
		response.msg = "";
		response.item = row_profile;
		callback(response);
	};

	ProvisioningLlmprofilesMng.prototype.hide_all_errors = function()
	{
		this.m_grid_profiles.hide_popup_msg();
	};

	ProvisioningLlmprofilesMng.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grid_profiles.show_popup_msg(err_info.msg, "Error"); //, ERROR_TIMEOUT_MS);
	};

	ProvisioningLlmprofilesMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ProvisioningLlmprofilesMng.prototype.make_ip_port_id = function(server_ip, server_port)
	{
		return FORMAT("{0}_{1}", server_ip, server_port).toUpperCase();
	};

	ProvisioningLlmprofilesMng.prototype.make_row_profile = function(current_row_item, profile)
	{
		let row_item = current_row_item || {};

		row_item.id                    = profile.id;
		row_item.description           = profile.description;
		row_item.provider              = profile.provider;
        row_item.domain                = profile.domain;
        row_item.auth_header_name      = profile.auth_header_name;
        row_item.auth_header_value     = profile.auth_header_value;
        row_item.model                 = profile.model;
        row_item.timeout               = profile.timeout;
        row_item.enabled               = profile.enabled;
		
		return row_item;
	};

	ProvisioningLlmprofilesMng.prototype.set = function(profiles)
	{
		const self = this;

		if (!profiles || profiles === "") {
			profiles = [];
		} else {
			profiles = JSON.parse(profiles);
		}
		let profiles_info = [];
		if (!profiles || !(Array.isArray(profiles) && profiles.length)) {
			this.m_grid_profiles.initialize($.noop, profiles_info);
			return;
		}
		var add_profile = function(profile) {
			const row_profile = self.make_row_profile(null, profile);
			profiles_info.push(row_profile);

		};

        let copilot_profile;
        let $InconcertCopilotOptions = "<option></option>";

        profiles.forEach(profile => {
            add_profile(profile);
            $InconcertCopilotOptions += FORMAT("<option value='{0}'>{1}</option>", profile.id, profile.id);
            if (profile.inconcert_copilot) {
                copilot_profile = profile.id;
            }
        });

        $("#" + self.prefix + "_copilot_profile")
            .html("")
            .append($InconcertCopilotOptions);
        
        $("#" + self.prefix + "_copilot_profile").val(copilot_profile);
		this.m_grid_profiles.initialize($.noop, profiles_info);
	};

	ProvisioningLlmprofilesMng.prototype.empty_controls = function()
	{
		this.m_grid_profiles.remove_all_rows();
	};

	ProvisioningLlmprofilesMng.prototype.set_events = function()
	{
		this.m_grid_profiles.set_events();
	};

	ProvisioningLlmprofilesMng.prototype.popup_show_ctrl_err = function(_, $ctrl/*, $popup_msg*/)
	{
		if ($ctrl) {
			const error_id = $ctrl.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$ctrl.focus();
		}
	};

	ProvisioningLlmprofilesMng.prototype.__init_profiles = function()
	{
		$("body").removeClass("popupDisplay").addClass("popupDisplay");
	};

	ProvisioningLlmprofilesMng.prototype.get = function()
	{
        let response = this.m_grid_profiles.get_items();
        let copilot_profile = $("#" + this.prefix + "_copilot_profile").val();

        response.forEach(profile => {
            profile.inconcert_copilot = (profile.id === copilot_profile ? true : false);
        });

		return response;
	};

	ProvisioningLlmprofilesMng.prototype.isUsedByVCC = function(_)
	{
		return false;
	};
}());
