/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Profile Management
	function ProvisioningTtsAsrEngineprofilesMng(params)
	{
		if (!params) {
			throw("ProvisioningTtsAsrEngineprofilesMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningTtsAsrEngineprofilesMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningTtsAsrEngineprofilesMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningTtsAsrEngineprofilesMng - 'params.container_id' is required");
		}

		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;

		this.m_grid_profiles = null;
		
		this.codecs_mng = new inConcert.ProvisioningCodecsMng({
			owner       : this,
			prefix      : "prov_form_ttsasr_engine_profiles_codecs",
			container_id: "prov_form_ttsasr_engine_profiles_codecs_panel",
			fields_per_row: 7,
			codecs_list: [
				{id: "pcmu"                    , name: "PCMU"                    },
				{id: "pcma"                    , name: "PCMA"                    },
				{id: "l16/96/8000"             , name: "L16/96/8000"             },
				{id: "telephone-event/101/8000", name: "telephone-event/101/8000"}
			],
			codecs_list_sep : " "
		});

		this.m_grid_voices = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.owner,
			row_id: "id",
			display_name: "id",
			rows_per_page: 15,
			show_at_least: 14,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Voices ...')%>",
			ids: {
				paginator: this.prefix + "voice_" + "paginator",
				grid: this.prefix + "voice_" + "list",
				not_results_found: this.prefix + "voice_"  + "not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Voices found.')%>",
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"                            },
				{id: "LanguageCode"      , name: "<%=_T('Language Code')%>"                     },
				{id: "LanguageName"      , name: "<%=_T('Language Name')%>"                     },
				{id: "VoiceName"         , name: "<%=_T('Voice Name')%>"                        },
				{id: "Gender"            , name: "<%=_T('Gender')%>"                            },
				{id: "Variant"           , name: "<%=_T('Variant')%>"                           },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: false,
						erase: false
					}
				}
			]
		});

		this.m_grid_grammars = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.owner,
			row_id: "grammar",
			display_name: "grammar",
			rows_per_page: 15,
			show_at_least: 14,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Grammars ...')%>",
			ids: {
				paginator: this.prefix + "grammar_" + "paginator",
				grid: this.prefix + "grammar_" + "list",
				not_results_found: this.prefix + "grammar_"  + "not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Grammars found.')%>",
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"                            },
				{id: "id"                , name: "<%=_T('Grammar Id')%>"                        },
				{id: "type"              , name: "<%=_T('Grammar Type')%>"                      },
				{id: "grammar"           , name: "<%=_T('Grammar')%>"                           },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: false,
						erase: false
					}
				}
			]
		});

		this.m_grid_recognitionLanguages = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.owner,
			row_id: "id",
			display_name: "id",
			rows_per_page: 15,
			show_at_least: 14,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Recognition Languages ...')%>",
			ids: {
				paginator: this.prefix + "recognitionLanguage_" + "paginator",
				grid: this.prefix + "recognitionLanguage_" + "list",
				not_results_found: this.prefix + "recognitionLanguage_"  + "not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Recognition Languages found.')%>",
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"                            },
				{id: "id"                , name: "<%=_T('Id')%>"                                },
				{id: "LanguageName"      , name: "<%=_T('Language Name')%>"                     },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: false,
						erase: false
					}
				}
			]
		});

		return true;
	}

	if (typeof inConcert.ProvisioningTtsAsrEngineprofilesMng !== "function") {
		inConcert.ProvisioningTtsAsrEngineprofilesMng = ProvisioningTtsAsrEngineprofilesMng;
	}

	ProvisioningTtsAsrEngineprofilesMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__init_profiles();
		this.__init_grids();
		this.codecs_mng.initialize();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.is_readonly = function()
	{
		return false;
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.get_grid_dom_id = function()
	{
		return this.prefix + "profiles_list";
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.__make_popup_html = function()
	{
		/*
			Palabra clave: AVOID_GENERAL_OR_DEFAULT_NAMES
			En ingles    : Avoid \"general\" or \"default\" names.
			En espanol   : Evite los nombres \"general\" o \"default\"

			Con este tipo de traducciones el "msgid" de los ".po" no puede llevar comillas dobles, la misma tiene que ser una palabra clave, ya que sino la _T no lo encuentra.
			La traduccion en el ".po" si puede llevar las comillas doble con su caracter de escape.
			Ej. en espanol:

				msgid "AVOID_GENERAL_OR_DEFAULT_NAMES"
				msgstr "Evite los nombres \"general\" o \"default\"."

			Esto si se va a agregar en un atributo html el cual se encuentra entre comillas dobles da conflictos y el atributo no queda correctamente asignado.
			Ej. del render con error:
				<input ....... placeholder="Evite los nombres "general" o "default".
			Como se puede ver el atributo "placeholder" queda con el valor "Evite los nombres "

			Para solucionar esto hay que escapear en javascript los caracteres html, ejecutando la funcion -> inConcert.Utils.escapeAttrib(...);
			Ej. de uso y respuesta:

				1)
					<%=_T("AVOID_GENERAL_OR_DEFAULT_NAMES")%>;
					En el servidor pasar por la _T y traduce la palabra hacia el lado del cliente.
				2)
					Del lado del cliente se ejeucta la siguiente linea para realizar el escapeo de caracteres html.
					inConcert.Utils.escapeAttrib('Evite los nombres "general" o "default".');
					Esto nos retorna
							"Evite los nombres &#34;general&#34; o &#34;default&#34"
					Este valor si puede ser asignado al atritubo "placeholder" y el mismo se reendirzara con el valor correcto.
		*/
		let translated_placeholder_profile_id = inConcert.Utils.escapeAttrib('<%=_T("AVOID_GENERAL_OR_DEFAULT_NAMES")%>');

		let html = '' +
			'<div id="' + this.prefix + 'new_container">' +
				'<div class="card-header d-flex flex-row justify-content-between">' +
					'<div id="' + this.prefix + 'search_panel" class="card-header-search">' +
						'<!-- Aca se renderiza el markup de busqueda, el mismo se genera en el js "conf_form_paginator" -->' +
					'</div>' +
					'<i id="' + this.prefix + 'new_list_validation" class="formFieldValidation formFieldError" hidden></i>' +
					'<button id="' + this.prefix + 'new" class="pageButtonGeneric buttonColorWhite pageButtonNew">' +
						'<i class="fal fa-plus"></i>' +
						'<%=_T([[New Profile]])%>' +
					'</button>' +
				'</div>' +
			'</div>' +
			'<div class="card-block">' +
				'<div id="' + this.prefix + 'container" class="ttsasrTableContainer">' +
					'<div id="' + this.prefix + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none;"></div>' +
					'<table id="' + this.get_grid_dom_id() + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
						'<thead class="viewGridHeader"></thead>' +
						'<tbody class="viewGridBody"></tbody>' +
					'</table>' +
				'</div>' +
				'<div id="' + this.prefix + 'paginator"></div>' +
			'</div>' +
			'<div id="' + this.prefix + 'popup" class="internalPopUpContainer profilesPopUpContainer" style="display:none;">' +
				'<div class="internalPopUpOverlay"></div>' +
				'<div class="internalPopUp">' +
					'<div id="' + this.prefix + 'popup_waiting" class="internalWaitingContainer" style="display:none;">' +
						'<div waiting_message="true" class="internalWaitingMessage"></div>' +
						'<div class="internalWaitingAnimation"></div>' +
						'<div class="internalWaitingOverlay"></div>' +
					'</div>' +
					'<div id="' + this.prefix + 'popup_title" class="internalPopUpTitle">' +
						'<span popup_title_text></span>' +
						'<button popup_close class="internalPopUpCloseButton"><i class="far fa-times"></i></button>' +
					'</div>' +
					'<div class="internalPopUpMiddle">' +
						'<div id="' + this.prefix + 'popup_message" class="inlineMessagesContainer messageInfo" style="display:none;">' +
							'<label class="messageText"></label>' +
						'</div>' +
						'<div>' +
							'<div class="card-block">' +
								'<div class="row">' +
									'<div class="col-6">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_profile_id" class="required"><%=_T([[Name]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_profile_id" class="form-control" type="text" placeholder="' + translated_placeholder_profile_id + '" </input>' +
											'<i id="' + this.prefix + 'popup_profile_id_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-6">' +	
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_description" class="required"><%=_T([[Description]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_description" class="form-control" type="text" placeholder="<%=_T([[Enter the description…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_description_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div class="row">' +
									'<div class="col-6">' +
										'<div engineType="unimrcp" class="form-group">' +
											'<label for="' + this.prefix + 'popup_provider" class="required"><%=_T([[Provider]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_provider" class="form-control" placeholder="<%=_T([[Select the provider…]])%>"></select>' +
											'<i id="' + this.prefix + 'popup_provider_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-6">' +
										'<div engineType="unimrcp" class="form-group">' +
											'<label for="' + this.prefix + 'popup_provider_type" class="required"><%=_T([[Provider Type]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_provider_type" class="form-control" placeholder="<%=_T([[Select the provider type…]])%>"></select>' +
											'<i id="' + this.prefix + 'popup_provider_type_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="speechmatic" class="row">' +
									'<div class="col-12">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_speechmatic_endpoint" class="required"><%=_T([[Uri]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_speechmatic_endpoint" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_speechmatic_endpoint_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="microsoft" class="row">' +
									'<div class="col-12">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_microsoft_endpoint" class="required"><%=_T([[Uri]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_microsoft_endpoint" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_microsoft_endpoint_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_server_ip" class="required"><%=_T([[Server IP]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_server_ip" class="form-control" type="text" placeholder="<%=_T([[Enter the value…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_server_ip_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_server_port" class="required"><%=_T([[Server Port]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_server_port" class="form-control" type="text" placeholder="<%=_T([[Enter the value…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_server_port_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_client_port" class="required"><%=_T([[Client Port]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_client_port" class="form-control" type="text" placeholder="<%=_T([[Enter the value…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_client_port_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_client_ext_ip" class=""><%=_T([[Client External (NAT) IP]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_client_ext_ip" class="form-control" type="text" placeholder="<%=_T([[Enter the value…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_client_ext_ip_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtp_ext_ip" class=""><%=_T([[RTP External (NAT) IP]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_rtp_ext_ip" class="form-control" type="text" placeholder="<%=_T([[Enter the value…]])%>"></input>' +
											'<i id="' + this.prefix + 'popup_rtp_ext_ip_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +								
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_version" class=""><%=_T([[Version]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_version" class="form-control"></select>' +
											'<i id="' + this.prefix + 'popup_version_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_sip_transport" class=""><%=_T([[SIP Transport]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_sip_transport" class="form-control"></select>' +
											'<i id="' + this.prefix + 'popup_sip_transport_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_ua_name" class=""><%=_T([[UA Name]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_ua_name" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_ua_name_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_sdp_origin" class=""><%=_T([[SDP Origin]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_sdp_origin" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_sdp_origin_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_server_username" class=""><%=_T([[Server Username]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_server_username" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_server_username_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtp_port_min" class=""><%=_T([[RTP Port Min]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_rtp_port_min" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_rtp_port_min_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtp_port_max" class=""><%=_T([[RTP Port Max]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_rtp_port_max" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_rtp_port_max_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-12">' +
										'<div class="form-group">' +
											'<div class="form-check">' +
												'<label for="' + this.prefix + 'popup_force_destination" class="form-check-label form-check-label-title">' +
													'<input id="' + this.prefix + 'popup_force_destination" class="form-check-input" type="checkbox"></input>' +
													'<%=_T([[Force Destination]])%>' +
												'</label>' +
												'<i id="' + this.prefix + 'popup_force_destination_validation" class="formFieldValidation formFieldError" hidden></i>' +
											'</div>' +
											'<span class="fa fa-question-circle" title="<%=_T([[When enabled, the system will skip the alternate IP generated by the server in response to the first contact request to the MRCP.]])%>"></span>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_min_playout_delay" class=""><%=_T([[Min Playout Delay (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_min_playout_delay" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_min_playout_delay_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_max_playout_delay" class=""><%=_T([[Max Playout Delay (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_max_playout_delay" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_max_playout_delay_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_playout_delay" class=""><%=_T([[Playout Delay (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_playout_delay" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_playout_delay_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-3">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_ptime" class=""><%=_T([[Ptime (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_ptime" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_ptime_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-12">' +
										'<div class="form-group">' +
											'<div class="form-check form-switch">' +
												'<label for="' + this.prefix + 'popup_rtcp" class="form-check-label">' +
													'<input id="' + this.prefix + 'popup_rtcp" class="form-check-input" type="checkbox"></input>' +
													'<span>' +
														'<%=_T([[RTCP]])%>' +
													'</span>' +
												'</label>' +
												'<i id="' + this.prefix + 'popup_rtcp_validation" class="formFieldValidation formFieldError" hidden></i>' +
											'</div>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<div engineType="unimrcp" class="row">' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtcp_bye" class=""><%=_T([[RTCP Bye]])%><%=_T([[:]])%></label>' +
											'<select id="' + this.prefix + 'popup_rtcp_bye" class="form-control"></select>' +
											'<i id="' + this.prefix + 'popup_rtcp_bye_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtcp_tx_interval" class=""><%=_T([[RTCP Tx Interval (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_rtcp_tx_interval" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_rtcp_tx_interval_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
									'<div class="col-4">' +
										'<div class="form-group">' +
											'<label for="' + this.prefix + 'popup_rtcp_rx_resolution" class=""><%=_T([[RTCP Rx Resolution (msec)]])%><%=_T([[:]])%></label>' +
											'<input id="' + this.prefix + 'popup_rtcp_rx_resolution" class="form-control" type="text"></input>' +
											'<i id="' + this.prefix + 'popup_rtcp_rx_resolution_validation" class="formFieldValidation formFieldError" hidden></i>' +
										'</div>' +
									'</div>' +
								'</div>' +
								'<!-- codecs -->' +
								'<div engineType="unimrcp" id="prov_form_ttsasr_engine_profiles_codecs_panel" class="">' +
									'<!-- Este markup se genera en el javascript "codecs_mng.js" -->' +
								'</div>' +
								'<hr />' +
								'<div class="card no-card">' +
									'<div class="card-header">' +
										'<h5><%=_T([[Supplier Provisions]])%></h5>' +
									'</div>' +
									'<div class="accordion mt-3" id="accordion-supplier-provisiongs">' +
									'<!-- voices -->' +
										'<div class="card" id="prov_form_ttsasr_engine_profiles_voices_panel">' +
											'<div class="card-header" id="headingOne">' +
												'<h2 class="mb-0">' +
													'<button class="btn btn-link btn-block text-left" type="button" data-toggle="collapse" data-target="#collapseOne" aria-expanded="true" aria-controls="collapseOne">' +
														'<%=_T([[Available voices]])%>' +
														'<i class="fas fa-chevron-down"></i>' +
													'</button>' +
												'</h2>' +
											'</div>' +
											'<div id="collapseOne" class="collapse show" aria-labelledby="headingOne" data-parent="#accordion-supplier-provisiongs">' +
												'<div class="card-body">' +
													'<div id="' + this.prefix + 'voice_' + 'container" class="tableContainerBox">' +
														'<div id="' + this.prefix + 'voice_'  + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none;"></div>' +
														'<table id="' + this.prefix + 'voice_' + "list" + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
															'<thead class="viewGridHeader"></thead>' +
															'<tbody class="viewGridBody"></tbody>' +
														'</table>' +
													'</div>' +
													'<div id="' + this.prefix + 'voice_' + 'paginator"></div>' +
												'</div>' +
											'</div>' +
										'</div>' +
										'<!-- grammars -->' +
										'<div class="card" id="prov_form_ttsasr_engine_profiles_grammars_panel">' +
											'<div class="card-header" id="headingTwo">' +
												'<h2 class="mb-0">' +
													'<button class="btn btn-link btn-block text-left collapsed" type="button" data-toggle="collapse" data-target="#collapseTwo" aria-expanded="false" aria-controls="collapseTwo">' +
														'<%=_T([[Builtin grammars]])%>' +
														'<i class="fas fa-chevron-down"></i>' +
													'</button>' +
												'</h2>' +
											'</div>' +
											'<div id="collapseTwo" class="collapse" aria-labelledby="headingTwo" data-parent="#accordion-supplier-provisiongs">' +
												'<div class="card-body">' +
													'<div id="' + this.prefix + 'grammar_' + 'container" class="tableContainerBox">' +
														'<div id="' + this.prefix + 'grammar_'  + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none;"></div>' +
														'<table id="' + this.prefix + 'grammar_' + "list" + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
															'<thead class="viewGridHeader"></thead>' +
															'<tbody class="viewGridBody"></tbody>' +
														'</table>' +
													'</div>' +
													'<div id="' + this.prefix + 'grammar_' + 'paginator"></div>' +
												'</div>' +
											'</div>' +
										'</div>' +
										'<!-- recognitionLanguages -->' +
										'<div class="card" id="prov_form_ttsasr_engine_profiles_recognitionLanguages_panel">' +
											'<div class="card-header" id="headingThree">' +
												'<h2 class="mb-0">' +
												'<button class="btn btn-link btn-block text-left collapsed" type="button" data-toggle="collapse" data-target="#collapseThree" aria-expanded="false" aria-controls="collapseThree">' +
													'<%=_T([[Recognition Languages]])%>' +
													'<i class="fas fa-chevron-down"></i>' +
												'</button>' +
												'</h2>' +
											'</div>' +
											'<div id="collapseThree" class="collapse" aria-labelledby="headingThree" data-parent="#accordion-supplier-provisiongs">' +
												'<div class="card-body">' +
													'<div id="' + this.prefix + 'recognitionLanguage_' + 'container" class="tableContainerBox">' +
														'<div id="' + this.prefix + 'recognitionLanguage_'  + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none;"></div>' +
															'<table id="' + this.prefix + 'recognitionLanguage_' + "list" + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
																'<thead class="viewGridHeader"></thead>' +
																'<tbody class="viewGridBody"></tbody>' +
															'</table>' +
														'</div>' +
														'<div id="' + this.prefix + 'recognitionLanguage_' + 'paginator"></div>' +
													'</div>' +
												'</div>' +
											'</div>' +
										'</div>' +
									'</div>' +
								'</div>' +
							'</div>' +
						'</div>' +
					'<div>' +
					'<div class="internalPopUpButtons">' +
					'<input id="' + this.prefix + 'popup_close" class="pageButtonGeneric buttonColorWhite float-right" type="button" value="<%=_T([[Close]])%>" />' +
						'<input id="' + this.prefix + 'popup_create_and_continue" class="pageButtonGeneric buttonColorBlue float-right" type="button" value="<%=_T([[Confirm & continue]])%>" />' +
						'<input id="' + this.prefix + 'popup_create" class="pageButtonGeneric buttonColorBlue float-right" type="button" value="<%=_T([[Create]])%>" />' +
					'</div>' +
				'</div>' +
			'</div>';

		const $container = $("#" + this.container_id);
		$container.html(html);
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.__init_grids = function()
	{
		this.__init_grid_profiles();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.__init_grid_profiles = function()
	{
		const self = this;

		var must_show_button = function(row)
		{
			return !self.isUsedByVCC(row.id);
		};

		this.m_grid_profiles = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner,
			row_id: "id",
			display_name: "id",
			rows_per_page: 10,
			show_at_least: 9,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Profiles ...')%>",
			ids: {
				paginator: this.prefix + "paginator",
				grid: this.get_grid_dom_id(),
				not_results_found: this.prefix + "not_results_found",
				container_new_btn: this.prefix + "new_container",
				new_item: this.prefix + "new",
			//search
				search_builtin_panel: this.prefix + "search_panel",
				search_container    : this.prefix + "search_container",
				search_text         : this.prefix + "search_text",
				search_reset        : this.prefix + "reset_text",
				search_apply        : this.prefix + "apply_text",
			//popup0
				popup: this.prefix + "popup",
				popup_msg: this.prefix + "popup_message",
				popup_create: this.prefix + "popup_create",
				popup_create_and_continue: this.prefix + "popup_create_and_continue",
				popup_title: this.prefix + "popup_title",
				popup_close: this.prefix + "popup_close",
				popup_input_id: this.prefix + "popup_question",
				popup_waiting: this.prefix + "popup_waiting"
			},
			strings: {
				not_results_found: "<%=_T('No Profiles found.')%>",
				can_remove       : "<%=_T('Do you want to delete Profile: {0} ?')%>"
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"       },
				{id: "id"                , name: "<%=_T('Profile Name')%>" },
				{id: "description"       , name: "<%=_T('Description')%>"  },
				{id: "provider"          , name: "<%=_T('Provider')%>"     },
				{id: "provider_type"     , name: "<%=_T('Provider Type')%>"},
				{id: "endpoint"          , name: "<%=_T('Endpoint')%>"     },
			// El resto de columnas de datos quedan ocultas
				{
					id: "_action",
					name: "",
					system_action: {
						edit: true,
						erase: /*true*/{
							must_show: function(row) {
								return must_show_button(row);
							}
						}
					}
				}
			],
			filter: {
				fields: {
					"id"               : {control: null},
					"description"      : {control: null},
					"provider"         : {control: null},
					"provider_type"    : {control: null}
				}
			},
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_row(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row_data) {
					self.on_open_popup(row_data);
				},
				on_removed_row: function(/*row, callback = $.noop*/) {},
 				on_closed_popup: function() {
 					self.on_closed_popup();
 				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one Profile.')%>",
				required: true
			}
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.start_grid_validation = function()
	{
		let response = {
			success: true
		};
		let success = this.m_grid_profiles.start_validation().success;
		if (!success) {
			response.success = false;
			response.message = this.m_grid_profiles.get_validation_message_err("required");
		}
		return response;
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_profile = null;
		this.m_current_profile = null;
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.add_or_update_row = function(callback/*, _, current_popup*/)
	{
		callback = callback || $.noop;

		const self = this;

		this.validate_profile_to_add_or_update(function(response)
		{
			if (!response.success) {
				callback(false);
				return;
			}
			let msg_response = {
				msg : "",
				type: "Info"
			};
	
			if (self.m_is_new_profile) {
				msg_response.msg = "<%=_T('Profile was successfully added.')%>";
				let must_add_many = false;
				callback(true, response.item, msg_response, must_add_many);
			}
			else {
				msg_response.msg = "<%=_T('Profile was successfully updated.')%>";
				self.m_current_profile = inConcert.Utils.cloneObject(response.item);
				callback(true, self.m_current_profile, msg_response);
			}
	
			self.clean_up_controls_profile();
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.clean_up_controls_profile = function()
	{
		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		const $description = $("#" + this.prefix + "popup_description");
		const $provider = $("#" + this.prefix + "popup_provider");
		const $provider_type = $("#" + this.prefix + "popup_provider_type");
		this.owner.readonly_control($profile_id, false, true);
		$profile_id.val("").focus();
		$description.val("");
		$provider.val(this.owner.currentEngine === "speechmatic" ? "speechmatic" : "");
		$provider_type.val(this.owner.currentEngine === "speechmatic" ? "ASR" : "");
		$provider.change();	//Force initialize voices and grammars grids
		
		//EngineType dependant controls
		if(this.owner.currentEngine === "unimrcp") {
			//Cleanup de controles asociados a unimrcp
			const $version = $("#" + this.prefix + "popup_version");
			const $server_ip = $("#" + this.prefix + "popup_server_ip");
			const $server_port = $("#" + this.prefix + "popup_server_port");
			const $server_username = $("#" + this.prefix + "popup_server_username");
			const $force_destination = $("#" + this.prefix + "popup_force_destination");
			const $client_port = $("#" + this.prefix + "popup_client_port");
			const $sip_transport = $("#" + this.prefix + "popup_sip_transport");
			const $ua_name = $("#" + this.prefix + "popup_ua_name");
			const $sdp_origin = $("#" + this.prefix + "popup_sdp_origin");
			const $rtp_port_min = $("#" + this.prefix + "popup_rtp_port_min");
			const $rtp_port_max = $("#" + this.prefix + "popup_rtp_port_max");
			const $playout_delay = $("#" + this.prefix + "popup_playout_delay");
			const $ptime = $("#" + this.prefix + "popup_ptime");
			const $min_playout_delay = $("#" + this.prefix + "popup_min_playout_delay");
			const $max_playout_delay = $("#" + this.prefix + "popup_max_playout_delay");
			const $rtcp = $("#" + this.prefix + "popup_rtcp");
			const $rtcp_bye = $("#" + this.prefix + "popup_rtcp_bye");
			const $rtcp_tx_interval = $("#" + this.prefix + "popup_rtcp_tx_interval");
			const $rtcp_rx_resolution = $("#" + this.prefix + "popup_rtcp_rx_resolution");
			const $client_ext_ip = $("#" + this.prefix + "popup_client_ext_ip");
			const $rtp_ext_ip = $("#" + this.prefix + "popup_rtp_ext_ip");

			// clean but aplying known default values
			$version.val("2");
			$server_ip.val("");
			$server_port.val("");
			$server_username.val("test");
			$force_destination.prop("checked", false);
			$client_port.val("");
			$sip_transport.val("udp");
			$ua_name.val("Asterisk");
			$sdp_origin.val("Asterisk");
			$rtp_port_min.val("4000");
			$rtp_port_max.val("5000");
			$playout_delay.val("50");
			$ptime.val("20");
			$min_playout_delay.val("50");
			$max_playout_delay.val("200");
			$rtcp.prop("checked", false);
			$rtcp_bye.val("2");
			$rtcp_tx_interval.val("5000");
			$rtcp_rx_resolution.val("1000");
			$client_ext_ip.val("");
			$rtp_ext_ip.val("");
			
			this.codecs_mng.empty_controls();
		}
		else if(this.owner.currentEngine === "speechmatic") {
			//Cleanup de controles asociados a speechmatic
			const $speechmatic_endpoint = $("#" + this.prefix + "popup_speechmatic_endpoint");
			$speechmatic_endpoint.val("");
		}
		else if(this.owner.currentEngine === "microsoft") {
			//Cleanup de controles asociados a microsoft
			const $microsoft_endpoint = $("#" + this.prefix + "popup_microsoft_endpoint");
			$microsoft_endpoint.val("");
		}
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.on_open_popup = function(item)
	{
		const self = this;
		const $title = $("#" + this.prefix + "popup_title").find("[popup_title_text]");
		this.m_is_new_profile = item ? false: true;
		this.m_current_profile = item || null;

		this.clean_up_controls_profile();
		
		this.showEnginesControls(this.owner.currentEngine);

		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		const $description = $("#" + this.prefix + "popup_description");
		const $provider = $("#" + this.prefix + "popup_provider");
		const $provider_type = $("#" + this.prefix + "popup_provider_type");

		if (this.m_is_new_profile) {
			$title.text(FORMAT("<%=_T('New Profile')%>", this.owner.currentEngine.toUpperCase()));
			this.owner.readonly_control($profile_id, false);
			if(this.owner.currentEngine === "speechmatic") {
				$provider.val("speechmatic");
				$provider_type.val("ASR");
				this.owner.readonly_control($provider, true);
				this.owner.readonly_control($provider_type, true);
			}
			else {
				this.owner.readonly_control($provider, false);
				this.owner.readonly_control($provider_type, false);
			}
		}
		else {
			$title.text(FORMAT("<%=_T('Update Profile')%>", this.owner.currentEngine.toUpperCase()));
			$profile_id.val(item.id);
			$description.val(item.description);
			$provider.val(item.provider);
			$provider_type.val(item.provider_type);
			this.owner.readonly_control($profile_id, true);
			this.owner.readonly_control($provider, this.isUsedByVCC(item.id) || this.owner.currentEngine === "speechmatic");
			this.owner.readonly_control($provider_type, this.isUsedByVCC(item.id) || this.owner.currentEngine === "speechmatic");
			
		}

		$provider.change(function() {
			$provider_type.change();
		});

		$provider_type.change(function() {
			let selected_provider = $provider.val();
			let selected_provider_type = $( this ).val();
			let voices_info = [];
			let grammars_info = [];
			let recognitionLanguages_info = [];
			self.owner.get_providers().forEach(function(provider) {
				if(provider.id === selected_provider) {
					if(selected_provider_type === "TTS" || selected_provider_type === "TTS-ASR") {
						voices_info = provider.voices;
					}
					if(selected_provider_type === "ASR" || selected_provider_type === "TTS-ASR") {
						grammars_info = provider.grammars;
						recognitionLanguages_info = provider.recognitionLanguages;
					}
				}
			});
			self.m_grid_voices.initialize($.noop, voices_info);
			self.m_grid_grammars.initialize($.noop, grammars_info);
			self.m_grid_recognitionLanguages.initialize($.noop, recognitionLanguages_info);
		});

		$provider.change();	//initialize voices and grammars grids

		//EngineType dependant controls
		if(this.owner.currentEngine === "unimrcp") {
			const $version = $("#" + this.prefix + "popup_version");
			const $server_ip = $("#" + this.prefix + "popup_server_ip");
			const $server_port = $("#" + this.prefix + "popup_server_port");
			const $server_username = $("#" + this.prefix + "popup_server_username");
			const $force_destination = $("#" + this.prefix + "popup_force_destination");
			const $client_port = $("#" + this.prefix + "popup_client_port");
			const $sip_transport = $("#" + this.prefix + "popup_sip_transport");
			const $ua_name = $("#" + this.prefix + "popup_ua_name");
			const $sdp_origin = $("#" + this.prefix + "popup_sdp_origin");
			const $rtp_port_min = $("#" + this.prefix + "popup_rtp_port_min");
			const $rtp_port_max = $("#" + this.prefix + "popup_rtp_port_max");
			const $playout_delay = $("#" + this.prefix + "popup_playout_delay");
			const $ptime = $("#" + this.prefix + "popup_ptime");
			const $min_playout_delay = $("#" + this.prefix + "popup_min_playout_delay");
			const $max_playout_delay = $("#" + this.prefix + "popup_max_playout_delay");
			const $rtcp = $("#" + this.prefix + "popup_rtcp");
			const $rtcp_bye = $("#" + this.prefix + "popup_rtcp_bye");
			const $rtcp_tx_interval = $("#" + this.prefix + "popup_rtcp_tx_interval");
			const $rtcp_rx_resolution = $("#" + this.prefix + "popup_rtcp_rx_resolution");
			const $codecs = $("#" + this.prefix + "codecs_panel");
			const $client_ext_ip = $("#" + this.prefix + "popup_client_ext_ip");
			const $rtp_ext_ip = $("#" + this.prefix + "popup_rtp_ext_ip");

			if (this.m_is_new_profile) {
				// apply known default values
				$version.val("2");
				$server_username.val("test");
				$sip_transport.val("udp");
				$ua_name.val("Asterisk");
				$sdp_origin.val("Asterisk");
				$rtp_port_min.val("4000");
				$rtp_port_max.val("5000");
				$playout_delay.val("50");
				$ptime.val("20");
				$min_playout_delay.val("50");
				$max_playout_delay.val("200");
				$rtcp_bye.val("2");
				$rtcp_tx_interval.val("5000");
				$rtcp_rx_resolution.val("1000");
				$client_ext_ip.val("");
				$rtp_ext_ip.val("");
				return;
			}
			// editando
			//Carga inicial de controles asociados a unimrcp
			$version.val(item.version);
			$server_ip.val(item["server-ip"]);
			$server_port.val(item["server-port"]);
			$server_username.val(item["server-username"]);
			$force_destination.prop("checked", item["force-destination"] === "1");
			$client_port.val(item["client-port"]);
			$sip_transport.val(item["sip-transport"]);
			$ua_name.val(item["ua-name"]);
			$sdp_origin.val(item["sdp-origin"]);
			$rtp_port_min.val(item["rtp-port-min"]);
			$rtp_port_max.val(item["rtp-port-max"]);
			$playout_delay.val(item["playout-delay"]);
			$ptime.val(item.ptime);
			$min_playout_delay.val(item["min-playout-delay"]);
			$max_playout_delay.val(item["max-playout-delay"]);
			$rtcp.prop("checked", item.rtcp === "1");
			$rtcp_bye.val(item["rtcp-bye"]);
			$rtcp_tx_interval.val(item["rtcp-tx-interval"]);
			$rtcp_rx_resolution.val(item["rtcp-rx-resolution"]);
			$client_ext_ip.val(item["client-ext-ip"]);
			$rtp_ext_ip.val(item["rtp-ext-ip"]);

			if(item.codecs) {
				let codecsArr = item.codecs.split(" ");
				codecsArr.forEach(function(codec) {
					$codecs.find("[codec_val='" + codec + "']").prop("checked", true);
				});
			}
		}
		else if(this.owner.currentEngine === "speechmatic") {
			const $speechmatic_endpoint = $("#" + this.prefix + "popup_speechmatic_endpoint");
			if (this.m_is_new_profile) {
				return;
			}
			// editando
			//Carga inicial de controles asociados a speechmatic
			$speechmatic_endpoint.val(item.endpoint);
		}
		else if(this.owner.currentEngine === "microsoft") {
			const $microsoft_endpoint = $("#" + this.prefix + "popup_microsoft_endpoint");
			if (this.m_is_new_profile) {
				return;
			}
			// editando
			//Carga inicial de controles asociados a microsoft
			$microsoft_endpoint.val(item.endpoint);
		}
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.validate_profile_to_add_or_update = function(callback = $.noop)
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Profile.')%>"};

		let item = {};
		let is_new = false;
		if (this.m_current_profile) {
			item = this.m_current_profile;
		}
		else {
			is_new = true;
		}

		const $profile_id = $("#" + this.prefix + "popup_profile_id");
		let profile_id = $profile_id.val();

		const $description = $("#" + this.prefix + "popup_description");
		let description = $description.val();

		let current_all_profile = this.m_grid_profiles.get_items();
		let current_all_profile_map = inConcert.Utils.arrayToMap(current_all_profile || [],
			function(profile) {
				return profile;
			},
			function(profile) {
				// el ID
				return profile.id;
			}
		);

		let validation_result;
		if (is_new) {
			// PROFILE ID validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(profile_id, "required");
			if (!validation_result.success) {
				// valida que el profile_id tenga al menos un caracter
				response.msg = "<%=_T('Profile Name is required.')%>";
				response.$ctrl = $profile_id;
				this.show_ctrl_err(response);
				return callback(response);
			}

			// Already exist id validation
			var show_already_id_error = function() {
				response.msg = "<%=_T('Profile already exist with that name.')%>";
				response.$ctrl = $profile_id;
				self.show_ctrl_err(response);
				return response;
			};
			const already_id = current_all_profile_map[profile_id];
			if (already_id) {
				return callback(show_already_id_error());
			}
		}
		// PROFILE DESCRIPTION validation -----
		//
		// Required validation
		validation_result = inConcert.Utils.validate(description, "required");
		if (!validation_result.success) {
			// valida que el description tenga al menos un caracter
			response.msg = "<%=_T('Description is required.')%>";
			response.$ctrl = $description;
			this.show_ctrl_err(response);
			return callback(response);
		}

		var profile = {};

		const $provider = $("#" + this.prefix + "popup_provider");
		let provider = $provider.val();

		const $provider_type = $("#" + this.prefix + "popup_provider_type");
		let provider_type = $provider_type.val();

		// PROVIDER validation -----
		//
		// Required validation
		validation_result = inConcert.Utils.validate(provider, "required");
		if (!validation_result.success) {
			// valida que el provider tenga al menos un caracter
			response.msg = "<%=_T('Provider is required.')%>";
			response.$ctrl = $provider;
			this.show_ctrl_err(response);
			return callback(response);
		}

		// PROVIDER TYPE validation -----
		//
		// Required validation
		validation_result = inConcert.Utils.validate(provider_type, "required");
		if (!validation_result.success) {
			// valida que el provider_type tenga al menos un caracter
			response.msg = "<%=_T('Provider Type is required.')%>";
			response.$ctrl = $provider_type;
			this.show_ctrl_err(response);
			return callback(response);
		}
		
		let totalVoices = this.m_grid_voices.get_items().length;
		let totalRecognitionLanguages = this.m_grid_recognitionLanguages.get_items().length;
		
		if((provider_type === "TTS" || provider_type === "TTS-ASR") && totalVoices === 0) {
			// valida que el provider_type TTS tenga al menos una voz disponible
			response.msg = FORMAT("<%=_T('Provider Type {0} requires to have at least one voice available.')%>", provider_type);
			response.$ctrl = $provider_type;
			this.show_ctrl_err(response);
			return callback(response);
		}
		
		if((provider_type === "ASR" || provider_type === "TTS-ASR") && totalRecognitionLanguages === 0) {
			// valida que el provider_type ASR tenga al menos una grammar y un recognition language disponibles
			response.msg = FORMAT("<%=_T('Provider Type {0} requires to have at least one recognition language available.')%>", provider_type);
			response.$ctrl = $provider_type;
			this.show_ctrl_err(response);
			return callback(response);
		}
		
		//EngineType dependant controls
		if(this.owner.currentEngine === "unimrcp") {
			const $version = $("#" + this.prefix + "popup_version");
			let version = $version.val();

			const $server_ip = $("#" + this.prefix + "popup_server_ip");
			let server_ip = $server_ip.val();

			const $server_port = $("#" + this.prefix + "popup_server_port");
			let server_port = $server_port.val();

			const $server_username = $("#" + this.prefix + "popup_server_username");
			let server_username = $server_username.val();

			const $force_destination = $("#" + this.prefix + "popup_force_destination");
			let force_destination = $force_destination.prop("checked") ? "1" : "0";

			const $client_port = $("#" + this.prefix + "popup_client_port");
			let client_port = $client_port.val();
			
			const $sip_transport = $("#" + this.prefix + "popup_sip_transport");
			let sip_transport = $sip_transport.val();

			const $ua_name = $("#" + this.prefix + "popup_ua_name");
			let ua_name = $ua_name.val();

			const $sdp_origin = $("#" + this.prefix + "popup_sdp_origin");
			let sdp_origin = $sdp_origin.val();

			const $rtp_port_min = $("#" + this.prefix + "popup_rtp_port_min");
			let rtp_port_min = $rtp_port_min.val();

			const $rtp_port_max = $("#" + this.prefix + "popup_rtp_port_max");
			let rtp_port_max = $rtp_port_max.val();

			const $playout_delay = $("#" + this.prefix + "popup_playout_delay");
			let playout_delay = $playout_delay.val();

			const $ptime = $("#" + this.prefix + "popup_ptime");
			let ptime = $ptime.val();

			const $min_playout_delay = $("#" + this.prefix + "popup_min_playout_delay");
			let min_playout_delay = $min_playout_delay.val();

			const $max_playout_delay = $("#" + this.prefix + "popup_max_playout_delay");
			let max_playout_delay = $max_playout_delay.val();

			const $rtcp = $("#" + this.prefix + "popup_rtcp");
			let rtcp = $rtcp.prop("checked") ? "1" : "0";

			const $rtcp_bye = $("#" + this.prefix + "popup_rtcp_bye");
			let rtcp_bye = $rtcp_bye.val();

			const $rtcp_tx_interval = $("#" + this.prefix + "popup_rtcp_tx_interval");
			let rtcp_tx_interval = $rtcp_tx_interval.val();

			const $rtcp_rx_resolution = $("#" + this.prefix + "popup_rtcp_rx_resolution");
			let rtcp_rx_resolution = $rtcp_rx_resolution.val();

			const $client_ext_ip = $("#" + this.prefix + "popup_client_ext_ip");
			let client_ext_ip = $client_ext_ip.val();
			
			const $rtp_ext_ip = $("#" + this.prefix + "popup_rtp_ext_ip");
			let rtp_ext_ip = $rtp_ext_ip.val();

			// SERVER IP validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(server_ip, "required");
			if (!validation_result.success) {
				// valida que el server_ip tenga al menos un caracter
				response.msg = "<%=_T('Server IP is required.')%>";
				response.$ctrl = $server_ip;
				this.show_ctrl_err(response);
				return callback(response);
			}

			// SERVER PORT validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(server_port, "required");
			if (!validation_result.success) {
				// valida que el server port tenga al menos un caracter
				response.msg = "<%=_T('Server Port is required.')%>";
				response.$ctrl = $server_port;
				this.show_ctrl_err(response);
				return callback(response);
			}
			//Port range
			const port_range = {between: {from: 0, to: 65535}};
			validation_result = inConcert.Utils.validate(server_port, "range", port_range);
			if (!validation_result.success) {
				// valida que el server port sea un numero de puerto valido
				response.msg = FORMAT('<%=_T("Server Port must be a valid port number between {0} and {1}.")%>', port_range.between.from, port_range.between.to);
				response.$ctrl = $server_port;
				this.show_ctrl_err(response);
				return callback(response);
			}
			//Server and port exist validation
			current_all_profile_map = inConcert.Utils.arrayToMap(current_all_profile || [],
				function(profile) {
					return profile;
				},
				function(profile) {
					// el ID
					return self.make_ip_port_id(profile["server-ip"], profile["server-port"]);
				}
			);
			var show_already_server_error = function() {
				response.msg = "<%=_T('Profile already exist for this server ip and port.')%>";
				response.$ctrl = $server_ip;
				self.show_ctrl_err(response);
				return response;
			};

			let new_uuid = this.make_ip_port_id(server_ip, server_port);
			const already_server = current_all_profile_map[new_uuid];
			if (already_server && already_server.id !== profile_id) {
				return callback(show_already_server_error());
			}
			// CLIENT PORT validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(client_port, "required");
			if (!validation_result.success) {
				// valida que el client port tenga al menos un caracter
				response.msg = "<%=_T('Client Port is required.')%>";
				response.$ctrl = $client_port;
				this.show_ctrl_err(response);
				return callback(response);
			}
			//Port range
			validation_result = inConcert.Utils.validate(client_port, "range", port_range);
			if (!validation_result.success) {
				// valida que el client port sea un numero de puerto valido
				response.msg = FORMAT('<%=_T("Client Port must be a valid port number between {0} and {1}.")%>', port_range.between.from, port_range.between.to);
				response.$ctrl = $client_port;
				this.show_ctrl_err(response);
				return callback(response);
			}
			// RTP MIN PORT validation -----
			if(rtp_port_min) {
				//Port range
				validation_result = inConcert.Utils.validate(rtp_port_min, "range", port_range);
				if (!validation_result.success) {
					// valida que el rtp_port_min port sea un numero de puerto valido
					response.msg = FORMAT('<%=_T("RTP Port Min must be a valid port number between {0} and {1}.")%>', port_range.between.from, port_range.between.to);
					response.$ctrl = $rtp_port_min;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			// RTP MAX PORT validation -----
			if(rtp_port_max) {
				//Port range
				validation_result = inConcert.Utils.validate(rtp_port_max, "range", port_range);
				if (!validation_result.success) {
					// valida que el rtp_port_max port sea un numero de puerto valido
					response.msg = FORMAT('<%=_T("RTP Port Max must be a valid port number between {0} and {1}.")%>', port_range.between.from, port_range.between.to);
					response.$ctrl = $rtp_port_max;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			const playout_delay_range = {between: {from: 10, to: 1000}};
			if(playout_delay) {
				validation_result = inConcert.Utils.validate(playout_delay, "range", playout_delay_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("Playout Delay must be a number between {0} and {1}.")%>', playout_delay_range.between.from, playout_delay_range.between.to);
					response.$ctrl = $playout_delay;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			if(ptime) {
				const ptime_range = {between: {from: 1, to: 100}};
				validation_result = inConcert.Utils.validate(ptime, "range", ptime_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("Ptime must be a number between {0} and {1}.")%>', ptime_range.between.from, ptime_range.between.to);
					response.$ctrl = $ptime;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			
			if(min_playout_delay) {
				validation_result = inConcert.Utils.validate(min_playout_delay, "range", playout_delay_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("Min Playout Delay must be a number between {0} and {1}.")%>', playout_delay_range.between.from, playout_delay_range.between.to);
					response.$ctrl = $min_playout_delay;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			if(max_playout_delay) {
				validation_result = inConcert.Utils.validate(max_playout_delay, "range", playout_delay_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("Max Playout Delay must be a number between {0} and {1}.")%>', playout_delay_range.between.from, playout_delay_range.between.to);
					response.$ctrl = $max_playout_delay;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			const interval_range = {between: {from: 0, to: 10000}};
			if(rtcp_tx_interval) {
				validation_result = inConcert.Utils.validate(rtcp_tx_interval, "range", interval_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("RTCP Tx Interval must be a number between {0} and {1}.")%>', interval_range.between.from, interval_range.between.to);
					response.$ctrl = $rtcp_tx_interval;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			if(rtcp_rx_resolution) {
				validation_result = inConcert.Utils.validate(rtcp_rx_resolution, "range", interval_range);
				if (!validation_result.success) {
					response.msg = FORMAT('<%=_T("RTCP Rx Resolution must be a number between {0} and {1}.")%>', interval_range.between.from, interval_range.between.to);
					response.$ctrl = $rtcp_rx_resolution;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}
			//Default values
			version = version || "2";
			server_username = server_username || "test";
			sip_transport = sip_transport || "udp";
			ua_name = ua_name || "Asterisk";
			sdp_origin = sdp_origin || "Asterisk";
			rtp_port_min = rtp_port_min || "4000";
			rtp_port_max = rtp_port_max || "5000";
			playout_delay = playout_delay || "50";
			ptime = ptime || "20";
			min_playout_delay = min_playout_delay || "50";
			max_playout_delay = max_playout_delay || "200";
			rtcp_bye = rtcp_bye || "2";
			rtcp_tx_interval = rtcp_tx_interval || "5000";
			rtcp_rx_resolution = rtcp_rx_resolution || "1000";

			let codecsObject = {};
			this.codecs_mng.get(codecsObject);
			
			if(!codecsObject.codecs){
				response.msg = "<%=_T('Must select at least one codec.')%>";
				//response.$ctrl = $rtcp_tx_interval;
				this.show_ctrl_err(response);
				return callback(response);
			}
			profile = {
				id : profile_id,
				description : description,
				provider : provider,
				provider_type : provider_type,
				version : version,
				'server-ip' : server_ip,
				'server-port' : server_port,
				'server-username' : server_username,
				'force-destination' : force_destination, 
				'client-port' : client_port, 
				'sip-transport' : sip_transport, 
				'ua-name' : ua_name, 
				'sdp-origin' : sdp_origin, 
				'rtp-port-min' : rtp_port_min, 
				'rtp-port-max' : rtp_port_max, 
				'playout-delay' : playout_delay, 
				'ptime' : ptime, 
				'min-playout-delay' : min_playout_delay, 
				'max-playout-delay' : max_playout_delay, 
				'rtcp' : rtcp, 
				'rtcp-bye' : rtcp_bye, 
				'rtcp-tx-interval' : rtcp_tx_interval, 
				'rtcp-rx-resolution' : rtcp_rx_resolution, 
				'codecs': codecsObject.codecs,
				'client-ext-ip': client_ext_ip,
				'rtp-ext-ip': rtp_ext_ip
			};
		}
		else if(this.owner.currentEngine === "speechmatic") {
			//Validar controles asociados a speechmatic
			const $speechmatic_endpoint = $("#" + this.prefix + "popup_speechmatic_endpoint");
			let speechmatic_endpoint = $speechmatic_endpoint.val();
			// ENDPOINT validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(speechmatic_endpoint, "required");
			if (!validation_result.success) {
				// valida que el speechmatic_endpoint tenga al menos un caracter
				response.msg = "<%=_T('Uri is required.')%>";
				response.$ctrl = $speechmatic_endpoint;
				this.show_ctrl_err(response);
				return callback(response);
			}
			profile = {
				id : profile_id,
				description : description,
				provider : provider,
				provider_type : provider_type,
				endpoint : speechmatic_endpoint
			};
		}
		else if(this.owner.currentEngine === "microsoft") {
			//Validar controles asociados a microsoft
			const $microsoft_endpoint = $("#" + this.prefix + "popup_microsoft_endpoint");
			let microsoft_endpoint = $microsoft_endpoint.val();
			// ENDPOINT validation -----
			//
			// Required validation
			validation_result = inConcert.Utils.validate(microsoft_endpoint, "required");
			if (!validation_result.success) {
				// valida que el microsoft_endpoint tenga al menos un caracter
				response.msg = "<%=_T('Uri is required.')%>";
				response.$ctrl = $microsoft_endpoint;
				this.show_ctrl_err(response);
				return callback(response);
			}
			profile = {
				id : profile_id,
				description : description,
				provider : provider,
				provider_type : provider_type,
				endpoint : microsoft_endpoint
			};
		}
		const row_profile = self.make_row_profile(item, profile);
		response.success = true;
		response.msg = "";
		response.item = row_profile;
		callback(response);
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.hide_all_errors = function()
	{
		this.m_grid_profiles.hide_popup_msg();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grid_profiles.show_popup_msg(err_info.msg, "Error"); //, ERROR_TIMEOUT_MS);
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.make_ip_port_id = function(server_ip, server_port)
	{
		return FORMAT("{0}_{1}", server_ip, server_port).toUpperCase();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.make_row_profile = function(current_row_item, profile)
	{
		let row_item = current_row_item || {};

		row_item.id                    = profile.id;
		row_item.description           = profile.description;
		row_item.provider              = profile.provider;
		row_item.provider_type         = profile.provider_type;

		//EngineType dependant controls
		if(this.owner.currentEngine === "unimrcp") {
			row_item.version               = profile.version;
			row_item.endpoint              = profile["server-ip"] + ":" + profile["server-port"];
			row_item["server-ip"]          = profile["server-ip"];
			row_item["server-port"]        = profile["server-port"];
			row_item["server-username"]    = profile["server-username"];
			row_item["force-destination"]  = profile["force-destination"];
			row_item["client-port"]        = profile["client-port"];
			row_item["sip-transport"]      = profile["sip-transport"];
			row_item["ua-name"]            = profile["ua-name"];
			row_item["sdp-origin"]         = profile["sdp-origin"];
			row_item["rtp-port-min"]       = profile["rtp-port-min"];
			row_item["rtp-port-max"]       = profile["rtp-port-max"];
			row_item["playout-delay"]      = profile["playout-delay"];
			row_item.ptime                 = profile.ptime;
			row_item["min-playout-delay"]  = profile["min-playout-delay"];
			row_item["max-playout-delay"]  = profile["max-playout-delay"];
			row_item.rtcp                  = profile.rtcp;
			row_item["rtcp-bye"]           = profile["rtcp-bye"];
			row_item["rtcp-tx-interval"]   = profile["rtcp-tx-interval"];
			row_item["rtcp-rx-resolution"] = profile["rtcp-rx-resolution"];
			row_item["client-ext-ip"]      = profile["client-ext-ip"];
			row_item["rtp-ext-ip"]         = profile["rtp-ext-ip"];
			row_item.codecs                = profile.codecs;
		}
		else if(this.owner.currentEngine === "speechmatic") {
			//Seteo de atributos asociados a speechmatic
			row_item.endpoint              = profile.endpoint;
		}
		else if(this.owner.currentEngine === "microsoft") {
			//Seteo de atributos asociados a microsoft
			row_item.endpoint              = profile.endpoint;
		}

		return row_item;
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.set = function(entity)
	{
		const self = this;
		let profiles_info = [];
		if (!entity || !(Array.isArray(entity.profiles) && entity.profiles.length)) {
			this.m_grid_profiles.initialize($.noop, profiles_info);
			return;
		}
		var add_profile = function(profile) {
			const row_profile = self.make_row_profile(null, profile);
			profiles_info.push(row_profile);
		};
		entity.profiles.forEach(profile => add_profile(profile));
		this.m_grid_profiles.initialize($.noop, profiles_info);
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.empty_controls = function()
	{
		this.m_grid_profiles.remove_all_rows();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.set_events = function()
	{
		this.m_grid_profiles.set_events();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.popup_show_ctrl_err = function(_, $ctrl/*, $popup_msg*/)
	{
		if ($ctrl) {
			const error_id = $ctrl.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$ctrl.focus();
		}
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.__init_profiles = function()
	{
		$("body").removeClass("popupDisplay").addClass("popupDisplay");
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.get = function()
	{
		return this.m_grid_profiles.get_items();
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.fill_versions = function()
	{
		const self = this;
		const $combo = $("#" + this.prefix + "popup_version");
		$combo.html("");
		let versions = this.owner.sorted_versions;
		$.each(versions, function(_, version) {
			self.owner.combo_add_value($combo, version.id, version.name, false);
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.fill_providers = function()
	{
		const self = this;
		const $combo = $("#" + this.prefix + "popup_provider");
		$combo.html("");
		let sorted_providers = this.owner.get_providers();
		if(self.owner.currentEngine === "speechmatic") {
			self.owner.combo_add_value($combo, "speechmatic", "Speechmatic", false);
			return;
		}
		$.each(sorted_providers, function(_, provider) {
			if(provider.id !== "speechmatic") {
				self.owner.combo_add_value($combo, provider.id, provider.name, false);
			}
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.fill_provider_types = function()
	{
		const self = this;
		const $combo = $("#" + this.prefix + "popup_provider_type");
		$combo.html("");
		let sorted_provider_types = this.owner.get_provider_types();
		$.each(sorted_provider_types, function(_, provider_type) {
			self.owner.combo_add_value($combo, provider_type.id, provider_type.name, false);
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.fill_sip_transports = function()
	{
		const self = this;
		const $combo = $("#" + this.prefix + "popup_sip_transport");
		$combo.html("");
		let sip_transports = this.owner.sorted_sip_transports;
		$.each(sip_transports, function(_, sip_transport) {
			self.owner.combo_add_value($combo, sip_transport.id, sip_transport.name, false);
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.fill_rtcp_bye = function()
	{
		const self = this;
		const $combo = $("#" + this.prefix + "popup_rtcp_bye");
		$combo.html("");
		let rtcp_byes = this.owner.sorted_rtcp_bye;
		$.each(rtcp_byes, function(_, rtcp_bye) {
			self.owner.combo_add_value($combo, rtcp_bye.id, rtcp_bye.name, false);
		});
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.showEnginesControls = function(engineType)
	{
		const $poppup = $("#" + this.prefix + "popup");
		//Hide all engine dependant controls
		$poppup.find("[enginetype]").hide();
		//Show this engine dependant controls
		if(engineType) {
			$poppup.find("[enginetype='" + engineType + "']").show();
		}
	};

	ProvisioningTtsAsrEngineprofilesMng.prototype.isUsedByVCC = function(profile_id)
	{
		if(this.owner.sorted_used_ttsasrprofiles) {
			for (var profile in this.owner.sorted_used_ttsasrprofiles) {
				if (Object.prototype.hasOwnProperty.call(this.owner.sorted_used_ttsasrprofiles, profile)) {
					if(profile_id === profile) {
						return true;
					}
				}
			}
		}
		return false;
	};
}());
