module(..., package.seeall, inconcert.orbit.application.Decorate)
local json = require "cjson"
require "lua_outboundengineclient"
local utils = require "shared.apps.utils"
local modelVccs = require("apps.provisioning.models.vccs")
local modelTrunks = require("apps.provisioning.models.trunks")
local modelSystem = require("apps.provisioning.models.system_monitoring")
local modelAgentCampaign = require("apps.agent.models.campaigns")
local m_helpers = require "helpers"
local supervisorshared = require "supervisorshared"

local ROWS_LIMIT = 200

local m_oeClient = nil
local function GetOeClient()
	if not m_oeClient then
		m_oeClient = lua_outboundengineclient.LuaOutboundEngineClient(
			inconcert.environment.apps.agent.cluster, "webhandler")
	end
	local oeAddresses = inconcert.environment.apps.agent.outboundengineservice or {}
	for _, address in pairs(oeAddresses) do
		m_oeClient:AddKnownOutboundEngine(address.host, address.port)
	end
	m_oeClient:ConfigureCassandra({
		{
			host = inconcert.environment.apps.provisioning.cassandra.ip,
			port = inconcert.environment.apps.provisioning.cassandra.port
		}
	})
	return m_oeClient
end

local definitionFields = {
    Vcc = {
        id = "Vcc",
        name = "Vcc",
        type = "string"
    },
    CampaignId = {
        id = "CampaignId",
        name = "Campaign Id",
        type = "string",
        analyticsKey = "Id",
    },
    OutboundProcessId = {
        id = "OutboundProcessId",
        name = "Process Id",
        type = "string",
        analyticsKey = "ProcessId",
    },
    ImportId = {
        id = "ImportId",
        name = "Import Id",
        type = "string",
    },
    BatchId = {
        id = "BatchId",
        name = "Batch Id",
        type = "string",
    },
    OutboundProcessState = {
        id = "State",
        name = "State",
        type = "enum",
        options = {
            STARTED = "Started",
            STOPPED = "Stopped",
            STARTING = "Starting",
            STOPPING = "Stopping",
        },
    },
    OutboundProcessStateCause = {
        id = "StateCause",
        name = "State Cause",
        type = "enum",
        options = {
            INITIALIZE = "Initializing process",
            RUNNING = "Running contacts",
            CASSANDRA_DISCONNECTED = "Cassandra disconnected",
            NOT_RUNNING = "Process is not running",
            NOT_READY = "Service is not ready yet",
            WITHOUT_CAMPAIGN = "Process has not campaign",
            WITHOUT_SCHEDULE = "Process has not schedule",
            ACTIVE_RULE_HASNT_CHANNEL = "Rule can't process the channel",
            NO_VALID_CHANNELS = "No valid rule channel found",
            BETWEEN_SCHEDULES = "Date is between day schedules",
            BEFORE_SCHEDULE = "Date is before day schedule",
            AFTER_SCHEDULE = "Date is after day schedules",
            OUT_OF_DAY_RANGE = "Schedule day is out of range",
            DAY_NOT_CONFIGURED = "Schedule day not configured",
            NO_ACTIVE_RULE = "No active rule found",
            NO_AVAILABLE_CHANNELS = "No available channels",
            NO_ALGORITHM_CHANNELS = "No available algorithm channels",
            NO_ACTIVE_AGENTS = "No active agents",
            NO_AVAILABLE_AGENTS = "No available agents",
            NO_AVAILABLE_AGENT_CHANNELS = "No available agent channels",
            NO_ASSIGNED_CONTACTS = "No assigned contacts",
            NO_REMAINING_FREE_CONTACTS = "No remaining free contacts",
            NO_REQUIRED_SKILL_AGENTS = "No agents for required skills",
            NO_PREFERRED_SKILL_AGENTS = "No agents for preferred skills",
            SORTING_CONTACTS = "Sorting contacts",
        },
    },
    OutboundContactState = {
        id = "State",
        name = "State",
        type = "enum",
        options = {
            PENDING = "Pending",
            SCHEDULED = "Scheduled",
            PROCESS = "Process",
            ACTIVE = "Active",
            PAUSED = "Paused",
            FINISHED = "Finished",
            CANCELED = "Canceled",
        },
    },
    OutboundContactStateCause = {
        id = "StateCause",
        name = "State Cause",
        type = "enum",
        options = {
            ACD = "ACD",
            ADDED_TO_BATCH = "Added to Batch",
            AGENT_UNAVAILABLE = "Agent Unavailable",
            API = "API",
            BATCH_DELETED = "Batch Deleted",
            BATCH_ENDS = "Batch Ends",
            BOT = "BOT",
            CANCELED_BY_RETRIES = "Canceled By Retries",
            CANCELED_BY_RULES = "Canceled By Rules",
            CONTACT_INTERACTION_NOT_FOUND = "Interaction Not Found",
            CONTACT_WITHOUT_INTERACTION = "Without Interaction",
            CONTACTED = "Contacted",
            MANUAL = "Manual",
            NEW_BATCH = "New Batch",
            NO_ACTIVE_RULE = "No Active Rule",
            NO_ADDRESS_DAILY_RETRIES = "No Address Daily Retries",
            NO_DAILY_RETRIES = "No Daily Retries",
            NO_ENABLED_ADDRESSES = "No Enabled Addresses",
            NO_RULE_ADDRESSES = "No Rule Addresses",
            PAUSED = "Paused",
            PREVIEW = "Preview",
            RECYCLED = "Recycled",
            REQUEST_AGENT_FAIL = "Request Agent Fail",
            RESERVING_AGENT = "Reserving Agent",
            RESUMED = "Resumed",
            RUNNING = "Running",
            SCHEDULED_BY_RULES = "Scheduled By Rules",
            START_PROCESS = "Start Process",
            STOP_PROCESS = "Stop Process",
            TAKEN = "Taken",
            TIMEOUT_REQUESTING_AGENT = "Timeout Requesting Agent",
            TIMEOUT_RUNNING = "Timeout Running",
            WRAPUP = "Wrapup",
        },
    },
    Number = {
        id = "Number",
        name = "Number",
        type = "int",
    },
    ExcludeWithoutAddress = {
        id = "ExcludeWithoutAddress",
        name = "Exclude Without Address",
        type = "boolean",
    },
    DelayScheduled = {
        id = "DelayScheduled",
        name = "Delay Scheduled",
        type = "boolean",
    },
    BatchPriority = {
        id = "BatchPriority",
        name = "Batch Priority",
        type = "int",
        batchKey = "batchPriority"
    },
    Priority = {
        id = "Priority",
        name = "Contact Priority",
        type = "int",
    },
    Contact = {
        id = "Contact",
        name = "Contact",
        type = "string"
    },
    DialerContactId = {
        id = "ContactId",
        name = "Contact Id",
        type = "string"
    },
    ContactId = {
        id = "ContactId",
        name = "Contact Id",
        type = "string"
    },
    ContactName = {
        id = "Contact Name",
        name = "Name",
        type = "string"
    },
    ExternalId = {
        id = "ExternalId",
        name = "External Id",
        type = "string"
    },
    ScheduleDate = {
        id = "ScheduleDate",
        name = "Schedule Date",
        type = "datetime"
    },
    ManualScheduleDate = {
        id = "ManualScheduleDate",
        name = "Manual Schedule Date",
        type = "datetime"
    },
    Agent = {
        id = "Agent",
        name = "Reserved Agent",
        type = "string"
    },
    LastContactDate = {
        id = "LastContactDate",
        name = "Last Contact Date",
        type = "datetime"
    },
    LastAddress = {
        id = "LastAddress",
        name = "Last Address",
        type = "string"
    },
    InteractionCode = {
        id = "InteractionCode",
        name = "Result",
        type = "enum",
        options = {
            CONNECTED = "Connected",
            NO_ANSWER = "No Answer",
            BLACK_LISTED = "Black Listed",
            CONFIG_ERROR = "Configuration Error",
            ABANDONED = "Abandoned",
            CANCELED = "Canceled",
            UNDEFINED = "Undefined Error",
            LINK_DOWN = "Link Down",
            CANT_ROUTE = "Can't route to endpoint",
            NORMAL_END = "Normal End",
            BUSY = "Busy",
            NOT_SERVED = "Not Served",
            NUMBER_CHANGED = "Number Changed",
            INVALID_USE = "Invalid Use of Network",
            IN_PROGRESS = "In Progress",
            CONGESTION = "Congestion",
            HANDSHAKE_FAILED = "Handshaking Failed",
            REJECTED = "Rejected",
            NETWORK_ERROR = "Network Error",
            PROTOCOL_ERROR = "Protocol Error",
            UNKNOWN = "Unknown Error",
            ANSWER_MACHINE = "Answering Machine",
            FAX_MACHINE = "Fax Machine",
            PREVIEW_CANCELED = "Preview Canceled",
            NO_AVAILABLE_CHANNELS = "No Available Channels",
            NO_AVAILABLE_AGENT_CHANNELS = "No Available Agent Channels",
            NO_REQUIRED_SKILL_AGENTS = "No Skill Agents",
            ["CANNOT VALIDATE NUMBER"] = "Cannot Validate Number",
            ["ALL PACKAGES REJECTED"] = "All Packages Rejected",
            CONTACT_PASSED_AWAY = "Contact Passed Away",
            INDIRECT_CONTACT = "Indirect Contact",
            TIMEOUT_RUNNING = "Timeout Running",
            AGENT_UNAVAILABLE = "Agent Unavailable",
            AGENT_LOGOUT = "Agent Logout",
        }
    },
    LowLevelCode = {
        id = "LowLevelCode",
        name = "Low Level Result",
        type = "enum",
        options = {
            ["0"] = "0 connected",
            ["1"] = "1 unallocated number",
            ["2"] = "2 no route to network",
            ["3"] = "3 no route to destination",
            ["16"] = "16 normal call clearing",
            ["17"] = "17 user busy",
            ["18"] = "18 no user responding",
            ["19"] = "19 no answer from the user",
            ["20"] = "20 subscriber absent",
            ["21"] = "21 call rejected",
            ["22"] = "22 number changed",
            ["23"] = "23 redirection to new destination",
            ["26"] = "26 non-selected user clearing",
            ["27"] = "27 destination out of order",
            ["28"] = "28 address incomplete",
            ["29"] = "29 facility rejected",
            ["31"] = "31 normal unspecified",
            ["-1"] = "-1 internal error",
        }
    },
    LastManagementResult = {
        id = "LastManagementResult",
        name = "Last Management Result",
        type = "string"
    },
    LastAgent = {
        id = "LastAgent",
        name = "Last Agent",
        type = "string"
    },
    Retries = {
        id = "Retries",
        name = "Retries",
        type = "string"
    },
    DailyRetries = {
        id = "DailyRetries",
        name = "Daily Retries",
        type = "string"
    },
    InteractionId = {
        id = "Id",
        name = "Interaction Id",
        type = "string"
    },
    StartDate = {
        id = "StartDate",
        name = "Start Date",
        type = "datetime"
    },
    EndDate = {
        id = "EndDate",
        name = "End Date",
        type = "datetime"
    },
    EngineType = {
        id = "EngineType",
        name = "Engine Type",
        type = "enum",
        options = {
            PROGRESSIVE = "Progressive",
            PREDICTIVE = "Predictive",
            AUTOMATIC = "Automatic",
        }
    },
    Address = {
        id = "Address",
        name = "Address",
        type = "string"
    },
    Channel = {
        id = "Channel",
        name = "Channel",
        type = "string"
    },
    Kind = {
        id = "Kind",
        name = "Kind",
        type = "string"
    },
    ManagementResult = {
        id = "ManagementResult",
        name = "Management Result",
        type = "string"
    },
    IsGoal = {
        id = "IsGoal",
        name = "Is Goal",
        type = "boolean"
    },
    RingingTime = {
        id = "RingingTime",
        name = "Ringing Time",
        type = "time"
    },
    WaitingTime = {
        id = "WaitingTime",
        name = "Waiting Time",
        type = "time"
    },
    AmdTime = {
        id = "AmdTime",
        name = "Amd Time",
        type = "time"
    },
    NextAddress = {
        id = "NextAddress",
        name = "Next Address",
        type = "string"
    },
    ScheduleDate = {
        id = "ScheduleDate",
        name = "Schedule Date",
        type = "datetime"
    },
    Direction = {
        id = "Direction",
        name = "Direction",
        type = "string"
    },
    Campaign = {
        id = "Campaign",
        name = "Campaign",
        type = "string"
    },
    Account = {
        id = "Account",
        name = "Account",
        type = "string"
    },
    AttentionLevel = {
        id = "AttentionLevel",
        name = "Attention Level",
        type = "string"
    },
    InteractionStateDate = {
        id = "InteractionStateDate",
        name = "State Date",
        type = "datetime"
    },
    LastEventDate = {
        id = "LastEventDate",
        name = "Last Event Date",
        type = "datetime"
    },
    InteractionState = {
        id = "InteractionState",
        name = "State",
        type = "string"
    },
    InteractionAgents = {
        id = "InteractionAgents",
        name = "Agents",
        type = "array"
    },
    AttentionTime = {
        id = "AttentionTime",
        name = "Attention Time",
        type = "time"
    },
    Slice = {
        id = "Slice",
        name = "Slice",
        type = "enum",
        options = {
            ["5Minutes"] = "5 Minutes",
            ["15Minutes"] = "15 Minutes",
            ["30Minutes"] = "30 Minutes",
            ["Date"] = "Date",
            ["Hour"] = "Hour",
            ["Month"] = "Month",
        }
    },
    SliceTrunk = {
        id = "Slice",
        name = "Slice",
        type = "enum",
        options = {
            ["5Minutes"] = "5 Minutes",
            ["15Minutes"] = "15 Minutes",
            ["30Minutes"] = "30 Minutes",
            ["Date"] = "Date",
            ["Hour"] = "Hour",
        }
    },
    SliceDate = {
        id = "Date",
        name = "Date",
        type = "datetime"
    },
    HostId = {
        id = "HostId",
        name = "Host Id",
        type = "string"
    },
    GatewayId = {
        id = "GatewayId",
        name = "Gateway Id",
        type = "string"
    },
    TrunkId = {
        id = "TrunkId",
        name = "Trunk Id",
        type = "string"
    },
    ShowPercentage = {
        id = "ShowPercentage",
        name = "Show Percentage",
        type = "boolean"
    },
    LoggedAgents = {
        id = "LoggedAgents",
        name = "Logged Agents",
        type = "int",
        
    },
    ActiveAgents = {
        id = "ActiveAgents",
        name = "Active Agents",
        type = "int"
    },
    FreeAgents = {
        id = "FreeAgents",
        name = "Free Agents",
        type = "int"
    },
    NoAcdAgents = {
        id = "NoAcdAgents",
        name = "No Acd Agents",
        type = "int"
    },
    BreakAgents = {
        id = "BreakAgents",
        name = "Break Agents",
        type = "int"
    },
    AttendingAgents = {
        id = "AttendingAgents",
        name = "Attending Agents",
        type = "int"
    },
    WrapupAgents = {
        id = "WrapupAgents",
        name = "Wrapup Agents",
        type = "int"
    },
    AttendingWrapupAgents = {
        id = "AttendingWrapupAgents",
        name = "Attending or Wrapup Agents",
        type = "int"
    },
    ReservedAgents = {
        id = "ReservedAgents",
        name = "Agents Reserved",
        type = "int"
    },
    OtherAgents = {
        id = "OtherAgents",
        name = "Agents in Another",
        type = "int"
    },
    DirectionFilter = {
        id = "Direction",
        name = "Direction",
        type = "enum",
        options = {
            ANY = "Any",
            INBOUND = "Inbound",
            OUTBOUND = "Outbound",
        }
    },
    TotalInteractions = {
        id = "Total",
        name = "Total Interactions",
        type = "int"
    },
    ProcessFlowInteractions = {
        id = "Bot",
        name = "Bot",
        type = "int"
    },
    WaitingInteractions = {
        id = "Waiting",
        name = "Waiting",
        type = "int"
    },
    QueuedInteractions = {
        id = "Queued",
        name = "Queued",
        type = "int"
    },
    AssignedInteractions = {
        id = "TakenWrapup",
        name = "Assigned",
        type = "int"
    },
    TakenInteractions = {
        id = "Taken",
        name = "Taken",
        type = "int"
    },
    WrapupInteractions = {
        id = "Wrapup",
        name = "Wrapup",
        type = "int"
    },
    RunningContacts = {
        id = "Running",
        name = "Running",
        type = "int"
    },
    TrunksStatus = {
        id = "TrunksStatus",
        name = "State",
        type = "string"
    },
    AssignedPackagesCapability = {
        id = "AssignedPackagesCapability",
        name = "Assigned Capacity",
        type = "int"
    },
    EffectiveCapability = {
        id = "EffectiveCapability",
        name = "Effective Capacity",
        type = "int"
    },
    BusyCapabilityPercentage = {
        id = "BusyCapabilityPercentage",
        name = "Busy Capacity (%)",
        type = "percentage"
    },
    BusyPackagesCapabilityPercentage = {
        id = "BusyPackagesCapabilityPercentage",
        name = "Busy Capacity Packages (%)",
        type = "percentage"
    },
    BusyTrunksCapabilityPercentage = {
        id = "BusyTrunksCapabilityPercentage",
        name = "Busy Capacity Trunks (%)",
        type = "percentage"
    },
    FailedTrunksCapabilityPercentage = {
        id = "FailedTrunksCapabilityPercentage",
        name = "Down Capacity Trunks (%)",
        type = "percentage"
    },
    HasOverbusyTrunks = {
        id = "HasOverbusyTrunks",
        name = "Overbusy Trunks",
        type = "boolean"
    },
    OverbusyTrunks = {
        id = "OverbusyTrunks",
        name = "Overbusy Trunks Number",
        type = "int"
    },
    OverbusyTrunksPercentage = {
        id = "OverbusyTrunksPercentage",
        name = "Overbusy Trunks (%)",
        type = "percentage"
    },
    RejectedDidsPercentage = {
        id = "RejectedDidsPercentage",
        name = "Rejected Dids (%)",
        type = "percentage"
    },
    DesiredAbandonRate = {
        id = "DesiredAbandonRate",
        name = "Desired Abandon Rate (%)",
        type = "percentage"
    },
    AlghoritmAbandonRate = {
        id = "AlghoritmAbandonRate",
        name = "Alghoritm Abandon Rate (%)",
        type = "int"
    },
    AlghoritmAbandonRateDeviation = {
        id = "AlghoritmAbandonRateDeviation",
        name = "Abandon Rate Deviation (%)",
        type = "int"
    },
    AlghoritmContactRate = {
        id = "AlghoritmContactRate",
        name = "Alghoritm Contact Rate (%)",
        type = "int"
    },
    AlghoritmCorrectionRatio = {
        id = "AlghoritmCorrectionRatio",
        name = "Calls per Available Agent (Predictive)",
        type = "int"
    },
    CorrectionManualRate = {
        id = "CorrectionManualRate",
        name = "Calls per Available Agent (Dialer Throttle)",
        type = "int"
    },
    LoadedContacts = {
        id = "LoadedContacts",
        name = "Loaded",
        type = "int"
    },
    EffectiveContacts = {
        id = "EffectiveContacts",
        name = "Finished",
        type = "int"
    },
    EffectiveContactsPercentage = {
        id = "EffectiveContactsPercentage",
        name = "Finished (%)",
        type = "percentage"
    },
    RemainingContacts = {
        id = "RemainingContacts",
        name = "Remaining",
        type = "int"
    },
    RemainingContactsPercentage = {
        id = "RemainingContactsPercentage",
        name = "Remaining (%)",
        type = "percentage"
    },
    ScheduledNextHour = {
        id = "ScheduledNextHour",
        name = "Remaining Next Hour",
        type = "int"
    },
    UnhandledContacts = {
        id = "UnhandledContacts",
        name = "Virgin",
        type = "int"
    },
    UnhandledContactsPercentage = {
        id = "UnhandledContactsPercentage",
        name = "Virgin (%)",
        type = "percentage"
    },
    ScheduledContacts = {
        id = "ScheduledContacts",
        name = "Scheduled",
        type = "int"
    },
    ScheduledContactsPercentage = {
        id = "ScheduledContactsPercentage",
        name = "Scheduled (%)",
        type = "percentage"
    },
    DelayedScheduled = {
        id = "DelayedScheduled",
        name = "Delayed Scheduled",
        type = "int"
    },
    CanceledContacts = {
        id = "CanceledContacts",
        name = "Canceled",
        type = "int"
    },
    CanceledContactsPercentage = {
        id = "CanceledContactsPercentage",
        name = "Canceled (%)",
        type = "percentage"
    },
    PausedContacts = {
        id = "PausedContacts",
        name = "Paused",
        type = "int"
    },
    Tries = {
        id = "Tries",
        name = "Calls",
        type = "int"
    },
    Connected = {
        id = "Connected",
        name = "Connected",
        type = "int"
    },
    NotConnected = {
        id = "NotConnected",
        name = "Not Connected",
        type = "int"
    },
    Abandoned = {
        id = "Abandoned",
        name = "Abandoned",
        type = "int"
    },
    AnsweringMachine = {
        id = "AnsweringMachine",
        name = "Answering Machine",
        type = "int"
    },
    TriesUnique = {
        id = "TriesUnique",
        name = "Called Contacts",
        type = "int"
    },
    RetriesPerContact = {
        id = "RetriesPerContact",
        name = "Retries Per Contact",
        type = "int"
    },
    ContactRate = {
        id = "ContactRate",
        name = "Contact Rate (%)",
        type = "percentage"
    },
    AbandonRate = {
        id = "AbandonRate",
        name = "Abandon Rate (%)",
        type = "percentage"
    },
    AbandonedByTimeoutRate = {
        id = "AbandonedByTimeoutRate",
        name = "Abandoned by Timeout (%)",
        type = "percentage"
    },
    AbandonedByAgentRate = {
        id = "AbandonedByAgentRate",
        name = "Abandoned by Agent (%)",
        type = "percentage"
    },
    BadServiceLevelRate = {
        id = "BadServiceLevelRate",
        name = "Bad Service Level (%)",
        type = "percentage"
    },
    Goal = {
        id = "Goal",
        name = "Goals",
        type = "int"
    },
    GoalsPerHour = {
        id = "GoalsPerHour",
        name = "Goals Per Hour",
        type = "int"
    },
    RetriesPerGoal = {
        id = "RetriesPerGoal",
        name = "Retries Per Goal",
        type = "int"
    },
    ConfigError = {
        id = "ConfigError",
        name = "Config Error",
        type = "int"
    },
    LinkDown = {
        id = "LinkDown",
        name = "Link Down",
        type = "int"
    },
}

local translateKeys = {
    "name",
    "description"
}

function GetField(fieldId, extraData)
    local fieldTemplate = definitionFields[fieldId]
    if not extraData then return fieldTemplate end

    local field = extraData
    for key, value in pairs(fieldTemplate) do
        if field[key] == nil then
            field[key] = value
        end
    end
    return field
end

function AddEnumToColumns(columns, field)
    for id, name in pairs(field.options) do
        table.insert(columns, {
            id = id,
            name = name,
            type = "int"
        })
    end
end

local function TranslateDefinition(definition, _T)
    for _, key in pairs(translateKeys) do
        if definition[key] then
            definition[key] = _T(definition[key])
        end
    end

    for _, subDefinition in pairs(definition) do
        if type(subDefinition) == "table" then
            TranslateDefinition(subDefinition, _T)
        end
    end
end

local function GetTranslatedDefinition(originalDefinition, _T)
    local definition = json.decode(json.encode(originalDefinition))
    TranslateDefinition(definition, _T)
    return definition
end

function GetCategoryDefinition(category, _T)
    return GetTranslatedDefinition(category, _T)
end

function GetViewDefinition(view, _T)
    return GetTranslatedDefinition(view, _T)
end

local function EmptyFilter(filter)
    return filter == nil or filter == ""
end

function FilterVccs(filters)
    local vccs = modelVccs.GetallVccs()
    local ids = {}
    local filterByVcc = filters.Vcc
    for vccId, _ in pairs(vccs) do
        if EmptyFilter(filterByVcc) or vccId == filterByVcc then
            table.insert(ids, vccId)
        end
    end
    table.sort(ids)
    return ids
end

function FilterCampaigns(filters)
    local vccs = FilterVccs(filters)
    local campaignIds = {}
    local filterByCampaign = filters.CampaignId
    local keyspace = utils.GetDatasource():CassandraConnection("Agent")
    for _, vcc in pairs(vccs) do
        local vccCampaigns = modelSystem.GetVccCampaigns(vcc, keyspace)
        for campaignWithoutVcc, _ in pairs(vccCampaigns) do
            local campaignId = utils.AppendVcc(campaignWithoutVcc, vcc)
            if EmptyFilter(filterByCampaign) or campaignId == filterByCampaign then
                table.insert(campaignIds, campaignId)
            end
        end
    end
	keyspace:Close()
    return campaignIds
end

function FilterAccounts(filters)
    local campaignIds = FilterCampaigns(filters)
    local accountIds = {}
    local filterByAccount = filters.AccountId
    local keyspace = utils.GetDatasource():CassandraConnection("Agent")
    for _, campaignId in pairs(campaignIds) do
        local campaignAccounts = modelAgentCampaign.GetCampaignAccounts(campaignId, keyspace)
        for accountId, _ in pairs(campaignAccounts) do
            if EmptyFilter(filterByAccount) or accountId == filterByAccount then
                table.insert(accountIds, accountId)
            end
        end
    end
	keyspace:Close()
    return campaignIds
end

local function GetVccProcesses(vcc)
    local vccProcesses = GetOeClient():GetVccCampaignProcesses(vcc)
    local ids = {}
    for _, processId in pairs(vccProcesses) do
        table.insert(ids, processId)
    end
    table.sort(ids)
    return ids
end

function FilterOutboundProcesses(filters)
    local vccs = FilterVccs(filters)
    local processIds = {}
    local filterByProcess = filters.OutboundProcessId
    for _, vcc in pairs(vccs) do
        local vccProcesses = GetVccProcesses(vcc)
        for _, processId in pairs(vccProcesses) do
            if EmptyFilter(filterByProcess) or processId == filterByProcess then
                table.insert(processIds, processId)
            end
        end
    end
    return processIds
end

function GetCamapaignProcessId(campaignId)
    local _, vcc = utils.SeparateVCC(campaignId)
    return GetOeClient():GetCampaignProcessId(vcc, campaignId)
end

function GetCampaignsProcessIds(campaignIds)
    local processIds = {}
    for _, campaignId in pairs(campaignIds) do
        local processId = GetCamapaignProcessId(campaignId)
        if processId then
            table.insert(processIds, processId)
        end
    end
    return processIds
end

function FilterOutboundProcessBatches(filters)
    local processIds = FilterOutboundProcesses(filters)
    local keyspace = utils.GetDatasource():CassandraConnection("OutboundEngine")
    local batchIds = {}
    local filterByBatch = filters.BatchId
    for _, processId in pairs(processIds) do
        local processBatches = modelSystem.GetOutboundProcessBatches(processId, keyspace)
        for batchId, _ in pairs(processBatches) do
            if EmptyFilter(filterByBatch) or batchId == filterByBatch then
                table.insert(batchIds, batchId)
            end
        end
    end
	keyspace:Close()
    return batchIds
end

local function MakeAnalyticsCampaignsListViewResult(rawRows, definition, _T)
    local columns = definition.columns
    local translate = {}

    local rows = {}
    for _, rawRow in pairs(rawRows) do
        local row = MakeAnalyticsCampaignsListViewRow(rawRow, columns, translate, _T)
    end
end

local function GetColumnValue(row, column)
    return row[column.id]
end

local function GetAnalyticsColumnId(column, slice)
    if column.analyticsKey then return column.analyticsKey end

    if column.bySlice and slice then
        return string.format("%sBy%s", column.id, slice)
    end

    return column.id
end

local function GetAnalyticsColumnValue(row, column, slice)
    if column.id == "Vcc" then
        local _, vcc = utils.SeparateVCC(row.Id)
        return vcc
    end
    if column.analyticsKey then return row[column.analyticsKey] end

    if column.bySlice and slice then
        local idBySlice = string.format("%sBy%s", column.id, slice)
        return GetColumnValue(row, { id = idBySlice })
    end

    return GetColumnValue(row, column)
end

local function GetBatchesColumnValue(row, column)
    if column.batchKey then return row[column.batchKey] end

    return GetColumnValue(row, column)
end

local function GetRowId(row, definition)
    local key = definition.rowId or "Id"
    return row[key]
end

local function TranslateColumnValue(value, translated, _T)
    if not value then return "" end

    if not translated[value] then
        translated[value] = _T(value)
    end
    return translated[value]
end

local makeResultColumnByType = {
    enum = function(column, value, translated, _T)
        local enumValue = column.options[value] or value
        return TranslateColumnValue(enumValue, translated, _T)
    end,
    int = function(column, value, translated, _T)
        return value or 0
    end,
    percentage = function(column, value, translated, _T)
        return string.format("%.2f", value or 0) .. "%"
    end,
    datetime = function(column, value, translated, _T)
        if (value or "") == "" then return "" end

        return m_helpers.utils.SplitString(value, ".")[1]
    end,
    time = function(column, value, translated, _T)
        return string.format("%.2f", value or 0)
    end,
}

local function MakeResultColumnValue(column, value, translated, _T)
    local method = makeResultColumnByType[column.type]
    if not method then return value end

    return method(column, value, translated, _T)
end

local function SortRowsByColumns(rows, columns, desc)
    table.sort(rows, function (rowA, rowB)
        for _, column in pairs(columns) do
            local valueA = rowA[column.id]
            local valueB = rowB[column.id]
            if valueA ~= valueB then
                if desc or column.desc then
                    return valueA > valueB
                else
                    return valueA < valueB
                end
            end
        end
		return false
    end)
end

local function MakeResultRows(params)
    local rawRows = params.rows
    local definition = params.definition
    local columns = params.columns
    local _T = params._T
    local getColumnValue = params.getColumnValue or GetColumnValue
    local getRowId = params.getRowId or GetRowId
    local sortByColumns = params.sortByColumns
    local translated = {}

    local rows = {}
    for _, rawRow in pairs(rawRows) do
        local row = {}
        for _, column in pairs(columns) do
            local value = getColumnValue(rawRow, column)
            row[column.id] = MakeResultColumnValue(column, value, translated, _T)
        end
        row._Id = getRowId(rawRow, definition)
        table.insert(rows, row)
    end
    if sortByColumns then
        SortRowsByColumns(rows, definition.sortByColumns or columns)
    end

    return { result = "OK", data = rows }
end

local function MakeAnalyticsCampaignsListIndicators(columns, slice)
    local indicators = {}
    for _, column in pairs(columns) do
        table.insert(indicators, GetAnalyticsColumnId(column, slice))
    end
    return indicators
end

function RunAnalyticsCampaignsListView(definition, request, _T, processes, columns, getRowId)
    local filters = request.filters or {}
    local slice = (not EmptyFilter(filters.Slice) and filters.Slice) or "Date"
    columns = columns or definition.columns
    processes = processes or FilterOutboundProcesses(filters)
    if #processes == 0 then return { result = "OK", data = {} } end

    local analyticsRequest = {
        indicators = MakeAnalyticsCampaignsListIndicators(columns, slice),
        includeInactive = true,
        processes = processes
    }
    local result = GetOeClient():AnalyticsCampaignsList(json.encode(analyticsRequest))
    if result.result ~= "OK" then return result end

    return MakeResultRows({
        rows = result.campaigns,
        definition = definition,
        columns = columns,
        _T = _T,
        getColumnValue = function(row, column)
            return GetAnalyticsColumnValue(row, column, slice)
        end,
        getRowId = getRowId or function(row)
            if not row.ProcessId then return nil end

            return string.format("%s|%s", row.ProcessId, slice)
        end,
    })
end

function AppendDialerCampaignFields(result, definition, _T)
    local campaignColumns = DefinitionFieldsWith(definition.columns, "dialerCampaignColumn")
    local wantedCampaignIds = UniqueFields(result.data, "CampaignId")
    local wantedProcesses = GetCampaignsProcessIds(wantedCampaignIds)

    local campaignsResult = RunAnalyticsCampaignsListView(definition, {}, _T, processes, campaignColumns,
        function(row) return row.Id end)
    return AppendResults(result, campaignsResult, "CampaignId")
end

function GetAnalyticsCampaignsDetailsView(itemComposedId, itemType)
    local itemParts = m_helpers.utils.SplitString(itemComposedId, "|")
    local itemId = itemParts[1]
    local slice = string.format("By%s", itemParts[2] or "Date")
    local key = itemType == "process" and "processId" or "campaignId"
    local _, vcc = utils.SeparateVCC(itemId)
    local analyticsRequest = {
        vcc = vcc,
        [key] = itemId,
        detailsSlice = slice,
        historicalSlice = slice,
    }
    local result = GetOeClient():AnalyticsCampaignDetails(json.encode(analyticsRequest))
    if result.result ~= "OK" then return result end

    return { result = "OK", data = result.indicators }
end

local m_globaltimer
local function Now()
    if not m_globaltimer then
		m_globaltimer = lua_globaltimer.LuaGlobalTimer(inconcert.environment.apps.agent.cluster)
		local timeServers = inconcert.environment.apps.agent.timeServers
		for _, server in pairs(timeServers or {}) do
			m_globaltimer:AddKnownServer(server.IP, server.port)
		end
	end
	return LuaOrm.Date(m_globaltimer:GetTime())
end

local searchSpecialConditions = {
    ExcludeWithoutAddress = function(value)
        if not value then return nil end

        return { type = "match", name = "Addresses.Status", value = "Enabled" }
    end,
    DelayScheduled = function(value)
        if not value then return nil end

        return {
            type = "and",
            conditions = {
                { type = "equal", name = "State", value = "SCHEDULED" },
                { type = "less", name = "ScheduleDate", value = Now():toString("%Y-%m-%d %H:%M:%S.000") }
            }
        }
    end,
    Contact = function(value)
        if not value then return nil end

        local containsValue = string.format("*%s*", value)
        return {
            type = "or",
            conditions = {
                { type = "contains", name = "Id", value = containsValue },
                { type = "contains", name = "Name", value = containsValue },
                { type = "contains", name = "ExternalId", value = containsValue },
            }
        }
    end,
    InteractionContact = function(value)
        if not value then return nil end

        local containsValue = string.format("*%s*", value)
        return {
            type = "or",
            conditions = {
                { type = "contains", name = "contact", value = containsValue },
                { type = "contains", name = "contact_name", value = containsValue },
            }
        }
    end,
    InteractionContent = function(value)
        if not value then return nil end

        local containsValue = string.format("*%s*", value)
        return { type = "contains", name = "content", value = containsValue }
    end,
}
local function MakeSearchFilterCondition(field, value)
    local specialCondition = searchSpecialConditions[field.id]
    if specialCondition then return specialCondition(value) end

    return { type = "equal", name = field.id, value = value }
end

local searchSpecialFilters = {
    Vcc = "vcc",
    BatchId = "batchId"
}
local function AddSearchFilter(field, value, request)
    if value == nil or (type(value) == "string" and value == "") then return end

    local specialId = searchSpecialFilters[field.id]
    if specialId then
        request[specialId] = value
        return
    end

    local condition = MakeSearchFilterCondition(field, value)
    if not condition then return end

    request.conditions = request.conditions or { type = "and", conditions = {} }
    table.insert(request.conditions.conditions, condition)
end

local function AddSearchFiltersRequest(definitionFilters, filters, request)
    for _, field in pairs(definitionFilters or {}) do
        AddSearchFilter(field, filters[field.id], request)
    end
end

local function GetSummaryFields(definition)
    local summaryFields = {}
    for _, column in pairs(definition.columns) do
        if column.id ~= "Number" then
            table.insert(summaryFields, column.id)
        end
    end
    return summaryFields
end

local function MakeSummaryRequest(definition, filters)
    local request = {}
    AddSearchFiltersRequest(definition.filters, filters, request)
    AddSearchFiltersRequest(definition.fixedFilters, filters, request)
    request.summaryFields = GetSummaryFields(definition)
    return request
end

local function GetSearchFields(columns)
    local fields = {}
    for _, column in pairs(columns) do
        table.insert(fields, column.id)
    end
    return fields
end

local function MakeSearchRequest(definition, clientRequest)
    local request = {
        size = clientRequest.size or ROWS_LIMIT,
        sortTerms = {
            {
                name = clientRequest.order or (definition.order and definition.order.default) or "Id",
                order = clientRequest.orderDirection or "asc",
            },
        },
        fields = GetSearchFields(definition.columns)
    }

    local filters = clientRequest.filters or {}
    AddSearchFiltersRequest(definition.filters, filters, request)
    AddSearchFiltersRequest(definition.fixedFilters, filters, request)
    return request
end

local function SummaryToRows(summary, columns, scope, position, rows)
    local column = columns[position]
    for fieldId, value in pairs(summary) do
        scope[column.id] = fieldId
        if type(value) == "table" then
            SummaryToRows(value.children, columns, scope, position + 1, rows)
        else
            local row = { Number = value }
            for key, value in pairs(scope) do
                row[key] = value
            end
            table.insert(rows, row)
        end
    end
end

local function MakeSummarySearchRows(summary, definition, _T)
    local rows = {}
    SummaryToRows(summary, definition.columns, {}, 1, rows)
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        sortByColumns = true
    })
end

function RunSummaryOutboundContacts(definition, request, _T)
    local summaryRequest = MakeSummaryRequest(definition, request.filters or {})
    local result = GetOeClient():SummaryContacts(json.encode(summaryRequest))
    if result.result ~= "OK" then return result end

    return MakeSummarySearchRows(result.summary, definition, _T)
end

function RunSearchOutboundContacts(definition, request, _T)
    local searchRequest = MakeSearchRequest(definition, request)
    local result = GetOeClient():SearchContacts(json.encode(searchRequest))
    if result.result ~= "OK" then return result end

    return MakeResultRows({
        rows = result.contacts,
        definition = definition,
        columns = definition.columns,
        _T = _T,
    })
end

function RunSearchOutboundContactResults(definition, request, _T)
    local searchRequest = MakeSearchRequest(definition, request)
    local result = GetOeClient():GetContactResults(json.encode(searchRequest))
    if result.result ~= "OK" then return result end

    return MakeResultRows({
        rows = result.contactResults,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        getRowId = function(row)
            return string.format("%s@%s", row.Id, row.Vcc)
        end,
    })
end

function RunSearchInteractions(definition, request, _T)
    local searchRequest = MakeSearchRequest(definition, request)
    local result = GetOeClient():SearchInteractions(json.encode(searchRequest))
    if result.result ~= "OK" then return result end

    return MakeResultRows({
        rows = result.interactions,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        getRowId = function(row)
            return string.format("%s|%s", row.id, request.filters.Vcc)
        end,
    })
end

function GetContactDetails(contactId)
    local _, vcc = utils.SeparateVCC(contactId)
    local result = GetOeClient():GetContact(json.encode({ contactId = contactId, vcc = vcc }))
    if result.result ~= "OK" then return result end

    return { result = "OK", data = result.contact }
end

function GetContactResultDetails(resultIdWithVcc)
    local resultId, vcc = utils.SeparateVCC(resultIdWithVcc)
    local result = GetOeClient():GetContactResult(json.encode({
        resultId = resultId,
        vcc = vcc,
        withInteraction = true,
    }))
    if result.result ~= "OK" then return result end

    return { result = "OK", data = result.contactResult }
end

function GetInteractionDetails(interactionIdWithVcc)
    local interactionIdParts = m_helpers.utils.SplitString(interactionIdWithVcc, "|")
    local result = GetOeClient():GetInteraction(json.encode({
        interactionId = interactionIdParts[1],
        vcc = interactionIdParts[2],
    }))
    if result.result ~= "OK" then return result end

    return { result = "OK", data = result.interaction }
end

function DefinitionFieldsWith(fields, withField)
    local result = {}
    for _, field in pairs(fields) do
        if field[withField] then table.insert(result, field) end
    end
    return result
end

function DefinitionFieldsWithout(fields, withoutField)
    local result = {}
    for _, field in pairs(fields) do
        if not field[withoutField] then table.insert(result, field) end
    end
    return result
end

local definitionWithoutFields = {
    columns = true,
    filters = true
}

function DefinitionWithout(definition, withoutField)
    local newDefinition = {}
    for key, value in pairs(definition) do
        if definitionWithoutFields[key] then
            newDefinition[key] = DefinitionFieldsWithout(value, withoutField)
        else
            newDefinition[key] = value
        end
    end
    return newDefinition
end

function UniqueFields(rows, fieldId)
    local fields = {}
    local fieldsDict = {}
    for _, row in pairs(rows) do
        local field = row[fieldId]
        if field and not fieldsDict[field] then
            fieldsDict[field] = true
            table.insert(fields, field)
        end
    end
    return fields
end

local function RowsToDict(rows)
    local dict = {}
    for _, row in pairs(rows) do
        dict[row._Id] = row
    end
    return dict
end

function AppendResults(result, toAppend, appendKey)
    local toAppendDict = RowsToDict(toAppend.data)
    for _, row in pairs(result.data) do
        local rowToAppend = toAppendDict[row[appendKey]] or {}
        for key, value in pairs(rowToAppend) do
            if key ~= "_Id" then row[key] = value end
        end
    end
    return result
end

function GetBatches(batchIds)
    local keyspace = utils.GetDatasource():CassandraConnection("OutboundEngine")
    local batches = {}
    for _, batchId in pairs(batchIds) do
        local processId = m_helpers.utils.SplitString(batchId, ":")[2]
        local batch = modelSystem.GetOutboundProcessBatch(processId, batchId, keyspace)
        if batch and batch.configuration then
            batch.configuration.Id = batchId
            table.insert(batches, batch.configuration)
        end
    end
	keyspace:Close()
    return batches
end

function AppendBatchFields(result, definition, _T)
    local batchColumns = DefinitionFieldsWith(definition.columns, "batchColumn")
    local wantedBatchesIds = UniqueFields(result.data, "BatchId")
    local batchesResult = MakeResultRows({
        rows = GetBatches(wantedBatchesIds),
        definition = definition,
        columns = batchColumns,
        _T = _T,
        getColumnValue = GetBatchesColumnValue,
    })
    return AppendResults(result, batchesResult, "BatchId")
end

local function GetRedisIndicatorsRowDate(key)
    key = key:gsub("::", "|")
    local keyParts = m_helpers.utils.SplitString(key, "|")
    return keyParts[#keyParts]
end

local function AccumulateRedisIndicatorsKey(row)
    local accumulatedRow = {}
    for key, value in pairs(row) do
        local keyParts = m_helpers.utils.SplitString(key, ":")
        local accumulateKey = keyParts[#keyParts]
        accumulatedRow[accumulateKey] = (accumulatedRow[accumulateKey] or 0) + tonumber(value)
    end
    return accumulatedRow
end

local function ShowRedisIndicatorsPercentage(row)
    local total = 0
    for _, value in pairs(row) do
        total = total + tonumber(value)
    end
    if total == 0 then return 0 end

    local percentages = {}
    for key, value in pairs(row) do
        percentages[key] = 100 * tonumber(value) / total
    end
    return percentages
end

local function MakeRedisIndicatorsRow(result, accumulateKey, showPercentage)
    local row = result.values
    if accumulateKey then
        row = AccumulateRedisIndicatorsKey(row)
    end
    if showPercentage then
        row = ShowRedisIndicatorsPercentage(row)
    end
    row.Date = GetRedisIndicatorsRowDate(result.key)
    return row
end

local function AddRedisIndicatorsRows(scope, section, slice, accumulateKey, rows, showPercentage, addExtraInfo)
    local results = modelSystem.GetRedisIndicators(scope, section, slice)
    for _, result in pairs(results) do
        local row = MakeRedisIndicatorsRow(result, accumulateKey, showPercentage)
        if addExtraInfo then
            addExtraInfo(result, row)
        end
        table.insert(rows, row)
    end
end

local function AddRedisDialerProcessRow(processId, section, slice, accumulateKey, rows, showPercentage)
    local _, vcc = utils.SeparateVCC(processId)
    local scope = string.format("OutboundProcessStatistics::%s", processId)
    return AddRedisIndicatorsRows(scope, section, slice, accumulateKey, rows, showPercentage, function(_result, row)
        row.OutboundProcessId = processId
        row.Vcc = vcc
    end)
end

function RunRedisDialerProcessView(definition, request, _T, section)
    local filters = request.filters or {}
    local slice = (not EmptyFilter(filters.Slice) and filters.Slice) or "Date"
    local processIds = FilterOutboundProcesses(filters)
    local rows = {}
    for _, processId in pairs(processIds) do
        AddRedisDialerProcessRow(processId, section, slice, true, rows, filters.ShowPercentage)
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        sortByColumns = true
    })
end

local function AddRedisDialerBatchRow(batchId, section, slice, accumulateKey, rows, showPercentage)
    local processId = m_helpers.utils.SplitString(batchId, ":")[2]
    local _, vcc = utils.SeparateVCC(processId)
    local scope = string.format("OutboundProcessBatchStatistics::%s", batchId)
    return AddRedisIndicatorsRows(scope, section, slice, accumulateKey, rows, showPercentage, function(_result, row)
        local _, vcc = utils.SeparateVCC(processId)
        row.BatchId = batchId
        row.OutboundProcessId = processId
        row.Vcc = vcc
    end)
end

function RunRedisDialerBatchView(definition, request, _T, section)
    local filters = request.filters or {}
    local slice = (not EmptyFilter(filters.Slice) and filters.Slice) or "Date"
    local batchIds = FilterOutboundProcessBatches(filters)
    local rows = {}
    for _, batchId in pairs(batchIds) do
        AddRedisDialerBatchRow(batchId, section, slice, true, rows, filters.ShowPercentage)
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        sortByColumns = true
    })
end

local function AddRedisDialerTrunkRow(trunk, section, slice, rows, columns, showPercentage)
    local scope = string.format("TelephonyTrunkStatistics::%s", trunk.id)
    return AddRedisIndicatorsRows(scope, section, slice, false, rows, showPercentage, function(_result, row)
        for _, column in pairs(columns) do
            if trunk[column.id] then
                row[column.id] = trunk[column.id]
            end
        end
    end)
end

local function FilterTrunk(trunk, filters)
    for key, filter in pairs(filters) do
        if not EmptyFilter(filter) and trunk[key] ~= filter then
            return false
        end
    end
    return true
end

function FilterTrunks(filters)
    local success, trunks = modelTrunks.ListAll()
    if not success then return {} end

    local filteredTrunks = {}
    for _, trunk in pairs(trunks) do
        if FilterTrunk(trunk, filters) then
            table.insert(filteredTrunks, trunk)
        end
    end
    return filteredTrunks
end

function RunRedisDialerTrunkView(definition, request, _T, section)
    local filters = request.filters or {}
    local showPercentage = filters.ShowPercentage
    local slice = (not EmptyFilter(filters.Slice) and filters.Slice) or "Date"
    filters.ShowPercentage = nil
    filters.Slice = nil

    local trunks = FilterTrunks(filters)
    local rows = {}
    for _, trunk in pairs(trunks) do
        AddRedisDialerTrunkRow(trunk, section, slice, rows, definition.columns, showPercentage)
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
        sortByColumns = true
    })
end

local function GetSupervisorCampaignView(params)
    local campaignId = params.campaignId
    local campaignIdWithOutVCC, vcc = utils.SeparateVCC(campaignId)
    local accountId = params.accountId or ""
    local alId = params.alId or ""
    local workgroups = params.workgroups or {}
    local realtimeWindow = params.realtimeWindow or "Last15Minutes"

    -- agents list
    local rowsPerPage = params.rowsPerPage or 20
    local showOfflineAgents = params.showOfflineAgents or false
    local showKickouts = params.showKickouts ~= false
    local interactionTimes = params.interactionTimes or false
    local agentNameFilter = params.agentNameFilter or nil
    local agentOrder = params.agentOrder or "ID"
    local agentPage = params.agentPage or 0

    local result = supervisorshared.DataFilter():FilterView({
		VCC = vcc,
		rowsPerPage = rowsPerPage,
		showOfflineAgents = showOfflineAgents,
		showKickouts = showKickouts,
		interactionTimes = interactionTimes,
		campaignFilters = {
			[campaignIdWithOutVCC] = {
				account				= accountId,
				attention_level		= alId,
				agent				= agentNameFilter,
				order				= agentOrder,
				page				= agentPage,
				realtimeWindow		= realtimeWindow,
				workgroup			= workgroups
			}
		}
	})
    return result and result[campaignIdWithOutVCC]
end

local function GetSupervisorAnalyticsCampaignIndicators(params)
    local campaignId = params.campaignId
    local campaignIdWithOutVCC, vcc = utils.SeparateVCC(campaignId)

    local requestStr = json.encode({
        vcc = vcc,
        campaignId = campaignIdWithOutVCC
    })
    local resultStr = supervisorshared.DataFilter():GetAnalyticsCampaignRealtimeAgentIndicators(requestStr)
    local result = json.decode(resultStr)
    return result and result.indicators
end

local function GetSupervisorAnalyticsVccIndicators(params)
    local vcc = params.vcc
    local channel = params.channel or ""

    local requestStr = json.encode({
        vcc = vcc,
        channel = channel
    })
    local resultStr = supervisorshared.DataFilter():GetAnalyticsVccRealtimeAgentIndicators(requestStr)
    local result = json.decode(resultStr)
    return result and result.indicators
end

local function MakeSupervisorAgentsSummaryRow(vcc, indicators)
    local states = indicators.states
    return {
        Vcc = vcc,
        LoggedAgents = indicators.logged,
        ActiveAgents = states.ACTIVE,
        FreeAgents = states.IDLE,
        NoAcdAgents = states.NOACD,
        BreakAgents = states.BREAK,
        AttendingAgents = states.TAKEN,
        WrapupAgents = states.WRAPUP,
        AttendingWrapupAgents = (states.TAKEN or 0) + (states.WRAPUP or 0),
        ReservedAgents = states.RESERVED,
        OtherAgents = (states["BUSY CROSS CAMPAIGN"] or 0) + (states["ATTENDING CROSS CAMPAIGN"] or 0)
             + (states["RESERVED CROSS CAMPAIGN"] or 0),
    }
end

local function GetSupervisorAgentsSummaryByCampaignRow(campaignId)
    local indicators = GetSupervisorAnalyticsCampaignIndicators({ campaignId = campaignId })
    if not indicators then return end
 
    local _, vcc = utils.SeparateVCC(campaignId)
    local indicators = MakeSupervisorAgentsSummaryRow(vcc, indicators)
    indicators.CampaignId = campaignId
    return indicators
end

function RunSupervisorAgentsSummaryByCampaign(definition, request, _T)
    local filters = request.filters or {}
    local campaignIds = FilterCampaigns(filters)
    local rows = {}
    for _, campaignId in pairs(campaignIds) do
        local row = GetSupervisorAgentsSummaryByCampaignRow(campaignId)
        if row then
            table.insert(rows, row)
        end
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
    })
end

local function GetSupervisorAgentsSummaryByVccRow(vcc, filters)
    local indicators = GetSupervisorAnalyticsVccIndicators({
        vcc = vcc,
        channel = (not EmptyFilter(filters.Channel) and filters.Channel) or nil
    })
    if not indicators then return end
 
    return MakeSupervisorAgentsSummaryRow(vcc, indicators)
end

function RunSupervisorAgentsSummaryByVcc(definition, request, _T)
    local filters = request.filters or {}
    local vccs = FilterVccs(filters)
    local rows = {}
    for _, vcc in pairs(vccs) do
        local row = GetSupervisorAgentsSummaryByVccRow(vcc, filters)
        if row then
            table.insert(rows, row)
        end
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
    })
end

local function GetTotalCountersRow(row)
    local total = 0
    for _, value in pairs(row) do
        total = total + value
    end
    return total
end

local function AppendCountersRows(row, rowToAppend)
    if not rowToAppend then return row end

    row = row or {}
    for key, value in pairs(rowToAppend) do
        if type(value) == "number" then
            row[key] = (row[key] or 0) + rowToAppend[key]
        end
    end
    return row
end

local function MakeInteractionsSummaryDirectionRow(interactions)
    if not interactions then return nil end

    local row = {
        Bot = interactions.processFlow,
        Waiting = interactions.waiting,
        Queued = interactions.queued,
        TakenWrapup = interactions.assigned,
    }
    row.Total = GetTotalCountersRow(row)
    return row
end

local function MakeInteractionsSummaryChannelRow(interactions, direction)
    if not interactions then return nil end

    if direction == "INBOUND" then
        return MakeInteractionsSummaryDirectionRow(interactions.inbound)
    end
    if direction == "OUTBOUND" then
        return MakeInteractionsSummaryDirectionRow(interactions.outbound)
    end

    local inbound = MakeInteractionsSummaryDirectionRow(interactions.inbound)
    local outbound = MakeInteractionsSummaryDirectionRow(interactions.outbound)
    return AppendCountersRows(inbound, outbound)
end

local function MakeInteractionsSummaryRow(indicators, filters)
    local interactions = indicators.interactions or {}
    local channel = (not EmptyFilter(filters.Channel) and filters.Channel) or nil
    local direction = (not EmptyFilter(filters.Direction) and filters.Direction) or "ANY"
    if channel then
        return MakeInteractionsSummaryChannelRow(interactions[channel], direction)
    end

    local row = nil
    for channel, channelInteractions in pairs(interactions) do
        local channelRow = MakeInteractionsSummaryChannelRow(channelInteractions, direction)
        if channelRow then
            row = AppendCountersRows(row, channelRow)
        end
    end
    return row
end

local function GetSupervisorInteractionsSummaryByCampaignRow(campaignId, filters)
    local indicators = GetSupervisorCampaignView({ campaignId = campaignId })
    if not indicators then return end
 
    local _, vcc = utils.SeparateVCC(campaignId)
    local row = MakeInteractionsSummaryRow(indicators, filters)
    if not row then return end

    row.CampaignId = campaignId
    row.Vcc = vcc
    return row
end

function RunSupervisorInteractionsSummaryByCampaign(definition, request, _T)
    local filters = request.filters or {}
    local campaignIds = FilterCampaigns(filters)
    local rows = {}
    for _, campaignId in pairs(campaignIds) do
        local row = GetSupervisorInteractionsSummaryByCampaignRow(campaignId, filters)
        if row then
            table.insert(rows, row)
        end
    end
    return MakeResultRows({
        rows = rows,
        definition = definition,
        columns = definition.columns,
        _T = _T,
    })
end

local function MixVccIndicatorsRow(originalRow, rows)
    local vcc = originalRow.Vcc
    local row = rows.dict[vcc] or { Vcc = vcc }
    row = AppendCountersRows(row, originalRow)
    if not rows.dict[vcc] then
        rows.dict[vcc] = row
        table.insert(rows.array, row)
    end
end

function MixVccIndicatorsResult(result)
    local originalRows = result.data
    local rows = { array = {}, dict = {} }
    for _, originalRow in pairs(originalRows) do
        MixVccIndicatorsRow(originalRow, rows)
    end
    return { result = "OK", data = rows.array }
end
