local monitoringUtils = require("apps.provisioning.system_monitoring.utils")

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_definition = {
    category = "outbound_engine",
    name = "Contact Details",
    description = "Details of system contacts",
    filters = {
        monitoringUtils.GetField("Vcc", { required = true }),
        monitoringUtils.GetField("OutboundProcessId"),
        monitoringUtils.GetField("ImportId"),
        monitoringUtils.GetField("BatchId"),
        monitoringUtils.GetField("Contact"),
        monitoringUtils.GetField("OutboundContactState"),
        monitoringUtils.GetField("OutboundContactStateCause"),
        monitoringUtils.GetField("DelayScheduled"),
    },
    columns = {
        monitoringUtils.GetField("DialerContactId", { id = "Id" }),
        monitoringUtils.GetField("ContactName", { id = "Name" }),
        monitoringUtils.GetField("ExternalId"),
        monitoringUtils.GetField("OutboundProcessId"),
        monitoringUtils.GetField("ImportId"),
        monitoringUtils.GetField("BatchId"),
        monitoringUtils.GetField("BatchPriority", { batchColumn = true }),
        monitoringUtils.GetField("Priority"),
        monitoringUtils.GetField("OutboundContactState"),
        monitoringUtils.GetField("OutboundContactStateCause"),
        monitoringUtils.GetField("ScheduleDate"),
        monitoringUtils.GetField("ManualScheduleDate"),
        monitoringUtils.GetField("Agent"),
        monitoringUtils.GetField("LastContactDate"),
        monitoringUtils.GetField("LastAddress"),
        monitoringUtils.GetField("InteractionCode", { id = "LastInteractionCode", name = "Last Result" }),
        monitoringUtils.GetField("LowLevelCode", { id = "LastLowLevelCode", name = "Last Low Level Result" }),
        monitoringUtils.GetField("LastManagementResult"),
        monitoringUtils.GetField("LastAgent"),
        monitoringUtils.GetField("Retries"),
        monitoringUtils.GetField("DailyRetries"),
    },
    downloadDetails = true,
    hasSize = true,
    order = {
        type = "columns",
        default = "Id",
    },
}

function GetDefinition(_T)
    return monitoringUtils.GetViewDefinition(m_definition, _T)
end

function Run(request, _T)
    local result = monitoringUtils.RunSearchOutboundContacts(
        monitoringUtils.DefinitionWithout(m_definition, "batchColumn"), request, _T)
    if result.result ~= "OK" then return result end

    return monitoringUtils.AppendBatchFields(result, m_definition, _T)
end

function GetDetails(itemId)
    return monitoringUtils.GetContactDetails(itemId)
end
