local monitoringUtils = require("apps.provisioning.system_monitoring.utils")

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_definition = {
    category = "interactions",
    name = "Interactions Realtime By Campaign",
    description = "Active interactions summary by campaign",
    filters = {
        monitoringUtils.GetField("Vcc"),
        monitoringUtils.GetField("CampaignId"),
        monitoringUtils.GetField("Channel"),
        monitoringUtils.GetField("DirectionFilter"),
    },
    columns = {
        monitoringUtils.GetField("Vcc"),
        monitoringUtils.GetField("CampaignId"),
        monitoringUtils.GetField("TotalInteractions"),
        monitoringUtils.GetField("ProcessFlowInteractions"),
        monitoringUtils.GetField("WaitingInteractions"),
        monitoringUtils.GetField("QueuedInteractions"),
        monitoringUtils.GetField("AssignedInteractions"),
        monitoringUtils.GetField("RunningContacts", { dialerCampaignColumn = true, name = "Running (Dialer)" }),
    },
}

function GetDefinition(_T)
    return monitoringUtils.GetViewDefinition(m_definition, _T)
end

function Run(request, _T)
    local result = monitoringUtils.RunSupervisorInteractionsSummaryByCampaign(
        monitoringUtils.DefinitionWithout(m_definition, "dialerCampaignColumn"), request, _T)
    if result.result ~= "OK" then return result end

    result = monitoringUtils.AppendDialerCampaignFields(result, m_definition, _T)
    if result.result ~= "OK" then return result end

    for _, row in pairs(result.data) do
        row.Running = row.Running or 0
        row.Total = row.Total + row.Running
    end
    return result
end

function GetDetails(itemId)
    return { result = "FAIL", reason = "Not supported" }
end
