--[[
-- @file xls_exporter.lua
--
-- @brief Exporter plugin which can handle XLS files
--]]

-- Plugin interfaces (defined later)
local exporter_interface;

-- class xls_exporter
local xls_exporter = {}
local __xls_exporter_mt = {}
__xls_exporter_mt.__index = xls_exporter

-- just opens a new handle
function xls_exporter:new()
	return setmetatable({
		interface = exporter_interface;
	}, __xls_exporter_mt);
end

-- writes the headers
function xls_exporter:write_headers(headers, callback)
	error("Not implemented");
end

-- writes a row
function xls_exporter:write_row(row, callback)
	error("Not implemented");
end

-- writes a lot of rows
function xls_exporter:bulk_write(rows, callback)
	error("Not implemented");
end

-- does nothing, but it is required
function xls_exporter:close()
	error("Not implemented");
end

--[[
-- Defines the interface required by the app
--]]
exporter_interface = {
	-- the type of file this exporter can handle
	type = "xls",

	practicable = false,

	-- indicates this file can be streamed on the network
	streamable = false,

	-- factory function used to create new exporters
	-- path is an interface requirement, we do not use it
	factory = function(report, _t, path)
	--	LogDebug("[REPORTS:xls_exporter:exporter_interface.factory]---> report: '%s', _t: '%s', path: '%s'", report, _t, path)
		return xls_exporter:new();
	end
};

return exporter_interface;