/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint newcap:false, quotmark:false */
/*global $, _, inConcert */

(function () {
	/*jsh--int validthis:true*/
	"use strict";

	var _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;
	var helpers = inConcert.reports.helpers;
	
	var filters = {
		Interaction : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Type')%>",
				"<%=_T('Campaign')%>",
				"<%=_T('Account')%>",
				"<%=_T('Disposition Code')%>"
			],
			Filters : {
				Campaign : {
					label : "<%=_T('Campaign')%>",
					fields : []
				},
				CampaignAgent : {
					label : "<%=_T('Campaign, Agent')%>",
					fields : ["<%=_T('Agent')%>"]
				},
				CampaignFacebookPost : {
					label : "<%=_T('Campaign, Facebook/Instagram Post')%>",
					fields : [
						"<%=_T('Facebook/Instagram Page')%>",
						"<%=_T('Facebook/Instagram Post')%>"
					]
				},
				CampaignTweet : {
					label : "<%=_T('Campaign, Tweet')%>",
					fields : [
						"<%=_T('Tweet')%>"
					]
				},
				DialerCampaign : {
					label : "<%=_T('Campaign, Batch, Phone Type')%>",
					fields : [
						"<%=_T('Batch')%>",
						"<%=_T('Phone Type')%>"
					]
				}
			}
		},
		InteractionAgent : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Type')%>",
				"<%=_T('Campaign')%>",
				"<%=_T('Account')%>",
				"<%=_T('Disposition Code')%>",
				"<%=_T('Agent')%>"
			],
			Filters : {
				CampaignAgent : {
					label : "<%=_T('Campaign, Agent')%>",
					fields : []
				}
			}
		},
		InteractionAttentionLevel : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Type')%>",
				"<%=_T('Campaign')%>",
				"<%=_T('Account')%>",
				"<%=_T('Disposition Code')%>",
				"<%=_T('Attention Level')%>"
			],
			Filters : {
				CampaignAgent : {
					label : "<%=_T('Campaign, Att Level')%>",
					fields : []
				}
			}
		},
		InteractionCount : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Type')%>",
				"<%=_T('Campaign')%>"
			],
			Filters : {
				Campaign : {
					label : "<%=_T('Campaign')%>",
					fields : []
				}
			}
		},
		InteractionRule : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Type')%>",
				"<%=_T('Campaign')%>",
				"<%=_T('Direction')%>",
				"<%=_T('Account')%>",
				"<%=_T('Rule')%>"
			],
			Filters : {
				Campaign : {
					label : "<%=_T('Campaign, Rule')%>",
					fields : []
				}
			}
		},
		InteractionVoiceBlaster : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Campaign')%>",
				"<%=_T('Batch')%>",
				"<%=_T('VB Id')%>",
				"<%=_T('Menu Id')%>"
			],
			Filters : {
				Campaign : {
					label : "<%=_T('Campaign, Batch, VB Id, Menu Id')%>",
					fields : []
				}
			}
		},
		Agent : {
			BaseFilters : [
				"<%=_T('Date')%>",
				"<%=_T('Agent')%>"
			],
			Filters : {
				Agent : {
					label : "<%=_T('Agent')%>",
					fields : []
				},
				AgentCampaign : {
					label : "<%=_T('Agent, Campaign')%>",
					fields : ["<%=_T('Campaign')%>"]
				}
			}
		}
	};

	function FormEditMetric(metric, controller, $container, context)
	{
		FormEditMetric.uber.constructor.call(this, "/inconcert/apps/reports/pages/forms/edit_metric.<%=_L%>.html#grunt-cache-bust", $container, context);
		$.extend(this, {
			metric    : metric,
			controller: controller,
			permissions: {
				creation: "save_custom_metrics"
			}
		});
	}

	inConcert.Utils.inherit(FormEditMetric, inConcert.reports.FormBase);

	FormEditMetric.prototype.onRender = function(callback) {
		var self = this;

		function makeCompletionHandler(callback) {
			return _.once(function() {
				self.once("__ready", callback);
			});
		}

		// filled when setting-up composite controls
		// this happens only when creating a new metric or editing a composite one
		var compositeData = {
			operation: undefined,
			shownMetrics: {
				percentage: { count: 0, format: undefined, metrics: { }, filterByFormat: true },
				average   : { count: 0, format: undefined, metrics: { }, filterByFormat: false },
				sum       : { count: 0, format: undefined, metrics: { }, filterByFormat: true },
				sub		  : { count: 0, format: undefined, metrics: { }, filterByFormat: true }
			}
		};

		var setupBasicData = function() {
			if (self.metric === null) {
				self.setTitle( "<%=_T('New Metric')%>" );
			} else {
				var title = self.hasPermissionToSave() ?
					"<%=_T('Edit Metric')%> <span>\"{0}\"</span>" :
					"<%=_T('View Metric')%> <span>\"{0}\"</span>";
				self.setTitle(format( title, self.metric.name || self.metric.id));

				var loadMetricToShow = function(item) {
					var metric = self.controller.loadMetric(item.id, {sync:true});
					if (!metric) {
						return false; // maybe the metric was deleted by another user...
					}

					// now I can set the format, if not set already
					var shown_data = compositeData.shownMetrics[compositeData.operation];
					if (!shown_data.format) {
						shown_data.format = metric.format && metric.format.type || "default";
					}
					// .. increase reference count (in order to reset the counter when becomes zero)
					shown_data.count += 1;
					// and mark the metric as used (see iterator in setupCompositeSelector())
					shown_data.metrics[metric.id] = true;
					return metric;
				};

				helpers.displayDataModel(self.container, self.metric, {
					disabledFields : [ "composite", "scope" ],
					onFieldDisabled: _.bind(self.onFieldDisabledChange, self, true),
					listIterator   : function(attrName, $field, item/*, key*/) {
						if ( $field.is('[rd-metric-ref]') ) {
						// help the loader to correctly populate the list holding the metrics being in use
						// this iterator also comes handy to setup some restrictions
							return loadMetricToShow(item);
						}
					},
					customLoaders: {
						'metadata.expression': function($field, value) {
							self.hExpressions.init(value);
						}
					}
				});
			}
		};

		var setupCompositeSelector = function(callback) {
			if (self.metric && !self.metric.composite) {
				return callback(true); // metric is not composite so..
			}

			var $selectFormat       = self.container.find('[rd-for-composite-metric="true"] [ic-data-bind="format.type"]');
			var $selectOperation    = self.container.find('[ic-data-bind="metadata.operation.name"]');
			var $operationContainer = $("#rpt_metrics_operations_container");

			compositeData.operation = self.metric && self.metric.metadata.operation.name || $selectOperation.val();

			// used to filter out available metrics while populating the list
			// called each time a metric is about to be added to the list
			var iterator = function(metric) {
				if (self.metric && metric.id === self.metric.id) {
					return false;
				}

				// note: the selected operation is used as base index in order to provide more flexibility
				var shownMetrics = compositeData.shownMetrics[compositeData.operation];
				if (shownMetrics.metrics[metric.id]) {
					return false;
				}

				if (shownMetrics.filterByFormat) {
					var fmt = metric.format && metric.format.type || "default";
					if (shownMetrics.format && shownMetrics.format !== fmt) {
						return false;
					}
				}
			};

			var has_permission_to_save = self.hasPermissionToSave();

			var selectorHandle;
			var onMetricListReady = function(success, errorThrown, $metrics_available) {
				if (!success) {
					return callback(false, errorThrown);
				}

				var linkSelectors = function() {

					var onMetricReceived = function(metric) {
						var shown = compositeData.shownMetrics[compositeData.operation];
						shown.metrics[metric.id] = true;
						shown.count += 1;
						return shown;
					};

					var resolveFormat = function($item, force) {
						var metric = self.controller.loadMetric($item.attr('rd-metric'), {sync:true});

						var shown = onMetricReceived(metric);

						if (shown.count === 1 || force) {
							shown.format = metric.format && metric.format.type || "default";
							if ($.inArray(compositeData.operation, "sum", "average", "sub") >= 0) {
								$selectFormat.val(shown.format);
							}
							if (shown.filterByFormat) {
								selectorHandle.reset(); // will filter-out metrics having different format type
							}
						}
					};

					// we use sortable to link metric lists
					// it's easier than implementing drag&drop separately
					var makeSortable = function($list, options) {
						if ( !has_permission_to_save ) {
							return;
						}
						const connect_with = "#" + $metrics_available.uniqueId().attr("id");
						$list.sortable($.extend({
							cursor: "move",
							connectWith: connect_with,
							items: ".sorting-initialize",
							receive: function(event, ui) {
								if ($(this).children('[ic-role!="template"]').length > 1) { // element is already in the list
									$metrics_available.sortable('cancel');
								} else {
									resolveFormat(ui.item, true);
								}
							},
							remove: function(event, ui) {
								var shown = compositeData.shownMetrics[compositeData.operation];
								shown.count -= 1;
								delete shown.metrics[ui.item.attr('rd-metric')];
								if (shown.count === 0) {
									shown.format = undefined;
									if (compositeData.operation !== "percentage") {
										$selectFormat.val("default");
									}
									if (shown.filterByFormat) {
										selectorHandle.reset();
									}
								}
							}
						}, options || {}));
					};

					var $container = $operationContainer.children('[rd-for-operation="percentage"]');
					makeSortable($container.find('[ic-role="percentage-field-total"] > ul'));
					makeSortable($container.find('[ic-role="percentage-field-value"] > ul'));

					$container = $operationContainer.children('[rd-for-operation="average"]');
					makeSortable($container.find('[ic-role="average-field-count"] > ul'), {
						receive: function(event, ui) {
							if ($(this).children('[ic-role!="template"]').length > 1) { // element is already in the list
								$metrics_available.sortable('cancel');
							} else {
								var metric = self.controller.loadMetric(ui.item.attr('rd-metric'), {sync:true});
								if (metric.format && metric.format.type && metric.format.type !== "default") {
									self.emit("error", "<%=_T('Metric must be a Counter in order to represent the element count')%>");
									$metrics_available.sortable('cancel');
								} else {
									onMetricReceived(metric);
								}
							}
						}
					});
					makeSortable($container.find('[ic-role="average-field-sum"] > ul'));

					$container = $operationContainer.children('[rd-for-operation="sub"]');
					makeSortable($container.find('[ic-role="sub-field-minuend"] > ul'), {
						receive: function(event, ui) {
							resolveFormat(ui.item);
						}
					});
					makeSortable($container.find('[ic-role="sub-field-subtrahend"] > ul'), {
						receive: function(event, ui) {
							resolveFormat(ui.item);
						}
					});

					$container = $operationContainer.children('[rd-for-operation="sum"]');
					makeSortable($container.find('[ic-role="sum-field-values"] > ul'), {
						receive: function(event, ui) {
							resolveFormat(ui.item);
						}
					});
				};

				linkSelectors();
				if ( has_permission_to_save ) {
					const operation_container_id = "#" + $operationContainer.uniqueId().attr("id");
					const connect_with = operation_container_id + ' > div[rd-for-operation] > div[ic-role] > ul';
					$metrics_available.sortable({
						cursor: "move",
						connectWith: connect_with,
						items: ".sorting-initialize",
					});
				}

				self.container.find('[ic-data-bind="composite"]').on('change', function() {
					var checked = $(this).is(':checked');
					self.switchUseCaseByAttr('rd-for-composite-metric', checked ? "true" : "false")
						.done(function() {
							if (checked) {
								$selectOperation.trigger('change');
							}
							if (self.hNavigation) {
								self.context.saveButton.prop('disabled', !self.hNavigation.allVisited());
							}
						});
				});

				$selectOperation.on('change', function() {
					compositeData.operation = $(this).val();
					self.switchUseCaseByAttr('rd-for-operation', compositeData.operation, $operationContainer)
						.done(function() {
							selectorHandle.reset();
							$selectFormat.val(
								compositeData.operation === "percentage" && "Percentage" || compositeData.shownMetrics[compositeData.operation].format || "default"
							);
							$selectFormat.trigger("change");  // needed to update `maxLength'
						});
				});

				callback(true);
			};

			selectorHandle = helpers.setupMetricsSelector({
				container : $('#rd_composite_metric_selector'),
				controller: self.controller,
				iterator  : iterator,
				callback  : onMetricListReady,
				update	  : function() {
					$("#form_edit_metric")
							.find("li[rd-metric]")
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).
									addClass("sorting-initialize")
									.parent()
									.sortable('refresh');
							});
				}
			});
		};

		// setups controls used only in base metric
		var setupBaseMetricControls = function(callback) {
			if (self.metric && self.metric.composite) {
				return callback(true); // metric is composite so..
			}

			var currentScope,
				$selectCci				= self.container.find('[ic-data-bind="cciProject"]'),
				$selectWebchat			= self.container.find('[ic-data-bind="webchatProject"]'),
				$selectWebcall			= self.container.find('[ic-data-bind="webcallProject"]'),
				$selectContactForm		= self.container.find('[ic-data-bind="contactformProject"]'),
				$selectEmailSurvey		= self.container.find('[ic-data-bind="surveyProject"]'),
				$selectApi				= self.container.find('[ic-data-bind="apiProject"]'),
				$selectBatchData		= self.container.find('[ic-data-bind="batch_dataProject"]'),
				$selectProcessFlow		= self.container.find('[ic-data-bind="process_flowProject"]'),
				$selectIntegration		= self.container.find('[ic-data-bind="integrationType"]');
			var $selectInteractionType	= self.container.find('[ic-data-bind="metadata.interactionType"]');
			var $selectIncrement		= self.container.find('[ic-data-bind="metadata.increment"]');

			function isInteractionType(candidate) {
				return candidate && candidate !== "all";
			}

			function resetExpressions() {
				if (!self.renderComplete || !self.hExpressions) {
					return;
				}

				var interactionType = $selectInteractionType.val();
				self.hExpressions.init(isInteractionType(interactionType) ? [{
					type      : 'simple',
					field     : { id: 'Type' },
					comparator: '==',
					value     : interactionType
				}] : { });
			}

			// note: this is a factory
			function makeChecker(extraCheck) {
				extraCheck = extraCheck || function() { return true; };

				var checkInteraction = currentScope.match(/^Interaction(Agent)?$/) && function() {
					var integrationScope = $selectIntegration.val();
					var projects = {
						cci				: $selectCci.val(),
						webchat			: $selectWebchat.val(),
						webcall			: $selectWebcall.val(),
						contactform		: $selectContactForm.val(),
						survey			: $selectEmailSurvey.val(),
						api				: $selectApi.val(),
						batch_data		: $selectBatchData.val(),
						process_flow	: $selectProcessFlow.val()
					};
					var interactionType = $selectInteractionType.val();
					return function(d) {
						if (d.sub_scope && d.sub_scope !== interactionType) {
							return false;
						}
						if (helpers.isScopeCustom(d.scope)) {
							return (d.scope === "cci" || d.scope === "batch_data" || d.scope === "process_flow" || 
									d.scope === integrationScope) && projects[d.scope] === d.projectId;
						}
						return true;
					};
				}() || function() {
					return true;
				};

				return function(d) {

					// matches work only with plain detail fields (aka the projectable ones)
					if ( !d.isProjectable || !helpers.isDimensionCompatible(d, currentScope) ) {
						return false;
					}

					var res = checkInteraction(d) && extraCheck(d);

					// show consistent displayName for custom fields
					if ( res && helpers.isScopeCustomField(d.scope) ) {
						return _.extend({}, d, {
							displayName: d.name // api.<name>
						});
					}

					return res;
				};
			}

			var updateWhenComplete = makeCompletionHandler(function() {
				updateIncrementOptions();
			});

			function updateIncrementOptions() {
				if (!self.renderComplete) {
					return updateWhenComplete();
				}

				if (currentScope === "InteractionCount") {
					helpers.populateList($selectIncrement, []);
				} else {
					self.controller.listDimensions().done(function(dimensions) {
						helpers.populateList($selectIncrement, dimensions, makeChecker(function(d) {
							return (d.type === "number" || d.type === "Time");
						}), {
							sort: true,
							candidateValue: self.metric && self.metric.metadata.increment || $selectIncrement.val()
						});
					});
				}
			}

			// called when changing the scope or any other important attribute
			function tryCleanupExpressions() {
				var deferred = new $.Deferred();

				function hasExpressions() {
					return self.hExpressions && _.reject(self.hExpressions.get().childs, function(expr) {
						return expr.type !== 'simple' || _.contains(['Type'], expr.field.id);
					}).length > 0;
				}

				if (!self.renderComplete) {
					_.defer(_.bind(deferred.resolve, deferred));
					return deferred;
				}

				if (! hasExpressions() ) {
					_.defer(_.compose(resetExpressions, _.bind(deferred.resolve, deferred)));
					return deferred;
				}

				var question = format("{0}<br><br>&nbsp;&nbsp;&nbsp;&nbsp;{1}",
					"<%=_T('This action is going to remove all rules you have already setup.')%>", "<%=_T('Continue?')%>"
				);

				helpers.askForConfirmation(question, "<%=_T('About to change settings')%>")
					.done(_.compose(resetExpressions, _.bind(deferred.resolve, deferred)))
					.fail(_.bind(deferred.reject, deferred));

				return deferred;
			}

			function completeSetup() {

				(function () {
					var $scope_selector = self.container.find('[ic-data-bind="scope"]');
					var $filter_selector = self.container.find('[ic-data-bind="metadata.filter"]');
					var $filter_text = self.container.find('#formFilterGroupBy');
					
					$filter_selector.on("change", function() {
						var scope = $scope_selector.val();
						var filter = $filter_selector.val();
						var text = "<%=_T('Can group by')%><%=_T(':')%> ";
						if (scope && filter) {
							//custom
							if (filter.indexOf("custom_") === 0) {
								filter = filter.replace("custom_", "");
								var filterData = self.customFilters[scope][filter];
								text += "<%=_T('Date')%>";
								$.each(filterData.columns, function(_, column) {
									if (column.id !== "Slice" && column.id !== "Shift") {
										text += ", " + column.label;
									}
								});
							}
							//default
							else {
								var scopeFilters = filters[scope];
								$.each(scopeFilters.BaseFilters, function(i, column) {
									text += (i > 0? ", " : "") + column;
								});
								$.each(scopeFilters.Filters[filter].fields, function(_, column) {
									text += ", " + column;
								});
							}
						}
						$filter_text.html(text);
					});
					
					currentScope = $scope_selector.on("change", function() {
						tryCleanupExpressions().done(function() {
							currentScope = $scope_selector.val();
							
							var html = "";
							if (currentScope) {
								var scopeFilters = filters[currentScope];
								var scopeCustomFilters = self.customFilters[currentScope] || {};

								$.each(scopeFilters.Filters, function(id, filter) {
									html += format("<option value='{0}'>{1}</option>",
										id, filter.label);
								});
								$.each(scopeCustomFilters, function(id, filter) {
									html += format("<option value='{0}'>{1}</option>",
										"custom_" + id, filter.label + " (<%=_T('custom')%>)");
								});
							}
							$filter_selector.html(html);
							if (self.metric && self.metric.metadata && self.metric.metadata.filter) {
								$filter_selector.val(self.metric.metadata.filter);
							}
							$filter_selector.change();
							
							//$scope_selector.data('rd-current-value', currentScope);
							self.switchUseCaseByAttr('rd-for-scope', $scope_selector.children(':selected').attr('rd-scope'))
								.done(function() {
									updateIncrementOptions();
								});
						}).fail(function() {
							$scope_selector.val(currentScope); //$scope_selector.data('rd-current-value'));
						});
					}).val();
					if (!self.metric) {
						_.defer(function() {
							$scope_selector.trigger('change');
						});
					}
				}());

				self.container.find('[rd-for-composite-metric="false"] [ic-data-bind="format.type"]').on('change', function() {
					self.switchUseCaseByAttr('rd-format-type', $(this).val());
				});

				$selectInteractionType.data(
					'rd-current-value',
					$selectInteractionType.on('change', function() {
						var $this = $(this);
						tryCleanupExpressions()
							.done(function() {
								$this.data('rd-current-value', $this.val());
								updateIncrementOptions();
							})
							.fail(function() {
								$this.val($this.data('rd-current-value'));
							});
					}).val()
				);

				(function() {
					var updateWhenComplete = makeCompletionHandler(function() {
						$selectIncrement.trigger('change');
					});

					// handle changes in the increment-by field
					$selectIncrement.on('change', function() {
						var $this = $(this);
						var $selected = $this.children(':selected');

						if (!self.renderComplete) {
							return updateWhenComplete();
						}
						if ($selected.is('[ic-role="template"]')) {
							return; // this is not the option I am looking for
						}

						var isDefaultIncrement = $this.val() === "default" && currentScope !== "InteractionCount";
						var $minMax = self.container.find('[ic-data-bind="extras.MinMaxIndicator"]')
							.prop('disabled', isDefaultIncrement);
						if (isDefaultIncrement) {
							$minMax.prop('checked', false);
						}
						self.onFieldDisabledChange(isDefaultIncrement, $minMax);

						$this.parent().attr('title', $selected.attr('title'));
					});
				})();

				var has_permission_to_save = self.hasPermissionToSave();

				self.hExpressions = helpers.booleanExpressionGenerator(
					self.container.find('[ic-role="expression-builder"]'),
					{
						readonly: !has_permission_to_save,
						fieldTemplate: '<option ic-role="template" value="{id}" ic-field-name="{name|id}" rd-dimension-type="{type}" style="display:none">{label|displayName|name}</option>',
						loadAvailableFields: function($target, callback, expectedField) {
							self.controller.listDimensions().done(function(dimensions) {
								helpers.populateList($target, dimensions, makeChecker(function(d) {
									if (d.isCondition === false ||
										(_.contains([ "Type", "WasLogged" ], d.id) && d.id !== expectedField) ||
										_.contains([ "ID" ], d.name) ||
										_.contains([ "datetime", "Date"  ], d.type)
									) {
										return false;
									}
								}), {
									sort: true,
									disableOnEmpty: true,
									defaultItem: { id: "", name: "", type: "string", displayName: "<%=_T('No fields available')%>"   },
									firstItem  : { id: "", name: "", type: "string", displayName: "<%=_T('Please select a field')%>" }
								});
								callback(expectedField && _.contains(["Type"], expectedField));
							});
						},
						selectedFieldChanged: function(params, callback) {
							var fieldType = params.selectedItem.attr('rd-dimension-type');

							if (self.metric && self.metric.metadata.orphanedFields) {
								var dimension = self.controller.loadDimension(params.value, {sync:true});
								var orphanedFound = _.some(self.metric.metadata.orphanedFields, function(item, index) {
									if ( item.field === dimension.name ) {
										self.metric.metadata.orphanedFields.splice(index, 1);
										return true;
									}
								});
								if (orphanedFound && self.metric.metadata.orphanedFields.length === 0) {
									self.emit("status", "<%=_T('Everything looks good now, you can go on saving this metric.')%>");
									self.context.saveButton.prop('disabled', !self.hNavigation.allVisited());
								}
							}
							switch(fieldType) {
								case "Time":
									return callback('number', '<%=_T("seconds...")%>');
								case "enum":
									return callback("enum", undefined, self.controller.loadDimension(params.value, {sync:true}).domain);
								default:
									return callback(fieldType, '<%=_T("value...")%>');
							}
						},
						formulaChanged: function(exprstr) {
							$('#expressionPreview').val(exprstr);
						}
					}
				);

				if (!self.metric) {
					updateIncrementOptions();
				}

				callback(true);
			}
			
			function LoadCuts(callback) {
				self.controller.listSummaryFilters("custom").then(function(summaryFilters) {
						var customFilters = {};
						$.each(summaryFilters, function(_, filter) {
							customFilters[filter.scope] = customFilters[filter.scope] || {};
							customFilters[filter.scope][filter.id] = filter;
						});
						self.customFilters = customFilters;
					callback();
				});
			}

			helpers.configureIntegration({
				container : self.container.find('[ic-role="configure-integration"]'),
				controller: self.controller,
				tweetyApi : inConcert.app.getTweetyAPI(),
				beforeChange: function(category, newScope, newValue, callback) {
					tryCleanupExpressions().done(callback)
						.fail(_.partial(callback, false));
				},
				afterChange: function(category, newScope/*, newValue*/) {
					if (category === 'frontend') {
						switch (newScope) {
							case 'webchat':
								$selectInteractionType.val('CHAT').prop('disabled', true);
								break;
							case 'webcall':
								$selectInteractionType.val('CALL').prop('disabled', true);
								break;
							case 'api':
							case 'contactform':
								$selectInteractionType.val('WEBCONTACT').prop('disabled', true);
								break;
							default:
								$selectInteractionType.prop('disabled', false);
						}
						$selectInteractionType.trigger('change');
					} else {
						updateIncrementOptions();
					}
					// TODO: Do it in the helper
					self.switchUseCaseByAttr('rd-for-integration-type', newScope, this.container.find('[rd-integration-category="'+ category +'"]'));
				}
			}).done(function(errorThrown) {
				if (errorThrown) {
					self.emit("error", format("<%=_T('Failed to load Integration Projects: {0}')%>", _T(errorThrown)));
				}

				LoadCuts(completeSetup);
			});
		};

		var setupHandlersForMaxLengthSync = function (callback) {
			self.controller.loadDefaultValues()
			.done(function (defaultValues) {
				var DEFAULT_FIELD_MAX_LENGTH = defaultValues.DEFAULT_FIELD_MAX_LENGTH;
				
				/////////////////////////////////////
				// rd-for-composite-metric="false" //
				/////////////////////////////////////
				
			  // Keep `maxLength' synchronized with `format.type' when `defaultMaxLength' is 
			  // checked.
			
			  var $type = self.container.find('[rd-for-composite-metric="false"] [ic-data-bind="format.type"]');
			  var $maxLength = self.container.find('[rd-for-composite-metric="false"] [ic-data-bind="maxLength"]');
			  var $defaultMaxLength = self.container.find('[rd-for-composite-metric="false"] [ic-data-bind="defaultMaxLength"]');
			
			  var updateMaxLength = function () {
			    var typeValue = $(this).val();
				  var maxLengthValue = typeValue ? 
					  DEFAULT_FIELD_MAX_LENGTH[typeValue.toUpperCase()] || 
					  DEFAULT_FIELD_MAX_LENGTH.DEFAULT : 
					  0;
				
				  $maxLength.val(maxLengthValue);
			  };
			
			  $defaultMaxLength
			  .off("change")
			  .on("change", function () {
				  if ($(this).prop("checked")) {
					  $maxLength.prop("disabled", false);
					
					  $type
					  .off("change.updateMaxLength")
					  .on("change.updateMaxLength", updateMaxLength);
					
					  $type.triggerHandler("change.updateMaxLength");
				  } else {
					  $type.off("change.updateMaxLength");
					  $maxLength.prop("disabled", true);
					  $maxLength.focus();
				  }
			  });
			
			  if (self.metric === null) {
				  // Initially `maxLength' is synchronized with `format.type'.
				  $defaultMaxLength.prop("checked", true);
				  $defaultMaxLength.triggerHandler("change");
			  } else {
				  // Keep `maxLength' synchronized with `format.type' if `defaultMaxLength' is 
			    // checked.
				  if ($defaultMaxLength.prop("checked")) {
				  	$maxLength.prop("disabled", true);
				  	
				    $type
				    .off("change.updateMaxLength")
				    .on("change.updateMaxLength", updateMaxLength);
				  }
			  }
			  
			  /////////////////////////////////////
				// rd-for-composite-metric="true" //
				/////////////////////////////////////
				
			  // Keep `maxLength' synchronized with `format.type' when `defaultMaxLength' is 
			  // checked.
			
			  var $typeForComposite = self.container.find('[rd-for-composite-metric="true"] [ic-data-bind="format.type"]');
			  var $maxLengthForComposite = self.container.find('[rd-for-composite-metric="true"] [ic-data-bind="maxLength"]');
			  var $defaultMaxLengthForComposite = self.container.find('[rd-for-composite-metric="true"] [ic-data-bind="defaultMaxLength"]');
			
			  var updateMaxLengthForComposite = function () {
			    var typeValue = $(this).val();
				  var maxLengthValue = typeValue ? 
					  DEFAULT_FIELD_MAX_LENGTH[typeValue.toUpperCase()] || 
					  DEFAULT_FIELD_MAX_LENGTH.DEFAULT : 
					  0;
				
				  $maxLengthForComposite.val(maxLengthValue);
			  };
			
			  $defaultMaxLengthForComposite
			  .off("change")
			  .on("change", function () {
				  if ($(this).prop("checked")) {
					  $maxLengthForComposite.prop("disabled", true);
					
					  $typeForComposite
					  .off("change.updateMaxLength")
					  .on("change.updateMaxLength", updateMaxLengthForComposite);
					
					  $typeForComposite.triggerHandler("change.updateMaxLength");
				  } else {
					  $typeForComposite.off("change.updateMaxLength");
					  $maxLengthForComposite.prop("disabled", false);
					  $maxLengthForComposite.focus();
				  }
			  });
			
			  if (self.metric === null) {
				  // Initially `maxLength' is synchronized with `format.type'.
				  $defaultMaxLengthForComposite.prop("checked", true);
				  $defaultMaxLengthForComposite.triggerHandler("change");
			  } else {
				  // Keep `maxLength' synchronized with `format.type' if `defaultMaxLength' is 
			    // checked.
				  if ($defaultMaxLengthForComposite.prop("checked")) {
				  	$maxLengthForComposite.prop("disabled", true);
				  	
				    $typeForComposite
				    .off("change.updateMaxLength")
				    .on("change.updateMaxLength", updateMaxLengthForComposite);
				  }
			  }

			  callback(true);
			})
			.fail(function (error) {
			  callback(false, error);
			});
		};

		setupBaseMetricControls(function(success, errorThrown) {
			if (!success) {
				return callback(false, errorThrown);
			}

			setupCompositeSelector(function(success, errorThrown) {
				if (!success) {
					return callback(false, errorThrown);
				}

				setupBasicData();

				setupHandlersForMaxLengthSync(function (success, errorThrown) {
					callback(success, errorThrown);
				});
			});
		});
	};

	FormEditMetric.prototype.onShow = function() {
		var self = this;

		self.initNavigation({
			allVisited: (self.metric !== null),
			onEnterPage: function(pageNumber, $pageElem, allVisited) {
				self.context.saveButton.prop('disabled',
					!allVisited || (self.metric && self.metric.metadata.orphanedFields && self.metric.metadata.orphanedFields.length > 0)
				);
			}
		});

		if (self.metric && self.metric.metadata.orphanedFields) {
			var fields = _.map(self.metric.metadata.orphanedFields, function(f) {
				return '<span style="font-weight:bold;">' + f.field + ' = ' + f.value + '</span>';
			});
			self.emit("error", format(
				"<%=_T('Metric has orphaned fields, please fix them before saving: {0}{1}')%>",
				fields.length > 1 ? "<br/>" : "",
				fields.join('<br/>')
			), true);
			self.context.saveButton.prop('disabled', true);
		}

		self.emit("__ready");
	};

	FormEditMetric.prototype.onConfirm = function(callback) {
		var self = this;

		var result = self.validate({
			iterator: _.bind(self.isFieldEnabled, self),
			customLoaders: {
				'metadata.expression': function() {
					return self.hExpressions.get();
				}
			}
		});
		if (!result.success) {
			return callback(false, result.errorMessage);
		}
		
		if ( !result.fields.maxLength.match(/^0*[1-9]\d*$/) ) {
			return callback(false, "<%=_T('Max length must be an integer greater than or equal to 1')%>");
		}

		var metric = $.extend({
			id: self.metric && self.metric.id || undefined
		}, result.fields);
		
		metric.maxLength = Number(metric.maxLength);

		self.controller.saveMetric(metric).done(function() {
			callback(true, "<%=_T('Metric saved successfully')%>");
		}).fail(function(errorThrown) {
			var message_error;
			switch ( errorThrown ) {
				case "forbidden":
					message_error = "<%=_T('You dont have permission to save')%>";
				break;
				default:
					message_error = errorThrown;
			}
			callback(false, format("<%=_T('Failed to save metric: {0}')%>", message_error));
		});
	};

	inConcert.reports.FormEditMetric = FormEditMetric;
}());
