/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint newcap:false, quotmark:false */
/*global $, _, inConcert*/

(function () {
	'use strict';

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	const helpers = inConcert.reports.helpers;

	function FormEditReport(report, controller, $container, context)
	{
		FormEditReport.uber.constructor.call(this, '/inconcert/apps/reports/pages/forms/edit_report.<%=_L%>.html#grunt-cache-bust', $container, context);
		$.extend(this, {
			report    : report,
			controller: controller,
			permissions: {
				creation: 'save_custom_reports'
			},
			pdfConfigurator: undefined
		});
	}

	inConcert.Utils.inherit(FormEditReport, inConcert.reports.FormBase);
	inConcert.reports.FormEditReport = FormEditReport;

	FormEditReport.prototype.onRender = function(callback)
	{
		const self = this;

		let context = {
			$selectType    : this.container.find('[ic-data-bind="type"]'),
			$selectCategory: this.container.find('[ic-data-bind="category"]'),

			// detail
			detail: {
				type                   : "detail",
				$selectFilter          : $('#rd_detail_filter_name'),
				selectedFilter         : undefined,
				$listDimensionsAvail   : $('#rd_detail_report_dimensions_available'),
				$listDimensionsInUse   : $('#rd_detail_report_project_fields'),
				$selectInteractionType : $('#rd_detail_filter_type'), // page 2
				selectedInteractionType: undefined,
				listFiltersFn          : "listDetailFilters",
				listDimensionsFn       : "listArchiveDimensions",
				loadingDimensions      : false
			},

			// live detail
			livedetail: {
				type                   : "livedetail",
				$selectFilter          : $('#rd_live_detail_filter_name'),
				selectedFilter         : undefined,
				$listDimensionsAvail   : $('#rd_live_detail_report_dimensions_available'),
				$listDimensionsInUse   : $('#rd_live_detail_report_project_fields'),
				$selectInteractionType : $('#rd_live_detail_filter_type'),  // page 2
				selectedInteractionType: undefined,
				listFiltersFn          : "listLiveDetailFilters",
				listDimensionsFn       : "listLiveDimensions",
				loadingDimensions      : false
			},

			// summary filter
			selectSummaryFilter : this.container.find('[ic-data-bind="cut"]'),
			listMetricsAvail    : $('#rd_rpt_available_filters_list'),
			listMetricsInUse    : this.container.find('[ic-data-bind="indicators"]'),
			summaryHelpers      : {
				selectFirstColumn : $('#summary_filter_first_field'),
				selectSecondColumn: $('#summary_filter_second_field')
			},

			selectedType            : undefined,
			selectedCategory        : undefined,
			selectedSummaryFilter   : undefined,

			integration: {
				frontend: {
					scope    : '',
					projectId: ''
				},
				backend: {
					scope    : 'cci',
					projectId: ''
				},
				batch_data: {
					scope    : 'batch_data',
					projectId: ''
				},
				process_flow: {
					scope    : 'process_flow',
					projectId: ''
				}
			},

			checkHandlers: {
				detail    : $.noop,
				livedetail: $.noop,
				summary   : $.noop
			},

			changeHandlers : {
				detail    : $.noop,
				livedetail: $.noop,
				summary   : $.noop
			}
		};

		this.context = $.extend(context, this.context);

		function makeCompletionHandler(callback) {
			return _.once(function() {
				self.once("__ready", callback);
			});
		}

		function renderDetailBase (callbackRenderDetail, detail_type, suggestedFields = [])
		{
			let detail_info;
			if (detail_type === "detail") {
				detail_info = context.detail;
			}
			else if (detail_type === "livedetail") {
				detail_info = context.livedetail;
			}
			else {
				throw("renderDetailBase - unrecognized detail_type: " + detail_type);
			}

			const allowedFilterScopes = {
				Interactions: [ "Interaction", "InteractionAgent", "InteractionAttentionLevel", "InteractionRule", "InteractionVoiceBlaster", "InteractionInternalChat" ],
				States      : [ "Agent", "AgentStateLapse" ]
			};

			function getDimensionById(id) {
				return self.controller.loadDimension(id, {sync: true});
			}

			function makeAppender($target) {
				return function(id) {
					return helpers.appendToList($target, getDimensionById(id));
				};
			}

			function dimensionAppliesToFilter (filter, d) {
				if (!helpers.isDimensionCompatible(d, filter.scope)) {
					return false;
				}

				if (helpers.isScopeCustom(d.scope)) {
					const filterProject = filter[d.scope + "Project"];
					return !filterProject || d.projectId === filter[d.scope + "Project"];
				}

				let selectedInteractionType = (detail_info.selectedInteractionType || "").toUpperCase();
				if (selectedInteractionType && d.sub_scope) {
					let belogneInteractions = d.scope.match(/^Interaction(Agent)?/);
					if (belogneInteractions && d.sub_scope.toUpperCase() !== selectedInteractionType) {
						return false;
					}
				}

				/*
					Valida si la dimension esta disponible, por canales.
					Quedo para que se pueda extender y agregar mas validaciones.
						available      = {
							channels = {
								MAIL = {show = true}
							}
						},
				*/
				if (d.available) {
					if (d.available.channels) {
						if (selectedInteractionType) {
							let channel = d.available.channels[selectedInteractionType];
							if (!channel) {
								return false;
							}
							if (!channel.show) {
								return false;
							}
						}
					}
				}
				return true;
			}

			function isUsedDimension (d) {
				const $used_dimension = detail_info.$listDimensionsInUse.children('[rd-dimension="' + d.name + '"]');
				const in_use_dimension = $used_dimension.length > 0;
				return in_use_dimension;
			}

			function isDimensionCompatible (filter, d) {
				if (detail_info.selectedFilter && !dimensionAppliesToFilter(filter, d)) {
					return false;
				}
				if (helpers.isScopeCustom(d.scope)) {
					if (d.scope === "cci") {
						return d.projectId === context.integration.backend.projectId;
					}
					if (d.scope === "batch_data") {
						return d.projectId === context.integration.batch_data.projectId;
					}
					if (d.scope === "process_flow") {
						return d.projectId === context.integration.process_flow.projectId;
					}
					var frontendConf = context.integration.frontend;
					return d.scope === frontendConf.scope && d.projectId === frontendConf.projectId;
				}
				return true;
			}

			var appendDimensionInUse = makeAppender(detail_info.$listDimensionsInUse);
			var appendDimensionAvail = makeAppender(detail_info.$listDimensionsAvail);
			var has_permission_to_save = self.hasPermissionToSave();

			var listDimensions = function(callbackDimensions = $.noop)
			{
				if (detail_info.loadingDimensions) {
					return;
				}
				detail_info.loadingDimensions = true;

				if (self.hNavigation) {
					// Mientras se realiza la carga en el DOM se desactiva la solapa 3
					self.hNavigation.disablePage(3);
				}

				// const uid = inConcert.Utils.generateGuid();
				// console.log("+START listDimensions - uid: " + uid);

				self.controller[detail_info.listDimensionsFn]().done(function(dimensions)
				{
					helpers.populateListAsync(
						detail_info.$listDimensionsAvail,
						dimensions,

						// la funcion Iteradora
						function(d)
						{
							if (d.isProjectable === false || _.findWhere(suggestedFields, {name: d.id})){
								return false;
							}
							let is_used = isUsedDimension(d);
							if (is_used) {
								return false;
							}
							let res = isDimensionCompatible(detail_info.selectedFilter, d);
							// show consistent displayName for custom fields
							if ( res && helpers.isScopeCustomField(d.scope) ) {
								return _.extend({}, d, {
									displayName: d.name // api.<name>
								});
							}
							return res;
						},

						// Los options
						{
							sort: true,
							maxAppendPerLoop: 20,
							pauseBetweenLoopMs: 100,
							waitingMsg: "<%=_T('Please wait, adding Dimensions rows, {0} of {1}.')%>",
							waitingFn: function(/*message*/) {
								// Aca se podria mostrar un waiting si fuese necesario
							//	console.log(message + " uid: " + uid);
							},
							// Valida si la fila se puede agregar
							canAddFn: function(item) {
								const selector = "[rd-dimension='" + item.id + "']";
								const $currentItem = detail_info.$listDimensionsAvail.find(selector);
								if (!$currentItem || $currentItem.length === 0) {
									// si ya existe no la agrega
									return true;
								}
								return false;
							}
						},

						// Esta es la funcion que se invoca cuando la carga de la lista asincronica ha finalizado.
						function(/*itemCount*/)
						{
							_.each(suggestedFields, function(suggestedField) {
								const $child = detail_info.$listDimensionsInUse.children('[rd-dimension="' + suggestedField.name + '"]');
								if (suggestedField.category !== context.selectedCategory) {
									$child.remove();
								} else if ($child.length === 0) {
									if (!self.report || _.findWhere(self.report.projectFields, {name: suggestedField.name})) {
										appendDimensionInUse(suggestedField.name);
									} else {
										appendDimensionAvail(suggestedField.name);
									}
								}
							});

							detail_info.$listDimensionsAvail
									.children()
									.off('mouseenter')
									.one('mouseenter',function(){
										$(this).addClass('sorting-initialize');
										detail_info.$listDimensionsAvail.sortable('refresh');
									});

							detail_info.$listDimensionsInUse
									.children()
									.off('mouseenter')
									.one('mouseenter',function(){
										$(this).addClass('sorting-initialize');
										detail_info.$listDimensionsInUse.sortable('refresh');
									});

							if (self.hNavigation) {
								self.hNavigation.enablePage(3);
							}
							//console.log("-END listDimensions - uid: " + uid);
							callbackDimensions();
							detail_info.loadingDimensions = false;
						}
					);
				});
			};

			const filter_type = detail_info.type;

			var on_changed_filter = function($listItem, $filter) {
				if (!self.renderComplete) {
					return;
				}

				const $filtersList = detail_info.$selectFilter.nextAll('ul');
				let filter_id = $filter.val();

				self.controller.loadFilter(filter_id, filter_type).done(function(filter)
				{
					var commitChange = function() {
						detail_info.selectedFilter = filter;
						// update list
						$filtersList.children('.optionSelected')
							.removeClass('optionSelected');

						const $item = $listItem || $filtersList.children('[filter-id="' + filter_id + '"]');
						$item.addClass('optionSelected');

						// check if this filter requires type
						if (_.findWhere(filter.mandatoryFields, { id: 'Type'} )) {
							detail_info.$selectInteractionType.parents('.row')
								.show()
								.removeAttr('ic-data-bind-disabled');
							detail_info.selectedInteractionType = detail_info.$selectInteractionType.val();
						} else {
							detail_info.$selectInteractionType.parents('.row')
								.hide()
								.attr('ic-data-bind-disabled', "true");
							detail_info.selectedInteractionType = undefined;
						}
						// and finally re-list available dimensions
						listDimensions();
					};

					// That kind of code which never makes it through code review (read it backwards)...
					let nonCompatibleDimensions = _.reject(detail_info.$listDimensionsInUse.children('[ic-role!="template"]'),
						_.compose(_.partial(dimensionAppliesToFilter, filter), function($element) {
							return self.controller.loadDimension($element.attr('rd-dimension'), {sync:true}) || {};
						}, $));

					if (nonCompatibleDimensions.length === 0) { // all good, go on
						commitChange();
					} else {
						const question = format("{0}<br><br>&nbsp;&nbsp;&nbsp;&nbsp;{1}",
							"<%=_T('This action is going to remove the projected fields you have already setup and are not compatible with the selected filter.')%>",
							"<%=_T('Continue?')%>"
						);
						helpers.askForConfirmation(question, "<%=_T('About to change selected filter')%>").done( function() {
							_.each(nonCompatibleDimensions, function(e) {
								$(e).remove();
							});
							commitChange();
						}).fail(function() {
							detail_info.$selectFilter.val(detail_info.selectedFilter ? detail_info.selectedFilter.id : "");
						});
					}
				});
			};

			detail_info.$selectFilter
				.off('change')
				.on('change', function(e, $listItem) {
					const $filter = $(this);
					on_changed_filter($listItem, $filter);
				}
			);

			let hnd_timeout_interaction_type = null;
			detail_info.$selectInteractionType
				.off('change')
				.on('change', function() {
					if (hnd_timeout_interaction_type) {
						window.clearTimeout(hnd_timeout_interaction_type);
						hnd_timeout_interaction_type = null;
					}
					const $this = $(this);
					hnd_timeout_interaction_type = window.setTimeout(function() {
						hnd_timeout_interaction_type = null;
						detail_info.selectedInteractionType = $this.val();
						detail_info.$selectFilter.trigger('change');
					}, 100);
				}
			);

			if (self.hasPermissionToSave()) {
				var on_selected_filter = function($filter) {
					if (!$filter.hasClass('optionSelected')) {
						let filter_id = $filter.attr('filter-id');
						detail_info.$selectFilter.val(filter_id).trigger('change', [$filter]);
					}
				};
				const $filters = detail_info.$selectFilter.nextAll('ul');
				$filters.on('click', 'li', function() {
					const $filter = $(this);
					on_selected_filter($filter);
				});
			}

			var listFilters = function()
			{
				// listDetailFilters o listLiveDetailFilters
				self.controller[detail_info.listFiltersFn]().done(function(detailFilters)
				{
					var displayFilters = function(candidateFilter, callbackFilter = $.noop)
					{
						if (self.hNavigation) {
							// Mientras se realiza la carga en el DOM se desactiva la solapa 2
							self.hNavigation.disablePage(2);
						}
						const $detail_filters = detail_info.$selectFilter.nextAll('ul');
						// var uid = inConcert.Utils.generateGuid();
						// console.log("+START displayFilters - uid: " + uid);

						helpers.populateListAsync(
							$detail_filters,
							detailFilters,

							// la funcion Iteradora
							function(filter) {
								if (_.some(helpers.listCustomScopes(), function(scope) {
									let integrationScope;
									switch(scope) {
										case "cci":
											integrationScope = "backend";
											break;
										case "batch_data":
											integrationScope = "batch_data";
											break;
										case "process_flow":
											integrationScope = "process_flow";
											break;
										default:
											integrationScope = "frontend";
											break;
									}
									const projectId = filter[scope + "Project"];
									return projectId && projectId !== context.integration[integrationScope].projectId;
								})) {
									return false;
								}
								return _.contains(allowedFilterScopes[context.selectedCategory], filter.scope);
							},

							// Los options
							{
								sort: true,
								maxAppendPerLoop: 10,
								pauseBetweenLoopMs: 100,
								waitingMsg: "<%=_T('Please wait, adding Filters rows, {0} of {1}.')%>",
								waitingFn: function(/*message*/) {
									// Aca se podria mostrar un waiting si fuese necesario
									//console.log(message + " uid: " + uid);
								},
								// Valida si la fila a agrega se puede agregar
								canAddFn: function(item) {
									const selector = "[filter-id='" + item.id + "']";
									const $currentItem = $detail_filters.find(selector);
									if (!$currentItem || $currentItem.length === 0) {
										// si NO existe, SI la agrega
										return true;
									}
									// si ya existe, NO la agrega
									return false;
								}
							},

							// Esta es la funcion que se invoca cuando la carga de la lista asincronica ha finalizado.
							function(/*itemCount*/) {
								if (candidateFilter) {
									detail_info.$selectInteractionType.parents('.row')
										.hide()
										.attr('ic-data-bind-disabled', 'true');
									detail_info.selectedInteractionType = undefined;

									detail_info.$selectFilter.val('');
									const $item = $detail_filters.children('[filter-id="' + candidateFilter + '"]');
									if ($item.length > 0) {
										detail_info.$selectFilter.val(candidateFilter)
											.trigger('change');
									}
								}
								if (self.hNavigation) {
									self.hNavigation.enablePage(2);
								}
								callbackFilter();
								//console.log("-END displayFilters - uid: " + uid);
							}
						);
					};

					if ( has_permission_to_save ) {
						detail_info.$listDimensionsAvail
								.sortable({
									cursor: 'move',
									connectWith: '#' + detail_info.$listDimensionsInUse.uniqueId().attr('id'),
									items: ".sorting-initialize",
									receive: function( event, ui ) {
										ui.item.children('[ic-inner-data-bind="alias"]').hide();
									}
								});

						detail_info.$listDimensionsInUse
								.sortable({
									cursor: 'move',
									connectWith: '#' + detail_info.$listDimensionsAvail.uniqueId().attr('id'),
									items: ".sorting-initialize",
									receive: function( event, ui ) {
										ui.item.children('[ic-inner-data-bind="alias"]').show();
									}
								});
					}
					else {
						detail_info.$listDimensionsAvail.find('input').prop( 'disabled', true );
						detail_info.$listDimensionsInUse.find('input').prop( 'disabled', true );
					}

					helpers.setupTextFiltering (
						detail_info.$listDimensionsAvail.parent(),
						'[ic-role="filter-by-content"]'
					);

					let updateWhenComplete = makeCompletionHandler(function() {
						context.changeHandlers[filter_type](true); // filter_type = detail o livedetail
					});
					context.changeHandlers[filter_type] = function(deferred) { // filter_type =  detail o livedetail
						if (!self.renderComplete) {
							return updateWhenComplete();
						}
						if (!deferred) {
							detail_info.$listDimensionsInUse.children('[ic-role!="template"]').remove();
						}
						displayFilters(detail_info.$selectFilter.val(), function() {
							listDimensions();
						});
					};

					context.checkHandlers[filter_type] = function() { // filter_type = detail o livedetail
						let nonCompatibleDimensions = _.reject(detail_info.$listDimensionsInUse.children('[ic-role!="template"]'),
							_.compose(_.partial(isDimensionCompatible, detail_info.selectedFilter), function($element) {
								return self.controller.loadDimension($element.attr('rd-dimension'), {sync:true}) || {};
							}, $));
						return {
							success: nonCompatibleDimensions.length === 0,
							cleanup: function() {
								_.each(nonCompatibleDimensions, function(e) {
									$(e).remove();
								});
							}
						};
					};

					displayFilters(undefined, function() {
						callbackRenderDetail(true);
					});
				}).fail(_.partial(callbackRenderDetail, false));
			};

			listFilters();
		} // renderBase (renderDetail & renderLiveDetail)

		function renderDetail (callbackRender) {
			// note: these fields are suggested but are not mandatory
			const suggestedFields = [
				{ name: "StartDate", category: "Interactions" },
				{ name: "EndDate",   category: "Interactions" },
				{ name: "From",      category: "States"       },
				{ name: "To",        category: "States"       }
			];
			renderDetailBase(callbackRender, "detail", suggestedFields);
		}

		function renderLiveDetail (callbackRender) {
			renderDetailBase(callbackRender, "livedetail");
		}

		function renderSummary (callback) {
			var metricsSelectorHandle;

			function isMetricCompatible(filter, metric) {
				if (metric.composite) {
					return _.every(metric.indicators, _.compose(
						_.partial(isMetricCompatible, filter),
						function(metricId) {
							return self.controller.loadMetric(metricId, {sync:true}) || {};
						})
					);
				}

				var frontendIntegration = context.integration.frontend;
				if ( metric.integrationType && (metric.integrationType !== frontendIntegration.scope || metric[frontendIntegration.scope + "Project"] !== frontendIntegration.projectId) ) {
					return false;
				}
				if (metric.cciProject && metric.cciProject !== context.integration.backend.projectId) {
					return false;
				}
				if (metric.batch_dataProject && metric.batch_dataProject !== context.integration.batch_data.projectId) {
					return false;
				}
				if (metric.process_flowProject && metric.process_flowProject !== context.integration.process_flow.projectId) {
					return false;
				}
				if (!filter) {
					return true;
				}
				if (!metric.metadata || !metric.metadata.groupFields) {
					return true;
				}
				var isCompatible = _.every(filter.columns, function(column) {
					if (column.id === "Slice") {
						return true;
					}
					var hasToInclude = false;
					for (var k in metric.metadata.groupFields) {
						if (metric.metadata.groupFields.hasOwnProperty(k)) {
							var currentField = metric.metadata.groupFields[k];
							if (column.name === currentField) { //|| column.id === currentField) {
								hasToInclude = true;
								break;
							}
						}
					}
					return hasToInclude;
				});
				return isCompatible;
			}

			var $summaryFiltersList = context.selectSummaryFilter.nextAll('ul');
			context.selectSummaryFilter.on('change', function(event, $listItem, ignore) {
				if (!self.renderComplete) {
					return;
				}

				var filter_id = $(this).val();

				self.controller.loadFilter(filter_id, "summary").done(function(filter)
				{
					var commitChange = function() {
						context.selectedSummaryFilter = filter;
						// update the list
						$summaryFiltersList.children('.optionSelected')
							.removeClass('optionSelected');

						var $item = $listItem || $summaryFiltersList.children('[filter-id="' + filter_id + '"]');
						$item.addClass('optionSelected');
						if (!$listItem) {
							$summaryFiltersList.scrollTo($item);
						}
						// and re-list available dimensions
						metricsSelectorHandle.reset();
					};

					if (ignore) {
						commitChange();
						return;
					}

					// That kind of code which never makes it through code review (read it backwards)...
					var nonCompatibleMetrics = _.reject(context.listMetricsInUse.children('[ic-role!="template"]'),
						_.compose(_.partial(isMetricCompatible, filter), function($element) {
							return self.controller.loadMetric($element.attr('rd-metric'), {sync:true});
						}, $));

					if (nonCompatibleMetrics.length === 0) { // all good, go on
						commitChange();
					} else {
						var question = format("{0}<br><br>&nbsp;&nbsp;&nbsp;&nbsp;{1}",
							"<%=_T('This action is going to remove the metrics you have already setup and are not compatible with the selected filter.')%>",
							"<%=_T('Continue?')%>"
						);
						helpers.askForConfirmation(question, "<%=_T('About to change selected filter')%>").done( function() {
							_.each(nonCompatibleMetrics, function(e) {
								$(e).remove();
							});
							commitChange();
						}).fail(function() {
							context.selectSummaryFilter.val(context.selectedSummaryFilter && context.selectedSummaryFilter.id || '');
						});
					}
				});
			});

			var has_permission_to_save = self.hasPermissionToSave();

			if ( has_permission_to_save ) {
				$summaryFiltersList.on('click', 'li', function() {
					var $this = $(this);
					if (!$this.hasClass('optionSelected')) {
						context.selectSummaryFilter
							.val($this.attr('filter-id'))
							.trigger('change', [ $this ]);
					}
				});
			}

			function listSummaryFilters()
			{
				var getColumsAtIndex = _.compose(_.compact, _.uniq, function(index, filterList) {
					return _.map(filterList, function(filter) {
						return filter.columns[index] && filter.columns[index].displayName;
					});
				});

				var filterByColumnAtIndex = function(index, datasource) {
					return (function() {
						var columnName = $(this).val();
						return columnName === "all" ? datasource : _.select(datasource, function(filter) {
							return filter.columns && filter.columns[index] && filter.columns[index].displayName === columnName;
						});
					});
				};

				var populateSummaryFilter = function(datasource) {
					var currentFilter = context.selectSummaryFilter.val();
					context.selectSummaryFilter.val('');
					helpers.populateList($summaryFiltersList, datasource, {sort: true});
					var $item = $summaryFiltersList.children('[filter-id="' + currentFilter + '"]');
					if ($item.length > 0) {
						context.selectSummaryFilter.val(currentFilter)
							.trigger('change', undefined, true);
					}
				};
				return self.controller.listSummaryFilters().then(function(summaryFilters)
				{
					summaryFilters = _.filter(summaryFilters, function(filter) {
						return _.every(helpers.listCustomScopes(), function(scope) {
							var projectId = filter[scope + "Project"];
							let integrationScope;
							switch(scope) {
								case "cci":
									integrationScope = "backend";
									break;
								case "batch_data":
									integrationScope = "batch_data";
									break;
								case "process_flow":
									integrationScope = "process_flow";
									break;
								default:
									integrationScope = "frontend";
									break;
							}
							return !projectId || projectId === context.integration[integrationScope].projectId;
						});
					});
					context.summaryHelpers.selectFirstColumn
						.off('change')
						.on('change', _.compose(
							_.partial(helpers.populateList, context.summaryHelpers.selectSecondColumn),
							_.partial(getColumsAtIndex, 1),
							function(datasource) {
								context.summaryHelpers.selectSecondColumn
									.off('change')
									.on('change', _.compose(
										populateSummaryFilter,
										filterByColumnAtIndex(1, datasource)
									));
								return datasource;
							},
							filterByColumnAtIndex(0, summaryFilters)
						));
					helpers.populateList(context.summaryHelpers.selectFirstColumn, getColumsAtIndex(0, summaryFilters));
				});
			}

			// used by the iterator below
			function makeMinMax(isMin, metric, $item) {
				var text = (isMin && "(Min) " || "(Max) ") + (metric.name || metric.id);
				$item.children('span')
					.text(text);
				$item.children('[ic-inner-data-bind="alias"]')
					.val(text);
				$item.children('[ic-inner-data-bind="min"]')
					.prop('checked', isMin);
				return $item;
			}

			listSummaryFilters().done(function() {
				metricsSelectorHandle = helpers.setupMetricsSelector({
					container : $('#rd_report_metric_selector'),
					controller: self.controller,
					iterator  : function(metric) {
						if (!isMetricCompatible(context.selectedSummaryFilter, metric)) {
							return false;
						}

						var $children = context.listMetricsInUse.children('[rd-metric="' + metric.id + '"]');
						if (metric.extras && metric.extras.MinMaxIndicator) {
							if ($children.length > 1) {
								return false;
							}

							if ($children.length === 0) {
								return function($item) {
									makeMinMax(false, metric, makeMinMax(true, metric, $item).clone())
										.insertAfter($item);
								};
							} else if ($children.find('[ic-inner-data-bind="min"]').prop('checked')) {
								return _.partial(makeMinMax, false, metric);
							} else {
								return _.partial(makeMinMax, true, metric);
							}
						} else {
							return $children.length === 0;
						}
					},
					callback  : function(success, errorThrown) {
						if (!success) {
							return callback(false, errorThrown);
						}

						if ( has_permission_to_save ) {
							context.listMetricsAvail.sortable({
								cursor: "move",
								connectWith: '#' + context.listMetricsInUse.uniqueId().attr('id'),
								items: ".sorting-initialize",
								receive: function( event, ui ) {
									ui.item.children('[ic-inner-data-bind="alias"]').hide();
								}
							});
							context.listMetricsInUse.sortable({
								cursor: "move",
								connectWith: '#' + context.listMetricsAvail.uniqueId().attr('id'),
								items: ".sorting-initialize",
								receive: function( event, ui ) {
									ui.item.children('[ic-inner-data-bind="alias"]').show();
								}
							});
						}
						else {
							context.listMetricsAvail.find('input').prop( "disabled", true );
							context.listMetricsInUse.find('input').prop( "disabled", true );
						}

						var updateWhenComplete = makeCompletionHandler(function() {
							context.changeHandlers.summary(true);
						});
						context.changeHandlers.summary = function(deferred) {
							if (!self.renderComplete) {
								return updateWhenComplete();
							}
							if (!deferred) {
								context.listMetricsInUse.children('[ic-role!="template"]').remove();
							}
							listSummaryFilters();
							metricsSelectorHandle.reset();
						};

						context.checkHandlers.summary = function() {
							var nonCompatibleMetrics = []; // TODO

							return {
								success: nonCompatibleMetrics.length === 0,
								cleanup: function() {
									_.each(nonCompatibleMetrics, function(e) {
										$(e).remove();
									});
								}
							};
						};

						callback(true);
					},
					update : function() {
						context.listMetricsAvail
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).addClass("sorting-initialize");
								context.listMetricsAvail.sortable('refresh');
							});

						context.listMetricsInUse
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).addClass("sorting-initialize");
								context.listMetricsInUse.sortable('refresh');
							});
					}
				});
			}).fail(_.partial(callback, false)); // call to listSummaryFilters()
		} // renderSummary

		function updateHandlers() {
			_.each(context.checkHandlers, function(fn) {
				var res = fn();
				if (res && res.success === false) {
					res.cleanup(); // TODO: Ask the user (and revert changes)
				}
			});
			_.each(context.changeHandlers, _.defer);
		}

		const $report_type = context.$selectType;  // combo "Tipo"
		const $category = context.$selectCategory; // combo "Categoria"
		const $configure_integration = self.container.find('[ic-role="configure-integration"]');

		// Combo "Tipo" (Detalle, Detalle de Activas, Resumen)
		context.selectedType = $report_type.val();
		$report_type.off('change')
			.on('change', function() {
				context.selectedType = $(this).val();
				self.switchUseCaseByAttr('rd-for-report-type', context.selectedType);
				if (context.selectedType === 'livedetail') {
					$category.val('Interactions').trigger('change');
					$configure_integration.hide();
					$category.prop("disabled", true);
					self.onFieldDisabledChange(true, $category);
				}
				else {
					$category.trigger('change');
					$category.prop("disabled", false);
					self.onFieldDisabledChange(false, $category);
				}
			}
		);

		// Combo "Categoria" (Conversaciones, Estados)
		context.selectedCategory = $category.val();
		$category.off('change')
			.on('change', function() {
				var $this = $(this);
				context.selectedCategory = $this.val();
				self.switchUseCaseByAttr('rd-for-report-category', context.selectedCategory);
				updateHandlers();
			}
		);

		//
		helpers.configureIntegration({
			container : $configure_integration,
			controller: self.controller,
			tweetyApi : inConcert.app.getTweetyAPI(),
			beforeChange: function(category, newScope, newValue, callback) {
				callback();
			},
			afterChange: function(category, newScope, newValue) {
				var disableSelect = true;

				switch(newScope) {
					case 'webchat':
						context.detail.$selectInteractionType.val('CHAT');
						break;
					case 'webcall':
						context.detail.$selectInteractionType.val('CALL');
						break;
					case 'contactform':
					case 'api':
						context.detail.$selectInteractionType.val('WEBCONTACT');
						break;
					default:
						disableSelect = false;
						context.detail.$selectInteractionType.val('');
				}
				// some scope are valid only for one type of interaction
				context.detail.$selectInteractionType.prop('disabled', disableSelect).trigger('change');
				// TODO: Do it in the helper
				self.switchUseCaseByAttr('rd-for-integration-type', newScope, self.container.find('[rd-integration-category="'+ category +'"]'));
				// TODO: remove this code (find something more elegant)
				context.integration[category].scope = newScope;
				context.integration[category].projectId = newValue;

				// #7101 IE: Este codigo esta de mas, se deja por las dudas.
				// El mismo se ejecuta todas las veces cuando se mueve en un item del combo, y renderiza nuevamente todos los elementos de las listas de las paginas 2 y 3.
				//
				updateHandlers();
			}
		}).done(function(errorThrown) {
			if (errorThrown) {
				self.emit("error", format("<%=_T('Failed to load Integration Projects: {0}')%>", _T(errorThrown)));
			}

			// Render pdf configurator redardless of the report type or category or 
			// if it is a new report or edited.
			self.pdfConfigurator = new inConcert.reports.PDFConfigurator("#form_edit_report_pdf_configurator_container");
			self.once("exit", function() {
				self.pdfConfigurator.destroy();
			});

			if (self.report === null) {
				var vTitle="<%=_T('New Report')%>";
				self.setTitle(vTitle);

				renderSummary(function(success, errorThrown) {
					if (success) {
						renderDetail(function(success, errorThrown) {
							if (success) {
								renderLiveDetail(callback);
							} else {
								callback(false, errorThrown);
							}
						});
					} else {
						callback(false, errorThrown);
					}
				});

				_.defer(_.bind($.fn.trigger, $report_type, 'change'));
			} else {
				var title = self.hasPermissionToSave() ?
					"<%=_T('Edit Report')%> <span>\"{0}\"</span>" : 
					"<%=_T('View Report')%> <span>\"{0}\"</span>";

				self.setTitle(format( title, self.report.name), '', self.report.description);

				var trySetPDFConfiguratorSettings = function () {
					var pdfConfiguratorSettings = window.__pdfConfiguratorSettings;
					var extraExportArgs_pdf = window.__extraExportArgs_pdf;
					if (!pdfConfiguratorSettings || !extraExportArgs_pdf) {
						return;
					}

					pdfConfiguratorSettings = $.extend(true, {}, pdfConfiguratorSettings);
					extraExportArgs_pdf = $.extend(true, {}, extraExportArgs_pdf);
					var new_settings = {};

					$.extend(true, new_settings, pdfConfiguratorSettings);

					// To ensure correct types, because the problem of Lua to determine 
					// if a table must be json-encoded as Array or Object.
					new_settings.report = castToArray(new_settings.report);
					new_settings.report[0].value = castToArray(new_settings.report[0].value);
					new_settings.report[1].value = castToArray(new_settings.report[1].value);

					// remove json-object marks needed by Lua
					unmarkAsJSONObject(new_settings.columnTypes);
					unmarkAsJSONObject(new_settings.columnWidths);
					unmarkAsJSONObject(new_settings.hiddenColumnWidths);
					unmarkAsJSONObject(new_settings.stickyColumns);

					new_settings.page.backgroundColor = extraExportArgs_pdf.page.backgroundColor;
					new_settings.margins = extraExportArgs_pdf.margins;

					new_settings.font = extraExportArgs_pdf.font;
					new_settings.font.family = castToArray(new_settings.font.family);

					new_settings.headerFontColor = extraExportArgs_pdf.headerFontColor;
					new_settings.lineSpacing = extraExportArgs_pdf.lineSpacing;
					new_settings.columnSpacing = extraExportArgs_pdf.columnSpacing;
					new_settings.oddRow = extraExportArgs_pdf.oddRow;

					// replicate fake rows
					for (var count = 0; count < 10 - 1 /* already one fake row came from data base */; count++) {
						new_settings.report.push({type: "row", value: new_settings.report[1].value.map(function (fakeCell) {
							return fakeCell;
						})});
					}

					// clear all current settings and set the new one
					self.pdfConfigurator.setSettings(new_settings, ["__{[(ROOT)]}__"]);

					delete window.__pdfConfiguratorSettings;
					delete window.__extraExportArgs_pdf;
				};

				var onRenderComplete = function(success, errorThrown) {
					if (success) {
						helpers.displayDataModel(self.container, self.report, {
							disabledFields : [ "type", "category" ],
							onFieldDisabled: _.bind(self.onFieldDisabledChange, self, true),
							listIterator   : function(propertyName, $field, item) {
								switch(propertyName) {
									case "indicators":
										return item.isMinMax && function($element) {
											$element.find('[ic-inner-data-bind="min"]')
												.prop('checked', item.min);
										} || undefined;
								}
							},
							customLoaders: {
								"pdfConfiguratorSettings": function ($field, value) {
									window.__pdfConfiguratorSettings = value;
									trySetPDFConfiguratorSettings();
								},
								"extraExportArgs.pdf": function ($field, value) {
									window.__extraExportArgs_pdf = value;
									trySetPDFConfiguratorSettings();
								}
							}
						});
						// Pequeno timeout para que la carga de los controles del frm se visualicen correctamente.
						window.setTimeout(function() {
							_.defer(_.bind($.fn.trigger, $report_type, 'change'));
						}, 500);
					}
					callback(success, errorThrown);
				};

				if (self.report.type === "summary") { // report type cannot be changed so...
					renderSummary(onRenderComplete);
				}
				else if (self.report.type === "detail") {
					renderDetail(onRenderComplete);
				}
				else if (self.report.type === "livedetail") {
					renderLiveDetail(onRenderComplete);
				}
				else {
					throw("Unrecognized - self.report.type: " + self.report.type);
				}
			}
		});
	};

	FormEditReport.prototype.onShow = function() {
		const self = this;

		this.pdfConfigurator.show();

		this.initNavigation({
			allVisited: (this.report !== null),
			onEnterPage: function(pageNumber) {
			// scrollTo does not work if the list is hidden
				if (pageNumber === 2) {
					let $list;
					const selected_type = self.context.selectedType;
					if (selected_type === 'detail') {
						$list = self.context.detail.$selectFilter.nextAll('ul');
					}
					else if (selected_type === 'livedetail') {
						$list = self.context.livedetail.$selectFilter.nextAll('ul');
					}
					else if (selected_type === 'summary') {
						$list = self.context.selectSummaryFilter.nextAll('ul');
					}
					else {
						throw("FormEditReport.prototype.onShow - Unrecognized selected_type: " + selected_type);
					}

					let $item = $list.children('.optionSelected');
					if ($item.length > 0) {
						$list.scrollTo($item);
					}
				}
				else if (pageNumber === 4) {
					self.__updatePDFConfigurator();
					self.pdfConfigurator.hackSetFixedColumns();  // temporary hack
				}
			}
		});

		this.emit("__ready");
	};

	FormEditReport.prototype.onConfirm = function(callback)
	{
		const self = this;

		var result = this.validate({
			iterator: _.bind(self.isFieldEnabled, self),
			customLoaders: {
				"pdfConfiguratorSettings": function (/*$field*/) {
					// Update pdf configurator before we get its settings because columns
					// may have changed after the user left out from page 4.
					self.__updatePDFConfigurator();
					let settings = self.pdfConfigurator.getSettings();

					// this info is already saved in `extraExportArgs.pdf'
					delete settings.page.backgroundColor;
					delete settings.margins;
					delete settings.font;
					delete settings.headerFontColor;
					delete settings.lineSpacing;
					delete settings.columnSpacing;
					delete settings.oddRow;

					// Remove fake rows (Xxxx) but first to allow recreate it later if 
					// editted.
					settings.report.splice(2, settings.report.length - 2);

					// help Lua to known these are Objects and not Arrays
					markAsJSONObject(settings.columnTypes);
					markAsJSONObject(settings.columnWidths);
					markAsJSONObject(settings.hiddenColumnWidths);
					markAsJSONObject(settings.stickyColumns);

				  return settings;
				},
				// extra arguments to be passed to the pdf exporter
				"extraExportArgs.pdf": function (/*$field*/) {
					// Hack: Before we get the fixed configuration of PDF we move to page
					// of pdf configurator and then return to current page. This is needed
					// to avoid 0-widths in columns with automatic width because PDF 
					// preview may be to smaller.
					const $index = self.container.find('[ic-role="navigation-index"]');
					const $currentIndex = $index.children(".navigationIndexCurrent");

					$index.children('[ic-navigation-page="4"]:visible').trigger("click");
					const fixedConfig = self.pdfConfigurator.getFixedConfig();
					$currentIndex.trigger("click");

					return fixedConfig;
				}
			}
		});
		if (!result.success) {
			return callback(false, result.errorMessage);
		}

		let report = $.extend({
			id: this.report && this.report.id || undefined
		}, result.fields);

		if (report.type === "summary") {
			if (report.indicators.length === 0) {
				return callback(false, "<%=_T('You must add at least one metric')%>");
			}

			let indicators = report.indicators;
			report.indicators = [];
			report.columnDetails = {};

			let all_columns_valid = _.every(indicators, function(item) {
				if (item.alias === '') {
					return false;
				}
				let itemId = item.id + (item.min === true ?  "|min" : '');
				report.indicators.push(itemId);
				report.columnDetails[itemId] = {
					columnName : item.alias,
					min        : item.min === true && item.min || undefined
				};
				return true;
			});
			if (!all_columns_valid) {
				return callback(false, "<%=_T('Columns names are not configured properly')%>");
			}
		}
		else if (report.type === "detail" || report.type === "livedetail") {
			let fields_length = report.projectFields.length;
			if (fields_length === 0) {
				return callback(false, "<%=_T('You must add at least one field')%>");
			}
			for (let i = 0; i < fields_length; i++) {
				let field = report.projectFields[i];
				if (field.alias) {
					field.columnName = field.alias;
				}
			}
		}
		else {
			throw("FormEditReport.prototype.onConfirm - Unrecognized report.type: " + report.type);
		}

		this.controller.saveReport(report).done(function()
		{
			callback(true, "<%=_T('Report saved successfully')%>");
		}).fail(function(errorThrown) {
			let message_error;
			switch ( errorThrown ) {
				case "forbidden":
					message_error = "<%=_T('You dont have permission to save')%>";
				break;
				default:
					message_error = errorThrown;
			}
			callback(false, format("<%=_T('Failed to save report: {0}')%>", message_error));
		});

		window._newreport = report;
	};

	FormEditReport.prototype.__updatePDFConfigurator = function () {
		const self = this;
		var dataModel = helpers.readDataModel(this.container, _.bind(this.isFieldEnabled, this));
		var report = [{type: "headers", value: []}];
		var headers = report[0].value;
		var columnTypes = {};
		var columnWidths = {};
		var hiddenColumnWidths = {};
		var stickyColumns = {};

		if (dataModel.fields.type === "detail" || dataModel.fields.type === "livedetail") {
			if (dataModel.fields.projectFields) {
				var projectFields = dataModel.fields.projectFields.map(function (projectField, index) {
					var dimension = self.controller.loadDimension(projectField.dimension, {sync: true});
					columnTypes[index] = dimension.type;

					// Default column width of at least `dimension.maxLength' characters if provided,
					// else minimum width without overflow plus free space (e.g expanded).
					columnWidths[index] = dimension.maxLength === undefined ? undefined : dimension.maxLength + "c+";
					return projectField.alias || dimension.displayName || dimension.name || "";
				});

				Array.prototype.push.apply(headers, projectFields);
			}
		}
		else if (dataModel.fields.type === "summary") {
			if (dataModel.fields.cut) {
				var cut = this.controller.getCut(dataModel.fields.cut);
				if (cut) {
					var columns = cut.columns.map(function (column, index) {
						columnTypes[index] = column.type;
						// Default column width of at least `column.maxLength' characters if provided,
						// else minimum width without overflow plus free space (e.g expanded).
						columnWidths[index] = column.maxLength === undefined ? undefined : column.maxLength + "c+";
						return column.label || column.name ||  "";
					});
					Array.prototype.push.apply(headers, columns);
					$.each(columns, function (index) { stickyColumns[index] = true; });
				}
			}

			if (dataModel.fields.indicators) {
				var indicators = dataModel.fields.indicators.map(function (indicator, index) {
					var metric = self.controller.loadMetric(indicator.id, {sync: true});
					columnTypes[headers.length + index] = metric.format && metric.format.type;
					// Default column width of at least `metric.maxLength' characters if provided,
						// else minimum width without overflow plus free space (e.g expanded).
						columnWidths[headers.length + index] = metric.maxLength === undefined ? undefined : metric.maxLength + "c+";
					return indicator.alias || "";
				});
				Array.prototype.push.apply(headers, indicators);
			}
		}

		var loopFn = function (_, index) {
			if (typeof(columnWidths[index]) === "string") {
				var fakeCell = Array(parseInt(columnWidths[index], 10) + 1).join("x");
				return fakeCell.charAt(0).toUpperCase() + fakeCell.slice(1);
			} else {
				return "Xx";
			}
		};
		// fake rows based in `column.maxLength'
		for (var count = 0; count < 10; count++) {
			report.push({type: "row",
				value: headers.map(loopFn)
			});
		}

		var oldSettings = this.pdfConfigurator.getSettings();
		var oldHeaders = oldSettings.report.length > 0 ? oldSettings.report[0].value : [];
		var oldColumnWidths = oldSettings.columnWidths;
		var oldHiddenColumnWidths = oldSettings.hiddenColumnWidths;

		$.each(headers, function (index, header) {
			var oldIndex = oldHeaders.indexOf(header);
			if (oldIndex !== -1) {
				// if column already exist then keep its values into new settings
				columnWidths[index] = oldColumnWidths[oldIndex];
				hiddenColumnWidths[index] = oldHiddenColumnWidths[oldIndex];
			} else {
				// HACK for make non-first added columns to NOT expand only if its 
				// widths are strings; that is to say for example: "6c+".
				if (oldHeaders.length > 0 && typeof(columnWidths[index]) === "string") {
					columnWidths[index] = columnWidths[index].slice(0, -1);  // remove "+"
				}
			}
		});

		this.pdfConfigurator.setSettings(
			{
				report: report, 
				columnTypes: columnTypes,
				columnWidths: columnWidths,
				hiddenColumnWidths: hiddenColumnWidths,
				stickyColumns: stickyColumns
			}, 
			[
				"report", 
				"columnTypes",
				"columnWidths", 
				"hiddenColumnWidths", 
				"stickyColumns"
			]
		);
	};

	/**
	 * castToArray
	 *
	 * Helper function to deal with the problem of Lua about determining if a 
	 * table must be json-encoded as an Array or as an Object.
	 */
	var castToArray = function (obj) {
		if (Object.prototype.toString.call(obj) === "[object Array]") {
			return obj;
		} else {
			var array = [];
			var index = 1;  // Lua indexes starts from 1
			var value;
			while (true) {
				value = obj[index];
				
				if (value === undefined) {
					break;
				}
				
				array.push(value);
				index ++;
			}
			
			return array;
		}
	};

	/**
	 * markAsJSONObject
	 *
	 * Helper function to deal with the problem of Lua about determining if a 
	 * table must be json-encoded as an Array or as an Object.
	 */
	var markAsJSONObject = function (obj) {
		if (Object.prototype.toString.call(obj) === "[object Object]") {
			obj.__IT_IS_A_MARK_TO_HELP_LUA_TO_JSON_ENCODE_THIS_OBJECT_AS_IS_AND_NOT_AS_AN_ARRAY__ = true;
		}
	};

	/**
	 * unmarkAsJSONObject
	 *
	 * Helper function.
	 *
	 * See `markAsJSONObject'
	 */
	var unmarkAsJSONObject = function (obj) {
		if (Object.prototype.toString.call(obj) === "[object Object]") {
			delete obj.__IT_IS_A_MARK_TO_HELP_LUA_TO_JSON_ENCODE_THIS_OBJECT_AS_IS_AND_NOT_AS_AN_ARRAY__;
		}
	};

	/**
	 * best$Extend
	 *
	 * It is a helper function.
	 */
	var best$Extend = function (target, source, fieldsToEmpty) {
		fieldsToEmpty = fieldsToEmpty || [];
		
		if (fieldsToEmpty.indexOf("__{[(ROOT)]}__") !== -1) {
			best$Extend({root: target}, {}, ["root"]);
		} else {
			$.each(fieldsToEmpty, function (_, fieldPath) {
				var targetValue = target;
				$.each(fieldPath.split("."), function (_, field) {
					try {
						targetValue = targetValue[field];
					} catch (err) {
						targetValue = undefined;
						return false;  // break
					}
				});
				if (Object.prototype.toString.call(targetValue) === "[object Object]") {
					for (var field in targetValue) {
						if (targetValue.hasOwnProperty(field)) {
							delete targetValue[field];
						}
					}
				} else if (Object.prototype.toString.call(targetValue) === "[object Array]") {
					targetValue.length = 0;
				}
			});
		}
		$.extend(true, target, source);
		return target;
	};

	/*
	  class PDFConfigurator
	
  settings = {
  	report: [
			{type: "headers", value: ["Campaign", "Agent", "Start Date"]},
			{type: "row", value: ["Support", "Pepe", "2016-03-16 12:35:20"]},
			...
			{type: "row", value: ["Sale", "Juan", "2016-03-18 09:40:23"]},
		],
    margins: {left: 5, top: 5, right: 5, bottom: 5},
    page: {
      size: PDFConfigurator.PAGE_SIZE.US_LEGAL,
      orientation: PDFConfigurator.PAGE_ORIENTATION.LANDSCAPE,
      backgroundColor: {red: 255, green: 255, blue: 255},
      responsivenessPolicy: PDFConfigurator.PAGE_RESPONSIVENESS_POLICY.EXPAND_TO_CONTAINER_WIDTH,
    },
    font: {
      family: ["Dejavu Sans Mono", "Courier", "monospace"],
      encoding: "StandardEncoding",
      size: 13,
      color: {red: 0, green: 0, blue: 0},
    },
    lineSpacing: 5,
    columnSpacing: 5,
    columnTypes: {
    	0: "time",
			1: "date",
			2: "string",
			3: "number",
			4: "boolean",
			5: "enum",
			6: "percentage",
			7: "decimal",
			8: "default",  // are treated like "number"
			9: undefined|null,  // are treated like "default"
    },
    columnWidths: {
      0: 20,                 // First column have fixed width.
      2: -1|null|undefined,  // Third column have minimun width without overflow plus free space (e.g expanded).
      5: false,              // Sixth column is hidden.
      7: "6c",               // Seventh column have a six-characters width.
      9: "6c+",              // Tenth column have a width of at least six-characters plus free space (e.g expanded).
    },
    hiddenColumnWidths: {
    	2: 20|"6c"|"6c+"|-1|undefined|null,  // Width of third column for when it is showed again.
    },
    stickyColumns: {
    	0: true,  // First and third columns
    	2: true,  // are sticky, that is, can not be hidden.
    },
    headerFontColor: {red: 150, green: 197, blue: 221},
    oddRow: {
    	backgroundColor: {red: 235, green: 235, blue: 235},
    	highlight: true,
    },
  }
  
  NOTE: `margins.*', `font.size', `*Spacing', fixed-`columnWidths', fixed-`hiddenColumnWidths' are in millimeters.
	*/

	function PDFConfigurator(parent, settings, fieldsToEmpty) {
		this.__parent = $(parent);
		this.__settings = $.extend(true, {}, PDFConfigurator.DEFAULT_SETTINGS);
		// cached computed page width needed to determine the zoom
		this.__computed_page_width = undefined;

		this.__parent.append($(PDFConfigurator.__HTML));
		this.setSettings(settings, fieldsToEmpty);

		// set events

		var self = this;
		
		$("#reports_pdf_popup_page_size").change(function () {
		  self.setSettings({page: {size: $("#reports_pdf_popup_page_size").val()}});
	  });
		$("#reports_pdf_popup_page_orientation").change(function () {
			self.setSettings({page: {orientation: $("#reports_pdf_popup_page_orientation").val()}});
		});
		$("#reports_pdf_popup_font_size").change(function () {
			self.setSettings({font: {size: Number($("#reports_pdf_popup_font_size").val())}});
		});
		$(window).on("resize.PDFConfigurator0__updatePage", function () {
			if (self.get$Node().is(":visible")) {
				self.__updatePage();
			}
		});
	}

	PDFConfigurator.PAGE_SIZE = {
		US_LETTER: "US_LETTER",
		US_LEGAL: "US_LEGAL",
		A4: "A4",
		A5: "A5",
	};

	PDFConfigurator.PAGE_ORIENTATION = {
		PORTRAIT: "PORTRAIT",
		LANDSCAPE: "LANDSCAPE",
	};

	PDFConfigurator.PAGES_DIMENSIONS = {};
	PDFConfigurator.PAGES_DIMENSIONS[PDFConfigurator.PAGE_SIZE.US_LETTER] = {WIDTH: 215.9, HEIGHT: 297.4};
	PDFConfigurator.PAGES_DIMENSIONS[PDFConfigurator.PAGE_SIZE.US_LEGAL]  = {WIDTH: 215.9, HEIGHT: 355.6};
	PDFConfigurator.PAGES_DIMENSIONS[PDFConfigurator.PAGE_SIZE.A4]        = {WIDTH: 210.0, HEIGHT: 297.0};
	PDFConfigurator.PAGES_DIMENSIONS[PDFConfigurator.PAGE_SIZE.A5]        = {WIDTH: 248.0, HEIGHT: 210.0};

	PDFConfigurator.PAGE_RESPONSIVENESS_POLICY = {
		EXPAND_TO_CONTAINER_WIDTH: "EXPAND_TO_CONTAINER_WIDTH",
		FIT_INTO_CONTAINER: "FIT_INTO_CONTAINER",
	};

	PDFConfigurator.DEFAULT_SETTINGS = {
		report: [],
		page: {
			size: PDFConfigurator.PAGE_SIZE.US_LEGAL,
			orientation: PDFConfigurator.PAGE_ORIENTATION.LANDSCAPE,
			backgroundColor: {red: 255, green: 255, blue: 255},
			responsivenessPolicy: PDFConfigurator.PAGE_RESPONSIVENESS_POLICY.EXPAND_TO_CONTAINER_WIDTH,
		},
		margins: {left: 7.0, top: 7.0, right: 7.0, bottom: 7.0},
		font: {
			family: ["Dejavu Sans Mono", "Courier", "monospace"],
			size: 3.0,
			color: {red: 0, green: 0, blue: 0},
		},
		headerFontColor: {red: 0, green: 0, blue: 0},
		lineSpacing: 1.5,
		columnSpacing: 3.0,
		columnTypes: {},
		columnWidths: {},
		hiddenColumnWidths: {},
		stickyColumns: {},
		oddRow: {
			backgroundColor: {red: 235, green: 235, blue: 235},  // lightgray
			highlight: false,
		},
	};

	PDFConfigurator.__HTML = "".concat(

		'<div id="reports_pdf_popup" class="row no-lastrow" style="display: none;">',
		'	<div id="reports_pdf_popup_page_container" class="col-8">',
		'		<div class="card">',		
		'			<div class="card-header">',
		'				<h5><%=_T("Pre-Visualization")%></h5>',
		'			</div>',
		'			<div class="card-block">',
		'				<div id="reports_pdf_popup_page" class="form-group">',
		'					<div id="reports_pdf_popup_page_content">',
		'						<!-- <div style="position: relative; width: 20px; height: 20px; background-color: blue; float: left; clear: both;"></div> -->',

		'						<div id="reports_pdf_popup_page_table"></div>',

		'						<!-- <div style="position: relative; width: 20px; height: 20px; background-color: blue; float: left; clear: both;"></div> -->',
		'					</div>',
		'				</div>',
		'			</div>',
		'		</div>',

		'	</div>',

		'	<div id="reports_pdf_popup_options_panel_container" class="col-4">',
		'		<div class="card">',
		'			<div class="card-header">',
		'				<h5><%=_T("Style")%></h5>',
		'			</div>',
		'			<div class="card-block">',
		'				<div class="form-group">',
		'					<label><%=_T("Page size")%><%=_T(":")%></label>',
		'		  			  <select id="reports_pdf_popup_page_size" class="form-control">',
		'			  	        <option value="US_LETTER"><%=_T("US Letter")%></option>',
		'					    <option value="US_LEGAL"><%=_T("US Legal")%></option>',
		'					    <option value="A4"><%=_T("A4")%></option>',
		'			    		<option value="A5"><%=_T("A5")%></option>',
		'				 	   </select>',
		'				</div>',

		'				<div class="form-group">',
		'			    	<label><%=_T("Page orientation")%><%=_T(":")%></label>',
		'		  			  <select id="reports_pdf_popup_page_orientation" class="form-control">',
		'				 		 <option value="PORTRAIT"><%=_T("Portrait")%></option>',
		'						  <option value="LANDSCAPE"><%=_T("Landscape")%></option>',
		'			  		</select>',
		'				</div>',

		'				<div class="form-group">',
		'				  <label><%=_T("Font size")%><%=_T(":")%></label>',
		'		 		   <select id="reports_pdf_popup_font_size" class="form-control">',
		'				 		 <option value="3">3</option>',
		'				 		 <option value="4">4</option>',
		'				 		 <option value="5">5</option>',
		'				  		<option value="6">6</option>',
		'			 		 </select>',
		'				</div>',
		//'				<hr/>',
		'				<div id="reports_pdf_popup_options_panel_option_columns_selection" class="form-group">',
		'					<div class="card-header">',
		'						<h5><%=_T("Columns")%></h5>',
		'					</div>',
		'					<div class="form-group">',
		'						<label class="list—label-title"><%=_T("Data group")%></label>',
		'						<i class="far fa-question-circle" title="Grupo de datos seleccionados en el paso #2."></i>',
		'					</div>',
		'		    		<div id="reports_pdf_popup_options_panel_columns_selector"></div>',
		'				</div>',
		'			</div>',
		'		</div>',
		'	</div>',
		'</div>'

	);

	/**
	 * get$Node
	 */
	PDFConfigurator.prototype.get$Node = function() {
		return $("#reports_pdf_popup");
	};

	/**
	 * show
	 */
	PDFConfigurator.prototype.show = function() {
		this.get$Node().show();
	};

	/**
	 * hide
	 */
	PDFConfigurator.prototype.hide = function() {
		this.get$Node().hide();
	};

	/**
	 * isDisplayed
	 */
	PDFConfigurator.prototype.isDisplayed = function() {
		return this.get$Node().css("display") !== "none";
	};

	/**
	 * destroy
	 */
	PDFConfigurator.prototype.destroy = function() {
		$(window).off("resize.PDFConfigurator0__updatePage");
		this.get$Node().remove();
	};

	/**
	 * getFixedConfig
	 *
	 * Compute the current configuration (e.g determine fixed dimensions of page and columns)
	 */
	PDFConfigurator.prototype.getFixedConfig = function () {
		var fixed_config = $.extend(true, {}, this.__settings);
		
		delete fixed_config.report;
		delete fixed_config.columnTypes;
		delete fixed_config.hiddenColumnWidths;
		delete fixed_config.stickyColumns;
		delete fixed_config.page.size;
		delete fixed_config.page.orientation;
		delete fixed_config.page.responsivenessPolicy;

		var page_width = PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].WIDTH;
		var page_height = PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].HEIGHT;

		if (this.__settings.page.orientation === PDFConfigurator.PAGE_ORIENTATION.LANDSCAPE) {
			var tmp = page_width;
			page_width = page_height;
			page_height = tmp;
		}

		fixed_config.page.width = page_width;
		fixed_config.page.height = page_height;

		fixed_config.columnWidths = [];  // Actually it is expected to be an Array and not an Object.

		if (this.__settings.report.length > 0) {
			for (var column_index = 0; column_index < this.__settings.report[0].value.length; column_index ++) {
				if (typeof(this.__settings.columnWidths[column_index]) === "string" ||
						this.__settings.columnWidths[column_index] === -1 ||
						this.__settings.columnWidths[column_index] === undefined ||
						this.__settings.columnWidths[column_index] === null) {
					var column_cell = $(".reportsPDFPopupPageTableRow:first-child .reportsPDFPopupPageTableCell:nth-child(" + (column_index + 1) + ")");
					fixed_config.columnWidths.push(this.__pixelsToMillimeters(column_cell.width()));
				} else {
					fixed_config.columnWidths.push(this.__settings.columnWidths[column_index]);
				}
			}
		}

		/*
		// HACK: this make the last column to have automatic width and then allow 
		// another columns to be dinamically added, for example, when scheduling a 
		// report and the user checks `Export conversations'.
		for (var column_index = fixed_config.columnWidths.length - 1; column_index >= 0; --column_index) {
			if (fixed_config.columnWidths[column_index] !== false) {
				fixed_config.columnWidths[column_index] = -1;
				break;
			}
		}
		*/

		return fixed_config;
	};

	/**
	 * getSettings
	 */
	PDFConfigurator.prototype.getSettings = function() {
		return $.extend(true, {}, this.__settings);
	};

	/**
	 * setSettings
	 */
	PDFConfigurator.prototype.setSettings = function(settings, fieldsToEmpty) {
		best$Extend(this.__settings, settings, fieldsToEmpty);
		this.__updateUI();
	};

	/**
	 * __updateUI
	 */
	PDFConfigurator.prototype.__updateUI = function () {
		this.__reloadReport();
		this.__updateConfigPanel();
		this.__updatePage();
	};

	/**
	 * __reloadReport
	 */
	PDFConfigurator.prototype.__reloadReport = function() {
	  this.__unloadReport();
	  this.__loadReport();
	};

	/**
	 * __loadReport
	 */
	PDFConfigurator.prototype.__loadReport = function() {
		var self = this;

		// fill table preview

		var row_is_odd = true;
		var makeRow = function (row) {
			var headers_class;
			var odd_class;
			var html_column_grip;
			if (row.type === "headers") {
				headers_class = 'reportsPDFPopupPageTableHeaders';
				odd_class = '';
				row_is_odd = true;
				html_column_grip = '<div class="reportsPDFPopupPageTableColumnGrip"></div>';
			} else {
				headers_class = '';
				odd_class = row_is_odd ? 'reportsPDFPopupPageTableOddRow' : '';
				row_is_odd = !row_is_odd;
				html_column_grip = '';
			}

			var html_row = '<div class="reportsPDFPopupPageTableRow ' + headers_class + ' ' + odd_class + '">';

			for (var index = 0; index < row.value.length; index++) {
				html_row += '<div class="reportsPDFPopupPageTableCell"><span>' + inConcert.Utils.escapeAttrib(row.value[index]) + '</span>' + html_column_grip + '</div>';
			}

			html_row += '</div>';
			return $(html_row);
		};

		var $table = $("#reports_pdf_popup_page_table");
		var index;
		for (index = 0; index < this.__settings.report.length; index++) {
			$table.append(makeRow(this.__settings.report[index]));
		}

		// fill columns selector

		if (this.__settings.report.length > 0) {
			var first_row = this.__settings.report[0].value;
			var html_selector_column_metric = "";
			var html_selector_column_data = "";
			var metrics_title_step3_tooltip ="<%=_T([[Metrics selected in step #3.]])%>"

			for (index = 0; index < first_row.length; index++) {

				const html = '<div class="form-check w-100 d-block"><label class="form-check-label form-check-label-title"><input type="checkbox"/>' + inConcert.Utils.escapeAttrib(first_row[index]) + '</label></div>';
				if (this.__settings.stickyColumns[index] === true) {
					html_selector_column_data += html;
				}
				else{
					html_selector_column_metric += html;
				} 	
			}

			if (html_selector_column_metric !== "") {
				html_selector_column_metric = 
				`<div class="form-group mt-2">
					<label class="list—label-title"><%=_T("Metrics")%></label>
					<i class="far fa-question-circle" title="${metrics_title_step3_tooltip}"></i>
				</div>
				${html_selector_column_metric}`;
			}
			if (html_selector_column_data !== "") {
				html_selector_column_data = 
				`${html_selector_column_data}`;
			}
			$("#reports_pdf_popup_options_panel_columns_selector").append($(html_selector_column_data + html_selector_column_metric));
		}

		// set events of table preview
		$(".reportsPDFPopupPageTableColumnGrip").on("mousedown", function (event) {
			$("body").on("mouseup.PDFConfigurator0__endResizeColumn", function () {
				$("body").off("mouseup.PDFConfigurator0__endResizeColumn");
				$("body").off("mousemove.PDFConfigurator0__resizeColumn");
				$("body").css("cursor", "initial");
			});
			$("body").css("cursor", "ew-resize");
			var initial_x = event.pageX;
			var $current_cell = $(this).parent();
			var current_column_index = $current_cell.index();
			var initial_column_width = $current_cell.width();
			var $page_content = $("#reports_pdf_popup_page_content");
			var margin_rigth = $page_content.offset().left + $page_content.width();
			var max_column_width = Math.max(2 /* make grip always visible */, margin_rigth - $current_cell.offset().left);

			$("body").on("mousemove.PDFConfigurator0__resizeColumn", function (event) {
				var delta_x = event.pageX - initial_x;
				var final_column_width = Math.min(max_column_width, 
				Math.max(2 /* make grip always visible */, initial_column_width + delta_x));
				var new_settings = {columnWidths: {}};
				new_settings.columnWidths[current_column_index] = self.__pixelsToMillimeters(final_column_width);
				self.setSettings(new_settings);
			});
		});

		// set events of columns selector

		$('#reports_pdf_popup_options_panel_columns_selector input[type="checkbox"]').on("change", function () {
			var column_index = $(this).parent().parent().index();
			var new_settings = {columnWidths: {}, hiddenColumnWidths: {}};
			if ($(this).prop("checked")) {
				// restore previous width

				/* `undefined' values do not overwrite settings, see `$.extend' */
				new_settings.columnWidths[column_index] = self.__settings.hiddenColumnWidths[column_index] === undefined ? -1 : self.__settings.hiddenColumnWidths[column_index];
			} else {
				// save current width and hide column
				/* `undefined' values do not overwrite settings, see `$.extend' */
				new_settings.hiddenColumnWidths[column_index] = self.__settings.columnWidths[column_index] === undefined ? -1 : self.__settings.columnWidths[column_index];
				new_settings.columnWidths[column_index] = false;
			}
			self.setSettings(new_settings);
		});
	};

	/**
	 * __unloadReport
	 */
	PDFConfigurator.prototype.__unloadReport = function() {
		// clear table preview
		$("#reports_pdf_popup_page_table").empty();
		// clear columns selector
		$("#reports_pdf_popup_options_panel_columns_selector").empty();
	};

	/**
	 * __updatePage
	 */
	PDFConfigurator.prototype.__updatePage = function () {
		// Update page dimensions. Must be the first thing to do because subsequent 
		// computations depends on this dimensions values to correctly determine the 
		// zoom and conversions beween pixels and millimeters.
		
		var self = this;
		var page_ratio;
 
	  if (this.__settings.page.orientation === PDFConfigurator.PAGE_ORIENTATION.PORTRAIT) {
			page_ratio = PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].HEIGHT / 
						PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].WIDTH;
		} else {
			page_ratio = PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].WIDTH / 
					PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].HEIGHT;
		}

		// set the page responsiveness policy before we get the page container 
		// dimensions
		this.get$Node().attr("rd-page-responsiveness-policy", this.__settings.page.responsivenessPolicy);

		var $page_container = $("#reports_pdf_popup_page_container");
		var page_container_width = Math.max(0, $page_container.width() - 2 /* page borders */);
		var page_container_height = Math.max(0, $page_container.height() - 2 /* page borders */);

		var computed_page_width;
		var computed_page_height;

		if (this.__settings.page.responsivenessPolicy === PDFConfigurator.PAGE_RESPONSIVENESS_POLICY.FIT_INTO_CONTAINER && 
				page_container_height / page_ratio <= page_container_width) {  // full page height fits in container width
			computed_page_height = page_container_height;
			computed_page_width = page_container_height/ page_ratio;
		} else {
			// full page width fits in container height or make it to follow the policy 
			// PDFConfigurator.PAGE_RESPONSIVENESS_POLICY.EXPAND_TO_CONTAINER_WIDTH
			computed_page_width = page_container_width;
			computed_page_height = page_container_width * page_ratio;
		}

		// cache the computed page width needed later to determine the zoom
		this.__computed_page_width = computed_page_width;

		$("#reports_pdf_popup_page").width(computed_page_width).height(computed_page_height);
		
		// update page background color
		
		var page_background_color = "rgb(" + this.__settings.page.backgroundColor.red + ", " + 
											this.__settings.page.backgroundColor.green + ", " + 
											this.__settings.page.backgroundColor.blue + ")";
		$("#reports_pdf_popup_page").css("background-color", page_background_color);
		// update odd row background color
		var $odd_rows = $(".reportsPDFPopupPageTableOddRow");
		var odd_row_background_color;

		if (this.__settings.oddRow.highlight) {
			odd_row_background_color = "rgb(" + this.__settings.oddRow.backgroundColor.red + ", " + 
											this.__settings.oddRow.backgroundColor.green + ", " + 
											this.__settings.oddRow.backgroundColor.blue + ")";
		} else {
			odd_row_background_color = "initial";
		}

		$odd_rows.css("background-color", odd_row_background_color);

		// update font family, size, color

		var font_family = this.__settings.font.family.
		map(function (name) { return '"' + name + '"'; }).join(", ");

		var font_size = this.__millimetersToPixels(this.__settings.font.size) + "px";
		var font_color = "rgb(" + this.__settings.font.color.red + ", " + 
									this.__settings.font.color.green + ", " + 
									this.__settings.font.color.blue + ")";

		$("#reports_pdf_popup_page").css({
			"font-family": font_family,
			"font-size": font_size,
			"color": font_color});

		// update header font color
		var header_font_color = "rgb(" + this.__settings.headerFontColor.red + ", " + 
										this.__settings.headerFontColor.green + ", " + 
										this.__settings.headerFontColor.blue + ")";

		$(".reportsPDFPopupPageTableRow.reportsPDFPopupPageTableHeaders").css("color", header_font_color);

		// update column spacing
		// FIXME: in chromium-browser version 47.0.2526.106-0ubuntu0.14.04.1.1107 this selector is not working (BUG with :visible when some ancestor is inlined ???)
		$(".reportsPDFPopupPageTableCell:visible:not(:last-child)").css("margin-right", this.__millimetersToPixels(this.__settings.columnSpacing) + "px");

		// update line spacing
		$(".reportsPDFPopupPageTableRow:not(:last-child)").css("margin-bottom", this.__millimetersToPixels(this.__settings.lineSpacing) + "px");

		// update margins
		var margin_left = this.__millimetersToPixels(this.__settings.margins.left) + "px";
		var margin_top = this.__millimetersToPixels(this.__settings.margins.top) + "px";
		var margin_right = this.__millimetersToPixels(this.__settings.margins.right) + "px";
		var margin_bottom = this.__millimetersToPixels(this.__settings.margins.bottom) + "px";

		$("#reports_pdf_popup_page_content").css({
			"left": margin_left,
			"top": margin_top,
			"width": "calc(100% - " + margin_left + " - " + margin_right + ")",
			"height": "calc(100% - " + margin_top + " - " + margin_bottom + ")"});

		if (this.__settings.report.length === 0) {
			return;
		}

		// update column widths, visivility and alignment
		var first_row = this.__settings.report[0];
		var $non_fixed_columns_to_expand = [];

		for (var column_index = 0; column_index < first_row.value.length; column_index ++) {
			var column_type = this.__settings.columnTypes[column_index];
			column_type = column_type === undefined || column_type === null ? column_type : column_type.toLowerCase();
			var column_width = this.__settings.columnWidths[column_index];
			var $column_cells = $(".reportsPDFPopupPageTableCell:nth-child(" + (column_index + 1) + ")");

			if (column_type === "percentage" || column_type === "decimal" || 
					column_type === "number" || column_type === "default" || 
					column_type === undefined || column_type === null) {
				
				$column_cells.css("text-align", "right");
			} else {
				$column_cells.css("text-align", "left");
			}

			if (column_width === false) {  // Hide column.
				$column_cells.hide();
			} else {
				$column_cells.show();
				if (typeof(column_width) === "string" || column_width === -1 || column_width === undefined || column_width === null) {  // Non-fixed width.
					var max_cell_width = 0;
					$column_cells.width("initial");

					$column_cells.each(function (row_index) {
						var cell_width;
						var cell_text = self.__settings.report[row_index].value[column_index];

						if ($(this).parent().hasClass("reportsPDFPopupPageTableHeaders")) { // header cell
							var cell_words = cell_text.split(/\s+/);

							for (var word_index = 0; word_index < cell_words.length; word_index ++) {
								$(this).children("span").text(inConcert.Utils.escapeAttrib(cell_words[word_index]));
								cell_width = $(this).width();
								if (cell_width > max_cell_width) {
									max_cell_width = cell_width;
								}
							}

							$(this).children("span").text(inConcert.Utils.escapeAttrib(cell_text));
						} else {  // row cell
							if (typeof(column_width) === "string") {  // [minimun] column width in characters
								var chars_length = parseInt(column_width, 10);
								chars_length = isNaN(chars_length) ? cell_text.length : chars_length;
								var truncated_cell_text = cell_text.substring(0, chars_length);
								$(this).children("span").text(inConcert.Utils.escapeAttrib(truncated_cell_text));
								cell_width = $(this).width();
								$(this).children("span").text(inConcert.Utils.escapeAttrib(cell_text));
							} else { // minimum column width without overflow
								cell_width = $(this).width();
							}
							
							if (cell_width > max_cell_width) {
								max_cell_width = cell_width;
							}
						}
					});

					$column_cells.width(max_cell_width);

					// Remember this column to be expanded except for when its width is a
					// string like "6c" (that do not ends with "+").
					if (typeof(column_width) !== "string" || column_width.match(/\+$/)) {
						$non_fixed_columns_to_expand.push($column_cells);
					}
				} else {  // Fixed width.
					$column_cells.width(this.__millimetersToPixels(column_width));
				}
			}
		}

		// Expand remembered columns if there is available space
		if ($non_fixed_columns_to_expand.length > 0) {
			var available_space = $("#reports_pdf_popup_page_content").width() - $("#reports_pdf_popup_page_table").width();
			
			if (available_space > 0) {
				var extra_space = available_space / $non_fixed_columns_to_expand.length;
				
				for (var index = 0; index < $non_fixed_columns_to_expand.length; index ++) {
					$non_fixed_columns_to_expand[index].width($non_fixed_columns_to_expand[index].width() + extra_space);
				}
			}
		}

		// fix header cells height

		$(".reportsPDFPopupPageTableHeaders .reportsPDFPopupPageTableCell").height("100%");

		$(".reportsPDFPopupPageTableHeaders").each(function () {
			var cell_heights = [];

			$(this).children().each(function () {
				cell_heights.push($(this).height());
			});

			$(this).children().height(Math.max.apply(null, cell_heights));
		});
	};

	/**
	 * __updateConfigPanel
	 */
	PDFConfigurator.prototype.__updateConfigPanel = function () {
		$("#reports_pdf_popup_page_size").val(this.__settings.page.size);
		$("#reports_pdf_popup_page_orientation").val(this.__settings.page.orientation);
		$("#reports_pdf_popup_font_size").val(String(this.__settings.font.size));

		var self = this;

		$('#reports_pdf_popup_options_panel_columns_selector input[type="checkbox"]').each(function (index) {
			$(this).prop("checked", self.__settings.columnWidths[index] !== false);
			$(this).prop("disabled", self.__settings.stickyColumns[index] === true);
		});
	};

	/**
	 * __millimetersToPixels
	 */
	PDFConfigurator.prototype.__millimetersToPixels = function(millimeters) {
		return millimeters * this.__getCurrentZoom();
	};

	/**
	 * __pixelsToMillimeters
	 */
	PDFConfigurator.prototype.__pixelsToMillimeters = function(pixels) {
		var zoom = this.__getCurrentZoom();

		return zoom && pixels / zoom;
	};
	
	/**
	 * __getCurrentZoom
	 */
	PDFConfigurator.prototype.__getCurrentZoom = function() {
		/* A T E N T I O N:
		   Do not get comptuted page width of the form `$(<page_selector>).width()'
		   because it is rounded and losses precision that is reflected later in preview.
		*/
		return this.__computed_page_width / 
			(this.__settings.page.orientation === PDFConfigurator.PAGE_ORIENTATION.PORTRAIT ? 
			PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].WIDTH : 
			PDFConfigurator.PAGES_DIMENSIONS[this.__settings.page.size].HEIGHT);
	};

	/**
	 * hackSetFixedColumns
	 *
	 * A T T E N T I O N: this code is a hack for set columns fixed once it is 
	 * rendered. This limitation is intentionally to """SIMPLIFY""" the user 
	 * interface. This should be removed in the future when someone decides to 
	 * expose all the configurations that it class supports in a compact and 
	 * usable way.
	 */
	PDFConfigurator.prototype.hackSetFixedColumns = function() {
		if (this.__settings.report.length === 0) {
			return;
		}

		var first_row = this.__settings.report[0];

		for (var column_index = 0; column_index < first_row.value.length; column_index ++) {
			var column_width = this.__settings.columnWidths[column_index];

			if (typeof(column_width) === "string" || 
					column_width === -1 || 
					column_width === undefined || 
					column_width === null) {
				var $column_cell = $(".reportsPDFPopupPageTableRow:first-child .reportsPDFPopupPageTableCell:nth-child(" + (column_index + 1) + ")");
				this.__settings.columnWidths[column_index] = this.__pixelsToMillimeters($column_cell.width());
			}
		}
	};

	if (typeof inConcert.reports.PDFConfigurator !== "function") {
		inConcert.reports.PDFConfigurator = PDFConfigurator;
	}
}());
