require "lunit"

local console = require "lunit-console"

--
-- Setup test environment. Install mock libraries for LuaConfigurationControl and Cassandra
--
local function prepare_environment()
	local cachandra = require "cachandra"

	--
	-- Stub out LuaConfigurationControl
	--
	package.preload["LuaConfigurationControl"] = function()
		return {
			Client = function()
				return {
					TriggerNodeReady = function()
						print("Client:TriggerNodeReady")
					end
				}
			end
		}
	end

	--
	-- Improve performance
	--
	package.preload["lua_cassandra"] = function()
		return cachandra
	end

	package.preload["apps.agent"] = function()
		return {}
	end

	dofile("bootstrap.lua")
end


prepare_environment()

require "test"
require "testMultipart"

lunit.setrunner(console)

local stats = lunit.run()

if stats.failed > 0 or stats.errors > 0 then
	return 1
end
