module(..., package.seeall, lunit.testcase)

local cjson = require "cjson"

local mock = require "wsapi.mock"
local repository = require "apps.repository"

local app = mock.make_handler(repository.run)



function test_upload_multipart()
	local f = assert(io.open("test.jpg", "rb"))
	local data = f:read("*a")
	f:close()

	local multipart_data = {
		[[------WebKitFormBoundaryYAgTHXAShFjyQ4IF]],
		[[Content-Disposition: form-data; name="Filedata"; filename="rociobb1.JPG"]],
		[[Content-Type: image/jpeg]],
		"",
		data,
		[[------WebKitFormBoundaryYAgTHXAShFjyQ4IF--]],
		""
	}
	multipart_data = table.concat(multipart_data, "\r\n")
	local response, request = app:post("/add/?id=0BEAAF2F28D7CE6770D10F246D0AF307&section=-1&format=JPG&contentType=application/octet-stream&fileName=rociobb1_jpg", 
		multipart_data, { ["Content-Type"] = [[multipart/form-data; boundary=----WebKitFormBoundaryYAgTHXAShFjyQ4IF]],
		["CONTENT_TYPE"] = [[multipart/form-data; boundary=----WebKitFormBoundaryYAgTHXAShFjyQ4IF]]
	})
	
	assert_equal(200, response.code)

	response, request = app:get("/get/", { id = "0BEAAF2F28D7CE6770D10F246D0AF307", section="-1", format="JPG", contentType="application/octet-stream", part=1})
	assert_equal(200, response.code)
	assert_equal(data, response.body)
end

---
-- Uploads an incomplete file (it is missing the multipart boundary)
--
function test_upload_multipart_incomplete()
	local f = assert(io.open("test.jpg", "rb"))
	local data = f:read("*a")
	f:close()

	local multipart_data = {
		[[------WebKitFormBoundaryYAgTHXAShFjyQ4IF]],
		[[Content-Disposition: form-data; name="Filedata"; filename="rociobb1.JPG"]],
		[[Content-Type: image/jpeg]],
		"",
		data
	}
	multipart_data = table.concat(multipart_data, "\r\n")
	local response, request = app:post("/add/?id=0BEAAF2F28D7CE6770D10F246D0AF307&section=-1&format=JPG&contentType=application/octet-stream&fileName=rociobb1_jpg", 
		multipart_data, { ["Content-Type"] = [[multipart/form-data; boundary=----WebKitFormBoundaryYAgTHXAShFjyQ4IF]],
		["CONTENT_TYPE"] = [[multipart/form-data; boundary=----WebKitFormBoundaryYAgTHXAShFjyQ4IF]]
	})
	
	assert_equal(500, response.code)
	--print(cjson.encode(response))

	response, request = app:get("/get/", { id = "0BEAAF2F28D7CE6770D10F246D0AF307", section="-1", format="JPG", contentType="application/octet-stream", part=1})
	assert_equal(404, response.code)
	--print(cjson.encode(response))
end