require "orbit"
require "inconcert.orbit.application"
local WrapRedis = require "wrapredis"
require "md5"

local m_applicationName = "seeredis"
local m_appLocation = select(2, ...)
assert(m_appLocation, "should be running under inconcertWebHandler")

local redis
local redisPwd = ""

module(..., package.seeall, orbit.new, inconcert.orbit.application.Decorate)

local htmlRender = require"inconcert.render".new(m_applicationName)
htmlRender:AddCommonScript("/inconcert/apps/seeredis/scripts/jquery-1.6.2.min.js")

-- 
-- This function gets called when the web server initializes this application (once per VM)
function Initialize()
	LogInfo("Starting application %s", m_applicationName)
	redisPwd = inconcert.environment and inconcert.environment.apps and inconcert.environment.apps.agent and inconcert.environment.apps.agent.redis and inconcert.environment.apps.agent.redis.pwd or ""
	--redis = WrapRedis("127.0.0.1", 6377)
	--redis = WrapRedis("192.168.20.202", 6377)
	return true
end

local function serve_file(app_module)
	return function (web)
		local filename = string.sub(web.path_info, 2, #web.path_info)
		return _M:serve_static(web, m_appLocation.."/"..filename)
	end
end
_M:dispatch_get(serve_file(app_module), "/images/.+", "/styles/.+", "/scripts/.+")


local function get_server_and_port (web)
	local server = web.cookies.redis_server or "127.0.0.1"
	if not server then
		web.status = 302
		web.headers.Location = "/"
		return nil
	end

	local port = tonumber(web.cookies.redis_port) or 6377
	if not port then
		web.status = 302
		web.headers.Location = "/"
		return nil
	end

	return server, port
end

---
--
--
_M:dispatch_get(function(web, key)
	local server, port = get_server_and_port(web)
	if not server then
		return ""
	end
	
	local redis = WrapRedis(server, port, redisPwd)
	
	local key_type = redis:type(key)
	local args = { redis = redis, key = key, key_type = key_type, redis_server = server, redis_port = port }
	
	if key_type == "set" then
		args.values = redis:smembers(key)
		
	elseif key_type == "zset" then
		args.values = redis:zrange(key, 0, -1, "WITHSCORES")
	
	elseif key_type == "list" then
		args.values = redis:lrange(key, 0, -1)
		
	elseif key_type == "hash" then
		args.values = redis:hgetall(key)
		
	elseif key_type == "string" then
		args.values = { redis:get(key) }
	end
	
	return htmlRender:RenderPageLP("pages/inspect_"..key_type..".html", web, args)
end, "/inspect/(.+)")

--solo para strings
_M:dispatch_post(function(web, key)
	local newValue = web.input["post_data"]
	
	local server, port = get_server_and_port(web)
	if not server then
		return ""
	end
	
	local redis = WrapRedis(server, port, redisPwd)
	--check key type
	local key_type = redis:type(key)
	if (key_type == "string" ) then
		redis:set(key, newValue)
	end
	return ""
end, "/update/(.+)")

---
--
--
_M:dispatch_get(function(web)
	local ok, redis = pcall(WrapRedis, web.input.server, tonumber(web.input.port) or 6377, redisPwd)
	if not ok then
		web.status = 302
		web.headers["Cache-Control"] = "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
		web.headers.Location = web:link("/")
		web:delete_cookie("redis_server")
		web:delete_cookie("redis_port")
		return ""
	end
	
	web.status = 302
	web.headers["Cache-Control"] = "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
	web.headers.Location = web:link("/keys/")
	
	web:set_cookie("redis_server", {
		value = web.input.server, 
		path = "/",
		sameSite = "None",
		secure = true
	})
	web:set_cookie("redis_port", {
		value = web.input.port, 
		path = "/",
		sameSite = "None",
		secure = true
	})
	
	return ""
end, "/connect")

---
--
--
_M:dispatch_get(function(web, filter)
	
	local query = web.input.query
	
	local server, port = get_server_and_port(web)
	if not server then
		return ""
	end
	
	local redis = WrapRedis(server, port, redisPwd)

    local page = htmlRender:RenderPageLP("pages/keys.html", web, {
    					redis_server = server,
    					redis_port = port,
						redis = redis,
						query = query
						})
	local hash = md5.sumhexa(page .. (query or "") .. server .. port)
	
	if web.vars.HTTP_IF_NONE_MATCH == hash then
		web.status = 304
		return ""
	end
	web.headers["ETag"] = hash
	return page
end, "/keys/(.*)")

---
--
--
_M:dispatch_get(function(web)
	web.headers["Cache-Control"] = "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
	
	local server = web.cookies.redis_server or "127.0.0.1"
	local port = web.cookies.redis_port or 6377
	return htmlRender:RenderPageLP("pages/main.html", web, {
		redis_server = server,
		redis_port = port
	})
end, "/")
