---
-- Peque�o wrapper que maneja el caso de desconexion del redis
-- actualmente la libreria redis-lua no hace nada cuando se desconecta, lo cual es un poco molesto
-- con este wrapper se consigue automaticamente reconectar y reintentar el comando fallido
--
-- (c) 2011 Datagnasco
--

local Redis = require "redis"

function WrapRedis (ip, port, pwd)
    local rediscore = Redis.connect (ip, port, pwd)
    
    Redis.error = function (err)
        if err == "closed" or err == "Socket is not connected" then
            rediscore = Redis.connect (ip, port, pwd)
        end
        error (err)
    end
    
    local redis = {}
    setmetatable (redis, {__index = function (t, k)
        local f = function (self, ...)
            local params = {...}
            local ok, err, result
            local retries = 3
            
            while not ok and retries > 0 do
                ok, err = pcall (function ()
                    result = {rediscore[k](rediscore, unpack (params))}
                end)
                
                if ok then
                    return unpack (result)
                end
                retries = retries - 1
            end
            
            if not ok then
                error (err)
            end
        end
        rawset(t, k, f)
        return f
    end})
    
    return redis
end

return WrapRedis