/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	const format = inConcert.Utils.format;
	const templates = inConcert.templates.social_networks;

	function social_networks() {
		const self = this;
		this.categories = [
			new inConcert.snPostsList.total()
		];
		this.archive = new inConcert.snPostsList.archive();
		this.scheduled = new inConcert.snPostsList.scheduled();
		this.active = new inConcert.snPostsList.active();
		self.categories_dictionary = {};
		$.each(this.categories, function(_, category) {
			self.categories_dictionary[category.GetView()] = category;
		});
		this.currentCategory = null;
		this.currentStatsSlice = null;
		this.defaultStatsSlice = inConcert.snStatisticsSlices.last7days;
	}

	inConcert.SocialNetworksApp = social_networks;

	social_networks.prototype.can_show = function() {
		return inConcert.permissions.to_application("social_networks");
	};

	social_networks.prototype.getActiveCategorie = function() {
		return this.active;
	};

	social_networks.prototype.__selectCategory = function(category, selectedAccount, callback = $.noop) {
		if (this.currentCategory && this.currentCategory.IsProcessing()) {
			return;
		}
		this.currentCategory = category;
		const $category = $("#snPostsBatchTab").find(format("[category={0}]", this.getSelectedTabId()));
		$category
			.addClass("selected")
			.siblings()
			.removeClass("selected");
		category.Load(callback, selectedAccount);
		$("#snTwitterUniversalSearch").removeClass("selected");
		$("#snSearchContainer").hide();
		$("#snPostsContainer").show();
		$("#snpageHeader").show();
		$("#snpageHeaderTwSearch").hide();
	};

	social_networks.prototype.__initializeTabs = function () {
		const self = this;
		this.m_tabs_config = new inConcert.conf_form_tab({
			frm: this,
			container_id: "snPostsBatchTab",
			must_initialize: true,
			tabs_list: {
				active: {
					id: "active",
					name: "<%=_T('Active')%>",
					tab_container_id: "snPostsBatchTabActive",
					selected: true,
					on_show: function () {
						self.__selectCategory(self.active);
					}
				},
				scheduled: {
					id: "scheduled",
					name: "<%=_T('Scheduled')%>",
					tab_container_id: "snPostsBatchTabScheduled",
					selected: false,
					on_show: function () {
						self.__selectCategory(self.scheduled);
					}
				},
				archive: {
					id: "archive",
					name: "<%=_T('Archive')%>",
					tab_container_id: "snPostsBatchTabArchive",
					selected: false,
					on_show: function () {
						self.__selectCategory(self.archive);
					}
				}
			}
		});
		this.m_tabs_config.show_tabs_container();
	};

	social_networks.prototype.getSelectedTabId = function() {
		return this.m_tabs_config.get_selected_tab_id().toUpperCase();
	};

	social_networks.prototype.getSelectedCategory = function() {
		let selectedCategory;
		let selectedTabId = this.getSelectedTabId();
		switch (selectedTabId) {
			case "ACTIVE":
				selectedCategory = this.active;
			break;
			case "ARCHIVE":
				selectedCategory = this.archive;
				break;
			case "SCHEDULED":
				selectedCategory = this.scheduled;
				break;
			default:
				throw("social_networks.prototype.getSelectedCategory - unrecognized category: " + selectedTabId);
		}
		return selectedCategory;
	};

	social_networks.prototype.MoveToCategory = function(categoryId, account, callback = $.noop) {
		const self = this;
		categoryId = (categoryId || "").toUpperCase();
		let category = this.categories_dictionary[categoryId];
		if (category === this.currentCategory) {
			return callback();
		}
		let cateogryInstance;
		let categoryTabId = categoryId.toLowerCase();
		switch (categoryId) {
			case "ACTIVE":
				cateogryInstance = this.active;
			break;
			case "ARCHIVE":
				cateogryInstance = this.archive;
				break;
			case "SCHEDULED":
				cateogryInstance = this.scheduled;
				break;
			default:
				throw(`social_networks.prototype.MoveToCategory - unrecognized category '${categoryId}'`);
		}
		this.__selectCategory(cateogryInstance, account, function() {
			self.m_tabs_config.switch_tab(categoryTabId);
			callback();
		});
	};

	social_networks.prototype.__enterToCategory = function($target) {
		this.__changeTitleByEmail($target);
		if (!$target.attr("category_selector")) {
			return;
		}
		let categorieId = "";
		switch (this.getSelectedTabId()) {
			case "ACTIVE":
				categorieId = this.active;
			break;
			case "ARCHIVE":
				categorieId = this.archive;
				break;
			case "SCHEDULED":
				categorieId = this.scheduled;
				break;
		}
		this.__selectCategory(categorieId);
	};

	social_networks.prototype.__loadCategory = function(category) {
		const self = this;
		let $category = $(Mustache.render(templates.menu_category_item, {
			id         : category.GetView(),
			name       : category.GetName(),
			description: category.GetDescription(),
			className  : category.GetClassName()
		}));
		$category.click(function(e) {
			//check if event is fot this or for an account
			const $target = $(e.target);
			self.__enterToCategory($target);
		});
		return $category;
	};

	social_networks.prototype.__$loadCategories = function() {
		const self = this;
		let $categories = [];
		let $firstCategory;
		$.each(self.categories, function(_, category) {
			if (category.HasPermission()) {
				const $category = self.__loadCategory(category);
				if (!$firstCategory) {
					$firstCategory = $category;
				}
				$categories.push($category);
			}
		});
		$("#snMenu")
			.html("")
			.append($categories);
		return $firstCategory;
	};

	social_networks.prototype.__loadTwitterSearch = function() {
		const self = this;
		this.twitterSearch = new inConcert.TwitterSearch();
		if (!self.twitterSearch.HasPermission()) {
			$("#snTwitterUniversalSearch").remove();
		}
		$("#snTwitterUniversalSearch")
			.off("click")
			.on("click", function(e) {
				//check if event is fot this or for an account
				const $target = $(e.target);
				if (!$target.attr("category_selector")) {
					return;
				}
				if (self.currentCategory && self.currentCategory.IsProcessing()) {
					return;
				}
				$("#snMenu").find("[category]").removeClass("selected");
				$("#snTwitterUniversalSearch").addClass("selected");
				$("#snMain").removeClass("splitedScenario");
				$("#snSearchContainer").show();
				$("#snPostsContainer").hide();
				$("#snStats").hide();
				$("#snpageHeader").hide();
				$("#snpageHeaderTwSearch").show();
				$("#snMenu").hide();
				self.twitterSearch.Show();
				self.currentCategory = self.twitterSearch;
			});
	};
	
	social_networks.prototype.UpdatePosts = function() {
		if (this.currentCategory && this.currentCategory.UpdatePosts) {
			this.currentCategory.UpdatePosts();
		}
	};

	social_networks.prototype.RefreshList = function(postId) {
		if (this.currentCategory && this.currentCategory.Search) {
			this.currentCategory.selectedPost = postId;
			this.currentCategory.Search();
		}
	};

	social_networks.prototype.__changeTitleByEmail = function (target) {
		let titleName = target[0].innerText;
		const listItems = $("#snPostsAccountList");
		const itemSelected = listItems.find(".selected").attr( "account" );
		let positionCharacter,typeIconTitle;
		let $snPostsTitleIcon = $("#snPostsTitleIcon");
		if (itemSelected) {
			 positionCharacter = itemSelected.indexOf("_");
			 typeIconTitle = itemSelected.substring(0, positionCharacter);
		}else{
			titleName = '';
		}
		let titleElementIcon = $("#snIconPageTitle");
		let titleElement = $("#snPageTitle");
		
		if (titleName.indexOf("\n") !== -1 ||
			titleName === "" ||
		 	titleName === `<%=_T('Total')%>`) {
				titleElementIcon.removeClass('ic-channels-solid-twitter')
					.removeClass('ic-channels-solid-youtube')
					.removeClass('ic-channels-solid-facebook')
					.removeClass('ic-channels-solid-Instagram')
					.addClass("fa-layer-group");
				titleElement.text(`<%=_T('Total')%>`);
				listItems.find(".selected").removeClass('selected');
				$snPostsTitleIcon.addClass('selected');
		}
		else{
			switch (typeIconTitle) {
				case 'FACEBOOK':
					titleElementIcon.removeClass('ic-channels-solid-twitter')
							.removeClass('ic-channels-solid-youtube')
							.removeClass('ic-channels-solid-Instagram')
							.removeClass('fa-layer-group')
							.addClass('ic-channels-solid-facebook');
					titleElement.text(titleName);
					$snPostsTitleIcon.removeClass('selected');
					break;
				case 'YOUTUBE':
					titleElementIcon.removeClass('ic-channels-solid-twitter')
							.removeClass('ic-channels-solid-facebook')
							.removeClass('ic-channels-solid-Instagram')
							.removeClass('fa-layer-group')
							.addClass('ic-channels-solid-youtube');
					titleElement.text(titleName);
					$snPostsTitleIcon.removeClass('selected');
					break;
				case 'INSTAGRAM':
					titleElementIcon.removeClass('ic-channels-solid-twitter')
							.removeClass('ic-channels-solid-facebook')
							.removeClass('ic-channels-solid-youtube')
							.removeClass('fa-layer-group')
							.addClass('ic-channels-solid-Instagram');
					titleElement.text(titleName);
					$snPostsTitleIcon.removeClass('selected');
					break;
				case 'TWITTER':
					titleElementIcon.removeClass('ic-channels-solid-Instagram')
							.removeClass('ic-channels-solid-youtube')
							.removeClass('fa-layer-group')
							.addClass('ic-channels-solid-facebook')
							.addClass('ic-channels-solid-twitter');
					titleElement.text(titleName);
					$snPostsTitleIcon.removeClass('selected');
					break;
			}
		}
	};

	social_networks.prototype.SelectStatsSlice = function(slice) {
		var self = this;
		if (self.currentCategory && self.currentCategory.SelectStatsSlice) {
			self.currentCategory.SelectStatsSlice(slice, function() {
				self.currentStatsSlice = slice;
			});
		}
	};

	social_networks.prototype.maxAllowedRows = function() {
		return 500;
	};

	social_networks.prototype.__load_text_max_allowed_rows = function() {
		const replaceText = this.maxAllowedRows();
		const $divTextInfo = $("#snPostsInfo");
		let newText = $divTextInfo.text();
		newText = format(newText, replaceText);
		$divTextInfo.text(newText);
	};

	social_networks.prototype.GetStatsSlice = function() {
		return this.currentStatsSlice || this.defaultStatsSlice;
	};

	social_networks.prototype.render = function(callback = $.noop) {
		const self = this;
		function show() {
			self.$dashboard.show();
			callback();
		}
		if (this.$dashboard) {
			return show();
		}
		this.load_inner_page("/inconcert/apps/social_networks/pages/main.<%=_L%>.html#grunt-cache-bust", function(content, error) {
			if (!content) {
				inConcert.app.getMainWindow().showBannerMessage({
					message : format("<%=_T('Rendering social networks dashboard failed with error {0}')%>", error),
					messageType : "Error",
					timeout : 5000
				});
				callback();
				return;
			}
			inConcert.app.getSNPostsManager().LoadConfiguration(function() {
				self.$dashboard = $('#social_networks_dashboard').html(content);
				const $firstCategory = self.__$loadCategories();
				self.__loadTwitterSearch();
				self.__initializeTabs();
				self.__selectCategory(self.categories[0]);
				self.__load_text_max_allowed_rows();
				if ($firstCategory) {
					self.__enterToCategory($firstCategory);
				}
				show();
			});
		});
	};

	social_networks.prototype.load_inner_page = function(page_url, callback) {
		inConcert.invoke({
			url        : page_url,
			type       : "GET",
			contentType: "text/html",
			success: function(response) {
				callback(response);
			},
			error: function(xhr, error) {
				callback(null, error.result);
			}
		});
	};
}());