/*jshint quotmark:false*/
/*global $, inConcert, _, Mustache, EmojiPicker*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.snPostsCreate) {
	inConcert.snPostsCreate = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.social_networks;

	function CreatePopup (account)
	{
		this.account = account;
	}
	
	if (typeof inConcert.snPostsCreate.base !== "function") {
		inConcert.snPostsCreate.base = CreatePopup;
	}
	
	CreatePopup.prototype.HasScheduledDate = function() {
		return false;
	};
	
	CreatePopup.prototype.HasBackdatedTime = function() {
		return false;
	};
	
	CreatePopup.prototype.__getScheduleDateLimits = function() {
		return null;
	};
	
	CreatePopup.prototype.__getBackdatedTimeLimits = function() {
		return null;
	};
	
	CreatePopup.prototype.__getTextLimit = function() {
		return null;
	};
	
	CreatePopup.prototype.__getAvailableContentTypes = function() {
		return null;
	};
	
	CreatePopup.prototype.__getMaxFileSizeMB = function() {
		return 0;
	};
	
	CreatePopup.prototype.__getClassName = function() {
		return "";
	};
	
	CreatePopup.prototype.__isScheduled = function(data) {
		return false;
	};
	
	CreatePopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	
	CreatePopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#snCreatePopup").remove();
				callback();
			});
		}
		else {
			$("#snCreatePopup").remove();
			callback();
		}
	};
	
	CreatePopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.create_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.__makeEmojiPicker();
		self.__onTextChange();
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "snCreatePopup",
			eventName     : "snCreatePopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#snCreatePopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	CreatePopup.prototype.__makeDateLimit = function(definition, now) {
		if (!definition) {
			return null;
		}
		var value = (definition.up? 1 : -1) * definition.value;
		var date = new Date(now);
		date.setMilliseconds(0);
		date.setSeconds(0);
		switch(definition.unity) {
			case "minutes":
				date.setMinutes(date.getMinutes() + value);
				break;
			case "hours":
				date.setHours(date.getHours() + value);
				break;
			case "days":
				date.setDate(date.getDate() + value);
				break;
			case "months":
				date.setMonth(date.getMonth() + value);
				break;
			case "years":
				date.setFullYear(date.getFullYear() + value);
				break;
		}
		return date;
	};
	
	CreatePopup.prototype.__makeDatetime = function($section, limits) {
		limits = limits || {};
		limits["default"] = limits["default"] || new Date();
		
		$section.find("[date]")
			.datepicker({
				dateFormat: "yy-mm-dd",
				minDate: limits.min, 
				maxDate: limits.max
			})
			.datepicker("setDate", limits["default"]);
			
		$section.find("[hour]")
			.timeEntry({
				spinnerImage : "",
				show24Hours : true
			})
			.val(inConcert.Utils.zeropad(limits["default"].getHours()) + 
				":" + inConcert.Utils.zeropad(limits["default"].getMinutes()));
	};
	
	CreatePopup.prototype.__makeScheduledDate = function($body) {
		var $scheduled = $body.find("[schedule_date]");
		if (!this.HasScheduledDate()) {
			$scheduled.remove();
			return false;
		}
		this.__makeDatetime($scheduled, this.__getScheduleDateLimits());
		return true;
	};
	
	CreatePopup.prototype.__makeBackdatedTime = function($body) {
		var $backdated = $body.find("[backdated_time]");
		if (!this.HasBackdatedTime()) {
			$backdated.remove();
			return false;
		}
		this.__makeDatetime($backdated, this.__getBackdatedTimeLimits());
		return true;
	};
	
	CreatePopup.prototype.__getDatetime = function($section) {
		if (!$section.find("input[type=checkbox]").prop("checked")) {
			return null;
		}
		var date = $section.find("[date]").val();
		var hours = $section.find("[hour]").val() || "00:00";
		return new Date(date + "T" + hours);
	};
	
	CreatePopup.prototype.__getScheduledDate = function() {
		return this.__getDatetime($("#snCreatePopup").find("[schedule_date]"));
	};
	
	CreatePopup.prototype.__getBackdatedTime = function() {
		return this.__getDatetime($("#snCreatePopup").find("[backdated_time]"));
	};
	
	CreatePopup.prototype.__makeAdvancedOptions = function($body) {
		var $options = $body.filter("[advanced_options]");
		$options.append(templates.create_popup_advanced);
		
		var options = this.__makeScheduledDate($body);
		options = this.__makeBackdatedTime($body) || options;
		
		if (!options) {
			$options.hide();
			return;
		}
		$options.find("input[type=checkbox]")
			.change(function() {
				var selected = $(this).prop("checked");
				$(this).parent()
					.toggleClass("disabled", !selected)
					.find("input[type=text], select")
					.prop("disabled", !selected);
			})
			.change();
	};
	
	CreatePopup.prototype.__updateImagePreview = _.throttle(function() {
		var url = null;
		if ($("#post_create_content_image_link").is(":checked")) {
			url = $("#post_create_content_image_link_text").val();
		}
		else if ($("#post_create_content_image_disk").is(":checked")) {
			url = this.__uploadedImage;
		}
		const background = url? format("url(\"{0}\")", url) : "none";
		$("#post_create_content_image_preview").css("background-image", background);
	}, 100);
	
	CreatePopup.prototype.__makeImageUploader = function($uploader) {
		var self = this;
		$uploader.parent().click(function() {
			if (self.processing) {
				return;
			}
			$uploader[0].click();
		});
		$uploader.change(function( event ) {
			if (self.processing) {
				return;
			}
			var file = event.target.files[0];
			if ( !file ) {
				return;
			}
			var reader = new FileReader();
			reader.onload = function (e) {
				self.__uploadedImage = e.target.result;
				self.__updateImagePreview();
            };
			reader.readAsDataURL(file);
		});
	};

	CreatePopup.prototype.__fill_combo_post_create = function($content, content) {
		const $post_create_content = $content.find("#contentSelectOption");
		const is_blank_first_item = false;
		const attr_id = "id";
		const attr_name = "description";
		const types = [
			{id: "nothing", description: "<%=_T('Just Text')%>"},
		];
		if (content.link) {
			types.push({id: "link", description: "<%=_T('Url link')%>"});
		}
		if (content.image) {
			types.push({id: "image", description: "<%=_T('Image')%>"});
		}

		inConcert.Utils.fill_combo($post_create_content, is_blank_first_item, types, attr_id, attr_name);
	};

	CreatePopup.prototype.__makeContent = function($body) {
		var self = this;
		var $content = $body.filter("[content_container]");
		var content = this.__getAvailableContentTypes();
		if (!content) {
			$content.hide();
			return;
		}
		$content.append(templates.create_popup_content);

		const $content_image_preview = $content.find("[image_preview]");
		const $postCreateContentImageDisk = $content.find("#postCreateContentImageDisk");
		const $postCreateContentImageLink = $content.find("#postCreateContentImageLink");

		this.__fill_combo_post_create($content, content);

		//link
		if (!content.link) {
			$content.find("[content_data=link]").remove();
		}

		//image
		if (content.image) {
			if (!content.image.link) {
				$content.find("[image_content=link]").remove();
				$postCreateContentImageLink.remove();
			}

			if (content.image.disk) {
				this.__makeImageUploader($content.find("#post_create_content_image_disk_upload"));
			}
			else {
				$content.find("[image_content=disk]").remove();
				$postCreateContentImageDisk.remove();
			}

			const $image_selector_link = $content.find("[image_selector=link]");
			$image_selector_link
				.click(function() {
					$content_image_preview.show();
					$postCreateContentImageDisk.hide();
					$postCreateContentImageLink.show();
					self.__updateImagePreview();
				});

			$content.find("[image_selector=disk]")
				.click(function() {
					$content_image_preview.show();
					$postCreateContentImageDisk.show();
					$postCreateContentImageLink.hide();
					self.__updateImagePreview();
				});
		}
		else {
			$content.find("[content=image]").remove();
		}

		$content.find("[content_data=nothing]").show();
		
		//selector
		$content.find("#contentSelectOption")
			.on('change', function() {
					const contentSelector = $(this).val();
					$content
						.find("[content_data]")
						.hide()
						.filter(format("[content_data={0}]", contentSelector))
						.show();

					if (contentSelector === 'image') {
						if (content.image.link) {
							$content.find("[image_selector=link]").click();
						}
						else{
							$content.find("[image_selector=disk]").click();
						}
					}
					if (contentSelector === 'nothing' || contentSelector === 'link') {
						$postCreateContentImageLink
							.hide();
						$postCreateContentImageDisk
							.hide();
					}
			}).change();

		$content.find("[image_link_text]")
			.change(function() {
				self.__updateImagePreview();
			});
			
	};
	
	CreatePopup.prototype.__getTitle = function() {
		return "<%=_T('Create Post on')%>";
	};
	
	CreatePopup.prototype.__make = function() {
		var self = this;
		var sn = inConcert.app.getSNPostsManager();
		var $body = $(Mustache.render(templates.create_popup, {
			title : self.__getTitle(),
			name : sn.GetAccountName(self.account.campaign, self.account.account, self.account.page)
		}));
		$body.find("#snPopupCreateButtonClose")
		.off("click")
		.on("click", function() {
			self.Close();
		});
		//text
		$body.find("[post_text]")
			.change(function() {
				self.__onTextChange();
			});
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//update
		$body.find("[create]").click(function() {
			self.Create();
		});
		self.__makeContent($body);
		self.__makeAdvancedOptions($body);
		return $body;
	};
	
	CreatePopup.prototype.__makeEmojiPicker = function() {
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: "#snCreatePopup_textarea",
			assetsPath: "/inconcert/lib/emoji-picker/lib/img/",
			popupButtonClasses: "buttonColorGray emoji-picker-button",
			norealTime: false,
			onPaste: function(pasted) {
				return pasted;
			}
		});
		window.emojiPicker.discover();
	};
	
	CreatePopup.prototype.__getText = function() {
		return $("#snCreatePopup").find("[post_text]").val();
	};
	
	CreatePopup.prototype.__validateDatetime = function(date, limits) {
		return !isNaN(date.getTime()) &&
				(!limits.max || limits.max >= date) &&
				(!limits.min || limits.min <= date);
	};
	
	CreatePopup.prototype.__setTextDisclaimer = function(disclaimer) {
		$("#snCreatePopup").find("[text_disclaimer]")
			.text(disclaimer || "")
			.toggle(!!disclaimer);
	};
	
	CreatePopup.prototype.__getTextLength = function() {
		return (this.__getText() || "").length;
	};
	
	CreatePopup.prototype.__showTextLimit = function(length, limit) {
		if (!limit) {
			return;
		}
		var remaining = limit - length;
		$("#snCreatePopup").find("[text_limit]")
			.toggleClass("neutral", remaining === 0)
			.toggleClass("positive", remaining > 0)
			.toggleClass("negative", remaining < 0)
			.text(Math.abs(remaining))
			.show();
	};
	
	CreatePopup.prototype.__validateText = function() {
		var textLength = this.__getTextLength();
		var textLimit = this.__getTextLimit();
		this.__showTextLimit(textLength, textLimit);
		return textLength > 0 && (!textLimit || textLength <= textLimit);
	};
	
	CreatePopup.prototype.__onTextChange = _.throttle(function() {
		$("#snCreatePopup").find("[create]").prop("disabled", !this.__validateText());
	}, 100);
	
	CreatePopup.prototype.Message = function(text, type) {
		var $message = $("#snCreatePopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	CreatePopup.prototype.Loading = function(loading) {
		$("#snCreatePopup").find("[loading]").toggle(!!loading);
	};
	
	CreatePopup.prototype.__addScheduleDate = function(data) {
		
	};
	
	CreatePopup.prototype.__addBackdatedTime = function(data) {
		
	};
	
	CreatePopup.prototype.__addAdvancedOptions = function(data) {
		return this.__addScheduleDate(data) ||
			   this.__addBackdatedTime(data);
	};
	
	CreatePopup.prototype.__addLinkToData = function(data, link, isImage) {
		
	};
	
	CreatePopup.prototype.__addLink = function(data) {
		if ($("#contentSelectOption").val() !== "link" ) {
			return;
		}
		var link = $("#post_create_content_link_text").val();
		if (!link) {
			return "<%=_T('Must enter a link.')%>";
		}
		this.__addLinkToData(data, link);
	};
	
	CreatePopup.prototype.__addImageFromLink = function(data) {
		var link = $("#post_create_content_image_link_text").val().trim();
		if (!link || link === "") {
			return "<%=_T('Must enter a link.')%>";
		}
		this.__addLinkToData(data, link, true);
	};
	
	CreatePopup.prototype.__addImageFromDisk = function(data) {
		var files = $("#post_create_content_image_disk_upload")[0].files;
		var file = files && files[0];
		if (!file) {
			return "<%=_T('Must select an image.')%>";
		}
		if ( file.size > this.__getMaxFileSizeMB() * 1024 * 1024 ) {
			return format("<%=_T('Unable upload file. Max file size until {0} MB.')%>", this.__getMaxFileSizeMB());
		}
		data.file = file;
	};
	
	CreatePopup.prototype.__addImage = function(data) {
		if ($("#contentSelectOption").val() !== "image") {
			return;
		}
		if ($("#post_create_content_image_link").is(":checked")) {
			return this.__addImageFromLink(data);
		}
		if ($("#post_create_content_image_disk").is(":checked")) {
			return this.__addImageFromDisk(data);
		}
	};
	
	CreatePopup.prototype.__addContent = function(data) {
		return this.__addLink(data) ||
			   this.__addImage(data);
	};
	
	CreatePopup.prototype.__getSeparator = function($separator) {
		var separator = $separator.val();
		switch(separator) {
			case "TAB":
				separator = "\t";
				break;
			case "LINE_BREAK":
				separator = "\n";
				break;
		}
		return separator;
	};
	
	CreatePopup.prototype.__getData = function() {
		var data = {};
		var error = this.__addText(data) ||
					this.__addContent(data) ||
					this.__addAdvancedOptions(data);
		if (error) {
			this.Message(error, "error");
			return null;
		}
		return data;
	};
	
	CreatePopup.prototype.Create = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		var data = self.__getData();
		if (!data) {
			return;
		}
		self.Message();
		$("#snCreatePopup").find("button").prop("disabled", true);
		self.processing = true;
		self.Loading(true);
		inConcert.app.getSNPostsManager().CreatePost(self.account, data, self.__isScheduled(data), function(status, message) {
			self.processing = false;
			self.Loading(false);
			$("#snCreatePopup").find("button").prop("disabled", false);
			if (status) {
				return self.Close();
			}
			self.Message(message, "error");
		});
	};

}());