/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.snPostsList) {
	inConcert.snPostsList = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	let format = inConcert.Utils.format;
	let templates = inConcert.templates.social_networks;

	// Defines a generic object with its common behaviour()
	// constructor
	function PostList ()
	{
		let self = this;
		this.columns = {
			type : {
				header	: { type : "text",	className : "columnType",			label : "<%=_T('Type')%>"			},
				row		: { type : "icon",	className : "columnType",			iconClass : "interactionType"		}
			},
			account_name : {
				header	: { type : "text",	className : "columnAccount",		label : "<%=_T('Account/Page')%>"	},
				row		: { type : "text",	className : "columnAccount",		textClass : "interactionAccount"	},
				value : function(post) {
					let account = self.accounts[post.campaign + ":" + post.account + ":" + (post.page || "")];
					if (account) {
						return account.name;
					}
					return post.GetAccountName();
				}
			},
			campaign : {
				header	: { type : "text",	className : "columnCampaign",		label : "<%=_T('Campaign')%>"		},
				row		: { type : "text",	className : "columnCampaign",		textClass : "interactionGroup"		},
				value : function(post) {
					return inConcert.Utils.getIdWithoutVCC(post.campaign);
				}
			},
			id : {
				header	: { type : "text",	className : "columnId",		label : "<%=_T('Id')%>"		},
				row		: { type : "abstract", className : "columnId",	textClass : "interactionId"	},
				value : function(post) {
					return post.GetIdString();
				}
			},
			content : {
				header	: { type : "text",	className : "columnAbstract",		label : "<%=_T('Content')%>"		},
				row		: { type : "abstract", className : "columnAbstract",	textClass : "interactionAbstract"	},
				value : function(post) {
					return post.GetText();
				}
			},
			attributes : {
				header	: { type : "text",	className : "columnAttributes",		label : "<%=_T('Attributes')%>"		},
				row		: { type : "abstract", className : "columnAttributes",	textClass : "interactionAttributes"	},
				value : function(post) {
					return post.GetAttributes();
				}
			},
			publish_timestamp : {
				header	: { type : "text",	className : "columnDate",			label : "<%=_T('Publish Date')%>"	},
				row		: { type : "date",	className : "creationDate",			textClass : "interactionDate"		},
				sort	: true
			},
			last_message_timestamp : {
				header	: { type : "text",	className : "columnDate",			label : "<%=_T('Last Message')%>"	},
				row		: { type : "date",	className : "stateDate",			textClass : "interactionDate"		},
				sort	: true
			}
		};
		this.columns_order = [
			"type",
			"account_name",
			"campaign",
			"id",
			"content",
			"attributes",
			"publish_timestamp",
			"last_message_timestamp"
		];
		this.default_sort = { column : "publish_timestamp", asc : false };
		this.batch_actions = [];
		this.current_batch_actions = [];
		this.accounts = {};
		this.query = {};
		this.sort = null;
		this.current_posts = {};
		this.MAX_ALLOWED_ROWS_IN_ES = inConcert.app.getServerInfo().maxAllowedRowsInES; // Este valor se obtiene del LUA "/apps/agent/shared/utils.lua", funcion "GetMaxAllowedRowsInES"
		this.rowsPerPage = 20;
		this.rowsPerPage = this.rowsPerPage > this.MAX_ALLOWED_ROWS_IN_ES ? this.MAX_ALLOWED_ROWS_IN_ES : this.rowsPerPage;
		this.paginationId = "snPostsPagination";
	}

	if (typeof inConcert.snPostsList.base !== "function") {
		inConcert.snPostsList.base = PostList;
	}

	PostList.prototype.__getPermission = function() {
		throw "Abstract method";
	};

	PostList.prototype.GetView = function() {
		throw "Abstract method";
	};

	PostList.prototype.RetrieveType = function() {
		throw "Abstract method";
	};

	PostList.prototype.GetName = function() {
		throw "Abstract method";
	};

	PostList.prototype.GetDescription = function() {
		throw "Abstract method";
	};
	
	PostList.prototype.GetClassName = function() {
		throw "Abstract method";
	};

	PostList.prototype.__postStateValid = function(post) {
		const tabSelectedId = inConcert.social_networks.getSelectedTabId();
		if (tabSelectedId === "ACTIVE" || tabSelectedId === "SCHEDULED") {
			return post.IsActive() || post.IsPaused();
		}
		else if (tabSelectedId === "ARCHIVE") {
			return post.IsClosed() || post.IsDeleted();
		}
		throw(`PostList.prototype.__postStateValid - unrecognized category '${tabSelectedId}'`);
	};

	PostList.prototype.GetRowsPerPage = function() {
		return this.rowsPerPage;
	};

	PostList.prototype.HasPermission = function(campaign) {
		return inConcert.app.getSNPostsManager().HasPermission(campaign, this.__getPermission());
	};
	
	PostList.prototype.HasDisabledBatchActions = function() {
		return false;
	};
	
	PostList.prototype.HasBatchActions = function() {
		return this.current_batch_actions.length > 0;
	};
	
	PostList.prototype.HasActions = function() {
		return true;
	};
	
	PostList.prototype.HasPagination = function() {
		return true;
	};

	PostList.prototype.__loadAccount = function(accountData) {
		const self = this;
		const $account = $(Mustache.render(templates.menu_account_item, accountData));
		$account.click(function() {
			if (self.IsProcessing()) {
				return;
			}
			const $this = $(this);
			$this
				.addClass("selected")
				.siblings()
				.removeClass("selected");
			self.selectedAccount = {
				campaign: $this.attr("campaign"),
				account : $this.attr("account"),
				page    : $this.attr("page") || null,
				refresh : true
			};
			self.selectedPost = null;
			self.Search();
		});
		return $account;
	};
	
	PostList.prototype.__getAvailableChannels = function() {
		return null;
	};
	
	PostList.prototype.__getViewAccounts = function() {
		let list = inConcert.app.getSNPostsManager().GetAccounts(true, this.__getPermission());
		let channels = this.__getAvailableChannels();
		if (channels) {
			let original = list;
			list = [];
			$.each(original, function(_, account) {
				if (channels[account.type]) {
					list.push(account);
				}
			});
		}
		return list;
	};
	
	PostList.prototype.__loadAccounts = function() {
		let self = this;
		let accounts = this.__getViewAccounts();
		self.accounts = {};
		let $accounts = [];
		$.each(accounts, function(_, account) {
			self.accounts[account.campaign + ":" + account.account + ":" + account.page] = account;
			$accounts.push(self.__loadAccount(account));
		});
		$("#snMenu")
			.find(format("[category={0}]", self.GetView()))
			.find("[accountsList]")
			.html("")
			.append($accounts);
	};
	
	PostList.prototype.__movePage = function(page) {
		if (this.IsProcessing()) {
			return;
		}
		this.page = page;
		this.Search(null, true);
	};
	
	PostList.prototype.__loadPagination = function() {
		const self = this;
		this.page = 1;
		const $pagination = $(templates.pagination());
		$pagination
			.find("[first]")
			.click(function() {
				self.__movePage(1);
			});
		$pagination
			.find("[prev]")
			.click(function() {
				self.__movePage(self.page - 1);
			});
		$pagination
			.find("[next]")
			.click(function() {
				self.__movePage(self.page + 1);
			});
		$pagination
			.find("[last]")
			.click(function() {
				let page;
				let rowsPerPage = self.GetRowsPerPage();
				if (self.total > self.MAX_ALLOWED_ROWS_IN_ES) {
					// Si ha llegado al maximo de rows permitidos por ES, me quedo con la pagina previa
					page = self.MAX_ALLOWED_ROWS_IN_ES / rowsPerPage;
				}
				else {
					page = self.total / rowsPerPage;
				}
				page = Math.floor(page) < page ? Math.floor(page) + 1 : page;
				self.__movePage(page);
			});
		return $pagination;
	};

	PostList.prototype.__updatePagination = function() {
		let rowsPerPage = this.GetRowsPerPage();
		let from = (this.page - 1) * rowsPerPage + 1;
		let to = Math.min(from + rowsPerPage - 1, this.total);

		let disableMorePages = false;
		if (this.total  === 0 || this.total  <= rowsPerPage) {
			// Si no tiene registros o el total de registros es menor o igual a la cantidad de registros por pagina, desactivo la navegacion para obtener mas Rows.
			disableMorePages = true;
		}
		else if ((this.page * rowsPerPage) >= this.MAX_ALLOWED_ROWS_IN_ES) {
			// Si llego al maximo de Rows permitidas desactivo la navegacion para obtener mas Rows.
			disableMorePages = true;
		}
		else {
			let lastPage =this.total / rowsPerPage;
			lastPage = Math.floor(lastPage) < lastPage ? Math.floor(lastPage) + 1 : lastPage;
			if (this.page === lastPage) {
				// Si estoy en la ultima pagina desactivo la navegacion para obtener mas Rows.
				disableMorePages = true;
			}
		}

		let $pagination = $("#" + this.paginationId);
		$pagination
			.find("[pagination_label]")
			.html(format("<%=_T('{0} - {1} of {2}')%>", from, to, this.total));
		$pagination
			.find("[first]")
			.prop("disabled", from === 1);
		$pagination
			.find("[prev]")
			.prop("disabled", from === 1);
		$pagination
			.find("[next]")
			.prop("disabled", disableMorePages);
		$pagination
			.find("[last]")
			.prop("disabled", disableMorePages);
	};
	
	PostList.prototype.__canRunAction = function(action) {
		if (!action) {
			return false;
		}
		if (this.query.campaign) {
			return inConcert.permissions.to_action("social_networks", action.permission, null, this.query.campaign);
		}
		return inConcert.permissions.to_action_on_any_campaign("social_networks", action.permission);
	};

	PostList.prototype.__runBatchAction = function(actionName) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		self.processing = true;
		self.__waiting(true);
		let posts = [];
		$("#snPostsList")
			.find("input[row_selector]:checked")
			.each(function() {
				posts.push(self.current_posts[$(this).attr("post_id")]);
			});
		inConcert.app.getSNPostsManager().RunBatchAction(posts, actionName, function(status, result) {
			self.processing = false;
			self.__waiting(false);
			if (status) {
				if (result !== "ABORTED" && result.unsuccessful_nodes.length) {
					inConcert.app.m_mainWindow.showBannerMessage({
						message: "<%=_T('Action failed for some posts.')%>", messageType: "Error", timeout: 5000 
					});
				}
			}
			else {
				inConcert.app.m_mainWindow.showBannerMessage({ message: result, messageType: "Error", timeout: 5000 });
			}
		}, null, true);
	};

	/*
		(*) Las acciones se tienen que ocultar ya que las mismas se encuentran en el menu contextual de acciones por batch.
 			Por el funcionamiento del codigo el boton $action tiene que existir en el DOM.
	*/
	PostList.prototype.__loadBatchAction = function(actionName) {
		const self = this;
		const action = inConcert.snPostsActions.all[actionName];
		if (!self.__canRunAction(action)) {
			return null;
		}
		this.current_batch_actions.push(actionName);
		const $action = $(Mustache.render(templates.batch_action_button, {
			action : actionName,
			name : action.name,
			description : action.description,
			iconClassName : actionName.toLowerCase() + "Post"
		}));
		$action.hide(); // (*) ver comentario de la funcion
		$action.click(function() {
			self.__runBatchAction($(this).attr("action"));
		});
		return $action;
	};

	PostList.prototype.__loadBatchActions = function() {
		const self = this;
		let $actions = [];
		this.current_batch_actions = [];
		if (!this.HasDisabledBatchActions()) {
			$.each(this.batch_actions, function(_, action) {
				$actions.push(self.__loadBatchAction(action));
			});
		}
		const $pagination = $("#snPostsBatchPagination");
		$pagination.html("").hide();
		if (this.HasPagination()) {
			$pagination.append(this.__loadPagination());
			$pagination.show();
		}
		else {
			this.pagination = null;
		}
		$("#snPostsBatchActions_actions_menu_options")
			.html("")
			.append($actions);
	};

	PostList.prototype.__updateBatchActions = function() {
		let $rows = $("#snPostsList").find("input[row_selector]:checked");
		let $actions = $("#snPostsBatchActions_actions_menu_options").find("a[action]");
		this.__createButtonActions();
		$.each(this.current_batch_actions, function(_, action) {
			const hide = $rows.length === 0 || $rows.filter(format("[{0}=false]", action)).length > 0;
			$actions
				.filter(format("[action={0}]", action))[hide?"hide":"show"]();
		});
	};
	
	PostList.prototype.__loadHeaderColumn = function(name, header, sort) {
		const self = this;
		let content = "";
		switch(header.type) {
			case "text":
				content = Mustache.render(templates.list_header_column_text, { label : header.label });
				break;
			case "icon":
				content = templates.list_header_column_icon;
				break;
			case "selector-all":
				content = templates.batch_action_select_all;
				break;
		}
		const className = header.className;
		const $column = $(Mustache.render(templates.list_header_column, { name : name, className : className, content : content }));
		if (sort) {
			$column
				.addClass("columnSorteable")
				.click(function() {
					if (self.IsProcessing()) {
						return;
					}
					self.__setSort(name, !$(this).hasClass("columnSortAsc"));
					self.Search();
				});
		}
		if (header.type === "selector-all") {
			const $selectAll = $column.find("[select-all='true']");
			$selectAll.change(function() {
				$("#snPostsList")
					.find("input[row_selector]")
					.prop("checked", $(this).prop("checked"));
				self.__updateBatchActions();
			});
		}
		return $column;
	};
	
	PostList.prototype.__loadHeaders = function() {
		const self = this;
		const $header = $("#snPostsListHeader");
		let $columns = [];
		if (this.HasBatchActions()) {
			$columns.push(this.__loadHeaderColumn("", { type : "selector-all", className : "columnBatch" }));
		}
		$.each(this.columns_order, function(_, columnName) {
			$columns.push(self.__loadHeaderColumn(columnName, self.columns[columnName].header, self.columns[columnName].sort));
		});
		if (this.HasActions()) {
			$columns.push(this.__loadHeaderColumn("actions", { type : "icon", className : "columnActions" }));
		}
		$header
			.html("")
			.append($columns);
	};
	
	PostList.prototype.__message = function(message) {
		if (message) {
			$("#snPostsMessages")
				.html(message)
				.show();
		}
		else {
			$("#snPostsMessages")
				.html("")
				.hide();
		}
	};
	
	PostList.prototype.__waiting = function(waiting) {
		$("#snPostsWaiting").toggle(!!waiting);
	};
	
	PostList.prototype.__clear = function() {
		$("#snPostsList").html("");
		$("#snPostsBatchSelect").prop("checked", false);
		this.__message();
	};
	
	PostList.prototype.__getActionsMenuId = function() {
		return "sn_posts_list_menu_actions";
	};
	
	PostList.prototype.__runPostAction = function(postId, actionName) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		self.processing = true;
		let $waiting = $("#snPostsList").find(format("div[post_waiting][post_id='{0}']", postId));
		let post = self.current_posts[postId];
		$waiting.show();
		post.RunAction(actionName, function(status, error) {
			$waiting.hide();
			self.processing = false;
			if (!status) {
				inConcert.app.m_mainWindow.showBannerMessage({ message: error, messageType: "Error", timeout: 5000 });
			}
		});
	};
	
	PostList.prototype.__loadPostAction = function(postId, action) {
		let self = this;
		let $action = $(Mustache.render(templates.post_actions_menu_item, { 
			id : postId, 
			action : action.id,
			name : action.name,
			tooltip : action.description,
			className : action.id.toLowerCase() + "Post"
		}));
		$action.click(function() {
			if (self.actionsMenu) {
				self.actionsMenu.finalize(function() {
					self.actionsMenu = null;
				});
			}
			self.__runPostAction($(this).attr("post_id"), $(this).attr("action"));
		});
		return $action;
	};
	
	PostList.prototype.__loadPostActions = function(postId) {
		let self = this;
		if (self.IsProcessing()) {
			return;
		}
		function show() {
			let $button = $("#snPostsList").find(format("input[post_actions][post_id='{0}']", postId));
			let menuId = self.__getActionsMenuId();
			let post = self.current_posts[postId];
			let $menu = $(Mustache.render(templates.post_actions_menu, { id : menuId }));
			$.each(post.GetActions(), function(_, action) {
				$menu.append(self.__loadPostAction(postId, action));
			});
			$("#" + menuId).remove();
			$("body").append($menu);
			self.actionsMenu = new inConcert.PopupWindow();
			self.actionsMenu.show({
				ownerId      : menuId,
				eventName    : menuId,
				canReposition: true,
				container    : $menu,
				parentNode   : $button,
				postId       : postId,
				callbacks: {
					onOpen: function() {
						$('#' + menuId).show();
					},
					onClose: function( callback = $.noop ) {
						if (self.actionsMenu) {
							self.actionsMenu.hide(function() {
								callback();
							});
						}
						else {
							callback();
						}
					}
				}
			});

			const $btnAction = $button;
			const btnActionOffset = $btnAction.offset();
			const dropdownLeft = btnActionOffset.left - ($menu.outerWidth() - $btnAction.outerWidth());
			$menu.css({left: dropdownLeft });
		}
		if (self.actionsMenu && self.actionsMenu.isVisible()) {
			self.actionsMenu.finalize(function() {
				let data = self.actionsMenu.getData();
				self.actionsMenu = null;
				if ( data && data.postId !== postId ) {
					show();
				}
			});
		}
		else {
			show();
		}
	};
	
	PostList.prototype.__loadPostColumn = function(name, column, post) {
		let row = column.row;
		let value = column.value? column.value(post) : post[name];
		let content = "";
		let className = row.className;
		switch(column.row.type) {
			case "date":
				value = inConcert.Utils.formatDate(value * 1000);
				className = "columnDate " + className;
				content = Mustache.render(templates.list_column_text, { className : row.textClass, text : value });
				break;
			case "text":
				content = Mustache.render(templates.list_column_text, { className : row.textClass, text : value });
				break;
			case "abstract":
				content = Mustache.render(templates.list_column_abstract, { className : row.textClass, text : value });
				break;
			case "icon":
				content = templates.list_column_icon;
		}
		if (this.sort && this.sort.column === name) {
			className += " columnSorted";
		}
		return Mustache.render(templates.list_column, { name : name, className : className, content : content });
	};
	
	PostList.prototype.__loadPost = function(post, hasBatch, hasActions) {
		let self = this;
		let postId = post.GetId();
		self.current_posts[postId] = post;
		let $columns = [];
		if (hasBatch) {
			let $select = $(Mustache.render(templates.list_row_selector, { id : postId }));
			$select.change(function() {
				$("#snPostsBatchSelect").prop("checked", false);
				self.__updateBatchActions();
			});
			$.each(self.current_batch_actions, function(_, action) {
				$select
					.find("input[row_selector]")
					.attr(action, post.CanRunAction(action));
			});
			$columns.push($select);
		}
		$.each(this.columns_order, function(_, columnName) {
			$columns.push($(self.__loadPostColumn(columnName, self.columns[columnName], post)));
		});
		if (hasActions && post.HasActions()) {
			let $actions = $(Mustache.render(templates.list_column_actions, { id : postId }));
			$columns.push($actions);
			$actions.click(function() {
				self.__loadPostActions($(this).attr("post_id"));
			});
		}
		let className = postId === self.selectedPost? "selected " : "";
		let iconTooltip = "";
		$.each(post.GetNativeProperties(), function(_, property) {
			className += property.className? ((className? " " : "") + property.className + "Post") : "";
			iconTooltip += property.tooltip? ((iconTooltip? " " : "") + property.tooltip) : "";
		});
		let $post = $(Mustache.render(templates.list_row, { id : postId, type : post.type, className : className }));
		$post.append($columns);
		$post.click(function(e) {
			let $target = $(e.target);
			if ($target.attr("row_selector") || $target.attr("post_actions")) {
				return;
			}
			$(this)
				.addClass("selected")
				.siblings()
				.removeClass("selected");
			self.selectedPost = $(this).attr("post_id");
			const $snStats = $("#snStats");
			if ( $snStats.children().length > 0 && !$snStats.is(":hidden")) {
				this.statistics = self.__makePostStatistics();
				this.statistics.Load();
			}else {
				self.__selectStatistics(self.__makePostStatistics());
			}
		});
		$post.find("[column=type]").attr("title", iconTooltip);
		return $post;
	};
	
	PostList.prototype.__loadPosts = function(posts, noMessage) {
		let self = this;
		self.current_posts = {};
		if (posts.length === 0) {
			if (!noMessage) {
				self.__message("<%=_T('No posts found')%><%=_T('.')%>");
			}
			return;
		}
		let $container = $("#snPostsList");
		let elements = [];
		let hasBatch = self.HasBatchActions();
		let hasActions = self.HasActions();
		$.each(posts, function(_, post) {
			if (self.__postStateValid(post)) {
				elements.push(self.__loadPost(post, hasBatch, hasActions));
			}
		});
		$container.append(elements);
		const uniqueRow = {};
		$container.contents().each(function() {
			const row = $(this).attr("post_id");
			if (uniqueRow[row]) {
				$(this).remove();
			}
			else {
				uniqueRow[row] = true;
			}
		});
	};
	
	PostList.prototype.__loadSearch = function() {
		let self = this;
		$("#snPostsRefresh")
			.off("click")
			.on("click", function() {
				if (self.selectedAccount) {
					self.selectedAccount.refresh = true;
				}
				self.Search();
			});
		$("#snPostsSearch")
			.val("")
			.off("keypress")
			.on("keypress", function(e) {
				if(e.which === 13) {
					self.Search();
				}
			});
	};
	
	PostList.prototype.__setSort = function(column, asc) {
		let $headers = $("#snPostsListHeader").children();
		$headers
			.removeClass("columnSortAsc")
			.removeClass("columnSortDesc");
		if (column) {
			this.sort = { column : column, asc : asc };
			$headers
				.filter(format("[column={0}]", column))
				.addClass(asc? "columnSortAsc" : "columnSortDesc");
		}
		else {
			this.sort = null;
		}
	}; 
	
	PostList.prototype.IsProcessing = function() {
		return !!this.processing;
	};
	
	PostList.prototype.UpdatePosts = function() {
		let self = this;
		if ($.isEmptyObject(self.current_posts)) {
			return;
		}
		let posts = [];
		$.each(self.current_posts, function(_, post) {
			posts.push(post);
		});
		self.__clear();
		self.__loadPosts(posts, true);
		if (self.selectedPost) {
			self.__selectStatistics(self.__makePostStatistics());
		}
		self.__updateBatchActions();
	};

	PostList.prototype.Search = function(callback, pagination, tabSelected) {
		callback = callback || $.noop;
		const self = this;
		if (this.IsProcessing()) {
			return callback();
		}
		this.processing = true;
		//sort
		let sort = null;
		if (this.sort) {
			let condition = {};
			condition[this.sort.column] = { order : this.sort.asc ? "asc" : "desc" };
			sort = [condition];
		}

		let request = {
			view : this.RetrieveType(),
			query: this.query,
			sort : sort,
			rows : this.GetRowsPerPage()
		};
		// contenido
		request.query.content = $("#snPostsSearch").val() || null;
		this.query.content = request.query.content;

		const listItems = $("#snPostsAccountList");
		const $itemSelected = listItems.find(".selected");
		if ($itemSelected.length > 0) {
			this.selectedAccount = {
				campaign: $itemSelected.attr("campaign"),
				account : $itemSelected.attr("account"),
				page    : $itemSelected.attr("page") || null,
				content : this.query.content,
				refresh : true
			};
			request.query = this.selectedAccount;
		}
		//selected account
		if (this.selectedAccount && !this.selectedPost) {
			this.__addAccountConditions(request);
		}
		//pagination
		if (this.HasPagination() && pagination) {
			request.from = (this.page - 1) * request.rows;
		}
		else {
			this.page = 1;
			request.from = 0;
		}
		this.__clear();
		this.__waiting(true);
		inConcert.app.getSNPostsManager().SearchPosts(request, function(data, error) {
			self.__waiting(false);
			self.processing = false;
			if (data) {
				self.__loadPosts(data.posts);
				self.total = data.total;
			}
			else {
				let message = error && error.reason || "<%=_T('Unexpected error searching posts')%>";
				self.__message(message);
				self.total = 0;
			}
			if (self.HasPagination()) {
				self.__updatePagination();
			}
			if (request.account_statistics && !tabSelected && $itemSelected.length > 0) {
				let statistics = data && data.account_statistics? self.__makeAccountStatistics(data.account_statistics) : null;
				self.__selectStatistics(statistics, undefined, true);
			}
			else {
				self.__selectStatistics(self.__makePostStatistics(), undefined, true);
			}
			
			self.__updateBatchActions();
			callback();
		});
	};
	
	PostList.prototype.__addAccountConditions = function(request) {
		request.query.campaign = this.selectedAccount.campaign || null;
		request.query.account = this.selectedAccount.account;
		request.query.page = this.selectedAccount.page || null;
		if (this.selectedAccount.refresh) {
			request.account_statistics = true;
			this.selectedAccount.refresh = false;
		}
	};
	
	PostList.prototype.SelectStatsSlice = function(slice, callback = $.noop) {
		if (this.statistics) {
			this.statistics.SelectSlice(slice, callback);
		}
		else {
			callback();
		}
	};

	PostList.prototype.__selectStatistics = function(statistics, beforeSearch, hideDetailPanel) {
		const $snStats = $("#snStats");
		const $buttonCreate = $("#snButtonCreate");
		const $main = $("#snMain");
		if (!beforeSearch) {
			$buttonCreate.show();
			const hasStatistics = !!statistics && !hideDetailPanel;
			$snStats.toggle(hasStatistics);
			$main.toggleClass("splitedScenario", hasStatistics);
		}
		if (statistics) {
			this.statistics = statistics;
			this.statistics.Load();
		}
		else {
			this.statistics = null;
			$snStats.html("");
			$buttonCreate.hide();
		}
	};

	PostList.prototype.__makeAccountStatistics = function(data) {
		let selected = this.selectedAccount;
		let account = this.accounts[selected.campaign + ":" + selected.account + ":" + (selected.page || "")];
		if (!account) {
			return null;
		}
		selected.statistics = data;
		return inConcert.app.getSNPostsManager().BuildAccountStatistics({
			accountName	: account.name,
			type		: account.type,
			campaign	: account.campaign,
			account		: account.account,
			page		: account.page || "",
			statistics	: data
		});
	};
	
	PostList.prototype.__makePostStatistics = function() {
		let post = this.current_posts[this.selectedPost || ""];
		if (!post) {
			this.selectedPost = null;
			if (this.selectedAccount && this.selectedAccount.statistics) {
				return this.__makeAccountStatistics(this.selectedAccount.statistics);
			}
			return null;
		}
		let account = this.accounts[post.campaign + ":" + post.account + ":" + (post.page || "")];
		return inConcert.app.getSNPostsManager().BuildPostStatistics({
			accountName : account.name,
			post		: post
		});
	};
	
	PostList.prototype.destroy_action_menu = function() {
		const self = this;
		if (!this.m_action_menu) {
			return;
		}
		this.m_action_menu.finalize(function() {
			self.m_action_menu = null;
		});
	};

	PostList.prototype.toggle_action_menu = function($parent) {
		const self = this;
		const interaction = $("#snPostsList").find("input[row_selector]:checked")
			.parent().parent().parent();
		let isPaused = false;
		let isReactivate = false;
		for (const option of interaction) {
			const $option = $(option);
			if ($option.hasClass("pausedPost")) {
				isPaused = true;
			}else {
				isReactivate = true;
			}
		}
		let custom_menu_options = [
			{
				id     : "pause",
				name   : "<%=_T('Pause')%>",
				execute: function() {
					self.__runBatchAction("PAUSE");
				}
			},
			{
				id     : "resume",
				name   : "<%=_T('Resume')%>",
				execute: function() {
					self.__runBatchAction("RESUME");
				}
			},
			{
				id     : "set attributes",
				name   : "<%=_T('Set Attributes')%>",
				execute: function() {
					self.__runBatchAction("SET_ATTRIBUTES");
				}
			},
			{
				id     : "close",
				name   : "<%=_T('Close')%>",
				execute: function() {
					self.__runBatchAction("CLOSE");
				}
			}
		];
		if (isPaused) {
			custom_menu_options = custom_menu_options.filter(option => option.id !== 'pause');
		}
		if(isReactivate) {
			custom_menu_options = custom_menu_options.filter(option => option.id !== 'resume');
		}
		if (this.m_action_menu && this.m_action_menu.isVisible()) {
			this.destroy_action_menu();
		}
		else {
			this.show_action_menu($parent, custom_menu_options);
		}
	};

	PostList.prototype.can_show_action_menu = function(can_show_action_menu) {
		can_show_action_menu = can_show_action_menu || function() {return true;};
		return can_show_action_menu();
	};

	PostList.prototype.show_action_menu = function($parent, menu_options, can_show_action_menu, callback = $.noop) {
		const self = this;

		if (!this.can_show_action_menu(can_show_action_menu)) {
			callback();
			return;
		}
		let $rows = $("#snPostsList").find("input[row_selector]:checked");
		if ($rows.length == 0) {
			$("#snContentMenuAction").hide();
			this.destroy_action_menu();
		}else{
			const menu_id =  "snContentMenuAction";
			$('#' + menu_id).remove();
			const $menu = $(''.concat(
				'<div id="' + menu_id + '" class="interactionMenuBlind snackBarActionsDropup" hidden></div>'
			));
	
			$.each(menu_options, function(_, option) {
				const $option = $("".concat(
					"<div option='" + option.id + "' class='interactionMenuItem '>",
						option.name,
					"</div>"
				));
				$option.click(function() {
					window.setTimeout(function() {
						option.execute($menu);
						self.destroy_action_menu();
					}, 50);
				});
				$menu.append($option);
			});

			const fnPosition = function() {
				const $btnAction = $parent;
				const btnActionOffset = $btnAction.offset();
				const dropdownLeft = btnActionOffset.left - ($menu.outerWidth() - $btnAction.outerWidth());
				$menu.css({left: dropdownLeft });
			};
	
			$('body').append($menu);
			this.m_action_menu = new inConcert.PopupWindow({ownerId: menu_id});
			this.m_action_menu.show(
			{
				ownerId       : menu_id,
				eventName     : menu_id,
				canReposition : false,
				container     : $menu,
				parentNode    : $parent,
	//			align         : "LEFT",
				callbacks: {
					onOpen: function() {
						self.m_action_menu.setPosition();
						fnPosition();
					},
				}
			});
			callback();
			const $snStats = $("#snStats");
			$(window).on('resize', function() {
				if ($menu.is(':visible') && self.m_action_menu && $snStats.is(":hidden")) {
						self.m_action_menu.setPosition();
						fnPosition();
				  }
			});
		}
	};

	PostList.prototype.__createButtonActions = function() {
		const self = this;
		const $snPostsBatchActions = $("#snPostsBatchActions");
		const tabSelectedId = inConcert.social_networks.getSelectedTabId();
		$snPostsBatchActions.hide();
		if (tabSelectedId === "ACTIVE" || tabSelectedId === "SCHEDULED") {
			const $rows = $("#snPostsList").find("input[row_selector]:checked");
			const $rowsnumber = $("#snPostsBatchActions_selected_number");
			$rowsnumber.html($rows.length);
			if ($rows.length > 0) {
				$snPostsBatchActions.show();
				$("#snPostsBatchActions_actions_menu")
				.off("click")
				.on("click", function() {
					const $this = $(this);
					self.toggle_action_menu($this);
				});

				$("#snPostsBatchActions_actions_cancel")
				.off("click")
				.on("click", function() {
					$("#snPostsBatchSelect").prop("checked", true).click();
				});
			}
		}
	};

	PostList.prototype.Load = function(callback, selectedAccount) {
		callback = callback || $.noop;
		this.__loadAccounts();
		this.__loadBatchActions();
		this.__loadHeaders();
		this.__loadSearch();
		this.query = {};
		this.selectedPost = null;
		this.selectedAccount = selectedAccount || null;
		this.__selectStatistics(null, false);
		if (selectedAccount) {
			$("#snMenu").find("[accountslist]")
				.find(format("[campaign='{0}'][account='{1}'][page='{2}']",
					selectedAccount.campaign, selectedAccount.account, selectedAccount.page))
				.addClass("selected")
				.siblings()
				.removeClass("selected");
		}
		this.__setSort(this.default_sort.column, this.default_sort.asc);
		const tabSelected = true;
		this.Search(callback, null, tabSelected);
		this.__createButtonActions();
	};

}());