/*jshint quotmark:false*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.snStatistics) {
	inConcert.snStatistics = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	//Statistics
	function Statistics (definition, slices, defaultSlice, rawData)
	{
		$.extend(true, this, rawData);
		this.definition = definition;
		this.slices = slices;
		this.defaultSlice = defaultSlice;
		this.indicators = {};
		this.now = this.Now();
	}
	
	if (typeof inConcert.snStatistics.base !== "function") {
		inConcert.snStatistics.base = Statistics;
	}
	
	Statistics.prototype.StatisticsEnabled = function() {
		return true;
	};
	
	Statistics.prototype.GetContent = function(callback) {
		throw "not implemented";
	};
	
	Statistics.prototype.GetAccountData = function() {
		throw "not implemented";
	};
	
	Statistics.prototype.GetData = function() {
		throw "not implemented";
	};
	
	Statistics.prototype.GetSlices = function() {
		return this.slices;
	};
	
	Statistics.prototype.GetSelectedSlice = function() {
		return this.selectedSlice;
	};
	
	Statistics.prototype.GetAccountName = function() {
		return this.accountName || "";
	};
	
	Statistics.prototype.GetClassName = function() {
		return "";
	};
	
	Statistics.prototype.GetNewPostText = function() {
		return "<%=_T('New')%>";
	};
	
	Statistics.prototype.AfterUiLoad = function(callback) {
		callback = callback || $.noop;
		callback();
	};
	
	Statistics.prototype.GetUpdateDate = function() {
		var timestamp = this.GetData().timestamp;
		return timestamp? inConcert.Utils.formatDate(timestamp * 1000) : "<%=_T('N/A')%>";
	};
	
	Statistics.prototype.GetDefinition = function() {
		return this.definition;
	};
	
	Statistics.prototype.__makeAnotherIndicator = function(data, indicator) {
		if (!data) {
			return;
		}
		var value = data[indicator.total || indicator.id] || 0;
		$.each(indicator.items, function(_, item) {
			value -= data[item.id] || 0;
		});
		data[indicator.another.id] = value;
	};
	
	Statistics.prototype.__makeAnotherIndicatorValues = function(values, indicator) {
		var self = this;
		$.each(values || [], function(_, value) {
			self.__makeAnotherIndicator(value.indicators, indicator);
		});
	};
	
	Statistics.prototype.__getIndicators = function() {
		var self = this;
		var data = this.selectedSlice.GetData(self.GetData(), self.now);
		//make "another" indicators
		$.each(this.definition, function(_, indicator) {
			if (!indicator.another) {
				return;
			}
			self.__makeAnotherIndicator(data.total, indicator);
			self.__makeAnotherIndicator(data.lastTotal, indicator);
			if (data.values) {
				self.__makeAnotherIndicatorValues(data.values, indicator);
			}
		});
		return data;
	};
	
	Statistics.prototype.GetIndicators = function() {
		var name = this.selectedSlice.GetName();
		this.indicators[name] = this.indicators[name] || this.__getIndicators();
		return this.indicators[name];
	};
	
	Statistics.prototype.Now = function() {
		var campaignId = this.GetAccountData().campaign;
		var campaign = inConcert.app.getCampaign(campaignId);
		return campaign.now();
	};
	
	Statistics.prototype.CanCreateNewPost = function() {
		var account = this.GetAccountData();
		return inConcert.app.getSNPostsManager().CanCreateNewPost(account.campaign, account.account);
	};
	
	Statistics.prototype.NewPost = function() {
		var account = this.GetAccountData();
		inConcert.app.getSNPostsManager().ShowCreatePopup(account);
	};
	
	Statistics.prototype.Load = function() {
		this.preview = this.preview || new inConcert.snStatisticsUi.preview(this);
		this.__selectSlice(inConcert.social_networks.GetStatsSlice());
		this.preview.Show();
	};
	
	Statistics.prototype.ShowPopup = function() {
		this.popup = this.popup || new inConcert.snStatisticsUi.popup(this);
		this.popup.Show();
	};
	
	Statistics.prototype.__selectSlice = function(selectedSlice) {
		var found = false;
		$.each(this.slices, function(_, slice) {
			if (slice === selectedSlice) {
				found = true;
				return false;
			}
		});
		this.selectedSlice = found? selectedSlice : this.defaultSlice;
	};
	
	Statistics.prototype.SelectSlice = function(slice, callback) {
		callback = callback || $.noop;
		this.__selectSlice(slice);
		if (this.preview) {
			this.preview.Reload();
		}
		if (this.popup) {
			this.popup.Reload();
		}
		callback();
	};

}());