/*jshint quotmark:false*/
/*global $, inConcert, FB*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.snStatistics) {
	inConcert.snStatistics = {};
}
if (!inConcert.snStatistics.post) {
	inConcert.snStatistics.post = {};
}

(function () {
/*jshint validthis:true*/
"use strict";
	var format = inConcert.Utils.format;

	var definition = [
		{
			id : "interactions",
			name : "<%=_T('Interactions')%>",
			description : "<%=_T('Number of interactions created related to this post.')%>"
		},
		{
			id : "people_reached",
			name : "<%=_T('People reached')%>",
			description : "<%=_T('Number of people reached by facebook post (estimated).')%>"
		},
		{
			id : "action_comment",
			name : "<%=_T('Comments')%>",
			description : "<%=_T('Number of post comments and replies.')%>"
		},
		{
			id : "action_share",
			name : "<%=_T('Share')%>",
			description : "<%=_T('Number of times post was shared.')%>"
		},
		{
			id : "reaction",
			name : "<%=_T('Reactions')%>",
			description : "<%=_T('Number of reactions (like, love, etc.) to post.')%>",
			total : "reaction_total",
			hide_empty : true,
			items : [
				{
					id : "reaction_like",
					name : "<%=_T('Like')%>",
					description : "<%=_T('Number of post like reactions.')%>",
					color : "#1872b2"
				},
				{
					id : "reaction_love",
					name : "<%=_T('Love')%>",
					description : "<%=_T('Number of post love reactions.')%>",
					color : "#D62728"
				},
				{
					id : "reaction_wow",
					name : "<%=_T('Wow')%>",
					description : "<%=_T('Number of post wow reactions.')%>",
					color : "#E8A032"
				},
				{
					id : "reaction_haha",
					name : "<%=_T('Haha')%>",
					description : "<%=_T('Number of post haha reactions.')%>",
					color : "#FF7F0E"
				},
				{
					id : "reaction_sorry",
					name : "<%=_T('Sorry')%>",
					description : "<%=_T('Number of post sorry reactions.')%>",
					color : "#2CA02C"
				},
				{
					id : "reaction_anger",
					name : "<%=_T('Anger')%>",
					description : "<%=_T('Number of post anger reactions.')%>",
					color : "#6F257F"
				}
			]
		},
		{
			id : "consumption",
			name : "<%=_T('Consumption')%>",
			description : "<%=_T('Number of times post resources were consumed.')%>",
			total : "consumption_total",
			hide_empty : true,
			items : [
				{
					id : "consumption_link_click",
					name : "<%=_T('Link Click')%>",
					description : "<%=_T('Number of times post link was clicked.')%>",
					color : "#D62728"
				},
				{
					id : "consumption_photo_view",
					name : "<%=_T('Photo View')%>",
					description : "<%=_T('Number of times post photo was opened.')%>",
					color : "#E8A032"
				},
				{
					id : "consumption_video_play",
					name : "<%=_T('Video Play')%>",
					description : "<%=_T('Number of times post video was played.')%>",
					color : "#FF7F0E"
				},
				{
					id : "consumption_other_click",
					name : "<%=_T('Other Click')%>",
					description : "<%=_T('Number times post was clicked in another places (comment, view actions, view reactions, etc.).')%>",
					color : "#2CA02C"
				}
			]
		},
		{
			id : "negative",
			name : "<%=_T('Negative Feedback')%>",
			description : "<%=_T('Number of negative actions generated in base of post.')%>",
			total : "negative_total",
			hide_empty : true,
			items : [
				{
					id : "negative_hide",
					name : "<%=_T('Hide')%>",
					description : "<%=_T('Hide post in costumers wall.')%>",
					color : "#D62728"
				},
				{
					id : "negative_hide_all",
					name : "<%=_T('Hide all')%>",
					description : "<%=_T('Hide all page publications in costumers wall.')%>",
					color : "#E8A032"
				},
				{
					id : "negative_spam",
					name : "<%=_T('Spam')%>",
					description : "<%=_T('Post marked as spam.')%>",
					color : "#FF7F0E"
				},
				{
					id : "negative_unlike",
					name : "<%=_T('Unlike')%>",
					description : "<%=_T('Costumer unlike the page.')%>",
					color : "#2CA02C"
				}
			]
		}
	];

	function Statistics (rawData)
	{
		var slices = [];
		if (!rawData.post.IsClosed() && !rawData.post.IsDeleted()) {
			slices.push(inConcert.snStatisticsSlices.today);
			slices.push(inConcert.snStatisticsSlices.yesterday);
		}
		slices.push(inConcert.snStatisticsSlices.last7days);
		slices.push(inConcert.snStatisticsSlices.last28days);
		slices.push(inConcert.snStatisticsSlices.total);
		Statistics.uber.constructor.call(this, definition, slices, inConcert.snStatisticsSlices.last7days, rawData);
	}
	
	inConcert.Utils.inherit(Statistics, inConcert.snStatistics.post.base);
	if (typeof inConcert.snStatistics.post.facebook !== "function") {
		inConcert.snStatistics.post.facebook = Statistics;
	}
	
	Statistics.prototype.GetClassName = function() {
		return "fbPost";
	};
	
	Statistics.prototype.GetContent = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.post.IsDeleted() || self.post.IsScheduled()) {
			return Statistics.uber.GetContent.call(self, callback);
		}
		self.post.GetEmbedHtml(function(response) {
			if (!response || !response.html) {
				return Statistics.uber.GetContent.call(self, callback);
			}
			var $html = $(format("<div id='embed_{0}'  class='embedFBPostContainer'></div>", self.post.id));
			$html.append(response.html);
			callback({
				$html : $html
			});
		});
	};
	
	Statistics.prototype.AfterUiLoad = function(callback = $.noop) {
		let self = this;
		self.post.LoadApi(function(status) {
			if (status) {
				FB.XFBML.parse($("#embed_" + self.post.id)[0]);
			}
			callback();
		});
	};

}());