/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!this.inConcert.snStatisticsUi) {
	inConcert.snStatisticsUi = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.social_networks;
	
	var CONTENT_CHARACTERS_LIMIT = 200;
	var INCREMENT_LIMIT = 1000;
	
	//Statistics UI
	function StatisticsUi (parent)
	{
		this.parent = parent;
	}
	
	if (!inConcert.snStatisticsUi.base) {
		inConcert.snStatisticsUi.base = StatisticsUi;
	}
	
	StatisticsUi.prototype.Show = function(callback) {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.Close = function(callback) {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.Reload = function(callback) {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__getViewName = function() {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__getPieChartSize = function() {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__getLineChartSize = function() {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__getBarsChartSize = function() {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__makeBody = function(callback) {
		throw "not implemented";
	};
	
	StatisticsUi.prototype.__getCompareChartType = function() {
		return "BARS";
	};
	
	StatisticsUi.prototype.__make = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.__makeBody(function($body) {
			self.__makeSliceSelector($body);
			self.__makeStatistics($body);
			callback($body);
		});
	};
	
	StatisticsUi.prototype.__makeSliceSelector = function($body) {
		var self = this;
		var slices = "";
		var selected = self.parent.GetSelectedSlice();
		$.each(self.parent.GetSlices(), function(index, slice) {
			slices += format("<option value='{0}' {1}>{2}</option>",
				index, slice === selected? "selected" : "", slice.GetName());
		});
		$body.find("select[range]")
			.html(slices)
			.change(function() {
				inConcert.social_networks.SelectStatsSlice(self.parent.GetSlices()[$(this).val()]);
			});
	};
	
	StatisticsUi.prototype.__makeContent = function($body, callback) {
		var self = this;
			
		function makeContent(content, isQuoted) {
			var $content;
			if (isQuoted) {
				$content = $(Mustache.render(templates.statistics_post_quoted_content, {
					name : content.user? content.user.name : "",
					avatar : content.user? content.user.avatar : ""
				}));
			}
			else {
				$content = $(templates.statistics_post_content);
			}
			self.__makeText($content, content);
			if (content.multimedia) {
				self.__makeMultimedia($content, content.multimedia);
			}
			if (!isQuoted && content.quoted) {
				$content.append(makeContent(content.quoted, true));
			}
			return $content;
		}
		
		if (!self.parent.StatisticsEnabled()) {
			$body.filter("div[stats_body]").children().hide();
		}
			
		self.parent.GetContent(function(content) {
			if (content) {
				var $content = content.$html? content.$html : makeContent(content);
				$body
					.filter("div[stats_body]")
					.prepend($content);
			}
			callback();
		});
	};
	
	StatisticsUi.prototype.__makeTextHtml = function(text) {
		var escAttrib = inConcert.Utils.escapeAttrib;
		var linkify = inConcert.Utils.linkify;
		var socialMedia = inConcert.Utils.socialMedia;
		var html = socialMedia(linkify(escAttrib(text)));
		return html;
	};
	
	StatisticsUi.prototype.__makeText = function($content, content) {
		var self = this;
		content.text = content.text || "";
		var $text = $(Mustache.render(templates.statistics_post_text, { text : content.text }));
		var $label = $text.find("label");
		var $viewMore = $text.find("a[view_more]");
		if (!content.processedText && (content.text.length > CONTENT_CHARACTERS_LIMIT)) {
			var truncated = content.text.substring(0, CONTENT_CHARACTERS_LIMIT);
			$label.append(self.__makeTextHtml(truncated));
			$viewMore.click(function() {
				$text.removeClass("truncated");
				$viewMore.remove();
				$label.html("").append(self.__makeTextHtml(content.text));
			});
		}
		else {
			$viewMore.remove();
			$label.append(content.processedText || self.__makeTextHtml(content.text));
		}
		$content.append($text);
	};
	
	StatisticsUi.prototype.__makeMultimedia = function($content, multimedia) {
		var $multimedia = [];
		$.each(multimedia || [], function(_, item) {
			switch(item.type) {
				case "image":
					$multimedia.push($(Mustache.render(templates.statistics_post_image, item)));
					break;
				case "link":
					$multimedia.push($(Mustache.render(templates.statistics_post_image_link, item)));
					break;
				case "video":
					$multimedia.push($(Mustache.render(templates.statistics_post_video, item)));
					break;
			}
		});
		$content.append($multimedia);
	};
	
	StatisticsUi.prototype.__makeNewPost = function($body) {
		var self = this;
		var $button = $("#snButtonCreate");
		
		if (!self.parent.CanCreateNewPost()) {
			$button.hide();
			$button.off('click');
			return;
		}
		$button.show();
		$button
			.off('click')
			.on('click', function() {
				self.parent.NewPost();
		});
	};
	
	StatisticsUi.prototype.__makeStatistics = function($body) {
		var self = this;
		inConcert.charts.cleanView(this.__getViewName());
		$body.find("label[update_date]").html(self.parent.GetUpdateDate());
		var data = self.parent.GetIndicators();
		var $indicators = [];
		$.each(self.parent.GetDefinition(), function(_, definition) {
			$indicators.push(self.__makeIndicatorStatistics(data, definition));
		});
		$body.find("div[list]").append($indicators);
	};
	
	StatisticsUi.prototype.__makeIndicatorStatistics = function(data, definition, subitem, hideEmpty) {
		var self = this;
		var key = definition.total || definition.id;
		var total = self.__getTotal(data, key);
		if (total === 0 && hideEmpty) {
			return null;
		}
		var $indicator = $(Mustache.render(templates.statistics_item, {
			id : definition.id,
			name : definition.name,
			description : definition.description,
			iconClass : "icon-indicator" + definition.id,
			total : total,
			totalDescription : "<%=_T('Current period total.')%>"
		}));
		self.__makeTrend($indicator, data, key, total);
		self.__makeLineChart($indicator, data, definition, key);
		if (subitem && definition.color) {
			$indicator
				.find("[name]")
				.css("color", definition.color);
		}
		else {
			$indicator.find("[statistic]").hover(
				function() {
					$(this).addClass("hover");
				},
				function() {
					$(this).removeClass("hover");
				});
		}
		self.__makeDetails($indicator, data, definition, total, subitem);
		return $indicator;
	};	
	
	StatisticsUi.prototype.__getTotal = function(data, key) {
		return data.total[key] || 0;
	};
	
	StatisticsUi.prototype.__getIncrement = function(data, key, total) {
		if (!data.lastTotal) {
			return null;
		}
		var last = data.lastTotal[key] || 0;
		if (last === total) {
			return 0;
		}
		if (last === 0) {
			total++;
			last = 1;
		}
		return Math.round(100 * (total - last) / last);
	};
	
	StatisticsUi.prototype.__makeTrend = function($indicator, data, key, total) {
		var increment = this.__getIncrement(data, key, total);
		var $trend = $indicator.find("label[trend]");
		if (increment === null) {
			$trend.remove();
			return;
		}
		var className = "neutral";
		if (increment > 0) {
			className = "positive";
			increment = increment < INCREMENT_LIMIT? increment : ("+" + INCREMENT_LIMIT);
		}
		else if (increment < 0) {
			className = "negative";
			increment = increment > -INCREMENT_LIMIT? (-1 * increment) : ("-" + INCREMENT_LIMIT);
		}
		$trend
			.html( "(" + increment + "% )")
			.addClass(className)
			.attr("title", "<%=_T('Compared with previous period.')%>");
	};
	
	StatisticsUi.prototype.__makeLineChart = function($indicator, data, definition, key) {
		var $chart = $indicator.find("div[line_chart]");
		if (!data.values || !data.values.length) {
			$chart.remove();
			return;
		}
		var chartItems = [];
		$.each(data.values, function(index, item) {
			var value = {
				x : {
					value : index,
					label : item.label,
					shortLabel : item.shortLabel
				},
				y : {
					value : item.indicators && item.indicators[key] || 0
				}
			};
			chartItems.push(value);
		});
		inConcert.charts.line($chart, chartItems, {
			viewName : this.__getViewName(),
			size : this.__getLineChartSize(),
			color : definition.color,
			xAxis : false,
			yAxis : false,
			responsive: true,
			margin : {
				left : 5,
				right : 5, 
				top : 5, 
				bottom : 5 
			}
		});
	};

	StatisticsUi.prototype.__closeStatsDetails = function($body) {
		const $button = $body.find("#snStatsPreviewTitle");
		$button.off("click")
			.on("click", function() {
				const selectedCategory = inConcert.social_networks.getSelectedCategory();
				inConcert.social_networks.__selectCategory(selectedCategory);
				$("#snStats").hide();
			});
	};

	StatisticsUi.prototype.__makeDetails = function($indicator, data, definition, total, subitem) {
		var $details = $indicator.find("div[details]");
		var $button = $indicator.find("button[view_details]");
		if (subitem || !definition.items || !definition.items.length || total === 0 && definition.hide_empty) {
			$details.remove();
			$button.remove();
			return;
		}
		var items = [];
		$.each(definition.items, function(_, item) {
			items.push(item);
		});
		if (definition.another) {
			items.push(definition.another);
		}
		this.__makeSubitems($details, data, items, definition.hide_empty);
		this.__makeCompareChart($details, data, items, total);
		$button.click(function() {
			$(this).toggleClass("selected");
			$details.toggle($(this).hasClass("selected"));
			$indicator.parent().scrollTo($indicator);
		});
	};
	
	StatisticsUi.prototype.__makeSubitems = function($details, data, items, hideEmpty) {
		var self = this;
		var $list = $details.find("div[sub_list]");
		var $items = [];
		$.each(items, function(_, item) {
			$items.push(self.__makeIndicatorStatistics(data, item, true, hideEmpty));
		});
		$list.append($items);
	};
	
	StatisticsUi.prototype.__makeCompareChart = function($details, data, items, total) {
		var $chart = $details.find("div[compare_chart]");
		if (total === 0) {
			$chart.remove();
			return;
		}
		var chartItems = [];
		$.each(items, function(_, item) {
			var value = data.total[item.id] || 0;
			if (value > 0) {
				chartItems.push({
					id : item.id,
					name : item.name,
					color : item.color,
					value : value
				});
			}
		});
		
		switch(this.__getCompareChartType()) {
			case "BARS":
				var size = this.__getBarsChartSize();
				$chart.attr("class", "statsBarChart");
				inConcert.charts.bars($chart, chartItems, {
					viewName : this.__getViewName(),
					size : size,
					percentages : true,
					id_key : "statistic",
					$references : $details.find("[statistic]")
				});
				break;
			case "PIE":
				$chart.attr("class", "statsCakeChart");
				inConcert.charts.pie($chart, chartItems, {
					viewName : this.__getViewName(),
					size : this.__getPieChartSize(),
					percentages : true,
					id_key : "statistic",
					$references : $details.find("[statistic]")
				});
				break;
		}
	};
	
	StatisticsUi.prototype.__afterLoad = function(callback) {
		this.parent.AfterUiLoad(callback);
	};

}());