/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.social_networks;
	
	var filters = [
		{
			id : "content",
			name : "<%=_T('Content')%>",
			filters : [
				{
					id : "text",
					type : "select_text",
					name : "<%=_T('Matching')%>",
					description : "<%=_T('Tweet contains text. You can search hashtags using # or mentions using @. Use quotes to literals.')%>",
					options : function() {
						return [
							{ id : "all", name : "<%=_T('All words')%>", text : true },
							{ id : "any", name : "<%=_T('Any word')%>", text : true }
						];
					},
					property: function(value) {
						if (!value.value) {
							return "";
						}
						if (value.category === "all") {
							return value.value;
						}
						var keywords = value.value.match(/\w+|"(?:\\"|[^"])+"/g);
						var text = "(";
						$.each(keywords, function(_, keyword) {
							text += (text !== ""? " OR " : "") + keyword;
						});
						text += ")";
						return text;
					}
				},
				{
					id : "exclude",
					type : "text",
					name : "<%=_T('Excluding')%>",
					description : "<%=_T('Tweet not contains text. You can search hashtags using # or mentions using @. Use quotes to literals.')%>",
					property: function(value) {
						if (!value.value) {
							return "";
						}
						var keywords = value.value.match(/\w+|"(?:\\"|[^"])+"/g);
						var text = "";
						$.each(keywords, function(_, keyword) {
							text += (text !== ""? " " : "") + "-" + keyword;
						});
						return text;
					}
				},
				{
					id : "since",
					type : "date",
					name : "<%=_T('Since')%>",
					description : "<%=_T('Date since tweet was sent.')%>"
				},
				{
					id : "until",
					type : "date",
					name : "<%=_T('Until')%>",
					description : "<%=_T('Date until tweet was sent.')%>"
				},
				{
					id : "language",
					type : "select",
					name : "<%=_T('Language')%>",
					description : "<%=_T('Tweet language.')%>",
					options : function() {
						var options = [];
						$.each(inConcert.app.cached_languages, function(_, language) {
							options.push({ id : language.id, name : language.name });
						});
						options.sort(function(a, b) {
							return a.name.localeCompare(b.name);
						});
						options.unshift({ id : "", name : "<%=_T('Any')%>" });
						return options;
					},
					defaultOption : function() {
						return inConcert.app.m_userLanguage;
					}
				},
				{
					id : "filter",
					type : "select",
					name : "<%=_T('Content')%>",
					description : "<%=_T('Tweet includes content.')%>",
					options : function() {
						return [
							{ id : "", name : "<%=_T('Any')%>" },
							{ id : "media", name : "<%=_T('Images and Videos')%>" },
							{ id : "images", name : "<%=_T('Images')%>" },
							{ id : "videos", name : "<%=_T('Videos')%>" },
							{ id : "links", name : "<%=_T('Links')%>" },
							{ id : "periscope", name : "<%=_T('Periscope')%>" },
							{ id : "vine", name : "<%=_T('Vine')%>" },
							{ id : "news", name : "<%=_T('News')%>" }
						];
					}
				},
				{
					id : "verified",
					type : "check",
					name : "<%=_T('Verified accounts')%>",
					description : "<%=_T('Include only tweets by verified accounts.')%>",
					property: function(value) {
						return value.value? "filter:verified" : "";
					}
				},
				{
					id : "include_replies",
					type : "check",
					name : "<%=_T('Include replies')%>",
					description : "<%=_T('Include replies (by default are excluded).')%>",
					property: function(value) {
						return value.value? "" : "exclude:replies";
					}
				}
			]
		},
		{
			id : "users",
			name : "<%=_T('Users')%>",
			filters : [
				{
					id : "from",
					type : "text",
					name : "<%=_T('By')%>",
					description : "<%=_T('Tweet was sent by user.')%>",
					clear : function(text) {
						return text.replace(/@/g, "");
					}
				},
				{
					id : "to",
					type : "text",
					name : "<%=_T('Reply To')%>",
					description : "<%=_T('Tweet was reply to user tweet.')%>",
					clear : function(text) {
						return text.replace(/@/g, "");
					}
				},
				{
					id : "mentions",
					type : "select_text",
					name : "<%=_T('Mentioning')%>",
					description : "<%=_T('Tweet mentions user.')%>",
					options : function() {
						return [
							{ id : "user", name : "<%=_T('User')%>", text : true },
							{ id : "me", name : "<%=_T('Me')%>" }
						];
					},
					property: function(value, data) {
						var name = "";
						switch(value.category || "") {
							case "me":
								name = inConcert.app.getSNPostsManager().GetAccountName(data.campaign, data.account);
								break;
							default:
								name = (value.value || "").replace(/@/g, "");
								break;
						}
						if (name) {
							return format("@{0}", name);
						}
						else {
							name = inConcert.app.getSNPostsManager().GetAccountName(data.campaign, data.account);
							return format("-@{0}", name);
						}
					}
				}
			]
		},
		{
			id : "engagement",
			name : "<%=_T('Engagement')%>",
			filters : [
				{
					id : "min_retweets",
					type : "number",
					name : "<%=_T('Retweets greater than')%>",
					description : "<%=_T('Has more retweets than enter value.')%>"
				},
				{
					id : "min_faves",
					type : "number",
					name : "<%=_T('Likes greater than')%>",
					description : "<%=_T('Has more likes than enter value.')%>"
				},
				{
					id : "min_replies",
					type : "number",
					name : "<%=_T('Replies greater than')%>",
					description : "<%=_T('Has more replies than enter value.')%>"
				}
			]
		}
	];
	
	var actions = [
		{
			action : "CREATE_INTERACTION",
			name : "<%=_T('Create interaction')%>",
			description : "<%=_T('Creates a new interaction with selected tweet.')%>",
			iconClassName : "createTweetInteraction",
			method : "create_from_search/"
		}
	];
	
	var actions_d = {};
	$.each(actions, function(_, action) {
		actions_d[action.action] = action;
	});
		
	function TwitterSearchBox (main, config)
	{
		this.main = main;
		this.isNew = !config;
		config = config || {};
		this.filters = config.filters || {};
		this.__make();
	}
	
	if (typeof inConcert.TwitterSearchBox !== "function") {
		inConcert.TwitterSearchBox = TwitterSearchBox;
	}
	
	TwitterSearchBox.prototype.GetName = function() {
		return this.filters.text || "";
	};
	
	TwitterSearchBox.prototype.__makeMenuItem = function() {
		var self = this;
		self.$menuItem = $(Mustache.render(templates.search_tweets_menu_item, {
			className : "column",
			name : self.GetName()
		}));
		self.$menuItem.click(function() {
			if (self.$box) {

				let newScrollPos   = 0;
				let diffPosScroll  = 0;
				let listPadding    = parseInt(self.main.$list().css('padding-right'));
				let listOffsetLeft = self.$menuItem[0].clientWidth + listPadding;
				let boxLeftPos     = self.$box.offset().left + self.main.$list().scrollLeft();
				let boxRightPos    = boxLeftPos + self.$box[0].clientWidth;
				let realScroll     = self.main.$list().scrollLeft() + listOffsetLeft;
				let listDivWidth   = self.main.$list()[0].clientWidth + realScroll;

				if (boxLeftPos >= realScroll && boxRightPos <= listDivWidth) {
					// Nothing to do so return
					return;
				}
				else if (boxLeftPos < realScroll && boxRightPos > listDivWidth) {
					// Invalid ranges
					console.warn("Invalid ranges. Set Position", self.$box.offset().left);
					newScrollPos = self.$box.offset().left;
				}
				else if (boxLeftPos < realScroll) {
					// Left Scrolling
					diffPosScroll = realScroll - boxLeftPos;
					newScrollPos  = self.main.$list().scrollLeft() - diffPosScroll;
				}
				else if (boxRightPos > listDivWidth) {
					// Right Scrolling
					diffPosScroll = boxRightPos - listDivWidth + listPadding;
					newScrollPos  = self.main.$list().scrollLeft() + diffPosScroll;
				}

				self.main.$list().animate({ scrollLeft: newScrollPos }, 1000);
			}
		});
		self.main.$menu().append(self.$menuItem);
	};
	
	TwitterSearchBox.prototype.__updateMenuItem = function() {
		this.$menuItem.find("[box_name]").text(this.GetName());
	};
	
	TwitterSearchBox.prototype.__makeEvents = function() {
		var self = this;
		var $filter = self.$box.find("[filter]");
		self.$box.find("[advanced_filter_button]").click(function() {
			self.$box.find("select[filter=text]").val("");
			self.$box.find("input[filter=text]").val("");
			self.$box.find("input[filter=exclude]").val("");
			self.$box.find("input[filter=since]" ).val("");
			self.$box.find("input[filter=until]" ).val("");
			self.$box.find("#verified").prop('checked', false);
			self.$box.find("#include_replies").prop('checked', false);
			self.$box.find("select[filter=language]").val("");
			self.$box.find("select[filter=filter]").val("");

			self.$box.find("input[filter=from]").val("");
			self.$box.find("input[filter=to]").val(""); 
			self.$box.find("select[filter=mentions]").val(""); 
			self.$box.find("input[filter=mentions]").val(""); 

			self.$box.find("input[filter=min_retweets]").val(""); 
			self.$box.find("input[filter=min_faves]").val(""); 
			self.$box.find("input[filter=min_replies]").val("");
		});
		$filter.find("[refresh_button]").click(function() {
			self.Search(self.results && self.results.since);
		});
		
		$filter.find("[filter_button]").click(function() {
			self.__showFilter();
		});
		
		$filter.find("[filter=account]")
			.on("paste", function() {
				self.__showFilter();
			})
			.on("keyup", function() {
				self.__showFilter();
			});
		
		$filter.find("[filter=basic]")
			.on("paste", function() {
				self.$box.find("[filter=text]").val($(this).val());
				self.__showFilter();
			})
			.on("keyup", function() {
				self.$box.find("[filter=text]").val($(this).val());
				self.__showFilter();
			});
		
		$filter.find("[filter=text]")
			.on("paste", function() {
				self.$box.find("[filter=basic]").val($(this).val());
			})
			.on("keyup", function() {
				self.$box.find("[filter=basic]").val($(this).val());
			});
		
		$filter.find("[close_button]").click(function() {
			self.Delete();
		});
		
		$filter.find("[update_button]").click(function() {
			self.__saveFilter();
		});
		
		$filter.find("[cancel_button]").click(function() {
			self.__closeFilter();
		});
	};
	
	TwitterSearchBox.prototype.__addTextFilter = function(filter) {
		var $filter = $(Mustache.render(templates.search_tweets_text, filter));
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addDateFilter = function(filter) {
		var $filter = $(Mustache.render(templates.search_tweets_date, filter));
		$filter.find("input[type=text]")
			.datepicker({
				dateFormat: "yy-mm-dd"
			});
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addNumberFilter = function(filter) {
		var $filter = $(Mustache.render(templates.search_tweets_number, filter));
		inConcert.Utils.numbersOnly($filter.find("input[type=text]"));
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addSelectFilter = function(filter) {
		var options = (typeof filter.options === "function")?
			filter.options() : filter.options;
		var options_s = "";
		$.each(options, function(_, option) {
			options_s += format("<option value='{0}'>{1}</option>", option.id, option.name);
		});
		var $filter = $(Mustache.render(templates.search_tweets_select, {
			id : filter.id,
			name : filter.name,
			description : filter.description,
			options : options_s
		}));
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addSelectTextFilter = function(filter) {
		var options = (typeof filter.options === "function")?
			filter.options() : filter.options;
		var options_s = "";
		var options_text = {};
		$.each(options, function(_, option) {
			options_s += format("<option value='{0}'>{1}</option>", option.id, option.name);
			options_text[option.id] = option.text || false;
		});
		var $filter = $(Mustache.render(templates.search_tweets_select_text, {
			id : filter.id,
			name : filter.name,
			description : filter.description,
			options : options_s
		}));
		$filter.find("select")
			.change(function() {
				$(this).siblings("input[type=text]").toggle(!!options_text[$(this).val()]);
			})
			.change();
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addCheckFilter = function(filter) {
		var $filter = $(Mustache.render(templates.search_tweets_check, filter));
		return $filter;
	};
	
	TwitterSearchBox.prototype.__addFilter = function(filter) {
		if (this.isNew && filter.defaultOption) {
			var defaultOption = (typeof filter.defaultOption === "function")?
				filter.defaultOption() : filter.defaultOption;
			this.filters[filter.id] = defaultOption;
		}
		switch(filter.type) {
			case "text":
				return this.__addTextFilter(filter);
			case "date":
				return this.__addDateFilter(filter);
			case "number":
				return this.__addNumberFilter(filter);
			case "select":
				return this.__addSelectFilter(filter);
			case "select_text":
				return this.__addSelectTextFilter(filter);
			case "check":
				return this.__addCheckFilter(filter);
		}
	};
	
	TwitterSearchBox.prototype.__addFilterGroup = function(group) {
		var self = this;
		var $group = $(Mustache.render(templates.search_tweets_advanced_group, group));
		var $title = $group.find("[advanced_filter_group_title]");
		var $list = $group.find("[advanced_filter_group_list]");
		$.each(group.filters, function(_, filter) {
			$list.append(self.__addFilter(filter));
		});
		$title.click(function() {
			$(this)
				.toggleClass("selected")
				.siblings("[advanced_filter_group_list]").toggle();
		});
		return $group;
	};
	
	TwitterSearchBox.prototype.__makeFilters = function() {
		var self = this;
		var $filters = self.$box.find("[advanced_filter_list]");
		$.each(filters, function(_, group) {
			$filters.append(self.__addFilterGroup(group));
		});
		$filters.find("[since]").append($filters.find("[until]").children());
		$filters.find("[until]").remove();
	};
	
	TwitterSearchBox.prototype.__showFilter = function() {
		if (this.IsProcessing() || this.optionsVisible) {
			return;
		}
		var $filter = this.$box.find("[filter]");
		
		$filter.find("[advanced_filter]").show();
		this.optionsVisible = true;
	};
	
	TwitterSearchBox.prototype.__getFilterValue = function(filter) {
		var $filter = this.$box.find(format("[filter={0}]", filter.id));
		switch(filter.type) {
			case "text":
			case "date":
			case "number":
			case "select":
				return { value : $filter.val() };
			case "select_text":
				var $select = $filter.filter("select");
				var $input = $filter.filter("input[type=text]");
				return { value : $input.val(), category : $select.val() };
			case "check":
				return { value : $filter.is(":checked") };
		}
	};
	
	TwitterSearchBox.prototype.__getFilterValues = function() {
		var self = this;
		var values = {};
		$.each(filters, function(_, group) {
			$.each(group.filters, function(_, filter) {
				var value = self.__getFilterValue(filter);
				if (value.value) {
					values[filter.id] = value.value;
				}
				if (value.category) {
					values[filter.id + "_category"] = value.category;
				}
			});
		});
		var campaignAccount = self.$box.find("[filter=account]").val();
		if (campaignAccount) {
			var campaignAccount_a = campaignAccount.split(":");
			values.campaign = campaignAccount_a[0];
			values.account = campaignAccount_a[1];
		}
		return values;
	};
	
	TwitterSearchBox.prototype.__setFilterValue = function(filter) {
		var $filter = this.$box.find(format("[filter={0}]", filter.id));
		var value = this.filters[filter.id] || "";
		switch(filter.type) {
			case "text":
			case "date":
			case "number":
			case "select":
				$filter.val(value);
				break;
			case "select_text":
				var $select = $filter.filter("select");
				var $input = $filter.filter("input[type=text]");
				$select.val(this.filters[filter.id + "_category"]).change();
				$input.val(value);
				break;
			case "check":
				$filter.prop("checked", !!value);
				break;
		}
	};
	
	TwitterSearchBox.prototype.__setFilterValues = function() {
		var self = this;
		$.each(filters, function(_, group) {
			$.each(group.filters, function(_, filter) {
				self.__setFilterValue(filter);
			});
		});
		if (self.filters.campaign && self.filters.account) {
			self.$box.find("[filter=account]").val(self.filters.campaign + ":" + self.filters.account);
		}
		self.$box.find("[filter=basic]").val(self.filters.text || "");
	};
	
	TwitterSearchBox.prototype.__setSearchFilter = function(filter) {
		var value = {
			value : this.filters[filter.id],
			category : this.filters[filter.id + "_category"]
		};
		if (filter.property) {
			var property = filter.property(value, this.filters);
			if (property) {
				this.searchFilters.text += (this.searchFilters.text !== ""? " " : "") + property;
			}
		}
		else if (value.value) {
			this.searchFilters[filter.id] = value.value;
		}
	};
	
	TwitterSearchBox.prototype.__setSearchFilters = function() {
		var self = this;
		self.campaign = self.filters.campaign;
		self.account = self.filters.account;
		self.searchFilters = {
			result_type : "recent",
			text : ""
		};
		$.each(filters, function(_, group) {
			$.each(group.filters, function(_, filter) {
				self.__setSearchFilter(filter);
			});
		});
	};
	
	TwitterSearchBox.prototype.__makeAccounts = function() {
		var accounts = inConcert.app.getSNPostsManager().GetAccounts(true, inConcert.TwitterSearchPermission);
		var options = "";
		$.each(accounts, function(_, account) {
			if (account.type === "TWITTER") {
				options += format("<option value='{0}'>{1}</option>", 
					account.campaign + ":" + account.account, account.name);
			}
		});
		return options;
	};
	
	TwitterSearchBox.prototype.__makeAction  = function(action) {
		var self = this;
		var $action = $(Mustache.render(templates.batch_action_button, action));
		$action.click(function() {
			self.__runBatchAction($(this).attr("action"));
		});
		return $action;
	};
	
	TwitterSearchBox.prototype.__makeSelectAll = function() {
		var self = this;
		var $selectAll = $(templates.batch_action_select_all_search);
		$selectAll.change(function() {
			self.$box
				.find("input[row_selector]")
				.prop("checked", $(this).prop("checked"));
			self.__updateBatchActions();
		});
		return $selectAll;
	};
	
	TwitterSearchBox.prototype.__makePagination = function() {
		var $pagination = $(templates.batch_action_pagination_tweet_search);
		return $pagination;
	};
	
	TwitterSearchBox.prototype.__makeBatchActions = function() {
		var self = this;
		var $actions = [];
		$actions.push(self.__makeSelectAll());
		$.each(actions, function(_, action) {
			$actions.push(self.__makeAction(action));
		});
		$actions.push(self.__makePagination());
		$actions.push($(templates.search_tweets_limit));
		self.$box.find("[actions]").append($actions);
	};
	
	TwitterSearchBox.prototype.__make = function() {
		var self = this;
		self.__makeMenuItem();
		
		self.$box = $(Mustache.render(templates.search_tweets_box, {
			accounts : self.__makeAccounts()
		}));
		self.main.$list().append(self.$box);
		self.__makeBatchActions();
		self.__makeFilters();
		self.__makeEvents();
		self.__setFilterValues();
		if (self.isNew) {
			self.$box.find("[refresh_button]").prop("disabled", true);
		}
		else {
			self.__setSearchFilters();
		}
		
		self.$menuItem.click();
	};
	
	TwitterSearchBox.prototype.__runBatchAction = function(actionId) {
		var self = this;
		if (self.IsProcessing()) {
			return;
		}
		var action = actions_d[actionId];
		if (!action) {
			return;
		}
		var tweets = [];
		self.$box.find("input[row_selector]:checked").each(function() {
			var id = $(this).attr("row_selector");
			if (self.results.tweets[id]) {
				tweets.push(self.results.tweets[id]);
			}
		});
		self.__processing(true);
		self.__message();
		var sn = inConcert.app.getSNPostsManager();
		var data = {
			proxyType : "TWITTER",
			campaignAtVCC : self.campaign,
			account : self.account
		};
		sn.RunBatchActionFromSearch(data, tweets, action, function(status, result) {
			self.__processing();
			if (status) {
				if (result !== "ABORTED" && result.unsuccessful_nodes.length) {
					self.__message("<%=_T('Action failed for some posts.')%>");
				}
				var $tweets = self.$box.find("[tweet]");
				$.each(result.successful_nodes, function(_, id) {
					var tweetId = id.replace(self.account + ":", "");
					var $tweet = $tweets.filter(format("[tweet={0}]", tweetId));
					$tweet.addClass("alredyInteraction");
					$tweet.find("[row_selector_container]").remove();
					$tweet.find("[disclaimer]").show();
				});
				self.__updateBatchActions();
			}
			else {
				self.__message(result);
			}
		});
	};
	
	TwitterSearchBox.prototype.GetConfiguration = function() {
		return {
			filters : this.filters
		};
	};
	
	TwitterSearchBox.prototype.Delete = function() {
		var self = this;
		if (self.IsProcessing()) {
			return;
		}
		
		function runAction() {
			self.$box.remove();
			self.$menuItem.remove();
			self.main.RemoveColumn(self, self.isNew);
		}
		
		if (self.isNew) {
			return runAction();
		}
		$.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			"<%=_T('Do you want to delete column?')%>",
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					return;
				}
				runAction();
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	TwitterSearchBox.prototype.__message = function(text, type) {
		text = text || "";
		type = type || "error";
		this.$box.find("[message]")
			.toggle(!!text)
			.attr("class", "genericFormMessagesContainer messageError mt-2 " + type)
			.find("label")
			.text(text);
	};
	
	TwitterSearchBox.prototype.__processing = function(processing) {
		this.processing = !!processing;
		this.$box.find("[loading]").toggle(processing);
	};
	
	TwitterSearchBox.prototype.IsProcessing = function() {
		return !!this.processing;
	};
	
	TwitterSearchBox.prototype.__closeFilter = function() {
		if (this.IsProcessing() || !this.optionsVisible) {
			return;
		}
		var $filter = this.$box.find("[filter]");
		$filter.find("[advanced_filter_group_title]").removeClass("selected");
		$filter.find("[advanced_filter_group_list]").hide();
		$filter.find("[advanced_filter]").hide();
		
		this.optionsVisible = false;
		this.__setFilterValues();
		this.__message();
	};
	
	TwitterSearchBox.prototype.__saveFilter = function() {
		var self = this;
		if (self.IsProcessing() || !self.optionsVisible) {
			return;
		}
		var values = self.__getFilterValues();
		if (!values.campaign || !values.account) {
			self.__message("<%=_T('Must select an account.')%>");
			return;
		}
		if (!values.text) {
			self.__message("<%=_T('Must select a matching condition.')%>");
			return;
		}
		self.filters = values;
		self.__message();
		self.__processing(true);
		self.main.SaveConfiguration(function() {
			self.isNew = false;
			self.$box.find("[refresh_button]").prop("disabled", false);
			self.__processing(false);
			self.__setSearchFilters();
			self.__updateMenuItem();
			self.Search();
		});
	};
	
	TwitterSearchBox.prototype.__clear = function(clearResults) {
		this.__closeFilter();
		this.__message();
		if (clearResults) {
			this.results = {
				tweets : {}
			};
			this.$box.find("[results]").html("");
			this.__setCount("");
		}
	};
	
	TwitterSearchBox.prototype.__updateBatchActions = function() {
		this.$box.find("[action]")
			.prop("disabled", this.$box.find("[row_selector]:checked").length === 0);
	};
	
	TwitterSearchBox.prototype.__makeTweetMultimedia = function(multimedia) {
		var $multimedia = [];
		$.each(multimedia || [], function(_, item) {
			switch(item.type) {
				case "image":
					$multimedia.push($(Mustache.render(templates.search_tweets_content_image, item)));
					break;
				case "link":
					$multimedia.push($(Mustache.render(templates.search_tweets_content_image_link, item)));
					break;
				case "video":
					$multimedia.push($(Mustache.render(templates.search_tweets_content_video, item)));
					break;
			}
		});
		return $multimedia;
	};
	
	TwitterSearchBox.prototype.__makeTweetContent = function(content, quoted) {
		var $content = $(Mustache.render(templates.search_tweets_content, {
			className : quoted? "quoted" : "",
			avatar : content.user.avatar,
			user : content.user.name,
			date : content.date
		}));
		//multimedia
		$content.append(this.__makeTweetMultimedia(content.multimedia));
		//text
		$content.append(Mustache.render(templates.search_tweets_content_text, {
			text : content.text
		}));
		//quoted
		if (!quoted && content.quoted) {
			$content.append(this.__makeTweetContent(content.quoted, true));
		}
		return $content;
	};
	
	TwitterSearchBox.prototype.__makeTweet = function(tweet) {
		var self = this;
		var id = tweet.id_str;
		var content = inConcert.app.getSNPostsManager().GetPostContent("TWITTER", tweet);
		var $tweet = $(Mustache.render(templates.search_tweets_item, { id : id }));
		//batch actions
		if (!tweet.interactionId) {
			$tweet.append(Mustache.render(templates.search_tweets_batch_selector, { id : id }));
			$tweet.find("input[row_selector]").click(function() {
				self.__updateBatchActions();
			});
		}
		//content
		$tweet.append(self.__makeTweetContent(content));
		
		var $disclaimer = $(Mustache.render(templates.search_tweets_item_disclaimer, {
			text : "<%=_T('Tweet already belongs to an interaction.')%>"
		}));
		$disclaimer.toggle(!!tweet.interactionId);
		$tweet
			.toggleClass("alredyInteraction", !!tweet.interactionId)
			.append($disclaimer);
		return $tweet;
	};
	
	TwitterSearchBox.prototype.__setCount = function(count) {
		this.$box.find("[pagination_label]").text(count? format("<%=_T('{0} tweets')%>", count) : "");
	};
	
	TwitterSearchBox.prototype.__processResults = function(tweets, since, max) {
		var self = this;
		var $list = self.$box.find("[results]");
		var $loadMore = $list.find("[load_more]");
		var $last = $list.find("[tweet]").last();
		var olderTweet = null;
		for (var i = tweets.length - 1; i >= 0; i--) {
			var tweet = tweets[i];
			self.results.tweets[tweet.id_str] = tweet;
			var $tweet = self.__makeTweet(tweet);
			if (max) {
				if (tweet.id_str === max) {
					continue;
				}
				$last.after($tweet);
			}
			else {
				$list.prepend($tweet);
			}
			olderTweet = olderTweet || tweet.id_str;
		}
		
		if (max) {
			if (olderTweet) {
				self.results.max = olderTweet;
			}
			else {
				$loadMore.remove();
			}
		}
		else if (tweets.length) {
			self.results.since = tweets[0].id_str;
			if (!since) {
				self.results.max = olderTweet;
			}
		}
		if (!since && !max) {
			if (tweets.length) {
				$loadMore = $(templates.search_tweets_load_more);
				$loadMore.click(function() {
					self.Search(null, self.results.max);
				});
				$list.append($loadMore);
			}
			else {
				$list.append($(templates.search_tweets_empty_search));
			}
		}
		
		this.__setCount($list.find("[tweet]").length);
	};
	
	TwitterSearchBox.prototype.__setApiLimits = function(limit, refreshTimestamp) {
		if (isNaN(limit) || isNaN(refreshTimestamp)) {
			return;
		}
		//me guardo 5 para evitar errores contra la API
		limit = limit > 5? (limit - 5) : 0;
		
		var $limits = this.$box.find("[limits]");
		$limits.show();
		$limits.find("[limit_label]").text(limit);
		$limits.find("[limit_refresh_label]").text(inConcert.Utils.formatDate(refreshTimestamp * 1000));
		this.main.SetApiLimits(this.campaign, this.account, limit, refreshTimestamp);
	};
	
	TwitterSearchBox.prototype.Search = function(since, max, callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.IsProcessing()) {
			return;
		}
		var exceed = self.main.CheckApiLimits(self.campaign, self.account);
		if (exceed) {
			self.__message(exceed);
			return;
		}
		self.__clear(!since && !max);
		self.__processing(true);
		self.searchFilters.since_id = since || null;
		self.searchFilters.max_id = max || null;
		var sn = inConcert.app.getSNPostsManager();
		sn.SearchTweets(self.campaign, self.account, self.searchFilters, function(status, tweets, limit, resetTime) {
			if (status) {
				self.__processResults(tweets || [], since, max);
				self.__setApiLimits(parseInt(limit), parseInt(resetTime));
				self.$box.find("[select_all_tweets]").prop('disabled', (tweets || []).length == 0);
			}
			else {
				self.__message("<%=_T('Error searching tweets.')%>");
				self.$box.find("[select_all_tweets]").prop('disabled', true);
			}
			self.__processing();
		});
	};

}());
