/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.social_networks;
	
	if (!inConcert.TwitterSearchPermission) {
		inConcert.TwitterSearchPermission = "sn_twitter_search";
	}
		
	function TwitterSearch ()
	{
		this.columns = [];
		this.limits = {};
	}
	
	if (typeof inConcert.TwitterSearch !== "function") {
		inConcert.TwitterSearch = TwitterSearch;
	}
	
	TwitterSearch.prototype.GetPermission = function() {
		return inConcert.TwitterSearchPermission;
	};
	
	TwitterSearch.prototype.HasPermission = function(campaign) {
		return inConcert.app.getSNPostsManager().HasPermission(campaign, this.GetPermission());
	};
	
	TwitterSearch.prototype.IsProcessing = function() {
		if (this.processing) {
			return true;
		}
		var processing = false;
		$.each(this.columns, function(_, column) {
			if (column.IsProcessing()) {
				processing = true;
				return false;
			}
		});
		return processing;
	};
	
	TwitterSearch.prototype.Show = function() {
		var self = this;
		var configuration = inConcert.app.getSNPostsManager().GetConfiguration().twitter_search || {};
		self.__make(configuration.columns || []);
		var buttonGoBack = `<button id="snGoBackList" class="goBackButtonText">
		<i class='fas fa-chevron-left'></i> <%=_T('Back')%>
		</button>`;
		self.$menu().before(buttonGoBack);
		self.__goBack();
	};
	
	TwitterSearch.prototype.$list = function() {
		return $("#snSearchContainer");
	};
	
	TwitterSearch.prototype.$menu = function() {
		return $("#snTwitterUniversalSearch").find("[options_list]");
	};
	
	TwitterSearch.prototype.__clean = function() {
		this.columns = [];
		this.$list().html("");
		this.$menu().html("");
		$("#snGoBackList").remove();

	};
	
	TwitterSearch.prototype.__make = function(columns) {
		var self = this;
		self.__clean();
		
		var $newColumn = $(Mustache.render(templates.search_tweets_menu_item, {
			className : "newColumn pageButtonGeneric buttonColorOrange pageButtonNew",
			name : "<%=_T('New column')%>"
		}));
		$newColumn.click(function() {
			self.__addColumn();
		});
		self.$menu().append($newColumn);
		$.each(columns, function(_, column) {
			self.__addColumn(column);
		});
	};

	TwitterSearch.prototype.__addColumn = function(config) {
		var column = new inConcert.TwitterSearchBox(this, config);
		this.columns.push(column);
	};

	TwitterSearch.prototype.__goBack = function() {
		$("#snGoBackList")
		.off("click")
		.on("click", function() {
			$("#snMenu").find("[category]").addClass("selected");
			$("#snTwitterUniversalSearch").removeClass("selected");
			$("#snSearchContainer").hide();
			$("#snPostsContainer").show();
			$("#snpageHeader").show();
			$("#snpageHeaderTwSearch").hide();
			$("#snMenu").show();
			$("#snGoBackList").remove();
		});
	};
	
	TwitterSearch.prototype.RemoveColumn = function(column, isNew) {
		var self = this;
		var currentColumns = self.columns;
		self.columns = [];
		$.each(currentColumns, function(_, currentColumn) {
			if (currentColumn !== column) {
				self.columns.push(currentColumn);
			}
		});
		if (!isNew) {
			self.SaveConfiguration();
		}
	};
	
	TwitterSearch.prototype.__processing = function(processing) {
		this.processing = !!processing;
		$("#snSearchLoading").toggle(this.processing);
	};
	
	TwitterSearch.prototype.SaveConfiguration = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var configuration = inConcert.app.getSNPostsManager().GetConfiguration().twitter_search || {};
		configuration.columns = [];
		$.each(self.columns, function(_, column) {
			configuration.columns.push(column.GetConfiguration());
		});
		self.__processing(true);
		inConcert.app.getSNPostsManager().UpdateConfiguration({ twitter_search : configuration }, function(status) {
			self.__processing(false);
			if (!status) {
				var error = "<%=_T('Error saving configuration.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: error, messageType: "Error", timeout: 5000 });
			}
			callback();
		});
	};
	
	TwitterSearch.prototype.SetApiLimits = function(campaign, account, limit, refreshTimestamp) {
		this.limits[campaign + ":" + account] = {
			limit : limit,
			refreshTimestamp : refreshTimestamp
		};
	};
	
	TwitterSearch.prototype.CheckApiLimits = function(campaign, account) {
		var limits = this.limits[campaign + ":" + account];
		if (!limits || limits.limit > 0 || new Date(limits.refreshTimestamp * 1000) < new Date()) {
			return;
		}
		return format("<%=_T('Search limits exceed. Wait until {0} to retry.')%>", 
			inConcert.Utils.formatDate(limits.refreshTimestamp * 1000));
	};

}());