local m_models = require("apps.supervisor.model"):GetModels()
local utils = require "shared.apps.utils"
local supervisorshared = require "supervisorshared"
local m_alarms = require "apps.supervisor.controllers.alarms"
local m_members = require "apps.supervisor.controllers.members"
local m_workgroups = require "apps.supervisor.controllers.workgroups"
require "LuaOrm.Date"

module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil

function MapUris (module, prefix, htmlRender)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	--TODO: hacer el control de permisos pertinente
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetSupervisorPanel , prefix .. "/get_supervisor_panel/" )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetSupervisorPanelAndMembersByCampaigns , prefix .. "/get_supview_and_members_by_campaigns/" )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetHistoricalData  , prefix .. "/get_historical_data/"  )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetHistoricalFlowData  , prefix .. "/get_historical_flows_data/"  )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetAvailablesProcessFlows  , prefix .. "/get_available_process_flows/"  )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. GetSystemIndicators, prefix .. "/get_system_indicators/")

	m_htmlRender = htmlRender
end

local function FormatAsTime (seconds, _T)
	_T = _T or function(s) return s end
	local days = math.floor(seconds / 86400)
	local hours = math.floor((seconds % 86400) / 3600)
	local minutes = math.floor((seconds % 3600) / 60)
	local secs = seconds % 60
	local strDays = ""
	local strHours = ""

	if days > 0 then
		strDays = days == 1 and _T("Day") or _T("Days")
		strHours = days .. " " .. strDays .. " " .. string.format("%02d", hours) .. ":"
	elseif hours > 0 then
		strHours = string.format("%02d", hours) .. ":"
	end
	return strHours .. string.format("%02d", minutes) .. ":" .. string.format("%02d", secs)
end

local function FormatAsPercentage (ratio)
	return ("%.2f%%"):format(100 * ratio)
end

local function getInboundChannels ()
	local channels = {
		["FACEBOOK"]     = true,
		["INSTAGRAM"]    = true,
		["YOUTUBE"]	     = true,
		["APPSTORE"]     = true,
		["GOOGLEPLAY"]   = true,
		["RCS"]          = true,
		["TWITTER"]      = true,
		["MAIL"]         = true,
		["WEBCONTACT"]   = true,
		["CHAT"]         = true,
		["IM"]           = true,
		["INBOUND_CALL"] = true,
		["CALL"]         = true,
		["VOICEMAIL"]    = true,
		["SMS"]          = true,
		["WHATSAPP"]     = true
	}
	return channels
end

local function getOutboundChannels ()
	local channels = {
		["OUTBOUND_CALL"] = true,
		["CALL"]          = true
	}
	return channels
end

local function getLiveChannels ()
	local channels = {
		["CHAT"]          = true,
		["IM"]            = true,
		["INBOUND_CALL"]  = true,
		["OUTBOUND_CALL"] = true,
		["CALL"]          = true
	}
	return channels
end

local function getSplittedChannels ()
	local channels = {
		["CALL"] = true
	}
	return channels
end

local function ChannelIsLive (channel)
	return getLiveChannels()[channel] or false
end

local function ChannelIsInbound (channel)
	return getInboundChannels()[channel] or false
end

local function ChannelIsOutbound (channel)
	return getOutboundChannels()[channel] or false
end

local function ChannelHasSplits (channel)
	return getSplittedChannels()[channel] or false
end

local function ChannelIsSplitted (channel)
	return channel:match("INBOUND_") or channel:match("OUTBOUND_") or channel == "VOICEMAIL"
end

local function GetSplitChannel (splitted)
	return splitted:gsub("OUTBOUND_", ""):gsub("INBOUND_", "")
end

local function AddRealtimeTables (t1, t2)
	t1 = t1 or {}
	t2 = t2 or {}
	return {
		total = (t1.total or 0) + (t2.total or 0),
		ofWorkgroups = (t1.ofWorkgroups or t2.ofWorkgroups) and ((t1.ofWorkgroups or 0) + (t2.ofWorkgroups or 0)) or nil
	}
end

local function TotalIndicatorsByCampaign (campaignView, _T, splitted)
	local indicators = {
		active         = 0,
		queued         = 0,
		waiting        = 0,
		goals          = 0,
		abandoned      = 0,
		forAbandoned   = 0,
		attentionTime  = 0,
		occupationTime = 0,
		occupationRate = 0
	}

	-- build the interactions totals for each channel
	for channel, interactions in pairs(campaignView.interactions) do
		-- inbound
		if interactions.inbound then
			indicators.active = indicators.active + (interactions.inbound.assignedOnWorkgroup or interactions.inbound.assigned)
			indicators.queued = indicators.queued + (interactions.inbound.queued or 0)
			indicators.waiting = indicators.waiting + (interactions.inbound.waiting or 0)
		end
		-- outbound
		if interactions.outbound then
			indicators.active = indicators.active + (interactions.outbound.assignedOnWorkgroup or interactions.outbound.assigned)
			indicators.queued = indicators.queued + (interactions.outbound.queued or 0)
			indicators.waiting = indicators.waiting + (interactions.outbound.waiting or 0)
		end
	end

	-- realtime summary
	local splittedAdded = {}
	for channel, realtime in pairs(campaignView.realtime) do 
		if realtime then
			--inbound
			if ChannelIsInbound(channel) then
				indicators.goals = indicators.goals + (realtime.inboundGoals.ofWorkgroups or realtime.inboundGoals.total)
				indicators.abandoned = indicators.abandoned + realtime.inboundAbandoned.total
				indicators.forAbandoned = indicators.forAbandoned + realtime.inboundClosed.total - realtime.ghost.total
			end
			--outbound
			if ChannelIsOutbound(channel) then
				indicators.goals = indicators.goals + (realtime.outboundGoals.ofWorkgroups or realtime.outboundGoals.total)
				indicators.abandoned = indicators.abandoned + realtime.outboundAbandoned.total
				indicators.forAbandoned = indicators.forAbandoned + realtime.outboundClosed.total
			end

			if splitted then
				local splitChannel = GetSplitChannel(channel)
				if not splittedAdded[splitChannel] then
					indicators.attentionTime = indicators.attentionTime + realtime.attentionTime.total
					indicators.occupationTime = indicators.occupationTime + realtime.occupationTime.total
					indicators.occupationRate = indicators.occupationRate + realtime.occupationRate.total
					splittedAdded[splitChannel] = true
				end
			end
		end
	end

	local result = {}

	local function AddResult (label, description, value, fn)
		value = value or 0
		table.insert(result, {
			ID				= label.."_campaign_total",
			description		= _T(description),
			total_source	= fn and fn == FormatAsPercentage and 100 * value or value,
			total			= fn and fn(value, _T) or tostring(value)
		})
	end

	AddResult("active", "Active", indicators.active)
	AddResult("queued", "Queued", indicators.queued)
	AddResult("waiting", "Waiting", indicators.waiting)
	AddResult("total", "Total", indicators.active + indicators.queued + indicators.waiting)
	AddResult("abandoned", "Abandoned %", indicators.forAbandoned > 0 and (indicators.abandoned / indicators.forAbandoned) or 0, FormatAsPercentage)
	AddResult("totalGoals", "Total Goals", indicators.goals)

	--In allegro these indicators are global
	if splitted then
		AddResult("attentionTime", "Time in conversation", indicators.attentionTime, FormatAsTime)
		AddResult("occupationTime", "Operator Occupancy", indicators.occupationTime, FormatAsTime)
		AddResult("occupationRate", "Occupancy %", indicators.occupationRate, FormatAsPercentage)
	end

	return result
end

local function IndicatorsByCampaignAndChannel (campaignView, channel, _T)
	local isVoicemail = channel == "VOICEMAIL"
	local isInbound = ChannelIsInbound(channel)
	local isOutbound = not isInbound and ChannelIsOutbound(channel)
	local isSplitted = ChannelIsSplitted(channel)
	local isOneChannel = utils.Cardinality(campaignView.interactions) == 1

	local interactions = campaignView.interactions[channel]
	local realtime = campaignView.realtime[channel:gsub(isOutbound and "OUTBOUND_" or "INBOUND_", "")]

	local inboundInfo = interactions.inbound or {}
	local outboundInfo = interactions.outbound or {}

	local active = { total = (inboundInfo.assigned or 0) + (outboundInfo.assigned or 0) }
	if inboundInfo.assignedOnWorkgroup or outboundInfo.assignedOnWorkgroup then
		active.ofWorkgroups = (inboundInfo.assignedOnWorkgroup or 0) + (outboundInfo.assignedOnWorkgroup or 0)
	end
	local queued = { total = (inboundInfo.queued or 0) + (outboundInfo.queued or 0) }
	local waiting = { total = (inboundInfo.waiting or 0) + (outboundInfo.waiting or 0) }
	local total = { total = (active.ofWorkgroups or active.total) + queued.total + waiting.total }
	local totalGoals = { total = 0 }

	local abandonedPercent = { total = 0 }
	local serviceLevel = { total = 0 }

	local answerTimeAverage = { total = 0 }
	local answerTimeAverageReopen = { total = 0 }
	local abandonTimeAverage = { total = 0 }
	local attentionTime = { total = 0 }
	local occupationTime = { total = 0 }
	local occupationRate = { total = 0 }
	local inboundAverageHandlingTime = { total = 0 }
	local inboundAverageHandlingTimeReopen = { total = 0 }
	local outboundAverageHandlingTime = { total = 0 }
	local outboundContactRate = { total = 0 }

	if realtime then
		local forAbandoned = 0
		local abandoned = 0
		--inbound indicators
		if isInbound then
			forAbandoned = realtime.inboundClosed.total - realtime.ghost.total
			abandoned = realtime.inboundAbandoned.total
			-- for realtime summary of SL, no data means service level at 100%.
			if realtime.inboundServiceLevel.total == 0 and realtime.inboundForServiceLevel.total == 0 then
				realtime.inboundServiceLevel.total = 1
			elseif realtime.inboundServiceLevel.ofWorkgroups == 0 and realtime.inboundForServiceLevel.ofWorkgroups == 0 then
				realtime.inboundServiceLevel.ofWorkgroups = 1
			end
			serviceLevel = realtime.inboundServiceLevel

			answerTimeAverage = realtime.inboundAverageAnswerSpeed
			answerTimeAverageReopen = realtime.inboundReopenAverageAnswerSpeed
			abandonTimeAverage = realtime.inboundAverageAbandonTime
			inboundAverageHandlingTime = realtime.inboundAverageHandlingTime
			inboundAverageHandlingTimeReopen = realtime.inboundReopenAverageHandlingTime
		end
		--outbound indicators
		if isOutbound then
			forAbandoned = forAbandoned + realtime.outboundClosed.total
			abandoned = abandoned + realtime.outboundAbandoned.total
			outboundContactRate = realtime.outboundContactRate
			outboundAverageHandlingTime = realtime.outboundAverageHandlingTime
		end
		--global indicators
		abandonedPercent = { total = forAbandoned > 0 and (abandoned / forAbandoned) or 0 }
		attentionTime = realtime.attentionTime
		occupationTime = realtime.occupationTime
		occupationRate = realtime.occupationRate
		totalGoals = AddRealtimeTables(realtime.inboundGoals, realtime.outboundGoals)
	end

	local lowerChannel = string.lower(channel)
	local result = {}

	local function AddResult (label, description, values, fn)
		values = values or {}
		values.total = values.total or 0
		table.insert(result, {
			ID                  = label.."_"..lowerChannel,
			description         = _T(description),
			total_source        = fn and fn == FormatAsPercentage and 100 * values.total or values.total,
			total               = fn and fn(values.total, _T) or tostring(values.total),
			ofWorkgroups_source = values.ofWorkgroups and (fn == FormatAsPercentage and 100 * values.ofWorkgroups or values.ofWorkgroups) or nil,
			ofWorkgroups        = values.ofWorkgroups and (fn and fn(values.ofWorkgroups, _T) or tostring(values.ofWorkgroups)) or nil
		})
	end

	AddResult("active", "Active", active)
	AddResult("queued", "Queued", queued)
	AddResult("waiting", "Waiting", waiting)
	AddResult("total", "Total", total)

	--only live for voicemail
	if isVoicemail then
		return result
	end

	--LogDebug("--1----[SUPERVISOR:IndicatorsByCampaignAndChannel]-------------------------> channel '%s' inboundInfo '%s'", channel, json.encode(inboundInfo or {}))

	AddResult("abandoned", "Abandoned %", abandonedPercent, FormatAsPercentage)
	if isInbound then
		AddResult("serviceLevel", "Service level", serviceLevel, FormatAsPercentage)
		AddResult("answerTime", "Average Answer time", answerTimeAverage, FormatAsTime)
		AddResult("abandonTime", "Average Abandon time", abandonTimeAverage, FormatAsTime)
		AddResult("inboundAverageHandlingTime", "Average Handling Time", inboundAverageHandlingTime, FormatAsTime)

		local elapsedOldestWaitingTime = 0
		if inboundInfo.oldestWaitingTimestamp and inboundInfo.oldestWaitingTimestamp > 0 then
			local nowUTCTime = LuaOrm.Date()
			nowUTCTimeSec = nowUTCTime:time()
			local campaignTimezone = utils.GetTimeZoneInstance():GetEnvironmentFormat(campaignView.timezone)
			local oldestWaitingUTCTime = utils.ConvertDateToUTCDate(inboundInfo.oldestWaitingTimestamp, nil, nil, true, campaignTimezone)
			local oldestWaitingUTCTimeSec = oldestWaitingUTCTime:time()
			elapsedOldestWaitingTime = os.difftime(nowUTCTimeSec, oldestWaitingUTCTimeSec) -- tiempo transcurrido

			--LogDebug("2---->[SUPERVISOR:IndicatorsByCampaignAndChannel] ->  (now UTC Time Sec) - (oldestWaitingTimestampUTC) = (elapsed Time seconds) -->  %s - %s = %s seconds.", nowUTCTimeSec, oldestWaitingUTCTimeSec, elapsedOldestWaitingTime)
		end
		local oldestWaiting = { total = elapsedOldestWaitingTime }

		AddResult("inboundOldestWaiting", "Longer Waiting time", oldestWaiting, FormatAsTime)
		--LogDebug("--3----[SUPERVISOR:IndicatorsByCampaignAndChannel]-------------------------------> oldestWaiting '%s'", json.encode(oldestWaiting or {}))

		--reopen indicators just apply to async channels
		if not ChannelIsLive(channel) then
			AddResult("answerTimeReopen", "Reopen Average Answer time", answerTimeAverageReopen, FormatAsTime)
			AddResult("inboundAverageHandlingTimeReopen", "Reopen Average Handling Time", inboundAverageHandlingTimeReopen, FormatAsTime)
		end
	end
	if isOutbound then
		AddResult("outboundContactRate", "Contact Rate %", outboundContactRate, FormatAsPercentage)
		AddResult("outboundAverageHandlingTime", "Average Handling Time", outboundAverageHandlingTime, FormatAsTime)
	end
	AddResult("totalGoals", "Total Goals", totalGoals)

	if not isSplitted or isOneChannel then
		-- some indicators just apply to live channels
		if ChannelIsLive(channel) then
			AddResult("attentionTime", "Time in ".. lowerChannel, attentionTime, FormatAsTime)
			AddResult("occupationTime", "Operator Occupancy", occupationTime, FormatAsTime)
		end
		AddResult("occupationRate", "Occupancy %", occupationRate, FormatAsPercentage)
	end

	return result
end

--- Will process and build the agent summary for a campaign
--
-- @param campaignView A table with the campaign supervisor view
-- @return agentsByStatus A table with the agent summary
local function processAgentsSummary (campaignView, filteredByWG, _T)
	local agentsByStatus = {
		ID = "agents_by_status",
		description = _T("Agents By Status"),
		indicators = {}
	}

	local loggedTable = {
		ID = "total_agents_logged",
		description = _T("Logged"),
		total = 0,
		ofWorkgroups = filteredByWG and 0 or nil
	}

	local function AddResult (label, description, state)
		table.insert(agentsByStatus.indicators, {
			ID = label,
			description = _T(description),
			total = campaignView.agentsPerState[state] or 0,
			ofWorkgroups = filteredByWG and (campaignView.filteredAgentsPerState[state] or 0) or nil
		})
		if state ~= "OFFLINE" then
			loggedTable.total = loggedTable.total + (campaignView.agentsPerState[state] or 0)
			if filteredByWG then
				loggedTable.ofWorkgroups = loggedTable.ofWorkgroups + (campaignView.filteredAgentsPerState[state] or 0)
			end
		end
	end

	AddResult("total_agents_idle", "Idle", "IDLE")
	AddResult("total_agents_break", "Break", "BREAK")
	AddResult("total_agents_busy", "Busy", "BUSY")
	AddResult("total_agents_busy_cross_campaign", "Busy In Another", "BUSY CROSS CAMPAIGN")
	AddResult("total_agents_attending", "Attending", "ATTENDING")
	AddResult("total_agents_attending_cross_campaign", "Attending Another", "ATTENDING CROSS CAMPAIGN")
	AddResult("total_agents_reserved", "Reserved", "RESERVED")
	AddResult("total_agents_reserved_cross_campaign", "Reserved Another", "RESERVED CROSS CAMPAIGN")
	AddResult("total_agents_noacd", "Online No ACD", "NOACD")

	table.insert(agentsByStatus.indicators, loggedTable)
	AddResult("total_agents_offline", "Offline", "OFFLINE")
	for _, t in pairs(agentsByStatus.indicators) do
		t.total_source = t.total
		t.total = tostring(t.total)
		if filteredByWG then
			t.ofWorkgroups_source = t.ofWorkgroups
			t.ofWorkgroups = tostring(t.ofWorkgroups)
		end
	end
	return agentsByStatus
end

--para las interacciones que corresponda, separa los datos en 2 cuentas: entrantes y salientes
local function SplitInboundAndOutboundIndicators (campaignResults, channel_properties)
	local function HasInbound(channel)
		return channel_properties[channel] and channel_properties[channel].inbound_part
	end

	local function HasOutbound(channel)
		return channel_properties[channel] and channel_properties[channel].outbound_part
	end

	local interactions = campaignResults.interactions or {}
	campaignResults.interactions = {}
	local splitted = false
	for channel, data in pairs(interactions) do
		if ChannelHasSplits(channel) then
			if HasInbound(channel) then
				campaignResults.interactions["INBOUND_" .. channel] = { inbound = data.inbound }
			end
			if HasOutbound(channel) then
				campaignResults.interactions["OUTBOUND_" .. channel] = { outbound = data.outbound }
			end
			splitted = true
		else
			if channel ~= "VOICEMAIL" or HasInbound("CALL") then
				campaignResults.interactions[channel] = data
			end
		end
	end
	return splitted
end

local function GetSupervisorPanelData (info, alarmsResult, _T, agentkeyspace)
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local campaignId = info.campaignId
	local accountId = info.account or ""
	local alId = info.attention_level or ""

	if utils.IsAllegroCampaign(campaignId, agentkeyspace) then
	-- Si la campana es de Allegro(Telefonia), NO se debe de pasar la account ni el AL.
		accountId = ""
		alId = ""
	end

	local viewResults, secondsOld = supervisorshared.DataFilter():FilterView
	{
		VCC = vcc,
		rowsPerPage = info.rows_per_page or 20,
		showOfflineAgents = info.showOfflineAgents,
		showKickouts = info.showKickouts,
		interactionTimes = m_alarms.HasInteractionTimes(info),
		campaignFilters = 
		{
			[campaignId] = {
				account				= accountId,
				attention_level		= alId,
				agent				= info.agentNameFilter,
				order				= info.agentOrder or "ID",
				page				= info.page or -1,
				realtimeWindow		= info.realtimeWindow,
				state				= info.agentStateFilter,
				workgroup			= info.workgroups
			}
		}
	}

	--make key for data entity
	local key
	if accountId == "" then
		key = campaignId
	elseif alId == "" then
		key = accountId
	else
		key = alId
	end

--LogDebug("[GetSupervisorPanelData]----1-----> viewResults: '%s'", json.encode(viewResults or {}))

	local campaignResults = viewResults and viewResults[key]
	if not campaignResults then
		local msg_err = string.format("Could not get supervisor panel for user: '%s', campaign: '%s' is not loaded yet", completeUsername, campaignId)
		LogError("[Supervisor:GetSupervisorPanel] - %s", msg_err)
		return false, { status = 404, result = "NOT_LOADED", reason = msg_err }
	end

	-- Asigno los valores de la campana que no lleguen en el campaignResults desde la supervisorshared (timezone)
	m_alarms.SetCampaignInfo(campaignId, vcc, campaignResults, agentkeyspace)

--LogDebug("[GetSupervisorPanelData] --> PRUEBA %s", json.encode(campaignResults))

	local splitted = SplitInboundAndOutboundIndicators(campaignResults, info.channel_properties or {})

	local result = {
		nowTimestamp    = campaignResults.nowTimestamp,
		realtimeSummary = {},
		realtimeCharts  = {},
		accountsData    = campaignResults.accountsData or {}
	}

	-- Build Agent Summary
	local filteredByWG = workgroups and #workgroups > 0
	table.insert(result.realtimeSummary, processAgentsSummary(campaignResults, filteredByWG, _T))

	-- Build "Campaign Total Summary"
	-- only if the campaign has multiple account configured
	-- Note: campaignResults.interactions is a hash table with the channels as keys
	if utils.Cardinality(campaignResults.interactions) > 1 then
		table.insert(result.realtimeSummary, {
			ID = 'campaignTotal',
			description = _T('Campaign Totals'),
			type = "entity",
			indicators = TotalIndicatorsByCampaign(campaignResults, _T, splitted)
		})
	end

	-- Build Summary by Channel
	for channel in pairs(campaignResults.interactions) do
		table.insert(result.realtimeSummary, {
			ID = string.lower(channel),
			description = _T(channel),
			type = "channel",
			indicators = IndicatorsByCampaignAndChannel(campaignResults, channel, _T)
		})
	end

	-- Special's charts
	local filtered_agents_per_state = campaignResults.filteredAgentsPerState

	table.insert(result.realtimeCharts,
	{
		id = "AgentStatusSummary",
		description = _T("Agent Summary by Status"),
		data = {
			{ "Idle" , filtered_agents_per_state["IDLE"]                     or 0, _T("Idle (without interactions)") },
			{ "Break", filtered_agents_per_state["BREAK"]                    or 0, _T("Break")                       },
			{ "Busy" , filtered_agents_per_state["BUSY"]                     or 0, _T("Busy (all channels full)")    },
			{ "Att"  , filtered_agents_per_state["ATTENDING"]                or 0, _T("Attending")                   },
			{ "Res"  , filtered_agents_per_state["RESERVED"]                 or 0, _T("Reserved")                    },
			{ "NACD" , filtered_agents_per_state["NOACD"]                    or 0, _T("Online No ACD")               },
			{ "BAn"  , filtered_agents_per_state["BUSY CROSS CAMPAIGN"]      or 0, _T("Busy In Another")             },
			{ "AAn"  , filtered_agents_per_state["ATTENDING CROSS CAMPAIGN"] or 0, _T("Attending Another")           },
			{ "RAn"  , filtered_agents_per_state["RESERVED CROSS CAMPAIGN"]  or 0, _T("Reserved Another")            }
		}
	})

	-- Se procesan todas las alarms de los INDICADORES
	m_alarms.ProcessAllIndicatorsAlarms(info, alarmsResult, result, campaignResults)

	return true, result
end

function GetSupervisorPanel (web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local campaignId = info.campaignId

	LogInfo("[Supervisor:GetSupervisorPanel] - User: '%s' is getting supervisor panel view for campaign: '%s'. Post data: '%s'.", completeUsername, campaignId, web.input["post_data"])

	-- Check required fields
	local ok, err = utils.HasFields(info, { campaignId = "string" })
	if not ok then
		web.status = 400
		LogError("[Supervisor:GetSupervisorPanel] - Invalid request - get supervisor panel for user: '%s', campaign: '%s' is not loaded yet. Err: '%s'.", completeUsername, campaignId, err)
		return { result = "INVALID_REQUEST", reason = err }
	end

	local camp_exists = m_models.Indicators.CampaignExists(campaignId)
	if not camp_exists then
		web.status = 404
		LogError("[Supervisor:GetSupervisorPanel] - Campaign does not exist - get supervisor panel for user: '%s', campaign: '%s' is not loaded yet", completeUsername, campaignId)
		return { result = "NOT_FOUND", reason = "Campaign: '"..campaignId.."' does not exist" }
	end

	local alarms = info.alarms and info.alarms.indicators or {}
	info.alarms = nil

	local account = info.account or ""
	local attention_level = info.attention_level or ""
	local result
	local alarmsResult = {}
	local isAllegroCampaign = utils.IsAllegroCampaign(campaignId)

	--campaign
	if not isAllegroCampaign and not utils.TableIsEmpty(alarms.campaign) then
		info.account = ""
		info.attention_level = ""
		info.alarms = alarms.campaign
		LogInfo("[Supervisor:GetSupervisorPanel] a- Retrieving alarms values for campaign: '%s'.", campaignId)
		local success, scopeResult = GetSupervisorPanelData(info, alarmsResult, _T)
		if success and account == "" then
			result = scopeResult
		end
	end

	--accounts
	for accountId, accountAlarms in pairs(alarms.accounts or {}) do
		info.account = accountId
		info.attention_level = ""
		if not utils.TableIsEmpty(accountAlarms.indicators) then
			info.alarms = accountAlarms.indicators
			--LogInfo("[Supervisor:GetSupervisorPanel] b- Retrieving alarms values for campaign: '%s', account: '%s'.", campaignId, accountId)
			local success, scopeResult = GetSupervisorPanelData(info, alarmsResult, _T)
			if success then
				if isAllegroCampaign or (account == accountId and attention_level == "") then
					result = scopeResult
				end
			end
		end
		--als
		if not isAllegroCampaign then
			for alId, alAlarms in pairs(accountAlarms.att_levels or {}) do
				if not utils.TableIsEmpty(alAlarms.indicators) then
					info.attention_level = alId
					info.alarms = alAlarms.indicators
					LogInfo("[Supervisor:GetSupervisorPanel] c- Retrieving alarms values for campaign: '%s', account: '%s', al: '%s'.", campaignId, accountId, alId)
					local success, scopeResult = GetSupervisorPanelData(info, alarmsResult, _T)
					if success and account == accountId and attention_level == alId then
						result = scopeResult
					end
				end
			end
		end
	end

	--get scope result if has not alarms
	if not result then
		--LogInfo("[Supervisor:GetSupervisorPanel] d- (without alarms) Retrieving values for current scope campaign: '%s'.", campaignId)
		info.account = account
		info.attention_level = attention_level
		info.alarms = nil
		local success
		success, result = GetSupervisorPanelData(info, {}, _T)
		if not success then
			web.status = 404
			LogError("[Supervisor:GetSupervisorPanel] - Campaign does not exist - get supervisor panel for user: '%s', campaign: '%s' is not loaded yet", completeUsername, campaignId)
			return result
		end
	end

	result.alarms = alarmsResult

	--LogInfo("[Supervisor:GetSupervisorPanel] Retrieving alarms for user: '%s' on campaign '%s', ALARMS: '%s'", completeUsername, campaignId, json.encode(result.alarms or {}))

	return result
end

local function ConvertHistoricalDataIntoPercent (data)
	local processedData = data
	return processedData
end

local function GetHistoricalIndicatorTemplate (_T)
	return {
		{
			id = "Interactions",
			description = _T("Conversations"),
			scope = {
				inbound = {
					summaryId = "Inbound.Total",
					summaryIdAttentionLevel = "InboundAttentionLevel.Total",
				},
				outbound = {
					summaryId = "Outbound.Total",
					summaryIdAttentionLevel = "OutboundAttentionLevel.Total",
				},
				all = {
					summaryId = "Interaction.Total",
					summaryIdAttentionLevel = "InteractionAttentionLevel.Total",
				}
			},
			default = 0
		},
		{
			id = "SLPositive",
			description = _T("Conversations with good service level"),
			scope = {
				inbound = {
					summaryId = "Inbound.SLPositive",
					summaryIdAttentionLevel = "InboundAttentionLevel.SLPositive",
				},
				all = {
					summaryId = "Inbound.SLPositive",
					summaryIdAttentionLevel = "InboundAttentionLevel.SLPositive",
				}
			},
			default = 0
		},
		{
			id = "GoodServiceLevel",
			description = _T("Conversations with good service level (%)"),
			scope = {
				inbound = {
					summaryId = "Inbound.ServiceLevel",
					summaryIdAttentionLevel = "InboundAttentionLevel.ServiceLevel",
				},
				all = {
					summaryId = "Inbound.ServiceLevel",
					summaryIdAttentionLevel = "InboundAttentionLevel.ServiceLevel",
				}
			},
			default = 100
		},
		{
			id = "UnansweredInteractions",
			description = _T("Unanswered Conversations"),
			scope = {
				inbound = {
					summaryId = "Inbound.Unanswered",
					summaryIdAttentionLevel = "InboundAttentionLevel.Unanswered",
				},
				outbound = {
					summaryId = "Outbound.Unanswered",
					summaryIdAttentionLevel = "OutboundAttentionLevel.Unanswered",
				},
				all = {
					summaryId = "Interaction.Unanswered",
					summaryIdAttentionLevel = "InteractionAttentionLevel.Unanswered",
				}
			},
			default = 0
		},
		{
			id = "Abandoned",
			description = _T("Abandoned Conversations"),
			scope = {
				inbound = {
					summaryId = "Inbound.Abandoned",
					summaryIdAttentionLevel = "InboundAttentionLevel.Abandoned",
				},
				outbound = {
					summaryId = "Outbound.Abandoned",
					summaryIdAttentionLevel = "OutboundAttentionLevel.Abandoned",
				},
				all = {
					summaryId = "Interaction.Abandoned",
					summaryIdAttentionLevel = "InteractionAttentionLevel.Abandoned",
				}
			},
			default = 0
		}
	}
end

local function GetHistoricalCut ( typeof, params )
	local cuts = {
		all = {
			campaign        = "SliceByCampaign",
			account         = "SliceByCampaignAccount",
			attention_level = "SliceByCampaignAccountAttentionLevel"
		},
		channel = {
			campaign        = "SliceByCampaignType",
			account         = "SliceByCampaignAccount",
			attention_level = "SliceByCampaignAccountAttentionLevel"
		}
	}
	local cut_type = cuts[typeof]
	if not cut_type then
		LogWarning("[Supervisor:GetHistoricalCut] - Unrecognized '%s' cut type.", typeof)
		return
	end
	local cut_name = "campaign"
	if params.account then
		cut_name = params.attention_level and "attention_level" or "account"
	end
	local cut = cut_type[cut_name]
	if not cut then
		LogWarning("[Supervisor:GetHistoricalCut] - Unrecognized '%s' cut name by type '%s'.", cut_name, typeof)
	end
	return cut
end

local function GetHistoricalTimeAgoConversation ( time_ago )
	local historical_time_ago_convertion = {
		["Hour"]  = 1,
		["2Hour"] = 2,
		["4Hour"] = 4,
		["8Hour"] = 8,
		["Day"]   = 24,
		["Week"]  = 24 * 7
	}
	local converstion_time_ago = historical_time_ago_convertion[ time_ago or "" ]
	if not converstion_time_ago then
		LogWarning("[Supervisor:GetHistoricalTimeAgoConversation] - Unrecognized '%s' time ago.", time_ago)
	end
	return converstion_time_ago
end

local function LoadHistoricalChannelsIndicators (indicatorsTable, params, _T, keyspace)
	local indicators_templates = GetHistoricalIndicatorTemplate(_T)
	params.indicators = indicators_templates
	params.cut = GetHistoricalCut("channel", params)
	params.hours_ago = GetHistoricalTimeAgoConversation(params.time_ago)

	local campaignAccountsTypes
	campaignAccountsTypes = m_models.Indicators.GetCampaignAccountTypes(params.campaign, params.vcc, keyspace)
	local indicatorsTemp = {}
	local i = 0

	for channel, _ in pairs(campaignAccountsTypes) do
		i = i +  1
		params.account_type = channel:gsub("INBOUND_", ""):gsub("OUTBOUND_", "")
		-- Go and get the data from Model
		params.initiative = ChannelIsOutbound(channel) and (ChannelIsInbound(channel) and "all" or "outbound") or "inbound"

		LogDebug("[Supervisor:LoadHistoricalChannelsIndicators] %d)---> Getting historical report for channel: '%s' on campaign: '%s', account: '%s', attention level: '%s', initiative: '%s'",
			i, channel, params.campaign, params.account, params.attention_level, params.initiative)

		local success, channelData = m_models.Indicators.GetHistoricalReport(params)
		if not success then
			-- error, me voy
			local err = channelData
			err.channel = channel
			return false, err
		end

		local processedData = ConvertHistoricalDataIntoPercent(channelData)
		-- The channel data is obtained with one report request
		-- for each indicator we need, after that, we extract the
		-- indicator we need to build one indicator with every channel inside
		for channelIndicatorId, data in pairs(processedData) do
			for _, indicator in ipairs(indicators_templates) do
				if not indicator.auxiliar and (indicator.scope[params.initiative]) then
					if not indicatorsTemp[indicator.id] then
						indicatorsTemp[indicator.id] = {
							id = indicator.id,
							description = indicator.description,
							containsSecondarySeries = true,
							data = {}
						}
					end
					if indicator.id == channelIndicatorId then
						indicatorsTemp[indicator.id].data[channel] = data
					end
				end
			end
		end
	end

	-- append every processed indicator
	for _, indicator in pairs(indicators_templates) do
		table.insert(indicatorsTable, indicatorsTemp[indicator.id])
	end

	return true, campaignAccountsTypes
end

local function LoadHistoricalIndicators ( indicatorsTable, params, channels, _T )
--	LogDebug("[Supervisor:LoadHistoricalIndicators] - Getting historical report data: '%s'", params)
--	LogDebug("[Supervisor:LoadHistoricalIndicators] - Getting historical report data: '%s'", json.encode(params))

	local function hasChannelInScope(scope)
		for channel, _ in pairs(channels) do
			local initiative = ChannelIsOutbound(channel) and (ChannelIsInbound(channel) and "all" or "outbound") or "inbound"
			if scope[initiative] then
				return true
			end
		end
		return false
	end

	local indicators_templates = GetHistoricalIndicatorTemplate(_T)
	params.cut = GetHistoricalCut("all", params)
	params.hours_ago = GetHistoricalTimeAgoConversation(params.time_ago)
	params.indicators = indicators_templates
	params.initiative = "all"

	local success, data = m_models.Indicators.GetHistoricalReport(params)
	if not success then
		local err = data
		return false, err
	end

	local processedData = ConvertHistoricalDataIntoPercent(data)
	if #indicatorsTable == 0 then
		for _, indicator in ipairs(indicators_templates) do
			if not indicator.auxiliar then
				table.insert(indicatorsTable, {
						id = indicator.id,
						description = indicator.description,
						containsSecondarySeries = true,
						data = { TOTAL = processedData[indicator.id] }
					}
				)
			end
		end
	else
		-- update result with processed data
		local result = {}
		for _, indicator in pairs(indicators_templates) do
			if not indicator.auxiliar and hasChannelInScope(indicator.scope) and processedData[indicator.id] then
				table.insert(result, processedData[indicator.id])
			end
		end
		for i, indicator in pairs(result) do
			if indicatorsTable[i] then
				indicatorsTable[i].data['TOTAL'] = indicator
			end
		end
	end
	-- ok
	return true
end

local function __getHistoricalData (info, vcc, keyspace, _T)
	_T = _T or m_htmlRender.extension.makeTextTranslator()
	local campaign_id = info.campaign_id
	local params = {
		vcc             = vcc,
		campaign        = campaign_id,
		-- Optional parameters
		account         = info.account,
		attention_level = info.attention_level,
		slice           = info.slice or "Hour",
		time_ago        = info.time_ago or "8Hour"
	}

	local result = {}
	local channels = {}
	if not info.account and not info.attention_level then
		-- Go and get the total by campaign and account type
		local success, info = LoadHistoricalChannelsIndicators(result, params, _T, keyspace)
		if success then
			channels = info
		else
			local err = info
			err.reason = "LOADING HISTORICAL CHANNELS INDICATORS"
			LogError("[Supervisor:GetHistoricalData] 2- Unable to retrieve Channels Historical Indicators - campaign: '%s' is not loaded yet. Err: '%s'.", campaign_id, err.result)
			return false, err, 400
		end
	else
		channels[info.channel] = true
	end

	-- Go and get the total by campaign
	local success, err = LoadHistoricalIndicators(result, params, channels, _T)
	if not success then
		err.reason = "LOADING HISTORICAL CAMPAIGN INDICATORS"
		LogError("[Supervisor:GetHistoricalData] 3- Unable to retrieve Campaigns Historical Indicators - campaign: '%s' is not loaded yet. Err: '%s'.", campaign_id, err.result)
		return false, err, 400
	end

	-- ok
	return result
end

function GetHistoricalData (web)
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local campaign_id = info.campaign_id

	LogInfo("[Supervisor:GetHistoricalData] - User: '%s' is getting historical data for campaign: '%s'. Post data: '%s'.", completeUsername, campaign_id, web.input["post_data"])

	-- Check required fields
	local ok, err = utils.HasFields(info, { campaign_id = "string" })
	if not ok then
		web.status = 400
		LogError("[Supervisor:GetHistoricalData] 1- Invalid request - getting historical data for user: '%s', campaign: '%s' is not loaded yet. Err: '%s'.", completeUsername, campaign_id, err)
		return {result = "INVALID_REQUEST", reason = err}
	end

	local result, err, code = __getHistoricalData(info, vcc)
	if not result then
		--Error
		web.status = code
		return err
	end

	return result
end


function GetHistoricalFlowData (web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local campaign_id = info.campaign


	local function parseDateToTable(dateStr)
		-- las fechas entrantes tienen este formato:2023-01-04T15:45:00.000Z y vienen por lo tanto en UTC
			local dateTable = {
				year  = 0,
				month = 0,
				day   = 0,
				hour  = 0,
				min   = 0
			}
			local parts = {}
			for part in string.gmatch(dateStr, "([^T]+)") do
				table.insert(parts, part)
			end
			local datePart = parts[1]
			local dateParts = {}
			for part in string.gmatch(datePart,"([^-]+)") do
				table.insert(dateParts, part)
			end
			dateTable.year = dateParts[1]
			dateTable.month = dateParts[2]
			dateTable.day = dateParts[3]
			local i,j = string.find(parts[2],":00.000Z")
			local timePart = string.sub(parts[2],1,i -1)
			local timeParts = {}
			for part in string.gmatch(timePart,"([^:]+)") do
				table.insert(timeParts,part)
			end
			dateTable.hour = timeParts[1]
			dateTable.min = timeParts[2]
			return dateTable
	end
	-- Check required fields
	local ok, err = utils.HasFields(info, { campaign = "string" })
	if not ok then
		web.status = 400
		LogError("[Supervisor:GetHistoricalFlowData] 1- Invalid request - getting historical data for user: '%s', campaign: '%s' is not loaded yet. Err: '%s'.", completeUsername, campaign_id, err)
		return {result = "INVALID_REQUEST", reason = err}
	end

	local customTime = false
	
	if info.time_option == "2Hour" then
		info.slice = "15Min"
		info.resultSize = 8
		info.time_ago = 2
	elseif info.time_option == "6Hour" then
		info.slice = "15Min"
		info.resultSize = 24
		info.time_ago = 6
	elseif info.time_option == "24Hour" then
		info.slice = "Hour"
		info.resultSize = 24
		info.time_ago = 24
	elseif info.time_option == "7Day" then
		info.slice = "Day"
		info.resultSize = 7
		info.time_ago = 24 * 7
	elseif info.time_option == "30Day" then
		info.slice = "Day"
		info.resultSize = 30
		info.time_ago = 24 * 30
	elseif info.time_option["from"] and info.time_option["to"] then
		--Resta entre to y from, dependiendo de valor se elige un slice u otro
		local parsedDateTo = LuaOrm.Date(parseDateToTable(info.time_option["to"]))
		local parsedDateFrom =  LuaOrm.Date(parseDateToTable(info.time_option["from"]))
		local seconds = (parsedDateTo:time() - parsedDateFrom:time())
		local hours = math.ceil(seconds / 60 / 60)
		-- LogDebug("[Supervisor:GetHistoricalFlowData] Difference: %s", hours)
		if hours < 24 then
			info.slice = "15Min"
			info.resultSize = hours * 4
		elseif hours < 24 * 7 then
			info.slice = "Hour"
			info.resultSize = hours 
		elseif hours <= 24 * 30 * 5 then
			info.slice = "Day"
			info.resultSize = math.ceil(hours / 24)
		else
			info.slice = "Month"
			info.resultSize = math.ceil(hours / (24 * 30))
		end
		customTime = true
	else
		web.status = 400
		LogError("[Supervisor:GetHistoricalFlowData] Invalid time option")
		return nil
	end

	---Generamos el reporte
	local now = LuaOrm.Date():toDifferentTimezone(inconcert.authUser.timezone)
	local sliceStartTable = {
		year  = now:year(),
		month = now:month(),
		day   = now:day(),
		hour  = now:hour(),
		min   = now:minute()
	}

	--Para generar las muestras vacías 
	local increment = { 1 }
	if info.slice == "15Min" then
		sliceStartTable.min = sliceStartTable.min - (sliceStartTable.min % 15)
		increment = { 0, 0, 0, 0, 15 }
	elseif info.slice == "30Min" then
		sliceStartTable.min = sliceStartTable.min - (sliceStartTable.min % 30)
		increment = { 0, 0, 0, 0, 30 }
	elseif info.slice == "Hour" then
		sliceStartTable.min = 0
		increment = { 0, 0, 0, 1 }
	elseif info.slice == "Day" then
		sliceStartTable.min = 0
		sliceStartTable.hour = 0
		increment = { 0, 0, 1 }
	elseif info.slice == "Month" then
		sliceStartTable.min = 0
		sliceStartTable.hour = 0
		increment = { 0, 1, 0 }
	end

	local nowSliceStart, fromSliceStart
	if not customTime then
		nowSliceStart = LuaOrm.Date(sliceStartTable)
		fromSliceStart = nowSliceStart:add(0, 0, 0, 0-info.time_ago)--HoursAgo
	else	
		nowSliceStart = LuaOrm.Date(parseDateToTable(info.time_option["to"])):toDifferentTimezone(inconcert.authUser.timezone)
		fromSliceStart =  LuaOrm.Date(parseDateToTable(info.time_option["from"])):toDifferentTimezone(inconcert.authUser.timezone)
		if info.slice == "15Min" then
			fromSliceStart = fromSliceStart:add(0,0,0,0,0-(fromSliceStart:minute() % 15), 0-fromSliceStart:second())
		elseif info.slice == "30Min" then
			fromSliceStart = fromSliceStart:add(0,0,0,0,0-(fromSliceStart:minute() % 30), 0-fromSliceStart:second())
		elseif info.slice == "Hour" then
			fromSliceStart = fromSliceStart:add(0,0,0,0,0-fromSliceStart:minute(), 0-fromSliceStart:second())
		elseif info.slice == "Day" then
			fromSliceStart = fromSliceStart:add(0,0,0,0-fromSliceStart:hour(),0-fromSliceStart:minute(), 0-fromSliceStart:second())
		elseif info.slice == "Month" then
			--LogDebug("[Supervisor JC] - Slice is Month")
			fromSliceStart = fromSliceStart:add(0,0,(0-fromSliceStart:day()) + 1,0-fromSliceStart:hour(),0-fromSliceStart:minute(), 0-fromSliceStart:second())
		end
	end
	
	local _T_Dummy = function(s) return s end, {}
	local idsToNames = nil
	local redis = utils.GetRedisClient()
	local authFilter = nil

	local all = {}
	local limit = 3
	local cont = 0
	local flows_to_fill = {}
	--for _, account in ipairs(info.accounts) do
		for _,flow in ipairs(info.flows) do
			cont = cont + 1
			if cont <= limit then
				local report = {
					sliceSize = info.slice,
					vcc = vcc,
					indicators = {"Interaction.Total", "Interaction.FlowTransferToAgent","Interaction.ProcessFlowDurationTime", "Interaction.IVRTime"},
					filter = {
						Campaign = info.campaign,
						VCC = vcc,
						DateTo = nowSliceStart:toString(),
						DateFrom = fromSliceStart:toString(),
						Account = flow.AccountId,
						ProcessFlow = flow.id
					},
					resultSize=info.resultSize,
					--resultSize=12,
					timezone = inconcert.authUser.timezone,
					reverse = true,
					cut = "SliceByCampaignProcessFlow"
				}
	
				--  LogDebug("[Supervisor JC] - Result Size  %s", info.resultSize)
				--  LogDebug("[Supervisor JC] - FROM %s", fromSliceStart:toString())
				--  LogDebug("[Supervisor JC] - TO %s Ref %s", nowSliceStart:toString(), info.time_option["to"])

				table.insert(flows_to_fill, flow.id)
				local result, handle, err = reporter.QuerySummary(report, utils, _T_Dummy, idsToNames, redis, authFilter)
				--local result, handle, err = reporter.QuerySummaryFromCache(report, utils, _T_Dummy, idsToNames, redis, authFilter)
				
				if not result then
					--err.reason = "LOADING HISTORICAL CAMPAIGN INDICATORS"
					web.status = 400
					LogError("[Supervisor:GetHistoricalFlowData] 3- Unable to retrieve Campaigns Historical Indicators - campaign: '%s' is not loaded yet. Err: '%s'.", campaign_id, err.result)
					return err
				end
	
				local resultFilled = {}
				for idx, indicator in ipairs(report.indicators) do
					resultFilled[indicator] = {}
				end

				function copy(t)
					local cop = {}
					for k,v in pairs(t) do
						cop[k] = v
					end
					return cop
				end

				local pool = copy(result.rows)
				local sliceStart = fromSliceStart
				while sliceStart:time() < nowSliceStart:time() do
					local candidateRow
					while #pool > 0 do
						local lastRow = pool[#pool] -- This is the earliest one
						local slicePeriod = lastRow[1]
						if info.slice == "15Min" or info.slice == "30Min" or info.slice == "Hour" then
							-- It comes as "YYYY-mm-dd HH:MM - HH':MM'". Convert it to a YYYY-mm-dd HH:MM:00.
							slicePeriod = slicePeriod:match("^(.*) %- ")..":00"
						end
						if info.slice == "Month" then
							-- It comes as "YYYY-mm'". Convert it to a YYYY-mm-dd HH:MM:00.
							slicePeriod = slicePeriod .. "-01 00:00:00"
						end
						local timeDate = LuaOrm.Date(slicePeriod)
						local timestamp = timeDate:time()
						if timestamp > sliceStart:time() then
							break
						end
						-- LogDebug("[Supervisor] - Processed data for %s (at %s)", tostring(timeDate), tostring(sliceStart))
						pool[#pool] = nil
						if timestamp == sliceStart:time() then
							-- LogDebug("[Supervisor] - Candidate row is %s (at %s)", tostring(timeDate), tostring(sliceStart))
							candidateRow = lastRow
							break
						end
					end
					-- if candidateRow then
					-- 	LogDebug("[Supervisor] - Candidate row for %s is %s", tostring(sliceStart), json.encode(candidateRow))
					-- end
					for idx, indicator in ipairs(report.indicators) do
						-- The indicator value is in the column idx+1 of the candidate row, or the default if it does not exist.
						table.insert(resultFilled[indicator], { tostring(sliceStart), candidateRow and candidateRow[idx+1] or 0 })
					end
					sliceStart = sliceStart:add(unpack(increment))
				end

				result.filled = resultFilled

				result.summary = 0
				result.summaryTransfered = 0
				result.summaryTime = 0
				result.summaryBeforeTransferTime = 0
				for _, row in ipairs(result.rows) do
					result.summary = result.summary + row[2]
					result.summaryTransfered = result.summaryTransfered + row[3]
					result.summaryTime = result.summaryTime + row[4]
					result.summaryBeforeTransferTime = result.summaryBeforeTransferTime + row[5]
				end
	
				if result.summary > 0 then
					result.summaryPorcentTransfered = (result.summaryTransfered / result.summary) * 100
					result.averageTime = result.summaryTime / result.summary
					result.averageBeforeTransferTime = result.summaryBeforeTransferTime / result.summary
				else
					result.averageTime = 0
					result.summaryPorcentTransfered = 0
					result.averageBeforeTransferTime = 0
				end
	
				
	
				all[flow.id] = result
			else
				web.status = 502
				LogError("[Supervisor:GetHistoricalFlowData] Limit of Process Flow IDs excedeed.")
				return nil
			end
			
		end
	--end

	local flows_filled =  m_models.Indicators.GetProcessFlows(vcc,flows_to_fill)

	local processedResult = {
		TotalInteractions = {total = 0},
		AverageTimeMinutes = {total = 0},
		AverageTimeMinutesBeforeTransfer = {total = 0},
		TransferedInteractions = {total = 0},
		TransferedInteractionsPorcent = {total = 0},
		labels = {},
		datasets = {
			interactions = {},
			interactionsTransfered = {},
			interactionsPorcentTransfered = {},
			averageBeforeTransferedTime = {},
			averageTime = {}
		}
	}
	local aux_total_time = 0
	local aux_total_time_before_tranfer = 0
	local labels_processed = false
	for flowId, reportResult in pairs(all) do
		local flowName = flows_filled[flowId]["Name"] or flowId
		--LogDebug("[Supervisor Debug] - processing %s with report result %s", flowId, json.encode(reportResult))
		--Interacciones totales
		processedResult["TotalInteractions"]["total"] = processedResult["TotalInteractions"]["total"] + reportResult["summary"]
		processedResult["TotalInteractions"][flowName] = reportResult["summary"]
		--Interacciones totales con transferencia a Agente
		processedResult["TransferedInteractions"]["total"] = processedResult["TransferedInteractions"]["total"] + reportResult["summaryTransfered"]
		processedResult["TransferedInteractions"][flowName] = reportResult["summaryTransfered"]
		--Interacciones totales con transferencia en porcentaje con respecto al total
		processedResult["TransferedInteractionsPorcent"][flowName] = reportResult["summaryPorcentTransfered"]
		--Auxiliar de tiempo medio
		aux_total_time = aux_total_time + reportResult["summaryTime"]
		processedResult["AverageTimeMinutes"][flowName] = reportResult["averageTime"] / 60
		--Auxiliar de tiempo medio antes de transferencia
		aux_total_time_before_tranfer = aux_total_time_before_tranfer + reportResult["summaryBeforeTransferTime"]
		processedResult["AverageTimeMinutesBeforeTransfer"][flowName] = reportResult["averageBeforeTransferTime"] / 60

		-- processedResult["datasets"]["interactions"][flowName] = {data = {}, label = flowName}
		-- processedResult["datasets"]["interactionsTransfered"][flowName] = {data = {}, label = flowName}
		-- processedResult["datasets"]["averageTime"][flowName] = {data = {}, label = flowName}
		local interactionDataset = {data = {}, label = flowName}
		local interactionsTransferedDataset = {data = {}, label = flowName}
		local interactionsPorcentTransferedDataset = {data = {}, label = flowName}
		local averageTimeDataset = {data = {}, label = flowName}
		local averageTimeTransferedDataset = {data = {}, label = flowName}
		local lastLabelProcessed = nil
		for ind, row in ipairs(reportResult.filled["Interaction.Total"]) do
			if not labels_processed then
				--Parseamos la fecha dependiendo de su formato:
				--Si coincide con 00:00:00.000, retiramos la parte de los minutos
				--Si no, quitamos la parte de la fecha, siempre y cuando sea la misma que el día anterior
				local parts = {}
				for part in string.gmatch(row[1], "([^ ]+)") do
					table.insert(parts, part)
				end
				local datePart = parts[1]
				local dateParts = {}
				for part in string.gmatch(datePart,"([^-]+)") do
					table.insert(dateParts, part)
				end
				datePart = dateParts[3].."-"..dateParts[2].."-"..dateParts[1]
				local i,j = string.find(parts[2],":00.000")
				local timePart = string.sub(parts[2],1,i -1)
				LogDebug("[Supervisor] - Date part: %s, Time part: %s", datePart, timePart)
				if timePart == "00:00" then
					-- Fecha terminada a 00.00
					table.insert(processedResult["labels"], datePart)
				else
					-- Fecha con minutos
					if lastLabelProcessed and lastLabelProcessed == datePart then
						table.insert(processedResult["labels"], timePart)
					else

						table.insert(processedResult["labels"], datePart .. " " .. timePart)
					end
				end
				lastLabelProcessed = datePart
			end
			table.insert(interactionDataset["data"], row[2])
			table.insert(interactionsTransferedDataset["data"], reportResult.filled["Interaction.FlowTransferToAgent"][ind][2])
			
			
			if(row[2] > 0) then
				table.insert(interactionsPorcentTransferedDataset["data"], (reportResult.filled["Interaction.FlowTransferToAgent"][ind][2] /  row[2]) * 100)
				table.insert(averageTimeDataset["data"], (reportResult.filled["Interaction.ProcessFlowDurationTime"][ind][2] / 60) / row[2]) 
				table.insert(averageTimeTransferedDataset["data"], (reportResult.filled["Interaction.IVRTime"][ind][2] / 60) / row[2]) 
			else
				table.insert(averageTimeDataset["data"], 0) 
				table.insert(averageTimeTransferedDataset["data"], 0) 
				table.insert(interactionsPorcentTransferedDataset["data"], 0)
			end
			
		end
		table.insert(processedResult["datasets"]["interactions"],interactionDataset)
		table.insert(processedResult["datasets"]["interactionsTransfered"],interactionsTransferedDataset)
		table.insert(processedResult["datasets"]["averageTime"],averageTimeDataset)
		table.insert(processedResult["datasets"]["averageBeforeTransferedTime"],averageTimeTransferedDataset)
		table.insert(processedResult["datasets"]["interactionsPorcentTransfered"],interactionsPorcentTransferedDataset )
		if not labels_processed then
			labels_processed = true
		end
	end

	if processedResult["TotalInteractions"]["total"] > 0 then
		processedResult["TransferedInteractionsPorcent"]["total"] = (processedResult["TransferedInteractions"]["total"] /  processedResult["TotalInteractions"]["total"]) * 100
		processedResult["AverageTimeMinutes"]["total"] = (aux_total_time / processedResult["TotalInteractions"]["total"])/60
		processedResult["AverageTimeMinutesBeforeTransfer"]["total"] = (aux_total_time_before_tranfer / processedResult["TotalInteractions"]["total"])/60
	else
		processedResult["TransferedInteractionsPorcent"]["total"] = 0
		processedResult["AverageTimeMinutes"]["total"] = 0
		processedResult["AverageTimeMinutesBeforeTransfer"]["total"] = 0
	end

	
	-- ok
	return processedResult
end

function GetAvailablesProcessFlows (web)
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter

	if not info.accounts then
		web.status = 400
		LogError("[Supervisor:GetAvailableProcessFlows] 1- Invalid request .")
		return {result = "INVALID_REQUEST", reason = err}
	end

	--local flows_filled =  m_models.Indicators.GetProcessFlows(vcc,info.flowsIds)
	local flows_filled = m_models.Indicators.GetProcessFlows(vcc)
	
	local processed_flows = {}
	for flowId, flow in pairs(flows_filled) do
		local accountsList = m_models.Indicators.GetAccountsByFlow(vcc, flowId) or {}
		for _, account in pairs(accountsList) do
			for _, accountSelected in pairs(info.accounts) do
				if account.account_id == accountSelected then
					processed_flows[flowId] = {
						Type = flow.Type,
						Name = flow.Name
					}
				end
			end
		end
	end
	--m_models.Indicators.GetAccountsByFlow(vcc, flowId) or {}
	
	-- local processed_flows = {}
	-- for flowId, flowInfo in pairs(flows_filled) do
	-- 	processed_flows[flowId] = {
	-- 		Name = flowInfo["Name"],
	-- 		Type = flowInfo["Type"]
	-- 	}
	-- end

	return processed_flows
	--return processed_flows

end


function GetSystemIndicators (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[Supervisor:GetSystemIndicators] - User: '%s' is getting system indicators.", completeUsername)
	return m_alarms.LoadSystemIndicators()
end

function GetSupervisorPanelAndMembersByCampaigns (web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)

	--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] - User: '%s' is getting supervisor panel view for campaign batch: '%s'.", completeUsername, web.input["post_data"])

	-- Check required fields
	if not next(info) or not info.campaigns then
		web.status = 400
		LogError("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] - Invalid request")
		return { result = "INVALID_REQUEST" }
	end
	local agentkeyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local supervisor_views
	local campaign_workgroups = nil
	local campaignsToProcess = info.campaigns
	if info.addWorkgroups then
		campaign_workgroups = m_workgroups.GetWorkgroupsData(vcc, agentkeyspace) or nil
	end
	
	for fullCampaignId, campaignData in pairs(campaignsToProcess) do
		local campaignId = campaignData.campaignId
		local camp_exists = campaignId and m_models.Indicators.CampaignExists(campaignId, agentkeyspace)
		if camp_exists then

			-- Proceso las alarmas, etc
			local alarms = campaignData.alarms and campaignData.alarms.indicators or {}
			campaignData.alarms = nil

			local account = campaignData.account or ""
			local attention_level = campaignData.attention_level or ""
			local result
			local alarmsResult = {}
			
			local isAllegroCampaign = utils.IsAllegroCampaign(campaignId, agentkeyspace)

			--campaign
			if not isAllegroCampaign and not utils.TableIsEmpty(alarms.campaign) then
				campaignData.account = ""
				campaignData.attention_level = ""
				campaignData.alarms = alarms.campaign
				--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] a- Retrieving alarms values for campaign: '%s'.",campaignId)
				local success, scopeResult = GetSupervisorPanelData(campaignData, alarmsResult, _T, agentkeyspace)
				if success and account == "" then
					result = scopeResult
				end
			end

			--accounts
			for accountId, accountAlarms in pairs(alarms.accounts or {}) do
				campaignData.account = accountId
				campaignData.attention_level = ""
				if not utils.TableIsEmpty(accountAlarms.indicators) then
					campaignData.alarms = accountAlarms.indicators
					--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] b- Retrieving alarms values for campaign: '%s', account: '%s'.", campaignId, accountId)
					local success, scopeResult = GetSupervisorPanelData(campaignData, alarmsResult, _T, agentkeyspace)
					if success then
						if isAllegroCampaign or (account == accountId and attention_level == "") then
							result = scopeResult
						end
					end
				end

				--als
				if not isAllegroCampaign then
					for alId, alAlarms in pairs(accountAlarms.att_levels or {}) do
						if not utils.TableIsEmpty(alAlarms.indicators) then
							campaignData.attention_level = alId
							campaignData.alarms = alAlarms.indicators
							--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] c- Retrieving alarms values for campaign: '%s', account: '%s', al: '%s'.", campaignId, accountId, alId)
							local success, scopeResult = GetSupervisorPanelData(campaignData, alarmsResult, _T, agentkeyspace)
							if success and account == accountId and attention_level == alId then
								result = scopeResult
							end
						end
					end
				end
			end

			local success = true
			if result then
				result.alarms = alarmsResult
			else
				--get scope result if has not alarms
				--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] d- (without alarms) Retrieving values for current scope campaign: '%s'.", campaignId)
				campaignData.account = account
				campaignData.attention_level = attention_level
				campaignData.alarms = nil
				
				success, result = GetSupervisorPanelData(campaignData, {}, _T, agentkeyspace)
			end

			if success and result then
				result.alarms = alarmsResult
				if not supervisor_views then
					supervisor_views = {}
				end

				--Busco los members si viene el requerimiento
				if campaignData.filterMembers then
					local members = m_members.GetSupervisorMembersData(campaignData.filterMembers, agentkeyspace)
					if members then
						result.memberData = members
					end
				end

				--Proceso los indicadores si es requerido
			if campaignData.filterIndicators then
				local resultIndicators, error = __getHistoricalData(campaignData.filterIndicators, vcc, agentkeyspace,  _T)
				result.indicatorsData = resultIndicators or error
			end

				--LogInfo("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] Retrieving alarms for user: '%s' on campaign '%s', ALARMS: '%s'", completeUsername, campaignId, json.encode(result.alarms or {}))
			else
				LogError("[Supervisor:GetSupervisorPanelAndMembersByCampaigns] - Campaign does not exist - get supervisor panel for user: '%s', campaign: '%s' is not loaded yet", completeUsername, campaignId)
			end

			if not supervisor_views then
				supervisor_views = {}
			end

			supervisor_views[fullCampaignId] = result
		end
	end

	agentkeyspace:Close()
	return { supervisor_views = supervisor_views, campaign_workgroups = campaign_workgroups}
end
