local m_models = require("apps.supervisor.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local m_agentservice = require "apps.agent.agentservice"
local m_supervisorshared = require 'supervisorshared'
local m_authFilter = (require "authfilter")("agent")
local m_permissions = require "shared.permissions.permissions"
local m_modelPermissions = require "shared.permissions.models.permissions"
local m_agentApp = require "apps.agent"
local m_agentInteractions = require "apps.agent.controllers.interactions"
local m_allegroSupervisor = require "apps.allegro.controllers.supervisor"
local m_campaigns = require "apps.supervisor.controllers.campaigns"
local m_admUsers = require "apps.administration.controllers.users"
local m_alarms = require "apps.supervisor.controllers.alarms"
local agentNotifications = require("apps.agent.agentNotifications")
local helpers = require "helpers"

m_htmlRender = nil

function MapUris( m, prefix, htmlRender)
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local vr = utils.JsonValidatorReply
	local app = "supervisor"

	m:dispatch_post(vr .. Log .. LoadUsersWithAccessToInteractionApp                                                              , p .. "/load_users_with_access_to_interaction_app/")
	m:dispatch_post(vr .. Log .. GetSupervisorMembers                                                                             , p .. "/get_supervisor_members/"                   )
	m:dispatch_post(vr .. Log .. IsSupervisorMemberLogged                                                                         , p .. "/is_supervisor_member_logged/"              )
	m:dispatch_post(vr .. Log .. GetCampaignsByMemberId                                                                             , p .. "/get_member_campaigns/"                   )
	-- uris with m_permissions
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_change_user_state", MembersChangeState         ), p .. "/change_state/"                             )
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_logoff_user"      , MembersLogout              ), p .. "/logout/"                                   )
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_watch_screen"     , StartScreenStreaming       ), p .. "/start_screen_streaming/"                   )
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_watch_screen"     , StopScreenStreaming        ), p .. "/stop_screen_streaming/"                    )
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_update_members"   , UpdateSupervisorMembers    ), p .. "/update_supervisor_members/"                )
	m:dispatch_post(vr .. Log .. m_permissions:CheckCampaignPermission( app, "sup_update_att_levels_members"   , UpdateAttentionLevelsMembers    ), p .. "/update_attention_levels_members/"                )
	m_htmlRender = htmlRender
end

function IsSupervisorMemberLogged (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local ok, err = utils.HasFields(web.request, {memberId = "string"})
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = err}
	end

	LogDebug("[Supervisor:IsSupervisorMemberLogged] - Supervisor %s is checking user %s is logged", userId, web.request.memberId)

	--need to obtain sessionId from redis
	local sessionId = m_authFilter:GetTokenFromUser(utils.GetId(web.request.memberId), vcc)
	return {result = "OK", logged = sessionId and true or false}
end

function MembersChangeState (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local ok, err = utils.HasFields(web.request, {memberId = "string", state = "string"})
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = err}
	end

	LogInfo("[Supervisor:MembersChangeState] - Supervisor %s is going to change %s state to %s", userId, web.request.memberId, web.request.state)

	--need to obtain sessionId from redis
	local sessionId = m_authFilter:GetTokenFromUser(utils.GetId(web.request.memberId), vcc)

	if not sessionId then
		web.status = 400
		return {result = "USER_NOT_LOGGED", reason = "User is not logged"}
	end

	local result = m_agentservice.ChangeAgentState(sessionId, web.request.memberId, vcc, web.request.state)
	if not result.success then
		LogWarning("[Supervisor:MembersChangeState] - Unable to change status. User %s, not has permission to change status.", web.request.memberId)
		web.status = result.httpCode
		local response = {result = result.errType, reason = ""}
		for name, value in pairs(result) do
			response[name] = value
		end
		return response
	end

	-- this is done to notify the agent poller
	-- from new agent statuses changes
	agentNotifications.NotifyAgents({
		scope = "agent",
		eventType = "state_changed_by_supervisor",
		pollerEventType = "server_changed_state",
		agents = { web.request.memberId.."@"..vcc },
		pollerMessage = {
			state      = web.request.state,
			supervisor = inconcert.authUser.userName,
			type       = "SUPERVISOR_HAS_MODIFIED"
		},
		serverMessage = {
			state = web.request.state,
			supervisor = inconcert.authUser.userName
		}
	}, keyspace)

	return {result = "OK"}
end

function GetCampaignsByMemberId (web )
	local vcc = inconcert.authUser.virtualContactCenter
	local info = web.request 
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)

	local ok, err = utils.HasFields(info, { memberId = "string" })
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST"}
	end

	LogInfo(
        "[Supervisor.Members:GetCampaignsByMemberId] User '%s' is getting all the campaigns of member '%s'",
        completeUsername, info.memberId)
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
  return m_models.Campaigns.GetMemberCampaigns(info.memberId.."@"..vcc, keyspace)
end

function GetSupervisorMembersData (info, agentkeyspace)

	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local campaign_id = info.campaignId

	--LogInfo("[Supervisor:GetSupervisorMembers] - User: '%s' is getting supervisor agents view for campaign: '%s'. Post data: '%s'.", completeUsername, campaign_id, json.encode(info))

	local isAllegroCampaign = utils.IsAllegroCampaign(campaign_id, agentkeyspace)
	local account_id, attention_level, search_al_type

	-- Las campanas de Call de Allegro, NO filtran los members por cuenta
	if not isAllegroCampaign then
		account_id = info.account_info and info.account_info.account
		if account_id then
			attention_level = info.account_info.attention_level
			search_al_type = info.account_info.search_type
		end
	end

	local viewResults, secondsOld = m_supervisorshared.DataFilter():FilterView
	{
		VCC = vcc,
		rowsPerPage = info.rows_per_page or 80,
		showOfflineAgents = info.showOfflineAgents,
		showKickouts = info.showKickouts,
		showInteractionDetails = info.showInteractionDetails or false,
		campaignFilters = 
		{
			[campaign_id] = {
				agent                               = info.agentNameFilter,
				order                               = info.agentOrder or "ID",
				orderDesc                           = info.orderDesc or false,
				page                                = info.page or -1,
				workgroup                           = info.workgroups,
				account                             = account_id,
				attention_level                     = attention_level,
				search_al_type                      = search_al_type,
				-- agent states filter
				state                               = info.agentStateFilter,
				agentStateTimeFilter                = info.agentStateTimeFilter,
				agentStateTimeOperatorFilter        = info.agentStateTimeOperatorFilter,
				-- interactions states filter
				interactionsStateFilter             = info.interactionsStateFilter,
				interactionsStateTimeFilter         = info.interactionsStateTimeFilter,
				interactionsStateTimeOperatorFilter = info.interactionsStateTimeOperatorFilter
			}
		}
	}

	--make key for data entity
	local key
	if not account_id or account_id == "" then
		key = campaign_id
	elseif not attention_level or attention_level == "" then
		key = account_id
	else
		key = attention_level
	end

	local campaignResults = viewResults and viewResults[key]
	if not campaignResults then
		LogError("[Supervisor:GetSupervisorMembers] - Could not get supervisor agents for user: '%s', campaign: '%s' is not loaded yet", completeUsername, campaign_id)
		return false, { result = "NOT_FOUND", reason = "VCC or campaign not found"}
	end

	local result = {
		agents        = campaignResults.agents,
		agentsInQuery = campaignResults.agentsInQuery,
		nowTimestamp  = campaignResults.nowTimestamp
	}

	local alarmsResult = {}
	local alarms = info.alarms and info.alarms.indicators or {}
	info.alarms = nil

	local function HasKickouts()
		--Si el supervisor no quiere ver las alarmas de kickouts, directamente devolvemos false
		if not info.showKickouts then
			--LogDebug("[SUPERVISOR:GetSupervisorMembers] HasKickouts return false because user '%s' dont want to see kickouts alarms", completeUsername)
			return false
		end	
		for agent in pairs(campaignResults.agents) do
			if campaignResults.agents[agent].kickouts and info.showKickouts then
				--LogDebug("[SUPERVISOR:GetSupervisorMembers] HasKickouts return true")
				return true
			end
		end
		--LogDebug("[SUPERVISOR:GetSupervisorMembers] HasKickouts return false")
		return false
	end

	--campaign
	if not isAllegroCampaign and ((not utils.TableIsEmpty(alarms.campaign)) or HasKickouts()) then
		info.account = ""
		info.attention_level = ""
		info.alarms = alarms.campaign
		-- Se procesan todas las alarms de los MEMBERS
		m_alarms.ProcessAllMembersAlarms(info, alarmsResult, result, campaignResults)
	elseif isAllegroCampaign then
		local MustProcessAllMembersAlarms = false
		if HasKickouts() then
			info.account = info.account_info and info.account_info.account
			info.attention_level = ""
			MustProcessAllMembersAlarms = true
		end

		for accountId, accountAlarms in pairs(alarms.accounts or {}) do
			info.account = accountId
			info.attention_level = ""
			if (not utils.TableIsEmpty(accountAlarms.indicators)) or HasKickouts() then
				info.alarms = accountAlarms.indicators
				MustProcessAllMembersAlarms = true
				break
			end
		end

		if MustProcessAllMembersAlarms then
			-- Se procesan todas las alarms de los MEMBERS
			m_alarms.ProcessAllMembersAlarms(info, alarmsResult, result, campaignResults)
		end
	end

	result.alarms = alarmsResult

	return result
end

function GetSupervisorMembers (web)
	local info = web.request

	-- Check required fields
	local ok, err = utils.HasFields(info, { campaignId = "string" })
	if not ok then
		web.status = 400
		LogError("[Supervisor:GetSupervisorMembers] - Invalid request - get supervisor agents for user: '%s', campaign: '%s' is not loaded yet. Err: '%s'.", completeUsername, campaign_id, err)
		return { result = "INVALID_REQUEST", reason = err }
	end

	local result, error = GetSupervisorMembersData(info)
	if not result then
		web.status = error and error.result == "NOT_FOUND" and 404 or 500
		return error
	end

	return result
end

function LoadUsersWithAccessToInteractionApp (web)
	LogInfo("[SUPERVISOR.MEMBERS:LoadUsersWithAccessToInteractionApp] START Loading Users with access to INTERACTION app. Post data: '%s'.", web.input["post_data"])

	utils.CollectAllMemory()

	local vcc = inconcert.authUser.virtualContactCenter
	local info = web.request
	local fullCampaignId = utils.AppendVcc(utils.GetId(info.campaignId), vcc)
	info.limit = nil
	info.numberPage = nil
	info.query = info.query or {}
	info.query.Enabled = "1"
	info.query.Status = "1"
	local require_als = info.require_als

	local startTime = os.time()
	local response = m_admUsers.SearchUsers(info)
	local usersWithAccessToInteractionApp = {}
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local applicationsActions = m_modelPermissions.GetApplicationsActions(keyspace, vcc)
	local numberBatchLog = 200
	local startCurrentProcessTime = os.time()
	local members = m_models.Campaigns.GetCampaignMembers(vcc, fullCampaignId, keyspace)
	--LogDebug("-->members %s", json.encode(members))

	-- Aqui cacheamos los permisos de la campana, para que en la validación de permisos sobre apps, no tenga que ir a buscar estos permisos a cassandra todas las veces, por cada usuario.
	local cachedCampaignsPermissions = m_modelPermissions.GetAllVCCCampaignsPermissions(vcc, keyspace)
	local cachedCurrentCampaignPermissions = (cachedCampaignsPermissions or {})[fullCampaignId] or {}
	-- lo obtenemos fuera del loop para todos los usuarios. para que acceda una única vez a cassandra.
	local usersOverridePermissions = m_modelPermissions.GetUsersOverridePermissions(vcc, keyspace)

	-- En esta tabla se almacena los permisos del Rol dentro de la campana, asi no lo va a buscar por cada Usuario que pertenezca al Rol.
	local permissionsByCampaigns = {}
	-- En esta tabla quedan los roles categorizados, según lo categorizan los permisos.
	-- Idem al comentario anterior, esto es para que no tenga que ir por cada Usuario que pertenezca al Rol.
	local categorizedPermissions = {
		roles = {}
	}

	--
	local userCampaigns = {}
	userCampaigns[fullCampaignId] = true
	local cachedRolesPermissions = {}

	for i, user in ipairs(response.rows) do
		cachedRolesPermissions[user.UserGroup] = cachedRolesPermissions[user.UserGroup] or m_modelPermissions.GetUserGroupPermissions(user.UserGroup, keyspace)

		local simpleUserId = utils.GetId(user.Id)
		local fullUserId = utils.AppendVcc(simpleUserId, vcc)
		local userOverrideVccPermissions = usersOverridePermissions[fullUserId] or utils.EMPTY_OBJECT

		---------------------------------------------------------
		-- Se realizan diversos "caches" para que la función "m_permissions.HasPermissionToApplication" NO tenga
		--  que realizar ciertos cálculos repetitivos (Que no cambian) para diferentes usuarios.
		--  Ej: la categorización de un Rol determinado.
		--
		permissionsByCampaigns[user.UserGroup] = permissionsByCampaigns[user.UserGroup] or m_modelPermissions.MakeGroupedPermissionsByCampaigns(
			{
				get_campaign_permissions = function(--[[campaignId--]])
					return cachedCurrentCampaignPermissions
				end,
				specific_user_group = user.UserGroup,
				user_campaigns = userCampaigns,
				--user_id = full_user_id
			}
		)

		local userCampaignPermissions = permissionsByCampaigns[user.UserGroup]

		-- los permisos del grupo dentro de la campana (ahora si el Usuario tiene override de permisos en la campana lo agrego)
		local memberPermissions = (cachedCurrentCampaignPermissions.members and cachedCurrentCampaignPermissions.members[simpleUserId]) or nil
		if memberPermissions then
			--LogDebug("---%s memberPermissions %s", simpleUserId, json.encode(memberPermissions))
			userCampaignPermissions[fullCampaignId].member = memberPermissions
		else
			userCampaignPermissions[fullCampaignId].member = nil
		end

		--LogDebug("--> user: %s, role: %s - userCampaignPermissions: %s", simpleUserId, user.UserGroup, json.encode(userCampaignPermissions))

		---------------------------------------------------------
		-- Se realizan diversos "caches" para que la función "m_permissions.HasPermissionToApplication" NO tenga
		--  que realizar ciertos cálculos repetitivos (Que no cambian) para diferentes usuarios.
		--  Ej: la categorización de un Rol determinado.
		--
		if not categorizedPermissions.roles[user.UserGroup] then
			--LogDebug("1 categorizando el rol %s", user.UserGroup)
			categorizedPermissions.roles[user.UserGroup] = m_modelPermissions.CategorizeRolePermmissions(cachedRolesPermissions[user.UserGroup], keyspace, applicationsActions)
		end

		local customPermissions = m_modelPermissions.CategorizeAllPermissions(nil, nil, nil, keyspace, applicationsActions, {
			rolePermissions = categorizedPermissions.roles[user.UserGroup],
			-- Estos 2 valores cambian siempre por cada usuario, asi que no me queda más remedio que procesarlo de a uno.
		--	campaignPermissions = nil,
		--	vccPermissions = nil
		})

		-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
		local hasPermissionToAppInteraction = m_permissions.HasPermissionToApplication(
			"interaction",
			user.Id,
			user.UserGroup,
			nil, --fullCampaignId,
			keyspace,
			{
				applicationsActions       = applicationsActions,
				group_permissions         = cachedRolesPermissions[user.UserGroup],
				campaigns_permissions     = cachedCampaignsPermissions,
				user_campaigns            = userCampaigns,
				user_vcc_permissions      = userOverrideVccPermissions,
				user_campaign_permissions = userCampaignPermissions,
				customPermissions         = customPermissions
			}
		)
		if hasPermissionToAppInteraction then
			--LogDebug("[Supervisor.members:LoadUsersWithAccessToInteractionApp] + User %s Role %s, HAS permission to INTERACTION App.", user.Id, user.UserGroup)
			user.BelongToCampaigns = user.BelongToCampaigns or {}
			user.BelongToCampaigns[fullCampaignId] = members and members[fullUserId] and true or false
			--LogDebug("--> '%s' -> '%s' -> user: '%s'", fullCampaignId, belogne, json.encode(user))
			table.insert(usersWithAccessToInteractionApp, user)
		--else
			--LogDebug("[Supervisor.members:LoadUsersWithAccessToInteractionApp] - User %s Role %s, NOT HAS permission to INTERACTION App.", user.Id, user.UserGroup)
		end

		if i == numberBatchLog then
			LogInfo("[SUPERVISOR.MEMBERS:LoadUsersWithAccessToInteractionApp] %d Members are loading for the campaign '%s'. ELAPSED Time '%s' seconds.", numberBatchLog, fullCampaignId, utils.GetElapsedTimeSec(startCurrentProcessTime))
			numberBatchLog = numberBatchLog + 200
			startCurrentProcessTime = os.time()
		end
	end
	
	--Obtengo los als si son requeridos.
	local als
	if require_als then
		als = utils.GetAllColumns {
			keyspace = keyspace or "Agent",
			columnFamily = "CampaignAttentionLevels",
			key = fullCampaignId
		}
	end
	keyspace:Close()

	utils.CollectAllMemory() -- invocamos a garbage ya que esta función realiza muchas conexiones contra cassandra y otras cositas, con esto nos cuidamos !!

	local elapsedTime = utils.GetElapsedTimeSec(startTime)

	LogInfo("[SUPERVISOR.MEMBERS:LoadUsersWithAccessToInteractionApp] END  Loaded '%d' of '%d' Users has access to INTERACTION app. ELAPSED Time '%s' seconds.", #usersWithAccessToInteractionApp, #response.rows, elapsedTime)

	return {users = usersWithAccessToInteractionApp, als = als}
end

function UpdateSupervisorMembers (web)
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[Supervisor.members:UpdateSupervisorMembers] User: '%s' is Updating supervisor agents on campaign: '%s'. Post data: '%s'.", completeUsername, info.campaignId, web.input["post_data"])

	local function ReturnValue( status, response )
		collectgarbage() -- invocamos a garbage ya que esta función realiza muchas conexion contra cassandra y otras cositas, con esto nos cuidamos !!
		web.status = status
		return response
	end

	local ok, err = utils.HasFields(info, {campaignId = "string"})
	if not ok then
		return ReturnValue(400, {result = "INVALID_REQUEST", reason = err})
	end
	local ok, err = utils.HasFields(info, {members = "table"})
	if not ok then
		return ReturnValue(400, {result = "INVALID_REQUEST", reason = err})
	end

	local postedMembers = info.members
	local totalPostedMembers = info.total_members
	if not totalPostedMembers or totalPostedMembers == 0 then
		return ReturnValue(400, {result = "INVALID_REQUEST", reason = "EMPTY MEMBERS LIST"})
	end

	local success, err = m_campaigns.UpdateMembers(info.campaignId, postedMembers, true)
	if not success then
		if err.result == "PROBLEMATIC_USERS" then
			local firstProblematicUsers = err.problematicUsers[1]
			return ReturnValue(400, {result = firstProblematicUsers.type, reason = firstProblematicUsers.description, problematicUsers = err.problematicUsers})
		end
		-- "ALLEGRO_INTEGRATION_ERROR"
		return ReturnValue(400, err)
	end

	return ReturnValue(200, {result = "OK"})
end

function UpdateAttentionLevelsMembers (web)
	local info = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local completeUsername = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[Supervisor.members:UpdateAttentionLevelsMembers] User: '%s' is updating agents the attention levels defined in the campaign: '%s'. Post data: '%s'.", completeUsername, info.campaignId, web.input["post_data"])

	local function ReturnValue( status, response )
		collectgarbage()
		web.status = status
		return response
	end

	local ok, err = utils.HasFields(info, {campaignId = "string"})
	if not ok then
		return ReturnValue(400, {result = "INVALID_REQUEST", reason = err})
	end
	local ok, err = utils.HasFields(info, {attLevelsToUpdate = "table"})
	if not ok then
		return ReturnValue(400, {result = "INVALID_REQUEST", reason = err})
	end

	local success, err = m_campaigns.UpdateAttentionLevelsMembers(info.campaignId, info.attLevelsToUpdate)

	if not success then
		return ReturnValue(err.status or 400, err)
	else
		return ReturnValue(200, {result = "OK"})
	end
end

function MembersLogout (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local ok, err = utils.HasFields(web.request, {memberId = "string"})
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = err}
	end

	LogInfo("[Supervisor:MembersLogout] - Supervisor %s is going to logout member: %s ", userId, web.request.memberId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- le paso el keyspace, ya que el controller de agente en esta funcion, realiza varias conexiones a cassandra.
	local assignedInteractions = m_agentInteractions.RetrieveUserInteractions(web.request.memberId, nil, keyspace)
	keyspace:Close()
	--If user has at least one live interaction then abort operation
	for _, interaction in ipairs(assignedInteractions or {}) do
		if utils.InteractionIsLive(interaction.type) then
			web.status = 400
			return {result = "AGENT_HAS_INTERACTIONS", reason = "Agent has interactions"}
		end
	end

	--notify via poller
	agentNotifications.NotifyAgents({
		scope = "agent",
		eventType = "supervisor_logout",
		agents = { web.request.memberId.."@"..vcc },
		pollerMessage = {
			supervisor = inconcert.authUser.userName
		},
		serverMessage = {
			supervisor = inconcert.authUser.userName
		}
	}, keyspace)

	--in order to avoid race condition
	socket.sleep(2)

	--do logout
	local token = m_authFilter:GetTokenFromUser(utils.GetId(web.request.memberId), vcc)
	if token then
		local userData = m_authFilter:GetUserFromToken(token)
		if userData then
			userData.kickout = web.request.kickout
			m_agentApp.FinalizeSession(userData)
		end
		m_authFilter:RevokeAllSessions(utils.AppendVcc(web.request.memberId, vcc))
	end

	return {result = "OK"}
end

function StartScreenStreaming (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local ok, err = utils.HasFields(web.request, {memberId = "string"})
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = err}
	end

	LogInfo("[Supervisor:StartScreenStreaming] - Supervisor %s is going to start streaming for member: %s ", userId, web.request.memberId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local has_allegro = utils.HasAllegroIntegration( vcc, keyspace )
	keyspace:Close()

	local status, response

	if(has_allegro) then
		status, response = m_allegroSupervisor.StartScreenStreaming(web.request.memberId)
	else
		status = true
		response = { data = { channel = web.request.memberId .. "@" .. vcc, isOCC = true } }
	end
	if not status then
		web.status = 500
		return { result = "FAIL", reason = response and response.reason or "unknown error" }
	end
	return { result = "OK", data = response.data }
end

function StopScreenStreaming (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local ok, err = utils.HasFields(web.request, {memberId = "string"})
	if not ok then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = err}
	end

	LogInfo("[Supervisor:StopScreenStreaming] - Supervisor %s is going to stop streaming for member: %s ", userId, web.request.memberId)
	
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local has_allegro = utils.HasAllegroIntegration( vcc, keyspace )
	keyspace:Close()

	local status, response
	if(has_allegro) then
		status, response = m_allegroSupervisor.StopScreenStreaming(web.request.memberId)
	else
		--La notificacion de stop del streaming le llegara al agent via poller desde el agentscreenservice cuando corresponda
		status = true
		response = { data = { channel = web.request.memberId .. "@" .. vcc, isOCC = true } }
	end
	if not status then
		web.status = 500
		return { result = "FAIL", reason = response and response.reason or "unknown error" }
	end
	return { result = "OK" }
end
