/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 9*/

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupAlarmsConfig
 */
(function () {

	'use strict';

	const format = inConcert.Utils.format;
	const DEFAULT_INSTANCE_TYPE = "CONFIGURATION-GUI";

	/**
	 * Creates a new supervisor alarms configuration instance.
	 *
	 * @author Ellio de Mello (cany@inconcertcc.com)
	 * @constructor
	 * @param params object
	 */
	function SupAlarmsConfig( params )
	{
		this.m_parent = params.parent;
		this.m_dashboard = this.m_parent.m_dashboard;
		this.m_campaign_id = params.campaign_id;
		this.m_campaign_dom_id = this.m_dashboard.get_normalized_id(this.m_campaign_id);
		this.m_prefix_container_id = params.prefix_container_id || '';
		this.m_instance_type = params.instance_type || DEFAULT_INSTANCE_TYPE;

		this.set_config(params.currents_alarms);
		var container_id = this.__get_container_id();

		// Ids del markup del popup
		this.popup_campaigns_selector_id = container_id + '_popup_selector_scope';
		this.show_container_selector_indicators_id = container_id + '_show_container_selector_indicators';
		this.container_selector_indicators_id = container_id + '_container_selector_indicators';
		this.popup_selector_indicators_list_id = container_id + '_popup_selector_indicators_list';
		this.update_indicators_id = container_id + '_update_indicators';
		this.cancel_update_indicators_id = container_id + '_cancel_update_indicators';
		// Ids del markup de la lista de alarmas configuradas
		this.list_categories_id = container_id + '_list_categories_container';
		this.save_id = container_id + '_save';
		this.close_id = container_id + '_close';
		this.message_no_alarms_id = container_id + '_no_alarms';

		this.m_sorted_categories = null;

		this.m_alerts_indicator_types = {
			'boolean'   : {operators: ['EQUAL', 'DISTINCT']},
			condition   : {operators: ['EQUAL']},
			'float'     : {operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER']},
			integer     : {operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER']},
			percentage  : {operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER'],
				sections : ['PERCENTAGE']
			},
			string: {operators: ['EQUAL', 'DISTINCT']},
			time: {
				operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER'],
				sections : ['SECOND', 'MINUTE', 'HOUR']
			},
			datetime: {
				operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER'],
				sections : ['SECOND', 'MINUTE', 'HOUR', 'DAY', 'MONTH']
			},
			date: {
				operators: ['EQUAL', 'DISTINCT', 'LESS', 'GREATER'],
				sections : ['DAY', 'MONTH']
			}
		};

		this.m_operators_labels = {
			EQUAL   : '<%=_T("Equal To")%>',
			DISTINCT: '<%=_T("Distinct From")%>',
			LESS    : '<%=_T("Less Than")%>',
			GREATER : '<%=_T("Greater Than")%>'
		};

		this.m_sections_labels = {
			AGENTS       : '<%=_T("Agents")%>',
		//
			CONVERSATIONS: '<%=_T("Conversations")%>',
		//
			PERCENTAGE   : '<%=_T("%")%>',
		//
			SECOND       : '<%=_T("Seconds")%>',
			MINUTE       : '<%=_T("Minutes")%>',
			HOUR         : '<%=_T("Hours")%>',
			DAY          : '<%=_T("Days")%>',
			MONTH        : '<%=_T("Months")%>'
		};

		this.m_scope_popup = null;

		this.__intialize();
	}

	if (typeof inConcert.sup_alarms_config !== "function") {
		inConcert.sup_alarms_config = SupAlarmsConfig;
	}

	SupAlarmsConfig.prototype.__intialize = function()
	{
		this.__sort_categories();
		this.__insert_alarms_in_categories();
		if (this.m_instance_type === "CONFIGURATION-GUI") {
			this.__build_markup();
			this.__set_events();
			this.__initialize_scope_ctrl();
			this.__insert_user_alarms_in_markup();
		}
	};

	/*
		Instancia el combo "selector" de scope para el popup, con la campana, las cuentas 
	*/
	SupAlarmsConfig.prototype.__initialize_scope_ctrl = function()
	{
		var self = this;

		this.m_scope_popup = new inConcert.campaign_selector({
			from_app: "supervisor",
			popup_config: {
				shown_header: false,
				blank_first_item  : false,
				include_accounts  : true,
				include_att_levels: true,
				disabled_select_all: true, //Se deshabilita el seleccionar todas las cuentas si solo tiene una.
				load_campaigns: function(callback)
				{
					callback = callback || $.noop;
					var campaigns = [{Id: self.m_campaign_id}]; // solo listamos en el combo la campana del panel, cons sus cuentas y als.
					window.setTimeout(callback(true, campaigns));
				},
				on_selected: function(selected_info)
				{
					// Cuando se ha seleccionado un item y se ha presionado el boton "select"
					var disabled = true;
					if (selected_info && !selected_info.blank) {
						// Dejamos enabled el boton para cargar los indicadores, si es que no se encuentra seleccionado el item "Blank" (none campaigns)
						disabled = false;
					}
					var $btn_show_selector_indicators = $("#" + self.show_container_selector_indicators_id);
					$btn_show_selector_indicators.prop("disabled", disabled);
				}
			},
			container_id: this.popup_campaigns_selector_id,
			selector_label: "<%=_T([[Campaign accounts]])%>:",
			selector_label_classes: "required",
			selector_text_classes: "form-control"
		});
	};

	/*
		Ordena las categorias
	*/
	SupAlarmsConfig.prototype.__sort_categories = function()
	{
		var indicators = this.__get_system_indicators();
		var sorted_categories = {};

		$.each(indicators.categories, function (category_id, category_def) {
			sorted_categories[category_id] = {
				id: category_id,
				desc: category_def.desc,
				order: category_def.order,
				list: []
			};
		});

		sorted_categories = inConcert.Utils.sorted_hash( 
			// hash desordenado
			sorted_categories,
			// funcion que retorna el valor por el cual se va realizar el ordenamiento
			function( category_def ) { return category_def.order; },
			// funcion que genera el hash ordenado
			function( target_hash, category_def ) { target_hash[ category_def.id ] = category_def; return target_hash; }
		);

		this.m_sorted_categories = sorted_categories;
	};

	/*
		Cuando se instancia la clase, se cargan en el markup las alarmas que el usuario tiene configurado
	*/
	SupAlarmsConfig.prototype.__insert_alarms_in_categories = function()
	{
		var self = this;

		var currents_alarms = this.__get_currents_alarms();
		var indicators = currents_alarms.indicators || {};

		var insert = function(alarm) {
			var indicator = self.__get_indicator(alarm.indicator_id);
			if (!indicator) {
				return true; //continue
			}
			var category = indicator.category_id;
			if (!self.m_sorted_categories[category]) {
				// TODO: hacer algo
				//throw('SupAlarmsConfig.__insert_alarms_in_categories');
				return;
			}
			self.m_sorted_categories[category].list.push(alarm);
		};

		var insert_list = function(list) {
			$.each(list || {}, function (_, alarm) {
				insert(alarm);
			});
		};

		insert_list(indicators.campaign);

		$.each(indicators.accounts || {}, function (_, info) {
			insert_list(info.indicators);

			$.each(info.att_levels || {}, function (_, info) {
				insert_list(info.indicators);
			});
		});
	};

	/*
		Esta funcion actualiza en memoria la lista con las alarmas, a partir de lo que el usuario ha configurado.
		La misma es invocada cuando el usuario ha presionado el boton "save".
	*/
	SupAlarmsConfig.prototype.__update_alarms_to_save = function()
	{
		var self = this;

		$.each(this.m_sorted_categories || {}, function (category_id)
		{
			var $categories = self.__$list_alarms_by_category(category_id);
			var alarms = [];
			$.each($categories.find("[alarm_item]"), function(_, alarm)
			{
				var $alarm = $(alarm);
				var alarm_id = $alarm.attr("alarm_item");
				var enabled = $alarm.find("[alarm_item_selector]").prop("checked");
				var operator = $alarm.find("[alarm_item_ctrl_type='operator']").val();
				var value = $alarm.find("[alarm_item_ctrl_type='alarm_value']").val();
				var other_value = $alarm.find("[alarm_item_ctrl_type='alarm_unit']").val();
				var info = $alarm.data("info");
				alarms.push({
					indicator_id: alarm_id,
					enabled: enabled,
					operator: operator,
					value: value,
					section: other_value,
					scope: info.alarm.scope,
					campaign_id: info.alarm.campaign_id,
					account_id: info.alarm.account_id,
					al_id: info.alarm.al_id
				});
			});
			self.m_sorted_categories[category_id].list = alarms;
		});
	};

	/*
		Esta funcion inserta en el markup todas las alarmas que el usuario tiene configuradas.
	*/
	SupAlarmsConfig.prototype.__insert_user_alarms_in_markup = function()
	{
		var self = this;

		$.each(this.m_sorted_categories, function (category_id, category_def)
		{
			$.each(category_def.list, function (_, user_alarm)
			{
				var alarm = {
					indicator_id: user_alarm.indicator_id,
					scope       : user_alarm.scope,
					campaign_id : user_alarm.campaign_id,
					account_id  : user_alarm.account_id,
					al_id       : user_alarm.al_id,
					values      : user_alarm
				};
				self.__add_alarm_to_list(alarm);
			});
		});
	};

	/*
		Retorna la defincion de indicadores que se existe server side.
	*/
	SupAlarmsConfig.prototype.__get_system_indicators = function()
	{
		return this.m_parent.get_system_indicators();
	};

	/*
		Retorna la lista de alarms tal y como se encuentra almacenada en la CF SupervisorConfig.
	*/
	SupAlarmsConfig.prototype.__get_currents_alarms = function()
	{
		return this.m_currents_alarms;
	};

	SupAlarmsConfig.prototype.__get_container_id = function()
	{
		return this.m_prefix_container_id + this.m_campaign_dom_id;
	};

	SupAlarmsConfig.prototype.__$container = function()
	{
		return $('#' + this.__get_container_id());
	};

	/*
		Retorna el markup de la lista de categorias
	*/
	SupAlarmsConfig.prototype.__$list_categories_container = function()
	{
		return $('#' + this.list_categories_id);
	};

	/*
		Retorna el markup de una categoria especifica con sus alarmas
	*/
	SupAlarmsConfig.prototype.__$category_container = function(category_id)
	{
		var $categories = this.__$list_categories_container();
		var $category = $categories.find("[category_id='" + category_id + "']");
		return $category;
	};

	/*
		Retorna el markup de todas las alarmas que se han configurado.
	*/
	SupAlarmsConfig.prototype.__$list_all_alarms = function()
	{
		var $categories = this.__$list_categories_container();
		var $all_alarms = $categories.find("[unique_id]");
		return $all_alarms;
	};

	/*
		Retorna el markup de todas las alarmas para una categoria determinada.
	*/
	SupAlarmsConfig.prototype.__$list_alarms_by_category = function(category_id)
	{
		var $category = this.__$category_container(category_id);
		var $alarms_container = $category.find("[alarms_list='true']");
		return $alarms_container;
	};

	/*
		Retorna el markup de una alarma especifica, buscando por el unique_id.
	*/
	SupAlarmsConfig.prototype.__$find_alarm = function(unique_id)
	{
		var $categories = this.__$list_categories_container();
		var $alarm = $categories.find("[unique_id='" + unique_id + "']");
		return $alarm;
	};

	/*
		Cuenta la cantidad de alarmas existentes en una categoria que se han configurado para el usuario.
	*/
	SupAlarmsConfig.prototype.__count_alarms_in_category = function(category_id)
	{
		var $category = this.__$list_alarms_by_category(category_id);
		var $alarms = $category.find("[alarm_item]");
		return $alarms.length;
	};

	SupAlarmsConfig.prototype.__get_sorted_categories = function()
	{
		return this.m_sorted_categories;
	};

	SupAlarmsConfig.prototype.__get_indicator = function(id)
	{
		var indicators = this.__get_system_indicators();
		return indicators.list[id];
	};

	SupAlarmsConfig.prototype.__get_indicator_type = function(type)
	{
		return this.m_alerts_indicator_types[type];
	};

	SupAlarmsConfig.prototype.__get_operator_label = function(operator)
	{
		return this.m_operators_labels[operator];
	};

	SupAlarmsConfig.prototype.__get_section_label = function(section)
	{
		return this.m_sections_labels[section];
	};

	SupAlarmsConfig.prototype.__get_field_label = function(section, indicator)
	{
		return indicator.label;
	};

	/*
		Rertorna el id unico que se agregara al markup en el attr "unique_id".
		Este id esta compueso por:
			"INDICATOR:{0}:CAMP:{1}:ACCOUNT:{2}:AL:{3}";
			El id del indicador
			El id de la campana
			El id del account
			El id del AL
	*/
	SupAlarmsConfig.prototype.__make_unique_id = function (alarm)
	{
		var unique_id = "INDICATOR:{0}:CAMP:{1}:ACCOUNT:{2}:AL:{3}";

		var indicator_id = alarm.indicator_id || "";
		var account_id = alarm.account_id || "";
		var al_id = alarm.al_id || "";

		return format(unique_id, indicator_id, this.m_campaign_id, account_id, al_id);
	};

	/*
		Rertorna el del scope al cual pertenece la alarma, el mismo se agregara al markup en el attr "scope_id".
		Este id esta compueso por:
			"INDICATOR:{0}:CAMP:{1}:ACCOUNT:{2}:AL:{3}";
			El id del indicador
			El id de la campana
			El id del account
			El id del AL
	*/
	SupAlarmsConfig.prototype.__make_scope_id = function (alarm)
	{
		var unique_id = "CAMP:{0}:ACCOUNT:{1}:AL:{2}";

		var account_id = alarm.account_id || "";
		var al_id = alarm.al_id || "";

		return format(unique_id, this.m_campaign_id, account_id, al_id);
	};

	/*
		El markup de los controles de las alarmas.
	*/
	SupAlarmsConfig.prototype.__get_html_input = function (indicator_id, indicator, sections)
	{
		var self = this;

		if (indicator.type.toLowerCase() === "condition") {
			return "";
		}

		var html = '<input alarm_item_ctrl="' + indicator_id + '" alarm_item_ctrl_type="alarm_value" class="form-control alarmValue type' + indicator.type + '" type="text" placeholder="" />';

		var total_sections = (sections || []).length;

		if (total_sections > 0) {
			var html_combo_values = "";
			var selected = "";

			$.each(sections, function(i, section_id)
			{
				selected = (i === 1 && total_sections === 1) ? "selected" : "";
				html_combo_values+=
					'<option class="placeholder" value="' + section_id + '" ' + selected + '>' +
						self.__get_section_label(section_id) +
					'</option> ';
			});

			var html_combo_blank = "";
			var readonly_classes = "";
			var disabled_attr = "";
			if (total_sections > 1) {
				html_combo_blank = '<option selected class="placeholder"></option>';
			}
			else {
				readonly_classes = "TU_CLASE_READONLY";
				disabled_attr = "disabled";
			}

			html+=
				'<select alarm_item_ctrl="' + indicator_id + '" alarm_item_ctrl_type="alarm_unit" ' +
					'alarm_item_ctrl="' + indicator_id + '" ' + disabled_attr + ' class="form-control alarmSection ' + readonly_classes + '">' +
						html_combo_blank +
						html_combo_values +
				'</select>';
		}

		return html;
	};

	/*
		Valida si el indicador puede ser agregado en la lista de alarmas del usuario.
	*/
	SupAlarmsConfig.prototype.__can_add_alarm_to_list = function(alarm)
	{
		var indicator = this.__get_indicator(alarm.indicator_id);
		if (!indicator) {
			//TODO: este caso se daria solo en el caso de que se haya eliminado un indicador del sistema, en la definicion en el servidor.
			return false;
		}

		var unique_id = this.__make_unique_id(alarm);
		var $category = this.__$list_alarms_by_category(indicator.category_id);
		var $alarm = $category.find("[unique_id='" + unique_id + "']");
		// Si el indicador ya existe, no lo vuelve a agregar
		if ($alarm.length > 0){
			return false;
		}

		return true;
	};

	/*
		Agrega una alarma a la lista de alarmas que pertenecen al usuario.
	*/
	SupAlarmsConfig.prototype.__add_alarm_to_list = function(alarm)
	{
		var self = this;

		var must_add = this.__can_add_alarm_to_list(alarm);
		if (!must_add) {
			return false;
		}
		var indicator = this.__get_indicator(alarm.indicator_id);
		var indicator_type = this.__get_indicator_type(indicator.type);
		var label = indicator.slice ? this.__get_field_label(indicator.slice, indicator) : indicator.label;

		var html_condition = "";
		if (indicator.type.toLowerCase() !== 'condition') {
			var operators = indicator.operators ? indicator.operators : indicator_type.operators;
			var html_operators = operators.reduce(function (html, operator) {
				return html +
					'<option class="placeholder" value="' + operator + '">' +
						self.__get_operator_label(operator)  +
					'</option>';
			}, '<option class="placeholder"></option>');
			html_condition = '<select alarm_item_ctrl="' + alarm.indicator_id + '" alarm_item_ctrl_type="operator" class="form-control operatorValue">' + html_operators + '</select>';
		}

		// recorro la lista de cuentas y las agrego
		var campaign = inConcert.app.getCampaign(this.m_campaign_id);
		if (!campaign) {
			// TODO: HACER ALGO
			return;
		}

		var account_id = (alarm.account_id || "");
		var account_description = "<%=_T([[All Accounts]])%>";
		var al_id = "";
		var al_description = "";
		var account = campaign.getAccount(account_id);
		if (alarm.account_id) {
			//Si la alarma esta configurada para una cuenta, pero no se ha podido recuperar dicha cuenta, es porque se elimino, y la alarma ya no tiene sentido
			if(!account) {
				self.__remove_alarm_to_list(alarm);
				self.__save_config(undefined);
				return;
			}
			account_description = campaign.getAccountDescription(account);
			if (alarm.al_id) {
				al_id = alarm.al_id;
				var att_level = campaign.getAttentionLevel(al_id);
				if (att_level) {
					al_description = att_level.name;
				}
			}
		}

		var unique_id = this.__make_unique_id(alarm);
		var scope_id = this.__make_scope_id(alarm);
		var chk_dom_id = inConcert.Utils.generateGuid(); // este id es utilizado unicamente para referenciar el chk con el label con el attr "for"

		var classes = "alarmListItem";
		if (alarm.scope === "campaign") {
			classes = classes + " " + "scopeCampaign";
		}
		else if (alarm.scope === "channels" || alarm.scope === "att_levels") {
			classes = classes + " " + "scopeAccount" + account.proxyType.toUpperCase();
			if (alarm.scope === "att_levels") {
				classes = classes + " " + "scopeAttentionLevel";
			}
		}

		var sections = indicator.sections || indicator_type.sections;

		var translated = {
			remove: "<%=_T([[Remove]])%>"
		};

		var $alarm = $(
			'<li unique_id="' + unique_id + '" scope_id="' + scope_id + '" alarm_item="' + alarm.indicator_id + '" alarm_category="' + indicator.category_id + '" class="' + classes + '" >' +
				'<div class="w50">' +
					'<input id="' + chk_dom_id + '" alarm_item_selector="' + alarm.indicator_id + '" type="checkbox" class="alarmCheckbox" checked />' +
					'<label for="' + chk_dom_id + '" class="alarmIndicatorName" ' + ' title="' + indicator.tooltip + '">' + label + '</label>' +
					'<div class="alarmIndicatorScope">' +
						'<label alarm_scope="' + alarm.scope + '" class="indicatorScopeText" hidden>' + alarm.scope + '</label>' +
						'<label alarm_account="' + account_id + '" class="indicatorScopeText">' + account_description + '</label>' +
						'<label alarm_al="' + al_id + '" class="indicatorScopeText">' + al_description + '</label>' +
					'</div>' +
				'</div>' +
				'<div class="w50 controlFields">' +
					'<div class="alarmIndicatorControl">' +
						html_condition +
						this.__get_html_input(alarm.indicator_id, indicator, sections) +
						'<i alarm_item_error="' + alarm.indicator_id + '" class="alarmFieldValidation alarmFieldError" hidden></i>' +
					'</div>' +
					'<button alarm_item_remove="' + unique_id + '" class="buttonColorGray gridActionButton" type="button">' +
						'<i class="far fa-trash-alt"></i>' +
					'</button>' +
				'</div>' +
			'</li>'
		);

		$alarm.data("info", {alarm: alarm, indicator: indicator});

		var $category = this.__$list_alarms_by_category(indicator.category_id);
		$category.append($alarm);

		var values = alarm.values;
		if (values) {
			// El operador
			var $operator = $alarm.find("[alarm_item_ctrl_type='operator']");
			$operator.val(values.operator || "");
			// El valor
			var $value = $alarm.find("[alarm_item_ctrl_type='alarm_value']");
			$value.val(values.value || "");
			// El otros valores
			var $other_value = $alarm.find("[alarm_item_ctrl_type='alarm_unit']");
			$other_value.val(values.section || "");
			// Si la alarma se encuentra disabled
			if (!values.enabled) {
				$alarm.find("[alarm_item_selector]").prop("checked", false).trigger("change");
			}
		}

		this.__show_category(indicator.category_id);

		return true;
	};

	/*
		Retorna el markup de todas las alarmas para un scope dado.
	*/
	SupAlarmsConfig.prototype.__$get_added_alarms_by_scope = function(scope)
	{
		var $categories = this.__$list_categories_container();
		var scope_id = this.__make_scope_id(scope);
		var $alarms_by_scope = $categories.find("[scope_id='" + scope_id + "']").parent();
		return $alarms_by_scope;
	};

	/*
		Muestra el div contenedor de una categoria dada.
	*/
	SupAlarmsConfig.prototype.__show_category = function(category_id)
	{
		var $category = this.__$category_container(category_id);
		$category.show();
		$("#" + this.message_no_alarms_id).hide();
	};

	/*
		Oculta el div contenedor de una categoria dada.
	*/
	SupAlarmsConfig.prototype.__hide_category = function(category_id)
	{
		var $category = this.__$category_container(category_id);
		$category.hide();
	};

	/*
		Oculta el div contenedor de las categorias que no tengan alarmas configuradas.
	*/
	SupAlarmsConfig.prototype.__hide_empty_categories = function()
	{
		var self = this;

		var total_categories = 0;
		var total_hidden_categories = 0;

		$.each(this.m_sorted_categories, function(category_id)
		{
			total_categories++;
			var count = self.__count_alarms_in_category(category_id);
			if (count === 0) {
				total_hidden_categories++;
				self.__hide_category(category_id);
			}
		});

		if (total_hidden_categories === total_categories) {
			$("#" + this.message_no_alarms_id).show();
		}
	};

	/*
		Retorna los indicadores del sistema para un scope dado.
	*/
	SupAlarmsConfig.prototype.__get_indicators_by_scope = function(scope_info)
	{
		var indicators_to_returned = {};

		var campaign = inConcert.app.getCampaign(this.m_campaign_id);
		if (!campaign) {
			// TODO: HACER ALGO
			return indicators_to_returned;
		}
		//
		var total_configured_channels = 0; // aca se almacena la cantidad de canales que se han configurado en la campana
		// El canal CALL, se agrega mas abajo si es que la campana es de telefonia/allegro, y el mismo se divide en 2 (inbound,outbound)
		var campaign_configured_channels = {
			CHAT      : true, FACEBOOK: true, INSTAGRAM : true, YOUTUBE: true, IM       : true,
			MAIL      : true, SMS     : true, WHATSAPP  : true, TWITTER: true, VOICEMAIL: true,
			WEBCONTACT: true, APPSTORE: true, GOOGLEPLAY: true, RCS    : true
		};
		// Marco como desactivados los canales que la campana no tenga ninguna cuenta.
		$.each(campaign_configured_channels, function(channel)
		{
			if (campaign.has_accounts_of_proxy(channel)) {
				total_configured_channels++;
			}
			else {
				campaign_configured_channels[channel] = false;
				return true; // continue
			}
		});

		let has_configured_call_channel_in_campaign = campaign.hasTelephonyChannel();
		if (has_configured_call_channel_in_campaign) {
			// Aca se agrega el canal de telefonia, discriminado por entrante saliente, pero el contador solo se incrementa en 1.
			total_configured_channels++;
			campaign_configured_channels.INBOUND_CALL = true;
			campaign_configured_channels.OUTBOUND_CALL = true;
		}

		var indicators = this.__get_system_indicators();
		var indicators_by_scope = indicators.by_scope[scope_info.scope];

		if (scope_info.scope === "campaign") {
		// Atencion: Para el scope campaign, se retornan los indicadores de los tipos de canales que se hayan configurado en la campana.
		//
			var campaign_indicators = {};

			// Armo la lista de indicadores de la campana (segun los canales configurados)
			$.each(indicators_by_scope, function(indicator_id, indicator)
			{
				if (indicator.scope.campaign === "NOT ALLOWED") {
					return true; //continue
				}
				// Si en la definicion del scope tiene el valor ALL o es undefined, quiere decir que el indicador se incluye independientemente
				// de si tiene o no configurado el canal.
				if (indicator.scope.campaign === "NOT DEPENDS ON CONFIGURED CHANNELS") {
					campaign_indicators[indicator_id] = indicator;
					return true; //continue
				}
				// Ahora se agregan los indicadores para la campana segun su configuracion.
				if (typeof(indicator.scope.channels) === "object") {
					var can_add = false;
					$.each(indicator.scope.channels, function(scope_channel, scope_enabled)
					{
						if (!scope_enabled) {
							return true; //continue
						}
						if (campaign_configured_channels[scope_channel] === true) {
							can_add = true;
							return false; // break
						}
					});
					if (can_add) {
						campaign_indicators[indicator_id] = indicator;
					}
				}
			});

			indicators_to_returned = campaign_indicators;
		}
		else if (scope_info.scope === "channels") {
			if (!has_configured_call_channel_in_campaign) {
				indicators_to_returned = indicators_by_scope[scope_info.channel];
			}
			else {
			// Para las campanas de allegro/telefonia, tengo que agregar los indicadores de forma manual
			// ya que los mismos no aparecen por un unico canal, y aparecen discrimiandos por inbound/outbound y voicemail
				var camp_allegro_indicators = {};

				// INBOUND indicators
				var inbound_call_indicators = indicators_by_scope.INBOUND_CALL;
				$.each(inbound_call_indicators || {}, function(indicator_id, indicator)
				{
					camp_allegro_indicators[indicator_id] = indicator;
				});
				// OUTBOUND indicators
				var outbound_call_indicators = indicators_by_scope.OUTBOUND_CALL;
				$.each(outbound_call_indicators || {}, function(indicator_id, indicator)
				{
					camp_allegro_indicators[indicator_id] = indicator;
				});
				// VOICEMAIL indicators
				var voicemail_indicators = indicators_by_scope.VOICEMAIL;
				$.each(voicemail_indicators || {}, function(indicator_id, indicator)
				{
					camp_allegro_indicators[indicator_id] = indicator;
				});

				indicators_to_returned = camp_allegro_indicators;
			}
		}
		else if (scope_info.scope === "att_levels") {
			var scope_channels = indicators.by_scope.channels;
			var indicators_by_al_scope = {};
			$.each(indicators_by_scope, function(indicator_id, indicator)
			{
				if ("CHANNEL DEPEND" === indicator.scope.att_levels) {
				// Si el indicador a nivel AL depende que el mismo exista para el CHANNEL
					if (scope_channels[scope_info.channel] && scope_channels[scope_info.channel][indicator_id]) {
						indicators_by_al_scope[indicator_id] = indicator;
					}
				}
				else if ("NOT CHANNEL DEPEND" === indicator.scope.att_levels) {
				// Si el indicador a nivel AL No depende que el mismo exista para el CHANNEL.
					indicators_by_al_scope[indicator_id] = indicator;
				}
				else {
					throw("SupAlarmsConfig.prototype.__get_indicators_by_scope. Unrecognized AL scope info '" + indicator.att_levels + "'");
				}
			});
			indicators_to_returned = indicators_by_al_scope;
		}
		else {
			throw("SupAlarmsConfig.prototype.__get_indicators_by_scope. Unrecognized scope '" + scope_info.scope + "'");
		}

		// --------------------------------------
		// FILTRO
		//    Despues de obtener los indicadores se filtran los mismos.
		//---------------------------------------

		let is_allegro_campaign = campaign.isAllegro();

		// Esta funcion es la encargada de validar/filtrar cada indicador
		var must_add = function(indicator)
		{
			let indicator_filter = (indicator.filter || "").toUpperCase();
			if (indicator_filter) {
				if (indicator_filter === "NOT_CONFIGURABLE") {
					// NO se visualiza el indicador, ya que no es configurable.
					// Tiene que estar para realizar determinados calculos.
					//  Ej: "kickout" indicator
					return false;
				}
				if (indicator_filter !== scope_info.scope.toUpperCase()) {
					return false;
				}
			}


			if (indicator.category_type === "total_conversations_states") {
			// De momento se realiza filtro unicamente contra la categoria de "Totales de conversaciones"
				if (!is_allegro_campaign) {
					// Esta validacion se realiza si la campana NO es de telefonia/allegro.
					if (scope_info.scope === "campaign") {
						if (total_configured_channels <= 1) {
							// NO se visualiza el indicador: Si el scope es "campaign" y en la campana, se han configurado cuentas de un unico canal.
							return false;
						}
					}
					else if (scope_info.scope === "channels" || scope_info.scope === "att_levels") {
						// NO se visualiza el indicador: Si el scope es "channels" o "att_levels".
						return false;
					}
					else {
						throw("SupAlarmsConfig.prototype.__get_indicators_by_scope. Unrecognized scope '" + scope_info.scope + "'");
					}
				}
			}

			if (indicator_filter === "ONLY_ALLEGRO" && !is_allegro_campaign) {
				// NO se visualiza el indicador: El mismo se debe de visualizar unicamente para Allegro y la campana no es de telefonia/allegro
				return false;
			}

			// SI se visualiza el indicador: ya que no pertenece a la categoria de "Totales de conversaciones" 
			return true;
		};

		var filter = function(list) {
			var filtered_indicators = {};
			// Se recorre la lista de indicadores y se evalua si se deben filtrar
			$.each(list || {}, function(indicator_id, indicator)
			{
				if (must_add(indicator)) {
					filtered_indicators[indicator_id] = indicator;
				}
			});

			return filtered_indicators;
		};

		// Se manda a filtrar
		indicators_to_returned = filter(indicators_to_returned);

		return indicators_to_returned;
	};

	/*
		Carga en el popup selector la lista de indicadores del sistema para el scope seleccionado.
	*/
	SupAlarmsConfig.prototype.__load_indicators_by_scope = function()
	{
		var self = this;

		var scope_info = this.m_scope_popup.get_selected_info();
		if (!scope_info) {
			return;
		}

		var $added_alarms_in_scope = this.__$get_added_alarms_by_scope(scope_info);
		var indicators_by_scope = this.__get_indicators_by_scope(scope_info);

		var $popup_list = $("#" + this.popup_selector_indicators_list_id);
		var $categories = $(this.__make_html_categories(true));
		$popup_list.html($categories);

		$.each(indicators_by_scope, function(indicator_id, indicator)
		{
			var unique_id = self.__make_unique_id({
				indicator_id : indicator_id,
				account_id   : scope_info.account_id || "",
				al_id        : scope_info.al_id || "",
			});

			var chk_dom_id = inConcert.Utils.generateGuid(); // este id es utilizado unicamente para referenciar el chk con el label con el attr "for"
			var label = indicator.slice ? self.__get_field_label(indicator.slice, indicator) : indicator.label;
			var $added_alarm = $added_alarms_in_scope.find("[unique_id='" + unique_id + "']");
			var checked = $added_alarm.length > 0 ? "checked" : "";

			var $indicator = $(
				'<li unique_id="' + unique_id + '" alarm_item="' + indicator_id + '" alarm_category="' + indicator.category_id + '" class="indicatorListItem" >' +
					'<input id="' + chk_dom_id + '" alarm_item_selector="' + indicator_id + '" type="checkbox" ' + checked + ' class="indicatorCheckbox" />' +
					'<label for="' + chk_dom_id + '" class="indicatorName" ' + ' title="' + indicator.tooltip + '">' + label + '</label>' +
				'</li>'
			);

			var info = {indicator: indicator, scope: scope_info};
			$indicator.data("info", info);
			$indicator.find("[id='" + chk_dom_id +"']").data("info", info);

			var $category = $popup_list.find("[category_id='" + indicator.category_id + "']");
			if (!$category.is(":visible")) {
				$category.show();
			}
			var $alarm_list = $category.find("[alarms_list='true']");
			$alarm_list.append($indicator);
		});

		this.__show_selector_indicators();
	};

	SupAlarmsConfig.prototype.__show_selector_indicators = function()
	{
		$("#" + this.container_selector_indicators_id).show();
		this.m_scope_popup.disable();
		$("#" + this.show_container_selector_indicators_id).prop("disabled", true);
	};

	SupAlarmsConfig.prototype.__hide_selector_indicators = function()
	{
		$("#" + this.container_selector_indicators_id).hide();
		this.m_scope_popup.enable();
		$("#" + this.show_container_selector_indicators_id).prop("disabled", false);
	};

	/*
		Carga en el popup selector la lista de indicadores del sistema para el scope seleccionado.
	*/
	SupAlarmsConfig.prototype.__remove_alarm_to_list = function(alarm)
	{
		var $added_alarms_in_scope = this.__$get_added_alarms_by_scope(alarm);
		var unique_id = this.__make_unique_id(alarm);
		var $alarm = $added_alarms_in_scope.find("[unique_id='" + unique_id + "']");

		if ($alarm.length > 0) {
			this.__remove_alarm(unique_id);
			return true;
		}

		return false;
	};

	/*
		Actualiza la lista de alarmas del usuario (markup), con las que se han seleccionado en el popup selector.
	*/
	SupAlarmsConfig.prototype.update_in_markup_selected_alarms = function()
	{
		var self = this;

		var $popup_list = $("#" + this.popup_selector_indicators_list_id);
		var $selected_indicators = $popup_list.find("[alarm_item_selector]");

		$.each($selected_indicators, function(_, selector)
		{
			var $selector = $(selector);
			var info = $selector.data("info");
			var alarm = {
				indicator_id: info.indicator.id,
				scope       : info.scope.scope, 
				campaign_id : info.scope.campaign_id,
				account_id  : info.scope.account_id,
				al_id       : info.scope.al_id
			};
			if ($selector.is(":checked")) {
				self.__add_alarm_to_list(alarm);
			}
			else {
				self.__remove_alarm_to_list(alarm);
			}
		});
	};

	/*
		Genera el markup para cada una de las categorias.
	*/
	SupAlarmsConfig.prototype.__make_html_categories = function(must_include_all_selector)
	{
		var sorted_categories = this.__get_sorted_categories();
		var html_categories = "";

		$.each(sorted_categories, function (category_id, category_def)
		{
			var html_all_selector = "";
			var attr_reference = "";
			if (must_include_all_selector) {
				var chk_dom_id = inConcert.Utils.generateGuid(); // este id es utilizado unicamente para referenciar el chk con el label con el attr "for"
				html_all_selector = "<input id='" + chk_dom_id + "' popup_alarm_all_items_selector='true' type='checkbox' class='alarmCheckbox' hidden />";
				attr_reference = " for='" + chk_dom_id + "' ";
			}
			html_categories+=
			"<div category_id='" + category_id + "' class='alarmCategoryContainer' hidden >" +
				"<div>" +
					"<span class='alarmCategoryTitle'>" + category_def.desc + "</span>" +
				"</div>" +
				html_all_selector + (must_include_all_selector? "<label " + attr_reference + " class='alarmCategorySelectAll'>" + "<%=_T([[Select all]])%>" + "</label>":"") +
				"<ul alarms_list='true' class='alarmList'></ul>" +
			"</div>";
		});

		return html_categories;
	};

	/*
		El markup contenedor para la configuracion.
	*/
	SupAlarmsConfig.prototype.__build_markup = function()
	{
		var translated = {
			cancel: "<%=_T([[Cancel]])%>",
			close: "<%=_T([[Close]])%>",
			condition_title: "<%=_T([[Alarms]])%>",
			condition_tooltip: "<%=_T([[Here you configure the conditions selected in the previous section.]])%>",
			scope_condition_tooltip: "<%=_T([[Select the account you want to alarm and then choose the conditions you are going to add. You can add conditions to more than one account separately.]])%>",
			error_msg: "<%=_T([[No alarms configuration yet.]])%>",
			save: "<%=_T([[Save]])%>",
			select_indicators: "<%=_T([[Select Indicators ...]])%>",
			scope_title: "<%=_T([[Alarms: Scope & Indicators Selection]])%>",
			update: "<%=_T([[Update]])%>",
			add: "<%=_T([[Add]])%>"
		};

		var $markup = $(
			//'<div class="card-header d-flex flex-row justify-content-between">' +
			//	'<h5>' + translated.scope_title + '</h5>' +
			//'</div>' +

			//form
			'<div class="card">'+
				'<div class="box-inside-card pb-0">'+
					'<div class="card-header">'+
						'<h5>'+
							'<%=_T([[Scope and conditions]])%>'+
							'<i class="far fa-question-circle" title="'+translated.scope_condition_tooltip+'"></i>'+
						'</h5>'+
					'</div>' +
					'<div class="card-block">' +
						'<div class="row">' +
							'<div class="col-12">' +
								'<div class="form-group input-button-form-group alarmConditionContainer">' +
									// En este contenedor se reenderizan los controles del combo personalizado "campaign selector"
									'<div class="input-form-group pr-2 mw33" id="' + this.popup_campaigns_selector_id + '" placeholder="<%=_T([[Select account]])%>"></div>' +
									'<div class="button-form-group buttonAddCondition">' +
										'<button id="' + this.show_container_selector_indicators_id + '" class="pageButtonGeneric buttonColorWhite " type="button" disabled="true">' +
									//	+ translated.select_indicators + 
											'<%=_T([[Add condition]])%>' +
											//'<i class="far fa-plus"></i>' +
										'</button>' +
									'</div>' +
								'</div>' +
							'</div>' +
						'</div>' +
						'<div id="' + this.container_selector_indicators_id + '" class="alarmFormRow" hidden>' +
							'<div class="col-12">' +
								'<div class="row">' +
									'<div id="' + this.popup_selector_indicators_list_id + '" class="indicatorSelectionList"></div>' +
				
									'<div class="d-flex justify-content-end w-100 bottomActions">' +
										'<button id="' + this.cancel_update_indicators_id + '" class="pageButtonGeneric buttonColorWhite alarmButtonCancel" type="button">' + translated.cancel + '</button>' +
										'<button id="' + this.update_indicators_id + '" class="pageButtonGeneric buttonColorBlue mr-3" type="button">' + translated.add + '</button>' +
									'</div>' +
								'</div>' +
							'</div>' +
						'</div>' +
					'</div>' +
				'</div>' +
			// contenedor de lista de alarmas configuradas
				'<div class="box-inside-card">' +
					'<div class="card-header">' +
							'<h5>' + translated.condition_title + '<i class="far fa-question-circle" title="'+translated.condition_tooltip+'"></i></h5>' +
					'</div>' +

					'<div id="' + this.list_categories_id + '" class=" flex-column d-flex">' +
						'<div id="' + this.message_no_alarms_id + '" class="inlineMessagesContainer messageInfo">' + translated.error_msg + '</div>' +
						// Aca va la lista de categorias vacias y ocultas
						'<div class="card-block">'+
							this.__make_html_categories(false) +
							// acciones (save, discard, etc ?)
								
						'</div>' +
					'</div>' +
				'</div>' +

				'<div class="d-flex justify-content-end alarmPageButtons">' +
					'<button id="' + this.close_id + '" class="pageButtonGeneric buttonColorWhite" type="button">' + translated.close + '</button>' +
					'<button id="' + this.save_id + '" class="pageButtonGeneric buttonColorBlue" type="button">' + translated.save + '</button>' +
				'</div>' +
				
			'</div>'
		);

		var $container = this.__$container();
		$container.html($markup);
	};

	/*
		Valida si se puede o no remover una alarma que se ha configurado para el usuario.
	*/
	SupAlarmsConfig.prototype.__can_remove_alarm = function(/*unique_id*/)
	{
		// TODO: ver si hay que implementar
		return true;
	};

	/*
		Remueve del markup una alarma configurada para el usuario.
	*/
	SupAlarmsConfig.prototype.__remove_alarm = function(unique_id)
	{
		if (!this.__can_remove_alarm(unique_id)) {
			return false;
		}
		var $alarm = this.__$find_alarm(unique_id);
		$alarm.remove();
		this.__hide_empty_categories();
		return true;
	};

	SupAlarmsConfig.prototype.__set_events = function()
	{
		var self = this;

		var $btn_show_selector_indicators = $("#" + this.show_container_selector_indicators_id);

		$btn_show_selector_indicators
			.off('click')
			.on('click', function() {
				self.__load_indicators_by_scope();
			});

		$("#" + this.update_indicators_id)
			.off('click')
			.on('click', function() {
				self.update_in_markup_selected_alarms();
				self.__hide_selector_indicators();
			});

		$("#" + this.cancel_update_indicators_id)
			.off('click')
			.on('click', function() {
				self.__hide_selector_indicators();
			});

		var $save = $('#' + this.save_id);
		$save
			.off('click')
			.on('click', function() {
				$save.prop("disabled", true);
				self.__save_config(function() {
					$save.prop("disabled", false);
				});
			});

		var $back = this.m_dashboard.$back_button(this.m_campaign_id);
		$('#' + this.close_id)
			.off('click')
			.on('click', function() {
				$back.trigger('click'); // este evento sirve tanto en modo simple screen como en full screen
			});

		var $categories = this.__$list_categories_container();

		var event_selector = "[alarm_item] [alarm_item_selector]";
		$categories
			.off("change", event_selector)
			.on("change", event_selector, function ()
			{
				// Habilito o inhabilio los controles asociados.
				var $this = $(this);
				var checked = $this.is(":checked");
				var indicator_id = $this.attr("alarm_item_selector");
				var $ctrls = $this.parent().find("[alarm_item_ctrl='" + indicator_id + "']");
				var disabled = !checked;

				$.each($ctrls, function(_, ctrl)
				{
					var $ctrl = $(ctrl);
					var alarm_item_ctrl_type = $ctrl.attr("alarm_item_ctrl_type");
					if (alarm_item_ctrl_type === 'alarm_unit') {
						// Si es un control de Unidad, y tiene un unico elemento, lo dejo siempre inhabilitado, para que quede como label.
						var $options = $ctrl.find("option");
						if ($options.length <= 1) {
							$ctrl.prop("disabled", true);
							return true; // continue
						}
					}
					$ctrl.prop("disabled", disabled);
				});
			});

		event_selector = "[alarm_item] [alarm_item_remove]";
		$categories
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				var unique_id = $(this).attr("alarm_item_remove");
				self.__remove_alarm(unique_id);
			});

		var $popup_list = $("#" + this.popup_selector_indicators_list_id);
		event_selector = "[popup_alarm_all_items_selector]";
		$popup_list
			.off("change", event_selector)
			.on("change", event_selector, function ()
			{
				var $this = $(this);
				var is_checked = $this.prop("checked");

				var $popup_alarms_container = $this.parent().find("[alarms_list='true']");
				$.each($popup_alarms_container.find("[alarm_item] [alarm_item_selector]"), function(_, alarm)
				{
					var $alarm = $(alarm);
					$alarm.prop("checked", is_checked);
				});
			});
	};

	/*
		Oculta todos los errores de la validacion de las alarmas del usuario.
	*/
	SupAlarmsConfig.prototype.__hide_all_alarms_errors = function()
	{
		var $all_alarms = this.__$list_all_alarms();
		const $alarm_item_ctrl = $all_alarms.find("[alarm_item_ctrl]");
		$alarm_item_ctrl.attr("title", "");
		$alarm_item_ctrl.removeClass("form-control-error");
	};

	/*
		Valida en el markup si se han completado los campos correctamente.
		Si no es asi, marca el error.
	*/
	SupAlarmsConfig.prototype.__can_save = function()
	{
		var response = { success: true };

		this.__hide_all_alarms_errors();

		var has_permission = this.m_dashboard.has_permission_to("sup_view_alarms_config", this.m_campaign_id);
		if (!has_permission) {
			inConcert.app.getMainWindow().showBannerMessage({ 
				message : "<%=_T([[NOT HAS PERMISSIONS TO SAVE ALARMS]])%>",
				timeout : 5000
			});
			response.success = false;
			response.show_field_msg_err = false;
			return response;
		}
		var $all_alarms = this.__$list_all_alarms();
		var mark_error = function($alarm, tooltip, alarm_item_ctrl_type) {
			response.show_field_msg_err = true;
			response.success = false;
			response.number_of_errors = (response.number_of_errors || 0) + 1;
			const $alarm_item_ctrl_type = $alarm.find(`[alarm_item_ctrl_type=${alarm_item_ctrl_type}]`);
			$alarm_item_ctrl_type.addClass("form-control-error");
			$alarm_item_ctrl_type.attr("title", tooltip);
		};

		$.each($all_alarms, function(_, alarm)
		{
			var $alarm = $(alarm);
			var is_enabled = $alarm.find("[alarm_item_selector]").is(":checked");
			if (!is_enabled) {
				return true; // continue
			}

			// operador
			var $operator = $alarm.find("[alarm_item_ctrl_type='operator']");
			var value = $operator.val();
			var valid_result = inConcert.Utils.validate(value, "required");
			if (!valid_result.success) {
				mark_error($alarm, "<%=_T([[Operator is required.]])%>", "operator");
				return true; // continue
			}

			// valor
			var $alarm_value = $alarm.find("[alarm_item_ctrl_type='alarm_value']");
			value = $alarm_value.val();
			valid_result = inConcert.Utils.validate(value, "required");
			if (!valid_result.success) {
				mark_error($alarm, "<%=_T([[Value is required.]])%>", "alarm_value");
				return true; // continue
			}

			valid_result = inConcert.Utils.validate(value, "number");
			if (!valid_result.success) {
				mark_error($alarm, "<%=_T([[Only numbers are allowed.]])%>", "alarm_value");
				return true; // continue
			}

			// otros valores (combo que no existe siempre)
			var $alarm_unit = $alarm.find("[alarm_item_ctrl_type='alarm_unit']");
			if ($alarm_unit.length > 0 && $alarm_unit.is(":visible")) {
				value = $alarm_unit.val();
				valid_result = inConcert.Utils.validate(value, "required");
				if (!valid_result.success) {
					mark_error($alarm, "<%=_T([[Time unit is required.]])%>", "alarm_unit");
					return true; // continue
				}
			}
		});

		if (!response.success) {
			response.message = format("<%=_T([[Unable to save alarms on campaign: {0}. There are {1} alarms with errors]])%>", this.m_campaign_id, response.number_of_errors);
		}

		return response;
	};

	/*
		Guarda server side la configuracion de alarmas del usuario.
	*/
	SupAlarmsConfig.prototype.__save_config = function(callback = $.noop)
	{
		const self = this;

		let response = this.__can_save();
		if (!response.success) {
			if (response.show_field_msg_err) {
				let message = response.message || format("<%=_T([[Unable to save alarms on campaign: {0}.]])%>", this.m_campaign_id);
				this.__show_error_message(message);
			}
			callback();
			return;
		}
		this.__update_alarms_to_save();
		this.m_dashboard.save_config(function() {
			let message = "<%=_T([[Alarms have been saved successfully on campaign: {0}.]])%>";
			self.__show_info_message(format(message, self.m_campaign_id), false);
			const current_config = self.get_config();
			self.set_config(current_config);
			callback();
		});
	};

	SupAlarmsConfig.prototype.__show_info_message = function(message) {
		inConcert.app.m_mainWindow.showBannerMessage({messageType: "Info", message: message, timeout: 3000});
	};

	SupAlarmsConfig.prototype.__show_error_message = function(message) {
		inConcert.app.m_mainWindow.showBannerMessage({messageType: "Error", message: message, timeout: 5000});
	};

	/**
	 * Retorna la configuracion de las alarmas definidas para la campana
	 *
	 * @public
	 * @return {object} - campaign alarms config: Ej:
		{
			indicators: {
				"campaign": {
					"indActiveAgents": {
					  "enabled": true,
					  "value": 14041996,
					  "operator": "DISTINCT"
					},
					indMaxWaitingCall: {
						value: 1,
						section: "SECOND",
						enabled: false,
						operator: "DISTINCT"
					},
					..
				},
				accounts : {..
					cuenta1 = {
						indicators: {
							"indActiveAgents": {
							  "enabled": true,
							  "value": 14041996,
							  "operator": "DISTINCT"
							},
							indMaxWaitingCall: {
								value: 1,
								section: "SECOND",
								enabled: false,
								operator: "DISTINCT"
							}
						}
						,att_levels: {
							al1: {
								indicators: {
									"indActiveAgents": {
									  "enabled": true,
									  "value": 14041996,
									  "operator": "DISTINCT"
									},
									indMaxWaitingCall: {
										value: 1,
										section: "SECOND",
										enabled: false,
										operator: "DISTINCT"
									}
								}
							}
						}
					}
				}
			}
	 */
	SupAlarmsConfig.prototype.get_config = function()
	{
//var alarms = {indicators: {campaign: {}, accounts: {}}, notifications: {}};
		var alarms = {indicators: {campaign: {}, accounts: {}}};

		$.each(this.m_sorted_categories || {}, function(_/*category*/, category_def)
		{
			$.each(category_def.list, function(_, alarm)
			{
				var indicator_id = alarm.indicator_id;
				if (alarm.scope === "campaign") {
					alarms.indicators.campaign[indicator_id] = alarm;
				}
				else if (alarm.scope === "channels") {
					alarms.indicators.accounts[alarm.account_id] = alarms.indicators.accounts[alarm.account_id] || {
						indicators: {}, att_levels: {}
					};
					alarms.indicators.accounts[alarm.account_id].indicators[indicator_id] = alarm;
				}
				else if (alarm.scope === "att_levels") {
					alarms.indicators.accounts[alarm.account_id] = alarms.indicators.accounts[alarm.account_id] || {
						indicators: {}, att_levels: {}
					};
					alarms.indicators.accounts[alarm.account_id].att_levels[alarm.al_id] = alarms.indicators.accounts[alarm.account_id].att_levels[alarm.al_id] || {
						indicators: {}
					};
					alarms.indicators.accounts[alarm.account_id].att_levels[alarm.al_id].indicators[indicator_id] = alarm;
				}
			});
		});
		return alarms;
	};

	SupAlarmsConfig.prototype.set_config = function(alarms)
	{
		this.m_currents_alarms = alarms;
	};

	/*
		Valida si se ha configurado al menos una alarma para la campana, cuenta, al
	*/
	SupAlarmsConfig.prototype.has_configured_alarms = function()
	{
		var currents_alarms = this.__get_currents_alarms();
		if (!currents_alarms) {
			return false;
		}

		var indicators = currents_alarms.indicators;
		if (!indicators) {
			return false;
		}

		var has_configured_alarms = false;
		// valido contra la campana
		$.each(indicators.campaign || {}, function () {
			// si tiene al menos una me voy
			has_configured_alarms = true;
			return false; // break
		});
		if (has_configured_alarms) {
			return true;
		}

		// valido contra las cuentas
		$.each(indicators.accounts || {}, function () {
			// si tiene al menos una me voy
			has_configured_alarms = true;
			return false; // break
		});

		if (has_configured_alarms) {
			return true;
		}

		return false;
	};

}());
