/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupervisorAbandoned
 */
(function () {

	'use strict';

	const _T = inConcert.i18n.getString;

	/**
	 * Creates a new dashboard abandoned instance.
	 *
	 * @author Sebastián Moreno (smoreno@inconcertcc.com)
	 * @constructor
	 * @memberOf SupervisorAbandoned
	 * @param dashboard_instance
	 */
	function DashboardAbandoned(dashboard_instance)
	{
		DashboardAbandoned.uber.constructor.call(this, 'abandoned', "<%=_T('Abandoned')%>", dashboard_instance);

		this.permission_id = 'sup_view_abandoned';
		this.navigation_folder_id = 'abandoned';
		this.m_view_classes_name = 'abandonedViewer';
		this.m_model = dashboard_instance.get_model_manager();
		// flag to know if i'm searching the abandoned.
		this.m_searching_abandoned = {};
		this.m_page_abandoned = {};
		this.m_dashboard = dashboard_instance;
		// current config
		this.m_config = dashboard_instance.m_config;
	}

	inConcert.Utils.inherit(DashboardAbandoned, inConcert.sup_dashboard_interactions_view);

	if (typeof inConcert.sup_dashboard_abandoned !== 'function') {
		inConcert.sup_dashboard_abandoned = DashboardAbandoned;
	}

	DashboardAbandoned.prototype.__get_interaction_gui_config = function()
	{
		var self = this;

		var config = DashboardAbandoned.uber.__get_interaction_gui_config.call(this);
		config.allowed_bulk_actions = false;

		config.get_specifics_interaction_classes = function( interaction, $interaction ) {
			return self.get_specifics_interaction_classes( interaction, $interaction );
		};

		return config;
	};

	DashboardAbandoned.prototype.on_change_config_selector = function( campaign_id )
	{
		DashboardAbandoned.uber.on_change_config_selector.call(this, campaign_id);	
	};

	DashboardAbandoned.prototype.__must_show_alarm = function (campaign_id, interaction)
	{
		var category = "conversation_time_on_state";
		return this.m_dashboard.eval_notifications_alarms(campaign_id, category, interaction);
	};

	DashboardAbandoned.prototype.get_specifics_interaction_classes = function( interaction, $interaction )
	{
		if ( typeof interaction !== 'object' ) {
			return '';
		}

		var classes = this.__get_alarm_classes(interaction, $interaction);
		return classes;
	};

	DashboardAbandoned.prototype.__get_prefix = function( /*interaction*/ )
	{
		return 'supervisor_int_abandoned_';
	};

	DashboardAbandoned.prototype.__get_dom_interaction_params = function( interaction )
	{
		return {
			prefix: this.__get_prefix( interaction ),
			allowed_bulk_actions: false // TODO: agregar permiso ?
		};
	};

	DashboardAbandoned.prototype.search_abandoned = function ($current_node, is_new_search, callback = $.noop)
	{
		const self = this;

		let campaign_id = $current_node.attr('campaign_id');

		var search_query = this.m_dashboard.getQueryBuilderFilter(campaign_id);
		if(!search_query) {
			return;
		}

		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		if (this.m_searching_abandoned[campaign_id]) {
			return;
		}
		// Dejo disabled la navegacion, hasta que culmine de procesar el request
		let navigation = this.get_navigation(campaign_id);
		if (navigation) {
			navigation.disable_navigation(true);
		}

		this.m_searching_abandoned[campaign_id] = true;
		if ( is_new_search ) {
			this.clear_list(campaign_dom_id);
			this.show_waiting(campaign_dom_id, "<%=_T('Searching abandoned, please wait.')%>");
		}

		var on_search_completed = function() {
			if ( is_new_search ) {
				self.disable_batch_actions(campaign_id);
			}
			self.m_searching_abandoned[campaign_id] = false;
			self.hide_waiting(campaign_dom_id);
			if (navigation) {
				// Dejo enabled la navegacion, ya que ha terminado de procesar el Request.
				navigation.disable_navigation(false);
			}
			callback();
		};

		var page_number = this.m_page_abandoned[campaign_id] && this.m_page_abandoned[campaign_id].filter.page ? this.m_page_abandoned[campaign_id].filter.page : 1;
		var $dynamic = $('#sup_dash_interaction_viewer_dynamic_' + campaign_dom_id);

		//Pills
		var $pillContainer = $dynamic.find(".form-group-pill-card-outlined");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(search_query && search_query.rules && $.isArray(search_query.rules) && search_query.rules.length > 0)
		{
			var addRemoveButtonClick;
			addRemoveButtonClick = function(ruleId) {
				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("sup_abandoned" + campaign_dom_id + "_pill_", ""));
					//Eliminar del queryBuilder
					if(self.m_dashboard.queryBuilderObj[campaign_dom_id].deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			};
			for (var i = 0; i < search_query.rules.length; i++) {
				var ruleId = "sup_abandoned" + campaign_dom_id + "_pill_" + i;
				var rulePillText = _T(search_query.rules[i].label || search_query.rules[i].id) + " | " + _T(search_query.rules[i].operator) + " | " + "<strong>" + search_query.rules[i].value + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				addRemoveButtonClick(ruleId);				
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}		

		//labels
		let selected_labels = navigation.get_selected_labels();
		if (selected_labels && $.isArray(selected_labels) && selected_labels.length > 0) {
			var labelsRule = {
				id : "labels",
				operator : "in",
				value : selected_labels
			};
			search_query.rules.push(labelsRule);
		}

		if(!search_query.rules.find(rule => rule.id === "campaign")) {
			//campaign
			var campaignRule = {
				id : "campaign",
				operator : "equal",
				value : campaign_id
			};
			search_query.rules.push(campaignRule);
		}

		search_query.rules = this.m_dashboard.getCommonConfigRules(search_query.rules, campaign_id);

		var filter = {
			//campaign_id        : campaign_id,
			//account_id         : campaign_config.account_id,
			//attention_level_id : campaign_config.al_id,
			page               : page_number,
			rows_per_page      : this.INTERACTIONS_PER_PAGE,
			query              : search_query,
			sort               : [{state_timestamp: { order: "asc" }, priority: {order: "desc"}}]
		};

		this.m_page_abandoned[campaign_id] = {
			paginating: false,
			filter    : filter
		};

		this.m_model.search_abandoned_interactions(filter, function(success, result) {
			self.m_page_abandoned[campaign_id].paginating = false;
			if ( success ) {
				$dynamic.find("[query_builder_clean]").prop("disabled", !self.m_dashboard.queryBuilderObj[campaign_dom_id].hasRules());
				// Show empty list conversations.
				if ( result.interactions.length === 0 && result.total === 0 ) {
					self.clear_list(campaign_dom_id);
					self.toggle_no_conversations_msg(campaign_dom_id, "<%=_T('Abandoned conversations list is empty. You can try searching conversations by text.')%>");
					on_search_completed();
				}
				else {
					if (self.view_has_changed(campaign_id)){
						on_search_completed();
					}
					else {
						self.remove_no_conversations_msg(campaign_dom_id);
						if ( is_new_search ) {
							self.clear_list(campaign_dom_id);
							var $container = self.get_interactions_container(campaign_dom_id);
							result.interactions.forEach(function(interaction)
							{
								var parmams_interaction = self.__get_dom_interaction_params(interaction);
								var $interaction_node = self.m_interaction_gui.$build_interaction_node( interaction, self.m_templates.interaction_node_tpl, parmams_interaction );
								self.insert_interaction_in_panel_list( interaction, $interaction_node, $container );
							});
						}
						else {
							// go and process the conversations
							self.process_off_dom_interaction_update(
								campaign_dom_id,
								result.interactions,
								self.m_templates.interaction_node_tpl,
								function(interaction, $interaction_node) {
									// update interaction on panel
									self.m_interaction_gui.update_interaction_data( interaction, $interaction_node );
								}
							);
						}
						on_search_completed();
					}
				}
				// Update pagination info.
				self.m_page_abandoned[campaign_id].totalRows = result.total || 0;
				self.m_page_abandoned[campaign_id].last_page = Math.ceil(result.total / self.INTERACTIONS_PER_PAGE);
				self.update_current_pagination(campaign_id, result.interactions.length || 0);
			}
			else {
				inConcert.app.getMainWindow().showBannerMessage({ message : "<%=_T('Error server finding abandoned conversations')%>", timeout : 5000});
				on_search_completed();
			}
		});
	};

	/**
	 * Will refresh the list of abandoned conversations
	 *
	 * @async
	 * @public
	 * @param  {Object} $current_node - The current jquery campaign object.
	 * @param callback
	 */
	DashboardAbandoned.prototype.refresh = function($current_node, callback)
	{
		callback = callback || $.noop;

		var campaign_id = $current_node.attr('campaign_id');

		if (this.view_has_changed(campaign_id)) {
			callback();
			return;
		}
		if ( !this.m_page_abandoned[campaign_id] ) {
			callback();
			return;
		}
		var current_filter = this.m_page_abandoned[campaign_id].filter;
		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);
		current_filter.account_id = campaign_config.account_id;
		current_filter.attention_level_id = campaign_config.al_id;
		this.m_page_abandoned[campaign_id].filter = current_filter;

		var is_new_search = false;
		this.search_abandoned($current_node, is_new_search, callback);
	};

	DashboardAbandoned.prototype.update_current_pagination = function(campaignId, pageInteractions)
	{
		let pageInfo = this.m_page_abandoned[campaignId];
		if (!pageInfo) {
			return;
		}
		let currentPage = pageInfo.filter.page;
		let lastPage = pageInfo.last_page;
		let totalRows = pageInfo.totalRows;

		DashboardAbandoned.uber.update_current_pagination.call(this, campaignId, currentPage, lastPage, pageInteractions, totalRows);
	};

	DashboardAbandoned.prototype.execute_new_search = function(campaign_id, callback, cleanup_search_text)
	{
		callback = callback || $.noop;

		if (cleanup_search_text) {
			this.cleanup_search_text(campaign_id);
		}

		if ( this.m_page_abandoned[campaign_id] ) {
			this.m_page_abandoned[campaign_id].filter.page = 1;
		}
		var $current_node = this.m_dashboard.get_campaign_node(campaign_id);
		var is_new_search = true;
		this.search_abandoned($current_node, is_new_search, function() {
			callback();
		});
	};

	DashboardAbandoned.prototype.get_view_paged_info = function()
	{
		return this.m_page_abandoned;
	};

	/**
	 * Will initialize the abandoned view, will get the conversations
	 * from the server and build the corresponding nodes.
	 *
	 * @async
	 * @public
	 * @param  {object}   $current_node - The current campaign node (jquery node)
	 * @param  {Function} callback     - A callback to be called once the initialization is finished
	 */
	DashboardAbandoned.prototype.init = function($current_node, callback)
	{
		DashboardAbandoned.uber.init.call(this, $current_node, { has_batch_actions : false });

		var self = this;
		callback = callback || $.noop;
		var campaign_id = $current_node.attr('campaign_id');
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		// Clear previous conversations (monitoring/archive) and show abandoned
		this.clear_list(campaign_dom_id);
		this.show_waiting(campaign_dom_id, 'Searching abandoned, please wait.');

		var $search = $current_node.find('[id="sup_dash_interaction_viewer_dynamic_' + campaign_dom_id + '"]');
		var $search_box = $search.find('[filter_search]');
		var $search_button = $search.find('[filter_search_execute]');
		var $chk_exact_search = $search.find('[id="exactSearchCheck"]');

		$search.find('[id="exactSearchCheck"],[for="exactSearchCheck"]').show();

		// The abandoned can be refreshed on demand (because it does not automatically refresh
		// because it is an expensive operation).
		$search_button.prop('disabled', false);
		$chk_exact_search.prop('disabled', true);

		// setup search button behaviour.
		$search_box.unbind('keyup.search').bind('keyup.search', function(){
			var facets = self.get_filter_facets(campaign_id);
			var searching_text = facets.some(function(facet){ return facet.text; });
			if (!searching_text){
				$search_button.prop('disabled', false);
				$chk_exact_search.prop('disabled', false);
			}
		});

		$search_button.unbind('click').bind('click', function() {
			self.execute_new_search(campaign_id);
		});

		// setup custom abandoned pagination.
		$chk_exact_search.unbind('click').bind('click', function() {
			self.execute_new_search(campaign_id);
		});

		this.setup_view_pagination($current_node, campaign_id);
		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);

		this.m_page_abandoned[campaign_id] = {
			paginating : false,
			filter : {
				campaign_id        : campaign_id,
				account_id         : campaign_config.account_id,
				attention_level_id : campaign_config.al_id,
				page               : 1,
				rows_per_page      : this.INTERACTIONS_PER_PAGE
			}
		};
		this.search_abandoned($current_node, true, function() {
			self.hide_waiting(campaign_dom_id);
			self.on_view_load_finished(campaign_id);
			callback();
		});
	};
}());
