/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true, strict:true, undef:true, unused:true,
	curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert, Mustache, _*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupervisorDashboardMembers
 */
(function () {

	//	'use strict';
	const format = inConcert.Utils.format;
	const nid = inConcert.Utils.getNormalizedId;
	const setMemberMissingSkill = function($member_skills_container, skills_detail, not_force_hide = true)
	{
		const $agent_mskills_val = $member_skills_container.find("[title]");

		if (skills_detail && skills_detail.missingSkillType && not_force_hide)
		{
			let message ="<%=_T('Agent without required skills.')%>";

			switch (skills_detail.missingSkillType) {
				case 1: // Only one missing skill in one account
					message = format("<%=_T('The agent does not have the required skill: {0}.')%>", skills_detail.firstMissingSkill);
				break;
				case 2: // Multiple missing skills in one account
					message = "<%=_T('The agent does not have several of the required skills.')%>";
				break;
				case 3: // One missing skill in various accounts
					message = format("<%=_T('The agent not have the required skill: {0} in several accounts.')%>", skills_detail.firstMissingSkill);
				break;
				case 4: // Multiple missing skills in various accounts
					message = "<%=_T('The agent does not have several of the required skills in several accounts.')%>";
				break;
			}

			$agent_mskills_val.attr('title',message);
			$member_skills_container.show();
		}
		else {
			$agent_mskills_val.attr('title','');
			$member_skills_container.hide();
		}
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class popup_member_detail.
	* @param params
	*/
	function popup_member_detail (params)
	{
		popup_member_detail.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "supervisor_members";
		this.ownerId = "window_sup_members_popup_container";
		this.canReposition = true;
		this.from_app = params.from_app;
		this.owner = params.owner;
		this.m_member = params.member;
		this.$member = params.$member;
		this.m_campaign_id = params.campaign_id;
		this.$campaign = params.$campaign;
	}

	inConcert.Utils.inherit(popup_member_detail, inConcert.PopupWindow);

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	popup_member_detail.prototype.set_events = function()
	{
		const self = this;

		// evento que cierra el popup
		var close_popup = function() { self.close(); };
		$("#window_sup_members_popup_close").bind("click", close_popup);

		$("#window_sup_members_popup_close_message")
			.bind("click", function() {
				self.hide_status_message();
			});

		let $status_list_node = $("#window_sup_members_popup_menu_change_status");
		$status_list_node.find('.statusList > li').unbind('click').bind('click', function() {
			// hide status menu
			$status_list_node.hide();
			const $state = $(this);
			self.__change_member_state($state);
		});
	};

	/*******************************************************************
	* Metodo que crea el markup contenedor de la ventana
	*/
	popup_member_detail.prototype.create_markup = function()
	{
		// tell the user wath are we doing
		//let title = format("<%=_T('User Detail: {0}')%>", this.m_member.ID);

		$("#window_sup_members_popup_container").remove();
		const $node = $("".concat(
			"<div id='window_sup_members_popup_container' class='windowSupervisorMemberDetailContainer windowContainer windowMax480' hidden>",

				"<div id='window_sup_members_popup_waiting' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

//				"<div class='windowTop'>",
//					"<div class='windowTitle windowSupervisorMemberDetail'>",
//						"<div id='window_sup_members_popup_title' class='windowSupervisorMemberDetailTitle'>", title , "</div>",
//						"<a id='window_sup_members_popup_close' class='windowMessagesClose' href='javascript:void(0);'></a>",
//					"</div>",
//				"</div>",

				"<div id='window_sup_members_popup_message' class='windowContainerMessages messageError' hidden>",
					"<a id='window_sup_members_popup_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('Close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div class='memberTopInfoContainer'>",
						"<div class='memberAvatarContainer'>",
							"<img id='window_sup_members_popup_avatar' src='' class='memberAvatar'/> ",
						"</div>",
						"<div id='window_sup_members_popup_status' class='memberStatusContainer'>" +
							"<img src='/inconcert/apps/agent/images/blank.png' class='memberStatus'/>" +
						"</div>" +
						"<div class='memberInfoContainer'>",
							"<div class='teamListItemBoxLeft'>",
								"<div id='window_sup_members_popup_user_name' class='memberName'></div>",
								"<div id='window_sup_members_popup_user_id' class='memberId'></div>",
								"<div class='d-flex afterImage'>",
									"<div total_conversations_val='true' class='memberPopUpFieldset'>",
										"<label class='teamWorkgroup'>", "<%=_T('Interactions')%>", "<%=_T(':')%> ", "</label>",
										"<span id='window_sup_members_popup_total_conversations' class='teamWorkgroup' ></span>",
									"</div>",
									"<div id='window_sup_members_popup_workgroup' class='memberPopUpFieldset'>",
										"<label class='teamWorkgroup'>", "<%=_T('Group')%>", "<%=_T(':')%> ", "</label>",
										"<label workgroup_val  class='teamWorkgroup'></label>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='teamListItemBoxRight'>",
								"<div class='memberStatusBox'>",
									"<span id='window_sup_members_popup_status_name' class='memberStatusName'></span>",
								"</div>",								
								"<div id='window_sup_members_popup_time' class='memberActivity'></div>",
								"<div id='member_detail_state_container' class='memberName memberBreakTime' style='display:none; max-width: 100% !important;'></div>",
								"<div class='memberActivityContainer'>",
									"<div id='agentMissingSkills' class='memberMissingSkills'>",
										"<span class='fa fa-star-exclamation' title=''></span>",
									"</div>",
									"<div id='window_sup_members_popup_attention_level' class='memberPopUpFieldset'>",
										"<label class='teamAttentionLevel'>", "<%=_T('Attention Level')%>", "<%=_T(':')%>", "</label>",
										"<input attention_level_val='true' type='text' class='teamAttentionLevel' readonly='true' />",
									"</div>",
								"</div>",
							"</div>",
							"<div class='memberMiddleInfoContainer'></div>",
						"</div>",
					"</div>",
					"</div>",
					"<div id='window_sup_members_popup_conversations' class='teamInteractions'></div>",
					"<div id='window_sup_members_popup_menu_change_status' class='popupStatusList' style='display:none'></div>",
					"<div id='window_sup_members_popup_actions' class='teamActions'>",
						"<div>",
							"<button action_id='member_chat' class='buttonToggle butTeamChat' type='button' hidden><i class='far far fa-comment-alt'></i></button>",
							"<button action_id='member_monitor' class='buttonToggle butTeamMonitor' type='button' hidden><i class='far fa-eye'></i></button>",
							"<button action_id='member_screen' class='buttonToggle butTeamScreen' type='button' hidden><i class='far fa-window-alt'></i></button>",
							"<button action_id='member_change_state' class='buttonToggle butTeamState' type='button' hidden><i class='far fa-flag-alt'></i></button>",
							"<button action_id='member_archive' class='buttonToggle butTeamArchive' type='button' hidden><i class='far fa-archive'></i></button>",
							"<button action_id='member_appointment' class='buttonToggle butTeamAppointment' type='button' hidden><i class='far fa-calendar-alt'></i></button>",
							"<button action_id='member_campaigns' class='buttonToggle butTeamCampaign' type='button' hidden><i class='far fa-bullhorn'></i></button>",
							"<button action_id='member_kickouts' class='buttonToggle butTeamKickoff' type='button' hidden><i class='far fa-user-slash'></i></button>",
							"<button action_id='member_loggoff' class='buttonToggle butTeamLogoff' type='button' hidden><i class='far fa-power-off'></i></button>",
						"</div>",
						"<a id='window_sup_members_popup_close' class='buttonToggle buttonToggleOutlined buttonClose' href='javascript:void(0);'><%=_T('Close')%></a>",
					"</div>",
					"<div id='window_sup_members_popup_actions_container' class='windowsContainerBox' hidden>",
					"<div id='window_sup_members_popup_kickouts' class='kickoutContainer' style='display:none'>",
						"<label class='teamWorkgroup'>", "<%=_T('Kickouts')%>", "<%=_T(':')%>", "</label>",
						"<table id='kickout_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid kickoutGrid'>",
							"<thead class='viewGridHeader'>",
								"<tr>",
									"<th>", "<%=_T('timestamp')%>", "</th>",
									"<th>", "<%=_T('Forced')%>", "</th>",
									"<th>", "<%=_T('UserID')%>", "</th>",
									"<th>", "<%=_T('Message')%>", "</th>",
								"</tr>",
							"</thead>",
							"<tbody class='viewGridBody'></tbody>",
						"</table>",
					"</div>",
					"<div id='window_sup_members_popup_campaigns' class='campaignsContainer' style='display:none'>",
//						"<label class='teamWorkgroup'>", "<%=_T('Campaigns')%>", "<%=_T(':')%>", "</label>",
						"<label>","<%=_T('Campaigns associated')%>","</label>",
						"<table id='campaigns_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid campaignsGrid'>",
							"<thead class='viewGridHeader'>",
								"<tr>",
									"<th class='numCol'>#</th>",
									"<th>", "<%=_T('Campaign')%>", "</th>",
									"<th>", "<%=_T('Description')%>", "</th>",
								"</tr>",
							"</thead>",
							"<tbody class='viewGridBody'></tbody>",
						"</table>",
					"</div>",
					"<div>",
				"</div>",

				//"<div class='windowBottom'></div>",
			"</div>"));

		$("body").append($node);
		return $node;
	};

	popup_member_detail.prototype.hide_status_message = function() {
		$("#window_sup_members_popup_message").hide();
	};

	/*******************************************************************
	*
	*/
	popup_member_detail.prototype.initialize = function()
	{
		this.data.container = this.create_markup();
		this.__setup_agent_status_popup();
		this.__set_member_info();
		// set events
		this.set_events();
	};

	/*******************************************************************
	* Fucion que es invocada por quien la ha instanciado.
	* La misma vailda si el Elemento DOM Member se encuentra visible; si el mismo NO se encuentra visible, el popup se cerrar de forma automatica.
	* Ejemplo, si tenemos el popup abierto, y el member fuera removido de la lista, porque el mismo cambia de estado y no matchea con el filtro; en este caso el popup se cerrara.
	*/
	popup_member_detail.prototype.auto_close = function()
	{
		if (this.$member && this.$member.length > 0) {
			if (!this.$member.is(":visible")) {
				this.close();
			}
		}
	};

	popup_member_detail.prototype.__setup_agent_status_popup = function()
	{
		let statuses = [];
		// build markup for each status
		let available_statuses = inConcert.app.getAvailableStates();
		available_statuses.forEach(state => {
			const status_view = {
				status_id : state.Id,
				status_name: state.Description
			};
			statuses.push(status_view);
		});

		// Build status group container
		const owner = this.getOwnerObject();
		let html_statuses = Mustache.render(owner.m_templates.agent_statuses_list_tpl, {title: "<%=_T('Change Agent Status')%>", list: statuses});
		$("#window_sup_members_popup_menu_change_status").html(html_statuses);
	};

	popup_member_detail.prototype.has_permission_to = function (action_id)
	{
		const owner = this.getOwnerObject();
		return owner.has_permission_to(action_id, this.m_campaign_id);
	};

	popup_member_detail.prototype.show_waiting = function(message)
	{
		const $waiting = $("#window_sup_members_popup_waiting");
		$waiting.find("[message='true']").text(message);
		$waiting.show();
	};

	popup_member_detail.prototype.hide_waiting = function()
	{
		$("#window_sup_members_popup_waiting").hide();
		this.setPosition();
	};

	popup_member_detail.prototype.show_status_message = function(text, type, closeDisabled, timeout_ms )
	{
		var $section = $("#window_sup_members_popup_message");
		$section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#window_sup_members_popup_close_message").hide();
		}
		else {
			$("#window_sup_members_popup_close_message").show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	popup_member_detail.prototype.hide_status_message = function() {
		$("#window_sup_members_popup_message").hide();
	};

	popup_member_detail.prototype.__action_logout = function(btn_config)
	{
		const self = this;
		if (!this.has_permission_to(btn_config.permission_id)) {
			return;
		}
		// tell the user wath are we doing
		this.show_waiting(format("<%=_T('Logging out agent: {0}')%>", this.m_member.ID));
		const owner = this.getOwnerObject();

		owner.m_model.logout_member(self.m_member.ID, this.m_campaign_id, owner.m_current_sup_id, function(ok, error)
		{
			self.hide_waiting();
			if (ok && !error ){
				self.close(); // cierro el popup

				inConcert.app.getMainWindow().showBannerMessage({ message : format("<%=_T('Agent {0} was succesfully logged out!')%>", self.m_member.ID), messageType : 'Info', timeout : 3000});

				let config = owner.m_config.get_panel_config(self.m_campaign_id, 'sup_dash_members_panel');
				if (config && !config.options.show_offline){
					self.$member.hide();
				}
				else{
					// Update member status class
					owner.get_status_img_node(self.$member)
						.attr('tooltip', "<%=_T('Offline')%>")
						.removeClass()
						.addClass('memberStatus memberStatusOFFLINE');
				}
				// set a custom attribute to prevent the server
				// to update this member node.
				self.$member
					.removeClass('selected')
					.attr('prevent_update', 'true');
			}
			else{
				let error_message = '';
				switch ( error.result.toUpperCase() ){
					case 'AGENT_HAS_INTERACTIONS':
						error_message = format("<%=_T('Unable to log off user. {0} has live interactions, you need to finalize them!')%>", self.m_member.ID);
						break;
					case 'FORBIDDEN':
						error_message = format("<%=_T('NOT HAS PERMISSIONS TO LOG OFF MEMBER')%>", self.m_member.ID );
						break;
					case 'INVALID_REQUEST':
					case 'default':
						error_message = format("<%=_T('Unable to log off user {0}, with error : {1}')%>", self.m_member.ID, error.reason);
						break;
				}
				self.show_status_message(error_message, "Error", false, 5000);
			}
		});
	};

	popup_member_detail.prototype.__change_member_state = function($state)
	{
		const self = this;
		const main_window = inConcert.app.getMainWindow();

		var show_ok_message = function(status){
			self.close();
			main_window.showBannerMessage({ message : format("<%=_T('Success changing {0} state to: {1}!')%>", self.m_member.ID, status), messageType : 'Info', timeout : 3000});
		};

		var show_failure_message = function(error)
		{
			let msg_err = '';
			switch((error.result || '').toUpperCase() ) {
				case 'INVALID_REQUEST':
					msg_err = "<%=_T('Invalid Request')%>";
				break;

				case 'USER_NOT_LOGGED':
					msg_err = format("<%=_T('User {0} not logged')%>", self.m_member.ID);
				break;

				case 'FORBIDDEN':
					msg_err = format("<%=_T('NOT HAS PERMISSIONS TO CHANGE MEMBER STATE')%>", self.m_member.ID);
				break;

				case 'USER_NOT_HAS_PERMISSION':
					var status_name = $state.find('.statusItemText').text();
					msg_err = format("<%=_T('The user does not have permissions for the state {0}')%>", status_name);
				break;

				case 'USER_HAS_ASSIGNED_LIVE_INTERACTIONS':
					if (error.assignedLiveInteractions === 1) {
						msg_err = "<%=_T('User has {0} live conversation assigned.')%>";
					}
					else {
						msg_err = "<%=_T('User has {0} live conversations assigned.')%>";
					}
					msg_err = format(msg_err, error.assignedLiveInteractions);
				break;
			}
			let error_message = format("<%=_T('Unable to change campaign state for user {0}: {1}')%>", self.m_member.ID, (msg_err || ''));
			self.show_status_message(error_message, "Error", false, 5000);
		};

		// show waiting
		this.show_waiting(format("<%=_T('Changing user {0} state')%>", this.m_member.ID));

		const owner = this.getOwnerObject();
		var selected_status = $state.attr('status');
		owner.m_model.change_member_state(this.m_member.ID, selected_status, this.m_campaign_id,
			function(ok, error) {
				if (ok) {
					// hide waitingselected_status
					self.hide_waiting();
					show_ok_message(selected_status);
				}
				else {
					// hide waiting
					self.hide_waiting();
					show_failure_message(error);
				}
			}
		);
	};

	popup_member_detail.prototype.__action_internal_chat = function(btn_config)
	{
		const self = this;
		var permission_fn = function() {
			return self.has_permission_to(btn_config.permission_id);
		};
		inConcert.app.getInteractionManager().createInternalChat(
			this.m_member.ID,
			this.m_member.name,
			permission_fn,
			function ( success, interaction) {
				if ( !success || !interaction ) {
					var error = interaction || {};

					if ( error.result === 'FORBIDDEN' ) {
						var error_message = "<%=_T('NOT HAS PERMISSIONS TO START CHAT')%>";
						self.show_status_message(error_message, "Error", false, 5000);
						return;
					}

					// default message
					inConcert.app.getMainWindow().showPopupMessage({
						message      : format( "<%=_T('Unable to communicate with {0}.')%>",self.m_member.name ),
						messageTitle : "<%=_T('Unable to start chat.')%>",
						popUpTitle   : "<%=_T('Chat not started.')%>",
						messageType  : 'Error'
					});

					return;
				}
				self.close(); // cierro el popup
				inConcert.app.m_mainWindow.updateInteractions([interaction]);
				inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null, {});
			}
		);
	};

	popup_member_detail.prototype.__action_screen = function()
	{
		const self = this;
		this.show_waiting(format("<%=_T('Loading user {0} screen')%>", this.m_member.ID));

		const owner = this.getOwnerObject();
		owner.m_model.start_screen_streaming(this.m_member.ID, this.m_campaign_id, function(ok, data)
		{
			self.hide_waiting();
			if (!ok) {
				let error_message = "";
				switch(data && data.reason || "") {
					case "channels_limit":
						error_message = "<%=_T('The system concurrent screens limit was reached. Please close some screens and retry.')%>";
						break;
					default:
						error_message = format("<%=_T('Unable to watch user {0} screen.')%>", self.m_member.ID);
						break;
				}
				self.show_status_message(error_message, "Error", false, 5000);
				return;
			}
			self.close(); // cierro el popup
			owner.showScreenStreamingPopup(self.m_campaign_id, self.m_member, data);
		});
	};

	popup_member_detail.prototype.__action_view_monitor = function(_, params)
	{
		const self = this;
		let options = $.extend(true, {campaign_id: this.m_campaign_id}, params);
		const owner = this.getOwnerObject();
		let opened = owner.m_dashboard.open_view('monitoring', options);
		if (!opened) {
			return;
		}
		this.close(); // cierro el popup
		owner.m_dash_monitor.init(this.$campaign, params, function() {
			owner.m_dash_monitor.execute_new_search(self.m_campaign_id, undefined, true, params);
		});
		owner.m_dashboard.save_config();
	};

	popup_member_detail.prototype.__action_view_appointment = function(_, params)
	{
		let options = $.extend(true, {campaign_id: this.m_campaign_id}, params);
		const owner = this.getOwnerObject();
		let opened = owner.m_dashboard.open_view('appointment', options);
		if (!opened) {
			return;
		}
		this.close(); // cierro el popup
		owner.m_dash_appointment.init(this.$campaign, this.m_member.ID);
	};

	popup_member_detail.prototype.__action_view_archive = function(_, params)
	{
		let options = $.extend(true, {campaign_id: this.m_campaign_id}, params);
		const owner = this.getOwnerObject();
		let opened = owner.m_dashboard.open_view('archive', options);
		if (!opened) {
			return;
		}
		this.close();
	};

	popup_member_detail.prototype.__action_view_kickouts = function()
	{		
		$("#window_sup_members_popup_kickouts").siblings().hide();
		var kickoutsDiv = $("#window_sup_members_popup_kickouts")[0];
		if (kickoutsDiv.style.display === "none") {
			kickoutsDiv.style.display = "block";
			kickoutsDiv.parent().style.display = "block";
		} else {
			kickoutsDiv.style.display = "none";
			kickoutsDiv.parent().style.display = "none";
		}
	};

	popup_member_detail.prototype.__action_view_campaigns = function(member)
	{
		const self = this;
		$("#window_sup_members_popup_campaigns").siblings().hide();
		const $campaigns = $("#window_sup_members_popup_campaigns");
		const $action_campaigns = $("#window_sup_members_popup_actions").find('[action_id="member_campaigns"]');
		if ($campaigns.is(':hidden')) {
			$('#window_sup_members_popup_actions').addClass('showMemberCampaigns');
			$action_campaigns.prop('disabled', true);
			const owner = this.getOwnerObject();
			owner.m_model.getMemberCampaigns(member, function(success, campaigns)
			{
				if(success)
				{
					let tCampaignsBody = $("#campaigns_list tbody");
					tCampaignsBody.html("");
					const allCampaigns = inConcert.app.getCampaigns().filter(function(campaign){
						return campaigns[campaign.Id]? true: false;
					}).sort(function(obj1, obj2){
						if (obj1.Id < obj2.Id) {return -1;}
						if (obj1.Id > obj2.Id) {return 1;}
						return 0;
					});

					allCampaigns.forEach((campaign, i) => {
						const index = i + 1;
						const row = `
							<tr>
								<td class="numCol">${ index }</td>
								<td>${ campaign.Name }</td>
								<td>${ campaign.Description }</td>
							</tr>
						`;
						tCampaignsBody.append(row);
					});
					
					$action_campaigns.prop('disabled', false);
					$campaigns.show();
					self.setPosition();
				}
				else{
					inConcert.app.getMainWindow().showBannerMessage({ message : format("<%=_T('Unable to get campaigns associated with the agent: {0}')%>", member), timeout : 5000});
					$action_campaigns.prop('disabled', false);
				}
			});			
			$campaigns.parent().show();	
		} else {
			$campaigns.hide();		
			$campaigns.parent().hide();			
			$('#window_sup_members_popup_actions').removeClass('showMemberCampaigns');
			$action_campaigns.prop('disabled', false);
		}
	};

	popup_member_detail.prototype.__action_change_state = function(btn_config)
	{
		if (!this.has_permission_to(btn_config.permission_id)) {
			return;
		}
		const $status_list = $("#window_sup_members_popup_menu_change_status");
		$status_list.toggle();
	};

	popup_member_detail.prototype.__update_member_actions = function()
	{
		const self = this;

		const owner = this.getOwnerObject();
		const $actions = $("#window_sup_members_popup_actions");

		var update_btn = function(btn_config)
		{
			let $button = $actions.find('[action_id="' + btn_config.action_id + '"]');
			$button.prop("disabled", true).unbind("click");
			if (btn_config.only_allegro && !inConcert.app.has_allegro_integration()) {
				$button.hide();
				return;
			}
			let has_permission = owner.m_dashboard.has_permission_to(btn_config.permission_id, self.m_campaign_id);
			if (has_permission) {
				if (btn_config.tooltip) {
					$button.inconcert_tooltip({message : btn_config.tooltip(), alignment: 'left-top'} );
				}
				if (btn_config.is_enabled()) {
					$button.prop("disabled", false);
					if (btn_config.click_action) {
						$button.bind("click", function() {btn_config.click_action(btn_config);});
					}
				}
				$button.show();
				return;
			}
			$button.hide();
		};
		var is_enabled = function() {
			return self.m_member.isLogged;
		};
		var allways_enabled = function() {
			return true;
		};
		var has_kickout = function() {
			return (self.m_member.kickouts && self.m_member.kickouts.length > 0);
		};

		let member_name = this.m_member.name || this.m_member.ID;
		const buttons = [
			{
				action_id: "member_chat",
				permission_id: "sup_internal_chat",
				only_allegro: false, is_enabled: is_enabled,
				tooltip: function() {
					return format("<%=_T('Chat to {0}')%>", member_name);
				},
				click_action: function(btn_config) {self.__action_internal_chat(btn_config);}
			},
			{
				action_id: "member_change_state",
				permission_id: "sup_change_user_state",
				only_allegro: false,
				is_enabled: is_enabled,
				tooltip: function() {
					return format("<%=_T('Change {0} state')%>", member_name);
				},
				click_action: function(btn_config) {self.__action_change_state(btn_config);}
			},
			{
				action_id: "member_loggoff",
				permission_id: "sup_logoff_user",
				only_allegro: false,
				is_enabled: is_enabled,
				tooltip: function() {
					return format("<%=_T('Logout {0}')%>", member_name);
				},
				click_action: function(btn_config) {self.__action_logout(btn_config);}
			},
			{
				action_id: "member_screen",
				permission_id: "sup_watch_screen",
				only_allegro: !inConcert.app.getAgentScreenServiceEnabled(),
				is_enabled: is_enabled,
				tooltip: function() {
					return format("<%=_T('View {0} screen')%>", member_name);
				},
				click_action: function(btn_config) {self.__action_screen(btn_config);}
			},
			{
				action_id: "member_monitor",
				permission_id: "sup_view_monitoring",
				only_allegro: false,
				is_enabled: allways_enabled,
				tooltip: function() {
					return format("<%=_T('MONITOR ACTIVITY OF {0}')%>", member_name);
				},
				click_action: function(btn_config) {
					self.__action_view_monitor(btn_config, {
						agent_id: self.m_member.ID,
						must_execute_new_search: false
					});
				}
			},
			{
				action_id: "member_appointment",
				permission_id: "sup_view_appointment" ,
				only_allegro: false, is_enabled: allways_enabled,
				tooltip: function() {
					return format("<%=_T('SHOW APPOINTMENTS OF {0}')%>", member_name);
				},
				click_action: function(btn_config) {
					self.__action_view_appointment(btn_config, {
						agent_id: self.m_member.ID,
						must_execute_new_search: true
					});
				}
			},
			{
				action_id: "member_archive",
				permission_id: "sup_view_archive",
				only_allegro: false,
				is_enabled: allways_enabled,
				tooltip: function() {
					return format("<%=_T('SHOW ARCHIVE OF {0}')%>", member_name);
				},
				click_action: function(btn_config) {
					self.__action_view_archive(btn_config, {
						agent_id: self.m_member.ID,
						must_execute_new_search: true
					});
				}
			},
			{
				action_id: "member_kickouts",
				permission_id: "sup_view_kickouts",
				only_allegro: false,
				is_enabled: has_kickout,
				tooltip: function() {
					return format("<%=_T('SHOW KICKOUTS OF {0}')%>", member_name);
				},
				click_action: function(/*btn_config*/) {
					self.__action_view_kickouts();
				}
			},
			{
				action_id: "member_campaigns",
				permission_id: "sup_view_archive",
				only_allegro: false,
				is_enabled: allways_enabled,
				tooltip: function() {
					return format("<%=_T('Show campaigns associated with the agent {0}')%>", member_name);
				},
				click_action: function(/*btn_config*/) {
					self.__action_view_campaigns(self.m_member.ID);
				}
			},

		];

		buttons.forEach(btn => {
			update_btn(btn);
		});
	};

	/*******************************************************************
	* Metodo que cierra el menu
	*/
	popup_member_detail.prototype.close = function()
	{
		this.data.container.remove();
		this.finalize();
	};

	popup_member_detail.prototype.__set_member_info = function()
	{
		const self = this;
		const owner = this.getOwnerObject();
		const must_show_complete_time_ago = true;

		const member_info = {
			agent_name        : this.m_member.name || this.m_member.ID,
			agent_id		  : this.m_member.ID,
			status_class      : owner.get_member_status_class(this.m_member),
			state_time        : owner.get_member_state_time_ago(this.m_member, this.m_campaign_id, must_show_complete_time_ago, true),
			avatar_uri        : inConcert.app.getBaseUrl() + '/users/get_avatar/' + this.m_member.ID,
			workgroup         : this.m_member.workgroup,
			member_state      : owner.get_member_state_msg(this.m_member, true),
			total_interactions: owner.get_member_total_interactions(this.m_member),
			attention_level   : owner.get_campaign_attention_level(this.m_member.ID, this.m_member.ID),
			kickout_class     : owner.get_member_kickout_class(this.m_member),
			missing_skills    : this.m_member.skillsDetail
		};

		$("#window_sup_members_popup_status").find(".memberStatus").removeClass()
			.addClass("memberStatus")
			.addClass(member_info.status_class);
		$("#window_sup_members_popup_status_name").text(member_info.member_state);
		$("#window_sup_members_popup_avatar").attr("src", member_info.avatar_uri);
		$("#window_sup_members_popup_user_name").text(member_info.agent_name);
		$("#window_sup_members_popup_user_id").text(member_info.agent_id);
		$("#window_sup_members_popup_time").text(member_info.state_time);
		$("#window_sup_members_popup_total_conversations").text(member_info.total_interactions);
		
		const $member_detail_state = $("#member_detail_state_container");
		$member_detail_state.hide();
		if (this.m_member?.state === "BREAK") {
			const break_full_state_time = owner.get_member_state_time_ago(this.m_member, this.m_campaign_id, must_show_complete_time_ago);
			const break_state_detail = owner.get_member_state_msg(this.m_member);
			if (break_state_detail && break_full_state_time) {
				$member_detail_state.html(format("<%=_T('Total in {0}')%>", break_state_detail) + ` ${break_full_state_time}`);
				$member_detail_state.show();
			}
		}
		// Agent missing skills
		const $agent_mskills = $("#agentMissingSkills");
		setMemberMissingSkill($agent_mskills, member_info.missing_skills);

		// Attention level
		const $attention_level = $("#window_sup_members_popup_attention_level");
		const $attention_level_val = $attention_level.find("[attention_level_val]");
		if (member_info.attention_level && member_info.attention_level.name) {
			$attention_level_val.val(member_info.attention_level.name);
			$attention_level.show();
		}
		else {
			$attention_level_val.val("");
			$attention_level.hide();
		}
		// Workgroup
		const $workgroup = $("#window_sup_members_popup_workgroup");
		const $workgroup_val = $workgroup.find("[workgroup_val]");
		if (member_info.workgroup) {
			$workgroup_val.text(member_info.workgroup);
			$workgroup.show();
		}
		else {
			$workgroup_val.text("");
			$workgroup.hide();
		}

		/*
		*/
		const on_click_msg_container = function (channel) {
			return function() {
				self.__action_view_monitor(undefined, {
					agent_id: self.m_member.ID,
					conversation_type: channel,
					must_execute_new_search: false
				});
			};
		};

		// Informacion sobre los canales
		let $pending_nodes = [];
		let campaign = inConcert.app.getCampaign(this.m_campaign_id);
		if (campaign) {
			let channels = campaign.get_channels();
			for (let channel in channels) {
				if (channels.hasOwnProperty(channel)) {
					let $msg_container = $(Mustache.render(
						owner.m_templates.members_channel_msg_tpl, {
						type_class: 'teamInteractionType ' + channel
					}
					));

					let channel_interactions = (self.m_member.interactions || {})[channel];
					let not_resumable_interactions =  (self.m_member.notResumableInteractions || {});
					// El total de interacciones para el canal y la cantidad en estado wrapup si es que el canal soporta este estado para las interacciones
					let result = owner.get_message_total_interactions_channel(channel, channel_interactions, not_resumable_interactions);
					const $total_interactions_msg = $msg_container.find("[total_interactions_msg]");
					$total_interactions_msg.html($("<span>" + result.total + "</span>"));
					$total_interactions_msg.attr("title", result.tooltip);
					// Agrego informacion sobre la capacidad de atencion del canal
					let channel_info = owner.__get_channel_info(channel);
					let $capability_msg = $msg_container.find("[capability_msg]");
					let has_available_attention_capacity = owner.__has_available_attention_capacity_in_channel(self.m_member, channel_info);
					if (has_available_attention_capacity) {
					// Si el canal SI tiene capacidad de atencion
						$msg_container.addClass("availableChannel");
						$capability_msg.text("").hide();
					}
					else {
					// Si el canal NO tiene capacidad de atencion
						$msg_container.addClass("busyChannel");
						$capability_msg.text("<%=_T('Channel without attention capability')%>").show();
					}
					$msg_container.off("click").off("click").on("click", on_click_msg_container(channel));
					$pending_nodes.push($msg_container);
				}
			}
		}

		const $conversations_detail = $("#window_sup_members_popup_conversations");
		$conversations_detail.html("");
		while ($pending_nodes.length > 0) {
			$conversations_detail.append($pending_nodes.pop());
		}

		this.__update_member_actions();

	// kickouts
		if(this.m_member.kickouts && Array.isArray(this.m_member.kickouts))
		{
			let tKickoutsBody = $("#kickout_list tbody");
			tKickoutsBody.empty();
			this.m_member.kickouts.forEach(function(kickout) {
				let jkickout = JSON.parse(kickout);
				let timestamp = (new Date(jkickout.timestamp * 1000)).toLocaleString();
				let forcedColTrue = "<%=_T('true')%>";
				let forcedColFalse = "<%=_T('false')%>";
				let forcedColText = jkickout.Forced ? forcedColTrue : forcedColFalse;
				let MessageColSystem = "<%=_T('Max strikes reached')%>";
				let MessageColSupervisor = "<%=_T('You have been logged out by supervisor')%>";
				let MessageColText = jkickout.Message === "Max strikes reached" ? MessageColSystem : MessageColSupervisor;
				let row = "<tr>";
					row += "<td>" + timestamp + "</td>";
					row += "<td>" + forcedColText + "</td>";
					row += "<td>" + jkickout.UserID + "</td>";
					row += "<td>" + MessageColText + "</td>";
				row += "</tr>";
				tKickoutsBody.append(row);
			});
		}
	};

	popup_member_detail.prototype.update_member_info = function(member, $member)
	{
		this.m_member = member;
		this.$member = $member;
		this.__set_member_info();
	};

	popup_member_detail.prototype.get_member = function()
	{
		return this.m_member;
	};

	popup_member_detail.prototype.get_campaign_id = function()
	{
		return this.m_campaign_id;
	};


	// ------------------------------------------------------------------------------------------------------------
	//
	//

	/**
	 * Creates a new members dashboard instance.
	 *
	 * @author Sebastian Moreno (smoreno@inconcertcc.com)
	 * @constructor
	 * @memberOf SupervisorDashboardMembers
	 * @param  {string} dashboard_instance
	 */

	const ROWS_PER_PAGE = 80;
	const AGENTS_ORDER_BY_NAME = 'Name';
	const AGENTS_ORDER_BY_SATE_TIME = 'StateTime';
	const AGENTS_ORDER_BY_INTERACTIONS_SATE_TIME = 'InteractionsStateTime';

	function DashboardMembers(dashboard_instance)
	{
		this.m_templates = inConcert.sup_templates.dashboard;
		this.m_model = dashboard_instance.get_model_manager();
		this.m_dashboard = dashboard_instance ? dashboard_instance : null;
		this.m_config = dashboard_instance.get_config_manager(); // current config
		this.m_workgroups = dashboard_instance.get_workgroup_manager();

		// current state by campaign, available states are:
		// idle, paginating, refreshing, agent_search
		this.m_state = {};
		this.m_page_by_campaign = {};
		this.m_custom_filter_by_campaign = {};
		this.m_previous_search_has_members = {};

		this.m_current_sup_id = inConcert.app.getLoggedUserId();

		this.m_dash_monitor = dashboard_instance.get_monitor_manager();
		this.m_dash_appointment = dashboard_instance.get_appointment_manager();
		this.m_dash_archive = dashboard_instance.get_archive_manager();

		this.m_member_states = {
			'ACTIVE': {
				desc: "<%=_T('Available')%>",
				filter: {state_id: 'IDLE', enabled: false}
			},
			'ATTENDING': {
				desc: "<%=_T('Attending')%>",
				filter: {state_id: 'ATTENDING', enabled: true}
			},
			'ATTENDING CROSS CAMPAIGN': {
				desc: "<%=_T('Attending Another')%>",
				filter: {state_id: 'ATTENDING CROSS CAMPAIGN', enabled: true}
			},
			'AWAY': {
				desc: "<%=_T('Away')%>",
				filter: {state_id: 'AWAY', enabled: true}
			},
			'BATHROOM': {
				desc: "<%=_T('Bathroom')%>",
				filter: {state_id: 'BATHROOM', enabled: true}
			},
			'ANY_BREAK': {
				desc: "<%=_T('Any Break')%>",
				filter: {state_id: 'ANY_BREAK', enabled: true}
			},
			'BREAK': {
				desc: "<%=_T('Break')%>",
				filter: {state_id: 'BREAK', enabled: true}
			},
			'BUSY': {
				desc: "<%=_T('Busy')%>",
				filter: {state_id: 'BUSY', enabled: true}
			},
			'BUSY CROSS CAMPAIGN': {
				desc: "<%=_T('Busy In Another')%>",
				filter: {state_id: 'BUSY CROSS CAMPAIGN', enabled: true}
			},
			'RESERVED': {
				desc: "<%=_T('Reserved')%>",
				filter: {state_id: 'RESERVED', enabled: true}
			},
			'RESERVED CROSS CAMPAIGN': {
				desc: "<%=_T('Reserved Another')%>",
				filter: {state_id: 'RESERVED CROSS CAMPAIGN', enabled: true}
			},
			'IDLE': {
				desc: "<%=_T('Idle')%>",
				filter: {state_id: 'IDLE', enabled: true}
			},
			'LUNCH': {
				desc: "<%=_T('Off to lunch')%>",
				filter: { state_id: 'LUNCH', enabled: true}
			},
			'NOACD': {
				desc: "<%=_T('Online No ACD')%>",
				filter: {state_id: 'NOACD', enabled: true}
			},
			'OFFLINE': {
				desc: "<%=_T('Offline')%>",
				filter: {state_id: 'OFFLINE', enabled: true}
			}
		};

		this.m_interactions_states = {
			'TAKEN': {
				desc: "<%=_T('Taken')%>",
				filter: {state_id: 'Taken', allowed_channels: "ALL"}
			},
			'WRAPUP': {
				desc: "<%=_T('Wrapup')%>",
				filter: {state_id: 'Wrapup', allowed_channels: {CALL: true, CHAT: true}}
			}
		};

		this.m_interactions_type_map = {
			'CALL': {
				channel: "CALL",
				validate_capability: true,
				desc: "<%=_T('call')%>",
				sync: true
			},
			'CHAT': {
				channel: "CHAT",
				validate_capability: true,
				desc: "<%=_T('webchat')%>",
				sync: true
			},
			'FACEBOOK': {
				channel: "FACEBOOK",
				validate_capability: true,
				desc: "<%=_T('facebook post')%>",
				sync: false
			},
			'IM': {
				channel: "IM",
				validate_capability: true,
				desc: "<%=_T('instant message')%>",
				sync: true
			},
			'INSTAGRAM': {
				channel: "INSTAGRAM",
				validate_capability: true,
				desc: "<%=_T('instagram post')%>",
				sync: false
			},
			'YOUTUBE': {
				channel: "YOUTUBE",
				validate_capability: true,
				desc: "<%=_T('youtube video')%>",
				sync: false
			},
			'APPSTORE': {
				channel: "APPSTORE",
				validate_capability: true,
				desc: "<%=_T('appstore review')%>",
				sync: false
			},
			'GOOGLEPLAY': {
				channel: "GOOGLEPLAY",
				validate_capability: true,
				desc: "<%=_T('googleplay review')%>",
				sync: false
			},
			'RCS': {
				channel: "RCS",
				validate_capability: true,
				desc: "<%=_T('rcs message')%>",
				sync: false
			},
			'MAIL': {
				channel: "MAIL",
				validate_capability: true,
				desc: "<%=_T('e-mail')%>",
				sync: false
			},
			'SMS': {
				channel: "SMS",
				validate_capability: true,
				desc: "<%=_T('sms message')%>",
				sync: false
			},
			'TWITTER': {
				channel: "TWITTER",
				validate_capability: true,
				desc: "<%=_T('tweet')%>",
				sync: false
			},
			'VOICEMAIL': {
				channel: "VOICEMAIL",
				validate_capability: false,
				desc: "<%=_T('voice mail')%>",
				sync: false
			},
			'WEBCONTACT': {
				channel: "WEBCONTACT",
				validate_capability: true,
				desc: "<%=_T('web contact')%>",
				sync: false
			},
			'WHATSAPP': {
				channel: "WHATSAPP",
				validate_capability: true,
				desc: "<%=_T('whatsapp message')%>",
				sync: false
			}
		};

		this.m_status_class_mapping = {
		// available/Active Statuses
			'ACTIVE'                   : 'memberStatusACTIVE',
			'AVAILABLE'                : 'memberStatusACTIVE',
			'IDLE'                     : 'memberStatusACTIVE',
			'NOACD'                    : 'memberStatusNOACD',
		// break Statuses
			'BREAK'                    : 'memberStatusBREAK',
			'LUNCH'                    : 'memberStatusBREAK',
			'BATHROOM'                 : 'memberStatusBREAK',
			'AWAY'                     : 'memberStatusBREAK',
		// busy Statuses
			'BUSY'                     : 'memberStatusBUSY',
			'BUSY CROSS CAMPAIGN'      : 'memberStatusBUSY_CROSS_CAMPAIGN',
		// Attending Statuses
			'ATTENDING'                : 'memberStatusATTENDING',
			'ATTENDING CROSS CAMPAIGN' : 'memberStatusATTENDING_CROSS_CAMPAIGN',
		// reserved Statuses
			'RESERVED'                 : 'memberStatusRESERVED',
			'BUSY RESERVED CAMPAIGN'   : 'memberStatusBUSY_RESERVED_CAMPAIGN',
		// Offline/Unavailable Statuses
			'OFFLINE'                  : 'memberStatusOFFLINE',
			'UNAVAILABLE'              : 'memberStatusOFFLINE',
		// default offline status.
			''                         : 'memberStatusOFFLINE',
		// this is an special case, this isn't a real state.
			'ACTIVE_BUSY'              : 'memberStatusACTIVEBUSY',
		// all custom states
			'CUSTOM'                   : 'memberStatusCUSTOM'
		};
	}

	if (typeof inConcert.sup_dashboard_members !== 'function') {
		inConcert.sup_dashboard_members = DashboardMembers;
	}

	DashboardMembers.prototype.on_change_config_selector = function( /*campaign_id*/ ) {};

	/**
	 * Get the members panel jQuery node
	 * 
	 * @private
	 * @param  {string} campaign_id - The campaign id (i.e. campaign@vcc)
	 * @return {object}             - The jQuery panel node
	 */
	DashboardMembers.prototype.__get_panel_node = function(campaign_id)
	{
		return $('#sup_dash_members_panel_' + this.m_dashboard.get_normalized_id(campaign_id));
	};

	/**
	 * Get the current member panel container
	 *
	 * @private
	 * @param  {String} campaign_id - The campaign id (i.e. campaign@vcc)
	 * @return {Object}             - The jQuery panel container node
	 */
	DashboardMembers.prototype.__get_panel_container = function(campaign_id)
	{
		return this.__get_panel_node(campaign_id).find('.columnContent');
	};

	/**
	 * Returns the message to show on the list.
	 *
	 * @private
	 * @param  {string} campaign_id The campaign id (i.e. campaign@vcc)
	 * @return {string}             The message
	 */
	DashboardMembers.prototype.__get_team_list_message_to_show = function(campaign_id)
	{
		return this.m_custom_filter_by_campaign[campaign_id] ? "<%=_T('Search does not match any users!')%>" : "<%=_T('There are no users for this view.')%>";
	};

	/**
	 * Returns if we can refresh the current panel info.
	 *
	 * @private
	 * @param  {string} campaign_id The campaign id (i.e. campaign@vcc)
	 * @return {boolean}             True if can refresh, false if not.
	 */
	DashboardMembers.prototype.__allowed_to_refresh_view = function(campaign_id)
	{
		return this.m_state[campaign_id] === 'idle';
	};

	DashboardMembers.prototype.__find_element_in_campaign = function(campaign_id, selector)
	{
		return $('#sup_dash_campaign_'+ this.m_dashboard.get_normalized_id(campaign_id) + ' ' + selector);
	};

	DashboardMembers.prototype.show_waiting = function (campaign_id, message)
	{
		var normalized_campaign = this.m_dashboard.get_normalized_id(campaign_id);
		$('#sup_dash_members_waiting_' + normalized_campaign).show();
		$('#sup_dash_members_waiting_msg_' + normalized_campaign).text(message || '');
	};

	DashboardMembers.prototype.hide_waiting = function (campaign_id)
	{
		$('#sup_dash_members_waiting_' + this.m_dashboard.get_normalized_id(campaign_id)).hide();
	};

	DashboardMembers.prototype.__$get_members_container_list = function ($current_node)
	{
		let campaign_id = $current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $members_container_list = $('#sup_dash_members_container_list_' + campaign_dom_id);
		return $members_container_list;
	};

	DashboardMembers.prototype.__$get_members_list = function ($current_node)
	{
		let campaign_id = $current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $members_list = $('#sup_dash_members_list_' + campaign_dom_id);
		return $members_list;
	};

	DashboardMembers.prototype.__$get_members_list_items = function ($current_node)
	{
		const $members_list = this.__$get_members_list($current_node);
		let $members_list_items = $members_list.find('[member_item]');
		return $members_list_items;
	};

	DashboardMembers.prototype.__clear_member_list = function ($current_node, $members_list)
	{
		$members_list = $members_list || this.__$get_members_list($current_node);
		$members_list.find('[member_item]').remove();
	};

	DashboardMembers.prototype.__get_current_members_from_list = function ($current_node, hidden_too)
	{
		const $members_list = this.__$get_members_list($current_node);
		let members_selector = hidden_too ? '[member_item]' : '[member_item]:visible';
		let members = [];

		$members_list.find(members_selector).each(function(_, e) {
			members.push($(e).data('member'));
		});

		return members;
	};

	DashboardMembers.prototype.__get_member_node = function (campaign_id, team_list, member_id)
	{
		return team_list.find('#sup_member_' + this.m_dashboard.get_normalized_id(campaign_id) + '_' + nid(member_id));
	};

	DashboardMembers.prototype.__toggle_member = function (current_node, member_id)
	{
		this.__get_member_node(current_node.attr('campaign_id'), current_node, member_id).toggle();
	};

	DashboardMembers.prototype.__remove_member_from_list = function (current_node, member_id)
	{
		this.__get_member_node(current_node.attr('campaign_id'), current_node, member_id).remove();
	};

	DashboardMembers.prototype.__hide_members_list = function ($current_node)
	{
		const $members_container_list = this.__$get_members_container_list($current_node);
		$members_container_list.hide();
	};

	DashboardMembers.prototype.__show_members_list = function ($current_node)
	{
		const $members_container_list = this.__$get_members_container_list($current_node);
		$members_container_list.show();
	};

	DashboardMembers.prototype.get_status_img_node = function (member_node)
	{
		return member_node.find('img[class^="memberStatus"]');
	};

	DashboardMembers.prototype.get_kickout_img_node = function (member_node)
	{
		return member_node.find('div[class^="kickoutStatus"]');
	};

	DashboardMembers.prototype.get_member_state_time_ago = function (member, campaign_id, must_show_complete, use_detail)
	{
		must_show_complete = must_show_complete || false;
		if ( !member || !campaign_id ){
			return "<%=_T('Never')%>";
		}
		
		let member_timestamp = member.state === "BREAK" && use_detail ? member.breakStateTimetamp || member.stateTimeTimestamp : member.stateTimeTimestamp;
		let campaign_timestamp = this.m_dashboard.get_campaign_timestamp(campaign_id) * 1000;
		let state_timestamp = member.isLogged ? member_timestamp * 1000 || "<%=_T('Never')%>" : member.logoutTimeTimestamp * 1000 || "<%=_T('Never')%>";
		if (typeof(state_timestamp) === 'string'){
			return state_timestamp;
		}
		let datetime = inConcert.Utils.timeAgo2(state_timestamp, campaign_timestamp, must_show_complete, true);
		//Aqui solo muestro si tiene annos o meses o dias o time. Vendria en caso de meses algo asi 2 meses 00:00:12 (me quedo solo con el mes)
		let split_datetime = datetime.split(" ");
		return split_datetime.length === 1? split_datetime[0]: split_datetime.slice(0, 2).join(" ");
	};

	DashboardMembers.prototype.__$get_msg_container = function ( $current_node )
	{
		const campaign_id = $current_node.attr('campaign_id');
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $msg_container = $(`#sup_dash_members_list_msg_container_${campaign_dom_id}`);
		return $msg_container;
	};

	DashboardMembers.prototype.__show_empty_member_list_msg = function (current_node, message)
	{
		const $msg_container = this.__$get_msg_container(current_node);
		$msg_container.show().find('[message]').text(message);
	};

	DashboardMembers.prototype.__hide_empty_member_list_msg = function (current_node)
	{
		const $msg_container = this.__$get_msg_container(current_node);
		if ($msg_container.not(':hidden')){
			$msg_container.hide();
		}
	};

	DashboardMembers.prototype.__team_list_have_space_left = function ($members_list)
	{
		return $members_list.find('[member_item]').length < ROWS_PER_PAGE;
	};

	DashboardMembers.prototype.__disable_input_actions = function (current_node)
	{
		current_node.find('.butPagerPrev, .butPagerNext, .butTeamAction').prop('disabled', true);
	};

	DashboardMembers.prototype.__enable_input_actions = function (current_node)
	{
		current_node.find('[filter_member_id="true"]').prop('disabled', false); // textbox agent id
	};

	DashboardMembers.prototype.__update_members_menu_options = function (current_node, total_members)
	{
		current_node.find('#sup_dash_show_avatars_members_option, #sup_dash_show_names_members').parent().toggle(total_members !== 0);
	};

	DashboardMembers.prototype.__get_description_member_state = function(state_id)
	{
		let state = this.m_member_states[state_id];
		if (state && state.desc) {
			return state.desc;
		}
		state = inConcert.app.getStateObject(state_id);
		if (state && state.Description) {
			return state.Description;
		}
		return state_id;
	};

	DashboardMembers.prototype.on_toggle_full_screen = function (campaign_id)
	{
		this.__update_colapsed_filter(campaign_id);
	};

	DashboardMembers.prototype.__get_description_interaction_state = function(state_id = "")
	{
		let state = this.m_interactions_states[state_id.toUpperCase()];
		if (state && state.desc) {
			return state.desc;
		}
		return state_id;
	};

	DashboardMembers.prototype.get_member_state_msg = function(member, use_detail)
	{
		var self = this;

		if (!member) {
			return;
		}

		var get_description = function(id) {
			return self.__get_description_member_state(id);
		};

		var state_id = member.state;
		var state_detail = member.stateDetail && member.stateDetail.currentState ? member.stateDetail.currentState : undefined;
		if (!state_id && !state_detail) {
			return get_description("OFFLINE");
		}

		state_id = state_id || '';

		if (state_id.toUpperCase() === 'BREAK' && use_detail) {
			if (state_detail) {
				if(state_detail.match(inConcert.app.idAtVCCRegExp)) {
					return get_description( RegExp.$1 );
				}
				return get_description(state_detail);
			}
		}
 
		return get_description(state_id);
	};

	DashboardMembers.prototype.get_member_kickout_msg = function(member)
	{
		let kickout_msg = "";
		if(member.kickouts && member.kickouts.length > 0)
		{
			let jkickout = JSON.parse(member.kickouts[member.kickouts.length - 1]);
			let timestamp = (new Date(jkickout.timestamp * 1000)).toLocaleString();
			kickout_msg = format("<%=_T('{0} was kicked out on {1} by {2}')%>", member.ID, timestamp, jkickout.UserID);
		}
		return kickout_msg;
	};

	DashboardMembers.prototype.get_member_total_interactions = function(member)
	{
		var total_interactions = 0;

		_.each(member.interactions, function(channel_interactions){
			// I know this is lame, but somehow the server returns a number (sometimes)
			total_interactions += typeof(channel_interactions) === 'number' ? channel_interactions : _.size(channel_interactions);
		});

		// Now count non-resumable interactions
		for (let key in member.notResumableInteractions) {
			if (member.notResumableInteractions.hasOwnProperty(key)) {
				total_interactions += member.notResumableInteractions[key];
			}
		}

		return total_interactions;
	};

	DashboardMembers.prototype.__get_member_name = function(member, show_names)
	{
		return show_names ? member.name : member.ID;
	};

	DashboardMembers.prototype.__get_channel_info = function(channel)
	{
		return this.m_interactions_type_map[channel] || {};
	};

	DashboardMembers.prototype.get_message_total_interactions_channel = function(type, channel_interactions, not_resumable_interactions)
	{
		let total_interactions_channel = (not_resumable_interactions && not_resumable_interactions[type]) ? not_resumable_interactions[type] : 0;
		let total_wrapup = 0; // para el caso de interacciones de wrapup se debe de visuaizar el numero (webchat, call)
		for (let interaction_id in channel_interactions) {
			if (channel_interactions.hasOwnProperty(interaction_id)) {
				total_interactions_channel++;
				let channel_int = channel_interactions[interaction_id];
				if ((channel_int.state || '').toLowerCase() === 'wrapup') {
					total_wrapup++;
				}
			}
		}
		if (total_interactions_channel === 0){
			return {
				tooltip: '',
				total: total_interactions_channel
			};
		}
		let msg;
		let channel_info = this.__get_channel_info(type);
		msg = format('{0} {1}', total_interactions_channel, total_interactions_channel === 1? channel_info.desc: channel_info.desc);

		if (total_wrapup === 0 || total_wrapup === total_interactions_channel) {
			return {
				tooltip: msg,
				total: total_interactions_channel
			};
		}
		return {
			tooltip: format('{0} {1} in Wrapup', msg, total_wrapup),
			total: total_wrapup
		};
	};

	DashboardMembers.prototype.get_member_status_class = function( member )
	{
		var offline_class = this.m_status_class_mapping[ '' ];
		if ( !member ) {
			return offline_class;
		}
		var state_id = member.state;
		var state_detail = member.stateDetail && member.stateDetail.currentState ? member.stateDetail.currentState : undefined;
		if ( !state_id && !state_detail ) {
			return offline_class;
		}

		state_id = state_id || '';

		if ( state_id.toUpperCase() === 'BREAK' ) {
			if ( state_detail ) {
				var class_custom_state = this.m_status_class_mapping.CUSTOM;
				if( state_detail.match( inConcert.app.idAtVCCRegExp ) ) {
				// aca van los customs
					var custom_state_without_vcc = RegExp.$1;
					return class_custom_state + ' ' + custom_state_without_vcc;
				}
				return ( this.m_status_class_mapping[ state_detail ] || class_custom_state ) + ' ' + state_detail;
			}
		}

		return this.m_status_class_mapping[ state_id ] || offline_class;
	};

	DashboardMembers.prototype.get_member_kickout_class = function( member )
	{
		if(member.kickouts && member.kickouts.length > 0)
		{
			return "kickoutStatusON";
		}
		return "kickoutStatusOFF";
	};

	/**
	 * Called when the user is going to make a new search that
	 * will affect the current member list. The list is cleared, shows a waiting
	 * and change the state of the current campaign.
	 *
	 * @private
	 * @param  {Object} current_node - Current campaign node
	 */
	DashboardMembers.prototype.__on_before_members_search = function(current_node)
	{
		let campaign_id = current_node.attr('campaign_id');
		// show member list list
		this.__show_members_list(current_node);
		// clear list.
		this.__clear_member_list(current_node);
		// hide *empty list* message
		this.__hide_empty_member_list_msg(current_node);
		// show waiting
		this.show_waiting(campaign_id);
		// set state
		this.m_state[campaign_id] = 'searching';
	};

	DashboardMembers.prototype.__on_after_members_search = function(current_node, total_search_members, total_members_left)
	{
		var campaign_id = current_node.attr('campaign_id');

		// hide waiting
		this.hide_waiting(campaign_id);
		// set to idle status
		this.m_state[campaign_id] = 'idle';

		// buttons status after search
		if (total_search_members === 0){
			this.__disable_input_actions(current_node);
		}
		else {
			this.__enable_input_actions(current_node);
		}

		// update menu options and member node events
		this.__update_members_menu_options(current_node, total_search_members);
		this.on_member_list_reloaded(current_node, campaign_id, total_search_members, total_members_left);
	};

	DashboardMembers.prototype.__on_member_list_empty = function(current_node, campaign_id)
	{
		this.__hide_members_list(current_node);
		this.__show_empty_member_list_msg(current_node, this.__get_team_list_message_to_show(campaign_id));
		this.__disable_input_actions(current_node);
	};

	DashboardMembers.prototype.__setup_agent_status_popup = function($current_node)
	{
		var statuses = [];
		// build markup for each status
		var available_statuses = inConcert.app.getAvailableStates();
		available_statuses.forEach(function(state){
			var status_view = {
				status_id : state.Id,
				status_name: state.Description
			};
			statuses.push(status_view);
		});

		// Build status group container
		var html_statuses = Mustache.render(this.m_templates.agent_statuses_list_tpl, {title: "<%=_T('Change Agent Status')%>", list: statuses});
		$current_node.find('.popupStatusList').html(html_statuses);
	};

	/**
	 * Will build the jquery object for a member using a mustache template
	 *
	 * @param  {Object} $current_node - The current campaign node
	 * @param  {Object} member       - The member info returned by the server
	 * @return {jQuery Object}       - The member jquery object
	 */
	DashboardMembers.prototype.__$build_member_markup = function ( $current_node, member )
	{
		const campaign_id = $current_node.attr('campaign_id');
		const config = this.m_config.get_panel_config(campaign_id, 'sup_dash_members_panel');
		const member_id = member.ID;
		const normalized_member_id = nid(member_id);

		let can_show_state_time = false;
		let can_show_state_name = false;
		let can_show_workgroup = false;
		let can_show_agent_id = false;
		let can_show_agent_names = true;
		let can_show_avatar = false;
		let can_show_state_visual_indicator = true;
		let can_show_total_conversations = false;
		let can_show_total_conversations_by_channel = false;
		if (config && config.options) {
			can_show_state_time = config.options.show_state_time || false;
			can_show_state_name = config.options.show_state_name || false;
			can_show_workgroup  = member.workgroup && member.workgroup !== "" && config.options.show_workgroups || false;
			can_show_agent_id = config.options.show_agents_id || false;
			can_show_agent_names = config.options.show_names || !can_show_agent_id; //Se muestran al menos uno, prioridad (names)
			can_show_avatar = config.options.show_avatars || false;
			can_show_total_conversations = config.options.show_total_conversations || false;
			can_show_total_conversations_by_channel = config.options.show_total_conversations_by_channel || false;
			can_show_state_visual_indicator = config.options.show_state_visual_indicator || !can_show_state_name;
		}

		let member_view = {
			'id'             : this.m_dashboard.get_normalized_id(campaign_id) + '_' + normalized_member_id,
			'agent_id'       : normalized_member_id,
			'show_agent_id'  : can_show_agent_id ? '' : 'display:none',
			'show_agent_name'  : can_show_agent_names ? '' : 'display:none',
			'agent_name'     : this.__get_member_name(member, can_show_agent_names || false),
			'status_class'   : this.get_member_status_class( member ),

			'show_state_time': can_show_state_time ? '' : 'display:none',
			'state_time'     : this.get_member_state_time_ago(member, campaign_id, false, true),

			'show_state_name': can_show_state_name ? '' : 'display:none',
			'state_name'     : this.get_member_state_msg(member, true),

			'show_state_visual_indicator': can_show_state_visual_indicator ? '' : 'display:none',

			'show_avatars'   : can_show_avatar ? '' : 'display:none',
			'avatar_uri'     : '' + inConcert.app.getBaseUrl() + '/users/get_avatar/' + member_id + '',
			'show_workgroups'  : can_show_workgroup ? '' : 'display:none',
			'workgroup'      : member.workgroup ? `<%=_T([[Group]])%>: ${ member.workgroup }` :"",
			'campaign_id'    : campaign_id,
			'kickout_class'  : this.get_member_kickout_class( member),
			'show_total_conversations' : can_show_total_conversations ? '' : 'display:none',
			'total_conversations': this.get_member_total_interactions(member),
			'show_total_conversations_by_channel': can_show_total_conversations_by_channel ? '' : 'display:none'
		};

		const $member = $(Mustache.render(this.m_templates.members_tpl, member_view));
		$member.data("campaign_id", campaign_id);

		this.__update_member_missing_skills(campaign_id, $member, member);
		this.__update_member_channels(campaign_id, $member, member);
		this.__update_member_attention_level(campaign_id, $member, member);

		return $member;
	};

	DashboardMembers.prototype.get_campaign_attention_level = function (campaign_id, member_id)
	{
		const al_info = {name: '', id: ''};
		if (campaign_id) {
			return al_info;
		}
		let campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);
		if (campaign_config.account_id && !campaign_config.al_id) {
			let campaign = inConcert.app.getCampaign(campaign_id);
			if (campaign) {
				let member_attention_level = campaign.getAttentionLevelByUserAndAccount(member_id, campaign_config.account_id);
				if (member_attention_level) {
					al_info.name = member_attention_level.name;
					al_info.id = member_attention_level.id;
				}
			}
		}
		return al_info;
	};

	DashboardMembers.prototype.__update_member_attention_level = function (campaign_id, $member, member)
	{
		let al_info = this.get_campaign_attention_level(campaign_id, member.ID);
		$member.find('[attention_level_id]')
			.text(al_info.name)
			.attr('attention_level_id', al_info.id)
			[al_info.name.length > 0 ? 'show' : 'hide']();
	};

	DashboardMembers.prototype.__has_available_attention_capacity_in_channel = function (member, channel_info)
	{
		if (!channel_info.validate_capability) {
			// SI tiene disponibilidad el channel, si en la definicion de los channels del panel, el atributo "validate_capability" contiene el valor false.
			return true;
		}
		let member_channel_info = member.channels ? member.channels[channel_info.channel] : undefined;
		if (!member_channel_info) {
			// NO tiene disponibilidad el channel, si NO viene informacion del channel en el member.
			return false;
		}
		let available = parseInt((member_channel_info.available || 0), 10);
		if (available === 0) {
			// NO tiene disponibilidad el channel
			return false;
		}
		// SI tiene disponibilidad el channel
		return true;
	};

	DashboardMembers.prototype.__update_member_missing_skills = function (_campaign_id, $member, member)
	{
		let $member_missing_skills  = $member.find("[member_missing_skills]");
		// let campaign_dom_id_normal  = this.m_dashboard.get_normalized_id(campaign_id);
		// let not_hide_missing_skills = $(`#sup_dash_show_all_${campaign_dom_id_normal}`).prop("checked");

		setMemberMissingSkill($member_missing_skills, member.skillsDetail);
	};

	DashboardMembers.prototype.__update_member_channels = function (campaign_id, $member, member)
	{
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign) {
			return;
		}
		let $member_channels_interactions = $member.find("[member_channels_interactions]");
		$member_channels_interactions.html("");
		let channels = campaign.get_channels();

		for (let channel in channels) {
			if (channels.hasOwnProperty(channel))
			{
				let channel_info = this.__get_channel_info(channel);
				let $channel = $("<label></label>");
				$channel.addClass("memberInteractions");
				$channel.addClass(channel);

				let channel_interactions = (member.interactions || {})[channel];
				let not_resumable_interactions =  (member.notResumableInteractions || {});
				let total_channel = (not_resumable_interactions[channel]) ? not_resumable_interactions[channel] : 0;
				let total_channel_wrapup = 0;
				for (let interaction_id in channel_interactions) {
					if (channel_interactions.hasOwnProperty(interaction_id)) {
						total_channel++;
						let channel_int = channel_interactions[interaction_id];
						if (channel_info.sync && (channel_int.state || "").toLowerCase() === "wrapup") {
							total_channel_wrapup++;
						}
					}
				}

				if (channel_info.sync) {
				// CALL, CHAT
					if (total_channel_wrapup === 0) {
						$channel.html(total_channel);
					}
					else {
						let html_info = "<span>" + total_channel_wrapup + "</span>" + "/" + total_channel;
						$channel.html(html_info);
					}
				}
				else {
				// MAIL, FACEBOOK, etc
					$channel.html(total_channel);
				}

				// Asigno informacion con el estado del canal para el member (busyChannel availableChannel noInteractionsChannel)
				if (total_channel === 0) {
					$channel.addClass("noInteractionsChannel");
				}
				else {
					let has_available_attention_capacity = this.__has_available_attention_capacity_in_channel(member, channel_info);
					if (has_available_attention_capacity) {
						$channel.addClass("availableChannel");
					}
					else {
						$channel.addClass("busyChannel");
					}
				}

				let result = this.get_message_total_interactions_channel(channel, channel_interactions, not_resumable_interactions);
				$channel.attr("tooltip", result.tooltip);
				$member_channels_interactions.append($channel);
			}
		}
	};

/*
Member values:
{
  "workgroup": "",
  "defaultCampaign": "tel1@allegro4",
  "oldestInteractionAssignment": [],
  "phone": "ASTERISK,SIP/2002",
  "stateDetail": {
    "currentState": "OFFLINE"
  },
  "logoutTime": "2016-12-08 23:39:48.270",
  "oldestInteractionAssignmentTimestamp": [],
  "interactions": [],
  "logoutTimeTimestamp": 1481240388.271,
  "isLogged": false,
  "state": "OFFLINE",
  "ID": "agent1",
  "stateTimeTimestamp": 1481240388.271,
  "stateTime": "2016-12-08 23:39:48.270",
  "name": "agent1 Ayelén  de Mello Aquino"
}
*/

	DashboardMembers.prototype.__must_show_alarm = function (campaign_id, member)
	{
		// 1 - Valido contra las alarmas de los estados/tiempo de los members (request obteniendo de summary)
		let category = "members";
		let response = this.m_dashboard.eval_notifications_alarms(campaign_id, category, member);
		if (response.must_show_alarm) {
			return response;
		}
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign) {
			return response;
		}
		// 2 - Valido contra las alarmas la cantidad de interacciones de cada member en los estados "taken" & "wrapup" (request obteniendo de members)
		// Recorro los canales de la campana, y valido si hay alarmas para alguno de los canales.
		let campaign_channels = campaign.get_channels();
		for (let channel in campaign_channels) {
			if (campaign_channels.hasOwnProperty(channel)) {
				let category_template = "members_conversations_by_{0}_channel"; // members_conversations_by_MAIL_channel
				let my_channels = [];
				if (channel.toLowerCase() === "call") {
					my_channels.push("inbound_call");
					my_channels.push("outbound_call");
				}
				else {
					my_channels.push(channel);
				}
				for (let i = 0; i < my_channels.length; i++) {
					let my_channel = my_channels[i];
					category = format(category_template, my_channel.toUpperCase());
					response = this.m_dashboard.eval_notifications_alarms(campaign_id, category, member);
					if (response.must_show_alarm) {
						return response;
					}
				}
			}
		}
		return response;
	};

	DashboardMembers.prototype.__show_alarm = function ($node, member)
	{
		var campaign_id = $node.attr('campaign_id');
		$node.attr('title', '').removeClass('alarmed').data('alarm', {});
		var response = this.__must_show_alarm(campaign_id, member);
		if (!response.must_show_alarm) {
			return false;
		}
		// (ALARMS) Marco/desmarco el indicador como alarmado.
		return this.m_dashboard.show_alarm($node, response, response.current_value);
	};

	DashboardMembers.prototype.__set_account_al_to_filter = function (campaign_id, filter)
	{
		if ( !campaign_id || !filter ) {
			return;
		}
		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);
		var account_id = campaign_config.account_id;
		if ( !account_id ) {
			return;
		}
		var campaign = inConcert.app.getCampaign(campaign_id);
		if ( !campaign ) {
			return;
		}
		filter.account_info = { account: account_id };
		var is_configured = campaign.has_attention_levels_configured_in_account(account_id);
		if ( !is_configured ) {
			return;
		}
		if ( campaign_config.al_id ) {
			filter.account_info.search_type = 'equal';
			filter.account_info.attention_level = campaign_config.al_id;
		}
		else {
			filter.account_info.search_type = 'content';
			filter.account_info.attention_level = "";
		}
	};

	/**
	 * Will setup the members search, this function
	 * should be called obnly once.
	 *
	 * @private
	 * @param  {Object} current_node - The campaign jQuery object
	 */
	DashboardMembers.prototype.__setup_members_search = function (current_node)
	{
		const self = this;

		let campaign_id = current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		// buttons and textboxes nodes
		const $agent_id = current_node.find('[filter_member_id="true"]'); // textbox agent id
		const $filter_button = $('#sup_dash_members_panel_search_' + campaign_dom_id); // search button

		// bind filter click button
		$filter_button.unbind('click').bind('click', function ()
		{
			let config = self.m_config.get_panel_config(campaign_id, 'sup_dash_members_panel');
			let show_offline_agents = config ? config.options.show_offline : false;
			let show_kickouts = config ? config.options.show_kickouts : true;
			let agent_search = inConcert.Utils.removeSpecialChars($agent_id.val());

			// just set first page if previous filter was not set.
			if (!self.m_custom_filter_by_campaign[campaign_id]){
				self.m_page_by_campaign[campaign_id].page = 1;
			}

			let filter = {
				agentNameFilter  : agent_search,
				agentOrder       : AGENTS_ORDER_BY_NAME,
				page             : self.m_page_by_campaign[campaign_id].page,
				rows_per_page    : ROWS_PER_PAGE,
				showOfflineAgents: show_offline_agents,
				showKickouts: show_kickouts,
				workgroups       : self.m_workgroups.get_campaign_workgroups(campaign_id)
			};
			self.__set_account_al_to_filter( campaign_id, filter );
			self.__set_member_state_to_filter(campaign_id, filter);
			self.__set_member_interactions_state_to_filter(campaign_id, filter);

			// Save this custom filter, later in the refresh timeout, we will use this filter
			// to to update the view
			self.m_custom_filter_by_campaign[campaign_id] = agent_search;
			// trigger event
			self.__on_before_members_search(current_node);

			self.m_model.get_supervisor_agents(campaign_id, filter, function(view, error)
			{
				if ( !view || (!view.agents && error) ){
					inConcert.app.getMainWindow().showBannerMessage({ message : format("<%=_T('Unable to get supervisor users: {0}')%>", error.result || ''), timeout : 5000});
				}
				else if ( view.agents.length === 0 ) {
					self.__hide_members_list(current_node);
					self.__show_empty_member_list_msg(current_node, self.__get_team_list_message_to_show(campaign_id));
				}
				else {
					// hide message
					self.__hide_empty_member_list_msg(current_node);

					// update members
					self.m_page_by_campaign[campaign_id].agents = view.agents;
					self.m_page_by_campaign[campaign_id].agentsInQuery = view.agentsInQuery;
					self.__show_members_list(current_node);

					// create markup for each member and setup selection handler.
					self.__process_members_markup(current_node, view.agents);

					// update menu options
					self.__update_members_menu_options(current_node, view.agents.length);
				}
				// trigger on member list reloaded event
				self.on_member_list_reloaded(current_node, campaign_id, view.agents.length, view.agentsInQuery);

				// update filter actions buttons
				self.__enable_disable_filter_actions(campaign_id);

				self.hide_waiting(campaign_id);

				// set to idle status
				self.m_state[campaign_id] = 'idle';
				self.__enable_input_actions(current_node);
			});
		});

		const $state_filter = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		const $order = $('#sup_dash_members_panel_order_' + campaign_dom_id);
		const $filter_interactions_state = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);
		const $filter_reset = $('#sup_dash_members_panel_reset_' + campaign_dom_id); // reset search button

		//reset button click handler
		$filter_reset.unbind('click').bind('click', function ()
		{
			// trigger event
			self.__on_before_members_search(current_node);
			// delete previous filter
			delete self.m_custom_filter_by_campaign[campaign_id];
			// set first page.
			self.m_page_by_campaign[campaign_id].page = 1;

			self.get_members(campaign_id, function(_, members, total_members_left)
			{
				// if not members, show message
				if ( !members || members.length === 0 ) {
					self.__on_member_list_empty(current_node, campaign_id);
				}
				else {
					// otherwise, process members
					self.__show_members_list(current_node);
					self.__hide_empty_member_list_msg(current_node);
					self.__process_members_markup(current_node, members);
				}

				// update button status
				$order.attr('current-order', 'asc');
				$state_filter.val('').trigger('change');
				$agent_id.val('').focus();
				$filter_interactions_state.val('').trigger('change');
				// update filter actions buttons
				self.__enable_disable_filter_actions(campaign_id);
				// trigger event
				self.__on_after_members_search(current_node, members.length, total_members_left);
			});
		});

		var keyup_search_textboxes = function (e)
		{
			if (e.keyCode === 13) {
				$filter_button.click();
			}
			// update filter actions buttons
			self.__enable_disable_filter_actions(campaign_id);
		};

		//handler for Search and Clear.
		const keyup_event_name = 'keyup.supervisor_members_search';
		// textbox nombre de agente
		$agent_id
			.unbind(keyup_event_name)
			.bind(keyup_event_name, keyup_search_textboxes);
		// textbox tiempo de agentes en el estado ...
		const $agent_state_time = current_node.find('[filter_member_state_time="true"]'); // textbox agents state time
		$agent_state_time
			.unbind(keyup_event_name)
			.bind(keyup_event_name, keyup_search_textboxes);
		// textbox tiempo de interacciones en el estado ...
		const $interactions_state_time = current_node.find('[filter_member_interaction_state_time="true"]'); // textbox interaction state time
		$interactions_state_time
			.unbind(keyup_event_name)
			.bind(keyup_event_name, keyup_search_textboxes);
	};

	DashboardMembers.prototype.change_status_filter = function (campaign_id, status, order, must_clean_member_text)
	{
		if (status !== '' && !this.m_member_states[status]) {
			throw('DashboardMembers.prototype.change_status_filter - Unrecognized status ' + status);
		}
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		var $state_filter = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		var $order = $('#sup_dash_members_panel_order_' + campaign_dom_id);
		if (must_clean_member_text) {
			var $filter_text = $('#sup_dash_members_panel_text_' + campaign_dom_id); // search text button
			$filter_text.val('');
		}
		$order.attr('current-order', order);
		$state_filter.val(status).trigger('change');
	};

	DashboardMembers.prototype.change_member_filter = function (campaign_id, member)
	{
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		var $filter_text = $('#sup_dash_members_panel_text_' + campaign_dom_id);
		$filter_text.val(member);

		let agent_search = inConcert.Utils.removeSpecialChars(member);
		this.m_custom_filter_by_campaign[campaign_id] = agent_search;
		this.__enable_disable_filter_actions(campaign_id);
		this.__update_colapsed_filter(campaign_id);
	};

	DashboardMembers.prototype.__toggle_offline_members = function (current_node, campaign_id, need_to_show)
	{
		var self = this;

		// trigger event
		this.__on_before_members_search(current_node);

		var filter = {
			agentOrder       : AGENTS_ORDER_BY_NAME,
			page             : this.m_page_by_campaign[campaign_id].page,
			rows_per_page    : ROWS_PER_PAGE,
			showOfflineAgents: need_to_show,
			workgroups       : this.m_workgroups.get_campaign_workgroups(campaign_id)
		};
		this.__set_account_al_to_filter( campaign_id, filter );
		this.__set_member_state_to_filter(campaign_id, filter);
		this.__set_member_interactions_state_to_filter(campaign_id, filter);

		this.m_model.get_supervisor_agents(campaign_id, filter, function(view, error)
		{
			if ( !view || !view.agents || error ){
				inConcert.app.getMainWindow().showBannerMessage({ message : format("<%=_T('Unable to get supervisor users: {0}')%>", error.result || ''), timeout : 5000});
			}
			if ( view.agents.length === 0 ) {
				self.__hide_members_list(current_node);
				self.__show_empty_member_list_msg(current_node, "<%=_T('There are no users for this view')%>");
				self.__disable_input_actions(current_node);
			}
			else {

				// update members
				self.m_page_by_campaign[campaign_id].agents = view.agents;
				self.m_page_by_campaign[campaign_id].agentsInQuery = view.agentsInQuery;

				// show list
				self.__show_members_list(current_node);

				// create markup for each member and setup selection handler.
				self.__process_members_markup(current_node, view.agents);
			}
			self.on_member_list_reloaded(current_node, campaign_id, view.agents.length, view.agentsInQuery);
			self.__on_after_members_search(current_node, view.agents.length, view.agentsInQuery);
		});
	};

	DashboardMembers.prototype.__toggle_members_avatars = function (current_node, members, need_to_show)
	{
		var self = this;
		var campaign_id = current_node.attr('campaign_id');

		// handle server weird errors.
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		var avatar_url = inConcert.app.getBaseUrl() + '/users/get_avatar/';
		// update avatar url
		members.forEach(function (member)
		{
			var $member = self.__get_member_node(campaign_id, current_node, member.ID).find('.memberAvatar');
			if ($member.length > 0) {
				if (need_to_show) {
					$member.attr('src', avatar_url + member.ID);
				}
				else {
					$member.removeAttr('src');
				}
			}
		});

		// add a custom class to the container
		// this is done to conditionate the table rows via css
		this.__get_panel_container(campaign_id).toggleClass('showAvatar', need_to_show);

		// toggle avatars
		current_node.find('.memberAvatarContainer').toggle(need_to_show);
	};

	DashboardMembers.prototype.__toggle_members_states_time = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member) {
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $member_state_time = $member.find('[teamTime]');
			if (can_show) {
				$member_state_time.show();
			}
			else {
				$member_state_time.hide();
			}
		});
	};

	DashboardMembers.prototype.__toggle_members_states_name = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member) {
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $member_state_name = $member.find('[teamStateName]');
			$member_state_name[can_show? "show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_members_states_visual_indicator = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member) {
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $memberStatusIndicator = $member.find('[memberStatusIndicator]');
			$memberStatusIndicator[can_show? "show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_members_workgroups = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member) {
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $member_workgroup = $member.find('[teamWorkgroup]');
			$member_workgroup[can_show? "show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_members_total_conversations = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member)
		{
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			let $member_channels_interactions = $member.find("[teamTotalNumInteractions]");
			$member_channels_interactions[can_show? "show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_members_total_conversations_by_channel = function (current_node, members, can_show)
	{
		const self = this;
		const campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member)
		{
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			let $member_channels_interactions = $member.find("[member_channels_interactions]");
			$member_channels_interactions[can_show? "show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_members_names = function (current_node, members, show_names)
	{
		const self = this;
		let campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member)
		{
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $member_name = $member.find('[teamName]');
			if (show_names) {
				$member_name.text(member.name);
				$member_name.show();
			}
			else {
				$member_name.hide();
			}
		});
	};

	DashboardMembers.prototype.__toggle_members_id = function (current_node, members, show_id)
	{
		const self = this;
		let campaign_id = current_node.attr('campaign_id');

		// handle server errors
		if ( !members || members.length === 0 || !campaign_id ){
			return;
		}

		members.forEach(function (member)
		{
			const $member = self.__get_member_node(campaign_id, current_node, member.ID);
			const $member_id = $member.find('[teamNameId]');
			$member_id[show_id?"show":"hide"]();
		});
	};

	DashboardMembers.prototype.__toggle_filter = function(campaign_id)
	{
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $filter_colapsed_container = $("#sup_dash_members_panel_filter_colapsed_container_" + campaign_dom_id);
		const is_visible_filter_colapsed = $filter_colapsed_container.is(":visible");
		const $filter_uncolapsed_container = $("#sup_dash_members_panel_filter_uncolapsed_container_" + campaign_dom_id);
		const $panel = this.__get_panel_node(campaign_id);

		$panel.removeClass("showFilter hideFilter");
		if (is_visible_filter_colapsed) {
			$panel.addClass("showFilter");
			$filter_uncolapsed_container.show();
			$filter_colapsed_container.hide();
		}
		else {
			$panel.addClass("hideFilter");
			$filter_uncolapsed_container.hide();
			$filter_colapsed_container.show();
		}
	};

	DashboardMembers.prototype.__update_colapsed_filter = function(campaign_id)
	{
		const self = this;
		let applied_filter = "";

		var add_filter_desc = function($filter_value, filter_desc, convert_value_fn = function(value) {return value;})
		{
			let filter_value = $filter_value.val().trim();
			if (filter_value.length === 0) {
				return false;
			}
			let separator = "";
			if (applied_filter.length > 0) {
				separator = ", ";
			}
			filter_value = convert_value_fn(filter_value);
			applied_filter = applied_filter + separator + format(filter_desc, filter_value);
			return true;
		};

		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		// 1) filtro agente en un estado
		const $filter_agent_state = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		let added = add_filter_desc($filter_agent_state, "<%=_T('Agent state: {0}')%>",
			function(id) {
				return self.__get_description_member_state(id);
			}
		);
		if (added) {
			// 1.1) filtro tiempo de un agente en un estado
			const $filter_agent_state_time = $('#sup_dash_members_panel_state_filter_time_' + campaign_dom_id);
			add_filter_desc($filter_agent_state_time, "<%=_T('State Time (min): {0}')%>");
		}

		// 2) filtro nombre de agente
		const $filter_agent_text = $('#sup_dash_members_panel_text_' + campaign_dom_id);
		add_filter_desc($filter_agent_text, "<%=_T('Agent name: {0}')%>");

		// 3) filtro interacciones en un estado
		const $filter_interactions_state = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);
		added = add_filter_desc($filter_interactions_state, "<%=_T('Conversation state: {0}')%>",
			function(id) {
				return self.__get_description_interaction_state(id);
			}
		);
		if (added) {
			// 3.1) filtro tiempo de interacciones en un estado
			const $filter_interactions_state_time = $('#sup_dash_members_panel_interactions_state_filter_time_' + campaign_dom_id);
			add_filter_desc($filter_interactions_state_time, "<%=_T('State Time (min): {0}')%>");
		}

		let tooltip = applied_filter;
		if (applied_filter.length === 0) {
			applied_filter = "<%=_T('Without Filters')%>";
			tooltip = "";
		}

		let full_screen = this.m_dashboard.is_full_screen_mode();
		if (!full_screen) {
			applied_filter = inConcert.Utils.truncateWord(applied_filter, 30);
		}
		else {
			applied_filter = inConcert.Utils.truncateWord(applied_filter, 200);
		}

		$("#sup_dash_members_panel_filter_colapsed_selected_" + campaign_dom_id)
			.attr("title", tooltip)
			.text(applied_filter);
	};

	DashboardMembers.prototype.__enable_disable_filter_actions = function (campaign_id)
	{
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $reset = $('#sup_dash_members_panel_reset_' + campaign_dom_id); // reset search button
		const $search = $('#sup_dash_members_panel_search_' + campaign_dom_id); // search button

		var enable = function() {
			$search.prop("disabled", false);
			$reset.prop("disabled", false);
		};
		// Estado del Agente
		const $filter_agent_state = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		if ($filter_agent_state.val().length > 0) {
			enable();
			return;
		}
		// Tiempo del Agente en el Estado
		const $filter_agent_state_time = $('#sup_dash_members_panel_state_filter_time_' + campaign_dom_id);
		if ($filter_agent_state_time.val().length > 0) {
			enable();
			return;
		}
		// Nombre del Agente
		const $filter_agent_text = $('#sup_dash_members_panel_text_' + campaign_dom_id);
		if ($filter_agent_text.val().length > 0) {
			enable();
			return;
		}
		// Estado de la Conversacion
		const $filter_interactions_state_time = $('#sup_dash_members_panel_interactions_state_filter_time_' + campaign_dom_id);
		if ($filter_interactions_state_time.val().length > 0) {
			enable();
			return;
		}
		// Estado de la Conversacion
		const $interactions_state_filter = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);
		if ($interactions_state_filter.val().length > 0) {
			enable();
			return;
		}

		// Desactivado
		$search.prop("disabled", true);
		$reset.prop("disabled", true);
	};

	DashboardMembers.prototype.__set_events = function ($current_node)
	{
		const self = this;

		let campaign_id = $current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $panel_node = this.__get_panel_node(campaign_id);
		const $members_menu_container = $panel_node.find('.columnOptionsContainer');
		const $popup_list_container = $panel_node.find('.popupStatusList');

		// Show/Hide options list handler
		$('#sup_dash_column_edit_members_' + campaign_dom_id)
			.unbind('click')
			.bind('click', function() {
				$members_menu_container.toggle();
			});

		$popup_list_container
			.unbind('clickoutside')
			.bind( 'clickoutside', function(e) {
				e.stopPropagation();
				if ( !$(e.target).hasClass('butTeamAction')){
					$popup_list_container.hide();
				}
			});

		$members_menu_container
			.unbind('clickoutside')
			.bind( 'clickoutside', function(e) {
				e.stopPropagation();
				if ( !$(e.target).closest(`#sup_dash_column_edit_members_${campaign_dom_id}`).length ){
					$members_menu_container.hide();
				}
			});

		const $members_options_container =  $panel_node.find('#sup_dash_members_options_container');
		
			$members_options_container.off('change')
			.on('change', "[action_show_option]", function ()
			{
				let members = self.__get_current_members_from_list($current_node, true);
				let option_checked = this.checked;	
				switch (this.id) {
					case 'sup_dash_show_offline_members_' + campaign_dom_id:
						self.__toggle_offline_members($current_node, campaign_id, option_checked);
						break;
					case 'sup_dash_show_avatars_members_option_' + campaign_dom_id:
						self.__toggle_members_avatars($current_node, members, option_checked);
						break;
					case 'sup_dash_show_names_members_' + campaign_dom_id:
						self.__toggle_members_names($current_node, members, option_checked);
						if (!option_checked && !$("#sup_dash_show_id_members_" + campaign_dom_id).prop("checked")) {
							$(`#${this.id}`).prop("checked", true).change();
						}
						break;
					case 'sup_dash_show_id_members_' + campaign_dom_id:
						self.__toggle_members_id($current_node, members, option_checked);
						if (!option_checked && !$(`#sup_dash_show_names_members_${campaign_dom_id}`).prop("checked")) {
							$(`#sup_dash_show_names_members_${campaign_dom_id}`).prop("checked", true).change();
						}
						break;
					case 'sup_dash_show_state_time_members_option_' + campaign_dom_id:
						self.__toggle_members_states_time($current_node, members, option_checked);
						break;
					case 'sup_dash_show_state_visual_indicator_members_option_' + campaign_dom_id:
						self.__toggle_members_states_visual_indicator($current_node, members, option_checked);
						if (!option_checked && !$(`#sup_dash_show_state_name_members_option_${campaign_dom_id}`).prop("checked")) {
							$(`#${this.id}`).prop("checked", true).change();
						}
						break;
					case 'sup_dash_show_state_name_members_option_' + campaign_dom_id:
						self.__toggle_members_states_name($current_node, members, option_checked);
						if (!option_checked && !$(`#sup_dash_show_state_visual_indicator_members_option_${campaign_dom_id}`).prop("checked")) {
							$(`#sup_dash_show_state_visual_indicator_members_option_${campaign_dom_id}`).prop("checked", true).change();
						}
						break;
					case 'sup_dash_show_workgroups_option_' + campaign_dom_id:
						self.__toggle_members_workgroups($current_node, members, option_checked);
						break;
					case 'sup_dash_show_total_conversations_option_' + campaign_dom_id:
						self.__toggle_members_total_conversations($current_node, members, option_checked);
						break;
					case 'sup_dash_show_total_conversations_by_channel_option_' + campaign_dom_id:
						self.__toggle_members_total_conversations_by_channel($current_node, members, option_checked);
						break;
					case 'sup_dash_show_all_' + campaign_dom_id:
							$.each($members_options_container.find("[action_option_all]"), function(_, action) {
								const $action = $(action);
								$action.prop("checked", option_checked).change();
							});
						break;
					default:
						break;
				}

				//Select all checkboxes if the checkbox for all is selected
				let all_are_checked = true;
				$.each($members_options_container.find("[action_option_all]"), function(_, action) {
					if ( !$(action).prop("checked")) {
						all_are_checked = false;
						return false;
					}
				});
				$(`#sup_dash_show_all_${campaign_dom_id}`).prop("checked", all_are_checked);

				// Save configuration
				self.m_dashboard.save_config();
			}
		);

		const $colapse_button = $('#sup_dash_members_panel_switch_colapse_mode_' + campaign_dom_id); // search button
		$colapse_button
			.unbind('click')
			.bind('click', function() {
				self.__toggle_filter(campaign_id);
			});

		const $filter_button = $('#sup_dash_members_panel_search_' + campaign_dom_id); // search button
		const $order = $('#sup_dash_members_panel_order_' + campaign_dom_id);
		const $state_filter = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		const $filter_agent_state_time = $('#sup_dash_members_panel_state_filter_time_' + campaign_dom_id);
		const $filter_agent_text = $('#sup_dash_members_panel_text_' + campaign_dom_id); // search agent text button

		$filter_agent_text
			.unbind('change')
			.bind('change', function () {
				self.__update_colapsed_filter(campaign_id);
			}
		);

		let hnd_agent_state_timeout = null;
		var clear_agent_state_timeout = function() {
			window.clearTimeout(hnd_agent_state_timeout);
			hnd_agent_state_timeout = null;
		};
		var change_state = function()
		{
			clear_agent_state_timeout();
			// El change de los state se realiza dentro de un timeout para que el request no se ejecute todas las veces
			// que se cambia en el combo, sino cuando ya ha seleccionado un state.
			hnd_agent_state_timeout = window.setTimeout(function()
			{
				clear_agent_state_timeout();
				self.__update_colapsed_filter(campaign_id);
				self.__enable_disable_filter_actions(campaign_id);
				if ($state_filter.val() === "") {
					$filter_agent_state_time.val("");
					$order.prop('disabled', true);
					$filter_button.trigger('click');
					return;
				}
				$filter_button.trigger('click');
				$order.prop('disabled', false);
			}, 500);
		};

		$state_filter
			.unbind('change')
			.bind('change', function () {
				change_state();
			}
		);

		$filter_agent_state_time
			.unbind('change')
			.bind('change', function () {
				self.__update_colapsed_filter(campaign_id);
			}
		);

		let hnd_interactions_state_timeout = null;
		var clear_interaction_state_timeout = function() {
			window.clearTimeout(hnd_interactions_state_timeout);
			hnd_interactions_state_timeout = null;
		};
		const $filter_interactions_state_time = $('#sup_dash_members_panel_interactions_state_filter_time_' + campaign_dom_id);
		const $interactions_state_filter = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);
		var change_interactions_state = function()
		{
			clear_interaction_state_timeout();
			// El change de los interactions state se realiza dentro de un timeout para que el request no se ejecute todas las veces
			// que se cambia en el combo, sino cuando ya ha seleccionado un interactions state.
			hnd_interactions_state_timeout = window.setTimeout(function()
			{
				clear_interaction_state_timeout();
				self.__update_colapsed_filter(campaign_id);
				self.__enable_disable_filter_actions(campaign_id);
				if ($interactions_state_filter.val() === "") {
					$filter_interactions_state_time.val("");
					$filter_button.trigger('click');
					return;
				}
				$filter_button.trigger('click');
			}, 500);
		};

		$interactions_state_filter
			.unbind('change')
			.bind('change', function () {
				change_interactions_state();
			}
		);

		$filter_interactions_state_time
			.unbind('change')
			.bind('change', function () {
				self.__update_colapsed_filter(campaign_id);
			}
		);

		let hnd_order_timeout = null;
		var clear_order_timeout = function() {
			window.clearTimeout(hnd_order_timeout);
			hnd_order_timeout = null;
		};
		var change_order = function()
		{
			clear_order_timeout();
			// El change del orden se realiza dentro de un timeout para que el request no se ejecute todas las veces
			// que se cambia orden, sino cuando ya ha seleccionado uno.
			hnd_order_timeout = window.setTimeout(function()
			{
				clear_order_timeout();
				let current_order = $order.attr('current-order');
				let new_order = current_order === 'asc' ? 'desc' : 'asc';
				$order.attr('current-order', new_order);
				$filter_button.trigger('click');
			}, 500);
		};

		$order
			.unbind('click')
			.bind('click', function () {
				change_order();
			}
		);
	};

	DashboardMembers.prototype.has_permission_to = function ( action_id, campaign_id )
	{
		return inConcert.permissions.to_action( 'supervisor', action_id, undefined /* category */, campaign_id, true /* no log */ );
	};

	DashboardMembers.prototype.__checkScreenStreaming = function()
	{
		var self = this;
		var $popup = $("#screenStreamingPopup");
		if (!$popup.length) {
			return;
		}
		var userId = $popup.data("user");
		self.m_model.is_member_logged(userId, function(logged) {
			if (!logged) {
				self.__hideScreenStreamingPopup(format("<%=_T('User {0} disconected.')%>", userId));
			}
		});
	};
	
	DashboardMembers.prototype.__hideScreenStreamingPopup = function(message)
	{
		var self = this;
		var $popup = $("#screenStreamingPopup");
		if (!$popup.length) {
			return;
		}
		if (self.m_checkScreenStreaming) {
			window.clearInterval(self.m_checkScreenStreaming);
			self.m_checkScreenStreaming = false;
		}
		var user = $popup.data("user");
		var campaign = $popup.data("campaign");
		if (message) {
			inConcert.app.getMainWindow().showBannerMessage({ message : message, timeout : 5000});
		}

		function supervisorPlayerClosed(){
			if (self.screenStreamingPopup && self.screenStreamingPopup.isVisible()) {
				self.screenStreamingPopup.finalize(function() {		
					self.screenStreamingPopup = null;
					$popup.remove();
				});
			}
			else {
				$popup.remove();
			}
			self.show_waiting(campaign, format("<%=_T('Clossing user {0} screen')%>", user));
			self.m_model.stop_screen_streaming(user, campaign, function() {
				self.hide_waiting(campaign);
			});
		}
		if (inConcert.app.has_allegro_integration()) {
			supervisorPlayerClosed();
		}
		else {
			if (!this.addedSupervisorPlayerClosedEvent) {
				// This is to avoid socket abort error on server 
				window.document.addEventListener("SupervisorPlayerClosed", supervisorPlayerClosed);
				this.addedSupervisorPlayerClosedEvent = true;
			}
			var event = new CustomEvent("SupervisorPlayerClosing", {});
			$("#iframePlayer")[0].contentDocument.dispatchEvent(event);
		}
	};

	DashboardMembers.prototype.showScreenStreamingPopup = function(campaign_id, member, data)
	{
		var self = this;
		var url = "/inconcert/apps/supervisor/pages/screenStreaming.<%=_L%>.html" +
			"?channel=" + encodeURIComponent(data.channel) +
			"&node=" + encodeURIComponent(data.node || "1") + 
			"&isOCC=" + encodeURIComponent(data.isOCC || false);
		$("#screenStreamingPopup").remove();
		var $popup = $(''.concat(
			'<div id="screenStreamingPopup" class="internalPopUpContainer screenStreamingPopup fullScreenPopUp" hidden>',
				'<div class="screenStreamingInternalPopUpInternalPopUp">',
					'<div class="screenStreamingInternalPopUpTitle">',
						format("<%=_T('User {0} screen')%>", member.ID),
						'<button close_button class="closeButton" type="button"><i class="buttons-icons-x"></button>',
					'</div>',
					'<div class="screenStreamingInternalPopUpMiddle">',
						'<iframe id="iframePlayer" src="', url, '">',
						'</iframe>',
					'</div>',
				'</div>',
			'</div>'
		));
		$popup.data("user", member.ID)
			.data("campaign", campaign_id);
		$popup.find("[close_button]").click(function() {
			self.__hideScreenStreamingPopup();
		});
		$("body").append($popup);
		self.screenStreamingPopup = new inConcert.PopupWindow();
		self.screenStreamingPopup.show({
			ownerId         : "screenStreamingPopup",
			eventName       : "screenStreamingPopup",
			canReposition   : true,
			closeOnFocusLost: false,
			overlay         : true,
			container       : $popup,
			inlineStyleValues : {
				["z-index"]: 998
			},
			callbacks: {
				onOpen: function() {
					$("#screenStreamingPopup").show();
				},
				onClose: function( callback = $.noop ) {
					if (self.screenStreamingPopup) {
						self.screenStreamingPopup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		self.m_checkScreenStreaming = window.setInterval(function() {
			self.__checkScreenStreaming();
		}, 3000);
	};

	DashboardMembers.prototype.__setup_broadcast_button_handlers = function (current_node)
	{
		const self = this;

		let campaign_id = current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		let $broadcast_button = $('#sup_dash_members_broadcast_' + campaign_dom_id);

		let has_permission = this.has_permission_to( "sup_broadcast", campaign_id );
		if ( !has_permission ) {
			$broadcast_button.hide();
		}
		else {
			$broadcast_button.show();
		}

		const $broadcast_container = current_node.find('.broadcastPopup');
		$broadcast_button.unbind('clickoutside')
			.bind('clickoutside', function() {
				$broadcast_container.hide();
			}
		);

		$broadcast_button
			.unbind('click')
			.bind('click', function () {
				if ( !self.has_permission_to( 'sup_broadcast', campaign_id ) ) {
					const error_message = "<%=_T('NOT HAS PERMISSIONS TO SEND BROADCAST MESSAGE')%>";
					inConcert.app.getMainWindow().showBannerMessage({ message : error_message , timeout : 5000});
					return;
				}
				let message_container = current_node.find('.broadcastArea');
				let send_button = $broadcast_container.find('.butBroadcastSend');

				// Show popup and focus on message container.
				$broadcast_container.toggle();
				message_container.focus();

				// Events for send message
				send_button
					.unbind('click')
					.bind('click', function() {
						let message_to_send = message_container.val();
						const filter = {
							agentOrder       : AGENTS_ORDER_BY_NAME,
							page             : -1,
							rows_per_page    : ROWS_PER_PAGE,
							showOfflineAgents: true,
							workgroups       : self.m_workgroups.get_campaign_workgroups(campaign_id)
						};
						self.__set_account_al_to_filter( campaign_id, filter );
						self.__set_member_state_to_filter(campaign_id, filter);
						self.__set_member_interactions_state_to_filter(campaign_id, filter);

						self.m_model.get_supervisor_agents(campaign_id, filter, function(view, error)
						{
							if ( !view && !view.agents && error ){
								inConcert.app.getMainWindow().showBannerMessage({
									message : format("<%=_T('Unable to send broadcast message: {0}')%>", error.result || ''),
									timeout : 5000
								});
								return self.m_dashboard.on_error();
							}

							// si se ha seleccionado corte por AL, el mensaje del broadcast va unicamente a los members de este AL.
							let al_users = {};
							let campaign_config = self.m_dashboard.get_campaign_selected_config(campaign_id);
							if (campaign_config.al_id) {
								let campaign = inConcert.app.getCampaign(campaign_id);
								if ( !campaign ) {
									inConcert.app.getMainWindow().showBannerMessage({
										message : format("<%=_T('Unable to send broadcast message: Campaign {0} does not exist.')%>", campaign_id),
										timeout : 5000
									});
									return self.m_dashboard.on_error();
								}
								al_users = campaign.getUsersInAttentionLevel(campaign_config.al_id);
							}
							var can_add = function(agent_id)
							{
								if (!campaign_config.al_id) {
									return true;
								}
								return al_users[agent_id] ? true : false;
							};
							let users_ids = [];
							$.each(view.agents, function(_, agent) { if (can_add(agent.ID) ) { users_ids.push(agent.ID); } });
							//----

							self.m_model.broadcast_message( message_to_send, users_ids, campaign_id,
								function( result, error ) {
									if ( result ){ 
										if (result.messagesSent > 0) {
											inConcert.app.getMainWindow().showBannerMessage({
												message : format("<%=_T('Message was succesfully sent to {0} users!')%>", result.messagesSent), 
												messageType : 'Info',
												timeout : 3000
											});
										}
										else {
											inConcert.app.getMainWindow().showBannerMessage({ 
												message : "<%=_T('All users are offline at this moment!')%>", 
												messageType : 'Info',
												timeout : 3000
											});
										}
										message_container.val('');
										$broadcast_container.hide();
									}
									else {
										let error_message = error;
										if ( error.toLowerCase() === 'forbidden' ) {
											error_message = "<%=_T('NOT HAS PERMISSIONS TO BROADCAST ACTION')%>";
										}
										inConcert.app.getMainWindow().showBannerMessage({
											message : format( "<%=_T('Unable to send broadcast message with error: {0}')%>", error_message ),
											timeout : 5000
										});
										self.m_dashboard.on_error();
									}
								}
							);
						});
					});

				// Events for cancel popup
				$broadcast_container.find('.butBroadcastCancel')
					.unbind('click')
					.bind('click', function(){
						$broadcast_container.hide();
					});
			});
	};

	DashboardMembers.prototype.__popup_open_member_info = function($member, $campaign)
	{
		const self = this;

		this.__popup_close_member_info(); // si ya estaba abierto, lo cierro

		let callbacks = {
		// Calbacks invocados de la clase Base PopupWindow
			onOpen: $.noop,
			onClose: function() {
				window.setTimeout(function() {
					self.m_popup_member_detail = null;
				});
			}
		};

		let member = $member.data("member");
		let campaign_id = $member.data("campaign_id");

		this.m_popup_member_detail = new popup_member_detail({
			from_app   : "supervisor",
			member     : member,
			$member    : $member,
			campaign_id: campaign_id,
			$campaign  : $campaign
		});

		this.m_popup_member_detail.show({
			parentNode  : $member,
			owner_object: this,
			callbacks   : callbacks
		});
	};

	DashboardMembers.prototype.__popup_close_member_info = function()
	{
		if (!this.m_popup_member_detail) {
			return;
		}
		this.m_popup_member_detail.close();
		this.m_popup_member_detail = null;
	};

	DashboardMembers.prototype.__popup_auto_close_member_info = function()
	{
		if (!this.m_popup_member_detail) {
			return;
		}
		this.m_popup_member_detail.auto_close();
	};

	DashboardMembers.prototype.__popup_update_member_info = function($member)
	{
		if (!this.m_popup_member_detail) {
			return;
		}
		if (!$member || $member.length === 0) {
			this.__popup_close_member_info();
			return;
		}
		let member = $member.data('member');
		let campaign_id = $member.data("campaign_id");
		if (!member) {
			this.__popup_close_member_info();
			return;
		}
		let current_popup_member = this.m_popup_member_detail.get_member();
		let current_popup_campaign_id = this.m_popup_member_detail.get_campaign_id();
		if (!current_popup_member || (current_popup_member.ID === member.ID && current_popup_campaign_id === campaign_id)) {
			this.m_popup_member_detail.update_member_info(member, $member);
		}
	};

	/**
	 * Called when the supervisor selects a member row.
	 *
	 * @private
	 * @param  {object} $current_node - Current campaign node.
	 */
	DashboardMembers.prototype.__setup_member_row_selection = function ($campaign)
	{
		const self = this;
		let $members_list_items = this.__$get_members_list_items($campaign);

		// When someone selects a member from the table.
		$members_list_items.unbind('click').bind('click', function ()
		{
			// remove other statuses and interaction text
			$members_list_items.find('.teamText').remove();
			// update selected member.
			let $member = $(this);
			$member.addClass('selected').siblings().removeClass('selected');

			// abre el popup con el detalle del member
			self.__popup_open_member_info($member, $campaign);
		});

		$members_list_items.unbind('clickoutside')
			.bind('clickoutside', function(e) {
				let target = $(e.target);
				let target_is_agent_action = target.hasClass('.butTeamAction');
				let target_is_agent_node = target.parents().is('li');

				if (!target_is_agent_node && !target_is_agent_action) {
					let $selected_members_list_items = self.__$get_members_list_items($campaign);
					$selected_members_list_items.removeClass('selected');
				}
			}
		);
	};

	DashboardMembers.prototype.__update_member_info = function (current_node, member, $member, show_names, show_offline)
	{
		// check if node is available to update,
		// there are cases when the server data is outdated.
		if ($member.attr('prevent_update') === 'true'){
			$member.removeAttr('prevent_update');
			return;
		}
		$member.removeAttr('prevent_update');

		// Toggle node based on current settings and member status.
		if (!member.isLogged){
			$member.toggle(show_offline);
		}
		else {
			$member.show();
		}

		// update internal data.
		$member.data('member', member);

		// muestra la alarma en el indicador (si corresponde visualizarla)
		this.__show_alarm($member, member);

		// update attention level
		var campaign_id = current_node.attr('campaign_id');
		this.__update_member_attention_level(campaign_id, $member, member);

		// update agent missing skills
		this.__update_member_missing_skills(campaign_id, $member, member);

		// update channels interactions
		this.__update_member_channels(campaign_id, $member, member);

		// update workgroup
		$member.find('[teamWorkgroup]').text( member.workgroup && member.workgroup !== "" ? `<%=_T([[Group]])%>: ${ member.workgroup }`: '');

		// Update logged in date
		var new_state_time = this.get_member_state_time_ago(member, campaign_id, false, true);
		$member.find('.teamTime').text(new_state_time);

		// Update state name
		let new_state_name = this.get_member_state_msg(member, true);
		$member.find('.teamStateName').text(new_state_name);

		// Update Name
		$member.find('.teamName').text(this.__get_member_name(member, show_names));

		// Campaign Status
		var previous_status_class = $member.find('.memberStatus').attr('class');
		var new_status_class = 'memberStatus ' + this.get_member_status_class( member );
		var $img = this.get_status_img_node($member);
		if ( previous_status_class !== new_status_class ) {
			// update status class
			$img.removeClass().addClass(new_status_class);
		}
		//update tooltip
		var member_state = this.get_member_state_msg( member, true);
		$img.inconcert_tooltip({ message: member_state, alignment: 'right-bottom'} );

	// Kickout Status
		var previous_kickout_status_class = $member.find('.kickoutStatus').attr('class');
		var new_kickout_status_class = 'kickoutStatus ' + this.get_member_kickout_class( member );
		var $imgKickout = this.get_kickout_img_node($member);
		if ( previous_kickout_status_class !== new_kickout_status_class ) {
			// kickout status class
			$imgKickout.removeClass().addClass(new_kickout_status_class);
		}
	//update kickout tooltip
		var kickout_msg = this.get_member_kickout_msg( member );
		$imgKickout.inconcert_tooltip({ message: kickout_msg, alignment: 'right-bottom'} );

		// actualiza los datos del detalle del popup de member, si es que se encuentra abierto.
		this.__popup_update_member_info($member);
	};

	/**
	 * Will update menu checkboxes based on previous supervisor config.
	 *
	 * @private
	 * @param  {string} campaign_dom_id - The campaign DOM id (i.e. id_campaign_vcc)
	 * @param  {object} config          - The configuration for the current panel
	 */
	DashboardMembers.prototype.__update_previous_config_menu_options = function (campaign_dom_id, config)
	{
		$('#sup_dash_show_offline_members_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_offline : false);
		$('#sup_dash_show_avatars_members_option_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_avatars : false);
		
		$('#sup_dash_show_id_members_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_agents_id : false);
		const $show_names = $('#sup_dash_show_names_members_' + campaign_dom_id);
		$show_names.prop('checked', config && config.options ? (config.options.show_names || !config.options.show_agents_id): true);

		$('#sup_dash_show_state_time_members_option_' + campaign_dom_id).prop('checked', config && config.options? config.options.show_state_time : false);
		$('#sup_dash_show_state_name_members_option_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_state_name : false);
		$('#sup_dash_show_kickouts_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_kickouts : false);

		const $show_state_visual_indicator = $('#sup_dash_show_state_visual_indicator_members_option_' + campaign_dom_id);
		$show_state_visual_indicator.prop('checked', config && config.options ? (config.options.show_state_visual_indicator || !config.options.show_state_name): true);

		$('#sup_dash_show_all_' + campaign_dom_id).prop('checked', config && config.options? config.options.show_all : false);
		$('#sup_dash_show_workgroups_option_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_workgroups : false);
		$('#sup_dash_show_total_conversations_option_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_total_conversations : false);
		$('#sup_dash_show_total_conversations_by_channel_option_' + campaign_dom_id).prop('checked', config && config.options ? config.options.show_total_conversations_by_channel : false);
	};

	/**
	 * @private
	 * @param  {Object} current_node - Current campaign node.
	 * @param  {Object} config       - The current config for the panel.
	 * @param  {Array} members       - Current list of members.
	 */
	DashboardMembers.prototype.__update_previous_config_team_list = function (current_node, config, members)
	{
		// Based on the current supervisor config, update member table options.
		this.__toggle_members_avatars(current_node, members, config ? config.options.show_avatars : false);
		this.__toggle_members_names(current_node, members, config ? config.options.show_names : true);
	};

	/**
	 * Given a campaign node, and a list of members (array)
	 * will create nodes if does not exist and if fits on the list,
	 * or will update the node data if the member already exist on the team list.
	 *
	 * @private
	 * @param  {Object}   current_node  - Current campaign node.
	 * @param  {Array}    members       - Current list of members.
	 * @param  {Function} callback      - Callback, called with true if node were added to the panel.
	 */
	DashboardMembers.prototype.__process_members_markup = function ($current_node, members, callback)
	{
		const self = this;
		// Nothing to do without members or campaign.
		if (!members || members.length === 0){
			if (typeof(callback) === 'function'){
				callback(false);
			}
			return;
		}

		let campaign_id = $current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		let workgroups = this.m_workgroups.get_campaign_workgroups(campaign_id);

		// config related variables
		let current_config = this.m_config.get_panel_config(campaign_id, 'sup_dash_members_panel');
		let show_names = current_config ? current_config.options.show_names : true;
		let show_offline = current_config ? current_config.options.show_offline : false;
		let show_workgroups = current_config ? current_config.options.show_workgroups : false;

		const $members_list = this.__$get_members_list($current_node);
		let scroll_top = $members_list.parent().parent().scrollTop();
		let scroll_container = $members_list.parent();
		let detached_team_list = $members_list.detach();

		// compare previous members ids, if we got differents members,
		// clean the members list.
		let previous_members = this.m_page_by_campaign[campaign_id].previous_agents || [];
		let previous_members_ids = _.pluck(previous_members, 'ID');
		let current_members_ids = _.pluck(members, 'ID');
		if ( !_.isEqual(previous_members_ids, current_members_ids )){
			this.__clear_member_list($current_node, detached_team_list);
		}

		let have_space_left = this.__team_list_have_space_left(detached_team_list);
		let nodes_were_added = false;

		members.forEach(function (member)
		{
			let $member = detached_team_list.find('#sup_member_' + campaign_dom_id + '_' + nid(member.ID));

			// Append new member if not exist, if have space left and I'm not currently searching.
			if ( $member.length === 0 && have_space_left && self.m_state[campaign_id] !== 'paginating'){
				$member = self.__$build_member_markup($current_node, member);
				$member.data('member', member);
				// muestra la alarma en el indicador (si corresponde visualizarla)
				self.__show_alarm($member, member);
				let member_state = self.get_member_state_msg(member, true);
				self.get_status_img_node($member).inconcert_tooltip({ message : member_state, alignment: 'right-bottom'} );
				//Agregar tooltip a icono de kickout
				var kickout_msg = self.get_member_kickout_msg( member );
				self.get_kickout_img_node($member).inconcert_tooltip({ message : kickout_msg, alignment: 'right-bottom'} );

				// attach node to the list.
				detached_team_list.append($member);
				nodes_were_added = true;
			}
			else {
				// Just update old info.
				self.__update_member_info($current_node, member, $member, show_names, show_offline, workgroups, show_workgroups);
			}
		});

		// re append the updated team list to the DOM
		scroll_container.append(detached_team_list);
		// adjust the scroll
		scroll_container.parent().scrollTop(scroll_top);

		if (typeof(callback) === 'function'){
			callback(nodes_were_added);
		}
	};

	DashboardMembers.prototype.__set_member_state_to_filter = function(campaign_id, filter)
	{
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		let $state_filter = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);
		let state = $state_filter.val();

		if (state === '') {
			return;
		}
		let order_desc = false;
		const $order = $('#sup_dash_members_panel_order_' + campaign_dom_id);
		if ($order.attr('current-order') === 'desc') {
			order_desc = true;
		}
		const $state_time = $('#sup_dash_members_panel_state_filter_time_' + campaign_dom_id);
		if ($state_time.length === 0) {
			return;
		}
		filter.agentStateFilter = state;
		filter.orderDesc = order_desc;
		filter.agentOrder = AGENTS_ORDER_BY_SATE_TIME;

		let state_time_min = parseInt($state_time.val().trim(), 10);
		if (isNaN(state_time_min)) {
			return;
		}
		let state_time_sec = state_time_min * 60;
		filter.agentStateTimeFilter = state_time_sec;
		filter.agentStateTimeOperatorFilter = ">=";
	};

	DashboardMembers.prototype.__set_member_interactions_state_to_filter = function(campaign_id, filter)
	{
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		let $interactions_state_filter = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);
		let interactions_state = $interactions_state_filter.val();

		if (interactions_state === '') {
			return;
		}
		let order_desc = false;
		const $order = $('#sup_dash_members_panel_order_' + campaign_dom_id);
		if ($order.attr('current-order') === 'desc') {
			order_desc = true;
		}
		const $state_time = $('#sup_dash_members_panel_interactions_state_filter_time_' + campaign_dom_id);
		if ($state_time.length === 0) {
			return;
		}
		filter.interactionsStateFilter = interactions_state;
		filter.orderDesc = order_desc;
		filter.agentOrder = AGENTS_ORDER_BY_INTERACTIONS_SATE_TIME;

		let state_time_min = parseInt($state_time.val().trim(), 10);
		if (isNaN(state_time_min)) {
			return;
		}
		let state_time_sec = state_time_min * 60;
		filter.interactionsStateTimeFilter = state_time_sec;
		filter.interactionsStateTimeOperatorFilter = ">=";
	};

	DashboardMembers.prototype.get_members_filters = function(campaign_id)
	{
		const config = this.m_config.get_panel_config(campaign_id, 'sup_dash_members_panel');
		var filter = {
			campaignId            : campaign_id.replace(/^(.*?)@(.*?)$/, '$1'),
			agentNameFilter       : this.m_custom_filter_by_campaign[campaign_id] || null,
			agentOrder            : AGENTS_ORDER_BY_NAME,
			page                  : this.m_page_by_campaign[campaign_id].page,
			rows_per_page         : ROWS_PER_PAGE,
			showInteractionDetails: true,
			showOfflineAgents     : config ? config.options.show_offline : false,
			showKickouts          : config ? config.options.show_kickouts : false,
			workgroups            : this.m_workgroups.get_campaign_workgroups(campaign_id)
		};
		this.__set_account_al_to_filter(campaign_id, filter);
		this.__set_member_state_to_filter(campaign_id, filter);
		this.__set_member_interactions_state_to_filter(campaign_id, filter);

		return filter;

	};

	DashboardMembers.prototype.initializePageByCampaign = function(campaign_id)
	{
		if (!this.m_page_by_campaign[campaign_id]){
			this.m_page_by_campaign[campaign_id] = {page: 0};
			// update current page.
			this.m_page_by_campaign[campaign_id].page ++;
		}
	};

	DashboardMembers.prototype.UpdateMembers = function(campaign_id, view)
	{
		if (!campaign_id || !view) {
			return;
		}
		// update members
		this.m_page_by_campaign[campaign_id].previous_agents = this.m_page_by_campaign[campaign_id].agents;
		this.m_page_by_campaign[campaign_id].agents = view.agents;
		this.m_page_by_campaign[campaign_id].agentsInQuery = view.agentsInQuery;
		this.m_page_by_campaign[campaign_id].lastPage = view.agentsInQuery < ROWS_PER_PAGE ? 1 : Math.ceil(view.agentsInQuery / ROWS_PER_PAGE);
	};

	DashboardMembers.prototype.get_members = function(campaign_id, on_search_finished, viewMembers)
	{
		const self = this;
		on_search_finished = on_search_finished || $.noop;

		this.initializePageByCampaign(campaign_id);

		const cb_get_supervisor_agents = function(view, error)
		{
			if (!view || !view.agents && error){
				on_search_finished(false, 0, 0);
				return;
			}
			
			self.UpdateMembers(campaign_id, view);

			// primero tengo que actualizar las alarmas en el popup de notificaciones y alarmas
			self.m_dashboard.notify_alarms(campaign_id, view.alarms, "REQUEST_MEMBERS_PANEL");

			// callback.
			on_search_finished(true, view.agents, view.agentsInQuery);
		};

		if (viewMembers) {
			let error;
			if (viewMembers.result && viewMembers.result !== "OK") {
				error = viewMembers;
			}
			else{
				this.m_model.setCurrentMembersByCampaign(campaign_id, viewMembers);
			}
			return cb_get_supervisor_agents(viewMembers, error);
		}
		
		const filter = this.get_members_filters(campaign_id);
		
		this.m_model.get_supervisor_agents(campaign_id, filter, cb_get_supervisor_agents);
	};

	DashboardMembers.prototype.__update_current_pagination = function(campaign_id, current_agents_in_page, total_agents)
	{
		var previous_page = this.m_page_by_campaign[campaign_id].page - 1;
		var current_page = this.m_page_by_campaign[campaign_id].page;
		var last_page = this.m_page_by_campaign[campaign_id].lastPage;

		var previous_button_disabled = current_page === 1;
		var next_button_disabled = true;

		// calculate start member counter.
		var start = null;
		if (total_agents === 0){
			start = 0;
		}
		else if (current_page === 1){
			start = 1;
		}
		else{
			start = ( ROWS_PER_PAGE * previous_page ) + 1;
		}

		// calculate finish member counter.
		var finish = null;
		if (current_page === last_page){
			finish = total_agents;
		}
		else {
			// just calculate given the current page number the last member
			finish = current_agents_in_page * current_page;
			next_button_disabled = false;
		}

		var normalized_campaign = this.m_dashboard.get_normalized_id(campaign_id);

		// update member's counters & buttons status.
		$('#team_page_text_' + normalized_campaign).text(format("<%=_T('{0} - {1} of {2} users')%>", start, finish, total_agents));
		$('#team_page_prev_' + normalized_campaign).prop('disabled', previous_button_disabled);
		$('#team_page_next_' + normalized_campaign).prop('disabled',  next_button_disabled);
	};

	DashboardMembers.prototype.__setup_pagination = function(current_node, campaign_id)
	{
		var self = this;
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		var next_button = current_node.find('#team_page_next_' + campaign_dom_id);
		var previous_button = current_node.find('#team_page_prev_' +  campaign_dom_id);

		next_button
			.attr('tooltip', "<%=_T('Next Page')%>")
			.unbind('click')
			.bind('click', function(){
				// disable until finish pagination.
				next_button.prop('disabled', true);
				previous_button.prop('disabled', true);

				self.m_state[campaign_id] = 'paginating';

				// trigger event
				self.__on_before_members_search(current_node);

				// TO-DO: This should be a parameter of get_members.
				// increment page number
				self.m_page_by_campaign[campaign_id].page++;

				// Go and get members from the server
				self.get_members(campaign_id, function(_, members, total_members_left) {
					// create members markup.
					self.__process_members_markup(current_node, members);

					// trigger event
					self.__on_after_members_search(current_node, members.length, total_members_left);
				});
			});

		previous_button
			.attr('tooltip', "<%=_T('Previous Page')%>")
			.unbind('click')
			.bind('click', function(){
				// disable until finish pagination.
				next_button.prop('disabled', true);
				previous_button.prop('disabled', true);

				// set state
				self.m_state[campaign_id] = 'paginating';

				// trigger event
				self.__on_before_members_search(current_node);

				// decrement page number
				self.m_page_by_campaign[campaign_id].page --;

				// Go and get members from the server
				self.get_members(campaign_id, function(_, members, total_members_left) {
					// create members markup.
					self.__process_members_markup(current_node, members);

					// trigger event
					self.__on_after_members_search(current_node, members.length, total_members_left);
				});
			});
	};

	// Will refresh current member's data for the current campaign node.
	DashboardMembers.prototype.refresh = function (current_node, membersView)
	{
		var self = this;
		var campaign_id = current_node.attr('campaign_id');

		const get_member_callback = function(_, members, total_members_left) {
			//console.log("----> DashboardMembers.prototype.refresh - members: " + JSON.stringify(members));
				// actualizo la informacion en modo full screen
				self.__full_screen_update({
					campaign_id: campaign_id,
					data: {total_members: total_members_left}
				});
				// handle no members from server, this is an error in most of the cases.
				// also, check if we can refresh the view, (check pagination status)
				if ( !members || !self.__allowed_to_refresh_view(campaign_id)){
					return;
				}
				// nothing to do without members, show msg and return.
				else if ( members.length === 0 ) {
	
					// this actions only apply if the previous members
					// search has members
					if ( self.m_previous_search_has_members[campaign_id] ){
						// clear member list and update button and pagination status
						self.__on_member_list_empty(current_node, campaign_id);
						self.on_member_list_reloaded(current_node, campaign_id, members.length, total_members_left);
	
						// go back to the first page
						self.m_page_by_campaign[campaign_id].page = 1;
	
						// show message to the user.
						self.__show_empty_member_list_msg(current_node, self.__get_team_list_message_to_show(campaign_id));
					}
				}
				else {
					var last_page = self.m_page_by_campaign[campaign_id].lastPage;
					var current_page = self.m_page_by_campaign[campaign_id].page;
	
					// check if we don't need to go back to the first page.
					// solves the case when a bunch of users are logged out at the same time.
					if (members.length < ROWS_PER_PAGE && current_page !== last_page){
						self.m_page_by_campaign[campaign_id].page = 1;
					}
					// if the user was in page that does not exist anymore,
					// go back to the last available page.
					if (current_page > last_page){
						self.m_page_by_campaign[campaign_id].page = last_page;
					}
						
					// this actions only apply if the last search does not
					// have members
					if ( !self.m_previous_search_has_members[campaign_id] ){
						// hide *empty list* message and enable buttons.
						self.__show_members_list(current_node);
						self.__hide_empty_member_list_msg(current_node);
						self.__enable_input_actions(current_node);
						self.__update_members_menu_options(current_node, members.length);
					}
	
					// create members markup.
					self.__process_members_markup(current_node, members, function(nodes_were_added){
						// go and bind events.
						if (nodes_were_added){
							self.on_member_list_reloaded(current_node, campaign_id, members.length, total_members_left);
						}
						else{
							// otherwise just update pagination data.
							self.__update_current_pagination(campaign_id, members.length, total_members_left);
						}
					});
				}
	
				// set the flag for the next refresh
				self.m_previous_search_has_members[campaign_id] = members.length > 0;
			};

		// go and search for members.
		this.get_members(campaign_id, get_member_callback, membersView);
	};

	/**
	 * Called every time a member list is cleared, will setup
	 * the corresponding events for the nodes, the current pagination
	 * text and button status, etc.
	 *
	 * @private
	 * @param  {object} current_node            - The current campaign node.
	 * @param  {string} campaign_id             - The campaign identificator
	 * @param  {number} total_members_in_search - Number of agents in the last search
	 * @param  {number} total_members_left      - Number of agents left in the page from the last search
	 */
	DashboardMembers.prototype.on_member_list_reloaded = function(current_node, campaign_id, total_members_in_search, total_members_left)
	{
		// Setup member selection row
		this.__setup_member_row_selection(current_node);
		// Update current page status.
		this.__update_current_pagination(campaign_id, total_members_in_search, total_members_left);
		// Setup pagination
		this.__setup_pagination(current_node, campaign_id);
	};

	DashboardMembers.prototype.__fill_state_filter = function (campaign_id)
	{
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $filter = $('#sup_dash_members_panel_state_filter_' + campaign_dom_id);

		$filter.html("");
		const first_item = "";
		$filter.append($('<option value="" selected>' + first_item + '</option>'));

		var add_state = function(state_id, state_desc) {
			var $state = $('<option></option>');
			$state.attr('value', state_id);
			$state.html(state_desc);
			$filter.append($state);
		};
		// 1 - Cargo los status del sistema
		$.each(this.m_member_states, function(_, state)
		{
			if (!state.filter.enabled) {
				return true; //continue
			}
			add_state(state.filter.state_id, state.desc);
		});
		// 2 - Cargo los status custom del vcc
		var custom_sates = inConcert.app.getCustomStates();
		$.each(custom_sates || [], function(_, state)
		{
			add_state(state.Id, state.Description);
		});
	};

	DashboardMembers.prototype.__fill_interactions_state_filter = function (campaign_id)
	{
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $filter = $('#sup_dash_members_panel_interactions_state_filter_' + campaign_dom_id);

		var add_state = function(state_id, state_desc) {
			let $state = $('<option></option>');
			$state.attr('value', state_id);
			$state.html(state_desc);
			$filter.append($state);
		};
		let channels;
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (campaign) {
			channels = campaign.get_channels();
		}
		var can_add = function(state_info) {
			if (!channels) {
				return false;
			}
			const allowed_channels = state_info.filter.allowed_channels;
			if (allowed_channels === "ALL") {
				return true;
			}
			let is_allowed_state_in_channel = false;
			for (let channel in channels) {
				if (channels.hasOwnProperty(channel)) {
					is_allowed_state_in_channel = allowed_channels[channel];
					if (is_allowed_state_in_channel) {
						break;
					}
				}
			}
			return is_allowed_state_in_channel;
		};

		$filter.html("");
		const first_item = "";
		$filter.append($('<option value="" selected>' + first_item + '</option>'));

		// Cargo los estados de las interacciones
		$.each(this.m_interactions_states, function(_, state_info) {
			if (!can_add(state_info)) {
				return true; //continue
			}
			add_state(state_info.filter.state_id, state_info.desc);
		});
	};

	/**
	 * Entry point function, will setup the entire member's column.
	 *
	 * @todo - Add a callback to this function
	 *
	 * @public
	 * @param  {[type]} $campaign - Current campaign node
	 */
	DashboardMembers.prototype.setup = function ($campaign, membersView)
	{
		const self = this;
		let campaign_id = $campaign.attr('campaign_id');
		let config = this.m_config.get_panel_config(campaign_id, 'sup_dash_members_panel');

		// Setup filter states
		this.__fill_state_filter(campaign_id);

		// Setup filter interactions states
		this.__fill_interactions_state_filter(campaign_id);

		// Setup Agent Statuses popup
		this.__setup_agent_status_popup($campaign);

		// Setup menu buttons.
		this.__set_events($campaign);

		// update menu options
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		this.__update_previous_config_menu_options(campaign_dom_id, config);

		// set current default members page if not exists for this campaign.
		if (!this.m_page_by_campaign[campaign_id]){
			this.m_page_by_campaign[campaign_id] = {page: 0};
		}
		// update current page.
		this.m_page_by_campaign[campaign_id].page ++;

		// trigger event
		this.__on_before_members_search($campaign);

		// go and get members
		this.get_members(campaign_id, function(_, members, total_members_left) {
			// If there are no members for this campaign,
			// show a message to the user and disable all members actions.
			if ( !members || members.length === 0 ) {
				self.__on_member_list_empty($campaign, campaign_id);
			}
			else {
				// hide message.
				self.__hide_empty_member_list_msg($campaign);
			}
			// create members markup.
			self.__process_members_markup($campaign, members);

			// search buttons handlers
			self.__setup_members_search($campaign);

			// setup broadcast message button.
			self.__setup_broadcast_button_handlers($campaign);

			// Based on panel previous config, configure team list.
			self.__update_previous_config_team_list($campaign, config, members);

			// trigger event
			self.__on_after_members_search($campaign, members.length, total_members_left);

			// actualizo la informacion en modo full screen
			self.__full_screen_update({
				campaign_id: campaign_id,
				data: {total_members: total_members_left}
			});
		}, membersView);
	};

	DashboardMembers.prototype.__full_screen_update = function (info)
	{
		return this.m_dashboard.full_screen_update_by_members(info);
	};
}());
