/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, Mustache, PubSub */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	/**
	 * The entry point for the application
	 *
	 * @author Sebastián Moreno (smoreno@inconcertcc.com)
	 * @constructor
	 * @memberOf Supervisor
	 */
	function Supervisor()
	{
		this.$supervisor = $('#supervisor_main');
		this.already_initialized = false;
		this.m_templates = inConcert.sup_templates;
		this.m_current_view = 'dashboard';
		this.m_dashboard = new inConcert.sup_dashboard();
	}

	if (typeof inConcert.SupervisorApp !== 'function') {
		inConcert.SupervisorApp = Supervisor;
		inConcert.SupervisorMessageBus = PubSub;
	}

	Supervisor.prototype.__show_waiting = function() {
		$('#sup_dash_main_waiting').show();
	};

	Supervisor.prototype.__hide_waiting = function() {
		$('#sup_dash_main_waiting').hide();
	};

	Supervisor.prototype.__waiting_is_visible = function() {
		return $('#sup_dash_main_waiting').is(':visible');
	};

	Supervisor.prototype.__update_waiting_message = function(message) {
		var $waiting = $('#sup_dash_main_waiting');
		$waiting.find('[waiting_message]').text(message || '');
	};

	Supervisor.prototype.__clear_waiting_message = function() {
		this.__update_waiting_message();
	};

	Supervisor.prototype.__refresh_campaign = function()
	{
		var self = this;
		// return if someone is already refreshing/updating the dashboard
		if (this.__waiting_is_visible()){
			return;
		}
		// show waiting and add selected button class
		this.__show_waiting();
		// Set timeout and do not freeze ui.
		switch( this.m_current_view ) {
			case 'dashboard':
				this.__update_waiting_message("<%=_T('Refreshing campaign dashboard')%>");
				if ( !this.m_dashboard.is_refresh_alive() ){
					this.m_dashboard.start_refresh_timer();
				}
				this.m_dashboard.refresh_bots_dashboard();
				// because the refresh actually do not refresh anything
				// (just check if timers are on and if not, he turn them on)
				// the waiting is almost invisible, so we mimic a *heavy refresh operation*
				window.setTimeout(function(){
					self.__hide_waiting();
				}, 1000);
				break;
			case 'campaign':
			case 'settings':
			case 'overview':
				// TO-DO
				break;
			default:
				break;
		}
	};

	Supervisor.prototype.setup_refresh = function()
	{
		var self = this;
		//var $refresh_button = $('#sup_title_container .supervisorRefreshButton');
		var $refresh_button = $('#sup_toolbar_options .supervisorRefreshButtonNew');
		// Refresh Button
		$refresh_button.unbind('click').bind('click', function() {
			self.__refresh_campaign();
		});
	};

	Supervisor.prototype.setup_campaigns_selector = function()
	{
		const self = this;
		const $campaign_selector = $('#sup_toolbar_options [action_id="campaign_selector"]');
		$campaign_selector
			.unbind('click')
			.bind('click', function() {
				self.m_dashboard.toggle_campaign_selector();
			});
	};

	Supervisor.prototype.initialize_sidebar = function()
	{
		this.setup_refresh();
		this.setup_campaigns_selector();
	};

	Supervisor.prototype.show_current_view = function(callback = $.noop)
	{
		const self = this;
		switch(this.m_current_view)
		{
			case 'dashboard':
				if ( !this.m_dashboard ){
					this.m_dashboard = new inConcert.sup_dashboard();
				}
				this.m_dashboard.render(function(_ /*success*/){
					self.__hide_waiting();
					self.__clear_waiting_message();
					callback();
				});
				break;
			case 'campaign':
			case 'settings':
			case 'overview':
				// TO-DO
				callback();
				break;
			default:
				callback();
				break;
		}
	};

	Supervisor.prototype.create = function()
	{
		if (this.already_initialized) {
			return;
		}
		const main_view = {
			dashboard_tooltip     : "<%=_T('Dashboard')%>",
			refresh_tooltip       : "<%=_T('Refresh')%>",
			campaign_tooltip      : "<%=_T('Campaign Selector')%>",
			alarms_tooltip        : "<%=_T('Notifications')%>",
			config_tooltip        : "<%=_T('Update agents')%>",
			menu_tooltip          : "<%=_T('View navegation menu')%>",
			menu_config_view_tooltip        : "<%=_T('View configurations menu')%>"                ,
			detail_tooltip        : "<%=_T('Detail')%>",
			full_screen_tooltip   : "<%=_T('Full Screen')%>",
			simple_screen_tooltip : "<%=_T('Normal Screen')%>",
			back                  : "<%=_T('Back')%>",
			dashboard_bots_tooltip: "<%=_T('Bots')%>",
			filter_tooltip		  : "<%=_T('Filter')%>" 
		};
		this.$supervisor.append(Mustache.render(this.m_templates.main, main_view));
		// set events
		this.__set_events();
		// inicio los tooltips
		this.__init_tooltips();
		this.already_initialized = true;
	};

	Supervisor.prototype.__init_tooltips = function()
	{
		const $sup_container = $("#sup_container");
		// update tooltip position
		$(".supervisorOptionsItem").inconcert_tooltip({alignment: "right-bottom"});
		// tooltips de los full screen header buttons
		const $full_screen_header_buttons = $sup_container.find("[full_screen_header_buttons]");
		$full_screen_header_buttons.inconcert_tooltip({alignment: "left-bottom", bind_selector: "[tooltip]"});
	};

	Supervisor.prototype.__set_events = function()
	{
		const self = this;

		const $sup_container = $("#sup_container");
		$sup_container
			.off("scroll")
			.on("scroll", function() {
				if (self.m_current_view === "dashboard" && self.m_dashboard) {
					self.m_dashboard.hideMenuActions();

					/*	Si esta en la vista minimizada de campannas y se mueve el scroll,
						Al refrescar no se ejecuta la funcion update_summary (Actualiza los Paneles "Summary" y "dashboard)
						Esto pq genera pausas en el scroll.
						No es hasta q se stopea el scroll que se actualiza, cuando refresque cada 3 seg.
					*/
					if (!(self.m_dashboard.m_full_screen && self.m_dashboard.m_full_screen.is_full_screen())) {
						self.m_dashboard.m_dash_container_scroll_is_moving = true;
					}
				}
				// Se publica el mensaje on_scroll para que todas las clases que herdan de la clase "inConcert.PopupWindow"
				//  reciban el mismo y los popups se cierren.
				inConcert.app.messagebus.publish("/application/popup_window/on_scroll");

				if (self.m_dashboard.m_dash_container_scroll_is_moving) {
					if (self.scroll_timeout) {
						clearTimeout(self.scroll_timeout);
					}
					self.scroll_timeout = setTimeout(function() {
						if (self.m_dashboard) {
							self.m_dashboard.m_dash_container_scroll_is_moving = false;
						}
					  }, 100);
				}
				
			});

//window.setTimeout(function() {
		var event_selector = "[sup_full_screen_header_container] [action_id]";
		$sup_container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				let action_id = $(this).attr("action_id");

				if (action_id === "campaign_detail") {
					let $btn = $(this);
					self.m_dashboard.full_screen_toggle_campaign_detail($btn);
				}
				else if (action_id === "campaign_menu") {
					self.m_dashboard.full_screen_toggle_campaign_menu();
				}
				else if (action_id === "campaign_menu_config_view") {
					self.m_dashboard.full_screen_toggle_campaign_menu_config_view();
				}
				else if (action_id === "campaign_refresh") {
					self.__refresh_campaign();
				}
				else if (action_id === "campaign_selector") {
					self.m_dashboard.toggle_campaign_selector();
				}
				else if (action_id === "campaign_alarms") {
					self.m_dashboard.toggle_alarms_notifications();
				}
				else if (action_id === "campaign_back_dashboard") {
					self.m_dashboard.full_screen_back_dashboard();
				}
				else if (action_id === "campaign_bots_filter") {
					if (self.m_dashboard.m_full_screen) {
						self.m_dashboard.m_full_screen.show_campaign_bots_filters();
					}
				}
				else if ( action_id === "campaign_accounts") {
					if (self.m_dashboard.m_full_screen) {
						self.m_dashboard.m_full_screen.show_campaign_accounts();
					}
				}
			});

		const $control_panel = $('[action_id="control_panel"]');
		$control_panel
				.unbind('click')
				.bind('click', function() {
					if (!$(this).prop("disabled") && self.m_dashboard.m_full_screen) {
						self.m_dashboard.m_full_screen.show_control_panel();
					}
				});

		const $campaign_hide_full_screen = $('[action_id="campaign_hide_full_screen"]');
		$campaign_hide_full_screen
			.unbind('click')
			.bind('click', function () {
				if ($(this).prop("disabled")) {
					return;
				}
				self.m_dashboard.full_screen_toggle_visibility({full_screen: false});
			});
//}, 5000);
	};

	Supervisor.prototype.on_leave = function() {
		this.m_dashboard.on_leave();
	};

	Supervisor.prototype.get_dashboard = function() {
		return this.m_dashboard;
	};

	Supervisor.prototype.render = function(callback = $.noop)
	{
		const self = this;
		if (!this.already_initialized) {
			this.create();
			this.__show_waiting();
			// first view, create dashboard
			this.m_current_view = 'dashboard';
		}
		this.$supervisor.show(1, 'swing', function() {
			self.initialize_sidebar();
			self.show_current_view(callback);
			return true;
		});
	};
}());
