/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const format = inConcert.Utils.format;

	
//------------------------------------------------------
	// web_designer navigation config
	var webdesigner_config = function()
	{
		var get_message_error = function(type, err, name)
		{
			var err_messages;
			var message;
			if (type === "delete") {
				err_messages = {
					"FORBIDDEN"                                     : "<%=_T('Unable to delete {0} project. You dont have permission.')%>",
					"TWEETY_PROJECT_IN_USE_ON_CONTACTFORM_CAMPAIGN" : "<%=_T('The contact form project {0} to delete is currently in use. You must release it before deletion.')%>",
					"TWEETY_PROJECT_IN_USE_ON_CCI_CAMPAIGN"         : "<%=_T('The CCI project {0} to delete is currently in use. You must release it before deletion.')%>",
					"TWEETY_PROJECT_IN_USE_ON_WEBCHAT_CAMPAIGN"     : "<%=_T('The web chat project {0} to delete is currently in use. You must release it before deletion.')%>",
					"TWEETY_PROJECT_IN_USE_ON_APPS_LANUCHER"        : "<%=_T('The generic project {0} to delete is currently in use. You must release it before deletion.')%>",
					"OPEN"                                          : "<%=_T('Unable to delete project.  {0} is opened.')%>"
				};
				if (typeof err === "object" && typeof err.result === "string") {
					err = err.result;
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} project.')%>";
				message = format(message, name);
			}
			else if (type === "edit") {
				err_messages = {
					"FORBIDDEN" : "<%=_T('Unable to edit {0} project. You dont have permission.')%>"
				};
				if (typeof err === "object" && typeof err.result === "string") {
					err = err.result;
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} project.')%>";
				message = format(message, name);
			}
			return message;
		};

		var web_api = inConcert.app.getTweetyAPI();
		// execute custom action
		var execute_custom_action = function( action_id, $row, view )
		{
			var project_id = $row.attr( "row_id" );
			var display_name = $row.attr("row_name");
			switch ( action_id ) {
				case "clone":
					if ( !view.form.has_permission_to_save() ) {
						view.show_status_message(
							format( "<%=_T('Unable to clone {0} project. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					var popup_params = {
						type: "clone",
						source_id: project_id
					};

					view.render_popup( popup_params );
					break;

				case "export":
					if ( !view.form.has_permission_to_save() ) {
						view.show_status_message(
							format( "<%=_T('Unable to export {0} project. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					web_api["export"]( project_id );
					break;
			}
		};

		// is action enabled
		var is_action_enabled = function( action /*, row */ )
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			switch ( action.toLowerCase() ) 
			{
				case "clone"  :
				case "edit"   :
				case "export" :
				if ( !inConcert.permissions.to_action( "web_designer" , "web_designer_save", "SAVE")) {
					return -1;
				}
				break;
			}
			return 1;
		};

		return {
			id: "web_designer",
			name: "<%=_T('Web Designer')%>",
			permissions: {
				view  : "web_designer_view",
				save  : "web_designer_save",
				erase : "web_designer_delete",
				permission_type : "vcc",
				verify_edit_permission : true
			},
			class_name: "itemTweety",
			i_class_name: "far fa-code",
			view: {
				render: "/inconcert/apps/web_designer/pages/view_web_designer_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Web Designer')%>",
				tagline: "<%=_T('Tagline for Projects')%>",
				new_button_name: "<%=_T('New Project')%>",
				row_id: "id",
				display_name: "id",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				messages: {
					erase: "<%=_T('Do you want to delete project {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  function( project_id, _, callback )
					{
						var can_erase = true;

						/*
					// TODO: de momento se quita la validacion de que un proyecto se encuentre abierto.
						var popups_windows = _.dashboard.get_popups_windows();
						if ( !$.isEmptyObject( popups_windows ) )
						{
							$.each( popups_windows || {}, function( id, popup )
							{
								if ( typeof popup.conf === "object" &&
									typeof popup.conf.project_id === "string" )
								{
									if ( popup.conf.project_id.toLowerCase() === project_id.toLowerCase() ) {
										var err = { result: "OPEN" };
										callback( false, err );
										can_erase = false;
										return false; // break
									}
								}
							});
						}
						*/
						if ( can_erase ) {
							return web_api.erase( project_id, callback );
						}
					},
					search: function( callback, filter ) {
						var query;
						if ( typeof filter === "object" && filter.query ) {
							query = JSON.stringify( filter.query );
						}
						return web_api.load_all_projects( callback, query );
					}
				},
				source: ["id", "type"],
				columns: [
					{ id: "_number"    , name: "", type: "counter" },
					{ id: "id"         , name: "<%=_T('Id')%>"     },
					{ id: "type"       , name: "<%=_T('Type')%>",
						get_value: function( project ) {
							var type = project.type || "";
							var display_type;
							switch ( type.toLowerCase() )
							{
								case "contactform":
									display_type = "<%=_T('Front End Integration - Contact Form')%>";
								break;
								case "cci":
									display_type = "<%=_T('Contact Center Integration')%>";
								break;
								case "chatcampaign":
									display_type = "<%=_T('Front End Integration - Web Chat')%>";
								break;
								case "webcallcampaign":
									display_type = "<%=_T('Front End Integration - Web Call')%>";
								break;
								case "survey_form":
									display_type = "<%=_T('Survey Form')%>";
								break;
								case "generic":
									display_type = "<%=_T('Generic')%>";
								break;
								default:
									display_type = type;
							}
							return {
								original_value: type,
								value: display_type
							};
						}
					},
					{ id: "_action"    , name: "" ,
						class_name: "gridColumnFourActions",
						system_action: { edit: true, erase: true },
						custom_action: {
							clone: {
								name: "<%=_T('Clone')%>",
								execute: execute_custom_action,
								class_name: "clone"
							},
							"export": {
								name: "<%=_T('Export')%>",
								execute: execute_custom_action,
								class_name: "export"
							}
						},
						is_enabled: is_action_enabled
					}
				],
				filter: {fields: {"id": {control: null}, "type": {control: null}}},
				realtime: null,
				must_init_full_screen: ()=> true
			},
			form: {
				instance : inConcert.adm_form_tweety
			}
		};
	};

	//------------------------------------------------------
	// get all navigation config
	const get_navigation_config = function()
	{
		return {
            web_designer                 : webdesigner_config(),
		};
	};

	function web_designer_config() {}

	if (typeof inConcert.web_designer_config !== "function") {
		inConcert.web_designer_config = web_designer_config;
	}

	web_designer_config.prototype.get_app_config = function()
	{
		return {
			id         : "web_designer",
			name       : "<%=_T('Web Designer')%>",
			selected   : false,
			class_name : "appAdministration",
			main_html  : "/inconcert/apps/web_designer/pages/conf_web_designer_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			allow_disable_all_navigation : true,
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category("web_designer", "view");
			},
			css : [
				{ id: "adm_messages_css"                            , href: "/inconcert/apps/administration/css/administrationRules.css#grunt-cache-bust"                       },
                { id: "skin_occ_internal_forms_css"                 , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"          },
                { id: "skin_occ_adm_messenger_css"                 	, href: "/inconcert/apps/administration/css/skin-2021/administrationPageMessenger.css#grunt-cache-bust"     },
                { id: "skin_occ_master_buttons_css"                 , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                            },
				{ id: "skin_occ_inbox_master_css"                   , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                              },
                { id: "skin_occ_messages_tooltips_css"              , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"               },
				{ id: "skin_occ_adm_master_css"                     , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"            },
				{ id: "skin_occ_inbox_master_popups_css"            , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                        },
				{ id: "skin_occ_master_client_window_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                       },
				{ id: "skin_occ_custom_visual_search_css"           , href: "/inconcert/apps/agent/css/skin-2021/customVisualSearch.css#grunt-cache-bust"                       }
			]
		};
	};
}());
