/*
 jQuery.ScrollTo
 Copyright (c) 2007-2013 Ariel Flesler - aflesler<a>gmail<d>com | http://flesler.blogspot.com
 Dual licensed under MIT and GPL.

 @projectDescription Easy element scrolling using jQuery.
 http://flesler.blogspot.com/2007/10/jqueryscrollto.html
 @author Ariel Flesler
 @version 1.4.7

 @id jQuery.scrollTo
 @id jQuery.fn.scrollTo
 @param {String, Number, DOMElement, jQuery, Object} target Where to scroll the matched elements.
   The different options for target are:
  - A number position (will be applied to all axes).
  - A string position ('44', '100px', '+=90', etc ) will be applied to all axes
  - A jQuery/DOM element ( logically, child of the element to scroll )
  - A string selector, that will be relative to the element to scroll ( 'li:eq(2)', etc )
  - A hash { top:x, left:y }, x and y can be any kind of number/string like above.
  - A percentage of the container's dimension/s, for example: 50% to go to the middle.
  - The string 'max' for go-to-end.
 @param {Number, Function} duration The OVERALL length of the animation, this argument can be the settings object instead.
 @param {Object,Function} settings Optional set of settings or the onAfter callback.
  @option {String} axis Which axis must be scrolled, use 'x', 'y', 'xy' or 'yx'.
  @option {Number, Function} duration The OVERALL length of the animation.
  @option {Boolean} interrupt If true, the scrolling animation will stop on user scroll
  @option {String} easing The easing method for the animation.
  @option {Boolean} margin If true, the margin of the target element will be deducted from the final position.
  @option {Object, Number} offset Add/deduct from the end position. One number for both axes or { top:x, left:y }.
  @option {Object, Number} over Add/deduct the height/width multiplied by 'over', can be { top:x, left:y } when using both axes.
  @option {Boolean} queue If true, and both axis are given, the 2nd axis will only be animated after the first one ends.
  @option {Function} onAfter Function to be called after the scrolling ends.
  @option {Function} onAfterFirst If queuing is activated, this function will be called after the first scrolling ends.
 @return {jQuery} Returns the same jQuery object, for chaining.

 @desc Scroll to a fixed position
 @example $('#container').scrollTo( 340 );

 @desc Scroll relatively to the actual position
 @example $('#container').scrollTo( '+=340px', { axis:'y' } );

 @desc Scroll using a selector (relative to the scrolled element)
 @example $(window).scrollTo( 'p.paragraph:eq(2)', 500, { easing:'swing', queue:true, axis:'xy' } );

 @desc Scroll to a DOM element (same for jQuery object)
 @example $(window).scrollTo( document.getElementById('element'), { duration:500, axis:'x', onAfter:function() {
    alert('scrolled!!');
   }});

 @desc Scroll on both axes, to different values
 @example $('div').scrollTo( { top: 300, left:'+=200' }, { axis:'xy', offset:-20 } );
*/
(function(g){function q(b){return"object"==typeof b?b:{top:b,left:b}}var l=g.scrollTo=function(b,f,a){return g(window).scrollTo(b,f,a)};l.defaults={axis:"xy",duration:1.3<=parseFloat(g.fn.jquery)?0:1,limit:!0};l.window=function(b){return g(window)._scrollable()};g.fn._scrollable=function(){return this.map(function(){if(this.nodeName&&-1==g.inArray(this.nodeName.toLowerCase(),["iframe","#document","html","body"]))return this;var b=(this.contentWindow||this).document||this.ownerDocument||this;return/webkit/i.test(navigator.userAgent)||
"BackCompat"==b.compatMode?b.body:b.documentElement})};g.fn.scrollTo=function(b,f,a){"object"==typeof f&&(a=f,f=0);"function"==typeof a&&(a={onAfter:a});"max"==b&&(b=9E9);a=g.extend({},l.defaults,a);f=f||a.duration;a.queue=a.queue&&1<a.axis.length;a.queue&&(f/=2);a.offset=q(a.offset);a.over=q(a.over);return this._scrollable().each(function(){function m(n){r.animate(c,f,a.easing,n&&function(){n.call(this,e,a)})}if(null!=b){var t=this,r=g(t),e=b,c={},x=r.is("html,body");switch(typeof e){case "number":case "string":if(/^([+-]=?)?\d+(\.\d+)?(px|%)?$/.test(e)){e=
q(e);break}e=g(e,this);if(!e.length)return;case "object":if(e.is||e.style)var u=(e=g(e)).offset()}g.each(a.axis.split(""),function(n,h){var p="x"==h?"Left":"Top",k=p.toLowerCase(),d="scroll"+p,v=t[d],w=l.max(t,h);u?(c[d]=u[k]+(x?0:v-r.offset()[k]),a.margin&&(c[d]-=parseInt(e.css("margin"+p))||0,c[d]-=parseInt(e.css("border"+p+"Width"))||0),c[d]+=a.offset[k]||0,a.over[k]&&(c[d]+=e["x"==h?"width":"height"]()*a.over[k])):(h=e[k],c[d]=h.slice&&"%"==h.slice(-1)?parseFloat(h)/100*w:h);a.limit&&/^\d+$/.test(c[d])&&
(c[d]=0>=c[d]?0:Math.min(c[d],w));!n&&a.queue&&(v!=c[d]&&m(a.onAfterFirst),delete c[d])});m(a.onAfter)}}).end()};l.max=function(b,f){var a="x"==f?"Width":"Height";f="scroll"+a;if(!g(b).is("html,body"))return b[f]-g(b)[a.toLowerCase()]();a="client"+a;var m=b.ownerDocument.documentElement;b=b.ownerDocument.body;return Math.max(m[f],b[f])-Math.min(m[a],b[a])}})(jQuery);
