import {
    ICCBAMLIBOVERLAY_CLASS,
    ICCBAMLIBEVENT_RESPONSE_TIMEOUT,
    ICCBAMLIBEVENT_REQUEST,
    ICCBAMLIBEVENT_RESPONSE
} from './Constants';
import ReactDOM  from 'react-dom/client';
import React, { useState } from 'react';
import ExtensionInstaller from './ExtensionInstaller';
import {setInterval,clearInterval,setTimeout,clearTimeout} from './utils';

export class ExtensionNotFoundError extends Error {
    constructor(){
        super("Extension not found");
    }
}


export class ExtensionProxy {
    #domNode // div of class ICCBAMLIBOVERLAY_CLASS inserted by a extension in all pages
    #eventResponse;
    constructor(extensionDOMNode) {
        this.#domNode = extensionDOMNode;
        this.#domNode.addEventListener(ICCBAMLIBEVENT_RESPONSE, (e) => {
            this.#eventResponse = e.detail;
        })
    }
    async #sendMessage(msgData) {
        let event = new CustomEvent(ICCBAMLIBEVENT_REQUEST, { detail: msgData });
        this.#domNode.dispatchEvent(event);
    }
    getResponse(tmout) {
        this.#eventResponse = undefined;
        return new Promise((resolve, reject) => {
            if (this.#eventResponse) {
                resolve(this.#eventResponse);
            } else {
                let eventResponseTimeout = setTimeout(() => {
                    clearTimeout(eventResponseTimeout);
                    clearInterval(eventResponsePoolHandler);
                    reject(new Error('response timeout!'));
                }, tmout)
                let eventResponsePoolHandler = setInterval(() => {
                    if (this.#eventResponse !== undefined) {
                        clearTimeout(eventResponseTimeout);
                        clearInterval(eventResponsePoolHandler);
                        resolve(this.#eventResponse);
                    }
                }, 100);
            }
        })
    }

    async registerRole(rolename) {
        this.#sendMessage({
            msgid: 'register-role',
            payload: { rolename: rolename }
        });
    }

    async getSystemMetrics() {
        this.#sendMessage({ msgid: 'get-system-metrics' });
        return this.getResponse(ICCBAMLIBEVENT_RESPONSE_TIMEOUT);
    }
    async getSystemEvents(opts) {
        this.#sendMessage({ msgid: 'get-system-events' , payload: opts});
        return this.getResponse(ICCBAMLIBEVENT_RESPONSE_TIMEOUT);
    }

    
    static getextensionDOMNode() {
        let domElement = document.getElementsByClassName(ICCBAMLIBOVERLAY_CLASS)[0];
        return domElement !== null && domElement !== undefined ? domElement : null;
        
    }

    static waitForExtension(tout) {
        return new Promise((resolve, reject) => {
            let timeoutHandler = setTimeout(() => {
                clearInterval(checkDOMInterval);
                clearTimeout(timeoutHandler);
                reject(new ExtensionNotFoundError());
            }, tout);
            let checkDOMInterval = setInterval(() => {
                let eNode = this.getextensionDOMNode();
                if (eNode) {
                    clearInterval(checkDOMInterval);
                    clearTimeout(timeoutHandler);
                    resolve(new this(eNode));
                }
            }, 100);
        });
    }
    static attemptToInstall(){
        let rootNode = document.getElementById("bamlib_root_setup");
        let checkInstallInterval = 0;
        let finishInstallation = ()=>{
           
            rootNode.parentNode.removeChild(rootNode)
            clearInterval(checkInstallInterval);
        }
        let userCancelled = ()=>{
            console.log("The user has canceled the installation of the extension")
            finishInstallation();
        }


        if(!rootNode){
            rootNode = document.createElement("div");
            rootNode.id = "bamlib_root_setup";
            document.body.appendChild(rootNode);

        }
        const root = ReactDOM.createRoot(rootNode);
        root.render(<ExtensionInstaller onUserCancelled = {userCancelled} />);
        checkInstallInterval = setInterval(()=>{
            ExtensionProxy.waitForExtension(500)
            .then(()=>{
                console.log("The installation of the extension is complete.")
                finishInstallation()
            })
            .catch(()=>{

            })
        },1000);

    }

}
