--[[
	reCAPTCHA V2:
	-------------
	Las claves pública y privada se crearon en el link de abajo,
	y se crearon con el usuario de gmail de inconcert:

		Usr: inconcertmcc@gmail.com
		Pwd: Inc0nc3rt!

		Clave del sitio:
		----------------
		(client side) Usada en el código HTML que se muestra a los usuarios.
			6Lf-J0UUAAAAAGmR_IKYvcsFhjsu9v_AXeTnsg87

		Clave secreta  :
		----------------
		(server side) Usada para las comunicaciones entre tu sitio y Google. No revelársela a nadie.
			6Lf-J0UUAAAAAFFGy0pjfQgF0tZRtaGYOc2NuWdw

	Quedó desactivada la validacion de dominios y subdominios, según la documentación no es recomendable desactivarlos pero bueno.
	Los dominios de prueba que había configurado eran: 
		i6.inconcert.cc        (este utilizado desde el designer en modo diseño)
		chat1-i6.inconcert.cc  (este utilizado desde el designer en modo runtime)

	Referencias:
		https://developers.google.com/recaptcha/docs/display#AJAX
		https://developers.google.com/recaptcha/docs/faq
		https://www.google.com/recaptcha/admin (aca se crea o administran las claves publica y privada)
		https://www.google.com/recaptcha/admin#site/340076542?setup
		https://developers.google.com/recaptcha/docs/domain_validation (documentacion de los dominios y subdominios)
]]--

module(..., package.seeall)

--[[
	ATENCION:
		Se utiliza "ssl.https" en lugar "socket.http" para poder realizar request "https",
		ya que la versión 2.0 del API CAPTCHA, solo soporta "https"
]]--
local synchronize = require "Synchronize"
local https = synchronize("luasec_mutex", function()
    return require "ssl.https"
end)

local m_verifyUriV2 = "https://www.google.com/recaptcha/api/siteverify"
--local "secretKey" = "6Lf-J0UUAAAAAFFGy0pjfQgF0tZRtaGYOc2NuWdw"

--[[
RESPONSE BODY:
--------------
OK:
 {
  "success": true,
  "challenge_ts": "2018-02-07T13:50:30Z",
  "hostname": "chat1-i6.inconcert.cc"
}

ERROR:
{
  "success": false,
  "error-codes": [
    "invalid-input-secret"
  ]
}

]]--
local function Post (url, body, headers, cookie)
	headers = headers or {}

	LogInfo ("[RECAPTCHA:Post] HTTP Post invoqued '%s'\nbody: '%s'\nheaders: '%s'\ncookie: '%s'", url, body, json.encode(headers), cookie)

	headers["content-length"] = string.len(body)

	if cookie and string.len(cookie) then
		headers["Cookie"] = cookie
	end

	local responseBody = {}
	local result, code, responeHeaders, statusLine = https.request{
		url = url,
		method = "POST",
		headers = headers,
		source = ltn12.source.string(body),
		sink = ltn12.sink.table(responseBody)
	}

	local encodedResponseBody = table.concat(responseBody, "")

	LogInfo("[RECAPTCHA:Post] HTTP Post result: '%s',\ncode: '%s',\nresponeHeaders: '%s',\nstatusLine: '%s',\nresponseBody: '%s'", result, code, json.encode(responeHeaders or {}), statusLine, encodedResponseBody)

	code = tonumber(code)
	if code == 200 then
		return true, encodedResponseBody
	end

	LogWarning("[RECAPTCHA:Post] error getting url %s  result code: %s\nbody:%s", url, tostring(code), encodedResponseBody)

	return false, encodedResponseBody, code
end

local function MakePostBody (reCaptchaToken, clientIP, secretKey)
	local body = {
		"secret=" .. tostring(secretKey),
		"response=" .. tostring(reCaptchaToken)
    }
    if clientIP then
        body = body .. "remoteip=" .. clientIP
    end
    local concatedBody = table.concat(body, "&")
	LogDebug("[RECAPTCHA:MakePostBody] body: '%s'.", concatedBody)
    return concatedBody
end

--[[
    https://developers.google.com/recaptcha/docs/verify
        missing-input-secret	The secret parameter is missing.
        invalid-input-secret	The secret parameter is invalid or malformed.
        missing-input-response	The response parameter is missing.
        invalid-input-response	The response parameter is invalid or malformed.
        bad-request	The request is invalid or malformed.
        timeout-or-duplicate	The response is no longer valid: either is too old or has been used previously.
]]--
local function GetError (result)
	if type(result) ~= "table" then
		return "Unknow Captcha Error"
    end

    local errors = result["error-codes"]
    local firstCodeErr = ""
	if type(errors) == "table" then
		firstCodeErr = errors[1]
    end
    if not firstCodeErr then
        if type(errors) == "string" then
            firstCodeErr = errors
        end
    end

    local captchaErrors = {
        ["missing-input-secret"]   = "The secret parameter is missing.",
        ["invalid-input-secret"]   = "The secret parameter is invalid or malformed.",
        ["missing-input-response"] = "The response parameter is missing.",
        ["invalid-input-response"] = "The response parameter is invalid or malformed.",
        ["bad-request"]            = "The request is invalid or malformed.",
        ["timeout-or-duplicate"]   = "The response is no longer valid: either is too old or has been used previously."
    }
    return captchaErrors[firstCodeErr] or "Unknow Captcha Error"
end

function ValidateCaptcha (reCaptchaToken, clientIP, secretKey)
    LogInfo("[RECAPTCHA:ValidateCaptcha] Validating reCaptcha. ReCaptcha Token '%s', clientIP: '%s', Secret Key: '%s'.", reCaptchaToken, clientIP, secretKey)

	local body = MakePostBody(reCaptchaToken, clientIP, secretKey)
	local status, encodedResult, code = Post(m_verifyUriV2, body, {["Content-Type"] = "application/x-www-form-urlencoded"})
	local result = json.decode(encodedResult)
	result.code = code

	if status then
        if result.success then
            LogInfo("[RECAPTCHA:ValidateCaptcha] Success reCaptcha. ReCaptcha Token '%s', secretKey: '%s'", reCaptchaToken, secretKey)
			return true
		else
			local error = GetError(result)
            LogError("[RECAPTCHA:ValidateCaptcha] Error Validating reCaptcha. ReCaptcha Token '%s', secretKey: '%s'\nErr: '%s'.", reCaptchaToken, secretKey, error)
			return false, result
		end
	end

    if code == 403 then
        result.reason = "forbidden"
        LogError("[RECAPTCHA:ValidateCaptcha] Error Validating reCaptcha. ReCaptcha Token '%s', secretKey: '%s'\nErr: '%s'.",reCaptchaToken, secretKey, json.encode(result))
        return false, result
    end

    result.reason = "Network error"
    LogError("[RECAPTCHA:ValidateCaptcha] Error Validating reCaptcha. ReCaptcha Token '%s', secretKey: '%s'\nErr: '%s'.", reCaptchaToken, secretKey, json.encode(result))

    return false, result
end
