/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:single, -W014, esversion: 11 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	function ConversationsNavigation (params)
	{
		params = params || {};
		this.callbacks = params.callbacks || {};
		this.permissions = params.permissions;

		if ( typeof params.container_id !== 'string' ) {
			throw 'ConversationsNavigation: params.container_id is required!!';
		}

		this.$container = $('#' + params.container_id);
		this.parent = params.parent;
		this.from_app = params.from_app;
		this.pubsub_labels_handle = {};
		this.filtered_labels = {};
		this.selected_labels = {};
		this.campaigns = {};
		this.m_disable_navigation = false;
		this.system_folders = this.load_system_folders();
		this.update_campaigns(params.campaigns);

		// --------------------------------------------------------
		// instancia de global search
		this.global_search = null;
		if (typeof params.global_search_container_id === 'string') {
			this.global_search = new inConcert.GlobalSearch(
			{
				queryBuilderObj       : params.queryBuilderObj,
				from_app              : this.from_app,
				container_id          : params.global_search_container_id,
				navigation            : this, // el parent ( quien me ha instanciado )
				filter_by_campaign    : params.global_search_filter_by_campaign,
				columns_sort          : params.global_search_columns_sort || {},
				interactions_per_page : 25,
				last_control          : true,
				callbacks             : {}, // TODO: implementar ?
				permissions : { 
					global_search: (this.permissions || {}).global_search // el permiso de la navegacion, segun app
				}
			});
		}
		this.search_tickets = null;
		/*if (typeof params.search_tickets_container_id === 'string') {
			this.search_tickets = new inConcert.GlobalSearch(
			{
				queryBuilderObj       : params.queryBuilderObj,
				from_app              : this.from_app,
				container_id          : params.search_tickets_container_id,
				navigation            : this, // el parent ( quien me ha instanciado )
				filter_by_campaign    : params.global_search_filter_by_campaign,
				columns_sort          : params.global_search_columns_sort || {},
				interactions_per_page : 25,
				last_control          : true,
				callbacks             : {}, // TODO: implementar ?
				permissions : { 
					global_search: (this.permissions || {}).global_search // el permiso de la navegacion, segun app
				}
			});
		}*/
		// --------------------------------------------------------
		// instancia integracion con helpdesk
		this.$helpdesk_container = null;
		if (typeof params.integration_helpdesk_container_id === 'string') {
			if (inConcert.helpdesk) {
				this.$helpdesk_container = $('#' + params.integration_helpdesk_container_id);
				this.helpdesk_tickets = new inConcert.helpdesk.tickets({
					container_id: params.integration_helpdesk_container_id
				});
			}
		}
		this.create();
		this.current_item = null;
		this.previous_item = null;
		this.events_suscribe_campaigns();
	}

	if (typeof inConcert.ConversationsNavigation !== 'function') { inConcert.ConversationsNavigation = ConversationsNavigation; }

	ConversationsNavigation.prototype.addSelectedLabel = function(label_id)
	{
		if(!label_id) {
			return;
		}
		this.selected_labels[label_id] = true;
	};

	ConversationsNavigation.prototype.delSelectedLabel = function(label_id)
	{
		if(!label_id || !this.selected_labels[label_id]) {
			return;
		}
		delete this.selected_labels[label_id];
	};

	ConversationsNavigation.prototype.clearSelectedLabels = function()
	{
		this.selected_labels = {};
	};

	ConversationsNavigation.prototype.isLabelSelected = function(label_id)
	{
		return label_id && this.selected_labels[label_id];
	};

	ConversationsNavigation.prototype.enter = function()
	{
		if (this.current_item === 'integration_helpdesk_tickets') {
			this.enter_tickets();
		}
		else {
			if(this.current_item && this.callbacks[this.current_item] && this.callbacks[this.current_item].filter_label instanceof Function) {
				// Aplico el filtro de labels por si viene de otra app y se cambio alli
				this.callbacks[this.current_item].filter_label();
			}
		}
	};

	ConversationsNavigation.prototype.enter_tickets = function()
	{
		if (this.helpdesk_tickets) {
			this.helpdesk_tickets.render();
		}
	};

	ConversationsNavigation.prototype.ReloadHelpdeskTickets = function(callback = $.noop)
	{
		if (!this.helpdesk_tickets) {
			callback({success: false, reason: 'there is no instance of the helpdesk tickets class'});
			return;
		}
		this.helpdesk_tickets.reload(callback);
	};

	ConversationsNavigation.prototype.get_parent_campaign_config = function( campaign_id )
	{
		if ( typeof campaign_id !== 'string' ) {
			return {}; // debe de retorna un objeto vacio, ya que el parent retorna asi (el supervisor)
		}
		if ( typeof this.parent.get_campaign_selected_config !== 'function' ) {
			return {}; // debe de retorna un objeto vacio, ya que el parent retorna asi (el supervisor)
		}
		return this.parent.get_campaign_selected_config(campaign_id);
	};

	ConversationsNavigation.prototype.update_campaigns = function( campaigns, action )
	{
		var self = this;

		action = action || 'ADD';
		this.campaigns = this.campaigns || {};

		$.each(campaigns || [], function(_, campaign)
		{
			var campaign_id = campaign.Id;
			if ( action === 'ADD' ) {
				self.campaigns[campaign_id] = inConcert.Utils.cloneObject( campaign || {} );
				self.events_suscribe_labels(campaign_id);
			}
			else {
				// si no es ADD es REMOVE
				self.events_unsuscribe_labels(campaign_id);
				delete self.campaigns[campaign_id];
			}
		});
	};

	ConversationsNavigation.prototype.events_suscribe_campaigns = function()
	{
		var self = this;
		// no me intersa quedarme con estos handle, ya que siempre queda escuchando
		inConcert.app.messagebus.subscribe('/application/campaigns/added',
			function(campaign_data) {
				if ( typeof campaign_data !== 'object' || !campaign_data.Id ) {
					return;
				}
				var campaign = inConcert.app.getCampaign( campaign_data.Id );
				if ( !campaign ) {
					return;
				}
				self.__on_campaign_added(campaign);
			}
		);
		inConcert.app.messagebus.subscribe('/application/campaigns/removed', function(campaign_data) {
			self.__on_campaign_removed(campaign_data.Id);
		});
	};

	ConversationsNavigation.prototype.events_suscribe_labels = function( campaign_id )
	{
		var self = this;

		this.pubsub_labels_handle = this.pubsub_labels_handle || {};
		//subscribe to labels changed
		var pub_lbl_id = '/update_labels/' + campaign_id + '/updated';
		this.pubsub_labels_handle[campaign_id] = inConcert.app.messagebus.subscribe(
			pub_lbl_id,
			function(differences)
			{
				var new_labels, updated_labels, removed_labels, label, cloned_label;

				$.each(differences || {}, function(label_id, difference)
				{
					switch(difference.type)
					{
						case 'NEW':
							new_labels = new_labels || {};
							label = difference.row;
							cloned_label = inConcert.Utils.cloneObject( label );
							label.campaigns = {};
							label.campaigns[campaign_id] = cloned_label;
							new_labels[label_id] = label;

						break;

						case 'UPDATE':
							updated_labels = updated_labels || {};
							label = difference.updated_row;
							cloned_label = inConcert.Utils.cloneObject( label );
							label.campaigns = {};
							label.campaigns[campaign_id] = cloned_label;
							updated_labels[label_id] = label;

						break;

						case 'DELETE':
							removed_labels = removed_labels || {};
							label = difference.deleted_row;
							label.removed_campaign = campaign_id;
							removed_labels[label_id] = label;
						break;
					}
				});

				self.add_campaigns_labels(new_labels, false /*cleanup*/);
				self.update_campaigns_labels(updated_labels);
				self.delete_campaigns_labels(removed_labels);
			}
		);
	};

	ConversationsNavigation.prototype.events_unsuscribe_labels = function(campaign_id)
	{
		if ( this.pubsub_labels_handle && this.pubsub_labels_handle[campaign_id] ) {
			inConcert.app.messagebus.unsubscribe(this.pubsub_labels_handle[campaign_id]);
		}
	};

	ConversationsNavigation.prototype.__on_campaign_added = function( campaign )
	{
		this.update_campaigns( [campaign] );

		var new_labels, cloned_label;

		campaign.loadLabels(function(labels)
		{
			$.each(labels || [], function(_, label)
			{
				new_labels = new_labels || {};
				cloned_label = inConcert.Utils.cloneObject( label );
				label.campaigns = {};
				label.campaigns[campaign.Id] = cloned_label;
				new_labels[label.name] = label;
			});
		});

		this.add_campaigns_labels(new_labels, false /*cleanup*/);
	};

	ConversationsNavigation.prototype.__on_campaign_removed = function( campaign_id )
	{
		var campaign = this.campaigns[campaign_id];
		if ( !campaign ) {
			return;
		}
		var removed_labels;

		campaign.loadLabels(function(labels)
		{
			$.each(labels || [], function(_, label)
			{
				removed_labels = removed_labels || {};
				label.removed_campaign = campaign_id;
				removed_labels[label.name] = label;
			});
		});

		this.delete_campaigns_labels(removed_labels);
		this.update_campaigns( [campaign], 'REMOVE' );
	};

	ConversationsNavigation.prototype.__has_permission = function( navigation_id )
	{
		if ( !this.permissions || typeof navigation_id !== 'string' ) {
			return false;
		}
		var permission_id = this.permissions[navigation_id];
		if ( typeof permission_id === 'string' ) {
			var has_permission = inConcert.permissions.to_action_on_any_campaign( this.from_app, permission_id );
			if ( !has_permission ) {
				return false;
			}
			return true;
		}
		else if( typeof permission_id === 'function' ) {
			return permission_id(navigation_id);
		}
		// si el que me instancia no ha definido permisos, retorno true
		return true;
	};

	ConversationsNavigation.prototype.has_license_for_helpdesk_tickets = function()
	{
		if (!this.can_show_integration_helpdesk_tickets()) {
			return false;
		}
		return inConcert.app.has_license_for_helpdesk_tickets();
	};

	ConversationsNavigation.prototype.load_system_folders = function()
	{
		const self = this;

		return {
		// Las interacciones asignadas
		//
			'assigned_conversations' : {
				type: { name: 'folder', title: true },
				name: function() {
					return self.from_app === 'interaction' ? _T('My Conversations') : _T('Assigned Conversations');
				},
				classes: function()   { return 'navigationFolderTitle folderTitleAssigned'; },
				tooltip: $.noop,
				onclick: $.noop,
				can_show : function() { return self.__has_permission('active') || self.can_show_live() || self.__has_permission('appointment') || self.__has_permission('archive'); },
				custom_attrs: $.noop
			},
			'active'  : {
				type: { name: 'folder', item: true },
				name: function()      {
					return self.from_app === 'interaction' ? _T('Active') : _T('Monitoring');
				},
				classes: function()   { return 'navigationFolderItem folderActive'; },
				tooltip: function()   { 
					return self.from_app === 'interaction' ? _T('Inbox') +': ' + _T('Active_Int') : _T('Active_Int');
				},
				onclick: function(options)   { return self.show_active(options); },
				can_show : function() { return self.__has_permission('active'); },
				belongs_to :          'assigned_conversations',
				selected :            true,
				custom_attrs: $.noop
			},
			'live'    : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Live'); },
				classes: function()   { return 'navigationFolderItem folderLive'; },
				tooltip: function()   { return _T('Inbox') +': ' + _T('Live'); },
				onclick: function()   { return self.show_live(); },
				can_show : function() { return self.can_show_live(); },
				belongs_to :          'assigned_conversations',
				custom_attrs: $.noop
			},
			'appointment'    : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Appointment'); },
				classes: function()   { return 'navigationFolderItem folderAppointment'; },
				tooltip: function()   { return _T('Appointment_Int'); },
				onclick: function()   { return self.show_appointment(); },
				can_show : function() { return self.__has_permission('appointment'); },
				belongs_to :          'assigned_conversations',
				custom_attrs: $.noop
			},
			'archive' : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Archive'); },
				classes: function()   { return 'navigationFolderItem folderArchive'; },
				tooltip: function()   { return _T('Archived_Int'); },
				onclick: function(options)   { return self.show_archive(options); },
				can_show : function() { return self.__has_permission('archive'); },
				belongs_to :          'assigned_conversations',
				custom_attrs: $.noop
			},

		// Las interacciones sin asignar
		//
			'unassigned_conversations' : {
				type: { name: 'folder', title: true },
				name: function()      { return _T('Unattended Conversations'); },
				classes: function()   { return 'navigationFolderTitle folderTitleUnassigned'; },
				tooltip: $.noop,
				onclick: $.noop,
				can_show : function() { return self.__has_permission('queued') || self.__has_permission('waiting')|| self.__has_permission('abandoned') || self.can_show_spam(); },
				custom_attrs: $.noop
			},
			'queued'  : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Queued'); },
				classes: function()   { return 'navigationFolderItem folderQueued'; },
				tooltip: function()   { return _T('Queued'); },
				onclick: function()   { return self.show_queued(); },
				can_show : function() { return self.__has_permission('queued'); },
				belongs_to :          'unassigned_conversations',
				custom_attrs: $.noop
			},
			'waiting' : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Waiting'); },
				classes: function()   { return 'navigationFolderItem folderWaiting'; },
				tooltip: function()   { return _T('Waiting_Int'); },
				onclick: function()   { return self.show_waiting(); },
				can_show : function() { return self.__has_permission('waiting'); },
				belongs_to :          'unassigned_conversations',
				custom_attrs: $.noop
			},
			'abandoned' : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Abandoned'); },
				classes: function()   { return 'navigationFolderItem folderAbandoned'; },
				tooltip: function()   { return _T('Abandoned_Int'); },
				onclick: function()   { return self.show_abandoned(); },
				can_show : function() { return self.__has_permission('abandoned'); },
				belongs_to :          'unassigned_conversations',
				custom_attrs: $.noop
			},
			'bot_interactions' : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('Bot Interactions'); },
				classes: function()   { return 'navigationFolderItem folderBots'; },
				tooltip: function()   { return _T('Bot_interactions_Int'); },
				onclick: function()   { return self.show_bot_interactions(); },
				can_show : function() { return self.__has_permission('bot_interactions') && self.from_app === 'supervisor'; },
				belongs_to :          'unassigned_conversations',
				custom_attrs: $.noop
			},
		// TODO: comentado hasta que se implemente correctamente este item
			// 'spam' : {
				// type: { name: 'folder', item: true },
				// name: function()      { return _T('Spam'); },
				// classes: function()   { return 'navigationFolderItem folderSpam'; },
				// tooltip: function()   { return _T('TODO: Spam tooltip ???'); },
				// onclick: function()   { return self.show_spam(); },
				// can_show : function() { return self.can_show_spam(); },
				// belongs_to :          'unassigned_conversations',
				// custom_attrs: $.noop
			// },

			'integrations' : {
				type: { name: 'folder', title: true },
				name: function() {
					return _T('Business Applications');
				},
				classes: function()   { return 'navigationFolderTitle folderTitleIntegrations'; },
				tooltip: $.noop,
				onclick: $.noop,
				can_show : function() { return self.__has_permission('active') || self.can_show_live() || self.__has_permission('appointment') || self.__has_permission('archive'); },
				custom_attrs: $.noop
			},
			'integration_helpdesk_tickets'    : {
				type: { name: 'folder', item: true },
				name: function()      { return _T('HelpDesk - Tickets'); },
				classes: function()   { return 'navigationFolderItem folderHelpDeskTickets'; },
				tooltip: function()   {
					if (!self.has_license_for_helpdesk_tickets()) {
						return '';
					}
					return _T('Tickets of HelpDesk');
				},
				onclick: function()   { return self.show_integration_helpdesk_tickets(); },
				can_show : function() { return self.can_show_integration_helpdesk_tickets(); },
				belongs_to :          'integrations',
				custom_attrs: function($folder) {
					$folder.removeClass('disabledFolderItem')
						.removeClass('infoFolderItem');
					if (self.has_license_for_helpdesk_tickets()) {
						$folder.prop('disabled', false);
						return;
					}
					$folder.prop('disabled', true);
					$folder.addClass('disabledFolderItem')
						.addClass('infoFolderItem');
				}
			},

		// la global search
			'global_search' : {
				type: { name: 'global_search', item: true },
				name: function()      { return _T('Search Global Conversations'); },
				classes: function()   { return 'TU_CLASE_GLOBAL_CONVERSATIONS'; },
				tooltip: function()   { return _T('Global search'); },
				onclick: function()   { return self.show_global_conversations(); },
				can_show : function() { return self.__has_permission('global_search'); },
				custom_attrs: $.noop
			},
			// #12296 - La busqueda de tickets se debe realizar desde el plugin inconcert_querybuilder
			//'search_tickets' : {
			//	type: { name: 'search_tickets', item: true },
			//	name: function()      { return _T('search_tickets'); },
			//	classes: function()   { return 'search_tickets'; },
			//	tooltip: function()   { return _T('search_tickets'); },
			//	onclick: function()   { return self.show_integration_search_tickets(); },
			//	can_show : function() { return self.__has_permission('unassigned_conversations'); },
			//	custom_attrs: $.noop
			//},
			'categorie' : {
				type: { name: 'categorie', item: true },
				name: function()      { return _T('Categories'); },
				classes: function()   { return 'categorie'; },
				tooltip: function()   { return _T('Labels'); },
				onclick: function()   { return self.show_integration_categories(); },
				can_show : function() { return self.__has_permission('unassigned_conversations'); },
				custom_attrs: $.noop
			}
		};
	};

	ConversationsNavigation.prototype.create = function()
	{
		var self = this;

		// Crea los folders/global search del sistema
		var $folders = this.$container.find('[folders]');
		$folders.html('');

		$.each(this.system_folders, function( id, folder )
		{
			var $folder = null;
			if ( folder.type.name === 'folder' ) {
				switch(id) {
					case 'appointment':
						$folder = $(format(
							'<li>' + 
								'<label folder_name="true"></label>' +
								'<div class="counterContainer">' +
									'<span title="{0}" class="activeCounter" active_counter="true"></span>' + 
									'<span title="{1}" class="inactiveCounter" inactive_counter="true"></span>' +
								'</div>' +
							'</li>',
							_T('Appointments on date'),
							_T('Appointments scheduled')
						));
						break;
					case 'queued':
					case 'abandoned':
						$folder = $(format(
							'<li>' + 
								'<label folder_name="true"></label>' + 
								'<div class="counterContainer">' +
									'<span title="{0}" class="agentCounter" agent_counter="true"></span>' + 
									'<span title="{1}" class="campaignCounter" campaign_counter="true"></span>' + 
								'</div>' +
							'</li>',
							_T('Agent'),
							_T('Campaign')
						));
						break;
					default:
						$folder = $('<li><label folder_name="true"></label><span class="elementCounter" counter="true"></span></li>');
						break;
				}
				$folder.attr('folder_id', id);
				$folder.addClass(folder.classes());
				var item_type = folder.type.title ? 'title' : 'item';
				$folder.attr('item_type', item_type);
				if (folder.belongs_to) {
					$folder.attr('belongs_to', folder.belongs_to);
				}
				$folders.append($folder);
			}
			else if ( folder.type.name === 'global_search' ) {
				var $global_search = $('<hr/><div global_search="true" class="interactionNavigationGlobalSearch"><label folder_name="true"></label><span counter="true"></span></div><hr/>');
				self.$container.append($global_search);
				$folder = $global_search;
			}
			else if ( folder.type.name === 'search_tickets' ) {
				var $search_tickets = $('<div id="int_conversations_navigation_search_button" global_search="true" class="interactionNavigationFolderAux"><label folder_name="true"></label><span counter="true"></span></div>');
				self.$container.append($search_tickets);
				$folder = $search_tickets;
			}
			else if ( folder.type.name === 'categorie' ) {
				var $categorie = $(format(
					'<div id="int_conversations_navigation_categorie_button" categories="true" global_search="true" class="interactionNavigationFolderAux">' +
						'<label folder_name="true"></label>' + 
						'<div class="counterContainer">' + 
							'<span title="{0}" class="selectedCategoriesCounter" selected_categories_counter="true"></span>' + 
						'</div>' +
					'</div>', _T('Labels')));
				self.$container.append($categorie);
				$folder = $categorie;
			}
			else {
				throw 'ConversationsNavigation.prototype.create: Unrecognized folder.type.name: ' + folder.type.name;
			}

			$folder.attr('folder_type', folder.type.name );
			$folder.find('[folder_name="true"]').text(folder.name());

			$folder.on('click', function() {
				if (!self.can_enter_folder()) {
					return;
				}
				if (!folder.can_show()) {
					return;
				}
				self.__select_folder_item(folder, $(this));
			});
			$folder.inconcert_tooltip({ message: folder.tooltip() });
			if (!folder.can_show()) {
				$folder.hide();
			}
			folder.custom_attrs($folder);
		});

		// aca realizo el merge de los labels de las campanas
		var merged_campaigns_labels = {};
		var count_campaigns = Object.keys(this.campaigns || {}).length;
		var processed_campaigns = 0;

		$.each(this.campaigns, function(_, campaign)
		{
			var campaign_id = campaign.Id;
			campaign.loadLabels(function(labels)
			{
				$.each(labels || [], function(_, label)
				{
					var cloned_label = inConcert.Utils.cloneObject(label);
					merged_campaigns_labels[label.name] = merged_campaigns_labels[label.name] || label;
					merged_campaigns_labels[label.name].campaigns = merged_campaigns_labels[label.name].campaigns || {};
					merged_campaigns_labels[label.name].campaigns[campaign_id] = cloned_label;
				});
				processed_campaigns++;
				if (processed_campaigns === count_campaigns) {
					self.add_campaigns_labels(merged_campaigns_labels, true /*cleanup*/);
				}
			});
		});
	};

	ConversationsNavigation.prototype.can_show_spam = function()
	{
		var can_show = false;

	// TODO: Comentado hasta que se comente este item!!!
	//
		// if ( !this.__has_permission('spam') ) {
			// return can_show;
		// }

		// if ( !inConcert.app.hasLicensesForChannel('MAIL') ) {
			// return can_show;
		// }
		// $.each( this.campaigns || {}, function(_, campaign)
		// {
			// $.each( campaign.accounts || {}, function(_, account)
			// {
				// if (account.proxyType !== 'MAIL') {
					// return true; // continue
				// }
				// can_show = true;
				// return false; // break
			// });
		// });

		return can_show;
	};

	ConversationsNavigation.prototype.can_show_live = function()
	{
		if ( this.from_app !== 'interaction' ) {
			// Solo se visualiza en la app interaction, ya que en esta estan separados.
			// En la app supervisor, se encuentra unificado en 'monitoring'
			return false;
		}
		if ( !this.__has_permission('live') ) {
			return false;
		}
		return true;
	};

	ConversationsNavigation.prototype.__$get_categories_container = function()
	{
		return this.$container.find('[categories]');
	};

	ConversationsNavigation.prototype.show_categories = function()
	{
		const $categories = this.__$get_categories_container();
		$categories.show();
	};

	ConversationsNavigation.prototype.hide_categories = function()
	{
		const $categories = this.__$get_categories_container();
		$categories.hide();
	};

	ConversationsNavigation.prototype.add_campaigns_labels = function( labels, cleanup )
	{
		var self = this;

		if (!labels) {
			return;
		}
		var $labels = this.$container.find('[labels]');
		if (cleanup) {
			$labels.html('');
		}

		var sorted_labels = inConcert.Utils.hash_to_sorted_array(labels, function(label) { return label.name; }) || {};

		$.each(sorted_labels || {}, function( _, label )
		{
			var $already_exist = $labels.find('[label_id="' + label.name + '"]');
			if ( $already_exist.length > 0 ) {
				var current_campaigns = $already_exist.data('campaigns');
				current_campaigns = $.extend(current_campaigns, label.campaigns);
				$already_exist.data('campaigns', current_campaigns);
				return true; // continue
			}
			var random_id = inConcert.Utils.generateGuid();
			var $label = $(
				'<li class="navigationTagItem">' +
					'<input id="' + random_id + '" selector="true" type="checkbox" class="tagItemCheck" />' + 
					'<label for="' + random_id + '" label_name="true" class="tagItemText"></label>' +
				'</li>'
				);
			$label.attr('label_id', label.name);
			$label.find('[selector="true"]').attr('label_id', label.name);
			$label.find('[label_name="true"]').text(label.name);
			$label.data('campaigns', label.campaigns);
			$labels.append($label);

			$label.find('[id="' + random_id + '"]').on('click', function()
			{
				var $this = $(this);
				var $parent = $this.parent();
				$parent.removeClass('selected');
				var label_id = $parent.attr('label_id').toUpperCase();
				if ( $this.prop('checked') ) {
					$parent.addClass('selected');
					self.filtered_labels[label_id] = true;
				}
				else {
					if (self.filtered_labels[label_id]) {
						delete self.filtered_labels[label_id];
					}
				}
				self.apply_filter_labels();
			});
		});
	};

	ConversationsNavigation.prototype.update_campaigns_labels = function( labels )
	{
		if (!labels) {
			return;
		}
		var $labels = this.$container.find('[labels]');

		$.each(labels || {}, function( _, label )
		{
			var $already_exist = $labels.find('[label_id="' + label.name + '"]');
			if ( $already_exist.length === 0 ) {
				return true; // continue
			}
			var current_campaigns = $already_exist.data('campaigns');
			current_campaigns = $.extend(current_campaigns, label.campaigns);
			$already_exist.data('campaigns', current_campaigns);
		});
	};

	ConversationsNavigation.prototype.delete_campaigns_labels = function( labels )
	{
		if (!labels) {
			return;
		}
		var $labels = this.$container.find('[labels]');

		$.each(labels || {}, function( _, label )
		{
			var $already_exist = $labels.find('[label_id="' + label.name + '"]');
			if ( $already_exist.length === 0 ) {
				return true; // continue
			}
			var current_campaigns = $already_exist.data('campaigns');
			delete current_campaigns[label.removed_campaign];
			$already_exist.data('campaigns', current_campaigns);
			var count_campaigns  = Object.keys( current_campaigns || {} ).length;
			if ( count_campaigns === 0 ) {
				var $chk = $already_exist.find('[selector="true"]');
				if ( $chk.prop('checked') ) {
					$chk.trigger('click');
				}
				$already_exist.remove();
			}
		});
	};

	ConversationsNavigation.prototype.__select_folder_item = function( system_folder, $system_folder, options)
	{
		var is_ok = system_folder.onclick(options);
		//copiar logica cuando entre ventana live
		if (is_ok) {
			var unselected_belongs_to;
			var $old_selected_folder = this.$container.find('[folders] [item_type="item"].selected');

			if ( $old_selected_folder.length > 0 ) {
				// 1 aca remuevo la clase selected al item previamente seleccionado
				$old_selected_folder.removeClass('selected');
				unselected_belongs_to = $old_selected_folder.attr('belongs_to');
			}

			// 2 aca le agrego la clase selected al item seleccionado
			$system_folder.addClass('selected');

			if (system_folder.belongs_to !== unselected_belongs_to) {
				if (unselected_belongs_to !== undefined) {
					// 3 aca elimino la clase selected al agrupador del item previamente seleccionada
					var $unselected_title = $system_folder.parent().find('[folder_id="' + unselected_belongs_to + '"][item_type="title"]');
					$unselected_title.removeClass('selected');
				}
				// 4 aca agrego la clase selected al agrupador del item seleccionado
				var $selected_title = $system_folder.parent().find('[folder_id="' + system_folder.belongs_to + '"][item_type="title"]');
				$selected_title.addClass('selected');
			}

			// 5 aca remuevo el caso especial del global search
			var folder_type = $system_folder.attr('folder_type');
			if ( folder_type === 'folder' ) {
				this.$container.find('[global_search]').removeClass('selected');
			}
		}

		return is_ok;
	};

	ConversationsNavigation.prototype.select_folder = function(folder_id, isLive, options)
	{
		if ( typeof folder_id !== 'string' ) {
			throw 'ConversationsNavigation.prototype.select_folder. Unrecognized folder_id: ' + folder_id;
		}
		var folder = this.system_folders[folder_id];
		if ( !folder ) {
			throw 'ConversationsNavigation.prototype.select_folder. Unrecognized folder_id: ' + folder_id;
		}
		var $folder;
		if ( folder.type.name === 'global_search' ) {
			$folder = this.$container.find('[global_search]');
		}
		else {
			var $folders = this.$container.find('[folders]');
			$folder = $folders.find('[folder_id="' + folder_id + '"]');
		}
		if ( $folder.length === 0 ) {
			throw 'ConversationsNavigation.prototype.select_folder. folder_id: ' + folder_id + ' does not exist in DOM';
		}
		if (isLive) {
			var unselected_belongs_to;
			var $old_selected_folder = this.$container.find('[folders] [item_type="item"].selected');

			if ( $old_selected_folder.length > 0 ) {
				// 1 aca remuevo la clase selected al item previamente seleccionado
				$old_selected_folder.removeClass('selected');
				unselected_belongs_to = $old_selected_folder.attr('belongs_to');
			}

			// 2 aca le agrego la clase selected al item seleccionado
			$folder.addClass('selected');

			if (folder.belongs_to !== unselected_belongs_to) {
				if (unselected_belongs_to !== undefined) {
					// 3 aca elimino la clase selected al agrupador del item previamente seleccionada
					var $unselected_title = $folder.parent().find('[folder_id="' + unselected_belongs_to + '"][item_type="title"]');
					$unselected_title.removeClass('selected');
				}
				// 4 aca agrego la clase selected al agrupador del item seleccionado
				var $selected_title = $folder.parent().find('[folder_id="' + folder.belongs_to + '"][item_type="title"]');
				$selected_title.addClass('selected');
			}

			// 5 aca remuevo el caso especial del global search
			var folder_type = $folder.attr('folder_type');
			if ( folder_type === 'folder' ) {
				this.$container.find('[global_search]').removeClass('selected');
			}
		}
		var is_ok = this.__select_folder_item( folder, $folder, options);
		return is_ok;
	};

	ConversationsNavigation.prototype.select_default_folder_item = function()
	{
		var $folders = this.$container.find('[folders]');
		var $selectable_folder, selectable_definition;
		var first_shown = false;

		$.each(this.system_folders, function( id, system_folder )
		{
			var is_folder = system_folder.type.name === 'folder';
			if ( is_folder ) {
				if ( system_folder.type.title ) {
					return true; // continue
				}
				var $folder = $folders.find('[folder_id="' + id + '"]');
				if ( $folder.length === 0 ) {
					return true; // continue
				}
				var is_visible = system_folder.can_show();
				if ( !is_visible ) {
					return true; // continue
				}
				if ( system_folder.selected ) {
					$selectable_folder = $folder;
					selectable_definition = system_folder;
					return false; // break
				}
				if ( !first_shown ) {
					$selectable_folder = $folder;
					selectable_definition = system_folder;
					first_shown = true;
				}
			}
		});

		if ( $selectable_folder ) {
			this.__select_folder_item(selectable_definition, $selectable_folder);
		}
	};

	ConversationsNavigation.prototype.get_selected_folder_id = function()
	{
		return this.current_item;
	};

	ConversationsNavigation.prototype.get_previous_selected_folder_id = function()
	{
		return this.previous_item;
	};

	ConversationsNavigation.prototype.apply_filter_labels = function()
	{
		var callbacks;

		switch( this.current_item )
		{
			case 'active':
				callbacks = this.callbacks.active || {};
			break;

			case 'live':
				callbacks = this.callbacks.live || {};
			break;

			case 'appointment':
				callbacks = this.callbacks.appointment || {};
			break;

			case 'archive':
				callbacks = this.callbacks.archive || {};
			break;

			case 'queued':
				callbacks = this.callbacks.queued || {};
			break;

			case 'waiting':
				callbacks = this.callbacks.waiting || {};
			break;

			case 'abandoned':
				callbacks = this.callbacks.abandoned || {};
			break;

			case 'global_search':
				// este es un caso especial, como ya se instancia en esta clase, listo la busqueda se realiza aca
				if ( this.global_search ) {
					var is_new_search = true;
					this.global_search.search(is_new_search);
				}
				callbacks = this.callbacks.global_search || {};
			break;
		}

		if (!callbacks) {
			// TODO: mensaje o jamas sucedera !!!
			return;
		}

		(callbacks.filter_label || $.noop)( this.filtered_labels );
	};

	/*
		Funcion encargada de realizar el show o hide de los items de la navegacion, en tiempo real.
		En si la misma debe ser invocada desde quien la haya instanciado, si es que necesita que la misma se actualice.
	*/
	ConversationsNavigation.prototype.toggle_navigation_items = function()
	{
		var self = this;
		var $folders = this.$container.find('[folders]');
		var $selectable_folder, selectable_definition;
		var current_item = this.get_selected_folder_id();
		var current_item_has_been_hidden = false;
		var first_shown = false;
		var title_container = {};

		// Aca se realiza el show o hide de los items del sistema (active, live, archive, queued, etc.)
		$.each(this.system_folders, function(id, system_folder)
		{
			var is_folder = system_folder.type.name === 'folder';
			if (is_folder) {
				if (!system_folder.type.item) {
					return true; // continue
				}
				var $folder = $folders.find('[folder_id="' + id + '"]');
				if ($folder.length === 0) {
					return true; // continue
				}
				if (!title_container.hasOwnProperty(system_folder.belongs_to)) {
					title_container[system_folder.belongs_to] = false;
				}
				var can_show = system_folder.can_show();
				if (can_show) {
					self.show_folder_item(id);
					title_container[system_folder.belongs_to] = true;
				}
				else {
					if (current_item === id) {
						current_item_has_been_hidden = true;
					}
					self.hide_folder_item(id);
				}
				if (!first_shown && system_folder.selected) {
					$selectable_folder = $folder;
					selectable_definition = system_folder;
					first_shown = true;
				}
			}
		});

		// Esto sirve para saber si se debe visualizar o no el Titulo contenedor de cada folder.
		// Si no ha quedado ningun item hijo (folder) el titulo debe estar oculto.
		$.each(title_container, function(parent_id, must_visualize) {
			if (must_visualize) {
				self.show_parent_folder(parent_id);
			}
			else {
				self.hide_parent_folder(parent_id);
			}
		});

		// El caso global search, es un caso especial y se debe de validar diferente.
		if (this.system_folders.hasOwnProperty('global_search')) {
			var $global_search = this.$container.find('[global_search]');
			if ($global_search.length > 0) {
				if (this.system_folders.global_search.can_show()) {
					$global_search.show();
				}
				else {
					if (current_item === 'global_search') {
						current_item_has_been_hidden = true;
					}
					$global_search.hide();
				}
			}
		}

		// Si el item en el cual nos encontramos posicionado, ahora pasa a estar oculto, debemos seleccionar el primer elemento seleccionable
		// de la lista, que se encuentre visible.
		if (current_item_has_been_hidden && $selectable_folder) {
			this.__select_folder_item(selectable_definition, $selectable_folder);
		}
	};

	ConversationsNavigation.prototype.show_parent_folder = function(parent_folder_id)
	{
		if (typeof parent_folder_id !== 'string') {
			throw 'ConversationsNavigation.prototype.show_parent_folder - parent_folder_id is required.';
		}
		var parent_folder = this.system_folders[parent_folder_id];
		if (!parent_folder) {
			throw 'ConversationsNavigation.prototype.show_parent_folder - Unrecognized parent_folder_id :' + parent_folder_id;
		}
		var $parent_folder = this.$container.find('[folders] [item_type="title"][folder_id="' + parent_folder_id + '"]');
		if ($parent_folder.length === 0) {
			return;
		}
		$parent_folder.show();
	};

	ConversationsNavigation.prototype.hide_parent_folder = function(parent_folder_id)
	{
		if (typeof parent_folder_id !== 'string') {
			throw 'ConversationsNavigation.prototype.hide_parent_folder - parent_folder_id is required.';
		}
		var parent_folder = this.system_folders[parent_folder_id];
		if (!parent_folder) {
			throw 'ConversationsNavigation.prototype.hide_parent_folder - Unrecognized parent_folder_id :' + parent_folder_id;
		}
		var $parent_folder = this.$container.find('[folders] [item_type="title"][folder_id="' + parent_folder_id + '"]');
		if ($parent_folder.length === 0) {
			return;
		}
		$parent_folder.hide();
	};

	ConversationsNavigation.prototype.show_folder_item = function( folder_id )
	{
		if ( typeof folder_id !== 'string' ) {
			throw 'ConversationsNavigation.prototype.show_folder_item - folder_id is required.';
		}
		var folder = this.system_folders[folder_id];
		if ( !folder ) {
			throw 'ConversationsNavigation.prototype.show_folder_item - Unrecognized folder_id :' + folder_id;
		}
		var $folder = this.$container.find('[folders] [item_type="item"][folder_id="' + folder_id + '"]');
		if ( $folder.length === 0 ) {
			return;
		}
		$folder.show();
	};

	ConversationsNavigation.prototype.hide_folder_item = function( folder_id )
	{
		if ( typeof folder_id !== 'string' ) {
			throw 'ConversationsNavigation.prototype.hide_folder_item - folder_id is required.';
		}
		var folder = this.system_folders[folder_id];
		if ( !folder ) {
			throw 'ConversationsNavigation.prototype.hide_folder_item - Unrecognized folder_id :' + folder_id;
		}
		var $folder = this.$container.find('[folders] [item_type="item"][folder_id="' + folder_id + '"]');
		if ( $folder.length === 0 ) {
			return;
		}
		$folder.hide();
	};

	ConversationsNavigation.prototype.on_show_folder = function()
	{
		if (this.current_item === null) {
			return;
		}
		if (this.current_item === 'integration_helpdesk_tickets') {
			this.hide_categories();
		}
		else {
			this.show_categories();
		}
		const callbacks = this.callbacks[this.current_item];
		if (!callbacks || !callbacks.on_show_folder) {
			return;
		}
		callbacks.on_show_folder();
	};
	ConversationsNavigation.prototype.clean_panel_current = function(panelName, isTaken = false)
	{
		const current_panel = inConcert.app.getMainWindow().getPanel(panelName);
		$.each(current_panel.m_windows || {}, function( window_int_id)
		{	
			const current_interaction = inConcert.app.getInteractionManager().getInteractions()[ window_int_id ] ;
			if (current_interaction && current_panel.m_windows[window_int_id].client) {
				if (current_interaction.state === 'TAKEN' || current_interaction.state === 'PENDING') {
					current_panel.m_windows[window_int_id].client.hide();
					current_panel.m_windows[window_int_id].isTakenMe = true;
				}else if(current_interaction.state === 'QUEUED' && current_panel.m_windows[window_int_id].isTakenMe) {
					current_panel.m_windows[window_int_id].client.hide();
					current_panel.m_windows[window_int_id].isTakenMe = true;
				}
				else if(current_interaction.isLocked()) {
					current_panel.m_windows[window_int_id].client.hide();
				}
				else {
					current_panel.m_windows[window_int_id].client.close();
					delete current_panel.m_windows[window_int_id];
				}
			}else{
				if (current_panel.m_windows[window_int_id].client) {
					current_panel.m_windows[window_int_id].client.close();
					delete current_panel.m_windows[window_int_id];
				}
			}
			if (isTaken) {
				const idWindowIframe = `#dialog_${window_int_id}`;
				const $windowLiveContainer = $(idWindowIframe).contents();
				const $sendMessageContainer = $windowLiveContainer.find('#sendMessageContainer');
				$sendMessageContainer.show();
			}
		}
		);	
	};

	ConversationsNavigation.prototype.show_active = function(options)
	{
		let is_ok = false;
		
		if ( this.current_item === 'active' ) {
			return is_ok;
		}
		if ( !this.__has_permission('active') ) {
			const error_message = _T('Unable to show active view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const active_callbacks = this.callbacks.active || {};

		this.previous_item = this.current_item;
		is_ok = (active_callbacks.show || $.noop)(options);
		if ( is_ok ) {
			this.current_item = 'active';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_live = function()
	{
		let is_ok = false;

		if ( this.current_item === 'live' ) {
			return is_ok;
		}
		if ( !this.can_show_live() ) {
			const error_message = _T('Unable to show live view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const live_callbacks = this.callbacks.live || {};

		this.previous_item = this.current_item;
		is_ok = (live_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'live';
		}
		this.on_show_folder();
		// TODO: Revisar esto NO va aca.
		//
		const panel = inConcert.app.getMainWindow().getPanel('AssignedInteractions');
		for (const currentInteraction in panel.m_windows) {
			if (panel.m_windows.hasOwnProperty(currentInteraction)) {
				const current_interaction = inConcert.app.getInteractionManager().getInteractions()[ currentInteraction ] ;
				const $interaction = inConcert.app.getMainWindow().getPanel('AssignedInteractions').$findInteraction(current_interaction);
				if ($interaction && $interaction.length > 0 && current_interaction.isLive()) {
					const $counterInteraction = $interaction.find(`#${current_interaction.id}_unreadEventCounter`);
					if (current_interaction.m_unread_messages === 0) {
						const idInteraction = current_interaction.id.replaceAll('-','_');
						const $counterInteractionContainer = $counterInteraction.find(`#assigned_interaction_${idInteraction}_unreadEventCounter`);
						$counterInteractionContainer.hide();
					}else {
						$counterInteraction.prop('title', current_interaction.m_unread_messages);
						$counterInteraction.text(current_interaction.m_unread_messages);
						$counterInteraction.show();
					}
				}
			}
		}
		return is_ok;
	};

	ConversationsNavigation.prototype.show_appointment = function()
	{
		let is_ok = false;

		if ( this.current_item === 'appointment' ) {
			return is_ok;
		}
		if ( !this.__has_permission('appointment') ) {
			const error_message = _T('Unable to show appointment view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const appointment_callbacks = this.callbacks.appointment || {};

		this.previous_item = this.current_item;
		is_ok = (appointment_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'appointment';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_archive = function(options)
	{
		let is_ok = false;
		let forceShow = options && options.forceShow;
		if ( this.current_item === 'archive' && !forceShow) {
			return is_ok;
		}
		if ( !this.__has_permission('archive') ) {
			const error_message = _T('Unable to show archive view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const archive_callbacks = this.callbacks.archive || {};

		this.previous_item = this.current_item;
		is_ok = (archive_callbacks.show || $.noop)(options);
		if ( is_ok ) {
			this.current_item = 'archive';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_queued = function()
	{
		let is_ok = false;

		if ( this.current_item === 'queued' ) {
			return is_ok;
		}
		if ( !this.__has_permission('queued') ) {
			const error_message = _T('Unable to show queued view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const queued_callbacks = this.callbacks.queued || {};

		this.previous_item = this.current_item;
		is_ok = (queued_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'queued';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_waiting = function()
	{
		let is_ok = false;

		if ( this.current_item === 'waiting' ) {
			return is_ok;
		}
		if ( !this.__has_permission('waiting') ) {
			const error_message = _T('Unable to show waiting view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const waiting_callbacks = this.callbacks.waiting || {};

		this.previous_item = this.current_item;
		is_ok = (waiting_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'waiting';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_abandoned = function()
	{
		let is_ok = false;

		if ( this.current_item === 'abandoned' ) {
			return is_ok;
		}
		if ( !this.__has_permission('abandoned') ) {
			const error_message = _T('Unable to show abandoned view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const abandoned_callbacks = this.callbacks.abandoned || {};

		this.previous_item = this.current_item;
		is_ok = (abandoned_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'abandoned';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_bot_interactions = function()
	{
		let is_ok = false;

		if ( this.current_item === 'bot_interactions' ) {
			return is_ok;
		}
		if ( !this.__has_permission('bot_interactions') ) {
			const error_message = _T('Unable to show bot interactions view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const bot_interactions_callbacks = this.callbacks.bot_interactions || {};

		this.previous_item = this.current_item;
		is_ok = (bot_interactions_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'bot_interactions';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.show_spam = function()
	{
		let is_ok = false;

		if ( this.current_item === 'spam' ) {
			return is_ok;
		}
		if ( !this.can_show_spam() ) {
			const error_message = _T('Unable to show spam view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const spam_callbacks = this.callbacks.spam || {};

		this.previous_item = this.current_item;
		is_ok = (spam_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'spam';
		}
		this.on_show_folder();
		return is_ok;
	};

	function __getTicketErrorMessage(error) {
		error = error || '';
		switch ( error.toUpperCase() ) {
			case 'ACCOUNT_DOES_NOT_EXIST':
				return _T('Account does not exist.');

			case 'ACCOUNT_DOES_NOT_HAVE_TICKET_CONFIG':
				return _T('This account dont have ticket configured.');

			case 'CAMPAIGN_DOES_NOT_EXIST':
				return _T('Campaign does not exist.');

			case 'INTERACTION_DOES_NOT_EXIST':
				return _T('Conversation does not exist.');

			case 'UNABLE_TO_EMIT_TICKET':
				return _T('Unable to emit ticket.');

			default:
				return _T('Server error');
		}
	}

	const showBannerMessage = function(error_message, message_type, timeout_ms, has_log)
	{
		message_type = message_type || 'Error';
		timeout_ms = timeout_ms || 5000;
		has_log = typeof(has_log) === 'undefined' ? true : has_log;

		inConcert.app.m_mainWindow.showBannerMessage(
			{
				message:     error_message,
				messageType: message_type,
				timeout:     timeout_ms
			},
			null,
			has_log);
	};

	ConversationsNavigation.prototype.show_integration_categories = function()
	{
		const $positionNode = this.$container.find('#int_conversations_navigation_categorie_button');
		var popupCategorie = new inConcert.CategorieUI({
			parent: $positionNode,
			conversations_navigation: this,
			interaction_id: null,
			canReposition: true,
			search_callback: function( result ) {
				if ( !result.status ) {
					var err_message = __getTicketErrorMessage( result.code );
					showBannerMessage( err_message );
				}
			}
		});

		popupCategorie.show({
			parentNode: $positionNode,
			callbacks: {
				onOpen: function() {
					$positionNode.addClass('selected');
					/*if (existsNode) {
						$interactionNode.addClass("selectedToSearchTicket");
					}*/
				},
				onClose: function() {
					$positionNode.removeClass('selected');
					/*if (existsNode) {
						$interactionNode.removeClass("selectedToSearchTicket");
					}*/
				}
			}
		});

		const $popupContiner = popupCategorie.data && popupCategorie.data.container;
		if ($popupContiner) {
			const window_height = window.innerHeight;
			const parentOffset = $positionNode.offset();
			const popupOffset = $popupContiner[0].getBoundingClientRect();
			const popup_height = popupOffset.height;
			const popup_top_position = parentOffset.top - 16; //A donde quiero que este el top del popup
			const popup_bottom_position = (popup_top_position + popup_height); //Donde terminaria el bottom del popup
			if (window_height <= popup_bottom_position) {
				popupCategorie.setPosition({
					top_position_config: {
						top_magic_number: (window_height - popup_height - 16) - parentOffset.top
					}
				});
			}
			else {
				popupCategorie.setPosition({
					top_position_config: {
						top_magic_number: -16
					}
				});
			}
		}
	};

	ConversationsNavigation.prototype.show_integration_search_tickets = function()
	{
		const $positionNode = $('#int_conversations_navigation_search_button');
		var popupTickets = new inConcert.TicketSearchUI({
			parent: $positionNode,
			interaction_id: null,
			canReposition: true,
			search_callback: function( result ) {
				if ( !result.status ) {
					var err_message = __getTicketErrorMessage( result.code );
					showBannerMessage( err_message );
				}
			}
		});

		popupTickets.show({
			parentNode: $positionNode,
			callbacks: {
				onOpen: function() {
					$positionNode.addClass('selected');
					/*if (existsNode) {
						$interactionNode.addClass("selectedToSearchTicket");
					}*/
				},
				onClose: function() {
					$positionNode.removeClass('selected');
					/*if (existsNode) {
						$interactionNode.removeClass("selectedToSearchTicket");
					}*/
				}
			}
		});
	};
	ConversationsNavigation.prototype.show_global_conversations = function()
	{
		let is_ok = false;

		if ( this.current_item === 'global_search' ) {
			return is_ok;
		}
		if ( !this.__has_permission('global_search') ) {
			const error_message = _T('Unable to show global conversations view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const global_search_callbacks = this.callbacks.global_search || {};

		this.previous_item = this.current_item;
		is_ok = (global_search_callbacks.show || $.noop)();
		if ( is_ok ) {
			// este es un caso especial, como ya se instancia en esta clase, listo la busqueda se realiza aca
			if ( this.global_search ) {
				// TODO: implementar el cambio de labels, para realizar una busqueda de nueva o no "is_new_search"
				const is_new_search = true;
				this.global_search.search(is_new_search);
			}			
			this.current_item = 'global_search';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.global_search_apply_filter = function(is_new_search = false)
	{
		if (!this.global_search ) {
			return;
		}
		this.global_search.search(is_new_search);
	};

	ConversationsNavigation.prototype.show_integration_helpdesk_tickets = function()
	{
		let is_ok = false;

		if ( this.current_item === 'integration_helpdesk_tickets' ) {
			return is_ok;
		}
		if (!this.can_show_integration_helpdesk_tickets()) {
			const error_message = _T('Unable to show HelpDesk tickets view: You not have permission for this action.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		if (!this.has_license_for_helpdesk_tickets()) {
			const error_message = _T('Unable to show HelpDesk tickets view: There are no licenses available.');
			inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
			return is_ok;
		}
		inConcert.app.m_mainWindow.hideBannerMessage();
		const helpdesk_tickets_callbacks = this.callbacks.integration_helpdesk_tickets || {};

		this.previous_item = this.current_item;
		is_ok = (helpdesk_tickets_callbacks.show || $.noop)();
		if ( is_ok ) {
			this.current_item = 'integration_helpdesk_tickets';
		}
		this.on_show_folder();
		return is_ok;
	};

	ConversationsNavigation.prototype.can_show_integration_search_tickets = function()
	{
		/*if (this.from_app !== 'interaction') {
			return false;
		}
		if (!this.permissions) {
			return false;
		}
		if (!this.helpdesk_tickets) {
			return false;
		}
		const is_enabled_helpdesk = inConcert.app.has_search_ticket_integration();
		if (!is_enabled_helpdesk) {
			return false;
		}
		const is_mobile = inConcert.app.isMobile();
		if (is_mobile) {
			return false;
		}
		const permissions = this.helpdesk_tickets.get_permissions();
		if (permissions && permissions.app_id && permissions.action_id) {
			const has_permission = inConcert.permissions.to_action(permissions.app_id, permissions.action_id);
			if (!has_permission) {
				return false;
			}
			return true;
		}*/
		// si el que me instancia no ha definido permisos, retorno FALSE
		return true;
	};

	ConversationsNavigation.prototype.can_show_integration_helpdesk_tickets = function()
	{
		if (this.from_app !== 'interaction') {
			return false;
		}
		if (!this.permissions) {
			return false;
		}
		if (!this.helpdesk_tickets) {
			return false;
		}
		const is_enabled_helpdesk = inConcert.app.has_helpdesk_integration();
		if (!is_enabled_helpdesk) {
			return false;
		}
		const is_mobile = inConcert.app.isMobile();
		if (is_mobile) {
			return false;
		}
		const permissions = this.helpdesk_tickets.get_permissions();
		if (permissions && permissions.app_id && permissions.action_id) {
			const has_permission = inConcert.permissions.to_action(permissions.app_id, permissions.action_id);
			if (!has_permission) {
				return false;
			}
			return true;
		}
		// si el que me instancia no ha definido permisos, retorno FALSE
		return false;
	};

	ConversationsNavigation.prototype.update_counters = function( info )
	{
		var $folders = this.$container.find('[folders]');
		if ( $folders.length === 0 ) {
			return;
		}
		$.each(info, function( name, value ) {
			var $folder = $folders.find('[folder_id="' + name + '"]');
			if ( $folder.length > 0 ) {
				if (value && typeof value === 'object') {
					$.each(value, function(sc_name, sc_value) {
						var text = sc_value > 0 ? sc_value : '';
						$folder.find('[' + sc_name + '_counter]').text(text);
					});
				}
				else {
					var text = value > 0 ? value : '';
					$folder.find('[counter]').text(text);
				}
			}
		});
	};

	ConversationsNavigation.prototype.get_selected_labels = function()
	{
		return Object.keys(this.selected_labels);
	};

	ConversationsNavigation.prototype.disable_navigation = function(disable)
	{
		this.m_disable_navigation = disable;
	};

	ConversationsNavigation.prototype.can_enter_folder = function()
	{
		if (this.m_disable_navigation) {
			return false;
		}
		return true;
	};
}());
