/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, Mustache, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	const escapeAttrib = inConcert.Utils.escapeAttrib;

	function InteractionGUI( App_interaction_actions, params )
	{
		this.m_labelsByInteraction = {};
		this.m_availables_labels = {};
		this.m_current_handled_nodes = {};
		if ( typeof App_interaction_actions !== 'function' ) {
			throw( 'InteractionGUI - parameter "App_interaction_actions" is required!!' );
		}
		this.params = params || {};
		this.owner = this.params.owner || {};
		this.m_interaction_actions = new App_interaction_actions({ parent: this, custom_allowed_actions: this.params.custom_allowed_actions });
		this.columns = params.columns || {};
		this.allowed_bulk_actions_fn = params.allowed_bulk_actions;
		this.from_app = this.params.from_app;
		this.$container = this.params.$container;
		this.callbacks = this.params.callbacks || {};
		this.set_events(this.$container);
		this.current_column_sort = null;
	}

	if (typeof inConcert.interaction_gui !== 'function') {
		inConcert.interaction_gui = InteractionGUI;
	}

	InteractionGUI.prototype.setTooltips = function( container_selector, bind_selector ) {
		$( container_selector ).inconcert_tooltip( { alignment: 'left-bottom', bind_selector: bind_selector });
	};

	InteractionGUI.prototype.updateInteractionDisposition = function(interaction, $container, labelMng) {
		var disposition = interaction.dispositionCode;
		if (!disposition) {
			return;
		}
		var dispositionHtml = labelMng.getSingleLabelMarkup({
			name       : disposition,
			className  : 'interactionResultTag',
			hasActions : false
		});
		if (dispositionHtml.length > 0) {
			$container.append(dispositionHtml);
		}
	};

	InteractionGUI.prototype.toggleInteractionLabels = function($labels_disposition)
	{
		//Remuevo el contendor con el # de labels ocultos si existey el wrap de los tags ocultos
		$labels_disposition.find("[grouped_tags='true']").remove();
		$labels_disposition.find("[interactionsTagContainer='true']").children().unwrap();
		const $container = $labels_disposition.parent();

		const $labels_disposition_detach = $labels_disposition.detach();
		//Texto (Asunto)
		const $int_abstract = $container.find('[int_abstract]');
		$container.removeClass("textContent");
		const int_abstract_width = $int_abstract.outerWidth(true);
		if ($int_abstract.text().length > 0) {
			$container.addClass("textContent");
		}

		$container.prepend($labels_disposition_detach);
		const labels_disposition_children = $labels_disposition.find("[key]");
		const halfWidthColumn = $container.innerWidth() / 2;

		/* */
		const WIDTH_MAGIC_NUMBER = 30; // Ancho maximo del contenedor del # de tags ocultos.
		const limit_width_to_labels = (int_abstract_width < halfWidthColumn ? halfWidthColumn + (halfWidthColumn - int_abstract_width) : halfWidthColumn);
		if (limit_width_to_labels <= 0) {
			return;
		}
		let totalWidtLabelsDisposition = 0;
		let position_label = 0;
		let key_label;
		
		labels_disposition_children.each(function() {
			//Itero por cada item (disposition code, label), voy sumando los anchos hasta que 
			// sean mayor al limit_width_to_labels
			const $this = $(this);
			totalWidtLabelsDisposition += $this.outerWidth(true);
			position_label += 1;
			if (totalWidtLabelsDisposition > (limit_width_to_labels - WIDTH_MAGIC_NUMBER) ) {
				//tomo la key para saber que item pasa el ancho
				key_label = $this.attr("key");
				return false;
			}
		});
		
		if (key_label) {
			let $childLimit = "";
			if (position_label > 1) {
				$childLimit = $(labels_disposition_children[position_label - 2]);
			}
			else{
				/*
					Cuando el primer label/disposition se lleva todo el limite permitido.
					Para que no quede todos los label ocultos, muestro el primero con max-width = limit_width_to_labels
					y oculto los demas. De esta forma garantizo que al menos se muestre un primer label/DC
				*/
				position_label++;
				$childLimit = $(labels_disposition_children[0]);
				$childLimit.attr("style", $childLimit.attr("style") + `max-width:${limit_width_to_labels - WIDTH_MAGIC_NUMBER}px;` ) 
			}

			if(labels_disposition_children.length > 1)
			{
				$childLimit.after($(`<div grouped_tags='true' class='totalHiddenLabels' style=''> +${labels_disposition_children.length - (position_label - 1)}</div>`));
			
				const $hiddenChildren = labels_disposition_children.slice(position_label - 1);
				$hiddenChildren.wrapAll('<div interactionsTagContainer="true" class="interactionsViewMoreTags" style="display:none"></div>');
				const $grouped_tags = $labels_disposition.find("[grouped_tags='true']");
				const $intViewMoreTags = $labels_disposition.find("[interactionsTagContainer='true']");
	
				$grouped_tags.hover(
					function() {
						//mouseenter 
						$intViewMoreTags.css({
							'top': $(this).offset().top + $(this).outerHeight() + 5, 
							'left': $(this).offset().left + 5
						});
					},
					function() {
						// mouseleave
					}
				);
			}
		}
		
	};

	InteractionGUI.prototype.updateInteractionLabels = function(interaction, $container, labelMng)
	{
		var labelHtml = labelMng.getOwnerLabelMarkup(interaction);
		if (labelHtml.length > 0) {
			$container.append(labelHtml);
			$container.find('.butDeleteTag').unbind('click').bind('click', function() {
				labelMng.deleteLabelOnInteractions($(this), [interaction]);
			});
			$container.parent().removeClass("labelContent").addClass("labelContent");
		}
		
		//update label reference
		var previousLabels = this.m_labelsByInteraction[interaction.id];
		var currentLabels = interaction.getLabels() || {};
		previousLabels = previousLabels ? previousLabels : {};
		for (var label in previousLabels) {
			if (previousLabels.hasOwnProperty(label) && !currentLabels[label]) {
				this.m_availables_labels[label]--;
				if (this.m_availables_labels[label] === 0) {
					delete this.m_availables_labels[label];
				}
			}
		}
		var newLabels = {};
		for (var new_label in currentLabels) {
			if (currentLabels.hasOwnProperty(new_label)) {
				newLabels[new_label] = true;
				if (!previousLabels[new_label]) {
					if (!this.m_availables_labels[new_label]) {
						this.m_availables_labels[new_label] = 0;
					}
					this.m_availables_labels[new_label]++;
				}
			}
		}
		this.m_labelsByInteraction[interaction.id] = newLabels;
	};

	InteractionGUI.prototype.update_labels_and_disposition = function(interaction, $node)
	{
		$node = $node || this.m_current_handled_nodes[interaction.id];
		if (!$node) {
			return;
		}
		var $container = $node.find('[int_labels_disposition]');
		if (!$container || $container.length === 0 ) {
			return;
		}
		$container.html('');
		var labelMng = new inConcert.LabelsManagement();
		this.updateInteractionDisposition(interaction, $container, labelMng);
		this.updateInteractionLabels(interaction, $container, labelMng);
		this.toggleInteractionLabels($container);
	};

	//retrieves the class to be applied to this interaction
	InteractionGUI.prototype.get_interaction_classes = function(interaction, $interaction)
	{
		var interactionClass = 'interactionListRow '; 
			interactionClass = interactionClass + (interaction.pending ? 'interactionWaiting ' : '');

		var channelClass = this.getChannelClass(interaction) + ' ';

		//add class for private messages
		if (typeof(interaction.isPrivateMessage) === 'function') {
			channelClass += interaction.isPrivateMessage()? 'privateMessage ' : '';
		}
		
		//add appointment class
		if (interaction.hasAppointment()) {
			channelClass += 'hasAppointment ';
		}
 
		//add class for groups messages
		if (typeof(interaction.isGroup) === 'function') {
			channelClass += interaction.isGroup()? 'groupMessage ' : '';
		}

		// Selected to work class (pencil)
		var selectedClass = interaction.opened ? ' selectedToWork' : '';

		if ( interaction.type === 'WEBCONTACT' ) {
		// agrega clase, si es que la interaccion de webcontact y se esta resolviendo.
			var resolving = interaction.hasRelationshipWithPendingInteraction();
			selectedClass = selectedClass + ( resolving ? ' selectedToResolve' : '' );
		}

		var contactConflictClass = interaction.hasContactConflict() ? ' interactionContactConflict' : '';

		if (interaction.isFinished()) {
			interactionClass = interactionClass + 'boxArchivedInteraction ' + channelClass + selectedClass;
		}
		else if (interaction.isAbandoned()) {
			interactionClass = interactionClass + 'boxAbandonedInteraction ' + channelClass + selectedClass +
					(interaction.belongsToAny()? 'agentAbandoned ' : 'campaignAbandoned ');
		}
		else {
			if (interaction.isQueued()) {
				interactionClass = interactionClass + 'boxQueuedInteraction ' +
					(interaction.belongsToAny()? 'agentQueued ' : 'campaignQueued ');
			}
			if (interaction.isAppointment()) {
				interactionClass = interactionClass + 'boxAppointmentInteraction ' +
					(interaction.isAppointmentActive()? 'activeAppointment ' : 'inactiveAppointment ');
			}
			interactionClass += channelClass;
			if (interaction.isLive()) {
				var expandMode = '', stateMode = '';
				if (interaction.isRead()) {
					expandMode = interaction.opened ? 'Select' : '';
				}
				else {
					stateMode = 'New';
				}
				interactionClass = interactionClass + 'boxActiveInteraction' + stateMode + expandMode + selectedClass + contactConflictClass;
			}
			else {
				interactionClass = interactionClass + 'boxOfflineInteraction' + (interaction.isRead() ? 'Read' : 'Unread') + selectedClass + contactConflictClass;
			}
		}

		if ( $interaction && this.params && typeof this.params.get_specifics_interaction_classes === 'function' ) {
			interactionClass = interactionClass + ' ' + this.params.get_specifics_interaction_classes( interaction, $interaction );
		}

		return interactionClass;
	};

	InteractionGUI.prototype.calculate_interaction_class = function( interaction, $interaction_node )
	{
		$interaction_node = $interaction_node || this.m_current_handled_nodes[interaction.id];
		if ( !$interaction_node ) {
			return;
		}
		var interactionClass = this.get_interaction_classes(interaction, $interaction_node);
		var currentClass = $interaction_node.attr('class');
		$interaction_node.removeClass(currentClass).addClass(interactionClass);
		if ( interaction.hasAttachments() ) {
			$interaction_node.addClass('interactionWithAttach');
		}
		var transfer_info = interaction.check_is_transfer();
		if ( transfer_info ) {
			var classTransfer = 'interactionTransferredTo' + (transfer_info.origin_data.type || '').toUpperCase();
			$interaction_node.addClass(classTransfer);
		}
		
		this.update_interaction_status_bot(interaction, $interaction_node);
		
		this.__calculate_column_class($interaction_node);
	};

	InteractionGUI.prototype.set_column_sort = function(current_column_sort) {
		// Esta propiedad contiene el siguiente formato: { column_id: column_info.id, header_column_name: header_column_name, order: order }
		// El mismo es dado segun la clase externa que ha invocado esta funcion: Ej: Cuando se invoca desde la clase "GlobalSearch"
		this.current_column_sort = current_column_sort;
	};

	InteractionGUI.prototype.__calculate_column_class = function($interaction_node)
	{
		if (this.current_column_sort) {
			var new_sorted = this.current_column_sort.header_column_name;
			var $new_sorted_column = $interaction_node.find('[int_column="' + new_sorted + '"]');
			if ($new_sorted_column.length > 0) {
				$new_sorted_column.removeClass('columnSorted').addClass('columnSorted');
			}
		}
	};

	InteractionGUI.prototype.update_contact_display = function(interaction, $interactionNode) {
		let contactDisplay = interaction.getDisplayName();
		$interactionNode.find('[int_contact_name]')
			.attr('contactID', interaction.contact && interaction.contact.length > 0 ? interaction.contact : '')
			.html(contactDisplay);
	};

	InteractionGUI.prototype.update_int_draft = function(interaction, $interactionNode) {
		const logged_user = inConcert.app.getLoggedUserId();
		const addText = interaction.hasDraft() && interaction.getDraft().user === logged_user;
		this.update_int_draft_display($interactionNode, addText);
	};

	InteractionGUI.prototype.update_int_draft_display = function($interactionNode, isAdd) {
		const $interaction_name = $interactionNode.find('[int_column="name"]');
		const $interactionDraftText = $interaction_name.find("#interactionDraftText");

		if ($interactionDraftText.length > 0) {
			$interactionDraftText.remove();
		}

		if (isAdd) {
			const $node = $("".concat(
				"<div id='interactionDraftText' class='interactionDraftText'>",
				($interactionNode.find('[int_contact_name]').text() !== "") ? "<span>, </span>" : "",
					"<span class='draftText'>", _T('Draft'), "</span>",
				"</div>"));

				$interaction_name.append($node);	
		}
	};

	InteractionGUI.prototype.update_interaction_status_bot = function(interaction, $interactionNode) {
		
		if (!$interactionNode || $interactionNode.length === 0) {
			$interactionNode = this.m_current_handled_nodes[interaction.id];
		}

		$interactionNode.removeClass("interactionTransferedToBot");
		$interactionNode.find('[int_batch_selector]').prop('disabled', false);
		$interactionNode.find('[int_menu_actions]').prop('disabled', false);
		const lastStep = interaction.getLastStep();
		const beforeLastStep = interaction.getBeforeLastStep();
		if ((interaction.state === "TRANSFERRING" && lastStep && lastStep.origin_data && lastStep.origin_data.type === "bot") ||
				(interaction.state === "BOT"  && beforeLastStep && beforeLastStep.action === "TRANSFERRING" && beforeLastStep.origin_data && beforeLastStep.origin_data.type === "bot")) {
			$interactionNode.addClass("interactionTransferedToBot");
			$interactionNode.find('[int_batch_selector]').prop('disabled', true);
			$interactionNode.find('[int_menu_actions]').prop('disabled', true);
		}
	};

	InteractionGUI.prototype.update_interaction_campaign = function(interaction, $interactionNode) {
		let display_campaign = this.__get_campaign_display(interaction);
		$interactionNode.find('[int_campaign]').text(display_campaign);
	};

	InteractionGUI.prototype.update_interaction_owner = function( interaction, $interactionNode )
	{
		let $owner = $interactionNode.find('[int_owner]');
		if ($owner.length === 0) {
			return;
		}
		let interaction_agent = this.__get_conversation_owner( interaction );
		if ( $owner.text() !== interaction_agent ) {
			$owner.text( interaction_agent );
		}
	};

	InteractionGUI.prototype.update_interaction_account = function(interaction, $interactionNode) {
		let display_account =  this.__get_account_display(interaction);
		$interactionNode.find('[int_account]').text(display_account);
	};

	InteractionGUI.prototype.update_contact_id = function(interaction, $interactionNode) {
		let display_contact_id =  this.__get_display_contact_id(interaction);
		$interactionNode.find('[int_contact_id]').text(display_contact_id);
	};

	InteractionGUI.prototype.update_interaction_attention_time = function(interaction, $interactionNode) {
		let display_attention_time =  this.__get_interaction_attention_time(interaction);
		$interactionNode.find('[int_attention_time]').text(display_attention_time);
	};

	InteractionGUI.prototype.update_interaction_attention_level = function(interaction, $interactionNode) {
		$interactionNode.find('[int_attention_level]').text(interaction.getAttentionLevelDisplayName());
	};

	InteractionGUI.prototype.__get_interaction_creation_date = function( interaction )
	{
		if ( !interaction ) {
			return 'NONE_DATE';
		}
		return interaction.creationDate();
	};

	InteractionGUI.prototype.__get_interaction_last_event_date = function( interaction )
	{
		if ( !interaction ) {
			return 'NONE_DATE';
		}
		return interaction.timeSinceLastEvent();
	};

	InteractionGUI.prototype.__get_interaction_last_step_date = function( interaction )
	{
		if ( !interaction ) {
			return 'NONE_DATE';
		}
		return interaction.lastStepTimestamp();
	};

	InteractionGUI.prototype.__get_interaction_last_date = function( interaction )
	{
		var last_date_type = 'last_step_date';

		if (typeof(this.callbacks.get_timestamp_type) === 'function') {
			last_date_type = this.callbacks.get_timestamp_type() || last_date_type;
		}

		if (last_date_type === 'last_event_date') {
			return this.__get_interaction_last_event_date(interaction);
		}

		return this.__get_interaction_last_step_date(interaction);
	};

	InteractionGUI.prototype.update_interaction_time = function(interaction, $interaction)
	{
		if (!interaction || !$interaction || $interaction.length === 0) {
			return;
		}
		if ($interaction.is('.asociatedInteractionsGroup')) {
			var self = this;
			$interaction.children().each(function() {
				var $child_interaction = $(this);
				var child_interaction = $child_interaction.data('interaction');
				//a node should always have an interaction associated, but if html mangling has taken place, consider it
				if (child_interaction) {
					self.update_interaction_time(child_interaction, $child_interaction);
				}
			});
		}else {
			var dom_interaction_date = $interaction.data('interaction_date');
			var time = this.__get_interaction_last_date(interaction);
			if ( dom_interaction_date !== time ) {
			// actualizo el DOM con la fecha, unicamente si el tiempo del ultimo evento ha cambiado
				$interaction.data('interaction_date', time);
				$interaction.find('[int_last_event_date_or_state_date]').text(time);
			}
		}
	};

	InteractionGUI.prototype.update_all_interactions_time = function()
	{
		var self = this;
		if (!this.$container || this.$container.length === 0){
			return false;
		}
		if ( !this.$container.is(':visible') ) {
			return false;
		}
		var $interactions = this.$container.children();
		if ($interactions.length === 0) {
			return false;
		}
		$.each($interactions, function(_, dom_interaction)
		{
			var $interaction = $(dom_interaction);
			var interaction = $interaction.data('interaction');
			self.update_interaction_time(interaction, $interaction);
		});
		return true;
	};

	InteractionGUI.prototype.update_interaction_text = function(interaction, $interactionNode) {
		const $int_abstract = $interactionNode.find('[int_abstract]');
		const int_abstract_text = inConcert.Utils.escapeAttrib(interaction.getExtendedInfo());
		$int_abstract.html(int_abstract_text);

		let internal_consultation_flag = interaction.interactionData && interaction.interactionData.internal_consultation_flag && interaction.state === "CREATED";
		if (internal_consultation_flag) {
            $interactionNode.addClass("botWaitingHelp");
		} else {
            $interactionNode.removeClass("botWaitingHelp");
		}
	};

	InteractionGUI.prototype.update_interaction_unread_messages = function( interaction, $interactionNode )
	{
		if( !interaction.supportsEvents() ) {
			// Don't show anything
			return;
		}
		var unread_events_count = interaction.getUnreadEventCount();
		var unread_message = format( _T('{0}'), unread_events_count );
		var $unread = $interactionNode.find('[int_unread_events]');
		$unread.text( unread_message );
		if (unread_events_count === 0) {
			$unread.hide();
			$unread.removeAttr("title");
		}
		else{
			$unread.show();
			$unread.attr("title", _T('Unread Messages'));
		}

	};

	InteractionGUI.prototype.update_interaction_unanswered_messages = function( interaction, $interactionNode )
	{
		const current_app = inConcert.app.getCurrentApp();
		const $columns_unanswered = $interactionNode.find('[int_column="unanswered_events"]');
		if(current_app !== "supervisor")
		{
		   $columns_unanswered.hide();
		}
		else{
			$columns_unanswered.show();
		}

		if( !interaction.supportsEvents()) {
			// Don't show anything
			return;
		}
		const events_count = interaction.getUnAnsweredEventCount();
		const $int_unanswered = $interactionNode.find('[int_answered_events]');

		if (events_count === 0) {
			$int_unanswered.removeAttr("title");
			$int_unanswered.hide();
		}
		else{
			$int_unanswered.text( format( _T('{0}'), events_count ) );
			$int_unanswered.attr("title", _T('Unanswered Messages'));
			$int_unanswered.show();
		}
	};

	InteractionGUI.prototype.update_interaction_comments = function(interaction, $interactionNode) {
		var self = this;
		var comments = interaction.getComments();
		if (comments && comments.length > 0) {
			var commentsNode = $interactionNode.find('[int_comment]');
			commentsNode.show();
			var totalComments = comments.length;
			var commentTooltip = format('{0} {1}', totalComments, totalComments === 1 ? _T('comment') : _T('comments'));
			commentsNode.attr('title', commentTooltip).show().find('span').text(totalComments);
			commentsNode
				.removeClass('interactionCommentsNew interactionCommentsRead interactionCommentUnread')
				.addClass(self.getCommentClass(interaction));
		}
	};

	InteractionGUI.prototype.update_tooltips = function(interaction, $interactionNode)
	{
		$.each($interactionNode.find('[int_column]'), function(_, cell) {
			const $cell = $(cell);
			let cell_column = $cell.attr('int_column');
			if (cell_column === 'type') {
				let transfer_info = interaction.check_is_transfer();
				if (transfer_info) {
					let target_data = transfer_info.target_data;
					if (target_data) {
						let display_transfer_type;
						let target_type = (target_data.type || '').toLowerCase();
						if (target_type === 'member') {
							display_transfer_type = _T('Agent');
						}
						else if (target_type === 'campaign') {
							display_transfer_type = _T('Campaign');
						}
						else if (target_type === 'attention_level') {
							display_transfer_type = _T('Attention Level');
						}
						else if (target_type === 'number') {
							display_transfer_type = _T('Number');
						}
						display_transfer_type = format(_T('Transfer Type: {0}'), display_transfer_type);
						$cell.attr('title', display_transfer_type);
					}
				}
			}
			else {
				let cell_value = $cell.text();
				if (cell_column === 'abstract') {
					cell_value = $cell.find("[int_abstract]").text();
				}
				else {
					cell_value = $cell.text();
				}
				let tooltip = cell_value;
				let column_name = $cell.attr('int_column_name');
				if (column_name) {
					tooltip = format("{0}: {1}", column_name, cell_value);
				}
				if (cell_column === 'abstract') {
					return $cell.find("[int_abstract]").attr('title', tooltip);
				}
				$cell.attr('title', tooltip);
			}
		});
	};

	//Refresh node data for an interaction
	InteractionGUI.prototype.update_interaction_data = function( interaction, $interaction_node )
	{
		interaction.opened = this.is_opened_client_window(interaction);

		// Update internal interaction data
		$interaction_node.data('interaction', interaction);

		this.calculate_interaction_class(interaction, $interaction_node);
		this.update_button_actions(interaction, { $interaction_node: $interaction_node });
		this.update_interaction_campaign(interaction, $interaction_node);
		this.update_interaction_account(interaction, $interaction_node);
		this.update_interaction_attention_level(interaction, $interaction_node);
		this.update_interaction_owner(interaction, $interaction_node);
		this.update_contact_display(interaction, $interaction_node);
		this.update_contact_id(interaction, $interaction_node);
		this.update_interaction_attention_time(interaction, $interaction_node);

		// Extended info and interaction date
		this.update_interaction_time(interaction, $interaction_node);
		this.update_interaction_text(interaction, $interaction_node);

		this.update_interaction_unread_messages(interaction, $interaction_node);
		this.update_interaction_unanswered_messages(interaction, $interaction_node);
		this.update_interaction_comments(interaction, $interaction_node);

		this.update_monitoring_info(interaction, $interaction_node);
		// Labels and Disposition Codes
		this.update_labels_and_disposition(interaction, $interaction_node);
		// muestra / oculta batch selector
		this.update_batch_selector(interaction, $interaction_node);
		// actualiza los tooltips
		this.update_tooltips(interaction, $interaction_node);

		this.update_interaction_status_bot(interaction, $interaction_node);

		//Draft
		this.update_int_draft(interaction, $interaction_node);
	};

	InteractionGUI.prototype.update_batch_selector = function( interaction, $interactionNode )
	{
		if (typeof(this.allowed_bulk_actions_fn) !== 'function') {
		// si no se ha definido la funcion que valida si se debe visualizar o no el chk selector de la interaction
		// la misma queda como estaba (estatico como se definio cuando se instancio)
			return;
		}
		// si se ha definido la funcion, la misma se invoca para determinar si se debe mostrar u ocultar de forma dinamica el selector.
		var $batch_selector = $interactionNode.find('[int_batch_selector]');
		if (this.allowed_bulk_actions_fn(interaction)) {
			$batch_selector.show();
			return;
		}
		$batch_selector.hide();
	};

	InteractionGUI.prototype.update_monitoring_info = function(interaction, $interaction)
	{
		if (!$interaction || $interaction.length === 0){
			$interaction = this.m_current_handled_nodes[interaction.id];
		}

		//set state
		var displayState = '';
		switch (interaction.state) {
			case 'QUEUED':
			case 'TRANSFER QUEUED':
				displayState = interaction.hasPendingLabel() ? _T('Active') : _T('Queued');
				break;
			case 'LOCKED':
				if (interaction.type === 'CALL') {
					displayState = interaction.isOutgoing()? _T('Dialing') : _T('Ringing');
				}
				else {
					displayState = _T('Active');
				}
				break;
			case 'RESERVED':
			case 'TAKEN':
				displayState = _T('Active');
				break;
			case 'CANCELED':
			case 'FINISHED':
			case 'SPAM':
				displayState = _T('Ended');
				break;
			case 'WAITING':
				displayState = _T('Waiting');
				break;
			case 'WRAPUP':
				displayState = _T('Wrapping Up');
				break;
			case 'ABANDONED':
				displayState = _T('Abandoned State');
				break;
			case 'OPEN':
				if (interaction.isAppointment()) {
					displayState = _T('Appointment');
				}
				else {
					displayState = _T('Open');
				}
				break;
			default:
				displayState = _T('Active');
				break;
		}
		var $int_state_owner = $interaction.find('[int_column="state"]');
		$int_state_owner.find('[int_state]').text(displayState);
		if (typeof(this.owner.onInteractionUpdated) === 'function') {
			this.owner.onInteractionUpdated(interaction);
		}
	};

	InteractionGUI.prototype.get_menu_action_class = function(_, action_id) {
		return 'action' + action_id;
	};

	InteractionGUI.prototype.is_visible_action = function(action_id, interaction, not_must_validate_custom_action) {
		return this.m_interaction_actions.is_visible_action(action_id, interaction, not_must_validate_custom_action);
	};

	InteractionGUI.prototype.count_visible_actions = function(interaction) {
		return this.m_interaction_actions.count_visible_actions(interaction);
	};

	InteractionGUI.prototype.execute_action = function(action_id, interaction, params) {
		return this.m_interaction_actions.execute(action_id, interaction, params);
	};

	//loads the interaction available buttons
	InteractionGUI.prototype.update_button_actions = function(interaction, options)
	{
		options = options || {};

		var $interaction_node = options.$interaction_node || this.m_current_handled_nodes[interaction.id];
		if ( !$interaction_node || $interaction_node.length === 0 ){
			return;
		}

		var $button = $interaction_node.find('[int_menu_actions]');
		if ( $button.length === 0 ) {
			return;
		}

		var is_visible_button = false;
		var actions = this.m_interaction_actions.get( interaction );
		options.forbiddenActions = options.forbiddenActions || [];

		for (var action_id in actions) {
			if (actions.hasOwnProperty(action_id)) {
				if (!_.contains(options.forbiddenActions, action_id)) {
					var action = actions[action_id];
					if ((typeof(action.visible) === 'function' && action.visible(interaction)) || action.visible === true) {
						is_visible_button = true;
						break;
					}
				}
			}
		}

		// muestra u oculta el boton que abre el popup de acciones
		if ( is_visible_button ) {
			$button.show();
			// actualizo las opciones del menu de acciones
			this.__update_menu_actions(interaction, {
				$parent: $button,
				$interaction_node: $interaction_node
			});
		}
		else {
			$button.hide();
		}
	};

	InteractionGUI.prototype.decrement_internal_list = function(interaction)
	{
		if ( this.m_labelsByInteraction && this.m_labelsByInteraction[ interaction.id ] ) {
			delete this.m_labelsByInteraction[ interaction.id ];
		}
	};

	InteractionGUI.prototype.on_closed_client_window = function(interaction, panel)
	{
		this.calculate_interaction_class(interaction);
		panel.onClientClosed(interaction); //agent/scripts/base_panel.js
	};

	InteractionGUI.prototype.is_opened_client_window = function(interaction)
	{
		// TODO:inter2022 - ver si hay que pasar por paremtro el panel (caso de global_search)
		return inConcert.app.getInteractionManager().isOpenInteractionClient(interaction);
	};

	InteractionGUI.prototype.close_client_window = function(interaction)
	{
		inConcert.app.getInteractionManager().closeInteractionClient(interaction);
	};

	InteractionGUI.prototype.onLocalInteractionUpdate = function() { /* dummy */ };

	//Calculates de class to apply to an interaction according to its base type
	InteractionGUI.prototype.getChannelClass = function(interaction)
	{
		var channelClasses = {
			CALL          : function(interaction) {
				if(interaction.isWeb()){
					return 'interactionTypeWEBCALL';
				}
				return 'interactionTypeCALL';
			},
			VOICEMAIL     : function() {return 'interactionTypeVOICEMAIL';     },
			CHAT          : function() {return 'interactionTypeCHAT';          },
			FACEBOOK      : function() {return 'interactionTypeFACEBOOK';      },
			INSTAGRAM     : function() {return 'interactionTypeINSTAGRAM';     },
			YOUTUBE	      : function() {return 'interactionTypeYOUTUBE';       },
			APPSTORE      : function() {return 'interactionTypeAPPSTORE';      },
			GOOGLEPLAY    : function() {return 'interactionTypeGOOGLEPLAY';    },
			RCS	          : function() {return 'interactionTypeRCS';       	   },
			IM            : function() {return 'interactionTypeIM';            },
			INTERNAL_CHAT : function() {return 'interactionTypeINTERNAL_CHAT'; },
			MAIL          : function() {return 'interactionTypeMAIL';          },
			SMS           : function() {return 'interactionTypeSMS';           },
			WHATSAPP      : function() {return 'interactionTypeWHATSAPP';      },
			WEBCONTACT    : function() {return 'interactionTypeWEBCONTACT';    },
			TWITTER       : function(interaction) {
				var twitterClass = 'interactionTypeTWITTER';
				if (interaction.isValid()) {
					if (interaction.isDirectMessage()) {
						return twitterClass + ' twitterDM';
					}
					else if(interaction.isQuoted()) {
						return twitterClass + ' twitterQuoted';
					}
				}
				return twitterClass;
			}
		};
		var channel = channelClasses[interaction.type];
		if (!channel) {
			throw('No class found for the received interaction of type ' + interaction.type);
		}
		var getStateClass = function()
		{
			var state_class;
			//Wrapup interaction. Only calls and im
			state_class = interaction.isWrappedUp() ? ' interactionEnded' : '';
			//Invalid interaction. Only facebook, mail, twitter
			state_class += !interaction.isValid() ? ' interactionInvalid' : '';
			return state_class;
		};
		var getInitiativeClass = function(initiative)
		{
			switch (initiative) {
				case 'OUTBOUND':
					return 'interactionOutbound';
				case 'INBOUND':
					return 'interactionInbound';
				default:
					return '';
			}
		};
		return channel(interaction) + ' ' + getStateClass() + ' ' + getInitiativeClass(interaction.initiative);
	};

	InteractionGUI.prototype.getCommentClass = function(interaction) {
		if (!interaction.commentReaders) {
			return 'interactionCommentsNew';
		}
		//check if the comments are unread for the current user
		if (interaction.youHasUnreadComments()) {
			return 'interactionCommentsUnread';
		}
		return 'interactionCommentsRead';
	};

	/**
	 *Get selected nodes from the interactions panel.
	 *
	 * @private
	 * @return {Object} result    - The result with the totals.
	 * @return {Array} result.all - All nodes selected.
	 * @return {Array} result.interactions - All interactions selected
	 */
	InteractionGUI.prototype.batch_select_all_interactions = function( mark_all )
	{
		if (!this.$container || this.$container.length === 0) {
			inConcert.logger.warn({
				action_name : 'InteractionGUI.prototype.batch_select_all_interactions',
				action_type : 'batch selections',
				message     : 'Container "this.$container" not found!!'
			});
			return false;
		}
		var $interactions = this.$container.children();
		// Check/UnCheck interactions
		$interactions.find('[int_batch_selector]').prop('checked', mark_all);
		// add/remove class to highlight selected interactions
		$interactions
			.addClass(mark_all ? 'interactionSelectToBatch' : '')
			.removeClass(mark_all ? '' : 'interactionSelectToBatch');
		return true;
	};

	/**
	 * Muestra todos los selectores de todas las interacciones (checkboxes) que seleccionan las interacciones para su ejecucion por batch.
	 */
	InteractionGUI.prototype.show_batch_selectors = function()
	{
		if (!this.$container || this.$container.length === 0) {
			inConcert.logger.warn({
				action_name : 'InteractionGUI.prototype.show_batch_selectors',
				action_type : 'batch hide selector',
				message     : 'Container "this.$container" not found!!'
			});
			return false;
		}
		var $interactions = this.$container.children();
		$interactions.find('[int_batch_selector_container]').show();
		return true;
	};

	/**
	 * Oculta todos los selectores de todas las interacciones (checkboxes) que seleccionan las interacciones para su ejecucion por batch.
	 */
	InteractionGUI.prototype.hide_batch_selectors = function()
	{
		if (!this.$container || this.$container.length === 0) {
			inConcert.logger.warn({
				action_name : 'InteractionGUI.prototype.hide_batch_selectors',
				action_type : 'batch hide selector',
				message     : 'Container "this.$container" not found!!'
			});
			return false;
		}
		var $interactions = this.$container.children();
		$interactions.find('[int_batch_selector_container]').hide();
		return true;
	};

	/**
	 *Get selected nodes from the interactions panel.
	 *
	 * @private
	 * @return {Object} result    - The result with the totals.
	 * @return {Array} result.all - All nodes selected.
	 * @return {Array} result.interactions - All interactions selected
	 */
	InteractionGUI.prototype.get_batch_selected_interactions = function()
	{
		var result = {
			interactions : []
		};

		if (!this.$container || this.$container.length === 0){
			return result;
		}

		var interactionManager = inConcert.app.getInteractionManager();

		// get the id of every interaction checked and push to the list of interactions
		this.$container.find('[int_batch_selector]:checked').attr('row_id', function(_, row_id)
		{
			var interaction = $('#' + row_id).data('interaction');
			var currentInteraction = interactionManager.getInteractions()[interaction.id];
			if (currentInteraction){
				result.interactions.push(currentInteraction);
			}
		});

		return result;
	};

	InteractionGUI.prototype.show_banner_message = function(error_message, message_type, timeout_ms, has_log)
	{
		message_type = message_type || 'Error';
		timeout_ms = timeout_ms || 5000;
		has_log = typeof(has_log) === 'undefined' ? true : has_log;

		inConcert.app.m_mainWindow.showBannerMessage(
			{
				message:     error_message,
				messageType: message_type,
				timeout:     timeout_ms
			},
			null,
			has_log);
	};

	InteractionGUI.prototype.hide_banner_message = function()
	{
		inConcert.app.m_mainWindow.hideBannerMessage();
	};

	InteractionGUI.prototype.__show_submenu = function(action, interaction, submenu_callbacks = {})
	{
		const self = this;

		var interactions;
		if ($.isArray(interaction)) {
			interactions = interaction;
			interaction = inConcert.Utils.cloneObject(interaction[0]);
		}
		else {
			interactions = [interaction];
		}

		var submenu_id = 'int_gui_submenu_actions_' + inConcert.Utils.generateGuid();
		$('#' + submenu_id).remove();
		var $submenu = $(''.concat(
			'<div id="' + submenu_id + '" class="interactionMenuBlind" hidden></div>'
		));
		var count_visible_actions = 0;
		var sub_actions = action.submenu;
		for (var sub_action_id in sub_actions) {
			if (sub_actions.hasOwnProperty(sub_action_id))
			{
				var sub_action = sub_actions[sub_action_id];
				if ((typeof(sub_action.visible) === 'function' && sub_action.visible(interaction)) || sub_action.visible === true) {

					var buttonClass = this.get_menu_action_class(interaction, sub_action_id);
					var name = typeof(sub_action.tooltip) === 'function' ? sub_action.tooltip(interaction) : sub_action.tooltip;
					var $sub_action = $(
						'<div class="interactionMenuItem ' + buttonClass + '">' +
							name +
						'</div>'
					);
					$sub_action.attr('actionid', sub_action_id);
					if ( typeof(sub_action.enabled) === 'function' && !sub_action.enabled(interaction) ) {
						$sub_action.prop('disabled', true);
					}
					$submenu.append( $sub_action );
					count_visible_actions++;
				}
			}
		}

		if ( count_visible_actions === 0 ) {
			this.show_banner_message(_T('Conversation not have available sub actions.'));
			return;
		}

		$('body').append($submenu);

		$submenu.find('div[actionid]')
			.unbind('click')
			.bind( 'click', function()
			{
				var $this = $(this);
				var action_id = $this.attr('actionid');
				var sub_action = sub_actions[action_id];
				if ( !sub_action ) {
					throw('InteractionGUI.prototype.show_menu_actions: Unrecognized button sub_action "' + action_id + '"');
				}
				self.submenu_actions.finalize(function() {
					self.submenu_actions = null;
				});
				sub_action.positionNode = action.parentPositionNode;
				sub_action.interactionNode = action.interactionNode;
				sub_action.execute(interactions, self.callbacks);
			});

		if (this.submenu_actions) {
			// si el submenu ya se encuentra abierto, lo cierro
			this.submenu_actions.finalize();
			let current_submenu_action = this.submenu_actions.__action_id;
			this.submenu_actions = null;
			if (current_submenu_action === action.tooltip) {
				// si es el mismo submenu, solo cierra
				return;
			}
			// si es un submenu diferente abre el nuevo submenu (ej: tiene abierto el submenu de transferencias, y presiono para visualizar el de conferencias)
		}
		this.submenu_actions = new inConcert.PopupWindow({ownerId: submenu_id});

		var align;
		if (action.align === undefined) {
			align = 'LEFT';
		}

		var cloned_interaction = inConcert.Utils.cloneObject( interaction );
		// Show popup
		this.submenu_actions.show(
		{
			ownerId       	  : submenu_id,
			eventName     	  : submenu_id,
			interaction   	  : cloned_interaction,
			canReposition 	  : true,
			container     	  : $submenu,
			parentNode    	  : action.positionNode,
			align         	  : align,
			submenuNewPosition: action.submenuNewPosition,
			callbacks: {
				onOpen: function() {
					$('#' + submenu_id).show();
					$('#' + submenu_id).css('left', '300');
					(submenu_callbacks.on_open_submenu || $.noop)(self.submenu_actions);
				},
				onClose: function(callback = $.noop) {
					if (self.submenu_actions) {
						self.submenu_actions.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
					$('#' + submenu_id).remove();
					(submenu_callbacks.on_close_submenu || $.noop)();
				}
			}
		});
		// marco el Id del submenu que se ha abierto
		this.submenu_actions.__action_id = action.tooltip;
	};

	InteractionGUI.prototype.__show_menu = function( interaction, params )
	{
		var self = this;

		this.hide_banner_message();

		var menu_id = 'int_gui_menu_actions';
		$('#' + menu_id).remove();
		var $menu = $(''.concat(
			'<div id="' + menu_id + '" class="interactionMenuBlind" hidden></div>'
		));

		var actions = this.m_interaction_actions.get( interaction );

		var count_visible_actions = 0;
		params.forbiddenActions = params.forbiddenActions || [];
		for (var action_id in actions)
		{
			if (actions.hasOwnProperty(action_id) && !_.contains(params.forbiddenActions, action_id))
			{
				var action = actions[action_id];
				if ((typeof(action.visible) === 'function' && action.visible(interaction)) || action.visible === true) {

					let buttonClass = this.get_menu_action_class(interaction, action_id);
					if (action.classes) {
						let classes = typeof(action.classes) === 'function' ? action.classes(interaction) : action.classes;
						if (classes) {
							buttonClass = buttonClass + " " + classes;
						}
					}

					let name = typeof(action.tooltip) === 'function' ? action.tooltip(interaction) : action.tooltip;

					/*
						Tiene que ser un <a> ya que sino no funciona la opcion print

						En el attr "rel" asignamos el valor "noreferrer" para que el popup/tab que realiza la impresion, se ejecute en un thread diferente
						al que se ejecuta la la ventana padre.
						Con esto lo que logramos es que cuando se abre el dialogo de impresion del sistema, el flujo de la ventana donde correr i6 no se detenga,
						y el no exista desconexion por timeout contra el server.

						Referencia: http://stackoverflow.com/questions/39574000/how-to-open-a-new-tab-in-separate-thread-with-javascript-chrome
					*/
					let $action = $(
						'<a class="interactionMenuItem ' + buttonClass + '" href="javascript:void(0);">' +
							name +
						'</a>'
					);

					$action.attr('actionid', action_id);
					if ( typeof(action.enabled) === 'function' && !action.enabled(interaction) ) {
						$action.prop('disabled', true);
					}
					$menu.append( $action );
					if (action.is_visible_only_in_toolbar && action.is_visible_only_in_toolbar()) {
						$action.hide();
					}
					else {
						count_visible_actions++;
					}
				}
			}
		}

		if ( count_visible_actions === 0 ) {
			this.show_banner_message(_T('Conversation not have available actions.'));
			return;
		}

		$('body').append($menu);

		$menu.find('a[actionid]')
			.unbind('click')
			.bind( 'click', function()
			{
				var $this = $(this);
				var action_id = $this.attr('actionid');
				var action = actions[action_id];
				if (!action) {
					throw('InteractionGUI.prototype.show_menu_actions: Unrecognized button action "' + action_id + '"');
				}

				var execute_action = function($positionNode, $interactionNode, $parentPositionNode, popup_main_menu) {
					action.positionNode = $positionNode;
					action.parentPositionNode = $parentPositionNode;
					action.interactionNode = $interactionNode;
					action.execute(interaction, self.callbacks, popup_main_menu, $this);
				};

				if (action.submenu && !action.submenu.external) {
					action.positionNode = $this;
					action.parentPositionNode = params.$parent;
					action.interactionNode = params.$interaction_node;
					self.__show_submenu(action, interaction);
				}
				else {
					if (action.submenu && action.submenu.external) {
						execute_action($this, params.$interaction_node, params.$parent, self.menu_actions);
					}
					else {
						self.menu_actions.finalize(function() {
							self.menu_actions = null;
						});
						execute_action(params.$parent, params.$interaction_node, undefined, self.menu_actions);
					}
				}
			});

		var cloned_interaction = inConcert.Utils.cloneObject( interaction );
		this.menu_actions = new inConcert.PopupWindow({ownerId: menu_id});
		// Show popup
		this.menu_actions.show(
		{
			ownerId       : menu_id,
			eventName     : menu_id,
			interaction   : cloned_interaction,
			canReposition : true,
			container     : $menu,
			parentNode    : params.$parent,
			callbacks: {
				onOpen: function() {
					$('#' + menu_id).show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.menu_actions) {
						self.menu_actions.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
					if(self.submenu_actions){
						self.submenu_actions.finalize(function() {
							self.submenu_actions = null;
						});
					}
					$('#' + menu_id).remove();
				}
			}
		});

		//Se alinea menu de acciones
		const $btnAction = params.$parent;
		const btnActionOffset = $btnAction.offset();
		const dropdownLeft = btnActionOffset.left - ($menu.outerWidth() - $btnAction.outerWidth());
		$menu.css({left: dropdownLeft });
	};

	InteractionGUI.prototype.show_menu_actions = function( interaction, params )
	{
		var self = this;

		if ( this.menu_actions && this.menu_actions.isVisible() ) {
			this.menu_actions.finalize(function()
			{
				var data = self.menu_actions.getData();
				self.menu_actions = null;
				if ( data && data.interaction.id !== interaction.id ) {
					self.__show_menu( interaction, params );
				}
			});
		}
		else {
			this.__show_menu( interaction, params );
		}
	};

	InteractionGUI.prototype.show_submenu_actions = function(interactions, params)
	{
		let interaction;
		if ($.isArray(interactions)) {
			interaction = interactions[0];
		}
		else {
			interaction = interactions;
		}
		let actions = this.m_interaction_actions.get(interaction);
		let action = actions[params.action_type];
		action.positionNode = params.$parent;
		action.parentPositionNode = params.$parent;
		action.align = params.align;
		action.submenuNewPosition = params.submenuNewPosition;
		let callbacks = params.callbacks;
		this.__show_submenu(action, interactions, callbacks);
	};

	InteractionGUI.prototype.__update_menu_actions = function( interaction, params )
	{
		if ( !interaction || !params ) {
			return;
		}
		if ( this.menu_actions && this.menu_actions.isVisible() ) {
			var data = this.menu_actions.getData();
			if ( data && data.interaction.id === interaction.id && data.interaction.state !== interaction.state) {
				this.__show_menu( interaction, params );
			}
		}
	};

	InteractionGUI.prototype.dispose_menu_actions = function()
	{
		var self = this;
		if (this.menu_actions) {
			this.menu_actions.finalize(function() {
				self.menu_actions = null;
			});
		}
	};

	InteractionGUI.prototype.hide_menu_actions = function()
	{
		if (this.menu_actions) {
			this.menu_actions.hide();
		}
	};

	InteractionGUI.prototype.set_events = function($container)
	{
		const self = this;

		if ($container && $container.length > 0) 
		{
			let is_opening_interaction_client = {};
			const openWindow = function(interaction, $row)
			{
				// TODO: #9778 - CANY - Docked.
				let im = inConcert.app.getInteractionManager();
				let panel;
				if (self.owner.panel_name) {
					panel = inConcert.app.getMainWindow().getPanel(self.owner.panel_name);
				}
				if (!panel) {
					panel = im.getInteractionPanel(interaction);
				}
				im.openInteractionClient(interaction, $row, {
					owner     : self,
					from_app  : self.from_app,
					panel_name: self.owner.panel_name || '',
					panel     : panel,
					on_window_open : function(_/*success*/, it) {
						delete is_opening_interaction_client[it.id];
					},
					on_window_close : function(it) {
						self.on_closed_client_window(interaction, panel);
						if (typeof(self.callbacks.on_client_window_close) === 'function') {
							self.callbacks.on_client_window_close(it);
						}
					}
				});
				// ------
			};

			// click sobre la row/interaction
			let event_selector = '[int_row]';
			$container
				.off('click.openclient', event_selector)
				.on('click.openclient', event_selector, function(e) {
					e.stopPropagation();
					const $row = $(this);
					const interaction = $row.data('interaction');
					let now = new Date();
					// para evitar que el dblclick se ejecute y nos abra mas de una vez la misma ventana.
					if (is_opening_interaction_client[interaction.id]) {
						let elapsed_time_ms = now.getTime() - is_opening_interaction_client[interaction.id].click_time;
						const MAX_TIME_MS = 1500; // 1.5 segundos
						if (elapsed_time_ms <= MAX_TIME_MS) {
							return;
						}
					}
					delete is_opening_interaction_client[interaction.id];
					is_opening_interaction_client[interaction.id] = {
						click_time: now.getTime()
					};
					const $body = $("body");
					$body.trigger('click');
					const shown_fn = self.callbacks.can_open_client_window ? self.callbacks.can_open_client_window : function( _, callback ) { callback(true); };
					shown_fn(interaction, function(can_open) {
						if (!can_open) {
							return;
						}
						openWindow(interaction, $row);
					});
				});

			// popup con lista de acciones
			event_selector = '[int_menu_actions]';
			$container
				.off('click', event_selector)
				.on('click', event_selector, function(e)
				{
					var $button = $(this);
					var row_id = $button.attr('row_id');
					var $interaction_node = $('#' + row_id);
					if ( $interaction_node.length === 0 ) {
						return;
					}
					var interaction = $interaction_node.data('interaction');
					if ( !interaction ) {
						return;
					}
					self.show_menu_actions(interaction, {
						$parent: $button,
						$interaction_node: $interaction_node
					});
					e.stopPropagation();
				})
				.off('dblclick', event_selector)
				.on('dblclick', event_selector, function(e) {
					// TODO: revisar tema de bulling. Se esta ejecutando y abriendo la ventana.
					e.stopPropagation();
				});

			// globito de los comentarios!!
			event_selector = '[int_comment]';
			$container
				.off('click', event_selector)
				.on('click', event_selector, function(e)
				{
					var $bubble = $(this);
					var row_id = $bubble.attr('row_id');
					var $interaction_node = $('#' + row_id);
					if ( $interaction_node.length === 0 ) {
						return;
					}
					var interaction = $interaction_node.data('interaction');
					if ( !interaction ) {
						return;
					}
					var comments = new inConcert.CommentsPopup({ from_app: self.from_app });
					comments.show({
						interaction: interaction,
						parentNode: $bubble,
						callbacks: {
							on_comment : function( success/*, error*/ ) {
								if ( self.callbacks.on_comment ) {
									self.callbacks.on_comment(success);
								}
							},
							onOpen: function() {
								//mark interaction comments as read
								interaction.readComments(function(success) {
									if (success) {
										$bubble.removeClass('interactionCommentsUnread').addClass('interactionCommentsRead');
									}
								});
							}
						}
					});

					e.stopPropagation();
				})
				.off('dblclick', event_selector)
				.on('dblclick', event_selector, function(e) {
					// TODO: revisar tema de bulling. Se esta ejecutando y abriendo la ventana.
					e.stopPropagation();
				});
		}
	};

	InteractionGUI.prototype.get_node_tpl = function()
	{
		var node_tpl =
		// Inicio de la fila
			'<div int_row="true" id="{{ dom_interaction_id }}" original_int_id="{{ original_int_id }}" class="interactionListRow {{ interaction_class }}" node_type="interaction" belongs_to_group="{{ interaction_belongs_to_group }}" >' +

			// el waiting
				'<div class="interactionWaitingContainer" style="display:none">' +
					'<i class="interactionWaitingAnima" ></i>' +
					'<div class="interactionWaitingOverlay"></div>' +
				'</div>' +

			// las columnas

				// batch selector > visible
				'<div int_column="batch_selector" class="interactionListColumn columnBatch">' +
					'<div int_batch_selector_container="true" class="selectionTriggerContainer" {{ hidden_int_batch_selector }}>' +
						'<input int_batch_selector="true" row_id="{{ dom_interaction_id }}" class="selectionTriggerCheck" type="checkbox"/> ' +
					'</div>' +
				'</div>' +

				// action  > optional - hidden now
				'<div int_column="inbound_outgoing" class="interactionListColumn columnAction" hidden>' +
					'<img src="/inconcert/apps/agent/images/blank.png" class="actionIconType" style="display:none"/>' +
				'</div>' +

				// type > visible
				'<div int_column="type" int_column_name="' + _T('Type') + '" class="interactionListColumn columnType">' +
					'<i class="interactionType"></i>' +
				'</div>' +

				// '<div int_column="avatar" class="interactionListColumn">' +
				// 	'<div int_avatar="true" class="interactionAvatar"><img src="{{ avatar_uri }}" width="22" height="22"/></div>' +
				// '</div>' +

				// solo en supervision
				'{{ #show_column_state }}' +
				'<div int_column="state" int_column_name="' + _T('State') + '" class="interactionListColumn columnState">' +
					'<label int_state="true" row_id="{{ dom_interaction_id }}" class="interactionState">{{ interaction_state }}</label>' +
				'</div>' +
				'{{ /show_column_state }}' +

				// contacto & delivery > visible
				'<div int_column="name" int_column_name="' + _T('Name') + '" class="interactionListColumn columnName">' +
					'<div int_delivery="true" class="delivery" style="display:none;"></div>' +
					'<label int_contact_name="true" class="interactionContactName" contactID="{{ contact_id }}"> {{{ display_name }}} </label>' +
				'</div>' +

				// contact address
				'<div int_column="contact_address" int_column_name="' + _T('Address') + '" class="interactionListColumn columnContactAddress">' +
					'<label int_contact_address="true" class="interactionContactAddress"> {{ display_contact_address }} </label>' +
				'</div>' +

				// contact id
				'<div int_column="contact_id" int_column_name="' + _T('Contact ID') + '" class="interactionListColumn columnContactId" hidden>' +
					'<label int_contact_id="true" class="interactionContactAddress"> {{ display_contact_id }} </label>' +
				'</div>' +

				// campaign
				'<div int_column="campaign" int_column_name="' + _T('Campaign') + '" class="interactionListColumn columnCampaign" hidden>' +
					'<label int_campaign="true" class="interactionGroup"> {{ display_campaign }} </label>' +
				'</div>' +

				// account
				'<div int_column="account" int_column_name="' + _T('Account') + '" class="interactionListColumn columnAccount">' +
					'<label int_account="true" class="interactionAccount"> {{ display_account }} </label>' +
				'</div>' +

				// attention time
				'<div int_column="attention_time" int_column_name="' + _T('Attention Time') + '" class="interactionListColumn columnAttentionTime" style="display:none;">' +
					'<label int_attention_time="true" class="interactionAccount"> {{ display_attention_time }} </label>' +
				'</div>' +

				// solo en supervisor
				'<div int_column="owner" int_column_name="' + _T('Owner') + '" class="interactionListColumn columnOwner">' +
				'{{ #show_column_user_owner }}' +
					'<label int_owner="true" class="interactionAgent"> {{ agent_name }} </label>' +
				'{{ /show_column_user_owner }}' +
				'</div>' +

				// unread events > visible
				'<div id="{{ dom_interaction_id }}_unreadEventCounter" int_column="unread_events" class="interactionListColumn columnNumber">' +
					'<label id="{{ original_int_id }}_unreadEventCounter" int_unread_events="true" class="interactionGroupedNumber" hidden></label>' +
				'</div>' +
				
				// unanswered events > solo en supervisor
				'<div int_column="unanswered_events" class="interactionListColumn columnNumber">' +
					'<label int_answered_events="true" class="interactionGroupedNumber"></label>' +
				'</div>' +

				// abstract > visible
				'<div int_column="abstract"  int_column_name="' + _T('Content') + '"class="interactionListColumn columnAbstract">' +
					'<div int_labels_disposition="true" class="interactionLabelsContainer"></div>' +
					'<label int_abstract="true" class="interactionAbstract"> {{ info }} </label>' +
				'</div>' +

				// attachment Col > visible
				'<div int_column_name="' + _T('Attachment') + '" class="interactionListColumn columnAttach">' +
				'</div>' +

				// creation date > visible
				'<div int_column="creation_date" int_column_name="' + _T('Creation Date') + '" class="interactionListColumn columnDate creationDate">' +
					'<label int_creation_date="true" class="interactionDate"> {{ creation_date }} </label>' +
				'</div>' +

				/*  > visible
					last event update date, o sino
					last state date
					esto depende de quien me haya instanciado
				*/
				'<div int_column="last_event_date_or_state_date" int_column_name="' + _T('State Date') + '" class="interactionListColumn columnDate stateDate">' +
					'<label int_last_event_date_or_state_date="true" class="interactionDate"> {{ last_event_date_or_state_date }} </label>' +
				'</div>' +

				// comments > visible
				'<div int_column="comments" class="interactionListColumn columnComments">' +
					'<label int_comment="true" row_id="{{ dom_interaction_id }}" style="display:{{ hidden_comments }}" class="interactionComments " {{ comment_class }} "><span>{{ total_comments }}</span></label>' +
				'</div>' +

				// actions > visible (**)
				'<div int_column="actions" class="interactionListColumn columnActions">' +
					//'<div int_actions_visibles="true" class="interactionActionsVisibleButtons"></div>' +
					'<button int_menu_actions="true" row_id="{{ dom_interaction_id }}" class="menuActionButton buttonColorGray"></button>' +
				'</div>' +

				// esto hay que ver como los mostramos
				'<div int_column="attention_level" class="interactionListColumn columnAttentionLevel" hidden>' +
					'<label int_attention_level="true" class="interactionAttentionLevel"> {{ attention_level }} </label>' +
				'</div>' +

			// (**) las acciones deben ir ultimas, pero ahora las agregue arriba porque sino no puedo presionar las mismas!

			// Fin de la fila
			'</div>';

		return node_tpl;
	};

	InteractionGUI.prototype.__get_conversation_owner = function( interaction ) { return interaction.getConversationOwner() || ''; };

	InteractionGUI.prototype.__get_contact_address = function(interaction) {
		if (interaction.type === 'CHAT') {
			return '';
		}
		let display_contact_address = interaction.getDisplayContactAddress();
		if (!display_contact_address) {
			return '';
		}
		return escapeAttrib(display_contact_address);
	};

	InteractionGUI.prototype.__get_campaign_display = function(interaction) {
		return interaction.getCampaignDisplayName();
	};

	InteractionGUI.prototype.__get_account_display = function(interaction) {
		return interaction.getAccountDisplayName();
	};

	InteractionGUI.prototype.__get_contact_id = function(interaction) {
		let contact_id = ((interaction.contact && interaction.contact.length > 0) ? interaction.contact : "");
		return contact_id;
	};

	InteractionGUI.prototype.__get_display_contact_id = function(interaction) {
		return this.__get_contact_id(interaction);
	};

	InteractionGUI.prototype.__get_interaction_attention_time = function(interaction) {
		let attention_time_sec = interaction.getAttentionTime();
		let attention_time = inConcert.Utils.formatSecondsAsTime(attention_time_sec);
		return attention_time;
	};

	InteractionGUI.prototype.$build_interaction_node = function(interaction, template, params)
	{
		if (!interaction) {
			return;
		}
		params = params || {};
		// valida si tiene permisos para las batch actions (de momento solo las queued)
		var hidden_int_batch_selector = 'hidden';

		var allowed_bulk_actions = typeof(params.allowed_bulk_actions) === 'function' ? params.allowed_bulk_actions(interaction) : params.allowed_bulk_actions;

		if ( allowed_bulk_actions ) {
			hidden_int_batch_selector = '';
		}

		var totalComments = interaction.getComments() ? interaction.getComments().length : '';

		var interaction_agent = '';
		var show_column_user_owner = this.columns.user_owner ? this.columns.user_owner.show : false;
		if ( show_column_user_owner ) {
			interaction_agent = this.__get_conversation_owner( interaction );
			if ( interaction_agent === '' && interaction.state !== "CREATED") {
				show_column_user_owner = false;
			}
		}

		var belongs_to_group = '';
		if ( interaction.belongsToSomeGroup() ) {
			belongs_to_group = interaction.getGroupId();
		}

		var interaction_state;
		var show_column_state = this.columns.interaction_state ? this.columns.interaction_state.show : false;
		if ( show_column_state ) {
			var interaction_states = {
				'ABANDONED'      : _T('Abandoned State'),
				'CANCELED'       : _T('Canceled'),
				'APPOINTMENT'    : _T('Appointment'),
				'FINISHED'       : _T('Archived'),
				'LOCKED'         : interaction.isOutgoing() ? _T('Dialing') : _T('Ringing'),
				'OPEN'           : _T('Open'),
				'QUEUED'         : _T('Queued'),
				'TAKEN'          : _T('Taken'),
				'TRANSFER QUEUED': _T('Queued'),
				'WAITING'        : _T('Waiting'),
				'WRAPUP'         : _T('Wrapping Up')
			};
			var stateKey = interaction.state;
			if (interaction.isAppointment()) {
				stateKey = 'APPOINTMENT';
			}
			interaction_state = interaction_states[stateKey] || '';
			show_column_state = interaction_state.length > 0;
		}

		interaction.opened = this.is_opened_client_window(interaction);


		var normalized_int_id = inConcert.Utils.getNormalizedId(interaction.id);
		var dom_interaction_id = ( params.prefix || '' ) + normalized_int_id;

		var interaction_data = {
			original_int_id             : interaction.id,
			dom_interaction_id          : dom_interaction_id,
			interaction_class           : this.getChannelClass(interaction),
			interaction_belongs_to_group: belongs_to_group,
			show_column_state           : show_column_state,
			interaction_state           : interaction_state,
			group                       : interaction.campaign,
			account                     : interaction.account,
			attention_level             : interaction.getAttentionLevelDisplayName(),
			hidden_comments             : totalComments ? '' : 'none',
			comments_class              : this.getCommentClass(interaction),
			total_comments              : totalComments,
			contact_id                  : this.__get_contact_id(interaction),
			display_name                : interaction.getDisplayName(),
			display_contact_address     : this.__get_contact_address(interaction),
			creation_date               : this.__get_interaction_creation_date(interaction),
			campaign                    : interaction.campaign,

			display_campaign            : this.__get_campaign_display(interaction),
			display_account             : this.__get_account_display(interaction),
			display_contact_id          : this.__get_display_contact_id(interaction),
			display_attention_time      : this.__get_interaction_attention_time(interaction),

			info                        : escapeAttrib(interaction.getExtendedInfo()),
			show_column_user_owner      : show_column_user_owner,
			agent_name                  : interaction_agent,
			actions                     : !interaction.isFinished(), // interaction actions apply only to non-finished.
			owner_and_state             : !interaction.isFinished(),
			avatar_uri                  : inConcert.app.getBaseUrl() + interaction.getAvatarURI(),
			hidden_int_batch_selector   : hidden_int_batch_selector
		};

		interaction_data.last_event_date_or_state_date = this.__get_interaction_last_date(interaction);

		template = template || this.get_node_tpl();
		var row_html = Mustache.render(template, interaction_data);
		var $interaction = $(row_html);
		$interaction.data('interaction', interaction);

		//Update unread and unanswered messages
		this.update_interaction_unanswered_messages(interaction, $interaction);
		this.update_interaction_unread_messages(interaction, $interaction);
		
		if (!allowed_bulk_actions) {
			$interaction.find('[int_batch_selector_container]').remove();
		}

		if (params.columns) {
			params.columns.forEach(column => {
				let selector = format('[int_column="{0}"]', column.id);
				const $cell = $interaction.find(selector);
				if ($cell.length > 0) {
					if (column.hide !== undefined && (
								column.hide === true ||
								(typeof(column.hide) === "function" && column.hide(interaction) === true)
							)
						)
					{
						$cell.css('display', 'none');
					}
					else {
						$cell.css('display', '');
					}
				}
			});
		}

		// Save interaction node by id.
		this.m_current_handled_nodes[interaction.id] = $interaction;

		this.calculate_interaction_class(interaction);
		this.update_labels_and_disposition(interaction, $interaction);

		return $interaction;
	};
}());