/*jshint quotmark:false, esversion: 11*/
/* global jQuery */

/*!
 * Plug-In				:	jQuery ToolTip
 * Version				:	1.0
 * Autor					:	Federico Ramos
 * Descripcion		:	Crea un dropdown que permite selecciones multiples a partir de un div determinado y una lista de opciones
 
 * ----------------------------------------------------------------------------------------------------------
 *
 * Sintaxis
 * ---------
 * HTML		:	<div id="miMultiselect" />
 * 
 * JS		:	var dropdownOptions = [{id: 1, name: "option 1"},{id: 2, name: "option 2"},{id: 3, name: "option 3"}];
 *              $('#miMultiselect').multipleSelectDropdown(dropdownOptions);
 *              $('#miMultiselect').getSelectedItems();
*/

(function($) {
    let translate;
    
    $.fn.selectItemsFromArray = function(itemsArray) {
        var $dropdown = $(this);
    
        itemsArray.forEach(function(item) {
            if (item.id === 'selectAll') {
                $dropdown.find('#selectAll').prop('checked', true);
            } else {
                var $option = $dropdown.find('.option[value="' + item.name + '"]');
                $option.prop('checked', true);
            }
        });
    
        updateButtonText($dropdown);
    };
    
    $.fn.getSelectedItems = function() {
        var selectedItems = [];

        this.each(function() {
            var $dropdown = $(this);

            var selectAllChecked = $dropdown.find('#selectAll').prop('checked');
            if (selectAllChecked) {
                selectedItems.push({ id: 'selectAll', name: 'Seleccionar todos' });
            }

            $dropdown.find('.option:checked').each(function() {
                var $selectedOption = $(this);
                var itemId = $selectedOption.attr('id') || $selectedOption.val();
                var itemName = $selectedOption.closest('label').text().trim();

                selectedItems.push({ id: itemId, name: itemName });
            });
        });

        return selectedItems;
    };

    // TODO: Revisar carga de elementos async por posible deslogueo del agente
    $.fn.multipleSelectDropdown = function(options, translateFunction) {
        translate = translateFunction;
        return this.each(function() {
            var $dropdown = $(this);
            var $btn = $('<button class="dropbtn form-control" id="dropbtn">'+translate('Select Options')+'</button>');
            var $dropdownContent = $('<div class="dropdown-content" id="dropdown-content"></div>');

            var dropdownOptions = options || [];
            var dropdownHTML = '';

            dropdownHTML += '<input type="text" class="search-input form-control" placeholder='+translate('Search')+'...>';
            dropdownHTML += '<div class="form-check"><label class="form-check-label"><input class="form-check-input" type="checkbox" id="selectAll"><strong>'+translate('Select All')+'</strong></label></div>';
            dropdownHTML += '<div class="options-list">';

            dropdownOptions.forEach(function(option) {
                dropdownHTML += '<div class="form-check"><label class="form-check-label"><input type="checkbox" class="option" id="' + option.id + '" value="' + option.name + '">' + option.name + '</label></div>';
            });

            dropdownHTML += '</div>';

            $dropdownContent.html(dropdownHTML);

            $dropdown.append($btn).append($dropdownContent);

            $dropdownContent
                .off('keyup.dropdown', '.search-input')
                .on('keyup.dropdown', '.search-input', function() {
                    var searchText = $(this).val().toLowerCase();
                    $dropdownContent.find('.form-check-label').each(function() {
                        var optionText = $(this).text().toLowerCase();
                        var $option = $(this);
                        if (optionText.indexOf(searchText) > -1) {
                            $option.closest('.form-check').show();
                        } else {
                            $option.closest('.form-check').hide();
                        }
                    });
                });

            $dropdownContent
                .off('change.dropdown', '#selectAll')
                .on('change.dropdown', '#selectAll', function() {
                    $dropdownContent.find('.option').prop('checked', $(this).prop('checked'));
                    updateButtonText($dropdown);
                });

            $btn
                .off('click.dropdown') 
                .on('click.dropdown', function() {
                    $dropdownContent.toggle();
                });

            $dropdownContent
                .off('change.dropdown', '.option')
                .on('change.dropdown', '.option', function() {
                    var selectedOptions = $dropdownContent.find('.option:checked').map(function() {
                        return $(this).val();
                    }).get();
                    $btn.text(selectedOptions.length > 0 ? selectedOptions.join(', ') : translate('Select Options'));

                    var allOptions = $dropdownContent.find('.option');
                    var allOptionsChecked = allOptions.length === $dropdownContent.find('.option:checked').length;
                    $dropdownContent.find('#selectAll').prop('checked', allOptionsChecked);
                    if (allOptionsChecked) {
                        $btn.text(translate('All Selected'));
                    }
                });

            $(document)
                .off('click.dropdown')
                .on('click.dropdown', function(event) {
                    if (!$dropdown.is(event.target) && !$dropdown.has(event.target).length) {
                        $dropdownContent.hide();
                    }
                });
        });
    };

    function updateButtonText($dropdown) {
        var $btn = $dropdown.find('.dropbtn');
        var selectedOptions = $dropdown.find('.option:checked').map(function() {
            return $(this).val();
        }).get();
        $btn.text(selectedOptions.length > 0 ? selectedOptions.join(', ') : translate('Select Options'));

        var selectAllChecked = $dropdown.find('#selectAll').prop('checked');
        if (selectAllChecked) {
            $btn.text(translate('All Selected'));
        }
    }
})(jQuery);
