/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, -W014, esversion: 11 */

/* global $, inConcert, GoldenLayout */

if (!this.inConcert) {
	this.inConcert = {};
}

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	function LayoutDesign (params)
	{
		// valida que exista el plugin GoldenLayout
		if (typeof(GoldenLayout) === "undefined") {
			throw("GoldenLayout plugin does not exist");
		}
		this.m_layout = null;
		this.m_current_layout_config = null;

		this.owner = params.owner;
		this.parent = params.parent;
		this.container_id = params.container_id;

		this.id_tabs                     = this.container_id + "_tabs";
		this.id_closed_tabs              = this.container_id + "_closed_tabs";
		this.id_closed_tabs_content      = this.container_id + "_closed_tabs_content";
		// IDS de los tabs
		this.id_main_tab                 = this.container_id + "_main_tab_content";
		this.id_conversations_tab        = this.container_id + "_conversations_tab_content";
		this.id_context_info_tab         = this.container_id + "_context_info_tab_content";
		this.id_contactform_tab          = this.container_id + "_contactform_tab_content";
		this.id_cc_integration_tab       = this.container_id + "_cc_integration_tab_content";
		this.id_canned_tab               = this.container_id + "_canned_tab_content";
		this.id_browsing_tab             = this.container_id + "_browsing_tab_content";
		this.id_qna_tab                  = this.container_id + "_qna_tab_content";
		this.id_helpdesk_integration_tab = this.container_id + "_helpdesk_integration_tab_content";
		this.id_mas_integration_tab      = this.container_id + "_mas_integration_tab_content";
		this.id_channel_selector         = this.container_id + "_channel_selector";
		this.id_assistance_tab           = this.container_id + "_assistance_tab_content";

		this.m_is_readonly = false;

		this.m_tabs = {
			// MAIN
			main:{
				tab_container_id: this.id_main_tab,
				id: 'main',
				translate: {
					title: _T("Conversation"),
					tooltip: _T("Conversation")
				},
				$layout_tab: null,
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Conversation");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// CC INTEGRATION
			client_sidebar_cc_integration_tab: {
				tab_container_id: this.id_cc_integration_tab,
				$layout_tab: null,
				id: 'client_sidebar_cc_integration_tab',
				translate: {
					title: _T("Integration"),
					tooltip: _T("Integration")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Integration");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// MAS INTEGRATION
			client_sidebar_mas_integration_tab: {
				tab_container_id: this.id_mas_integration_tab,
				$layout_tab: null,
				id: 'client_sidebar_mas_integration_tab',
				translate: {
					title: _T("M&S Integration"),
					tooltip: _T("Integration with Marketing & Sales")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("M&S Integration");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// HELPDESK INTEGRATION
			client_sidebar_helpdesk_integration_tab: {
				tab_container_id: this.id_helpdesk_integration_tab,
				$layout_tab: null,
				id: 'client_sidebar_helpdesk_integration_tab',
				translate: {
					title: _T("HelpDesk Integration"),
					tooltip: _T("Integration with HelpDesk")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("HelpDesk Integration");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// INFORMATION
			client_sidebar_context_tab: {
				tab_container_id: this.id_context_info_tab,
				$layout_tab: null,
				id: 'client_sidebar_context_tab',
				translate: {
					title: _T("Context"),
					tooltip: _T("Context")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Context");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// PREVIOUS CONVERSATIONS
			client_sidebar_conversations_tab: {
				tab_container_id: this.id_conversations_tab,
				$layout_tab: null,
				id: 'client_sidebar_conversations_tab',
				translate: {
					title: _T("Historical"),
					tooltip: _T("Previous Conversations")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Previous Conversations");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// WEB HISTORY
			client_sidebar_browsing_tab: {
				tab_container_id: this.id_browsing_tab,
				$layout_tab: null,
				id: 'client_sidebar_browsing_tab',
				translate: {
					title: _T("Web-Historical"),
					tooltip: _T("Navigation History")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Navigation History");
				},
				get_html_help_content_tab: function() {
					return _T("Used only in Web Chat");
				}
			},
			// CANNED
			client_sidebar_canned_tab: {
				tab_container_id: this.id_canned_tab,
				$layout_tab: null,
				id: 'client_sidebar_canned_tab',
				translate: {
					title: _T("Canned"),
					tooltip: _T("Canned")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Canned");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// QNA
			client_sidebar_qna_maker_tab: {
				tab_container_id: this.id_qna_tab,
				$layout_tab: null,
				id: 'client_sidebar_qna_maker_tab',
				translate: {
					title: _T("QnA Maker"),
					tooltip: _T("QnA Maker")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("QnA Maker");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			},
			// CONTACT FORM
			client_sidebar_contactform_tab: {
				tab_container_id: this.id_contactform_tab,
				$layout_tab: null,
				id: 'client_sidebar_contactform_tab',
				translate: {
					title: _T("Contact Form"),
					tooltip: _T("Contact Form")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Contact Form");
				},
				get_html_help_content_tab: function() {
					return _T("Used only in Contact Form");
				}
			},
            // BOT ASSISTANCE
			client_sidebar_assistance_tab: {
				tab_container_id: this.id_assistance_tab,
				$layout_tab: null,
				id: 'client_sidebar_assistance_tab',
				translate: {
					title: _T("Assistance"),
					tooltip: _T("Assistance")
				},
				it_can_be_shown: function( callback ) {
					callback( true );
				},
				get_html_content_tab: function() {
					return _T("Assistance");
				},
				get_html_help_content_tab: function() {
					return "";
				}
			}
		};

		this.__load_dymaic_tabs();
	}

	if (typeof(inConcert.LayoutDesign) !== "function") {
		inConcert.LayoutDesign = LayoutDesign;
	}

	LayoutDesign.prototype.initialize = function()
	{
		this.__create_markup();
		this.__initialize_layout();
	};

	LayoutDesign.prototype.dispose = function()
	{
		if (this.m_layout) {
			this.m_layout.destroy();
			const $container = $("#" + this.container_id);
			$container.html("");
		}
	};

	LayoutDesign.prototype.__load_dymaic_tabs = function()
	{
		this.dynamic_integrations = [];

		let widgets = inConcert.app.get_vcc_all_widgets_integrations();
		if (widgets.length === 0) {
			return;
		}

		widgets.forEach(widget => {
			let widget_id = widget.Id.toUpperCase();
			let id = FORMAT("client_sidebar_{0}_integration_tab", widget_id);
			let display_name = widget.DisplayName || widget.Name;
			let text = FORMAT(_T("{0} Widget"), display_name);
			let dynamic_tab = {
				widget_id: widget_id,
				name: widget.Name,
				is_dynamic: true,
				tab_container_id: FORMAT(this.container_id + "_{0}_integration_tab_content", widget_id),
				$layout_tab: null,
				id: id,
				translate: {
					title: text,
					tooltip: text
				},
				it_can_be_shown: function(callback) {
					callback(true);
				},
				get_html_content_tab: function() {
					return text;
				},
				get_html_help_content_tab: function() {
					return "";
				}
			};
			this.dynamic_integrations.push(dynamic_tab);
		});

		for (let i = 0; i < this.dynamic_integrations.length; i++) {
			let dynamic_tab = this.dynamic_integrations[i];
			this.m_tabs[dynamic_tab.id] = dynamic_tab;
		}
	};

	LayoutDesign.prototype.__create_markup = function()
	{
		const self = this;
		var get_html_all_tabs_content = function() {
			let html_all_tabs = "";
			for (let id in self.m_tabs) {
				if (self.m_tabs.hasOwnProperty(id)) {
					const tab = self.m_tabs[id];
					const get_html_content_tab = tab.get_html_content_tab;
					const get_html_help_content_tab = tab.get_html_help_content_tab;
					let html_tab = "" +
						"<div id='" + tab.tab_container_id + "'>" +
							"<h6 class='tab-title-description'>" + get_html_content_tab() + "</h6>" +
							"<span class='tab-subtitle-description'>" + get_html_help_content_tab() + "</span>" +
						"</div>";
					html_all_tabs = html_all_tabs + html_tab;
				}
			}
			return html_all_tabs;
		};

/*
NO borrar esto!! Sirve para hacer pruebas.
Hay que descomentar y descomentar (*)
Tambien hay que comentar en el css "administrationMaster.css" y en "masterClientSidebar.css" las clases referentes al layout
*/
// let css_html = "".concat(
// 	"<style type='text/css'>",
// 		"",
// 		"",
// 		"#", this.id_tabs, "{",
// 			"width: 80%;",
// 			"height: 100%;",
// 			"position:absolute;",
// 			"top: 0;",
// 			"left: 20%;",
// 			"box-shadow: -3px 0px 9px 0px rgba( 0, 0, 0, 0.4 );",
// 		"}",

// 	"</style>"
// );
//---------------------------------------

		const $html = $("".concat(
//css_html, // (*) !! Descomentar para pruebas
			"<div class='card'>",
				"<div class='card-block'>",
					"<div class='row'>",
						"<div class='col-3'>",
							"<div class='form-group'>",
								"<label for='", this.id_channel_selector, "' class='required'>", _T("Channel"), _T(':'), "</label>",
								"<select id='", this.id_channel_selector, "' class='form-control' ></select>",
								"<i id='", this.id_channel_selector, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='card'>",
				"<div class='card-header'>",
						"<h5>", _T("Available tabs"), "</h5>",
				"</div>",
				"<div class='card-block available-tabs-container'>",
					"<div class='row'>",
						"<div class='col-12'>",
							// UL que contiene los Nombres/IDs de los TABs disponibles que han sido cerrados del GOLDEN LAYOUT
							"<ul id='", this.id_closed_tabs, "' class='formClientsLayoutClosedTabs'></ul>",
						"</div>",
					"</div>",
					"<div class='row'>",
						"<div class='col-12'>",
							// DIV que contiene los TABs disponibles del GOLDEN LAYOUT
							"<div id='", this.id_tabs, "' class='formClientsLayoutTabs'>",
								get_html_all_tabs_content(),
							"</div>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			// DIV que contiene el contenido (MARKUP) de los TABs disponibles que han sido cerrados del GOLDEN LAYOUT
			"<div id='", this.id_closed_tabs_content, "' style='display:none;' class='card'></div>"
		));

		const $container = $("#" + this.container_id);
		$container.html($html);
	};

	LayoutDesign.prototype.__initialize_layout = function()
	{
		const self = this;

		// // 1 - Recorro la lista de Tabs disponibles en la sidebar y evaluo si son visibles.
		// //		Esto como fue siempre, dependiendo del estado de la interaccion, del tipo de interaccion, etc.
		let visibles_tabs = {};

		var it_can_be_shown = function (tab, tab_id) {
			tab.it_can_be_shown(function(is_visible) {
				if (is_visible) {
					visibles_tabs[tab_id] = {
						is_visible: true,
						tab_config: tab
					};
				}
			});
		};

		for (let tab_id in this.m_tabs) {
			if (this.m_tabs.hasOwnProperty(tab_id)) {
				let tab = this.m_tabs[tab_id];
				it_can_be_shown(tab, tab_id);
			}
		}

		// (*) Funcion Recursiva que recorre el tree de configuracion de GoldenLayout.
		//		La misma realiza un filtro de la lista Tabs disponibles que se obtuvieron en el punto 1)
		//		y las elimina, segun la visibilidad personalizada que se obtuvo en el punto 2).
		var filter = function(content = []) {
			let tabs_in_stack = 0;
			let len = content.length - 1;
			for (let i = len; i >= 0; i--) {
				let item = content[i];
				if (item.type === "component") {
				// si es un componente es decir es un TAB
					let is_visible = visibles_tabs[item.id] ? visibles_tabs[item.id].is_visible: false;
					if (!is_visible) {
						// Quito el TAB (componente) si:
						//   El TAB es visible segun la validacion de la sidebar 1), y el mismo NO se encuentra visible dentro de la configuracion personalizada.
						content.splice(i, 1);
					}
					else {
						// Si el mismo es visible, marco como que existe un tab en el stack y lo elimino de la lista temporal de tabs "visibles_tabs".
						tabs_in_stack++;
						delete visibles_tabs[item.id];
					}
				}
				else {
				// Si NO es un componente (TAB), valido si es un stack (contenedor de TABs)
					let is_stack = false;
					if (item.type === "stack") {
						is_stack = true;
					}
					// Invoco a la funcion recursiva para que busque TABs dentro del Stack
					let total_tabs_in_stack = filter(item.content);
					if (is_stack && total_tabs_in_stack === 0) {
						// Si es un stack y el mismo NO contienen ningun TAB, lo debeo de eliminar de la configuracion.
						// quito el contenedor de TABs (stack)
						content.splice(i, 1);
					}
				}
			}
			return tabs_in_stack;
		};

		// 2 - Retorna la configuracion, del layout:
		//	a - Primero si el usuario tiene configuracion personalizada.
		//	b - Si el vcc tiene configuracion personalizada.
		//	c - Si no tiene ninguna de las dos, se obtiene la configuracion por defecto.
		const config = this.__load_layout_config();
		// 3 - Ahora realizo un filtro (*)
		filter(config.content);

		const $layout_container = $("#" + this.id_tabs);
		//console.log("--> LayoutDesign.initializeLayout . config: " + JSON.stringify(config));
		this.m_layout = new GoldenLayout(config, $layout_container);

		/*
			Funcion encargada de agregar en la lista de Tabs disponibles, pero que NO estan visibles en el layout.
		*/
		var addMenuItem = function(tab_id)
		{
			const tab_config = self.m_tabs[tab_id];
			if (!tab_config) {
				console.log("LayoutDesign.__initialize_layout.addMenuItem - Tab does not exist. tab_id: " + tab_id);
				//throw("error tab config does not exist. " + tab_id);
				return;
			}
			const tab_item_config = self.__get_layout_default_tab_config(tab_id);
			if (!tab_item_config) {
				return;
			}
			const tab_container_id = typeof(tab_config.tab_container_id) === "function" ? tab_config.tab_container_id() : tab_config.tab_container_id;

			// Aca meto el contenido del TAB.
			const $tab_info = $("#" + tab_container_id); //$tab_content.find("[id='" + tab_container_id + "']");
			$tab_info.hide();
			$tab_info.detach();
			$("#" + self.id_closed_tabs_content).append($tab_info);

			// Ahora meto en la lista de tabs disponibles para que se pueda realizar Drag & Drop.
			const $tab_element = $("<li class='drag_tabs_item' tab_id='" + tab_container_id + "'>" + tab_item_config.tooltip + "</li>");
			$("#" + self.id_closed_tabs).append($tab_element);

			// Por ultimo indico creo el drag source en el GoldenLayout.
			if (!self.m_is_readonly) {
				self.m_layout.createDragSource($tab_element, tab_item_config);
			}
		};

		// 4 - Ahora recorro la lista de tabs temporales que se obtuvieron de la validacion de la sidebar en el punto 1)
		//  y que los mismos NO tiene que ser visibles porque se filtraron en el punto 3).
		//  Entonces ahora paso a dejarlo disponibles en la lista para que estos Tabs puedan ser visualizados, mediante Drag & Drop.
		for (let tab_id in visibles_tabs) {
			if (visibles_tabs.hasOwnProperty(tab_id)) {
				if (visibles_tabs[tab_id].is_visible) {
					addMenuItem(tab_id);
				}
			}
		 }

		// registramos el componente
		this.m_layout.registerComponent('testComponent', function(container, componentState)
		{
			const tab_config = self.m_tabs[componentState.id];
			if (!tab_config) {
				console.log("LayoutDesign.__initialize_layout.registerComponent - Tab does not exist. tab_id: " + componentState.id);
				//throw("error tab config does not exist. " + componentState.id);
				return;
			}
			//console.warn('a --> registerComponent container.getState(): ' + container.getState());
			//console.warn('b --> registerComponent container.hide(): ' + container.hide());
			const tab_container_id = tab_config.tab_container_id;
			const $tab_container = $("#" + tab_container_id);

			if ($tab_container) {
				$tab_container.show();
				// aca inserto el div del contenido de la solapa de la sidebar o el main del cliente, dentro del html del plugin GoldenLayout
				container.getElement().html($tab_container);
				// aca seteo el jQuery element a nuestra estructura de tabs.
//				self.setLayoutComponent(componentState.id, container);
			}
		});

		// cuando se crea el TAB, hacemos esta pisada, para poder tener el evento, cuando se selecciona un TAB,
		// ya que el GoldenLayout no tiene el evento onselecttab o algo asi.
		this.m_layout.on('tabCreated', function(tab)
		{
			tab.element.attr("title", tab.contentItem.config.tooltip);
			const tab_id = tab.contentItem.config.id;
			// aca seteo el jQuery element a nuestra estructura de tabs.
//			self.setLayoutTab(tab_id, tab.element);

			const tab_config = self.m_tabs[tab_id];
			if (!tab_config) {
				throw("error tab config does not exist. " + tab_id);
			}
			const tab_container_id = typeof(tab_config.tab_container_id) === "function" ? tab_config.tab_container_id() : tab_config.tab_container_id;

			const $closed_tab_name = $("#" + self.id_closed_tabs).find("[tab_id='" + tab_container_id + "']");
			if ($closed_tab_name.length > 0) {
				$closed_tab_name.remove();
				if($(".drag_tabs_item").length === 0 ){
					$(".formClientsLayoutTabsContainer").removeClass("mt-4");
					$(".formClientsLayoutTabsContainer").addClass("mt-0");
				} else {
					$(".formClientsLayoutTabsContainer").removeClass("mt-0");
					$(".formClientsLayoutTabsContainer").addClass("mt-4");
				}
			}
			tab.closeElement
				.off('click') //unbind the current click handler
				.click(function(){
					//add your own
					if (true) {
						addMenuItem(tab_id);
						tab.contentItem.remove();
						if($(".drag_tabs_item").length === 0 ){
							$(".formClientsLayoutTabsContainer").removeClass("mt-4");
							$(".formClientsLayoutTabsContainer").addClass("mt-0");
						} else {
							$(".formClientsLayoutTabsContainer").removeClass("mt-0");
							$(".formClientsLayoutTabsContainer").addClass("mt-4");
						}
					}
				});
		});

		//let is_created = false;
		this.m_layout.on('stackCreated', function(stack)
		{
			stack
				.header
				.controlsContainer
				.find( '.lm_close' ) //get the close icon
				.off( 'click' ) //unbind the current click handler
				.click(function() {
					//window.alert("stack closed todo implement ?")
					//add your own
					//if( confirm( 'really close this?' ) ) {
						//stack.remove();
					//}
				});
		});

		// iniciamos la instancia del layout.
		this.m_layout.init();
	};

	/*
	*/
	LayoutDesign.prototype.__load_layout_config = function()
	{
		let current_layout_config;
		if (this.m_current_layout_config) {
			let channel = this.m_current_layout_config.Id;
			let layout_config = this.m_current_layout_config[channel];
			if (layout_config) {
				current_layout_config = layout_config;
			}
		}

		if (!current_layout_config) {
			current_layout_config = this.get_layout_default_config();
		}

		let translated_layout = this.translate_config(current_layout_config);
		if (!this.m_is_readonly) {
			return translated_layout;
		}
	
		let readonly_config = this.set_readonly_config(translated_layout);
		return readonly_config;
	};

	LayoutDesign.prototype.set_readonly_config = function(layout_config)
	{
		if (!layout_config) {
			return;
		}

		if (!this.m_is_readonly) {
			return layout_config;
		}

		var readonly_tabs = function(content = []) {
			let len = content.length - 1;
			for (let i = len; i >= 0; i--) {
				let item = content[i];
				if (item.type === "component") {
					item.isClosable = false;
				}
				else {
					// Invoco a la funcion recursiva para que busque TABs dentro del Stack
					readonly_tabs(item.content);
				}
			}
		};

		readonly_tabs(layout_config.content);

		if (layout_config.settings) {
			layout_config.settings.reorderEnabled = false;
			layout_config.settings.selectionEnabled = false;
			layout_config.settings.showMaximiseIcon = false;
		}

		return layout_config;
	};

	LayoutDesign.prototype.load_translated_layout_default_config = function()
	{
		let layout_config = this.get_layout_default_config();
		return this.translate_config(layout_config);
	};

	LayoutDesign.prototype.load_translated_layout_default_tabs_config = function()
	{
		let layout_tabs_config = this.get_layout_default_tabs_config();
		let config = {
			content: layout_tabs_config
		};
		let translated_config= this.translate_config(config);
		return translated_config.content;
	};

	LayoutDesign.prototype.translate_config = function(layout_config)
	{
		const self = this;

		if (!layout_config) {
			return;
		}
		var translate_tabs = function(content = []) {
			let len = content.length - 1;
			for (let i = len; i >= 0; i--) {
				let item = content[i];
				if (item.type === "component") {
					const tab_id = item.id;
					const tab_config = self.m_tabs[tab_id];
					if (tab_config) {
						if (tab_config.translate) {
							item.title = tab_config.translate.title || item.title;
							item.tooltip = tab_config.translate.tooltip || item.tooltip;
						}
					}
					else {
						console.log("LayoutDesign.translate_config - Tab does not exist. tab_id: " + tab_id);
						//throw("error tab config does not exist. " + tab_id);
					}
				}
				else {
				// Si NO es un componente (TAB), valido si es un stack (contenedor de TABs)
					let is_stack = false;
					if (item.type === "stack") {
						is_stack = true;
					}
					// Invoco a la funcion recursiva para que busque TABs dentro del Stack
					translate_tabs(item.content);
				}
			}
		};

		translate_tabs(layout_config.content);
		if (layout_config.labels) {
			let default_layout = this.get_layout_default_config();
			if (default_layout.labels) {
				layout_config.labels = default_layout.labels;
			}
		}

		return layout_config;
	};

	LayoutDesign.prototype.get_layout_default_config = function()
	{
		const default_config = {
			settings: {
				"hasHeaders": true,
				"constrainDragToContainer": true,
				"reorderEnabled": true,
				"selectionEnabled": true,
				"blockedPopoutsThrowError": true,
				"closePopoutsOnUnload": true,
				"showPopoutIcon": false,
				"showMaximiseIcon": true
			},
			dimensions: {
				headerHeight : 36,
				minItemHeight: 120, //alto minimo en px al resizear
				minItemWidth : 280, //ancho minimo en px al resizear
				borderWidth: 10,
				//dragProxyWidth: 300,
				//dragProxyHeight: 200
			},
			labels: {
				maximise   : _T("Maximise"),
				minimise   : _T("Minimise"),
				tabDropdown: _T("Show more")
			},
			content: [
				{
					type: 'row',
					content:[
						{
							type: 'column',
							width: 60,
							content:[
								{
									type: 'component',
									componentName: 'testComponent',
									id: 'main',
									title: "CONVERSATION",
									tooltip: "CONVERSATION",
									componentState: { id: 'main' },
									isClosable: false,
									//activeItemIndex: 0
								}
							]
						},
						{
							type: 'stack',
							content: this.get_layout_default_tabs_config()
						}
					]
				}
			]
		};

		return default_config;
	};

	/*
		Los Tabs que NO se pueden cerrar "isClosable: false", tienen que ir primeros.
	*/
	LayoutDesign.prototype.get_layout_default_tabs_config = function()
	{
		let default_tabs_config = [
			// CC INTEGRATION
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_cc_integration_tab',
				title: "INTEGRATION",
				tooltip: "INTEGRATION",
				componentState: { id: 'client_sidebar_cc_integration_tab' },
				isClosable: false
			},
			// MAS INTEGRATION
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_mas_integration_tab',
				title: "M&S INTEGRATION",
				tooltip: "M&S INTEGRATION",
				componentState: { id: 'client_sidebar_mas_integration_tab' },
				isClosable: false
			},
			// HELPDESK INTEGRATION
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_helpdesk_integration_tab',
				title: "HELPDESK INTEGRATION",
				tooltip: "HELPDESK INTEGRATION",
				componentState: { id: 'client_sidebar_helpdesk_integration_tab' },
				isClosable: false
			},
			// INFORMATION
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_context_tab',
				title: "CONTEXT INFO",
				tooltip: "CONTEXT INFO",
				componentState: { id: 'client_sidebar_context_tab' },
				isClosable: true
			},
			// PREVIOUS CONVERSATIONS
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_conversations_tab',
				title: "PREVIOUS CONVERSATIONS",
				tooltip: "PREVIOUS CONVERSATIONS",
				componentState: { id: 'client_sidebar_conversations_tab' },
				isClosable: true
			},
			// WEB HISTORY
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_browsing_tab',
				title: "HISTORY",
				tooltip: "HISTORY",
				componentState: { id: 'client_sidebar_browsing_tab' },
				isClosable: true
			},
			// CANNED
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_canned_tab',
				title: "CANNED",
				tooltip: "CANNED",
				componentState: { id: 'client_sidebar_canned_tab' },
				isClosable: true
			},
			// QNA
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_qna_maker_tab',
				title: "QNA MAKER",
				tooltip: "QNA MAKER",
				componentState: { id: 'client_sidebar_qna_maker_tab' },
				isClosable: true
			},
			// CONTACT FORM
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_contactform_tab',
				title: "CONTACT FORM",
				tooltip: "CONTACT FORM",
				componentState: { id: 'client_sidebar_contactform_tab' },
				isClosable: true
			},
			// ASSISTANCE CONVERSATIONS
			{
				type: 'component',
				componentName: 'testComponent',
				id: 'client_sidebar_assistance_tab',
				title: "ASSISTANCE",
				tooltip: "ASSISTANCE",
				componentState: { id: 'client_sidebar_assistance_tab' },
				isClosable: true
			}
		];

		let position;
		for (let i = default_tabs_config.length -1; i >= 0; i--) {
			let default_tab = default_tabs_config[i];
			if (!default_tab.isClosable) {
				position = i;
				break;
			}
		}

		if (position) {
			//let dynamic_tabs = [];
			for (let id in this.m_tabs) {
				if (this.m_tabs.hasOwnProperty(id)) {
					const tab = this.m_tabs[id];
					if (tab.is_dynamic) {
						let tooltip = tab.translate.tooltip.toUpperCase();
						let tabs_config = {
							type: 'component',
							componentName: 'testComponent',
							id: id, //'client_sidebar_mas_integration_tab',
							title: tooltip, //"M&S INTEGRATION",
							tooltip: tooltip, //"M&S INTEGRATION",
							componentState: {
								id: id // 'client_sidebar_mas_integration_tab'
							},
							isClosable: false,
							is_dynamic: true,
							name: tab.name,
							widget_id: tab.widget_id
						};
						position++;
						default_tabs_config.splice(position, 0, tabs_config);
					}
				}
			}
		}
		return default_tabs_config;
	};

	LayoutDesign.prototype.__get_layout_default_tab_config = function(tab_id) {
		let default_tabs_config = this.load_translated_layout_default_tabs_config();

		for (let i = 0; i < default_tabs_config.length; i++) {
			let tab_config = default_tabs_config[i];
			if (tab_config.id === tab_id) {
				return tab_config;
			}
		}
	};

	LayoutDesign.prototype.set_layout = function(layout_config)
	{
		this.m_current_layout_config = layout_config || null;
	};

	LayoutDesign.prototype.set_readonly = function(is_readonly)
	{
		this.m_is_readonly = is_readonly;
	};

	LayoutDesign.prototype.readonly = $.noop;
	LayoutDesign.prototype.edit = $.noop;
	LayoutDesign.prototype.empty_controls = $.noop;
	LayoutDesign.prototype.set_events = $.noop;
	LayoutDesign.prototype.on_show = $.noop;

	LayoutDesign.prototype.hide_messages = function()
	{
		if (this.parent.frm && this.parent.frm.hide_messages) {
			this.parent.frm.hide_messages(); // oculto todos los errores de los campos
		}
	};

	LayoutDesign.prototype.__validate_field = function(/*$ctrl*/)
	{
		return {success: true};
	};

	LayoutDesign.prototype.validate = function()
	{
		let layout_info = {
			success: false,
			layout: null,
			layout_id: null,
			$ctrl: null,
			error_message: null
		};

		if (!this.m_layout) {
			return layout_info;
		}
		this.hide_messages(); // oculto todos los errores de los campos

		const $channel_selector = $("#" + this.id_channel_selector);

		const result = this.__validate_field($channel_selector);
		if (!result.success) {
			layout_info.error_message = result.error_message;
			layout_info.$ctrl = result.$ctrl;
			return layout_info;
		}

		layout_info.success = true;
		layout_info.layout_id = $channel_selector.val();
		layout_info.layout = this.m_layout.toConfig();

		return layout_info;
	};

	LayoutDesign.prototype.make_layouts_array_to_map = function(layouts)
	{
		return inConcert.Utils.arrayToMap(layouts || [], 
			function(layout) {
				return layout;
			},
			function(layout) {
				return layout.Id;
			}
		);
	};

	LayoutDesign.prototype.set_availables_layouts_types = function(all_current_layouts)
	{
		let all_layouts = this.make_layouts_array_to_map(all_current_layouts);
		this.__fill_layouts_types(all_layouts);

		const $channel_selector = $("#" + this.id_channel_selector);
		let channel = "";
		let is_readonly = false;
		if (this.m_current_layout_config) {
			//channel = this.m_current_layout_config.__layout_id;
			channel = this.m_current_layout_config.Id;
			is_readonly = true;
		}
		$channel_selector.val(channel);
		this.readonly_control($channel_selector, is_readonly, true);
	};

	/*
		Esta funcion es idem a como se encuentra en "conf_form".
		Pero no se utiliza desde esa clase, ya que el layout puede ser instanciado de otros lugares que no tienen porque tener la clase "conf_form"
	*/
	LayoutDesign.prototype.readonly_control = function($ctrl, is_readonly, is_required)
	{
		/*
			A partir de un $control retorna su par $label
		*/
		var $get_label = function($control)
		{
			const control_id = $control.attr("id");
			const $parent = $control.parent().parent();
			let $label = $parent.find("div[for='" + control_id + "']");
			if ( $label.length === 0 ) {
				$label = $parent.find("label[for='" + control_id + "']");
			}
			return $label;
		};

		var readonly_control = function($control, readonly, is_required)
		{
			is_required = is_required === undefined ? false : is_required;
			if (typeof $control !== "object") {
				return;
			}
			const $label = $get_label( $control );
			if ( readonly ) {
				$control.prop("disabled", true).addClass("readonly");
				$label.removeClass("formLabel-rq").addClass("formLabel-ds readonly").prop("disabled", true);
				$label.removeClass("required").addClass("formLabel-ds readonly").prop("disabled", true);
			}
			else {
				$control.prop("disabled", false).removeClass("readonly");
				$label.removeClass("formLabel-ds readonly").prop("disabled", false);
				if ( is_required ) {
					$label.addClass("formLabel-rq");
					$label.addClass("required");
				}
			}
		};

		readonly_control($ctrl, is_readonly, is_required);
	};

	LayoutDesign.prototype.__fill_layouts_types = function(all_current_layouts)
	{
		const self = this;
		const is_new = !this.m_current_layout_config;

		var can_add = function(channel_id) {
			if (channel_id !== "ALL" && !inConcert.app.hasLicensesForChannel(channel_id)) {
				return false;
			}
			if (all_current_layouts[channel_id]) {
				if (!is_new && all_current_layouts[self.m_current_layout_config.Id]) {
					return true;
				}
				return false;
			}
			return true;
		};

		let channels = [];
		if (can_add("ALL")) {
			channels.push({id: "ALL", name: " " + _T("All")});
		}

		let vcc_channels = inConcert.app.load_vcc_channels();
	
		for (let channel_id in vcc_channels) {
			if (vcc_channels.hasOwnProperty(channel_id)) {
				if (can_add(channel_id)) {
					let channel_info = vcc_channels[channel_id];
					channel_info.id = channel_id;
					channels.push(channel_info);
				}
			}
		}

		const $channel_selector = $("#" + this.id_channel_selector);
		inConcert.Utils.fill_combo($channel_selector, true, channels, "id", "name", true);
	};

	/*
		Funcion existente en "conf_form_validation"
	*/
	LayoutDesign.prototype.validate_field = function( $ctrl, validation_type, message, options, callback_err )
	{
		if (this.parent.frm && this.parent.frm.validation) {
			return this.parent.frm.validation.validate_field($ctrl, validation_type, message, options, callback_err);
		}
		return;
	};

	LayoutDesign.prototype.refreshLayout = function() {
		const self = this;
		if (!this.m_layout) {
			return;
		}
		if (this.m_hnd_refresh_layout) {
			window.clearTimeout(this.m_hnd_refresh_layout);
			this.m_hnd_refresh_layout = undefined;
		}
		this.m_hnd_refresh_layout = window.setTimeout(function() {
			if (self.m_layout) {
				self.m_layout.updateSize();
				self.m_hnd_refresh_layout = undefined;
			}
		});
	};

	/*
	=====================================================================================================================================================
	*/

	function LayoutVccDesign (params)
	{
		LayoutVccDesign.uber.constructor.call(this, params);
	}
	inConcert.Utils.inherit(LayoutVccDesign, LayoutDesign);

	if (typeof(inConcert.LayoutVccDesign) !== "function") {
		inConcert.LayoutVccDesign = LayoutVccDesign;
	}

	LayoutVccDesign.prototype.__validate_field = function($ctrl)
	{
		let result = {
			success: this.validate_field($ctrl, "required", _T("Channel is required.")),
			$ctrl: $ctrl
		};
		return result;
	};

	/*
	=====================================================================================================================================================
	*/

	function LayoutUserDesign (params)
	{
		LayoutUserDesign.uber.constructor.call(this, params);
	}
	inConcert.Utils.inherit(LayoutUserDesign, LayoutDesign);

	if (typeof(inConcert.LayoutUserDesign) !== "function") {
		inConcert.LayoutUserDesign = LayoutUserDesign;
	}

	LayoutUserDesign.prototype.__validate_field = function($ctrl)
	{
		let result = inConcert.Utils.validate($ctrl.val(), "required");
		if (!result.success) {
			result.error_message = _T("Channel is required.");
			result.$ctrl = $ctrl;
		}
		return result;
	};

}());

