/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W014, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	//
	function permissions()
	{
		this.group_perm = {};
		this.user_perm = {};
		this.apps_actions = null;

		return true;
	}

	if (typeof inConcert.perm !== "function") {
		inConcert.perm = permissions;
	}

	permissions.prototype.set = function( group_permissions, user_permissions )
	{
		this.set_group_permissions( group_permissions );
		this.set_user_permissions( user_permissions );
	};

	permissions.prototype.set_group_permissions = function( group_permissions )
	{
		if ( group_permissions ) {
			this.group_perm = group_permissions;
		}
	};

	permissions.prototype.set_user_permissions = function( user_permissions )
	{
		if ( user_permissions ) {
			this.user_perm = user_permissions;
		}
	};

	permissions.prototype.set_user_vcc_permissions = function( user_vcc_permissions )
	{
		this.user_perm = this.user_perm || {};
		this.user_perm.vcc = user_vcc_permissions;
	};

	permissions.prototype.set_user_campaigns_permissions = function( user_campaigns_permissions )
	{
		this.user_perm = this.user_perm || {};
		this.user_perm.campaigns = user_campaigns_permissions;
	};

	permissions.prototype.attach_user_one_campaign_permissions = function( campaign_id, role_id, campaign_permissions )
	{
		this.user_perm = this.user_perm || {};
		this.user_perm.campaigns = this.user_perm.campaigns || {};
		this.user_perm.campaigns[ campaign_id ] = {};

		if ( !campaign_permissions ) {
			return;
		}

		this.user_perm.campaigns[ campaign_id ].group = ( campaign_permissions.groups && campaign_permissions.groups[ role_id ] ) ?
			campaign_permissions.groups[ role_id ] : [];

		this.user_perm.campaigns[ campaign_id ].member = campaign_permissions.member;
	};

	permissions.prototype.detach_user_one_campaign_permissions = function( campaign_id )
	{
		if ( this.user_perm && this.user_perm.campaigns && this.user_perm.campaigns[ campaign_id ] ) {
			delete this.user_perm.campaigns[ campaign_id ];
		}
	};

	permissions.prototype.get_user_perm = function( override_type )
	{
		if ( !this.user_perm ) {
			return {};
		}
		if ( !override_type ) {
			return this.user_perm; // todos los overrides ( por vcc, y por campaigns ( roles/members ) ) 
		}
		if ( override_type === "vcc" ) {
			return this.user_perm.vcc || {};
		}
		else {
			return this.user_perm.campaigns || {};
		}
	};

	permissions.prototype.load_logged_user_permissions = function(callback = $.noop)
	{
		const self = this;
		const uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/permissions/load_user_permissions/";
		const request = inConcert.invoke({
			url: uri,
			dataType : "json",
			success: function( response ) {
				self.set( response.groupPermissions, response.userPermissions );
				inConcert.app.messagebus.publish( "/loaded_user_permissions/updated", response );
				callback( true );
			},
			error: function( /*code, error_type */ ) {
				inConcert.app.messagebus.publish( "/loaded_user_permissions/updated", false );
				callback( false );
			}
		});
		return request;
	};

	permissions.prototype.get_one_app_actions = function(app_id, callback = $.noop)
	{
		this.load_apps_actions(function(success, apps_actions) {
			if (!success || !apps_actions) {
				callback(false);
				return;
			}
			let app_actions = apps_actions[app_id];
			callback(true, app_actions);
		});
	};

	permissions.prototype.load_apps_actions = function(callback, force)
	{
		callback = callback || $.noop;
		if (!this.apps_actions || force) {
			this.apps_actions = inConcert.app.getAppsActions();
		}
		callback(true, this.apps_actions);
	};

	permissions.prototype.action_belongs_to = function( app_id, action_id, override_type )
	{
		if ( !this.apps_actions ) {
			this.load_apps_actions();
		}
		var app = this.apps_actions[ app_id ];
		if ( !app ) {
			return false;
		}
		var action = app.Actions[ action_id ];
		if ( ! action ) {
			return false;
		}
		return ( action.Domain.override_type || "" ).toLowerCase() === override_type.toLowerCase();
	};

	/*
		Esta funcion valida si se ha configurado permisos para una app dada (sin importar si son true o false)
	*/
	permissions.prototype.has_configured_app = function( app_id )
	{
		var configured_in = {
			group             : false ,
			override_campaign : false ,
			override_vcc      : false
		};
		if ( this.group_perm[ app_id ] ) {
			configured_in.group = true;
		}

		var full_user_perm = this.get_user_perm(); // todos los overrides permisos de usuarios ( vcc y campaigns )

		// los overrides de vcc
		if ( full_user_perm.vcc ) {
			// valido si se ha configurado permisos el user a nivel vcc
			if ( full_user_perm.vcc[ app_id ] ) {
				configured_in.override_vcc = true;
			}
		}

		// los overrides de campaigns
		$.each( full_user_perm.campaigns || {}, function( i, perm )
		{
			// valido si se ha configurado permisos para el grupo de las campanas
			if ( perm.group && perm.group[ app_id ] ) {
				configured_in.override_campaign = true;
				return false; // break
			}
			// valido si se ha configurado permisos para el member de las campanas
			if ( perm.member && perm.member[ app_id ] ) {
				configured_in.override_campaign = true;
				return false; // break
			}
		});

		return configured_in;
	};

	permissions.prototype.__to_application_vcc_override = function( app_id )
	{
		var has_permission = false;
		var user_perm = this.get_user_perm( "vcc" );

		if ( user_perm[ app_id ] || this.group_perm[ app_id ] ) {
			var template_id = "{0}_{1}";
			var allowed_actions = {};

			var add_allowed_action = function(type, action_id)
			{
				var id = FORMAT(template_id, type, action_id);
				allowed_actions[id] = true;
			};
			var del_allowed_action = function(type, action_id)
			{
				var id = FORMAT(template_id, type, action_id);
				if (allowed_actions.hasOwnProperty(id)) {
					delete allowed_actions[id];
				}
			};

			var fill_allowed_actions = function(list)
			{
				list = list || {};
				$.each( list, function( type, action_list )
				{
					$.each( action_list, function( action_id, enabled ) {
						if ( enabled ) {
							add_allowed_action( type, action_id );
						}
						else {
							del_allowed_action( type, action_id );
						}
					});
				});
			};

			fill_allowed_actions( this.group_perm[ app_id ] );
			fill_allowed_actions( user_perm[app_id]  );

			for (var id in allowed_actions) {
				if (allowed_actions.hasOwnProperty(id)) {
					has_permission = true;
					break;
				}
			}
		}

		return has_permission;
	};

	permissions.prototype.__to_application_campaigns_override = function( app_id, campaign_id )
	{
		var has_permission = false;

		var template_id = "{0}_{1}";
		var allowed_actions = {};
		// el peso del grupo al cual pertenece la accion.
		var perm_type_weight = { group: 0, campaign_group: 1, campaign_member: 2 };

		var add_allowed_action = function( type, action_id, perm_type )
		{
			var id = FORMAT( template_id, type, action_id );
			allowed_actions[ id ] = { enabled: true, weight: perm_type_weight[ perm_type ] };
		};
		var del_allowed_action = function( type, action_id, perm_type )
		{
			var id = FORMAT( template_id, type, action_id );
			if ( allowed_actions.hasOwnProperty( id ) ) {
				var delete_action_weight = perm_type_weight[ perm_type ];
				var allowed_action = allowed_actions[ id ];
				if ( allowed_action && delete_action_weight > allowed_action.weight ) {
				// solo la elimina a la accion previamente agregada, si tiene un peso mayor.
					delete allowed_actions[ id ];
				}
			}
		};

		var fill_allowed_actions = function( list, perm_type )
		{
			list = list || {};
			$.each(list, function( type, action_list )
			{
				$.each(action_list, function( action_id, enabled )
				{
					if ( enabled ) {
						add_allowed_action( type, action_id, perm_type );
					}
					else {
						del_allowed_action( type, action_id, perm_type );
					}
				});
			});
		};

		// 1 - cargo las acciones permitidas para el grupo (rol)
		fill_allowed_actions( this.group_perm[ app_id ], "group" );

		// 2 - cargo las acciones permitidas para el grupo (rol) dentro de cada campana ( realizo override del anterior )
		var user_perm = this.get_user_perm( "campaign" );

		$.each( user_perm, function( cmp_id, perm )
		{
			if ( typeof campaign_id !== "undefined" && ( campaign_id.toLowerCase() !== cmp_id.toLowerCase() ) ) {
				return true; // continue
			}
			if ( perm.group ) {
				fill_allowed_actions( perm.group[ app_id ], "campaign_group" );
			}
		});

		// 3 - cargo las acciones permitidas para el member dentro de cada campana ( realizo override de los 2 anteriores )
		$.each( user_perm, function( cmp_id, perm )
		{
			if ( typeof campaign_id !== "undefined" && ( campaign_id.toLowerCase() !== cmp_id.toLowerCase() ) ) {
				return true; // continue
			}
			if ( perm.member ) {
				fill_allowed_actions( perm.member[ app_id ], "campaign_member" );
			}
		});

		var count_allowed_actions = Object.keys( allowed_actions ).length;
		if ( count_allowed_actions > 0 ) {
			has_permission = true;
		}
		return has_permission;
	};

	permissions.prototype.__to_application = function( app_id )
	{
		return this.__to_application_vcc_override( app_id );
	};

	/*
		campaign_id : is optional
		campaigns   : is optional
	*/
	permissions.prototype.to_application = function( app_id, campaign_id, campaigns )
	{
		var self = this;
		if (!app_id) {
			return true;
		}

		// valida si a nivel VCC se ha configurado.
		const {hasPermission} = this._is_enabled_app_vcc_override(app_id);
		if (!hasPermission) return false; // se ha inhabilitado la app a nivel vcc

		//
		//FIXME: by now we disable analytics until we get it's data from the reports
		//
		if (app_id === "analytics") {
			return false;
		}

		var configured_in = this.has_configured_app( app_id );

		var has_permission = false;

		if ( !configured_in.override_vcc && !configured_in.override_campaign ) {
			has_permission = this.__to_application( app_id );
		}
		else {
			if ( configured_in.override_vcc ) {
				has_permission = this.__to_application_vcc_override( app_id );
				if ( has_permission ) {
					return true;
				}
			}

			if ( configured_in.override_campaign ) {
				if ( campaign_id ) {
					has_permission = this.__to_application_campaigns_override( app_id, campaign_id );
				}
				else {
					campaigns = campaigns || inConcert.app.getCampaigns(); // current user campaigns
					$.each( campaigns, function( i, campaign )
					{
						has_permission = self.__to_application_campaigns_override( app_id, campaign.Id );
						if ( has_permission ) {
							return false; //break
						}
					});
				}
			}
		}

		return has_permission;
	};

	permissions.prototype.to_application_gui = function( app_id )
	{
		return this.to_category( app_id, "panel" );
	};

	permissions.prototype.to_navigation = function( app_id, navigation_id )
	{
		return this.to_action_on_any_campaign( app_id, navigation_id, undefined /* campaigns */, true /* has_validate_all_overrides */ );
	};

	/*
		Obtiene los permisos que se han inhabilitado a nivel VCC en la CF "VCCs.Permissions"
		Ej del JSON:
			{
				"override": {
					"interaction": {
						"actions": {
							"int_listen_call_records": false,
							"int_download_call_records": false
						}
					},
					"supervisor": {
					"actions": {
							"sup_download_call_records": false,
							"sup_listen_call_records": false
						}
					},
					"administration": {
						"hasPermission": false
					}
				}
			}
		En el ejemplo anterior se han inhabilitado las acciones de escuchar y descargar grabaciones en las apps "interaction" y "supervisor", tambien se desactivo por completo la app "administration"
	*/
	permissions.prototype._get_app_vcc_override = function(app_id)
	{
		const { Permissions: vccPermissions } = inConcert.app.get_vcc_info();
		if (!vccPermissions) return;
		return vccPermissions.override ? vccPermissions.override[app_id] : undefined;
	};

	/*
		Valida si tiene permisos sobre una app dada, validando si se ha configurado a nivel vcc.
	*/
	permissions.prototype._is_enabled_app_vcc_override = function(app_id)
	{
		const resp = {
			hasPermission : true,                              // por defecto siempre tiene permiso en el vcc
			vccAppOverride: this._get_app_vcc_override(app_id) // valida si a nivel VCC se ha configurado
		};
		if (!resp.vccAppOverride) return resp;
		if (typeof resp.vccAppOverride.hasPermission === "boolean" && resp.vccAppOverride.hasPermission === false) {
			resp.hasPermission = false;
		}
		return resp;
	};

	/*
		Valida si tiene permisos sobre una app dada, validando si se ha configurado a nivel vcc.
	*/
	permissions.prototype._is_enabled_action_app_vcc_override = function(app_id, action_id)
	{
		// valida si a nivel VCC se ha configurado.
		const {hasPermission, vccAppOverride = {}} = this._is_enabled_app_vcc_override(app_id);
		if (!hasPermission) return false; // se ha inhabilitado la app a nivel vcc
		// valida las acciones a nivel vcc
		const {actions} = vccAppOverride;
		if (actions) {
			if (typeof actions[action_id] === "boolean")
				return actions[action_id]; // true or false
		}
		// por defecto
		return true;
	};

	permissions.prototype.to_action = function( app_id, action_id, action_type /* opcional */, campaign_id /* opcional */, no_log /* opcional */, has_validate_all_overrides /* opcional */ )
	{
		if ( !app_id || !action_id ) {
			return true;
		}
		// valida si a nivel VCC se ha configurado y tiene permisos sobre la accion dentro de la app.
		const hasPermissionInVcc = this._is_enabled_action_app_vcc_override(app_id, action_id);
		if (!hasPermissionInVcc) return false; // se ha inhabilitado la app a nivel vcc

		var configured_in = this.has_configured_app( app_id );
		if ( !this.group_perm[ app_id ] && !configured_in.group && !configured_in.override_vcc && !configured_in.override_campaign ) {
			return false;
		}

		var type = typeof action_type === "string" ? action_type.toLowerCase() : undefined;
		var has_permission = false;
		var configured_in_override_type = false;

		var validate = function( list, override_type )
		{
			if ( typeof type !== "undefined" ) {
				var actions = list[ type ];
				if ( actions && typeof actions[ action_id ] !== "undefined" ) {
					has_permission = actions[ action_id ];
					configured_in_override_type = override_type;
				}
			}
			else {
				$.each( list, function( i, actions ) {
					if ( typeof actions[ action_id ] !== "undefined" ) {
						has_permission = actions[ action_id ];
						configured_in_override_type = override_type;
						return false; // break
					}
				});
			}
		};

		// 1 - valido la accion permitida sobre los permisos del grupo al cual pertenece el usuario
		var group_app = this.group_perm[ app_id ];
		if ( group_app ) {
			validate( group_app, false );
		}

		var action_belongs_to_vcc = this.action_belongs_to( app_id, action_id, "vcc" );
		if ( configured_in_override_type === false && ( action_belongs_to_vcc || has_validate_all_overrides ) ) {
		// aca valido contra los overrides del vcc
			var user_vcc_override_perm = this.get_user_perm( "vcc" );
			var user_app = user_vcc_override_perm[ app_id ];
			if ( user_app ) {
				validate( user_app, "vcc" );
			}
		}

		if ( configured_in_override_type === false && ( !action_belongs_to_vcc || has_validate_all_overrides ) ) {
		// aca valido contra los overrides de la campana
			if ( typeof campaign_id === "string" ) {
				// 2 - valido contra la accion permitida para el grupo (rol) dentro de cada campana ( realizo override del anterior )
				var user_campaign_override_perm = this.get_user_perm( "campaign" );
				$.each( user_campaign_override_perm, function( cmp_id, perm )
				{
					if ( campaign_id.toLowerCase() !== cmp_id.toLowerCase() ) {
						return true; // continue
					}
					if ( perm.group ) {
						var campaign_group_app = perm.group[ app_id ];
						if ( campaign_group_app ) {
							validate( campaign_group_app, "camp_role" );
						}
					}
					return false; // break
				});

				// 3 - valido contra la accion permitida para el member dentro de cada campana ( realizo override del anterior )
				$.each( user_campaign_override_perm, function( cmp_id, perm )
				{
					if ( campaign_id.toLowerCase() !== cmp_id.toLowerCase() ) {
						return true; // continue
					}
					if ( perm.member ) {
						var campaign_member_app = perm.member[ app_id ];
						if ( campaign_member_app ) {
							validate( campaign_member_app, "camp_member" );
						}
					}
					return false; // break
				});
			}
		}

		if ( no_log !== true && !has_permission ) {
			inConcert.logger.warn({
				action_name:      "permission to action",
				action_type:      "permission",
				message:          FORMAT("User {0} does not have permission to action '{1}' on application '{2}.", inConcert.app.getCompleteId(), action_id, app_id)
			});
		}

		var apply_has_permission = app_id !== "administration" || (action_id !== "import_users" && action_id !== "deactivate_user") || !inConcert.app.HasExternalUsersSync();
		has_permission = has_permission && apply_has_permission;

		return has_permission;
	};

	permissions.prototype.to_action_on_any_campaign = function( app_id, action_id, campaigns, has_validate_all_overrides /* opcional */ )
	{
		var self = this;
		var campaign_id;
		var has_permission = false;
		campaigns = campaigns || inConcert.app.getCampaigns(); // current user campaigns

		if ( campaigns.length === 0 ) {
			campaign_id = undefined;
			has_permission = this.to_action( app_id, action_id, undefined, campaign_id, true /* no log */, has_validate_all_overrides );
		}
		else {
			$.each( campaigns, function( _, campaign )
			{
				campaign_id = campaign.Id;
				has_permission = self.to_action( app_id, action_id, undefined, campaign_id, true /* no log */, has_validate_all_overrides );
				if ( has_permission ) {
					return false; //break
				}
			});
		}

		return has_permission;
	};

	permissions.prototype.to_category = function( app_id, category_type )
	{
		if ( !app_id || !category_type ) {
			throw("Permissions - application id and category are required.");
		}
		// valida si a nivel VCC se ha configurado.
		const {hasPermission} = this._is_enabled_app_vcc_override(app_id);
		if (!hasPermission) return false; // se ha inhabilitado la app a nivel vcc

		const self = this;
		category_type = category_type.toLowerCase();

		//-------------------------------------
		// aca se validan los permisos posta del rol ( sin overrides )
		//
		var group_categories;
		if ( this.group_perm[ app_id ] && this.group_perm[ app_id ][ category_type ] ) {
			group_categories = this.group_perm[ app_id ][ category_type ];
		}
		var permission_in_role = {};
		$.each( group_categories || {}, function( action_id, enabled )
		{
			if ( enabled ) {
				permission_in_role[ action_id ] = true;
			}
		});

		//-------------------------------------
		// aca se validan los override a nivel vcc (frm user)
		//
		var user_vcc_override_perm = this.get_user_perm( "vcc" );
		var user_vcc_app = user_vcc_override_perm[ app_id ];
		var user_vcc_categories;
		if ( user_vcc_app && user_vcc_app[ category_type ] ) {
			user_vcc_categories = user_vcc_app[ category_type ];
		}

		$.each( user_vcc_categories || {}, function( action_id , enabled )
		{
			if ( enabled ) {
				permission_in_role[ action_id ] = true;
			}
			else {
				delete permission_in_role[ action_id ];
			}
		});

		// esta funcion valida la cantidad de permisos en true existentes ( role + ov user ) ( en la lista solo estan los true )
		var is_enabled_role_permissions = function() {
			var enabled_permissions = Object.keys( permission_in_role ).length;
			return enabled_permissions > 0;
		};

		//-------------------------------------
		// ( ** ) aca se validan los override a nivel campaign, los del role y los del member (frm campaign)
		//  y valida que al menos una campana tenga en true un action_id de la categoria

		var campaigns = inConcert.app.getCampaigns() || {}; // current user campaigns
		var user_camp_override_perm = self.get_user_perm( "campaign" );

		var count_user_campaigns = Object.keys( campaigns ).length;
		var count_user_camp_override_perm = Object.keys( user_camp_override_perm ).length;


		var campaign_permission = {};
		var has_configured_campaign_permission = null;
		var is_enabled_role_camp_permissions = function() {
			if ( has_configured_campaign_permission !== null ) {
				return true;
			}
			var enabled_camp_permissions = Object.keys( campaign_permission ).length;
			return enabled_camp_permissions > 0;
		};

		var has_permission_in_campaign_override = null;

		if ( count_user_campaigns === count_user_camp_override_perm ) {
			$.each( user_camp_override_perm || {}, function( i, perm )
			{
				campaign_permission = {};
				has_configured_campaign_permission = null;

				var perm_group = perm.group[ app_id ];
				if ( perm_group ) {

					$.each( perm_group[ category_type ] || {}, function( action_id, enabled )
					{
						if ( enabled ) {
							has_configured_campaign_permission = true;
							campaign_permission[ action_id ] = true;
						}
					});
				}

				var perm_member = perm.member[ app_id ];
				if ( perm_member ) {

					$.each( perm_member[ category_type ] || {}, function( action_id, enabled )
					{
						has_configured_campaign_permission = true;
						if ( enabled ) {
							campaign_permission[ action_id ] = true;
						}
						else {
							delete campaign_permission[ action_id ];
						}
					});
				}

				if ( is_enabled_role_camp_permissions() ) {
					has_permission_in_campaign_override = true;
					return false; // break
				}
			});
		}

		if ( has_permission_in_campaign_override !== null ) {
			// ( ** )
			return true;
		}

		// el permiso posta del role + los override de users
		return is_enabled_role_permissions();
	};

	/*
		Esta funcion lo que realiza es mergear los override sobre los permisos originales.
	*/
	permissions.prototype.merge_permissions = function( source_permissions, override_permissions )
	{
		if ( !override_permissions ) {
			return source_permissions;
		}
		var merged_permissions = inConcert.Utils.cloneObject( source_permissions );

		// actualizo la lista de aplicaciones
		$.each( override_permissions.apps || {}, function( app_id, enabled )
		{
			if ( enabled ) {
				if ( !merged_permissions[ app_id ] ) {
					merged_permissions[ app_id ] = {};
				}
			}
			else {
				if ( merged_permissions[ app_id ] ) {
					delete merged_permissions[ app_id ];
				}
			}
		});

		// actualizo la lista de acciones de las aplicaciones
		$.each( override_permissions.actions || {}, function( app_id, permissions )
		{
			if ( !merged_permissions[ app_id ] ) {
				return true; //continue
			}
			$.each( permissions, function( action_id, enabled )
			{
				if ( enabled ) {
					merged_permissions[ app_id ][ action_id ] = true;
				}
				else {
					delete merged_permissions[ app_id ][ action_id ];
				}
			});
		});

		return merged_permissions;
	};
}());
