/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";
	var format = inConcert.Utils.format;

	function FileUpload () {}

	if (typeof inConcert.FileUpload !== "function") {
		inConcert.FileUpload = FileUpload;
	}

	var conversions = {
		stringToBinaryArray: function(string) {
			return Array.prototype.map.call(string, function(c) {
				return c.charCodeAt(0) & 0xff;
			});
		},
		base64ToString: function(b64String) {
			return atob(b64String);
		}
	};

	var DEFAULT_CALL_OPTS = {
		url: "",
		type: "POST",
		cache: false,
		success: function(response) {
			console.log(response);
		},
		error: function() {
			console.error(arguments);
		},
		// we compose the data manually, thus
		processData: false,
		/**
		 *  Override the default send method to send the data in binary form
		 */
		xhr: function() {
			var xhr = $.ajaxSettings.xhr();
			xhr.send = function(string) {
				var bytes = conversions.stringToBinaryArray(string);
				XMLHttpRequest.prototype.send.call(this, new Uint8Array(bytes).buffer);
			};
			return xhr;
		}
	};

	var postImage = function(opts) {
		var callObj = $.extend({}, DEFAULT_CALL_OPTS, opts);
		
		$.ajax(callObj);
	};

	var getImage = function(file, callback) {
		var reader = new FileReader();
		reader.onload = function (e) {
			var data = e.target.result;
			data = data.replace(/^data:image\/(png|jpe?g);base64,/, "");
			data = conversions.base64ToString(data);
			callback(data);
		};
		reader.readAsDataURL(file);
	};
	
	FileUpload.prototype.getFileExtension = function(file) {
		var file_a = (file.name || "").split(".");
		return file_a.length && file_a[file_a.length - 1] || "";
	};

	FileUpload.prototype.uploadImage = function(url, file, data, callback) {
		var self = this;
		//get image
		getImage(file, function(image) {
			data.file = {
				name : file.name,
				type : file.type,
				extension : self.getFileExtension(file)
			};
			url += "?p=" + encodeURIComponent(JSON.stringify(data));
			//upload image
			postImage({
				data: image,
				url: url,
				success: function(response) {
					callback(true, response);
				},
				error: function(_, error) {
					callback(false, error);
				}
			});
		});
	};
	
	FileUpload.prototype.__uploadFile = function(url, file, callback, progress) {
		progress = progress || function() {};
		var xhr = new XMLHttpRequest();
		xhr.open("POST", url, true);
		xhr.onload = function(e) {
			callback(e.target.status === 200, xhr.responseText);
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function(e) {
			var percentage = 0;
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 );
			}
			progress(percentage);
		};
		xhr.send(file);
	};
	
	FileUpload.prototype.uploadFile = function(url, file, data, callback, progress) {
		data.file = {
			name     : file.name,
			type     : file.type,
			extension: this.getFileExtension(file)
		};
		let encodedData = JSON.stringify(data);
		url += "?p=" + encodeURIComponent(encodedData);
		this.__uploadFile(url, file, callback, progress);
	};
	
	FileUpload.prototype.addFileToRepository = function(file, callback, progress) {
		var fileData = {
			id : inConcert.Utils.generateGuid(),
			type : file.type,
			format : inConcert.Utils.getFileExtension(file.name),
			name : file.name
		};
		var url = format(
			"{0}/inconcert/apps/repository/new/?id={1}&section=-1&contentType={2}&format={3}&fileName={4}&vcc={5}",
			inConcert.app.getBaseDomain(),
			encodeURIComponent(fileData.id),
			encodeURIComponent(fileData.type),
			encodeURIComponent(fileData.format),
			encodeURIComponent(fileData.name),
			encodeURIComponent(inConcert.Utils.md5(inConcert.app.get_vcc_id()))
		);
		this.__uploadFile(url, file, 
			function(status) {
				callback(status? fileData : false);
			}, progress);
	};
	
	FileUpload.prototype.removeFileFromRepository = function(id, callback) {
		var url = format(
			"{0}/inconcert/apps/repository/erase/?id={1}&section=-1",
			inConcert.app.getBaseDomain(),
			encodeURIComponent(id)
		);
		inConcert.invoke({
			url: url,
			dataType: "json",
			contentType: "application/json",
			success: function() {
				callback(true);
			},
			error: function() {
				callback(false);
			}
		});
	};
	
	FileUpload.prototype.removeFilesFromRepository = function(files, callback) {
		var url = format("{0}/inconcert/apps/repository/multipleserase/", inConcert.app.getBaseDomain());
		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(files),
			dataType: "json",
			contentType: "application/json",
			success: function() {
				callback(true);
			},
			error: function() {
				callback(false);
			}
		});
	};
}());