/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */
/* global $, inConcert, dataURLtoBlob */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	let FORMAT = inConcert.Utils.format;
	let REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function RepositoryAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("RepositoryAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("RepositoryAPI.Constructor - base URI is required");
		}

		this.path = params.path;
		this.uri = params.uri;
	}

	if (typeof inConcert.RepositoryAPI !== "function") {
		inConcert.RepositoryAPI = RepositoryAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	RepositoryAPI.prototype.__Get = function(uri, callback = $.noop)
	{
		if (typeof(uri) !== "string") {
			throw("RepositoryAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RepositoryAPI.prototype.__Get: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (response)    {callback(true, response);},
			error  : function (_, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	RepositoryAPI.prototype.__Post = function(uri, callback, data, contentType, dataType)
	{
		if (typeof(uri) !== "string") {
			throw("RepositoryAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RepositoryAPI.prototype.__Post: callback variable must be function type.");
		}
		contentType = contentType || "application/json";
		dataType = dataType !== undefined? dataType : "json";

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : contentType === "application/json"? JSON.stringify(data) : data,
			dataType    : dataType,
			contentType : contentType,
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (_, error) { callback(false, error);   }
		});
		return request_handler;
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.__PostFile = function(uri, file, callback, onProgress = $.noop, options = {})
	{
		callback = callback || $.noop;

		let xhr = new XMLHttpRequest();
		xhr.open("POST", uri, true);

		if (options.responseType) {
			xhr.responseType = options.responseType;
		}

		xhr.onload = function(e)  {
			const response = options.responseType ? xhr.response : e.target.responseText;
			callback(e.target.status === 200, response);
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function( e ) {
			let percentage = "";
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
			}
			onProgress(percentage);
		};

		xhr.send( file );
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.__fileExtension = function(fileName) {
		fileName = fileName || "";
		var match = fileName.match(/[^.]*(.)$/);
		if ( !match ) {
			return "";
		}
		var extension = match[0];
		if (extension === fileName) {
			extension = "";
		}
		return extension || "";
	};

/********************************************************************************************************

********************************************************************************************************/
	/*
	
	*/
	RepositoryAPI.prototype.get = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/download/?id=ID&section=-1&part=-1
		 */
		let uri = this.getUrl(params);

		let request_handler = this.__Get(uri, function(success, content) {
			if (success) {
				callback(true, content);
				return;
			}
			// error
			callback(false, content);
		});
		return request_handler;
	};

	/*
	
	*/
	RepositoryAPI.prototype.getUrl = function(params) {
		let url = FORMAT("{0}/download/?id={1}&section={2}&part={3}",
			params.relative? this.path : this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1)
		);
		if (params.name) {
			url += "&name=" + encodeURIComponent(params.name);
		}
		return url;
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.exists = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/is_found/?id=ID&section=-1&part=-1
		 */

		let uri = FORMAT("{0}/is_found/?id={1}&section={2}&part={3}",
			this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1)
		);

		let request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				if (result && result.exists) {
					return callback(true, result);
				}
				return callback(false, "file not found");
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};
	
	RepositoryAPI.prototype.getMetadata = function(params, callback = $.noop)
	{
		this.exists(params, function(status, result) {
			if (status) {
				callback(true, result.metadata);
			}
			else {
				callback(false, result);
			}
		});
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.delete = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/erase/?id=ID&section=-1&part=-1
		 */

		let uri = FORMAT("{0}/erase/?id={1}&section={2}&part={3}",
			this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1)
		);

		let request_handler = this.__Get(uri, function(success, result) {
			if (success) {
				return callback(true);
			}
			// error
			callback(false, result);
		});
		return request_handler;
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.multiplesdelete = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/multipleserase/
		 */

		var uri = FORMAT("{0}/multipleserase/", this.uri);
		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.add = function(params, content, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/new/?id=ID&section=-1&part=-1&contentType=image/png&format=png&fileName=myPicture.png
		 */
		let uri = FORMAT("{0}/new/?id={1}&section={2}&part={3}&contentType={4}&format={5}&fileName={6}",
			this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1),
			encodeURIComponent(params.type),
			encodeURIComponent(params.format),
			encodeURIComponent(params.name)
		);

		var request_handler = this.__Post(uri, callback, content, params.type, null);
		return request_handler;
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.addBase64 = function(params, base64Content, callback, onProgress)
	{
		let blobContent = FORMAT("data:{0};base64,", params.type) + base64Content;
		this.addBlob(params, blobContent, callback, onProgress);
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.addBlob = function(params, blobContent, callback, onProgress = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/new/?id=ID&section=-1&part=-1&contentType=image/png&format=png&fileName=myPicture.png
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}/new/?id={1}&section={2}&part={3}&contentType={4}&format={5}&fileName={6}",
			this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1),
			encodeURIComponent(params.type),
			encodeURIComponent(params.format),
			encodeURIComponent(params.name)
		);

		let blob = dataURLtoBlob(blobContent);
		this.__PostFile(uri, blob, callback, onProgress);
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.addFile = function(params, file, callback, onProgress = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/new/?id=ID&section=-1&part=-1&contentType=image/png&format=png&fileName=myPicture.png
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}/new/?id={1}&section={2}&part={3}&contentType={4}&format={5}&fileName={6}",
			this.uri,
			encodeURIComponent(params.id),
			encodeURIComponent((params.section || params.section ===0)? params.section : -1),
			encodeURIComponent((params.part || params.part ===0)? params.part : -1),
			encodeURIComponent(file.type),
			encodeURIComponent(this.__fileExtension(file.name) || ""),
			encodeURIComponent(file.name)
		);

		this.__PostFile(uri, file, callback, onProgress);
	};
	
	/*
	
	*/
	RepositoryAPI.prototype.convertTelephonyAudio = function(file, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/repository/convert_telephony_audio/?id=ID&section=-1&part=-1&contentType=image/png&format=png&fileName=myPicture.png
		 */
		callback = callback || $.noop;

		let uri = FORMAT("{0}/convert_telephony_audio/", this.uri);

		this.__PostFile(uri, file, (status, convertedContent) => {
			if (status) {
				this.makeTelephonyAudioConvertedFile(file, convertedContent, callback);
			} else {
				callback(null);
			}
		}, $.noop, { responseType: "blob" });
	};

	RepositoryAPI.prototype.makeTelephonyAudioConvertedFile = function(file, convertedContent, callback)
	{
		const fileName = (file.name || "").split(".")[0] + ".wav";
		const convertedFile = new File([convertedContent], fileName, { type: "audio/wav" });
		callback(convertedFile);
	};

}());
