/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert, VS */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Visual Search Management
	function VisualSearchMng(params)
	{
		if (!params) {
			throw("VisualSearchMng - 'params' is required");
		}
		if (!params.owner) {
			throw("VisualSearchMng - 'params.owner' is required");
		}
		if (!params.$container) {
			throw("VisualSearchMng - 'params.$container' is required");
		}

		this.owner             = params.owner;
		this.$filter_container = params.$container;
		this.filter_fields     = params.filter_fields || [];
		let callbacks          = params.callbacks || {};
		this.not_duplicate_filters = params.not_duplicate_filters;
		this.callbacks = {
			on_search: function() {
			 	(callbacks.on_search || $.noop)();
			},
			on_changed_filter: function(filter_values) {
			 	(callbacks.on_changed_filter || $.noop)(filter_values);
			}
		};

		if (params.auto_init) {
			this.initialize();
		}

		return true;
	}

	if (typeof inConcert.VisualSearchMng !== "function") {
		inConcert.VisualSearchMng = VisualSearchMng;
	}

	VisualSearchMng.prototype.initialize = function()
	{
		// make html
		let html = this.get_markup_visual_search();
		this.$filter_container.html(html).show();
		// create instance
		this.create_instance();
		// set events
		this.set_events();
	};

	VisualSearchMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	VisualSearchMng.prototype.set_events = function()
	{
		const self = this;

		const $btn_search = this.$filter_container.find("[filter_search_execute]");
		$btn_search
			.prop("disabled", false)
			.off("click")
			.on("click", function() {
				self.search();
			}
		);
	};

	VisualSearchMng.prototype.make_filters_by_name = function(filters)
	{
		const self = this;
		this.filters_by_name = {};
		let facet_labels = [];
		$.each(filters, function(_, filter) {
			self.filters_by_name[filter.label.toLowerCase()] = filter.value;
			facet_labels.push({value: filter.label, label: filter.label});
		});
		return facet_labels;
	};

	VisualSearchMng.prototype.get_search_filter_values = function(type, callback = $.noop)
	{
		type = (type || "").toLowerCase();

		let fn = function(filter_field) {
			return function() {
				// tiene que ser un array de strings, sample: ["uno", "dos"]
				let values = filter_field.values();
				return callback(values);
			};
		};

		let filter_values = {};
		for (let key in this.filter_fields) {
			if (this.filter_fields.hasOwnProperty(key)) {
				let filter_field = this.filter_fields[key];
				if (typeof(filter_field.values) === "function") {
					key = key.toLowerCase();
					filter_values[key] = fn(filter_field);
				}
			}
		}

		if (filter_values[type]) {
			filter_values[type]();
			return;
		}
		callback();
	};

	VisualSearchMng.prototype.get_markup_visual_search = function()
	{
		return "" +
			"<div>" +
				"<div filter_search='true' class='interactionFilterSearch'></div>" +
			"</div>";
	};

	/*
		El campo por defecto
	*/
	VisualSearchMng.prototype.get_remainder = function()
	{
		let remainder;
		for (let key in this.filter_fields) {
			if (this.filter_fields.hasOwnProperty(key)) {
				let filter_field = this.filter_fields[key];
				if (filter_field.default) {
					remainder = filter_field.label;
					break;
				}
			}
		}
		return remainder || _T("Text");
	};

	VisualSearchMng.prototype.__set_vs_facets_values_position = function()
	{
		//Cambiar la posicion del menu de facets o values cuando sobrepasa el ancho de la caja de busqueda.
		//Busco el input que se esta editando y el menu que le pertenece
		let $container = this.m_current_visual_search && this.m_current_visual_search.options.container && this.m_current_visual_search.options.container.find(".is_editing").find("input");
		let $menu = $container && $container.data('uiAutocomplete') && $container.data('uiAutocomplete').menu && $container.data('uiAutocomplete').menu.element;
		if ($menu) {
			const $vSearchBox = this.m_current_visual_search.searchBox.$el;
			const vSearchBoxOffset = $vSearchBox.offset();
			const vSearchBoxOffsetLeft = vSearchBoxOffset.left;
			const vSearchBoxWidth = $vSearchBox.outerWidth();

			const menuOffset = $menu.offset();
			const menuOffsetLeft = menuOffset && menuOffset.left;
			const menuWidth = $menu.outerWidth();

			// Se calcula la cantidad de desplazamiento necesario
			let newOffset = (vSearchBoxOffsetLeft + vSearchBoxWidth) - (menuOffsetLeft + menuWidth);
			if (newOffset < 0) {
				// Mover el elemento a la izquierda para que sea completamente visible
				$menu.css("left", (menuOffsetLeft + newOffset).toString() + "px");
			  }
		}
	};

	VisualSearchMng.prototype.create_instance = function()
	{
		const self = this;
		const $search_box = this.$filter_container.find('[filter_search]');

		this.m_current_visual_search = VS.init(
		{
			remainder  : this.get_remainder(),
			container  : $search_box,
			autocompleteHelpKey : '?',
			callbacks  : {
				search : function(_, vs_info) {
					window.setTimeout(function() {
						self.search();
						self.changed_filter(vs_info);
					});
				},
				facetMatches : function(callback) {
					let filters = self.get_filter_fields();
					let facet_labels = self.make_filters_by_name(filters);
					if (self.not_duplicate_filters) {
						facet_labels = self.__remove_duplicate_facets_selected(facet_labels);
					}
					callback(facet_labels);
					self.__set_vs_facets_values_position();
				},
				valueMatches: function(filter, _ , callback) {
					filter = self.get_filter_by_name(filter);
					self.get_search_filter_values(filter, callback);
					self.__set_vs_facets_values_position();
				},
				clearSearch: function(actualClearSearch) {
					actualClearSearch();
					window.setTimeout(function() {
						self.search();
					});
				}
			}
		});

		let filters = this.get_filter_fields();
		this.make_filters_by_name(filters);
	};

	VisualSearchMng.prototype.get_filter_by_name = function(name)
	{
		if (!this.filters_by_name) {
			return name;
		}
		return this.filters_by_name[name.toLowerCase()];
	};

	VisualSearchMng.prototype.search = function() {
		this.callbacks.on_search();
	};

	VisualSearchMng.prototype.changed_filter = function(vs_info) {
		let filter_values = [];
		if (vs_info) {
			(vs_info.models || []).forEach(model => {
				if (model.attributes && model.attributes.category) {
					filter_values.push({
						name : model.attributes.category,
						value: model.attributes.value
					});
				}
			});
		}
		this.callbacks.on_changed_filter(filter_values);
	};

	/*
		return sample:
			let filters = [
				{ value: 'type'      , label: "<%=_T('Type')%>"      },
				{ value: 'language'  , label: "<%=_T('Language')%>"  },
				{ value: 'visibility', label: "<%=_T('Visibility')%>"},
				{ value: 'text'      , label: "<%=_T('Text')%>"      }
			];
	*/
	VisualSearchMng.prototype.get_filter_fields = function()
	{
		if (this.array_filter_fields) {
			return this.array_filter_fields;
		}

		this.array_filter_fields = [];
		for (let key in this.filter_fields) {
			if (this.filter_fields.hasOwnProperty(key)) {
				let filter_field = this.filter_fields[key];
				this.array_filter_fields.push({
					value: filter_field.id,
					label: filter_field.label
				});
			}
		}

		return this.array_filter_fields;
	};

	VisualSearchMng.prototype.__remove_duplicate_facets_selected = function(facet_labels) {
		const self = this;
		return facet_labels.filter(facet => {
			return !self.m_current_visual_search.searchQuery.has(facet.value);
		});
	};

	VisualSearchMng.prototype.get_filter_values = function() {
		const self = this;
		let filters = this.m_current_visual_search.searchQuery.facets() || [];
		let filtersById = [];
		filters.forEach(function(filter) {
			let facetById = {};
			// each filter has a single key-value pair: type -> value
			$.each(filter, function(type, value) {
				let filter_type = self.get_filter_by_name(type);
				facetById[filter_type] = value;
			});
			filtersById.push(facetById);
		});
		return filtersById;
	};

	VisualSearchMng.prototype.set_filter_values = function(filter_values, can_execut_search)
	{
		let filter_text = "";
		if (filter_values) {
			for (let i = 0; i < filter_values.length; i++) {
				let filter_value = filter_values[i];
				let field_value = FORMAT("'{0}':'{1}'", filter_value.name, filter_value.value);
				filter_text = filter_text + (i > 0 ? " " : "") + field_value;
			}
		}
		this.m_current_visual_search.searchBox.value(filter_text);
		if (can_execut_search) {
			this.search();
		}
	};

	//-----------------------------------------------------------
}());
