/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert,	inConcertLabels */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	
	//"use strict";

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	const KB = 1024;
	const MB = KB * KB;

	let types = {
		image : {
			accept : "image/*",
			invalid: function() {
				return _T("File must be an image");
			},
			preview : true
		},
		video : {
			accept : "video/*",
			invalid: function() {
				return _T("File must be a video");
			}
		},
		audio : {
			accept : "audio/*",
			invalid: function() {
				return _T("File must be a audio");
			}
		},
		file : {
			accept : "*"
		},
		json : {
			extensions : [ "json" ]
		},
		xml : {
			extensions : [ "xml" ]
		},
		asterisk_audio : {
			id : "audio",
			accept : "audio/*",
			invalid: function() {
				return _T("File must be a audio");
			},
			telephonyAudio: true
		},
		whatsapp_cloud_image : {
			id : "image",
			extensions : [ "jpg", "jpeg", "jfif", "png" ],
			preview : true
		},
		whatsapp_cloud_video : {
			id : "video",
			extensions : [ "mp4" ]
		},
		whatsapp_cloud_document : {
			id : "file",
			extensions : [ "pdf" ]
		},
		whatsapp_cloud_file : {
			id : "agent_file",
			extensions : [ "pdf", "txt", "doc", "docx", "ppt", "pptx", "pps", "ppsx", "xls", "xlsx", "csv", "log", "7z", "eml", "ics", "odt" ]
		}
	};
	$.each(types, function(id, type) {
		type.id = type.id || id;
	});

	let labels = {
		"BUSSY"				: "Uploader is bussy",
		"FILES_LIMIT"		: "Files number cannot be greater than {0}",
		"SIZE_LIMIT"		: "File size must be less than {0} MB",
		"FILE_EXTENSION"	: "File extension must be one of these: {0}",
		"FIXED_EXTENSION"	: "File extension must be: {0}",
		"INVALID"			: "Invalid file",
		"UPLOAD_ERROR"		: "Error uploading file",
		"DELETE_ERROR"		: "Error deleting file",
		"UPLOADED_FILE"		: "File uploaded",
		"REMOVING_FILE"		: "Deleting file",
		"REMOVED_FILE"		: "File deleted"
	};

	if (!inConcert.FileUploaderTypes) {
		inConcert.FileUploaderTypes = types;
	}

	function FileUploaderControl(data) {
		this.files = [];
		this.initialFiles = [];
		//uploader control
		this.uploader = data.uploader && {
			$control : data.uploader.$control,
			disabledClassName : data.uploader.disabledClassName || "uploaderDisabled"
		} || null;
		//drag and drop control
		this.dragger = data.dragger && {
			$control : data.dragger.$control,
			overClassName : data.dragger.overClassName || "draggerOver",
			disabledClassName : data.dragger.disabledClassName || "draggerDisabled"
		} || null;
		//preview control
		this.preview = data.preview && {
			$control : data.preview.$control,
			canRemove : data.preview.canRemove !== false,
			uploadStatus : data.preview.uploadStatus !== false,
			image : data.preview.image !== false,
			name : data.preview.name !== false,
			size : data.preview.size !== false
		} || null;
		//properties
		this.m_allowsMultipleFiles = null;
		this.filesLimit = data.filesLimit || null;
		this.sizeLimitMB = data.sizeLimitMB || null;
		this.telephonyAudio = data.telephonyAudio || false;
		this.setMultipleFiles(data.allowsMultipleFiles || false);

		this.labels = data.labels || {};
		this.type = data.type || types.file;
		this.overrideFiles = data.overrideFiles !== false;
		this.uploadOnAdd = data.uploadOnAdd || false;
		this.removeOverridedInitialFiles = data.removeOverridedInitialFiles || false;
		this.fixedExtension = data.fixedExtension || null;
		this.alwaysValid = data.alwaysValid || false;
		this.m_editedFile = null;

		this.__make();
		this.__processing(false);
		this.toggleEnabled(!data.disabled);
		this.addInitialFiles(data.initialFiles || []);
	}

	if (!inConcert.FileUploaderControl) {
		inConcert.FileUploaderControl = FileUploaderControl;
	}

	/*
	 * Events
	 */

	FileUploaderControl.prototype.onBatchFilesUploadStart = function(callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onBatchFilesUploadEnd = function(callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileAdded = function(fileData, callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileAddError = function(file, result, callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileUploadStart = function(fileData, callback = $.noop) {
		callback();
	};
	
	FileUploaderControl.prototype.onFileUploadProgress = function(fileData, perc, callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileUploadEnd = function(fileData, result, callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileRemoveStart = function(fileData, callback = $.noop) {
		callback();
	};

	FileUploaderControl.prototype.onFileRemoveEnd = function(fileData, result, callback = $.noop) {
		callback();
	};

	/*
	 * Main
	 */

	FileUploaderControl.prototype.edit = function(fileData, fileType)
	{
		this.changeType(inConcert.FileUploaderTypes[fileType], this.getFileExtension(fileData.file));
		this.addInitialFile(fileData);
		this.m_editedFile = fileData;
	};

	FileUploaderControl.prototype.allowsMultipleFiles = function()
	{
		return this.m_allowsMultipleFiles;
	};

	FileUploaderControl.prototype.setMultipleFiles = function(allow)
	{
		this.m_allowsMultipleFiles = allow || false;
		this.filesLimit = this.m_allowsMultipleFiles ? this.filesLimit: 1; // -1 no limit
		this.sizeLimitMB = this.m_allowsMultipleFiles ? this.sizeLimitMB: 16; // TODO: Ver si hay que modificar el default

		if (!this.m_inputFileDomId) {
			return;
		}
		const $inputFile = $("#" + this.m_inputFileDomId);
		if ($inputFile.length === 0) {
			return;
		}
		$inputFile.removeAttr("multiple");
		if (allow) {
			$inputFile.attr("multiple", "multiple");
		}
	};

	FileUploaderControl.prototype.__make = function() {
		this.__makeUploader();
		this.__makeDragger();
		this.__makePreview();
	};

	FileUploaderControl.prototype.destroy = function() {
		this.disable();
		this.__destroyUploader();
		this.__destroyDragger();
		this.__destroyPreview();
		this.m_editedFile = null;
	};

	FileUploaderControl.prototype.toggleEnabled = function(enabled) {
		if (enabled) {
			this.enable();
		}
		else {
			this.disable();
		}
	};

	FileUploaderControl.prototype.enable = function() {
		if (this.enabled === true) {
			return;
		}
		this.enabled = true;
		this.__enableUploader();
		this.__enableDragger();
	};
	
	FileUploaderControl.prototype.disable = function() {
		if (this.enabled === false) {
			return;
		}
		this.enabled = false;
		this.__disableUploader();
		this.__disableDragger();
	};
	
	FileUploaderControl.prototype.__processing = function(processing) {
		this.processing = !!processing;
	};
	
	FileUploaderControl.prototype.__isProcessing = function() {
		return this.processing;
	};
	
	FileUploaderControl.prototype.__label = function(message) {
		message = message || "";
		return this.labels[message] || 
			(labels[message] && (inConcertLabels[labels[message]] || labels[message])) ||
			message;
	};
	
	FileUploaderControl.prototype.__errorMessage = function(result, data) {
		let label = this.__label(result);
		switch(result) {
			case "FILES_LIMIT":
				return format(label, this.filesLimit);
			case "SIZE_LIMIT":
				return format(label, this.sizeLimitMB);
			case "FILE_EXTENSION":
				if (this.fixedExtension) {
					return format(this.__label("FIXED_EXTENSION"), this.fixedExtension);
				}
				else {
					let extensions = data.list?
						data.list.reduce(function(list, type) {
							list.push.apply(list, type.extensions);
							return list;
						}, []) : data.extensions;
					return format(label, extensions.reduce(function(current, extension) {
						return current + (current? ", " : "") + "." + extension;
					}, ""));
				}
			case "FILE_TYPE":
				return this.__label(data.invalid()) || label;
			default:
				return label;
		}
	};
	
	FileUploaderControl.prototype.__makeOk = function(data) {
		let result = data || {};
		result.result = "OK";
		return result;
	};
	
	FileUploaderControl.prototype.__makeError = function(result, data) {
		result = result || "FAIL";
		return { result : result, reason : this.__errorMessage(result, data) };
	};
	
	FileUploaderControl.prototype.__isOk = function(result) {
		return result && result.result === "OK";
	};
	
	FileUploaderControl.prototype.makeFileId = function(file) {
		return inConcert.Utils.generateGuid();
	};
	
	FileUploaderControl.prototype.getFiles = function() {
		return this.files;
	};
	
	FileUploaderControl.prototype.getFile = function() {
		return this.files.length? this.files[0] : null;
	};
	
	FileUploaderControl.prototype.getFileById = function(fileId) {
		return this.files.find(function(file) {
			return file.id === fileId;
		});
	};
	
	FileUploaderControl.prototype.getFileName = function(file) {
		return file.name;
	};
	
	FileUploaderControl.prototype.getFileType = function(file) {
		return file.type || "text/plain";
	};
	
	FileUploaderControl.prototype.getFileSize = function(file) {
		if (file.uploaded && file.file) {
			return file.file.size;
		}
		return file.size;
	};
	
	FileUploaderControl.prototype.getFileSizeLabel = function(file) {
		return inConcert.Utils.fileSizeText(this.getFileSize(file));
	};
	
	FileUploaderControl.prototype.getFileExtension = function(file) {
		return inConcert.Utils.getFileExtension(this.getFileName(file));
	};
	
	FileUploaderControl.prototype.getValidFileTypeFromList = function(file, list) {
		const self = this;
		return list.find(function(type) {
			return !self.validateFileType(file, type);
		});
	};
	
	FileUploaderControl.prototype.validateFileType = function(file, type) {
		const self = this;
		if (self.alwaysValid) {
			return;
		}
		if (type.list) {
			//all types are validated by extension
			let extensionsError = true;
			let fileOk = type.list.find(function(type) {
				let error = self.validateFileType(file, type);
				if (error) {
					extensionsError = extensionsError && error.result === "FILE_EXTENSION";
					return false;
				}
				else {
					return true;
				}
			});
			return fileOk? 
				null : self.__makeError(extensionsError? "FILE_EXTENSION" : "FILE_TYPE", type);
		}
		let fileExtension = self.getFileExtension(file).toLowerCase();
		if (type.accept && !self.getFileType(file).toLowerCase().match(
			new RegExp(type.accept.toLowerCase().replace("*", ".*")))
		) {
			return self.__makeError("FILE_TYPE", type);
		}
		if (type.extensions && !type.extensions.find(function(extension) {
			return fileExtension === extension.toLowerCase();
		})) {
			return self.__makeError("FILE_EXTENSION", type);
		}
	};
	
	FileUploaderControl.prototype.validateFile = function(file) {
		if (this.alwaysValid) {
			return;
		}
		if (!file || !this.getFileSize(file) || !this.getFileName(file) || !this.getFileType(file)) {
			return this.__makeError("INVALID");
		}
		let fileSizeBytes = this.getFileSize(file);
		let sizeLimitBytes =  this.sizeLimitMB * MB;
		if (fileSizeBytes > sizeLimitBytes) {
			return this.__makeError("SIZE_LIMIT");
		}
		let fileExtension = this.getFileExtension(file).toLowerCase();
		if (this.fixedExtension) {
			if (fileExtension !== this.fixedExtension.toLowerCase()) {
				return this.__makeError("FILE_EXTENSION");
			}
		}
		else {
			let error = this.validateFileType(file, this.type);
			if (error) {
				return error;
			}
		}
	};

	FileUploaderControl.prototype.addFiles = function(files, callback = $.noop) {
		const self = this;

		if (this.__isProcessing()) {
			return callback(this.__makeError("BUSSY"));
		}

		// batch files validation
		let sizeLimitBytes =  this.sizeLimitMB * MB;
		let filesSizeBytes = 0;

		const validate_files = function(files_list = []) {
			for (let i = 0; i < files_list.length; i++) {
				let file = files_list[i];
				//size
				let fileSizeBytes = self.getFileSize(file);
				filesSizeBytes = filesSizeBytes + fileSizeBytes;
				if (filesSizeBytes > sizeLimitBytes) {
					let err = self.__makeError("SIZE_LIMIT");
					self.onFileAddError(file, err, function() {
						callback(err);
					});
					return false;
				}
			}
			return true;
		};

		// valido el tamanio de los nuevos archivos
		let is_ok = validate_files(files);
		if (!is_ok) {
			return;
		}
		// ahora valido incrementado contra los archivos previamente subidos
		is_ok = validate_files(this.files);
		if (!is_ok) {
			return;
		}

		// ok - start
		this.onBatchFilesUploadStart();

		inConcert.Utils.asyncReduce(files,
			function(result, file, next) {
				self.addFile(file, function(fileResult) {
					result[self.__isOk(fileResult)? "success" : "fail"].push(fileResult);
					next(result);
				});
			},
			function(result) {
				// on end all files
				self.onBatchFilesUploadEnd();
				callback(result);
			},
			this.__makeOk({success : [], fail : [] })
		);
	};

	FileUploaderControl.prototype.addFile = function(file, callback, initialData, converted = false) {
		const self = this;

		var onEnd = function(result) {
			if (self.__isOk(result)) {
				callback(result);
			}
			else {
				self.onFileAddError(file, result, function() {
					callback(result);
				});
			}
		};

		if (this.__isProcessing()) {
			return onEnd(this.__makeError("BUSSY"));
		}

		//validate
		let error = this.validateFile(file);
		if (error) {
			return onEnd(error);
		}

		var onFileAdded = function(fileData) {
			//event
			self.onFileAdded(fileData, function() {
				//upload
				self.__uploadAddedFile(fileData, function(result) {
					onEnd(result);
				});
			});
		};

		if (!converted && this.telephonyAudio) {
			this.onConvertFileStart();
			return this.convertTelephonyAudio(file, (convertedFile) => {
				this.onConvertFileEnd();
				if (convertedFile) {
					this.addFile(convertedFile, callback, initialData, true);
				} else {
					onEnd(this.__makeError("UPLOAD_ERROR"));
				}
			});
		}

		//add to list
		this.__addFileToList(file,
			function(fileData, error) {
				if (!fileData) {
					return onEnd(error);
				}
				if (initialData && initialData.noLoadBlob) {
					onFileAdded(fileData);
					return;
				}
				//preview
				if (!self.allowsMultipleFiles()) {
					self.showFilePreview(fileData, function() {
						onFileAdded(fileData);
					});
				}
				else {
					self.__loadFileBlob(fileData, function() {
						onFileAdded(fileData);
					});
				}
			},
			initialData
		);
	};

	FileUploaderControl.prototype.addInitialFiles = function(files, callback = $.noop) {
		const self = this;
		if (self.__isProcessing()) {
			return callback(self.__makeError("BUSSY"));
		}
		inConcert.Utils.asyncReduce(files,
			function(result, file, next) {
				self.addInitialFile(file, function(fileResult) {
					result[self.__isOk(fileResult)? "success" : "fail"].push(fileResult);
					next(result);
				});
			}, 
			function(result) {
				callback(result);
			},
			self.__makeOk({success : [], fail : [] })
		);
	};
	
	FileUploaderControl.prototype.addInitialRepositoryFile = function(data, callback = $.noop) {
		const self = this;
		self.getFileServerMetadata(data, function(result) {
			if (!self.__isOk(result)) {
				return callback(result);
			}
			let file = self.makeManualFile({
				name : result.metadata.FileName,
				type : result.metadata.ContentType,
				size : result.metadata.FileSize
			});
			self.addFile(file, callback, data, true);
		});
	};
	
	FileUploaderControl.prototype.addInitialFile = function(data, callback = $.noop) {
		this.addFile(data.file, callback, data, true);
	};
	
	FileUploaderControl.prototype.makeManualFile = function(data) {
		return {
			name : data.name,
			type : data.type,
			size : parseInt(data.size)
		};
	};
	
	FileUploaderControl.prototype.__addFileToList = function(file, callback, initialData = {}) {
		const self = this;

		function addFile() {
			let fileData = {
				id         : initialData.id || self.makeFileId(file),
				Section    : initialData.Section,
				Part       : initialData.Part,
				file       : file,
				initial    : !$.isEmptyObject(initialData),
				uploaded   : initialData.uploaded || false,
				url        : initialData.url || null,
				blobContent: initialData.blobContent || null
			};
			let editedFile = self.m_editedFile || {};
			if (!fileData.Section) {
				fileData.Section = editedFile.Section;
			}
			if (!fileData.Part) {
				fileData.Part = editedFile.Part;
			}
			self.files.push(fileData);
			if (fileData.initial) {
				self.initialFiles.push(fileData);
			}
			callback(fileData);
		}
		
		if (self.filesLimit > 0 && self.filesLimit <= self.files.length) {
			//files limit exceed
			if (self.overrideFiles) {
				//override removing last file
				let oldestFileData = self.files[0];
				let dontRemoveOnServer = !self.removeOverridedInitialFiles && oldestFileData.initial;
				self.removeFile(oldestFileData, function(result) {
					if (self.__isOk(result)) {
						addFile();
					}
					else {
						callback(false, result);
					}
				}, dontRemoveOnServer);
			}
			else {
				callback(false, self.__makeError("FILES_LIMIT"));
			}
		}
		else {
			addFile();
		}
	};
	
	FileUploaderControl.prototype.__uploadAddedFile = function(fileData, callback = $.noop) {
		if (this.uploadOnAdd && !fileData.uploaded) {
			this.uploadFile(fileData, callback);
		}
		else {
			callback(this.__makeOk());
		}
	};
	
	FileUploaderControl.prototype.changeType = function(type, fixedExtension, callback = $.noop) {
		this.type = type;
		this.fixedExtension = fixedExtension || null;
		this.__updateUploaderType();
		this.removeInvalidFiles(callback);
	};
	
	FileUploaderControl.prototype.cleanFile = function(fileData) {
		fileData.removed = true;
		this.files = this.files.filter(function(filteredFile) {
			return filteredFile.id !== fileData.id;
		});
		this.destroyFilePreview(fileData);
	};
	
	FileUploaderControl.prototype.cleanAllFiles = function() {
		while(this.files.size) {
			this.cleanFile(this.files[0]);
		}
	};
	
	FileUploaderControl.prototype.removeFile = function(fileData, callback, dontRemoveOnServer) {
		const self = this;
		if (self.__isProcessing()) {
			return callback(self.__makeError("BUSSY"));
		}
		
		self.startPreviewRemoveFile(fileData);
		self.onFileRemoveStart(fileData, function() {
			function remove(result) {
				self.endPreviewRemoveFile(fileData, result);
				self.onFileRemoveEnd(fileData, result, function() {
					if (self.__isOk(result)) {
						self.cleanFile(fileData);
					}
					callback(result);
				});
			}

			if (fileData.uploaded && !dontRemoveOnServer) {
				self.__processing(true);
				self.removeFileOnServer(fileData, function(result) {
					self.__processing(false);
					if (self.__isOk(result)) {
						fileData.uploaded = false;
					}
					remove(result);
				});
			}
			else {
				remove(self.__makeOk());
			}
		});
	};
	
	FileUploaderControl.prototype.removeFileById = function(fileId, callback = $.noop) {
		let fileData = this.getFileById(fileId);
		if (!fileData) {
			return callback(this.__makeError("FILE_NOT_FOUND"));
		}
		return this.removeFile(fileData, callback);
	};
	
	FileUploaderControl.prototype.removeFiles = function(filesToRemove, callback = $.noop) {
		const self = this;
		inConcert.Utils.asyncReduce(filesToRemove, 
			function(result, file, next) {
				self.removeFile(file, function(fileResult) {
					result[self.__isOk(fileResult)? "success" : "fail"].push(fileResult);
					next(result);
				});
			}, 
			function(result) {
				callback(result);
			},
			self.__makeOk({success : [], fail : [] })
		);
	};
	
	FileUploaderControl.prototype.removeAllFiles = function(callback = $.noop) {
		this.removeFiles(this.files.map(function(file) { return file; }), callback);
	};
	
	FileUploaderControl.prototype.removeInvalidFiles = function(callback = $.noop) {

		const self = this;
		let filesToRemove = self.files.filter(function(fileData) {
			let validationResult = self.validateFile(fileData.file);
			return !validationResult;
		});
		self.removeFiles(filesToRemove, function(result) {
			callback(result);
		});
	};
	
	FileUploaderControl.prototype.removeInitialOverridedFiles = function(callback = $.noop) {
		const self = this;
		let filesToRemove = self.initialFiles.filter(function(fileData) {
			return fileData.removed && fileData.uploaded;
		});
		self.removeFiles(filesToRemove, function(result) {
			callback(result);
		});
	};
	
	FileUploaderControl.prototype.removeNewFiles = function(callback = $.noop) {
		const self = this;
		let filesToRemove = self.files.filter(function(fileData) {
			return !fileData.initial;
		});
		self.removeFiles(filesToRemove, function(result) {
			callback(result);
		});
	};

	FileUploaderControl.prototype.uploadFile = function(fileData, callback = $.noop) {
		const self = this;
		if (self.__isProcessing()) {
			return callback(self.__makeError("BUSSY"));
		}
		self.startPreviewUploadFile(fileData);
		self.__processing(true);
		self.onFileUploadStart(fileData, function() {
			self.uploadFileOnServer(fileData,
				function(result) {
					if (self.__isOk(result)) {
						fileData.uploaded = true;
					}
					self.__processing(false);
					self.endPreviewUploadFile(fileData, result);
					self.onFileUploadEnd(fileData, result, function() {
						callback(result);
					});
				},
				function(perc) {
					self.updatePreviewUploadFileProgress(fileData, perc);
					self.onFileUploadProgress(fileData, perc, $.noop);
				});
		});
	};

	FileUploaderControl.prototype.getFileBlob = function(fileData, callback = $.noop) {
		const self = this;
		this.__loadFileBlob(fileData, function(result) {
			if (self.__isOk(result)) {
				result.content = fileData.blobContent;
			}
			callback(result);
		});
	};

	FileUploaderControl.prototype.__loadFileBlob = function(fileData, callback = $.noop) {
		const self = this;
		if (fileData.blobContent) {
			return callback(self.__makeOk());
		}
		const reader = new FileReader();
		reader.onload = function (e) {
			fileData.blobContent = e.target.result;
			callback(self.__makeOk());
		};
		reader.readAsDataURL(fileData.file);
	};

	FileUploaderControl.prototype.getFileText = function(fileData, callback = $.noop) {
		const self = this;
		self.__loadFileText(fileData, function(result) {
			if (self.__isOk(result)) {
				result.content = fileData.textContent;
			}
			callback(result);
		});
	};

	FileUploaderControl.prototype.__loadFileText = function(fileData, callback = $.noop) {
		const self = this;
		if (fileData.textContent) {
			return callback(self.__makeOk());
		}
		let reader = new FileReader();
		reader.onload = function (e) {
			fileData.textContent = e.target.result;
			callback(self.__makeOk());
		};
		reader.readAsText(fileData.file);
	};

	FileUploaderControl.prototype.getFileContent = function(fileData, callback = $.noop) {
		const self = this;
		self.__loadFileContent(fileData, function(result) {
			if (self.__isOk(result)) {
				result.content = fileData.content;
			}
			callback(result);
		});
	};
	
	FileUploaderControl.prototype.__loadFileContent = function(fileData, callback = $.noop) {
		const self = this;
		if (fileData.content) {
			return callback(this.__makeOk());
		}
		this.__loadFileText(fileData, function(result) {
			if (!self.__isOk(result)) {
				return callback(result);
			}
			self.__parseFileContent(fileData, function(content, err) {
				if (content) {
					fileData.content = content;
					callback(self.__makeOk());
				}
				else {
					callback(self.__makeError(err));
				}
			});
		});
	};

	FileUploaderControl.prototype.__parseFileContent = function(fileData, callback = $.noop) {
		let type = this.getFileType(fileData.file);
		switch(type.toLowerCase()) {
			case "application/json":
				try {
					let content = JSON.parse(fileData.textContent);
					return callback(content);
				}
				catch(e) {
					return callback(null, "FAIL");
				}
				break;
		}
		callback(null, "FAIL");
	};

	/*
	 * Server
	 */
	
	FileUploaderControl.prototype.uploadFileOnServer = function(fileData, callback, onProgress) {
		const self = this;
		let request = self.__makeUploadFileRequest(fileData);
		let api = inConcert.app.getRepositoryAPI();
		api.addFile(request, fileData.file,
			function(uploadedFile) {
				callback(uploadedFile? self.__makeOk() : self.__makeError("UPLOAD_ERROR"));
			}, function(perc) {
				onProgress(perc);
			});
	};
	
	FileUploaderControl.prototype.removeFileOnServer = function(fileData, callback = $.noop) {
		const self = this;
		let request = self.__makeRemoveFileRequest(fileData);
		let api = inConcert.app.getRepositoryAPI();
		api.delete(request, function(status) {
			callback(status? self.__makeOk() : self.__makeError("DELETE_ERROR"));
		});
	};
	
	FileUploaderControl.prototype.getFileServerUrl = function(fileData) {
		let request = this.__makeGetFileRequest(fileData);
		let api = inConcert.app.getRepositoryAPI();
		return api.getUrl(request);
	};
	
	FileUploaderControl.prototype.getFileServerMetadata = function(fileData, callback = $.noop) {
		const self = this;
		let request = this.__makeGetFileMetadataRequest(fileData);
		let api = inConcert.app.getRepositoryAPI();
		api.getMetadata(request, function(status, metadata) {
			callback(status? self.__makeOk({ metadata : metadata }) : self.__makeError("GET_METADATA_ERROR"));
		});
	};
	
	FileUploaderControl.prototype.__makeUploadFileRequest = function(fileData) {
		return {
			id     : fileData.id,
			section: fileData.Section,
			part   : fileData.Part,
			name   : this.getFileName(fileData.file),
			type   : this.getFileType(fileData.file),
			format : this.getFileExtension(fileData.file)
		};
	};
	
	FileUploaderControl.prototype.__makeRemoveFileRequest = function(fileData) {
		return {
			id     : fileData.id,
			section: fileData.Section,
			part   : fileData.Part
		};
	};
	
	FileUploaderControl.prototype.__makeGetFileRequest = function(fileData) {
		return {
			id     : fileData.id,
			section: fileData.Section,
			part   : fileData.Part
		};
	};
	
	FileUploaderControl.prototype.__makeGetFileMetadataRequest = function(fileData) {
		return {
			id     : fileData.id,
			section: fileData.Section,
			part   : fileData.Part
		};
	};
	
	/*
	 * Uploader
	 */
	
	FileUploaderControl.prototype.__makeUploaderAccept = function() {
		if (this.fixedExtension) {
			return "." + this.fixedExtension;
		}
		if (this.type.extensions) {
			return this.type.extensions.reduce(function(current, extension) {
				return current + (current? "," : "") + "." + extension;
			}, "");
		}
		return this.type.accept || "";
	};

	FileUploaderControl.prototype.__makeUploader = function() {
		if (!this.uploader) {
			return;
		}
		this.m_inputFileDomId = this.m_inputFileDomId || inConcert.Utils.generateGuid();
		let acceptedFiles = this.__makeUploaderAccept();
		let multipleAttr = this.m_allowsMultipleFiles ? "multiple" : "";
		let htmlFileSelector = format("<input id='{0}' type='file' hidden accept='{1}' {2}>", this.m_inputFileDomId, acceptedFiles, multipleAttr);
		this.uploader.$input = $(htmlFileSelector);
		this.uploader.$control.append(this.uploader.$input);
	};

	FileUploaderControl.prototype.__updateUploaderType = function() {
		if (!this.uploader) {
			return;
		}
		this.uploader.$input.prop("accept", this.__makeUploaderAccept());
	};

	FileUploaderControl.prototype.__destroyUploader = function() {
		if (!this.uploader) {
			return;
		}
		this.uploader.$input.remove();
		this.uploader.$input = null;
	};

	FileUploaderControl.prototype.__enableUploader = function() {
		const self = this;
		if (!self.uploader) {
			return;
		}
		self.uploader.$control
			.removeClass(self.uploader.disabledClassName)
			.prop("disabled", false)
			.on("click", function() {
				self.openUploadPopup();
			});
		self.uploader.$input.on("change", function(event) {
			self.addFiles(event.target.files);
		});
	};
	
	FileUploaderControl.prototype.openUploadPopup = function() {
		if (!this.uploader) {
			return;
		}
		this.uploader.$input[0].click();
	};
	
	FileUploaderControl.prototype.__disableUploader = function() {
		if (!this.uploader) {
			return;
		}
		this.uploader.$control
			.addClass(this.uploader.disabledClassName)
			.prop("disabled", true)
			.off("click");
		this.uploader.$input.off("change");
	};
	
	/*
	 * Dragger
	 */
	
	FileUploaderControl.prototype.__makeDragger = $.noop;
	FileUploaderControl.prototype.__destroyDragger = $.noop;

	FileUploaderControl.prototype.__enableDragger = function() {
		const self = this;
		if (!self.dragger) {
			return;
		}
		self.dragger.$control
			.removeClass(self.dragger.overClassName)
			.removeClass(self.dragger.disabledClassName)
			.on("dragover", function(e) {
				e.preventDefault();
				e.stopPropagation();
				$(this).addClass(self.dragger.overClassName);
			})
			.on("dragenter", function(e) {
				e.preventDefault();
				e.stopPropagation();
			})
			.on("dragleave", function(e) {
				$(this).removeClass(self.dragger.overClassName);
			})
			.on("drop", function(e) {
				$(this).removeClass(self.dragger.overClassName);
				if(e.originalEvent.dataTransfer && e.originalEvent.dataTransfer.files.length) {
					e.preventDefault();
					e.stopPropagation();
					self.addFiles(e.originalEvent.dataTransfer.files);
				}
			});
	};
	
	FileUploaderControl.prototype.__disableDragger = function() {
		if (!this.dragger) {
			return;
		}
		this.dragger.$control
			.removeClass(this.dragger.overClassName)
			.addClass(this.dragger.disabledClassName)
			.off("dragover")
			.off("dragenter")
			.off("dragleave")
			.off("drop");
	};

	/*
	 * Preview
	 */

	FileUploaderControl.prototype.__makePreview = $.noop;
	FileUploaderControl.prototype.__destroyPreview = $.noop;

	FileUploaderControl.prototype.showFilePreview = function(fileData, callback = $.noop) {
		const self = this;
		if (!this.preview || fileData.$preview) {
			return callback(this.__makeOk());
		}
		this.__loadFilePreviewImage(fileData, function(result) {
			if (!self.__isOk(result)) {
				return callback(result);
			}
			self.__makeFilePreview(fileData);
			self.__appendFilePreview(fileData);
			callback(self.__makeOk());
		});
	};

	FileUploaderControl.prototype.__hasFilePreviewImage = function(fileData) {
		if (this.type.list) {
			let type = this.getValidFileTypeFromList(fileData.file, this.type.list);
			return type && type.preview || false;
		}
		else {
			return this.type.preview;
		}
	};

	FileUploaderControl.prototype.__loadFilePreviewImage = function(fileData, callback = $.noop) {
		if (fileData.uploaded || fileData.blobContent || !this.__hasFilePreviewImage(fileData)) {
			return callback(this.__makeOk());
		}
		this.__loadFileBlob(fileData, function(result) {
			callback(result);
		});
	};
	
	FileUploaderControl.prototype.__getFilePreviewImageUrl = function(fileData) {
		if (fileData.url) {
			return fileData.url;
		}
		else if (fileData.uploaded) {
			return this.getFileServerUrl(fileData);
		}
		else {
			return fileData.blobContent;
		}
	};
	
	FileUploaderControl.prototype.__getFilePreviewType = function(fileData) {
		return this.type.id;
	};
	
	FileUploaderControl.prototype.__appendFilePreview = function(fileData) {
		this.preview.$control.append(fileData.$preview);
	};
	
	FileUploaderControl.prototype.__$makefilePreviewContent = function(fileData) {
		return $(format("".concat(
			"<div file_preview='{0}' class='uploaderFilePreview'>",
				"<div preview_upload_container class='uploaderFilePreviewUploadContainer'></div>",
				"<div preview_image class='uploaderFilePreviewImage'></div>",
				"<div preview_properties_container class='uploaderFilePreviewPropertiesContainer'></div>",
			"</div>"
		), fileData.id));
	};
	
	FileUploaderControl.prototype.__$filePreviewPropertiesContainer = function(fileData) {
		return fileData.$preview.find("[preview_properties_container]");
	};
	
	FileUploaderControl.prototype.__$filePreviewUploadContainer = function(fileData) {
		return fileData.$preview.find("[preview_upload_container]");
	};
	
	FileUploaderControl.prototype.__$filePreviewImage = function(fileData) {
		return fileData.$preview.find("[preview_image]");
	};
	
	FileUploaderControl.prototype.__makeFilePreview = function(fileData) {
		fileData.$preview = this.__$makefilePreviewContent(fileData);
		//add content
		this.__makeFilePreviewName(fileData);
		this.__makeFilePreviewSize(fileData);
		this.__makeFilePreviewUploadStatus(fileData);
		this.__makeFilePreviewRemoveButton(fileData);
		this.__makeFilePreviewImage(fileData);
		//remove empty containers
		if (this.__$filePreviewPropertiesContainer(fileData).is(":empty")) {
			this.__$filePreviewPropertiesContainer(fileData).remove();
		}
		if (this.__$filePreviewUploadContainer(fileData).is(":empty")) {
			this.__$filePreviewUploadContainer(fileData).remove();
		}
	};
	
	FileUploaderControl.prototype.__makeFilePreviewName = function(fileData) {
		if (!this.preview.name) {
			return;
		}
		let $control = $(format("<label file_preview_name class='uploaderFilePreviewName'>{0}</label>",
			this.getFileName(fileData.file)));
		this.__$filePreviewPropertiesContainer(fileData).append($control);
	};
	
	FileUploaderControl.prototype.__makeFilePreviewSize = function(fileData) {
		if (!this.preview.size) {
			return;
		}
		let $control = $(format("<label file_preview_size class='uploaderFilePreviewSize'>{0}</label>", this.getFileSizeLabel(fileData.file)));
		this.__$filePreviewPropertiesContainer(fileData).append($control);
	};
	
	FileUploaderControl.prototype.__makeFilePreviewUploadStatus = function(fileData) {
		if (!this.preview.uploadStatus) {
			return;
		}
		let $control = $("<label file_preview_upload_status class='uploaderFilePreviewStatus'></label>");
		this.__$filePreviewUploadContainer(fileData).append($control);
		this.__setPreviewUploadStatus(fileData, fileData.uploaded? "uploaded" : "notUploaded");
	};
	
	FileUploaderControl.prototype.__$filePreviewUploadStatus = function(fileData) {
		return fileData.$preview.find("[file_preview_upload_status]");
	};
	
	let uploadStatusList = [
		"notUploaded",
		"uploading",
		"uploaded",
		"removing",
		"removed"
	];
	FileUploaderControl.prototype.__setPreviewUploadStatus = function(fileData, status, data) {
		if (!this.preview.uploadStatus) {
			return;
		}
		let label = "";
		switch(status) {
			case "uploading":
				label = (data || 0) + "%";
				break;
			case "uploaded":
				label = fileData.initial? "" : this.__label("UPLOADED_FILE");
				break;
			case "removing":
				label = this.__label("REMOVING_FILE");
				break;
			case "removed":
				label = this.__label("REMOVED_FILE");
				break;
		}
		let classSufix = "FilePreviewStatus";
		let $control = this.__$filePreviewUploadStatus(fileData);
		uploadStatusList.forEach(function(uploadStatus) {
			$control.toggleClass(uploadStatus + classSufix, uploadStatus === status);
		});
		$control.text(label);
	};
	
	FileUploaderControl.prototype.__makeFilePreviewRemoveButton = function(fileData) {
		const self = this;
		if (!self.preview.canRemove) {
			return;
		}
		let $control = $("<button file_preview_remove class='uploaderFilePreviewRemove'></button>");
		$control
			.data("fileId", fileData.id)
			.click(function() {
				self.removeFileById($(this).data("fileId"));
			});
		self.__$filePreviewUploadContainer(fileData).append($control);
	};
	
	FileUploaderControl.prototype.__makeFilePreviewImage = function(fileData) {
		const self = this;
		let $control = self.__$filePreviewImage(fileData);
		if (!self.preview.image) {
			$control.remove();
			return;
		}
		$control.addClass(this.__getFilePreviewType(fileData) + "FilePreviewImage");
		if (!self.__hasFilePreviewImage(fileData)) {
			return;
		}
		let content = self.__getFilePreviewImageUrl(fileData);
		$control.css("background-image", "url(\"" + content + "\")");
	};
	
	FileUploaderControl.prototype.destroyFilePreview = function(fileData) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		fileData.$preview.remove();
		fileData.$preview = null;
	};
	
	FileUploaderControl.prototype.startPreviewUploadFile = function(fileData) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		this.__setPreviewUploadStatus(fileData, "uploading");
	};
	
	FileUploaderControl.prototype.updatePreviewUploadFileProgress = function(fileData, progress) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		this.__setPreviewUploadStatus(fileData, "uploading", progress);
	};
	
	FileUploaderControl.prototype.endPreviewUploadFile = function(fileData, result) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		this.__setPreviewUploadStatus(fileData, "uploaded");
	};
	
	FileUploaderControl.prototype.startPreviewRemoveFile = function(fileData) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		this.__setPreviewUploadStatus(fileData, "removing");
	};
	
	FileUploaderControl.prototype.endPreviewRemoveFile = function(fileData, result) {
		if (!this.preview || !fileData.$preview) {
			return;
		}
		this.__setPreviewUploadStatus(fileData, "removed");
	};

	FileUploaderControl.prototype.convertTelephonyAudio = function(file, callback) {
		inConcert.app.getRepositoryAPI().convertTelephonyAudio(file, callback);
	};

	FileUploaderControl.prototype.onConvertFileStart = function() {};

	FileUploaderControl.prototype.onConvertFileEnd = function() {};
}());