/* jshint smarttabs:true, quotmark:false, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const FORMAT = inConcert.Utils.format;

	function TweetyAPI( params )
	{
		if(typeof( params ) !== "object") {
			throw( "TweetyAPI.Constructor - expected an object with configuration" );
		}
		if(typeof( params.uri ) !== "string") {
			throw( "TweetyAPI.Constructor - base uri is required" );
		}

		this.uri = params.uri;
	}

	TweetyAPI.prototype.__load_projects = function(filter, callback)
	{
		var uri = FORMAT( '{0}/applications/published/{1}', this.uri, filter );
		callback = callback || $.noop;

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function (list)        { callback( true  , list  ); },
			error:     function (code, error) { callback( false , error ); }
		});

		return request_handler;
	};

	TweetyAPI.prototype.load_contactform_projects = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "contactform"}
		 */

		var filter = encodeURIComponent( '{"type": "contactform"}' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_cci_projects = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "cci"}
		 */

		var filter = encodeURIComponent( '{"type": "cci", "include_forms" : true }' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_webchat_projects = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "chatcampaign"}
		 */

		var filter = encodeURIComponent( '{"type": "chatcampaign"}' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_webcall_projects = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "webcallcampaign"}
		 */

		var filter = encodeURIComponent( '{"type": "webcallcampaign"}' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_survey_projects = function(callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "survey_form"}
		 */

		var filter = encodeURIComponent( '{"type": "survey_form"}' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_generic_projects = function(callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published/{"type": "generic"}
		 */

		var filter = encodeURIComponent( '{"type": "generic"}' );
		return this.__load_projects(filter, callback);
	};

	TweetyAPI.prototype.load_all_projects = function( callback, filter )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/published
		 */

		var uri = FORMAT( '{0}/applications/published', this.uri );
		callback = callback || $.noop;

		if ( filter ) {
			uri = uri + "/" + encodeURIComponent( filter );
		}

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function (list) {
				callback( true  , list  );
			},
			error:     function (code, error) { callback( false , error ); }
		});

		return request_handler;
	};

	TweetyAPI.prototype.clone = function( new_id, param, callback )
	{
		new_id = new_id || "";

		var uri_param = JSON.stringify({
		  copyFrom : param.source_id,
		  overwrite: param.overwrite
		});

		var uri = FORMAT( '{0}/designer/applications/create/{1}/{2}', this.uri, new_id, uri_param );
		callback = callback || $.noop;

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function ( response ) {
				if ( typeof response === "string" ) {
					try {
						response = JSON.parse( response );
					}
					catch(e) {
						callback(	false, response ); /* error */
						return;
					}
				}
				if ( response.status ) {
					callback( true ); /* success */
					return;
				}
				callback( false, response ); /* error */
			},
			error: function ( code, error ) { callback( false , error ); }
		});

		return request_handler;
	};

	TweetyAPI.prototype["export"] = function( project_id, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/designer/applications/export/TUPROYECTO.bundle
		 */
		callback = callback || $.noop;
		var uri = FORMAT( '{0}/designer/applications/export/{1}.bundle', this.uri, project_id );

		$.fileDownload( uri, {
				httpMethod : "GET",
				cookieName : "downloadStarted"
			})
			.done(function() {
				callback( true );
			})
			.fail(function(response) {
				try {
					response = JSON.parse( response ).result;
				} catch ( err ) {}
				callback( false, response );
			});
	};

	/*
	 * Para realizar la eliminacion de un proyecto de tweety, se realizan 2 requests:
	 *   1 - El primero contra ADMINISTRATION, para determinar si el proyecto se encuentra en uso en alguna campana
	 *   2 - El segundo si el mismo no se encuentra en uso, contra TWEETY para dar de baja de forma definitiva el proyecto.
	 */
	TweetyAPI.prototype.erase = function( project_id, callback )
	{
		var self = this;

		project_id = project_id || "";

		/*
		 * Segundo request - si es que el proyecto (CCI, webchat, etc) no se encuentra en uso en ninguna campana.
		 *  ejecuta el mismo contra tweety.
		 */
		var erase_tweety_project = function()
		{
			/*
			 * sample URI:
			 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/designer/applications/delete/pito
			 */
			var uri = FORMAT( '{0}/designer/applications/delete/{1}', self.uri, project_id );
			callback = callback || $.noop;

			inConcert.invoke({
				url:       uri,
				type:      "GET",
				dataType:  "json",
				success:   function ( response ) {
					if ( typeof response === "string" ) {
						try {
							response = JSON.parse( response );
						}
						catch(e) {
							callback( false, response ); /* error */
							return;
						}
					}
					if ( response.status ) {
						callback( true ); /* success */
						return;
					}
					callback( false, response ); /* error */
				},
				error: function ( code, error ) { callback( false , error ); }
			});
		};

		/*
		 * Primer request de validacion contra la app "web_designer" para determinar si el proyecto se encuentra
		 * en uso en alguna campana
		 *
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/web_designer/tweety/in_use/pito
		 */
		var uri = FORMAT( '{0}/inconcert/apps/web_designer/tweety/in_use/{1}', inConcert.app.getBaseDomain(), project_id );

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function () {
				erase_tweety_project();
			},
			error: function ( code, error ) {
				callback( false , error );
			}
		});

		return request_handler;
	};

	TweetyAPI.prototype.set_authfilter = function( project_id, token, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/applications/TUPROYECTO/setauthfilterarguments/{"token":"BLA BLA BLA"}
		 */
		callback = callback || $.noop;
		var data = encodeURIComponent( FORMAT('{ "token": "{0}" }', token) );
		var uri = FORMAT( '{0}/applications/{1}/setauthfilterarguments/{2}', this.uri, project_id, data );

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function ( status )    { callback( status  );       },
			error:     function (code, error) { callback( false , error ); }
		});

		return request_handler;
	};

	TweetyAPI.prototype.get_info = function( project_id, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.24.147:23182/inconcert/apps/webdesigner/designer/applications/TUPROYECTO/load/
		 */
		callback = callback || $.noop;
		var uri = FORMAT( '{0}/designer/applications/{1}/load', this.uri, project_id );

		var request_handler = inConcert.invoke({
			url:       uri,
			type:      "GET",
			dataType:  "json",
			success:   function ( info )    { callback( info  );       },
			error:     function (code, error) { callback( false , error ); }
		});

		return request_handler;
	};

	if (typeof inConcert.TweetyAPI !== 'function') {
		inConcert.TweetyAPI = TweetyAPI;
	}

}());
