module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_admModel = require "apps.administration.model"
local m_models = m_admModel:GetModels()
local PM = require("provisioningmanager")
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local model_perm = require "shared.permissions.models.permissions"
local m_moduleLocation = select(2, ...)
local channelsController = require "apps.administration.controllers.channels"
local agentCampaignController = require "apps.agent.controllers.campaigns"
local agentUsersController = require "apps.agent.controllers.users"
local agentCampaignModel = require "apps.agent.models.campaigns"
local agentUsersModels = require "apps.agent.models.users"
local modelRestrictions = require "apps.agent.models.config_restrictions"
local admUserController = require("apps.administration.controllers.users")
local agentservice = require "apps.agent.agentservice"
local allegro_campaigns = require "apps.allegro.controllers.campaigns"
local ir = require "lua_iragentlink"
local http_mapping = require "apps.agent.result_codes_to_http_codes"
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")
local authFilter = (require "authfilter")("agent")
local audits = require "audits"
local Fullcontact = require "fullcontact"
local m_validations = require("apps.administration.controllers.validations.campaign").GetDefinitions()

--#18604: Agregado para que no se rompan algunos emojis que pueden incluirse en los templates de whatsapp
local cjson = require "cjson"

require "LuaOrm.Date"
require "uuid"

m_htmlRender = nil

-----------------------------------------
-- SYNC ALLEGRO MEMBERS CAMPAIGNS
-----------------------------------------
local m_syncAllegroMembersCampaigns = {}
local m_sync = m_syncAllegroMembersCampaigns -- shortcut
-----------------------------------------

function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	-- campaign ( get, search, save, delete )
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. Search, prefix.."/search/")
	module:dispatch_get ( LogRequest .. utils.JsonReply .. Get, prefix.."/get/(.+)")

	-- el permiso del API se valida en la propia función para esta URI, ya que puede ser contra un permiso u otro, dependiendo de si va a crear o modificar.
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply ..
		utils.WebJsonValidatorData(
			m_validations.base_campaign,
			SaveCampaign
		),
		prefix .. "/save/"
	)
	-- la validación del API se realiza en la misma función para esta URI.
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. DeleteCampaign, prefix.."/delete/")

	-- campaign permissions
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetUserCampaignPermissions, prefix.."/get_user_campaign_permissions/([^/]+)/(.*)")

	-- members
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetMembers, prefix.."/get_campaign_members/(.+)")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. GetOnlyMembers, prefix.."/get_only_campaign_members/")	
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_members", UpdateMembers), prefix.."/update_members_list/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. UpdateAgentServiceMembers, prefix.."/update_agent_service_members/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. m_sync.Synchronize, prefix.."/synchronize_allegro_members/")

	-- dispositions
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission(
			"administration",
			"save_campaign_dispositions",
			utils.WebJsonValidatorData(
				m_validations.disposition_codes,
				SaveDispositions
			)
		),
		prefix.."/save_disposition_codes/"
	)

	-- scheduler
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_schedule", SaveScheduler), prefix.."/save_scheduler/")

	-- accounts
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", IsAvailableSaveAccount), prefix .. "/is_available_save_account/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission(
			"administration",
			"save_campaign_accounts",
			utils.WebJsonValidatorData(
				m_validations.account,
				AddAccount
			)
		), prefix.."/add_account/"
	)

	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", UpdateAccount), prefix.."/update_account/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "delete_campaign_accounts", RemoveAccount), prefix.."/remove_account/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "deactivate_campaign_accounts", ChangeAccountState), prefix.."/change_account_state/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. UpdateRedisAccounts, prefix.."/update_redis_account/")
	-- messages
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetAccountMessages, prefix.."/get_account_messages/([^/]+)/(.*)")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", SaveAccountMessages), prefix.."/save_account_messages/")

	-- integration
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_cc_integration", SaveIntegrationCCI), prefix.."/save_integration_cci/")

	-- Interaction Notifications
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_interaction_notifications", SaveInteractionNotifications), prefix.."/save_interaction_notifications/")

	-- Labels
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_labels",
			utils.WebJsonValidatorData(
				m_validations.label,
				SaveLabels
			)
		),
		prefix.."/save_labels/"
	)

	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "delete_campaign_labels", DeleteLabels), prefix.."/delete_labels/")

	-- Rules
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_rules",
			utils.WebJsonValidatorData(
				m_validations.rule,
				SaveRules
			)
		),
		prefix.."/save_rules/"
	)
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "delete_campaign_rules", DeleteRules), prefix.."/delete_rules/")
	module:dispatch_get ( LogRequest .. utils.JsonReply .. permissions:CheckCampaignPermission("administration", "view_campaigns_rules", GetRules), prefix.."/get_rules/(.+)")

	-- Canned messages
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_canned", SaveCannedMessages), prefix.."/save_canned_messages/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "delete_campaign_canned", DeleteCannedMessages), prefix.."/delete_canned_messages/")
	module:dispatch_get ( LogRequest .. utils.JsonReply ..  GetCannedMessages, prefix.."/get_canned_messages/(.+)")

	-- Attention levels
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_attention_level",
			utils.WebJsonValidatorData(
				m_validations.attention_levels,
				SaveAttentionLevels
			)
		),
		prefix.."/save_attention_levels/"
	)
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "delete_campaign_attention_level", DeleteAttentionLevels), prefix.."/delete_attention_levels/")
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetAttentionLevels, prefix.."/get_attention_levels/(.+)")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. SaveAllAttentionLevels, prefix .. "/save_all_attention_levels/")     -- unicamente para system users (backup.lua)
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. DeleteAllAttentionLevels, prefix .. "/delete_all_attention_levels/") -- unicamente para system users (backup.lua)

	-- thresholds
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. SaveThresholds, prefix .. "/save_thresholds/" )

	-- groups permissions
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission( "administration", "save_campaign_roles_permissions", SaveRolePermissions ), prefix .. "/save_role_permissions/" )

	-- transfer
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetCampaignAccountsForTransfer, prefix.."/get_campaign_accounts_for_transfer/")

	-- webcontact campaigns
	module:dispatch_get ( LogRequest .. utils.JsonReply .. GetWebcontactCampaignAccounts, prefix.."/get_webcontact_campaign_accounts/")

	-- facebook messenger
	module:dispatch_get(RenderMessengerExample, prefix .. "/messenger_examples/(.+)/(.+)")

	-- authorize whatsapp
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", StartWhatsappAuthorization), prefix.."/start_whatsapp_authorization/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", StopWhatsappAuthorization), prefix.."/stop_whatsapp_authorization/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", GetWhatsappAuthorizationStatus), prefix.."/get_whatsapp_authorization_status/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", SetWhatsappAuthorizationCode), prefix.."/set_whatsapp_authorization_code/")

	-- whatsapp phone registering and verification
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappGetPhoneStatus), prefix .. "/whatsapp_get_phone_status/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappRegisterPhone), prefix .. "/whatsapp_register_phone/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappDeregisterPhone), prefix .. "/whatsapp_deregister_phone/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappRequestCode), prefix .. "/whatsapp_request_code/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappVerifyCode), prefix .. "/whatsapp_verify_code/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappMigratePhone), prefix .. "/whatsapp_migrate_phone/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappLockActivationCode), prefix .. "/whatsapp_validate_activation_code/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappUnlockActivationCode), prefix .. "/whatsapp_unlock_activation_code/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappGetMessageTemplates), prefix .. "/whatsapp_get_message_templates/")	
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappSetMessageTemplates), prefix .. "/whatsapp_set_message_templates/")	
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappDelMessageTemplates), prefix .. "/whatsapp_del_message_templates/")

	-- whatsapp public url for repository files	
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappGetCloudApiPublicRepositoryFileUrl), prefix .. "/whatsapp_get_cloud_api_public_repository_file_url/")	

	--whatsapp groups
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappListGroups), prefix.."/whatsapp/list_groups/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappGetGroup), prefix.."/whatsapp/get_group/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappCreateGroup), prefix.."/whatsapp/create_group/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappUpdateGroup), prefix.."/whatsapp/update_group/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappCreateGroupInviteLink), prefix.."/whatsapp/create_group_invite_link/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappDeleteGroupInviteLink), prefix.."/whatsapp/delete_group_invite_link/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappRemoveGroupParticipants), prefix.."/whatsapp/remove_group_participants/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappLeaveGroup), prefix.."/whatsapp/leave_group/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappSetGroupIcon), prefix.."/whatsapp/set_group_icon/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappGetGroupIcon), prefix.."/whatsapp/get_group_icon/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappDeleteGroupIcon), prefix.."/whatsapp/delete_group_icon/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappAddGroupAdmins), prefix.."/whatsapp/add_group_admins/")
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. permissions:CheckCampaignPermission("administration", "save_campaign_accounts", WhatsappRemoveGroupAdmins), prefix.."/whatsapp/remove_group_admins/")

	-- render frm
	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
end

--[[

]]--
local function _AssignUserToCampaign (memberId, roleId, members, appsPermission, fullCampaignId, ksAgent)
	appsPermission = appsPermission or {
		interaction = model_perm.HasPermissionToApplication("interaction", memberId, inconcert.authUser.userGroup, nil, nil, nil, ksAgent),
		supervisor  = model_perm.HasPermissionToApplication("supervisor" , memberId, inconcert.authUser.userGroup, nil, nil, nil, ksAgent)
	}
	LogInfo("[ADMINISTRATION._AssignUserToCampaign] Assign user '%s' role '%s' to campaign '%s'. appsPermission: '%s'.", memberId, roleId, fullCampaignId, json.encode(appsPermission))
	-- Agrego al usuario que creo la campana, como member de la misma
	local member = {
		apps_permisson = appsPermission,
		id = memberId,
		role = roleId
	}
	table.insert(members, member)
end

--[[

]]--
local function _AssignSuperUsersToCampaign (vcc, loggedUserId, members, fullCampaignId, ksAgent)
	local roleId = "full-system-access"
	LogInfo("[ADMINISTRATION._AssignSuperUsersToCampaign] Assign SUPER USERS of role '%s' to campaign'%s'. loggedUserId: %s", roleId, fullCampaignId, loggedUserId)
	local superUsers = m_models.Users.GetVCCSuperUsers(vcc, ksAgent)
	--LogDebug("[ADMINISTRATION._AssignSuperUsersToCampaign] ---> superUsers: '%s'.", json.encode(superUsers))
	for superUserId in pairs(superUsers) do
		if superUserId:lower() ~= loggedUserId:lower() then
			-- Agrego el Current User a la Campana.
			_AssignUserToCampaign(
				superUserId,
				roleId,
				members,
				{
					interaction = false,
					supervisor  = true
				},
				fullCampaignId,
				ksAgent
			)
		end
	end
end

local function GetSystemConfiguration (elementId)
	assert(elementId)
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
	local success, systemConfiguration = pcall(PM.GetSystemConfiguration, ksProv, elementId)
	ksProv:Close()
	if not success then
		local errMsg = systemConfiguration
		return false, errMsg
	end
	return true, systemConfiguration
end

local function ExecuteFullcontactMethod (callback)
	local agentApp = require "apps.agent"
	agentApp.InitializeFullcontact(callback)
end

local function hasLicensesForChannel (channel)
	local agent = require "apps.agent.agent"
	return agent.hasLicensesForChannel(channel)
end

local function ReplyJSON(web, data)
	assert(type(data) ~= "string", "The parameter can not be string")
	web.headers["Content-Type"] = "application/json"
	return json.encode(data)
end

local returnError = function(web, err_type)
	web.status = err_type.status or 400
	LogError(json.encode(err_type or {}))
	err_type.reason = nil -- para que no viaje al lado del cliente (no es necesario)
	err_type.result = err_type.type or err_type.result or "INVALID_REQUEST"
	return err_type
end

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model:BuildModels()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

local function GetUserAuditData (fullUserId, keyspace, info)
	info = info or {}
	local campaigns = {}
	for _, campaignIdWithVCC in ipairs(info.campaigns or {}) do
		local campaignWithOutVCC = utils.GetId(campaignIdWithVCC)
		campaigns[campaignIdWithVCC] = {
			id = campaignWithOutVCC
		}
	end
	local user = {
		Id = utils.GetId(fullUserId),
		overridePermissions = info.overridePermissions or nil,
		campaigns = campaigns
	}
	--return admUserController.GetUserAuditData(fullUserId, keyspace, usersOverridePermissions, vcc, userColumns)
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:GetUserAuditData] --> user: '%s'.", json.encode(user))
	return user
end

local function RenderMessengerExample (web, page, params_s)
	local params = json.decode(params_s)
	local html = m_htmlRender:RenderPageLP("pages/messenger_example_" .. page .. ".html", web, params)
	return html
end

local function ValidateBelongsCampaign (web, action, userId, campaignId, keyspace)
    assert(web       , "web is required")
    assert(action    , "action is required")
    assert(userId    , "userId is required")
    assert(campaignId, "campaignId is required")

    local closeKeyspace = false

	LogInfo("[ADMINISTRATION.ValidateBelongsCampaign] Validating user: %s belongs to campaign: %s with action: %s", userId, campaignId, action)
    
    if (action ~= "create" and action ~= "modify" and action ~= "delete") then
        assert(action, "Unknown action")
    end
    
    if not keyspace then
		keyspace      = utils.GetDatasource():CassandraConnection( "Agent" )
		closeKeyspace = true
	end
    
    local belongs       = true
    local response      = {}
    local hasPermission = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil, keyspace)
    
    if hasPermission and not inconcert.authUser.api then
        local message = nil
        
        if action == "create" then
            message = LogError("[ADMINISTRATION.ValidateBelongsCampaign] Unable to %s campaign '%s'. You don't have permissions.", action, campaignId)
        else
			local isCampaignMember = GetModel("Campaigns").UserIsMember(campaignId, userId, keyspace)
			
            if not isCampaignMember then
				message = LogError("[ADMINISTRATION.ValidateBelongsCampaign] Unable to %s campaign '%s'. You don't have permissions because you don't belong to it.", action, campaignId)
			end
        end
        
        if message then
            belongs                     = false
            web.status                  = 403
            web.headers["Content-Type"] = "application/json"
            closeKeyspace               = true
            response                    = { result = "forbidden", reason = message }
        end
    end
    
    if closeKeyspace then
		keyspace:Close()
	end
    
    return belongs, response
end

function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form Campaign with sections for user %s, group %s", userId, inconcert.authUser.userGroup)

	local sections = {
		{ id = "basic"                    , file = "frm_campaign_content_basic.html"                    } ,
		{ id = "channels"                 , file = "frm_campaign_content_channels.html"                 } ,
		{ id = "attention_level"          , file = "frm_campaign_content_attention_level.html"          } ,
		{ id = "accounts"                 , file = "frm_campaign_content_accounts.html"                 } ,
		{ id = "groups"                   , file = "frm_campaign_content_groups.html"                   } ,
		{ id = "members"                  , file = "frm_campaign_content_members.html"                  } ,
		{ id = "labels"                   , file = "frm_campaign_content_labels.html"                   } ,
		{ id = "schedule"                 , file = "frm_campaign_content_schedule.html"                 } ,
		{ id = "disposition"              , file = "frm_campaign_content_dispositions.html"             } ,
		{ id = "canned_messages"          , file = "frm_campaign_content_canned_messages.html"          } ,
		{ id = "integration_cci"          , file = "frm_campaign_content_integration_cci.html"          } ,
		{ id = "integration_cci_allegro"  , file = "frm_campaign_content_integration_cci_allegro.html"  } ,
		{ id = "rules"                    , file = "frm_campaign_content_rules.html"                    } ,
		{ id = "interaction_notifications", file = "frm_campaign_content_interaction_notifications.html"}
	}
	local page_dir = "pages/"
	local html_content = ""
	for _, section in ipairs(sections) do
		local html_file = page_dir .. section.file
		local html_section = m_htmlRender:RenderPageLP(html_file, web, env)
		if html_section then
			html_content = html_content .. 
				"<div form_section='" .. section.id .. "' class='pageFormContainerAux' hidden>" ..
					html_section ..
				"</div>"
		else
			LogError("HTML Campaign section does not exist: %s", html_file)
		end
	end

	return html_content
end

local function DelAccountWhatsappTemplates(account)
	if not account or account.integrationType ~= "ICC_CLOUD_API" or not account.HSMConfig then
		return
	end

	for templateName, template in pairs(account.HSMConfig.TemplateMessages or {}) do
		local request = {
			campaignId = account.campaignId, 
			accountId = account.id, 
			wabaId = account.wabaId, 
			templateName = templateName, 
			mustSaveAccount = false
		}
		WhatsappDelMessageTemplates({request = request})
	end
end

local function WhatsappSetCloudApiPublicRepositoryFileUrl (campaignId, accountId, fileId, usedById)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Administration - User %s on vcc %s setting public url for '%s' with repository file '%s' on whatsapp account '%s' and campaign '%s'",
		userId,
		vcc,
		usedById,
		fileId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappSetCloudApiPublicRepositoryFileUrl({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		fileId = fileId,
		usedById = usedById
	})

	if res.result ~= "OK" then
		LogError("Error setting public url for '%s' with repository file '%s' on whatsapp account '%s' and campaign '%s' -> '%s'", usedById, fileId, accountId, campaignIdAtVCC, res.reason)
	end
end

local function WhatsappDelCloudApiPublicRepositoryFileUrl (campaignId, accountId, fileId, usedById)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Administration - User %s on vcc %s deleting public url for '%s' with repository file '%s' on whatsapp account '%s' and campaign '%s'",
		userId,
		vcc,
		usedById,
		fileId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappDelCloudApiPublicRepositoryFileUrl({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		fileId = fileId,
		usedById = usedById
	})

	if res.result ~= "OK" then
		LogError("Error deleting public url for '%s' with repository file '%s' on whatsapp account '%s' and campaign '%s' -> '%s'", usedById, fileId, accountId, campaignIdAtVCC, res.reason)
	end
end

local function RemoveHsmRepositoryFilesPublicURL (campaignId, accountId, oldHSMConfig, newHSMConfig)
	if oldHSMConfig and oldHSMConfig.TemplateMessages then
		for templateId, template in pairs(oldHSMConfig.TemplateMessages or {}) do
			for lang_code, language in pairs(template.languages or {}) do
				for oldC, component in pairs(language.components or {}) do
					if component.type == "HEADER" and component.sharedFile and component.sharedFile.fileId then
						if not(newHSMConfig and newHSMConfig.TemplateMessages and newHSMConfig.TemplateMessages[templateId] and newHSMConfig.TemplateMessages[templateId].languages and newHSMConfig.TemplateMessages[templateId].languages[lang_code]) then
							--language template deleted from HSM: remove related public url
							WhatsappDelCloudApiPublicRepositoryFileUrl(campaignId, accountId, component.sharedFile.fileId, templateId .. "_" .. lang_code)
						else
							if newHSMConfig and newHSMConfig.TemplateMessages and newHSMConfig.TemplateMessages[templateId] and newHSMConfig.TemplateMessages[templateId].languages and  newHSMConfig.TemplateMessages[templateId].languages[lang_code] and newHSMConfig.TemplateMessages[templateId].languages[lang_code].components then
								--search if new header still has that fileId
								local fileIdFound = false
								for newC, newComponent in pairs(newHSMConfig.TemplateMessages[templateId].languages[lang_code].components) do
									if newComponent.type == "HEADER" and newComponent.sharedFile and newComponent.sharedFile.fileId and newComponent.sharedFile.fileId == component.sharedFile.fileId then
										fileIdFound = true
									end
								end
								if not fileIdFound then
									WhatsappDelCloudApiPublicRepositoryFileUrl(campaignId, accountId, component.sharedFile.fileId, templateId .. "_" .. lang_code)
								end
							end
						end 
					end
				end
			end
		end
	end
end

local function IncludeAllegroOtherInfo (allegro_info, allegroOtherInfo)
	if not allegro_info then
		return
	end
	allegro_info.otherInfo = allegroOtherInfo
end

local function MakeAllegroOtherInfo (httpVars)
--	LogDebug(" 1- httpVars %s", httpVars)
--	LogDebug(" 2- httpVars.HTTP_X_ICC_RID '%s'", httpVars.HTTP_X_ICC_RID)
--	LogDebug(" 3- httpVars.HTTP_X_REAL_IP '%s'", httpVars.HTTP_X_REAL_IP)
	local allegroOtherInfo = {}
	if not httpVars then
		return allegroOtherInfo
	end
	allegroOtherInfo.remoteServerIP = httpVars.HTTP_X_REAL_IP
	return allegroOtherInfo
end

local function CompareListDifferences(campaignId, new_list, old_list, validation_type)
	new_list = new_list or {}
	local differences = {}
	local has_changed = false

	--LogDebug("[CompareListDifferences] 1 - new_list: %s", json.encode(new_list))
	--LogDebug("[CompareListDifferences] 2 - old_list: %s", json.encode(old_list))

	local log_differences =
[[

-----------------------------------------
--   List Differences for campaign ']] .. campaignId .. [[' - validation type: ']] .. tostring(validation_type) .. [[' --
-----------------------------------------

]]

	if not validation_type or validation_type == "add_or_update" then
	-- recorro los nuevos rows y si existe alguno nuevo, o si ha actualizado alguno existente.
		for row_id, row in pairs(new_list) do
			local old_row = old_list[row_id]
			if not old_row then
				differences[row_id] = {
					type = "NEW",
					row = row
				}
				old_list[row_id] = nil
				has_changed = true
				log_differences = log_differences .. "\n  + NEW row:    " .. json.encode(row) .. "\n"
			else
				for name, value in pairs(row) do
					if value ~= old_row[name] then
						differences[row_id] = {
							type = "UPDATE",
							updated_row = row,
							old_row = old_list[row_id]
						}
						old_list[row_id] = nil
						has_changed = true
						log_differences = log_differences .. "\n  * UPDATE row: \n" ..
							"NEW: " .. json.encode(row) .. "\n" ..
							"OLD: " .. json.encode(differences[row_id].old_row) .. "\n"
						break
					end
				end
			end
		end
	end

	if not validation_type or validation_type == "delete" then
	-- recorro los currents y valido si se ha eliminado alguno
		for row_id, row in pairs(new_list) do
			local old_row = old_list[row_id]
			if old_row then
				differences[row_id] = {
					type = "DELETE",
					deleted_row = old_row
				}
				has_changed = true
				log_differences = log_differences .. "\n  - DELETE row:  " .. json.encode(old_row) .. "\n"
			end
		end
	end

	if not has_changed then
		log_differences = log_differences .. "\n NOT DIFFERENCES \n"
	end
	LogInfo("%s", log_differences)
	return has_changed, differences
end

local function HasChangedStructuralList( new_list, old_list, log_changes_fn )
	new_list = new_list or {}
	old_list = old_list or {}
	log_changes_fn = log_changes_fn or function() end

	local has_changed = false
	for item_id, new_item in pairs( new_list ) do
		local current_item = old_list[ item_id ]
		if not current_item then
			has_changed = true
			log_changes_fn( item_id, current_item, new_item )
			break
		end
		has_changed = not utils.StructuralCompare( current_item, new_item )
		if has_changed then
			log_changes_fn( item_id, current_item, new_item )
			break
		end
	end
	return has_changed
end

local function GetThresholds(campaignId, keyspace)
	if type(campaignId) ~= "string" then
		return false
	end
	--LogInfo("Get campaign thresholds to campaign: %s", campaignId)
--[[
	{
		CHAT = {AutoWrapupTimeout = 1, Ghost = 2, InactivityTimeout = 3, ServiceLevel = 15 },
		IM = {..},
		..
	}
--]]
	return agentCampaignModel.GetCampaignThresholds(campaignId, keyspace)
end

local function __UpdateThresholds (campaignId, thresholds, keyspace, allegroOtherInfo)
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignIdAtVCC = utils.AppendVcc(utils.GetId(campaignId), vcc)
	local user_id = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.__UpdateThresholds] - Saving thresholds for campaign '%s' by User '%s'. thresholds: %s", campaignIdAtVCC, user_id, json.encode(thresholds or {}))

	if type(campaignId) ~= "string" or type(thresholds) ~= "table" then
		return false
	end

	-- valida si los Thresholds han cambiado, si no han cambiado no guarda, pero no es un error
	local oldThresholds = GetThresholds( campaignId, keyspace ) or {}
	
	local operation_audit = nil
	local hasChanged = HasChangedStructuralList(
		thresholds,
		oldThresholds,
		-- función que realiza un log de los cambios
		function( channel, current_channel_th, new_channel_th )
			LogInfo("Campaign '%s', channel '%s' thresholds has been changed.", campaignId, channel)
			LogInfo("    current thresholds: %s", json.encode( current_channel_th or {} ))
			LogInfo("    new     thresholds: %s", json.encode( new_channel_th or {} ))

			if current_channel_th and new_channel_th then
				operation_audit = "Update"
			else
				operation_audit = "Create"
			end
		end
	)
	local outThresholds
	if hasChanged then
		local oldAuditThresholds = GetAuditCampaign( campaignIdAtVCC, keyspace, "Thresholds")
		local saved = agentCampaignModel.SaveThresholds(campaignIdAtVCC, thresholds, keyspace, allegroOtherInfo)
		if saved then
			outThresholds = GetThresholds( campaignIdAtVCC, keyspace ) or {}
			LogInfo("Campaign %s, thresholds successfully saved", campaignIdAtVCC)
			agentNotifications.NotifyAgents({
				eventType = "administrator_thresholds_change",
				agentsLevel = "CAMPAIGN_USERS",
				pollerMessage = {
					campaignId = campaignIdAtVCC,
					thresholds = outThresholds
				}
			}, keyspace)

			local newAuditThresholds = GetAuditCampaign( campaignIdAtVCC, keyspace, "Thresholds")
			--Operacion inversa para saber si se elimino DC
			if not operation_audit then
				HasChangedStructuralList(oldAuditThresholds, newAuditThresholds,
					function( dc_id, currThresholds, deleteThresholds )
						if deleteThresholds then
							operation_audit = "Delete"
							oldAuditThresholds = deleteThresholds
						else
							operation_audit = "Update"
						end
				end) 
			end

			if oldAuditThresholds or newAuditThresholds then
				local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
				audits.Create({
					vcc = vcc,
					entity = "Campaign",
					user = inconcert.authUser.userName,
					fieldKey =  utils.GetId(campaignIdAtVCC),
					oldFields = operation_audit ~= "Create" and oldAuditThresholds or nil,
					newFields =  operation_audit ~= "Delete" and newAuditThresholds or nil,
					subEntity = "Thresholds",
					connection = ksHistorical
				})
				ksHistorical:Close()
			else
				LogWarning("[ADMINISTRATION.Thresholds] Audit record not created")
			end
		else
			LogWarning("Campaign %s, thresholds has not been saved", campaignIdAtVCC)
		end
		return saved, outThresholds
	else
		outThresholds = GetThresholds( campaignIdAtVCC, keyspace ) or {}
	end
	return true, outThresholds
end

function SaveThresholds (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	local campaignId = web.request.campaignId
	LogInfo("[ADMINISTRATION.SaveThresholds] - User %s on vcc %s saving thresholds configuration for campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s thresholds. Field campaignId is invalid."
		})
	end

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local thresholds = web.request.thresholds
	if type(thresholds) ~= "table" or utils.TableIsEmpty(thresholds) then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign thresholds. Invalid field thresholds values: '%s'.", campaignIdAtVCC, type(thresholds))
		})
	end

	for channel, channelThresholds in pairs(thresholds) do
		if utils.TableIsEmpty(channelThresholds) then
			return returnError(web, {
				reason = string.format("Unable to save %s campaign thresholds. Thresholds is empty for '%s' channel.", campaignIdAtVCC, channel)
			})
		end
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign thresholds. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return ReplyJSON(web, belResp)
	end

	local has_permission = true
	local account_is_created = web.request.account_is_created
	if account_is_created then
		-- si se esta creando la account se debe de validar los permisos contra los permisos de la account.
		has_permission = model_perm.IsPermissionEnabledForUser( "administration", "save_campaign_accounts", campaignId, keyspace )
	else
		-- si se modifican los thresholds se tienen que validar contra los propios permisos de thresholds (channels)
		has_permission = model_perm.IsPermissionEnabledForUser( "administration", "save_campaign_channels", campaignId, keyspace )
	end

	if not has_permission then
		keyspace:Close()
		return returnError(web, {
			status = 403,
			result = "FORBIDDEN",
			reason = string.format("Unable to save %s campaign thresholds. You do not have permission.", campaignId)
		})
	end

	local success, upd_thresholds = __UpdateThresholds(campaignIdAtVCC, thresholds, keyspace, allegroOtherInfo)
	keyspace:Close()

	return {result = "OK", thresholds = success and upd_thresholds or nil}
end

function Get(web, campaign_id_without_vcc)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local campaign_id_with_vcc = utils.AppendVcc(campaign_id_without_vcc, vcc)
	LogInfo("Get campaignId %s, user %s, vcc %s", campaign_id_without_vcc, userId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local campaign = GetModel("Campaigns").GetCampaign(vcc, campaign_id_without_vcc, keyspace)

	if next(campaign) then
		-- al frontend no deben viajar la lista de members del estado de la actualizacion.
		if campaign.UpdateMembersResult then
			campaign.UpdateMembersResult.members = {}
		end
		-- accounts
		local accounts = GetModel("Campaigns").GetCampaignAccounts( campaign_id_with_vcc, keyspace )
		campaign.accounts = {}
		for _, account in pairs(accounts) do
			table.insert(campaign.accounts, account)
		end

		campaign.Avatar = nil
		campaign.scheduler       = agentCampaignController.GetCampaignScheduler(campaign_id_without_vcc) -- schedule
		campaign.Thresholds      = GetThresholds(campaign_id_with_vcc, keyspace)                                   -- thresholds
		campaign.CannedMessages  = GetModel("Campaigns").GetCannedMessages(campaign_id_with_vcc, keyspace)            -- canneds
		campaign.AttentionLevels = GetModel("Campaigns").GetAttentionLevels(campaign_id_with_vcc, keyspace)           -- attention levels
		campaign.Rules           = GetModel("Campaigns").GetRules(campaign_id_with_vcc, keyspace)                     -- rules
		campaign.Labels          = GetModel("Campaigns").GetLabels(campaign_id_with_vcc, keyspace)                    -- labels

		-- integration
		-- TODO: Realizar la obtencion de la parte de integracion. Mirar como esta hecha la parte del model de campaigns
		--LogDebug("Retrieving campaign %s", json.encode(campaign))

		keyspace:Close()
		return campaign
	end
	-- error
	keyspace:Close()
	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[
	Estos valores se utilizan en VCC de Allegro y si la campana es de tipo CALL.
	Los mismos se utilizan para determinar si la sincronizacion de Usuarios entre OCC vs Allegro ha fallado (ejemplo por caidas de los ambientes, webhandlers, etc.)
	Estos valores se utilizan en \administration\scripts\adm_form_campaigns_section_members.js funcion "validate_entity"
	Sample:
		{
			warnings = {
				stepNumber     = 4, -- o menor
				expiredTimeSec = 10
			}
		}
]]--
local function GetUpdateMembersInfo ()
	return utils.GetUpdateMembersInfo()
end

--Returns all campaigns in the VCC
function Search (web)
	local startTime  = os.time()
	local vcc        = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.CAMPAIGNS:SEARCH] 1 - (START) Search campaigns for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local keyspace      = utils.GetDatasource():CassandraConnection( "Agent" )
	local hasPermission = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil, keyspace)
	local postData      = web.request
	local totalAdded    = 0
	local filteredRows  = {}
	local vccCampaigns  = {}

	-- Get campaigns from model who belongs to the vcc or to the loged user
	if hasPermission and not inconcert.authUser.api then
		vccCampaigns = GetModel("Campaigns").GetUserCampaigns(vcc, inconcert.authUser.userName, keyspace)
	else
		vccCampaigns = GetModel("Campaigns").GetVCCCampaigns(vcc, keyspace)
	end

	local totalUnfilteredRows = utils.GetRowCount(vccCampaigns)

	--LogInfo("[ADMINISTRATION.CAMPAIGNS:SEARCH] 2 - Filtering campaigns vccCampaigns %s", json.encode(vccCampaigns))

	local campaignsWarnings = {}
	local now = os.time()
	--[[
	]]--
	local function AddCampaignsWarning (campaign)
		local mustExcludeMembersInErr = true -- el detalle de los usuarios no debe de viajar al navegador
		utils.AddCampaignsWarning(vcc, campaign, campaignsWarnings, now, nil, keyspace, mustExcludeMembersInErr)
	end

	local hashColumns = utils.ArrayToHash(postData.columns or {}) or {}
	for campaignId in pairs(vccCampaigns) do
		--LogDebug("  -> campaignId: '%s'", campaignId)
		local campaign = GetModel("Campaigns").GetCampaign(vcc, campaignId, keyspace)
		if campaign and not utils.TableIsEmpty(campaign) then
			if campaign.SystemType ~= 'sandbox' and utils.MatchFilter(campaign, postData) then
				totalAdded = totalAdded + 1
				table.insert(filteredRows, campaign)
			end
			--
			AddCampaignsWarning(campaign)
		end
	end

	local response = {
		rows                  = filteredRows,
		last                  = "",
		total_rows            = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows,
		warnings              = campaignsWarnings
	}

	LogInfo("[ADMINISTRATION.CAMPAIGNS:SEARCH] 3 - Sorting campaigns")

	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end,
		TransformRowFn = function (row)
			--LogInfo("[ADMINISTRATION.CAMPAIGNS:SEARCH] 3.1 - TransformRowFn %s", json.encode(row))
			local campaign_id_with_vcc = utils.AppendVcc(row.campaignId, vcc)
			row.Thresholds = GetThresholds(campaign_id_with_vcc, keyspace)
			row.Scheduler  = agentCampaignController.GetCampaignScheduler(row.campaignId, keyspace)
			row.Accounts   = GetModel("Campaigns").GetCampaignAccounts(campaign_id_with_vcc, keyspace)
			--Si tiene Als, Members, Disposition Codes
			if hashColumns.AttentionLevels then
				row.AttentionLevels = GetModel("Campaigns").GetAttentionLevels(campaign_id_with_vcc, keyspace) or {}
			end
			if hashColumns.DispositionCodes then
				local dispositionCodes = GetModel("Campaigns").GetSpecificColumn( campaign_id_with_vcc, "dispositionCodes", keyspace )
				row.DispositionCodes = dispositionCodes and json.decode(dispositionCodes) or {}
			end

			if hashColumns.Members then
				row.Members = GetModel("Campaigns").GetCampaignMembers(vcc, campaign_id_with_vcc, keyspace)
			end	
			return row
		end
	}

	local resultData  = utils.MakeSearchResponse(response, postData, responseParams)
	local endTime     = os.time()
	local elapsedTime = endTime - startTime
	keyspace:Close()
	LogInfo("[ADMINISTRATION.CAMPAIGNS:SEARCH] 4 - (END) Search campaigns. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

-- esta función valida si el member se encuentra asociado a algun attention level
local function member_in_attention_level( member_id, attention_levels )
	local result = {
		success = true
	}

	if not attention_levels then
		return result -- is ok
	end
	member_id = member_id:lower()
	for att_level_id, att_level in pairs(attention_levels) do
		for user_id in pairs(att_level.users or {}) do
			if user_id:lower() == member_id then
				result.success = false
				result.att_level_id = att_level_id
				return result -- error
			end
		end
	end
	return result -- is ok
end

-- esta función valida si el member puede ser removido de la campana
local function __CanRemoveMember( campaignId, memberId, attention_levels, keyspace )
	local result = {
		success = false,
		err = { id = memberId, description = "", type = "" }
	}

	local vcc = inconcert.authUser.virtualContactCenter
	local campaign_with_vcc = utils.AppendVcc( campaignId, vcc )

	-- 1 - valida que el member no tenga ninguna interaccion tomada.
	local totalAssignedInteractions = admUserController.CountAssignedInteractions( memberId, campaign_with_vcc, keyspace )
	if totalAssignedInteractions > 0 then
		result.err.type = "SOME_USERS_HAS_INTERACTIONS"
		result.err.description = string.format("Member '%s' has assigned interactions for campaign '%s'. Can not remove now.", memberId, campaignId)
		result.err.totalAssignedInteractions = totalAssignedInteractions
		return result
	end

	-- 2 - valida que el member no pertenezca a ningun nivel de atencion.
	local resp = member_in_attention_level( memberId, attention_levels )
	if not resp.success then
		result.err.type = "SOME_USERS_ASSIGNED_IN_ATTENTION_LEVELS"
		result.err.att_level_id = resp.att_level_id
		result.err.description = string.format("Member %s has assigned to '%s' attention level for campaign '%s'. Can not remove now.", memberId, resp.att_level_id, campaignId)
		return result
	end

	result.success = true
	return result
end

function UpdateUserDefaultCampaign ( vcc, memberId, hasInteraction, campaignId, keyspace, action )
	if campaignId ~= "internal_calls" then
		local defaultCampaign = GetModel("Users").GetUserValue( vcc, memberId, "DefaultCampaign", keyspace )
		local campaignExists  = GetModel("Campaigns").CampaignExists(defaultCampaign, keyspace)
		if action == "added" then
			if not campaignExists and hasInteraction then
				-- The user default campaign not exists
				LogInfo("[Administration.Campaigns:UpdateUserDefaultCampaign] Default campaign not set or not exists. Set %s as default campaign to user: %s", campaignId, memberId)
				m_models.Users.SetUserDefaultCampaign(vcc, memberId, campaignId, keyspace)
			end
		elseif action == "removed" then
			if defaultCampaign and (defaultCampaign == utils.AppendVcc(utils.GetId(campaignId), vcc)) then
				local newDefaultCampaign = GetModel("Campaigns").GetAnotherUserAssignedCampaign(vcc, memberId, keyspace, defaultCampaign)
				if newDefaultCampaign then
					-- Assign another campaign as default campaign
					LogInfo("[Administration.Campaigns:UpdateUserDefaultCampaign] The new default campaign will be: %s for the user: %s", newDefaultCampaign, memberId)
					m_models.Users.SetUserDefaultCampaign(vcc, memberId, newDefaultCampaign, keyspace)
				else
					-- Default campign exists & remove user from the campaign & default campaign is equal to campaign to remove
					LogInfo("[Administration.Campaigns:UpdateUserDefaultCampaign] User: %s not have another assigned campaign so the default campaign: %s will be removed", memberId, campaignId)
					m_models.Users.SetUserDefaultCampaign(vcc, memberId, nil, keyspace)
				end
			end
		end
	end
end

--[[

]]--
local function GetCampaignMembersResult (campaignWithVCC, ksAgent)
	assert(type(campaignWithVCC) == "string")
	assert(ksAgent)
	local updateMembersResult = GetModel("Campaigns").GetSpecificColumn(campaignWithVCC, "UpdateMembersResult", ksAgent)
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:GetCampaignMembersResult] Campaign '%s' updateMembersResult: '%s'", campaignWithVCC, updateMembersResult or "NONE")
	if type(updateMembersResult) == "string" then
		updateMembersResult = json.decode(updateMembersResult)
	end
	return updateMembersResult
end

--[[

]]--
local function UpdateCampaignMembersResult (campaignWithVCC, updateMembersResult, ksAgent)
	assert(type(campaignWithVCC) == "string")
	assert(type(updateMembersResult) == "table")
	assert(ksAgent)
	if not updateMembersResult.members and updateMembersResult.finish ~= "OK" then
		local currentUpdateMembersResult = GetCampaignMembersResult(campaignWithVCC, ksAgent)
		if currentUpdateMembersResult then
			updateMembersResult.members = currentUpdateMembersResult.members
		end
	end
	updateMembersResult.user = inconcert.authUser.userName:lower()
	GetModel("Campaigns").UpdateCampaign(campaignWithVCC, {
		UpdateMembersResult = updateMembersResult
	}, ksAgent)
	-- local WAIT_SEC = 5
	-- LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateCampaignMembersResult] Update Campaign '%s' Members Current Result: '%s'. WAIT_SEC: %s.", campaignWithVCC, json.encode(updateMembersResult), WAIT_SEC)
	-- utils.Sleep(WAIT_SEC)
end

--[[
	1. Se agregan los Members contra el AgentService "AgentServiceUpdateMembers".
	2. Se realiza actualizacion de los Members contra Allegro (BD, ActorService).
	3. Si el paso 2 da error.
		- Se realiza Rollback del paso 1.
		- Se marca en Cassandra la actualizacion de Members como erronea, para poder tener feedback y seguimiento client side.
	4. Si el paso 2 da ok.
		- Retorna true (ok) para que la funcion que ha invocado UpdateMembersInAgentServiceAndAllegro realice el resto de actualizaciones contra OCC (permisos, etc)
]]--
local function UpdateMembersInAgentServiceAndAllegro (vcc, campaignWithoutVCC, new_members, sourceMembers, totalMembers, ksAgent, executionType)
	local campaignWithVCC = utils.AppendVcc(campaignWithoutVCC, vcc)
	--[[
		Funciones que agregan o remueven Members de la Campana contra el AS.
	]]--
	local function AgentServiceAddAgentToCampaign (member_id, role, actionType)
		--if actionType == "ROLLBACK" then
			--LogDebug("   ROLLBACK: AS adding -> member_id: %s, role: %s", member_id, role)
		--else
			--LogDebug("   AS adding -> member_id: %s, role: %s", member_id, role)
		--end
		agentservice.AddAgentToCampaign(member_id, campaignWithoutVCC, vcc, role or "agent")
	end
	--
	local function AgentServiceRemoveAgentFromCampaign (member_id, actionType)
		--if actionType == "ROLLBACK" then
			--LogDebug("   ROLLBACK: AS removing -> member_id: %s", member_id)
		--else
			--LogDebug("   AS removing -> member_id: %s", member_id)
		--end
		agentservice.RemoveAgentFromCampaign(member_id, campaignWithoutVCC, vcc)
	end
	--[[
		Funcion utilizada para ejecutar las acciones de UPDATE o ROLLBACK de Members contra el AS.
		En el caso en que el ActorService o el Webhandler de Allegro se caigan, se ejecuta el Rollback.
	]]--
	local function AgentServiceUpdateMembers (actionType)
		LogDebug("")
		LogInfo("-------------------------------------------------------------------------------------------------------------------------------------------------------")
		if actionType == "UPDATE" then
			LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersInAgentServiceAndAllegro->AgentServiceUpdateMembers] 1- Updating Users to AgentService .... actionType: '%s'", actionType)
		else
			LogWarning("[ADMINISTRATION.CAMPAIGNS:UpdateMembersInAgentServiceAndAllegro->AgentServiceUpdateMembers] 2- ROLLBACK Users to AgentService .... actionType: '%s'", actionType)
		end
		LogInfo("-------------------------------------------------------------------------------------------------------------------------------------------------------")
		local updatedMembers = {}
		local totalUpdatedMembersInBatch = 0
		--[[

		]]--
		local function Update (member_id, member_info)
--			member_info.included = true -- debo marcarlo como incluido, para que en una siguiente invocacion de la funcion "AgentServiceUpdateMembers"
			updatedMembers[member_id] = member_info
			totalUpdatedMembersInBatch = totalUpdatedMembersInBatch + 1
		end
		---
		for member_id, member_info in pairs(new_members) do
			local member = member_info.member
			if member_info.action == "added" then
				if actionType == "UPDATE" then
					--if not member_info.included then
						AgentServiceAddAgentToCampaign(member_id, member.role, actionType)
						Update(member_id, member_info)
					--end
				else
					-- ROLLBACK
					AgentServiceRemoveAgentFromCampaign(member_id, actionType)
				end
			elseif member_info.action == "removed" then
				if actionType == "UPDATE" then
					--if not member_info.included then
						AgentServiceRemoveAgentFromCampaign(member_id, actionType)
						Update(member_id, member_info)
					--end
				else
					-- ROLLBACK
					AgentServiceAddAgentToCampaign(member_id, member.role, actionType)
				end
			else
				-- updated
				--LogDebug("    updated -> member_id: %s", member_id)
				-- if actionType == "UPDATE" and not member_info.included then
				if actionType == "UPDATE" then
					Update(member_id, member_info)
				end
			end
		end

		------------------------------------
		-- Queda comentado para meter Pausas en el Webhandler y dar tiempo a reiniciar el webhandler y que NO queden sincronizados los Members de OCC con Allegro.
		------------------------------------
		-- local WAIT_SEC = 5
		-- LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersInAgentServiceAndAllegro->AgentServiceUpdateMembers] ----> WAIT_SEC: %s.", WAIT_SEC)
		-- utils.Sleep(WAIT_SEC)
		-------------------------------------

		-- ok
		return updatedMembers, totalUpdatedMembersInBatch
	end

	--LogDebug("--> new_members: %s", json.encode(new_members))

	---------------------------------------------------------------------------------------------------------
	-- 1. Se agregan los Members contra el AgentService "AgentServiceUpdateMembers".
	---------------------------------------------------------------------------------------------------------
	-- Marco en Cassandra que ha comenzado la actualizacion de Members contra el AS de OCC
	local updateMembersResult = {
		status        = "START UPDATE MEMBERS IN AGENT_SERVICE OF OCC",
		stepNumber    = 1,
		time          = os.time(),
		executionType = executionType or "MANUAL BY USER",
		members       = sourceMembers
	}
	UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)
	--
	local updatedMembers = AgentServiceUpdateMembers("UPDATE")
	-- Marco en Cassandra que ha finalizado la actualizacion de Members contra el AS de OCC
	local updateMembersResult = {
		status     = "END UPDATE MEMBERS IN AGENT_SERVICE OF OCC",
		stepNumber = 2,
		time       = os.time()
	}
	UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)

	---------------------------------------------------------------------------------------------------------
	-- 2. Se realiza actualizacion de los Members contra Allegro (BD, ActorService).
	---------------------------------------------------------------------------------------------------------
	local allegro_info = { action_id = "UPDATE_MEMBERS", members = updatedMembers, mustLoadMembers = true }
	IncludeAllegroOtherInfo(allegro_info, allegroOtherInfo)
	local startSubProcessTime = os.time()
	-- Marco en Cassandra que ha comenzado la actualizacion de Members contra ALLEGRO
	local updateMembersResult = {
		status     = "START UPDATE MEMBERS IN ALLEGRO",
		stepNumber = 3,
		time       = os.time()
	}
	UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)
	--------------------------------------------------------------------------
	-- SI comento todas estas lineas (**), puedo simular el Error de Actualizacion de los Members OCC->Allegro.
	--   Tambien debo descomentar las 2 lineas que estan comentadas.
	--   Con ello despues se van a ejecutar las sincronizaciones de Members en el Login
	--   local success = false
	--   local err = {type = "ALLEGRO_INTEGRATION_ERROR"}
	--(**)------------------------------------------------------------------------
	local success, err = GetModel("Campaigns").UpdateAllegroCampaignIn55( campaignWithVCC, ksAgent, allegro_info )
	LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersInAgentServiceAndAllegro]    ... END sub process -> Updated Members to Allegro. %d total members. UpdateAllegroCampaignIn55. Elapsed time %s seconds.",
		totalMembers,
		utils.GetElapsedTimeSec(startSubProcessTime)
	)
	-- Marco en Cassandra que ha comenzado la actualizacion de Members contra ALLEGRO
	local updateMembersResult = {
		status     = "END UPDATE MEMBERS IN ALLEGRO" .. (not success and " WITH ERROR" or ""),
		stepNumber = 4,
		time       = os.time()
	}
	UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)
	--comentar hasta aca--------------------------------------------------------------------------
	--[[---------------------------------------------------------------------------------------------------------
	3. Si el paso 2 da error.
		- Se realiza Rollback del paso 1.
		- Se marca en Cassandra la actualizacion de Members como erronea, para poder tener feedback y seguimiento client side.
		---------------------------------------------------------------------------------------------------------]]--
	if not success then
		-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
		--  la actualizacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			-- Ejecuto metodo para ROLLBACK de los Members contra el AS (Add y Remove)
			--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersInAgentServiceAndAllegro]  ==> err: '%s'.", json.encode(err))
			local updateMembersResult = {
				finish     = "ERROR",
				errType    = err.type,
				rawErr     = err,
				stepNumber = 4,
				status     = "START ROLLBACK MEMBERS IN AGENT_SERVICE OF OCC",
				time       = os.time()
			}
			UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)
			AgentServiceUpdateMembers("ROLLBACK")
			local updateMembersResult = {
				finish     = "ERROR",
				errType    = err.type,
				rawErr     = err,
				stepNumber = 4,
				status     = "END ROLLBACK MEMBERS IN AGENT_SERVICE OF OCC",
				time       = os.time()
			}
			UpdateCampaignMembersResult(campaignWithVCC, updateMembersResult, ksAgent)
			LogInfo("-------------------------------------------------------------------------------------------------------------------------------------------------------")
			LogDebug("")
			updateMembersResult.members = {} -- en el error que se devuelve al cliente (frontend) NO debe de viajar la lista de members
			return false, {result = err.type, err = updateMembersResult}
		end
	end

	LogInfo("-------------------------------------------------------------------------------------------------------------------------------------------------------")
	LogDebug("")
	-- ok
	return true
end

--[[
	Funcion que se ejecuta desde el mismo controller y de forma externa desde la App "supervisor"
]]--
function UpdateMembersList (camp_id, sourceMembers, keyspace, allegroOtherInfo, isCreateCampaign, skipLimitsValidation, skipAlreadyExecute, skipUpdate, currentCampaignMembers)
	assert(camp_id)
	assert(sourceMembers)

	local startCurrentProcessTime     = os.time()
	-- campaign id
	local simple_camp_id, vccCampaign = utils.SeparateVCC(camp_id)
	local vcc                         = vccCampaign or inconcert.authUser.virtualContactCenter
	simple_camp_id                    = simple_camp_id or utils.GetId(camp_id)
	local campaign_with_vcc           = utils.AppendVcc(simple_camp_id, vcc)
	-- userid
	local userIdWithoutVCC            = inconcert.authUser.userName
	local userId                      = utils.AppendVcc( userIdWithoutVCC, inconcert.authUser.virtualContactCenter )

	LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] ==> START Updating Members info, user '%s', campaign_with_vcc '%s'. skipLimitsValidation: %s, skipAlreadyExecute: %s", userId, campaign_with_vcc, skipLimitsValidation, skipAlreadyExecute)
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] Updating Members-> sourceMembers '%s'", json.encode(sourceMembers))

	-- Agrego los Super Usuarios a la Campana.
	_AssignSuperUsersToCampaign(vcc, userIdWithoutVCC, sourceMembers, campaign_with_vcc, keyspace)
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] (WITH superUsers) Updating Members-> sourceMembers '%s'", json.encode(sourceMembers))

	--[[
		Valido que NO exista otro proceso de actualizacion de Members en curso para la campana actual.
	]]--
	-- cuando se ejecuta desde la sincronizacion automatica de usuarios de allegro (por error) el valor de skipAlreadyExecute = true
	if not skipAlreadyExecute then
		local currentUpdateMembersResult = GetCampaignMembersResult(campaign_with_vcc, keyspace)
		if currentUpdateMembersResult then
			local now = os.time()
			local updateMemberElapsedTimeSec = now - currentUpdateMembersResult.time
			local updateMembersInfo = GetUpdateMembersInfo()
			if currentUpdateMembersResult.finish ~= "OK"
				and currentUpdateMembersResult.user ~= inconcert.authUser.userName:lower()
				and updateMemberElapsedTimeSec <= updateMembersInfo.warnings.expiredTimeSec
			then
				return false, {result = "THERE_IS_ANOTHER_USERS_UPDATE_IN_PROGRESS_FOR_THIS_CAMPAIGN", info = currentUpdateMembersResult}
			end
		end
	end

	local startSubProcessTime = os.time()
	currentCampaignMembers = GetModel("Campaigns").GetCampaignMembers( vcc, simple_camp_id, keyspace )
	LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process GetCampaignMembers. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] Updating Members-> currentCampaignMembers'%s'", json.encode(currentCampaignMembers or {}))

	currentCampaignMembers = utils.Remap( currentCampaignMembers, utils.GetId )
	LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] 1- Remap(..) Updating Members = currentCampaignMembers '%s'", json.encode(currentCampaignMembers or {}))
	local newCampaignMembers = utils.Remap( sourceMembers, function(_, m) return m.id end )
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] 2- Remap(..) Updating Members = newCampaignMembers (sourceMembers) '%s'", json.encode(currentCampaignMembers or {}))

	local problematicUsers = {}
	local added   = 0
	local updated = 0
	local removed = 0
	startSubProcessTime = os.time()
	local attention_levels = GetModel("Campaigns").GetAttentionLevels(campaign_with_vcc, keyspace)
	LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process GetAttentionLevels. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

	local new_members = {}
	--LogDebug("1--> currentCampaignMembers: '%s'", json.encode(currentCampaignMembers or {}))
	--LogDebug("2--> newCampaignMembers: '%s'", json.encode(newCampaignMembers or {}))

	startSubProcessTime = os.time()
	-- 1 - Primero miro si hay diferencias en los members actuales y los que han llegado posteados (sourceMembers).
		-- Si skipUpdate (Cuando se gestionan desde supervisor) y no es allegro, no se agregan los updates
	skipUpdate = skipUpdate and not utils.HasAllegroIntegration(vcc, keyspace) or nil
	
	utils.ProcessDiffs(
		currentCampaignMembers,
		newCampaignMembers,

		-- Adding
		function( member_id, member )
			new_members[member_id] = {
				action = "added",
				member = member
			}
			added = added + 1
		end,

		-- Deleting
		function( member_id, member )
			local can_remove = __CanRemoveMember( simple_camp_id, member_id, attention_levels, keyspace )
			if not can_remove.success then
				LogWarning(can_remove.err.description)
				table.insert( problematicUsers, can_remove.err )
				return
			end
			new_members[member_id] = {
				action = "removed",
				member = member
			}
			removed = removed + 1
		end,

		--Updating
		function ( member_id, old_member, new_member )
			--LogDebug("utils.ProcessDiffs --- UPDATING MEMBER %s\nnew_member_info: %s\nold_member_info: %s", member_id, json.encode(new_member), json.encode(old_member))
			if skipUpdate then --Desde el supervisor No se actualiza 
				return
			end
			new_members[member_id] = {
				action = "updated",
				member = new_member,
				old_member = old_member
			}
			updated = updated + 1
		end
	)

	--LogDebug("3--> new_members: '%s'", json.encode(new_members or {}))
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process utils.ProcessDiffs. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

	--1.1 Valido que los members a agregar no excedan el limite de config definido por vcc
	if not skipLimitsValidation and (added - removed) > 0 then
		local success, error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxCampaignsUsers", keyspace, (added - removed))
		if not success then
			return false, { result = error or "ERROR_USERS_EXCEEDS_LIMIT_ALLOWED" }
		end
	end

	-- 1.2 Valido si existe comunicacion con el Servidor de Allegro.
	local isReadyAllegroVcc, res = utils.IsReadyAllegroVcc(vcc)
	if not isReadyAllegroVcc then
		if res.result == "UNABLE TO CONNECT WITH ALLEGRO" then
			local err = res
			return false, err
		end
	end

	-- 1.3 Se invoca a la funcion "UpdateMembersInAgentServiceAndAllegro" para que actualice los Members contra el AS y contra Allegro.
	--     En el caso en que Allegro falle, realiza Rollback del AS.
	local executionType = skipAlreadyExecute and "AUTOMATIC SYNC WITH ALLEGRO"
	local total = (added + updated + removed)
	local success, err = UpdateMembersInAgentServiceAndAllegro(vcc, simple_camp_id, new_members, sourceMembers, total, keyspace, executionType)
	if not success then
		return false, err
	end

	-- 2 - Ahora si actualizo contra i6
	--
	-- Marco en Cassandra que ha comenzado la actualizacion de Members contra ALLEGRO
	local updateMembersResult = {
		status        = "START UPDATE MEMBERS OTHERS VALUES",
		stepNumber    = 5,
		time          = os.time(),
		executionType = executionType or "MANUAL BY USER"
	}
	UpdateCampaignMembersResult(campaign_with_vcc, updateMembersResult, keyspace)

	local membersByAction = {}

	-- Tabla con los miembros a actualizar, eliminar o agregar y los datos para auditar
	local arr_old_audit_members
	local connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
	local cachedInteractionTypes
	if ( added + updated + removed ) > 0 then
		startSubProcessTime = os.time()
		arr_old_audit_members, cachedInteractionTypes = GetAuditMembers(new_members, campaign_with_vcc, keyspace)
		LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process GetAuditMembers. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	end

	-- Actualizo los WORKGROUPS del supervisor!!!
	local function UpdateWorkgroup ( member_id, member )
		if not member.supervisor_workgroups then
			return
		end
		local user = GetModel("Users").GetUser(vcc, member_id, keyspace, {"SupervisorWorkgroups"} )
		--LogDebug("1 [UpdateMembersList] user: %s", json.encode(user) )
		--LogDebug("2 [UpdateMembersList] member.supervisor_workgroups: %s", json.encode(member.supervisor_workgroups) )
		local userSupervisorWorkgroup = user.SupervisorWorkgroups or {}
		userSupervisorWorkgroup[simple_camp_id] = member.supervisor_workgroups
		GetModel("Users").SaveUser(vcc, {Id = member_id, SupervisorWorkgroups = userSupervisorWorkgroup}, keyspace)
	end

	-- Actualizo los overrides de PERMISOS para el Member!!!
	local membersPermissions = {}
	local function UpdateMemberPermissions ( member_id, campaign_permissions )
		if not campaign_permissions then
			return
		end
		membersPermissions[ member_id ] = campaign_permissions
	end

	local current_interaction_types
	-- Actualizo los overrides de CAPABILITY para el Member!!!
	local function UpdateMemberCapability ( member_id, capability, action )
		local has_update = false

		if action == "UPDATE" and current_interaction_types and capability then
			local channels = channelsController.get_availables_channels()

			for channel_id in pairs(channels) do
				local interaction_type_id = channelsController.make_capability_id( channel_id, simple_camp_id, member_id )
				local user_channel_capability = current_interaction_types[interaction_type_id]
				local new_user_channel_values = capability[channel_id]
				if type(new_user_channel_values) ~= type(user_channel_capability) then
					has_update = true
					break
				end
				if user_channel_capability then
					if tostring(user_channel_capability.attentionCapability) ~= tostring(new_user_channel_values.attentionCapability) then
						has_update = true
						break
					end
				end
			end
		else
			has_update = true
		end

		if has_update then
			current_interaction_types = channelsController.UpdateCapabilityCampaignChannelAndMember( simple_camp_id, member_id, capability, current_interaction_types, keyspace )
		end
	end

	-- campaign, work, permisos, capability
	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, keyspace)

	startSubProcessTime = os.time()
	--Obtengo algunos datos relacionados con helpdesk para determinar si se debe consumir licencia.
	local isCampaignIntegrationHelpdesk
	local hasHelpDeskIntegration
	local applicationsActions
	if added > 0 then -- Se agregan nuevos usuarios
		if not isCreateCampaign then
			--La campanna tiene integracion helpdesk
			local campIntegration = GetModel("Campaigns").GetSpecificColumn( campaign_with_vcc, "Integration", keyspace )
			campIntegration = type(campIntegration) ~= "string" and {} or json.decode(campIntegration)
			isCampaignIntegrationHelpdesk = campIntegration and campIntegration.othersIntegrations and campIntegration.othersIntegrations.HELPDESK and campIntegration.othersIntegrations.HELPDESK.Enable
			if isCampaignIntegrationHelpdesk then
				--Si tiene el vcc integracion helpdesk
				hasHelpDeskIntegration = utils.HasHelpDeskIntegration(vcc, keyspace)
				applicationsActions = model_perm.GetApplicationsActions(keyspace, vcc)
			end
		end
	end

	LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] 2- Updating Users Other Information (permissions, workgroups, capability, etc) ....")
	for member_id, member_info in pairs(new_members) do
		local simpleUserId = utils.GetId(member_id)
		local fullUserId = utils.AppendVcc(simpleUserId, vcc)
		local overridePermissions = usersOverridePermissions[fullUserId] and usersOverridePermissions[fullUserId].actions or nil

		if member_info.action == "added" then
			--LogDebug("    added -> member_id: %s", member_id)
			local member = member_info.member
			-- LogDebug("A --- ADDING MEMBER %s, %s", member_id, json.encode(member))
			--Obtener los datos del usuario para registrar record de auditoria en Entidad User (Se agrega vinculo de campaign)
			local beforeSaveUser = GetUserAuditData(fullUserId, keyspace, {overridePermissions = overridePermissions}) or {}
			UpdateUserDefaultCampaign( vcc, member_id, member.apps_permisson.interaction, simple_camp_id, keyspace, member_info.action )
			UpdateWorkgroup( member_id, member )
			UpdateMemberPermissions( member_id, member.campaign_permissions )
			UpdateMemberCapability( member_id, member.member_capability, "ADD" )

			if not membersByAction.added then
				membersByAction.added = {}
			end
			membersByAction.added[member_id] = member_info

			--Crear registro de auditoria para Entidad User (Se agrega vinculo de campaign)
			--LogDebug("1 - ADDING MEMBER Arranco GetUserAuditData")
			local newFields = GetUserAuditData(fullUserId, keyspace, {
				overridePermissions = overridePermissions,
				campaigns = {campaign_with_vcc}
			})
			audits.Create({
				vcc = vcc,
				entity = "User", --Super Column
				user = inconcert.authUser.userName,
				fieldKey = member_id, --Column
				newFields = newFields or {},
				oldFields = beforeSaveUser or {},
				connection = connectionHistorical
			})
			--LogDebug("1.1--- Termino audits.Create")

			--[[
				Si mientras el usuario esta logueado se lo agrega a un campaña que tiene integracion con helpdesk y no fue licenciado antes se debe,
				consumir licencias y enviar notificacion al cliente.
				Aca valido:
					 - si se agregron miembros nuevos a una campana.
					 - si existe integracion con helpdesk del vcc.
					 - si la campanna tiene integracion helpdesk.
 					 Para cada usuario asignado:
					 	- si esta logueado
						- si tiene permisos de helpdesk tickets y permiso de la app interaccion
						- Si no fue licenciado (helpdesk) previamente. (Esto lo saco de la data del usuario en redis authFilter:userBySID:AGENT (sessionId))
						- Si no existen licencias disponibles, se deja log de error.
			]]--
			if isCampaignIntegrationHelpdesk and hasHelpDeskIntegration and agentservice.IsAgentLoggedIn(simpleUserId, vcc) then
				local userData = authFilter:GetLoggedUserInfo(simpleUserId, vcc) or {}
				local userHelpdeskIntegration = userData.Licenses and userData.Licenses.AppsIntegrations and
				userData.Licenses.AppsIntegrations.HelpDesk or {}
				local isUserLicensed = userHelpdeskIntegration.IsLicensed

				LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] Validate Consume helpdesk license for user %s --> isUserLicensed %s", fullUserId, isUserLicensed)
				if not isUserLicensed then
					local hasPermissions = permissions.IsPermissionEnabledForSpecificUser("apps_integrations",
						"helpdesk_tickets_in_app_interaction",
						fullUserId,
						userData.usergroup,
						usersOverridePermissions[fullUserId],
						nil,
						nil,
						keyspace,
						applicationsActions
					)
					hasPermissions = hasPermissions and member.apps_permisson and member.apps_permisson.interaction
					if hasPermissions then
						userData.AppsPermissions = userData.AppsPermissions or {}
						userData.AppsPermissions.helpdesk = true
						--LogInfo("[AGENT.USERS:UpdateUser] Consume helpdesk license --> userData %s.", json.encode(userData))
						local success, noAvailableLicense = authFilter:DecreaseVCCAvailableLicenses({
							username = simpleUserId,
							vcc = vcc,
							user = userData
						})
						-- Si success no es true, no significa que exista error.
						if success and not noAvailableLicense then
							--Mando notificacion al usuario de consumo de licencias para que habilite la visualizacion.
							agentNotifications.NotifyAgents({
								vcc = vcc,
								level = "ALWAYS",
								scope = "agent",
								eventType = "consume_helpdesk_licenses",
								agents = { fullUserId },
								pollerMessage = {
									status = "ENABLE" --Para habilitar las opciones de helpdesk del agente
								}
							}, keyspace)
							LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] A license helpdesk for user %s was consumed.", fullUserId)
						else
							-- Aqui no se pudo consumir licencia de helpdesk porque no existen disponibles.
							-- Solo se deja log
							LogError("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] Error consumming helpdesk license for user %s, no available licenses", fullUserId)
						end
					end
				end
			end

		elseif member_info.action == "removed" then
			--LogDebug("    removed -> member_id: %s", member_id)
			local member = member_info.member
			-- LogDebug("B --- REMOVING MEMBER %s, %s", member_id, json.encode(member))

			--Obtener los datos  del usuario para registrar record de auditoria en Entidad User (Se remueve vinculo de campaign)
			local beforeSaveUser = GetUserAuditData(fullUserId, keyspace, {
				overridePermissions = overridePermissions,
				campaigns = {campaign_with_vcc}
			})

			UpdateUserDefaultCampaign ( vcc, member_id, false, simple_camp_id, keyspace, member_info.action )
			UpdateMemberPermissions( member_id, nil --[[ elimina los permisos del member --]] )
			UpdateMemberCapability( member_id, nil, "REMOVE" )

			if not membersByAction.removed then
				membersByAction.removed = {}
			end
			membersByAction.removed[member_id] = member_info

			--Crear registro de auditoria para Entidad User (Se remueve vinculo de campaign)
			--LogDebug("2 - ADDING MEMBER Arranco GetUserAuditData")
			local newFields = GetUserAuditData(fullUserId, keyspace, {overridePermissions = overridePermissions})
			audits.Create({
				vcc = vcc,
				entity = "User",
				user = inconcert.authUser.userName,
				fieldKey =  member_id,
				newFields = newFields or {},
				oldFields = beforeSaveUser or {},
				connection = connectionHistorical
			})
			--LogDebug("2.1--- Termino audits.Create")
		else
			-- updated
			--LogDebug("    updated -> member_id: %s", member_id)
			local member = member_info.member
			--	LogDebug("C --- UPDATING MEMBER %s\nnew_member_info: %s\nold_member_info: %s", member_id, json.encode(member), json.encode(old_member))
			UpdateWorkgroup( member_id, member )
			UpdateMemberPermissions( member_id, member.campaign_permissions )
			UpdateMemberCapability( member_id, member.member_capability, "UPDATE" )
			if not membersByAction.updated then
				membersByAction.updated = {}
			end
			membersByAction.updated[member_id] = member_info
		end
	end

	LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process LOOP new_members. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	LogInfo("==> [ADMINISTRATION.CAMPAIGNS:UpdateMembersList] added: %d, updated: %d, removed: %d,", added, updated, removed)

	if (added - removed) ~= 0 then
		-- Actualizo los totales en redis de la relacion campana/users.
		modelRestrictions.UpdateRedisConfigTotals(vcc, {
			MaxCampaignsUsers = (added - removed)
		}, keyspace)
	end

	local allPermissionsOnCampaign
	if ( added + updated + removed ) > 0 then
		-- si al menos agrego, o actualizo, o elimino uno, guardo los permisos de los members en la campana
		startSubProcessTime = os.time()
		applicationsActions = applicationsActions or model_perm.GetApplicationsActions(keyspace, vcc)
		LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process GetApplicationsActions. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
		startSubProcessTime = os.time()
		local response = SaveAllPermissionsOnCampaign(simple_camp_id,
			nil --[[ roles permissions --]],
			membersPermissions,
			new_members,
			keyspace,
			applicationsActions,
			connectionHistorical,
			isCreateCampaign
		)
		LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process SaveAllPermissionsOnCampaign. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

		allPermissionsOnCampaign = response and response.allPermissionsOnCampaign
		if not response.success then 
		-- Si existen usuarios en conflicto con los Attention levels, los agrego a lista de usuarios en conflicto
			for user_id, fail in pairs(response.problematic_users or {}) do
				local err = {
					id = utils.GetId(user_id), -- id sin @vcc
					type = response.result,
					description = response.reason
				}
				if response.result == "USER_HAS_INTERACTIONS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
					err.assigned_interactions = fail
				elseif response.result == "USER_IN_ATTENTION_LEVELS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
					err.als = fail
				end
				table.insert(problematicUsers, err)
			end
		end

		-- Se crean los registros con los usuarios a remover, actualizar o agregar en la campaign para la subEntidad members
		startSubProcessTime = os.time()
		for action, members in pairs(membersByAction) do
			local FilterMembers = function (members, allMembersData)
				allMembersData = allMembersData or {}
				local result = {}
				for key, _ in pairs(members) do
					result[key] = allMembersData[key]
				end
				return result
			end
			local paramsAudit = {
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =  utils.GetId(campaign_with_vcc),
				subEntity = "Members",
				connection = connectionHistorical
			}

			if  action == "added" then
				--LogDebug("3.1 - ADDING MEMBER Arranco ..")
				paramsAudit.newFields = GetAuditMembers(members, campaign_with_vcc, keyspace, cachedInteractionTypes)
				audits.Create(paramsAudit)
				--LogDebug("3.2--- Termino audits.Create")
			elseif action == "updated" then
				--LogDebug("3.3 - ADDING MEMBER Arranco ..")
				paramsAudit.newFields = GetAuditMembers(members, campaign_with_vcc, keyspace, cachedInteractionTypes)
				paramsAudit.oldFields = FilterMembers(members, arr_old_audit_members)
				audits.Create(paramsAudit)
				--LogDebug("3.3--- Termino audits.Create")
			elseif action == "removed" then
				--LogDebug("3.4 - ADDING MEMBER Arranco ..")
				paramsAudit.oldFields = FilterMembers(members, arr_old_audit_members)
				audits.Create(paramsAudit)
				--LogDebug("3.5--- Termino audits.Create")
			end
		end
		LogDebug("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList]    ...END sub process LOOP actualizacion AUDITORIA. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	end

	if connectionHistorical then
		connectionHistorical:Close()
	end

	LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembersList] --> END Updated Campaign Members '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))

	-- Marco en Cassandra que ha comenzado la actualizacion de Members contra ALLEGRO
	local updateMembersResult = {
		finish        = "OK",
		stepNumber    = 6,
		status        = "END UPDATE MEMBERS",
		time          = os.time(),
		executionType = executionType or "MANUAL BY USER"
	}
	UpdateCampaignMembersResult(campaign_with_vcc, updateMembersResult, keyspace)

	if next( problematicUsers ) then
		--LogWarning("Error updating '%s' campaign members. Problematic members: %s", simple_camp_id, json.encode( problematicUsers ) )
		return false, { result = "PROBLEMATIC_USERS", problematicUsers = problematicUsers }
	end

	return true, nil, allPermissionsOnCampaign
end

--[[
]]--
function UpdateMembers (web)
	--LogInfo("[ADMINISTRATION.UpdateMembers Init date %s", LuaOrm.Date())
	local startCurrentProcessTime = os.time()
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	local function returnValue( status, response )
		keyspace:Close()
		utils.CollectAllMemory() -- invocamos a garbage ya que esta función realiza muchas conexion contra cassandra y otras cositas, con esto nos cuidamos !!
		web.status = status
		LogInfo("[ADMINISTRATION.CAMPAIGNS:UpdateMembers] End Update Campaign Members '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
		return response
	end

	local simple_camp_id = utils.GetId( web.request.campaignId )
	if not simple_camp_id then
		return returnValue( 400, {result = "INVALID_REQUEST", reason = "Missing field campaignId"} )
	end

	local members = web.request.members
	if type(members ) ~= "table" then
		return returnValue( 400, {result = "INVALID_REQUEST", reason = "Missing or invalid type in field members"} )
	end

	local vcc           = inconcert.authUser.virtualContactCenter
	local userId        = inconcert.authUser.userName.."@"..vcc
	local campaignId    = simple_camp_id .. "@" .. vcc
	local hasPermission = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil, keyspace)

	if hasPermission and not inconcert.authUser.api then
		keyspace:Close()
		return returnError(web, {
			status = 403,
			result = "forbidden",
			reason = string.format("[ADMINISTRATION.UpdateMembers] Unable to update member. Campaign '%s'. You don't have permissions.", campaignId)
		})
	end

	local success, err = UpdateMembersList(simple_camp_id, members, keyspace, allegroOtherInfo)
 	if not success then
		if err then
			if err.result == "PROBLEMATIC_USERS" then
				local first_problematic_member = err.problematicUsers[1]
				return returnValue( 400, { result = first_problematic_member.type, reason = first_problematic_member.description, problematicUsers = err.problematicUsers } )
			end
			return returnValue( 400, err )
		end
	end
	--LogInfo("[ADMINISTRATION.UpdateMembers End date %s", LuaOrm.Date())

	return returnValue( 200, { result = "OK" } )
end

--[[
]]--
local function __SaveCampaignScheduler (camp_id, scheduler, keyspace, allegroOtherInfo)
	if not scheduler then
		return false
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.__SaveCampaignScheduler] - Saving scheduler for campaign '%s' by User '%s'. Scheduler: %s", camp_id, user_id, json.encode(scheduler))

	camp_id = utils.GetId(camp_id)
	local full_camp_id = utils.AppendVcc(camp_id, vcc)
	local success, err = GetModel("Campaigns").SaveScheduler(full_camp_id, scheduler, keyspace, allegroOtherInfo)
	if not success then
		if not err then
			LogError("[ADMINISTRATION.__SaveCampaignScheduler] - Unable to save campaign '%s' scheduler.", full_camp_id)
		elseif err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			LogError("[ADMINISTRATION.__SaveCampaignScheduler] - Unable to save campaign '%s' scheduler. Allegro Integration error - Err: '%s'", full_camp_id, json.encode(err.type) )
		end
	end

	--notifications
	notificationsEngine.SendNotification("CampaignSchedulerUpdated", 0, { campaignId = camp_id, scheduler = scheduler })

	return success, err
end

function SaveScheduler (web)
	local campaignId = web.request.campaignId
	local scheduler = web.request.scheduler
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	if not campaignId then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field campaignId"}
	end

	if not scheduler then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing table scheduler"}
	end

	if not scheduler.scheduleSpecialDays then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field scheduleSpecialDays"}
	end

	if not scheduler.scheduleSpecialDayAllYearss then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field scheduleSpecialDayAllYearss"}
	end

	if not scheduler.scheduleDays then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field scheduleDays"}
	end

	for _, scheduleDay in ipairs(scheduler.scheduleDays) do
		if not scheduleDay.dayOfWeek or not scheduleDay.startTime or not scheduleDay.endTime then
			web.status = 400
			return {result = "INVALID_REQUEST", reason = "Schedule days has invalid format"}
		end
		if tonumber(scheduleDay.dayOfWeek) < 1 or tonumber(scheduleDay.dayOfWeek) > 7 then
			web.status = 400
			return {result = "INVALID_REQUEST", reason = "Schedule dayOfWeek must be between 1 and 7"}
		end
	end


	-- Obtengo datos actuales de auditoria para la campaign antes de salvar scheduler
	local connection       = utils.GetDatasource():CassandraConnection( "Agent" )
	local full_camp_id     = utils.AppendVcc(utils.GetId(campaignId), vcc)
	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, full_camp_id, connection)
	
	if not belongs then
		return belResp
	end

	local old_campaign_schedulers = GetAuditCampaign(full_camp_id, connection, "Scheduler")
	local success, err = __SaveCampaignScheduler(campaignId, scheduler, nil, allegroOtherInfo)

	if not success then
		if not err then
			web.status = 400
			connection:Close()
			return { result = "UNABLE_TO_SAVE_CAMPAIGN_SCHEDULER", reason = "Unable to save campaign scheduler"}
		else
			-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
			--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
			--  la actualizacion en i6.
			if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
				web.status = 400
				connection:Close()
				return { result = "UNABLE_TO_SAVE_CAMPAIGN_SCHEDULER", reason = err.type }
			else
			-- no es un error
			end
		end
	end

	--Success: Se crea el registro de auditoria.
	local curr_campaign_schedulers = GetAuditCampaign(full_camp_id, connection, "Scheduler")
	connection:Close()
	if old_campaign_schedulers and  curr_campaign_schedulers then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignId),
			oldFields = old_campaign_schedulers,
			newFields = curr_campaign_schedulers,
			subEntity = "Scheduler",
			connection = ksHistorical
		})
		ksHistorical:Close()
	else
		LogWarning("[ADMINISTRATION.SaveScheduler] Audit record not created")
	end

	return { result = "OK" }
end

local function __UpdateAccountMessages (full_camp_id, account_id, messages, keyspace, allegroOtherInfo)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.__UpdateAccountMessages] - Saving '%s' account messages for campaign '%s' by User '%s'. Messages: %s", account_id, full_camp_id, user_id, messages)
	--LogDebug("[ADMINISTRATION.__UpdateAccountMessages] ---> Saving '%s' account messages for campaign '%s'. Messages: %s", account_id, full_camp_id, json.encode(messages))

	local vccTtsAsrEngines = GetModel("TtsAsrEngines").GetTtsAsrEngines(vcc, keyspace)
	local function CanIncludeMessageOption (messageOption)
		if messageOption.MessageType == "audio" then
			return true
		end
		if #(vccTtsAsrEngines or {}) == 0 then
			-- Si el VCC no tiene configurado TTS, solo se guardaran los MessageType "audio" y "variable" de tipo "file".
			if messageOption.MessageType == "tts" or (messageOption.MessageType == "variable" and messageOption.Engine:upper() == "TTS") then
				-- No lo agrega
				return false
			end
			-- Si, lo agrega
		end
		return true
	end
	--[[
		Funcion encargada de limpiar los mensajes que no se pueden incluir.
		Ejemplo: los mensajes de TTS si el VCC no tenía configurado TTS y después se le removió la configuración.
	]]--
	local function CleanUpAccountMessages (sourceMessages)
		local targetMessages = {}
		for messageType, message in pairs(sourceMessages) do
			if message.isComposite and message.type:lower() == "file" then
				local options = {}
				for _, messageOption in ipairs(message.options or {}) do
					if CanIncludeMessageOption(messageOption) then
						table.insert(options, messageOption)
					else
						LogWarning("[ADMINISTRATION.__UpdateAccountMessages] Exclude Option Message '%s'", json.encode(messageOption))
					end
				end
				message.options = options
			end
			targetMessages[messageType] = message
		end
		--LogDebug("[ADMINISTRATION.__UpdateAccountMessages] ---> CleanUpAccountMessages - targetMessages: %s", json.encode(targetMessages))
		return targetMessages
	end

	messages = CleanUpAccountMessages(messages)

	if not full_camp_id then
		return false, "Field campaignId is missing"
	end

	if not account_id then
		return false, "Field accountId is missing"
	end

	if not messages then
		return false, "Field messages is missing"
	end

	local old_account_audit_data = GetAuditAccount(account_id, full_camp_id, keyspace)
	GetModel("Campaigns").SaveAccountMessages(full_camp_id, account_id, messages, keyspace, allegroOtherInfo)
	--Create audit record for Account Message
	local new_account_audit_data = GetAuditAccount(account_id, full_camp_id, keyspace)
	if old_account_audit_data and new_account_audit_data then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(full_camp_id),
			oldFields = old_account_audit_data,
			newFields = new_account_audit_data,
			subEntity = "Accounts",
			connection = ksHistorical
		})
		ksHistorical:Close()
	else
		LogWarning("[ADMINISTRATION.__UpdateAccountMessages] - Audit record not created")
	end

	return true
end

function SaveCampaign (web)
	local ok, campaign = pcall(json.decode, web.input["post_data"])
	if not ok then
		LogError("[ADMINISTRATION.SaveCampaign] Error saving Campaign. posted data: %s", web.input["post_data"])
		web.status = 400
		web.headers["Content-Type"] = "application/json"
		return {result = "INVALID_REQUEST"}
	end

	local is_new_campaign = campaign.isNew
	local vcc = inconcert.authUser.virtualContactCenter
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, keyspace)
	local userIdWithoutVCC = inconcert.authUser.userName
	local userId = userIdWithoutVCC.."@"..vcc
	
	campaign.Id = campaign.Id:lower()
	-- TODO: This campaign id is duplicated. seems wrong to me.
	campaign.campaignId = campaign.Id:lower()
	campaign.VCC = vcc
	campaign.isNew = nil

	local full_campaign_id = utils.AppendVcc(campaign.Id, vcc)

	LogInfo("[ADMINISTRATION.SaveCampaign] Saving campaign, user %s, vcc %s, \ncampaign_post_data: %s", userId, vcc, web.input["post_data"])

	local action           = is_new_campaign and "create" or "modify"
	local belongs, belResp = ValidateBelongsCampaign (web, action, userId, full_campaign_id, keyspace)

	if not belongs then
		return belResp
	end

	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)
	--Save audits params
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	local auditRecord = {
		vcc = vcc,
		entity = "Campaign",
		user = userIdWithoutVCC,
		fieldKey = utils.GetId(campaign.campaignId),
		connection = ksHistorical
	}

	if is_new_campaign then
	-- 1 - Creando la CAMPANA
	--
		-- aca valido el permisos de creacion de campanas
		local creation_action = "create_campaign"
		local has_permission = permissions.IsPermissionEnabledForLoggedUser("administration" , creation_action, nil, keyspace)
		if not has_permission then
			-- Error
			keyspace:Close()
			ksHistorical:Close()
			web.status = 403
			local reason = LogError("[ADMINISTRATION.SaveCampaign] Unable to create campaign '%s'. You not has permissions.", full_campaign_id)
			return { result = "forbidden", reason = reason }
		end

		local success, saved_campaign

		--Valido que no exceda el limite de campanas
		local canAddCampaign , error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxCampaigns", keyspace)
		if canAddCampaign then
			--Valido si con el agregado del usuario por defecto, no excede el limite permitidos de la relacion CampaignsUsers 
			canAddCampaign, error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxCampaignsUsers", keyspace)
			saved_campaign = not canAddCampaign and {result = error or "ERROR_CAMPS_USERS_EXCEEDS_LIMIT_ALLOWED"}  
		else
			saved_campaign = {result = error or "ERROR_CAMPS_EXCEEDS_LIMIT_ALLOWED"}
		end

		if canAddCampaign then
			success, saved_campaign = agentCampaignController.CreateCampaign(campaign, keyspace, allegroOtherInfo)
		end

		if success then
			-- OK
			-- Create new Audit Record
			local curr_campaign = GetAuditCampaign(full_campaign_id, keyspace)
			auditRecord.newFields = curr_campaign
			audits.Create(auditRecord)

			-- Update Campaigns totals en redis
			modelRestrictions.UpdateRedisConfigTotals(vcc, {MaxCampaigns = 1}, keyspace)

			local members = {}
			-- Agrego el Current User a la Campana.
			_AssignUserToCampaign(userIdWithoutVCC, inconcert.authUser.userGroup, members, nil, full_campaign_id, keyspace)
			-- Agrego los Super Usuarios a la Campana.
			_AssignSuperUsersToCampaign(vcc, userIdWithoutVCC, members, full_campaign_id, keyspace)
			local successUpd, err, allPermissionsOnCampaign = UpdateMembersList(campaign.Id, members, keyspace, allegroOtherInfo, true, true)

			keyspace:Close()
			ksHistorical:Close()
			return {result = "OK", saved_campaign = saved_campaign, allPermissionsOnCampaign = allPermissionsOnCampaign or nil}
		end
		-- Error
		keyspace:Close()
		ksHistorical:Close()
		web.status = 500
		local err = saved_campaign
		LogError("[ADMINISTRATION.SaveCampaign] Unable to create campaign '%s'. Err: %s ", full_campaign_id, json.encode(err))
		return err
	end

	-- 2 - Actualizando la CAMPANA
	--
	-- aca valido el permisos de actualizacion de campanas
	local has_permission = permissions.IsPermissionEnabledForLoggedUser("administration" , "update_campaign_basic", full_campaign_id, keyspace)
	if not has_permission then
		keyspace:Close()
		ksHistorical:Close()
		web.status = 403
		local reason = LogError("[ADMINISTRATION.SaveCampaign] Unable to update campaign '%s'. You not has permissions.", full_campaign_id)
		return { result = "forbidden", reason = reason }
	end

	local old_campaign = GetAuditCampaign(full_campaign_id, keyspace)
	local success, err = agentCampaignController.SaveCampaign(campaign, keyspace, allegroOtherInfo)
	local curr_campaign = GetAuditCampaign(full_campaign_id, keyspace)

	if not success then
		-- Error
		LogError("[ADMINISTRATION.SaveCampaign] Unable to update campaign '%s'. Err: %s ", full_campaign_id, json.encode(err))
		if err.result == "UNEXISTENT CAMPAIGN" then
			web.status = 404
		else
			web.status = 500
		end
		return err
	end
	-----
	-- OK
	--
	-- Create Audit Record
	LogInfo("[ADMINISTRATION.SaveCampaign] Create Audit Record campaign %s curr_campaign %s", full_campaign_id, json.encode(curr_campaign))
	if curr_campaign then
		auditRecord.newFields = curr_campaign
		auditRecord.oldFields = old_campaign
		audits.Create(auditRecord)
	end
	keyspace:Close()
	ksHistorical:Close()
	return {result = "OK", saved_campaign = ""}
end

--[[
	-- Delete en Allegro la campana
]]--
local function __DeleteAllegroCampaignIn55 (campaign_with_vcc, keyspace, allegroOtherInfo)
	local vcc = inconcert.authUser.virtualContactCenter

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION.CAMPAIGNS:__DeleteAllegroCampaignIn55] - Allegro integration. Deleting campaign '%s'. Vcc '%s' has not configured integration with allegro.", campaign_with_vcc, vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_campaigns.Delete(campaign_with_vcc, vcc, allegroOtherInfo)
	if not success then
		if err.type == "ALLEGRO_INTEGRATION_ERROR" then
			LogError("[ADMINISTRATION.CAMPAIGNS:__DeleteAllegroCampaignIn55] - Allegro integration error. Deleting campaign '%s'. Response: '%s'.", campaign_with_vcc, json.encode(err))
		else
			LogInfo("[ADMINISTRATION.CAMPAIGNS:__DeleteAllegroCampaignIn55] - Allegro integration. Deleting campaign '%s'. Response: '%s'.", campaign_with_vcc, json.encode(err))
		end
		return false, err
	end

	return true
end

--[[
	Valida si la campana se encuentra configurada en algun Proceso de la App OutboundEngine.
]]--
local function HasConfiguredCampaignInSomeOutboundEngineProcess (campaignIdAtVCC, keyspace)
	assert(campaignIdAtVCC)
	local vcc = inconcert.authUser.virtualContactCenter
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, keyspace)

	if hasAllegroIntegration then
		-- Para VCC con Integracion con Allegro no Aplica dicha validacion.
		return false
	end

	LogInfo("[ADMINISTRATION.CAMPAIGNS:HasConfiguredCampaignInSomeOutboundEngineProcess] Validating if the Campaign is Configured in an Outbound Engine. Campaign: %s.", campaignIdAtVCC)

	local success, OEConfig = GetModel("Campaigns").GetOEConfig(campaignIdAtVCC, keyspace)
	--LogDebug("0.5 OEConfig: %s:", utils.JSON_Encode(OEConfig))
	if not success then
		--LogDebug("1")
		return false
	end
	if not OEConfig or not OEConfig.processId then
		--LogDebug("2")
		return false
	end
	-- Si, la campana esta configurada en un proceso de la App OE.
	local err = {httpCode = 400, errType = "CAMPAIGN HAS AN OUTBOUND PROCESS CONFIGURED", processId = OEConfig.processId, campaignId = campaignIdAtVCC}
	return true, err, OEConfig
end

local function ValidateCampaignToRemove (info, keyspace, allegroOtherInfo)
	assert(keyspace)
	local vcc = inconcert.authUser.virtualContactCenter
	local campExists = GetModel("Campaigns").CampaignExists(info.campaignIdAtVCC, keyspace)
	if not campExists then
		return false, {httpCode = 404, errType = "UNEXISTENT CAMPAIGN"}
	end
	local isInternal = GetModel("Campaigns").GetSpecificColumn(info.campaignIdAtVCC, "IsInternal", keyspace)
	if isInternal == "1" then
		return false, {httpCode = 404, errType = "UNABLE TO DELETE INTERNAL CAMPAIGN"}
	end
	-- Valida si la campana se encuentra en asignada en algun Proceso de la App OutboundEngine.
	local isUsedInOEProcess, result = HasConfiguredCampaignInSomeOutboundEngineProcess(info.campaignIdAtVCC, keyspace)
	if isUsedInOEProcess then
		local err = result
		return false, err
	end
	local hasPermission = permissions.IsPermissionEnabledForLoggedUser("administration" , "deletecampaign", info.campaignIdAtVCC, keyspace)
	if not hasPermission then
		return false, {httpCode = 403, errType ="FORBIDDEN"}
	end
	local campaignAccounts = GetModel("Campaigns").GetCampaignAccounts(info.campaignIdAtVCC, keyspace)
	local success, err = agentCampaignController.ValidateCampaignAccountsToRemove(info.campaignIdAtVCC, campaignAccounts, keyspace)
	if not success then
		return false, {httpCode = 500, errType = err.errType, errors = err.errors}
	end
	success, err = __DeleteAllegroCampaignIn55(info.campaignIdAtVCC, keyspace, allegroOtherInfo)
	if not success then
		-- Solo tiene que retornar sin eliminar la campana en i6, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que es otro tipo de error, y puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar la eliminacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			return false, {httpCode = 500, errType = err.type}
		end
	end
	-- ok
	return true, {
		campaignAccounts = campaignAccounts
	}
end

local function GetAllAuditAccounts (full_camp_id, keyspace, campaignAccounts)
	local accounts = campaignAccounts or GetModel("Campaigns").GetCampaignAccounts( full_camp_id, keyspace )

	local result = {}
	for _, account in pairs( accounts or {} ) do
		local auditAccountData = GetAuditAccount(account, full_camp_id, keyspace)
		if auditAccountData then
			table.insert(result, auditAccountData)
		end
	end
	return result
end	

local function EraseCampaign (campaignIdAtVCC, removeData, allegroOtherInfo, ksAgent)
	assert(ksAgent)
	local vcc = inconcert.authUser.virtualContactCenter

	local info = {
		campaignIdAtVCC = campaignIdAtVCC,
		allegroOtherInfo = allegroOtherInfo
	}
	local success, resp = ValidateCampaignToRemove(info, ksAgent, allegroOtherInfo)
	if not success then
		--LogDebug("[ADMINISTRATION.EraseCampaign] --> Validation error: %s", json.encode(resp or {}))
		local err = resp
		return false, err.httpCode, err
	end

	-- ok - se puede eliminar
	local campaignAccounts = resp.campaignAccounts
	local campaignAccountsMessages = GetModel("Campaigns").GetCampaignAccountsMessages(campaignIdAtVCC, campaignAccounts, ksAgent) or {}
	local campaignId = utils.GetId(campaignIdAtVCC)

	-- local currentOEConfig
	-- local success, config = GetModel("Campaigns").GetOEConfig(campaignIdAtVCC, ksAgent)
	-- if success then
	-- 	currentOEConfig = config
	-- end

	--[[
		Audit:
		Obtengo todos los datos actuales por cada subEntidad, para crear registro de auditoria de Delete por cada uno,
		ya que se elimina la Campaign
	]]--
	local old_audit_campaign_data = GetAuditCampaign(campaignIdAtVCC, ksAgent) -- base
	local old_audit_scheduler_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Scheduler")
	local old_audit_permissions_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Permissions")
	local old_audit_attentionLevels_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "AttentionLevels")
	local old_audit_integrations_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Integrations")
	local old_audit_notifications_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Notifications")
	local old_labels = GetModel("Campaigns").GetLabels(campaignIdAtVCC, ksAgent)
	local old_templates = GetModel("Campaigns").GetCannedMessages(campaignIdAtVCC, ksAgent)
	local old_audit_dispotition_code_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "DispositionCodes")
	local old_audit_channels_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Channels")
	local old_audit_thresholds_data = GetAuditCampaign(campaignIdAtVCC, ksAgent, "Thresholds")
	local old_rules = GetModel("Campaigns").GetRules(campaignIdAtVCC, ksAgent)
	local old_audit_accounts_data = GetAllAuditAccounts(campaignIdAtVCC, ksAgent, campaignAccounts)
	local members = GetModel("Campaigns").GetCampaignMembers(vcc, campaignIdAtVCC, ksAgent )
	local old_audit_members_data = GetAuditMembers(members, campaignIdAtVCC, ksAgent)
	--Para Entidad User (Como se remueve vinculo de campaign), se obtienen los datos del campo campaigns

	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, ksAgent)
	local old_audit_users_data = {}
	local cont_campaigns_members = 0
	if old_audit_members_data and not utils.TableIsEmpty(old_audit_members_data) then
		for key in pairs(old_audit_members_data) do
			cont_campaigns_members = cont_campaigns_members + 1
			local simpleUserId = utils.GetId(key)
			local fullUserId = utils.AppendVcc(simpleUserId, vcc)
			local overridePermissions = usersOverridePermissions[fullUserId] and usersOverridePermissions[fullUserId].actions or nil
			old_audit_users_data[key] = GetUserAuditData(fullUserId, ksAgent, {
				overridePermissions = overridePermissions,
				campaigns = {campaignIdAtVCC}
			})
		end
	end

	--We must do this BEFORE campaign had been deleted from cassandra

	for _, account in pairs(campaignAccounts) do
		account.campaignId = campaignIdAtVCC
		DelAccountWhatsappTemplates(account)
	end

	local deleted = agentservice.DeleteCampaign(campaignId, vcc)
	if not deleted then
		return false, 500, "AGENT_SERVICE_ERROR"
	end

	-- xxxxxxxxxxxxxxxxxxxxxx
	GetModel("Campaigns").CleanUpMailCampaignAccountsMessagesRepoFiles(campaignIdAtVCC, campaignAccountsMessages, ksAgent)

	-- remueve todos los flows asociados a las cuentas
	GetModel("Campaigns").RemoveAccountsChannelTasks(campaignIdAtVCC, campaignAccounts, ksAgent, removeData)

	-- remueve todos los signatures asociados a las cuentas
	GetModel("Campaigns").RemoveAccountsSignature(campaignIdAtVCC, campaignAccounts, ksAgent)

	-- remueve todos los flows asociados a las cuentas
	GetModel("Campaigns").RemoveAccountsFlow(campaignIdAtVCC, campaignAccounts, ksAgent)

	-- actualizo la relación de la campaña de Call de i6 (sin allegro), con los paquetes (vccpackages)
	local ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
	GetModel("Campaigns").RemoveCalli6AccountPackagesRelations(campaignIdAtVCC, campaignAccounts, ksAgent, ksProv)
	ksProv:Close()

	-- Elimino las relaciones de las blacklists con las cuentas.
	GetModel("Campaigns").RemoveBlacklistsAccounts(campaignIdAtVCC, campaignAccounts, ksAgent)

	-- Disposition codes and Schedulers are deleted together with the campaign
	-- We must delete supervisorworkgroups from all supervisors
	-- remueve el capability para la campana. ( campaign id, members )
	channelsController.OnRemovedCampaign(campaignIdAtVCC, members, ksAgent)

	-- remueve todos los attention levels de la campana
	GetModel("Campaigns").DeleteAttentionLevels(campaignIdAtVCC, nil, ksAgent)

	-- remueve todos los rules de la campana
	GetModel("Campaigns").DeleteRules(campaignIdAtVCC, nil, ksAgent)

	-- remueve todos los schedulers special days de la campana
	GetModel("Campaigns").RemoveSchedulerSpecialDays(campaignIdAtVCC, ksAgent)

	-- remuevo las relaciones de shared files con las cuentas de la campana
	local hasRemoveRepositoryFiles = true
	for _, account in pairs(campaignAccounts or {}) do
		GetModel("Campaigns").RemoveAccountMessages(campaignIdAtVCC, account.id, hasRemoveRepositoryFiles, nil, ksAgent)
	end

	local applicationsActions = model_perm.GetApplicationsActions(ksAgent, vcc)

	for user_id, role in pairs( members ) do
		LogInfo("User %s has role %s, we need to delete the associated workgroup", user_id, role)
		local user = GetModel("Users").GetUser(vcc, user_id, ksAgent, { "Id", "UserGroup", "SupervisorWorkgroups" })
		if user then
			-- si tiene permisos sobre la app supervisor, actualiza los supervisor workgroups
			local hasPermissionToSupApp = permissions.HasPermissionToApplication(
				"supervisor",
				user.Id,
				user.UserGroup,
				campaignIdAtVCC,
				ksAgent,
				{
					applicationsActions = applicationsActions
				}
			)
			if hasPermissionToSupApp then
				user.SupervisorWorkgroups = user.SupervisorWorkgroups or {}
				user.SupervisorWorkgroups[campaignId] = nil
				GetModel("Users").SaveUser(vcc, user, ksAgent)
			end

			-- Update or delete the default campaign for each user belonging to the campaign to be deleted
			UpdateUserDefaultCampaign ( vcc, user.Id, false, campaignId, ksAgent, "removed" )
			
		end
	end

	--finally we must delete campaign members
	GetModel("Campaigns").DeleteCampaignMembers(vcc, campaignId, ksAgent)

	local connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
	--Agrego registro de auditoria para delete campaign
	audits.Create({
		vcc = vcc,
		entity = "Campaign",
		user = inconcert.authUser.userName,
		fieldKey =   campaignId,
		oldFields = old_audit_campaign_data,
		connection = connectionHistorical
	})

	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, ksAgent)

	--Agrego registro de auditoria para delete members
		--1 Crear audit record para miembros de la campanna
		--2 Crear audit record para User (ya q se elimina la relacion)
	if old_audit_users_data and not utils.TableIsEmpty(old_audit_users_data) then
		--LogInfo("[ADMINISTRATION.DeleteCampaign] Audit old_audit_users_data %s", json.encode(old_audit_users_data))

		for key, value in pairs(old_audit_users_data) do
			--Crear registro de auditoria para Entidad User (Se remueve vinculo de campaign)
				local simpleUserId = utils.GetId(key)
				local fullUserId = utils.AppendVcc(simpleUserId, vcc)
				local overridePermissions = usersOverridePermissions[fullUserId] and usersOverridePermissions[fullUserId].actions or nil
				local newUserAuditData = GetUserAuditData(fullUserId, ksAgent, {overridePermissions = overridePermissions})
				local oldUserAuditData = value
				--LogInfo("[ADMINISTRATION.DeleteCampaign] Audit newUserAuditData %s", json.encode(newUserAuditData or {}))
				if newUserAuditData and oldUserAuditData then
					audits.Create({
						vcc        = vcc,
						entity     = "User",
						user       = inconcert.authUser.userName,
						fieldKey   =  simpleUserId,
						newFields  = newUserAuditData,
						oldFields  = oldUserAuditData,
						connection = connectionHistorical
					})
				end
		end

		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =   campaignId,
			oldFields = old_audit_members_data,
			subEntity = "Members",
			connection = connectionHistorical
		})
	end

	--Agrego registro de auditoria para delete Schedulers
	if old_audit_scheduler_data and not utils.TableIsEmpty(old_audit_scheduler_data) then
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =   campaignId,
			oldFields = old_audit_scheduler_data,
			subEntity = "Schedulers",
			connection = connectionHistorical
		})
	end

	--Agrego registro de auditoria para delete Integrations
	if old_audit_integrations_data and not utils.TableIsEmpty(old_audit_integrations_data) then
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =   campaignId,
			oldFields = old_audit_integrations_data,
			subEntity = "Integrations",
			connection = connectionHistorical
		})
	end

	--Agrego registro de auditoria para delete Notifications
	if old_audit_notifications_data and not utils.TableIsEmpty(old_audit_notifications_data) then
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =   campaignId,
			oldFields = old_audit_notifications_data,
			subEntity = "Notifications",
			connection = connectionHistorical
		})
	end

	--Agrego registro de auditoria para delete Permissions
	if old_audit_permissions_data and not utils.TableIsEmpty(old_audit_permissions_data) then
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =   campaignId,
			oldFields = old_audit_permissions_data,
			subEntity = "Permissions",
			connection = connectionHistorical
		})
	end

	--Agrego registro de auditoria para delete rules
	for key, value in pairs(old_rules or {}) do
		local audit_rule_data = FilterAuditFields(value, "Rules")
		if audit_rule_data and not utils.TableIsEmpty(audit_rule_data) then
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = {[key] = audit_rule_data},
				subEntity = "Rules",
				connection = connectionHistorical
			})
		end
	end

	--Agrego registro de auditoria para delete Labels
	for key, value in pairs(old_labels or {}) do
		local audit_label_data = FilterAuditFields(value, "Labels")
		if audit_label_data and not utils.TableIsEmpty(audit_label_data) then
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = {[key] = audit_label_data},
				subEntity = "Labels",
				connection = connectionHistorical
			})
		end
	end

	--Agrego registro de auditoria para delete Templates
	for key, value in pairs(old_templates or {}) do
		local audit_template_data = FilterAuditFields(value, "Templates")
		if audit_template_data and not utils.TableIsEmpty(audit_template_data) then
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = {[key] = audit_template_data},
				subEntity = "Templates",
				connection = connectionHistorical
			})
		end
	end

	--Agrego registro de auditoria para delete Thresholds
	if old_audit_thresholds_data and not utils.TableIsEmpty(old_audit_thresholds_data) then
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = old_audit_thresholds_data,
				subEntity = "Thresholds",
				connection = connectionHistorical
			})
	end

	--Agrego registro de auditoria para delete attention levels
	local cont_campaign_als = 0
	local cont_campaign_als_users = 0
	if old_audit_attentionLevels_data and not utils.TableIsEmpty(old_audit_attentionLevels_data) then
		for key, value in pairs(old_audit_attentionLevels_data) do
			cont_campaign_als = cont_campaign_als + 1
			cont_campaign_als_users = cont_campaign_als_users + utils.GetRowCount(value.users or {})
			audits.Create({  
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = {[key] =  value},
				subEntity = "AttentionLevels",
				connection = connectionHistorical
			})
		end
	end

	--Agrego registro de auditoria para delete DispositionCodes
	if old_audit_dispotition_code_data and not utils.TableIsEmpty(old_audit_dispotition_code_data) then
		for _, value in pairs(old_audit_dispotition_code_data) do
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = value,
				subEntity = "DispositionCodes",
				connection = connectionHistorical
			})
		end
	end

	--Agrego registro de auditoria para delete Channels
	if old_audit_channels_data and not utils.TableIsEmpty(old_audit_channels_data) then
		for key, value in pairs(old_audit_channels_data) do
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = {[key] =  value},
				subEntity = "Channels",
				connection = connectionHistorical
			})
		end
	end
	
	--Agrego registro de auditoria para delete Accounts
	if old_audit_accounts_data and not utils.TableIsEmpty(old_audit_accounts_data) then
		for _, account in pairs(old_audit_accounts_data) do
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =   campaignId,
				oldFields = account,
				subEntity = "Accounts",
				connection = connectionHistorical
			})
		end
	end

	-- Actualizo los totales en redis una vez eliminada la campanna.
	modelRestrictions.UpdateRedisConfigTotals(vcc, {
		MaxCampaigns = -1,
		MaxCampaignsUsers = -(cont_campaigns_members),
		MaxAttsLevel = -(cont_campaign_als),
		MaxAttsLevelUsers = -(cont_campaign_als_users)
	}, ksAgent)

	connectionHistorical:Close()

	--notifications
	notificationsEngine.SendNotification("CampaignDeleted", 0, {campaignId = campaignId})

	return true
end

function DeleteCampaign (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local post_data = web.request
	local campaignIdAtVCC = post_data[1]
	local data = nil

	if type(campaignIdAtVCC) == "table" then
		data = campaignIdAtVCC.data
		campaignIdAtVCC = campaignIdAtVCC.id
	end

	local ksAgent          = utils.GetDatasource():CassandraConnection( "Agent" )
	local campaign         = utils.GetId(campaignIdAtVCC)
	campaignIdAtVCC        = utils.AppendVcc(campaign, vcc)
	local appPermission    = "administration"
	local actionPermission = "admin_restricted"
	local hasPermission    = model_perm.IsPermissionEnabledForUser(appPermission, actionPermission, nil, ksAgent)

	LogInfo("[ADMINISTRATION.DeleteCampaign] Deleting campaign '%s'. User %s. hasPermission: '%s' for '%s->%s', inconcert.authUser.api: '%s'.",
		campaignIdAtVCC,
		userId,
		hasPermission,
		appPermission,
		actionPermission,
		inconcert.authUser.api
	)

	if hasPermission and not inconcert.authUser.api then
		ksAgent:Close()
		return returnError(web, {
			status   = 403,
			result   = "forbidden",
			reason   = string.format("[ADMINISTRATION.DeleteCampaign] Unable to delete campaign '%s'. You don't have permissions.", campaignIdAtVCC)
		})
	end

	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)
	local resultInfo
	local success, httpCode, result = EraseCampaign(campaignIdAtVCC, data, allegroOtherInfo, ksAgent)
	ksAgent:Close()
	LogDebug("[ADMINISTRATION.DeleteCampaign] Result of deleting campaign %s . success: %s, httpCode: %s, result: %s", campaignIdAtVCC, success, httpCode, json.encode(result or {}))
	local rawError = result
	local reason
	if type(result) == "table" then
		resultInfo = result.errors
		reason = result.reason or result.errType
	else
		reason = rawError
	end
	reason = reason or "UNRECOGNIZED ERROR"
	if not success then
		return returnError(web, {
			status   = httpCode,
			result   = reason,
			errors   = resultInfo,
			rawError = rawError,
			reason   = string.format("[ADMINISTRATION.DeleteCampaign] Unable to delete %s campaign, reason: %s", campaignIdAtVCC, reason)
		})
	end

	return {result = "OK"}
end

-- Valida si existe algun usuario de los que pertenezcan al Rol que tengan interacciones asignadas.
--  En el caso de que esto sea asi, tengo que validar si los permisos del Rol que se ha modificado, tiene acceso a la app "interaction".
--  Si no tiene acceso a la aplicacion "interaction" falla, con la lista de usuarios en conflicto.
--
local function ValidateRolePermissionInConflictWithUserAssignedInteractions( full_campaign_id, roles, new_roles_permissions, cachedCampaignsPermissions, keyspace, applicationsActions)
	local response = { success = true }
	local user_campaigns = {}
	user_campaigns[full_campaign_id] = true

	local cached_users_assigned_interactions = {}
	for role_id, role_info in pairs( roles or {} ) do

		for full_user_id in pairs( role_info.users or {} ) do

			local simple_user_id = utils.GetId(full_user_id)
			cached_users_assigned_interactions[full_user_id] = {
				count = admUserController.CountAssignedInteractions(simple_user_id, nil, keyspace)
			}

			--LogDebug("role_id: %s, simple_user_id: %s, assigned_interactions_count: %d", role_id, simple_user_id, cached_users_assigned_interactions[full_user_id].count)

			if cached_users_assigned_interactions[full_user_id].count > 0 then
				-- Obtengo los permisos del usuario, por campanas (la unica)
				-- con ella luego realizo la validación para ver si el rol del usuario quedaron con permisos a la aplicacion "interaction"
				--
				local permissions_by_campaigns = model_perm.MakeGroupedPermissionsByCampaigns(
					{
						get_campaign_permissions = function( campaign_id )
							return { groups = new_roles_permissions or {} }
						end,
						specific_user_group = role_id,
						user_campaigns = user_campaigns,
						user_id = full_user_id
					}
				)
				-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
				local hasPermissionToAppInteraction = permissions.HasPermissionToApplication(
					"interaction",
					full_user_id,
					role_id,
					full_campaign_id,
					keyspace,
					{
						user_campaign_permissions = permissions_by_campaigns,
						group_permissions         = role_info.permissions,
						campaigns_permissions     = cachedCampaignsPermissions,
						user_campaigns            = user_campaigns,
						applicationsActions       = applicationsActions
						-- user_vcc_permissions  -- no es necesario ya que unicamente se va a loopear una solo vez cada usuario, porque la relacion usuario role es 1 a 1,
					}
				)

				--LogDebug("  --A- full_user_id: '%s', role_id: '%s', perm: '%s'.", full_user_id, role_id, hasPermissionToAppInteraction)

				if not hasPermissionToAppInteraction then
					response.success = false
					response.result = "USER_HAS_INTERACTIONS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP"
					response.roles = response.roles or {}
					response.roles[role_id] = true
					response.count_users = (response.count_users or 0) + 1
					response.problematic_users = response.problematic_users or {}
					response.problematic_users[ full_user_id ] = cached_users_assigned_interactions[full_user_id].count
				end
			end
		end
	end

	return response, cached_users_assigned_interactions
end

-- Valida si existe algun usuario de los que pertenezcan al Rol que se encuentren asignados a algun AL de la campana.
--  En el caso de que esto sea asi, tengo que validar si los permisos del Rol que se ha modificado, tiene acceso a la app "interaction".
--  Si no tiene acceso a la aplicacion "interaction" falla, con la lista de usuarios en conflicto.
--
local function ValidateRolePermissionInConflictWithAttentionLevels(full_campaign_id, roles, new_roles_permissions, cachedCampaignsPermissions, keyspace, applicationsActions)
	local response = { success = true }
	local user_campaigns = {}
	user_campaigns[full_campaign_id] = true

	local cached_users_als = {}
	for role_id, role_info in pairs( roles or {} ) do

		-- recorro la lista de users y valido si tienen ALs, y los agrego a una lista por usuarios con sus ALS.
		local users_with_als = {}
		for full_user_id in pairs( role_info.users or {} ) do
			local als_by_user = GetModel("Campaigns").GetAttentionLevelsByUser( full_user_id, keyspace )
			local has_als = not utils.TableIsEmpty(als_by_user)
			--LogDebug(" full_user_id: '%s', has_als: '%s'", full_user_id, has_als)
			cached_users_als[full_user_id] = {
				has_als = has_als
			}
			if has_als then
				users_with_als[full_user_id] = als_by_user
				cached_users_als[full_user_id].als = als_by_user
			end
		end

		-- recorro la lista de users que tengan attention levels, y valido si tienen permisos contra la aplicacion "interaction".
		for full_user_id, als_by_user in pairs( users_with_als ) do
			-- Obtengo los permisos del usuario, por campanas (la unica)
			-- con ella luego realizo la validación para ver si el rol del usuario quedaron con permisos a la aplicacion "interaction"
			--
			local permissions_by_campaigns = model_perm.MakeGroupedPermissionsByCampaigns(
				{
					get_campaign_permissions = function( campaign_id )
						return { groups = new_roles_permissions or {} }
					end,
					specific_user_group = role_id,
					user_campaigns = user_campaigns,
					user_id = full_user_id
				}
			)
			-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
			local hasPermissionToAppInteraction = permissions.HasPermissionToApplication(
				"interaction",
				full_user_id,
				role_id,
				full_campaign_id,
				keyspace,
				{
					user_campaign_permissions = permissions_by_campaigns,
					group_permissions         = role_info.permissions,
					campaigns_permissions     = cachedCampaignsPermissions,
					user_campaigns            = user_campaigns,
					applicationsActions       = applicationsActions
					-- user_vcc_permissions  -- no es necesario ya que unicamente se va a loopear una solo vez cada usuario, porque la relacion usuario role es 1 a 1
				}
			)
			--LogDebug("  --A- full_user_id: '%s', role_id: '%s', perm: '%s'.", full_user_id, role_id, hasPermissionToAppInteraction)

			if not hasPermissionToAppInteraction then
				response.success = false
				response.result = "USER_IN_ATTENTION_LEVELS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP"
				response.roles = response.roles or {}
				response.roles[role_id] = true
				response.count_users = (response.count_users or 0) + 1
				response.problematic_users = response.problematic_users or {}
				response.problematic_users[ full_user_id ] = als_by_user
			end
		end
	end

	return response, cached_users_als
end

-- Valida si alguno de los members tiene interacciones asignadas.
--  En el caso de que esto sea asi, tengo que validar si los permisos del usuario que se ha modificado, tiene acceso a la app "interaction".
--  Si no tiene acceso a la aplicacion "interaction" falla, con la lista de usuarios en conflicto.
--
local function ValidateUsersPermissionInConflictWithUserAssignedInteractions(full_campaign_id, members_perm, new_members_permissions, cachedCampaignsPermissions, roles, keyspace, applicationsActions)

--LogDebug("1111 roles: %s", json.encode(roles or {}))

	local vcc = inconcert.authUser.virtualContactCenter
	local response = { success = true }
	local user_campaigns = {}
	user_campaigns[full_campaign_id] = true

	for simple_user_id, member_info in pairs( members_perm ) do

		local assigned_interactions_count = member_info.count_assigned_interactions or admUserController.CountAssignedInteractions(simple_user_id)

		--LogDebug("simple_user_id: %s, assigned_interactions_count: %d", simple_user_id, assigned_interactions_count)

		if assigned_interactions_count > 0 then

			local user = GetModel("Users").GetUser( vcc, simple_user_id, keyspace, {"UserGroup"} )
			if user then
				local role_id = user.UserGroup
				-- Obtengo los permisos del usuario, por campanas (la unica)
				-- con ella luego realizo la validación para ver si los permisos del usuario quedaron con permisos a la aplicacion "interaction"
				--
				local permissions_by_campaigns = model_perm.MakeGroupedPermissionsByCampaigns(
					{
						get_campaign_permissions = function( campaign_id )
							return { groups = {}, members = new_members_permissions or {} }
						end,
						specific_user_group = role_id,
						user_campaigns = user_campaigns,
						user_id = simple_user_id
					}
				)

				local full_user_id = utils.AppendVcc(simple_user_id, vcc )
				-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
				local role_info = roles[role_id]
				--LogDebug(" role_id: %s, role_info: %s", role_id, role_info)
				local hasPermissionToAppInteraction = permissions.HasPermissionToApplication(
					"interaction",
					full_user_id,
					role_id,
					full_campaign_id,
					keyspace,
					{
						user_campaign_permissions = permissions_by_campaigns,
						group_permissions         = role_info.permissions,
						campaigns_permissions     = cachedCampaignsPermissions,
						user_campaigns            = user_campaigns,
						applicationsActions       = applicationsActions
						-- user_vcc_permissions  -- no es necesario ya que unicamente se va a loopear una solo vez cada usuario, porque la relacion usuario role es 1 a 1
					}
				)

				--LogDebug("  --B- full_user_id: '%s', role_id: '%s', full_campaign_id: '%s' perm: '%s'.", full_user_id, role_id, full_campaign_id, hasPermissionToAppInteraction)

				if not hasPermissionToAppInteraction then
					response.success = false
					response.result = "USER_HAS_INTERACTIONS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP"
					response.roles = response.roles or {}
					response.roles[role_id] = true
					response.count_users = (response.count_users or 0) + 1
					response.problematic_users = response.problematic_users or {}
					response.problematic_users[ full_user_id ] = assigned_interactions_count
				end
			end
		end
	end

	return response
end

-- Valida si existe algun usuario se encuentra asignado a algun AL de la campana.
--  En el caso de que esto sea asi, tengo que validar si los permisos del usuario que se ha modificado, tiene acceso a la app "interaction".
--  Si no tiene acceso a la aplicacion "interaction" falla, con la lista de usuarios en conflicto.
--
local function ValidateUsersPermissionInConflictWithAttentionLevels( full_campaign_id, members_perm, new_members_permissions, cachedCampaignsPermissions, roles, keyspace, applicationsActions)
	local vcc = inconcert.authUser.virtualContactCenter
	local response = { success = true }

	local user_campaigns = {}
	user_campaigns[full_campaign_id] = true

	-- recorro la lista de usuarios a los que se les ha modificado los permisos y valido si tienen ALs, y los agrego a una lista por usuarios con sus ALS.
	local users_with_als = {}
	for user_id_without_vcc, member_info in pairs( members_perm ) do
		local full_user_id = utils.AppendVcc(user_id_without_vcc, vcc )

		if member_info.configured_als then
			if member_info.configured_als.has_als then
				users_with_als[full_user_id] = member_info.configured_als.als
			end
		else
			local als_by_user = GetModel("Campaigns").GetAttentionLevelsByUser( full_user_id, keyspace )
			local has_als = not utils.TableIsEmpty(als_by_user)
			--LogDebug(" -B- full_user_id: '%s', has_als: '%s'", full_user_id, has_als)
			if has_als then
				users_with_als[full_user_id] = als_by_user
			end
		end
	end


	-- recorro la lista de users que tengan attention levels, y valido si tienen permisos contra la aplicacion "interaction".
	for full_user_id, als_by_user in pairs( users_with_als ) do

		local user_id_without_vcc = utils.GetId( full_user_id )
		local user = GetModel("Users").GetUser( vcc, user_id_without_vcc, keyspace, {"UserGroup"} )
		if user then
			local role_id = user.UserGroup
			-- Obtengo los permisos del usuario, por campanas (la unica)
			-- con ella luego realizo la validación para ver si los permisos del usuario quedaron con permisos a la aplicacion "interaction"
			--
			local permissions_by_campaigns = model_perm.MakeGroupedPermissionsByCampaigns(
				{
					get_campaign_permissions = function( campaign_id )
						return { groups = {}, members = new_members_permissions or {} }
					end,
					specific_user_group = role_id,
					user_campaigns = user_campaigns,
					user_id = user_id_without_vcc
				}
			)

			-- aca realizo la validación si el usuario tiene permisos para la app "INTERACTION"
			local role_info = roles[role_id]
			local hasPermissionToAppInteraction = permissions.HasPermissionToApplication(
				"interaction",
				full_user_id,
				role_id,
				full_campaign_id,
				keyspace,
				{
					user_campaign_permissions = permissions_by_campaigns,
					group_permissions         = role_info.permissions,
					campaigns_permissions     = cachedCampaignsPermissions,
					user_campaigns            = user_campaigns,
					applicationsActions       = applicationsActions
					-- user_vcc_permissions  -- no es necesario ya que unicamente se va a loopear una solo vez cada usuario, porque la relacion usuario role es 1 a 1
				}
			)
			--LogDebug("  --B- full_user_id: '%s', role_id: '%s', full_campaign_id: '%s' perm: '%s'.", full_user_id, role_id, full_campaign_id, hasPermissionToAppInteraction)

			if not hasPermissionToAppInteraction then
				response.success = false
				response.result = "USER_IN_ATTENTION_LEVELS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP"
				response.roles = response.roles or {}
				response.roles[role_id] = true
				response.count_users = (response.count_users or 0) + 1
				response.problematic_users = response.problematic_users or {}
				response.problematic_users[ full_user_id ] = als_by_user
			end
		end
	end

	return response
end

--[[members_list, keyspace = optionals]]--
function SaveAllPermissionsOnCampaign (campaignId, new_roles_permissions, new_members_permissions, members_list, keyspace, applicationsActions, ksHistorical, isCreateCampaign)
	local vcc = inconcert.authUser.virtualContactCenter
	local simple_campaign_id = utils.GetId( campaignId )
	local full_campaign_id = utils.AppendVcc( simple_campaign_id, vcc )
	local response = { success = true }

	if not new_roles_permissions and not new_members_permissions then
		local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save override '%s' campaign permissions. Not exist permissions.", full_campaign_id)
		response.success = false
		response.result = "NOT_EXIST_PERMISSIONS"
		response.reason = err_msg
		return response
	end

	local camp_exists = GetModel("Campaigns").CampaignExists( full_campaign_id, keyspace )
	--LogDebug("camp_exists: %s", camp_exists)
	if not camp_exists then
		local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save override '%s' campaign permissions. Campaign does not exist.", full_campaign_id)
		response.success = false
		response.result = "CAMPAIGN_DOES_NOT_EXIST"
		response.reason = err_msg
		return response
	end

	local allPermissions = {}
	local encoded_permissions = GetModel("Campaigns").GetSpecificColumn( full_campaign_id, "permissions", keyspace )
	--LogDebug("encoded_permissions: %s", encoded_permissions)
	if type(encoded_permissions) == "string" then
		allPermissions = json.decode(encoded_permissions)
	end

	-- Aqui cacheamos los permisos de todas las campanas, para que en la validación de permisos sobre apps, no tenga que ir a buscar estos permisos a cassandra todas las veces, por cada usuario.
	local cachedCampaignsPermissions = model_perm.GetAllVCCCampaignsPermissions( vcc, keyspace )

	-- 1 - roles es utilizado en las funciónes "ValidateRolePermissionInConflictWithAttentionLevels" y "ValidateRolePermissionInConflictWithUserAssignedInteractions"
	-- donde se valida si el role modificado tiene permisos para la aplicacion "interaction"
	local roles = {}
	local function filter ( roles_list )
		for role_id in pairs( roles_list ) do
			roles[role_id] = {
				permissions = model_perm.GetUserGroupPermissions( role_id, keyspace )
			}
		end
	end
	local system_roles = GetModel("Groups").GetGroups( "system", nil, keyspace ) --Get system roles from model
	filter( system_roles )
	local vcc_roles = GetModel("Groups").GetGroups( vcc, nil, keyspace ) --Get vcc roles from model
	filter( vcc_roles  )

	local cached_users_assigned_interactions = {}
	local cached_users_als = {}
	local hasChanged = false

	if new_roles_permissions then
		-- valida si se ha realizado cambios sobre los permisos ( roles/campaign y members/campaign )
		hasChanged = not utils.StructuralCompare( allPermissions.groups, new_roles_permissions )
		if not hasChanged then
			-- si no tiene ningun cambio, no guardo y me voy
			return response -- success (no es un error)
		end

		LogInfo("Has changed role permissions on campaign %s.", full_campaign_id)
		allPermissions.groups = new_roles_permissions

		-- agrego los usuarios por rol
		for role_id, role_info in pairs(roles) do
			local users, count = GetModel("Users").GetUsersByRole( role_id, keyspace )
			if count > 0 then
				role_info.users = users
			end
		end

		-- valido si existen permisos en conflicto con las interacciones que los usuarios tienen asignadas (si es que el permiso del rol queda sin acceso a la aplicacion "interaction")
		response, cached_users_assigned_interactions = ValidateRolePermissionInConflictWithUserAssignedInteractions(full_campaign_id, roles, new_roles_permissions, cachedCampaignsPermissions, keyspace, applicationsActions)
		if not response.success and response.result == "USER_HAS_INTERACTIONS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
		-- bueno si tuvo error con los permisos y las interacciones asignadas, aca tiro el error
			local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save role override '%s' campaign permissions. Error: %s.", full_campaign_id, json.encode(response))
			response.reason = err_msg
			return response
		end

		-- valido si existen permisos en conflicto con los attention levels (si es que el permiso del rol queda sin acceso a la aplicacion "interaction")
		response, cached_users_als = ValidateRolePermissionInConflictWithAttentionLevels(full_campaign_id, roles, new_roles_permissions, cachedCampaignsPermissions, keyspace, applicationsActions)
		if not response.success and response.result == "USER_IN_ATTENTION_LEVELS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
		-- bueno si tuvo error con los permisos y los attention levels, aca tiro el error
			local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save role override '%s' campaign permissions. Error: %s.", full_campaign_id, json.encode(response))
			response.reason = err_msg
			return response
		end
	end

	if new_members_permissions then
		if not hasChanged then
			hasChanged = not utils.StructuralCompare( allPermissions.members, new_members_permissions )
			if not hasChanged then
				-- si no tiene ningun cambio, no guardo y me voy
				return response -- success (no es un error)
			end
		end
		LogInfo("Has changed members permissions on campaign %s.", full_campaign_id)
		allPermissions.members = new_members_permissions

		-- 2 - members es utilizado en la funciónes "ValidateUsersPermissionInConflictWithUserAssignedInteractions" y "ValidateUsersPermissionInConflictWithAttentionLevels"
		-- donde se valida si el member modificado tiene permisos para la aplicacion "interaction"
		local members = {}
		for simple_member_id, info in pairs(members_list or {}) do
			if info.action == "added" or info.action == "updated" then
				local full_member_id = utils.AppendVcc( simple_member_id, vcc )
				members[simple_member_id] = {
					count_assigned_interactions = cached_users_assigned_interactions[full_member_id] and cached_users_assigned_interactions[full_member_id].count,
					configured_als = cached_users_als[full_member_id]
				}
			end
		end

		-- valido si existen permisos en conflicto con las interacciones que los usuarios tienen asignadas (si es que el permiso del rol queda sin acceso a la aplicacion "interaction")
		response = ValidateUsersPermissionInConflictWithUserAssignedInteractions( full_campaign_id, members, new_members_permissions, cachedCampaignsPermissions, roles, keyspace, applicationsActions)
		if not response.success and response.result == "USER_HAS_INTERACTIONS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
			local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save member override '%s' campaign permissions. Error: %s.", full_campaign_id, json.encode(response))
			response.reason = err_msg
			return response
		end

		-- valido si existen permisos en conflicto con los attention levels (si es que el permiso del rol queda sin acceso a la aplicacion "interaction")
		response = ValidateUsersPermissionInConflictWithAttentionLevels(full_campaign_id, members, new_members_permissions, cachedCampaignsPermissions, roles, keyspace, applicationsActions)
		if not response.success and response.result == "USER_IN_ATTENTION_LEVELS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP" then
			local err_msg = LogError("[SaveAllPermissionsOnCampaign] Unable to save member override '%s' campaign permissions. Error: %s.", full_campaign_id, json.encode(response))
			response.reason = err_msg
			return response
		end
	end

	local oldPermissions = GetAuditCampaign(full_campaign_id, keyspace, "Permissions")
	-- bueno a guardar y notificar a los agentes logueados
	GetModel("Campaigns").SaveCampaign( vcc, { Id = full_campaign_id, permissions = allPermissions }, keyspace )
	local newPermissions = allPermissions and allPermissions.groups  and not utils.TableIsEmpty(allPermissions.groups) and allPermissions.groups or nil
	LogWarning("[ADMINISTRATION.Permissions] oldPermissions %s newPermissions %s", json.encode(oldPermissions), json.encode(newPermissions))
	if oldPermissions or newPermissions then
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(full_campaign_id),
			oldFields = oldPermissions,
			newFields = newPermissions,
			subEntity = "Permissions",
			connection = ksHistorical
		})
	else
		LogWarning("[ADMINISTRATION.Permissions] Audit record not created")
	end

	agentNotifications.NotifyAgents({
		eventType = "administrator_campaign_permissions_change",
		agentsLevel = "CAMPAIGN_USERS",
		excludeAgents = isCreateCampaign and {[inconcert.authUser.userName .. "@" .. vcc] = true} or nil, --Elimino el user actual, ya que se actualiza desde el cliente
		pollerMessage = {
			campaignId = full_campaign_id,
			permissions = allPermissions
		}
	}, keyspace)

	response.allPermissionsOnCampaign = allPermissions
	
	--notifications
	if allPermissions then
		local overridedMembers = allPermissions.members or {}
		local overridedRoles = {}
		for roleId, role in pairs(allPermissions.groups or {}) do
			overridedRoles[utils.GetId(roleId)] = role
		end
		notificationsEngine.SendNotification("CampaignRolesUpdated", 0, { campaignId = utils.GetId(full_campaign_id), overridedRoles = overridedRoles, overridedMembers = overridedMembers })
	end

	return response
end

local function GetCleanUpCampaignMemberOverride( campaign_id, new_roles_permissions, keyspace )
	new_roles_permissions = new_roles_permissions or {}
	LogInfo("Validating clean up Campaign Members Overrides, campaign_id: %s, new_roles_permissions: %s", campaign_id, json.encode(new_roles_permissions))

	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( campaign_id, keyspace )
	if not camp_exists then
		return nil
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local overrides
	local encoded_permissions = GetModel("Campaigns").GetSpecificColumn( campaign_id, "permissions", keyspace )
	--LogDebug("encoded_permissions: %s", encoded_permissions)
	if type(encoded_permissions) == "string" then
		overrides = json.decode(encoded_permissions)
	end
	local members_permissions
	local changed = false

	if overrides and overrides.members then
		-- si tenemos override de members

		for member_id, member_overrides in pairs( overrides.members ) do
			local full_member_id =  member_id .. "@" .. vcc
			local member = GetModel("Users").GetUser( vcc, member_id, keyspace, {"UserGroup", "VccGroup"} )
			if member then
				local member_role_id = member.UserGroup .. "@" .. member.VccGroup

				local old_role_perm
				if overrides.groups then
					old_role_perm = overrides.groups[ member_role_id ] or { actions = {} }
				else
					old_role_perm = { actions = {} }
				end

				local new_role_perm = new_roles_permissions[ member_role_id ] or { actions = {} }

				-- LogInfo(" 1 - MEMBERS   member: %s, role_id: %s, overrides: %s", full_member_id, member_role_id, json.encode(member_overrides) )
				-- LogInfo(" 2 - OLD ROLES member: %s, role_id: %s, overrides: %s", full_member_id, member_role_id, json.encode(old_role_perm) )
				-- LogInfo(" 3 - NEW ROLES member: %s, role_id: %s, overrides: %s", full_member_id, member_role_id, json.encode(new_role_perm) )

				if member_overrides.actions then
					for app_id, actions in pairs( member_overrides.actions ) do

						local old_role_app_perm = old_role_perm.actions[ app_id ] or {}
						local new_role_app_perm = new_role_perm.actions[ app_id ] or {}

						for action_id, enabled in pairs( actions ) do
							if old_role_app_perm[ action_id ] ~= new_role_app_perm[ action_id ] then
								changed = true
								overrides.members[ member_id ].actions[ app_id ][ action_id ] = nil 
							end
						end
					end
				end
			end
		end
	end

	if changed then
		members_permissions = overrides.members

		-- si exsitieron cambios, tengo que validar que no queden apps sin acciones de override, y grupos de apps sin ninguna app con acciones de override.
		-- si se da el caso antes mencionado, tengo que eliminar tambien de los override la app y/o la lista de apps
		for member_id, apps in pairs( members_permissions ) do
			local count_apps = 0
			local count_removed_apps = 0
			-- cuento la cantidad de aplicaciones y acciones que existen con override, la cantidad de apps removidas
			for app_id, apps_actions in pairs( apps.actions ) do
				count_apps = count_apps + 1
				local count_actions_in_app = 0
				for action_id, actions in pairs( apps_actions ) do
					count_actions_in_app = count_actions_in_app + 1
				end
				if count_actions_in_app == 0 then
				-- si la app no tiene acciones, la elimino de los overrides
					members_permissions[ member_id ].actions[ app_id ] = nil
					count_removed_apps = count_removed_apps + 1
				end
			end
			if count_apps == count_removed_apps then
			-- si no existe ninngua app con override tengo que eliminar la lista de contenedor de acciones por app
				members_permissions[ member_id ].actions = nil
				members_permissions[ member_id ] = nil
			end
		end

		LogInfo("Clean Up Campaign Members Overrides, campaign_id: %s, member permissions: %s", campaign_id, json.encode(members_permissions))
	end

	return members_permissions
end

function SaveRolePermissions( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local campaignId = web.request.campaignId

	LogInfo("Administration - User %s on vcc %s saving role permissions configuration for campaign '%s' posted info %s", 
		userId, 
		vcc, 
		campaignId, 
		web.input["post_data"]
	)

	if type( campaignId ) ~= "string" then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Field campaignId is invalid"}
	end

	local new_roles_permissions = web.request.roles_permissions
	if type( new_roles_permissions ) ~= "table" then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Field new_roles_permissions is invalid"}
	end

	local keyspace      = utils.GetDatasource():CassandraConnection( "Agent" )
	local hasPermission = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil, keyspace)

	if hasPermission and not inconcert.authUser.api then
		keyspace:Close()
		return returnError(web, {
			status   = 403,
			result   = "forbidden",
			reason   = string.format("[ADMINISTRATION.SaveRolePermissions] Unable to save role permission campaign '%s'. You don't have permissions.", campaignId)
		})
	end

	local members_permissions = GetCleanUpCampaignMemberOverride( campaignId, new_roles_permissions, keyspace )
	local applicationsActions = model_perm.GetApplicationsActions(keyspace, vcc)
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	local response = SaveAllPermissionsOnCampaign(campaignId, new_roles_permissions, members_permissions, nil, keyspace, applicationsActions, ksHistorical)
	keyspace:Close()
	ksHistorical:Close()

	if not response.success then
		web.status = 400
	else
		web.status = 200
		response = {result = "OK"}
	end
	return response
end

function SaveDispositions (web)
	local request = web.request
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	if not request or type(request.dispositionCodes) ~= "string" then
		return returnError(web, {
			reason = string.format("[ADMINISTRATION.SaveDispositions] - Invlaid request saving dispositions in campaigns: '%s', dispositions: %s", request.campaignId, request.dispositionCodes)
		})
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaign_id_with_vcc = request.campaignId

	LogInfo("[ADMINISTRATION.SaveDispositions] - User %s on vcc %s saving disposition codes configuration for campaign '%s' dispositions %s", 
		userId,
		vcc,
		campaign_id_with_vcc,
		request.dispositionCodes
	)
	-- Obtengo datos actuales de auditoria para la campaign antes de salvar los DC
	local connection = utils.GetDatasource():CassandraConnection( "Agent" )
	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaign_id_with_vcc, connection)

	if not belongs then
		return belResp
	end

	local old_dispotitions = GetAuditCampaign(campaign_id_with_vcc, connection, "DispositionCodes")
	LogInfo("[ADMINISTRATION.SaveDispositions] - old_dispotitions %s", json.encode(old_dispotitions))
	local success, err = GetModel("Campaigns").SaveCampaignDisposition(campaign_id_with_vcc, request.dispositionCodes, keyspace, allegroOtherInfo)
	if not success then
		err = err or {}
		err.reason = string.format("[ADMINISTRATION.SaveDispositions] - Error saving dispositions in campaigns: '%s', dispositions: %s", request.campaignId, request.dispositionCodes)
		connection:close()
		return returnError(web, err)
	end

	--Success: Se crea el registro de auditoria.
	local curr_dispotitions = GetAuditCampaign(campaign_id_with_vcc, connection, "DispositionCodes")
	local operation_audit = nil
	HasChangedStructuralList(
		curr_dispotitions,
		old_dispotitions,
		function( dc_id, current_dc, new_dc )
			if current_dc and new_dc then
				operation_audit = "Update"
			else
				operation_audit = "Create"
			end
		end
	)
	--Operacion inversa para saber si se elimino DC
	if not operation_audit then
		HasChangedStructuralList(old_dispotitions, curr_dispotitions,
		function( dc_id, current_dc, delete_dc )
			if delete_dc then
				operation_audit = "Delete"
				old_dispotitions = delete_dc
			else
				operation_audit = "Update"
			end
		end) 
	end
	
	if old_dispotitions or curr_dispotitions then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(request.campaignId),
			oldFields = operation_audit ~= "Create" and old_dispotitions or nil,
			newFields =  operation_audit ~= "Delete" and curr_dispotitions or nil,
			subEntity = "DispositionCodes",
			connection = ksHistorical
		})
		ksHistorical:Close()
	else
		LogWarning("[ADMINISTRATION.SaveDispositions] Audit record not created")
	end
	
	--Notify logged users that disposition codes has changed
	local dispositions = json.decode(request.dispositionCodes)
	agentNotifications.NotifyAgents({
		eventType = "administrator_disposition_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId = campaign_id_with_vcc,
			dispositions = dispositions
		}
	}, keyspace)

	--notifications
	notificationsEngine.SendNotification("CampaignDispositionsUpdated", 0, 
		{ campaignId = utils.GetId(campaign_id_with_vcc), dispositions = dispositions })

	return ReplyJSON(web, {result = "OK"})
end

function GetUserCampaignPermissions( web, campaignId, userId )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("GetUserCampaignPermissions, campaignId: %s - user %s", campaignId, userId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- los override de permisos de la campana (member y roles) categorizados
	local campaigns = {}
	campaigns[campaignId] = true
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	local override_camp_perm = model_perm.GetUserPermissionsOnCampaigns( userId, nil --[[ specific_user_group --]] , campaigns, keyspace )
	local categorized_override_camp_perm = model_perm.CategorizeCampaignsPermissions(override_camp_perm, keyspace, applicationsActions)

	keyspace:Close()
	return categorized_override_camp_perm
end

--[[
	Esta función es invocada desde este mismmo controller, y desde la app "allegro"
]]--
function LoadMembers (fullCampaignId, keyspace)
	LogInfo("[ADMINISTRATION.CAMPAIGNS:LoadMembers] (START) Loading Members of Campaign '%s'.", fullCampaignId)

	utils.CollectAllMemory()

	local simpleCampaignId, vccCampaign = utils.SeparateVCC(fullCampaignId)
	local startTime = os.time()

	-- Aqui cacheamos los permisos de todas las campanas, para que en la validación de permisos sobre apps, no tenga que ir a buscar estos permisos a cassandra todas las veces, por cada usuario.
	-- (NECESITO todas las campanas, para la validación de overrride por campanas), ya que la validación del permisos es que al menos tenga permisos sobre una campana a la cual pertenece el member.
	--
	--local startCurrentProcessTime = os.time()
	local cachedCampaignsPermissions = model_perm.GetAllVCCCampaignsPermissions(vccCampaign, keyspace)
	local cachedCurrentCampaignPermissions = (cachedCampaignsPermissions or {})[fullCampaignId] or {}

	--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] A - (START) Loaded All VCC '%s' Campaigns Permissions. ELAPSED Time '%s' seconds.", vccCampaign, utils.GetElapsedTimeSec(startCurrentProcessTime))

	-- En esta variable quedaran cacheados los permisos por roles a los cuales pertenezcan los usuarios.
	local cachedRolesPermissions = {}
	-- En esta variable quedaran cacheados los diferentes interaction types, al igual que las demas variables de cached es para restringir el acceso a Cassandra y que la busqueda sea mas optima.
	local cachedInteractionTypes

	local startCurrentProcessTime = os.time()
	local members = {}
	local vccUsers = GetModel("Users").GetVCCUsers(vccCampaign, keyspace)

	--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] B - Loading All VCC '%s' Users. ELAPSED Time '%s' seconds.", vccCampaign, utils.GetElapsedTimeSec(startCurrentProcessTime))

	local i = 0
	local numberBatchLog = 200
	local applicationsActions = model_perm.GetApplicationsActions(keyspace, vccCampaign)

	local startCurrentProcessTime = os.time()
	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vccCampaign, keyspace)
	--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] B.1 - Loading All Users Override permissions VCC '%s' Users. ELAPSED Time '%s' seconds.", vccCampaign, utils.GetElapsedTimeSec(startCurrentProcessTime))
	--LogDebug("usersOverridePermissions: %s", json.encode(usersOverridePermissions))

	local membersOfCampaign = GetModel("Campaigns").GetCampaignMembers(vccCampaign, fullCampaignId, keyspace)
	--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] B.2 - Loading Members of campaign '%s'. ELAPSED Time '%s' seconds.", vccCampaign, utils.GetElapsedTimeSec(startCurrentProcessTime))
	--LogDebug("%s", json.encode(membersOfCampaign))

	startCurrentProcessTime = os.time()

	-- En esta tabla se almacena los permisos del Rol dentro de la campana, asi no lo va a buscar por cada Usuario que pertenezca al Rol.
	local permissionsByCampaigns = {}
	-- En esta tabla quedan los roles categorizados, según lo categorizan los permisos.
	-- Idem al comentario anterior, esto es para que no tenga que ir por cada Usuario que pertenezca al Rol.
	local categorizedPermissions = {
		roles = {}
	}

	for simpleUserId, vccUser in pairs(vccUsers) do
		if tonumber(vccUser.Status) == 1 then
			local isSystemUser = tonumber(vccUser.IsSystemUser) == 0
			local isUserFullSystemAccess = utils.IsUserFullSystemAccess(vccUser)
			-- local startCurrentProcessTime = os.time()
			-- LogDebug("--------------")
			-- LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] C - Loaded User '%s', isSystemUser '%s', isUserFullSystemAccess '%s'. ELAPSED Time '%s' seconds.", simpleUserId, isSystemUser, isUserFullSystemAccess, utils.GetElapsedTimeSec(startCurrentProcessTime))
			if
				isSystemUser
				and not isUserFullSystemAccess
			then
				i = i + 1
				--startCurrentProcessTime = os.time()
				local fullUserId = utils.AppendVcc(simpleUserId, vccCampaign)
				--local role = GetModel("Campaigns").UserIsMember(fullCampaignId, fullUserId, keyspace)
				local isMemberRole = membersOfCampaign[fullUserId]
				--LogDebug("--> fullUserId '%s' isMemberRole: '%s'", fullUserId, isMemberRole)
				--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] D - %d - Validated if User '%s' is member of campaign '%s' role: '%s'. ELAPSED Time '%s' seconds.", i, simpleUserId, fullCampaignId, role, utils.GetElapsedTimeSec(startCurrentProcessTime))

				local member = {
					Id           = fullUserId,
					Name         = vccUser.Name,
					LastName     = vccUser.LastName,
					FirstSurname = vccUser.FirstSurname,
					LastSurname  = vccUser.LastSurname,
					Role         = isMemberRole
				}
				member.UserGroup = vccUser.UserGroup or member.Role or "agent"

				if member.UserGroup and not cachedRolesPermissions[member.UserGroup] then
				-- cacheo los permisos del rol, para que no tenga que ir a buscarlo por cada usuario a cassandra
					--local startCurrentProcessTime = os.time()
					cachedRolesPermissions[member.UserGroup] = model_perm.GetUserGroupPermissions(member.UserGroup, keyspace)
					--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] E - Loaded role '%s' permissions. ELAPSED Time '%s' seconds.", member.UserGroup, utils.GetElapsedTimeSec(startCurrentProcessTime))
				end

				-- los override de permisos
				--local startCurrentProcessTime = os.time()
				member.OverrideVccPermissions = usersOverridePermissions[fullUserId] or utils.EMPTY_OBJECT
				--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] F - Loaded User '%s' Override permissions. ELAPSED Time '%s' seconds.", fullUserId, utils.GetElapsedTimeSec(startCurrentProcessTime))

				-- paso las campana a la cual pertenece el usuario, para que no tenga que ir contra cassandra
				local userCampaigns = {}
				userCampaigns[fullCampaignId] = true

				permissionsByCampaigns[member.UserGroup] = permissionsByCampaigns[member.UserGroup] or model_perm.MakeGroupedPermissionsByCampaigns(
					{
						get_campaign_permissions = function(--[[campaignId--]])
							return cachedCurrentCampaignPermissions
						end,
						specific_user_group = member.UserGroup,
						user_campaigns = userCampaigns,
						--user_id = full_user_id
					}
				)

				local userCampaignPermissions = permissionsByCampaigns[member.UserGroup]

				-- los permisos del grupo dentro de la campana (ahora si el Usuario tiene override de permisos en la campana lo agrego)
				local memberPermissions = (cachedCurrentCampaignPermissions.members and cachedCurrentCampaignPermissions.members[simpleUserId]) or nil
				if memberPermissions then
					--LogDebug("---%s memberPermissions %s", simpleUserId, json.encode(memberPermissions))
					userCampaignPermissions[fullCampaignId].member = memberPermissions
				else
					userCampaignPermissions[fullCampaignId].member = nil
				end

				--LogDebug("--> user: %s, role: %s - userCampaignPermissions: %s", simpleUserId, member.UserGroup, json.encode(userCampaignPermissions))

				if not categorizedPermissions.roles[member.UserGroup] then
					--LogDebug("1 categorizando el rol %s", member.UserGroup)
					categorizedPermissions.roles[member.UserGroup] = model_perm.CategorizeRolePermmissions(cachedRolesPermissions[member.UserGroup], keyspace, applicationsActions)
				end

				-- groupPermissions, userCampaignsPermissions, userVccPermissions, keyspace, applicationsActions, custom
				local customPermissions = model_perm.CategorizeAllPermissions(nil, nil, nil, keyspace, applicationsActions, {
					rolePermissions = categorizedPermissions.roles[member.UserGroup],
					-- Estos 2 valores cambian siempre por cada usuario, asi que no me queda más remedio que procesarlo de a uno.
				--	campaignPermissions = nil,
				--	vccPermissions = nil
				})

				-- carga si tiene acceso a una app determinada
				local HasPermissionToApp = function(app_id, campId)
					--local startCurrentProcessTime = os.time()
					--local hasPermission = true -- flag utilizado para que NO valide los permisos. Si lo quitamos el comment hay que aplicar comment a la invocación de la función "permissions.HasPermissionToApplication"

					local hasPermission = permissions.HasPermissionToApplication(
						app_id,
						member.Id,
						member.UserGroup,
						campId,
						keyspace,
						{
							group_permissions         = cachedRolesPermissions[member.UserGroup],
							user_vcc_permissions      = member.OverrideVccPermissions,
							campaigns_permissions     = cachedCampaignsPermissions,
							user_campaigns            = userCampaigns,
							applicationsActions       = applicationsActions,
							user_campaign_permissions = userCampaignPermissions,
							customPermissions         = customPermissions
						}
					)
					--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] G - Validated If User '%s' Has Permission to App '%s'. Campaign: '%s'. ELAPSED Time '%s' seconds.", member.Id, app_id, camp_id, utils.GetElapsedTimeSec(startCurrentProcessTime))

					return hasPermission
				end
				-- de momento se valida contra las apps "interaction" y "supervisor"
				member.apps_permisson = {
					interaction = HasPermissionToApp("interaction", nil),
					supervisor  = HasPermissionToApp("supervisor", nil) -- si la campana es nil, es porque tiene que buscar el permiso contra todas las campanas
				}

				-- los supervisor workgroups
				if member.apps_permisson.supervisor then
					member.supervisor_workgroups = (vccUser.SupervisorWorkgroups or {})[simpleCampaignId] or {}
				end
				-- -- los overrides de capability de campaign members
				if isMemberRole then
					-- a la variable "cachedInteractionTypes", la paso como parametro para que la función "GetCapabilityByCampaignAndMember" no tenga que ir contra cassandra todas las veces a buscar los mismos datos.
					--local startCurrentProcessTime = os.time()
					local memberCapabilities
					memberCapabilities, cachedInteractionTypes = channelsController.GetCapabilityByCampaignAndMember(simpleCampaignId, fullUserId, cachedInteractionTypes, keyspace)
					member.OverrideCapabilities = memberCapabilities

					--LogDebug("[ADMINISTRATION:CAMPAIGNS:LoadMembers] H - Loaded Override Capability for User '%s' in Campaign: '%s'. ELAPSED Time '%s' seconds.", fullUserId, camp_id, utils.GetElapsedTimeSec(startCurrentProcessTime))
				end

				table.insert(members, member)

				if i == numberBatchLog then
					LogInfo("[ADMINISTRATION:CAMPAIGNS:LoadMembers] %d Members are loading for the campaign '%s'. ELAPSED Time '%s' seconds.", numberBatchLog, fullCampaignId, utils.GetElapsedTimeSec(startCurrentProcessTime))
					numberBatchLog = numberBatchLog + 200
					startCurrentProcessTime = os.time()
				end
			end
		end
	end

	utils.CollectAllMemory() -- invocamos a garbage ya que esta función realiza muchas conexiones contra cassandra y otras cositas, con esto nos cuidamos !!

	local elapsedTime = utils.GetElapsedTimeSec(startTime)
	LogInfo("[ADMINISTRATION.CAMPAIGNS:LoadMembers] (END) Loaded %d Members of Campaign '%s'. ELAPSED Time '%s' seconds.", #members, vccCampaign, elapsedTime)

	return members
end

function LoadOnlyMembers (fullCampaignId, keyspace)
	LogInfo("[ADMINISTRATION.CAMPAIGNS:LoadOnlyMembers] (START) Loading Members of Campaign '%s'.", fullCampaignId)

	utils.CollectAllMemory()

	local simpleCampaignId, vccCampaign = utils.SeparateVCC(fullCampaignId)
	local startTime = os.time()

	-- Aqui cacheamos los permisos de todas las campanas, para que en la validación de permisos sobre apps, no tenga que ir a buscar estos permisos a cassandra todas las veces, por cada usuario.
	-- (NECESITO todas las campanas, para la validación de overrride por campanas), ya que la validación del permisos es que al menos tenga permisos sobre una campana a la cual pertenece el member.
	--
	--local startCurrentProcessTime = os.time()
	local cachedCampaignsPermissions = model_perm.GetAllVCCCampaignsPermissions(vccCampaign, keyspace)
	local cachedCurrentCampaignPermissions = (cachedCampaignsPermissions or {})[fullCampaignId] or {}

	-- En esta variable quedaran cacheados los permisos por roles a los cuales pertenezcan los usuarios.
	local cachedRolesPermissions = {}
	-- En esta variable quedaran cacheados los diferentes interaction types, al igual que las demas variables de cached es para restringir el acceso a Cassandra y que la busqueda sea mas optima.
	local cachedInteractionTypes

	local startCurrentProcessTime = os.time()
	local members = {}
	local vccUsers = GetModel("Users").GetVCCUsers(vccCampaign, keyspace)

	local i = 0
	local numberBatchLog = 200
	local applicationsActions = model_perm.GetApplicationsActions(keyspace, vccCampaign)

	local startCurrentProcessTime = os.time()

	local membersOfCampaign = GetModel("Campaigns").GetCampaignMembers(vccCampaign, fullCampaignId, keyspace)

	startCurrentProcessTime = os.time()

	-- En esta tabla se almacena los permisos del Rol dentro de la campana, asi no lo va a buscar por cada Usuario que pertenezca al Rol.
	local permissionsByCampaigns = {}
	-- En esta tabla quedan los roles categorizados, según lo categorizan los permisos.
	-- Idem al comentario anterior, esto es para que no tenga que ir por cada Usuario que pertenezca al Rol.
	local categorizedPermissions = {
		roles = {}
	}

	for simpleUserId, vccUser in pairs(vccUsers) do
		if tonumber(vccUser.Status) == 1 then

			if tonumber(vccUser.IsSystemUser) == 0 then
				i = i + 1
				--startCurrentProcessTime = os.time()
				local fullUserId = utils.AppendVcc(simpleUserId, vccCampaign)
				--local role = GetModel("Campaigns").UserIsMember(fullCampaignId, fullUserId, keyspace)
				local isMemberRole = membersOfCampaign[fullUserId]
				
				if isMemberRole then

					local member = {
						Id           = fullUserId,
						Name         = vccUser.Name,
						LastName     = vccUser.LastName,
						FirstSurname = vccUser.FirstSurname,
						LastSurname  = vccUser.LastSurname,
						Role         = isMemberRole
					}
					member.UserGroup = vccUser.UserGroup or member.Role or "agent"

					if member.UserGroup and not cachedRolesPermissions[member.UserGroup] then
					-- cacheo los permisos del rol, para que no tenga que ir a buscarlo por cada usuario a cassandra
						--local startCurrentProcessTime = os.time()
						cachedRolesPermissions[member.UserGroup] = model_perm.GetUserGroupPermissions(member.UserGroup, keyspace)
					end

					-- paso las campana a la cual pertenece el usuario, para que no tenga que ir contra cassandra
					local userCampaigns = {}
					userCampaigns[fullCampaignId] = true

					permissionsByCampaigns[member.UserGroup] = permissionsByCampaigns[member.UserGroup] or model_perm.MakeGroupedPermissionsByCampaigns(
						{
							get_campaign_permissions = function(--[[campaignId--]])
								return cachedCurrentCampaignPermissions
							end,
							specific_user_group = member.UserGroup,
							user_campaigns = userCampaigns,
							--user_id = full_user_id
						}
					)

					local userCampaignPermissions = permissionsByCampaigns[member.UserGroup]

					-- los permisos del grupo dentro de la campana (ahora si el Usuario tiene override de permisos en la campana lo agrego)
					local memberPermissions = (cachedCurrentCampaignPermissions.members and cachedCurrentCampaignPermissions.members[simpleUserId]) or nil
					if memberPermissions then
						--LogDebug("---%s memberPermissions %s", simpleUserId, json.encode(memberPermissions))
						userCampaignPermissions[fullCampaignId].member = memberPermissions
					else
						userCampaignPermissions[fullCampaignId].member = nil
					end

					if not categorizedPermissions.roles[member.UserGroup] then
						categorizedPermissions.roles[member.UserGroup] = model_perm.CategorizeRolePermmissions(cachedRolesPermissions[member.UserGroup], keyspace, applicationsActions)
					end

					-- groupPermissions, userCampaignsPermissions, userVccPermissions, keyspace, applicationsActions, custom
					local customPermissions = model_perm.CategorizeAllPermissions(nil, nil, nil, keyspace, applicationsActions, {
						rolePermissions = categorizedPermissions.roles[member.UserGroup],
						-- Estos 2 valores cambian siempre por cada usuario, asi que no me queda más remedio que procesarlo de a uno.
					--	campaignPermissions = nil,
					--	vccPermissions = nil
					})

					-- carga si tiene acceso a una app determinada
					local HasPermissionToApp = function(app_id, campId)
						--local startCurrentProcessTime = os.time()
						--local hasPermission = true -- flag utilizado para que NO valide los permisos. Si lo quitamos el comment hay que aplicar comment a la invocación de la función "permissions.HasPermissionToApplication"

						local hasPermission = permissions.HasPermissionToApplication(
							app_id,
							member.Id,
							member.UserGroup,
							campId,
							keyspace,
							{
								group_permissions         = cachedRolesPermissions[member.UserGroup],
								user_vcc_permissions      = member.OverrideVccPermissions,
								campaigns_permissions     = cachedCampaignsPermissions,
								user_campaigns            = userCampaigns,
								applicationsActions       = applicationsActions,
								user_campaign_permissions = userCampaignPermissions,
								customPermissions         = customPermissions
							}
						)

						return hasPermission
					end
					-- de momento se valida contra las apps "interaction" y "supervisor"
					member.apps_permisson = {
						interaction = HasPermissionToApp("interaction", nil),
						supervisor  = HasPermissionToApp("supervisor", nil) -- si la campana es nil, es porque tiene que buscar el permiso contra todas las campanas
					}

					local memberWorkgroup = GetModel("Users").GetUser( vccCampaign, member.Id, keyspace, {"WorkgroupId"} )
					if (next(memberWorkgroup) ~= nil) then
						member.WorkgroupId = memberWorkgroup.WorkgroupId
					end

					-- los supervisor workgroups
					if member.apps_permisson.supervisor then
						member.supervisor_workgroups = (vccUser.SupervisorWorkgroups or {})[simpleCampaignId] or {}
					end

					-- los overrides de capability de campaign members
					local memberCapabilities
					memberCapabilities, cachedInteractionTypes = channelsController.GetCapabilityByCampaignAndMember(simpleCampaignId, fullUserId, cachedInteractionTypes, keyspace)
					member.OverrideCapabilities = memberCapabilities

					table.insert(members, member)

					if i == numberBatchLog then
						LogInfo("[ADMINISTRATION:CAMPAIGNS:LoadOnlyMembers] %d Members are loading for the campaign '%s'. ELAPSED Time '%s' seconds.", numberBatchLog, fullCampaignId, utils.GetElapsedTimeSec(startCurrentProcessTime))
						numberBatchLog = numberBatchLog + 200
						startCurrentProcessTime = os.time()
					end
				end
			end
		end
	end

	utils.CollectAllMemory() -- invocamos a garbage ya que esta función realiza muchas conexiones contra cassandra y otras cositas, con esto nos cuidamos !!

	local elapsedTime = utils.GetElapsedTimeSec(startTime)
	LogInfo("[ADMINISTRATION.CAMPAIGNS:LoadOnlyMembers] (END) Loaded %d Members of Campaign '%s'. ELAPSED Time '%s' seconds.", #members, vccCampaign, elapsedTime)

	return members
end

function GetMembers (web, fullCampaignId)
	local loggedUserId = utils.GetCurrentLoggedUser()

	LogInfo("[ADMINISTRATION:CAMPAIGNS:GetMembers] + START loading '%s' campaign members, user '%s'.", fullCampaignId, loggedUserId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	web.input.post_data = "null"
	local members = LoadMembers(fullCampaignId, keyspace)
	keyspace:Close()

	LogInfo("[ADMINISTRATION:CAMPAIGNS:GetMembers] - END loaded '%s' campaign. Total '%d' members", fullCampaignId, #members)

	return members
end

function GetOnlyMembers (web)
	local loggedUserId = utils.GetCurrentLoggedUser()
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time() 
	local postData = web.request
	local totalAdded = 0
	local filteredRows = {}
	local camp = postData.campaigns
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	for _,nameCamp in pairs(camp) do
		local campaign = GetModel("Campaigns").GetCampaign(vcc, nameCamp:lower(), keyspace)
		if (next(campaign) ~= nil) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, campaign)
			local fullCampaignId = nameCamp:lower() .. "@" .. vcc
			web.input.post_data = "null"
			local members = LoadOnlyMembers(fullCampaignId, keyspace)
			campaign.members = members
		end
	end
	keyspace:Close()
	--LogInfo("[ADMINISTRATION:CAMPAIGNS:GetOnlyMembers] - END loaded filteredRows '%s'", json.encode(filteredRows))
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded
	}
	local resultData = response
	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.CAMPAIGNS:GetOnlyMembers] 4 - (END) Search campaigns. Retrieve %d of %d matched rows. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, elapsedTime)
	return resultData
end

function IsAvailableSaveAccount (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local account = web.request.account

	LogInfo("[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] User %s validating is save a new account %s in campaign %s.", userId, json.encode(account or {}), campaignId)

	if not account then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field account"}
	end

	if account.is_new then
		local responseAccount, responseCampaign = agentCampaignModel.FindAccountByUID( account )

		if responseAccount then
			-- error
			web.status = 400
			if responseCampaign == campaignId then
				LogError("[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] Unable to add account %s on campaign %s. Account already exists on current campaign.", account.id, campaignId)
				return {result = "ALREADY_EXISTS_IN_CURRENT_CAMPAIGN", reason = "account already exists in current campaign"}
			end
			local currentId,  currentVCC = utils.SeparateVCC(campaignId)
			local responseId, responseVCC = utils.SeparateVCC(responseCampaign)
			if currentVCC ~= responseVCC then
				LogError("[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] Unable to add account %s on campaign %s. Account already exists in another vcc %s.", account.id, campaignId, responseVCC)
				return {result = "ALREADY_EXISTS_IN_ANOTHER_VCC", reason = "account already exists in another vcc"}
			else
				LogError("[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] Unable to add account %s on campaign %s. Account already exists in %s campaign.", account.id, campaignId, responseId)
				return {result = "ALREADY_EXISTS_IN_ANOTHER_CAMPAIGN", reason = "account already exists in another campaign", campaign = responseId}
			end
		end

		local existingAccountInVCC = GetModel("VCCs").GetAccounts( vcc, account.proxyType, account.uid )
		if existingAccountInVCC then
			web.status = 400
			local err = {
				result = "ACCOUNT_ALREADY_EXISTS_IN_CURRENT_VCC",
				reason = string.format( "Unable to add account. Account id: '%s', type: '%s', Already exists in current VCC: %s", account.uid, account.proxyType, vcc )
			}
			LogError("[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] " .. err.reason)
			return err
		end
	else
		if account.proxyType == "FACEBOOK" or account.proxyType == "INSTAGRAM" then
		-- valida la reautorizacion de cuentas de facebook!!!
			local current_account = agentCampaignModel.FindAccountById(account)
			if not current_account then
				web.status = 400
				local reason = LogError( "[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] Unable to update '%s' account. This account does not exist.", account.id )
				return {result = "INVALID_REQUEST", reason = reason}
			end

			if current_account.uid ~= account.uid then
				web.status = 400
				local reason = LogError( "[ADMINISTRATION:CAMPAIGNS:IsAvailableSaveAccount] Unable to re-authorize '%s' facebook account. This account is authorized by '%s' facebook user. Your user authorization is '%s'.", account.id, current_account.uid, account.uid )
				return {result = "UNABLE_REAUTHORIZE_FACEBOOK_ACCOUNT", reason = reason}
			end
		end
	end

	-- is ok (available)
	return { result = "OK" }
end

local function InjectBuiltinAccountValues (account, ksAgent)
	if not account then
		return
	end
	local vcc = inconcert.authUser.virtualContactCenter
	if account.proxyType == "TWITTER" then
		account.autoFollowBack = "0"
		account.autoFollowMentions = "0"
		account.autoFollowOutgoing = "0"
	elseif account.proxyType == "CALL" and utils.HasAllegroIntegration(vcc, ksAgent) then
		-- Esto aca se agrega porque el IR necesita este package asociado para que funcione correctametne.
		-- TODO: En algun momento hay que cambiar esto y obtener le package de forma correcta, pero por el momento asi queda!!!
		account.packages = {["+598-2400-7885"] = ""}
	end
end

local function __AddUpdateAccount (action, account, full_camp_id, allegroOtherInfo, ksAgent)
	local success, response
	if action == "new" then
		success, response = agentCampaignController.AddCampaignAccount(account, full_camp_id, allegroOtherInfo, ksAgent)
	else
		success, response = agentCampaignController.UpdateCampaignAccount(account, full_camp_id, allegroOtherInfo, ksAgent)
	end
	return success, response
end

function AddAccount (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	LogInfo("[ADMINISTRATION.AddAccount] Adding new account on campaign: '%s'.\naccount data: %s", web.request.campaignId, web.input["post_data"])

	if not web.request.campaignId then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field campaignId"}
	end
	local account = web.request.account
	if not account then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field account"}
	end
	if not account.id or not account.proxyType then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field account.id or account.proxyType"}
	end

	local vcc              = inconcert.authUser.virtualContactCenter
	local userId           = inconcert.authUser.userName.."@"..vcc
	local campaignIdAtVCC  = utils.AppendVcc( utils.GetId(web.request.campaignId), vcc)
	local ksAgent          = utils.GetDatasource():CassandraConnection("Agent")
	local hasPermission    = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil , ksAgent)

	InjectBuiltinAccountValues( account, ksAgent )

	if hasPermission and not inconcert.authUser.api then
		ksAgent:Close()
		return returnError(web, {
			status   = 403,
			result   = "forbidden",
			reason   = string.format("[ADMINISTRATION.AddAccount] Unable to add account. Campaign '%s'. You don't have permissions.", campaignIdAtVCC)
		})
	end

	local success, response = __AddUpdateAccount("new", account, campaignIdAtVCC, allegroOtherInfo, ksAgent)
	if not success then
		ksAgent:Close()
		web.status = response.err_type
		return ReplyJSON(web, response)
	end
	-- success
	LogInfo("[ADMINISTRATION.AddAccount] Account %s in campaign %s is added.", account.id, campaignIdAtVCC)

	--Create audit record for Account
	local curr_account = GetAuditAccount(account.id, campaignIdAtVCC, ksAgent)
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	audits.Create({
		vcc = vcc,
		entity = "Campaign",
		user = inconcert.authUser.userName,
		fieldKey =  utils.GetId(campaignIdAtVCC),
		newFields = curr_account,
		subEntity = "Accounts",
		connection = ksHistorical
	})
	ksHistorical:Close()

	agentNotifications.NotifyAgents({
		eventType = "administrator_added_new_account",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId = campaignIdAtVCC,
			account    = response
		}
	}, ksAgent)
	ksAgent:Close()

	return ReplyJSON(web, response)
end

--[[
	Valida si la cuenta se encuentra configurada en algun Proceso de la App OutboundEngine.
]]--
local function HasConfiguredAccountInSomeOutboundEngineProcess (campaignIdAtVCC, accountId, keyspace)
	assert(accountId)
	local vcc = inconcert.authUser.virtualContactCenter
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, keyspace)

	if hasAllegroIntegration then
		-- Para VCC con Integracion con Allegro no Aplica dicha validacion.
		return false
	end

	LogInfo("[ADMINISTRATION.CAMPAIGNS:HasConfiguredAccountInSomeOutboundEngineProcess] Validating if the Account is Configured in an Outbound Engine. Campaign: %s, Account: %s.", campaignIdAtVCC, accountId)

	local success, OEConfig = GetModel("Campaigns").GetOEConfig(campaignIdAtVCC, keyspace)
	--LogDebug("0.5 OEConfig: %s:", utils.JSON_Encode(OEConfig))
	if not success then
		--LogDebug("1")
		return false
	end
	if not OEConfig or not OEConfig.processId then
		--LogDebug("2")
		return false
	end
	for channelId, accountConfig in pairs(OEConfig.channels or {}) do
		if accountId:lower() == (accountConfig.account or ""):lower() then
			-- Si, la cuenta esta configurada en un proceso de la App OE.
			local err = {httpCode = 400, errType = "ACCOUNT HAS AN OUTBOUND PROCESS CONFIGURED", processId = OEConfig.processId, campaignId = campaignIdAtVCC, accountId = accountId, channel = channelId}
			return true, err, OEConfig
		end
	end
	--LogDebug("3")
	return false
end

local function CanUpdateAccount (campaignId, account, ksAgent)
	if not campaignId then
		return false, {result = "INVALID_REQUEST", reason = "Missing field campaignId"}
	end
	if not account then
		return false, {result = "INVALID_REQUEST", reason = "Missing field account"}
	end
	if not account.id or not account.proxyType then
		return false, {result = "INVALID_REQUEST", reason = "Missing field account.id or account.proxyType"}
	end

	local modelCampaign = GetModel("Campaigns")
	local currentAccount = modelCampaign.GetCampaignAccount(campaignId, account.id, ksAgent)

	if not currentAccount then
		return false, {result = "INVALID_REQUEST", reason = "Account does not exist"}
	end

	-- Aca se valida que el Attention Level a remover de la cuenta, no tenga interacciones en WAITING.
	local accountAttentionLevels = account.AttentionLevels or {}
	local attentionLevelsToRemove = {}
	for currentAttentionLevelId in pairs(currentAccount.AttentionLevels or {}) do
		if not accountAttentionLevels[currentAttentionLevelId] then
			attentionLevelsToRemove[currentAttentionLevelId] = true
		end
	end

	for attLevelIdToRemove in pairs(attentionLevelsToRemove) do
		local numberWaitingConversations = modelCampaign.CountWaitingConversationsForAttentionLevel(campaignId, account.id, attLevelIdToRemove)
		if numberWaitingConversations > 0 then
			return false, {
				result = "ATTENTION_LEVEL_WITH_INTERACTIONS_IN_WAITING",
				reason = "Unable to remove attention level of account. Attention level in account has conversations waiting.",
				att_level_id = attLevelIdToRemove,
				account_id = account.id,
				number_waiting_conversations = numberWaitingConversations
			}
		end
	end

	if account.proxyType == "CALL" then
		local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, ksAgent)
		-- si no es una cuenta de Allegro.
		if not hasAllegroIntegration then
			-- si la cuenta esta configurada en un proceso del OE.
			local isConfigured, _, OEConfig = HasConfiguredAccountInSomeOutboundEngineProcess(campaignId, account.id, ksAgent)
			if isConfigured then
				-- valido que al menos tenga un paquete saliente
				local hasOutboundPackages = false
				local ksProv =  utils.GetDatasource():CassandraConnection( "Provisioning" )
				for packageId in pairs(account.packages) do
					local package = GetModel("Packages").GetPackage(packageId, ksProv)
					if package then
						local ioMode = (package.ioMode or ""):upper()
						if ioMode == "O" or ioMode == "I/O" then
							-- Si tiene al menos un paquete saliente.
							hasOutboundPackages = true
							break
						end
					end
				end
				ksProv:Close()
				if not hasOutboundPackages then
					return false, {
						result = "CALL_I6_NOT_ALL_OUTGOING_PACKETS_CAN_BE_REMOVED_THE_ACCOUNT_IS_CONFIGURED_IN_AN_OUTGOING_PROCESS",
						reason = "NOT ALL OUTGOING PACKETS CAN BE REMOVED - THE ACCOUNT IS CONFIGURED IN AN OUTGOING PROCES",
						process_id = OEConfig.processId,
						account_id = account.id
					}
				end
			end
		end
	end

	return true
end

function UpdateRedisAccounts (web)
	local VCC = web.request.vcc
	
	local all_accounts = GetModel("Campaigns").GetVccAccounts(VCC)
	
	for campaign_id, campaign_accounts in pairs( all_accounts ) do	
		for accountId, account in pairs(campaign_accounts) do
			local stateAccount = account.state or "0"
			-- actualizo entrada en redis
			local activeKey = "ActiveAccountsByChannel:" .. account.proxyType .. "@" .. VCC
			--LogDebug("[UpdateRedisAccounts] accountId: %s activeKey: %s",accountId, activeKey)
			GetModel("Campaigns").UpdateAccountStateInRedis (activeKey, stateAccount, accountId, campaign_id)
		end
	end
end

function UpdateAccount (web)
	local campaignId = web.request.campaignId
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)
	local ksAgent =  utils.GetDatasource():CassandraConnection( "Agent" )

	LogInfo("[ADMINISTRATION.UpdateAccount] Updating account on campaign: '%s'.", campaignId)
	-- LogDebug("[ADMINISTRATION.UpdateAccount] Updating account on campaign: '%s'.\naccount data: %s", campaignId, web.input["post_data"])

	local account = web.request.account
	local success, err = CanUpdateAccount(campaignId, account, ksAgent)
	if not success then
		ksAgent:Close()
		return returnError(web, err)
	end

	local vcc              = inconcert.authUser.virtualContactCenter
	local userId           = inconcert.authUser.userName.."@"..vcc
	local campaignIdAtVCC  = utils.AppendVcc( utils.GetId(campaignId), vcc )

	InjectBuiltinAccountValues( account, ksAgent )

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, ksAgent)
	if not belongs then
		return ReplyJSON(web, belResp)
	end

	local old_account_data = GetAuditAccount(account.id, campaignIdAtVCC, ksAgent)
	local success, response = __AddUpdateAccount("update", account, campaignIdAtVCC, allegroOtherInfo, ksAgent)
	if not success then
		ksAgent:Close()
		web.status = response.err_type
		return ReplyJSON(web, response)
	end
	-- success

	--Create audit record for Account
	local new_account_data = GetAuditAccount(account.id, campaignIdAtVCC, ksAgent)
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	audits.Create({
		vcc        = vcc,
		entity     = "Campaign",
		user       = inconcert.authUser.userName,
		fieldKey   = utils.GetId(campaignIdAtVCC),
		oldFields  = old_account_data,
		newFields  = new_account_data,
		subEntity  = "Accounts",
		connection = ksHistorical
	})
	ksHistorical:Close()

	LogInfo("[ADMINISTRATION.UpdateAccount] Account %s in campaign %s is updated.", account.id, campaignIdAtVCC)
	agentNotifications.NotifyAgents({
		eventType = "administrator_updated_account",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId = campaignIdAtVCC,
			account    = response
		}
	}, ksAgent)
	ksAgent:Close()

	RemoveHsmRepositoryFilesPublicURL(campaignId, account.id, old_account_data and old_account_data.HSMConfig or {}, account and account.HSMConfig or {})

	return ReplyJSON(web, response)
end

function RemoveAccount (web)
	LogInfo("[ADMINISTRATION.RemoveAccount] Removing account on campaign: '%s'.\naccount data: %s", web.request.campaignId, web.input["post_data"])

	local account = web.request
	if not account.campaignId then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field campaignId"}
	end
	if not account.accountId or not account.proxyType then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field accountId or proxyType"}
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	-- Valida si la cuenta de se encuentra configurada en algun Proceso de la App OutboundEngine.
	local isConfigured, outboundEngineResult = HasConfiguredAccountInSomeOutboundEngineProcess(account.campaignId, account.accountId, keyspace)
	if isConfigured then
		keyspace:Close()
		web.status = outboundEngineResult.httpCode
		return {
			result = outboundEngineResult.errType,
			reason = outboundEngineResult.errType,
			processId = outboundEngineResult.processId,
			campaignId = outboundEngineResult.campaignId,
			accountId = outboundEngineResult.accountId,
			channel =  outboundEngineResult.channel
		}
	end

	local vcc              = inconcert.authUser.virtualContactCenter
	local userId           = inconcert.authUser.userName.."@"..vcc
	local campaign_at_vcc  = utils.AppendVcc( utils.GetId(account.campaignId), vcc )
	local hasPermission    = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", nil, keyspace)

	if hasPermission and not inconcert.authUser.api then
		keyspace:Close()
		return returnError(web, {
			status   = 403,
			result   = "forbidden",
			reason   = string.format("[ADMINISTRATION.RemoveAccount] Unable to remove account. Campaign '%s'. You don't have permissions.", campaign_at_vcc)
		})
	end

	local old_account = agentCampaignModel.GetAccount(account.accountId, account.campaignId, keyspace)
	local account_copy = json.decode(json.encode(account))
	account_copy.integrationType = old_account.integrationType
	account_copy.HSMConfig = old_account.HSMConfig
	account_copy.id = old_account.id
	account_copy.wabaId = old_account.wabaId
	DelAccountWhatsappTemplates(account_copy)	

	local old_account_data = GetAuditAccount(account.accountId, campaign_at_vcc, keyspace)
	local success, result = agentCampaignController.RemoveCampaignAccount(account, campaign_at_vcc, keyspace)
	if success then
		LogInfo("[ADMINISTRATION.RemoveAccount] Successfully removed account %s in campaign %s.", account.accountId, campaign_at_vcc)

		-- 1 remuevo los overrides de capability del channel/campaign
		channelsController.OnRemovedAccount( campaign_at_vcc, account.proxyType, keyspace )
		-- 2 remuevo los messages de la cuenta
		local hasRemoveRepositoryFiles = true
		GetModel("Campaigns").RemoveAccountMessages(campaign_at_vcc, account.accountId, hasRemoveRepositoryFiles, nil, keyspace)

		agentNotifications.NotifyAgents({
			eventType = "administrator_removed_account",
			agentsLevel = "CAMPAIGN_USERS",
			pollerMessage = {
				campaignId = campaign_at_vcc,
				accountId  = account.accountId
			},
			serverMessage = {
				account = result
			}
		}, keyspace)

		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc       = vcc,
			entity    = "Campaign",
			user      = inconcert.authUser.userName,
			fieldKey  = utils.GetId(campaign_at_vcc),
			oldFields = old_account_data,
			subEntity = "Accounts",
			connection = ksHistorical
		})
		ksHistorical:Close()

		result = {result = "OK"}
		web.status = 200
	else
		web.status = 400
		LogError("[ADMINISTRATION.RemoveAccount] Error removing %s account in campaign %s. Error: %s", account.accountId, campaign_at_vcc, json.encode(result))
	end
	keyspace:Close()
	return ReplyJSON(web, result)
end

function ChangeAccountState (web)
	--LogDebug("[ADMINISTRATION.ChangeAccountState] post_data %s", web.input["post_data"])

	if not web.request.campaignId then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field campaignId"}
	end
	if not web.request.accountId or not web.request.proxyType or type(web.request.state) ~= "boolean" then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field accountId or proxyType"}
	end
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	-- Valida si la cuenta de se encuentra configurada en algun Proceso de la App OutboundEngine.
	local isConfigured, outboundEngineResult = HasConfiguredAccountInSomeOutboundEngineProcess(web.request.campaignId, web.request.accountId, keyspace)
	if isConfigured and not (web.request.proxyType == "CALL" and web.request.state == true) then
		keyspace:Close()
		web.status = outboundEngineResult.httpCode
		return {
			result = outboundEngineResult.errType,
			reason = outboundEngineResult.errType,
			processId = outboundEngineResult.processId,
			campaignId = outboundEngineResult.campaignId,
			accountId = outboundEngineResult.accountId,
			channel =  outboundEngineResult.channel
		}
	end

	local vcc        = inconcert.authUser.virtualContactCenter
	local userId     = inconcert.authUser.userName.."@"..vcc
	local campaignId = utils.GetId(web.request.campaignId) .. "@" .. vcc

	LogInfo("[ADMINISTRATION.ChangeAccountState] User %s is change status %s to account %s in campaign %s", userId, web.request.state, web.request.accountId, campaignId)

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignId, keyspace)
	if not belongs then
		return belResp
	end

	local old_account_audit_data = GetAuditAccount(web.request.accountId, campaignId, keyspace)
	local success, result = agentCampaignController.ChangeAccountState(web.request, campaignId, keyspace)
	if success then
		result = {result = "OK"}
		web.status = 200
		LogInfo("[ADMINISTRATION.ChangeAccountState] Successfully changed account status %s. Account %s in campaign %s", web.request.state, web.request.accountId, campaignId)
	else
		web.status = 400
		LogError("[ADMINISTRATION.ChangeAccountState] Error changed account status %s. Account %s in campaign %s. Error: %s", web.request.state, web.request.accountId, campaignId, json.encode(result))
	end

	if success then
		--Create audit record for Account
		local new_account_audit_data = GetAuditAccount(web.request.accountId, campaignId, keyspace)
		if old_account_audit_data and new_account_audit_data then
			local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
			audits.Create({
				vcc = vcc,
				entity = "Campaign",
				user = inconcert.authUser.userName,
				fieldKey =  utils.GetId(campaignId),
				oldFields = old_account_audit_data,
				newFields = new_account_audit_data,
				subEntity = "Accounts",
				connection = ksHistorical
			})
			ksHistorical:Close()
		else
			LogWarning("[ADMINISTRATION.ChangeAccountState] - Audit record not created")
		end
	end
	keyspace:Close()
	return ReplyJSON(web, result)
end

function GetAccountMessages (web, campaignId, accountId)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("[ADMINISTRATION.GetAccountMessages] User '%s' on vcc '%s' retrieving '%s' account messages configuration for campaign '%s'", name, vcc, accountId, campaignId)

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local accountMessages = GetModel("Campaigns").GetAccountMessages(campaignId, accountId, keyspace)

	--LogDebug("[ADMINISTRATION.GetAccountMessages] 1 --> campaignId: '%s', accountId: '%s', accountMessages '%s'", campaignId, accountId, json.encode(accountMessages or {}))

	-- Obtengo la información de la relación de los archivos de "SharedFiles"
	for msgId, messageInfo in pairs(accountMessages) do
		for i, message in ipairs(messageInfo.options or {}) do
			if (message.MessageType or ""):lower() == "audio" then
				local fileId = message.Id
				local file = GetModel("SharedFiles").GetFiles(vcc, fileId, keyspace)
				if not utils.TableIsEmpty(file) then
				--	LogDebug("[ADMINISTRATION.GetAccountMessages] --===> File Id '%s' Shared File: %s", fileId, json.encode(file))
					for name, value in pairs(file) do
						message[name] = value
					end
				else
					message.error = {
						result = "FILE DOES NOT EXIST IN SHARED FILES"
					}
					LogWarning("[ADMINISTRATION.GetAccountMessages] ===> File Id '%s' does not exist in Shared File. Campaign: %s, Account %s.", fileId, campaignId, accountId)
				end
			end
		end
	end

	--LogDebug("[ADMINISTRATION.GetAccountMessages] 2 --> campaignId: '%s', accountId: '%s', accountMessages '%s'", campaignId, accountId, json.encode(accountMessages or {}))

	keyspace:Close()

	return accountMessages
end

function SaveAccountMessages (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	local campaign_id = web.request.campaignId
	local account_id = web.request.accountId
	local messages = web.request.messages
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	LogInfo("[ADMINISTRATION.SaveAccountMessages] User %s on vcc %s saving %s account messages configuration for campaign %s.", name, vcc, account_id, campaign_id)

	local success, err = __UpdateAccountMessages(campaign_id, account_id, messages, nil, allegroOtherInfo)
	if not success then
		web.status = 400
		LogError("[ADMINISTRATION.SaveAccountMessages] Unable to save %s account messages configuration for campaign %s. Reason: %s", account_id, campaign_id, err )
		return { result = "INVALID_REQUEST", reason = err }
	end

	LogInfo("[ADMINISTRATION.SaveAccountMessages] Succesufully saved %s account messages configuration on campaign %s.", account_id, campaign_id )
	return { result = true }
end

function InUse_CONTACTFORM ( tweetyId, keyspace )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo( "Validating tweety '%s contact form in use on campaigns. User '%s', group '%s'", tweetyId, userId, inconcert.authUser.userGroup )

	local response = { InUse = false, tweetyId = tweetyId }
	if type( tweetyId ) ~= "string" then
		LogWarning( "Validation tweety project in use invalid. Tweety project id is invalid: %s.", tweetyId )
		return response
	end

	local close_ks = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		close_ks = true
	end
	--Get all campaigns from model who belongs to the vcc
	local vccCampaigns = GetModel("Campaigns").GetVCCCampaigns( vcc, keyspace )

	for campaignId in pairs( vccCampaigns ) do
		local full_camp_id = utils.AppendVcc(campaignId, vcc)
		local camp_exists = GetModel("Campaigns").CampaignExists( full_camp_id, keyspace )
		if camp_exists then
			local contactFormInfo = utils.JSON_Decode( GetModel("Campaigns").GetSpecificColumn( full_camp_id, "ContactFormInfo", keyspace ) )
			if type( contactFormInfo ) == "table" then
				if contactFormInfo.TweetyAppName and contactFormInfo.TweetyAppName:lower() == tweetyId:lower() then
					response.campaignId = campaignId
					response.InUse = true
					break
				end
			end
		end
	end

	if close_ks then
		keyspace:Close()
	end

	if response.InUse then
		LogWarning( "Tweety project '%s' in use on contact form campaign '%s'.", tweetyId, response.campaignId )
	else
		LogInfo( "Tweety project '%s' is not in use on contact form campaigns.", tweetyId )
	end

	return response
end

function InUse_CCIPOPUP( tweetyId, keyspace )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo( "Validating tweety '%s integration in use on campaigns. User '%s', group '%s'", tweetyId, userId, inconcert.authUser.userGroup )

	local response = { InUse = false, tweetyId = tweetyId }
	if type( tweetyId ) ~= "string" then
		LogWarning( "Validation tweety project in use invalid. Tweety project id is invalid: %s.", tweetyId )
		return response
	end

	local close_ks = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		close_ks = true
	end
	--Get all campaigns from model who belongs to the vcc
	local vccCampaigns = GetModel("Campaigns").GetVCCCampaigns( vcc, keyspace )

	for campaignId in pairs( vccCampaigns ) do

		local full_camp_id = utils.AppendVcc(campaignId, vcc)
		local camp_exists = GetModel("Campaigns").CampaignExists( full_camp_id, keyspace )
		if camp_exists then

			integration = utils.JSON_Decode( GetModel("Campaigns").GetSpecificColumn( full_camp_id, "Integration", keyspace ) )
			if type( integration ) == "table" then

				if integration.type and integration.type:upper() == "CTIPOPUP" then

					if integration.application:lower() == tweetyId:lower() then
						response.campaignId = campaignId
						response.InUse = true
						break
					end
				end
			end
		end
	end

	if close_ks then
		keyspace:Close()
	end
	if response.InUse then
		LogWarning( "Tweety project '%s' in use on integration campaign '%s'.", tweetyId, response.campaignId )
	else
		LogInfo( "Tweety project '%s' is not in use on  integration campaigns.", tweetyId )
	end

	return response
end

function InUse_WEBCHAT( tweetyId, keyspace )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo( "Validating tweety '%s webchat in use on campaigns. User '%s', group '%s'", tweetyId, userId, inconcert.authUser.userGroup )

	local response = { InUse = false, tweetyId = tweetyId }
	if type( tweetyId ) ~= "string" then
		LogWarning( "Validation tweety project in use invalid. Tweety project id is invalid: %s.", tweetyId )
		return response
	end

	local close_ks = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		close_ks = true
	end

	--Get all campaigns from model who belongs to the vcc
	local vccCampaigns = GetModel("Campaigns").GetVCCCampaigns( vcc, keyspace )
	local tweetyId = tweetyId:lower()

	for campaign_id_without_vcc in pairs( vccCampaigns ) do
		local campaign_id_with_vcc = utils.AppendVcc(campaign_id_without_vcc, vcc)
		local accounts = GetModel("Campaigns").GetCampaignAccounts( campaign_id_with_vcc, keyspace )
		for _, account in pairs( accounts ) do
			if account.proxyType and account.proxyType:upper() == "CHAT" then
				if account.TweetyAppName and account.TweetyAppName:lower() == tweetyId then
					response.campaignId = campaign_id_without_vcc
					response.InUse = true
					break
				end
			end
		end
	end

	if close_ks then
		keyspace:Close()
	end

	if response.InUse then
		LogWarning( "Tweety project '%s' in use on webchat campaign '%s'.", tweetyId, response.campaignId )
	else
		LogInfo( "Tweety project '%s' is not in use on webchat campaigns.", tweetyId )
	end

	return response
end

local function ValidateIntegration(integration)
	-- basic web validation
	local function WebValidation()
		local result = {result = "CCI_CONFIGURATION_BAD_INFORMATION", reason = ""}
		-- requerido
		if type(integration.application) ~= "string" then
			result.reason = "Attribute 'integration.application' is invalid."
			return false, result
		end
		-- requerido
		if type(integration.popup) ~= "table" then
			result.reason = "Attribute 'integration.popup' is invalid."
			return false, result
		end
		-- TODO: validar mediante reg exp, si la url es valida. (integration.popup.url)
		if type(integration.popup.url) ~= "string" then
			result.reason = "Attribute 'integration.popup.url' is invalid."
			return false, result
		end
		-- requerido
		if type(integration.popup.parameters) ~= "table" then
			result.reason = "Attribute 'integration.popup.parameters' is invalid."
			return false, result
		end
		-- requerido
		if type(integration.popup.method) ~= "string" then
			result.reason = "Attribute 'integration.popup.method' is invalid."
			return false, result
		end
		-- no es requerido por eso pregunta por nil
		if type(integration.override_actions) ~= "table" and type(integration.override_actions) ~= "nil" then
			result.reason = "Attribute 'integration.override_actions' is invalid."
			return false, result
		end

		result.result = ""
		return true, result
	end
	-- tweety validation
	local function TweetyValidation()
		local success, result = WebValidation()
		if not success then
			return false, result
		end
		-- requerido y unicamente GET
		local method = integration.popup.method:upper()
		if method ~= "GET" then
			result.result = "CCI_CONFIGURATION_BAD_INFORMATION"
			result.reason = "Tweety configuration only supports the GET method. Current value attribute 'integration.popup.method=" .. method .. "'."
			return false, result
		end
		return true
	end
	-- external webs validation
	local function ExternalWebValidation()
		local success, result = WebValidation()
		if not success then
			return false, result
		end
		-- requerido
		local method = integration.popup.method:upper()
		if method ~= "GET" and method ~= "POST" then
			result.result = "CCI_CONFIGURATION_BAD_INFORMATION"
			result.reason = "External web configuration only supports the GET or POST method. Current value attribute 'integration.popup.method=" .. method .. "'."
			return false, result
		end
		return true
	end

	local int_type = integration.type

	if type(int_type) == "string" then
		local recognisedIntegrations = { CTIPOPUP = TweetyValidation, CTIALLEGROPOPUP = TweetyValidation, CTIPOPUPWEB = ExternalWebValidation }

		if not recognisedIntegrations[int_type] then
			LogError("CCI configuration type %s not recognised", int_type)
			return false, {result = "CCI_CONFIGURATION_TYPE_NOT_RECOGNISED", reason = "Integration type '" .. int_type .. "' not recognised"}
		end

		local success, error_type = recognisedIntegrations[int_type]()
		if not success then
			LogError("CCI configuration error. %s", error_type.reason)
			return false, error_type
		end
	end

	return true
end

--[[
	1 - Esta función se encarga de guardar unicamente la estructura necesaria de la conf del CCI.
	Por las dudas que en el POST llegue basura, datos que no se necesitan. (Ejemplo, si se utiliza el API de una app externa)
	2 - Tambien agrega los valores por defecto. En el caso de una integración de TWEETY, el parámetro "interaction_id" con el valor "{INTERACTIONID}"
	3 - Convierte a minúsculas las claves de los override actions
--]]
local function IntegrationSetDefaultDataAndCleanUp (postedIntegration)
	local integration = {
		-- M&S Integration, HelpDesk, etc.
		othersIntegrations = postedIntegration.othersIntegrations
	}

	if postedIntegration.type then
		integration.type             = postedIntegration.type
		integration.application      = postedIntegration.application
		integration.window_open_mode = postedIntegration.window_open_mode
		integration.trigger          = { embed = true }  -- siempre es embebido
		integration.popup = {
			url             = postedIntegration.popup.url,
			url_is_variable = postedIntegration.popup.url_is_variable,
			method          = postedIntegration.popup.method,
			parameters      = postedIntegration.popup.parameters
		}
		--override_actions -> agregan abajo
	end

	-- Agrego los parametros por defecto
	local function AddDefaultParameters (popupParametersDestination)
		local defaultParameters = { 
			interaction_id = "{INTERACTIONID}",
			INTERACTIONID  = "{INTERACTIONID}",
			AGENTID        = "{AGENTID}",
			VIRTUALCC      = "{VIRTUALCC}"
		}
		for name, value in pairs(defaultParameters) do
			popupParametersDestination[name] = value
		end
	end
	-- Agrego los parametros al popup (cci) para los otros tipos de integraciones (helpdesk, m&s)
	for integrationType, info in pairs(integration.othersIntegrations or {}) do
		if info.Config then
			info.Config.popup = info.Config.popup or {}
			info.Config.popup.parameters = info.Config.popup.parameters or {}
			AddDefaultParameters(info.Config.popup.parameters)
		end
	end

	-- set default values by integration type.
	if integration.type == "CTIPOPUP" or integration.type == nil then
		--[[
			Esta lista de parámetros (variables) por defecto, que se insertan se debe de mantener igual en los luas de 5.5.
			Específicamente en: 
				\Web\inConcert\apps\omnichannelintegration\templates.lua
				funcion "__MakeAllegroCampaignIntegration"
			Estas listas tienen que tener los mismo valores, es decir, si se modifica acá también se debe de modificar allá.
		]]--

		if postedIntegration.popup then
			-- Se debe parsear las URLs del WebDesigner
			integration.popup.url_parts = utils.ParseWebDesignerURL(postedIntegration.popup.url)
			-- Agrego los parametros al popup (cci)
			AddDefaultParameters(integration.popup.parameters)
		end
	end

	--sanitize override actions
	if postedIntegration.override_actions then
		integration.override_actions = {}
		for action_id, action in pairs(postedIntegration.override_actions) do
			integration.override_actions[action_id:lower()] = action
		end
	end

	return integration
end

function SaveIntegrationCCI (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local postedCampaignId = web.request.campaignId
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)

	LogInfo("[ADMINISTRATION.SaveIntegrationCCI] - User %s on vcc %s saving integration CCI configuration for campaign '%s' posted info %s",
		userId, 
		vcc, 
		postedCampaignId, 
		web.input["post_data"]
	)

	if type(postedCampaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign integration. Field campaignId is invalid."
		})
	end

	local campaignIdAtVCC = utils.AppendVcc(postedCampaignId, vcc)
	local integration = web.request.integration
	if type(integration) ~= "table" and type(integration) ~= "nil" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign integration. Invalid field integration type: '%s'.", campaignIdAtVCC, type(integration))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign integration. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	if integration then
		if type(integration.type) == "string" or integration.othersIntegrations then
			LogInfo("[ADMINISTRATION.SaveIntegrationCCI] -Validate CCI for campaign %s.", campaignIdAtVCC)
			local success, result = ValidateIntegration(integration)
			if not success then
				keyspace:Close()
				return returnError(web, result )
			else
				LogInfo("[ADMINISTRATION.SaveIntegrationCCI] Successfully validated posted data for campaign %s... !!!!", campaignIdAtVCC)
				integration = IntegrationSetDefaultDataAndCleanUp(integration)
			end
		else
			integration = nil
		end
	end

	local currentIntegration = utils.JSON_Decode( GetModel("Campaigns").GetSpecificColumn( campaignIdAtVCC, "Integration", keyspace ) )
	local hasntChanged = utils.StructuralCompare(currentIntegration, integration)
	if hasntChanged and not web.request.forceSave then
		keyspace:Close()
		LogInfo("[ADMINISTRATION.SaveIntegrationCCI] - Campaign %s. The integration has not changed. Current information is identical to the new one.", campaignIdAtVCC)
		LogInfo("\n currentIntegration : %s", json.encode(currentIntegration))
		LogInfo("\n newIntegration     : %s", json.encode(integration))
		return { result = "OK" }
	end

	--[[
		Esta función actualiza las notificaciones que se van a actualizar en la CF "Campaigns" Column "InteractionNotifications".
		Esto se hace así, ya que si en CCI se activa (SOLO se activa) una integracion "helpdesk" o "m&s", de forma automática se tiene que activar en las notificaciones,
			ya que existe una dependencia entre ellas.
		
		En la CF "InteractionNotifications" se recupera y actualiza el JSON:
			{"Apps":{"MarketingAndSales":true,"HelpDesk":true}}
	]]--
	local function UpdateNotifications ()
		if not integration.othersIntegrations then
			return false
		end

		local currentInteractionNotifications = utils.JSON_Decode(GetModel("Campaigns").GetSpecificColumn(campaignIdAtVCC, "InteractionNotifications", keyspace))
		--LogDebug("--1---------> %s", currentInteractionNotifications and json.encode(currentInteractionNotifications))

		-- Genero la estructura basica si es que la misma no existe en cassandra.
		if not currentInteractionNotifications then
			currentInteractionNotifications = {Apps = {}}
		else
			if not currentInteractionNotifications.Apps then
				currentInteractionNotifications.Apps = {}
			end
		end

		-- Se deja comentada la integracion de notificaciones relacionada con HELPDESK por si en algún momento se necesita activar!!!
		local mappedNotificatons = {
			--HELPDESK = "HelpDesk",
			MAS = "MarketingAndSales"
		}

		-- Aca recorro lo que llega en la integración para las integraciones y habilito en las notificaciones si es necesario.
		local hasUpdated = false
		for integrationId, info in pairs(integration.othersIntegrations) do
			local entityName = mappedNotificatons[integrationId]
			if entityName then
				if info.Enable then
					-- Tengo que actualizar
					hasUpdated = true
					currentInteractionNotifications.Apps[entityName] = true
				end
			end
		end

		--LogDebug("--2---has update: %s------> %s", hasUpdated, currentInteractionNotifications and json.encode(currentInteractionNotifications))

		return hasUpdated, currentInteractionNotifications
	end

	local success, err, hasUpdatedInteractionNotifications, interactionNotifications
	local auditOperation = "Deleting"
	local old_audit_integrations = GetAuditCampaign(campaignIdAtVCC, keyspace, "Integrations")
	if integration then
		local actionName = currentIntegration and "Updating" or "Adding"
		LogInfo("[ADMINISTRATION.SaveIntegrationCCI] - %s integration to campaign %s.", actionName, campaignIdAtVCC)

		local campaign = {Id = campaignIdAtVCC, Integration = integration}
		-- Invoco a la funcion que actualiza la tabla de Notificaciones
		hasUpdatedInteractionNotifications, interactionNotifications = UpdateNotifications()
		if hasUpdatedInteractionNotifications then
			-- Actualizo las "InteractionNotifications" para que se actualicen en cassandra
			campaign.InteractionNotifications = interactionNotifications
		end

		local allegro_info = { action_id = "SAVE_INTEGRATION", Integration = integration }

		success, err = GetModel("Campaigns").SaveCampaign(vcc, campaign, keyspace, allegro_info)
		auditOperation = actionName
	else
		LogInfo("[ADMINISTRATION.SaveIntegrationCCI] - Removing integration from campaign %s.", campaignIdAtVCC)
		success, err = GetModel("Campaigns").RemoveIntegration(campaignIdAtVCC, {"Integration"}, keyspace)
	end

	if not success then
		keyspace:Close()
		return returnError(web, err)
	end

	local update_audit_integrations = GetAuditCampaign(campaignIdAtVCC, keyspace, "Integrations")
	if old_audit_integrations or update_audit_integrations then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignIdAtVCC),
			oldFields = auditOperation ~= "Adding" and old_audit_integrations or nil,
			newFields = auditOperation ~= "Deleting" and update_audit_integrations or nil,
			subEntity = "Integrations",
			connection = ksHistorical
		})
		ksHistorical:Close()
	else
		LogWarning("[ADMINISTRATION.Labels] Audit record not created")
	end

	agentNotifications.NotifyAgents({
		eventType = "administrator_integration_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId  = campaignIdAtVCC,
			integration = integration
		}
	}, keyspace)

	if hasUpdatedInteractionNotifications then
	-- Como las "InteractionNotifications" se han actualizado en cassandra, tengo que notificar el cambio
		agentNotifications.NotifyAgents({
			eventType = "administrator_interaction_notifications_change",
			agentsLevel = "CAMPAIGN_USERS",
			pollerMessage = {
				campaignId  = campaignIdAtVCC,
				interactionNotifications = interactionNotifications
			}
		}, keyspace)
	end

	--notifications
	notificationsEngine.SendNotification("CampaignIntegrationUpdated", 0, { campaignId = utils.GetId(postedCampaignId), integration = integration or false })

	keyspace:Close()
	return { result = "OK" }
end

local function ValidateInteractionNotifications (interactionNotifications)
	local result = {result = "", reason = ""}
	return true, result
end

--[[
	1 - Esta función se encarga de guardar unicamente la estructura necesaria de la conf del Interaction Notifications.
		Por las dudas que en el POST llegue basura, datos que no se necesitan. (Ejemplo, si se utiliza el API de una app externa)
--]]
local function InteractionNotificationsSetDefaultDataAndCleanUp (data)
	-- clean up
	return {
		Apps = data.Apps or {}
	}
end

local function HasConfiguredInteractionNotificationsInVCC (vcc, keyspace)
	local agent = require "apps.agent.agent"
	return agent.HasConfiguredInteractionNotificationsInVCC(vcc, keyspace)
end

function SaveInteractionNotifications (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local postedCampaignId = web.request.campaignId

	LogInfo("[ADMINISTRATION.SaveInteractionNotifications] User %s on vcc %s saving Interaction Notifications configuration for campaign '%s' posted info %s",
		userId,
		vcc, 
		postedCampaignId,
		web.input["post_data"]
	)

	if type(postedCampaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign Interaction Notifications. Field campaignId is invalid."
		})
	end

	local campaignIdAtVCC = utils.AppendVcc(postedCampaignId, vcc)
	local interactionNotifications = (web.request.Data or {})
	if type(interactionNotifications) ~= "table" and type(interactionNotifications) ~= "nil" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign Interaction Notifications. Invalid field Interaction Notifications type: '%s'.", campaignIdAtVCC, type(interactionNotifications))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local hasConfigured = HasConfiguredInteractionNotificationsInVCC(vcc, keyspace)
	if not hasConfigured then
		keyspace:Close()
		return returnError(web, {
			result = "NOT_SUPPORTED_INTERACTION_NOTIFICATIONS_IN_VCC",
			reason = string.format("Unable to save %s campaign Interaction Notifications. The VCC '%s' does not support Interaction Notifications.", campaignIdAtVCC, vcc)
		})
	end

	local campExists = GetModel("Campaigns").CampaignExists(campaignIdAtVCC, keyspace)
	if not campExists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign Interaction Notifications. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	if interactionNotifications then
		LogInfo("[ADMINISTRATION.SaveInteractionNotifications] Validate Interaction Notifications for campaign %s.", campaignIdAtVCC)
		local success, result = ValidateInteractionNotifications(interactionNotifications)
		if not success then
			keyspace:Close()
			return returnError(web, result )
		else
			LogInfo("[ADMINISTRATION.SaveInteractionNotifications] Successfully validated posted data for campaign %s... !!!!", campaignIdAtVCC)
			interactionNotifications = InteractionNotificationsSetDefaultDataAndCleanUp(interactionNotifications)
		end
	else
		interactionNotifications = nil
	end

	local currentInteractionNotifications = utils.JSON_Decode(GetModel("Campaigns").GetSpecificColumn(campaignIdAtVCC, "InteractionNotifications", keyspace))
	local hasChanged = utils.StructuralCompare(currentInteractionNotifications, interactionNotifications)
	if hasChanged then
		keyspace:Close()
		LogInfo("[ADMINISTRATION.SaveInteractionNotifications] - Campaign %s. The Interaction Notifications has not changed. Current information is identical to the new one.", campaignIdAtVCC)
		LogInfo("\n currentInteractionNotifications : %s", json.encode(currentInteractionNotifications))
		LogInfo("\n newInteractionNotifications     : %s", json.encode(interactionNotifications))
		return { result = "OK" }
	end

	local success, err
	local operationAudit = "Deleting"
	if interactionNotifications then
		local actionName = currentInteractionNotifications and "Updating" or "Adding"
		LogInfo("[ADMINISTRATION.SaveInteractionNotifications] %s Interaction Notifications to campaign %s.", actionName, campaignIdAtVCC)
		local campaign = {Id = campaignIdAtVCC, InteractionNotifications = interactionNotifications}
		success, err = GetModel("Campaigns").SaveCampaign(vcc, campaign, keyspace)
		operationAudit = actionName
	else
		LogInfo("[ADMINISTRATION.SaveInteractionNotifications] Removing Interaction Notifications from campaign %s.", campaignIdAtVCC)
		success, err = GetModel("Campaigns").RemoveInteractionNotifications(campaignIdAtVCC, {"InteractionNotifications"}, keyspace)
	end

	if not success then
		keyspace:Close()
		return returnError(web, err)
	end
	
	agentNotifications.NotifyAgents({
		eventType = "administrator_interaction_notifications_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId  = campaignIdAtVCC,
			interactionNotifications = interactionNotifications
		}
	}, keyspace)

	keyspace:Close()

	--Create audit record
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	audits.Create({
		vcc = vcc,
		entity = "Campaign",
		user = inconcert.authUser.userName,
		fieldKey =  utils.GetId(campaignIdAtVCC),
		oldFields = operationAudit ~= "Creating" and FilterAuditFields(currentInteractionNotifications, "Notifications") or nil,
		newFields = operationAudit ~= "Deleting" and FilterAuditFields(interactionNotifications,"Notifications") or nil,
		subEntity = "Notifications",
		connection = ksHistorical
	})
	ksHistorical:Close()

	return { result = "OK" }
end

local function CanDeleteAttentionLevels (campaign_at_vcc, attention_levels_to_remove, keyspace)
	local response = { success = true }
	local modelCampaign = GetModel("Campaigns")

	local accounts = modelCampaign.GetCampaignAccounts( campaign_at_vcc, keyspace )
	if not accounts then
		return response
	end

	for _, att_level_id_to_remove in pairs(attention_levels_to_remove) do
		for _, account in pairs(accounts) do
		-- 1 - Valida que el attention level no tenga ninguna interaccion en WAITING.
			local numberWaitingConversations = modelCampaign.CountWaitingConversationsForAttentionLevel(campaign_at_vcc, account.id, att_level_id_to_remove)
			if numberWaitingConversations > 0 then
				response.success = false
				response.result = "ATTENTION_LEVEL_WITH_INTERACTIONS_IN_WAITING"
				response.att_level_id = att_level_id_to_remove
				response.account_id = account.id
				response.number_waiting_conversations = numberWaitingConversations
				break
			end
		-- 2 - Valida que el attention level no se encuentre en uso en alguna account, seteado con distribution en true.
			for _, assigned_att_level in pairs(account.AttentionLevels or {}) do
				if assigned_att_level.distribution and att_level_id_to_remove == assigned_att_level.attention_level_id then
					response.success = false
					response.result = "ATTENTION_LEVEL_IN_USE_IN_ACCOUNT"
					response.att_level_id = att_level_id_to_remove
					response.account_id = account.id
					break
				end
			end
			if not response.success then
				break
			end
		end
		if not response.success then
			break
		end
	end

	return response
end

-- canned messages
function SaveCannedMessages (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s saving canned messages configuration for campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign canned messages. Field campaignId is invalid."
		})
	end

	-- Check if campaign exists
	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	if type(web.request.canned_messages) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign canned messages. Invalid field canned messages type: '%s'.", campaignIdAtVCC, type(web.request.canned_messages))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign canned messages. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	local canned_messages = {}
	for i, canned in ipairs(web.request.canned_messages) do
		local canned_id = canned.canned_id
		--si el canned id no llega en el request, genero uno nuevo
		if not canned_id then
			canned_id = uuid.new()
			canned.canned_id = canned_id
		end
		canned_messages[canned_id] = canned
	end

	-- valida si los canned han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "add_or_update"
	local old_canned_messages = GetModel("Campaigns").GetCannedMessages(campaignIdAtVCC, keyspace)
	local has_changed, differences = CompareListDifferences(campaignIdAtVCC, canned_messages, old_canned_messages, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Canned Differences: %s", json.encode(differences))
	GetModel("Campaigns").SaveCannedMessages(campaignIdAtVCC, canned_messages, keyspace)

	LogInfo("Campaign %s, canned messages successfully saved", campaignIdAtVCC)
	agentNotifications.NotifyAgents({
		eventType = "administrator_canned_messages_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId      = campaignIdAtVCC,
			canned_messages = differences
		}
	}, keyspace)

	keyspace:Close()

	--Agrego registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end
		local old_audit_cannend = nil
		local new_audit_cannend = nil
		if value.type == "UPDATE" then
			local filterOld = FilterAuditFields(value.old_row or {}, "Templates")
			local filterNew = FilterAuditFields(value.updated_row or {}, "Templates")
			old_audit_cannend = {
				[key] = filterOld
			}
			new_audit_cannend = {
				[key] = filterNew
			}
		else
			new_audit_cannend = {
				[key] = FilterAuditFields(value.row or {}, "Templates")
			}
		end
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignIdAtVCC),
			oldFields = old_audit_cannend,
			newFields = new_audit_cannend,
			subEntity = "Templates",
			connection = connectionHistorical
		})

	end

	if connectionHistorical then
		connectionHistorical:Close()
	end
	return {result = "OK"}
end

function DeleteCannedMessages(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s removing canned messages configuration on campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to delete %s campaign canned messages. Field campaignId is invalid."
		})
	end
	-- Check if campaign exists
	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local canned_list = web.request.canned_messages
	if type(canned_list) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to delete %s campaign canned messages. Invalid field canned messages type: '%s'.", campaignIdAtVCC, type(canned_list))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to delete %s campaign canned messages. Invalid field canned messages type: '%s'.", campaignIdAtVCC, type(web.request.canned_messages))
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	local arr_canned_messages = {}
	for i, canned_id in ipairs(canned_list) do
		arr_canned_messages[canned_id] = true
	end

	-- valida si los canned han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "delete"
	local old_canned_messages = GetModel("Campaigns").GetCannedMessages(campaignIdAtVCC, keyspace)
	local has_changed, differences = CompareListDifferences(campaignIdAtVCC, arr_canned_messages, old_canned_messages, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Canned Differences: %s", json.encode(differences))
	GetModel("Campaigns").DeleteCannedMessages(campaignIdAtVCC, canned_list, keyspace)

	LogInfo("Campaign %s, canned messages successfully deleted", campaignIdAtVCC)
	agentNotifications.NotifyAgents({
		eventType = "administrator_canned_messages_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId      = campaignIdAtVCC,
			canned_messages = differences
		}
	}, keyspace)

	keyspace:Close()

	--Se crea registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do
		local old_audit_canned = {
			[key] = FilterAuditFields(value.deleted_row or {}, "Templates")
		}
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end

		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignIdAtVCC),
			oldFields = old_audit_canned,
			newFields = nil,
			subEntity = "Templates",
			connection = connectionHistorical
		})

	end
	if connectionHistorical then
		connectionHistorical:Close()
	end

	return {result = "OK"}
end

function GetCannedMessages(web, campaignId)
	LogInfo("Getting canned messages for campaign: %s ", campaignId)
	if not campaignId then
		web.status = 400
		return false, { result = "INVALID_REQUEST", reason = "You must provide a campaign id to get canned messages" }
	end
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)

	-- Check if campaign exists
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		web.status = 400
		return false, { result = "CAMPAIGN_DOES_NOT_EXIST", reason = string.format("Campaign %s does not exist", campaignIdAtVCC) }
	end

	local canned_messages = GetModel("Campaigns").GetCannedMessages(campaignIdAtVCC, keyspace)
	keyspace:Close()
	LogInfo("Retrieving canned messages to campaign: %s,  messages: %s", campaignIdAtVCC, canned_messages)
	return canned_messages
end

-- labels
function SaveLabels(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s saving labels configuration for campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign labels. Field campaignId is invalid."
		})
	end

	-- Check if campaign exists
	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local posted_labels = web.request.labels
	if type(posted_labels) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign labels. Invalid field labels type: '%s'.", campaignIdAtVCC, type(posted_labels))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign labels. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	
	if not belongs then
		return belResp
	end

	local LABEL_MAX_LENGTH = 80
	local reserved_words = { PENDING = true, RESULT = true }
	local labels = {}
	for i, label in ipairs(posted_labels) do
		local id = (label.id or uuid.new()):upper()
		if #id > LABEL_MAX_LENGTH then
			keyspace:Close()
			return returnError(web, {
				result = "OVERFLOW",
				data   = { label_id = label.name, maxLength = LABEL_MAX_LENGTH },
				reason = string.format("Unable to save %s campaign labels. '%s' label has exceeded the size of %d characters.", campaignIdAtVCC, label.name, LABEL_MAX_LENGTH)
			})
		end
		if reserved_words[id] then
			keyspace:Close()
			return returnError(web, {
				result = "RESERVED_WORD",
				data   = { label_id = label.name },
				reason = string.format("Unable to save %s campaign labels. '%s' label is a reserved word.", campaignIdAtVCC, label.name )
			})
		end
		label.id = id
		labels[id] = label
	end

	-- valida si los label han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "add_or_update"
	local old_labels = GetModel("Campaigns").GetLabels(campaignIdAtVCC, keyspace)
	local old_labels_copy = old_labels and not utils.TableIsEmpty(old_labels) and utils.DeepCopy(old_labels) or nil
	local has_changed, differences = CompareListDifferences(campaignIdAtVCC, labels, old_labels, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Labels Differences: %s", json.encode(differences))
	GetModel("Campaigns").SaveLabels(campaignIdAtVCC, labels, keyspace)

	--Se crea registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do

		local old_audit_label = nil
		local new_audit_label = nil
		if value.type == "UPDATE" then
			local filterOld = FilterAuditFields(value.old_row or {}, "Labels")
			local filterNew = FilterAuditFields(value.updated_row or {}, "Labels")
			old_audit_label = {
				[key] = filterOld
			}
			new_audit_label = {
				[key] = filterNew
			}
		else
			new_audit_label = {
				[key] = FilterAuditFields(value.row or {}, "Labels")
			}
		end
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end

		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignIdAtVCC),
			oldFields = old_audit_label,
			newFields = new_audit_label,
			subEntity = "Labels",
			connection = connectionHistorical
		})

	end
	if connectionHistorical then
		connectionHistorical:Close()
	end
	LogInfo("Campaign %s, labels successfully saved", campaignIdAtVCC)

	agentNotifications.NotifyAgents({
		eventType = "administrator_labels_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId = campaignIdAtVCC,
			labels     = differences
		}
	}, keyspace)

	keyspace:Close()
	return {result = "OK"}
end

local function CanDeleteLabels(campaign_at_vcc, labels_to_remove)
	-- a esta función la dejo, para realizar una validación en un futuro
	local response = { success = true }
	return response
end

function DeleteLabels (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s removing labels configuration on campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to delete %s campaign labels. Field campaignId is invalid."
		})
	end
	-- Check if campaign exists
	local campaign_at_vcc = utils.AppendVcc(campaignId, vcc)
	local labels = web.request.labels
	if type(labels) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to delete %s campaign labels. Invalid field labels type: '%s'.", campaign_at_vcc, type(labels))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local camp_exists = GetModel("Campaigns").CampaignExists( campaign_at_vcc, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to delete %s campaign labels. Invalid field labels type: '%s'.", campaign_at_vcc, type(web.request.labels))
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaign_at_vcc, keyspace)
	if not belongs then
		return belResp
	end

	local result = CanDeleteLabels(campaign_at_vcc, labels)
	if not result.success then
		keyspace:Close()
		result.reason = string.format("Unable to delete labels in '%s' campaign. Error type: '%s'.", campaign_at_vcc, json.encode(result))
		return returnError(web, result)
	end

	local arr_labels = {}
	for i, label_id in ipairs(labels) do
		arr_labels[label_id] = true
	end

	-- valida si los labels han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "delete"
	local old_labels = GetModel("Campaigns").GetLabels(campaign_at_vcc, keyspace)
	local has_changed, differences = CompareListDifferences(campaign_at_vcc, arr_labels, old_labels, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Labels Differences: %s", json.encode(differences))

	GetModel("Campaigns").DeleteLabels(campaign_at_vcc, labels, keyspace)
	LogInfo("Campaign %s, labels successfully deleted", campaign_at_vcc)

	--Se crea registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do
		local old_audit_label = {
			[key] = FilterAuditFields(value.deleted_row or {}, "Labels")
		}
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaign_at_vcc),
			oldFields = old_audit_label,
			newFields = nil,
			subEntity = "Labels",
			connection = connectionHistorical
		})

	end
	if connectionHistorical then
		connectionHistorical:Close()
	end

	agentNotifications.NotifyAgents({
		eventType = "administrator_labels_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId = campaign_at_vcc,
			labels     = differences
		}
	}, keyspace)

	keyspace:Close()
	return {result = "OK"}
end

-- rules
function SaveRules(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s saving rules configuration for campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign rules. Field campaignId is invalid."
		})
	end

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local posted_rules = web.request.rules
	if type(posted_rules) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign rules. Invalid field rules type: '%s'.", campaignIdAtVCC, type(posted_rules))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save %s campaign rules. Campaign does not exist.", campaignIdAtVCC)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	local rules = {}
	for i, rule in ipairs(posted_rules) do
		local id = (rule.id or uuid.new()):upper()
		rule.id = id
		rules[id] = rule
	end

	-- valida si los rules han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "add_or_update"
	local old_rules = GetModel("Campaigns").GetRules(campaignIdAtVCC, keyspace)
	local has_changed, differences = CompareListDifferences(campaignIdAtVCC, rules, old_rules, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Rules Differences: %s", json.encode(differences))
	GetModel("Campaigns").SaveRules(campaignIdAtVCC, rules, keyspace)
	--Se crea registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do

		local old_audit_rule = nil
		local new_audit_rule = nil
		if value.type == "UPDATE" then
			local filterOldRule = FilterAuditFields(value.old_row or {}, "Rules")
			local filterNewRule = FilterAuditFields(value.updated_row or {}, "Rules")
			old_audit_rule = {
				[key] = filterOldRule
			}
			new_audit_rule = {
				[key] = filterNewRule
			}
		else
			new_audit_rule = {
				[key] = FilterAuditFields(value.row or {}, "Rules")
			}
		end
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaignIdAtVCC),
			oldFields = old_audit_rule,
			newFields = new_audit_rule,
			subEntity = "Rules",
			connection = connectionHistorical
		})

	end
		
	if connectionHistorical then
		connectionHistorical:Close()
	end
	keyspace:Close()
	LogInfo("Campaign %s, rules successfully saved", campaignIdAtVCC)

	return {result = "OK"}
end

local function CanDeleteRules(campaign_at_vcc, rules_to_remove)
	-- a esta función la dejo, para realizar una validación en un futuro
	local response = { success = true }
	return response
end

local function FixRulesIndexes (web, campaign_at_vcc, keyspace)
	LogInfo("Fixing rules indexes in campaign: %s", campaign_at_vcc)
	-- acomoda los indices para no dejar espacios en blanco
	local changed_rules = {}

	local new_rules = GetModel("Campaigns").GetRules(campaign_at_vcc, keyspace)
	local sorted_rules = {}
	for _, rule in pairs(new_rules) do
		table.insert(sorted_rules, rule)
	end
	table.sort(sorted_rules, function (a, b)
		return tonumber(a.index) < tonumber(b.index)
	end)

	local current_index = 0
	for _, rule in pairs(sorted_rules) do
		if tonumber(rule.index) ~= current_index then
			rule.index = current_index
			changed_rules[rule.id] = rule
		end
		current_index = current_index + 1
	end

	LogInfo("Saving fixed rules: %s %s", campaign_at_vcc, json.encode(changed_rules))
	GetModel("Campaigns").SaveRules(campaign_at_vcc, changed_rules, keyspace)
	return {result = "OK"}
end

function DeleteRules (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	local campaignId = web.request.campaignId
	LogInfo("Administration - User %s on vcc %s removing rules configuration on campaign '%s'", 
		userId, 
		vcc, 
		campaignId
	)

	if type(campaignId) ~= "string" then
		return returnError(web, {
			reason = "Unable to delete %s campaign rules. Field campaignId is invalid."
		})
	end
	-- Check if campaign exists
	local campaign_at_vcc = utils.AppendVcc(campaignId, vcc)
	local rules = web.request.rules
	if type(rules) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to delete %s campaign rules. Invalid field rules type: '%s'.", campaign_at_vcc, type(rules))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( campaign_at_vcc, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to delete %s campaign rules. Invalid field rules type: '%s'.", campaign_at_vcc, type(web.request.rules))
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaign_at_vcc, keyspace)
	if not belongs then
		return belResp
	end

	local result = CanDeleteRules(campaign_at_vcc, rules)
	if not result.success then
		keyspace:Close()
		result.reason = string.format("Unable to delete rules in '%s' campaign. Error type: '%s'.", campaign_at_vcc, json.encode(result))
		return returnError(web, result)
	end

	local arr_rules = {}
	for i, rule_id in ipairs(rules) do
		arr_rules[rule_id] = true
	end

	-- valida si los rules han cambiado, si no han cambiado no guarda, pero no es un error
	local validation_type = "delete"
	local old_rules = GetModel("Campaigns").GetRules(campaign_at_vcc, keyspace)
	local has_changed, differences = CompareListDifferences(campaign_at_vcc, arr_rules, old_rules, validation_type)
	if not has_changed then
		keyspace:Close()
		return {result = "OK"}
	end
	LogInfo("Rules Differences: %s", json.encode(differences))

	GetModel("Campaigns").DeleteRules(campaign_at_vcc, rules, keyspace)
	--Se crea registro de auditoria
	local connectionHistorical
	for key, value in pairs(differences) do
		local old_audit_rule = {
			[key] = FilterAuditFields(value.deleted_row or {}, "Rules")
		} 
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end
		audits.Create({
			vcc = vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(campaign_at_vcc),
			oldFields = old_audit_rule,
			newFields = nil,
			subEntity = "Rules",
			connection = connectionHistorical
		})
	end
	if connectionHistorical then
		connectionHistorical:Close()
	end
	keyspace:Close()

	LogInfo("Campaign %s, rules successfully deleted", campaign_at_vcc)

	return FixRulesIndexes(web, campaign_at_vcc, keyspace)
end

function GetRules(web, campaignId)
	LogInfo("Getting rules for campaign: %s ", campaignId)
	if not campaignId then
		web.status = 400
		return false, { result = "INVALID_REQUEST", reason = "You must provide a campaign id to get rules" }
	end

	local vcc             = inconcert.authUser.virtualContactCenter
	local userId          = inconcert.authUser.userName.."@"..vcc
	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( campaignIdAtVCC, keyspace )
	if not camp_exists then
		keyspace:Close()
		web.status = 400
		return false, { result = "CAMPAIGN_DOES_NOT_EXIST", reason = string.format("Campaign %s does not exist", campaignIdAtVCC) }
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", userId, campaignIdAtVCC, keyspace)
	if not belongs then
		return belResp
	end

	local rules = GetModel("Campaigns").GetRules(campaignIdAtVCC, keyspace)
	keyspace:Close()
	LogInfo("Retrieving rules to campaign: %s,  messages: %s", campaignIdAtVCC, rules)

	return rules
end

function GetCampaignAccountsForTransfer(web)

	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[GetCampaignsAccountsForTransfer] (START) Retrieving campaigns accounts to transfer for '%s' VCC.", vcc)

	local campaigns_accounts = { ALL = {} }
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Todas las cuentas de todas las campanas del vcc
	local vccCampaigns = GetModel("Campaigns").GetVCCCampaigns(vcc, keyspace)
	if not vccCampaigns then
		keyspace:Close()
		LogWarning("[GetCampaignsAccountsForTransfer] (END) Unable to retrieving campaigns accounts to transfer. Not exist campaigns in '%s' VCC.", vcc)
		return campaigns_accounts
	end

	for campaign_id_without_vcc in pairs( vccCampaigns ) do

		local full_campaign_id = utils.AppendVcc(campaign_id_without_vcc, vcc)
		local campaign_name = GetModel("Campaigns").GetSpecificColumn( full_campaign_id, "Name", keyspace )
		local accounts = GetModel("Campaigns").GetCampaignAccounts(full_campaign_id, keyspace, { "proxyType", "name" })

		for account_id, account in pairs( accounts ) do
			campaigns_accounts[account.proxyType] = campaigns_accounts[account.proxyType] or {}

			campaigns_accounts[account.proxyType][full_campaign_id] = campaigns_accounts[account.proxyType][full_campaign_id] or { Name = campaign_name, accounts = {} }
			campaigns_accounts.ALL[full_campaign_id] = campaigns_accounts.ALL[full_campaign_id] or { Name = campaign_name, accounts = {} }

			campaigns_accounts[account.proxyType][full_campaign_id].accounts[account_id] = { Name = account.name }
			campaigns_accounts.ALL[full_campaign_id].accounts[account_id] = { Name = account.name }
		end
	end

	LogInfo("[GetCampaignsAccountsForTransfer] (END) Retrieving campaigns and accounts to transfer for '%s' VCC.", vcc)

	keyspace:Close()
	return campaigns_accounts
end

function GetWebcontactCampaignAccounts(web)

	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[GetWebcontactCampaignAccounts] (START) Retrieving campaigns accounts for '%s' VCC.", vcc)

	local campaigns_accounts = {}
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Todas las cuentas de todas las campanas del vcc
	local vccCampaigns = GetModel("Campaigns").GetVCCCampaigns(vcc, keyspace)
	if not vccCampaigns then
		keyspace:Close()
		LogWarning("[GetWebcontactCampaignAccounts] (END) Unable to retrieving campaigns accounts. Not exist campaigns in '%s' VCC.", vcc)
		return campaigns_accounts
	end

	for campaign_id_without_vcc in pairs( vccCampaigns ) do

		local full_campaign_id = utils.AppendVcc(campaign_id_without_vcc, vcc)
		local campaign_name = GetModel("Campaigns").GetSpecificColumn( full_campaign_id, "Name", keyspace )
		local accounts = GetModel("Campaigns").GetCampaignAccounts(full_campaign_id, keyspace, { "proxyType", "name", "Token" })

		for account_id, account in pairs( accounts ) do
			if account.proxyType == "WEBCONTACT" then
				campaigns_accounts[full_campaign_id] = campaigns_accounts[full_campaign_id] or { Name = campaign_name, accounts = {} }

				campaigns_accounts[full_campaign_id].accounts[account_id] = { Name = account.name, Token = account.Token }
			end
		end
	end

	LogInfo("[GetWebcontactCampaignAccounts] (END) Retrieving campaigns and accounts for '%s' VCC.", vcc)

	keyspace:Close()
	return campaigns_accounts
end

function UpdateAgentServiceMembers( web )

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. inconcert.authUser.virtualContactCenter

	local camp_id_without_vcc, camp_vcc = utils.SeparateVCC( web.request.full_camp_id )
	local data = {
		camp_id_without_vcc = camp_id_without_vcc,
		camp_vcc            = camp_vcc,
		action              = web.request.action,
		members             = web.request.members
	}

	LogInfo("[UpdateAgentServiceMembers] Updating Agent Service Members info, user '%s', action: '%s', camp_vcc '%s' camp_id '%s', members: '%s'", userId, data.action, data.camp_vcc, data.camp_id_without_vcc, data.members)

	if not data.camp_id_without_vcc then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field camp_id_without_vcc"}
	end
	if not data.camp_vcc then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field camp_vcc"}
	end
	if not data.action then
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Missing field action"}
	end

	local remove_all_members = function()
		local current_members = GetModel("Campaigns").GetCampaignMembers( data.camp_vcc, data.camp_id_without_vcc )
		for full_user_id in pairs(current_members) do
			local user_id = utils.GetId(full_user_id)
			LogDebug("-[UpdateAgentServiceMembers] agentservice.RemoveAgentFromCampaign %s, %s, %s", user_id, data.camp_id_without_vcc, data.camp_vcc)
			agentservice.RemoveAgentFromCampaign( user_id, data.camp_id_without_vcc, data.camp_vcc )
		end
	end

	if data.action == "DELETE ALL" then
		remove_all_members()

	elseif data.action == "ADD MEMBERS" then
		if type(data.members ) ~= "table" then
			web.status = 400
			return {result = "INVALID_REQUEST", reason = "Missing or invalid type in field members"}
		end

		remove_all_members()

		for _, user in ipairs(data.members) do
			local user_id = utils.GetId(user.id)
			local role_id = user.role or "agent"
			LogDebug("+[UpdateAgentServiceMembers] agentservice.AddAgentToCampaign %s, %s, %s, %s", user_id, data.camp_id_without_vcc, data.camp_vcc, role_id)
			agentservice.AddAgentToCampaign( user_id, data.camp_id_without_vcc, data.camp_vcc, role_id )
		end

	else
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Unrecognized action: " ..tostring(data.action)}
	end

	return { result = "OK" } 
end

function GetAttentionLevels (web, full_campaign_id)
	LogInfo("Getting attention levels for campaign: %s ", full_campaign_id)
	if not full_campaign_id then
		web.status = 400
		return false, { result = "INVALID_REQUEST", reason = "You must provide a campaign id to get attention levels" }
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( full_campaign_id, keyspace )
	if not camp_exists then
		keyspace:Close()
		return false, { result = "CAMPAIGN_DOES_NOT_EXIST", reason = string.format("Campaign %s does not exist", full_campaign_id) }
	end

	local attention_levels = GetModel("Campaigns").GetAttentionLevels(full_campaign_id, keyspace)
	keyspace:Close()

	--LogDebug("Retrieving attention levels to campaign: %s, attention levels: %s", full_campaign_id, attention_levels)
	return attention_levels
end

-- update attention levels members
function UpdateAttentionLevelsMembers(campaignId, attLevelsToUpdate)

	local web = {
		request = {
			campaignId = campaignId,
			attention_levels = attLevelsToUpdate
		},
		status = 200,
	}

	LogInfo("Administration - UpdateAttentionLevelsMembers")
	--LogInfo("Administration - UpdateAttentionLevelsMembers web '%s'", json.encode(web))
	local response = SaveAttentionLevels(web)

	if response.result ~= "OK" then
		return false, response
	else
		return true , response
	end

end

-- save attention levels
function SaveAttentionLevels(web)

	local active_interactions = false
	local user_vcc = inconcert.authUser.virtualContactCenter
	local user_id = utils.AppendVcc(inconcert.authUser.userName, user_vcc)

	local full_campaign_id = web.request.campaignId
	LogInfo("Administration - User %s saving attention levels configuration for campaign '%s'", 
		user_id,
		full_campaign_id
	)

	if type(full_campaign_id) ~= "string" then
		return returnError(web, {
			reason = "Unable to save %s campaign attention levels. Field campaignId is invalid."
		})
	end

	local posted_attention_levels = web.request.attention_levels
	if type(posted_attention_levels) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to save %s campaign attention levels. Invalid field attention levels type: '%s'.", full_campaign_id, type(posted_attention_levels))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( full_campaign_id, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to save '%s' campaign attention levels. Campaign does not exist.", full_campaign_id)
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", user_id, full_campaign_id, keyspace)

	if not belongs then
		return belResp
	end

	local attention_levels = {}
	for _, attention_level in pairs(posted_attention_levels) do
		local id = attention_level.id
		--si el attention level id no llega en el request, genero uno nuevo
		if not id then
			id = uuid.new()
			attention_level.id = id
		end
		attention_levels[id] = attention_level
	end

	-- valida si los Als han cambiado, si no han cambiado no guarda, pero no es un error
	local old_attention_levels = GetModel("Campaigns").GetAttentionLevels(full_campaign_id, keyspace)
	local operation_audit = nil
	local cont_new_als = 0
	local cont_new_als_users = 0
	local hasChanged = HasChangedStructuralList(
		attention_levels,
		old_attention_levels,
		-- función que realiza un log de los cambios
		function( al_id, current_al, new_al )
			LogInfo("Campaign '%s', al id '%s' attention levels has been changed.", full_campaign_id, al_id)
			--LogInfo("    current attention levels: %s", json.encode( current_al or {} ))
			--LogInfo("    new     attention levels: %s", json.encode( new_al or {} ))

			operation_audit = current_al and "Update"
		end
	)

	if hasChanged then

		-- Validate if removed a problematic user from old_attention_levels
		for item_id, new_item in pairs( attention_levels ) do
			
			local old_item = old_attention_levels[ item_id ]

			if old_item then
				for user_id, old_user in pairs( old_item.users ) do
					local att_user = new_item.users[user_id]
					-- The user was removed from the AL.
					if not att_user then
						-- Check if user have active interactions.
						local totalAssignedInteractions = admUserController.CountAssignedInteractions( user_id, full_campaign_id, keyspace )
						if totalAssignedInteractions > 0 then
							LogInfo("[Agent.SaveAttentionLevels] User: %s have active interactions and cannot be removed from attention list %s", user_id, new_item.name)
							new_item.users[user_id] = old_user
							active_interactions     = true
						end
					end
				end
				cont_new_als_users = cont_new_als_users + (utils.GetRowCount(new_item.users or {}) - utils.GetRowCount(old_item.users or {}))
			else
				cont_new_als_users = cont_new_als_users + utils.GetRowCount(new_item.users or {})
				cont_new_als = cont_new_als + 1
			end
		end
		--valido que el total de als a agregar no exceda el limite permitido
		--valido que la cantidad de usuarios a agregar de los als no exceda el limite permitido
		if cont_new_als > 0 or cont_new_als_users > 0 then
			local success_val_config, error_val_config
			if cont_new_als > 0 then
				success_val_config, error_val_config = modelRestrictions.ValidateConfigurationsLimits(user_vcc, "MaxAttsLevel", keyspace, cont_new_als)
				if not success_val_config then
					return returnError(web, {
						result = error_val_config or "ERROR_ALS_EXCEEDS_LIMIT_ALLOWED",
				   })
				end
			else
				success_val_config, error_val_config = modelRestrictions.ValidateConfigurationsLimits(user_vcc, "MaxAttsLevelUsers", keyspace, cont_new_als_users)
				if not success_val_config then
					return returnError(web, {
						result = error_val_config or "ERROR_ALS_USERS_EXCEEDS_LIMIT_ALLOWED",
				   })
				end
			end 
		end

		local oldAls = GetAuditCampaign(full_campaign_id, keyspace, "AttentionLevels")

		GetModel("Campaigns").SaveAttentionLevels(full_campaign_id, attention_levels, keyspace)

		local newAls = GetAuditCampaign(full_campaign_id, keyspace, "AttentionLevels")
		--Crear Registro para auditoria

		LogInfo("Administration - Audit operation '%s'", operation_audit)
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = user_vcc,
			entity = "Campaign",
			user = inconcert.authUser.userName,
			fieldKey =  utils.GetId(full_campaign_id),
			oldFields = operation_audit and oldAls or nil ,
			newFields = newAls,
			subEntity = "AttentionLevels",
			connection = ksHistorical
		})
		ksHistorical:Close()

		--Actualizo en redis el total de Als y AlsUsers del VCC
		if cont_new_als > 0 or cont_new_als_users ~= 0 then
			modelRestrictions.UpdateRedisConfigTotals(user_vcc, 
			{
				MaxAttsLevel = cont_new_als,
				MaxAttsLevelUsers = cont_new_als_users
			} , keyspace)
		end

		agentNotifications.NotifyAgents({
			eventType = "administrator_attention_levels_change",
			agentsLevel = "CAMPAIGN_USERS",
			pollerMessage = {
				campaignId       = full_campaign_id,
				attention_levels = GetModel("Campaigns").GetAttentionLevels(full_campaign_id, keyspace)
			}
		}, keyspace)
		LogInfo("+Campaign %s, attention levels successfully saved.", full_campaign_id)

		--notifications
		for id, attLevel in pairs(attention_levels) do
			notificationsEngine.SendNotification(not old_attention_levels[attLevel.id] and "AttLevelCreated" or "AttLevelUpdated", 0, 
				{ campaignId = utils.GetId(full_campaign_id), attentionLevel = attLevel })
		end
	else
		LogInfo("-Campaign %s, attention levels has not changed.", full_campaign_id)
	end

	keyspace:Close()

	if active_interactions then
		return returnError(web, {
		 	result = "PENDING_INT",
		 	reason = "Some users cannot be removed from attention levels because have pending interactions."
		})
	else
		return {result = "OK"}
	end
end

function SaveAllAttentionLevels (web)

	local user_vcc = inconcert.authUser.virtualContactCenter

	if user_vcc:lower() ~= "system" then
		return returnError(web, {
			status = 403,
			result = "FORBIDDEN",
			reason = string.format("Unable to delete %s campaign attention levels. You do not have permission.", web.request.campaignId)
		})
	end
	--LogDebug("[SaveAllAttentionLevels] web.request.attention_levels: %s", json.encode(web.request.attention_levels))
	return SaveAttentionLevels(web)
end

function DeleteAttentionLevels (web)

	local user_vcc = inconcert.authUser.virtualContactCenter
	local user_id = utils.AppendVcc(inconcert.authUser.userName, user_vcc)

	local full_campaign_id = web.request.campaignId
	LogInfo("Administration - User %s removing attention levels configuration on campaign '%s'", user_id, full_campaign_id)

	if type(full_campaign_id) ~= "string" then
		return returnError(web, {
			reason = "Unable to delete %s campaign attention levels. Field campaignId is invalid."
		})
	end

	local posted_attention_levels = web.request.attention_levels
	if type(posted_attention_levels) ~= "table" then
		return returnError(web, {
			reason = string.format("Unable to delete %s campaign attention levels. Invalid field attention levels type: '%s'.", full_campaign_id, type(posted_attention_levels))
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( full_campaign_id, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to delete %s campaign attention levels. Invalid field attention levels type: '%s'.", full_campaign_id, type(web.request.attention_levels))
		})
	end

	local belongs, belResp = ValidateBelongsCampaign (web, "modify", user_id, full_campaign_id, keyspace)
	if not belongs then
		return belResp
	end

	local result = CanDeleteAttentionLevels(full_campaign_id, posted_attention_levels, keyspace)
	if not result.success then
		keyspace:Close()
		result.reason = string.format("Unable to delete attention levels in '%s' campaign. Error type: '%s'.", full_campaign_id, json.encode(result))
		return returnError(web, result)
	end

	local als = GetAuditCampaign(full_campaign_id, keyspace, "AttentionLevels")
	GetModel("Campaigns").DeleteAttentionLevels(full_campaign_id, posted_attention_levels, keyspace)
	
	local als_to_delete = {}
	local cont_als_to_remove = 0
	local cont_als_users_to_remove = 0

	for _, value in ipairs(posted_attention_levels) do
		local al_info =  als[value]
		if  al_info then
			als_to_delete[value] = al_info
			cont_als_to_remove = cont_als_to_remove + 1
			cont_als_users_to_remove = cont_als_users_to_remove + utils.GetRowCount(al_info.users or {})
		end
	end

	--Se actualizan totales en redis de los Als y Als/Users
	modelRestrictions.UpdateRedisConfigTotals(user_vcc, 
			{
				MaxAttsLevel = -cont_als_to_remove,
				MaxAttsLevelUsers = -cont_als_users_to_remove
			} , keyspace)

	--Crear Registro para auditoria
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	audits.Create({
		vcc = user_vcc,
		entity = "Campaign",
		user = inconcert.authUser.userName,
		fieldKey =  utils.GetId(full_campaign_id),
		oldFields = als_to_delete,
		subEntity = "AttentionLevels",
		connection = ksHistorical
	})
	ksHistorical:Close()
	keyspace:Close()

	LogInfo("Campaign %s, attention levels successfully deleted", full_campaign_id)

	-- notifico de los cambios a los members logeados
	agentNotifications.NotifyAgents({
		eventType = "administrator_attention_levels_change",
		agentsLevel = "CAMPAIGN_USERS",
		pollerMessage = {
			campaignId       = full_campaign_id,
			attention_levels = GetModel("Campaigns").GetAttentionLevels(full_campaign_id, keyspace)
		}
	}, keyspace)

	--notifications
	for _, attLevelId in pairs(posted_attention_levels) do
		notificationsEngine.SendNotification("AttLevelDeleted", 0, { campaignId = utils.GetId(full_campaign_id), attentionLevelId = attLevelId })
	end

	return {result = "OK"}
end

function DeleteAllAttentionLevels (web)
	local user_vcc = inconcert.authUser.virtualContactCenter
	local user_id = utils.AppendVcc(inconcert.authUser.userName, user_vcc)
	local full_campaign_id = web.request.campaignId

	LogInfo("Administration - User %s removing all attention levels on campaign '%s'.", user_id, full_campaign_id)

	if user_vcc:lower() ~= "system" then
		return returnError(web, {
			status = 403,
			result = "FORBIDDEN",
			reason = string.format("Unable to delete %s campaign attention levels. You do not have permission.", full_campaign_id)
		})
	end

	if type(full_campaign_id) ~= "string" then
		return returnError(web, {
			reason = "Unable to delete %s campaign attention levels. Field campaignId is invalid."
		})
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	-- Check if campaign exists
	local camp_exists = GetModel("Campaigns").CampaignExists( full_campaign_id, keyspace )
	if not camp_exists then
		keyspace:Close()
		return returnError(web, {
			result = "CAMPAIGN_DOES_NOT_EXIST",
			reason = string.format("Unable to delete %s campaign attention levels. Invalid field attention levels type: '%s'.", full_campaign_id, type(web.request.attention_levels))
		})
	end

	GetModel("Campaigns").DeleteAttentionLevels(full_campaign_id, nil, keyspace)
	keyspace:Close()

	LogInfo("Campaign %s, all attention levels successfully deleted", full_campaign_id)

	return {result = "OK"}
end

function StartWhatsappAuthorization (web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = "Unable to authorize. Not has licensed for the Whatsapp channel."
		LogFatal("[ADMINISTRATION.CAMPAIGNS.StartWhatsappAuthorization] %s", reason)
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local data = web.request.data or {}
	LogInfo("[ADMINISTRATION.CAMPAIGNS.StartWhatsappAuthorization] User %s on vcc %s authorizating whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		accountId,
		campaignId
	)

	GetModel("Campaigns").StartWhatsappAuthorization(accountId)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappAuthenticate({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		data = data
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("[ADMINISTRATION.CAMPAIGNS.StartWhatsappAuthorization] Error authorizating whatsapp account: '%s' in campaign '%s' -> '%s'", accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason }
	end
	return { result = "OK" }
end

function StopWhatsappAuthorization (web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to authorize. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	LogInfo("Administration - User %s on vcc %s ending authorization for whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		accountId,
		campaignId
	)

	GetModel("Campaigns").StopWhatsappAuthorization(accountId)
	return { result = "OK" }
end

function SetWhatsappAuthorizationCode (web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to authorize. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local code = web.request.code
	LogInfo("Administration - User %s on vcc %s setting authorization code for whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		accountId,
		campaignId
	)

	GetModel("Campaigns").SetWhatsappAuthorizationCode(accountId, code)
	return { result = "OK" }
end

function GetWhatsappAuthorizationStatus (web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to authorize. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local accountId = web.request.accountId
	LogInfo("Administration - User %s on vcc %s getting whatsapp qr code for account '%s'", 
		userId, 
		vcc,
		accountId
	)

	local status = GetModel("Campaigns").GetWhatsappAuthorizationStatus(accountId)
	return {
		result = "OK",
		data = status
	}
end

function WhatsappListGroups(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappListGroups] Retrieving groups for user %s@%s , By campaign: %s account: %s",
		userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappListGroups({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		cache = false
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappGetGroup(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappListGroups] Retrieving group %s for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappGetGroup({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
		cache = false
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappCreateGroup(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappCreateGroup] Creating group %s for user %s@%s , By campaign: %s account: %s",
		json.encode(request.group), userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappCreateGroup({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		group = request.group,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappUpdateGroup(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappUpdateGroup] Updating group %s for user %s@%s , By campaign: %s account: %s",
		json.encode(request.group), userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappUpdateGroup({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		group = request.group,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappCreateGroupInviteLink(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappCreateGroupInviteLink] Creating group %s invite link for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappCreateGroupInviteLink({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappDeleteGroupInviteLink(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappDeleteGroupInviteLink] Deleting group %s invite link for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappDeleteGroupInviteLink({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappRemoveGroupParticipants(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappRemoveGroupParticipants] Removing group %s participants %s for user %s@%s , By campaign: %s account: %s",
		request.groupId, json.encode(request.participants), userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappRemoveGroupParticipants({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
		participants = request.participants,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappLeaveGroup(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappLeaveGroup] Leaving group %s for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappLeaveGroup({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappSetGroupIcon(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappSetGroupIcon] Setting group %s icon for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappSetGroupIcon({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
		contentType = request.contentType,
		content = request.content,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappGetGroupIcon(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappGetGroupIcon] Getting group %s icon for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappGetGroupIcon({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappDeleteGroupIcon(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappDeleteGroupIcon] Deleting group %s icon for user %s@%s , By campaign: %s account: %s",
		request.groupId, userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappDeleteGroupIcon({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappAddGroupAdmins(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappAddGroupAdmins] Adding group %s admins %s for user %s@%s , By campaign: %s account: %s",
		request.groupId, json.encode(request.admins), userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappAddGroupAdmins({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
		admins = request.admins,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end

function WhatsappRemoveGroupAdmins(web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = inconcert.authUser.userName
	local response, request = pcall(json.decode, web.input["post_data"])
	if not response or not request then
		web.status = 400
		return "INVALID_REQUEST"
	end

	LogInfo("[WhatsappRemoveGroupAdmins] Removing group %s admins %s for user %s@%s , By campaign: %s account: %s",
		request.groupId, json.encode(request.admins), userName, vcc, request.campaignAtVCC, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:WhatsappRemoveGroupAdmins({
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		groupId = request.groupId,
		admins = request.admins,
	})
	web.status = http_mapping.getHTTPCode(result.result) or 500
	return json.encode(result)
end


local function auditFieldsByEntity(entity)

	--[[
		key = true|false| { alias = "key_alias"}
	
	]]--

	local entitiesFields = {
		Scheduler = {
			scheduleSpecialDays = true,
			scheduleDays = true,
			scheduleSpecialDayAllYearss = true
		},
		DispositionCodes = {
			isGoal = true,
			reopen = true,
			name = true,
			childs = true,
			finalizationMessage = true,
			needReschedule = true,
			contactCode = true,
			isUseFul = true,
			code = true,
		},
		Campaign = {
			Id = true,
			Name = true, 
			Description = true,
			ValidationTimeZone = true,
			LastUpdate = false,
			updated_at = false,
			permissions = false,
			dispositionCodes = false,
			timezoneOffset = false,
			AlwaysValid = false,
			defaultAgentCriteria = false,
			Integration = false,
			creationDate = false,
			VCC  = false
		},
		Thresholds = {
			ACDFirstMessageThreshold = true,
        	Ghost = true,
        	ACDWaitThreshold = true,
        	AnswerTimeout = true,
        	InactivityTimeout = true,
        	ServiceLevel = true,
        	AutoWrapupTimeout = true,
        	ACDRepeatedMessageThreshold = true,
        	ShortInteraction = true,
			ACDRepeatedMessageThreshold_INBOUND = true,
			ServiceLevel_INBOUND = true,
			ShortInteraction_INBOUND = true,
			ShortInteraction_OUTBOUND = true,
			ServiceLevel_OUTBOUND = true,
			Ghost_INBOUND = true,
			LongInteraction_OUTBOUND = true,
			ACDWaitThreshold_INBOUND = true,
			InactivityTimeout_INBOUND = true,
			LongInteraction_INBOUND = true,
			ACDFirstMessageThreshold_INBOUND = true,
			AverageCallDuration_OUTBOUND = true,

		},
		Members = {
			Permissions = true,
			SupervisorWorkgroups = true,
			Capability = true
		},
		Rules = {
			direction = true,
        	filters_matches = true,
        	filters_limit = true,
        	actions_labels = true,
        	stop_processing = true,
        	actions = true,
        	filters = true,
        	post = true,
        	id = true,
        	page = true,
        	batch = true,
        	name = true,
        	channel = true,
        	disabled = true,
        	description = true,
        	interaction_type = true,
        	account = true,
		},
		AttentionLevels = {
			id = true,
			users = true,
			name = true,
			description = true,
		},
		Integrations = {
			application = true,
			popup = true,
			trigger = true,
			override_actions = true,
			type = true,
			othersIntegrations = true
		},
		Templates  ={
			message = true,
			format = true,
			plain_message = true,
			group = true
		},
		Labels = {
			id = true,
			name = true,
			system = true
		},
		Notifications = {
			Apps = true
		},
		Channels = {
			attentionCapability = true
		},
		Accounts = {
			CHAT = {
        		name = true,
        		CCAlgorithm = true,
        		language = true,
       			Priority = true,
        		checkActiveAgents = true,
        		GUI = true,
        		TweetyAppName = true,
        		window_type = true,
        		start_button = true,
        		snippet_parameters = true,
        		AttentionLevels = true,
        		google_key = true,
        		google_bySections = true,
        		google_enabled = true,
        		GoChatIfTransfer = true,
        		AnnounceEntry = true,
        		FlowInformation = true,
        		Messages = true,
        		Tracking = true,
        		state = true,
        		triggers = true,
        		TicketEmitter = true,
        		ContactManagement = true,
        		outHours_enabled = true,
        		outHours_account = true,
        		outHours_campaign = true,
        		Skills = true,
        		ScreenRecording = true,
        		containerId = true,
        		CampaignBotName = true,
			},
			CALL = {
        		name = true,
        		language = true,
        		Skills = true,
        		Priority = true,
        		packages = true,
        		Messages = true,
        		FlowInformation = true,
        		ScreenRecording = true,
        		state = true,
        		optimizeNodeDistribution = true,
        		GUI = true,
        		webcallConfig = true,
        		outgoingConfig = true,
        		AttentionLevels = true,
        		incomingConfig = true,
        		ContactManagement = true,
        		CCAlgorithm = true,
        		TicketEmitter = true,
        		BotsToTransferFromAgent = true,
        		ttsDefaultVoice = true,
        		LifeTimeAppointmentCalls = true,
        		CallbackIsAllowed = true,
        		LifeTimeAbandonedCalls = true,
        		AfterInteractionWork = true,
			},
			WHATSAPP = {
        		AttentionLevels = true,
        		HSMConfig = true,
        		CCAlgorithm = true,
        		FlowInformation = true,
        		integrationType = true,
        		Priority = true,
        		ContactManagement = true,
        		Messages = true,
        		language = true,
        		name = true,
        		phoneNumber = true,
        		Skills = true,
        		Survey = true,
        		state = true,
        		GUI = true,
        		TicketEmitter = true,
        		userName = true,
        		userToken = true,
        		baseUrl = true,
        		newContacts = true,
			},
			FACEBOOK = {
				AttentionLevels = true,
				Messages = true,
				userId = true,
				CCAlgorithm = true,
				language = true,
				Priority = true,
				ContactManagement = true,
				state = true,
				pages = true,
				pagesNames = true,
				TicketEmitter = true,
				Survey = true,
				name = true,
				pagesMessenger = true,
				Skills = true,
				userName = true,
				GUI = true,
			},
			MAIL = {
				CCAlgorithm = true,
				incomingConfig = true,
				FlowInformation = true,
				language = true,
				downloadEmailsOutOfHour = true,
				Priority = true,
				outgoingConfig = true,
				Skills = true,
				TicketEmitter = true,
				name = true,
				ContactManagement = true,
				state = true,
				GUI = true,
				Survey = true,
				outgoingBCC = true,
				AttentionLevels = true,
				Messages = true,
				Signature = true,
				enableSpamFilter = true,
				groupInThreads = true,
				type = true,
				oauth = true,
				defaultName = true,
			},
			YOUTUBE = {
				CCAlgorithm = true,
				AttentionLevels = true,
				Priority = true,
				ContactManagement = true,
				TicketEmitter = true,
				language = true,
				state = true,
				name = true,
				Skills = true,
				screen_name = true,
				GUI = true,
			},
			SMS = {
				name = true,
				CCAlgorithm = true,
				AttentionLevels = true,
				FlowInformation = true,
				Priority = true,
				ContactManagement = true,
				sendMessageUrl = true,
				state = true,
				language = true,
				Skills = true,
				GUI = true,
				TicketEmitter = true,
			},
			TWITTER = {
				AttentionLevels = true,
				Messages = true,
				CCAlgorithm = true,
				FlowInformation = true,
				ContactManagement = true,
				GUI = true,
				state = true,
				name = true,
				Priority = true,
				TicketEmitter = true,
				language = true,
				Skills = true,
				Survey = true,
			},
			INSTAGRAM = {
				AttentionLevels = true,
				Messages = true,
				userId = true,
				CCAlgorithm = true,
				language = true,
				Priority = true,
				ContactManagement = true,
				state = true,
				pages = true,
				pagesNames = true,
				TicketEmitter = true,
				Survey = true,
				name = true,
				pagesMessenger = true,
				Skills = true,
				userName = true,
				GUI = true,
				pagesFbIds = true,
			},
			WEBCONTACT = {
				CCAlgorithm = true,
				AttentionLevels = true,
				Priority = true,
				ContactManagement = true,
				TicketEmitter = true,
				language = true,
				state = true,
				name = true,
				Skills = true,
				GUI = true,
				containerId = true,
				PreferredAccounts = true,
				TweetyAppName = true,
				AlwaysEnabled = true,
				snippet_parameters = true,
			},
			APPSTORE = {
				GUI = true,
				Skills = true,
				name = true,
				language = true,
				CCAlgorithm = true,
				TicketEmitter = true,
				state = true,
				ContactManagement = true,
				screen_name = true,
				countries = true,
				AttentionLevels = true,
				Priority = true,
				FlowInformation = true
			},
			GOOGLEPLAY = {
				GUI = true,
				Skills = true,
				name = true,
				language = true,
				CCAlgorithm = true,
				TicketEmitter = true,
				state = true,
				ContactManagement = true,
				screen_name = true,
				countries = true,
				AttentionLevels = true,
				Priority = true,
				FlowInformation = true
			}
		}
	}
	return entitiesFields[entity]
end

--Metodo q dado un objeto, dependiente de una entidad, filtra y devuelve solo los elementos auditables.
-- En caso de que precise alias, se realiza el cambio de nombre de key
function FilterAuditFields (entityData, entity, proxyType)
	local auditFields = auditFieldsByEntity(entity)

	if not auditFields or not entityData then
		return nil
	end

	if entity == "Accounts" then
		auditFields = auditFields[proxyType or ""]
		if not auditFields then
			return entityData
		end
	end

	local result = {}
	for key, value in pairs(entityData) do
		local auditField = auditFields[key]
		if auditField and type(auditField) == "table" then
			result[auditField.alias or key] = value
		elseif auditField then
			result[key] = value
		end
	end

	return result
end

-- Metodo que filtra los valores de objetos en un array
--[[
	--- attentions levels
	{
		123rfds4533434 = { --values
			id = 123rfds4533434,
			name = "AL_1"
		}
	}

	FIltra y arma una nueva tabla con las keys y objetos con los valores ya filtrados
]]--

local function FilterAuditFieldValues (fieldValues, entity)
	local array = nil
	for key, value in pairs(fieldValues) do
		local new = FilterAuditFields(value, entity)
		if  new and not utils.TableIsEmpty(new) then
			if not array then
				array = {}
			end
			array[key] = new
		end
	end	
	return array
end

--Account Audit Data
function GetAuditAccount (accountId, campaignIdWithVCC, keyspace)
	local has_close_keyspace = false

	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		has_close_keyspace = true
	end

	local account = type(accountId) == "table" and accountId or agentCampaignModel.GetAccount(accountId, campaignIdWithVCC, keyspace)
	
	if not account then
		if has_close_keyspace then
			keyspace:Close()
		end
		return
	end

	account.Messages = GetModel("Campaigns").GetAccountMessages(campaignIdWithVCC,account.id, keyspace)
	if has_close_keyspace then
		keyspace:Close()
	end
	return FilterAuditFields(account, "Accounts", account.proxyType) or account
end

--Members Audit Data
-- members una table de miembros
function GetAuditMembers (members, campaignIdWithVCC, keyspace, cachedInteractionTypes)
	local vcc = inconcert.authUser.virtualContactCenter
	local has_close_keyspace = false
	local simple_campaign = utils.GetId(campaignIdWithVCC)
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		has_close_keyspace = true
	end

	local startSubProcessTime = os.time()
	local permissions = GetModel("Campaigns").GetSpecificColumn( campaignIdWithVCC, "permissions", keyspace )
	permissions = permissions and json.decode(permissions)
	local membersPermissions = permissions and permissions.members or {}
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:GetAuditMembers]    ...END sub process GetSpecificColumn 'permissions'. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

	--startSubProcessTime = os.time()
	local membersByCamp = GetModel("Campaigns").GetCampaignMembers( vcc, simple_campaign, keyspace ) or {}
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:GetAuditMembers]    ...END sub process GetCampaignMembers. Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))
	local auditMembersData = nil

	startSubProcessTime = os.time()
	for _key, _ in pairs(members or {}) do
		local permissions = membersPermissions[_key]
		local user = GetModel("Users").GetUser(vcc, _key, keyspace, {"SupervisorWorkgroups"}) or {}
		local workgroups = user and user.SupervisorWorkgroups and user.SupervisorWorkgroups[simple_campaign]
		local capability
		capability, cachedInteractionTypes = channelsController.GetCapabilityByCampaignAndMember(campaignIdWithVCC, utils.AppendVcc(utils.GetId(_key), vcc), cachedInteractionTypes, keyspace)
		
		if not auditMembersData then
			auditMembersData = {}
		end

		if membersByCamp[utils.AppendVcc(utils.GetId(_key), vcc)] then
			auditMembersData[_key] = FilterAuditFields({
				Permissions = permissions or {},
				SupervisorWorkgroups = workgroups or {},
				Capability = capability or {}
			}, "Members")
		end
	end
	LogDebug("[ADMINISTRATION.CAMPAIGNS:GetAuditMembers]    ...END sub process LOOP members . Elapsed time %s seconds.", utils.GetElapsedTimeSec(startSubProcessTime))

	if has_close_keyspace then
		keyspace:Close()
	end
	return auditMembersData, cachedInteractionTypes
end

--[[
	*Devuelve los datos para auditar por sub entidad:
		- Scheduler, Permissions, DispositionCode, DispositionCodes, AttentionLevels, etc
	*Para los datos basico de la Campaign, no se pasa subEntity por parametro
]]
function GetAuditCampaign (campaignIdWithVCC, keyspace, subEntity)
	local vcc = inconcert.authUser.virtualContactCenter
	local campaign_id = utils.GetId(campaignIdWithVCC)
	local subEntities = {
		Scheduler       = function() 
			return FilterAuditFields(agentCampaignModel.GetCampaignScheduler(campaignIdWithVCC, keyspace), "Scheduler")
		end,
		AttentionLevels = function() 
			local als = agentCampaignModel.GetAttentionLevels(campaignIdWithVCC, keyspace)
			return als and FilterAuditFieldValues(als,"AttentionLevels" ) or nil
		end,
		--Labels			= function() return agentCampaignModel.GetCampaignLabels(campaignIdWithVCC, keyspace) end,
		DispositionCodes = function() 
			local dispositionCodes = GetModel("Campaigns").GetSpecificColumn( campaignIdWithVCC, "dispositionCodes", keyspace )
			local decodeDispositionCodes = dispositionCodes and json.decode(dispositionCodes)
			--Convierto el arreglo de DC a json para comparar.
			local dc = decodeDispositionCodes and not utils.TableIsEmpty(decodeDispositionCodes) and utils.ArrayToHash(decodeDispositionCodes, "name") or {}
		
			local arrayDcs = FilterAuditFieldValues(dc,"DispositionCodes" )
			return arrayDcs
		end,
		Permissions = function()
			local permissions = GetModel("Campaigns").GetSpecificColumn( campaignIdWithVCC, "permissions", keyspace )
			local groups = permissions and json.decode(permissions).groups
			return groups and not utils.TableIsEmpty(groups) and groups or nil
		end,
		Channels = function ()
			local capabilitties =  channelsController.GetCapabilityByCampaignAndMember(campaignIdWithVCC, nil, nil,  keyspace)
			return capabilitties and FilterAuditFieldValues(capabilitties,"Channels" ) or nil
		end,
		Thresholds = function()
			local thresholds = GetThresholds(campaignIdWithVCC, keyspace)
			return thresholds and FilterAuditFieldValues(thresholds,"Thresholds" ) or nil
		end,
		Integrations = function ()
			local currentIntegrations = utils.JSON_Decode(GetModel("Campaigns").GetSpecificColumn(campaignIdWithVCC, "Integration", keyspace))
			return currentIntegrations and FilterAuditFields(currentIntegrations,"Integrations" ) or nil
		end,
		Notifications = function ()
			local currentNotifications = utils.JSON_Decode(GetModel("Campaigns").GetSpecificColumn(campaignIdWithVCC, "InteractionNotifications", keyspace))
			return currentNotifications and FilterAuditFields(currentNotifications,"Notifications" ) or nil
		end
	}
	if subEntity then
		return (subEntities[subEntity])()
	end
	
	-- Basic Data
	local campaign = GetModel("Campaigns").GetCampaign(vcc, campaign_id, keyspace, {})
	return FilterAuditFields(campaign, "Campaign")
end

function WhatsappGetPhoneStatus(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to get phone status. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local phoneId = web.request.phoneId

	LogInfo("Administration - User %s on vcc %s retrieving phoneId '%s' status on whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		phoneId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappGetPhoneStatus({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		phoneId = phoneId
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error retrieving phoneId '%s' status on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			phoneId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason }
	end
	return { result = "OK", status = res.status, messaging_limit = res.messaging_limit, code_verification_status = res.code_verification_status }
end

function WhatsappRegisterPhone(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to register phone. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local phoneId = web.request.phoneId
	local pin = web.request.pin

	LogInfo("Administration - User %s on vcc %s registering phoneId '%s' on whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		phoneId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappRegisterPhone({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		phoneId = phoneId,
		pin = pin
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error registering phoneId '%s' on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			phoneId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason }
	end
	return { result = "OK" }
end

function WhatsappDeregisterPhone(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to deregister phone. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local phoneId = web.request.phoneId

	LogInfo("Administration - User %s on vcc %s deregistering phoneId '%s' on whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		phoneId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappDeregisterPhone({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		phoneId = phoneId
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error deregistering phoneId '%s' on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			phoneId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason }
	end
	return { result = "OK" }
end

function WhatsappRequestCode(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to request phone code. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local phoneId = web.request.phoneId
	local method = web.request.method
	local language = web.request.language

	LogInfo("Administration - User %s on vcc %s requesting code for phoneId '%s' on whatsapp account '%s' in campaign '%s' using method '%s' and language '%s'", 
		userId, 
		vcc,
		phoneId,
		accountId,
		campaignId,
		method,
		language
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappRequestCode({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		phoneId = phoneId,
		method = method,
		language = language
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error requesting code for phoneId '%s' on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			phoneId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason, reason_detail = res.reason_detail }
	end
	return { result = "OK" }
end

function WhatsappVerifyCode(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to verify phone code. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local phoneId = web.request.phoneId
	local pin = web.request.pin

	LogInfo("Administration - User %s on vcc %s verifying code for phoneId '%s' on whatsapp account '%s' in campaign '%s' using pin '%s'", 
		userId, 
		vcc,
		phoneId,
		accountId,
		campaignId,
		pin
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappVerifyCode({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		phoneId = phoneId,
		pin = pin
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error verifying code for phoneId '%s' on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			phoneId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason, reason_detail = res.reason_detail }
	end
	return { result = "OK" }
end

function WhatsappMigratePhone(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to migrate phone. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local fromWaba = web.request.fromWaba
	local toWaba = web.request.toWaba
	local fullPhoneNumber = web.request.fullPhoneNumber

	LogInfo("Administration - User %s on vcc %s migrating phone '%s' on whatsapp account '%s' in campaign '%s' from waba %s to wabaId %s", 
		userId, 
		vcc,
		fullPhoneNumber,
		accountId,
		campaignId,
		fromWaba,
		toWaba
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappMigratePhone({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		fromWaba = fromWaba,
		toWaba = toWaba,
		fullPhoneNumber = fullPhoneNumber
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error migrating phone '%s' on whatsapp account: '%s' in campaign '%s' from waba %s to wabaId %s -> '%s'", 
		fullPhoneNumber, accountId, campaignIdAtVCC, fromWaba, toWaba, res.reason)
		return { result = res.result, reason = res.reason }
	end
	
	return res
end

function WhatsappLockActivationCode(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to lock activation code. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local code = web.request.code

	LogInfo("Administration - User %s on vcc %s locking activation code '%s' on whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		code,
		accountId,
		campaignId
	)

	local code_is_valid = agentCampaignModel.GetWhatsappInconcertCloudApiCode(code)
	if not code_is_valid then
		local reason = LogError("Activation Code is invalid.")
		web.status = 500
		return {
			result = "INVALID",
			reason = reason
		}
	end

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappLockActivationCode({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		code = code
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error locking activation code '%s' on whatsapp account: '%s' in campaign '%s'", 
			code, accountId, campaignIdAtVCC)
		return { result = res.result, reason = res.reason }
	end
	
	return res
end

function WhatsappUnlockActivationCode(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogError("Unable to unlock activation code. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local code = web.request.code

	LogInfo("Administration - User %s on vcc %s unlocking activation code '%s' on whatsapp account '%s' in campaign '%s'", 
		userId, 
		vcc,
		code,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappUnlockActivationCode({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		code = code
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error unlocking activation code '%s' on whatsapp account: '%s' in campaign '%s'", 
			code, accountId, campaignIdAtVCC)
		return { result = res.result, reason = res.reason }
	end
	
	return res
end

function WhatsappGetMessageTemplates(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogError("Unable to get message templates. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local wabaId = web.request.wabaId

	LogInfo("Administration - User %s on vcc %s retrieving message templates on whatsapp account '%s' in campaign '%s' and waba '%s'", 
		userId, 
		vcc,
		accountId,
		campaignId,
		wabaId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappGetMessageTemplates({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		wabaId = wabaId
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error retrieving message templates on whatsapp account: '%s' in campaign '%s' and waba '%s'", 
			accountId, campaignIdAtVCC, wabaId)
		return { result = res.result, reason = res.reason }
	end
	
	return res
end

function WhatsappDelMessageTemplates(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogError("Unable to del message template. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local wabaId = web.request.wabaId
	local templateName = web.request.templateName
	local mustSaveAccount = web.request.mustSaveAccount or true

	LogInfo("Administration - User %s on vcc %s deleting message template '%s' on whatsapp account '%s' in campaign '%s' and waba '%s'", 
		userId,
		vcc,
		templateName,
		accountId,
		campaignId,
		wabaId
	)

	local ok, info = GetSystemConfiguration("INSTALLATION_ID")
	if not ok then
		LogError("Error on GetSystemConfiguration: '%s'", installationId)
		return { result = "FAIL", reason = "Error deleting whatsapp template" }
	end

	local account = GetModel("Campaigns").GetCampaignAccount(campaignId, accountId, nil, { "HSMConfig" })
	if not account then
		LogError("WhatsappDelMessageTemplates %s - account not found", accountId)
		return { result = "FAIL", reason = "account not found" }
	end

	--shadow copy to use in RemoveHsmRepositoryFilesPublicURL call
	local oldHSMConfig = json.decode(json.encode(account.HSMConfig))
	
	local messages = account.HSMConfig and account.HSMConfig.TemplateMessages or {}

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappDelMessageTemplates({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		wabaId = wabaId,
		templateName = templateName,
		installationId = info.ID
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error deleting message template '%s' on whatsapp account: '%s' in campaign '%s' and waba '%s': %s", 
			templateId, accountId, campaignIdAtVCC, wabaId, res.reason)
		return { result = res.result, reason = res.reason }
	end

	-- Eliminar el template de HSMConfig.TemplateMessages
	messages[templateName] = nil
	-- Actualizar account.HSMConfig 
	account.HSMConfig.TemplateMessages = messages

	--Si estamos eliminando la account o la campaign, no guardamos
	if mustSaveAccount then
		-- Guardar la account
		local success, errInfo = GetModel("Campaigns").SaveAccount(campaignId, account, false, nil)
		if not success then
			LogError("WhatsappDelMessageTemplates Unable to save campaign account %s", accountId)
			return { result = success, reason = errInfo }
		end
	end

	RemoveHsmRepositoryFilesPublicURL(campaignId, accountId, oldHSMConfig, account.HSMConfig or {})

	return res
end

local function getPlaceholders(text)
	local firstBadPlaceholder = ""
	local placeholders = {}
	for w in string.gmatch(text, "{{%d}}") do
		table.insert(placeholders, string.sub(w, string.find(w, "%d")))
	end

	for _, placeholder in pairs(placeholders) do
		if tostring(_) ~= placeholder then
			firstBadPlaceholder = placeholder
			return firstBadPlaceholder, 0
		end
	end
	return firstBadPlaceholder, #placeholders
end

function WhatsappSetMessageTemplates(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogError("Unable to set message templates. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local wabaId = web.request.wabaId
	local data = web.request.data

	LogInfo("Administration - User %s on vcc %s setting message templates on whatsapp account '%s' in campaign '%s' and waba '%s'", 
		userId, 
		vcc,
		accountId,
		campaignId,
		wabaId
	)

	assert(data and data.languages and type(data.languages) == "table")

	local account = GetModel("Campaigns").GetCampaignAccount(campaignId, accountId, nil, { "HSMConfig" })
	if not account then
		LogError("WhatsappSetMessageTemplates %s - account not found", accountId)
		return { result = "FAIL", reason = "account not found" }
	end

	local bAddUlrParam = false

	--shadow copy to use in RemoveHsmRepositoryFilesPublicURL call
	local oldHSMConfig = json.decode(json.encode(account.HSMConfig))

	local messages = account.HSMConfig and account.HSMConfig.TemplateMessages or {}

	local templates = {}

	-- languages validation
	local success, response, errInfo
	for lang_code, language in pairs(data.languages) do
		-- shadow copy needed here
		--#18604: Agregado para que no se rompan algunos emojis que pueden incluirse en los templates de whatsapp
		local template = cjson.decode(cjson.encode(language))
		-- This occ language family. One of: "es", "en", "fr", "pt"
		template.code = lang_code
		template.text = nil
		for component_number, component in pairs(language.components) do
			if component.type == "HEADER" and component.format == "TEXT" then
				local firstBadPlaceholder, totalPlaceHolders = getPlaceholders(component.text)
				if firstBadPlaceholder ~= "" or totalPlaceHolders > 1 then
					LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s': Invalid placeholders on header text for language '%s'", 
						accountId, campaignId, wabaId, lang_code)
					return { result = "FAIL", reason = "Only one placeholder is allowed on header", error_data = { lang_code = lang_code, error_component = "HEADER", error_component_field = "header_text"  } }
				elseif totalPlaceHolders > 0 and not component.header_param then
					LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s': Invalid header parameter for language '%s'", 
						accountId, campaignId, wabaId, lang_code)
					return { result = "FAIL", reason = "Header parameter is required.", error_data = { lang_code = lang_code, error_component = "HEADER", error_component_field = "example_header_text" } }
				end
			elseif component.type == "BODY" then
				language.text = component.text
				language.params = json.decode(json.encode(template.components[component_number].params))

				local firstBadPlaceholder, totalPlaceHolders = getPlaceholders(language.text)
				if firstBadPlaceholder ~= "" then
					LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s': Invalid placeholders on body text for language '%s'", 
						accountId, campaignId, wabaId, lang_code)
					return { result = "FAIL", reason = "Text placeholders must be consecutive numbers starting at one", error_data = { lang_code = lang_code, error_component = "BODY", error_component_field = "message"  } }
				else
					if totalPlaceHolders ~= (language.params and #language.params or 0) then
						LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s': Invalid body parameters for language '%s'", 
							accountId, campaignId, wabaId, lang_code)
						return { result = "FAIL", reason = "Invalid text parameters", error_data = { lang_code = lang_code, error_component = "BODY", error_component_field = "params" } }
					end
				end				
			elseif component.type == "BUTTONS" then
				for button_number, button in pairs(component.buttons) do
					if button.type == "PHONE_NUMBER" then
						ExecuteFullcontactMethod(function()
							success = Fullcontact.Dialer:ValidatePhone(button.phone_number, nil, nil, vcc)
						end)
						if not success then
							LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s': Invalid phone number '%s' in button %s and language '%s'", 
								accountId, campaignId, wabaId, button.phone_number, button_number, lang_code)
							return { result = "FAIL", reason = "Invalid phone number", error_data = { lang_code = lang_code, error_component = "BUTTONS", button_type = button.type, button_value = button.phone_number } }
						end
					elseif button.type == "URL" and string.find(button.url, "{{1}}") then
						bAddUlrParam = true
					end
				end
			end
		end
		-- not needed data in occ
		language.name = nil
		language.language = nil
		--data to send to proxy
		table.insert(templates, template)
	end

	local ok, info = GetSystemConfiguration("INSTALLATION_ID")
	if not ok then
		LogError("Error on GetSystemConfiguration: '%s'", installationId)
		return { result = "FAIL", reason = "set_language_template_error" }
	end

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappSetMessageTemplates({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		installationId = info.ID,
		wabaId = wabaId,
		data = templates
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	-- General error
	if res.result ~= "OK" then
		LogError("Error setting message templates on whatsapp account: '%s' in campaign '%s' and waba '%s'", 
			accountId, campaignIdAtVCC, wabaId)
		return { result = res.result, reason = res.reason }
	end

	local hasLanguageErrors = false
	--	Eliminar de data los languages que hayan dado error, y agregar los datos nuevos
	for l, language in pairs(res.languages) do
		if language.success then
			data.languages[l].id = language.id
			data.languages[l].quality_score = language.quality_score
			data.languages[l].status = language.status
			data.languages[l].rejected_reason = language.rejected_reason
			data.languages[l].category = language.category
			data.languages[l].urlButtonParam = bAddUlrParam

			--Datos extra a devolver al cliente para que actualice en su lado
			language.urlButtonParam = bAddUlrParam
			for component_number, component in pairs(data.languages[l].components) do
				if component.type == "BODY" then
					language.text = component.text
					language.params = json.decode(json.encode(component.params))
				end
			end
		else
			data.languages[l] = nil
			hasLanguageErrors = true
		end
	end

	--Si no hubo errores guardamos la cuenta
	if not hasLanguageErrors then
		-- Actualizar HSMConfig.TemplateMessages
		messages[data.id] = data
		-- Actualizar account.HSMConfig 
		account.HSMConfig = account.HSMConfig or {}
		account.HSMConfig.Enabled = "1"
		account.HSMConfig.TemplateMessages = messages

		-- Guardar la account
		success, errInfo = GetModel("Campaigns").SaveAccount(campaignId, account, false, nil)
		if not success then
			LogError("WhatsappSetMessageTemplates Unable to save campaign account %s", accountId)
			return { result = success, reason = errInfo }
		end

		RemoveHsmRepositoryFilesPublicURL(campaignId, accountId, oldHSMConfig, account.HSMConfig or {})
	end

	return res
end

function WhatsappGetCloudApiPublicRepositoryFileUrl(web)
	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to get public url for repository file . Not has licensed for the Whatsapp channel.")
		web.status = 500
		return {
			result = "NOT_LICENSED",
			reason = reason
		}
	end
	
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local campaignId = web.request.campaignId
	local accountId = web.request.accountId
	local fileId = web.request.fileId
	local fileName = web.request.fileName

	LogInfo("Administration - User %s on vcc %s retrieving public url for repository file '%s' on whatsapp account '%s' in campaign '%s'", 
		userId,
		vcc,
		fileId,
		accountId,
		campaignId
	)

	local campaignIdAtVCC = utils.AppendVcc(campaignId, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:WhatsappGetCloudApiPublicRepositoryFileUrl({
		campaignAtVCC = campaignIdAtVCC,
		account = accountId,
		fileId = fileId,
		fileName = fileName
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error retrieving public url for repository file '%s' status on whatsapp account: '%s' in campaign '%s' -> '%s'", 
			fileId, accountId, campaignIdAtVCC, res.reason)
		return { result = res.result, reason = res.reason }
	end
	return { result = "OK", publicFileUrl = res.publicFileUrl }
end

-----------------------------------------
-- SYNC ALLEGRO MEMBERS CAMPAIGNS
-----------------------------------------

--[[
]]--
m_sync.MakeKey = function(vcc, campaignId)
	assert(type(vcc) == "string")
	assert(type(campaignId) == "string")
	local key = string.format("AutomaticSynchronizeAllegroCampaignsMembers:Vcc:%s:Campaign:%s", vcc:upper(), campaignId:upper())
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:MakeKey]------------> key: '%s'", key)
	return key
end

--[[

]]--
m_sync.Get = function(vcc, campaignId)
	local key = m_sync.MakeKey(vcc, campaignId)
	local GetRedisValue = function()
		local redis = m_admModel.GetRedisClient()
		local encodedCurrentRunning = redis:get(key)
		local runningInfo = utils.JSON_Decode(encodedCurrentRunning)
		--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Get]------------> key: '%s', encodedCurrentRunning: '%s'", key, encodedCurrentRunning)
		return runningInfo, encodedCurrentRunning
	end
	local success, runningInfo, encodedCurrentRunning = pcall(GetRedisValue)
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Get]-----pcall-------> success: %s, key: '%s', runningInfo: %s, encodedCurrentRunning: '%s'", success, key, runningInfo, encodedCurrentRunning)
	if not success then
		return {}, json.encode({})
	end
	-- ok
	return runningInfo, encodedCurrentRunning
end

--[[

]]--
m_sync.Set = function(runningInfo)
	assert(type(runningInfo) == "table")
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Set] 1 ....... runningInfo: %s", json.encode(runningInfo))
	assert(type(runningInfo.vcc) == "string")
	assert(type(runningInfo.campaignId) == "string")
	runningInfo.user = inconcert.authUser.userName
	local encodedRunningInfo
	local key = m_sync.MakeKey(runningInfo.vcc, runningInfo.campaignId)
	local SetRedisValue = function()
		local redis = m_admModel.GetRedisClient()
		if runningInfo.status == "RUNNING" then
			local updateMembersInfo = utils.GetUpdateMembersInfo()
			--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Set] 2 ....... runningInfo.status: RUNNING action: %s, expiredTimeSec: %s.", runningInfo.action, updateMembersInfo.warnings.expiredTimeSec)
			encodedRunningInfo = utils.JSON_Encode(runningInfo)
			redis:set(key, utils.JSON_Encode(runningInfo))
			redis:expire(key, updateMembersInfo.warnings.expiredTimeSec)
		else
			--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Set] 3 ....... runningInfo.status: ", runningInfo.status)
			redis:del(key)
			local deletedRunningInfo = m_sync.Get(runningInfo.vcc, runningInfo.campaignId)
			if not utils.TableIsEmpty(deletedRunningInfo) then
				LogWarning("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Set] Unable to Set. running key '%s'", key)
				return false
			end
		end
	end
	local success = pcall(SetRedisValue)
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Set]-----pcall-------> success: %s, key: '%s', runningInfo: %s, encodedCurrentRunning: '%s'", success, key, runningInfo, json.encode(runningInfo))
	if not succcess then
		return false
	end
	return key, runningInfo, encodedRunningInfo
end

--[[

]]--
m_sync.Start = function(vcc, campaignId)
	assert(vcc)
	assert(campaignId)
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:Start]------------>  vcc: '%s', campaignId: '%s'.", vcc, campaignId)
	local runningInfo = {
		action     = "START",
		status     = "RUNNING",
		vcc        = vcc,
		user       = inconcert.authUser.userName,
		campaignId = campaignId,
		runningId  = utils.Uuid(true),
		startDate  = os.time(os.date("!*t")) -- current date in UTC
	}
	return m_sync.Set(runningInfo)
end

--[[

]]--
m_sync.End = function(vcc, campaignId)
	local runningInfo = m_sync.Get(vcc, campaignId)
	--LogDebug("[runningInfo.ALLEGRO_MEMBERS_SYNC:End] 1--->  ending")
	if utils.TableIsEmpty(runningInfo) then
		LogDebug("[runningInfo.ALLEGRO_MEMBERS_SYNC:End] 2--->  ending")
		return false
	end
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:End] 3--->  ending")
	runningInfo.status = "ENDING"
	runningInfo.endDate = os.time(os.date("!*t")) -- current date in UTC
	LogDebug("[runningInfo.ALLEGRO_MEMBERS_SYNC:End] 4 -->  ended - All Info: %s", json.encode(runningInfo))
	m_sync.Set(runningInfo)
	return true
end

--[[

]]--
m_sync.IsRunning = function(vcc, campaignId)
	--m_sync.End()
	local runningInfo, encodedCurrentRunning = m_sync.Get(vcc, campaignId)
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:IsRunning] 1------------> runningInfo: %s, encodedCurrentRunning: '%s'", not utils.TableIsEmpty(runningInfo), (encodedCurrentRunning or "NADA"))
	if utils.TableIsEmpty(runningInfo) then
		--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:IsRunning] 2--NOT IS RUNNING----------")
		return false
	end
	--LogDebug("[ADMINISTRATION.ALLEGRO_MEMBERS_SYNC:IsRunning] 3--IS RUNNING----------")
	return true, runningInfo
end

--[[

]]--
m_sync.MustSynchronize = function(vcc, campaignId, ksAgent, campaignsWarnings)
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:m_sync.MustSynchronize] 1----> vcc: '%s', campaignId: '%s'. campaignsWarnings: '%s'.", vcc, campaignId, json.encode(campaignsWarnings or {}))
	-- 1. Valido que se un VCC que tenga integracion con Allegro.
	local hasAllegroIntegration, vccAllegroIntegrationInfo = utils.HasAllegroIntegration(vcc, ksAgent)
	if not hasAllegroIntegration then
		-- no es un vcc que tenga integracion con allegro
		return false, {result = "VCC NOT HAS ALLEGRO INTEGRATION", campaignId = campaignId, vcc = vcc}
	end
	-- 2. Valido si la actualizacion automatica se encuentra habilitada en el VCC
	if vccAllegroIntegrationInfo.disabledSynchronizeAllegroMembersInLogin == true then
		-- el vcc tiene inhabilitada las actualizaciones automaticas.
		return false, {result = "AUTOMATIC ALLEGRO MEMBERS SYNCRHONIZE IS DISABLED IN VCC", campaignId = campaignId, vcc = vcc, desc = "Attribute: VCC.AllegroIntegration.disabledSynchronizeAllegroMembersInLogin = true"}
	end
	-- 3. Valida que el error que ha ocurrido en Allegro sea "ALLEGRO SERVER OR WEBHANDLER IS DOWN" ("STD::EXCEPTION") para asi SI poder realizar la sincronizacion
	local err = campaignsWarnings and campaignsWarnings.error
	--LogDebug("[ADMINISTRATION.CAMPAIGNS:m_sync.MustSynchronize] 2----> campaignsWarnings.error: '%s'.",  json.encode(err or {}))
	if err and err.errType == "ALLEGRO_INTEGRATION_ERROR" then
		if err.rawErr
			and err.rawErr.sub_type
		then
			LogInfo("[ADMINISTRATION.CAMPAIGNS:m_sync.MustSynchronize] 3--> err.rawErr: '%s'.",  json.encode(err.rawErr or {}))
			local allegroErrType = (err.rawErr.sub_type.result or ""):upper()
			-- errores que permiten la sincronizacion
			local errorsThatEenableSynchronization = {
				-- ERROR: Cuando el Servidor de Allegro se ha caido!!!
				{
					fn = function()
						return allegroErrType == "ALLEGRO SERVER OR WEBHANDLER IS DOWN"
					end
				},
				-- ERROR: Cuando SQL de Allegro ha dado algun error
				{
					fn = function()
						local initialPos = string.find(allegroErrType, "%[MICROSOFT%]%[%ODBC SQL SERVER DRIVER%]") --Ej: [MICROSOFT][ODBC SQL SERVER DRIVER]QUERY TIMEOUT EXPIRED
						return initialPos and true or false
					end
				}
			}
			-- aca recorro los posibles errores que SI permiten la sincronizacion de members.
			local isAllowed = false
			for _, errDef in ipairs(errorsThatEenableSynchronization) do
				isAllowed = errDef.fn()
				if isAllowed then
					break -- SI se permite la sincronizacion
				end
			end
			if isAllowed then
				-- 3.1
				LogInfo("[ADMINISTRATION.CAMPAIGNS:m_sync.MustSynchronize] 3.1 (YES)----> vcc: '%s', campaignId: '%s'. The error '%s' has allows Synchronize.", vcc, campaignId, allegroErrType)
			else
				-- 3.2
				LogInfo("[ADMINISTRATION.CAMPAIGNS:m_sync.MustSynchronize] 3.2 (NO)----> vcc: '%s', campaignId: '%s'. The error '%s' has NOT allows Synchronize.", vcc, campaignId, allegroErrType)
				return false, {result = "CURRENT ALLEGRO ERROR DOES NOT ALLOW SYNC OF MEMBERS", campaignId = campaignId, vcc = vcc, allegroErrType = allegroErrType, errorInfo = err.rawErr}
			end
		end
	end
	-- 4. Valido que NO exista otra sincronizacion en curso para el VCC y la campana.
	-------> m_sync.Start(vcc, campaignId) -- esto es para desarrollo para forzar a un FAKE de que existe corriendo una actualizacion.
	local isRunning, currentRunning = m_sync.IsRunning(vcc, campaignId)
	if isRunning then
		-- ya existe una actualizacion automatica para el vcc/campaign
		return false, {result = "THERE IS CURRENTLY AN UPDATE IN PROGRESS", campaignId = campaignId, vcc = vcc, currentRunning = currentRunning}
	end
	----------------------------------------------------------------
	-- 5. Valido si existe comunicacion con el Servidor de Allegro.
	--	ATENCION:
	--            Esta validacion siempre se debe de realizar por ULTIMO, ya que sino se realizaran muchos REQUEST contra Allegro por cada Login de Usuario mientras aun no haya culminado la sincronizacion.
	----------------------------------------------------------------
	local isReadyAllegroVcc, res = utils.IsReadyAllegroVcc(vcc, ksAgent)
	if not isReadyAllegroVcc then
		return false, {result = "UNABLE TO CONNECT WITH ALLEGRO", campaignId = campaignId, vcc = vcc, err = res}
	end
	-- 6. OK
	return true, {result = "OK"}
end

--[[
	Funcion encargada de sincronizar los usuarios de las campanas de occ con los de allegro.
	Para ello invoca a la funcion "UpdateMembersList" que es quien se encarga de realizar todas las tareas.
	Es invocada mediante un request asincronico en el login de Usuario. (agent.lua)
]]--
m_sync.Synchronize = function (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local startCurrentProcessTime = os.time()
	local campaignId = web.request.campaignId
	local members = web.request.members
	--[[]]--
	local function ReturnValue (status, response)
		ksAgent:Close()
		utils.CollectAllMemory() -- invocamos a garbage ya que esta función realiza muchas conexiones contra cassandra y otras cositas, con esto nos cuidamos !!
		web.status = status
		LogInfo("[ADMINISTRATION.CAMPAIGNS:m_sync.Synchronize] (END) Allegro campaign: '%s' member synchronization is finished. Result: '%s'. Elapsed Time '%s' seconds.", campaignId, json.encode(response), utils.GetElapsedTimeSec(startCurrentProcessTime))
		return response
	end
	-- valida si puede ejecutar la sincronizacion automatica
	local musExecute, resp = m_sync.MustSynchronize(vcc, campaignId, ksAgent)
	if not musExecute then
		local err = resp
		return ReturnValue(400, err)
	end
	-- START
	LogInfo("[ADMINISTRATION.CAMPAIGNS:m_sync.Synchronize] (START) Synchronizing the %d users of the Allegro '%s' campaign", #members, campaignId)
	m_sync.Start(vcc, campaignId)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)
	local isCreateCampaign = false
	local skipLimitsValidation = true
	local skipAlreadyExecute = true
	-- ejecuto a la funcion que se encarga de actualizar la lista de members.
	local success, err = UpdateMembersList(campaignId, members, ksAgent, allegroOtherInfo, isCreateCampaign, skipLimitsValidation, skipAlreadyExecute)
 	if not success then
		if err then
			m_sync.End(vcc, campaignId)
			return ReturnValue(400, err)
		end
	end
	m_sync.End(vcc, campaignId)
	return ReturnValue(200, {result = "OK"})
end

-----------------------------
--[[
	Funcion publica que es invocada en el login de los usuarios de la app "agent" archivo "agent.lua", funcion "CallbackOnAddWarningCampaign"
]]--
function MustExecuteAutomaticAllegroMembersSynchronize (vcc, campaignId, ksAgent, warning)
	return m_sync.MustSynchronize(vcc, campaignId, ksAgent, warning)
end
-------------------------------------------------