/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, maxlen:200 */

/* global inConcert, _*/

/*
The sole purpose of this file is to work as a ajax wrapper in order to be easily replaced on testability scenarios
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	// 'use strict';

	var m_loggedUser = null;
	var asteriskServer = "192.168.22.209";

	//lista de campanias entre iteracion e iteracion
	var lastCampaignsSeen;

	//funciona como el REDIS local, todas las interacciones, inclusive las terminadas, van a parar aca
	var interactionList = {};
	var campaignsAccountsUpdated = {};
	var g_inconcertChat;
	var g_prefixCounter = 0;
	var m_telephone_codes = {"52":{"areasByCode":{"236":["Ajalpan (Puebla)"]},"areasByName":{"Ajalpan (Puebla)":["236"]},"name":"Mexico","code":"52"}};

	function createInteractionSession(interaction) {
		switch (interaction.type) {
			case "CALL":
				break;
			case "INTERNAL_CHAT":
				interaction.internal = "1";
				interaction.origin = interaction.origin ? interaction.origin : "burguer@inconcert";
				interaction.localID = interaction.localID ? interaction.localID : interaction.id;
				g_inconcertChat.chat_client.newSession(interaction);
				break;
			case "CHAT":
			case "IM":
			case "SMS":
				if (interaction.state !== "TAKEN") {
					return;
				}
				g_inconcertChat.chat_client.newSession(interaction);
				break;
		}
	}

	function setInteractionIndex(interaction, indexName, indexValue) {
		if (!interaction.indices) {
			interaction.indices = {};
		}
		if (!interaction.indices[indexName]) {
			interaction.indices[indexName] = [];
		}
		interaction.indices[indexName].push(indexValue);
	}

	function getInteractionByIndex(indexName, indexValue) {
		var iid;
		var interaction;
		for (iid  in inConcert.app.interactionList) {
			if (inConcert.app.interactionList.hasOwnProperty(iid)) {
				interaction = inConcert.app.interactionList[iid];
				if (interaction.indices && interaction.indices[indexName]) {
					if (interaction.indices[indexName].indexOf(indexValue) >= 0) {
						return inConcert.app.interactionList[iid];
					}
				}
			}
		}
	}

	function wrap_interaction(interaction, action){
		var msg = {
			"id": interaction.id,
			"action": action,
			"data": interaction
		};
		return msg;
	}

	/**
	 * Will queue to the local storage an interaction
	 * If the interaction already exists, will be updating the current array.
	 *
	 * @param  {object} interaction - The interaction object
	 */
	function queueInteraction(interaction) {
		var wrapped_interaction = wrap_interaction(interaction, 'create_interaction');
		var previous_interactions = inConcert.Storage.get('interactions');

		// empty interactions list, push a new array of one interaction.
		if (!previous_interactions || previous_interactions.length === 0 ) {
			inConcert.Storage.put('interactions', [ wrapped_interaction ]);
			return;
		}

		// otherwise, update the previous array and push it to the storage.
		previous_interactions.push(wrapped_interaction);
		inConcert.Storage.put('interactions', previous_interactions);
	}

	function afterInteractionTaken(interaction){
		if (interaction.type === "CALL") {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			if (!phone) {
				return;
			}
			interaction.from = "999999";
			interaction.to = "88888";
			phone.m_phoneObject.PhoneEvent.OnInvited(interaction);
		}
	}

	function readStorageDirectory() {
		var result = [];
		var interactions = inConcert.Storage.get('interactions_storage');

		if (!interactions){
			return result;
		}

		interactions.forEach(function(data){
			inConcert.app.interactionList[data.id] = data.data;
			createInteractionSession(data.data);
			result.push(data.data);
		});
		return result;
	}

	/**
	 * Will load the interactions from the local storage,
	 * This method is called on every main loop.
	 *
	 * @return {array} - An array of the read interactions.
	 */
	function readInteractionsDirectory() {

		//go search for interactions
		var result = [];
		var interactions = inConcert.Storage.get('interactions');

		if (!interactions){
			return result;
		}

		interactions.forEach(function(data){
			var channel = "CAMPAIGN";
			if (data.action === "create_interaction") {
				inConcert.app.interactionList[data.id] = data.data;
				//this decision is obviously lame, but better than nothing
				if (data.data.assignedAgents && data.data.assignedAgents.length > 0) {
					channel = "AGENT";
				}
				createInteractionSession(data.data);
				result.push({
					channel: channel,
					message: data.data
				});
			}
			else if (data.action === "finalize_interaction") {
				if (!inConcert.app.interactionList) {
					return;
				}
				var interactionId = data.id;
				var interaction_to_finalize = inConcert.app.interactionList[interactionId];
				if (!interaction_to_finalize) {
					return;
				}
				//this decision is obviously lame, but better than nothing
				if (data.data.assignedAgents && data.data.assignedAgents.length > 0) {
					channel = "AGENT";
				}
				//delete inConcert.app.interactionList[interactionId];
				result.push({
					channel: channel,
					message: data.data
				});
			}
			else if (data.action === "add_message_to_interaction") {
				g_inconcertChat.SendMessage(data.id, data.data.message, data.data.user, data.data.name);
			}
			else if (data.action === "internal_chat_new_session_and_send_im") {
				createInteractionSession(data.data);
			}
			else if (data.action === "add_mail_to_interaction") {
				var mail = inConcert.app.interactionList[data.id];
				if (!mail) {
					return;
				}
				var timestamp = getCurrentTimestamp();
				inConcert.logger.guiAction({action_name : "send_mail_by_client", window_id: window.id}, mail);
				var mailEvent = {
					"attachments": data.data.attachments,
					"body": data.data.body,
					"cc": data.data.cc,
					"date": timestamp,
					"from": data.data.from,
					"name": data.data.name,
					"subject": data.data.subject,
					"to": data.data.to
				};

				mail.events.push(mailEvent);
				mail.timestamp = timestamp;
				queueInteraction(mail);
			}
			else if (data.action === "add_twitter_reply") {
				var tweet = inConcert.app.interactionList[data.id];
				if (!tweet){
					return;
				}
				inConcert.logger.guiAction({action_name : "send_twitter_by_client", window_id: window.id}, tweet);

				tweet.events.push(data.data);
				tweet.timestamp = getCurrentTimestamp();
				queueInteraction(tweet);
			}
		});

		// before return, clean the interactions localstorage
		inConcert.Storage.put("interactions", []);

		return result;
	}

	function getInteractionFromMemory(interaction_id){
		return inConcert.app.interactionList[interaction_id] || null;
	}

	function getSpamInteractions() {
	}

	function deleteSpamInteractions(interactionIDs) {
		localStorage.spam_interactions = [];
	}

	function markNoSpamInteractions(interactionIDs) {
	}

	function saveSpamInteraction(interaction) {
	}

	function readCampaignsDirectory() {
		var actions = inConcert.Storage.get('campaigns');
		if (!actions){
			return;
		}
		actions.forEach(function(parsedData){
			var campaign_info = parsedData.data;

			if (parsedData.action === "add_campaign"){
				// already exists, nothing to do.
				if (inConcert.app.database.campaigns[campaign_info.Id]){
					return;
				}

				inConcert.app.database.campaigns[campaign_info.Id] = campaign_info;
				inConcert.app.database.vccCampaigns[campaign_info.VCC][campaign_info.Id] = campaign_info.Name;

				// add current user to this campaign if is the same vcc as the campaign
				var userWithVCC = m_loggedUser.Id + "@" + campaign_info.VCC;
				if (inConcert.app.database.userCampaigns[userWithVCC]){
					inConcert.app.database.userCampaigns[userWithVCC][campaign_info.Id] = 'administrator';
				}
			}
			else if (parsedData.action === "update_account_status") {
				var campaign = inConcert.app.database.campaigns[campaign_info.Id];
				if (!campaign || !campaign.accounts) {
					return;
				}
				campaign.accounts.forEach(function(account){
					if (account.id !== campaign_info.accountId) {
						return;
					}
					if (!campaign_info.accountStatus.state && !campaign_info.accountStatus.status) {
						return;
					}
					if (campaign_info.accountStatus.state) {
						account.state = campaign_info.accountStatus.state;
					}
					if (campaign_info.accountStatus.status) {
						account.status = campaign_info.accountStatus.status;
					}
					inConcert.app.database.campaigns[campaign_info.Id].accounts[account.id] = account;
					var campaignsAccountsUpdated = inConcert.app.campaignsAccountsUpdated;
					campaignsAccountsUpdated[campaign.Id] = campaignsAccountsUpdated[campaign.Id] === undefined ? [] : campaignsAccountsUpdated[campaign.Id];
					campaignsAccountsUpdated[campaign.Id].push(account);
					inConcert.app.campaignsAccountsUpdated = campaignsAccountsUpdated;
					return;
				});
			}
		});
		return;
	}

	function setEventRead(interaction,events) {
		var user = m_loggedUser.Id;

		if (!interaction.readers) {
			interaction.readers = {};
		}
		if (!interaction.readers[user]) {
			interaction.readers[user] = [];
		}

		if (events.length === 0) {
			interaction.events.forEach(function(interaction_event, index){
				if (interaction.readers[user].indexOf(index) < 0) {
					interaction.readers[user].push(interaction_event);
				}
			});
		}
		else {
			events.forEach(function(current_event, index){
				if (interaction.readers[user].indexOf(index) < 0) {
					interaction.readers[user].push(current_event);
				}
			});
		}
	}

	var get_campaign_state = function(agent_state){
		if (agent_state === "ACTIVE" || agent_state === "AVAILABLE"){
			return "IDLE";
		}
		else if (agent_state === "BREAK" || agent_state === "LUNCH"){
			return "BREAK";
		}
		else if (agent_state === "BUSY"){
			return "BUSY";
		}
		else {
			return "OFFLINE";
		}
	};

	/**
	 * Returns a random number between min and max
	 */
	function getRandomArbitary (min, max) {
	    return Math.random() * (max - min) + min;
	}

	/**
	 * Returns a random integer between min and max
	 * Using Math.round() will give you a non-uniform distribution!
	 */
	function getRandomInt (min, max) {
	    return Math.floor(Math.random() * (max - min + 1)) + min;
	}

	function addMinutes(date, minutes) {
		return (date.getTime() + minutes*60000) * 1000;
	}

	function getCurrentTimestamp(){
		var now = new Date();
		var zp = inConcert.Utils.zeropad;
		var timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(),
										zp(now.getUTCMonth()+1),
										zp(now.getUTCDate()),
										zp(now.getUTCHours()),
										zp(now.getUTCMinutes()),
										zp(now.getUTCSeconds())
									);
		return timestamp;
	}


	function createInteraction(id, campaign, vcc, account, initiative, initialState, initialAgent, contactID, contactAddress, proxyType, proxyIP, proxyPort, initialData) {
		var campaignId = campaign + "@" + vcc;
		var now = new Date();
		var zp = inConcert.Utils.zeropad;
		var timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
			now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
		var interaction = {
			id : id || inConcert.Utils.generateGuid(),
			assignedAgents: [
				initialAgent
			],
			VCC: vcc,
			account: account,
			readers: {
				"admin@inconcert": []
			},
			events : [
			],
			steps : [
				{
					"action" : "CREATED",
					"timestamp" : timestamp
				},
				{
					"action" : "QUEUED",
					"timestamp" : timestamp
				}
			],
			proxyIP: proxyIP,
			proxyPort: proxyPort,
			contact: contactID,
			contactAddress: contactAddress,
			type: proxyType,
			live: proxyType==="0"?false:true,
			interactionData: initialData,
			initiative: initiative,
			requiredAgent: "",
			state: initialState,
			timestamp : timestamp,
			campaign : campaignId
		};
		inConcert.app.interactionList[interaction.id] = interaction;
		return interaction;
	}

	function xFormToJavascript(data) {
		var matcher = new RegExp("([^&=]+)=([^&]+)", "g");
		var match = matcher.exec(data);
		var result = {};
		while (match) {
			result[match[1]] = match[2];
			match = matcher.exec(data);
		}
		return result;
	}

	function addressAlreadyThere(address, addressesList) {
		for (var i = 0; i < addressesList.length; i++) {
			var current = addressesList[i];
			if (address.Type === current.Type) {
				switch (address.Type) {
					case "Phone":
						if (address.Number === current.Number) {return true;}
						break;
					case "Facebook":
						if (address.UID === current.UID) {return true;}
						break;
					case "Twitter":
						if (address.User === current.User) {return true;}
						break;
					case "IM":
						if (address.buddy === current.buddy) {return true;}
						break;
					case "Mail":
						if (address.address === current.address) {return true;}
						break;
					default:
				} //switch
			} //if equal types
		} //for
		return false;
	}

	var getAddressDisplayText = function(address) {
		switch (address.Type) {
			case "Phone":
				return address.Number;
			case "Facebook":
				return address.UID;
			case "Twitter":
				return address.User;
			case "IM":
				return address.buddy;
			case "Mail":
				return address.address;
			case "Webchat":
				return address.customerId;
			default:
				throw("Someone is trying get address display text with an invalid address type: " + address);
		}
	};

	if (this.app && this.app.m_interactionManager) {
		// TO-DO: WTF IS THIS? No comment?
		this.app.m_interactionManager.oldDiscard = this.app.m_interactionManager.discard;
		this.app.m_interactionManager.discard = function(element) {
			this.oldDiscard(element);
			var interaction =  inConcert.app.interactionList[element.id];
			if (interaction) {
				//send interaction to storage
				inConcert.Storage.put('interactions_storage', wrap_interaction(interaction, 'create_interaction'));
			}
			else {
				throw ("Discard de algo que no es interaction");
			}
		};
	}

	////////////////////////////////////////////////////////////////////////
	// mockup de InconcertChat.js
	//
	function inconcertChatMockup(parameters) {
		var self = this;
		g_inconcertChat = self;
		self.chat_client = typeof(parameters) === "object" ? paramters : {};

		self.chat_client.newSession = function(interaction) {
			var session = {
				id: interaction.id
			};
			var values = {
				id: interaction.id,
				session_id: interaction.id,
				internal: interaction.internal,
				origin: interaction.origin,
				localID: interaction.localID,
				message: interaction.message
			};
			if (inConcert.app.interactionList[interaction.id]){
				inConcert.app.interactionList[interaction.id].session = { messages : [], session_id : interaction.id };
			}
			self.onNewSession(session, values);
		};

		self.chat_client.getInitialSessionMessages = function(session) {
			/*Despues veremos si se guarda algo mas del chat, como en repositorio o algo de eso*/
			if (!session) {
				return;
			}
			var getMessageInfo = function() {
				var message = {
					name: "defaultName",
					text: "defaultText",
					owned: false
				};
				if (session.internal === "1") {
					message.name = session.origin;
					message.text = session.message;
				}
				else {
					if (!inConcert.app.interactionList) {
						return;
					}
					var interaction = inConcert.app.interactionList[session.id];
					if (!interaction) {
						return;
					}
					message.name = interaction.interactionData.contactName;
					message.text = interaction.interactionData.firstMessage;
				}
				return message;
			};
			var messageInfo = getMessageInfo();
			if (!messageInfo) {
				return;
			}
			var messages = [];
			messages.push(messageInfo);
			return messages;
		};
	}

	inconcertChatMockup.prototype.onNewSession = function(session, values) {
		var self = this;
		self.chat_client.onNewSession(session, values);
		var initialMessages = self.chat_client.getInitialSessionMessages(values);
		if (!initialMessages) {
			return;
		}
		for (var i in initialMessages) {
			if (initialMessages.hasOwnProperty(i)) {
				var message = initialMessages[i];
				if (message.text) {
					self.SendMessage(session.id, message.text, message.name, message.name);
				}
			}
		}
	};

	inconcertChatMockup.prototype.onUserLeft = function(m_url) {
	};

	inconcertChatMockup.prototype.onSessionClosed = function(m_url) {
	};

	inconcertChatMockup.prototype.SetChatUrl = function(m_url) {
		return true;
	};

	inconcertChatMockup.prototype.Login = function() {
		var self = this;
		self.chat_client.onServerReconnected();
	};

	inconcertChatMockup.prototype.CloseSession = function(sessionId) {
		var interaction = getInteractionFromMemory(sessionId);
		console.debug(interaction);
		if (interaction){
			var agent_id = m_loggedUser.Id;
			var ids_answered = _.pluck(interaction.session.messages, 'user_id');
			var agent_answered = _.contains(ids_answered, agent_id);

			interaction.state = agent_answered ? "WRAPUP" : "FINISHED";
			interaction.steps.push({
				action : agent_answered ? "WRAPUP" : "FINISHED",
				agent: agent_id,
				timestamp : getCurrentTimestamp()
			});
			this.chat_client.onSessionClosed({ id : sessionId }, {});
			queueInteraction(interaction);
			return true;
		}
		else {
			return false;
		}
	};

	inconcertChatMockup.prototype.SendMessage = function(sessionid, message, userId, userName) {
		var self = this;
		var session = {
			id: sessionid
		};
		var values = {
			session_id: sessionid,
			message: "OnNewMessage",
			text: message,
			user_id: userId ? userId : m_loggedUser.Id,
			display_name: userName ? userName : m_loggedUser.Id,
			msgtype: typeof(message) === "string"  ? "text" : null,
			msgtime: new Date().getTime() / 1000
		};

		inConcert.app.interactionList[sessionid].session.messages.push(values);
		self.chat_client.onMessage(session, values, true);
	};

	inconcertChatMockup.prototype.SendIndication = function(sessionid, indication_type, other, values) {
		return true;
	};

	inconcertChatMockup.prototype.GetVcc = function() {
		return "inconcert";
	};

	inconcertChatMockup.prototype.InviteUser = function(userId, callbacksFn, options) {
		var self = this;
		self.chat_client.newSession({id: options.localID});
		callbacksFn(true, options.localID);
	};

	//----
	InconcertChat = inconcertChatMockup;

	////////////////////////////////////////////////////////////////////////
	// mockup del objeto flash del telefono
	//
	if (window.app) {
		window.app.newPhone = function() {
			return {
				eventList : [],
				interactionActionExecute: function(action) {
					if (!action) {
						return;
					}
					var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
					if (!phone) {
						return;
					}
					var currentInteraction = phone.getInteraction();
					if (!currentInteraction) {
						return;
					}
					switch (action) {
						case "archive":
							currentInteraction.archive();
							break;
						case "requeue":
							currentInteraction.requeue();
							break;
						case "create_outgoing_call":
							var match = currentInteraction.campaign.match(new RegExp("([^@]+)@(.*)"));
							var campaign = match[1];
							var vcc = match[2];
							var contactName = currentInteraction.contactAddress;
							var loggedUser = inConcert.app.loggedUserApi;
							var interaction = createInteraction(inConcert.Utils.generateGuid(), campaign, vcc, currentInteraction.account,
																	"OUTBOUND", "TAKEN", loggedUser.id, currentInteraction.contactAddress,
																	"", "CALL", "127.0.0.1", "12000",
																	{
																		contactName: contactName,
																		completeAddress: {
																			DialedNumber: currentInteraction.contactAddress
																		}
																	});
							queueInteraction(interaction);
							return;
						default:
							throw("Interaction action '" + action + "' not found.");
					}
					inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode(currentInteraction);
				},
				accept: function() {
					this.eventList.push({ type : "PhoneEvent.OnAccepted" });
				},
				bye: function() {
					this.interactionActionExecute("archive");

				},
				connect: function(host, port, username, password, displayname) {
					this.eventList.push({ type : "PhoneEvent.OnConnected" });
				},
				disconnect: $.noop,
				invite: function(number) {
					this.eventList.push({ type : "PhoneEvent.OnAccepted" });
					this.interactionActionExecute("create_outgoing_call");
				},
				reject: function() {
					this.interactionActionExecute("requeue");
				},
				sendDigit: function(key) {
					return;
				},
				sendHold: function(hold) {
					return;
				},
				getNextEvent: function() {
					var result = null;
					if (this.eventList.length > 0) {
						result = this.eventList[0];
						this.eventList.splice(0, 1);
					}
					return result;
				}
			};
		};
	}
	//////////////////////////////////////////////////////////////////

	var database = {
		auto_attendants : {

		},
		packages : {
			CALL : {
				"24007885" : {
						id : "24007885",
						description : "24007885",
						ioMode : "I/O",
						capacity: " 40",
						DID : "24007885",
						outgoingRoute : "UYRoute",
						callerIDIsDID : "true"
				},
				"08002020" : {
						id : "080002020",
						description : "08002020",
						ioMode : "I/O",
						capacity: " 40",
						DID : "080002020",
						outgoingRoute : "UYRoute",
						callerIDIsDID : "true"
				}
			}
		},
		application: {
			"administration": {
				"Name": "Administration",
				"Actions": {
				"deletecampaign": {
					"Entity": {id: "campaign", name: "Campaign"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"viewuserstates": {
					"Entity": {id: "state", name: "State"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"saveworkgroup": {
					"Entity": {id: "workgroup", name: "Workgroup"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"create_campaign": {
					"Entity": {id: "campaign", name: "Campaign"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"saveuser": {
					"Entity": {id: "user", name: "User"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"savemessage": {
					"Entity": {id: "message", name: "Message"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"viewgroups": {
					"Entity": {id: "group", name: "Group"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"deleteskill": {
					"Entity": {id: "skill", name: "Skill"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"viewcampaigns": {
					"Entity": {id: "campaign", name: "Campaign"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"deactivate_user": {
					"Entity": {id: "user", name: "User"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"saveuserstate": {
					"Entity": {id: "state", name: "State"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"deletegroup": {
					"Entity": {id: "group", name: "Group"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"viewskill": {
					"Entity": {id: "skill", name: "Skill"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"deletemessage": {
					"Entity": {id: "message", name: "Message"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"savegroup": {
					"Entity": {id: "group", name: "Group"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"viewworkgroup": {
					"Entity": {id: "workgroup", name: "Workgroup"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"deleteuserstate": {
					"Entity": {id: "state", name: "State"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"viewmessage": {
					"Entity": {id: "message", name: "Message"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				},
				"saveskill": {
					"Entity": {id: "skill", name: "Skill"},
					"Domain": "entities",
					"Description": "Save",
					"Category": "save"
				},
				"deleteworkgroup": {
					"Entity": {id: "workgroup", name: "Workgroup"},
					"Domain": "entities",
					"Description": "Delete",
					"Category": "delete"
				},
				"viewusers": {
					"Entity": {id: "user", name: "User"},
					"Domain": "navigation",
					"Description": "View",
					"Category": "view"
				}
				}
			},
			"supervisor": {
				"Name": "Supervisor",
				"Actions": {
					"everything": {
						"Entity": {id: "everything", name: "everything"},
						"Domain": "navigation",
						"Description": "View",
						"Category": "view"
					}
				}
			},
			"reports": {
				"Name": "Reports",
				"Actions": {
					"everything": {
						"Entity": {id: "everything", name: "everything"},
						"Domain": "navigation",
						"Description": "View",
						"Category": "view"
					}
				}
			},
			"interaction": {
				"Name": "Interaction",
				"Actions": {
					"everything": {
						"Entity": {id: "everything", name: "everything"},
						"Domain": "navigation",
						"Description": "View",
						"Category": "view"
					}
				}
			}
		},
		groups: {
			administrator: {
				Id: 'administrator',
				Name: 'administrator',
				Description: "Administrator system role",
				VCC: "system",
				IsSystemGroup: '1'
			},
			agent: {
				Id: 'agent',
				Name: 'agent',
				Description: "Agent system role",
				VCC: "system",
				IsSystemGroup: '1'
			},
			supervisor: {
				Id: 'supervisor',
				Name: 'supervisor',
				Description: "Supervisor system role",
				VCC: "system",
				IsSystemGroup: '1'
			}
		},
		grouppermission: {
			"administrator@system": {
				"administration": {
					"view": {
						"viewuserstates":true,
						"viewgroups":true,
						"viewskill":true,
						"viewcampaigns":true,
						"viewworkgroup":true,
						"viewmessage":true,
						"viewusers":true
					},
					"delete": {
						"deletecampaign":true,
						"deleteskill":true,
						"deletegroup":true,
						"deletemessage":true,
						"deleteworkgroup":true,
						"deleteuserstate":true,
						"deleteuser":true,
						"deletecontact":true
					},
					"save": {
						"create_campaign":true,
						"saveuser":true,
						"saveworkgroup":true,
						"saveuserstate":true,
						"saveskill":true,
						"savemessage":true,
						"savegroup":true
					}
				},
				"supervisor": {
					"view": {
						"everything":true
					}
				},
				"reports": {
					"view": {
						"everything":true
					}
				},
				"interaction": {
					"view": {
						"everything":true
					},
					"delete": {
						"deletecontact":true
					}
				}
			},
			"supervisor@system": {
				"supervisor": {
					"view": {
						"everything":true
					}
				},
				"reports": {
					"view": {
						"everything":true
					}
				}
			},
			"agent@system": {
				"interaction": {
					"view": {
						"interaction":true
					}
				}
			}
		},
		userpermission: {

		},
		workgroups: {
			cany_workgroup: {
				Id: 'cany_workgroup',
				Name: 'cany workgroup',
				Description: "Grupo de trabajo de prueba de Cany"
			},
			rocio_workgroup: {
				Id: 'rocio_workgroup',
				Name: 'Rocio workgroup',
				Description: "Grupo de trabajo de prueba de Rocio"
			}
		},
		skills: {
			cany_skill: {
				Id: 'cany_skill',
				Name: 'cany_skill',
				DefaultValue: 14041996,
				Description: "Skill de prueba"
			}
		},
		availableStates: {
			LUNCH: {
				Name: "LUNCH",
				overrideable: '0',
				Description: "Off to lunch",
				isBreak: '1'
			},
			ACTIVE: {
				Name: "ACTIVE",
				Description: "Available",
				isBreak: '0'
			},
			BATHROOM: {
				Name: "BATHROOM",
				overrideable: '0',
				Description: "Bathroom",
				isBreak: '1'
			},
			BREAK: {
				Name: "BREAK",
				isInitial: '1',
				overrideable: '1',
				Description: "Busy",
				isBreak: '1'
			},
			OFFLINE: {
				Name: "OFFLINE",
				isFinal: '1',
				overrideable: '0',
				Description: "Unavailable",
				isBreak: '1'
			}
		},
		permissions: {
			supervisor: {
				im_accept_contact: "1",
				create_contact: "1",
				im_block_contact: "1",
				modify_campaign: "1",
				deletecontact: "1",
				call_free_dial: "1",
				localization_settings: "1",
				call_dial_contact: "1",
				complete_contact_history: "1",
				change_campaign: "1",
				change_phone_settings: "1",
				initiate_chat: "1",
				personal_contact_history: "1",
				personal_campaign_history: "1",
				modify_contact: "1",
				im_add_contact: "1",
				change_sounds_settings: "1",
				create_campaign: "1",
				invite_to_campaign: "1",
				complete_campaign_history: "1"
			},
			agent: {
				im_accept_contact: "1",
				create_contact: "1",
				change_sounds_settings: "1",
				initiate_chat: "1",
				personal_campaign_history: "1",
				localization_settings: "1",
				personal_contact_history: "1",
				im_add_contact: "1",
				call_dial_contact: "1"
			},
			teammember: {
				complete_contact_history: "1",
				personal_contact_history: "1",
				initiate_chat: "1",
				modify_contact: "1",
				im_accept_contact: "1",
				create_contact: "1",
				change_phone_settings: "1",
				create_campaign: "1",
				im_add_contact: "1",
				personal_campaign_history: "1",
				deletecontact: "1",
				call_free_dial: "1",
				localization_settings: "1",
				change_sounds_settings: "1",
				complete_campaign_history: "1",
				call_dial_contact: "1"
			},
			member: {
				complete_contact_history: "1",
				personal_contact_history: "1",
				initiate_chat: "1",
				modify_contact: "1",
				im_accept_contact: "1",
				create_contact: "1",
				change_phone_settings: "1",
				create_campaign: "1",
				im_add_contact: "1",
				personal_campaign_history: "1",
				deletecontact: "1",
				call_free_dial: "1",
				localization_settings: "1",
				change_sounds_settings: "1",
				complete_campaign_history: "1",
				call_dial_contact: "1"
			},
			administrator: {
				complete_contact_history: "1",
				personal_contact_history: "1",
				initiate_chat: "1",
				modify_contact: "1",
				im_accept_contact: "1",
				create_contact: "1",
				change_phone_settings: "1",
				create_campaign: "1",
				im_add_contact: "1",
				personal_campaign_history: "1",
				deletecontact: "1",
				call_free_dial: "1",
				localization_settings: "1",
				change_sounds_settings: "1",
				complete_campaign_history: "1",
				call_dial_contact: "1",
				archive_interaction: "1"
			},
			moderator: {
				archive_interaction: "1"
			}
		},
		peers: {
			"asterix_SIP_1001": {
				tech: "SIP",
				user: "1001",
				name: "1001",
				password: "1234",
				codecs: "alaw,ulaw,g729,speex",
				hostId: "asterix",
				hostIP: "127.0.0.1"
			},
			"asterix_SIP_1002": {
				tech: "SIP",
				user: "1002",
				name: "1002",
				password: "1234",
				codecs: "alaw,ulaw,g729,speex",
				hostId: "asterix",
				hostIP: "127.0.0.1"
			},
			"asterix_SIP_1003": {
				tech: "SIP",
				user: "1003",
				name: "1003",
				password: "1234",
				codecs: "alaw,ulaw,g729,speex",
				hostId: "asterix",
				hostIP: "127.0.0.1"
			},
			"asterix_SIP_1004": {
				tech: "SIP",
				user: "1004",
				name: "1004",
				password: "1234",
				codecs: "alaw,ulaw,g729,speex",
				hostId: "asterix",
				hostIP: "127.0.0.1"
			}
		},
		vccpeers: {
			"VCC:inconcert": [
				"asterix_SIP_1001",
				"asterix_SIP_1002",
				"asterix_SIP_1003",
				"asterix_SIP_1004"
			]
		},
		vccusers: {
			inconcert: {
				"admin": "All Mighty Administrator"
			}
		},
		usersbyemail: {
			"admin@inconcertcc.com": {
				"admin@inconcert": ""
			}
		},
		usersbypeer: {
			"asterix_SIP_1001": {
				"admin@inconcert": ""
			}
		},
		agentaddresses: { // super column
		},
		agentskills: {
		},
		users: {
			"admin@inconcert" : {
				password : "a0654d8bed35bfe4cd9309b8ef142149",
				userId  :"admin",
				Id: "admin",
				VCC : "inconcert",
				StatusMessage : "",
				status: {
					state: "ACTIVE"
				},
				Name : "All Mighty Administrator",
				initialState : "BREAK",
				countryCode: "598",
				areaCode: "98",
				userRole: "administrator",
				UserGroup: "administrator",
				Enabled: "1",
				campaignState: "IDLE",
				stateTimeTimestamp : new Date().getTime() / 1000,
				campaignLiveInteractionsCount: 0,
				InteractionClients : {
					IM :{
						username: "admin",
						vcc:"inconcert",
						resourceAddress:"",
						address_id:"willy",
						ClientType:"IM",
						address:"admin",
						alias:"willy",
						"resource_address":"",
						initialized: true
					},
					MAIL:{
						Type:"Mail",
						address_id:"admin@inconcertcc.com",
						ClientType:"MAIL",
						address:"admin@inconcertcc.com",
						resource_address:"",
						initialized: true
					},
					CALL:{
						Type:"Phone",
						phoneType: "builtin",
						tech:"SIP",
						resource_address:asteriskServer + ":5060",
						username:"1000",
						ClientType:"CALL",
						rtmpServer:asteriskServer,
						address_id:"SIP/1000",
						password:"1234",
						sipServer:asteriskServer,
						initialized: true
					},
					CHAT:{
						Type:"Chat",
						resource_address:"",
						username:"admin",
						vcc:"inconcert",
						resourceAddress:"",
						address_id:"willy",
						address:"admin",
						alias:"willy",
						ClientType:"CHAT",
						initialized: true
					},
					FACEBOOK: {
						Type:"Facebook",
						address_id:"admin@facebook.com",
						ClientType:"FACEBOOK",
						address:"admin@facebook.com",
						resource_address:"",
						initialized: true
					},
					TWITTER: {
						Type:"Twitter",
						address_id:"admin@twitter.com",
						ClientType:"TWITTER",
						address:"admin@twitter.com",
						resource_address:"",
						initialized: true
					}
				}
			},
		},
		contacts : {
		},
		members: {
			"admin@inconcert" : {
				addresses: [
					{
						address_id: "SIP/1014",
						ClientType: "CALL",
						password: "sinclave",
						physical_phone: "0",
						resource_address: "64.76.241.27:5060",
						rtmpServer: "64.76.241.27",
						sipServer: "64.76.241.27",
						tech: "SIP",
						Type: "Phone",
						name: "1014"
					},
					{
						address: "admin",
						address_id: "admin@inconcert",
						alias: "admin",
						ClientType: "CHAT",
						resource_address: "64.76.241.28",
						resourceAddress: "64.76.241.28",
						Type: "Chat",
						username: "admin",
						vcc: "inconcert"
					},
					{
						address: "admin",
						address_id: "admin",
						alias: "admin",
						ClientType: "IM",
						resource_address: "64.76.241.28",
						resourceAddress: "64.76.241.28",
						username: "admin",
						vcc: "inconcert"
					},
					{
						address: "admin@inconcertcc.com",
						address_id: "admin@inconcertcc.com",
						ClientType: "MAIL",
						resource_address: "",
						Type: "Mail"
					}
				],
				campaigns: [
					"productdevelopment@inconcert"
				],
				Country: "598",
				DefaultCampaign: "ryd",
				Id: "admin",
				initialState: "ACTIVE",
				IsSystemUser: "0",
				LanguageId: "en",
				Name: "All Mighty Administrator",
				Password: "ee72417fa8d3104fedf49b4079e9e8c7",
				State: "ENABLED",
				campaignState: "IDLE",
				campaignLiveInteractionsCount: 0,
				status: {
					addresses: {
						CALL: {},
						CHAT: {},
						IM: {},
						MAIL: {}
					},
					agentID: "admin",
					campaigns: {
						"productdevelopment": true
					},
					defaultCampaign: "ryd",
					description: "All Mighty Administrator",
					interactions: [],
					lastLoginTime: 1320841919,
					lastStatusChangeTime: 1320841920,
					state: "ACTIVE",
					VCC: "inconcert"
				},
				StatusMessage: "",
				TimeZoneId: "UYT",
				UserGroup: "supervisor",
				ValidationOrigin: "0",
				VCC: "inconcert",
				VccGroup: "system"
			}
		},
		userContactFavorites : {
		},
		campaignSchedulers: {
			"productdevelopment@inconcert": {}
		},
		campaignAccounts: {
			"productdevelopment@inconcert": {
				"CALL_elidCALL": {
					Id: "CALL_elidCALL",
					proxyType: "CALL",
					id: "elidCALL",
					phoneType: "builtin",
					outboundPrefix: "99",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"CHAT_elgransite.com-chat": {
					Id: "CHAT_elgransite.com-chat",
					"proxyType": "CHAT",
					"url_regexp": "[\"elgransite.com\"]",
					"autochat": "{\"url_regexp\":\"elgransite.com\"}",
					"id": "elgransite.com-chat",
					"Token": "Rocio_TOKEN",
					"Tracking": "1",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"SMS_elidSMS": {
					Id: "SMS_elidSMS",
					proxyType: "SMS",
					id: "elidSMS",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"EMAIL_inconcert6@inconcertcc.com": {
					Id: "EMAIL_inconcert6@inconcertcc.com",
					type: "POP3",
					user: "inconcert6",
					id: "inconcert6@inconcertcc.com",
					proxyType: "MAIL",
					sendUrl: "smtp: //mail.inconcertcc.com",
					password: "cacho",
					encryption: "0",
					receiveUrl: "pop3: //mail.inconcertcc.com",
					defaultName: "Campania Fabrica inConcert 6",
					groupInThreads: "1",
					signature: "inConcert 6",
					authentication: "0",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"IM_inconcertbot@prpl-yahoo": {
					Id: "IM_inconcertbot@prpl-yahoo",
					password: "cacho",
					type: "IM",
					protocol: "prpl-yahoo",
					user: "inconcertbot",
					id: "inconcertbot@prpl-yahoo",
					proxyType: "IM",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"FACEBOOK_100001492544293": {
					Id: "FACEBOOK_100001492544293",
					proxyType: "FACEBOOK",
					id: "100001492544293",
					accessToken: "125846710792839|91560aa9fe4c20737233edd6.1-100001492544293|_opT5sHOetS2lr97zyNi61MUN1I",
					code: "3sQXhJBK9reU-BsI1rlCeWiEiFp7U7XNEIcRynUJTjM.eyJpdiI6IkZTUEFmNmxRUVBnMC1oRDdfZFlTTVEifQ.vOZPlgvohZOtVPPJG66oyUQBxJI-azszrvsIc3sPQXcv-7JPfeEwSkUSO8aZQHgmfsg9a5b5JCRRE4T06p56AUWRZkQfm3AePgS8OU-xyhcPf3OTOY_jrXic2qd_DpY8",
					homeMatchRules: "{'status':[],'video':[],'link':[]}",
					name: "Cany Prueba",
					onlyCheckPages: "1",
					pagesNames: "{\"page1\":\"pagina1\",\"page2\":\"pagina2\"}",
					pages: "{}",
					uid: "100001492544293",
					state: "1",
					status: {
						status: "OK"
					}
				},
				"TWITTER_rumpelstink": {
					Id: "TWITTER_rumpelstink",
					id: "rumpelstink",
					oauth_verifier: "Q3n4DPjcxiouZQ0hIaQQpZROageSVVgaEICCLXH7xcY",
					proxyType: "TWITTER",
					screen_name: "rumpelstink",
					tokenKey: "261412447-LSA2p3a67ZXtRv5WY5AJADx4M94QN2aDMlzaNgYL",
					tokenSecret: "VAoi8AG6qUF5Vl0hh901vqNIeSVtYoGgBHCTi2CcyI",
					user_id: "261412447",
					state: "1",
					status: {
						status: "OK"
					}
				}
			}
		},
		campaignContacts : {
			"productdevelopment@inconcert" : [
				"140419961234567890ABCDEF0123456@inconcert",
				"512FE95AFE1011A5D2037B488207505C@inconcert",
				"512FE95AFE1011A5D2037B488207509D@inconcert"
			]
		},
		userCampaigns : {
			"admin@inconcert" : {
				"productdevelopment@inconcert": "administrator"
			}
		},
		campaignMembers : {
			"productdevelopment@inconcert" : {
				"admin@inconcert": "administrator"
			}
		},
		vccCampaigns: {
			"inconcert": {
				"productdevelopment@inconcert": "productdevelopment"
			}
		},
		campaigns : {
			"productdevelopment@inconcert": {
				Id: "productdevelopment@inconcert",
				Description: "R&D @ inConcert",
				Integration: [

				],
				ValidationTimeZone: "UYT",
				scheduler: {
					scheduleDays: [
							{
								dayOfWeek: "1",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "2",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "3",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "4",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "5",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "6",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							},
							{
								dayOfWeek: "7",
								endTime: "1970-01-01 00:00:00",
								startTime: "1970-01-01 00:00:00"
							}
					]
				},
				accounts: [
					{
						proxyType: "CALL",
						id: "elidCALL",
						phoneType: "builtin",
						outboundPrefix: "99",
						state: "1",
						status: {
							status: "OK"
						}
					},
					{
						"proxyType": "CHAT",
						"url_regexp": "[\"elgransite.com\"]",
						"autochat": "{\"url_regexp\":\"elgransite.com\"}",
						"id": "elgransite.com-chat",
						"Token": "Rocio_TOKEN",
						state: "1",

						status: {
							status: "OK"
						}
					},
					{
						proxyType: "SMS",
						id: "elidSMS",
						state: "1",
						status: {
							status: "OK"
						}
					},
					{
						type: "POP3",
						user: "inconcert6",
						id: "inconcert6@inconcertcc.com",
						proxyType: "MAIL",
						sendUrl: "smtp: //mail.inconcertcc.com",
						password: "cacho",
						encryption: "0",
						receiveUrl: "pop3: //mail.inconcertcc.com",
						defaultName: "Campania Fabrica inConcert 6",
						groupInThreads: "1",
						signature: "inConcert 6",
						authentication: "0",
						state: "1",
						status: {
							status: "OK"
						}
					},
					{
						password: "cacho",
						type: "IM",
						protocol: "prpl-yahoo",
						user: "inconcertbot",
						id: "inconcertbot@prpl-yahoo",
						proxyType: "IM",
						state: "1",
						status: {
							status: "OK"
						}
					},
					{
						proxyType: "FACEBOOK",
						id: "100001492544293",
						accessToken: "125846710792839|91560aa9fe4c20737233edd6.1-100001492544293|_opT5sHOetS2lr97zyNi61MUN1I",
						code: "3sQXhJBK9reU-BsI1rlCeWiEiFp7U7XNEIcRynUJTjM.eyJpdiI6IkZTUEFmNmxRUVBnMC1oRDdfZFlTTVEifQ.vOZPlgvohZOtVPPJG66oyUQBxJI-azszrvsIc3sPQXcv-7JPfeEwSkUSO8aZQHgmfsg9a5b5JCRRE4T06p56AUWRZkQfm3AePgS8OU-xyhcPf3OTOY_jrXic2qd_DpY8",
						homeMatchRules: "{'status':[],'video':[],'link':[]}",
						//id: "100001492544293",
						Id: "FACEBOOK_100001492544293",
						name: "Cany Prueba",
						onlyCheckPages: "1",
						pagesNames: "{\"page1\":\"pagina1\",\"page2\":\"pagina2\"}",
						pages: "{}",
						uid: "100001492544293",
						state: "1",
						status: {
							status: "OK"
						}
					},
					{
						id: "rumpelstink",
						Id: "TWITTER_rumpelstink",
						oauth_verifier: "Q3n4DPjcxiouZQ0hIaQQpZROageSVVgaEICCLXH7xcY",
						proxyType: "TWITTER",
						screen_name: "rumpelstink",
						tokenKey: "261412447-LSA2p3a67ZXtRv5WY5AJADx4M94QN2aDMlzaNgYL",
						tokenSecret: "VAoi8AG6qUF5Vl0hh901vqNIeSVtYoGgBHCTi2CcyI",
						user_id: "261412447",
						state: "1",
						status: {
							status: "OK"
						}
					}
				],
				ValidationEndTime: "01-01-2020",
				AlwaysValid: "1",
				Name: "productdevelopment",
				AfterInteractionWork: "0",
				status: "Active",
				defaultAgentCriteria: "SKILL",
				ValidationStartTime: "01-01-1970",
				VCC: "inconcert"
			}
		},
		campaign_routing_config: {},
		campaign_labels: {},
		campaign_canned_messages: {
			"productdevelopment@inconcert" : {
				contact_support : "Hello Mr Client, you can contact support via support@inconcertcc.com",
				contact_sales : "Hello Mr Client, you can contact sales team via sales@inconcertcc.com",
				contact_dev : "Hello Mr Client, you can contact dev team via dev@inconcertcc.com",
				nice_day : "Thanks for contacting us! Mr Client!"
			}
		},
		labels: {
			"PENDING": {
				"id": "PENDING",
				"description": "Pending",
				"system": "1"
			}/*,
			"INTEGRATION": {
				"id": "INTEGRATION",
				"description": "Integration",
				"system": "1"
			}*/
		},
		timezones: {
			"TMT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "TMT",
				"Description": "Turkmenistan Time"
			},
			"MSTMX": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-7",
				"DaylightSavingTime": "0",
				"Id": "MSTMX",
				"Description": "Chihuahua, La Paz, Mazatlan"
			},
				"MMT": {
				"TimeZoneMinutes": "30",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "MMT",
				"Description": "Myanmar Time"
				},
				"IRST": {
				"TimeZoneMinutes": "30",
				"TimeZone": "3",
				"DaylightSavingTime": "0",
				"Id": "IRST",
				"Description": "Iran Standard Time"
				},
				"IRKT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "IRKT",
				"Description": "Irkutsk Time"
				},
				"PIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "PIT",
				"Description": "Paracel Islands Time"
				},
				"SIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "SIT",
				"Description": "Spratly Islands Time"
				},
				"WAT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "1",
				"DaylightSavingTime": "0",
				"Id": "WAT",
				"Description": "Western Africa Time"
				},
				"DTAT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "10",
				"DaylightSavingTime": "0",
				"Id": "DTAT",
				"Description": "District de Terre Adelie Time"
				},
				"WIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "9",
				"DaylightSavingTime": "0",
				"Id": "WIT",
				"Description": "Waktu Indonesia Bagian Timur"
				},
				"GMT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "0",
				"DaylightSavingTime": "0",
				"Id": "GMT",
				"Description": "Greenwich Meantime"
				},
				"DAVT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "7",
				"DaylightSavingTime": "0",
				"Id": "DAVT",
				"Description": "Davis Time"
				},
				"MVT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "MVT",
				"Description": "Maldives Time"
				},
				"YEKT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "YEKT",
				"Description": "Yekaterinburg Time"
				},
				"PSTMX": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-8",
				"DaylightSavingTime": "0",
				"Id": "PSTMX",
				"Description": "Tijuana, Baja California"
				},
				"EAT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "3",
				"DaylightSavingTime": "0",
				"Id": "EAT",
				"Description": "East Africa Time"
				},
				"PMST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "PMST",
				"Description": "Pierre and Miquelon Standard Time"
				},
				"ACIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "ACIT",
				"Description": "Ashmore and Cartier Islands Time"
				},
				"BIOT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "BIOT",
				"Description": "British Indian Ocean Time"
				},
				"BTT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "BTT",
				"Description": "Bhutan Time"
				},
				"EKST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "EKST",
				"Description": "East Kazakhstan Standard Time"
				},
				"MSTMXSON": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-7",
				"DaylightSavingTime": "0",
				"Id": "MSTMXSON",
				"Description": "Hermosillo - Sonora"
				},
				"GFT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "GFT",
				"Description": "French Guiana Time"
				},
				"CHAST": {
				"TimeZoneMinutes": "45",
				"TimeZone": "12",
				"DaylightSavingTime": "0",
				"Id": "CHAST",
				"Description": "Chatham Island Standard Time"
				},
				"PONT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "11",
				"DaylightSavingTime": "0",
				"Id": "PONT",
				"Description": "Pohnpei Standard Time"
				},
				"SBT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "11",
				"DaylightSavingTime": "0",
				"Id": "SBT",
				"Description": "Solomon Island Time"
				},
				"TJT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "TJT",
				"Description": "Tajikistan Time"
				},
				"NOVT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "NOVT",
				"Description": "Novosibirsk Time"
				},
				"KOVT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "7",
				"DaylightSavingTime": "0",
				"Id": "KOVT",
				"Description": "Khovd Time"
				},
				"MBT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "MBT",
				"Description": "Macclesfield Bank Time"
				},
				"PYT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-4",
				"DaylightSavingTime": "0",
				"Id": "PYT",
				"Description": "Paraguay Time"
				},
				"MUT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "4",
				"DaylightSavingTime": "0",
				"Id": "MUT",
				"Description": "Mauritius Time"
				},
				"UYT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "UYT",
				"Description": "Uruguay Standard Time"
				},
				"BIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-12",
				"DaylightSavingTime": "0",
				"Id": "BIT",
				"Description": "Baker Island Time"
				},
				"ChST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "10",
				"DaylightSavingTime": "0",
				"Id": "ChST",
				"Description": "Chamorro Standard Time"
				},
				"TRUT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "10",
				"DaylightSavingTime": "0",
				"Id": "TRUT",
				"Description": "Truk Time"
				},
				"TKT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-10",
				"DaylightSavingTime": "0",
				"Id": "TKT",
				"Description": "Tokelau Time"
				},
				"AST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-4",
				"DaylightSavingTime": "0",
				"Id": "AST",
				"Description": "Atlantic Standard Time"
				},
				"CXT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "7",
				"DaylightSavingTime": "0",
				"Id": "CXT",
				"Description": "Christmas Island Time"
				},
				"FJT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "12",
				"DaylightSavingTime": "0",
				"Id": "FJT",
				"Description": "Fiji Time"
				},
				"PKT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "PKT",
				"Description": "Pakistan Time"
				},
				"CLST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-4",
				"DaylightSavingTime": "0",
				"Id": "CLST",
				"Description": "Chile Standard Time"
				},
				"BOT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-4",
				"DaylightSavingTime": "0",
				"Id": "BOT",
				"Description": "Bolivia Time"
				},
				"ACST": {
				"TimeZoneMinutes": "30",
				"TimeZone": "9",
				"DaylightSavingTime": "0",
				"Id": "ACST",
				"Description": "Australian Central Standard Time"
				},
				"JST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "9",
				"DaylightSavingTime": "0",
				"Id": "JST",
				"Description": "Japan Standard Time"
				},
				"IST": {
				"TimeZoneMinutes": "30",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "IST",
				"Description": "Indian Standard Time"
				},
				"SCT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "4",
				"DaylightSavingTime": "0",
				"Id": "SCT",
				"Description": "Seychelles Time"
				},
				"PGT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "10",
				"DaylightSavingTime": "0",
				"Id": "PGT",
				"Description": "Papua New Guinea Time"
				},
				"CCT": {
				"TimeZoneMinutes": "30",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "CCT",
				"Description": "Cocos Islands Time"
				},
				"EST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-5",
				"DaylightSavingTime": "0",
				"Id": "EST",
				"Description": "U.S. Eastern Standard Time"
				},
				"OMST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "OMST",
				"Description": "Omsk Standard Time"
				},
				"CST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-6",
				"DaylightSavingTime": "0",
				"Id": "CST",
				"Description": "U.S. Central Standard Time"
				},
				"CAT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "2",
				"DaylightSavingTime": "0",
				"Id": "CAT",
				"Description": "Central Africa Time"
				},
				"VUT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "11",
				"DaylightSavingTime": "0",
				"Id": "VUT",
				"Description": "Vanuatu Time"
				},
				"BRT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "BRT",
				"Description": "Brazilia Time"
				},
				"KRAT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "7",
				"DaylightSavingTime": "0",
				"Id": "KRAT",
				"Description": "Krasnoyarsk Time"
				},
				"PETT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "12",
				"DaylightSavingTime": "0",
				"Id": "PETT",
				"Description": "Petropavlovsk Time"
				},
				"CAST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "CAST",
				"Description": "Chinese Antarctic Standard Time"
				},
				"VOST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "6",
				"DaylightSavingTime": "0",
				"Id": "VOST",
				"Description": "Vostok Time"
				},
				"EAST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-6",
				"DaylightSavingTime": "0",
				"Id": "EAST",
				"Description": "Easter Island Standard Time"
				},
				"EET": {
				"TimeZoneMinutes": "0",
				"TimeZone": "2",
				"DaylightSavingTime": "0",
				"Id": "EET",
				"Description": "Eastern Europe Time"
				},
				"SAMT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "4",
				"DaylightSavingTime": "0",
				"Id": "SAMT",
				"Description": "Samara Time"
				},
				"MAGT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "11",
				"DaylightSavingTime": "0",
				"Id": "MAGT",
				"Description": "Magadan Island Time"
				},
				"KOST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "11",
				"DaylightSavingTime": "0",
				"Id": "KOST",
				"Description": "Kosrae Standard Time"
				},
				"ROTT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "ROTT",
				"Description": "Rothera Time"
				},
				"PET": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-5",
				"DaylightSavingTime": "0",
				"Id": "PET",
				"Description": "Peru Time"
				},
				"SCST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "12",
				"DaylightSavingTime": "0",
				"Id": "SCST",
				"Description": "Santa Claus Standard Time"
				},
				"ICT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "7",
				"DaylightSavingTime": "0",
				"Id": "ICT",
				"Description": "Indochina Time"
				},
				"SLT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-4",
				"DaylightSavingTime": "0",
				"Id": "SLT",
				"Description": "San Luis Time"
				},
				"MSTA": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-7",
				"DaylightSavingTime": "0",
				"Id": "MSTA",
				"Description": "U.S. Arizona Standard Time"
				},
				"NPT": {
				"TimeZoneMinutes": "45",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "NPT",
				"Description": "Nepal Time"
				},
				"BEST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-2",
				"DaylightSavingTime": "0",
				"Id": "BEST",
				"Description": "Brazil Eastern Standard Time"
				},
				"TPT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "9",
				"DaylightSavingTime": "0",
				"Id": "TPT",
				"Description": "East Timor Time"
				},
				"MSK": {
				"TimeZoneMinutes": "0",
				"TimeZone": "3",
				"DaylightSavingTime": "0",
				"Id": "MSK",
				"Description": "Moscow Standard Time"
				},
				"EGT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-1",
				"DaylightSavingTime": "0",
				"Id": "EGT",
				"Description": "Eastern Greenland Time"
				},
				"MHT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "12",
				"DaylightSavingTime": "0",
				"Id": "MHT",
				"Description": "Marshall Islands Time"
				},
				"KGT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "5",
				"DaylightSavingTime": "0",
				"Id": "KGT",
				"Description": "Kyrgyzstan Time"
				},
				"ART": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-3",
				"DaylightSavingTime": "0",
				"Id": "ART",
				"Description": "Argentina Time"
				},
				"WITA": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "WITA",
				"Description": "Waktu Indonesia Bagian Tengah"
				},
				"ACWST": {
				"TimeZoneMinutes": "45",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "ACWST",
				"Description": "Australian Central Western Standard Time"
				},
				"GIT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "-9",
				"DaylightSavingTime": "0",
				"Id": "GIT",
				"Description": "Gambier Island Time"
				},
				"RET": {
				"TimeZoneMinutes": "0",
				"TimeZone": "4",
				"DaylightSavingTime": "0",
				"Id": "RET",
				"Description": "Reunion Time"
				},
				"AWST": {
				"TimeZoneMinutes": "0",
				"TimeZone": "8",
				"DaylightSavingTime": "0",
				"Id": "AWST",
				"Description": "Australian Western Standard Time"
				},
				"LINT": {
				"TimeZoneMinutes": "0",
				"TimeZone": "14",
				"DaylightSavingTime": "0",
				"Id": "LINT",
				"Description": "Line Island Time"
				}
			}
	};

	// FIX-ME: This is a hack
	m_loggedUser = database.users['admin@inconcert'];

	var twitter = {
		twitterRelationShips: {
			"rumpelstink": { //this one will follow me
				"cany74" : {
					"relationship": {
						"source": {
							"notifications_enabled": false,
							"id": 1,
							"followed_by": true, //igual
							"want_retweets": true,
							"screen_name": "rumpelstink",
							"all_replies": false,
							"blocking": false,
							"marked_spam": false,
							"following": true,
							"id_str": "1",
							"can_dm": true //igual
						},
						"target": {
							"screen_name": "cany74",
							"id_str": "2",
							"following": true, //igual
							"followed_by": true,
							"id": "2",
							"blocking": false
						},
						"result":"OK"
					}
				},
				"nacho" : { //this one won't follow me
					"relationship": {
						"source": {
							"notifications_enabled": false,
							"id": 1,
							"followed_by": false, //igual
							"want_retweets": true,
							"screen_name": "nacho",
							"all_replies": false,
							"blocking": false,
							"marked_spam": false,
							"following": true,
							"id_str": "1",
							"can_dm": false //igual
						},
						"target": {
							"screen_name": "nacho",
							"id_str": "3",
							"following": false, //igual
							"followed_by": true,
							"id": "2",
							"blocking": true
						},
						"result":"OK"
					}
				}
			}
		},
		twitterUsers: {
			inConcertSeba: {
				favourites_count: 136,
				followers_count: 76,
				friends_count: 151,
				statuses_count: 1313
			},

			guilespi: {
				favourites_count: 987,
				followers_count: 116,
				friends_count: 133,
				statuses_count: 991
			},
			cany74: {
				favourites_count: 22,
				followers_count: 12,
				friends_count: 51,
				statuses_count: 313
			},
			rumpelstink: {
				favourites_count: 11,
				followers_count: 111,
				friends_count: 888,
				statuses_count: 4463
			}

		},
		follow: function(info) {
			if (!twitter.twitterRelationShips[info.account]) {
				return false;
			}
			if (!twitter.twitterRelationShips[info.account][info.userId]) {
				return false;
			}
			twitter.twitterRelationShips[info.account][info.userId].relationship.target.followed_by = true;
			return true;
		},
		unfollow: function(info) {
			if (!twitter.twitterRelationShips[info.account]) {
				return false;
			}
			if (!twitter.twitterRelationShips[info.account][info.userId]) {
				return false;
			}
			twitter.twitterRelationShips[info.account][info.userId].relationship.target.followed_by = false;
			return true;
		},
		blockExists: function(info) {
			if (!twitter.twitterRelationShips[info.account]) {
				return false;
			}
			if (!twitter.twitterRelationShips[info.account][info.userId]) {
				return false;
			}
			if (twitter.twitterRelationShips[info.account][info.userId].relationship.target.blocking){
				return { blocked_user : info.userId };
			}
			else{
				return true;
			}
		},
		blockUser: function(info) {
			if (!twitter.twitterRelationShips[info.account]) {
				return false;
			}
			if (!twitter.twitterRelationShips[info.account][info.userId]) {
				return false;
			}
			twitter.twitterRelationShips[info.account][info.userId].relationship.target.blocking = true;
			return true;
		},
		unblockUser: function(info) {
			if (!twitter.twitterRelationShips[info.account]) {
				return false;
			}
			if (!twitter.twitterRelationShips[info.account][info.userId]) {
				return false;
			}
			twitter.twitterRelationShips[info.account][info.userId].relationship.target.blocking = false;
			return true;
		},
		getRelationShip: function(info) {
			if (twitter.twitterRelationShips[info.account]) {
				if (twitter.twitterRelationShips[info.account][info.to_account]) {
					return twitter.twitterRelationShips[info.account][info.to_account];
				}
			}
			else {
				twitter.twitterRelationShips[info.account] = {};
			}
			twitter.twitterRelationShips[info.account][info.to_account] = {
				"relationship": {
					"source": {
						"notifications_enabled": false,
						"id": 1,
						"followed_by": true, //igual
						"want_retweets": true,
						"screen_name": info.account,
						"all_replies": false,
						"blocking": false,
						"marked_spam": false,
						"following": true,
						"id_str": "1",
						"can_dm": true //igual
					},
					"target": {
						"screen_name": info.to_account,
						"id_str": "2",
						"following": true, //igual
						"followed_by": true,
						"id": "2"
					},
					"result":"OK"
				}
			};
			return twitter.twitterRelationShips[info.account][info.to_account];
		},
		getTwitterUser: function(screen_name) {
			if (twitter.twitterUsers[screen_name]) {
				return twitter.twitterUsers[screen_name];
			}
			else{
				throw("There is no twitter user with that id");
			}
		}
	};

	var models = {
		users: {
			findUserByEmail: function(emailAddress, vcc) {
				var userList = inConcert.app.database.usersbyemail[emailAddress];
				if (!userList) {
					return false;
				}
				for (var userId in userList) {
					if (userList.hasOwnProperty(userId)) {
						var match = userId.match(new RegExp("([^@]+)@([^@]+)"));
						var vccMail = match[2];
						if (vccMail === vcc) {
							return userId;
						}
					}
				}
				return false;
			},
			findUserByPeer: function(peerId) {
				if (!inConcert.app.database.usersbypeer[peerId]) {
					return {};
				}
				return inConcert.app.database.usersbypeer[peerId];
			},
			userExists: function(userId) {
				var user = inConcert.app.database.users[userId];
				return typeof(user) !== "undefined";
			},
			getVccUsers: function(vcc) {
				return inConcert.app.database.vccusers[vcc];
			},
			get: function(userId) {
				if (!userId) {
					return;
				}
				var user = inConcert.app.database.users[userId];
				if (!user) {
					return;
				}
				user.addresses = inConcert.app.database.agentaddresses[userId];
				return user;
			},
			saveWorkgroups: function(userId, campaignId, workgroups) {
				if (!userId || !campaignId) {
					return false;
				}
				var user = inConcert.app.database.users[userId];
				if (!user) {
					return false;
				}
				if (!user.SupervisorWorkgroups) {
					user.SupervisorWorkgroups = {};
				}
				user.SupervisorWorkgroups[campaignId] = workgroups;
				return true;
			},
			save: function(user) {
				/*
				* Users (column)
				* VCCUsers (column)
				* UsersByEmail (column)
				*
				* UserCampaigns (column)
				* CampaignMembers (column)
				AgentAddresses (supercolumn)
				-- si es call
				UsersByPeer (column)
				AgentSkills (column)
				*/

				var vcc = "inconcert";
				var userWithVCC = user.Id + "@" + vcc;

				// guardo en la column family de Users -------------------------------
				if (!inConcert.app.database.users[userWithVCC]) {
					inConcert.app.database.users[userWithVCC] = {};
				}
				for (var columnName in user) {
					if (user.hasOwnProperty(columnName)) {
						var columnValue = user[columnName];
						if (typeof(columnValue) === "string") {
							inConcert.app.database.users[userWithVCC][columnName] = columnValue;
						}
					}
				}

				// guardo en la column family de VCCUsers -------------------------------
				if (!inConcert.app.database.vccusers[vcc]) {
					inConcert.app.database.vccusers[vcc] = {};
				}
				inConcert.app.database.vccusers[vcc][user.Id] = user.Name;

				// guardo en la column family de UsersByMail -------------------------------
				if (!inConcert.app.database.usersbyemail[user.Mail]) {
					inConcert.app.database.usersbyemail[user.Mail] = {};
				}
				inConcert.app.database.usersbyemail[user.Mail][userWithVCC] = "";

				// guardo las campanas en las columns familys de userCampaigns y campaignMembers
				if (!$.isArray(user.groups) || user.groups.length === 0) {
					inConcert.app.database.userCampaigns[userWithVCC] = {};
				}
				else {
					inConcert.app.database.userCampaigns[userWithVCC] = {};
					for (var index in user.groups) {
						if (user.groups.hasOwnProperty(index)) {
							var group = user.groups[index];
							if (typeof(group.role) === "string") {
								var campaignId = group.id;
								models.campaigns.addMember(campaignId, userWithVCC, group.role);
							}
						}
					}
				}

				// guardo los addresses de los users y en usersbypeer
				if (user.addresses) {
					var agentAddresses = inConcert.app.database.agentaddresses;
					if (!agentAddresses[userWithVCC]) {
						agentAddresses[userWithVCC] = {};
					}
					for (var proxyType in user.addresses) {
						if (user.addresses.hasOwnProperty(proxyType)) {
							if (!agentAddresses[userWithVCC][proxyType]) {
								agentAddresses[userWithVCC][proxyType] = {};
							}
							var address = user.addresses[proxyType];
							if (typeof(address) === "object") {
								for (var column in address) {
									if (address.hasOwnProperty(column)) {
										var value = address[column];
										if (typeof(value) === "string") {
											// guardo en la super column family de AgentAddresses -------------
											agentAddresses[userWithVCC][proxyType][column] = value;
										}
									}
								}
								if (proxyType === "CALL") {
									if (address.getId) {
										var peerId = address.getId();
										// guardo en la column family de usersbypeer ---------------
										if (!inConcert.app.database.usersbypeer[peerId]) {
											inConcert.app.database.usersbypeer[peerId] = {};
										}
										inConcert.app.database.usersbypeer[peerId][userWithVCC] = "";
									}
								}
							}
						}
					}
					inConcert.app.database.agentaddresses = agentAddresses;
				}

				// guardo los skills del user
				if (user.skills) {
					inConcert.app.database.agentskills[userWithVCC] = {};
					for (index in user.skills) {
						if (user.skills.hasOwnProperty(index)) {
							var userSkill = user.skills[index];
							// guardo en la column family de agentskills ---------------
							inConcert.app.database.agentskills[userWithVCC][userSkill.skill] = userSkill.weight;
						}
					}
				}
				return true;
			}
		},
		peers: {
			getPeer: function(peerId) {
				if (!peerId) {
					return {};
				}
				if (!inConcert.app.database.peers[peerId]) {
					return {};
				}
				return inConcert.app.database.peers[peerId];
			}
		},
		campaigns: {
			_getAccountId: function(account) {
				if (!account) {
					return;
				}
				if (account.proxyType === "CALL") {
					return "CALL_" + account.id;
				}
				else if (account.proxyType === "IM") {
					return "IM_" + account.id;
				}
				else if (account.proxyType === "MAIL") {
					return "EMAIL_" + account.id;
				}
				else if (account.proxyType === "CHAT") {
					return "CHAT_" + account.id;
				}
				else if (account.proxyType === "FACEBOOK") {
					return "FACEBOOK_" + account.id;
				}
				else if (account.proxyType === "SMS") {
					return "SMS_" + account.id;
				}
				else if (account.proxyType === "TWITTER") {
					return "TWITTER_" + account.id;
				}
				return;
			},
			get: function(campaignId) {
				return inConcert.app.database.campaigns[campaignId];
			},
			save: function(campaign) {
				if (!campaign) {
					return false;
				}
				for (var name in campaign) {
					if (campaign.hasOwnProperty(name)) {
						var value = campaign[name];
						if (typeof(value) !== "object") {
							inConcert.app.database.campaigns[campaign.Id] = inConcert.app.database.campaigns[campaign.Id] || {};
							inConcert.app.database.campaigns[campaign.Id][name] = value;
						}
					}
				}
				var match = campaign.Id.match(new RegExp("([^@]+)@([^@]+)"));
				var campaignId = match[1];
				var campaignVCC = match[2];
				inConcert.app.database.vccCampaigns[campaignVCC] = inConcert.app.database.vccCampaigns[campaignVCC] || {};
				inConcert.app.database.vccCampaigns[campaignVCC][campaignId] = campaign.Name;

				var currentAccounts = {};
				if (campaign.accounts) {
					for (var i = 0; i < campaign.accounts.length; i++) {
						var account = campaign.accounts[i];
						var accountId = models.campaigns._getAccountId(account);
						if (!accountId) {
							continue;
						}
						currentAccounts[accountId] = true;
						//PrepareAccountForWrite (id, account)
						inConcert.app.database.campaignAccounts[campaign.Id] = inConcert.app.database.campaignAccounts[campaign.Id] || {};
						inConcert.app.database.campaignAccounts[campaign.Id][accountId] = inConcert.app.database.campaignAccounts[campaign.Id][accountId] || {};
						inConcert.app.database.campaignAccounts[campaign.Id][accountId].Id = accountId;
						for (var name in account) {
							if (account.hasOwnProperty(name)) {
								var value = account[name];
								if (typeof(value) === "object") {
									var value = JSON.stringify(value);
								}
								inConcert.app.database.campaignAccounts[campaign.Id][accountId][name] = value;
							}
						}
						inConcert.app.campaignsAccountsUpdated[campaign.Id] = inConcert.app.campaignsAccountsUpdated[campaign.Id] === undefined ? [] : inConcert.app.campaignsAccountsUpdated[campaign.Id];
						inConcert.app.campaignsAccountsUpdated[campaign.Id].push(account);
					}
				}
				if (campaign.scheduler) {
					delete inConcert.app.database.campaignSchedulers[campaign.Id];
					if (campaign.scheduler.scheduleDays) {
						for (var key in campaign.scheduler.scheduleDays) {
							if (campaign.scheduler.scheduleDays.hasOwnProperty(key)) {
								var scheduleDay = campaign.scheduler.scheduleDays[key];
								var schedKey = scheduleDay.dayOfWeek +"_" + scheduleDay.startTime +"_" + scheduleDay.endTime;
								inConcert.app.database.campaignSchedulers[campaign.Id] = inConcert.app.database.campaignSchedulers[campaign.Id]|| {};
								inConcert.app.database.campaignSchedulers[campaign.Id][schedKey] = "";
							}
						}
					}
				}
				var currentCampaign = models.campaigns.get(campaign.Id);
				if (currentCampaign) {
					for (var i = 0; i < currentCampaign.accounts.length; i++) {
						var account = campaign.accounts[i];
						if (account.Id && !currentAccounts[account.Id]) {
							delete inConcert.app.database.campaignAccounts[campaign.Id][accountId];
						}
					}
					//NotifyCampaignChanged(id, "UPDATE")
				}
				return true;
			},
			getUserCampaigns: function(user, vcc) {
				var userId = user + "@" + vcc;
				return inConcert.app.database.userCampaigns[userId];
			},
			isMember: function(campaignAtVCC, userAtVCC) {
				var campaignMembers = inConcert.app.database.campaignMembers[campaignAtVCC];
				if (!campaignMembers) {
					return false;
				}
				return campaignMembers[userAtVCC];
			},
			addMember: function(campaignAtVCC, userAtVCC, role) {
				// guardo en la column family de userCampaigns -------------------------------
				if (!inConcert.app.database.userCampaigns[userAtVCC]) {
					inConcert.app.database.userCampaigns[userAtVCC] = {};
				}
				inConcert.app.database.userCampaigns[userAtVCC][campaignAtVCC] = role;
				// guardo en la column family de campaignMembers -------------------------------
				if (!inConcert.app.database.campaignMembers[campaignAtVCC]) {
					inConcert.app.database.campaignMembers[campaignAtVCC] = {};
				}
				inConcert.app.database.campaignMembers[campaignAtVCC][userAtVCC] = role;
			},
			getCampaignMembers: function(campaignAtVCC) {
				return inConcert.app.database.campaignMembers[campaignAtVCC];
			}
		}
	};

	var provisioningmanager = {
		listPeersByVCC: function() {
			return inConcert.app.database.vccpeers;
		}
	};

	var apps = {
		installer : {
			controllers : {
				resources : {
					"update-descriptor.xml" : function() {
						var xml ='<?xml version="1.0" encoding="UTF-8"?>';
						xml +=	'<application>';
						xml +=		'<latestversion>5.0.0</latestversion>';
						xml +=		'<releasenotes>';
						xml +=		'</releasenotes>';
						xml +=	'</application>';
						var xmlObject = new DOMParser().parseFromString(xml, "text/xml");
						return xmlObject;
					}
				}
			}
		},
		chat : {
			login : function() {
				return { status :true,
							userId : m_loggedUser.Id
					};
			}
		},
		repository: {
			add: function() {
				return {
					status: 200
				};
			},
			"delete": function() {
				return true;
			},
			multiplesdelete: function() {
				return true;
			},
			get : function(queryString, postedData) {
				var match = queryString.match(new RegExp("id=(.*?)&"));
				var interaction = inConcert.app.interactionList[match[1]];
				return interaction ? JSON.stringify(interaction.session) : false;
			}
		},
		administration: {
			controllers: {

				campaigns: {

					get: function(queryString, postedData) {
						var id = queryString;
						var match = id.match(new RegExp("([^@]+)@(.*)"));
						id =  ((match ? match[1] : id) + "@" + m_loggedUser.VCC).toLowerCase();
						return inConcert.app.database.campaigns[id];
					},
					save_all: function(queryString, postedData) {
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id, name;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							name = key;
							id = key;
							entity = list[key];
							entity.Id = id;
							var full_id = id + "@" + m_loggedUser.VCC.toLowerCase();
							if (entity.isNew && inConcert.app.database.campaigns[full_id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"Campaign must have an unique name"}'
								});
							}
							delete entity.isNew;
							response.idMapping[name] = id;
							break;
						}}
						var encoded_campaign = JSON.stringify(entity);
						apps.agent.controllers.campaigns.create(queryString, encoded_campaign);
						return response;
					},
					save: function(queryString, postedData) {
						//var response = {result: true, campaign: null};
						var campaign = JSON.parse(postedData);
						var full_id = campaign.Id + "@" + m_loggedUser.VCC.toLowerCase();
						if (campaign.isNew && inConcert.app.database.campaigns[full_id]) {
							throw({
								webStatus: '400',
								textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"Campaign must have an unique name"}'
							});
						}
						delete campaign.isNew;
						//response.idMapping[name] = id;

						var encoded_campaign = JSON.stringify(campaign);
						var response = apps.agent.controllers.campaigns.create(queryString, encoded_campaign);
						return response;
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						//var decoded = JSON.parse(postedData);

						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";

						var match = id.match(new RegExp("([^@]+)@(.*)"));

						var campaign = {
							campaignId: ((match ? match[1] : id) + "@" + m_loggedUser.VCC).toLowerCase()
						};
						var encoded_campaign = JSON.stringify(campaign);
						apps.agent.controllers.campaigns["delete"](queryString, encoded_campaign);
						response.success = true;
						return response;
					},
					search: function(queryString, postedData) {
						var result = {
							rows: [],
							last: "last"
						};
						var campaigns = inConcert.app.database.campaigns;
						for (var key in campaigns) {
							if (campaigns.hasOwnProperty(key)) {
								var campaign = campaigns[key];
								result.rows.push(campaign);
							}
						}
						result.total_rows = result.rows.length;
						return result;
					},


					// metodos del API replicados contra el controllers de agent
					exists: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.exists(queryString, postedData);
					},

					save_campaign_messages: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (!request || !request.campaignId) {
							throw({webStatus: "400", textStatus: {"result":"INVALID_REQUEST"," reason":"Invalid request - Field campaignId is missing"}});
						}
						if (!request.rule) {
							throw({webStatus: "400", textStatus: {"result":"INVALID_REQUEST"," reason":"Invalid request - Field rule is missing"}});
						}
						if (!inConcert.app.database.campaigns[request.campaignId]) {
							throw({webStatus: "500", textStatus: {"result":"UNABLE_TO_SAVE"," reason":"Campaign does not exist"}});
						}
						return apps.agent.controllers.campaigns.save_routing_config(queryString, postedData);
					},
					get_campaign_messages: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_routing_config(queryString, JSON.stringify({campaignId: queryString}));
					},
					get_members: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_members(queryString, postedData);
					},
					save_disposition_codes: function(queryString, postedData, campaignId) {
						var request = JSON.parse(postedData);
						if (!request || !request.dispositionCodes) {
							throw({webStatus: "400", textStatus: {"result":"INVALID_REQUEST"," reason":"Invalid request"}});
						}
						if (!inConcert.app.database.campaigns[request.campaignId]) {
							throw({webStatus: "500", textStatus: {"result":"UNABLE_TO_SAVE"," reason":"Campaign does not exist"}});
						}
						inConcert.app.database.campaigns[request.campaignId].dispositionCodes = request.dispositionCodes;
						return "";
					},
					save_labels: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.save_labels(queryString, postedData);
					},
					delete_labels: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.delete_labels(queryString, postedData);
					},
					get_labels: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_labels(queryString, postedData);
					},
					get_members_summary : function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_members_summary(queryString, postedData);
					},
					get_campaign_members: function(queryString, postedData) {
						postedData = JSON.stringify({
							campaignId: queryString
						});
						return apps.agent.controllers.campaigns.get_campaign_members(queryString, postedData);
					},
					get_member_archive: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_member_archive(queryString, postedData);
					},
					get_campaign_archive : function(queryString, postedData) {
						return apps.agent.controllers.campaigns.get_campaign_archive(queryString, postedData);
					},
					update_members_list : function(quertyString, postedData) {
						var request = JSON.parse(postedData);
						var members = request.members;
						inConcert.app.database.campaignMembers[request.campaignId] = {};
						for (var i = 0; i < members.length; i++) {
							var member = members[i];
							var match = member.id.match(new RegExp("([^@]+)@(.*)"));
							var member_id = ((match ? match[1] : member.id) + "@" + m_loggedUser.VCC).toLowerCase();
							inConcert.app.database.campaignMembers[request.campaignId][member_id] = member.role;
							if (member.role.toLowerCase() === "supervisor") {
								models.users.saveWorkgroups(member_id, request.campaignId, member.supervisor_workgroups);
							}
						}
						return true;
					},
					set_available: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.set_available(queryString, postedData);
					},
					add_account: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.add_account(queryString, postedData);
					},
					update_account: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.update_account(queryString, postedData);
					},
					remove_account: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.remove_account(queryString, postedData);
					},
					change_account_state: function(queryString, postedData) {
						return apps.agent.controllers.campaigns.change_account_state(queryString, postedData);
					},
					get_canned_messages: function(queryString, postedData) {
						return inConcert.app.database.campaign_canned_messages["productdevelopment@inconcert"];
					},
					save_scheduler: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (!request || !request.scheduler) {
							throw({webStatus: "400", textStatus: {"result":"INVALID_REQUEST"," reason":"Invalid request"}});
						}
						var campaignId = request.campaignId;
						var scheduler = request.scheduler;
						if (!inConcert.app.database.campaigns[campaignId]) {
							throw({webStatus: "500", textStatus: {"result":"UNABLE_TO_SAVE"," reason":"Campaign does not exist"}});
						}
						inConcert.app.database.campaigns[campaignId].scheduler = request.scheduler;
						delete inConcert.app.database.campaignSchedulers[campaignId];
						if (scheduler.scheduleDays) {
							for (var key in scheduler.scheduleDays) {
								if (scheduler.scheduleDays.hasOwnProperty(key)) {
									var scheduleDay = scheduler.scheduleDays[key];
									var schedKey = scheduleDay.dayOfWeek +"_" + scheduleDay.startTime +"_" + scheduleDay.endTime;
									inConcert.app.database.campaignSchedulers[campaignId] = inConcert.app.database.campaignSchedulers[campaignId]|| {};
									inConcert.app.database.campaignSchedulers[campaignId][schedKey] = "";
								}
							}
						}
						return "";
					}
				},

				users: {
					activate: function(queryString, postedData) {
						var data = JSON.parse(postedData);
						var id = data.memberId;
						var match = id.match(new RegExp("([^@]+)@(.*)"));
						if (!match) {
							id = id + "@" + m_loggedUser.VCC;
						}
						var user = inConcert.app.database.users[id];

						if (!user) {
							throw({
								webStatus: '400',
								textStatus: '{"result":"USER_DOES_NOT_EXIST"," reason":"User Id does not exist."}'
							});
						}
						user.Status = "1";
						return {
							result: true
						};
					},

					deactivate: function(queryString, postedData) {
						var data = JSON.parse(postedData);
						var id = data.memberId;
						var match = id.match(new RegExp("([^@]+)@(.*)"));
						if (!match) {
							id = id + "@" + m_loggedUser.VCC;
						}
						var user = inConcert.app.database.users[id];
						if (!user) {
							throw({
								webStatus: '400',
								textStatus: '{"result":"USER_DOES_NOT_EXIST"," reason":"User Id does not exist."}'
							});
						}
						user.Status = "0";
						return {
							result: true
						};
					},

					get: function(queryString, postedData) {
						var id = queryString;

						var match = id.match(new RegExp("([^@]+)@(.*)"));
						if (!match) {
							id = id + "@" + m_loggedUser.VCC;
						}

						var user = inConcert.app.database.users[id];
						user.custom_permissions = inConcert.app.database.userpermission[id] || {};
						user.skills = inConcert.app.database.agentskills[id] || {};
						user.user_campaigns = inConcert.app.database.userCampaigns[id] || {};
						return user;
					},

					save: function(queryString, postedData)
					{
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id, name;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							name = key;
							id = key;
							var match = id.match(new RegExp("([^@]+)@(.*)"));
							if (!match) {
								id = id + "@" + m_loggedUser.VCC;
							}
							entity = list[key];
							if (entity.isNew && inConcert.app.database.users[id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"User must have an unique name"}'
								});
							}
							delete entity.isNew;
							response.idMapping[name] = id;
							break;
						}}
						entity.VCC = m_loggedUser.VCC;
						entity.userRole = entity.userRole || "administrator";
						var encoded_user = JSON.stringify(entity);
						apps.agent.controllers.users.save(queryString, encoded_user);

						return response;
					},

					search: function(queryString, postedData) {
						var data = JSON.parse(postedData);
						var result = {
							rows: [],
							last: "last"
						};
						var users = inConcert.app.database.users;
						for (var key in users) {
							if (users.hasOwnProperty(key)) {
								var user = users[key];
								if (data.query.Enabled === "" || data.query.Enabled === user.Status) {
									result.rows.push(user);
								}
							}
						}
						result.total_rows = result.rows.length;
						return result;
					}
				},

				groups: {
					get: function(queryString, postedData) {
						var id = queryString;
						var group = inConcert.app.database.groups[id];
						var full_id = id + "@" + m_loggedUser.VCC;
						group.grouppermissions = inConcert.app.database.grouppermission[full_id];
						return group;
					},
					save: function(queryString, postedData) {
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							id = key;
							entity = list[id];
							if (entity.isNew && inConcert.app.database.groups[id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"Group must have an unique name"}'
								});
							}
							response.idMapping[id] = id;
							break;
						}}
						entity.vcc = m_loggedUser.VCC;
						entity.IsSystemGroup =  '0';
						var full_id = id + "@" + entity.vcc;
						inConcert.app.database.grouppermission[full_id] = entity.grouppermissions;
						entity.grouppermissions = undefined;
						inConcert.app.database.groups[id] = entity;
						return response;
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";
						if (!inConcert.app.database.groups[id]) {
							return response;
						}
						delete inConcert.app.database.groups[id];
						var full_id = id + "@" + m_loggedUser.VCC;
						delete inConcert.app.database.grouppermission[full_id];
						response.success = true;
						return response;
					},
					get_system_groups: function(queryString, postedData){
						var system_roles = {};
						for (var key in inConcert.app.database.groups) {
							if (inConcert.app.database.groups.hasOwnProperty(key)) {
								var role = inConcert.app.database.groups[key];
								// esto no valida con jslint, pero no importa, esta as? para que entre ya sea que es numero o texto
								if (role.IsSystemGroup == 1) {
									system_roles[key] = role;
									var full_id = role.Id + "@system";
									system_roles[key].grouppermissions = inConcert.app.database.grouppermission[full_id] || {};
								}
							}
						}
						return system_roles;
					},
					get_all_groups: function(queryString, postedData){
						var system_roles = {};
						for (var key in inConcert.app.database.groups) {
							if (inConcert.app.database.groups.hasOwnProperty(key)) {
								var role = inConcert.app.database.groups[key];
								// esto no valida con jslint, pero no importa, esta as? para que entre ya sea que es numero o texto
								var full_id = role.Id;
								if (role.IsSystemGroup == 1) {
									full_id = full_id + "@system";
								}
								else {
									full_id = full_id + "@" + m_loggedUser.VCC;
								}

								system_roles[key] = role;
								system_roles[key].grouppermissions = inConcert.app.database.grouppermission[full_id] || {};
							}
						}
						return system_roles;
					},

					find_all: function(queryString, postedData){
						return inConcert.app.database.groups;
					},
					search: function(queryString, postedData) {
						var result = {
							rows: [],
							last: "last"
						};
						var roles = inConcert.app.database.groups;
						for (var key in roles) {
							if (roles.hasOwnProperty(key)) {
								var role = roles[key];
								// esto no valida con jslint, pero no importa, esta as? para que entre ya sea que es numero o texto
								if (role.IsSystemGroup != 1) {
									result.rows.push(role);
								}
							}
						}
						result.total_rows = result.rows.length;
						return result;
					}
				},

				states: {
					get: function(queryString, postedData) {
						var id = queryString;
						return inConcert.app.database.availableStates[id];
					},
					save: function(queryString, postedData) {
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							id = key;
							entity = list[id];
							if (entity.isNew && inConcert.app.database.availableStates[id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"State must have an unique name"}'
								});
							}
							delete entity.isNew;
							entity.Id = key;
							response.idMapping[id] = id;
							break;
						}}
						inConcert.app.database.availableStates[id] = entity;
						return response;
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";
						if (!inConcert.app.database.availableStates[id]) {
							return response;
						}
						delete inConcert.app.database.availableStates[id];
						response.success = true;
						return response;
					},
					find_all: function(queryString, postedData){
						return inConcert.app.database.availableStates;
					},
					search: function(queryString, postedData) {
						var result = {
							rows: [],
							last: "last"
						};
						var states = inConcert.app.database.availableStates;
						for (var key in states) {
							if (states.hasOwnProperty(key)) {
								var state = states[key];
								state.Id = key;
								result.rows.push(state);
							}
						}
						result.total_rows = result.rows.length;
						return result;
					}
				},

				workgroups: {
					get: function(queryString, postedData) {
						var id = queryString;
						return inConcert.app.database.workgroups[id];
					},
					save: function(queryString, postedData) {
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							id = key;
							entity = list[id];
							if (entity.isNew && inConcert.app.database.workgroups[id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"Workgroup must have an unique name"}'
								});
							}
							delete entity.isNew;
							entity.Id = key;
							response.idMapping[id] = id;
							break;
						}}
						inConcert.app.database.workgroups[id] = entity;
						return response;
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";
						if (!inConcert.app.database.workgroups[id]) {
							return response;
						}
						delete inConcert.app.database.workgroups[id];
						response.success = true;
						return response;
					},
					search: function(queryString, postedData) {
						var result = {
							rows: [],
							last: "last"
						};
						var workgroups = inConcert.app.database.workgroups;
						for (var key in workgroups) {
							if (workgroups.hasOwnProperty(key)) {
								var workgroup = workgroups[key];
							//	workgroup.Name = key;
								result.rows.push(workgroup);
							}
						}
						result.total_rows = result.rows.length;
						return result;
					}
				},

				skills: {
					get: function(queryString, postedData) {
						var id = queryString;
						var skill = inConcert.app.database.skills[id];
						return skill;
					},
					save: function(queryString, postedData) {
						var response = {result: true, idMapping: {}};
						var list = JSON.parse(postedData);
						var entity;
						var id;
						// este loop esta aca para quedarme con el primer elemento de la lista!!
						for (var key in list) { if (list.hasOwnProperty(key)) {
							id = key;
							entity = list[id];
							if (entity.isNew && inConcert.app.database.skills[id]) {
								throw({
									webStatus: '400',
									textStatus: '{"result":"MUST_HAVE_UNIQUE_NAME","reason":"Skill must have an unique name"}'
								});
							}
							delete entity.isNew;
							entity.Id = key;
							response.idMapping[id] = id;
							break;
						}}
						inConcert.app.database.skills[id] = entity;
						return response;
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";
						if (!inConcert.app.database.skills[id]) {
							return response;
						}
						delete inConcert.app.database.skills[id];
						response.success = true;
						return response;
					},
					search: function(queryString, postedData) {
						var result = {
							rows: [],
							last: "last"
						};
						var skills = inConcert.app.database.skills;
						for (var key in skills) {
							if (skills.hasOwnProperty(key)) {
								var skill = skills[key];
								result.rows.push(skill);
							}
						}
						result.total_rows = result.rows.length;
						return result;
					}
				},

				permissions: {
					get_application_actions: function(queryString, postedData) {
						return inConcert.app.database.application;
					},
					get_permissions: function(queryString, postedData) {
						return true;
					}
				}
			}
		},
		agent : {
			controllers : {
				login: {
					getToken: function() {
						return {
							sessionId: 3,
							loginToken: "74aaa873-8b11-408c-ad86-db6888dcb315"
						};
					},
					logout: function(queryString, postedData){

					},
					authenticate: function(queryString, postedData){
							//posted data has the form "hash=74fc3263d6826d2c8aff08d9fb038626&username=admin&sessionid=3&callcenter=inconcert&app=agent"
							var parameters = xFormToJavascript(postedData);
							if (parameters.hash && parameters.username && parameters.callcenter && parameters.loginToken) {
								var userId = parameters.username + "@" + parameters.callcenter;
								var user = inConcert.app.database.users[userId];
								if (user) {
									if (user.password === parameters.hash) {
										m_loggedUser = user;
										inConcert.app.loggedUserApi = m_loggedUser;
										m_loggedUser.Id = userId;
										return {
											status: true,
											applicationPage: "/inconcert/apps/dashboard/",
											message: "Login succeeded",
											authToken: "f3d1b0e7-d91e-4ddf-b73c-b4ea3c95644d"
										};
									}
									else {
										return {
											status: false,
											message: "Invalid password"
										};
									}
								}
								else {
									return {
										status: false,
										message: "User does not exists"
									};
								}
							}
							else {
								throw ("Malformed request");
							}
						}
				},

				states: {
					find_all : function(queryString, postedData) {
						return inConcert.app.database.availableStates;
					}
				},

				auto_attendant : {
					get : function(queryString, postedData) {
						var match = queryString.match(new RegExp("([^/]+)/(.*)"));
						var proxyType = match[1];
						var addressID = match[2];
						var key = proxyType + "_" + addressID;
						if (database.auto_attendants[key]) {
							return database.auto_attendants[key];
						}
					},
					save : function(queryString, postedData) {

					}
				},
				packages : {
					get : function(packageType) {
						if (database.packages && database.packages[packageType]) {
							return database.packages[packageType];
						}
						else {
							return {};
						}
					}
				},
				pages : {
					"newGroupWizard.html" : function() {
					},
					"customMessagesConfig.html" : function() {
					}
				},
				labels : {
					system: function(queryString, postedData) {
						return inConcert.app.database.labels;
					}
				},
				timezones: {
					find_all: function(queryString, postedData) {
						return inConcert.app.database.timezones;
					}
				},
				campaigns : {
					save: function () {
						return "";
					},
					exists: function(queryString, postedData) {
						//function returns if reserved or not
						var campaignId = queryString + "@" + m_loggedUser.VCC;
						var reserved = true;
						if (inConcert.app.database.campaigns[campaignId]) {
							reserved = false;
						}
						return {reserved : reserved};
					},
					change_account_state: function(queryString, postedData) {
						var response = {result: false};
						var request = JSON.parse(postedData);
						if (!request) {
							return response;
						}
						var campaign = models.campaigns.get(request.campaignId);
						if (!campaign || !campaign.accounts) {
							return response;
						}
						for (var i = 0; i < campaign.accounts.length; i++) {
							var account = campaign.accounts[i];
							if (account.id === request.accountId && account.proxyType === request.proxyType) {
								campaign.accounts[i].state = request.state ? "1" : "0";
								response.result = models.campaigns.save(campaign);
								break;
							}
						}
						return response;
					},
					save_routing_config: function(queryString, postedData) {
						var response = {result: false};
						var request = JSON.parse(postedData);
						if (!request) {
							return response;
						}
						inConcert.app.database.campaign_routing_config[request.campaignId] = request.rule;
						response.result = true;
						return response;
					},
					get_routing_config: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						return inConcert.app.database.campaign_routing_config[request.campaignId];
					},
					get_members: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (!request) {
							throw({webStatus: "400", textStatus: 'INVALID_REQUEST'});
						}
						//return inConcert.app.database.members;
						var buildTeamMembersHash = function(campaignId) {
							var vcc = m_loggedUser.VCC;
							var name = m_loggedUser.userId;
							var teamMembers = {};
							var myselfAtVCC = name + "@" + vcc;
							var userCampaigns = models.campaigns.getUserCampaigns(name, vcc);
							for (var campaignAtVCC in userCampaigns) {
								if (userCampaigns.hasOwnProperty(campaignAtVCC)) {
									var role = userCampaigns[campaignAtVCC];
									if (role === "invited") {
										continue;
									}
									if (campaignId !== undefined && campaignId !== campaignAtVCC) {
										continue;
									}
									var campaignMembers = models.campaigns.getCampaignMembers(campaignAtVCC) || {};
									for (var userAtVCC in campaignMembers) {
										if (campaignMembers.hasOwnProperty(userAtVCC)) {
											if (myselfAtVCC === userAtVCC) {
												continue;
											}
											if (!teamMembers[userAtVCC]) {
												var member = models.users.get(userAtVCC);
												if (!member) {
													continue;
												}
												//set the agentservice to get each member status
												teamMembers[userAtVCC] = member;
												teamMembers[userAtVCC].campaigns = [];
											}
											teamMembers[userAtVCC].campaigns.push(campaignAtVCC);
										}
									}
								}
							}
							return teamMembers;
						};
						return buildTeamMembersHash(request.campaignId);
					},
					save_disposition_codes: function(queryString, postedData, campaignId) {
						var request = JSON.parse(postedData);
						if (!request || !request.dispositionCodes) {
							throw({webStatus: "400", textStatus: 'INVALID_REQUEST'});
						}
						if (!inConcert.app.database.campaigns[request.campaignId]) {
							throw({webStatus: "500", textStatus: 'UNABLE_TO_SAVE'});
						}
						inConcert.app.database.campaigns[request.campaignId].dispositionCodes = request.dispositionCodes;
						return "";
					},
					save_labels: function(queryString, postedData) {
						var response = {result: false};
						var request = JSON.parse(postedData);
						if (!request || !request.campaignLabels) {
							return response;
						}
						var campaignLabels = request.campaignLabels;
						for (var labelId in campaignLabels) {
							if (campaignLabels.hasOwnProperty(labelId)) {
								var campaignLabel = campaignLabels[labelId];
								if (!inConcert.app.database.campaign_labels[request.campaignId]) {
									inConcert.app.database.campaign_labels[request.campaignId] = {};
								}
								inConcert.app.database.campaign_labels[request.campaignId][labelId] = campaignLabel;
							}
						}
						response.result = true;
						return response;
					},
					delete_labels: function(queryString, postedData) {
						var response = {success: false};
						var request = JSON.parse(postedData);
						if (!request || !request.deleteLabels) {
							return response;
						}
						response.success = true;
						if (!inConcert.app.database.campaign_labels[request.campaignId]) {
							return response;
						}
						var deleteLabels = request.deleteLabels;
						for (var labelId in deleteLabels) {
							if (deleteLabels.hasOwnProperty(labelId)) {
								delete inConcert.app.database.campaign_labels[request.campaignId][labelId];
							}
						}
						return response;
					},
					get_labels: function(queryString, postedData) {
						if ( !postedData || !queryString ){
							return {};
						}
						var request = JSON.parse(postedData || {});
						return inConcert.app.database.campaign_labels[request.campaignId];
					},
					get_members_summary : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var vcc = m_loggedUser.VCC;
						var vccUsers = models.users.getVccUsers(vcc);
						if (!vccUsers) {
							return [];
						}
						var result = [];
						var interactionTypes = [ "IM", "MAIL", "SMS", "TWITTER", "FACEBOOK", "CALL", "CHAT"];
						//builds the summary for this user in this campaign iterating over the created interactions
						var interactionList = inConcert.app.interactionList;
						var buildUserSummary = function(user, campaign) {
							var summary = {};
							for (var i = 0; i < interactionTypes.length ; i++) {
									summary[interactionTypes[i]] = 0;
							}
							for (var interactionId in interactionList) {
								if (interactionList.hasOwnProperty(interactionId)) {
									var interaction = interactionList[interactionId];
									if (interaction.campaign === request.campaignId && interaction.assignedAgents.indexOf(user) >= 0) {
										summary[interaction.type]++;
									}
								}
							}
							return summary;
						};
						for (var keyUser in vccUsers) {
							if (vccUsers.hasOwnProperty(keyUser)) {
								var fullUserId = keyUser + "@"  + vcc;
								if (fullUserId !== m_loggedUser.Id) {
									var role = models.campaigns.isMember(request.campaignId, fullUserId);
									if (role) {
										var summary = buildUserSummary(fullUserId, request.campaignId);
										var member = models.users.get(fullUserId);
										result.push({
											Id: fullUserId,
											Name: vccUsers[keyUser],
											role: role,
											interactions : summary,
											status : member.status
										});
									}
								}
							}
						}
						return result;
					},
					get_campaign_members: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var vcc = m_loggedUser.VCC;
						var vccUsers = models.users.getVccUsers(vcc);
						if (!vccUsers) {
							return [];
						}
						var result = [];
						for (var keyUser in vccUsers) {
							if (vccUsers.hasOwnProperty(keyUser)) {
								var fullUserId = keyUser + "@"  + vcc;
								var supervisor_workgroups = null;
								var user = models.users.get(fullUserId);
								if (user && user.SupervisorWorkgroups) {
									supervisor_workgroups = user.SupervisorWorkgroups[request.campaignId];
								}
								var isMember = models.campaigns.isMember(request.campaignId, fullUserId);
								result.push({
									Id: fullUserId,
									Name: vccUsers[keyUser],
									Role: isMember,
									UserGroup: user.UserGroup,
									supervisor_workgroups: supervisor_workgroups
								});
							}
						}
						return result;
					},
					get_member_archive: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var keyFound = false;
						var result = [];
						var id;
						var i;
						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state === "FINISHED") {
								//iterate over each step and found if the member took part of the interaction
								var interaction = inConcert.app.interactionList[id];
								for (i = 0; i < interaction.steps.length; i++) {
									var step = interaction.steps[i];
									if (step.assignedAgent === request.memberId) {
										if (request.lastKey === interaction.id) {
											keyFound = true;
										}
										if (request.lastKey === "" || keyFound) {
											result.push(interaction);
											if (result.length >= request.top) {
												break;
											}
										}
									}
								}
							}
						}
						return {interactions : result};
					},
					get_campaign_archive : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var keyFound = false;
						var result = [];
						var id;
						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state === "FINISHED") {
								var interaction = inConcert.app.interactionList[id];
								if (interaction.campaign === request.campaignId) {
										if (request.lastKey === interaction.id) {
											keyFound = true;
										}
										if (request.lastKey === "" || keyFound) {
											result.push(interaction);
											if (result.length >= request.top) {
												break;
											}
										}
									}
							}
						}
						return {interactions : result};
					},
					create : function(quertyString, postedData) {
						var campaign = JSON.parse(postedData);
						var match = campaign.Id.match(new RegExp("([^@]+)@(.*)"));
						campaign.Id =  ((match ? match[1] : campaign.Id) + "@" + m_loggedUser.VCC).toLowerCase();
						//start as available on the campaign
						campaign.status = true;
						campaign.accounts = [];
						campaign.scheduler =  {
							scheduleDays: [
								{
									dayOfWeek: "1",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "2",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "3",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "4",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "5",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "6",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								},
								{
									dayOfWeek: "7",
									endTime: "1970-01-01 00:00:00",
									startTime: "1970-01-01 00:00:00"
								}
							]
						};
						inConcert.app.database.campaigns[campaign.Id] = campaign;
						//add this user as campaign administrator
						inConcert.app.database.userCampaigns[m_loggedUser.Id][campaign.Id] = "administrator";
						inConcert.app.database.campaignContacts[campaign.Id] = [];
						//TODO this shuold notify somehow the main_loop?
						return {campaign: campaign};
					},
					'delete' : function(queryString, postedData) {
						var data = JSON.parse(postedData);
						var campaignId = data.campaignId;
						var deleted = false;
						var userId;
						var cid;
						if (inConcert.app.database.campaigns[campaignId]) {
							delete inConcert.app.database.campaigns[campaignId];
							//le saco la campania a todos los usuarios de la misma
							for (userId in inConcert.app.database.userCampaigns) {
								if (inConcert.app.database.userCampaigns.hasOwnProperty(userId)) {
									var userCampaigns = inConcert.app.database.userCampaigns[userId];
									for (cid in userCampaigns) {
										if (userCampaigns.hasOwnProperty(cid)) {
											if (cid === campaignId) {
												delete inConcert.app.database.userCampaigns[userId][campaignId];
												break;
											}
										}
									}
								}
							}
							deleted = true;
						}
						return "";
					},
					set_available: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (inConcert.app.database.campaigns[request.campaign]) {
							inConcert.app.database.campaigns[request.campaign].status = request.enable;
							return {result : true};
						}
						else {
							throw("Campaign does not exists");
						}
					},
					add_account: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (request.account) {
							request.account.state = "1";
							request.account.status = {
								status: "OK"
							};
							if (request.account.proxyType === "CALL" && !request.account.outboundPrefix) {
								request.account.outboundPrefix = "" + (++g_prefixCounter) + "*";
							}
						}
						var campaignId = request.campaignId.search("@") > -1 ? request.campaignId : request.campaignId + "@" + m_loggedUser.VCC;
						if (inConcert.app.database.campaigns[campaignId]) {
							var accounts = inConcert.app.database.campaigns[campaignId].accounts;
							if (typeof(accounts.push) !== "function") {
								inConcert.app.database.campaigns[campaignId].accounts = [];
							}
							inConcert.app.database.campaigns[campaignId].accounts.push(request.account);
							inConcert.app.campaignsAccountsUpdated[campaignId] = inConcert.app.campaignsAccountsUpdated[campaignId] === undefined ? [] : inConcert.app.campaignsAccountsUpdated[campaignId];
							inConcert.app.campaignsAccountsUpdated[campaignId].push(request.account);
							return request.account;
						}
						else {
							throw("Campaign does not exists '" + campaignId + "'");
						}
					},
					update_account: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (request.account) {
							request.account.state = "1";
							request.account.status = {
								status: "OK"
							};
							if (request.account.proxyType === "CALL" && !request.account.outboundPrefix) {
								request.account.outboundPrefix = "" + (++g_prefixCounter) + "*";
							}
						}
						var campaignId = request.campaignId.search("@") > -1 ? request.campaignId : request.campaignId + "@" + m_loggedUser.VCC;
						if (inConcert.app.database.campaigns[campaignId]) {
							for (var i = 0; i < inConcert.app.database.campaigns[campaignId].accounts.length; i++) {
								if (inConcert.app.database.campaigns[campaignId].accounts[i].Id === request.account.Id) {
									inConcert.app.database.campaigns[campaignId].accounts[i] = null;
									inConcert.app.database.campaigns[campaignId].accounts[i] = request.account;
									break;
								}
							}
							inConcert.app.campaignsAccountsUpdated[campaignId] = inConcert.app.campaignsAccountsUpdated[campaignId] === undefined ? [] : inConcert.app.campaignsAccountsUpdated[campaignId];
							inConcert.app.campaignsAccountsUpdated[campaignId].push(request.account);
							return request.account;
						}
						else {
							throw("Campaign does not exists '" + campaignId + "'");
						}
					},
					remove_account: function(queryString, postedData) {
						return "";
					},
					change_member_role: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var memberId = request.memberId;
						var campaign = request.campaignId;

						// Empty role, delete user from campaign.
						if (!request.roleId){
							delete inConcert.app.database.campaignMembers[campaign][memberId];
						}
						else{
							inConcert.app.database.campaignMembers[campaign][memberId].Role = request.roleId;
						}
						return true;
					}
				},
				users: {
					save: function(queryString, postedData) {
						var user = JSON.parse(postedData);
						if (!user) {
							throw({webStatus: '400', textStatus: 'invalid request'});
						}
						var vcc = "inconcert";
						var userWithVCC = user.Id + "@" + vcc;
						// user id validation ------------------
						if (user.newUser && models.users.userExists(userWithVCC)) {
							throw({webStatus: '400', textStatus: 'user already exists'});
						}
						// mail validation ------------------
						var ownerUserId = models.users.findUserByEmail(user.Mail, vcc);
						if (ownerUserId && (user.newUser || (!user.newUser && ownerUserId !== userWithVCC))) {
							throw({webStatus: '400', textStatus: 'mail already exists'});
						}
						//get new user data and validation availables resources
						if (user.newUser) {
							var reserveAvailablePeer = function() {
								var response = {success: false};
								var listPeerByVCC = provisioningmanager.listPeersByVCC();
								if (!listPeerByVCC) {
									return;
								}
								var availablePeerId;
								var currentVCCKey = "VCC:" + vcc ;
								var isAvailablePeer = function (userListByPeer) {
									if (typeof(userListByPeer) !== "object") {
										return false;
									}
									// no tiene que tener ningun elemento en la lista para estar disponible.
									for (var peer in userListByPeer) {
										if (userListByPeer.hasOwnProperty(peer)) {
											return false;
										}
									}
									return true;
								};
								for (var vccPeer in listPeerByVCC) {
									if (listPeerByVCC.hasOwnProperty(vccPeer)) {
										var peersByVCC = listPeerByVCC[vccPeer];
										if (vccPeer === currentVCCKey) {
											for (var index in peersByVCC) {
												if (peersByVCC.hasOwnProperty(index)) {
													var peer = peersByVCC[index];
													var userListByPeer = models.users.findUserByPeer(peer);
													if (isAvailablePeer(userListByPeer)) {
														availablePeerId = peer;
														break;
													}
												}
											}
											break;
										}
									}
								}
								if (!availablePeerId) {
									return response;
								}
								var availablePeer = models.peers.getPeer(availablePeerId);
								if (!availablePeer) {
									return response;
								}
								response.success = true;
								response.peer = {
									getId: function() {
										return this.resource_id + "_" +  this.tech + "_" +  this.name;
									},
									id: availablePeer.id,
									address_id: availablePeer.tech + "/" + availablePeer.name,
									resource_address: availablePeer.hostIP,
									resource_id: availablePeer.hostId,
									tech: availablePeer.tech,
									rtmpServer: availablePeer.hostIP,
									sipServer: availablePeer.hostIP,
									name: availablePeer.name,
									password: availablePeer.password
								};
								return response;
							};
							var getNewUserData = function(user) {
								var response = reserveAvailablePeer(user);
								if (!response.success) {
									return false;
								}
								var reservedPeer = response.peer;
								user.Password = user.Password || "1234";
								user.UserGroup = user.UserGroup || "agent";
								user.VccGroup = user.VccGroup || "system";
								user.LanguageId = user.LanguageId || "en";
								user.countryCode = user.countryCode || "598";
								user.areaCode = user.areaCode || "";
								user.VCC = vcc;
								user.ValidationOrigin = "0";
								user.IsSystemUser = "0";
								user.addresses = {
									CALL: reservedPeer,
									CHAT: {
										address_id:  user.Id + "@" + vcc,
										resource_address: "",
										resourceAddress: "",
										vcc: user.VCC,
										alias:  user.Name,
										username:  user.Id,
										address:  user.Id
									},
									IM: {
										address_id:  user.Id + "@" + vcc,
										resource_address: "",
										resourceAddress: "",
										vcc: user.VCC,
										alias:  user.Name,
										username:  user.Id,
										address:  user.Id
									},
									MAIL: {
										address_id: user.Mail,
										resource_address: "",
										address: user.Mail
									}
								};
								return true;
							};

							//-----------------
							// saving user
							var response_user_data = getNewUserData(user);
							if (!response_user_data) {
								throw({webStatus: '400', textStatus: 'not resources availables'});
							}
						}

						var full_id = user.Id + "@" + user.VCC;
						inConcert.app.database.userpermission[full_id] = user.custom_permissions;
						user.custom_permissions = undefined;
						user.Status = "1";

						if (!models.users.save(user)) {
							throw({webStatus: '500', textStatus: 'unable to save'});
						}

						return "";
					},
					"delete": function(queryString, postedData){
						var response = {
							success: false
						};
						var request = JSON.parse(postedData);
						if (!request || !$.isArray(request) || request.length === 0) {
							return response;
						}
						var id = request[0] || "";
						if (!inConcert.app.database.groups[id]) {
							return response;
						}
						delete inConcert.app.database.users[id];
						var full_id = id + "@" + m_loggedUser.VCC;
						delete inConcert.app.database.userpermission[full_id];
						response.success = true;
						return response;
					}
				},
				contacts : {
					"delete" : function(querystring, postedData) {
						var contact = JSON.parse(postedData);
						if (inConcert.app.database.contacts[contact.id]) {
							delete inConcert.app.database.contacts[contact.id];
							if (inConcert.app.database.userContactFavorites[m_loggedUser.Id] && inConcert.app.database.userContactFavorites[m_loggedUser.Id][contact.id]) {
								delete inConcert.app.database.userContactFavorites[m_loggedUser.Id][contact.id];
							}
							return "";
						}
						throw("Unexistent contact");//TODO this should send a 404
					},
					add_to_favorites : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (inConcert.app.database.contacts[request.contactID]) {
							if (!inConcert.app.database.userContactFavorites[m_loggedUser.Id]) {
								inConcert.app.database.userContactFavorites[m_loggedUser.Id] = {};
							}
							inConcert.app.database.userContactFavorites[m_loggedUser.Id][request.contactID] = true;
							return '{"result":"OK"}';
						}
						else {
							throw("Unexistent contact");//TODO this should send a 404
						}
					},
					remove_from_favorites : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (inConcert.app.database.userContactFavorites[m_loggedUser.Id] && inConcert.app.database.userContactFavorites[m_loggedUser.Id][request.contactID]) {
							delete inConcert.app.database.userContactFavorites[m_loggedUser.Id][request.contactID];
							return '{"result":"OK"}';
						}
						throw("Unexistent contact");//TODO this should send a 404
					},
					get_favorites : function(queryString, postedData) {
						var result = [];
						var contactID;
						var favorites = inConcert.app.database.userContactFavorites[m_loggedUser.Id];
						if (favorites) {
							for (contactID in favorites) {
								if (favorites.hasOwnProperty(contactID)) {
									var contact = inConcert.app.database.contacts[contactID];
									contact.UsersFavorite = {};
									contact.UsersFavorite[ m_loggedUser.Id ] = true;
									result.push(contact);
								}
							}
						}
						return result;
					},
					get_archive: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var keyFound = false;
						var result = [];
						var id;
						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state === "FINISHED") {
								var interaction = inConcert.app.interactionList[id];
								if (interaction.contact === request.contactID) {
										if (request.lastKey === interaction.id) {
											keyFound = true;
										}
										if (request.lastKey === "" || keyFound) {
											result.push(interaction);
											if (result.length >= request.top) {
												break;
											}
										}
									}
							}
						}
						return { interactions : result};
					},
					save : function(queryString, postedData) {
						// TODO: Hacer idem a como lo realiza el controller de contactos / save del model de contactos.
						var contact = JSON.parse(postedData);
						contact.isNew = false;
						var match = contact.Id.match(new RegExp("(.+)@([^@]+)$"));
						contact.Id = ((match && match[1]) ? match[1] : contact.Id) + "@" + m_loggedUser.VCC;
						inConcert.app.database.contacts[contact.Id] = contact;
						//this contact will belong to all the campaigns of the user
						var userCampaigns = inConcert.app.database.userCampaigns[m_loggedUser.Id];
						if (userCampaigns) {
							for (var campaignId in userCampaigns) {
								if (userCampaigns.hasOwnProperty(campaignId)) {
									inConcert.app.database.campaignContacts[campaignId].push(contact.Id);
								}
							}
						}
						return JSON.stringify(contact);
					},
					search : function(queryString, postedData) {
						var searchQuery = JSON.parse(postedData);
						var contactsFound = 0;
						var result = { contacts: []};

						var contactMatchesFn = function(contact, filterText) {
							var stringifiedContact = JSON.stringify(contact);
							return stringifiedContact.match(new RegExp(filterText, "i"));
						};

						//to keep track which contacts were pushed and avoid repeating
						var hashContacts = {};

						//iterate over all the campaigns of the user and retrieve contacts from local database
						var userCampaigns = inConcert.app.database.userCampaigns[m_loggedUser.Id];
						var campaignId;
						var i;
						var pageStarted = false;
						if (userCampaigns) {
							for (campaignId in userCampaigns) {
								if (userCampaigns.hasOwnProperty(campaignId)) {
									var contactList = inConcert.app.database.campaignContacts[campaignId];
									if (contactList) {
										for (i = 0; i < contactList.length; i++) {
											var contact = inConcert.app.database.contacts[contactList[i]];
											if (contact) {
												if (contactMatchesFn(contact, searchQuery.query) && !hashContacts[contact.Id]) {
													contactsFound++;
													if (!pageStarted) {
														if (searchQuery.last === "") {
															pageStarted = true;
														}
														else  {
															if (contact.Id === searchQuery.last) {
																pageStarted = true;
																continue; //don't want to process this contact again
															}
														}
													}
													if (pageStarted) {
														result.last = contact.Id;
														//tag each contact if it's favorite or not
														if (inConcert.app.database.userContactFavorites[m_loggedUser.Id] && inConcert.app.database.userContactFavorites[m_loggedUser.Id][contact.Id]) {
															contact.UsersFavorite = {};
															contact.UsersFavorite[ m_loggedUser.Id ] = true;
														}
														else {
															if ( contact.UsersFavorite ) {
																//esto lo tengo que hacer porque PISA el valor en la base local no es una copia
																delete contact.UsersFavorite[ m_loggedUser.Id ];
															}
														}
														hashContacts[contact.Id] = true;
														result.contacts.push(contact);
														if (result.contacts.length >= searchQuery.rows) {
															return result;
														}
													}
												}
											}
										}
									}
								}
							}
						}
						return result;
					},
					merge : function (queryString, postedData) {
						//comes: var request = {id : self.Id, target : targetContact.Id};
						var request = JSON.parse(postedData);
						var idA = request.id;
						var idB = request.target;

						if (!inConcert.app.database.contacts[idA]) {
							throw("Tried to merge an unexistent contact ("+idA+")");
						}
						if (!inConcert.app.database.contacts[idB]) {
							throw("Tried to merge an unexistent contact ("+idB+")");
						}
						var contactA = inConcert.app.database.contacts[idA];
						var contactB = inConcert.app.database.contacts[idB];

						// MOVE A INTO B!

						/*Move addresses*/
						for (var i = 0; i < contactA.addresses.length; i++) {
							var address = contactA.addresses[i]; //each address is a dictionary
							if (!addressAlreadyThere(address, contactB.addresses)) {
								contactB.addresses.push(address);
							}
						} //for each address in A

						/*Move interactions*/
						//find each iteraction of contact A and change its id to contactB's id
						for (var id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id)) {
								var interaction = inConcert.app.interactionList[id];
								if (interaction.contact === idA) {
									interaction.contact = idB;
								}
							} else {
								//do something
								throw("Eror in api_mockup: inConcert.app.interactionList.hasOwnProperty("+id+") returned false");
							}
						}
						/*
						* DO SOMETHING ELSE?
						*/

						/*remove contact A from contacts' list*/
						delete inConcert.app.database.contacts[idA];
						return contactB;
					},
					get : function (queryString, postedData) {
						var contactId = queryString;
						if (inConcert.app.database.contacts[contactId]) {
							return inConcert.app.database.contacts[contactId];
						} else {
							return false;
						}
					},
					get_by_address : function(queryString, postedData){
						var request = JSON.parse(postedData);
						var addressType = request.addressType;
						var address = request.address;
						var contactFound = null;

						var eachAddresses = function(contactId) {
							var contact = inConcert.app.database.contacts[contactId];
							contact.addresses.forEach(function(contactAddress){
								if (addressType === contactAddress.Type){
									if (getAddressDisplayText(contactAddress) === getAddressDisplayText(address)){
										contactFound = JSON.stringify(contact);
										return;
									}
								}
							});
						};
						for (var contactId in inConcert.app.database.contacts) {
							if (inConcert.app.database.contacts.hasOwnProperty(contactId)) {
								eachAddresses(contactId);
							}
						}
						return contactFound;
					},
					twitter : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						switch (queryString) {
							case "follow_user":
								return twitter.follow(request);
							case "unfollow_user":
								return twitter.unfollow(request);
							case "show_relationship":
								return twitter.getRelationShip(request);
							case "block_exists":
									return twitter.blockExists(request);
							case "block_user":
									return twitter.blockUser(request);
							case "unblock_user":
									return twitter.unblockUser(request);
							case "get_user":
								return twitter.getTwitterUser(request.userId);
							case "refresh_social_info" :
								var contactId = request.contactID;
								if (inConcert.app.database.contacts[contactId]) {
									var contact = inConcert.app.database.contacts[contactId];
									for (var i = 0; i < contact.addresses.length; i++){
										if (contact.addresses[i].Type === "Twitter"){
											var twitterInfo = twitter.getTwitterUser(contact.addresses[i].User);
											contact.addresses[i].favourites_count = twitterInfo.favourites_count;
											contact.addresses[i].followers_count = twitterInfo.followers_count;
											contact.addresses[i].friends_count = twitterInfo.friends_count;
											contact.addresses[i].statuses_count = twitterInfo.statuses_count;
										}
									}
									// save the contact
									inConcert.app.database.contacts[contact.Id] = contact;
									return JSON.stringify(contact);
								} else {
									return "Contact not found";
								}
							}
						}
				},
				//most of the interaction methods just modify the JSON and send it to disk again in order to be retrieved by the main loop
				interactions: {
					get_members_for_transfer: function(queryString, postedData) {
						return inConcert.app.database.members;
					},
					set_interaction_contact: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.interactionId];
						if (!interaction) {
							return false;
						}
						var match = request.contactId.match(new RegExp("(.+)@([^@]+)$"));
						interaction.contact = ((match && match[1]) ? match[1] : request.contactId) + "@" + m_loggedUser.VCC;
						interaction.interactionData.contactName = request.contactName;
						queueInteraction(interaction);
						return true;
					},
					get_top_queued: function(queryString, postedData) {
						var interactions = readStorageDirectory();
						return  {
							interactions: interactions
						};
					},
					get_campaign_top_queued: function(queryString, postedData) {
						return  {
							interactions: readStorageDirectory()
						};
					},
					update_labels_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interactions = request.interactions;
						var successful_nodes = [];
						var unsuccessful_nodes = [];

						interactions.forEach(function(interaction){
							interaction = inConcert.app.interactionList[interaction.id];
							if (interaction) {
								if (!request.newLabels) {
									unsuccessful_nodes.push({id:interaction.id, reason:"Update label request without labels"});
								}
								var timestamp = getCurrentTimestamp();
								interaction.labels =  request.newLabels;
								if (interaction.labels.PENDING) {
									interaction.requiredAgent = inConcert.app.getCompleteId();
									interaction.state = "QUEUED";
									interaction.steps.push({
										action : "QUEUED",
										timestamp : timestamp
									});
									interaction.assignedAgents = [];
								}
								else {
									interaction.requiredAgent = "";
									interaction.state = "TAKEN";
									interaction.steps.push({
										action : "TAKEN",
										agent: inConcert.app.getCompleteId(),
										timestamp : timestamp
									});
									interaction.assignedAgents = [];
									interaction.assignedAgents.push(inConcert.app.getCompleteId());
								}
								interaction.timestamp = timestamp;
								queueInteraction(interaction);
								successful_nodes.push({ id:interaction.id , current_labels:interaction.labels});
							}
							else {
								unsuccessful_nodes.push({id:interaction.id,reason:"Tried to label an unexistent interaction"});
							}
						});
						return {
							successful_nodes : successful_nodes,
							unsuccessful_nodes : unsuccessful_nodes
						};
					},
					delete_labels_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interactions = request.interactions;
						var successful_nodes = [];
						var unsuccessful_nodes = [];

						interactions.forEach(function(interaction){
							interaction = inConcert.app.interactionList[request.interactionId];
							if (interaction) {
								if (!request.deleteLabels) {
									unsuccessful_nodes.push({id:interaction.id, reason:"Delete label request without labels"});
								}
								var timestamp = getCurrentTimestamp();
								var deleteLabels = function(interactionId, labelsToDelete, labelsToAdd) {
									var result = true;
									if (!labelsToDelete) {
										return result;
									}
									var canDelete = function(labelId) {
										if (!labelsToAdd) {
											return true;
										}
										if (labelsToAdd[labelId]) {
											return false;
										}
										return true;
									};
									for (var labelId in labelsToDelete) {
										if (labelsToDelete.hasOwnProperty(labelId)) {
											if (canDelete(labelId)) {
												if (interaction.labels[labelId]) {
													interaction.requiredAgent = "";
													interaction.state = "TAKEN";
													interaction.steps.push({
														action : "TAKEN",
														agent: inConcert.app.getCompleteId(),
														timestamp : timestamp
													});
													interaction.assignedAgents = [];
													interaction.assignedAgents.push(inConcert.app.getCompleteId());
												}
												delete interaction.labels[labelId];
											}
										}
									}
									return result;
								};

								deleteLabels(request.interactionId, request.deleteLabels);
								interaction.timestamp = timestamp;
								queueInteraction(interaction);
								successful_nodes.push({ id:interaction.id , current_labels:interaction.labels});
							}
							else {
								unsuccessful_nodes.push({id:interaction.id, reason:"Tried to delete a label on an unexistent interaction"});
							}
						});
						return {
							successful_nodes : successful_nodes,
							unsuccessful_nodes : unsuccessful_nodes
						};
					},
					requeue_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interactions = request.interactions;
						var successful_nodes = [];
						var unsuccessful_nodes = [];
						if (!interactions){
							return false;
						}
						interactions.forEach(function(interaction){
							interaction = inConcert.app.interactionList[interaction.id];
							if (interaction) {
								var timestamp = getCurrentTimestamp();
								var newStep = {
									action : "QUEUED",
									timestamp : timestamp
								};
								interaction.timestamp = timestamp;
								interaction.state = "QUEUED";
								interaction.steps.push(newStep);
								interaction.assignedAgents = [];
								queueInteraction(interaction);
								successful_nodes.push({ id:interaction.id });
							}
							else {
								unsuccessful_nodes.push({id:interaction.id,reason:"Tried to requeue an unexistent interaction"});
							}
						});
						return {
							successful_nodes : successful_nodes,
							unsuccessful_nodes : unsuccessful_nodes
						};
					},
					take_interaction : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];
						if (interaction) {
							var timestamp = getCurrentTimestamp();
							var newStep = {
								action : "TAKEN",
								timestamp : timestamp,
								assignedAgent : m_loggedUser.Id
							};
							interaction.timestamp = timestamp;
							interaction.state = "TAKEN";
							interaction.steps.push(newStep);
							interaction.assignedAgents = [];
							interaction.assignedAgents.push(inConcert.app.getCompleteId());
							queueInteraction(interaction);
							afterInteractionTaken(interaction);
						}
						else {
							throw("Tried to take an unexistent interaction");
						}
					},
					reopen_interaction : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];
						if (interaction) {
							var timestamp = getCurrentTimestamp();
							var newStep = {
								action : "TAKEN",
								timestamp : timestamp,
								assignedAgent : m_loggedUser.Id
							};
							interaction.timestamp = timestamp;
							interaction.state = "TAKEN";
							interaction.steps.push(newStep);
							interaction.assignedAgents = [];
							interaction.assignedAgents.push(inConcert.app.getCompleteId());
							queueInteraction(interaction);
							afterInteractionTaken(interaction);
							return true;
						}
						else {
							throw("Tried to reopen an unexistent interaction");
						}
					},
					archive_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interactions = request.interactions;
						var disposition = request.disposition;
						var comment = request.comment;
						var successful_nodes = [];
						var unsuccessful_nodes = [];

						if (comment && comment.length > 0) {
							interactions.forEach(function(interaction){
								var commentRequest = {
									id : interaction.id,
									comment: comment
								};
								apps.agent.controllers.interactions.comment_interaction("", JSON.stringify(commentRequest));
							});
						}

						interactions.forEach(function(interaction){
							interaction = inConcert.app.interactionList[interaction.id];
							if (interaction) {
								var timestamp = getCurrentTimestamp();
								var newStep = {
									action : "FINISHED",
									timestamp : timestamp,
									assignedAgents : m_loggedUser.Id
								};
								if (disposition) {
									interaction.dispositionCode = disposition.name;
									newStep.wrapupData = disposition;
									newStep.wrapupData.comment = comment;
									if (disposition.changeContactState && disposition.contactId && disposition.contactId.length > 0) {
										inConcert.app.database.contacts[disposition.contactId].state = disposition.changeContactState;
									}
								}
								interaction.timestamp = timestamp;
								interaction.state = "FINISHED";
								interaction.steps.push(newStep);
								interaction.assignedAgents = [];

								queueInteraction(interaction);
								successful_nodes.push({ id:interaction.id });
							}
							else {
								unsuccessful_nodes.push({id:interaction.id,reason:"Tried to finish an unexistent interaction"});
							}
						});
						return {
							successful_nodes : successful_nodes,
							unsuccessful_nodes : unsuccessful_nodes
						};
					},
					comment_interaction : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];
						if (interaction) {
							var timestamp = getCurrentTimestamp();
							var newComment = {
								comment : request.comment,
								comment_date : timestamp,
								author : m_loggedUser.Id,
								authorName : m_loggedUser.Name
							};
							if (!interaction.comments) {
								interaction.comments = [];
							}
							interaction.comments.push(newComment);

							interaction.timestamp = timestamp;

							queueInteraction(interaction);
							return true;
						}
						else {
							throw("Tried to comment an unexistent interaction");
						}
					},
					read_interaction : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];
						var timestamp = getCurrentTimestamp();
						if (interaction) {
							setEventRead(interaction,request.events);
							interaction.timestamp = timestamp;
							queueInteraction(interaction);
							return true;
						}
					},
					get_campaign_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var result = [];
						var id;
						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state !== "FINISHED") {
								var interaction = inConcert.app.interactionList[id];
								if (interaction.campaign === request.campaignId) {
									//check if matches with requested agent
									if(typeof(request.agentId) !== "undefined" && request.agentId !== null && interaction.assignedAgents.indexOf(request.agentId) === -1) {
										continue;
									}
									if(typeof(request.interactionType) !== "undefined" && interaction.type !== request.interactionType) {
										continue;
									}
									result.push(interaction);
								}
							}
						}
						return result;
					},
					get_user_interactions : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var result = [];
						var i;
						var id;
						var interactionBelongsToUserFn = function(interaction, user) {
							for (i = 0; i < interaction.assignedAgents.length; i++) {
								if (interaction.assignedAgents[i] === user) {
									return true;
								}
							}
							return false;
						};
						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state !== "FINISHED") {
								var interaction = inConcert.app.interactionList[id];
								if (interactionBelongsToUserFn(interaction, request.memberId)) {
									result.push(interaction);
								}
							}
						}
						return result;
					},
					search : function(queryString, postedData) {
						var searchQuery = JSON.parse(postedData);
						var interactionsFound = 0;
						var result = { interactions: [] };
						var id;
						var pageStarted = false;

						for (id in inConcert.app.interactionList) {
							if (inConcert.app.interactionList.hasOwnProperty(id) && inConcert.app.interactionList[id].state === "FINISHED") {
								var interaction = inConcert.app.interactionList[id];
									interactionsFound++;
									if (!pageStarted) {
										if (searchQuery.last === "") {
											pageStarted = true;
										}
										else {
											if (searchQuery.last === id) {
												pageStarted = true;
												continue;
											}
										}
									}
									if (pageStarted) {
										result.last = id;
										result.interactions.push(interaction);
										if (result.interactions.length >= searchQuery.rows) {
											break;
										}
									}
							}
						}
						return result;
					},
					facebook : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction;
						var now;
						var zp;
						var timestamp;
						var fbtimestamp;

						switch (queryString) {
							case "create_outgoing":
								//this api will create a new outgoing email
								var match = request.campaign.match(new RegExp("([^@]+)@(.*)"));
								var campaign = match[1];
								var vcc = match[2];
								var contactName = "";
								if (inConcert.app.database.contacts[request.destination]) {
									contactName = inConcert.app.database.contacts[request.destination].Name;
								}
								interaction = createInteraction(request.id, campaign, vcc, request.account, "OUTBOUND", "TAKEN", m_loggedUser.Id,
									request.destination, "", "FACEBOOK", "127.0.0.1", "12000", {contactName : contactName});
								queueInteraction(interaction);
								return true;
							case "comment_post":
								interaction = getInteractionByIndex("Facebook Post ID", request.postId);
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									var commentEvent = {
										from : request.account,
										message : {
											created_time : fbtimestamp,
											from : {
												id : request.account,
												name : "inConcert"
											},
											id : "100002209687702_106156619467994_49764",
											message : request.text
										},
										parentId : request.postId,
										agent : m_loggedUser.Id,
										timestamp : timestamp
									};
									interaction.events.push(commentEvent);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to comment a post from a non existent interaction");
								}
								break;
							case "new_post":
								interaction = inConcert.app.interactionList[request.interactionID];
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									var postEvent = {
										agent : m_loggedUser.Id,
										message : {
											actions : [
												{
													link : "http://www.facebook.com/540059077/posts/10150161567779078",
													name : "Comment"
												},
												{
													link : "http://www.facebook.com/540059077/posts/10150161567779078",
													name : "Like"
												}
											],
											application : {
												"id" : "159345327456008",
												"name" : "inConcert Social Agent"
											},
											created_time : fbtimestamp,
											from : {
												"id" : request.account,
												"name" : "inConcert"
											},
											icon : "http://photos-h.ak.fbcdn.net/photos-ak-snc1/v43/8/159345327456008/app_2_159345327456008_2735.gif",
											id : inConcert.Utils.generateGuid(),
											message : request.text,
											to : {
												data : [
													{
														id : request.destination,
														name : request.destination
													}
												]
											},
											type : "status",
											updated_time : fbtimestamp
										}
									};
									setInteractionIndex(interaction, "Facebook Post ID", postEvent.message.id);
									interaction.events.push(postEvent);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send post a new message from a non existent interaction");
								}
								break;
							case "get_account_pages":
								return '{"facebookpage1": {"name": "facebook page1"}, "facebookpage2": {"name": "facebook page2"}, "facebookpage3": {"name": "facebook page3"}}';
							case "like":
								interaction = inConcert.app.interactionList[request.interactionId];
								if (!interaction) {
									return false;
								}
								interaction.interactionData.likes = interaction.interactionData.likes || {};
								var key = request.commentId || request.postId;
								interaction.interactionData.likes[key] = interaction.interactionData.likes[key] || [];

								for (var i=0; i < interaction.interactionData.likes[key].length; i++) {
										var currentLike = interaction.interactionData.likes[key][i];
										if (currentLike.id === request.account) {
											if (!request.like) {
												interaction.interactionData.likes[key].splice(i,1);
												queueInteraction(interaction);
											}
											return true;
										}
								}
								if (request.like) { //like but not found in previous for loop
									interaction.interactionData.likes[key].push({id: request.account, name: "inConcert"});
									queueInteraction(interaction);
								}
								return true;
							default:
								throw("Non existent facebook method " + queryString);
						}
					},
					mail: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction;
						switch (queryString) {
							case "create_outgoing":
								//this api will create a new outgoing email
								var match = request.campaign.match(new RegExp("([^@]+)@(.*)"));
								var campaign = match[1];
								var vcc = match[2];
								var contactName = "";
								if (inConcert.app.database.contacts[request.contact]) {
									contactName = inConcert.app.database.contacts[request.contact].Name;
								}
								interaction = createInteraction(request.id, campaign, vcc, request.account, "OUTBOUND", "TAKEN", m_loggedUser.Id,
									request.contact, "", "MAIL", "127.0.0.1", "12000", {contactName : contactName});
								interaction.contact = '';
								queueInteraction(interaction);
								return true;
							case "send_message":
								interaction = inConcert.app.interactionList[request.id];
								var now = new Date();
								var zp = inConcert.Utils.zeropad;
								var timestampISO = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}Z",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								var timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									var mailEvent =  {
										attachments : request.attachments || {},
										body : [
											{
												content : request.body,
												type : "text/plain"
											}
										],
										cc : {
										},
										date : timestampISO,
										"timestamp" : timestamp,
										"from" : request.from,
										"name" : request.campaign,
										"subject" : request.subject,
										"to" : {
										},
										agent : m_loggedUser.Id
									};
									if (request.cc && request.cc.length > 0) {
										mailEvent.cc[request.cc] = request.cc;
									}
									if (request.to && request.to.length > 0) {
										mailEvent.to[request.to] = request.to;
									}
									interaction.events.push(mailEvent);
									setEventRead(interaction, [interaction.events.length-1]);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send a mail from a non existent interaction");
								}

						}
					},
					im : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						if (queryString === "create_outgoing") {
							//this api will create a new outgoing email
							var match = request.campaign.match(new RegExp("([^@]+)@(.*)"));
							var campaign = match[1];
							var vcc = match[2];
							var contactName = "";
							if (inConcert.app.database.contacts[request.contact]) {
								contactName = inConcert.app.database.contacts[request.contact].Name;
							}
							var interaction = createInteraction(request.id, campaign, vcc, null, "OUTBOUND", "TAKEN", m_loggedUser.Id,
										request.contact, "", "IM", "127.0.0.1", "12000",
										{
											contactName: contactName,
											protocol: request.address.protocol
										});
							queueInteraction(interaction);
							return true;
						}
						else {
							throw("Unexpected operation for IM " + queryString);
						}
					},
					twitter : function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction;
						var now;
						var zp;
						var timestamp;
						var fbtimestamp;
						var postEvent;
						switch (queryString) {
							case "create_outgoing":
								//this api will create a new outgoing TWITTER
								var match = request.campaign.match(new RegExp("([^@]+)@(.*)"));
								var campaign = match[1];
								var vcc = match[2];
								var contactName = "";
								if (inConcert.app.database.contacts[request.destination]) {
									contactName = inConcert.app.database.contacts[request.destination].Name;
								}
								interaction = createInteraction(request.id, campaign, vcc, request.account, "OUTBOUND", "TAKEN", m_loggedUser.Id,
									request.destination, "", "TWITTER", "127.0.0.1", "12000", {contactName : contactName});
								queueInteraction(interaction);
								return true;

							case "tweet":
								interaction = inConcert.app.interactionList[request.interactionID];
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									postEvent = {
										agent : m_loggedUser.Id,
										//id : inConcert.Utils.generateGuid(),
										created_at: fbtimestamp,
										favorited: false,
										id:	60093386262184000,
										id_str: "60093386262183936",
										in_reply_to_screen_name: "rumpelstink",
										in_reply_to_user_id: 261412447,
										in_reply_to_user_id_str: "261412447",
										retweet_count: 0,
										retweeted: false,
										source: '<a href="http://www.inconcertcc.com" rel="nofollow">oauth_testing_app</a>',
										text: inConcert.Utils.escapeAttrib(request.status),
										timestamp: fbtimestamp,
										truncated: false,
										user: {
											contributors_enabled: false,
											created_at: "Sat Mar 05 22:26:09 +0000 2011",
											default_profile: true,
											default_profile_image: true,
											favourites_count: 0,
											follow_request_sent: false,
											followers_count: 3,
											following: false,
											friends_count: 6,
											geo_enabled: false,
											id: 261412447,
											id_str: "261412447",
											is_translator: false,
											lang: "en",
											listed_count: 0,
											name: "Rumpel Stinky",
											notifications: false,
											profile_background_color: "C0DEED",
											profile_background_image_url: "http://a3.twimg.com/a/1301681483/images/themes/theme1/bg.png",
											profile_background_tile: false,
											profile_image_url: "http://a3.twimg.com/sticky/default_profile_images/default_profile_2_normal.png",
											profile_link_color: "0084B4",
											profile_sidebar_border_color: "C0DEED",
											profile_sidebar_fill_color: "DDEEF6",
											profile_text_color: "333333",
											profile_use_background_image: true,
											//protected: false,
											screen_name: "rumpelstink",
											show_all_inline_media: false,
											statuses_count: 544,
											verified: false
										}
									};
									setInteractionIndex(interaction, "Twitter Post ID", postEvent.id);
									interaction.events.push(postEvent);
									setEventRead(interaction, [interaction.events.length - 1]);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send post a new message from a non existent interaction");
								}
								break;
							case "retweet":
								interaction = inConcert.app.interactionList[request.interactionID];
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									postEvent = {
										agent: m_loggedUser.Id,
										//id: inConcert.Utils.generateGuid(),
										created_at: "Wed Apr 20 17:33:57 +0000 2011",
										favorited: false,
										id: 60758111782309992,
										id_str: "60758111782309889",
										retweet_count: 0,
										retweeted: false,
										retweeted_status: {
											created_at: "Mon Apr 18 17:29:35 +0000 2011",
											favorited: false,
											id: 60032237743247000,
											id_str: "60032237743247360",
											in_reply_to_screen_name: "rumpelstink",
											in_reply_to_user_id: 261412447,
											in_reply_to_user_id_str: "261412447",
											retweet_count: 0,
											retweeted: false,
											source: "web",
											text: "@rumpelstink hi3",
											truncated: false,
											user: {
												contributors_enabled: false,
												created_at: "Thu Dec 02 11:32:16 +0000 2010",
												default_profile: false,
												default_profile_image: false,
												description: "",
												favourites_count: 0,
												follow_request_sent: false,
												followers_count: 5,
												following: true,
												friends_count: 4,
												geo_enabled: false,
												id: 222072652,
												id_str: "222072652",
												is_translator: false,
												lang: "es",
												listed_count: 0,
												location: "Uruguay",
												name: "Ellio de Mello",
												notifications: false,
												profile_background_color: "1A1B1F",
												profile_background_image_url: "http://a3.twimg.com/a/1302646548/images/themes/theme9/bg.gif",
												profile_background_tile: false,
												profile_image_url: "http://a2.twimg.com/profile_images/1180778703/aquiles3_normal.jpg",
												profile_link_color: "2FC2EF",
												profile_sidebar_border_color: "181A1E",
												profile_sidebar_fill_color: "252429",
												profile_text_color: "666666",
												profile_use_background_image: true,
												//protected: false,
												screen_name: "cany74",
												show_all_inline_media: false,
												statuses_count: 118,
												time_zone: "Buenos Aires",
												utc_offset: -10800,
												verified: false
											}
										},
										source: '<a href="http://www.inconcertcc.com" rel="nofollow">oauth_testing_app</a>',
										text: 'RT @cany74: @rumpelstink hi3',
										timestamp: fbtimestamp,
										truncated: false,
										user: {
											contributors_enabled: false,
											created_at: "Sat Mar 05 22:26:09 +0000 2011",
											default_profile: true,
											default_profile_image: true,
											favourites_count: 0,
											follow_request_sent: false,
											followers_count: 3,
											following: false,
											friends_count: 6,
											geo_enabled: false,
											id: 261412447,
											id_str: "261412447",
											is_translator: false,
											lang: "en",
											listed_count: 0,
											name: "Rumpel Stinky",
											notifications: false,
											profile_background_color: "C0DEED",
											profile_background_image_url: "http://a3.twimg.com/a/1301681483/images/themes/theme1/bg.png",
											profile_background_tile: false,
											profile_image_url: "http://a3.twimg.com/sticky/default_profile_images/default_profile_2_normal.png",
											profile_link_color: "0084B4",
											profile_sidebar_border_color: "C0DEED",
											profile_sidebar_fill_color: "DDEEF6",
											profile_text_color: "333333",
											profile_use_background_image: true,
//											protected: false,
											screen_name: "rumpelstink",
											show_all_inline_media: false,
											statuses_count: 555,
											verified: false
										}
									};
									setInteractionIndex(interaction, "Twitter Post ID", postEvent.id);
									interaction.events.push(postEvent);
									setEventRead(interaction, [interaction.events.length - 1]);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send post a new message from a non existent interaction");
								}
								break;
							case "reply_tweet":
								interaction = inConcert.app.interactionList[request.interactionID];
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									postEvent = {
										agent : m_loggedUser.Id,
										//id : inConcert.Utils.generateGuid(),
										created_at: fbtimestamp,
										favorited: false,
										id:	60093386262184000,
										id_str: "60093386262183936",
										in_reply_to_screen_name: "rumpelstink",
										in_reply_to_user_id: 261412447,
										in_reply_to_user_id_str: "261412447",
										retweet_count: 0,
										retweeted: false,
										source: '<a href="http://www.inconcertcc.com" rel="nofollow">oauth_testing_app</a>',
										text: inConcert.Utils.escapeAttrib(request.status),
										timestamp: fbtimestamp,
										truncated: false,
										user: {
											contributors_enabled: false,
											created_at: "Sat Mar 05 22:26:09 +0000 2011",
											default_profile: true,
											default_profile_image: true,
											favourites_count: 0,
											follow_request_sent: false,
											followers_count: 3,
											following: false,
											friends_count: 6,
											geo_enabled: false,
											id: 261412447,
											id_str: "261412447",
											is_translator: false,
											lang: "en",
											listed_count: 0,
											name: "Rumpel Stinky",
											notifications: false,
											profile_background_color: "C0DEED",
											profile_background_image_url: "http://a3.twimg.com/a/1301681483/images/themes/theme1/bg.png",
											profile_background_tile: false,
											profile_image_url: "http://a3.twimg.com/sticky/default_profile_images/default_profile_2_normal.png",
											profile_link_color: "0084B4",
											profile_sidebar_border_color: "C0DEED",
											profile_sidebar_fill_color: "DDEEF6",
											profile_text_color: "333333",
											profile_use_background_image: true,
											//protected: false,
											screen_name: "rumpelstink",
											show_all_inline_media: false,
											statuses_count: 544,
											verified: false
										}
									};
									setInteractionIndex(interaction, "Twitter Post ID", postEvent.id);
									interaction.events.push(postEvent);
									setEventRead(interaction, [interaction.events.length - 1]);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send post a new message from a non existent interaction");
								}
								break;
							case "send_dm":
								interaction = inConcert.app.interactionList[request.interactionID];
								now = new Date();
								zp = inConcert.Utils.zeropad;
								timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								fbtimestamp = inConcert.Utils.format("{0}-{1}-{2}T{3}:{4}:{5}",
										now.getUTCFullYear(), zp(now.getUTCMonth()+1), zp(now.getUTCDate()), zp(now.getUTCHours()), zp(now.getUTCMinutes()), zp(now.getUTCSeconds()));
								if (interaction) {
									postEvent = {
										agent: m_loggedUser.Id,
										created_at: fbtimestamp,
										id: -781622921,
										id_str: "3513344375",
										recipient: {
											contributors_enabled: false,
											created_at: fbtimestamp,
											default_profile: false,
											default_profile_image: false,
											description: "",
											favourites_count: 0,
											follow_request_sent: false,
											followers_count: 6,
											following: true,
											friends_count: 4,
											geo_enabled: false,
											id: 222072652,
											id_str: "222072652",
											is_translator: false,
											lang: "es",
											listed_count: 0,
											location: "Uruguay",
											name: "Ellio de Mello",
											notifications: false,
											profile_background_color: "1A1B1F",
											profile_background_image_url: "http://a1.twimg.com/images/themes/theme9/bg.gif",
											profile_background_image_url_https: "https://si0.twimg.com/images/themes/theme9/bg.gif",
											profile_background_tile: false,
											profile_image_url: "http://a0.twimg.com/profile_images/1366233020/15_de_rocio_1_normal.jpg",
											profile_image_url_https: "https://si0.twimg.com/profile_images/1366233020/15_de_rocio_1_normal.jpg",
											profile_link_color: "2FC2EF",
											profile_sidebar_border_color: "181A1E",
											profile_sidebar_fill_color: "252429",
											profile_text_color: "666666",
											profile_use_background_image: true,
											'protected': false,
											screen_name: "cany74",
											show_all_inline_media: false,
											statuses_count: 153,
											time_zone: "Buenos Aires",
											utc_offset: -10800,
											verified: false
										},
										recipient_id: 22207265,
										recipient_screen_name: "cany74",
										sender: {
											contributors_enabled: false,
											created_at: fbtimestamp,
											default_profile: true,
											default_profile_image: true,
											favourites_count: 0,
											follow_request_sent: false,
											followers_count: 7,
											following: false,
											friends_count: 9,
											geo_enabled: false,
											id: 261412447,
											id_str: "261412447",
											is_translator: false,
											lang: "en",
											listed_count: 0,
											name: "Rumpel Stinky",
											notifications: false,
											profile_background_color: "C0DEED",
											profile_background_image_url: "http://a0.twimg.com/images/themes/theme1/bg.png",
											profile_background_image_url_https: "https://si0.twimg.com/images/themes/theme1/bg.png",
											profile_background_tile: false,
											profile_image_url: "http://a1.twimg.com/sticky/default_profile_images/default_profile_2_normal.png",
											profile_image_url_https: "https://si0.twimg.com/sticky/default_profile_images/default_profile_2_normal.png",
											profile_link_color: "0084B4",
											profile_sidebar_border_color: "C0DEED",
											profile_sidebar_fill_color: "DDEEF6",
											profile_text_color: "333333",
											profile_use_background_imag: true,
											'protected': false,
											screen_name: "rumpelstink",
											show_all_inline_media: false,
											statuses_count: 1565,
											verified: false
										},
										sender_id: 261412447,
										sender_screen_name: "rumpelstink",
										text: request.message,
										timestamp: fbtimestamp
									};
									setInteractionIndex(interaction, "Twitter Post ID", postEvent.id);
									interaction.events.push(postEvent);
									interaction.timestamp = timestamp;
									queueInteraction(interaction);
									return true;
								}
								else {
									throw("Tried to send post a new message from a non existent interaction");
								}
								break;
							default:
								throw("Non existent twitter method " + queryString);
						}
					},
					spam : function (queryString, postedData) {
						var request = postedData && JSON.parse(postedData) || null;

						switch(queryString) {
							case "get_spam":
								var spamInteractions = getSpamInteractions();

								//sort by timestamp desc
								spamInteractions.sort(function(a, b) {
									return a.timestamp > b.timestamp ? -1 : 1;
								});

								//keep 30 most recent
								var result = [];
								for (var i = 0; i < spamInteractions.length && i < 30; i++) {
									result.push(spamInteractions[i]);
								}
								return {totalCount: spamInteractions.length, interactions: result};

							case "mark_as_spam":
								request.interactionIDs.forEach(function(id){
									var interaction = inConcert.app.interactionList[id];
									if (interaction) {
										var timestamp = getCurrentTimestamp();
										var newStep = {
											action : "SPAM",
											timestamp : timestamp,
											assignedAgents : m_loggedUser.Id
										};

										interaction.timestamp = timestamp;

										interaction.state = "SPAM";
										interaction.steps.push(newStep);
										interaction.assignedAgents = [];
										queueInteraction(interaction);

										saveSpamInteraction(interaction);
									}
								});
							break;

							case "mark_not_spam":
								markNoSpamInteractions(request.interactionIDs);
							break;

							case "delete_spam":
								deleteSpamInteractions(request.interactionIDs);
							break;
							default: throw("Non existant spam method: " + queryString);
						}
					},
					get_archived_interaction: function (queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.interactionID];
						for (var id in inConcert.app.interactionList) {
							if (id === interaction.id) {
								return inConcert.app.interactionList[id];
							}
						}
						return false;
					},
					take_next_best: function (queryString, postedData) {
						var interactionList = inConcert.app.interactionList;
						var bestInteraction = null;

						var belongsToUserCampaigns = function (interaction){
							var userCampaigns = inConcert.app.database.userCampaigns[m_loggedUser.Id];
							var campaignId;
							for (campaignId in userCampaigns){
								if (userCampaigns.hasOwnProperty(campaignId)) {
									if (interaction.campaign === campaignId){
										return true;
									}
								}
							}
						};

						//Go and search for the best interaction
						for (var interactionId in interactionList) {
							if (interactionList.hasOwnProperty(interactionId)) {
								var interaction = interactionList[interactionId];
								//If the interaction campaign belongs to the user and it's not already taken
								if (belongsToUserCampaigns(interaction) && (interaction.state === "QUEUED")) {
									if (bestInteraction){
										if (interaction.priority > bestInteraction.priority){
											bestInteraction = interaction;
										}
										else if (interaction.priority === bestInteraction.priority) {
											bestInteraction = (interaction.timestamp > bestInteraction.timestamp) ? interaction : bestInteraction;
										}
									}
									else {
										bestInteraction = interaction;
									}
								}
							}
						}

						if (!bestInteraction) {
							//nothing to take
							return;
						}

						var takeInteractionFn = function(){
							inConcert.apps.agent.controllers.interactions.take_interaction("", JSON.stringify(bestInteraction));
						};

						// Only bestInteraction
						if (bestInteraction) {
							takeInteractionFn();
						}
					},
					transfer_interactions: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interactions = request.interactions;
						var to_agent = request.target+"@inconcert";
						var successful_nodes = [];
						var unsuccessful_nodes = [];

						interactions.forEach(function(interaction){
							var current_interaction = inConcert.app.interactionList[interaction.id];
							if (current_interaction){
								current_interaction.initialAgent = to_agent;
								current_interaction.assignedAgent = to_agent;

								var newState = "TRANSFER LOCKED";
								var timestamp = getCurrentTimestamp();
								var newStep = {
									action : newState,
									timestamp : timestamp,
									assignedAgent : to_agent
								};
								current_interaction.timestamp = timestamp;
								current_interaction.state = newState;
								current_interaction.steps.push(newStep);

								// change assigned agents
								current_interaction.assignedAgents = [];
								current_interaction.assignedAgents.push(to_agent);

								setTimeout(function() {
									var newState = "TRANSFER TAKEN";
									var timestamp = getCurrentTimestamp();
									var newStep = {
										action : newState,
										timestamp : timestamp,
										assignedAgent : to_agent
									};
									current_interaction.timestamp = timestamp;
									current_interaction.state = newState;
									current_interaction.steps.push(newStep);

									queueInteraction(current_interaction);
									successful_nodes.push({ id:interaction.id });
								}, 1000);
							}
							else {
								unsuccessful_nodes.push({id:interaction.id,reason:"Tried to transfer an unexistent interaction"});
							}
							return {
								successful_nodes : successful_nodes,
								unsuccessful_nodes : unsuccessful_nodes
							};
						});
					},
					start_attended_transfer: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];

						var newState = "TRANSFER LOCKED";
						var timestamp = getCurrentTimestamp();
						var newStep = {
							action : newState,
							timestamp : timestamp,
							assignedAgent : m_loggedUser.Id
						};
						interaction.timestamp = timestamp;
						interaction.state = newState;
						interaction.steps.push(newStep);
						//interaction.assignedAgents.push("pepe@inconcert");

						setTimeout(function() {
							var newState = "TRANSFER TAKEN";
							var timestamp = getCurrentTimestamp();
							var newStep = {
								action : newState,
								timestamp : timestamp,
								assignedAgent : m_loggedUser.Id
							};
							interaction.timestamp = timestamp;
							interaction.state = newState;
							interaction.steps.push(newStep);
						}, 3000);

						// TODO: consultar a ese moreno
						return true;
					},
					commit_attended_transfer: function(queryString, postedData) {
						var request = JSON.parse(postedData);

						var interaction = inConcert.app.interactionList[request.id];
						var zp = inConcert.Utils.zeropad;
						var now = new Date();

						var newState = "TAKEN";
						var timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
								now.getUTCFullYear(),
								zp(now.getUTCMonth()+1),
								zp(now.getUTCDate()),
								zp(now.getUTCHours()),
								zp(now.getUTCMinutes()),
								zp(now.getUTCSeconds())
							);
						var newStep = {
							action : newState,
							timestamp : timestamp,
							assignedAgent : m_loggedUser.Id
						};
						interaction.timestamp = timestamp;
						interaction.state = newState;
						interaction.steps.push(newStep);

						for ( var i = 0; i < interaction.assignedAgents.length; i++) {
							if (interaction.assignedAgents[i] === inConcert.app.getCompleteId()) {
								interaction.assignedAgents.splice(i, 1);
								break;
							}
						}

						return true;
					},
					cancel_attended_transfer: function(queryString, postedData) {
						var request = JSON.parse(postedData);
						var interaction = inConcert.app.interactionList[request.id];
						var zp = inConcert.Utils.zeropad;
						var now = new Date();
						var newState = "TAKEN";
						var timestamp = inConcert.Utils.format("{0}-{1}-{2} {3}:{4}:{5}",
								now.getUTCFullYear(),
								zp(now.getUTCMonth()+1),
								zp(now.getUTCDate()),
								zp(now.getUTCHours()),
								zp(now.getUTCMinutes()),
								zp(now.getUTCSeconds())
							);
						var newStep = {
							action : newState,
							timestamp : timestamp,
							assignedAgent : m_loggedUser.Id
						};
						interaction.timestamp = timestamp;
						interaction.state = newState;
						interaction.steps.push(newStep);

						for ( var i = 0; i < interaction.assignedAgents.length; i++) {
							if (interaction.assignedAgents[i] !== inConcert.app.getCompleteId()) {
								interaction.assignedAgents.splice(i, 1);
								break;
							}
						}

						return true;
					}
				},
				telephones: {
					codes: function(queryString, postedData) {
						if (queryString === "find_all") {
							return m_telephone_codes;
						}
					}
				}
			},
			main_loop : function() {
				var interactions = readInteractionsDirectory();
				readCampaignsDirectory();
				var userCampaigns = inConcert.app.database.userCampaigns[m_loggedUser.Id +"@"+m_loggedUser.VCC];
				var deltaCampaigns = {};
				var campaignsCounters = {};
				var campaignId;

				var interactionList = inConcert.app.interactionList;
				var getTotalCategorized = function(campaign, type, status){
					var total = 0;
					for (var interactionId in interactionList) {
						if (interactionList.hasOwnProperty(interactionId)) {
							var interaction = interactionList[interactionId];
							if (campaign === interaction.campaign && interaction.state === status && interaction.type === type) {
								total += 1;
							}
						}
					}
					return total;
				};
				// update campaign interactions counters
				for (campaignId in userCampaigns){
					if (userCampaigns.hasOwnProperty(campaignId)) {
						campaignsCounters[campaignId] = {
							CALL: {
									Assigned:	getTotalCategorized(campaignId,	"CALL",		"ASSIGNED"),
									Queued: getTotalCategorized(campaignId,	"CALL",		"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"CALL",		"FINISHED")
									},
							CHAT: {
									Assigned:	getTotalCategorized(campaignId,	"CHAT",		"ASSIGNED"),
									Queued: getTotalCategorized(campaignId,	"CHAT",		"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"CHAT",		"FINISHED")
									},
							FACEBOOK: {
									Assigned:	getTotalCategorized(campaignId,	"FACEBOOK",	"ASSIGNED"),
									Queued: getTotalCategorized(campaignId,	"FACEBOOK",	"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"FACEBOOK",	"FINISHED")
									},
							IM: {
									Assigned:	getTotalCategorized(campaignId,	"IM",		"ASSIGNED"),
									Queued:		getTotalCategorized(campaignId,	"IM",		"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"IM",		"FINISHED")
									},
							MAIL: {
									Assigned:	getTotalCategorized(campaignId,	"MAIL",		"ASSIGNED"),
									Queued:		getTotalCategorized(campaignId,	"MAIL",		"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"MAIL",		"FINISHED")
									},
							SMS: {
									Assigned:	getTotalCategorized(campaignId, "SMS",		"ASSIGNED"),
									Queued:		getTotalCategorized(campaignId, "SMS",		"QUEUED"),
									Finished:	getTotalCategorized(campaignId, "SMS",		"FINISHED")
									},
							TWITTER: {
									Assigned:	getTotalCategorized(campaignId,	"TWITTER",	"ASSIGNED"),
									Queued:		getTotalCategorized(campaignId,	"TWITTER",	"QUEUED"),
									Finished:	getTotalCategorized(campaignId,	"TWITTER",	"FINISHED")
									}
						};
					}
				}

				if (lastCampaignsSeen) {
					//search for campaign removals
					for (campaignId in lastCampaignsSeen) {
						if (lastCampaignsSeen.hasOwnProperty(campaignId)) {
							if (!userCampaigns[campaignId]) {
								deltaCampaigns[campaignId] = {
									deleted:true,
									Id:campaignId
								};
							}
						}
					}
					//search for campaign adds
					for (campaignId in userCampaigns) {
						if (userCampaigns.hasOwnProperty(campaignId)) {
							if (!lastCampaignsSeen[campaignId]) {
								deltaCampaigns[campaignId] = inConcert.app.database.campaigns[campaignId];
								deltaCampaigns[campaignId].role = userCampaigns[campaignId];
							}
						}
					}
				}
				//actualizo los campaign accounts para que se actualicen en el main_loop
				for (campaignId in inConcert.app.campaignsAccountsUpdated) {
					if (userCampaigns.hasOwnProperty(campaignId)) {
						deltaCampaigns[campaignId] = inConcert.app.database.campaigns[campaignId];
						deltaCampaigns[campaignId].role = userCampaigns[campaignId];
					}
				}

				inConcert.app.campaignsAccountsUpdated = {}; // lo dejo en 0

				var tempCopy = {};
				jQuery.extend(true, tempCopy, userCampaigns);
				lastCampaignsSeen = tempCopy;


				return  {
					__timeout : 1000,
					refresh_campaigns: {
						delta: deltaCampaigns,
						accounts: []
					},
					refresh_inbox: interactions,
					save_settings: true,
					campaign_counters: campaignsCounters
				};
			},
			initialize : function(queryString, postedData) {
				//"{\\\"utcDate\\\":{\\\"year\\\":2011,\\\"month\\\":3,\\\"day\\\":19,\\\"hours\\\":19,\\\"minutes\\\":46,\\\"seconds\\\":50}}\",

				//get clients configuration for this user
				var settings = {
					InteractionClients : m_loggedUser.InteractionClients
				};

				//get the campaigns for this user
				var campaigns = {};
				var userCampaigns = inConcert.app.database.userCampaigns[m_loggedUser.Id + '@' +  m_loggedUser.VCC];
				var campaignId;
				if (userCampaigns) {
					for (campaignId in userCampaigns) {
						if (userCampaigns.hasOwnProperty(campaignId)) {
							campaigns[campaignId] = inConcert.app.database.campaigns[campaignId];
							campaigns[campaignId].role = userCampaigns[campaignId];
						}
					}
				}
				var campaignCounters = {};
				for (campaignId in userCampaigns) {
					if (userCampaigns.hasOwnProperty(campaignId)) {
						//TODO proper counting against local database
						campaignCounters[campaignId] = {
							MAIL: {
								Assigned: "0",
								Queued: "0",
								Waiting: "0"
							}
						};
					}
				}
				var currentInteractions = [];
				//TODO get current interactions for user from local database
				var response = {
					result: true,
					timeDifference: -6,
					settings: settings,
					campaigns: campaigns,
					custom_permissions: inConcert.app.database.permissions,
					user: m_loggedUser.userId,
					domain: m_loggedUser.VCC,
					userName: m_loggedUser.Name,
					initialState: m_loggedUser.initialState,
					country: m_loggedUser.countryCode,
					area: m_loggedUser.areaCode,
					userRole: m_loggedUser.userRole,
					status: m_loggedUser.initialState,
					availableStates: inConcert.app.database.availableStates,
					statusMessage: m_loggedUser.StatusMessage,
					interactions: currentInteractions,
					campaignCounters: campaignCounters,
					groupPermissions: inConcert.app.database.grouppermission[m_loggedUser.UserGroup +'@system'],
					userPermissions: inConcert.app.database.userpermission[m_loggedUser.UserGroup +'@system']
				};
				return response;
			},
			"save_settings": function(queryString, postedData) {
				var settings = JSON.parse(postedData);
				var vcc = "inconcert";
				if (settings && settings.personal) {
					var user = settings.personal;
					user.Id = m_loggedUser.userId + "@" + vcc;
					if (settings.telephony) {
						user.addresses = {
							CALL: settings.telephony
						};
					}
					var result = models.users.save(user);
					return {result: result};
				}
				else {
					throw("Save personal setting, incorrect posted data!!! " + postedData);
				}
			},
			"advise_agent" : function(queryString, postedData) {
				//do nothing
				return true;
			},
			"activate_address": function(queryString, postedData) {
				return "";
			}
		},
		oauth : {

		},
		logstorage: {
			upload: function(queryString, postedData){
				return true;
			}
		},
		reports: {
			get_cuts: function(queryString, postedData){
				return {};
			},
			get_filters: function(queryString, postedData){
				return {};
			},
			list_reports: function(queryString, postedData){
				return [];
			},
			get_export_types: function(queryString, postedData){
				return [];
			}
		},
		chat: {
			get_user_messages: function(queryString, postedData){
				return [];
			},
			login: function(queryString, postedData){
				return true;
			}
		},
		trackme: {
			get_visited_pages: function(queryString, postedData){
				return {
					page_list : [
						{
							page : "http://www.google.com",
							timestamp : getCurrentTimestamp()
						},
						{
							page : "https://git.inconcert",
							timestamp : getCurrentTimestamp()
						}
					]
				};
			}
		},
		supervisor: {
			controllers : {
				workgroups : {
					get : function(queryString, postedData){
						return {};
					}
				},
				configuration : {
					get : function(queryString, postedData){
						return inConcert.Storage.get('supervisor_config') || {};
						//	Configuration Example
						//	var config =  {
						//				"productdevelopment@inconcert" : {
						//					dashboard : {
						//						current_view : "dashboard",
						//						visible : true,
						//						panels : {
						//							sup_dash_indicators_panel : {
						//								visible : true
						//							},
						//							sup_dash_members_panel : {
						//								visible : true,
						//								options : {
						//									show_avatars : true,
						//									show_names : true,
						//									show_offline : true
						//								}
						//							},
						//							sup_dash_summary_panel : {
						//								visible : true,
						//								groups : {
						//									sup_dash_group_summary_agents : {
						//										visible : false,
						//										indicators : {
						//											sup_dash_summary_indicator_abandoned_percentage : {
						//												visible : true
						//											}
						//										}
						//									}
						//								}
						//							}
						//						}
						//					}
						//				}
						//			};
						//
					},
					save : function(queryString, postedData){
						var request = JSON.parse(postedData);
						// Save config to disk
						inConcert.Storage.put('supervisor_config', request.configuration);
						return true;
					}
				},
				members : {
					change_state : function(queryString, postedData){
						var request = JSON.parse(postedData);
						var userWithVCC = request.memberId + "@inconcert";
						if (!inConcert.app.database.users[userWithVCC]) {
							return false;
						}
						inConcert.app.database.users[userWithVCC].State = request.newState;
						return true;
					},
					logout : function(queryString, postedData){
						var request = JSON.parse(postedData);
						var userWithVCC = request.memberId + "@inconcert";
						if (!inConcert.app.database.users[userWithVCC]) {
							return false;
						}
						inConcert.app.database.users[userWithVCC].State = "OFFLINE";
						return true;
					},
					get_supervisor_members : function(queryString, postedData){

						var request = JSON.parse(postedData);
						var supervisor_agents = [];

						// TO-DO:
						//
						// workgroups (Array)
						// agentNameFilter
						// agentStateFilter

						// create an array for each member
						_.forEach(inConcert.app.database.users, function(agent){
							supervisor_agents.push({
								ID : agent.Id.replace(/^(.*?)@(.*?)$/, '$1'),
								name : agent.Name,
								isLogged : agent.status.state === "OFFLINE" ? false : true,
								stateTimeTimestamp : agent.stateTimeTimestamp,
								state : get_campaign_state(agent.status.state),
								logoutTimeTimestamp : 1373626481.695,
								interactions : {}
							});
						});

						// filter logged agents
						if (!request.showOfflineAgents){
							supervisor_agents = _.reject(supervisor_agents, function(agent){
								return !agent.isLogged;
							});
						}

						if (request.agentOrder === 'Name'){
							supervisor_agents = _.sortBy(supervisor_agents, function(agent){
								return agent.name;
							});
						}
						else if (request.agentOrder === 'NameAndState'){
							supervisor_agents = _.sortBy(supervisor_agents, function(agent){
								return agent.name;
							});
							supervisor_agents = _.sortBy(supervisor_agents, function(agent){
								return agent.state;
							});
						}

						var agents_filtered_by_page = [];

						// filter pagination
						if (request.page){
							var rows_per_page = request.rows_per_page || 20;
							var start_cut = request.page === 0 ? 1 : (request.page * rows_per_page ) - rows_per_page;
							var finish_cut = request.page === 0 ? supervisor_agents.length : request.page * rows_per_page;
							agents_filtered_by_page = supervisor_agents.slice(start_cut, finish_cut);
						}

						return {
							agents : agents_filtered_by_page.length > 0 ? agents_filtered_by_page : supervisor_agents,
							agentsInQuery : agents_filtered_by_page.length > 0 ? supervisor_agents.length : 0
						};
					}
				},
				indicators : {
					get_historical_data : function(queryString, postedData){
						var now = new Date();
						return [
								{
									id : 'TotalInteractions',
									description : 'Conversations',
									data : [
										[ addMinutes(now, 0) / 1000,  getRandomInt(1, 30)],
										[ addMinutes(now, 15) / 1000, getRandomInt(1, 30)],
										[ addMinutes(now, 30) / 1000, getRandomInt(1, 30)],
										[ addMinutes(now, 45) / 1000, getRandomInt(1, 30)],
										[ addMinutes(now, 60) / 1000, getRandomInt(1, 30)]
									]
								},
								{
									id : 'Channel-TotalInteractions',
									description : 'Channel - Conversations',
									containsSecondarySeries : true,
									data : {
										"CHAT" : {
											data : [
												[ addMinutes(now, 0) / 1000,  getRandomInt(1, 30)],
												[ addMinutes(now, 15) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 30) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 45) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 60) / 1000, getRandomInt(1, 30)]
											]
										},
										"MAIL" : {
											data : [
												[ addMinutes(now, 0) / 1000,  getRandomInt(1, 30)],
												[ addMinutes(now, 15) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 30) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 45) / 1000, getRandomInt(1, 30)],
												[ addMinutes(now, 60) / 1000, getRandomInt(1, 30)]
											]
										}
									}
								}
						];
					},
					get_supervisor_panel : function(queryString, postedData){
						var supervisor_agents = [];

						_.forEach(inConcert.app.database.users, function(agent){
							supervisor_agents.push({
								ID : agent.Id.replace(/^(.*?)@(.*?)$/, '$1'),
								name : agent.Name,
								isLogged : agent.status.state === "OFFLINE" ? false : true,
								stateTimeTimestamp : agent.stateTimeTimestamp,
								state : get_campaign_state(agent.status.state),
								logoutTimeTimestamp : 1373626481.695,
								interactions : {}
							});
						});

						var logged_agents = _.filter(supervisor_agents, function(agent){ return agent.isLogged });
						var idle_agents = _.filter(supervisor_agents, function(agent){ return agent.state === "IDLE" });
						var break_agents = _.filter(supervisor_agents, function(agent){ return agent.state === "BREAK" });
						var busy_agents = _.filter(supervisor_agents, function(agent){ return agent.state === "BUSY" });
						var busy_cross_campaign_agents = _.filter(supervisor_agents, function(agent){ return agent.state === "BUSY CROSS CAMPAIGN" });

						return {
							nowTimestamp : new Date().getTime() / 1000,
							realtimeSummary : [
								{
									ID: 'agents',
									indicators : [
										{
											ID: 'abandoned_percentage',
											description : "Abandoned percentage",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'answer_time',
											description : "Answer Time",
											total : "00:00:"+getRandomInt(1, 60)
										},
										{
											ID: 'agents_logged',
											description : "Logged Agents",
											total : logged_agents.length
										},
										{
											ID: 'agents_active',
											description : "Active Agents",
											total : busy_agents.length + busy_cross_campaign_agents.length
										}
									],
									description : "Agents"
								},
								{
									ID: 'inbound_calls',
									indicators : [
										{
											ID: 'queued',
											description : "Queued",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'waiting',
											description : "Waiting",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'active',
											description : "Active",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'total',
											description : "Total Calls",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'abandon_rate',
											description : "Abandon Rate",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'service_level',
											description : "Service Level",
											total : getRandomInt(1, 100)
										},
										{
											ID: 'avg_answer_speed_calls',
											description : "Avg Answer Speed",
											total : "00:00:"+getRandomInt(1, 60)
										},
										{
											ID: 'avg_attention_time',
											description : "Avg Attention Time",
											total : "00:00:"+getRandomInt(1, 60)
										},
										{
											ID: 'agent_occupancy',
											description : "Agent Occupancy",
											total : 13
										},
										{
											ID: 'total_goals',
											description : "Total Goals",
											total : getRandomInt(1, 100)
										}
									],
									description : "Inbound Calls"
								},
							],
							realtimeCharts : [
								{
									description : "Agent By Status",
									data : [
										["Idle", idle_agents.length],
										["Break", break_agents.length],
										["Busy", busy_agents.length],
										["Busy Cross Campaign", busy_cross_campaign_agents.length]
									],
									id : "AgentStatusSummary"
								}
							]
						};
					}
				}
			}
		}
	};

	/*
		Parses url and finds out what to execute, what is controller, what is method, what are parameters
	*/
	function parseAppMethod(parameters) {
		var matcher = new RegExp("([^/]+)", "g");
		var match = matcher.exec(parameters);
		var matches = [];
		while (match) {
			matches.push(match[1]);
			match = matcher.exec(parameters);
		}
		switch (matches.length) {
			case 0 :
				throw("Cannot execute empty method");
			case 1 :
				//it's only a method to execute
				return {
					method : matches[0]
				};
			case 2 :
				//it's controller + method
				return  {
					controller : matches[0],
					method : matches[1]
				};
		default:
				//it's controller + method + some parameter, build up the string again and return that
				var rebuiltParams = "";
				var i;
				for (i = 2; i < matches.length; i++) {
					rebuiltParams += (i > 2 ? "/" : "") + matches[i];
				}
				return  {
					controller : matches[0],
					method : matches[1],
					parameters : rebuiltParams
				};
		}
	}

	function executeMethod(methodFn, queryString, options) {
		var error, response;
		var timeout = 10;
		var throwError;
		try {
			response = methodFn(queryString, options.data);
			if (typeof(response) === "object" && response.__timeout) {
				timeout = response.__timeout;
			}
		}
		catch(e) {
			error = true;
			throwError = e;
		}
		//response is always called with a timeout in
		//order to simulate the async nature of ajax
		setTimeout(function() {
			if (!error) {
				if (typeof(options.success === "function")) {
					var data = response;
					if (typeof(data) === "string" && data.length > 0 && options.dataType === "json") {
						data = JSON.parse(data);
					}
					options.success(data);
				}
			}
			else {
				if (typeof(options.error === "function")) {
					if (typeof(throwError) === "object") {
						throw throwError.message;
						options.error({ status : 500 }, throwError.message);
					}
					else {
						options.error({status: 500});
					}
				}
			}
		}, timeout);

	}

	//just invokes ajax request as specified by the caller and gives a default error handler in case none specified
	function invoke(parameters) {
		if (!inConcert.app){
			inConcert.app = {};
		}

		//si el script fue levantado desde otro HTML ya tendria un logged user, interaction list y otras yerbas disponibles a partir del inconcert.app
		if (inConcert.app.loggedUserApi) {
			m_loggedUser = inConcert.app.loggedUserApi;
		}
		if (typeof(inConcert.app.database)==='undefined') {
			inConcert.app.database = database;
		}
		if (typeof(inConcert.app.campaignsAccountsUpdated) === 'undefined') {
			inConcert.app.campaignsAccountsUpdated = campaignsAccountsUpdated;
		}
		if (typeof(inConcert.app.interactionList) === 'undefined') {
			inConcert.app.interactionList = interactionList;
		}

		var methodFn;
		var match = parameters.url.match(new RegExp("/inconcert/apps/([^/]+)/(.*)"));
		if (match && apps[match[1]]) {
			var app = apps[match[1]];
			var method = parseAppMethod(match[2]);

			//parche horrible para el get de archived iteraction
			if ((match[1] === "repository") && (method.controller === "get" || method.controller === "add")) {
				method.parameters = method.method;
				method.method = method.controller;
				method.controller = false; //si, esto es horrible
			}

			if (method.controller) {
				var controller = app.controllers[method.controller];
				if (controller) {
					methodFn = controller[method.method];
					if (typeof(methodFn) === "function") {
						executeMethod(methodFn, method.parameters, parameters);
					}
					else {
						throw("Tried to execute method unimplemented method " + method.method);
					}
				}
				else {
					throw("Tried to execute method on unexistent controller " + method.controller);
				}
			}
			else {
				methodFn = app[method.method];
				if (typeof(methodFn) === "function") {
					executeMethod(methodFn, method.parameters, parameters);
				}
				else {
					throw("Tried to execute method unimplemented method " + method.method);
				}
			}
		}
		else {
			throw("call for unexistant application, url: " + parameters.url);
		}
		return {
			abort: $.noop
		};
	}

	if (typeof inConcert.invoke !== 'function') {
		inConcert.invoke = invoke;
	}
	if (typeof inConcert.apps !== 'function') {
		inConcert.apps = apps;
	}
}());
